<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

	<xsl:template match="/">
		<html>
			<head>
				<meta http-equiv="Content-Type" content="text/html" />
				<link rel="stylesheet" type="text/css" href="doc.css" />
				<title>Design Management REST API Usage Scenarios</title>
			</head>
			<body>
				<h1>Rational Design Management REST API Usage Scenarios</h1>
				<p>
					<a href="https://jazz.net/products/design-management/">Rational Design Management (DM)</a> provides 
					a <a href="http://www.ics.uci.edu/~fielding/pubs/dissertation/top.htm">REST</a> API that can be used by
					authorized clients. This REST API relies heavily
					on <a href="http://www.ietf.org/rfc/rfc2616.txt">HTTP</a> and <a href="http://www.w3.org/RDF/">RDF</a> 
					principals, and implements the OSLC <a href="http://open-services.net/wiki/architecture-management/">Architecture
					Management 2.0</a> and <a href="http://open-services.net/bin/view/Main/RmSpecificationV2">Requirements 
					Management 2.0</a> specifications.

				</p>
				<p>
					DM is a <a href="https://jazz.net/">Jazz</a> based application, built on the <a href="https://jazz.net/products/jazz-foundation/">
					Jazz Foundation Services (JFS)</a>. All DM REST calls must be authenticated first. JFS uses <a href="http://oauth.net/">OAuth</a>, 
					and all service calls must contain valid OAuth tokens in the session cookies in order to
					be processed. Web clients get routed automatically to an
					authentication dialog, and then redirected back to the originally
					requested web resource, however programmatic clients must be
					prepared to authenticate, and 'do the OAuth dance' before working
					with DM and JFS services. The details of the OAuth protocol, and
					clients of JFS's implementation are very specific to the client language and environment and are beyond the scope of this document.
					See the developer works article, <a href="http://www.ibm.com/developerworks/web/library/wa-oauthsupport/index.html">"Add OAuth authentication support
					to HttpClient"</a> for information about extending the Apache HTTP
					Client Java libraries for OAuth, and the open source <a href="http://www.eclipse.org/lyo/">Eclipse Lyo</a> project
					for Java based OSLC and Jazz client examples.
				</p>
				<p>
					This document provides a number of scenario based examples of common
					uses of the API.  They span multiple services. Documentation of <a href="services">individual 
					services</a> are also available on this server instance.  More documentation for 
					this and other versions of DM can be found on the Jazz.net <a href="https://jazz.net/wiki/bin/view/Main/DesignManagement">Design 
					Management wiki</a> (requires Jazz.net account). 
				</p>


				<xsl:for-each select="/scenarios/category">
					<h2>
						<xsl:value-of select="./@name" />
					</h2>
					
					<p><xsl:value-of select="./description" /></p>

					<table>
						<tr>
							<th>Scenario</th>
							<th>Summary</th>
						</tr>

						<xsl:for-each select="./scenario">

							<xsl:if test="position() mod 2 = 1">
								<tr>
									<td>
										<a href="/dm/doc/scenario?id={@id}">
											<xsl:value-of select="@title" />
										</a>
									</td>
									<td>
										<xsl:value-of select="@summary" />
									</td>
								</tr>
							</xsl:if>
							<xsl:if test="position() mod 2 = 0">
								<tr class="odd">
									<td>
										<a href="/dm/doc/scenario?id={@id}">
											<xsl:value-of select="@title" />
										</a>
									</td>
									<td>
										<xsl:value-of select="@summary" />
									</td>
								</tr>
							</xsl:if>
						</xsl:for-each>
					</table>
				</xsl:for-each>


			</body>
		</html>
	</xsl:template>
</xsl:stylesheet>

