<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:template match="/service">
<html>
<head>
<meta http-equiv="Content-Type" content="text/html"/>
<link rel="stylesheet" type="text/css" href="doc.css" />
<title>Service API Documentation: <xsl:value-of select="service-title"/></title>
<script language="JavaScript" type="text/javascript">
function toggleDiv(id) {
  var div = document.getElementById(id);
  if( div.style.display == 'none') {
  	div.style.display = 'block';
  } else {
  	div.style.display = 'none';
  }
}
</script> 
</head>
<body>
	<p>[<a href="services">Service Listing</a>]</p>
	
	<h1><xsl:value-of select="service-title"/></h1>
	<xsl:if test="service-visibility/text()">
		Visibility: <code></code><b><xsl:value-of select="service-visibility"/></b> <br/> 
	</xsl:if>
	
	<xsl:if test="service-description/text()">
		<h2>Description</h2>
		<xsl:copy-of select="service-description"/>
	</xsl:if>
	
	<xsl:if test="service-motivation/text()">
		<h2>Motivation</h2>
		<xsl:copy-of select="service-motivation"/>
	</xsl:if>

	<xsl:if test="architecture">
		<h2>Architecture Info</h2>
		<p><b><i>Implementing Class:</i></b> <code><xsl:value-of select="architecture/implementing-class"/></code></p>
		<p><b><i>Annotated Class:</i></b> <xsl:value-of select="architecture/implementing-class/@annotated-service"/></p>
		<xsl:if test="count(architecture/known-clients/known-client) &gt; 0">
			<p><b><i>Known Clients</i></b>: 
			<xsl:for-each select="architecture/known-clients/known-client">
			  	<xsl:value-of select="."/>,
			</xsl:for-each>
			</p>
		</xsl:if>
		<xsl:if test="architecture/comment">
			<b><i>Comments:</i></b><br/>
			<xsl:copy-of select="architecture/comment"/>
		</xsl:if>
	</xsl:if>


	<xsl:if test="count(resource-examples/resource-example) &gt; 0">
		<h2>Resource Examples</h2>
		<xsl:for-each select="resource-examples/resource-example">
		  	<p><a href="javascript:toggleDiv('{@title}')"><xsl:value-of select="@title"/></a></p>
			<div id="{@title}" style="overflow:hidden;display:none">
			<p><xsl:copy-of select="./resource-example-description"/></p>
			<pre><xsl:copy-of select="./resource-example-content"/></pre></div>
		</xsl:for-each>
	</xsl:if>

	<h2>Specification</h2>

	<xsl:if test="count(common-status-codes/common-status-code) &gt; 0">
		<h3>Common Response Status Codes</h3>
		<table class="sample">
			<tr><th>Status Code</th><th>Comment</th></tr>
			<xsl:for-each select="common-status-codes/common-status-code">
				<xsl:if test="position() mod 2 = 1">
				  	<tr>
				  		<td><xsl:value-of select="code"/></td>
						<td><xsl:copy-of select="comment"/></td>
					</tr>
				</xsl:if>
				<xsl:if test="position() mod 2 = 0">
				  	<tr class="odd">
				  		<td><xsl:value-of select="code"/></td>
						<td><xsl:copy-of select="comment"/></td>
					</tr>
				</xsl:if>
			</xsl:for-each>
		</table>
	</xsl:if>

	<xsl:if test="count(url-definitions/url-definition) &gt; 0">
		<h3>Common URL Definitions</h3>
		<table class="sample">
			<tr><th>Label</th><th>Path</th><th>Description</th></tr>
			<xsl:for-each select="url-definitions/url-definition">
				<xsl:if test="position() mod 2 = 1">
				  	<tr>
				  		<td><code>{<xsl:value-of select="label"/>}</code></td>
				  		<td><code><xsl:value-of select="path"/></code></td>
						<td><xsl:copy-of select="description"/></td>
					</tr>
				</xsl:if>
				<xsl:if test="position() mod 2 = 0">
				  	<tr class="odd">
				  		<td><code>{<xsl:value-of select="label"/>}</code></td>
				  		<td><code><xsl:value-of select="path"/></code></td>
						<td><xsl:copy-of select="description"/></td>
					</tr>
				</xsl:if>
			</xsl:for-each>
		</table>
	</xsl:if>

	<h3>HTTP Methods</h3>
	<p><b>Summary</b></p>
	<p>Listing of methods in the service. Internal methods require the <code>rmps-internal</code> header.</p>
	<table>
		<tr><th>Methods</th><th>URL</th><th>Comments</th><th>Internal</th></tr>
		<xsl:for-each select="service-methods/service-method">
			<xsl:if test="position() mod 2 = 1">
				<tr>
			  		<td><xsl:value-of select="method"/></td>
			  		<td><code><xsl:value-of select="url"/></code></td>
					<td><xsl:copy-of select="method-comment"/></td>
					<td><xsl:value-of select="internal"/></td>
				</tr>
			</xsl:if>
			<xsl:if test="position() mod 2 = 0">
				<tr class="odd">
			  		<td><xsl:value-of select="method"/></td>
			  		<td><code><xsl:value-of select="url"/></code></td>
					<td><xsl:copy-of select="method-comment"/></td>
					<td><xsl:value-of select="internal"/></td>
				</tr>
			</xsl:if>
		</xsl:for-each>
	</table>
	
	<p><b>Method Details</b></p>
	<xsl:for-each select="/service/service-methods/service-method">
		<h3><xsl:value-of select="method"/> <code> <xsl:value-of select="url"/></code></h3>
		<xsl:if test="internal/text()">
			<p><b><xsl:value-of select="internal"/></b></p>
		</xsl:if>
		<xsl:copy-of select="method-comment"/>
		<xsl:if test="produces/text()">
			<p><b>Produces:</b> <code> <xsl:value-of select="produces"/></code></p>
		</xsl:if>
		<xsl:if test="consumes/text()">
			<p><b>Consumes:</b><code> <xsl:value-of select="consumes"/></code></p>
		</xsl:if>
		<xsl:if test="count(./known-clients/known-client) &gt; 0">
			<p><b>Known Clients:</b> 
			<xsl:for-each select="./known-clients/known-client">
			  	<xsl:value-of select="."/>,
			</xsl:for-each>
			</p>
		</xsl:if>
		
		<xsl:if test="query-parameters/query-parameter">
			<table>
				<tr><th>Query Parameter</th><th>Description</th></tr>
				<xsl:for-each select="query-parameters/query-parameter">
					<xsl:if test="position() mod 2 = 1">
						<tr>
							<td><code><xsl:value-of select="name"/></code></td>
							<td><xsl:value-of select="description"/></td>
						</tr>
					</xsl:if>
					<xsl:if test="position() mod 2 = 0">
						<tr class="odd">
							<td><code><xsl:value-of select="name"/></code></td>
							<td><xsl:value-of select="description"/></td>
						</tr>
					</xsl:if>
				</xsl:for-each>
			</table>
		</xsl:if>
		<xsl:if test="headers/header">
			<table>
				<tr><th>Headers</th><th>Description</th></tr>
				<xsl:for-each select="headers/header">
					<xsl:if test="position() mod 2 = 1">
						<tr>
							<td><xsl:value-of select="name"/></td>
							<td><xsl:value-of select="description"/></td>
						</tr>
					</xsl:if>
					<xsl:if test="position() mod 2 = 0">
						<tr class="odd">
							<td><xsl:value-of select="name"/></td>
							<td><xsl:value-of select="description"/></td>
						</tr>
					</xsl:if>
				</xsl:for-each>
			</table>
		</xsl:if>
		
		<xsl:if test="count(method-examples/method-example) &gt; 0">
		<xsl:for-each select="method-examples/method-example">
		  	<h4>Example: <xsl:value-of select="./@title"/></h4>
			<xsl:copy-of select="."/>
		</xsl:for-each>
	</xsl:if>
	</xsl:for-each>

</body>
</html>
</xsl:template>
</xsl:stylesheet>

