<?xml version="1.0" encoding="ISO-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:template match="/">
<html>
<head>
<meta http-equiv="Content-Type" content="text/html"/>
<link rel="stylesheet" type="text/css" href="doc.css" />
<title>Design Management Service Documentation Listing</title>
</head>
<body>
<h1>Rational Design Management Service API</h1>

<p>
<a href="https://jazz.net/products/design-management/">Rational Design Management (DM)</a> provides 
a <a href="http://www.ics.uci.edu/~fielding/pubs/dissertation/top.htm">REST</a> API, that it uses internally and is available 
for use by other clients.  This API document is generated from annotations embedded in the service implementation. It describes 
the individual services, thier declared methods, parameters and payloads.  They represent the exact version of service documentation
for this instance of the DM server.
</p>
<p>
DM is a <a href="https://jazz.net/">Jazz</a> based application, built on the <a href="https://jazz.net/products/jazz-foundation/">
Jazz Foundation Services (JFS)</a>. All DM REST calls must be authenticated first. JFS uses <a href="http://oauth.net/">OAuth</a>, 
and all service calls must contain valid OAuth tokens in the session cookies in order to be processed. Web clients get routed 
automatically to an authentication dialog, and then redirected back to the originally requested web resource, however programmatic 
clients must be prepared to authenticate, and 'do the OAuth dance' before working with DM and JFS services. The details of the 
OAuth protocol, and clients of JFS's implementation are very specific to the client language and environment and are beyond the 
scope of this document. See the developer works article, <a href="http://www.ibm.com/developerworks/web/library/wa-oauthsupport/index.html">
"Add OAuth authentication support to HttpClient"</a> for information about extending the Apache HTTP Client Java libraries for 
OAuth, and the open source <a href="http://www.eclipse.org/lyo/">Eclipse Lyo</a> project for Java based OSLC and Jazz client examples.
</p>
<p>
This set of documentation provides the details of all the services exposed by DM. The OSLC 
<a href="http://open-services.net/wiki/architecture-management/">Architecture 
Management 2.0</a> and <a href="http://open-services.net/bin/view/Main/RmSpecificationV2">Requirements 
Management 2.0</a> specifications are implemented by DM and their required services are considered part of the public API. 
This documentation details the level at which the specifications are implemented. Most of the services exposed have a 
<a href="services?visibility=*">private</a> or a '<a href="services?visibility=public,partner">partner</a>' level of visibility.  
Private services are intended for use only by DM provided clients (Web UI, RSA and Rhapsody desktops).  Partner level services are 
expected to be used outside of DM clients, however they are more likely to evolve in the coming versions of DM, before potentially 
becoming part of the public API. 
</p>
<p>
<a href="scenarios">Scenario based documentation</a> is also available on this server. It includes a set of common uses of this server's API.
</p>

	<table>
		<tr>
			<th>Service</th>
			<th>Path</th>
			<th>Visibility</th>
			<th>Description</th>
		</tr>
		
		<xsl:for-each select="//service">
			<xsl:if test="position() mod 2 = 1">
			  	<tr>
			  		<td><a href="/dm/doc/service?serviceUrl={path}"><xsl:value-of select="actionUrl"/><xsl:value-of select="name"/></a></td>
			  		<td><xsl:value-of select="path"/></td>
			  		<td><xsl:value-of select="visibility"/></td>
					<td><xsl:copy-of select="description"/></td>
				</tr>
			</xsl:if>
			<xsl:if test="position() mod 2 = 0">
			  	<tr class="odd">
			  		<td><a href="/dm/doc/service?serviceUrl={path}"><xsl:value-of select="actionUrl"/><xsl:value-of select="name"/></a></td>
			  		<td><xsl:value-of select="path"/></td>
			  		<td><xsl:value-of select="visibility"/></td>
					<td><xsl:copy-of select="description"/></td>
				</tr>
			</xsl:if>
		</xsl:for-each>

</table>

</body>
</html>
</xsl:template>
</xsl:stylesheet>

