/*
*+------------------------------------------------------------------------+
*| Licensed Materials - Property of IBM                           |
*| (C) Copyright IBM Corporation 2012. All Rights Reserved.       |
*| @author Sandeep Kohli                                                              |
*| US Government Users Restricted Rights - Use, duplication or disclosure |
*| restricted by GSA ADP Schedule Contract with IBM Corp.         |
*+------------------------------------------------------------------------+
*/

tree grammar Generator;
options {
  tokenVocab=Sparql11;
  ASTLabelType=CommonTree;
  language = Java;
  output=template;
}


@header {
/*
*+------------------------------------------------------------------------+
*| Licensed Materials - Property of IBM.                          |
*| (C) Copyright IBM Corporation 2012. All Rights Reserved.       |
*| @author Sandeep Kohli                                          |
*| US Government Users Restricted Rights - Use, duplication or disclosure |
*| restricted by GSA ADP Schedule Contract with IBM Corp.         |
*+------------------------------------------------------------------------+
*/
package com.ibm.xtools.rmpx.sparqlRDF;
import org.antlr.stringtemplate.*;
import java.util.HashMap;
}

@members {

    
    protected void mismatch(IntStream input, int ttype, BitSet follow ) throws RecognitionException 
    {
        throw new MismatchedTokenException(ttype, input) ;
    }
    
    public Object recoverFromMismatchedSet(IntStream input, RecognitionException e, BitSet follow ) throws RecognitionException 
    {
    
        throw e;
    }

    public void notImplemented() throws SparqlUnsupportedConstructException
    {
        throw new SparqlUnsupportedConstructException("Unsupported Sparql construct" );
    }

        // This function will replace \ with \\
        private String escapeEscape(String str )
        {
        	StringBuffer sb = new StringBuffer();
        	char[] charArray = str.toCharArray();
        	for (char c : charArray) {
        		
				if(c == '\\') // If it is a escape char
					sb.append("\\\\");  // add one more char
				else sb.append(c);
			}
        	return sb.toString();
        }
		
		// All inbuild SPARLQ function should be upcased.
        private String upCase(String str )
        {
        	return str.toUpperCase();
			
        }
		
		private String generateUniqueBlankNodeSym()
		{
		 	return  "_:" + symPrefix + numOfBlankSyms++;
		}
		
		private String generateUniqueAnonSym() 
		{
		 	return  "_:" + anonPrefix + numOfAnonSyms++;
		}
		
    		void storeSym(String sym) 
    		{
    			sym = sym.substring(1); //remove the leading '$' or '?'
              	if(symMap.get(sym) == null ) 
            	{ 
            		symMap.put(sym, generateUniqueBlankNodeSym());
            	}                        
            }
            
            String retriveSym(String sym) 
    		{
            	sym = sym.substring(1); //remove the leading '$' or '?'
              	return symMap.get(sym);                      
            }
        
	    public HashMap<String, String> symMap = new HashMap<String, String>();
	    int numOfBlankSyms = 0;
	    int numOfAnonSyms = 0;
	    String symPrefix = "xx";
	    String anonPrefix = "a" ;
}

@rulecatch 
{
    catch (RecognitionException e ) {
        throw e;
    }
    
}

// END:       header

    queryUnit                   :       q = query EOF 
                                        -> queryUnitTemplate(text = {$q.text}, query={$q.st})
                                ;
    query                       :       prologue ( sq=selectQuery | cq=constructQuery | dq=describeQuery | aq = askQuery ) bindingsClause
                                        ->queryTemplate(prologue={$prologue.st}, selectQuery={$selectQuery.st}, constructQuery = {$cq.st}, describeQuery = {$dq.st} , askQuery = {$aq.st})
                                ;
    updateUnit                  :       update 
                                        ->updateUnitTemplate(update = {$update.st})
                                ;
    prologue                    :       (b+=baseDecl | p+=prefixDecl )*
                                        ->prologueTemplate(baseDecl={$b}, prefixDecl={$p})
                                        
                                ;       
    baseDecl                    :       BASE IRI_REF
                                        ->baseDeclTemplate(iri={$IRI_REF.text})
                                ;
    prefixDecl                  :       PREFIX PNAME_NS IRI_REF
                                        ->prefixDeclTemplate(pname={$PNAME_NS.text}, iri={$IRI_REF.text})
                                ;
    selectQuery                 :       selectClause (d+=datasetClause)* whereClause solutionModifier
                                        -> selectQueryTemaplte(selectClause={$selectClause.st}, datasetClause={$d}, whereClause={$whereClause.st}, solutionModifier={$solutionModifier.st})
                                ;
    subSelect                   :       s=selectClause w=whereClause so=solutionModifier
                                       
                                        ->subSelectTemplate(selectClause={$s.st}, whereClause={$w.st}, solutionModifier={$so.st})
                                ;
    selectClause                :       SELECT ( dis=DISTINCT | red=REDUCED )? ( (v+=varClause)+ | star='*' )
                                        ->selectClauseTemplate(distinct={$dis.text}, reduced={$red.text}, varClause={$v}, star={$star.text} )
                                ;
    varClause                   :      var 
                                        ->{$var.st}
                                |      '(' expression AS var ')'  
                                        ->expressionTemplate(expression={$expression.st}, var={$var.st})
                                ;
    constructQuery              :       CONSTRUCT c=constructTemplate (d+=datasetClause)* w=whereClause s=solutionModifier 
                                        ->constructQueryTemplate1(constructTemplate = {$c.st}, datasetClause = {$d}, whereClause = {$w.st}, solutionModifier = {$s.st})
                                |       CONSTRUCT (d+=datasetClause)* WHERE '{' t=triplesTemplate? '}' s=solutionModifier 
                                        ->constructQueryTemplate2(datasetClause = {$d}, triplesTemplate = {$t.st}, solutionModifier = {$s.st})
                                ;
    describeQuery               :       DESCRIBE ( (v+=varOrIRIref)+ | star='*' ) (d+=datasetClause)* w=whereClause? so=solutionModifier 
                                        ->describeQueryTemplate(varOrIRIref={$v}, star={$star.text}, datasetClause={$d}, whereClause={$w.st}, solutionModifier={$so.st})
                                ;
    askQuery                    :       ASK (d+=datasetClause)* whereClause solutionModifier 
                                        ->askQueryTemplate(datasetClause = {$d}, whereClause={$whereClause.st}, solutionModifier={$solutionModifier.st})
                                ;
    datasetClause               :       FROM ( defaultGraphClause | namedGraphClause )
                                        ->datasetClauseTemplate(defaultGraphClause = {$defaultGraphClause.st}, namedGraphClause = {$namedGraphClause.st} )
                                ;
    defaultGraphClause          :       sourceSelector 
                                        ->{$sourceSelector.st}
                                ;
    namedGraphClause            :       NAMED sourceSelector 
                                        ->{$sourceSelector.st}
                                ;
    sourceSelector              :       iriRef  -> {$iriRef.st};
    whereClause                 :       WHERE? groupGraphPattern
                                        -> whereClauseTemplate(groupGraphPattern={$groupGraphPattern.st})
                                ;
    solutionModifier            :       g=groupClause? h=havingClause? o=orderClause? l=limitOffsetClauses? 
                                        ->solutionModifierTemplate(groupClause = {$g.st}, havingClause = {$h.st}, orderClause = {$o.st}, limitOffsetClauses = {$l.st})
                                ;
    groupClause                 :       GROUP BY (g+=groupCondition)+ 
                                        ->groupClauseTemplate(groupCondition = {$g})
                                ;
    groupCondition              :       ^(FUNC0 NIL)    ->  FUNC0Template(name={upCase($FUNC0.text)})
                                |       ^(FUNC1 exp1 = expression) -> FUNC1Template(name={upCase($FUNC1.text)}, exp1={$exp1.st})
                                |       ^(FUNC1OPT exp1 = expression)  -> FUNC1OPTTemplate(name={$FUNC1OPT.text}, exp1={$exp1.st})
                                |       ^(FUNC2 exp1 = expression exp2 = expression)    ->  FUNC2TEMPLATE(name={upCase($FUNC2.text)}, exp1={$exp1.st}, exp2={$exp2.st})
                                |       ^(FUNC3 exp1 = expression exp2 = expression exp3 = expression?) -> FUNC3TEMPLATE(name={upCase($FUNC3.text)}, exp1={$exp1.st}, exp2={$exp2.st}, exp3={$exp3.st})
                                |       ^(FUNC4 exp1 = expression exp2 = expression exp3 = expression exp4 = expression?) -> FUNC4TEMPLATE(name={upCase($FUNC4.text)}, exp1={$exp1.st}, exp2={$exp2.st}, exp3={$exp3.st}, exp4={$exp4.st})
                                |       ^(BOUNDFUNC  v = var)   -> BOUNDFUNCTemplate(var={$v.st})
                                |       functionCall -> {$functionCall.st}
                                |       '(' expression ( AS var )? ')'  -> expressionTemplate(expression={$expression.st}, var={$var.st})
                                |       v = var  -> groupConditionVarTemplate(var = {$v.st} )
                                ;
    havingClause                :       HAVING (h += havingCondition)+ 
                                        ->havingClauseTemplate(havingConditions = {$h})
                                ;
    havingCondition             :       constraint 
                                        ->{$constraint.st}
                                ;
    orderClause                 :       ORDER BY (o+=orderCondition)+ 
                                        ->orderClauseTemplate(orderConditions ={$o})
                                ;
    orderCondition              :       ( ( asc=ASC | desc=DESC ) be=brackettedExpression ) 
                                        -> orderConditionTemplate(asc={$asc.text}, desc={$desc.text}, brackettedExpression={$be.st})
                                |       ( constraint ->{$constraint.st} | var -> {$var.st} ) 
                                ;
    limitOffsetClauses          :       s+=limitClause s+=offsetClause? 
                                        ->limitOffsetClausesTemplate(clauses={$s})
                                |       s+=offsetClause s+=limitClause? 
                                        ->limitOffsetClausesTemplate(clauses={$s})
                                ;
    
    limitClause                 :       LIMIT INTEGER 
                                        ->limitClauseTemplate(integer={$INTEGER.text})
                                ;
    
    offsetClause                :       OFFSET INTEGER 
                                        ->offsetClauseTemplate(integer={$INTEGER.text})
                                ;
    bindingsClause              :       ( BINDINGS var* '{' ( '(' bindingValue* ')' | NIL )* '}' )? ;
    bindingValue                :       iriRef | rdfLiteral | numericLiteral | booleanLiteral | UNDEF ;
    update                      :       prologue ( update1 ( ';' u=update )? )? 
                                        ->updateTemplate(prologue = {$prologue.st}, update1 = {$update1.st}, update={$u.st})
                                ;
    update1                     :       load 
                                        ->{$load.st}
                                |       clear 
                                        ->{$clear.st}
                                |       drop 
                                        ->{$drop.st}
                                |       add 
                                        ->{$add.st}
                                |       move 
                                        ->{$move.st}
                                |       copy 
                                        ->{$copy.st}
                                |       create 
                                        ->{$create.st}
                                |       insertData 
                                        ->{$insertData.st}
                                |       deleteData 
                                        ->{$deleteData.st}
                                |       deleteWhere 
                                        ->{$deleteWhere.st}
                                |       modify 
                                        ->{$modify.st}
                                ;
    load                        :       LOAD s=SILENT? iriRef ( INTO g=graphRef )? 
                                        ->loadTemplate(silent = {$s.text}, iriRef={$iriRef.st}, graphRef = {$g.st})
                                ;
    clear                       :       CLEAR s=SILENT? graphRefAll 
                                        ->clearTemplate(silent = {$s.text}, graphRefAll = {$graphRefAll.st})
                                ;
    drop                        :       DROP s=SILENT? graphRefAll 
                                        ->dropTemplate(silent = {$s.text}, graphRefAll = {$graphRefAll.st})
                                ;
    create                      :       CREATE s=SILENT? graphRef 
                                        ->createTemplate(silent = {$s.text}, graphRef = {$graphRef.st})
                                ;
    add                         :       ADD s=SILENT? f=graphOrDefault TO t=graphOrDefault 
                                        ->addTemplate(silent={$s.text}, graphOrDefaultFrom = {$f.st}, graphOrDefaultTo = {$t.st})
                                ;
    move                        :       MOVE s=SILENT? f=graphOrDefault TO t=graphOrDefault 
                                        ->moveTemplate(silent={$s.text}, graphOrDefaultFrom = {$f.st}, graphOrDefaultTo = {$t.st})
                                ;
    copy                        :       COPY s=SILENT? f=graphOrDefault TO t=graphOrDefault 
                                        ->copyTemplate(silent={$s.text}, graphOrDefaultFrom = {$f.st}, graphOrDefaultTo = {$t.st})
                                ;
    insertData                  :       INSERT DATA quadData 
                                        ->insertDataTemplate(quadData = {$quadData.st})
                                ;
    deleteData                  :       DELETE DATA quadData 
                                        ->deleteDataTemplate(quadData = {$quadData.st})
                                ;
    deleteWhere                 :       DELETE WHERE quadPattern 
                                        ->deleteWhereTemplate(quadPattern = {$quadPattern.st})
                                ;
    modify                      :       ( WITH iri=iriRef )? ( d1=deleteClause i=insertClause? | i=insertClause ) (u+=usingClause)* WHERE ggp=groupGraphPattern 
                                        ->modifyTemplate(iriRef = {$iri.st}, deleteClause = {$d1.st}, insertClause = {$i.st}, usingClause = {$u}, groupGraphPattern = {$ggp.st})
                                ;
    deleteClause                :       DELETE quadPattern 
                                        ->deleteClauseTemplate(quadPattern = {$quadPattern.st})
                                ;
    insertClause                :       INSERT quadPattern 
                                        ->insertClauseTemplate(quadPattern = {$quadPattern.st})
                                ;
    usingClause                 :       USING ( i1 = iriRef | NAMED i2 = iriRef ) 
                                        ->usingClauseTemplate(iriRef = {$i1.st}, nameIRIRef = {$i2.st})
                                ;
    graphOrDefault              :       DEFAULT | GRAPH? iriRef 
                                
                                ;
    graphRef                    :       GRAPH iriRef 
                                        ->graphRefTemplate(iriRef={$iriRef.st})
                                ;
    graphRefAll                 :       g=graphRef | d=DEFAULT | n=NAMED | a=ALL 
                                        ->graphRefAllTemplate(graphRef = {$g.st}, def = {$d.text}, named = {$n.text}, all={$a.text})
                                ;
    quadPattern                 :       '{' quads '}' 
                                        -> {$quads.st}
                                ;
    quadData                    :       '{' quads '}' 
                                        -> {$quads.st}
                                ;
    quads                       :       t = triplesTemplate? ( q+=quadsNotTriples_tripleTemplate )* 
                                        ->quadsTemplate(triplesTemplate = {$t.st}, quadsNotTriples_tripleTemplate = {$q})
                                ;
    quadsNotTriples_tripleTemplate  :   q = quadsNotTriples '.'? t = triplesTemplate?
                                        ->quadsNotTriples_tripleTemplate(quadsNotTriples = {$q.st}, triplesTemplate = {$t.st})
                                ;
    quadsNotTriples             :       GRAPH varOrIRIref '{' triplesTemplate? '}' 
                                        ->quadsNotTriplesTemplate(varOrIRIref = {$varOrIRIref.st}, triplesTemplate = {$triplesTemplate.st})
                                ;
    triplesTemplate             :       tss = triplesSameSubject ( '.' tt= triplesTemplate? )? 
                                        ->triplesTemplateTemplate(triplesSameSubject = {$tss.st}, triplesTemplate = {$tt.st})
                                ;
    groupGraphPattern           :       '{' ( s=subSelect | g=groupGraphPatternSub ) '}' 
                                        -> groupGraphPatternTemplate(subSelect={$s.st}, groupGraphPatternSub={$g.st})
                                ;
    groupGraphPatternSub        :       t+=triplesBlock? ( g+=graphPatternNotTriples '.'? t+=triplesBlock? )* 
                                        ->groupGraphPatternSubTemplate(triplesBlock={$t}, graphPatternNotTriples={$g})
                                ;
    triplesBlock                :       t=triplesSameSubjectPath ( '.' b=triplesBlock? )? 
    
                                        ->triplesBlockTemplate(triplesSameSubjectPath={$t.st}, triplesBlock={$b.st})
                                ;
    graphPatternNotTriples      :       groupOrUnionGraphPattern
                                        ->{$groupOrUnionGraphPattern.st}
                                |       optionalGraphPattern 
                                        ->{$optionalGraphPattern.st}
                                |       minusGraphPattern 
                                        ->{$minusGraphPattern.st}
                                |       graphGraphPattern 
                                        ->{$graphGraphPattern.st}
                                |       serviceGraphPattern 
                                        ->{$serviceGraphPattern.st}
                                |       filter
                                        ->{$filter.st}
                                |       bind 
                                        ->{$bind.st}
                                ;
    optionalGraphPattern        :       OPTIONAL groupGraphPattern
                                        ->optionalGraphPatternTemplate(groupGraphPattern={$groupGraphPattern.st})
                                ;
    graphGraphPattern           :       GRAPH varOrIRIref groupGraphPattern 
                                        -> graphGraphPatternTemplate(varOrIRIref={$varOrIRIref.st}, groupGraphPattern={$groupGraphPattern.st})
                                ;
    serviceGraphPattern         :       SERVICE silent=SILENT? varOrIRIref groupGraphPattern 
                                        ->serviceGraphPatternTemplate(silent={$silent.text}, varOrIRIref={$varOrIRIref.st}, groupGraphPattern={$groupGraphPattern.st})
                                ;
    bind                        :       BIND '(' expression AS var ')' 
                                        ->bindTemplate(expression = {$expression.st}, var={$var.st})
                                ;
    minusGraphPattern           :       MINUS groupGraphPattern 
                                        ->minusGraphPatternTemplate(groupGraphPattern={$groupGraphPattern.st})
                                ;
    groupOrUnionGraphPattern    :       g=groupGraphPattern ( UNION list+=groupGraphPattern )* 
                                        ->groupOrUnionGraphPatternTemplate(g={$g.st}, list={$list})
                                ;
    filter                      :       FILTER constraint 
                                        ->filterTemplate(constraint={$constraint.st})
                                ;
    constraint                  :       brackettedExpression -> {$brackettedExpression.st} 
                                |       ^(FUNC0 NIL)    ->  FUNC0Template(name={upCase($FUNC0.text)})
                                |       ^(FUNC1 exp1 = expression) -> FUNC1Template(name={upCase($FUNC1.text)}, exp1={$exp1.st})
                                |       ^(FUNC1OPT exp1 = expression)  -> FUNC1OPTTemplate(name={$FUNC1OPT.text}, exp1={$exp1.st})
                                |       ^(FUNC2 exp1 = expression exp2 = expression)    ->  FUNC2TEMPLATE(name={upCase($FUNC2.text)}, exp1={$exp1.st}, exp2={$exp2.st})
                                |       ^(FUNC3 exp1 = expression exp2 = expression exp3 = expression?) -> FUNC3TEMPLATE(name={upCase($FUNC3.text)}, exp1={$exp1.st}, exp2={$exp2.st}, exp3={$exp3.st})
                                |       ^(FUNC4 exp1 = expression exp2 = expression exp3 = expression exp4 = expression?) -> FUNC4TEMPLATE(name={upCase($FUNC4.text)}, exp1={$exp1.st}, exp2={$exp2.st}, exp3={$exp3.st}, exp4={$exp4.st})
                                |       ^(BOUNDFUNC  v = var)   -> BOUNDFUNCTemplate(var={$v.st})
                                |       functionCall -> {$functionCall.st}
                                |       ^(NOTEXISTS  groupGraphPattern) -> NOTEXISTSTemplate(groupGraphPattern={$groupGraphPattern.st})
                                |       ^(EXISTS  groupGraphPattern) -> EXISTSTemplate(groupGraphPattern={$groupGraphPattern.st})
                                ;
    functionCall                :       iriRef argList
                                        ->functionCallTemplate(iriRef={$iriRef.st}, argList={$argList.st})
                                ;
    argList                     :       NIL 
                                        ->
                                |       '(' dist=DISTINCT? exprs += expression ( ',' exprs+=expression )* ')' 
                                        ->argListTemplate(distinct={$dist.text}, exprs={$exprs})
                                ;
    expressionList              :       NIL 
                                |       '(' exprs+=expression ( ',' exprs+=expression )* ')' 
                                        ->expressionListTemplate(exprs={$exprs})
                                ;
    constructTemplate           :       '{' constructTriples? '}' 
                                        ->constructTemplateTemplate(constructTriples = {$constructTriples.st})
                                ;
    constructTriples            :       t=triplesSameSubject ( '.' c=constructTriples? )? 
                                        ->constructTriplesTemplate(triplesSameSubject = {$t.st}, constructTriples={$c.st})
                                ;
    triplesSameSubject          :       varOrTerm propertyListNotEmpty 
                                        ->triplesSameSubjectTemplate1(varOrTerm = {$varOrTerm.st}, propertyListNotEmpty = {$propertyListNotEmpty.st})
                                |       triplesNode propertyList 
                                        ->triplesSameSubjectTemplate2(triplesNode = {$triplesNode.st}, propertyList={$propertyList.st})
                                ;
                                
    propertyListNotEmpty        :       vo+=verbObjectList ( ';' ( vo+=verbObjectList )? )* 
                                        ->propertyListNotEmptyTemplate(verbObjectList = {$vo})
                                ;
                                
    verbObjectList              :       verb objectList
                                        ->verbObjectListTemplate(verb = {$verb.st}, objectList = {$objectList.st})
                                ;
    
    propertyList                :       propertyListNotEmpty? 
                                        ->{$propertyListNotEmpty.st}
                                ;
    
    objectList                  :       o+=object ( ',' o+=object )* 
                                        ->objectListTemplate(objects={$o})
                                ;
    object                      :       graphNode 
                                        ->{$graphNode.st}
                                ;
    verb                        :       varOrIRIref 
                                        ->{$varOrIRIref.st}
                                |       'a' -> pathPrimary_aTemplate()
                                
                                ;
                                
    triplesSameSubjectPath      :       varOrTerm propertyListNotEmptyPath 
                                        ->triplesSameSubjectPathTemplate1(varOrTerm={$varOrTerm.st}, propertyListNotEmptyPath={$propertyListNotEmptyPath.st})
                                |
                                        triplesNode propertyListPath 
                                        ->triplesSameSubjectPathTemplate2(triplesNode={$triplesNode.st}, propertyListPath={$propertyListPath.st})
                                ;
                                      
    propertyListNotEmptyPath    :       v+=verbPathOrSimpleObjectList ( ';' ( v+=verbPathOrSimpleObjectList )? )* 
                                        ->propertyListNotEmptyPathTemplate(verbPathOrSimpleObjectList={$v})
                                ;
    
    verbPathOrSimpleObjectList  :       ( vp=verbPath | vs=verbSimple ) vo=objectList
                                        ->verbPathOrSimpleObjectListTemplate(verbPath={$vp.st}, verbSimple={$vs.st}, objectList={$vo.st})
                                ;
    propertyListPath            :       propertyListNotEmpty? ;
    verbPath                    :       path 
                                        ->{$path.st}
                                ;
    verbSimple                  :       var 
                                        ->{$var.st}
                                ;
    
    path                        :       ^('|' path1 = path path2 = path) -> AltPathTemplate(path1 = {$path1.st}, path2 = {$path2.st})
                                |       ^('/' path1 = path path2 = path) -> SeqPathTemplate(path1 = {$path1.st}, path2 = {$path2.st})
                                |       ^('^' path1 = path ) -> InvPathTemplate(path1 = {$path1.st})
                                |       ^(ELT path1 = path pmod = pathMod?)  -> EltPathTemplate(path1 = {$path1.st}, pathMod = {$pmod.st})
                                |       ^('!' path1 = path) -> negatedPathTemplate(path1={$path1.st})
                                |       iriRef -> {$iriRef.st}
                                |       'a' -> pathPrimary_aTemplate()
                                
                                ;
    
/*   
    path                        :       pathAlternative 
                                        ->{$pathAlternative.st}
                                ;
    pathAlternative             :       ps+=pathSequence ( '|' ps+=pathSequence )* 
                                        ->pathAlternativeTemplate(pathSequence={$ps})
    
                                ;
    pathSequence                :       peoi+=pathEltOrInverse ( '/' peoi+=pathEltOrInverse )* 
                                        ->pathSequenceTemplate(pathEltOrInverse={$peoi})
                                ;
    pathElt                     :       pathPrimary pathMod? 
                                        ->pathEltTemplate(pathPrimary={$pathPrimary.st}, pathMod={$pathMod.st})
                                ;
    pathEltOrInverse            :       pathElt 
                                        ->{$pathElt.st}
                                |       '^' pathElt 
                                        ->pathInverseTemplate(pathElt={$pathElt.st})
                                ; */
    pathMod                     :       '*' 
                                        ->pathModStarTemplate()
                                |       '?' 
                                        ->pathModQuestionTemplate()
                                |       '+' 
                                        ->pathModPlusTemplate()
                                |       '{' integer ',' '}'
                                        ->pathModToOpenTemplate(i1 = {$integer.st})
                                |       '{' i1 = integer ',' i2 = integer '}'
                                        ->pathModRangeTemplate(i1 = {$i1.st}, i2 = {$i2.st})
                                |       '{' integer '}'
                                        ->pathModFixedTemplate(i1 = {$integer.st})
                                |       '{' ',' integer '}'
                                        ->pathModFromOpenTemplate(i1 = {$integer.st})
                                ;
                                
                                
/*   
    pathPrimary                 :       iriRef 
                                        -> pathPrimary_iriRefTemplate(iriRef={$iriRef.st})
                                |       'a' 
                                        -> pathPrimary_aTemplate()
                                |       '!' pathNegatedPropertySet 
                                        ->pathPrimary_pathNegatedPropertySetTemaplte(pathNegatedPropertySet={$pathNegatedPropertySet.st})
                                |       '(' path ')' 
                                        ->{$path.st}
                                ;
    pathNegatedPropertySet      :       pathOneInPropertySet | '(' ( pathOneInPropertySet ( '|' pathOneInPropertySet )* )? ')' ;
    pathOneInPropertySet        :       iriRef | 'a' | '^' ( iriRef | 'a' ) ;
*/
    integer                     :       INTEGER 
                                        ->integerTemplate(integer={$INTEGER.text})
                                ;
    triplesNode                 :       collection 
                                        ->{$collection.st}
                                |       blankNodePropertyList 
                                        ->{$blankNodePropertyList.st}
                                ;
    blankNodePropertyList       :       '[' propertyListNotEmpty ']' 
                                        ->blankNodePropertyListTemplate(propertyListNotEmpty={$propertyListNotEmpty.st})
                                ;
    collection                  :       '(' g+=graphNode+ ')' 
                                        ->collectionTemplate(graphNodes={$g})
                                ;
    graphNode                   :       varOrTerm 
                                        ->{$varOrTerm.st}
                                | 
                                        triplesNode 
                                        ->{$triplesNode.st}
                                ;
    varOrTerm                   :       var 
                                        ->{$var.st}
                                | 
                                        graphTerm 
                                        ->{$graphTerm.st}
                                ;
    varOrIRIref                 :       var 
                                        ->{$var.st}
                                | 
                                        iriRef 
                                        ->{$iriRef.st}
                                ;
    var                         :        VAR1 {storeSym($VAR1.text);  }    
                                        ->VARTemplate(var={retriveSym($VAR1.text)})
                                |        VAR2 { storeSym($VAR2.text); }
                                        ->VARTemplate(var={retriveSym($VAR2.text)})
                                ;
    graphTerm                   :       
                                        iriRef 
                                        ->{$iriRef.st}
                                |       rdfLiteral 
                                        ->{$rdfLiteral.st}
                                |       numericLiteral
                                        ->{$numericLiteral.st}
                                |       booleanLiteral 
                                        ->{$booleanLiteral.st}
                                |       blankNode 
                                        ->{$blankNode.st}
                                |       NIL 
                                        ->NILTemplate()
                                ;
    
    expression                  :       ^('||' a = expression b = expression)  -> logicalOrTemplate(a={$a.st}, b={$b.st})
                                |       ^('&&' a = expression b = expression)  -> logicalANDTemplate(a={$a.st}, b={$b.st})
                                |       ^('=' a = expression b = expression)  -> logicalEqualsTemplate(a={$a.st}, b={$b.st})
                                |       ^('!=' a = expression b = expression)  -> logicalNotEqualsTemplate(a={$a.st}, b={$b.st})
                                |       ^('<' a = expression b = expression)  -> logicalLTTemplate(a={$a.st}, b={$b.st})
                                |       ^('>' a = expression b = expression)  -> logicalGTTemplate(a={$a.st}, b={$b.st})
                                |       ^('>=' a = expression b = expression)  -> logicalGTETemplate(a={$a.st}, b={$b.st})
                                |       ^('<=' a = expression b = expression)  -> logicalLTETemplate(a={$a.st}, b={$b.st})
                                |       ^(IN a = expression b = expression)  -> logicalINTemplate(a={$a.st}, b={$b.st})
                                |       ^(NOT a = expression b = expression)  -> logicalNOTINTemplate(a={$a.st}, b={$b.st})
                                |       ^('+' a = expression b = expression)  -> ADDTemplate(a={$a.st}, b={$b.st})
                                |       ^('-' a = expression b = expression)  -> SUBTemplate(a={$a.st}, b={$b.st})
                                |       ^('*' a = expression b = expression)  -> MIULTTemplate(a={$a.st}, b={$b.st})
                                |       ^('/' a = expression b = expression)  -> DIVIDEOrTemplate(a={$a.st}, b={$b.st})
                                |       ^('!' a = expression)  -> NOTOrTemplate(a={$a.st})
                                |       ^(UADD a = expression)  -> UNARYPLUSOrTemplate(a={$a.st})
                                |       ^(UMINUS a = expression)  -> UNARYMINUSTemplate(a={$a.st})
                                |       ^(FUNC0 NIL)    ->  FUNC0Template(name={upCase($FUNC0.text)})
                                |       ^(FUNC1 exp1 = expression)  -> FUNC1Template(name={upCase($FUNC1.text)}, exp1={$exp1.st})
                                |       ^(FUNC1OPT exp1 = expression) -> FUNC1OPTTemplate(name={$FUNC1OPT.text}, exp1={$exp1.st})
                                |       ^(FUNC2 exp1 = expression exp2 = expression)    ->  FUNC2TEMPLATE(name={upCase($FUNC2.text)}, exp1={$exp1.st}, exp2={$exp2.st})
                                |       ^(FUNC3 exp1 = expression exp2 = expression exp3 = expression?) -> FUNC3TEMPLATE(name={upCase($FUNC3.text)}, exp1={$exp1.st}, exp2={$exp2.st}, exp3={$exp3.st})
                                |       ^(FUNC4 exp1 = expression exp2 = expression exp3 = expression exp4 = expression?) -> FUNC4TEMPLATE(name={upCase($FUNC4.text)}, exp1={$exp1.st}, exp2={$exp2.st}, exp3={$exp3.st}, exp4={$exp4.st})
                                |		^(FUNC_WITH_LIST expressionList) -> FUNC_WITH_LISTTemplate(name ={$FUNC_WITH_LIST.text}, expressionList = {$expressionList.st})
                                |       ^(BOUNDFUNC  v = var)   -> BOUNDFUNCTemplate(var={$v.st})
                                |       ^(SUM dist = DISTINCT? exp1 = expression) -> SUMTemplate(distinct={$dist.text}, exp1={$exp1.st})
                                |       ^(MIN dist = DISTINCT? exp1 = expression) -> MINTemplate(distinct={$dist.text}, exp1={$exp1.st})
                                |       ^(MAX dist = DISTINCT? exp1 = expression) -> MAXTemplate(distinct={$dist.text}, exp1={$exp1.st})
                                |       ^(AVG dist = DISTINCT? exp1 = expression) -> AVGTemplate(distinct={$dist.text}, exp1={$exp1.st})
                                |       ^(SAMPLE dist = DISTINCT? exp1 = expression) -> SAMPLETemplate(distinct={$dist.text}, exp1={$exp1.st})
                                |       ^(NOTIMPLEMENTED NIL)   ->  NOTIMPLEMENTEDTemplate()
                                |       ^(VIRIREF argList?) ->  VIRIREFTemplate(argList={$argList.st})
                                |       ^(COUNT dist = DISTINCT? star = '*'? exp1 = expression) ->  COUNTTemplate(distinct={$dist.text}, star={$star.text}, exp1={$exp1.st})
                                |       ^(NOTEXISTS  groupGraphPattern) -> NOTEXISTSTemplate(groupGraphPattern={$groupGraphPattern.st})
                                |       ^(EXISTS  groupGraphPattern) -> EXISTSTemplate(groupGraphPattern={$groupGraphPattern.st})
                                |       ^(VIRIREF_OR_FUNCTION_CALL iriRef argList?) -> VIRIREF_OR_FUNCTION_CALLTemplate(iriRef={$iriRef.st}, argList={$argList.st})
                                |       rdfLiteral  -> {$rdfLiteral.st}
                                |       numericLiteral ->{$numericLiteral.st}
                                |       booleanLiteral ->{$booleanLiteral.st}
                                |       var ->  {$var.st}
                                |       brackettedExpression -> {$brackettedExpression.st}
                                ;
    brackettedExpression        :       '(' expression ')'
                                        ->brackettedExpressionTemplate(expression={$expression.st})
                                ;
    rdfLiteral                  :       string ( langtag=LANGTAG | ( '^^' iri=iriRef ) )? 
                                        ->rdfLiteralTemplate(str={escapeEscape($string.text)}, langtag={$langtag.text}, iriref={$iri.st})
                                ;
    numericLiteral              :       nl1 = numericLiteralUnsigned 
                                        ->numericLiteralUnsignedTemplate(numericLiteral = {$nl1.text})
                                |       nl2 = numericLiteralPositive 
                                        ->numericLiteralPositiveTemplate(numericLiteral = {$nl2.text})
                                |       nl3 = numericLiteralNegative 
                                        ->numericLiteralNegativeTemplate(numericLiteral = {$nl3.text})
                                ;
    numericLiteralUnsigned      :       INTEGER | DECIMAL | DOUBLE ;
    numericLiteralPositive      :       INTEGER_POSITIVE | DECIMAL_POSITIVE | DOUBLE_POSITIVE ;
    numericLiteralNegative      :       INTEGER_NEGATIVE | DECIMAL_NEGATIVE | DOUBLE_NEGATIVE ;
    booleanLiteral              :       TRUE 
                                        ->TRUE()
                                | 
                                        FALSE 
                                        -> FALSE()
                                ;
    string                      :       STRING_LITERAL1 | STRING_LITERAL2 | STRING_LITERAL_LONG1 | STRING_LITERAL_LONG2 ;
    iriRef                      :       IRI_REF 
                                        ->iriRef_IRI_REFTemplate(iri={$IRI_REF.text})
                                |       prefixedName 
                                        ->iriRef_prefixedNameTemplate(prefixedName={$prefixedName.text})
                                ;
    prefixedName                :       PNAME_LN | PNAME_NS ;
    blankNode                   :       BLANK_NODE_LABEL 
                                        ->blankNodeTemplate(label={$BLANK_NODE_LABEL.text})
                                |       ANON 
                                		->blankNodeTemplate(label={generateUniqueAnonSym()})
                                        
                                ;
