/*
*+------------------------------------------------------------------------+
*| Licensed Materials - Property of IBM                           |
*| (C) Copyright IBM Corporation 2012. All Rights Reserved.       |
*| @author Sandeep Kohli                                                              |
*| US Government Users Restricted Rights - Use, duplication or disclosure |
*| restricted by GSA ADP Schedule Contract with IBM Corp.         |
*+------------------------------------------------------------------------+
*/

grammar Sparql11;

options {
  language = Java;
  output=AST;
  ASTLabelType=CommonTree;
}

tokens {
    SELQUERY;
    VAR;
    PROLOGUE;
    FUNC1OPT;
    FUNC0;
    FUNC1;
    FUNC2;
    FUNC3;
    FUNC4;
    BOUNDFUNC;
    NOTEXISTS;
    NOTIMPLEMENTED;
    VIRIREF_OR_FUNCTION_CALL;
    BRACKETTED_EXPRESSION;
    UMINUS;
    UADD;
    ELT;
    FUNC_WITH_LIST;

}
@header {

/*
*+------------------------------------------------------------------------+
*| Licensed Materials - Property of IBM                           |
*| (C) Copyright IBM Corporation 2012. All Rights Reserved.       |
*| @author Sandeep Kohli                                                              |
*| US Government Users Restricted Rights - Use, duplication or disclosure |
*| restricted by GSA ADP Schedule Contract with IBM Corp.         |
*+------------------------------------------------------------------------+
*/
    package com.ibm.xtools.rmpx.sparqlRDF;
}

@lexer::header {

/*
*+------------------------------------------------------------------------+
*| Licensed Materials - Property of IBM                           |
*| (C) Copyright IBM Corporation 2012. All Rights Reserved.       |
*| @author Sandeep Kohli                                                              |
*| US Government Users Restricted Rights - Use, duplication or disclosure |
*| restricted by GSA ADP Schedule Contract with IBM Corp.         |
*+------------------------------------------------------------------------+
*/
    package com.ibm.xtools.rmpx.sparqlRDF;
}


@members 
{
    
  @Override
  protected Object recoverFromMismatchedToken(IntStream input, int ttype, BitSet follow) throws RecognitionException {
    throw new MismatchedTokenException(ttype, input);
  }

  @Override
  public Object recoverFromMismatchedSet(IntStream input, RecognitionException e, BitSet follow) throws RecognitionException {
    throw e;
  }


}

@rulecatch 
{
    catch (RecognitionException e ) {
    // Don't want error recovery. Just throw exception and get out!
        throw e;
    }
    
}
    queryUnit                   :       query EOF! ;
    query                       :       prologue ( selectQuery |    constructQuery | describeQuery | askQuery ) bindingsClause
                                        
                                ;
    updateUnit                  :       update ;
    prologue                    :       (baseDecl |    prefixDecl )*
                                   
                                ;
    baseDecl                    :       BASE IRI_REF
                                  
                                ;
    prefixDecl                  :       PREFIX PNAME_NS IRI_REF
                                    
                                ;
    selectQuery                 :       selectClause datasetClause* whereClause solutionModifier
                                   
                                ;
    subSelect                   :       selectClause whereClause solutionModifier ;
    
    selectClause                :       SELECT ( dis=DISTINCT |    red=REDUCED )? ( ( varClause )+ | '*' )
                    
                                  
                                ;
    varClause                   :       var 
                                    
                                |       '(' expression AS var ')'  
                                ;
    
    constructQuery              :       CONSTRUCT ( constructTemplate datasetClause* whereClause solutionModifier |    datasetClause* WHERE '{' triplesTemplate? '}' solutionModifier ) ;
    describeQuery               :       DESCRIBE ( varOrIRIref+ |    '*' ) datasetClause* whereClause? solutionModifier ;
    askQuery                    :       ASK datasetClause* whereClause solutionModifier ;
    datasetClause               :       FROM ( defaultGraphClause |    namedGraphClause );
    defaultGraphClause          :       sourceSelector ;
    namedGraphClause            :       NAMED sourceSelector ;
    sourceSelector              :       iriRef ;
    whereClause                 :       WHERE? groupGraphPattern ;
    solutionModifier            :       groupClause? havingClause? orderClause? limitOffsetClauses? ;
    groupClause                 :       GROUP BY groupCondition+ ;
    groupCondition              :       builtInCall |    functionCall | '(' expression ( AS var )? ')' | var ;
    havingClause                :       HAVING havingCondition+ ;
    havingCondition             :       constraint ;
    orderClause                 :       ORDER BY orderCondition+ 
                                ;
    orderCondition              :       ( ( ASC |    DESC ) brackettedExpression ) 
                                        |       ( constraint | var ) ;
    limitOffsetClauses          :       limitClause offsetClause? |    offsetClause limitClause? ;
    limitClause                 :       LIMIT INTEGER ;
    offsetClause                :       OFFSET INTEGER ;
    bindingsClause              :       ( BINDINGS var* '{' ( '(' bindingValue* ')' |    NIL )* '}' )? ;
    bindingValue                :       iriRef |    rdfLiteral | numericLiteral | booleanLiteral | UNDEF ;
    update                      :       prologue ( update1 ( ';' update )? )? ;
    update1                     :       load |    clear | drop | add | move | copy | create | insertData | deleteData | deleteWhere | modify ;
    load                        :       LOAD SILENT? iriRef ( INTO graphRef )? ;
    clear                       :       CLEAR SILENT? graphRefAll ;
    drop                        :       DROP SILENT? graphRefAll ;
    create                      :       CREATE SILENT? graphRef ;
    add                         :       ADD SILENT? graphOrDefault TO graphOrDefault ;
    move                        :       MOVE SILENT? graphOrDefault TO graphOrDefault ;
    copy                        :       COPY SILENT? graphOrDefault TO graphOrDefault ;
    insertData                  :       INSERT DATA quadData ;
    deleteData                  :       DELETE DATA quadData ;
    deleteWhere                 :       DELETE WHERE quadPattern ;
    modify                      :       ( WITH iriRef )? ( deleteClause insertClause? |  insertClause ) usingClause* WHERE groupGraphPattern ;
    deleteClause                :       DELETE quadPattern;
    insertClause                :       INSERT quadPattern;
    usingClause                 :       USING ( iriRef |    NAMED iriRef );
    graphOrDefault              :       DEFAULT |    GRAPH? iriRef ;
    graphRef                    :       GRAPH iriRef ;
    graphRefAll                 :       graphRef |    DEFAULT | NAMED | ALL ;
    quadPattern                 :       '{' quads '}' ;
    quadData                    :       '{' quads '}' ;
    quads                       :       triplesTemplate? ( quadsNotTriples '.'? triplesTemplate? )* ;
    quadsNotTriples             :       GRAPH varOrIRIref '{' triplesTemplate? '}' ;
    triplesTemplate             :       triplesSameSubject ( '.' triplesTemplate? )? ;
    groupGraphPattern           :       '{' ( subSelect |    groupGraphPatternSub ) '}' ;
    groupGraphPatternSub        :       triplesBlock? ( graphPatternNotTriples '.'? triplesBlock? )* ;
    triplesBlock                :       triplesSameSubjectPath ( '.' triplesBlock? )? ;
    graphPatternNotTriples      :       groupOrUnionGraphPattern |    optionalGraphPattern | minusGraphPattern | graphGraphPattern | serviceGraphPattern | filter | bind ;
    optionalGraphPattern        :       OPTIONAL groupGraphPattern ;
    graphGraphPattern           :       GRAPH varOrIRIref groupGraphPattern ;
    serviceGraphPattern         :       SERVICE SILENT? varOrIRIref groupGraphPattern ;
    bind                        :       BIND '(' expression AS var ')' ;
    minusGraphPattern           :       MINUS groupGraphPattern ;
    groupOrUnionGraphPattern    :       groupGraphPattern ( UNION groupGraphPattern )* ;
    filter                      :       FILTER constraint ;
    constraint                  :       brackettedExpression |    builtInCall | functionCall ;
    functionCall                :       iriRef argList ;
    argList                     :       NIL |    '(' DISTINCT? expression ( ',' expression )* ')' ;
    expressionList              :       NIL |    '(' expression ( ',' expression )* ')' ;
    constructTemplate           :       '{' constructTriples? '}' ;
    constructTriples            :       triplesSameSubject ( '.' constructTriples? )? ;
    triplesSameSubject          :       varOrTerm propertyListNotEmpty |    triplesNode propertyList ;
    propertyListNotEmpty        :       verbObjectList ( ';' ( verbObjectList )? )* ;
    
    verbObjectList              :       verb objectList
                                ;
    
    propertyList                :       propertyListNotEmpty? ;
    objectList                  :       object ( ',' object )* ;
    object                      :       graphNode ;
    verb                        :       varOrIRIref |    'a' ;
    triplesSameSubjectPath      :       varOrTerm propertyListNotEmptyPath |    triplesNode propertyListPath ;
    propertyListNotEmptyPath    :       verbPathOrSimpleObjectList ( ';' ( verbPathOrSimpleObjectList )? )* ;
    verbPathOrSimpleObjectList  :       ( verbPath |    verbSimple ) objectList
                                ;
    propertyListPath            :       propertyListNotEmpty? ;
    verbPath                    :       path ;
    verbSimple                  :       var ;
    path                        :       pathAlternative ;
    pathAlternative             :       pathSequence ( '|'^ pathSequence )* ;
    pathSequence                :       pathEltOrInverse ( '/'^ pathEltOrInverse )* ;
    pathEltOrInverse            :       pathElt 
                                
                                |       '^'^ pathElt 
                                
                                ;
    pathElt                     :       pathPrimary pathMod? 
                                        ->^(ELT pathPrimary pathMod?)
                                ;
    pathMod                     :       '*' |    '?' | '+' | '{' ( integer ( ',' ( '}' | integer '}' ) | '}' ) | ',' integer '}' ) ;
    pathPrimary                 :       iriRef |    'a' | '!'^ pathNegatedPropertySet | '('! path ')'! ;
    pathNegatedPropertySet      :       pathOneInPropertySet |    '('! ( pathOneInPropertySet ( '|'^ pathOneInPropertySet )* )? ')'! ;
    pathOneInPropertySet        :       iriRef |    'a' | '^'^ ( iriRef | 'a' ) ;
    integer                     :       INTEGER ;
    triplesNode                 :       collection |    blankNodePropertyList ;
    blankNodePropertyList       :       '[' propertyListNotEmpty ']' ;
    collection                  :       '(' graphNode+ ')' ;
    graphNode                   :       varOrTerm |    triplesNode ;
    varOrTerm                   :       var |    graphTerm ;
    varOrIRIref                 :       var |    iriRef ;
    var                         :       VAR1 
                                  
                                |       VAR2
                                    
                                ;
    graphTerm                   :       iriRef |    rdfLiteral | numericLiteral | booleanLiteral | blankNode | NIL ;
    
    expression                  :       conditionalOrExpression ;
    conditionalOrExpression     :       conditionalAndExpression ( '||'^ conditionalAndExpression )* ;
    conditionalAndExpression    :       valueLogical ( '&&'^ valueLogical )* ;
    valueLogical                :       relationalExpression ;
    relationalExpression        :       numericExpression ( '='^ numericExpression |    '!='^ numericExpression | '<'^ numericExpression | '>'^ numericExpression | '<='^ numericExpression | '>='^ numericExpression | IN^ expressionList | NOT^ IN! expressionList )? ;
    numericExpression           :       additiveExpression ;
    additiveExpression          :       multiplicativeExpression ( '+'^ multiplicativeExpression |    '-'^ multiplicativeExpression | ( numericLiteralPositive | numericLiteralNegative ) ( ( '*'^ unaryExpression ) | ( '/'^ unaryExpression ) )? )* ;
    multiplicativeExpression    :       unaryExpression ( '*'^ unaryExpression |    '/'^ unaryExpression )* ;
    unaryExpression             :       '!'^ primaryExpression 
                                |       '+' primaryExpression -> ^(UMINUS primaryExpression)
                                |       '-' primaryExpression -> ^(UADD primaryExpression)
                                |       primaryExpression ;
    primaryExpression           :       brackettedExpression 
                                |       builtInCall 
                                |       iriRefOrFunction 
                                |       rdfLiteral 
                                |       numericLiteral 
                                |       booleanLiteral 
                                |       var 
                                |       aggregate ;
    brackettedExpression        :       '(' expression ')'
                                ;
    builtInCall                 :       func = STR '(' expression ')'  -> ^(FUNC1[$func] expression )
                                |       func = LANG '(' expression ')'        ->  ^(FUNC1[$func]  expression )
                                |       func = LANGMATCHES '(' exp1 = expression ',' exp2 = expression ')'        ->  ^(FUNC2[$func] $exp1 $exp2)
                                |       func = DATATYPE '(' expression ')'        ->  ^(FUNC1[$func]  expression )
                                |       func = BOUND '(' var ')'      ->  ^(BOUNDFUNC[$func] var)
                                |       func = IRI '(' expression ')'     ->  ^(FUNC1[$func]  expression )
                                |       func = URI '(' expression ')'     ->  ^(FUNC1[$func]  expression )
                                |       func = BNODE ( '(' expression ')' | NIL )     ->  ^(FUNC1OPT[$func]  expression? )
                                |       func = RAND NIL       ->^(FUNC0[$func] NIL)
                                |       func = ABS '(' expression ')'     ->  ^(FUNC1[$func]  expression )
                                |       func = CEIL '(' expression ')'        ->  ^(FUNC1[$func]  expression )
                                |       func = FLOOR '(' expression ')'       ->  ^(FUNC1[$func]  expression )
                                |       func = ROUND '(' expression ')'       ->  ^(FUNC1[$func]  expression )
                                |       func = CONCAT expressionList		  -> ^(FUNC_WITH_LIST[$func] expressionList)
                                |       substringExpression
                                |       func = STRLEN '(' expression ')'      ->  ^(FUNC1[$func]  expression )
                                |       strReplaceExpression
                                |       func = UCASE '(' expression ')'       ->  ^(FUNC1[$func]  expression )
                                |       func = LCASE '(' expression ')'       ->  ^(FUNC1[$func]  expression )
                                |       func = 'ENCODE_FOR_URI' '(' expression ')'      ->  ^(FUNC1[$func]  expression )
                                |       func = CONTAINS '(' exp1 = expression ',' exp2 = expression ')'       ->  ^(FUNC2[$func] $exp1 $exp2)
                                |       func = STRSTARTS '(' exp1 = expression ',' exp2 = expression ')'      ->  ^(FUNC2[$func] $exp1 $exp2)
                                |       func = STRENDS '(' exp1 = expression ',' exp2 = expression ')'        ->  ^(FUNC2[$func] $exp1 $exp2)
                                |       func = STRBEFORE '(' exp1 = expression ',' exp2 = expression ')'      ->  ^(FUNC2[$func] $exp1 $exp2)
                                |       func = STRAFTER '(' exp1 = expression ',' exp2 = expression ')'       ->  ^(FUNC2[$func] $exp1 $exp2)
                                |       func = YEAR '(' expression ')'        ->  ^(FUNC1[$func]  expression )
                                |       func = MONTH '(' expression ')'       ->  ^(FUNC1[$func]  expression )
                                |       func = DAY '(' expression ')'     ->  ^(FUNC1[$func]  expression )
                                |       func = HOURS '(' expression ')'       ->  ^(FUNC1[$func]  expression )
                                |       func = MINUTES '(' expression ')'     ->  ^(FUNC1[$func]  expression )
                                |       func = SECONDS '(' expression ')'     ->  ^(FUNC1[$func]  expression )
                                |       func = TIMEZONE '(' expression ')'        ->  ^(FUNC1[$func]  expression )
                                |       func = TZ '(' expression ')'      ->  ^(FUNC1[$func]  expression )
                                |       func = NOW NIL       ->^(FUNC0[$func] NIL)
                                |       func = MD5 '(' expression ')'     ->  ^(FUNC1[$func]  expression )
                                |       func = SHA1 '(' expression ')'        ->  ^(FUNC1[$func]  expression )
                                |       func = SHA256 '(' expression ')'      ->  ^(FUNC1[$func]  expression )
                                |       func = SHA384 '(' expression ')'      ->  ^(FUNC1[$func]  expression )
                                |       func = SHA512 '(' expression ')'      ->  ^(FUNC1[$func]  expression )
                                |       func = COALESCE expressionList		  ->  ^(FUNC_WITH_LIST[$func] expressionList)
                                |       func = IF '(' expression ',' expression ',' expression ')'
                                |       func = STRLANG '(' expression ',' expression ')'      ->  ^(FUNC2[$func] expression expression)
                                |       func = STRDT '(' expression ',' expression ')'        ->  ^(FUNC2[$func] expression expression)
                                |       func = SAMETERM '(' expression ',' expression ')'     ->  ^(FUNC2[$func] expression expression)
                                |       func = ISIRI '(' expression ')'       ->  ^(FUNC1[$func]  expression )
                                |       func = ISURI '(' expression ')'       ->  ^(FUNC1[$func]  expression )
                                |       func = ISBLANK '(' expression ')'     ->  ^(FUNC1[$func]  expression )
                                |       func = ISLITERAL '(' expression ')'       ->  ^(FUNC1[$func]  expression )
                                |       func = ISNUMERIC '(' expression ')'       ->  ^(FUNC1[$func]  expression )
                                |       regexExpression
                                |       existsFunc
                                |       notExistsFunc ;
    regexExpression             :       func = REGEX '(' exp1=expression ',' exp2=expression ( ',' exp3=expression )? ')' 
                                        ->^(FUNC3[func] $exp1 $exp2 $exp3?)
                                ;
    substringExpression         :       func = SUBSTR '(' exp1=expression ',' exp2=expression ( ',' exp3=expression )? ')' 
                                        ->^(FUNC3[func] $exp1 $exp2 $exp3?)
                                ;
    strReplaceExpression        :       func = REPLACE '(' exp1=expression ',' exp2=expression ',' exp3=expression ( ',' exp4=expression )? ')' 
                                        ->^(FUNC4[func] $exp1 $exp2 $exp3 $exp4?)
                                ;
    existsFunc                  :       EXISTS^ groupGraphPattern 
                                        
                                ;
    notExistsFunc               :       NOT EXISTS groupGraphPattern 
                                        ->^(NOTEXISTS  groupGraphPattern)
                                ;
    aggregate                   :       COUNT '(' dist=DISTINCT? ( star='*' |    expression ) ')'
                                        ->^(COUNT $dist? $star? expression?)
                                |       SUM '(' dist=DISTINCT? expression ')'
                                        ->^(SUM $dist? expression)
                                |       MIN '(' dist=DISTINCT? expression ')'
                                        ->^(MIN $dist? expression)
                                |       MAX '(' dist=DISTINCT? expression ')'
                                        ->^(MAX $dist? expression)
                                |       AVG '(' dist=DISTINCT? expression ')'
                                        ->^(AVG $dist? expression)
                                |       SAMPLE '(' dist=DISTINCT? expression ')'
                                        ->^(SAMPLE $dist? expression)
                                |       'GROUP_CONCAT' '(' DISTINCT? expression ( ';' SEPARATOR '=' string )? ')'
                                        ->^(NOTIMPLEMENTED NIL)
                                ;
                                
    iriRefOrFunction            :       iriRef argList? 
                                        ->^(VIRIREF_OR_FUNCTION_CALL iriRef argList?)
                                ;
    rdfLiteral                  :       string ( LANGTAG |    ( '^^' iriRef ) )? 
                                ;
    numericLiteral              :       numericLiteralUnsigned |    numericLiteralPositive | numericLiteralNegative ;
    numericLiteralUnsigned      :       INTEGER |    DECIMAL | DOUBLE ;
    numericLiteralPositive      :       INTEGER_POSITIVE |    DECIMAL_POSITIVE | DOUBLE_POSITIVE ;
    numericLiteralNegative      :       INTEGER_NEGATIVE |    DECIMAL_NEGATIVE | DOUBLE_NEGATIVE ;
    booleanLiteral              :       TRUE |    FALSE ;
    string                      :       STRING_LITERAL1 |    STRING_LITERAL2 | STRING_LITERAL_LONG1 | STRING_LITERAL_LONG2 ;
    iriRef                      :       IRI_REF |    prefixedName ;
    prefixedName                :       PNAME_LN |    PNAME_NS ;
    blankNode                   :       BLANK_NODE_LABEL |    ANON ;
    
 
    ABS                         : ('A'|'a')('B'|'b')('S'|'s') ;
    ADD                         : ('A'|'a')('D'|'d')('D'|'d') ;
    ALL                         : ('A'|'a')('L'|'l')('L'|'l') ;
    AS                         : ('A'|'a')('S'|'s') ;
    ASC                         : ('A'|'a')('S'|'s')('C'|'c') ;
    ASK                         : ('A'|'a')('S'|'s')('K'|'k') ;
    AVG                         : ('A'|'a')('V'|'v')('G'|'g') ;
    BASE                         : ('B'|'b')('A'|'a')('S'|'s')('E'|'e') ;
    BIND                         : ('B'|'b')('I'|'i')('N'|'n')('D'|'d') ;
    BINDINGS                         : ('B'|'b')('I'|'i')('N'|'n')('D'|'d')('I'|'i')('N'|'n')('G'|'g')('S'|'s') ;
    BNODE                         : ('B'|'b')('N'|'n')('O'|'o')('D'|'d')('E'|'e') ;
    BOUND                         : ('B'|'b')('O'|'o')('U'|'u')('N'|'n')('D'|'d') ;
    BY                         : ('B'|'b')('Y'|'y') ;
    CEIL                         : ('C'|'c')('E'|'e')('I'|'i')('L'|'l') ;
    CLEAR                         : ('C'|'c')('L'|'l')('E'|'e')('A'|'a')('R'|'r') ;
    COALESCE                         : ('C'|'c')('O'|'o')('A'|'a')('L'|'l')('E'|'e')('S'|'s')('C'|'c')('E'|'e') ;
    CONCAT                         : ('C'|'c')('O'|'o')('N'|'n')('C'|'c')('A'|'a')('T'|'t') ;
    CONSTRUCT                         : ('C'|'c')('O'|'o')('N'|'n')('S'|'s')('T'|'t')('R'|'r')('U'|'u')('C'|'c')('T'|'t') ;
    CONTAINS                         : ('C'|'c')('O'|'o')('N'|'n')('T'|'t')('A'|'a')('I'|'i')('N'|'n')('S'|'s') ;
    COPY                         : ('C'|'c')('O'|'o')('P'|'p')('Y'|'y') ;
    COUNT                         : ('C'|'c')('O'|'o')('U'|'u')('N'|'n')('T'|'t') ;
    CREATE                         : ('C'|'c')('R'|'r')('E'|'e')('A'|'a')('T'|'t')('E'|'e') ;
    DATA                         : ('D'|'d')('A'|'a')('T'|'t')('A'|'a') ;
    DATATYPE                         : ('D'|'d')('A'|'a')('T'|'t')('A'|'a')('T'|'t')('Y'|'y')('P'|'p')('E'|'e') ;
    DAY                         : ('D'|'d')('A'|'a')('Y'|'y') ;
    DEFAULT                         : ('D'|'d')('E'|'e')('F'|'f')('A'|'a')('U'|'u')('L'|'l')('T'|'t') ;
    DELETE                         : ('D'|'d')('E'|'e')('L'|'l')('E'|'e')('T'|'t')('E'|'e') ;
    DESC                         : ('D'|'d')('E'|'e')('S'|'s')('C'|'c') ;
    DESCRIBE                         : ('D'|'d')('E'|'e')('S'|'s')('C'|'c')('R'|'r')('I'|'i')('B'|'b')('E'|'e') ;
    DISTINCT                         : ('D'|'d')('I'|'i')('S'|'s')('T'|'t')('I'|'i')('N'|'n')('C'|'c')('T'|'t') ;
    DROP                         : ('D'|'d')('R'|'r')('O'|'o')('P'|'p') ;
    EXISTS                         : ('E'|'e')('X'|'x')('I'|'i')('S'|'s')('T'|'t')('S'|'s') ;
    FILTER                         : ('F'|'f')('I'|'i')('L'|'l')('T'|'t')('E'|'e')('R'|'r') ;
    FLOOR                         : ('F'|'f')('L'|'l')('O'|'o')('O'|'o')('R'|'r') ;
    FROM                         : ('F'|'f')('R'|'r')('O'|'o')('M'|'m') ;
    GRAPH                         : ('G'|'g')('R'|'r')('A'|'a')('P'|'p')('H'|'h') ;
    GROUP                         : ('G'|'g')('R'|'r')('O'|'o')('U'|'u')('P'|'p') ;
    HAVING                         : ('H'|'h')('A'|'a')('V'|'v')('I'|'i')('N'|'n')('G'|'g') ;
    HOURS                         : ('H'|'h')('O'|'o')('U'|'u')('R'|'r')('S'|'s') ;
    IF                         : ('I'|'i')('F'|'f') ;
    IN                         : ('I'|'i')('N'|'n') ;
    INSERT                         : ('I'|'i')('N'|'n')('S'|'s')('E'|'e')('R'|'r')('T'|'t') ;
    INTO                         : ('I'|'i')('N'|'n')('T'|'t')('O'|'o') ;
    IRI                         : ('I'|'i')('R'|'r')('I'|'i') ;
    LANG                         : ('L'|'l')('A'|'a')('N'|'n')('G'|'g') ;
    LANGMATCHES                         : ('L'|'l')('A'|'a')('N'|'n')('G'|'g')('M'|'m')('A'|'a')('T'|'t')('C'|'c')('H'|'h')('E'|'e')('S'|'s') ;
    LCASE                         : ('L'|'l')('C'|'c')('A'|'a')('S'|'s')('E'|'e') ;
    LIMIT                         : ('L'|'l')('I'|'i')('M'|'m')('I'|'i')('T'|'t') ;
    LOAD                         : ('L'|'l')('O'|'o')('A'|'a')('D'|'d') ;
    MAX                         : ('M'|'m')('A'|'a')('X'|'x') ;
    MD5                         : ('M'|'m')('D'|'d')('5') ;
    MIN                         : ('M'|'m')('I'|'i')('N'|'n') ;
    MINUS                         : ('M'|'m')('I'|'i')('N'|'n')('U'|'u')('S'|'s') ;
    MINUTES                         : ('M'|'m')('I'|'i')('N'|'n')('U'|'u')('T'|'t')('E'|'e')('S'|'s') ;
    MONTH                         : ('M'|'m')('O'|'o')('N'|'n')('T'|'t')('H'|'h') ;
    MOVE                         : ('M'|'m')('O'|'o')('V'|'v')('E'|'e') ;
    NAMED                         : ('N'|'n')('A'|'a')('M'|'m')('E'|'e')('D'|'d') ;
    NOT                         : ('N'|'n')('O'|'o')('T'|'t') ;
    NOW                         : ('N'|'n')('O'|'o')('W'|'w') ;
    OFFSET                         : ('O'|'o')('F'|'f')('F'|'f')('S'|'s')('E'|'e')('T'|'t') ;
    OPTIONAL                         : ('O'|'o')('P'|'p')('T'|'t')('I'|'i')('O'|'o')('N'|'n')('A'|'a')('L'|'l') ;
    ORDER                         : ('O'|'o')('R'|'r')('D'|'d')('E'|'e')('R'|'r') ;
    PREFIX                         : ('P'|'p')('R'|'r')('E'|'e')('F'|'f')('I'|'i')('X'|'x') ;
    RAND                         : ('R'|'r')('A'|'a')('N'|'n')('D'|'d') ;
    REDUCED 						: ('R'|'r')('E'|'e')('D'|'d')('U'|'u')('C'|'c')('E'|'e')('D'|'d') ;
    REGEX                         : ('R'|'r')('E'|'e')('G'|'g')('E'|'e')('X'|'x') ;
    REPLACE                         : ('R'|'r')('E'|'e')('P'|'p')('L'|'l')('A'|'a')('C'|'c')('E'|'e') ;
    ROUND                         : ('R'|'r')('O'|'o')('U'|'u')('N'|'n')('D'|'d') ;
    SAMPLE                         : ('S'|'s')('A'|'a')('M'|'m')('P'|'p')('L'|'l')('E'|'e') ;
    SECONDS                         : ('S'|'s')('E'|'e')('C'|'c')('O'|'o')('N'|'n')('D'|'d')('S'|'s') ;
    SELECT                         : ('S'|'s')('E'|'e')('L'|'l')('E'|'e')('C'|'c')('T'|'t') ;
    SEPARATOR                         : ('S'|'s')('E'|'e')('P'|'p')('A'|'a')('R'|'r')('A'|'a')('T'|'t')('O'|'o')('R'|'r') ;
    SERVICE                         : ('S'|'s')('E'|'e')('R'|'r')('V'|'v')('I'|'i')('C'|'c')('E'|'e') ;
    SHA1                         : ('S'|'s')('H'|'h')('A'|'a')('1') ;
    SHA256                         : ('S'|'s')('H'|'h')('A'|'a')('2')('5')('6') ;
    SHA384                         : ('S'|'s')('H'|'h')('A'|'a')('3')('8')('4') ;
    SHA512                         : ('S'|'s')('H'|'h')('A'|'a')('5')('1')('2') ;
    SILENT                         : ('S'|'s')('I'|'i')('L'|'l')('E'|'e')('N'|'n')('T'|'t') ;
    STR                         : ('S'|'s')('T'|'t')('R'|'r') ;
    STRAFTER                         : ('S'|'s')('T'|'t')('R'|'r')('A'|'a')('F'|'f')('T'|'t')('E'|'e')('R'|'r') ;
    STRBEFORE                         : ('S'|'s')('T'|'t')('R'|'r')('B'|'b')('E'|'e')('F'|'f')('O'|'o')('R'|'r')('E'|'e') ;
    STRDT                         : ('S'|'s')('T'|'t')('R'|'r')('D'|'d')('T'|'t') ;
    STRENDS                         : ('S'|'s')('T'|'t')('R'|'r')('E'|'e')('N'|'n')('D'|'d')('S'|'s') ;
    STRLANG                         : ('S'|'s')('T'|'t')('R'|'r')('L'|'l')('A'|'a')('N'|'n')('G'|'g') ;
    STRLEN                         : ('S'|'s')('T'|'t')('R'|'r')('L'|'l')('E'|'e')('N'|'n') ;
    STRSTARTS                         : ('S'|'s')('T'|'t')('R'|'r')('S'|'s')('T'|'t')('A'|'a')('R'|'r')('T'|'t')('S'|'s') ;
    SUBSTR                         : ('S'|'s')('U'|'u')('B'|'b')('S'|'s')('T'|'t')('R'|'r') ;
    SUM                         : ('S'|'s')('U'|'u')('M'|'m') ;
    TIMEZONE                         : ('T'|'t')('I'|'i')('M'|'m')('E'|'e')('Z'|'z')('O'|'o')('N'|'n')('E'|'e') ;
    TO                         : ('T'|'t')('O'|'o') ;
    TZ                         : ('T'|'t')('Z'|'z') ;
    UCASE                         : ('U'|'u')('C'|'c')('A'|'a')('S'|'s')('E'|'e') ;
    UNDEF                         : ('U'|'u')('N'|'n')('D'|'d')('E'|'e')('F'|'f') ;
    UNION                         : ('U'|'u')('N'|'n')('I'|'i')('O'|'o')('N'|'n') ;
    URI                         : ('U'|'u')('R'|'r')('I'|'i') ;
    USING                         : ('U'|'u')('S'|'s')('I'|'i')('N'|'n')('G'|'g') ;
    WHERE                         : ('W'|'w')('H'|'h')('E'|'e')('R'|'r')('E'|'e') ;
    WITH                         : ('W'|'w')('I'|'i')('T'|'t')('H'|'h') ;
    YEAR                         : ('Y'|'y')('E'|'e')('A'|'a')('R'|'r') ;
    FALSE                         : ('F'|'f')('A'|'a')('L'|'l')('S'|'s')('E'|'e') ;
    ISBLANK                         : ('I'|'i')('S'|'s')('B'|'b')('L'|'l')('A'|'a')('N'|'n')('K'|'k') ;
    ISIRI                         : ('I'|'i')('S'|'s')('I'|'i')('R'|'r')('I'|'i') ;
    ISLITERAL                         : ('I'|'i')('S'|'s')('L'|'l')('I'|'i')('T'|'t')('E'|'e')('R'|'r')('A'|'a')('L'|'l') ;
    ISNUMERIC                         : ('I'|'i')('S'|'s')('N'|'n')('U'|'u')('M'|'m')('E'|'e')('R'|'r')('I'|'i')('C'|'c') ;
    ISURI                         : ('I'|'i')('S'|'s')('U'|'u')('R'|'r')('I'|'i') ;
    SAMETERM                         : ('S'|'s')('A'|'a')('M'|'m')('E'|'e')('T'|'t')('E'|'e')('R'|'r')('M'|'m') ;
    TRUE                         : ('T'|'t')('R'|'r')('U'|'u')('E'|'e') ;
    /*IRI_REF                   :       '<' ([^<>"{}|    ^`\]-[\u0000-\u0020])* '>' ;  */
    IRI_REF                     :       '<' ~('<'|    '>'| '"'| '{'| '}'| '|'| '^'| '`'| '\\' | '\u0000'..'\u0020')* '>' ;
    PNAME_NS                    :       PN_PREFIX? ':' ;
    PNAME_LN                    :       PNAME_NS PN_LOCAL ;
    /* BLANK_NODE_LABEL         :       '_:' ( PN_CHARS_U |    [0-9] ) ((PN_CHARS|'.')* PN_CHARS)? */
    BLANK_NODE_LABEL            :       '_:' (PN_CHARS_U |    '0'..'9')((PN_CHARS|'.')*PN_CHARS)? ;
    VAR1                        :       '?' VARNAME ;
    VAR2                        :       '$' VARNAME ;
    /*LANGTAG                   :       '@' [a-zA-Z]+ ('-' [a-zA-Z0-9]+)* ;*/
    LANGTAG                     :       '@' ('a'..'z'|    'A'..'Z')+ ('-' ('a'..'z'|'A'..'Z'|'0'..'9')+)* ;
    /*INTEGER                   :       [0-9]+ ;*/
    INTEGER                     :       '0'..'9'+ ;
    /*DECIMAL                   :       [0-9]* '.' [0-9]+ ;*/
    DECIMAL            			:       '0'..'9'* '.' '0'..'9'+ ;
    /*DOUBLE                    :       [0-9]+ '.' [0-9]* EXPONENT |    '.' ([0-9])+ EXPONENT | ([0-9])+ EXPONENT ;*/
    DOUBLE                      :       '0'..'9'+ '.' '0'..'9'* EXPONENT |    '.' '0'..'9'+ EXPONENT | '0'..'9'+ EXPONENT ;
    INTEGER_POSITIVE            :       '+' INTEGER ;
    DECIMAL_POSITIVE            :       '+' DECIMAL ;
    DOUBLE_POSITIVE             :       '+' DOUBLE ;
    INTEGER_NEGATIVE            :       '-' INTEGER ;
    DECIMAL_NEGATIVE            :       '-' DECIMAL ;
    DOUBLE_NEGATIVE             :       '-' DOUBLE ;
    /*EXPONENT                  :       [eE] [+-]? [0-9]+ ; */
    fragment EXPONENT           :       ('e'|    'E') ('+'|'-')? '0'..'9'+ ;
    /*STRING_LITERAL1           :       "'" ( ([^\u0027\u005C\u000A\u000D]) |    ECHAR )* "'" ; */
    STRING_LITERAL1             :       '\'' ( ~('\u0027'|    '\u005C'|'\u000A'|'\u000D') | ECHAR )* '\'' ;
    /*STRING_LITERAL2           :       '"' ( ([^#x22#x5C#xA#xD]) |    ECHAR )* '"'*/
    STRING_LITERAL2             :       '"' (~('\u0022'|    '\u005C'|'\u000A'|'\u000D') | ECHAR )* '"' ;
    /*STRING_LITERAL_LONG1      :       "'''" ( ( "'" |    "''" )? ( [^'\] | ECHAR ) )* "'''" ;*/
    STRING_LITERAL_LONG1        :       '\'' '\'' '\'' ( ( '\'' |    '\'' '\'' )? ( ~('\''|'\\') | ECHAR ) )* '\'' '\'' '\'' ;
    /*STRING_LITERAL_LONG2      :       '"""' ( ( '"' |    '""' )? ( [^"\] | ECHAR ) )* '"""' ;*/
    STRING_LITERAL_LONG2        :       '"""' ( ( '"' |    '""' )? ( ~('"'|'\\') | ECHAR ) )* '"""' ;
    /*ECHAR                     :       '\' [tbnrf\"'] ;*/
    fragment ECHAR              :       '\\' ('t'|'b'|'n'|'r'|'f'|'\\'|'"'|'\'') ;
    NIL                         :       '(' ')' ;
    WS                          :       (' ' | '\t' |'\n')+ {$channel=HIDDEN;};
    ANON                        :       '['  ']' ;
    /*  PN_CHARS_BASE           :       [A-Z] |    [a-z] | [#x00C0-#x00D6] | [#x00D8-#x00F6] | [#x00F8-#x02FF] | [#x0370-#x037D] | [#x037F-#x1FFF] | [#x200C-#x200D] | [#x2070-#x218F] | [#x2C00-#x2FEF] | [#x3001-#xD7FF] | [#xF900-#xFDCF] | [#xFDF0-#xFFFD] | [#x10000-#xEFFFF]*/
    fragment PN_CHARS_BASE      :       'A'..'Z' |    'a'..'z' | '\u00C0'..'\u00D6' | '\u00D8'..'\u00F6' | '\u00F8'..'\u02FF' | '\u0370'..'\u037D' | '\u037F..\u1FFF' | '\u200C'..'\u200D' | '\u2070'..'\u218F' | '\u2C00'..'\u2FEF' | '\u3001'..'\uD7FF' | '\uF900'..'\uFDCF' | '\uFDF0'..'\uFFFD';
    fragment PN_CHARS_U         :       PN_CHARS_BASE |    '_' ;
    /*VARNAME                   :       ( PN_CHARS_U |    [0-9] ) ( PN_CHARS_U | [0-9] | \u00B7 | [\u0300-\u036F] | [\u203F-\u2040] )* ;*/
     VARNAME                    :       ( PN_CHARS_U |    '0'..'9' ) ( PN_CHARS_U | '0'..'9' | '\u00B7' | '\u0300'..'\u036F' | '\u203F'..'\u2040' )* ;
    /*PN_CHARS                  :       PN_CHARS_U |    '-' | [0-9] | \u00B7 | [\u0300-\u036F] | [\u203F-\u2040] ;*/
     PN_CHARS                   :       PN_CHARS_U |    '-' | '0'..'9' | '\u00B7' | '\u0300'..'\u036F' | '\u203F'..'\u2040' ;
     fragment PN_PREFIX         :       PN_CHARS_BASE ((PN_CHARS|    '.')* PN_CHARS)? ;
    /*PN_LOCAL                  :       (PN_CHARS_U |    [0-9] | PLX ) ( ( PN_CHARS | '.' | PLX )* ( PN_CHARS | PLX ) ) ? > ;*/
     fragment PN_LOCAL          :       (PN_CHARS_U |    '0'..'9' | PLX ) ( ( PN_CHARS | '.' | PLX )* ( PN_CHARS | PLX ) ) ? ;
    fragment PLX                :       PERCENT |    PN_LOCAL_ESC ;
    fragment PERCENT            :       '%' HEX HEX ;
    /*HEX                       :       [0-9] |    [A-F] | [a-f] ;*/
    fragment HEX                :       '0'..'9'|    'A'..'F'|'a'..'f';
    fragment PN_LOCAL_ESC       :       '\\' ( '_' |    '~' | '.' | '-' | '!' | '$' | '&' | '\'' | '(' | ')' | '*' | '+' | ',' | ';' | '=' | ':' | '/' | '?' | '#' | '@' | '%' ) ;

    
    


