/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.Accordion");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.layout.AccordionContainer");

dojo.require("jazz.ui.SimpleToolbar");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.ui", "AccordionMessages");

var messages = dojo.i18n.getLocalization("rmps.ui", "AccordionMessages");

/**
 * @author Christian Vogt (cvogt)
 */
dojo.declare("rmps.ui.Accordion", dijit.layout.AccordionContainer, {

	buttonWidget: "rmps.ui.AccordionButton",

	resize: function(size, resultSize) {
		if (size && size.w) {
			// don't set the width
			var newSize = dojo.clone(size);
			delete newSize.w;
			this.inherited(arguments, [newSize, resultSize]);
		} else {
			this.inherited(arguments);
		}
	},

	showChild: function(widget){
		this.selectChild(widget);
		dijit.focus(this.focusNode);
	},

	_setupChild: function(child) {
		dojo.addClass(child, "rmps-ui-accordion-content");
		this.inherited(arguments);
	}
});

dojo.declare("rmps.ui.AccordionButton", [dijit._Widget, dijit._Templated], {

	templateString: '<div class="rmps-ui-accordion-button" wairole="tab" waiState="expanded-false">' +
				'<div class="separator-top"></div>' +
				'<div class="title" dojoAttachPoint="focusNode" wairole="tab" waiState="expanded-false" dojoAttachEvent="ondijitclick: _onTitleClick, onkeypress: _onTitleKeyPress">' +
					'<div dojoAttachPoint="headerNode">' +
						'<img src="${_blankGif}" alt="" dojoAttachPoint="iconNode" style="vertical-align: middle" waiRole="presentation"/>' +
						'<span waiRole="presentation" dojoAttachPoint="titleTextNode"></span>' +
					'</div>' +
				'</div>' +
				'<div dojoAttachPoint="_sb" style="display: none;" class="separator-bottom"></div>' +
			'</div>',

	attributeMap: dojo.mixin(dojo.clone(dijit.layout.ContentPane.prototype.attributeMap), {
		label: {node: "titleTextNode", type: "innerHTML" },
		title: {node: "titleTextNode", type: "attribute", attribute: "title"},
		iconClass: { node: "iconNode", type: "class" }
	}),

	selected: false,

	postCreate: function() {
		//create toolbar widget
		this.toolbar = new jazz.ui.SimpleToolbar({hoverStyle: jazz.ui.simpleToolbar.hoverStyles.NONE});
		this.headerNode.parentNode.insertBefore(this.toolbar.domNode, this.headerNode);
		this._addExpandCollapseButtons();

		dojo.setSelectable(this.domNode, false);
		this.setSelected(this.selected);
		var titleTextNodeId = this.id.replace(' ', '_') + "_title";
		dojo.attr(this.titleTextNode, "id", titleTextNodeId);
		dijit.setWaiState(this.focusNode, "labelledby", titleTextNodeId);

		this.connect(this.contentWidget, "onShow", this._onContentShow);
		this.connect(this.contentWidget, "onHide", this._onContentHide);
	},

	destroy: function() {
		if (this.toolbar) {
			this.toolbar.destroyRecursive();
			delete this.toolbar;
		}
		this.inherited(arguments);
	},

	_addExpandCollapseButtons: function() {
		this._expandButton = this.toolbar.addButton(this, this._onTitleClick, {
			className: "expand-button",
			label: messages.expand,
			imgSrc: dojo.moduleUrl("rmps.ui", "graphics/accordion/expand.gif"),
			index: 0
		});
	},

	setLabel: function(value) {
		if (dojo.isString(value)) {
			this.attr("label", value);
		} else {
			this.titleTextNode.innerHTML = "";
			this.titleTextNode.appendChild(value);
		}
	},

	getTitleHeight: function() {
		return dojo.marginBox(this.domNode).h;
	},

	getParent: function() {
		// summary:
		//		Returns the parent.
		// tags:
		//		private
		return this.parent;
	},
	
	/* 
	 * Clicking on the expanded child, should collapse it, and expand the child down next to it. If the child is the very last one, then, we expand,
	 * the child node up next to it 
	 */
	_onTitleClick: function(e) {
		if (e) {
			dojo.stopEvent(e);
		}

		var parent = this.getParent();
		
		var children= parent.getChildren();
		
		/*If children length is zero, then, no need for this procedure */
		if(children.length<=1){	
			return;
		}
		
		var selectedChildWidget= parent.selectedChildWidget;
		
		if(!selectedChildWidget){
			parent.showChild(this.contentWidget);
		}
		
		var currentIndex= dojo.indexOf(children, selectedChildWidget);
		
		if(this.contentWidget===selectedChildWidget){
			if(currentIndex===children.length|| currentIndex+1=== children.length){
				parent.showChild(children[currentIndex-1]);	
			}else{				
				parent.showChild(children[currentIndex+1]);
			}
		}else{
			parent.showChild(this.contentWidget);
		}
			
	},

	_onTitleKeyPress: function(e) {
		// Allow keyboard traversal to pass though this button.
		var k = dojo.keys;
		if (e.keyCode != k.TAB) {
			dojo.stopEvent(e);
		}

		return this.getParent()._onKeyPress(e, this.contentWidget);
	},

	_setSelectedState: function(isSelected) {
		this.selected = isSelected;
		if (isSelected) {
			this.toolbar.disableButton(this._expandButton);
			dojo.removeClass(this.domNode, "collapsed");
			dojo.addClass(this.domNode, "expanded");
		} else {
			this.toolbar.enableButton(this._expandButton);
			dojo.removeClass(this.domNode, "expanded");
			dojo.addClass(this.domNode, "collapsed");
		}
		dijit.setWaiState(this.focusNode, "expanded", isSelected);
		dijit.setWaiState(this.focusNode, "selected", isSelected);
		this.focusNode.setAttribute("tabIndex", isSelected ? "0" : "-1");
	},

	setSelected: function(/*Boolean*/ isSelected) {
		this._setSelectedState(isSelected);
//		if (isSelected){
//			var cw = this.contentWidget;
//			if(cw.onSelected){ cw.onSelected(); }
//		}
	},

	_onContentShow: function() {
		this._sb.style.display = "";
	},

	_onContentHide: function() {
		this._sb.style.display = "none";
	}

});

})();