/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.ActionsBar");

(function() {

dojo.require("rmps.ui.SimpleImageDescription");
dojo.require("rmps.ui.SimpleButton");
dojo.require("jazz.ui.StyledBox");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");


var ActiveStyles = { // Defines the style used for the box highlighting active nodes
	ROUND: "round-corners",
	SQUARE: "square-corners"	
};

/* 
 *Used to provide actions bar. It could show read only actions or button actions. 
 */
dojo.declare("rmps.ui.ActionsBar", [dijit._Widget, dijit._Templated],{
	
	templateString: '<div dojoAttachPoint="mainContainer">'+
					'<div dojoType="jazz.ui.StyledBox"  trim="false" headerDefaultToHoverState="true" closable="false" showBackground="true" headerless="true">'+
					'	<div dojoAttachPoint="mainToolbar" dojoType="jazz.ui.SimpleToolbar"><div>' +
					'</div>'+
					'</div>',
	
	widgetsInTemplate: true,
	activeStyle: ActiveStyles.ROUND,
	trimStyle: jazz.ui.StyledBox.TrimStyles.TRIMLESS,
	
	"class": "rmps-actions-bar",

	postCreate: function(){
		this.inherited(arguments);
		this.startPath();
	},
		
	startPath: function(){
		this.clearAllActions();
		var descriptors= this.getDescriptors();
		if(!dojo.isArray(descriptors)|| descriptors.length===0){
			return;
		}
		if(descriptors.length===1){
			return this.addItem(descriptors.func, descriptors.isActive);
		}
		
		for(var index=0; index<descriptors.length-1; index++){
			var current= descriptors[index];
			if (current.noTransition) {
				current.func(current.isActive);
			} else {
				this.addItem(current.func, current.isActive);
			}
		}
		var last= descriptors[descriptors.length-1];
		last.func(last.isActive);
	},
	
	/* 
	 * Provide an array of descriptors. A descriptors consists of a creation
	 * function and isActive flag. 
	 * The function method provides the way to create the action entry, and
	 * isActive flag provides the way to determine if this item would be marked as active
	 * in UI
	 *  
	*/
	getDescriptors: function(){
		/*to be implemented by sub-classes */
	},
	
	clearAllActions: function(){
		while(this.mainToolbar.items.length>0){
			this.mainToolbar.removeItem(this.mainToolbar.items[0]);
		}
	},
	
	addItem: function(func, isActive){
		func(isActive);
		this.insertTransition();
	},
	
	/* 
	 *Insert Action Button.
	 * 		The entry is expected to provide label and onClick callback function
	 */
	insertActionEntry: function(entry, isActive){
		var button= new rmps.ui.SimpleButton({
			label: entry.label,
			onClick: entry.onClick
		});
		var item= this.mainToolbar.addWidget(isActive?this.makeActive(button): button);
		dojo.addClass(item.domNode.parentNode, "action");
		
		dojo.addClass(button.domNode, "actionButton");
		
		if(entry.disabled){
			button.domNode.disabled= entry.disabled;			
		}
		
		return item;
	},
	
	/* 
	 *Insert information entry that shows an icon and helping text.
	 * 		The entry is expected to provide label and imgSrc
	 */
	insertInformationEntry: function(entry, isActive){
		var simpleImageDescription= new rmps.ui.SimpleImageDescription({
			imgUrl: entry.imgSrc,
			label: entry.label 
		});
		
		var item;
		if(isActive){
			item = this.mainToolbar.addWidget(this.makeActive(simpleImageDescription));
		}else{
			item= this.mainToolbar.addWidget(simpleImageDescription);
			dojo.addClass(item.domNode.parentNode, "entry");
		}
		return item;
	},
	
	insertTextEntry: function(entry, isActive){
		var span= dojo.create("span");
		span.appendChild(dojo.doc.createTextNode(entry.label));
		var item= {domNode: span};
		var widget= this.mainToolbar.addWidget(isActive?this.makeActive(item):item);
		dojo.addClass(widget.domNode.parentNode, "textEntry");		
	},
	
	/*No expcted to be called externally, but it could be overidden for styling */
	insertTransition: function(){
		var simpleImageDescription= new rmps.ui.SimpleImageDescription({
			imgUrl: dojo.moduleUrl("rmps.ui", "graphics/actionsBar/state-arrow.gif")
		});
		
		var transition= this.mainToolbar.addWidget(simpleImageDescription);
		dojo.addClass(transition.domNode.parentNode, "transitionEntry");
		
		return transition;
	},
	
	insertDivider: function(){
		var divider= this.mainToolbar.addSeparator();
		dojo.addClass(divider.domNode.parentNode, "dividerEntry");
		return divider;
	},
	
	/*Make current node active */
	
	makeActive: function(widget, activeBoxClss, activeNodeClss){
		var node= widget.domNode?widget.domNode: widget;
		var box = widget.domNode?this.getActiveStyleBox(node):widget;
		dojo.removeClass(box);
		this._addActiveClass(box, activeBoxClss, activeNodeClss, this.activeStyle);
		return widget.domNode?{domNode: box}:box;
	},
	
	_addActiveClass: function(node, activeBoxClss, activeNodeClss, activeStyle){
		// Default is round
		var style = activeStyle?activeStyle:ActiveStyles.ROUND;
		
		dojo.addClass(node, activeBoxClss?activeBoxClss:"activeBox" + "-" + style);
		dojo.addClass(node, activeNodeClss?activeNodeClss:"activeNode" + "-" + style);		
	},
	
	/*Could be overridden if someone would like to change arguments */
	getActiveStyleBox: function(node){
		var divTag = document.createElement("div");
		divTag.appendChild(node);
		return divTag;
	},
	
	/*Could be overriden by callers to change active trim style */
	getActiveTrimStyle: function(){
		return this.trimStyle;
	}

});

rmps.ui.ActionsBar.ActiveStyles = ActiveStyles;

})();