/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2012, 2013. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("rmps.ui.ActionsTable");

dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.toolbar.Button");
dojo.require("rmps.ui.util.ToolbarFocusUtil");

var ToolbarFocusUtil = rmps.ui.util.ToolbarFocusUtil;
var LOCALE_FORMAT_OPTIONS = {selector: "date", fullYear: true};

/*
 * A table API provides extensiblity on Jazz Stlyed Table to help us to define row, action and cell creation descriptors 
*/
dojo.declare("rmps.ui.ActionsTable", jazz.ui.StyledTable, {
	actionDescriptors: null,
	messages: null,
	showHover: true,
	showHeaders: true,
	
	/*Used to get the required information for a given row */
	_rowCache: null,
	
	constructor: function(){
		this._rowCache = new rmps.ui.ActionsTable._RowsCache();
	},
	
	buildRendering: function(){
		this.inherited(arguments);
		dojo.addClass(this.domNode, "rmps-ui-actions-table");
		this.domNode.style.width= "100%";
		this.domNode.style.overflow= "hidden";
	},
	
	postCreate: function(){
		if(this.actionDescriptors&& this.actionDescriptors.length>0){
			var width= this.getActionsColumnWidth();
			var difference= width/this.columns.length;
			for(var index = 0, column; column = this.columns[index]; index++) {
				if (column.width) {
					column.width = parseInt(column.width.replace("%", "")) - difference + "%";
				} 
			}
			this.columns.push({	label: this.messages.actions,width: width+"%"});	
		}else{
			this.actionDescriptor = [];		
		}
		
		if(!this.showHeaders){
			this._headerRow.style.display= "none";
			this._globalHeader.style.display = "none";
			this._globalLineBottom.style.display = "none";
			this._globalLineTop.style.display = "none";
			this._headerLineTop.style.display = "none";
		}
		this.inherited(arguments);
	},
	
	postMixInProperties: function(){
		dojo.requireLocalization("rmps.ui", "ActionsTableMessages");
		this.messages = dojo.i18n.getLocalization("rmps.ui", "ActionsTableMessages");
		this.inherited(arguments);
	},
	
	fillTable: function(tableData){
		this.clearBody();
		var tbody= this.tbody;
		for(var index = 0, dataEntry; dataEntry = tableData[index]; index++) {
			this.addRow(dataEntry);
		}
	},
	
	createCells: function(tableRow, dataEntry){
		var length= this.columns.length;
		if(this.actionDescriptors&& this.actionDescriptors.length){
			length--;	//If actions column is defined, then make sure not to loop through it
		}
		
		var rowInformationDescriptor= {
			fieldsGetter: {},
			actionsGetter: {}
		};
		
		for(var index=0; index<length; index++){
			var descriptor= this.columns[index];
			var cell = dojo.create("td", null, tableRow);
			if (descriptor.actionDescriptors) {
				// add actions in the column if needed
				dojo.addClass(cell, "td-with-toolbar");
				var div = dojo.create("div", {style : {position: "relative"}});
				cell.appendChild(div);			
				descriptor.creator(div, tableRow, dataEntry, descriptor);				
				var toolbar = this.createActionsToolbar(div, descriptor.actionDescriptors, null, dataEntry, tableRow, descriptor);
				this.connect(cell, "onmouseenter",   dojo.hitch(this, this._onCellWithToolbarsShowHide, cell, true));
				this.connect(cell, "onmouseleave",   dojo.hitch(this, this._onCellWithToolbarsShowHide, cell, false));
				dojo.removeClass(toolbar.domNode, "jazz-ui-Toolbar");
				dojo.addClass(toolbar.domNode, "jazz-ui-Toolbar-cell");
			} else {
				descriptor.creator(cell, tableRow, dataEntry, descriptor);
			}
			var fieldName= descriptor.fieldName;
			if(!fieldName){
				fieldName= "cell"+index;
			}
			
			rowInformationDescriptor.fieldsGetter[fieldName]= dojo.hitch(this, function(cell){
				return cell;
			},cell);
		}
		
		if(!this.actionDescriptors){
			rowInformationDescriptor.dataEntry = dataEntry;
			this._rowCache.add(tableRow, rowInformationDescriptor);
			return;
		}
		// actions column
		var actionsCell = dojo.create("td", {style : {minWidth: "40px"}}, tableRow);		
		this.createActionsToolbar(actionsCell, this.actionDescriptors, rowInformationDescriptor, dataEntry, tableRow);
	},
	
	createActionsToolbar: function(actionsCell, actionDescriptors, rowInformationDescriptor, dataEntry, tableRow, columnDescriptor) {
		var simpleToolbar = new jazz.ui.Toolbar();
		
		simpleToolbar.connect(this, "uninitialize", function() {
			simpleToolbar.destroyRecursive();
		});
		
		if (rowInformationDescriptor) {
			rowInformationDescriptor.getActions = dojo.hitch(this, function(simpleToolbar){
				return simpleToolbar;
			}, simpleToolbar);
		}
		actionsCell.appendChild(simpleToolbar.domNode);
		
		if (actionDescriptors) {
			for (var index = 0, actionDescriptor; actionDescriptor = actionDescriptors[index]; index++) {
				var buttonArgs = actionDescriptor.args;
				buttonArgs.onClick = dojo.hitch(this, function(currentActionDescriptor, currentDataEntry, currentRow, currentActionsCell, currentColumnDescriptor){
					var currentData = this.getRowInfomation(currentRow).getDataEntry();
					currentActionDescriptor.actionCallback(currentData, currentRow, currentActionsCell, currentColumnDescriptor);
				}, actionDescriptor, dataEntry, tableRow, actionsCell, columnDescriptor);
						
				var button= new jazz.ui.toolbar.Button(buttonArgs);
								
				var actionId= actionDescriptor.id;
				if(!actionId){
					actionId= "action"+index;
				}
			
				if (rowInformationDescriptor) {
					rowInformationDescriptor.actionsGetter[actionId] = dojo.hitch(this, function(button){
						return button;
					}, button);
				}
				
				var onCreateCallback= actionDescriptor.onCreateCallback;
				if(onCreateCallback){
					onCreateCallback(actionDescriptor, button, tableRow, dataEntry);
					delete actionDescriptor.onCreateCallback;
				}
				
				simpleToolbar.addChild(button);
			}
		}
		if (rowInformationDescriptor) {
			rowInformationDescriptor.dataEntry = dataEntry;
			this._rowCache.add(tableRow, rowInformationDescriptor);
		}
		
		simpleToolbar.domNode.align= "right";
		simpleToolbar.startup();
		
        ToolbarFocusUtil.addToolbarFocusListeners( { 
            actionToolbar: simpleToolbar, 
            containerNode: tableRow, 
            actionClassName: ".jazz-ui-Toolbar"
        });
		
		return simpleToolbar;
	},
	
	_onCellWithToolbarsShowHide: function(cell, show) {
		if (show) {
			cell.childNodes[0].childNodes[1].style.visibility = "visible";
		} else {
			cell.childNodes[0].childNodes[1].style.visibility = "hidden";
		}
	},

	getActionDescriptorById: function(id){
		for (var index = 0, actionDescriptor; actionDescriptor = this.actionDescriptors[index]; index++) {
			if(actionDescriptor.id=== id){
				return actionDescriptor;
			}
		}
	},
	
	getRowInfomation: function(row){
		return this._rowCache.item(row);
	},
	
	getRowsInformation: function(){
		return this._rowCache;
	},
    
    /*It could be overridden to increase/reduce the action column width */
    getActionsColumnWidth: function(){
    	var number= this.actionDescriptors.length;
    	if(number=== 0){
    		return number;
    	}
    	return (number*3) + (Math.round(number/2));
    },
    
    /*If it is true, this means that the actions width will be increased to make sure that it will be in one line */
    enableDynamicActionsWidth: function(){
    	return true; /*by default */
    },
	
	addRow: function(dataEntry){
		var row = dojo.create("tr");
		this.tbody.appendChild(row);
		this.createCells(row, dataEntry);
		return row;
	},
	
	removeRow: function(row){
		this.tbody.deleteRow(dojo.indexOf(this.tbody.rows, row));	
		this._rowCache.remove(row);
	},
	
	updateRowCell: function(row, fieldId, dataEntry){
		var column= this._getCellColumnDataByFieldId(fieldId);
		if(!column){
			console.error("Invalid fieldId was used for updating row");
			return false;
		}
		var cell= row.cells[column.index];
		if(cell && column.creator){
			var toolbar;
			if (column.actionDescriptors) {
				// have to make sure toolbar node is preserved, pass in enclosing div node 
				// (parent of both cell contents and cell toolbar nodes)
				cell = cell.childNodes[0];
			} else {
				dojo.empty(cell);
			}
			column.creator(cell, row, dataEntry, column);
		}
		return true;
	},
	
	getRowIndex: function(row){
		return dojo.indexOf(this.tbody.rows, row);
	},
	
	getRowByIndex: function(index){
		return this.tbody.rows[index];
	},
	
	getRowsCount: function(){
		return this.tbody.rows.length;
	},
	
	getLastRow:function(){
		return this.getRowByIndex(this.tbody.rows.length-1);
	},
	
	moveRowDown: function(row){
		var rows= this.tbody.rows;
		var rowIndex= this.getRowIndex(row);
		if(rowIndex=== rows.length-1){
			//Can't move the last row down
			return false;
		}
		
		var nextRowIndex= rowIndex+1;
		var nextRow= rows[nextRowIndex];
		this.tbody.deleteRow(nextRowIndex);
		this.tbody.insertBefore(nextRow, row);
		return true; 
	},
	
	moveRowUp: function(row){
		var rows= this.tbody.rows;
		var rowIndex= this.getRowIndex(row);
		if(rowIndex<1){
			//Can't move the first row up
			return;
		}
		
		var previousRowIndex= rowIndex-1;
		var previousRow= rows[previousRowIndex];
		this.tbody.deleteRow(rowIndex);
		this.tbody.insertBefore(row, previousRow); 
	},
	
	showHeader: function(show) {
		if (show) {
			this._headerRow.style.display= "";
		} else {
			this._headerRow.style.display= "none";
		}
	},
	
	initiateInlineCellEdit: function(typeUri, currentRow, currentColumnDescriptor, onInlineEditingDone, tableContainer) {
		var value = this.getRowInfomation(currentRow).getDataEntry();
		if (currentColumnDescriptor && currentColumnDescriptor.actionDescriptors) {
			value = value[currentColumnDescriptor.fieldName];
		}
		var control;
		if (typeUri === "http://www.w3.org/2001/XMLSchema#boolean") {
			control = new dijit.form.CheckBox({});
			control.checked = (value && value !== "false");
		} else {
			control = typeUri === "http://www.w3.org/2001/XMLSchema#integer"? new dijit.form.NumberTextBox({}):
					  typeUri === "http://www.w3.org/2001/XMLSchema#dateTime"? new dijit.form.DateTextBox({}):
					  new dijit.form.ValidationTextBox({});
			control.set("value", value);
		}			  
		var cell;
		if (currentColumnDescriptor && currentColumnDescriptor.actionDescriptors) {
			// cell has toolbar
			var column = this._getCellColumnDataByFieldId(currentColumnDescriptor.fieldName);
			cell = currentRow.cells[column.index].childNodes[0];
		} else {
			var column = this._getCellColumnDataByFieldId("name");
			cell = currentRow.cells[column.index];
		}
		cell.replaceChild(control.domNode, cell.childNodes[0]);
		control.focus();
		dojo.connect(control, "onBlur", dojo.hitch(tableContainer, onInlineEditingDone, control, cell, currentRow, currentColumnDescriptor, typeUri));
	},
	
	inlineCellEditDone: function(typeUri, cell, control, currentRow, currentColumnDescriptor, allowBlankCells) {
		// editing done - called when the cell looses focus;
		// get the value from the control, replace the control with the text node with that value, and create a change
		var strValue;
		var value = null;
		var oldValue = this.getRowInfomation(currentRow).getDataEntry();
		if (currentColumnDescriptor && currentColumnDescriptor.actionDescriptors) {
			oldValue = oldValue[currentColumnDescriptor.fieldName];
		}
		if (typeUri === "http://www.w3.org/2001/XMLSchema#dateTime" && oldValue) {
			oldValue = new Date(oldValue);
		}
		if (typeUri === "http://www.w3.org/2001/XMLSchema#boolean") {
			value = control.checked;
			if (!value) {
				value = false;
				strValue = value.toString();
			}
		} else {
			value = control.getValue();
			if (!value && !allowBlankCells) {
				// don't allow blank cells, can cause problems
				if (typeUri === "http://www.w3.org/2001/XMLSchema#dateTime") {
					value = new Date();
				} else if (typeUri === "http://www.w3.org/2001/XMLSchema#integer") {
					value = 0;
					strValue = "0";
				} else {
					value = "<empty>";
				}
			}
			if (value instanceof Date){
				strValue = this.formatDateObject(value);
			}
		}		
		// replace control with text in cell
		cell.replaceChild(dojo.doc.createTextNode(strValue ? strValue : (value ? value.toString() : "")), cell.childNodes[0]);
		if (!currentColumnDescriptor || !currentColumnDescriptor.actionDescriptors) {
			this.getRowInfomation(currentRow)._dataEntry = value;
		}
		
		control.destroy();
		
		return {
			oldValue: oldValue,
			value: value
		};
	},
	
	formatDateObject: function(object) {
		return dojo.date.locale.format(object, LOCALE_FORMAT_OPTIONS);
	},	
	
	uninitialize: function(){
		this._rowCache.clear();
		this.inherited(arguments);
	},
	
	_getCellColumnDataByFieldId: function(fieldName){
		for(var index = 0, column; column = this.columns[index]; index++) {
			if(column.fieldName===fieldName){
				return dojo.mixin({index:index},column); 
			}
		}
		return null;
	}
});

dojo.declare("rmps.ui.ActionsTable._RowsCache", null, {
	_rowsInformation: null,
	
	constructor: function(){
		this._rowsInformation= [];
	},
	
	add: function(row, rowInformation){
		this._rowsInformation.push(new rmps.ui.ActionsTable._RowCache(row, rowInformation));
	},
	
	remove: function(row){
		for(var index = 0, rowInformation; rowInformation = this._rowsInformation[index]; index++) {
			if(rowInformation.getRow()=== row){
				return this._rowsInformation.splice(index, 1);
			}
		}
	},
	
	item: function(row){
		for(var index = 0, rowInformation; rowInformation = this._rowsInformation[index]; index++) {
			if(rowInformation.getRow()=== row){
				return rowInformation;
			}
		}
	},
	
	length: function(){
		return this._rowsInformation.length;
	},
	
	clear: function(){
		this._rowsInformation= [];
	}
});

dojo.declare("rmps.ui.ActionsTable._RowCache", null, {
	_row: null,
	_fieldsGetter: null,
	_actionsGetter: null,
	_dataEntry: null,
	
	constructor: function(row, rowInformation){
		this._row= row;
		if(!rowInformation){
			return;
		}
		this._actionsGetter= rowInformation.actionsGetter||{};
		this._fieldsGetter= rowInformation.fieldsGetter||{};
		this._dataEntry = rowInformation.dataEntry;
	},
	
	getRow: function(){
		return this._row;	
	},
	
	getActionWidget: function(action){
		var getter= this._actionsGetter[action];
		return getter?getter():null;
	},
	
	getFieldWidget: function(action){
		var getter= this._fieldsGetter[action];
		return getter?getter():null;
	},
	
	getDataEntry: function() {
		return this._dataEntry;
	}
});

})();
