/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.layout.graph.CalculateRadialInfo");

dojo.require("rmpw.diagrams.render.util.Point");

/**
 * Captures info about a particular radial circle used in the Radial layout
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.layout.graph.CalculateRadialInfo", null, {
	rootNode: null,
	firstCircleNodes: null,
	theta: null,

	constructor: function(rn, fcn, t) {
		this.rootNode = rn;
		this.firstCircleNodes = fcn;
		this.theta = t;
	},
	
	calculateNeededRadius: function(circleNodes, sectionAngle){
		var count, diagonal, i, maxDiagonal, neededDiameter, node, rad;
		neededDiameter = 0;
		count = circleNodes.size;
		maxDiagonal = 0;
		for (i = 0; i < count; ++i) {
			checkIndex(i, circleNodes.size);
			node = circleNodes.array[i];
			diagonal = this.getViewWorstExtent(node);
			neededDiameter += diagonal;
			if (diagonal > maxDiagonal)
				maxDiagonal = diagonal;
		}
		rad = neededDiameter / sectionAngle;
		return lowBits(fromDouble(Math.round(rad)));
	},
	
	getDelta: function(){
		if (this.rootNode) {
			return new rmpw.diagrams.render.util.Point(this.rootNode.x, this.rootNode.y);
		}
		return new rmpw.diagrams.render.util.Point(0, 0);
	},


	getRadius: function(){
		var radius, rootDiagonal;
		radius = this.calculateNeededRadius(this.firstCircleNodes, this.firstCircleNodes.size * this.theta);
		rootDiagonal = this.getViewWorstExtent(this.rootNode);
		if (2 * rootDiagonal > radius) {
			radius += rootDiagonal;
		}
		radius = 180 > radius?180:radius;
		return radius;
	},

	getViewWorstExtent: function(node){
		var ext;
		ext = new rmpw.diagrams.render.util.Dimension(node.width, node.height);
		return Math.sqrt(ext.width * ext.width + ext.height * ext.height) * 0.8;
	}
		
});
