/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.CollapsibleSection");

dojo.require("jazz.app.ua.SectionHoverHelp");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.fx");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.ui", "CollapsibleSectionMessages");

var messages = dojo.i18n.getLocalization("rmps.ui", "CollapsibleSectionMessages");

rmps.ui.CollapsibleSectionStyles = {
	GOLD: "gold",
	BLUE: "blue",
	GREY: "grey",
	GREY_ORANGE: "grey-orange",
	GREY_GREY: "grey-grey"
};

/**
 * @author Christian Vogt (cvogt)
 */
dojo.declare("rmps.ui.CollapsibleSection", [dijit._Widget, dijit._Templated], {

	templateString: '<div class="rmps-ui-collapsible-section">' +
				'<div class="outline">' +
					'<div tabIndex="0" dojoAttachEvent="onkeyup: _onKeyUp" class="rmps-ui-collapsible-section-header" dojoAttachPoint="headerNode">' +
						'<span dojoAttachPoint="primaryTitleNode" class="title"></span> ' +
						'<span dojoAttachPoint="secondaryTitleNode" class="secondary-title"></span>' +
					'</div>' +
					'<div dojoAttachPoint="containerNode" style="position: relative; zoom: 1;"></div>' +
				'</div>' +	
			'</div>',

	// mixin properties
	trim: rmps.ui.CollapsibleSectionStyles.GREY_GREY,
	primaryTitle: "", // String | DOM Node
	secondaryTitle: "", // String | DOM Node
	collapsible: true,
	isCollapsed: false,
	boxed: false,
	uaKey: "",

	// private properties
	_animation: null,

	postCreate: function() {
		if (this.uaKey) {
			this._help = new jazz.app.ua.SectionHoverHelp({uaKey: this.uaKey, title: messages.help});
			dojo.addClass(this._help.domNode, "action help");
			this._help.domNode.style.margin = "";
			this.connect(this._help.domNode, "onclick", dojo.stopEvent);
			this.headerNode.insertBefore(this._help.domNode, this.headerNode.firstChild);
		}
		if (this.boxed) {
			dojo.addClass(this.domNode, "boxed");
		}

		this.setPrimaryTitle(this.primaryTitle);
		this.setSecondaryTitle(this.secondaryTitle);

		if (this.collapsible) {
			dojo.addClass(this.domNode, "collapsible");
			this.connect(this.headerNode, "onclick", function() {
				if (!this._animation) {
					this.toggle();
				}
			});

			if (this.isCollapsed) {
				this.containerNode.style.display = "none";
				this.headerNode.title = messages.expandSection;
			} else {
				this.headerNode.title = messages.collapseSection;
			}
		} else {
			dojo.addClass(this.domNode, "nonCollapsible");
		}

		dojo.addClass(this.domNode, this.trim);
	},

	startup: function() {
		this.inherited(arguments);
		if (this._help) {
			this._help.startup();
		}
	},

	destroy: function() {
		if (this._help) {
			this._help.destroyRecursive();
		}
		if (this._animation) {
			this._animation.stop();
			delete this._animation;
		}
		this.inherited(arguments);
	},

	setPrimaryTitle: function(/*String | DOM Node*/title) {
		this.primaryTitle = title;
		this.primaryTitleNode.innerHTML = "";

		if (dojo.isString(title)) {
			this.primaryTitleNode.appendChild(dojo.doc.createTextNode(title));
		} else {
			this.primaryTitleNode.appendChild(title);
		}
	},

	setSecondaryTitle: function(title) {
		this.secondaryTitle = title;
		this.secondaryTitleNode.innerHTML = "";

		if (title) {
			if (dojo.isString(title)) {
				this.secondaryTitleNode.appendChild(dojo.doc.createTextNode(title));
			} else {
				this.secondaryTitleNode.appendChild(title);
			}
		}
	},

	addAction: function(name, tooltip, onClick) {
		var action = dojo.doc.createElement("a");
		action.className = "action";
		if (tooltip) {
			action.title = tooltip;
		}
		action.appendChild(dojo.doc.createTextNode(name));

		if (dojo.isString(onClick)) {
			action.href = onClick;
		} else {
			action.href = "#";
			this.connect(action, "onclick", onClick);
			this.connect(action, "onclick", dojo.stopEvent);
		}
		this.addHeaderNode(action);
		return action;
	},
	
	addImageAction: function(src, tooltip, onClick) {
		var button = dojo.create("a", {
			className: "action",
			href: "#",
			title: tooltip
		});
		
		dijit.setWaiRole(button,"button");
		var img = dojo.create("img", {src: src, alt: tooltip }, button);
		button.onclick= function(e){
			dojo.stopEvent(e);
			onClick();
		};
		this.addHeaderNode(button);
		return button;
	},

	addHeaderNode: function(node) {
		this.headerNode.insertBefore(node, this._help ? this._help.domNode : this.primaryTitleNode);
	},

	toggle: function() {
		if (this._animation) {
			this._animation.stop();
			delete this._animation;
		}

		if (this.isCollapsed) {
			this._animation = dojo.fx.wipeIn({
				node: this.containerNode,
				onEnd: dojo.hitch(this, function() {
					this.headerNode.title = messages.collapseSection;
					this.isCollapsed = false;
					delete this._animation;
					dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
				})
			});
		} else {
			this._animation = dojo.fx.wipeOut({
				node: this.containerNode,
				onEnd: dojo.hitch(this, function() {
					this.headerNode.title = messages.expandSection;
					this.isCollapsed = true;
					delete this._animation;
					dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
				})
			});
		}
		if (this._animation) {
			this._animation.play();
		}
	},

	collapse: function() {
		if (this.isCollapsed) {
			return;
		}
		this.toggle();
	},

	expand: function() {
		if (!this.isCollapsed) {
			return;
		}
		this.toggle();
	},
	
	_onKeyUp: function(e){
		if(e&& (dojo.keys.ENTER === e.keyCode|| dojo.keys.SPACE === e.keyCode)){
			this.toggle();			
		}
	}
});

})();