/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.CommandController");

dojo.require("rmpw.diagrams.edit.commands.MoveCommand");
dojo.require("rmpw.diagrams.edit.commands.ResizeCommand");
dojo.require("rmpw.diagrams.edit.commands.SetConnectionBendpointsCommand");
dojo.require("rmpw.diagrams.edit.commands.ChangeRouterCommand");
dojo.require("rmpw.diagrams.edit.commands.ChangeColorCommand");
dojo.require("rmpw.diagrams.edit.commands.ChangeLineStyleCommand");
dojo.require("rmpw.diagrams.edit.commands.ChangeLinePropertiesCommand");
dojo.require("rmpw.diagrams.edit.commands.ChangeFontStyleCommand");

dojo.require("rmpw.diagrams.edit.tools.AcceleratorScope");

/**
 * Central authority for invoking commands.  Maintains a map of command id's to commands, and provides
 * methods for invoking commands from various contexts in the editor (context menu, palette drag and drop)
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.diagrams.edit.CommandController", null, {
	
	constructor: function(viewer, attachPoint, commandStack) {
		this.viewer = viewer;
		this.commandStack = commandStack;
		
		this.commands = {};
		this.registerCommand(new rmpw.diagrams.edit.commands.MoveCommand());
		this.registerCommand(new rmpw.diagrams.edit.commands.ResizeCommand());
		this.registerCommand(new rmpw.diagrams.edit.commands.SetConnectionBendpointsCommand());
		this.registerCommand(new rmpw.diagrams.edit.commands.ChangeRouterCommand());
		this.registerCommand(new rmpw.diagrams.edit.commands.ChangeColorCommand());
		this.registerCommand(new rmpw.diagrams.edit.commands.ChangeLineStyleCommand());
		this.registerCommand(new rmpw.diagrams.edit.commands.ChangeLinePropertiesCommand());
		this.registerCommand(new rmpw.diagrams.edit.commands.ChangeFontStyleCommand());
	},
	
	registerCommand: function(command) {
		if(typeof command.id !== 'string') {
			throw "Cannot register command without id: " + command.declaredClass;
		} else {
			var existingCommand = this.commands[command.id];
			if(existingCommand) {
				if(command.declaredClass === existingCommand.declaredClass) {
					throw "Cannot register same command twice: " + command.declaredClass;
				} else {
					throw "Cannot register command with duplicate id: " + command.declaredClass + "'s id already registered to " + existingCommand.declaredClass;
				}
			}
		}
		
		this.commands[command.id] = command;
	},
	
	unregisterCommand: function(id) {
		return delete this.commands[id];
	},
	
	/*
	 * accessor determining if a command is currently being executed by the command controller
	 */
	isExecuting: function() {
		return this.commandStack.isExecuting();
	},
	
	/*
	 * accessor determining if undo / redo is currently executing
	 */
	isUndoRedoExecuting: function() {
		return this.commandStack.isUndoRedoExecuting();
	},
	
	execute: function(id, args) {
		args = args || {};
		args.viewer = args.viewer || this.viewer;
		args.editController = args.editController || this.viewer.getSelectionController().getPrimarySelection();
		return this.commandStack.execute(this.commands[id], args);	
	},
	
	executeUUIDlist: function(id, args, elements) {
		var diagramMapping = this.viewer.diagram;
		var diagram = this.viewer.diagram; 
		var selection = dojo.map(elements, function(id) {
			return diagram.getChildFromId(id);
		});
		
		args = args || {};
		args.viewer = this.viewer;
		args.selection = selection;
		return this.commandStack.execute(this.commands[id], args);
	}
});