/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("rmps.ui.DetailedProgress");

dojo.require("jazz.ui.StyledTable");
dojo.require("rmps.ui.DescriptionAreaButton");
dojo.require("rmps.ui.InfoArea");
dojo.require("rmps.ui.DescriptionArea");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("rmps.ui.util.UIUtil");

var DescriptionAreaButton= rmps.ui.DescriptionAreaButton;
var InfoArea= rmps.ui.InfoArea;
var DescriptionArea= rmps.ui.DescriptionArea;
var UIUtil= rmps.ui.util.UIUtil;

dojo.requireLocalization("rmps.ui", "DetailedProgressMessages");

/* 
 * Requiered:
 
 * Optional:
 * 		progressTitle: The title for the progress containing styled box
 *		progressIntroTitle: The intro title for the progress containing styled box (used to show numbering if required)
 * 		progressInstructions: The description for the progress containing styled box
 * 
 * 
 * 
 * Description: Provide a way to add and describe a set of steps in one or more of categoreis (styled boxes), and show the progress of the steps being run
 * 				in another styled box (progressArea) to give us a track on what the progress is. and if there is something happened with a step, we could
 * 				have warning/error/success.
 * 
 * Usage:
 * 				Create an instance for DetailedProgress
 * 					this.progress= new DetailedProgress({
 *							progressTitle: "View Progress"
 *					});
 * 					
 *				Then, add your categories (it could be one or more):
 * 				this.progress.addCategoryDescriptor("unlockTheDoor", {
 *					buttonTitle: "Unlock",
 *					instructions: "Unlock The Door",
 *					title: "Unlock"
 *		    	});
 *				this.progress.addCategoryDescriptor("openTheDoor", {
 *					buttonTitle: "Open",
 *					instructions: "Open The Door",
 *					title: "Open"
 *		    	});   	
 *    	
 *		    	Then, you will need to add your progress detailed messages. Each step had an associated callback for it, and you could make sure to notify
 *		    	each step entry if a warning/success/error happened. Each cateogry button will be disabled except for the first one. Once you click on the frist
 *		    	category button, it will be diabled, and after the end of the cateory steps operations. 
 *		    	Here is example:
 * 				    	this.progress.addStep("unlockTheDoor", {
 *								header: "Please Unlock the door",
 *								description: "Unlock The door",
 *								performCallback: dojo.hitch(this, this.unlockTheDoor)
 *						});
 *		    	
 *				    	this.progress.addStep("openTheDoor", {
 *							header: "Please Unlock the door",
 *							description: "Open The door",
 *							performCallback: dojo.hitch(this, this.openTheDoor)
 *						});
 *		    	In your function you could do whatever oeprations you want and according to how the oeprations or the work gone, you could notify error/warning/success
 * 				Exzample:
 * 					unlockTheDoor: function(infoArea, detailedProgress){
 *							detailedProgress.notifySuccess(infoArea, "Yes Yes, I found the key, let's start next to open the door");
 *					}, 
 *		    	 
 * 				Please, note that you have to call to #render() in order to start rendering your detailedProgress
 * 				You could provide a stepId for your step on your args, but it must be unique. Ensuring that step id is unique is the clients responsbility
 *  
 */
dojo.declare("rmps.ui.DetailedProgress", [dijit._Widget, dijit._Templated], {
	
	templateString: '<div>' +
						'<div dojoAttachPoint="categoriesNode"></div>' +
						'<div dojoAttachPoint="progressNode"></div>' +
					'</div>',

    categories: null,
    steps: null,
    progressTitle: null,
    progressIntroTitle: null,
    progressInstructions: null,
    progressArea: null,
    
    "class": "rmps-detailed-progress",
    
    /*Tracker for the step being called */
    stepIndex: 0,
    
    numberOfStepsExecuted:0,
    numberOfSuccess:0,
    numberOfErrors:0,
    numberOfWarnings:0,
    numberOfSteps: 0,
	
	/*A public field to show if the current DetailedProgress instance is completely done (for all categories */
	isCompleted: false,
	
	/*If true, this means that each step will be prefixed by "Step num.". For example: Step 1. My Step  */
	numberSteps: true,
	
	messages: dojo.i18n.getLocalization("rmps.ui", "DetailedProgressMessages"),
	    
    constructor: function(params){
        this.categories = [];
        this.steps = [];
        
        if(!params){
        	params= {};
        }
        
		this.progressTitle= UIUtil.associateParamValue(params.progressTitle);
		this.progressIntroTitle= UIUtil.associateParamValue(params.progressIntroTitle);
		this.progressInstructions= UIUtil.associateParamValue(params.progressInstructions);        	
    },
    
    postCreate: function() {
		this.inherited(arguments);
		
		this.progressArea= new DescriptionArea({
							instructions: this.progressInstructions,
							introTitle: this.progressIntroTitle,
							title: this.progressTitle
		});
		this.progressNode.appendChild(this.progressArea.domNode);
	},
    
    
    
    /*You have to provide a unique key for your cateogory descritpor */
    addCategoryDescriptor: function(key, args, predefinedCallback){
    	var descriptor = new rmps.ui.DescriptionAreaButton({
    		title: args.title,
    		statusMessage: args.statusMessage,
    		buttonTitle: args.buttonTitle,
			instructions: args.instructions,
			introTitle: args.introTitle
    	});
    	descriptor.onExecute= dojo.hitch(this, function() {
    		/* If the callback function is deferred then we have to make sure to add a callback to out peromExecute to maintain
    		 * the steps execution orders*/
    		if(predefinedCallback instanceof dojo.Deferred){
    			predefinedCallback.addCallback(this, dojo.hitch(this,this.performExecute),key);
    		}else if(dojo.isFunction(predefinedCallback)){
    			var dfd= predefinedCallback();
    			if(dfd instanceof dojo.Deferred){
					dfd.addCallback(this, dojo.hitch(this,this.performExecute),key);    				
    			}else{
    				this.performExecute(key);
    			}
    		}else{
    			/*predefinedCallback is optional */
				if(predefinedCallback){
					/*We do not hitch this callback, as we have to maintain the caller context */
					predefinedCallback();    			
    			}
				this.performExecute(key);    			
    		}
		});
    	descriptor.setEnablement(false);
    	this.categories[key]= descriptor;
    	this.categories[key].steps= [];
    	return descriptor;
    },
    
    addStep: function(key, args){
    	var infoArea = new InfoArea({
    		header: args.header,
    		areaId: args.stepId,
			description: args.description
    	});
    	infoArea.performCallback= this._getOperationCallback(args.performCallback, this.defaultStepCallback);
    	infoArea.successCallback= this._getOperationCallback(args.successCallback, this.defaultSuccessCallback);
    	infoArea.warningCallback= this._getOperationCallback(args.warningCallback, this.defaultWarningCallback);
    	infoArea.errorCallback= this._getOperationCallback(args.errorCallback, this.defaultErrorCallback); 
    	infoArea.key= key;
    	var currenyCategory= this.categories[key];
    	
    	var index= parseInt(args.index);
    	if(index> -1){
    		currenyCategory.steps.splice(index,0,infoArea);
    	}else{
    		currenyCategory.steps.push(infoArea);
    	}
    	
    	this.numberOfSteps++;
    },
    
    getStep: function(key, stepId){
    	var currenyCategory= this.categories[key];
    	if(!currenyCategory){
    		return null;
    	}
    	var steps= currenyCategory.steps;
    	if(!steps){
    		return null;
    	}
    	
    	for(var index=0; index<steps.length; index++){
    		var current= steps[index];
    		if(current.areaId === stepId){
    			return current; 
    		}
    	}
    },
    
    
    notifySuccess: function(infoArea, message){
    	this.stepIndex++;
    	this.numberOfSuccess++;
    	infoArea.successCallback(infoArea, message);
    	infoArea.unsetInProgress();
    	this.performExecute(infoArea.key);
    },
    
    notifyError: function(infoArea, message){
    	this.stepIndex++;
    	this.numberOfErrors++;
    	infoArea.errorCallback(infoArea, message);
    	infoArea.unsetInProgress();
    	this.performExecute(infoArea.key);
    },
    
    /*A way to notify errors, and enable the users to retry the action by re-enabling the execution button */
    notifyErrorAndSupress: function(infoArea, message, reEnable, removeHighlight){
    	infoArea.errorCallback(infoArea, message);
    	if(removeHighlight){
			infoArea.unsetInProgress();    		
    	}
    	var category= this.categories[infoArea.key];
    	category.setEnablement(reEnable);
    },
    
    notifyWarning: function(infoArea, message){
    	this.stepIndex++;
    	this.numberOfWarnings++;
    	infoArea.warningCallback(infoArea, message);
    	infoArea.unsetInProgress();
    	this.performExecute(infoArea.key);
    },
    
    
    performExecute: function(key){
    	this.numberOfStepsExecuted++;
    	var category= this.categories[key];
    	category.showStatusMessage();
    	var steps= category.steps;
    	var step= steps[this.stepIndex];
    	if(step){
    		step.setInProgress();
    		step.displayInfo(step.INFO_TYPES.LOADING, "");
    		step.performCallback(step, this);	
    	}else{ /*means that we reached the last step. so we notify the stauts, and do not make another callback. so we need to disable the info button */
    		/*Enable next button. If found is null this means that we are at the last  category of steps */
    		var categoryIndex=0;
    		var found= false;
    		for(var categoryKey in this.categories){
				if(found){
					found= this.categories[categoryKey];
					break;
				}
				if(categoryKey===key){
					found= true;
				}
   			}
   			if(found&& found.setEnablement){
   				found.setEnablement(true);	
   				this.stepIndex=0;	/*Start from the first index for the next cateogry */
   				category.isComplete= true;
   			}else{
   				this.isCompleted= true;	/*This means that all categoreis have been executed */
   				category.hideStatusMessage();
   				this.whenFinalized();
   			}
    		
    	}
		
    },
    
    isCategoryFinalized: function(key){
    	var category= this.categories[key];
    	return category&& category.isComplete;
    },
    
    isFinalized: function(){
    	return this.isCompleted;
    },
    
    whenFinalized: function(){
    	/*This method could be extended by callers in order to do a spceific action when all processes are executed*/
    },
    
    defaultStepCallback: function(infoArea){
    	this.notifySuccess(infoArea);
    },
    
    defaultSuccessCallback: function(infoArea, message){
    	infoArea.displayInfo(infoArea.INFO_TYPES.OK,message);
    },
    
    defaultErrorCallback: function(infoArea, message){
    	infoArea.displayInfo(infoArea.INFO_TYPES.ERROR,message);
    },
    
    defaultWarningCallback: function(infoArea, message){
    	infoArea.displayInfo(infoArea.INFO_TYPES.WARNING,message);
    },
    
    _getOperationCallback: function(callback, defaultCallback){
    	var operationCallback= callback;	/*No need to change the context as the callers expect the methods to be in their contexts */
    	if(!operationCallback){
    		operationCallback= dojo.hitch(this, defaultCallback);	/*Here we need our context, as we call for our default methods */
    	}
    	return operationCallback;
    },
    
    /*Must be called by the client caller in order to start showing the detailed progress instance */
    render: function(){

		var appendStep= dojo.hitch(this, function(step){
			this.progressArea.contents.appendChild(step.domNode);
			/*Add a space between each two progress description entry */
			var div = dojo.doc.createElement("div");
			div.innerHTML = "<br>";
			this.progressArea.contents.appendChild(div);
		});
		
		var prepareStep= this.numberSteps? dojo.hitch(this, function(step, stepIndex){
			var stepNumber= stepIndex+1;
			step.headerNode.innerHTML= this.messages.step+ " "+stepNumber+ ". "+ step.headerNode.innerHTML;  
			appendStep(step);
		}):
		dojo.hitch(this, function(step){
			appendStep(step);
		});

    	var first= true;
    	for(var key in this.categories){
    		var category= this.categories[key];
    		if(first){
    			category.setEnablement(true);	/*enable first cateogry only */
    			first= false;
    		}
    		this.categoriesNode.appendChild(category.domNode);
    		var steps= category.steps;
    		    		    		
    		for(var stepIndex=0; stepIndex<steps.length; stepIndex++){
    			prepareStep(steps[stepIndex], stepIndex);
    		}
    	}
    },
    
    destroy: function(){
    	for(var key in this.categories){
    		var category= this.categories[key];
    		if(category&& category.destroyRecursive){
    			category.destroyRecursive();
    			delete category.destroyRecursive;
    		}
    	}
    	this.inherited(arguments);
    }
});



})();
