/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.panner.DiagramOutline");

dojo.require("rmpw.panner.Outline");

dojo.require("rmpw.panner.LayerConstants");

dojo.require("dojox.gfx");
dojo.require("dojox.gfx.matrix");

var LayerConstants = rmpw.panner.LayerConstants;

dojo.declare("rmpw.panner.DiagramOutline", rmpw.panner.Outline, {
	
	diagram: null,
	
	// private properties
	_surface: null,
	_scalingGroup: null,
	_translateGroup: null,
	
	_layer: null,
	
	postCreate: function() {
		this.inherited(arguments);
		this._surface = dojox.gfx.createSurface(this.panContainerNode, 1, 1);
		
		this._translateGroup = this._surface.createGroup();
		this._scalingGroup = this._translateGroup.createGroup();
	},
	
	startup: function() {
		this.inherited(arguments);
		
		var root = this.diagram.getRootEditPart();
		this._layer = root.getLayer(LayerConstants.SCALED_LAYERS);
		var um = this._layer.getUpdateManager();
		this.connect(um, "fireUpdated", this._update);
		this._update([this._layer]);
		this._surface.setDimensions(this._overview.offsetWidth, this._overview.offsetHeight);
	},
	
	_update: function(figures) {
		var found = false;
		var l = this._layer;
		for (var i = 0, len = figures.length; !found && i < len; ++i) {
			var f = figures[i];
			do {
				if (f == l) {
					found = true;
				}
			} while (!found && (f = f.getParent()));
		}
		
		if (!found) {
			return;
		}
		
		var rawNode = this._scalingGroup.rawNode;
		while (rawNode.firstChild) {
			rawNode.removeChild(rawNode.firstChild);
		}
		
		if (l.getGroup()) {
			var size = l.getSize();
			var ratio = this.getRatio();
			var scale = this.getScale();
			
			var clone = l.getGroup().rawNode.cloneNode(true);
			
			rawNode.appendChild(clone);
		}
	},
	
   	ratioChanged: function(ratio) {
   		if (this._scalingGroup) {
			this._scalingGroup.setTransform(dojox.gfx.matrix.scale(ratio, ratio));
   		}
	},
	
	movePanContainerNode: function(x, y) {
		this._translateGroup.setTransform(dojox.gfx.matrix.translate(x, y));
	}
});

})();