/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.Document");

dojo.require("dojox.xml.DomParser");
dojo.require("jazz.client.xhr");

rmps.core.Namespaces = {
	RDF: "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
	OSLC_AM_1_0: "http://open-services.net/xmlns/am/1.0/",
	OSLC_RM_1_0: "http://open-services.net/xmlns/rm/1.0/",
	RMPS_1_0: "http://jazz.net/xmlns/rmps/1.0/",
	JP_0_6: "http://jazz.net/xmlns/prod/jazz/process/0.6/",
	JP_1_0: "http://jazz.net/xmlns/prod/jazz/process/1.0/",
	JD_1_0: "http://jazz.net/xmlns/prod/jazz/discovery/1.0/",
	JFS_1_0: "http://jazz.net/xmlns/prod/jazz/jfs/1.0/",
	DC_TERMS: "http://purl.org/dc/terms/",
	JU: "http://jazz.net/ns/ui#",
	JUENTRY: "http://jazz.net/ns/ui/entry#",
	FP: "http://jazz.net/xmlns/prod/jazz/frontingapp/process/1.0/",
	JDB:"http://jazz.net/xmlns/prod/jazz/dashboard/1.0/"
};

// mixin the latest version 
dojo.mixin(rmps.core.Namespaces, {
	OSLC_AM: rmps.core.Namespaces.OSLC_AM_1_0,
	OSLC_RM: rmps.core.Namespaces.OSLC_RM_1_0,
	RMPS: rmps.core.Namespaces.RMPS_1_0,
	JP: rmps.core.Namespaces.JP_1_0,
	JD: rmps.core.Namespaces.JD_1_0,
	JFS: rmps.core.Namespaces.JFS_1_0,
	JDB: rmps.core.Namespaces.JDB
});

dojo.declare("rmps.core.Document", null, {
	
	constructor: function(doc) {
		this._doc = doc;
		this._rdfPrefix = this._getPrefix("http://www.w3.org/1999/02/22-rdf-syntax-ns#");
	},
	
	_getPrefix: function(namespace) {
		for (var key in this._doc.namespaces) {
			var n = this._doc.namespaces[key];
			if (n === namespace) {
				return key;
			}
		}
		return null;
	},
	
	getUriForPath: function(pathDetails){
		return this._getUriForPath(this._doc, pathDetails.reverse());
	},
	
	_getUriForPath: function(node, pathDetails){
		var current= pathDetails.pop();
		if(pathDetails.length==0){
			return this.getUriForNode(node, current.namespace, current.name);
		}
		return this._getUriForPath(this.getNode(node, current.namespace, current.name), pathDetails);
	},
	
	getUriForNode: function(node, namespace, name) {
		var element= this.getNode(node,namespace, name);	
		return element.getAttribute(this._rdfPrefix + ":resource");
	},
	
	getUri: function(namespace, name) {
		var element= this.getNode(this._doc,namespace, name);	
		return element.getAttribute(this._rdfPrefix + ":resource");
	},
	
	getNode: function(node, namespace, name){
		var prefix = this._getPrefix(namespace);
		if (!prefix) {
			throw new Error(this.getError(namespace,name));
		}
		
		var element = node.getElementsByTagName(prefix + ":" + name)[0];
		if (!element) {
			throw new Error(this.getError(namespace,name));
		}
		return element;
	},
	
	getValue: function(namespace, name) {
		var prefix = this._getPrefix(namespace);
		
		if (!prefix) {
			throw new Error(this.getError(namespace,name));
		}
		
		var element = this._doc.getElementsByTagName(prefix + ":" + name)[0];
		if (!element) {
			throw new Error(this.getError(namespace,name));
		}

		return element.childNodes[0].nodeValue;
	}
	
});

})();