/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.dsv.exporter.EcoreExportEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("jazz.ui.internal.SwapList");
dojo.require("rmps.ui.SimpleButton");
dojo.require("rmpw.dsv.exporter.ExportStatusEditorInput");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmpw.dsv.exporter", "EcoreExportEditorMessages");
var messages = dojo.i18n.getLocalization("rmpw.dsv.exporter", "EcoreExportEditorMessages");


var ECORE_EXPORTER_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "ecore-exporter");
var BASELINE_PARAM = "rmps.context=";


var connectOnInput = function(obj, widget, method) {
	widget.connect(obj, "onchange", method);
};

dojo.declare("rmpw.dsv.exporter.EcoreExportEditor", rmps.app.ui.internal.editors.BaseEditor, {
		
	_requestButton: null,
	isReadOnly: true,
	
	createContentWidget: function() {
		return new rmpw.dsv.exporter.EcoreExportEditor._Content({_editor: this});
	},
	
	addButtons: function(toolbar) {
		this.inherited(arguments);
		this._requestButton = new rmps.ui.SimpleButton({
			label: messages.requestExport,
			onClick: dojo.hitch(this, this.requestExport)
		});
		
		toolbar.addWidget(this._requestButton);
		this._requestButton.setEnabled(false);
	},
	
	requestExport: function() {
	    this._requestExport(this);
	},
	
	_requestExport: function(editor, duplicates) {
		if (!editor) {
			editor = this;
		}
		var selections = this.contentWidget._swapList.getSelectedEntries();
		
		var uris = [];
		for ( i = 0; i < selections.length; i++) {
		   uris.push(selections[i].id);
		}
		
		var args = {
			headers: {
				"Content-Type": "application/json",
				"X-ibm-rmps-internal": "true"
			},
			url: ECORE_EXPORTER_PATH + "?" + BASELINE_PARAM + jazz.app.context.get().baseline,
			hasBody: true,
			postData: dojo.toJson(uris),
		}
		var deferred = jazz.client.xhrPost(args);
				
		deferred.addCallback(editor, function(result) {
			this._refreshStatus();
			editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.OK,
					summary: dojo.string.substitute(
						messages.importRequestSuccessful,
						[rmps.core.util.IRmpsConstants.ADMIN_EXPORT_STATUS]),
					closable: true
				});
		});
		deferred.addErrback(editor, function(error) {
			editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.error_request_general,
				details:[{message: messages['error_request_general.explanation'] + '<br>' + messages['error_request_general.useraction']}],
				closable: true
			});
		});
	},
	
	resize: function() {
		//override: do not resize
	},

    _refreshStatus: function() {
		var helper = rmps.ui.editor.PageHelper.getInstance();
		var statusEditor = helper.getEditor(
				new rmpw.dsv.exporter.ExportStatusEditorInput({projectId: this.getEditorInput().projectId}));
		if (statusEditor) {
			statusEditor.refresh();
		}
	}
});

dojo.declare("rmpw.dsv.exporter.EcoreExportEditor._Content", [dijit._Widget, dijit._Templated], {
	
	_editor: null,
	
	templateString: '<table>' +
			'<tr>' +
				 '<td dojoAttachPoint="_section">' +
				 '</td>' +
				 '<td>' +
				 '</td>' +
			'</tr>' +
		'</table>',
	
	_init: function() {
	    this._swapList = new jazz.ui.internal.SwapList({
		unselectedListTitleText: messages.unselectedListTitleText,
		selectedListTitleText: messages.selectedListTitleText,
		sortField: "label",
		sortSelected: false
		});
		// Get the ontology uris
		var url = ECORE_EXPORTER_PATH + "/ontologies" + "?" + BASELINE_PARAM + jazz.app.context.get().baseline;
		var getArgs = {
			url: url,
			headers: {
				"Accept": "application/json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json"
		};
		var handler = jazz.client.xhrGet(getArgs);
				
		this.connect(this._swapList, "_updateCount", function(count) {
		     var size = this._swapList.getSelectedEntries().length;
		     if ( size > 0 ) {
		        this._editor._requestButton.setEnabled(true);
		     } else {
		        this._editor._requestButton.setEnabled(false);
		     }
		});		
		this._swapList.showLoading(true);
		handler.addCallback(dojo.hitch(this, function(response) {
			this._swapList.showLoading(false);
			var unselectedEntries = [];
			var selectedEntries = [];
			for (var i = 0; i < response.length; i++) {
				var type = response[i];
				var item = {
					label: type.label,
					id: type.uri,
					type: i
				};
				unselectedEntries.push(item);
			}			
			this._swapList.setUnselectedEntries(unselectedEntries);
		}));
		handler.addErrback(dojo.hitch(this, function(response) {
		    this._swapList.showLoading(false);
		    this._editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.error_request_general,
				closable: true
			});
		}));
		this._section.appendChild(this._swapList.domNode);
		this._swapList.startup();
		
	},
	
	postCreate: function() {
		this.inherited(arguments);
		this._init();
	}
	
});

})();
