/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.dsv.exporter.EcoreExportStatus");

dojo.require("rmps.core.Enum");
dojo.require("rmps.core.user.UserClient");
dojo.require("dojo.date.stamp");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmpw.dsv.exporter", "EcoreExportStatusMessages");
var messages = dojo.i18n.getLocalization("rmpw.dsv.exporter", "EcoreExportStatusMessages");

var STATUS_CURRENT_TIME = "http://jazz.net/xmlns/rmps/status/1.0/currentTime";
var STATUS_START_TIME = "http://jazz.net/xmlns/rmps/status/1.0/startTime";
var STATUS_SYNC_KEY = "http://jazz.net/xmlns/rmps/status/1.0/syncKey";
var STATUS_REQUESTER = "http://jazz.net/xmlns/rmps/status/1.0/requester";
var STATUS_REQUEST_ID = "http://jazz.net/xmlns/rmps/status/1.0/requestId";
var STATUS_REQUEST_TIME = "http://jazz.net/xmlns/rmps/status/1.0/requestTime";
var STATUS_STATE = "http://jazz.net/xmlns/rmps/status/1.0/state";


// error
var STATUS_EXCEPTION = "http://jazz.net/xmlns/rmps/status/1.0/exception";

// completed
var STATUS_STOP_TIME = "http://jazz.net/xmlns/rmps/status/1.0/stopTime";

// pending
var STATUS_POSITION = "http://jazz.net/xmlns/rmps/status/1.0/position";

// canceled
var CANCELLER = "http://jazz.net/xmlns/rmps/status/1.0/canceller";


// exporter
var EXPORTER_ECOREMODELS_URI = "https://jazz.net/xmlns/rmps/export-status/1.0/exportedEcoreModelsUri";
var EXPORTER_ONTOLOGY_NAMES = "https://jazz.net/xmlns/rmps/export-status/1.0/ontologyNames";
var EXPORTER_DANGLING_REFERENCES = "https://jazz.net/xmlns/rmps/export-status/1.0/danglingReferences";
var EXPORTER_WARNINGS = "https://jazz.net/xmlns/rmps/export-status/1.0/warnings";


rmpw.dsv.exporter.EcoreExportStatusStates = new rmps.core.Enum({
	UNKNOWN: new rmps.core.EnumLiteral({name: messages.unknown, value: "unknown", icon: dojo.moduleUrl("rmpw.dsv.exporter", "graphics/icons/obj16/state_unknown.gif")}),
	PENDING: new rmps.core.EnumLiteral({name: messages.pending, value: "pending", icon: dojo.moduleUrl("rmpw.dsv.exporter", "graphics/icons/obj16/state_pending.gif")}),
	IN_PROGRESS: new rmps.core.EnumLiteral({name: messages.inProgress, value: "inProgress", icon: dojo.moduleUrl("rmpw.dsv.exporter", "graphics/icons/obj16/state_inProgress.gif")}),
	ERROR: new rmps.core.EnumLiteral({name: messages.error, value: "error", icon: dojo.moduleUrl("rmpw.dsv.exporter", "graphics/icons/obj16/state_error.gif")}),
	COMPLETED: new rmps.core.EnumLiteral({name: messages.completed, value: "completed", icon: dojo.moduleUrl("rmpw.dsv.exporter", "graphics/icons/obj16/state_completed.gif")}),
	WARNING: new rmps.core.EnumLiteral({name: messages.warning, value: "warning", icon: dojo.moduleUrl("rmpw.dsv.exporter", "graphics/icons/obj16/state_warning.gif")}),
	CANCELED: new rmps.core.EnumLiteral({name: messages.canceled, value: "canceled", icon: dojo.moduleUrl("rmpw.dsv.exporter", "graphics/icons/obj16/state_canceled.gif")}),
	CANCELING: new rmps.core.EnumLiteral({name: messages.canceling, value: "canceling", icon: dojo.moduleUrl("rmpw.dsv.exporter", "graphics/icons/obj16/state_canceling.gif")})
});

var UserClient = rmps.core.user.UserClient;
var States = rmpw.dsv.exporter.EcoreExportStatusStates;

dojo.declare("rmpw.dsv.exporter.EcoreExportStatus", null, {
	
	uri: null,
	
	currentTime: null, // Date
	startTime: null, // Date
	requestTime: null, // Date
	syncKey: null, 
	requester: null,
	requestId: null,
	state: null,
	position: null,	
	stopTime: null, // Date
	
	// error
	exception: null,
	
	
	//exporter
	exporter_ontologyNames: null, 
	exporter_exportedEcoreModelsUri : null,
	exporter_danglingReferences: null, 
	exporter_warnings: null, 
	
	
	// private properties
	_requester: null,
	_canceller: null,
	
	/**
	 * Return a deferred for the requester of this status.
	 * 
	 * @return a Deferred for requester of this status
	 * @type dojo.Deferred
	 */
	getRequester: function() {
		if (this._requester && this._requester.uri == this.requester) {
			var deferred = new dojo.Deferred();
			setTimeout(dojo.hitch(this, function() {
				deferred.callback(this._requester);
			}), 0);
			return deferred;
		}
		var deferred = UserClient.getUser(this.requester);
		deferred.addCallback(dojo.hitch(this, function(user) {
			this._requester = user;
		}));
		return deferred;
	},
	
	/**
	 * Return a deferred for the canceller of this status.
	 * 
	 * @return a Deferred for canceller of this status
	 * @type dojo.Deferred
	 */
	getCanceller: function() {
		if (this._canceller && this._canceller.uri == this._canceller) {
			var deferred = new dojo.Deferred();
			setTimeout(dojo.hitch(this, function() {
				deferred.callback(this._canceller);
			}), 0);
			return deferred;
		}
		var deferred = UserClient.getUser(this.canceller);
		deferred.addCallback(dojo.hitch(this, function(user) {
			this._canceller = user;
		}));
		return deferred;
	},
	
	parseRdfJson: function(uri, rdf) {
		this.uri = uri;
		
		this.state = States.get(rdf[STATUS_STATE][0].value);
		this.syncKey = rdf[STATUS_SYNC_KEY][0].value;
		this.requester = rdf[STATUS_REQUESTER][0].value;
		this.requestId = rdf[STATUS_REQUEST_ID][0].value;
		this.requestTime = dojo.date.stamp.fromISOString(rdf[STATUS_REQUEST_TIME][0].value);
		
		if (rdf[STATUS_CURRENT_TIME]) {
			this.currentTime = dojo.date.stamp.fromISOString(rdf[STATUS_CURRENT_TIME][0].value);
		}
		
		if (rdf[STATUS_START_TIME]) {
			this.startTime = dojo.date.stamp.fromISOString(rdf[STATUS_START_TIME][0].value);
		}
		
		if (rdf[STATUS_STOP_TIME]) {
			this.stopTime = dojo.date.stamp.fromISOString(rdf[STATUS_STOP_TIME][0].value);
		}
		
		// error
		if (rdf[STATUS_EXCEPTION]) {
			this.exception = rdf[STATUS_EXCEPTION][0].value;
		}
		
		// pending
		if (rdf[STATUS_POSITION]) {
			this.position = parseInt(rdf[STATUS_POSITION][0].value);
		}
		
		// canceled
		if (rdf[CANCELLER]) {
			this.canceller = rdf[CANCELLER][0].value;
		}
		
		
		// exporter
		if (rdf[EXPORTER_ECOREMODELS_URI]) {
			this.exporter_exportedEcoreModelsUri = rdf[EXPORTER_ECOREMODELS_URI][0].value;
		}
		
		function getArray(values) {
			var result = null;
			if (values) {
				result = [];
				for (var i = 0; i < values.length; ++i) {
					result.push(values[i].value);
				}
				result.sort();
			}
			return result;
		}
		
		this.exporter_danglingReferences= getArray(rdf[EXPORTER_DANGLING_REFERENCES]);
		this.exporter_ontologyNames= getArray(rdf[EXPORTER_ONTOLOGY_NAMES]);
		this.exporter_warnings= getArray(rdf[EXPORTER_WARNINGS]);
	}
});
})();