/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.EditController");

dojo.require("rmpw.common.policy.Host");

dojo.require("rmpw.diagrams.edit.tools.ContextActionBar");

/**
 * Controller that manages all interactivity attached to a diagram element, delegating
 * most of its responsibilities to a collection of edit policies.
 * 
 * @author Steve Shaw, Marc Gobeil
 */
dojo.declare("rmpw.diagrams.edit.EditController", rmpw.common.policy.Host, {
	constructor: function(renderMapping) {
		if(renderMapping) { // TODO: rendermapping not optional once legacy support removed
			this.setRenderMapping(renderMapping);
		}
		
		// publish an event for clients to add their own custom extensibility
		dojo.publish("rmpw.diagrams.edit.EditController/constructor", [this]);
		
		this._connectSleep = dojo.connect(renderMapping, "sleep", dojo.hitch(this, function() {
			this.destroyPolicies();
			dojo.disconnect(this._connectEnforce);
			
			this._connectAwake = dojo.connect(renderMapping, "awaken", dojo.hitch(this, function() {
				this.policies = [];
				this.createPolicies();
				this._connectEnforce = dojo.connect(this.renderMapping, "enforcePolicies", this, "enforcePolicies");
			}));
		}));
	},
	
	destroy: function() {
		this.inherited(arguments);
		
		dojo.disconnect(this._connectSleep);
		delete this._connectSleep;
		dojo.disconnect(this._connectAwake);
		delete this._connectAwake;
		dojo.disconnect(this._connectEnforce);
		delete this._connectEnforce;
	},
	
	/**
	 * @deprecated
	 * TODO: collapse into constructor body
	 */
	setRenderMapping: function(renderMapping) {
		this.renderMapping = renderMapping;
		this.renderMapping.__editController = this;
		
		// TODO (MG): remove, shim for old code
		this.viewer = renderMapping.root;	
	},
	
	/*
	 * Gets the default dojox shape object that would be the typical top level target for edit operations
	 */
	getDefaultTargetShape: function() {
		if (this.renderMapping.policies && this.renderMapping.policies.graphElement) {
			return this.renderMapping.getShape();
		}
		else if (this.renderMapping.rootShapes) {
			return this.renderMapping.rootShapes[0];
		}
		
		return null;
	},

	setParent: function(parent) {
		this.inherited(arguments);
		this.createPolicies();
		
		// initialize to all existing children
		var renderChildren = this.renderMapping.children;
		for(var i = 0, child; child = renderChildren[i]; i++) {
			this._attachChild(child);
		}
		
		// watch for future additions (no need to clean up render mapping listeners because of shared lifetime)
		dojo.connect(this.renderMapping, "addChild", this, "_attachChild");
		this._connectEnforce = dojo.connect(this.renderMapping, "enforcePolicies", this, "enforcePolicies");
		dojo.connect(this.renderMapping, "destroy", this, "destroy");
	},
	
	_attachChild: function(renderChild) {
		this._currentMapping = renderChild;
		this.attachChild(renderChild);
	},
	
	attachChild: function(renderChild) {
		// TODO: remove, only here for legacy support
		this._renderMappingChildAdded(renderChild);
	},
	
	createChild: function(constructorFunc) {
		var child = new constructorFunc(this._currentMapping);
		this.addChild(child);
		delete this._currentMapping; // prevent same child being mapped twice
		return child;
	},
	
	/**
	 * @deprecated override attachChild() instead, and make calls to createChild()
	 */
	_renderMappingChildAdded: function(renderChild) {
		var editChild = this.connectEditMapping(renderChild);
		if (editChild) {
			editChild.setRenderMapping(renderChild);
			this.addChild(editChild);
		}
	},
	
	/**
	 * @deprecated use attachChild instead
	 */
	connectEditMapping: function(mapping) {
		// subclasses should override
	},
	
	/*
	 * Separates policy creation into actions and tools
	 */
	createPolicies: function() {
		this.createActions();
		this.createActionBindings();
		this.createTools();
	},
	
	/*
	 * Creates actions that will be invokable through this
	 * edit controller
	 */
	createActions: function() {
	},
	
	/*
	 * Creates action bindings that will be invokable through this
	 * edit controller
	 */
	createActionBindings: function() {
	},
	
	/*
	 * Creates tools that will be utilized through this
	 * edit controller
	 */
	createTools: function() {
	},
	
	/**
	 * Used to contribute an external action.  For example, an action to delete
	 * the item from its container's collection. By default, adds the action to
	 * a context action bar, creating one if necessary.
	 * 
	 * @param action an action binding
	 */
	contributeAction: function(action) {
		var contextMenu = this.policies.contextActionBar;
		if(!contextMenu) {
			contextMenu = this.createPolicy(rmpw.diagrams.edit.tools.ContextActionBar);
		}
		contextMenu.addActionBinding(action);
	}
});