/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.EditControllerExtension");

dojo.require("rmpw.diagrams.edit.EditController");

/**
 * This is used by clients wishing to extend / override the behavior generation 
 * with hand-coded extensions to the behavior runtime.  
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.EditControllerExtension", null, {
	
	subscribeEC: dojo.subscribe("rmpw.diagrams.edit.EditController/constructor", function(editController) { 
		var ECStatic = rmpw.diagrams.edit.EditControllerExtension.prototype;
		if (!ECStatic.extensions) {
			return;
		}
		
		var editControllerExt = ECStatic.extensions[editController.declaredClass];
		if (editControllerExt) {
			if (editControllerExt.createActionsExtension) {
				editControllerExt._connectCreateActions = dojo.connect(editController, "createActions", dojo.hitch(editControllerExt, function() {
					editControllerExt.createActionsExtension(editController);
					dojo.disconnect(ECStatic._connectCreateActions);
				}));
			}
			
			if (editControllerExt.createActionBindingsExtension) {
				editControllerExt._connectCreateActionBindings = dojo.connect(editController, "createActionBindings", dojo.hitch(editControllerExt, function() {
					editControllerExt.createActionBindingsExtension(editController);
					dojo.disconnect(ECStatic._connectCreateActionBindings);
				}));
			}
			
			if (editControllerExt.createToolsExtension) {
				editControllerExt._connectCreateTools = dojo.connect(editController, "createTools", dojo.hitch(editControllerExt, function() {
					editControllerExt.createToolsExtension(editController);
					dojo.disconnect(ECStatic._connectCreateTools);
				}));
			}
		}
	}),
	
	// static map of class name to extensions
	extensions: {},
	
	constructor: function(extensionClass) {
		rmpw.diagrams.edit.EditControllerExtension.prototype.extensions[extensionClass] = this;
	},
	
	createActionsExtension: null,
	createActionBindingsExtension: null,
	createToolsExtension: null,
	
	/*
	 * Creates actions that will be invokable through this
	 * edit controller
	 */
	setCreateActionsExtension: function(createActionsExt) {
		this.createActionsExtension = dojo.hitch(this, createActionsExt);
	},
	
	/*
	 * Creates action bindings that will be invokable through this
	 * edit controller
	 */
	setCreateActionBindingsExtension: function(createActionBindingsExt) {
		this.createActionBindingsExtension = dojo.hitch(this, createActionBindingsExt);
	},
	
	/*
	 * Creates tools that will be utilized through this
	 * edit controller
	 */
	setCreateToolsExtension: function(createToolsExt) {
		this.createToolsExtension = dojo.hitch(this, createToolsExt);
	}
});