/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.EditRoot");

dojo.require("rmpw.common.vcs.Repository");
dojo.require("rmpw.common.command.CommandStack");
dojo.require("rmpw.diagrams.render.RenderRoot");
dojo.require("rmpw.diagrams.edit.CommandController");
dojo.require("rmpw.diagrams.edit.tools.ContextActionBarScope");
dojo.require("rmpw.diagrams.edit.tools.ActionBarScope");
dojo.require("rmpw.diagrams.edit.tools.SelectionScope");
dojo.require("rmpw.diagrams.edit.tools.FocusScope");
dojo.require("rmpw.diagrams.edit.tools.AcceleratorScope");

dojo.require("rmpw.diagrams.edit.actions.ToggleRouter");
dojo.require("rmpw.diagrams.edit.actionBindings.KeyStroke");
dojo.require("rmpw.diagrams.edit.actions.Nudge");
dojo.require("rmpw.diagrams.edit.actions.NavigateToPeer");
dojo.require("rmpw.diagrams.edit.actions.NavigateToEdge");
dojo.require("rmpw.diagrams.edit.actions.NavigateToHandle");
dojo.require("rmpw.diagrams.edit.actions.NavigateToParent");
dojo.require("rmpw.diagrams.edit.actions.NavigateToChildren");
dojo.require("rmpw.diagrams.edit.actions.Cancel");
dojo.require("rmpw.diagrams.edit.actions.Commit");
dojo.require("rmpw.diagrams.edit.actions.ToggleSelect");
dojo.require("rmpw.diagrams.edit.actions.DeleteBendpoint");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmpw.diagrams.edit", "BehaviorMessages");

/**
 * Domain agnostic root to the edit controller tree.
 */
dojo.declare("rmpw.diagrams.edit.EditRoot", rmpw.diagrams.render.RenderRoot, {
	constructor: function(args) {
		this.behaviorMessages = dojo.i18n.getLocalization("rmpw.diagrams.edit", "BehaviorMessages"); 
		this.repository = args.repository;
		this.diagramLayer = args.diagramLayer;
		this.createPolicies();
	},
	
	createPolicies: function() {
		this.createPolicy(rmpw.diagrams.edit.tools.SelectionScope);
		this.createPolicy(rmpw.diagrams.edit.tools.ContextActionBarScope);
		if (this.configuration.toolbar) {
			// diagram toolbar is passed in, it needs to be populated
			var actionBarScope = this.viewer.createPolicy(rmpw.diagrams.edit.tools.ActionBarScope);
			actionBarScope.setToolbarWidget(this.configuration.toolbar);
			actionBarScope.addStaticToolbarWidgets({
				toolInfo : this.configuration.tools, 
				diagramLayer: this.diagramLayer, 
				rootLayer: this.configuration.rootLayer});
		}
		
		var undoKeyStroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		undoKeyStroke.setCtrlKey(true);
		undoKeyStroke.setKeyChar('z');
		undoKeyStroke.setAction(new rmpw.diagrams.edit.actions.Undo());
		var redoKeyStroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		redoKeyStroke.setCtrlKey(true);
		redoKeyStroke.setKeyChar('y');
		redoKeyStroke.setAction(new rmpw.diagrams.edit.actions.Redo());
		var toggleRouterKeyBinding = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		toggleRouterKeyBinding.setCtrlKey(true);
		toggleRouterKeyBinding.setKeyChar('l');
		toggleRouterKeyBinding.setAction(new rmpw.diagrams.edit.actions.ToggleRouter());

		// nudge
		var nudgeNorth = new rmpw.diagrams.edit.actions.Nudge();
		nudgeNorth.setDirection("North");
		var nudgeNorthKeystroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		nudgeNorthKeystroke.setKeyCode(dojo.keys.UP_ARROW);
		nudgeNorthKeystroke.setAction(nudgeNorth);
		var nudgeSouth = new rmpw.diagrams.edit.actions.Nudge();
		nudgeSouth.setDirection("South");
		var nudgeSouthKeystroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		nudgeSouthKeystroke.setKeyCode(dojo.keys.DOWN_ARROW);
		nudgeSouthKeystroke.setAction(nudgeSouth);
		var nudgeWest = new rmpw.diagrams.edit.actions.Nudge();
		nudgeWest.setDirection("West");
		var nudgeWestKeystroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		nudgeWestKeystroke.setKeyCode(dojo.keys.LEFT_ARROW);
		nudgeWestKeystroke.setAction(nudgeWest);
		var nudgeEast = new rmpw.diagrams.edit.actions.Nudge();
		nudgeEast.setDirection("East");
		var nudgeEastKeystroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		nudgeEastKeystroke.setKeyCode(dojo.keys.RIGHT_ARROW);
		nudgeEastKeystroke.setAction(nudgeEast);
		
		// navigate
		var navigateNorth = new rmpw.diagrams.edit.actions.NavigateToPeer();
		navigateNorth.setDirection("North");
		var navigateNorthKeystroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		navigateNorthKeystroke.setCtrlKey(true);
		navigateNorthKeystroke.setKeyCode(dojo.keys.UP_ARROW);
		navigateNorthKeystroke.setAction(navigateNorth);
		var navigateSouth = new rmpw.diagrams.edit.actions.NavigateToPeer();
		navigateSouth.setDirection("South");
		var navigateSouthKeystroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		navigateSouthKeystroke.setCtrlKey(true);
		navigateSouthKeystroke.setKeyCode(dojo.keys.DOWN_ARROW);
		navigateSouthKeystroke.setAction(navigateSouth);
		var navigateWest = new rmpw.diagrams.edit.actions.NavigateToPeer();
		navigateWest.setDirection("West");
		var navigateWestKeystroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		navigateWestKeystroke.setCtrlKey(true);
		navigateWestKeystroke.setKeyCode(dojo.keys.LEFT_ARROW);
		navigateWestKeystroke.setAction(navigateWest);
		var navigateEast = new rmpw.diagrams.edit.actions.NavigateToPeer();
		navigateEast.setDirection("East");
		var navigateEastKeystroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		navigateEastKeystroke.setCtrlKey(true);
		navigateEastKeystroke.setKeyCode(dojo.keys.RIGHT_ARROW);
		navigateEastKeystroke.setAction(navigateEast);
		
		// navigate containment
		var navigateParent = new rmpw.diagrams.edit.actions.NavigateToParent();
		var navigateParentKeystroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		navigateParentKeystroke.setAltKey(true);
		navigateParentKeystroke.setKeyCode(dojo.keys.UP_ARROW);
		navigateParentKeystroke.setAction(navigateParent);
		var navigateChildren = new rmpw.diagrams.edit.actions.NavigateToChildren();
		var navigateChildrenKeystroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		navigateChildrenKeystroke.setAltKey(true);
		navigateChildrenKeystroke.setKeyCode(dojo.keys.DOWN_ARROW);
		navigateChildrenKeystroke.setAction(navigateChildren);
		
		// navigate edges
		var navigateForwardEdge = new rmpw.diagrams.edit.actions.NavigateToEdge();
		navigateForwardEdge.setForward(true);
		var navigateForwardEdgeKeystroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		navigateForwardEdgeKeystroke.setKeyChar('/');
		navigateForwardEdgeKeystroke.setAction(navigateForwardEdge);
		var navigateBackwardEdge = new rmpw.diagrams.edit.actions.NavigateToEdge();
		navigateBackwardEdge.setForward(false);
		var navigateBackwardEdgeKeystroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		navigateBackwardEdgeKeystroke.setKeyChar('\\');
		navigateBackwardEdgeKeystroke.setAction(navigateBackwardEdge);
		
		var cancel = new rmpw.diagrams.edit.actions.Cancel();
		var cancelKeystroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		cancelKeystroke.setKeyCode(dojo.keys.ESCAPE);
		cancelKeystroke.setAction(cancel);
		var commit = new rmpw.diagrams.edit.actions.Commit();
		var commitKeystroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		commitKeystroke.setKeyCode(dojo.keys.ENTER);
		commitKeystroke.setAction(commit);
		
		var toggleSelect = new rmpw.diagrams.edit.actions.ToggleSelect();
		var toggleSelectKeystroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		toggleSelectKeystroke.setCtrlKey(true);
		toggleSelectKeystroke.setKeyChar(' ');
		toggleSelectKeystroke.setAction(toggleSelect);
		
		var accessibleHandles = new rmpw.diagrams.edit.actions.NavigateToHandle();
		var accessibleHandlesKeystroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		accessibleHandlesKeystroke.setKeyChar(' ');
		accessibleHandlesKeystroke.setAction(accessibleHandles);
		
		var deleteBendpoint = new rmpw.diagrams.edit.actions.DeleteBendpoint();
		var deleteBendpointKeystroke = new rmpw.diagrams.edit.actionBindings.KeyStroke();
		deleteBendpointKeystroke.setKeyCode(dojo.keys.DELETE);
		deleteBendpointKeystroke.setAction(deleteBendpoint);
		
		var policy = this.createPolicy(rmpw.diagrams.edit.tools.AcceleratorScope);
		policy.setActionBindings(undoKeyStroke, redoKeyStroke, toggleRouterKeyBinding, 
				nudgeNorthKeystroke, nudgeSouthKeystroke, nudgeWestKeystroke, nudgeEastKeystroke,
				navigateNorthKeystroke, navigateSouthKeystroke, navigateWestKeystroke, navigateEastKeystroke,
				navigateForwardEdgeKeystroke, navigateBackwardEdgeKeystroke, deleteBendpointKeystroke,
				navigateParentKeystroke, navigateChildrenKeystroke,
				cancelKeystroke, commitKeystroke, toggleSelectKeystroke, accessibleHandlesKeystroke);
		
		this.createPolicy(rmpw.diagrams.edit.tools.FocusScope);
	},
	
	getSelectionController: function() {
		return this.policies.selectionScope;
	},
	
	getAcceleratorController: function() {
		return this.policies.acceleratorScope;
	},
	
	destroy: function() {
		this.inherited(arguments);
		
		if (this._commandController) {
			this._commandController.destroy();
		}
		
		dojo.disconnect(this._connectCommandStack);
	},
		
	getCommandController: function( attachPoint ) {
		if(!this._commandController) {			
			var commandStack = new rmpw.common.command.CommandStack(this.repository);
			this._connectCommandStack = dojo.connect(commandStack, "postExecuteHook", dojo.hitch(this, function() {
				this.getInvalidationController().performInvalidation();
			}));
            this._commandController = this.configuration.commandControllerClass ? new this.configuration.commandControllerClass(this, 
                attachPoint, commandStack) : new rmpw.diagrams.edit.CommandController(this, attachPoint, commandStack);
        }
		return this._commandController;
	},
	
	// TODO (MG): move to domain specific root rather than diagram root
	getCommandPackage: function() {
		return this.commandPackage;
	},

	execute: function( command, data ){
		if( !data ){ 
		    data = {}; 
		}
		data.viewer = this;
		this.getCommandController().commandStack.execute( command, data );	
	},
	
	/**
	 * TODO (MG): the diagram should be destroyed at a higher level to avoid
     * this manual resetting business.
	 */
	clear: function() {
		// Don't clear if we're consructing
		if (this._constructing) {
			return;
		}
		
		for (var key in this.policies) {
			var policy = this.policies[key];
			if (policy.destroy) {
				policy.destroy();
			}
		}
		this.inherited(arguments);
		this.policies = [];
		this.createPolicies();
	}
});