/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.EditableCollapsibleSection");

dojo.require("rmps.ui.CollapsibleSection");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.ui", "EditableSectionMessages");

var messages = dojo.i18n.getLocalization("rmps.ui", "EditableSectionMessages");

/**
 * @author Christian Vogt (cvogt)
 */
dojo.declare("rmps.ui.EditableCollapsibleSection", rmps.ui.CollapsibleSection, {
	
	readOnly: false,
	
	_editButton: null,
	_previewButton: null,
	_editMode: false,
	
	postCreate: function() {
		this.inherited(arguments);
		dojo.addClass(this.domNode, "mode-preview");
		
		// create Edit/Preview command
		this.connect(this.containerNode, "onclick", function(e) {
			if (!this.readOnly && !this._editMode) {
				this.edit();
				if (e.target.nextSibling && e.target.nextSibling.nodeName.toLowerCase() == "input") {
					e.target.nextSibling.focus();
				}
			}
		});
		
		this._editButton = this.addAction(messages.edit, messages.edit, this.edit);
		this._previewButton = this.addAction(messages.preview, messages.preview, this.preview);
		
		var readOnly = this.readOnly;
		this.readOnly = null;
		this.setReadOnly(readOnly);
	},
	
	setReadOnly: function(readOnly) {
		if (this.readOnly !== readOnly) {
		    this.readOnly = readOnly;
			if (readOnly) {
			    this.containerNode.title = "";
				this._editButton.style.display = this._previewButton.style.display = "none";
				if (this._editMode) {
					this.preview();
				}
			} else {
			    this.containerNode.title = messages.clickToEdit;
			    if (this._editMode) {
					this._previewButton.style.display = "";
					this._editButton.style.display = "none";
				} else {
					this._previewButton.style.display = "none";
				    this._editButton.style.display = "";
				}
			}
		}
	},
	
	edit: function() {
	    this.expand();
		if (this._editMode || this.isReadOnly) {
			return;
		}
		
		dojo.addClass(this.domNode, "mode-edit");
		dojo.removeClass(this.domNode, "mode-preview");
		
		this._editMode = true;
		this.containerNode.title = "";
		
		this._editButton.style.display = "none";
		this._previewButton.style.display = "";
		
		this.onEdit();
	},
	
	preview: function() {
	    this.expand();
		if (!this._editMode) {
			return;
		}
		
		dojo.addClass(this.domNode, "mode-preview");
		dojo.removeClass(this.domNode, "mode-edit");
		
		this._editMode = false;
		this.containerNode.title = messages.clickToEdit;
		
		this._editButton.style.display = "";
		this._previewButton.style.display = "none";
		
		this.onPreview();
	},
	
	// events
	onEdit: function() {},
	onPreview: function() {}
});

})();