/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.EllipsisLabel");

dojo.require("rmps.core.util.StringUtils");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.html");

var StringUtils = rmps.core.util.StringUtils;

/**
 * @author Christian Vogt (cvogt)
 */
dojo.declare("rmps.ui.EllipsisLabel", [dijit._Widget, dijit._Templated], {
	
	templateString: '<div><div dojoAttachPoint="_wrapperNode"><span dojoAttachPoint="containerNode"></span></div></div>',
	
	// mixin properties
	
	// ellipsis location
	// one of: s|m|e (start, middle, end), default is 'e'
	ellipsisLocation: 'e',
	
	// whether or not the ellipsis should occur at a word boundary, default is false
	ellipsisWord: false,
	
	label: null,
	
	// If true, the html title attribute will always be set to the label value.
	// If false, only when the label has been ellipsised will the title be set.
	alwaysSetTitle: false,
	
	// If true (default), the title will be set according to the state of 'alwaySetTitle'.
	// If false, the title will never be set.
	showTitle: true,
	
	// The max number of lines this label will show.
	// Defaults to a single line of text.
	lines: 1,
	
	// Constrains the width of the outter div not to exceed the width of the label node
	// when the label is displayed in full.
	constrainWidth: false,
	
	// delay in ms between updates
	delay: 0,
	
	// properties
	isVisible: true,
	
	// the child node containing the text to ellipsis
	// default: containerNode
	target: null,
	
	offset: 0,
	
	isTitleLink: false,
	titleHref: null,
	
	// private properties
	_wrapperNode: null,
	_length: 0,
	_timeout: null,
	_showFullLabel: false,
	_isEllipsised: false,
	
	_nativeSupport: false,
	_minEllipsisWidth: null,
	
	constructor: function() {
		// TODO fix native support for IE 8
		this._nativeSupport = !(dojo.isFF || dojo.isIE >= 8 || this.ellipsisWord || this.ellipsisLocation != 'e');
	},
	
	postCreate: function() {
		if (this._nativeSupport) {
			this._wrapperNode.style.whiteSpace = "nowrap";
			this._wrapperNode.style.textOverflow = "ellipsis";
			this._wrapperNode.style.overflow = "hidden";
		} else {
			this.domNode.style.overflow = "hidden";
		}
		if (this.target) {
			this.initTarget(this.target);
		} else {
			var childNodes = this.containerNode.childNodes;
			for (var i = 0; i < childNodes.length; ++i) {
				if (childNodes[i].nodeType != 3) {
					// user must explicitly call initTarget
					return;
				}
			}
			this.initTarget(this.containerNode);
		}
	},
	
	initTarget: function(target) {
		this.target = target;
		if (this.label == null) {
			this.label = "";
			var childNodes = this.target.childNodes;
			for (var i = 0; i < childNodes.length; ++i) {
				this.label += childNodes[i].nodeValue;
			}
		}
		this._setLabel(this.label);
	},
	
	destroy: function() {
		if (this._timeout) {
			clearTimeout(this._timeout);
			delete this._timeout;
		}
		this.inherited(arguments);
	},
	
	setVisibility: function(visible) {
		if (this.isVisible != visible) {
			this.isVisible = visible;
			if (this.isVisible) {
				this.domNode.style.display = "";
			} else {
				this.domNode.style.display = "none";
			}
		}
	},
	
	setLabel: function(newLabel, silently) {
		if (this.label == newLabel) {
			return;
		}
		this.label = newLabel;
		this._length = this.label.length;
		this._isEllipsised = false;
		
		this._setTitle();
		
		if (this._nativeSupport) {
			this._findMinEllipsisWidth();
		}
		this._setLabel(this.label);
		
		if (!silently) {
			this._queueResize(true);
		}
	},
	
	_findMinEllipsisWidth: function() {
		var testLabel = StringUtils.truncate(this.label, 3, this.ellipsisLocation, true, this.ellipsisWord);
		testLabel += "...";
		this._setLabel(testLabel);
		this._minEllipsisWidth = this.target.offsetWidth;
		this._setLabel(this.label);
	},
	
	_setTitle: function() {
		if (this.showTitle) {
			if (this.alwaysSetTitle || (!this._showFullLabel && this.isEllipsised())) {
				this.target.title = this.label;
			} else {
				this.target.title = "";
			}
		}
	},
	
	resize: function(size) {
		this._resize(size, false);
	},
	
	forceResize: function(size) {
		this._resize(size, true);
	},
	
	showFullLabel: function(show) {
		if (this._showFullLabel != show) {
			this._showFullLabel = show;
			if (this._showFullLabel) {
				if (this._nativeSupport) {
					this._wrapperNode.style.whiteSpace = "";
					this._wrapperNode.style.overflow = "";
				} else {
					this._wrapperNode.style.minWidth = this.getLabelNodeWidth() + "px";
					this._setLabel(this.label);
				}
				this._setTitle();
			} else {
				if (this._nativeSupport) {
					this._wrapperNode.style.whiteSpace = "nowrap";
					this._wrapperNode.style.overflow = "hidden";
				} else {
					this._wrapperNode.style.minWidth = "";
					this._setEllipsisedLabel(this._length);
				}
			}
			this._queueResize(true);
		}
	},
	
	_resize: function(size, force) {
		if (!size || size.w == null) {
			return;
		}
		
		this._queueResize(force, size.w);
	},
	
	isEllipsised: function() {
		if (this._nativeSupport) {
			return this._isEllipsised;
		} else {
			return this._length != this.label.length;
		}
	},
	
	_hasMaxHeight: function() {
		if (this._maxHeight == null) {
			if (this.target.innerHTML.length > 0 && this.domNode.offsetHeight == 0 &&
			this.target.offsetHeight === 0) {
				return false;
			}
			var temp = this.target.innerHTML;
			var testHtml = "&nbsp;";
			for (var i = 1; i < this.lines; ++i) {
				testHtml += "&nbsp;<br>";
			}
			dojo.html.set(this.target, testHtml);
			var maxHeight = this.domNode.offsetHeight;
			dojo.html.set(this.target, temp);
			if (maxHeight == 0) {
				return false;
			}
			this._maxHeight = maxHeight;
		}
		return true;
	},
	
	_queueResize: function(force, width) {
		if (!this.isVisible || this.label == null) {
			return;
		}
		
		if (this._nativeSupport) {
			if (width != null) {
				this._isEllipsised = this._wrapperNode.scrollWidth > width;
			
				if (!this.isEllipsised() && this.constrainWidth) {
					this._wrapperNode.style.width = (this.target.offsetWidth + this.offset) + "px";
					this._wrapperNode.style.textOverflow = "ellipsis";
				} else {
					this._wrapperNode.style.width = width + "px";
					
					// Fix for web kit bug where the target will show in full when 
					// wrapperNode width is 0.
					if (!this._showFullLabel && dojo.isWebKit) {
						if (width == 0) {
							this.containerNode.style.display = "none";
						} else {
							this.containerNode.style.display = "";
						}
					}
					
					if (this._minEllipsisWidth == null) {
						this._findMinEllipsisWidth();
					}
					
					if (width < (this.offset + this._minEllipsisWidth)) {
						this._setEllipsisedLabel(3);
						this._wrapperNode.style.textOverflow = "";
					} else {
						this._wrapperNode.style.textOverflow = "ellipsis";
						this._setLabel(this.label);
					}
				}
				
				this._setTitle();
			}
			
		} else {
			if (this._showFullLabel) {
				this._wrapperNode.style.minWidth = this.getLabelNodeWidth() + "px";
				if (width != null) {
					this._wrapperNode.style.width = width + "px";
				}
				return;
			}
			
			this._wrapperNode.style.minWidth = "";
			
			if (!this._hasMaxHeight()) {
				width = 0;
			}
			
			if (this._timeout != null) {
				clearTimeout(this._timeout);
				this._timeout = null;
			}
			
			if (force || this.delay == 0) {
				this._doResize(width);
			} else {
				this._timeout = setTimeout(dojo.hitch(this, function() {
					this._doResize(width);
				}), this.delay);
			}
		}
	},
	
	_doResize: function(width) {
		var setWidth = true;
		if (width == null) {
			setWidth = false;
			width = this._wrapperNode.clientWidth;
		} else {
			this.domNode.style.width = this._wrapperNode.style.width;
			this._wrapperNode.style.width = width + "px";
		}
		
		if (width == 0) {
			this._length = 0;
			this._setEllipsisedLabel(this._length);
		} else {
			var curWidth;
			
			while (this._length < this.label.length && this.target.offsetHeight <= this._maxHeight && ((curWidth = (this.target.offsetWidth + this.offset)) < width)) {
				curWidth = curWidth - (2 * this.offset);
				if (this._length == 0) {
					++this._length;
				} else if (this.lines == 1) {
					this._length = Math.min(Math.ceil((width - curWidth) / curWidth * this._length) + this._length, this.label.length);
				} else {
					var newLength = Math.min(Math.ceil(width / curWidth * this._length) * this.lines, this.label.length);
					this._length = Math.max(this._length + 1, newLength);
				}
				this._setEllipsisedLabel(this._length);
			}
			
			while (this._length > 0 && (this.target.offsetHeight > this._maxHeight || (this.target.offsetWidth + this.offset) > width)) {
				// TODO optimize label reduction
				this._setEllipsisedLabel(--this._length);
			}
		}
		
		if (this.label.length == this._length && this.constrainWidth && setWidth) {
			this._wrapperNode.style.width = (this.target.offsetWidth + this.offset) + "px";
		}
		
		this._setTitle();
		
		if (setWidth) {
			this.domNode.style.width = "";
		}
	},
	
	_setEllipsisedLabel: function(length) {
		var ellipsisedLabel = StringUtils.truncate(this.label, length, this.ellipsisLocation, true, this.ellipsisWord);
		this._setLabel(ellipsisedLabel);
	},
	
	getLabelNodeWidth: function() {
		// TODO fix IE8 native support
		if (dojo.isIE >= 8) {
			var width = this._wrapperNode.style.width;
			this._wrapperNode.style.width = "";
			var result = this.target.offsetWidth;
			this._wrapperNode.style.width = width;
			return result;
		}
		if (this._nativeSupport && this.isEllipsised()) {
			return this._wrapperNode.offsetWidth;
		}
		return this.target.offsetWidth;
	},
	
	_setLabel: function(newLabel) {
		var textnode, hrefnode;
		while (this.target.firstChild) {
			this.target.removeChild(this.target.firstChild);
		}
		
		textnode = dojo.doc.createTextNode(newLabel);
		
		if (this.isTitleLink) {
			hrefnode = dojo.doc.createElement("a");
			hrefnode.href = this.titleHref;
			hrefnode.appendChild(textnode);
			this.target.appendChild(hrefnode);
		} else {
			this.target.appendChild(textnode);
		}
		
	},
	
	reset: function() {
		this.domNode.style.width = "";
		this._length = this.label.length;
		this._isEllipsised = false;
		this._setLabel(this.label);
	}
});

})();