/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.EllipsisLabelContainer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("rmps.ui.EllipsisLabel");

/**
 * @author Christian Vogt (cvogt)
 */
dojo.declare("rmps.ui.EllipsisLabelContainer", [dijit._Widget, dijit._Templated], {
	
	templateString: '<div style="overflow: hidden; white-space: nowrap;"><nobr dojoAttachPoint="containerNode"></nobr></div>',
	
	// mixin properties
	delay: 0,
	
	// private properties
	_ellipsisLabels: null, //[]
	_nodes: null, //[]
	_prevWidth: -1,
	_timeout: null,
	
	startup: function() {
		this.inherited(arguments);
		this.init();
	},
	
	destroy: function() {
		if (this._timeout) {
			clearTimeout(this._timeout);
			delete this._timeout;
		}
		this.inherited(arguments);
	},
	
	init: function() {
		this._ellipsisLabels = [];
		this._nodes = [];
		var children = this.containerNode.childNodes;
		for (var i = 0; i < children.length; ++i) {
			var child = children[i];
			var widget = dijit.byNode(child);
			if (widget instanceof rmps.ui.EllipsisLabel) {
				var inserted = false;
				for (var j = 0; j < this._ellipsisLabels.length; ++j) {
					if (widget.priority < this._ellipsisLabels[j].priority) {
						this._ellipsisLabels.splice(j, 0, widget);
						inserted = true;
						break;
					}
				}
				if (!inserted) {
					this._ellipsisLabels.push(widget);
				}
				
				this.connect(widget, "setLabel", this._labelChanged);
			} else {
				this._nodes.push(child);
			}
		}
	},
	
	_labelChanged: function() {
		for (var i = 0; i < this._ellipsisLabels.length; ++i) {
			this._ellipsisLabels[i].reset();
		}
		this._prevWidth = -1;
		this.resize();
	},
	
	resize: function(size) {
		this._resize(size, false);
	},
	
	forceResize: function(size) {
		this._resize(size, true);
	},
	
	_resize: function(size, force) {
		var width;
		if (size && size.w != null) {
			width = size.w;
		} else {
			width = this.domNode.offsetWidth;
		}
		
		if (this._prevWidth == width) {
			return;
		}
		
		this._queueResize(force, width);
	},
	
	_queueResize: function(force, width) {
		if (this._timeout != null) {
			clearTimeout(this._timeout);
			this._timeout = null;
		}
		
		if (force || this.delay == 0) {
			this._doResize(width);
		} else {
			this._timeout = setTimeout(dojo.hitch(this, function() {
				this._doResize(width);
			}), this.delay);
		}
	},
	
	_doResize: function(width) {
		var labelsWidth = 0;
		var constrainedLabelsWidth = 0;
		var newWidths = new Array(this._ellipsisLabels.length);
		var unconstrainedLabels = [];
		var isEllipsised = false;
		
		var nodesWidth = 0, i, labelWidth;
		for (i = 0; i < this._nodes.length; ++i) {
			nodesWidth += this._nodes[i].offsetWidth;
		}
		
		for (i = 0; i < this._ellipsisLabels.length; ++i) {
			labelWidth = 0;
			if ((width > this._prevWidth || this._prevWidth == -1) && 
					!isEllipsised && this._ellipsisLabels[i].isEllipsised()) {
				labelWidth = Math.max(width - labelsWidth - nodesWidth, 0);
				
				// ignore delay
				this._ellipsisLabels[i].forceResize({ w: labelWidth });
				if (this._ellipsisLabels[i].isEllipsised()) {
					isEllipsised = true;
				} else {
					labelWidth = this._ellipsisLabels[i].getLabelNodeWidth();
				}
			} else {
				labelWidth = this._ellipsisLabels[i].getLabelNodeWidth();
			}
			newWidths[i] = labelWidth;
			labelsWidth += labelWidth;
			if (!this._ellipsisLabels[i].constrainWidth) {
				unconstrainedLabels.push(i);
			} else {
				constrainedLabelsWidth += labelWidth;
			}
		}
		
		if (width < this._prevWidth || this._prevWidth == -1) {
			var offset = labelsWidth;
			for (i = this._ellipsisLabels.length - 1; i >= 0; --i) {
				offset -= this._ellipsisLabels[i].getLabelNodeWidth();
				
				labelWidth = Math.max(width - offset - nodesWidth, 0);
				newWidths[i] = labelWidth;
				
				if (labelWidth > 0) {
					break;
				}
			}
		}
		
		if (unconstrainedLabels.length > 0 && (width - nodesWidth - labelsWidth) > 0) {
			var availableWidth = width - nodesWidth - constrainedLabelsWidth;
				
			while (unconstrainedLabels.length > 0 && availableWidth > 0) {
				var splitWidth = availableWidth / unconstrainedLabels.length;
				
				var retry = false;
				for (i = 0; i < unconstrainedLabels.length; ++i) {
					var newWidth = newWidths[unconstrainedLabels[i]];
					if (splitWidth < newWidth) {
						availableWidth -= newWidth;
						unconstrainedLabels.splice(i, 1);
						retry = true;
						break;
					}
				}
				if (!retry) {
					for (i = 0; i < unconstrainedLabels.length; ++i) {
						availableWidth -= splitWidth;
						newWidths[unconstrainedLabels[i]] = splitWidth;
					}
					break;
				}
			}
		}
		
		// set the new widths of each label
		for (i = 0; i < this._ellipsisLabels.length; ++i) {
			// ignore delay
			this._ellipsisLabels[i].forceResize({ w: newWidths[i] });
		}
		
		this._prevWidth = width;
	}
	
});

dojo.extend(rmps.ui.EllipsisLabel, {
	// priority: Number
	//		lower number means higher priority
	priority: 0
});
})();