/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.core.Enum");

/**
 * Implementation of an Enumeration.
 * 
 * Each literal of the enum must be of type rmps.core.EnumLiteral.
 * 
 * Example:
 * var myEnum = new rmps.core.Enum({
 *   ONE: new rmps.core.EnumLiteral({name: "One", value: 1}),
 *   TWO: new rmps.core.EnumLiteral({name: "Two, value: 2}),
 *   THREE: new rmps.core.EnumLiteral({name: "Three", value: 3})
 * });
 */
dojo.declare("rmps.core.Enum", null, {
	
	constructor: function(mixin) {
		if (mixin) {
			dojo.mixin(this, mixin);
		}
	},
	
	values: function() {
		var values = [];
		for (var key in this) {
			var property = this[key];
			// TODO find root cause for instanceof check failing in IE
			if (property && dojo.isObject(property) && property.declaredClass == "rmps.core.EnumLiteral") {
				values.push(property);
			}
		}
		return values;
	},
	
	get: function(value) {
		for (var key in this) {
			var property = this[key];
			if (property && dojo.isObject(property) && property.declaredClass == "rmps.core.EnumLiteral" && property.value == value) {
				return property;
			}
		}
		return null;
	}
});

dojo.declare("rmps.core.EnumLiteral", null, {
	
	// display name
	name: null,
	
	value: null,
	
	constructor: function(mixin) {
		if (mixin) {
			dojo.mixin(this, mixin);
		}
	}
});