/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.ErrorReport");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.i18n");
dojo.require("dojo.date.stamp");

dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.ui.MessageBox");

dojo.require("rmps.core.error");
dojo.require("rmps.ui.PagingControl");

dojo.requireLocalization("rmps.ui", "ErrorReportMessages");

var Severity = rmps.core.error.Severity;

dojo.declare("rmps.ui.ErrorReport", jazz.ui.MessageBox, {
	
	_messages: dojo.i18n.getLocalization("rmps.ui", "ErrorReportMessages"),
	
	"class": "rmps-ui-error-report",
	
	error: null,	/*Current error being displayed */
	
	// private properties
	_detailsPopups: null,
	_lessMore: null,
	
	constructor: function() {
		this._detailsPopups = [];
	},
	
	postCreate: function() {
		this.containerNode = this._tbody;
		this.subscribe(rmps.core.error.Channel, dojo.hitch(this, this._handleError));
		this.inherited(arguments);
	},
	
	destroy: function() {
		this._destroyPopups();
		if(this._lessMore){
			this._lessMore.destroyRecursive();
		}
		this.inherited(arguments);
	},
	
	_destroyPopups: function() {
		for (var i = 0; i < this._detailsPopups; ++i) {
			this._detailsPopups.destroyRecursive();
		}
		this._detailsPopups = [];
	},
	
	close: function() {
		while (this._tbody.firstChild) {
			dojo._destroyElement(this._tbody.firstChild);
		}
		this._destroyPopups();
		this._lessMore.domNode.style.display = "none";
		this.showLess();
		this.domNode.style.display = "none";
		dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
	},
		
	showLess: function() {
		if (!dojo.hasClass(this.messageArea, "showLess")) {
			dojo.addClass(this.messageArea, "showLess");
			this._lessMore.setLabel(this._messages.more);
			this.detailedContents.innerHTML = "";
			dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
		}
	},
	
	showMore: function() {
		if (dojo.hasClass(this.messageArea, "showLess")) {
			dojo.removeClass(this.messageArea, "showLess");
			this._lessMore.setLabel(this._messages.less);
			
			if(!this.error){return;}
			
			var cause= this.error.cause;
			if(!cause){return;}
			
			var message= cause.message;
			if(!message){return;}
			
			this.detailedContents.innerHTML = message;
			dojo.publish("jazz.util.ResizeNotifier.resizeEvent");	
		}
	},
		
	_handleError: function(error) {
		var messageContainer = dojo.doc.createElement("div");
		messageContainer.className = "message";
		var message = dojo.doc.createElement("span");
		message.appendChild(dojo.doc.createTextNode(error.message));
		messageContainer.appendChild(message);
		
		var nobr = dojo.doc.createElement("nobr");
		
		dojo.addClass(nobr, "dateNode");
		var date = dojo.date.locale.format(error.date, {formatLength: "medium"});
		nobr.appendChild(dojo.doc.createTextNode(date));
		
		message.appendChild(nobr);
		
		// show more / less button
		this._lessMore = new rmps.ui.ErrorReport._MoreLess({errorReport: this});
		this._lessMore.setLabel(this._messages.more);
		dojo.addClass(this._lessMore.domNode, "moreInfo");
		
		messageContainer.appendChild(this._lessMore.domNode);
		
		/*TODO: No Critical color is in jazz.ui.MessageBox.STATUS*/
		this.showMessage({
			type: error.severity&&error.severity!== rmps.core.error.Severity.CRITICAL ? error.severity : jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: messageContainer,
			closable: true
		});
		
		this._detailsPopups.push(this._createPopup(message, error));
		
		this.detailedContents = dojo.create("div");
		this.messageArea.appendChild(this.detailedContents);
		dojo.addClass(this.messageArea, "showLess");
		
		this.error= error;
	},
		
	_createPopup: function(node, error) {
		return new jazz.ui.HoverPopup({
			around: node,
			createContents: dojo.hitch(this, function() {
				var widget = new rmps.ui.ErrorReport._DetailsContainer({
					errorReport: this,
					"error": error
				});
				widget.startup();
				return widget.domNode;
			})
		});
	}
});

dojo.declare("rmps.ui.ErrorReport._MoreLess", [dijit._Widget], {
	
	errorReport: null,
	
	buildRendering: function() {
		this.domNode = dojo.create("a", {href: "#", "class": "lessMore"}, null);
	},
	
	postCreate: function() {
		this.connect(this.domNode, "onclick", function(e) {
			dojo.stopEvent(e);
			if (dojo.hasClass(this.errorReport.messageArea, "showLess")) {
				this.errorReport.showMore();
			} else {
				this.errorReport.showLess();
			}
		});
	},
	
	setLabel: function(label) {
		this.domNode.innerHTML = "";
		this.domNode.appendChild(dojo.doc.createTextNode(label));
	}
});

dojo.declare("rmps.ui.ErrorReport._DetailsContainer", [dijit._Widget], {
	
	"class": "rmps-ui-error-report-details",
	
	errorReport: null,
	error: null,
	
	_container: null,
	_errors: null,
	_errorWidgets: null,
	
	constructor: function() {
		this._errorWidgets = [];
		this._errors = [];
	},
	
	postCreate: function() {
		// collect all errors
		var parentError = this.error;
		while (parentError) {
			this._errors.push(parentError);
			parentError = parentError.cause;
		}
		
		if (this._errors.length > 1) {
			var currentIdx = 0;
			
			var navContainer = dojo.doc.createElement("div");
			navContainer.className = "navigation";
			var pagingControl = new rmps.ui.ErrorReport._PagingControl({
				detailsContainer: this,
				size: this._errors.length,
				item: 1
			});
			navContainer.appendChild(pagingControl.domNode);
			this.domNode.appendChild(navContainer);
			pagingControl.startup();
			
			this._container = dojo.doc.createElement("div");
			this.domNode.appendChild(this._container);
			
			this._showError(0);
			
		} else {
			var widget = new rmps.ui.ErrorReport._Details({
				errorReport: this.errorReport,
				"error": this.error
			});
			this.domNode.appendChild(widget.domNode);
			widget.startup();
		}
	},
	
	_showError: function(idx) {
		if (this._container.firstChild) {
			this._container.removeChild(this._container.firstChild);
		}
		
		var widget = this._errorWidgets[idx];
		if (widget) {
			this._container.appendChild(widget.domNode);
		} else {
			widget = new rmps.ui.ErrorReport._Details({
				errorReport: this.errorReport,
				"error": this._errors[idx]
			});
			this._errorWidgets[idx] = widget;
			this._container.appendChild(widget.domNode);
			widget.startup();
		}
	}
	
});

dojo.declare("rmps.ui.ErrorReport._PagingControl", rmps.ui.PagingControl, {
	
	detailsContainer: null,
	
	update: function() {
		this.inherited(arguments);
		this.detailsContainer._showError(this.item - 1);	
	},
	
	getNextItemLabel: function() {
		return dojo.string.substitute(
			this.detailsContainer.errorReport._messages.gotoCausedBy,
			[this.detailsContainer._errors[this.item].message]);
	},
	
	getPreviousItemLabel: function() {
		return dojo.string.substitute(
			this.detailsContainer.errorReport._messages.gotoCaused,
			[this.detailsContainer._errors[this.item - 2].message]);
	}
	
});

dojo.declare("rmps.ui.ErrorReport._Details", [dijit._Widget], {
	
	errorReport: null,
	error: null,
	_causedError: null,
	
	postCreate: function() {
		var table = dojo.doc.createElement("table");
		this.domNode.appendChild(table);
		var tbody = dojo.doc.createElement("tbody");
		table.appendChild(tbody);
		
		// date (required)
		this._appendRow(tbody, "date", this.errorReport._messages.date,
			dojo.date.locale.format(this.error.date, {formatLength: "medium"}));
		
		// status
		if (this.error.status && this.error.status !== "") {
			this._appendRow(tbody, "status", this.errorReport._messages.status, this.error.status);
		}
		
		// message (required)
		this._appendRow(tbody, "message", this.errorReport._messages.message, this.error.message);

		// sub status
		if (this.error.subStatus && this.error.subStatus !== "") {
			this._appendRow(tbody, "subStatus", this.errorReport._messages.subStatus, this.error.subStatus);
		}
		
		// sub severity
		if (this.error.severity && this.error.severity !== "") {
			this._appendRow(tbody, "severity", this.errorReport._messages.severity,
				dojo.hitch(this, function(parentNode) {
					var span = dojo.doc.createElement("span");
					var img = dojo.doc.createElement("img");
					img.src = dojo.moduleUrl("rmps.ui", "graphics/icons/" + this.error.severity.toLowerCase() + ".gif");
					img.alt = "";
					span.appendChild(img);
					parentNode.appendChild(span);
					parentNode.innerHTML += "&nbsp;";
					parentNode.appendChild(dojo.doc.createTextNode(this.errorReport._messages["severity_" + this.error.severity.toLowerCase()]));
			}));
		}
		
		// see also
		if (this.error.seeAlso && this.error.seeAlso !== "") {
			this._appendRow(tbody, "seeAlso", this.errorReport._messages.seeAlso,
				dojo.hitch(this, function(parentNode) {
					var link = dojo.doc.createElement("a");
					link.href = this.error.seeAlso;
					link.target = "_blank";
					parentNode.appendChild(link);
			}));
		}
		
		// detailed message
		if (this.error.detailedMessage && this.error.detailedMessage !== "") {
			this._appendRow(tbody, "detailedMessage", this.errorReport._messages.detailedMessage,
					dojo.hitch(this, function(parentNode) {
						if (this.error.detailedMessage) {
							if (dojo.isString(this.error.detailedMessage)) {
								var div = dojo.doc.createElement("div");
								div.appendChild(dojo.doc.createTextNode(this.error.detailedMessage));
								parentNode.innerHTML = div.innerHTML.replace(/\n/g, "<br>");
							} else {
								parentNode.appendChild(this.error.detailedMessage);
							}
						}
				}));
		}
		
		table = null;
		tbody = null;
	},
	
	_appendRow: function(tbody, id, label, value) {
		var tr = dojo.doc.createElement("tr");
		var labelTd = dojo.doc.createElement("td");
		var labelContainer = dojo.doc.createElement("div");
		labelContainer.className = "label";
		labelContainer.appendChild(dojo.doc.createTextNode(label));
		labelTd.appendChild(labelContainer);
		tr.appendChild(labelTd);
		
		var valueTd = dojo.doc.createElement("td");
		var valueContainer = dojo.doc.createElement("div");
		valueContainer.className = id;
		if (dojo.isFunction(value)) {
			value(valueContainer);
		} else {
			valueContainer.appendChild(dojo.doc.createTextNode(value));
		}
		valueTd.appendChild(valueContainer);
		tr.appendChild(valueTd);
		
		tbody.appendChild(tr);
	}
});

})();