/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.dsv.exporter.ExportStatusEditor");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("dijit._Widget");
dojo.require("jazz.util.date");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.ui.ResourceLink");
dojo.require("rmpw.dsv.exporter.EcoreExportStatus");
dojo.require("rmpw.dsv.exporter.EcoreExportClient");

dojo.require("dojo.date.locale");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmpw.dsv.exporter", "ExportStatusEditorMessages");
var messages = dojo.i18n.getLocalization("rmpw.dsv.exporter", "ExportStatusEditorMessages");

var EcoreExportClient = rmpw.dsv.exporter.EcoreExportClient;

dojo.declare("rmpw.dsv.exporter.ExportStatusEditor", rmps.app.ui.internal.editors.BaseEditor, {
	
	"class": "rmpw-domains-exporter-status",
	
	// init properties
	isReadOnly: true,
	
	// private properties
	_table: null,
	_deferred: null,
	_statusRows: null, //[]
	_doNotDelete: null,
	
	postCreate: function() {
		this.inherited(arguments);
		this.refresh();
	},
	
	constructor: function() {
		this.uaKey = "com.ibm.xtools.rmpw.dsv.exporter.page_ExportStatusEditor001";
	},
	
	refresh: function() {
		if (this._deferred) {
			return;
		}
		
		dojo.forEach(dijit.findWidgets(this.containerNode), function(widget) {
			if (widget.destroyRecursive) {
				widget.destroyRecursive();
			}
		});
		
		this.showLoading();
		this._deferred = EcoreExportClient.getStatuses();
		this._deferred.addBoth(this, function() {
			delete this._deferred;
			this.hideMessage();
		});
		
		this._deferred.addCallback(this, function(statuses) {
			function statusComparator(a, b) {
				return a.requestTime < b.requestTime ? 1 : a.requestTime == b.requestTime ? 0 : -1;
			}
			
			var table = new jazz.ui.StyledTable({
				columns: [
					{
						label: "",
						className: "state-icon"
					},
					{
						label: messages.state
					},
					{
						label: messages.exportedEcoreModels
					},
					{
						label: messages.requester
					},
					{
						label: messages.requestTime
					},
					{
						label: messages.startTime
					},
					{
						label: messages.duration
					},
					{
						label: messages.actions,
						className: "actions"
					}
				]	
			});
			table.containerNode = table.tbody;
			this.containerNode.appendChild(table.domNode);
			
			statuses.sort(statusComparator);
			
			// these properties are updated by the rows
			this._statusRows = [];
			delete this._doNotDelete
			
			for (var i = 0, s; s = statuses[i]; ++i) {
				var row = new rmpw.dsv.exporter.EcoreExportStatus._Row({editor: this, status: s});
				table.tbody.appendChild(row.domNode);
				row.startup();
			}
		});
		
		this._deferred.addErrback(this, function(error) {
			this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.error_loadingStatus,
					details:[{message: messages['error_loadingStatus.explanation'] + '<br>' + messages['error_loadingStatus.useraction']}],
					closable: true
				});
		});
	},
	
	_canDelete: function(status) {
		for (var i = 0, s; s = this._statusRows[i]; ++i) {
			if (s.status.state == rmpw.dsv.exporter.EcoreExportStatusStates.ERROR ||
				(s.status.state == rmpw.dsv.exporter.EcoreExportStatusStates.CANCELED && !s.status.startTime) ||
				s.status.state == rmpw.dsv.exporter.EcoreExportStatusStates.COMPLETED ||
				s.status.state == rmpw.dsv.exporter.EcoreExportStatusStates.WARNING) {
				return s.status !== status;
			}
		}
		return true;
	}
});

dojo.declare("rmpw.dsv.exporter.EcoreExportStatus._Row", dijit._Widget, {
	
	status: null,
	editor: null,
	
	_requesterDeferred: null,
	
	_duration: null,
	
	_stateLink: null,
	_stateImg: null,
	_startTimeTd: null,
	_durationTd: null,
	
	_exportedEcoreModelsLink: null,
	_exportedEcoreModelsLinkTd : null,
	
	_updateDeferred: null,
	_cancelExportDeferred: null,
	_deleteStatusDeferred: null,
	
	_popupContent: null,
	_popup: null,
	_actionBar: null,
	
	buildRendering: function() {
		this.domNode = this.containerNode = dojo.create("tr");
	},
	
	postCreate: function() {
	
		this.editor._statusRows.push(this);
		
		this._requesterDeferred = this.status.getRequester();
		
		// state img
		var td = dojo.create("td", null, this.domNode);
		this._stateImg = dojo.create("img", null, td);
		
		// state
		td = dojo.create("td", null, this.domNode);
		this._stateLink = dojo.create("a", { style: { cursor: "pointer" }, href: "#" }, td);
		
		this._updateState();
		
		this._popup = new jazz.ui.HoverPopup({
			around: this._stateLink,
			programmatic: false,
			onOpen: dojo.hitch(this, this._createPopupContents),
			maxWidth: 800,
			maxHeight: dojo.isIE ? 570 : 550
		});
		dojo.connect(this._stateLink, "onkeydown", this, function(e) {
			if(!e|| e.keyCode== dojo.keys.ESCAPE){
				this._popup.close();
				return;
			}
			if(dojo.keys.ENTER == e.keyCode|| dojo.keys.SPACE == e.keyCode){
				setTimeout(dojo.hitch(this, function() {
					this._popup.open();
				}), 0);			
			}
			if(e.keyCode!= dojo.keys.TAB){
				dojo.stopEvent(e);
			}
		});
		
		dojo.connect(this._stateLink, "onclick", this, function(e) {
			dojo.stopEvent(e);
		});
		
		// exported ecore models link
		this._exportedEcoreModelsLinkTd = dojo.create("td", null, this.domNode);
		if ( this.status.exporter_exportedEcoreModelsUri ) {
		    var link = this.status.exporter_exportedEcoreModelsUri;
		    var label = link.substring(link.lastIndexOf("/")+1);
		    this._exportedEcoreModelsLink = dojo.create("a", {href: link, innerHTML: label, title: messages.downloadNow}, this._exportedEcoreModelsLinkTd);
		}
		this._updateExportedEcoreModelsLink();
		
		// requester
		td = dojo.create("td", null, this.domNode);
		var requesterLink = new jazz.ui.ResourceLink({
				uri: this.status.requester,
				label: "-"
			});
		td.appendChild(requesterLink.domNode);
		
		// request time
		td = dojo.create("td", null, this.domNode);
		td.appendChild(dojo.doc.createTextNode(
				dojo.date.locale.format(this.status.requestTime, {formatLength: "medium"})
			));
		
		// start time
		this._startTimeTd = dojo.create("td", null, this.domNode);
		this._updateStartTime();
		
		// duration
		this._durationTd = dojo.create("td", null, this.domNode);
		this._updateDuration();
		
		// actions
		var actionsTd = dojo.create("td", null, this.domNode);
		this._actionBar = new jazz.ui.SimpleToolbar();
		actionsTd.appendChild(this._actionBar.domNode);
		this._actionBar.startup();
		this._updateActions();
		
		// start update 
		if (this.status.state == rmpw.dsv.exporter.EcoreExportStatusStates.PENDING ||
				this.status.state == rmpw.dsv.exporter.EcoreExportStatusStates.IN_PROGRESS ||
				this.status.state == rmpw.dsv.exporter.EcoreExportStatusStates.CANCELING) {
			this._updateTimer = setInterval(dojo.hitch(this, this._update), 3000); // 3 seconds
		}
	},
	
	_cancelExport: function() {
		this._actionBar.disableButton(this._cancelAction);
		this._cancelExportDeferred = EcoreExportClient.cancelExport(this.status.uri);
		this._cancelExportDeferred.addBoth(this, function() {
			delete this._cancelExportDeferred;
			this._update();
		});
	},
	
	_deleteStatus: function() {
		this._actionBar.disableButton(this._deleteAction);
		
		this._deleteStatusDeferred = EcoreExportClient.deleteStatus(this.status.uri);
		this._deleteStatusDeferred.addCallback(this, function() {
			delete this._deleteStatusDeferred;
			this.destroyRecursive();
		});
		this._deleteStatusDeferred.addErrback(this, function() {
			delete this._deleteStatusDeferred;
			this._update();
		});
	},
	
	_update: function() {
		if (this._updateDeferred) {
			return;
		}
		this._updateDeferred = EcoreExportClient.getStatus(this.status.uri);
		this._updateDeferred.addBoth(this, function(status) {
			delete this._updateDeferred;
			if (status instanceof Error) {
				if (status.status == 410) {
					this.destroyRecursive();
				}
				return;
			}
			
			if (!status.uri) {
				// protect against corrupt data
				return;
			}
			
			var oldStatus = this.status;
			this.status = status;
			
			// check if the update interval can be cleared
			if (this.status.state != rmpw.dsv.exporter.EcoreExportStatusStates.PENDING &&
					this.status.state != rmpw.dsv.exporter.EcoreExportStatusStates.IN_PROGRESS &&
					this.status.state != rmpw.dsv.exporter.EcoreExportStatusStates.CANCELING) {
				clearInterval(this._updateTimer);
				delete this._updateTimer;
			}
			
			if (this.status.state != oldStatus.state) {
				this._updateActions();
				this._updateState();
				this._updateExportedEcoreModelsLink();
				if (oldStatus.state == rmpw.dsv.exporter.EcoreExportStatusStates.PENDING) {
					this._updateStartTime();
				}
			}
			if (this.status.state != rmpw.dsv.exporter.EcoreExportStatusStates.PENDING) {
				this._updateDuration();
			}
		});
	},
	
	_updateActions: function() {
		if (!this._deleteAction && (this.status.state == rmpw.dsv.exporter.EcoreExportStatusStates.ERROR ||
				this.status.state == rmpw.dsv.exporter.EcoreExportStatusStates.CANCELED ||
				this.status.state == rmpw.dsv.exporter.EcoreExportStatusStates.COMPLETED ||
				this.status.state == rmpw.dsv.exporter.EcoreExportStatusStates.WARNING)) {
			if (this._cancelAction) {
				this._actionBar.removeItem(this._cancelAction);
				delete this._cancelAction;
			}
			
			// do not allow the first status affecting resource state to be deleted
			if (this.editor._canDelete()) {
				this._deleteAction = this._actionBar.addButton(this, this._deleteStatus, {
					label: messages.deleteStatus,
					imgSrc: dojo.moduleUrl("rmpw.dsv.exporter", "graphics/icons/remove.gif")
				});
			} else {
				if (this.editor._doNotDelete && this.editor._doNotDelete !== this) {
					var dnd = this.editor._doNotDelete;
					delete this.editor._doNotDelete;
					dnd._updateActions();
				}
				this.editor._doNotDelete = this;
			}			
		} else if (!this._cancelAction) {
			if (this._deleteAction) {
				this._actionBar.removeItem(this._deleteAction);
				delete this._deleteAction;
			}
			
			this._cancelAction = this._actionBar.addButton(this, this._cancelExport, {
					label: messages.cancelExport,
					imgSrc: dojo.moduleUrl("rmpw.dsv.exporter", "graphics/icons/cancel.gif")
				});
			
			if (this.status.state == rmpw.dsv.exporter.EcoreExportStatusStates.CANCELING) {
				this._actionBar.disableButton(this._cancelAction);
			}
		}
	},
	
	_updateState: function() {
		this._stateImg.src = this.status.state.icon;
		this._stateLink.innerHTML = "";
		this._stateLink.appendChild(dojo.doc.createTextNode(this.status.state.name));
	},
	
	_updateExportedEcoreModelsLink: function() {
		if ( this.status.exporter_exportedEcoreModelsUri && !this._exportedEcoreModelsLink) {
		    var link = this.status.exporter_exportedEcoreModelsUri;
		    var label = link.substring(link.lastIndexOf("/")+1);
		    this._exportedEcoreModelsLink = dojo.create("a", {href: link, innerHTML: label, title:messages.downloadNow}, this._exportedEcoreModelsLinkTd);
		}
	},
	
	_updateStartTime: function() {
		this._startTimeTd.innerHTML = "";
		if (this.status.startTime) {
			this._startTimeTd.appendChild(dojo.doc.createTextNode(
				dojo.date.locale.format(this.status.startTime, {formatLength: "medium"})
			));
		} else {
			this._startTimeTd.appendChild(dojo.doc.createTextNode("-"));
		}
	},
	
	_updateDuration: function() {
		this._durationTd.innerHTML = "";
		
		var timestamp = new Date();
		var endTime = this.status.stopTime || this.status.currentTime;
		
		if (this.status.startTime && endTime) {
			this._duration = jazz.util.date.formatDuration(endTime.getTime() - this.status.startTime.getTime());
			this._durationTd.appendChild(dojo.doc.createTextNode(this._duration));
		} else {
			this._durationTd.appendChild(dojo.doc.createTextNode("-"));
		}
	},
	
	_createPopupContents: function() {
		if (this._popupContent) {
			this._popupContent.destroyRecursive();
		}
		this._popupContent = new rmpw.dsv.exporter.EcoreExportStatus._PopupContent({row: this});
		this._popup.content = this._popupContent.domNode;
		
		if (this.status.state != rmpw.dsv.exporter.EcoreExportStatusStates.PENDING &&
				this.status.state != rmpw.dsv.exporter.EcoreExportStatusStates.IN_PROGRESS &&
				this.status.state != rmpw.dsv.exporter.EcoreExportStatusStates.CANCELING) {
			delete this._popup.onOpen;
		}
	}
});

dojo.declare("rmpw.dsv.exporter.EcoreExportStatus._PopupContent", dijit._Widget, {
	
	"class": "rmpw-domains-exporter-status-popup",
	
	row: null,
	
	postCreate: function() {
		this._createExporterSection();
		
	},
	
	_createNode: function(tag, refNode, value) {
		var node = dojo.create(tag, null, refNode);
		if (dojo.isFunction(value)) {
			value(node);
		} else {
			node.appendChild(dojo.doc.createTextNode(value));
		}
		return node;
	},
	
	_createCanceledSection: function() {
		var status = this.row.status;
		
		if (!status.canceller) {
			return;
		}
		
		var table = dojo.create("table", null, this.domNode);
		var tbody = dojo.create("tbody", null, table);
		
		tr = dojo.create("tr", null, tbody);
		this._createNode("td", tr, messages.label_canceller);
		this._createNode("td", tr, dojo.hitch(this, function(parentNode) {
			status.getCanceller().addCallback(this, function(user) {
				parentNode.appendChild(dojo.doc.createTextNode(user.name));
			});
		}));
	},
	
	_createExporterSection: function() {
		var status = this.row.status;
		
		var table = dojo.create("table", null, this.domNode);
		var tbody = dojo.create("tbody", null, table);
		
		var tr = dojo.create("tr", null, tbody);
		dojo.create("th", {innerHTML:"<strong>"+messages.details+"</strong>"}, tr);
		
		if (status.canceller) {
			var table = dojo.create("table", null, this.domNode);
			var tbody = dojo.create("tbody", null, table);
			
			tr = dojo.create("tr", null, tbody);
			this._createNode("td", tr, messages.label_canceller);
			this._createNode("td", tr, dojo.hitch(this, function(parentNode) {
				status.getCanceller().addCallback(this, function(user) {
					parentNode.appendChild(dojo.doc.createTextNode(user.name));
				});
			}));
		}
	
		table = dojo.create("table");
		tbody = dojo.create("tbody", null, table);
			
		
		if (status.exporter_ontologyNames) {
			this._createListRow(tbody, 'Ontologies to Export:', status.exporter_ontologyNames);
		}
		
		if (status.exporter_danglingReferences) {
			this._createListRow(tbody, messages.label_danglingReferences, status.exporter_danglingReferences);
		}
		
		if (status.exporter_warnings) {
			this._createListRow(tbody, messages.label_warnings, status.exporter_warnings);
		}
		
		if (status.exception) {
			this._createListRow(tbody,  messages.label_exception, [status.exception]);
		}
						
		if (tbody.firstChild) {
			this.domNode.appendChild(table);
		}
	},
	
	_createListRow: function(tbody, label, values) {
		var tr = dojo.create("tr", null, tbody);
		var valueTd = dojo.create("td", null, tr);
		var dl = dojo.create("dl", null, valueTd);
		var dt = dojo.create("dt", {innerHTML:"<h4>"+label+"</h4>"}, dl);
		for (var i = 0; i < values.length; ++i) {
			this._createNode("dd", dl, "-  " + values[i]);
		}
	}
});

})();
