/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.layout.graph.GraphConfigurator");

dojo.require("rmpw.diagrams.render.graph.BaseGraphConfigurator");
dojo.require("rmpw.diagrams.layout.client.figures.MoveablePolygonAnchor");
dojo.require("rmpw.diagrams.layout.client.figures.FixedPositionAnchor");
dojo.require("rmpw.diagrams.layout.client.figures.ConnectionFigure");
dojo.require("rmpw.diagrams.layout.client.figures.ContainerFigure");
dojo.require("rmpw.diagrams.layout.client.figures.Figure");

/**
 * A utility class for configuring a graph
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.layout.graph.GraphConfigurator", rmpw.diagrams.render.graph.BaseGraphConfigurator, {
	
	runGraphLayoutAlgorithm: function(graph, layoutCallBack, args) {
		var nodeLayout = null;
		if (args.type == "tree") {
			nodeLayout = this.createTreeLayout(args);
		}
		else if (args.type == "hierarchical" ||
				 args.type == "graph") { // default 
			nodeLayout = this.createHierarchicalLayout(args);
		}
		else if (args.type == "radial") {
			nodeLayout = this.createRadialLayout(args);
		}
		
		if (nodeLayout) {
			if (nodeLayout.runLayoutAlgorithm(graph)) {
				return layoutCallBack(graph, args);
			}
		}
		
		return false;
	},
	
	createGraph: function(mapping) {
		return new rmpw.diagrams.layout.client.figures.ContainerFigure(mapping);
	},
	
	createGraphNode: function(mapping, graph, figRect) {
		return new rmpw.diagrams.layout.client.figures.Figure(mapping, graph, figRect);
	},
	
	createGraphEdge: function(mapping, graph, sourceNode, targetNode) {
		return new rmpw.diagrams.layout.client.figures.ConnectionFigure(mapping, graph, sourceNode, targetNode);
	},
	
	/*
	 * Create an anchor owned by graphElement of a particular type - one of FIXED_ANCHOR,
	 * MOVEABLE_ANCHOR
	 */
	createAnchor: function(anchorType, graphElement, args) {
		if (anchorType == this.FIXED_ANCHOR) {
			var fixedAnchor = new rmpw.diagrams.layout.client.figures.FixedPositionAnchor(graphElement);
			fixedAnchor.addFixedLocation(args.side, args.percentage, args.inset);
			return fixedAnchor;
		}
		else if (anchorType == this.MOVEABLE_ANCHOR) {
			return new rmpw.diagrams.layout.client.figures.MoveablePolygonAnchor(graphElement);
		}
	},
	
	/**
	 * Creates a tree layout based on the current form settings
	 */
	createTreeLayout: function(args) {
		dojo.require("rmpw.diagrams.layout.graph.TreeGraphLayout");
		return new rmpw.diagrams.layout.graph.TreeGraphLayout();
	},
	
	/**
	 * Creates a radial layout based on the current form settings
	 */
	createRadialLayout: function(args) {
		dojo.require("rmpw.diagrams.layout.graph.RadialGraphLayout");
		return new rmpw.diagrams.layout.graph.RadialGraphLayout();
	},

	/**
	 * Creates a hierarchical layout based on the current form settings
	 */
	createHierarchicalLayout: function(args) {
		dojo.require("rmpw.diagrams.layout.graph.GMFDirectedGraphLayout");
		return new rmpw.diagrams.layout.graph.GMFDirectedGraphLayout();
	}
});