/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.panner.HTMLContentOutline");

dojo.require("rmpw.panner.Outline");

dojo.declare("rmpw.panner.HTMLContentOutline", rmpw.panner.Outline, {
	
	referenceNode: null,
	
	// private properties
	_isUpdating: false,
	_pendingUpdate: null,
	_lastScale: 0,
	_scalingTimer: null,
	
	update: function() {
		this._updateContent();
	},
	
	setScale: function(scale) {
		var oldScale = this.getScale();
		if (oldScale != scale) {
			this.inherited(arguments);
			if (this._scalingTimer) {
				clearTimeout(this._scalingTimer);
				delete this._scalingTimer;
			}
			this._scalingTimer = setTimeout(dojo.hitch(this, function() {
				delete this._scalingTimer;
				if (oldScale > scale) {
					this._updateSize();
				} else {
					this._updateContent();
				}
			}), 250);
		}
	},
	
	_processPendingUpdate: function() {
		this._isUpdating = false;
		if (this._pendingUpdate) {
			this._pendingUpdate = false;
			this._updateContent();
		}
	},
	
	ratioChanged: function() {
		this._updateContent();
	},
	
	_updateSize: function() {
		var ratio = this.getRatio();
		if (ratio > 0) {
			var oldContent = this.panContainerNode.firstChild;
			if (oldContent) {
				if (!this._isUpdating) {
					this._isUpdating = true;
					
					var newScale = ratio * this.getScale();
					var scale = newScale / this._lastScale;
					this._lastScale = newScale;
					
					this._resizeContent(oldContent, scale, this._fixNodeSize, this._resizeNode);
					setTimeout(dojo.hitch(this, this._processPendingUpdate), 0);
				} else {
					this._pendingUpdate = true;
				}
			}
		}
	},
	
	_updateContent: function() {
		if (this._scalingTimer) {
			clearTimeout(this._scalingTimer);
			delete _scalingTimer;
		}
		if (this._isUpdating) {
			this._pendingUpdate = true;
			return;
		}
		this._isUpdating = true;
		
		var ratio = this.getRatio();
		this._lastScale = ratio * this.getScale();
		
		var oldContent = this.panContainerNode.firstChild;
			var contentContainer = dojo.doc.createElement("div");
			if (!dojo.isIE) {
				contentContainer.style.visibility = "hidden";
				contentContainer.style.fontSize = Math.floor(100 * ratio) + "%";
				contentContainer.style.lineHeight = Math.floor(100 * ratio) + "%";
			}
			contentContainer.innerHTML = this.referenceNode.innerHTML;
			
			// IE's zoom feature degrades borders such that they may not be visible.
			// Although accurate, a border is very useful for finding the bounding box of 
			// a node. The manual scaling process holds borders as high priority. 
//			if (dojo.isIE) {
//				this.panContainerNode.style.zoom = scale;
//				this.panContainerNode.appendChild(contentContainer);
//				
//			} else {
				// find all images
				var imgCount = 0;
				this._resizeContent(contentContainer, ratio, this._purgeNodes, dojo.hitch(this, function(node, scale) {
					if (node instanceof Image && node.src && node.src.length > 0) {
						imgCount++;
						node.onload = dojo.hitch(this, function() {
							if (--imgCount == 0) {
								this._resizeContent(contentContainer, ratio, this._fixNodeSize, this._resizeNode);
								if (oldContent) {
									try {
									this.panContainerNode.removeChild(oldContent);
									} catch (e) {
										throw (e);
									}
								}
								contentContainer.style.visibility = "visible";
								contentContainer = null;
								oldContent = null;
								setTimeout(dojo.hitch(this, this._processPendingUpdate), 0);
							}
						});
					}
				}));
				
				this.panContainerNode.appendChild(contentContainer);
				if (imgCount == 0) {
					this._resizeContent(contentContainer, ratio, this._fixNodeSize, this._resizeNode);
					if (oldContent) {
						this.panContainerNode.removeChild(oldContent);
					}
					contentContainer.style.visibility = "visible";
					setTimeout(dojo.hitch(this, this._processPendingUpdate), 0);
				}
//			}
	},
	
	_purgeNodes: function(node, scale) {
		if (node.style.display == "none" ||
				node.tagName.toLowerCase() == "script" ||
				node.tagName.toLowerCase() == "noscript") {
			node.parentNode.removeChild(node);
			return false;
		}
		
		return true;
	},
	
	_resizeNode: function(node, scale) {
		if (node.nodeType != 1) {
			return;
		}
		node.removeAttribute("id");
		node.removeAttribute("widgetId");
		node.removeAttribute("dojoAttachEvent");
		node.removeAttribute("dojoType");
		node.removeAttribute("dojoAttachPointR");
		node.removeAttribute("title");
		
		function resizeStyle(style, min, size, unit) {
			if (size != null) {
				node.style[style] = size + (unit || "px");
			} else {
				var value = cs[style];
				if (value && !value.match(/^\D+$/)) {
					var unit = value.replace(/\-|\d|\./g, '');
					if (unit != "%") {
						value = parseFloat(value);
						if (isNaN(value)) {
							value = 0;
						} else {
							value = value * scale;
						}
						if (value < min) {
							value = min;
						}
						node.style[style] = Math.floor(value) + unit;
						return value;
					}
				}
			}
		}
		
		// TODO add support for width / height attributes
		// TODO svg does not have offsetWidth / offsetHeight
		
		// this is slow
		var cs = dojo.getComputedStyle(node);
		
		var width = Math.floor(node.offsetWidth * scale);
		var height = Math.floor(node.offsetHeight * scale);
		
		// borders
		if (!(!node.style.borderStyle || node.style.borderStyle == "none")) {
			width -= resizeStyle("borderLeftWidth", 1) || 0;
			if (width > 0) {
				width -= resizeStyle("borderRightWidth", 1) || 0;
			} else {
				resizeStyle("borderRightWidth", null, 0);
			}
			height -= resizeStyle("borderTopWidth", 1) || 0;
			if (height > 0) {
				height -= resizeStyle("borderBottomWidth", 1) || 0;
			} else {
				resizeStyle("borderBottomWidth", null, 0);
			}
		}
		
		// width / height
		resizeStyle("width", null, width);
		resizeStyle("height", null, height);
		resizeStyle("minWidth");
		resizeStyle("minHeight");
		
		// margins
		resizeStyle("marginLeft");
		resizeStyle("marginRight");
		resizeStyle("marginTop");
		resizeStyle("marginBottom");
		
		resizeStyle("fontSize");
		resizeStyle("lineHeight");
		
		if (node.offsetWidth != 0 && node.offsetHeight != 0) {
			// padding
			resizeStyle("paddingLeft");
			resizeStyle("paddingRight");
			resizeStyle("paddingTop");
			resizeStyle("paddingBottom");
		}
		
		// location
		resizeStyle("left");
		resizeStyle("right");
		resizeStyle("top");
		resizeStyle("bottom");
	},
	
	_resizeContent: function(node, scale, downTraversalCallback, upTraversalCallback) {
		// create an array copy of child nodes
		var childNodes = [];
		var len = node.childNodes.length;
		for (var i = 0; i < len; ++i) {
			var child = node.childNodes[i];
			if (child.nodeType != 3) {
				childNodes.push(child);
			}
		}
		
		len = childNodes.length;
		for (var i = 0; i < len; ++i) {
			var child = childNodes[i];
			switch (child.nodeType) {
				case 8:
					node.removeChild(child);
				case 3:
				case 4:
				case 7:
					break;
				default:
					if (!downTraversalCallback || downTraversalCallback(child, scale)) {
						this._resizeContent(child, scale, downTraversalCallback, upTraversalCallback);
					}
			}
		}
		if (upTraversalCallback) {
			upTraversalCallback(node, scale);
		}
	}
});

})();