/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.layout.ilog.ILogBootstrapGraph");

dojo.require("rmpw.diagrams.render.graph.AbstractGraph");
dojo.require('ibm_ilog.diagram.Graph');

dojo.require("rmpw.diagrams.layout.ilog.ILogBootstrapNode");
dojo.require("rmpw.diagrams.layout.ilog.ILogBootstrapLink");

/*
 * Bootstrap for integrating with iLog graph Graph 
 * 
 * @author sshaw
 */
dojo.declare("rmpw.diagrams.layout.ilog.ILogBootstrapGraph", [rmpw.diagrams.render.graph.AbstractGraph, ibm_ilog.diagram.Graph], {
	
	/*
	 * Pre-hook to allow layout to process invalid edges before
	 * calling layout algorithm
	 */
	_preRouteInvalidEdges: function() {
		if (this._invalidEdges) {
			for (var id in this._invalidEdges) {
				var edge = this._invalidEdges[id];
				edge.route();
			}
		}
	},
	
	fireNodeLayoutInvoked: function() {
		this._nodeLayoutInvoked = true;
	},
	
	hasNodeLayoutBeenInvoked: function() {
		return this._nodeLayoutInvoked !== undefined;
	},
	
	resetNodeLayoutInvoked: function() {
		if (this._nodeLayoutInvoked) {
			delete this._nodeLayoutInvoked;
		}
	},
	
	/*
	 * Hook for re-adjusting the edge routing after the node positions have been moved
	 */
	_doRouteInvalidEdges: function() {
		// only do edge routing if node layout hasn't just been previously invoked since
		// node layout algorithms include edge routing
		if (this._invalidEdges && !this.hasNodeLayoutBeenInvoked()) {
			var nodeLayoutActive = this.isNodeLayoutActive();
			this.setNodeLayoutActive(false);
			try {
				this.performGraphLayout();
			}
			catch (e) {
				throw e;
			}
			finally {
				this.setNodeLayoutActive(nodeLayoutActive);
			}
		}
		
		this.resetNodeLayoutInvoked();
	},
	
	toRawData: function () {
		var children = this.children;
		var jsonNodes = [];
		var jsonEdges = [];
		var idMap = {};
		
		for (var i = 0; child = children[i]; i++) {
			if (child instanceof rmpw.diagrams.layout.ilog.ILogBootstrapNode) {
				var id = child.mapping.id ? child.mapping.id : child.mapping;
				var bounds = child.getBounds();
				jsonNodes.push( { id: id, /* bounds */ b: { x: Math.round(bounds.x), y: Math.round(bounds.y), w: Math.round(bounds.width), h: Math.round(bounds.height)} } );
				idMap[id] = {mapping: child.mapping, nativeNode: child};
			}
		}
		
		for (var i = 0; child = children[i]; i++) {
			if (child instanceof rmpw.diagrams.layout.ilog.ILogBootstrapLink && child.getStartPort() && child.getEndPort()) {
				var start = child.getStartPort().getOwner();
				var end = child.getEndPort().getOwner();
				var id = child.mapping.id ? child.mapping.id : child.mapping;
				
				var modelPoints = [];
				var pts = child.getPointList().points;
				for (var j=0, pt; pt = pts[j]; j++) {
					modelPoints.push({x: Math.round(pt.x), y: Math.round(pt.y)});
				}
				
				jsonEdges.push( { id: id,  
						/* source ID */ sId: start.mapping.id ? start.mapping.id : start.mapping, 
						/* target ID */ tId: end.mapping.id ? end.mapping.id : end.mapping,
						pts: modelPoints});
				idMap[id] = {mapping: child.mapping, nativeNode: child};
			}
		}
		
		return { jsonGraph: {nodes: jsonNodes, edges: jsonEdges}, idMap: idMap};
	}
});