/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.layout.ilog.ILogBootstrapLink");

dojo.require("rmpw.diagrams.render.graph.BaseGraphConfigurator");
dojo.require("rmpw.diagrams.render.graph.AbstractEdge");
dojo.require("ibm_ilog.diagram.Link");
dojo.require("ibm_ilog.diagram.Port");
dojo.require("ibm_ilog.graphlayout.internalutil.LinkStyles");
dojo.require('rmpw.diagrams.layout.ilog.ILogBootstrapPort');

/*
 * Bootstrap for integrating with iLog graph Link 
 * 
 * @author sshaw
 */
dojo.declare("rmpw.diagrams.layout.ilog.ILogBootstrapLink", [ibm_ilog.diagram.Link, rmpw.diagrams.render.graph.AbstractEdge], {
	
	constructor: function(mapping, graph) {
		this._routerConstraint = null;
		this.avoidObstructions = true; // turn on by default
	},
	
	setAvoidObstructions: function(avoid) {
		this.avoidObstructions = avoid;
	},
	
	isAvoidingObstructions: function() {
		return this.avoidObstructions;
	},
	
	applyPath: function(target) {
		// do nothing - stub out dojo calls
	},
	
	setTransform: function(matrix) {
		// do nothing - stub out dojo calls
	},
	
	destroy: function() {
		this.inherited(arguments);
		
		var graph = this.getGraph();
		graph.children.splice(dojo.indexOf(graph.children, this), 1);
		this.dispose();
		
		var oldStartPort = this.getStartPort();
		if (oldStartPort) {
			if (oldStartPort.__connectRelativeReference) {
				dojo.disconnect(oldStartPort.__connectRelativeReference);
			}
			oldStartPort.destroy();
		}
		
		var oldEndPort = this.getEndPort();
		if (oldEndPort) {
			if (oldEndPort.__connectRelativeReference) {
				dojo.disconnect(oldEndPort.__connectRelativeReference);
			}
			oldEndPort.destroy();
		}
	},
	
	invalidate: function() {
		this.invalidateLinkShape();
	},
	
	invalidateLinkShape: function() {
		// this is an implementation variable - could be brittle to new
		// versions of ILog.  Necessary to override this method since 
		// we need to control the invalidate / validate process to integrate
        this._pathPointsValid = false;
        this.getGraph()._addInvalidEdge(this);
   },
	
   _createProxyPort: function(anchor) {
	   // create a volatile anchor for a feedback connection
		var ilogPort = new rmpw.diagrams.layout.ilog.ILogBootstrapPort(anchor.getOwner());
		ilogPort.__feedbackAnchor = anchor;
		ilogPort.__connectRelativeReference = dojo.connect(anchor, "setRelativeReferencePoint", dojo.hitch(ilogPort, function(pt) {
			this.setPosition( pt == null ? { x: 0.5, y: 0.5 } : { x: pt.x, y: pt.y });
		}));  
		ilogPort.getOwner = dojo.hitch(ilogPort, function() { 
			var owner = this.__feedbackAnchor.getOwner();
			this._owner = owner; // TODO brittle to iLog changes
			return owner;
		});
		
		return ilogPort;
   },
   
	setSourceAnchor: function(startAnchor) {
		var oldStartPort = this.getStartPort();
		if (oldStartPort) {
			oldStartPort.destroy();
		}
		
		if (startAnchor instanceof rmpw.diagrams.layout.ilog.ILogBootstrapPort) {
			this.setStartPort(startAnchor);
		}
		else {
			// create a volatile anchor for a feedback connection
			var startPort = this._createProxyPort(startAnchor);
			this.setStartPort(startPort);
		}
	},
	
	setTargetAnchor: function(targetAnchor) {
		var oldEndPort = this.getEndPort();
		if (oldEndPort) {
			oldEndPort.destroy();
		}
		
		if (targetAnchor instanceof rmpw.diagrams.layout.ilog.ILogBootstrapPort) {
			this.setEndPort(targetAnchor);
		}
		else {
			// create a volatile anchor for a feedback connection
			var endPort = this._createProxyPort(targetAnchor);
			this.setEndPort(endPort);
		}
	},
	
	getSourceAnchor: function () {
		if (this.getStartPort() != null) {
			if (this.getStartPort().__feedbackAnchor) {
				return this.getStartPort().__feedbackAnchor;
			}
			return this.getStartPort();
		}
		
		return null;
	},
	
	getTargetAnchor: function (){
		if (this.getEndPort() != null) {
			if (this.getEndPort().__feedbackAnchor) {
				return this.getEndPort().__feedbackAnchor;
			}
			return this.getEndPort();
		}
		
		return null;
	},
	
	/*
	 * Returns whether this link is orthogonally routed
	 */
	isOrthogonal: function() {
		return this.getShapeType() == ibm_ilog.diagram.LinkShapeType.OrthogonalEditable;
	},
	
	/*
	 * Returns the type of router for this connection figure
	 */
	getRouterType: function() {
		if (this.getShapeType() == ibm_ilog.diagram.LinkShapeType.OrthogonalEditable) {
			return rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.RECTILINEAR_ROUTING;
		}
		else {
			return rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.OBLIQUE_ROUTING;
		}
	},
	
	setRouterType: function(routerType) {
		var shapeType, linkStyle;
		
		if (routerType == rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.RECTILINEAR_ROUTING ||
			routerType == rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.TREE_ROUTING) {
			shapeType = ibm_ilog.diagram.LinkShapeType.OrthogonalEditable;
			linkStyle = ibm_ilog.graphlayout.internalutil.LinkStyles.ORTHOGONAL_STYLE;
		}
		else {
			shapeType = ibm_ilog.diagram.LinkShapeType.Free;
			linkStyle = ibm_ilog.graphlayout.internalutil.LinkStyles.DIRECT_STYLE;
		}

		this.setShapeType(shapeType);
		var linkLayout = this.getGraph().getLinkLayout();
		if (linkLayout && linkLayout.setLinkStyle && !this._feedback) {
			linkLayout.setLinkStyle(this, linkStyle);
		}
	},
	
	setFeedback: function(feedback) {
		this._feedback = feedback;
	},
	
	isFeedback: function() {
		return this._feedback;
	},
	
	setPointList: function (points){
		var startPoint, endPoint;
		
		if (points == null || points.length < 2) {
			return;
		}
		
		if (points.size > 2) {
			var ipts = [];
			for (var i=1; i<points.size - 1; i++) {
				ipts.push(points.getPoint(i));
			}
			this.setIntermediatePoints(ipts);
		}
		else {
			this.setIntermediatePoints(null);
		}
	},
	
	getPointList: function (){
		var points = this.getPoints();
		
		var fullPoints = new rmpw.diagrams.render.util.PointList();
		for (var i=0, point; point=points[i]; i++) {
			fullPoints.addPoint_0(point.x, point.y);
		}
		
		return fullPoints;
	},
	
	_getPointsFromConstraint: function() {
		var points = new rmpw.diagrams.render.util.PointList();
		
		if (this._routerConstraint) {
			var a1 = this.getStartPort().getReferencePoint();
			this.getStartPort().getOwner().translateToRelative(a1);
			var a2 = this.getEndPort().getReferencePoint();
			this.getEndPort().getOwner().translateToRelative(a2);
			
			var numPoints = this._routerConstraint.length;
			for (var i=0; i<numPoints; i++) {
				var weight = i / (numPoints - 1);
					
				var src = {x: this._routerConstraint[i].sourceX, y: this._routerConstraint[i].sourceY};
				var trg = {x: this._routerConstraint[i].targetX, y: this._routerConstraint[i].targetY};
				points.addPoint_0( (a1.x + src.x) * (1.0 - weight) + weight * (a2.x + trg.x), 
						(a1.y + src.y) * (1.0 - weight) + weight * (a2.y + trg.y) );
			}
		}
		
		return points;
	},
	
	/*
	 * Returns true if source anchor is fixed or false otherwise
	 */
	isFixedSourceAnchor: function() {
		var linkLayout = this.getGraph().getLinkLayout();
		return linkLayout.getOriginPointMode(this) == linkLayout.__linkLayoutType.FIXED_MODE;
	},
	
	/*
	 * Returns true if target anchor is fixed or false otherwise
	 */
	isFixedTargetAnchor: function() {
		var linkLayout = this.getGraph().getLinkLayout();
		return linkLayout.getDestinationPointMode(this) == linkLayout.__linkLayoutType.FIXED_MODE;
	},
	
	/*
	 * @param fix will ensure that the source anchor position is
	 * fixed and won't change during incremental layout
	 */
	setFixSourceAnchor: function(fix) {
		var linkLayout = this.getGraph().getLinkLayout();
		linkLayout.setOriginPointMode(this, fix ? 
				linkLayout.__linkLayoutType.FIXED_MODE : linkLayout.__linkLayoutType.FREE_MODE);
	},
	
	/*
	 * @param fix will ensure that the target anchor position is
	 * fixed and won't change during incremental layout
	 */
	setFixTargetAnchor: function(fix) {
		var linkLayout = this.getGraph().getLinkLayout();
		linkLayout.setDestinationPointMode(this, fix ? 
				linkLayout.__linkLayoutType.FIXED_MODE : linkLayout.__linkLayoutType.FREE_MODE);
	},
	
	/*
	 * @param fix will ensure that the bendpoints position is
	 * fixed and won't change during incremental layout
	 */
	setFixBendpoints: function(fix) {
		this.getGraph().getLinkLayout().setFixed(this, fix);
	},
	
	/*
	 * Sets router constraint for this edge based on a set of bendpoints
	 * If relativeToEnds is true, then the bendpoints are stored as 2 tuples that are relative
	 * to the end point locations.
	 */
	setRouterConstraint: function(bendpoints, relativeToEnds) {
		if (relativeToEnds) {
			if (bendpoints.length > 2) {
				this._routerConstraint = bendpoints;
				// any link with a constraint should be set as fixed
				this.setFixBendpoints(!this.getGraph().hasNodeLayoutBeenInvoked());
			}
			else {
				this.setFixBendpoints(false);
			}
			this.setIntermediatePoints(null);
		}
		else {
			var rawPts = bendpoints.points;
			this.setFallbackStartPoint(rawPts[0]);
			this.setFallbackEndPoint(rawPts[rawPts.length - 1]);
			
			this.setPointList(bendpoints);
		}

		this.invalidateLinkShape();
	},
	
	/*
	 * Routes this edge based on it's constraint bendpoints
	 */
	route: function() {
		if (this._routerConstraint) {
			this.setPointList(this._getPointsFromConstraint());
		}
	}
});