/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.layout.ilog.ILogBootstrapNode");

dojo.require("rmpw.diagrams.render.graph.AbstractNode");
dojo.require('ibm_ilog.diagram.Node');
dojo.require("rmpw.diagrams.render.util.Rectangle");

/*
 * Bootstrap for integrating with iLog graph Node 
 * 
 * @author sshaw
 */
dojo.declare("rmpw.diagrams.layout.ilog.ILogBootstrapNode", [ibm_ilog.diagram.Node, rmpw.diagrams.render.graph.AbstractNode], {
	
	constructor: function(mapping, graph, x, y, width, height) {
		this.setBounds({x:x, y:y, width:width, height:height});
		if (mapping.policies) {
			this._connectUpdateShape = dojo.connect(mapping.policies.graphElement, "updateShape", dojo.hitch(this, function() {
				this._dojoValid = true;
			}));
		}
	},
	
	destroy: function() {
		dojo.disconnect(this._connectUpdateShape);
		var graph = this.getGraph();
		graph.children.splice(dojo.indexOf(graph.children, this), 1);
		this.dispose();
	},
	
	/*
	 * Returns true if the dojo shape data is in sync with the graph data, or false otherwise
	 */
	isDojoValid: function() {
		return this._dojoValid;
	},

	setLayout: function(layout) {
		// do nothing - stub out dojo calls
	},
	
	setBounds: function (rect){
		this._bounds = new rmpw.diagrams.render.util.Rectangle(rect.x, rect.y, rect.width, rect.height);
		this._dojoValid = false;
	},
	
	getBounds: function(/*dojox.gfx.Group*/parent) {
		return this._bounds.getCopy();
    },
    
    // from dojo shape
    getBoundingBox: function() {
    	var insets = this.getAnchorInsets();
		return new rmpw.diagrams.render.util.Rectangle(this._bounds.x + insets.left, 
				this._bounds.y + insets.top, 
				this._bounds.width - insets.left - insets.right, 
				this._bounds.height - insets.top - insets.bottom);
    },
    
    move: function(/*number*/x, /*number*/ y, /*dojox.gfx.Group*/ parent) {
    	this._bounds.x = x;
        this._bounds.y = y;
        this._dojoValid = false;
    }
});