/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.layout.ilog.ILogBootstrapPort");

dojo.require("rmpw.diagrams.render.graph.AbstractEdgeAnchor");
dojo.require("ibm_ilog.diagram.Port");
dojo.require("rmpw.diagrams.render.util.Point");

/*
 * Bootstrap for integrating with iLog graph Port 
 * 
 * @author sshaw
 */
dojo.declare("rmpw.diagrams.layout.ilog.ILogBootstrapPort", [rmpw.diagrams.render.graph.AbstractEdgeAnchor, ibm_ilog.diagram.BasicPort], {
	constructor: function(owner) {
		if (owner) {
			owner.addPort(this);
		}
	},
	
	destroy: function() {
		if (this.getOwner()) {
			this.getOwner().removePort(this);
		}
	},	
	
	getClipOnShape: function(){
		return true;
	}, 
	
	getBaseShape : function() {
		var owner = this.getOwner();
		if (owner && owner.mapping.policies && owner.mapping.policies.anchorable) {
			var anchorShapes = owner.mapping.policies.anchorable.getAnchorShapes();
			var baseShape = null;
			
			if (anchorShapes) {
				var anchorShape = anchorShapes[0];
				if (anchorShape && anchorShape instanceof dojox.gfx.shape.Polyline) {
					baseShape = new dojox.gfx.shape.Polyline();
					var pts = owner.mapping.policies.graphElement.getPolygonPoints();
					baseShape.shape = { points: pts.points }; // fake shape
				} 
				
				if (baseShape) {
					baseShape.getShapeToContainerTransform = function() { return dojox.gfx.matrix.identity;};
					baseShape.getBoundingBox = function() { return owner.getBoundingBox(); };
					
					return baseShape;
				}
			}
		}
		
		return this.inherited(arguments);
	},
	
	computeConnectionPoint: function(link, parent, referencePoint, connectionPoint, originPoint, start){
		connectionPoint = this.inherited(arguments);
		
		if (this.isDefaultAnchor()) {
			connectionPoint.x = referencePoint.x;
			connectionPoint.y = referencePoint.y;
		}
		
		return connectionPoint;
	},
	
	needsReferencePoint: function() {
		return true;
	}, 
	
	getRelativeReferencePoint: function() {
		var pos = this.getPosition();
		return new rmpw.diagrams.render.util.Point(pos.x, pos.y);
	},
	
	setRelativeReferencePoint: function(relativeReference) {
		if (relativeReference != null) {
			this.setPosition({x: relativeReference.x, y:relativeReference.y});
		}
		else {
			this.setPosition({x: 0.5, y:0.5});
		}
	},
	
	getCopy: function() {
		var anchor = new rmpw.diagrams.layout.ilog.ILogBootstrapPort(this.getOwner());
		var relativeReference = this.getRelativeReferencePoint();
		if (relativeReference) {
			anchor.setRelativeReferencePoint(relativeReference);
		}
		return anchor;
	}
});