/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.layout.ilog.ILogGraphConfigurator");

dojo.require("rmpw.diagrams.render.graph.BaseGraphConfigurator");

dojo.require('ibm_ilog.diagram.Graph');
dojo.require('ibm_ilog.diagram.Node');
dojo.require('ibm_ilog.diagram.Link');
dojo.require('ibm_ilog.diagram.Port');

dojo.require("rmpw.diagrams.layout.ilog.ILogBootstrapGraph");
dojo.require("rmpw.diagrams.layout.ilog.ILogBootstrapNode");
dojo.require("rmpw.diagrams.layout.ilog.ILogBootstrapLink");
dojo.require("rmpw.diagrams.layout.ilog.ILogBootstrapPort");

dojo.require('ibm_ilog.graphlayout.Direction');
dojo.require('ibm_ilog.graphlayout.internalutil.LinkStyles');
dojo.require('ibm_ilog.graphlayout.tree.TreeLayout');
dojo.require('ibm_ilog.graphlayout.hierarchical.HierarchicalLayout');
dojo.require('ibm_ilog.graphlayout.forcedirected.ForceDirectedLayout');
dojo.require('ibm_ilog.graphlayout.shortlink.ShortLinkLayout');
dojo.require('ibm_ilog.graphlayout.longlink.LongLinkLayout');

/**
 * A utility class for configuring a graph
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.layout.ilog.ILogGraphConfigurator", rmpw.diagrams.render.graph.BaseGraphConfigurator, {
	
	/*
	 * ILog support self-relations in their layout
	 */
	_nodeLayoutSupportsSelfRelations: function() {
		return true;
	},
	
	/*
	 * Check whether layout algorithms support an incremental mode
	 */
	_nodeLayoutSupportsIncremental: function() {
		return true;
	},
	
	runGraphLayout: function(layoutCallBack, args) {
		var currentGraph = args.targetController.renderMapping ? args.targetController.renderMapping.policies.graphElement.getGraphContainer().getInternalGraphElement() : 
			args.targetController.policies.graphElement.getGraphContainer().getInternalGraphElement();
		currentGraph.fireNodeLayoutInvoked(); // flag indicating node layout is being invoked
		
		return this.inherited(arguments);
	},
	
	runGraphLayoutAlgorithm: function(graph, layoutCallBack, args) {
		var nodeLayout = null;
		var linkLayout = null;
		
		var type = args.type.toLowerCase();
		if (type == "tree") { // default
			nodeLayout = this.createTreeLayout(args);
		}
		else if (type == "hierarchical" ||  
				 type == "graph") {
			nodeLayout = this.createHierarchicalLayout(args);
		}
		else if (type == "radial") {
			nodeLayout = this.createRadialLayout(args);
			linkLayout = this.createShortLinkLayout(args);
		}
		else if (type == "forcedirected") {
			nodeLayout = this.createForceDirectedLayout(args);
			linkLayout = this.createShortLinkLayout(args);
		}
		else if (type == "shortlink") {
			nodeLayout = this.createShortLinkLayout(args);
		}
		else if (type == "longlink") {
			nodeLayout = this.createLongLinkLayout(args);
		}
		
		var oldNodeLayout = graph.getNodeLayout();
		var oldLinkLayout = graph.getLinkLayout();
		if (oldLinkLayout) {
			oldLinkLayout.unfixAllLinks();
		}
		
		try {
			if (nodeLayout) {
				graph.setNodeLayout(nodeLayout);
				if (linkLayout) {
					graph.setLinkLayout(linkLayout);
				}
				else {
					graph.setLinkLayout(null);
				}
				graph.performGraphLayout();
				return layoutCallBack(graph, args);
			}
		}
		catch (e) {
			throw e;
		}
		finally {
			if (oldNodeLayout) {
				graph.setNodeLayout(oldNodeLayout);
			}
			if (oldLinkLayout) {
				graph.setLinkLayout(oldLinkLayout);
			}
		}
		
		return false;
	},
	
	createGraph: function() {
		var graph = new rmpw.diagrams.layout.ilog.ILogBootstrapGraph();
		var linkLayout = this.createShortLinkLayout({});
		linkLayout.setIncrementalMode(true);
		linkLayout.setPreserveFixedLinks(true);
		linkLayout.setGlobalLinkStyle(ibm_ilog.graphlayout.internalutil.LinkStyles.MIXED_STYLE);
		graph.setLinkLayout(linkLayout);
		linkLayout.__linkLayoutType = ibm_ilog.graphlayout.shortlink.ShortLinkLayout;
		
		return graph;
	},
	
	createGraphNode: function(mapping, graph, figRect) {
		var ilogNode = new rmpw.diagrams.layout.ilog.ILogBootstrapNode(mapping, graph, figRect.x, figRect.y, figRect.w, figRect.h);
		graph.children.push(ilogNode);
		ilogNode.parent = graph;
		ilogNode.mapping = mapping;
		
		return ilogNode;
	},
	
	createGraphEdge: function(mapping, graph, sourceNode, targetNode) {
		var ilogLink = new rmpw.diagrams.layout.ilog.ILogBootstrapLink(mapping, graph);
		
		if (sourceNode) {
			var startPort = new rmpw.diagrams.layout.ilog.ILogBootstrapPort(sourceNode);
			ilogLink.setStartPort(startPort);
		}
		
		if (targetNode) {
			var endPort = new rmpw.diagrams.layout.ilog.ILogBootstrapPort(targetNode);
			ilogLink.setEndPort(endPort);
		}
		
		graph.children.push(ilogLink);
		ilogLink.parent = graph;
		ilogLink.mapping = mapping;
		
		return ilogLink;
	},
	
	/*
	 * Create an anchor owned by graphElement of a particular type - one of FIXED_ANCHOR,
	 * MOVEABLE_ANCHOR
	 */
	createAnchor: function(anchorType, graphElement, args) {
		var port = new rmpw.diagrams.layout.ilog.ILogBootstrapPort(graphElement);
		return port;
	},
	
	/*
	 * Gets the flow direction of the layout
	 */
	getFlowDirection: function(args) {
		if (!args.flowDirection) {
			return ibm_ilog.graphlayout.Direction.BOTTOM;
		}
		
		if (args.flowDirection == "top") {
			return ibm_ilog.graphlayout.Direction.TOP;
		}
		else if (args.flowDirection == "bottom") {
			return ibm_ilog.graphlayout.Direction.BOTTOM;
		}
		else if (args.flowDirection == "left") {
			return ibm_ilog.graphlayout.Direction.LEFT;
		}
		else if (args.flowDirection == "right") {
			return ibm_ilog.graphlayout.Direction.RIGHT;
		}
	},
	
	/**
	 * Creates a tree layout based on the current form settings
	 */
	createTreeLayout: function(args) {
		var layout = new ibm_ilog.graphlayout.tree.TreeLayout();
		layout.setLayoutMode(ibm_ilog.graphlayout.tree.TreeLayout.LEVEL);
		layout.setFlowDirection(this.getFlowDirection(args));
		
		var globalStyle = this.isOrthogonalLinks(args) ? 
				ibm_ilog.graphlayout.tree.TreeLayout.ORTHOGONAL_STYLE : ibm_ilog.graphlayout.tree.TreeLayout.MIXED_STYLE;		
		layout.setGlobalLinkStyle(globalStyle);
		
		layout.setConnectorStyle(ibm_ilog.graphlayout.tree.TreeLayout.CENTERED_PINS);
		layout.setGlobalAlignment(ibm_ilog.graphlayout.tree.TreeLayout.CENTER);
		layout.setIncrementalMode(args.incremental ? args.incremental : false);
		return layout;
	},
	
	/**
	 * Creates a radial layout based on the current form settings
	 */
	createRadialLayout: function(args) {
		var layout = new ibm_ilog.graphlayout.tree.TreeLayout();
		layout.setLayoutMode(ibm_ilog.graphlayout.tree.TreeLayout.ALTERNATING_RADIAL);
		layout.setFlowDirection(this.getFlowDirection(args));
		layout.setGlobalLinkStyle(ibm_ilog.graphlayout.tree.TreeLayout.STRAIGHT_LINE_STYLE); // simply rely on edge routing after the layout
		layout.setConnectorStyle(ibm_ilog.graphlayout.tree.TreeLayout.AUTOMATIC_PINS);
		layout.setGlobalAlignment(ibm_ilog.graphlayout.tree.TreeLayout.CENTER);
		layout.setIncrementalMode(args.incremental ? args.incremental : false);
		return layout;
	},

	/**
	 * Creates a hierarchical layout based on the current form settings
	 */
	createHierarchicalLayout: function(args) {
		var layout = new ibm_ilog.graphlayout.hierarchical.HierarchicalLayout();
		layout.setFlowDirection(this.getFlowDirection(args));
		if (this.isOrthogonalLinks(args)) {
			layout.setGlobalLinkStyle(ibm_ilog.graphlayout.hierarchical.HierarchicalLayout.ORTHOGONAL_STYLE);
		}
		else {
			layout.setGlobalLinkStyle(ibm_ilog.graphlayout.hierarchical.HierarchicalLayout.POLYLINE_STYLE);
		}
		
		layout.setConnectorStyle(ibm_ilog.graphlayout.hierarchical.HierarchicalLayout.AUTOMATIC_PINS);
		layout.setLevelingStrategy(ibm_ilog.graphlayout.hierarchical.HierarchicalLayout.SEMI_OPTIMAL);
		layout.setHorizontalLinkOffset(15);
		layout.setHorizontalNodeOffset(35);
		layout.setHorizontalNodeLinkOffset(20);
		layout.setVerticalLinkOffset(15);
		layout.setVerticalNodeOffset(35);
		layout.setVerticalNodeLinkOffset(20);
		layout.setIncrementalMode(args.incremental ? args.incremental : false);
		layout.setLinkWidthUsed(false);
		layout.setFromFork(true);
        layout.setToFork(true);
		return layout;
	},

	/**
	 * Creates a force-directed layout based on the current form settings
	 */
	createForceDirectedLayout: function (args) {
		var layout = new ibm_ilog.graphlayout.forcedirected.ForceDirectedLayout();
		
		if (args.incremental) {
			layout.setLayoutMode(ibm_ilog.graphlayout.forcedirected.ForceDirectedLayout.INCREMENTAL_MODE);
		}
		else {
			layout.setLayoutMode(ibm_ilog.graphlayout.forcedirected.ForceDirectedLayout.FAST_MULTILEVEL_MODE);
		}
		layout.setPreferredLinksLength(40);
		layout.setAllowedNumberOfIterations(1000);
		layout.setRespectNodeSizes(true);
		return layout;
	},
	
	createLongLinkLayout: function (args) {
		var layout = new ibm_ilog.graphlayout.longlink.LongLinkLayout();
		layout.setNodeObstacleEnabled(true);
		if (this.isOrthogonalLinks(args)) {
			layout.setGlobalLinkStyle(ibm_ilog.graphlayout.longlink.LongLinkLayout.ORTHOGONAL_STYLE);
		}
		else {
			layout.setGlobalLinkStyle(ibm_ilog.graphlayout.longlink.LongLinkLayout.DIRECT_STYLE);
		}
		
		return layout;
	},
	
	createShortLinkLayout: function (args) {
        var layout = new ibm_ilog.graphlayout.shortlink.ShortLinkLayout();
        
        if (this.isOrthogonalLinks(args)) {
        	layout.setGlobalLinkStyle(ibm_ilog.graphlayout.shortlink.ShortLinkLayout.ORTHOGONAL_STYLE);
		}
        else {
        	layout.setGlobalLinkStyle(ibm_ilog.graphlayout.shortlink.ShortLinkLayout.DIRECT_STYLE);
        }
        layout.setLinkOverlapNodesForbidden(true);
        layout.setGlobalConnectorStyle(ibm_ilog.graphlayout.shortlink.ShortLinkLayout.AUTOMATIC_PINS);
        layout.setLinkBundlesMode(ibm_ilog.graphlayout.shortlink.ShortLinkLayout.FIRST_LAST_SEGMENT);
        layout.setMinFinalSegmentLength(20);
        layout.setLinkOffset(15);
        
        return layout;
	}
});