/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("rmps.ui.InfoArea");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("rmps.ui.util.UIUtil");


var INFO= {OK: "ok", WARNING: "warning", ERROR: "error"};
var CLASS_MAP= {"ok": "okInfo", "warning": "warningInfo", "error": "errorInfo"};
var UIUtil= rmps.ui.util.UIUtil;

/* 
 * Requiered:
 * 		header: the title for info area
 * 		description: the description assocaited in the info area
 * Optional:
 * 
 * 
 * Description: A Web UI control to show an info area for the user. So we could have a title with a description for this area, and we could show any outstnading
 * updates on the area. It could be whether rmps.ui.InfoArea.Types.OK, rmps.ui.InfoArea.Types.ERROR, or rmps.ui.InfoArea.Types.WARNING
 * We could call for this.displayInfo(type, message) to show any updates on the info area
 *  
 */
dojo.declare("rmps.ui.InfoArea", [dijit._Widget, dijit._Templated], {

	templateString: dojo.cache("rmps.ui", "templates/InfoArea.html"),
	widgetsInTemplate: true,
	"class": "rmps-info-area",
	
	areaId: null,
	
	constructor: function(params) {
		if(!params){
			console.warn("No paramaters specified");
			params= {};
		}
		
		/*Extract Parameters */
		
		/*If the user did not specify a trim style, then we assume it to be editor as defualt */
		this.header= UIUtil.associateParamValue(params.header, 
							"You have to provide a title for your info area");	//$NON-NLS-1$
		this.description= UIUtil.associateParamValue(params.description, 
							"You have to provide a description for your info area");	//$NON-NLS-1$
		this.INFO_TYPES= new rmps.ui.InfoArea.Types();
		
		this.areaId= params.areaId;
	},
	
	setInProgress: function(){
		dojo.addClass(this.mainNode,"inProgress");
	},
	
	unsetInProgress: function(){
		dojo.removeClass(this.mainNode,"inProgress");
	},
	
	/* 
	 *Example: myInfoArea.displayInfo(myInfoArea.INFO_TYPES.ERROR, "This is an error!!!!"); 
	 */
	displayInfo: function(type, message){
		this._displayInfo(type, message);
	},
	
	/* 
	 * Called to hide any info message. 
	 */
	hideInfo: function(){
		dojo.removeClass(this.infoNode);
		this.infoNode.innerHTML= "";
		dojo.addClass(this.infoNode, "noDisplay");
	},
	
	_displayInfo: function(className, message){
		this._prepareHeaderNode(className);
		dojo.removeClass(this.infoNode);
		if(message){
			dojo.addClass(this.infoNode, CLASS_MAP[className]);
			this.infoNode.innerHTML= message;
		}else{
			this.hideInfo();
		}
	},
	
	_prepareHeaderNode: function(className){
		dojo.removeClass(this.headerNode);
		dojo.addClass(this.headerNode, className);
	}
	

});

dojo.declare("rmps.ui.InfoArea.Types", null, {

	OK: "ok",			//$NON-NLS-1$ 
	WARNING: "warning", //$NON-NLS-1$
	ERROR: "error",		//$NON-NLS-1$
	LOADING: "loading"	//$NON-NLS-1$
});

})();