/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.panner.LayerContainer");

dojo.require("rmpw.panner.Layer");

dojo.declare("rmpw.panner.LayerContainer", rmpw.panner.Layer, {

	_layerIds: null,
	_layers: null,
	
	constructor: function() {
		this._layerIds = [];
		this._layers = [];
	},
	
	startup: function() {
		this.inherited(arguments);
		for (var i = 0, len = this._layers.length; i < len; ++i) {
			this._layers[i].startup();
		}
	},
	
	resize: function(size) {
		this.inherited(arguments);
		
		for (var i = 0, len = this._layers.length; i < len; ++i) {
			if (this._layers[i].resize) {
				this._layers[i].resize(size);
			}
		}
	},
	
	addLayer: function(id, layer, index) {
		if (this.getLayer(id)) {
			throw Error("A layers with ID " + id + " already exists.");
		}
		
		layer._parent = this;
		
		if (index != null && index < this._layers.length) {
			this._layers.splice(index, 0, layer);
			this._layerIds.splice(index, 0, id);
			this.containerNode.insertBefore(this._layers[i + 1].layer.domNode);
		} else {
			this._layers.push(layer);
			this._layerIds.push(id);
			this.containerNode.appendChild(layer.domNode);
		}
		
		if (this._started && !layer._started) {
			layer.startup();
		}
	},
	
	getLayers: function() {
		return this._layers;
	},
	
	getLayer: function(id) {
		for (var i = 0, len = this._layerIds.length; i < len; ++i) {
			if (this._layerIds[i] == id) {
				return this._layers[i].layer;
			}
		}
		return null;
	},
	
	removeLayer: function(layer) {
		for (var i = 0, len = this._layers.length; i < len; ++i) {
			if (this._layers[i] == layer) {
				this.containerNode.removeChild(this._layers[i].domNode);
				this._layers.splice(i, 1);
				this._layerIds.splice(i, 1);
				return;
			}
		}
	},
	
	removeLayerById: function(id) {
		for (var i = 0, len = this._layerIds.length; i < len; ++i) {
			if (this._layerIds[i] == id) {
				this.containerNode.removeChild(this._layers[i].domNode);
				this._layers.splice(i, 1);
				this._layerIds.splice(i, 1);
				return;
			}
		}
	},
	
	setScale: function(scale, cx, cy, origin) {
		var prevScale = this.getScale();
		if (prevScale != scale) {
			this.inherited(arguments);
			for (var i = 0, len = this._layers.length; i < len; ++i) {
				if (this._layers[i].setScale) {
					this._layers[i].setScale(scale, cx, cy, origin);
				}
			}
		}
	}
});

})();