/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("rmps.ui.LoginArea");

	dojo.require("rmps.app.ui.internal.util.UrlUtil");

	/*
	 * This is a control used to provide a link to enable logging into other "friend" servers. It helps us to proivde a link-like text in whic we click in order
	 * to show the login dialog.
	 *
	 * Required:
	 * 		loginLink: The link for the server. We could provide any link for a given friend server by any form.
	 * 					Examples:
	 * 							https://tbmoraby.ottlab.rationalswg.ibm.com:39443/jts
	 * 							https://tbmoraby.ottlab.rationalswg.ibm.com:39443/rmps/web
	 *
	 * Optional:
	 * 		loggedText: The string for the login link in case the user is authorized. If the user did not provide one, then it would be blank
	 * 		unloggedText: The string for the login link in case the user is still unauthorized. If the user did not provide one, then it would be "-"
	 * 		errorText: The string for the login link in case the user's failed to login. If the user did not provide one, then nothing will happen
	 * 		lazyCheck: If <code>false</code>, this means that we would like to update the login text based on the user's login information. So, if the user
	 *  				is already logged in the server, then use the loggedText text; otherwise, it will use the unloggedText. This check will not invoke
	 * 					the oAuth Interceptor in order to prvent asking the users to loggin right away (just only when they want). If it is <code>false</code>,
	 * 					nothing will happen; in other word, the unloggedText will be shown by default
	 *
	 * @author orabim (Mahmoud M.Orabi)
	 */
    dojo.declare("rmps.ui.LoginArea", [dijit._Widget, dijit._Templated], {

		/*Public fields */
        loginLink: null,
        loggedText: null,
        unloggedText: null,
        errorText: null,
        lazyCheck: true,


		/*Private fields */
        _textNode: null,


        templateString: '<div dojoAttachPoint="containerNode">' +
        				'<a href="#" dojoAttachPoint="anchor" dojoAttachEvent="onclick:_onLogin"></a>' +
        				'</div>',

		constructor: function(params){
			if(!params.loginLink){
				alert("loginLink must be provided");	//$NON-NLS-1$
			}
			if(!params.unloggedText){
				params.unloggedText= "-";	//$NON-NLS-1$
			}

			if(!params.loggedText){
				params.loggedText= "";		//$NON-NLS-1$
			}
		},

        postCreate: function(){
			this.inherited(arguments);
			if(!this.lazyCheck){
				this.requestLogin(true);
			}
			this._showInformation(this.unloggedText);
        },

        _onLogin: function(e){
    		dojo.stopEvent(e);
			this.requestLogin(false);
        },

		/*Public Method */
        requestLogin: function(skipInterceptors){
        	var args = {
				url: this.loginLink,
				handleAs: "xml"		//$NON-NLS-1$
			};

			var dfd = jazz.client.xhrGet(args);
			if(skipInterceptors){
				args.skipInterceptors = ["jazz.oauth"];	//$NON-NLS-1$
			}

			dfd.addCallback(dojo.hitch(this,function() {
				this._onAuthenticated();
			}, dfd));
			dfd.addErrback(dojo.hitch(this, function (error) {
				this._onLoginError(error);
			}));
        },

        _showInformation: function(text){
        	if(this._textNode){
        		/*No need to check if anchor has  textNode in its children as we already know that*/
        		this.anchor.removeChild(this._textNode);

        		/*A fresh start with a new text node */
        		dojo.destroy(this._textNode);
        	}
        	this._textNode= dojo.doc.createTextNode(text);
			this.anchor.appendChild(this._textNode);
        },

        _onAuthenticated: function(){
        	this.showAsLogged();
        	this.onAuthenticated();
        },

		_onLoginError: function(response){
			if(this.errorText){
				this._showInformation(this.loggedText);
			}
			this.onLoginError(response);
        },

        showAsLogged: function(){
        	this._showInformation(this.loggedText);
        },

        onAuthenticated: function(){
        	/*To be implemented by clients */
        },

		onLoginError: function(response){
			/*To be implemented by clients */
        }


    });

})();
