/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("rmps.ui.MenuSection");
    
    dojo.require("dijit._Widget");

    dojo.require("jazz.app.ua.SectionHoverHelp");

    dojo.requireLocalization("rmps.ui", "MenuSectionMessages");
    
    dojo.require("dojox.data.dom");
	
    var messages = dojo.i18n.getLocalization("rmps.ui", "MenuSectionMessages");
    
    dojo.declare("rmps.ui.MenuSection", [dijit._Widget, dijit._Templated], {
        ITEM_SLOT: "__menuItem",
        templateString: '<div class="rmps-app-editors-views-links">' +
        '<div class="LinksMenuWidget">' +
        '<div dojoAttachPoint="_actionDIV" class="LinksMenuActionBar">' +
        '<div dojoAttachPoint="_actionBar" class="LinksMenuDropDownActionWidget LinksMenuActionBarWidget">' +
        '<span dojoAttachPoint="_labelNode" class="LinksMenuLabel"></span>' +
        '<span dojoAttachPoint="_labelSep" class="LinksMenuLabel">: </span>' +
        '<a href="#" dojoAttachPoint="_actionContainerNode" dojoAttachEvent="onclick: _handleDropDownClick" class="LinksMenuActionContainer">' +
        '<span dojoAttachEvent="onclick: _handleDropDownClick" dojoAttachPoint="_actionNodeImg" class="LinksMenuAction"><span dojoAttachPoint="_actionNode"></span></span>' +
        '<img alt="" dojoAttachPoint="_downArrowIMG" dojoAttachEvent="" class="LinksMenuDownArrowImage" />' +
        '</a>' +
        '</div>' +
        '</div>' +
        '</div>' +
        '</div>',
        
        parentWidget: null,
        actionSelector: null,
        label: null,
        menuLabel: null,
        menuIconUrl: null,
        menuActions: null,
        uaKey: null,
        identifierList: null,
        isTitleButton: false,
        
        constructor: function(){
            this.identifierList= [];
        },
        
        postCreate: function(){
            this.containerNode = this.domNode;
            this._downArrowIMG.src = dojo.moduleUrl("rmps.ui", "graphics/popup-down-arrow.gif");
            
    		if (this.uaKey) {
    			this._help = new jazz.app.ua.SectionHoverHelp({uaKey: this.uaKey, title: messages.help});
    			dojo.addClass(this._help.domNode, "action help");
    			dojo.style(this._help.domNode, {
    				"float":"right",
    				"margin":"5px 5px 0px 0px"
    			});
    			this.connect(this._help.domNode, "onclick", dojo.stopEvent);
    			this._actionBar.appendChild(this._help.domNode);
    		}
    		
    		if (this.isTitleButton) {
    			this._actionBar.removeChild(this._labelNode);
    			this._actionBar.removeChild(this._labelSep);
    			this._actionNode.appendChild(document.createTextNode(this.menuLabel));
                if (this.menuIconUrl) {
        			this._actionNode.style.paddingLeft = "20px";
                    this._actionNodeImg.style.background = "url(" + this.menuIconUrl + ")";
                    this._actionNodeImg.style.padding = "2px 0";
                    this._downArrowIMG.style.padding = "4px 0";
                }
    		}
    		
    		this.inherited(arguments);
        },
        
        fillMenuActions: function(data){
        	/*To be implemnted by sub classes */
        },
        
        addMenuAction: function(data){
        	/*To be implemnted by sub classes */
        },
        
        removeAction: function(identifier){
        	var index;
        	for(index=0; index<this.menuActions.length; index++){
        		var current= this.menuActions[index];
        		if(current.identifier== identifier){
        			this.menuActions.splice(index,1);
        		}
        	}
        	
        	var self= this;
        	var resetSelectAction= function(){
        		if(self._currentAction.identifier== identifier){
        			self.setCurrentAction(self.menuActions[0]);
        		}
        	};
			if (!this._menuWidget) {
				resetSelectAction();
				return;
			}
        	
        	
        	var children= this._menuWidget.getChildren();
        	for(index=0; index<children.length; index++){
        		var child= children[index];
        		if(this.identifierList[child.id]==identifier){
        			delete this.identifierList[child.id];
        			this._menuWidget.removeChild(child);
        			resetSelectAction();
        			return;
        		}
        	}
        },
        
        modifyAction: function(args){
        	var identifier= args.identifier;
        	var field= args.field;
        	var value= args.value;
        	var index;
        	
        	for(index=0; index<this.menuActions.length; index++){
        		var current= this.menuActions[index];
        		if(current.identifier== identifier){
        			current[field]=  value;
        			current.value[field]= value;
        		}
        	}
        	
        	var self= this;
        	var resetSelectAction= function(){
        		if(self._currentAction.identifier== identifier){
        			self.setCurrentAction(self.menuActions[0]);
        		}
        	};
        	if(!this._menuWidget){
        		resetSelectAction(); 
        		return;
        	}
        	
        	var children= this._menuWidget.getChildren();
        	for(index=0; index<children.length; index++){
        		var child= children[index];
        		if(this.identifierList[child.id]==identifier){
        			child.attr(field, value);
        			var action = child[this.ITEM_SLOT];
        			action[field]= value;
        			action.value[field]= value;
        			resetSelectAction();
        		}
        	}
        },
        
        modifyIdentifier: function(args){
        	var currentIdentifier= args.currentIdentifier;
        	var newIdentifier= args.newIdentifier;
        	var index;
        	
        	if(this._currentAction.identifier== currentIdentifier){
        		this._currentAction.identifier= newIdentifier;
        		this._currentAction.value.identifier= newIdentifier;
        	} 
        	
        	for(index=0; index<this.menuActions.length; index++){
        		var current= this.menuActions[index];
        		if(current.identifier== currentIdentifier){
        			current.identifier= newIdentifier;
        			current.value.identifier= newIdentifier;
        		}
        	}
        	
        	if(!this._menuWidget){ return;}
        	
        	var children= this._menuWidget.getChildren();
        	for(index=0; index<children.length; index++){
        		var child= children[index];
        		if(this.identifierList[child.id]==currentIdentifier){
        			this.identifierList[child.id]=newIdentifier;
        			var action = child[this.ITEM_SLOT];
        			action.identifier= newIdentifier;
        			action.value.identifier= newIdentifier; 
        		}
        	}
        },
        
        fillMenu: function(data){
	        this._setMenu(this.fillMenuActions(data));
	    },
        
        _setMenu: function(menuActions){
            if (dojo.isArray(menuActions)) {
                this.menuActions = [];
                for (var i = 0; i < menuActions.length; i++) {
                    var action = menuActions[i];
                    var menuAction = {
                        receiverSelector: this.actionSelector,
                        label: action.label,
                        iconSrc: action.iconUrl,
                        identifier: action.identifier,
                        value: action
                    };
                    this.menuActions.push(menuAction);
                }
                
                this._actionContainerNode.style.color = "#075F87";
                this._actionContainerNode.title= "";
                this._labelNode.appendChild(document.createTextNode(this.menuLabel));
                this.actionIconSrcProperty = "iconSrc";
                this.setCurrentAction(this.menuActions[0]);
            }
        },
        
        addAction: function(data){
        	var action= this.addMenuAction(data);
        	var menuAction;
        	if(action){
                menuAction = {
                    receiverSelector: this.actionSelector,
                    label: action.label,
                    iconSrc: action.iconUrl,
                    identifier: action.identifier,
                    value: action
                };
                this.menuActions.push(menuAction);
        	}
        	if(this._menuWidget){
        		this._menuWidget.addChild(this._createMenuItem(menuAction));	
        	}
        },
        
    	startup: function() {
    		this.inherited(arguments);
    		if (this._help) {
    			this._help.startup();
    		}
    	},
        
        destroy: function(){
    		if (this._help) {
    			this._help.destroyRecursive();
    		}
    		
            this.menuActions = null;
            
            delete this.parentWidget;
        	this.parentWidget= null;
            this.inherited(arguments);
        },
        
        setCurrentAction: function(action){
            if (action) {
                this._currentAction = action;
                if (!this.isTitleButton) {
	                dojox.data.dom.replaceChildren(this._actionNode, document.createTextNode(action.label));
	                if (action.iconSrc) {
	                    this._actionNode.style.paddingLeft = "18px";
	                    this._actionNodeImg.style.background = "url(" + action[this.actionIconSrcProperty] + ")";
	                }
                }
            }
        },
        performAction: function(action, section, table){
            if (action.receiverSelector &&
            this.parentWidget[action.receiverSelector]) {
                this.parentWidget[action.receiverSelector](action, section, table);
            }
        },
        
        // Handle an action selected in the menu
        _handleActionClick: function(e){
            dojo.stopEvent(e);
            var action = this._currentAction;
            
            dojo.addClass(this._actionNode, "LinksMenuAction_clicked");
            
            setTimeout(dojo.hitch(this, function(){
                dojo.removeClass(this._actionNode, "LinksMenuAction_clicked");
            }), 10);
            this.performAction(action, this.parentWidget, this.table);
        },
        
        _handleDropDownClick: function(e){
            e.preventDefault();
            if (!this._menuWidget) {
                this._createMenuWidget();
            }
            dojo.addClass(this._actionContainerNode, "LinksMenuActionContainer_open");
            var pos = dojo.coords(this._actionContainerNode, true);
            dijit.popup.open({
                popup: this._menuWidget,
                onCancel: dojo.hitch(this, this._handleMenuClose),
                parent: this,
                x: pos.x,
                y: pos.y + pos.h - 5,
                onExecute: dojo.hitch(this, this._handleMenuClose),
                orient: 'TL'
            });
            this._menuCloseOnBody = this.connect(this, "_onBlur", this._handleMenuClose);
            this._menuCloseOnWidget = this.connect(this.domNode, "onmouseup", this._handleMenuClose);
            this._menuWidget.focus();
        },
        
        _createMenuWidget: function(){
            this._menuWidget = new dijit.Menu({});
            dojo.addClass(this._menuWidget.domNode, "LinksMenuDijitMenu");
            this.connect(this._menuWidget, "onItemClick", "_handleActionSelect");
            this._setMenuItems(this.menuActions);
        },
        
        _setMenuItems: function(items){
            if (items.length > 0) {
                for (var i = 0; i < items.length; i++) {
                    this._menuWidget.addChild(this._createMenuItem(items[i]));
                }
            }
        },
        
        _createMenuItem: function(item){
            var label = item.label;
            var menuItem;
            
            if (item[this.actionIconSrcProperty]) {
                try {
                    menuItem = new dijit.MenuItem({
                        label: label,
                        iconClass: "imageIcon"
                    }); // TODO: Support images
                    var div = menuItem.domNode/* TR */.firstChild/* TD */.firstChild/* DIV */;
                    if (!dojo.isIE) {
                        div.style.background = "url(" +
                        item[this.actionIconSrcProperty] +
                        ")";
                    }
                } 
                catch (e) {
                } // well, we tried
            }
            else {
                menuItem = new dijit.MenuItem({
                    label: label,
                    iconClass: "pixelIcon"
                }); // TODO: Support images
            }
            
            menuItem[this.ITEM_SLOT] = item;
            this.identifierList[menuItem.id]= item.identifier;
            return menuItem;
        },
        
        _handleMenuClose: function(){
            dijit.popup.close(this._menuWidget);
            dojo.removeClass(this._actionContainerNode, "LinksMenuActionContainer_open");
            this.disconnect(this._menuCloseOnBody);
            this.disconnect(this._menuCloseOnWidget);
        },
        
        _handleActionSelect: function(/* MenuItem */menuItem){
            var action = menuItem[this.ITEM_SLOT];
            this.setCurrentAction(action);
            this.performAction(action,this.parentWidget,this.table);
            
        }
    });
    
})();
