/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.panner.OffScreenTracker");

dojo.require("rmpw.panner.PerimeterTracker");

dojo.require("dijit._Templated");

dojo.require("dojox.gfx");
dojo.require("dojox.gfx.matrix");

dojo.require("dojo.string");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmpw.panner", "OffScreenTrackerMessages");

var messages = dojo.i18n.getLocalization("rmpw.panner", "OffScreenTrackerMessages");
var MAX_DURATION = 1000;

dojo.declare("rmpw.panner.OffScreenTracker", [rmpw.panner.PerimeterTracker, dijit._Templated], {
	
	templateString: '<div class="rmpw-panner-offscreen-tracker">' +
				'<div style="width: 0px; height: 0px; position: absolute; right: 0px;">' +
					'<div class="legend" dojoAttachPoint="_legend" style="display: none;"></div>' +
				'</div>' +
			'</div>',
	
	// properties
	inset: 5,
	navigationStepDuration: 15,
	
	// attach points
	_legend: null,
	
	// private properties
	_movingPLayer: false,
	_legendAnimation: null,
	
	postCreate: function() {
		this.inherited(arguments);
		dojo._setOpacity(this._legend, 0);
	},
	
	startup: function() {
		this.inherited(arguments);
		// work around for IE - removing legend handling
//		if (dojo.isIE) {
//			setTimeout(dojo.hitch(this, function() {
//				for (var i = 0, len = this._trackers.length; i < len; ++i) {
//					var entry = new rmpw.panner.OffScreenTracker._LegendEntry({'tracker': this._trackers[i]});
//					this._legend.appendChild(entry.domNode);
//					entry.startup();
//					this._trackers[i].__legendEntry = entry;
//				}
//			}), 1);
//		}
	},
	
	destroy: function() {
		if (this._navigationAnimation) {
			this._navigationAnimation.stop();
			delete this._navigationAnimation;
		}
		this.inherited(arguments);
	},
	
	addTracker: function(tracker, params) {
		tracker.__color = params.color;
		
		// add to legend
//		if (!dojo.isIE) {
//			var entry = new rmpw.panner.OffScreenTracker._LegendEntry({'tracker': tracker});
//			this._legend.appendChild(entry.domNode);
//			entry.startup();
//			tracker.__legendEntry = entry;
//		}
		
		this.inherited(arguments);
	},
	
	removeTracker: function(tracker) {
		this._legend.removeChild(tracker.__legendEntry.domNode);
		tracker.__legendEntry.destroyRecursive();
		delete tracker.__legendEntry;
		this.inherited(arguments);
	},
	
	resize: function(size) {
		// relocate the legend
		if (size && size.h) {
			this._legend.parentNode.style.top = size.h + "px";
		}
		
		this.inherited(arguments);
	},
	
	_update: function() {
		if (!this._movingPLayer) {
			if (this._navigationAnimation) {
				this._navigationAnimation.stop();
				delete this._navigationAnimation;
			}
		}
		
		this.inherited(arguments);
	},
	
	createMarker: function(item) {
		return new rmpw.panner.OffScreenTracker._TrackedMarker({
			osTracker: this,
			'item': item,
			color: item.tracker.__color
		});
	},
	
	navigateTo: function(item) {
		if (this._navigationAnimation) {
			if (this._navigationAnimation.__item == item) {
				return;
			}
			this._navigationAnimation.stop();
			delete this._navigationAnimation;
		}
		
		var size = this.getSize();
		
		var p = item.getPoint();
		
		var hw = Math.floor(size.w / 2);
		var hh = Math.floor(size.h / 2);
		
		var loc = this.pannableLayer.getLocation();
		var startTop = loc.y;
		var startLeft = loc.x;
		
		var endTop = hh - p.y;
		var endLeft = hw - p.x;
		
		var duration = Math.min(Math.max(Math.abs(endTop - startTop), Math.abs(endLeft - startLeft)) * this.navigationStepDuration, MAX_DURATION);
		var args = {
			duration: duration,
			curve: [0.01, 1],
			onEnd: dojo.hitch(this, function() {
				delete this._navigationAnimation;
			}),
			onAnimate: dojo.hitch(this, function(r) {
				var x = startLeft + Math.floor((endLeft - startLeft) * r);
				var y = startTop + Math.floor((endTop - startTop) * r);
				
				this._movingPLayer = true;
				this.pannableLayer.setLocation(x, y);
				this._movingPLayer = false;
			})
		};
		
		this._navigationAnimation = new dojo._Animation(args);
		this._navigationAnimation.__item = item;
		this._navigationAnimation.play();
	},
	
	_showLegend: function() {
		if (this._legendAnimation) {
			this._legendAnimation.stop()
		}
		this._legendAnimation = dojo.fadeIn({
			node: this._legend,
			duration: 400,
			beforeBegin: dojo.hitch(this, function() {
				this._legend.style.display = "";
			})
		});
		this._legendAnimation.play();
	},
	
	_hideLegend: function() {
		if (this._legendAnimation) {
			this._legendAnimation.stop()
		}
		this._legendAnimation = dojo.fadeOut({
			node: this._legend,
			duration: 400,
			onEnd: dojo.hitch(this, function() {
				this._legend.style.display = "none";
			})
		});
		this._legendAnimation.play();
	}
});

dojo.declare("rmpw.panner.OffScreenTracker._LegendEntry", [dijit._Widget], {
	
	"class": "entry",
	
	tracker: null,
	_marker: null,
	
	postCreate: function() {
		this._marker = new rmpw.panner.OffScreenTracker._Marker({color: this.tracker.__color});
		dojo.addClass(this._marker.domNode, "marker");
		this.domNode.appendChild(this._marker.domNode);
		
		var entry = dojo.doc.createElement("nobr");
		entry.appendChild(dojo.doc.createTextNode(this.tracker.getLabel()));
		this.domNode.appendChild(entry)
	},
	
	destroy: function() {
		this._marker.destroyRecursive();
		this.inherited(arguments);
	},
	
	startup: function() {
		this.inherited(arguments);
		this._marker.startup();
	}
});

dojo.declare("rmpw.panner.OffScreenTracker._Marker", dijit._Widget, {
	color: null,
	shape: null,
	
	_surface: null,
	
	buildRendering: function() {
		this.inherited(arguments);
		this.domNode.style.width = "15px";
		this.domNode.style.height = "15px";
	},
	
	postCreate: function() {
		this._surface = dojox.gfx.createSurface(this.domNode, 15, 15);
		if (!dojo.isIE) {
			this.draw(this._surface);
		}
	},
	
	startup: function() {
		this.inherited(arguments);
		
		if (dojo.isIE) {
			// IE bug
			// surface must be attached to the document prior to drawing otherwise this.rawNode.path is undefined
			this.draw(this._surface);
		}
	},
	
	draw: function(surface) {
		this.shape = surface.createPolyline([{x: 0, y: 8}, {x: 8, y: 15}, {x: 15, y: 8}, {x: 0, y: 8}]);
		this.shape.setFill(this.color);
		this.shape.setStroke({width: 1, color: "#000000"});
	}
});

dojo.declare("rmpw.panner.OffScreenTracker._TrackedMarker", rmpw.panner.OffScreenTracker._Marker, {
	
	osTracker: null,
	
	item: null,
	
	draw: function(surface) {
		this.inherited(arguments);
		
		this.shape.connect("onclick", dojo.hitch(this, this._navigateTo));
		this.shape.connect("onmouseover", dojo.hitch(this.osTracker, this.osTracker._showLegend));
		this.shape.connect("onmouseout", dojo.hitch(this.osTracker, this.osTracker._hideLegend));
		
		this.shape.rawNode.style.cursor = "pointer";
		var tooltip = dojo.string.substitute(messages.navigateTo, [this.item.getLabel()]);
		
		if (dojo.isIE) {
			this.shape.rawNode.title = tooltip;
		} else {
			this.shape.rawNode.setAttribute("title", tooltip);
		}
		
		if (dojo.isWebKit) {
			// title on raw shape does not show up in chrome
			this.domNode.setAttribute("title", tooltip);
		}
	},
	
	_navigateTo: function(e) {
		this.osTracker.navigateTo(this.item);
		dojo.stopEvent(e);
	},
	
	rotate: function(angle) {
		var rotationMatrix = dojox.gfx.matrix.rotategAt(-(angle + 90), 8, 8);
		this.shape.setTransform(rotationMatrix);
	}
});

})();