/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.panner.Outline");

dojo.require("dojo.dnd.Moveable");
dojo.require("dojox.layout.ResizeHandle");

dojo.require("rmpw.panner.PannableLayer");

var MAX_DURATION = 1000;

dojo.declare("rmpw.panner.Outline", rmpw.panner.PannableLayer, {
	
	templateString: '<div class="rmpw-panner-outline">' +
			'<div class="rmpw-panner-outline-overview rmpw-panner-outline-size" dojoAttachPoint="_overview">' +
				'<div class="rmpw-panner-outline-blocker rmpw-panner-outline-size"></div>' +
				'<div dojoAttachPoint="panContainerNode" style="width:80%; height:80%;"></div>' +
				'<div dojoAttachPoint="_cView" class="rmpw-panner-outline-cview">' +
					'<div class="inner"></div>' +
				'</div>' +
				'<div dojoAttachPoint="_view" class="rmpw-panner-outline-view">' +
					'<div class="inner"></div>' +
					'<div dojoAttachPoint="_resizer"></div>' +
				'</div>' +
				'<div dojoAttachPoint="containerNode"></div>' +
				'<div dojoAttachPoint="_scaleText" class="rmpw-panner-outline-scaleText" style="display: none;"></div>' +
			'</div>' +
		'</div>',
	
	// mixin properties
	isRealTime: true,
	pannableLayer: null,
	
	// properties
	autoPanInterval: 20,
	autoPanStep: 4,
	animationStepDuration: 15,
	minResize: 0.25,
	maxResize: 5,
	
	// attach points
	_overview: null,  // outline shape (mini diagram outline)
	_view: null,      // dotted hollow movable inner view--invisible in realtime mode
	_cView: null,     // center yellow highlight inner view
	_resizer: null,   // resize handle
	
	// private proeprties
	_ratio: 0,
	_ratios: null, // {}
	_viewSize: null, // {}
	
	_autoPanHandle: null,
	_autoPan: null,
	
	_relocateAnimation: null,
	_centerPause: null,
	_centerAnimation: null,
	_cLeftTop: null,
	_moving: false,
	_movingPLayer: false,
	_resizeHandle: null,
	_scaleText: null,
	_offset: null, //{}
	_stopRelocateMove: false,
	
	// does not support set location
	setLocation: function() {},
	
	postCreate: function() {
		this.inherited(arguments);
		
		if (this.pannableLayer.getScale && this.pannableLayer.setScale) {
			this.createResizeHandle();
		}
		
		this._setRealTime(this.isRealTime);
		
		if( this.pannableLayer.viewer.rootLayer ){
			dojo.connect(this.pannableLayer.viewer.rootLayer.viewer.viewport, "onEndPan", this, "_pLayerMoved");
		}
		
		this._viewMoveable = new dojo.dnd.Moveable(this._view, {handle: this._view});
		this.connect(this._viewMoveable, "onMoveStart", this._onViewMoveStart);
		this.connect(this._viewMoveable, "onMoveStop", this._onViewMoveStop);
		this.connect(this._viewMoveable, "onMoved", this._onViewMoved);
		this.connect(this._viewMoveable, "onFirstMove", this._setDragCursor);
		this.connect(this._viewMoveable, "onMoveStart", this._setMoveCursor);
		this.connect(this._viewMoveable, "onMoveStop", this._setDefaultCursor);
		
		if (this.isRealTime) {
			this.connect(this._viewMoveable, "onMoved", this._onRealTimeViewMoved);
		}
				
		/* Dojo 1.6 introduced a bug where the outline view was sized to be
		 * 100%. I can't tell what is setting that size. Forcing it to be
		 * 125px in the postCreate.
		 */
		
		this.domNode.style.width = "125px";
	},
	
	createResizeHandle: function() {
		this._resizeHandle = new rmpw.panner.Outline._ResizeHandle({
			outline: this,
			targetContainer: this._view.firstChild,
			resizeAxis: 'xy',
			activeResize: true,
			minSize: {w: 0, h: 0},
			onResize: dojo.hitch(this, this._onResize),
			onChange: dojo.hitch(this, this._onResizeChange)
		}, this._resizer);
		this._resizeHandle.domNode.className = "resize";
	},
	
	_setRealTime: function(isRealTime) {
		if (isRealTime) {
			this._cView.style.visibility = "hidden";
			dojo.addClass(this.domNode, "rmpw-panner-outline-realtime");
		} else {
			this._cView.style.visibility = "";
			dojo.removeClass(this.domNode, "rmpw-panner-outline-realtime");
		}
	},
	
	destroy: function() {
		this._viewMoveable.destroy();
		if (this._autoPanHandle) {
			clearInterval(this._autoPanHandle);
			delete this._autoPanHandle;
		}
		if (this._relocateAnimation) {
			this._relocateAnimation.stop();
			delete this._relocateAnimation;
		}
		if (this._centerPause) {
			clearTimeout(this._centerPause);
			delete this._centerPause;
		}
		if (this._centerAnimation) {
			this._centerAnimation.stop();
			delete this._centerAnimation;
		}
		if (this._scaleAnimation) {
			this._scaleAnimation.stop();
			delete this._scaleAnimation;
		}
		if (this._resizeHandle) {
			this._resizeHandle.destroyRecursive();
			delete this._resizeHandle;
		}
		this.inherited(arguments);
	},
	
	getRatio: function() {
		return this._ratio;
	},
	
	ratioChanged: function(ratio) {
		
	},
	
	resize: function(s) {
		if (!this._overview) {
			return;
		}
		var size = {
			w: this._overview.offsetWidth,
			h: this._overview.offsetHeight
		};
		
		var layers = this.getLayers();
		for (var i = 0; i < layers.length; ++i) {
			if (layers[i].resize) {
				layers[i].resize(size);
			}
		}
		
		/*
		 * Need to get the size of the diagram (represented by _cView) from the marginBox,  
		 * this lets us use CSS styles and percentages instead of needing explicit sizes 
		 * and requiring expensive calculations.
		 */ 
		if (!s) {
			s = dojo.marginBox(this.pannableLayer.domNode);
		}
		var w = s.w;
		var h = s.h;
		
		// size of screen (represented by _overview)
		var sw = window.screen.width;
		var sh = window.screen.height;
		
		// size of _overview
		var ow = size.w;
		var oh = size.h;
		
		if (!this._ratios) {
			var osrw = ow / sw;
			var osrh = oh / sh;
			this._ratios = {w: osrw, h: osrh};
		} 
		
		if (this._ratios.w < this._ratios.h) {
			var ratio = ow / sw;
		} else {
			var ratio = oh / sh;
		}
		
		// size of _cView
		var height = Math.floor(h * ratio);
		var width = Math.floor(w * ratio);
		
		if (ratio != this._ratio) {
			this._ratio = ratio;
			this.ratioChanged(ratio);
		}
		
		this._viewSize = {
			w: width,
			h: height
		};
		// resize the view
		this._cView.firstChild.style.width = this._view.firstChild.style.width = this._viewSize.w + "px";
		this._cView.firstChild.style.height = this._view.firstChild.style.height = this._viewSize.h + "px";
		
		// center the view
		this._offset = {};
		this._offset.l = Math.floor((ow - width) / 2);
		this._offset.t = Math.floor((oh - height) / 2);
		this._cView.style.left = this._view.style.left = this._offset.l - this._view.clientLeft + "px";
		this._cView.style.top = this._view.style.top = this._offset.t - this._view.clientTop + "px";
		
		// locate content
		var loc = this.pannableLayer.getLocation();
		var scale = this.pannableLayer.getScale();
		this._setLocation(loc.x * this._ratio / scale + this._offset.l,
				loc.y * this._ratio /scale + this._offset.t);
		
		if (this._resizeHandle) {
			var scale = this.getScale();
			this._resizeHandle.minSize = {
				w: Math.floor(Math.max(this._viewSize.w / this.maxResize * scale, this._resizeHandle.domNode.offsetWidth)),
				h: Math.floor(Math.max(this._viewSize.h / this.maxResize * scale, this._resizeHandle.domNode.offsetHeight))
			};
			this._resizeHandle.maxSize = {
				w: Math.ceil(this._viewSize.w / this.minResize * scale),
				h: Math.ceil(this._viewSize.h / this.minResize * scale)
			};
		}
		
		
	},
	
	_onResize: function() {
		if (this._autoPanHandle) {
			clearInterval(this._autoPanHandle);
			delete this._autoPanHandle;
		}
		delete this._autoPan;
		
		this._setScaleText();
		
		var endSize = dojo.marginBox(this._view.firstChild);
		var scale = 1 / (1 - ((this._viewSize.w - endSize.w) / this._viewSize.w)) * this.pannableLayer.getScale();
		
		if (scale == this.pannableLayer.getScale()) {
			return;
		}
		
		// maintain scale within bounds
		scale = Math.max(this.minResize, Math.min(scale, this.maxResize));
		
		this._resizeHandle.minSize = {
			w: Math.floor(Math.max(this._viewSize.w / this.maxResize * scale, this._resizeHandle.domNode.offsetWidth)),
			h: Math.floor(Math.max(this._viewSize.h / this.maxResize * scale, this._resizeHandle.domNode.offsetHeight))
		};
		this._resizeHandle.maxSize = {
			w: Math.ceil(this._viewSize.w / this.minResize * scale),
			h: Math.ceil(this._viewSize.h / this.minResize * scale)
		};
		
		// scale around a specific point
		var ratio = scale / this.pannableLayer.getScale();
		var loc = this.pannableLayer.getLocation();
		var viewBox = this.pannableLayer.getViewBox();
		var x = viewBox.w / 2 / ratio - loc.x;
		var y = viewBox.h / 2 / ratio - loc.y;
		
		var restore = this._offset;
		this._offset = {
			l: this._view.offsetLeft + this._view.clientLeft,
			t: this._view.offsetTop + this._view.clientTop
		};
		this.pannableLayer.viewer.rootLayer.setScale(scale, null, null, true );
		this._offset = restore;
		
		// resize the view
		this._cView.firstChild.style.width = this._view.firstChild.style.width;
		this._cView.firstChild.style.height = this._view.firstChild.style.height;
		
		var duration = Math.min(Math.max(Math.abs(endSize.h - this._viewSize.h), Math.abs(endSize.w - this._viewSize.w)) * this.animationStepDuration, MAX_DURATION);
		
		var args = {
			node: this._view.firstChild,
			duration: duration,
			properties: {
				width: {
					end: this._viewSize.w,
					unit: "px"
				},
				height: {
					end: this._viewSize.h,
					unit: "px"
				}
			},
			
			onEnd: dojo.hitch(this, function() {
				delete this._scaleAnimation;
			}),
			
			onAnimate: dojo.hitch(this, function(prop) {
				this._cView.firstChild.style.width = prop.width;
				this._cView.firstChild.style.height = prop.height;
			})
		};
		
		this._centerView();
		this._scaleAnimation = dojo.animateProperty(args).play();
	},
	
	_onResizeChange: function() {
		if (this._viewSize.w > this._viewSize.h) {
			var panStepL = this.autoPanStep;
			var panStepT = this.autoPanStep / this._viewSize.w * this._viewSize.h; 
		} else {
			var panStepL = this.autoPanStep / this._viewSize.h * this._viewSize.w;
			var panStepT = this.autoPanStep; 
		}
		
		this._autoPan = {};
		
		var rEdge = this._view.offsetLeft + this._view.offsetWidth + 1;
		var bEdge = this._view.offsetTop + this._view.offsetHeight + 1;
		
		if (rEdge > this._overview.offsetWidth || bEdge > this._overview.offsetHeight) {
			this._autoPan.l = -panStepL;
			this._autoPan.t = -panStepT;
		} else if ((this._view.offsetLeft < 0 || this._view.offsetTop < 0) &&
				rEdge < (this._overview.offsetWidth - 10) && bEdge < (this._overview.offsetHeight - 10)) {
			this._autoPan.l = panStepL;
			this._autoPan.t = panStepT;
		} else {
			delete this._autoPan;
		}
		
		panStepL = panStepT = rEdge = bEdge = null;
		
		if (this._autoPan) {
			this._autoPan.vl = this._view.offsetLeft;
			this._autoPan.vt = this._view.offsetTop;
			
			var loc = this.getLocation();
			this._autoPan.pl = loc.x;
			this._autoPan.pt = loc.y;
			
			this._autoPan.w = this._view.firstChild.offsetWidth;
			this._autoPan.h = this._view.firstChild.offsetHeight;
			
			this._autoPan.sx = this._resizeHandle.startPoint.x;
			this._autoPan.sy = this._resizeHandle.startPoint.y;
			
			this._autoPan.sl = 0;
			this._autoPan.st = 0;
		}
		
		this._setScaleText(1 / (1 - ((this._viewSize.w - this._view.firstChild.offsetWidth) / this._viewSize.w)) * this.pannableLayer.getScale());
		
		if (!this._autoPan) {
			if (this._autoPanHandle) {
				clearInterval(this._autoPanHandle);
				delete this._autoPan;
				delete this._autoPanHandle;
			}
		} else if (!this._autoPanHandle) {
			this._autoPanHandle = setInterval(dojo.hitch(this, function() {
				this._autoPan.sl += this._autoPan.l;
				this._autoPan.st += this._autoPan.t;
				
				var width = this._autoPan.w - this._autoPan.sl;
				var height = this._autoPan.h - this._autoPan.st;
				
				var hitMax = false;
				if (this._resizeHandle.minSize) {
					if (width <= this._resizeHandle.minSize.w) {
						var diff = this._resizeHandle.minSize.w - width;
						height = height + diff * height / width;
						width = this._resizeHandle.minSize.w;
					}
					
					if (height <= this._resizeHandle.minSize.h) {
						var diff = this._resizeHandle.minSize.h - height;
						width = width + diff * width / height;
						height = this._resizeHandle.minSize.h;
					}
				}
				if (this._resizeHandle.maxSize) {
					if (width >= this._resizeHandle.maxSize.w) {
						var diff = this._resizeHandle.maxSize.w - width;
						height = height + diff * height / width;
						width = this._resizeHandle.maxSize.w;
						hitMax = true;
					}
					
					if (height >= this._resizeHandle.maxSize.h) {
						var diff = this._resizeHandle.maxSize.h - height;
						width = width + diff * width / height;
						height = this._resizeHandle.maxSize.h;
						hitMax = true;
					}
				}
				
				width = Math.round(width);
				height = Math.round(height);
				
				this._setScaleText(1 / (1 - ((this._viewSize.w - width) / this._viewSize.w)) * this.pannableLayer.getScale());
				this._view.firstChild.style.width = width + "px";
				this._view.firstChild.style.height = height + "px";
				
				this._resizeHandle.startPoint.x = this._autoPan.sx + this._autoPan.sl;
				this._resizeHandle.startPoint.y = this._autoPan.sy + this._autoPan.st;
				
				this._view.style.left = this._cView.style.left = this._autoPan.vl + this._autoPan.sl + "px";
				
				this._view.style.top = this._cView.style.top = this._autoPan.vt + this._autoPan.st + "px";
				
				this._setLocation(this._autoPan.pl + this._autoPan.sl,
						this._autoPan.pt + this._autoPan.st);
				
				if (hitMax || (this._autoPan.l > 0 && this._view.offsetLeft >= 0 &&
						this._autoPan.t > 0 && this._view.offsetTop >= 0)) {
					clearInterval(this._autoPanHandle);
					delete this._autoPan;
					delete this._autoPanHandle;
				}
			}), this.autoPanInterval);
		}
	},
	
	_setScaleText: function(scale) {
		if (scale) {
			this._scaleText.innerHTML = Math.round(Math.max(this.minResize, Math.min(this.maxResize, scale)) * 100) + "%";
			this._scaleText.style.display = "";
		} else {
			this._scaleText.style.display = "none";
		}
	},
	
	addLayer: function(id, layer, index) {
		this.inherited(arguments);
		var size = {
			w: this._overview.offsetWidth,
			h: this._overview.offsetHeight
		};
		layer.resize(size);
	},
	
	_pLayerMoved: function(x, y) {
		if (!this._movingPLayer && this._ratio && this._offset) {
			if (this._relocateAnimation) {
				this._stopRelocateMove = true;
			}
			
			// locate content
			if (this._relocateAnimation || this._centerAnimation || this._centerPause) {
				var ol = this._view.offsetLeft + this._view.clientLeft;
				var ot = this._view.offsetTop + this._view.clientTop;
			} else {
				var ol = this._offset.l;
				var ot = this._offset.t;
			}
			
			this._setLocation(x * this._ratio + ol, y * this._ratio + ot);
		}
	},
	
	_onMoved: function(mover, leftTop) {
		this._setLocation(leftTop.l, leftTop.t);
	},
	
	_setLocation: function(x, y) {
		this._x = x;
		this._y = y;
		this.movePanContainerNode(x, y);
	},
	
	movePanContainerNode: function(x, y) {
		this.panContainerNode.style.top = y + "px";
		this.panContainerNode.style.left = x + "px";
	},
	
	_onViewMoved: function(mover, leftTop) {
		this._autoPan = {};
		
		if (leftTop.l < 0) {
			this._autoPan.l = this.autoPanStep;
		} else if ((leftTop.l + this._view.offsetWidth + 1) > this._overview.offsetWidth) {
			this._autoPan.l = -this.autoPanStep;
		}
		
		if (leftTop.t < 0) {
			this._autoPan.t = this.autoPanStep;
		} else if ((leftTop.t + this._view.offsetHeight + 1) > this._overview.offsetHeight) {
			this._autoPan.t = -this.autoPanStep;
		}
		
		if (!this._autoPan.t && !this._autoPan.l) {
			if (this._autoPanHandle) {
				clearInterval(this._autoPanHandle);
				delete this._autoPan;
				delete this._autoPanHandle;
			}
		} else if (!this._autoPanHandle) {
			this._autoPanHandle = setInterval(dojo.hitch(this, function() {
				var loc = this.getLocation();
				var x = loc.x;
				var y = loc.y
				if (this._autoPan.l) {
					this._cView.style.left = this._cView.offsetLeft + this._autoPan.l + "px";
					x = loc.x + this._autoPan.l;
				}
				
				if (this._autoPan.t) {
					this._cView.style.top = this._cView.offsetTop + this._autoPan.t + "px";
					y = loc.y + this._autoPan.t;
				}
				
				this._setLocation(x, y);
				
				if (this.isRealTime) {
					this._onRealTimeViewMoved();
				}
			}), this.autoPanInterval);
		}
	},
	
	_centerView: function() {
		var width = this._viewSize.w;
		var height = this._viewSize.h;
		
		var vStartTop = this._view.offsetTop;
		var vStartLeft = this._view.offsetLeft;
		
		var vEndLeft = this._offset.l - this._view.clientLeft;
		var vEndTop = this._offset.t - this._view.clientTop;
		
		if (vStartTop == vEndTop && vStartLeft == vEndLeft && this._resizeHandle) {
			this._resizeHandle.domNode.style.display = "";
			return;
		}
		
		var lastTop = vStartTop;
		var lastLeft = vStartLeft;
		
		var duration = Math.min(Math.max(Math.abs(vEndTop - vStartTop), Math.abs(vEndLeft - vStartLeft)) * this.animationStepDuration, MAX_DURATION);
		var args = {
			node: this._view,
			duration: duration,
			properties: {
				left: {
					end: vEndLeft,
					unit: "px"
				},
				top: {
					end: vEndTop,
					unit: "px"
				}
			},
			onEnd: dojo.hitch(this, function() {
				delete this._centerAnimation;
				if (this._resizeHandle) {
					this._resizeHandle.domNode.style.display = "";
				}
				this.onEndCenterView();
			}),
			onAnimate: dojo.hitch(this, function(prop) {
				var l = Math.round(parseFloat(prop.left));
				var t = Math.round(parseFloat(prop.top));
				this._cView.style.left = l + "px";
				this._cView.style.top = t + "px";
				
				var loc = this.getLocation();
				this._setLocation(loc.x + l - lastLeft, loc.y + t - lastTop);
				lastLeft = l;
				lastTop = t;
			})
		};
		this._centerPause = setTimeout(dojo.hitch(this, function() {
			delete this._centerPause;
			if (!this._relocateAnimation) {
				this._centerAnimation = dojo.animateProperty(args).play();
			}
		}), 200);
	},
	
	_onRealTimeViewMoved: function() {
		var loc = this.getLocation();
		var x = loc.x - this._view.offsetLeft - 1;
		var y = loc.y - this._view.offsetTop - 1;
		this._setPannableLayerLocation(Math.round(x / this._ratio), Math.round(y / this._ratio));
	},
	
	_onViewMoveStart: function() {
		if (this._resizeHandle) {
			this._resizeHandle.domNode.style.display = "none";
		}
		this._movingPLayer = true;
		this._moving = true;
		if (this._centerPause) {
			clearTimeout(this._centerPause);
			delete this._centerPause;
		}
		if (this._centerAnimation) {
			this._centerAnimation.stop();
			delete this._centerAnimation;
		}
	},
	
	_onViewMoveStop: function() {
		this._moving = false;
		this._stopRelocateMove = false;
		
		if (this._autoPanHandle) {
			clearInterval(this._autoPanHandle);
			delete this._autoPan;
			delete this._autoPanHandle;
		}
		
		if (this._relocateAnimation) {
			this._relocateAnimation.stop();
			delete this._relocateAnimation;
		}
		
		var cvStartTop = this._cView.offsetTop;
		var cvStartLeft = this._cView.offsetLeft;
		
		var cvEndTop = this._view.offsetTop;
		var cvEndLeft = this._view.offsetLeft;
		
		if (cvStartTop == cvEndTop && cvStartLeft == cvEndLeft) {
			return;
		}
		
		var duration = Math.min(Math.max(Math.abs(cvEndTop - cvStartTop), Math.abs(cvEndLeft - cvStartLeft)) * this.animationStepDuration, MAX_DURATION);
		var args = {
			node: this._cView,
			duration: duration,
			properties: {
				left: {
					end: cvEndLeft,
					unit: "px"
				},
				top: {
					end: cvEndTop,
					unit: "px"
				}
			},
			onEnd: dojo.hitch(this, function() {
				this._stopRelocateMove = false;
				delete this._relocateAnimation;
				if (!this._moving) {
					this._centerView();
				}
			})
		};
		
		if (!this.isRealTime) {
			var pLocation = this.pannableLayer.getLocation();
			var pStartTop = pLocation.y;
			var pStartLeft = pLocation.x;
			
			var ratio = this._ratio;
			
			args.onAnimate = dojo.hitch(this, function(prop) {
				if (!this._stopRelocateMove) {
					var movedLeft = parseFloat(prop.left) - cvStartLeft;
					var movedTop = parseFloat(prop.top) - cvStartTop;
					this.movingOutlineView = true;
					this._setPannableLayerLocation(pStartLeft - Math.round(movedLeft / ratio),
							pStartTop - Math.round(movedTop / ratio));
					this.movingOutlineView = false;
				}
			});
		}
		
		this._relocateAnimation = dojo.animateProperty(args).play();
	},
	
	_onContentMoveStart: function() {
		if (this._resizeHandle) {
			this._resizeHandle.domNode.style.display = "none";
		}
		var loc = this.getLocation();
		this._cMb = dojo.marginBox(this.panContainerNode);
		this._cLeftTop = {
			l: loc.x,
			t: loc.y
		};
		this._cvLeftTop = {
			l: this._cView.offsetLeft,
			t: this._cView.offsetTop
		};
		this._stopMoving();
		this._moving = true;
	},
	
	_onContentMove: function(mover, leftTop) {
		var newLeftTop = {
			l: leftTop.l - this._cMb.l + this._cLeftTop.l,
			t: leftTop.t - this._cMb.t + this._cLeftTop.t
		};
		this.movePanContainerNode(newLeftTop.l, newLeftTop.t);
		
		this._cView.style.left = this._cvLeftTop.l + newLeftTop.l + "px";
		this._cView.style.top = this._cvLeftTop.t + newLeftTop.t + "px";
		
		if (this.isRealTime) {
			this._onRealTimeViewMoved();
		}
	},
	
	_onContentMoveStop: function() {
		delete this._cMb;
		this._onViewMoveStop();
	},
	
	_stopMoving: function() {
		if (this._relocateAnimation) {
			this._relocateAnimation.stop();
			delete this._relocateAnimation;
		}
		if (this._centerPause) {
			clearTimeout(this._centerPause);
			delete this._centerPause;
		}
		if (this._centerAnimation) {
			this._centerAnimation.stop();
			delete this._centerAnimation;
		}
	},
	
	_setPannableLayerLocation: function(x,y) {
		this._movingPLayer = true;
		this.pannableLayer.setLocation(x,y);
		this._movingPLayer = false;
	},
	
	onEndCenterView: function() {
	}
});

dojo.declare("rmpw.panner.Outline._ResizeHandle", dojox.layout.ResizeHandle, {
	
	outline: null,
	maxSize: null,
	
	_prevCoords: null,
	
	_getNewCoords: function(e) {
		var result = this.inherited(arguments);
		if (result === false) {
			return false;
		}
		
		var resizeWidth = false;
		
		if (this.maxSize) {
			result.w = Math.min(result.w, this.maxSize.w);
		}
		
		result.h = (result.w - this.startSize.w) / this.startSize.w * this.startSize.h + this.startSize.h;
		
		if (this.minSize && result.h < this.minSize.h){
			result.h = this.minSize.h;
			resizeWidth = true;
		}
		
		if (this.maxSize && result.h > this.maxSize.h) {
			result.h = this.maxSize.h;
			resizeWidth = true;
		}
		
		if (resizeWidth) {
			result.w = (result.h - this.startSize.h) / this.startSize.h * this.startSize.w + this.startSize.w;
		}
		
		if (this._prevCoords && this._prevCoords.w == result.w && this._prevCoords.h == result.h) {
			return false;
		}
		this._prevCoords = result;
		return result;
	},
	
	_beginSizing: function(e) {
		delete this._prevCoords;
		this.outline._stopMoving();
		if (this.outline._scaleAnimation) {
			this.outline._scaleAnimation.stop();
			delete this.outline._scaleAnimation;
		}
		
		this.outline._setRealTime(false);
		
		this.inherited(arguments);
		
		dojo.stopEvent(e);
	},
	
	_endSizing: function(e) {
		this.outline._setRealTime(this.outline.isRealTime);
		this.inherited(arguments);
	},
	
	_changeSizing: function(e) {
		var tmp = this._getNewCoords(e);
		if (tmp === false) {
			return;
		}
		dojo.style(this.targetDomNode, {
			"width": tmp.w + "px",
			"height": tmp.h + "px"
		});
		this.onChange();
	},
	
	onChange: function() {}
});

})();