/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.PagingControl");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.html");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.ui", "PagingControlMessages");

dojo.declare("rmps.ui.PagingControl", [dijit._Widget, dijit._Templated], {
	
	templateString: dojo.cache("rmps.ui", "templates/PagingControl.html"),
	
	messages: dojo.i18n.getLocalization("rmps.ui", "PagingControlMessages"),
	
	// properties
	size: 0,
	page: 0,
	pageSize: -1,
	limit: -1,
	item: null,
	hasDetails: false,
	
	// private properties
	_nextHandle: null,
	_previousHandle: null,
	
	_nextDetailsHover: null,
	_previousDetailsHover: null,
	
	postCreate: function() {
		this._createDetailsButton(this.previousDetailsNode);
		this._createDetailsButton(this.nextDetailsNode);
		
		this._refresh();
	},
	
	_createDetailsButton: function(node) {
		var a = dojo.doc.createElement("a");
		var img = dojo.doc.createElement("img");
		img.src = dojo.moduleUrl("rmps.ui", "graphics/icons/popup-down-arrow.gif");
		img.alt = "";
		img.className = "details";
		a.appendChild(img);
		node.appendChild(a);
	},
	
	destroy: function() {
		if (this._nextDetailsHover) {
			this._nextDetailsHover.destroy();
		}
		if (this._previousDetailsHover) {
			this._previousDetailsHover.destroy();
		}
		this.inherited(arguments);
	},
	
	getUpperBound: function() {
		return this.limit == -1 ? this.size : Math.min(this.size, this.limit);
	},
	
	getPageSize: function() {
		return this.pageSize != -1 ? this.pageSize : this.size;
	},
	
	update: function(page, pageSize, limit, size, item) {
		if (size != null) {
			this.size = size;
		}
		if (limit != null) {
			this.limit = limit;
		}
		if (item != null) {
			this.item = item;
		}
		
		var start = this.pageSize == -1 ? 0 : this.pageSize * this.page + 1;
		if (pageSize != null) {
			this.pageSize = pageSize;
		}
		
		if (this.pageSize == -1) {
			this.page = 0;
			
		} else if (this.item != null) {
			this.page = Math.floor((this.item - 1) / this.pageSize);
			
		} else {
			if (page != null) {
				this.page = page;
				if (page * this.pageSize > this.getUpperBound()) {
					this.page = 0;
				}
			} else {
				this.page = Math.floor(start / this.pageSize);
			}
		}
		this._refresh();
	},
	
	_refresh: function() {
		if (this._nextDetailsHover) {
			this._nextDetailsHover.close();
			this._nextDetailsHover.destroyRecursive();
			delete this._nextDetailsHover;
		}
		if (this._previousDetailsHover) {
			this._previousDetailsHover.close();
			this._previousDetailsHover.destroyRecursive();
			delete this._previousDetailsHover;
		}
		
		if (this.hasNext()) {
			this._enableNext();
		} else {
			this._disableNext();
		}
		
		if (this.hasPrevious()) {
			this._enablePrevious();
		} else {
			this._disablePrevious();
		}
		
		this._refreshInfo();
	},
	
	_refreshInfo: function() {
		var pageSize = this.getPageSize();
		
		var boldSize = dojo.doc.createElement("b");
		boldSize.innerHTML = this.size;
			
		var info = dojo.doc.createElement("span");
		
		if (this.item != null) {
			var boldItem = dojo.doc.createElement("b");
			boldItem.innerHTML = this.item;
			
			info.appendChild(boldItem);
			info.innerHTML += "&nbsp;";
		}
		
		if (this.item == null || pageSize < this.size) {
			if (this.item != null) {
				info.appendChild(dojo.doc.createTextNode(this.messages._in));
				info.innerHTML += "&nbsp;";
			}
			
			var start = this.getUpperBound() == 0 ? 0 : this.page * pageSize + 1;
			var end = Math.min(this.page * pageSize + pageSize, this.getUpperBound());
			
			var boldStart = dojo.doc.createElement("b");
			boldStart.innerHTML = start;
			
			var boldEnd = dojo.doc.createElement("b");
			boldEnd.innerHTML = end;
		
			info.appendChild(boldStart);
			info.innerHTML += "&nbsp;-&nbsp;";
			info.appendChild(boldEnd);
			info.innerHTML += "&nbsp;";
		}
		
		info.appendChild(dojo.doc.createTextNode(this.messages.of));
		info.innerHTML += "&nbsp;";
		info.appendChild(boldSize);
			
		if (this.limit != -1 && this.size > this.limit) {
			info.innerHTML += "&nbsp;(" + this.size + ")";
		}
		
		dojo.html._setNodeContent(this.infoNode, info, true);
	},
	
	_disablePrevious: function() {
		this.previousDetailsNode.style.display = "none";
		dojo.addClass(this.previousNode, "disabled");
		dojo.html._setNodeContent(this.previousNode, dojo.doc.createTextNode(this.messages.previous), true);
	},
	
	_disableNext: function() {
		this.nextDetailsNode.style.display = "none";
		dojo.addClass(this.nextNode, "disabled");
		dojo.html._setNodeContent(this.nextNode, dojo.doc.createTextNode(this.messages.next), true);
	},
	
	_enablePrevious: function() {
		var link = dojo.doc.createElement("a");
		if (this._previousHandle) {
			this.disconnect(this._previousHandle);
		}
		dojo.connect(link, "onclick", dojo.hitch(this, this.onPreviousClick));
		link.href = this.getPreviousHref();
		link.appendChild(dojo.doc.createTextNode(this.messages.previous));
		
		if (this.item == null && this.pageSize != 1) {
			var start = Math.max(1, this.page * this.pageSize - this.pageSize + 1);
			var end = this.page * this.pageSize;
			link.title = dojo.string.substitute(this.messages.previous_title, [start, end]);
			
		} else if (this.item != null) {
			var label = this.getPreviousItemLabel();
			if (label) {
				link.title = dojo.string.substitute(this.messages.previous_details, [label]);
			}
		}
		
		if (this.hasDetails) {
			this._previousDetailsHover = new jazz.ui.HoverPopup({
				around: this.previousDetailsNode,
				createContents: dojo.hitch(this, this.getPreviousDetails)
			});
			this.previousDetailsNode.style.display = "";
		}
		
		dojo.removeClass(this.previousNode, "disabled");
		dojo.html._setNodeContent(this.previousNode, link, true);
	},
	
	_enableNext: function() {
		var link = dojo.doc.createElement("a");
		if (this._nextHandle) {
			this.disconnect(this._nextHandle);
		}
		dojo.connect(link, "onclick", dojo.hitch(this, this.onNextClick));
		link.href = this.getNextHref();
		link.appendChild(dojo.doc.createTextNode(this.messages.next));
		
		if (this.item == null && this.pageSize != 1) {
			var start = this.page * this.pageSize + this.pageSize + 1;
			var end = Math.min(this.page * this.pageSize + (this.pageSize * 2), this.getUpperBound());
			link.title = dojo.string.substitute(this.messages.next_title, [start, end]);
		
		} else if (this.item != null) {
			var label = this.getNextItemLabel();
			if (label) {
				link.title = dojo.string.substitute(this.messages.next_details, [label]);
			}
		}
		
		if (this.hasDetails) {
			this._nextDetailsHover = new jazz.ui.HoverPopup({
				around: this.nextDetailsNode,
				createContents: dojo.hitch(this, this.getNextDetails)
			});
			this.nextDetailsNode.style.display = "";
		}
		
		dojo.removeClass(this.nextNode, "disabled");
		dojo.html._setNodeContent(this.nextNode, link, true);
	},
	
	// event handlers
	
	onNextClick: function(e) {
		var href = this.getNextHref();
		if (!href || href.length == 0) {
			e.preventDefault();
			if (this.item != null) {
				var pageSize = this.getPageSize();
				var end = Math.min(this.page * pageSize + pageSize, this.getUpperBound());
				var page = this.item + 1 > end ? this.page + 1 : null;
				this.update(page, null, null, null, this.item + 1);
				
			} else {
				this.update(this.page + 1);
			}
		} else {
			// scroll to top of window
			window.scrollTo(0, 0);
		}
	},
	
	onPreviousClick: function(e) {
		var href = this.getPreviousHref();
		if (!href || href.length == 0) {
			e.preventDefault();
			if (this.item != null) {
				var start = this.getUpperBound() == 0 ? 0 : this.page * this.getPageSize() + 1;
				var page = this.item - 1 < start ? this.page - 1 : null;
				this.update(page, null, null, null, this.item - 1);
				
			} else {
				this.update(this.page - 1);
			}
		} else {
			// scroll to top of window
			window.scrollTo(0, 0);
		}
	},
	
	// overridable functions
	
	hasNext: function() {
		if (this.item != null) {
			return this.item != this.getUpperBound();
		}
		var pageSize = this.getPageSize();
		return (this.page * pageSize + pageSize) < this.getUpperBound();
	},
	
	hasPrevious: function() {
		if (this.item != null) {
			return this.item != 1;
		}
		return this.page != 0 && this.getUpperBound() != 0;
	},
	
	getPreviousHref: function() {
		return '';
	},
	
	getNextHref: function() {
		return '';
	},
	
	getNextItemLabel: function() {
		
	},
	
	getPreviousItemLabel: function() {
		
	},
	
	getNextDetails: function() {
		
	},
	
	getPreviousDetails: function() {
		
	}

});

})();