/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.panner.PannableLayer");

dojo.require("dijit._Templated");
dojo.require("rmpw.panner.LayerContainer");
dojo.require("dojo.dnd.Moveable");
dojo.require("rmpw.diagrams.render.util.Events");

dojo.declare("rmpw.panner.PannableLayer", [rmpw.panner.LayerContainer, dijit._Templated], {
	
	templateString: '<div style="position: absolute; width: 100%; height: 100%;">' +
			'<div dojoAttachPoint="panContainerNode"></div>' +
			'<div dojoAttachPoint="containerNode"></div>' +
		'</div>',
	
	// attach points
	panContainerNode: null,
	
	// properties
	moveable: null,
	
	topics: rmpw.diagrams.render.util.Events.EventTopics,
	
	// private properties
	_moving: null,
	_px: 0,
	_py: 0,
	
	postCreate: function() {
		this.inherited(arguments);
		
		this.panContainerNode.style.position = "absolute";
		this.panContainerNode.style.left = "0px";
		this.panContainerNode.style.top = "0px";
		
		this._setDefaultCursor();
		
		this.loc = this.getLocation();
	},
	
	handleMotion: function(e){
		this.ax = e.accelerationIncludingGravity.x;
		this.ay = e.accelerationIncludingGravity.y;
	},
	
	motionPanner: function(){
		
		if( this.locked == false ){
		
			var landscapeOrientation = window.innerWidth/window.innerHeight > 1;
			
			if( landscapeOrientation ){
				
				this.vx = this.vx + this.ay;
				this.vy = this.vy + this.ax;
			
			} else {
				
				this.vy = this.vy - this.ay;
				this.vx = this.vx + this.ax;
			}
			
			this.vx = this.vx * 0.98;
			this.vy = this.vy * 0.98;
			this.y = parseInt( this.y + this.vy / 50 ) * 0.9;
			this.x = parseInt( this.x + this.vx / 50 ) * 0.9;
					
			this.loc.y = this.loc.y + this.y;
			this.loc.x = this.loc.x + this.x;
			
			this.setLocation( this.loc.x, this.loc.y );
		}
	},
	
	unlock: function(){
		this.locked = false;
	},
	
	lock: function(){
		this.locked = true;
	},
	
	destroy: function() {
		this.inherited(arguments);
	},
	
	resize: function(size) {
		// Intentionally blank.  See WI33654
	},
	
	_onFirstMove: function() {
		this._setDragCursor();
	},
	
	_onMoveStart: function() {
		this._moving = true;
		dojo.publish("/gfx/move/start");
		this._setMoveCursor();
	},
	
	_onMoveStop: function() {
		this._moving = false;
		this._setDefaultCursor();
		dojo.publish("/gfx/move/stop");
	},
	
	_onMoved: function(mover, leftTop) {
		this.setLocation(leftTop.l, leftTop.t);
	},
	
	_setDragCursor: function() {
	},
	
	_setMoveCursor: function() {
		this.domNode.style.cursor = "move";
	},
	
	_setDefaultCursor: function() {
		this.domNode.style.cursor = "";
	},
	
	isMoving: function() {
		return this._moving;
	},
	
	setLocation: function(x, y, fromScale, oneLayer) {
		this.inherited(arguments);
		
		for (var i = 0, len = this._layers.length; i < len; ++i) {
			if (this._layers[i].setLocation) {
				this._layers[i].setLocation(x, y, fromScale);
				// if DiagramWidget already broadcasts
				//  new location to other layers
				if (oneLayer) {
					break;
				}
			}
		}
	},
	
	setScale: function(scale, cx, cy, origin ) {
		var prevScale = this.getScale();
		
		if(scale == NaN){
			scale = 1.0;
		}
		
		if(prevScale == NaN){
			prevScale = 1.0;
		}
		
		if (prevScale != scale) {
			var ratio = scale / prevScale;
			var loc = this.getLocation();
			var viewBox = this.getViewBox();
			
			/* scale from 0,0 if origin set */
			
			if( origin != null ){
				var cvw = 0;
				var cvh = 0;
			} else {
				var cvw = viewBox.w / 2;
				var cvh = viewBox.h / 2;
			}
			
			if (cx != null && cy != null) {
				var ox = cx + loc.x - cvw
				var oy = cy + loc.y - cvh;
			} else {
				var ox = oy = 0;
			}
			
			var x = (loc.x - cvw - ox) * ratio + cvw;
			var y = (loc.y - cvh - oy) * ratio + cvh;
			
			if (loc.x != x || loc.y != y) {
				this.setLocation(x, y, true);
			}
			
			this.inherited(arguments);
		}
	}
});

dojo.declare("rmpw.panner.PannableLayer._Moveable", dojo.dnd.Moveable, {
	
	enabled: true,
	
	onDragDetected: function() {
		if (this.enabled) {
			this.inherited(arguments);
		}
	}
});

dojo.declare("rmpw.panner.PannableLayer._Mover", dojo.dnd.Mover, {
	/**
	 * Workaround for bug 63291
	 */
	onMouseUp: function(e) {
		this.destroy();
		dojo.stopEvent(e);
	}
});

})();