/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.panner.PerimeterTracker");

dojo.require("rmpw.panner.TransparentLayer");

dojo.declare("rmpw.panner.PerimeterTracker", rmpw.panner.TransparentLayer, {
	
	pannableLayer: null,
	
	inset: 5,
	
	// private properties
	_trackers: null,
	
	constructor: function() {
		this._trackers = [];
	},
	
	postCreate: function() {
		this.inherited(arguments);
		this.connect(this.pannableLayer, "setLocation", this._update);
		this.connect(this.pannableLayer, "setScale", this._update);
	},
	
	destroy: function() {
		for (var i = 0, len = this._trackers.length; i < len; ++i) {
			this._trackers[i].destroy();
		}
		this._trackers = [];
		
		this.inherited(arguments);
	},
	
	addTracker: function(tracker, params) {
		tracker.__handles = [];
		tracker.__handles.push(this.connect(tracker, "startTracking", this._startTracking));
		tracker.__handles.push(this.connect(tracker, "stopTracking", this._stopTracking));
		this._trackers.push(tracker);
		tracker.start();
	},
	
	removeTracker: function(tracker) {
		for (var i = 0, len = this._trackers.length; i < len; ++i) {
			if (this._trackers[i] == tracker) {
				dojo.forEach(tracker.__handles, dojo.hitch(this, this.disconnect));
				delete tracker.__handles;
				tracker.destroy();
				this._trackers.splice(i, 1);
				return;
			}
		}
	},
	
	resize: function(size) {
		this.inherited(arguments);
		this._update();
	},
	
	_update: function() {
		var size = this.getSize();
		if (!size.w || !size.h) {
			return;
		}
		
		var offset = this.pannableLayer.getLocation();
		for (var i = 0, len = this._trackers.length; i < len; ++i) {
			var items = this._trackers[i].getItems();
			for (var j = 0, len2 = items.length; j < len2; ++j) {
				this._relocateItem(items[j], offset);
			}
		}
	},
	
	_relocateItem: function(item, offset) {
		var b = item.getBoundingBox();
		var size = this.getSize();
		if (size.h && size.w &&
				((b.y + offset.y) > size.h ||
				(b.x + offset.x) > size.w ||
				(b.y + offset.y + b.height) < 0 ||
				(b.x + offset.x + b.width) < 0)) {
			
			if (item.__marker.domNode.style.display == "none") {
				item.__marker.domNode.style.visibility = "hidden";
				item.__marker.domNode.style.display = "";
			}
			
			var p = item.getPoint();
			
			var pl = p.x + offset.x;
			var pt = p.y + offset.y;
			
			var x = Math.floor(size.w / 2);
			var y = Math.floor(size.h / 2);
			
			var rx = pl - x;
			var ry = y - pt;
			
			var angleR = Math.atan(ry / rx);
			var angle = angleR / Math.PI * 180;
			
			if (rx < 0) {
				angle += 180;
			} else if (ry < 0) {
				angle += 360;
			}
			
			var w = item.__marker.domNode.offsetWidth;
			var h = item.__marker.domNode.offsetHeight;
			
			var hw = Math.floor(w / 2);
			var hh = Math.floor(h / 2);
			
			if (angle == 0) {
				var l = size.w - this.inset - w;
				var t = y - hh;
			} else if (angle == 90) {
				var l = x - hw;
				var t = this.inset;
			} else if (angle == 180) {
				var l = this.inset;
				var t = y - hh;
			} else if (angle == 270) {
				var l = x - hw;
				var t = size.h - this.inset - h;
			} else {
				var l = (ry > 0 ? 1 : -1) * y / Math.tan(angleR) + x;
				l = Math.max(Math.min(l, size.w - this.inset - w), this.inset);
				
				var t = (rx < 0 ? 1 : -1) * x * Math.tan(angleR) + y;
				t = Math.max(Math.min(t, size.h - this.inset - h), this.inset);
			}
			
			item.__marker.domNode.style.left = l + "px";
			item.__marker.domNode.style.top = t + "px";
			
			if (item.__marker.rotate) {
				item.__marker.rotate(angle);
			}
			
			item.__marker.domNode.style.visibility = "";
		} else {
			item.__marker.domNode.style.display = "none";
		}
	},
	
	_startTracking: function(item) {
		item.__marker = this.createMarker(item);
		
		item.__marker.domNode.style.display = "none";
		item.__marker.domNode.style.position = "absolute";
		this.domNode.appendChild(item.__marker.domNode);
		
		item.__marker.startup();
		
		item.__boundsHandle = this.connect(item, "boundsChanged", function() {
			this._relocateItem(item, this.pannableLayer.getLocation());
		});
		this._relocateItem(item, this.pannableLayer.getLocation());
	},
	
	_stopTracking: function(item) {
		this.disconnect(item.__boundsHandle);
		delete item.__boundsHandle;
		item.__marker.destroyRecursive();
		delete item.__marker;
	},
	
	createMarker: function(item) {
		// clients must override
	}
});

})();