/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() { 
dojo.provide("rmps.ui.QuerySection");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("rmps.ui.CollapsibleSection");
dojo.require("rmps.ui.queryeditor.QueryEditorWidget");
	
dojo.declare("rmps.ui.QuerySection", [dijit._Widget, dijit._Templated], {
	
	// mix-in
	sectionId: null,
	title: null,
	queryUri: null,
	queryWidgetInputData: "{}", // json
	onDirtyCallback: null,
	onCloseCallback: null,
	onCreateHeader: null,
	isReadOnly: false,
	headerData: null,
	
	_qewidget: null,
	
	widgetsInTemplate: true,
	templateString: '<div class="rmps-ui-QuerySection">' +
			'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_section">' +
				'<div class="header" dojoAttachPoint="_header"></div>' +
				'<div dojoAttachPoint="_qewidgetNode"></div>' +
			'</div>' +
		'</div>',
		
	postCreate: function() {
		this._section.setPrimaryTitle(this.title);
		
		this._qewidget = new rmps.ui.queryeditor.QueryEditorWidget({
			inputData: dojo.fromJson(this.queryWidgetInputData),
			queryUri: this.queryUri
		});
		this._qewidgetNode.appendChild(this._qewidget.domNode);
		
		if (this.onDirtyCallback) {
			this._qewidget.onDirty = this.onDirtyCallback;
		}
		
		if (this.onCreateHeader) {
			this.onCreateHeader(this, this._header, this.headerData);
		}
		
		if (!this.isReadOnly && this.onCloseCallback) {
			// create Close command
			this._section.addAction(messages.close, messages.close, dojo.hitch(this, function() {
				this.onCloseCallback(this.sectionId, this.id);
			}));				
		}		
		
		this.inherited(arguments);
	},
	
	destroy: function() {
		if (this._qewidget) {
			this._qewidget.destroy();
			delete this._qewidget;
		}
		
		this.inherited(arguments);
	},
	
	getHeaderNode: function() {
		return this._header;
	},
	
	markDirty: function(key) {
		this._qewidget.onDirty(key);
	},
	
	isDirty: function() {
		return this._qewidget.isDirty();
	},
	
	doSave: function(taskMonitor, synchronous) {
		var def;
		if (!this._qewidget.isDirty()) {
			def = new dojo.Deferred();
			// Just return the associated query model
			def.callback(this.getQueryModel());
			return def;	
		}
		
		// Query is dirty, so create a task for it and save
		var task = taskMonitor.beginTask(dojo.string.substitute(messages.saveChildQueryTask, [this.title]), 1);
		def = this._qewidget.saveQueryData(synchronous);
		def.addCallback(this, function(queryModel) {
			this.queryUri = queryModel.uri;
		});
		
		task.handleDeferredWork(def, 1);
		
		return def;
	},
	
	getQueryModel: function() {
		return this._qewidget.getQuery();
	},
	
	setSecondaryTitle: function(title) {
		this._section.setSecondaryTitle(title);
	}
});

})();