/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.layout.graph.RadialGraphLayout");

dojo.require("rmpw.diagrams.layout.graph.JavaUtil");
dojo.require("rmpw.diagrams.layout.graph.CalculateRadialInfo");
dojo.require("rmpw.diagrams.layout.graph.AbstractGraphLayout");


$RadialLayout$LayoutEstheticsException = function(this$static, message){
  this$static.detailMessage = message;
  return this$static;
};

getClass_LayoutEstheticsException = function(){
  return Lorg_eclipse_gmf_runtime_draw2d_ui_graph_RadialLayout$LayoutEstheticsException_2_classLit;
};

RadialLayout$LayoutEstheticsException = function(){
};

_ = RadialLayout$LayoutEstheticsException.prototype = new RuntimeException();
_.getClass$ = getClass_LayoutEstheticsException;
_.typeId$ = 78;

/**
 * An autolayout algorithm that will arrange a set of nodes in a radial fashion.
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.layout.graph.RadialGraphLayout", rmpw.diagrams.layout.graph.AbstractGraphLayout, {
	
	rootNode: null,
	rootPositionLocked: false,
	startTheta: 0,
	totalTheta: 0,
	
	constructor: function() {
	},

	runLayoutAlgorithm: function(graph) {
		var success = this.runRadialLayout(graph.nodes, graph.edges);
		if (!success) {
			dojo.require("rmpw.diagrams.layout.graph.GMFDirectedGraphLayout");
			var layout = new rmpw.diagrams.layout.graph.GMFDirectedGraphLayout();
			success = layout.runLayoutAlgorithm(graph);
		}
		
		return success;
	},
	
	findRootNode: function(nodes){
		var count;
		count = nodes.size;

		// look for the first node with no incoming edges and some outgoing edges
		var rootNode = null, node;
		for (var i = 0; i<count; i++) {
			node = nodes.array[i];
			if (node.incoming.size == 0 && node.outgoing.size > 0) {
				rootNode = node;
				break;
			}
		}

		if (rootNode == null && count > 0) {
			checkIndex(0, nodes.size);
			rootNode = nodes.array[0];
		}

		return rootNode;
	},
	
	runRadialLayout: function(nodes, edges) {
  		var $e0, edge, graph2, li, op, points, radialLayout;
  		return this.runRadialLayoutWithRoot(this.findRootNode(nodes), nodes, edges);
	},
	
	runRadialLayoutWithRoot: function(rootNode, nodes, edges) {
  		var $e0, edge, graph2, li, op, points, radialLayout, origRect;
  		try {
    		this.layout(rootNode, nodes, 0, 6.283185307179586, false);
      		li = $AbstractList$ListIteratorImpl(new AbstractList$ListIteratorImpl(), edges);
      		while (li.i < li.this$0_0.size_0()) {
        		edge = $next_0(li);
        		points = new rmpw.diagrams.render.util.PointList();
        		points.addPoint_0(edge.source.x + (edge.source.width / 2), edge.source.y + (edge.source.height / 2));
        		points.addPoint_0(edge.target.x + (edge.target.width / 2), edge.target.y + (edge.target.height / 2));
        		edge.points = points;
        		edge.start = points.getPoint( 0);
        		edge.end = points.getPoint( points.size - 1);
      		}
      		
      		return true;
  		}
   		catch ($e0) {  // layout esthetics exception
    		return false;
  		}
	},
	
	layout: function(rootNode, nodes, startTheta, totalTheta, rootPositionLocked){
		this.rootNode = rootNode,
		this.rootPositionLocked = rootPositionLocked,
		this.startTheta = startTheta;
		this.totalTheta = totalTheta;
		
		var childAndSectionMap, circleSectionParts, firstCircleParts, firstCircleShapes, i, increaseTheta, innerRingCount, node, radialInfo, restRestViews, restViews, theta;
		restViews = $NodeList(new NodeList());
		firstCircleParts = $NodeList(new NodeList());
		innerRingCount = this.findChildViews(this.rootNode, nodes, firstCircleParts, restViews);
		theta = 3.141592653589793;
		if (innerRingCount > 1) {
			theta = this.totalTheta / innerRingCount;
		}
		childAndSectionMap = $HashMap_0(new HashMap(), firstCircleParts.size);
		circleSectionParts = null;
		firstCircleShapes = $NodeList(new NodeList());
		for (i = 0; i < firstCircleParts.size; ++i) {
			checkIndex(i, firstCircleParts.size);
			node = firstCircleParts.array[i];
			circleSectionParts = $NodeList(new NodeList());
			restRestViews = $NodeList(new NodeList());
			this.findChildViews(node, restViews, circleSectionParts, restRestViews);
			setCheck(firstCircleShapes.array, firstCircleShapes.size++, node);
			!node?$putNullSlot(childAndSectionMap, circleSectionParts):$putHashValue(childAndSectionMap, node, circleSectionParts, ~~(node.$H || (node.$H = ++sNextHashId)));
			!circleSectionParts?$putNullSlot(childAndSectionMap, restRestViews):$putHashValue(childAndSectionMap, circleSectionParts, restRestViews, ~~circleSectionParts.hashCode$());
		}
		radialInfo = new rmpw.diagrams.layout.graph.CalculateRadialInfo(this.rootNode, firstCircleShapes, theta);
		this.positionRings(firstCircleParts, childAndSectionMap, theta, radialInfo);
		increaseTheta = theta;
		this.positionNextRings(firstCircleParts, childAndSectionMap, increaseTheta);
	},
	
	findChildViews: function(rootNode, nodes, childNodes, restNodes){
		var allSet, connectionEPs, count, el, node, fromEP, i, posViewCount, toEP;
		allSet = $HashSet_1(new HashSet(), nodes.size);
		posViewCount = 0;
		count = nodes.size;
		for (i = 0; i < count; ++i) {
			checkIndex(i, nodes.size);
			node = nodes.array[i];
			if (node != rootNode) {
				$put(allSet.map, node, allSet);
			}
		}
		connectionEPs = $EdgeList(new EdgeList());
		for (i = 0; i < count; ++i) {
			checkIndex(i, nodes.size);
			node = nodes.array[i];
			$addAll_0(connectionEPs, node.outgoing);
			$addAll_0(connectionEPs, node.incoming);
		}
		for (i = 0; i < connectionEPs.size; ++i) {
			checkIndex(i, connectionEPs.size);
			node = connectionEPs.array[i];
			fromEP = node.source;
			toEP = node.target;
			el = null;
			if (fromEP == rootNode) {
				el = toEP;
			}
			else if (toEP == rootNode) {
				el = fromEP;
			}
			if (!!el && $containsKey(allSet.map, el)) {
				setCheck(childNodes.array, childNodes.size++, el);
				++posViewCount;
				$remove_0(allSet.map, el) != null;
			}
		}
		$addAll_0(restNodes, allSet);
		return posViewCount;
	},

	positionNextRings: function(firstCircleParts, childAndSectionMap, theta){
		var dTheta, i, key, n, nextViews, part, parts, posViewCount, radialLayout, restRestViews, startThetaPrim, thetaPrim, totalThetaPrim;
		n = 0;
		for (i = 0; i < firstCircleParts.size; ++i) {
			totalThetaPrim = theta;
			checkIndex(i, firstCircleParts.size);
			part = firstCircleParts.array[i];
			parts = !part?childAndSectionMap.nullSlot:$getHashValue(childAndSectionMap, part, ~~(part.$H || (part.$H = ++sNextHashId)));
			if (!!parts && parts.size != 0) {
				posViewCount = 0;
				n = i;
				while (i + 1 < firstCircleParts.size) {
					checkIndex(i + 1, firstCircleParts.size);
					key = firstCircleParts.array[i + 1];
					nextViews = key == null?childAndSectionMap.nullSlot:key != null && canCast(key.typeId$, 1)?childAndSectionMap.stringMap[':' + key]:$getHashValue(childAndSectionMap, key, ~~hashCode__devirtual$(key));
					if (!!nextViews && nextViews.size == 0) {
						totalThetaPrim = 3.141592653589793 < totalThetaPrim + theta?3.141592653589793:totalThetaPrim + theta;
						++i;
					}
					else 
						break;
				}
				posViewCount = parts.size;
				dTheta = this.startTheta + n * theta;
				thetaPrim = totalThetaPrim / posViewCount;
				if (posViewCount < 2) {
					startThetaPrim = dTheta;
				}
				else {
					startThetaPrim = dTheta - totalThetaPrim / 2 + thetaPrim / 2;
				}
				restRestViews = !parts?childAndSectionMap.nullSlot:$getHashValue(childAndSectionMap, parts, ~~parts.hashCode$());
				if (restRestViews) {
					$addAll_0(parts, restRestViews);
				}
				var nextRadial = new rmpw.diagrams.layout.graph.RadialGraphLayout();
				nextRadial.layout(part, parts, startThetaPrim, totalThetaPrim, true);
			}
		}
	},

	getRadialNodePosition: function(node, theta, radialInfo, useDelta) {
		var ptLocation = new rmpw.diagrams.render.util.Point(0, 0);
		if (radialInfo) {
			ptLocation.x = Math.round(radialInfo.getRadius() * Math.cos(theta));
			ptLocation.y = Math.round(radialInfo.getRadius() * Math.sin(theta));
			if (useDelta)
				ptLocation.translate_1(radialInfo.getDelta());
		}
		ptLocation.translate(~~(-node.width / 2), ~~(-node.height / 2));
		
		return ptLocation;
	},
	
	positionRings: function(firstCircleParts, childAndSectionMap, theta, radialInfo){
		var li, n, node, pos, ptPos, rootPos;
		n = 0;
		if (theta < 0.09817477042468103 && this.rootPositionLocked)
			throw $RadialLayout$LayoutEstheticsException(new RadialLayout$LayoutEstheticsException(), 'Angle is too small to resulting in very large radius');
		this.sortFirstCircleParts(firstCircleParts, childAndSectionMap);
		li = $AbstractList$ListIteratorImpl(new AbstractList$ListIteratorImpl(), firstCircleParts);
		while (li.i < li.this$0_0.size_0()) {
			node = $next_0(li);
			ptPos = this.getRadialNodePosition(node, this.startTheta + n * theta, radialInfo, this.rootPositionLocked);
			node.x = ptPos.x;
			node.y = ptPos.y;
			++n;
		}
		if (!this.rootPositionLocked) {
			ptPos = this.getRadialNodePosition(this.rootNode, 0, null, this.rootPositionLocked);
			this.rootNode.x = ptPos.x;
			this.rootNode.y = ptPos.y;
		}
	},

	sortFirstCircleParts: function(firstCircleParts, childAndSectionMap){
		var addInc, circleList, emptyNextCircleList, firstCircleShapeParts, i, nextCircleList, rest;
		firstCircleShapeParts = $NodeList_0(new NodeList(), firstCircleParts);
		rest = $NodeList(new NodeList());
		emptyNextCircleList = $NodeList(new NodeList());
		nextCircleList = $NodeList(new NodeList());
		for (i = 0; i < firstCircleShapeParts.size; ++i) {
			checkIndex(i, firstCircleShapeParts.size);
			circleList = $get(childAndSectionMap, firstCircleShapeParts.array[i]);
			if (!!circleList && circleList.size_0() == 0)
			{
				checkIndex(i, firstCircleShapeParts.size);
				$add_0(emptyNextCircleList, firstCircleShapeParts.array[i]);
			}
			else 
			{
				checkIndex(i, firstCircleShapeParts.size);
				$add_0(nextCircleList, firstCircleShapeParts.array[i]);
			}
		}
		firstCircleParts.array = initDim(_3Ljava_lang_Object_2_classLit, 84, 0, 0, 0);
		firstCircleParts.size = 0;
		if (nextCircleList.size > 1) {
			addInc = ~~(firstCircleShapeParts.size / nextCircleList.size);
			i = 0;
			while (nextCircleList.size > 0 || emptyNextCircleList.size > 0) {
				if (i % addInc == 0 && nextCircleList.size > 0) {
					$add_0(firstCircleParts, $remove_2(nextCircleList, 0));
				}
				else {
					if (emptyNextCircleList.size > 0)
						$add_0(firstCircleParts, $remove_2(emptyNextCircleList, 0));
				}
				++i;
			}
		}
		else 
			$addAll_0(firstCircleParts, firstCircleShapeParts);
		$addAll_0(firstCircleParts, rest);
	}
});