/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.render.RenderRoot");

dojo.require("rmpw.common.policy.Host");
dojo.require("rmpw.common.transform.InvalidationController");
dojo.require("rmpw.diagrams.render.util.Workarounds");
dojo.require("rmpw.diagrams.render.Viewport");
dojo.require("rmpw.diagrams.render.graph.BaseGraphConfigurator");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.graphics.ui", "DiagramEditorMessages");

(function(){
	var messages = dojo.i18n.getLocalization("rmps.graphics.ui", "DiagramEditorMessages");
	
	dojo.declare("rmpw.diagrams.render.RenderRoot", rmpw.common.policy.Host , {
		
		diagram: null,
		fixedDiagramSize: null, // related to attribute ModelDiagramOutline::_usingFixedDiagram
		
		constructor: function(args) {
			args = args || {};
		    
			this._metadataMap = {}; // add a meta data root where policy data will be stored
			
			this._constructing = true;
		    this.configuration = args.configuration;
		    this.container = args.node;
		    
		    // respond to parent.viewer in role of ViewMapping hierarchy root
		    this.viewer = this; // TODO (MG): temporary shim for mapping/policy host refactor 
		    this.root = this;
		    this.level = 0;
		    
			this.allowAnimation = this.configuration.allowAnimation ? true : this.configuration.allowAnimation === undefined ? true : false;
			this._subScribeAnimation = dojo.subscribe('gef/diagram/animation', dojo.hitch(this, function(allow) {
				this.allowAnimation = allow;
			}));
			
			if (this.configuration.graphConfigurator) {
				this.graphConfigurator = new this.configuration.graphConfigurator();
			}
			else {
				// install the default
				this.graphConfigurator = new rmpw.diagrams.render.graph.BaseGraphConfigurator();
			}
			
			if(dojo.isIE == 8){
				this.surface = dojox.gfx.createSurface(this.container, 1, 1);
			} else {
				this.surface = dojox.gfx.createSurface(this.container, "100%", "100%");
			}
			this.surface.whenLoaded(dojo.hitch(this, this._whenLoaded));
			
			delete this._constructing;
		},
		
		_whenLoaded: function(surface) {
            // create viewport
			this.rearFeedbackLayer = surface.createGroup();
			this.viewport = new rmpw.diagrams.render.Viewport(surface, this.container);
			this.viewport.viewer = this;
			this.feedbackLayer = surface.createGroup();
			this.attachListeners();
			
			this.configure( this.configuration );
		},
		
	    configure: function( config ){
			if( config.store ){
                if(config.feed) {
                    var diagramStore = new config.store(this);
                    diagramStore.fetch(config.feed, config.interpretation, config.tooling);
                }
             }
		},
		
		destroy: function() {
			this.inherited(arguments);
			
			this.clear();
			dojo.unsubscribe(this._subScribeAnimation);
			
			if (this._invalidationController) {
				this._invalidationController.destroy();
			}
		},
		
		setDiagram: function(diagram, diagramWidget) {
			this.diagram = diagram;
			diagram.setParent(this);
			
			// hide diagram until first refresh
			if (!dojo.isOffscreen) {
				this._hideBeforeRefresh(diagram, diagramWidget);
			}
		},
		
		_hideBeforeRefresh: function(diagram, diagramWidget) {
			if (diagramWidget && diagramWidget.outline && diagramWidget.outline.setDiagram) {
				diagramWidget.outline.setDiagram(diagram);
			}
			if (diagram.policies.graphElement && diagram.children.length >= 1 &&
					this.diagram.rootShapes && this.diagram.rootShapes.length > 0) {
				if (diagram.children.length > 1 && this.diagram.policies.graphLayout && this.diagram.policies.graphLayout.hideBeforeLayout &&
						diagramWidget && diagramWidget.showMessage) {
					this.diagram.policies.graphLayout.hideBeforeLayout(diagram, diagramWidget);
				} else {
					diagramWidget.showMessage(messages.loading);
					this.diagram.rootShapes[0].rawNode.style.display = 'none';
					var graphManager = this.diagram.policies.graphElement.getGraphManager();
					this._connectContainerRefresh = dojo.connect(graphManager, "onRefreshComplete", dojo.hitch(this, function() {
						diagramWidget.hideMessage();
						dojo.disconnect(this._connectContainerRefresh);
						this.diagram.rootShapes[0].rawNode.style.display = '';
					}));
				}
			}
		},
	
	
		attachListeners: function() {
			this.listeners = [];
			this.listeners.push(dojo.connect(this.viewport, "setZoom", this, "refreshViewport"));
		},
		
		detachListeners: function() {
			dojo.forEach(this.listeners, dojo.disconnect);
		},
		
		clear: function() {
			this.feedbackLayer.clear();
			this.viewport.contents.clear();
			this.rearFeedbackLayer.clear();
	
			if(this.diagram) {
				this.diagram.destroy();
				this.diagram = null;
			}
			
			this.viewport.reset();
		},
		
		getInvalidationController: function() {
			if(!this._invalidationController) {
				this._invalidationController = new rmpw.common.transform.InvalidationController(this);
			}
			return this._invalidationController;
		},
		
		getMarginBox: function() {
		    if (dojo.isOffscreen) {
		        return {w: 0, h: 0};
		    } else if (this.diagramLayer && this.diagramLayer.rootLayer) {
				return dojo.marginBox(this.diagramLayer.rootLayer.domNode);
			} else {
				return dojo.marginBox(this.container);
			}
		},
		
		getCanvasSize: function() {
			if (this.canvasSize && !this.canvasInFullSize) {
				return this.canvasSize;	
			} else if (this.canvasSize && this.canvasInFullSize) {
				// return whichever is bigger
				var box = this.getMarginBox();		
				return {
					w: Math.max(this.canvasSize.w, box.w),
					h: Math.max(this.canvasSize.h, box.h)
				};
			}
			return this.getMarginBox();
		},
		
		setFixedDiagramSize: function(size) {
			// related to attribute ModelDiagramOutline::_usingFixedDiagram
			this.fixedDiagramSize = size;
		},
		
		scaleDiagram: function(scale) {
			if (this.diagramLayer.outline.isUsingFixedDiagram()) {
				var newSize = {
					w: this.fixedDiagramSize.width * scale,
					h: this.fixedDiagramSize.height * scale
				};
				this.resize(newSize);
			}
		},
		
		resize: function(s){
			if (!s) {
				if (this.canvasInFullSize && this.canvasSize) {
					// no new size is provided and the caller doesn;t want to change canvas size, if it is already set
					// (will enter here when called by DiagramWidget when this is main diagram's root and window is resized) 
					return;
				}
				s = dojo.marginBox(this.container);//this.getMarginBox(); 
			}
			if (this.canvasSize === undefined || (s.w !== this.canvasSize.w || s.h !== this.canvasSize.h)) {
				this.canvasSize = {w: s.w, h: s.h};
				if( s.w && s.h )
				{
					this.surface.setDimensions(s.w, s.h);
				}
			}
		},
					
		setupSizeAfterLayoutComplete: function() {
			var fullSize = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(this.surface);
			if (this.diagramLayer.outline.isUsingFixedDiagram()) {
				if (this.fixedDiagramSize.width <= fullSize.width && this.fixedDiagramSize.height <= fullSize.height) {					
					fullSize = this.fixedDiagramSize;
				} // else, diagram area larger than viewport, ensure scrollbars by usign fullSize
			}
			var newSize = {
				w: fullSize.width,
				h: fullSize.height
			}
			this.resizeAfterLayoutComplete(newSize);
			this.diagramLayer.outline.initialize(newSize);
		},
		
		resizeAfterLayoutComplete: function(size) {
			this.resize(size);
		},
		
		refreshViewport: function() {
			// Intentionally blank.  See WI33654
		},
		
		updateViewport: function(bbox) {
			this.bbox = bbox;
			this.refreshViewport();
		},
		
		invalidate: function() {
			// indication that the contents of the diagram may have changed, needed to implement ViewMapping contract
		}
	});
}());