/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.ResourceDetails");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.html");
dojo.require("dojox.validate.regexp");

dojo.require("jazz.util.html");
dojo.require("jazz.ui.HoverPopup");

dojo.require("rmps.ui.EllipsisLabel");
dojo.require("rmps.ui.EllipsisLabelContainer");
dojo.require("rmps.ui.user.UserLabel");

dojo.require("rmps.core.util.StringUtils");
dojo.require("rmps.core.util.DateUtils");
dojo.require("rmps.core.error");

dojo.requireLocalization("rmps.ui", "ResourceDetailsMessages");

var DateUtils = rmps.core.util.DateUtils;
var BODY_LENGTH = 300;
var BODY_LINES = 5;

var URL_REGEX = dojox.validate.regexp.url({
	scheme: true,
	allowDecimal: false,
	allowHex: false,
	allowDottedHex: false,
	allowDottedOctal: false
});

dojo.declare("rmps.ui.ResourceDetails", [dijit._Widget, dijit._Templated], {
	
	templateString: dojo.cache("rmps.ui", "templates/ResourceDetails.html"),
	widgetsInTemplate: true,
	
	messages: dojo.i18n.getLocalization("rmps.ui", "ResourceDetailsMessages"),
	
	// attach points
	_subject: null,
	_author: null,
	_createdTimestamp: null,
	_contributor: null,
	_updatedTimestamp: null,
	_bodyText: null,
	_bodyTextMore: null,
	_info: null,
	_authorLabel: null,
	_contributorLabel: null,
	_contributorContainer: null,
	
	// mixin properties
	object: null,
	
	// properties
	bodyLength: BODY_LENGTH, // -1 for full length
	bodyLines: BODY_LINES, // only applies if bodyLength != -1
	
	// private properties
	_resourcePopup: null,
	_createdTimestampHandle: null,
	_updatedTimestampHandle: null,
	_popupContents: null,
	_authorInfo: null,
	_contributorInfo: null,
	
	postCreate: function() {
		this.connect(this._authorLabel, "initUserInfo", function(userInfo) {
			this._authorInfo = userInfo;
		});
		
		this._author.initTarget(this._authorLabel.linkNode);
		this.connect(this._contributorLabel, "initUserInfo", function(userInfo) {
			this._contributorInfo = userInfo;
		});
		
		this._contributor.initTarget(this._contributorLabel.linkNode);
	},
	
	refresh: function() {
		if (this.object.created.getTime() == this.object.updated.getTime()) {
			this._contributorContainer.domNode.style.display = "none";
		} else {
			this._contributorContainer.domNode.style.display = "";
			
			if (this._updatedTimestampHandle) {
				DateUtils.removeDateDiffUpdater(this._updatedTimestampHandle);
				delete this._updatedTimestampHandle;
			}
		
			var deferred = this.object.getContributor();
			deferred.addCallback(this, this._updateContributor);
			deferred.addErrback(function(error) {
				if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
					rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
				} else {
					rmps.core.error.createError(error).dispatch();
				}
			});
			
			// time stamp
			this._updatedTimestampHandle = DateUtils.addDateDiffUpdater(this.object.updated, dojo.hitch(this, function(newDateDiff) {
				this._updatedTimestamp.setLabel(newDateDiff);
			}));
		}
		
		this.refreshLabel();
		
		// body
		if (this._popupContents) {
			this._popupContents.destroyRecursive();
			delete this._popupContents;
		}
		if (this._resourcePopup) {
			this._resourcePopup.destroy();
			delete this._resourcePopup;
		}
		
		this.refreshDescription();		
		this.refreshInfo();
	},
	
	startup: function() {
		var deferred = this.object.getCreator();
		deferred.addCallback(this, this._updateAuthor);
		deferred.addErrback(function(error) {
			if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
				rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
			} else {
				rmps.core.error.createError(error).dispatch();
			}
		});
		
		// time stamp
		this._createdTimestampHandle = DateUtils.addDateDiffUpdater(this.object.created, dojo.hitch(this, function(newDateDiff) {
			this._createdTimestamp.setLabel(newDateDiff);
		}));
			
		this.inherited(arguments);
		this.refresh();
	},
	
	_updateAuthor: function(user) {
		this._authorLabel.setUser(user);
		this._author.setLabel(user.name);
		if (this._resourcePopup && !this._resourcePopup._isOpen) {
			this._resourcePopup.destroy();
			this._resourcePopup = this.createResourcePopup(this._bodyTextMore);
		}
	},
	
	_updateContributor: function(user) {
		this._contributorLabel.setUser(user);
		this._contributor.setLabel(user.name);
		if (this._resourcePopup && !this._resourcePopup._isOpen) {
			this._resourcePopup.destroy();
			this._resourcePopup = this.createResourcePopup(this._bodyTextMore);
		}
	},
	
	createResourcePopup: function(node) {
		return new jazz.ui.HoverPopup({
			around: node,
			createContents: dojo.hitch(this, function() {
				this._popupContents = this.createPopupContents();
				
				var temp = dojo.doc.createElement("div");
				dijit.popup.moveOffScreen(temp);
				temp.appendChild(this._popupContents.domNode);
				this._popupContents.startup();
				temp.removeChild(this._popupContents.domNode);
				dojo.destroy(temp);
				
				return this._popupContents.domNode;
			})
		});
	},
	
	createPopupContents: function(popupWidget) {
		var details = new rmps.ui.ResourceDetails({
			object: this.object,
			bodyLength: -1
		});
		details.showFullLabels(true);
		return details;
	},
	
	showFullLabels: function(showFull) {
		this._subject.showFullLabel(showFull);
		this._author.showFullLabel(showFull);
		this._contributor.showFullLabel(showFull);
		this._createdTimestamp.showFullLabel(showFull);
		this._updatedTimestamp.showFullLabel(showFull);
	},
	
	destroy: function() {
		if (this._popupContents) {
			this._popupContents.destroyRecursive();
			delete this._popupContents;
		}
		if (this._resourcePopup) {
			this._resourcePopup.destroyRecursive();
			delete this._resourcePopup;
		}
		if (this._createdTimestampHandle) {
			DateUtils.removeDateDiffUpdater(this._createdTimestampHandle);
			delete this._createdTimestampHandle;
		}
		if (this._updatedTimestampHandle) {
			DateUtils.removeDateDiffUpdater(this._updatedTimestampHandle);
			delete this._updatedTimestampHandle;
		}
		this.destroyInfo();
		this.inherited(arguments);
	},
	
	resize: function() {
		var size = { w: this.domNode.offsetWidth };
		this._subject.resize(size);
		this._authorContainer.resize(size);
		this._contributorContainer.resize(size);
	},
	
	destroyInfo: function() {
		if (this._contributions) {
			for (var i = 0; i < this._contributions.length; ++i) {
				this._contributions[i].destroyRecursive();
			}
			delete this._contributions;
		}
	},
	
	refreshLabel: function(){
		this._subject.setLabel(this.object.subject);
	},
	
	refreshDescription: function(){
		/*To be implemnted by sub classes */
	},
	
	refreshInfo: function() {
		/*To be implemnted by sub classes */
	},
	
	createBodyHtml: function(text) {
		var skipWordWrap = dojo.isIE || dojo.isFF >= 3.1;
		var embeddedText = "";
		var word = "";
		var prevC = null;
		var lines = 1;
		var len = 0;
		var clipped = false;
		
		text = text.replace(/^(\s*?\n)*/, '').replace(/\s\s*$/, '');
		
		for (var i = 0; i < text.length; i++) {
			var c = text.charAt(i);
			if (c == " " || c == "\n" || i == text.length - 1) {
				if (i == text.length -1) {
					word += c;
				}
				
				var escaped = jazz.util.html.escape(word);
				if (this.bodyLength != -1 && (len + escaped.length) > this.bodyLength) {
					escaped = escaped.substr(0, this.bodyLength - len) + "...";
					clipped = true;
					len += 3;
				}
				len += escaped.length;
				
				if (!skipWordWrap && escaped.length > 20) {
					// add word breaks for long words
					escaped = this._breakUpWord(escaped);
				}
				
				if (word.match(URL_REGEX)) {
					embeddedText += '<a href="' + word + '" target="_blank">' + escaped + '</a>';
				} else {
					embeddedText += escaped;
				}
				
				if (clipped) {
					break;
				}
				
				if (c == " ") {
					if (c == prevC) {
						embeddedText += "&nbsp;";
					} else {
						embeddedText += c;
					}
					len++;
				} else if (c == "\n") {
					if (this.bodyLength != -1 && ++lines > this.bodyLines) {
						clipped = true;
						embeddedText += "...";
						len += 3;
						break;
					}
					embeddedText += "<br>";
				}
				word = "";
			} else {
				word += c;
			}
			prevC = c;
		}
		return {innerHTML: embeddedText, more: clipped};
	},
	
	_breakUpWord: function(word) {
		var result = "";
		for (var j = 0; j < word.length; j++) {
			result += word.charAt(j) + "<wbr></wbr>";
		}
		return result;
	},
	
	// more events
	
	_onMoreClicked: function(e) {
		dojo.stopEvent(e);
		if (this._resourcePopup && !this._resourcePopup._isOpen) {
			this._resourcePopup.open();
		}
	}

});

})();