/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.doc = document;
dojo.isIE = true; // required for text handling
dojo.isOffscreen = true; // helps runtime detect scenarios for offscreen rendering

require.defaultEval = require.eval;

require.eval = function(text, hint) {
	if (hint) {
		return javaScriptFileLoader.evaluateFileContents(globalRhinoContext, text, hint);
	}
	
	return require.defaultEval(text, hint);
};

initializeHttpArguments = function(args) {
	var firstParam = true;
	var fullURL = args.url;
	
	// add stream context and project ID
	if (baselineURL != null) {
		args.content["rmps.context"] = baselineURL;
	}
	if (projectID != null) {
		args.content.projectId = projectID;
	}
	if (changeset != null) {
		args.content.changeset = changeset;
	}
	
	for (var paramName in args.content) {
		var param = args.content[paramName];
		if (dojo.isString(param)) { 
			if (httpServiceWrapper.isURI(param)) {
				param = httpServiceWrapper.decode(param); // ensure URI is decoded
			}
			param = encodeURIComponent(param);
		}
		
		fullURL += (firstParam ? "?" : "&") + paramName + "=" + param;
		firstParam = false;
	}
	
	var headerKeys = [];
	var headerValues = [];
	var i=0;
	for (var key in args.headers) {
		headerKeys[i] = key;
		headerValues[i] = args.headers[key];
		i++;
	}
	
	return {fullURL: fullURL, headerKeys: headerKeys, headerValues: headerValues};
};

// Redirect xhrGet to java code which will call the services directly
var jazzClient = dojo.getObject('jazz.client', true);
jazzClient.xhrGet = dojo.xhrGet = function(args) {
	var xhrParms = initializeHttpArguments(args);
	var result = httpServiceWrapper.xhrGet(xhrParms.fullURL, xhrParms.headerKeys, xhrParms.headerValues);
	if (result) {
		args.load(result.content, result.headers);
	}
};

jazzClient.xhrPut = dojo.xhrPut = function(args) {
	var xhrParms = initializeHttpArguments(args);
	var result = httpServiceWrapper.xhrPut(xhrParms.fullURL, xhrParms.headerKeys, xhrParms.headerValues);
	if (result) {
		args.load(result, result.headers);
	}
};

jazzClient.xhrPost = dojo.xhrPost = function(args) {
	var xhrParms = initializeHttpArguments(args);
	var result = httpServiceWrapper.xhrPost(xhrParms.fullURL, xhrParms.headerKeys, xhrParms.headerValues);
	if (result) {
		args.load(result.content, result.headers);
	}
};

var SPARQL = new Object();
SPARQL.select = function(select, where) {
	var javaObject = null;
	dojo.xhrGet({
		url: net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/modelvalidation/select",
		handleAs: 'json',
		content: {
			selectClause: select,
			whereClause: where
		},
		load: function(data) {
			javaObject = dojox.json.ref.fromJson(data);
		},
		error: function(error) {
			console.error(error);
		}
	});
	
	return javaObject;
};

dojo.require("dojox.json.ref");


