/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.Sidebar");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.html");

// constants
var CONTENT_AREA_MIN_WIDTH = 200;
var SIDEBAR_SNAP_WIDTH = 80;
var SIDEBAR_MIN_WIDTH = 180;
var SIDEBAR_MAX_WIDTH = 600;
var SIDEBAR_DEFAULT_WIDTH = 240;
var SHOW_HIDE_CTRL_WIDTH = 8;

/*
 * Utility function that performs a dojo.addClass and dojo.removeClass in
 * one step, avoiding intermediate flickering and unnecessary painting. Useful
 * for switching back and forth between classes.
 */
function switchClass(/* Node */ node, /* string */ oldClass, /* string */ newClass) {
	if (!node) {
		return;
	}
	// remove old class if present
	var orig = node.className;
	var cls = orig;
	var t = dojo.trim((" " + cls + " ").replace(" " + oldClass + " ", " "));
	if (cls != t){ cls = t; }

	// add new class if present
	if ((" "+cls+" ").indexOf(" "+newClass+" ") < 0){
		cls += (cls ? ' ' : '') + newClass;
	}
	
	// apply change if needed
	if (cls != orig) {
		node.className = cls;
	}
}

dojo.declare("rmps.ui.Sidebar", [dijit._Widget, dijit._Templated], {
	
	// summary:
	//		A reusable layout widget which features a collapsible sidebar area and a main content area.
	// description:
	//		You can override some of the default styles for this widget; see
	//		resources/templates/sidebar/SidebarLayout.css for details.
	
	templateString: dojo.cache("rmps.ui", "templates/SidebarLayout.html"),
	
	dragResizeDelay: 300,
	
	setMainContent: function(mainContentDomNode) {
		if (this._contentWidget) {
			this._contentWidget.destroyRecursive();
		}
		this._contentWidget = (mainContentDomNode.nodeType == 1 && dijit.byNode(mainContentDomNode));
		dojo.html._setNodeContent(this._contentAreaContainer, mainContentDomNode, true);
	},
	
	setSidebarContent: function(sidebarNode) {
		if (this._sidebarWidget) {
			this._sidebarWidget.destroyRecursive();
		}
		this._sidebarWidget = (sidebarNode.nodeType == 1 && dijit.byNode(sidebarNode));
//		dojo.addClass(this._sidebarWidget.domNode, "sidebarArea-contents");
		
		dojo.html._setNodeContent(this._sidebar, sidebarNode, true);
		
	},
	//
	// Internal methods and properties
	//
	postCreate: function() {
		this.containerNode = this._contentAreaContainer;
		this._initDimensions();
		this.connect(window, "onscroll", this._onScroll);
	},
	
	destroy: function() {
		if (this._contentWidget) {
			this._contentWidget.destroy();
			this._contentWidget = null;
		}
		if (this._sidebarWidget) {
			this._sidebarWidget.destroy();
			this._sidebarWidget = null;
		}
		this.inherited(arguments);
	},
	
	startup: function() {
		if (this._started) {
			return;
		}
		
		if (this._contentWidget) {
			this._contentWidget.startup();
		}
		if (this._sidebarWidget) {
			this._sidebarWidget.startup();
		}
		this.inherited(arguments);
	},
	
	resize: function(size) {
		if (this._started) {
			var height = size && size.h ? size.h : dojo.contentBox(this.domNode.parentNode).h;
			
			if (this._contentWidget) {
				if (this._contentWidget.resize) {
					var clientWidth = parseInt(this.domNode.clientWidth, 10);
					var widthToUse = Math.max(0, (clientWidth - this._prevWidth - SHOW_HIDE_CTRL_WIDTH));
					this._contentWidget.resize({ w: widthToUse, h: height });
				}
				if (this._contentWidget.domNode.offsetHeight > height) {
					height = this._contentWidget.domNode.offsetHeight;
				}
			}
			
			if (this._currentSidebarState === "open" && this._sidebarWidget && this._sidebarWidget.resize) {
				this._sidebarWidget.resize({h: height });
			}
			this._onScroll();
		}
	},
	
	_onScroll: function(event) {
		var height = document.body.clientHeight;
		var offset = dojo.coords(this.domNode);
		this._showHideButton.style.top = (height / 2 - offset.y - 26) + "px";
	},
	
	_initDimensions: function() {
		// summary:
		//		Set the initial state of the elements.
		if (dojo.isIE == 6) {
			this._showHideButton.style.visibility = "hidden";
			this._showHideControl.style.cursor = "normal";
		}
		this._currentSidebarState = "open";
		this._sidebar.style.width = SIDEBAR_DEFAULT_WIDTH + "px";
		this._prevWidth = this._sidebarWidth = SIDEBAR_DEFAULT_WIDTH;
		switchClass(this._showHideButton, "rmps-ui-sidebar-layout-showHideButtonCollapsed", "rmps-ui-sidebar-layout-showHideButtonExpanded");
	},
	
	_onToggleBtnDown: function() {
		this._toggleBtnDown = true;
	},
	
	_toggleSidebar: function(e) {
		if (!this._toggleBtnDown) {
			delete this._toggleBtnDown;
			return;
		}
		// disable sidebar resizing on IE6
		if (!(dojo.isIE && dojo.isIE <= 6)) {
			// description:
			//		If the sidebar is open, close it. If the sidebar is closed,
			//		open it.
			if (dojo.isIE >= 7) {
				this._showHideButton.style.visibility = "hidden";
			}
			
			// animate by simultaneously changing margin-right and left
			var args = {
				node: this._sidebar,
				duration: 200,
				properties: {
					left: {
						unit: "px"
					},
					marginRight: {
						unit: "px"
					}
				}
			};
			
			var anim;
			if (this._currentSidebarState === "open") {
				// hide
				args.properties.left.end = -this._sidebarWidth;
				args.properties.marginRight.end = -this._sidebarWidth;

				args.onEnd = dojo.hitch(this, this._onHideAnimationEnd);
				anim = dojo.animateProperty(args);
				
				this._prevWidth = 0;
				if (dojo.isIE) {
					this._sidebar.style.visibility = "hidden";
				}
			} else {
				// show
				args.properties.left.end = 0;
				args.properties.marginRight.end = 0;
				args.properties.width = {end: this._sidebarWidth, unit: "px" };
				args.onEnd = dojo.hitch(this, this._onShowAnimationEnd);
				args.beforeBegin = dojo.hitch(this, this._onShowAnimationBeforeBegin);
				anim = dojo.animateProperty(args);
				this._prevWidth = this._sidebarWidth;
			}
			switchClass(this._showHideControl, "rmps-ui-sidebar-layout-showHideControl-hover", "rmps-ui-sidebar-layout-showHideControl");
			anim.play();
		}
	},
	
	_onShowAnimationBeforeBegin: function() {
		this._sidebar.style.visibility = "";
	},
	
	_onHideAnimationEnd: function() {
		this._currentSidebarState = "close";
		switchClass(this._showHideButton, "rmps-ui-sidebar-layout-showHideButtonExpanded", "rmps-ui-sidebar-layout-showHideButtonCollapsed");
		if (dojo.isIE >= 7){
			this._showHideButton.style.visibility = "visible";
		}
		this._sidebar.style.visibility = "hidden";
		this._snapped = true;
		
		if (this._contentWidget && this._contentWidget.resize) {
			var widthToUse = this._contentAreaContainer.offsetWidth;
			this._contentWidget.resize({ w: widthToUse });
		}
	},
	
	_onShowAnimationEnd: function() {
		this._currentSidebarState = "open";
		switchClass(this._showHideButton, "rmps-ui-sidebar-layout-showHideButtonCollapsed", "rmps-ui-sidebar-layout-showHideButtonExpanded");
		if (dojo.isIE >= 7){
			this._showHideButton.style.visibility = "visible";
		}
		this._sidebar.style.visibility = "";
		this._snapped = false;
		
		if (this._sidebarWidget && this._sidebarWidget.resize) {
			this._sidebarWidget.resize({ w: this._sidebarWidth });
		}
		
		if (this._contentWidget && this._contentWidget.resize) {
			var widthToUse = this._contentAreaContainer.offsetWidth;
			this._contentWidget.resize({ w: widthToUse });
		}
	},
	
	//
	// Internal event handlers
	//
	
	_highlightShowHideControl: function(e) {
		e.preventDefault();
		e.cancelBubble = true;
		if (!(dojo.isIE && dojo.isIE <= 6) && !this._isDragging) {
			switchClass(this._showHideControl, "rmps-ui-sidebar-layout-showHideControl", "rmps-ui-sidebar-layout-showHideControl-hover");
		}
	},
	
	_unhighlightShowHideControl: function(e) {
		e.preventDefault();
		e.cancelBubble = true;
		if (!(dojo.isIE && dojo.isIE <= 6) && !this._isDragging) {
			switchClass(this._showHideControl, "rmps-ui-sidebar-layout-showHideControl-hover", "rmps-ui-sidebar-layout-showHideControl");
		}
	},

	_highlightShowHideButton: function(e) {
		e.preventDefault();
		e.cancelBubble = true;
		if (!(dojo.isIE && dojo.isIE <= 6) && !this._isDragging) {
			switchClass(this._showHideButton, "rmps-ui-sidebar-layout-showHideButton", "rmps-ui-sidebar-layout-showHideButton-hover");
			switchClass(this._showHideControl, "rmps-ui-sidebar-layout-showHideControl-hover", "rmps-ui-sidebar-layout-showHideControl");
		}
	},
	
	_unhighlightShowHideButton: function(e) {
//		e.preventDefault();
//		e.cancelBubble = true;
		if (!(dojo.isIE && dojo.isIE <= 6) && !this._isDragging) {
			switchClass(this._showHideButton, "rmps-ui-sidebar-layout-showHideButton-hover", "rmps-ui-sidebar-layout-showHideButton");
//			switchClass(this._showHideControl, "rmps-ui-sidebar-layout-showHideControl", "rmps-ui-sidebar-layout-showHideControl-hover");
		}
	},
	
	_startDragging: function(e) {
		dojo.stopEvent(e);
		// disable sidebar resizing on IE6
		if (!(dojo.isIE && dojo.isIE <= 6)) {
			dojo.addClass(document.body, "rmps-ui-sidebar-layout-dragCursor");
			if (this._dragListeners) {
				for (var i = 0; i < this._dragListeners.length; i++) {
					this.disconnect(this._dragListeners[i]);
				}
			}
			this._dragListeners = [
				this.connect(dojo.doc, "onmousemove", this._drag),
				this.connect(dojo.body(), "onmouseout", this._drag),
				this.connect(dojo.body(), "onmouseup", this._stopDragging),
				this.connect(dojo.body(), "ondragstart", this._cancelEvent),
				this.connect(dojo.body(), "onselectstart", this._cancelEvent)
			];
			this._isDragging = true;
			if (this.dragResizeDelay && this.dragResizeDelay > 0) {
				this._dragResizer = {};
			}
		}
	},

	_stopDragging: function(e) {
		dojo.removeClass(document.body, "rmps-ui-sidebar-layout-dragCursor");
		if (this._dragListeners) {
			for (var i = 0; i < this._dragListeners.length; i++) {
				this.disconnect(this._dragListeners[i]);
			}
			delete this._dragListeners;
		}
		var widthpx = dojo.marginBox(this._sidebar).w;
		if (widthpx >= SIDEBAR_MIN_WIDTH) {
			this._sidebarWidth = widthpx;
		}
		this._isDragging = false;
		if (this._dragResizer && this._dragResizer.timeout) {
			this._dragResize();
		}
	},
	
	_cancelEvent: function(e) {
		if (dojo.hasClass(e.currentTarget,"rmps-ui-sidebar-layout-showHideButtonCommon")) {
			return;
		}
		dojo.stopEvent(e);
	},
	
	_drag: function(e) {
		delete this._toggleBtnDown;
		var coordx = dojo.coords(this._top).x;
		var x = this._top.clientWidth + coordx - e.pageX - (SHOW_HIDE_CTRL_WIDTH/2);
		var sidebarLayoutWidth = this._top.clientWidth;
		
		if (this._prevWidth == null) {
			if (x < SIDEBAR_SNAP_WIDTH) {
				this._prevWidth = 0;
			} else {
				this._prevWidth = dojo.marginBox(this._sidebar).w;
			}
		}
		var width = this._prevWidth;
		if (x < SIDEBAR_SNAP_WIDTH) {
			if (!this._snapped) {
				// snap closed
				width = 0;
//				var lwidth = dojo.coords(this._sidebar).w;
				this._sidebar.style.visibility = "hidden";
				
				this._sidebar.style.left = -this._sidebarWidth + "px";
				this._sidebar.style.marginRight = -this._sidebarWidth + "px";
				
				this._currentSidebarState = "close";
				switchClass(this._showHideButton, "rmps-ui-sidebar-layout-showHideButtonExpanded", "rmps-ui-sidebar-layout-showHideButtonCollapsed");
//				this._sidebarWidth = SIDEBAR_MIN_WIDTH;
				this._snapped = true;				
			}
		} else {
			this._sidebar.style.visibility = "";
			this._snapped = false;
			if (x < SIDEBAR_MIN_WIDTH) {
				// prevent further shrinking
				width = SIDEBAR_MIN_WIDTH;
				
				this._sidebar.style.left = "0px";
				this._sidebar.style.marginRight = "0px";
				this._currentSidebarState = "open";
				switchClass(this._showHideButton, "rmps-ui-sidebar-layout-showHideButtonCollapsed", "rmps-ui-sidebar-layout-showHideButtonExpanded");
			} else if (x > SIDEBAR_MAX_WIDTH) {
				// prevent further expansion
				width = SIDEBAR_MAX_WIDTH;
				
				this._sidebar.style.left = "0px";
				this._sidebar.style.marginRight = "0px";
				this._currentSidebarState = "open";
				switchClass(this._showHideButton, "rmps-ui-sidebar-layout-showHideButtonCollapsed", "rmps-ui-sidebar-layout-showHideButtonExpanded");

			} else if (x > (sidebarLayoutWidth - CONTENT_AREA_MIN_WIDTH)) {
				// prevent further expansion
				width = (sidebarLayoutWidth - CONTENT_AREA_MIN_WIDTH);
				this._currentSidebarState = "open";
				dojo.addClass(this._showHideButton, "rmps-ui-sidebar-layout-sidebarShowHideButtonExpanded");
				dojo.removeClass(this._showHideButton, "rmps-ui-sidebar-layout-sidebarShowHideButtonCollapsed");
				
			} else {
				// slide with mouse
				width = x;
				
				if (this._sidebar.style.left != "0px") {
					this._sidebar.style.left = "0px";
				}
				if (this._sidebar.style.marginRight != "0px") {
					this._sidebar.style.marginRight = "0px";
				}
				this._currentSidebarState = "open";
				switchClass(this._showHideButton, "rmps-ui-sidebar-layout-showHideButtonCollapsed", "rmps-ui-sidebar-layout-showHideButtonExpanded");
			}
		}
		
		
		if (this._dragResizer && this._dragResizer.timeout) {
			clearTimeout(this._dragResizer.timeout);
			delete this._dragResizer.timeout;
		}
		
		if (width != 0) {
			this._sidebar.style.width = width + "px";
			if (this._sidebarWidget && this._sidebarWidget.resize) {
				if (this._dragResizer) {
					this._dragResizer.sideBarWidgetWidth = width;
				} else {
					this._sidebarWidget.resize({ w: width });
				}
			}
		}
		
		if (this._prevWidth != width && this._contentWidget && this._contentWidget.resize) {
			var cWidth = this._contentAreaContainer.offsetWidth;
			if (this._dragResizer) {
				this._dragResizer.contentWidgetWidth = cWidth;
			} else {
				this._contentWidget.resize({ w: cWidth });
			}
		}

		this._prevWidth = width;
		
		if (this._dragResizer && (this._dragResizer.sideBarWidgetWidth != null || this._dragResizer.contentWidgetWidth != null)) {
			this._dragResizer.timeout = setTimeout(dojo.hitch(this, this._dragResize), this.dragResizeDelay);
		}
	},
	
	_dragResize: function() {
		if (this._dragResizer.timeout) {
			clearTimeout(this._dragResizer.timeout);
			delete this._dragResizer.timeout;
		}
		if (this._dragResizer.sideBarWidgetWidth != null && this._sidebarWidget.resize) {
			this._sidebarWidget.resize({ w: this._dragResizer.sideBarWidgetWidth });
		}
		if (this._dragResizer.contentWidgetWidth != null && this._contentWidget.resize) {
			this._contentWidget.resize({ w: this._dragResizer.contentWidgetWidth });
		}
	}
});

})();
