/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.Table");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dojo.html");

dojo.require("jazz.ui.ResourceLink");

dojo.require("rmps.ui.PagingControl");
dojo.require("rmps.ui.TextFilterInput");
dojo.require("rmps.ui.util.ToolbarFocusUtil");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.ui", "TableMessages");
	
//	headerConfigs: [
//		{
//			id: null, // also used as the className for each cell
//			label: null, // the header label
//			iconUrl: null,
//			isOrderable: false,
//			defaultAscending: true, // true | false
//			filterParticipant: null, // true | false | function(string) (provide custom filtering)
//		}
//	]
//	tableData = [
//		{
//			init: null, // function(rowConfig, tr, rowIdx)
//			link: null,
//			columns: [
//				{
//					data: null, // required to support default ordering
//					formatter: null, // formats the data into a string
//					link: null,
//					iconUrl: null,
//					iconOnly: false,
//					className: null,
//					style: {},
//					deferred: null,
//					init: null, // function(rowConfig, tr, rowIdx, columnConfig, td, columnIdx)
//					resourceReference: null // true or false, indicates that the column data is a resource ref
//				}
//			]
//		}
//	]

var ROW_CONFIG_PROP = "_tableWidgetRowConfig";
var TH_IMG_PROP = "_tableWidgetThImg";
var TH_ORDER_IMG_PROP = "_tableWidgetThOrderImg";

var PAGE_SIZE_INTERVALS = [10, 25, 50, 75, 100, 250, 500];

var DEFAULT_SORT_ASCENDING = true;

var ToolbarFocusUtil = rmps.ui.util.ToolbarFocusUtil;

dojo.declare("rmps.ui.Table", [dijit._Widget, dijit._Templated], {
	
	messages: dojo.i18n.getLocalization("rmps.ui", "TableMessages"),
	
	templateString: dojo.cache("rmps.ui", "templates/Table.html"),
	widgetsInTemplate: true,
	
	// properties
	pageSizeDefault: 50,
	noEntriesLabel: null,
	noFilterMatchesLabel: null,
	
	// private properties
	_deferredColumns: null,
	_headerConfigs: null,
	_tableData: null,
	_orderById: null,
	_secondaryOrder: null, //[{id: , isAscending:},...]
	_ascending: DEFAULT_SORT_ASCENDING,
	_thCache: null,
	_rowCache: null,
	_prevFilter: null,
	_filteredOutRowCount: 0,
	
	_headerHandles: null,
	
	postCreate: function() {
		this.containerNode = this._tableBody;
		this.pagingControl.table = this;
		this.footerPagingControl.table = this;
		this.pagingControl.linked = this.footerPagingControl;
		this.footerPagingControl.linked = this.pagingControl;
		
		this.connect(this._filterInput, "filterChanged", this._filterChanged);
	},
	
	_constructPageSizeDropDown: function() {
		dojo.empty(this._pageSizeSelect);
		
		// populate the drop-down
		var optionIndex = 0;
		var pageSize = this.pagingControl.pageSize;
		var count = this.pagingControl.getUpperBound();
		var smallestSize = PAGE_SIZE_INTERVALS[0];
		var showAllSize = count <= PAGE_SIZE_INTERVALS[PAGE_SIZE_INTERVALS.length - 1] ? count : null; 
		
		// check for custom size
		var i;
		var option;
		var customSize = false;
		var insertIdx = PAGE_SIZE_INTERVALS.length;
		
		if (showAllSize !== pageSize) {
			for (i = 0; i < PAGE_SIZE_INTERVALS.length; i++) {
				if (PAGE_SIZE_INTERVALS[i] === pageSize) {
					break;
				} else if (PAGE_SIZE_INTERVALS[i] > pageSize) {
					customSize = true;
					insertIdx = i;
					if (pageSize < smallestSize) {
						smallestSize = pageSize;
					}
					break;
				}
			}
		}
		
		for (i = 0; i <= PAGE_SIZE_INTERVALS.length; i++) {
			if (customSize && i === insertIdx) {
				option = new Option("" + pageSize, "" + pageSize);
				this._pageSizeSelect.options[optionIndex++] = option;
				option.selected = true;
			}
			if (i < PAGE_SIZE_INTERVALS.length && count > PAGE_SIZE_INTERVALS[i]) {
				option = new Option("" + PAGE_SIZE_INTERVALS[i], "" + PAGE_SIZE_INTERVALS[i]);
				this._pageSizeSelect.options[optionIndex++] = option;
				option.selected = (PAGE_SIZE_INTERVALS[i] === pageSize);
			} else {
				break;
			}
		}
		
		// show all not supported for large queries
		if (showAllSize !== null) {
			option = new Option(this.messages.showall, "" + showAllSize);
			this._pageSizeSelect.options[optionIndex] = option;
			option.selected = (pageSize >= showAllSize);
		}
		
		// disable drop-down if only one choice
		if (count <= smallestSize) {
			this._pageSizeSelect.disabled = true;
			dojo.addClass(this._pageSizeSelect, "disabled");
		} else {
			this._pageSizeSelect.disabled = false;
			dojo.removeClass(this._pageSizeSelect, "disabled");
		}
	},
	
	setTableData: function(headers, tableData) {
		this._headerConfigs = headers;
		this._tableData = tableData;
	},
	
	update: function(preserveFilter, page, pageSize, size, limit, orderBy, direction, secondaryOrder) {
		this._resetTableBody();
		this._resetTableHeader();
		
		this._ascending = direction === true ? true : (direction === false ? false : DEFAULT_SORT_ASCENDING);
		this._orderById = orderBy;
		this._secondaryOrder = secondaryOrder;
		
		this.pagingControl.update(
			page !== null && page >= 0 ? page : 0,
			pageSize !== null && pageSize > 0 ? pageSize : this.pageSizeDefault,
			limit !== null && limit > 0 ? limit : -1,
			size !== null && size >= 0 ? size : (this._tableData ? this._tableData.length : 0)
		);
		
		this._constructPageSizeDropDown();
		
		if (!preserveFilter) {
//			delete this._prevFilter;
			this._filterInput.clearFilter();
		}
		
		if (!this._tableData || this._tableData.length === 0) {
			this._filterInput.setEnabled(preserveFilter && this._prevFilter);
			
			this._populateTableHeader(true);
//			this._filterInput.clearFilter();
			
			this.messageNode.innerHTML = this.noEntriesLabel || '';
			
		} else {
			this._filterInput.setEnabled(true);
			this.messageNode.innerHTML = "";
			
			this._populateTableHeader(false);
			
			if (this._orderById) {
				this._reorder();
			}
			
			this._populateTableBody();
		}
	},
	
	_selectPageSize: function(pageSize) {
		var options = this._pageSizeSelect.options;
		for (var i = 0; i < options.length; ++i) {
			if (options[i].value === pageSize) {
				this._pageSelect.selectedIndex = i;
			}
		}
	},
	
	redraw: function() {
		if (this._tableData) {
			this._resetTableBody();
			this._populateTableBody();
		}
	},
	
	onPreviousClick: function(e) {
		if (this.search && this.search._onPrevPage) {
			this.search._onPrevPage();
		} else {
			this.redraw();
		}
	},
	
	onNextClick: function(e) {
		if (this.search && this.search._onNextPage) {
			this.search._onNextPage();
		} else {
			this.redraw();
		}
	},
	
	_populateTableBody: function() {
		var pageSize = this.pagingControl.pageSize !== -1 ? this.pagingControl.pageSize : this.pagingControl.size;
		this._populatePagedTableBody(this.pagingControl.page, pageSize);
	},
	
	setOrder: function(columnId, ascending) {
		if (this._orderById !== columnId || this._ascending !== ascending) {
			this._orderById = columnId;
			this._ascending = ascending;
			if (this._reorder()) {
				this.redraw();
			}
		}
	},
	
	enableFiltering: function(enable) {
		if (enable) {
			this._filterInput.style.display = "";
		} else {
			this._filterInput.style.display = "none";
		}
	},
	
	_reorder: function() {
		if (this._tableData && this.pagingControl.getUpperBound() > 1) {
			var idToIdx = {}, i;
			
			for (i = 0; i < this._headerConfigs.length; ++i) {
				if (this._headerConfigs[i].isOrderable) {
					idToIdx[this._headerConfigs[i].id] = i;
				}
			}
			
			var idx = idToIdx[this._orderById];

			if (idx !== null) {
				delete idToIdx[this._orderById];
				var orderData = [{
					id: this._orderById,
					isAscending: this._ascending,
					'idx': idx
				}];
				
				if (this._secondaryOrder) {
					for (i = 0; i < this._secondaryOrder.length; ++i) {
						var secondaryOrder = this._secondaryOrder[i];
						var _idx = idToIdx[secondaryOrder.id];
						if (_idx !== null) {
							delete idToIdx[secondaryOrder.id];
							orderData.push({
								id: secondaryOrder.id,
								isAscending: secondaryOrder.isAscending,
								'idx': _idx
							});
						}
					}					
				}
				
				for (i = 0; i < this._thCache.length; ++i) {
					if (this._headerConfigs[i].isOrderable) {
						var orderIcon = this._thCache[i][TH_ORDER_IMG_PROP];
						if (i === idx) {
							if (this._ascending) {
								orderIcon.src = dojo.moduleUrl("rmps.ui", "graphics/table/order-up-arrow.gif");
								orderIcon.alt = this.messages.ascending;
							} else {
								orderIcon.src = dojo.moduleUrl("rmps.ui", "graphics/table/order-down-arrow.gif");
								orderIcon.alt = this.messages.descending;
							}
							orderIcon.style.display = "";
						} else {
							orderIcon.style.display = "none";
						}
						var title = this.getHeaderTooltip(this._headerConfigs[i]);
						if (title) {
							this._thCache[i].title = title;
						}
					}
				}
				this.sortTableData(this._tableData, orderData);
				return true;
			}
		}
		return false;
	},
	
	sortTableData: function(tableData, /* {id: , idx: , isAscending: } */ orderBy) {
		function compare(entry1, entry2, columnIdx, isAscending) {
			var a = entry1.columns[columnIdx] && entry1.columns[columnIdx].data ? entry1.columns[columnIdx].data : "";
			var b = entry2.columns[columnIdx] && entry2.columns[columnIdx].data ? entry2.columns[columnIdx].data : "";
			
			if (dojo.isString(a)) {
				a = a.toLowerCase();
			}
			if (dojo.isString(b)) {
				b = b.toLowerCase();
			}
			
			if (isAscending) {
				return (a < b) ? -1 : ((a > b) ? 1 : 0);
			} else {
				return (a > b) ? -1 : ((a < b) ? 1 : 0);
			}
		}
		
		function comparator(entry1, entry2) {
			for (var i = 0; i < orderBy.length; ++i) {
				var result = compare(entry1, entry2, orderBy[i].idx, orderBy[i].isAscending);
				if (result !== 0) {
					return result;
				}
			}
			return 0;
		}
		tableData.sort(comparator);
	},
	
	getHeaderTooltip: function(headerConfig) {
		if (headerConfig && headerConfig.isOrderable) {
			var ascendingMsg = DEFAULT_SORT_ASCENDING;
			if (this._orderById === headerConfig.id) {
				ascendingMsg = !this._ascending;
			} else if (headerConfig.defaultAscending) {
				ascendingMsg = headerConfig.defaultAscending;
			}
			if (ascendingMsg) {
				return dojo.string.substitute(this.messages.orderBy_ascending, [headerConfig.label]);
			}
			return dojo.string.substitute(this.messages.orderBy_descending, [headerConfig.label]);
		}
		return null;
	},
	
	isAscendingOrder: function() {
		return this._ascending;
	},
	
	getOrder: function() {
		return this._orderById;
	},
	
	// clear header rows
	_resetTableHeader: function() {
		dojo.empty(this._headerRowLine);
		dojo.empty(this._headerRow);
		dojo.empty(this._headerRowBottomLine);
		
		this.disconnect(this._headerHandles);
		
		this._thCache = [];
		this._headerHandles = [];
	},
	
	// clear body rows
	_resetTableBody: function() {
		// destroy all widgets
		var childWidgets = dijit.findWidgets(this._tableBody);
		dojo.forEach(childWidgets, function(widget) {
			if (widget.destroyRecursive) {
				widget.destroyRecursive();
			}
		});
		
		dojo.empty(this._tableBody);
		
		this._rowCache = [];
		this._deferredColumns = {};
		
		this._filteredOutRowCount = 0;
		this.messageNode.innerHTML = "";
	},
	
	_populateTableHeader: function(preventOrdering) {
		dojo.forEach(this._headerConfigs,
			dojo.hitch(this, function(config) {
				this._addHeaderColumn(preventOrdering, config);
			}));
	},
	
	_addHeaderColumn: function(preventOrdering, config) {
		// Add top white line trim
		var headerColumnLine = dojo.doc.createElement("th");
		dojo.addClass(headerColumnLine, config.id);
		dojo.addClass(headerColumnLine, "headerColumnLine");
		headerColumnLine.innerHTML = "&nbsp;";
		this._headerRowLine.appendChild(headerColumnLine);
		
		var th = dojo.doc.createElement("th");
		dojo.attr(th, 'scope', 'col');
		dojo.addClass(th, "headerColumn");
		dojo.addClass(th, config.id);
		
		var contentParent = dojo.doc.createElement("nobr");
		th.appendChild(contentParent);
		
		// Add bottom white line trim
		var headerBottomLine = dojo.doc.createElement("th");
		dojo.addClass(headerBottomLine, config.id);
		dojo.addClass(headerBottomLine, "headerBottomLine");
		headerBottomLine.innerHTML = "&nbsp;";
		this._headerRowBottomLine.appendChild(headerBottomLine);
		
		if (!preventOrdering && config.isOrderable) {
			if (this._orderById === config.id) {
				if (this._ascending) {
					dojo.addClass(th, "headerColumnOrder-ascending");
				} else {
					dojo.addClass(th, "headerColumnOrder-descending");
				}	
			} else {
				dojo.addClass(th, "headerColumnUnsorted");
			}
			var link = dojo.doc.createElement("a");
			if (this._orderById === config.id) {
				link.href = this.getReorderHref(config.id, !this._ascending, this._secondaryOrder);
			} else {
				link.href = this.getReorderHref(config.id,
					config.defaultAscending ? config.defaultAscending : DEFAULT_SORT_ASCENDING,
					this._secondaryOrder);
			}
			contentParent.appendChild(link);
			contentParent = link;
			dojo.addClass(th, "orderable");
		} else {
			dojo.addClass(th, "unorderable");
		}
		
		var icon = dojo.doc.createElement("img");
		dojo.addClass(icon, "iconSpacer");
		if (config.iconUrl) {
			icon.src = config.iconUrl;
		} else {
			icon.style.display = "none";
		}
		icon.alt= "";
		contentParent.appendChild(icon);
		th[TH_IMG_PROP] = icon;
			
		if (config.label) {
			var span = dojo.doc.createElement("span");
			span.appendChild(dojo.doc.createTextNode(config.label));
			contentParent.appendChild(span);
		}
		
		if (!preventOrdering && config.isOrderable) {
			var orderIcon = dojo.doc.createElement("img");
			orderIcon.alt= "";
			dojo.addClass(orderIcon, "order-icon");
			orderIcon.style.display = "none";
			contentParent.appendChild(orderIcon);
			th[TH_ORDER_IMG_PROP] = orderIcon;
			
			this._headerHandles.push(
				this.connect(contentParent, "onclick", function(e) {
					var href = null;
					if (this._orderById === config.id) {
						href = contentParent.href = this.getReorderHref(config.id, !this._ascending, this._secondaryOrder);
						this.setOrder(config.id, !this._ascending);
					} else {
						var ascending = config.defaultAscending ? config.defaultAscending : DEFAULT_SORT_ASCENDING;
						href = contentParent.href = this.getReorderHref(config.id, ascending, this._secondaryOrder);
						this.setOrder(config.id, ascending);
					}
					
					if (!href || href.length === 0) {
						e.preventDefault();
					}
				})
			);
		}
		
		var title = this.getHeaderTooltip(config);
		if (title) {
			th.title = title;
		}
		
		this._thCache.push(th);
		
		this._headerRow.appendChild(th);
	},
	
	_populatePagedTableBody: function(page, pageSize) {
		for (var i = 0; i < this._tableData.length; i++) {
			var item = this._tableData[i];
			var tableRow = dojo.doc.createElement("tr");
			tableRow.className = "table-grid-row";
			this._rowCache.push(tableRow);
			
			tableRow[ROW_CONFIG_PROP] = i;
			
			if (this._prevFilter && this._filterRow(this._prevFilter, item)) {
				tableRow.style.display = "none";
				this._filteredOutRowCount++;
			}
			
			this._tableBody.appendChild(tableRow);
			
			if (item.init) {
				item.init(item, tableRow, i);
			}
			
			this._populateRow(item, tableRow, i);
		}
		this._updateFilterMessage();
	},
	
	_populateRow: function(rowConfig, tableRow, rowIdx) {
	    var makeAccessible = function(tableRow, cssClass){
	        if(tableRow.actionToolbar){
                ToolbarFocusUtil.addToolbarFocusListeners( { 
                    actionToolbar: tableRow.actionToolbar, 
                    containerNode: tableRow, 
                    actionClassName: cssClass
                });
            }
	    }
		var i = 0;
		dojo.forEach(rowConfig.columns, dojo.hitch(this, function(columnConfig) {
			var row = tableRow;
			var cell = tableRow.insertCell(i);
			cell.className = "table-grid-cell";
			
			if (columnConfig.style) {
				for (var style in columnConfig.style) {
					cell.style[style] = columnConfig.style[style];
				}
			}
			
			if (columnConfig.className) {
				dojo.addClass(cell, columnConfig.className);
			}
			
			if (!columnConfig.deferred) {
				this._populateColumn(rowConfig, tableRow, rowIdx, columnConfig, cell, i);
				
			} else {
				if (!this._deferredColumns[i]) {
					this._deferredColumns[i] = {
						total: 0,
						returned: 0	
					};
					var th = this._thCache[i];
					var icon = th[TH_IMG_PROP];
					icon.src = dojo.moduleUrl("rmps.ui", "common/images/spinner.gif");
					icon.style.display = "";
				}
				this._deferredColumns[i].total++;
				
				var idx = i;
				columnConfig.deferred.addBoth(dojo.hitch(this, function(response) {
					if (cell.firstChild) {
						cell.removeChild(cell.firstChild);						
					}
					if (!(response instanceof Error)) {
						this._populateColumn(rowConfig, tableRow, rowIdx, columnConfig, cell, idx);
						
						if (row.style.display === "none" && !this._filterColumn(this._prevFilter, this._headerConfigs[idx], columnConfig)) {
							row.style.display = "";
							this._filteredOutRowCount--;
							this._updateFilterMessage();
						}
					}
					
					if (++this._deferredColumns[idx].returned === this._deferredColumns[idx].total) {
						var th = this._thCache[idx];
						if (this._headerConfigs[idx].iconUrl) {
							th[TH_IMG_PROP].src = this._headerConfigs[idx].iconUrl;
						} else {
							th[TH_IMG_PROP].style.display = "none";
							th[TH_IMG_PROP].src = "";
						}
					}
					makeAccessible(tableRow, ".jazz-ui-Toolbar");
				}));
				
				cell.innerHTML = "&nbsp;";
			}
			
			i++;
		}));
		
		makeAccessible(tableRow, ".jazz-ui-Toolbar");
	},
	
	_populateColumn: function(rowConfig, tableRow, rowIdx, columnConfig, tableCell, colIdx) {
		if (columnConfig.init) {
			columnConfig.init(rowConfig, tableRow, rowIdx, columnConfig, tableCell, colIdx);
			return;
		}
		var span = dojo.doc.createElement("span");
		
		var parentElement = span;
		
		if (columnConfig.resourceReference) {
			//Now how do we make this a fancy link with the hover thingy?
			var uri = columnConfig.referenceData.uri;
			//If there was no label provided use a placeholder
			var name = columnConfig.data ? columnConfig.data : "-";
			var viewUri = columnConfig.referenceData.viewUri;

			if (uri) {
				var link = dojo.doc.createElement("a");
				link.href = uri;
				span.appendChild(link);
				link.appendChild(dojo.doc.createTextNode(name));				
				parentElement = link;
	
				var resLink = new jazz.ui.ResourceLink({
					srcNodeRef: link,
					uri: rowConfig.owned ? net.jazz.ajax._contextRoot+ "/rendering?resourceUri=" + encodeURIComponent(uri) : uri,
					presentationUri: viewUri,
					lazyFetch: true,
					avoidBaselineCheck: true
					});
			} else {
				//No URI, just show the label as text
				parentElement.appendChild(dojo.doc.createTextNode(name));						
			}
		} else {
			if (columnConfig.link) {
				var link = dojo.doc.createElement("a");
				if (columnConfig.link === true || columnConfig.link === rowConfig.link) {
					link.className = "row-link";
					link.href = rowConfig.link;
					if ( rowConfig.linkTarget ) {
						link.target = rowConfig.linkTarget;
					}
				} else {
					link.href = columnConfig.link;
				}		
				span.appendChild(link);
				parentElement = link;
			}
			
			var label = this._formatLabel(columnConfig.data, this._headerConfigs[colIdx].formatter);
			
			if (columnConfig.iconUrl) {
				var icon = dojo.doc.createElement("img");
				icon.alt= "";
				icon.src = columnConfig.iconUrl;
				
				if (label) {
					icon.alt = label;
				}				
			}
			
			if (!columnConfig.iconOnly) {
				if (!label) {
					label = "-";
					dojo.addClass(parentElement, "emptyContent");
				}
				parentElement.appendChild(dojo.doc.createTextNode(label));						
			}
		}
		
		tableCell.appendChild(span);
	},
	
	_formatLabel: function(data, formatter) {
		if (formatter) {
			return formatter(data);
		}
		if (!dojo.isString(data) || data.length === 0) {
			return null;
		}
		return data;
	},
	
	_filterChanged: function(filter) {
		this._filteredOutRowCount = 0;
		var i;
		if (!filter) {
			for (i = 0; i < this._rowCache.length; ++i) {
				this._rowCache[i].style.display = "";
			}
		} else {
			var shortcut = this._prevFilter && filter.indexOf(this._prevFilter) === 0;
			
			for (i = 0; i < this._rowCache.length; ++i) {
				var config = this._tableData[this._rowCache[i][ROW_CONFIG_PROP]];
				if (shortcut) {
					if (this._rowCache[i].style.display === "") {
						if (this._filterRow(filter, config)) {
							this._rowCache[i].style.display = "none";
							this._filteredOutRowCount++;
						}
					} else {
						this._filteredOutRowCount++;
					}
				} else if (this._filterRow(filter, config)) {
					this._rowCache[i].style.display = "none";
					this._filteredOutRowCount++;
				} else {
					this._rowCache[i].style.display = "";
				}
			}
		}
		this._prevFilter = filter;
		this._updateFilterMessage();
	},
	
	_filterRow: function(filter, config) {
		if (!filter) {
			return false;
		}
		for (var i = 0; i < config.columns.length; ++i) {
			if (!this._filterColumn(filter, this._headerConfigs[i], config.columns[i])) {
				return false;
			}
		}
		return true;
	},
	
	_filterColumn: function(filter, headerConfig, config) {
		if (!filter || !headerConfig) {
			return false;
		}
		if (dojo.isFunction(headerConfig.filterParticipant)) {
			if (headerConfig.filterParticipant(filter, config)) {
				return false;
			}
		} else if (headerConfig.filterParticipant && config.data) {
			if (config.data.deferred) {
				return false;
			}

			var label = config.data instanceof Object ? config.data.label : config.data; 
			if (label && label.toLowerCase().indexOf(filter.toLowerCase()) !== -1) {
				return false;
			}
		}
		return true;
	},
	
	_updateFilterMessage: function() {
		var size = this.pagingControl.pageSize;
		var upperBound = this.pagingControl.getUpperBound();
		if ((this.pagingControl.page * this.pagingControl.pageSize +
				this.pagingControl.pageSize) > upperBound) {
			size = upperBound - (Math.floor(upperBound / this.pagingControl.pageSize) * this.pagingControl.pageSize);
		}
		if (this._filteredOutRowCount === size) {
			this.messageNode.innerHTML = this.noFilterMatchesLabel || '';
		} else {
			this.messageNode.innerHTML = "";
		}
	},
	
	/*
	 * Event handlers
	 */
	
	_handleTbodyMouseOver: function(e) {
		var row = this._getRow(e);
		if (!row) {
			return;
		}
		
		dojo.addClass(row, "table-row-hover");
		
		if (this._tableData[row[ROW_CONFIG_PROP]].link) {
			dojo.addClass(row, "table-row-linked");
		}
	},
	
	_handleTbodyMouseOut: function(e) {
		var row = this._getRow(e);
		if (!row) {
			return;
		}
		
		dojo.removeClass(row, "table-row-hover");
		
		if (this._tableData[row[ROW_CONFIG_PROP]].link) {
			dojo.removeClass(row, "table-row-linked");
		}
	},
	
	_handleTbodyMouseDown: function(e) {
		var mouseMoveHandle = this.connect(this._tableBody, "onmousemove", function() {
			this.disconnect(mouseMoveHandle);
			this.disconnect(mouseUpHandle);
		});
		
		var mouseUpHandle = this.connect(this._tableBody, "onmouseup", function(e) {
			this.disconnect(mouseUpHandle);
			this.disconnect(mouseMoveHandle);
			this._handleTbodyMouseClick(e);
		});
	},
	
	_handleTbodyMouseClick: function(e) {
		if (e.ctrlKey || e.shiftKey || e.altKey || e.button === 2) { // alt clicks open tabs or windows
			return;
		}
		
		var row = this._getRow(e);
		if (!row || !this._tableData[row[ROW_CONFIG_PROP]].link ||
				this._tableData[row[ROW_CONFIG_PROP]].linkTarget ) { // rows with linkTarget open tabs or windows
			return;
		}
		
		window.location.href = this._tableData[row[ROW_CONFIG_PROP]].link;
	},
	
	_getRow: function(e) {
		var element = e.target || e.srcElement;
		// walk up the DOM tree until we find a row element
		while (element && element.tagName.toLowerCase() !== "tr" && element !== this._tableBody) {
			element = element.parentNode;
		}
		return element !== this._tableBody ? element || null : null;
	},
	
	_onPageSizeChange: function(e) {
		var pageSize = parseInt(this._pageSizeSelect.value, 10);
		this.pagingControl.update(null, pageSize, null, null);
		var href = this.getPageHref(this.pagingControl.page, pageSize);
		if (href && href.length > 0) {
			window.location.href = href;
			this.redraw();
		} else {
			if (this.search && this.search._onChangePageSize) {
				this.search._onChangePageSize(this.pagingControl.page, pageSize);
			}
		}
	},
	
	getPageHref: function(page, pageSize) {
		return '';
	},
	
	getReorderHref: function(orderBy, ascending, secondaryOrder) {
		return '';
	}
});

dojo.declare("rmps.ui.table._PagingControl", rmps.ui.PagingControl, {
	
	table: null,
	linked: null,
	
	update: function(page, pageSize, limit, size, item) {
		this.inherited(arguments);
		if (this.linked) {
			var temp = this.linked.linked;
			this.linked.linked = null;
			this.linked.update(page, pageSize, limit, size, item);
			this.linked.linked = temp;
		}
	},
	
	onPreviousClick: function(e) {
		this.inherited(arguments);
		this.table.onPreviousClick(e);
	},
	
	onNextClick: function(e) {
		this.inherited(arguments);
		this.table.onNextClick(e);
	},
	
	getPreviousHref: function() {
		return this.table.getPageHref(this.page - 1, this.pageSize);
	},
	
	getNextHref: function() {
		return this.table.getPageHref(this.page + 1, this.pageSize);
	}
});

})();