/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.TaskMonitor");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("rmps.core.error");

dojo.requireLocalization("rmps.core", "TaskMonitorMessages");

var messages = dojo.i18n.getLocalization("rmps.core", "TaskMonitorMessages");

dojo.declare("rmps.core.TaskMonitor", null, {
	
	_taskName: null,
	_canComplete: true,
	_complete: false,
	_tasks: null,
	
	_errors: null,
	
	_errCnt: 0,
	_taskCnt: 0,
	_doneCnt: 0,
	
	constructor: function(name) {
		this.setTaskName(name);
		this._tasks = {};
		this._errors = [];
	},
	
	setTaskName: function(name) {
		this._taskName = name;
	},
	
	getTaskName: function() {
		return this._taskName;
	},
	
	/*Task*/ beginTask: function(/*string*/name, /*int*/totalWork) {
		var handle = "task" + this._taskCnt++;
		
		var task = new rmps.core.TaskMonitor._Task(
			this,
			handle,
			name,
			totalWork
		);
		
		this._tasks[handle] = task;
		
		this.onNewTask(task);
		
		return task;
	},
	
	_error: function(/*Task*/task, error) {
		this._errors.push(error);
		this._errCnt++;
		if (this._canComplete) {
			this._onError(error);
		}
	},
	
	_done: function(task) {
		this._doneCnt++;
		if (this._canComplete && (this._doneCnt + this._errCnt) === this._taskCnt) {
			this._onComplete();
		}
	},
	
	setCanComplete: function(canComplete) {
		if (!this._complete) {
			this._canComplete = canComplete;
			if (canComplete) {
				if (this._doneCnt === this._taskCnt) {
					this._onComplete();
				} else if (this._errCnt > 0) {
					for (var handle in this._tasks) {
						if (this._tasks.hasOwnProperty(handle)) {
							var task = this._tasks[handle];
							if (task._error) {
								this._onError(task._error);
							}							
						}
					}
				}
			}
		}
	},
	
	_onComplete: function() {
		if (!this._complete) {
			this._complete = true;
			this.onComplete(this._errCnt > 0 ? this._errors : null);
		}
	},
	
	_onError: function(error) {
		this.onError(error);
		if ((this._doneCnt + this._errCnt) === this._taskCnt) {
			this._onComplete();
		}
	},
	
	// events
	/*
	 * Fired on addtion of a new Task;
	 */
	onNewTask: function(task) {},
	
	/*
	 * Fired on completion of all tasks if 'canComplete' is set to true.
	 */
	onComplete: function(errored) {},
	
	/*
	 * Fired for each errored task.
	 */
	onError: function(error) {}
	
});

dojo.declare("rmps.core.TaskMonitor._Task", null, {
	
	_handle: null,
	
	_monitor: null,
	
	name: null,
	unitsWorked: 0,
	totalWork: 0,
	
	_done: false,
	_error: null,
	
	constructor: function(monitor, handle, name, totalWork) {
		this._monitor = monitor;
		this._handle = handle;
		this.name = name;
		this.totalWork = totalWork;
	},
	
	worked: function(units) {
		if (!this._done && !this._error) {
			var unitsToAdd = units;
			if (this.unitsWorked + units > this.totalWork) {
				unitsToAdd = this.totalWork - this.unitsWorked;
			}
			this.unitsWorked += unitsToAdd;
			
			this.onWorked(unitsToAdd, this.unitsWorked, this.totalWork);
			
			if (this.unitsWorked === this.totalWork) {
				this.done();
			}
		}
	},
	
	done: function() {
		if (!this._done && !this._error) {
			this.unitsWorked = this.totalWork;
			this._done = true;
			this.onDone();
			this._monitor._done(this);
		}
	},
	
	error: function(error) {
		if (!this._done && !this._error) {
			this._error = error;
			this.onError(error);
			this._monitor._error(this, error);
		}
	},
	
	// eventing 
	onDone: function() {},
	onWorked: function(units, worked, totalWork) {},
	onError: function(error) {},
	
	// utility methods
	handleDeferredWork: function(deferred, units) {
		deferred.addBoth(this, function(response) {
			if (response instanceof Error) {
				var error = response;
				if (response.dojoType === "cancel") {
					error = new Error(dojo.string.substitute(messages.cancelledError, [this.name]));
				} else if (deferred.ioArgs && deferred.ioArgs.xhr && !dojo._isDocumentOk(deferred.ioArgs.xhr)) {
					error = rmps.core.error.createXhrError(deferred.ioArgs);
				}
				this.error(error);
			}else {
				this.worked(units);
			}
			
		});
	}
});

})();
