/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.TextFilterInput");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.ui", "TextFilterInputMessages");

dojo.declare("rmps.ui.TextFilterInput", [dijit._Widget, dijit._Templated], {
	
	templateString: dojo.cache("rmps.ui", "templates/TextFilterInput.html"),
	
	messages: dojo.i18n.getLocalization("rmps.ui", "TextFilterInputMessages"),
	
	// properties
	delay: 200, // delay in milliseconds
	
	// private properties
	_filterText: null,
	_timer: null,
	_isFocused: null,
	
	constructor: function() {
		this._filterText = '';
		this._isFocused= false;
	},
	
	postCreate: function() {
		this.connect(this.filterInput, "onkeyup", this._onSetFilterValue);
		dojo.addClass(this.filterBox, "inactive");
		this.filterInput.value = this.messages.inactiveText;
	},
	
	destroy: function() {
		if (this._timer) {
			clearTimeout(this._timer);
		}
		this.inherited(arguments);
	},
	
	_onSetFilterValue: function() {
		if(!this._isFocused){
			/*Do not apply filter if the filter input is not focused */
			return;
		}
		var newFilterText = this.filterInput.value;
		if (this._filterText != newFilterText) {
			this._filterText = newFilterText;
			if (this._timer) {
				clearTimeout(this._timer);
			}
			this._timer = setTimeout(dojo.hitch(this, function() {
				this.filterChanged(this._filterText);
			}), this.delay);
		}
	},
	
	_onFilterFocus: function(e) {
		this._isFocused= true;
		if (this._filterText.length == 0) {
			dojo.removeClass(this.filterBox, "inactive");
			this.filterInput.value = '';
		}
	},
	
	_onFilterBlur: function(e) {
		this._isFocused= false;
		if (this._filterText.length == 0) {
			dojo.addClass(this.filterBox, "inactive");
			this.filterInput.value = this.messages.inactiveText;
		}
	},
	
	_onClearFilterClick: function(e) {
		this.clearFilter();
	},
	
	getFilterValue: function(){
		return this._filterText;
	},
	
	setFilter: function(filterText) {
		this._filterText = filterText != null ? filterText : '';
		if (filterText && filterText.length > 0) {
			this._filterText = filterText;
			dojo.removeClass(this.filterBox, "inactive");
			this.filterInput.value = filterText;
		} else {
			this._filterText = '';
			dojo.addClass(this.filterBox, "inactive");
			this.filterInput.value = this.messages.inactiveText;
		}
		this.filterChanged(this._filterText);
	},
	
	clearFilter: function() {
		if (this._filterText.length > 0) {
			this._filterText = '';
			dojo.addClass(this.filterBox, "inactive");
			this.filterInput.value = this.messages.inactiveText;
			this.filterChanged(this._filterText);
		}
	},
	
	filterChanged: function(filter) {
		
	},
	
	setEnabled: function(enabled) {
		this.filterInput.disabled = !enabled;
		if (enabled) {
			this.clearButton.style.display = "";
			dojo.addClass(this.filterInputWrapper, "filterInputWrapper");
		} else {
			this.clearButton.style.display = "none";
			dojo.removeClass(this.filterInputWrapper, "filterInputWrapper");
		}
	}
});

})();