/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.panner.Tracker");

dojo.declare("rmpw.panner.Tracker", null, {
	
	label: null,
	
	items: null,
	
	stopped: true,
	
	constructor: function(label) {
		this.label = label;
		this.items = [];
	},
	
	destroy: function() {
		this.stop();
	},
	
	getLabel: function() {
		return this.label;
	},
	
	startTracking: function(item) {
		item.activate();
	},
	
	stopTracking: function(item) {
		item.deactivate();
	},
	
	addItem: function(item) {
		var i = dojo.indexOf(this.items, item);
		if (i === -1) {
			item.tracker = this;
			this.items.push(item);
			if (!this.stopped) {
				this.startTracking(item);
			}
		}
	},
	
	removeItem: function(item) {
		var i = dojo.indexOf(this.items, item);
		if (i !== -1) {
			var item = this.items.splice(i, 1)[0];
			if (!this.stopped) {
				this.stopTracking(item);
			}
			delete item.tracker;
		}
	},
	
	getItems: function() {
		return this.items.slice();
	},
	
	start: function() {
		if (this.stopped) {
			this.stopped = false;
			dojo.forEach(this.items, this.startTracking, this);
		}
	},
	
	stop: function() {
		if (!this.stopped) {
			dojo.forEach(this.items, this.stopTracking, this);
			this.stopped = true;
		}
	}
});

dojo.declare("rmpw.panner.Tracker.TrackedItem", null, {
	
	label: "",
	tracker: null,
	
	constructor: function(params) {
		if (params) {
			dojo.mixin(this, params);
		}
	},
	
	getLabel: function() {
		return this.label;
	},
	
	/*
	 * The object might look like:
	 * `{ l: 50, t: 200, w: 300: h: 150 }`
	 */
	getBoundingBox: function() {
		return {x: 0, y: 0, width: 0, height: 0};
	},
	
	/*
	 * A point located within the bounding box.
	 * 
	 * The object might look like:
	 * `{ x: 50, y: 200 }`
	 */
	getPoint: function() {
		// default is the center of the bounding box
		var bb = this.getBoundingBox();
		return {
			x: bb.x + Math.floor(bb.width / 2),
			y: bb.y + Math.floor(bb.height / 2)
		};
	},
	
	boundsChanged: function() {},
	
	activate: function() {},
	
	deactivate: function() {}
});

dojo.declare("rmpw.panner.Tracker.NodeTrackedItem", rmpw.panner.Tracker.TrackedItem, {
	
	tracker: null,
	node: null,
	
	getBoundingBox: function() {
		return {
			x: this.node.offsetLeft,
			y: this.node.offsetTop,
			width: this.node.offsetWidth,
			height: this.node.offsetHeight
		};
	}
});

})();