/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.layout.graph.TreeGraphLayout");

dojo.require("rmpw.diagrams.layout.graph.AbstractGraphLayout");
dojo.require("rmpw.diagrams.layout.client.util.HashMap");
dojo.require("rmpw.diagrams.layout.client.util.ArrayList");

/**
 * Utility class used by tree layout to encapsule a set of data
 * for the extreme values in the graph.
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.layout.TreeLayout$Extreme", null, {
	lev: 0,
	off: 0,
	t: null,

	constructor: function(lev, off, t) {
		this.lev = lev;
		this.off = off;
		this.t = t;
	}
});

/**
 * Utility class used by tree layout to encapsule a set of data
 * for the different data metrics in the tree.
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.layout.TreeLayout$TreeMetrics", null, {
	count: 0,
	height: 0,
	xMax: 0,
	xMin: 0,

	constructor: function(count, height, xMax, xMin) {
		this.count= count;
		this.height = height;
		this.xMax = xMax;
		this.xMin = xMin;
	}
});

/**
 * Utility class used by tree layout to encapsule the data
 * for a single node in the tree graph
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.layout.TreeLayout$TreeNode", null, {
	members: null,
	isRoot: false,
	isThread: false,
	lthoff: 0,
	offset: 0,
	rthoff: 0,

	constructor: function(rect, data){
		dojo.mixin(this, new Node());
		$Node_0(this, data, null);
		this.members = $NodeList(new NodeList());
		new rmpw.diagrams.render.util.Insets(1, 1, 1, 1);
		this.isRoot = true;
		this.rank = 0;
		this.isThread = false;
		this.lthoff = 0;
		this.rthoff = 0;
		this.offset = 0;
		this.width = rect.width;
		this.height = rect.height;
		this.x = rect.x;
		this.y = rect.x;
	},
	
	getChild: function(i){
		return $getNode(this.members, i);
	},

	setThread: function (x) {
		if (this.isThread) {
			$set_0(this.members, 0, x);
		}
		else {
			this.isThread = true;
			$add_0(this.members, x);
		}
	},

	sortChildren: function() {
		var childrenArray, i;
		childrenArray = $toArray(this.members);
		mergeSort(childrenArray, 0, childrenArray.length, this?this:NATURAL);
		for (i = 0; i < this.members.size; ++i)
			$set_0(this.members, i, childrenArray[i]);
	},

	compare: function(obj1, obj2) {
		var c1, c2;
		c1 = obj1;
		c2 = obj2;
		if (c1.x + ~~(c1.width / 2) < c2.x + ~~(c2.width / 2))
			return -1;
		else if (c1.x + ~~(c1.width / 2) > c2.x + ~~(c2.width / 2))
			return 1;
		return 0;
	}
});

/**
 * Utility class used by tree layout to encapsule the data
 * for a root node in the tree graph
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.layout.TreeLayout$TreeRoot", null, {
	metrics: null,
	leftmost: null,
	rightmost: null,
	root: null,

	constructor: function(theRoot){
	  this.metrics = new rmpw.diagrams.layout.TreeLayout$TreeMetrics();
	  this.leftmost = new rmpw.diagrams.layout.TreeLayout$Extreme();
	  this.rightmost = new rmpw.diagrams.layout.TreeLayout$Extreme();
	  this.root = theRoot;
	}
});

/**
 * An autolayout algorithm that will arrange a set of nodes in a radial fashion.
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.layout.graph.TreeGraphLayout", rmpw.diagrams.layout.graph.AbstractGraphLayout, {
	
	assignAbsoluteXCoordinates: function(t, fatherpos){
		var c;
		t.x = fatherpos + t.offset - ~~(t.width / 2);
		if (t.isThread) {
			$remove_3(t.members, $get_0(t.members, 0));
			t.isThread = false;
		}
		else {
			for (c = 0; c < t.members.size; ++c) {
				this.assignAbsoluteXCoordinates($getNode(t.members, c), t.x + ~~(t.width / 2));
			}
		}
	},

	assignRelativeXCoordinates: function(t, level, lmost, rmost, mm){
		var c, count, cursep, islthread, isrthread, l, loffsum, newlmost, newrmost, off, r, requiredsep, roffsum, rootsep, separation, totalsep;
		++mm.count;
		count = t.members.size;
		t.rank = level;
		if (count == 0) {
			rmost.t = t;
			lmost.t = t;
			rmost.lev = level;
			lmost.lev = level;
			rmost.off = 0;
			lmost.off = 0;
		}
		else {
			newlmost = new rmpw.diagrams.layout.TreeLayout$Extreme();
			newrmost = new rmpw.diagrams.layout.TreeLayout$Extreme();
			separation = $ArrayList(new ArrayList());
			totalsep = 0;
			t.sortChildren();
			l = $getNode(t.members, 0);
			this.assignRelativeXCoordinates(l, level + 1, lmost, rmost, mm);
			for (c = 1; c < count; ++c) {
				r = $getNode(t.members, c);
				this.assignRelativeXCoordinates(r, level + 1, newlmost, newrmost, mm);
				cursep = 20;
				rootsep = cursep;
				loffsum = 0;
				roffsum = 0;
				while (!!l && !!r) {
					requiredsep = 20 + ~~((l.width + r.width) / 2);
					if (cursep < requiredsep) {
						rootsep += requiredsep - cursep;
						cursep = requiredsep;
					}
					islthread = l.isThread;
					if (l.members.size < 1) {
						l = null;
					}
					else {
						l = l.getChild(l.members.size - 1);
						if (islthread) {
							cursep -= l.rthoff;
							loffsum += l.rthoff;
						}
						else {
							cursep -= l.offset;
							loffsum += l.offset;
						}
					}
					isrthread = r.isThread;
					if (r.members.size < 1) {
						r = null;
					}
					else {
						r = $getNode(r.members, 0);
						if (isrthread) {
							cursep += r.lthoff;
							roffsum -= r.lthoff;
						}
						else {
							cursep += r.offset;
							roffsum -= r.offset;
						}
					}
				}
				$add_0(separation, rootsep);
				totalsep += rootsep;
				if (r) {
					lmost.t.setThread(r);
					r.lthoff = totalsep - lmost.off - roffsum;
				}
				if (l) {
					newrmost.t.setThread(l);
					l.rthoff = -rootsep - newrmost.off + loffsum;
				}
				if (newrmost.lev == rmost.lev) {
					rmost.t = newrmost.t;
					rmost.off = newrmost.off;
					rmost.lev = newrmost.lev;
					rmost.off += totalsep;
				}
				else if (newrmost.lev > rmost.lev) {
					rmost.t = newrmost.t;
					rmost.off = newrmost.off;
					rmost.lev = newrmost.lev;
					rmost.off += totalsep;
					lmost.t = newlmost.t;
					lmost.off = newlmost.off;
					lmost.lev = newlmost.lev;
					lmost.off += totalsep;
				}
				l = $getNode(t.members, c);
			}
			off = ~~(-(totalsep + 1) / 2);
			lmost.off += off;
			rmost.off += off;
			$add_0(separation, 0);
			if (off < mm.xMin) {
				mm.xMin = off;
			}
			for (c = 0; c < count; ++c) {
				$getNode(t.members, c).offset = off;
				off += separation.array[c];
			}
			if (off > mm.xMax) {
				mm.xMax = off;
			}
		}
	},
	
	assignYCoordinates: function(root, initialYcoord, metrics){
		var c, h, lev, q, t, ycoord, node;
		q = [];
		lev = 0;
		ycoord = initialYcoord;
		h = 0;
		q.push(root);
		while (q.length > 0) {
			t = q.shift();

			if (t.rank != lev) {
				ycoord += h + 64;
				h = 0;
				lev = t.rank;
			}
			t.y = ycoord;
			if (t.height > h) {
				h = t.height;
			}
			for (c = 0; c < t.members.size; ++c) {
				q.push($getNode(t.members, c));
			}
		}
		metrics.height = ycoord + h - initialYcoord;
	},

	calculateTreeRoots: function (nodeMap, edges) {
		var edge, headNode, iterMap, li, n, roots, tailNode, tr, entrySet, outerIter, value;
		li = $AbstractList$ListIteratorImpl(new AbstractList$ListIteratorImpl(), edges);
		while (li.i < li.this$0_0.size_0()) {
			edge = $next_0(li);
			headNode = $get(nodeMap, edge.target);
			tailNode = $get(nodeMap, edge.source);
			if (!headNode || !tailNode) {
				return null;
			}
			if (tailNode.isRoot) {
				$add_0(headNode.members, tailNode);
				tailNode.isRoot = false;
			}
			else {
				return null;
			}
		}
		roots = [];
		entrySet = $AbstractHashMap$EntrySet(new AbstractHashMap$EntrySet(), nodeMap);
		outerIter = $AbstractHashMap$EntrySetIterator(new AbstractHashMap$EntrySetIterator(), $AbstractMap$2(new AbstractMap$2(), nodeMap, entrySet).val$entrySet.this$0);
		iterMap = $AbstractMap$2$1(new AbstractMap$2$1(), outerIter);
		while ($hasNext(iterMap.val$outerIter.iter)) {
			value = $next_0(iterMap.val$outerIter.iter).getValue();
			n = value;
			if (n.isRoot) {
				tr = new rmpw.diagrams.layout.TreeLayout$TreeRoot(n);
				roots.push(tr);
			}
		}
		
		return roots;
	},

	runTreeLayout: function (nodes, edges){
		var child, edge, graph2, initialY, iter, li, node, nodeMap, origRect, points, roots, tr, treeNode, entrySet, outerIter, entry;
		li = $AbstractList$ListIteratorImpl(new AbstractList$ListIteratorImpl(), nodes);
		nodeMap = $HashMap(new HashMap());
		while (li.i < li.this$0_0.size_0()) {
			child = $next_0(li);
			treeNode = new rmpw.diagrams.layout.TreeLayout$TreeNode(new rmpw.diagrams.render.util.Rectangle(child.x, child.y, child.width, child.height), child);
			!child?$putNullSlot(nodeMap, treeNode):$putHashValue(nodeMap, child, treeNode, ~~(child.$H || (child.$H = ++sNextHashId)));
		}
		initialY = 0;
		roots = this.calculateTreeRoots(nodeMap, edges);
		if (!roots || roots.length == 0) {
			return false;
		}
		else {
			for (var i=0, lg=roots.length; i<lg; i++) {
				tr = roots[i];
				tr.metrics.xMin = tr.root.x;
				tr.metrics.xMax = tr.metrics.xMin + tr.root.width;
				this.assignRelativeXCoordinates(tr.root, 0, tr.leftmost, tr.rightmost, tr.metrics);
				this.assignAbsoluteXCoordinates(tr.root, 0);
				this.assignYCoordinates(tr.root, initialY, tr.metrics);
				initialY += tr.metrics.height + 64;
			}
			entrySet = $AbstractHashMap$EntrySet(new AbstractHashMap$EntrySet(), nodeMap);
			outerIter = $AbstractHashMap$EntrySetIterator(new AbstractHashMap$EntrySetIterator(), $AbstractMap$1(new AbstractMap$1(), nodeMap, entrySet).val$entrySet.this$0);
			iter = $AbstractMap$1$1(new AbstractMap$1$1(), outerIter);
			while ($hasNext(iter.val$outerIter.iter)) {
				entry = $next_0(iter.val$outerIter.iter);
				node = entry.getKey();
				treeNode = !node?nodeMap.nullSlot:$getHashValue(nodeMap, node, ~~(node.$H || (node.$H = ++sNextHashId)));
				node.x = treeNode.x;
				node.y = treeNode.y;
				node.width = treeNode.width;
				node.height = treeNode.height;
			}
			li = $AbstractList$ListIteratorImpl(new AbstractList$ListIteratorImpl(), edges);
	  		while (li.i < li.this$0_0.size_0()) {
	    		edge = $next_0(li);
	    		points = new rmpw.diagrams.render.util.PointList();
	    		points.addPoint_0(edge.source.x + ~~(edge.source.width / 2), edge.source.y + ~~(edge.source.height / 2));
	    		points.addPoint_0(edge.target.x + ~~(edge.target.width / 2), edge.target.y + ~~(edge.target.height / 2));
	    		edge.points = points;
	    		edge.start = points.getPoint( 0);
	    		edge.end = points.getPoint( points.size - 1);
	  		}
	  		
	  		return true;
		}
	},

	runLayoutAlgorithm: function(graph) {
		var success = this.runTreeLayout(graph.nodes, graph.edges);
		if (!success) {
			dojo.require("rmpw.diagrams.layout.graph.GMFDirectedGraphLayout");
			var layout = new rmpw.diagrams.layout.graph.GMFDirectedGraphLayout();
			success = layout.runLayoutAlgorithm(graph);
		}
		
		return success;
	}
});