/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.panner.ZoomSlider");

dojo.require("dijit._Widget");
dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.require("dijit.form.HorizontalSlider");

dojo.require("dijit._Templated");
dojo.require("dojo.fx.easing");

dojo.requireLocalization("rmpw.panner", "ZoomSliderMessages");

var messages = dojo.i18n.getLocalization("rmpw.panner", "ZoomSliderMessages");

dojo.declare("rmpw.panner.ZoomSlider", dijit._Widget, {
	
	slider: null,
	
	scale: 1,
	
	postCreate: function(toolsLayer) {
		this.slider = this.createSlider();
		this.domNode.appendChild(this.slider.domNode);
		this.decorate(this.slider);
	},
	
	destroy: function() {
		this.slider.destroyRecursive();
		this.inherited(arguments);
	},
	
	createSlider: function() {
		var slider = new rmpw.panner.ZoomSlider._Slider({
				value: this.scale,
				minimum: 0.25,
				maximum: 5,
				showButtons: true,
				intermediateChanges: true,
				discreteValues: 20, // 25% - 500% at 25% intervals
				onChange: dojo.hitch(this, this._onChange)
			});
		
		slider.decrementButton.title = messages.zoomIn;
		slider.incrementButton.title = messages.zoomOut;
		slider.sliderHandle.title = "100%";
		
		// exploding text container
		this._exTextContainer = dojo.doc.createElement("div");
		this._exTextContainer.style.position = "relative";
		this._exTextContainer.style.height = "0px";
		this._exTextContainer.style.color = "#31598C";
		if (dojo.isIE) {
			// fix empty div height
			var ieFix = dojo.doc.createElement("span");
			ieFix.style.lineHeight = "0px";
			this._exTextContainer.appendChild(ieFix);
		}
		slider.containerNode.appendChild(this._exTextContainer);
		
		return slider;
	},
	
	decorate: function(slider) {
		// add 100% mark
		var ruleContainer = dojo.doc.createElement("div");
		ruleContainer.className = "dijitRuleContainer dijitRuleContainerH";
		ruleContainer.style.height = "5px";
		
		var rule = dojo.doc.createElement("div");
		rule.className = "dijitRuleMark dijitRuleMarkH";
		rule.style.left = ((1 - slider.minimum) / (slider.maximum - slider.minimum)) * 100 + "%";
		
		ruleContainer.appendChild(rule);
		slider.bottomDecoration.appendChild(ruleContainer);
		
		// add 100% label
		var ruleLabelContainer = dojo.doc.createElement("div");
		ruleLabelContainer.className = "dijitRuleContainer dijitRuleContainerH dijitRuleLabelsContainer dijitRuleLabelsContainerH";
		ruleLabelContainer.style.height = "1em";
		ruleLabelContainer.style.fontSize = "75%";
		ruleLabelContainer.style.color = "#808080";
		
		var ruleLabel = dojo.doc.createElement("div");
		ruleLabel.className = "dijitRuleLabelContainer dijitRuleLabelContainerH";
		ruleLabel.style.left = ((1 - slider.minimum) / (slider.maximum - slider.minimum)) * 100 + "%";
		
		var ruleText = dojo.doc.createElement("span");
		ruleText.className = "dijitRuleLabel dijitRuleLabelH";ruleText
		ruleText.appendChild(dojo.doc.createTextNode("100%"));
		
		ruleLabel.appendChild(ruleText);
		ruleLabelContainer.appendChild(ruleLabel);
		slider.bottomDecoration.appendChild(ruleLabelContainer);
	},
	
	_onChange: function(scale) {
		this.scale = scale;
		var percentage = Math.round(scale * 100) + "%";
		this.slider.sliderHandle.title = percentage;
		
		var childNodes = this._exTextContainer.childNodes;
		for (var i = 0; i < childNodes.length; ++i) {
			var opacity = dojo._getOpacity(childNodes[i]);
			dojo._setOpacity(childNodes[i], opacity * 0.5);
		}
		
		this._expandText(percentage);
		
		childNodes = null;
		percentage = null;
		
		setTimeout(dojo.hitch(this, function() {
			this.onChange(scale);
		}), 100);
	},
	
	_expandText: function(text) {
		var expandingText = new rmpw.panner.ZoomSlider._ExpandingText({
				text: text,
				onEnd: dojo.hitch(this, function() {
					expandingText.destroyRecursive();
				})
			});
		expandingText.domNode.style.position = "absolute";
		expandingText.domNode.style.left = "50%";
		this._exTextContainer.appendChild(expandingText.domNode);
		
		expandingText.startup();
	},
	
	onChange: function(scale) {}
});

dojo.declare("rmpw.panner.ZoomSlider._Slider", dijit.form.HorizontalSlider, {
	
	decrement: function(e) {
		this._fixValue(1);
		this.inherited(arguments);
	},

	increment: function(e) {
		this._fixValue(0);
		this.inherited(arguments);
	},
	
	_fixValue: function(steps) {
		if (this.discreteValues != Infinity) {
			var step = (this.maximum - this.minimum) / (this.discreteValues - 1);
			var offset = this.value % step;
			if (offset != 0) {
				this.value = this.value - offset + (step * steps);
			}
		}
	}
});

dojo.declare("rmpw.panner.ZoomSlider._ExpandingText", [dijit._Widget, dijit._Templated], {
	
	templateString: '<div style="display: inline-block; text-align: center; font-weight: bold; z-index: 500;">' +
			'<span dojoAttachPoint="containerNode" style="left: -50%; position: relative;"></span>' +
		'</div>',
	
	text: null,
	
	duration: 1000,
	
	startFontSize: 25, // percentage
	endFontSize: 300, // percentage
	
	postCreate: function() {
		this.containerNode.appendChild(dojo.doc.createTextNode(this.text));
		this.domNode.style.fontSize = this.startFontSize + "%";
	},
	
	startup: function() {
		this.inherited(arguments);
		
		var args = {
			duration: this.duration,
			curve: [this.startFontSize, this.endFontSize],
//			easing: dojo.fx.easing.bounceOut,
			easing: dojo.fx.easing.expoOut,
			onEnd: dojo.hitch(this, function() {
				delete this._animation;
				this.onEnd();
			}),
			onAnimate: dojo.hitch(this, function(r) {
				this.domNode.style.fontSize = r + "%";
			})
		};
		
		this._animation = new dojo._Animation(args);
		this._animation.play();
	},
	
	onEnd: function() {
		
	}
});

})();