/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.actions.Action");

dojo.require("rmpw.common.policy.Policy");

/**
 * Generic action for invoking a command and executing it on the command stack
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.actions.Action", rmpw.common.policy.Policy, {
	
	constructor: function() {
		this._options = {};
	},

	setCommandArguments: function(args) {
		this._options = args;
	},
	
	/*
	 * Native code can be set from a string into this action.  In this implementation it is
	 * assumed to be JavaScript code.  
	 */
	setEnablementCallback: function(codeBody) {
		var index = {};
		
		var codeObj = new Function("args", codeBody);
		this._enablementCallback = dojo.hitch(this, codeObj);
	},
	
	/*
	 * Convenience method for direct runtime extensibility.  It is easier to pass a function
	 * for callback then a JSON string.
	 */
	setEnablementCallbackFunction: function(codeFunction) {
		this._enablementCallback = dojo.hitch(this, codeFunction);
	},
	
	/*
	 * Call to see if this action can be enabled or not
	 */
	isEnabled: function(args) {
		if (this._enablementCallback) {
			return this._enablementCallback(args);
		}
		
		return true;
	},
	
	/*
	 * Gets the command to be execution by the action
	 */
	getCommand: function(e) {
		return new this.commandClass;
	},
	
	setChainedInput: function(chainedInput) {
		this._chainedInput = chainedInput;
	},
	
	getChainedInput: function() {
		return this._chainedInput;
	},
	
	setChainedOutput: function(chainedOutput) {
		this._chainedOutput = chainedOutput;
	},
	
	getChainedOutput: function() {
		return this._chainedOutput;
	},
	
	/*
	 * Gets the arguments passed into the command at execution time
	 */
	getExecutionArguments: function (e) {
		// TODO (MG): doesn't seem like a command should be passed the "viewer"
		var args = { viewer: e.viewer ? e.viewer : this.host.renderMapping.root, targetController: this.host };
		dojo.mixin(args, this._options);
		return args;
	},
	
	executeCommand: function(e) {
		var root = e.viewer ? e.viewer : this.host.root;
		var commandStack = root.getCommandController().commandStack;
		var command = this.getCommand(e);
		if (command) {
			var args = this.getExecutionArguments(e);
			if (dojo.isFunction(this.onCancel)) {
				args.onCommitBlocked = dojo.hitch(this, this.onCancel);
			}
			return commandStack.execute(command, args);
		}
		
		return false;
	},
	
	run: function(e){
		if (this.isEnabled(e)) {
			return this.executeCommand(e);
		}
	},
	
	waiting: function(wait) {
		this.setCursors(this.host, wait ? 'progress': '');
	},
	
	setCursors: function(editController, cursorType) {
		editController.renderMapping.viewer.container.style.cursor=cursorType;	
		var shape = editController.getDefaultTargetShape ? editController.getDefaultTargetShape() : null;
		if (shape) {
			shape.rawNode.style.cursor=cursorType;	
		}
	}

	
});