/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.actions.AddElement");

dojo.require("rmpw.diagrams.edit.actions.Action");
dojo.require("rmpw.forms.ui.EditOrCreateResource");
dojo.require("rmpw.forms.ui.SelectOrCreateResource");
dojo.require("rmpw.common.util.IdentifierUtil");

/**
 *Add class 
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.actions.AddElement", rmpw.diagrams.edit.actions.Action, {
	
	constructor: function() {
		this._options.isDefinable = true; // assume true until we support Fragment on freeform diagram
	},
	
	setOwnerTrace: function(ownerTrace) {
		this._options.ownerTrace = ownerTrace;
	},
	
	setTypeId: function(typeId) {
		this._options.typeId = typeId;
	},
	
	setAddTypeId: function(addTypeId) {
		this._options.addTypeId = addTypeId;
	},
	
	setOwnerChildrenProperty: function(ownerChildrenProperty) {
		this._options.ownerChildrenProperty = ownerChildrenProperty;
	},
	
	suppressLabelIncrement: function(suppress) {
		this._options.suppress = suppress;
	},
	
	setRealtimeMode: function(realtime) {
		this._realtime = realtime;
	},
	
	getElementPosition: function(e) {
		var renderMapping = this.host.renderMapping;
		
		// try to get creation point from context action bar, else use action button position
		var controller = renderMapping.root.policies.contextActionBarScope;
		var offset = renderMapping.root.viewport.getAbsoluteCanvasOffset();
        var e = e.touches ? e.touches[0] : e; 
		var p = controller.getCreationPoint() || {x: e.clientX - offset.x, y:e.clientY - offset.y};
		var relpos = new rmpw.diagrams.render.util.Point(p.x, p.y); 
		
		// find relative location
		if (this.host.renderMapping.policies.graphContainer) {
			var isShapeContainer = !this.host.renderMapping.policies.graphContainer || 
			!this.host.renderMapping.policies.graphContainer.isTopLevel();
			renderMapping.policies.graphContainer.translateToRelative(relpos);
			if (isShapeContainer) {
				if (relpos.x<10){ 
				    relpos.x=10;
				}
				if (relpos.y<22){ 
				    relpos.y=22;
				}
			}
		}
		
		return relpos;
	},
	
	/*
	 * Gets the arguments passed into the command at execution time
	 */
	getExecutionArguments: function (e) {
		var editController = this.host;
		var relpos = this.getElementPosition(e);
		var ownerElement = editController.renderMapping.getSourceByPath(this._options.ownerTrace);
		
		return {
			ownerElement: ownerElement,
			x: relpos.x,
			y: relpos.y,
			suppressIncrement: this._options.suppress,
			viewer: editController.root,
			diagram: editController.renderMapping
		};
	},
	
	/*
	 * Gets the command to be execution by the action
	 */
	getCommand: function(e) {
		var commandPackage = this.host.root.getCommandPackage();
		
		var createElCmd = commandPackage.select({
			typeId: [this._options.typeId],
			addTypeId: this._options.addTypeId,
			ownerChildrenProperty: this._options.ownerChildrenProperty,
			commandId: "rmpw.common.command.AddElement"
		});
		
		return createElCmd;
	},
	
	run: function(e) {
		
		var editController = this.host;
		var relpos = this.getElementPosition(e);
		
		if (!this._realtime) {		
			this.waiting(true);
			this._connectAddChild = dojo.connect(this.host.renderMapping, "addChild", dojo.hitch(this, function(child) {
				dojo.disconnect(this._connectAddChild);
				delete this._connectAddChild;
				
				if (child.policies.graphNode) {
					child.policies.graphNode.setInitialPosition(relpos.x, relpos.y);
				}
				if (child.__editController.policies.selectable) {
					child.__editController.policies.selectable.selectOnUpdate();
				}
				
				child.enforcePolicies();
				this.waiting(false);
			}));
		}
		else {
			if (!this._pendingChildren) this._pendingChildren = [];
			this._pendingChildren.push(relpos);
			this.waiting(true);
			this.addCreationFeedback(editController);
			
			// action may run multiple times before children are created
			if (!this._connectAddChild) {
				this._connectAddChild = dojo.connect(this.host.renderMapping, "addChild", dojo.hitch(this, function(child) {
					// get location of child if it's a node
					var pt = this._pendingChildren.shift();
					if (child.policies.graphNode) {
						child.policies.graphNode.setInitialPosition(pt.x, pt.y);
					}
					if (child.__editController.policies.selectable) {
						child.__editController.policies.selectable.selectOnUpdate();
					}
					child.enforcePolicies();
					
					this._cleanup(true);
				}));
			}
		}
			
		var command = this.getCommand(e);

		if (command && ((command._options && command._options.isDocument) || !this._options.isDefinable)) {
			var dialog = new rmpw.forms.ui.SelectOrCreateResource({
				typeUri: command._options.addTypeId,
				typeLabel: command._options.defaultName,
				typeForm: rmpw.common.util.IdentifierUtil.getIdentifier(command._options.addTypeId), isFragment: false,
				isDocument: command._options.isDocument, definableTypes: [],
				isReadOnly: false, canSelect: true,
				formObject: this.host.root.diagram.getDefaultSource(), isConstrained: false,
				documentObject : this.host.root.diagram.getDefaultSource(),
				editor: this.host.root.definition.uri,
				page: this.host.root.page,
				onSelectTarget: this,
				onSelectFunctionName: "addAllReferences",
				onCancelFunctionName: "_cleanup",
				onSelectArgs: {e: e, command: command}
			});

			dialog.useSelectDialog();
		}
		else if (command && command._options && !command._options.generateIdentifier) {
			// check if identifier is not generated - if not invoke creation dialog
			var target = { rdf_type: [{rdf_about: command._options.addTypeId}] };

			var dialog = new rmpw.forms.ui.EditOrCreateResource();
			var dialogArgs = {
				typeName: command._options.defaultName, 
				typeForm: rmpw.common.util.IdentifierUtil.getIdentifier(command._options.addTypeId), 
				defaultUri: "", 
				defaultLabel: null, 
				parent: this, 
				featureId: "dmcodegen_defines", 
				target: target
			};			
			dialog.editOrCreateResource(dialogArgs);
			dojo.connect(dialog, "onCancel", dojo.hitch(this, function() {
				this._cleanup();
			}));
			dojo.connect(dialog, "onSubmit", dojo.hitch(this, function(createdResource) {
				this.addReference({e: e, target: createdResource, command: command});
				this.waiting(false);
			}));

			return true;
		}
		else {
			var bret = this.inherited(arguments);
			if (!bret) {
				this._cleanup();
			}
		}
	},
	
	onCancel: function() {
		this._cleanup();
	},
	
	_cleanup: function(incremental) {
		if (this._pendingChildren && this._pendingChildren.length == 0 || !incremental) {
			dojo.disconnect(this._connectAddChild);
			delete this._connectAddChild;
			delete this._pendingChildren;
		}
		this.waiting(false);
		this.removeCreationFeedback();
	},
	
	addAllReferences: function(args) {
		if (dojo.isArray(args)) {
			for (var i = 0, arg; arg = args[i]; i++) {
				this.addReference(arg);
			}
		} else {
			this.addReference(args);
		}		
	},
	
	addReference: function(args) {
		var exargs = this.getExecutionArguments(args.e);
		var command = args.command;
		exargs.semanticModel = args.target;
		
		// need to get the contents so it can be displayed and edited in the diagram
		if (command._options.isDocument && !args.target.rdf_type) {
			rmps.core.client.xhrGet({
				url : net.jazz.ajax._contextRoot + "/rdfdirectory/?logicalURI=" + encodeURIComponent(args.target.rdf_about),
				headers: {
				"Accept": "application/x-rdf+json-graph",
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "text",
				load: dojo.hitch(this, function(data) {
					var documents = rmpw.common.vcs.ref.fromJson(data, {
						index: {}, idAttribute: "rdf_about", loader: function() { }
					});
					exargs.semanticModel = documents[0];
					this._doAddReference(command, exargs);
				})
			});	
		}
		else {
			this._doAddReference(command, exargs);
		}
	},
	
	_doAddReference: function(command, args) {
		var commandStack =this.host.root.getCommandController().commandStack;
		
		var bret = commandStack.execute(command, args);
		if (!bret) {
			this._cleanup();
		}
	},
	
	addCreationFeedback: function(editController) {
		if (editController.renderMapping.policies.graphContainer) {
			// add feedback to show where shape will be created
			var feedback = this._createFeedback(editController);
			if (feedback) {
				if (!this.creationFeedback) {
					this.creationFeedback = [];
				}
				this.creationFeedback.push(feedback);
			}
		}
	},

	_createFeedback: function(editController) {
		if (editController.renderMapping.policies.graphContainer) {
			var pt = editController.root.policies.contextActionBarScope.createAt;
			if (pt) {
				var creationFeedback = editController.renderMapping.root.feedbackLayer.createGroup();
				var rect = creationFeedback.createRect( { x:pt.x,  y: pt.y, width: 8, height: 8 } );
				rect.setStroke({color: 'black', lineWidth: 1, lineStyle: "solid"});
				rect.rawNode.setAttribute("pointer-events", "none");
				rect.rawNode.style.shapeRendering = "crispEdges";
				return creationFeedback;				
			}
		}
		return null;
	},
	
	removeCreationFeedback: function(editController, pt) {
		if(this.creationFeedback){
			for (var i = 0; i < this.creationFeedback.length; i++) {
				this.creationFeedback[i].removeShape();
			}
			delete this.creationFeedback;	
		}
	}

	
});

