/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.actions.Compound");

dojo.require("rmpw.diagrams.edit.actions.Action");
dojo.require("rmpw.diagrams.edit.actions.CommandWrapper");

/**
 * Action which can contain a number of sub actions to be executed as one
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.actions.Compound", rmpw.diagrams.edit.actions.Action, {
	
	/*
	 * Add actions
	 */
	setActions: function () {
		this._actions = arguments;
	},
	
	addAction: function (action) {
		if (!this._actions) {
			this._actions = [];
		}
		this._actions.push(action);
	},
	
	/*
	 * Gets the arguments passed into the command at execution time
	 */
	getExecutionArguments: function (e) {
		return e;
	},
	
	isEnabled: function(args) {
		var enable = true;
		for (var i=0, action; action = this._actions[i]; i++) {
			if (action.isEnabled) {
				enable &= action.isEnabled(args);
			}
		}
		
		return enable;
	},
	
	/*
	 * Gets the command to be execution by the action
	 */
	getCommand: function(e) {
		var compoundCmd = new rmpw.common.command.CompoundCommand();
		
		for (var i=0, action; action = this._actions[i]; i++) {
			var cmd = action.getCommand(e);
			if (!cmd) {
				cmd = new rmpw.diagrams.edit.actions.CommandWrapper({}, action);
			}
			
			compoundCmd.add(cmd, action.getExecutionArguments(e), action.getChainedInput(), action.getChainedOutput());
		}
		
		return compoundCmd;
	}
});