/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.actions.DeleteDependentEdgeElement");

dojo.require("rmpw.diagrams.edit.actions.DeleteElement");
dojo.require("rmpw.common.command.CompoundCommand");

/**
 * Utility for deleting dependent edge elements given a target node.  This is probably a tactical solution
 * for getting delete working until the command generation can understand dependent elements and ensure
 * they are deleted with the owning element.
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.actions.DeleteDependentEdgeElement", rmpw.diagrams.edit.actions.DeleteElement, {

	getCommand: function(e) {
	    var i = 0,
	       edge;
		var graphAnchorable = e.targetController.renderMapping.policies.anchorable;
		if (!e.deletedEdges) {
			e.deletedEdges = {};
		}
		var edges = {};
		if (graphAnchorable.ownedConnections) {
			for (i=0, edge=null; edge=graphAnchorable.ownedConnections[i]; i++) {
				if (!e.deletedEdges[edge.id]) {
					edges[edge.id] = edge;
				}
			}
		}
		if (graphAnchorable.targetConnections) {
			for (i=0, edge=null; edge=graphAnchorable.targetConnections[i]; i++) {
				if (!e.deletedEdges[edge.id]) {
					edges[edge.id] = edge;
				}
			}
		}
		
		var compoundCmd = new rmpw.common.command.CompoundCommand();
		for (var id in edges) {
			var cmd = this.inherited(arguments); // get a new command for each edge
			if (cmd) {
				var args = {};
				dojo.mixin(args, e);
				args.targetController = edges[id].__editController;
				args.elementToDelete = args.targetController.renderMapping.getSourceByPath(this._options.elementTrace);
				args.ownerElement = this.host.renderMapping.getSourceOwnerByPath(this._options.ownerTrace);
				
				compoundCmd.add(cmd, args);
			}
		}
		
		dojo.mixin(e.deletedEdges, edges);
		return compoundCmd;
	}
});

