/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.actions.DeleteElement");

dojo.require("rmpw.diagrams.edit.actions.Action");

/**
 * Delete semantic action
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.actions.DeleteElement", rmpw.diagrams.edit.actions.Action, {

	setTypeId: function(typeId) {
		this._options.typeId = typeId;
	},
	
	/**
	 * Name of the trace on the target shape for the semantic element to be deleted.
	 */
	setElementTrace: function(elementTrace) {
		this._options.elementTrace = elementTrace;
	},

	/**
	 * Name of the trace on the host shape for the semantic element that owns the
	 * collection the element to be deleted belongs to.
	 */
	setOwnerTrace: function(ownerTrace) {
		this._options.ownerTrace = ownerTrace;
	},
	
	setRealtimeMode: function(realtime) {
		this._realtime = realtime;
	},
	
	onCancel: function() {
		if (this._hiddenController) {
			this.setElementVisibility(this._hiddenController, '');
		}
	},
	
	/*
	 * Gets the arguments passed into the command at execution time
	 */
	getExecutionArguments: function (e) {
		var elementToDelete;
		if(this._options.elementTrace) {
			elementToDelete = e.targetController.renderMapping.getSourceByPath(this._options.elementTrace);
		} else {
			elementToDelete = e.targetController.renderMapping.getDefaultSource();
		}
		
		return {
			elementToDelete: elementToDelete,
			ownerElement: this.host.renderMapping.getSourceOwnerByPath(this._options.ownerTrace)
		};
	},
	
	/*
	 * Gets the command to be executed by the action
	 */
	getCommand: function() {
		var commandPackage = this.host.root.getCommandPackage();
		var ownerTrace = this._options.ownerTrace;
		var ownerProperty;
		if (ownerTrace) {
			// last segment in ownerTrace corresponds to the property name on the owner;
			// it is needed in cases when there can be more that one reference of the same type 
			// referenced by a different property of the owner (e.g. three kinds of connections)
			var index = ownerTrace.indexOf(".");
			if (index > -1) {
				ownerProperty = ownerTrace.slice(index + 1);
			}
		}		
		var deleteCmd = commandPackage.select({
			typeId: this._options.typeId,
			commandId: this.getCommandId(),
			ownerChildrenProperty: ownerProperty
		});
		
		return deleteCmd;
	},
	
	
	run: function(e){
		delete this._hiddenController;
		if (this._realtime) {
			// hide shape immediately
			this._hiddenController = e.targetController;
			this.hideAllFeedback(this._hiddenController);
			this.setElementVisibility(this._hiddenController, 'hidden');
		}
		var bret = this.inherited(arguments);
		// if error, restore shape
		if (!bret && this._realtime) {
			this.setElementVisibility(this._hiddenController, '');
		}
	},
	
	setElementVisibility: function(controller, visibility) {
		var shape = controller.renderMapping && controller.renderMapping.rootShapes[0] ? controller.renderMapping.rootShapes[0] : null;
		if (shape) {
			shape.rawNode.style.visibility = visibility;
			
			// if a anchorable, also set visibility on connections
			var graphAnchorable = controller.renderMapping.policies.anchorable;
			if (graphAnchorable) {
				this._setConnectionVisibility(graphAnchorable.ownedConnections, visibility);
				this._setConnectionVisibility(graphAnchorable.targetConnections, visibility);
			}
		}
	},

	_setConnectionVisibility: function(connections, visibility) {
		if (connections) {
			for (var i=0, conn; conn=connections[i]; i++) {
				var shape = conn.rootShapes[0] ? conn.rootShapes[0] : null;
				if (shape) {
					shape.rawNode.style.visibility = visibility;
				}
			}
		}
	},
	
	hideAllFeedback: function(controller) {
		if (controller.policies.focusable) {
			controller.policies.focusable.hideFeedbackNoFade();			
		}
		for (var policy in controller.policies) {
			if (controller.policies[policy].hideFeedback) {
				controller.policies[policy].hideFeedback();
			}
		}
	},
	
	getCommandId: function() {
		return "rmpw.common.command.DeleteElement"; 
	}
});

