/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.actions.DirectEditText");

dojo.require("dojo._base.html");
dojo.require("rmpw.diagrams.render.util.Workarounds");
dojo.require("dojox.html.metrics");
dojo.require("rmpw.diagrams.edit.actions.ChangeProperty");

dojo.declare("rmpw.diagrams.edit.actions.DirectEditText", rmpw.diagrams.edit.actions.ChangeProperty, {
	textShape: null,
	wrappingText: false,
	input: null,
	subscriptions: null,
	
	constructor: function(host) {
		this._viewer = host.renderMapping.root;
		
		this._listeners = [];
	},
	
	setTextShape: function(textShape) {
		if(dojo.isString(textShape)) {
			this.textShape = this.host.renderMapping.template[textShape];
		} else {
			this.textShape = textShape;
		}
		this.wrappingText = this.textShape.shape.type === "wrappingText";
	},
	
	setEditExpression: function(regExp) {
		this._regExp = new RegExp(regExp);
	},
	
	run: function(args) {
	    this._viewer.getSelectionController().deselectAll();
	    this._viewer.policies.focusScope.disable();
	    this._oldValue = this.textShape.shape.text;		
		dojo.publish("/gef/directEdit/start");

	    // create a matching "text" input node or "textarea" for wrapping text
	    var textBounds;
	    this.inputBuffer = 20;
	    this._firstCharacter = args.charOrCode && dojo.isString(args.charOrCode) ? args.charOrCode : null;
	    var initialText = this._firstCharacter ?  this._firstCharacter :  this.textShape.shape.text;
	    
	    var matrix = this.host.renderMapping.root.viewport.contents.matrix;
	    if (!matrix) {
	    	matrix = dojox.gfx.matrix.identity;
	    }
	    this._preserveSize = matrix.xx <= 1 && matrix.yy <= 1;
	    if (this.wrappingText) {
            this.input = dojo.create("textarea", {
                value : initialText
            }, this._viewer.container);
            textBounds = this.textShape.getTransformedBoundingBox(true, this._preserveSize);
            if(this.textShape.isWrapped()){
                textBounds.height += 3;
            }
            dojo.style(this.input, "overflow", "hidden");
            // no scrollbars
            this._refreshTextAlignment();
            this.inputBuffer = 10; 
	    } else {
            this.input = dojo.create("input", {type:"text", value: initialText}, this._viewer.container);
		    textBounds = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(this.textShape);
	    }
	    
	    var fontSize = this._preserveSize ? this.textShape.fontStyle.size :  Math.floor(100 * matrix.xx) + "%";    
        if(this.textShape.fontStyle) {
		    dojo.style(this.input, "fontFamily", this.textShape.fontStyle.family);
		    dojo.style(this.input, "fontSize", fontSize);
		    dojo.style(this.input, "fontWeight", this.textShape.fontStyle.weight);
		}
	    
	    // if nothing in label right now, initialize to a size equal to 3 x's
	    if (textBounds.width===0) {
            var bnds = this._getBoundingTextBox("xxx", this.textShape.fontStyle);
            textBounds.width = bnds.w;
            textBounds.height = bnds.h;
	    }
	    
	    // position it directly over the text we're editing
	    dojo.style(this.input, "position", "absolute");
	    dojo.style(this.input, "left", textBounds.x + "px");
	    dojo.style(this.input, "top", textBounds.y + "px");
	    dojo.style(this.input, "width", textBounds.width + this.inputBuffer + "px");
	    dojo.style(this.input, "height", textBounds.height + "px");
	    dojo.style(this.input, "zIndex", 1000);	// by default low value, will be hidden under the canvas
	    
	    // react to mouse events
	    this._listeners.push(dojo.connect(this.input, "onmousedown", this, "_onMouseDown"));
	    
	    // handle enter and escape
	    this._listeners.push(dojo.connect(this.input, "onkeypress", this, "_onKeyPress"));

	    // handle expanding the textArea box to fit text as you type
	    this._listeners.push(dojo.connect(this.input, "onkeyup", this, "_onKeyUp"));

		// handle loss of focus.
		// this is needed to handle the user clicking outside of the diagram, like the Create a Comment column to enter a Subject.
		// fixes defect 44303: Cannot create free text in comment on Web client
	    this._listeners.push(dojo.connect(this.input, "onblur", this, "_commit"));

	    // do what onblur should be doing for us... but for some reason only worked every second click
	    this.subscriptions = [
                dojo.subscribe("/gef/directEdit/start", this, dojo.hitch(function() { this._commit(); })),
	            dojo.subscribe("/gfx/move/start", this, dojo.hitch(function() { this._commit(); }))];
	    
	    // focus the box
	    this.input.focus();
	    
	    if (this._firstCharacter) {
            // if there's a first character, put caret at end
            this.input.value = '';
            this.input.value = this._firstCharacter;
	    } else {
		    // else select all text
            this.input.select();
	    }
	    
	    // reset the focus shape focus to avoid conflict with accelerators
	    this._currentSelection = this._viewer.getSelectionController().getPrimarySelection();
		
	    this._listeners.push(dojo.connect(this._viewer.getSelectionController(), "onSelectionChanged", this, "_commit"));
	    
	    return true;
	},
	
	_onMouseDown: function(e) {
		if (e.button === dojo.mouseButtons.LEFT && this.input.setSelectionRange) {
			this.input.setSelectionRange(0,0);
		}
	},
	
	_onKeyPress: function(e) {
		if(e.keyCode === dojo.keys.ESCAPE) {
			this._cancel();
		// with wrapping text, allow user to enter multiple lines with shift-enter key
		} else if(e.keyCode === dojo.keys.ENTER && (!this.wrappingText || !e.shiftKey)) { 
			this._commit();
		}
	},
	
	_onKeyUp: function(e) {
	    var textBounds;
		if (!this.wrappingText || !this.host.renderMapping.policies || 
				!this.host.renderMapping.policies.resizeable) {
			textBounds = dojox.html.metrics.getTextBox(this.input.value, {
				fontFamily: dojo.style(this.input, "fontFamily"),
				fontSize: dojo.style(this.input, "fontSize"),
				fontWeight: dojo.style(this.input, "fontWeight"),
				whiteSpace: "nowrap"
			});
            var oldWidth = dojo.style(this.input, "width");
            if(oldWidth < textBounds.w + this.inputBuffer) {
                dojo.style(this.input, "width", textBounds.w + this.inputBuffer + "px");
            }
		} else {
			this.textShape.setText(this.input.value);
			this.host.renderMapping.policies.resizeable.resize();
            textBounds = this.textShape.getTransformedBoundingBox(true, this._preserveSize);
            if(this.textShape.isWrapped()){
                textBounds.height += 3;
            }
            if(textBounds.width < 20){
                textBounds.width = 20;
            }
            if(textBounds.height < 20){
                textBounds.height = 20;
            }
            dojo.style(this.input, "left", textBounds.x + "px");
            dojo.style(this.input, "top", textBounds.y + "px");
            dojo.style(this.input, "width", textBounds.width + this.inputBuffer + "px");
            dojo.style(this.input, "height", textBounds.height + "px");
            this._refreshTextAlignment(); 
		}
    },
    
    _commit: function () {
        var newValue = this.input.value;
        if (this._regExp) {
            // TODO (MG): for real regex support, the result has to be multi-valued, each component
			var components = this._regExp.exec(newValue);
			
			if(components[2]) {
				var argumentz = components[2].split(",");
				for(var i=0; i<argumentz.length; i++) {
					var argument = argumentz[i].split(":");
				}
			}
			
			newValue = components[1];
		}
        
        // XSS safeguard
    	newValue = newValue.replace(/(<+([^>]+)>+)/ig, "$2");

		if (newValue !== this._oldValue) {
			this._doCommit(newValue);
			this.host.renderMapping.parent.invalidate();
		}
		this._cancel();
    },
    
    _doCommit: function( newValue ) {
    	var commandStack = this.host.root.getCommandController().commandStack;
		var command = this.getCommand();
		if (command) {
			commandStack.execute(command, this.getExecutionArguments({newValue: newValue}));
		}
    },
    
	_cancel: function() {
		dojo.forEach(this.subscriptions, dojo.unsubscribe);
		dojo.destroy(this.input);
		dojo.forEach(this._listeners, dojo.disconnect);
        this._viewer.policies.focusScope.enable();

        if(this._viewer.getSelectionController().getPrimarySelection() === null && this._currentSelection !== null) {
            this._viewer.getSelectionController().selectExclusive(this._currentSelection);
        }
        this._wrapping = undefined; 

    },
    
    _refreshTextAlignment: function() {
        var wrapping = this.textShape.isWrapped();
        if(wrapping !== this._wrapping) {
            this._wrapping = wrapping;
            dojo.style(this.input, "textAlign", wrapping ? this.textShape.getAlign() : "left");
        }
    },
	
	_getBoundingTextBox: function(text, fontStyle) {
		var style = null;
		if(fontStyle) {
			style = {
				fontFamily: fontStyle.family,
				fontSize: fontStyle.size,
				fontWeight: fontStyle.weight,
				whiteSpace: "nowrap"
			};
		}		
		return dojox.html.metrics.getTextBox(text, style);
	}
    
    
});