/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.actions.GraphLayout");

dojo.require("rmpw.common.command.Command");
dojo.require("rmpw.diagrams.edit.actions.Action");
dojo.require("rmpw.diagrams.render.graph.BaseGraphConfigurator");

dojo.declare("rmpw.diagrams.edit.actions.GraphLayoutCommit", rmpw.common.command.Command, {
	
	modifyNode: function(graphNode, x, y, changeSet) {
		var editController = graphNode.host.__editController;
		if (editController.__setUpdatePosition) {
			editController.__setUpdatePosition(changeSet, x, y);
		}
	},
	
	modifyEdge: function(graphEdge, bendpoints, sourceAnchor, targetAnchor, changeSet) {
		var editController = graphEdge.host.__editController;
		
		if (editController.policies.polylineEditable.setUpdateBendpoints) {
			editController.policies.polylineEditable.setUpdateBendpoints(changeSet, bendpoints, sourceAnchor, targetAnchor, true);
		}
	},
	
	updateChangeSet: function(changeSet, args) {
		var layoutCmd = this;
		var modifyNode = function(graphNode, x, y) {
			layoutCmd.modifyNode(graphNode, x, y, changeSet);
		};
		var modifyEdge = function(graphEdge, bendpoints, sourceAnchor, targetAnchor) {
			layoutCmd.modifyEdge(graphEdge, bendpoints, sourceAnchor, targetAnchor, changeSet);
			var editController = graphEdge.host.__editController;
			
			if (args.routerType && editController.__setChangeRouter) {
				editController.__setChangeRouter(changeSet, args.routerType);
			}
		};
		
		return args.viewer.graphConfigurator.onCommit(args.graph, args, modifyNode, modifyEdge);
	}
});

/**
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.actions.GraphLayout", rmpw.diagrams.edit.actions.Action, {
	
	constructor: function() {
		this._options.type = "graph"; // default
		this._options.routerType = rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.RECTILINEAR_ROUTING;
		this._options.flowDirection = "bottom";
		this._options.incremental = false;
	},
	
	setType: function(type) {
		this._options.type = type;
	},

	setFlowDirection: function(flowDirection) {
		this._options.flowDirection = flowDirection;
	},
	
	setRouterType: function(routerType) {
		this._options.routerType = routerType;
	},
	
	setIncrementalMode: function(incremental) {
		this._options.incremental = incremental;
	},
	
	run: function(e){
		var args = this.getExecutionArguments(e);
		
		var layoutCallback = function(graph, args) {
			var root = e.viewer ? e.viewer : this.host.root;
			var commandStack = root.getCommandController().commandStack;
			var command = new rmpw.diagrams.edit.actions.GraphLayoutCommit();
			if (command) {
				args.graph = graph;
				return commandStack.execute(command, args);
			}
		};
		e.viewer.graphConfigurator.runGraphLayout(layoutCallback, args);
		return true;
	}
});