/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.actions.MoveScaleBoundElement");

dojo.require("rmpw.diagrams.edit.actions.Action");
dojo.require("rmpw.diagrams.edit.commands.MoveCommand");

/**
 * Moves an element whose position is derived from model data through a scale binding.
 * 
 * @author Marc Gobeil 
 * 
 */
dojo.declare("rmpw.diagrams.edit.actions.MoveScaleBoundElement", rmpw.diagrams.edit.actions.Action, {
	singleton: "moveAction",
	
	/**
	 *	e: {
	 *		dx: pixels,
	 *		dy: pixels
	 *	} 
	 */
	run: function(e) {
		if(e.dx === 0 && e.dy === 0) {
			return;
		}
		
		var KEY = 0, VALUE = 1;

		// get current position
		// TODO: put in loop so not xy specific
		var pos = {}, placement;
		placement = this._xBinding.getPlacement();
		pos[placement[KEY]] = placement[VALUE];
		placement = this._yBinding.getPlacement();
		pos[placement[KEY]] = placement[VALUE];

		// calculate new position in "model value space"
		var x = this._xBinding.getInversePlacement(pos.x + e.dx);
		var y = this._yBinding.getInversePlacement(pos.y + e.dy);
		
		// apply changes to the model
		// TODO: don't assume x and y sources are both on same object
		var renderMapping = this.host.renderMapping;
		var commandStack = this.host.root.getCommandController().commandStack;
		commandStack.execute(this._updateCommand, {
			obj: renderMapping.getSourceOwnerByPath(this._xSourcePath),
			xProperty: this._xSourcePath.substring(this._xSourcePath.lastIndexOf('.') + 1),
			yProperty: this._ySourcePath.substring(this._ySourcePath.lastIndexOf('.') + 1),
			xValue: x,
			yValue: y
		});
		
		// TODO (MG): assumes the parent has something that depends on this movement.  The parent might not depend on it
		// at all, and there may be other dependents. find a better way to do this.
		this.host.renderMapping.parent.invalidate();
	},
	
	setXBinding: function(scaleBinding) {
		// NOTE: scale bindings are *not* singletons, so they're added by variable name with an underscore prefix
		this._xBinding = this.host.renderMapping.policies['_' + scaleBinding];
	},
	
	setYBinding: function(scaleBinding) {
		this._yBinding = this.host.renderMapping.policies['_' + scaleBinding];
	},
	
	setXSource: function(sourcePath) {
		this._xSourcePath = sourcePath;
	},
	
	setYSource: function(sourcePath) {
		this._ySourcePath = sourcePath;
	},
	
	_updateCommand: {
		execute: function(commandStack, args) {
			var changeSet = new rmpw.common.vcs.ChangeSet();
			var change = changeSet.createChange(args.obj);
			change.set(args.xProperty, args.xValue);
			change.set(args.yProperty, args.yValue);
			commandStack.repository.commit(changeSet);
			return true;
		}
	}
});