/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.actions.NavigateToEdge");

dojo.require("rmpw.diagrams.edit.actions.Action");

/**
 * 
 * @author Steve Shaw
 * 
 * Action for navigating to edges from a node
 */
dojo.declare("rmpw.diagrams.edit.actions.NavigateToEdge", rmpw.diagrams.edit.actions.Action, {
	
	counter: 0,
	cachedNode: null,
	
	setForward: function(forward) {
		this._forward = forward;
	},
	
	/*
	 * Determine if this node is focsuable or not.
	 */
	getEditController: function(nodeMapping) {
		if (nodeMapping.__editController && nodeMapping.__editController.policies.focusable) {
			return nodeMapping.__editController;
		}
		
		return null;
	},
	
	/**
	 * Finds the next edge on a node
	 */
	findEdge: function(node, currentConn, forward) {
		var connections = [];
		
		var graphAnchorable = node.policies.anchorable;
		if (!graphAnchorable) {
			return connections;
		}
		
		if ((!graphAnchorable.ownedConnections || graphAnchorable.ownedConnections.length === 0) &&
			 (!graphAnchorable.targetConnections || graphAnchorable.targetConnections.length === 0)) {
			return;
		}
		
		dojo.forEach(graphAnchorable.ownedConnections, function(conn) { connections.push(conn);});
		dojo.forEach(graphAnchorable.targetConnections, function(conn) { connections.push(conn);});
		
		if (forward){
			this.counter++;
		}else{
			this.counter--;
		}
		var lg = connections.length;
		while (this.counter < 0) {
			this.counter += lg;
		}
		this.counter %= lg;
		return connections[this.counter % lg];
	},
	
	/**
	 * This method navigates through connections based on the keys pressed.
	 */
	navigateEdges: function(viewer, forward) {
		var focus = viewer.policies.focusScope.getFocus();
		var currentConn = null;
		if (focus && ("graphEdge" in focus.renderMapping.policies)) {
			currentConn = focus.renderMapping;
			if (!this.cachedNode
			  || (this.cachedNode !== currentConn.policies.graphEdge.sourceMapping && this.cachedNode !== currentConn.policies.graphEdge.targetMapping)) {
				this.cachedNode = currentConn.policies.graphEdge.sourceMapping;
				this.counter = 0;
			}
		} else {
			if (focus) {
				this.cachedNode = focus.renderMapping;
			}
		}

		if (this.cachedNode) {
			var nextConn = this.findEdge(this.cachedNode, currentConn, forward);
			if (nextConn) {
				var editController = this.getEditController(nextConn);
				if (editController) {
					viewer.policies.focusScope.setFocus(editController);
					return true;
				}
			}
		}
		
		return false;
	},
	
	run: function(args) {
		return this.navigateEdges(args.viewer, this._forward);
	}
});