/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.actions.NavigateToPeer");

dojo.require("rmpw.diagrams.edit.actions.Navigate");

/**
 * 
 * @author Steve Shaw
 * 
 * Action for navigating between shapes via keyboard
 */
dojo.declare("rmpw.diagrams.edit.actions.NavigateToPeer", rmpw.diagrams.edit.actions.Navigate, {
	
	setDirection: function(direction) {
		if (direction === "West" || direction === "East" || direction === "North" || direction === "South") {
			this._direction = direction;
		}
		else {
			console.error(direction + " is not value.  Value should be one of 'North', 'West', 'South', 'East'");
		}
	},

	/*
	 * Determine if this node is focsuable or not.
	 */
	isFocusable: function(nodeMapping) {
		return this.getEditController(nodeMapping)? true: false;
	},
	
	/*
	 * Determine if this node is focsuable or not.
	 */
	getEditController: function(nodeMapping) {
		if (nodeMapping.__editController && nodeMapping.__editController.policies.focusable) {
			return nodeMapping.__editController;
		}
		
		return null;
	},
	
	/**
	 * Finds the closest mapping object in the given direction to the start point
	 */
	focusOnClosestPeer: function(viewer, anchorNodeEl, direction) {
		var currentNode;
		var closest = null;
		var distance = null;
		var anchorNode = anchorNodeEl.renderMapping;
		var pStart = anchorNode.policies.graphElement.getBounds().getCenter();
		anchorNode.policies.graphElement.translateToAbsolute(pStart);
		
		var nodes = anchorNode.parent.children;
		for (var i=0, lg=nodes.length; i<lg; i++) {
			currentNode = nodes[i];
			
			// ignore the current node, connections and nodes that aren't focusable
			if (currentNode === anchorNode || currentNode.policies.graphEdge || !this.isFocusable(currentNode)){
				continue;
			}
			
			var pCurrent = currentNode.policies.graphElement.getBounds().getCenter();
			currentNode.policies.graphElement.translateToAbsolute(pCurrent);
			
			if (pStart.getPosition(pCurrent) !== direction){
				continue;
            }
			var d = pStart.getDistanceOrthogonal(pCurrent);
			if ( !distance || d < distance) {
				distance = d;
				closest = currentNode;
			}
		}
		
		if (closest) {
			viewer.policies.focusScope.setFocus(this.getEditController(closest));
			return true;
		}
		
		return false;
	},
	
	run: function(args) {
		var focusEl = this.findFocus(args);
		if (!focusEl) {
			return false;
		}
		
		var point = new rmpw.diagrams.render.util.Point();
		
		if (this._direction === "North") {
			return this.focusOnClosestPeer(args.viewer, focusEl, point.NORTH);
		}
		else if (this._direction === "South") {
			return this.focusOnClosestPeer(args.viewer, focusEl, point.SOUTH);
		}
		else if (this._direction === "West") {
			return this.focusOnClosestPeer(args.viewer, focusEl, point.WEST);
		}
		else if (this._direction === "East") {
			return this.focusOnClosestPeer(args.viewer, focusEl, point.EAST);
		}
		
		return false;
	}
});