/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.actions.Nudge");

dojo.require("rmpw.diagrams.edit.actions.Action");

/**
 * 
 * @author Steve Shaw
 * 
 * Action for nudging shapes
 */
dojo.declare("rmpw.diagrams.edit.actions.Nudge", rmpw.diagrams.edit.actions.Action, {
	id: "rmpw.diagrams.edit.actions.Nudge",
	description: "Nudge shape",
	
	setDirection: function(direction) {
		if (direction === "West" || direction === "East" || direction === "North" || direction === "South") {
			this._direction = direction;
		}
		else {
			console.error(direction + " is not value.  Value should be one of 'North', 'West', 'South', 'East'");
		}
	},

	setIncrement: function(increment) {
		this._nudgeInc = increment;
	},
	
	_nudgeInc: 10,

	run: function(args) {
		var delta = null;
		
		if (this._direction === "North") {
			delta = {dx: 0, dy: -this._nudgeInc};
		}
		else if (this._direction === "South") {
			delta = {dx: 0, dy: this._nudgeInc};
		}
		else if (this._direction === "West") {
			delta = {dx: -this._nudgeInc, dy: 0};
		}
		else if (this._direction === "East") {
			delta = {dx: this._nudgeInc, dy: 0};
		}
		
		if (delta) {
			var ss = args.viewer.policies.selectionScope;
			var fs = args.viewer.policies.focusScope;
			var target = fs.getCaptureFocus();
			
			if (!target || (target.policies && !target.policies.moveable)) {
				target = ss.getPrimarySelection();
			}
			
			if (target && target.policies && target.policies.moveable) {
				// turn off animation for nudge
				var allowAnimation = args.viewer.allowAnimation;
				args.viewer.allowAnimation = false;
				try {
					target.policies.moveable.doMove(delta.dx, delta.dy, target.policies.selectable ? target.policies.selectable.isSelected() : false);
				}
				finally {
					args.viewer.allowAnimation = allowAnimation;
				}
				return true;
			}
		}
		
		return false;
	}
});