/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.actions.ReorientRelationship");

dojo.require("rmpw.diagrams.edit.actions.Action");

/**
 *	Reorient relationship action
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.actions.ReorientRelationship", rmpw.diagrams.edit.actions.Action, {
	
	setTypeId: function(typeId) {
		this._options.typeId = typeId;
	},
	
	setOwnerTrace: function(ownerTrace) {
		this._options.ownerTrace = ownerTrace;
	},
	
	setElementTrace: function(elementTrace) {
		this._options.elementTrace = elementTrace;
	},
	
	setSourceTypeIds: function() {
		this._options.sourceTypeIds = arguments;
	},
	
	setTargetTypeIds: function() {
		this._options.targetTypeIds = arguments;
	},
	
	setElementTypeTrace: function(elementTypeTrace) {
		this._options.elementTypeTrace = elementTypeTrace;
	},
	
	isEnabled: function(args) {
		var sourceMapping = this.host,
		  i = 0,
		  type;
		if (args.newTargetEditController && this._options.elementTypeTrace && this._options.targetTypeIds) {
			var targetType = args.newTargetEditController.renderMapping.getSourceByPath(this._options.elementTypeTrace);
			for (i=0, type=undefined; type=this._options.targetTypeIds[i]; i++) {
				if (type === targetType) {
					return this.inherited(arguments);
				}
			}
			
			return false;
		}
		else if (args.newSourceEditController && this._options.elementTypeTrace && this._options.sourceTypeIds) {
			var sourceType = args.newSourceEditController.renderMapping.getSourceByPath(this._options.elementTypeTrace);
			for (i=0, type=undefined; type=this._options.sourceTypeIds[i]; i++) {
				if (type === sourceType) {
					return this.inherited(arguments);
				}
			}
			
			return false;
		}
		
		return this.inherited(arguments);
	},
	
	/*
	 * Gets the arguments passed into the command at execution time
	 */
	getExecutionArguments: function (e) {
		var relElement = e.targetController.renderMapping.getSourceByPath(this._options.elementTrace);
		var ownerElement = this.host.renderMapping.getSourceByPath(this._options.ownerTrace);
		var newTargetElement = e.newTargetEditController ? e.newTargetEditController.renderMapping.getDefaultSource() : null;
		var newSourceElement = e.newSourceEditController ? e.newSourceEditController.renderMapping.getDefaultSource() : null;
		
		return {ownerElement: ownerElement, relElement: relElement, newSourceElement: newSourceElement, newTargetElement: newTargetElement};
	},
	
	/*
	 * Gets the command to be execution by the action
	 */
	getCommand: function(e) {
		var commandPackage = this.host.root.getCommandPackage();
		
		var ownerTrace = this._options.ownerTrace;
		var lastDot = ownerTrace.lastIndexOf('.');
		var lastProperty = ownerTrace.substring(lastDot + 1, ownerTrace.length);
			
		var reorientCmd = commandPackage.select({
			typeId: this._options.typeId,
			ownerChildrenProperty: lastProperty,
			commandId: "rmpw.common.command.ReorientRelationship"
		});
		
		return reorientCmd;
	}
});

