/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.actions.SplitRelationship");

dojo.require("rmpw.diagrams.edit.actions.Action");
dojo.require("rmpw.common.command.CompoundCommand");
dojo.require("rmps.core.error");
dojo.require("rmpw.diagrams.edit.actions.AddElement");


/**
 * Action which will split a relationship into 2 and add a class in between
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.actions.SplitRelationship", [rmpw.diagrams.edit.actions.AddElement], {
	
	_chainedInput: {element: "element", relElement: "relElement"},
	_chainedOutput: {element: "element", relElement: "relElement"},
	
	setOwnerTrace: function(ownerTrace) {
		this._options.ownerTrace = ownerTrace;
	},
	
	setElementTrace: function(elementTrace) {
		this._options.elementTrace = elementTrace;
	},
	
	setTypeId: function(typeId) {
		this._options.typeId = typeId;
	},
	
	setAddTypeId: function(addTypeId) {
		this._options.addTypeId = addTypeId;
	},
	
	setOwnerRelTrace: function(ownerRelTrace) {
		this._options.ownerRelTrace = ownerRelTrace;
	},
	
	setRelTypeId: function(relTypeId) {
		this._options.relTypeId = relTypeId;
	},
	
	setAddRelTypeId: function(addRelTypeId) {
		this._options.addRelTypeId = addRelTypeId;
	},
	
	setTargetTrace: function(targetTrace) {
		this._options.targetTrace = targetTrace;
	},

	/*
	 * Gets the command to be execution by the action
	 */
	getCommand: function(e) {
		var editController = e.targetController;
		var commandPackage = editController.root.getCommandPackage();
		
		var ownerElement = this.host.renderMapping.getSourceByPath(this._options.ownerTrace);
		var ownerRelElement = this.host.renderMapping.getSourceByPath(this._options.ownerRelTrace);
		
		var targetElement = editController.renderMapping.getSourceByPath(this._options.targetTrace);
		var relElement = editController.renderMapping.getSourceByPath(this._options.elementTrace);
		
		var splitCmd = new rmpw.common.command.CompoundCommand();
		
		var createCmd = commandPackage.select({
			// TODO add namespace to this API
			typeId: this._options.typeId,
			addTypeId: this._options.addTypeId,
			commandId: "rmpw.common.command.AddElement"
		});
		
		var createRelCmd = commandPackage.select({
			// TODO add namespace to this API
			typeId: this._options.relTypeId,
			addTypeId: this._options.addRelTypeId,
			commandId: "rmpw.common.command.AddRelationship"
		});
		
		var reorientCmd = commandPackage.select({
			typeId: this._options.typeId,
			commandId: "rmpw.common.command.ReorientRelationship"
		});
		
		if (createCmd && createRelCmd && reorientCmd) {
			splitCmd.add(createCmd, {ownerElement: ownerElement}, {referenceElement: "element"},  {element: "semanticModel"});
			splitCmd.add(createRelCmd, {ownerElement: ownerRelElement, targetElement: targetElement}, {sourceElement: "element", referenceElement: "relElement"},  {relElement: "semanticModel"});
			splitCmd.add(reorientCmd, {relElement: relElement}, {newTargetElement: "element"});
			
			return splitCmd;
		} else {
			rmps.core.error.createError("Unable to assemble all commands necessary to split relationship.", "Attempting to get the command to split the relationship.").dispatch();
			console.error("Unable to assemble all commands necessary to split relationship");
		}
		
		return null;
	}
});

