/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.base.BaseClient");

dojo.require("jazz.client.xhr");
dojo.require("rmps.core.base.BaseClientUtil");

rmps.core.base.BaseClient = {
	
	/* 
	 *Generic GET way to handle resposnes with different forms. 
	 * 
	 * Paramaters:
	 * 		url: The url for the service
	 * 		path: This is the way we detemine a base for occurence. So, this means we could get a list of entries that resise at the same path level
	 * 					Path is an array of string in which each item shows the segment on our way to the main occurence node
	 * 		objectForm: The format for the response to be constructed.
	 * 				objectform must be an array. Each entry in that array could be one of the follows:
	 * 						1) 
	 * 								name: "internal",					//The name of the attributes
	 *					    		root: ["jfs:entry",jfs:internal"],	//The path to the node		
	 *					    		node: "#text",						//This means that the final node would be text (so we get nodeValue)
	 *					    		typeFunction: function(val){return Boolean(val)	//Optional: Ig not deifned, then the type would be string
	 *					   	2)	
	 *					   			name: "internal",					//The name of the attributes
	 *					    		root: ["jfs:entry",jfs:someNode"],	//The path to the node		
	 *					    		attribute: "rdf:resource",			//This means that we want when getting the last node to get that specific attribute
	 * 		
	 * 	extraArg: The args to be passed to the xhrGET. For example, if we need this get to be sync, so the extraArgs would be: {sync: true}
	 * 			Important to mention that the args of load, url, and handleAs in that paramater would be discarded, so no need to pass in values for them
	 * 
	 * Example:
	 * 	var form= [
	 *   	{
	 *   		name: "internal",
	 *  		root: ["jfs:internal"],
	 * 		node: "#text",
	 *   		typeFunction: function(val){return Boolean(val)}
	 *   	},
	 *   	{
	 *   		name: "rootService",
	 *   		root: ["jd:rootServices"],
	 *   		attribute: "rdf:resource"
	 *   	},
	 *   	{
	 *   		name: "title",
	 *   		root: ["dc:title"],
	 *   		node: "#text"
	 *   	}
	 *   	];
     *	rmps.core.base.BaseClient.getResponse(firendsServiceUrl,{path:["rdf:RDF","jd:Entry"],form: form}, {sync: true}); 
	 */
	getResponse: function(url, objectForm, extraArg){
		var load= dojo.hitch(this, function(response, ioArgs) {
			return rmps.core.base.BaseClientUtil.getModel(response, objectForm);
		});
		
		var args = {};
		dojo.mixin(args, extraArg);
		
		args.url= url;
		
		if(!args.handleAs){
			args.handleAs= "text";			
		}
		
		args.load= load;
		
		return jazz.client.xhrGet(args);
	}
};

})();