/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("rmps.core.base.DataFeed");

dojo.declare("rmps.core.base.DataFeed", null, {
	
	data: null,
	dataSetDetails: null,
	count:null,
	uniqueIdentifier: null,
	widgets: null,
	dataSet: null,
	
	constructor: function(params){
		if(!params){
			params= {};
		}
		if(params.dataSet){
			this.dataSet= params.dataSet;
		}else {
			this.dataSet= [];
		}
		if(dojo.isString(params.uniqueIdentifier)){
			this.uniqueIdentifier= params.uniqueIdentifier;
		}
		this.count=0;
		this.dataSetDetails= [];
		this.widgets=[];
	},
	
	initEntriesForWidget: function(widget, lazyInitialization){
		/*TODO: check for widget kind 
		 * TODO: Provide an interface to be implemnted
		 * */
		if(dojo.indexOf(this.widgets, widget)== -1){	/*Avoid this for refresh possiblities by other places */
			this.widgets.push(widget);
		}else{
			/*Get rid of old data. And start listening and intializing the new details */
			var details= this.getDetailsForWidget(widget);
			if(details){
				var detailsIndex= dojo.indexOf(this.dataSetDetails, details);
				if(detailsIndex>-1){
					this.dataSetDetails.splice(detailsIndex, 1);
					delete details;
				}
			}
		}
		
		if(lazyInitialization){
			/*
			 * Make sure at least to add the widget details
			 * 
			 * This is flag to be used in widgets that only need to listen to data changes not the initial data
			 */
			this._addDescriptionDetails(data, widget);
			return;
		}
		var dataSet= this.dataSet;
      	for (var i = 0; i < dataSet.length; i++) {
      		var data= dataSet[i];
      		this._addDataEntryForWidget(data, widget);
      	}
	},
	
	stopListening: function(widget){
		var index= dojo.indexOf(this.widgets, widget);
		if(index!= -1){	
			/*Get rid of old data. And start listening and intializing the new details */
			var details= this.getDetailsForWidget(widget);
			if(details){
				var detailsIndex= dojo.indexOf(this.dataSetDetails, details);
				if(detailsIndex>-1){
					this.dataSetDetails.splice(detailsIndex, 1);
					delete details;
				}
			}
			this.widgets.splice(index,1);
		}
	},
	
	_addDescriptionDetails: function(data, widget){
    	var descriptions= [];
    	this.addDescriptionDetailsForData(widget, data, descriptions);
   	 	return descriptions;
    },
    
    addDataEntry: function(data){
    	var exist = false, index;
    	for (index = 0; index < this.dataSet.length; index++) {
			if (this.dataSet[index] == data){
				exist = true;
			}
		}
    	if (!exist){
        	this.dataSet.push(data);    		
    	}
		for(index=0;index<this.widgets.length;index++){
			this._addDataEntryForWidget(data, this.widgets[index]);
		}
	},
	
	addDataEntries: function(dataSet){
		this.dataSet= this.dataSet.concat(dataSet);
		for(var index=0;index<this.widgets.length;index++){
			this._addDataSetForWidget(dataSet, this.widgets[index]);
		}
	},
	
	/*Should not be called outside this class.  This could cause out of sync data set*/
	/*private */ _addDataSetForWidget: function(dataSet, widget){
      	for (var i = 0; i < dataSet.length; i++) {
      		var data= dataSet[i];
      		this._addDataEntryForWidget(data, widget);
      	}
	},
	
	/*Should not be called outside this class.  This could cause out of sync data set*/
	/*private */ _addDataEntryForWidget: function(data, widget){
		/*Checking for widget not being null is the client responsiblity */
		widget.addDataEntryNotifier(data, this._addDescriptionDetails(data, widget), this);
	},
	
	removeDataEntry: function(identifier){
		var current;
		for(var index= 0;index<this.dataSetDetails.length; index++){
			for(var subIndex= 0;subIndex<this.dataSet.length; subIndex++){
				current= this.dataSet[subIndex];
				if(current[this.uniqueIdentifier]== identifier){
					this.dataSet.splice(subIndex, 1);
					break;
				}
			}
			
			current= this.dataSetDetails[index];
			var widget= current.widget;
			if(widget /*Safe check */){
				var widgetDetails= current.widgetDetails;
				var widgetDetail= this.getWidgetDetail(identifier, widgetDetails);
				widget.removeDataEntryNotifier(widgetDetails, this.uniqueIdentifier, identifier, this);
				widgetDetails.splice(dojo.indexOf(widgetDetails, widgetDetail), 1);				
			}
		}
	},
	
	getWidgetDetail: function(identifier, widgetDetails){
		for(var index= 0;index<widgetDetails.length; index++){
			var widgetDetail= widgetDetails[index];
			var data= widgetDetail.data;
			if(data&&data[this.uniqueIdentifier]==identifier){
				return widgetDetail;
			}
		}
	},
	
	/*To be called if someone would like to update the unique identifier field value itself */
	updateIdentifier: function(identifierOldValue, identifier, notify, dataObject){
		for(var index= 0;index<this.dataSet.length; index++){
			var current= this.dataSet[index];
			if(current[this.uniqueIdentifier]== identifierOldValue){
				current[this.uniqueIdentifier]= identifier;
				break;
			}
		}
		if(notify){
			this._updateDataEntry(identifierOldValue, this.uniqueIdentifier, identifier, dataObject);			
		}
	},
	
	updateDataEntriesFeilds: function(identifier, fieldNames, dataObject){
		for(var index=0; index<fieldNames.length; index++){
			var fieldName= fieldNames[index];
			this.updateDataEntry(identifier, fieldName, dataObject[fieldName], dataObject);	
		}
	},
	
	updateDataEntries: function(fieldName, fieldValue, dataObject){
		for(var index= 0;index<this.dataSet.length; index++){
			var current= this.dataSet[index];
			this.updateDataEntry(current.resourceUri, fieldName, fieldValue, dataObject);
		}
	},
	
	updateDataEntry: function(identifier, fieldName, fieldValue, dataObject){
		for(var index= 0;index<this.dataSet.length; index++){
			var current= this.dataSet[index];
			if(current[this.uniqueIdentifier]== identifier){
				current[fieldName]= fieldValue;
				break;
			}
		}
		/*FIXME: Too many loops. There could be a way to resuce the loops here */
		this._updateDataEntry(identifier, fieldName, fieldValue, dataObject);
		
	},
	
	_updateDataEntry: function(identifier, fieldName, fieldValue, dataObject){
		var fields= this.getFields(fieldName);
		for(var index=0; index<fields.length; index++){
			var field= fields[index];
			if(field[this.uniqueIdentifier]==identifier|| !identifier){	/*If identifier is null, then, we would like to update all entries */
				field[fieldName]= fieldValue;				
			}
		}
		for(index= 0;index<this.dataSetDetails.length; index++){
			var current= this.dataSetDetails[index];
			var widget= current.widget;
			if(widget /*Safe check */){
				widget.editDataEntryNotifier(current, this.uniqueIdentifier, identifier, fieldName, fieldValue, dataObject, this);
			}
		}
	},
	
	editDataEntryNotifier: function(entry, fieldName, fieldValue){},
	
	addDataEntryNotifier: function(data, descriptions){},
	
	removeDataEntryNotifier: function(data){},
	
	addDescriptionDetailsForData: function(widget, data, details){
		var widgetDetails= this.getDetailsForWidget(widget).widgetDetails;
		widgetDetails.push({
						data: data, 
						details: details
		});
	},
	
	getDetailsForWidget: function(widget){
		for(var index= 0;index<this.dataSetDetails.length; index++){
			var current= this.dataSetDetails[index];
			if(current.widget== widget){
				return current;
			}
		}
		var entry= {
				widget: widget,
				widgetDetails:	[]
		};
		this.dataSetDetails.push(entry);
		return entry;
		
	},
	
	getFields: function(fieldName){
		var fields= [];
		for(var index= 0;index<this.dataSet.length; index++){
			var current= this.dataSet[index];
			var value= current[fieldName];
			fields.push(current);
		}
		return fields;
	},	
	
	getFieldValues: function(fieldName){
		var values= [];
		var fields= this.getFields(fieldName);
		for(var index= 0;index<fields.length; index++){
			var current= fields[index];
			values.push(current[fieldName]);
		}
		return values;
	},	
	
	getValues: function(fieldName){
		var values= [];
		for(var index= 0;index<this.dataSet.length; index++){
			var current= this.dataSet[index];
			values.push(current[fieldName]);
		}
		return values;
	},	
	
	getFieldValue: function(fieldName){
		var values= this.getFieldValues(fieldName);
		if(values.length>0){
			return values[0];
		}
		return null;
	},	
	
	/*Exepct to pass the unique identifier value for the data feed */
	getItem: function(identifier){
		for(var index= 0;index<this.dataSet.length; index++){
			var current= this.dataSet[index];
			var value= current[this.uniqueIdentifier];
			if(value==identifier){
				return current;
			}
		}
	},
	
	getFieldValuesInWidgets: function(fieldName){
		var values= [];
		for(var index= 0;index<this.dataSetDetails.length; index++){
			var current= this.dataSetDetails[index];
			var widgetDetails= current.widgetDetails;
			if(widgetDetails /*Safe check */){
				for(var widgetDetailsIndex=0;widgetDetailsIndex<widgetDetails .length;widgetDetailsIndex++){
					var widgetDetail= widgetDetails[widgetDetailsIndex];
					var details= widgetDetail.details;
					for(var detailsIndex=0;detailsIndex<details.length;detailsIndex++){
						var detail= details[detailsIndex];
						if(detail.fieldName==fieldName){
							values.push(detail.value);
						}
					}
				}
			}
		}
		return values;
	},	
	
	getFieldValueInWidgets: function(fieldName){
		var values= this.getFieldValues(fieldName);
		if(values.length>0){
			return values[0];
		}
		return null;
	},	
	
	removeDescriptionDetailsForData: function(data,widget){
		for(var index=0; index<this.dataSetDetails.length;index++){
			var current= this.dataSetDetails[index];
			if(current.widget==widget && current.data==data){
				this.dataSetDetails.splice(index, 1);
				return true;
			}
		}
		return false;
	},
	
	getDescriptionDetailsForData: function(data, widget){
		var widgetDetails= this.getDetailsForWidget(widget).widgetDetails;
		for(var index=0; index<widgetDetails.length;index++){
			var current= widgetDetails[index];
			if(current.data==data){
				return current.details;
			}
		}
	},
	
	getEntryForDetail: function(details, fieldName){
		for(var index=0; index<details.length; index++){
			var current= details[index];
			if(current.fieldName== fieldName){
				return current;
			}
		}
	},
	
	getAllEntries: function(description, fieldName){
		var entries= [];
		for(var index=0; index<description.length; index++){
			var current= description[index];
			var details= current.details;
			for(var detailsIndex=0;detailsIndex<details.length;detailsIndex++){
				var detail= details[detailsIndex];
				if(detail.fieldName==fieldName){
					entries.push(detail);
				}
			} 
		}
		return entries;
	},
	
	getEntry: function(identifier, description, fieldName){
		var entries= [];
		for(var index=0; index<description.length; index++){
			var current= description[index];
			if(current.data[this.uniqueIdentifier]==identifier){
				var details= current.details;
				for(var detailsIndex=0;detailsIndex<details.length;detailsIndex++){
					var detail= details[detailsIndex];
					if(detail.fieldName==fieldName){
						return detail;
					}
				} 				
			}
		}
	},
	
	getDataBasedOnField: function(fieldName, fieldValue, widget){
		var details= this.getDetailsForWidget(widget);
		for(var index=0; index<details.length;index++){
			var current= details[index];
			var details= current.data;
			if(details[fieldName]==fieldValue){
				return details;
			}
		}
	}
			
});

})();
