/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.baseline.BaselineClient");

dojo.require("jazz.app.context");
dojo.require("rmps.core.baseline.Baseline");
dojo.require("rmps.core.RootServices");
dojo.require("rmps.core.client");

var BASELINES_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "dm-baselines");
var BASELINES_LINK_PATH = BASELINES_PATH + "/related";

rmps.core.baseline.BaselineClient = {

	getBaselines: function(filter, projectId) {
		var query = {};
		if (filter) {
			query.filter = filter;
		}
		query.projectId = projectId ? projectId : jazz.app.context.get().itemId;
		var args = {
			url: BASELINES_PATH + "?" + dojo.objectToQuery(query),
			headers: {
				"Accept": "application/rdf+json",
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				var baselines = [];
				for (var uri in response) {
					var baseline = new rmps.core.baseline.Baseline();
					baseline.parseRdfJson(uri, response[uri]);
					baselines.push(baseline);
				}
				return baselines;
			}
		};
		return rmps.core.client.xhrGet(args, false);	
	},
	
	/**
	 * Baseline URL must be specified. The project ID is optional, but *only* for backwards compatibility;
	 * new code calling this method should always pass in a project ID, to be on the safe side.  If a project
	 * ID is not specified, the current project ID from the jazz context will be used.
	 */
	getBaseline: function(baselineUrl, projectId) {
		var query = {};
		query.baselineUri = baselineUrl;
		query.projectId = projectId ? projectId : jazz.app.context.get().itemId;
		var args = {
			url: BASELINES_PATH + "?" + dojo.objectToQuery(query),
			headers: {
				"Accept": "application/rdf+json",
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				for (var uri in response) {
					var baseline = new rmps.core.baseline.Baseline();
					baseline.parseRdfJson(uri, response[uri]);
					return baseline;
				}
				return null;
			}
		};
		return rmps.core.client.xhrGet(args, false, baselineUrl);
	},
	
	/**
	 * Returns URIs of resources related to this baseline.  Currently, the only type
	 * supported is "review".
	 */
	getBaselineLinks: function(baselineUrl, type) {
		var query={};
		query.type = type;
		var args = {
			url: BASELINES_LINK_PATH + "?" + dojo.objectToQuery(query),
			headers: {
				"Accept": "application/json",
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				return response;
			}
		};
		return rmps.core.client.xhrGet(args, false, baselineUrl);
	},
	
	deleteBaseline: function(projectId, baselineUrl) {
		var query = {};
		query.baselineUri = baselineUrl;
		query.projectId = projectId ? projectId : jazz.app.context.get().itemId;
		var args = {
			url: BASELINES_PATH + "?" + dojo.objectToQuery(query),
			headers: {},
			load: function(response, ioArgs) {
				return null;
			}
		};
		return rmps.core.client.xhrDelete(args, false, baslineUrl);
	},
	
	/**
	 * name, projectId: required
	 * description: optional
	 */
 	createBaseline: function(streamUri, name, description) {
 		var query = {};
 		query.name = name;
 		if (description) {
 			query.description = description;
 		}
 		var args = {
 			url: BASELINES_PATH + "?" + dojo.objectToQuery(query),
 			hasBody: false,
 			load: function(response, ioArgs) {
 				//return the URI of the baseline created
				return ioArgs.xhr.getResponseHeader("Location");
 			}
 		};

 		return rmps.core.client.xhrPost(args, true, streamUri);
 	}

};

})();