/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.baseline.BaselineSelectionBox");

dojo.require("rmps.ui.common.SelectionBox");
dojo.require("rmps.core.baseline.BaselineClient");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmps.core.util.ContextUtils");
dojo.require("rmps.core.streams.StreamClient");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.ui.baseline", "BaselineSelectionBoxMessages");
	
var messages = dojo.i18n.getLocalization("rmps.ui.baseline", "BaselineSelectionBoxMessages");

var BaselineClient = rmps.core.baseline.BaselineClient;
var IRmpsConstants= rmps.core.util.IRmpsConstants;
var ContextUtils= rmps.core.util.ContextUtils;
var StreamClient= rmps.core.streams.StreamClient;

dojo.declare("rmps.ui.baseline.BaselineSelectionBox", rmps.ui.common.SelectionBox, {

	_deferredList: null,
	
	constructor: function() {
		this.idProp = "uri";
	},
	
	fetch: function() {
		var projectId = jazz.app.context.get().itemId;
		var currentContext = jazz.app.context.get().baseline;
		var contextType = ContextUtils.getContextType(currentContext);
		var deferreds= [];
		var snapshots= [];
		var workspace;
		
		if ( contextType === rmps.core.util.ContextTypes.SNAPSHOT){
			var deferred = BaselineClient.getBaseline(currentContext, projectId);
			deferred.addCallback(this, function (baseline){
				var contexts = [];
				contexts.push(baseline);
				return contexts;
			});
			return deferred;
		} else if (contextType === rmps.core.util.ContextTypes.WORKSPACE){
			var deferredStream = StreamClient.getStream(currentContext, null);
			deferredStream.addCallback(this, function (stream) {
				workspace= stream;
			});
			deferreds.push(deferredStream);			

			var deferredSnapshot = BaselineClient.getBaselines(null, projectId);
			deferredSnapshot.addCallback(this, function (baselines) {
				snapshots= snapshots.concat(baselines);
			});
			deferreds.push(deferredSnapshot);			
			
			this._deferredList = new dojo.DeferredList(deferreds);
			this._deferredList.addCallback(this, function () {
				// Building the contexts list
				var contexts= [];
				contexts.push(workspace);
				var strSnapshots = workspace.snapshots;

				for (var j = 0; j<snapshots.length; j++) {
					if (dojo.indexOf(strSnapshots, snapshots[j].uri)>-1){
						contexts.push(snapshots[j]);
					}
				}
				return contexts;
			});
			return this._deferredList;
		}
	},
	
	onChange: function(baseline) {
	},
	
	sortingMethod: function (e1, e2) { 
        //snapshots are sorted most-recent last               
        var a = !e1.timestamp ? -1 : e1.timestamp.getTime(),
            b = !e2.timestamp ? -1 : e2.timestamp.getTime();              
        return (a > b) ? 1 : ((a < b) ? -1 : 0);
    }
	
	
});

}());