/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.web.comment.sketch.behavior.AbstractMaker");

dojo.require("rmps.web.comment.sketch.commands.shapes.CreateRectangle");

dojo.require("rmps.web.comment.sketch.serializer.SVGUtil");
dojo.require("dojox.gfx.path");
dojo.require("rmpw.diagrams.render.util.Workarounds");

dojo.declare("rmps.web.comment.sketch.behavior.AbstractMaker", null, {
	
	mode: "SINGLE_HIT",

	currentTool: { staticRef: null },
	
	suppressed: false,
	
	mdconnection: null, 
    mmconnection: null,
    muconnection: null,
    tsconnection: null,
    tmconnection: null,
    teconnection: null, 
	
	getStrokeStyle: function( value ){
		
		var style = "Solid";
		
		switch( value ){
		
			case "1":
				style = "Dash";
				break;
		
			case "2": 
				style = "Dot";
				break;
		}	
		
		return style;
	},
	
	style: function(){
		
		this.linecolor = this.page.lineStyler.getLineColor();
		this.strokeopacity = this.page.lineStyler.getLineOpacity();
		this.strokedash = this.page.lineStyler.getLineStyle();
		
		var drawingColor = new dojo.Color( this.linecolor );
		drawingColor.a = this.strokeopacity;
		
		this.linewidth = this.page.lineStyler.getLineWidth();
    	if (dojo.isString(this.linewidth)) this.linewidth = parseInt(this.linewidth);

		
		this.fillcolor = this.page.fillpicker.color;
		this.fillopacity = this.page.fillpicker.decimalValue;		
		
		var drawingFill = new dojo.Color( this.fillcolor );
		drawingFill.a = this.fillopacity;
		
		var dojoStyle = this.getStrokeStyle( this.strokedash );
		
		this.drawing.setStroke( {cap: "round", color: drawingColor, width: this.linewidth, style: dojoStyle } );
		
		this.drawing.setFill( drawingFill );
	},
	
	/* Want to temporarily turn off listeners on other shapes so
	 * that we can draw on them.
	 */
	
	suppressShapes: function(){
		
		/* Deselect all selected shapes and prevent selection of
		 * any new shape that is added.
		 */
		
		this.page.selectionController.deselectAll();
		
		if( this.suppressed == false ){

			// prevent context toolbar from opening
			this.page.viewer.sketchController.policies.contextActionBar.toolBarController.disableToolbar(true);
		
			if( this.page.viewer.sketchController ){
				
				var children = this.page.viewer.sketchController.renderMapping.__editController.children;
				
				for( var child = 0; child < children.length; child++ ){
					children[child].policies.selectable.detachListeners();
					children[child].policies.focusable.detachListeners();
					children[child].policies.moveable.detachListeners();
				}
			}
			
			this.suppressed = true;	
		}
	},
	
	/* Want to turn on listeners of other shapes so they can be
	 * edited again
	 */
	
	engageShapes: function(){

		if(this.suppressed == true ){
				
			if( this.page.viewer.sketchController ){
				
				this.page.viewer.sketchController.policies.contextActionBar.toolBarController.disableToolbar(false);
				
				var children = this.page.viewer.sketchController.renderMapping.__editController.children;
				
				if( children != null ){
				
					for( var child = 0; child < children.length; child++ ){
						children[child].policies.selectable.attachListeners();
						children[child].policies.focusable.attachListeners();
						children[child].policies.moveable.reAttach();
					}
				}
			}
			
			this.suppressed = false;	
		}
	},
	
	capture: function(args) {
			
		if (this.currentTool.staticRef != null) {
			this.currentTool.staticRef.release();
		}
		this.currentTool.staticRef = this;
		
		this.args = args;
					
		this.surface = args.viewport.surface;
		this.viewport = args.viewport;
		
		if( args.mode ){
			this.mode = args.mode; // SINGLE_HIT, MULTIPLE_HIT
		}
		
		this.surface._parent.style.cursor = 'crosshair';
		
		this.page = this.controller.modelingPage;
		
		this.suppressShapes();
		
		if( this.page.linetools == false ){
			
			var toolbar;
			
			if( this.page.layout.toolbar ){
				toolbar = this.page.layout.toolbar;
			}else{
				toolbar = this.page.toolbar;
			}
			
			this.page.lineStyler.addStyleTools( this.page.layout.toolbar );							
			this.page.linetools = true;
		}

		this.listen();

		return true;
	},
	
	disconnect: function(){
		
		var connections = [ this.mdconnection, 
		                    this.mmconnection,
		                    this.muconnection,
		                    this.tsconnection,
		                    this.tmconnection,
		                    this.teconnection ];
		
		for( var c=0; c < connections.length; c++ ){
			if( connections[c] ){
				dojo.disconnect( connections[c] );
			}
		}
	},
	
	listen: function(){		
		this.disconnect();						
		this.engageShapes();	
		this.suppressShapes();		
		this.mdconnection = dojo.connect( this.surface._parent, "onmousedown", dojo.hitch( this, "mousedown" ) );
		this.tsconnection = dojo.connect( this.surface._parent, "touchstart", dojo.hitch( this, "mousedown" ) );
	},
	
	release: function() {		
		this.disconnect();		
		this.surface._parent.style.cursor = '';		
		this.engageShapes();
	},
	
	setListeners: function(){
		
		/* Drawing modes can work by drawing individual instances
		 * or by allowing the user to draw multiple shapes of the
		 * same kind one after the other without reselection.
		 */
		
		if( this.mode == "SINGLE_HIT" ){		
			this.release();	
		}else{
			this.listen();
		}	
	},
	
	getTouch: function( event, type ){
		
		var e = event;
		
		if( event.type == type ){
			event.preventDefault();	
			e = event.touches[0];
		}
		
		return e;
	}
	
});