/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.dsv.editor.domain.behavior.DMCoreDocumentDiagram");

dojo.require("rmpw.diagrams.edit.EditControllerExtension");

var docDiagramShapeControllerExtension = new rmpw.diagrams.edit.EditControllerExtension("rmpw.dsv.codegen.DMCoreDocumentDiagram.behavior.DiagramShapeBehaviorController");
docDiagramShapeControllerExtension.setCreateToolsExtension(function(editController) {
	// An interim solution for problems with adding document and anonymous classes to document diagram: 
	// remove all add actions for now (work item 23656) 
	removeActionBindings(editController, function(actionBinding) {
		// keeping only non-add element actions
		return !(actionBinding.getAction() instanceof rmpw.diagrams.edit.actions.AddElement);
	});	
});

var docShapeControllerExtension = new rmpw.diagrams.edit.EditControllerExtension("rmpw.dsv.codegen.DMCoreDocumentDiagram.behavior.DocumentShapeBehaviorController");
docShapeControllerExtension.setCreateToolsExtension(function(editController) {
	// interim solution for shape not changing when type is added (work item 23139)	
	removeActionBindings(editController, isNotAddTypeMenu);
});

var classShapeControllerExtension = new rmpw.diagrams.edit.EditControllerExtension("rmpw.dsv.codegen.DMCoreDocumentDiagram.behavior.ClassShapeBehaviorController");
classShapeControllerExtension.setCreateToolsExtension(function(editController) {
	// interim solution for shape not changing when type is added (work item 23139)	
	removeActionBindings(editController, isNotAddTypeMenu);
});

var anonClassShapeControllerExtension = new rmpw.diagrams.edit.EditControllerExtension("rmpw.dsv.codegen.DMCoreDocumentDiagram.behavior.AnonymousShapeBehaviorController");
anonClassShapeControllerExtension.setCreateToolsExtension(function(editController) {
	// interim solution for shape not changing when type is added (work item 23139)
	removeActionBindings(editController, isNotAddTypeMenu);
});

/**
 * Returns true if actionBinding is not related to the drop-down menu with add type action.
 */
function isNotAddTypeMenu(actionBinding) {
	if (actionBinding instanceof rmpw.diagrams.edit.actionBindings.Menu) {
		var dropDownBindings = actionBinding.getActionBindings();
		// currently, there is only one menu, but just in case...
		if (dropDownBindings && dropDownBindings[0].getAction() instanceof rmpw.diagrams.edit.actions.AddType) {
			return false;
		}
	}
	return true;
}

/**
 * Removes those action bindings from editController's context action bar
 * which do not pass the condition given by keepActionBinding.
 *
 * @param editController 
 * 			EditController whose context bar is being filtered
 * @param keepActionBinding 
 * 			Method that takes in an action binding and returns true if it should be kept, false otherwise.
 */
function removeActionBindings(editController, keepActionBinding){	
	var actionBindings = editController.policies.contextActionBar.getActionBindings();
	var newBindings = [];
	for (var i = 0; i < actionBindings.length; i++) {
		if (keepActionBinding(actionBindings[i])) {
			newBindings.push(actionBindings[i]);
		}
		delete actionBindings[i];
	}
	// reset current action bindings
	editController.policies.contextActionBar.setActionBindings();
	// re-add those that passed keepActionBinding condition
	for (var i = 0, lg = newBindings.length; i < lg; i++) {
		editController.policies.contextActionBar.addActionBinding(newBindings[i]);
	}	
}