/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.dsv.editor.sketcher.behavior.DuplicateElement");

dojo.require("rmpw.common.command.Command");
dojo.require("rmpw.common.util.IdentifierUtil");

/**
 * Command that will reparent a graph node
 * 
 * @author John Swanke
 */
dojo.declare("rmpw.dsv.editor.sketcher.behavior.DuplicateElement", rmpw.common.command.Command, {
	id: "rmpw.common.command.DuplicateElement",
	
	updateChangeSet: function(changeSet, args) {
	
		var mappingToDuplicate = args.duplicate.renderMapping;
		var elementToDuplicate = mappingToDuplicate.getDefaultSource();
		
		var mappingParent = mappingToDuplicate.parent;
		var elementParent = mappingParent.getDefaultSource();

		var addChange = changeSet.createChange(elementParent);
		var y = 0;
		var x = args.cellSize.width / args.matrix.xx;
		for (var row = 0; row < args.size.rows; row++) {
			for (var col = 0; col < args.size.cols; col++) {
				if (row==0 && col==0) continue;
				
				// create dup element
				this.duplicateElement = { };
				for (var propName in elementToDuplicate) {
					if(propName.substring(0,1) != "_") { // ignore junk	
			        	var prop = elementToDuplicate[propName];
						if (prop && (!dojo.isArray(prop) || prop.length>0) && !dojo.isFunction(prop)) {
				        	if (propName == "rdf_about") {
								prop = rmpw.common.util.IdentifierUtil.guid();
				        	} else if (propName == "rdf_type") {
								prop = [{ rdf_about: elementToDuplicate.rdf_type[0].rdf_about }];
				        	} else if (propName == "sketch_x") {
					        	if (dojo.isString(prop)) prop = parseFloat(prop);
				            	prop +=x;
				        	} else if (propName == "sketch_y") {
					        	if (dojo.isString(prop)) prop = parseFloat(prop);
				            	prop +=y;
				        	} else if (propName == "sketch_customImage") {
				        		if (!prop.rdf_about && prop.$ref) prop.rdf_about = prop.$ref;
				        	}
							this.duplicateElement[propName] = prop;
						}
					}
				}
				
				// add to changeset
				addChange.push("sketch_shape", this.duplicateElement);
				x += args.cellSize.width / args.matrix.xx;
			}
			x = 0;
			y += args.cellSize.height / args.matrix.yy;
		}
		
		return true;
	}
	
	
});
