/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.graphics.behavior.HotSpotController");
dojo.require("rmpw.diagrams.edit.EditController");

dojo.require("rmpw.diagrams.edit.tools.Focusable");
dojo.require("rmpw.diagrams.edit.tools.Selectable");
dojo.require("rmpw.diagrams.edit.tools.ContextActionBar");

dojo.require("rmps.graphics.actions.ShowView");
dojo.require("rmps.graphics.actions.ShowHyperlinks");
dojo.require("rmps.graphics.actions.ShowCompartment");
dojo.require("rmps.graphics.policies.PropertiesPolicy");
dojo.require("rmps.graphics.policies.OneClickHyperlinkPolicy");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.graphics.ui", "DiagramEditorMessages");

(function(){
	var messages = dojo.i18n.getLocalization("rmps.graphics.ui", "DiagramEditorMessages");
	
	/**
	 * A node figure that represents a class, with a label, properties compartment,
	 * and operation compartment. 
	 * 
	 * @author Anton McConville
	 */
	dojo.declare("rmps.graphics.behavior.HotSpotController", rmpw.diagrams.edit.EditController, {	
		
		COMPARTMENT_VIEW: false,
		
		setFeedbackLayer: function( layer ){
			this.feedbackLayer = layer;
		},
		
		createActions: function() {
			var action;
			
			action = this.createPolicy(rmps.graphics.actions.ShowView, "showView");
			action.setZoneData( this.renderMapping.sources.model.zonedata );
			
			action = this.createPolicy(rmps.graphics.actions.ShowHyperlinks, "showHyperlinks");
			action.setZoneData( this.renderMapping.sources.model.zonedata );
			
			action = this.createPolicy(rmps.graphics.actions.ShowCompartment, "showCompartment");
			action.setZoneData( this.renderMapping.sources.model.zonedata );
		},
		
		createActionBindings: function() {
			var actionBinding;
			
			actionBinding = this.createPolicy(rmpw.diagrams.edit.actionBindings.Button, "viewButton");
			actionBinding.setIconURL(dojo.moduleUrl("rmps.graphics", "images/open.gif") );
			actionBinding.setDescription(messages.view);
			actionBinding.setAction(this.policies.showView);
			
			actionBinding = this.createPolicy(rmpw.diagrams.edit.actionBindings.Button, "hyperlinkButton");
			actionBinding.setIconURL(dojo.moduleUrl("rmps.graphics", "images/hyperlinks.gif") );
			actionBinding.setDescription(messages.hyperlinks);
			actionBinding.setAction(this.policies.showHyperlinks);
			
			actionBinding = this.createPolicy(rmpw.diagrams.edit.actionBindings.Button, "compartmentButton");
			actionBinding.setIconURL(dojo.moduleUrl("rmps.graphics", "images/prototype/orangearrow.png") );
			actionBinding.setDescription(messages.compartment);
			actionBinding.setAction(this.policies.showCompartment);
		},
		
		createTools: function() {
			
			var policy;		
			
			policy = this.createPolicy(rmpw.diagrams.edit.tools.Focusable);
			policy.setFeedbackLayer( this.renderMapping.getFrontFeedbackLayer() );
			
			policy = this.createPolicy(rmpw.diagrams.edit.tools.Selectable);
			
			policy = this.createPolicy(rmps.graphics.policies.PropertiesPolicy);
			
			policy = this.createPolicy(rmps.graphics.policies.OneClickHyperlinkPolicy);
			
			policy = this.createPolicy(rmpw.diagrams.edit.tools.ContextActionBar);
			
			if( this.renderMapping.sources.model.zonedata.graph ) { 
				policy.setActionBindings( this.policies.compartmentButton, this.policies.viewButton  );
			}else if (this.renderMapping.sources.model.zonedata.hyperlinks ){
				policy.setActionBindings( this.policies.viewButton, this.policies.hyperlinkButton );	
			}else{
				policy.setActionBindings( this.policies.viewButton );	
			}
		},
		
		init: function(parent) {
			this.inherited(arguments);
		}
	});
})();