/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmps.diagram.behavior.HotSpotController");
dojo.require("rmpw.diagrams.edit.EditController");

dojo.require("rmpw.diagrams.edit.tools.Focusable");
dojo.require("rmpw.diagrams.edit.tools.Selectable");
dojo.require("rmpw.diagrams.edit.tools.ContextActionBar");
dojo.require("rmpw.diagrams.edit.tools.RichHover");

dojo.require("rmps.diagram.actions.ShowView");
dojo.require("rmps.diagram.actions.ShowHyperlinks");
dojo.require("rmps.diagram.actions.ShowCompartment");
dojo.require("rmpw.diagrams.edit.actions.InvokeTool");
dojo.require("rmps.graphics.policies.OneClickHyperlinkPolicy");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.graphics.ui", "DiagramEditorMessages");

var messages = dojo.i18n.getLocalization("rmps.graphics.ui", "DiagramEditorMessages");

/**
 * A node figure that represents a class, with a label, properties compartment,
 * and operation compartment. 
 * 
 */
dojo.declare("rmps.diagram.behavior.HotSpotController", rmpw.diagrams.edit.EditController, {	
	
	COMPARTMENT_VIEW: false,
	
	setFeedbackLayer: function( layer ){
		this.feedbackLayer = layer;
	},
	
	createActions: function() {
		var action;
		
		action = this.createPolicy(rmps.diagram.actions.ShowView, "showView");
		action.setZoneData( this.renderMapping.sources.model.zonedata );
		
		action = this.createPolicy(rmps.diagram.actions.ShowHyperlinks, "showHyperlinks");
		action.setZoneData( this.renderMapping.sources.model.zonedata );
		
		action = this.createPolicy(rmps.diagram.actions.ShowCompartment, "showCompartment");
		action.setZoneData( this.renderMapping.sources.model.zonedata );
	},
	
	createActionBindings: function() {
		var actionBinding;
		
		actionBinding = this.createPolicy(rmpw.diagrams.edit.actionBindings.Button, "viewButton");
		actionBinding.setIconURL(dojo.moduleUrl("rmps.graphics", "images/open.gif") );
		actionBinding.setDescription(messages.view);
		actionBinding.setAction(this.policies.showView);
		
		
		actionBinding = this.createPolicy(rmpw.diagrams.edit.actionBindings.Button, "hyperlinkButton");
		actionBinding.setIconURL(dojo.moduleUrl("rmps.graphics", "images/hyperlinks.gif") );
		actionBinding.setDescription(messages.hyperlinks);
		actionBinding.setAction(this.policies.showHyperlinks);
		
		actionBinding = this.createPolicy(rmpw.diagrams.edit.actionBindings.Button, "compartmentButton");
		actionBinding.setIconURL(dojo.moduleUrl("rmps.graphics", "images/prototype/orangearrow.png") );
		actionBinding.setDescription(messages.compartment);
		actionBinding.setAction(this.policies.showCompartment);
	},
	
	createTools: function() {
		var policy;		
		
		policy = this.createPolicy(rmpw.diagrams.edit.tools.Focusable);
		policy.setFeedbackLayer( this.renderMapping.getFrontFeedbackLayer() );
		
		policy = this.createPolicy(rmpw.diagrams.edit.tools.Selectable);
		
		policy = this.createPolicy(rmpw.diagrams.edit.tools.RichHover);
		
		policy = this.createPolicy(rmps.graphics.policies.OneClickHyperlinkPolicy);
		
		policy = this.createPolicy(rmpw.diagrams.edit.tools.ContextActionBar);
		
		if( this.renderMapping.sources.model.zonedata.graph ) { 
			policy.setActionBindings( this.policies.compartmentButton, this.policies.viewButton  );
		}else if (this.renderMapping.sources.model.zonedata.hyperlinks ){
			policy.setActionBindings( this.policies.viewButton, this.policies.hyperlinkButton );	
		}else{
			policy.setActionBindings( this.policies.viewButton );	
		}
	},
	
	init: function(parent) {
		this.inherited(arguments);
	}
});
