/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmps.web.comment.sketch.behavior.SketchController");

dojo.require("rmpw.diagrams.edit.EditController");
dojo.require("rmps.web.comment.sketch.behavior.ShapeController");
dojo.require("rmps.web.comment.sketch.commands.shapes.CreateShapeAction");
dojo.require("rmps.web.comment.sketch.behavior.ShapeWithTextController");
dojo.require("rmpw.diagrams.edit.tools.ContextActionBar");
dojo.require("rmpw.diagrams.edit.tools.Selectable");
dojo.require("rmpw.diagrams.edit.actionBindings.Button");
dojo.require("rmpw.diagrams.edit.actions.DeleteElement");
dojo.require("rmpw.diagrams.edit.actionBindings.KeyStroke");

dojo.requireLocalization("rmps.web.comment.sketch", "ToolStrings");
var strings = dojo.i18n.getLocalization("rmps.web.comment.sketch", "ToolStrings");

dojo.declare("rmps.web.comment.sketch.behavior.SketchController", rmpw.diagrams.edit.EditController, {	
	
	attachChild: function(mapping) {
		var controller = null;
		
		if( "graphEdge" in mapping.policies ){
			throw "unexpected graphEdge found";
		} else {
			switch( mapping.sources.svgRoot.elementId ){
				case "Path":
					controller = this.createChild(rmps.web.comment.sketch.behavior.ShapeController);
					this._nodeInjections(controller);
					break;
					
				default:
					controller = this.createChild(rmps.web.comment.sketch.behavior.ShapeWithTextController);	
					this._nodeInjections(controller);
					break;			
			}			
		}

		return controller;
	},	
	
	createActions: function() {
		var action;
		
		action = this.createPolicy(rmps.web.comment.sketch.commands.shapes.CreateShapeAction, "addText");
		action.setCommandArguments({
			typeId: "Sketch",
			addTypeId: "Text"
		});
		
		action = this.createPolicy(rmps.web.comment.sketch.commands.shapes.CreateShapeAction, "addCloud");
		action.setCommandArguments({
			typeId: "Sketch",
			addTypeId: "Cloud"
		});
		
		action = this.createPolicy(rmps.web.comment.sketch.commands.shapes.CreateShapeAction, "addOval");
		action.setCommandArguments({
			typeId: "Sketch",
			addTypeId: "Oval"
		});
		
		action = this.createPolicy(rmps.web.comment.sketch.commands.shapes.CreateShapeAction, "addScribble");
		action.setCommandArguments({
			typeId: "Sketch",
			addTypeId: "Scribble"
		});
		
		action = this.createPolicy(rmps.web.comment.sketch.commands.shapes.CreateShapeAction, "addLine");
		action.setCommandArguments({
			typeId: "Sketch",
			addTypeId: "Line"
		});
		
		action = this.createPolicy(rmps.web.comment.sketch.commands.shapes.CreateShapeAction, "addRectangle");
		action.setCommandArguments({
			typeId: "Sketch",
			addTypeId: "Rectangle"
		});
		
		action = this.createPolicy(rmpw.diagrams.edit.actions.DeleteElement, "deleteSketchItem");
		action.setTypeId("Sketch");
		action.setElementTrace("svgRoot");
		action.setOwnerTrace("svgRoot.g");
	},
	
	createActionBindings: function() {
		var actionBinding;
		
		actionBinding = this.createPolicy(rmpw.diagrams.edit.actionBindings.Button, "addTextButton");
		actionBinding.setIconURL( dojo.moduleUrl("rmps.web.comment.sketch", "icons/pal/text_pal16.png" ) );
		actionBinding.setDescription(strings.text_label);
		actionBinding.setAction(this.policies.addText);
		
		actionBinding = this.createPolicy(rmpw.diagrams.edit.actionBindings.Button, "addCloudButton");
		actionBinding.setIconURL( dojo.moduleUrl("rmps.web.comment.sketch", "icons/pal/cloud_pal16.png" ) );
		actionBinding.setDescription(strings.cloud_label);
		actionBinding.setAction(this.policies.addCloud);
		
		actionBinding = this.createPolicy(rmpw.diagrams.edit.actionBindings.Button, "addOvalButton");
		actionBinding.setIconURL( dojo.moduleUrl("rmps.web.comment.sketch", "icons/pal/oval_pal16.png" ) );
		actionBinding.setDescription(strings.oval_label);
		actionBinding.setAction(this.policies.addOval);
		
		actionBinding = this.createPolicy(rmpw.diagrams.edit.actionBindings.Button, "addScribbleButton");
		actionBinding.setIconURL( dojo.moduleUrl("rmps.web.comment.sketch", "icons/pal/scribble_pal16.png" ) );
		actionBinding.setDescription(strings.pencil_label);
		actionBinding.setAction(this.policies.addScribble);
		
		actionBinding = this.createPolicy(rmpw.diagrams.edit.actionBindings.Button, "addLineButton");
		actionBinding.setIconURL( dojo.moduleUrl("rmps.web.comment.sketch", "icons/pal/line_pal16.png" ) );
		actionBinding.setDescription(strings.line_label);
		actionBinding.setAction(this.policies.addLine);
		
		actionBinding = this.createPolicy(rmpw.diagrams.edit.actionBindings.Button, "addRectangleButton");
		actionBinding.setIconURL( dojo.moduleUrl("rmps.web.comment.sketch", "icons/pal/rectangle_pal16.png" ) );
		actionBinding.setDescription(strings.rectangle_label);
		actionBinding.setAction(this.policies.addRectangle);
		
		actionBinding = this.createPolicy(rmpw.diagrams.edit.actionBindings.Button, "deleteSketchButton");
		actionBinding.setAction(this.policies.deleteSketchItem);
		actionBinding.setIconURL(dojo.moduleUrl("rmps.web.comment.sketch", "icons/elcl16/delete.png") );
		actionBinding.setDescription(strings.delete_label);
		actionBinding.setIconClass("deleteFromDiagramIcon");
	},
	
	_nodeInjections: function(child) {
		child.contributeAction(this.policies.deleteSketchButton);
		
		// needed for clearing of a
		child.policies.clearAction = this.policies.deleteSketchItem;
		
		var deleteFromDiagramKeyBinding = child.createPolicy(rmpw.diagrams.edit.actionBindings.KeyStroke, "deleteSketchKeyBinding");
		deleteFromDiagramKeyBinding.setKeyCode(dojo.keys.DELETE);
		deleteFromDiagramKeyBinding.setAction(this.policies.deleteSketchItem);
	},
	
	createTools: function(){
		var policy;
	
		policy = this.createPolicy(rmpw.diagrams.edit.tools.Selectable);
		policy.setIsSelectionContainer(true);
		
		if( this.root.__editMode ){
		
			policy = this.createPolicy(rmpw.diagrams.edit.tools.ContextActionBar);
			policy.setActionBindings( this.policies.addTextButton, this.policies.addRectangleButton, this.policies.addOvalButton, 
			                   this.policies.addLineButton, this.policies.addCloudButton, this.policies.addScribbleButton );
		}	
	}

});