/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.dsv.editor.sketcher.behavior.SketcherMoveable");

dojo.require("rmpw.diagrams.edit.tools.Moveable");

/**
 * @author John Swanke
 */
dojo.declare("rmpw.dsv.editor.sketcher.behavior.SketcherMoveable", rmpw.diagrams.edit.tools.Moveable, {
	
	constructor: function() {
		this.setLiveFeedback(true);
		this.host.__setUpdatePosition = dojo.hitch(this, function(changeSet, x, y) {
			// floating labels might not have x,y yet
			var safeTrace;
			var isFloatingLabel = this.host.renderMapping.policies.graphFloatingLabel !== undefined;
			if (isFloatingLabel) {
	            safeTrace = this.host.renderMapping.traces.labelText;				
			} else {
	            safeTrace = this.host.renderMapping.traces.x;
			}
            var traceX = this.host.renderMapping.traces.x;
            var traceY = this.host.renderMapping.traces.y;
            if (safeTrace && traceX && traceY) {
            	var owner = safeTrace.getOwner();
            	if (!owner) return false;
            	var change = changeSet.createChange(owner);
            	change.set(traceX.propertyName, x);
            	change.set(traceY.propertyName, y);
            	
            	// if shape was moved a significant distance, remove any bendpoints from connections to shape
            	if (!isFloatingLabel) {
	        		var graphAnchorable = this.host.renderMapping.policies.anchorable;
	        		if (graphAnchorable) {
		        		if ( Math.abs(x-traceX.get())>100 || Math.abs(y-traceY.get())>100) {
			        		this._resetConnections(changeSet, graphAnchorable.ownedConnections);
			        		this._resetConnections(changeSet, graphAnchorable.targetConnections);
	            		}
	        		}
            	}
            	
            	// also need to update internal x,y so that GraphNode#getConstraintBounds() works
				var xTrace = this.host.renderMapping.traces[this.host.renderMapping.policies.graphElement._getInternalTraceName("x")];
				var yTrace = this.host.renderMapping.traces[this.host.renderMapping.policies.graphElement._getInternalTraceName("y")];
				if (xTrace && yTrace) {
					var change = changeSet.createChange(xTrace.getOwner());
					change.set(xTrace.propertyName, x);
					change.set(yTrace.propertyName, y);
					return true;
				} else {
					return false;
				}
            }
			return false;
		});
	},
	
	_resetConnections: function(changeSet, connections) {
		if (connections) {
			for (var i=0, conn; conn=connections[i]; i++) {
				var traceSourceShape = conn.traces.sourceShape;
				var traceBendpoints = conn.traces.bendpoints;
				var traceSourceAnchor = conn.traces.sourceAnchor;
				var traceTargetAnchor = conn.traces.targetAnchor;
				var traceAvoid = conn.traces.avoidObstructions;
				var traceRouter = conn.traces.routing;
				var traceSmoothness = conn.traces.smoothness;
				if (traceSourceShape && traceBendpoints && traceSourceAnchor && 
						traceTargetAnchor && traceRouter && traceSmoothness && traceAvoid) {
	            	var change = changeSet.createChange(traceSourceShape.getOwner());
 	            	change.set(traceBendpoints.propertyName, undefined);        					
 	            	change.set(traceSourceAnchor.propertyName, undefined);  
 	            	change.set(traceTargetAnchor.propertyName, undefined);  
 	            	change.set(traceRouter.propertyName, undefined);  
 	            	change.set(traceSmoothness.propertyName, undefined);
 	            	change.set(traceAvoid.propertyName, true);
				}
			}
		}
	}

});