/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2012. All Rights
 * Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

(function() {
	dojo.provide("rmps.core.changesets.ChangesetClient");

	dojo.require("rmps.core.RootServices");
	dojo.require("rmps.core.changesets.ChangesetSummary");
	dojo.require("rmps.core.changesets.Changeset");
	dojo.require("rmps.core.client");
	dojo.require("dojo.i18n");

	var CHANGESETS_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "dm-changesets");
	var MODELS_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "models");
	

	var addErrorCallback= function(deferred){
		deferred.addErrback(this, function(error){
			if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
				return rmps.core.error.createXhrError(deferred.ioArgs);
			}
			return rmps.core.error.createError(error);
		});
		return deferred;
	};

	/*Delegate some required jazz.client functions and add the required handling for errors and baseline*/
	var CLIENT= {};

	CLIENT.xhrGet= dojo.hitch(CLIENT, function(args){
		return addErrorCallback(rmps.core.client.xhrGet(args));
	});

	rmps.core.changesets.ChangesetClient = {

		// Returns the URI's of all the changesets
		// user Uri as paramater
		getChangesetsForUser: function(userUri, includeCreated){
			var query = {};
			// To include the non comitted changesets
			if (includeCreated){
				query.includeCreated = true;
			}
			query.user = userUri;
			var args = {
				url: CHANGESETS_PATH + "?" + dojo.objectToQuery(query),
				headers: {
					"Accept" : "application/json",
					"X-ibm-rmps-internal" : "true"
				},
				handleAs: "json",
				load: function( response, ioArgs ) {
					var changesetSummaries = [];
					for (var i=0; i<response.length; i++) {
						var changesetSummary = new rmps.core.changesets.ChangesetSummary();
						changesetSummary.parseJson(response[i]);
						changesetSummaries.push(changesetSummary);
					}
					return changesetSummaries;
				}
			};
			return CLIENT.xhrGet(args);
		},

	/**
	 * Gets a single changeset
	 */
	getChangeset: function(changesetUri){
		// Need to replace the VVC uri prefix by the DM server one
		var dmCsUri = this.convertUri(changesetUri);
		

		var args = {
			url: dmCsUri,
			headers: {
				"Accept": "application/rdf+json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				for (var uri in response) {
					var changeset = new rmps.core.changesets.Changeset();
					changeset.parseRdfJson(uri, response[uri]);
					return changeset;
				}

			}
		};
		return CLIENT.xhrGet(args);
	},
	
	/**
	 * Gets the worspace owning a given changeset
	 */
	getChangesetWorkspace: function(changesetUri, projectId){
		// Need to replace the VVC uri prefix by the DM server one
		var dmCsUri = this.convertUri(changesetUri);

		var query = {
			"workspace" : "true",
			"project" : projectId|| jazz.app.context.get().itemId	
		};

		var args = {
			url: dmCsUri + "?" + dojo.objectToQuery(query),
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			sync: true,
			handleAs: "text",
			load: function(response, ioArgs) {
				return response;
			}
		};
		return CLIENT.xhrGet(args);
	},

	/**
	 * Gets a changeset resources
	 */
	getChangesetResources: function(changesetUri){
		// Need to replace the VVC uri prefix by the DM server one
		var dmCsUri = this.convertUri(changesetUri);
		
		var query = {};
		query.resources = true;


		var args = {
			url: dmCsUri  + "?" + dojo.objectToQuery(query),
			headers: {
				"Accept": "application/json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				var changesetResources = [];
				for (var i=0; i<response.length; i++) {
					if (response[i].uri.indexOf(MODELS_PATH)>-1)
						changesetResources.push(response[i].uri);
				}
				return changesetResources;
			}
		};
		return CLIENT.xhrGet(args);
	},

	
	convertUri: function(VvcChangesetUri){
		if (VvcChangesetUri.indexOf(CHANGESETS_PATH) === -1){
			return CHANGESETS_PATH + VvcChangesetUri.substring(VvcChangesetUri.lastIndexOf("/"));
		} else{
			return VvcChangesetUri;
		}
	}
	
	};
})();