/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.core.clipboard.Clipboard");

dojo.require("dojox.collections._base");
dojo.require("rmps.ui.explorer.ExplorerSetting");
dojo.require("rmps.ui.explorer.ExplorerUtilities");
dojo.require("rmps.core.explorer.ExplorerConstants");

var ExplorerUtilities = rmps.ui.explorer.ExplorerUtilities;
var SettingTypes = rmps.core.explorer.ExplorerConstants.SettingTypes;

(function(){

//"singleton"
rmps.core.clipboard._clipboard = null;

rmps.core.clipboard.clipboard = function(projectUri) {
	var def = new dojo.Deferred();
	if (!rmps.core.clipboard._clipboard){
		var userUri = dojo.getObject("jazz.app.currentApplication.config.user.uri");
		// The URI of the user could not be detected, however project uri was passed as a parameter.
		// This means that call was done from the context where neither current project id nor current user are defined,
		// for example, from resource picker wrapped by iframe.
		// If so, let's indicate to server, that user uri should be extracted from cookies.
		var detectUser = projectUri && !userUri;
		if (!projectUri) {
			projectUri = dojo.getObject("jazz.app.currentApplication.context");
		}
		if (!projectUri || (!userUri && !detectUser)) {
			def.errback();
			return def;
		}
		var setting = new rmps.ui.explorer.ExplorerSetting({
			settingName: "clipboard",
			projectUri: projectUri,
			contextType: SettingTypes.PROJECTUSER
		});
		if (userUri) {
			setting.userUri = userUri;
		}
		if (detectUser) {
			setting.detectUser = detectUser;
		}

		setting.get().then(function(value) {
			var settingdata = value || {};
			var resources = settingdata[projectUri] || [];			
			rmps.core.clipboard._clipboard = new rmps.core.clipboard.Clipboard(resources, setting, settingdata);
			def.callback(rmps.core.clipboard._clipboard);				
		});
	} else {
		def.callback(rmps.core.clipboard._clipboard);
	}
	return def;
};
	
rmps.core.clipboard.Clipboard = function(resources, setting, settingdata) {
	var _stack = resources || [];
	
	this.size = _stack.length;
	this.maxsize = 100;
	
	// Add to top of clipboard
	this.push = function(resourceURI, typeURI, name, parentName) {
		var data = {
			resourceURI: resourceURI,
			typeURI: typeURI,
			Name: name,
			NameOfParent: parentName
		};
		if (!this._executeOnContained(resourceURI, function(index) {
			// Already in stack, move to top
			_stack.splice(index, 1);
			_stack.splice(0, 0, data);
		})) {
			if (this.size === this.maxsize) {
				// throw the last one
				this.size = _stack.splice(this.size - 1, 1);
			}	
			this.size = _stack.push(data);
		}
		
		this._onChange();
	};
	
	this.remove = function(resourceURI) {
		var self = this;
		return this._executeOnContained(resourceURI, function(index) {
			// Found, remove.
			_stack.splice(index, 1);
			self.size--;
			self._onChange();
		});
	};
	
	this.iterator = function() {
		return new dojox.collections.Iterator(_stack);
	};
	
	this._handlePageSet = function(list, def, page, pageSize) {
		if (page * pageSize > list.length) {
			def.callback([]);
		}
		var startindex = page * pageSize;
		var nextpageindex = (page + 1) * pageSize;
		if (this.size >= nextpageindex) {
			def.callback(list.slice(startindex, nextpageindex));	
		} else {
			def.callback(list.slice(startindex));	
		}		
	};
	
	this.getPageSet = function(page, pageSize, onlyType) {
		var def = new dojo.Deferred();
		if (onlyType) {
			var filterDef = ExplorerUtilities.filterOnType(dojo.map(_stack, function(item) {
				return item.resourceURI;
			}), onlyType);
			filterDef.addCallback(this, function(list) {
				this._handlePageSet(list, def, page, pageSize);
			});
			filterDef.addErrback(this, function() {
				this._handlePageSet([], def, page, pageSize);
			});
		} else {
			var list = dojo.map(_stack, function(item) {
				return item.resourceURI;
			});
			this._handlePageSet(list, def, page, pageSize);
		}
		return def;
	};
	
	this._executeOnContained = function(resourceURI, fn) {
		for (var i = 0; i < _stack.length; i++){
			if (_stack[i].resourceURI === resourceURI){
				if (fn) {
					fn(i);
				}
				return true;
			}
		}
		return false;
	};
	
	this._onChange = function() {
		dojo.publish("rmps.core.clipboard.Change");
		
		settingdata[setting.projectUri] = _stack;
		setting.value = settingdata;
		setting.set();
	};
	
	this.getIconURL = function() {
		var path = "graphics/icons/clipboard_";
		if (_stack.length === 0) {
			path += "empty";
		} else if (_stack.length > 9) {
			path += "many";
		} else {
			path += _stack.length;
		}
		path += "_co.gif";
		return dojo.moduleUrl("rmps.ui", path).toString();
	};
};

}());