/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.clipboard.ClipboardTable");

dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.ui.toolbar.TwoStateButton");
dojo.require("rmps.core.clipboard.Clipboard");
dojo.require("rmps.core.query.BasicQueryResult");
dojo.require("rmps.app.core.internal.util.QueryUtil");
dojo.require("rmps.app.ui.internal.search.SearchResults");

dojo.requireLocalization("rmps.ui.clipboard", "ClipboardWidgetMessages");

(function(){
	
var ResultSet = rmps.core.query.BasicQueryResult;
var QueryUtil = rmps.app.core.internal.util.QueryUtil;
var clipboard = rmps.core.clipboard.clipboard;

dojo.declare("rmps.ui.clipboard.ClipboardTable", rmps.app.ui.internal.search.SearchResults, {
	messages: dojo.i18n.getLocalization("rmps.ui.clipboard", "ClipboardWidgetMessages"),
	
	tableClass: "rmps.ui.clipboard._Table",
	readOnly: true,
	onSelect: null,
	isFullPage: false,
	onlyType: null,
	projectUri: null,
	isOslc: false,
	disabledList: null,

	_selection: null,
	_updateDef: null,
	
	postCreate: function() {
		this.createToolbar = false;

		if (this.disabledList === null) {
			this.disabledList = [];
		}
		
		this.inherited(arguments);
		
		// Hide the header completely.
		dojo.style(this.headerContainerNode, {display : "none"});
	
		this._selection = [];		
		this._resultTable._onPageChange = dojo.hitch(this, this.onPageChange);
		
		if (!this.isFullPage) {
			// Set page size to 10
			this._resultTable.pagingControl.update(0, 10, null, null, null);
		}
		
		this._showPage(0);
	},
	
	createTableHeaders: function(parentNode) {
		var headers= this.inherited(arguments);
		if (!this.readOnly) {
			headers.unshift(this.createTableHeader("checkBox", "", {filterParticipant: false, isOrderable: false}));
		}
		return headers;
	},

	_checkBoxHandler: function(result, rowObj, owned, resourceUri, presentationUri) {
		if (this.isOslc) {
			result.deferred = new dojo.Deferred();
			result.deferred.addBoth(dojo.hitch(this, function(info) {
				result.checkBox = true;
				if (!info.existsInCurrentContext) {
					this.disabledList.push(resourceUri);
				}
				delete result.deferred;
			}));
		} else {
			result.checkBox = true;
		}
	},
	
	refresh: function() {
		// recreate page
		if (this._updateDef) {
			this._updateDef.addCallback(this, function() {
				this.onPageChange(this.getQueryResult().query.page);				
			});
		}
	},
	
	getRowResourceUri: function(entry) {
		return entry;
	},
	
	getRowShortName: function(entry) {
		return null;
	},
	
	_showPage: function(page, preserveFilter) {
		this._updateDef = this._getResultSet(page);
		this._updateDef.addCallback(this, function(rs) {
			this.showResults(rs, preserveFilter, this.baseline);			
		});
	},
	
	_getResultSet: function(page) {
		var def = new dojo.Deferred();
		var cbdef = clipboard(this.projectUri);
		cbdef.addCallback(this, function(cb) {
			var pageSize = this._resultTable.pagingControl.pageSize;
			cb.getPageSet(page, pageSize, this.onlyType).addCallback(this, function(entries) {
				var rs = new ResultSet();
				dojo.mixin(rs, {
					id: "clipboard",
					entries: entries, 
					totalResults: cb.size,
					query: {
						page: page,
						pageSize: pageSize,
						limit: -1
					}
				});
				def.callback(rs);				
			});
		});
		cbdef.addErrback(function() {
			def.errback();
		});
		return def;
	},
	
	onPageChange: function(page) {
		this._showPage(page, true /* preserveFilter */);
	},
	


	_actionsHandler: function(result, rowObj, owned, resourceUri, presentationUri) {
		if (owned) {
			result.className = "actions";
			result.deferred = new dojo.Deferred();
			result.deferred.addCallback(dojo.hitch(this, function(info) {
				
				result.init = dojo.hitch(this, function(rowConfig, tableRow, rowIdx, columnConfig, cell, colIdx) {
					var toolbar = new jazz.ui.Toolbar();
					this.createRowActions(toolbar, rowObj, resourceUri, presentationUri,info.existsInCurrentContext);
					cell.style.width = ((toolbar.getChildren().length * 26) + 1) + "px";
					// use the Toolbar to connect so that disconnect is called on destroy
					toolbar.connect(cell, "onmousedown", dojo.hitch(dojo, dojo.stopEvent));
					cell.appendChild(toolbar.domNode);
					tableRow.actionToolbar = toolbar;
					if (!info.existsInCurrentContext) {
						dojo.forEach(tableRow.children, function(td) {
							if (td != cell) {
								dojo.addClass(td,"dimmed")
							}
						});
					}
				});
				
				delete result.deferred;
			}));
			result.deferred.addErrback(function() {
				delete rowObj[propertyCache];
				delete result.deferred;
			});
		}
	},
	
	createRowActions: function(toolbar, rowObj, resourceUri, presentationUri,existsInCurrentContext) {
		if (existsInCurrentContext) {
			this.createPreviewAction(toolbar, rowObj, resourceUri, presentationUri);
			if (this.isFullPage) {
				this.createExplorerAction(toolbar, rowObj, resourceUri, presentationUri);	
			}
		}
		this.createRemoveAction(toolbar, rowObj, resourceUri, presentationUri);
	},
	
	createRemoveAction: function(toolbar, rowObj, resourceUri, presentationUri) {
		var removebutton = new jazz.ui.toolbar.Button({
			title: this.messages.remove,
			imgSrc: dojo.moduleUrl("rmps.ui", "graphics/icons/remove_grey_co.gif").toString(),
			onClick: dojo.hitch(this, function() {
				clipboard(this.projectUri).addCallback(this, function(cb) {
					cb.remove(resourceUri);
					
					// If in selection, remove.
					var index = dojo.indexOf(this._selection, resourceUri);
					if (index !== -1) {
						this._selection.splice(index, 1);
						this.onSelect(this._selection);
					}
					
					this.refresh();
				});
			})
		});
		toolbar.addChild(removebutton);
	},
	
	getSelection: function() {
		return this._selection;
	},
	
	createTable: function() {
		var tableConstructor = dojo.getObject(this.tableClass);

		return new tableConstructor({
			noEntriesLabel: this.messages.noResults,
			noFilterMatchesLabel: this.messages.noFilterMatches,
			pageSizeSelection: this.isFullPage,
			search: this
		});
	},

	removeallResources: function(resources, properties, project, deferreds) {
		dojo.forEach(resources, function(resourceUri, i) {
			var resourceInfo = new rmps.core.resource.ResourceInfo();
			resourceInfo.uri = null;
			resourceInfo.existsInCurrentContext = false;
			dojo.forEach(deferreds[resourceUri], function(def) {
				def.callback(resourceInfo);
			});
			delete deferreds[resourceUri];
			delete project[resourceUri];
		});
	},
	
	checkOtherbaselines: function(deferreds, project, properties, resources, key) {
		dojo.when(
				this._getResourceInfo(key, resources[key], properties, ""),
				dojo.hitch(this, function(infos) {
					dojo.forEach(infos, function(info) {
						info.existsInCurrentContext = false;
						dojo.forEach(deferreds[info.uri], function(def) {
							def.callback(info);
						});
						delete deferreds[info.uri];
						delete project[info.uri];

						var index = resources[key].indexOf(info.uri);
						if (index != -1) {
							resources[key].splice(index,1);
						}						
					});
					if (resources[key].length > 0) {
						//some more elements left which does not exist in any baselines of project area
						this.removeallResources(resources[key], properties, project, deferreds);
					}
					this._handleError(project);
				}),
				dojo.hitch(this,function(error) {
					//this indicates the resource(s) which does not exist in any of the project area baselines
					this.removeallResources(resources[key], properties, project, deferreds);
				})
			);
	},
	
	_handleProjectResult: function(deferreds, project, properties, baseline) {
		var resources = {};
		for (var resourceUri in project) {
			if (resourceUri) {
				var groupKey = this._getGroupKey(resourceUri);
				if (!resources[groupKey]) {
					resources[groupKey] = [];
				}
				resources[groupKey].push(resourceUri);
			}
		}

		for (var key in resources) {
			if (key) {
				dojo.when(
					this._getResourceInfo(key, resources[key], properties.concat(["creatorUri"]), baseline), // add creatorUri to make sure deleted elements are detected
					dojo.hitch(this, function(infos) {
						
						dojo.forEach(infos, function(info) {
							info.existsInCurrentContext = true;
							dojo.forEach(deferreds[info.uri], function(def) {
								def.callback(info);
							});
							delete deferreds[info.uri];
							delete project[info.uri];
							
							var resourceList = resources[key];
							var index = dojo.indexOf(resourceList, info.uri);
							if (index != -1) {
								resourceList.splice(index,1);
							}
						});
						
						if (resources[key].length > 0) {
							this.checkOtherbaselines(deferreds, project, properties, resources, key);
						} else {
							this._handleError(project);
						}
					}),
					dojo.hitch(this,function(error) {
						if (resources[key].length > 0) {
							//no resource exist in the current baseline
							this.checkOtherbaselines(deferreds, project, properties, resources, key);
						} else {
						
							if (deferred.ioArgs && !dojo._isDocumentOk(deferred.ioArgs.xhr)) {
								rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
							} else {
								rmps.core.error.createError(error).dispatch();
							}
							this._handleError(project);
						}
					})
				);
			}
		}
	}
});

dojo.declare("rmps.ui.clipboard._Table", rmps.ui.Table, {
	
	pageSizeSelection: false,
	
	_onPageChange: null,
	
	postCreate: function() {
		this.inherited(arguments);
		
		if (!this.pageSizeSelection) {
			// No page size selection
			this._itemsPerPage.style.display = "none";		
		}
	},
	
	onPreviousClick: function(e) {
		this._onPageChange(this.pagingControl.page);
		this.inherited(arguments);
	},
	
	onNextClick: function(e) {
		this._onPageChange(this.pagingControl.page);
		this.inherited(arguments);
	},
	
	_populateColumn: function(rowConfig, tableRow, rowIdx, columnConfig, tableCell, colIdx) {

		if (columnConfig.checkBox) {
			var isChecked= false;
			dojo.forEach(this.search.getSelection(), function(resourceUri) {
				if (resourceUri === rowConfig.resourceUri){
					isChecked = "checked";
					return;
				}
			});
			var checkBox = 
					new dijit.form.CheckBox(this._getCheckBoxArgs(isChecked, rowConfig), "checkBox");
			if (this.search.disabledList && dojo.indexOf(this.search.disabledList, rowConfig.resourceUri)!==-1){
				checkBox.attr("disabled", true);
			}
			tableCell.appendChild(checkBox.domNode);
		} else {
			// For pinboard resources in the DM resource picker, do not
			// allow the label of the resource or its parent to be hyper links
			// to the resource or else the corresponding editor tries to open
			// the resource in the search dialog itself ... not pretty (WI 37759)
			if (columnConfig.resourceReference && !this.search.readOnly) {
				columnConfig.resourceReference = false;
			}
			
			this.inherited(arguments);
		}
	},
	
	
	_getCheckBoxArgs: function(isChecked, rowConfig) {
		return {
		    name: "checkBox",
            value: "agreed",
            checked: isChecked,
            onChange: dojo.hitch(this ,function(buttonIsChecked) {
                var resourceUri = rowConfig.resourceUri;
                if (resourceUri) {
					var index = dojo.indexOf(this.search.getSelection(), resourceUri);
					if (buttonIsChecked && index === -1) {
						this.search.getSelection().push(resourceUri);
					} else if (!buttonIsChecked && index !== -1) {
						this.search.getSelection().splice(index, 1);
					}
					this.search.onSelect(this.search.getSelection());
				}
            })
		}
	},

	_handleTbodyMouseClick: function(e) {
		/*We do not allow navigation for the table links in the picker */
		return;
	},
	
	_onPageSizeChange: function(e) {
		var pageSize = parseInt(this._pageSizeSelect.value, 10);
		this.pagingControl.update(null, pageSize, null, null);
		this._onPageChange(this.pagingControl.page);
	},
	
	_constructPageSizeDropDown: function() {
		if (!this.pageSizeSelection) {
			// No page size selection
			return;			
		}
		
		this.inherited(arguments);
	}
});

}());