/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.clipboard.ClipboardWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("rmps.ui.clipboard.ClipboardTable");

dojo.requireLocalization("rmps.ui.clipboard", "ClipboardWidgetMessages");

(function(){

dojo.declare("rmps.ui.clipboard.ClipboardWidget", [dijit._Widget, dijit._Templated], {
	templatePath: dojo.moduleUrl("rmps.ui", "clipboard/templates/ClipboardWidget.html"),
	messages: dojo.i18n.getLocalization("rmps.ui.clipboard", "ClipboardWidgetMessages"),

	// mixin
	// ok button callback
	onOkButton: null,
	// cancel button callback
	onCancelButton: null,
	// read only state, i.e. no selection
	readOnly: true,
	// if the widget is part of a full page
	isFullPage: false,
	// the type URI if the result should be restricted to a single type
	onlyType: null,
	// the project URI, to restrict the result, optional
	projectUri: null,
	// elements that should have their selection button disabled 
	disabledList: null,
	// invoked in the OSLC resource picker context
	isOslc: false,

	_table: null,
	
	postCreate: function() {
		this.inherited(arguments);
		
		if (this.isFullPage || this.readOnly) {
			this._controlArea.style.display = "none";
		}
		
		this._table = new rmps.ui.clipboard.ClipboardTable({
			readOnly: this.readOnly,
			onSelect: dojo.hitch(this, this.onTableSelect),
			isFullPage: this.isFullPage,
			onlyType: this.onlyType,
			projectUri: this.projectUri,
			baseline: this.baseline,
			isOslc: this.isOslc,
			disabledList: this.disabledList
		});
		this._clipboardContent.appendChild(this._table.domNode);
	},
	
	destroy: function() {
		if (this._table) {
			this._table.destroy();
			delete this._table;
		}
		
		this.inherited(arguments);
	},
	
	refresh: function() {
		this._table.refresh();
	},
	
	onClickOkButton: function() {
		if (this.onOkButton) {
			this.onOkButton(this._table.getSelection());
		}
	},
	
	onClickCancelButton: function() {
		if (this.onCancelButton) {
			this.onCancelButton();
		}
	},
	
	onTableSelect: function(selection) {
		var isDisabled = dojo.hasAttr(this._okButton, "disabled");
		var hasSelection = selection.length > 0;
		if (hasSelection && isDisabled) {
			dojo.removeAttr(this._okButton, "disabled");
		} else if (!hasSelection && !isDisabled) {
			dojo.attr(this._okButton, "disabled", "disabled");
		}
	},
	
	addEditorToolbarButtons: function(toolbar) {
		this._table.addEditorToolbarButtons(toolbar);
	}
});

}());