/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("rmpw.common.command.AddElement");

dojo.require("rmpw.common.command.Command");
dojo.require("rmpw.common.command.Command");
dojo.require("rmpw.common.util.IdentifierUtil");

dojo.require("rmps.core.error");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmpw.common", "AddElementMessages");

var messages = dojo.i18n.getLocalization("rmpw.common", "AddElementMessages");

/**
 * Command that creates a new uml element and creates notation for it
 * 
 * @author Marc Gobeil / Steve Shaw
 */
dojo.declare("rmpw.common.command.AddElement", rmpw.common.command.Command, {
	id: "rmpw.common.command.AddElement",
	
	updateChangeSet: function(changeSet, args) {
		if(!args.ownerElement) {
			rmps.core.error.createError(messages.UPDATE_ERROR, messages['UPDATE_ERROR.useraction'])
			     .details( messages['UPDATE_ERROR.explanation'] + messages.OWNER_ELEMENT + args.ownerElement )
		         .dispatch();
		    throw "owner element is " + args.ownerElement;
		}
		
		// create elements
		this.semanticModel = { };
		if (args.semanticModel) {
			// passed in - pre-created by dialog
			this.semanticModel = args.semanticModel;
		}
		else {
			if (args.propertyTemplates) {
				var self = this;
				
				dojo.forEach(args.propertyTemplates, function(propTemplate) {
					self.semanticModel[propTemplate.name] = propTemplate.isCollection ? [] : propTemplate.defaultValue ? propTemplate.defaultValue : null;
				});
			}
			else if(args.type) {
				this.semanticModel = this.createSemanticModel(args);
				if(!this.semanticModel) {
					rmps.core.error.createError(messages.UPDATE_ERROR, messages.ADD_ELEMENT_ACTION)
					    .details(messages['UPDATE_ERROR.explanation'] + messages.SEMANTIC_MODEL + this.semanticModel)
	                    .dispatch();
	                throw "semantic model is " + this.semanticModel;
				}
			}
			this.semanticModel.rdf_type = [{ rdf_about: args.addTypeId }];
			
			// if edge, never generated the label-- only use label from template if provided 
			if (args.nameProperty && args.defaultName && !args.isRelationship) {
				var containerProperty = args.ownerElement[args.ownerChildrenProperty];
				var defaultLabel = args.defaultName + (dojo.isArray(containerProperty) && !args.suppressIncrement? (containerProperty.length + 1) : "");
				this.semanticModel[args.nameProperty] = defaultLabel;
			}
			
			// use a faux guid 
			this.semanticModel.rdf_about = args.generateIdentifier ? rmpw.common.util.IdentifierUtil.guid() : null;
			this.semanticModel.__parentUri = args.ownerElement.rdf_about;
		}
		
		if (args.referenceElementProperty && args.referenceElement) {
			modelElement = changeSet.createChange(this.semanticModel);
			modelElement.set(args.referenceElementProperty, args.referenceElement);
		}
		
		// insert the model into the master model
		var semanticChange = changeSet.createChange(args.ownerElement);
		semanticChange.push(args.ownerChildrenProperty, this.semanticModel);
		
		/*
		 * If the new element is a fragment and it's part of the document being saved, the we need to add it
		 * to the dmcodegen_defines collection which is a derived collection that contains all the fragments in a 
		 * document.  Reason is that this collection is used after save to ensure that all the URI of the new
		 * elements are updated to be fully qualified with the document URI.  This prevents some optimization code
		 * in ServerChangeSet from ignoring subsequent changes to the object after save thinking that it's a new
		 * object.  If fully qualified, then we know it's been persisted to the server and hence not new.
		 */
		if (args.ownerElement.__mappings && args.ownerChildrenProperty != "dmcodegen_defines" && !this._options.isDocument) {
			var diagramModel = args.ownerElement.__mappings[0].root.diagram.getDefaultSource();
			
			if (diagramModel) {
				var diagramChange = new rmpw.common.vcs.Change(diagramModel);
				changeSet.add(diagramChange);
				diagramChange.push("dmcodegen_defines", this.semanticModel);
			}
		}
		
		return true;
	},
	
	createSemanticModel: function(args) {
		return new args.type(args);
	},
	
	setGenerateIdentifier: function(generateIdentifier) {
		this._options.generateIdentifier = generateIdentifier;
	},
	
	setType: function(type) {
		this._options.type = type;
	},
	
	setDefaultName: function(defaultName) {
		this._options.defaultName = defaultName;
	},
	
	setNameProperty: function(nameProperty) {
		this._options.nameProperty = nameProperty;
	},
	
	setReferenceElementProperty: function(referenceElementProperty) {
		this._options.referenceElementProperty = referenceElementProperty;
	},
	
	setOwnerChildrenProperty: function(ownerChildrenProperty) {
		this._options.ownerChildrenProperty = ownerChildrenProperty;
	},
	
	setAddTypeId: function(typeId) {
		this._options.addTypeId = typeId;
	},
	
	setPreserveLabel: function(preserveLabel) {
		this._options.preserveLabel = preserveLabel;
	},
	
	getOutput: function() {
		return {semanticModel: this.semanticModel};
	}
});

}());