/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("rmpw.common.command.AddReference");

dojo.require("rmpw.common.command.Command");
dojo.require("rmps.core.error");
dojo.require("rmpw.common.util.IdentifierUtil");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmpw.common", "AddReferenceMessages");

var messages = dojo.i18n.getLocalization("rmpw.common", "AddReferenceMessages");

/**
 * Command to add a reference from one element to another.
 * 
 * options:
 *      sourceObject: object that will reference the target 
 *      sourceProperty: property on the source object that points to the target
 *      sourcePropertyIsCollection: true if the sourceProperty is a collection
 *      targetObject: target object
 *      backreference (optional): property on the targetObject that points back to the sources
 *      backreferenceIsCollection: true if backreference property on targetObject is a collection  
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.common.command.AddReference", rmpw.common.command.Command, {
	
	updateChangeSet: function(changeSet, args) {
		if(!args.sourceObject) {
			rmps.core.error.createError(messages.SOURCE_ERROR, messages['SOURCE_ERROR.useraction'])
			     .details(messages['SOURCE_ERROR.explanation'])
			     .dispatch();
		}
		if(!args.targetObject) {
			rmps.core.error.createError(messages.TARGET_ERROR, messages['TARGET_ERROR.useraction'])
			     .details(messages['TARGET_ERROR.explanation'])
			     .dispatch();
		}
		
		// build chain of intermediate objects between source and target
		var head = {};
		var tail = head;
		for(var i = 1; i < args.objectChain.length; i++) {
			var link = args.objectChain[i];
			var prevLink = args.objectChain[i - 1];
			var node = {};
			if (link.propertyTemplates) {
				dojo.forEach(link.propertyTemplates, function(propTemplate) {
					node[propTemplate.name] = propTemplate.isCollection ? [] : propTemplate.defaultValue ? propTemplate.defaultValue : null;
				});
			}
			// note: have to set rdf:type after property templates, or a default value for it will wipe this out
			node.rdf_type = [{rdf_about: link.type}];
			node.rdf_about = rmpw.common.util.IdentifierUtil.guid();
			node.__parentUri = args.sourceObject.rdf_about;
			if(prevLink.isCollection) {
				tail[prevLink.property] = [node];
			} else {
				tail[prevLink.property] = node;
			}
			tail = node;
		}
	
		var firstLink = args.objectChain[0];
		var source = changeSet.createChange(args.sourceObject);
		if(head == tail) {
			// connect source to target directly with a single reference
			if(firstLink.isCollection) {
				source.pushReference(firstLink.property, args.targetObject);
			} else {
				source.set(firstLink.property, args.targetObject);
			}
		} else {
			// wire in the chain of intermediates between source and target
			if(firstLink.isCollection) {
				source.pushReference(firstLink.property, head[firstLink.property][0]);
			} else {
				source.set(firstLink.property, head[firstLink.property]);
			}
			var tailLink = args.objectChain[args.objectChain.length - 1];
			tail[tailLink.property] = args.targetObject;
		}
		return true;
	}
});

}());