/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.common.command.Command");

dojo.require("rmpw.common.vcs.ChangeSet");

/**
 * Abstract class for managing execution the transaction
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.common.command.Command", null, {
	
	constructor: function(options) {
		this._options = options;
	},
	
	execute: function(commandStack, args) {
		var changeSet = new rmpw.common.vcs.ChangeSet();
	
		dojo.mixin(args, this._options);
		if (this.updateChangeSet(changeSet, args)) {
			
			// perform any cleanup on commit block
	        if (dojo.isFunction(args.onCommitBlocked)) {
				this._commitBlocked = dojo.connect(commandStack.repository, "commitBlocked", dojo.hitch(this, function(){
					dojo.disconnect(this._commitBlocked);
					delete this._commitBlocked;
					args.onCommitBlocked();
				}));
				dojo.connect(commandStack.repository, "afterCommit", dojo.hitch(this, function(){
					if (this._commitBlocked) {
						dojo.disconnect(this._commitBlocked);
						delete this._commitBlocked;
					}
				}));
	        }
						
			
			commandStack.repository.commit(changeSet, true);
			return true;
		}
		
		return false;
	},

	/*
	 * Sub-classes should override to populate the given change set
	 */
	updateChangeSet: function(changeSet, args) {
		return true;
	},
	
	/*
	 * Returns the output object for this command
	 */
	getOutput: function() {
		return null;
	},
	
	/*
	 * Serializes command arguments to json
	 */
	argsToJson: function(args) {
		return dojo.toJson(args);
	},
	
	/*
	 * Deserializes command arguments from json
	 * idLookupFn can be provided to support index lookup
	 */
	argsFromJson: function(json, idLookupFn) {
		return dojo.fromJson(json);
	}
});