/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.common.command.DeleteElement");

dojo.require("rmpw.common.command.Command");

/**
 * Command that deletes an element
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.common.command.DeleteElement", rmpw.common.command.Command, {
	id: "rmpw.common.command.DeleteElement",
	
	updateChangeSet: function(changeSet, args) {
		var semanticChange = changeSet.createChange(args.ownerElement);
		semanticChange.removeArray(args.ownerChildrenProperty, args.elementToDelete);
		
		/*
		 * Since element may be in the dmcodegen_defines collection as well if it's a fragment
		 * @see comment in AddElement.js.  We need to remove it from there as well.
		 */
		if (args.elementToDelete.__mappings && args.ownerChildrenProperty != "dmcodegen_defines") {
			var diagramModel = args.elementToDelete.__mappings[0].root.diagram.getDefaultSource();
			
			if (diagramModel && diagramModel.dmcodegen_defines && dojo.indexOf(diagramModel.dmcodegen_defines, args.elementToDelete) >= 0) {
				var diagramChange = new rmpw.common.vcs.Change(diagramModel);
				changeSet.add(diagramChange);
				diagramChange.removeArray("dmcodegen_defines", args.elementToDelete);
			}
		}
		
		return true;
	},
	
	setOwnerChildrenProperty: function(ownerChildrenProperty) {
		this._options.ownerChildrenProperty = ownerChildrenProperty;
	},
	
	getOutput: function() {
		return {};
	}
});