/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.common.command.DomainCommandSet");

/**
 * Package of commands that apply to the corresponding semantic package's classes,
 * indexed by class id.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.common.command.DomainCommandSet", null, {
	/**
	 * Type that every type implicitly inherits from
	 */
	globalBaseType: 'http://www.w3.org/2000/01/rdf-schema#Resource',
	
	allSuperClasses: {},
	
	constructor: function() {
		if(!this.classesByName) {
			this.classesByName = {};
		}
		if(!this.allSuperClasses) {
			this.allSuperClasses = {};
		}
		// TODO (MG): deprecate init method
		if(this.init) {
			this.init();
		}
	},

	// TODO (MG): deprecate, require class definition to have classesByName specified
	add: function(classDef) {
		this.classesByName[classDef.typeId] = classDef;
	},
	
	select: function(query) {
		var typeList = query.typeId;
		
		// TODO: standardize this so it's always an array
		if(!dojo.isArray(typeList)) {
			console.warn("typeId parameter for selection of commands should be passed as an array");
			typeList = [typeList];
		}
		
		var command = null;
		for(var i = 0, typeId; !command && (typeId = typeList[i]); i++) {
			var allSuperClasses = this.allSuperClasses[typeId] || [];
			var j = allSuperClasses.length;
			do {
				var classDef = this.classesByName[typeId];
				if(classDef) {
					command = classDef.select(query);
				}
				j--;
				if (j >= 0 && j < allSuperClasses.length) {
					typeId = allSuperClasses[j];
				}
			}
			while (j >= 0 && !command && typeId);
		}
		if(!command) {
			throw "No commands found for query " + dojo.toJson(query);
		}
		return command;
	},
	
	isTypeOf: function(obj, type) {
		if(type == this.globalBaseType) {
			return true;
		}
		
		var isOfType = false;
		var typeList = null;
		if(obj && (typeList = obj.rdf_type)) {
			// extract uri's from type objects
			typeList = dojo.map(typeList, this._getRdfAbout);
			
			// do cheap check for the class being directly in the list
			var index = dojo.indexOf(typeList, type);
			if(index != -1) {
				isOfType = true;
			}
			// perform more expensive deep inheritance tree check
			else {
				for(var i = 0, possibleType; possibleType = typeList[i]; i++) {
					var superTypes = this.allSuperClasses[possibleType];
					if(superTypes) {
						index = dojo.indexOf(superTypes, type);
						if(index != -1) {
							isOfType = true;
							break;
						}
					}
				}
			}
		}
		return isOfType;
	},
	
	_getRdfAbout: function(obj) {
		return obj.rdf_about || obj.$ref;
	}
});