/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.common.command.DuplicateElement");

dojo.require("rmpw.common.command.Command");
dojo.require("rmpw.common.util.IdentifierUtil");

/**
 * Command that will reparent a graph node
 * 
 * @author John Swanke
 */
dojo.declare("rmpw.common.command.DuplicateElement", rmpw.common.command.Command, {
	id: "rmpw.common.command.DuplicateElement",
	
	updateChangeSet: function(changeSet, args) {
	
		var mappingToDuplicate = args.duplicate.renderMapping;
		var elementToDuplicate = mappingToDuplicate.getDefaultSource();
		
		var mappingParent = mappingToDuplicate.parent;
		var elementParent = mappingParent.getDefaultSource();

		var addChange = changeSet.createChange(elementParent);
		var mappingState = new rmpw.common.transform.MappingState(mappingToDuplicate, true);
		var y = mappingState.state.y;
		var x = mappingState.state.x + (args.cellSize.width / args.matrix.xx);
		for (var row = 0; row < args.size.rows; row++) {
			for (var col = 0; col < args.size.cols; col++) {
				if (row==0 && col==0) continue;
				
				// create dup element
				this.duplicateElement = { };
				var newMappingState = new rmpw.common.transform.MappingState();
				newMappingState.state = dojo.clone(mappingState.state);
				newMappingState.copy = true;
				newMappingState.key = mappingState.key;
				newMappingState.root = mappingState.root;
				for (var propName in elementToDuplicate) {
					if(propName.substring(0,1) != "_") { // ignore junk	
			        	var prop = elementToDuplicate[propName];
						if (prop && (!dojo.isArray(prop) || prop.length>0) && !dojo.isFunction(prop)) {
				        	if (propName == "rdf_about") {
								prop = rmpw.common.util.IdentifierUtil.guid();
				        	} else if (propName == "rdf_type") {
								prop = [{ rdf_about: elementToDuplicate.rdf_type[0].rdf_about }];
				        	} else if (!prop.rdf_about && prop.$ref) {
				        		prop.rdf_about = prop.$ref;
				        	}
							this.duplicateElement[propName] = prop;
						}
					}
				}
				
				// inject the x and y positions
				newMappingState.state.x = x;
				newMappingState.state.y = y;
				this.duplicateElement.__inactiveMappings = [newMappingState];
				
				// add to changeset
				addChange.push(args.ownerChildrenProperty, this.duplicateElement);
				x += args.cellSize.width / args.matrix.xx;
			}
			x = mappingState.state.x;
			y += args.cellSize.height / args.matrix.yy;
		}
		
		return true;
	}
	
	
});
