/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.common.command.ElementCommandSet");


/**
 * Set of commands that apply to a given semantic class. 
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.common.command.ElementCommandSet", null, {
	typeId: null,
	
	constructor: function(typeId) {
		this._commands = {};
		this.configureCommands();
	},
	
	configureCommands: function() {
		// abstract, subclasses to implement
	},
	
	add: function(command) {
		var commandList = this._commands[command.declaredClass];
		if(!commandList) {
			commandList = [];
			this._commands[command.declaredClass] = commandList;
		}
		commandList.push(command);
	},
	
	select: function(query) {
		var commands = this._commands[query.commandId];

		var found = null;
		if(commands) {
			// save used query parameters
			var typeId = query.typeId;
			var commandId = query.commandId;

			// remove used query parameters so they don't interfere with "matches all" check in loop
			delete query.typeId;
			delete query.commandId;
			
			// search for commands that match all remaining parameters
			for(var i = 0, command; command = commands[i]; i++) {
				var match = true;
				var options = command._options;
				for(var prop in query) {
					if (query[prop] !== undefined) {
						if(options[prop] != query[prop]) {
							match = false;
							break;
						}
					}
				}
				if(match) {
					found = dojo.clone(command);
					break;
					// TODO (MG): debug mode that checks to assert no more than one match exists?
				}
			}
			
			// restore used parameters if this particular class is not a match (a superclass checked next might)
			query.typeId = typeId;
			query.commandId = commandId;
		}
		
		return found;
	}
});