/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.common.command.ReorientRelationship");

dojo.require("rmpw.common.command.Command");

/**
 * Command that will reorient a relationship
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.common.command.ReorientRelationship", rmpw.common.command.Command, {
	id: "rmpw.common.command.ReorientRelationship",
	
	updateChangeSet: function(changeSet, args) {
		
		// handle the semantics
		var modelElement = changeSet.createChange(args.relElement);
		var newSourceElement = args.newSourceElement;
		if (newSourceElement) {
			var oldSourceElement = args.relElement[args.sourceProperty];
			if (dojo.isArray(oldSourceElement)) {
				// pick first one for now - don't support multiple sources
				oldSourceElement = oldSourceElement[0];
				
				if(args.sourceContainerProperty in newSourceElement) {
					modelElement.push(args.sourceProperty, newSourceElement);
				} else {
					modelElement.set(args.sourceProperty, [newSourceElement]);
				}
			}
			else {
				modelElement.set(args.sourceProperty, newSourceElement);
			}
			
			if (args.sourceIsContainer && (oldSourceElement != newSourceElement)) {
				var oldSourceChange = changeSet.createChange(oldSourceElement);
				oldSourceChange.removeArray(args.ownerChildrenProperty, args.relElement);
				
				if(args.sourceContainerProperty in newSourceElement) {
					oldSourceChange.push(args.ownerChildrenProperty, args.relElement);
				} else {
					oldSourceChange.set(args.ownerChildrenProperty, [args.relElement]);
				}
			}
		}
		
		var newTargetElement = args.newTargetElement;
		if (newTargetElement) {
			modelElement.set(args.targetProperty, newTargetElement);
			var oldTargetElement = args.relElement[args.targetProperty];
			if (dojo.isArray(oldTargetElement)) {
				// pick first one for now - don't support multiple sources
				oldTargetElement = oldTargetElement[0];
				
				if(args.targetContainerProperty in newTargetElement) {
					modelElement.push(args.targetProperty, newTargetElement);
				} else {
					modelElement.set(args.targetProperty, [newTargetElement]);
				}
			}
			else {
				modelElement.set(args.targetProperty, newTargetElement);
			}
		}
		
		return true;
	},
	
	setSourceIsContainer: function(iscontainer) {
		this._options.sourceIsContainer = iscontainer;
	},
	
	setSourceProperty: function(sourceProperty) {
		this._options.sourceProperty = sourceProperty;
	},
	
	setTargetProperty: function(targetProperty) {
		this._options.targetProperty = targetProperty;
	},
	
	setOwnerChildrenProperty: function(ownerChildrenProperty) {
		this._options.ownerChildrenProperty = ownerChildrenProperty;
	}
});