/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.commands.ChangeColorCommand");

dojo.require("rmpw.common.command.Command");

/**
 * 
 * @author Steve Shaw
 * 
 * Command for changing the color of a shape
 */
dojo.declare("rmpw.diagrams.edit.commands.ChangeColorCommand", rmpw.common.command.Command, {
	id: "rmpw.diagrams.edit.commands.ChangeColorCommand",
	description: "Change Color",
	
	updateChangeSet: function(changeSet, args) {
		var traceName = null;
		var color = null;
		if (args.fillColor) {
			traceName = "fillColor";
			color = args.fillColor;
		}
		if (args.lineColor) {
			traceName = "lineColor";
			color = args.lineColor;
		}
		if (args.fontColor) {
			traceName = "fontColor";
			color = args.fontColor;
		} 
		
		if (!color) {
			return false;
		}
		
		for(var i = 0, editController; editController = args.editControllers[i]; i++) {
			var renderMapping = editController.renderMapping;
			
			var styleable = renderMapping.policies.styleable;
			if (!styleable) {
				continue;
			}
			
			var colorTrace = renderMapping.traces[styleable._getInternalTraceName(traceName)];
			
			if (colorTrace && color) {
				var change = changeSet.createChange(colorTrace.getOwner());
				var dojoColor = new dojo.Color(color);
				dojoColor.a = args.opacity;
				change.set(colorTrace.propertyName, dojoColor);
			}
		}
		
		return true;
	}

});