/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.commands.ChangeFontStyleCommand");

dojo.require("rmpw.common.command.Command");

/**
 * Command for changing the font style.
 * 
 * @author Cherie Revells
 */
dojo.declare("rmpw.diagrams.edit.commands.ChangeFontStyleCommand", rmpw.common.command.Command, {
	id: "rmpw.diagrams.edit.commands.ChangeFontStyleCommand",
	description: "Change Font Style",
	
	updateChangeSet: function(changeSet, args) {
		if (!args.font) {
			return false;
		}
		for(var i = 0, editController; editController = args.editControllers[i]; i++) {
			var renderMapping = editController.renderMapping;	
			var styleable = editController.renderMapping.policies.styleable;
			
			if (!styleable) {
				continue;
			}
	
			if (args.font.family) {
				var trace = renderMapping.traces[styleable._getInternalTraceName("fontFamily")];
				if (trace) {
					var change = changeSet.createChange(trace.getOwner());
					change.set(trace.propertyName, args.font.family);
				}
			}
			if (args.font.style) {
				var trace = renderMapping.traces[styleable._getInternalTraceName("fontStyle")];
				if (trace) {
					var change = changeSet.createChange(trace.getOwner());
					change.set(trace.propertyName, args.font.style);
				}
			}
			if (args.font.weight) {
				var trace = renderMapping.traces[styleable._getInternalTraceName("fontWeight")];
				if (trace) {
					var change = changeSet.createChange(trace.getOwner());
					change.set(trace.propertyName, args.font.weight);
				}
			}
			if (args.font.size) {
				var trace = renderMapping.traces[styleable._getInternalTraceName("fontSize")];
				if (trace) {
					var change = changeSet.createChange(trace.getOwner());
					change.set(trace.propertyName, args.font.size);
				}
			}			

		}
		return true;
	}
});

