/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.commands.ChangeLinePropertiesCommand");

dojo.require("rmpw.common.command.Command");

/**
 * Command for changing the line properties.
 * 
 * @author sshaw
 */
dojo.declare("rmpw.diagrams.edit.commands.ChangeLinePropertiesCommand", rmpw.common.command.Command, {
	id: "rmpw.diagrams.edit.commands.ChangeLinePropertiesCommand",
	description: "Change Line Properties",
	
	updateChangeSet: function(changeSet, args) {
		if (!args.router && !args.avoidObstructions && !args.smoothness) {
			return false;
		}
		for(var i = 0, editController; editController = args.editControllers[i]; i++) {
			var renderMapping = editController.renderMapping;		
			var graphEdge = renderMapping.policies.graphEdge;
			
			if (!graphEdge) {
				continue;
			}
			
			if (args.router) {
				var routerTrace = renderMapping.traces[graphEdge._getInternalTraceName("router")];
				if (routerTrace) {
					var change = changeSet.createChange(routerTrace.getOwner());
					change.set(routerTrace.propertyName, args.router);
				}
			}
			
			if (args.smoothness) {
				var smoothnessTrace = renderMapping.traces[graphEdge._getInternalTraceName("smoothness")];
				if (smoothnessTrace) {
					var change = changeSet.createChange(smoothnessTrace.getOwner());
					change.set(smoothnessTrace.propertyName, args.smoothness);
				}
			}
			
			if (args.avoidObstructions !== undefined) {
				var avoidObstructionsTrace = renderMapping.traces[graphEdge._getInternalTraceName("avoidObstructions")];
				if (avoidObstructionsTrace) {
					var change = changeSet.createChange(avoidObstructionsTrace.getOwner());
					change.set(avoidObstructionsTrace.propertyName, args.avoidObstructions);
				}
			}
			
		}
		
		return true;
	}

});