/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.web.comment.sketch.commands.EditFontStyle");

dojo.require("dojox.gfx._base");

dojo.require("rmpw.common.command.Command");

dojo.declare("rmps.web.comment.sketch.commands.EditFontStyle", rmpw.common.command.Command, {
	
	id: "rmps.web.comment.sketch.commands.EditFontStyle",
	icon: null,
	description: "EditFontStyle",
	
	updateChangeSet: function(changeSet, args) {
				
		for( var node = 0; node < args.nodes.length; node++ ){				
			this.stylize( changeSet, args.nodes[node].renderMapping.sources.svgRoot, args.font );
		}
	
		return true;
	},
	
	stylize: function( changeSet, drawing, font ){		
		var type = drawing.eClass;
		type = type.substring(0, type.length - 4).toLowerCase();
		this._stylize[type].call(this, changeSet, drawing, font );
	},
	
	handleFont: function( change, font ){
		
		if( font.family ){
			change.set( "fontFamily", font.family );
		}
		
		if( font.size ){
			change.set( "fontSize", font.size );
		}
		
		if( font.style ){
			change.set( "fontStyle", font.style );
		}
		
		if( font.weight ){
			change.set( "fontWeight", font.weight );
		}
	},
	
	_stylize: {
		g: function( changeSet, drawing, font ){
			dojo.forEach(["g", "rect", "path", "line", "ellipse"], function(type) {
				if(type in drawing) {
					var typeList = drawing[type];
					var stylizeFunction = this._stylize[type];
					for( var child, i = 0; child = typeList[i]; i++ ) {
						stylizeFunction.call(this, changeSet, child, font );
					}
				}
			}, this);
		},
		
		line: function(changeSet, drawing, font ){
			var change = changeSet.createChange(drawing);
			this.handleFont( change, font );
		},
	
		ellipse: function(changeSet, drawing, font ){
			var change = changeSet.createChange(drawing);
			this.handleFont( change, font );			
		},
	
		rect: function(changeSet, drawing, font ){
			var change = changeSet.createChange(drawing);
			this.handleFont( change, font );
		},
	
		path: function(changeSet, drawing, font ){
			var change = changeSet.createChange(drawing);
			this.handleFont( change, font );
		}
	}
});