/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.web.comment.sketch.commands.EditStyle");

dojo.require("dojox.gfx._base");

dojo.require("rmpw.common.command.Command");

dojo.declare("rmps.web.comment.sketch.commands.EditStyle", rmpw.common.command.Command, {
	id: "rmps.web.comment.sketch.commands.EditStyle",
	icon: null,
	description: "EditStyle",
	
	updateChangeSet: function(changeSet, args) {
				
		for( var node = 0; node < args.nodes.length; node++ ){				
			this.stylize( changeSet, args.nodes[node].renderMapping.sources.svgRoot, args.stroke, args.fill, args.text );
		}
	
		return true;
	},
	
	stylize: function( changeSet, drawing, stroke, fill, text ){
		var type = drawing.eClass;
		// take out the "Type" postfix and lowercase it
		type = type.substring(0, type.length - 4).toLowerCase();
		this._stylize[type].call(this, changeSet, drawing, stroke, fill, text );
	},
	
	handleStroke: function( change, stroke ){	
		
		if( stroke ){
			if(stroke.color){
				change.set("stroke", stroke.color);
				
				if(stroke.opacity){
					change.set("strokeOpacity", stroke.opacity);
				}
				
			}else if(stroke.width){
				change.set("strokeWidth", stroke.width);
			}else if(stroke.dash){
				change.set("strokeDasharray", stroke.dash);
			}
		}
	},
	
	handleFill: function( change, fill ){
		
		if( fill ){
			
			if( fill.color ){
				change.set("fill", fill.color);
			}

			if( fill.opacity ){
				change.set("fillOpacity", fill.opacity);
			}
		}
	},
	
	handleText: function( change, fill ){
		
		if( fill ){
			
			if( fill.color ){
				change.set("textColor", fill.color);
			}

			if( fill.opacity ){
				change.set("textOpacity", fill.opacity);
			}
		}
	},
	
	/* TODO: Add handling for these types. */
	//		"circle"
	//		"PolylineType":	
	//		"ImageType":				    	
	//		"TextType":
	
	_stylize: {
		g: function( changeSet, drawing, stroke, fill, text ){
			dojo.forEach(["g", "rect", "path", "line", "ellipse"], function(type) {
				if(type in drawing) {
					var typeList = drawing[type];
					var stylizeFunction = this._stylize[type];
					for( var child, i = 0; child = typeList[i]; i++ ) {
						stylizeFunction.call(this, changeSet, child, stroke, fill, text );
					}
				}
			}, this);
		},
		
		line: function(changeSet, drawing, stroke, fill, text){
			var change = changeSet.createChange(drawing);
			this.handleStroke( change, stroke );
			this.handleFill( change, fill );
			this.handleText( change, text );
		},
	
		ellipse: function( changeSet, drawing, stroke, fill, text ){
			var change = changeSet.createChange(drawing);
			this.handleStroke( change, stroke );
			this.handleFill( change, fill );
			this.handleText( change, text );
		},
	
		rect: function( changeSet, drawing, stroke, fill, text ){
			var change = changeSet.createChange(drawing);
			this.handleStroke( change, stroke );
			this.handleFill( change, fill );
			this.handleText( change, text );
		},
	
		path: function( changeSet, drawing, stroke, fill, text ){
			var change = changeSet.createChange(drawing);
			this.handleStroke( change, stroke );
			this.handleFill( change, fill );
			this.handleText( change, text );
		}
	}
});