/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.commands.MoveCommand");

dojo.require("rmpw.common.command.Command");

/**
 * 
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.diagrams.edit.commands.MoveCommand", rmpw.common.command.Command, {
	id: "rmpw.diagrams.edit.commands.MoveCommand",
	icon: null,
	description: "Move",
	
	updateChangeSet: function(changeSet, args) {
	    var editController;
		for(var i = 0, len = args.nodes.length; i < len; i++) {
			var editmapping = null;
			if (args.renderMappings) {
				// TODO: get edit mapping some other way
				editController = args.nodes[i].__editController;
			} else {
				editController = args.nodes[i];
			}
			var policies = editController.renderMapping.policies;
			
			var parentPolicies = editController.parent.renderMapping.policies;
			var isInShapeContainer = parentPolicies.graphContainer && parentPolicies.graphNode; 
			
			var processed = [];
			for (var id in policies) {
				var graphPolicy = policies[id];
				// avoid repeating ourselves
				if (dojo.indexOf(processed, graphPolicy) === -1) {
					processed.push(graphPolicy);
					if (graphPolicy.getConstraintBounds && editController.__setUpdatePosition) {
						var oldBounds = graphPolicy.getConstraintBounds();
						var x = args.location ? args.location.x : oldBounds.x + args.shift.dx;
						var y = args.location ? args.location.y : oldBounds.y + args.shift.dy;
						
						// if moving in a shape container, don't allow moving past -1,-1
						if (isInShapeContainer) {
							if (x<0){ 
							    x=10;
							}if (y<0){ 
							    y=10;
						    }
						}
						
						editController.__setUpdatePosition(changeSet, x, y);
					}
				}
			}
			delete processed;
		}
	
		return true;
	},
	
	argsToJson: function(args) {
		var nodes = dojo.map(args.nodes, function(item) {
			return item.renderMapping.id;
		});
		return dojo.toJson({
			nodes: nodes,
			shift: {
				dx: args.shift.dx,
				dy: args.shift.dy
			}
		});
	},
	
	argsFromJson: function(json, idLookupFn) {
		var args = dojo.fromJson(json);
		var nodes = [];
		if (idLookupFn) {
			nodes = dojo.map(args.nodes, idLookupFn);
		}
		return {
			nodes: nodes,
			shift: args.shift,
			renderMappings: true
		};
	}
});