/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.commands.SetConnectionBendpointsCommand");

dojo.require("rmpw.common.command.Command");

/**
 * A command that will set the bendpoints of an edge view.
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.commands.SetConnectionBendpointsCommand", rmpw.common.command.Command, {
	id: "rmpw.diagrams.edit.commands.SetConnectionBendpointsCommand",
	icon: null,
	
	updateChangeSet: function(changeSet, args) {
		var mapping = null;
		var sourceRef = null;
		var targetRef = null;
		
		// TODO: get edit mapping some other way in serialization case
		var editController = args.serialized ? args.mapping.__editController : args.editController;
		
		if (args.serialized) {
			mapping = args.mapping;
			
			sourceRef = args.sourceRef;
			targetRef = args.targetRef;
		} else {
			mapping = args.editController.renderMapping;
			
			sourceRef = args.sourceAnchor.getReferencePoint();
			mapping.policies.graphElement.translateToRelative(sourceRef);
			targetRef = args.targetAnchor.getReferencePoint();
			mapping.policies.graphElement.translateToRelative(targetRef);
		}
		
		var graphEdge = mapping.policies.graphEdge;
		if (!graphEdge) {
			return false;
		}
				
		var bendpts = [];
		for (var j = 0, lg = args.points.length; j < lg; j++) {
			// create svg styled bendpoint string
			bendpts.push(args.points[j].x - sourceRef.x); //sourceX
			bendpts.push(args.points[j].y - sourceRef.y); //sourceY
			bendpts.push(args.points[j].x - targetRef.x); //targetX
			bendpts.push(args.points[j].y - targetRef.y); //targetY
		}
		var bendpoints = bendpts.join(" ");
		
		if (args.points.length > 2) {
			// turn off avoid obstructions if more then 1 bendpoint
			var avoidObstructionsTrace = editController.renderMapping.traces[graphEdge._getInternalTraceName("avoidObstructions")];
			if (avoidObstructionsTrace) {
				var change = changeSet.createChange(avoidObstructionsTrace.getOwner());
				change.set(avoidObstructionsTrace.propertyName, false);
			}
		}
		
		var sourceTerminalString = args.serialized ? args.sourceTerminalString : args.sourceAnchor.composeTerminalString(); 
		var targetTerminalString = args.serialized ? args.targetTerminalString : args.targetAnchor.composeTerminalString();
		
		if (editController.policies.polylineEditable.setUpdateBendpoints) {
			editController.policies.polylineEditable.setUpdateBendpoints(changeSet, bendpoints, 
					sourceTerminalString, targetTerminalString, false, args.autoLock);
		}
		
		return true;
	},
	
	argsToJson: function(args) {
		var mapping = args.editController.renderMapping;
		var id = mapping.id;
		
		var sourceRef = args.sourceAnchor.getReferencePoint();
		mapping.policies.graphElement.translateToRelative(sourceRef);
		var targetRef = args.targetAnchor.getReferencePoint();
		mapping.policies.graphElement.translateToRelative(targetRef);
		
		var sourceTerminalString = args.sourceAnchor.composeTerminalString(); 
		var targetTerminalString = args.targetAnchor.composeTerminalString();

		return dojo.toJson({
			id: id,
			points: args.points,
			sourceRef: sourceRef,
			targetRef: targetRef,
			sourceTerminalString: sourceTerminalString,
			targetTerminalString: targetTerminalString
		});
	},
	
	argsFromJson: function(json, idLookupFn) {
		var args = dojo.fromJson(json);
		var renderMapping = null;
		if (idLookupFn) {
			renderMapping = idLookupFn(args.id);
		}
		args.mapping = renderMapping;
		args.serialized = true;
		return args;
	}	
});

