/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmps.web.comment.sketch.commands.shapes.CreateShapeAction");

dojo.require("rmpw.diagrams.edit.actions.Action");
dojo.require("rmps.web.comment.sketch.behavior.Scribbler");
dojo.require("rmps.web.comment.sketch.behavior.LineMaker");
dojo.require("rmps.web.comment.sketch.behavior.RectangleMaker");
dojo.require("rmps.web.comment.sketch.behavior.TextMaker");
dojo.require("rmps.web.comment.sketch.behavior.OvalMaker");

dojo.declare("rmps.web.comment.sketch.commands.shapes.CreateShapeAction", rmpw.diagrams.edit.actions.Action, {
	
	constructor: function() {
	},
	
	run: function(args){
		var editController = this.host;
		if(editController && editController.viewer && editController.viewer.getCommandController) {
			this.doRun(editController.viewer.getCommandController(), editController, args);
		}
	},
	
	doRun: function(controller, editController, args) {
		// parse args
		var relpos = editController.viewer.viewport.getSurfaceRelativeMousePosition(args.pageX, args.pageY);
		
		var commandArgs = {
				svgParent: editController.renderMapping.sources.svgRoot,
				shape: this._options.addTypeId,
				x: relpos.x,
				y: relpos.y,
				viewport: controller.viewer.viewport
		};
		
		switch( this._options.addTypeId ){
		
			case "Scribble":
			
				if(!this.scribbler){
					this.scribbler = new rmps.web.comment.sketch.behavior.Scribbler( controller );
				}
					
				this.scribbler.capture( commandArgs );
				
				break;
			
			case "Line":
				
				if(!this.lineMaker){
					this.lineMaker = new rmps.web.comment.sketch.behavior.LineMaker( controller );
				}
					
				this.lineMaker.capture( commandArgs );
				
				break;
				
			case "Rectangle":
				
				if(!this.rectMaker){
					this.rectMaker = new rmps.web.comment.sketch.behavior.RectangleMaker( controller );
				}
					
				this.rectMaker.capture( commandArgs );
				
				break;
				
			case "Oval":
				
				if(!this.ovalMaker){
					this.ovalMaker = new rmps.web.comment.sketch.behavior.OvalMaker( controller );
				}
					
				this.ovalMaker.capture( commandArgs );
				
				break;
				
				
			case "Cloud":
				
				if(!this.cloudMaker){
					this.cloudMaker = new rmps.web.comment.sketch.behavior.CloudMaker( controller );
				}
					
				this.cloudMaker.capture( commandArgs );
				
				break;
				
			case "Text":
				
				if(!this.textMaker){
					this.textMaker = new rmps.web.comment.sketch.behavior.TextMaker( controller );
				}
					
				this.textMaker.capture( commandArgs );
				
				break;
				
			case "TextBox":
				
				console.log( 'CREATE SHAPE ACTION - TEXTBOX' );
				
				if(!this.boxMaker){
					this.boxMaker = new rmps.web.comment.sketch.behavior.TextMaker( controller );
				}
					
				this.boxMaker.capture( commandArgs );
				
				break;
				
			
			default:
				throw "Trying to call an unknown creation command";	
				break;
		}		
	}	
});

