/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.commenting.AbstractCommentEntry");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.declare("rmps.ui.commenting.AbstractCommentEntry", [dijit._Widget, dijit._Templated], {
	
	templateString: dojo.cache("rmps.ui", "commenting/templates/AbstractCommentEntry.html"),
	
	// attach points
	pane: null,
	contentPane: null,
	containerNode: null,
	actionBar: null,
	
	// mixin properties
	group: null,
	comment: null,
	
	// properties
	isVisible: true,
	replies: null, //[]
	parent: null,
	
	// private properties
	_selected: false,
	_root: null,
	
	constructor: function(view) {
		this.replies = [];
	},
	
	postCreate: function() {
		this.refresh();
	},
	
	uninitialize: function() {
		if (this.parent) {
			this.parent.removeReply(this);
		}
	},
	
	refresh: function() {
		// override
	},
	
	resize: function(size) {
		
	},
	
	isSelected: function() {
		return this._selected;
	},
	
	select: function() {
		this._selected = true;
		dojo.addClass(this.pane, "entry-pane-highlight");
	},
	
	deselect: function() {
		this._selected = false;
		dojo.removeClass(this.pane, "entry-pane-highlight");
	},
	
	setVisibility: function(visible) {
		if (this.isVisible != visible) {
			this.isVisible = visible;
			if (this.isVisible) {
				this.domNode.style.display = "";
			} else {
				this.domNode.style.display = "none";
			}
		}
	},
	
	setExpanded: function(expanded) {
		// override
	},
	
	// action bar
	showActionBar: function(show) {
		if (show) {
			this.actionBar.style.visibility = "";
		} else {
			this.actionBar.style.visibility = "hidden";
		}
	},
	
	// pane events
	
	_onPaneMouseDown: function(e) {
		this.group.handleSelection(e, this);
	},
	
	_onPaneKeyUp: function(e) {
	   var keyCode = (e.keyCode ? e.keyCode : e.which);
	   if(keyCode === dojo.keys.ENTER){
	       this._onPaneMouseDown(e);
	   }  
	},
	
	_onPaneMouseOver: function(e) {
		dojo.addClass(this.pane, "entry-pane-hover");
		this.showActionBar(true);
	},
	
	_onPaneMouseOut: function(e) {
		dojo.removeClass(this.pane, "entry-pane-hover");
		if (!this.isSelected() || this.group.commentingWidget.hasSelectedComments()) {		
			this.showActionBar(false);
		}
	},
	
	// heirarchy support
	
	isRoot: function() {
		return this.parent == null;
	},
	
	getRoot: function() {
		if (!this._root) {
			this._root = this;
			var parent = this;
			while (parent = parent.parent) {
				this._root = parent;
			}
		}
		return this._root;
	},
	
	getAllReplies: function() {
		var allReplies = [];
		for (var i = 0; i < this.replies.length; ++i) {
			allReplies.push(this.replies[i]);
			var grandReplies = this.replies[i].getAllReplies();
			for (var j = 0; j < grandReplies.length; ++j) {
				allReplies.push(grandReplies[j]);
			}
		}
		return allReplies;
	},
	
	addReply: function(reply) {
		this.replies.push(reply);
	},
	
	removeReply: function(reply) {
		var idx = dojo.indexOf(this.replies, reply);
		if (idx !== -1) {
			this.replies.splice(idx, 1);
		}
	}

});

})();