/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.commenting.AbstractGroup");

dojo.require("dijit._Widget");

dojo.require("rmps.ui.commenting.CommentEntry");

dojo.declare("rmps.ui.commenting.AbstractGroup", dijit._Widget, {
	
	// mixin properties
	commentingWidget: null,
	comments: null,
	
	// properties
	commentEntries: null,
	commentEntriesHash: null,
	commentEntryRoots: null,
	
	// private properties
	_handlingSelection: false,
	_lastSelectedEntry: null,
	
	constructor: function() {
		this.commentEntries = [];
		this.commentEntriesHash = {};
		this.commentEntryRoots = [];
	},
	
	buildRendering: function(){
		this.inherited(arguments);
		this.containerNode = this.domNode;
	},
	
	postCreate: function() {
		this.connect(this.commentingWidget, "onSelectionChanged", this.selectionChanged);
		this.connect(this.commentingWidget, "onCommentCreated", this.addComment);
		this.connect(this.commentingWidget, "onCommentInserted", this.addComment);
		this.connect(this.commentingWidget, "onCommentUpdated", this.refreshComment);
		this.connect(this.commentingWidget, "onFilterComments", this._filterComments);
		this.connect(this.commentingWidget, "onCommentDeleted", this.deleteComment);
		this.connect(this.commentingWidget, "onCommentRemoved", this.deleteComment);
		this.connect(this.commentingWidget, "onMarkCommentToDelete", this._markCommentToDelete);
		this.connect(this.commentingWidget, "onUnmarkCommentToDelete", this._unmarkCommentToDelete);
	},
	
	startup: function() {
		if (this._started) {
			return;
		}
		
		this.createEntries();
		this.prepareEntries();
		this.filterComments(this.commentingWidget.getFilteredComments(), this.commentingWidget.getUnfilteredComments());
		this.addEntries();

		var selected = this.commentingWidget.getSelectedComments();
		if (selected.length > 0) {
			for (var i = 0; i < selected.length; ++i) {
				var entry = this.commentEntriesHash[selected[i].uri];
				if (entry) {
					entry.select();
					if (selected.length == 1) {
						entry.showActionBar(true);
					}
				}
			}
		}
		
		this.inherited(arguments);
	},
	
	resize: function(size) {
		var newSize = size ? {w: size.w } : null;
		for (var i = 0; i < this.commentEntries.length; ++i) {
			var entry = this.commentEntries[i];
			if (entry.resize && entry.isVisible) {
				entry.resize(newSize);
			}
		}
	},
	
	addEntries: function() {
		for (var i = 0; i < this.commentEntries.length; ++i) {
			var entry = this.commentEntries[i];
			this.domNode.appendChild(entry.domNode);
			entry.startup();
		}
	},
	
	selectionChanged: function(selection, previousSelection, selected, deselected) {
		var i, entry;
		for (i = 0; i < deselected.length; ++i) {
			entry = this.commentEntriesHash[deselected[i].uri];
			if (entry) {
				entry.deselect();
			}
		}
		for (i = 0; i < selected.length; ++i) {
			entry = this.commentEntriesHash[selected[i].uri];
			if (entry) {
				entry.select();
			}
		}
		if (previousSelection.length == 1) {
			entry = this.commentEntriesHash[previousSelection[0].uri];
			if (entry) {
				entry.showActionBar(false);
			}
		}
		if (selection.length == 1) {
			entry = this.commentEntriesHash[selection[0].uri];
			if (entry) {
				entry.showActionBar(true);
				if (!this._handlingSelection) {
					this._lastSelectedEntry = entry;
					dijit.scrollIntoView(entry.domNode);
				}
			}
		}
	},
	
	addComment: function(comment) {
		// To be implemented by clients
	},
	
	deleteComment: function(comment) {
		var entry = this.commentEntriesHash[comment.uri];
		if (entry) {
			entry.destroyRecursive();
			delete this.commentEntriesHash[comment.uri];
		}
		
		this.commentEntries.splice(dojo.indexOf(this.commentEntries, entry), 1);
		var rIdx = dojo.indexOf(this.commentEntryRoots, entry);
		if (rIdx !== -1) {
			this.commentEntryRoots.splice(rIdx, 1);
		}
	},
	
	_markCommentToDelete: function(comment) {
		var entry = this.commentEntriesHash[comment.uri];
		if (entry) {
			entry.markToDelete();
		}
	},
	
	_unmarkCommentToDelete: function(comment) {
		var entry = this.commentEntriesHash[comment.uri];
		if (entry) {
			entry.unmarkToDelete();
		}
	},
	
	refreshComment: function(comment) {
		var entry = this.commentEntriesHash[comment.uri];
		if (entry) {
			entry.refresh();
			if (entry.isVisible) {
				entry.resize();
			}
		}
	},
	
	_filterComments: function(filtered, unfiltered) {
		this.filterComments(filtered, unfiltered);
		this.resize();
	},
	
	filterComments: function(filtered, unfiltered) {
		var i, entry;
		if (filtered) {
			for (i = 0; i < filtered.length; ++i) {
				entry = this.commentEntriesHash[filtered[i].uri];
				if (entry) {
					entry.setVisibility(false);
				}
			}
		}
		if (unfiltered) {
			for (i = 0; i < unfiltered.length; ++i) {
				entry = this.commentEntriesHash[unfiltered[i].uri];
				if (entry) {
					entry.setVisibility(true);
				}
			}
		}
	},
	
	sortEntries: function(entries, ascending, func) {
		function comparator(entry1, entry2) {
			var a = func(entry1);
			var b = func(entry2);
			if (ascending) {
				return (a < b) ? -1 : ((a > b) ? 1 : 0);
			} else {
				return (a > b) ? -1 : ((a < b) ? 1 : 0);
			}
		}
		entries.sort(comparator);
	},
	
	createEntries: function() {
		var key, entry;
		for (key in this.comments) {
			var _comment = this.comments[key];
			entry = this.createCommentEntry(_comment);
			if (this.commentingWidget.isMarkedToDelete(_comment)) {
				entry.markToDelete();
			}

			this.commentEntries.push(entry);
			this.commentEntriesHash[_comment.uri] = entry;
		}
		
		// setup relationships
		for (key in this.commentEntries) {
			entry = this.commentEntries[key];
			var parent = this.commentEntriesHash[entry.comment.inReplyTo];
			if (parent) {
				entry.parent = parent;
				parent.addReply(entry);
			} else {
				this.commentEntryRoots.push(entry);
			}
		}
	},
	
	createCommentEntry: function(_comment) {
		var entry = new rmps.ui.commenting.CommentEntry({
				group: this,
				comment: _comment,
				showResourceLabel: this.commentingWidget.showResourceLabel,
				isResourceLabelLink: this.commentingWidget.isResourceLabelLink
			});
		return entry;
	},
	
	prepareEntries: function() {
		// To be implmented by clients.
		// All entries have been created and parent child relationships finalized.
	},
	
	handleSelection: function(e, entry) {
		if (entry._actionBarWidget) {
			var box = dojo.position(entry._actionBarWidget.domNode);
			var x = e.clientX - box.x;
			var y = e.clientY - box.y;
			if (x >= 0 && x <= box.w && y >=0 && y <= box.h) {
				// let the actionbar catch the click
				return;
			}
		}
		dojo.stopEvent(e);
		this._handlingSelection = true;
		
		if (e && e.shiftKey) {
			var firstEntry = this._lastSelectedEntry && this._lastSelectedEntry.isVisible
					? this._lastSelectedEntry : null;
			
			var entries;
			if (firstEntry == entry) {
				entries = [entry];
			} else {
				entries = this.getInclusiveEntries(firstEntry, entry);
			}
			
			if (entries) {
				var toSelect = [];
				for (var i = 0; i < entries.length; ++i) {
					toSelect.push(entries[i].comment);
				}
				if (e.ctrlKey) {
					this.commentingWidget.appendCommentSelection(toSelect);
				} else {
					this.commentingWidget.selectComments(toSelect);
				}
			}
			
		} else {
			if (e && e.ctrlKey) {
				this._toggleSelection(entry);
			} else if (this.commentingWidget.hasSelectedComments() || !entry.isSelected()){
				this.commentingWidget.selectComments(entry.comment);
			} else {
				this.commentingWidget.deselectComment(entry.comment);
			}
			this._lastSelectedEntry = entry;
		}
		
		this._handlingSelection = false;
		
		dijit.scrollIntoView(entry.domNode);
	},
	
	getInclusiveEntries: function(/*Object?*/firstEntry, secondEntry) {
		// default assumes this.commentEntries is ordered properly
		firstEntry = firstEntry || this.commentEntries[0];
		if (!firstEntry) {
			return;
		}
		
		var startIdx = dojo.indexOf(this.commentEntries, firstEntry);
		var endIdx = dojo.indexOf(this.commentEntries, secondEntry);
		
		if (startIdx > endIdx) {
			var temp = startIdx;
			startIdx = endIdx;
			endIdx = temp;
		}
		
		return this.commentEntries.slice(startIdx, endIdx + 1);
	},
	
	_toggleSelection: function(entry) {
		if (entry.isSelected()) {
			this.commentingWidget.deselectComment(entry.comment);
		} else {
			this.commentingWidget.appendCommentSelection(entry.comment);
		}
	}

});

})();