/*******************************************************************************
 * Licensed Materials - Property of IBM
 *
 * com.ibm.xtools.rmps.web.core
 *
 * Copyright IBM Corporation 2009, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.commenting.Comment");

dojo.require("rmps.core.resource.Resource");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.core.user.UserClient");
dojo.require("rmps.core.commenting.range.Factory");
dojo.require("rmps.core.Enum");

dojo.require("jazz.util.html");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.core.commenting", "CommentMessages");

var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;
var UserClient = rmps.core.user.UserClient;
var messages = dojo.i18n.getLocalization("rmps.core.commenting", "CommentMessages");

//rmps.core.commenting.Priority = {
//  HIGH: { value: "HIGH", name: messages.priority_high },
//  MEDIUM: { value: "MEDIUM", name: messages.priority_medium },
//  LOW: { value: "LOW", name: messages.priority_low },
//
//  valueOf: function(value) {
//      for (var key in this) {
//          var property = this[key];
//          if (typeof property == "object" && property.value == value) {
//              return property;
//          }
//      }
//      return null;
//  }
//};

rmps.core.commenting.BodyType = new rmps.core.Enum({
	PLAIN_TEXT: new rmps.core.EnumLiteral({ value: "text/plain",    name: messages.bodyType_plainText }),
	SKETCH:     new rmps.core.EnumLiteral({ value: "image/svg+xml", name: messages.bodyType_sketch })
});

// var Priority = rmps.core.commenting.Priority;

dojo.declare("rmps.core.commenting.Comment", rmps.core.resource.Resource, {

	// serialized properties

	/**
	 * Resource identifier.
	 */
	resourceUri: null,

	/**
	 * The URI of the user that created this comment.
	 */
	creator: null,

	/**
	 * The URI of the user that last edited this comment.
	 */
	contributor: null,

	/**
	 * The date and time that this comment was created.
	 * The expected format is ISO8601.
	 */
	created: null,

	/**
	 * The date and time that this comment was last updated.
	 * The expected format is ISO8601.
	 */
	updated: null,

//  /**
//   * An array of user URIs that this comment is directed towards.
//   */
//  directedTo: null, // []

	/**
	 * The resource URI of another comment entry
	 * that this comment is a reply to.
	 */
	inReplyTo: null,

//  /**
//   * The creator's defined priority for this
//   * comment. There are only three allowed values
//   * (LOW, MEDIUM, HIGH).
//   */
//  priority: null, // default Priority.MEDIUM

	/**
	 * Indicates that the creator's concerns have been addressed.
	 */
	resolved: false,

	/**
	 * Revision URI.
	 */
	revision: null,

	/**
	 * A simple one line summary of the comment.
	 */
	subject: null,

	/**
	 * An array of body objects. Each object must support the following
	 * three properties:
	 * type - Content type of the body.
	 * href - The URI of the body contents if not serialized in the comment.
	 * contents - If the body contents should be serialized in
	 *            the comment, then this method must return a string
	 *            representation of this body.
	 */
	bodies: null, // []

	/**
	 * An array of range objects whose #toXMLString function returns the XML
	 * representation of the range.
	 */
	range: null, // []

	// END serialized properties

	// properties
	contentType: "application/rdf+xml;type=entry",

	// private properties
	_resourceInfo: null,
	_creator: null,
	_contributor: null,

	constructor: function() {
//      this.directedTo = [];
		this.bodies = [];
		this.range = [];
//      this.priority = Priority.MEDIUM.value;
	},

	/**
	 * Return a deferred for the info of the resource referenced by this comment.
	 *
	 * @return a Deferred for the info of the resource referenced by this comment
	 * @type dojo.Deferred
	 */
	getResourceInfo: function(properties, contextURI) {
		var deferred;
		if (this._resourceInfo) {
			deferred = new dojo.Deferred();
			setTimeout(dojo.hitch(this, function() {
				deferred.callback(this._resourceInfo);
			}), 0);
			return deferred;
		}
		deferred = ResourceInfoClient.getResourceInfo(this.resourceUri, properties, contextURI);
		deferred.addCallback(dojo.hitch(this, function(resourceInfo) {
			this._resourceInfo = resourceInfo[0];
			return this._resourceInfo;
		}));
		return deferred;
	},

	/**
	 * Return a deferred for the creator of this comment.
	 *
	 * @return a Deferred for creator of this comment
	 * @type dojo.Deferred
	 */
	getCreator: function() {
		var deferred;
		if (this._creator && this._creator.uri == this.creator) {
			deferred = new dojo.Deferred();
			setTimeout(dojo.hitch(this, function() {
				deferred.callback(this._creator);
			}), 0);
			return deferred;
		}
		deferred = UserClient.getUser(this.creator);
		deferred.addCallback(dojo.hitch(this, function(user) {
			this._creator = user;
		}));
		return deferred;
	},

	/**
	 * Return a deferred for the contributor of this comment.
	 *
	 * @return a Deferred for contributor of this comment
	 * @type dojo.Deferred
	 */
	getContributor: function() {
		var deferred;
		if (this._contributor && this._contributor.uri == this.contributor) {
			deferred = new dojo.Deferred();
			setTimeout(dojo.hitch(this, function() {
				deferred.callback(this._contributor);
			}), 0);
			return deferred;
		}
		deferred = UserClient.getUser(this.contributor);
		deferred.addCallback(dojo.hitch(this, function(user) {
			this._contributor = user;
		}));
		return deferred;
	},

	/**
	 * Return the XML representation of the comment.
	 *
	 * @return the XML representation of the comment
	 * @type String
	 */
	toXMLString: function() {
		var i;
		var commentXML = '<?xml version="1.0" encoding="UTF-8"?>\n'
					   + '<rdf:RDF\n'
					   + '  xmlns="http://jazz.net/xmlns/rmps/comments/1.0/"\n'
					   + '  xmlns:dc="http://purl.org/dc/terms/"\n'
					   + '  xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"\n'
					   + '  xmlns:rmps="https://www.ibm.com/rmps">\n'
					   + ' <comment rdf:about="">\n'
					   + '  <dc:title>' + jazz.util.html.escape(this.subject) + '</dc:title>\n'
					   + '  <target-resource rdf:resource="' + jazz.util.html.escape(this.resourceUri) + '"/>\n'
					   + (this.inReplyTo ? ('  <in-reply-to rdf:resource="'     + jazz.util.html.escape(this.inReplyTo) + '"/>\n') : '')
					   + (this.revision  ? ('  <target-revision rdf:resource="' + jazz.util.html.escape(this.revision)  + '"/>\n') : '');

		if (this.range.length > 0) {
			commentXML += '  <target-range rdf:parseType="Literal">\n';

			for (i = 0; i < this.range.length; ++i) {
				commentXML += this.range[i].toXMLString();
			}

			commentXML += '  </target-range>\n';
		}

		for (i = 0; i < this.bodies.length; ++i) {
			var body = this.bodies[i];
			var escapedContents = body.contents
									  .replace(/&/gm, "&amp;")
									  .replace(/</gm, "&lt;")
									  .replace(/>/gm, "&gt;")
									  .replace(/"/gm, "&quot;");
			commentXML += '  <content rdf:parseType="Resource">\n'
						+ (body.type ? ('   <dc:format>' + jazz.util.html.escape(body.type) + '</dc:format>\n') : '')
						+ '   <body' + (body.href ? (' rdf:resource="' + jazz.util.html.escape(body.href) + '"/>\n') : ('>' + escapedContents + '</body>\n'))
						+ '  </content>\n';
		}

		commentXML += ' </comment>\n'
					+ '</rdf:RDF>\n';

		return commentXML;
	},

	parseEntry: function(entry, ioArgs) {
		this.eTag = entry["rmps:etag"][0].value;

		var links = entry.link;

		for (var j = 0; j < links.length; j++) {
			var link = links[j];
			if (link.rel == "self") {
				this.uri = links[j].href;
			}
		}

		var commentNode = entry.content[0]
								["rdf:RDF"][0]
								["rmpsComments:comment"][0];

		this.parse(commentNode, null);
	},

	parse: function(commentNode, ioArgs) {
		var i;
		if (ioArgs) {
			var xhr = ioArgs.xhr;
			if (xhr.getResponseHeader("Last-Modified")) {
				this.lastModified = new Date(xhr.getResponseHeader("Last-Modified"));
			}
			if (xhr.getResponseHeader("ETag")) {
				this.eTag = xhr.getResponseHeader("ETag");
			}
			if (xhr.getResponseHeader("Location")) {
				// TODO revision
//              this.revisionUri = xhr.getResponseHeader("Content-Location");
//              this.uri = this.revisionUri.substring(0, this.revisionUri.indexOf("?"));
				this.uri = xhr.getResponseHeader("Location");
			}
		}

		// resource
		this.resourceUri = commentNode["rmpsComments:target-resource"][0]["rdf:resource"];

		// creator
		this.creator = commentNode["dcterms:creator"][0]["rdf:resource"];

		// created
		this.created = dojo.date.stamp.fromISOString(commentNode["dcterms:created"][0].value);

		// contributor
		this.contributor = commentNode["dcterms:contributor"][0]["rdf:resource"];

		// updated
		this.updated = dojo.date.stamp.fromISOString(commentNode["dcterms:modified"][0].value);

		// range
		if (commentNode["rmpsComments:target-range"]) {
			var range = commentNode["rmpsComments:target-range"][0];
			this.range = [];
			for (var key in range) {
				var rangeElements = range[key];
				if (dojo.isArray(rangeElements)) {
					for (i = 0; i < rangeElements.length; ++i) {
						var rangeObj = rmps.core.commenting.range.Factory.create(rangeElements[i]);
						if (rangeObj) {
							this.range.push(rangeObj);
						}
					}
				}
			}
		}

		// inReplyTo
		if (commentNode["rmpsComments:in-reply-to"]) {
			this.inReplyTo = commentNode["rmpsComments:in-reply-to"][0]["rdf:resource"];
		}

		// revision
		if (commentNode["rmpsComments:target-revision"]) {
			this.revision = commentNode["rmpsComments:target-revision"][0]["rdf:resource"];
		}

		// subject
		if (commentNode["dcterms:title"]) {
			this.subject = commentNode["dcterms:title"][0].value;
		}

		// body
		this.bodies = [];
		var bodies = commentNode["rmpsComments:content"];
		for (i = 0; i < bodies.length; i++) {
			var body = {};
			body.href = bodies[i]["rmpsComments:body"]["rdf:resource"];
			if (bodies[i]["dcterms:format"]) {
				body.type = bodies[i]["dcterms:format"][0].value;
			}
			body.contents = bodies[i]["rmpsComments:body"][0].value;
			this.bodies.push(body);
		}
	},

	addBody: function(body) {
		var index = -1;
		for (var i = 0; i < this.bodies.length; i++) {
			if (this.bodies[i].type == body.type) {
				index = i;
				break;
			}
		}

		if (index == -1) {
			this.bodies.push(body);
		} else {
			this.bodies.splice(index, 1, body);
		}
	},

	getBody: function(type) {
		for (var i = 0; i < this.bodies.length; i++) {
			if (this.bodies[i].type == type) {
				return this.bodies[i];
			}
		}
		return null;
	}
});

})();