/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.commenting.CommentClient");

dojo.require("rmps.core.RootServices");
dojo.require("rmps.core.commenting.Comment");
dojo.require("rmps.core.util.ContextUtils");
dojo.require("dojo.date.stamp");

dojo.require("jazz.app.context");
dojo.require("jazz.client.xhr");

dojo.require("rmps.app.ui.internal.util.UrlUtil");

var COMMENTS_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "comments");
var COMMENTS_SET_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "comments") + "/set";
var UrlUtil = rmps.app.ui.internal.util.UrlUtil;
var contextUtils = rmps.core.util.ContextUtils;

rmps.core.commenting.CommentClient = {
	
	getComment: function(commentUri) {
		var args = {
			url: commentUri,
			headers: {
				"Accept": "application/json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				var comment = new rmps.core.commenting.Comment();
				comment.parse(response, ioArgs);
				return comment;
			}
		};
		return jazz.client.xhrGet(args);
	},
	
	getCommentsFor: function(resourceUri, context, page, pageSize, groupByDiscussion, pageToken) {
		var queryParams = {
			"resourceUri": resourceUri,
			"pageSize": -1
		};
		
		if (context) {
			queryParams.commentContext = context;
		}
		
		if (pageSize) {
			queryParams.pageSize = pageSize;
		}
		if (page != undefined && page != null) {
			queryParams.page = page;
		}
		if (groupByDiscussion) {
			queryParams.groupByDiscussion = groupByDiscussion;
		}
		if (pageToken) {
			queryParams.pageToken = pageToken;
		}
		
		var args = {
			url: COMMENTS_PATH + "?"
				+ dojo.objectToQuery(queryParams),
			headers: {
				"X-ibm-rmps-internal": "true",
				"Accept": "application/atom+json"
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				var result = [];
				var entries = response.entry;
				
				result.totalResults = parseInt(response["opensearch:totalResults"][0].value);
				
				if (entries && entries.length > 0) {
					for (var i = 0; i < entries.length; i++) {
						var comment = new rmps.core.commenting.Comment();
						comment.parseEntry(entries[i], ioArgs);
						result.push(comment);
					}
				}
				
				var links = response.link;
				if (links && links.length > 0) {
					for (var l = 0; l < links.length; l++) {
						var link = links[l];
						if (link.rel == "self") {
							result.pageToken = UrlUtil.getParamater(link.href, "pageToken");
							break;
						}
					}
				}
				
				return result;
			}
		};
		return jazz.client.xhrGet(args);
	},

	getCommentsForSet: function(resourceSet, context, page, pageSize, groupByDiscussion, pageToken) {
		var queryParams = {
			"pageSize": -1
		};

		if (context) {
			queryParams.commentContext = context;
		}

		if (pageSize) {
			queryParams.pageSize = pageSize;
		}
		if (page != undefined && page != null) {
			queryParams.page = page;
		}
		if (groupByDiscussion) {
			queryParams.groupByDiscussion = groupByDiscussion;
		}
		if (pageToken) {
			queryParams.pageToken = pageToken;
		}
		
		var resourceSetXML = this._generateCommentSetXML(resourceSet);
		
		var args = {
			url: COMMENTS_SET_PATH + "?"
				+ dojo.objectToQuery(queryParams),
			headers: {
				"Content-Type": "text/xml",
				"X-ibm-rmps-internal": "true",
				"Accept": "application/atom+json"
			},
			handleAs: "json",
			hasBody: true,
			postData: resourceSetXML,
			load: function(response, ioArgs) {
				var result = [];
				var entries = response.entry;
				
				result.totalResults = parseInt(response["opensearch:totalResults"][0].value);
				
				if (entries && entries.length > 0) {
					for (var i = 0; i < entries.length; i++) {
						var comment = new rmps.core.commenting.Comment();
						comment.parseEntry(entries[i], ioArgs);
						result.push(comment);
					}
				}
				
				var links = response.link;
				if (links && links.length > 0) {
					for (var l = 0; l < links.length; l++) {
						var link = links[l];
						if (link.rel == "self") {
							result.pageToken = UrlUtil.getParamater(link.href, "pageToken");
							break;
						}
					}
				}
				
				return result;
			}
		};
		return jazz.client.xhrPost(args);
	},
	
	_generateCommentSetXML: function (resourceSet) {				
		var resourcesContent = "";
		for ( var i = 0; i < resourceSet.length; i++) {
			var resource = resourceSet[i];
			resourcesContent += '<rdfs:member rdf:resource="' + resource.uri + '"/>\n';		
		}
		
		var commentSetXML = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
			+ "<ResourceSet"
			+ ' xmlns="http://jazz.net/xmlns/rmps/1.0/"'
			+ ' xmlns:rdfs="http://www.w3.org/2000/01/rdf-schema#" '
			+ ' xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#">\n'
			+ resourcesContent
			+ "</ResourceSet>";

		return commentSetXML;
	},

	createComment: function(comment) {
		comment.created = comment.updated = new Date();
		// Baseline
		var baseline = jazz.app.context.get().baseline;
		if (baseline) {
			comment.revision = baseline;
		}
		var args = {
			headers: {
				"Content-Type": comment.contentType,
				"Accept": "application/atom+json",
				"X-ibm-rmps-internal": "true"
			},
			url: COMMENTS_PATH,
			handleAs: "json",
			hasBody: true,
			postData: comment.toXMLString(),
			load: function(response, ioArgs) {
				comment.created = comment.updated = comment.lastModified =
						new Date(ioArgs.xhr.getResponseHeader("Last-Modified"));
				comment.eTag = ioArgs.xhr.getResponseHeader("ETag");
				comment.uri = ioArgs.xhr.getResponseHeader("Location");
				return comment;
			}
		};
		return jazz.client.xhrPost(args);
	},
	
	updateComment: function(comment) {
		function getETag(ioArgs) {
			return ioArgs.xhr.getResponseHeader("ETag");
		}
		
		function doUpdateComment(eTag) {
			var args = {
				headers: {
					"Content-Type": comment.contentType,
					"If-Match": eTag,
					"X-ibm-rmps-internal": "true"
				},
				url: comment.uri,
				hasBody: true,
				putData: comment.toXMLString(),
				load: function(response, ioArgs) {
					comment.updated = comment.lastModified =
							new Date(ioArgs.xhr.getResponseHeader("Last-Modified"));
					comment.eTag = ioArgs.xhr.getResponseHeader("ETag");
					return comment;
				}
			};
			return jazz.client.xhrPut(args);
		}
		
		if (comment.eTag) {
			return doUpdateComment(comment.eTag);
		}
		
		var deferred = this.getCommentHeaders(comment);
		deferred.addCallback(getETag);
		deferred.addCallback(doUpdateComment);
		return deferred;
	},
	
	getCommentHeaders: function(comment) {
		var args = {
			url: comment.uri,
			handleAs: "text",
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			load: function(response, ioArgs) {
				return ioArgs;
			}
		};
		// TODO should be a head request
//		return jazz.client.xhr("HEAD", args, false);
		return jazz.client.xhrGet(args);
	},
	
	getRecentComments: function(projectId, createdAfter, page, pageSize, contextURI, pageToken) {
		var queryParams = {};
		if (createdAfter) {
			queryParams.createdAfter = createdAfter;
		}
		if (page != undefined && page != null) {
			queryParams.page = page;
		}
		if (pageSize) {
			queryParams.pageSize = pageSize;
		}
		if (pageToken) {
			queryParams.pageToken = pageToken;
		}
		if (contextURI) {
			queryParams.commentContext = contextURI;
		}
		queryParams.projectId = projectId;
		
		var args = {
			url: COMMENTS_PATH + "/recent?"
				+ dojo.objectToQuery(queryParams),
			headers: {
				"Accept": "application/atom+json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs)
			{
				var result = [];
				var entries = response.entry;
				
				result.pageToken = "";
				result.totalResults = parseInt(response["opensearch:totalResults"][0].value);
				
				if (entries && entries.length > 0) {
					for (var i = 0; i < entries.length; i++) {
						var comment = new rmps.core.commenting.Comment();
						comment.parseEntry(entries[i], ioArgs);
						result.push(comment);
					}
				}
				
				var links = response.link;
				if (links && links.length > 0) {
					for (var l = 0; l < links.length; l++) {
						var link = links[l];
						if (link.rel == "self") {
							result.pageToken = UrlUtil.getParamater(link.href, "pageToken");
							break;
						}
					}
				}
				
				return result;
			}
		};
		return jazz.client.xhrGet(args);
	},
	
	getMostCommented: function(projectId, pageSize, createdAfter, contextUri) {
		var queryParams = {};
		if (createdAfter) {
			queryParams.createdAfter = createdAfter;
		}
		if (pageSize) {
			queryParams.pageSize = pageSize;
		}
		if (contextUri != undefined && contextUri != null) {
			queryParams.commentContext = contextUri;
		} else {
			queryParams.commentContext = contextUtils.getCurrentContextURI();			
		}
		queryParams.projectId = projectId;
		
		var args = {
			url: COMMENTS_PATH + "/metric?"
				+ dojo.objectToQuery(queryParams),
			headers: {
				"Accept": "application/atom+json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs)
			{
				var results = {};
				var entries = response.entry;
								
				if (entries && entries.length > 0) {
					for (var i = 0; i < entries.length; i++) {
						var links = entries[i].link;
						var counter = entries[i]["opensearch:totalResults"];
						if (links.length == 1 && counter.length == 1) {
							results[links[0].href] = parseInt(counter[0].value);
						}
					}
				} else {
					results.length = 0;
				}
				return results;
			}
		};
		return jazz.client.xhrGet(args);
	},

	deleteComment: function(comment) {
		function getETag(ioArgs) {
			return ioArgs.xhr.getResponseHeader("ETag");
		}
		
		function doDeleteComment(eTag) {
			var args = {
				headers: {
					"If-Match": eTag,
					"X-ibm-rmps-internal": "true"
				},
				url: comment.uri,
				load: function(response, ioArgs) {
					return comment;
				}
			};
			return jazz.client.xhrDelete(args);
		}
		
		if (comment.eTag) {
			return doDeleteComment(comment.eTag);
		}
		
		var deferred = this.getCommentHeaders(comment);
		deferred.addCallback(getETag);
		deferred.addCallback(doUpdateComment);
		return deferred;
	}
};

})();