/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.commenting.CommentContributionRegistry");

dojo.require("jazz.core.loader");

// Cache of comment providers.
var providers = [];

rmps.ui.commenting.CommentContributionRegistry = {
	
	//
	// Registers the provider. The propvider must implement
	// the #createContribution() method which must either 
	// return a domNode if a contribution is to be made for the
	// given comment, otherwise return null.
	//
	registerProvider: function(className) {
        jazz.core.loader.load_async(className, function(){
            var classConstructor = dojo.getObject(className), 
                provider;
            if (classConstructor) {
                provider = new classConstructor();
                if (provider) {
                    if (provider.createContribution) {
                        providers.push(provider);
                    } else {
                        throw new Error("Comment contributions providers must implement the #createContribution() method.");
                    }
                }
            }
		});
	},
	
	//
	// Creates the contributions for the given comment. 
	//
	// @return an array of domNode contributions
	//
	createContributions: function(comment) {
		var results = [];
		for (var i = 0; i < providers.length; ++i) {
			var result = providers[i].createContribution(comment);
			if (result) {
				results.push(result);
			}
		}
		return results;
	}

};

})();