/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.commenting.CommentDetails");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.html");
dojo.require("dojox.validate.regexp");
dojo.require("rmps.ui.resource.ResourceIconLabel");

dojo.require("jazz.app.context");
dojo.require("jazz.util.html");
dojo.require("jazz.ui.HoverPopup");

dojo.require("rmps.ui.EllipsisLabel");
dojo.require("rmps.ui.EllipsisLabelContainer");
dojo.require("rmps.ui.user.UserLabel");
dojo.require("rmps.ui.commenting.CommentContributionRegistry");

dojo.require("rmps.core.baseline.BaselineClient");
dojo.require("rmps.core.util.BaselineUtil");

dojo.require("rmps.core.util.StringUtils");
dojo.require("rmps.core.util.DateUtils");

dojo.requireLocalization("rmps.ui.commenting", "CommentDetailsMessages");

var BaselineClient = rmps.core.baseline.BaselineClient;
var BaselineUtil = rmps.core.util.BaselineUtil;

var DateUtils = rmps.core.util.DateUtils;
var BODY_LENGTH = 300;
var BODY_LINES = 5;

var CommentContributionRegistry = rmps.ui.commenting.CommentContributionRegistry;
var URL_REGEX = dojox.validate.regexp.url({
	scheme: true,
	allowDecimal: false,
	allowHex: false,
	allowDottedHex: false,
	allowDottedOctal: false
});

dojo.declare("rmps.ui.commenting.CommentDetails", [dijit._Widget, dijit._Templated], {
	
	templateString: dojo.cache("rmps.ui", "commenting/templates/CommentDetails.html"),
	widgetsInTemplate: true,
	
	creatorDeferred: null,
	
	messages: dojo.i18n.getLocalization("rmps.ui.commenting", "CommentDetailsMessages"),
	
	// attach points
	_subject: null,
	_author: null,
	_createdTimestamp: null,
	_contributor: null,
	_updatedTimestamp: null,
	_bodyText: null,
	_bodyTextMore: null,
	_info: null,
	_authorLabel: null,
	_contributorLabel: null,
	_contributorContainer: null,
	_resource: null,
	_resourceIconLabel: null,
	
	// mixin properties
	comment: null,
	showResourceLabel: false,
	isResourceLabelLink: false, 
	
	// properties
	bodyLength: BODY_LENGTH, // -1 for full length
	bodyLines: BODY_LINES, // only applies if bodyLength != -1
	clipping: false,
	
	// private properties
	_commentPopup: null,
	_createdTimestampHandle: null,
	_updatedTimestampHandle: null,
	_popupContents: null,
	_authorInfo: null,
	_contributorInfo: null,
	
	maxWidth: 500,
	
	postCreate: function() {
		this.connect(this._authorLabel, "initUserInfo", function(userInfo) {
			this._authorInfo = userInfo;
			this._authorInfo.emailProperties.setSubject(
					dojo.string.substitute(this.messages.replySubject, [this.comment.subject]));
		});
		
		this._author.initTarget(this._authorLabel.linkNode);
		
		this.connect(this._contributorLabel, "initUserInfo", function(userInfo) {
			this._contributorInfo = userInfo;
			this._contributorInfo.emailProperties.setSubject(
					dojo.string.substitute(this.messages.replySubject, [this.comment.subject]));
		});
		
		this._contributor.initTarget(this._contributorLabel.linkNode);

		// resource info
		if (this.showResourceLabel) {
			this._resourceIconLabel= new rmps.ui.resource.ResourceIconLabel({
				isLink: true
			});
			this._resource.appendChild(this._resourceIconLabel.domNode);
			
			if (!this.isResourceLabelLink){
				this._resourceIconLabel.isLink = false;
			} 
			this._resourceIconLabel.linkHref = rmps.app.core.util.getViewResourceUri(this.comment.resourceUri);
			this.comment.getResourceInfo(['shortName', 'iconUri', 'longName']).addCallback(this, function(resourceInfo) {
				var info;
				if(dojo.isArray(resourceInfo)){
					info= resourceInfo[0];
				}else{
					info= resourceInfo;
				}
				this._resourceIconLabel.setResourceInfo(info);
			});
		} else {
			this._resource.style.display = "none";
		}
		
		// snapshot
        var context = jazz.app.context.get();
        var baseline = context.baseline ? context.baseline : null;
        var commentsBaseline = this.comment.revision;
        if (commentsBaseline !== baseline) {
            if (commentsBaseline) {
                // make snapshot link, and have it navigate to the comment's baseline                
                BaselineClient.getBaseline(commentsBaseline).addCallback( dojo.hitch( this, function(baseline) {
                    var tooltip = dojo.string.substitute(this.messages.snapShotActionTooltip, [baseline.name]);
                    this._createSnapshot(commentsBaseline, baseline.name, tooltip );
                }));                
            } else {
                // make snapshot link, and have it navigate to current (no baseline)
                this._createSnapshot(null, this.messages.noSnapshot, this.messages.latestActionTooltip);
            }
        }
	},
	
	refresh: function() {
		if (this.comment.created.getTime() == this.comment.updated.getTime()) {
			this._contributorContainer.domNode.style.display = "none";
		} else {
			this._contributorContainer.domNode.style.display = "";
			
			if (this._updatedTimestampHandle) {
				DateUtils.removeDateDiffUpdater(this._updatedTimestampHandle);
				delete this._updatedTimestampHandle;
			}
		
			var deferred = this.comment.getContributor();
			deferred.addCallback(this, this._updateContributor);
			
			// time stamp
			this._updatedTimestampHandle = DateUtils.addDateDiffUpdater(this.comment.updated, dojo.hitch(this, function(newDateDiff) {
				this._updatedTimestamp.setLabel(newDateDiff);
			}));
		}
		
		if (this._authorInfo) {
			this._authorInfo.emailProperties.setSubject("RE: " + this.comment.subject);
		}
		
		if (this._contributorInfo) {
			this._contributorInfo.emailProperties.setSubject("RE: " + this.comment.subject);
		}
		
		// subject
		this._subject.setLabel(this.comment.subject);		
		
		// body
		if (this._popupContents) {
			this._popupContents.destroyRecursive();
			delete this._popupContents;
		}
		if (this._commentPopup) {
			this._commentPopup.destroy();
			delete this._commentPopup;
		}
		
		var body = this.comment.getBody(rmps.core.commenting.BodyType.PLAIN_TEXT.value);
		if (body && body.contents && body.contents.length > 0) {
			var bodyObject = this.createBodyHtml(body.contents);
			this._bodyText.innerHTML = bodyObject.innerHTML;
			if (bodyObject.more) {
				this._bodyTextMore.style.display = "";
			} else {
				this._bodyTextMore.style.display = "none";
			}
		} else {
			dojo.html._emptyNode(this._bodyText);
			this._bodyTextMore.style.display = "none";
		}
		
		this.refreshInfo();
	},
	
	startup: function() {
		this.creatorDeferred = this.comment.getCreator();
		this.creatorDeferred.addCallback(this, this._updateAuthor);
		
		// time stamp
		this._createdTimestampHandle = DateUtils.addDateDiffUpdater(this.comment.created, dojo.hitch(this, function(newDateDiff) {
			this._createdTimestamp.setLabel(newDateDiff);
		}));
			
		this.inherited(arguments);
		if(this.bodyLength== -1){
			var size= this.calculateTextSize(this._bodyText, 
			this.comment.getBody(rmps.core.commenting.BodyType.PLAIN_TEXT.value).contents);
		
			/*The property max-width does not work properly for the lenghty words. So, we need to calculate the width for that text, and make sure
			 * that the node width would be similar to the default max width. Otherwise, the hover contnet will have a width larget than th exepcted */
			if(size.width> this.maxWidth){
				this.domNode.style.width= this.maxWidth+ "px";
			}	
		}
				
		this.refresh();
	},
	
    _createSnapshot: function( /*URI*/baseline, /*string*/name, /*string*/tooltip){
        var snapshotDiv, 
            snapshotIcon, 
            snapshotLabel, 
            setBaseline, 
            elements, 
            e;
        
        snapshotDiv = dojo.create("div", {"class": "rmps-ui-resource-resourceIconLabel"});
        snapshotDiv.appendChild(dojo.create("span", {style: "color: #666", 
                                                     innerHTML: this.messages.related}));
        
        snapshotIcon = dojo.create("a", {"class": "iconContainer", 
                                         style: "cursor: pointer", 
                                         title: tooltip}); 
        snapshotIcon.appendChild(dojo.create("img", {"class": "icon", 
                                                     src: dojo.moduleUrl("rmps.ui", "graphics/icons/change_snapshot_co.gif" )}));
        
        snapshotLabel = dojo.create("a", {"class": "label",
                                          style: "cursor: pointer",
                                          title: tooltip,
                                          innerHTML: " " + name });
                                             
        setBaseline = function(){BaselineUtil.setBaseline(baseline);};
        elements = [snapshotIcon, snapshotLabel];
        for(e in elements){
            elements[e].onclick = setBaseline;
            snapshotDiv.appendChild(elements[e]);
        }
        
        this._resource.appendChild(snapshotDiv);  
    },

	
	_updateAuthor: function(user) {
		this._authorLabel.setUser(user);
		this._author.setLabel(user.name);
		if (this._commentPopup && !this._commentPopup._isOpen) {
			this._commentPopup.destroy();
			delete this._commentPopup;
		}
		delete this.creatorDeferred;
	},
	
	_updateContributor: function(user) {
		this._contributorLabel.setUser(user);
		this._contributor.setLabel(user.name);
		if (this._commentPopup && !this._commentPopup._isOpen) {
			this._commentPopup.destroy();
			delete this._commentPopup;
		}
	},
	
	createCommentPopup: function(node) {
		return new jazz.ui.HoverPopup({
			around: node,
			createContents: dojo.hitch(this, function() {
				this._popupContents = this.createPopupContents();
				
				this._popupContents.startup();
				return this._popupContents.domNode;
			})
		});
	},
	
	/*
	 *TODO: Move this into a util class in case other places required using it
	 *Get the size for a given text.
	 * Paramaters:
	 * 			1) node: The node that contains this text. Required in order
	 * 				to make sure that width calculation is based on its style attributes (fonts, etc)
	 * 			2) text: The text to get its size calcualted 
	 */
	calculateTextSize: function(node, text){
		/*clone the node, so we make sure no side effects would happen */
		var cloned= dojo.clone(node);
		var span = document.createElement("span");
		span.innerHTML = text;
		span.style.visibility= "hidden";
		
		/*Append the hidden span to the current node so it would inherit its style properties */
		cloned.appendChild(span);
		
		/*Required to force client height/width to get intialized for the created span */    
		dijit.popup.moveOffScreen(span);
		
		var size= {
			width: parseInt(span.offsetWidth, 10),
			height: parseInt(span.offsetHeight, 10)
		};
		
		/*Remove and destroy the temp span after getting the text size information */
		span.parentNode.removeChild(span);
		dojo.destroy(cloned);
		
		return size;
	},
	
	createPopupContents: function(popupWidget) {
		var details = new rmps.ui.commenting.CommentDetails({
			comment: this.comment,
			bodyLength: -1
		});
		details.showFullLabels(true);
		return details;
	},
	
	showFullLabels: function(showFull) {
		this._subject.showFullLabel(showFull);
		this._author.showFullLabel(showFull);
		this._contributor.showFullLabel(showFull);
		this._createdTimestamp.showFullLabel(showFull);
		this._updatedTimestamp.showFullLabel(showFull);
	},
	
	destroy: function() {
		if (this._popupContents) {
			this._popupContents.destroyRecursive();
			delete this._popupContents;
		}
		if (this._commentPopup) {
			this._commentPopup.destroyRecursive();
			delete this._commentPopup;
		}
		if (this._createdTimestampHandle) {
			DateUtils.removeDateDiffUpdater(this._createdTimestampHandle);
			delete this._createdTimestampHandle;
		}
		if (this._updatedTimestampHandle) {
			DateUtils.removeDateDiffUpdater(this._updatedTimestampHandle);
			delete this._updatedTimestampHandle;
		}
		
		if(this.creatorDeferred){
			/*
			 * Stop the creator deferred in order to avoid any issues with refreshes, as
			 * the callback will be invoked while the widget is being destoyed 
			 */
			this.creatorDeferred.cancel();
			delete this.creatorDeferred;
		}
		this.destroyInfo();
		this.inherited(arguments);
	},
	
	resize: function() {
		var size = { w: this.domNode.offsetWidth };
		this._subject.resize(size);
		this._authorContainer.resize(size);
		this._contributorContainer.resize(size);
	},
	
	destroyInfo: function() {
		if (this._contributions) {
			for (var i = 0; i < this._contributions.length; ++i) {
				this._contributions[i].destroyRecursive();
			}
			delete this._contributions;
		}
	},
	
	refreshInfo: function() {
		dojo.html._emptyNode(this._info);
		this.destroyInfo();
		
		this._contributions = CommentContributionRegistry.createContributions(this.comment);
		for (var i = 0; i < this._contributions.length; ++i) {
			this._info.appendChild(this._contributions[i].domNode);
		}
	},
	
	createBodyHtml: function(text) {
		var skipWordWrap = dojo.isIE || dojo.isFF >= 3.1;
		var embeddedText = "";
		var word = "";
		var prevC = null;
		var lines = 1;
		var len = 0;
		var clipped = false;
		
		text = text.replace(/^(\s*?\n)*/, '').replace(/\s\s*$/, '');
		
		if (text.length > 0) {
			if (this.bodyLength === 0) {
				this._bodyText.style.display = "none";
			} else {
				this._bodyText.style.display = "";
			}
			for (var i = 0; i < text.length; i++) {
				var c = text.charAt(i);
				if (c == " " || c == "\n" || i == text.length - 1) {
					if (i == text.length -1) {
						word += c;
					}
					
					var escaped = jazz.util.html.escape(word);
					if (this.bodyLength != -1 && (len + escaped.length) > this.bodyLength && this.clipping == true ) {
						escaped = escaped.substr(0, this.bodyLength - len) + "...";
						clipped = true;
						len += 3;
					}
					len += escaped.length;
					
					if (!skipWordWrap && escaped.length > 20) {
						// add word breaks for long words
						escaped = this._breakUpWord(escaped);
					}
					
					if (word.match(URL_REGEX)) {
						embeddedText += '<a href="' + word + '" target="_blank">' + escaped + '</a>';
					} else {
						embeddedText += escaped;
					}
					
					if (clipped) {
						break;
					}
					
					if (c == " ") {
						if (c == prevC) {
							embeddedText += "&nbsp;";
						} else {
							embeddedText += c;
						}
						len++;
					} else if (c == "\n") {
						if (this.bodyLength != -1 && ++lines > this.bodyLines && this.clipping == true ){
							clipped = true;
							embeddedText += "...";
							len += 3;
							break;
						}
						embeddedText += "<br>";
					}
					word = "";
				} else {
					word += c;
				}
				prevC = c;
			}
		}
		return {innerHTML: embeddedText, more: clipped};
	},
	
	_breakUpWord: function(word) {
		var result = "";
		for (var j = 0; j < word.length; j++) {
			result += word.charAt(j) + "<wbr></wbr>";
		}
		return result;
	},
	
	// more events
	
	_onMoreClicked: function(e) {
		dojo.stopEvent(e);
		if(!this._commentPopup){
			this._commentPopup = this.createCommentPopup(this._bodyTextMore);			
		}
		if (!this._commentPopup._isOpen) {
			this._commentPopup.open();
		}
	}

});

})();