/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.commenting.CommentEntry");

dojo.require("rmps.ui.commenting.AbstractCommentEntry");
dojo.require("rmps.ui.commenting.CommentDetails");
dojo.require("rmps.ui.commenting.CommentEntryActionBar");
dojo.require("rmps.ui.util.ToolbarFocusUtil");

dojo.require("rmps.core.util.BaselineUtil");

var BaselineUtil = rmps.core.util.BaselineUtil;
var ToolbarFocusUtil = rmps.ui.util.ToolbarFocusUtil;

dojo.declare("rmps.ui.commenting.CommentEntry", rmps.ui.commenting.AbstractCommentEntry, {
	
	// mixin properties
	comment: null,
	showResourceLabel: false,
	isResourceLabelLink: false,
	
	// properties
	details: null,
	
	// private properties
	_actionBarWidget: null,
	
	postCreate: function() {
		this.details = new rmps.ui.commenting.CommentDetails({
			comment: this.comment,
			showResourceLabel: this.showResourceLabel,
			isResourceLabelLink: this.isResourceLabelLink
		});
		this.containerNode.appendChild(this.details.domNode);
				
		this._actionBarWidget = this.createActionBar();
		if (this._actionBarWidget) {
			this.actionBar.appendChild(this._actionBarWidget.domNode);
		}
		
		ToolbarFocusUtil.addToolbarFocusListeners( { 
            actionToolbar: this._actionBarWidget, 
            containerNode: this.domNode, 
            actionClassName: ".jazz-ui-SimpleToolbar-button"
        });
		
		this.inherited(arguments);
	},
	
	startup: function() {
		if (this._started) {
			return;
		}
		this.details.startup();
		this.inherited(arguments);
	},
	
	destroy: function() {
		if (this._actionBarWidget) {
			this._actionBarWidget.destroyRecursive();
			delete this._actionBarWidget;
		}
		this.inherited(arguments);
	},
	
	createActionBar: function() {
		var actionBar =  new rmps.ui.commenting.CommentEntryActionBar({
			entry: this,
			commentingWidget: this.group.commentingWidget
		});
		
		this.domNode.appendChild(actionBar.domNode);
		
		this.connect(actionBar, "deleteComment", function(entry) {
			this.group.commentingWidget.markCommentToDelete(entry.comment, true);
		});
		
		this.connect(actionBar, "restoreComment", function(entry) {
			this.group.commentingWidget.unmarkCommentToDelete(entry.comment, true, true);
		});
		
		this.connect(actionBar, "editComment", function(entry) {
			dojo.removeClass(this.pane, "entry-pane-hover");
			this.group.commentingWidget.editComment(entry.comment);
		});
		
		this.connect(actionBar, "replyToComment", function(entry) {
			dojo.removeClass(this.pane, "entry-pane-hover");
			this.group.commentingWidget.replyToComment(entry.comment);
		});
		
		return actionBar;
	},
	
	refresh: function() {
		this.details.refresh();
	},
	
	resize: function(size) {
		if (this.isVisible) {
			this.details.resize({ w: this.containerNode.offsetWidth });
		}
	},
	
	// deletion
	
	markToDelete: function() {
		dojo.addClass(this.domNode, "deleted");
		if (this._actionBarWidget.restoreAction) {
			this._actionBarWidget.showItem(this._actionBarWidget.restoreAction);
		}
		if (this._actionBarWidget.deleteAction) {
			this._actionBarWidget.hideItem(this._actionBarWidget.deleteAction);
		}
		if (this._actionBarWidget.editAction) {
			this._actionBarWidget.hideItem(this._actionBarWidget.editAction);
		}
		if (this._actionBarWidget.replyAction) {
			this._actionBarWidget.hideItem(this._actionBarWidget.replyAction);
		}
	},
	
	unmarkToDelete: function() {
		dojo.removeClass(this.domNode, "deleted deleteChild");
		if (this._actionBarWidget.restoreAction) {
			this._actionBarWidget.hideItem(this._actionBarWidget.restoreAction);
		}
		if (this._actionBarWidget.deleteAction) {
			this._actionBarWidget.showItem(this._actionBarWidget.deleteAction);
		}
		if (this._actionBarWidget.editAction) {
			this._actionBarWidget.showItem(this._actionBarWidget.editAction);
		}
		if (this._actionBarWidget.replyAction) {
			this._actionBarWidget.showItem(this._actionBarWidget.replyAction);
		}
	}
});

})();