/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.commenting.CommentForm");

dojo.require("rmps.core.util.StringUtils");
dojo.require("rmps.core.commenting.Comment");
dojo.require("com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog");
dojo.require("rmps.ui.user.UserSelectionBox");
dojo.require("rmps.ui.commenting.autocomplete.AutoComplete");
dojo.require("rmps.ui.commenting.autocompletematchers.ContributorProposalProvider");
dojo.require("rmpw.common.util.TextUtil");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dijit.form.Form");
dojo.require("dijit.form.Button");

dojo.requireLocalization("rmps.ui.commenting", "CommentFormMessages");

var BODY_TO_SUBJECT_LENGTH = 40;
var StringUtils = rmps.core.util.StringUtils;

var commenting = rmps.ui.commenting;
var AutoComplete= commenting.autocomplete.AutoComplete;
var autocompletematchers= commenting.autocompletematchers;
var ContributorProposalProvider= autocompletematchers.ContributorProposalProvider;

var TeamContributorSelectionDialog= com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog;

dojo.declare("rmps.ui.commenting.CommentForm", [dijit._Widget, dijit._Templated], {
	
	templateString: dojo.cache("rmps.ui", "commenting/templates/CommentForm.html"),
	widgetsInTemplate: true,
	
	messages: dojo.i18n.getLocalization("rmps.ui.commenting", "CommentFormMessages"),
	
	// attach points
	_subjectInput: null,
	_bodyInput: null,
	_cancelButton: null,
	_controlBar: null,
	_subjectContainer: null,
	
	// mixin properties
	commentingWidget: null,
	comment: null,
	showSubjectField: true,
	
	onPost: null,
	onSave: null,
	onCancel: null,
	
	// private properties
	_posted: false,
	_isDerivedSubject: false,
	_isValid: false,
	
	postCreate: function() {
		this.inherited(arguments);
		
		this._autoComplete= AutoComplete.create(new ContributorProposalProvider(this.commentingWidget.resourceInfo.uri));
		
		this._autoComplete.on('open', dojo.hitch(this, function(callbackArg) { //$NON-NLS-1$
			this._bookmarkSelectionStart();
			this._autoCompleteOn = true;
			this._autoComplete.setFilter(this._getText());
			this._autoComplete.setPosition(this._getCursorPosition());
		}));
		this._autoComplete.on('close', dojo.hitch(this, function(callbackArg) { //$NON-NLS-1$
			this._autoCompleteOn = false;
		}));
		this._autoComplete.on('complete', dojo.hitch(this, function(event) { //$NON-NLS-1$
			// insert new text
			this._insertUserText(event.text);
		}));
	},
	
	_insertUserText: function(sText) {   
		if ('selectionStart' in this._bodyInput) {
			var iStart = this._autoCompleteStart;   
			var iEnd = this._bodyInput.selectionStart;
			this._bodyInput.value = this._bodyInput.value.substring(0, iStart) + sText + this._bodyInput.value.substring(iEnd, this._bodyInput.value.length);   
			this._bodyInput.setSelectionRange(iStart + sText.length, iStart + sText.length);                                 
	    } else {   
	    	// IE
	    	this._bodyInput.focus();
	    	var oRange = document.selection.createRange();  
	    	range.setStart(this._autoCompleteStart);
	    	range.setEnd(document.selection.anchorOffset);
	    	oRange.text = sText;   
	    	oRange.collapse(true);   
	    	oRange.select();    
	    }   

		this._bodyInput.focus();   
	},
	
	_bookmarkSelectionStart: function() {
		var selectionStart;
		 if ('selectionStart' in this._bodyInput) {
			 selectionStart = this._bodyInput.selectionStart;
		 }
		 else {
			// IE
			 this._bodyInput.focus();
			 selectionStart = document.selection.anchorOffset;
		 }
		 
		 this._autoCompleteStart = selectionStart;
		 if (this._autoCompleteStart > 0) {
			 var prevText = this._bodyInput.value.substring(0, this._autoCompleteStart);
			 var lastAtIndex = prevText.lastIndexOf("@");
			 var lastSpaceIndex = prevText.lastIndexOf(" ");
			 
			 if (lastAtIndex != -1 && lastAtIndex > lastSpaceIndex) {
				 this._autoCompleteStart = lastAtIndex;
			 }
		 }
	},
	
	_getSelectionEnd: function() {
		if ('selectionEnd' in this._bodyInput) {
			return this._bodyInput.selectionEnd;
		}
		else {
			// IE
			this._bodyInput.focus();
			return document.selection.focusOffset;
		 }
	},
	
	_updateFilter: function() {
		if (this._autoCompleteOn) {
			if (!this._hasValidRange()) {
				this._autoComplete.setActive(false);
				return;
			}
			this._autoComplete.setFilter(this._getText());
		}
	},
	
	_getText: function() {
		if (this._autoCompleteOn) {
			var text = this._bodyInput.value.substring(this._autoCompleteStart, this._getSelectionEnd());
			return text.replace("\u200B", ""); //$NON-NLS-1$ //$NON-NLS-2$
		}
		return "";
	},
	
	_hasValidRange: function() {
		var text= this._getText();
		return text.match(new RegExp("\\s", "g")) == null; //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	// Local functions for readability of the previous code
	_appendPart: function (text, fakeClone, cssDefaultStyles, start, end){
        var span = document.createElement("span");
        span.style.cssText = cssDefaultStyles; //Force styles to prevent unexpected results
        span.textContent = text.substring(start, end);
        fakeClone.appendChild(span);
        return span;
    },
    
    // Computing offset position
    _getInputOffset: function (input){
        var body = document.body,
            win = document.defaultView,
            docElem = document.documentElement,
            box = document.createElement('div');
        box.style.paddingLeft = box.style.width = "1px";
        body.appendChild(box);
        var isBoxModel = box.offsetWidth == 2;
        body.removeChild(box);
        box = input.getBoundingClientRect();
        var clientTop  = docElem.clientTop  || body.clientTop  || 0,
            clientLeft = docElem.clientLeft || body.clientLeft || 0,
            scrollTop  = win.pageYOffset || isBoxModel && docElem.scrollTop  || body.scrollTop,
            scrollLeft = win.pageXOffset || isBoxModel && docElem.scrollLeft || body.scrollLeft;
        return {
            top : box.top  + scrollTop  - clientTop,
            left: box.left + scrollLeft - clientLeft};
    },
    
    _getInputCSS: function (input, prop, isnumber){
        var val = document.defaultView.getComputedStyle(input, null).getPropertyValue(prop);
        return isnumber ? parseFloat(val) : val;
    },
	
	_getCursorPosition: function() {
		var input = this._bodyInput;
        
		if (!input) return; // No (input) element found
		
	    // Styles to simulate a node in an input field
        var cssDefaultStyles = "white-space:pre;padding:0;margin:0;",
            listOfModifiers = ['direction', 'font-family', 'font-size', 'font-size-adjust', 'font-variant', 'font-weight', 'font-style', 'letter-spacing', 'line-height', 'text-align', 'text-indent', 'text-transform', 'word-wrap', 'word-spacing'];

        var fontStyle = {};
        for (var i=0; i<listOfModifiers.length; i++) {
            var property = listOfModifiers[i];
            var value = this._getInputCSS(input, property);
            cssDefaultStyles += property + ':' + value +';';
            
            if (property == "font-weight") {
            	fontStyle.weight = value;
            }
            else if (property == "font-size") {
            	fontStyle.size = value;
            }
            else if (property == "font-family") {
            	fontStyle.family = value;
            }
        }
        
        // find text height
        var textHeight = dojox.html.metrics.getTextBox("X", cssDefaultStyles).h;    
        
        if ('selectionStart' in input) {
            // Standard-compliant browsers
        	var selectionStart = input.selectionStart;
        	var selectionEnd = input.selectionEnd;
        	
        	// createTextRange is not supported, create a fake text range
            var offset = this._getInputOffset(input),
                topPos = offset.top,
                leftPos = offset.left,
                width = this._getInputCSS(input, 'width', true),
                height = this._getInputCSS(input, 'height', true);

            topPos += this._getInputCSS(input, 'padding-top', true);
            topPos += this._getInputCSS(input, 'border-top-width', true);
            leftPos += this._getInputCSS(input, 'padding-left', true);
            leftPos += this._getInputCSS(input, 'border-left-width', true);
            leftPos += 1; //Seems to be necessary

            var text = input.value,
                textLen = text.length,
                fakeClone = document.createElement("div");
            
            text = rmpw.common.util.TextUtil.getWrappedText(text, width, height, fontStyle);
            
            if(selectionStart > 0) this._appendPart(text, fakeClone, cssDefaultStyles, 0, selectionStart);
            var fakeRange = this._appendPart(text, fakeClone, cssDefaultStyles, selectionStart, selectionEnd);
            if(textLen > selectionEnd) this._appendPart(text, fakeClone, cssDefaultStyles, selectionEnd, textLen);

            // Styles to inherit the font styles of the element
            fakeClone.style.cssText = cssDefaultStyles;

            // Styles to position the text node at the desired position
            fakeClone.style.position = "absolute";
            fakeClone.style.top = topPos + "px";
            fakeClone.style.left = leftPos + "px";
            fakeClone.style.width = width + "px";
            fakeClone.style.height = height + "px";
            document.body.appendChild(fakeClone);
            var returnValue = fakeRange.getBoundingClientRect(); //Get rect

            fakeClone.parentNode.removeChild(fakeClone); //Remove temp
            return {x: returnValue.left, y: returnValue.top + textHeight};
        } else if (document.selection) {
            // IE
            input.focus();
            var sel = document.selection.createRange();
            var rect = sel.getBoundingClientRect();
            
            return {x: rect.x, y: rect.y + textHeight};
        }
	},
	
	resize: function(size) {
		if (size && size.h != null) {
			var bodyInputMb = dojo.marginBox(this._bodyInput);
			var mb = dojo.marginBox(this.domNode);
			var newHeight = Math.max(size.h - (mb.h - bodyInputMb.h), dojo.marginBox(this._subjectInput).h);
			dojo.marginBox(this._bodyInput, {h: newHeight });
		}
	},
	
	update: function(comment) {
		this._isDerivedSubject = false;
		dojo.removeClass(this._subjectInput, "derivedInput");
		this._enable(true);
		this._posted = false;
		
		this.comment = comment;
		
		var body = this.comment.getBody(rmps.core.commenting.BodyType.PLAIN_TEXT.value);
		
		if (this.showSubjectField) {
			this._subjectContainer.style.display = "";
		} else {
			this._subjectContainer.style.display = "none";
		}
		
		this._subjectInput.value = this.comment.subject ? this.comment.subject : "";
		this._bodyInput.value = (body && body.contents ? body.contents : "");
		this._validateInput();
	},
	
	setFocus: function() {
		if (!this.showSubjectField || (this.comment.subject && this.comment.subject.length > 0)) {
			this._bodyInput.focus();
		} else {
			this._subjectInput.focus();
		}
		
		if (this._autoComplete) {
			this._autoComplete.setActive(false);
		}
	},
	
	/**
	 * <p>
	 * Validates the commenting form, dispatching an error and returning <code>FALSE</code> if 
	 * there is a problem.  
	 * </p>
	 * 
	 * @return <code>TRUE</code> if the form is valid, <code>FALSE</code> otherwise 
	 */
	validate: function() {
		if(this._isValid)
			return true;
		
		// #_validateInput() is called for every keypress, therefore we cannot dispatch the error there.
		// Instead we must create the error when someone tries to do something requiring a valid input form.
		rmps.core.error.create(this.messages.validate_commentSubject, "", this.messages.validateBeforeSave).dispatch();
		return false;
	},
	
	_validateInput: function() {
		if (!this._posted) {
			if (dojo.trim(this._subjectInput.value).length == 0) {
				this._isValid = false;
			} else {
				this._isValid = true;
			}
		}
	},
	
	postFailed: function() {
		this._posted = false;
		this._cancelButton.disabled = false;
	},
	
	_enable: function(enabled) {
		this._subjectInput.disabled = !enabled;
		this._bodyInput.disabled = !enabled;
		this._cancelButton.disabled = !enabled;
	},
	
	_onSubjectKeyPress: function(e) {
		this._isDerivedSubject = this._subjectInput.value.length === 0;
		this._validateInput();
		
		if (this._autoComplete) {
			this._autoComplete.setActive(false);
		}
	},
	
	_onBodyKeyPress: function(e) {
		if (this._autoComplete.handleKey(e.charCode ? e.charCode : e.keyCode, e.ctrlKey, e.altKey, e.shiftKey)) {
			e.preventDefault();
			return;
		}
		
		this._computeDerivedSubject();
		this._validateInput();
	},
	
	_onBodyKeyUp: function(e) {
		this._updateFilter();
	},
	
	_onBodyClick: function(e) {
		this._autoComplete.setActive(false);
	},
	
	_destroyUserDialog: function() {
		if (this.userListWidget) {
			this.userListWidget.destroy(false);
		}
		this.userListWidget = null;
	},	
	
	_computeDerivedSubject: function() {
		if (this._subjectInput.value.length === 0 || this._isDerivedSubject) {
			var bodySubject = dojo.trim(this._bodyInput.value);
			var newLineIdx = bodySubject.indexOf("\n");
			if (newLineIdx != -1) {
				bodySubject = dojo.trim(bodySubject.substring(0, newLineIdx));
			}
			this._subjectInput.value = StringUtils.truncate(bodySubject, BODY_TO_SUBJECT_LENGTH, null, true, true);
			this._isDerivedSubject = true;
			dojo.addClass(this._subjectInput, "derivedInput");
		} else {
			this._isDerivedSubject = false;
			dojo.removeClass(this._subjectInput, "derivedInput");
		}
	},
	
	_onSubjectFocus: function(e) {
		if (this._isDerivedSubject) {
			dojo.removeClass(this._subjectInput, "derivedInput");
		}
		
		this._autoComplete.setActive(false);
	},
	
	_onSubjectBlur: function(e) {
		this._computeDerivedSubject();
		this._validateInput();
	},
	
	// post
	_onPost: function(e) {
		this.post();
	},
	
	post: function() {
		if (this._posted || !this._isValid) {
			return null;
		}
		this._posted = true;
		this._enable(false);

		this.comment.subject = dojo.trim(this._subjectInput.value);
		this.comment.addBody({
			type: rmps.core.commenting.BodyType.PLAIN_TEXT.value,
			contents: this._bodyInput.value.replace(/^(\s*?\n)*/, '').replace(/\s\s*$/, '')
		});
		if (this.onPost) {
			return this.onPost(this.comment);
		}
		return null;
	},
	
	// cancel
	_onCancel: function(e) {
		this.cancel();
	},
		
	cancel: function() {
		if (this.onCancel) {
			this.onCancel(this.comment);
		}
	},
	
	_onSave: function(e) {
		if (this.onSave) {
			this.onSave(this.comment);
		}		
	}
	
});

})();