/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.commenting.CommentingController");

dojo.require("rmps.core.commenting.Comment");
dojo.require("rmps.core.commenting.CommentClient");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dojo.DeferredList");
dojo.require("jazz.app.context");


dojo.requireLocalization("rmps.ui.commenting", "CommentingWidgetMessages");

var messages = dojo.i18n.getLocalization("rmps.ui.commenting", "CommentingWidgetMessages");
var CommentClient = rmps.core.commenting.CommentClient;

dojo.declare("rmps.ui.commenting.CommentingController", null,  {
	
	// properties
	resourceInfo: null,
	
	// private properties
	_view: null,
	
	_getCommentsDeferred: null,
	_createCommentDeferred: null,
	_updateCommentDeferred: null,
	
	constructor: function(view) {
		this._view = view;
	},
	
	destroy: function() {
		if (this._getCommentsDeferred) {
			this._getCommentsDeferred.cancel();
			delete this._getCommentsDeferred;
		}
		if (this._createCommentDeferred) {
			this._createCommentDeferred.cancel();
			delete this._createCommentDeferred;
		}
		if (this._updateCommentDeferred) {
			this._updateCommentDeferred.cancel();
			delete this._updateCommentDeferred;
		}
		if (this._deleteCommentDeferred) {
			this._deleteCommentDeferred.cancel();
			delete this._deleteCommentDeferred;
		}
	},
	
	_getCommentsErrback: function(error) {
		if( error.dojoType !== "cancel" ){

			error = rmps.core.error.createError(error).getGenericError();
			
			error.message = messages.error_load_comments;
			error.detailedMessage = messages['error_load_comments.explanation'];
			error.action = messages['error_load_comments.useraction'];
			this._view.onError(error);
			error.dispatch();
			return error;
		}
	},
	
	getComments: function() {
		if (this._getCommentsDeferred) {
			this._getCommentsDeferred.cancel();
		}
		
		var deferred = null;
		var page = this._view.getActivePage();
		var pageSize = this._view.getPageSize();
		var groupByDiscussion = this._view.isGroupedByDiscussion();
		var pageToken = this._view.getPageToken();
		var resourceInfo = this._view.resourceInfo;
		var context = jazz.app.context.get().baseline;
		if (resourceInfo instanceof Array) {
			// Several resources
			deferred = CommentClient.getCommentsForSet(resourceInfo, context, page, pageSize, groupByDiscussion, pageToken);
		} else if(resourceInfo.uri) { // null if creating new resource
			// One resource
			deferred = CommentClient.getCommentsFor(resourceInfo.uri, context, page, pageSize, groupByDiscussion, pageToken);
		}
		if(deferred) {
			deferred.addCallback(this._view, this._view._updateCache);
			deferred.addErrback(this, this._getCommentsErrback);
				
			this._getCommentsDeferred = deferred;
		}
		return deferred;			
	},
	
	createComment: function(comment) {
		var deferred = CommentClient.createComment(comment);
		deferred.addCallback(this._view, this._view._addComment);
		deferred.addErrback(this, function(error) {
			if (error.dojoType !== "cancel") {
				error = this._createError(error, deferred.ioArgs);
				if (error.status == 403) {
					error.message = messages.error_create_permission;
				} else {
					error.message = dojo.string.substitute(messages.error_create_fail, [comment.subject]);
					error.detailedMessage = dojo.string.substitute( messages['error_create_fail.explanation'], [error.status]);
					error.action = messages['error_create_fail.useraction'];
				}
				this._view._saveCommentFailed(error);
				error.dispatch();
				return error;
			}
		});
		
		this._createCommentDeferred = deferred;
		return deferred;
	},
	
	updateComment: function(comment, tempComment) {
		function success(_comment) {
			dojo.mixin(comment, _comment);
			return comment;
		}
		
		var deferred = CommentClient.updateComment(tempComment);
		deferred.addCallback(success);
		deferred.addCallback(this._view, this._view._commentEdited);
		deferred.addErrback(this, function(error) {
			if (error.dojoType !== "cancel") {
				error = this._createError(error, deferred.ioArgs);
				if (error.status == 403) {
					error.message = dojo.string.substitute(messages.error_edit_permission, [comment.subject]);
					error.detailedMessage = dojo.string.substitute(messages.error_edit_fail, [comment.subject]);
				} else {
					error.message = dojo.string.substitute(messages.error_edit_fail, [comment.subject]);
					error.detailedMessage = messages.error_precondition_failed;
				}
				this._view._saveCommentFailed(error);
				error.dispatch();
				return error;
			}
		});
		
		this._updateCommentDeferred = deferred;
		return deferred;
	},
	
	deleteComment: function(comment) {
		var deferred = CommentClient.deleteComment(comment);
		
		deferred.addCallback(this._view, this._view._commentDeleted);
		deferred.addErrback(this, function(error) {
			if (error.dojoType !== "cancel") {
				error = this._createError(error, deferred.ioArgs);
				if (error.status == 410) {
					// handle 410 as a delete
					this._view._commentDeleted(comment);
					return;
				}
				if (error.status == 403) {
					error.message = dojo.string.substitute(messages.error_delete_permission, [comment.subject]);
				} else {
					error.message = dojo.string.substitute(messages.error_delete_fail, [comment.subject]);
					error.detailedMessage = dojo.string.substitute(messages['error_delete_fail.explanation'], [error.status]);
					error.action = messages['error_delete_fail.useraction'];
				}
				if (error.status !== 500) {
					delete error.details; // delete the unlocalized details if any
				}
				this._view._saveCommentFailed(error);
				console.log(error);
				error.dispatch();
				return error;
			}
		});
		
		this._deleteCommentDeferred = deferred;
		return deferred;
	},
	
	_createError: function(error, ioArgs){
		if (!dojo._isDocumentOk(ioArgs.xhr)) {
			return rmps.core.error.createXhrError(ioArgs).getGenericError();
		} else {
			return rmps.core.error.createError(error).getGenericError();
		}
	}
});

})();