/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.commenting.CommentingToolbarMenu");

dojo.require("rmps.ui.commenting.utils");
dojo.require("rmps.core.commenting.Comment");
dojo.require("rmps.core.util.StringUtils");
dojo.require("rmps.ui.commenting.utils");
dojo.require("rmps.core.TaskMonitor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Toolbar");
dojo.require("dijit.form.Button");
dojo.require("dijit.Menu");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.requireLocalization("rmps.ui.commenting", "CommentingToolbarMenuMessages");

var GroupBy = rmps.ui.commenting.utils.GroupBy;
var StringUtils = rmps.core.util.StringUtils;

var TRUNCATE_LENGTH = 20;

dojo.declare("rmps.ui.commenting.CommentingToolbarMenu", [dijit._Widget, dijit._Templated], {
	
	templateString: dojo.cache("rmps.ui", "commenting/templates/CommentingToolbarMenu.html"),
	widgetsInTemplate: true,
	
	messages: dojo.i18n.getLocalization("rmps.ui.commenting", "CommentingToolbarMenuMessages"),
	
	// attach points
	_optionsDrowpdown: null,
	_optionsMenu: null,
	_createNewButton: null,
//	_createNewMenuItem: null,
//	_createForMenuItem: null,
	_sortDrowpdown: null,
	_sortMenu: null,
	_noGroupingMenuItem: null,
	_groupByDiscussionMenuItem: null,
	_linkWithEditorToggleButton: null,
	_expandAlButton: null,
	_collapseAllButton: null,
	_filterButton: null,
	_settingsButton: null,
	_sortButton: null,
	
	// mixin properties
	commentingWidget: null,
	
	// private properties
//	_createForMenuItemTitle: null,
	_groupingMenuItem: null,
	
	postCreate: function() {
		// set default grouping
		this._filterButton.set("disabled", true);
		this._settingsButton.set("disabled", false);
		this._sortButton.set("disabled", true);
		this._expandAllButton.set("disabled", true);
		this._collapseAllButton.set("disabled", true);
		this._applyGroupSettings(this.commentingWidget.getGrouping());
		
		this.connect(this.commentingWidget, "showFilters", this._onShowFilters);
		this.connect(this.commentingWidget, "hideFilters", this._onHideFilters);

		this.connect(this.commentingWidget, "showSettings", this._onShowSettings);
		this.connect(this.commentingWidget, "hideSettings", this._onHideSettings);

		dojo.removeClass(this.toolbar.domNode, "dijitToolbar");
//		this._createForMenuItemTitle = this._createForMenuItem.containerNode;
		
		this.connect(this.commentingWidget, "onFilterComments", this._filterComments);
		
		this.updateCanCreateState();
		
		if (!this.commentingWidget.externalSave) {
			this._refreshButton.domNode.style.display = "";
			this._saveButton.domNode.style.display = "";
			
			this.connect(this.commentingWidget, "onStartCreateComment", this._updateSaveState);
			this.connect(this.commentingWidget, "onStartEditComment", this._updateSaveState);
			this.connect(this.commentingWidget, "onCancelEditingComment", this._updateSaveState);
			this.connect(this.commentingWidget, "onCommentCreated", this._updateSaveState);
			this.connect(this.commentingWidget, "onCommentUpdated", this._updateSaveState);
			this.connect(this.commentingWidget, "onMarkCommentToDelete", this._updateSaveState);
			this.connect(this.commentingWidget, "onUnmarkCommentToDelete", this._updateSaveState);
			this.connect(this.commentingWidget, "onCommentDeleted", this._updateSaveState);
			
			this._updateSaveState();
		}
	},
	
	updateCanCreateState: function() {
	    var canCreate = this.commentingWidget.operationReport.canCreate();
		this._createNewButton.set("disabled", !canCreate);
		this._createNewButton.set("iconClass", canCreate? "addComment": "addComment disabled");
	},
	
	_updateSaveState: function() {
		if (!this.commentingWidget.externalSave) {
			this._saveButton.set("disabled", !this.commentingWidget.isDirty());
		}
	},
	
	_truncate: function(text) {
		if (text == null) {
			return "";
		}
		if (text.length <= TRUNCATE_LENGTH) {
			return text;
		}
		return StringUtils.truncate(text, TRUNCATE_LENGTH);
	},
	
	_onFilterClick: function() {
		if (this._filterButton.checked) {
			this.commentingWidget.showFilters();
		} else {
			this.commentingWidget.hideFilters();
		}
	},

	_onSettingsClick: function() {
		if (this._settingsButton.checked) {
			this.commentingWidget.showSettings();
		} else {
			this.commentingWidget.hideSettings();
		}
	},
	_onShowFilters: function() {
		this._filterButton.set("checked", true);
	},
	
	_onHideFilters: function() {
		this._filterButton.set("checked", false);
	},

	_onShowSettings: function() {
		this._settingsButton.set("checked", true);
	},
	
	_onHideSettings: function() {
		this._settingsButton.set("checked", false);
	},
	
	_filterComments: function(filtered, unfiltered, enabled) {
		if (enabled) {
			dojo.addClass(this._filterButton.domNode, "filter-enabled");
			this._filterButton.titleNode.title = this.messages.filtersEnabled;
		} else {
			dojo.removeClass(this._filterButton.domNode, "filter-enabled");
			this._filterButton.titleNode.title = this.messages.filtersDisabled;
		}
		
		if (this.commentingWidget.getGrouping() === GroupBy.DISCUSSION) {
			if (unfiltered.length == 0) {
				this._expandAllButton.set("disabled", true);
				this._collapseAllButton.set("disabled", true);
			} else {
				this._expandAllButton.set("disabled", false);
				this._collapseAllButton.set("disabled", false);
			}
		}
		if (filtered.length > 0 || unfiltered.length > 0) {
			this.filterEnabled(false);
			this._sortButton.set("disabled", false);
		} else {
			this.filterEnabled(true);
			this._sortButton.set("disabled", true);
		}
	},
	
	filterEnabled: function(state){
		this._filterButton.set("disabled", state);
	},
	
	_onOptionsMenuOpen: function(event) {
//		if (this.commentingWidget.commentContext != null) {
//			var text = dojo.string.substitute(this.messages.createNewCommentFor, [this._truncate(this.commentingWidget.commentContext.label)]);
//			dojo.isIE ? this._createForMenuItemTitle.innerText = text : this._createForMenuItemTitle.textContent = text;
//			this._createForMenuItem.attr("disabled", !this.commentingWidget.operationReport.canCreate());
//		} else {
//			var text = dojo.string.substitute(this.messages.createNewCommentFor, [""]);
//			dojo.isIE ? this._createForMenuItemTitle.innerText = text : this._createForMenuItemTitle.textContent = text;
//			this._createForMenuItem.attr("disabled", true);
//		}
	},
	
	_onCreateNewClick: function(e) {
		this.commentingWidget.createNewComment();
	},
	
	_onCreateForClick: function(e) {
		this.commentingWidget.createNewCommentForContext();
	},
	
	_onNoGroupingClick: function(e) {
		this._updateGrouping(this._noGroupingMenuItem, GroupBy.NOGROUPING);
	},
	
	_onGroupByDiscussionClick: function(e) {
		this._updateGrouping(this._groupByDiscussionMenuItem, GroupBy.DISCUSSION);
	},
	
	_applyGroupSettings: function(grouping, updateDisabledState) {
		switch (grouping) {
			case GroupBy.NOGROUPING:
				if (updateDisabledState) {
					this._expandAllButton.set("disabled", true);
					this._collapseAllButton.set("disabled", true);
				}
				this._groupingMenuItem = this._noGroupingMenuItem;
				break;
			case GroupBy.DISCUSSION:
				if (updateDisabledState) {
					this._expandAllButton.set("disabled", false);
					this._collapseAllButton.set("disabled", false);
				}
				this._groupingMenuItem = this._groupByDiscussionMenuItem;
				break;
		}
		if(this._groupingMenuItem.iconNode){
			dojo.addClass(this._groupingMenuItem.iconNode, "dropdownBullet");			
		}
	},
	
	_updateGrouping: function(menuItem, newGrouping) {
		if (menuItem == this._groupingMenuItem) {
			return;
		}
		
		if(this._groupingMenuItem.iconNode){
			dojo.removeClass(this._groupingMenuItem.iconNode, "dropdownBullet");			
		}
		
		this._applyGroupSettings(newGrouping, true);
		
		this.commentingWidget.setGrouping(newGrouping);
	},
	
	_onLinkWithEditorClick: function(e) {
		if (this._linkWithEditorToggleButton.checked) {
			this.commentingWidget.linkWithEditor();
		} else {
			this.commentingWidget.unlinkWithEditor();
		}
	},
	
	_onExpandAllClick: function(e) {
		this.commentingWidget.expandAll();
	},
	
	_onCollapseAllClick: function(e) {
		this.commentingWidget.collapseAll();
	},
	
	_onRefreshClick: function(e) {
		this.commentingWidget.refresh();
	},
	
	_onSaveClick: function(e) {
		var errors = [];
		this.commentingWidget.validate(errors);
		if (errors.length > 0) {
			this.commentingWidget.onError(new Error(errors[0]));
		} else {
			var taskMonitor = new rmps.core.TaskMonitor(this.messages.saving);
			this.commentingWidget.doSave(taskMonitor);
			
			var handles = [
				this.connect(taskMonitor, "onError", function(error) {
					this.commentingWidget.onError(error);
				}),
				this.connect(taskMonitor, "onComplete", function(error) {
					dojo.forEach(handles, this.disconnect, this);
				})
			];
		}
	}
	
});

})();