/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.commenting.CommentingWidget");

dojo.require("rmps.ui.commenting.CommentingController");
dojo.require("rmps.ui.commenting.CommentingToolbarMenu");
dojo.require("rmps.ui.commenting.utils");
dojo.require("rmps.ui.commenting.CommentForm");
dojo.require("rmps.ui.commenting.GroupChronological");
dojo.require("rmps.ui.commenting.GroupDiscussion");
dojo.require("rmps.ui.commenting.utils");
dojo.require("rmps.ui.EllipsisLabel");
dojo.require("rmps.ui.commenting.FilterWidget");

dojo.require("rmps.ui.PagingControl");
dojo.require("rmps.ui.commenting.SettingsWidget");

dojo.require("rmps.core.process.SecurityClient");

dojo.require("rmps.core.commenting.Comment");

dojo.require("rmps.core.util.DateUtils");

dojo.require("rmps.app.ui.internal.util.UrlUtil");

dojo.require("jazz.ui.MessageBox");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.html");
dojo.require("dojo.string");
dojo.require("dijit.layout.StackContainer");
dojo.require("dojo.cookie");
dojo.require("dojo.fx.easing");
dojo.require("dojo.date.stamp");

dojo.requireLocalization("rmps.ui.commenting", "CommentingWidgetMessages");

var util = rmps.app.ui.internal.util.UrlUtil;

var messages = dojo.i18n.getLocalization("rmps.ui.commenting", "CommentingWidgetMessages");

var ProcessConstants = rmps.ui.commenting.utils.Process;
var SecurityClient = rmps.core.process.SecurityClient;

var REFRESH_COMMENT_WIDGET_TOPIC_PREFIX= "rmps.web.app.comments.refesh.";

var FILTER_HANDLE_PROP = "___filterHandle_property";

var GroupBy = rmps.ui.commenting.utils.GroupBy;

function getAllReplies(comment) {
	var allReplies = [];
	for (var i = 0, r; r = comment.__replies[i]; ++i) {
		allReplies.push(r);
		var grandReplies = getAllReplies(r);
		for (var j = 0, gr; gr = grandReplies[j]; ++j) {
			allReplies.push(gr);
		}
	}
	return allReplies;
}

dojo.declare("rmps.ui.commenting.CachedData", null, {

	view: null,
	
	pages: null, // []
	
	filteredComments: null, //[]
	unfilteredComments: null, //[]
	expandedComments: null, //[], uri

	selectedComments: null,
	preservedSelection: null,

	commentsToDelete: null,
	
	missedParents: null,	

	constructor: function(view) {
		this.view = view;
		
		this.pages = [];
		
		this.filteredComments = [];
		this.unfilteredComments = [];
		this.expandedComments = [];

		this.selectedComments = [];
		
		this.commentsToDelete = {};
		this.missedParents = {};
	},

	getComment: function(uri) {
		for (var i = 0; i < this.pages.length; ++i) {
			var comment = this.pages[i][uri];
			if (comment) {
				return comment; 
			}
		}
		return null;
	},
	
	getCommentsOnPage: function(page) {
		var activePage = (page != undefined && page != null) ? page : this.view.getActivePage();
		this._initPages(activePage);
		return this.pages[activePage];
	},
	
	getRootsOnPage: function(page) {
		var _rootsOnPage = {};		
		var _commentsOnPage = this.getCommentsOnPage(page);
			
		for (var key in _commentsOnPage) {
			var _comment = _commentsOnPage[key];
			if (_comment.inReplyTo == null) {
				_rootsOnPage[key] = _comment;
			}
		}
			
		return _rootsOnPage;
	},
	
	getAllComments: function() {
		var allComments = this.filteredComments.slice();
		allComments = allComments.concat(this.unfilteredComments);
		return allComments;
	},

	getCommentsToDelete: function () {
		// returns top most comments only
		var _commentsToDelete = [];		
		if (this.commentsToDelete) {
			// only send delete requests for the top most comments
				
			for (var key in this.commentsToDelete) {
				var c = this.commentsToDelete[key];
				if (c.__parent && this.commentsToDelete[c.__parent.uri] != null) {
					continue;
				}
					
				_commentsToDelete.push(c);
			}
		}

		return _commentsToDelete;		
	},
	
	clear: function() {
		this.pages = [];
		this.filteredComments = [];
		this.unfilteredComments = [];
		this.expandedComments = [];
		this.selectedComments = [];
		this.commentsToDelete = {};
		this.missedParents = {};
	},
	
	isPageCached: function(page) {
/*		if (!page || page >= this.pages.length) {
			return false;
		}
		
		for (var key in this.pages[page]) {
			return true;
		}

		return false;*/
		return this.pages && this.pages[page];
	},
	
	updateActivePage: function(comments) {
		var _commentsOnPage = {}, c, parent;

		// setup parent child relationships
		for (var i = 0; i < comments.length; ++i) {
			c = comments[i];
			_commentsOnPage[c.uri] = c;
			c.__replies = [];
		}

		if (!this.view.isGroupedByDiscussion()) {
			for (var _childUri in this.missedParents) {
				var _parentUri = this.missedParents[_childUri];
				parent = _commentsOnPage[_parentUri];				
				if (parent) {
					var child = this.getComment(_childUri);
					if (child) {
						child.inReplyTo = _parentUri;
						child.__parent = parent;
						parent.__replies.push(child);
						delete this.missedParents[_childUri];
					}
				}
			}
		}
		
		for (var key in _commentsOnPage) {
			c = _commentsOnPage[key];
			if (c.inReplyTo) {
				parent = _commentsOnPage[c.inReplyTo];
				if (parent == null) {
					parent = this.getComment(c.inReplyTo);
				}
				if (parent) {
					c.__parent = parent;
					parent.__replies.push(c);
				} else {
					// handle unparented comments as roots until parent is loaded
					if (!this.view.isGroupedByDiscussion()) {
						this.missedParents[c.uri] = c.inReplyTo;
					}
					delete c.inReplyTo;
				}
			}
		}
		
		var activePage = this.view.getActivePage();
		this._initPages(activePage);		
		this.pages[activePage] = _commentsOnPage;
	},

	compactPages: function() {
		for (var i = 0; i < this.pages.length - 1;) {
			if (!this._compactPage(i)) {
				continue;
			}
				
			++i;
		}
	},
	
	addComment: function(comment, filters, page) {
		var activePage = page != null ? page : this.view.getActivePage();
		if (!this.isPageCached(activePage) && activePage != 0) {
			return;
		}
		
		var pageSize = this.view.getPageSize();
		var isGroupedByDiscussion = this.view.isGroupedByDiscussion();
		
		var _commentsOnPage = null;
		
		if (isGroupedByDiscussion) {
			if (comment.inReplyTo == null) {
				_commentsOnPage = this.getRootsOnPage(activePage);		
			}
		} else {
			_commentsOnPage = this.getCommentsOnPage(activePage);
		}
		
		if (_commentsOnPage != null) {
			if (this._getNumberOfComments(_commentsOnPage) == pageSize) {
				var _commentToMove = this._getCommentToRemove(_commentsOnPage, isGroupedByDiscussion, false);
				
				this._removeFromPage(this.pages[activePage], _commentToMove, isGroupedByDiscussion);
				this.addComment(_commentToMove, filters, activePage + 1);
			} 
		}

		this.pages[activePage][comment.uri] = comment;
		comment.__replies = [];
		var parent = this.getComment(comment.inReplyTo);
		if (parent) {
			comment.__parent = parent;
			parent.__replies.push(comment);
		}

		if (this._isFiltered(filters, comment)) {
			this.filteredComments.push(comment);	
		} else {
			this.unfilteredComments.push(comment);
		}
	},
	
	markCommentToDelete: function(comment) {
		var _comment = this.commentsToDelete[comment.uri];
		if (_comment == null || _comment != comment) {
			this.commentsToDelete[comment.uri] = comment;
			return true;
		}
		
		return false;
	},

	unmarkCommentToDelete: function(comment) {
		if (this.commentsToDelete[comment.uri] != null) {
			delete this.commentsToDelete[comment.uri];
			return true;
		}
		
		return false;
	},

	hasCommentsToDelete: function() {
		for (var key in this.commentsToDelete) {
			return true;
		}
		
		return false;
	},
	
	deleteComment: function(uri) {
		for (var i = 0; i < this.pages.length; ++i) {
			var _commentsOnPage = this.pages[i];
			delete _commentsOnPage[uri];
		}
	},
	
	filterComments: function(filters) {
		this.filteredComments = [];
		this.unfilteredComments = [];

		var _commentsOnPage = this.getCommentsOnPage(), key;		
		if (filters == null) {
			for (key in _commentsOnPage) {
				this.unfilteredComments.push(_commentsOnPage[key]);
			}
		} else {
			for (key in _commentsOnPage) {
				var comment = _commentsOnPage[key];
				if (this._isFiltered(filters, comment)) {
					this.filteredComments.push(comment);
				} else {
					this.unfilteredComments.push(comment);
				}
			}
		}
	},
	
	_isFiltered: function(filters, comment) {
		for (var i = 0, f; f = filters[i]; ++i) {
			if (f.isFiltered(comment)) {
				return true;
			}
		}
		return false;
	},
		
	_initPages: function(page) {
		if (page >= this.pages.length) {
			for (var i = this.pages.length; i <= page; ++i) {
				this.pages[i] = {};
			}
		}
	},

	_compactPage: function(page) {
		var pageSize = this.view.getPageSize();
		var isGroupedByDiscussion = this.view.isGroupedByDiscussion();
		
		var _commentsOnPage = null;
		
		if (isGroupedByDiscussion) {
			_commentsOnPage = this.getRootsOnPage(page);
		} else {
			_commentsOnPage = this.getCommentsOnPage(page);
		}

		var _numberOfComments = this._getNumberOfComments(_commentsOnPage);
		if (_numberOfComments == 0) {
			this.pages.splice(page, 1);
			return false;
		}
		
		if (_numberOfComments == pageSize) {
			return true;
		}
		
		for (var nextPage = page + 1; nextPage < this.pages.length; ++nextPage) {
			var _commentsOnNextPage = null;
			if (isGroupedByDiscussion) {
				_commentsOnNextPage = this.getRootsOnPage(nextPage);
			} else {
				_commentsOnNextPage = this.getCommentsOnPage(nextPage);
			}
			
			while (_numberOfComments < pageSize) {
				var _commentToMove = this._getCommentToRemove(_commentsOnNextPage, isGroupedByDiscussion, true);
				if (_commentToMove == null) {
					break;
				}
			
				this._removeFromPage(this.pages[nextPage], _commentToMove, isGroupedByDiscussion);
				this._insertToPage(this.pages[page], _commentToMove, isGroupedByDiscussion);

				_numberOfComments = _numberOfComments + 1;
			}
			
			if (_numberOfComments == pageSize) {
				break;
			}
		}
		
		return true;
	},
	
	_removeFromPage: function(commentsOnPage, comment, removeReplies) {
		if (removeReplies) {
			var _replies = getAllReplies(comment);
			for (var i = 0; i < _replies.length; ++i) {
				this._removeFromPage(commentsOnPage, _replies[i], false);
			}			
		}

		delete commentsOnPage[comment.uri];
		this.view.onCommentRemoved(comment);
	},
	
	_insertToPage: function(commentsOnPage, comment, insertReplies) {
		commentsOnPage[comment.uri] = comment;
		this.view.onCommentInserted(comment);
		
		if (insertReplies) {
			for (var i = 0, reply; reply = comment.__replies[i]; ++i) {
				this._insertToPage(commentsOnPage, reply, true);
			}
		}
	},
	
	_getNumberOfComments: function(comments) {
		var counter = 0;
		for (var key in comments) {
			++counter;
		}
		
		return counter;
	},
	
	_getCommentToRemove: function (comments, checkReplies, getLastUpdated) {
		var _commentToRemove = null;

		for (var key in comments) {
			var _lastUpdated = comments[key];
			
			if (checkReplies) {
				var _replies = getAllReplies(_lastUpdated);
				for (var i = 0; i < _replies.length; ++i) {
					var _reply = _replies[i];
					if (_lastUpdated.updated < _reply.updated) {
						_lastUpdated = _reply;
					}
				}
			}
							
			if (_commentToRemove == null) {
				_commentToRemove = _lastUpdated;
				continue;
			}
			
			if (getLastUpdated) {
				if (_commentToRemove.updated < _lastUpdated.updated) {
					_commentToRemove = _lastUpdated;
				}
			} else {
				if (_commentToRemove.updated > _lastUpdated.updated) {
					_commentToRemove = _lastUpdated;
				}
			}
		}
		
		if (_commentToRemove != null && checkReplies) {
			while (_commentToRemove.__parent) {
				_commentToRemove = _commentToRemove.__parent;
			}
		}
		
		return _commentToRemove;
	}
	
});

dojo.declare("rmps.ui.commenting.CommentingWidget", [dijit._Widget, dijit._Templated], {
	
	messages: messages,
	templateString: dojo.cache("rmps.ui", "commenting/templates/CommentingWidget.html"),
	widgetsInTemplate: true,
	
	// attach points
	_menuToolbar: null,
	_groupStack: null,
	_commentForm: null,
	_titleBarNode: null,
	_formTitleBarNode: null,
	_listContainer: null,
	_formContainer: null,
	_showHideLink: null,
	_filterContainer: null,
	_settingsContainer: null,
	
	_titleBar: null,
	_formTitleBar: null,

	_pagingControl: null,
	_settingsUpdated: false,
	
	// mixin properties
	resourceInfo: null,
	userInfo: null,
	externalSave: false,
	showResourceLabel: false,
	isResourceLabelLink: false,
	snapshotFilterEnabled: true,
	snapshotSettingsEnabled: true,
	
	// properties
	commentContext: null,
	operationReport: null,
	
	// private properties
	_cache: null,
	_controller: null,
	_grouping: GroupBy.DISCUSSION,
	_groupWidgets: null, // {}
	_toolbarWidget: null,
	_sortAscending: false,
	_editingComment: null,
	_filters: null, //[]
	_settings: null,
	_fixedHeight: false,
	_commentFormVisible: false,
	_commentListVisible: true,
	_filtersEnabled: false,
	_pagingControlVisible: false,
	_securityDeferred: null,
	
	_prevListHeight: null,
	_listHeight: 0,
	
	_isLoading: false,
	
	_pageToken: null,
	
	_forceRefresh: null,

	constructor: function(view) {
		this._groupWidgets = {};
		this._filters = [];

		if (!this.resourceInfo) {
			this.resourceInfo = {};
		}
		// FIXME
		this.self = this;
	},
	
	postCreate: function() {
		var obj = this._getCookie();
		if (obj.grouping) {
			this._grouping = obj.grouping;
		}
		
		this._showHideLink.appendChild(dojo.doc.createTextNode(messages.showCommentsList));
		
		this._titleBar = new rmps.ui.commenting.CommentingWidget._TitleBar({
			defaultTitle: messages.title_comments,
			defaultState: dojo.hitch(this, function() {
				return dojo.string.substitute(messages.state_commentsCount,
						[this._cache.unfilteredComments.length, this._cache.filteredComments.length + this._cache.unfilteredComments.length]);
			})
		});
		this._updateTitleBar(null, messages.state_loading);
		this._titleBarNode.appendChild(this._titleBar.domNode);
		this._titleBar.startup();
		
		this._formTitleBar = new rmps.ui.commenting.CommentingWidget._TitleBar();
		this._formTitleBarNode.appendChild(this._formTitleBar.domNode);
		this._formTitleBar.startup();
		this.setCommentingController();		
		this.createSettings(this._settingsContainer.firstChild);

		// cache
		this._cache = new rmps.ui.commenting.CachedData(this);

		// setup paging control
		this._pagingControl.setView(this);
		
		// setup comment form
		this._commentForm = new rmps.ui.commenting.CommentForm({
			commentingWidget: this,
			onCancel: dojo.hitch(this, this._cancelCommentForm),
			onSave: dojo.hitch(this, this._saveCommentForm),
			onPost: dojo.hitch(this, this._postCommentForm)
		}, this._commentForm);
		
		this.connect(this._formTitleBar.domNode, "onmousedown", this._onStartFormDrag);
		this.connect(this._formTitleBar.domNode, "ondragstart", dojo.stopEvent);
		this.connect(this._formTitleBar.domNode, "onselectstart", dojo.stopEvent);
		
		this.addListeners();
		
		this._securityDeferred = SecurityClient.isActionAllowed(
			ProcessConstants.Operations.COMMENT,
			dojo.getObject("jazz.app.currentApplication").context,
			[
				ProcessConstants.Actions.CREATE,
				ProcessConstants.Actions.MODIFY,
				ProcessConstants.Actions.DELETE,
				ProcessConstants.Actions.ADMIN_MODIFY,
				ProcessConstants.Actions.ADMIN_DELETE
			]);
			
		var self = this;
		this._securityDeferred.addCallback(this, function(report) {
			delete this._securityDeferred;
			this.operationReport = report;
			
			// add helper methods
			dojo.mixin(this.operationReport, {
				userUri: this.userInfo.uri,
				
				canCreate: function() {
					return !(self.resourceInfo instanceof Array) &&
					util.isUri(self.resourceInfo.uri) &&
					this.isAllowed(ProcessConstants.Actions.CREATE);
				},
				
				canEdit: function(comment) {
					return (this.isAllowed(ProcessConstants.Actions.MODIFY)
							&& decodeURIComponent(comment.creator) === decodeURIComponent(this.userUri))
							|| this.isAllowed(ProcessConstants.Actions.ADMIN_MODIFY);
				},
				
				canDelete: function(comment) {
					if (this.isAllowed(ProcessConstants.Actions.ADMIN_DELETE)) {
						return true;
					}
					if (decodeURIComponent(comment.creator) === decodeURIComponent(this.userUri) && this.isAllowed(ProcessConstants.Actions.DELETE)) {
						var replies = getAllReplies(comment);
						for (var i = 0, r; r = replies[i]; ++i) {
							if (decodeURIComponent(r.creator) !== decodeURIComponent(this.userUri)) {
								return false;
							}
						}
						return true;
					}
					return false;
				}
			});
			
			// populate the toolbar
			this._toolbarWidget = this._createToolbarWidget();
			
			if (this._toolbarWidget) {
				this._menuToolbar.appendChild(this._toolbarWidget.domNode);
				this._toolbarWidget.startup();
				this.resize();
			}
		});
	},
	
	addListeners: function(){
		this._subscriptions = [];
		if(dojo.isArray(this.resourceInfo)){
			for(var index=0; index<this.resourceInfo.length; index++){
				this._subscriptions.push(this.subscribe(REFRESH_COMMENT_WIDGET_TOPIC_PREFIX+this.resourceInfo[index].uri, this.afterSave));
			}
		}else{
			this._subscriptions.push(this.subscribe(REFRESH_COMMENT_WIDGET_TOPIC_PREFIX+this.resourceInfo.uri, this.afterSave));
		}
	},
	
	removeListeners: function(){
		for (var i=0; i<this._subscriptions.length; i++) {
			this.unsubscribe(this._subscriptions[i]);
		}
	},
	
	setCommentingController: function(){
		this._controller = new rmps.ui.commenting.CommentingController(this);
	},
	
	startup: function() {
		if (this._started) {
			return;
		}
		
		if (this._toolbarWidget) {
			this._toolbarWidget.startup();
		}
		
		if (this._settingsWidget) {
			this._settingsWidget.startup();
		}

		this.inherited(arguments);
	},
	
	destroy: function() {
		this.removeListeners();
		this._titleBar.destroyRecursive();
		this._formTitleBar.destroyRecursive();
		this._commentForm.destroyRecursive();
		this._filterWidget && this._filterWidget.destroyRecursive();
		this._settingsWidget && this._settingsWidget.destroyRecursive();
		this._toolbarWidget && this._toolbarWidget.destroyRecursive();
		this._controller.destroy();
		for (var i = 0, f; f = this._filters[i]; ++i) {
			delete f[FILTER_HANDLE_PROP];
		}
		
		if(this._securityDeferred){
			this._securityDeferred.cancel();
			delete this._securityDeferred;
		}
		
		this.inherited(arguments);
	},
	
	// layout methods
	
	resize: function(size) {
		if (size && size.h != null) {
			this._fixedHeight = true;
			this.domNode.style.height = size.h + "px";
		}
		this.layout();
	},
	
	layout: function() {
		if (!this._fixedHeight) {
			return;
		}
		var height = this.domNode.offsetHeight;
		
		if (this._commentFormVisible && this._commentListVisible) {
			height = Math.min(this._listHeight, height - this._formTitleBarNode.offsetHeight);
		}
		
		if (!this._commentFormVisible || this._commentListVisible) {
			this._titleBar.resize();
			var stackHeight = Math.max(0, height - this._groupStack.domNode.offsetTop);
			
			if (this._pagingControlVisible) {
				stackHeight = Math.max(0, stackHeight - this._pagingControl.domNode.offsetHeight);
			}
			
			this._groupStack.domNode.style.height = stackHeight + "px";
			
			if (this._groupStack.selectedChildWidget && this._groupStack.selectedChildWidget.resize){
				this._groupStack.selectedChildWidget.resize({ w: this._groupStack.domNode.offsetWidth });
			}
		}
		
		if (this._commentFormVisible) {
			this._formTitleBar.resize();
			var c = dojo.coords(this.domNode, true);
			var v = dijit.getViewport();
			height = Math.min(c.y + this.domNode.offsetHeight, v.h) - c.y - this._commentForm.domNode.offsetTop;
			this._commentForm.resize({ w: this.domNode.offsetWidth, h: height });
		}
	},
	
	// end layout
	
	refresh: function() {
		//perf: ignore all refreshes before settings have been updated.  When
		//ctrl is created, settings (such as paging ctrl) will be updated,
		//requiring and triggering a refresh anyway 
		if (this._settingsUpdated) {
			if (this._toolbarWidget) {
				this._toolbarWidget.updateCanCreateState();
			}
			if (!this.isDirty() || this.forceRefresh ||  confirm(messages.confirmRefresh)) {
				this.forceRefresh = false;
				if (this._editingComment) {
					delete this._editingComment;
					this.onCancelEditingComment(this._editingComment);
					this._hideCommentForm();
				}
				this._cache.preservedSelection = this._cache.selectedComments;
				this._cache.clear();
				this._resetPagingControl();
				this._updateTitleBar(null, messages.state_loading);
				this._isLoading = true;

				if (this._securityDeferred) {
					this._securityDeferred.addCallback(this, function() {
						this._updatePagingControl();
					});
				} else {
					this._updatePagingControl();
				}
			}
		}
	},
	
	// pagination
	
	getActivePage: function() {
		return this._pagingControl.page;
	},
	
	getPageSize: function() {
		return this._settings.pageSize;
	},

	getTotalResults: function() {
		return this._pagingControl.size;
	},
	
	getPageToken: function() {
		return this._pageToken;
	},
	
	_showPagingControl: function() {
		if (!this._pagingControlVisible) {
			this._pagingControlVisible = true;
			this._pagingControl.domNode.style.display = "";
			// TODO: do we need this call?
			this.layout();
		}
	},
	
	_hidePagingControl: function() {
		if (this._pagingControlVisible) {
			this._pagingControlVisible = false;
			this._pagingControl.domNode.style.display = "none";
			// TODO: do we need this call?			
			this.layout();
		}
	},
	
	_resetPagingControl: function() {
		this._hidePagingControl();
		this._pagingControl.reset();		
	},
	
	_updatePagingControl: function(page, pageSize, limit, size, item) {
		if (page != null && pageSize != null) {
			this._pagingControl.update(page, pageSize, null, size, null);
		} else {
			this._pagingControl.update(0);
		}
	},
	
	// cookie
	
	_getCookie: function() {
		var value = dojo.cookie(this.declaredClass);
		if (value) {
			return dojo.fromJson(value)[this.userInfo.uri] || {};
		}
		return {};
	},
	
	_setCookie: function(obj) {
		var value = dojo.cookie(this.declaredClass), cookie;
		if (value) {
			cookie = dojo.fromJson(value);
		} else {
			cookie = {};
		}
		cookie[this.userInfo.uri] = obj;
		dojo.cookie(this.declaredClass, dojo.toJson(cookie), {expires: 90});
	},
	
	processSelection: function( previous, newSelection ){
		
		var deselect = [];
		var select = [];
		
		for( var i = 0; i < newSelection.length; ++i ){
			
			var comment = newSelection[i];
			
			/* Check if any of the new comments were previously selected */
			
			var idx = dojo.indexOf( previous, comment );
			
			if( idx >= 0 ){
				
				/* A comment was previously selected, we need to decide if 
				 * the action is:
				 * 
				 * 1] Re-selection of a single previous selection - which is a deselect
				 * 2] A multiple selection including a previously selected item
				 * 3] A multiple selection excluding some previous selections 
				 * 
				 */
				
				if( newSelection.length == 1 && previous.length == 1 ){
					
					/* Selection of a previously selected comment when there was only 
					 * one comment previously selected can only mean deselection 
					 * of a single comment */
					
					deselect.push( comment );
				
				}else{
					
					/* Either this is a single previously unselected comment, or the 
					 * (new) comment was previously selected, but this time it is part of
					 * a multiple selection ... so want to keep it selected now.
					 */ 
					
					select.push( comment );
					
					/* Remove it from the previous selection list. This will leave
					 * the previous list with only items that are no longer part
					 * of a new multiple selection - we'll use that list to deselect 
					 * the left overs later */
					
					previous.splice( idx, 1 );
				}
			}else{
				
				/* The new comment was not previously selected so select it now */
				
				select.push( comment );
			}		
		}
		
		/* Now add all of the previously selected comments that are no
		 * longer selected to the deselect list.
		 */
		
		deselect = deselect.concat( previous );
				
		return { deselect: deselect, select: select };
	},
	
	
	// comment selection
	
	selectComments: function( newSelection ) {
		
		newSelection = newSelection ? (dojo.isArray(newSelection) ? newSelection : [newSelection]) : [];

		var previousSelected = this._cache.selectedComments.slice(0);
				
		var state = this.processSelection( previousSelected, newSelection );
		
		this._cache.selectedComments = state.select;
		
		this.onSelectionChanged(this._cache.selectedComments, previousSelected, state.select, state.deselect);		
	},
	
	appendCommentSelection: function(comments) {
		comments = dojo.isArray(comments) ? comments : [comments];
		var newSelection = this._cache.selectedComments.slice();
		for (var i = 0; i < comments.length; ++i) {
			if (dojo.indexOf(newSelection, comments[i]) == -1) {
				newSelection.push(comments[i]);
			}
		}
		if (newSelection.length > this._cache.selectedComments.length) {
			this.selectComments(newSelection);
		}
	},
	
	deselectComment: function(comment) {
		var idx = dojo.indexOf( this._cache.selectedComments, comment );
		if (idx >= 0) {
			var newSelection = this._cache.selectedComments.slice();
			newSelection.splice(idx, 1);
			this.selectComments(newSelection);
		}
	},
	
	// title
	_updateTitleBar: function(newTitle, newState) {
		if (!this._isLoading) {
			this._titleBar.update(newTitle, newState);
		}
	},
	
	_updateFormTitleBar: function(newTitle, newState) {
		this._formTitleBar.update(newTitle, newState);
	},
	
	// populate the toolbar
	_createToolbarWidget: function() {
		return new rmps.ui.commenting.CommentingToolbarMenu({
			commentingWidget: this
		});
	},
	
	// collapse & expand
	expandAll: function() {
		this.onExpandAllComments();
	},
	
	collapseAll: function() {
		this.onCollapseAllComments();
	},
	
	// Sorting
	sortAscending: function(bool) {
		this._sortAscending = bool;
	},
	
	isSortAscending: function() {
		return this._sortAscending;
	},
	
	// Grouping
	getGrouping: function() {
		return this._grouping;
	},

	isGroupedByDiscussion: function() {
		return this._grouping == GroupBy.DISCUSSION;
	},
	
	setGrouping: function(grouping) {
		if (grouping != this._grouping) {
			this._grouping = grouping;
			// save setting in a cookie
			var obj = this._getCookie();
			obj.grouping = this._grouping;
			this._setCookie(obj);
			this.refresh();
		}
	},
	
	_createGroupWidget: function() {
		switch (this._grouping) {
			case GroupBy.DISCUSSION:
				return new rmps.ui.commenting.GroupDiscussion({
					commentingWidget: this,
					comments: this._cache.getCommentsOnPage()
				});
				
			case GroupBy.CHRONOLOGICAL:
			case GroupBy.NOGROUPING:
			default:
				return new rmps.ui.commenting.GroupChronological({
					commentingWidget: this,
					comments: this._cache.getCommentsOnPage()
				});
		}
	},
	
	_reGroup: function() {
		if (this._cache.isPageCached(this.getActivePage())) {
			var groupWidget = this._groupWidgets[this._grouping];
			if (!groupWidget) {
				groupWidget = this._createGroupWidget();
				this._groupWidgets[this._grouping] = groupWidget;
				this._groupStack.addChild(groupWidget);
			}
			this._groupStack.selectChild(groupWidget);
			if (groupWidget.resize) {
				groupWidget.resize({ w: this._groupStack.domNode.clientWidth});
			}
		}
	},
	
	// Filtering
	setFilteringEnabled: function(enabled) {
		if (this._filtersEnabled != enabled) {
			this._filtersEnabled = enabled;
			this._cache.filterComments(null);
			this._fireFilterCommentsEvent();
		}
	},
	
	isFilteringEnabled: function() {
		return this._filtersEnabled;
	},
	
	createFilters: function() {
		if (!this._filterWidget) {
			this._filterWidget = this.getFilterWidget();
			this._filterContainer.firstChild.appendChild(this._filterWidget.domNode);
			this._filterWidget.startup();			
		}
	},
	
	getFilterWidget: function() {
		return new rmps.ui.commenting.FilterWidget({
			commentingWidget: this,
			snapshotFilterEnabled: this.snapshotFilterEnabled
		});		
	},
	
	showFilters: function() {
		this.createFilters();
		
		if (!this._filtersVisible) {
			this._filtersVisible = true;
			this._filterContainer.style.display = "";
			this.layout();
		}
	},
	
	hideFilters: function() {
		if (this._filtersVisible) {
			this._filtersVisible = false;
			this._filterContainer.style.display = "none";
			this.layout();
		}
	},

	addFilter: function(filter) {
		if (!filter[FILTER_HANDLE_PROP]) {
			this._filters.push(filter);
			filter[FILTER_HANDLE_PROP] = this.connect(filter, "onChange", function(enabled) {
					this._onFilterChange(filter, enabled);
				});
			this._applyFilters();
		}
	},

	removeFilter: function(filter) {
		for (var i = 0, f; f = this._filters[i]; ++i) {
			if (f === filter) {
				this._filters.splice(i, 1);
				this.disconnect(f[FILTER_HANDLE_PROP]);
				delete f[FILTER_HANDLE_PROP];
				break;
			}
		}
		this._applyFilters();
	},
	
	_onResetFiltersClicked: function(e) {
		dojo.stopEvent(e);
		this._resetFilters = true;
		for (var i = 0, f; f = this._filters[i]; ++i) {
			f.reset();
		}
		delete this._resetFilters;
		this.setFilteringEnabled(false);
	},
	
	_onCloseFiltersClicked: function(e) {
		dojo.stopEvent(e);
		this.hideFilters();
	},

	_onFilterChange: function(filter, enabled) {
		var fe = 0;
		for (var i = 0, f; f = this._filters[i]; ++i) {
			if (f.isEnabled()) {
				fe++;
			}
		}
		
		if (fe === 0) {
			this.setFilteringEnabled(false);
		} else {
			this._filtersEnabled = true;
			this._applyFilters();
		}
	},
	
	_applyFilters: function(force) {
		if (!this._resetFilters && (this._filtersEnabled || force)) {
			if (this._filtersEnabled) {
				this._updateTitleBar(null, messages.state_filtering);
			}

			if (this._filtersEnabled) {
				this._cache.filterComments(this._filters);
			} else {
				this._cache.filterComments(null);
			}
			
			this._fireFilterCommentsEvent();
					
			// We need to clear the previously selected nodes to prevent a deselection
			var previous = this._cache.selectedComments.slice(0);
			this._cache.selectedComments = [];
			this.selectComments(previous);
		}
	},
	
	_fireFilterCommentsEvent: function() {
		this._updateTitleBar(null, null);
		this.onFilterComments(this._cache.filteredComments, this._cache.unfilteredComments, this._filtersEnabled);
	},
	
	_isFiltered: function(comment) {
		if (this._filtersEnabled) {
			for (var i = 0, f; f = this._filters[i]; ++i) {
				if (f.isFiltered(comment)) {
					return true;
				}
			}
		}
		return false;
	},
	
	_confirmCommentOverwrite: function() {
		if (!this._editingComment) {
			return true;
		}
		if (confirm(messages.overwriteComment)) {
			this.onCancelEditingComment(this._editingComment);
			delete this._editingComment;
			return true;
		}
		return false;
	},
	
	// settings
	
	createSettings: function(parentNode) {
		this._settingsWidget = this.getSettingsWidget();
		parentNode.appendChild(this._settingsWidget.domNode);
	},
	
	getSettingsWidget: function(){
		var obj = this._getCookie();
		var pageSize = obj.pageSize ? obj.pageSize : -1; 
		
		return new rmps.ui.commenting.SettingsWidget({
			commentingWidget: this,
			pageSize: pageSize
		});		
	},

	showSettings: function() {
		if (!this._settingsVisible) {
			this._settingsVisible = true;
			this._settingsContainer.style.display = "";
			this.layout();
		}
	},
	
	hideSettings: function() {
		if (this._settingsVisible) {
			this._settingsVisible = false;
			this._settingsContainer.style.display = "none";
			this.layout();
		}
	},

	_onCloseSettingsClicked: function(e) {
		dojo.stopEvent(e);
		this.hideSettings();
	},

	addSettings: function(settings) {
		this._settings = settings;
		this.connect(this._settings, "onChange", function(enabled) {
			this._onSettingsChange(enabled);
		});
	},

	_onSettingsChange: function(enabled) {
		this._settingsUpdated = true;
		if (this._settings.isEnabled() && this._settings.pageSize != this._pagingControl.pageSize) {
			var obj = this._getCookie();
			obj.pageSize = this._settings.pageSize;
			this._setCookie(obj);			
			this.refresh();
		}
	},
	
	// Reply
	replyToComment: function(replyToComment) {
		if (!this._confirmCommentOverwrite()) {
			return;
		}
		
		var newComment = new rmps.core.commenting.Comment();
		
		this._editingComment = newComment;
		
		newComment.resourceUri = replyToComment.resourceUri;
		newComment.inReplyTo = replyToComment.uri;
		newComment.range = replyToComment.range;
		newComment.subject = replyToComment.subject.indexOf(dojo.string.substitute(messages.replySubject, [""])) == 0
				? replyToComment.subject
				: dojo.string.substitute(messages.replySubject, [replyToComment.subject]);
		newComment.creator = this.userInfo.uri;
		
		this._commentForm.showSubjectField = true;
		this._commentForm.update(newComment);

		this._cache.preservedSelection = this._cache.selectedComments.slice();
		this.selectComments(this.findSelectedCommentsInThread(replyToComment));
		
		this.onStartCreateComment(newComment);
		this._showCommentForm(dojo.string.substitute(messages.title_replyTo, [replyToComment.subject]));
	},
	
	// New comment
	
	createNewComment: function() {
		this.createNewCommentFor(this.resourceInfo.uri, this.resourceInfo.shortName);
	},
	
	createNewCommentForContext: function() {
		this.createNewCommentFor(this.commentContext.uri, this.commentContext.label);
	},
	
	createNewCommentFor: function(resourceUri, resourceLabel) {
	    var newComment, selections;
	    
		if (!this._confirmCommentOverwrite()) {
			return;
		}
		
		newComment = new rmps.core.commenting.Comment();
		
		this._editingComment = newComment;
		
		newComment.resourceUri = resourceUri;
		newComment.creator = this.userInfo.uri;
		
		this._commentForm.showSubjectField = true;
		this._commentForm.update(newComment);

		this._cache.preservedSelection = this._cache.selectedComments.slice();
		
		if(this.diagramWidget){
            selections = this.diagramWidget.viewer.getSelectionController().saveSelectionToMemento();
            this.selectComments();
            this.diagramWidget.viewer.getSelectionController().restoreSelectionFromMemento(selections);
		}else{
		    this.selectComments();
		}
		this.onStartCreateComment(newComment);
		this._showCommentForm(dojo.string.substitute(messages.title_create, [resourceLabel]));
	},
	
	// Edit comment
	editComment: function(comment) {
		if (!this._confirmCommentOverwrite()) {
			return;
		}
		this._editingComment = comment;
		// Make sure the comment is selected at this point
		var currentSelection = this._cache.selectedComments.slice();
		if (!currentSelection || dojo.indexOf(currentSelection, comment) == -1) {
			this.selectComments([comment]);			
		}
		
		var parent = comment.__parent;
		delete comment.__parent;
		var replies = comment.__replies;
		delete comment.__replies;
		
		var tempComment = dojo.clone(comment);
		
		comment.__parent = tempComment.__parent = parent;
		comment.__replies = tempComment.__replies = replies;
		
		tempComment.contributor = this.userInfo.uri;
		
		this._commentForm.showSubjectField = true;
		this._commentForm.update(tempComment);
		
		this.onStartEditComment(tempComment);
		this._showCommentForm(dojo.string.substitute(messages.title_edit, [comment.subject]));
	},
	
	// comment form
	_postCommentForm: function(comment) {
		this._updateFormTitleBar(undefined, messages.state_saving);
		this.onEndEditingComment(comment);
		if (comment.uri) {
			// edit mode
			return this._controller.updateComment(this._editingComment, comment);
		} else if (comment.inReplyTo) {
			// reply mode
			return this._controller.createComment(comment);
		} else {
			// new mode
			return this._controller.createComment(comment);
		}
	},
	
	_saveCommentFailed: function() {
		this._updateFormTitleBar(undefined, messages.state_savingError);
		this._commentForm.postFailed();
	},
	
	_cancelCommentForm: function(comment) {
		delete this._editingComment;
		this.onCancelEditingComment(comment);
		this._hideCommentForm();
	},
	
	_saveCommentForm: function(comment) {
		if(this.validate()) {
			this.onShowMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: messages.state_saving
			});

			var taskMonitor = new rmps.core.TaskMonitor(
					messages.saving);
			taskMonitor.setCanComplete(false);
			this.doSave(taskMonitor);
			
			var handleComplete = this.connect(taskMonitor, "onComplete", function(errors) {
				this.disconnect(handleComplete);
				if (errors) {
					this._saveCommentFailed();
					this.forceRefresh = true;
					this.onHideMessage();
					this.refresh();
				} else {
					this.onSaveComplete();
				}
				// Note: If an error occurs while saving the comment, the Controller in place should handle the display of them in an addErrBack.  For an example,
				// see CommentingController.updateComment();
			});
			taskMonitor.setCanComplete(true);			
		}
	},
	
	_hideCommentForm: function() {
		this._updateFormTitleBar(null, null);
		this._formContainer.style.display = "none";
		this._updateShowHideLink(true);
		this._commentFormVisible = false;
		this.layout();
	},
	
	_showCommentForm: function(titleLabel) {
		this._updateShowHideLink(false);
		this._formContainer.style.display = "";
		
		this._updateFormTitleBar(titleLabel, null);
		
		this._commentFormVisible = true;
		
		var uagent = navigator.userAgent.toLowerCase();
		
		if( uagent.search( 'ipad' ) == -1 ){
			this._commentForm.setFocus();
		}
		this.layout();
	},
	
	_loadCommentsFailed: function() {
		this._isLoading = false;
		this._updateTitleBar(null, messages.state_loadingError);
	},
	
	// comments
	
	getComment: function(uri) {
		return this._cache.getComment(uri);
	},
	
	hasSelectedComments: function() {
		return this._cache.selectedComments.length > 0;
	},

	getSelectedComments: function() {
		return this._cache.selectedComments;
	},

	getExpandedComments: function() {
		return this._cache.expandedComments;
	},
	
	getFilteredComments: function() {
		return this._cache.filteredComments;
	},
	
	getUnfilteredComments: function() {
		return this._cache.unfilteredComments;
	},
	
	isMarkedToDelete: function(comment) {
		return this._cache.commentsToDelete[comment.uri] != null;
	},

	markCommentToExpand: function(comment) {
		var idx = dojo.indexOf(this._cache.expandedComments, comment.uri);
		if (idx === -1) {
			this._cache.expandedComments.push(comment.uri);
		}
	},

	unmarkCommentToExpand: function(comment) {
		var idx = dojo.indexOf(this._cache.expandedComments, comment.uri);
		if (idx !== -1) {
			this._cache.expandedComments.splice(idx, 1);
		}
	},

	_updateCache: function(comments) {
		if (comments.totalResults && comments.totalResults != 0) {
			this._pageToken = comments.pageToken;
			this._cache.updateActivePage(comments);
			this._updatePagingControl(this.getActivePage(), this.getPageSize(), null, comments.totalResults, null);
		}
	},

	_showCommentsOnActivePage: function() {
		this._isLoading = false;
		
		// delete all groups
		for (var key in this._groupWidgets) {
			var groupWidget = this._groupWidgets[key];
			this._groupStack.removeChild(groupWidget);
			groupWidget.destroyRecursive();
			delete this._groupWidgets[key];
		}

		if (this.hasSelectedComments()) {
			this._cache.preservedSelection = this._cache.selectedComments.slice();
			this.selectComments();
		}

		this._applyFilters(true);
		this._reGroup();
		
		if (this._cache.preservedSelection != null) {
			// reselect previous selection
			this.selectComments(this._cache.preservedSelection);
			this._cache.preservedSelection = null;
		}
				
		this.onLoad();
	},
	
	_commentEdited: function(comment) {
		delete this._editingComment;
		this._hideCommentForm();
		this.selectComments([comment]);
		// FIXME: Not sure why we make this call as the function called by onCommentUpdated is yet
		// called by this.selectComments()
		this.onCommentUpdated(comment);

		this._fireFilterCommentsEvent();
		if (this._isFiltered(comment)) {
			this.selectComments(this._cache.preservedSelection);
		} else {
			this.appendCommentSelection(comment);
		}
		
		this._cache.preservedSelection = null;
	},
	
	_addComment: function(comment) {
		delete this._editingComment;
		this._cache.addComment(comment, this._filters);		

		this._hideCommentForm();
		this.onCommentCreated(comment); 
		
		this._fireFilterCommentsEvent();
		if (this._isFiltered(comment)) {
			this.selectComments(this._cache.preservedSelection);
		} else {
			this.appendCommentSelection(comment);
		}
		
		this._cache.preservedSelection = null;
		
		if (this._groupWidgets[this._grouping] == null) {
			this._showCommentsOnActivePage();
		}
	},
	
	findSelectedCommentsInThread: function(comment) {		
		var curComment = comment;
		var newSelection = [];
		newSelection.push(curComment);
		while (curComment.__parent)
		{
			if (dojo.indexOf(this._cache.selectedComments, curComment) < 0) {
				newSelection.push(curComment);
			}
			curComment = curComment.__parent;
		}
		return newSelection;
	},
	
	markCommentToDelete: function(comment, includingReplies) {
		if (this.operationReport.canDelete(comment) 
				&& confirm(messages.removeConfirm)) {
			var saveRequired = false; 
			if (this._cache.markCommentToDelete(comment)) {
				this.onMarkCommentToDelete(comment);
				saveRequired = true;
			}

			if (includingReplies) {
				var _replies = getAllReplies(comment);
				for (var i = 0, reply; reply = _replies[i]; ++i) {
					if (this._cache.markCommentToDelete(reply)) {
						this.onMarkCommentToDelete(reply);
						saveRequired = true;
					}
				}
			}
			
			if (saveRequired) {
				this._saveCommentForm();
			}
		}
	},
	
	unmarkCommentToDelete: function(comment, includingReplies, includingParents) {
		if (this.operationReport.canDelete(comment)) {
			if (this._cache.unmarkCommentToDelete(comment)) {
				this.onUnmarkCommentToDelete(comment);
			}

			if (includingReplies) {
				var _replies = getAllReplies(comment);
				for (var i = 0, reply; reply = _replies[i]; ++i) {
					if (this._cache.unmarkCommentToDelete(reply)) {
						this.onUnmarkCommentToDelete(reply);
					}
				}
			}
			
			if (includingParents) {
				var parent = comment.__parent;
				while (parent) {
					this.unmarkCommentToDelete(parent, false, true);
					parent = parent.__parent;
				}
			}
		}
	},
	
	_commentDeleted: function(comment) {
		var deleted = getAllReplies(comment);
		deleted.push(comment);
		
		// the comment and all of its replies were deleted
		for (var i = 0, c; c = deleted[i]; ++i) {
			this._cache.unmarkCommentToDelete(c);
			var parent = this._cache.getComment(c.inReplyTo);
			if (parent) {
				parent.__replies.splice(dojo.indexOf(parent.__replies, c), 1);
			}

			this._cache.deleteComment(c.uri);
			this.onCommentDeleted(c);
		}
				
		this._applyFilters(true);
		this.selectComments(this._cache.selectedComments.slice());
		
		this.sortAscending(true);
		this._cache.compactPages();
		this.sortAscending(false);
		
		if (!this._cache.isPageCached(this.getActivePage())) {
			this._updatePagingControl();
		}
	},
	
	// link with editor
	
	linkWithEditor: function() {
		this.onLinkWithEditor(true, this._cache.getAllComments());
	},
	
	unlinkWithEditor: function() {
		this.onLinkWithEditor(false, this._cache.getAllComments());
	},
	
	// toggle list when viewing form
	_toggleList: function(e) {
		dojo.stopEvent(e);
		
		var endValue;
		if (this._commentListVisible) {
			this._prevListHeight = this._listHeight = this._formContainer.offsetTop;
			endValue = 0;
		} else {
			endValue = this._prevListHeight != null
					? this._prevListHeight
					: Math.round(this._formContainer.offsetHeight / 2);
		}
		
		new dojo._Animation({
			duration: 300,
			curve: [this._listHeight, endValue],
			easing: dojo.fx.easing.cubicOut,
			onBegin: dojo.hitch(this, function() {
				this._listContainer.style.overflow = "hidden";
				this._updateShowHideLink(!this._commentListVisible);
			}),
			onEnd: dojo.hitch(this, function() {
				this._listContainer.style.overflow = "";
				this._listContainer.style.height = "";
			}),
			onAnimate: dojo.hitch(this, function(r) {
				this._listContainer.style.height = r + "px";
				this._listHeight = r;
				this.layout();
			})
		}).play();
	},
	
	_updateShowHideLink: function(show) {
		if (show) {
			this._formTitleBarNode.firstChild.style.display = "";
			dojo.html._setNodeContent(this._showHideLink, dojo.doc.createTextNode(messages.hideCommentsList), true);
			this._listContainer.style.display = "";
			this._commentListVisible = true;
		} else {
			this._formTitleBarNode.firstChild.style.display = "none";
			dojo.html._setNodeContent(this._showHideLink, dojo.doc.createTextNode(messages.showCommentsList), true);
			this._listContainer.style.display = "none";
			this._commentListVisible = false;
		}
	},
	
	// form drag
	
	_onStartFormDrag: function(e) {
		var d = this._formTitleBar.domNode.ownerDocument;
		this._dragEvents = [
			this.connect(d, "onmousemove", this._onFormDrag),
			this.connect(d, "onmouseup", this._onStopFormDrag),
			// cancel text selection and text dragging
			this.connect(d, "ondragstart", dojo.stopEvent),
			this.connect(d.body, "onselectstart", dojo.stopEvent)
		];
		
		this._listHeight = this._formContainer.offsetTop;
		this._dragY = e.pageY;
	},
	
	_onFormDrag: function(e) {
		
		var diff = e.pageY - this._dragY;
		
		var maxHeight = this.domNode.offsetHeight - this._formTitleBarNode.offsetHeight;
		
		if ((this._listHeight + diff) < 0) {
			this._dragY = this._dragY - this._listHeight;
			this._listHeight = 0;
		} else if ((this._listHeight + diff) > maxHeight) {
			this._dragY = this._dragY - (maxHeight - this._listHeight);
			this._listHeight = maxHeight;
		} else {
			this._listHeight += diff;
			this._dragY = e.pageY;
		}
		
		if ((this._listHeight > 0) != this._commentListVisible) {
			this._updateShowHideLink(hidden);
		}
		
		this.layout();
	},
	
	_onStopFormDrag: function() {
		this._listHeight = this._formContainer.offsetTop;
		if (this._listHeight > 0) {
			this._prevListHeight = this._listHeight;
		}
		dojo.forEach(this._dragEvents, this.disconnect, this);
		delete this._dragEvents;
	},
	
	// saving
	
	validate: function() {
		if (this._editingComment) {
			return this._commentForm.validate();
		}
		return true;
	},
	
	doSave: function(taskMonitor) {
		var task;
		if (this._editingComment) {
			var subjectLabel = this._editingComment.subject;
			var saveDeferred = this._commentForm.post();
			if (saveDeferred ) {
				if (subjectLabel) {
					task = taskMonitor.beginTask(dojo.string.substitute(messages.task_saveComment, [subjectLabel]), 1);
					task.handleDeferredWork(saveDeferred, 1);
					saveDeferred.addCallback(this, function(report) {
						this.notifyAllWidgets();
					});
				}
			}
		}
		
		var commentsToDelete = this._cache.getCommentsToDelete();
		if (commentsToDelete.length > 0) {
			for (var i = 0, c; c = commentsToDelete[i]; ++i) {
				var deleteDeferred = this._controller.deleteComment(c);
				if (deleteDeferred) {
					task = taskMonitor.beginTask(dojo.string.substitute(messages.task_deleteComment, [c.subject]), 1);
					task.handleDeferredWork(deleteDeferred, 1);
					deleteDeferred.addCallback(this, function(report) {
						this.notifyAllWidgets();
					});
				}
			}
		}
	},
	
	/* 
	 * Send notification for all ocmmenting widgets that have the same resource uri 
	 */
	notifyAllWidgets: function(){
		if(dojo.isArray(this.resourceInfo)){
			for(var index=0; index<this.resourceInfo.length; index++){
				dojo.publish(REFRESH_COMMENT_WIDGET_TOPIC_PREFIX+this.resourceInfo[index].uri, [this]);		
			}
		}else{
			dojo.publish(REFRESH_COMMENT_WIDGET_TOPIC_PREFIX+this.resourceInfo.uri, [this]);
		}
	},
	
	afterSave: function(widget){
		/*If widget has the same value of "this", this means that the notification came from the curent widget, thus we call for onInternalSave*/
		if(widget=== this){
			this.onInternalSave();
		}else{
			/*Make sure to refresh on external saves, in order to make sure that other widgets are in sync with the widgets that made some saves */
			this._cache.clear();
			this.refresh();
			this.onExternalSave();		
		}
	},
	
	onInternalSave: function(){/*To be called or connected by other places prior a save fom the curent widget */},
	
	onExternalSave: function(){/*To be called or connected by other places prior a save fom another widget but uses the same resource ui*/},
	
	isDirty: function() {
		return this._editingComment || this._cache.hasCommentsToDelete();
	},	
	
	// eventing
	
	onShowMessage: function(args, showDelay) {},
	
	onHideMessage: function() {},

	onSaveComplete: function() {},
	
	onLoad: function() {},
	
	onSelectionChanged: function(selection, previousSelection, newSelected, deselected) {},
	
	onCommentCreated: function(comment) {},
	
	onCommentUpdated: function(comment) {},
	
	onMarkCommentToDelete: function(comment) {},
	
	onUnmarkCommentToDelete: function(comment) {},
	
	onCommentRemoved: function(comment) {},
	
	onCommentInserted: function(comment) {},
	
	onCommentDeleted: function(comment) {},
	
	onStartCreateComment: function(comment) {},
	
	onStartEditComment: function(comment) {},
	
	onEndEditingComment: function(comment) {},
	
	onCancelEditingComment: function(comment) {},
	
	onLinkWithEditor: function(isLinked) {},
	
	onExpandAllComments: function() {},
	
	onCollapseAllComments: function() {},
	
	onFilterComments: function(filtered, unfiltered) {}
});


dojo.declare("rmps.ui.commenting.CommentingWidget._TitleBar", [dijit._Widget, dijit._Templated], {
	
	templateString: '<div dojoAttachPoint="_title" class="rmps-ui-commenting-title">' +
			'<div>' +
				'<nobr>' +
					'<div dojoAttachPoint="_titleLabel" dojoType="rmps.ui.EllipsisLabel" class="title dijitInline" constrainWidth="true"></div>' +
					'<span>&nbsp;</span>' +
					'<span dojoAttachPoint="_titleState" class="state"></span>' +
				'</nobr>' +
			'</div>' +
		'</div>',
	
	widgetsInTemplate: true,
	
	defaultTitle: null,
	defaultState: null,
	
	postCreate: function() {
		this._setTitle(null);
		this._setState(null);
	},
	
	resize: function() {
		this._titleLabel.resize({ w: this._title.firstChild.offsetWidth - this._titleState.offsetWidth});
	},
	
	update: function(newTitle, newState) {
		if (newTitle !== undefined) {
			this._setTitle(newTitle);
		}
		if (newState !== undefined) {
			this._setState(newState);
		}
		this.resize();
	},
	
	_setTitle: function(title) {
		if (title) {
			this._titleLabel.setLabel(title);
			
		} else if (this.defaultTitle) {
			// default
			if (dojo.isFunction(this.defaultTitle)) {
				this._titleLabel.setLabel(this.defaultTitle());
			} else {
				this._titleLabel.setLabel(this.defaultTitle);
			}
			
		}
	},
	
	_setState: function(state) {
		if(!this.resourceInfo|| !this.resourceInfo.uri){
			dojo.html._emptyNode(this._titleState);
			return;
		}
		if (state == null && this.defaultState) {
			// default
			if (dojo.isFunction(this.defaultState)) {
				state = this.defaultState();
			} else {
				state = this.defaultState;
			}
		}
		if (state) {
			state = dojo.string.substitute(messages.state, [state]);
			dojo.html._setNodeContent(this._titleState, dojo.doc.createTextNode(state), true);
		} else {
			dojo.html._emptyNode(this._titleState);
		}
	}
});

dojo.declare("rmps.ui.commenting.CommentingWidget._PagingControl", rmps.ui.PagingControl, {
	
	view: null,
	
	_isRefreshRequest: false,
	
	setView: function(view) {
		this.view = view;
		this.connect(this.view, "onCommentCreated", this._commentCreated);
		this.connect(this.view, "onCommentDeleted", this._commentDeleted);
	},
	
	reset: function() {
		this.size = 0;
		this.page = 0;
		this.pageSize = -1;
		this.limit = -1;
		this.item = null;
		this.hasDetails = false;
	},
	
	update: function(page, pageSize, limit, size, item) {
		if (this.page != page || this.pageSize == -1 || this.size != size) {
			this.inherited(arguments);
			this._visualize();
		}
		
		if (!this._isRefreshRequest) {
			if (!this.view._cache.isPageCached(page)) {
				this.view._controller.getComments();
			} else {
				this.view._showCommentsOnActivePage();
			}
		}
	},
	
	_commentCreated: function(comment) {
		var isGroupedByDiscussion = this.view.isGroupedByDiscussion();
			
		if ((isGroupedByDiscussion && comment.inReplyTo == null) || !isGroupedByDiscussion) {
			this._isRefreshRequest = true;
			this.update(this.page, this.pageSize, this.limit, this.size + 1, this.item);
			this._isRefreshRequest = false;
		}
	},
	
	_commentDeleted: function(comment) {
		var isGroupedByDiscussion = this.view.isGroupedByDiscussion();
		
		if ((isGroupedByDiscussion && comment.inReplyTo == null) || !isGroupedByDiscussion) {
			this._isRefreshRequest = true;
			this.update(this.page, this.pageSize, this.limit, this.size - 1, this.item);
			this._isRefreshRequest = false;
		}
	},
	
	_visualize: function() {
		if (0 < this.page || this.pageSize < this.size) {
			this.view._showPagingControl();
		} else {
			this.view._hidePagingControl();
		}
	}
});


})();