/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.commenting.FilterWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.cookie");

dojo.require("rmps.core.util.DateUtils");
dojo.require("rmps.core.util.ContextUtils");

dojo.require("rmps.ui.user.UserSelectionBox");
dojo.require("rmps.ui.baseline.BaselineSelectionBox");
dojo.require("rmps.ui.commenting.SnapshotFilterWidget");

dojo.requireLocalization("rmps.ui.commenting", "FilterWidgetMessages");

dojo.declare("rmps.ui.commenting.FilterWidget", [dijit._Widget, dijit._Templated], {
	
	templateString: dojo.cache("rmps.ui", "commenting/templates/FilterWidget.html"),
	widgetsInTemplate: true,
	
	messages: dojo.i18n.getLocalization("rmps.ui.commenting", "FilterWidgetMessages"),
	
	// attach points
//	_authorSelectInput: null,
//	_dateRangeInput: null,
	
	// mixin properties
	commentingWidget: null,
	snapshotFilterEnabled: true,
	
	// private properties
	_creator: null,
	_dateRange: "all",
	// initial selection
	_initialAuthor: null,
	_initialDateRange: null,
	_initialSnapshot: null,
	
	postMixInProperties: function() {

		this._init = true;
	
		// load saved filter state
		var saveState = dojo.cookie(this._getSaveStateCookieName());

		if ( saveState ) {
			saveState = dojo.fromJson(saveState);
			
			this._creator = saveState.creator;
			if ( this._creator ) {
				var temp = this._creator.match(/\/([^\/]+)$/);
				if (temp != null && temp[1]) {
					this._initialAuthor = temp[1];
				}
			}
			
			this._initialDateRange = saveState.date;
			
			if ( saveState.snapshot ) {
				this._initialSnapshot = saveState.snapshot;
			}
			
			return saveState;
		}
		return null;		
	},
	
	postCreate: function() {

		
		// create author selection
		this._authorSelectInput= new rmps.ui.user.UserSelectionBox({
			initialSelection: this._initialAuthor,
			onChange: dojo.hitch(this, this._onChange)
		});
		this.authorContainer.appendChild(this._authorSelectInput.domNode);
		
		// populate date range
		var dateSelectedIndex = 0;
		dojo.create("option", {value: "all"}, this._dateRangeInput)
					.appendChild(dojo.doc.createTextNode(this.messages.all));
		var du = rmps.core.util.DateUtils;
		for (var i = 0, r; r = du.ranges[i]; ++i) {
			if ( this._initialDateRange == r ) {
				dateSelectedIndex = i+1;
			}
			dojo.create("option", {value: r}, this._dateRangeInput)
					.appendChild(dojo.doc.createTextNode(du.rangeToString(r)));
		}
		this._dateRangeInput.selectedIndex = dateSelectedIndex;
		this._dateRange = this._dateRangeInput.options[dateSelectedIndex].value;
		
		this.connect(this._authorSelectInput, "onChange", this._onAuthorChanged);
		this.connect(this._dateRangeInput, "onchange", this._onDateChanged);
		
		// If the configuration is a snapshot we don't need the configuration filter as we will
		// only show the comments associated to the snapshot
		if(this.snapshotFilterEnabled && !rmps.core.util.ContextUtils.isSnapshot(jazz.app.context.get().baseline)){
			this.snapshotWidget= new rmps.ui.commenting.SnapshotFilterWidget({
				initialSelection: this._initialSnapshot,
				onChange: dojo.hitch(this, this._onChange)
			});
			this.snapshotContainer.appendChild(this.snapshotWidget.domNode);
		}
		this.commentingWidget.addFilter(this);
	},
	
	startup: function() {
		this._onChange();
		delete this._init;
	},
	
	refresh: function() {
		this._authorSelectInput.refresh();
	},
	
	_onAuthorChanged: function(user) {
		if (user === this._authorSelectInput.UNSPECIFIED) {
			this._creator = null;
		} else {
			this._creator = user.uri;
		}
		this._onChange();
	},
	
	_onDateChanged: function(e) {
		dojo.stopEvent(e);
		this.updateSelection();
	},
	
	updateSelection: function(){
		this._dateRange = this._dateRangeInput.options[this._dateRangeInput.selectedIndex].value;
		this._onChange();
	},
	
	// Filter contract methods
	
	reset: function() {
		this._reset = true;
		this._authorSelectInput.reset();
		if(this.snapshotWidget){
			this.snapshotWidget.reset();	
		}
		
		this._dateRange = "all";
		this._dateRangeInput.selectedIndex = 0;
		delete this._reset;
		this._onChange();
	},
	
	isEnabled: function() {
		return this._creator || this._dateRange !== "all" || (this.snapshotWidget&& this.snapshotWidget.isEnabled());
	},
	
	isFiltered: function(comment) {
		// return true if the comment should be hidden, false to be visible
		if ((!this._creator || this._creator === comment.creator)
			 && (this._dateRange === "all" || comment.created.getTime() >= rmps.core.util.DateUtils.rangeToDate(this._dateRange).getTime())
			 && (!this.snapshotWidget|| !this.snapshotWidget.isFiltered(comment))) {
			return false;
		}
		return true;
	},
	
	_getFilterState: function() {
		var saveState = {
				creator: this._creator,
				date: this._dateRange
				};
		if(this.snapshotWidget){
			saveState.snapshot = this.snapshotWidget._baseline;
		}

		return saveState;
	},
	
	_getSaveStateCookieName: function() {
		return this.declaredClass + "_saveState_" ;
	},
	
	_saveFilterState: function() {
		//save filter state
		dojo.cookie(this._getSaveStateCookieName(), dojo.toJson(this._getFilterState()));
	},
	
	_onChange: function() {
		if (this._reset) {
			return;
		}
		if (!this._init) {
			this._saveFilterState();
		}
		this.onChange(this.isEnabled());
	},
	
	onChange: function(enabled) {}
});