/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.commenting.GroupChronological");

dojo.require("rmps.ui.commenting.AbstractGroup");

dojo.declare("rmps.ui.commenting.GroupChronological", [rmps.ui.commenting.AbstractGroup], {
	
	"class": "rmps-ui-commenting-group-chronological",
	
	_firstEntry: null,
	
	addEntries: function() {
		if (this.commentEntries.length > 0) {
			this._firstEntry = this.commentEntries[0];
			dojo.addClass(this._firstEntry.domNode, "chronological-entry-first");
		}
		this.inherited(arguments);
	},
	
	addComment: function(comment) {
		var entry = this.createCommentEntry(comment);
		
		if (this.commentingWidget.isSortAscending()) {
			this.domNode.appendChild(entry.domNode);
			this.commentEntries.push(entry);
		} else {
			this.domNode.insertBefore(entry.domNode, this.domNode.firstChild);
			this.commentEntries.splice(0, 0, entry);
		}
		
		if (entry.comment.inReplyTo) {
			this.commentEntryRoots.push(entry);
		}
		this.commentEntriesHash[entry.comment.uri] = entry;
		this._updateFirstEntry();
		
		entry.startup();
	},
	
	_updateFirstEntry: function() {
		for (var i = 0, c; c = this.commentEntries[i]; ++i) {
			if (c.isVisible) {
				dojo.addClass(c.domNode, "chronological-entry-first");
				if (this._firstEntry && this._firstEntry !== c) {
					dojo.removeClass(this._firstEntry.domNode, "chronological-entry-first");
				}
				this._firstEntry = c;
				break;
			}
		}
	},
	
	deleteComment: function(comment) {
		this.inherited(arguments);
		if (this._firstEntry && this._firstEntry.comment === comment) {
			delete this._firstEntry;
			this._updateFirstEntry();
		}
	},
	
	filterComments: function(filtered, unfiltered) {
		this.inherited(arguments);
		this._updateFirstEntry();
	},
	
	createCommentEntry: function(_comment) {
		var entry = this.inherited(arguments);
		dojo.addClass(entry.domNode, "rmps-ui-commenting-group-chronological-entry");
		dojo.addClass(entry.pane, "rmps-ui-commenting-group-chronological-entry-pane");
		return entry;
	},
	
	prepareEntries: function() {
		if (this.commentEntries.length == 0) {
			return;
		}
		
		this.sortEntries(this.commentEntries, this.commentingWidget.isSortAscending(), function(entry) {
			return entry.comment.created;
		});
		this.inherited(arguments);
	}
});