/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.commenting.GroupDiscussion");

dojo.require("rmps.ui.commenting.CommentEntry");
dojo.require("rmps.ui.commenting.AbstractGroup");
dojo.require("rmps.ui.EllipsisLabel");

dojo.requireLocalization("rmps.ui.commenting", "GroupDiscussionMessages");

dojo.require("rmps.core.util.DateUtils");

var DateUtils = rmps.core.util.DateUtils;

var INDENT_STEP = 12;
var MIN_REPLY_WIDTH = 150;

var getLastUpdated = function (entry) {
	var lastUpdated = entry.comment.created;
	for (var i = 0; i < entry.replies.length; ++i) {
		var replyLastUpdated = getLastUpdated(entry.replies[i]);
		if (replyLastUpdated > lastUpdated) {
			lastUpdated = replyLastUpdated;
		}
	}
	return lastUpdated;
};

dojo.declare("rmps.ui.commenting.GroupDiscussion", [rmps.ui.commenting.AbstractGroup], {
	
	"class": "rmps-ui-commenting-group-discussion",
	
	// private properties
	_bars: null, //[]
	_isExpandCollapse: false,
	
	messages: dojo.i18n.getLocalization("rmps.ui.commenting", "GroupDiscussionMessages"),
	
	postCreate: function() {
		this.inherited(arguments);
		this.connect(this.commentingWidget, "onLoad", this._expandComments);
		this.connect(this.commentingWidget, "onExpandAllComments", this._expandAll);
		this.connect(this.commentingWidget, "onCollapseAllComments", this._collapseAll);
	},
	
	createEntries: function() {
		this._bars = [];
		this.inherited(arguments);
	},
	
	resize: function(size) {
		if (!this._isExpandCollapse && this._bars) {
			var newSize = size ? {w: size.w } : {w: this.domNode.offsetWidth};
			for (var i = 0; i < this._bars.length; ++i) {
				if (this._bars[i].isVisible) {
					this._bars[i].resize(newSize);
				}
			}
		}
	},
	
	selectionChanged: function(selection, previousSelection, selected, deselected) {
		for (var i = 0; i < selection.length; ++i) {
			var entry = this.commentEntriesHash[selection[i].uri];
			if (entry) {
				var root = entry.getRoot();
				if (!root.bar.isOpen) {
					root.bar.open();
				}
			}
		}
		
		this.inherited(arguments);
	},
	
	addEntries: function() {
		for (var i = 0; i < this.commentEntryRoots.length; ++i) {
			this.domNode.appendChild(this.commentEntryRoots[i].bar.domNode);
			this.commentEntryRoots[i].bar.startup();
		}
	},
	
	addComment: function(comment) {
		var entry = this.createCommentEntry(comment);
		this.commentEntriesHash[entry.comment.uri] = entry;
		
		var bar;
		
		// setup relationships
		var parent = this.commentEntriesHash[entry.comment.inReplyTo];
		if (parent) {
			entry.parent = parent;
			
			var root = entry.getRoot();
			bar = root.bar;
			
			if (this.commentingWidget.isSortAscending() && parent.replies.length > 0) {
				dojo.place(entry.domNode, parent.replies[parent.replies.length - 1].domNode, "after");
				parent.replies.push(entry);
				this.commentEntries.splice(dojo.indexOf(this.commentEntries, parent.replies[parent.replies.length - 1]) + 1, 0, entry);
			} else {
				dojo.place(entry.domNode, parent.domNode, "after");
				parent.replies.splice(0, 0, entry);
				this.commentEntries.splice(dojo.indexOf(this.commentEntries, parent) + 1, 0, entry);
			}
			
			entry.startup();
			
			var rootIdx = dojo.indexOf(this.commentEntryRoots, root);
			
			var startIdx = dojo.indexOf(this.commentEntries, root);
			var endIdx;
			
			if (this.commentEntryRoots.length == rootIdx + 1) {
				endIdx = this.commentEntries.length - 1;
			} else {
				endIdx = dojo.indexOf(this.commentEntries, this.commentEntryRoots[rootIdx + 1]);
			}
			
			// reposition the bar
			var rootsEntries;
			if (this.commentingWidget.isSortAscending()) {
				this.domNode.appendChild(bar.domNode);
				
				if (root != this.commentEntryRoots[this.commentEntryRoots.length - 1]) {
					this.commentEntryRoots.splice(dojo.indexOf(this.commentEntryRoots, root), 1);
					this.commentEntryRoots.push(root);
					
					rootsEntries = this.commentEntries.splice(startIdx, endIdx - startIdx + 1);
					this.commentEntries = this.commentEntries.concat(rootsEntries);
				}
				
			} else if (this.commentEntryRoots[0] != root) {
				this.domNode.insertBefore(bar.domNode, this.domNode.firstChild);
					
				this.commentEntryRoots.splice(dojo.indexOf(this.commentEntryRoots, root), 1);
				this.commentEntryRoots.splice(0, 0, root);
				
				rootsEntries = this.commentEntries.splice(startIdx, endIdx - startIdx + 1);
				this.commentEntries = rootsEntries.concat(this.commentEntries);
			}
			
			bar.refresh();
			
		} else {
			bar = entry.bar;
			
			// insert the bar
			if (this.commentingWidget.isSortAscending()) {
				this.domNode.appendChild(bar.domNode);
				this.commentEntryRoots.push(entry);
				this.commentEntries.push(entry);
				
			} else {
				this.domNode.insertBefore(bar.domNode, this.domNode.firstChild);
				this.commentEntryRoots.splice(0, 0, entry);
				this.commentEntries.splice(0, 0, entry);
			}
			
			bar.startup();
		}
		
		bar.open();
	},
	
	refreshComment: function(comment) {
		var entry = this.commentEntriesHash[comment.uri];
		if (entry) {
			entry.refresh();
			if (entry.getRoot().bar.isVisible) {
				entry.getRoot().bar.resize();
			}
		}
	},
	
	deleteComment: function(comment) {
		var entry = this.commentEntriesHash[comment.uri];
		this.inherited(arguments);
		if (entry && entry.bar) {
			this._bars.splice(dojo.indexOf(this._bars, entry.bar), 1);
			// do not call #destroyRecursive()
			entry.bar.destroy();
		}
	},
	
	filterComments: function(filtered, unfiltered) {
		if (filtered) {
			for (var i = 0, f; f = filtered[i]; ++i) {
				var entry = this.commentEntriesHash[f.uri];
				if (entry) {
					entry.getRoot().bar.setVisibility(false);
					entry.setFiltered(true);
				}
			}
		}
		if (unfiltered) {
			for (var i = 0, uf; uf = unfiltered[i]; ++i) {
				var entry = this.commentEntriesHash[uf.uri];
				if (entry) {
					entry.getRoot().bar.setVisibility(true);
					entry.setFiltered(false);
				}
			}
		}
	},
	
	_expandComments: function() {
		var expandedComments = this.commentingWidget.getExpandedComments();
		if (expandedComments.length > 0) {
			this._isExpandCollapse = true;
			for (var i = 0, ec; ec = expandedComments[i]; ++i) {
				var entry = this.commentEntriesHash[ec];
				if (entry && entry.isRoot()) {
					entry.bar._doOpen();
				}
			}
			this._isExpandCollapse = false;
			this.resize();
		}
	},
	
	_expandAll: function() {
		this._isExpandCollapse = true;
		for (var i = 0; i < this.commentEntryRoots.length; ++i) {
			this.commentEntryRoots[i].bar._doOpen();
		}
		this._isExpandCollapse = false;
		this.resize();
	},
	
	_collapseAll: function() {
		var hasScrollBars = this._hasScrollBars();
		this._isExpandCollapse = true;
		for (var i = 0; i < this.commentEntryRoots.length; ++i) {
			this.commentEntryRoots[i].bar._doClose();
		}
		this._isExpandCollapse = false;
		if (hasScrollBars != this._hasScrollBars()) {
			this.resize();
		}
	},
	
	_hasScrollBars: function() {
		return this.domNode.parentNode.clientHeight < this.domNode.parentNode.scrollHeight;
	},
	
	createCommentEntry: function(_comment) {
		var entry; 
		if (_comment.inReplyTo) {
			entry = new rmps.ui.commenting.DiscussionReply({
				group: this,
				comment: _comment,
				showResourceLabel: false,
				isResourceLabelLink: this.commentingWidget.isResourceLabelLink 
			});
		} else {
			entry = new rmps.ui.commenting.DiscussionRoot({
				group: this,
				comment: _comment,
				showResourceLabel: this.commentingWidget.showResourceLabel,
				isResourceLabelLink: this.commentingWidget.isResourceLabelLink
			});
			var bar = new rmps.ui.commenting.DiscussionBar({
				rootEntry: entry,
				group: this
			});
			entry.bar = bar;
			this._bars.push(bar);
		}
		return entry;
	},
	
	prepareEntries: function() {
		if (this.commentEntries.length == 0) {
			return;
		}
		
		this.commentEntries = [];
		this.sortEntries(this.commentEntryRoots, this.commentingWidget.isSortAscending(), getLastUpdated);
		
		for (var i = 0; i < this.commentEntryRoots.length; ++i) {
			this._prepareReplyEntries(this.commentEntryRoots[i]);
		}
	},
	
	_prepareReplyEntries: function(parent) {
		this.commentEntries.push(parent);
		if (parent.replies.length > 0) {
			this.sortEntries(parent.replies, false, function(entry) {
				return entry.comment.created;
			});
			var replies = parent.replies;
			for (var i = 0; i < replies.length; ++i) {
				this._prepareReplyEntries(replies[i]);
			}
		}
	}
});

dojo.declare("rmps.ui.commenting.DiscussionEntry", rmps.ui.commenting.CommentEntry, {
	
	postCreate: function() {
		this.inherited(arguments);
		this.defaultBodyLength = this.details.bodyLength;
	},
	
	setFiltered: function(filtered) {
		if (this._isFiltered != filtered) {
			this._isFiltered = filtered;
			/* 
			 * 18274: Font for user name is blurry in comments view in IE 7 and 8
			 * 
			 * dojo._setOpacity(this.domNode, filtered ? 0.5 : 1); 
			 * 
			 */
			if (filtered) {
				this.details.bodyLength = 0;
			} else {
				this.details.bodyLength = this.defaultBodyLength;
			}
			this.refresh();
		}
	}
});

dojo.declare("rmps.ui.commenting.DiscussionReply", rmps.ui.commenting.DiscussionEntry, {
	
	// private propeties
	_indent: 0,
	_idealIndent: INDENT_STEP,
	
	postCreate: function() {
		this.inherited(arguments);
		dojo.addClass(this.domNode, "discussion-entry-reply");
	},
	
	startup: function() {
		var parent = this.parent;
		while (parent = parent.parent) {
			this._idealIndent += INDENT_STEP;
		}
		
		var bar = this.getRoot().bar;
		this.connect(this.details, "_timestampChanged", dojo.hitch(bar, bar.refreshLastUpdated));
		this.connect(this.details, "refresh", this._detailsRefreshed);
		
		this.inherited(arguments);
	},
	
	resize: function(size) {
		this._calculateIndent();
		this.inherited(arguments);
	},
	
	_trim: function(str) {
		return str.replace(/^(\s+|RE:)*|(\s+)$/ig, '');
	},
	
	_calculateIndent: function() {
		var width = this.details.domNode.offsetWidth;
		if (this._indent < this._idealIndent || width < MIN_REPLY_WIDTH) {
			var diff = width - MIN_REPLY_WIDTH;
			this._indent = Math.min(Math.max(this._indent + diff, INDENT_STEP), this._idealIndent);
			this.containerNode.style.paddingLeft = this._indent + "px";
		}
	},
	
	_detailsRefreshed: function() {
		var subject = this._trim(this.comment.subject);
		var parentSubject = this._trim(this.parent.comment.subject);
		if (subject == parentSubject) {
			this.details._subject.domNode.style.display = "none";
		}
		
		var root = this.getRoot();
		root.bar.refresh();
	}
});

dojo.declare("rmps.ui.commenting.DiscussionRoot", rmps.ui.commenting.DiscussionEntry, {
	
	// mixin properties
	bar: null,
	
	// properties
	isOpen: false,
	
	isRoot: function() {
		return true;
	},
	
	postCreate: function() {
		this.inherited(arguments);
		dojo.addClass(this.domNode, "discussion-entry-root");
		this.details._subject.domNode.style.display = "none";
	},
	
	startup: function() {
		this.connect(this.details, "_timestampChanged", dojo.hitch(this.bar, this.bar.refreshLastUpdated));
		this.connect(this.details, "refresh", this._detailsRefreshed);
		this.inherited(arguments);
	},
	
	setExpanded: function(expanded) {
		if (expanded) {
			this.group.commentingWidget.markCommentToExpand(this.comment);
		} else {
			this.group.commentingWidget.unmarkCommentToExpand(this.comment);
		}
	},
	
	_detailsRefreshed: function() {
		this.bar.refresh();
	}	
});

dojo.declare("rmps.ui.commenting._ShowMore", [dijit._Widget], {
	
	buildRendering: function() {
		this.domNode = dojo.create("a", {href: "#", "class": "showMore"}, null);
		this.domNode.appendChild(dojo.doc.createTextNode(this.messages.showMore));
	},
	
	postCreate: function() {
		this.connect(this.domNode, "onclick", function(e) {
			dojo.stopEvent(e);
			// TODO
		});
	}
});

dojo.declare("rmps.ui.commenting.ShowMoreEntry", rmps.ui.commenting.AbstractCommentEntry, {
	// properties
	_showMore: null,
		
	postCreate: function() {
		this._showMore = new rmps.ui.commenting._ShowMore();
		this.containerNode.appendChild(this._showMore.domNode);
		this.actionBar.style.visibility = "hidden";
		this.inherited(arguments);
	},

	showActionBar: function(show) {
		// do nothing
	},
	
	destroy: function() {
		if (this._showMore) {
			this._showMore.destroyRecursive();
		}
		this.inherited(arguments);
	}
});

dojo.require("dojo.html");
dojo.require("rmps.ui.EllipsisLabelContainer");
dojo.require("dojo.fx");

dojo.declare("rmps.ui.commenting.DiscussionBar", [dijit._Widget, dijit._Templated], {
	
	templateString: dojo.cache("rmps.ui", "commenting/templates/GroupDiscussionBar.html"),
	widgetsInTemplate: true,
	
	// attach points
	_header: null,
	_headerContainer: null,
	_title: null,
	_replies: null,
	_lastUpdated: null,
	_titleLastUpdatedContainer: null,
	
	// mixin properties
	rootEntry: null,
	group: null,
	
	// properties
	isOpen: false,
	isVisible: true,
	
	// private properties
	_openAnimation: null,
	_closeAnimation: null,
	
	postCreate: function() {
		this._createOpenAnimation();
		this._createCloseAnimation();
	},
	
	_createOpenAnimation: function() {
		var groupHasScrollBars;
				
		var args = {
			node: this.containerNode,
			duration: 300,
			beforeBegin: dojo.hitch(this, function() {
				groupHasScrollBars = this.group._hasScrollBars();
				this.containerNode.style.height = "1px";
				this.containerNode.style.display = "";
				this.layout();
			}),
			onEnd: dojo.hitch(this, function() {
				this._doOpen();
				if (groupHasScrollBars != this.group._hasScrollBars()) {
					this.group.resize();
				} else {
					this.layout();
				}
			})
		};
		this._openAnimation = dojo.fx.wipeIn(args);
	},
	
	_createCloseAnimation: function() {
		var groupHasScrollBars;
		
		var args = {
			node: this.containerNode,
			duration: 300,
			beforeBegin: dojo.hitch(this, function() {
				groupHasScrollBars = this.group._hasScrollBars();
			}),
			onEnd: dojo.hitch(this, function() {
				this._doClose();
				if (groupHasScrollBars != this.group._hasScrollBars()) {
					this.group.resize();
				}
			})
		};
		this._closeAnimation = dojo.fx.wipeOut(args);
	},
	
	startup: function() {
		if (this._started) {
			return;
		}

		var allReplies = this.rootEntry.getAllReplies();
		
		this.containerNode.appendChild(this.rootEntry.domNode);
		
		for (var i = 0; i < allReplies.length; ++i) {
			this.containerNode.appendChild(allReplies[i].domNode);
		}

		this.rootEntry.startup();
		for (var key in allReplies) {
			allReplies[key].startup();
		}

		this.inherited(arguments);
		
		this.refresh();
	},
	
	resize: function(size) {
		if (this.isVisible) {
			if (this.isOpen) {
				this.layout(size);
			}
			
			var width = dojo.contentBox(this._header).w;
			this._titleLastUpdatedContainer.resize({ w: width });
//			this._titleLastUpdatedContainer.resize({ w: this._headerContainer.offsetWidth });
		}
	},
	
	layout: function(size) {
		var newSize = size ? {w: size.w } : {w: this.containerNode.offsetWidth };
		if (this.rootEntry.isVisible) {
			this.rootEntry.resize(newSize);
		}
		var allReplies = this.rootEntry.getAllReplies();
		for (var i = 0; i < allReplies.length; ++i) {
			if (allReplies[i].isVisible) {
				allReplies[i].resize(newSize);
			}
		}
	},
	
	refresh: function() {
		if (!this._started) {
			return;
		}
		
		// # replies
		var allReplies = this.rootEntry.getAllReplies();
		dojo.html._setNodeContent(this._replies, dojo.doc.createTextNode('(' + (allReplies.length + 1) + ')'), true);
		
		// title
		this._title.setLabel(this.rootEntry.comment.subject);
		
		// last updated
		this.refreshLastUpdated();
	},
	
	refreshLastUpdated: function(dateRange) {
		if (!this._started) {
			return;
		}
		if (dateRange) {
			this._lastUpdated.setLabel(dateRange);
		} else {
			this._lastUpdated.setLabel(DateUtils.getDateDiff(getLastUpdated(this.rootEntry)));
		}
	},
	
	// bar events
	
	_onHeaderClicked: function(event) {
		if (this.isOpen) {
			this.close(true);
		} else {
			this.open(true);
		}
	},
	
	_onHeaderKeyPress: function(e){
	  var keyCode = (e.keyCode ? e.keyCode : e.which);
	  if(keyCode === dojo.keys.ENTER){
	      this._onHeaderClicked(e);
	  }
	},
	
	_onHeaderMouseOver: function(event) {
		dojo.addClass(this._header, "bar-header-hover");
	},
	
	_onHeaderMouseOut: function(event) {
		dojo.removeClass(this._header, "bar-header-hover");
	},
	
	setVisibility: function(visible) {
		if (this.isVisible != visible) {
			this.isVisible = visible;
			if (this.isVisible) {
				this.domNode.style.display = "";
			} else {
				this.domNode.style.display = "none";
			}
		}
	},
	
	open: function(animate) {
		this.rootEntry.setExpanded(true);
		if (animate) {
			this._closeAnimation.stop();
			this._openAnimation.play();
		} else {
			this._openAndResize();
		}
	},
	
	close: function(animate) {
		this.rootEntry.setExpanded(false);
		if (animate) {
			this._openAnimation.stop();
			this._closeAnimation.play();
		} else {
			this._closeAndResize();
		}
	},
	
	_doOpen: function() {
		this.isOpen = true;
		this.containerNode.style.height = "auto";
		this.containerNode.style.display = "";
	},
	
	_openAndResize: function() {
		var groupHasScrollBars = this.group._hasScrollBars();
		this._doOpen();
		if (groupHasScrollBars != this.group._hasScrollBars()) {
			this.group.resize();
		} else {
			this.layout();
		}
	},
	
	_doClose: function() {
		this.isOpen = false;
		this.containerNode.style.display = "none";
		this.containerNode.style.height = "auto";
	},
	
	_closeAndResize: function() {
		var groupHasScrollBars = this.group._hasScrollBars();
		this._doClose();
		if (groupHasScrollBars != this.group._hasScrollBars()) {
			this.group.resize();
		}
	}
});
})();