/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.commenting.SelectionCommentingWidget");

dojo.require("rmps.ui.commenting.CommentingWidget");
dojo.require("rmps.ui.commenting.SelectionFilterWidget");
dojo.require("rmps.ui.commenting.FilterWidget");

dojo.declare("rmps.ui.commenting.SelectionCommentingWidget", rmps.ui.commenting.CommentingWidget, {
	
	selections: null,
	
	getFilterWidget: function(){
		var filter=  new rmps.ui.commenting.ExFilterWidget({
			commentingWidget: this,
			doCreate: dojo.hitch(this, this.doCreateFilter),
			applySelectionFilter: dojo.hitch(this, this.applySelectionFilter)
		});	
		
		/*a way to delegate Filterwidget#_onChange */	
		this.updateSelection= dojo.hitch(filter, filter.updateSelection);
		
		return filter;
	},
	
	/*To be implemnted by other places to apply more filter conditions */
	applySelectionFilter: function(comment, dateRange){},
	
	doCreateFilter: function(dateRangeInput){
		/*To be called by other places to add more logic for filters */
	},
	
	updateSelection: function(){/*a way to delegate Filterwidget#_onChange */},
	
	_createToolbarWidget: function() {
		return new rmps.ui.commenting.SelectionCommentingWidget._TitleBar({
			commentingWidget: this
		});
	},
	
	addSelection: function(selection){
		this.createFilters();
		this._filterWidget.selectionFilterWidget.addSelection(selection);
		this.selections.push(selection);
		this.refresh();
	},
	
	removeSelection: function(resourceUri){
		this.createFilters();
		this._filterWidget.selectionFilterWidget.removeSelection(resourceUri);
		this.removeByResourceUri(resourceUri);
		this.refresh();
	},
	
	removeByResourceUri: function(resourceUri){
		for(var index=0 ;index< this.selections.length; index++){
			var cur= this.selections[index];
			if(cur.uri== resourceUri){
				this.selections.splice(index, 1);
			}
		}
	}
	
});


dojo.declare("rmps.ui.commenting.ExFilterWidget", rmps.ui.commenting.FilterWidget, {
	
	// initial selection
	_initialResource: null,

	postMixInProperties: function() {
		var saveState = this.inherited(arguments);
		
		if ( saveState ) {
			this._initialResource = saveState.resource;
		}
		
		return saveState;
	},
	
	postCreate: function() {
		this.inherited(arguments);

		this.doCreate(this._dateRangeInput,this._initialDateRange);
		this._dateRange = this._dateRangeInput.options[this._dateRangeInput.selectedIndex].value;
		
		this.selectionFilterWidget = new rmps.ui.commenting.SelectionFilterWidget({
			initialSelection: this._initialResource,
			commentingWidget: this.commentingWidget,
			onChange: dojo.hitch(this, this._onChange)
		});
		this.domNode.appendChild(this.selectionFilterWidget.domNode);
	},
	
	_getFilterState: function() {
		var saveState = this.inherited(arguments);
		
		saveState.resource = this.selectionFilterWidget.currentSelection;

		return saveState;
	},
	
	_getSaveStateCookieName: function() {
		return this.declaredClass + "_saveState_";
	},
	
	reset: function() {
		this._reset = true;
		this.selectionFilterWidget.reset();
		this.inherited(arguments);
	},
	
	isEnabled: function() {
		return this.selectionFilterWidget.isEnabled() || this.inherited(arguments);
	},
	
	/**
	 * <p>
	 * Determines if a comment should be filtered out by answering the question "Is this comment filtered?".
	 * If <code>TRUE</code> then the comment is filtered out.
	 * </p>
	 * 
	 * @param {Comment} comment The comment in question
	 * @return <code>TRUE</code> if the comment should be filtered out (does not match the filter),
	 * 		or <code>FALSE</code> if the comment should not be filtered out (matches the filter.)
	 */
	isFiltered: function(comment) {
		// return true if the comment should be hidden, false to be visible
		if ((!this._creator || this._creator === comment.creator)
			&& (!this.snapshotWidget|| !this.snapshotWidget.isFiltered(comment))
			&& !this.selectionFilterWidget.isFiltered(comment)){
			if ( this._dateRange === "all" ) {
				return false;
			} else {
				var range = rmps.core.util.DateUtils.rangeToDate(this._dateRange);
				if(range){
					if(comment.created.getTime() >= range.getTime()){
						return false;
					}
				} else {
					return this.applySelectionFilter(comment, this._dateRange);
				}
			}
		}
		return true;
	},
	
	/*To be implemnted by other places to apply more filter conditions */
	applySelectionFilter: function(comment, dateRange){
		return false;
	},
	
	doCreate: function(dateRangeInput){/*To be invoked by other callers */}
});


dojo.declare("rmps.ui.commenting.SelectionCommentingWidget._TitleBar", rmps.ui.commenting.CommentingToolbarMenu, {
	
	postCreate: function() {	
		this._createNewButton.attr("style", "display: none;");
		this.inherited(arguments);
	},
	
	filterEnabled: function(state){
		/*Filter always persisits */
		this._filterButton.attr("disabled", false);
	}
	
});

})();