/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.commenting.SelectionFilterWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");

dojo.require("rmps.ui.user.UserSelectionBox");

dojo.requireLocalization("rmps.ui.commenting", "SelectionFilterWidgetMessages");

dojo.declare("rmps.ui.commenting.SelectionFilterWidget", [dijit._Widget, dijit._Templated], {
	
	templateString: dojo.cache("rmps.ui", "commenting/templates/SelectionFilterWidget.html"),
	widgetsInTemplate: true,
	
	messages: dojo.i18n.getLocalization("rmps.ui.commenting", "SelectionFilterWidgetMessages"),
	
	currentSelection: null,
	
	postCreate: function() {
		// Add "All" represented by uri as null
		this.addSelection({
			uri: null,
			shortName: this.messages.all
		});
		
		var selections= this.commentingWidget.selections;
		for (var i = 0; i< selections.length; ++i) {
			var selection= selections[i];
			this.addSelection(selection);
		}
		
		this.connect(this._selectionInput, "onchange", this.onSelectionChanged);
	},
	
	addSelection: function(selection){
		var option = dojo.create("option", {value: selection.uri, title:selection.longName}, this._selectionInput);
		option.appendChild(dojo.doc.createTextNode(selection.shortName));
		if ( selection.uri == this.initialSelection ) {
			this.currentSelection = selection.uri;
			option.selected = true;
		}
	},
	
	removeSelection: function(resourceUri){
		var selection= this._getSelectionFor(resourceUri);
		var shouldRefresh= selection.index== this._selectionInput.selectedIndex;	/*Refresh if we are removing the current selection  */
		this._selectionInput.removeChild(selection);
		if(shouldRefresh){
			this.commentingWidget.refresh();
		}
	},
	
	onSelectionChanged: function(e) {
		dojo.stopEvent(e);
		// Update the current selection.  Might return the string "null" which is not the NULL object.  In this case, explicitly
		// set it to null.
		this.currentSelection = this._selectionInput.options[this._selectionInput.selectedIndex].value;
		if(this.currentSelection === "null"){
			this.currentSelection = null;
		} 
		this.onChange();
	},
	
	_getSelectionFor: function(resourceUri){
		var options= this._selectionInput.options;
		for(var index=0; index<options.length; index++){
			var option= options[index];
			if(option.value== resourceUri){
				return option;
			}
		}
	},
		
	// Filter contract methods
	
	reset: function() {
		this.currentSelection = null;
		this._selectionInput.selectedIndex = 0;
	},
	
	isEnabled: function() {
		return this.currentSelection;
	},
	
	/**
	 * <p>
	 * Determines if a comment should be filtered out by answering the question "Is this comment filtered?".
	 * If <code>TRUE</code> then the comment is filtered out.
	 * </p>
	 * 
	 * @param {Comment} comment The comment in question
	 * @return <code>TRUE</code> if the comment should be filtered out (does not match the filter),
	 * 		or <code>FALSE</code> if the comment should not be filtered out (matches the filter.)
	 */
	isFiltered: function(comment) {
		// return true if the comment should be hidden, false to be visible
		if (!this.currentSelection || this.currentSelection === comment.resourceUri ) {
			return false;
		}
		return true;
	},
	
	onChange: function(enabled) {}
});