/*******************************************************************************
 * Licensed Materials - Property of IBM
 *
 * com.ibm.xtools.rmps.web.core
 *
 * Copyright IBM Corporation 2009, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.commenting.range.Factory");

dojo.require("rmps.core.commenting.range.Default");

dojo.require("jazz.core.RegistryFactory");
dojo.require("jazz.core.loader");

var load_sync = jazz.core.loader.load_sync;
var EXT_PT = "com.ibm.xtools.rmps.web.core.commentRanges";
var E_RANGE = "range";
var A_JSCLASS = "jsclass";
var A_NAMESPACE = "namespace";

/*
 * Cache of range namespace to range class.
 */
var cache = null;

rmps.core.commenting.range.Factory = {

	_init: function() {
		cache = {};
		var extRegistry = jazz.core.RegistryFactory.getRegistry();
		var extPoint = extRegistry.getExtensionPoint(EXT_PT);

		dojo.forEach(extPoint.getExtensions(), function(extension) {
			dojo.forEach(extension.getConfigurationElements(), function(config) {
				if (E_RANGE == config.getName()) {
					var className = config.getAttribute(A_JSCLASS);
					var namespace = config.getAttribute(A_NAMESPACE);
					if (className && namespace) {
						cache[namespace] = className;
					}
				}
			});
		});
	},

	create: function(range) {
		// supports only a single range
		if (cache == null) {
			this._init();
		}
		for (var key in range) {
			if (key.toLowerCase() == "xmlns" || key.toLowerCase().indexOf("xmlns:") == 0) {
				var namespace = range[key];
				var className = cache[namespace];
				if (className) {
					var classConstructor = dojo.getObject(className);
					if (!classConstructor) {
						load_sync(className);
						classConstructor = dojo.getObject(className);
					}
					if (classConstructor) {
						var rangeObject = new classConstructor();
						var prefix = key.split(":");
						if (prefix.length == 2) {
							prefix = prefix[1];
						} else {
							prefix = "";
						}
						rangeObject.parse(range, prefix);
						return rangeObject;
					} else {
						// failed to find range
						delete cache[namespace];
					}
				}
			}
		}
		return new rmps.core.commenting.range.Default(range);
	}
};

})();