/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.common.SelectionBox");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.html");

dojo.require("rmps.core.error");

dojo.requireLocalization("rmps.ui.common", "SelectionBoxMessages");
var messages = dojo.i18n.getLocalization("rmps.ui.common", "SelectionBoxMessages");

var unspecifiedValue = "";
var UNSPECIFIED = {};

dojo.declare("rmps.ui.common.SelectionBox", [dijit._Widget, dijit._Templated], {

	templateString: '<select class="rmps-ui-common-selection-box" dojoAttachEvent="onchange:_onChange"></select>',

	// mixin
	idProp: "id",
	nameProp: "name",
	unspecifiedName: messages.any,

	widgetsInTemplate: false,
	
	// properties
	initialSelection: unspecifiedValue,
	defaultSelection: unspecifiedValue,
	projectScope: true,
	
	// private properties
	_hash: null,
	_selectedValue: null,
	
	postCreate: function() {
		if (!this.initialSelection) {
			this.initialSelection = unspecifiedValue;
		}
		if (!this.defaultSelection) {
			this.defaultSelection = unspecifiedValue;
		}
		this._selectedValue = this.initialSelection;
		this.refresh();
	},
	
	// Override
	// Elements should have 'id' property
	fetch: function() {
		var def = dojo.Deferred();
		def.errback(); // Noop
		return def;
	},
		
	refresh: function() {
		dojo.html._emptyNode(this.domNode);
		var deferred = this.fetch();
		deferred.addCallback(this, this._populate);
		deferred.addErrback(function(error) {
			if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
				rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
			} else {
				rmps.core.error.createError(error).dispatch();
			}
		});
	},
	
	reset: function() {
		var options = this.domNode.options;
		for (var i = 0; i < options.length; ++i) {
			var option = options[i];
			if (option.value == this.defaultSelection) {
				this.domNode.selectedIndex = i;
				if (this.defaultSelection == unspecifiedValue) {
					this.onChange(UNSPECIFIED);
				} else {
					this.onChange(this._hash[this.defaultSelection]);
				}
				break;
			}
		}
	},
	
	_populate: function(elements) {
		this._hash = {};
		
		elements.sort(this.sortingMethod);
		
		var prevSelectedValue = this._selectedValue;
		this._selectedValue = null;
		
		this.domNode.appendChild(this._createOption(unspecifiedValue, this.unspecifiedName, prevSelectedValue == unspecifiedValue));
		
		for (var i = 0; i < elements.length; ++i) {
			var element = elements[i];
			this._hash[element[this.idProp]] = element;
			this.domNode.appendChild(this._createOption(element[this.idProp], element[this.nameProp],  prevSelectedValue == element[this.idProp]));
		}
		
		if (this._selectedValue != prevSelectedValue) {
			if (!this._selectedValue || this._selectedValue == unspecifiedValue) {
				this.onChange(UNSPECIFIED);
			} else {
				this.onChange(this._hash[this._selectedValue.value]);
			}
		}
		
		this.domNode.style.width = "100%";
	},
	
	sortingMethod: function (e1, e2) { 
        //sorts alphabetically
        var a = e1.name.toLowerCase(),
            b = e2.name.toLowerCase();
        return (a > b) ? 1 : ((a < b) ? -1 : 0);
    },
	
	_createOption: function(value, label, selected) {
		var option = dojo.doc.createElement("option");
		option.value = value;
		option.appendChild(dojo.doc.createTextNode(label));
		if (selected) {
			option.setAttribute("selected", "");
			this._selectedValue = value;
		}
		return option;
	},
	
	_onChange: function(event) {
		dojo.stopEvent(event);
		this._selectedValue = this.domNode.value;
		if (this.domNode.value == unspecifiedValue) {
			this.onChange(UNSPECIFIED);
		} else {
			this.onChange(this._hash[this.domNode.value]);
		}
	},
	
	onChange: function(element) {
		
	}
});

})();