/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| Copyright IBM Corp. 2010, 2011.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

( function() {

dojo.provide( "rmps.graphics.compact.CompactRenderingDiagram" );

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.graphics.compact", "CompactRenderingViewerMessages");

messages = dojo.i18n.getLocalization("rmps.graphics.compact", "CompactRenderingViewerMessages");

dojo.declare( "rmps.graphics.compact.CompactRenderingDiagram", null, { 

   node: null,
   containerNode: null,
   movedimage: null,
   imageobject: null,
   
   lastX: null,
   lastY: null,
   
   /*We have to ensure that the moveable image should have at least a visible part on the
    * diagram container all the time*/
   minimumVisiblePart: 100,
   
   _DELTA:5,
   
   
   /*A dictionary for the mouse listers. So, we can disconnect any of them when required */
   mouseListeners: new dojox.collections.Dictionary(),
   
   moving: false,
   
   uid: null,
   rawimage: null,

   loadingnode: null, LOADED: false, imageArea: null, 
   
   constructor: function( attachpoint, containerNode, diagramURI){
      try {
	  	this.uid = diagramURI;
	    this.node = attachpoint;
	    this.containerNode= containerNode;
		 	 
		/*Hide scrollbars */
		var body= dojo.body();
		dojo.style(body, "overflow", "hidden");
		
		 dojo.doc.onmousemove = function(e) {
			return false; 
		 };
		
		this.addLoadingNode();
	    this.resize();
	    this.handleDiagramAssets(diagramURI);
	    
	    /* We have a onmouseup on the doc because we have to make usre that we encounter the mouse up
	     * event while we are outside the iframe */
		this.mouseListeners.add(this.getListenerId("doc", "onmouseup"), dojo.connect( dojo.doc, "onmouseup", dojo.hitch( this, "onMouseUp")));
      
      } catch (e) {
      		alert(e);
      }
      
   },
   
   addLoadingNode: function(){
   		if( this.LOADED == false ) {
         if( this.loadingnode == null) {
            this.loadingnode = document.createElement( "div" );
            this.loadingnode.style.padding = "30px";
            this.loadingnode.style.color = "#BBBBBB";
            this.loadingnode.style.fontSize = "2em";
            this.loadingnode.innerHTML = messages.loading;
            this.node.appendChild( this.loadingnode );
         }
      }
   },
   
 
   
   resize: function(){
      /*Empty Block */
   },

   addImage: function( iobject ){
      if( this.loadingnode != null ) {
         this.node.removeChild( this.loadingnode );
         this.loadingnode = null;
      }
       this.LOADED = true;
      
       /*Show the diagram image */
       this.imageArea.src =  this.uid;
       this.imageArea.width = this.imageobject.width;
	   this.imageArea.height = this.imageobject.height;
	   
	   
	   /*We have to make sure that the image container is fit to the summary section in the IFrame 
	    * We set the height/width in this part after we make sure that the table node, and other sectinos
	    * have been initialized and taken their places on the IFrame
	    * The image container width is the same as the IFrame, while its height is the same as the IFrame but
	    * we have to reduce the height of the table node.
	    * 
	    * Get the width from body (away from window frame element) as this will cause permission errors in 
	    * Firefox when it comes at different domains 
	    * */
	   var frameElement= dojo.body();
	   this.node.style.width =    frameElement.clientWidth-this._DELTA+ "px";
	   this.node.style.height =   frameElement.clientHeight-dojo.byId("tableNode").clientHeight-this._DELTA+"px";
	   
	   
	   this.imageArea.style.left= "0px";
	   this.imageArea.style.top= "0px";
	   
	   this.supportPanning();
	   
	   return this.imageArea;
   },
   
   getListenerId: function(nodeString, eventString){
   		return (nodeString+"::"+eventString).toString();
   },
   
   
   supportPanning: function(){
   	   try 	{
	      /*We should be able to pan the image by panning the container as well */
	       this.mouseListeners.add(this.getListenerId("node", "onmousedown"), dojo.connect( this.node, "onmousedown", dojo.hitch( this, "onMouseDown")));
	       
	       /*The editor image should be pannable (moveable) */
	       this.mouseListeners.add(this.getListenerId("node", "onmousemove"), dojo.connect( this.node, "onmousemove", dojo.hitch( this, "onMouseMove" )));
   	   } catch (e) {
   	   	    alert(e);
   	   }
   	  
   },
   
   onMouse: function(e){
   		return false;
   },
   
   onMouseDown: function(e){
      try {
		this.lastX= e.clientX;
      	this.lastY= e.clientY;
      	this.moving= true;
      	this.node.style.cursor = 'move';      	
      } catch (exception) {
      	alert(exception);
      }
    },
    
    onMouseUp: function(e){
		try {
			this.resetMouse();	    		
		} catch (exception) {
			alert(exception);
		}
    },
    
    resetMouse: function(){
    	this.moving= false;
		this.node.style.cursor = '';
    },
	
	onMouseMove: function(e){
		try {
			if(this.moving){
				/*Get Current Image Coordinates */
				var xPosition= parseInt(this.imageArea.style.left.substring(0, this.imageArea.style.left.length-2));
				var yPosition= parseInt(this.imageArea.style.top.substring(0, this.imageArea.style.top.length-2));
	
				/*Know how much x,y the image is expected to be moved  */
				var xDiff= this.lastX- e.clientX;
				var yDiff= this.lastY- e.clientY;
				
				/*This is the expected left and top values that the image will be shifted. */
				var left= xPosition- xDiff;
			    var top= yPosition- yDiff;
				
				/*In the next lines, we will make sure that the image will not go beoynd the diagram container
				 * area */
				 
				/*Flags to determine the movement direction */
				var direction= {
					UP:yDiff>0,
					DOWN: yDiff<0,
					RIGHT:xDiff<0,
					LEFT:xDiff>0 
				};
				
				/*Handle Moving Left or Right (the movement cannot be both, so we handle them in one
				 * if else if block) */
				if(direction.LEFT && 
					left+this.imageArea.width<this.minimumVisiblePart){
						left= this.minimumVisiblePart- this.imageArea.width ;
				}else if(direction.RIGHT&& 
					left>this.imageArea.width+this.minimumVisiblePart){
						left= this.imageArea.width+ this.minimumVisiblePart;
				}
				
				
				/*Handle Moving up or down (the movement cannot be both, so we handle them in one
				 * if else if block) */
				if(direction.UP&& 
					top< this.minimumVisiblePart- this.imageArea.height){
						top= this.minimumVisiblePart- this.imageArea.height;
				}else if(direction.DOWN&&
					top> this.minimumVisiblePart){
						top= this.minimumVisiblePart;
				}
				
				
				/*Set the new position for the image */
				this.imageArea.style.left= left+"px";
			    this.imageArea.style.top= top+"px";
				
				/*Keep track of the new last coordinates for the next move action */
				this.lastX= e.clientX;
				this.lastY= e.clientY;		
			}		
		} catch (exception) {
			alert(exception);
		}

    },
    
    
   handleImage: function(){
      this.imageobject = { x:0, y:0, 
      					   src: this.rawimage.src, height: this.rawimage.height, 
      					   width: this.rawimage.width };
      this.movedimage = this.addImage( this.imageobject );
    },
           
   handleDiagramAssets: function( resourceUri ){
      /*Will be used to get the required image dim */
      
      this.rawimage = new Image();
      this.rawimage.onload = dojo.hitch( this, "handleImage" );
      this.rawimage.src = resourceUri;   
      
      /*Create image area that will host the diagram image */
      this.imageArea = document.createElement("img");
	  this.imageArea.width = 1;		//Will be set later
	  this.imageArea.height = 1; 	//Will be set later
	  this.imageArea.src= resourceUri;
	  this.imageArea.id= "imageArea";
	  this.imageArea.style.zIndex= "1";	/*To prevent having the pannable image overlayed on other places*/
	  this.imageArea.style.position= "relative";
	  this.imageArea.draggable= false;
	  this.imageArea.ondragstart = function(e) {return false;};
	  
	  this.disableSelection(this.imageArea);
	  this.node.appendChild(this.imageArea);
   },
   
   disableSelection: function(element){
		if(dojo.isMozilla){
			element.style.MozUserSelect = "none";
		}else if(dojo.isWebKit){
			element.style.KhtmlUserSelect = "none"; 
		}else if(dojo.isIE){
			element.unselectable = "on";
		}
	},
    
   releaseListeners: function(){
      /*Release all listeners */
      dojo.forEach(this.mouseListeners, function(listener) {
			dojo.disconnect(listener);	
	  });
	  this.mouseListeners.clear();
   },
      
   destroy: function(){
      this.releaseListeners();
      this.inherited(arguments);
   }
           
});

})();
