/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2011. All Rights
 * Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

(function() {
	dojo.provide("rmps.core.contexts.Context");
	dojo.require("rmps.core.Enum");
	dojo.require("dojo.i18n");
	dojo.require("rmps.core.util.ContextUtils");

	dojo.requireLocalization("rmps.core.contexts", "ContextMessages");

	var messages = dojo.i18n.getLocalization("rmps.core.contexts",
			"ContextMessages");

	// RDF tags
	var CONTEXT_NAME = "http://purl.org/dc/terms/title";
	var CONTEXT_DESCRIPTION = "http://purl.org/dc/terms/description";
	var CONTEXT_TIMESTAMP = "http://purl.org/dc/terms/issued";
	var CONTEXT_STATUS = "https://jazz.net/xmlns/rmps/baselines/1.0/status";
	
	// Context states
	rmps.core.contexts.ContextStates = new rmps.core.Enum({
		PENDING: new rmps.core.EnumLiteral({name: messages.Pending, value: "PENDING", icon: dojo.moduleUrl("rmps.core", "graphics/icons/state_pending.gif")}),
		IN_PROGRESS: new rmps.core.EnumLiteral({name: messages.In_Progress, value: "IN_PROGRESS", icon: dojo.moduleUrl("rmps.core", "graphics/icons/state_inProgress.gif")}),
		COMPLETED: new rmps.core.EnumLiteral({name: messages.Completed, value: "COMPLETED", icon: dojo.moduleUrl("rmps.core", "graphics/icons/state_completed.gif")}),
		ERROR: new rmps.core.EnumLiteral({name: messages.Error, value: "ERROR", icon: dojo.moduleUrl("rmps.core", "graphics/icons/state_error.gif")}),
		LOADING: new rmps.core.EnumLiteral({name: messages.Loading, value: "LOADING", icon: dojo.moduleUrl("rmps.core", "graphics/icons/state_inProgress.gif")})
	});
	
	var ContextUtils = rmps.core.util.ContextUtils;
	var States = rmps.core.contexts.ContextStates;
	
	dojo.declare("rmps.core.contexts.Context", null, {
		
		uri: null,
		name: null,
		date: null,
		description: null,
		status: null,
		type: null,
				
		/**
		 * Contexts can be created in two ways.  The "good" way is by a call to ContextUtils.getContextDeferred
		 * which will 
		 */
	constructor: function(uri){
		this.uri = uri;
	},
		
		/**
		 * Sets object properties based on supplied RDF data.
		 */
	parseRdfJson: function(uri, rdf){
		this.uri = uri;
		this.name = rdf[CONTEXT_NAME][0].value;
		this.timestamp = dojo.date.stamp.fromISOString(rdf[CONTEXT_TIMESTAMP][0].value);
		this.status = States.get(rdf[CONTEXT_STATUS][0].value);

		// description is optional
		if (rdf[CONTEXT_DESCRIPTION] != null) {
			this.description = rdf[BASELINE_DESCRIPTION][0].value;
		}
	}
		
	});
})();