/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.core.internal.WebUiInitializers");

//
// Clients must simply require this class in their application.
//

dojo.require("dojox.lang.aspect");
dojo.require("jazz.core.RegistryFactory");
dojo.require("jazz.core.loader");

var EXT_PT = "com.ibm.xtools.rmps.web.app.webUiInitializers";
var E_INITIALIZER = "initializer";
var A_JS_CLASS = "jsClass";
var A_JS_MODULE = "jsModule";
var A_RESOURCE_BINDING = "resourceBinding";
var A_CLASS_BINDING = "classBinding";

var resourceBindings = null;
var classBindings = null;

var queue = [];

//
// Reads the server extension. All unbound initializers are executed
// immediately while all bound initializers are cached.
//
var init = function() {
	dojo.forEach(jazz.core.RegistryFactory.getRegistry().getExtensionPoint(EXT_PT).getExtensions(), function(extension) {
		dojo.forEach(extension.getConfigurationElements(), function(config) {
			if (E_INITIALIZER == config.getName()) {
				var className = config.getAttribute(A_JS_MODULE);
				
				var binding = config.getChildren()[0];
				
				if (binding) {
					var bindings;
					if (binding.getName() === A_RESOURCE_BINDING) {
						if (!resourceBindings) {
							resourceBindings = {};
						}
						var module = binding.getAttribute(A_JS_MODULE);
						if (dojo.getObject(module)) {
							queue.push(className);
						} else {
							bindings = resourceBindings[module];
							if (!bindings) {
								resourceBindings[module] = bindings = [];
								
							}
							bindings.push(className);
						}
						
					} else if (binding.getName() === A_CLASS_BINDING) {
						if (!classBindings) {
							classBindings = {};
						}
						var module = binding.getAttribute(A_JS_CLASS);
						if (dojo.getObject(module)) {
							queue.push(className);
						} else {
							bindings = classBindings[module];
							if (!bindings) {
								classBindings[module] = bindings = [];
								
							}
							bindings.push(className);
						}
					}	
				} else {
					queue.push(className);
				}
			}
		});
	});
};

var handleBindings = function(originalModules, modules, bindings) {
	var i = 0, initializers, j;
	while (i < originalModules.length && bindings) {
		initializers = bindings[originalModules[i]];
		if (initializers) {
			for (j in initializers) {
				modules.push(initializers[j]);
			}
		}
		i++;
	}
	return bindings;
}

var isEmpty = function(object) {
	for (i in object) {
		return false;
	}
	return true;
}

var handle = dojox.lang.aspect.advise(jazz.core.loader, "batch_load_async", {
	before: function() {
		var removeBlankCSSinIE = function() { //TODO delete this step when WI 39263 is resolved
		    var i, styleSheet, parent, cssRules;
		    if(dojo.isIE <= 9 && document.styleSheets.length >= 20){
		        //Remove all the stylesheets without any rules, as they contribute to IE's 31 stylesheet maximum.
		        //When there's > 20 stylesheets, this becomes something to worrying about (20 is an arbitrary #)
		        for(i = document.styleSheets.length - 1; i >= 0; i = i-1){
		            cssRules = document.styleSheets[i].cssRules || document.styleSheets[i].rules;
		            if(cssRules.length === 0){
		                styleSheet = document.styleSheets[i].ownerNode || document.styleSheets[i].owningElement;
		                parent = styleSheet.parentNode;
		                parent.removeChild(styleSheet);
		            }
		        }
		    }
		};

		var modules = arguments[0];
		var originalModules;
		if (queue) {
			// Postponed loading...
			for (var i in queue) {
				modules.push(queue[i]);
			}

			var callback = arguments.length == 2 ? arguments[1] : null;
			if (callback != null) {
				var wrapper = function() {
					removeBlankCSSinIE();
					callback();
				}

				arguments[1] = wrapper;
			}

			queue = null;
		}

		originalModules = modules.slice(0);

		if (resourceBindings) {
			resourceBindings = handleBindings(originalModules, modules, resourceBindings);
		}
		if (classBindings) {
			classBindings = handleBindings(originalModules, modules, classBindings);
		}
	}
});


// START initialization
// The real loading is postponed till the workbench loading
init(); 

})();