/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.core.internal.Workarounds");

/* set default to blue instead of orange
   NGB: removed this override since it doesn't actually do anything (the default hoverStyle is ORANGE_BORDER)
dojo.require("jazz.ui.SimpleToolbar");
dojo.extend(jazz.ui.SimpleToolbar, {
	hoverStyle: jazz.ui.SimpleToolbar.HoverStyles.ORANGE_BORDER
});
*/

/*
 * set the default text of jazz.ui.FilterBox to "Type here to filter"
 * related to WI 196241 on jts
 */
dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.core.internal", "WorkAroundMessages");
var messages = dojo.i18n.getLocalization("rmps.app.core.internal", "WorkAroundMessages");
dojo.require("jazz.ui.FilterBox");
dojo.extend(jazz.ui.FilterBox, {
	defaultText: messages.filterBoxDefaultText
});



dojo.require("dojox.lang.aspect");
dojo.require("rmps.app.core.util");
dojo.require("rmps.core.RootServices");
dojo.require("jazz.ui.ResourceLink");
dojo.require("rmps.core.util.BaselineUtil");
dojo.require("rmps.core.util.IRmpsConstants");
var IRmpsConstants = rmps.core.util.IRmpsConstants;
var BaselineUtil = rmps.core.util.BaselineUtil;
var util = rmps.app.core.util;

// Forcing the load of the jazz loader to avoid css problems later
dojo.require("jazz.core.loader");

/**
 * From RELM Workarounds.js:
 * 
 * HACK: This is a copy of this method with one change: if oldParent is
 * `null`, then it will now use document.body instead. Otherwise, it throws an
 * error. This was needed after changing to dojo 1.8 and foundation 4.0.4.
 * When a text node is created, its width is being measured while it's being
 * created and before it's in the dom. This means there is not
 * rawNode.parentNode. I (Jay) have no idea why this happens or what changed
 * in this release to surface this problem.
 *
 * Note that the reason getTextWidth is called *seems* to be because of a hack
 * that dojo.diagrammer is doing which swaps some gfx methods and results in
 * this method getting called early. Not sure of the details, but if you set a
 * breakpoint here, you can see dojo.diagrammer in the stack, even if you're
 * on the Views page.
 */
dojo.require('dojox.gfx');
dojo.require('dojox.gfx.svg');

dojox.gfx.svg.Text.prototype.getTextWidth = function() {
	var rawNode = this.rawNode;
	var oldParent = rawNode.parentNode || document.body;
	var measurementNode = rawNode.cloneNode(true);
	measurementNode.style.visibility = 'hidden'; //$NON-NLS-1$

	// Some default value.
	var widthPx;
	var text = measurementNode.firstChild.nodeValue;
	oldParent.appendChild(measurementNode);

	if (text) {
		try {
			if (measurementNode.getBBox) {
				widthPx = parseInt(measurementNode.getBBox().width, 10);
			}
		}
		catch (e) {
			// Ignore. Not sure why getBBox fails sometimes, though I don't see any impact
			// in the diagrams.
			// This comment was here before, though I couldn't see how the code worked, and it always
			// was wrong when I ran it before.
			// Yang: work around svgweb bug 417 -- http://code.google.com/p/svgweb/issues/detail?id=417
		}
	}

	oldParent.removeChild(measurementNode);
	if (!widthPx) widthPx = 68;
	return widthPx;
};

/**
 * If no presentation URI is provided and the resource URI refers
 * to the front server, then set a local presentation URI.
 */
dojox.lang.aspect.advise(jazz.ui.ResourceLink, "buildRendering", {
	before: function() {
		var self = dojox.lang.aspect.getContext().instance;
		if (!self.presentationUri && !self.absoluteDMLinksOnly) {
			var modelsUri = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "models");
			var uri;
			if (self.uri) {
				uri = self.uri;
			} else if (!self.uri && self.srcNodeRef && self.srcNodeRef.tagName === "A") {
				uri = self.srcNodeRef.href;
			} else {
				return;
			}

			if (uri.indexOf(modelsUri) === 0) {
				// Possible query parameters needs to be handled as well 
				if (uri.indexOf("?") !== -1) {
					var query = uri.substring(uri.indexOf("?") + 1, uri.length);
					var resourceUri = uri.substring(0, uri.indexOf("?"));

					var params = dojo.queryToObject(query);
					var contextUri = params ? params[IRmpsConstants.BASELINE] : null;
					// If we have a context set to the models resource, then use that to create the link.
					if (contextUri) {
						self.presentationUri = util.getDesignViewResourceUriForContext(resourceUri, contextUri);	
					} else {
						self.presentationUri = util.getDesignViewResourceUri(resourceUri);
					}
				} else {
					self.presentationUri = util.getDesignViewResourceUri(uri);
				}
			}
		}
		var currentUri= self.uri;
		if(currentUri&& !self.avoidBaselineCheck&& (currentUri.indexOf(net.jazz.ajax._contextRoot) == 0|| currentUri.indexOf(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL) == 0)){
			/*Make sure only to add baseline paramater for our server services calls.
			 * We do not overwrite the baseline value if other callers already assigned it
			 * */
			self.uri= BaselineUtil.addBaselineParamaterToUrl(currentUri, null, false);
		}
	}
});

dojox.lang.aspect.advise(jazz.ui.ResourceLink, "buildRendering", {
	after: function() {
		var self = dojox.lang.aspect.getContext().instance;
		if(self.domNode){
			var currentUri= self.uri;
			var index;
			
			/*If a link does not reside in the same application context, then, we have to make sure that it will open a new tab */
			var friends = net.jazz.ajax.friendDomains();
			friends[friends.length] = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + '/';  // add DM
			
			for (index = 0; index < friends.length; ++index) {
				if (currentUri.indexOf(friends[index]) === 0)
					return; // this URI is for DM or a friend, so open in same window
			}
			
			self.domNode.target = "_blank";	// External URIs should open in a new tab		
		}
	}
});

dojo.require("dojox.rpc.JsonRest");
//Check defect 7155 on dojo toolkit
dojo.mixin(dojox.rpc.JsonRest, {
	
	_loader: function(callback, args){
		// load a lazy object
		var serviceAndId = dojox.rpc.JsonRest.getServiceAndId(this.__id);
		var self = this;
		dojox.rpc.JsonRest.query(serviceAndId.service, serviceAndId.id, args).addBoth(function(result){
			// if they are the same this means an object was loaded, otherwise it
			// might be a primitive that was loaded or maybe an error
			if(result == self){
				// we can clear the flag, so it is a loaded object
				delete result.$ref;
				delete result._loadObject;
			}else{
				// it is probably a primitive value, we can't change the identity of an object to
				// the loaded value, so we will keep it lazy, but define the lazy loader to always
				// return the loaded value
				self._loadObject = function(callback){
					callback(result);
				};
			}
			callback(result);
		});
	} 
});


})();