/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.core.internal.query.AbstractClientPagedQueryResult");

dojo.declare("rmps.app.core.internal.query.AbstractClientPagedQueryResult", null, {
	
	query: null,
	id: null,
	//totalResults may be negative to indicate that there were more results not returned because they were over the limit
	totalResults: 0,
	uri: null,
	rawEntries: null,

	entries: null,
	
	filtered: null,
	
	constructor: function(params) {
		this.entries = [];
		if (params) {
			this.query = params.query ? params.query : null;
			this.id = params.id ? params.id : null;
			this.totalResults = params.totalResults ? params.totalResults : 0;
			this.uri = params.uri ? params.uri : null;
			this.filtered = params.filtered ? params.filtered : null;
			
			if (params.rawEntries) {
				this.rawEntries = params.rawEntries;
				this.sortEntries();
				this.pageEntries();
			} else {
				this.rawEntries = [];
			}
		} else {
			this.rawEntries = [];
		}
	},
	
	getTotalResults: function() {
		if (this.filtered) {
			return this.filtered.length;
		} else {
			return this.totalResults;
		}
	},
	
	sortEntries: function() {
		var orderBy = this.query.orderBy;
		if (orderBy == null || orderBy.length == 0) {
			return;
		}
		
		function compare(entry1, entry2, columnId, isAscending) {
			var a;
			var b;
			if (columnId === "id") {
				a = entry1["rdf:Description"][0]["rdf:about"];
				b = entry2["rdf:Description"][0]["rdf:about"];
			} else if (columnId === "name") {
				a = entry1["rdf:Description"][0]["rdfs:label"][0].value;
				b = entry2["rdf:Description"][0]["rdfs:label"][0].value;
				
			} else if (columnId === "modified") {
				a = entry1.updated[0].value;
				b = entry2.updated[0].value;
				
			} else if (columnId === "modifiedBy") {
				a = entry1.contributor[0].name[0].value;
				b = entry2.contributor[0].name[0].value;
				
			} else if (columnId === "displayType") {
				a = entry1.__displayType;
				b = entry2.__displayType;
			}
			
			if (dojo.isString(a)) {
				a = a.toLowerCase();
			} else {
				a = '';
			}
			if (dojo.isString(b)) {
				b = b.toLowerCase();
			} else {
				b = '';
			}
			
			if (isAscending) {
				return (a < b) ? -1 : ((a > b) ? 1 : 0);
			} else {
				return (a > b) ? -1 : ((a < b) ? 1 : 0);
			}
		}
		
		function comparator(entry1, entry2) {
			var orderedBy = {};
			for (var i = 0; i < orderBy.length; ++i) {
				if (!orderedBy[orderBy[i].id]) {
					var result = compare(entry1, entry2, orderBy[i].id, orderBy[i].isAscending);
					if (result != 0) {
						return result;
					}
					orderedBy[orderBy[i].id] = true;
				}
			}
			return 0;
		}
		
		if (this.filtered) {
			this.filtered.sort(comparator);
		} else {
			this.rawEntries.sort(comparator);
		}
	},
	
	pageEntries: function() {
		var start = this.query.page * this.query.pageSize;	
		var end, i;
		
		if (this.filtered) {
			end = Math.min(this.query.page * this.query.pageSize + this.query.pageSize, this.filtered.length);
			
			if (0 <= start && start < this.filtered.length) {
				for (i = start; i < end; ++i) {
					this.entries.push(this.filtered[i]);
				}
			}
		} else {
			end = Math.min(this.query.page * this.query.pageSize + this.query.pageSize, this.rawEntries.length);
		
			if (start >= 0 && start < this.rawEntries.length) {
				for (i = start; i < end; ++i) {
					this.entries.push(this.rawEntries[i]);
				}
			}
		}
	}
});

})();