/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.core.internal.util.QueryUtil");

dojo.require("dojo.date.stamp");
dojo.require("dojo.date.locale");
dojo.requireLocalization("rmps.app.core.internal.util", "QueryUtilMessages");

var messages = dojo.i18n.getLocalization("rmps.app.core.internal.util", "QueryUtilMessages");

rmps.app.core.internal.util.QueryUtil = {
	
	SCOPE: {
		ALL: {name: messages.allModelingArtifacts, value: 0, icon: dojo.moduleUrl("rmps.app", "graphics/icons/search_resources.png") },
		DIAGRAMS: {name: messages.allDiagrams, value: 1, icon: dojo.moduleUrl("rmps.app", "graphics/icons/search_diagrams.png") },
		TYPE: {name: messages.singleType, value: 2, icon: dojo.moduleUrl("rmps.app", "graphics/icons/search_resources.png") },
		SKETCHES: {name: messages.allSketches, value: 3, icon: dojo.moduleUrl("rmps.app", "graphics/icons/search_diagrams.png") },
		REVIEWS: {name: messages.allReviews, value: 4, icon: dojo.moduleUrl("rmps.app", "graphics/icons/search_reviews.gif") },

		valueOf: function(value) {
			for (var key in this) {
				var property = this[key];
				if (typeof property == "object" && property.value == value) {
					return property;
				}
			}
			return null;
		}
	},
	
	getModifiedLabel: function(modified) {
		var messages = dojo.i18n.getLocalization("rmps.app.core.internal.util", "QueryUtilMessages");

		var now= new Date();
		if (!modified) {
			return '';
		}
		var modDate = dojo.date.stamp.fromISOString(modified);
		var deltaMinutes = dojo.date.difference(modDate, now, "minute");
	
		// if less than an hour ago, display in minutes
		if (deltaMinutes >= 0 && deltaMinutes < 60) {
			if (deltaMinutes === 0) {
				deltaMinutes = 1;
			}
			return (deltaMinutes === 1) ? messages.oneMinuteAgo : dojo.string.substitute(messages.minutesAgo,[deltaMinutes]);
		}

		var yearOffset = dojo.date.difference(modDate, now, "year");
	
		var totalDayOffset = dojo.date.difference(modDate, now, "day");
		// if same day or less than 12 hours ago, display in hours
		if (((totalDayOffset === 0) && (deltaMinutes >= 0)) || (deltaMinutes >= 0 && deltaMinutes < 720)) {
			var hours = Math.round(deltaMinutes / 60);
			return (hours === 1) ? messages.oneHourAgo : dojo.string.substitute(messages.hoursAgo,[hours]);
		}
		// if previous calendar date, display "Yesterday"
		if (totalDayOffset === 1) {
			var option = {
					selector: "time",
					formatLength: "medium"
			};
			var time = dojo.date.locale.format(modDate, option);
			return dojo.string.substitute(messages.yesterdayAt,[time]);
		}
		return dojo.date.locale.format(modDate, {formatLength: 'medium'});
	}
};

})();