/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.core.util");

dojo.require("jazz.app.context");
dojo.require("jazz.client.xhr");
dojo.require("net.jazz.ajax.viewletServiceRoot");
dojo.require("net.jazz.ajax.BootstrapProperties");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("rmps.app.ui.ReviewEditorUtil");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("rmps.core.util.IRmpsConstants");

var ReviewEditorUtil= rmps.app.ui.ReviewEditorUtil;
var DesignUtil= rmps.app.ui.internal.pages.design.DesignUtil;
var BootstrapProperties = net.jazz.ajax.BootstrapProperties;
var IRmpsConstants = rmps.core.util.IRmpsConstants;

rmps.app.core.util = {
	
	MODELING_PAGE: "rmps.app.modeling",
	
	/*We keep this one to be used by all methods in the util, as getBasicPrefix is designed only to work for two methods getViewResourceUri and getSearchUri*/
	_basicPrefix: "#action=rmps.app.design.",
	
	adminBasicPrefix: "#action=dm.app.admin.",
	
	reviewBasicPrefix: "#action=dm.app.review.",
	
	queryBasicPrefix: "#action=rmps.app.design.query",
	
	_getBasicPrefix: function(resourceUri){
		/*
		 * Workaround till resolving:
		 * 
		 * 22213: rmps.app.core.util#getViewResourceUri always makes assumption that the view links belong to the models page
		 * 
		 * We assume tha the pageId would be passed, and if the page id is anything else other than dm.app.review, then, we will return the default _basicPrefix
		 * 
		 */
		var prefix= this._basicPrefix;
		if (resourceUri && resourceUri.indexOf(ReviewEditorUtil.RES_REVIEW_PREFIX) >= 0){
			prefix= this.reviewBasicPrefix;
		} else if (resourceUri && resourceUri.indexOf("/dm/queries/") >= 0){
			prefix= this.queryBasicPrefix;
		}
		return prefix;
	},
	
	getWelcomeUri: function() {
		return this._basicPrefix + "welcome";
	},
	
	// TODO: Should call getDesignViewResourceUri once there are no clients of the modeling page left.
	getViewResourceUri: function(resourceUri)  {
		var obj = {"resource": resourceUri};
		return this._getBasicPrefix(resourceUri) + "edit&" + dojo.objectToQuery(obj);
	},
 
	getDesignViewResourceUri: function(resourceUri)  {
		var prefix = this._getBasicPrefix(resourceUri);
		// Send everything to the design page, with the exception of reviews.
		if (prefix === this.reviewBasicPrefix) {
			return prefix + "viewResource&" + dojo.objectToQuery({"resourceUri": resourceUri});			
		} else if (prefix === this.queryBasicPrefix) {
			return prefix + "&" + dojo.objectToQuery({"resource": resourceUri});			
		}
		return DesignUtil.getEditResourceUri(resourceUri);
	},

	getDesignViewResourceUriForContext: function(resourceUri, contextUri, projectName) {
		if (!contextUri) {
			return this.getDesignViewResourceUri(resourceUri);
		}
		
		// See if it's the same context as current or otherwise create an
		// absolute link instead that includes the context.
		var currentContextURI = jazz.app.context.get().baseline;
		if (currentContextURI && currentContextURI.indexOf(contextUri) !== 0) {
			// Different context
			projectName = projectName ? projectName : jazz.app.context.get().name;
			var url = BootstrapProperties.FRONTSIDE_URL + "/web/projects/" + escape(projectName);
			var query = {};
			if (dojo.config.isDebug) {
				query.debug = true;
			}
			query[IRmpsConstants.BASELINE] = contextUri;			
			var querystr = dojo.objectToQuery(query);
			if (querystr) {
				url += '?' + querystr;
			}
			url += this.getDesignViewResourceUri(resourceUri);
			return url;
		}
	
		return this.getDesignViewResourceUri(resourceUri);
	},
	
	getSearchUri: function(queryString)  {
		var obj = {query: queryString};
		return this._getBasicPrefix() + "search&" + dojo.objectToQuery(obj);
	},
		
	getShowViewpointUri: function(viewpointUri) {
		if (viewpointUri) {
			var obj = {"viewpoint": viewpointUri};
			return this._basicPrefix + "explore&" + dojo.objectToQuery(obj);			
		}
		return this._basicPrefix + "explore";
	},

	closeNavigationPanel: function(){
		var wb = dojo.query(".show-hide-arrow");
		var b = wb[0];
	
		var clickevent=document.createEvent("MouseEvents");
		clickevent.initEvent("click", true, true);
		b.dispatchEvent(clickevent);
	},
	
	checkIfMobile: function(){
		
		var mobile = false;
		
		var uagent = navigator.userAgent.toLowerCase();
		
		if( uagent.search( 'ipad' ) > -1 ){
			mobile = true;
		}
		
		return mobile;
	}
	
};

}());