/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmps.diagram.data.JtsRdfStore");

dojo.require("rmps.core.client");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.graphics.ui", "DiagramEditorMessages");

(function(){
	var messages = dojo.i18n.getLocalization("rmps.graphics.ui", "DiagramEditorMessages");
	
	dojo.declare("rmps.diagram.data.JtsRdfStore", null, {
		// NOTE: statically declared because Layout.js creates a new data store every fetch.
		// TODO (MG): fix.
		loadedEditorCode: {},
	
		constructor: function(viewer, forceCodeReload) {
			this.viewer = viewer;
			this.forceCodeReload = forceCodeReload;
		},
		
		fetch: function(resourceInfo, definition, diagramWidget) {
			if(!resourceInfo || !definition) {
				return;
			}
			this.modelInstance = resourceInfo.model;
			this.modelIndex = resourceInfo.modelIndex;
			this.resourcePath = resourceInfo.uri;
			this.definition = definition;
			this.isReadOnly = resourceInfo.isReadOnly;
			this.diagramWidget = diagramWidget;
			
			// load editor code
			this.viewer.clear();
			if (this.diagramWidget && this.diagramWidget.showMessage) {
				this.diagramWidget.showMessage(messages.loading);
			}
			if(this.loadedEditorCode[definition.uri] && !this.forceCodeReload) {
				this._createDiagram();
			} else {
				rmps.core.client.xhrGet({
					url: net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/dsv/codegen",
					content: {
						editor: definition.uri,
						isReadOnly: this.isReadOnly,
						debug: dojo.config.isDebug
					},
					handleAs: "text",
					load: dojo.hitch(this, this._onCodeLoaded),
					error: function(e) {
						console.error(e);
					}
				});
			}
		},
		
		_onCodeLoaded: function(data, ioArgs) {		
			// enumerate required modules
			var requiredModules = [];
			var requiredModulesHeader = ioArgs.xhr.getResponseHeader("required-modules");
			if (requiredModulesHeader) {
				requiredModules = dojo.fromJson(requiredModulesHeader);
			} else {
				console.warn("required-modules header not found for generated code");
			}
			
			// enumerate extension modules
			var extensionModulesHeader = ioArgs.xhr.getResponseHeader("extension-modules");					
			if (extensionModulesHeader) {
				var extensionModules = dojo.fromJson(extensionModulesHeader);
				for(var i = 0, module; module = extensionModules[i]; i++) {
					requiredModules.push(module);
				}
			}
			
			// load all external modules and create diagram when done
			jazz.core.loader.batch_load_async(requiredModules, dojo.hitch(this, function() {
				dojo["eval"](data);
				this.loadedEditorCode[this.definition.uri] = true;
				this._createDiagram();
			}));
		},
		
		_createDiagram: function() {
			if (this.diagramWidget && this.diagramWidget.hideMessage) {
				this.diagramWidget.hideMessage();
			}
			// initialize repository
			this.viewer.resource = this.modelInstance;			
			this.viewer.resourceId = this.resourcePath;
			
			var editorPackage = null
			try {
				editorPackage = new Function("return rmpw.dsv.codegen." + this.definition.id)();
			} catch(e) {
				console.error("Error loading generated code: ", e);
				return;
			}
	
			// initialize domain package
			if (editorPackage.domain && editorPackage.domain.Package) {
				this.viewer.commandPackage = new editorPackage.domain.Package();
			}
			
			// create root shape
			var renderMetadata = editorPackage.render._Metadata.prototype.rootShapesByType;
			// TODO (MG): remove this hack /////////////////////////////
			// find the root type
			for(var key in renderMetadata) {
				renderMetadata = renderMetadata[key];
				break;
			}
			// end hack ////////////////////////////////////////////////
			
			var rootShape = new editorPackage.render[renderMetadata.shapeClass]();
			rootShape.root = this.viewer;
			rootShape[renderMetadata.graphicsSetter](this.viewer.viewport.contents);
			rootShape[renderMetadata.modelSetter](this.modelInstance);
	
			if (!this.isReadOnly) {
				// create root behaviour controller
				var behaviorMetadata = editorPackage.behavior._Metadata.prototype.controllersByShapeType;
				var rootBehaviorClassName = behaviorMetadata[renderMetadata.shapeClass];
				var rootBehavior = new editorPackage.behavior[rootBehaviorClassName](rootShape);
	
				// do a bunch of stuff that should be done by the behaviour controller itself...
				// TODO (MG): remove
				var viewer = this.viewer;
				rootBehavior._connectConstruction = dojo.connect(rootShape, "applyModelChanges", this, function() {
					rootBehavior.setParent(viewer);
					dojo.disconnect(rootBehavior._connectConstruction);
					delete rootBehavior._connectConstruction;
				});
				rootBehavior._connectDestroy = dojo.connect(rootShape, "destroy", this, function() {
					dojo.disconnect(rootBehavior._connectDestroy);
					rootBehavior.destroy();
				});
			}
	
			// hook for post diagram load initialization
			this.viewer.setDiagram(rootShape, this.diagramWidget);
		}
	});
})();