/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.common.datastruct.TreeNode");

/**
 * A base class for tree nodes.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.common.datastruct.TreeNode", null, {
	constructor: function() {
		this.children = [];
	},
	
	setParent: function(parent) {
		this.parent = parent;
		this.root = parent.root;
	},
	
	addChild: function(child) {
		child.setParent(this);
		this.children.push(child);
	},
	
	removeChild: function(child) {
		if (this.children) {
			var removed = this.children.splice(dojo.indexOf(this.children, child), 1);
			return removed != null;
		}
		return false;
	},
	
	destroy: function() {
		// destroy children
		if (this.children) {
			dojo.forEach(this.children.slice(), function(child) {
				if (child.destroy) {
					child.destroy();
				}
			});
		}
		delete this.children;

		// detach self from parent (last, other destructors might require parent reference for cleanup)
		if (this.parent && this.parent.removeChild) {
			this.parent.removeChild(this);
		}
		delete this.parent;
	}
});