/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.render.decorations.Grid");

/**
 * A layer which draws a grid with major and minor marks.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.diagrams.render.decorations.Grid", null, {
	minorSpacing: 10,
	minorStyle: [0, 0, 255, 0.04],
	majorSpacing: 100, // must be divisible by minor spacing
	majorStyle: [0, 0, 255, 0.08],
	
	constructor: function(parentFigure, config) {
		dojo.mixin(this, parentFigure.createGroup());
		dojo.mixin(this, config);
		this.isDecoration = true;
		
		// walk up the tree to find the root surface
		this.surface = this.parent;
		while(this.surface.parent) {
			this.surface = this.surface.parent;
		}		
		
		this.drawGrid();
		dojo.connect(this.surface, "setDimensions", this, "updateGrid");
	},
	
	destroy: function() {
		this.removeShape();
	},
	
	updateGrid: function() {
		this.clear();
		this.drawGrid();
	},
	
	drawGrid: function() {
		// TODO (MG): should not depend on zoom being 100% at the time of creation
		var canvasArea = this.surface.getDimensions();
		for(var x=0; x < canvasArea.width; x += this.minorSpacing) {
			this._createLine({x1: x, x2: x, y1: 0, y2: canvasArea.height}, (x % this.majorSpacing) === 0);
		}
		for(var y=0; y < canvasArea.height; y += this.minorSpacing) {
			this._createLine({y1: y, y2: y, x1: 0, x2: canvasArea.width}, (y % this.majorSpacing) === 0);
		}
	},
	
	_createLine: function(rawNode, major) {
		var line = this.createLine(rawNode);
		if(major) {
			line.setStroke(this.majorStyle);
		} else {
			line.setStroke(this.minorStyle);
		}
		if(dojox.gfx.svg) {
			line.rawNode.setAttribute('shape-rendering', 'crispEdges');
		}
	    return line;
	}
});