/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/


dojo.provide("rmpw.diagrams.render.decorations.LabelLocator");
dojo.require("rmpw.diagrams.render.util.PointList");
dojo.require("rmpw.diagrams.render.util.PointListUtilities");
dojo.require("rmpw.diagrams.render.util.LabelHelper");
dojo.require("rmpw.diagrams.render.util.Point");

/**
 * 
 * Label locator that supports locating labels whose parent is either a Node or
 * and Edge.
 *
 * 
 * @author Susanta Datta
 */

dojo.declare("rmpw.diagrams.render.decorations.LabelLocator", null, {
	/**
	 * the parent figure of this locator
	 */
	parent : null,
	alignment : null,
	target : null,
	TARGET_LOCATION : 85,
	SOURCE_LOCATION : 15,
	MIDDLE_LOCATION : 50,
	
	constructor: function(target) {
		this.target = target;
		this.alignment = "MIDPOINT";
	},
	
	getParentFigure: function() {
		return this.target.parent.policies.graphElement.getInternalGraphElement();
	},
	
	/**
	 * Positions the lable relative to the reference point with the
	 * given offsets.
	 */
	relocate: function( ) {
		var graphFloatingLabel = this.target.policies.graphFloatingLabel;
		var modelBounds = graphFloatingLabel.getConstraintBounds();
		var offset = new rmpw.diagrams.render.util.Point(modelBounds.x, modelBounds.y);
		var refPt = this.getReferencePoint();
		var helper = new rmpw.diagrams.render.util.LabelHelper();
		var util = new rmpw.diagrams.render.util.PointListUtilities();
		
		var points = this.getParentPointList();
		var offsetOrient = graphFloatingLabel.getDefaultOffsetOrientation();
		var location;
		
		if (offset.x === 0 && offset.y === 0 && offsetOrient !== "Center") {
			// find nearest line segment so we can find the slope
			var index = util.findNearestLineSegIndexOfPoint(points, refPt);
			var segment = util.getLineSegments(points).pop(index - 1);
			var buffer = 5;
			if (segment) {
				if (segment.isHorizontal()) {
					if (offsetOrient === "West") {
						offset = new rmpw.diagrams.render.util.Point(0, -(modelBounds.height / 2) - buffer);
					}
					else if (offsetOrient === "East") {
						offset = new rmpw.diagrams.render.util.Point(0, (modelBounds.height / 2) + buffer);
					}
				} else if (segment.isVertical()) {
					if (offsetOrient === "West") {
						offset = new rmpw.diagrams.render.util.Point(0, -(modelBounds.width / 2) - buffer);
					}
					else if (offsetOrient === "East") {
						offset = new rmpw.diagrams.render.util.Point(0, (modelBounds.width / 2) + buffer);
					}
				} else {
					location = helper.relativeCoordinateFromOffset(modelBounds, refPt, offset, points);
					
					var rect = new rmpw.diagrams.render.util.Rectangle(location.x, location.y, modelBounds.width, modelBounds.height);
					var polyRect = rmpw.diagrams.render.util.PointListUtilities.prototype.getRectanglePolyline(rect);
					
					for (var i=0; i<polyRect.size; i++) {
						var pt = polyRect.getPoint(i);
						polyRect.setPoint(helper.getOrthogonalDistances(segment, refPt, pt), i);	
					}
					
					if (offsetOrient === "West") {
						offset =  new rmpw.diagrams.render.util.Point(0, -(polyRect.getBounds().height / 2) - buffer);
					}
					else if (offsetOrient === "East") {
						offset = new rmpw.diagrams.render.util.Point(0, (polyRect.getBounds().height / 2) + buffer);
					}
				}
			}
		}
		
		location = helper.relativeCoordinateFromOffset(modelBounds, refPt, offset, points);
		graphFloatingLabel.setBounds({x: location.x, y: location.y, width: modelBounds.width, height: modelBounds.height});
	},
	/**
	 * gets the point list using the passed figure to get the parent
	 * 
	 * @param label the <code>IFigure</code> to use to retrieve the parent points
	 * @return List of points
	 */
	getParentPointList: function() {
		var parent = this.getParentFigure();
		if (parent.mapping.policies.graphEdge) {
			return parent.getPointList().getCopy();
		} else {
			var ptList = new rmpw.diagrams.render.util.PointList();
			var point = parent.getBounds().getTopLeft();		
			ptList.addPoint_0(point.x, point.y);
			return ptList;
		}			
	},	
	/**
	 * Returns the reference point for the locator.
	 * @return the reference point
	 */
	getReferencePoint: function() {
		var parent = this.getParentFigure();
		if (parent.mapping.policies.graphEdge) {
			var ptList = parent.getPointList().getCopy();
			var util = new rmpw.diagrams.render.util.PointListUtilities();
			return util.calculatePointRelativeToLine(ptList, 0, this.getLocation(), true);						
		} else {
			return parent.getBounds().getTopLeft();		
		}
	},
	
	/**
	 * Returns the location %
	 */
	getLocation: function() {
		switch (this.alignment) {
			case "TARGET":
				return this.TARGET_LOCATION;
			case "SOURCE":
				return this.SOURCE_LOCATION;
			case "MIDPOINT":
				return this.MIDDLE_LOCATION;
			default:
				return this.MIDDLE_LOCATION;
		}
	},
	
	setAlignment: function(alignment) {
		this.alignment = alignment;
	},

	/**
	 * Returns the <code>PointList</code> describing the label's parent.
	 * 
	 * @return pointList
	 */
	getPointList: function() {
		var parent = this.getParentFigure();
		if (parent.mapping.policies.graphEdge) {
			return parent.getPointList();
		} else {
			var ptList = new rmpw.diagrams.render.util.PointList();
			ptList.addPoint(parent.getBounds().getTopLeft());
			return ptList;
		}
	}
});
