/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.render.decorations.RectangleShadow");

/**
 * A decoration which adds a shadow down and to the right of any sized rectangle
 * with configurable shadow size.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.diagrams.render.decorations.RectangleShadow", null, {
    shadowSize: 4,
    shadowColors: [
        {offset: 0, color: [0, 0, 0, 0.1]},
        {offset: 1, color: [0, 0, 0, 0]}],
    rectLineWidth: 1,
    
	constructor: function(rect) {
	    if(!dojox.gfx.vml) {
			var group = rect.getParent().createGroup();
			group.isDecoration = true;
			
			var bounds = rect.getBoundingBox();
		    // upper right corner cap
		    var shadowCorner = group.createRect({

                x: bounds.width + this.rectLineWidth,
                y: 0,
                width: this.shadowSize,
                height: this.shadowSize});
                shadowCorner.setFill({
                    type : "radial",
                    cx : bounds.width + this.rectLineWidth,
                    cy : this.shadowSize,
                    r : this.shadowSize,
                    colors : this.shadowColors
                });

                // right shadow
                var shadowRight = group.createRect({
                    x : bounds.width,
                    y : this.shadowSize,
                    width : this.shadowSize,
                    height : bounds.height + this.rectLineWidth - this.shadowSize
                });
                shadowRight.setFill({
                    type : "linear",
                    x1 : bounds.width + this.rectLineWidth,
                    y1 : this.shadowSize,
                    x2 : bounds.width + this.rectLineWidth + this.shadowSize,
                    y2 : this.shadowSize,
                    colors : this.shadowColors
                });

                // lower right corner cap
                shadowCorner = group.createRect({
                    x : bounds.width + this.rectLineWidth,
                    y : bounds.height + this.rectLineWidth,
                    width : this.shadowSize,
                    height : this.shadowSize
                });
                shadowCorner.setFill({
                    type : "radial",
                    cx : bounds.width + this.rectLineWidth,
                    cy : bounds.height + this.rectLineWidth,
                    r : this.shadowSize,
                    colors : this.shadowColors
                });

                // bottom shadow
                var shadowBottom = group.createRect({
                    x : this.shadowSize,
                    y : bounds.height + this.rectLineWidth,
                    width : bounds.width + this.rectLineWidth - this.shadowSize,
                    height : this.shadowSize
                });
                shadowBottom.setFill({
                    type : "linear",
                    x1 : this.shadowSize,
                    y1 : bounds.height + this.rectLineWidth,
                    x2 : this.shadowSize,
                    y2 : bounds.height + this.rectLineWidth + this.shadowSize,
                    colors : this.shadowColors
                });

                // lower left corner cap
                shadowCorner = group.createRect({
                    x : 0,
                    y : bounds.height + this.rectLineWidth,
                    width : this.shadowSize,
                    height : this.shadowSize
                });
                shadowCorner.setFill({
                    type : "radial",
                    cx : this.shadowSize,
                    cy : bounds.height + this.rectLineWidth,
                    r : this.shadowSize,
                    colors : this.shadowColors
                }); 
	    }
	}
});