/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.editor.AbstractPage");

dojo.require("rmps.ui.editor.PageInitializer");
dojo.require("net.jazz.ajax.ui.ErrorPage");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("rmps.ui.editor.PageHelper");
dojo.require("dijit.layout._LayoutWidget");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.ui.editor", "AbstractPageMessages");
var messages = dojo.i18n.getLocalization("rmps.ui.editor", "AbstractPageMessages");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

/*
 * This class is designed to be overriden in order to provide an easy way to define an application page. A page can extend this class and extend/use the required
 * methods for its needs. 
 * 
 */
dojo.declare("rmps.ui.editor.AbstractPage", [rmps.ui.editor.PageInitializer, dijit.layout._LayoutWidget], {
	
	pageHelper: null,
	
	/*private fields */
	_pageRegistry: null,
	
	postCreate: function() {
		var actionRegistry = PlatformUI.getWorkbench().getActionRegistry();
		var pageRemovalId= this.getPageRemovalId();
		if (pageRemovalId&& dojo.getObject(pageRemovalId)) {
			var errorPage = new net.jazz.ajax.ui.ErrorPage({
					message: messages.pageRemoved,
					hideRunDefaultAction: true
				});
			this.addChild(errorPage);
			/*We return without subscribing any of the actions. Thus, it won't make a difference */
			return;	
		}
		
		this.registerActions();
		this.pageHelper= rmps.ui.editor.PageHelper.getInstance(this.getPageId());
		this.pageHelper.init(this);
		this.createPageContents();
		this.pageHelper.setEditorView(this);		
		
		this.subscribeActions();
		this.inherited(arguments);
		
		/*
		 * Why do we call for startup here? JTS does not assume that the page is instance of widget, and do not call for the startup method after constructing
		 * a page. Thus, this means that the page will remianed not started all the time. And the only place that starts the page and constructs it is the
		 * jts Workbench.js place. Thus, we have to make that call to esnure that the page is started, and thus all of its children.
		 * 
		 * 
		 */
		this.startup();
	},
	
	getPageHelper: function(){
		return this.pageHelper;
	},
	
	getEditorView: function() {
		return this.pageHelper.getEditorView();
	},
		
	/*Sub classes can extend this method in order to append more controls (such as side navigation, etc)*/
	createPageContents: function(){/*Empty block */},
	
	/*
	 * This method must be overriden by sub classes in order to add their contribution on the page actions
	 * There are two ways of definning actions. 
	 * 		1) Use subscribeAllActions helper method:
	 * 			We can make a call like:
	 * 				this.subscribeAllActions(someController);
	 * 				this.subscribeAllActions(anotherController);
	 * 			This call means that all actions defined in the page, will be subscribed to its equivalent method in the passed controller. If an action name
	 * 			does not exist as a method name in the passed controller, this action will be just ignored. Thus, we can use more than one controller if required.
	 * 			This is helpful when another place is using the page, and needs to add its own contribution to the page
	 * 		1) Apply step by step registration. For example:
	 * 			this.subscribePageAction("somePrefix.action1", dojo.hitch(this, this.action1));
	 * 			this.subscribePageAction("somePrefix.action1", dojo.hitch(this.someObjects, this.someObjects.action2));	
	 * 
	 */
	subscribeActions: function(){/*to be extended by sub classes */},
	
	/* 
	 * Helper method, and expected to be used by sub classes. @see this.subscribeActions
	 * 
	 * It is the clients repsonsbility to pass a valid controller value
	 */
	subscribeAllActions: function(controller){
		var prefix= this.getActionsPrefix();
		for(var index=0; index<this._actions.length; index++){
			var action= this._actions[index];
			var method= controller[action];
			if(method){
				this.subscribePageAction(action, dojo.hitch(controller, method));
			}
		}
	},
	
	/* 
	 * Helper method, and expected to be used by sub classes. @see this.subscribeActions
	 * 
	 * It is the clients repsonsbility to pass valid parameters values
	 */
	subscribePageAction: function(action, callback){
		this.subscribe(this.getActionsPrefix()+ "."+action, callback);
	},
	
	/*
	* Required to register the actions by sub classes. An example:
	* this.registerActionsList(["_default","action1","action2", "action3"]);
	* Can be used at different places that continutre the same page in order to register more actions to the list. 
	*/
	registerActions: function(){/*to be extended by sub classes */},
	
	/* 
	 * Helper method, and expected to be used by sub classes in order to register actions list. @see this.registerActions
	 * 
	 */
	registerActionsList: function(actionsList){
		if(dojo.isArray(actionsList)){
			this._actions= this.getActionsList().concat(actionsList);			
		}else{
			console.error("registerActionsList expects an array");
		}
	},
		
	/*
	* Return the defined value for the page removal id. Must be the same as what we defined using net.jazz.ajax.pages extension point 
	* 
	* For our implementation ,page removal is required property in order to maintain consitency among different pages.
	* 
	* *@see: 179010: PageRegistry should be a public API
	* https://jazz.net/jazz/resource/itemName/com.ibm.team.workitem.WorkItem/179010
	* */	
	getPageRemovalId: function(){
		rmps.core.error.createError(new Error("Removal Page id is not implemented")).dispatch();
	},
	
	/*
	 * The owing view if the AbstractPage itslef, unless other place needs to extned it for
	 * cases such as side navigation 
	 * 
	 * View must be an instance of _Widget
	 */
	getView: function(){
		return this;
	},
	
	getContainerNode: function(){
		return this.domNode;
	}
	
});


})();
