/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.editor.CapabilityComponentMenu");

dojo.require("dojo.i18n");

dojo.require("rmps.app.core.util");
dojo.require("jazz.app._ComponentMenu");
dojo.require("rmps.ui.editor.PageHelper");
dojo.require("rmps.ui.editor.EditorTrackingWidget");
dojo.require("rmps.ui.editor.EditorMRU");

dojo.require('jazz.util.html');

//FIXME
var wrapHint = jazz.ui.internal.MegaMenu.wrapHint;
var PageHelper= rmps.ui.editor.PageHelper;

/* 
 * This class provides a csutomized ComponentMenu enables tracking items when recent or
 * new unsaved work is added, the menu is updated to include these new items.
 * 
 * a Sub classe can extend this class, and then MUST make a call for addCapabilityComponentMenuListeners
 * with an instance of editorTracker object with its specialized actions for loading, adding, etc  
 */
dojo.declare("rmps.ui.editor.CapabilityComponentMenu", [jazz.app._ComponentMenu, rmps.ui.editor.EditorTrackingWidget], {
	pageHelper: null,
	
	_editorTracker: null,
	_recentlyViewedTitle: null,
	_unsavedWorkTitle: null,
	_titles: null,
	_capabilityMessages: null,
	_blankIcon: null,
	
	//The numer of allowed recently viewed items in the menu.
	recentlyViewedlimit: 10,	//Set to 10 by default
	
	constructor: function(){
		this._titles= {};
		this._blankIcon= dojo.moduleUrl("dojo", "resources/blank.gif");
	},
	
	postCreate: function() {
		this.addCapabilityComponentMenuItems();
		
		/*Start listening after we added the items in order to make sure that the most recent viewed items will be appened after the last added item */
		this._addCapabilityComponentMenuListeners(this.getPageHelper());
		this.inherited(arguments);
	},
	
	postMixInProperties: function(){
		this.inherited(arguments);
		
		dojo.requireLocalization("rmps.ui.editor", "CapabilityComponentMenuMessages");
		this._capabilityMessages = dojo.i18n.getLocalization("rmps.ui.editor", "CapabilityComponentMenuMessages");
	},
	
	/*Should add the menu items here */
	addCapabilityComponentMenuItems: function(){
		/*To be implemented by sub classes */
	},
	
	getSelected: function() {
		return this.getPageHelper().getSelected(this.getItems());
	},
	
	/** Get the list of items for a specific category */
	getItemsFor: function(key){
		var items= [];
		var titleNode= this._titles[key];
		if(!titleNode){
			return items;
		}
		
		var categoryIndex= titleNode.node.getIndexInParent();
		var allItems= this.getItems();
		
		for(var index = categoryIndex+1, item; item = allItems[index]; index++) {
			if(item instanceof jazz.ui.menu.NodeMenuItem){
				break;
			}else if(item instanceof dijit.MenuSeparator|| item=== titleNode.showMoreNode){
				continue;
			}
			items.push(item);
		}
		return items;
	},
	
	/* (non-Javadoc)
	 * @see jazz.app._ComponentMenu#addTitle
	 * 
	 * example: this.addTitle(designMessages.create, {key:"create", after: "browse"});
	 * 		after param is to determine which category the tite will come after
	 * 		key param is used to identify the title to be added
	 * 
	 */
	addTitle: function(title, args){
		if(args){
			var after= args.after;
			if(after){
				var titleNode= this._titles[after];
				if(titleNode){
					var categoryIndex= titleNode.node.getIndexInParent();
					var items= this.getItems();
					
					for(var index = categoryIndex+1, item; item = items[index]; index++) {
						if(item instanceof jazz.ui.menu.NodeMenuItem){
							break;
						}
					}
					args.insertIndex= index;
				}
			}else{
				var before= args.before;
				var titleNode= this._titles[before];
				if(titleNode){
					args.insertIndex= titleNode.node.getIndexInParent();
				}
			}
		}
		
		var node= this.inherited(arguments);
		
		if(!args){
			args= {};
		}
		var key= args?args.key||title:title;
		this._titles[key]= {
				node: node,
				minLimit: args.minLimit,
				maxLimit:  args.maxLimit,
				handleMore: args.handleMore,
				expanded: false,
				pendingAdd: [],
				footerLength: args.footerLength,
				footer: [],
				key: key
		};
		
		return node;
	},
	
	/**
	 * Insert a new menu item, and remove an existing one. The argument "inserIndex" is used to determine if we want to insert the new menu item in a new index other than the index
	 * of the menu item that will be replaced. If undefined, then the new meni item will just simply take the index of its replacing item.  
	*/
	replaceBy: function(args, /*jazz.ui.menu.MenuItem */ menuItem, insertIndex, preserve){
		var index= menuItem.getIndexInParent();
		if(index>0){
			this.remove(menuItem);
		}
		if(!preserve){
			menuItem.destroyRecursive();
		}
		return this.addItemByKey(args, insertIndex||index);
	},
	
	/** 
	 * insertIndexOrKey param is used to determine which cateogry the item will be placed in  
	 * 
	 */
	addItemByKey: function(args, /*String|Integer */ insertIndexOrKey) {
		varifymInLimit: {
			var key= args.key;
			if(!key){
				break varifymInLimit; 
			}
			
			if(!insertIndexOrKey){
				insertIndexOrKey= this.getIndexForKey(key);
			}
			
			var titleDescriptor= this._titles[key]; 	
			if(!titleDescriptor|| titleDescriptor.expanded){
				break varifymInLimit;
			}
			
			//If show more is added, then make sure to return but add the item to the pending changes
			if(titleDescriptor.showMoreNode){
				return titleDescriptor.pendingAdd.push({args: args});	
			}
			
			var minLimit= titleDescriptor.minLimit;
			
			if(!minLimit){
				break varifymInLimit;
			}
			
			var items= this.getItems();
			var categoryIndex= titleDescriptor.node.getIndexInParent();
			var separatorsNumber=0; 
			for(var index = categoryIndex+1, item; item = items[index]; index++) {
				if(item instanceof jazz.ui.menu.NodeMenuItem){
					break;
				}else if(item instanceof dijit.MenuSeparator){
					separatorsNumber++;
				}
			}
			
			var addedItemsNumber= index- categoryIndex-separatorsNumber;
			
			if(addedItemsNumber>minLimit){
				titleDescriptor.pendingAdd.push({args: args});
				this.addShowMoreNode(key);
				return;
			}
			
			if(!insertIndexOrKey){
				insertIndexOrKey= index;
			}
		}
		
		return this.addItemByArgs(args, insertIndexOrKey);
	},
	
	/**
	 * It is used to add footer items for a specific category. The footer is distiguished by having a separator after the main items. The argument "overDue" is used to force adding
	 * a footer item even when this exceeds the maxLimit for the given category  
	 *  
	 */
	addFooterItemByArgs: function(argsList, /*String*/ key, overDue) {
		if(!argsList|| !key){
			return;
		}
		
		if(!dojo.isArray(argsList)){
			argsList= [argsList];
		}
		
		var titleDescriptor= this._titles[key];
		if(!titleDescriptor){
			return;
		}
		
		var footerLength= titleDescriptor.footerLength;
		
		if(!footerLength){	//Safe check. This shall be the client's responsability
			return this.addItemsByKey(args, key);
		}
		
		var footer= titleDescriptor.footer;
		
		var currentFooterLength= footer.length;
		if(!overDue&& currentFooterLength>=footerLength){
			return titleDescriptor.footer.concat(dojo.map(argsList, function(entry){return {args: entry}}));
		}
		
		if(currentFooterLength===0&& !titleDescriptor.separatorNode){
			titleDescriptor.separatorNode= this.addSeparatorByKey(key);
		}
		var insertIndex= titleDescriptor.separatorNode.getIndexInParent()+1;
		
		var remainedToAdd= argsList.length;
		if(!overDue){
			var allowedToAdd= footerLength- titleDescriptor.footer.length;
			var argsListLength= argsList.length;
			remainedToAdd= argsListLength<= allowedToAdd?allowedToAdd: argsListLength- allowedToAdd;
			
			if(remainedToAdd>argsListLength){
				remainedToAdd= argsListLength;
			}	
		}		
		
		for(var index=0; index<remainedToAdd; index++){
			var currentArg = argsList[index];
			this.addItemByArgs(argsList[index], insertIndex+index);
			titleDescriptor.footer.push({args: currentArg});
		}
		
		for(var index=remainedToAdd; index<currentFooterLength; index++){
			var currentArg = argsList[index];
			titleDescriptor.footer.push({args: currentArg});
		}
		
		if(titleDescriptor.footer.length>footerLength){
			this.addShowMoreNode(key);
		}
	},
	
	/**
	 * It is used to add a show more menu item at the end of specific category. When the total number of items is less than maxLimit, then the list will be expanded; otherwise, a show
	 * more handler must be implemented in order to handle the show more process.
	 *  
	 */
	addShowMoreNode: function(key){
		var titleDescriptor= this._titles[key];
		if(!titleDescriptor|| titleDescriptor.expanded|| titleDescriptor.showMoreNode){
			return;
		}
		
		var index= this.getIndexForKey(key);
		
		titleDescriptor.showMoreNode= this.addItemByArgs({
			label: '<a style="color: #6DA903" href="#">'+ this._capabilityMessages.more+ '</a>'
		},index);
		
		titleDescriptor.showMoreNode._onClick= dojo.hitch(this, function(titleDescriptor, event){
			dojo.stopEvent(event);
			titleDescriptor.expanded= true;
			
			var maxLimit= titleDescriptor.maxLimit;
			if(this.getItemsFor(titleDescriptor.key).length+ titleDescriptor.pendingAdd.length > maxLimit){
				try{
					this.close();
					return titleDescriptor.handleMore(titleDescriptor);
				}catch(err){
					//Wrap in try/catch to avoid multiple function check everytime, as providing a valid function is the caller responsability
					console.error(err);
				}
			}else{
				var lastIndex= titleDescriptor.showMoreNode.getIndexInParent();
				var pendingAdd= titleDescriptor.pendingAdd;
				for(var index = 0, itemToAdd; itemToAdd = pendingAdd[index]; index++) {
					this.addItemByKey(itemToAdd.args, lastIndex+ index);
				}
			}
			
			var pendingAdd= titleDescriptor.pendingAdd;
			
			//Remove the show more title node only when no max limit is used, as we need to have it shown all the time for the that case, but here, just remove it
			this.remove(titleDescriptor.showMoreNode);
			titleDescriptor.showMoreNode.destroyRecursive();
			delete titleDescriptor.showMoreNode;
			
			titleDescriptor.pendingAdd= [];
			
			this.inherited("_onClick", arguments);
		}, titleDescriptor);
	},
	
	addSeparatorByKey: function(/*String|Integer */ insertIndexOrKey){
		return this.addSeparator(this.getIndexForKey(insertIndexOrKey));
	},
	
	/** inset a list of items in a specific category  */
	addItemsByKey: function(argsList, insertIndexOrKey){
		var insertIndex= this.getIndexForKey(insertIndexOrKey);
		for(var index = 0, arg; arg = argsList[index]; index++) {
			arg.key= insertIndexOrKey;
			this.addItemByKey(arg, insertIndex+ index);
		}
	},
	
	getIndexForKey: function(insertIndexOrKey){
		if(!insertIndexOrKey|| !dojo.isString(insertIndexOrKey)){
			return insertIndexOrKey; 
		}
		
		var cateogry= this._titles[insertIndexOrKey];
		if(!cateogry){
			return insertIndexOrKey; 
		}
		
		var categoryIndex= cateogry.node.getIndexInParent();
		if(categoryIndex>-1){
			var items= this.getItems();
			
			for(var index = categoryIndex+1, item; item = items[index]; index++) {
				if(item instanceof jazz.ui.menu.NodeMenuItem){
					break;
				}
			}
			return index; 
		}
		
		return insertIndexOrKey;
	},
	
	destroy: function(){
		this._titles= {};
		return this.inherited(arguments);
	},
	
	/**************************************************(Start) EditorTrackingWidget methods implementation **************************************/
	onAddRecentlyViewed: function(editorInput) {
		this._addRecentlyViewedTitle();
		var untitled = this._capabilityMessages.untitled;
		var name= jazz.util.html.escape(editorInput.getPresentationName() ? editorInput.getPresentationName() : untitled);
		var icon= editorInput.getIconUri() || editorInput.isLoaded() ? editorInput.getIconUri() : this.getPageHelper().getEditorTracker().loadingIcon;
		var args = {
			_id: editorInput.getId(),
			editorInput: editorInput,
			label: editorInput.isLoaded() ? name : this.getPageHelper().getEditorTracker().loadingMessage,
			key: "recentlyViewed",
			onClick: dojo.hitch(editorInput.__cmItem, function() {
				location.href = this.editorInput.getPresentationUri();
			}),
			title: editorInput.getName() !== editorInput.getTooltip() ? editorInput.getTooltip() : untitled,
		};
		if(icon){
			args.icon= icon;
		}
		editorInput.__cmItem = this.addItemByKey(args, this._recentlyViewedTitle.getIndexInParent() + 1);
		editorInput.__cmItem._isLoaded = editorInput.isLoaded();
		
		if(this.recentlyViewedlimit){
			var items= this.getItemsFor("recentlyViewed");
			var itemsLength= items.length;
			if(itemsLength>this.recentlyViewedlimit){
				var item= items[itemsLength-1];
				PageHelper.getInstance(this.getPageId()).closeEditorInput(item.params.editorInput, true, true);
				this.remove(item);
				item.destroyRecursive();
			}			
		}
		
	},
	
	handlePostResourceDeleted: function(data) {
		var	deletedResourceUri = data.uri;
		var items= this.getItemsFor("recentlyViewed");
		if (items && items.length > 0) {
			var itemCount = items.length;
			for (var i = 0; i < itemCount; i++) {
				var item = items[i];
				if (item.editorInput && item.editorInput.resourceInfo) {
					var uri = item.editorInput.resourceInfo.uri;
					if (uri == deletedResourceUri) {
						this.remove(item);
						item.destroyRecursive();
						if (itemCount === 1 && this._recentlyViewedTitle) {
							this.remove(this._recentlyViewedTitle).destroyRecursive();
							delete this._recentlyViewedTitle;
						}
						return;
					}
				}
			}
		}
	},
	
	onAddUnsavedWork: function(editorInput) {
		this._addUnsavedWorkTitle();
		this._updateUnsavedWorkTitle();
		
		var presentationName= jazz.util.html.escape(editorInput.getPresentationName() ? editorInput.getPresentationName() : this._capabilityMessages.untitled);
		var name= presentationName? dojo.string.substitute(this._capabilityMessages.dirtyName, [presentationName]) : this._capabilityMessages.untitled;
		var icon= editorInput.getIconUri() || editorInput.isLoaded() ? editorInput.getIconUri() : this.getPageHelper().getEditorTracker().loadingIcon;
		
		var args= {
			_id: editorInput.getId(),
			editorInput: editorInput,
			label: name,
			onClick: dojo.hitch(editorInput.__cmItem, function() {
				location.href = editorInput.getPresentationUri();
			}),
			title: editorInput.getName() !== editorInput.getTooltip() ? editorInput.getTooltip() : this._capabilityMessages.untitled
		};
		
		if(icon){
			args.icon= icon;
		}
		editorInput.__cmItem = this.addItemByArgs(args, this._unsavedWorkTitle.getIndexInParent() + 1);
	},
	
	onRemoveRecentlyViewed: function(editorInput) {
		this.remove(editorInput.__cmItem).destroyRecursive();
		delete editorInput.__cmItem;
		if (this.getPageHelper().getEditorTracker().getRecentlyViewedCount() === 0 && this._recentlyViewedTitle) {
			this.remove(this._recentlyViewedTitle).destroyRecursive();
			delete this._recentlyViewedTitle;
		}
	},
	
	onRemoveUnsavedWork: function(editorInput) {
		this.remove(editorInput.__cmItem).destroyRecursive();
		delete editorInput.__cmItem;
		if (this.getPageHelper().getEditorTracker().getUnsavedWorkCount() === 0 && this._unsavedWorkTitle) {
			this.remove(this._unsavedWorkTitle).destroyRecursive();
			delete this._unsavedWorkTitle;
		}
	},
	
	onUpdate: function(editorInput, isDirty) {
		var item = editorInput.__cmItem,
			presentationName;
		if (item) {
			if(item._destroyed){
				//This means that this item has been removed to due to whatever reason (such as maintaining maximum number)
				delete editorInput.__cmItem;
				return;
			}
			item._isLoaded = true;
			presentationName = jazz.util.html.escape(editorInput.getPresentationName() ? editorInput.getPresentationName() : this._capabilityMessages.untitled);
			item.set("label", isDirty
					? dojo.string.substitute(this._capabilityMessages.dirtyName, [presentationName])
					: presentationName);
			item.iconNode.src = editorInput.getIconUri()||this._blankIcon;
			item._setStyleAttr("title:"+ editorInput.getName() !== editorInput.getTooltip() ? editorInput.getTooltip() : this._capabilityMessages.untitled);
		}
	},
	/**************************************************(End) EditorTrackingWidget methods implementation ****************************************/
	
	_addRecentlyViewedTitle: function() {
		if (!this._recentlyViewedTitle) {
			if (this._unsavedWorkTitle) {
				this._unsavedWorkTitle.wrapHint = wrapHint.AUTO_WRAP;
				this._recentlyViewedTitle = this.addTitle(this._capabilityMessages.recentlyViewed, {
					key: "recentlyViewed",
					insertIndex: this._unsavedWorkTitle.getIndexInParent(),
					wrapHint: wrapHint.FORCE_WRAP});
			} else {
				this._recentlyViewedTitle = this.addTitle(this._capabilityMessages.recentlyViewed, {
					key: "recentlyViewed",
					wrapHint: wrapHint.FORCE_WRAP});
			}
		}
	},

	_addUnsavedWorkTitle: function() {
		if (!this._unsavedWorkTitle) {
			if (this._recentlyViewedTitle) {
				this._unsavedWorkTitle = this.addTitle(this._capabilityMessages.unsavedWork);
			} else {
				this._unsavedWorkTitle = this.addTitle(this._capabilityMessages.unsavedWork, {
					wrapHint: wrapHint.FORCE_WRAP});
			}
			dojo.addClass(this._unsavedWorkTitle.containerNode, "unsaved");
		}
	},

	_updateUnsavedWorkTitle: function() {
		this._unsavedWorkTitle.set("label", dojo.string.substitute(this._capabilityMessages.unsavedWorkCount, [this.getPageHelper().getEditorTracker().getUnsavedWorkCount()]));
	},
	
	/*
	 * Start listening
	 */
	_addCapabilityComponentMenuListeners: function(pageHelper) {
		if(!pageHelper){
			console.error("Must provide a pageHelper to start listening ");
			return;
		}
		
		if (pageHelper.isStarted()) {
			this.startListeningOnEditorTracker();
		} else {
			/*The page helper is not started yet. So, wait till it becomes started, and then start listening */
			var tempConnect = this.connect(pageHelper, "startup", dojo.hitch(this, function(){
				this.disconnect(tempConnect);
				this.startListeningOnEditorTracker();
			}));
		}
		dojo.subscribe("rmps.web.app.explorer.deleteResource", dojo.hitch(this, this.handlePostResourceDeleted));
	}
	
});

})();
