/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.editor.EditorInput");

dojo.declare("rmps.ui.editor.EditorInput", null, {
	
	/*Required to determine the pageId that this editorInput belongs to */
	_pageId: null,
	
	_params: null,
	_state: null,
	
	_hasSidebarExplorer: false, // Default
	
	constructor: function(/*Object*/params) {
		this._params = dojo.isObject(params) ? params : {};
		this._state = {};
		
		/*Assign the pageId at the contructor as this is where the page is currently active
		* So, we will keep the original page id that was iniitlaized when creating
		* the EditorInput regarsless the current active page is changed or not */
		this._pageId= net.jazz.ajax.ui.PlatformUI.getWorkbench().currentPageId;
	},
	
	getParams: function() {
		return this._params;
	},
	
	setParams: function(/*Object*/params) {
		this._params = params;
	},
	
	getId: function() {
		// to be implemented by clients
	},
	
	isLoaded: function() {
		return true;
	},
	
	unload: function() {
		// to be implemented by clients to perform cleanup
	},
	
	load: function() {
		// to be implemented by clients if loading is supported
		return null;/* Deferred */
	},
	
	getIconUri: function() {
		return null;
	},
	
	getLargeIconUri: function() {
		return null;
	},
	
	getName: function() {
		// to be implemented by clients
	},
	
	getPresentationName: function() {
		//Used for the browser title to be shown when opening the editor associated with this editor input
		//Return name by default as the presentation name 
		return this.getName();
	},
	
	getTooltip: function() {
		return this.getName();
	},
	
	/**
	 * @param state optional state parameter. When present the presentation URI
	 *              should use the passed in state to calculate the URI, otherwise
	 *              the state of the editor input should be used.
	 */
	getPresentationUri: function(state) {
		state = state? dojo.mixin(state, this.getState()): this.getState();
		var baseUri = this.getBasePresentationUri(state);
		var stateQuery = dojo.objectToQuery(state);
		if (stateQuery.length > 0) {
			if (baseUri.indexOf('#') === -1) {
				stateQuery = "#" + stateQuery;
			} else {
				stateQuery = "&" + stateQuery;
			}
		}
		return baseUri + stateQuery;
	},
	
	getBasePresentationUri: function(state) {
		// return the base presentation uri used by #getPresentationUri() as the uri prefix
		return "#";
	},
	
	removeState: function(key) {
		if (this._state[key]) {
			delete this._state[key];
			this.onStateChanged(dojo.clone(this._state));
		}
	},
	
	addState: function(key, value) {
		if (this._state[key] !== value) {
			if (value === null) {
				delete this._state[key];
			} else {
				this._state[key] = value;
			}
			this.onStateChanged(dojo.clone(this._state));
		}
	},
	
	getState: function(key) {
		if (key) {
			return this._state[key];
		}
		return dojo.clone(this._state);
	},
	
	getPageId: function(){
		return this._pageId;
	},
	
	onStateChanged: function(state) {
		// listeners should connect to this method
	},
	
	isRecentlyViewedSupported: function() {
		return true; // Default
	},
	
	hasSidebarExplorer: function() {
		return this._hasSidebarExplorer; 
	},
	
	hasExplorerBreadbrumbs: function() {
		return false;
	}
});

}());
