/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.editor.EditorRegistry");

dojo.require("jazz.core.loader");
dojo.require("jazz.util.array");

var entries = [];
var typeToEditorInput = {};

rmps.ui.editor.EditorRegistry = {
	
	createEditorInput: function(type, args) {
		var editorInputClass = typeToEditorInput[type];
		var deferredInput = new dojo.Deferred();
		
		if (editorInputClass) {
			if (dojo.isString(editorInputClass)) {
				var classConstructor = dojo.getObject(editorInputClass);
				if (!classConstructor) {
					jazz.core.loader.load_async(editorInputClass, dojo.hitch(this, function() {
						classConstructor = dojo.getObject(editorInputClass);
						deferredInput.callback(new classConstructor(args));
					}));
				}
				else {
					deferredInput.callback(new classConstructor(args));
				}
			} else {
				deferredInput.callback(new editorInputClass(args));
			}
		}
		else {
			deferredInput.callback(null);
		}
		
		return deferredInput;
	},
	
	getEditorClass: function(editorInput) {
		var deferredEditorClass = new dojo.Deferred();
		
		for (var i = 0; i < entries.length; ++i) {
			var entry = entries[i];
			if (entry.matcher(editorInput)) {
				if (dojo.isString(entry.editorClass)) {
					var classConstructor = dojo.getObject(entry.editorClass);
					if (!classConstructor) {
						jazz.core.loader.load_async(entry.editorClass, dojo.hitch(this, function() {
							classConstructor = dojo.getObject(entry.editorClass);
							deferredEditorClass.callback(classConstructor);
						}));
					}
					else {
						deferredEditorClass.callback(classConstructor);
					}
					return deferredEditorClass;
				} else {
					deferredEditorClass.callback(entry.editorClass);
					return deferredEditorClass;
				}
			}
		}
		
		deferredEditorClass.callback(null);
		return deferredEditorClass;
	},
	
	createEditor: function(editorInput) {
		var classConstructorDeferred = this.getEditorClass(editorInput);
		var editorDeferred = new dojo.Deferred();
		if (classConstructorDeferred) {
			classConstructorDeferred.then(function(classConstructor) {
				if (classConstructor) {
					editorDeferred.callback(new classConstructor({editorInput: editorInput}));
				}
			});
		}
		return editorDeferred;
	},
	
	register: function(editorClass, matcher, priority) {
		if (isNaN(priority)) {
			priority = 0;
		}
		jazz.util.array.insertByPriority(entries, {
				p: priority,
				matcher: matcher,
				editorClass: editorClass
			});
	},
	
	registerEditorInput: function(editorInputClass, type) {
		if (typeToEditorInput[type]) {
			throw new Error("Editor input class '" + editorInputClass + "' is already registered for type '" + type + "'");
		}
		typeToEditorInput[type] = editorInputClass;
	}
};

})();