/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.editor.EditorTracker");

dojo.requireLocalization("rmps.ui.common", "CommonMessages");
var commonMessages = dojo.i18n.getLocalization("rmps.ui.common", "CommonMessages");

/* 
 * This class is designed to be overriden in order to enable the sub classes to add listeners on their current used editor util 
 */
dojo.declare("rmps.ui.editor.EditorTracker", null, {
		
	_listenrs: null,
	
	/*Can be extended to change the icon or loading message */
	loadingMessage: commonMessages.loading,
	loadingIcon: dojo.moduleUrl("rmps.ui", "common/images/spinner.gif"),
	
	
	constructor: function(params) {
		this._listenrs= [];	
		this.pageHelper= params.pageHelper;
		if(!this.pageHelper){
			rmps.core.error.createError(new Error("Must pass in editor util in order to construct an EditorTracker")).dispatch();
			return;
		}
		
		/*Let's start listening */
		this._startListening();
	},
	
	getPageHelper: function(){
		return this.pageHelper;
	},
	
	/*
	 * Add listeners on the passed editor util object 
	 * 
	 *This method is not expected to be called by subclasses 
	 */
	_startListening: function(){
		this._listenrs.push(dojo.connect(this.pageHelper, "onEditorLoading", this, this.onEditorLoading));
		this._listenrs.push(dojo.connect(this.pageHelper, "onEditorLoadFailed", this, this.onEditorLoadFailed));
		this._listenrs.push(dojo.connect(this.pageHelper, "onEditorClosed", this, this.onEditorClosed));
		this._listenrs.push(dojo.connect(this.pageHelper, "onEditorOpened", this, this.onEditorOpened));
		this._listenrs.push(dojo.connect(this.pageHelper, "onEditorActivated", this, this.onEditorActivated));
	},
	
	/*
	 * Disconnect the defined listeners on the editor util object. 
	 * 
	 * MUST be called by the client (not necessary in the sub class) in order to stop listening when destroying the object 
	 */	
	stopListening: function(){
		if(!this._listenrs){
			return;
		}
		
		while (this._listenrs.length > 0) {
			dojo.disconnect(this._listenrs.pop());
		}
	},
	
	/*listeners  methods to be implemented */	
	onEditorLoading: function(editorInput) {
		console.error("onEditorLoading is not implemented for:"+ this.declaredClass);
	},
	onEditorLoadFailed: function(editorInput, error) {
		console.error("onEditorLoadFailed is not implemented for:"+ this.declaredClass);
	},
	onEditorClosed: function(editor) {
		console.error("onEditorClosed is not implemented for:"+ this.declaredClass);
	},
	onEditorOpened: function(editor) {
		console.error("onEditorOpened is not implemented for:"+ this.declaredClass);
	},
	onEditorActivated: function(editor) {
		console.error("onEditorActivated is not implemented for:"+ this.declaredClass);
	},
	
	/*Eventing */
	onAddRecentlyViewed: function(editorInput) {},
	onAddUnsavedWork: function(editorInput) {},
	onUpdate: function(editorInput) {},
	onRemoveRecentlyViewed: function(editorInput) {},
	onRemoveUnsavedWork: function(editorInput) {},
	
	/*Methods to be overridden */
	getUnsavedWorkCount: function() {
		console.error("getUnsavedWorkCount is not implemented for:"+ this.declaredClass);
		},
	getRecentlyViewedCount: function() {
		console.error("getRecentlyViewedCount is not implemented for:"+ this.declaredClass);
		},
	getRecentlyViewed: function() {
		console.error("getRecentlyViewed is not implemented for:"+ this.declaredClass);
		},
	getUnsavedWork: function() {
		console.error("getUnsavedWork is not implemented for:"+ this.declaredClass);
		}
	
	
});

})();
