/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.editor.EditorTrackingWidget");


dojo.require("dijit._Widget");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("net.jazz.ajax.ui.ErrorPage");
dojo.require("rmps.core.error");
dojo.require("rmps.ui.editor.PageHelper");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

/* 
 * If we want to have a widget that supports listening for a given editor tracker, we can extend this class.
 * 
 * It is expected that a widget must call for the method startListeningOnEditorTracker at some point in order to get the listening process started. This
 * can be done at place like postcreate. Once it is started, the listenrs will be only disconnected at the destroying time for the widget
 */
dojo.declare("rmps.ui.editor.EditorTrackingWidget", dijit._Widget, {
	
	editorTracker: null,
	
	getPageHelper: function(){
		if(!this.pageHelper){
			this.pageHelper= rmps.ui.editor.PageHelper.getInstance(this.getPageId());
		}
		return this.pageHelper;
	},
	
	startListeningOnEditorTracker: function() {
		try {
			this._addEditorTrackingListeners(this.getPageHelper().getEditorTracker());
		} catch (error) {
			/*Make sure that the sub classes will load even if we have some issues here */
			rmps.core.error.createError(error).getGenericError();
		}
	},
		
	_addEditorTrackingListeners: function(editorTracker){
		this.connect(editorTracker, "onAddRecentlyViewed", this.onAddRecentlyViewed);
		this.connect(editorTracker, "onAddUnsavedWork", this.onAddUnsavedWork);
		this.connect(editorTracker, "onRemoveRecentlyViewed", this.onRemoveRecentlyViewed);
		this.connect(editorTracker, "onRemoveUnsavedWork", this.onRemoveUnsavedWork);
		this.connect(editorTracker, "onUpdate", this.onUpdate);
		dojo.forEach(editorTracker.getRecentlyViewed(), this.onAddRecentlyViewed, this);
		dojo.forEach(editorTracker.getUnsavedWork(), this.onAddUnsavedWork, this);		
	},
	
	/*These methods should be implemented by clients in order to add the required listeners on the editor tracker */
	onAddRecentlyViewed: function(editorInput){/*Empty Block */},
	onAddUnsavedWork: function(editorInput){/*Empty Block */},
	onRemoveRecentlyViewed: function(editorInput){/*Empty Block */},
	onRemoveUnsavedWork: function(editorInput){/*Empty Block */},
	onUpdate: function(editorInput, isDirty){/*Empty Block */}
	
	
});

})();
