/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.editor.EditorUtil");

dojo.require("jazz.util.Navigation");

dojo.require("rmps.core.error");

dojo.require("rmps.ui.editor.EditorRegistry");
dojo.require("dojox.collections.Dictionary");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.requireLocalization("rmps.ui.editor", "EditorUtilMessages");

var messages = dojo.i18n.getLocalization("rmps.ui.editor", "EditorUtilMessages");

/*Deprecated: Should use rmps.ui.editor.PageHelper instead*/
dojo.declare("rmps.app.ui._EditorUtil", null, {
	
	// mixin
	domNode: null,
	defaultUri: null,
	
	// private properties
	_currentView: null, //{}
	_editorCache: null, // Dictionary
	_editorTracker: null,
	
	constructor: function() {
		this._currentView = {};
		this._editorCache = new dojox.collections.Dictionary();
		if (registerNavigateHandler) {
			registerNavigateHandler();
			registerNavigateHandler = null;
		}
		dojo.deprecated("rmps.app.ui.EditorUtil", "Use PageHelper instead", "M5");
	},
	
	// eventing
	
	onEditorLoading: function(editorInput) {},
	
	onEditorLoadFailed: function(editorInput, error) {
		window.location.href = this.defaultUri;
	},
	
	onEditorClosed: function(editor) {},
	
	onEditorOpened: function(editor) {},
	
	onActiveEditorChanged: function(editor) {},
	
	onEditorDeactivated: function(editor) {},
	
	/*Could be connected by other clients. The implemntation here is to set the browser title when the editor gets activated */
	onEditorActivated: function(editor) {
		editor.setTitle();
	},
	
	// public API
	
	getActiveEditorInput: function() {
		return this._currentView.editorInput;
	},
	
	openEditor: function(editorInput) {
		if (this._currentView.editorInput && editorInput.getId() === this._currentView.editorInput.getId()) {
			this._currentView.editorInput.setParams(editorInput.getParams());
			if (this._currentView.editor) {
				// the editor for the given resource URI is initialized
				if (this._currentView.editor.activate) {
					this._currentView.editor.activate();
				}
				this.onEditorActivated(this._currentView.editor);
				return;
			}
		}
		
		if (this._currentView.editor) {
			if (this._currentView.editor.deactivate) {
				this._currentView.editor.deactivate();
			}
			this._currentView.editor.domNode.style.display = "none";
			this.onEditorDeactivated(this._currentView.editor);
			delete this._currentView;
		}
		
		this._currentView = {};
		
		var item = this._editorCache.item(editorInput.getId());
		
		if (item) {
			this._editorCacheUpdate(editorInput.getId(), item);
			item.editorInput.setParams(editorInput.getParams());
			
			this._currentView.editorInput = item.editorInput;
			
			if (item.editor) {
				this._currentView.editor = item.editor;
				
				this._currentView.editor.domNode.style.display = "";
				
				if (!this._currentView.editor._started) {
					this._currentView.editor.startup();
				}
				if (this._currentView.editor.activate) {
					this._currentView.editor.activate();
				}
				
				this.onEditorActivated(this._currentView.editor);
				this.onActiveEditorChanged(this._currentView.editor);
				
			} else {
				this.onEditorLoading(item.editorInput);
			}
			
			return;
		}
		
		this._currentView.editorInput = editorInput;
		
		item = this._cacheEditor(editorInput);
		
		this.onEditorLoading(editorInput);
		
		if (editorInput.isLoaded()) {
			setTimeout(dojo.hitch(this, function() {
				this._openEditor(editorInput);
			}), 0);
			
		} else {
			var loaded = editorInput.load();
			item.loadDeferred = loaded;
			
			loaded.then(
				dojo.hitch(this, function() {
					delete item.loadDeferred;
					this._openEditor(editorInput);
				}),
				dojo.hitch(this, function(error) {
					delete item.loadDeferred;
					this._editorCache.remove(editorInput.getId());
					this.onEditorLoadFailed(editorInput, error);
					// Error was dispatched in "editorInput.load" function
				}));
		}
	},
	
	closeEditor: function(editor, force, doNotOpenPreviousEditor) {
		return this.closeEditorInput(editor.getEditorInput(), force, doNotOpenPreviousEditor);
	},
	
	closeEditorInput: function(editorInput, force, doNotOpenPreviousEditor) {
		return this._closeEditorInput(editorInput, force, doNotOpenPreviousEditor);
	},
	
	closeAllEditors: function() {
		// TODO
//		var values = this._editorCache.getValueList();
//		for (var i = 0; i < values.length; ++i) {
//			// TODO
//		}
//		this._editorCache.clear();
//		
//		if (this._navigateHandle) {
//			this._unregisterNavigateHandler();
//		}
	},
	
	getEditor: function(editorInput) {
		if (editorInput) {
			var item = this._editorCache.item(editorInput.getId());
			if (item) {
				return item.editor;
			}
		} else if (this._currentView) {
			return this._currentView.editor;
		}
		return null;
	},
	
	// private
	
	_closeEditorInput: function(editorInput, force, doNotOpenPreviousEditor) {
		var item = this._editorCache.item(editorInput.getId());
		
		if (item && (!item.editor || !item.editor.isDirty() || force ||
				confirm(dojo.string.substitute(messages.confirmCloseIfDirty, [editorInput.getName()])))) {
			
			this._editorCache.remove(editorInput.getId());
			
			var editor = item.editor;
			
			var currentInputId = this._currentView.editorInput && this._currentView.editorInput.getId();
			var areInputsEqual = currentInputId === editorInput.getId();
			var openNewEditor = !doNotOpenPreviousEditor && areInputsEqual;
			
			if (editor) {
				if (areInputsEqual) {
					if (editor.deactivate) {
						editor.deactivate();
					}
					this.onEditorDeactivated(editor);
					this._currentView = {};
				}
				this.domNode.removeChild(editor.domNode);
				editor.destroyRecursive();
				
				this.onEditorClosed(editor);
			} else if (item.loadDeferred) {
				item.loadDeferred.cancel();
				delete item.loadDeferred;
			}
			
			editorInput.unload();
			
			if (openNewEditor) {
				var items = this._editorCache.getValueList();
				if (items.length > 0) {
					window.location.href = items[items.length - 1].editorInput.getPresentationUri();
				} else if (this.defaultUri){
					window.location.href = this.defaultUri;
				}
			}
			
			return true;
		}
		return false;
	},
	
	_openEditor: function(editorInput) {
		var editorDeferred = rmps.ui.editor.EditorRegistry.createEditor(editorInput);
		
		editorDeferred.then(dojo.hitch(this, function(editor) {
			if (!editor) {
				var error = new Error(dojo.string.substitute(messages.error_noEditor, [editorInput.getName()]));
				this.onEditorLoadFailed(editorInput, error);
				rmps.core.error.createError(error,
					dojo.string.substitute(messages.error_noEditor_action, [editorInput.getName()])).dispatch();
				return;
			}
			
			var item = this._editorCache.item(editorInput.getId());
			item.editor = editor;
			
			if (this._currentView.editorInput === editorInput) {
				this._currentView.editor = editor;
				this.domNode.appendChild(editor.domNode);
				editor.startup();
				
				this.onEditorOpened(editor);
				
				if (editor.activate) {
					editor.activate();
				}
				
				this.onEditorActivated(editor);
				this.onActiveEditorChanged(editor);
			} else {
				editor.domNode.style.display = "none";
				this.domNode.appendChild(editor.domNode);
				this.onEditorOpened(editor);
			}
		}));
	},
	
	_cacheEditor: function(editorInput) {
		var item = {
			editorInput: editorInput
		};
		this._editorCache.add(editorInput.getId(), item);
		return item;
	},
	
	_editorCacheUpdate: function(key, value) {
		this._editorCache.remove(key);
		this._editorCache.add(key, value);
	}
	
});

var editorUtilInstances = {};

var registerNavigateHandler = function() {
	jazz.util.Navigation.registerNavigateHandler(dojo.hitch(this, function() {
		var dirtyResources = "";
		for (var key in editorUtilInstances) {
			editorUtilInstances[key]._editorCache.forEach(function(item) {
				if (item.value.editor.isDirty()) {
					dirtyResources += "\n  " + item.value.editorInput.getName();
				}
			});
		}
		if (dirtyResources.length > 0) {
			return dojo.string.substitute(messages.navigateMessage, [dirtyResources]);
		}
		return dirtyResources;
	}),
	function() {
		return false;
	});
};

rmps.ui.editor.EditorUtil = {
	
	/**
	 * Creating a new editor util instance:
	 * The key must be a string and the owner dom node must be provided.
	 * The editor instance is associated with the owner dom node.
	 * 
	 * Getting an editor util instance:
	 * The key may be a string or a dom node. If the key is a string, a hash lookup
	 * is used to retrieve the editor util for the given key. If the key is a dom node,
	 * this method will walk up the dom node heirarchy until the owner of an editor util
	 * is found.
	 */
	getInstance: function(/*String|domNode*/key, /*domNode*/owner) {
		if (key && owner) {
			if (!dojo.isString(key)) {
				throw new Error("The 'key' must be a String when the owner is specified.");
			}
			var _key = owner.getAttribute("_editorUtilKey");
			if (_key) {
				return editorUtilInstances[key];
			}
			owner.setAttribute("_editorUtilKey", key);
			var instance = new rmps.app.ui._EditorUtil();
			editorUtilInstances[key] = instance;
			return instance;
		}
		if (dojo.isString(key)) {
			var instance = editorUtilInstances[key];
			if (!instance) {
				instance = new rmps.app.ui._EditorUtil();
				editorUtilInstances[key] = instance;
			}
			return instance;
		}
		
		var parentNode = key;
		while (parentNode) {
			if(parentNode.getAttribute){
				var _key = parentNode.getAttribute("_editorUtilKey");
				if (_key) {
					return editorUtilInstances[_key];
				}
			}
			parentNode = parentNode.parentNode;
		}
		return null;
	}
	
}

})();
