/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.editor.PageInitializer");
dojo.require("rmps.ui.editor.EditorMRU");
dojo.require("net.jazz.ajax.ui.PlatformUI");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;

/*
 * An object must be instance of  PageInitializer in order to be able to initialize PageHelper instances. It's exepcted that an application page (such as
 * AbstractPage) must extend this class and add its own contribution for some methods based on its requirement. This is required if we want to use PageHelper
 * and make sure that we will be able to control the lifecycle of the page (such as opening editors, etc)
 */
dojo.declare("rmps.ui.editor.PageInitializer", null, {
	
	_actions: null,
	
	constructor: function(){
		this._actions= [];
	},
	
	/*
	 * Can be extneded by subclasses to provide specilaized handeling for most recent viewed items. Otheriwse, we use the default implementation of
	 * rmps.ui.editor.EditorMRU 
	 */
	getEditorTrackerClass: function(){
		return rmps.ui.editor.EditorMRU;
	},
	
	/*
	 * Return the prefix for the actions. The prefix is mostly the same as page id, but in some cases, clients can define different prefix for their actions
	 * other than the page id. Thus, a page can extend this class to define different prefix 
	 */
	getActionsPrefix: function(){
		return this.getPageId();
	},
	
	/*Return the id for the page. Must be the same id that is defined using net.jazz.ajax.pages extnesion point*/
	getPageId: function(){
		return PlatformUI.getWorkbench().currentPageId;
	},
	
	/*Returns the list of actions that have been defined in the page in all places (the main page, or other places that extend/contibute it) */
	getActionsList: function(){
		return this._actions;
	},
	
	/*What's the default uri for the page? */
	getDefaultUri: function(){
		return null;
	},
	
	/*
	 * The view for the page itself. By default it should be the for the page itselft. It is used to create the editor view
	 * In some cases, clients can require creating the editor view based on a nother node 
	 */
	getView: function(){
		throw new Error("getView has null implemntation");
	},
	
	/*The appending ode for the current view. It should be the dom node by default */
	getContainerNode: function(){
		throw new Error("getContainerNode has null implemntation");
	}
		
});


})();
