/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.explorer.Explorer");

dojo.require("dijit._Widget");
dojo.require("dijit.Menu");

dojo.require("rmps.core.RootServices");

dojo.require("rmps.ui.explorer.ExplorerTree");
dojo.require("rmps.ui.explorer.ExplorerModel");

dojo.require("rmps.ui.explorer.ExplorerUtilities");
var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;

dojo.requireLocalization("rmps.ui.explorer", "ExplorerMessages");
var messages = dojo.i18n.getLocalization("rmps.ui.explorer", "ExplorerMessages");
/*
 * Explorer widget class
 */

dojo.declare("rmps.ui.explorer.Explorer", dijit._Widget, {
	
	"class": "rmps-app-explorer",

	// Call back function to allow explorer to create a new explorer instance.
	// Can be left null if operation is not supported.
	onNewExplorer: null,
	// The URI of the explorer root resource
	store: null,
	// Call back function which is called when the left arrow key is pressed
	// when focus is at the uppermost node in the tree.
	// Can be left null
	onLeftArrowExit: null,
	// Call back function which is called when the up arrow key is pressed
	// when focus is at the uppermost node in the tree. 
	// Can be left null
	onUpArrowExit: null,
	// Call back function which is called when the down arrow key is pressed
	// when focus is at the bottom node in the tree.
	// Can be left null
	onDownArrowExit: null,
	// Call back function which is called when a top node is removed from the explorer
	// Takes one parameter with the updated list of top nodes
	// Can be left null
	onRemoveTopNode: null,
	// The top nodes of the model should be set here (list of dojo.data.Item)
	topNodes: null,
	// If present, the nodes in visibleTopNodes will be the only visible top nodes from
	// the topNodes parameter. The rest will be added as an ellipsis node next to the
	// visible top nodes.
	visibleTopNodes: null,
	// A list of top nodes ids hidden by the ellipsis node
	hiddenTopNodes: null,
	
	// a flag determine how to handle new trees
	showNewTrees: true,

	
	_model: null,
	_tree: null,
	_menu: null,
	_popup: null,
	
	_back: null,
	_forward: null,
    
    _historyEnabled: false,
	_history: null,
	_dropTargets: null, //[]
	
	constructor: function() {
		this._dropTargets = [];
	},
	
	setShowNewTrees: function(value) {
		this.showNewTrees = value;
	},

	postCreate: function() {
		var state = null;
		var history = this._history;
		if (history && history.stack && history.depth>0)
		{
			state = history.stack[history.depth-1];
			this.setShowNewTrees(state.showNewTrees);
		}
		this._model = new rmps.ui.explorer.ExplorerModel({
	        store: this.store,
	        //rootChildren: rootChildren, Will set up after creation instead
	        childrenAttrs: ["children"],
	        deferItemLoadingUntilExpand: true
		});
		// Root item is not properly set up in ForestStoreModel. WA
		this._model.root.id = this._model.rootId;
		this._model.root.label = this._model.rootLabel;
		// Need to have the model already created to have the root element
		this._model.root.children = this._createRootChildrenList();
		this._tree = this._createTree(state);
		if (!state) {
		    this._enableHistory(true);
		    this._tree._saveState();
		    dojo.publish("rmps.ui.explorer.Explorer.loaded", [{
				explorerID: this._expMain.explorerID
			}]);
		}   // otherwise the history will be enabled in onLoad
	    this.domNode.appendChild(this._tree.domNode);
		this._tree.startup();
	},
	
	/*
	 * protected: type: Object
	 * 
	 * Hook to add client specific initialization for the tree, e.g.
	 * double click handling.  The contents of this Object will be passed
	 * to the ExplorerTree that is constructed
	 */
	treeInitializationData : null,
	
	_createTree: function(saved) {
		var tmpData = {
				model: this._model,
				showRoot: false,
				_explorer: this,
				onLeftArrowExit: this.onLeftArrowExit,
				onUpArrowExit: this.onUpArrowExit,
				onDownArrowExit: this.onDownArrowExit,
				fnIsChecked: this.fnIsChecked,
				fnOnCheck: this.fnOnCheck,
				onlyTypes: this.onlyTypes,
				enableMultiSelect: this.enableMultiSelect,
				noResourceLinks: this.noResourceLinks,
				_pinnedItems: [],
				_pinnedPaths: [],
				//dndController: "rmps.ui.explorer.ExplorerDndSource",
				_dropTargets: this._dropTargets
			};
		
		var treeData = this.treeInitializationData || {};
		
		// use treeData as the target so any properties set 
		// are not overridden
		dojo.mixin(treeData, tmpData);
		
		if (saved) {
			if (saved.treeState) {
				treeData._treeState = dojo.clone(saved.treeState);
			}
			if (saved.pinnedPaths) {
				treeData._pinnedPaths = dojo.clone(saved.pinnedPaths);
			}
		}
		return new rmps.ui.explorer.ExplorerTree(treeData);
	},
	
	destroy: function() {
		if (this._popup) {
			this._popup.destroyRecursive();
			delete this._popup;
		}
		if (this._menu) {
			this._menu.destroyRecursive();
			delete this._menu;
		}
		if (this._tree) {
			this._tree.destroyRecursive();
			delete this._tree;
		}
		if (this._model) {
			this._model.destroy();
			delete this._model;
		}
		
		this.inherited(arguments);
	},
	
	onContextMenu: function(aroundNode)
	{
		if (!this._menu) {

			this._menu = new dijit.Menu();
			dojo.addClass(this._menu.domNode, "rmps-app-explorer-node-context-menu");
					
			// FIXME: Move to main toolbar/context menu
			
			this._forward = new dijit.MenuItem({
				label: messages.forward,
				onClick: dojo.hitch(this, this.onForward),
				disabled : false
			});
			this._menu.addChild(this._forward);
			
			this._back = new dijit.MenuItem({
				label: messages.back,
				onClick: dojo.hitch(this, this.onBack),
				disabled : false
			});			
			this._menu.addChild(this._back);
			
			
			this._menu.startup();
			
			this.connect(this._menu, "_onBlur", this._closeMenu);
		}
		
		this._back.attr('disabled', !this._canGoBack());
		this._forward.attr('disabled', !this._canGoForward());
		
		// open pop-up menu
		dijit.popup.open({
			popup: this._menu,
			parent: this,
			around: aroundNode,
			orient: {'BR':'TL', 'BL':'TR', 'TR':'BL', 'TL':'BR'},
			onExecute: dojo.hitch(this, this._closeMenu),
			onCancel: dojo.hitch(this, this._closeMenu)
		});
		
		this._menu.focus();
	},
	
	setTopNodes: function(topNodes, visibleTopNodes, hiddenTopNodes) {
		this.topNodes = topNodes;
		this.visibleTopNodes = visibleTopNodes ? visibleTopNodes : topNodes;
		this.hiddenTopNodes = hiddenTopNodes ? hiddenTopNodes : [];
		
		var rootChildren = this._createRootChildrenList();
		
		this._model.setRootChildren(rootChildren);
	},
	
	getCurrentTopNodes: function() {
		// Get the current top nodes without ellipsis
		return dojo.filter(this._model.getRootChildren(), function(c) {
			return !("isEllipsis" in c);
		});
	},
	
	getTopNodes: function() {
		return this.topNodes;
	},
	
	getHiddenTopNodes: function() {
		return dojo.filter(this.topNodes, function(n) {
			return dojo.indexOf(this.visibleTopNodes, n) === -1;
		});
	},
	
	_createRootChildrenList: function() {
		if (!this.visibleTopNodes || !this.visibleTopNodes.length){
			this.setShowNewTrees(true);
			return this.topNodes;
		}
		if (this.showNewTrees) {
			this.visibleTopNodes = dojo.filter(this.topNodes, dojo.hitch(this, 
									function (id) {
										return dojo.indexOf(this.hiddenTopNodes, id) === -1;
									})); 
		} else {
			this.hiddenTopNodes = dojo.filter(this.topNodes, dojo.hitch(this, 
									function (id) {
										return dojo.indexOf(this.visibleTopNodes, id) === -1;
									}));
		}
			
		if (this.hiddenTopNodes.length)
		{
			var ellipsis = this._createEllipsisItem(this.hiddenTopNodes.length, this._model.root);
			var rootChildren = this.visibleTopNodes.slice(); // Copy array
			rootChildren.push(ellipsis);
			return rootChildren;
		}
		else {
			return this.visibleTopNodes.slice();
		}
	},
	
	// The list with all top nodes as first parameter. Second parameter the top node. 
	addTopNodeFromList: function(topNodes, topNode) {
		var topnode_id = this.getNodeURI(topNode);
		var currentNodes = this.getCurrentTopNodes();

		var found = false;
		for (var i = 0; i < currentNodes.length; i++) {
			var current_id = this.getNodeURI(currentNodes[i]);
			if (current_id === topnode_id)
			{ 
				found = true;
				break;
			}
		}
		if (found){
			currentNodes.splice(i, 1, topNode);
		}
		else {
			currentNodes.push(topNode);
			for (var j = 0; j < this.hiddenTopNodes.length; j++) {
				var hidden_id = this.getNodeURI(this.hiddenTopNodes[j]);
				if (hidden_id === topnode_id) {
					this.hiddenTopNodes.splice(j,1);
				}
			}
		}
		
		this.setTopNodes(topNodes, currentNodes, this.hiddenTopNodes);
	},
	
	// Takes the top node uri as parameter
	removeTopNode: function(topnode_id) {
		if (!topnode_id) {
			console.warn("topnode without an id was requsted to be removed.");
			return;
		}
		 
		var currentNodes = this.getCurrentTopNodes();
		var topNodes = this.getTopNodes();
		var found = false;
		for (var i = 0; i < currentNodes.length; i++) {
			if (topnode_id === this.getNodeURI(currentNodes[i])) {
				this.hiddenTopNodes.push(currentNodes[i]);
				currentNodes.splice(i, 1);
				found = true;
				break;
			}
		}
		if (!found) {
			console.warn("topnode id not found for removal.");
			return;
		}
		this.setShowNewTrees(false);
		this.setTopNodes(topNodes, currentNodes, this.hiddenTopNodes);
		
		if (this.onRemoveTopNode) {
			this.onRemoveTopNode(currentNodes);
		}
	},
		
	_closeMenu: function()
	{
		dijit.popup.close(this._menu);
	},
	
	_getHistoryEnabled : function() {
		return this._historyEnabled;
	},

	_enableHistory : function(value) {
		this._historyEnabled = value;
	},
	
	
	_canGoForward : function() {
		if (!this._history || !this._history.stack) {
			return false;
		}
		return this._history.depth < this._history.stack.length;
	},
	
	_canGoBack : function() {
		return this._history.depth > 1;
	},
	

	
	_rememberState : function(state) {
		if (this._history && this._history.stack && this._historyEnabled) {
			var hist = this._history;
			while (hist.stack.length > hist.depth)
			{
				hist.stack.pop();
			}
				
			while (hist.stack.length >= this._history.maxDepth) {
				hist.stack.shift();
				hist.depth--;
			}

			hist.stack.push(dojo.clone(state));
			hist.depth++;
			if (this._expMain) {
				this._expMain._saveHistory();
			}
		}
	},
	
	_restoreState : function(state) {
		this._enableHistory(false);
		this._tree.destroy();
	    this._tree = this._createTree(dojo.clone(state));	    
	    this.domNode.appendChild(this._tree.domNode); 
	    this._tree.startup();
	    
	    this.setShowNewTrees(state.showNewTrees);
	    if (this._expMain){
		    this.setTopNodes(this._expMain._topNodes, this._expMain.getTopNodesWithIds(state.visibleTopNodes), this._expMain.getTopNodesWithIds(state.hiddenTopNodes));
		    this._expMain._updateToolbarButtons();
		    this._expMain.resize();		    
	    }
	    this._enableHistory(true);
	},
	
	onForward : function() {
		if (this._canGoForward()) {
			var hist = this._history;
			this._restoreState(hist.stack[hist.depth]);
		    hist.depth++;
			if (this._expMain) {
				this._expMain._saveHistory();
			}
		}
	},
	
	onBack : function() {
		if (this._canGoBack()) {
			var hist = this._history;
			hist.depth--;
			this._restoreState(hist.stack[hist.depth-1]);
			if (this._expMain) {
				this._expMain._saveHistory();
			}
		}
	},
	
	onCollapseAll: function() {
		if (this._tree) {
			this._tree.collapseRootAll(true);
		}
	},
	
	onShowPinnedOnly: function() {
		if (this._tree) {
			this._tree.OnCollapseAllExceptPinned();
		}
	},
	
	_createEllipsisItem : function(count, parentitem, parentchain)
	{ 
		var uuid = dojox.uuid.generateRandomUuid();
		
		if (parentitem.isEllipsis) {
			// If parent is ellipsis item then get the correct parent item.
			var parentparentchain = parentitem.ellipsisParentChain;
			parentitem = parentparentchain[parentparentchain.length - 1];
		}
		
		var ellipsis = {
			isEllipsis: true,
			ellipsisNumber: count,
			ellipsisUUID: uuid,
			ellipsisParent: parentitem,
			ellipsisSiblings: parentchain ? false : true,
			ellipsisParentChain: parentchain
		};
		
		if (parentitem.root) {
			this.setShowNewTrees(false);
		}
		
		return ellipsis;
	},
	
	unpinAll: function() {
		if (this._tree) {
			this._tree.unpinAll();
		}
	},
	
	getNodeURI: function(treenode) {
		var id = this.store.getIdentity(treenode);
		return ExplorerUtil.getResourceURIFromNodeID(id);
	},
	
	resize: function(size) {
		if (size && size.h) {
			// padding-top: 4px;
			var h = size.h - 4;
			this.domNode.style.height = h + "px";
			this._tree.domNode.style.height = h + "px";
		}
	}
});

}());
