/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.explorer.ExplorerDndSource");

dojo.require("dijit.tree.dndSource");

dojo.require("rmps.ui.explorer.ExplorerUtilities");
var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;

dojo.declare("rmps.ui.explorer._DropTarget", null, {
	
	// mix-in
	widgetId: null,
	dndSource: null,
	
	constructor: function(args) {
		dojo.safeMixin(this, args);
	}
});

dojo.declare("rmps.ui.explorer.ExplorerDndSource", dijit.tree.dndSource, {
	
	_topics: null, //[]
	_widgets: null, //[]
	_widgetevents: null, //{}
	_targetDropped: null,
	
	constructor: function(args) {
		this._widgets = args._dropTargets || [];
		this._widgetevents = {};
		dojo.forEach(this._widgets, this.registerDropTarget, this);
		
		this._topics = [
		    dojo.subscribe("rmps.web.app.explorer.registerDropTarget", dojo.hitch(this, this.registerDropTarget)),
		    dojo.subscribe("rmps.web.app.explorer.unregisterDropTarget", dojo.hitch(this, this.unregisterDropTarget))
		];
	},
		
	destroy: function() {
		dojo.forEach(this._topics, dojo.unsubscribe);		
		dojo.forEach(this._widgets, this.unregisterDropTargetEvents, this);
		
		this.inherited(arguments);
	},
	
	registerDropTarget: function(data) {
		var widget = dijit.byId(data.widgetId);
		var droptarget = new rmps.ui.explorer._DropTarget({
			source: this,
			widget: data.widgetId
		});
		var events = [
			dojo.connect(widget.domNode, "onmouseenter", this, "onTargetOverEvent"),
			dojo.connect(widget.domNode, "onmouseleave", this, "onTargetOutEvent"),
			dojo.connect(widget.domNode, "onmouseup", this, "onTargetMouseUpEvent"),
			dojo.connect(widget.domNode, "onmousemove", this, "onTargetMouseMoveEvent")
		];
		this._widgetevents[data.widgetId] = { events: events, droptarget: droptarget };
	},
	
	unregisterDropTarget: function(data) {
		this.unregisterDropTargetEvents(data.widgetId);
		this._widgets.splice(this._widgets.indexOf(data.widgetid), 1);
	},
	
	unregisterDropTargetEvents: function(widgetid) {
		var item = this._widgetevents[widgetid];
		if (!item) {
			return;
		}
		dojo.forEach(item.events, dojo.disconnect);
		delete item.droptarget;
		this._widgetevents[widgetid] = null;
	},
	
	onTargetOverEvent: function(e) {
		if (!this.isDragging) {
			return;
		}
		dojo.dnd.manager().overSource(e.currentTarget);
	},
	
	onTargetOutEvent: function(e) {
		if (!this.isDragging) {
			return;
		}
		dojo.dnd.manager().outSource(e.currentTarget);
	},
	
	onTargetMouseUpEvent: function(e) {
		if (!this.isDragging) {
			return;
		}
		
		var node = e.currentTarget;
		this._targetDropped = dijit.getEnclosingWidget(node);
	},
	
	onDndDrop: function(source, nodes, copy){
		this.inherited(arguments);
		
		var w = this._targetDropped;
		if (!w || !w.onExplorerDrop) {
			return;
		}
		
		console.log("Target drop on widget: " + w.id);
		
		var items = [];
		dojo.forEach(nodes, function(n) {
			var w = dijit.getEnclosingWidget(n);
			var id = w.tree.model.getIdentity(w.item);
			items.push(ExplorerUtil.getResourceURIFromNodeID(id));
		});

		w.onExplorerDrop(items, copy);
	},
	
	onTargetMouseMoveEvent: function(e) {
		if (!this.isDragging) {
			return;
		}

		var node = e.currentTarget;
		var w = dijit.getEnclosingWidget(node);
		var canDrop = w.onExplorerDrag();
		dojo.dnd.manager().canDrop(canDrop);
	}
});

})();