/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.explorer.ExplorerMain");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.app.context");
dojo.require("jazz.ui.SwapListDialog");
dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.ui.toolbar.MenuButton");
dojo.require("jazz.ui.menu.MenuItem");
dojo.require("jazz.ui.MessageBox");

dojo.require("rmps.ui.explorer.Explorer");
dojo.require("rmps.ui.explorer.ExplorerStore");
dojo.require("rmps.ui.explorer.ExplorerUtilities");
dojo.require("rmps.core.explorer.ExplorerTreeService");
dojo.require("rmps.core.explorer.ExplorerConstants");
dojo.require("rmps.core.explorer.Viewpoint");
dojo.require("rmps.core.RootServices");
dojo.require("rmps.core.user.UserClient");
dojo.require("rmps.core.client");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("rmps.ui.explorer.ExplorerSetting");
dojo.require("rmps.core.explorer.ExplorerMainState");
dojo.require("rmps.app.ui.internal.sections.ExplorerFilterSection");
dojo.require("rmps.app.ui.internal.pages.design.widgets.ViewpointSelector");
dojo.require("rmpw.common.util.IdentifierUtil");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.ui.explorer", "ExplorerMessages");

var messages = dojo.i18n.getLocalization("rmps.ui.explorer", "ExplorerMessages");

var UserClient = rmps.core.user.UserClient;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var ExplorerTreeService = rmps.core.explorer.ExplorerTreeService;
var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;
var ViewpointUtilities = rmps.core.explorer.ViewpointUtilities;
var explorerMainState = rmps.core.explorer.explorerMainState;
var ViewpointSelector = rmps.app.ui.internal.pages.design.widgets.ViewpointSelector;
var SettingTypes = rmps.core.explorer.ExplorerConstants.SettingTypes;

var EXPLORER_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "explorer");

var EXPLORER_TREE_PATH = EXPLORER_PATH + "/tree";
var EXPLORER_CHAIN_PATH = EXPLORER_PATH + "/chain";
var EXPLORER_FILTER_PATH = EXPLORER_PATH + "/filters";

var USE_QUERIES_SETTING_NAME = "useQueries";

dojo.declare("rmps.ui.explorer.ExplorerMain", [dijit._Widget, dijit._Templated], {
	
	templateString:  '<div dojoAttachPoint="_div">' +
		'<div dojoAttachPoint="_divNoTrees">' + messages.noTreesToDisplay + '</div>' + 
		'<div dojoAttachPoint="_divShowTreesLink">' +
			'<a href="#" dojoAttachPoint="_linkNode" dojoAttachEvent="onclick: _onShowAllTreesLink" class="link-text"></a><br>' +
			'<span dojoAttachPoint="_spanNode"></span>' +
		'</div>' +
	'</div>',
	
	_topNodes: null,
	_subhandle: null,
	_subhandleFocusMany: null,
	_registerDropTarget: null,
	_unregisterDropTarget: null,
	_explorer: null,
	_menu : null,
	_userUri: null,
	_projectUri: null,
	_configurationUri: null,
	_toolbarContainer: null,
	_dropTargets: null, //[]
	_forceRefresh: null, //[]
	store: null,
	_useChildQueriesCB: null,
	
	constructor: function() {
		this._dropTargets = [];
		this._forceRefresh = [];
	},
	
	callTopService : function(callback, args, bForceRefresh) {
		dojo.addClass(this._divNoTrees, "rmps-app-explorer-main-hidden");
		// Call explorer/top service to get the trees
		if(this.showLoadingIcon) {
			this.showLoadingIcon();
		}

		var def = new dojo.Deferred();
		explorerMainState(bForceRefresh, this._projectUri, this._userUri).addCallback(this, function(ems) {
			var projectId = rmpw.common.util.IdentifierUtil.getIdentifier(this._projectUri);
			var viewpointURI = ems.getCurrentViewpointURI(this.explorerID);
			var handler = this.store.loadTopNodes(projectId, null, viewpointURI, this._configurationUri);
			handler.addCallback(dojo.hitch(this, function(response) {
				callback(response, args);
				if (this.hideLoadingIcon) {
					this.hideLoadingIcon();
				}
				def.callback(response);
			}));
			handler.addErrback(function(error) {
				if (this.hideLoadingIcon) {
					this.hideLoadingIcon();
				}
				def.errback(error);
			});			
		});
		return def;
	},
	
    restoreFromServer: function() {
    	if (this._userUri) {  
    		var def = explorerMainState(false, this._projectUri, this._userUri);
    		def.addCallback(dojo.hitch(this, function(ems){
    			var value = ems.get(this.explorerID);
    			if (value) {
    				this._history = value;
    			} else {
    				this.initHistory();
    			}
    		}));
    		def.addErrback(dojo.hitch(this, function() {
    			this.initHistory();
    		}));
    		return def;
    	}
    	
    	var def = new dojo.Deferred(); 
    	def.addCallback(function() { /* do nothing */ });
		def.resolve();
		return def;
    },
    
	_saveHistory : function(viewpointURI) {
		this._updateToolbarButtons();
		
		if (this._userUri) {  // can only save history if there is a user id - invoking from OSLC there won't be a user
			if (viewpointURI) {
				this._history.viewpointURI = viewpointURI;	
			}
			var def = new dojo.Deferred();
			if (this._history) {
				var defEms = explorerMainState(false, this._projectUri, this._userUri);
				defEms.addCallback(dojo.hitch(this, function(ems) {
					ems.set(this._history, this.explorerID).then(def.callback, def.errback);
				}));
				defEms.addErrback(function() {
					def.errback();
				});				
			} else {
				def.errback();
			}
			return def;	
		}
		
		var def = new dojo.Deferred(); 
		def.addCallback(function() { /* do nothing */ });
		def.resolve();
		return def;
	},
		 
	postCreate: function() {
		// Subscribe to show in explorer channel
		this._subhandle = dojo.subscribe("rmps.web.app.explorer.sidebar.show", dojo.hitch(this, this._onShowExplorer));
		this._subhandleFocusMany = dojo.subscribe("rmps.web.app.explorer.sidebar.focusMany", dojo.hitch(this, this._onFocusMany));
		this._subhandleRefresh = dojo.subscribe("rmps.web.app.explorer.refresh", dojo.hitch(this, this.onRefresh));
		this._registerDropTarget = dojo.subscribe("rmps.web.app.explorer.registerDropTarget", dojo.hitch(this, this.registerDropTarget));
		this._unregisterDropTarget = dojo.subscribe("rmps.web.app.explorer.unregisterDropTarget", dojo.hitch(this, this.unregisterDropTarget));

		// Get the user uri
		if (jazz.app.currentApplication) {
			this._userUri = UserClient.getUserUri(jazz.app.currentApplication.config.user.userId);
		}
		else {
			// use the default user
			this._userUri = rmps.core.user.UserClient.getUserUri("dm_user");
		}

		// Get the projec uri
		if (!this._projectUri) {
			this._projectUri = jazz.app.context.get() ? jazz.app.context.get().uri : null;
		}
		
		this.restoreFromServer();
		this._messageBox = new jazz.ui.MessageBox();
		this._div.appendChild(this._messageBox.domNode);
	},
	
	startup: function() {
		if (this._started) { // if already started, don't restart
			return;
		}
		
		this.inherited(arguments);
		
		this.onRefresh();
		dojo.subscribe("rmps.ui.explorer.Explorer.resourceAdded", this, "onLazyRefresh");
		dojo.subscribe("rmps.web.app.explorer.deleteResource", this, "onLazyRefresh");
	},
	
	registerDropTarget: function(widgetid) {
		this._dropTargets.push(widgetid);
	},
	
	unregisterDropTarget: function(widgetid) {
		this._dropTargets.splice(dojo.indexOf(this._dropTargets, widgetid), 1);
	},
	
	addToolbar: function(domNode) {
		// Create toolbar
		this._toolbarContainer = domNode;
		this._toolbar = new jazz.ui.Toolbar();
		domNode.appendChild(this._toolbar.domNode);
		this._addToolbarButtons();
		this._toolbar.domNode.style.textAlign = "right";
	},
	
	_updateToolbar: function() {
		this._destroyToolbar();
		this.addToolbar(this._toolbarContainer);
	},
	
	_addToolbarButtons: function() {
		if (this._explorer) {
			this._backButton = new jazz.ui.toolbar.Button({
				title: messages.back,
				imgSrc: dojo.moduleUrl("rmps.ui", "graphics/icons/previous.gif").toString(),
				onClick: dojo.hitch(this, function(){
					if (this._explorer) {
						this._explorer.onBack();
					}
				})
			});
			this._toolbar.addChild(this._backButton);
			
			this._forwardButton = new jazz.ui.toolbar.Button({
				title: messages.forward,
				imgSrc: dojo.moduleUrl("rmps.ui", "graphics/icons/next.gif").toString(),
				onClick: dojo.hitch(this, function() {
					if (this._explorer) {
						this._explorer.onForward();
					}
				})
			});
			this._toolbar.addChild(this._forwardButton);

			this._toolbar.addChild(new jazz.ui.toolbar.Button({
				title: messages.collapseAll,
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/collapse_all_co.gif").toString(),
				onClick: dojo.hitch(this, function() {
					if(this._explorer) {
						this._explorer.onCollapseAll();
					}
				}) 
			}));

			this._updateToolbarButtons();
		} else {
			this._toolbar.addChild(new jazz.ui.toolbar.Button({
				title: messages.refresh,
				imgSrc : dojo.moduleUrl("rmps.app", "graphics/icons/refresh.gif").toString(),
				onClick: dojo.hitch(this, this.onRefresh)
			}));
		}
		// Viewpoint selector
		this._toolbar.addChild(new ViewpointSelector({
			title: messages.selectViewpoint,
			explorerID: this.explorerID,
			projectUri: this._projectUri,
			userUri: this._userUri,
			onViewpointChange: dojo.hitch(this, 
				function(viewpoint) {
					this.setViewpoint(viewpoint.uri);
				}),
			onGetViewpoint: function(ems) {
				return ems.getCurrentViewpoint(this.explorerID);
			},
			showViewpointLabel: false
		}));
		
		// Context menu
		this._toolbar.addChild(new jazz.ui.toolbar.MenuButton({
			title: messages.more,
			imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/context_menu_co.gif").toString(),
			menuProvider: dojo.hitch(this, function() {
				var menu = new jazz.ui.Menu();
				
				menu.addChild(new dijit.MenuItem({
					label: messages.showPinnedOnly,
					onClick: dojo.hitch(this, function() { 
						if (this._explorer) {
							this._explorer.onShowPinnedOnly();
						}
					}),
					iconClass: "rmps-app-explorer-focus-pinned-icon"
				}));
				
				menu.addChild(new dijit.MenuItem({
					label: messages.unpinAll,
					onClick: dojo.hitch(this, function() {
						if (this._explorer) {
							this._explorer.unpinAll();
							if (this._explorer._tree) {
								this._explorer._tree._saveState();
							}
						}
					})
				}));
				
				menu.addChild(new dijit.MenuItem({
					label: messages.refresh,
					onClick: dojo.hitch(this, this.onRefresh),
					iconClass: "rmps-app-explorer-section-menu-refresh"
				}));
					
				menu.addChild(new dijit.MenuItem({
					label: messages.settings,
					onClick: dojo.hitch(this, this.onSettings)
				}));
				
				var cb = this._useChildQueriesCB = new dijit.CheckedMenuItem({
					label: messages.useChildQueries,
					disabled: true,
					checked: false,
					onClick: dojo.hitch(this, function() {
						var setting = new rmps.ui.explorer.ExplorerSetting({
							settingName: USE_QUERIES_SETTING_NAME,
							projectUri: this._projectUri,
							userUri: this._userUri,
							contextType: SettingTypes.PROJECTUSER,
							value: this._useChildQueriesCB.get("checked")
						});
						setting.set();
					})
				});
				menu.addChild(cb);
				var setting = new rmps.ui.explorer.ExplorerSetting({
					settingName: USE_QUERIES_SETTING_NAME,
					projectUri: this._projectUri,
					userUri: this._userUri,
					contextType: SettingTypes.PROJECTUSER
				});
				setting.get().then(function(value) {
					cb.set("checked", value);
					cb.set("disabled", false);
				});
				
				return menu;
			})
		}));
	},
	
	_setEnabled: function(button, value) {
		button.set("enabled", value);
	},
	
	_updateToolbarButtons: function() {
		if (this._explorer) {
			this._setEnabled(this._backButton, this._explorer._canGoBack());
			this._setEnabled(this._forwardButton, this._explorer._canGoForward());
		}
	},
	
	_destroyToolbar: function() {
		if (this._toolbar) {
			this._toolbar.destroy();
			delete this._toolbar;
		}
	},
	
	_createStores: function(viewpointURI) {
		// Create the store
		this.store = new rmps.ui.explorer.ExplorerStore({
			viewpoint: viewpointURI,
			explorerID: this.explorerID,
			projectId: rmpw.common.util.IdentifierUtil.getIdentifier(this._projectUri),
			configurationUri: this._configurationUri
		});
	},
	
	_destroyStores: function() {
		if (this.store) {
			this.store.cleanCache();
			delete this.store;
		}		
	},
	
	destroy: function()
	{
		dojo.unsubscribe(this._subhandle);
		dojo.unsubscribe(this._subhandleFocusMany);
		dojo.unsubscribe(this._subhandleRefresh);
		dojo.unsubscribe(this._registerDropTarget);
		dojo.unsubscribe(this._unregisterDropTarget);
		
		if(this._menu) {
			this._menu.destroyRecursive();
			delete this._menu;
		}
		
		if (this._explorer) {
			this._explorer.destroyRecursive();
			delete this._explorer;			
		}
		
		this._destroyStores();
		
		this.inherited(arguments);
	},
	
	_onTopResult: function(data, args) {
		// Destroy the current explorer
		if (this._explorer) {
			this._explorer.destroyRecursive();
			delete this._explorer;
		}
		
		this._topNodes = data;
		
		if (this._topNodes.length == 0) {
			dojo.removeClass(this._divNoTrees, "rmps-app-explorer-main-hidden");
			return;
		} 
		var historyVP = this._history && this._history.viewpointURI;
		var history = historyVP && this._history.byVP[historyVP];
		this._spanNode.innerHTML = messages.alternativeShowTrees;
		if (args && args.refresh) {
			this._instanceExplorer(this._topNodes, true, this.getTopNodesWithIds(args.visibleTopNodes), this.getTopNodesWithIds(args.hiddenTopNodes));
		} else if (history && history.stack && history.stack.length) {
			var state = history.stack[history.depth-1];
			if (!state.hiddenTopNodes || this._topNodes.length != state.hiddenTopNodes.length) {
				this._showTopNodes(false, state.visibleTopNodes, state.hiddenTopNodes);
			}
		}
		else {		
			this._showTopNodes(false);
		}
	},
	
	_onExploreInTreeTopResult : function(response, chain) {
		var topuri = chain[chain.length - 1];
		
		var trees = response;
		if(!trees.length) {
			return false;
		}
		
		var found = false;
		for(var i = 0; i < trees.length; i++) {
			if(trees[i].uri === topuri) {
				found = true;
				break;
			}
		}
		
		if(!found) {
			return false;
		}
		
		this._onTopResult(response);
		return true;
	},		
	
	_onChainResult: function(data, repeatIfFailed, callback) {
		var chain = data; // dojo.fromJson(data);
		if (!chain.length || chain.length < 1) {
			return;
		}
		
		var chaincopy = [];
		for(var i = 0; i < chain.length; i++) {
			chaincopy.push(chain[i]);
		}
		
		var topuri = chain[chain.length - 1]; 

		// Get the top node
		var topNode = null;
		for (var index = 0; index < this._topNodes.length; index++) {
			var id = this.store.getIdentity(this._topNodes[index]);
			if (ExplorerUtil.getResourceURIFromNodeID(id) == topuri) {
				topNode = this._topNodes[index];
				break;
			}
		}
		
		// when top node is not found and there is only one root
		// then use the single root node
		if (!topNode && this._topNodes.length == 1) {
			topNode = this._topNodes[0];
		}
		
		// If tree isn't found, refresh top nodes and try again
		if (!topNode) {
			if(repeatIfFailed) {
				this.callTopService(dojo.hitch(this, function(response) {
					if (this._onExploreInTreeTopResult(response, chaincopy)) {
						this._onChainResult(chaincopy, false);
					} else {
						callback.resolve(null);
					}
				}));
			}
			return;
		}
		
		// Make sure the top node is there
		this._showTopNode(topNode);
		
		// Then expand to the correct node
		this._explorer._tree._expandNode(this._explorer._tree.rootNode).addCallback(dojo.hitch(this,
			function() {
				if(!callback){
					callback= new dojo.Deferred();
				}
				
				this._explorer._tree.expandChain(chain,
					{
						found : null,
						repeatIfFailed : repeatIfFailed,
						repeatFunction : dojo.hitch(this, function() {
							this.callTopService(dojo.hitch(this, function(response) {
								if (this._onExploreInTreeTopResult(response, chaincopy)) {
									this._onChainResult(chaincopy, false);
								} else {
									callback.resolve(null);
								}
							}));
						}),
						callback: callback,
						enableHistory: this._explorer._getHistoryEnabled()
					}
				);
				
			})
		);
	},
	
	getCurrentViewpointName: function() {
		var def = new dojo.Deferred();
		var emsDef = explorerMainState(false, this._projectUri, this._userUri);
		emsDef.addCallback(this, function(ems) {
			var viewpoint = ems.getCurrentViewpoint(this.explorerID);
			if (viewpoint) {
				def.callback(viewpoint.label);
			} else {
				def.errback();
			}
		});
		emsDef.addErrback(function() {
			def.errback();
		});
		return def;
	},
	
	_createExplorer: function(topNodes, visibleTopNodes, hiddenTopNodes)
	{
		explorerMainState(false, this._projectUri, this._userUri).addCallback(this, function(ems) {
			var viewpointURI = ems.getCurrentViewpointURI(this.explorerID);
			if (!this._history.byVP) {
				// Most likely an old setting so start from scratch.
				this.initHistory();
			}
			if (!this._history.byVP[viewpointURI]) {
				this._history.byVP[viewpointURI] = this.getEmptyHistory(); 
			}
			
			this._explorer = new rmps.ui.explorer.Explorer({
					store: this.store,
					topNodes: topNodes,
					visibleTopNodes: visibleTopNodes ? visibleTopNodes : [],
					hiddenTopNodes: hiddenTopNodes ? hiddenTopNodes : [],
					onRemoveTopNode: dojo.hitch(this, this._onRemoveTopNode),
					_history: this._history.byVP[viewpointURI],
					_expMain: this,
					_dropTargets: this._dropTargets,
					fnIsChecked: this.fnIsChecked,
					fnOnCheck: this.fnOnCheck,
					enableMultiSelect: this.enableMultiSelect,
					onlyTypes: this.onlyTypes,
					noResourceLinks: this.noResourceLinks
			});
			this._div.appendChild(this._explorer.domNode);
			// Remove the 'show all tress' text
			dojo.addClass(this._divShowTreesLink, "rmps-app-explorer-main-hidden");
			this._explorer.style.minHeight = "300px";
			this._explorer.style.paddingBottom = "7px";
				// Update toolbar
			this._updateToolbar();
			this.resize();			
		});
	},
	
	resize: function(size) {
		// Intentionally disabled, see WI33654
	},
		
	_instanceExplorer: function(topNodes, alwaysSet, visibleTopNodes, hiddenTopNodes) {
		if (!this._explorer) {
			if (this._history) {
				this._createExplorer(topNodes, visibleTopNodes, hiddenTopNodes);
			} else {
				this.restoreFromServer().addBoth(dojo.hitch(this, function() {this._createExplorer(topNodes, visibleTopNodes, hiddenTopNodes);}));
			}
		} else if (alwaysSet) {
			this._explorer.setTopNodes(topNodes, visibleTopNodes, hiddenTopNodes);
		}
		return this._explorer;
	},
	
	_showTopNode: function(topNode) {
		if (!this._explorer) {
			var x = dojo.indexOf(this._topNodes, topNode);
			var hiddenNodes = this._topNodes.slice();
			hiddenNodes.splice(x,1);
			this._instanceExplorer(this._topNodes, true, [topNode], hiddenNodes);
			return;
		}

		this._explorer.addTopNodeFromList(this._topNodes, topNode);
	},
	
	_onRemoveTopNode: function(topNodes) {
		// Show the 'show all trees' div when there are no top nodes in explorer
		if (topNodes.length == 0) {
			dojo.removeClass(this._divShowTreesLink, "rmps-app-explorer-main-hidden");
			if (this._explorer) {
				this._explorer.destroyRecursive();
				delete this._explorer;			
			}
			this._updateToolbar();
		}
	},
	
	saveStateEnableHistory : function() {
		if (this._explorer) {
			var historyEnabled = this._explorer._getHistoryEnabled();
			this._explorer._enableHistory(true);
			this._explorer._tree._saveState();
			this._explorer._enableHistory(historyEnabled);
		}
	},
	
	_showTopNodes : function(saveState, visibleTopNodeIds, hiddenTopNodeIds) {
		// Hide the div
		dojo.addClass(this._divShowTreesLink, "rmps-app-explorer-main-hidden");

		// Show the loading message
		if (this.showLoadingIcon) {
			this.showLoadingIcon();
		}
		
		this._instanceExplorer(this._topNodes, true, this.getTopNodesWithIds(visibleTopNodeIds),  this.getTopNodesWithIds(hiddenTopNodeIds));
		
		if (this.hideLoadingIcon) {
			this.hideLoadingIcon();
		}
		if (saveState) {
			this.saveStateEnableHistory();
		}
	},
	
	_onShowAllTreesLink: function(e) {
		dojo.stopEvent(e);
		this._showTopNodes(true);
	},

	_onShowExplorer: function(message) {
		if (message && message.explorerID !== this.explorerID) {
			return;
		}
		/*Start showing the loading indicator now*/
		PlatformUI.getWorkbench().showMessage(messages.loading);
		explorerMainState(false, this._projectUri, this._userUri).addCallback(this, function(ems) {
			var viewpointURI = ems.getCurrentViewpointURI(this.explorerID);
				
			// First get the owner chain from service
			var url = EXPLORER_CHAIN_PATH;
			var content = {
				resourceURI : message.uri,
				userURI: this._userUri,
				checkFiltered: false,
				viewpointURI: viewpointURI
			};
		
			var getArgs = {
				url: url,
				headers: {
					"Accept": "application/json",
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "json",
				content: content
			};	
			var handler = rmps.core.client.xhrGet(getArgs);
			handler.addCallback(dojo.hitch(this, function(response) {
					var callback = new dojo.Deferred();
					var historyEnabled = this._explorer._getHistoryEnabled();
					this._explorer._enableHistory(false);
					callback.addBoth(dojo.hitch(this, function(selectedURI) {
						this._explorer._enableHistory(historyEnabled);
						this._explorer._tree._saveState();
						PlatformUI.getWorkbench().hideMessageArea();
						if (selectedURI !== message.uri) {	
							var wrn = rmps.core.error.createInfo(messages.elementNotFoundWithCurrentViewpoint);
							wrn.details(messages.elementNotFoundWithCurrentViewpointDetails);
							wrn.dispatch();
						}
					}));
					
					var switchViewpoint = false;
					if (viewpointURI === ExplorerUtil.EXPLORER_PROVIDERS_VIEWPOINT_URI && response[0] && response[0].search("#") !== -1) {
						if (confirm(messages.elementIsNotDisplayed)) {
							switchViewpoint = true;
						}
					} 
					
					if (switchViewpoint) {
						var def = this.setViewpoint(ExplorerUtil.EXPLORER_PROVIDERS_DETAIL_VIEWPOINT_URI);
						def.addBoth(dojo.hitch(this, function() { this._onChainResult(response, true, callback); }));
					} else {
						this._onChainResult(response, true, callback);
					}
					return response;
				})
			);
			handler.addErrback(function(error) {
				// TODO: Perhaps some feedback?
			});
		});
	},

	_onFocusMany: function(message)	{
		if (message && message.explorerID !== this.explorerID) {
			return;
		}
		/*Start showing the loading indicator now*/
		PlatformUI.getWorkbench().showMessage(messages.loading);
		
		this._onFocusManyHandlers = {
			uris: message.uris
		};
		var emsDef = explorerMainState(false, this._projectUri, this._userUri);
		emsDef.addCallback(this, function(ems) {
			var viewpointURI = ems.getCurrentViewpointURI(this.explorerID);
			this._onFocusManyHandlers.explorerChains = ExplorerUtil.getManyParentChains(this._onFocusManyHandlers.uris, true, viewpointURI); 
			this._onFocusManyHandlers.explorerChains.addCallback(this, function(response) {
				var checkFroElements = true;
				var switchViewpoint = false;
	
				var filtersInChains = [];
				var elementsHidden = !this.store.areElementsShown();
				for (var r = 0; r < response.length; r++) {
					if (elementsHidden && checkFroElements) {
						// Check if the last element in chain is an element and if ask if resource contents should be displayed
						if (response[r].chain.length === 0) {
							continue;
						}
						
						if (response[r].chain[0].search("#") !== -1) {
							checkFroElements = false;
							if (confirm(messages.elementsAreNotDispayed)) {
								switchViewpoint = true;
							}
						}
					}
					
					if (response[r].filters.length === 0) {
						continue;
					}
					
					for (var f = 0; f < response[r].filters.length; f++) {
						var filter = response[r].filters[f];
						
						var idx = dojo.indexOf(filtersInChains, filter.uri);
						if (idx >= 0) {
							continue;
						}
						
						filtersInChains.push(filter.uri);
					}
				}
	
				this._resetFiltersInChains(filtersInChains, switchViewpoint, response);
				if (this._onFocusManyHandlers) {
					delete this._onFocusManyHandlers.explorerChains;
				}
				PlatformUI.getWorkbench().hideMessageArea();
			});
			this._onFocusManyHandlers.explorerChains.addErrback(this, function(error) {
				if (this._onFocusManyHandlers) {
					PlatformUI.getWorkbench().hideMessageArea();
					delete this._onFocusManyHandlers.explorerChains;
				}
			});
		});
	},

	_getAvailabledFilters: function() {
		var proj = jazz.app.context.get().itemId;	
		return jazz.client.xhrGet({
			url: EXPLORER_FILTER_PATH + "/" + proj,
			headers: {
				"Accept": "application/json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			content: {
				userUri: this._userUri,
				projectUri: this._projectUri,
				contextType: SettingTypes.PROJECTUSER
			}
		});
	},

	_resetFiltersInChains: function(filtersInChains, switchViewpoint, response) {
		var changesInFilters = false;
		if (filtersInChains.length > 0 && confirm(messages.elementIsFiltered)) {
			this._onFocusManyHandlers.availableFilters = this._getAvailabledFilters();
			this._onFocusManyHandlers.availableFilters.addCallback(this, function(response) {
				// Create list of filters
				var enabledFilters = [];
				for (var e = 0; e < response.length; e++) {
					if (!response[e].enabled) {
						continue;
					}
					
					var fIdx = dojo.indexOf(filtersInChains, response[e].uri);
					if (fIdx >= 0) {
						changesInFilters = true;
						continue;
					}
						
					enabledFilters.push(response[e].uri);
				}

				// Create explorer setting object
				var explorerSetting = new rmps.ui.explorer.ExplorerSetting({
					settingName: "appliedFilters",
					projectUri: this._projectUri,
					userUri: this._userUri,
					contextType: SettingTypes.PROJECTUSER,
					value: enabledFilters
				});	
				
				// Save the setting
				this._onFocusManyHandlers.explorerSettings = explorerSetting.set();
				this._onFocusManyHandlers.explorerSettings.addCallback(this, function(response) {
					this._updateExplorerChains(switchViewpoint || changesInFilters);
					delete this._onFocusManyHandlers.explorerSettings;
				});
				this._onFocusManyHandlers.explorerSettings.addErrback(this, function(error) {
					this._updateExplorerChains(switchViewpoint);
					delete this._onFocusManyHandlers.explorerSettings;
				});
				
				delete this._onFocusManyHandlers.availableFilters;
			});
			this._onFocusManyHandlers.availableFilters.addErrback(this, function() {
				this._updateExplorerChains(switchViewpoint);
				delete this._onFocusManyHandlers.availableFilters;
			});
		} else {
			this._updateExplorerChains(switchViewpoint, response);
		}
	},
	
	_updateExplorerChains: function(switchViewpoint, response) {
		if (switchViewpoint) {
			var def = this.setViewpoint(ExplorerUtil.EXPLORER_PROVIDERS_DETAIL_VIEWPOINT_URI);
			def.addBoth(dojo.hitch(this, function() { this._updateExplorerChains(false); }));
			return;
		}
		if (response) {
			var chains = dojo.map(response, function(x) {return x.chain;});
			this._focusOnChains(chains);
		} else {
			var emsDef = explorerMainState(false, this._projectUri, this._userUri);
			emsDef.addCallback(this, function(ems) {
				var viewpointURI = ems.getCurrentViewpointURI(this.explorerID);
				this._onFocusManyHandlers.explorerChains = ExplorerUtil.getManyParentChains(this._onFocusManyHandlers.uris, false, viewpointURI);
				this._onFocusManyHandlers.explorerChains.addCallback(this, this._focusOnChains);
				this._onFocusManyHandlers.explorerChains.addErrback(this, function(error) {
					// TODO: Perhaps some feedback?
					delete this._onFocusManyHandlers;
				});
			});
		}
	},
	
	_focusOnChains: function(response) {
		this._onFocusManyHandlers.expandedUris = [];
		
		var expandTriggers = new Array(response.length);
		for (var i = 0; i < response.length; i++) {
			expandTriggers[i] = new dojo.Deferred();
			expandTriggers[i].addBoth(this, function(res) {
				var found = false;
				if (res) {
					for (var i = 0; i < this._onFocusManyHandlers.expandedUris.length; i++) {
						if (this._onFocusManyHandlers.expandedUris[i] === res) {
							found = true;
							break;
						}
					}
					if (!found) {
						this._onFocusManyHandlers.expandedUris.push(res);
					}
				}
			});
		}
		var historyEnabled = this._explorer._getHistoryEnabled();
		var completed = new dojo.Deferred();
		completed.addBoth(dojo.hitch(this, function() {
			this._explorer._enableHistory(historyEnabled);
			this._explorer._tree._saveState();
			PlatformUI.getWorkbench().hideMessageArea();
			delete this._onFocusManyHandlers;	
		}));
		this._explorer._enableHistory(false);
		this._onFocusManyHandlers.expandCompleted = new dojo.DeferredList(expandTriggers);
		this._onFocusManyHandlers.expandCompleted.addBoth(this, function(){
			this._collapseRest(this._onFocusManyHandlers.expandedUris, completed);			
		});
		
		for (var i = 0; i < response.length; i++) {
			this._onChainResult(response[i], true, expandTriggers[i]);
		}

	},
	
	_collapseRest: function(ids, completed) {
		var explorerID = this.explorerID;
		var uris = dojo.map(ids, function(id) { return ExplorerUtil.getNodeIDFromResourceURI(id, explorerID); });
		this._explorer._tree.collapseAllExceptList(uris, completed);
	},

	onLazyRefresh: function(){
	    //wait until the Sidebar is displayed before refreshing
	    this._forceRefresh[0] = dojo.connect(this.sidebarContainer, "resizeContentWidth", this, "_lazyRefresh");
		this._forceRefresh[1] = dojo.connect(this.sidebarContainer, "resize", this, "_lazyRefresh");
	},
	
	_lazyRefresh: function(){
	    this.onRefresh();
	    dojo.disconnect(this._forceRefresh[0]);
	    dojo.disconnect(this._forceRefresh[1]);
	},
	
	onRefresh : function(preserveScope) {
		// Get the current top nodes
		var visibleTopNodes = [];
		var hiddenTopNodes = [];
		if (this._explorer) {
			var node2id = function(tn) {
				return this.store ? this.store.getIdentity(tn) : tn.id;
			};
			visibleTopNodes = dojo.map(this._explorer.visibleTopNodes, node2id , this);
			hiddenTopNodes = dojo.map(this._explorer.hiddenTopNodes, node2id, this);
		}
		var savedScope = preserveScope && this.store ? this.store.scope : null;
		// Delete and recreate the store
		this._destroyStores();
		
		var emsDef = explorerMainState(false, this._projectUri, this._userUri);
		var whenDone = new dojo.Deferred();
		emsDef.addCallback(this, function(ems) {
			var viewpointURI = ems.getCurrentViewpointURI(this.explorerID);
			this._createStores(viewpointURI);
			if (savedScope) {
				this.store.scope = savedScope;
			}
			var topDef = this.callTopService(dojo.hitch(this, this._onTopResult), {
				refresh: visibleTopNodes.length > 0,
				visibleTopNodes: visibleTopNodes,
				hiddenTopNodes: hiddenTopNodes
			}, true);
			topDef.addCallback(function (result){
				whenDone.resolve(result);
			});
		});
		
		return whenDone;
	},
	
	onSettings : function() {
		var dialog = new jazz.ui.SwapListDialog({
			dialogTitleText: messages.filterDialogTitle,
			unselectedListTitleText: messages.filterDialogUnselectedTitle,
			selectedListTitleText: messages.filterDialogSelectedTitle,
			sortField: "label",
			onOK: dojo.hitch(this, function(dialog) {
				var entries = dialog.getSelectedEntries();
				if (entries.length === 0){
						var segments = this._projectUri.split('/');
						var dummyFilter = "DUMMY_FILTER";
						var project_id = segments.length ? segments[segments.length - 1] : null;
						if (project_id) {
							entries.push({id : project_id + "/"+dummyFilter+project_id});
						}
					}
				// Set the user setting for filters
				var filters = dojo.map(entries, function(item) {
					return item.id;
				});

				
				
				// Create explorer setting object
				var setting = new rmps.ui.explorer.ExplorerSetting({
					settingName: "appliedFilters",
					projectUri: this._projectUri,
					userUri: this._userUri,
					contextType: SettingTypes.PROJECTUSER,
					value: filters
				});
				
				// Save the setting
				setting.set();

				dialog.destroy();
			}),
			onCancel: function(dialog) {
				dialog.destroy();
			}
		});

		// Add help message to dialog
		dojo.place("<div style=\"padding: 10px\">" + messages.filterDialogHelpMessage + "</div>",dialog._contentArea,"first");
		
		// Show the dialog
		dialog.show();
		dialog.showLoading(true);            
		
		// Get the filters
		var proj = jazz.app.context.get().itemId;	
		var url = EXPLORER_FILTER_PATH + "/" + proj;
		var etag = null;
		var uri = null;
		var getArgs = {
			url: url,
			headers: {
				"Accept": "application/json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			content: {
				userUri: this._userUri,
				projectUri: this._projectUri,
				contextType: "PROJECTUSER"
			}
		};
		var handler = jazz.client.xhrGet(getArgs);

		handler.addCallback(dojo.hitch(this, function(response) {
			// Create list of filters
			var filters = dojo.map(response, function(i) {
				return new rmps.ui.sections.ExplorerFilter(i); 
			});

			var unselectedEntries = [];
			var selectedEntries = [];
			for (var i = 0; i < filters.length; i++) {
				var item = {
					label: filters[i].name,
					id: filters[i].uri
				};
				if (filters[i].enabled) {
					selectedEntries.push(item);
				} else {
					unselectedEntries.push(item);
				}
			}
			dialog.setUnselectedEntries(unselectedEntries);
			dialog.setSelectedEntries(selectedEntries);
			dialog.showLoading(false);
		}));
		handler.addErrback(function(error) {
			dialog.showLoading(false);
			dialog.hide();
			dialog.destroy();
		});

	},
		
	getTopNodesWithIds: function (idList) {
		return idList? 
				dojo.filter(this._topNodes, dojo.hitch(this, function (tn)
				{
					var id = this.store.getIdentity(tn);
					return id && -1 != dojo.indexOf(idList,id);
				})) 
				: [];
	},
	
	initHistory: function() {
		if (this._history) {
			delete this._history;
		}
		this._history = {
			byVP: {}
		}; 
    },
	
	getEmptyHistory: function() {
		return {
			maxDepth: 26,
			depth: 0,
			stack: []
		};
    },
    
    setViewpoint: function(viewpointURI) {
        var def = new dojo.Deferred(); 
        this._saveHistory(viewpointURI).addBoth(this, function(){
			var refreshed = this.onRefresh();
			refreshed.then(function(){
				def.resolve();
			}, function() {
				def.reject();
			});
		});
		return def;
    },
    
    showLoadingIcon: function() {
    	if (this._messageBox) {
    		this._messageBox.showMessage({
    			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
    			summary: messages.loading 
    		});
    		if (this._explorer) {
        		dojo.addClass(this._explorer.domNode, "rmps-app-explorer-main-hidden");    			
    		}
    	}
    },
    
    hideLoadingIcon: function() {
    	if (this._messageBox.domNode) {
			this._messageBox.hideMessage();
			if (this._explorer){
				dojo.removeClass(this._explorer.domNode, "rmps-app-explorer-main-hidden");
			}
		}
    }
    
    
	
});

}());
