/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.core.explorer.ExplorerMainState");

dojo.require("rmps.core.explorer.ViewpointClient");
dojo.require("rmps.core.user.UserClient");
dojo.require("rmps.app.ui.internal.util.DomainRegistryUtil");
dojo.require("rmpw.common.util.IdentifierUtil");

dojo.require("rmps.core.explorer.ExplorerConstants");
var IDs = rmps.core.explorer.ExplorerConstants.IDs;
var SettingTypes = rmps.core.explorer.ExplorerConstants.SettingTypes;

var viewpointClient = rmps.core.explorer.ViewpointClient;
var DomainRegistryUtil = rmps.app.ui.internal.util.DomainRegistryUtil;

(function(){

var EXPLORER_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "explorer");
var EXPLORER_TREE_PATH = EXPLORER_PATH + "/tree";
var EXPLORER_VIEWPOINT_PATH = EXPLORER_PATH + "/viewpoint";
var EXPLORER_DEFAULT_VIEWPOINT_URI = EXPLORER_VIEWPOINT_PATH + "/DefaultViewpoint";
var EXPLORER_DOMAINS_VIEWPOINT_URI = EXPLORER_VIEWPOINT_PATH + "/DomainsViewpoint";
	
var rce = rmps.core.explorer;

//"singleton"
rce._explorerMainStateDef = null;

rce.explorerMainState = function(bForceRequest, mainProjectUri, defaultUser) {
	if (!rce._explorerMainStateDef || bForceRequest){
		var def = rce._explorerMainStateDef = new dojo.Deferred();
		
		var context = jazz.app.context.get();
		var projectUri = context.uri ? context.uri : mainProjectUri;
		var user = dojo.getObject("jazz.app.currentApplication.config.user");
		var userId, userUri;
		if (user) {
			userId = user.userId;
			userUri = user.uri || rmps.core.user.UserClient.getUserUri(userId);
		}
		else {
			userUri = defaultUser;
		}
		
		if (!projectUri || !userUri) {
			def.errback();
		}
		
		// Get viewpoints associated with project
		var def1 = viewpointClient.getViewpoints(true, rmpw.common.util.IdentifierUtil.getIdentifier(projectUri));
		
		var def2 = new dojo.Deferred();
		DomainRegistryUtil.isCurrentProjectDSP(function(result) {
			def2.callback(result);
		});
		
		
		var defs = [def1, def2];
		var settings = {};
		var get_def;
		for (var c in IDs) {
			var id = IDs[c];
			settings[id] = new rmps.ui.explorer.ExplorerSetting({
				settingName: id +"_state",
				projectUri: projectUri,
				userUri: userUri,
				contextType: SettingTypes.PROJECTUSER
			});
			
			get_def = settings[id].get();
			defs.push(get_def);
		}

		var defAll = new dojo.DeferredList(defs);
		defAll.addCallback(function(results) {
			var result1 = results[0];
			var result2 = results[1];
			var resultsById = {};
			i = 2;
			for (var c in IDs) {
				var id = IDs[c];
				resultsById[id] = results[i] && results[i][0] ? results[i][1] : {};
				i++;
			}
			var associatedViewpoints = result1 && result1[0] && result1[1] instanceof Array ? result1[1] : [];
			var isDSP = result2 && result2[0] && result2[1];
			def.callback(new rce.ExplorerMainState(settings, resultsById, associatedViewpoints, isDSP));				
		});
		defAll.addErrback(function() {
			def.errback();
		});
	}
	return rce._explorerMainStateDef;
};
	
rce.ExplorerMainState = function(settings, settingValues, associatedViewpoints, isDSP) {

	this.get = function(explorerID) {
		return settingValues[explorerID];
	};
	
	this.set = function(value, explorerID) {
		var setting = settings[explorerID];
		if (setting) {
			setting.value = value;
			settingValues[explorerID] = value;
			return setting.set();
		} else {
			var def = dojo.Deferred();
			def.errback("invalid id" + explorerID);
			return def;
		}
	};
	
	this.getCurrentViewpointURI = function(explorerID) {
		var data = settingValues[explorerID];
		if (data && data.viewpointURI) {
			return data.viewpointURI;
		} else {
			return this._getDefaultViewpoint();
		}
	};
	
	this.getCurrentViewpoint = function(explorerID) {
		var i, uri = this.getCurrentViewpointURI(explorerID);
		for (i=0; i < associatedViewpoints.length; i++) {
			if (associatedViewpoints[i].uri === uri) {
				return associatedViewpoints[i];
			}
		}
		uri = this._getDefaultViewpoint();
		for (i=0; i < associatedViewpoints.length; i++) {
			if (associatedViewpoints[i].uri === uri) {
				return associatedViewpoints[i];
			}
		}
		return null;
	};
	
	this.getProjectAssociatedViewpoints = function() {
		return associatedViewpoints;
	};
	
	this._getDefaultViewpoint = function() {
		if (associatedViewpoints && associatedViewpoints.length > 0)
		{
			for (var i = 0; i < associatedViewpoints.length; i++) {
				var vp = associatedViewpoints[i]; 
				if (vp.isDefault()) {
					return vp.uri;
				}
			}
		}

		// Fall back if there for some reason are no associated viewpoints, or no default. 
		if (isDSP) {
			// Domains viewpoint should be default for DSPs.
			return EXPLORER_DOMAINS_VIEWPOINT_URI;
		} else {
			// Use the Documents viewpoint
			return EXPLORER_DEFAULT_VIEWPOINT_URI;
		}
	};
};

}());