/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.explorer.ExplorerSearchWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("rmps.ui.explorer.ExplorerMain");
dojo.require("rmps.ui.EllipsisLabel");
dojo.require("rmps.core.explorer.ExplorerConstants");
dojo.require("rmps.ui.explorer.ExplorerUtilities")

dojo.requireLocalization("rmps.ui.explorer", "ExplorerMessages");
var messages = dojo.i18n.getLocalization("rmps.ui.explorer", "ExplorerMessages");

var ExplorerUtilities = rmps.ui.explorer.ExplorerUtilities;

dojo.declare("rmps.ui.explorer.ExplorerSearchWidget", [dijit._Widget, dijit._Templated], {

	messages: messages,
	templateString: dojo.cache("rmps.ui", "explorer/templates/ExplorerSearchWidget.html"),
	widgetsInTemplate: true,

	// attach points
	_topContainer: null,
	_titleBarNode: null,
	_menuToolbar: null,
	_explorerMain: null, //Widget

	// mixin
	// ok button callback
	onOkButton: null,
	// cancel button callback
	onCancelButton: null,
	
	// props
	_titleBar: null,
		
	postCreate: function() {
		this.inherited(arguments);	
		
		this._doPostCreate();
	},
	
	_doPostCreate: function() {
		this.initializeTypes([]);
	},
	
	initializeTypes: function(projects) {
		if (this.onlyType && this.onlyType != "http://www.w3.org/2000/01/rdf-schema#Resource" && 
			this.onlyType != "http://jazz.net/ns/dm/core#Document" &&
			this.onlyType != "http://jazz.net/ns/dm/diagram#MappingState") {
			// next have to find all subclasses of onlyType
			var whereAllTypes = "?value rdfs:subClassOf* <"+ this.onlyType +">."+
			"?value rdf:type owl:Class ."+
			"?value rdf:type ?classType .";
		
			var self = this;
			// use simple select type dialog, till we have the search dialog works 
			rmps.core.client.xhrGet({
				url: net.jazz.ajax._contextRoot + "/modelvalidation/values",
				handleAs: 'json',
				headers: {
					"Accept" : "application/json",
					"X-ibm-rmps-internal" : "true"
				},
				content: {
					resourceUri: this.onlyType,
					adjustedRangeUri: this.onlyType,
					propertyUri: "http://www.w3.org/1999/02/22-rdf-syntax-ns#type",
					adjustedDomainUri: this.onlyType,
					whereClause: whereAllTypes
				},
				load: function(data, ioArgs) {
					var types = [];
					for (var i=0, lg=data.length; i<lg; i++) {
						var type = data[i].value || data[i].rdf_about;		
	                    types.push(type);
					}
					
					self.initializeExplorer(projects, types);
				}
			});
		}
		else {
			this.initializeExplorer(projects);
		}
		
	},
	
	initializeExplorer: function(projects, types) {
		this.onlyTypes = types;
		
		this.resetExplorerForProject(); // set to current project area
	},
	
	startup: function() {
		this.inherited(arguments);
		
		if (this._explorerMain) {
			this._explorerMain.startup();
		}
	},
		
	resize: function() {
		// Intentionally disabled: see WI33654
	},
		
	layout: function() {
	},
	
	onClickOkButton: function() {
		if (this.onOkButton) {
			var items = [];
			for (var id in this._selectedItems) {
				var item = this._selectedItems[id];
				if (!item.rdf_about) {
					item.rdf_about = ExplorerUtilities.getItemURI(item);
				}
				items.push(item);
			}
			
			this.onOkButton(items);
		}
	},
	
	onClickCancelButton: function() {
		if (this.onCancelButton) {
			this.onCancelButton();
		}
	},
	
	resetExplorerForProject: function(projectUri, contextUri) {
		if (this._explorerMain) {
			this._explorerMain._destroyToolbar();
			this._explorerMain.destroyRecursive();
		}
		
		this._explorerMain = new rmps.ui.explorer.ExplorerMain({
			explorerID: rmps.core.explorer.ExplorerConstants.IDs.EXPLORER_SEARCH,
			noResourceLinks: true,
			_projectUri: projectUri ? projectUri : this.projectUri,
			_configurationUri: contextUri ? contextUri : this.contextUri,
			enableMultiSelect: this.enableMultiSelect,
			onlyTypes: this.onlyTypes,
			fnIsChecked: dojo.hitch(this, function(item) {
				if (!this._selectedItems) {
					this._selectedItems = {};
					this._numSelected = 0;
				}
				return this._selectedItems[item.__id] !== undefined;
			}),
			fnOnCheck: dojo.hitch(this, function(item, check) {
				if (!this._selectedItems) {
					this._selectedItems = {};
					this._numSelected = 0;
				}
				
				item._isSelected = check;
				if (check) {
					this._selectedItems[item.__id] = item;
					this._numSelected++;
				}
				else {
					delete this._selectedItems[item.__id];
					this._numSelected--;
				}
				
				var isDisabled = dojo.hasAttr(this._okButton, "disabled");
				var hasSelection = this._numSelected > 0;
				if (hasSelection && isDisabled) {
					dojo.removeAttr(this._okButton, "disabled");
				} else if (!hasSelection && !isDisabled) {
					dojo.attr(this._okButton, "disabled", "disabled");
				}
			})
		});
		this._explorerContent.appendChild(this._explorerMain.domNode);
		this._explorerMain.domNode.style.overflow = "hidden";
		this._explorerMain.domNode.style.minHeight = "100px";
		
		// Create the tool bar
		this._explorerMain.addToolbar(this._menuToolbar);	
		this._explorerMain.startup();
	}
});



}());
