/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.explorer.ExplorerSearchWidgetWithProjectSelection");

dojo.require("rmps.ui.explorer.ExplorerSearchWidget");
dojo.require("rmpw.common.util.IdentifierUtil");

dojo.requireLocalization("rmps.ui.explorer", "ExplorerMessages");
var messages = dojo.i18n.getLocalization("rmps.ui.explorer", "ExplorerMessages");

dojo.declare("rmps.ui.explorer.ExplorerSearchWidgetWithProjectSelection", [rmps.ui.explorer.ExplorerSearchWidget], {

	messages: messages,
	templateString: dojo.cache("rmps.ui", "explorer/templates/ExplorerSearchWidgetWithProjectSelection.html"),
	
	_doPostCreate: function() {
		var uri = net.jazz.ajax._contextRoot + '/projects/dependent/' + rmpw.common.util.IdentifierUtil.getIdentifier(this.projectUri);
		rmps.core.client.xhrGet({
			url : uri,
			headers: {
				"Accept": "application/rdf+json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "text",
			error: function(error, ioArgs) {
			    rmps.core.error.createXhrError(ioArgs, "Attempting to get stream service's project dfd.").dispatch();
			    console.error(error);
			}
		}, true).then(dojo.hitch(this, function(projectsJson){	
			this.initializeTypes(dojo.fromJson(projectsJson));
		}));
	},
	
	projectToContext: {},
		
	initializeExplorer: function(projects, types) {
		for (var property in projects) {
			var obj = projects[property];
			var nameObj = obj['http://jazz.net/xmlns/rmps/groups/1.0/projectName'];
			for (var i=0; i<nameObj.length; i++){
					
				var option = dojo.doc.createElement("option");
				option.appendChild(dojo.doc.createTextNode(nameObj[i].value));
				option.value = property;
				this.projectToContext[property] = obj['configurationURI'];
				option.title= nameObj[i].value;
				this.projectAreaSelect.appendChild(option);
				break;
			}
		}
		
		this.projectAreaSelect.value = this.projectUri;
		
		this.inherited(arguments);
	},
	
	_onProjectAreaSelectChange: function(event){
		if(!event){
			return;
		}
		var element = event.target || event.srcElement;
		var projectUri= element.value;
		var contextUri = this.projectToContext[element.value];
		
		if(!projectUri){	//Safe check. Do you think that this can ever happen? Nope
			return;
		}

		this.resetExplorerForProject(projectUri, contextUri);
	}
});



}());
