/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("rmps.ui.explorer.ExplorerSetting");

    dojo.require("rmps.core.RootServices");
    dojo.require("rmps.core.resource.Resource");
    dojo.require("rmps.core.explorer.ExplorerConstants");

    var SettingTypes = rmps.core.explorer.ExplorerConstants.SettingTypes;
    var EXPLORER_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "explorer");
    var EXPLORER_SETTINGS_PATH = EXPLORER_PATH + "/settings";
    
    dojo.declare("rmps.ui.explorer.ExplorerSetting", rmps.core.resource.Resource, {
        settingName: null,
        projectUri: null,
        userUri: null,
        detectUser: null,
        value: null,
        contextType: SettingTypes.PROJECT, // PROJECT or USER or PROJECTUSER

        set: function() {	
            var url = EXPLORER_SETTINGS_PATH + "/" + this.settingName;
            var headers = {
				"Content-Type": "application/json",
				"X-ibm-rmps-internal": "true"
			};
            if (this.eTag) {
                headers.ETag = this.eTag;
            }

            var putArgs = {
                url: url,
                headers: headers,
                handleAs: "json",
                putData: dojo.toJson({
                    value: this.value,
                    projectUri: this.projectUri,
                    userUri: this.userUri,
                    contextType: this.contextType
                })
            };
            var def = jazz.client.xhrPut(putArgs);
            def.addCallback(this, function() {
                this.uri = def.ioArgs.xhr.getResponseHeader("Location");
                this.eTag = def.ioArgs.xhr.getResponseHeader("ETag");
            });
            return def;
        },

        get: function() {	
            var url = EXPLORER_SETTINGS_PATH + "/" + this.settingName;
            var headers = {
                "Accept": "application/json",
                "X-ibm-rmps-internal": "true"
            };
            if (this.eTag) {
                headers.ETag = this.eTag;
            }
            var content = {
                projectUri: this.projectUri,
                contextType: this.contextType
            };
            if (this.userUri) {
                content.userUri = this.userUri;
            }
            if (this.detectUser) {
                content.detectUser = this.detectUser;
            }
            var getArgs = {
                url: url,
                headers: headers,
                content: content,
                handleAs: "json"
            };	
            var def = jazz.client.xhrGet(getArgs);
            def.addCallback(this, function(result) {
                this.value = result;
                this.uri = def.ioArgs.xhr.getResponseHeader("Location");
                this.eTag = def.ioArgs.xhr.getResponseHeader("ETag");
            });
            return def;
        }
    });
})();