/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.explorer.ExplorerStore");

dojo.require("dojox.data.JsonRestStore");
dojo.require("rmps.core.explorer.ExplorerTreeService");
dojo.require("rmps.ui.explorer.ExplorerUtilities");
dojo.require("rmps.app.core.internal.Workarounds"); // needs to be loaded here so loader is overwritten
		
var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;

/* Private scope 
 * 
 * Usage:  var memento = createMemento(this, nodeToSaveFrom);
 *			...
 *			memento(nodeToSaveTo);
 */
var createMemento = function(self, node) {
		/* 
		 * This method will returns a memento snap shot of our current state
		 * that we want to retain and restore after a node has been loaded.
		 */

		// What we are saving
		var labelAttribute;
		var label;
		var savedPinned;
		var iconUri;
		var viewpoint;
		var actions;
		var isGroup;

		if (node) {
			if (self.labelAttribute) {
				labelAttribute = self.labelAttribute;
				label = node[labelAttribute];
			}
			savedPinned = node.isPinned;
			iconUri = node.iconUri;
			viewpoint = node.viewpoint;
			actions = node.actions;
			isGroup = node.group;
		}
		
		return function(node) {
			// Restore the state
			if (node) {
				if (label && !(labelAttribute in node)) {
					node[labelAttribute] = label;
				}
				if (iconUri && !("iconUri" in node)) {
					node.iconUri = iconUri;
				}
				if (savedPinned && !("isPinned" in node)) {
					node.isPinned = savedPinned;
				}
				if (viewpoint && !("viewpoint" in node)) {
					node.viewpoint = viewpoint;
				}
				if (actions && !("actions" in node)) {
					node.actions = actions;
				}
				if (isGroup && !("group" in node)) {
					node.group = isGroup; 
				}
			}
		};
};

var EXPLORER_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "explorer");
var EXPLORER_TREE_PATH = EXPLORER_PATH + "/tree";

dojo.declare("rmps.ui.explorer.ExplorerStore", dojox.data.JsonRestStore, {
	service: rmps.core.explorer.ExplorerTreeService.explorerTreeService,
	
	
	
	constructor: function() {
		this.inherited(arguments);
		if (!this.scope) {
			this.scope = ExplorerUtil.getScopeFromViewpointURI(this.viewpoint);
		}
	},
	
	areElementsShown: function() {
		return this.scope !== ExplorerUtil.SCOPE_RESOURCES;
	},
	
	setScopeElements: function() {
		this.scope = ExplorerUtil.SCOPE_ALL;
	},
		
	// Overriding loadItem to be able to copy already loaded label attribute from partial object
	// to fully loaded object.
	loadItem: function(args) {
		// Start change
		
		var memento = createMemento(this, args.item); // create a memento
		
		// End change
		var item;
		if(args.item && args.item._loadObject){
			args.item._loadObject(function(result){
				// this is the callback from the TreeService query (expanding a child)
				item = result; // in synchronous mode this can allow loadItem to return the value
				delete item._loadObject;
				// Start change
				memento(result); // restore state
				// End change
				var func = result instanceof Error ? args.onError : args.onItem;
				if(func){
					func.call(args.scope, item);
				}
			}, 
			{
				viewpoint: this.viewpoint,
				explorerID: this.explorerID,
				scope: this.scope,
				projectId: this.projectId,
				configurationUri: this.configurationUri
			});
		}else if(args.onItem){
			// even if it is already loaded, we will use call the callback, this makes it easier to
			// use when it is not known if the item is loaded (you can always safely call loadItem).
			args.onItem.call(args.scope, args.item);
		}
		return item;
    },
    
    loadTopNodes: function(contextUri, fromNode, viewpoint, configurationUri) {
		var def = new dojo.Deferred();
		
		var query = {};
		if (contextUri) {
			query.root = contextUri;
		}
		if (viewpoint) {
			query.viewpoint = viewpoint;
		}
		
		if (this.explorerID) {
			query.explorerID = this.explorerID;
		}
		
		if (configurationUri) {
			query.configurationUri = configurationUri;
		}
		
		var memento = createMemento(this, fromNode); // create a memento
		
		this.fetch({
			query: query,
			onComplete: function(result) {
				memento(result); // restore state
				def.callback(result);
			},
			onError: function(result) {
				def.errback(result);
			}
		});

		return def;
    },
    
    isItem: function(item, anyStore){
		return item.isEllipsis || this.inherited(arguments);
    },
    
    cleanCache: function() {
		var index = dojox.rpc.Rest._index;
		var suffix = ExplorerUtil.getExplorerIDSuffix(this.explorerID);
		for (var id in index) {
			if (index.hasOwnProperty(id)) {
				var end = id.lastIndexOf('#');
				if (end === - 1) {
					end = id.length;
				}
				var sub = id.substr(end - suffix.length, suffix.length);
				if (end > suffix.length && sub === suffix) {
					delete index[id];
				}
			}
		} 
    }
});

}());
