/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.explorer.ExplorerTooltipStore");

dojo.require("rmps.core.resource.ResourceInfoClient");
var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;

dojo.require("rmps.ui.explorer.ExplorerUtilities");
var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;

dojo.declare("rmps.ui.explorer.ExplorerTooltipStore", null, {
	
	lockdef: null,
	
	index: {},
	store: null,
	
	isLoaded: function(id) {
		if (id in this.index) {
			return true;
		}
		return false;
	},
	
	getTooltip: function(id) {
		return this.index[id];
	},
	
	getTooltipText: function(id) {
		return this.index[id].text;
	},
	
	loadTooltips: function(ids, lock) {
		var d = new dojo.Deferred();
		
		// Check if locked, if so, continue when unlocked
		if (this.lockdef) {
			this.lockdef.addBoth(this, function() {
				this.lockdef = null;
				var def = this.loadTooltips(ids, lock);
				def.addCallback(function(updates) {
					d.callback(updates);
				});
				def.addErrback(function(error) {
					d.errback(error);
				});
			});
			return d;
		}
		
		if (lock) {
			this.lockdef = d;
		}

		var updated = [];
		
		// Filter already loaded IDs but add not yet updated tool tips to update list
		ids = dojo.filter(ids, function(id) {
			if (id in this.index) {
				var tooltip = this.index[id];
				if (!tooltip.updated) {
					updated.push(id);
				}
				return false;
			}
			return true; 
		}, this);

		if (ids.length == 0) {
			d.callback(updated);
			return d;
		}
		
		// Get ids and decode to uris
		var uris = dojo.map(ids, function(id) {
			return ExplorerUtil.getResourceURIFromNodeID(id);
		}, this);
		
		var def = ResourceInfoClient.getResourceInfo(uris, ['longName', 'displayType']);
		def.addCallback(this, function(infos) {
			for (var i = 0; i < ids.length && i < infos.length; i++) {
				if (infos[i].uri != uris[i]) {
					//just in case...
					continue;
				}
				var tooltipText = "";
				if (infos[i].longName) {
					tooltipText = infos[i].longName;
					if (infos[i].displayType) {
						tooltipText += (" (" + infos[i].displayType + ")");
					}
				}
				var id = ids[i];
				this.index[id] = {
					text: tooltipText,
					updated: false
				};
				updated.push(id);
			}
			d.callback(updated);
		});
		def.addErrback(function(error) {
			console.warn("Failed to load tooltip for among others " + uris[0]);
			error.log = false; // Don't log the error. We've already done that above.
			d.errback(error);
		});

		return d; 
	}
});

})();