/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.explorer.ExplorerTree");

dojo.require("dijit.Tree");

dojo.require("dojo.NodeList-traverse");

dojo.require("dojox.uuid.generateRandomUuid");

dojo.require("jazz.app.context");

dojo.require("rmps.core.RootServices");

dojo.require("rmps.app.ui.internal.common.ImageService");
dojo.require("rmps.ui.explorer.ExplorerTreeNode");
dojo.require("rmps.core.explorer.ExplorerTreeService");
dojo.require("rmps.ui.explorer.ExplorerUtilities");
dojo.require("rmps.core.resource.ResourceInfoClient");

dojo.requireLocalization("rmps.ui.explorer", "ExplorerMessages");
var messages = dojo.i18n.getLocalization("rmps.ui.explorer", "ExplorerMessages");

var INFO_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "info");
var FOLDERS_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "folders");
var VIEWPOINTS_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "explorer") + "/viewpoint";
var RENDERING_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "rendering");

var ICON_QUERY_VAR = "icon";
var BASELINE_PARAM = "rmps.context";
var ImageService = rmps.app.ui.internal.common.ImageService;
var SILENT_ERROR = {
		log: false
};

var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;
var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;

dojo.declare("rmps.ui.explorer.ExplorerTree", dijit.Tree, {

	_explorer: null,
	onDownArrowExit: null,
	onLeftArrowExit: null,
	onUpArrowExit: null,
	fnIsChecked: null,
	fnOnCheck: null,
	_pinnedItems: null,
	//this array holds paths to items pinned earlier (from history), but not yet loaded into the tree
	_pinnedPaths: null,
	_preventCache: (new Date()).getTime(),
	_dropTargets: null,
	_treeState : null, //{}

	constructor: function() {
		this._treeState = {
			openedItems : {},
			siblingEllipsis : {},
			hiddenByParentEllipsis : {},
			parentEllipsis : {}
		};
	},
	
	postCreate: function() {
		this.inherited(arguments);
		
		this.dndParams.push("_dropTargets");
	},
	
	_state: function(item,expanded){
		// summary:
		//		Query or set expanded state for an item,
		if(!this.persist){
			return false;
		}
		var id=this.model.getIdentity(item), parentchain, bottom, bottomid;
		if(arguments.length === 1){
			if(this._treeState.openedItems[id]) {
				return true;
			}
			if(this._treeState.hiddenByParentEllipsis[id]) {
				return true;
			}
			if(item.isEllipsis && !item.ellipsisSiblings) {
				parentchain = item.ellipsisParentChain;
				bottom = parentchain[parentchain.length - 1];
				bottomid = this.model.getIdentity(bottom);
				if(this._treeState.parentEllipsis[bottomid].expanded) {
					return true;
				}
			}
			return false;
		}

		if(!item.isEllipsis) {
			if(expanded){
				this._treeState.openedItems[id] = true;
			}else{
				delete this._treeState.openedItems[id];
			}
		}

		if(item.isEllipsis && !item.ellipsisSiblings) {
			parentchain = item.ellipsisParentChain;
			bottom = parentchain[parentchain.length - 1];
			bottomid = this.model.getIdentity(bottom);
			if(expanded) {
				this._treeState.parentEllipsis[bottomid].expanded = true;
			} else {
				this._treeState.parentEllipsis[bottomid].expanded = false;
			}
		}
	},

	_saveState : function() {
		if (!this.persist) {
			return;
		}
		var node_mapper = dojo.hitch(this, function(tn) {
			return this._explorer.store.getIdentity(tn);
		});
		
		var state = {
			treeState: this._treeState,
			pinnedPaths: this._pinnedPaths.concat(dojo.map(this._pinnedItems, dojo.hitch(this, this._getPath)))
		};


		state.showNewTrees = this._explorer.showNewTrees;
		if (state.showNewTrees) {
			if (this._explorer.hiddenTopNodes) {
				state.hiddenTopNodes = dojo.map(this._explorer.hiddenTopNodes, node_mapper);
			}
		} else {
			if (this._explorer.visibleTopNodes) {
				state.visibleTopNodes = dojo.map(this._explorer.visibleTopNodes,node_mapper);
			}
		}	

		this._explorer._rememberState(state);
	},

	destroy : function() {
		this.persist = false;

		this.inherited(arguments);
	},

	_createTreeNode: function(/*Object*/ args){
		// summary:
		//		creates a TreeNode
		// description:
		//		Developers can override this method to define their own TreeNode class;
		//		However it will probably be removed in a future release in favor of a way
		//		of just specifying a widget for the label, rather than one that contains
		//		the children too.

		args.fnIsChecked = this.fnIsChecked;
		args.fnOnCheck = this.fnOnCheck;
		args.enableMultiSelect = this.enableMultiSelect;

		return new rmps.ui.explorer.ExplorerTreeNode(args);
	},

	_onNodeMouseEnter: function(/*dijit._Widget*/ node){
		// summary:
		//		Called when mouse is over a node (onmouseenter event)

		// Position and show the context menu div
		if (node._menuNode) {
			// icon width (16) + distance from edge (4) = 20
			var left = this.domNode.clientWidth - 20 + this.domNode.scrollLeft;
			dojo.style(node._menuNode, {
				"left": left + "px",
				"visibility": "visible"
			});
		}
	},

	_onNodeMouseLeave: function(/*dijit._Widget*/ node){
		// summary:
		//		Called when mouse is over a node (onmouseenter event)
		if (node._menuNode) {
			dojo.style(node._menuNode, {
				"left": 0,
				"visibility": "hidden"
			});
		}
	},
	
	_onNodeKeypress: function(/*dijit._Widget*/ node, e){
		if(e.charCode && e.charCode === dojo.keys.SPACE) {
			dojo.stopEvent(e);
			node.onContextMenu();
		}
	},
	
	_onNodeBlur: function(/*dijit._Widget*/ node){
		dojo.removeClass(node.rowNode, "dijitTreeRowHover");
		if (node._menuNode) {
			dojo.style(node._menuNode, {
				"left": 0,
				"visibility": "hidden"
			});
		}
	},
	
	_onNodeFocus: function(/*dijit._Widget*/ node){
		// summary:
		//		Called when a TreeNode gets focus, either by user clicking
		//		it, or programatically by arrow key handling code.
		// description:
		//		It marks that the current node is the selected one, and the previously
		//		selected node no longer is.

		this.inherited(arguments);
		dojo.addClass(node.rowNode, "dijitTreeRowHover");
		
		if (node._menuNode) {
			// icon width (16) + distance from edge (4) = 20
			var left = this.domNode.clientWidth - 20 + this.domNode.scrollLeft;
			dojo.style(node._menuNode, {
				"left": left + "px",
				"visibility": "visible"
			});
		}
	},

	_isFolderId: function(id) {
		var decid = String(ExplorerUtil.getResourceURIFromNodeID(id));
		return decid.indexOf(FOLDERS_PATH) !== -1;
	},

	_isFolderItem: function(item) {
		var id = this.model.store.getIdentity(item);
		return id && this._isFolderId(id);
	},

	_isViewpointId: function(id) {
		var decid = String(ExplorerUtil.getResourceURIFromNodeID(id));
		return decid.indexOf(VIEWPOINTS_PATH) !== -1;
	},
	
	_isViewpointItem: function(item) {
		var id = this.model.store.getIdentity(item);
		return id && this._isViewpointId(id);
	},
	
	getIconStyle: function(/*dojo.data.Item*/ item, /*Boolean*/ opened){
		var style;
		if (item.isEllipsis)
		{
			//var url = item.ellipsisSiblings ? ImageService.ELLIPSIS_V : ImageService.ELLIPSIS_H;
			var url = ImageService.ELLIPSIS_H;
			style = {
				backgroundImage: "url('" + url +"')",
				"background-repeat": "no-repeat",
				width: "16px",
				height: "16px"
			};
			return style;
		}

		// Allow explorer providers to provide custom icons
		if (item.iconUri) {
			return { // style
				backgroundImage: "url('" + item.iconUri +"')",
				"background-repeat": "no-repeat",
				width: "16px",
				height: "16px"
			};
		}
				
		if (!this.model || !this.model.store) {
			return;
		}

		// If this is a folder node we use our CSS classes. See getIconClass
		if (this._isFolderItem(item) || (!item.iconUri && item.group)) {
			return;
		}

		var id = this.model.store.getIdentity(item);
		if (id) {
			// Make sure the icon is not cached
			var iconURL = ResourceInfoClient.getIconUrl(ExplorerUtil.getResourceURIFromNodeID(id), this._preventCache);
			style = {
				backgroundImage: "url('" + iconURL + "')",
				"background-repeat": "no-repeat",
				width: "16px",
				height: "16px"
			};
		}
		return style;
	},

	getIconClass: function(item, opened) {
		// For folders we return the folder icons.
		if (this._isFolderItem(item) || (!item.iconUri && item.group)) {
			if (opened) {
				return "rmps-app-explorer-folder-open-icon";
			} else {
				return "rmps-app-explorer-folder-closed-icon";
			}
		}
	},

	getLabel: function(/*dojo.data.Item*/ item){
		// summary:
		//		Overridable function to get the label for a tree node (given the item)
		// tags:
		//		extension
		if (item.isEllipsis)
		{
			return "(" + item.ellipsisNumber + ")";
		}

		return this.model.getLabel(item);	// String
	},

	getLabelClass: function(/*dojo.data.Item*/ item, /*Boolean*/ opened){
		// summary:
		//		Overridable function to return CSS class name to display label
		// tags:
		//		extension
		if (this._isPinned(item)) {
			return "rmps-app-explorer-tree-label-pinned";
		} else {
			return "rmps-app-explorer-tree-label";
		}
	},

	_replaceItem: function(node, olditem, newitem)
	{
		// Get the children of the parent, and then reinsert them
		var children = node.getChildren();
		var items = dojo.map(children, function(n) {return n.item;});
		var ix = dojo.indexOf(items, olditem);
		if (ix === -1) {
			console.debug("Reinsertion of item failed.");
			return;
		}

		var oldnode = children[ix];
		// Verify that the indexes match
		if (oldnode.item !== olditem) {
			console.debug("Node children and items list indexes don't match!");
		}

		// Orphan the children of the old node
		this._orphanChildren(oldnode);

		// Remove old item and add new item
		items.splice(ix, 1, newitem);

		// Set the new items
		node.setChildItems(items);
	},

	_reloadChildren: function(node, item, ignoreitem)
	{
		var id = this.model.getIdentity(item);
		if(this._treeState.siblingEllipsis[id]) {
			delete this._treeState.siblingEllipsis[id];
			this._saveState();
		}
		// Check if there is a chain ellipsis as child to the parent
		// and if so keep track of the hidden items
		var hiddenitems = [];
		var ellipsisitems = [];
		dojo.forEach(node.getChildren(), function(n, ix) {
			if (n.item && n.item.isEllipsis && ignoreitem !== n.item) {
				if (!n.item.ellipsisSiblings) {
					ellipsisitems.push(n.item);
					hiddenitems.push(n.item.ellipsisParentChain[0]);
				}
			}
		});

		// Root is a special case.
		// reloadChildren at this point will show all the top nodes
		if (item === this.model.root) {
			this._explorer.setTopNodes(this._explorer.getTopNodes(), null, null);
		}

		// Partly copied code from _expandNode in Tree.js
		// Get the children of the parent, and then reinsert them
		node.markProcessing();
		this.model.getChildren(item, function(items){
				node.unmarkProcessing();
				// Replace the child items that are hidden by chain ellipsis with the ellipsis items
				if (hiddenitems.length > 0) {
					// Create a new list of items so the one in store or model is kept intact.
					var newitems = [];
					dojo.forEach(items, function(i) {
						var ix = dojo.indexOf(hiddenitems, i);
						if (ix === -1) {
							newitems.push(i);
						} else {
							newitems.push(ellipsisitems[ix]);
						}
					});
					node.setChildItems(newitems);
				} else {
					node.setChildItems(items);
				}
			},
			function(err){
				console.error(this, ": error loading root children: ", err);
			}
		);
	},

	_getNodeFromId: function(id, parentnode)
	{
		var thenode = null;		
		var nodes = this._itemNodesMap[id];
		if (!nodes) {
			nodes = this._itemNodesMap[id.replace(/\./g,"%2E")];
			if (nodes) {
				console.warn("partially encoded ID was passed!" + arguments.callee.caller);
			}
		}
		if (nodes && nodes.length) {
			if (nodes.length === 1) {
				return nodes[0];
			} else if (nodes.length > 1) {
				if (parentnode) {
					dojo.some(nodes, function(n){
						if(n.getParent() === parentnode){
							thenode = n;
							return true;
						}
						return false;
					});
				} else {
					console.warn("Multi-parented item accessed and parent node is not known. First parent selected.");
				}
			}
		}
		return thenode;
	},

	_deleteEllipsis: function(node)
	{
		if (node.getChildren().length > 0 || node.getParent()) {
			console.error("Deleting unattached ellipsis!");
		}
		this.dndController.removeTreeNode(node);
		node.destroyRecursive();
		delete this._itemNodesMap[node.item.ellipsisUUID];
		node._deadEllipsis = true;
	},

	_expandParentEllipsis: function(node, item, nbrItems)
	{
		// Check if the ellipsis should remain or not and get the parent item
		var topShowItem = null;
		var bottomShowItem = null;
		var parentItem = null;
		var deleteNode = false;
		var cid;
		var i;
		node._destroyPopup();
		node._createPopup();
		if (nbrItems > (item.ellipsisParentChain.length - 2)) {
			// Ellipsis should be removed
			topShowItem = item.ellipsisParentChain[0];
			bottomShowItem = item.ellipsisParentChain[item.ellipsisParentChain.length - 1];
			parentItem = item.ellipsisParent;
			item.ellipsisNumber = 0;
			deleteNode = true;
			//All items included to parent chain are marked with property "owningEllipsisNode"
			//This property points to ellipsis node containing item with this parent chain
			//We should unmark all items in parent chain now
			for(i = 0; i < item.ellipsisParentChain.length; i++) {
				item.ellipsisParentChain[i].owningEllipsisNode = false;
				cid = this.model.getIdentity(item.ellipsisParentChain[i]);
				delete this._treeState.hiddenByParentEllipsis[cid];
			}
		} else {
			topShowItem = bottomShowItem = item.ellipsisParentChain.pop();
			//Unmark item removed form parent chain
			topShowItem.owningEllipsisNode = false;
			cid = this.model.getIdentity(topShowItem);
			delete this._treeState.hiddenByParentEllipsis[cid];
			for (i = 0;i < nbrItems-1; ++i) {
				topShowItem = item.ellipsisParentChain.pop();
				//Unmark item removed form parent chain
				topShowItem.owningEllipsisNode = false;
				cid = this.model.getIdentity(topShowItem);
				delete this._treeState.hiddenByParentEllipsis[cid];
			}
			item.ellipsisNumber = item.ellipsisParentChain.length;
			parentItem = item.ellipsisParentChain[item.ellipsisNumber - 1];
			// Update node appearance
			this._onItemChange(item);
		}

		// Get the nodes for all the items
		var parentid = this.model.getIdentity(parentItem);
		var parentNode = this._getNodeFromId(parentid);
		// Get the items of all the children of the parent
		var parentItems = dojo.map(parentNode.getChildren(), function(node) {return node.item;});

		var topshowid = this.model.getIdentity(topShowItem);
		var topShowNode = this._getNodeFromId(topshowid);
		var bottomshowid = this.model.getIdentity(bottomShowItem);
		var bottomShowNode = this._getNodeFromId(bottomshowid);

		// Get the children items of the ellipsis node
		var ellipsisItems = dojo.map(node.getChildren(), function(node) {return node.item;});

		// Orphan the nodes that will get new parent nodes
		this._orphanChildren(node);

		var expanded = this._treeState.parentEllipsis[bottomshowid].expanded;
		delete this._treeState.parentEllipsis[bottomshowid];

		// Set children on their new owners
		if (deleteNode) {
			this._replaceItem(parentNode, item, topShowItem);
		} else {
			this._treeState.parentEllipsis[parentid] = {expanded : expanded, length : item.ellipsisParentChain.length};
			this._orphanChildren(parentNode);
			node.setChildItems(parentItems);
		}

		this._saveState();

		bottomShowNode.setChildItems(ellipsisItems);

		// Select the top show node
		//this.selectNode(topShowNode);

		if (deleteNode) {
			this._deleteEllipsis(node);
		}
	},

	_expandSiblingEllipsis: function(node, item) {
		var parent = node.getParent();
		if (!parent) {
			return;
		}

		this._reloadChildren(parent, item.ellipsisParent, item);

		this._deleteEllipsis(node);

		// Let's focus the first child of the parent
		//var children = parent.getChildren();
		//if(children.length)
		//	this.selectNode(children[0]);

		// TODO: Expand down to child node
	},

	onClick: function(/* dojo.data */ item, /*TreeNode*/ node){
		// summary:
		//		Callback when a tree node is clicked
		// tags:
		//		callback
		if (item.isEllipsis)
		{
			if (item.ellipsisSiblings) {
				// Show the elements behind the ellipsis
				this._expandSiblingEllipsis(node, item);
			} else {
				// Expand the bottom element of parent chain
				this._expandParentEllipsis(node, item, 1);
			}
		} else if (this._isFolderItem(item) || item.group || item.page) {
			// This is a folder or group node which does not have an associated editor
			// Expand the node instead
			this._onExpandoClick({node:node});
		} else if (this._isViewpointItem(item)) {
			this._onExpandoClick({node:node});
		} else if (node) {
			if (this.noResourceLinks) {
				if (node._checkbox) {
					node._checkbox.setChecked(!node.item._isSelected);
				}
			}
			else {
				node.openResource();
			}
		}
	},

	onDblClick: function(/* dojo.data */ item, /*TreeNode*/ node){
		// summary:
		//		Callback when a tree node is double-clicked
		// tags:
		//		callback
	},

	_onEnterKey: function(/*Object*/ message){
		this._publish("execute", { item: message.item, node: message.node} );
		this.onClick(message.item, message.node);
	},

	_recursiveSiblingCollapse : function(node, recursive, ignorePinned)
	{
		var parent = node.getParent();
		if (!parent) {
			return;
		}

		if (recursive) {
			this._recursiveSiblingCollapse(parent, recursive, ignorePinned);
		}

		var children = parent.getChildren();

		var items = [];

		var siblingellipsisitem = null;
		dojo.forEach(children, function(n){
			// Check if item is an ellipsis type
			// and check if any of them are pinned and add node to get it correct in order
			if (n.item.isEllipsis) {
				if (n.item.ellipsisSiblings) {
					siblingellipsisitem = n.item;
				} else {
					items.push(n.item);
				}

			} else if(n === node) {
				items.push(n.item);
			} else if(!ignorePinned) {
				if( this._isPinned(n.item) || this._havePinnedChildren(n, node) || this._isPinned(parent.item)) {
					items.push(n.item);
				}
			}
		}, this);
		if (!siblingellipsisitem && children.length < (items.length + 1)) {
			return; // No ellipsis needed
		}

		if (!siblingellipsisitem) {
			siblingellipsisitem = this._explorer._createEllipsisItem(children.length - items.length, parent.item);
		}

		// Finally add the sibling ellipsis node
		items.push(siblingellipsisitem);

		var parentid = this.model.getIdentity(parent.item);
		this._treeState.siblingEllipsis[parentid] = {shown : {}};
		for(var i = 0; i < items.length; i++) {
			if(!items[i].isEllipsis) {
				var itemid = this.model.getIdentity(items[i]);
				this._treeState.siblingEllipsis[parentid].shown[itemid] = {};
			}
		}
		this._saveState();

		// Set the children
		parent.setChildItems(items);
	},

	// Collects a top down list of the parent chain. Returns the top parent
	// ignorePinned parameter indicates if we need to respect pinned items or not
	_collectParentChainRecursive : function(node, chain, ignorePinned)
	{
		var parent = node.getParent();
		if (!parent) {
			return node;
		}

		// Stop if parent is root.
		var parentid = this.model.getIdentity(parent.item);
		if (parentid === this._getRootID()) {
			return parent;
		}

		// Stop if parent is pinned or have children that are pinned
		if(!ignorePinned) {
			if (this._havePinnedChildren(parent, node)) {
				return parent;
			}
			if (this._isPinned(parent.item)) {
				return parent;
			}
			var parentOfParent = parent.getParent();
			if(parentOfParent && this._isPinned(parentOfParent.item)) {
				return parent;
			}
		}

		if(parent.item.isEllipsis)
		{
			this._expandParentEllipsis(parent, parent.item, parent.item.ellipsisParentChain.length);
			return this._collectParentChainRecursive(node, chain, ignorePinned);
		}

		var top = this._collectParentChainRecursive(parent, chain, ignorePinned);

		chain.push(parent.item);

		return top;
	},


	_getRootID: function()
	{
		return this.model.getIdentity(this.model.root);
	},

	_getRootNode: function()
	{
		return this._getNodeFromId(this._getRootID());
	},

	_orphanChildren : function(node)
	{
		node.getChildren().forEach(function(child){
			dijit._Container.prototype.removeChild.call(this, child);
		}, this);
	},

	_collapseParentChainEx : function(parent, topnode, parentchain, expand) {
		// Get the items of the parent's children which will become the children of the ellipsis
		var ellipsisitems = dojo.map(parent.getChildren(), function(node) {return node.item;});

		// Create the ellipsis item
		var ellipsis = this._explorer._createEllipsisItem(parentchain.length, topnode.item, parentchain);

		// Get the children of the top node and replace the item which is the top of the chain with the ellipsis.
		var items = [];
		var topchilditem = parentchain[0];
		dojo.forEach(topnode.getChildren(), function(n){
			var i = n.item;
			if (topchilditem === i) {
				items.push(ellipsis);
			}
			else {
				items.push(i);
			}
		}, this);

		// Save state
		for(var i = 0; i < parentchain.length; i++) {
			var cid = this.model.getIdentity(parentchain[i]);
			this._treeState.hiddenByParentEllipsis[cid] = {};
		}
		var parid = this.model.getIdentity(parent.item);
		this._treeState.parentEllipsis[parid] = {expanded : expand, length : parentchain.length};
		this._saveState();

		dojo.forEach(parentchain, function(chainitem) {
			chainitem.owningEllipsisNode = true;
		});

		// Orphan the children nodes of the parent since they will be owned by the ellipsis node
		this._orphanChildren(parent);
		var done = new dojo.Deferred();
		var done_reject = function() {
			done.reject();
		};
		// Set the ellipsis to the owner of the hidden parent chain
		topnode.setChildItems(items).addCallback(dojo.hitch(this, function(){
			// Get the ellipsis node
			var ellipsisnode = this._getNodeFromId(ellipsis.ellipsisUUID);

			// Add items to ellipsis node
			var setChildren = ellipsisnode.setChildItems(ellipsisitems);
			setChildren.addCallback(dojo.hitch(this, function(){
				//Mark parent chain items
				//Later we can distinguish "collapsed" nodes from usual ones
				dojo.forEach(parentchain, function(chainitem) {
					chainitem.owningEllipsisNode = ellipsisnode;
				});

				// Expand ellipsis node
				if(expand) {
					this._expandNode(ellipsisnode).then(function() {
						done.resolve();
					}, done_reject );
				} else {
					done.resolve();
				}
			}));
			setChildren.addErrback(done_reject);
		}));
		return done;
	},

	_collapseParentChain : function(node, ignorePinned)
	{
		if (!node) {
			return;
		}

		// Get parent
		var parent = node.getParent();
		if (!parent) {
			return;
		}

		// node will be the child of the ellipsis node

		// collect all parents up to the top (or pinned node)
		var parentchain = [];

		var topnode = this._collectParentChainRecursive(node, parentchain, ignorePinned);
		if (!topnode) {
			return; // parent is root
		}

		// Only hide chains with 2 or more levels
		if (parentchain.length < 2) {
			return;
		}

		// parent can be changed after collecting chain
		parent = node.getParent();

		return this._collapseParentChainEx(parent, topnode, parentchain, true);
	},

	// Collapses everything down from a node. Returns true if node should be collapsed.
	_recursiveCollapseDown: function(node, collapsepinned, skiponedown)
	{
		var mayCollapse = !skiponedown;

		dojo.forEach(node.getChildren(), function(child) {
			if (!this._recursiveCollapseDown(child, collapsepinned, false)) {
				mayCollapse = false;
			}
		}, this);

		if (mayCollapse) {
			this._collapseNode(node);
		}

		if (!collapsepinned && this._isPinned(node.item)) {
			return false;
		}

		return mayCollapse;
	},

	_recursiveCollapseUp: function(node, root, collapsepinned)
	{
		var parent = node.getParent();

		if (parent)
		{
			dojo.forEach(parent.getChildren(), function(child) {
				if (child !== node) {
					this._recursiveCollapseDown(child, collapsepinned, false);
				}
			}, this);
		}

		if (parent !== root) {
			this._recursiveCollapseUp(parent, root, collapsepinned);
		}
	},

	collapseRootAll: function(bCollapsePinned)
	{
		this.collapseAll(null, bCollapsePinned, true);
	},

	// Collapses the tree. If node is provided, the parent chain to it is left expanded
	collapseAll: function(node, collapsepinned, skiponedown)
	{
		var root = this._getRootNode();
		var enabled = this._explorer._historyEnabled;
		this._explorer._enableHistory(false);

		if (node)
		{
			this._recursiveCollapseUp(node, root, collapsepinned);
			this._recursiveCollapseDown(node, collapsepinned, skiponedown);
		}
		else
		{
			// Collapse from the root, but not the root itself!
			this._recursiveCollapseDown(root, collapsepinned, true /*skiponedown*/);
		}

		// Wipe the state cookie
		delete this._openedItemIds;
		this._openedItemIds = {};
		this._explorer._enableHistory(enabled);
		this._saveState();
	},

	_showPinnedNodesRecursive : function(node) {
		var completed = new dojo.Deferred();
		this._expandNode(node).addCallback(dojo.hitch(this, function() {
			var children = node.getChildren();
			var defs = [];
			for(var i = 0; i < children.length; i++) {
				if(this._havePinnedChildren(children[i])) {
					defs.push(this._showPinnedNodesRecursive(children[i]));
				}
			}
			if (defs.length > 0) {
                var deflist = new dojo.DeferredList(defs);
                deflist.addBoth(function(){ completed.resolve(); });
			}else{
                completed.resolve();
			}
		}));
		return completed;
	},

	_getExpandedParent : function(node)
	{
		var parent = node.getParent();
		if(parent) {
			if(parent.item.isEllipsis) {
				this._expandParentEllipsis(parent, parent.item, 1);
				parent = node.getParent();
			}
		}
		return parent;
	},
	
	_equals: function(p1,p2){
		if (p1.length === p2.length) {
			var i = 0;
			while (i < p1.length && p1[i] == p2[i]) {
				i++;
			}
			if (i === p1.length) {
				return true;
			}
		}
		return false;
	},
	
	_isPathPinned: function(path){
		for (var i = 0;i < this._pinnedPaths.length; ++i) {
			if (this._equals(this._pinnedPaths[i], path)) {
				return i;
			}
		}
		return -1;
	},
	
	_popPath: function(path) {
		var ind = this._isPathPinned(path);
		if (ind !== -1){
			this._pinnedPaths.splice(ind, 1);
		}
		return ind;
	},
	
	_getPinnedNodes: function() {
		var nodes = [];
		dojo.forEach(this._pinnedPaths, dojo.hitch(this, function (p) {
		
			var node = this._getNodeFromId(p[0]);	
			if (node) {
				nodes.push(node);
			}
		}));
		return nodes;
	},

	collapseAllExceptPinned: function()
	{
		var enabled = this._explorer._historyEnabled;
		this._explorer._enableHistory(false);
		var def_array = [];
		dojo.forEach(this._pinnedItems, function(p) {
			var id = this.model.getIdentity(p);
			var node = this._getNodeFromId(id);
			if (node) {
				// Collapse siblings on everything node and up.
				this._recursiveSiblingCollapse(node, true /*recursive*/, false /*ignorePinned*/);
				// Collapse the parents
				var parent = this._getExpandedParent(node);
				if (parent) {
					def_array.push(this._collapseParentChain(parent, false /*ignorePinned*/));
				}
			}
		}, this);
		
		// remove undefine elements from def_array
		if (def_array.length > 0) {
			var i = 0;
			while (i < def_array.length) {
				if (!def_array[i]) {
					def_array.splice(i, 1);
					continue;
				}
				i++;
			}
		}
		
		var parent_done;
		if (def_array.length > 0) {
			parent_done = new dojo.DeferredList(def_array);
		} else  {
			parent_done = new dojo.Deferred();
		}
		var done = new dojo.Deferred();
		parent_done.then(dojo.hitch(this, function() {
			this._explorer._enableHistory(enabled);
			// Collapse all other nodes
			this.collapseAll(null /*root*/, false /*collapse pinned*/, false /*skip one down*/);
			// Make sure pinned nodes are visible
			this._showPinnedNodesRecursive(this._getRootNode()).then(function() {
				done.resolve();
			});	
		}), function() {
			done.reject();
		});
		return done;
	},
	

	OnCollapseAllExceptPinned: function()
	{
		this._unhideAll();
		if (this._pinnedPaths.length) {
			var def_array = [], i;			
			for (i = 0; i < this._pinnedPaths.length; ++i) {
				def_array.push(new dojo.Deferred());
				this.expandChain(dojo.map(this._pinnedPaths[i], ExplorerUtil.getResourceURIFromNodeID),
								{
									found : null,
									repeatIfFailed : false,
									repeatFunction : null,
									callback: def_array[i],
									enableHistory: this._explorer._historyEnabled
								}
				);
			}
			var def_list = new dojo.DeferredList(def_array);
			def_list.addCallback(dojo.hitch(this, function () {
				this.collapseAllExceptPinned();
			}));
		} else {
			this.collapseAllExceptPinned();
		}
	
	},

	_pinItemsFromList: function(list) {
		this.unpinAll();
		dojo.forEach(list, dojo.hitch(this, function(item) {
			this._togglePinned(item);
		}));
	},

	collapseAllExceptList: function(idList, completed)
	{
		var pinnedItems = [];
		dojo.forEach(this._pinnedItems, function (item) { pinnedItems.push(item);} );
		var items = [];
		dojo.forEach(idList, dojo.hitch(this, function(id) {
			var node = this._getNodeFromId(id);
				if (node && node.item) {
					items.push(node.item);	
				}
			}));
		this._pinItemsFromList(items);
		var collapsed = this.collapseAllExceptPinned();
		collapsed.addBoth(dojo.hitch(this, function(){
			this._pinItemsFromList(pinnedItems);
			completed.resolve();
		}));
		
	},



	contextCollapseSelection: function()
	{
		var selection = this.lastFocused;

		var existingNode = selection;
		if (!existingNode) {
			return;
		}

		this.contextCollapse(existingNode, true /*collapse pinned*/);
	},

	contextCollapse: function(existingNode /*Node*/, collapsePinned /*Boolean*/)
	{

		// No collapse of ellipsis nodes
		if (existingNode && existingNode.item.isEllipsis) {
			return;
		}
		var enabled = this._explorer._historyEnabled;
		this._explorer._enableHistory(false);

		var parent = this._getExpandedParent(existingNode);

		// Collapse the tree except chain to existingNode
		this.collapseAll(existingNode, collapsePinned, true /*skip collapsing closest children*/);

		// Collapse siblings on everything node and up.
		this._recursiveSiblingCollapse(existingNode, true /*recursive*/, true /*ignorePinned*/);

		/*
		// Collapse children (if more than one)
		var children = existingNode.getChildren();
		if (children.length > 1)
		{
			//destroy existing siblings ellipsis if any
			for(var i = 0; i < children.length; i++) {
				if(children[i].item.isEllipsis && children[i].item.ellipsisSiblings) {
					children[i].destroyRecursive();
					delete children[i];
					break;
				}
			}
			var items = [
				this._createEllipsisItem(children.length, existingNode.item)
			];

			existingNode.setChildItems(items);
		}
		*/
		// Make sure immediate children of the node are not hidden
		this._unhideNodeRecursive(existingNode, false);

		// Collapse parent chain
		this._collapseParentChain(parent, true /*ignorePinned*/);

		// Focus/Select node
		this.selectNode(existingNode);
		this._explorer._enableHistory(enabled);
		this._saveState();

	},

	unpinAll : function()
	{
		this._pinnedPaths = [];
		while(this._pinnedItems.length > 0) {
			var item = this._pinnedItems[0];
			this._togglePinned(item);
		}
	},

	_canPin: function(item)
	{
		return item ? !item.isEllipsis : false;
	},
	
	_getPath: function(item){
		var path = [];
		var id = this.model.getIdentity(item);
		var node = this._getNodeFromId(id);
		while (node) {
			id = this.model.getIdentity(node.item);
			path.push(id);
			node = node.getParent();
		}
		return path;
	},
	
	_setPinned: function(item, pinned){

		item.isPinned = pinned;

		this._onItemChange(item);
		var path = this._getPath(item);
		
		var ix = dojo.indexOf(this._pinnedItems, item);
		if (item.isPinned && -1 === ix) {
			this._pinnedItems.push(item);
		} else if (!item.isPinned && -1 !== ix){
			this._pinnedItems.splice(ix, 1);
		}
	},

	_togglePinned: function(item)
	{
		this._setPinned(item, !this._isPinned(item));
	},

	_isPinned: function(item)
	{
		if (!item) {
			return false;
		}

		if ("isPinned" in item) {
			return item.isPinned;
		}

		return false;
	},

	// Checks if a node have pinned children.
	_havePinnedChildren: function(node, ignorenode)
	{
		var children = node.getChildren();
		for (var i=0,l=children.length; i<l; ++i){
			var n = children[i];
			if (!ignorenode || n !== ignorenode) {
				if (this._isPinned(n.item)) {
					return true;
				}
				if (this._havePinnedChildren(n)) {
					return true;
				}
			}
		}

		return false;
	},

	_unhideNodeRecursive : function(node, recursive) {
		var children = node.getChildren();
		for(var i = 0; i < children.length; i++) {
			var child = children[i];
			if(child.item.isEllipsis) {
				if(child.item.ellipsisSiblings) {
					this.tree._expandSiblingEllipsis(child, child.item);
				}
				else {
					this.tree._expandParentEllipsis(child, child.item, child.item.ellipsisParentChain.length);
				}
			}
		}
		if(recursive) {
			children = node.getChildren();
			for(i = 0; i < children.length; i++) {
				this._unhideNodeRecursive(children[i], recursive);
			}
		}
	},

	_unhideAll : function() {
		this._unhideNodeRecursive(this._getRootNode(), true);
	},

	togglePinned: function()
	{
		this._togglePinned(this._getSelectedItemAttr().item);
	},

	dumpNodes: function()
	{
// Need to be updated for Dojo 1.4.1
//		for (var ix in this._itemNodeMap) {
//			var n = this._itemNodeMap[ix];
//			var string = "";
//			string += n.getParent() ? "p.c" : "|.c";
//			string += n.getChildren().length;
//			if (n.item) {
//				string += ".i.";
//				string += n.item.isEllipsis ? "e." : "|.";
//				string += this.getLabel(n.item);
//			} else {
//				string += ".|.|.|.";
//			}
//			string += ix;
//			console.debug(string);
//		}
	},

	_checkAndExpandSiblingEllipsis : function(node, chain) {
		if(chain.length > 0) {
			//check if we need to expand some sibling ellipsis
			//if we can't find the next node from chain among children of this node or corresponding ellipsis node
			//let's try to expand sibling ellipsis
			var topChainId = ExplorerUtil.getNodeIDFromResourceURI(chain[chain.length - 1], this._explorer._expMain.explorerID);
			var nodeFound = false;
			var nodeChildren;
			if(node.item.owningEllipsisNode) {
				var parentchain = node.item.owningEllipsisNode.item.ellipsisParentChain;
				if(node.item === parentchain[parentchain.length-1]) {
					nodeChildren = node.item.owningEllipsisNode.getChildren();
				}
			}
			if(!nodeChildren) {
				nodeChildren = node.getChildren();
			}
			for(var i = 0; i < nodeChildren.length; i++) {
				if(!nodeChildren[i].item.isEllipsis) {
					var childNodeId = this.model.getIdentity(nodeChildren[i].item);
					if(childNodeId === topChainId) {
						nodeFound = true;
						break;
					}
				}
			}
			if(!nodeFound) {
				for(i = 0; i < nodeChildren.length; i++) {
					var nodeChild = nodeChildren[i];
					if(nodeChild.item.isEllipsis && nodeChild.item.ellipsisSiblings) {
						this._expandSiblingEllipsis(nodeChild, nodeChild.item);
						break;
					}
				}
			}
		}
	},

	_checkIfItemInPages: function(pagednode, i, pageitems, pagechildid, def) {
		if (i >= pageitems.length) {
			def.errback(SILENT_ERROR);
			return;
		}

		var pageitem = pageitems[i];
		var id = this.model.getIdentity(pageitem);
		var pagenode = this._getNodeFromId(id, pagednode);
		if (!pagenode) {
			def.errback(SILENT_ERROR);
			return;
		}

		var self = this;
		if (pagenode.state === "UNCHECKED") {
			// Check for children, if not found then recurse.
			pagenode.markProcessing();
			this.model.getChildren(pageitem, function(subitems) {
				pagenode.setChildItems(subitems).addBoth(function(){
					pagenode.unmarkProcessing();
					if (self._getNodeFromId(pagechildid, pagenode)) {
						// Found. Expand page node and call back
						self._expandNode(pagenode).addCallback(function(){
							def.callback();
						});
					} else {
						// Check next
						self._checkIfItemInPages(pagednode, ++i, pageitems, pagechildid, def);
					}
				});
			}, function() {});
		} else {
			// Since loaded and not previously found, just check the next one.
			this._checkIfItemInPages(pagednode, ++i, pageitems, pagechildid, def);
		}
	},
	
	_checkForPagedChild: function(node, chain, pos, found) {
		if (pos < 0 || !chain.length) {
			found.errback(SILENT_ERROR);
		}
		
		var pagechildid = ExplorerUtil.getNodeIDFromResourceURI(chain[pos], this._explorer._expMain.explorerID);
		var pagechildnode = this._getNodeFromId(pagechildid);
		if (!pagechildnode) {
			// Not added to tree, so we need to find it. Get children and load them one by one.
			var self = this;
			var def = new dojo.Deferred();
			this.model.getChildren(node.item, function(items) {
				self._checkIfItemInPages(node, 0, items, pagechildid, def);
			}, function() {});
			def.addCallback(function() {
				found.callback();
			});
			def.addErrback(dojo.hitch(this, function() {
				// check if items further in the chain can be found among paged children
				this._checkForPagedChild(node, chain, pos - 1, found);
			}));
		} else {
			// Found. Expand page node and call back
			this._expandNode(pagechildnode.getParent()).addCallback(function(){
				found.callback();
			});
		}
	},
	

	// Check if this is a paged node and if so, load pages until next node in chain is found
	_checkForPagedChildren: function(node, chain) {
		var def = new dojo.Deferred();
		if (chain.length > 0 && node.item.paged) {
			this._checkForPagedChild(node, chain, chain.length - 1, def);
		} else {
			def.callback(); // Not paged. No op.
		}
		return def;
	},
	
	_checkForChildrenInGroup: function(group, chain) {
		var def = new dojo.Deferred();
		var item = group.item;
		var requestedItemId = ExplorerUtil.getNodeIDFromResourceURI(chain[chain.length-1], this._explorer._expMain.explorerID);
		var self = this;
		this.model.getChildren(item, function(items) {
			if (dojo.some(items, function(item) {
				var id = self.model.getIdentity(item);
				if (id === requestedItemId) {
					return true;
				}
				return false;
			})) {
				// The wanted item is in the group so expand the group
				self._expandNode(group).addCallback(function() {
					def.callback(group);
				});
			} else {
				// The wanted item is not in the group. Check for paging as well
				// Check for children with paged nodes
				self._checkForPagedChildren(group, chain).addBoth(function(){
					// Since we didn't find the requested item in group we return an error
					def.errback(SILENT_ERROR);
				});
			}
		}, function() {
			// Error, check the next group
			def.errback(SILENT_ERROR);
		});
		return def;
	},
	
	_checkForChildrenInGroupRecursive: function(children, index, chain) {
		var def = new dojo.Deferred();
		var child = children[index];
		if (!child.item.group) {
			// Groups first in list, so no need to continue
			def.callback();
			return def;
		}
		
		var groupDef = this._checkForChildrenInGroup(child, chain);
		index++;
		groupDef.addCallback(function(group) {
			def.callback();
		});
		var self = this;
		groupDef.addErrback(function() {
			// Not in group, check the next
			if (index < children.length) {
				self._checkForChildrenInGroupRecursive(children, index, chain).addCallback(function() {
					def.callback();
				});
			} else {
				// Not found in any group
				def.callback();
			}
		});
		return def;
	},
	
	_checkForChildrenInGroups: function(node, chain) {
		var children = node.getChildren();
		if (children.length === 0) {
			// No children. No op
			var def = new dojo.Deferred();
			def.callback();
			return def;
		}
		
		return this._checkForChildrenInGroupRecursive(children, 0, chain);
	},

	expandChain: function(chain, info)	{
		var node;
		if (!chain.length || chain.length < 1) {
			if(!info.found && info.repeatIfFailed){
				info.repeatFunction();
			}
			if (info.found) {
				node = info.found;
				this.selectNode(node);
				dijit.scrollIntoView(node.domNode);
				this._explorer._enableHistory(info.enableHistory);
				this._saveState();
				if (info.callback) {
					info.callback.callback(node.getNodeURI());
				} 
			} else {
				if (info.callback) {
					info.callback.callback(null);
				}
			}
			return;
		}
	
		var id = ExplorerUtil.getNodeIDFromResourceURI(chain.pop(), this._explorer._expMain.explorerID);
		node = this._getNodeFromId(id);
		if (!node) {
			// continue expand with the rest of the chain
			this.expandChain(chain, info);
			return;
		}
	
		info.found = node;
		this._explorer._enableHistory(false);
		var i;
		if(node.item.owningEllipsisNode) {
			//This node is collapsed
			//We need to know if we need to expand corresponding parent chain and to which extent
			//To do this we find the last element in parent chain present in chain (including node)
			//Then we need to expand parent chain below this found element
			var parentchain = node.item.owningEllipsisNode.item.ellipsisParentChain;
			var startSearch = 0;
			for(i = -1; i < chain.length; i++) {
				var nodeToCompare;
				if(i === -1) {
					nodeToCompare = node;
				} else {
					nodeToCompare = this._getNodeFromId(encodeURIComponent(chain[i]));
				}
				if(!nodeToCompare) {
					continue;
				}
				for(var j = startSearch; j < parentchain.length; j++) {
					if(parentchain[j] === nodeToCompare.item) {
						startSearch = j;
						break;
					}
				}
			}
			var numberToExpand = parentchain.length - startSearch - 1;
			if(numberToExpand > 0) {
				this._expandParentEllipsis(node.item.owningEllipsisNode, node.item.owningEllipsisNode.item, numberToExpand);
			}
		}
	
		// TODO: Code below check if the element is the last selectable or not. This does not work for
		// resources or elements that are hidden with a filter though. Instead of calling expandNode directly,
		// the children should be loaded and only if there is a element in chain further down should the
		// node be expanded.
		// If elements are filtered out then we must select the last resource in the chain
		var elementsFiltered = !this._explorer.store.areElementsShown();
		var onlyElementsRemain = true;
	
		// Check here whether the remaining items in the chain are elements with no more resources
		if (elementsFiltered) {
			for (i = chain.length-1; i >= 0; i--) {
				if (chain[i].search("#") === -1) {
					onlyElementsRemain = false;
					break;
				}
			}
		}
	
		// This is the last item in the chain that is selectable. Select it now.
		if (chain.length === 0	|| (elementsFiltered && onlyElementsRemain)) {
				this.selectNode(node);
				dijit.scrollIntoView(node.domNode);
				this._explorer._enableHistory(info.enableHistory);
				this._saveState();
				if (info.callback) {
					info.callback.callback(node.getNodeURI());
				}
				return;
			}
	
		var self = this;
		// Expand the node, and on completion expand the next part of chain
		this._expandNode(node).addCallback(function(){
			self._checkAndExpandSiblingEllipsis(node, chain);
			// Ensure that children of groups are loaded to make it possible to locate elements within
			self._checkForChildrenInGroups(node, chain).addBoth(function() {
					// Check for children with paged nodes
					self._checkForPagedChildren(node, chain).addBoth(function(){
						// Expand chain when all groups are loaded and check for pages is complete
						self.expandChain(chain, info);
					}); 
				});
			});
	},


	//Redefine keyboard handlers to allow navigation to subsection top nodes
	_onUpArrow : function(message) {
		var processed = false;
		var node = message.node;
		var previousSibling = node.getPreviousSibling();
		if (this.onUpArrowExit && !previousSibling) {
			var parent = node.getParent();
			if (!this.showRoot && parent === this.rootNode) {
				processed = this.onUpArrowExit();
			}
		}
		if (!processed) {
			this.inherited(arguments);
		}
	},

	_onDownArrow : function(message) {
		var processed = false;

		var node = this._getNextNode(message.node);
		if (!node && this.onDownArrowExit) {
			processed = this.onDownArrowExit();
		}

		if (!processed) {
			this.inherited(arguments);
		}
	},

	_onLeftArrow : function(message) {
		var processed = false;

		var node = message.node;
		if (this.onLeftArrowExit && (!node.isExpandable || !node.isExpanded)) {
			var parent = node.getParent();
			if (!this.showRoot && parent === this.rootNode) {
				processed = this.onLeftArrowExit();
			}
		}

		if (!processed) {
			this.inherited(arguments);
		}
	},

	getLastNode : function() {
		var rootChildren = this.rootNode.getChildren();
		if(rootChildren.length > 0) {
			return this._getLastNodeRecursive(rootChildren[rootChildren.length - 1]);
		}
		return null;
	},

	_getLastNodeRecursive : function(node) {
		if(!node.isExpandable || !node.isExpanded) {
			return node;
		}
		var children = node.getChildren();
		if(children.length > 0) {
			return this._getLastNodeRecursive(children[children.length - 1]);
		}
		else {
			return node;
		}
	},

	onLoad: function() {
		this._explorer._enableHistory(true);
		dojo.publish("rmps.ui.explorer.Explorer.loaded",[{
			explorerID: this._explorer._expMain.explorerID
		}]);
	},
	
	selectNode: function(node) {
		if (this.dndController) {
			this.dndController.setSelection([node]);
		}
	},
	
	focusNode: function(node) {
		if (!node._deadEllipsis) {
			this.inherited(arguments);
			this.selectNode(node);
		}
	},
	
	getEnclosingTreeNode: function(node) {
		var widget =  dijit.getEnclosingWidget(node);
		while (widget && !dojo.hasClass(widget.domNode, "dijitTreeNode")) {
			widget = dijit.getEnclosingWidget(widget.domNode.parentNode);
		}	
		return widget; 
	},
	
	//from dojo 1.8.0; to be removed after dojo 1.8.0 adopted
	_expandNode: function(/*TreeNode*/ node, /*Boolean?*/ recursive){
		var def = this.inherited(arguments);	
		this._startPaint(def);
		return def;
	},
	
	_collapseNode: function(/*TreeNode*/ node){
		var ret = this.inherited(arguments);
		if (ret) {
			this._startPaint(ret);
			return ret;
		}
		return;
	},
	
	resize: function(changeSize){
		// Intentionally disabled.  See WI33654
	},

	_outstandingPaintOperations: 0,
	_startPaint: function(/*Promise|Boolean*/ p){
		// summary:
		//		Called at the start of an operation that will change what's displayed.
		// p:
		//		Promise that tells when the operation will complete.  Alternately, if it's just a Boolean, it signifies
		//		that the operation was synchronous, and already completed.

		this._outstandingPaintOperations++;
		if(this._adjustWidthsTimer){
			clearTimeout(this._adjustWidthsTimer);
			delete this._adjustWidthsTimer;
		}

		var oc =dojo.hitch(this, function(){
			this._outstandingPaintOperations--;

			if(this._outstandingPaintOperations <= 0 && !this._adjustWidthsTimer && this._started){
				// Use defer() to avoid a width adjustment when another operation will immediately follow,
				// such as a sequence of opening a node, then it's children, then it's grandchildren, etc.
				var self = this;
				this._adjustWidthsTimer = setTimeout(function(){
					self._adjustWidths();
				});	
			}
		});
		dojo.when(p, oc, oc);
	},

	_adjustWidths: function(){
		// summary:
		//		Get width of widest TreeNode, or the width of the Tree itself, whichever is greater,
		//		and then set all TreeNodes to that width, so that selection/hover highlighting
		//		extends to the edge of the Tree (#13141)

		if(this._adjustWidthsTimer){
			clearTimeout(this._adjustWidthsTimer);
			delete this._adjustWidthsTimer;
		}
		
		if (!this.rootNode) {
			return;
		}

		var maxWidth = 0,
			nodes = [];
		function collect(/*TreeNode*/ parent){
			var node = parent.rowNode;
			node.style.width = "auto";		// erase setting from previous run
			maxWidth = Math.max(maxWidth, node.clientWidth);
			nodes.push(node);
			if(parent.isExpanded){
				dojo.forEach(parent.getChildren(), collect);
			}
		}
		collect(this.rootNode);
		maxWidth = Math.max(maxWidth, dojo.contentBox(this.domNode).w);	// do after node.style.width="auto"
		dojo.forEach(nodes, function(node){
			node.style.width = maxWidth + "px";		// assumes no horizontal padding, border, or margin on rowNode
		});
	},
	//from dojo 1.8.0; to be removed after dojo 1.8.0 adopted
	
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//from dojo 1.6.1
//TODO: update, check https://csnext.ibm.com:8002/jazz/web/projects/IBM%20Dojo%20Toolkit%20Public#action=com.ibm.team.workitem.viewWorkItem&id=6821	
	
	_onKeyPress: function(/*Event*/ e){
		// summary:
		//		Translates keypress events into commands for the controller
		if(e.altKey){ return; }
		var dk = dojo.keys;
		//----------------------------------------------------------------------
		var treeNode = this.getEnclosingTreeNode(e.target);
		//----------------------------------------------------------------------
		if(!treeNode){ return; }

		var key = e.charOrCode;
		if(typeof key === "string" && key !== " "){	// handle printables (letter navigation)
			// Check for key navigation.
			if(!e.altKey && !e.ctrlKey && !e.shiftKey && !e.metaKey){
				this._onLetterKeyNav( { node: treeNode, key: key.toLowerCase() } );
				dojo.stopEvent(e);
			}
		}else{	// handle non-printables (arrow keys)
			// clear record of recent printables (being saved for multi-char letter navigation),
			// because "a", down-arrow, "b" shouldn't search for "ab"
			if(this._curSearch){
				clearTimeout(this._curSearch.timer);
				delete this._curSearch;
			}

			var map = this._keyHandlerMap;
			if(!map){
				// setup table mapping keys to events
				map = {};
				map[dk.ENTER]="_onEnterKey";
				//On WebKit based browsers, the combination ctrl-enter
				//does not get passed through. To allow accessible
				//multi-select on those browsers, the space key is
				//also used for selection.
				map[dk.SPACE]= map[" "] = "_onEnterKey";
				map[this.isLeftToRight() ? dk.LEFT_ARROW : dk.RIGHT_ARROW]="_onLeftArrow";
				map[this.isLeftToRight() ? dk.RIGHT_ARROW : dk.LEFT_ARROW]="_onRightArrow";
				map[dk.UP_ARROW]="_onUpArrow";
				map[dk.DOWN_ARROW]="_onDownArrow";
				map[dk.HOME]="_onHomeKey";
				map[dk.END]="_onEndKey";
				this._keyHandlerMap = map;
			}
			if(this._keyHandlerMap[key]){
				this[this._keyHandlerMap[key]]( { node: treeNode, item: treeNode.item, evt: e } );
				dojo.stopEvent(e);
			}
		}
	}
	
});

}());
