/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.explorer.ExplorerTreeNode");

dojo.require("dijit.Menu");
dojo.require("dijit.MenuItem");
dojo.require("dijit.CheckedMenuItem");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.RadioButton");

dojo.require("jazz.ui.HoverPopup");
dojo.require("rmps.app.core.util");


dojo.require("rmps.ui.explorer.ExplorerUtilities");
var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;

dojo.requireLocalization("rmps.ui.explorer", "ExplorerMessages");
var messages = dojo.i18n.getLocalization("rmps.ui.explorer", "ExplorerMessages");

var CoreUtil = rmps.app.core.util;

dojo.declare("rmps.ui.explorer.ExplorerTreeNode", dijit._TreeNode, {

	_popup: null,
	_popupContent: null,
	_menuNode : null,
	_menu : null,
	_pinnedMenuItem: null,
	_resourceLink : null,
	fnIsChecked : null,
	fnOnCheck : null,

	// Minor changes from dijit.tree.TreeNode. Added a <a> for jazz.ui.ResourceLink to attach to.
	templateString: "<div class=\"dijitTreeNode\" role=\"presentation\">" +
						"<div dojoAttachPoint=\"rowNode\" class=\"dijitTreeRow\" role=\"presentation\">" +
						//from dojo 1.8.0; to be removed after dojo 1.8.0 adopted
						"<div data-dojo-attach-point=\"indentNode\" class=\"dijitInline\"></div>" +
						//from dojo 1.8.0; to be removed after dojo 1.8.0 adopted
						"<img src=\"${_blankGif}\" alt=\"\" dojoAttachPoint=\"expandoNode\" class=\"dijitTreeExpando \" role=\"presentation\"/>" +
							"<span dojoAttachPoint=\"expandoNodeText\" class=\"dijitExpandoText\" role=\"presentation\">" +
							"</span>" +
							"<span dojoAttachPoint=\"contentNode\" class=\"dijitTreeContent\" role=\"presentation\">" +
								"<img src=\"${_blankGif}\" alt=\"\" dojoAttachPoint=\"iconNode\" class=\"dijitIcon dijitTreeIcon\" role=\"presentation\"/>" +
								// Added <a>
								"<a data-dojo-attach-point=\"_anchorResourceLink\" style=\"position: relative; bottom: -2px;\" >" +
									"<span dojoAttachPoint=\"labelNode\" class=\"dijitTreeLabel\" role=\"treeitem\" tabindex=\"-1\" aria-selected=\"false\">" +
									"</span>" +
								"</a>"+
							"</span>" +
						"</div>" +
						"<div dojoAttachPoint=\"containerNode\" class=\"dijitTreeContainer\" role=\"presentation\" style=\"display: none; height: 100%; padding-bottom: 1px;\">" +
						"</div>" +
					"</div>",
	
	onContextCollapse : function() {
		this.tree.contextCollapse(this, true /*collapse pinned*/);
	},
	
	onTogglePinned : function()
	{
		this.tree._togglePinned(this.item);
		this.tree._saveState();
	},
	
	isTopNode: function() {
		return this.getParent().item.root ? true : false;
	},
	
	removeTopNode: function() {
		this.tree._explorer.removeTopNode(this.getNodeURI());
	},
	
	onContextMenu : function(event) 
	{
		if(!this._menu) {
			this._menu = new dijit.Menu();
			dojo.addClass(this._menu.domNode, "rmps-app-explorer-node-context-menu");
			
			if (this.isTopNode()) {
				this._menu.addChild(new dijit.MenuItem({
					label: messages.showPinnedOnly,
					onClick: dojo.hitch(this.tree, this.tree.collapseAllExceptPinned)
				}));
				this._menu.addChild(new dijit.MenuItem({
					label: messages.removeTree,
					iconClass: "rmps-app-explorer-remove-tree-icon",
					onClick: dojo.hitch(this, this.removeTopNode)
				}));
			}
			
			
			this._menu.addChild(new dijit.MenuItem({
				label: messages.collapseContext,
				iconClass: "rmps-app-explorer-focus-on-element-icon",
				onClick: dojo.hitch(this, this.onContextCollapse)
			}));
			
			this._pinnedMenuItem = new dijit.CheckedMenuItem({
				label: messages.pin,
				onChange: dojo.hitch(this, this.onTogglePinned),
				checked: this.tree._isPinned(this.item)
			});
			this._menu.addChild(this._pinnedMenuItem);
			
			if (!this.item.isEllipsis && !this.tree._isFolderItem(this.item) && !this.item.group) {
			    this.addContextMenuItemsForElement(this._menu, this.getNodeURI());
			}
			
			this._menu.startup();
			this.connect(this._menu, "_onBlur", dojo.hitch(this, function(){dijit.popup.close(this._menu);}));
		} else if (this._pinnedMenuItem) {
			this._pinnedMenuItem._setCheckedAttr(this.tree._isPinned(this.item));
		}
		
		this.tree.focusNode(this);
	
		// open pop-up menu
		dijit.popup.open({
			popup: this._menu,
			parent: this,
			around: this._menuNode,
			orient: {'BR':'TL', 'BL':'TR', 'TR':'BL', 'TL':'BR'},
			onExecute: dojo.hitch(this, function(){dijit.popup.close(this._menu);}),
			onCancel: dojo.hitch(this, function(){dijit.popup.close(this._menu);})
		});
		
		this._menu.focus();
		dojo.stopEvent(event);
	},
	addContextMenuItemsForElement : function (menu, resourceUri)
	{
		// do nothing... Implementations to be provided via dojox.lang.aspect.advise
	},
	_createPopup : function()
	{
		this._popup = new jazz.ui.HoverPopup({
			around: this.contentNode,
			createContents: dojo.hitch(this, this.getEllipsisInfo)
		});
	},
		
	postCreate: function()
	{
		this.inherited(arguments);
		
		dojo.addClass(this.domNode, "explorer-tree-node");
		
		dojo.setSelectable(this.rowNode, false);
		
		var item = this.item;
		if (item.isEllipsis) {
			this._createPopup();
		}
		
		var model = this.tree.model;
		
		if (!item.isEllipsis && !item.page && !item.group && item !== model.root) {
			var uri = ExplorerUtil.getResourceURIFromNodeID(model.getIdentity(item));
			this._resourceLink = ExplorerUtil.createResourceLink(uri, this._anchorResourceLink);  
		}	
		
		var self = this;
		dojo.connect(this.labelNode, "onblur", function(evt){
			self.tree._onNodeBlur(self);
		});
		dojo.connect(this.labelNode, "onkeypress", function(evt){
			self.tree._onNodeKeypress(self, evt);
		});
	},
	
	destroyDescendants: function(preserveDom) {
		dojo.forEach(this.getChildren(), dojo.hitch(this, function(node){
			if(node.item.isEllipsis){
				//expand this node
				if(node.item.ellipsisSiblings) {
					this.tree._expandSiblingEllipsis(node, node.item);
				} else {
					this.tree._expandParentEllipsis(node, node.item, node.item.ellipsisParentChain.length);
				}
			}
		}));
		
		this.inherited(arguments, [preserveDom]);
	},
	
	_destroyPopup : function()
	{
		if (this._popupContent) {
			this._popupContent.destroyRecursive();
			delete this._popupContent;
			this._popupContent = null;
		}
			
		if (this._popup) {
			//disable popup to prevent opening it after this node is destroyed
			this._popup.enabled = false;
			this._popup.destroyRecursive();
			delete this._popup;
			this._popup = null;
		}
	},
	
	destroy: function()
	{
		this._destroyPopup();
		
		if(this._menu) {
			this._menu.destroyRecursive();
			delete this._menu;
		}
		
		if (this._resourceLink) {
			this._resourceLink.destroyRecursive();
			delete this._resourceLink;
		}
		
		this.inherited(arguments);
	},
	
	getEllipsisInfo: function(popupWidget)
	{
		this._popupContent = new rmps.ui.explorer._PopupContent({
			tree: this.tree,
			node: this,
			item: this.item
		});
		return this._popupContent.domNode;
	},
	
	openResource: function()
	{
		// Open in design page
		setTimeout(dojo.hitch(this, function() {
			var uri = CoreUtil.getDesignViewResourceUri(this.getNodeURI());
			window.location.href = uri;
		}), 0);
	},
	
	_restoreSiblingEllipsis : function() {
		var id = this.tree.model.getIdentity(this.item);
		var hideDef = null;
		if(this.tree._treeState.siblingEllipsis[id]) {
			//check if we have a sibling ellipsis node
			//if not we need to create it
			var children = this.getChildren();
			var siblingEllipsisItem = null;
			var toShow = [];
			var toHide = 0;
			for(var i = 0; i < children.length; i++) {
				if(children[i].item.isEllipsis) {
					if(children[i].item.ellipsisSiblings) {
						siblingEllipsisItem = children[i].item;
						break;
					} else {
						toShow.push(children[i].item);
					}
				} else {
					var childid = this.tree.model.getIdentity(children[i].item);
					if(this.tree._treeState.siblingEllipsis[id].shown[childid]) {
						toShow.push(children[i].item);
					} else {
						toHide++;
					}
				}
			}
			if(!siblingEllipsisItem && children.length) {
				if (this.tree && this.tree._explorer) {
					siblingEllipsisItem = this.tree._explorer._createEllipsisItem(toHide, this.item);
					toShow.push(siblingEllipsisItem);
				} else {
					console.warn("Cannot create sibling ellipsis without explorer.");
				}
				
				this.tree._treeState.siblingEllipsis[id] = {shown : {}};
				for(i = 0; i < toShow.length; i++) {
					if(!toShow[i].isEllipsis) {
						var itemid = this.tree.model.getIdentity(toShow[i]);
						this.tree._treeState.siblingEllipsis[id].shown[itemid] = {};
					}
				}
				this.tree._saveState();
				
				hideDef = this.setChildItems(toShow);
			}
		}
		return hideDef;
	},
	
	_restoreParentEllipsis : function() {
		if(this.item.isEllipsis) {
			return;
		}
		var id = this.tree.model.getIdentity(this.item);
		if(this.tree._treeState.parentEllipsis[id] && !this.item.owningEllipsisNode) {
			var expanded = this.tree._treeState.parentEllipsis[id].expanded;
			var length = this.tree._treeState.parentEllipsis[id].length;
			var chainrev = [];
			var cnode = this;
			var cid, i;
			while(length > 0 && cnode) {
				cid = this.tree.model.getIdentity(cnode.item);
				if(!this.tree._treeState.hiddenByParentEllipsis[cid]) {
					break;
				}
				chainrev.push(cnode.item);
				cnode = cnode.getParent();
				length--;
			}
			if(length !== 0) {
				for(i = 0; i < chainrev.length; i++) {
					cid = this.tree.model.getIdentity(chainrev[i]);
					delete this.tree._treeState.hiddenByParentEllipsis[cid];
				}
				delete this.tree._treeState.parentEllipsis[id];
				return;
			}
			var topnode = cnode;
			var chain = [];
			for(i = chainrev.length - 1; i >= 0; i--) {
				chain.push(chainrev[i]);
			}
			
			this.tree._collapseParentChainEx(this, topnode, chain, expanded);		
		}
	},
	
	_addControls : function() {
		if (!this.item.isEllipsis && !this.item.page && !this.item.group) {			
			this._menuNode = dojo.create("div", null, this.rowNode);
			dojo.addClass(this._menuNode, "explorer-tree-node-context-menu-div");
		
			this.connect(this._menuNode, "onclick", dojo.hitch(this, this.onContextMenu));
			
			// Add check box
			if (this.fnIsChecked && this.fnOnCheck && !this._checkbox) {
				
				// check if the item is selectable or not
				var selectable = true;
				if (this.tree.onlyTypes) {
					// ensure that the item type is allowed
					selectable = false;
					for (var i=0, lg=this.tree.onlyTypes.length; i<lg; i++) {
						if (this.item.type) {
							if (this.item.type.indexOf(this.tree.onlyTypes[i]) != -1) {
								selectable = true;
								break;
							}
						}
					}
				}
				
				if (selectable) {
					var checkboxArgs = {
						checked: this.fnIsChecked(this.item),
						onChange: dojo.hitch(this, function(b) {
							this.fnOnCheck(this.item, b);
						})
					};
								
					this._checkbox = this.enableMultiSelect ? 
							new dijit.form.CheckBox(checkboxArgs, "checkBox") : 
							new dijit.form.RadioButton(checkboxArgs, "checkBox");
					dojo.place(this._checkbox.domNode, this.expandoNode, 'after');
				}
			}
		}
	},
	
	
	hasParentWithId: function (id) {
		if (this.tree.model.getIdentity(this.item) === id) {
			return true;
		}
		var parent = this.getParent();
		return parent ? parent.hasParentWithId(id) : false;
	},
	
	// This is a copy of setChildItems borrowed from dojo 1.6.1-20110519-IBM
	// TODO: Update with dojo!
	// It was copied to prevent endless expand of cyclic containment hierarchies
	// Check 69845	Cyclic containment hierarchy not handled on the web client explorer
	setChildItems_parent: function (items)
	{
		// summary:
        //        Sets the child items of this node, removing/adding nodes
        //        from current children to match specified items[] array.
        //        Also, if this.persist == true, expands any children that were previously
        //         opened.
        // returns:
        //        Deferred object that fires after all previously opened children
        //        have been expanded again (or fires instantly if there are no such children).

        var tree = this.tree,
            model = tree.model,
            defs = [];    // list of deferreds that need to fire before I am complete


		// Orphan all my existing children.
		// If items contains some of the same items as before then we will reattach them.
		// Don't call this.removeChild() because that will collapse the tree etc.
		dojo.forEach(this.getChildren(), function(child){
			dijit._Container.prototype.removeChild.call(this, child);
		}, this);

        this.state = "LOADED";

        if(items && items.length > 0){
            this.isExpandable = true;

            // Create _TreeNode widget for each specified tree node, unless one already
            // exists and isn't being used (presumably it's from a DnD move and was recently
            // released
            dojo.forEach(items, function(item){
                var id = model.getIdentity(item),
                    existingNodes = tree._itemNodesMap[id],
                    node;
                if(existingNodes){
                    for(var i=0;i<existingNodes.length;i++){
                        if(existingNodes[i] && !existingNodes[i].getParent()){
                            node = existingNodes[i];
                            node.set('indent', this.indent+1);
                            break;
                        }
                    }
                }
                if(!node){
                    node = this.tree._createTreeNode({
                            item: item,
                            tree: tree,
                            isExpandable: model.mayHaveChildren(item),
                            label: tree.getLabel(item),
                            tooltip: tree.getTooltip(item),
							dir: tree.dir,
							lang: tree.lang,
                            indent: this.indent + 1
                        });
                    if(existingNodes){
                        existingNodes.push(node);
                    }else{
                        tree._itemNodesMap[id] = [node];
                    }
                }
                this.addChild(node);

                // If node was previously opened then open it again now (this may trigger
                // more data store accesses, recursively)
                var bExpand = this.tree.autoExpand || (this.tree._state(item));
                
                if (bExpand && existingNodes && existingNodes.length > 1)
                { // Check 69845	Cyclic containment hierarchy not handled on the web client explorer
                  // Do not expand node if it has a parent with the same id, since otherwise we will not be able to stop
                    bExpand = !this.hasParentWithId(id);
                }
                if (bExpand) {
                    defs.push(tree._expandNode(node));
                }
                
            }, this);

            // note that updateLayout() needs to be called on each child after
            // _all_ the children exist
            dojo.forEach(this.getChildren(), function(child, idx){
                child._updateLayout();
            });
        }else{
            this.isExpandable=false;
        }

        if(this._setExpando){
            // change expando to/from dot or + icon, as appropriate
            this._setExpando(false);
        }
        
		// Set leaf icon or folder icon, as appropriate
		this._updateItemClasses(this.item);

        // On initial tree show, make the selected TreeNode as either the root node of the tree,
        // or the first child, if the root node is hidden
        if(this === tree.rootNode){
            var fc = this.tree.showRoot ? this : this.getChildren()[0];
            if(fc){
                fc.setSelected(true);
                tree.lastFocused = fc;
            }else{
                // fallback: no nodes in tree so focus on Tree <div> itself
                tree.domNode.setAttribute("tabIndex", "0");
            }
        }

        return new dojo.DeferredList(defs);    // dojo.Deferred
	},
	
	setChildItems : function(/* Object[] */ items)
	{
		var isRoot = (this === this.tree.rootNode);
		// A trick to prevent selection corruption by inherited setChildItems called on the tree root
		if (isRoot) {
			this.tree.rootNode = null;
		}

		// Check 69845	Cyclic containment hierarchy not handled on the web client explorer
		// Call a modified copy of inherited method.		
		var def = this.setChildItems_parent(items);
		if(isRoot) {
			this.tree.rootNode = this;
			this.tree._saveState();
		}
				
		def.addCallback(dojo.hitch(this, function(result) {
			var enabled = this.tree._explorer._historyEnabled;
			this.tree._explorer._enableHistory(false);
			if(this.tree.persist) {
				var hideDef = this._restoreSiblingEllipsis();
				if(hideDef) {
					hideDef.addCallback(dojo.hitch(this, function(data) {
						this._restoreParentEllipsis();
						return data;
					}));
				} else {
					this._restoreParentEllipsis();
				}				
			}
			dojo.forEach(items, dojo.hitch(this, function (child){
				var path = this.tree._getPath(child);
				if (this.tree._popPath(path) !== -1) {
					this.tree._setPinned(child, true);
				}	
			}));
			
			this.tree._explorer._enableHistory(enabled);
			return result;
		}));
		
		dojo.forEach(this.getChildren(), dojo.hitch(this, function(node){
				node._addControls();
			}));
		//from dojo 1.8.0; to be removed after dojo 1.8.0 adopted
		this.tree._startPaint(def);
		//from dojo 1.8.0; to be removed after dojo 1.8.0 adopted
		return def;
	},
	
	getNodeURI: function() {
		var id = this.tree.model.getIdentity(this.item);
		
		if (this.item.viewpoint !== undefined) {
			var viewpointRE = new RegExp("_viewpoint_" + this.item.viewpoint, "g");
			id = id.replace(viewpointRE, "");
		}

		return ExplorerUtil.getResourceURIFromNodeID(id);
	},
	
	//from dojo 1.8.0; to be removed after dojo 1.8.0 adopted
	_updateItemClasses: function(item){
		this.inherited(arguments);
		this.tree._startPaint(true);
	},
	
	_setIndentAttr: function(indent){
		// summary:
		//		Tell this node how many levels it should be indented
		// description:
		//		0 for top level nodes, 1 for their children, 2 for their
		//		grandchildren, etc.

		// Math.max() is to prevent negative padding on hidden root node (when indent == -1)
		var pixels = (Math.max(indent, 0) * this.tree._nodePixelIndent) + "px";

		dojo.style(this.domNode, "backgroundPosition",	pixels + " 0px");
		dojo.style(this.indentNode, this.isLeftToRight() ? "paddingLeft" : "paddingRight", pixels);

		dojo.forEach(this.getChildren(), function(child){
			child.set("indent", indent+1);
		});
		
		this._set("indent", indent);
	},
	//from dojo 1.8.0; to be removed after dojo 1.8.0 adopted
	
	getParent: function(){
		var parent = this.inherited(arguments);
		// Since 1.8 dojo's getParent return Tree widget as a parent of a root node, while we expect null.
		return parent && parent.declaredClass === "rmps.ui.explorer.ExplorerTreeNode" ? parent : null;
	}
});

dojo.declare("rmps.ui.explorer._PopupContent", dijit._Widget, {
	
	"class": "rmps-app-explorer-hover",
	pagesize : 5,
	
	postCreate: function()
	{
		if (this.item.ellipsisSiblings) {
			this._createSiblingEllipsisInfo();
		} else {
			this._createParentEllipsisInfo(0 /*page number*/);
		}
	},
	
	_expandSiblingEllipsisAll: function()
	{
		this.tree._expandSiblingEllipsis(this.node, this.item);
	},
	
	_createSiblingEllipsisInfo: function()
	{
		var table = dojo.create("table", null, this.domNode);

		// Create the first table body with static content
		var tbody1 = dojo.create("tbody", null, table);

		var tr1_2 = dojo.create("tr", null, tbody1);
		var td1 = dojo.create("td", null, tr1_2);
		td1.appendChild(dojo.doc.createTextNode(messages.showAll));
		this.connect(td1, "onclick", this._expandSiblingEllipsisAll);
	},
	
	_expandParentEllipsisAll: function()
	{
		this.tree._expandParentEllipsis(this.node, this.item, this.item.ellipsisParentChain.length);
	},
	
	
	_creteExpandEllipsisHandler: function (td, toRemove) {
		this.connect(td, "onclick", dojo.hitch(this, function() {
			this.tree._expandParentEllipsis(this.node, this.item, toRemove);
		}));
	},
	
	_createParentEllipsisInfo: function(page)
	{
		this.domNode.innerHTML = "";
		var table = dojo.create("table", null, this.domNode);
		
		// Create the first table body with static content
		var tbody1 = dojo.create("tbody", null, table);

		var chain = this.item.ellipsisParentChain;
		var len = chain.length;
		
		var tr1 = dojo.create("tr", null, tbody1);
		var th1 = dojo.create("td", null, tr1);
		th1.appendChild(dojo.doc.createTextNode(dojo.string.substitute(messages.showAllNbr, [len])));
		this.connect(th1, "onclick", this._expandParentEllipsisAll);

		// Create the second table body with dynamic content
		var tbody2 = dojo.create("tbody", null, table);

		var tr2 = dojo.create("tr", null, tbody2);
		var th2 = dojo.create("th", null, tr2);
		th2.appendChild(dojo.doc.createTextNode(messages.ellipsisHoverTitle));
		
		// TODO: No sense in starting at index 0 since the we at least hide two levels with the ellipsis.
		var start = this.pagesize * page;
		var end = start + this.pagesize;
		if(end > len) {
			end = len;
		}
		var tr;
		for (var i = start; i < end; i++) {
			tr = dojo.create("tr", null, tbody2);
			var td = dojo.create("td", null, tr);
			dojo.attr(td, "colspan", 2);
			td.appendChild(dojo.doc.createTextNode(this.tree.model.getLabel(chain[i])));
			this._creteExpandEllipsisHandler(td, chain.length - i);
		}
		var showprev = (page > 0) ? true : false;
		var shownext = (this.pagesize * (page + 1) < len) ? true : false;
		var span;
		if(showprev || shownext) {
			tr = dojo.create("tr", null, tbody2);
			var td_prev = dojo.create("td", null, tr);
			td_prev.style.textDecoration = "none";
			if(showprev) {
				span = dojo.create("span", null, td_prev);
				dojo.addClass(span, "rmps-app-explorer-ellipsis-hover-previous");
				span.innerHTML = '<img src="' + 
					dojo.moduleUrl("rmps.app", "graphics/icons/previous_arrow.gif") +
					'" style="vertical-align : bottom">' +
					messages.previous;
				this.connect(span, "onclick", dojo.hitch(this, function() {
					this._createParentEllipsisInfo(page - 1);
				}));
			}
			var td_next = dojo.create("td", null, tr);
			td_next.style.textDecoration = "none";
			td_next.style.textAlign = "right";
			if(shownext) {
				span = dojo.create("span", null, td_next);
				dojo.addClass(span, "rmps-app-explorer-ellipsis-hover-next");
				span.innerHTML = messages.next + '<img src="' + 
				dojo.moduleUrl("rmps.app", "graphics/icons/next_arrow.gif") +
				'" style="vertical-align : bottom">';
				this.connect(span, "onclick", dojo.hitch(this, function() {
					this._createParentEllipsisInfo(page + 1);
				}));
			}
		}
	} 
});

}());