/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	
	dojo.provide("rmps.core.explorer.ExplorerTreeService");
	
	dojo.require("jazz.app.context");
	dojo.require("rmps.core.RootServices");
	dojo.require("rmps.core.user.UserClient");
	dojo.require("rmps.core.client");
	dojo.require("rmps.core.util.IRmpsConstants");

	var bts = rmps.core.explorer.ExplorerTreeService;
	var UserClient = rmps.core.user.UserClient;
	var IRmpsConstants = rmps.core.util.IRmpsConstants;
	
	var EXPLORER_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "explorer");
	var EXPLORER_TREE_PATH = EXPLORER_PATH + "/tree";
	
	dojo.mixin(bts, {
		
		explorerTreeService: function(query, args) {
			var content = {};
			
			// Get the user uri
			var userId = dojo.getObject("jazz.app.currentApplication.config.user.userId");
			if (userId) {
				content.userURI = UserClient.getUserUri(userId);
			}
			
			if (query) {
				if (query.root) {
					content.root = query.root;
				}
				if (query.uri) {
					content.resourceURI = query.uri;
				} else if (typeof query === 'string') {
					content.resourceURI = query;
				}
			}
			
			content[IRmpsConstants.PROJECT_ID] = (args && args.projectId) || jazz.app.context.get().itemId;
						
			var viewpoint = (query && query.viewpoint) || (args && args.viewpoint); 
			if (viewpoint) {
				content.viewpointURI = viewpoint;
			}
			
			var explorerID = (query && query.explorerID) || (args && args.explorerID);
			if (explorerID) {
				content.explorerID = explorerID;
			} else {
				console.log("Explorer ID is missing");
			}
			
			// Scope
			var scope = (query && query.scope) || (args && args.scope);
			if (scope && scope !== "all") {
				content.scope = scope;
			}
			
			return rmps.core.client.xhrGet({
				url: EXPLORER_TREE_PATH,
				handleAs: 'json',
				headers: {
					"Accept" : "application/json"
				},
				content: content
			}, false /*sendProjectId*/, (query && query.configurationUri) || (args && args.configurationUri));
		}
	});
	
	dojo.mixin(bts.explorerTreeService, {
		
		servicePath: EXPLORER_TREE_PATH + "/",
		
		put: function(id, value) {
			// Noop
			var d = new dojo.Deferred();
			d.callback(); 
			return d;
		},
		post: function(id, value) {
			// Noop
			var d = new dojo.Deferred();
			d.callback(); 
			return d;
		}/*,
		delete: function(id, value) {
			// Noop
			var d = new dojo.Deferred();
			d.callback(); 
			return d;
		}*/
	});

}());