/*******************************************************************************
* Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	
	dojo.provide("rmps.ui.explorer.ExplorerUtilities");
	
	dojo.require("rmps.core.RootServices");
	dojo.require("rmps.core.user.UserClient");
	dojo.require("rmps.core.client");	
	dojo.require("rmps.core.util.ContextUtils");

	dojo.require("rmps.app.core.util");
	
	dojo.require("jazz.ui.ResourceLink");
	dojo.require("jazz.app.context");
	
	var EXPLORER_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "explorer");
	var EXPLORER_CHAIN_PATH = EXPLORER_PATH + "/chain";
	var EXPLORER_TREE_PATH = EXPLORER_PATH + "/tree";
	var EXPLORER_GROUP_INFO_PATH = EXPLORER_PATH + "/groupinfo";
	var EXPLORER_FILTER_ON_TYPE_PATH = EXPLORER_PATH + "/filterontype";
	
	//must be the same as used on the server
	var EXPLORER_ID_PREFIX = "__";
	
	var UserClient = rmps.core.user.UserClient;
	var ExplorerUtilities = rmps.ui.explorer.ExplorerUtilities;
	var ContextUtils = rmps.core.util.ContextUtils;

	
	dojo.mixin(ExplorerUtilities, {
		
		EXPLORER_PROVIDERS_VIEWPOINT_URI: "http://jazz.net/xmlns/rmps/explorer/1.0/providerViewpoint/providersViewpoint",
		EXPLORER_PROVIDERS_DETAIL_VIEWPOINT_URI: "http://jazz.net/xmlns/rmps/explorer/1.0/providerViewpoint/providersScopeAllViewpoint",
		SCOPE_RESOURCES: "resources",
		SCOPE_ALL: "all",
		
		
		canShowInTree: function(uri, viewpointURI) {
			// Check the explorer chain service if resource is visible in explorer
			var url = EXPLORER_CHAIN_PATH;
			var content = {
				resourceURI: uri,
				viewpointURI: viewpointURI,
				isEnabled: true
			};
			var getArgs = {
				url: url,
				headers: {
					"Accept": "application/json",
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "json",
				content: content
			};	
			var def = new dojo.Deferred();
			var handler = rmps.core.client.xhrGet(getArgs);
			handler.addCallback(function(response) {
				var enabled = response && response.enabled;
				def.callback({enabled: enabled});
			});
			handler.addErrback(function(error) {
				// Interpret as disabled
				def.callback({enabled: false});
			});
			return def;
		},
		
		// Returns the URI of an item. Handles the case when the item is only partial.
		getItemURI: function(item) {
			var id = item.id || item.$ref;

			if (!id) {
				return null;
			}
			
			return this.getResourceURIFromNodeID(id);
		},
		
		// Returns the children of the element based on the URI.
		getChildren: function(uri, viewpointURI) {
			var content = {
				
				scope: this.getScopeFromViewpointURI(viewpointURI),
				viewpointURI: viewpointURI,
				pageSize: 25
			};
			if (uri) {
				content.resourceURI = uri;
			} else {
				content.root = jazz.app.context.get().itemId;
			}

			var args = {
				url: EXPLORER_TREE_PATH,
				content: content,
				handleAs: "json",
				headers: {
					accept: "application/json",
					"X-ibm-rmps-internal": "true"
				}
			};
				
			return rmps.core.client.xhrGet(args);
		},
		
		getParentChain: function(uri, viewpointURI, dataFormat) {
			var url = EXPLORER_CHAIN_PATH;
			var content = {
				resourceURI : uri,
				userURI: dojo.getObject("jazz.app.currentApplication.config.user.uri"),
				checkFiltered: false
			};
			if (viewpointURI) {
				content.viewpointURI = viewpointURI;
			}
			if (dataFormat) {
				content.isDataFormat = true;
			}

			var getArgs = {
				url: url,
				headers: {
					"Accept": "application/json",
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "json",
				content: content
			};	
			return rmps.core.client.xhrGet(getArgs);
		},
		
		isGroup: function(uri) {
			var result = uri.match(/(.*)_group_(.*)$/);
			return result && result[1] && result[2];
		},
		
		getGroupInfo: function(groupUri, viewpointURI) {
			var url = EXPLORER_GROUP_INFO_PATH;
			var content = {
				groupURI: groupUri
			};
			if (viewpointURI) {
				content.viewpointURI = viewpointURI;
			}

			var getArgs = {
				url: url,
				headers: {
					"Accept": "application/json",
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "json",
				content: content
			};	
			var def = rmps.core.client.xhrGet(getArgs);
			def.addCallback(function(info) {
				// If there is no icon associated with group, use the folder icon for groups
				if (!info.iconUri) {
					info.iconUri = dojo.moduleUrl("rmps.app", "graphics/icons/folder_open_co.gif").toString();
				}
			});
			return def;
		},
		
		filterOnType: function(/*Array*/ resources, typeURI) {
			var url = EXPLORER_FILTER_ON_TYPE_PATH;

			var postArgs = {
				url: url,
				headers: {
					"Accept": "application/json",
					"Content-Type": "application/json",
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "json",
				postData: dojo.toJson({
					typeURI: typeURI,
					resources: resources
				})
			};	
			return rmps.core.client.xhrPost(postArgs);
		},
		
		
		getManyParentChains: function(uris, checkFiltered, viewpointURI) {
			var url = EXPLORER_CHAIN_PATH;
			var content = {
				userURI: dojo.getObject("jazz.app.currentApplication.config.user.uri"),
				checkFiltered: checkFiltered
			};
			
			if (viewpointURI) {
				content.viewpointURI = viewpointURI;
			}

			return rmps.core.client.xhrPost({
				url: url,
				headers: {
					"Accept" : "application/json",
					"Content-Type" : "application/json",
					"X-ibm-rmps-internal" : "true"
				},
				handleAs: 'json',				
				postData: dojo.toJson({
					uris: uris,
					content: content
				})
			});
		},
				
		showItemInSidebarExplorer: function(resourceURI, expID) {
			this._ensureSidebarIsShown(expID).then(function() {
				dojo.publish("rmps.web.app.explorer.sidebar.show", 
					[{
						uri: resourceURI,
						explorerID: expID 
					}]);
			});
		},
		
		focusManyItemsInSidebarExplorer: function(resourceURIs, expID) {
			this._ensureSidebarIsShown(expID).then(function() {
				dojo.publish("rmps.web.app.explorer.sidebar.focusMany", 
					[{
						uris: resourceURIs,
						explorerID: expID 
					}]);
			});
		},
		
		
		
		_ensureSidebarIsShown: function(expID) {
			var def = new dojo.Deferred();
			var handler = dojo.subscribe("rmps.ui.explorer.Explorer.loaded", dojo.hitch(this, function(message){
				if (message.explorerID !== expID) {
					return;
				}
				dojo.unsubscribe(handler);
				def.resolve();
					
			}));
			
			dojo.publish("rmps.app.ui.internal.pages.ShowSidebar",[{
				explorerID: expID
			}]);
			return def;
		},
		
		getResourceURIFromNodeID: function (nodeID) {
			var pureID = nodeID;
			if (nodeID.slice(-EXPLORER_ID_PREFIX.length) === EXPLORER_ID_PREFIX) {
				var tmp = nodeID.substr(0, nodeID.length - EXPLORER_ID_PREFIX.length);
				var ind = tmp.lastIndexOf(EXPLORER_ID_PREFIX);
				if (ind !== -1) {
					pureID = nodeID.substr(0, ind);
				}
			}
			return decodeURIComponent(pureID);
		},
		
		getExplorerIDSuffix: function(explorerID) {
			return EXPLORER_ID_PREFIX + explorerID + EXPLORER_ID_PREFIX;
		},

		getNodeIDFromResourceURI: function(uri, explorerID) {
			// This method must use the same encoding as the server, which is using URLEncoder with the addition that '.' are
			// also encoded.
			// URLEncoder differ from encodeURIComponent that [~'()!] is not encoded by encodeURIComponent, but URLEncoder does
			// Another difference is that the server replaces spaces with '+', while the wncodeURIComponent put a '%20'
			return encodeURIComponent(uri).replace(/\./g,"%2E").replace(/!/g,"%21").replace(/'/g,"%27").replace(/\(/g,"%28").replace(/\)/g,"%29").replace(/~/g,"%7E").replace("%20","+")
				+ this.getExplorerIDSuffix(explorerID);
		},
				
		createResourceLinkForOntology: function(uri, domNode) {
			var root = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL;
			var linkUri = root + "/rendering?resourceUri=" + encodeURIComponent(uri) + "&" + ContextUtils.computeContextParams();
			return new jazz.ui.ResourceLink({
				// ResourceLink will fail resolving a relative DM URI, so using an absolute path for now
				srcNodeRef: domNode,
				uri: linkUri,
				presentationUri: net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/logical?resource=" + encodeURIComponent(uri) + "&" + ContextUtils.computeContextParams(),
				lazyFetch: true
			});
		},
		
		createResourceLink: function(uri, domNode) {
			var root = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL;
			var linkUri = root + "/rendering?resourceUri=" + encodeURIComponent(uri) + "&" + ContextUtils.computeContextParams();
			return new jazz.ui.ResourceLink({
				// ResourceLink will fail resolving a relative DM URI, so using an absolute path for now
				srcNodeRef: domNode,
				uri: linkUri,
				presentationUri: rmps.app.core.util.getDesignViewResourceUri(uri),
				lazyFetch: true
			});		
		},
		
		getScopeFromViewpointURI: function (viewpointURI) {
			return  viewpointURI === this.EXPLORER_PROVIDERS_VIEWPOINT_URI ? 
				this.SCOPE_RESOURCES :
				this.SCOPE_ALL;
		}

		
	});
}());