/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.explorer.Viewpoint");

dojo.require("rmps.core.resource.Resource");

var DEFINITION_NAME_PROPERTY = "label";
var DEFINITION_DESCRIPTION_PROPERTY = "comment";
var DEFINITION_QUERY_PROPERTY = "query";

var VIEWPOINT_TYPE = "Viewpoint";
var VIEWPOINT_ROOT_GROUP_PROPERTY = "rootGroup";
var VIEWPOINT_ONTOLOGY_PROPERTY = "ontology";
var VIEWPOINT_NODE_DEFINITION_PROPERTY = "nodeDefinition";

var NODE_DEFINITION_TYPE = "NodeDefinition";
var NODE_DEF_CHILD_GROUP_PROPERTY = "childGroup";
var NODE_DEF_CLASS_PROPERTY = "class";
var NODE_DEF_OPERATION_WITH_SUPERCLASS_PROPERTY = "operationWithSuperclass";

var GROUP_DEFINITION_TYPE = "GroupDefinition";
var GROUP_DEF_ICON_URI_PROPERTY = "iconURI";

var RDFS_PREFIX = "rdfs";

var EXPLORER_PREFIX = "rmpsExplorer";
var EXPLORER_VIEWPOINT_DEFAULT = "default";

rmps.core.explorer.ViewpointUtilities = {
	createChildQuery: function(clazz, operation, queryURI, childGroups) {
		var obj = {};
		var nodedef = obj[NODE_DEFINITION_TYPE] = {};
		nodedef[NODE_DEF_CLASS_PROPERTY] = clazz;
		if (operation) {
			nodedef[NODE_DEF_OPERATION_WITH_SUPERCLASS_PROPERTY] = operation;
		}
		if (queryURI) {
			nodedef[DEFINITION_QUERY_PROPERTY] = queryURI;
		}
		if (childGroups) {
			nodedef[NODE_DEF_CHILD_GROUP_PROPERTY] = childGroups;
		}
		return obj;
	},
	
	createGroup: function(name, queryURI, description, iconURI) {
		var obj = {};
		var groupdef = obj[GROUP_DEFINITION_TYPE] = {};
		groupdef[DEFINITION_NAME_PROPERTY] = name;
		groupdef[DEFINITION_QUERY_PROPERTY] = queryURI;
		if (description) {
			groupdef[DEFINITION_DESCRIPTION_PROPERTY] = description;
		}
		if (iconURI) {
			groupdef[GROUP_DEF_ICON_URI_PROPERTY] = iconURI;
		}
		return obj;
	},
	
	getChildQueryType: function(query) {
		return query[NODE_DEFINITION_TYPE][NODE_DEF_CLASS_PROPERTY];
	},

	getChildQueryOperation: function(query) {
		return query[NODE_DEFINITION_TYPE][NODE_DEF_OPERATION_WITH_SUPERCLASS_PROPERTY];
	},

	getChildQueryUri: function(query) {
		return query[NODE_DEFINITION_TYPE][DEFINITION_QUERY_PROPERTY];
	},
	
	getChildQueryGroups: function(query) {
		var groups = query[NODE_DEFINITION_TYPE][NODE_DEF_CHILD_GROUP_PROPERTY]; 
		return groups ? groups : [];
	},
	
	getGroupName: function(group) {
		return group[GROUP_DEFINITION_TYPE][DEFINITION_NAME_PROPERTY];
	},
	
	getGroupDescription: function(group) {
		var desc = group[GROUP_DEFINITION_TYPE][DEFINITION_DESCRIPTION_PROPERTY];
		return desc ? group[GROUP_DEFINITION_TYPE][DEFINITION_DESCRIPTION_PROPERTY] : "";
	},
	
	getGroupQueryUri: function(group) {
		return group[GROUP_DEFINITION_TYPE][DEFINITION_QUERY_PROPERTY];
	},
	
	getGroupIconUri: function(group) {
		return group[GROUP_DEFINITION_TYPE][GROUP_DEF_ICON_URI_PROPERTY];
	},
	
	isGroup: function(thing) {
		return GROUP_DEFINITION_TYPE in thing;
	},
	
	getIdFromUri: function(uri) {
		var segments = uri.split('/');
		if (!segments || segments.length < 1) {
			return null;
		}
		return segments[segments.length - 1];		
	}
};

dojo.declare("rmps.core.explorer.Viewpoint", rmps.core.resource.Resource, {

	contentType: "application/json",
	
	constructor: function() {
		this[DEFINITION_NAME_PROPERTY] = "";
		this[DEFINITION_DESCRIPTION_PROPERTY] = "";
		this[DEFINITION_QUERY_PROPERTY] = null;
	},
	
	getId: function() {
		return rmps.core.explorer.ViewpointUtilities.getIdFromUri(this.uri);
	},
	
	getName: function() {
		return this[DEFINITION_NAME_PROPERTY];
	},
	
	setName: function(name) {
		this[DEFINITION_NAME_PROPERTY] = name;
	},
	
	getDescription: function() {
		return this[DEFINITION_DESCRIPTION_PROPERTY];
	},
	
	setDescription: function(description) {
		this[DEFINITION_DESCRIPTION_PROPERTY] = description;
	},
	
	getRootQueryUri: function() {
		return this[DEFINITION_QUERY_PROPERTY];
	},
	
	setRootQueryUri: function(rootQueryUri) {
		this[DEFINITION_QUERY_PROPERTY] = rootQueryUri;
	},
	
	getRootGroups: function() {
		return this[VIEWPOINT_ROOT_GROUP_PROPERTY];
	},
	
	setRootGroups: function(rootGroups) {
		this[VIEWPOINT_ROOT_GROUP_PROPERTY] = rootGroups; 
	},
	
	getChildQueries: function() {
		return this[VIEWPOINT_NODE_DEFINITION_PROPERTY];
	},
	
	getChildQueryUris: function() {
		return dojo.map(this.getChildQueries(), function(childquery) {
			return childquery[DEFINITION_QUERY_PROPERTY];
		});
	},
	
	setChildQueries: function(childQueries) {
		this[VIEWPOINT_NODE_DEFINITION_PROPERTY] = childQueries;
	},
	
	isDefault: function() {
		return !!this[EXPLORER_VIEWPOINT_DEFAULT];
	},
	
	serialize: function() {
		var obj = {};
		var viewpoint = obj[VIEWPOINT_TYPE] = {};
		viewpoint[DEFINITION_NAME_PROPERTY] = this[DEFINITION_NAME_PROPERTY];
		if (this[DEFINITION_DESCRIPTION_PROPERTY]) {
			viewpoint[DEFINITION_DESCRIPTION_PROPERTY] = this[DEFINITION_DESCRIPTION_PROPERTY];
		}
		if (this[DEFINITION_QUERY_PROPERTY]) {
			viewpoint[DEFINITION_QUERY_PROPERTY] = this[DEFINITION_QUERY_PROPERTY];
		}
		if (this[VIEWPOINT_NODE_DEFINITION_PROPERTY]) {
			viewpoint[VIEWPOINT_NODE_DEFINITION_PROPERTY] = this[VIEWPOINT_NODE_DEFINITION_PROPERTY];
		}
		if (this[VIEWPOINT_ROOT_GROUP_PROPERTY]) {
			viewpoint[VIEWPOINT_ROOT_GROUP_PROPERTY] = this[VIEWPOINT_ROOT_GROUP_PROPERTY];
		}
		
		return dojo.toJson(obj);
	},
	
	deserialize: function(dataObject) {
		if (!dataObject[VIEWPOINT_TYPE]) {
			return;
		}
		dojo.safeMixin(this, dataObject[VIEWPOINT_TYPE]);
	},
	
	deserializeFromAtomJson: function(entry, ioArgs) {
		if (ioArgs) {
			var xhr = ioArgs.xhr;
			if (xhr.getResponseHeader("Last-Modified")) {
				this.lastModified = new Date(xhr.getResponseHeader("Last-Modified"));
			}
			if (xhr.getResponseHeader("ETag")) {
				this.eTag = xhr.getResponseHeader("ETag");
			}
			if (xhr.getResponseHeader("Location")) {
				this.uri = xhr.getResponseHeader("Location");
			}
		}
		
		// link
		var links = entry.link;
		for (var j = 0; j < links.length; j++) {
			var link = links[j];
			if (link.rel == "self") {
				this.uri = links[j].href;
			}
		}
		
		// name
		this[DEFINITION_NAME_PROPERTY] = entry.title[0].value;
		
		// description
		this[DEFINITION_DESCRIPTION_PROPERTY] = entry[RDFS_PREFIX + ":" + DEFINITION_DESCRIPTION_PROPERTY][0].value;
		if (this[DEFINITION_DESCRIPTION_PROPERTY] == undefined) {
			this[DEFINITION_DESCRIPTION_PROPERTY] = "";
		}
		
		// default
		var defaultvalue = entry[EXPLORER_PREFIX + ":" + EXPLORER_VIEWPOINT_DEFAULT];
		if (defaultvalue && defaultvalue[0] && defaultvalue[0].value) {
			this[EXPLORER_VIEWPOINT_DEFAULT] = defaultvalue[0].value === "true";
		} 
	}
});
})();