/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.explorer.ViewpointClient");

dojo.require("jazz.app.context");
dojo.require("rmps.core.explorer.Viewpoint");
dojo.require("rmps.core.RootServices");
dojo.require("rmps.core.client");

var EXPLORER_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "explorer");
var EXPLORER_VIEWPOINT_PATH = EXPLORER_PATH + "/viewpoint";
var COPY_EXPLORER_VIEWPOINT_PATH = EXPLORER_VIEWPOINT_PATH + "/copy";  
var EXPLORER_VIEWPOINT_ASSOCIATED_PARAM = "associated";

// "dummy" viewpoint URI that actually means usage of explorer providers
var EXPLORER_PROVIDERS_VIEWPOINT_ID = "providersViewpoint";

rmps.core.explorer.ViewpointClient = {

	getViewpoints: function(associatedWithProject, projectId) {
		var content = {
		};
		if (associatedWithProject) {
			content[EXPLORER_VIEWPOINT_ASSOCIATED_PARAM] = "true";
			content.projectId = projectId;
		}
		var args = {
			url: EXPLORER_VIEWPOINT_PATH,
			content: content,
			headers: {
				"X-ibm-rmps-internal": "true",
				"Accept": "application/atom+json"
			},
			handleAs: "json",
			load: function(response, ioArgs)
			{
				var result = [];
				var entries = response.entry;
				
				result.totalResults = parseInt(response["opensearch:totalResults"][0].value, 10);
				result.canEdit = ioArgs.xhr.getResponseHeader("canEdit");
				
				if (entries && entries.length > 0) {
					for (var i = 0; i < entries.length; i++) {
						// Note that viewpoint only contain partial data
						var viewpoint = new rmps.core.explorer.Viewpoint();
						viewpoint.deserializeFromAtomJson(entries[i], ioArgs);
						result.push(viewpoint);
					}
				}
				return result;
			}
		};
		return rmps.core.client.xhrGet(args);
	},
	
	getViewpoint: function(viewpointUri) {
		var args = {
			url: viewpointUri,
			headers: {
				"Accept": "application/json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				var location = ioArgs.xhr.getResponseHeader("Location");
				var viewpoint = new rmps.core.explorer.Viewpoint({
					eTag: ioArgs.xhr.getResponseHeader("ETag"),
					uri: location ? location : viewpointUri
					//lastModified = new Date(ioArgs.xhr.getResponseHeader("Last-Modified")),
					
				});
				viewpoint.deserialize(response);
				return viewpoint;
			}
		};
		return rmps.core.client.xhrGet(args);
	},
		
	deleteViewpoint: function(viewpoint) {
		var args = {
			headers: {
				"If-Match": viewpoint.eTag,
				"X-ibm-rmps-internal": "true"
			},
			url: viewpoint.uri
		};
		return rmps.core.client.xhrDelete(args);
	},
	
	createViewpoint: function(viewpoint, projectId) {
		var query = {
			projectId: projectId	
		};
		var args = {
			url: EXPLORER_VIEWPOINT_PATH + "?" + dojo.objectToQuery(query),
			headers: {
				"X-ibm-rmps-internal": "true",
			"Content-type": viewpoint.contentType
			},
			content: {
				projectId: projectId
			},
			hasBody: true,
			postData: viewpoint.serialize(),
			load: function(response, ioArgs) {
				// Update the viewpoint object
				viewpoint.eTag = ioArgs.xhr.getResponseHeader("ETag");
				viewpoint.uri = ioArgs.xhr.getResponseHeader("Location");
				//viewpoint.lastModified = new Date(ioArgs.xhr.getResponseHeader("Last-Modified"));
			}
		};
	 		
	 	return rmps.core.client.xhrPost(args);
	},

	updateViewpoint: function(viewpoint, projectId) {
 		var query = {
 			projectId: projectId	
 		};
		var args = {
			headers: {
				"Content-Type": viewpoint.contentType,
				"If-Match": viewpoint.eTag,
				"X-ibm-rmps-internal": "true"
			},
 			content: {
 				projectId: projectId
 			},
			url: viewpoint.uri + "?" + dojo.objectToQuery(query),
			hasBody: true,
			putData: viewpoint.serialize(),
			load: function(response, ioArgs) {
 				viewpoint.eTag = ioArgs.xhr.getResponseHeader("ETag");
				//viewpoint.lastModified = new Date(ioArgs.xhr.getResponseHeader("Last-Modified"));
			}
		};
		return rmps.core.client.xhrPut(args);
	},
	
	isProvidersViewpoint: function(viewpoint) {
		return viewpoint.getId() === EXPLORER_PROVIDERS_VIEWPOINT_ID;
	},
	
	createViewpointCopy: function(originalViewpointURI, projectId, newTitle) {
		var query = {
				projectId: projectId	
			};
			var content = {
				original: originalViewpointURI.substring(originalViewpointURI.lastIndexOf('/')+1),
				newTitle: newTitle
			};
			var args = {
				url: COPY_EXPLORER_VIEWPOINT_PATH + "?" + dojo.objectToQuery(query),
				headers: {
					"X-ibm-rmps-internal": "true",
					"Content-Type" : "application/json"
				},
				postData: dojo.toJson(content),
				load: function(response, ioArgs) {
					var newViewpointUri = ioArgs.xhr.getResponseHeader("Location");
					return newViewpointUri;
				}

			};
			return rmps.core.client.xhrPost(args);	
	}
};

})();