/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * Copyright IBM Corp. 2010, 2011. All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("rmpw.diagrams.layout.client.figures.ConnectionFigure");		

dojo.require("rmpw.diagrams.render.util.Point");
dojo.require("rmpw.diagrams.render.util.PointList");
dojo.require("rmpw.diagrams.render.graph.AbstractEdge");
dojo.require("rmpw.diagrams.layout.client.Edge");
dojo.require("rmpw.diagrams.layout.client.routers.AbstractRouter");

/**
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.layout.client.figures.ConnectionFigure", rmpw.diagrams.render.graph.AbstractEdge, {
	
	constructor: function(mapping, graph, sourceNode, targetNode) {
		dojo.mixin(this, $Edge_0(new Edge(), null, null));
		this.data = mapping;
		
		if (graph) {
			$add_0(graph.edges, this);
		}
		if (sourceNode) {
			this.setSource(sourceNode);
		}
		if (targetNode) {
			this.setTarget(targetNode);
		}
		
		this.avoidObstructions = mapping ? mapping.root.configuration.avoidObstructions === undefined ? true : mapping.root.configuration.avoidObstructions : true; // turn on by default
		
		this.endAnchor = null;
		this.startAnchor = null;
	
		this.points = new rmpw.diagrams.render.util.PointList();
		this._routerType = rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.IDENTITY_ROUTING;
		this.points.addPoint_1(new rmpw.diagrams.render.util.Point( 0, 0));
		this.points.addPoint_1(new rmpw.diagrams.render.util.Point( 100, 100));
	},
	
	setSource: function(source) {
		if (!this.source || source.mapping.id != this.source.mapping.id) {
			if (this.source) {
				$remove_3(this.source.outgoing, this);
			}
			this.source = source;
			if (source) {
				$add_0(source.outgoing, this);
			}
		}
		
	},
	
	setTarget: function(target) {
		if (!this.target || target.mapping.id != this.target.mapping.id) {
			if (this.target) {
				$remove_3(this.target.incoming, this);
			}
			this.target = target;
			if (target) {
				$add_0(target.incoming, this);
			}
		}
	},
	
	/*
	 * Returns true if source anchor is fixed or false otherwise
	 */
	isFixedSourceAnchor: function() {
		return !this.getSourceAnchor().isDefaultAnchor();
	},
	
	/*
	 * Returns true if target anchor is fixed or false otherwise
	 */
	isFixedTargetAnchor: function() {
		return !this.getTargetAnchor().isDefaultAnchor();
	},
	
	/*
	 * @param fix will ensure that the source anchor position is
	 * fixed and won't change during incremental layout
	 */
	setFixSourceAnchor: function(fix) {
		var anchor = this.getSourceAnchor();
		if (fix) {
			anchor.setRelativeReferencePoint(anchor.getAnchorRelativeLocation(this.getPointList().getFirstPoint(), anchor.getBox()));
		}
		else {
			anchor.setRelativeReferencePoint(null);
		}
	},
	
	/*
	 * @param fix will ensure that the target anchor position is
	 * fixed and won't change during incremental layout
	 */
	setFixTargetAnchor: function(fix) {
		var anchor = this.getTargetAnchor();
		if (fix) {
			anchor.setRelativeReferencePoint(anchor.getAnchorRelativeLocation(this.getPointList().getLastPoint(), anchor.getBox()));
		}
		else {
			anchor.setRelativeReferencePoint(null);
		}
	},
	
	
	setAvoidObstructions: function(avoid) {
		this.avoidObstructions = avoid;
	},
	
	isAvoidingObstructions: function() {
		return this.avoidObstructions;
	},
	
	setFeedback: function(feedback) {
		this.feedback = feedback;
		this.avoidObstructions = false;
	},
	
	isFeedback: function() {
		return this.feedback;
	},
	
	destroy: function() {
		this.inherited(arguments);
		
		if (this.connectionRouter) {
			this.connectionRouter.remove(this);
		}
		
		if (this.graph) {
			$removeEdge(this.graph, this);
		}
	},
	
	invalidate: function() {
		if (this.connectionRouter && this.endAnchor && this.startAnchor) {
			this.connectionRouter.invalidate(this);
		}
		this.getGraph()._addInvalidEdge(this);
	},
	
	setPointList: function (points){
		this.points = points;
	},
	
	getPointList: function () {
		return this.points;
	},

	getConnectionRouter: function() {
		if(this.connectionRouter == null) {
			// use same router as internalRouter setup in GraphEdge so that 
			// this.setRouterType doesn't think the first move is a new router 
			this.connectionRouter = this.getGraph().getConnectionRouter("Manual");			
		}
		return this.connectionRouter;
	},

	getRoutingConstraint: function (){
		var constraint = null;
		if (this.connectionRouter) {
			constraint = this.connectionRouter.getConstraint(this);
		}
		return constraint;
	},
	
	/*
	 * Sets router constraint for this edge based on a set of bendpoints
	 * If relativeToEnds is true, then the bendpoints are stored as 2 tuples that are relative
	 * to the end point locations.
	 */
	setRouterConstraint: function(bendPoints, relativeToEnds) {
		if (!this.endAnchor || !this.startAnchor) {
			return;
		}
		
		var router = this.getConnectionRouter();
		var constraint = null;
		
		if (relativeToEnds) {
			constraint = rmpw.diagrams.layout.client.routers.AbstractRouter.prototype.getRelativeBendpointConstraint(this, bendPoints);
		}
		else {
			constraint = rmpw.diagrams.layout.client.routers.AbstractRouter.prototype.calculateRouterConstraint(this, bendPoints);
		}

		router.setConstraint(this, constraint);
		router.invalidate(this);
	},

	getSourceAnchor: function (){
		return this.startAnchor;
	},
	
	setSourceAnchor: function(startAnchor) {
		this.startAnchor = startAnchor;
		this.setSource(startAnchor.getOwner());
	},
	
	getTargetAnchor: function (){
		return this.endAnchor;
	},
	
	setTargetAnchor: function(endAnchor) {
		this.endAnchor = endAnchor;
		this.setTarget(endAnchor.getOwner());
	},
	
	isOrthogonal: function() {
		if (this.connectionRouter) {
			return this.connectionRouter.isOrthogonal();
		}
		
		return false;
	},
	
	setRouterType: function(routerType) {
		// subclasses should implement
		var currentRouter = this.getGraph().getConnectionRouter(routerType);
		if (this.connectionRouter != currentRouter) {
			var oldRouter;
			oldRouter = this.connectionRouter;
			if (oldRouter != currentRouter && oldRouter != null) {
				this.connectionRouter.remove(this);
				this.connectionRouter = currentRouter;
				if (this.connectionRouter) this.connectionRouter.invalidate(this);
			}
			this.connectionRouter = currentRouter;
		}
	},
	
	/*
	 * Returns the type of router for this connection figure
	 */
	getRouterType: function() {
		if (this.connectionRouter) {
			return this.connectionRouter.getType();
		}
		
		if (this._routerType) {
			return this._routerType;
		}
	},
	
	/*
	 * Routes this edge based on it's constraint bendpoints
	 */
	route: function() {
		if (this.connectionRouter == null) {
			this.setRouterType(rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.OBLIQUE_ROUTING);
		}
		
		// make sure there are anchors on this connection before routing
		if (this.endAnchor && this.startAnchor) {
			this.connectionRouter.route(this);
		}
	}
});