/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * Copyright IBM Corp. 2010, 2013. All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("rmpw.diagrams.layout.client.figures.ContainerFigure");

dojo.require("rmpw.diagrams.render.graph.AbstractGraph");
dojo.require("rmpw.diagrams.layout.client.EdgeList");
dojo.require("rmpw.diagrams.layout.client.NodeList");
dojo.require("rmpw.diagrams.layout.client.Graph");
dojo.require("rmpw.diagrams.layout.client.routers.ObliqueRouter");
dojo.require("rmpw.diagrams.layout.client.routers.FanRouter");
dojo.require("rmpw.diagrams.layout.client.routers.RectilinearRouter");
dojo.require("rmpw.diagrams.layout.client.routers.TreeRouter");
dojo.require("rmpw.diagrams.layout.client.routers.ShortestPathConnectionRouter");

dojo.declare("rmpw.diagrams.layout.client.figures.ContainerFigure", rmpw.diagrams.render.graph.AbstractGraph, {
	
	obliqueRouter: null,
	rectilinearRouter: null,
	treeRouter: null,
	avoidObstructionsRouter: null,
	
	constructor: function() {
		var graph = $DirectedGraph(new DirectedGraph());
		graph.nodes = $NodeList(new NodeList());
		graph.edges = $EdgeList(new EdgeList());	
		dojo.mixin(this, graph);
	},
	
	addNode: function(node) {
		$add_0(this.nodes, node);
	},
	
	removeNode: function(node) {
		$removeNode(this, node);
	},
	
	destroy: function() {
		for (i = 0; i < this.nodes.size; ++i) {
			var node = $get_0(this.nodes, i);
			node.destroy();
		}
		
		for (i = 0; i < this.edges.size; ++i) {
			var edge = $get_0(this.edges, i);
			edge.destroy();
		}
		
		if (this.avoidObstructionsRouter) {
			this.avoidObstructionsRouter.destroy();
			this.avoidObstructionsRouter = null;
		}
		
		if (this.rectilinearRouter) {
			this.rectilinearRouter.destroy();
			this.rectilinearRouter = null;
		}
		
		if (this.obliqueRouter) {
			this.obliqueRouter.destroy();
			this.obliqueRouter = null;
		}
		
		if (this.treeRouter) {
			this.treeRouter.destroy();
			this.treeRouter = null;
		}
	},
	
	/*
	 * Gets the common routing that allows for obstruction avoidance
	 */
	getAvoidObstructionsRouter: function() {
		if (this.avoidObstructionsRouter == null) {
			this.avoidObstructionsRouter = new rmpw.diagrams.layout.client.routers.ShortestPathConnectionRouter(this);
		}
		return this.avoidObstructionsRouter;
	},
	
	/*
	 * Gets the connection router from the graph configurator
	 */
	getConnectionRouter: function(routerType) {
	    var connectionRouter;
	    switch( routerType )
	    {
	       case "Rectilinear": // legacy
	       case rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.RECTILINEAR_ROUTING:
	    	   if (this.rectilinearRouter == null) {
	    		   dojo.require("rmpw.diagrams.layout.client.routers.RectilinearRouter");
	    		   this.rectilinearRouter = new rmpw.diagrams.layout.client.routers.RectilinearRouter(this);
	    	   }
			   connectionRouter = this.rectilinearRouter;
	    	   break;
	    	
	       case "Manual": // legacy
	       case rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.OBLIQUE_ROUTING:
	    	   if (this.obliqueRouter == null) {
	    		   dojo.require("rmpw.diagrams.layout.client.routers.ObliqueRouter");
		    	   dojo.require("rmpw.diagrams.layout.client.routers.FanRouter");
		    	   this.obliqueRouter = new rmpw.diagrams.layout.client.routers.FanRouter(
									new rmpw.diagrams.layout.client.routers.ObliqueRouter(this));
	    	   }
	    	   connectionRouter = this.obliqueRouter;
	    	   break;
	    	
	       case "Tree": // legacy
	       case rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.TREE_ROUTING:
	    	   if (this.treeRouter == null) {
	    		   dojo.require("rmpw.diagrams.layout.client.routers.TreeRouter");
	    		   this.treeRouter = new rmpw.diagrams.layout.client.routers.TreeRouter(this);
	    	   }
			   connectionRouter = this.treeRouter;
	    	   break;
	    }
	    return connectionRouter;
	},
	
	_doRouteInvalidEdges: function() {
		if (this._invalidEdges) {
			for (var id in this._invalidEdges) {
				var edge = this._invalidEdges[id];
				edge.route();
			}
		}
	},
	
	toRawData: function () {
		var jsonNodes = [];
		var jsonEdges = [];
		var idMap = {};
		var nodes = this.nodes;
		var edges = this.edges;
		
		for (var i = 0, len = nodes.size; i < len; i++) {
			var node = $getNode(nodes, i);
			var id = node.data.id ? node.data.id : node.data;
			
			jsonNodes.push( { id: id, 
				/* bounds */ b: {x: Math.round(node.x), y: Math.round(node.y), w: Math.round(node.width), h: Math.round(node.height)} } );
			idMap[id] = { mapping: node.data, nativeNode: node };
		}
		for (var i=0; i<edges.size; i++) {
			var edge = $getEdge(edges, i);
			var modelPoints = [];
			for (var j=0; j<edge.points.size; j++) {
				var pt = edge.points.getPoint(j);
				modelPoints.push({x: Math.round(pt.x), y: Math.round(pt.y)});
			}
			var id = edge.data.id ? edge.data.id : edge.data;
			
			if (edge.source && edge.target) {
				jsonEdges.push( { id: id,  
						/* source ID */ sId: edge.source.data.id ? edge.source.data.id : edge.source.data, 
						/* target ID */ tId: edge.target.data.id ? edge.target.data.id : edge.target.data,
						pts: modelPoints});
				idMap[id] = { mapping: edge.data, nativeEdge: edge };
			}
		}
		
		return { jsonGraph: {nodes: jsonNodes, edges: jsonEdges}, idMap: idMap};
	}
});