/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * Copyright IBM Corp. 2010, 2011. All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("rmpw.diagrams.layout.client.figures.FixedPositionAnchor");

dojo.require("rmpw.diagrams.render.graph.AbstractEdgeAnchor");
dojo.require("rmpw.diagrams.render.util.Point");

/*
 * Anchor that allows for fixed position anchoring on a given shape.  This is useful for contained shapes
 * such as list compartments or list items where it is more valuable to anchor an edge to a fixed
 * location rather then dynamic.  This class also allows for multiple fixed anchor locations.
 */
dojo.declare("rmpw.diagrams.layout.client.figures.FixedPositionAnchor", rmpw.diagrams.render.graph.AbstractEdgeAnchor, {
	
	constructor: function(figure) {
		this._anchors = []; // { side, percentage, inset }
	},
	
	/*
	 * Adds a new fixed location to this anchor.  
	 * @param side one of NORTH, SOUTH, WEST, EAST (see definitions in Point.js
	 * @param percentage integer value from 0 - 100 indicating how far along the side the fixed point is
	 * @param inset integer value how for inset from the side the point is
	 */
	addFixedLocation: function(side, percentage, inset) {
		this._anchors.push( {side: side, percentage: percentage, inset: inset} );
	},

	getCopy: function() {
		var copyAnchor = new rmpw.diagrams.layout.client.figures.FixedPositionAnchor(this.owner);
		dojo.forEach( anchor._anchors, function(anchor) {
			copyAnchor._anchors.push({side: anchor.side, percentage: anchor.percentage, inset: anchor.inset});
		});
		return copyAnchor;
	},
	
	equals$: function (obj){
		if (this.inherited(arguments)) {
			if (obj._anchors && obj._anchors.length == this._anchors.length) {
				for (var i=0, lg=this._anchors.length; i<lg; i++) {
					if (this._anchors[i].side != obj._anchors[i].side ||
						this._anchors[i].percentage != obj._anchors[i].percentage ||
						this._anchors[i].inset != obj._anchors[i].inset) {
						return false;
					}
				}
				
				return true;
			}
		}
		
		return false;
	},
	
	/*
	 * Gets the anchor location based on the current size of the owner bounding box
	 */
	getAnchorLocation: function(anchor) {
		var rBox = this.getBox();
		
		var x, y;
		switch (anchor.side) {
			case NORTH:
				y = rBox.y;
				y += anchor.inset;
				x = rBox.x + (rBox.width * anchor.percentage / 100);
				break;
			case WEST:
				x = rBox.x;
				x += anchor.inset;
				y = rBox.y + (rBox.height * anchor.percentage / 100);
				break;
			case EAST:
				x = rBox.x + rBox.width - 1;
				x -= anchor.inset;
				y = rBox.y + (rBox.height * anchor.percentage / 100);
				break;
			default:
			case SOUTH:
				y = rBox.y + rBox.height - 1;
				y -= anchor.inset;
				x = rBox.x + (rBox.width * anchor.percentage / 100);
				break;
		}
		
		return new rmpw.diagrams.render.util.Point(x, y); 
	},
	
	getLocation: function(reference) {
		var closestPoint = null;
		var closestDistance = null;
		for (var i=0, lg=this._anchors.length; i<lg; i++) {
			var point = this.getAnchorLocation(this._anchors[i]);
			var currentDistance = point.getDistance(reference);
			
			if (closestDistance == null || currentDistance < closestDistance) {
				closestDistance = currentDistance;
				closestPoint = point;
			}
		}
		
		if (closestPoint) {
			return closestPoint;
		}
		
		return reference;
	}
});