/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.form.SimpleSelection");

dojo.require("dijit._Widget");

/*
 * Provide a simple selection native javascript control represented in a form of widget.
 *   
 */
dojo.declare("rmps.ui.form.SimpleSelection", dijit._Widget, {
	
	selectionsList: null,
	label: null,
	value: null,
	onClick: null,
	readOnly: null,
	
	buildRendering: function() {
		this.inherited(arguments);
		this.domNode = dojo.create("select", {type:this.type, name: this.name, value: this.value});
		
		if(this.label){
			this.domNode.appendChild(dojo.doc.createTextNode(this.label));			
		}
	},
	
	postCreate: function() {
		this.inherited(arguments);
		if(!this.readOnly){
			this.connect(this.domNode,"onkeyup", this._onChange);
        	this.connect(this.domNode,"onchange", this._onChange);			
		}
		
		this.addOption("", "");
		if(this.selectionsList){
			for(var index = 0, selection; selection = this.selectionsList[index]; index++) {
				this.addOption(selection.label, selection.id);
			}
		}
	},
	
	addOption: function(name, value) {
		var option = dojo.doc.createElement("option");
		if (value) {
			option.value = value;
		}
		option.appendChild(dojo.doc.createTextNode(name));
		this.domNode.appendChild(option);
		return option;
	},
	
	_setValueAttr: function(val){
		if(!val){
			this.domNode.value= "";
		}else if(dojo.isString(val)){
			var option= this.getOption(val);
			if(option){
				option.selected= true;
			}
			this.domNode.value= val;
		}
		this.value= this.domNode.value;
	},
	
	getOption: function(value){
		var options= this.domNode.options;
		
		for(var index = 0, option; option = options[index]; index++) {
			if(option.value === value){
				return option;				
			}
		}
	},
	
	_getValueAttr: function(){
		return this.domNode.options[this.domNode.selectedIndex].value;
	},
	
	_setReadOnlyAttr: function(val){
		if(val){
			this.readOnly= true;
			this.domNode.readOnly=true;
		}else{
			this.readOnly= false;
			this.domNode.readOnly=false;
		}
	},
	
	_getReadOnlyAttr: function(){
		return this.readOnly;
	},
	
	_onChange: function(value){
		this.set("value", value);
		this.onChange(this.value);
	},
	
	onChange: function(value){
		/*To be implemented by sub-classes */
	}
});

})();