/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.render.gfx.WrappingText");

dojo.require('dojox.gfx.shape');

/**
 * Text Shape that wraps.
 * 
 * @author John Swanke
 */
dojo.declare("rmpw.diagrams.render.gfx.WrappingText", dojox.gfx.Group, {
	
	fontStyle: null,
	
	_hasEllipse: false,
	
	_lastText: "",
	
	_actualLabelHeight: -1,
	
	_lastWrapWidth: -1,
	
	_lastWrapHeight: -1,
	
	_align: 'center',
	
	constructor: function(){
		this._listeners = [];
		this.shape = dojo.clone(rmpw.diagrams.render.gfx.WrappingText.defaultWrappingText);
	},
	
	setFont: function(font){
		this.fontStyle = font;
		for(var i = 0; i < this.children.length; i++){
			this.children[i].setFont(font);
		}
		return this;
	},
	
	getFont: function(){
		return this.fontStyle;
	},
	
	setFill: function(fill){
		this.inherited(arguments);
		for(var i = 0; i < this.children.length; i++) {
			this.children[i].setFill(fill);
		}
	},
	
	setCursor: function(cursor){
		this._cursor = cursor;

        if(this.children && !dojo.isOffscreen) {
            for(var i = 0, child; child = this.children[i]; i++) {
                child.rawNode.style.cursor = this._cursor;
            }
        }
	},
	
	getUnwrappedBoundingBox: function(){
		return this._getBoundingTextBox(this.shape.text, this.fontStyle);
    },
	
	getBoundingBox: function() {
    	 if (this._totalLabelBounds) {
    		 return dojo.clone(this._totalLabelBounds);
    	 } else {
  			var style = {
				fontFamily: this.fontStyle.family,
				fontSize: this.fontStyle.size,
				fontWeight: this.fontStyle.weight
   			};
   			var bnds = dojox.html.metrics.getTextBox("X", style);    		 
   			return {x:0, y:0, width:0, height:bnds.h};
    	 }
	},
	
	getTransformedBoundingBox: function(trueHeight, preserveSize) {
		var boundingBox = this.getBoundingBox(),
		  w, h;
		if (trueHeight && this._hasEllipse) {
			boundingBox.height += this._lineHeight * 3;
		}
		var shape = this;
		if (preserveSize) {
			w = boundingBox.width;
			h = boundingBox.height;
		}
		for(var node = shape; node !== undefined; node = node.parent) {
			this._transformBoundingBox(node.matrix, boundingBox);
		}
		if (preserveSize) {
			boundingBox.width = w;
			boundingBox.height = h;
		}
	    return boundingBox;
	},
	
    applyLayout: function(rect, align) {
		this._appliedLayout = true;
        this._align = align;
        this.shape.width = rect.width;
        this.shape.height = rect.height;
		return this._doWrap(rect.width, rect.height);
    },
	
	setShape: function(shape){
		if(this._noSetShape){ return this; }
		this.shape = dojo.mixin(dojo.clone(rmpw.diagrams.render.gfx.WrappingText.defaultWrappingText), shape);
		this._lastWrapWidth = -1;
		this.setText(this.shape.text ? this.shape.text : "");		
		return this;
	},

	setText: function(text) {
	   text = text.replace(/(\r\n|\r)/g, '\n');
	   text = text.replace( /<br\b[^>]*>/gi, '\n' );
	   this.hasNewLines = text.indexOf('\n') !== -1;
	   this.shape.text = text;
	   // don't bother creating text shapes until we have a bounds to work with
	   if (this._appliedLayout) {
		   this._doWrap(this.shape.width, this.shape.height);
	   }
	},

	getText: function() {
	   return this.shape.text;
	},

	getAlign: function() {
		switch(this._align) {
		case "start":
		case "left":
			return "left";
		case "end":
		case "right":
			return "right";
		}
		return "center";
	},	
	
	isWrapped: function() {
		return this._isWrapped;
	},	


    _doWrap: function(width, height) {
		var text = this.shape.text;
	
		if (text === this._lastText && width === this._lastWrapWidth
				&& height === this._lastWrapHeight
				&& this._align === this._lastWrapAlign
				&& this._lastFontFamily === this.fontStyle.family
				&& this._lastFontSize === this.fontStyle.size
				&& this._lastFontWeight === this.fontStyle.weight) {
			return;
		}
        this._lastText = text;
        this._lastWrapWidth = width;
        this._lastWrapHeight = height;
        this._lastWrapAlign = this._align;
		this._lastFontFamily = this.fontStyle.family;
		this._lastFontSize = this.fontStyle.size;
		this._lastFontWeight = this.fontStyle.weight;

		if(this.shape.text === "") {
            this._createLines([""]);
        }
        this._hasEllipse = false;
        var labelBounds = this.getUnwrappedBoundingBox();
        var lines = this._getLines(this.shape.text, labelBounds, width, height);
        this._createLines(lines, labelBounds, width);
        this._isWrapped = lines.length > 1;
        this._totalLabelBounds = {
            x : this._mostLeftX ? this._mostLeftX : 0,
            y : -labelBounds.h,
            width : this._largestLineWidth,
            height : this._actualLabelHeight
        };
    },

    _getLines: function(text, labelBounds, width, height) {
		// break text into paragraphs (paragraphs terminate in '\n' or '\r\n' or '<br>')
		var paragraphs = text.split('\n'),
		  s;
		if (width === -1) {
			return paragraphs;
		}
		
		// break each paragraph into lines
		var lines = [];
		var remainingShapeHeight = (height === -1) ? 4294967295 : height; // if -1, we got all the space in the world
		var outOfSpace = false;
		var lineHeight = labelBounds.h; 
		for(var pinx = 0; pinx < paragraphs.length && !outOfSpace; pinx++){
			var str = paragraphs[pinx];
			var len = str.length;
			// process empty line (\n\n\n)
			if (len === 0) {
				remainingShapeHeight -=lineHeight;
		        lines.push(" ");
		        continue;
			}
			var inx = 0;
			var isLastLine = false;
			var charsPerRow = Math.round(text.length * width / labelBounds.w);
			while (inx < len) {
				remainingShapeHeight -=lineHeight;
				outOfSpace = (remainingShapeHeight - lineHeight) < 0;
				
				// fine-tune charsPerRow if last line doesn't fit shape
				if (inx+charsPerRow >= len && !isLastLine) {
					charsPerRow = this._fitInWidth(str.substr(inx), this.fontStyle, width).length;
				}
				
				// should we wrap?
				if ((inx+charsPerRow) < len) {
					// get rough string
					s = str.substr(inx, charsPerRow);
					
					// break on a space if we can
			        var i = s.lastIndexOf(" ");
			        if (i > 0) {
                        s = s.substring(0, i);
			        } else {
                        // is rough string still too long for shape?
						s = this._fitInWidth(s, this.fontStyle, width);
			        }
					inx += s.length;
			        s = this._addEllipse(s, outOfSpace, this.fontStyle, width);
			        lines.push(s);
				} else {

                    s = str.substr(inx);
                    if(outOfSpace && pinx + 1 !== paragraphs.length) {
                        s = this._addEllipse(s, outOfSpace, this.fontStyle, width);
                    }
                    lines.push(s);
                    inx = len;
                }
                if(outOfSpace){
                   break;
                }
            }
        }

		return lines;		
    },
	
	_createLines: function(lines, labelBounds, width) {
        // remove all old text shapes
        if(this.children && this.children.length > 0) {
            this._disconnect();
            this.clear();
        }

        // get raw lineWidths
        this._lineWidths = []; 

		this._largestLineWidth = -1;
		for(var i = 0; i < lines.length; i++){
			var lineWidth = this._getBoundingTextBox(lines[i], this.fontStyle).w;
			if (lineWidth > this._largestLineWidth) {
				this._largestLineWidth = lineWidth;
			}
			this._lineWidths.push(lineWidth);
		}

        // create text shapes
        var y = this.shape.y;
        this._mostLeftX = undefined; 

        for(i = 0; i < lines.length; i++){
			var textObj = this._createLine(i, lines[i], y, width);
			if(textObj && labelBounds){
				y += labelBounds.h;
			}
		}
		this._lineHeight = labelBounds ? labelBounds.h : 0;
		this._actualLabelHeight = y - this.shape.y; 
	},
	
	_createLine: function(i, text, y, width) {
		var sh = this.shape;
		
		var textShape = dojo.clone(dojox.gfx.defaultText);
		for(var p in textShape) {
			if (p !== "type" && sh[p]) {
				textShape[p] = sh[p];
			}
		}
		textShape.text = text;
		
		// align this line
		var w = width === -1 ? this._largestLineWidth : width; 
		var lineWidth = (this._align !== "left") ? this._lineWidths[i] : w;
		var x = w - lineWidth;
		switch(this._align) {
		case "start":
		    x = 0; 
            break;
		case "left":
			x = 0; 
			break;
		case "end":
		    break;
		case "right":
			break;
		default:
	        x /= 2;
            break;
		case "center":
		    x /= 2;
            break;
		case "middle":
			x /= 2;
			break;
		}
		if (this._mostLeftX === undefined || x < this._mostLeftX) {
			this._mostLeftX = x;
		}
		
		this._noSetShape = true;
		var textObj = this.createText(textShape);
		textObj.applyTransform({ dx: x, dy: y });
		
		delete this._noSetShape;
		if(textObj){
			// copy font and fill:
			var s;
			s = this.getFont(); if(s) textObj.setFont(s);
			s = this.getFill(); if(s) textObj.setFill(s);
			
			if (!dojo.isOffscreen) {
				textObj.rawNode.style.cursor = this._cursor ? this._cursor : "default";
				
				// pass double-click on to the WrappingText shape to open direct edit
				if (this.dblclick) {
					this._listeners.push(textObj.connect("ondblclick", this, dojo.hitch( function(e) {
						this.dblclick();
						dojo.stopEvent(e);
					})));
				}
			}
		}	
		return textObj;
	},
	
	_fitInWidth: function(s, fontStyle, width) {
		var isTooBig = false;
		if (s.length>0) {
			do {
				isTooBig = this._getBoundingTextBox(s, fontStyle).w > width;
				if (isTooBig) {
		        	s = s.substring(0, s.length-2);
				}
			} while (isTooBig);
		}
	    return s;
	},	
	
    _addEllipse: function(s, outOfSpace, fontStyle, width) {
		s = dojo.trim(s);
	    if (outOfSpace) {
	    	var lineWidth = this._getBoundingTextBox(s, fontStyle).w;
	    	if (lineWidth > width) {
	        	s = s.substring(0, s.length-3);
	    	}
	    	this._hasEllipse = true;
	    	s +="...";
	    }
	    return s;
	},
	
	_getBoundingTextBox: function(text, fontStyle) {
		var style = null;
		if(fontStyle) {
			style = {
				fontFamily: fontStyle.family,
				fontSize: fontStyle.size,
				fontWeight: fontStyle.weight,
				whiteSpace: "nowrap"
			};
		}
		// dojox.html.metrics.getTextBox( ) doesn't like "<"
		// replace with character of equal size
		if (text.indexOf("<") !=-1) {
			text = text.replace(/</g,">");
		}
		return dojox.html.metrics.getTextBox(text, style);
	},

	_transformBoundingBox: function(transform, boundingBox) {
	    if(transform && boundingBox) {
	        boundingBox.x = boundingBox.x * transform.xx + transform.dx;
	        boundingBox.y = boundingBox.y * transform.yy + transform.dy;
	        boundingBox.width *= transform.xx;
	        boundingBox.height *= transform.yy;
	    }
	},
	
	_disconnect: function() {
		dojo.forEach(this.listeners, dojo.disconnect);
	}
	
});


/////////////////////////////// ADD createWrappingText() TO Surface/Group //////////////////////////////////////
rmpw.diagrams.render.gfx.WrappingText.defaultWrappingText = dojo.mixin(dojo.clone(dojox.gfx.defaultText),	{ type: 'wrappingText', width: -1, height: -1});
rmpw.diagrams.render.gfx.WrappingText.nodeType = dojox.gfx.Group.nodeType;

function createWrappingText() {	
	var shape = this.createObject(rmpw.diagrams.render.gfx.WrappingText, arguments[0]);
	// Need to do vml-specific createGroup tasks
	// see dojox.gfx.vml.js/createGroup()
	if (!dojox.gfx.svg && !dojo.isOffscreen) {
		// Call Creator._overrideSize
		var node = shape.rawNode;
		this._overrideSize(node);
		
		// Need to do vml-specific createGroup tasks
		// see dojox.gfx.vml.js/createGroup()
		var r = shape.rawNode.ownerDocument.createElement("v:rect");
		r.style.left = r.style.top = 0;
		r.style.width = shape.rawNode.style.width;
		r.style.height = shape.rawNode.style.height;
		r.filled = r.stroked = "f";
		shape.rawNode.appendChild(r);
		shape.bgNode = r;
	}
	return shape;
}

dojo.extend(dojox.gfx.Surface, {createWrappingText: createWrappingText});
dojo.extend(dojox.gfx.Group,   {createWrappingText: createWrappingText});
