/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.render.graph.AbstractEdge");		

dojo.require("rmpw.diagrams.render.util.PointList");
dojo.require("rmpw.diagrams.render.graph.BaseGraphConfigurator");
dojo.require("rmpw.diagrams.render.graph.AbstractElement");

/**
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.render.graph.AbstractEdge", rmpw.diagrams.render.graph.AbstractElement, {
	
	destroy: function() {
		this.inherited(arguments);
		
		this.getGraph()._removeInvalidEdge(this);
	},
	
	setPointList: function (points){
		// subclasses should implement
	},
	
	getPointList: function (){
		// subclasses should implement
		return new rmpw.diagrams.render.util.PointList();
	},
	
	/*
	 * Sets router constraint for this edge based on a set of bendpoints
	 * If relativeToEnds is true, then the bendpoints are stored as 2 tuples that are relative
	 * to the end point locations.
	 */
	setRouterConstraint: function(bendPoints, relativeToEnds) {
		// do nothing, implemented by concrete graph
	},

	getSourceAnchor: function (){
		// subclasses should implement
		return null;
	},
	
	setSourceAnchor: function(startAnchor) {
		// subclasses should implement
	},
	
	getTargetAnchor: function (){
		// subclasses should implement
		return null;
	},
	
	setTargetAnchor: function(endAnchor) {
		// subclasses should implement
	},
	
	setSource: function(source) {
		// subclasses should implement
	},
	
	setTarget: function(target) {
		// subclasses should implement
	},
	
	isOrthogonal: function() {
		// subclasses should implement
		return false;
	},
	
	/*
	 * Returns the type of router for this connection figure
	 */
	getRouterType: function() {
		return rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.IDENTITY_ROUTING;
	},
	
	setRouterType: function(routerType) {
		// subclasses should implement
	},
	
	setFeedback: function(feedback) {
		// subclasses should implement
	},
	
	isFeedback: function() {
		// subclasses should implement
	},
	
	getID: function() {
		if (this.isFeedback()) {
			return "feedback";
		}
		
		return this.inherited(arguments);
	},
	
	setAvoidObstructions: function(avoid) {
		// subclasses should implement
	},
	
	getBounds: function() {
		return this.getPointList().getBounds();
	},
	
	isAvoidingObstructions: function() {
		// subclasses should implement
	},
	
	/*
	 * Returns true if source anchor is fixed or false otherwise
	 */
	isFixedSourceAnchor: function() {
		return false;
	},
	
	/*
	 * Returns true if target anchor is fixed or false otherwise
	 */
	isFixedTargetAnchor: function() {
		return false;
	},
	
	/*
	 * @param fix will ensure that the source anchor position is
	 * fixed and won't change during incremental layout
	 */
	setFixSourceAnchor: function(fix) {
	},
	
	/*
	 * @param fix will ensure that the target anchor position is
	 * fixed and won't change during incremental layout
	 */
	setFixTargetAnchor: function(fix) {
	},
	
	/*
	 * @param fix will ensure that the bendpoints position is
	 * fixed and won't change during incremental layout
	 */
	setFixBendpoints: function(fix) {
	}
});