/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.render.graph.AbstractGraph");

dojo.require("rmpw.diagrams.render.graph.AbstractElement");

dojo.declare("rmpw.diagrams.render.graph.AbstractGraph", rmpw.diagrams.render.graph.AbstractElement, {
	
	constructor: function(mapping) {
		this.mapping = mapping;
	},
	
	destroy: function() {
	},
	
	/*
	 * Removes an edge from the invalid list (usually on destroy)
	 */
	_removeInvalidEdge: function(edge) {
		if (this._invalidEdges) {
			delete this._invalidEdges[edge.mapping ? edge.mapping.id : "null"];
		}
	},
	
	/*
	 * Method that will add an invalid edge to be routed
	 */
	_addInvalidEdge: function(edge) {
		if (!this._invalidEdges) {
			this._invalidEdges = {};
		}
		
		this._invalidEdges[edge.mapping ? edge.mapping.id : "null"] = edge;
	},
	
	/*
	 * Pre-hook to allow layout to process invalid edges before
	 * calling layout algorithm
	 */
	_preRouteInvalidEdges: function() {
		// do nothing by default
	},
	
	/*
	 * Main entry point for calling the layout algorithm which
	 * will route the edges
	 */
	_doRouteInvalidEdges: function() {
		// do nothing by default
	},
	
	/*
	 * Hook for re-adjusting the edge routing after the node positions have been moved
	 */
	routeInvalidEdges: function() {
		this._preRouteInvalidEdges();
		
		this._doRouteInvalidEdges();
		
		this._postRouteInvalidEdges();
	},
	
	/*
	 * Post-hook to allow clean-up of invalid edges after
	 * calling layout algorithm
	 */
	_postRouteInvalidEdges: function() {
		if (this._invalidEdges) {
			delete this._invalidEdges;
		}
	},
	
	getProxyGraph: function() {
		var rawData = this.toRawData();
		this.resolveRawData(rawData);
		return rawData.jsonGraph;
	},
	
	/*
	 * resolves identifiers to actual objects in the raw data graph
	 */
	resolveRawData: function( rawData ) {
		var nodeMap = {}, i, node, edge, mapData;
		for (i=0, node; node=rawData.jsonGraph.nodes[i]; i++) {
			mapData = rawData.idMap[node.id];
			node.mapping = mapData.mapping;
			node.nativeNode = mapData.nativeNode;
			nodeMap[node.id] = node;
		}
		for (i=0, edge; edge=rawData.jsonGraph.edges[i]; i++) {
			mapData = rawData.idMap[edge.id];
			edge.mapping = mapData.mapping;
			edge.nativeEdge = mapData.nativeEdge;
			edge.source = nodeMap[edge.sId];
			edge.target = nodeMap[edge.tId];
		}
	},
	
	toRawData: function () {
		// subclass should implement
		return { jsonGraph: {nodes: [], edges: []}, idMap: {}};
	}
});