/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.render.graph.AnimatedNode");		

dojo.require("rmpw.diagrams.render.graph.AnimatedElement");
dojo.require("rmpw.diagrams.render.util.Rectangle");

/**
 * Stores state required for animating a figure
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.render.graph.AnimatedNode", rmpw.diagrams.render.graph.AnimatedElement, {
	constructor: function(nodePolicy) {
		this.nodePolicy = nodePolicy;
		this.initialBounds = nodePolicy.getDojoBounds();
	},
	
	/* 
	 * method returns the initial state information of the object before the animation occurs
	 */
	getInitialState: function() {
		return this.initialBounds;
	},
	
	/* 
	 * method returns the final state information of the object before the animation occurs
	 */
	getFinalState: function() {
		return this.finalBounds;
	},
	
	startingAnimation: function() {
		this.finalBounds = this.nodePolicy.getBounds();
	},
	
	/*
	 * runs a step in the animation
	 */
	runAnimationStep: function(progress) {
		var rect1 = this.getInitialState();
		var rect2 = this.getFinalState();
		var ssergorp = 1 - progress;
		
		this.nodePolicy.setBounds(new rmpw.diagrams.render.util.Rectangle(
			Math.round(progress * rect2.x + ssergorp * rect1.x),
			Math.round(progress * rect2.y + ssergorp * rect1.y),
			Math.round(progress * rect2.width + ssergorp * rect1.width),
			Math.round(progress * rect2.height + ssergorp * rect1.height)
		));
	}

});