/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.handles.ConnectionCreationHandle");

dojo.require("rmpw.diagrams.render.util.Workarounds");
dojo.require("rmpw.diagrams.edit.handles.MoveableEndHandle");

dojo.requireLocalization("rmpw.diagrams.edit", "ConnectionCreationHandleMessages");
var _messages = dojo.i18n.getLocalization("rmpw.diagrams.edit", "ConnectionCreationHandleMessages");

/**
 * Class for displaying a connection creation handle for creating connections
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.handles.ConnectionCreationHandle", rmpw.diagrams.edit.handles.MoveableEndHandle, {

	constructor: function(args) {
		this.source = false; // creation handle is always the target
	},
	
	createHandleShape: function(args) {
		
	    var bounds,
	        iconsource,
	        alt,
            icon = { width:20, height:10, src:null },
            top = {x : 0, y : 0};
            
	    if (args.editController.renderMapping.policies.graphElement) {
			bounds = args.editController.renderMapping.policies.graphElement.getBounds();
			args.editController.renderMapping.policies.graphElement.translateToAbsolute(bounds);
	    }
	    else {
			bounds = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(args.target);
	    }		

        // depending on orientation argument, this connection handle will show one of 4 arrow icons, 
		// and will be attached to one of 4 sides of bounds: east, west, north or south.  
		var dm = net.jazz.ajax._contextRoot;
		if (args.orientation === 1) {
			// east
			icon.src = dm+"/web/rmpw.diagrams.edit/handles/connectionHandleE.png";
			top.x = bounds.x + bounds.width + 1;
			top.y = bounds.y + (bounds.height / 2) - (icon.height / 2);
			alt = _messages.connectionHandleE;
		} else if (args.orientation === 2) {
			// north
			icon.src = dm+"/web/rmpw.diagrams.edit/handles/connectionHandleN.png";
			icon.width = 10;
			icon.height = 20;
			top.x = bounds.x + (bounds.width / 2) - (icon.width / 2);
			top.y = bounds.y - icon.height - 1;
			alt = _messages.connectionHandleN;
		} else if (args.orientation === 3) {
			// west
			icon.src = dm+"/web/rmpw.diagrams.edit/handles/connectionHandleW.png";
			top.x = bounds.x - icon.width - 1;
			top.y = bounds.y + (bounds.height / 2) - (icon.height / 2);
			alt = _messages.connectionHandleW;		
		} else {
			// south
			icon.src = dm+"/web/rmpw.diagrams.edit/handles/connectionHandleS.png";
			icon.width = 10;
			icon.height = 20;
			top.x = bounds.x + (bounds.width / 2) - (icon.width / 2);
			top.y = bounds.y + bounds.height + 1;
			alt = _messages.connectionHandleS;
		}

		this.handleShape = args.owner.createImage(icon);
	    if(dojox.gfx.svg) {
			// workaround for bug fixed here: http://trac.dojotoolkit.org/changeset/18536
			this.handleShape.rawNode.setAttribute("fill", null);
			if(alt){
                this.handleShape.rawNode.setAttribute("title", alt);
            }
	    }
	    
		this.handleShape.setTransform({dx: top.x, dy: top.y});
		
		this.pos = {x: top.x + (icon.width / 2), y: top.y + (icon.height / 2)};
	},
	
	onFirstMove: function(/* dojox.gfx.Mover */ mover) {
		if (!this.pos) {
			return;
		}
		this.inherited(arguments);
		this.startPos = {x: this.pos.x, y: this.pos.y};
	},
	
	onMoveStop: function(/* dojox.gfx.Mover */ mover) {
		this.inherited(arguments);
		this.pos = this.startPos;
	}
});