/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.handles.MoveableHandle");

dojo.require("rmpw.diagrams.edit.handles.Handle");
dojo.require("rmpw.diagrams.edit.tools.FocusableBase");
dojo.require("rmpw.diagrams.render.util.Events");
dojo.require("rmpw.common.policy.Host");
dojo.require("rmpw.diagrams.edit.tools.HandleScope");
dojo.require("rmpw.diagrams.edit.tools.figures.RectangularHighlight");
dojo.require("rmpw.diagrams.edit.tools.MoveableBase");

/**
 * Class to handle movement of bendpoints
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.handles.MoveableHandle", [rmpw.diagrams.edit.handles.Handle, rmpw.common.policy.Host], {
	
	constructor: function(args) {
		this.editController = args.editController;
		this.renderMapping = args.editController ? args.editController.renderMapping : null;
		this.root = args.editController ? args.editController.root : null;
		this._index = args.index;
		this.moveable = args.moveable;	
		this.affordance = args.affordance;
		this.handleShape.id = args.id;
		
		this.createPolicies();
		this.initializeListeners();
	},
	
	destroy: function() {
		this.inherited(arguments);
		
		dojo.forEach(this.listeners, dojo.disconnect);
		
		if (this.editController) {
			this.editController.policies.handleScope.unregisterHandle(this);
		}
		
		this.handleShape.removeShape();
	},
	
	getIndex: function() { return this._index ? this._index : -1; },
	setIndex: function(index) { this._index = index; },
	
	createPolicies: function() {
		this.inherited(arguments);
		
		var focusEffect = this.createPolicy(rmpw.diagrams.edit.tools.FocusableBase);
		focusEffect.setTargetShape(this.handleShape);
		focusEffect.setFeedbackFigureType(rmpw.diagrams.edit.tools.figures.RectangularHighlight);
		
		var moveable = this.createPolicy(this.moveable ? this.moveable : rmpw.diagrams.edit.tools.MoveableBase);
		moveable.setTargetShape(this.handleShape);
		moveable.setRenderMapping(this.renderMapping);
		moveable.setEnableFeedback(false);
		
		// ensure handle scope policy is installed on parent
		if (this.editController) {
			if (!this.editController.policies.handleScope) {
				this.editController.createPolicy(rmpw.diagrams.edit.tools.HandleScope);
			}
			this.editController.policies.handleScope.registerHandle(this);
		}
		
		// reference handle scope locally
		this.policies.handleScope = this.editController.policies.handleScope;
	},

	initializeListeners: function() {	
		// TODO (mg): should be some way to implement this cleanly via inheriting from Moveable instead
		this.listeners = [
			dojo.connect(this.policies.moveable, "onFirstMove", this, "onFirstMove"),
			dojo.connect(this.policies.moveable, "onMove", this, "onMove"),
			dojo.connect(this.policies.moveable, "onMoveStop", this, "onMoveStop"),
			dojo.connect(this.policies.moveable, "onCancel", this, "onCancel")
		];
	},
	
	onFirstMove: function(/* dojox.gfx.Mover */ mover) {
		this.policies.focusable.deactivate();
		if(dojox.gfx.svg) {
			dojo.attr(this.handleShape.rawNode, "cursor", "move");
		}
	},
	
	onMove: function(/* dojox.gfx.Mover */ mover, /* Object */ shift)  {
		if (shift) {
			this.pos = {x: this.pos.x + shift.dx, y: this.pos.y + shift.dy};
		}
	},
	
	onMoveStop: function(/* dojox.gfx.Mover */ mover) {
		
	},
	
	onCancel: function() {
		this.policies.focusable.activate();
		if(dojox.gfx.svg) {
			dojo.attr(this.handleShape.rawNode, "cursor", "default");
		}
	}
	
});