/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("rmps.viewlets.internal.ContextPreferenceProvider");
	
dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");

dojo.require("com.ibm.team.vvc.shared.web.ui.picker.WorkspacePickerDialog");
dojo.require("rmps.core.util.ContextUtils");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.viewlets.internal", "ContextPreferenceProviderMessages");
var messages = dojo.i18n.getLocalization("rmps.viewlets.internal", "ContextPreferenceProviderMessages");

var PREF_CONTEXT = "contextURI";
var PREF_PROJECT = "_scopeItem";
var ContextUtils = rmps.core.util.ContextUtils;

dojo.declare("rmps.viewlets.internal.ContextPreferenceProvider", com.ibm.team.dashboard.web.ui.CustomPreferenceProvider, {
	
	_viewlet: null,
	_contextUri: null,
	_contextCache: null,

	constructor: function(viewlet) {
		this._viewlet = viewlet;
		this._contextCache = {};
		this.cancel();
	},
	
	_cacheContext: function(contextURI) {
		if (!contextURI) {
			var def = new dojo.Deferred();
			def.callback(null);
			this.contextCache = {
				uri: null,
				context: null
			};
			return def;
		}
		var def = ContextUtils.getContextDeferred(contextURI, this._getProjectId());
		def.addCallback(this, function(context) {
			this._contextCache = {
				uri: contextURI,
				context: context
			};
		});
		def.addErrback(this, function(err) {
			this.contextCache = {
				uri: contextURI,
				context: null
			};
		});
		return def;
	},

	_getProjectId: function() {
		// Check the preference first
		var id = this._viewlet._site.context.getPreferences()[PREF_PROJECT];

		// If no preference then try context
		return id ? id : jazz.app.context.get().itemId;
	},

	getLabel: function(value) {
		var message;
		if (!value) {
			var projectId = jazz.app.context.get().itemId;
			if (projectId){
				// if we are in a project use current context
				message = messages.currentContextSelected;
			} else {
				// if we are in a user dashboard use default workspace
				message = messages.defaultContextSelected;
			}
			return "<input type='text' readonly='readonly' value='" + message + "'/>";
		}
		
		if (this._contextCache.uri === value) {
			if (!this._contextCache.context) {
				return "";
			}
			message = this._contextCache.context.name;
		} else {
			this._cacheContext(value);
			// Will be called again until a value is returned.
			return null;			
		}
		
		return "<input type='text' readonly='readonly' value='" + message + "'/>";
	},

	getLoadingLabel: function() {
		return messages.loading;
	},

	createChooser: function(onOk, settings, viewlet) {
		if (!settings.scopeItemId) {
			alert(messages.noProjectSelected);
			return;
		}
		var self = this;
		ContextUtils.getProjectSpace(settings.scopeItemId).addCallback(this, function(spaceUri) {
			var picker = new com.ibm.team.vvc.shared.web.ui.picker.WorkspacePickerDialog({
				spaceUri : spaceUri,
			});			
			picker.onOk = dojo.hitch(this, function(configuration) {
				var contextURI = ContextUtils.getContextUriFromVvcContextUri(configuration.uri, settings.scopeItemId);	
				this._contextUri = contextURI;
				this._viewlet.setPreference(PREF_CONTEXT, contextURI);
				onOk(contextURI, PREF_CONTEXT);
			});
		});
	},

	cancel: function() {
		this._contextUri = this._viewlet.getViewletState().get(PREF_CONTEXT);
	}
	
});

}());