/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.viewlets.internal.PagedTableViewlet");

dojo.require("com.ibm.team.dashboard.web.ui.PagedTableViewlet");

dojo.require("jazz.app.context");
dojo.require("jazz.util.ResizeNotifier");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("rmps.core.RootServices");
dojo.require("rmps.core.util.ContextUtils");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmps.viewlets.internal.ContextPreferenceProvider");
dojo.require("jazz.util.i18n");
dojo.requireLocalization("rmps.viewlets.internal", "ViewletMessages");

var PREF_PAGE_SIZE = "pageSize";
var PREF_PROJECT = "_scopeItem";
var PREF_CONTEXT_URI = "contextURI";
var PROJECT_PATH_AREA_PATH= rmps.core.RootServices.getUri(rmps.core.Namespaces.JP_0_6, "projectAreas");
var I18nUtil = jazz.util.i18n;
var IRmpsConstants = rmps.core.util.IRmpsConstants;
var ContextUtils = rmps.core.util.ContextUtils;
var messages = dojo.i18n.getLocalization("rmps.viewlets.internal", "ViewletMessages");

dojo.declare("rmps.viewlets.internal.PagedTableViewlet", com.ibm.team.dashboard.web.ui.PagedTableViewlet, {

	contentClass: "rmps-viewlets-pagedtable",

	isClientPaged: false,

	// private properties
	_totalResults: -1,
	_pageToken: null,
	_updating: false,
	_currentWidgets: null,
	_resizeHandle: null,
	_owningPage: null,
	_getProjectsDef: null,
	_projects: null,
	_errorMsgNode: null,
	_getContextURIDef: null,
	_contextURI: null,
	_contextProvider: null,

	postCreate: function() {
		this.refreshContext();
		
		var wb = net.jazz.ajax.ui.PlatformUI.getWorkbench();
		if (wb) {
			this._owningPage = wb.currentPageId;
		}
		this._resizeHandle = jazz.util.ResizeNotifier.register({context: this, callback: this._handleResize});
		this.inherited(arguments);
	},

	init: function() {
		this.inherited(arguments);
		dojo.addClass(this._messageDiv, "message");
		if (this.getSite().isRemote() && (this.getSite().getTitle().indexOf(messages.isRemote) == -1)) {
			this.getSite().setTitle(this.getSite().getTitle() + messages.isRemote);
		}

	},

	destroy: function() {
		jazz.util.ResizeNotifier.unregister(this._resizeHandle);
		this.cleanup();
		this.inherited(arguments);
	},

	cleanup: function() {
		if (this._currentWidgets) {
			for (var i = 0, len = this._currentWidgets.length; i < len; ++i) {
				 this._currentWidgets[i].destroyRecursive();
			}
			delete this._currentWidgets;
		}
		if (this.pagedTable){
			this.pagedTable.setResultCount(-1);
			this.pagedTable._updateFooter();
		}
		this.setCurrentPage(1);		
		this._totalResults = -1;
		this._pageToken = null;
	},

	getItemsPerPage: function() {
		var size = this.getPreference(PREF_PAGE_SIZE);
		return size < 0 ? 10 : size;
	},

	getResultCount: function() {
		return this._totalResults;
	},

	refresh: function() {
		this.cleanup();
		
		// Re-retrieve context
		this.refreshContext();
		
		this.inherited(arguments);
	},
	
	refreshContext: function() {
		this._getContextURIDef = new dojo.Deferred(); 
		var def = this._getContextUri();
		def.addCallback(this, function(contextURI) {
			this._contextURI = contextURI;
			this._getContextURIDef.callback(contextURI);
		});
		def.addErrback(this, function() {
			// Happens for example when there is no context on a newly created project
			this._contextURI = null;
			this._getContextURIDef.callback(null);
		});
	},

	settingsChanged: function(oldSettings, newSettings) {
		if (newSettings != oldSettings) {
			this.refresh();
		} else {
			this.update();
		}
	},

	reset: function(refresh) {
		if (refresh) {
			this._totalResults = -1;
			this.pagedTable.refresh(refresh);
			this.updateSecondaryTitle();
		}
		this.inherited(arguments);
	},

	fetch: function (/* ServiceResponseHandler */ srh, /* int */ startIndex, /* int */ count, /* boolean */ entries, /* boolean */ details) {
		if (count <= 0) {
			srh.doCallback();
			return;
		}
		this._totalResults = -1;
		
		var projectId = this._getProjectId();
		this._errorMsgNode = null;
		if (!projectId) {
			this._noProjectSpecified(srh);			
			return;
		}
		
		this.fetchInternal(srh, startIndex, count, entries, details, projectId);
	},

	fetchInternal: function (srh, startIndex, count, entries, details, projectId) {
		if (this.preFetch) {
			// Pre fetch has the possibility to abort and use the ServiceResponseHandler
			if (this.preFetch(srh)) {
				return;
			}
		}

		var data = this.fetchData(startIndex, count, entries, details, projectId);

		if (data instanceof dojo.Deferred) {
			data.addCallback(this, function(result) {
				this._handleResult(srh, startIndex, count, entries, details, result);
			});

			data.addErrback(this, function(error) {
				this._totalResults = -1;
				this.updateSecondaryTitle();
				srh.doErrorCallback(error);
			});
		}
	},

	_handleResult: function(srh, startIndex, count, entries, details, result) {
		var dataStructure = null;
		if (result) {
			this._totalResults = this.parseTotalResults(result);
			this._pageToken = this.parsePageToken(result);
			dataStructure = {
				"entries": entries ? this.parseEntries(result) : null,
				"details": details ? this.parseDetails(result) : null
			};
		} else {
			this._totalResults = 0;
			this._pageToken = null;
			dataStructure = {
				"entries": null,
				"details": null
			};
		}
		this.updateSecondaryTitle();
		srh.doCallback(dataStructure);
	},

	preRender: function(entries) {
		this._updating = true;
		if (entries.length > 0) {
			this._currentWidgets = [];
			setTimeout(dojo.hitch(this, this._resizeContents), 0);
		}
	},

	_handleResize: function() {
		var wb = net.jazz.ajax.ui.PlatformUI.getWorkbench();
		if (wb && wb.currentPageId == this._owningPage) {
			this._resizeContents();
		}
	},

	_resizeContents: function() {
		if (this._currentWidgets) {
			var size = {};
			size.w = dojo.contentBox(this.domNode).w;
			for (var i = 0, len = this._currentWidgets.length; i < len; ++i) {
				var widget = this._currentWidgets[i];
				if (!widget._started) {
					widget.startup();
				}
				if (widget.resize) {
					widget.resize(size);
				}
			}
		}
	},

	renderEntry: function(/* object */ entryData, /* object */ detailsData, /* boolean */ zoomed) {
		if (!zoomed) {
			var widget = this.createEntryWidget(entryData);
			this._currentWidgets.push(widget);

			this.styleEntry(widget, this._updating);
			this._updating = false;

			var td = dojo.doc.createElement("td");
			td.style.padding = "0px";
			td.appendChild(widget.domNode);
			return [td];
		}
		return [];
	},

	renderDetails: function(/* object */ detailsData) {
		var widget = this.createDetailsWidget(detailsData);
		this._currentWidgets.push(widget);

		this.styleEntry(widget, this._updating);
		this._updating = false;

		var td = dojo.doc.createElement("td");
		td.style.padding = "0px";
		td.appendChild(widget.domNode);
		return [td];
	},

	styleEntry: function(widget, isFirst) {
		if (isFirst) {
			dojo.removeClass(widget.domNode, "entry-separator");
		} else {
			dojo.addClass(widget.domNode, "entry-separator");
		}
	},

	styleDetails: function(isLast) {
		this.styleEntry(isLast);
	},

	updateSecondaryTitle: function() {
		this.updateTotalPageFooter();
		this.getSite().setSecondaryTitle(this.getSecondaryTitle());
	},

	getSecondaryTitle: function() {
		var title = '';
		if (this.getResultCount() >= 0) {
			title += '(' + this.getResultCount() + ')';
		}
		return title;
	},

	createEntryWidget: function(data) {
		// to be implemented by sub class
	},

	createDetailsWidget: function(data) {
		// to be implemented by sub class
	},

	fetchData: function(startIndex, count, entries, details) {
		// to be implemented by clients
		// return dojo.Deferred
	},

	parseTotalResults: function(result) {
		// may be overridden by clients
		return result.totalResults;
	},

	parsePageToken: function(result) {
		// may be overridden by clients
		return result.pageToken;
	},
	
	parseEntries: function(response) {
		// may be overridden by clients
		// return []
		return response;
	},

	parseDetails: function(response) {
		// may be overridden by clients
		// return []
		return this.parseEntries(response);
	},

	getProjectPrefix: function(){
		var context= jazz.app.context.get();
		var name;
		var scope= this.getScopeItem();
		if(scope){
			name= this.getScopeItem().name;
		}
		if(!name&& context){
			if(context){
				name= context.name;
			}
		}
		if(name){
			return this._webURL+ "/web/projects/"+ encodeURIComponent(name);
		}
		return null;
	},

	_getProjectId: function() {
		// Check the preference first
		var id = this._site.context.getPreferences()[PREF_PROJECT];

		// If no preference then try context
		return id ? id : jazz.app.context.get().itemId;
	},

	_getProjectUri: function() {
		var id = this._getProjectId();
		if (!id) {
			return null;
		}
		return PROJECT_PATH_AREA_PATH + "/" + id;
	},
	
	_getProjectUriFromId: function(id) {
		if (!id) {
			return null;
		}
		return PROJECT_PATH_AREA_PATH + "/" + id;
	},

	_getContextUri: function() {
		var def = new dojo.Deferred();
		var contextURI = this.getPreference(PREF_CONTEXT_URI);
		if (contextURI) {
			def.callback(contextURI);
		} else {
			// we set to the current context, so use the current project
			var projectId = jazz.app.context.get().itemId;
			
			if (projectId){
				// if there is a current project overwrite the project settings to this project
				this._site.context.getPreferences()[PREF_PROJECT] = projectId;
			} else {
				// if there is no current project (user dashboard) then use getProjectId
				projectId = this._getProjectId();				
			}
			// First, we have to check if space is configured for the selected project
			if (projectId){
				dojo.when(ContextUtils.getProjectSpace(projectId),
					  dojo.hitch(this, function(spaceUri) {
						  if (spaceUri) {
							  ContextUtils.findCurrentContextUriAndExecute(def.callback, projectId, def.errback);
						  } else {
							  def.callback(spaceUri);
						  }
					  })
				);
			}
		}
		
		return def;
	},

	_noProjectSpecified: function(srh) {
		if (!this._noProjectMsg) {
			// this part always shows
			this._noProjectMsg = document.createElement("span");
			this._noProjectMsg.appendChild(document.createTextNode(messages.noProjects));

			// this part only shows when dashboard in edit mode
			this._noProjectExtraMsg = document.createElement("span");
			var links = I18nUtil.addMessage(this._noProjectExtraMsg, messages.editSettings);
			this.connect(links[0], "onclick", dojo.hitch(this, function(e) {
				e.preventDefault();
				this.getSite().toggleSettings();
			}));
			this._noProjectExtraMsg.className = "editModeText";
			this._noProjectMsg.appendChild(this._noProjectExtraMsg);
			if (!this.getSite().isDashboardEditing()) {
				this._noProjectExtraMsg.style.display = "none";
			}
		}

		this._errorMsgNode = this._noProjectMsg;
		this._totalResults = 0;
		this._pageToken = null;		
		this.updateSecondaryTitle();

		var dataStructure = {
			"entries": null,
			"details": null
		};
		srh.doCallback(dataStructure);
	},

	getNoResultsMessage: function() {
		if (this._errorMsgNode) {
			return this._errorMsgNode;
		}
		return document.createTextNode(messages.pagedTableNoResults);
	},
	
	getPreferenceProvider: function(id) {
		if (id === PREF_CONTEXT_URI) {
			if (!this._contextProvider) {
				this._contextProvider = new rmps.viewlets.internal.ContextPreferenceProvider(this);
			}
			return this._contextProvider;
		}
	},
	
	settingsCanceled: function() {
		if (this._contextProvider) {
			this._contextProvider.cancel();
		}
	}

});

}());