/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.text.internal.RichTextChangeMixin");

dojo.require("rmpw.forms.ui.ChangeMixin");
dojo.require("rmpw.text.internal.widgets.RichTextPanel");
dojo.require("rmpw.text.internal.widgets.RichTextWidget");
dojo.require("rmps.core.commenting.CommentClient");
dojo.require("rmps.core.util.DateUtils");
dojo.require("rmpw.text.internal.PrintConfirmDialog");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.app.ui.internal.pages.design", "DesignMessages");
var designMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages.design", "DesignMessages");

dojo.requireLocalization("rmpw.text.internal.widgets", "Messages");
var messages = dojo.i18n.getLocalization("rmpw.text.internal.widgets", "Messages");


/**
 * Parent form for rich text document form. Controls UI toggle between preview mode (panel) and edit mode (ckeditor) 
 * depending on the current locking state of the resource. Also handles printing preview of the document, 
 * with or without comments.
 *
 *  @author lgrahek
 *
 */
dojo.declare("rmpw.text.internal.RichTextChangeMixin", rmpw.forms.ui.ChangeMixin, {
	
	panelWidget: null,
	lockButton: null, // lock button from the toolbar

	postCreate: function(){	
		// find out two rich text widgets as we will need the access to them later to control 
		// their visibility depending on the resource lock state
		var control;
		for (var key in this.template) {
			control = this.template[key];			
			if (control instanceof rmpw.text.internal.widgets.RichTextPanel) {
				this.panelWidget = control;
			} else if (control instanceof rmpw.text.internal.widgets.RichTextWidget) {
				this.editorWidget = control;
			}			
		}
		
		this.editorWidget.isInRichDocument = true;
		
		this.inherited(arguments);
	},
		
	updateToolbar: function(toolbar) {
		// add print button
		toolbar.addButton(this, 
			dojo.hitch(this, this.print), 
			{
				label: messages.viewPrintablePage,
				imgSrc: dojo.moduleUrl("rmpw.text", "internal/icons/print.gif"),
				index: 0
			}
		);
					
		//identify the lock button and its initial state
		var lockLabel;
		var item;
		for (var i = 0, item; i < toolbar.items.length && !this.lockButton; i++) {
			item = toolbar.items[i];
			lockLabel  = item.args ? item.args.label : null;
			if (lockLabel === designMessages.UnLock || lockLabel === designMessages.Lock) {
				this.lockButton = item;
			}
		}
		if (!this.lockButton || 
				(this._resource.documentObject.rdf_about != "http://new" && this.lockButton.args.label === designMessages.Lock)) {
			// there is either no lock button (read only form) or the resource exists and is unlocked,
			// so the initial control to show is panel (preview)
			this.editorWidget.setVisible(false);				
		} else {
			// creation mode or viewing your own locked resource that can be edited
			this.panelWidget.setVisible(false);		
		}
		
		// listen to lock and unlock events as we need to toggle the ui between the two rich text control
		var resourceState = this._resource.page.editor.editorInput.store.resourceState;
		this.connect(resourceState, "onLock", this._onLock);
		this.connect(resourceState, "onUnLock", this._onUnLock);		
	},
	
	_onLock: function() {
		// edit mode
		this.panelWidget.setXhtmlValue("");
		this.panelWidget.setVisible(false);
		this.editorWidget.setVisible(true);
		this.editorWidget.resize(); // needed if the window size was changed while invisible
	},
	
	_onUnLock: function() {
		// preview mode
		// text value may have changed, grab the most recent text from the edit control and set it on the preview panel;
		var value = this.editorWidget.getXhtmlValue();
		this.editorWidget.setVisible(false);
		this.panelWidget.setVisible(true);
		this.panelWidget.setXhtmlValue(value);
	},
	
	print: function() {
		var dialog = new rmpw.text.internal.PrintConfirmDialog(this.setIncludeComments);
		var onDialogOkListener = this.connect(dialog, "onOk", function() {
			this.disconnect(onDialogOkListener);
			this.disconnect(onDialogCloseListener);			
			this.doPrint(dialog.includeComments);
		});
		var onDialogCloseListener = this.connect(dialog, "onCancel", function() {
			this.disconnect(onDialogOkListener);
			this.disconnect(onDialogCloseListener);
		});		
	},
	
	doPrint: function(includeComments) {
		var comments = includeComments ? this._resource.page.editor.commentingWidget._cache.getAllComments() : null;
		// need to use panelWidget as intermediary as it will ensure links to imported images are converted properly
		this.panelWidget.setXhtmlValue(this.editorWidget.getXhtmlValue());
		
		var win = window.open('');		
		var doc = win.document;

		// make sure all the style sheets are available on the new page 
		var parentStyleSheets = dojo.doc.styleSheets;
		
		doc.write('<html xmlns="http://www.w3.org/1999/xhtml">\n');
		doc.write('<head>\n');
		doc.write('<meta http-equiv="content-type" content="text/html; charset=UTF-8"/>\n');
		var n;
		for (var i = parentStyleSheets.length -1; i >= 0; i--) {
			n = parentStyleSheets[i].ownerNode;
			if (n.rel && n.type && n.href) {
				doc.write('<link rel="' + n.rel + '" type="' + n.type + '" href="' + n.href + '"></link>\n');
			}
		}
		doc.write('</head>\n');
		doc.write('<body>\n');
		doc.write("<div id='printContent' class='dm-rich-text-editor dijitLayoutContainer dijitContainer jazz-ui-StyledBox content dijitContentPane'>");
		doc.write(this.panelWidget.get('content'));
		doc.write("</div>");
		doc.write('\n');
		doc.write('</body>\n');
		doc.write('</html>\n');
		
		// set up comments, if asked for and if any available
		if (comments) {
			var element;
			var printContentEl = doc.getElementById("printContent");		
			for (var i = 0, comment; comment = comments[i]; i++) {
				var addedComment = doc.createElement("div");
				addedComment.setAttribute("id", comment.uri.substring(comment.uri.lastIndexOf("/") + 1));
				addedComment.setAttribute("class", "printComment");
				var body = comment.getBody(rmps.core.commenting.BodyType.PLAIN_TEXT.value).contents;
				addedComment.innerHTML = comment.subject + "  <i>Author: <span style='color:#16A1BC;'><b>" + comment._creator.name + "</b></span>  Created: " + rmps.core.util.DateUtils.formatDateLabel(comment.created) + "</i>";
				if (body) {
					addedComment.innerHTML += "<br>" + body;
				}
				if (comment.inReplyTo) {
					// append comment to the comment it replies to
					element = doc.getElementById(comment.inReplyTo.substring(comment.inReplyTo.lastIndexOf("/") + 1));
					addedComment.setAttribute("class", "printComment printReply");
				} else if (comment.range.length > 0) {
					// append comment to the related doc tag
					element = doc.getElementById(comment.range[0].ids[0]);
				}
				if (element) {					
					element.appendChild(addedComment);
				} else {
					// comment is not related to any tag (or tag could not be found), 
					// and is not reply to any comment, so append to the bottom
					printContentEl.appendChild(addedComment);
				}
			}
		}
		
		doc.close(); 
	}
});

})();