/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.viewlets.internal.SimpleDetails");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.html");

dojo.require("rmps.app.core.util");

dojo.require("rmps.ui.resource.ResourceIconLabel");
dojo.require("rmps.ui.EllipsisLabel");
dojo.require("rmps.ui.user.UserLabel");

dojo.require("rmps.core.util.DateUtils");

var DateUtils = rmps.core.util.DateUtils;

dojo.declare("rmps.viewlets.internal.SimpleDetails", [dijit._Widget, dijit._Templated], {

	templateString: dojo.cache("rmps.viewlets", "internal/templates/SimpleDetails.html"),
	widgetsInTemplate: true,

	// attach points
	resourceIconLabel: null,
	timestamp: null,
	author: null,
	pane: null,

	// mixin properties
	object: null,
	contextURIDef: null,
	useJazzResourceLinks: true,

	_timestampHandle: null,
	_popupContents: null,
	_resourcePopup: null,
	_rilReady: false,

	postCreate: function() {
		this.setUser();

		// resource info
		this.connect(this.resourceIconLabel, "onUpdate", dojo.hitch(this, function() {
			this._rilReady = true;
			if (this.resize) {
				this.resize({w: this.domNode.offsetWidth});	
			}
		}));

		var def = this.contextURIDef;
		if (!def) {
			def = new dojo.Deferred();
			def.callback(null);
		}
		
		def.addBoth(dojo.hitch(this, function(contextURI) {		
			var url = this.getProjectPrefix();
			var query = {};
			if (dojo.config.isDebug) {
				query["debug"] = true;
			}
			if (contextURI) {
				query["rmps.context"] = contextURI;			
			}
			var querystr = dojo.objectToQuery(query);
			if (querystr) {
				url += '?' + querystr;
			}
			url += rmps.app.core.util.getDesignViewResourceUri(this.object.resourceUri);
			this.resourceIconLabel.linkHref = url;
			this.resourceIconLabel.hasJazzResourceLink = this.useJazzResourceLinks;
			var properties = ['shortName', 'iconUri', 'longName'];
			if (this.useJazzResourceLinks) {
				var properties = ['shortName', 'iconUri'];
			}
			
			this.object.getResourceInfo(properties, contextURI).addCallback(this, function(resourceInfo) {
				var info;
				if(dojo.isArray(resourceInfo)){
					info= resourceInfo[0];
				}else{
					info= resourceInfo;
				}
				this.resourceIconLabel.setResourceInfo(info);
			}).addErrback(this, function (err) {
				var info = {
					uri: this.object.resourceUri,
					shortName: this.object.title,
					longName: this.object.title
				};
				this.resourceIconLabel.setResourceInfo(info);
			});
	
			// date stamp
			this.timestamp.appendChild(dojo.doc.createTextNode(DateUtils.getDateDiff(this.object.created)));
	
			this._timestampHandle = DateUtils.addDateDiffUpdater(this.object.created,
				dojo.hitch(this, function(newDateRange) {
					dojo.html._setNodeContent(this.timestamp, dojo.doc.createTextNode(newDateRange), true);
				}));
	
			this.setLabel();
	
			if (!this.useJazzResourceLinks) {
				this._resourcePopup = this.createResourcePopup(this.domNode);	
			}
	
			this.connect(this.resourceIconLabel, "linkClicked", function(e) {
				dojo.removeClass(this.pane, "pane-hover");
			});
		}));
	},

	setUser: function(){
		// creator
		if(this.author){
			this.object.getCreator().addCallback(dojo.hitch(this, function(user) {
				this.author.setUser(user);
				if (this._resourcePopup && !this._resourcePopup._isOpen) {
					this._resourcePopup.destroy();
					this._resourcePopup = this.createResourcePopup(this.domNode);
				}
			}));
		}
	},

	destroy: function() {
		if (this._popupContents) {
			this._popupContents.destroyRecursive();
			delete this._popupContents;
		}
		if (this._resourcePopup) {
			this._resourcePopup.destroyRecursive();
			delete this._resourcePopup;
		}
		if (this._timestampHandle) {
			DateUtils.removeDateDiffUpdater(this._timestampHandle);
			delete this._timestampHandle;
		}
		this.inherited(arguments);
	},

	createResourcePopup: function(node) {
		return new jazz.ui.HoverPopup({
			around: this.domNode,
			createContents: dojo.hitch(this, this._createPopupContents)
		});
	},

	_createPopupContents: function(popupWidget) {
		this._popupContents = this.getFullDetails();
		if(this._popupContents){
			return this.createPopupContents();
		}
	},

	getFullDetails: function(){
		/*To be implemented by sub-classes */
	},

	setLabel: function(){
		/*To be implemented by sub-classes */
	},

	createPopupContents: function(){
		this._popupContents.showFullLabels(true);
		var temp = dojo.doc.createElement("div");
		dijit.popup.moveOffScreen(temp);
		temp.appendChild(this._popupContents.domNode);
		this._popupContents.startup();
		temp.removeChild(this._popupContents.domNode);
		dojo.destroy(temp);
		return this._popupContents.domNode;
	},

	_formatDate: function(date) {
		var dateStr = dojo.date.locale.format(date, {formatLength: "medium", selector: "date"});
		var timeStr = dojo.date.locale.format(date, {formatLength: "short", selector: "time"});
		return dateStr + ' ' + timeStr;
	},

	_trim: function(str) {
//		return str.replace(/^(RE:\s)(\s+|RE:)*|(\s+)$/ig, '$1');
		return str;
	},

	// events
	_onPaneMouseEnter: function(event) {
		dojo.addClass(this.pane, "pane-hover");
	},

	_onPaneMouseLeave: function(event) {
		dojo.removeClass(this.pane, "pane-hover");
	},
	
	resize: function(size) {
		if (size && size.w) {
			this.domNode.style.width = size.w + "px";
		}
	}
});

})();