/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.viewlets.internal.comments.RecentCommentsViewlet");

dojo.require("rmps.core.commenting.CommentClient");
dojo.require("rmps.core.util.DateUtils");

dojo.require("rmps.viewlets.internal.comments.SimpleCommentDetails");
dojo.require("rmps.viewlets.internal.comments.FullCommentDetails");
dojo.require("rmps.viewlets.internal.PagedTableViewlet");

dojo.require("jazz.app.context");

dojo.require("dojo.date.stamp");

var DateUtils = rmps.core.util.DateUtils;
var CommentClient = rmps.core.commenting.CommentClient;

var PREF_DATE_RANGE = "dateRange";

dojo.declare("rmps.viewlets.internal.comments.RecentCommentsViewlet", rmps.viewlets.internal.PagedTableViewlet, {

	isClientPaged: true,

	getStartDate: function() {
		var range = this.getPreference(PREF_DATE_RANGE);
		var date = DateUtils.rangeToDate(range);
		if (date) {
			return dojo.date.stamp.toISOString(date);
		}
		return null;
	},

	getSecondaryTitle: function() {
		var title = this.inherited(arguments);
		var rangeString = DateUtils.rangeToString(this.getPreference(PREF_DATE_RANGE));
		if (rangeString) {
			title += ' ' + rangeString;
		}
		return title;
	},

	createEntryWidget: function(data) {
		return new rmps.viewlets.internal.comments.SimpleCommentDetails({
				comment: data,
				getProjectPrefix: dojo.hitch(this, this.getProjectPrefix),
				contextURIDef: this._getContextURIDef
			});
	},

	createDetailsWidget: function(data) {
		return new rmps.viewlets.internal.comments.FullCommentDetails({
				comment: data,
				getProjectPrefix: dojo.hitch(this, this.getProjectPrefix),
				contextURIDef: this._getContextURIDef
			});
	},

	fetchData: function(startIndex, count, entries, details, projectId) {
		var deferred = new dojo.Deferred();
		this._errorMsgNode = null;
		
		this._getContextURIDef.addCallback (this, function (contextURI) {
		

			var commentDef = CommentClient.getRecentComments(projectId, this.getStartDate(), startIndex/count, count, contextURI, this._pageToken);
			commentDef.addCallback(function(comments) {
				function comparator(a, b) {
					var ta = a.created.getTime();
					var tb = b.created.getTime();
					return ta < tb ? 1 : ta == tb ? 0 : -1;
				}
				comments.sort(comparator);
				deferred.callback(comments);
			});
			commentDef.addErrback(deferred.errback);
		});	
		this._getContextURIDef.addErrback(deferred.errback);
		deferred.addErrback(console.error);
	
		return deferred;
	}
	
});

})();