/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.viewlets.internal.comments.SimpleCommentDetails");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.html");

dojo.require("rmps.app.core.util");

dojo.require("rmps.viewlets.internal.comments.FullCommentDetails");

dojo.require("rmps.ui.resource.ResourceIconLabel");
dojo.require("rmps.ui.EllipsisLabel");
dojo.require("rmps.ui.user.UserLabel");

dojo.require("rmps.core.util.DateUtils");

var DateUtils = rmps.core.util.DateUtils;

dojo.declare("rmps.viewlets.internal.comments.SimpleCommentDetails", [dijit._Widget, dijit._Templated], {

	templateString: dojo.cache("rmps.viewlets", "internal/comments/templates/SimpleCommentDetails.html"),
	widgetsInTemplate: true,

	// attach points
	resourceIconLabel: null,
	timestamp: null,
	author: null,
	pane: null,
	subject: null,
	_marker: null,

	// mixin properties
	comment: null,
	contextURIDef: null,
	useJazzResourceLinks: true,

	// private properties
	_controller: null,
	_timestampHandle: null,
	_popupContents: null,
	_commentPopup: null,
	_rilReady: false,

	postCreate: function() {
		// creator
		this.comment.getCreator().addCallback(dojo.hitch(this, function(user) {
			this.author.setUser(user);
			if (this._commentPopup && !this._commentPopup._isOpen) {
				this._commentPopup.destroy();
				this._commentPopup = this.createCommentPopup(this.domNode);
			}
		}));

		// resource info
		this.connect(this.resourceIconLabel, "onUpdate", dojo.hitch(this, function() {
			this._rilReady = true;
			this.resize({w: this.domNode.offsetWidth});
		}));
		
		var def = this.contextURIDef;
		if (!def) {
			def = new dojo.Deferred();
			def.callback(null);
		}
		
		def.addBoth(dojo.hitch(this, function(contextURI) {
			var url = this.getProjectPrefix();
			var query = {};
			if (dojo.config.isDebug) {
				query["debug"] = true;
			}
			if (contextURI) {
				query["rmps.context"] = contextURI;			
			}
			var querystr = dojo.objectToQuery(query);
			if (querystr) {
				url += '?' + querystr;
			}
			url += rmps.app.core.util.getDesignViewResourceUri(this.comment.resourceUri) + "&comment=" + encodeURIComponent(this.comment.uri);
			this.resourceIconLabel.linkHref = url;
			this.resourceIconLabel.hasJazzResourceLink = this.useJazzResourceLinks;
			var properties = ['shortName', 'iconUri', 'longName'];
			if (this.useJazzResourceLinks) {
				var properties = ['shortName', 'iconUri'];
			}
			
			this.comment.getResourceInfo(properties, contextURI).addCallback(this, function(resourceInfo) {
				this.resourceIconLabel.setResourceInfo(resourceInfo);
			});
	
			// date stamp
			this.timestamp.appendChild(dojo.doc.createTextNode(DateUtils.getDateDiff(this.comment.created)));
	
			this._timestampHandle = DateUtils.addDateDiffUpdater(this.comment.created,
				dojo.hitch(this, function(newDateRange) {
					dojo.html._setNodeContent(this.timestamp, dojo.doc.createTextNode(newDateRange), true);
				}));
	
			this.subject.setLabel(this._trim(this.comment.subject));
	
			this._commentPopup = this.createCommentPopup(this.domNode);
	
			this.connect(this.resourceIconLabel, "linkClicked", function(e) {
				dojo.removeClass(this.pane, "pane-hover");
			});
	
			this.connect(this.author, "initUserInfo", function(userInfo) {
				userInfo.emailProperties.setSubject("RE: " + this.comment.subject);
			});
		}));
	},

	destroy: function() {
		if (this._popupContents) {
			this._popupContents.destroyRecursive();
			delete this._popupContents;
		}
		if (this._commentPopup) {
			this._commentPopup.destroyRecursive();
			delete this._commentPopup;
		}
		if (this._timestampHandle) {
			DateUtils.removeDateDiffUpdater(this._timestampHandle);
			delete this._timestampHandle;
		}
		this.inherited(arguments);
	},

	resize: function(size) {
		if (size && size.w) {
			this.domNode.style.width = size.w + "px";
			if (this._rilReady) {
				var subjectSize = {w: size.w - this._marker.offsetLeft - this._marker.offsetWidth - 1};
				this.subject.resize(subjectSize);
			}
		}
	},

	createCommentPopup: function(node) {
		return new rmps.viewlets.internal.comments.popup({
			around: this.domNode,
			createContents: dojo.hitch(this, this._createPopupContents)
		});
	},

	_createPopupContents: function(popupWidget) {
		this._popupContents = new rmps.viewlets.internal.comments.FullCommentDetails({
			comment: this.comment,
			getProjectPrefix: this.getProjectPrefix
		});
		this._popupContents.showFullLabels(true);

		var temp = dojo.doc.createElement("div");
		dijit.popup.moveOffScreen(temp);
		temp.appendChild(this._popupContents.domNode);
		this._popupContents.startup();
		temp.removeChild(this._popupContents.domNode);
		dojo.destroy(temp);

		return this._popupContents.domNode;
	},

	_formatDate: function(date) {
		var dateStr = dojo.date.locale.format(date, {formatLength: "medium", selector: "date"});
		var timeStr = dojo.date.locale.format(date, {formatLength: "short", selector: "time"});
		return dateStr + ' ' + timeStr;
	},

	_trim: function(str) {
//		return str.replace(/^(RE:\s)(\s+|RE:)*|(\s+)$/ig, '$1');
		return str;
	},

	// events
	_onPaneMouseEnter: function(event) {
		dojo.addClass(this.pane, "pane-hover");
	},

	_onPaneMouseLeave: function(event) {
		dojo.removeClass(this.pane, "pane-hover");
	}
});

dojo.declare("rmps.viewlets.internal.comments.popup", jazz.ui.HoverPopup, {
_hover: function(e) {
	if (this.programmatic)
		return;
	if(this._closeTimer){
		clearTimeout(this._closeTimer);
		delete this._closeTimer;
	}
	if (!this._isOpen && !this._openTimer) {
		var target = e.target;
		this._openTimer = setTimeout(dojo.hitch(this,function(){	
			//this is needed in order not to show popup of comment if popup of resource link is already open and target of comment popup is resource link
			var m = jazz.ui._getMasterPopup();
			if (m._showing && target  && m._showing._target == target){
				return;
			} else {
				this.open(this._determineTarget(target));
			}
		}), this.openDelay);
	}
}
		
});

})();