/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.viewlets.internal.dashboard.AssociatedDomainsInfo");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.TooltipDialog");
dojo.require("dojo.DeferredList");
dojo.require("dojox.xml.parser");
dojo.require("jazz.app.context");
dojo.require("jazz.util.i18n");
dojo.require("jazz.ui.StyledTable");
dojo.require("rmps.app.ui.internal.util.DomainRegistryUtil");
dojo.require("rmps.core.RootServices");
dojo.require("rmps.app.core.util");
dojo.require("rmps.app.ui.internal.dialogs.SelectDomainVersionDialog");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.viewlets.internal.dashboard", "ProjectContextInfoMessages");

(function(){
	
var DomainRegistryUtil = rmps.app.ui.internal.util.DomainRegistryUtil;
var Util = rmps.app.core.util;
var DOMAIN_REG_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "domainRegistry");
var PROJECT_AREA_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.JP_0_6, "projectAreas") + '/';
	
var messages;

dojo.declare("rmps.viewlets.internal.dashboard.AssociatedDomainsInfo", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("rmps.viewlets", "internal/dashboard/templates/AssociatedDomainsInfo.html"),
	
	_domainDataArray: null,
	_isManaged: false,
	
	constructor: function(args) {
		if (!messages) {
			messages = dojo.i18n.getLocalization("rmps.viewlets.internal.dashboard", "ProjectContextInfoMessages");
		}
		this.messages = messages;
	},

	postCreate: function() {
		this.inherited(arguments);
	
		var domainsLink = PROJECT_AREA_PATH + jazz.app.context.get().itemId;
		this._titleLink.href = domainsLink;	
		
		var def1 = new dojo.Deferred();
		var def2 = new dojo.Deferred();
		DomainRegistryUtil.isProjectActivelyManaged(def1.callback);
				
		rmps.core.client.xhrGet({
			url: DOMAIN_REG_PATH + "/getDomainVersions",
			handleAs: 'json',
			headers: {
				"Accept" : "application/json"
			},
			load: function(response, ioArgs) {
				def2.callback(response);
			}
		}, true);

		var defList = new dojo.DeferredList([def1, def2]);
		defList.addCallback(this, function(results) {
			this._isManaged = results[0][1];
			this._domainDataArray = results[1][1];
			this._bottom.innerHTML = ""; // Remove loading message
			this._renderAssociatedDomains();
			this._renderDomainVersions();
			dojo.create("div", {innerHTML: this._isManaged ? messages.managed : messages.notManaged}, this._bottom);
			jazz.util.i18n.addMessage(dojo.create("div",{},this._bottom), messages.toDomainsArea)[0].href = domainsLink;
		});
		
	},
	
	_setDependencies: function() {
		var dialog = rmps.app.ui.internal.dialogs.SelectDomainVersionDialog({
			domainDataArray: this._domainDataArray,
			readOnly: false,
		});
		dialog.show();
	},
	
	renderDomains: function() {
		for (var i = 0; i < this._domainDataArray.length; i++) {
			var domain = this._domainDataArray[i];
			dojo.create("a", {
					className: "domain",
					innerHTML: jazz.util.html.escape(domain.label),
					href: Util.getDesignViewResourceUriForContext(domain.id, domain.defaultStreamUri, domain.projectName)
			}, this._contentArea);
		}
	},
	
	_renderAssociatedDomains: function() {
		if (this._domainDataArray) {
			this.renderDomains();	
			// Update number of associated domains
			this._totalCount.innerHTML = dojo.string.substitute(messages.totalCount, [this._domainDataArray.length]);
		} else {
			this._totalCount.innerHTML = dojo.string.substitute(messages.totalCount, [0]);
		}
	},
	
	_renderDomainVersions: function() {
		dojox.xml.parser.removeChildren(this._dependencyArea);				
		
		if (this._domainDataArray.length === 0) {
			return;
		}
		
		var columnData = [];
		columnData.push({ label: messages.domainProject });
		columnData.push({ label: messages.version });

		var table = new jazz.ui.StyledTable({
			trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
			columns: columnData,
			showHover: false
		});
		this._dependencyArea.appendChild(table.domNode);
		table.domNode.style.width = "100%";
		var projectNameToDomainNameTd = [];
		
		for (var i = 0; i < this._domainDataArray.length; i++) {
			var domain = this._domainDataArray[i];
			var projectName = domain.projectName;
			
			// if domain project is already created, then include the domain name in the same row
			td = projectNameToDomainNameTd[projectName];
			if (td) {
				continue;
			}
			
			var tr = document.createElement("tr");
			
			projectNameToDomainNameTd[projectName] = true;
			
			// domain project column
			td = document.createElement("td");
			// createTextNode escapes the string
			td.appendChild(dojo.doc.createTextNode(domain.projectName));
			tr.appendChild(td);
			
			// version column
			td = document.createElement("td");
			var snapshots = domain.snapshots;
			
			var n = 0;
			if (snapshots) {
				for (n = 0; n < snapshots.length; n++) {
					if (snapshots[n].selected) {
						// createTextNode escapes the string
						td.appendChild(dojo.doc.createTextNode(snapshots[n].name));
						break;
					}
				}
			}
			if (!snapshots || n >= snapshots.length) {
				// createTextNode escapes the string
				td.appendChild(dojo.doc.createTextNode(messages.unset));
			}
			tr.appendChild(td);
			
			table.tbody.appendChild(tr);
		}
	}, 
	
});
}());