/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.viewlets.internal.links.RecentLinksViewlet");

dojo.require("rmps.viewlets.internal.PagedTableViewlet");
dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");

dojo.require("rmps.app.pickers.internal.LinkTypePicker");
dojo.require("rmps.app.ui.internal.editors.views.link.LinkClient");
dojo.require("rmps.core.util.DateUtils");
dojo.require("rmps.core.util.IRmpsConstants");

dojo.require("rmps.viewlets.internal.links.SimpleLinkDetails");
dojo.require("rmps.viewlets.internal.links.FullLinkDetails");

dojo.require("dojo.date.stamp");
dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.viewlets.internal.links", "RecentLinksViewletMessages");
var messages = dojo.i18n.getLocalization("rmps.viewlets.internal.links", "RecentLinksViewletMessages");

var DateUtils = rmps.core.util.DateUtils;
var IRmpsConstants = rmps.core.util.IRmpsConstants;
var LinkClient = rmps.app.ui.internal.editors.views.link.LinkClient;

var PREF_DATE_RANGE = "dateRange";
var PREF_LINKTYPES = "linkTypes";

dojo.declare("rmps.viewlets.internal.links.RecentLinksViewlet", rmps.viewlets.internal.PagedTableViewlet, {

	_linktypeProvider: null,

	isClientPaged: true,

	getStartDate: function() {
		var range = this.getPreference(PREF_DATE_RANGE);
		var date = DateUtils.rangeToDate(range);
		if (date) {
			return dojo.date.stamp.toISOString(date);
		}
		return null;
	},

	getSecondaryTitle: function() {
		var title = this.inherited(arguments);
		var rangeString = DateUtils.rangeToString(this.getPreference(PREF_DATE_RANGE));
		if (rangeString) {
			title += ' ' + rangeString;
		}
		return title;
	},

	getLinkTypes: function() {
		var preference = this.getPreference(PREF_LINKTYPES);
		var selection = preference && preference !== "" ? dojo.fromJson(preference) : null;
		if (!selection || selection[messages.allDeployed]) {
			return null;
		}

		var linkTypes = '<?xml version="1.0" encoding="UTF-8"?>\n';
		linkTypes += '<rmps:linkTypes xmlns:rmps="' + IRmpsConstants.RMPS_XML_NAMESPACE + '"\n';
		linkTypes += '                xmlns:oslc_am="' + IRmpsConstants.OSLC_AM2_XML_NAMESPACE + '"\n';
		linkTypes += '                xmlns:rdf="' + IRmpsConstants.ProcessConstants.NAMESPACE_RDF + '">\n';

		for (var title in selection) {
			if (!title) {
				continue;
			}

			linkTypes += '  <oslc_am:LinkType rdf:resource="' + selection[title] + '"/>\n';
		}

		linkTypes += '</rmps:linkTypes>\n';

		return linkTypes;
	},

	createEntryWidget: function(data) {
		return new rmps.viewlets.internal.links.SimpleLinkDetails({
				object: data,
				getProjectPrefix: dojo.hitch(this, this.getProjectPrefix),
				contextURIDef: this._getContextURIDef
			});
	},

	createDetailsWidget: function(data) {
		return new rmps.viewlets.internal.links.FullLinkDetails({
				object: data,
				getProjectPrefix: dojo.hitch(this, this.getProjectPrefix)
			});
	},

	fetchData: function(startIndex, count, entries, details, projectId) {
		var deferred = new dojo.Deferred();

		this._getContextURIDef.addCallback(this, function(contextURI) {
		
			this._errorMsgNode = null;
			var linksDef = LinkClient.getRecentLinks(projectId, this.getStartDate(), startIndex/count, count, this._pageToken,
					this.getLinkTypes(), contextURI, true /*fallbackToDefaultContext*/);
			
			linksDef.addCallback(function(links) {
				function comparator(a, b) {
	
					if (a.created && b.created) {
						var ta = a.created.getTime();
						var tb = b.created.getTime();
						return ta < tb ? 1 : ta === tb ? 0 : -1;
					}
					else if (a.created && !b.created) {
						return -1;
					}
					else if (!a.created && b.created){
						return 1;
					}
					else {
						return 0;
					}
				}
				links.sort(comparator);
				deferred.callback(links);
			});
			linksDef.addErrback(deferred.errback);
			this._getContextURIDef.addErrback(deferred.errback);
		});
		deferred.addErrback(console.error);
		return deferred;
	},
	
	getPreferenceProvider: function(id) {
		if (id === PREF_LINKTYPES) {
			if (!this._linktypeProvider) {
				this._linktypeProvider = new rmps.viewlets.internal.links.RecentLinksViewlet._Provider(this);
			}
			return this._linktypeProvider;
		}

		return this.inherited(arguments);
	}
});

dojo.declare("rmps.viewlets.internal.links.RecentLinksViewlet._Provider", com.ibm.team.dashboard.web.ui.CustomPreferenceProvider, {

	_viewlet: null,

	constructor: function(viewlet) {
		this._viewlet = viewlet;
	},

	getLabel: function(linkTypes) {
		var message = null;
		var selection = dojo.fromJson(linkTypes ? linkTypes : this._initDefaultValue());
		if (!selection || selection[messages.allDeployed]) {
			message = messages.allDeployed;
		} else {
			var addTail = false;
			for (var title in selection) {
				if (!title) {
					continue;
				}

				if (addTail) {
					message += ", '" + title + "'";
				} else {
					message = "'" + title + "'";
					addTail = true;
				}
			}
		}

		return '<input type="text" readonly="readonly" value="' + message + '"/>';
	},

	getLoadingLabel: function() {
		return messages.loading;
	},

	createChooser: function(onOk, settings, viewlet) {
		var dialog = null;
		var onOk2 = dojo.hitch(this, function(selection) {
			if (dialog) {
				dialog.close();
				dialog.destroyRecursive();
			}
			if (selection) {
				viewlet.setPreference(PREF_LINKTYPES, selection);
				onOk(selection, PREF_LINKTYPES);
			}
		});

		var onCancel = function() {
			if (dialog) {	
				dialog.close();
				dialog.destroyRecursive();
			}
		};

		this._viewlet._getContextURIDef.addCallback(this, function(contextURI){
			var selection = viewlet.getPreference(PREF_LINKTYPES) || this._initDefaultValue();

			var picker = new rmps.app.pickers.internal.LinkTypePicker({
				projectId: settings.scopeItemId || viewlet._getProjectId(),
				contextURI: contextURI,
				selection: selection,
				selectionSize: 10,
				multiple: true,
				onOk: onOk2,
				onCancel: onCancel
			});

			var dialogParams = {
				primaryTitle: messages.linkTypePreferenceTitle,
				width: "400px",
				contentNode: picker.domNode
			};
			dialog = new jazz.ui.Dialog(dialogParams);
		});
	},

	_initDefaultValue: function() {
		var defaultValue = {};
		defaultValue[messages.allDeployed] = true;
		var selection = dojo.toJson(defaultValue); 
		this._viewlet.setPreference(PREF_LINKTYPES, selection);
		return selection;
	}
});

}());