/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.viewlets.internal.links.SimpleLinkDetails");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.html");
dojo.require("jazz.ui.ResourceLink");

dojo.require("rmps.ui.user.UserLabel");
dojo.require("rmps.viewlets.internal.SimpleDetails");
dojo.require("rmps.viewlets.internal.links.FullLinkDetails");
dojo.require("rmps.core.util.ILinksConstants");

var ResourceLink = jazz.ui.ResourceLink;
var ILinksConstants = rmps.core.util.ILinksConstants;

dojo.declare("rmps.viewlets.internal.links.SimpleLinkDetails", rmps.viewlets.internal.SimpleDetails, {

	templateString: dojo.cache("rmps.viewlets", "internal/links/templates/SimpleLinkDetails.html"),

	_resourceLink: null,
	useJazzResourceLinks: false,

	setLabel: function() {
		
		var iconSpan = document.createElement("SPAN");
        dojo.addClass(iconSpan, "icon");
		this.subject.domNode.appendChild(iconSpan);
		
		var anchor = document.createElement("a");
		this.subject.domNode.appendChild(anchor);
		
		var value= this.object.value;
		var _label= this.object.label;
		var label = jazz.util.html.escape(_label?_label:value);
		
	    anchor.href = value;
	    if (!this.object.dmProjectInternal) {
		    anchor.target = "_blank";
	    } 

	    /*We do not require having a compact rednering for External Link Type */
	    if (this.object.predicate != ILinksConstants.EXTERNAL_LINK_TYPE) {
			anchor.appendChild(document.createTextNode(dojo.doc.createTextNode(label)));
			this._resourceLink= new ResourceLink({
			    uri: value,
			    iconNode: iconSpan,
			    label: label,
			    absoluteDMLinksOnly: !this.object.dmProjectInternal,
			    avoidBaselineCheck: true
			}, anchor);

		    /*Make sure to disable showing the detailed popup when we hover over the resource link; otherwise, this will cause the detailed popup
			 * to show up while (after its delay time), and causes the resource link rich hover to be closed therefore */
			this.connect(this._resourceLink.domNode, "onmouseenter", dojo.hitch(this, function() {
				if(this._resourcePopup){
					this._resourcePopup.close();
					this._resourcePopup.enabled= false;					
				}
			}));
			
			/*After we leave the resource link anchor, we should re-enable detailed popup*/	
			this.connect(this._resourceLink.domNode, "onmouseleave", dojo.hitch(this, function() {
				if(this._resourcePopup){
					this._resourcePopup.enabled= true;					
				}
			}));
	    }else{
		    var val= this.object.description;
		    if(!val|| dojo.trim(val)== ""){
			    /*If there is no description for the external links, then, we show the link itself */
			    val= value;
			}
		    anchor.appendChild(dojo.doc.createTextNode(val));
	    }
	},

	resize: function(size) {
		if (size && size.w) {
			this.domNode.style.width = size.w + "px";
			if (this._rilReady) {
				this.subject.resize(size);
			}
		}
	},

	createPopupContents: function(node) {
	    if (this.object.predicate == ILinksConstants.EXTERNAL_LINK_TYPE){
			this.object.title= this.object.value;
		}else{
			this.object.title= this._resourceLink.domNode.text;
		}
		return this.inherited(arguments);
	},
	
	destroy: function() {
		if(this._resourceLink){
			this._resourceLink.destroyRecursive();
			delete this._resourceLink;
		}
		this.inherited(arguments);
	},

	getFullDetails: function(){
		this._popupContents = new rmps.viewlets.internal.links.FullLinkDetails({
			object: this.object,
			compactRendering: false,
			getProjectPrefix: this.getProjectPrefix
		});
		return this._popupContents;
	}
});

})();