/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
	// This type defines custom plugins for the embedded rich text CKEditor.
	// This allows for encapsulation of the plugin code and a single point of
	// definition for the plugins which are dynamically loaded at runtime.
    dojo.provide("rmpw.text.internal.plugins.DMCKEditorPlugins");
	dojo.require("rmpw.common.jazz.internal.widgets.RemoteItemPicker");
	dojo.require("rmps.core.resource.ResourceInfoClient");
	dojo.require("jazz.app.context");
	dojo.require("rmpw.text.internal.util.ImageSrcUtil");
	dojo.require("jazz.util.html");
	
	dojo.requireLocalization("rmpw.text.internal.plugins", "DMCKEditorPluginsMessages");

	var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;
	var messages = dojo.i18n.getLocalization("rmpw.text.internal.plugins", "DMCKEditorPluginsMessages");

	var imageType = "http://jazz.net/ns/dm/file#Image";

	var openDialog = function(url, editor, onOk, onClose){
        
        var self = this;
        if (dojo.config.isDebug) {
            var idx = url.indexOf('?');
            if (idx !== -1) {
                url = url.substring(0, idx + 1) + 'debug=true&' + url.substring(idx + 1);
            }
            else {
                url = url + '?debug=true';
            }
        }
        var dialogDescription = {
            title: messages.itemSelection,
            width: "oslc:hintWidth",
            height: "oslc:hintHeight",
            widget: "oslc:dialog",
            protocol: "#oslc-core-windowName-1.0",
			namespace: "oslc",
            path: {
                descriptor: ["oslc:ServiceProvider", "oslc:Service"],
                categories: ["oslc:selectionDialog"],
                name: "oslc:Dialog"
            }
        };
        var params = {
            dialogTitle: dialogDescription.title,
            secondaryTitle: "",
            width: "750px",
            height: "550px",
            namespace: dialogDescription.namespace,
            protocol: dialogDescription.protocol,
            parentElement: editor.domNode,
            onOk: onOk,
            onClose: onClose,
            pickerURL: url
        };
        this.itemPicker = new rmpw.common.jazz.internal.widgets.RemoteItemPicker(params);
    };

    //FIXME null guard condition?
    var extractId = function(resourceUri){
        return resourceUri.substring(resourceUri.lastIndexOf("/")+1); 
    };

    var buildPickerUrl = function (extraParameter){
        var pickerUrl = net.jazz.ajax._contextRoot + "/_ajax-modules/rmps.pickers.RMPSPicker?project=" 
                        + jazz.app.context.get().itemId 
                        + "&context=" + jazz.app.context.get().baseline;
        if (extraParameter){
            pickerUrl += extraParameter;
        }
        if (jazz.app.context.get().changeset) {
            pickerUrl += rmpw.text.internal.util.ImageSrcUtil.changesetPrefix + jazz.app.context.get().changeset;
        }
        return pickerUrl; 
    };

    var defineCustomLinkPlugin = function() {
		CKEDITOR.plugins.add('customLink', {
			itemPicker: null,
			init: function(editor) {
				var pluginName = 'customLink';
				var linkCommandName = pluginName;
				var styleDef = {
					element: "span",
					attributes: {
						"class": "markedForLink"
					}
				};
				
				// Link Command, override CKEditor behavior with our custom create link behavior.
				var style = new CKEDITOR.style(styleDef);
				var linkCmd = new CKEDITOR.styleCommand(style);
				editor.addCommand(linkCommandName, linkCmd);
				// override the CKEditor link behavior
				editor.addCommand( 'link', linkCmd);			
				dojo.connect(linkCmd, "exec", dojo.hitch(this, function() {
					this.createHyperlink(editor);
				}));
				
				// Unlink Command, extend the CKEditor behavior to remove associated
				// links from server.
				var unlinkCommandName = 'customUnlink';
				var unlinkCmd = editor.addCommand(unlinkCommandName, {
					exec: function(editor) {
						// leaving this here for now incase we want 
						// any custom behaviour on unlink.
						editor.execCommand('unlink');
					} 
				});
				
				var customLinkDef = {
					label: editor.lang.link.toolbar,
					command: linkCommandName,
					group: 'customlink',
					icon: net.jazz.ajax._contextRoot + '/web/rmpw.text/internal/plugins/images/hyperlink.gif',
					order : 1
				};
				
				var customUnlinkDef = {
					label: editor.lang.link.unlink,
					command: unlinkCommandName,
					group: 'customlink',
					icon: net.jazz.ajax._contextRoot + '/web/rmpw.text/internal/plugins/images/hyperlink-cut.gif',
					order : 1
				};
				// Toolbar buttons
				editor.ui.addButton('CustomLink', customLinkDef);
				editor.ui.addButton('Unlink', customUnlinkDef);
				
				// Context menu items
				
				if (editor.addMenuItems) {
					editor.addMenuGroup('customlink',1);
					
					editor.addMenuItems({
						customlink: customLinkDef,
						unlink: customUnlinkDef
					});
				}
				
				
				if (editor.contextMenu) {
					editor.contextMenu.addListener( function( element, selection )
					{
						if (element && !element.is( 'a' ) && !element.is('img')){
							return { customlink : CKEDITOR.TRISTATE_ON };															
						}
					});
				}

				// Enable/Disable unlink and link buttons on toolbar as the selection changes
				editor.on('selectionChange', dojo.hitch(this, function(evt) {
					var unlinkCommand = editor.getCommand(unlinkCommandName),
						linkCommand = editor.getCommand(linkCommandName),
						embeddedLink = false,
						selection = editor.getSelection();
					
					var element = evt.data.path.lastElement && evt.data.path.lastElement.getAscendant('a', true),
					startElement;
					
					if (!element) {
						element = selection.getStartElement();
					}


					// Enable or disable the link button
					if (this.isValidLinkInsertionSelection(editor,selection)) {
						linkCommand.setState(CKEDITOR.TRISTATE_OFF);		// really means Enable button!
					} else {
						linkCommand.setState(CKEDITOR.TRISTATE_DISABLED);
						embeddedLink = true;
					}
					
					// Enable or disable the unlink button
					if (!embeddedLink && element && element.getName() === 'a' && element.getAttribute('href')) {
						unlinkCommand.setState(CKEDITOR.TRISTATE_OFF);		// really means Enable button!
					} else {
						unlinkCommand.setState(CKEDITOR.TRISTATE_DISABLED);
					}
				}));
				
			},
			isValidLinkInsertionSelection: function(editor, selection) {
				return !editor.document.$.body.disabled;
			},
            createHyperlink: function(/*CKEditor*/ editor){
                // summary:
                //		Create a document link
                // description:
                //		This is a utility method for creating links from rich text documents.
                
                // on IE when dialog is opened, the editor selection is lost,
                // cache the selection and reinstate before creating the link
                var selectedRange = null;
                if (dojo.isIE) {
                    selectedRange = editor.document.$.selection.createRange();
                }
                var resetSelection = function(){
                    if (dojo.isIE) {
                        selectedRange.select();
                    }
                };
                var self = this;
                var createImpl = function(results){
                    var result = results[0];
                    var label = result.label;
                    // With the new change in the picker it seems that now that result.label
                    // contains an object and not just the label string
                    if (dojo.isObject(result.label)){
                    	label = result.label.label;
                    }
					self._insertHyperlink(editor, result.url, label);
                };
 
                var onDialogClose = function(){
                    resetSelection();
                    dojo.query(".markedForLink", editor.window.$.document.body).removeClass("markedForLink");
                    editor.focus();
                };

                var pickerURL = buildPickerUrl("&noShowAll=true");

				openDialog(pickerURL, editor, createImpl, onDialogClose);
            },
            _insertHyperlink: function(editor, targetURI, linkText){
                // summary:
                //		Create a hyperlink for the given editor and create a link object for the hyperlink
                // description:
                //		This is a utility method for creating links in rich text editors
                var node = dojo.query(".markedForLink", editor.window.$.document.body)[0];
                var hyperLink = editor.document.createElement('a');
                hyperLink.setAttribute("href", targetURI);
                var html;
                if (node.textContent && dojo.trim(node.textContent) !== ""){
                	html = node.textContent;                	
                } else {
                	html = linkText;
                }
                html = jazz.util.html.escape(html);
                hyperLink.setHtml(html);
                // Test for IE.  Adding link as first thing in a new document
                // fails to find node with dojo.query.
                if (!node || !node.parentNode) {
                    if (node) {
                        dojo.removeClass(node, "markedForLink");
                    }
                    editor.insertElement(hyperLink);
                }
                else {
                    dojo.removeClass(node, "markedForLink");
                    node.parentNode.replaceChild(hyperLink.$, node);
                }
                // Set cursor at the end of the hyperlink
                // TODO: Not working on IE
                editor.focus();
                var ranges = new CKEDITOR.dom.rangeList([new CKEDITOR.dom.range(editor.document)]);
                ranges[0].setStartAfter(hyperLink);
                ranges[0].setEndAfter(hyperLink);
                editor.getSelection().selectRanges(ranges);
                if (editor.richTextWidget && editor.richTextWidget.onChange) {
					editor.richTextWidget.onChange();
				}
            }
		});

	};

    var defineDiagramPlugin = function() {
        var buttonName = 'InsertDiagram';
		CKEDITOR.plugins.add(buttonName.toLowerCase(), {
			itemPicker: null,
			imageobject: null,
			rawimage: null,
			movedimage: null,
			imageArea: null,
			init: function(editor) {
				var pluginName = buttonName.toLowerCase();
				var diagramCommandName = pluginName;
				var styleDef = {
					element: "span",
					attributes: {
						"class": "markedForDiagram"
					}
				};
				
				// Insert Diagram Command.
				var style = new CKEDITOR.style(styleDef);
				var diagramCmd = new CKEDITOR.styleCommand(style);
				editor.addCommand(diagramCommandName, diagramCmd);
				dojo.connect(diagramCmd, "exec", dojo.hitch(this, function() {
					this.insertDiagram(editor);
				}));
				
				//Insert Image Command
				var imageCmd = new CKEDITOR.styleCommand(style);
				editor.addCommand('image', imageCmd);
				dojo.connect(imageCmd, "exec", dojo.hitch(this, function() {
					this.insertImage(editor);
				}));
				
								
				var diagramDef = {
					label: messages.insertDiagram,
					command: diagramCommandName,
					group: 'diagram',
					icon: net.jazz.ajax._contextRoot + '/web/rmpw.text/internal/plugins/images/diagram.gif',
					order: 2
				};
				
				// Toolbar buttons
				editor.ui.addButton(buttonName, diagramDef);
				
				// Context menu items
				if (editor.addMenuItems) {
					editor.addMenuGroup('diagram',1);
					
					editor.addMenuItems({
						diagram: diagramDef
					});
				}
				
				// Register filters for the diagrams.
				var dataProcessor = editor.dataProcessor,
					dataFilter = dataProcessor && dataProcessor.dataFilter,
					htmlFilter = dataProcessor && dataProcessor.htmlFilter;
				if ( dataFilter ) {
					dataFilter.addRules(
						{
							elements :
							{
								img : function( element )
								{
									var realUri = element.attributes.src;
									var className = element.attributes["class"];
									element.attributes.src = rmpw.text.internal.util.ImageSrcUtil.getImageFromResourceUri(realUri, className);
									element.attributes.draggable= false;
								}
							}
						});
				}
				
				if (htmlFilter) {
					htmlFilter.addRules(
							{
								elements :
								{
									img : function( element )
									{
										var modifiedUri = element.attributes.src;
										var className = element.attributes["class"];
										element.attributes.src = rmpw.text.internal.util.ImageSrcUtil.extractResourceUriFromImage(modifiedUri, className);
										delete element.attributes.draggable;
									}
								}
							}
					);
				}

			},
			isValidDiagramInsertionSelection: function(selection) {
				// TODO implement logic to make sure that a diagram can be inserted into the selection.
				return true;
			},
			insertImage: function(/*CKEditor*/ editor){
                // summary:
                //		Insert an uploaded image
                // description:
                //		This is a utility method for inserting a DM uploaded image into rich text documents.
                
                // on IE when dialog is opened, the editor selection is lost,
                // cache the selection and reinstate before creating the link
                var selectedRange = null;
                if (dojo.isIE) {
                    selectedRange = editor.document.$.selection.createRange();
                }
                var resetSelection = function(){
                    if (dojo.isIE) {
                        selectedRange.select();
                    }
                };
                var self = this;
                var createImgImpl = function(results){
                    var result = results[0];
                    var imageId = extractId(result.url);
                    var imageUri = rmpw.text.internal.util.ImageSrcUtil.getImageFromResourceUri(result.url, rmpw.text.internal.util.ImageSrcUtil.imageClassName); 
                    self._insertDiagram(editor, imageId, imageUri, result.label, false);
                };
 
                var onDialogClose = function(){
                    resetSelection();
                    dojo.query(".markedForDiagram", editor.window.$.document.body).removeClass("markedForDiagram");
                    editor.focus();
                };
                var pickerURL = buildPickerUrl("&onlyType=" + encodeURIComponent(imageType) + "&customSearch=true");
				openDialog(pickerURL,editor, createImgImpl, onDialogClose);
            },
			insertDiagram: function(/*CKEditor*/ editor){
                // summary:
                //		Insert a diagram
                // description:
                //		This is a utility method for inserting DM diagrams into rich text documents.
                
                // on IE when dialog is opened, the editor selection is lost,
                // cache the selection and reinstate before creating the link
                var selectedRange = null;
                if (dojo.isIE) {
                    selectedRange = editor.document.$.selection.createRange();
                }
                var resetSelection = function(){
                    if (dojo.isIE) {
                        selectedRange.select();
                    }
                };
                var self = this;
                var createImpl = function(results){
                    var result = results[0];
                    var diagramId = extractId(result.url);
                    var diagramUri = rmpw.text.internal.util.ImageSrcUtil.getImageFromResourceUri(result.url, rmpw.text.internal.util.ImageSrcUtil.embbedClassName); 
					self._insertDiagram(editor, diagramId, diagramUri, result.label, true);
                };
 
                var onDialogClose = function(){
                    resetSelection();
                    dojo.query(".markedForDiagram", editor.window.$.document.body).removeClass("markedForDiagram");
                    editor.focus();
                };
                var pickerURL = buildPickerUrl("&diagram=true");
				openDialog(pickerURL,editor, createImpl, onDialogClose);
            },
	        _handleImage: function(){
               this.hideMessage();
	           this.imageArea.src = this.rawimage.src;
	           this.imageArea.width = this.rawimage.width;
	           this.imageArea.height = this.rawimage.height;
	           
	           this.imageArea.style.left= "0px";
	           this.imageArea.style.top= "0px";
               this.imageArea.style.zIndex= "1";	/*To prevent having the pannable image overlayed on other places*/
               this.imageArea.style.position= "relative";
               this.imageArea.draggable= false;
               this.imageArea.ondragstart = function(e) {return false;};
	         },
            _insertDiagram: function(editor, imageId, targetURI, linkText, embbed){
                // summary:
                //		Create a hyperlink for the given editor and create a link object for the hyperlink
                // description:
                //		This is a utility method for inserting diagrams in rich text editors
                var node = dojo.query(".markedForDiagram", editor.window.$.document.body)[0];
                
                this.rawimage = new Image();
                this.rawimage.onload = dojo.hitch( this, "_handleImage" );
                this.rawimage.src = targetURI;
                
                /*Create image area that will host the diagram image */
                this.imageArea = document.createElement("img");
                if (embbed){
                    this.imageArea.setAttribute("class", rmpw.text.internal.util.ImageSrcUtil.embbedClassName);
                } else {
                    this.imageArea.setAttribute("class", rmpw.text.internal.util.ImageSrcUtil.imageClassName);
                }
                this.imageArea.src= targetURI;
                this.imageArea.id= "imageArea_" + imageId;                
                node.appendChild(this.imageArea);

                this.domNode = editor.domNode;
                this.showMessage(messages.loading);
            },
            
            showMessage: function showMessage(msg) {
            	if (this.msgNode == null) {
            		this.msgNode = document.createElement("div");
            		this.msgNode.style.padding = "125px";
            		this.msgNode.style.position = "absolute";
            		this.msgNode.style.color = "#BBBBBB";
            		this.msgNode.style.fontSize = "2em";
            		this.msgNode.style.top = "50px";
            		this.msgNode.style.left = "400px";
            		this.msgNode.style.background = "white";
            		this.domNode.appendChild(this.msgNode);
            	}
            	this.msgNode.innerHTML = msg;
            },
            	
           	hideMessage: function hideMessage() {
            	if (this.msgNode != null) {
           			this.domNode.removeChild(this.msgNode);
           			delete this.msgNode;
           		}
           	}
            
		});

	};
	
    var defineOpenResourcePlugin = function() {
		var pluginName = "openresource";
		CKEDITOR.plugins.add(pluginName, {
			init: function(editor) {
				// Open resource Command
				editor.addCommand(pluginName, {
					exec: dojo.hitch(this, function() {
						this.openResource(editor);
					}) 
				});

				var openDef = {
						label: messages.openResource,
						command: pluginName,
						group: 'openResource',
						icon: net.jazz.ajax._contextRoot + '/web/rmpw.text/internal/plugins/images/open-linked-artifact.gif',
						order: 1
					};

								
				// Context menu items
				if (editor.addMenuItems) {
					editor.addMenuGroup('openResource',1);
					
					editor.addMenuItems({
						openresource: openDef
					});
				}
				
				if (editor.contextMenu) {
					editor.contextMenu.addListener( function( element, selection )
					{
						if (element && (element.is( 'img' ) || element.is('a'))){
							return { openresource : CKEDITOR.TRISTATE_ON };															
						}
					});
				}
			},
			
			openResource : function(editor){
                // summary:
                //		Navigate to a resource that has been added to the document as either a link
				//      or an image
                // description:
                //		This is a utility method for navigating to a resource inserted into a rich 
				//      text documents as a link or an image. Precedence is on the link.

				var uri, element, lnk;
				var selectedElement = editor.getSelection().getSelectedElement();
				if (selectedElement){
					element = editor.getSelection().getSelectedElement().$;
				} else {
					lnk = CKEDITOR.plugins.link.getSelectedLink( editor );
					if (lnk){
						element = lnk.$;
					} else {
						return;
					}
				}
				if (element.attributes.href){
					uri = element.attributes.href.nodeValue;
				} else if (element.attributes.src) {
					uri = rmpw.text.internal.util.ImageSrcUtil.extractResourceUriFromImage(element.attributes.src.nodeValue, element.attributes["class"].nodeValue);
				}
				window.parent.location.href = uri + "?" + rmpw.text.internal.util.ImageSrcUtil.computeContextParams();
			}

		});

	};


	rmpw.text.internal.plugins.DMCKEditorPlugins.defineDMPlugins = function() {
		defineCustomLinkPlugin();
		defineDiagramPlugin();
		defineOpenResourcePlugin();
	};
	
})();
