/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("rmps.viewlets.internal.queries.QueryResultViewlet");

dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider");
dojo.require("jazz.util.i18n");
dojo.require("rmps.app.core.internal.util.QueryUtil");
dojo.require("rmps.app.core.internal.query.ClientPagedQueryResult");
dojo.require("rmps.app.pickers.internal.QueryPicker");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("rmps.core.query.QueryClient");
dojo.require("rmps.core.query.SearchQuery");
dojo.require("rmps.core.querymodel.QueryModelClient");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.viewlets.internal.SimpleDetails");
dojo.require("rmps.viewlets.internal.PagedTableViewlet");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.viewlets.internal.queries", "QueryViewletMessages");
var messages = dojo.i18n.getLocalization("rmps.viewlets.internal.queries", "QueryViewletMessages");

var I18nUtil = jazz.util.i18n;
var QueryUtil = rmps.app.core.internal.util.QueryUtil;
var QueryClient = rmps.core.query.QueryClient;
var QueryModelClient = rmps.core.querymodel.QueryModelClient;
var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;
var DesignUtil = rmps.app.ui.internal.pages.design.DesignUtil;

var PREF_QUERY_URI = "queryURI";
var PREF_ID = "id";

dojo.declare("rmps.viewlets.internal.queries.QueryResultViewlet", rmps.viewlets.internal.PagedTableViewlet, {

	isClientPaged: true,
	
	_noQueryMsg: null,
	_noQueryExtraMsg: null,
	_queryDef: null,
	_query: null,
	_queryProvider: null,

	getTitleHyperlinkURL: function() {
		var queryURI = this.getPreference(PREF_QUERY_URI);
		if (queryURI) {
			return DesignUtil.getQueryUri(queryURI);
		}
		return null; 
	},

	createEntryWidget: function(data) {
		return new rmps.viewlets.internal.SimpleDetails({
			object: data,
			getProjectPrefix: dojo.hitch(this, this.getProjectPrefix),
			contextURIDef: this._getContextURIDef
		});
	},

	preFetch: function(srh) {
		var queryID = this.getPreference(PREF_ID);
		var queryURI = this.getPreference(PREF_QUERY_URI);
		if (!queryID && !queryURI) {
			this._noQuerySpecified(srh);
			return true; // Abort
		}
		return false;
	},

	fetchData: function(startIndex, count, entries, details, projectId, srh) {
		this._errorMsgNode = null;
		this._query = null;
		
		var def = new dojo.Deferred(); 
		
		var queryID = this.getPreference(PREF_ID);
		var queryURI = this.getPreference(PREF_QUERY_URI);
		
		this._getContextURIDef.addCallback(this, function(contextURI) {
			var queryDef = this._queryDef = queryID ? QueryModelClient.getQueryModelByID(queryID, projectId) : QueryModelClient.getQueryModel(queryURI);
			queryDef.addCallback(this, function(query) {
				this._query = query;
				var searchquery = new rmps.core.query.SearchQuery({
					queryUri: query.uri,
					projectId: projectId,
					limit: 200,
					queryPage : startIndex/count, 
					pageSize: this.getItemsPerPage(),
					name: query.getName(),
					scope: QueryUtil.SCOPE.ALL.value,
					baseline: contextURI
				});			
				
				var searchDef = QueryClient.execute(searchquery);
				searchDef.addCallback(function(result) {
					var queryResult = new rmps.app.core.internal.query.ClientPagedQueryResult({
						query: searchquery,
						id: result.id,
						rawEntries: result.entries,
						totalResults: result.totalResults,
						uri: result.uri
					});
					
					queryResult.sortEntries();
					
					def.callback(queryResult);
				});
				searchDef.addErrback(function(err) {
					def.errback(err);
				});
			});
			queryDef.addErrback(def.errback);			
		});
		this._getContextURIDef.addErrback(def.errback);

		def.addErrback(console.error);
		
		return def;
	},
	
	parseEntries: function(response) {
		// may be overridden by clients
		// return []
		if (response && response.entries && response.entries.length) {
			return dojo.map(response.entries, dojo.hitch(this, function(contextURI, entry) {
				var rdfdesc = entry["rdf:Description"][0];
				var uri = rdfdesc["rdf:about"];
				return {
					resourceUri: uri,
					title: entry.title[0].value,
					getResourceInfo: function(properties, contextURI) {
						return ResourceInfoClient.getResourceInfo([uri], properties, contextURI);
					}
				};
			},this._contextURI));
		}
		return response;
	},
	
	getPreferenceProvider: function(id) {
		if (id === PREF_QUERY_URI) {
			if (!this._queryProvider) {
				this._queryProvider = new rmps.viewlets.internal.queries.QueryResultViewlet._Provider(this);
			}
			return this._queryProvider;
		}
		
		return this.inherited(arguments);
	},
	
	settingsChanged: function(oldSettings, newSettings) {
		this.inherited(arguments);
		
		if (newSettings.preferences[PREF_QUERY_URI] != oldSettings.preferences[PREF_QUERY_URI]) {
			// Update dynamic title when query is changed
			if (this._queryDef) {
				this._queryDef.addCallback(this, function(query) {
					if (query) {
						if (this.getSite().isRemote())
							this.getSite().setTitle(query.getName() + messages.isRemote);
						else
							this.getSite().setTitle(query.getName());						
					}
				});
			}
		}
	},
	

	settingsCanceled: function() {
		if (this._queryProvider) {
			this._queryProvider.cancel();
		}
	},
	
	_noQuerySpecified: function(srh) {
		if (!this._noQueryMsg) {
			// this part always shows
			this._noQueryMsg = document.createElement("span");
			this._noQueryMsg.appendChild(document.createTextNode(messages.noQuery));

			// this part only shows when dashboard in edit mode
			this._noQueryExtraMsg = document.createElement("span");
			var links = I18nUtil.addMessage(this._noQueryExtraMsg, messages.editSettings);
			this.connect(links[0], "onclick", dojo.hitch(this, function(e) {
				e.preventDefault();
				this.getSite().toggleSettings();
			}));
			this._noQueryExtraMsg.className = "editModeText";
			this._noQueryMsg.appendChild(this._noQueryExtraMsg);
			if (!this.getSite().isDashboardEditing()) {
				this._noQueryExtraMsg.style.display = "none";
			}
		}

		this._errorMsgNode = this._noQueryMsg;
		this._totalResults = 0;
		this._pageToken = null;		
		this.updateSecondaryTitle();

		var dataStructure = {
			"entries": null,
			"details": null
		};
		srh.doCallback(dataStructure);
	}
});

dojo.declare("rmps.viewlets.internal.queries.QueryResultViewlet._Provider", com.ibm.team.dashboard.web.ui.CustomPreferenceProvider, {

	_constraintIds: null,
	_viewlet: null,
	_queryCache: null, //{}

	constructor: function(viewlet) {
		this._viewlet = viewlet;
		this._queryCache = {};
		this.cancel();
	},

	_cacheQuery: function(queryURI) {
		var def = QueryModelClient.getQueryModel(queryURI);
		def.addCallback(this, function(query) {
			this._queryCache = {
				uri: queryURI,
				query: query
			};
		});
		def.addErrback(this, function(err) {
			this._queryCache = {
				uri: queryURI,
				query: null
			};
		});
		return def;
	},

	getLabel: function(value) {
		var message = null;
		if (!value) {
			message = messages.noQuerySelected;
		} else if (this._queryCache.uri === value) {
			if (this._queryCache.query) {
				message = this._queryCache.query.label;
			} else {
				message = "";
			}
		}
		
		if (message) {
			return "<input type='text' readonly='readonly' value='" + message + "'/>";
		}
		
		this._cacheQuery(value);
		// Will be called again until a value is returned.
		return null;
	},

	getLoadingLabel: function() {
		return messages.loading;
	},

	createChooser: function(onOk, settings, viewlet) {
		var dialog = null;
		var onOk2 = function(selectedQuery) {
			if (selectedQuery && selectedQuery.uri) {
				viewlet.setPreference(PREF_QUERY_URI, selectedQuery.uri);
			}

			if (dialog) {	
				dialog.close();
				dialog.destroyRecursive();
			}

			onOk(selectedQuery.uri, PREF_QUERY_URI);
		};
		
		var onCancel = function() {
			if (dialog) {	
				dialog.close();
				dialog.destroyRecursive();
			}
		};
		 
		var def = null;
		var queryURI = viewlet.getPreference(PREF_QUERY_URI);
		if (queryURI && this._queryCache.uri !== queryURI) {
			def = this._cacheQuery(queryURI);
		} else {
			def = new dojo.Deferred();
			def.callback(this._queryCache.query || null);
		}
		
		def.addBoth(function(query) {
			var picker = new rmps.app.pickers.internal.QueryPicker({
				projectId: settings.scopeItemId || viewlet._getProjectId(),
				selection: query, 
				onOk: onOk2,
				onCancel: onCancel
			});
			
			var dialogParams = {
	            primaryTitle: messages.queryPreferenceTitle,
		        width: "400px",
		        contentNode: picker.domNode
	        };
			dialog = new jazz.ui.Dialog(dialogParams);
		});
	},

	cancel: function() {
		this._queryURI = this._viewlet.getViewletState().get(PREF_QUERY_URI);
	}
	
});

}());
