/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.viewlets.internal.resource.ResourcePopularityViewlet");

dojo.require("rmps.core.commenting.CommentClient");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.core.util.DateUtils");

dojo.require("rmps.viewlets.internal.resource.TagCloudPresentation");
dojo.require("rmps.viewlets.internal.resource.ListPresentation");
dojo.require("rmps.viewlets.internal.PagedTableViewlet");

dojo.require("jazz.app.context");

dojo.require("dojo.date.stamp");

var DateUtils = rmps.core.util.DateUtils;
var CommentClient = rmps.core.commenting.CommentClient;
var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;

var PREF_PRESENTATION = "presentation";
var PREF_DATE_RANGE = "dateRange";
var PREF_SIZE = "pageSize";


var PRESENTATIONS = {
	list: "list",
	tagCloud: "tagCloud"
};


dojo.declare("rmps.viewlets.internal.resource.ResourcePopularityViewlet", rmps.viewlets.internal.PagedTableViewlet, {

	useJazzResourceLinks: true,
	isClientPaged: true,


	getStartDate: function() {
		var range = this.getPreference(PREF_DATE_RANGE);
		var date = DateUtils.rangeToDate(range);
		if (date) {
			return dojo.date.stamp.toISOString(date);
		}
		return null;
	},

	getSize: function() {
		var size = this.getPreference(PREF_SIZE);
		return size < 0 ? 0 : size;
	},

	getSecondaryTitle: function() {
		var title = this.inherited(arguments);
		var rangeString = DateUtils.rangeToString(this.getPreference(PREF_DATE_RANGE));
		if (rangeString) {
			title += ' ' + rangeString;
		}
		return title;
	},

	createEntryWidget: function(data) {

		var presentation = this.getPreference(PREF_PRESENTATION);
		if (presentation == PRESENTATIONS.tagCloud) {
			return contentWidget = new rmps.viewlets.internal.resource.TagCloudPresentation(
					{
						results: data, 
						getProjectPrefix: dojo.hitch(this, this.getProjectPrefix), 
						contextURIDef: this._getContextURIDef
					});
		} else { // if (presentation == PRESENTATIONS.list) { // default
			return new rmps.viewlets.internal.resource.ListPresentation(
					{
						results: data,
						getProjectPrefix: dojo.hitch(this, this.getProjectPrefix),
						contextURIDef: this._getContextURIDef,
						useJazzResourceLinks: this.useJazzResourceLinks
					});
		}
		
	},

	fetchData: function(startIndex, count, entries, details, projectId) {
		var deferred = new dojo.Deferred();

		var size = this.getSize();
		this._getContextURIDef.addCallback(this, function(contextURI) {
			var commentDef = CommentClient.getMostCommented(projectId, size, this.getStartDate(), contextURI);
			commentDef.addCallback(this, function(resources) {
				if (resources.length == 0){
					// build empty result - we just create a result as our viewers will tackle the list of results (Tag cloud viewer)
					var result = [];
					result.pageToken = "";
					this._totalResults = 0;
					result.totalResults = this._totalResults;
					deferred.callback(result);
					return;
				}
				
				var resourceUris = [];
				for (var key in resources) {
					resourceUris.push(key);
				}
				
				var entries = [];

				var properties = ['shortName', 'iconUri', 'longName'];
				var presentation = this.getPreference(PREF_PRESENTATION);
				if (this.useJazzResourceLinks && presentation !== PRESENTATIONS.tagCloud) {
					properties = ['shortName', 'iconUri'];
				}
				
				var infoDeferred = ResourceInfoClient.getResourceInfo(resourceUris, properties, contextURI);
	
				infoDeferred.addCallback(this, function(infos) {
					function comparator(a, b) {
						if (a.value == b.value) {
							var aName = a.info.shortName.toLowerCase();
							var bName = b.info.shortName.toLowerCase();
							return (aName < bName) ? -1 : ((aName > bName) ? 1 : 0);
						}
						return (a.value > b.value) ? -1 : 1;
					}

					for (var i = 0; i < infos.length; ++i) {
						var value = resources[infos[i].uri];
						if (value > entries.totalResults){
							entries.totalResults = value;
						}
						var obj = {
							info: infos[i],
							value : value
						};
						entries.push(obj);
					}
					this._totalResults = entries.length;
					
					// sort results
					entries.sort(comparator);
					
					// build result - we just create a result as our viewers will tackle the list of results (Tag cloud viewer)
					var result = [];
					result.pageToken = "";
					result.totalResults = this._totalResults;
					result.push(entries);
					deferred.callback(result);
					
				});
	
				infoDeferred.addErrback(this, this._handleError);
			});
		});
		this._getContextURIDef.addErrback(deferred.errback);
		deferred.addErrback(this, this._handleError);
		
		return deferred;
	}
	
});

})();