/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("rmps.viewlets.internal.reviews.ReviewsViewlet");

dojo.require("rmps.core.review.ReviewClient");
dojo.require("rmps.core.util.DateUtils");

dojo.require("rmps.viewlets.internal.reviews.SimpleReviewDetails");

dojo.require("jazz.app.context");

dojo.require("rmps.viewlets.internal.PagedTableViewlet");

dojo.require("dojo.date.stamp");
dojo.require("rmps.core.util.DateUtils");

dojo.require("rmps.core.review.util.ReviewTypes");
dojo.require("jazz.app.context");
dojo.require("rmps.core.user.UserClient");
dojo.require("rmps.core.review.util.ReviewUtil");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var DateUtils = rmps.core.util.DateUtils;
var ReviewClient = rmps.core.review.ReviewClient;
var ReviewTypes = rmps.core.review.util.ReviewTypes;
var UserClient = rmps.core.user.UserClient;
var ReviewUtils = rmps.core.review.util.ReviewUtil;

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.viewlets.internal.reviews", "ReviewFilterMessages");
var messages = dojo.i18n.getLocalization("rmps.viewlets.internal.reviews", "ReviewFilterMessages");

// viewlet preferences
var PREF_SCOPE_TYPE = "_scope";
var PREF_SCOPE_ITEM = "_scopeItem";
var PREF_DATE_RANGE = "dateRange";
var PREF_FILTER = "filter";

// preference values
var PREFV_CONTRIBUTER_SCOPE = "CONTRIBUTOR";
var PREFV_PROJECT_SCOPE = "PROJECT_AREA";

dojo.declare("rmps.viewlets.internal.reviews.ReviewsViewlet", rmps.viewlets.internal.PagedTableViewlet, {

	userCache: null,
	isClientPaged: true,

	postCreate: function() {
		this.inherited(arguments);
		this.userCache = {};
	},
	
	getUser: function() {
		var scopeType = this._site.context.getPreferences()[PREF_SCOPE_TYPE];
		var filter = this.getPreference(PREF_FILTER);
		if (filter === "myReviews"){
			var userId = jazz.app.currentApplication ? jazz.app.currentApplication.config.user.userId : null;
			if (!userId) {
				var contributor = dojo.getObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR");
				userId = contributor ? contributor.userId : null;
			}
			return {userId: userId};
		} else if (scopeType === PREFV_CONTRIBUTER_SCOPE) {
			var userUUID = this._site.context.getPreferences()[PREF_SCOPE_ITEM];;
			
			if (!this.userCache[userUUID]) {
				var srh = new ServiceResponseHandler(this, "_getUserFromUUID", "_getUserFromUUIDError");
				com.ibm.team.repository.web.client.internal.AdminClient.getContributorByUUID({uuid: userUUID}, srh);

				this.userCache[userUUID] = { def: new dojo.Deferred() };
			}
			return this.userCache[userUUID];
		} else {
			return null;
		}
	},
	
	_getUserFromUUID: function(response) {
		this.userCache[response.itemId].userId = response.userId;
		
		if (this.userCache[response.itemId].def) {
			this.userCache[response.itemId].def.resolve(this.userCache[response.itemId]);
			 this.userCache[response.itemId].def = null;
		}
	},
	
	_getUserFromUUIDError: function(response) {
		if (this.userCache[response.itemId].def) {
			this.userCache[response.itemId].def.resolve(null);
			 this.userCache[response.itemId].def = null;
		}
	},

	getStartDate: function() {
		var range = this.getPreference(PREF_DATE_RANGE);
		var date = DateUtils.rangeToDate(range);
		if (date) {
			return dojo.date.stamp.toISOString(date);
		}
		return null;
	},

	getSecondaryTitle: function() {
		var title = this.inherited(arguments);
		var reviewPrefName = this.getPreference(PREF_FILTER);
		if (reviewPrefName) {
			title += ' ' + messages[reviewPrefName];
		}
		return title;
	},

	createEntryWidget: function(data) {
		return new rmps.viewlets.internal.reviews.SimpleReviewDetails({
				revSummary: data,
				getProjectPrefix: dojo.hitch(this, this.getProjectPrefix),
				contextURIDef: this._getContextURIDef,
				webURL: this._webURL
			});
	},

	fetch: function (/* ServiceResponseHandler */ srh, /* int */ startIndex, /* int */ count, /* boolean */ entries, /* boolean */ details) {
		if (count <= 0) {
			srh.doCallback();
			return;
		}
		this._totalResults = -1;
		
		this.fetchInternal(srh, startIndex, count, entries, details, this._getProjectId());
	},
	
	fetchData: function(startIndex, count, entries, details, projectId) {
		this._errorMsgNode = null;

/*	TODO: Changes on server side are needed, that will returns paged result...
		var reviewDef = ReviewClient.getReviews(projectId, this.getUser(), startIndex/count, count, this._pageToken);
*/
		
		var user = this.getUser();
		var userId;
		var reviewDef;
		
		if (user !== null) {
			if (user.def) {
				reviewDef = user.def.addCallback(function(user) {
					var userId = user.userId ? user.userId : null;
					
					return ReviewClient.getReviews(projectId, userId);
				});
			} else {
				userId = user.userId ? user.userId : null;
			}
		} else {
			userId = null;
		}
		
		if (!reviewDef) {
			reviewDef = ReviewClient.getReviews(projectId, userId);
		}
		this._getContextURIDef.addCallback (this, function(contextURI) {
			
			
			reviewDef.addCallback(function(reviewSummaries) {
				function comparator(a, b) {
					if(!a.dueDate|| !b.dueDate){
						return 1;
					}
					var ta = a.dueDate.getTime();
					var tb = b.dueDate.getTime();
					return ta < tb ? 1 : ta == tb ? 0 : -1;
				}
				var reviews = [];
				for (var i = 0; i < reviewSummaries.length; i++) {
					var revSummary = reviewSummaries[i];
					if (revSummary.status == ReviewTypes.REVIEW_STATE_TYPES.Draft) {
						var userUri = userId ? ReviewUtils.decodeUserUri(userId) : null;
						var sumUri = ReviewUtils.decodeUserUri(revSummary.moderator);
						if (sumUri == userUri ){
							reviews.push(revSummary);					
						}
					} else {
						reviews.push(revSummary);
					}			
				}					
				reviews.sort(comparator);
				
			});
			reviewDef.addErrback(console.error);
		});
		this._getContextURIDef.addErrback(reviewDef.errback);
	
		
			
		return reviewDef;
	},

});

})();
