/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.viewlets.internal.reviews.SimpleReviewDetails");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.html");

dojo.require("rmps.app.core.util");
dojo.require("rmps.core.resource.ResourceInfo");
dojo.require("rmps.app.ui.ReviewEditorUtil");

dojo.require("rmps.viewlets.internal.comments.FullCommentDetails");

dojo.require("rmps.ui.resource.ResourceIconLabel");
dojo.require("rmps.ui.EllipsisLabel");
dojo.require("rmps.ui.user.UserLabel");

dojo.require("rmps.core.util.DateUtils");
var DateUtils = rmps.core.util.DateUtils;

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.viewlets.internal.reviews", "SimpleReviewMessages");
var messages = dojo.i18n.getLocalization("rmps.viewlets.internal.reviews", "SimpleReviewMessages");


dojo.declare("rmps.viewlets.internal.reviews.SimpleReviewDetails", [dijit._Widget, dijit._Templated], {

	templateString: dojo.cache("rmps.viewlets", "internal/reviews/templates/SimpleReviewDetails.html"),
	widgetsInTemplate: true,

	// attach points
	resourceIconLabel: null,
	timestamp: null,
	author: null,
	pane: null,
	subject: null,
	role: null,
	modified: null,
	dueDate: null,
	_marker: null,

	// mixin properties
	revSummary: null,
	contextURIDef: null,
	useJazzResourceLinks: true,
	webURL: null,

	// private properties
	_controller: null,
	_timestampHandle: null,
	_popupContents: null,
	_reviewPopup: null,
	_icon: null,
	_title: null,
	_rilReady: false,

	postCreate: function() {
		// creator
		this.revSummary.getModerator().addCallback(dojo.hitch(this, function(user) {
			this.author.setUser(user);
			if (this._reviewPopup && !this._reviewPopup._isOpen) {
				this._reviewPopup.destroy();
				this._reviewPopup = this.createReviewPopup(this.domNode);
			}
		}));

		// status
		var status = dojo.doc.createElement("a");
		var image = dojo.create("img", null, status);
		_icon = "review-" + this.revSummary.status.toLowerCase() + ".gif";
		image.src=dojo.moduleUrl("rmps.core.review.util", "graphics/" + _icon);
		image.title = messages[this.revSummary.status];
		this.status.appendChild(status);

		// review info
		this.connect(this.resourceIconLabel, "onUpdate", dojo.hitch(this, function() {
			this._rilReady = true;
			this.resize({w: this.domNode.offsetWidth});
		}));
		
		var url = null;
		var projectName = this.revSummary.owningProjectName;
		if (projectName) {
			url = this.webURL + "/web/projects/" + encodeURIComponent(projectName);	
		} else {
			url = this.getProjectPrefix();	
		}
		
		var def = this.contextURIDef;
		if (!def) {
			def = new dojo.Deferred();
			def.callback(null);
		}
		
		def.addBoth(dojo.hitch(this, function(contextURI) {		
			var query = {};
			if (dojo.config.isDebug) {
				query.debug = dojo.config.isDebug;	
			}
			if (contextURI) {
				query["rmps.context"] = contextURI;			
			} else {
				if (this.revSummary.context) {
					query["rmps.context"] = this.revSummary.context.uri;	
				} 
			}
			var querystr = dojo.objectToQuery(query);
			if (querystr) {
				url += '?' + querystr;
			}
			url += rmps.app.core.util.getDesignViewResourceUri(this.revSummary.link);
			this.resourceIconLabel.linkHref = url;
			this.resourceIconLabel.hasJazzResourceLink = this.useJazzResourceLinks;
			
			var appendTs = new Date().getTime();
			// Using the review uri as link and delegating the construction
			// of the full URL to the review service
			//this.resourceIconLabel.linkHref = this.revSummary.link;
			this.resourceIconLabel.setResourceInfo(this.revSummary.getReviewInfo());
	
			// date stamp diff
			var modified = this.revSummary.updated;
			this.timestamp.appendChild(dojo.doc.createTextNode(DateUtils.getDateDiff(modified)));
	
			this._timestampHandle = DateUtils.addDateDiffUpdater(modified,
				dojo.hitch(this, function(newDateRange) {
					dojo.html._setNodeContent(this.timestamp, dojo.doc.createTextNode(newDateRange), true);
				}));
	
			// modified
	//		this.modified.appendChild(dojo.doc.createTextNode(messages["modified"] + " "));
	
			// due date
			var dateStr = dojo.date.locale.format(new Date(this.revSummary.dueDate), {formatLength: "medium", selector: "date"});
			this.dueDate.appendChild(dojo.doc.createTextNode(dateStr));
			this.dueDate.title= messages.dueDate;
	
			// role
			var a = dojo.doc.createElement("a");
			var image = dojo.create("img", null, a);
			if ( this.revSummary.role == null){
				_icon = "role-not-reviewer.gif";
				_title = messages.NotReviewer;
			} else if (this.revSummary.role == "OptionalReviewer"){
				_icon = "role-opt-reviewer.gif";
				_title = messages.OptionalReviewer;
			} else {
				_icon = "role-" + this.revSummary.role.toLowerCase() + ".gif";
				_title = messages[this.revSummary.role];
			}
			image.src=dojo.moduleUrl("rmps.core.review.util", "graphics/" + _icon);
			image.title= _title;
			this.role.appendChild(a);
	
	
	
	//		this._reviewPopup = this.createReviewPopup(this.domNode);
	
			this.connect(this.resourceIconLabel, "linkClicked", function(e) {
				dojo.removeClass(this.pane, "pane-hover");
			});
	
			this.connect(this.author, "initUserInfo", function(userInfo) {
				userInfo.emailProperties.setSubject("RE: " + this.revSummary.name);
			});
		}));
	},

	destroy: function() {
		if (this._popupContents) {
			this._popupContents.destroyRecursive();
			delete this._popupContents;
		}
		if (this._reviewPopup) {
			this._reviewPopup.destroyRecursive();
			delete this._reviewPopup;
		}
		if (this._timestampHandle) {
			DateUtils.removeDateDiffUpdater(this._timestampHandle);
			delete this._timestampHandle;
		}
		this.inherited(arguments);
	},

	resize: function(size) {
		if (size && size.w) {
			this.domNode.style.width = size.w + "px";
			if (this._rilReady) {
//				var subjectSize = {w: size.w - this._marker.offsetLeft - this._marker.offsetWidth - 1};
//				this.subject.resize(subjectSize);
			}
		}
	},
/*
	createReviewPopup: function(node) {
		return new jazz.ui.HoverPopup({
			around: this.domNode,
			createContents: dojo.hitch(this, this._createPopupContents)
		});
	},

	_createPopupContents: function(popupWidget) {
		this._popupContents = new rmps.viewlets.internal.comments.FullCommentDetails({
			revSummary: this.revSummary,
			getProjectPrefix: dojo.hitch(this, this.getProjectPrefix)
		});
		this._popupContents.showFullLabels(true);

		var temp = dojo.doc.createElement("div");
		dijit.popup.moveOffScreen(temp);
		temp.appendChild(this._popupContents.domNode);
		this._popupContents.startup();
		temp.removeChild(this._popupContents.domNode);
		dojo.destroy(temp);

		return this._popupContents.domNode;
	},
*/
	_formatDate: function(date) {
		var dateStr = dojo.date.locale.format(date, {formatLength: "medium", selector: "date"});
		return dateStr;
	},

	// events
	_onPaneMouseEnter: function(event) {
		dojo.addClass(this.pane, "pane-hover");
	},
	

	_onPaneMouseLeave: function(event) {
		dojo.removeClass(this.pane, "pane-hover");
	}
});

})();
