/*
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 */

(function(){
    dojo.provide("rmpw.text.internal.widgets.CKEditorFactory");
	dojo.require("ck.core");
	dojo.require("ck.plugins.a11yhelp");
	dojo.require("ck.plugins.bidi");
	dojo.require("ck.plugins.colorbutton");
	dojo.require("ck.plugins.find");
	dojo.require("ck.plugins.font");
	dojo.require("ck.plugins.format");
	dojo.require("ck.plugins.image");
	dojo.require("ck.plugins.indentlist");
	dojo.require("ck.plugins.justify");
	dojo.require("ck.plugins.link");
	dojo.require("ck.plugins.list");
	dojo.require("ck.plugins.liststyle");
	dojo.require("ck.plugins.removeformat");
	dojo.require("ck.plugins.specialchar");
	dojo.require("ck.plugins.stylescombo");
	dojo.require("ck.plugins.table");
	dojo.require("ck.plugins.tableresize");
	dojo.require("rmpw.text.internal.plugins.DMCKEditorPlugins");

	// Register DM custom plugins
	rmpw.text.internal.plugins.DMCKEditorPlugins.defineDMPlugins();

	// Default plugins
	// Already added: "basicstyles,clipboard,floatingspace,enterkey,toolbar,undo,wysiwygarea"
	// Last three custom DM plugins
	var extraPlugins = 'a11yhelp,bidi,colorbutton,find,format,font,image,' +
		'indentlist,justify,link,list,liststyle,removeformat,specialchar,' +
		'stylescombo,table,tableresize,customLink,insertdiagram,openresource'; 
   
    // CKEditor Configuration objects...
    var EditorTypes = rmpw.text.internal.widgets.CKEditorFactory.EditorTypes = {};

	// Toolbars
    var commentToolbarItems = [['Source', '-', 'Font', 'FontSize', '-', 'Bold', 'Italic', 'Underline', 'Strike', '-', 'TextColor', 'BGColor', '-', 'RemoveFormat', '-', 'JustifyLeft', 'JustifyCenter', 'JustifyRight', 'JustifyBlock', 'Outdent', 'Indent', '-', 'BulletedList', 'NumberedList', 'Table', 'Paste', 'PasteText', '-', 'CustomLink', 'Unlink']];
    var toolbarItems = [['Source', '-', 'Format', 'Font', 'FontSize', '-', 'Bold', 'Italic', 'Underline', 'Strike', 'Subscript', 'Superscript', '-', 'Cut', 'Copy', 'Paste', 'PasteText', '-', 'Find']];
	toolbarItems.push(['TextColor', 'BGColor', '-', 'RemoveFormat', '-', 'JustifyLeft', 'JustifyCenter', 'JustifyRight', 'JustifyBlock', 'Outdent', 'Indent', '-', 'BulletedList', 'NumberedList', 'Table', '-', 'BidiLtr', 'BidiRtl', '-', 'InsertDiagram', 'Image', '-', 'CustomLink', 'Unlink']);
    var slimToolbarItems = [['Source', '-', 'Font', 'FontSize', '-', 'Bold', 'Italic', 'Underline', '-', 'Cut', 'Copy', 'Paste', '-', 'BulletedList', 'NumberedList', '-', 'CustomLink', 'Unlink']];
    if (!djConfig.isDebug) {
        commentToolbarItems.splice(0, 2);
        toolbarItems[0].splice(0, 2);
        slimToolbarItems[0].splice(0, 2);
    }

	// Common configuration
    var BASE_CONFIG = {
        coreStyles_bold: {
            element: "b" // use a <b> instead of a <strong>
        },
        coreStyles_italic: {
            element: "i" // use an <i> instead of an <em>
        },
        coreStyles_strike: {
            element: "del"
        },
		//skin: "rational",
        language: djConfig.locale, // for FF
        enableTabKeyTools: true,
        menu_groups: "clipboard,tablecell,tablecellproperties,tablerow,tablecolumn,table,insert,glossary,customlink,openresource,diagram,externalIntegration,edit",
        fontSize_sizes: "8/8pt;9/9pt;10/10pt;11/11pt;12/12pt;14/14pt;16/16pt;18/18pt;20/20pt;22/22pt;24/24pt;26/26pt;28/28pt;36/36pt;48/48pt;72/72pt",
        font_defaultLabel: "Arial",
        fontSize_defaultLabel: "10"
    };

    EditorTypes.COMMENT_EDITOR = dojo.delegate(BASE_CONFIG, {
		toolbar: commentToolbarItems,
        toolbarCanCollapse: false
    });
    
    EditorTypes.RICH_TEXT_ARTIFACT_EDITOR = dojo.delegate(EditorTypes.COMMENT_EDITOR, {
        toolbar: toolbarItems,
        format_tags: 'p;h1;h2;h3;h4',
        extraPlugins: extraPlugins,
    });
    
    EditorTypes.RICH_TEXT_SLIM_EDITOR = dojo.delegate(EditorTypes.COMMENT_EDITOR, {
        toolbar: slimToolbarItems,
        format_tags: 'p;h1;h2;h3;h4',
        extraPlugins: extraPlugins, 
    });
    
    EditorTypes.RICH_TEXT_NO_TOOLBAR_EDITOR = dojo.delegate(EditorTypes.COMMENT_EDITOR, {
        toolbar: [[]],
        format_tags: 'p;h1;h2;h3;h4',
        extraPlugins: extraPlugins, 
    });
    
    //
    //      The "factory" exposes a single function called create().
    //
    //      sample usage would be :
    //		dojo.require("com.ibm.rdm.web.editor.CKEditorFactory");
    //		...
    //		...
    //		var nodeToReplaceWithEditor = dojo.byId("foo"),
    //			types = com.ibm.rdm.web.editor.CKEditorFactory.EditorTypes,
    //			onCreate = dojo.hitch(this, function (ckeditorInstance) {
    //				this.localCKEditorReference = ckeditorInstance;
    //			});
    //
    //		com.ibm.rdm.web.editor.CKEditorFactory.create(nodeToReplaceWithEditor, types.COMMENT_EDITOR, onCreate);
    //			
 	
    rmpw.text.internal.widgets.CKEditorFactory.create = function(/*Element*/node, /*CKEDITOR.configuration*/ options, /*Function*/ callback, /*Boolean?*/ append){

		var cb = null;
        if (!append) {
             cb = CKEDITOR.replace(node, options);
        }
        else {
            cb = CKEDITOR.appendTo(node, options);
        }
        if (callback) {
            callback(cb);
        }
    };
    
})();
