/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
    dojo.provide("rmpw.text.internal.widgets.RichTextPanel");
    
    dojo.require("dijit.form._FormWidget");
    dojo.require("dijit.layout.ContentPane");    
	dojo.require("rmpw.text.internal.util.Converter");
	dojo.require("rmps.core.commenting.range.IDList");
	dojo.require("rmpw.text.internal.util.ImageSrcUtil");
	dojo.require("dojo.i18n");
	
	dojo.requireLocalization("rmpw.text.internal.widgets", "Messages");
	var messages = dojo.i18n.getLocalization("rmpw.text.internal.widgets", "Messages");
    
	/**
 	* Widget used for preview mode of rich-text document. Supports association of text elements with comments.
 	* 
 	* @author lgrahek
 	*/
	dojo.declare("rmpw.text.internal.widgets.RichTextPanel", [dijit.layout.ContentPane], {

		templateString: dojo.cache("rmpw.text", "internal/widgets/templates/RichTextPanel.html"),
		"class": "dm-rich-text-editor",
		selectedNodes: [], // keeps track of selected nodes (alternative: search the document by css class every time it's needed)
		
		startup: function(){
			this.inherited(arguments);

			dojo.connect(this, 'onClick', function(arg) {
				// check if the selected node has id, if not, find the first parent that does
				var selectedNode = arg.target;
				while (selectedNode && !selectedNode.id) {
					selectedNode = selectedNode.parentElement;
				}
				if (!selectedNode || (selectedNode.className && selectedNode.className.indexOf("dm-rich-text-editor") > -1)) {
					// probably newly created tag and no refresh has been done yet 
					// (unlikely to happen - commenting on just created text; will be possible after refresh)
					return;
				}
				var idxInSelectedNodes = dojo.indexOf(this.selectedNodes, selectedNode);
				var alreadySelected = idxInSelectedNodes > -1;
				if (!arg.ctrlKey) {
					// deselect all
					this.deselectAll();
					if (!alreadySelected) {
						// and select the new one, unless already selected 
						// (clicking on the selected element deselects it)
						this.selectElement(selectedNode);
					}					
				} else if (!alreadySelected) {
					// append to the current selection
					this.selectElement(selectedNode);				
				} else {
					// deselect just the new one
					// (clicking on the selected element deselects it)
					this.selectedNodes.splice(idxInSelectedNodes, 1);
					dojo.removeClass(selectedNode, "richTextNodeSelected");
				}
			});
			
        	this._commentingWidget = this.form._resource.page.editor.commentingWidget;
        	this.subscribe();
		},
		
		_getReferenceValueAttr: function(){
			// always return the original value from the form to prevent the panel content change 
			// from creating a vcs change
			return this.form.target["dmxhtml_body"];
		},
		
		_setReferenceValueAttr: function(value){
			if (this.isVisible()) {
				this.setXhtmlValue(rmpw.text.internal.util.Converter.parse(value, true));
			}
		},
		
		setXhtmlValue: function(xhtmlValue) {
			this.set('content', xhtmlValue);
			
			// check if there are any images to update their src 
			var dmImages = dojo.query("[class='dm-document-image']");
			this.replaceImageSrc(dmImages, rmpw.text.internal.util.ImageSrcUtil.imageClassName);
			dmImages = dojo.query("[class='dm-document-embbed']");
			this.replaceImageSrc(dmImages, rmpw.text.internal.util.ImageSrcUtil.embbedClassName);			
			
			this.resize();			
		},
		
		_getReadOnlyAttr: function(){
			return true;
		},
        
		_setReadOnlyAttr: function(value){
			// do nothing, read-only by design;
		},
		
		replaceImageSrc: function(dmImages, className) {
			if (dmImages) {
				for (var i = 0, dmImage; dmImage = dmImages[i]; i++) {
					var realUri = dmImage.attributes.src;
					if (realUri.textContent) {
						realUri = realUri.textContent;
					}
					var newSrc = rmpw.text.internal.util.ImageSrcUtil.getImageFromResourceUri
								(realUri, className);
					dojo.attr(dmImage, "src", newSrc);
				}
			}			
		},
		
		resize: function(changeSize, resultSize) {			
			// Calculate using the similar way as RichTextWidget is
			var myDim = dojo.position(this.containerNode, true);
			var bodyHeight = document.body.offsetHeight;
			var height = bodyHeight - myDim.y - 52;
			// Max height might not be set, need to account for that when limiting the height.
			if(!this.maxHeight || (this.maxHeight && height < this.maxHeight)){
				this.containerNode.style.height = height + "px";
			}
			this.inherited(arguments); 			
		},
		
		setVisible: function(visible) {
			this.domNode.style.display = visible ? "" : "none";	
		},
		
		isVisible: function() {
			return this.domNode.style.display === "";	
		},
		
		deselectAll: function() {
			if (this.selectedNodes.length > 0) {
				for (var i = 0, selectedNode; selectedNode = this.selectedNodes[i]; i++) {
					dojo.removeClass(selectedNode, "richTextNodeSelected");
				}
				this.selectedNodes = [];
			}
		},
		
		selectElement: function(element) {
			this.selectedNodes.push(element);
			dojo.addClass(element, "richTextNodeSelected");
		},
		
		deselectElement: function(element) {
			var index = this.selectedNodes.indexOf(element);
			this.selectedNodes.splice(index, 1);
			dojo.removeClass(element, "richTextNodeSelected");
		},
		
		subscribe: function() {
			this.connectCommenting("onSelectionChanged", this.commentSelected);
			this.connectCommenting("onStartCreateComment", this.startCreateComment);
			this.connectCommenting("onEndEditingComment", this.endComment);
		},
		
		connectCommenting: function(action, method) {
			this.connect(this._commentingWidget, action, method);
		},
		
		startCreateComment: function(comment) {
			if (!this.form.bodyUsingRdfAbout && this.selectedNodes.length > 0) {
				// saved in pre-402 format, using anonymous classes so text elements cannot be identified;
				// workaround - modify and save the document; issue the message about that
				var div = dojo.create("div", { innerHTML: "<p style='margin: 10px'>" +  messages.oldFromatMsg + "</p>" });		
				this.dialog = new jazz.ui.Dialog({
					contentNode: div,
					primaryTitle: messages.warning,
					closeOnBlur: true
				});
			}			
		},
		
		endComment: function(comment) {
			if (!this.form.bodyUsingRdfAbout) {
				// saved in pre-402 format, using anonymous classes so text elements cannot be identified;
				return;
			}
			// tie all currently selected text nodes to the comment, if any	
			if (this.selectedNodes.length === 0) {
				return;
			}				
			var elements = [];
			for (var i = 0, selectedNode; selectedNode = this.selectedNodes[i]; i++) {
				elements.push(selectedNode.id);	
			}
			var range = new rmps.core.commenting.range.IDList(elements);
			comment.range = [ range ];
		},
		
		commentSelected: function(sel, previous, comments, deselected) {
			if (!this.isVisible()) {
				// don't react when not active
				return;
			}
			
			if (!comments || comments.length === 0) {
				// nothing to select
				if (deselected && deselected.length > 0 && this.selectedNodes) {
					// deselect nodes in selectedNodes that match deselected comments
					var nodesToDeselect = [];
					for (var i = 0; i < this.selectedNodes.length; i++) {
						var selectedId = this.selectedNodes[i].id;
						var found = false;
						for (var j = 0; (j < deselected.length) && !found; j++) {
							var deselectedComm = deselected[j];
							if (deselectedComm.range.length > 0) {
								for (var k = 0; (k < deselected.length) && !found; k++) {
									var id = deselectedComm.range[0].ids[k];
									if (selectedId === id) {
										nodesToDeselect.push(this.selectedNodes[i]);
										found = true;
									}
								}
							}
						}
					}
					for (var i = 0, nodeToDeselect; nodeToDeselect = nodesToDeselect[i]; i++) {
						this.deselectElement(nodeToDeselect);
					}
				} // otherwise, probably just clicked on new comment button
				return;
			}
			// comment is selected, remove current selection first
			this.deselectAll();
			
			// based on the range field of each comment - select the elements with-in the rich text accordingly.
			var element;
			for (var i = 0, comment; comment = comments[i]; i++) {
				if (comment.range.length > 0) {
					for (var j = 0, id; id = comment.range[0].ids[j]; j++) {
						element = dojo.byId(id);
						if (element && (dojo.indexOf(this.selectedNodes, element) === -1)) {							
							this.selectElement(element);
						}
					}
				} else {
					// comment not tied to any specific paragraph, refers to the whole document
					this.selectElement(this.domNode);
				}
			}
			if (element) {
				element.scrollIntoView();
			}
		}
    });
})();
