/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("rmpw.text.internal.widgets.RichTextWidget");
    
    dojo.require("dijit.form._FormWidget");
    dojo.require("dijit.layout._LayoutWidget");
    dojo.require("dojo.cache");    
    dojo.require("rmpw.text.internal.widgets.CKEditorFactory");
	dojo.require("rmpw.text.internal.util.Converter");
	dojo.require("rmpw.text.internal.util.ImageSrcUtil");
	
    dojo.declare("rmpw.text.internal.widgets.RichTextWidget", [dijit.form._FormValueWidget, dijit.layout._LayoutWidget], {
    
        templateString: dojo.cache("rmpw.text", "internal/widgets/templates/RichTextWidget.html"),
        _editorReady: false,
        onEditorReadyDeferred: null,
        _deferredLayout: false,
        disabled: false,
		_localValue: null,
		_holdValue: null,
		_editingReferenceValue: false,
		editorType: 'RICH_TEXT_SLIM_EDITOR',
		maxHeight: null,
		isInRichDocument: false, // set to true by the RichTextChangeMixin form that creates it, to indicate it belongs to the rich text document
		
		// Needed to enable/disable cut & paste menus when the editor is read-write/read-only
		_cutMenu: null,
		_pasteMenu: null,
        constructor: function(){
            this.onEditorReadyDeferred = new dojo.Deferred();
            this.inherited(arguments);
        },
        
        destroy: function() {
        	this.uninitialize();
        },
        
        postMixInProperties: function(){
            // summary:
            //		Called after the parameters to the widget have been read-in,
            //		but before the widget template is instantiated. Especially
            //		useful to set properties that are referenced in the widget
            //		template.
            // tags:
            //		protected
            this.inherited(arguments);
        },
        
        postCreate: function(){
            // summary:
            //		Processing after the DOM fragment is created
            // description:
            //		Called after the DOM fragment has been created, but not necessarily
            //		added to the document.  Do not include any operations which rely on
            //		node dimensions or placement.
            // tags:
            //		protected
            this.inherited(arguments);
        },
        
        startup: function(){
            this.inherited(arguments);
            //NGB Inserted the setTimeout call on the actual CKEditor startup due to rendering
            //issues on generated forms if this delay isn't introduced.  Must be some issues
            //relating to DOM insertion that caused the CKEditor not to get inserted into the DOM
            //without this delay.
            setTimeout(dojo.hitch(this, this._startup), 100);
        },
        _startup: function(){
			var type = rmpw.text.internal.widgets.CKEditorFactory.EditorTypes[this.editorType];
			if (!type) {
				type = rmpw.text.internal.widgets.CKEditorFactory.EditorTypes.RICH_TEXT_SLIM_EDITOR;
			}
			
			/*
			 * While in development, there was an intermittant problem of the document
			 * editor suddenly expanding infinitely.  Traced this to a strange order in the
			 * eventing model.  maxHeight is to prevent an sudden infinite expansion of the
			 * rich text editor.
			 */  
			if(!this.maxHeight){
				var maxHeightCss = this.domNode.style.maxHeight;
				this.maxHeight = maxHeightCss ? maxHeightCss : "2560"; 
			}
			
            rmpw.text.internal.widgets.CKEditorFactory.create(this.editor, type, dojo.hitch(this, function(ckeditor){
                this.ckeditor = ckeditor;
                this.ckeditor.domNode = this.domNode;
				this.ckeditor.richTextWidget = this;
                this.ckeditor.on("instanceReady", this._instanceReadyCallback, this);
            }), false);

			// listen to comments events, but only in rich text document, not otherwise	
			if (this.isInRichDocument) {
				this._commentingWidget = this.form._resource.page.editor.commentingWidget;
				this.subscribe();
			}
        },
        
        _instanceReadyCallback: function(event){
            this.ckeditor.removeListener("instanceReady", this._instanceReadyCallback);
            this._editorReady = true;
            if (this.onEditorReadyDeferred) {
                this.onEditorReadyDeferred.callback(this);
            }

            if (dojo.isIE) {
            	// Need a timeout with IE
				setTimeout(this.ckeditor.resetDirty, 10);
			} else {
				this.ckeditor.resetDirty();				
			}
            this._addDirtyStateListeners();
        },
        
        _addDirtyStateListeners: function(){
            // summary:
            //		Add listeners to key press and certain other ckeditor events to allow
            //		propagation of dirty state to the _FormWidget API via an onChange() call.
            this.ckeditor.on("key", this._checkDirtyAfterTimeout, this);
            this.ckeditor.on("afterCommandExec", this._checkDirtyAfterTimeout, this);
			this.ckeditor.on('insertHtml', this._checkDirtyAfterTimeout, this);
			this.ckeditor.on('insertElement', this._checkDirtyAfterTimeout, this);
			this.ckeditor.on('dialogHide', this._checkDirtyAfterTimeout, this);
			this.ckeditor.on('paste', this._onPaste, this);
			this.ckeditor.on('dataReady', this._checkSize, this);
        },
        _removeDirtyStateListeners: function(){
            // summary:
            //		Remove listeners to key press and certain other ckeditor events used for
            //		dirty state propagation.
            this.ckeditor.removeListener("key", this._checkDirtyAfterTimeout);
            this.ckeditor.removeListener("afterCommandExec", this._checkDirtyAfterTimeout);
			this.ckeditor.removeListener('insertHtml', this._checkDirtyAfterTimeout);
			this.ckeditor.removeListener('insertElement', this._checkDirtyAfterTimeout);
			this.ckeditor.removeListener('dialogHide', this._checkDirtyAfterTimeout);
			this.ckeditor.removeListener('paste', this._onPaste);
			this.ckeditor.removeListener('dataReady', this._checkSize);
			
			if (this._checkUncollapseInterval) {
				clearInterval(this._checkUncollapseInterval);
				delete this._checkUncollapseInterval;
			}
        },
		_onPaste: function(evt) {
			// have to remove all pasted id attributes as they would be duplicates when pasting the content 
			// from the same document
			// (alternative: ensure id uniqueness on save, but then duplicate ids would be present in control until refresh)
			var pastedContent = evt.data.html;
			if (pastedContent) {
				var data = rmpw.text.internal.util.Converter.removeAttribute(pastedContent, "id");
				evt.data.html = data;
			}			
			this._checkDirtyAfterTimeout;
		},
		
        uninitialize: function(){
            this.inherited(arguments);
            if (this.ckeditor && this._editorReady) {
                this._removeDirtyStateListeners();
                this.ckeditor.destroy(true);
                this.ckeditor = null;
                this._editorReady = false;
            }
			return true;
        },
		
		resize: function(changeSize, resultSize) {
			if (!this.isVisible()) {
				return;
			}
            if (!this._editorReady) {
                if (this.onEditorReadyDeferred) {
                    this.onEditorReadyDeferred.addCallback(this, function(){
                        this.resize(changeSize, resultSize);
                    });
                }
                return;
            }
			 
            if (this.isInRichDocument) {
	            // Calculate the height
	            var myDim = dojo.position(this.containerNode, true); // Get dimensions of myself    		
				var bodyHeight = document.body.offsetHeight;
	            var height = bodyHeight - myDim.y - 52;	// 52px because it looks good, no method to the madness.
	    		
	    		// Max height might not be set, need to account for that when limiting the height.
	    		if(!this.maxHeight || (this.maxHeight && height < this.maxHeight)){
	    			this.containerNode.style.height = height + "px";
	    		}
            }
    		
			this.inherited(arguments);
		},
        
        layout: function(){
            if (!this._editorReady) {
                if (this.onEditorReadyDeferred) {
                    this.onEditorReadyDeferred.addCallback(this, function(){
                        this.layout();
                    });
                }
                return;
            }
            
            var cb = this._contentBox || dojo.contentBox(this.containerNode);
            if (cb.h > 0) {				
                this.ckeditor.resize(cb.w, cb.h);
            }
			
        },
		
		setVisible: function(visible) {
			this.domNode.style.display = visible ? "" : "none";
		},
		
		isVisible: function() {
			return this.domNode.style.display === "";	
		},		
        
        _toggleDisabled: function(){
            this.set('disabled', !this.disabled);
        },
        
        _getReadOnlyAttr: function(){
            return this.get('disabled');
        },
        
        _setReadOnlyAttr: function(value){
            return this.set('disabled', value);
        },
        
        _getValueAttr: function(){
			// only used when this widget is within a non-rich text document
            if (this._editorReady) {
                return this._editingReferenceValue ? this._localValue : this.ckeditor.getData();
            }
            return null;
        },
        
        _setValueAttr: function(value){
            if (!this._editorReady) {
            	this._holdValue = value;
                if (this.onEditorReadyDeferred) {
                    this.onEditorReadyDeferred.addCallback(this, function(){
                        this._setValueAttr(this._holdValue);
                    });
                }
                return;
            }
            
			if (this._editingReferenceValue) {
				this._localValue = value;
				return;
			}

            this.ckeditor.setData(value, function(){
            	if (dojo.isIE){
            		// Need a timeout with IE as the callback is called before the data is ready
            		setTimeout(this.resetDirty, 10);
            	} else {
            		this.resetDirty();
            	}
            });
        },
		
		doGetReferenceValueAttr: function() {
			if (this._editorReady) {
				var xhtmlData = this.ckeditor.getData();
				// this method is only called when in rich text document, so passing in isInRichDocument is just an extra precaution
				var jsonData = rmpw.text.internal.util.Converter.parseToJson(xhtmlData, null, this.isInRichDocument, null);
				return jsonData;
			}
			return null;
		},		
        
		// The following two methods (as well as the _localValue and _editingReferenceValue fields) were added to support
		// reusing the same rich text editor widget in generated properties pages in DTK where the property is typed as
		// and XHTML body.  In this case the properties infrastructure uses get/setReferenceValue as well as get/setValue
		// to support reference handling.  
        _getReferenceValueAttr: function(){
			// only used when this widget is within a rich text document;
			// accessed by generated form for to get the value in order to create the change;
			// this woud cause conversion and we don't need it until Save is clicked 
            return null;
        },
        
        _setReferenceValueAttr: function(value){
			this._editingReferenceValue = true;
            if (!this._editorReady) {
                if (this.onEditorReadyDeferred) {
                    this.onEditorReadyDeferred.addCallback(this, function(){
                        this._setReferenceValueAttr(value);
                    });
                }
                return;
            }
            
			var xhtmlValue = rmpw.text.internal.util.Converter.parse(value, this.isInRichDocument);
            this.ckeditor.setData(xhtmlValue);
			
			// if setting the value after loading, check if rdf_about indicates anonymous class
			// (if so, means old way of saving without ids, no way to associate paragraphs with comments)
			if (this.isInRichDocument && !this.form.bodyUsingRdfAbout) {
				this.form.bodyUsingRdfAbout = false;
				if (value.rdf_about && value.rdf_about.indexOf("#T0") > -1) {
					this.form.bodyUsingRdfAbout = true; 	
				}
			}
						
            this.ckeditor.resetDirty();
        },
		
		getXhtmlValue: function() {
			return this.ckeditor.getData();
		},
        
        // We cannot use ckeditor.setReadOnly to make the editor read only, 
        // because after that we would disable also the open resource feature
        _setDisabledAttr: function(value){
            if (!this._editorReady) {
                if (this.onEditorReadyDeferred) {
                    this.onEditorReadyDeferred.addCallback(this, function(){
                        this._setDisabledAttr(value);
                    });
                }
                return;
            }
            this.disabled = !!value;
            // Turn off contentEditable.
            this.ckeditor.document.$.body.disabled = this.disabled;
            if (dojo.isIE || dojo.isFF || dojo.isWebKit || dojo.isOpera) {
                this.ckeditor.document.$.body.contentEditable = !this.disabled;
            }
            else {
                this.ckeditor.document.$.designMode = this.disabled ? "off" : "on";
            }
            
        	// when in read only mode, we need to remove the cut and paste menus as they are still enable,
            // even if the corresponding commands are disabled
            if (this.disabled){
            	// --Keep the menu objects so we can restore them when/if we go in read-write mode
            	var cutMenu = this.ckeditor.getMenuItem('cut');
            	if (cutMenu)
            		this._cutMenu = cutMenu;
            	var pasteMenu = this.ckeditor.getMenuItem('paste');
            	if (pasteMenu)
            		this._pasteMenu = pasteMenu;
            	// remove the menus from the context menu
            	this.ckeditor.removeMenuItem('cut');
            	this.ckeditor.removeMenuItem('paste');
            } else {
            	// if we are transitioning from a read-only mode we need to add back the cut and paste menus
            	if (this._cutMenu){
            		this.ckeditor.addMenuItem('cut', {
						label : this._cutMenu.label,
						command : this._cutMenu.command,
						group : 'clipboard',
						order : this._cutMenu.order
					});
            		delete this._cutMenu;
            	}
            	if (this._pasteMenu){
            		this.ckeditor.addMenuItem('paste', 	{
            			label : this._pasteMenu.label,
            			command : this._pasteMenu.command,
            			group : 'clipboard',
            			order : this._pasteMenu.order
					});
            		delete this._pasteMenu;
            	}
            }

            
            // Prevent key handling and paste commands.
            this.ckeditor[this.disabled ? 'on' : 'removeListener']('key', this._cancelCKEvent, this, null, 0);
            this.ckeditor[this.disabled ? 'on' : 'removeListener']('paste', this._cancelCKEvent, this, null, 0);
            this.ckeditor[this.disabled ? 'on' : 'removeListener']('pasteDialog', this._cancelCKEvent, this, null, 0);
            
            // Disable all commands in wysiwyg mode.
            var cmd, cmds = this.ckeditor._.commands, mode = this.ckeditor.mode;
            for (var name in cmds) {
        		cmd = cmds[name];
        		// we want to keep the openresource context menus in disabled mode
            	if (name === "openresource"){
        			cmd[cmd.modes[mode] ? 'enable' : 'disable']();
            	} else {
            		if (this.disabled) {
            			cmd.disable();
                		cmd.on('state', this._cancelCKEvent, this, null, 0);
           		}
            		else {
                		cmd.removeListener('state', this._cancelCKEvent, this, null, 0);
            			cmd[cmd.modes[mode] ? 'enable' : 'disable']();
            		}
            	}
            }
                                                
            // we need to hide or show the toolbar...
            var toolbar = dojo.query(".cke_top", this.domNode)[0];
            if (this.disabled) {
                this._toolbarDisplayStyle = dojo.style(toolbar, 'display');
				if (this._toolbarDisplayStyle === 'none') {
					//work around starting in disabled mode then switching to enabled mode.
					this._toolbarDisplayStyle = 'table-cell';
				}
            }
            dojo.style(toolbar, 'display', this.disabled ? 'none' : this._toolbarDisplayStyle);
        },
        _toolbarDisplayStyle: '',
        
        _cancelCKEvent: function(evt){
            evt.cancel();
        },
        
        _getHeightDelta : function( editor )
        {
           var delta = 0;
           var contents = CKEDITOR.document.getById( 'cke_contents_' + editor.name );
           if (contents) {
               var outer = contents.getAscendant( 'table' ).getParent().getParent().getParent();
           
               // Get the height delta between the outer table and the content area.
               delta =  ( outer.$.offsetHeight || 0 ) - ( contents.$.clientHeight || 0 );
           }
           return delta;
        },
       
        _getEffectiveHeight : function( height, maxheight, minheight, padding )
        {
           var totalheight = parseInt(height,10) + parseInt(padding,10);
           if ( totalheight < minheight )
              totalheight = minheight;
           else
           {
              if ( maxheight && maxheight > 0 && totalheight > maxheight )
                 totalheight = maxheight;
           }
       
           return totalheight;
        },
        
        _checkSize: function( evt )
        {
          if (this.isInRichDocument) {
        	   // do nothing
        	   return;
           }
           
           var editor = evt.editor;
           
           if (!editor.document) {
        	   // do nothing - probably in "source" mode
        	   return;
           }
           var delta = this._getHeightDelta(editor) ;
           
           if (delta == 0) {
        	   // rich text is collapsed - only way to listen to uncollapse is to add an interval to poll
        	   this._checkUncollapseInterval = setInterval(dojo.hitch(this, function() {
        		   var newDelta = this._getHeightDelta(editor);
        		   if (newDelta != 0) {
        			   clearInterval(this._checkUncollapseInterval);
        			   delete this._checkUncollapseInterval;
        			   
        			   this._checkSize( { editor: this.ckeditor } );
        		   }
        	   }), 500);
        	   return;
           }
           
           var maxheight = 500;
           var minheight = 100;
           var padding = 30;
           var $body = editor.document.$.body;
           var newHeight = delta + $body.offsetHeight;
           //add a little padding as sometimes it just doesn't go big enough
           //especially if you remove Plugin resize
           newHeight = this._getEffectiveHeight( newHeight , maxheight, minheight, padding );
           var contents = CKEDITOR.document.getById( 'cke_contents_' + editor.name );
           newHeight = Math.max( newHeight, 0 ) + 'px';
           
           if(this.domNode.style.height != newHeight)
           {
        	   this.domNode.style.height = newHeight;
        	   this.resize();
           }
        },
        
        _checkDirty: function (evt){
            if (this._editorReady) {
                if (this.ckeditor.checkDirty()) {
                    this.onChange();
                    this.ckeditor.resetDirty();
                }
                
                this._checkSize(evt);
            }
        },
        
        _checkDirtyAfterTimeout: function (evt){
            window.setTimeout(dojo.hitch(this, function() {
            		this._checkDirty(evt);
            	}), 100);
        },
		
		subscribe: function() {
			this.connectCommenting("onSelectionChanged", this.commentSelected);
		},
		
		connectCommenting: function(action, method) {
			this.connect(this._commentingWidget, action, method);
		},
		
		commentSelected: function(sel, previous, comments, deselected) {
			if (!this.isVisible() || !comments || comments.length === 0) {
				// don't react when not active, or no comments passed in
				return;
			}
			this.ckeditor.focus(); // needed for IE
			if (!this.ckeditor.getSelection()) {
				return;
			}			
			var ranges = [], range;
			var element, firstElement;
			for (var i = 0, comment; comment = comments[i]; i++) {
				if (comment.range.length > 0) { // otherwise comment refers to the whole document, don't select anything
					for (var j = 0, id; id = comment.range[0].ids[j]; j++) {
						element = this.ckeditor.document.getById(id);
						if (element) {
							if (!firstElement) {
								firstElement = element;
							}
							range = new CKEDITOR.dom.range(this.ckeditor.document);
							range.selectNodeContents(element);
							ranges.push(range);
						}
					}
				}
			}
			if (ranges.length > 0) {
				if (!dojo.isFF) {
					// multi selection in ckeditor works well in FF, so just select the first one for others
					this.ckeditor.getSelection().selectElement(firstElement);					
				} else {
					// this will also clear the current selection
					this.ckeditor.getSelection().selectRanges(ranges);					
				}
				this.ckeditor.getSelection().scrollIntoView();
			} else {
				// just clear the current selection
				var nativeSel = this.ckeditor.getSelection().getNative();
				if (nativeSel) {
					nativeSel.removeAllRanges();
				}
			}
		}	

    });
})();
