/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.domains.diagrams.layout.GraphConfigurator");

dojo.require("rmpw.diagrams.render.graph.BaseGraphConfigurator");
dojo.require("rmpw.diagrams.layout.client.figures.ConnectionFigure");
dojo.require("rmpw.diagrams.layout.client.figures.ContainerFigure");
dojo.require("rmpw.diagrams.layout.client.figures.Figure");
dojo.require("rmpw.diagrams.layout.client.figures.MoveablePolygonAnchor");
dojo.require("rmpw.diagrams.layout.client.figures.FixedPositionAnchor");
dojo.require("dojox.json.ref");

/**
 * A utility class for configuring a graph for server side layout
 * 
 * @author Steve Shaw
 */
dojo.declare("rmps.domains.diagrams.layout.GraphConfigurator", rmpw.diagrams.render.graph.BaseGraphConfigurator, {

	createGraph: function(mapping) {
		return new rmpw.diagrams.layout.client.figures.ContainerFigure(mapping);
	},
	
	createGraphNode: function(mapping, graph, figRect) {
		return new rmpw.diagrams.layout.client.figures.Figure(mapping, graph, figRect);
	},
	
	createGraphEdge: function(mapping, graph, sourceNode, targetNode) {
		return new rmpw.diagrams.layout.client.figures.ConnectionFigure(mapping, graph, sourceNode, targetNode);
	},
	
	/*
	 * ILog support self relations
	 */
	_nodeLayoutSupportsSelfRelations: function() {
		return true;
	},
	
	/*
	 * Check whether layout algorithms support an incremental mode
	 */
	_nodeLayoutSupportsIncremental: function() {
		return true;
	},
	
	/*
	 * Create an anchor owned by graphElement of a particular type - one of FIXED_ANCHOR,
	 * MOVEABLE_ANCHOR
	 */
	createAnchor: function(anchorType, graphElement, args) {
		if (anchorType == this.FIXED_ANCHOR) {
			var fixedAnchor = new rmpw.diagrams.layout.client.figures.FixedPositionAnchor(graphElement);
			fixedAnchor.addFixedLocation(args.side, args.percentage, args.inset);
			return fixedAnchor;
		}
		else if (anchorType == this.MOVEABLE_ANCHOR) {
			return new rmpw.diagrams.layout.client.figures.MoveablePolygonAnchor(graphElement);
		}
	},
	
	runGraphLayoutAlgorithm: function(graph, layoutCallBack, args) {
		var rawData = graph.toRawData();
		var jsonGraph = rawData.jsonGraph;
		var idMap = rawData.idMap;
		
		var idLookupFn = function(id) {
			return id == "Head" ? id : idMap[id].mapping;
		}
		
		var self = this;
		var dataInput = dojo.mixin({graph: jsonGraph}, args);
		dataInput.resetGraphPosition = false;
		
		delete dataInput["targetController"]; // don't serialize objects
		delete dataInput["viewer"];
		
		var url = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/graphLayout";
		dojo.xhrPost({
			url: url,
			handleAs: "text",
			content: { graphData: dojox.json.ref.toJson(dataInput) },
			load: function(dataOutput) {
				var resultGraph = self.fromRawData(dojox.json.ref.fromJson(dataOutput), idLookupFn);
				layoutCallBack(resultGraph, args);
				resultGraph.destroy();
			},
			error: function(error) {
				console.error(error);
			}
		});	
		
		return true;
	}
});