/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| Copyright IBM Corp. 2009.  All Rights Reserved.                        |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

( function() {

dojo.provide("rmpw.widgets.palettes.ColorPalette");

dojo.require("dijit.ColorPalette");

dojo.require("rmpw.widgets.Events");

dojo.require("dijit.form.ComboBox");

dojo.require("dojo.data.ItemFileWriteStore");

dojo.requireLocalization("rmpw.widgets", "ToolStrings");

var strings = dojo.i18n.getLocalization("rmpw.widgets", "ToolStrings");

var topics = rmpw.widgets.Events.topics;

dojo.declare( "rmpw.widgets.palettes.ColorPalette", [dijit.ColorPalette], {
   
   target: topics.LINE_COLOR, 
      
   value: "#DC143C",
   
   decimalValue: 0,

   postCreate: function(){

      this.inherited(arguments);
      
      var comboDiv = document.createElement( "div" );
            
      var label = document.createElement( "label" );      
      label.innerHTML = strings.transparency_label;
      label.style.fontSize = "8pt";
      
      this.opacityDiv = document.createElement( "div" );           
      this.opacityDiv.style.marginLeft = "5px";      
      this.opacityDiv.style.marginBottom = "5px";     
      this.opacityDiv.appendChild( label );
      this.opacityDiv.appendChild( comboDiv );

      this.domNode.appendChild( this.opacityDiv );
                  
      var labels = [ { name: "Clear", label: "Clear", value: 0.01 }, 
                     { name: "90%", label: "90%", value: 0.1 },
                     { name: "80%", label: "80%", value: 0.2 },
                     { name: "70%", label: "70%", value: 0.3 },
                     { name: "60%", label: "60%", value: 0.4 },
                     { name: "50%", label: "50%", value: 0.5 },
                     { name: "40%", label: "40%", value: 0.6 },
                     { name: "30%", label: "30%", value: 0.7 },
                     { name: "20%", label: "20%", value: 0.8 },
                     { name: "10%", label: "10%", value: 0.9 },
                     { name: "Solid", label: "Solid", value: 1 } ];
      
      var options = new dojo.data.ItemFileWriteStore({
          data: { identifier : "name", items : labels }
      });
       
      this.combo = new dijit.form.ComboBox({
          store : options,
          value: this.transparencyLabel,
          searchAttr: "name",
          onChange: dojo.hitch( this, "readFromStore" )
      }, comboDiv );
           
      this.combo.domNode.style.width = "90px";   
      this.combo.domNode.style.fontSize = "8pt";
      
      this._initialized = true;
   },
   
   readFromStore: function(){	   

	   var myVal; 
	   this.combo.store.fetch({ 
		   query: {name: this.combo.value }, 
           onComplete: function(items, request){  
                myVal = items[0].value[0]; 
           } 
	   }); 
	   
	   this.decimalValue = myVal; 
	   
	   console.log( 'read from store: ' + this.decimalValue );
	   this._opacityChanged = true;
	   this.onChange( this.value, this.decimalValue );
   },
   
   onChange: function( color, opacity ){
	   // if user has changed color but didn't touch opacity and opacity is 0, then kick opacity up to 70%
	   if (this._initialized && this.decimalValue==0 && !this._opacityChanged) {
		   this.decimalValue = 0.7;
		   this.combo.setValue("70%");
	   }
	   dojo.publish( this.target, [ { color: this.value, opacity: this.decimalValue } ] );
   },
   
   reset: function(){   
	   dojo.publish( this.target, [ { color: this.value, opacity: this.decimalValue } ] );
   },
   
   destroy: function(){
      this.inherited( arguments );
   }
});

}());
