/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function () {
dojo.provide("rmpw.widgets.palettes.TextPalette");

dojo.require("dijit.form.Select");
dojo.requireLocalization("rmpw.widgets", "TextPaletteMessages");

var _messages = dojo.i18n.getLocalization("rmpw.widgets", "TextPaletteMessages");
var _SELECTBOX_STYLING = {width: "140px"};

dojo.declare( "rmpw.widgets.palettes.TextPalette", [dijit._Widget,dijit._Templated], {
	
	_messages: _messages,
	widgetsInTemplate: true,

	DEFAULT_FONT : new Object({
		family : "sans-serif",
		size : "10pt",
		style : "normal",
		weight : "normal"
	}),

    templateString: '<div class="dijitInline dijitColorPalette" data-dojo-attach-point="shell">' + 
                        '<div class="dijitColorPaletteInner" data-dojo-attach-point="divNode">' +
                            '<div data-dojo-type="dijit.layout.ContentPane" doLayout="false" style="font-size:8pt;">' +
                                '<table>' +
                                    '<tr>' +
		                                '<td>${_messages.FONT_FAMILY}</td>' +
		                                '<td>' +
		                                    '<select data-dojo-attach-point="familyNode" data-dojo-type="dijit.form.Select" value="sans-serif" >' +
		                                        '<option value="serif">${_messages.SERIF}</option>' +
		                                        '<option value="sans-serif" selected="true">${_messages.SANS_SERIF}</option>' +
		                                    '</select>' +
		                                '</td>' +
		                            '</tr>' +
	                                '<tr>' +
		                                '<td >${_messages.FONT_STYLE}</td>' +
		                                '<td>' +
		                                    '<select data-dojo-attach-point="styleNode" data-dojo-type="dijit.form.Select" value="normal">' +
		                                        '<option value="normal" selected="true">${_messages.NORMAL}</option>' +
		                                        '<option value="italic">${_messages.ITALIC}</option>' +
		                                        '<option value="oblique">${_messages.OBLIQUE}</option>' +
		                                    '</select>' +
		                                '</td>' +
	                                '</tr>' +
	                                '<tr>' +
		                                '<td >${_messages.FONT_WEIGHT}</td>' +
		                                '<td>' +
		                                    '<select data-dojo-attach-point="weightNode" data-dojo-type="dijit.form.Select" value="normal" >' +
		                                        '<option value="normal" selected="true">${_messages.NORMAL}</option>' +
		                                        '<option value="bold">${_messages.BOLD}</option>' +
		                                    '</select>' +
		                                '</td>' +
		                            '</tr>' +
		                            '<tr>' +
		                                '<td >${_messages.FONT_SIZE}</td>' +
		                                '<td>' +
		                                    '<select data-dojo-attach-point="sizeNode" data-dojo-type="dijit.form.Select" data-dojo-props=\'dropDownPosition:["above"]\' value="12pt" >' +
		                                        '<option value="8pt">8pt</option>' +
		                                        '<option value="10pt">10pt</option>' +
		                                        '<option value="12pt" selected="true">12pt</option>' +
		                                        '<option value="14pt">14pt</option>' +
		                                        '<option value="16pt">16pt</option>' +
		                                        '<option value="20pt">20pt</option>' +
		                                        '<option value="30pt">30pt</option>' +
		                                        '<option value="40pt">40pt</option>' +
		                                        '<option value="50pt">50pt</option>' +
		                                    '</select>' +
		                                '</td>' +
		                            '</tr>' +
		                        '</table>' +
                                '<div data-dojo-attach-point="buttonNode" style="float:right;padding-top:5px;padding-right:5px;font-size:7pt">' +
                                '</div>' +
                            '</div>' +
                        '</div>' +
                    '</div>',

    postCreate: function () {

        this.inherited(arguments);
        
        this.familyNode.set("style", _SELECTBOX_STYLING);
        this.styleNode.set("style", _SELECTBOX_STYLING);
        this.weightNode.set("style", _SELECTBOX_STYLING);
        this.sizeNode.set("style", _SELECTBOX_STYLING);
        
        this.initializeValues(this.DEFAULT_FONT);

        this.ok = new dijit.form.Button({
            label : _messages.OK,
            onClick : dojo.hitch(this, "setStyles")
        });

        this.buttonNode.appendChild(this.ok.domNode);

        this.cancel = new dijit.form.Button({
            label : _messages.CANCEL,
            onClick : dojo.hitch(this, "hide")
        });

        this.buttonNode.appendChild(this.cancel.domNode);
    },
    
    hide: function() {
        dojo.style(this.domNode, {
            display : "none"
        });
    },
    
    initializeValues: function (fontObject) {
    	if (!this.initialFont) {
    		this.initialFont = new Object();
    	}
    	
		this.initialFont.family = fontObject.family ? fontObject.family : this.DEFAULT_FONT.family;
		this.initialFont.weight = fontObject.weight ? fontObject.weight : this.DEFAULT_FONT.weight;
		this.initialFont.style = fontObject.style ? fontObject.style : this.DEFAULT_FONT.style;
		this.initialFont.size = fontObject.size ? fontObject.size : this.DEFAULT_FONT.size;
		
		if (!this.familyNode.getOptions(this.initialFont.family)) {
			this.familyNode.addOption({
				value : this.initialFont.family,
				label : this.initialFont.family
			});
		}			
		if (!this.styleNode.getOptions(this.initialFont.style)) {
			this.styleNode.addOption({
				value : this.initialFont.style,
				label : this.initialFont.style
			});
		}			
		if (!this.weightNode.getOptions(this.initialFont.weight)) {
			this.weightNode.addOption({
				value : this.initialFont.weight,
				label : this.initialFont.weight
			});
		}			
		if (!this.sizeNode.getOptions(this.initialFont.size)) {
			this.sizeNode.addOption({
				value : this.initialFont.size,
				label : this.initialFont.size
			});
		}			

		this.familyNode.set('value', this.initialFont.family);
		this.weightNode.set('value', this.initialFont.weight);
		this.styleNode.set('value', this.initialFont.style); 
		this.sizeNode.set('value', this.initialFont.size); 

    },

    setStyles: function() {
    	this.chosenFont = new Object();
    	if (this.familyNode.value != this.initialFont.family) {
            this.chosenFont.family = this.familyNode.value;    		
    	}
    	if (this.weightNode.value != this.initialFont.weight) {
            this.chosenFont.weight = this.weightNode.value;    		
    	}
    	if (this.styleNode.value != this.initialFont.style) {
            this.chosenFont.style = this.styleNode.value;    		
    	}
    	if (this.sizeNode.value != this.initialFont.size) {
            this.chosenFont.size = this.sizeNode.value;    		
    	}

        this.hide();
    },
    
    getFontObject: function () {
        return this.chosenFont;
    }

});

}());
