/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.pathmaps.PathmapsSection");

dojo.require("rmps.ui.EditableCollapsibleSection");

dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.util.array");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.requireLocalization("rmps.ui", "PathmapsSectionMessages");


var PATHMAPS_UAKEY = "com.ibm.xtools.rmps.web.importer.section_ImportDefinitionEditor002";
//var DIRTY_GROUP_ID = "rmps.importer.ImportDefinitionEditor.dirty";

var IMPORT_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "importer");

var IMPORT_PATH_MAPS = IMPORT_PATH + "/pathmaps";


var messages = dojo.i18n.getLocalization("rmps.ui", "PathmapsSectionMessages");

var connectOnInput = function(obj, widget, method) {
	if (dojo.isMoz || dojo.isOpera) {
		widget.connect(obj, "oninput", method);
	} else {
		widget.connect(obj, "onkeydown", method);
		widget.connect(obj, "onkeyup", method);
		widget.connect(obj, "onpaste", function() {
			setTimeout(dojo.hitch(this, method), 1);
		});
		widget.connect(obj, "oncut", function() {
			setTimeout(dojo.hitch(this, method), 1);
		});
		widget.connect(obj, "onblur", method);
	}
};

var getSystemPathMaps = function() {
	var args = {
		url: IMPORT_PATH_MAPS,
		headers: {
			"X-ibm-rmps-internal": "true"
		},
		handleAs: "text",
		load: function(response, ioArgs) {
			var pathMaps = response.split("\n");
			pathMaps.sort();
			return pathMaps;
		}
	};
	return jazz.client.xhrGet(args);
};

dojo.declare("rmps.ui.pathmaps.PathmapsSection._PathMaps", [dijit._Widget, dijit._Templated], {
	
	dirtyGroupId: null,
	
	isReadOnly: false,
	
	uaKey: PATHMAPS_UAKEY,
	
	messages: messages,
	
	widgetsInTemplate: true,
	
	templateString: '<div>' +
			'<div dojoType="rmps.ui.EditableCollapsibleSection" dojoAttachPoint="_section" primaryTitle="${messages.pathMaps}" isCollapsed=true readOnly="${isReadOnly}" uaKey="${uaKey}"></div>' +
		'</div>',
	
	editor: null,
	
	_definition: null,
	
	_pathMaps : null,
	
	
	postCreate: function() {
		this._definition = this.editor._definition;
		
		// sort path maps
		this._definition.pathMaps.sort(function(a, b) {
			var la = a.variable.toLowerCase();
			var lb = b.variable.toLowerCase();
			return (la > lb) ? 1 : ((la < lb) ? -1 : 0);
		});
		
		
		// create table
		this._table = new jazz.ui.StyledTable({
			columns: [
					{label: messages.variable, width: "50%"},
					{label: messages.value},
					{label: "", className: "actions"}
				]
		});
		//add tooltip
		var headerEntered = false;
	    var oldHeaderTooltip = null;
		this.connect(this._table._headerRow, "onmouseover", function(){
		    if (!headerEntered){
		    	oldHeaderTooltip = this._section.containerNode.title;
		    }
		    headerEntered = true;
		    this._section.containerNode.title = dojo.string.substitute(messages.pathMapsHeaderTooltip, 
		    				[messages.projectDirectoryName, messages.projects]);
		});
		this.connect(this._table._headerRow, "onmouseout", function(){
			headerEntered = false;
			this._section.containerNode.title = oldHeaderTooltip;
		});
		
		this._table.containerNode = this._table.tbody;
		this._section.containerNode.appendChild(this._table.domNode);
		this._table.startup();
		
		// create Add command
		if (!this.isReadOnly) {
			this._section.addAction(messages.add, messages.addPathMap, dojo.hitch(this, function() {
				this._section.edit();
				this._addPathMapRow();
			}));
		}
		
		// init
		var i;
		for (i = this._definition.pathMaps.length - 1; i >= 0; --i) {
			this._addPathMapRow(this._definition.pathMaps[i]);
		}
		
		// add system defined path maps
		var loadingTr = dojo.create("tr", {className: "systemPathMap"});
		dojo.create("span", null,
			dojo.create("td", {colSpan: "3"}, loadingTr)).appendChild(dojo.doc.createTextNode(messages.loadingSystemDefinedPathMaps));
		this._table.tbody.appendChild(loadingTr);
		this._getSystemPathMaps().addCallback(this, function(pathMaps) {
			dojo.destroy(loadingTr);
			for (var i = 0, p; p = pathMaps[i]; ++i) {
				var row = new rmps.ui.pathmaps.PathmapsSection._SystemPathMapRow({pathMap: p});
				this._table.tbody.appendChild(row.domNode);
				row.startup();
			}
			this._pathMaps = pathMaps;
			this._updatePathMapCount();
		});
		
	},
	
	//projectDirNames is optional
	validate: function(errors, projectDirNames) {
		// check for redeclaration of system path maps
		var spDeferred = new dojo.Deferred();
		errors.push(spDeferred);
		spDeferred.addCallback(this, function(systemPathMaps) {
			var spm = dojo.isArray(systemPathMaps) ? systemPathMaps : [];
			var duplicateVariables = [];
			var variables = {};
			var i, p;
			
			for (i = 0; i < this._definition.pathMaps.length; ++i) {
				var pathMap = this._definition.pathMaps[i];
				
				if (pathMap.variable.length > 0) {
					if (variables[pathMap.variable] != null) {
						if (variables[pathMap.variable] === false) {
							variables[pathMap.variable] = true;
							duplicateVariables.push(pathMap.variable);
						}
					} else {
						variables[pathMap.variable] = false;
					}
				}
			
				if (pathMap.variable.length === 0 && pathMap.value.length !== 0 && dojo.indexOf(spm, pathMap.variable) === -1) {
					errors.push(dojo.string.substitute(messages.error_no_pathMap_variable, [pathMap.value]));
					continue;
				}
				
				if (pathMap.variable.length !== 0 && pathMap.value.length === 0 && dojo.indexOf(spm, pathMap.variable) === -1) {
					errors.push(dojo.string.substitute(messages.error_no_pathMap_value, [pathMap.variable]));
					continue;
				}
				
				var segments = pathMap.value.split("/");
				if (projectDirNames && projectDirNames[segments[0] ? segments[0] : segments[1]] == null) {
					errors.push(dojo.string.substitute(messages.error_pathMap_value_project, [pathMap.variable]));
				}
			}
			
			if (duplicateVariables.length > 0) {
				duplicateVariables.sort();
				errors.push(dojo.string.substitute(messages.error_duplicate_pathMap_variables, [duplicateVariables.join(", ")]));
			}
			
			if (spm.length > 0) {
				var redeclaredPathMaps = [];
				
				for (i = 0, p; p = this._definition.pathMaps[i]; ++i) {
					if (dojo.indexOf(spm, p.variable) >= 0 && dojo.indexOf(redeclaredPathMaps, p.variable) === -1) {
						redeclaredPathMaps.push(p.variable);
					}
				}
				
				if (redeclaredPathMaps.length > 0) {
					redeclaredPathMaps.sort();
					throw new Error(dojo.string.substitute(messages.error_redeclare_system_pathMap, [redeclaredPathMaps.join(", ")]));
				}
			}
		});
		this._getSystemPathMaps().addBoth(spDeferred, spDeferred.callback);				
	},
	
	_countUserDefinedPathMaps: function() {
		var result = 0;
		var pathMaps = this._definition.pathMaps;
		for (var i = 0; i < pathMaps.length; ++i) {
			if (pathMaps[i].variable.length > 0 || pathMaps[i].value.length) {
				result++;
			}
		}
		return result;
	},
	
	_updatePathMapCount: function() {
		var counter = this._countUserDefinedPathMaps() + this._pathMaps.length - 1;
		this._section.setSecondaryTitle('(' + counter + ')');
	},
	
	_addPathMapRow: function(data) {
		var row = new rmps.ui.pathmaps.PathmapsSection._PathMapRow({
			dirtyGroupId: this.dirtyGroupId,
			pathMapSection: this,
			editor: this.editor,
			pathMap: data
		});
		this._table.tbody.insertBefore(row.domNode, this._table.tbody.firstChild);
		row.startup();
		
		if (!data) {
			row.focus();
		}
	},
	
	pruneEmpty: function() {
		dojo.forEach(dijit.findWidgets(this._table.tbody), function(row) { 
			if (!row.isSystem && !row.pathMap.variable && !row.pathMap.value) {
				row.remove();
				row.destroyRecursive();
			}
		});
	},
	
	_getSystemPathMaps: function() {
		if (!this._pathMapsDeferred) {
			this._pathMapsDeferred = getSystemPathMaps();
		}
		return this._pathMapsDeferred;
	}
	
});

dojo.declare("rmps.ui.pathmaps.PathmapsSection._SystemPathMapRow", [dijit._Widget, dijit._Templated], {
	
	isSystem: true,
	pathMap: null,
	messages: messages,
	
	templateString: '<tr class="systemPathMap">' +
			'<td>' +
				'<div>' +
					'<span>${pathMap}</span>' +
				'</div>' +
			'</td>' +
			'<td colspan="2">' +
				'<div>' +
					'<span>${messages.systemDefined}</span>' +
				'</div>' +
			'</td>' +
		'</tr>'
});

dojo.declare("rmps.ui.pathmaps.PathmapsSection._PathMapRow", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	
	templateString: '<tr>' +
			'<td>' +
				'<div>' +
					'<span class="pathmaps-preview" dojoAttachPoint="_variablePreview"></span>' +
					'<input class="pathmaps-editable" style="width: 100%;" dojoAttachPoint="_variableInput" type="text"></input>' +
				'</div>' +
			'</td>' +
			'<td>' +
				'<div>' +
					'<span class="pathmaps-preview" dojoAttachPoint="_valuePreview"></span>' +
					'<input class="pathmaps-editable" style="width: 100%;" dojoAttachPoint="_valueInput" type="text"></input>' +
				'</div>' +
			'</td>' +
			'<td class="actions" style="width: 26px;">' +
				'<div dojoType="jazz.ui.SimpleToolbar" dojoAttachPoint="toolbar"></div>' +
			'</td>' +
		'</tr>',
	
	dirtyGroupId: null,
	pathMapSection: null,
	editor: null,
	pathMap: null,
	
	_definition: null,
	_originalPathMap: null,
	
	postCreate: function() {
		this._definition = this.editor._definition;
		
		if (!this.pathMap) {
			this.pathMap = {
				variable: "",
				value: ""
			};
			this._definition.pathMaps.push(this.pathMap);
			
		} else {
			this._variableInput.value = this.pathMap.variable;
			this._variablePreview.appendChild(dojo.doc.createTextNode(this.pathMap.variable));
			this._valueInput.value = this.pathMap.value;
			this._valuePreview.appendChild(dojo.doc.createTextNode(this.pathMap.value));
			this._variableInput.focus();
			
			this._originalPathMap = dojo.clone(this.pathMap);
		}
		
		// add remove button
		if (!this.isReadOnly) {
			this.toolbar.addButton(this, function(e) {
						dojo.stopEvent(e);
						this.remove();
						this.destroyRecursive();
					}, {
					label: messages.removePathMap,
					imgSrc: dojo.moduleUrl("rmps.ui", "graphics/icons/remove.gif")
				});
			
			connectOnInput(this._variableInput, this, function() {
				this.pathMap.variable = dojo.trim(this._variableInput.value);
				if (this.dirtyGroupId != null) {
					this.editor._updateDirty(this._originalPathMap ? this._originalPathMap.variable : "", this.pathMap.variable, this._variableInput);
				}
				this.pathMapSection._updatePathMapCount();
				this._variablePreview.innerHTML = "";
				this._variablePreview.appendChild(dojo.doc.createTextNode(this.pathMap.variable));
			});
			
			connectOnInput(this._valueInput, this, function() {
				this.pathMap.value = dojo.trim(this._valueInput.value);
				if (this.dirtyGroupId != null) {
					this.editor._updateDirty(this._originalPathMap ? this._originalPathMap.value : "", this.pathMap.value, this._valueInput);
				}
				this.pathMapSection._updatePathMapCount();
				this._valuePreview.innerHTML = "";
				this._valuePreview.appendChild(dojo.doc.createTextNode(this.pathMap.value));
			});
		}
	},
	
	focus: function() {
		this._variableInput.focus();
	},
	
	remove: function() {
		jazz.util.array.removeByReference(this._definition.pathMaps, this.pathMap);
		this.pathMapSection._updatePathMapCount();
		
		if (this.dirtyGroupId != null) {
			if (this._originalPathMap) {
				this.editor.addDirty(this._originalPathMap, this.dirtyGroupId);
			}
			this.editor.removeDirty(this._valueInput, this.dirtyGroupId);
			this.editor.removeDirty(this._variableInput, this.dirtyGroupId);
		}
	}
});

})();
