/******************************** ***********************************************
* Licensed Materials - Property of IBM
* Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
* 
* US Government Users Restricted Rights - Use, duplication or disclosure
* restricted by GSA ADP Schedule Contract with IBM Corp.
***************************************** **************************************/

( function() {

dojo.provide("rmpw.widgets.pickers.ColorPicker");

dojo.require("dijit.form.Button");
dojo.require("rmpw.widgets.palettes.ColorPalette");

dojo.require("rmpw.widgets.Events");

dojo.requireLocalization("rmpw.widgets", "ToolStrings");

var strings = dojo.i18n.getLocalization("rmpw.widgets", "ToolStrings");

var topics = rmpw.widgets.Events.topics;

dojo.declare( "rmpw.widgets.pickers.ColorPicker", dijit.form.DropDownButton, {
   
	/* Line of Fill customizers */
	
	type: "LINE",
	title: strings.lineColor_tooltip,
	target: topics.LINE_COLOR,
	transparencyLabel: "Solid",
	decimalValue: 1,
	
	/* Icon display */
	   
	showLabel: false,
	iconClass: "pickerIcon",
	  
	/* Swatch management */ 
	
	swatch: null,
	color: "#DC143C",
	leftOffset: 12,
	topOffset: 10,
	   
	subscriptions: null,

	postCreate: function(){

		this.inherited( arguments );
			      

        if(this.type === "FILL") {
            this.target = topics.FILL_COLOR;
            this.decimalValue = 0;
            this.transparencyLabel = "Clear";
            this.color= "#FFFFFF";
            this.attr('iconClass', "fillIcon");
            this.attr('title', strings.fillColor_tooltip);
        }

        if(this.type === "TEXT") {
            this.target = topics.TEXT_COLOR;
            this.transparencyLabel = "Clear";
            this.attr('iconClass', "textFillIcon");
            this.attr('title', strings.textColor_tooltip);
        }


		this.dropDown = new rmpw.widgets.palettes.ColorPalette({ 
			target: this.target, 
			value: this.color,
			transparencyLabel: this.transparencyLabel,
			decimalValue: this.decimalValue
		});
	
		this.subscriptions = [];      
		this.subscriptions.push( dojo.subscribe( this.target, dojo.hitch( this, "colorChange" ) ) );
	},

	startup: function(){
		this.inherited( arguments );
		this.updateSwatch( this.color, this.decimalValue );
	},
	   
	colorChange: function( value ){	 
		 
		this.decimalValue = value.opacity;
		this.updateSwatch( value.color, value.opacity ); 
	},
	   
	updateSwatch: function( color, opacity ){

		if( !this.swatch ){
			this.swatch = document.createElement( "div" );
			this.iconNode.appendChild( this.swatch );

			var style = this.swatch.style;
			style.height = "6px";
			style.width = "6px";
			style.position = "relative"; 
			style.fontSize = "8px";
			style.fontFamily = "sans-serif";
			style.lineHeight = "0px";
		}
		   
		this.swatch.style.left = this.leftOffset + "px";
		this.swatch.style.top = this.topOffset +  "px";
		if (color) {
			this.color = color;
		}
		
		
		if( opacity && opacity < 0.1 ){
			this.swatch.style.background = "none";
			this.swatch.style.border = "1px solid grey";
		}else{
			this.swatch.style.border = "none";
			this.swatch.style.background = this.color;	
		}
	},
	   
	reset: function(){
		this.updateSwatch( this.color, this.decimalValue );
		dojo.publish( this.target, [ { color: this.color } ] );
	}, 
	   
	destroy: function(){	   
		var i = 0, subscriptionLength = this.subscriptions.length;
		for (i = 0; i < subscriptionLength; i++) {
			dojo.unsubscribe(this.subscriptions[i]);
		}
		   
		this.inherited( arguments );
	}
	   
});

}());
