/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("rmps.app.pickers.internal.CLMResourcePicker");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.layout._LayoutWidget");
dojo.require("dijit.layout.ContentPane");
dojo.require("dojox.layout.TableContainer");

dojo.require("jazz.core.loader");
dojo.require("jazz.ui.RemoteChooserResponder");
dojo.require("jazz.app.auth");

dojo.require("rmps.core.error");

dojo.require("rmps.core.projects.ProjectClient");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.core.streams.StreamClient");

dojo.require("rmps.core.util.ILinksConstants");
dojo.require("rmps.core.util.TypeConstants");
dojo.require("rmps.core.util.ContextUtils");

dojo.require("rmpw.forms.ui.SelectCreatePicker");

dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants");

dojo.require("rmps.app.pickers.internal.PickerController");
dojo.require("rmps.app.pickers.internal.CustomizedPickerController");

var ProjectClient= rmps.core.projects.ProjectClient;
var ResourceInfoClient= rmps.core.resource.ResourceInfoClient;
var StreamClient= rmps.core.streams.StreamClient;

var ILinksConstants = rmps.core.util.ILinksConstants;
var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
var TypeConstants= rmps.core.util.TypeConstants;
var ContextUtils= rmps.core.util.ContextUtils;

var QuickSearchSelection= "rmpw.forms.ui.QuickSearchSelection";
var ResourcesSelection= "rmpw.forms.ui.ResourcesSelection";
var ClipboardWidget= "rmps.ui.clipboard.ClipboardWidget";
var ExplorerWidget = "rmps.ui.explorer.ExplorerSearchWidget";
var RMPSPicker= "rmps.app.pickers.internal.RMPSPicker";

var DEFAULT_OSLC_CONTEXT_PROPERTY = "defaultOslcContext";

// Define the OSLC protocols for both versions 1.0 and 2.0 (default is version 2.0)
var _OSLC1_PROTOCOL_WINDOWNAME = '#oslc-windowName-1.0';
var _OSLC1_PROTOCOL_POSTMSG    = '#oslc-postMessage-1.0';
var _OSLC2_PROTOCOL_WINDOWNAME = '#oslc-core-windowName-1.0';
var _OSLC2_PROTOCOL_POSTMSG    = '#oslc-core-postMessage-1.0';


/**
 * An OSLC style delegated selection dialog for selecting DM resource.
 * The picker can either be initialized via parameters in the window's URL (i.e., in a delegated UI scenario),
 * or directly via parameters passed in to the constructor.
 * In the direct instantiation case, results are returned by a provided function;
 * otherwise, the picker returns results using the OSLC Window Name or Post Message protocols.
 *
 * This picker is registered as an ajax module (in plugin.xml) with the ID "rmps.pickers.RMPSPicker"
 * so that other OSLC applications can invoke this picker to select a DM resource. In this mode, only
 * a single resource can be selected by default.
 *
 * This picker is also used internally in DM to pick DM resources, in which case the caller can
 * specify the "enableMultiSelect" argument as true to select multiple resources.
 */
dojo.declare("rmps.app.pickers.internal.CLMResourcePicker", [dijit._Widget, dijit._Templated], {
	"class" : "rmps-app-clm-resource-picker",
		
	templateString: '<div class="rmps-app-clm-resource-picker">' +
	                    '<div class="content" data-dojo-type="dijit.layout._LayoutWidget" data-dojo-attach-point="containerNode"></div>' +
	                '</div>',

	projectId : null,
	context: null,
	changeset: null,
	typeUri : null,
	propertyUri: null,
	propertyDomainUri: null,
	documentUri: null,
	resourceUri: null,
	filteredList: [],
	findFragments: false,
	enableMultiSelect: true,
	oslcConfiguration: null,
	
	// When this CLMREsourcePicker class is used internally in DM, the
	// results do not need to be formatted as an OSCL response on the
	// corresponding remote frame. Instead, the DM caller can specify
	// an onOK(results) function to process the results of the
	// resource selection.
	onOK: null,

	// There are many issues with the quick search picker (WI # 35007)
	// When they are fixed, remove this parameter and its reference below
	_excludeQuickSearch: true,

	pickerMessages: null,

	_url: null,
	_remoteFrame: null,
	_isInternal: false,
	_OSLCprotocol: null,

	constructor: function(args) {
	    // Get the OSLC protocol to determine what format of data to return
		this._OSLCprotocol = window.location.hash;
		
		this._url = new dojo._Url(location.href);
		var queryString = this._url.query;
		if (queryString) { // ensure the query string is defined before parsing
			var qParams = dojo.queryToObject(queryString);

			if (qParams.project) {
				qParams.projectId = qParams.project;
				delete qParams.project;
			}

			if (qParams.diagram) {
				qParams.onlyDiagram = qParams.diagram;
				delete qParams.diagram;
			}

			if (qParams.onlyType) {
				qParams.typeUri = decodeURIComponent(qParams.onlyType);
				delete qParams.onlyType;
			}
			
			if (qParams.enableMultiSelect && dojo.isString(qParams.enableMultiSelect)) {
				this.enableMultiSelect = (qParams.enableMultiSelect === "true");
				delete qParams.enableMultiSelect;
			}
			
			if (qParams['oslc.configuration']) {
				this.oslcConfiguration = qParams['oslc.configuration'];
				delete qParams['oslc.configuration'];
			}

			dojo.mixin(this, qParams);
		}
		
		dojo.mixin(this, args);

		if (!this.typeUri) {
			this.typeUri = TypeConstants.DOCUMENT; 
		}

		if (!this.propertyUri) {
			this.propertyUri = ILinksConstants.DMLinkType.NAMESPACE + ProcessConstants.LinkTypes.ELABORATED_BY;
		}

		if (!this.propertyDomainUri) {
			this.propertyDomainUri = TypeConstants.DOCUMENT;
		}

		if (!this.resourceUri) {
			this.resourceUri = TypeConstants.DOCUMENT; 
		}

		if (!this.documentUri) {
			this.documentUri = this.resourceUri;
		}
	},

	postCreate: function() {
		dojo.requireLocalization("rmps.app.pickers.internal", "CLMResourcePickerMessages");
		this.pickerMessages = dojo.i18n.getLocalization("rmps.app.pickers.internal", "CLMResourcePickerMessages");

		if (this.context) {
			this._isInternal = true;
			this._createPicker();
		} else {
			// External call means the global configuration or default OSLC context has to be used.
			// If there is no global configuration, and if no default OSCL context has been set,
			// then default stream of the target project will be used.
			if (this.oslcConfiguration) {
				var context = ContextUtils.getContextUriFromVvcContextUri(this.oslcConfiguration, this.projectId);
				if (context) {
					this.context = context;
					this._createPicker();
					return;
				}
			}
			
			this._handleExternalCall();
		}
	},
	
	// Is the OSLC version 1.0 protocol being used (must be explicitly set or else v2.0 is used).
	_isOSLC1Protocol : function() {
		return this._OSLCprotocol && (this._OSLCprotocol === _OSLC1_PROTOCOL_WINDOWNAME || this._OSLCprotocol === _OSLC1_PROTOCOL_POSTMSG);
	},

	_handleExternalCall: function() {
		/* 
		 * Ensure we're actually authenticated.  As this is an external call, the user may not yet be authenticated.
		 */
		jazz.app.auth.loginIfNecessary();
		
		dojo.when(ProjectClient.getProjectProperty(this.projectId, DEFAULT_OSLC_CONTEXT_PROPERTY),
			dojo.hitch(this, function(oslcContextURI) {
				this.context = oslcContextURI && oslcContextURI.length > 0 ? oslcContextURI : null;
				if (this.context === null) {
					// Default OSLC context is not set, let's proceed with default stream of target project
					this._proceedWithDefaultStream(true);
				} else {
					this._createPicker();
				}
			}),
			dojo.hitch(this, function(error) {
				if (error.status === 404) {
					// Default OSLC context is not set, let's proceed with default stream of target project
					this._proceedWithDefaultStream(true);
				}
				// FIXME: Should we show error message? 
			})
		);
	},

	_proceedWithDefaultStream: function(showWarning) {
/* FIXME: show warning
		if (showWarning) {
		}
*/
		dojo.when(StreamClient.getDefaultStream(this.projectId),
				  dojo.hitch(this, function(contextURI) {
					  this.context = contextURI && contextURI.length > 0 ? contextURI : null;
					  this._createPicker();
				  })
		);
	},

	_createPicker: function() {
		this._picker = new rmpw.forms.ui.SelectCreatePicker({
			creationArgs:		dojo.hitch(this, function(callback) { /* Do nothing */ }),
			selectionsLabel:	this.pickerMessages.selectionsLabel,
			creationsLabel:		this.pickerMessages.creationsLabel,
			typeLabel:			this._typeLabel,
			selectionArgs:		this._getDialogSelectionsDescriptors()
		});

		this.containerNode.appendChild(this._picker.domNode);

		// We do not allow resource creation, so disable radio buttons
		this._picker._topArea.style.display = "none";
		this._picker._horizontalRule.style.display = "none";
	},

	_getDialogSelectionsDescriptors: function() {
		var descriptors = [ ];

		if (!this.onlyDiagram && !this._excludeQuickSearch) {
			descriptors.push(this._getQuickSearchDialogDescriptor());
		}

		descriptors.push(this._getExplorerDialogDescriptor());
		descriptors.push(this._getSearchDialogDescriptor());
		descriptors.push(this._getClipboardDialogDescriptor());
		

		return descriptors;
	},

	_getQuickSearchDialogDescriptor: function() {
		return {
			classConstructor: this.typeUri === TypeConstants.RESOURCE ? ResourcesSelection : QuickSearchSelection,
			args: {
				context: this.context,
				changeset: this.changeset,
				typeUri: this.typeUri,
				propertyUri: this.propertyUri,
				propertyDomainUri: this.propertyDomainUri,
				documentUri: this.documentUri,
				resourceUri: this.resourceUri,
				internalOslcRequest: this._isInternal,
				findFragments: this.findFragments,
				onOK: dojo.hitch(this, function(item) {
					var entry = { "rdf:resource": item.rdf_about };
				    if (this._isOSLC1Protocol()) {
				    	entry["oslc_rm:label"] = item.rdfs_label || "";
				    } else {
				    	entry["oslc:label"] = item.rdfs_label || "";
				    }
					this._populateResultResponse([entry]);
				}),
				onCancel: dojo.hitch(this, function() {
					this._populateResultResponse();
				})
			},
			title: this.pickerMessages.quickSearch
		};
	},
	
	_getSearchDialogDescriptor: function() {
		var controller;
		// If a custom search is wanted (does not use the search service)
		if (this.customSearch) {
			// Use the customizable picker controller
			controller = new rmps.app.pickers.internal.CustomizedPickerController({
				typeUri: this.typeUri 
			});
		} else {
			// Use the default controller for picking DM resources
			controller = new rmps.app.pickers.internal.CLMResourcePicker_Controller({
				enableMultiSelect: this.enableMultiSelect,
				noShowAll: this.noShowAll
			});
		}
		return {
			classConstructor: RMPSPicker,
			args: {
				projectId: this.projectId,
				context: this.context,
				changeset: this.changeset,
				includeSuppliers: this._isInternal,
				filteredList: this.filteredList,
				onlyType: this.typeUri,
				onlyDiagram: this.onlyDiagram,
				controller: controller,
				onOK: dojo.hitch(this, function(selections) {
					var items = [];
					for (var index = 0; index < selections.length; index++) {
						var selection = selections[index];
						var entry = { "rdf:resource": selection.link };
				    	if (this._isOSLC1Protocol()) {
				    		entry["oslc_rm:label"] = selection.shortName || "";
				    	} else {
				    		entry["oslc:label"] = selection.shortName || "";
				    	}
						items.push(entry);
					}

					this._populateResultResponse(items);
				}),
				onCancel: dojo.hitch(this, function(selections) {
					this._populateResultResponse();
				})
			},
			title: this.pickerMessages.search
		};
	},

	_getClipboardDialogDescriptor: function() {
		return {
			classConstructor: ClipboardWidget,
			args: {
				projectUri: this._url.scheme + "://" + this._url.authority + net.jazz.ajax.contextRoot() + "/process/project-areas/" + this.projectId,
				baseline: this.context,
				isOslc: true,
				onlyType: this.typeUri,
				readOnly: false,
				disabledList: this.filteredList,
				onOkButton: dojo.hitch(this, function(selections) {
					dojo.when(ResourceInfoClient.getResourceInfo(selections, ["uri","shortName"]),
							dojo.hitch(this, function(info) {
								var items = [];
								for (var index = 0; index < info.length; index++) {
									var selection = info[index];
									var entry = { "rdf:resource": selection.uri };
				    				if (this._isOSLC1Protocol()) {
				    					entry["oslc_rm:label"] = selection.shortName || "";
				    				} else {
				    					entry["oslc:label"] = selection.shortName || "";
				    				}
									items.push(entry);
								}
								this._populateResultResponse(items);
							}),
							dojo.hitch(this, function(error) {
								this._populateResultResponse();
							})
					);
				}),
				onClickCancelButton: dojo.hitch(this, function() {
					this._populateResultResponse();
				})
			},
			title: this.pickerMessages.clipboard
		};
	},
	
	_getExplorerDialogDescriptor: function() {
		return {
			classConstructor: ExplorerWidget,
			args: {
				projectUri: this._url.scheme + "://" + this._url.authority + net.jazz.ajax.contextRoot() + "/process/project-areas/" + this.projectId,
				contextUri: this.context,
				onlyType: this.typeUri,
				readOnly: false,
				disabledList: this.filteredList,
				enableMultiSelect: this.enableMultiSelect,
				onOkButton: dojo.hitch(this, function(selections) {
					var items = [];
					for (var index = 0; index < selections.length; index++) {
						var selection = selections[index];
						var entry = { "rdf:resource": selection.rdf_about };
				    	if (this._isOSLC1Protocol()) {
				    		entry["oslc_rm:label"] = selection.label || "";
				    	} else {
				    		entry["oslc:label"] = selection.label || "";
				    	}
						items.push(entry);
					}
					this._populateResultResponse(items);
				}),
				onClickCancelButton: dojo.hitch(this, function() {
					this._populateResultResponse();
				})
			},
			title: this.pickerMessages.explorer
		};
	},

	_getRemoteFrameProtocol: function() {
		if (this._remoteFrame === null) {
			this._remoteFrame= new jazz.ui.RemoteChooserResponder();
		}
		return this._remoteFrame;
	},

	_populateResultResponse: function(selectedItems) {
		if (dojo.isFunction(this.onOK)) {
			this.onOK(selectedItems);
			return;
		}
		
		var results = (selectedItems && selectedItems.length > 0) ? selectedItems : "";
		var response = {};
		if (this._isOSLC1Protocol()) {
			response["oslc_rm:message"] = "oslc.select";
			response["oslc_rm:results"] = results;
		} else {
			response["oslc:message"] = "oslc.select";
			response["oslc:results"] = results;
		}
		
		var data = dojo.toJson(response);
		
		this._getRemoteFrameProtocol().sendResponse(data);
	},

	destroy: function() {
		if (this._picker) {
			this._picker.destroy();
			delete this._picker;
		}
		this.inherited(arguments);
	}
});
	
/**
 * Customization of rmps.app.pickers.internal.PickerController that customizes DM resource selection dialog. 
 */
dojo.declare("rmps.app.pickers.internal.CLMResourcePicker_Controller", rmps.app.pickers.internal.PickerController, {
		
	enableMultiSelect: true,
	allowSearchSystemDomains: false,
	allowEmptySearches: true,
	enableShowAll: true,	
	orderBy: [], // query results are already ordered by name
	pageSize: 100,
	limit: 600, // maximum number of returned results
	limitResultTableHeight: true,
	
	constructor: function(params){
		if (params) {
			if (params.noShowAll) {
				this.allowEmptySearches = false;
				this.enableShowAll = false;
				delete params.noShowAll;
			}
			dojo.mixin(this, params);				
		}
	},
	
	validateQuery: function(queryToExecute) {
		if (this.allowEmptySearches) {
			// parent checks for empty search and returns false in that case, here we want to allow that
			return true;
		} else {
			return this.inherited(arguments);
		}
	},
});

}());