/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("rmps.app.pickers.internal.ContextPicker");

/*######################################### */
/*Since the context picker will be invoked as delegated UI (possibly from
 * another server), these imports are required to invoke auth dialogs, and
 * users' credentials checking utils. Otherwise, the users,
 * will not be authorized when trying to search for an element
 * even if they are already logged in */
dojo.require("jazz.app.proxy");
dojo.require("jazz.app.auth");
dojo.require("jazz.app.events");
/*######################################### */

dojo.require("jazz.ui.RemoteChooserResponder");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
		
dojo.requireLocalization("rmps.app.pickers", "ContextPickerMessages");

var messages =dojo.i18n.getLocalization("rmps.app.pickers", "ContextPickerMessages");

/**
 * An OSLC style delegated selection dialog for selecting DM contexts (baselines or streams).  The picker can either
 * be initialized via parameters in the window's URL (i.e., in a delegated UI scenario), or directly via parameters
 * passed in to the constructor.  In the direct instantiation case, results are returned by a provided function; otherwise,
 * the picker returns results using the OSLC Window Name or Post Message protocols.
 * This class is not meant to be instantiated directly; instead, use one of the subclasses of this widget.
 */
dojo.declare("rmps.app.pickers.internal.ContextPicker", [dijit._Widget, dijit._Templated], {
	
	messages: messages,
	templateString: '<div class="root"">'+
						'<div><span dojoAttachPoint="_messageNode"/></div>' +
						'<select size="5" dojoAttachPoint="_selectionList" dojoAttachEvent="onchange: _onSelectionChange"></select>' +
						'<div class="buttonContainer">' +
							'<button dojoAttachPoint="_okButton" dojoAttachEvent="onclick: _contextSelected" disabled="true">${messages.ok_button_label}</button><button dojoAttachEvent="onclick: _handleCancel">${messages.cancel_button_label}</button>' +
						'</div>' + 
					'</div>',
		
	//mixin			
    projectId: null, //String
    onOk: null,		//Function (can be null, in which case results will be returned via the OSLC delegated UI protocol)
    onCancel: null,	 //Function (can be null, in which case results will be returned via the OSLC delegated UI protocol instead)
	selection: null,
	_contexts: null, //[]
	
	//
		
	constructor: function(params) {
	    if (params.projectId) {
	    	//the picker is being instanstiated manually; initialize from incoming params
	    	this.projectId = params.projectId;
	    	this.onOk = params.onOk;
	    	this.onCancel = params.onCancel;
	    } else {
	    	//initialize from arguments in URL string
			var searchString = location.search.replace(/^\?/,'');
			this._pickerArguments = dojo.queryToObject(searchString);
			this.projectId = this._pickerArguments.project;
		}
		this._contexts = [];
	},
	
	postCreate: function() {
		var deferred = this.fetchContexts(this.projectId); 
		deferred.addCallback(this, function(contexts){
			var selection = this.selection;
			for (var i = 0; i<contexts.length; i++) {
				var option = dojo.create("option", null, this._selectionList);
				var context = contexts[i];
				option.value = context.uri;
				if (selection && context.uri === selection.uri) {
					option.selected = true;
					// Enable the ok button if there is a selection
					this._okButton.disabled = false;
				}
				var name = context.name || context.label;
				option.appendChild(dojo.doc.createTextNode(name));
			}
			this._contexts = contexts;
		});
		deferred.addErrback(this, function(error){
			var textNode = dojo.doc.createTextNode(messages.error_loading_contexts);
			this._messageNode.appendChild(textNode);
		});
	},
	
	/**
	 * Returns the appropriate responder object for conveying a result back to the client (see OSLC
	 * delegated UI specification for details).
	 */
	_getRemoteFrameProtocol: function(){
		if(!this._remoteFrame){
			this._remoteFrame= new jazz.ui.RemoteChooserResponder();
		}
		return this._remoteFrame;
	},
	
	/**
	 * Called when a new entry in the select box is chosen.
	 */
	_onSelectionChange: function(event) {
		//if it's not already, make sure the "OK" button is enabled
		this._okButton.disabled = false;
		var selectedIndex = this._selectionList.selectedIndex;
		this.selection = this._contexts[selectedIndex];
	},
	
	/**
	 * Called when the OK button is pushed.  Relays selected result back to the caller
	 * via the appropriate protocol.
	 */
	_contextSelected: function(event) {
		if (this.onOk) {
			this.onOk(this.selection);
		} else {
			var result = [{ "oslc:label" : this.selection.name || this.selection.label, "rdf:resource" : this.selection.uri }];
			var response = {"oslc:results" : result};
			this._getRemoteFrameProtocol().sendResponse(dojo.toJson(response));
		}
	},
	
	/**
	 * To be implemented by subclasses.  Expected to return a deferred object that fetches the contexts
	 * to be displayed in the picker.  The deferred should return an array of contexts; contexts may 
	 * be of any type, but are expected to have attributes "name" (or "label") and "uri" at minimum.
	 * 
	 * param - projectId - the ID of the project to get contexts for
	 */
	fetchContexts: function(projectId /*String*/) {
		
	},
	
	/**
	 * Called when the cancel button is pressed.  Returns an empty response object.
	 */
	_handleCancel: function(event) {
		if (this.onCancel) {
			this.onCancel();
		} else {
			this._getRemoteFrameProtocol().sendResponse(dojo.toJson({}));
		}
	}
	
});
}());