/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.pickers.internal.CustomizedPickerController");

dojo.require("rmps.app.pickers.internal.PickerController");
dojo.require("rmpw.common.util.IdentifierUtil");

/**
 * Customization of rmps.app.pickers.internal.PickerController that customizes DM resource selection 
 * dialog. Customizations consist of changing the values of some parameters from the parent PickerController 
 * as well as a different way of search (not using SearchService but ValidationFrontService) 
 */
dojo.declare("rmps.app.pickers.internal.CustomizedPickerController", rmps.app.pickers.internal.PickerController, {
	
	enableMultiSelect: false,
	allowSearchSystemDomains: true,
	allowEmptySearches: true,
	enableShowAll: true,	
	orderBy: [], // query results are already ordered by name
	pageSize: 100,
	limit: 600, // maximum number of returned results
	limitResultTableHeight: true,
	
	constructor: function(params){
		this.resourceUri = params.resourceUri;
		this.propertyUri = params.propertyUri;
		this.propertyDomainUri = params.propertyDomainUri;
		this.typeUri = params.typeUri;
		this.internalOslcRequest = params.internalOslcRequest;
		
		this.enableMultiSelect = params.enableMultiSelect;
	},
	
	doSearch: function(queryToExecute, query){
		var whereClause = undefined;
		if (this.definableTypes && this.definableTypes.length > 1) {
			whereClause = "";
			for (var i=0, lg=this.definableTypes.length; i<lg; i++) {
				var whereLine = "?type rdfs:subClassOf* <" + this.definableTypes[i] + "> . ?value rdf:type ?type";
				if (i==0) {
					whereClause += "{ " + whereLine + " }";
				}
				else 
					whereClause += "UNION {" + whereLine + " }";
			}
		}
		var self = this;
		var args = {
			url: net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/modelvalidation/values",
			handleAs: 'json',
			headers: {
				"Accept" : "application/json",
				"X-ibm-rmps-internal" : "true"
			},
			content: {
				resourceUri: this.resourceUri ? this.resourceUri : (this.form && this.form._resource.id ? this.form._resource.id : (this.documentObject ? this.documentObject.rdf_about : (this.form ? this.form._resource.documentObject.rdf_about : null))),
				propertyUri: this.propertyUri, 
				adjustedDomainUri: this.propertyDomainUri, 
				adjustedRangeUri: this.typeUri,
				userText: query.text,
				whereClause: whereClause,
				thisProjectOnly: queryToExecute.includeSystemDomains === "true" ? "false" : "true",
				limit: this.limit,			
				internalOslcRequest: this.internalOslcRequest !== undefined ? this.internalOslcRequest : ""
			},
			load: function(response, ioArgs) {
				var totalResults = response.length;
				if (totalResults > self.limit) {
					// remove the last one (there should only be one extra)
					response.splice(totalResults-1, 1);
					// indicate that there are more results that were not returned
					totalResults = -1 * (totalResults - 1);
				}
				var result = {
					entries: response,
					id: rmpw.common.util.IdentifierUtil.guid(),
					totalResults: totalResults,
					uri: ioArgs.url
				 };
				return result;
			},
			error: function(error) {
				if (error.dojoType != "cancel") {
					rmps.core.error.create(rmpw.common.util.ErrorUtil.getErrorMessage(error)).dispatch();
				}
				delete this._currentValueRequest;
			}
		};
		var ccObject = {}
		if (queryToExecute.baseline) {
			ccObject[rmps.core.util.IRmpsConstants.BASELINE] = queryToExecute.baseline;
		}
		if (queryToExecute.changeset) {
			ccObject[rmps.core.util.IRmpsConstants.CHANGESET] = queryToExecute.changeset;
		}
		rmps.core.util.xhrUtil.addParamater(args, ccObject);

		return rmps.core.client.xhrGet(args, true);		
	},
	
	validateQuery: function(queryToExecute) {
		// parent checks for empty search and returns false in that case, here we want to allow that
		return true;
	},
	
	getRowResourceUri: function getRowResourceUri(rowObj) {
		return rowObj.value;
	},
	
	getRowShortName: function getRowShortName(rowObj) {
		return rowObj.rdfs_label;
	} 
});

})();