/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("rmps.app.pickers.internal.LinkTypePicker");

dojo.require("rmps.app.pickers.internal.ContextPicker");
dojo.require("rmps.app.ui.internal.editors.views.link.LinkClient");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.pickers", "LinkTypePickerMessages");
var messages = dojo.i18n.getLocalization("rmps.app.pickers", "LinkTypePickerMessages");

// An OSLC style delegated selection dialog for selecting DM link types.
dojo.declare("rmps.app.pickers.internal.LinkTypePicker", [rmps.app.pickers.internal.ContextPicker], {

	contextURI: null,
	selectionSize: null,
	multiple: null,

	constructor: function(params) {
        if (params.projectId) {
            this.contextURI = params.contextURI;
            this.selectionSize = params.selectionSize;
            this.multiple = params.miltiple;
		}
	},

	postCreate: function() {
		if (this.selectionSize) {
			this._selectionList.size = this.selectionSize;
		}

		if (this.multiple) {
			this._selectionList.multiple = "multiple";

			var deferred = this.fetchContexts(this.projectId);
			deferred.addCallback(this, function(linkTypes) {
				var selection = this.selection ? dojo.fromJson(this.selection) : null;
				var allSelected = selection && selection[messages.allDeployed];
				for (var l = 0; l < linkTypes.length; l++) {
					var option = dojo.create("option", null, this._selectionList);
					var linkType = linkTypes[l];
					
					option.value = linkTypes.about;
					option.appendChild(dojo.doc.createTextNode(linkType.title));

					if (allSelected || (selection && selection[linkType.title] === linkType.about)) {
						option.selected = true;
						// Enable the ok button if there is a selection
						this._okButton.disabled = false;
					}
				}
				this._contexts = linkTypes;
			});
			deferred.addErrback(this, function(error){
				var textNode = dojo.doc.createTextNode(messages.error_loading_linktypes);
				this._messageNode.appendChild(textNode);
			});
		} else {
			this.inherited(arguments);
		}
	},

	_onSelectionChange: function(event) {
		if (this.multiple) {
			if (this._selectionList.selectedIndex === -1) {
				this.selection = null;
				this._okButton.disabled = true;
			} else {
				var allSelected = true;
				this._okButton.disabled = false;
				var selection = {};
				for (var o = 0; o < this._selectionList.options.length; o++) {
					if (this._selectionList.options[o].selected) {
						var linkType = this._contexts[o];
						selection[linkType.title] = linkType.about;
					} else {
						allSelected = false;
					}
				}
				if (allSelected) {
					// reset
					selection = {};
					selection[messages.allDeployed] = true;
				}
				this.selection = dojo.toJson(selection);
			}
		} else {
			this.inherited(arguments);
		}
	},

	/**
	 * Overridden from superclass. Fetches link types for the given project ID.
	 */
	fetchContexts: function(projectId) {
		var def = rmps.app.ui.internal.editors.views.link.LinkClient.getLinkTypes(this.projectId, {orderBy: "title"}, this.contextURI); 
		def.addCallback(this, function(linkTypes) {
			/*
			 * The link type objects returned by the client already have the appropriate format,
			 * so just return them as-is.
			 */
			return linkTypes;
		});
		return def;
	}
});
}());