/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.pickers.internal.RMPSPicker");
dojo.require("dojox.html");
dojo.require("rmps.app.pickers.internal.PickerController");
dojo.require("rmps.app.ui.internal.search.SearchResults");
dojo.require("rmps.app.ui.internal.search.QuickSearch");
dojo.require("dojo.string");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.RadioButton");
dojo.require("dijit._Widget");
dojo.require("rmps.ui.ErrorReportController");
dojo.require("jazz.app.context");
dojo.require("rmps.app.ui.internal.context.ContextSelection");
dojo.require("rmps.app.ui.internal.search.QuickSearch");
dojo.require("rmps.core.util.ContextUtils");
dojo.require("rmps.app.ui.internal.util.DomainRegistryUtil");

dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.ui.RemoteChooserResponder");
dojo.require("rmps.core.changesets.ChangesetClient");

var ContextUtils= rmps.core.util.ContextUtils;
var ChangesetClient = rmps.core.changesets.ChangesetClient;
var DEFAULT_OSLC_CONTEXT_PROPERTY = "defaultOslcContext";
var ROW_CONFIG_PROP = "_tableWidgetRowConfig";

var FindInput= function(node) {
    var result = null;
    for (var index=0; index<node.childNodes.length; index++) {
        var current= node.childNodes[index];
        var tagName = current.tagName;
        if (tagName && current.tagName.toLowerCase() === "input") {
            return current;
        } else if (result) {
            break;
        } else {
            result = FindInput(current);
        }
    }
    return result;
};

/**
 * DM resource picker that uses rmps.app.pickers.internal.PickerController (or its customization) 
 * which controls some aspects of search - for more details, see documentation for PickerController.
 * 
 */
dojo.declare("rmps.app.pickers.internal.RMPSPicker", [dijit._Widget, dijit._Templated], {

	"class": "root",

	messages: dojo.i18n.getLocalization("rmps.app.pickers", "RMPSPickerMessages"),
	templateString: dojo.cache("rmps.app", "pickers/internal/templates/RMPSPickerTemplate.html"),

	// private properties
	_controller: null,
	_searchResults: null,
	_quickSearch: null,
	_pickerArguments: null,
	_selectedRows: [],
	
	projectId: null,
	_selectedContext: null,
	filteredList: null,
	// restrict the picker to a certain type
	onlyType: null,
	onlyTypeLabel: null,
	// restrict the picker to diagrams
	onlyDiagram: null,

	// All external applications should not use current context
	context: null, // required parameter
	changeset: null,
	includeSuppliers: null,

	constructor: function(params){
        if(params){
            var searchString = location.search.replace(/^\?/,'');
            this._pickerArguments = dojo.queryToObject(searchString);
            this.projectId = this._pickerArguments.project;

            // to restrict the picker to only search diagrams
            this.onlyDiagram = this._pickerArguments.diagram;

            // to restrict the picker to a given type
            this.onlyType = (this._pickerArguments.onlyType)?decodeURIComponent(this._pickerArguments.onlyType):null;

            /* Use context passed in the uri or use baseline passed as a parameter
               if null/undefined then context is either default OSLC context or default stream */
            this.context = this._pickerArguments.context ? this._pickerArguments.context : null;
            this.changeset= this._pickerArguments.changeset ? this._pickerArguments.changeset : params.changeset;
            this.findFragments = params.findFragments;
            
            if (params.includeSuppliers) {
            	this.includeSuppliers = params.includeSuppliers;
            }
			
			if (params.controller) {
				this._controller = params.controller;
			}
        }
        this.filteredList= [];
    },

	buildRendering: function() {
		this.inherited(arguments);
		this.containerNode = this.domNode;
	},

	postCreate: function() {
		if (!this._controller) {
			this._controller = new rmps.app.pickers.internal.PickerController(this);
		} else {
			this._controller._pickerWidget = this;
		}
		if (this._controller.allowSearchSystemDomains) {
			var dspDef = new dojo.Deferred();
			var isDsp, isSystem;
			rmps.app.ui.internal.util.DomainRegistryUtil.isCurrentProjectDSP(dojo.hitch(this, function(_isDsp, _isSystem){
				isDsp = _isDsp;
				isSystem = _isSystem;
				dspDef.callback();
			}));
			dspDef.addCallback(dojo.hitch(this, function(){
				this.initialize(isDsp, isSystem);
			}));
		} else {
			this.initialize(null, null);
		}
	},
		
	initialize: function(isDsp, isSystem) {
		// The SearchService uses ContextRestrictionsUtils method detectContextRestrictions() to compute the
		// contexts (project areas) to be included in the search. It will only include System Domain
		// projects if the includeSuppliers flag is true. We want to be able to include just dependent
		// projects but not System Domain projects so the valid combination of flags to do this
		// is includeSuppliers = true but includeSystemDomains = false.
		// Now, the context radio buttons will be shown if system domains are to be
		// searched or if only dependent projects (suppliers) are to be searched.
		var defaultIncludeSysDom = false;
		var defaultIncludeSuppliers = false;
		if (this._selectedContext.childNodes.length === 0) {
			if (this._controller.allowSearchSystemDomains || this.includeSuppliers) {
				// give user the option to choose between just the current project, 
				// and the current project will all projects it has a dependency to
				this._selectedContext.style.display = "none";
				if (isDsp && !isSystem) {
					// for users' domains project, set default context to include dependent domains
					this._contextThisAndDependProj.checked = "checked";
					defaultIncludeSysDom = true;
					defaultIncludeSuppliers = true;	// must be true or else system domains will not be checked
				} else {
					this._contextThisProj.checked = "checked";
				}
			} else {
				this._contextThisProj.style.display = "none";
				this._lblCntext1.style.display = "none";				
				this._contextThisAndDependProj.style.display = "none";
				this._lblCntext2.style.display = "none";
				if (this.changeset) {
					dojo.when(ChangesetClient.getChangeset(this.changeset), dojo.hitch(this, this._updateSelectedContext));
				}
				else {
					dojo.when(ContextUtils.getContextDeferred(this.context, this.projectId), dojo.hitch(this, this._updateSelectedContext));
				}
			}
		}
		this._quickSearch = new rmps.app.ui.internal.QuickSearch_({
			projectId  : this.projectId,
			baseline: this.context,
			changeset: this.changeset,
			includeSuppliers: defaultIncludeSuppliers,
			includeSystemDomains: defaultIncludeSysDom, 
			onlyType: this.onlyType,
			onlyTypeLabel: this.onlyTypeLabel,
			onlyDiagram: this.onlyDiagram,
			_controller: this._controller,
			resourcesOnly: !this.onlyType && !this.findFragments,
			allowEmptySearches: this._controller.allowEmptySearches
		});
		
		// Display Show All link only if controller allows it
		if (!this._controller.enableShowAll) {
			this.showAllLink.style.display = "none";	
		} else {			
			this.connect(this.showAllLink,"onclick", function (e) {
				// the same happens as when the user clicks on the search button while search box is empty
				dojo.stopEvent(e);
				this._quickSearch.searchInputWidget._searchText.value = "";
				this._quickSearch.searchInputWidget._currentQuery = "";
				this._quickSearch.searchInputWidget._performExecute();				
			});
		}

		setTimeout(dojo.hitch(this, function() {
			var inputNode = FindInput(this._quickSearch.domNode);
			inputNode.focus();		
		}), 1);
		this.quickSearch.appendChild(this._quickSearch.domNode);
		
		var errorReport = new rmps.ui.ErrorReportController();
		errorReport.startup();
		this.errorReportNode.appendChild(errorReport.domNode);
	},

	_updateSelectedContext: function(contextVar) {
		this._selectedContext.appendChild(dojo.doc.createTextNode(contextVar.name));
	},

	onClickOKButton: function(e) {
		if (dojo.isFunction(this.onOK)) {
			this.onOK(this._selectedRows);
			return;
		}

		this.onClose(this._selectedRows);
	},
	
	onClose: function(selections){
		/*To be implemented by other clients */
	},

	onClickCancelButton: function(e) {
		if (this._controller && this._controller.cancelSearchRequest) {
			this._controller.cancelSearchRequest();
		}
		if (dojo.isFunction(this.onCancel)) {
			this.onCancel(this._selectedRows);
			return;
		}

		this.onClose();
	},

	_handleWindowResize: function(event) {
		setTimeout(dojo.hitch(this, this.resize), 0);
	},

	resize: function() {
		/*Empty Block */
	},

	destroy: function() {
		if (this._searchResults) {
			this._searchResults.destroy();
			delete this._searchResults;
		}
		this.inherited(arguments);
	},

	onCheckboxChange: function(rowConfig, checked){
		if(checked) {
			this._addSelectedRow(rowConfig);
		} else {
			this._removeSelectedRow(rowConfig);
		}
		this.okCmd.disabled= this._selectedRows.length <= 0;
	},

	_addSelectedRow : function (selectedRow) {
		if (this._controller.enableMultiSelect && this._selectedRows) {
			this._selectedRows.push(selectedRow);
		} else {
			this._selectedRows = [selectedRow];
		}
	},
	
	clearSelections: function (){
		this.okCmd.disabled= true;
		this._clearSelectedRows();
	},
	
	_clearSelectedRows: function (){
		this._selectedRows= [];
	},
	
	_removeSelectedRow: function (selectedRow){
		var rowIndex = this._findRow(selectedRow, this._selectedRows);
		if (rowIndex === -1){
			return;
		}
		this._selectedRows.splice(rowIndex, 1);
	},

	_findRow : function (selectedRow, selectedRowSet){
		for (var i=0; i<selectedRowSet.length; i++){
			if (selectedRowSet[i].link === selectedRow.link){
				return i;
			}
		}
		return -1;
	},
	
	showResultsForResourceInfo: function(resourceInfos, query, preserveFilter){
		this._prepareSearch(resourceInfos, preserveFilter);
		this._searchResults._showResultsForResourceInfo(resourceInfos,query, preserveFilter);
		this.searchResults.appendChild(this._searchResults.domNode);
	},
	
	hideSearchResults: function(){
		if(this._searchResults&& this._searchResults._resultTable){
			this._searchResults._resultTable.setTableData(this._searchResults._headers, []);
			this._searchResults._resultTable.update(false, 0, 0, 0, 0, null, false,null);			
		}
	},

	showSearchResults: function(queryResults, preserveFilter) {
		this._prepareSearch(queryResults, preserveFilter);
		this._searchResults.showResults(queryResults, preserveFilter, this.context);
		this.searchResults.appendChild(this._searchResults.domNode);
	},

	_prepareSearch: function(queryResults, preserveFilter){
		if (!this._searchResults) {
			this._searchResults = new rmps.app.ui.internal.search.SearchResults_({
					_quickSearch: this._quickSearch,
					filteredList: this.filteredList,
					controller: this._controller
					});
			this._searchResults.startup();
			this._searchResults._resultTable.onCheckboxChange = dojo.hitch(this , this.onCheckboxChange);
		}

		this._searchResults.checkBoxes= this._selectedRows;
	},

	showStatusNode: function(text) {
		this.statusNode.innerHTML = "";
		if (text !== null) {
			this.statusNode.appendChild(dojo.doc.createTextNode(text+"..."));	//$NON-NLS-1$
			this.statusNode.style.display= "inline";
		}else {
			this.statusNode.style.display= "none";
		}
	},

	hideStatusNode: function() {
		this.statusNode.innerHTML = "";
		this.statusNode.style.display= "none";
	},
	
	// When "this Project" context button is selected, restrict
	// searches to just this project and configuration.
	_onSelectThisProj: function() {
		this._contextThisAndDependProj.checked = "";
		this._quickSearch.includeSuppliers = false;
		this._quickSearch.includeSystemDomains = false;
	},
	
	// When "all dependent projects" is selected, include
	// all supplier projects and optionally include system domain projects.
	// The UI and the user cannot choose whether to include system domain projects.
	// This is controlled by the caller who is performing the search. Unless
	// an ontology is being edited, system domains are typically not included
	// in the search.
	_onSelectThisAndDependProj: function() {
		this._contextThisProj.checked = "";
		this._quickSearch.includeSuppliers = true;
		this._quickSearch.includeSystemDomains = this._controller.allowSearchSystemDomains;
	}

});



dojo.declare("rmps.app.ui.internal.search.SearchResults_", rmps.app.ui.internal.search.SearchResults, {

	tableClass: "rmps.app.ui.internal.search._TableEx",
	_quickSearch: null,
	_machingLabel: null,
	checkBoxes: null,
	controller: null,
	messages: dojo.i18n.getLocalization("rmps.app.pickers", "RMPSPickerMessages"),
	
	postCreate: function() {		
		this.createToolbar = false;
		this._showclipboardIcon = false;
		this.inherited(arguments);
		if (this.controller) {
			this._resultTable.enableMultiSelect = this.controller.enableMultiSelect;
			if (this.controller.limitResultTableHeight) {
				dojo.addClass(this.containerNode, "containerMaxHeight");
			}
		}		
	},
	
	_showResultsForResourceInfo: function(resourcesInfo,query, preserveFilter){
		this._queryResult = {};
		this._queryResult.entries= resourcesInfo;
		var tableData= [];
		for(var index=0; index<resourcesInfo.length; index++){
			var resourceInfo= resourcesInfo[index];
			var INFO={
					"shortName": [],
					"longName": [],
					"displayType": [],
					"modificationTime": [{"style": { color: "#808080"}}]
			 } ;
			
			var columns= [];
			
			columns.push({checkBox: true, link: false});
			columns.push({data: query.text, link: false});
			
			for (var att in INFO) {
				var info= INFO[att];
				var val= resourceInfo[att];
				if(val instanceof Date){
					val= dojo.date.stamp.toISOString(val);
				}
				var object= {data: val, link: false};
				for (var sub in info) {
					object[sub]= info[sub];
				}
				columns.push(object);
			}
			tableData.push({
				"columns": columns,
				link: resourceInfo.uri,
				resourceUri: resourceInfo.uri
			});
		}
		this._resultTable.setTableData(this._headers, tableData);
		
		var secondaryOrder;
		if (query.orderBy && query.orderBy.length > 1) {
			secondaryOrder = query.orderBy.slice();
			secondaryOrder.splice(0, 1);
		}
		
		this._resultTable.update(preserveFilter,
			query.page, query.pageSize,
			resourcesInfo.length, query.limit,
			query.orderBy && query.orderBy.length > 0 ? query.orderBy[0].id : null,
			query.orderBy && query.orderBy.length > 0 ? query.orderBy[0].isAscending : false,
			secondaryOrder);
		if(resourcesInfo.length<1){
			this._resultTable.messageNode.innerHTML = dojo.string.substitute(messages.tooManyResults, [Math.abs(queryResult.totalResults)]);
		}
		this._resultTable.pagingControl.domNode.style.display = "";
		this._resultTable.footerPagingControl.domNode.style.display = "";
	},

	createTableHeaders: function(parentNode) {
		var headers= this.inherited(arguments);
		headers.unshift(this.createTableHeader("checkBox", "", {filterParticipant: false, isOrderable: false}));
		return headers;
	},

	createColumnObject: function(rowObj, columnId) {
		var result= this.inherited(arguments);

		/*We do not want by any means to enable having links at any column, as this
		 * will not work in the picker iframe */
		result.link= false;

		return result;
	},

	_checkBoxHandler: function(result, rowObj, owned, resourceUri, presentationUri) {
		result.checkBox = true;
	},

	createTable: function() {
		var table= this.inherited(arguments);
		table._quickSearch= this._quickSearch;
		return table;
	},
	
	createHeader: function(parentNode) {
		// no header
		this.headerContainerNode.style.display = "none";
	},

	createExplorerAction: function(toolbar, rowObj, resourceUri, presentationUri) {
		/*Do Nothing */
	},
	
	addButtons: function(toolbar) {
		/*Do Nothing */
	},

	getRowResourceUri: function getRowResourceUri(rowObj) {
		if (this.controller && this.controller.getRowResourceUri) {
			return this.controller.getRowResourceUri(rowObj);
		}
		return this.inherited(arguments);
	},
	
	getRowShortName: function getRowShortName(rowObj) {
		if (this.controller && this.controller.getRowShortName) {
			return this.controller.getRowShortName(rowObj);
		}
		return this.inherited(arguments);
	}
});


dojo.declare("rmps.app.ui.internal.search._TableEx", rmps.app.ui.internal.search._Table, {

	_quickSearch: null,
	_idToIdx: {},

		/*Only show the row if its resource uri is not in the fitlered list */
	setTableData: function(headers, tableData) {
		this._headerConfigs = headers;
		this._tableData = tableData;
	},
	
	getReorderHref: function(orderBy, isAscending, secondaryOrder) { 
		return "";
	},

	_populateColumn: function(rowConfig, tableRow, rowIdx, columnConfig, tableCell, colIdx) {
		var isChecked= false;
		dojo.forEach(this.search.checkBoxes, function(row) {
			if (row.link === rowConfig.resourceUri){
				isChecked= "checked";
				return;
			}
		});

		if (columnConfig.checkBox) {
			var checkBox = this.enableMultiSelect ? 
					new dijit.form.CheckBox(this._getCheckBoxArgs(isChecked, rowConfig), "checkBox") : 
					new dijit.form.RadioButton(this._getCheckBoxArgs(isChecked, rowConfig), "checkBox");
			if (this.search.filteredList && dojo.indexOf(this.search.filteredList, rowConfig.resourceUri)!==-1){
				checkBox.attr("disabled", true);
			}
			tableCell.appendChild(checkBox.domNode);
		} else {
			// For search resource in this DM resource picker, do not
			// allow the label of the resource or its parent to be hyper links
			// to the resource or else the corresponding editor tries to open
			// the resource in the search dialog itself ... not pretty (WI 37759)
			if (columnConfig.resourceReference) {
				columnConfig.resourceReference = false;
			}
			
			this.inherited(arguments);
		}
	},
	
	_getCheckBoxArgs: function(isChecked, rowConfig) {
		return {
		    name: "checkBox",
            value: "agreed",
            checked: isChecked,
            onChange: dojo.hitch(this ,function(event) {
                var link = rowConfig.resourceUri;
                if(link) {
//					var idx = link.lastIndexOf('#');
//					if (idx != -1) {
//						link = link.substr(0, idx);
//					}
//                  var rowItem = {
//                      name: rowConfig.columns[this._idToIdx["name"]].data,
//                      link: link
//                  };
					var rowItem = {};
					var ignoreList= ["checkBox","actions"];
					for (var i = 0; i < this._headerConfigs.length; ++i) {
						var headerConfig= this._headerConfigs[i];
						var id= headerConfig.id;
						this._idToIdx[id] = i;
						if (dojo.indexOf(ignoreList, id) === -1){
							var column= rowConfig.columns[i];
							rowItem[id]= column.data;
						}
					}
					rowItem.link= rowConfig.resourceUri;
					this.onCheckboxChange(rowItem, event);
				}
            })
		}
	},

	_handleTbodyMouseClick: function(e) {
		/*We do not allow navigation for the table links in the picker */
		return;
	},

	onCheckboxChange: function(rowConfig, checked){
		/*Do Nothing */
	},

	_constructPageSizeDropDown: function() {
		/*we do not want to allow the user to change the value of the number of the displayed entries;
		 * this is in order to avoid having any layout problems in the Resource Picker in which will added
		 * in other contents */
		this._pageSizeSelect.style.display= "none";
		this._itemsPerPage.style.display= "none";
	},

	onPreviousClick: function(e) {
		dojo.stopEvent(e);
		this.doQuery(this.pagingControl.page-1, this.pagingControl.pageSize);
	},

	onNextClick: function(e) {
		dojo.stopEvent(e);
		this.doQuery(this.pagingControl.page+1, this.pagingControl.pageSize);
	},

	doQuery: function(page, pageSize){
		var query= this.createQuery(page, pageSize);
		var queryString= query.toQueryString();
		var timestamp = new Date().getTime();
		this._quickSearch.doQuery(query);
	},

	createQuery:function(page, pageSize){
		var queryResult = this.pagingControl.table.search.getQueryResult();
		var query = dojo.clone(queryResult.query);
		query.pageSize = pageSize;
		query.page = page;
		query.token = queryResult.id;
		return query;
	},

	_filterChanged: function(filter) {
		this._filteredOutRowCount = 0;
		var i;
		if (!filter) {
			for (i = 0; i < this._rowCache.length; ++i) {
				this._rowCache[i].style.display = "";
			}
		} else {
			var shortcut = this._prevFilter && filter.indexOf(this._prevFilter) === 0;
			
			for (i = 0; i < this._rowCache.length; ++i) {
				var config = this._tableData[this._rowCache[i][ROW_CONFIG_PROP]];
				if (shortcut) {
					if (this._rowCache[i].style.display === "") {
						if (this._filterRow(filter, config)) {
							this._rowCache[i].style.display = "none";
							this._filteredOutRowCount++;
						}
					} else {
						this._filteredOutRowCount++;
					}
				} else if (this._filterRow(filter, config)) {
					this._rowCache[i].style.display = "none";
					this._filteredOutRowCount++;
				} else {
					this._rowCache[i].style.display = "";
				}
			}
		}
		this._prevFilter = filter;
		this._updateFilterMessage();
	}

});


dojo.declare("rmps.app.ui.internal.QuickSearch_", rmps.app.ui.internal.search.QuickSearch, {

    _controller: null,
	pageSize: 5,
	resourcesOnly: true,
	
	postCreate: function() {
		this.inherited(arguments);
		if (this._controller.pageSize) {
			this.pageSize = this._controller.pageSize;
		}
		if (this._controller.orderBy) {
			this.orderBy = this._controller.orderBy;
		}
		if (this._controller.limit) {
			this.limit = this._controller.limit;
		}		
	},

	doQuery: function(query) {
		this._controller.search({query: query.toQueryString()});
	}
});

}());
