/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.pickers.internal.SelectionsPicker");

dojo.require("dijit._WidgetBase");
dojo.require("dijit._Templated");
dojo.require("dijit.layout._LayoutWidget");
dojo.require("jazz.util.html");

/*
 * Provides a selection dialog in where the users get a selection input, and when they select an entry, it will
 * render a given wdiget contents in a lazy manner. 
 */
dojo.declare("rmps.app.pickers.internal.SelectionsPicker", [dijit._Widget, dijit._Templated], {

	"class": "rmps-ui-resources-picker",

	templateString: dojo.cache("rmps.app", "pickers/internal/templates/SelectionsPicker.html"),
	widgetsInTemplate: true,
	
	argsList: null,
	selectionsLabel: null,
	unique: null,
	messages: null,
		
	_contentsCache: null,
	
	constructor: function(params){
		this._contentsCache= {};
	},
	
	postMixInProperties: function(){
		dojo.requireLocalization("rmps.app.pickers.internal", "SelectionsPickerMessages");
		this.messages =dojo.i18n.getLocalization("rmps.app.pickers.internal", "SelectionsPickerMessages");
		
		dojo.requireLocalization("rmps.ui.common", "CommonMessages");
		this._commonMessages = dojo.i18n.getLocalization("rmps.ui.common", "CommonMessages");
		
		if(!this.selectionsLabel){
			this.selectionsLabel= jazz.util.html.escape(this.messages.emptyLabel);
		}
	},
	
	postCreate: function(){
		this.inherited(arguments);
		this._loadingWidget= new dijit.layout.ContentPane({content: this._commonMessages.loading});
				
		if(dojo.isFunction(this.argsList)){
			return this.argsList(dojo.hitch(this, function(argsList){
				this.argsList= argsList;
				this.createSelectionsContents();
			}));
		}
		if(!dojo.isArray(this.argsList)|| this.argsList.length<1){
			return rmps.core.error.create(this.declaredClass+ ": Exepect at least one selection defined").dispatch();
		}
		this.createSelectionsContents();
	},
	
	createSelectionsContents: function(){
		for(var index = 0, arg; arg = this.argsList[index]; index++) {
			this._createSelection(this.searchTypeSelect, arg.title, arg.classConstructor, arg.identifier, arg.tooltip);
		}
		
		//Render first selection
		var first= this.argsList[0];
		this._renderSelection(dojo.isFunction(first.classConstructor)?first.identifier:first.classConstructor);
	},
	
	_onSearchTypeChange: function(event){
		if(!event){
			return;
		}
		var element = event.target || event.srcElement;
		var value= element.value;
		
		if(!value){	//Safe check. Do you think that this can ever happen? Nope
			return;
		}
		this._renderSelection(value);
	},
	
	_renderSelection: function(value){
		var widget= this._contentsCache[value];
		
		if(widget){
			if(widget.domNode.parentNode===this.containerNode.DomNode){
				return;
			}
			this._clearSearchTypeContents();
			return this.containerNode.addChild(widget);
		}
		
		//Clear last views
		this._clearSearchTypeContents();
			
		var selectionArgs= this._getSelectionArgs(value);
		var classConstructor= selectionArgs.classConstructor;
		
		if(classConstructor instanceof dijit._WidgetBase){
			this._contentsCache[value]= classConstructor;
			return this.containerNode.addChild(classConstructor);
		}
		
		//Loading inidication
		this.containerNode.addChild(this._loadingWidget);
		
		if(dojo.isFunction(classConstructor)){
			return classConstructor(value, dojo.hitch(this, function(widget){
				//remove loading
				this._clearSearchTypeContents();
				
				this._contentsCache[value]= widget;
				return this.containerNode.addChild(widget);
			}));
		}
			
		jazz.core.loader.batch_load_async([value], dojo.hitch(this, function() {
			//remove loading
			this._clearSearchTypeContents();
				
			var classConstructor= dojo.getObject(value);
			if(!classConstructor){
				//Error handling. Widget is not found
				return rmps.core.error.createError("Failed to load widget:"+ value).dispatch();
			}
			
			var widget= new classConstructor(selectionArgs.args);
			this._contentsCache[value]= widget;
			return this.containerNode.addChild(widget);			
		}));
	},
	
	_getSelectionArgs: function(value){
		for(var index = 0, args; args = this.argsList[index]; index++) {
			//Search by identifier first
			var identifier= args.identifier;
			if(args.identifier=== value){
				return args;
			}
			
			//Search by class constructor next
			var currentClassConstructor= args.classConstructor;
			if(currentClassConstructor instanceof dijit._WidgetBase? currentClassConstructor.id=== value: currentClassConstructor=== value){
				return args;
			}
		}
	},
	
	destroy: function(){
		this.inherited(arguments);
		
		for(var att in this._contentsCache){
			var content= this._contentsCache[att];
			if(content&& !content._beingDestroyed){	//The active view will be being destroyed since it is a child of the current widget
				content.destroyRecursive();
			}
		}
		
		if(this._loadingWidget){
			this._loadingWidget.destroyRecursive();
			delete this._loadingWidget;
		}
		this._contentsCache= {};
	},
	
	_renderSearchTypeContents: function(classConstructor, value){
		var widget= this._contentsCache[value];
		if(widget.domNode.parentNode===this.containerNode.DomNode){
			return;
		}
	},
	
	_clearSearchTypeContents: function(){
		while(this.containerNode.hasChildren()){
			this.containerNode.removeChild(this.containerNode.getChildren()[0]);
		}
	},
	
	_createSelection: function(node, label, value, identifier, tooltip){
		var id= identifier|| (value instanceof dijit._WidgetBase?value.id: value);
		var option = dojo.doc.createElement("option");
		option.appendChild(dojo.doc.createTextNode(label));
		option.value= identifier|| (value instanceof dijit._WidgetBase?value.id: value);
		option.title= tooltip?tooltip: label;
		node.appendChild(option);
	},
});

})();
