/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.render.policies.Anchor");

dojo.require("rmpw.common.policy.Policy");

/**
 * Anchors a shape relatively to another shape.
 * 
 * TODO (MG): merge with GraphFloatingLabel?
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.diagrams.render.policies.Anchor", rmpw.common.policy.Policy, {
	_distance: 4,
	_align: "middle",
	
	setShape: function(shape) {
		this._shape = shape;
	},
	
	setTarget: function(shape) {
		this._target = shape;
	},
	
	setPosition: function(pos) {
		this._position = pos;
	},
	
	setAlign: function(align) {
		this._align = align;
	},
	
	setDistance: function(distance) {
		this._distance = distance;
	},
	
	enforce: function() {
		var shape = rmpw.diagrams.render.util.Workarounds.getBoundingBox(this._shape);
		var target = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(this._target, null, this._target.parent);
		
		var x, y;
		switch(this._position) {
			case "left":
				x = target.x - shape.width - this._distance;
				y = this._getVerticalAlign(shape, target);
				break;
			case "right":
				x = target.x + target.width + this._distance;
				y = this._getVerticalAlign(shape, target);
				break;
			case "top":
				x = this._getHorizontalAlign(shape, target);
				y = target.y - shape.height - this._distance;
				break;
			case "bottom":
				x = this._getHorizontalAlign(shape, target);
				y = target.y + target.height + this._distance;
				break;
			case "center":
				x = this._getHorizontalAlign(shape, target);
				y = target.y + (target.height / 2) - (shape.height / 2);
				break;
		}
		this._shape.setTransform({dx: x, dy: y});
	},
	
	_getVerticalAlign: function(shape, target) {
		switch(this._align) {
			case "start": 
				return target.y;
			case "middle":
				return target.y + (target.height / 2) - (shape.height / 2);
			case "end":
				return target.y + target.height - shape.height;
		}
	},
	
	_getHorizontalAlign: function(shape, target) {
		switch(this._align) {
			case "start": 
				return target.x;
			case "middle":
				return target.x + (target.width / 2) - (shape.width / 2);
			case "end":
				return target.x + target.width - shape.width;
		}
	}
});