/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.render.policies.ChartSeries");

dojo.require("rmpw.common.policy.Policy");
dojo.require("rmpw.diagrams.render.util.Workarounds");

/**
 * Modifies a path to connect shapes.
 * - If given a container, uses all its children (in order) as waypoints.
 * - TODO (MG): position and rotate head and tail shapes to match the line's beginning and end points
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.diagrams.render.policies.ChartSeries", rmpw.common.policy.Policy, {
	_dampening: 0.2,
	
	setBody: function(shape) {
		this._body = shape;
	},
	
//	setHead: function(shape) {
//		this._head = shape;
//	},
//	
//	setTail: function(shape) {
//		this._tail = shape;
//	},
	
	setContainer: function(group) {
		this._container = group;
	},
	
	setDampening: function(amount) {
		this._dampening = amount;
	},
	
	enforce: function() {
		// boil down shapes to a series of points
		var waypoints = dojo.map(this._container.children, function(child) {
			var bounds = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(child, null, child.parent); 
			return {
				x: bounds.x + bounds.width / 2,
				y: bounds.y + bounds.height / 2
			};
		});
		
		// build smoothed path connecting those points
		var path = this._body;
		var waypointCount = waypoints.length;
		path.setShape({ path: "" });
		if(waypointCount > 0) {
			path.moveTo(waypoints[0].x, waypoints[0].y);
			for(var i = 1; i < waypointCount; i++) {
				var prev = waypoints[i - 1];
				var curr = waypoints[i];
				var next = waypoints[i + 1];
				if(!next) {
					next = curr;
				}
				
				// create vector from next to previous to approximate the tangent
				var ctrl = {};
				ctrl.x = prev.x - next.x;
				ctrl.y = prev.y - next.y;
				
				// reduce its magnitude
				ctrl.x *= this._dampening;
				ctrl.y *= this._dampening;
	
				// make it absolute
				ctrl.x += curr.x;
				ctrl.y += curr.y;
				path.smoothCurveTo(ctrl.x, ctrl.y, curr.x, curr.y);
				
				// show control point markers, for debugging
//				var line = path.parent.createLine({x1: ctrl.x, y1: ctrl.y, x2: curr.x, y2: curr.y});
//				line.setStroke('red');
			}
		}
	}
});