/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.render.policies.GraphAnchorable");

dojo.require("rmpw.diagrams.render.policies.GraphElement");
dojo.require("rmpw.diagrams.render.util.Insets");
dojo.require("rmpw.common.policy.Policy");
dojo.require("rmpw.diagrams.render.util.PointListUtilities");
dojo.require("rmps.core.error");
dojo.require("rmpw.diagrams.render.util.Rectangle");
dojo.require("rmpw.diagrams.render.util.Image");

/**
 * Mappings that install this policy are anchorable by edges
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.render.policies.GraphAnchorable", rmpw.common.policy.Policy, {
	singleton: ["anchorable", "graphAnchorable"],
	
	constructor: function() {
		this.ownedConnections = [];  // collection of source connections used for layout and navigation purposes
		this.targetConnections = []; // collection of target connections used for navigation purposes
	},
	
	setAnchorShape: function() {
		this._anchorShapes = arguments;
		
		// pre-initialize image anchors
		this._getAnchorShapes();
	},
	
	/*
	 * Gets a polyline array of the anchor points
	 */
	getAnchorPoints: function() {
		var anchorShapes = this.getAnchorShapes();
		var polyShapes = [];
		
		if (anchorShapes && anchorShapes[0]) {
			var graphElement = this.host.policies.graphElement;
			var offset = graphElement.getDojoShapeUpdateOffset();
			var i, j, anchorShape, point, lg, pt;
			var wholeAnchorBox = new rmpw.diagrams.render.util.Rectangle(0, 0, 0, 0);
			
			for (i = 0, anchorShape; anchorShape = anchorShapes[i]; i++) {
				
				var anchorBox = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(
						anchorShape, rmpw.diagrams.render.util.Workarounds.IGNORE_DECORATIONS);
				
				if (wholeAnchorBox.contains_4(anchorBox)) {
					continue; // don't process this one since it's completely contained in another anchor shape
				}
				wholeAnchorBox = wholeAnchorBox.union_0(anchorBox);
				
				anchorBox = new rmpw.diagrams.render.util.Rectangle(anchorBox.x, anchorBox.y, anchorBox.width, anchorBox.height);
				graphElement.translateToRelative(anchorBox);
				anchorBox.translate(offset.dx, offset.dy);
				
				var polyShape = new rmpw.diagrams.render.util.PointList();
				if (anchorShape instanceof dojox.gfx.Rect &&  anchorShape.shape.r !== 0) {
					var dim = new rmpw.diagrams.render.util.Dimension(anchorShape.shape.r, 0);
					graphElement.translateToRelative(dim);
					var arcSize = dim.width;
					
					// Working our way counter-clockwise.
					var keyPoint1 = new rmpw.diagrams.render.util.Point(anchorBox.getTopLeft().x + arcSize / 2, anchorBox.getTopLeft().y);
					var keyPoint2 = new rmpw.diagrams.render.util.Point(anchorBox.getTopLeft().x, anchorBox.getTopLeft().y + arcSize / 2);
					var keyPoint3 = new rmpw.diagrams.render.util.Point(anchorBox.getBottomLeft().x, anchorBox.getBottomLeft().y - arcSize / 2);
					var keyPoint4 = new rmpw.diagrams.render.util.Point(anchorBox.getBottomLeft().x + arcSize / 2, anchorBox.getBottomLeft().y);
					var keyPoint5 = new rmpw.diagrams.render.util.Point(anchorBox.getBottomRight().x - arcSize / 2, anchorBox.getBottomRight().y);
					var keyPoint6 = new rmpw.diagrams.render.util.Point(anchorBox.getBottomRight().x, anchorBox.getBottomRight().y- arcSize / 2);
					var keyPoint7 = new rmpw.diagrams.render.util.Point(anchorBox.getTopRight().x, anchorBox.getTopRight().y + arcSize / 2);
					var keyPoint8 = new rmpw.diagrams.render.util.Point(anchorBox.getTopRight().x - arcSize / 2, anchorBox.getTopRight().y);
	
					// create point lists for the rounded corners
					var upperLeftPointList = new rmpw.diagrams.render.util.PointList();
					upperLeftPointList.addPoint_1(keyPoint1);
					upperLeftPointList.addPoint_0(anchorBox.getTopLeft().x + arcSize / 4, anchorBox.getTopLeft().y + arcSize / 12);
					upperLeftPointList.addPoint_0(anchorBox.getTopLeft().x + arcSize / 12, anchorBox.getTopLeft().y + arcSize / 4);
					upperLeftPointList.addPoint_1(keyPoint2);
	
					var lowerLeftPointList = new rmpw.diagrams.render.util.PointList();
					lowerLeftPointList.addPoint_1(keyPoint3);
					lowerLeftPointList.addPoint_0(anchorBox.getBottomLeft().x + arcSize / 12, anchorBox.getBottomLeft().y - arcSize / 4);
					lowerLeftPointList.addPoint_0(anchorBox.getBottomLeft().x + arcSize / 4, anchorBox.getBottomLeft().y - arcSize / 12);
					lowerLeftPointList.addPoint_1(keyPoint4);
	
					var lowerRightPointList = new rmpw.diagrams.render.util.PointList();
					lowerRightPointList.addPoint_1(keyPoint5);
					lowerRightPointList.addPoint_0(anchorBox.getBottomRight().x - arcSize / 4, anchorBox.getBottomRight().y - arcSize / 12);
					lowerRightPointList.addPoint_0(anchorBox.getBottomRight().x - arcSize / 12, anchorBox.getBottomRight().y - arcSize / 4);
					lowerRightPointList.addPoint_1(keyPoint6);
	
					var upperRightPointList = new rmpw.diagrams.render.util.PointList();
					upperRightPointList.addPoint_1(keyPoint7);
					upperRightPointList.addPoint_0(anchorBox.getTopRight().x - arcSize / 12, anchorBox.getTopRight().y + arcSize / 4);
					upperRightPointList.addPoint_0(anchorBox.getTopRight().x - arcSize / 4, anchorBox.getTopRight().y + arcSize / 12);
					upperRightPointList.addPoint_1(keyPoint8);
	
					// combine all the points
					var plutil = new rmpw.diagrams.render.util.PointListUtilities();
					polyShape.addPoint_1(keyPoint1);
					polyShape.addAll(plutil.calcSmoothPolyline(
							upperLeftPointList, plutil.SMOOTH_FACTOR_NORMAL,
							plutil.DEFAULT_BEZIERLINES));
					polyShape.addPoint_1(keyPoint2);
					polyShape.addPoint_1(keyPoint3);
					polyShape.addAll(
							plutil.calcSmoothPolyline(lowerLeftPointList, plutil.SMOOTH_FACTOR_NORMAL,
									plutil.DEFAULT_BEZIERLINES));
					polyShape.addPoint_1(keyPoint4);
					polyShape.addPoint_1(keyPoint5);
					polyShape.addAll(plutil.calcSmoothPolyline(
							lowerRightPointList, plutil.SMOOTH_FACTOR_NORMAL,
							plutil.DEFAULT_BEZIERLINES));
					polyShape.addPoint_1(keyPoint6);
					polyShape.addPoint_1(keyPoint7);
					polyShape.addAll(plutil.calcSmoothPolyline(
							upperRightPointList, plutil.SMOOTH_FACTOR_NORMAL,
							plutil.DEFAULT_BEZIERLINES));
					polyShape.addPoint_1(keyPoint8);
					polyShape.addPoint_1(keyPoint1);
	
					polyShape.normalize(0);
				}
				else if (anchorShape instanceof dojox.gfx.Polyline &&  anchorShape.shape.points) {
					for (j=0, lg=anchorShape.shape.points.length; j<lg; j++) {
						pt = new rmpw.diagrams.render.util.Point(anchorShape.shape.points[j].x, 
								anchorShape.shape.points[j].y);
						polyShape.addPoint_1(pt);
					}
					
					if (anchorShape.matrix) {
						polyShape.transform(anchorShape.matrix);
					}
					graphElement.translateToAbsolute(polyShape, anchorShape);
					graphElement.translateToRelative(polyShape);
					polyShape.translate(offset.dx, offset.dy);
				}
				else if (anchorShape instanceof dojox.gfx.Line) {
					polyShape.addPoint_0(anchorShape.shape.x1, anchorShape.shape.y1);
					polyShape.addPoint_0(anchorShape.shape.x2, anchorShape.shape.y2);
					
					if (anchorShape.matrix) {
						polyShape.transform(anchorShape.matrix);
					}
					graphElement.translateToAbsolute(polyShape, anchorShape);
					graphElement.translateToRelative(polyShape);
					polyShape.translate(offset.dx, offset.dy);
				}
				else if (anchorShape instanceof dojox.gfx.Image) {
					var shapeSource = anchorShape.shape.src;
					var imagePoints = rmpw.diagrams.render.util.Image.getImageInfo(shapeSource);
					
					if (imagePoints) {
						var wRatio = anchorShape.shape.width / imagePoints.width;
						var hRatio = anchorShape.shape.height / imagePoints.height;
						
						for (j=0, pt; pt=imagePoints.points[j]; j++) {
							polyShape.addPoint_0(pt.x, pt.y);
						}
						
						polyShape.scale(wRatio, hRatio);
						
						if (anchorShape.matrix) {
							polyShape.transform(anchorShape.matrix);
						}
						graphElement.translateToAbsolute(polyShape, anchorShape);
						graphElement.translateToRelative(polyShape);
						polyShape.translate(offset.dx + anchorShape.shape.x, offset.dy + anchorShape.shape.y);
					}
				}
				else if (anchorShape instanceof dojox.gfx.Text) {
					if (anchorShape.parent.shape.type == "wrappingText") {
						anchorBox.y -= rmpw.diagrams.render.util.Workarounds.getTextBounds(anchorShape).height;
					}
					polyShape = rmpw.diagrams.render.util.PointListUtilities.prototype.getRectanglePolyline(anchorBox);
				}
				else {
					polyShape = rmpw.diagrams.render.util.PointListUtilities.prototype.getRectanglePolyline(anchorBox);
				}
				polyShapes.push(polyShape);
			}
		}
		
		return polyShapes;
	},
	
	_getAnchorShapes: function(anchorShapes) {
		var shapes = [];
		
		if (!anchorShapes) {
			return [];
		}
		
		var numShapes = 0;
		for (var i=0, lg=anchorShapes.length; i<lg; i++) {
			if (anchorShapes[i] instanceof dojox.gfx.Group) {
				for (var j=0, lg2=anchorShapes[i].children.length; j<lg2; j++) {
					var subshapes = this._getAnchorShapes([anchorShapes[i].children[j]]);
					shapes = shapes.concat(subshapes);
				}
			}
			else {
				if (anchorShapes[i] instanceof dojox.gfx.Image) {
					var imageShape = anchorShapes[i];
					if (imageShape) {
						var shapeSource = imageShape.shape.src;
						
						var imagePoints = rmpw.diagrams.render.util.Image.getImageInfo(shapeSource);
						if (!imagePoints) {
							var callback = dojo.hitch(this, function(imageInfo) {
								this.host.policies.graphElement.resetPolygonShapes();
								this.host.invalidate();
							});
							rmpw.diagrams.render.util.Image.getImageInfo(shapeSource, callback);
						}
					}
				}
				shapes[numShapes++] = anchorShapes[i];
			}
		}
		
		return shapes;
	},
	
	/*
	 * Gets the set of anchor shapes.  If one of the anchor shapes is a group,
	 * it will recursively visit it's contents to obtain the contained items that
	 * are anchorable.
	 */
	getAnchorShapes: function() {
		return this._getAnchorShapes(this._anchorShapes);
	},
	
	/*
	 * Return the bounds of the anchor shapes
	 */
	getBounds: function() {
		if (this._anchorShapes && this.host) {
			var anchorBox = null;
			var offset = this.host.policies.graphElement.getDojoShapeUpdateOffset();
			
			for (var i=0, anchorShape; anchorShape = this._anchorShapes[i]; i++) {
				
				var shapeBox = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(anchorShape);
				shapeBox = new rmpw.diagrams.render.util.Rectangle(shapeBox.x, shapeBox.y, shapeBox.width, shapeBox.height);
				this.host.policies.graphElement.translateToRelative(shapeBox);
				shapeBox.translate(offset.dx, offset.dy);
				
				if (!anchorBox) {
					anchorBox = shapeBox;
				}
				else {
					anchorBox.union_0(shapeBox);
				}
			}
			
			return anchorBox;
		}
		
		return null;
	},
	
	/*
	 * Return the insets of the anchor shape relative to the top level shape
	 */
	getAnchorInsets: function() {
		var anchorBox = this.getBounds();
		if (anchorBox) {
			var box = this.host.policies.graphElement.getBounds();
			
			var top = anchorBox.y - box.y;
			var left = anchorBox.x - box.x;
			var bottom = box.height - anchorBox.height - top;
			var right = box.width - anchorBox.width - left;
			return new rmpw.diagrams.render.util.Insets(top, left, bottom, right);
		}
		
		return new rmpw.diagrams.render.util.Insets(0, 0, 0, 0);
	},

	/*
	 * Adds a new fixed location to this anchor.  
	 * @param side one of NORTH, SOUTH, WEST, EAST (see definitions in Point.js
	 * @param percentage integer value from 0 - 100 indicating how far along the side the fixed point is
	 * @param inset integer value how for inset from the side the point is
	 */
	addFixedAnchorPosition: function(side, percentage, inset) {
		if (!this._fixedAnchor) {
			this._fixedAnchor = this.host.viewer.graphConfigurator.createAnchor(
					rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.FIXED_ANCHOR,
					this.host.policies.graphElement.getInternalGraphElement(), {side: side, percentage: percentage, inset: inset});
		}
	},
	
	destroy: function() {
		var conns = [];
		if(this.targetConnections ){
			dojo.forEach(this.targetConnections, function(edgeMap){
				conns.push(edgeMap);
			});
		}
		if(this.ownedConnections ){
			dojo.forEach(this.ownedConnections, function(edgeMap){
				conns.push(edgeMap);
			});
		}
		
		var self = this;
		dojo.forEach(conns, function(edgeMap){
			// track the edge destroys so that they can be restored by undo
			if (!dojo.isOffscreen && !self.host.root.getCommandController().isUndoRedoExecuting()) {
				var changeSet = self.host.root.repository.getLastChangeSet();
				var edgeContainerElement = edgeMap.parent.getDefaultSource() ? edgeMap.parent.getDefaultSource() : edgeMap.parent.__deletedSource;
				
				// check to see if element has been deleted already - if so, no need to add refresh
				// since it will refresh itself on undo
				if (edgeContainerElement) { 
					var semanticChange = new rmpw.common.vcs.Change(edgeContainerElement);
					semanticChange.pushRefresh(edgeMap.getReferredProperty(), edgeMap.getDefaultSource());
					changeSet.add(semanticChange);
					semanticChange.commitToTarget(edgeContainerElement); // no events
				}
			}
			
			edgeMap.destroy();
		});
		
		this.inherited(arguments);
	},
	
	/*
	 * Returns the graph element policy for a given mapping
	 */
	_getGraphElement: function(mapping) {
		if (mapping.policies) {
			return mapping.policies.graphElement;
		}
		
		return null;
	},
	
	/*
	 * Returns the graph element for this element.  This is necessary
	 * since some items may be anchorable that aren't graph elements.  In order
	 * to do layout, the edges must link to a graph element.
	 */
	_getAnchorableGraphElement: function() {
		if(!this._anchorableElement) {
			
			// check if the associated mapping is also a graph element
			if (this._getGraphElement(this.host)) {
				this._anchorableElement = this;
			}
			else {
				// walk up the parent chain for the first graph container
				var parent = this.host.parent;
				while(!this._graphElement && parent) {
					var graphElement = this._getGraphElement(parent);
					if (graphElement) {
						var anchorableElement = graphElement.getAnchorable(graphElement.mapping);
						if (anchorableElement) {
							this._anchorableElement = anchorableElement;
						}
					}
					
					parent = parent.parent;
				}
			}
		}
		
		return this._anchorableElement;
	},
	
	addSourceConnection: function(sourceConnection) {
		var anchorable = this._getAnchorableGraphElement();
		if (anchorable) {
			anchorable.ownedConnections.push(sourceConnection);
		}
		else {
			console.debug("Edge is trying to attach to anchorable that isn't in a graph containment hierarchy");
		}
	},
	
	addTargetConnection: function(sourceConnection) {
		var anchorable = this._getAnchorableGraphElement();
		if (anchorable) {
			anchorable.targetConnections.push(sourceConnection);
		}
		else {
			console.debug("Edge is trying to attach to anchorable that isn't in a graph containment hierarchy");
		}
	},
	
	removeFromConnectionCollection: function(connection, connectionCollection) {
		var newConnections = [];
		for (var i=0, lg=connectionCollection.length; i<lg; i++) {
			if (connectionCollection[i] !== connection) {
				newConnections[newConnections.length] = connectionCollection[i];
			}
		}
		
		return newConnections;
	},
	
	removeSourceConnection: function(sourceConnection) {
		var anchorable = this._getAnchorableGraphElement();
		if (anchorable) {
			anchorable.ownedConnections = anchorable.removeFromConnectionCollection(sourceConnection, anchorable.ownedConnections);
		}
	},
	
	removeTargetConnection: function(targetConnection) {
		var anchorable = this._getAnchorableGraphElement();
		if (anchorable) {
			anchorable.targetConnections = anchorable.removeFromConnectionCollection(targetConnection, anchorable.targetConnections);
		}
	},
	
	/**
	 * Returns a connection anchor appropriate for the source of this connection on this node
	 */
	getSourceConnectionAnchor : function(connEP) {
		if (this._fixedAnchor) {
			return this._fixedAnchor;
		}
		
		return this.host.viewer.graphConfigurator.createAnchor(
				rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.MOVEABLE_ANCHOR,
				this.host.policies.graphElement.getInternalGraphElement());
	},

	/**
	 * Returns a connection anchor appropriate for the target of this connection on this node
	 */
	getTargetConnectionAnchor : function(connEP) {
		return this.getSourceConnectionAnchor(connEP); // by default no different between source and target anchors
	}
});