/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.render.policies.GraphElement");

dojo.require("rmpw.common.policy.Policy");
dojo.require("rmpw.diagrams.render.util.Workarounds");
dojo.require("rmpw.diagrams.render.util.Rectangle");
dojo.require("rmpw.diagrams.render.util.PointList");
dojo.require("rmpw.diagrams.render.util.PointListUtilities");
dojo.require("rmpw.diagrams.render.util.Point");
/**
 * Data needed to layout the graph node
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.render.policies.GraphElement", rmpw.common.policy.Policy, {
	constructor: function() {
		this.listeners = null;
		this.constructing = dojo.connect(this, "refresh", this, "postConstruction");
		
		// TODO (MG): avoid creating methods and properties on host
		this.host.getGraphContainerPolicy = dojo.hitch(this, function() {
			return this.getGraphContainer();
		});
		
		this._oldBounds = new rmpw.diagrams.render.util.Rectangle(0, 0, 0, 0);
	},
	
	postConstruction: function() {
		if (!this.isInitialized()) {
			dojo.disconnect(this.constructing);
			this.setInitialized();
		}
		
		this.attachListeners();
	},
	
	destroy: function() {
		this.detachListeners();
		
		var graphManager = this.getGraphManager();
		if (graphManager) {
			graphManager.remove(this.host);
		}
		
		if (this.outlineShape) {
			this.outlineShape.removeShape();
		}
		
		this.getInternalGraphElement().destroy();
	},
	
	refresh: function() {
	},

	/*
	 * called during refresh only if bounds changed
	 */
	onBoundsChanged: function() {
	},
	
	/*
	 * Update the outline shape with the polyline points
	 */
	updateOutlineShape: function(parentShape) {
		if (this.getPolygonShapes) {
			
			if (this.outlineShape) {
				this.outlineShape.removeShape();
			}
			this.outlineShape = parentShape.createGroup();
			
			var polyShapes = this.getPolygonShapes();
			for (var i = 0, polyShape; polyShape = polyShapes[i]; i++) {
				
				var poly = this.outlineShape.createPolyline();

				poly.setShape(polyShape);
				
				/* IE renders very thick, very black lines at small scale. Compensating by
				 * changing the line width and color for IE. WI 18290
				 */
				
				var width = ( dojo.isIE ) ? 1 : 2;	
				
				var color = ( dojo.isIE ) ? [117, 117, 117, 1]: [0, 0, 0, 1];

				poly.setStroke({color: color, width: width });
			}
		}
	},
	
	setShape: function(shape) {
		this._shape = shape;
		
		// TODO (MG): should avoid injecting functions into host
		this.host.getShape = dojo.hitch(this.host, this._getShape);
		
		if (this._updateDojo === undefined) {
			this._updateDojo = true;
		}
	},
	
	/*
	 * @deprecated - use setShape instead
	 */
	setFigure: function(figure) {
		this.setShape(figure);
	},
	
	/*
	 * This method will prevent graphNode from updating the dojo shape based on the 
	 * initialization of the graph node position (setX, setY).  This is useful if you
	 * want to trace directly to the dojo shape in your mapping as opposed to delegating
	 * this functionality to the graphNode.
	 */
	useDojoPosition: function(use) {
		this._updateDojo = !use;
	},
	
	/*
	 * Getter which returns a boolean indicating where this graph node will manage and 
	 * update the position of the dojo shape or not.  If true then then the graph node
	 * will always get it's position from the shape (shape is master).  Otherwise, the
	 * graph Node is master for the position coordinate.
	 */
	isUsingDojoPosition: function() {
		return !this._updateDojo;
	},
	
	/*
	 * Gets the internal graph element that contributes to the graph
	 */
	getInternalGraphElement: function() {
		if (!this._figure) {
			this._figure = this._createInternalGraphElement();
		}
		
		return this._figure;
	},
	
	/*
	 * DEPRECATED - use Anchorable::setAnchorShape instead
	 * Sets the figure that will represent the anchorable outline of the shape.
	 * This is also used for hover feedback and will be the default shape used for
	 * the outline visualization.
	 */
	setAnchorFigure: function() {
		if (this.host.policies && this.host.policies.anchorable) {
			this.host.policies.anchorable.setAnchorShape(arguments[0]);
		}
	},
	
	/*
	 * DEPRECATED - use Anchorable::getAnchorShape instead
	 * return an array of shapes.  To do multiple shapes - user should
	 * define a path.
	 */
	_getAnchorShapes: function() {
		if (this.host.policies && this.host.policies.anchorable) {
			return this.host.policies.anchorable.getAnchorShapes();
		}
		
		return null;
	},
	
	/*
	 * resets the polygon shape cache
	 */
	resetPolygonShapes: function() {
		delete this._polyShapes;
	},
	
	/*
	* Gets the offset from the cached bounds in the graph object to the actual
	* position of the dojo shape.
	*/
	getDojoShapeUpdateOffset: function() {
		if (this.isUsingDojoPosition()) {
			return {dx: 0, dy: 0};
		}
	
		var figBox = this.getBounds();
		var shapeBox = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(
		this.getDojoShape(), rmpw.diagrams.render.util.Workarounds.IGNORE_DECORATIONS);
		shapeBox = new rmpw.diagrams.render.util.Rectangle(shapeBox.x, shapeBox.y, shapeBox.width, shapeBox.height);
		this.translateToRelative(shapeBox);
	
		return {dx: figBox.x - shapeBox.x, dy: figBox.y - shapeBox.y};
	},
	
	/*
	 * Return an array of polygon shapes that represent this figure
	 */
	getPolygonShapes: function() {	
		if ((!this._polyShapes && this.isInitialized()) || this.isUsingDojoPosition()) {
			var polyShapes = this.host.policies.anchorable ? this.host.policies.anchorable.getAnchorPoints() : [];
			if (polyShapes.length > 0) {
				this._polyShapes = polyShapes;
			}
			else { 
				this._polyShapes = [rmpw.diagrams.render.util.PointListUtilities.prototype.getRectanglePolyline(this.getBounds())];
			}
		}
		
		if (this._polyShapes) {
			return this._polyShapes;
		}
		
		return [rmpw.diagrams.render.util.PointListUtilities.prototype.getRectanglePolyline(this.getBounds())];
	},
	
	/* 
	 * Returns the dojo shape associated with this graph policy element
	 */
	getDojoShape: function() {
		return this._shape;
	},
	
	/* 
	 * Hitched to the mapping scope
	 */
	_getShape: function() {
		var shape = null;
		var graphElement = this.policies.graphElement;
		if(graphElement) { 
			shape = graphElement.getDojoShape();
		}
		return shape;
	},
	
	isInitialized: function() {
		return (!this.constructing && this.constructing !== null);
	},
	
	setInitialized: function() {
		delete this.constructing;
	},
	
	/*
	 * Connect to any relevant method calls to listen to incoming events.
	 */
	attachListeners: function() {
		if(this.listeners && this.listeners.length !== 0) {
			console.debug("Exception: listeners already connected");
			return;
		}
		
		this.listeners = [];
		this.listeners.push(dojo.connect(this.getInternalGraphElement(), "revalidate", this.host, "invalidate"));
		this.listeners.push(dojo.connect(this.host, "invalidate", this, "_invalidate"));
	},
	
	detachListeners: function() {
		if(!this.listeners) {
			console.debug("Listeners already disconnected");
			return;
		}
	
		dojo.forEach(this.listeners, dojo.disconnect);
		delete this.listeners;
	},
	
	_invalidate: function() {
		this.getGraphManager().remove(this.host);
		this.getGraphManager()._addToInvalidMappings(this.host);
	},
	
	enforce: function() {
		if (!this.isInitialized()) {
			this._completeInternalTraces();
			this._invalidate();
		}
	},
	
	_createInternalGraphElement: function() {
		// subclasses should override
	},
	
	/*
	 * Returns the graph container for this element
	 */
	getGraphContainer: function() {
		if(!this._graphContainer) {
			this._graphContainer = this.findNearestParentByKey("graphContainer");
		}
		return this._graphContainer;
	},
	
	/*
	 * Returns the graph manager of the closest container
	 */
	getGraphManager: function() {
		var graphContainer = this.getGraphContainer();
		if (graphContainer) {
			return graphContainer.graphManager;
		}
		
		return null;
	},
	
	/*
	 * Gets the associated anchorable policy if it is installed or null otherwise...
	 */
	getAnchorable: function(mapping) {
		if (mapping.policies.anchorable) {
			return mapping.policies.anchorable;
		}
		
		return null;
	},
	
	/*
	 * Translates a geometric data structure coordinates to absolute screen coodinates
	 */
	translateToAbsolute: function (t, dojoShape){
		var shape = dojoShape ? dojoShape : this.getDojoShape();
		return rmpw.diagrams.render.util.Workarounds.translateToAbsolute(t, shape);
	},

	/*
	 * Translates a geometric data structure coordinates to relative model coordinates
	 */
	translateToRelative: function (t, dojoShape){
		var shape = dojoShape ? dojoShape : this.getDojoShape();
		return rmpw.diagrams.render.util.Workarounds.translateToRelative(t, shape);
	},
	
	getBounds: function() {
		var bounds = this.getInternalGraphElement().getBounds();
		if (bounds.isEmpty()) {
			var box = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(this._shape);
			var relBox = new rmpw.diagrams.render.util.Rectangle(box.x, box.y, box.width, box.height);
			this.translateToRelative(relBox);
			return relBox;
		}
		
		return bounds;
	},
	
	/*
	 * Returns the bounds that are flush with the viewable extent of the shape (anchorable area)
	 */
	getTightBounds: function() {
		var bounds = null;
		if (this.host.policies.anchorable) {
			bounds = this.host.policies.anchorable.getBounds();
		}
		
		if (!bounds) {
			bounds = this.getBounds();
		}
		
		return bounds;
	},
	
	/*
	 * Returns the combined set of points from all shapes.
	 */
	getPolygonPoints: function() {
		var polyShapes = this.getPolygonShapes();
		var points = new rmpw.diagrams.render.util.PointList();
		
		for (var i = 0, polyShape; polyShape = polyShapes[i]; i++) {
			points.addAll(polyShape);
		}
		
		return points;
	},
	
	/*
	 * Finds the closest intersection to this figure given a line defined
	 * by start and end.
	 */
	getClosestIntersectionPointToLine: function(start, end) {
		var polyShapes = this.getPolygonShapes();
		var intersections = new rmpw.diagrams.render.util.PointList();
		for (var i = 0, polyShape; polyShape = polyShapes[i]; i++) {
			var shapeintersects = new rmpw.diagrams.render.util.LineSeg(start, end).getLineIntersectionsWithLineSegs(polyShape); 
			if (shapeintersects && shapeintersects.size!==0) {
				intersections.addAll(shapeintersects);
			}
		}
		
		var closest = null;
		if (intersections.size !== 0) {
			closest = intersections.getFirstPoint();
			for (i=1; i<intersections.size; i++) {
				var temp = intersections.getPoint(i);
				if (Math.abs(temp.x - end.x) < Math.abs(closest.x - end.x)){
					closest = temp;
				}else if (Math.abs(temp.y - end.y) < Math.abs(closest.y - end.y)){
					closest = temp;
			    }
			}
		}
		return closest;
	},
	
	/*
	 * Gets the stroke width of the outline shape
	 */
	getStrokeWidth: function() {
		var anchorShapes = this._getAnchorShapes();
		if (this.anchorShapes && this.anchorShapes[0] && this.anchorShapes[0].strokeStyle) {
			return this.anchorShapes[0].strokeStyle.width;
		}
		
		return 1;
	},
	
	/*
	 * Finalizes any updates from the graph manager and ensures the dojo 
	 * shape has been updated accordingly
	 */
	updateShape: function () {
		// subclasses should implement
	}

});