/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.render.policies.GraphFloatingLabel");

dojo.require("rmpw.diagrams.render.policies.GraphNode");
dojo.require("rmpw.diagrams.render.decorations.LabelLocator");

/**
 * Data needed to layout the graph node
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.render.policies.GraphFloatingLabel", rmpw.diagrams.render.policies.GraphNode, {
	singleton: ["graphElement", "graphFloatingLabel"],
	
	locator: null,
	
	constructor: function() {
		this.locator = new rmpw.diagrams.render.decorations.LabelLocator(this.host);	
		this._initialOffset = 'Center';
	},
	
	_getInitialPosition: function() {
		return  { x: 0, y: 0 };  // override from graph node since floating labels store a relative position
	},
	
	attachListeners: function() {
		this.inherited(arguments);
		
		this.listeners.push(dojo.connect(this.host.parent, "invalidate", this.host, "invalidate"));
	},

	_createInternalGraphElement: function() {
		var internalFigure = this.inherited(arguments);
		
		return internalFigure;
	},
	
	setInitialOffset: function( offset ) {
		if (offset === 'Center' || offset === 'West' || offset === 'East') {
			this._initialOffset = offset;
		}
	},
	
	setTextShape: function(textShape) {
		this.textShape = textShape;
	},
	
	setLabelText: function(label) {
		this.textShape.setShape({text: label});
	},
	
	setAlignment: function(alignment) {
		this.locator.setAlignment(alignment);
	},
	
	invalidateOverlappingEdges: function() {
		// do nothing - don't care about labels overlapping edges
	},
	
	getDefaultOffsetOrientation: function() {
		return  this._initialOffset;  
	},
	
	/*
	 * Returns the bounds of the dojo shape
	 */
	getDojoBounds: function(cache) {
	    var bounds;
	    
		if (cache && this._oldBounds) {
			return this._oldBounds;
		}
		
		// else calculate from diagram
		bounds = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(
				this.getDojoShape(), rmpw.diagrams.render.util.Workarounds.IGNORE_DECORATIONS);
		var relBounds = new rmpw.diagrams.render.util.Rectangle(bounds.x, bounds.y, bounds.width, bounds.height);
		this.translateToRelative(relBounds);
		return relBounds;
	},
	
	refresh: function() {
		var rect = this.getConstraintBounds();
		this.getInternalGraphElement().setSize({width: rect.width, height:rect.height});
		
		this.locator.relocate();
		this.getGraphManager().updateLocation(this.host);
	}
});