/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.render.policies.MatchSize");

dojo.require("rmpw.diagrams.render.util.Workarounds");
dojo.require("rmpw.diagrams.render.util.PointList");
dojo.require("rmpw.diagrams.render.util.ResizeObject");
dojo.require("rmpw.common.policy.Policy");

/**
 * Policy that matches the size of one or more slave shapes to a master shape,
 * or union of several master shapes.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.diagrams.render.policies.MatchSize", rmpw.common.policy.Policy, {
	/**
	 * @varargs list of master shapes
	 */
	setMaster: function() {
		this._master = arguments;
	},

	/**
	 * @varargs list of slave shapes
	 */
	setSlave: function() {
		this._slave = arguments;
	},

	/**
	 * true if matching slave's width
	 */
	setMatchWidth: function(match) {
		this._matchWidth = match;
	},

	/**
	 * true if matching slave's height
	 */
	setMatchHeight: function(match) {
		this._matchHeight = match;
	},
	
	setMinWidth: function(minWidth) {
		this._minWidth = minWidth;
	},
	
	setMinHeight: function(minHeight) {
		this._minHeight = minHeight;
	},	
	
	deactivate: function() {
		this._deactivated = true;
	},	
	
	enforce: function() {
		if (this._deactivated){ return;}
		var matchWidth, matchHeight;
		if(!this._matchWidth && !this._matchHeight) {
			// default to matching width and height if neither specified
			matchWidth = true;
			matchHeight = true;
		} else {
			matchWidth = this._matchWidth;
			matchHeight = this._matchHeight;
		}

		// make all slaves that are contained within the master very small so they don't count towards measuring the master's size
		var master = this._master[0];
		for(var i = 0, slave; slave = this._slave[i]; i++) {
			// Note: figure.getParent() function is not present on dojx.gfx.Surface that's at the root of all parent chains
			var figure = slave;
			while(figure.getParent) {
				figure = figure.getParent();
				if(figure === master) {
					this._applySize(slave, { width: 1, height: 1 }, matchWidth, matchHeight);
					break;
				}
			}
		}
		
		// measure master's size
		var bounds = null;
		if(this._master.length > 1) {
			console.warn("multiple master shapes not implemented yet");
		} else {
			bounds = rmpw.diagrams.render.util.Workarounds.getBoundingBox(this._master[0]);
		}
		
		// apply min height/width
		bounds.width = Math.max(this._minWidth || 0, bounds.width);
		bounds.height = Math.max(this._minHeight || 0, bounds.height);
		
		// apply to slaves
		for(i = 0, slave; slave = this._slave[i]; i++) {
			this._applySize(slave, bounds, matchWidth, matchHeight);
		}
	},
	
	_applySize: function(slave, bounds, matchWidth, matchHeight) {
		if (!this._resizeUtil) {
			this._resizeUtil = new rmpw.diagrams.render.util.ResizeObject();
		}
		
		this._resizeUtil.applySize(slave, bounds, matchWidth, matchHeight);
	}
	
});