/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.render.policies.NumericScale");

dojo.require("rmpw.diagrams.render.policies.AbstractScale");

/**
 * Sets up a numerical scale of values, to which shapes can be aligned.  Primarily designed to set up x
 * and y scales for charting.
 * 
 * TODO (MG): Assumes 1:1 scale between some group on this policy's mapping, and the group that owns a binding.
 * The scale should be relative to a specified graphics Group, and when children are positioned, they
 * should be set to coordinates relative to their immediate parent Group, transformed from coordinates
 * "absolute" to the group this scale is attached to.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.diagrams.render.policies.NumericScale", rmpw.diagrams.render.policies.AbstractScale, {
	_valueMin: 0,
	_valueMax: 0,
	_displayMin: 0,
	_displayMax: 0,
	
	setValueMin: function(min) {
		this._valueMin = min;
	},
	
	setValueMax: function(max) {
		this._valueMax = max;
	},
	
	setDisplayMin: function(min) {
		this._displayMin = min;
	},
	
	setDisplayMax: function(max) {
		this._displayMax = max;
	},
	
	setInverted: function(inverted) {
		this._inverted = inverted;
	},
	
	setScale: function(scale) {
		this._scale = scale;
	},
	
//	setFunction: function(f) {
//		if(f == "log") {
//			this._function = function(x) { return Math.log(x); }
//		} else if(dojo.isFunction(f)) {
//			this._function = f;
//		}
//		// fall through to linear is this._function = undefined
//	},
	
	/**
	 * Get the pixel position given a value on the scale.
	 */
	_getPlacement: function(value) {
//		if(!dojo.lang.isNumeric(value)) {
//			throw "Attempted to assign non-numeric value to linear scale"; 
//		}
		var scaleFactor = this._getScaleFactor();
		var relativeDisplayPosition = (value - this._valueMin) * scaleFactor;
		if(this._inverted) {
			return this._displayMax - relativeDisplayPosition;
		} else {
			return this._displayMin + relativeDisplayPosition;
		}
	},
	
	/**
	 * Get the value on scale from a given pixel position.
	 */
	getInversePlacement: function(pixels) {
		var scaleFactor = this._getScaleFactor();
		var valuePosition = (pixels - this._displayMin) / scaleFactor;
		if(this._inverted) {
			return this._valueMax - valuePosition;
		} else {
			return this._valueMin + valuePosition;
		}
	},
	
	_getScaleFactor: function() {
		var scaleFactor = null;
		if(this._scale) {
			scaleFactor = this._scale;
		} else {
			var valueRange = Math.abs(this._valueMax - this._valueMin);
			if(valueRange === 0) {
				console.error("value range for scale is zero");
			}
			var displayRange = Math.abs(this._displayMax - this._displayMin);
			if(displayRange === 0) {
				console.error("display range for scale is zero");
			}
			scaleFactor = displayRange / valueRange;
		}
		return scaleFactor;
	},
	
	getValueMin: function() {
		return this._valueMin;
	},
	
	getValueMax: function() {
		return this._valueMax;
	},
	
	getDisplayMin: function() {
		return this._displayMin;
	},
	
	getDisplayMax: function() {
		return this._displayMax;		
	}
});