/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.render.policies.Repeat");

dojo.require("rmpw.common.policy.Policy");

/**
 * Generates a sequence of values, then create a duplicates of a template render mapping
 * for each value it produces.  Primarily designed for building major and minor division
 * lines on charts. 
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.diagrams.render.policies.Repeat", rmpw.common.policy.Policy, {
	
	setTemplate: function(template) {
		this._template = template;
		this._parametersChanged = true;
	},
	
	setMin: function(min) {
		this._min = min;
		this._parametersChanged = true;
	},
	
	setMax: function(max) {
		this._max = max;
		this._parametersChanged = true;
	},
	
	setSpacing: function(spacing) {
		this._spacing = spacing;
		this._parametersChanged = true;
	},
	
	/**
	 * Key of scale to bind to
	 * TODO (MG): make this optional
	 */
	setScale: function(key) {
		this._scale = this.findNearestParentByKey("scale_" + key, true);
		if(!this._scale) {
			throw key + " scale not found";
		}
		this._parametersChanged = true;
	},
	
	enforce: function() {
	    var i, shape, value, argSetter;
		if(this._parametersChanged) {
			if(this._templateInstances) {
				for(i = 0, shape; shape = this._templateInstances[i]; i++) {
					shape.destroy();
				}
				this._templateInstances = null;
			}
		
			// TODO (MG): don't assume a numeric scale
			var valueMin;
			if(this._min !== undefined) {
				valueMin = Math.max(this._scale.getValueMin(), this._min);
			} else {
				valueMin = this._scale.getValueMin();
			}
			// snap minimum to a multiple of the spacing
			valueMin = Math.ceil(valueMin / this._spacing) * this._spacing;
			
			var valueMax;
			if(this._max !== undefined) {
				valueMax = Math.min(this._scale.getValueMax(), this._max);
			} else {
				valueMax = this._scale.getValueMax();
			}
	
			this._templateInstances = [];
			var range =  Math.abs(valueMax - valueMin);
			var repeats = 1 + Math.floor(range / this._spacing);
			for(i = 0; i < repeats; i++) {
				var template = new this._template.relation();
				var args = this._template.arguments;
				// Note: null is the special flag to inject the generated value, needs to be set first
				for(argSetter in args) {
					value = args[argSetter];
					if(!value) {
						template[argSetter](valueMin + i * this._spacing);
					}
				}
				for(argSetter in args) {
					value = args[argSetter];
					if(value) {
						template[argSetter](value);
					}
				}
				this.host.addChild(template);
				this._templateInstances.push(template);
			}
			this._parametersChanged = false;
		}
	}
});