/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.render.policies.ScaleBinding");

dojo.require("rmpw.common.policy.Policy");
dojo.require("rmpw.diagrams.render.util.Workarounds");

/**
 * Binds a shape to a value on a scale.  Primarily designed to place data points on a chart, by adding
 * scale bindings to each of the x and y axes for the same shape.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.diagrams.render.policies.ScaleBinding", rmpw.common.policy.Policy, {
	
	/**
	 * Key of scale to bind to
	 */
	setScale: function(key) {
		this._scale = this.findNearestParentByKey("scale_" + key);
		if(!this._scale) {
			console.error(key + " scale not found");
		}
	},
	
	/**
	 * Value on the scale for this shape (could be a number, date, string key, etc...)
	 */
	setValue: function(value) {
		this._value = value;
	},
	
	setAnchor: function(anchor) {
		this._anchor = anchor;
	},
	
	/**
	 * Shape to reposition based on its position in the scale.
	 */
	setShape: function() {
		this._shapes = arguments;
	},
	
	/**
	 * When setting a line's endpoints instead of setting a position transform, specifies the
	 * point(s) of the line to affect. 
	 */
	setVertex: function() {
		this._vertexIndexes = arguments;
	},
	
	enforce: function() {
	    var i, shape;
		var assign = this.getPlacement();
		var KEY = 0, VALUE = 1;
		
		var shapeCount = this._shapes.length;
		if(this._vertexIndexes) {
			// assign vertex coordinates
			var vertexCount = this._vertexIndexes.length;
			for(i = 0; i < shapeCount; i++) {
				shape = this._shapes[i];
				var shapeDef = shape.shape;
				if(shapeDef.type === "line") {
					for(var j = 0; j < vertexCount; j++) {
						var vertex = this._vertexIndexes[j];
						if(vertex <= 1) {
							shapeDef[assign[KEY] + (vertex + 1)] = assign[VALUE];
						}
					}
				}
				shape.setShape(shapeDef);
			}
		} else {
			// position element by transform
			for(i = 0; i < shapeCount; i++) {
				shape = this._shapes[i];
				var transform = shape.getTransform() || {};
				if(assign[KEY] === 'x') {
					this._updateTransform(transform, shape, assign[VALUE], 'dx', 'width');
				} else if(assign[KEY] === 'y') {
					this._updateTransform(transform, shape, assign[VALUE], 'dy', 'height');
				} else {
					throw assign[KEY] + " axis not supported";
				}
				shape.setTransform(transform);
			}
		}
	},
	
	getPlacement: function() {
		return this._scale.getPlacement(this._value);
	},
	
	getInversePlacement: function(pixelValue) {
		return this._scale.getInversePlacement(pixelValue);
	},
	
	_updateTransform: function(transform, shape, value, component, span) {
		if(this._anchor === "start") {
			transform[component] = value;
		} else {
			var bounds = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(shape, null, shape.parent);
			if(this._anchor === "middle") {
				transform[component] = value - (bounds[span] / 2);
			} else {
				transform[component] = value - bounds[span];
			}
		}		
	}
});