/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.render.policies.Styleable");

dojo.require("rmpw.diagrams.render.util.ColorConstants");
dojo.require("rmpw.diagrams.render.util.Image");

/**
 * Mappings that install this policy can have persisted styles
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.render.policies.Styleable", rmpw.common.policy.Policy, {
	singleton: ["styleable"],
	
	constructor: function() {
	},
	
	setLineShape: function() {
		if (!this._lineColorShapes) {
			this._lineColorShapes = [];
		}
			
		for (var i=0, lg=arguments.length; i<lg; i++) {
			if (arguments[i] && arguments[i].shape) {
				this._lineColorShapes.push(arguments[i]);
			}
		}
	},
	
	isLineStyled: function(shape) {
		for (var i=0, lg=this._lineColorShapes.length; i<lg; i++) {
			if (this._lineColorShapes[i] == shape) {
				return true;
			}
		}
		
		return false;
	},
	
	removeLineShape: function(shape) {
		if (this._lineColorShapes) {
			var idx = dojo.indexOf(this._lineColorShapes, shape);
			if (idx != -1) {
				this._lineColorShapes.splice(idx, 1);
			}
		}
	},
	
	removeFillShape: function(shape) {
		if (this._fillColorShapes) {
			var idx = dojo.indexOf(this._fillColorShapes, shape);
			if (idx != -1) {
				this._fillColorShapes.splice(idx, 1);
			}
		}
	},
	
	setFillShape: function() {
		if (!this._fillColorShapes) {
			this._fillColorShapes = [];
		}
			
		for (var i=0, lg=arguments.length; i<lg; i++) {
			if (arguments[i]) {
				this._fillColorShapes.push(arguments[i]);
			}
		}
	},
	
	setTextShape: function() {
		this._textShapes = arguments;
	},
	
	destroy: function() {
		this.inherited(arguments);
	},
	
	setDefaultFillColor: function(fillColor) {
		this.defaultFillColor = new dojo.Color(fillColor);
	},
	
	getDefaultFillColor: function() {
		if (this.defaultFillColor) {
			return this.defaultFillColor;
		}
		return rmpw.diagrams.render.util.ColorConstants.RSA_FILL;
	},
	
	setDefaultLineColor: function(lineColor) {
		this.defaultLineColor = new dojo.Color(lineColor);
	},
	
	getDefaultLineColor: function() {
		if (this.defaultLineColor) {
			return this.defaultLineColor;
		}
		return rmpw.diagrams.render.util.ColorConstants.RSA_OUTLINE;
	},
	
	_completeInternalTraces: function() {
		this.inherited(arguments);
		
		// determine which aspects of notation aren't traced yet and add internal traces to complete
		var internalTraces = [];
		
		this._createInternalTrace("fillColor", this.getDefaultFillColor(), internalTraces);
		this._createInternalTrace("lineColor", this.getDefaultLineColor(), internalTraces);
		this._createInternalTrace("lineWidth", null, internalTraces);
		this._createInternalTrace("lineStyle", null, internalTraces);
		this._createInternalTrace("fontColor", null, internalTraces);
		this._createInternalTrace("fontFamily", null, internalTraces);
		this._createInternalTrace("fontStyle", null, internalTraces);
		this._createInternalTrace("fontWeight", null, internalTraces);
		this._createInternalTrace("fontSize", null, internalTraces);
		
		if (internalTraces.length > 0) {
			this.host.connectTraces(this.host.sources.__metadata, internalTraces);
		}
	},
	
	_resetImageSource: function (image) {
		var maintainAspectRatio = false;
		if (this.host.policies.graphNodeLabelLayout) {
			maintainAspectRatio = this.host.policies.graphNodeLabelLayout.maintainImageAspectRatio();
		}
		
		rmpw.diagrams.render.util.Image.resetImage(image, maintainAspectRatio,
				this.host.sources.__metadata.fillColor.toHex(), 
				this.host.sources.__metadata.lineColor.toHex());
	},
	
	enableFillColorTool: function() {
		if (this._fillColorShapes && this._fillColorShapes.length > 0) {
			return true;
		}
		return false;
	},
	
	enableLineColorTool: function() {
		if (this._lineColorShapes && this._lineColorShapes.length > 0) {
			return true;
		}
		return false;
	},
	
	enableLineStyleTools: function() {
		if (this._lineColorShapes) {
			for (var i = 0, lg = this._lineColorShapes.length; i<lg; i++) {
				var lineShape = this._lineColorShapes[i]; 
				if (lineShape.shape.type == "polyline") {
					return true;
				}
			}
		}
		return false;
	},
	
	enableFontTools: function() {
		if (this._textShapes && this._textShapes.length > 0) {
			return true;
		}
		return false;
	},
	
	setFillColor: function( fillColor ){
		if( fillColor){
			this.host.sources.__metadata.fillColor = new dojo.Color(fillColor);
			
			if (this._fillColorShapes) {
				for (var i = 0, lg = this._fillColorShapes.length; i<lg; i++) {
					var fillColorShape = this._fillColorShapes[i]; 
					if (fillColorShape.shape.type == "image" ||
						fillColorShape.shape.type == "scalingImage") {
						this._resetImageSource(fillColorShape);
					}
					else {
						fillColorShape.setFill(this.host.sources.__metadata.fillColor);  
					}
				}
			}
		}
	},
	
	getFillColor: function() {
		if (this.host.sources.__metadata.fillColor) {
			return this.host.sources.__metadata.fillColor;
		}
		if (this._fillColorShapes && this._fillColorShapes.length > 0) {
			return this._fillColorShapes[0].getFill();
		}
		return null;
	},
	
	setLineColor: function( lineColor ){
		if( lineColor){
			this.host.sources.__metadata.lineColor = new dojo.Color(lineColor);
			
			if (this._lineColorShapes) {
				for (var i = 0, lg = this._lineColorShapes.length; i<lg; i++) {
					var lineColorShape = this._lineColorShapes[i]; 
					if (lineColorShape.shape.type == "image" ||
						lineColorShape.shape.type == "scalingImage") {
						this._resetImageSource(lineColorShape);
					}
					else {
						var stroke = lineColorShape.getStroke();
						var width = dojox.gfx.defaultStroke.width;
						var style = dojox.gfx.defaultStroke.style;
						if (stroke) {
							width = stroke.width ? stroke.width : width;
							style = stroke.style ? stroke.style : style;
						}
						lineColorShape.setStroke({color: this.host.sources.__metadata.lineColor, width: width, style: style});  
					}
				}
			}
		}
	},
	
	getLineColor: function() {
		if (this.host.sources.__metadata.lineColor) {
			return this.host.sources.__metadata.lineColor;
		}
		if (this._lineColorShapes && this._lineColorShapes.length > 0) {
			var stroke = this._lineColorShapes[0].getStroke();
			if (stroke) {
				return stroke.color;
			}
		}

		return null;
	},
	
	setLineWidth: function( lineWidth ){
		if (lineWidth) {
			this.host.sources.__metadata.lineWidth = lineWidth;
			
			if (this._lineColorShapes) {
				for (var i = 0, lg = this._lineColorShapes.length; i<lg; i++) {
					var lineColorShape = this._lineColorShapes[i]; 
					if (lineColorShape.shape.type == "polyline") {
						var stroke = lineColorShape.getStroke();
						var color = dojox.gfx.defaultStroke.color;
						var style = dojox.gfx.defaultStroke.style;
						if (stroke) {
							color = stroke.color ? stroke.color : color;
							style = stroke.style ? stroke.style : style;
						}
						lineColorShape.setStroke({color: color, width: this.host.sources.__metadata.lineWidth, style: style});  
					}
				}
			}
		}
	},	
	
	setLineStyle: function( lineStyle ){
		if (lineStyle) {
			this.host.sources.__metadata.lineStyle = lineStyle;
			
			if (this._lineColorShapes) {
				for (var i = 0, lg = this._lineColorShapes.length; i<lg; i++) {
					var lineColorShape = this._lineColorShapes[i]; 
					if (lineColorShape.shape.type == "polyline") {
						var stroke = lineColorShape.getStroke();
						var color = dojox.gfx.defaultStroke.color;
						var width = dojox.gfx.defaultStroke.width;
						if (stroke) {
							color = stroke.color ? stroke.color : color;
							width = stroke.width ? stroke.width : width;
						}
						lineColorShape.setStroke({color: color, width: width, style: this.host.sources.__metadata.lineStyle});  
					}
				}
			}
		}
	},	
	
	setFontColor: function( fontColor ){
		if( fontColor){
			this.host.sources.__metadata.fontColor = new dojo.Color(fontColor);
			
			if (this._textShapes) {
				for (var i = 0, lg = this._textShapes.length; i<lg; i++) {
					var textShape = this._textShapes[i]; 
					textShape.setFill(this.host.sources.__metadata.fontColor);
				}
			}
		}
	},
	
	getFontColor: function() {
		if (this.host.sources.__metadata.fontColor) {
			return this.host.sources.__metadata.fontColor;
		}
		if (this._textShapes && this._textShapes.length > 0) {
			return this._textShapes[0].getFill();
		}
		return null;
	},
	
	setFontFamily: function( fontFamily ){
		if( fontFamily){
			this.host.sources.__metadata.fontFamily = fontFamily;
			
			if (this._textShapes) {
				for (var i = 0, lg = this._textShapes.length; i<lg; i++) {
					var textShape = this._textShapes[i]; 
					var font = textShape.getFont();
					font.family = fontFamily;
					textShape.setFont(font);  
				}
			}
		}
		if (this.host.policies.graphNodeLabelLayout) {
			this.host.policies.graphNodeLabelLayout.enforce();
		}
	},
	
	setFontStyle: function( fontStyle ){
		if( fontStyle){
			this.host.sources.__metadata.fontStyle = fontStyle;
			
			if (this._textShapes) {
				for (var i = 0, lg = this._textShapes.length; i<lg; i++) {
					var textShape = this._textShapes[i]; 
					var font = textShape.getFont();
					font.style = fontStyle;
					textShape.setFont(font);  
				}
			}
		}
		if (this.host.policies.graphNodeLabelLayout) {
			this.host.policies.graphNodeLabelLayout.enforce();
		}
	},
	
	setFontWeight: function( fontWeight ){
		if( fontWeight){
			this.host.sources.__metadata.fontWeight = fontWeight;
			
			if (this._textShapes) {
				for (var i = 0, lg = this._textShapes.length; i<lg; i++) {
					var textShape = this._textShapes[i]; 
					var font = textShape.getFont();
					font.weight = fontWeight;
					textShape.setFont(font);  
				}
			}
		}
		if (this.host.policies.graphNodeLabelLayout) {
			this.host.policies.graphNodeLabelLayout.enforce();
		}
	},
	
	setFontSize: function( fontSize ){
		if( fontSize){
			this.host.sources.__metadata.fontSize = fontSize;
			
			if (this._textShapes) {
				for (var i = 0, lg = this._textShapes.length; i<lg; i++) {
					var textShape = this._textShapes[i]; 
					var font = textShape.getFont();
					font.size = fontSize;
					textShape.setFont(font);  
				}
			}
		}
		if (this.host.policies.graphNodeLabelLayout) {
			this.host.policies.graphNodeLabelLayout.enforce();
		}
	},
	
	getFont: function() {
		if (this._textShapes && this._textShapes.length > 0) {
			return this._textShapes[0].getFont(); 
		}	
	},
	
	/*
	 * method that gets called for an extra logic required by graph elements for invalidation
	 */
	enforce: function() {
		
		this._enforceInternalTrace("fillColor", "setFillColor");
		this._enforceInternalTrace("lineColor", "setLineColor");
		this._enforceInternalTrace("lineWidth", "setLineWidth");
		this._enforceInternalTrace("lineStyle", "setLineStyle");
		this._enforceInternalTrace("fontColor", "setFontColor");
		this._enforceInternalTrace("fontFamily", "setFontFamily");
		this._enforceInternalTrace("fontStyle", "setFontStyle");
		this._enforceInternalTrace("fontWeight", "setFontWeight");
		this._enforceInternalTrace("fontSize", "setFontSize");
		
		var graphEl = this.host.policies.graphElement;
		if (graphEl && !graphEl.isInitialized()) {
			this._completeInternalTraces();
			graphEl._invalidate();
		}
		
		this.inherited(arguments);
	},
	
	getState: function() {
		var state = {};
		
		var fillColor = this._getInternalTraceData("fillColor");
		if (fillColor != this.getDefaultFillColor()) {
			state.fillColor = fillColor;	
		}
		
		var lineColor = this._getInternalTraceData("lineColor");
		if (lineColor != this.getDefaultLineColor()) {
			state.lineColor = lineColor;
		}
		
		var lineWidth = this._getInternalTraceData("lineWidth");
		if (lineWidth) {
			state.lineWidth = lineWidth;
		}
		
		var lineStyle = this._getInternalTraceData("lineStyle");
		if (lineStyle) {
			state.lineStyle = lineStyle;
		}
		
		var fontColor = this._getInternalTraceData("fontColor");
		if (fontColor) {
			state.fontColor = fontColor;
		}
		
		var fontFamily = this._getInternalTraceData("fontFamily");
		if (fontFamily) {
			state.fontFamily = fontFamily;
		}
		
		var fontStyle = this._getInternalTraceData("fontStyle");
		if (fontStyle) {
			state.fontStyle = fontStyle;
		}
		
		var fontWeight = this._getInternalTraceData("fontWeight");
		if (fontWeight) {
			state.fontWeight = fontWeight;
		}
		
		var fontSize = this._getInternalTraceData("fontSize");
		if (fontSize) {
			state.fontSize = fontSize;
		}
		
		return state;
	},
	
	setState: function(state) {
		if (!state) {
			return;
		}
		
		if (state.fillColor) {
			// force line color update because resetImageSource is dependent on both being set
			this.host.sources.__metadata.lineColor = new dojo.Color(state.lineColor ? state.lineColor : this.getDefaultLineColor());
			
			this.setFillColor(state.fillColor);	
			if (this.host.policies.graphElement) {
				this.host.policies.graphElement.getInternalGraphElement().invalidate();
			}
		}
		if (state.lineColor) {
			this.setLineColor(state.lineColor);
			if (this.host.policies.graphElement) {
				this.host.policies.graphElement.getInternalGraphElement().invalidate();
			}
		}
		if (state.lineWidth) {
			this.setLineWidth(state.lineWidth);
			if (this.host.policies.graphElement) {
				this.host.policies.graphElement.getInternalGraphElement().invalidate();
			}
		}
		if (state.lineStyle) {
			this.setLineStyle(state.lineStyle);
			if (this.host.policies.graphElement) {
				this.host.policies.graphElement.getInternalGraphElement().invalidate();
			}
		}
		if (state.fontColor) {
			this.setFontColor(state.fontColor);
			if (this.host.policies.graphElement) {
				this.host.policies.graphElement.getInternalGraphElement().invalidate();
			}
		}
		if (state.fontFamily) {
			this.setFontFamily(state.fontFamily);
			if (this.host.policies.graphElement) {
				this.host.policies.graphElement.getInternalGraphElement().invalidate();
			}
		}
		if (state.fontStyle) {
			this.setFontStyle(state.fontStyle);
			if (this.host.policies.graphElement) {
				this.host.policies.graphElement.getInternalGraphElement().invalidate();
			}
		}		
		if (state.fontWeight) {
			this.setFontWeight(state.fontWeight);
			if (this.host.policies.graphElement) {
				this.host.policies.graphElement.getInternalGraphElement().invalidate();
			}
		}		
		if (state.fontSize) {
			this.setFontSize(state.fontSize);
			if (this.host.policies.graphElement) {
				this.host.policies.graphElement.getInternalGraphElement().invalidate();
			}
		}

	}
	
});