/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.common.policy.Policy");

/**
 * Base class for all policies.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.common.policy.Policy", null, {
	constructor: function(host) {
		this.host = host;
	},
	
	destroy: function() {
		delete this.host;
	},
	
	/**
	 * Walk up the parent chain of mappings to find the first render policy
	 * stored at the given key, and return a reference to that policy.
	 */ 
	findNearestParentByKey: function(key, allowSelf) {
		var policy = null;
		var host = allowSelf ? this.host : this.host.parent;
		while(host && !policy) {
			policy = host.policies[key];
			host = host.parent;
		}
		return policy;
	},
	
	/**
	 * Look for child mapping of this key.
	 */ 
	findChildByKey: function(key) {
		for (var i=0, lg=this.host.children.length; i<lg; i++) {
			var child = this.host.children[i];
			if (child.policies[key]) {
				return child.policies[key];
			}
		}
		return null;
	},
	
	/**
	 * Enforces the policy.
	 * 
	 * Subclasses should implement.
	 */
	enforce: function() {
		// do nothing
	},
	
	/*
	 * gets the state of this policy so it can be persisted
	 */
	getState: function() {
		return {};  // no state by default
	},
	
	/*
	 * restores the state of this policy based on persisted data
	 */
	setState: function(state) {
		// do nothing
	},
	
	_getInternalTraceName: function(propertyName) {
		return "__internal" + propertyName;
	},
	
	_getInternalTraceData: function(propertyName) {
		var trace = this.host.traces[this._getInternalTraceName(propertyName)];
		if (trace) {
			return trace.get();
		}
		
		return null;
	},
	
	_createInternalTrace: function(propertyName, initialValue, internalTraces) {
		var internalTraceName = this._getInternalTraceName(propertyName);
		if (!this.host.sources.__metadata) {
			this.host.sources.__metadata = {};
		}
		var graphSource = this.host.sources.__metadata;
		if (graphSource[propertyName] === undefined) {
			graphSource[propertyName] = initialValue;
		}
		this.host.createTrace(internalTraceName, "__metadata." + propertyName, initialValue);
		internalTraces.push(internalTraceName);
	},
	
	_enforceInternalTrace: function(propertyName, setTraceFunction) {
		var trace = this.host.traces[this._getInternalTraceName(propertyName)];
		if (trace && !trace.synced) {
			this[setTraceFunction](trace.get());
			trace.reset();
		}
	},
	
	/*
	 * Parses through the existing notation traces and fills in the remainder with traces to internal notation
	 * injected into the mappings.
	 */
	_completeInternalTraces: function() { }
});