/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.process.OperationReport");

rmps.core.process.OperationStatus = {
	ERROR: "ERROR",
	OK: "OK"
};

dojo.declare("rmps.core.process.OperationReport", null, {
	
	overallStatus: null, // "ERROR"||"OK"
	
	liscenseInfo: null, // {granted: false|true, message: ""}
	
	actions: null, // array of {actionId: "", allowed: false|true}
	
	constructor: function() {
		this.actions = [];
	},
	
	getAction: function(actionId) {
		for (var i = 0; i < this.actions.length; ++i) {
			if (actionId == this.actions[i].actionId) {
				return this.actions[i];
			}
		}
		return null;
	},
	
	mixin: function(report){
		var reportActions= report.actions;
		var getAction= function(reportActions, id){
			for (var i = 0; i < reportActions.length; ++i) {
				if (id == reportActions[i].actionId) {
					return reportActions[i];
				}
			}
			return null;	
		};
		if(report instanceof rmps.core.process.OperationReport){
			for (var i = 0; i < this.actions.length; ++i) {
				var action= this.actions[i];
				var reportAction= getAction(reportActions, action.actionId);
				
				if (reportAction) {
					/*Make sure to make the values for overlapped operations to be true
					 * in case one report enables it*/
					if(reportAction.allowed){
						action.allowed= true;	
					}
					/*splice this entry so, we will know that we already searched it */
					reportActions.splie(dojo.indexOf(reportAction, reportAction),1);
				}
			}
			/*concact the rest operations in case there are some more exist in the report to be mixened*/
			this.actions= this.actions.concat(reportActions);
		}
	},
	
	/*
	 * Return true if the action exists and is allowed, false otherwise.
	 */
	isAllowed: function(actionId) {
		var action = this.getAction(actionId);
		return action ? action.allowed : false;
	},
	
	parse: function(report) {
		var prefix = "jp06:";
		
		for (var key in report.namespaces) {
			var namespace = report.namespaces[key];
			if (namespace == "http://jazz.net/xmlns/prod/jazz/process/0.6/") {
				prefix = key + ":";
			}
		}
		
		this.overallStatus = report.getElementsByTagName(prefix + "operation-report")[0].getAttribute(prefix + "overallStatus");
		
		var licenseInfo = report.getElementsByTagName(prefix + "license-info")[0];
		
		this.liscenseInfo = {};
		this.liscenseInfo.granted = licenseInfo.getAttribute(prefix + "granted") == "true";
		
		var message = licenseInfo.getElementsByTagName(prefix + "message")[0];
		if (message) {
			this.liscenseInfo.message = message.nodeValue;
		}
		
		var actions = report.getElementsByTagName(prefix + "action");
		
		for (var i = 0; i < actions.length; ++i) {
			this.actions.push({
				actionId: actions[i].getAttribute(prefix + "actionId"),
				allowed: actions[i].getAttribute(prefix + "allowed") == "true"
			});
		}
	}
});

})();