/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.process.SecurityClient");

dojo.require("rmps.core.process.OperationReport");
dojo.require("rmps.core.process.ProjectRolePermissions");
dojo.require("dojo.DeferredList"); 
dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants");

dojo.require("jazz.client.xhr");

dojo.require("dojox.xml.DomParser");

dojo.require("rmps.core.RootServices");

var PUBLIC_CONTEXT = "/_PUBLIC_CONTEXT";

var SECURITY_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.JP_0_6, "processSecurity");
var PERMISSIONS_SERVICE= "/service/com.ibm.team.process.internal.service.web.IProcessWebUIService/permissions";
var PROJECT_AREAS= "/process/project-areas";
var MEMBERS= "members";
var ROLE_ASSIGMENTS= "role-assignments";


var PermissionContext= com.ibm.team.process.web.common.internal.ProcessConstants.PermissionContext;

rmps.core.process.SecurityClient = {
	
	isActionAllowed: function(operation, context, actions) {
		var uri = SECURITY_PATH + "/operations/" + operation + "/report?context=" + context;
		
		for (var i = 0; i < actions.length; ++i) {
			uri += "&action=" + actions[i];
		}
		
		var args = {
			url: uri,
			handleAs: "text",
			load: function(response, ioArgs) {
				var doc = dojox.xml.DomParser.parse(response);
				var report = new rmps.core.process.OperationReport();
				report.parse(doc);
				return report;
			}
		};
		return jazz.client.xhrGet(args);
	},
	
	/*Get all possible permissions in for a given user in all existing defined roles in a specific project area. 
	 * 
	 * Example:
	 * 		SecurityClient.getUserPermissionsInProject(test, function(results){alert(results)}, null, true)
	 * 
	 */
	getUserPermissionsInProject: function(userId, callback, context, sync){
		if(!context){
			context= jazz.app.context.get().itemId;	
		}
		var deferred = rmps.core.process.SecurityClient.getUserRoleAssigmentsInProject(userId, context, sync); 
		deferred.addCallback(this, function(roles){
			var deferreds= [];
			for(var index=0; index< roles.length; index++){
				var role= roles[index];
				if(!dojo.isString(role)){
					continue;
				}
				deferreds.push(rmps.core.process.SecurityClient.getProjectRolePermissions(role.substring(role.lastIndexOf("/")+1, role.length),context, null,sync));	
			}
			var deferredList = new dojo.DeferredList(deferreds);
			deferredList.addCallback(function(results){
				var all= {};
				
				/*This check in order to get the true permissions in all roles associated to the current project member. As we could have
				 * false value for a specific permission in a role, while on other role it could be true; in that case, the true role is the one
				 * to be effective */
				var getPermittedRoles= function(object){
					for(var att in object){
						var current= object[att];
						if(current== "true"){
							all[att]= true;
						}
					}
				};
				for(var index=0; index<results.length; index++){
					var current= results[index];
					if(dojo.isArray(current)){
						for(var j=0; j<current.length; j++){
							var subCurrent= current[j];
							if(subCurrent instanceof rmps.core.process.ProjectRolePermissions){
								getPermittedRoles(subCurrent.rolePermissions);		
							}
						}						
					}else  if(current instanceof rmps.core.process.ProjectRolePermissions){	/*Safe check, we cannot be here */
						getPermittedRoles(current.rolePermissions);		
					}
				}
				if(callback){
					callback(all);					
				}
			});
			return deferredList;
		});
		return deferred;
	},
	
	getUserRoleAssigmentsInProject: function(userId, context, sync){
		if(!context){
			context= jazz.app.context.get().itemId;	
		}
		
		var args = {
			url: net.jazz.ajax._contextRoot+ PROJECT_AREAS + "/"+ context+ "/"+ MEMBERS+ "/"+ encodeURIComponent(userId)+ "/"+ ROLE_ASSIGMENTS,
			handleAs: "text",
			sync: sync,
			load: function(response, ioArgs) {
				var doc = dojox.xml.DomParser.parse(response);
				var assigments= [];
				var response= doc.getElementsByTagName("jp06:role-assignments")[0];
				if(!response){
					return;
				}
				var roleAssigments= response.getElementsByTagName("jp06:role-assignment");				
				for(var index=0; index<roleAssigments.length; index++){
					var current= roleAssigments[index];
					var roleUrl= roleAssigments[index].getElementsByTagName("jp06:role-url")[0];
					if(!roleUrl){
						continue;
					}
					var roleUrlChild= roleUrl.childNodes[0];
					
					if(!roleUrlChild){
						continue;
					}
					var value= roleUrlChild.nodeValue;
					if(value){
						assigments.push(value);
					}
				}
				return assigments;
			}
		};
		return jazz.client.xhrGet(args);
	},
	
	getProjectRolePermissions: function(roleId, context, params, sync){
		if(!params){params= {};}
		
		if(!context){
			context= jazz.app.context.get().itemId;	
		}
		
		var object= {
			processAreaItemId:    context,
			processAreaContext:   params.processAreaContext?params.processAreaContext:PermissionContext.PROJECT,	/*We are getting roles at the project scope */
			context: 			  params.context?params.context:PermissionContext.TEAM,								/*The specified roles are defined in the team configurations */
			contextId: 			  params.contextId?params.contextId:"", 
			roleId: 			  roleId
		};
		var args = {
			url: net.jazz.ajax._contextRoot+ PERMISSIONS_SERVICE + "?" + dojo.objectToQuery(object),
			handleAs: "text",
			sync: sync,
			load: function(response, ioArgs) {
				var doc = dojox.xml.DomParser.parse(response);
				var permissions= new rmps.core.process.ProjectRolePermissions();
				permissions.parse(doc);
				return permissions;
			}
		};
		return jazz.client.xhrGet(args);
	}
};

})();