/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.properties.HyperlinksPopup");

dojo.require("rmps.ui.properties.HyperlinksSection");
dojo.require("rmps.ui.EllipsisLabel");
dojo.require("rmps.core.error");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.ui.resource.ResourceIconLabel");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.layout.StackContainer");
dojo.require("dijit.form.Button");

dojo.require("dojo.dnd.Moveable");
dojo.require("dojo.i18n");

dojo.require("dojox.layout.ResizeHandle");

dojo.requireLocalization("rmps.ui.common", "CommonMessages");
dojo.requireLocalization("rmps.ui.properties", "HyperlinksPopupMessages");

var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;
var load_sync = jazz.core.loader.load_sync;

dojo.declare("rmps.ui.properties.HyperlinksPopup", [dijit._Widget, dijit._Templated], {
	
	templateString: dojo.cache("rmps.ui", "properties/templates/HyperlinksPopup.html"),
	widgetsInTemplate: true,
	
	commonMessages: dojo.i18n.getLocalization("rmps.ui.common", "CommonMessages"),
	messages: dojo.i18n.getLocalization("rmps.ui.properties", "HyperlinksPopupMessages"),
	
	// static properties
	_allPopups: [],
	
	// attach points
	_container: null,
	_titleBar: null,
	_titleNode: null,
	_toolbar: null,
	_resizeHandle: null,
	_closeButton: null,
	_footer: null,
	_ril: null,
	
	// mixin properties
	resizable: true,
	duration: 400,
	initialSize: null,
	resourceUri: null,
	linkHref: null,
	
	// resizeAxis: String
	//		One of: x | xy | y to limit pane's sizing direction
	resizeAxis: "xy",
	activeResize: true,
	minSize: null,
	
	// properties
	isOpen: false,
	
	// private properties
	_moveHandle: null,
	_defaultSize: null,
	_startZ: 750,
	_titleLabel: null,
	
	constructor: function() {
		this.initialSize = {};
		var v = dijit.getViewport();
		this._defaultSize = {
			l: Math.max(Math.floor(v.l + ((v.w - 500) / 2)), 0),
			t: Math.max(Math.floor(v.t + ((v.h - 300) / 2)), 0),
			w: 500,
			h: 300
		};
		
	},
	
	postCreate: function() {
		dojo.body().appendChild(this.domNode);
		
		this.domNode.style.zIndex = this._startZ;
		
		this.connect(this._toolbar, "onmousedown", function(e) {
			e.stopPropagation();
		});
		
		this._moveHandle = new dojo.dnd.Moveable(this.domNode, { handle: this._titleBar });
		if (!this.resizable) {
			this._resizer.style.display = "none"; 	
		}
		
		dojo.marginBox(this.domNode, dojo.mixin(this._defaultSize, this.initialSize));
		
		// add this popup to the start of the list of all popups
		this._allPopups.splice(0, 0, this);
		
		// resize handle
		this.connect(this._titleBar, "onmousedown", this.bringToTop);
		this.connect(this._resizer, "onmousedown", this.bringToTop);
		
		this._container.domNode.style.overflow = "auto";
		if (this.resizable) {
			this._resizeHandle = new dojox.layout.ResizeHandle({ 
				targetId: this.id, 
				resizeAxis: this.resizeAxis,
				activeResize: this.activeResize,
				minSize: this.minSize
			}, this._resizer);
			this._resizeHandle.domNode.className = "resizeHandle";
		}
	},
	
	startup: function(){
		if (this._started) {
			return;
		}
		
		this.inherited(arguments);

		this._titleLabel = dojo.doc.createTextNode(this.messages.title);
		this._titleNode.appendChild(this._titleLabel);
		
		// show the links
		load_sync("rmps.ui.properties.HyperlinksSection");
		var widgetConstructor = dojo.getObject("rmps.ui.properties.HyperlinksSection");
		
		var args = this.multipleHyperlinks || {};
		
		var page = new widgetConstructor(args);
		this._container.addChild(page);
		this._updateMinSize();
		this.resize();
		
		this.isOpen = false;
		this.count = 0;
	},
	
	destroy: function() {
		this._controller.destroy();
		
		this._allPopups.splice(dojo.indexOf(this._allPopups, this), 1);
		if (this._moveHandle) {
			this._moveHandle.destroy();
			delete this._moveHandle;
		}
		this.inherited(arguments);
	},
	
	onFocus: function() {
		this.bringToTop();
	},
	
	onBlur: function() {
		if (dojo.isIE) {
			// FIXME work around to prevent popup from closing in IE
			// when a rich text editor is loaded. 
			// The rich text editor causes focus to occur outside of the popup
			// when initializing the editor commands in IE.
			// Simple check is to see if the call stack contains the
			// rich text editor method "_localizeEditorCommands".
			var currentFunction = arguments.callee.caller;
			var callbackFunction = function() {
				dijit.focus(this.domNode);
			};
			while (currentFunction) {
				if (currentFunction.nom == "_localizeEditorCommands") {
					setTimeout(dojo.hitch(this, callbackFunction), 1);
					return;
				}
				currentFunction = currentFunction.caller;
			}
		}
		this.close();
	},
	
	resize: function(size) {
		if (!this.isOpen) {
			return;
		}
		if (!size) {
			size = dojo.marginBox(this.domNode);
		} else {
			dojo.marginBox(this.domNode, size);
		}
		
		var mb = {
			w: Math.max(size.w - 6, 0),
			h: Math.max(size.h - 6 - this._titleBar.offsetHeight - this._footer.offsetHeight, 0)
		};
		
		if (this._titleLabel) {
			var titleWidth = mb.w - this._toolbar.offsetWidth;
			dojo.marginBox(this._titleNode, { w: titleWidth });
		}
		
		this._container.resize(mb);
		
		dojo.marginBox(this._footer, { w: mb.w });
	},
	
	show: function(callback) {
		
		if( this.isOpen == false ) {
							
			this.bringToTop();
			
			this.domNode.style.visibility = "visible";
			this.domNode.style.display = "";
			
			dijit.focus(this.domNode);
			this._updateMinSize();
			this.isOpen = true;
			this.resize(dojo.coords(this.domNode));
		}
	},
	
	hide: function(callback) {
		
		if( this.isOpen == true ){
			this.isOpen = false;
			this.domNode.style.display = "none";
			this.domNode.style.visibility = "hidden";
		}
	},
	
	close: function(){
		this.hide(dojo.hitch(this, function() {
			this.destroyRecursive();
		})); 
	},
	
	_updateMinSize: function() {
		if (this._resizeHandle && !this.minSize) {
			var width = this._toolbar.offsetWidth + 6;
			if (this._ril) {
				width += this._ril.labelNode.offsetLeft;
			}
			this._resizeHandle.minSize = {
				w: width,
				h: this._titleBar.offsetHeight + this._footer.offsetHeight + 6
			};
		}
	},
	
	bringToTop: function() {
		if (this._allPopups.length > 0 && this._allPopups[this._allPopups.length - 1] != this) {
			this._allPopups.splice(dojo.indexOf(this._allPopups, this), 1);
			this._allPopups.push(this);
			
			dojo.forEach(this._allPopups, function(popup, i){
				popup.domNode.style.zIndex = this._startZ + (i * 2);
			}, this);
		}
	},
	
	_onCloseClick: function(event) {
		this.close();
	}
});

rmps.ui.properties.HyperlinksPopup.show = function(data) {
	var popup = new rmps.ui.properties.HyperlinksPopup(data || {});
	popup.startup();
	setTimeout(function() {
		popup.show();
	}, 0);
	return popup;
};

dojo.subscribe("rmps.ui.properties.HyperlinksPopup.show", null, rmps.ui.properties.HyperlinksPopup.show);
})();