/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.properties.HyperlinksSection");

dojo.require("dijit._Widget");
dojo.require("dojo.i18n");

dojo.require("rmps.app.core.util");

dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.ResourceLink");

dojo.requireLocalization("rmps.ui.properties", "HyperlinksSectionMessages");
	
var messages = dojo.i18n.getLocalization("rmps.ui.properties", "HyperlinksSectionMessages");
var modelsFrontService = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "models");			

dojo.declare("rmps.ui.properties.HyperlinksSection", dijit._Widget, {
	
	"class": "rmps-ui-hyperlinks-section",
	
	// private properties
	_table: null,
	
	postCreate: function() {
		this.containerNode = dojo.create("div", null, this.domNode);
		
		var columns = [
			{
				label: messages.column_property
			},
			{
				label: messages.column_value
			}
		];
		
		this._table = new jazz.ui.StyledTable({
			columns: columns	
		});
		this._table.containerNode = this._table.domNode;
		
		if (this.names && this.urls) {
			for (var i = 0, name; name = this.names[i]; ++i) {
				this._table.tbody.appendChild(this._createLinkRow(name, this.urls[i]));
			}
		}
		
		this.domNode.appendChild(this._table.domNode);
	},
	
	_createLinkRow: function(label, value) {
		var tr = dojo.create("tr");
		var propertyCell = dojo.create("td", {"class": "label"}, tr);
		var valueCell = dojo.create("td", {"class": "value"}, tr);
		
		propertyCell.appendChild(dojo.doc.createTextNode(label));
		this._populateValueCell(valueCell, value);
		
		return tr;
	},

	_populateValueCell: function(valueCell, value) {
		var linkNode;
		if (value && value.indexOf(modelsFrontService) === 0) {
			// remove fragment -- TODO: use fragment to select view in opened diagram
			var uri = value;
			var idx = uri.lastIndexOf('#');
			if (idx != -1) {
				uri = uri.substr(0, idx);
			}
			var resourceLink = new jazz.ui.ResourceLink({
				uri: net.jazz.ajax._contextRoot + "/rendering?resourceUri=" +  encodeURIComponent(uri),
				presentationUri: "#action=rmps.app.design.edit&resource=" + uri,
				lazyFetch : true,
				label: uri
			});
			
			valueCell.appendChild(resourceLink.domNode);
			resourceLink.startup();
            this.connect(resourceLink.domNode, "onclick", "close");
		} else {
			linkNode = dojo.create("a", { href: value, target: "_blank" }, valueCell);
			linkNode.appendChild(dojo.doc.createTextNode(value));
            this.connect(linkNode, "onclick", "close");
		}
		return true;
	},
	
	
	close: function(){
		var popup= this.domNode.parentNode.parentNode.parentNode.parentNode;
		popup.style.display = "none";
		popup.style.visibility = "hidden";
		this.destroyRecursive(); 
	}
	
	
});

})();