/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.properties.PropertiesController");

dojo.require("rmps.core.error");
dojo.require("rmps.core.properties.PropertiesClient");
dojo.require("rmps.ui.properties.SectionContainer");

dojo.require("jazz.core.loader");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.requireLocalization("rmps.ui.properties", "PropertiesControllerMessages");

var messages = dojo.i18n.getLocalization("rmps.ui.properties", "PropertiesControllerMessages");

var load_sync = jazz.core.loader.load_sync;

var PropertiesClient = rmps.core.properties.PropertiesClient;

dojo.declare("rmps.ui.properties.PropertiesController", null,  {
	
	// private properties
	_view: null,
	
	constructor: function(view) {
		this._view = view;
	},
	
	destroy: function() {
		if (this._deferred) {
			this._deferred.cancel();
			delete this._deferred;
		}
	},
	
	getProperties: function() {
		var deferred = PropertiesClient.getPropertiesFor(this._view.resourceUri);
		
		deferred.addCallback(this._view, this._view.onPropertiesLoaded);
		deferred.addCallback(this, this._constructSections);
		deferred.addErrback(this, function(error) {
			if (error.dojoType != "cancel") {
				var action = dojo.string.substitute(messages.error_loadingProperties_action, [this._view.resourceUri]);
				if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
					error = rmps.core.error.createXhrError(deferred.ioArgs, action);
				} else {
					error = rmps.core.error.createError(error, action);
				}
				
				if (this._view.onLoadPropertiesFailed) {
					this._view.onLoadPropertiesFailed(error.getGenericError());
				} else {
					error.dispatch();
				}
			}
		});
		
		this._deferred = deferred;
		return deferred;
	},
	
	_createPage: function() {
		return new rmps.ui.properties.SectionContainer();
	},
	
	_constructSections: function(data) {
//		if (this._view.setTitle) {
//			this._view.setTitle(data.label, data.icon);
//		}
//		
		for (var i = 0; i < data.categories.length; ++i) {
			var category = data.categories[i];
			
			var page = this._createPage();
			page.title = category.label;
			
			for (var j = 0; j < category.sections.length; ++j) {
				var section = category.sections[j];
				
				load_sync(section.widget);
				
				var widgetConstructor = dojo.getObject(section.widget);
				var args = section.mixin || {};
				
				// override templatePath
				if (args.templateString) {
					args.templatePath = null;
				}
				
				args.sectionId = section.id;
				
				var sectionWidget = new widgetConstructor(args);
				sectionWidget.sectionId = section.id;
				page.addSection(sectionWidget);
			}
			
			this._view.addPage(page);
		}
	}
});

})();