/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.properties.PropertiesPopup");

dojo.require("rmps.ui.properties.PropertiesController");
dojo.require("rmps.ui.EllipsisLabel");
dojo.require("rmps.core.error");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.ui.resource.ResourceIconLabel");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.layout.StackContainer");
dojo.require("dijit.form.Button");
//dojo.require("dijit.Toolbar");
//dojo.require("dijit.Menu");

dojo.require("dojo.dnd.Moveable");
dojo.require("dojo.i18n");

dojo.require("dojox.layout.ResizeHandle");

dojo.requireLocalization("rmps.ui.common", "CommonMessages");
dojo.requireLocalization("rmps.ui.properties", "PropertiesPopupMessages");

var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;

dojo.declare("rmps.ui.properties.PropertiesPopup", [dijit._Widget, dijit._Templated], {
	
	templateString: dojo.cache("rmps.ui", "properties/templates/PropertiesPopup.html"),
	widgetsInTemplate: true,
	
	commonMessages: dojo.i18n.getLocalization("rmps.ui.common", "CommonMessages"),
	messages: dojo.i18n.getLocalization("rmps.ui.properties", "PropertiesPopupMessages"),
	
	// static properties
	_allPopups: [],
	
	// attach points
	_container: null,
	_titleBar: null,
	_titleNode: null,
	_toolbar: null,
	_resizeHandle: null,
	_prevButton: null,
	_nextButton: null,
	_pinButton: null,
	_openButton: null,
	_closeButton: null,
	_pageSelect: null,
	_footer: null,
	_status: null,
	_ril: null,
	
	// mixin properties
	resizable: true,
	duration: 400,
	initialSize: null,
	resourceUri: null,
	linkHref: null,
	
	// resizeAxis: String
	//		One of: x | xy | y to limit pane's sizing direction
	resizeAxis: "xy",
	activeResize: true,
	minSize: null,
	
	// properties
	isOpen: false,
	
	// private properties
	_controller: null,
	_moveHandle: null,
	_defaultSize: null,
	_startZ: 750,
	_titleLabel: null,
	
	constructor: function() {
		this.initialSize = {};
		var v = dijit.getViewport();
		this._defaultSize = {
			l: Math.max(Math.floor(v.l + ((v.w - 500) / 2)), 0),
			t: Math.max(Math.floor(v.t + ((v.h - 300) / 2)), 0),
			w: 500,
			h: 300
		};
		
	},
	
	postCreate: function() {
		this._getResourceInfo();
		if (!this.linkHref) {
			this._openButton.domNode.style.display = "none";
		}
		
		dojo.body().appendChild(this.domNode);
		
		this._controller = new rmps.ui.properties.PropertiesController(this);
		this.domNode.style.zIndex = this._startZ;
		this._updatePrevNextButtonsTitles();
		
		this.connect(this._toolbar, "onmousedown", function(e) {
			e.stopPropagation();
		});
		
		this._moveHandle = new dojo.dnd.Moveable(this.domNode, { handle: this._titleBar });
		if (!this.resizable) {
			this._resizer.style.display = "none"; 	
		}
		
		dojo.marginBox(this.domNode, dojo.mixin(this._defaultSize, this.initialSize));
		
		this.subscribe(this._container.id + "-selectChild", "_childSelected");
		this.subscribe(this._container.id + "-addChild", "_childAdded");
		
		var pages = this._container.getChildren();
		for (var i = 0; i < pages.length; ++i) {
			this._childAdded(pages[i]);
		}
		if (this._container.selectedChildWidget) {
			this._childSelected(this._container.selectedChildWidget);
		}
		
		this._updatePinState();
		
		// add this popup to the start of the list of all popups
		this._allPopups.splice(0, 0, this);
		
		// resize handle
		this.connect(this._titleBar, "onmousedown", this.bringToTop);
		this.connect(this._resizer, "onmousedown", this.bringToTop);
		
		this._container.domNode.style.overflow = "auto";
		if (this.resizable) {
			this._resizeHandle = new dojox.layout.ResizeHandle({ 
				targetId: this.id, 
				resizeAxis: this.resizeAxis,
				activeResize: this.activeResize,
				minSize: this.minSize
			}, this._resizer);
			this._resizeHandle.domNode.className = "resizeHandle";
		}
	},
	
	destroy: function() {
		this._controller.destroy();
		
		this._allPopups.splice(dojo.indexOf(this._allPopups, this), 1);
		if (this._moveHandle) {
			this._moveHandle.destroy();
			delete this._moveHandle;
		}
		this.inherited(arguments);
	},
	
	startup: function(){
		if (this._started) {
			return;
		}
		
		this.inherited(arguments);
		
		// get the properties
		this.setStatus(this.commonMessages.loading);
		this._controller.getProperties();
		this.isOpen = false;
		this.count = 0;
		this._pinButton.checked = true;
	},
	
	_getResourceInfo: function() {
		var deferred = ResourceInfoClient.getResourceInfo(this.resourceUri, ['shortName', 'iconUri', 'longName']);
		deferred.addCallback(this, function(infos) {
			this._ril = new rmps.ui.resource.ResourceIconLabel({
				isLink: this.linkHref != null,
				linkHref: this.linkHref,
				resourceInfo: infos[0]
			});
			
			this._titleLabel = new rmps.ui.EllipsisLabel({
				target: this._ril.labelNode,
				showTitle: false
			});
			
			this._titleLabel.containerNode.appendChild(this._ril.domNode);
			
			this._ril.startup();
			this._titleLabel.startup();
			
			this._titleNode.appendChild(this._titleLabel.domNode);
			
			this._titleLabel.offset = this._ril.labelNode.offsetLeft;
			this.resize();
		});
	},
	
	onPropertiesLoaded: function() {
		this.setStatus(null);
	},
	
	onLoadPropertiesFailed: function() {
		this.setStatus(this.commonMessages.loadingError);
	},
	
	onFocus: function() {
		this.bringToTop();
	},
	
	onBlur: function() {
		if (dojo.isIE) {
			// FIXME work around to prevent popup from closing in IE
			// when a rich text editor is loaded. 
			// The rich text editor causes focus to occur outside of the popup
			// when initializing the editor commands in IE.
			// Simple check is to see if the call stack contains the
			// rich text editor method "_localizeEditorCommands".
			var currentFunction = arguments.callee.caller;
			var callbackFunction = function() {
				dijit.focus(this.domNode);
			};
			while (currentFunction) {
				if (currentFunction.nom == "_localizeEditorCommands") {
					setTimeout(dojo.hitch(this, callbackFunction), 1);
					return;
				}
				currentFunction = currentFunction.caller;
			}
		}
		if (!this._pinButton.checked) {
			this.close();
		}
	},
	
	resize: function(size) {
		if (!this.isOpen) {
			return;
		}
		if (!size) {
			size = dojo.marginBox(this.domNode);
		} else {
			dojo.marginBox(this.domNode, size);
		}
		
		var mb = {
			w: Math.max(size.w - 6, 0),
			h: Math.max(size.h - 6 - this._titleBar.offsetHeight - this._footer.offsetHeight, 0)
		};
		
		if (this._titleLabel) {
			this._titleLabel.offset = this._ril.labelNode.offsetLeft;
			var titleWidth = mb.w - this._toolbar.offsetWidth;
			this._titleLabel.resize({w: titleWidth});
		}
		
		this._container.resize(mb);
		
		dojo.marginBox(this._footer, { w: mb.w });
	},
	
	setStatus: function(text) {
		this._status.innerHTML = "";
		if (text != null) {
			this._status.appendChild(dojo.doc.createTextNode(text));
		}
		this.resize();
	},
	
	show: function(callback) {
		
		if( this.isOpen == false ) {
							
			this.bringToTop();
			
			this.domNode.style.visibility = "visible";
			this.domNode.style.display = "";
			
			dijit.focus(this.domNode);
			this._updateMinSize();
			this.resize(dojo.coords(this.domNode));
			
			this.isOpen = true;
		}
	},
	
	hide: function(callback) {
		
		if( this.isOpen == true ){
			this.isOpen = false;
			this.domNode.style.display = "none";
			this.domNode.style.visibility = "hidden";
		}
	},
	
	close: function(){
		this.hide(dojo.hitch(this, function() {
			this.destroyRecursive();
		})); 
	},
	
	_updateMinSize: function() {
		if (this._resizeHandle && !this.minSize) {
			var width = this._toolbar.offsetWidth + 6;
			if (this._ril) {
				width += this._ril.labelNode.offsetLeft;
			}
			this._resizeHandle.minSize = {
				w: width,
				h: this._titleBar.offsetHeight + this._footer.offsetHeight + 6
			};
		}
	},
	
	bringToTop: function() {
		if (this._allPopups.length > 0 && this._allPopups[this._allPopups.length - 1] != this) {
			this._allPopups.splice(dojo.indexOf(this._allPopups, this), 1);
			this._allPopups.push(this);
			
			dojo.forEach(this._allPopups, function(popup, i){
				popup.domNode.style.zIndex = this._startZ + (i * 2);
			}, this);
		}
	},
	
	selectChild: function(idx){
		var pages = this._container.getChildren();
		if (idx < pages.length && idx >= 0) {
			this._doSelectChild(idx);
		}
	},
	
	addPage: function(page) {
		this._container.addChild(page);
		this._updateMinSize();
		this.resize();
	},
	
	_doSelectChild: function(idx) {
		var pages = this._container.getChildren();
//		this._container.selectChild(pages[idx]);
		// new
		this._container.selectChild(pages[this._pageSelect.selectedIndex]);
	},
	
	_childSelected: function(page) {
		var pages = this._container.getChildren();
		var idx = dojo.indexOf(pages, page);
//		
//		var menuItems = this._pageSelectMenu.getChildren();
//		if (this._pageIdx != -1) {
//			dojo.removeClass(menuItems[this._pageIdx].iconNode, "dropdownBullet");
//		}
//		dojo.addClass(menuItems[idx].iconNode, "dropdownBullet");
//		
//		this._pageSelect.attr("label", page.title);
//		
//		this._pageIdx = idx;

		// new
		this._pageSelect.selectedIndex = idx;

		this._updatePrevNextButtonsTitles();
		
		this._updateMinSize();
		this.resize();
	},
	
	_childAdded: function(page) {
//		var pages = this._container.getChildren();
//		var idx = pages.length - 1;
//		var menuItem = new dijit.MenuItem({
//			label: page.title,
//			onClick: dojo.hitch(this, function() {
//				this._doSelectChild(idx);
//			})
//		});
//		this._pageSelectMenu.addChild(menuItem);

		// new
		// add option to the select box
		var option = dojo.doc.createElement("option");
		option.value = page.title;
		option.appendChild(dojo.doc.createTextNode(page.title));
		this._pageSelect.appendChild(option);
		
		
		this._updatePrevNextButtonsTitles();
	},
	
	_updatePrevNextButtonsTitles: function() {
		var pages = this._container.getChildren();
		if (pages.length <= 1) {
			this._prevButton.attr("disabled", true);
			this._nextButton.attr("disabled", true);
			this._prevButton.domNode.title = "";
			this._nextButton.domNode.title = "";
			return;
		}
		this._prevButton.attr("disabled", false);
		this._nextButton.attr("disabled", false);
		
//		var idx = this._pageIdx;
		
		// new
		var idx = this._pageSelect.selectedIndex;
		
		var prevPageIdx = idx == 0 ? pages.length - 1 : idx - 1;
		var nextPageIdx = idx == pages.length - 1 ? 0 : idx + 1;
		this._prevButton.domNode.title = dojo.string.substitute(this.messages.goto_page, [pages[prevPageIdx].title]);
		this._nextButton.domNode.title = dojo.string.substitute(this.messages.goto_page, [pages[nextPageIdx].title]);
	},
	
	_updatePinState: function() {
		if (this._pinButton.checked) {
			this._pinButton.title = this.messages.pin;
		} else {
			this._pinButton.title = this.messages.unpin;
		}
	},
	
	// button events
	
	_onPrevClick: function(event) {
		var pages = this._container.getChildren();
		if (pages.length == 0) {
			return;
		}
		
		// new
		if (this._pageSelect.selectedIndex == 0) {
			this._pageSelect.selectedIndex = (pages.length - 1);
		} else {
			this._pageSelect.selectedIndex--;
		}
		this._doSelectChild();
	},
	
	_onNextClick: function(event) {
		var pages = this._container.getChildren();
		if (pages.length == 0) {
			return;
		}
		
		// new
		if (this._pageSelect.selectedIndex == pages.length - 1) {
			this._pageSelect.selectedIndex = 0;
		} else {
			this._pageSelect.selectedIndex++;
		}
		this._doSelectChild();
	},
	
	_onPinClick: function(event) {
		this._updatePinState();
	},
	
	_onCloseClick: function(event) {
		this.close();
	},
	
	_onOpenClick: function(event) {
		window.location.href = this.linkHref;
	},
	
	// select
	_onPageSelectChange: function(event) {
		this._doSelectChild();
	}
});

rmps.ui.properties.PropertiesPopup.show = function(data) {
	var popup = new rmps.ui.properties.PropertiesPopup(data || {});
	popup.startup();
	setTimeout(function() {
		popup.show();
	}, 0);
	return popup;
};

dojo.subscribe("rmps.ui.properties.PropertiesPopup.show", null, rmps.ui.properties.PropertiesPopup.show);
})();