/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.properties.PropertiesViewer");

dojo.require("rmps.ui.properties.PropertiesController");

dojo.require("dijit._Widget");

dojo.require("jazz.ui.StyledBox");

dojo.declare("rmps.ui.properties.PropertiesViewer", [dijit._Widget], {
	
	"class": "rmps-ui-properties-viewer",
	
	// mixin properties
	resourceUri: null,
	numColumns: 1,
	
	// private properties
	_boxes: null, //{}
	_columns: null, //[]
	
	constructor: function() {
		this._boxes = {};
		this._columns = [];
	},
	
	postCreate: function() {
		this.containerNode = this.domNode;
		this._controller = new rmps.ui.properties.PropertiesController(this);
		
		var percent = Math.floor(100 / this.numColumns); 
		for (var i = 0; i < this.numColumns; ++i) {
			var column = dojo.doc.createElement("div");
			dojo.addClass(column, "rmps-ui-properties-viewer-column");
			column.style.width = percent + '%';
			this.domNode.appendChild(column);
			this._columns.push(column);
		}
	},
	
	startup: function() {
		if (this._started) {
			return;
		}
		
		this.inherited(arguments);
		
		// get the properties
		this._controller.getProperties();
	},
	
	destroy: function() {
		this._controller.destroy();
		this.inherited(arguments);
	},
	
	addPage: function(page) {
		var box = new jazz.ui.StyledBox({
			contentNode: page.domNode,
			primaryTitle: page.title,
			trim: jazz.ui.StyledBox.TrimStyles.GRAY,
			closable: false,
			collapsible: true,
			destroyContents: true
		});
		
		dojo.addClass(box.domNode, "rmps-ui-properties-viewer-page");
		this._boxes[page.id] = box;
		
		var idx = 0;
		for (var i = 1; i < this._columns.length; ++i) {
			if (this._columns[i].childNodes.length < this._columns[idx].childNodes.length) {
				idx = i;
			}
		}
		this._columns[idx].appendChild(box.domNode);
		dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
	},
	
	removePage: function(page) {
		var box = this._boxes[page.id];
		box.domNode.parentNode.removeChild(box.domNode);
		dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
	},
	
	onPropertiesLoaded: function() {},
	
	onLoadPropertiesFailed: function(error) {}
});

})();