/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.properties.form.RichTextArea");

dojo.require("dijit._Widget");

dojo.require("dijit._editor.plugins.EnterKeyHandling");
dojo.require("dijit._editor.plugins.TextColor");
dojo.require("dijit._editor.plugins.LinkDialog");
dojo.require("dijit._editor.plugins.TabIndent");

dojo.require("dijit.Editor");

dojo.declare("rmps.ui.properties.form.RichTextArea", dijit._Widget, {
	
	// mixin properties
	"class": "rmps-ui-properties-form-richTextArea",
	label: null,
	name: null,
	value: null,
	readOnly: false,
	height: 150,
	useExtraVerticalSpace: false,
	
	// properties
	editor: null,
	labelNode: null,
	
	postCreate: function() {
		if (this.label) {
			this.labelNode = dojo.create("div", null, this.domNode);
			this.labelNode.className = "label";
			this.labelNode.appendChild(dojo.doc.createTextNode(this.label));
		}
		
		this.editor = this.createEditor();
		if (this.editor.toolbar) {
			dojo.addClass(this.editor.toolbar.domNode, "rmpsToolbar");
		}
		if (this.value) {
			this.editor.setValue(this.value);
		}
		
		this.domNode.appendChild(this.editor.domNode);
		this.editor.startup();
	},
	
	destroy: function() {
		this.editor.destroyRecursive();
		this.inherited(arguments);
	},
	
	resize: function(size) {
		if (this.editor && this.useExtraVerticalSpace && size && size.h) {
			var height = size.h - this.domNode.offsetTop;
			if (height > 0 && (this.height == null || height > parseInt(this.height))) {
				this.editor.resize({h: height});
			}
		}
	},
	
	createEditor: function() {
		return new rmps.ui.properties.form.RichTextArea._Editor({
			height: this.height + "px",
			name: this.name,
			disabled: this.readOnly,
			plugins: [
				'undo',
  				'redo',
				'|',
				'cut',
				'copy',
				'paste',
				'|',
				'bold',
				'italic',
				'underline',
				'strikethrough',
				'|',
				'foreColor',
				'hiliteColor',
				'|',
				'subscript',
				'superscript',
				'|',
				'justifyLeft',
				'justifyRight',
				'justifyCenter',
				'justifyFull',
  				'|',
				'removeFormat',
				'|',
				'insertOrderedList',
				'insertUnorderedList',
				'indent',
				'outdent',
				'|',
				'createLink',
				'unlink',
				'insertImage',
				'insertHorizontalRule'
			]         	
		});
	}
	
});

dojo.declare("rmps.ui.properties.form.RichTextArea._Editor", dijit.Editor, {
	layout: function() {
		this.inherited(arguments);
		if (dojo.isIE >= 8) {
			this.editingArea.style.height = (parseInt(this.editingArea.style.height) - 2) + "px"; 
		}
	}
});