/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.query.TextQuery");

dojo.require("rmps.core.util.IRmpsConstants");
var IRmpsConstants = rmps.core.util.IRmpsConstants;
//a regular expression that matches any search terms that start with a wildcard (*, ?)
var WILDCARD_START_REGEXP = /(^|\s)(\?|\*)*/g;

dojo.requireLocalization("rmps.core.query", "TextQueryMessages");
var messages = dojo.i18n.getLocalization("rmps.core.query", "TextQueryMessages");

dojo.declare("rmps.core.query.TextQuery", null, {
	
	// properties
	name: null,
	description: null,
	
	text: null,
	page: null,
	pageSize: null,
	orderBy: null,
	limit: null,
	token: null,
	projectId: null,
	resourcesOnly: null,
	baseline: null,
	changeset: null,
	/**True if at least one of the original search terms starts with a wildcard*/
	hasWildcardStart: false,
	onlyType: null,
	includeSuppliers: null,
	includeSystemDomains: null,
	includeLibraries: null,
	inLibrariesOnly: null,
	searchById: null,

	constructor: function(params) {
		if (params) {
			this.page = params.queryPage ? parseInt(params.queryPage) : null;
			this.pageSize = params.pageSize ? parseInt(params.pageSize) : null;
			this.text = params.q || "";

			this.hasWildcardStart = this.text.match(WILDCARD_START_REGEXP) !== null;
			if (this.hasWildcardStart) {
				// sanitize the text to strip wildcards from the beginning of search terms
				this.text = this.text.replace(WILDCARD_START_REGEXP, '$1');
			}

			this.baseline = params[IRmpsConstants.BASELINE] ? params[IRmpsConstants.BASELINE] : params.baseline ? params.baseline : null;
			this.changeset = params[IRmpsConstants.CHANGESET] ? params[IRmpsConstants.CHANGESET] : params.changeset ? params.changeset : null;
			this.limit = params.limit ? parseInt(params.limit) : null;
			this.token = params.queryToken;
			this.resourcesOnly = params.resourcesOnly;
			if (params.sort) {
				this.orderBy = params.sort.split(",");
				for (var i = 0; i < this.orderBy.length; ++i) {
					if (this.orderBy[i].indexOf("up_") === 0) {
						this.orderBy[i] = {id: this.orderBy[i].substring(3), isAscending: true};
					} else {
						this.orderBy[i] = {id: this.orderBy[i].substring(5), isAscending: false};
					}
				} 
			}
			this.projectId = params.projectId;
			this.name = this.text;
			if (this.name) {
				this.name = dojo.trim(this.name.replace(/((typeName|package)#[^\s]+)|^\s+|\s+$/ig, " "));
			}
			this.description = params.description;
			this.onlyType = params.onlyType;
			this.includeSuppliers = params.includeSuppliers;
			this.includeSystemDomains = params.includeSystemDomains;
			this.includeLibraries = params.includeLibraries;
			this.inLibrariesOnly = params.inLibrariesOnly;
			this.searchById = params.searchById;
		}
	},

	toQueryString: function() {
		var params = {};
		this._getQueryStringParams(params);
		return dojo.objectToQuery(params);
	},

	_getQueryStringParams: function(params) {
		params.queryPage = this.page;
		params.pageSize = this.pageSize;
		
		if (this.orderBy) {
			var orderValue = '';
			for (var i = 0; i < this.orderBy.length; ++i) {
				if (orderValue.length > 0) {
					orderValue += ',';
				}
				orderValue += this.orderBy[i].isAscending ? "up_" : "down_";
				orderValue += this.orderBy[i].id;
			}
			params.sort = orderValue;
		}

		params.limit = this.limit;
		params.queryToken = this.token;
		params.projectId = this.projectId;
		
		if (this.baseline !== null) {
			params[IRmpsConstants.BASELINE] = this.baseline;
		} else {
			params[IRmpsConstants.BASELINE] = "";
		}
		if (this.changeset !== null) {
			params[IRmpsConstants.CHANGESET] = this.changeset;
		}
		
		params.includeSuppliers = this.includeSuppliers;
		params.includeSystemDomains = this.includeSystemDomains;
		params.includeLibraries = this.includeLibraries;
		params.inLibrariesOnly = this.inLibrariesOnly;
		params.searchById = this.searchById;
		
		this._getQueryStringSearchParams(params);
	},

	_getQueryStringSearchParams: function(params) {
		params.q = this.text;
		params.resourcesOnly = this.resourcesOnly; 
		params.onlyType = this.onlyType;
	},

	equals: function(obj) {
		if (obj === null || !(obj instanceof rmps.core.query.TextQuery)) {
			return false;
		}
		if (this.text === obj.text &&
			this.page === obj.page &&
			this.pageSize === obj.pageSize &&
			this.limit === obj.limit &&
			this.token === obj.token&&
			this.baseline === obj.baseline&&
			this.changeset === obj.changeset&&
			this.resourcesOnly === obj.resourcesOnly&&
			this.includeSuppliers === obj.includeSuppliers&&
			this.includeSystemDomains === obj.includeSystemDomains&&
			this.includeLibraries === obj.includeLibraries&&
			this.inLibrariesOnly === obj.inLibrariesOnly&&
			this.searchById === obj.searchById) {

			if ((this.orderBy === null || obj.orderBy === null) && this.orderBy !== obj.orderBy) {
				return false;
			}

			if (this.orderBy !== null && obj.orderBy !== null) {
				for (var i = 0; i < this.orderBy.length; ++i) {
					if ((this.orderBy[i].id !== obj.orderBy[i].id) || (this.orderBy[i].isAscending !== obj.orderBy[i].isAscending)) {
						return false;
					}
				}
			}
			return true;
		}
		return false;
	},

	/**
	 * Returns null if there are no problems with the query; if there are, returns an array of strings listing any issues.  
	 * If a query is not valid, it can still be sent to the server, but the server may return an error. 
	 *
	 */	
	validate: function() {		
		var errors = []; 

		if (dojo.trim(this.text).length === 0) {
			if(this.hasWildcardStart){
				//we "sanitized" this query string, but the resulting string was empty
				errors.push(messages.error_searchContainNonWildcard);
			} else {
				errors.push(messages.error_searchStringEmpty);
			}
		}

		if (errors.length === 0) {
			return null;
		} else {
			return errors;
		}
	}
});

})();