/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011,2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.ContainerSection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojox.xml.parser"); 
dojo.require("rmps.ui.MenuSection");
dojo.require("rmps.ui.queryeditor.PopupButton");

dojo.requireLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

(function(){

var messages = dojo.i18n.getLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");
	
dojo.declare("rmps.ui.queryeditor.ContainerSection", [dijit._Widget, dijit._Templated], {
	templatePath: dojo.moduleUrl("rmps.ui", "queryeditor/templates/ContainerSection.html"),
		
	_labelClose: messages.labelClose,
	_closeIconAlt: messages.labelClose,
	_closeIconTitle: messages.labelClose,
	_closeIconSrc: dojo.moduleUrl("rmps.ui", "graphics/queryWidget/close.gif"),
	_closeIconHoverSrc: dojo.moduleUrl("rmps.ui", "graphics/queryWidget/close-hov.gif"),
	_loadingIconSrc: dojo.moduleUrl("rmps.ui", "graphics/queryWidget/loading.gif"),
	_loadingText: messages.loadingText,
	_menuWidget: null,
	
	// mix-in
	sectionWidget: null,
	canDelete: true,
	
	postCreate: function() {
		if (this.sectionWidget) {
			if (!this.canDelete) {
				this._headerButtons.removeChild(this._close);
			}
			var sectionWidget = this.sectionWidget;
			sectionWidget.onAddContainer(this);
			this._headerText.innerHTML = sectionWidget.getTitle();
			this._bodyContainer.appendChild(sectionWidget.domNode);
			var isContainer = sectionWidget.isContainer();
			if (!isContainer) {
				dojo.addClass(this.domNode, "non-container");
			}
			
			// Setup the menu
			this.setupMenu(sectionWidget);
			
			dojo.connect(sectionWidget, "onMenuChange", this, function () {
				this.updateMenu(sectionWidget);
			});
		}
		
		this.inherited(arguments);
	},
	
	setupMenu: function (sectionWidget) {
		var menuItems = sectionWidget.getMenuItems();
		if (menuItems && menuItems.length > 0) {
			var menu = new rmps.ui.queryeditor.PopupButton({
				labelProperty: "label",
				callback: dojo.hitch(this, this._menuCallback)
			});
			menu.setValues(menuItems);
			// Set current selection
			var curItem = sectionWidget.getMenuSelection();
			menu.setSelection(curItem);
			if (sectionWidget.getMenuTitle) {
				menu.domNode.title = sectionWidget.getMenuTitle();
			}
			this._headerMenu.appendChild(menu.domNode);
			this._menuWidget = menu;
		}
	},
	
	updateMenu: function (sectionWidget) {
		var menuItems = sectionWidget.getMenuItems();
		if (menuItems && menuItems.length > 0) {
			this._menuWidget.setValues(menuItems);
			// Set current selection
			var curItem = sectionWidget.getMenuSelection();
			this._menuWidget.setSelection(curItem);
		}
	},
	
	destroy: function() {
		if (this._menuWidget) {
			this._menuWidget.destroy();
			delete this._menuWidget;
		}
		
		this.inherited(arguments);
	},
		
	onClose: function(e) {
		if(e) {
			dojo.stopEvent(e);
		}
		
		var destroy= true;
		var sectionWidget = this.sectionWidget;
		if (sectionWidget && sectionWidget.onClose) {
			// Add dirty
			sectionWidget.mainWidget.onMarkDirty("removedWidget");
			destroy= sectionWidget.onClose(e);
		}
		if (destroy) {
			this.destroy();
		}

		return destroy;
	},
		
	checkEnterForClose: function(e) {
		var characterCode;
		characterCode = e.keyCode;
		if(characterCode === 13) { //the key pressed was 'enter'
			this.onClose();
		}
	},
	
	performAction: function(action) {
		this.parentWidget.onWidgetAddition(action);
	},
		
	onCloseMouseOver: function(e) {
		this._closeImage.src = this._closeIconHoverSrc;
	},

	onCloseMouseOut: function(e) {
		this._closeImage.src = this._closeIconSrc;
	},
	
	showLoading: function(show) {
		if (show) {
			this._loadingMessage.style.display = "inline";
		} else {
			this._loadingMessage.style.display = "none";
		}
	},

	enableCloseButton: function(value) {
		if (value) {
			this._close.style.display = "";
			this._close.style.visibility = "visible";
		} else {
			this._close.style.display = "none";
			this._close.style.visibility = "hidden";
		}
	},
	
	_menuCallback: function(item) {
		this.sectionWidget.onMenuAction(item);
		// Update the title
		dojox.xml.parser.replaceChildren(this._headerText, document.createTextNode(this.sectionWidget.getTitle()));
		// Add dirty
		this.sectionWidget.mainWidget.onMarkDirty("menuChange");
	}
});

})();