/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.LinksWidget");

dojo.require("dojo.i18n");
dojo.require("rmps.ui.queryeditor.ListWidget");
dojo.require("rmps.ui.queryeditor.QueryConstants");

dojo.requireLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

(function(){

var messages = dojo.i18n.getLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");
var QueryConstants = rmps.ui.queryeditor.QueryConstants;
var SOURCE_ID = "source";
var TARGET_ID = "target";

dojo.declare("rmps.ui.queryeditor.LinksWidget", rmps.ui.queryeditor.ListWidget, {

	_queryConditionName: QueryConstants.LINK_CONDITION_TYPE, 
	_queryConditionListProperty: QueryConstants.LINK_CONDITION_LINK_TYPE_PROPERTY,
	_itemsName: messages.linkTypes,
	
	_direction: QueryConstants.LINK_CONDITION_DIRECTION_SOURCE,

	_menuItems: [{
		id: SOURCE_ID,
		label: messages.labelSource 
	}, {
		id: TARGET_ID,
		label: messages.labelTarget
	}],
	
	
	initializeFromConditionData: function () {
		// Make sure the list property is correct
		var conditionData = this.getConditionData();
		if (conditionData) {
			if (QueryConstants.LINK_CONDITION_LINK_TYPE_PROPERTY in conditionData) {
				this._queryConditionListProperty = QueryConstants.LINK_CONDITION_LINK_TYPE_PROPERTY;
			}
			if (conditionData[QueryConstants.LINK_CONDITION_DIRECTION_PROPERTY] === QueryConstants.LINK_CONDITION_DIRECTION_TARGET) {
				this._direction = QueryConstants.LINK_CONDITION_DIRECTION_TARGET; 
			}
		}
		
		this.setMultiple(true);
	},
	
	postCreate: function() {
		this.initializeFromConditionData();
		this.inherited(arguments);
	},
	
	getWidgetDataOptions: function() {
		var dataOptions = {
			predicateEnd: this._direction
		};

		return dataOptions;
	},
	
	getMenuItems: function() {
		// Return the different conditions
		return this._menuItems; 
	},
	
	_getMenuSelection: function() {
		if (this._direction === QueryConstants.LINK_CONDITION_DIRECTION_SOURCE) {
			// Source
			return this._menuItems[0];
		} else {
			// Target
			return this._menuItems[1];
		}		
	},
	
	getMenuSelection: function() {
		return this._getMenuSelection();
	},
	
	_onMenuAction: function(item) {
		var id = item.id;
		if (id === SOURCE_ID) {
			this._direction = QueryConstants.LINK_CONDITION_DIRECTION_SOURCE;
		} else if (id === TARGET_ID) {
			this._direction = QueryConstants.LINK_CONDITION_DIRECTION_TARGET;
		}
	},
	
	onMenuAction: function(item) {
		this._onMenuAction(item);
	},
	
	serialize: function() {
		var obj = this.inherited(arguments);
		
		var condition = obj[this._queryConditionName];
		condition[QueryConstants.LINK_CONDITION_DIRECTION_PROPERTY] = this._direction;
		
		return obj;
	},
	
	validate: function(rootTab) {
		return this.inherited(arguments); 
	}
});
}());