/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.ListWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.FilteringSelect");
dojo.require("dojo.i18n");
dojo.require("jazz.ui.ListBox");
dojo.require("jazz.ui.FilterBox");
dojo.require("rmps.ui.queryeditor.IQueryWidget");
dojo.require("rmps.ui.queryeditor.QueryConstants");
dojo.require("dojo.store.Memory");

dojo.requireLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

(function(){

var QueryConstants = rmps.ui.queryeditor.QueryConstants;
var DUMMY_ID = "dummy";
var LOADING_ID = "loading";

dojo.declare("rmps.ui.queryeditor.ListWidget", [dijit._Widget, dijit._Templated, rmps.ui.queryeditor.IQueryWidget], {
	templatePath: dojo.moduleUrl("rmps.ui", "queryeditor/templates/ListWidget.html"),
	widgetsInTemplate: true,
	messages: dojo.i18n.getLocalization("rmps.ui.queryeditor", "QueryWidgetMessages"),
	
	_addImg: dojo.moduleUrl("rmps.ui", "graphics/queryWidget/add-condition.gif"),
	_closeImg: dojo.moduleUrl("rmps.ui", "graphics/queryWidget/close.gif"),
	_closeHoverImg: dojo.moduleUrl("rmps.ui", "graphics/queryWidget/close-hov.gif"),

	_queryConditionListProperty: null,
	_queryConditionName: null,
	_skipOnChange: 1,
	_defWidgetData: null,
	_listBoxWidget: null,
	_multiple: false,
	_selection: null,
	_selectionChangeCallbacks: null,
	_initialized: false,
	
	constructor: function() {
		this._selectionChangeCallbacks = [];
	},
		
	onClose: function(e) {
		this.inherited(arguments);
		this.destroy();
		return true;
	},
	
	postCreate: function() {		
		if (this._multiple) {
			this._currentSelectionList = new jazz.ui.ListBox({
				allowMultipleSelections: false,
				margin: "10px 2px 2px 2px",
				height: "100px",
				sortOrder: 2
			}); 
			this._listContainer.insertBefore(this._currentSelectionList.domNode, this._editListButton.domNode);
			this._currentSelectionList.startup();
		} else {
			this._creteSingleSelectionUI();
		}

		this.refreshWidgetData();
		this._hideWarning();
		this.inherited(arguments);
		this._initialized = true;
	},
	
	_creteSingleSelectionUI: function() {
		dojo.style(this._listContainer, "display", "none");
		var stateStore = new dojo.store.Memory({
			data: [ {label : this.messages.loading, id: LOADING_ID} ]
		});
		
		
		this._filteringSelect = new dijit.form.FilteringSelect({
			value: LOADING_ID,
	        store: stateStore,// this.getDataStore()
	        searchAttr: "label",
	        onChange: dojo.hitch(this, this._onFilteringSelectChange),
	        queryExpr: "*${0}*",
	        autoComplete: false
		});
		
		this._filteringSelect.set('disabled',true);
		
		this._filteringSelectContainer.appendChild(this._filteringSelect.domNode);
		this._filteringSelect.startup();
	},
	
	_hideWarning: function() {
		this._warningMessage.style.display = 'none';
	},

	
	destroy: function() {
		if (this._currentSelectionList) {
			this._currentSelectionList.destroy();
			delete this._currentSelectionList;
		}
		if (this._filteringSelect) {
			this._filteringSelect.destroy();
			delete this._filteringSelect;
		}
		
		this.inherited(arguments);
	},
		
	refreshWidgetData: function() {
		if (this._multiple) {
			this._currentSelectionList.showLoading(true);
			this._selection = [];
		}
		this._setTempEnabled(false);
		
		var conditionData = this.getConditionData();
		if (conditionData) {
			this._selection = conditionData[this._queryConditionListProperty];
		}
		
		this._clearValues();
		
		// Assuming compatible data format		
		var store = this.getDataStore();
		def = this._defWidgetData = store.getWidgetData(this.getIdentifier(), this.getWidgetDataOptions());
		def.addCallback(this, function (data) {
			this._onWidgetDataReady(data);
			if ('_enabled' in this) {
				this.setEnabled(this._enabled);
			} else {
				this._setTempEnabled(true);
			}
		});
		def.addBoth(this, function() {
			if (this._multiple) {
				this._currentSelectionList.showLoading(false);
			}			
		});
	},
	
	_onWidgetDataReady: function (data) {
		this._dataReady = true;
		
		if (!data|| data.length === 0) {
			return;
		}
		if (this._multiple) {
			this._availableValues = {};
			this._unselectedEntries = [];
			var selectedEntries = [];
			var selectedObj = {};
			for (var i = 0; i < this._selection.length; ++i) {
				selectedObj[this._selection[i]] = true;
			}
			
			for (i = 0; i < data.length; ++i) {
				var entry = {
						label: data[i].label,
						id: data[i].id,
						value: data[i].targetType,
						type: 0	
				};
				this._availableValues[entry.id] = entry;
				if (selectedObj[entry.id]) {
					selectedEntries.push(entry);
				} else {
					this._unselectedEntries.push(entry);
				}
			}
	
			if (this._selection.length === 0) {
				selectedEntries.push(this._getDummyEntry());
			}
				
			this._currentSelectionList.addEntries(selectedEntries);
			this._onSelectionChange();
		} else {
			var new_store = new dojo.store.Memory({ 'data': data});
			this._filteringSelect.set('store', new_store);
			if (this._selection && this._selection.length > 0)  {
				this._filteringSelect.set('value', this._selection[0]);
			} else {
				this._filteringSelect.set('value', "");
			}
		}
	},
	
	_getWidgets: function() {
		if (this._multiple) {
			return [ this._currentSelectionList, this._editListButton];
		} else {
			return [this._filteringSelect];
		}
	},
	
	_setTempEnabled: function(value) {
		var widgets = this._getWidgets();
		for (var i = 0; i < widgets.length; ++i) {
			widgets[i].set('disabled', !value);
		}
	},

	setEnabled: function(value) {
		if (this._defWidgetData) {
			this._defWidgetData.addCallback(dojo.hitch(this, function(){
				this._setTempEnabled(value);
			}));
		} else {
			this._enabled = value;
		}
		
	},
	
	setMultiple: function(multiple) {
		var oldMultiple = this._multiple;
		this._multiple = multiple;
		if (!multiple && oldMultiple) {
			this._creteSingleSelectionUI();
			if (this._dataReady) {
				this.refreshWidgetData();
			}
		}
		
	},
		
	getSelectedValues: function() {
		if (!this._dataReady) {
			return this._selection ? dojo.map(this._selection, function(uri){ return { id: uri }; }) : []; 
		}
		
		var selection;
		if (this._multiple) {
			selection = this._getSelectedEntries();
			return dojo.map(selection, function(item) {
				return {
					id: item.id,
					targetType: item.value
				};
			});
		} else {
			selection = this._filteringSelect.item;
			return selection ? [selection] : [];
		}
	},
	
	addOnSelectionChangeHandler: function(callback) {
		if (callback) {
			this._selectionChangeCallbacks.push(callback);
		}
	},
	

	_onSelectionChange: function(newValue) {
		var result;
		if (this._skipOnChange === 0) {
			this.mainWidget.onMarkDirty("listSelection");
			result = true;
		} else {
			this._skipOnChange--;
			result = false;
		}
		this._hideWarning();
		for (var i = 0; i < this._selectionChangeCallbacks.length; ++i) {
			this._selectionChangeCallbacks[i]();
		}
		return result;
	},
	
	_clearValues: function() {
		if (this._multiple) {
			this._currentSelectionList.removeAllEntries();
		}
	},
			
	preSerialize: function() {
		return this._defWidgetData;
	},
	
	serialize: function() {
		var obj = {};
		var condition = obj[this._queryConditionName] = {};
		condition[this._queryConditionListProperty] = dojo.map(this.getSelectedValues(), function(item) {
			return item.id;
		});
		condition[QueryConstants.CONDITION_PRESENTATION_PROPERTY] = this.getIdentifier();
		condition[QueryConstants.CONDITION_IGNORE_CONTEXT_PROPERTY] = false;
		return obj;
	},
	
	validate: function (rootTab) {
		var valResult = { 
			hasNonContainerPortlet: true,
			errors: []
		};
		var obj = this.serialize();
		var list = obj[this._queryConditionName];
		if (list[this._queryConditionListProperty].length === 0) {
			var str = dojo.string.substitute(this.messages.valEmptyDataNotAllowed, [this._queryConditionName, this._queryConditionListProperty]);
			valResult.errors.push(str);
			this._showWarning(str);
		}
		return valResult;
	},
	
	_showWarning: function (message, lowSeverity) {
		while (this._warningMessage.hasChildNodes()) {
			this._warningMessage.removeChild(this._warningMessage.childNodes[0]);
		}
		if (lowSeverity) {
			dojo.addClass(this._warningMessage, 'low-severity');
		} else {
			dojo.removeClass(this._warningMessage, 'low-severity');
		}
		
		if (message) {
			this._warningMessage.appendChild(document.createTextNode(message));
			this._warningMessage.style.display = '';
		} else {
			this._warningMessage.style.display = 'none';
		}
	},
	
	isViewpointEditor: function() {
		var page_id = net.jazz.ajax.ui.PlatformUI.getWorkbench().currentPageId;
		var pageHelper = rmps.ui.editor.PageHelper.getInstance(page_id);
		return (pageHelper.getActiveEditorInput().getParams().action === rmps.app.ui.internal.pages.design.DesignUtil.viewpointAction);
	},
	
	
	_getSelectedEntries: function() {
		var entries = this._currentSelectionList && this._currentSelectionList.getAllEntries();
		if (entries && entries.length === 1 && entries[0].id === DUMMY_ID) {
			return [];
		}
		return entries;
	},
	
	_onModifyList: function() {
		
		var subs = {
			items: this._itemsName || this.messages.defaultItems
		};
		
		var dialog = new jazz.ui.SwapListDialog({
			dialogTitleText: dojo.string.substitute(this.messages.editDialogTitle, subs),
			unselectedListTitleText: dojo.string.substitute(this.messages.unselectedTitle, subs),
			selectedListTitleText: dojo.string.substitute(this.messages.selectedTitle, subs),
			sortField: "label",
			iconMap: [],
			onOK: dojo.hitch(this, function(dialog) {
				this._unselectedEntries = dialog.getUnselectedEntries();
				this._updateSelection(dialog.getSelectedEntries());
				dialog.destroy();
			}),
			onCancel: function(dialog) {
				dialog.destroy();
			}
		});
		
		dialog.setUnselectedEntries(this._unselectedEntries);
		dialog.setSelectedEntries(this._getSelectedEntries());
		dialog.show();
		if (dialog._multiSelectArea && dialog._multiSelectArea._unselectedFilterBox) {
			dialog._multiSelectArea._unselectedFilterBox.focus();
		}
		
	},
	
	_updateSelection: function(entries) {
		this._currentSelectionList.removeAllEntries();
		if (!entries || entries.length === 0) {
			entries.push(this._getDummyEntry());
		}
		this._currentSelectionList.addEntries(entries);
		this._onSelectionChange();
	},
	
	_getDummyEntry: function() {
		if (!this._dummy) {
			this._dummy = {
					label: this.messages.emptySelectionLabel,
					id: DUMMY_ID
				};
		}
		return this._dummy;
	},
	
	hideList: function() {
		dojo.style(this._listBox, "display", "none"); 
		dojo.style(this._editListButton.domNode, "display", "none");
	},
	
	_onFilteringSelectChange: function(newValue) {
		this._onSelectionChange(newValue);
	}


});
}());