/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.OntologyPropertiesChainWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.DeferredList");
dojo.require("rmps.ui.queryeditor.IQueryWidget");
dojo.require("rmps.ui.queryeditor.QueryWidgetRegistry");
dojo.require("rmps.ui.queryeditor.ContainerSection");
dojo.require("rmps.ui.queryeditor.OntologyPropertiesWidget");
dojo.require("rmps.ui.queryeditor.WidgetsContainer");

dojo.requireLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

(function(){

var messages = dojo.i18n.getLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");
var QueryWidgetRegistry = rmps.ui.queryeditor.QueryWidgetRegistry;
var QueryConstants = rmps.ui.queryeditor.QueryConstants;
	
dojo.declare("rmps.ui.queryeditor.OntologyPropertiesChainWidget", [rmps.ui.queryeditor.WidgetsContainer, rmps.ui.queryeditor.IQueryWidget], {
	_init: true,
	
	postCreate:function() {
		this.inherited(arguments);

		dojo.style(this._addConditionNode, {display: "none"});

		var conditionData = this.getConditionData();
		this.createWidget(QueryConstants.PREDICATE_WIDGET_ID, conditionData);
	},
	
	_queryChooserOptions: function () {
		return {
			parentWidget: this,
			useCategories: false,
			showItems: [QueryConstants.PREDICATE_WIDGET_ID, QueryConstants.PREDICATE_VALUE_WIDGET_ID, QueryConstants.PREDICATE_RESOURCE_WIDGET_ID],
			showOptionalPart: false,
			inEmptySection: false
		};
	},
		
	createSection: function(widget) {
		var section = this.inherited(arguments);
		
		this.connect(section, "onClose", "enablePredecessor");						
		this.connect(section, "onClose", "refreshConditionNode");							
	},
	
	onClose: function(e) {
		this.inherited(arguments);
		this.destroy();
		return true;
	},
	
	refreshConditionNode: function() {
		if (this._sections !== null && this._sections.children.length > 0) {
			var idx = this._sections.children.length - 1;
			var containerWidget = dijit.getEnclosingWidget(this._sections.children[idx]);
			if ((containerWidget.sectionWidget.getIdentifier() === QueryConstants.PREDICATE_WIDGET_ID) &&
				(containerWidget.sectionWidget.getSelectedValues().length !== 0)) {
				dojo.style(this._addConditionNode, {display: ""});
			} else {
				dojo.style(this._addConditionNode, {display: "none"});
			}
		}
	},
	
	disablePredecessor: function() {
		if (this._sections !== null && this._sections.children.length > 0) {
			var idx = this._sections.children.length === 1 ? 0 : this._sections.children.length - 2;
			var containerWidget = dijit.getEnclosingWidget(this._sections.children[idx]);
			containerWidget.enableCloseButton(false);
			containerWidget.sectionWidget.setEnabled(this._sections.children.length === 1);
		}
	},

	enablePredecessor: function() {
		if (this._sections !== null && this._sections.children.length > 0) {
			var containerWidget = dijit.getEnclosingWidget(this._sections.children[this._sections.children.length - 1]);
			containerWidget.sectionWidget.setEnabled(true);		
			containerWidget.enableCloseButton(this._sections.children.length > 1);
		}
	},
	
	createWidget: function(identifier, conditionData) {
		var widget_def = QueryWidgetRegistry.get(identifier);
		if (widget_def && widget_def.create) {
			var self = this;
			
			var predecessorWidget = null;
			if (this._sections !== null && this._sections.children.length > 0) {
				var containerWidget = dijit.getEnclosingWidget(this._sections.children[this._sections.children.length - 1]);
				predecessorWidget = containerWidget !== null ? containerWidget.sectionWidget : null;
			}
			
			var def = widget_def.create({
				mainWidget: this.mainWidget,
				conditionData: conditionData,
				predecessorWidget: predecessorWidget
			});
			def.addCallback(function(widget) {
				if (widget) {
					widget.setMultiple(false);
					self.createSection(widget);

					self.disablePredecessor();

					if (widget.getIdentifier() === QueryConstants.PREDICATE_WIDGET_ID) {
						widget.addOnSelectionChangeHandler(dojo.hitch(self, self.refreshConditionNode));
					}

					var widgetConditionData = widget.getConditionData();
					if (widgetConditionData && widgetConditionData[QueryConstants.CHAIN_CONDITION_NEXT_PROPERTY] !== undefined) {
						var nextWidgetId = null; 
						var nextWidgetConditionData = null;

						var nextCondition = widgetConditionData[QueryConstants.CHAIN_CONDITION_NEXT_PROPERTY];
						for (var condType in nextCondition) {
							nextWidgetConditionData = nextCondition[condType];
							nextWidgetId = nextWidgetConditionData[QueryConstants.CONDITION_PRESENTATION_PROPERTY].__value;
						}
						
						nextWidgetId = nextWidgetId === QueryConstants.PREDICATE_CHAIN_WIDGET_ID ? 
								QueryConstants.PREDICATE_WIDGET_ID : nextWidgetId;

						if (widget._init) {
							widget._defWidgetData.addCallback(self, function() {
								self.createWidget(nextWidgetId, nextWidgetConditionData);
							});
						} else {
							self.createWidget(nextWidgetId, nextWidgetConditionData);							
						}
					} else {
						if (widget._init) {
							widget._defWidgetData.addCallback(self, function() {
								self._init = false;
								self.refreshConditionNode();
							});
						} else {
							self._init = false;
							self.refreshConditionNode();
						}
					}
				}				
			});
		}
	},

	preSerialize: function() {
		var defs = [];
		dojo.forEach (this._allWidgets(), function (w) {
			var def = w.sectionWidget.preSerialize(); 
			if (def && def instanceof dojo.Deferred) {
				defs.push(def);
			}
		});
		if (defs.length === 0) {
			return null; // No need for a deferred.
		}
		return new dojo.DeferredList(defs);
	},
	
	serialize: function() {
		var obj = null;
		var condition = null;
		dojo.forEach (this._allWidgets(), function (w) {
			var svQueryConditionName = w.sectionWidget._queryConditionName;
			if (svQueryConditionName === QueryConstants.PREDICATE_CONDITION_TYPE) {
				// rename the first in chain
				w.sectionWidget._queryConditionName = QueryConstants.CHAIN_CONDITION_TYPE;
			}
			
			var sectionCondition = w.sectionWidget.serialize();
			if (obj === null) {
				obj = sectionCondition;
			} else {
				condition[QueryConstants.CHAIN_CONDITION_NEXT_PROPERTY] = sectionCondition;
			}
			
			condition = sectionCondition[w.sectionWidget._queryConditionName];
			if (condition[QueryConstants.CONDITION_PRESENTATION_PROPERTY] === QueryConstants.PREDICATE_WIDGET_ID) {
				condition[QueryConstants.CONDITION_PRESENTATION_PROPERTY] = this.getIdentifier();
			}
		
			w.sectionWidget._queryConditionName = svQueryConditionName;
        }, this);
		return obj;
	},
 
	validate: function (rootTab) {
		var valResult = {
			hasNonContainerPortlet: true,
			errors: []
		};

		// TODO: Add a check that no predicates appear in the chain after Value or Range conditions

		dojo.forEach(this._allWidgets(), function(w) {
			if (valResult.errors.length === 0) {
				var secResult = w.sectionWidget.validate(false);
				dojo.forEach(secResult.errors, function (e) {
					valResult.errors.push(e);
				});
			}
		});			

		return valResult;
	}
	
});
})();