/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.OntologyPropertiesResourcesWidget");

dojo.require("rmps.ui.queryeditor.OntologyPropertiesValueWidget");
dojo.require("rmps.ui.queryeditor.filters.EnumerationFilterWidget");
dojo.require("rmps.ui.queryeditor.QueryConstants");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

(function(){

var messages = dojo.i18n.getLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");
var QueryConstants = rmps.ui.queryeditor.QueryConstants;

dojo.declare("rmps.ui.queryeditor.OntologyPropertiesResourcesWidget", rmps.ui.queryeditor.OntologyPropertiesWidget, {

	_queryConditionName: QueryConstants.PREDICATE_RESOURCE_CONDITION_TYPE,
	_resourceType: null,
			
	postCreate: function() {
		this.inherited(arguments);
		
		this._defWidgetData.addCallback(this, function() {
			// Make sure the filter is updated
			this._skipOnChange++;
			this._onSelectionChange();
			// Set the value if any
			if (this._filterWidget) {
				this.updateFilterData();
			}
		});
	},
	
	getMenuItems: function() {
		// Return the different conditions
		return this._menuItems; 
	},
	
	getMenuSelection: function() {
		return this._getMenuSelection();
	},
	
	onMenuAction: function(item) {
		this._onMenuAction(item);
		
		// And make sure filters are updated
		this.inherited(arguments);
	},
	
	createFilter: function(targetType) {
		if (QueryConstants.USER_DATATYPE === targetType) {
			return new rmps.ui.queryeditor.filters.UserFilterWidget({
				changeCallback: dojo.hitch(this, this._filterChangeCallback),
				parentWidget: this
			});
		} else if (targetType.indexOf(QueryConstants.ENUM_DATATYPE) === 0) {
			return new rmps.ui.queryeditor.filters.EnumerationFilterWidget({
				changeCallback: dojo.hitch(this, this._filterChangeCallback),
				targetType: targetType,
				parentWidget: this
			});
		}
		
		// Other resources
		return new rmps.ui.queryeditor.filters.ResourcesFilterWidget({
			changeCallback: dojo.hitch(this, this._filterChangeCallback),
			dataType: targetType,
			parentWidget: this
		});
	}, 
	
	updateFilterData: function() {
		var conditionData = this.getConditionData();
		if (conditionData) {
			var resources = conditionData[QueryConstants.PREDICATE_RESOURCE_CONDITION_RESOURCE_PROPERTY];
			if (resources && this._filterWidget) {
				this._filterWidget.setResources(resources);
			}
		}
	},
	
	serialize: function() {
		var obj = this.inherited(arguments);
		
		var condition = obj[this._queryConditionName];
		
		// ResourceCondition
		condition[QueryConstants.PREDICATE_RESOURCE_CONDITION_RESOURCE_PROPERTY] = this._filterWidget.getResources();			
		
		return obj;
	},
	
	_filterChangeCallback: function() {
		if (!this._skipOnChange) {
			this.mainWidget.onMarkDirty("valueChange");
		}
		this._hideWarning();
	},
	
	_onSelectionChange: function() {
		var isUserChange = this.inherited(arguments);
	
		// Get selection
		var selection = this.getSelectedValues();
		if (selection.length !== 1) {
			return;			
		}

		var value = selection[0];
		var targetType = value.targetType;

		// Destroy old value filter widget
		var resources;
		if (this._filterWidget) {
			resources = this._filterWidget.getResources();
			this._filterWidget.destroy();
			delete this._filterWidget;
		}
		
		// Create value filter widget
		this._filterWidget = this.createFilter(targetType);
		if (this._filterWidget) {
			this._contentArea.appendChild(this._filterWidget.domNode);
		}

		this._filterWidget.setDataType(targetType);
		
		if (!isUserChange && resources) {
			this._filterWidget.setResources(resources);
		}
	},
	
	refreshWidgetData: function () {
		this.inherited(arguments);

		this.updateFilterData();
	},
	
	validate: function(rootTab) {
		var valResult = this.inherited(arguments); 
		
		if (!this._filterWidget) {
			valResult.errors.push(messages.valNoAssociatedResourceFilter);
		} else {
			this._filterWidget.validate(valResult);
		}
		
		return valResult;
	},
	
	_clearValues: function() {
		this.inherited(arguments);
		// Destroy old value filter widget
		if (this._filterWidget) {
			this._filterWidget.destroy();
			delete this._filterWidget;
		}

	}
});
}());