/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011,2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.OntologyPropertiesValueWidget");

dojo.require("dojo.i18n");
dojo.require("rmps.ui.queryeditor.OntologyPropertiesWidget");
dojo.require("rmps.ui.queryeditor.QueryConstants");
dojo.require("rmps.ui.queryeditor.filters.StringFilterWidget");
dojo.require("rmps.ui.queryeditor.filters.IntegerFilterWidget");
dojo.require("rmps.ui.queryeditor.filters.BooleanFilterWidget");
dojo.require("rmps.ui.queryeditor.filters.DateFilterWidget");
dojo.require("rmps.ui.queryeditor.filters.ResourcesFilterWidget");
dojo.require("rmps.ui.queryeditor.filters.UserFilterWidget");

dojo.requireLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

(function(){

var messages = dojo.i18n.getLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");
var QueryConstants = rmps.ui.queryeditor.QueryConstants;

dojo.declare("rmps.ui.queryeditor.OntologyPropertiesValueWidget", rmps.ui.queryeditor.OntologyPropertiesWidget, {

	_queryConditionName: QueryConstants.PREDICATE_VALUE_CONDITION_TYPE,
	_filterWidget: null,
	_valueType: null,
	_menuItemsValues: null,
	
	postCreate: function() {
		this.inherited(arguments);
		
		this._defWidgetData.addCallback(this, function() {
			// Make sure the filter is updated
			this._init = true; // List widgets call back is called first and reset this
			this._skipOnChange++;
			this._onSelectionChange();
			// Set the value if any
			if (this._filterWidget) {
				this.updateFilterData();
			}
			this._init = false;
		});
	},
	
	destroy: function() {
		if (this._filterWidget) {
			this._filterWidget.destroy();
			delete this._filterWidget;
		}
		
		this.inherited(arguments);
	},
	
	createFilter: function(targetType) {
		if (dojo.indexOf(QueryConstants.BOOLEAN_DATATYPES, targetType) !== -1) {
			return new rmps.ui.queryeditor.filters.BooleanFilterWidget({
				changeCallback: dojo.hitch(this, this._filterChangeCallback),
				parentWidget: this
			});			
		} else if (dojo.indexOf(QueryConstants.INTEGER_DATATYPES, targetType) !== -1) {
			return new rmps.ui.queryeditor.filters.IntegerFilterWidget({
				changeCallback: dojo.hitch(this, this._filterChangeCallback),
				parentWidget: this
			});
		} else if (dojo.indexOf(QueryConstants.STRING_DATATYPES, targetType) !== -1) {
			return new rmps.ui.queryeditor.filters.StringFilterWidget({
				changeCallback: dojo.hitch(this, this._filterChangeCallback),
				parentWidget: this
			});
		} else if (QueryConstants.DATE_DATATYPE === targetType) {
			return new rmps.ui.queryeditor.filters.DateFilterWidget({
				changeCallback: dojo.hitch(this, this._filterChangeCallback),
				parentWidget: this
			});
		}			
		
		// FIXME: Report that either data type is not supported, range is not or value is not
		console.error(targetType + " is not a supported value type.");
		return null;
	},
	
	updateFilterData: function() {
		var conditionData = this.getConditionData();
		if (conditionData) {
			if (this._queryConditionName === QueryConstants.PREDICATE_VALUE_CONDITION_TYPE) {
				var value = conditionData[QueryConstants.PREDICATE_VALUE_CONDITION_VALUE_PROPERTY];
				if (value && dojo.isObject(value)) {
					value = value[QueryConstants.JSON_VALUE_FIELD];					
				}
				if (value) {
					this._filterWidget.setValue(value);	
				}
				var sign = conditionData[QueryConstants.PREDICATE_VALUE_CONDITION_SIGN_PROPERTY];
				if (sign) {
					this._filterWidget.setSign(sign);
				}
				this.onMenuChange();
			}
		}
	},
	
	getMenuItems: function() {
		if (this._valueType === QueryConstants.STRING_DATATYPE) {
			this._menuItemsValues = [{
				id: QueryConstants.PREDICATE_VALUE_CONDITION_SIGN_EQUAL,
				label: messages.labelMatch
			}];
		} else {
		// Return the different alternatives for values
		var isDate = this._valueType === QueryConstants.DATE_DATATYPE;
			this._menuItemsValues = [{
				id: QueryConstants.PREDICATE_VALUE_CONDITION_SIGN_EQUAL,
				label: isDate ? messages.labelIsDate : messages.labelEqual		
			}, {
				id: QueryConstants.PREDICATE_VALUE_CONDITION_SIGN_LESS,
				label: isDate ? messages.labelBeforeDate : messages.labelLess
			}, {
				id: QueryConstants.PREDICATE_VALUE_CONDITION_SIGN_LESS_OR_EQUAL,
				label: isDate ? messages.labelBeforeIncludingDate : messages.labelLessOrEqual
			}, {
				id: QueryConstants.PREDICATE_VALUE_CONDITION_SIGN_MORE,
				label: isDate ? messages.labelAfterDate : messages.labelMore
			}, {
				id: QueryConstants.PREDICATE_VALUE_CONDITION_SIGN_MORE_OR_EQUAL,
				label: isDate ? messages.labelAfterIncludingDate : messages.labelMoreOrEqual
			}];
		}
		return this._menuItemsValues; 
	},
	
	getMenuSelection: function() {
		var sign = this._filterWidget ? this._filterWidget.getSign() : QueryConstants.PREDICATE_VALUE_CONDITION_SIGN_EQUAL;
		var items = dojo.filter(this._menuItemsValues, function(item) {
			return item.id === sign; 
		});
		return items[0];
	},
	
	
	getMenuTitle: function() {
		if (this._valueType === QueryConstants.STRING_DATATYPE) {
			return messages.tooltipStringMenu;
		} else {
			return "";
		}
	},
	
	onMenuAction: function(item) {
		if (this._filterWidget) {
			this._filterWidget.setSign(item.id);
		}
	},
	
	_onSelectionChange: function() {
		this.inherited(arguments);
		
		// Get selection
		var selection = this.getSelectedValues();
		if (selection.length !== 1) {
			return;			
		}

		var value = selection[0];
		var targetType = value.targetType;
		if (this._valueType === targetType) {
			return;
		}
		this._valueType = targetType;
		
		// Destroy old value filter widget
		if (this._filterWidget) {
			this._filterWidget.destroy();
			delete this._filterWidget;
		}
		
		// Create value filter widget
		this._filterWidget = this.createFilter(targetType);
		if (this._filterWidget) {
			this._contentArea.appendChild(this._filterWidget.domNode);
		}
		
		this.onMenuChange();
	},
	
	onMenuChange: function () {
		// empty function, used as callback for section widget
	},
	
	_filterChangeCallback: function() {
		// TODO: Support removeDirty as well
		if (!this._init) {
			this.mainWidget.onMarkDirty("valueChange");
		}
	},
	
	serialize: function() {
		var obj = this.inherited(arguments);
		
		var condition = obj[this._queryConditionName];
		
		if (this._filterWidget && this._queryConditionName === QueryConstants.PREDICATE_VALUE_CONDITION_TYPE) {
			// ValueCondition
			// Data type must be saved as well, since the value property should point to the concrete type.
			var value = condition[QueryConstants.PREDICATE_VALUE_CONDITION_VALUE_PROPERTY] = {};
			value[QueryConstants.JSON_DATATYPE_FIELD] = this._filterWidget.getDataType();
			value[QueryConstants.JSON_VALUE_FIELD] = this._filterWidget.getValue();			
			
			condition[QueryConstants.PREDICATE_VALUE_CONDITION_SIGN_PROPERTY] = this._filterWidget.getSign();
		}
		
		condition[QueryConstants.CONDITION_IGNORE_CONTEXT_PROPERTY] = false;
		
		return obj;
	},
	
	validate: function(rootTab) {
		var valResult = this.inherited(arguments); 
		
		if (!this._filterWidget) {
			valResult.errors.push(messages.valNoAssociatedValueFilter);
		} else {
			this._filterWidget.validate(valResult);
		}
		
		return valResult;
	}
	
});
}());