/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.OntologyPropertiesWidget");

dojo.require("dojo.i18n");
dojo.require("rmps.ui.queryeditor.ListWidget");
dojo.require("rmps.ui.queryeditor.QueryConstants");

dojo.require("rmps.ui.editor.EditorUtil");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");

dojo.requireLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

(function(){

var messages = dojo.i18n.getLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");
var QueryConstants = rmps.ui.queryeditor.QueryConstants;
var INCOMING_ID = "incoming";
var INCOMING_EXACT_ID = "incomingExact";
var OUTGOING_ID = "outgoing";
var OUTGOING_EXACT_ID = "outgoingExact";

dojo.declare("rmps.ui.queryeditor.OntologyPropertiesWidget", rmps.ui.queryeditor.ListWidget, {

	_queryConditionName: QueryConstants.PREDICATE_CONDITION_TYPE, 
	_queryConditionListProperty: QueryConstants.PREDICATE_CONDITION_EXACT_PREDICATE_LIST_PROPERTY,
	
	_predicateEnd: QueryConstants.PREDICATE_CONDITION_PREDICATE_END_OBJECT,
	predecessorWidget: null,
	
	_itemsName: messages.properties,
	
	_menuItems: [{
		id: OUTGOING_EXACT_ID,
		label: messages.labelOutgoingExact
	}, {
		id: OUTGOING_ID,
		label: messages.labelOutgoing 
	}, {
		id: INCOMING_EXACT_ID,
		label: messages.labelIncomingExact 
	}, {
		id: INCOMING_ID,
		label: messages.labelIncoming
	}],
	
	
	initializeFromConditionData: function () {
		// Make sure the list property is correct
		var conditionData = this.getConditionData();
		if (conditionData) {
			if (QueryConstants.PREDICATE_CONDITION_PREDICATE_LIST_PROPERTY in conditionData) {
				this._queryConditionListProperty = QueryConstants.PREDICATE_CONDITION_PREDICATE_LIST_PROPERTY;
			}
			if (QueryConstants.PREDICATE_CONDITION_EXACT_PREDICATE_LIST_PROPERTY in conditionData) {
				this._queryConditionListProperty = QueryConstants.PREDICATE_CONDITION_EXACT_PREDICATE_LIST_PROPERTY;
			}
			if (conditionData[QueryConstants.PREDICATE_CONDITION_PREDICATE_END_PROPERTY] === QueryConstants.PREDICATE_CONDITION_PREDICATE_END_SUBJECT) {
				this._predicateEnd = QueryConstants.PREDICATE_CONDITION_PREDICATE_END_SUBJECT; 
			}
		}
		
		this.setMultiple(this._queryConditionName === QueryConstants.PREDICATE_CONDITION_TYPE);
		
	},
	
	postCreate: function() {
		this.initializeFromConditionData();
		this.inherited(arguments);
	},
	
	getWidgetDataOptions: function() {
		var dataOptions = {
			predicateEnd: this._predicateEnd,
			selection: this._selection, 
			localAndDependantOntologies: this.isViewpointEditor() 
		};
		
		if (this.predecessorWidget !== null) {
			var values = this.predecessorWidget.getSelectedValues();
			if (values.length > 0) {
				dataOptions.typeURI = values[0].targetType;
			}
		}
		
		return dataOptions;
	},
		
	getMenuItems: function() {
		// Return the different conditions
		return this._menuItems; 
	},
	
	_getMenuSelection: function() {
		if (this._predicateEnd === QueryConstants.PREDICATE_CONDITION_PREDICATE_END_OBJECT) {
			// Outgoing
			if (this._queryConditionListProperty === QueryConstants.PREDICATE_CONDITION_EXACT_PREDICATE_LIST_PROPERTY) {
				// Exact
				return this._menuItems[0];
			} else {
				return this._menuItems[1];
			}
		} else {
			// Incoming
			if (this._queryConditionListProperty === QueryConstants.PREDICATE_CONDITION_EXACT_PREDICATE_LIST_PROPERTY) {
				// Exact
				return this._menuItems[2];
			} else {
				return this._menuItems[3];
			}
		}		
	},
	
	getMenuSelection: function() {
		return this._getMenuSelection();
	},
	
	_onMenuAction: function(item) {
		var id = item.id;
		var oldEnd = this._predicateEnd;
		if (id === OUTGOING_ID) {
			this._queryConditionListProperty = QueryConstants.PREDICATE_CONDITION_PREDICATE_LIST_PROPERTY;
			this._predicateEnd = QueryConstants.PREDICATE_CONDITION_PREDICATE_END_OBJECT;
		} else if (id === OUTGOING_EXACT_ID) {
			this._queryConditionListProperty = QueryConstants.PREDICATE_CONDITION_EXACT_PREDICATE_LIST_PROPERTY;
			this._predicateEnd = QueryConstants.PREDICATE_CONDITION_PREDICATE_END_OBJECT;
		} else if (id === INCOMING_ID) {
			this._queryConditionListProperty = QueryConstants.PREDICATE_CONDITION_PREDICATE_LIST_PROPERTY;
			this._predicateEnd = QueryConstants.PREDICATE_CONDITION_PREDICATE_END_SUBJECT;
		} else if (id === INCOMING_EXACT_ID) {
			this._queryConditionListProperty = QueryConstants.PREDICATE_CONDITION_EXACT_PREDICATE_LIST_PROPERTY;
			this._predicateEnd = QueryConstants.PREDICATE_CONDITION_PREDICATE_END_SUBJECT;
		}
		if (oldEnd !== this._predicateEnd) {
			this.refreshWidgetData();
		}
	},
	
	onMenuAction: function(item) {
		this._onMenuAction(item);
	},
	
	serialize: function() {
		var obj = this.inherited(arguments);
		
		var condition = obj[this._queryConditionName];
		condition[QueryConstants.PREDICATE_CONDITION_PREDICATE_END_PROPERTY] = this._predicateEnd;
		
		return obj;
	},
	
	validate: function(rootTab) {
		return this.inherited(arguments); 
	}
	
});
}());