/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011,2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.OperatorWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("rmps.ui.queryeditor.IQueryWidget");
dojo.require("rmps.ui.queryeditor.QueryWidgetChooser");
dojo.require("rmps.ui.queryeditor.QueryWidgetRegistry");
dojo.require("rmps.ui.queryeditor.ContainerSection");
dojo.require("rmps.ui.queryeditor.QueryConstants");
dojo.require("rmps.ui.queryeditor.CompositeWidgetsContainer");

dojo.requireLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

(function(){

var messages = dojo.i18n.getLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");
var QueryWidgetRegistry = rmps.ui.queryeditor.QueryWidgetRegistry;
var QueryConstants = rmps.ui.queryeditor.QueryConstants;
	
dojo.declare("rmps.ui.queryeditor.OperatorWidget", [rmps.ui.queryeditor.CompositeWidgetsContainer, rmps.ui.queryeditor.IQueryWidget], {

	_addConditionWidget: null,
	_menuItems: [{
		id: QueryConstants.LOGICAL_CONDITION_OPERATOR_AND_VALUE,
		label: messages.labelAND
	}, {
		id: QueryConstants.LOGICAL_CONDITION_OPERATOR_OR_VALUE,
		label: messages.labelOR 
	}, {
		id: QueryConstants.LOGICAL_CONDITION_OPERATOR_NOT_VALUE,
		label: messages.labelNOT
	}],
	
	postCreate:function() {
		var currentOperator = QueryConstants.LOGICAL_CONDITION_OPERATOR_AND_VALUE;
		var conditionData = this.getConditionData();
		var empty = true;
		if (conditionData) {
			var subconditions = conditionData[QueryConstants.LOGICAL_CONDITION_SUBCONDITION_PROPERTY];
			if (subconditions && subconditions.length) {
				dojo.forEach(subconditions, function(subcondition) {
					for (var condtype in subcondition) {
						var subconditionData = subcondition[condtype];
						var widgetid = subconditionData[QueryConstants.CONDITION_PRESENTATION_PROPERTY];
						// TODO: Handle missing presentation
						this.createWidget(widgetid, subconditionData);
						empty = false;
					}
				}, this);
			}
			currentOperator = conditionData[QueryConstants.LOGICAL_CONDITION_OPERATOR_PROPERTY];
		}
		
		this.inherited(arguments);

		this._setOperator(currentOperator);
	},
	
	getTitle: function() {
		// Overridden to use the operator instead
		var operator = this._currentOperator;
		if (operator === QueryConstants.LOGICAL_CONDITION_OPERATOR_AND_VALUE) {
			return messages.AND;
		}
		if (operator === QueryConstants.LOGICAL_CONDITION_OPERATOR_OR_VALUE) {
			return messages.OR;
		}
		if (operator === QueryConstants.LOGICAL_CONDITION_OPERATOR_NOT_VALUE) {
			return messages.NOT;
		}
		
		this.inherited(arguments);
	},
	
	getMenuItems: function() {
		// Return the different conditions
		return this._menuItems; 
	},
	
	getMenuSelection: function() {
		var operator = this._currentOperator;
		if (operator === QueryConstants.LOGICAL_CONDITION_OPERATOR_AND_VALUE) {
			return this._menuItems[0];
		}
		if (operator === QueryConstants.LOGICAL_CONDITION_OPERATOR_OR_VALUE) {
			return this._menuItems[1];
		}
		if (operator === QueryConstants.LOGICAL_CONDITION_OPERATOR_NOT_VALUE) {
			return this._menuItems[2];
		}
	},
	
	onMenuAction: function(item) {
		this._setOperator(item.id);
	},
	
	_setOperator: function (id) {
		this._currentOperator = id;
		this._queryChooser.updateAddGroupButton(id);
	},
		
	onClose: function() {
		this.inherited(arguments);
		this.destroy();
		return true;
	},
	
	createWidget: function(identifier, conditionData) {
		this._hideMessage();
		var widget_def = QueryWidgetRegistry.get(identifier);
		if (widget_def && widget_def.create) {
			var self = this;
			var def = widget_def.create({
				mainWidget: this.mainWidget,
				conditionData: conditionData
			});
			def.addCallback(function(widget) {
				if (widget) {
					self.createSection(widget);
				}				
			});
		}
	},

	preSerialize: function() {
		var defs = [];
		dojo.forEach(this._allWidgets(), function (w) {
			var def = w.sectionWidget.preSerialize(); 
			if (def && def instanceof dojo.Deferred) {
				defs.push(def);
			}
		});
		if (defs.length === 0) {
			return null; // No need for a deferred.
		}
		return new dojo.DeferredList(defs);
	},
	
	serialize: function() {
		var obj = {};
		var condition = obj[QueryConstants.LOGICAL_CONDITION_TYPE] = {};
		condition[QueryConstants.LOGICAL_CONDITION_SUBCONDITION_PROPERTY] = 
			dojo.map(this._allWidgets(), function(w) {
				return w.sectionWidget.serialize();
			}); 
		condition[QueryConstants.LOGICAL_CONDITION_OPERATOR_PROPERTY] = this._currentOperator;
		condition[QueryConstants.CONDITION_PRESENTATION_PROPERTY] = this.getIdentifier();
		return obj;
	},
	
	validate: function(rootTab) {
		var valResult = {
			hasNonContainerPortlet: false,
			errors: []
		};

		if (!this._isEmpty()) {
			dojo.forEach(this._allWidgets(), function (w) {
				if (valResult.errors.length === 0) {
					var secResult = w.sectionWidget.validate(rootTab);
					valResult.hasNonContainerPortlet = valResult.hasNonContainerPortlet || secResult.hasNonContainerPortlet;
					valResult.errors = valResult.errors.concat(secResult.errors);
				}
			});
		} else {
			var str = dojo.string.substitute(messages.valEmptyDataNotAllowed, [QueryConstants.LOGICAL_CONDITION_TYPE, QueryConstants.LOGICAL_CONDITION_SUBCONDITION_PROPERTY]);
			valResult.errors.push(str);
			this._showWarning(str);
		}

		return valResult;
	},
	
	_showWarning: function (message) {
		while (this._warningMessage.hasChildNodes()) {
			this._warningMessage.removeChild(this._warningMessage.childNodes[0]);
		}

		if (message) {
			this._warningMessage.appendChild(document.createTextNode(message));
			this._warningMessage.style.display = 'inline-block';
		} else {
			this._warningMessage.style.display = 'none';
		}
	},
	
	_hideMessage: function() {
		this._showWarning(false);
	}

});
}());