/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.QueryEditorWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.DeferredList");
dojo.require("jazz.app.context");
dojo.require("rmps.core.querymodel.QueryModel");
dojo.require("rmps.core.querymodel.QueryModelClient");
dojo.require("rmps.ui.queryeditor.ContainerSection");
dojo.require("rmps.ui.queryeditor.QueryWidgetRegistry");
dojo.require("rmps.ui.queryeditor.WidgetDataStore");
dojo.require("rmps.ui.queryeditor.QueryConstants");
dojo.require("rmps.ui.queryeditor.SortingGroupingWidget");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

(function(){

var messages = dojo.i18n.getLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");
	
var QueryWidgetRegistry = rmps.ui.queryeditor.QueryWidgetRegistry;
var QueryModelClient = rmps.core.querymodel.QueryModelClient;
var QueryConstants = rmps.ui.queryeditor.QueryConstants;

dojo.declare("rmps.ui.queryeditor.QueryEditorWidget", [dijit._Widget, dijit._Templated], {
	templatePath: dojo.moduleUrl("rmps.ui", "queryeditor/templates/QueryEditorWidget.html"),

	// mix-in
	onDirty: null,
	queryUri: "",
	inputData: {},
	canDeleteTopSection: false,
	
	_queryModel: null,
	_topSection: null,
	_isDirty: false,
	
	widgetDataStore: null,
	
	postCreate: function() {
		var self = this, def;
		
		// Make sure project ID is in inputData if not already
		if (!this.inputData.projectId) {
			this.inputData.projectId = jazz.app.context.get().itemId; 
		}
		
		this.widgetDataStore = new rmps.ui.queryeditor.WidgetDataStore({
			inputData: this.inputData
		});
		
		if (this.queryUri && this.queryUri != "") {
			def = QueryModelClient.getQueryModel(this.queryUri);
			// TODO: Add loading icon
			def.addCallback(function(querymodel) {
				self._queryModel = querymodel;
				var conditionData = querymodel.getDataObject();
				var defs = []
				for (var conditionType in conditionData) {
					var condition = conditionData[conditionType];
					var widgetid = condition[QueryConstants.CONDITION_PRESENTATION_PROPERTY];
					// TODO: Handle missing presentation
					defs.push(self.createWidget(widgetid, condition));					
				}
				var allDone = new dojo.DeferredList(defs);
				allDone.addCallback(function () {
					self._sortingSection = self.createSortingAndGroupingWidget(querymodel);
				});
			});
		} else {
			// Start out with an AND operator widget
			var operatorConditionData = {};
			operatorConditionData[QueryConstants.LOGICAL_CONDITION_OPERATOR_PROPERTY] = QueryConstants.LOGICAL_CONDITION_OPERATOR_AND_VALUE;
			def = this.createWidget(QueryConstants.LOGICAL_WIDGET_ID, operatorConditionData);
			def.addCallback(dojo.hitch(this, function() {
				this._sortingSection = this.createSortingAndGroupingWidget();
			}));

		}
		

		
		this.inherited(arguments);
	},
	
	destroy: function() {
		if (this._topSection) {
			this._topSection.destroy();
			delete this._topSection;
		}
		
		if (this.widgetDataStore) {
			delete this.widgetDataStore;
		}
		this.inherited(arguments);
	},
	
	createWidget: function(widgetid, conditionData) {
		var def = QueryWidgetRegistry.get(widgetid).create({
			mainWidget: this,
			conditionData: conditionData
		});
		// TODO: Add loading icon
		var self = this;
		def.addCallback(function(widget) {
			self._topSection = self.addSectionForWidget(widget, self.CanDeleteTopSection);			
		});	
		return def;
	},
	
	preSerialize: function() {
		var topWidget = this._topSection.sectionWidget;
		var sortWidget = this._sortingSection ? this._sortingSection.sectionWidget : null;
		
		var defTopPreSerialize = topWidget.preSerialize();
		var defSortPreSerialize = sortWidget ? sortWidget.preSerialize() : null;

		if (defTopPreSerialize) {
			if (defSortPreSerialize) {
				return new dojo.DeferredList([defTopPreSerialize, defSortPreSerialize]);
			} else {
				return defTopPreSerialize;
			}
		} else if (defSortPreSerialize) {
			return defSortPreSerialize;
		}
		
		// No-op
		var def = new dojo.Deferred();
		def.callback();
		return def;
	},
	
	serialize: function() {
		var res = {};
		res.query = this._topSection.sectionWidget.serialize();		
		if (this._sortingSection) {
			dojo.mixin(res, this._sortingSection.sectionWidget.serialize());
		}
		return res;		
	},
	
	getQueryData: function() {
		var def = new dojo.Deferred();
		
		var defPreSerialize = this.preSerialize();
		defPreSerialize.addCallback(this, function() {
			def.callback(this.serialize());
		});
		defPreSerialize.addErrback(def.errback);	
		
		return def;
	},
	
	getQuery: function() {
		return this._queryModel;
	},
	
	isDirty: function() {
		return this._isDirty;
	},
	
	onMarkDirty: function(key) {
		// Set this particular widget as dirty
		this._isDirty = true;

		if (this.onDirty) {
			this.onDirty(key);
		}
	},
	
	saveQueryData: function(synchronous, queryModel) {
		var def = new dojo.Deferred();
		
		if (!this._isDirty) {
			def.errback();
			return def;
		}
		
		var projectId = jazz.app.context.get().itemId;
		
		var defData = this.getQueryData();
		defData.addCallback(this, function(data) {
			var defQuery = null;
			if (!this._queryModel) {
				if (queryModel) {
					this._queryModel = queryModel;
					this._queryModel.setDataObject(data.query);
				} else {
					this._queryModel = new rmps.core.querymodel.QueryModel({
						dataObject: data.query
					}); 
				}
				if (data[QueryConstants.QUERY_SORTING_PROPERTY]) {
					this._queryModel.setSorting(data[QueryConstants.QUERY_SORTING_PROPERTY]);
				}
				if (data[QueryConstants.QUERY_GROUPING_PROPERTY]) {
					this._queryModel.setGrouping(data[QueryConstants.QUERY_GROUPING_PROPERTY]);
				}
				defQuery = QueryModelClient.createQueryModel(this._queryModel, projectId, synchronous);
			} else {
				if (queryModel) {
					dojo.mixin(this._queryModel, queryModel);
				}
				this._queryModel.setDataObject(data.query);
				if (data[QueryConstants.QUERY_SORTING_PROPERTY]) {
					this._queryModel.setSorting(data[QueryConstants.QUERY_SORTING_PROPERTY]);
				}
				if (data[QueryConstants.QUERY_GROUPING_PROPERTY]) {
					this._queryModel.setGrouping(data[QueryConstants.QUERY_GROUPING_PROPERTY]);
				}
				defQuery = QueryModelClient.updateQueryModel(this._queryModel, projectId, synchronous);
			}
			defQuery.addCallback(this, function(querymodel) {
				def.callback(querymodel);
				this._isDirty = false;
			});
			defQuery.addErrback(def.errback);
		});
		defData.addErrback(def.errback);
		return def;
	},
	
	validate: function(rootTab) {
		var def = new dojo.Deferred();
		
		// Validate depends on serialization, so check preSerialize first
		var defPreSerialize = this.preSerialize();
		defPreSerialize.addCallback(this, function() {
			var valResult = this._topSection.sectionWidget.validate(rootTab);
			if (!valResult.hasNonContainerPortlet) {
				valResult.errors.push(messages.valAtLeastOneNonContainer);
			}
			def.callback(valResult.errors);
		});
		defPreSerialize.addErrback(def.errback);
		
		return def;
	},
	
	resize: function(size) {
		this.inherited(arguments);
		
		if (size) {
			if (size.h) {
				var h = size.h - 14 /*padding*/;
				dojo.style(this.domNode, {
					height: h + "px"
				});
			}
			if (size.w) {
				var w = size.w - 24 /*padding*/;
				dojo.style(this.domNode, {
					width: w + "px"
				});				
			}
		}
	},
	
	addSectionForWidget: function(widget, canDelete) {
		var section = new rmps.ui.queryeditor.ContainerSection({
			sectionWidget: widget,
			canDelete: canDelete
		});
		
		this._sections.appendChild(section.domNode);
		return section; 
		
	},
	
	createSortingAndGroupingWidget: function(queryModel) {
		var widget = new rmps.ui.queryeditor.SortingGroupingWidget({
			mainWidget: this,
			conditionData: queryModel,
			hideGrouping: this.hideGrouping
		});
		return this.addSectionForWidget(widget);
		
	}
	
});
	
}());