/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmps.ui.queryeditor.QueryWidgetChooser");

dojo.require("dijit.Tree");
dojo.require("rmps.ui.queryeditor.QueryWidgetTreeModel");
dojo.require("rmps.ui.queryeditor.QueryConstants");
dojo.require("dojo.i18n");

(function () {
	var messages = dojo.i18n.getLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");
	var widgetRegistry = rmps.ui.queryeditor.QueryWidgetRegistry;
	var QueryConstants = rmps.ui.queryeditor.QueryConstants;
	
	dojo.declare ("rmps.ui.queryeditor.QueryWidgetTree", [dijit.Tree], {
		getLabelClass: function (item, opened) {
			return "query-" + item.kind + "-label";
		}
	});
	
	dojo.declare ("rmps.ui.queryeditor.QueryWidgetChooser", [dijit._Widget, dijit._Templated], {
		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl("rmps.ui", "queryeditor/templates/QueryWidgetChooser.html"),
		
		_expanded: false,
		_inEmptySection: true,
		
		_addImg: dojo.moduleUrl("rmps.ui", "graphics/queryWidget/add-condition.gif"),
		_closeImg: dojo.moduleUrl("rmps.ui", "graphics/queryWidget/close.gif"),
		_closeHoverImg: dojo.moduleUrl("rmps.ui", "graphics/queryWidget/close-hov.gif"),
		_showOptionalPart: true,
		_messages: messages,
		// Widget template parser does not handle variable substitution when string contain single quotes (like in French strings) so escape them
		_addConditionText: messages.addCondition.replace(/'/g, "\\'"),
		_addConditionLabelText: messages.addConditionLabel.replace(/'/g, "\\'"),
		_addPropChainLabelText: messages.addPropChainLabel.replace(/'/g, "\\'"),
		
		constructor: function (args) {
			_treeModel = new rmps.ui.queryeditor.QueryWidgetTreeModel(args);
			this._showOptionalPart = args.showOptionalPart === undefined || args.showOptionalPart;
			this._inEmptySection = args.inEmptySection === undefined || args.inEmptySection;
		},
		
		postCreate: function () {
			this._addConditionButton.onClick = dojo.hitch(this, this.onAddCondition);
			this._addConditionButtonInHelp.onClick = dojo.hitch(this, this.onAddCondition);
			
			this._updateWidgetVisibility();
			this.updateAddWidgetButton();
			
			this._tree.dndController.singular = true;
			
			if (!this._showOptionalPart) {
				dojo.style(this._optionalPart, {display: "none"});
			}
		},
		
		///////////////////////////////////////////
		// 
		
		_treeModel: null,
		
		onAddCondition: function () {
			this._expanded = true;
			this._updateWidgetVisibility();
		},
		
		onAddWidget: function () {
			this.parentWidget.onWidgetAddition(this._tree.selectedItem.identifier);
		},
		
		onAddPropChain: function () {
			this.parentWidget.onWidgetAddition(QueryConstants.PREDICATE_CHAIN_WIDGET_ID);
		},
		
		onAddGroup: function () {
			var params = {};
			params[QueryConstants.LOGICAL_CONDITION_OPERATOR_PROPERTY] = this._groupId;
			this.parentWidget.onWidgetAddition(QueryConstants.LOGICAL_WIDGET_ID, params);
		},
		
		onWidgetAddition: function (widget, widgets) {
			this._inEmptySection = widgets.length === 0;
			this._updateWidgetVisibility();
		},
		
		onWidgetClose: function (widget, widgets) {
			var exists = false;
			dojo.forEach(widgets, function (w) {
				exists = exists || w !== widget;
			});
			this._inEmptySection = !exists;
			this._updateWidgetVisibility();
		},
		
		updateAddWidgetButton: function () {
			this._addAttrButton.set('disabled', (this._tree.selectedItem === null || this._tree.selectedItem.kind !== "widget"));
		},
		
		updateAddGroupButton: function (parentGroupId) {
			var addGroupLabel;
			if (parentGroupId === QueryConstants.LOGICAL_CONDITION_OPERATOR_AND_VALUE) {
				this._groupId = QueryConstants.LOGICAL_CONDITION_OPERATOR_OR_VALUE;
				addGroupLabel = messages.addGroupLabel1 + messages.OR + messages.addGroupLabel2;
			} else {
				this._groupId = QueryConstants.LOGICAL_CONDITION_OPERATOR_AND_VALUE;
				addGroupLabel = messages.addGroupLabel1 + messages.AND + messages.addGroupLabel2;
			}
			this._addGroupButton.set('label', addGroupLabel);
		},
		
		onClose: function (e) {
			if (e) {
				dojo.stopEvent(e);
			}
			this._expanded = false;
			this._updateWidgetVisibility();
			return false;
		},
		
		_updateWidgetVisibility: function () {
			this.setVisible(this._attrChooser, this._expanded);
			this.setVisible(this._addButtonContainer, !this._expanded && !this._inEmptySection);
			this.setVisible(this._actionHelp, !this._expanded && this._inEmptySection);
		},
		
		setVisible: function (node, visible) {
			if (visible) {
				dojo.style(node, {display: ''});
			} else {
				dojo.style(node, {display: 'none'});
			}
		},
		
		checkEnterForClose: function () {
			
		},
		
		onCloseMouseOver: function () {
			this._closeImage.src = this._closeHoverImg;
		},
		
		onCloseMouseOut: function () {
			this._closeImage.src = this._closeImg;
		}
				
	});
}) ();