/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.QueryWidgetRegistry");

dojo.require("jazz.core.RegistryFactory");
dojo.require("jazz.core.loader");

(function(){

var EXT_PT = "com.ibm.xtools.rmps.web.ui.queryEditorWidgets";
var E_WIDGET = "widget";
var E_CATEGORY = "category";
var A_JS_MODULE = "jsModule";
var A_NAME = "name";
var A_IDENTIFIER = "identifier";
var A_DESCRIPTION = "description";
var A_CONTAINER = "isContainer";
var A_CATEGORY = "category";

var widgets = {};

var extensions = {};
	
//
// Reads the extension. Widgets will be cached
//
var init = function() {
	dojo.forEach(jazz.core.RegistryFactory.getRegistry().getExtensionPoint(EXT_PT).getExtensions(), function(extension) {
		dojo.forEach(extension.getConfigurationElements(), function(config) {
			if (E_WIDGET === config.getName()) {
				var className = config.getAttribute(A_JS_MODULE);
				var name = config.getAttribute(A_NAME);
				var description = config.getAttribute(A_DESCRIPTION);
				var identifier = config.getAttribute(A_IDENTIFIER);
				var category = config.getAttribute(A_CATEGORY);
				var isContainer = config.getAttribute(A_CONTAINER).toLowerCase() === "true";
				
				var widget = {
					kind: E_WIDGET,
						
					loaded: false,
					load: function() {
						var def = new dojo.Deferred();
						var self = this;
						jazz.core.loader.load_async(this.className, function() {
							self.loaded = true;
							def.callback();
						});
						return def;
					},
					construct: function(args) {
						var classConstructor = dojo.getObject(this.className);
						if (classConstructor) {
							args = args || {};
							dojo.mixin(args, {
								name: this.name,
								description: this.description,
								identifier: this.identifier,
								container: this.isContainer
							});
							return new classConstructor(args);
						}
						return null;
					},
					create: function(args){
						var def = new dojo.Deferred();
						if (!this.loaded) {
							var self = this;
							this.load().addCallback(function() {
								def.callback(self.construct(args));
							});
						} else {
							def.callback(this.construct(args));
						}
						return def;
					},
					className: className,
					name: name,
					description: description,
					identifier: identifier,
					isContainer: isContainer,
					category: category
				};
				
				extensions[identifier] = widget;
			} else if (E_CATEGORY == config.getName()) {
				var identifier = config.getAttribute(A_IDENTIFIER);
				var name = config.getAttribute(A_NAME);
				var description = config.getAttribute(A_DESCRIPTION);
				
				var category = {
					kind: E_CATEGORY,
					
					identifier: identifier,
					name: name,
					description: description
				};
				
				extensions[identifier] = category;
			}
		});
	});
};
	
//Initialization
init();

rmps.ui.queryeditor.QueryWidgetRegistry = {	
	
	// Load for several widgets at once
	load: function(identifiers) {
		var def = new dojo.Deferred();
		// Since 'widgets' is an associative array, dojo.filter/map can't be used.
		var modules = [];
		var widgetsToLoad = dojo.filter(
								dojo.map(identifiers, function (id) { return extensions[id]; }),
								function (ext) { return ext.kind == E_WIDGET && !ext.isLoaded; });
		var modules = dojo.map(widgetsToLoad, function (widget) { return widget.className; });
		
		jazz.core.loader.batch_load_async(modules, function() {
			dojo.forEach(widgetsToLoad, function(widget) {
				widget.loaded = true;
			});
			def.callback();
		});
		return def;
	},
	
	// Get the registered widgets
	get: function(identifier) {
		return extensions[ identifier && identifier.__value ? identifier.__value : identifier];
	},
	
	getAll: function() {
		// 'widgets' is an associative array, return as list instead.
		var allWidgets = [];
		for (var i in extensions) {
			if (extensions[i].kind == E_WIDGET)
			allWidgets.push(extensions[i]);
		}
		return allWidgets;
	},
	
	getExtensions: function (filter) {
		if (filter == null)
			return extensions;
		
		var exts = [];
		
		for (var id in extensions) {
			if (filter(extensions[id])) {
				exts.push(extensions[id]);
			}
		}
		
		return exts;
	}
	
};
	
})();