/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmps.ui.queryeditor.QueryWidgetTreeModel");

dojo.require("rmps.ui.queryeditor.QueryWidgetRegistry");

(function () {

var registry = rmps.ui.queryeditor.QueryWidgetRegistry;

dojo.declare("rmps.ui.queryeditor.QueryWidgetTreeModel", null, {
	
	_er: {
		kind: "registry",
		identifier: "registry",
		name: "registry"
	},
	
	constructor: function (args) {
		this.useCategories = args.useCategories === undefined || args.useCategories;
		this.ignoreItems = args.ignoreItems || [];
		this.showItems = args.showItems || [];
	},
	
	destroy: function () {
		// nothing to do
	},
	
	getRoot: function (onItem) {
		onItem(this._er);
	},
	
	mayHaveChildren: function (item) {
		return item.kind === "registry" || item.kind === "category";
	},
	
	getChildren: function (parentItem, onComplete) {
		var extensions = this._er.extensions;
		var children = [];
		if (parentItem.kind === "registry") {
			children = children.concat(this._getCategories());
			children = children.concat(this._getWidgets(null));
		} else if (parentItem.kind === "category") {
			children = this._getWidgets(parentItem.identifier);
		}
		
		onComplete(children);
	},
	
	isItem: function (something) {
		return something.kind !== null;
	},
	
	fetchItemIdentity: function (keywordArgs) {
		// TODO
	},
	
	getIdentity: function (item) {
		return item.identifier;
	},
	
	getLabel: function (item) {
		return item.name;
	},
	
	newItem: function () {
		throw "not implemented";
	},
	
	pasteItem: function () {
		throw "not implemented";
	},
	
	onChange: function () {
		// nothing to do
	},
	
	onChildrenChange: function () {
		// nothing to do
	},
	
	////////////////////////////
	//
	
	_showItem: function (item) {
		return dojo.indexOf(this.ignoreItems, item.identifier) === -1 && (this.showItems.length === 0 || dojo.indexOf(this.showItems, item.identifier) !== -1);
	},
	
	_getWidgets: function (category) {
		return registry.getExtensions(dojo.hitch(this, function (ext) {
			return ext.kind === "widget" && (!this.useCategories || ext.category === category) && !ext.isContainer && this._showItem(ext);
		})).sort(function (a, b) { a = a.name; b = b.name; return a < b ? -1 : a > b ? 1 : 0;});
	},
	
	_getCategories: function () {
		if (!this.useCategories) { return []; }
		return registry.getExtensions(function (ext) {
			return ext.kind === "category";
		}).sort(function (a, b) { a = a.name; b = b.name; return a < b ? -1 : a > b ? 1 : 0;});
	}
	
});

}());