/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.SortingGroupingWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("rmps.ui.queryeditor.IQueryWidget");
dojo.require("rmps.ui.queryeditor.ContainerSection");
dojo.require("rmps.ui.queryeditor.QueryConstants");
dojo.require("rmps.ui.queryeditor.SortingPropertiesWidget");
dojo.require("rmps.ui.queryeditor.GroupingPropertiesWidget");

dojo.requireLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

(function(){

var messages = dojo.i18n.getLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");
var QueryConstants = rmps.ui.queryeditor.QueryConstants;
	
dojo.declare("rmps.ui.queryeditor.SortingGroupingWidget", [dijit._Widget, dijit._Templated, rmps.ui.queryeditor.IQueryWidget], {
	templatePath: dojo.moduleUrl("rmps.ui", "queryeditor/templates/SortingGroupingWidget.html"),

	postCreate:function() {
		var conditionData = this.getConditionData();
		var sortingData = null, groupingData = null;
		if (conditionData) {
			sortingData = conditionData[QueryConstants.QUERY_SORTING_PROPERTY];
			groupingData = conditionData;
		}
		this._sortingPropertyWidget = new rmps.ui.queryeditor.SortingPropertiesWidget({
			mainWidget: this.mainWidget,
			sectionWidget: this,
			identifier: "dm.query.ontology.predicate.sorting",
			conditionData: sortingData
		});
		
		var section = new rmps.ui.queryeditor.ContainerSection({
			sectionWidget: this._sortingPropertyWidget,
			canDelete: false
		});
		this.addSection(section, this._sortingPropertyWidget.isContainer());
		
		if (!this.hideGrouping) {
			this._groupingPropertyWidget = new rmps.ui.queryeditor.GroupingPropertiesWidget({
				mainWidget: this.mainWidget,
				sectionWidget: this,
				identifier: "dm.query.ontology.predicate",
				conditionData: groupingData
			});
			
			section = new rmps.ui.queryeditor.ContainerSection({
				sectionWidget: this._groupingPropertyWidget,
				canDelete: false
			});
			this.addSection(section, this._groupingPropertyWidget.isContainer());
		}
	},
	
	getTitle: function() {
		return this.hideGrouping ? messages.labelSortingOptions : messages.labelSortingAndGrouping;
	},
	
	
	addSection: function(section, /*Boolean*/top) {
		if (top) {
			this._sections.insertBefore(section.domNode, this._sections.firstChild);
		} else {
			this._sections.appendChild(section.domNode);			
		}
	},
	
	createSection: function(widget) {
		var section = new rmps.ui.queryeditor.ContainerSection({sectionWidget:widget});
		// Add the section at top if container
		this.addSection(section, widget.isContainer());
	},
	
	onClose: function() {
		this.destroy();
		return true;
	},
	
	onWidgetAddition: function(action) {
		this.createWidget(action.identifier, null);
		
		// Dirty operation
		this.mainWidget.onMarkDirty("addSection");
	},
	
	serialize: function() {
		var obj = {};
		if (this._sortingPropertyWidget) {
			obj[QueryConstants.QUERY_SORTING_PROPERTY] = this._sortingPropertyWidget.serialize();
		}
		if (this._groupingPropertyWidget) {
			obj[QueryConstants.QUERY_GROUPING_PROPERTY] = this._groupingPropertyWidget.serialize();
		}
		return obj;
	},

	validate: function(rootTab) {
		var valResult = {
			errors: []
		};	
		if (this._sections.hasChildNodes()) {
			dojo.forEach(this._sections.children, function(section) {
				if (valResult.errors.length === 0) {
					var w = dijit.getEnclosingWidget(section);
					if (w) {
						var secResult = w.sectionWidget.validate(rootTab);
						for (var i = 0; i < secResult.errors.length; i++) {
							valResult.errors.push(secResult.errors[i]);
						}
					}
				}
			});
		}
		return valResult;
	}
});
}());