/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.WidgetsContainer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("rmps.ui.queryeditor.QueryWidgetChooser");

dojo.declare("rmps.ui.queryeditor.WidgetsContainer", [dijit._Widget, dijit._Templated], {
	templatePath: dojo.moduleUrl("rmps.ui", "queryeditor/templates/WidgetsContainer.html"),
	
	constructor: function () {
		this._sectionsCount = 0;
	},
	
	postCreate: function () {
		this._queryChooser = new rmps.ui.queryeditor.QueryWidgetChooser(this._queryChooserOptions());
		this._addConditionNode.appendChild(this._queryChooser.domNode);
	},
	
	_queryChooserOptions: function () {
		return {
			mainWidget: this.mainWidget,
			parentWidget: this,
			inEmptySection: this._sectionsCount === 0
		};
	},

	
	addSection: function (section, isContainer) {
		var order = section._order = this._sectionsCount++;
		var sections = this._sections;
		var beforeSection = dojo.filter(sections.children, function (sec) { return sec._order > order; })[0];
		
		if (beforeSection) {
			sections.insertBefore(section.domNode, beforeSection);
		} else {
			sections.appendChild(section.domNode);
		}
	},
	
	createSection: function(widget) {
		var section = new rmps.ui.queryeditor.ContainerSection({sectionWidget: widget, parentWidget: this});
		
		// Add the section at top if container
		this.addSection(section, widget.isContainer());

		if (this._queryChooser) {
			this._queryChooser.onWidgetAddition(section, this._allWidgets());
		}
		
		dojo.connect(section, "onClose", this, function () {
			if (this._queryChooser) {
				this._queryChooser.onWidgetClose(widget, this._allWidgets());
			}
		});
		
		return section;
	},
	
	onWidgetAddition: function(action, args) {
		this.createWidget(action, args);
		
		// Dirty operation
		this.mainWidget.onMarkDirty("addSection");
	},
	
	_allSections: function () {
		var sections = [];
		dojo.forEach(this._sections.children, function (s) { sections.push(s); });
		return sections;
	},
	
	_allWidgets: function () {
		return dojo.filter(
				dojo.map(this._allSections(), function (s) {
					return dijit.getEnclosingWidget(s);
				}),
				function (e) {
					return e !== null;
				});
	},
	
	_isEmpty: function () {
		return this._allWidgets().length === 0;
	}
	
});
