/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.filters.DateFilterWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.DateTextBox");
dojo.require("rmps.ui.queryeditor.filters.IValueFilter");
dojo.require("rmps.ui.queryeditor.QueryConstants");
dojo.require("dojo.date.stamp");

dojo.requireLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

(function(){
	
var QueryConstants = rmps.ui.queryeditor.QueryConstants;
var RELATIVE_REGEXP = new RegExp("-?PT?\\d+[YMDH]");

	
dojo.declare("rmps.ui.queryeditor.filters.DateFilterWidget", [dijit._Widget, dijit._Templated, rmps.ui.queryeditor.filters.IValueFilter], {

	templatePath: dojo.moduleUrl("rmps.ui", "queryeditor/filters/templates/DateFilterWidget.html"),
	widgetsInTemplate: true,
	
	_dateTimeSelection: null,
	
	// mix-in
	future: false,
	dateOnly: false,
	
	///////////////////////////////////////////////////////////////////////////////////
	// API
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");
	},
	
	postCreate: function() {
		this._calendarImage.src = dojo.moduleUrl("rmps.ui", "graphics/queryWidget/calendar.gif");
		this._handleFutureVariables();
		this._connectListeners();
		this._hideUnusedControls();
	},
	
	getDataType: function() {
		if (this._isRelative(this.getValue())) {
			return QueryConstants.DURATION_DATATYPE;
		} else {
			return QueryConstants.DATE_DATATYPE;
		}
	},
	
	getValue: function() {
		var self = this;
		var value = null;
		dojo.query(".values input", this.domNode).forEach(function(input) {
			if (input.checked) {
				if ("custom" === input.value) {
					value = self._getCustomVariable();
				} else if ("absolute" === input.value) {
					var date = self._dateTextBox.getValue();
					if (date) {
						// Set to noon local
						date.setHours(12);
						date.setMinutes(0);		
						date.setSeconds(0);
						date.setMilliseconds(0);
						value = dojo.date.stamp.toISOString(date);
					}
				} else {
					value = input.value;
				}
			}
		});	
		return value;
	},
	
	setValue: function(value) {
		if (value === "P") {
			// 0D is omitted by the server
			value = "P0D";
		}
		// Check the value if it's a relative or absolute value
		if (this._isRelative(value))
		{
			// Relative custom value
			this._setRelative(value);
		} else {
			// Absolute value
			var date = dojo.date.stamp.fromISOString(value);
			this._dateTextBox.setValue(date);
			this._check(this._getAbsoluteRadio());
		}
	},
	
	validate: function(valResult) {
		dojo.query(".values input", this.domNode).forEach(dojo.hitch(this, function(input) {
			if (input.checked && "custom" === input.value) {
				var inputValue = this._timeUnitInput.value;
				var duration = inputValue ? parseInt(inputValue, 10) : Number.NaN;
				if (isNaN(duration)) {
					valResult.errors.push(this.messages.valNotAnInteger);
				} else if (duration <= 0) {
					valResult.errors.push(this.messages.valDurationPositiveInt);
				}
			}
		}));	
	},

			
	///////////////////////////////////////////////////////////////////////////////////
	// Internals
	
	_isRelative: function(value) {
		return RELATIVE_REGEXP.test(value);
	},
	
	_connectListeners: function() {
		var radios= dojo.query('input[type="radio"]', this.domNode);
		for (var i in radios) {
			this.connect(radios[i], "onclick", this._handleRadioChange);
		}
		this.connect(this._dateTextBox, "onChange", this._handleDateSelected);
	},
	
	_getCustomVariable: function() {
		var inputValue = this._timeUnitInput.value;
		var duration = inputValue ? parseInt(inputValue, 10) : Number.NaN;
		if (!isNaN(duration)) {
			var unit = this._timeUnitSelect.value;
			var args = this._createArgument(unit, duration);
			return args;
		}
		return null;
	},
		
	_setRelative: function(value) {
		var inputList = dojo.query('input[value="' + value + '"]', this.domNode);
		if (inputList.length > 0) {
			this._check(inputList[0]);
		} else {
			var timeUnit = this._parseArgument(value);
			this._timeUnitInput.value = timeUnit.duration;
			this._timeUnitSelect.value = timeUnit.unit;
			this._checkCustomVariableControl();
		}
	},
	
	_handleFutureVariables: function() {
		if (!this.future) {
			dojo.query("[future]", this.domNode).forEach(function(node) {
				node.parentNode.removeChild(node);
			});			
		}
	},
	
	_hideUnusedControls: function() {
		if (this.dateOnly) {
			dojo.query(".rmps-ui-queryeditor-filters-date-widget-hide", this.domNode).forEach(function(node) {
				node.style.display = "none";
			});
		}
	},
	
	_createArgument: function(timeUnit, duration) {
		var sign = '', prefix = '';
		var pureUnit = timeUnit;
		if (timeUnit.charAt(0) === '-') {
			sign = '-';
			pureUnit = timeUnit.substr(1);
		}
		if (pureUnit === "H") {
			prefix = "T";
		}
		
		if (duration < 0) {
			duration = -duration;
			sign = sign === '' ? '-' : '';
		}
		
		return sign + "P" + prefix + duration + pureUnit;
	},
	
	_parseArgument: function(arg) {
		var unit = "";
		var duration = "";
		var lastIndex = arg.length - 1;
		var firstIndex = 1;

		if (arg.charAt(0) === '-') {
			unit = "-";
			firstIndex = 2;
		}
		if (arg.charAt(firstIndex) === 'T') {
			firstIndex++;
		}

		duration = arg.substring(firstIndex, lastIndex);
		unit += arg.substring(lastIndex);
		
		return { duration: duration, unit: unit };
	},
	
	_findRadio: function(value) {
		var inputs = dojo.query('input[value="' + value + '"]', this.domNode);
		return inputs.length > 0 ? inputs[0] : null;
	},
	
	_getAbsoluteRadio: function() {
		return this._findRadio("absolute");
	},
	
	_checkCustomVariableControl: function() {
		var radio = this._findRadio("custom");
		this._check(radio);
	},
	
	_onSelectCalendar: function() {
		this._dateTextBox._open();
	},
	
	_onTimeInputFocus: function() {
		this._checkCustomVariableControl();
	},
	
	_handleRadioChange: function(event) {
		this._check(event.target);
		this._onChange();
	},
	
	_handleDateSelected: function() {
		var dateRadio = this._getAbsoluteRadio();
		var radios = dojo.query('input[type="radio"]', this.domNode);
		var otherChecked = dojo.some(radios, function(radio) {
			return radio === dateRadio ? false : radio.defaultChecked;
		});
		this._check(dateRadio);

		if (this._dateTimeSelection || otherChecked) {
			this._onChange();	
		}
		this._dateTimeSelection = this._dateTextBox.getValue().getTime();
	},
	
	_onTimeInputChange: function() {
		this._onChange();
	},
	
	_onTimeUnitChange: function() {
		this._checkCustomVariableControl();
		this._onChange();
	},
	
	_onChange: function() {
		if (this.changeCallback) {
			this.changeCallback();
		}
	},
	
	_check: function(input) {
		var radios = dojo.query('input[type="radio"]', this.domNode);
		for (var i in radios) {
			radios[i].defaultChecked = false;
		}
		input.checked = input.defaultChecked = true;
	}
});
	
}());

