/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.filters.ResourcesFilterWidget");

dojo.require("dijit._Widget");
dojo.require("rmps.app.pickers.internal.CLMResourcePicker");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.Dialog");
dojo.require("rmps.ui.queryeditor.filters.IResourcesFilter");
dojo.require("rmps.ui.queryeditor.QueryConstants");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

(function(){

var messages = dojo.i18n.getLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");
var QueryConstants = rmps.ui.queryeditor.QueryConstants;

dojo.declare("rmps.ui.queryeditor.filters.ResourcesFilterWidget", [dijit._Widget, dijit._Templated, rmps.ui.queryeditor.filters.IValueFilter], {
	templatePath: dojo.moduleUrl("rmps.ui", "queryeditor/filters/templates/ResourcesFilterWidget.html"),
	widgetsInTemplate: true,
	_resourcesBoxWidget: null,
	_inputName: messages.labelResourcesFilter,
	_label: messages.labelResourcesFilter,
	_addButtonLabel: messages.addResource,
	_removeButtonLabel: messages.removeResource,
	
	postCreate: function() {
		this._resourcesBoxWidget = new jazz.ui.ListBox({
			onClick: dojo.hitch(this, this._onSelectionChange),
			margin: "2px",
			height: "50px"
		});
		this._listBox.appendChild(this._resourcesBoxWidget.domNode);
		
		//this.refreshWidgetData();
		this._removeResourcesButton.set('disabled', true);
		
		this.inherited(arguments);
	},
	
	getResources: function() {
		if (this._resInfoDeferred) {
			return this._resources;
		} else {
			return dojo.map(this._resourcesBoxWidget.getAllEntries(), function (entry) {
				return entry.id;
			});
		}
	},

	setResources: function(resources) {
		this._resourcesBoxWidget.removeAllEntries();
		this._resourcesBoxWidget.showLoading(true);
		this._resInfoDeferred = rmps.core.resource.ResourceInfoClient.getResourceInfo(resources, ["uri","shortName","type"]);
		this._resources = resources;
		this._resInfoDeferred.addCallback(this, function(infos) {
			delete this._resInfoDeferred;
			delete this._resources;
			this._addResources(infos);
			this._resourcesBoxWidget.showLoading(false);
		});
	},
	
	validate: function(valResult) {
		var resources = this._resourcesBoxWidget.getAllEntries();
		if (resources.length === 0) {
			valResult.errors.push(messages.valNoResourceSelected);
			if (this.parentWidget && this.parentWidget._showWarning) {
				this.parentWidget._showWarning(messages.valNoResourceSelected);
			}
		}
 
		
//		if (!dojo.every(resources, function (resource) {
		//TODO: implement a check here, explicit type comparison is not correct, since resource.type can be a subtype of this.datatype
//			return resource.type === this.dataType;
//		}, this)) {
//			valResult.errors.push(messages.valSomeResourceOfInvalidType);
//			}
//		}
	},
	
	_onSelectionChange: function(e) {
		if (this._resourcesBoxWidget.getSelection().length === 0) {
			this._removeResourcesButton.set('disabled', true);
		} else {
			this._removeResourcesButton.set('disabled', false);
		}
	},
	
	_onRemoveButton: function(e) {
		this._resourcesBoxWidget.removeEntries(this._resourcesBoxWidget.getSelection());
		this._removeResourcesButton.set('disabled', true);
		this._onChange();
	},
	
	_onAddButton: function(e) {
		var dialog = null;
		var addResources = dojo.hitch(this, function(resources) {
			if (dialog) {	
				dialog.close();
				dialog.destroyRecursive();
			}
			this._addResourcesOSLC(resources);
			this._onChange();
		});
		var picker = new rmps.app.pickers.internal.CLMResourcePicker({
			projectId: jazz.app.context.get().itemId,
			context: jazz.app.context.get().baseline,
			filteredList: this.getResources(),
			enableMultiSelect: true,
			typeUri: this.getDataType(),
			onOK: addResources
		});
		
		var dialogParams = {
            primaryTitle: messages.resourceSelectionPrimaryTitle,
            secondaryTitle: messages.resourceSelectionSecondaryTitle,
	        width: "800px",
	        height: "550px",
	        contentNode: picker.domNode
        };
		dialog = new jazz.ui.Dialog(dialogParams);
	},
		
	_addResourcesOSLC: function(resources) {
		var entries = dojo.map(resources, function (resource) {
			return {
				id: resource["rdf:resource"],
				label: resource["oslc:label"],
				type: this.getDataType()
			};
		}, this);
		this._resourcesBoxWidget.addEntries(entries);
	},
	
	_addResources: function(resources) {
		var entries = dojo.map(resources, function (resource) {
			return {
				type: resource.type || this.dataType,
				id: resource.link || resource.uri,
				label: resource.shortName.label || resource.shortName
			};
		}, this);
		this._resourcesBoxWidget.addEntries(entries);
	},
	
	_onChange: function() {
		if (this.changeCallback) {
			this.changeCallback();
		}
	}
});

}());