/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.ui.queryeditor.filters.UserFilterWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("jazz.ui.ListBox");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog");
dojo.require("rmps.core.user.UserClient");
dojo.require("rmps.ui.queryeditor.filters.IResourcesFilter");
dojo.require("rmps.ui.queryeditor.QueryConstants");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

(function(){

var ServiceResponseHandler= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var SelectContributorDialog = com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog;
var UserClient = rmps.core.user.UserClient;
var QueryConstants = rmps.ui.queryeditor.QueryConstants;
var Messages = dojo.i18n.getLocalization("rmps.ui.queryeditor", "QueryWidgetMessages");

dojo.declare("rmps.ui.queryeditor.filters.UserFilterWidget", [dijit._Widget, dijit._Templated, rmps.ui.queryeditor.filters.IResourcesFilter], {

	templatePath: dojo.moduleUrl("rmps.ui", "queryeditor/filters/templates/UserFilterWidget.html"),
	widgetsInTemplate: true,
	
	_addButtonLabel: Messages.labelAddUser,
	_removeButtonLabel: Messages.labelRemoveUser,
	
	_hasSelection: false,
	_selection: null,
	_listBoxWidget: null,
	_userObjs: null,

	constructor: function() {
		this._selection = [];
	},
	
	postCreate: function() {
		this._listBoxWidget = new jazz.ui.ListBox({
			onChange: dojo.hitch(this, this._onSelectionChange),
			margin: "2px",
			height: "100px"
		});
		this._listBox.appendChild(this._listBoxWidget.domNode);
		this._userObjs = [];
		this._updateButtons();
		//this._usersPromise = UserClient.getUsers(dojo.getObject("jazz.app.currentApplication").context).then(dojo.hitch(this, this._initValues));
	},
	
	destroy: function() {
		try {
			if (this._addUserButton) {
				this._addUserButton.destroy();
				delete this._addUserButton;
			}
			if (this._removeUserButton) {
				this._removeUserButton.destroy();
				delete this._removeUserButton;
			}
			
			this._destroyDialog();
			
			if (this._listBoxWidget) {
				this._listBoxWidget.destroy();
				delete this._listBoxWidget;
			}
			
			this.inherited(arguments);
		} catch (e) {
			
		}
	},
	
	/* String[] */ getResources: function() {
		
		
		return dojo.map(this._listBoxWidget.getAllEntries(), function(entry) {
			return entry.id;
		});
	},
	
	setSelection: function(selection) {
		this._listBoxWidget.showLoading(true);
		var defs = [];
		var def;
		var rememberUser = dojo.hitch(this, function(user){
			var entry = {
					label: user.name,
					id: user.uri,
					value: null,
					type: 0
				};
			this._userObjs[user.uri] = entry;
		}); 
		for (var i = 0; i < selection.length; ++i) {
			def = UserClient.getUser(selection[i]);
			def.then(rememberUser);
			defs.push(def);
		}
		var defList = new dojo.DeferredList(defs);
		
		defList.then(dojo.hitch(this, function() {
			var entries = dojo.map(selection, dojo.hitch(this, function(id) {
				return this._userObjs[id];
			}));
			this._listBoxWidget.removeAllEntries();
			this._listBoxWidget.addEntries(entries);
			this._listBoxWidget.showLoading(false);
			this._updateButtons();
		}));
	},
	
	setResources: function(value) {
		this.setSelection(value);
	},
	
	getDataType: function() {
		return QueryConstants.USER_DATATYPE;
	},
	
	validate: function(valResult) {
		if (this.getResources().length === 0) {
			valResult.errors.push(Messages.valNoUserSelected);
			if (this.parentWidget && this.parentWidget._showWarning) {
				this.parentWidget._showWarning(Messages.valNoUserSelected);
			}
		}
	},
	
	///////////////////////////////////////////////////////////////////////////////////
	// Internals

	
	_getInitialSelectionValues: function() {
		var values = this._selection || [];
		var selection = [];
		for (var i = 0; i < values.length; i++) {
			selection.push(values[i]);
		}
		return selection;
	},
		
	_onSelectionChange: function() {
		if (this.changeCallback) {
			this.changeCallback();
		}
		this._updateButtons();
	},
	
	
	_onAddButton: function() {
		this._destroyDialog();
		var self= this;
		var params = {
				title: Messages.dialogTitleAddUsers,
				parentElement: this.domNode,
				disableUsers: [],
				onOK: function(selectedContribs) {
					if (self.parentWidget) {
						self.parentWidget._hideWarning();
					}
					self._addUsers(dojo.map(selectedContribs, function(contrib) {
						return {
							label: contrib.name,
							id: UserClient.getUserUri(contrib.userId),
							value: null,
							type: 0
						};
					}), true);
				},
				onClose: function(selectedContribs) {
					self._destroyDialog();
				}	
			};
		this.userListWidget = new SelectContributorDialog(params);
			
	},
	_destroyDialog: function() {
		if (this.userListWidget) {
			this.userListWidget.destroy(false);
		}
		this.userListWidget = null;
	},		
	
	_addUsers: function(selection, select) {
		var entries = [];
		var newentries = [];
		var w = this._listBoxWidget;
		for (var i= 0; i < selection.length; i++) {
			var selected = selection[i];
			var entry = w.getEntry(selected.id);
			if (entry) {
				entries.push(entry);
			} else {
				entries.push(selected);
				newentries.push(selected);
				this._userObjs[selected.id] = selected;
			}
			
			w.addEntries(newentries);
			if (select) {
				w.setSelection(entries, true);
				this._disableRemoveButton(entries.length === 0);
			}
		}
		if (this.changeCallback) {
			this.changeCallback();
		}
	},
	
//	_initValues: function(users) {
//		if (!users || users.length === 0) {
//			return false;
//		}
//		this._listBoxWidget.removeAllEntries();
//		this._addUsers(users, false);
//		this._initSema--;
//		return true;
//	},
	
	_setSelection: function() {
		if (!this._selection) {
			return;
		}
		if (this._selection.length > 0) {
			this._hasSelection = true;			
		}
		// Unselect all
		this._listBoxWidget.setSelection(this._listBoxWidget.getAllEntries(), false);
		this._usersPromise.then(dojo.hitch(this, function() {
			this._addUsers(this._selection, true);
		}));
	},
		
	_onRemoveButton: function(e) {
		this._listBoxWidget.removeEntries(this._listBoxWidget.getSelection());
		this._removeUserButton.set('disabled', true);
		this._onSelectionChange();
	},
	
	_disableRemoveButton: function(value) {
		this._removeUserButton.set('disabled', value);
	},
	
	_updateButtons: function() {
		this._disableRemoveButton(this._listBoxWidget.getSelection().length === 0);		
	}
});
	
}());

