/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.querymodel.QueryModel");

dojo.require("rmps.core.resource.Resource");

var QUERY_CLASS = "Query";
var NAME_PROPERTY = "label";
var DESCRIPTION_PROPERTY = "comment";
var CONDITION_PROPERTY = "condition";
var SORTING_PROPERTY = "sorting";
var GROUPING_PROPERTY = "groupingProperty";

var RDFS_PREFIX = "rdfs";

dojo.declare("rmps.core.querymodel.QueryModel", rmps.core.resource.Resource, {

	// mix-in
	dataObject: null,
	contentType: "application/json",
	
	constructor: function() {
		this[NAME_PROPERTY] = "";
		this[DESCRIPTION_PROPERTY] = "";
	},
	
	getName: function() {
		return this[NAME_PROPERTY];
	},
	
	setName: function(name) {
		this[NAME_PROPERTY] = name;
	},
	
	getDescription: function() {
		return this[DESCRIPTION_PROPERTY];
	},
	
	setDescription: function(description) {
		this[DESCRIPTION_PROPERTY] = description;
	},
	
	getDataObject: function() {
		return this.dataObject;
	},
	
	setDataObject: function(data) {
		this.dataObject = data;
	},
	
	setSorting: function(sorting) {
		this[SORTING_PROPERTY] = sorting;
	},
	
	setGrouping: function(grouping) {
		this[GROUPING_PROPERTY] = grouping;
	},
	
	serialize: function() {		
		var obj = {};
		var query = obj[QUERY_CLASS] = {};
		
		var name = this[NAME_PROPERTY]; 
		if (name && name.length > 0) {
			query[NAME_PROPERTY] = name; 
		}
		var desc = this[DESCRIPTION_PROPERTY];
		if (desc && desc.length > 0) {
			query[DESCRIPTION_PROPERTY] = desc;
		}
		if (this.dataObject) {
			query[CONDITION_PROPERTY] = this.dataObject;
		}
		
		var grouping = this[GROUPING_PROPERTY];
		if (grouping) {
			query[GROUPING_PROPERTY] = grouping;
		}
		var sorting = this[SORTING_PROPERTY];		
		if (sorting) {
			query[SORTING_PROPERTY] = sorting;
		}
		return dojo.toJson(obj);
	},
	
	deserialize: function(dataObject) {
		var query = dataObject[QUERY_CLASS];
		if (!query) {
			return;
		}
		
		if (query[CONDITION_PROPERTY]) {
			this.dataObject = query[CONDITION_PROPERTY];
		}
		if (query[NAME_PROPERTY]) {
			this[NAME_PROPERTY] = query[NAME_PROPERTY]; 
		} else {
			this[NAME_PROPERTY] = "";
		}
		if (query[DESCRIPTION_PROPERTY]) {
			this[DESCRIPTION_PROPERTY] = query[DESCRIPTION_PROPERTY];
		} else {
			this[DESCRIPTION_PROPERTY] = "";
		}
		if (query[SORTING_PROPERTY]) {
			this[SORTING_PROPERTY] = query[SORTING_PROPERTY];
		}
		if (query[GROUPING_PROPERTY]) {
			this[GROUPING_PROPERTY] = query[GROUPING_PROPERTY];
		}
	},
	
	deserializeFromAtomJson: function(entry, ioArgs) {
		if (ioArgs) {
			var xhr = ioArgs.xhr;
			if (xhr.getResponseHeader("Last-Modified")) {
				this.lastModified = new Date(xhr.getResponseHeader("Last-Modified"));
			}
			if (xhr.getResponseHeader("ETag")) {
				this.eTag = xhr.getResponseHeader("ETag");
			}
			if (xhr.getResponseHeader("Location")) {
				this.uri = xhr.getResponseHeader("Location");
			}
		}
		
		// link
		var links = entry.link;
		for (var j = 0; j < links.length; j++) {
			var link = links[j];
			if (link.rel == "self") {
				this.uri = links[j].href;
			}
		}
		
		// name
		this[NAME_PROPERTY] = entry.title[0].value;
		
		// description
		this[DESCRIPTION_PROPERTY] = entry[RDFS_PREFIX + ":" + DESCRIPTION_PROPERTY][0].value;
		if (this[DESCRIPTION_PROPERTY] == undefined) {
			this[DESCRIPTION_PROPERTY] = "";
		}
	}
});
})();