/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.querymodel.QueryModelClient");

dojo.require("rmps.core.client");
dojo.require("rmps.core.querymodel.QueryModel");
dojo.require("rmps.core.RootServices");

var QUERY_MODEL_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "queries");
var QUERY_WIDGET_DATA_PATH = QUERY_MODEL_PATH + "/widgetdata/";
var QUERY_COPY_PATH = QUERY_MODEL_PATH + "/copy";

rmps.core.querymodel.QueryModelClient = {

	getQueryModels: function(projectId) {
		var args = {
			url: QUERY_MODEL_PATH,
			content: {
				projectId: projectId,
				matchAssociatedOntologies: true
			},
			headers: {
				"X-ibm-rmps-internal": "true",
				"Accept": "application/atom+json"
			},
			handleAs: "json",
			load: function(response, ioArgs)
			{
				var result = [];
				var entries = response.entry;
				
				result.pageToken = "";
				result.totalResults = parseInt(response["opensearch:totalResults"][0].value);
				
				if (entries && entries.length > 0) {
					for (var i = 0; i < entries.length; i++) {
						// Note that viewpoint only contain partial data
						var queryModel = new rmps.core.querymodel.QueryModel();
						queryModel.deserializeFromAtomJson(entries[i], ioArgs);
						result.push(queryModel);
					}
				}
				
				var links = response.link;
				if (links && links.length > 0) {
					var getParamater= function(/*String */ url, /*String */ key) {
				    	var query= new dojo._Url(url).query;
				    	var object= dojo.queryToObject(new dojo._Url(url).query);
				    	return object[key];
				    };
					for (var l = 0; l < links.length; l++) {
						var link = links[l];
						if (link.rel == "rel") {
							result.pageToken = getParamater(link.href, "pageToken");
							break;
						}
					}
				}
				return result;
			}
		};
		return rmps.core.client.xhrGet(args, false /*sendProjectId*/);
	},
	
	getQueryModel: function(queryModelUri) {
		var args = {
			url: queryModelUri,
			headers: {
				"Accept": "application/json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				var location = ioArgs.xhr.getResponseHeader("Location"); 
				var queryModel = new rmps.core.querymodel.QueryModel({
					eTag: ioArgs.xhr.getResponseHeader("ETag"),
					uri: location ? location : queryModelUri
					//lastModified = new Date(ioArgs.xhr.getResponseHeader("Last-Modified")),
				});
				queryModel.deserialize(response);
				return queryModel;
			}
		};
		return rmps.core.client.xhrGet(args);
	},
	
	getQueryModelByID: function(queryModelId, projectId) {
		var args = {
			url: QUERY_MODEL_PATH + '/id/' + queryModelId,
			content: {
				projectId: projectId,
				id: queryModelId
			},
			headers: {
				"Accept": "application/json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				var location = ioArgs.xhr.getResponseHeader("Location"); 
				var queryModel = new rmps.core.querymodel.QueryModel({
					eTag: ioArgs.xhr.getResponseHeader("ETag"),
					uri: location
					//lastModified = new Date(ioArgs.xhr.getResponseHeader("Last-Modified")),
				});
				queryModel.deserialize(response);
				return queryModel;
			}
		};
		return rmps.core.client.xhrGet(args);		
	},
	
	getWidgetData: function(widgetID, inputData) {
		var args = {
			url: QUERY_WIDGET_DATA_PATH + widgetID,
			headers: {
				"Accept": "application/json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				return response;
			},
			content: inputData
		};
		return rmps.core.client.xhrGet(args);		
	},
	
	deleteQueryModel: function(queryModel) {
		var args = {
			headers: {
				"If-Match": queryModel.eTag,
				"X-ibm-rmps-internal": "true"
			},
			url: queryModel.uri
		};
		return rmps.core.client.xhrDelete(args);
	},
	
 	createQueryModel: function(queryModel, projectId, synchronous) {
 		var query = {
 			projectId: projectId	
 		};
 		var args = {
 			url: QUERY_MODEL_PATH + "?" + dojo.objectToQuery(query),
 			headers: {
 				"X-ibm-rmps-internal": "true",
 				"Content-type": queryModel.contentType
 			},
 			hasBody: true,
 			content: {
 				projectId: projectId
 			},
 			postData: queryModel.serialize(),
 			sync: synchronous,
 			load: function(response, ioArgs) {
 				// Update the queryModel object
 				queryModel.eTag = ioArgs.xhr.getResponseHeader("ETag");
				queryModel.uri = ioArgs.xhr.getResponseHeader("Location");
				//queryModel.lastModified = new Date(ioArgs.xhr.getResponseHeader("Last-Modified"));
				return queryModel;
 			}
 		};
 	 		
 	 	return rmps.core.client.xhrPost(args);
 	},
 	
	updateQueryModel: function(queryModel, projectId, synchronous) {
 		var query = {
 			projectId: projectId	
 		};
		var args = {
			headers: {
				"Content-Type": queryModel.contentType,
				"If-Match": queryModel.eTag,
				"X-ibm-rmps-internal": "true"
			},
			sync: synchronous, 
			url: queryModel.uri + "?" + dojo.objectToQuery(query),
			hasBody: true,
 			content: {
 				projectId: projectId
 			},
			putData: queryModel.serialize(),
			load: function(response, ioArgs) {
 				queryModel.eTag = ioArgs.xhr.getResponseHeader("ETag");
				//queryModel.lastModified = new Date(ioArgs.xhr.getResponseHeader("Last-Modified"));
 				return queryModel;
			}
		};
		return rmps.core.client.xhrPut(args);
	},
	
	createQueryCopy: function(originalQueryURI, projectId, newTitle) {
		var query = {
			projectId: projectId	
		};
		var content = {
			original: originalQueryURI.substring(originalQueryURI.lastIndexOf('/')+1),
			newTitle: newTitle
		};
		var args = {
			url: QUERY_COPY_PATH + "?" + dojo.objectToQuery(query),
			headers: {
				"X-ibm-rmps-internal": "true",
				"Content-Type" : "application/json"
			},
			postData: dojo.toJson(content),
			load: function(response, ioArgs) {
				var newQueryUri = ioArgs.xhr.getResponseHeader("Location");
				return newQueryUri;
			}

		};
		return rmps.core.client.xhrPost(args);				

	}
	
};
}());