/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.web.comment.sketch.render.Ellipse");

dojo.require("rmps.web.comment.sketch.render.Primitive");
dojo.require("rmps.web.comment.sketch.serializer.SVGStyle");
dojo.require("rmps.web.comment.sketch.serializer.SVGUtil");
dojo.require("rmpw.diagrams.render.policies.GraphNode");

dojo.declare("rmps.web.comment.sketch.render.Ellipse", rmps.web.comment.sketch.render.Primitive, {
	
	setSvgEllipse: function(svgEllipse) {
		this.sources.svgRoot = svgEllipse;
		this.sources.svgShape = svgEllipse.ellipse[0];
	},
	
	/*
	 * @param {graphics::Group} graphicsParent
	 * 
	 */
	setGraphicsParent: function(graphicsParent) {
		this.inherited(arguments);
		
		var template = this.template;
		var gfxEllipse;
		
		var svgEllipse = this.sources.svgShape;
		gfxEllipse = template.group.createEllipse();
		
		var style = new rmps.web.comment.sketch.serializer.SVGStyle();
		
		style.deriveStyle( this.sources.svgShape );
						
		gfxEllipse.setStroke( style.getStroke() );		
		gfxEllipse.setFill( style.getFill() );
				
		this.template.gfxShape = gfxEllipse;
	},
	
	applyModelChanges: function() {
		this.inherited(arguments);
		var anyChanged = false;
		for(var traceKey in this.traces) {
			var trace = this.traces[traceKey];
			if(!trace.synced) {
				anyChanged = true;
			}
			trace.reset();
		}
		
		if (anyChanged) {
			var traces = this.traces;
			
			var cx = traces.cx.get();
			var cy = traces.cy.get();
			var rx = traces.rx.get();
			var ry = traces.ry.get();
			
			// let GraphNodeLabelLayout / GraphNode adjust it's position
			this.policies.graphNode.setX(cx - rx);
			this.policies.graphNode.setY(cy - ry);
			this.policies.graphNode.enforce();
			
			this.policies.graphNodeLabelLayout.setWidth(rx * 2);
			this.policies.graphNodeLabelLayout.setHeight(ry * 2);
			this.policies.graphNodeLabelLayout.enforce();
			
			var stroke = this.template.gfxShape.getStroke();	
			var SVGUtil = new rmps.web.comment.sketch.serializer.SVGUtil();	
			SVGUtil.buildStroke( stroke, traces );
			this.template.gfxShape.setStroke(stroke);
			
			var fill = traces.fill.get();		
			
			if(fill!=null){
				var fillOpacity = traces.fillOpacity.get();			
				var fillColor = new dojo.Color( fill );			
				fillColor.a = fillOpacity;		
				this.template.gfxShape.setFill(fillColor);
			}
		}
	},
	
	attachListeners: function() {
		this.inherited(arguments);
		
		this.connectTraces(this.sources.svgShape, ["cx", "cy", "rx", "ry", "stroke", "strokeOpacity", "strokeWidth", "strokeDasharray", "fill", "fillOpacity"]);
	},
		
	createTraces: function() {
		this.inherited(arguments);
		
		this.createTrace("cx", "svgShape.cx");
		this.createTrace("cy", "svgShape.cy");
		this.createTrace("rx", "svgShape.rx");
		this.createTrace("ry", "svgShape.ry");
		this.createTrace("strokeWidth", "svgShape.strokeWidth");
		this.createTrace("stroke", "svgShape.stroke");
		this.createTrace("strokeOpacity", "svgShape.strokeOpacity");
		this.createTrace("strokeDasharray", "svgShape.strokeDasharray");
		this.createTrace("fill", "svgShape.fill");
		this.createTrace("fillOpacity", "svgShape.fillOpacity");
	}
});
