/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.web.comment.sketch.render.Group");

dojo.require("rmpw.diagrams.render.policies.GraphNode");
dojo.require("rmpw.diagrams.render.util.Workarounds");

dojo.require("rmps.web.comment.sketch.render.Line");
dojo.require("rmps.web.comment.sketch.render.Path");
dojo.require("rmps.web.comment.sketch.render.Ellipse");
dojo.require("rmps.web.comment.sketch.render.Rectangle");

dojo.require("rmps.web.comment.sketch.serializer.SVGStyle");
dojo.require("rmpw.common.vcs.ChangeSet");
dojo.require("rmpw.common.transform.Mapping");

dojo.declare("rmps.web.comment.sketch.render.Group", rmpw.common.transform.Mapping, {
	
	elementId: null,
	
	transform: null,
	
	/*
	 * @param {svg::GType} g
	 * 
	 */
	setG: function(g) {
		this.sources.svgRoot = g;
	},
	
	setTransform: function( transform ){
		this.transform = transform;
	},
	
	getBounds: function(){
		
		// TODO: should be done by selectable policy
		var bounds = rmpw.diagrams.render.util.Workarounds.getBoundingBox(this.template.contents);
		
		if(bounds && bounds.b) {
			bounds = {x: bounds.l, y: bounds.t, width: bounds.r - bounds.l, height: bounds.b - bounds.t};
		}
		
		return bounds;
	},
	
	/*
	 * @param {graphics::Group} graphicsParent
	 * 
	 */
	setGraphicsParent: function(graphicsParent) {
		
		this.template.figure = graphicsParent.createGroup();
		
		this.template.hitzone = this.template.figure.createRect();
		this.template.hitzone.setFill([0,0,0,0.0001]);
		
		this.template.contents = this.template.figure.createGroup();
		
		this.rootShapes.push(this.template.figure);
	},
	
	createPolicies: function() {

		var policy = this.createPolicy(rmpw.diagrams.render.policies.GraphNode);
		policy.setFigure(this.template.figure);
		policy.useDojoPosition(true);
       
		return policy;
	},	
	
	applyModelChanges: function() {
		var trace, child, added;
		
		this.inherited(arguments);
		
		trace = this.traces.svgRoot$g;
		if (! trace.synced) {
			added = trace.getAdded();
			
			for (var i = 0, gItem; gItem = added[i]; i++) {
				if (gItem.elementId == "Path") {
					child = new rmps.web.comment.sketch.render.Path();
					child.setSvgPath(gItem);
					child.setGraphicsParent(this.template.contents);
					this.addChild(child);
				}
				else if (gItem.elementId == "Rectangle" || gItem.elementId == "TextBox" || gItem.elementId == "Text") {
					child = new rmps.web.comment.sketch.render.Rectangle();
					child.setSvgRectangle(gItem);
					child.setGraphicsParent(this.template.contents);
					this.addChild(child);
				}
				else if (gItem.elementId == "Line") {
					child = new rmps.web.comment.sketch.render.Line();
					child.setSvgLine(gItem);
					child.setGraphicsParent(this.template.contents);
					this.addChild(child);
				}
				else if (gItem.elementId == "Oval") {
					child = new rmps.web.comment.sketch.render.Ellipse();
					child.setSvgEllipse(gItem);
					child.setGraphicsParent(this.template.contents);
					child.groupTraces = this.traces;
					this.addChild(child);
				}
			}
			trace.reset();
		}
				
		trace = this.traces.transform;
		if(!trace.synced){			
			this.template.figure.remove( this.template.hitzone );
			trace.reset();
		}		
		
		this.template.hitzone.setShape(this.getBounds());
	},
	
	attachListeners: function() {
		this.inherited(arguments);
		
		this.connectTraces(this.sources.svgRoot, [
			"svgRoot$g",
			"transform"
		]);
		
		if(!this.sources.svgRoot.transform) {
			this.sources.svgRoot.transform = null;
		}
		
	},
	
	createTraces: function() {
		this.inherited(arguments);
		
		this.createTrace("svgRoot$g", "svgRoot.g", true);
		this.createTrace("transform", "svgRoot.transform", true);
	}
});
