/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.graphics.render.HotspotDiagramViewMapping");

dojo.require("rmps.graphics.render.TileViewMapping");
dojo.require("rmps.graphics.render.BaseMapping");
dojo.require("rmpw.common.transform.Mapping");
dojo.require("rmpw.diagrams.render.policies.GraphContainer");
dojo.require("rmpw.diagrams.render.policies.GraphLayout");
dojo.require("rmps.graphics.render.RectangleHotspotMapping");
dojo.require("rmps.graphics.render.LineHotspotMapping");

/** 
 * @author Anton McConville
 */
dojo.declare("rmps.graphics.render.HotspotDiagramViewMapping", rmpw.common.transform.Mapping, {
	isImageDiagram: true,
	
	setModel: function(model) {
		this.sources.model = model;
		this.sources.entities = [];
	},

	setParentFigure: function(parentFigure) {
		// Create figure template
		var figure = parentFigure.createGroup();
		
		// Save attach points
		var template = this.template;
		template.figure = figure;
	},
	
	createTraces: function() {
		this.inherited(arguments);	
		this.createTrace("entities", "model.entities", true);
	},
	
	attachListeners: function() {
		this.inherited(arguments);
		this.connectTraces(this.sources.model, ["entities"]);
	},
	
	applyModelChanges: function() {
		this.inherited(arguments);
		
		var trace;
		
		trace = this.traces.entities;
		if (!trace.synced) {
			var child;
			
			var added = trace.getAdded();
			
			for (var i in added) {
				
				var renderClass = trace.added[i].declaredClass;
				
				// console.log( "RENDER CLASS: " + renderClass );
				
				switch( renderClass ){
				
					case "rmps.graphics.models.Base":						
						child = new rmps.graphics.render.BaseMapping();
						break;
						
					case "rmps.graphics.models.Tile":						
						child = new rmps.graphics.render.TileViewMapping();
						break;
						
					case "rmps.graphics.models.RectangleHotspot":						
						if (trace.added[i].width==0 || trace.added[i].height==0) continue;
						child = new rmps.graphics.render.RectangleHotspotMapping();
						break;
						
					case "rmps.graphics.models.LineHotspot":
						child = new rmps.graphics.render.LineHotspotMapping();
						break;
				}

				child.setModel(trace.added[i]);
				child.setParentFigure(this.template.figure);
				this.addChild(child);
			}
			trace.reset();
		}
	},

	createPolicies: function() {
		var template = this.template;		
		policy = this.createPolicy(rmpw.diagrams.render.policies.GraphContainer);
		policy.setFigure(template.figure);
		
		// allow drag of diagram
		if (policy.getBackground() && this.viewer.policies.dragPan) {
			this.viewer.policies.dragPan.allowImageDragPan(true);
			policy.getBackground().connect("onmousedown", this.viewer.policies.dragPan, "_onMouseDown");
		}
	}
});