/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.web.comment.sketch.render.Line");

dojo.require("rmps.web.comment.sketch.serializer.SVGStyle");
dojo.require("rmps.web.comment.sketch.serializer.SVGUtil");
dojo.require("rmps.web.comment.sketch.render.Primitive");
dojo.require("rmpw.diagrams.render.policies.GraphNode");
dojo.require("rmpw.diagrams.render.util.PointList");

dojo.declare("rmps.web.comment.sketch.render.Line", rmps.web.comment.sketch.render.Primitive, {
	
	setSvgLine: function(svgLine) {
		this.sources.svgRoot = svgLine;
		this.sources.svgShape = svgLine.line[0];
	},
	
	setGraphicsParent: function(graphicsParent) {
		this.inherited(arguments);
		
		var template = this.template;
		
		var svgLine = this.sources.svgShape;
		var gfxLine = template.group.createLine();
		
		var style = new rmps.web.comment.sketch.serializer.SVGStyle();	
		style.deriveStyle(this.sources.svgShape);
		gfxLine.setStroke( style.getStroke() );		
		
		// save attach points
		this.template.gfxShape = gfxLine;
	},
	
	applyModelChanges: function() {
		this.inherited(arguments);

		var anyChanged = false;
		for(var traceKey in this.traces) {
			var trace = this.traces[traceKey];
			if(!trace.synced) {
				anyChanged = true;
			}
			trace.reset();
		}
		
		if (anyChanged) {
			var traces = this.traces;
			
			var x1 = traces.x1.get();
			var y1 = traces.y1.get();
			var x2 = traces.x2.get();
			var y2 = traces.y2.get();
			var ps = new rmpw.diagrams.render.util.PointList(2);
			ps.addPoint_0(x1, y1);
			ps.addPoint_0(x2, y2);
			
			var bbox = ps.getBounds();
			
			// let GraphNodeLabelLayout / GraphNode adjust it's position
			this.policies.graphNode.setX(bbox.x);
			this.policies.graphNode.setY(bbox.y);
			this.policies.graphNode.enforce();
			
			this.policies.graphNodeLabelLayout.setWidth(bbox.width);
			this.policies.graphNodeLabelLayout.setHeight(bbox.height);
			this.policies.graphNodeLabelLayout.enforce();
			
			x1 -= bbox.x;
			y1 -= bbox.y;
			x2 -= bbox.x;
			y2 -= bbox.y;
			this.template.gfxShape.setShape({
				x1: x1, y1: y1, x2: x2, y2: y2
			});

			var stroke = this.template.gfxShape.getStroke();		
			var SVGUtil = new rmps.web.comment.sketch.serializer.SVGUtil();		
			SVGUtil.buildStroke( stroke, traces );
			
			stroke.cap = "round";
			
			this.template.gfxShape.setStroke(stroke);
		}
	},
	
	getBounds: function( x1, y1, x2, y2){

		var bounds = {};
		bounds.x = x1;
		bounds.y = y1;

		if( x1 > x2 ){ bounds.x = x2; }
		if( y1 > y2 ){ bounds.y = y2; }

		bounds.width = Math.abs(x1 - x2);
		bounds.height = Math.abs(y1 - y2);

		return bounds;
	},
	
	attachListeners: function() {
		this.inherited(arguments);	
		this.connectTraces(this.sources.svgShape, ["x1", "y1", "x2", "y2",  "stroke", "strokeWidth", "strokeDasharray", "strokeOpacity"]);
	},
	
	createPolicies: function() {
		this.inherited(arguments);
		
		var policy = this.createPolicy(rmpw.diagrams.render.policies.GraphAnchorable);
		policy.setAnchorShape(this.template.gfxShape);
	},	
	
	createTraces: function() {
		this.inherited(arguments);
		
		this.createTrace("x1", "svgShape.x1");
		this.createTrace("y1", "svgShape.y1");
		this.createTrace("x2", "svgShape.x2");
		this.createTrace("y2", "svgShape.y2");
		this.createTrace("style", "svgShape.style");
		this.createTrace("strokeWidth", "svgShape.strokeWidth");
		this.createTrace("stroke", "svgShape.stroke");
		this.createTrace("strokeDasharray", "svgShape.strokeDasharray");
		this.createTrace("strokeOpacity", "svgShape.strokeOpacity");
	}
});
