/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.web.comment.sketch.render.Path");

dojo.require("rmps.web.comment.sketch.render.Primitive");
dojo.require("rmps.web.comment.sketch.serializer.SVGStyle");
dojo.require("rmps.web.comment.sketch.serializer.SVGUtil");
dojo.require("dojox.gfx.path");

dojo.declare("rmps.web.comment.sketch.render.Path", rmpw.common.transform.Mapping, {
	
	setSvgPath: function(svgRoot) {
		this.sources.svgRoot = svgRoot;
		this.sources.svgShape = svgRoot.path[0];
	},
	
	setGraphicsParent: function(graphicsParent) {
		this.inherited(arguments);
		
		this.template.group = graphicsParent.createGroup();	
		var template = this.template;
		var path;
		
		template.hitZoneRect = template.group.createRect();
		template.hitZoneRect.setFill([0,0,0,0.0001]);
						
		template.gfxShape = template.group.createPath();
		
		var style = new rmps.web.comment.sketch.serializer.SVGStyle();			
		style.deriveStyle(this.sources.svgShape);
		template.gfxShape.setStroke( style.getStroke() );		
		template.gfxShape.setFill( style.getFill() );
		
		this.rootShapes.push(this.template.group);
	},
	
	/*
	 * @param {String} value
	 * 
	 */
	setPathString: function(value) {
		if (this.template.gfxShape) {
			this.template.gfxShape.setShape({path: value});
		} else {
			this.sources.pathString = value;
		}
	},
	
	applyModelChanges: function() {
		this.inherited(arguments);
		
		var anyChanged = false;
		for(var traceKey in this.traces) {
			var trace = this.traces[traceKey];
			if(!trace.synced) {
				anyChanged = true;
			}
			trace.reset();
		}
		
		if (anyChanged) {
			var traces = this.traces;
			
			var style = traces.style.get();	
			
			var pathString = traces.pathString.get();		
			this.template.gfxShape.setShape({path: pathString});
			
			var bounds = rmpw.diagrams.render.util.Workarounds.getBoundingBox(this.template.gfxShape);				
			
			if( bounds != null ){
				this.template.hitZoneRect.setShape({x: bounds.x, y: bounds.y, width: bounds.width, height: bounds.height });
			}
			
			var stroke = this.template.gfxShape.getStroke();		
			var SVGUtil = new rmps.web.comment.sketch.serializer.SVGUtil();		
			SVGUtil.buildStroke( stroke, traces );
			stroke.cap = "round";
			this.template.gfxShape.setStroke(stroke);
			
			var fill = traces.fill.get();	
			
			if( fill ){
				var fillOpacity = traces.fillOpacity.get();			
				var fillColor = new dojo.Color( fill );			
				fillColor.a = fillOpacity;		
				this.template.gfxShape.setFill(fillColor);
			}
		}
	},
	
	
	attachListeners: function() {
		this.inherited(arguments);
		
		this.connectTraces(this.sources.svgShape, ["pathString", "style", "stroke", "strokeWidth", "strokeOpacity", "strokeDasharray", "fill", "fillOpacity"]);
	},
	
	createTraces: function() {
		this.inherited(arguments);
		
		this.createTrace("pathString", "svgShape.d");
		this.createTrace("style", "svgShape.style");
		this.createTrace("strokeWidth", "svgShape.strokeWidth");
		this.createTrace("stroke", "svgShape.stroke");
		this.createTrace("strokeOpacity", "svgShape.strokeOpacity");
		this.createTrace("strokeDasharray", "svgShape.strokeDasharray");
		this.createTrace("fill", "svgShape.fill");
		this.createTrace("fillOpacity", "svgShape.fillOpacity");
	},
	
	createPolicies: function() {
		var policy = this.createPolicy(rmpw.diagrams.render.policies.GraphNode);
		policy.useDojoPosition(true);	
		policy.setFigure(this.template.group);
	},
});
