/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corporation 2009, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.web.comment.sketch.render.Primitive");

dojo.require("rmpw.common.transform.Mapping");

dojo.require("rmps.web.comment.sketch.serializer.SVGStyle");

dojo.require("rmps.web.comment.sketch.serializer.SVGUtil");
dojo.require("rmpw.diagrams.render.util.Workarounds");

dojo.require("rmpw.diagrams.render.policies.GraphNode");
dojo.require("rmpw.diagrams.render.gfx.WrappingText");
dojo.require("rmpw.diagrams.render.policies.GraphNodeLabelLayout");

dojo.require("dojox.html.metrics");

dojo.require("dojox.gfx.shape");

dojo.require("dojox.gfx");

dojo.declare("rmps.web.comment.sketch.render.Primitive", rmpw.common.transform.Mapping, {
	
	textTraces: [ "text", "fontSize", "fontWeight", "fontFamily", "align", "textColor", "textOpacity", "stroke", "strokeOpacity", "fontStyle"],
	
	node: null,
		
	setGraphicsParent: function(graphicsParent) {
		this.template.group = graphicsParent.createGroup();	
		
		var background = this.template.group.createRect();
		
		var wrappingTextExpression = this.template.group.createWrappingText();
		wrappingTextExpression.customStyle = {
			paddingTop: 4.0,
			paddingBottom: 4.0,
			paddingLeft: 4.0,
			paddingRight: 4.0
		};
		
		this.template.background = background;
		this.template.wrappingTextExpression = wrappingTextExpression;
		
		this.rootShapes.push(this.template.group);
	},
	
	connectTraces: function(source, traces) {
		traces = traces.concat( this.textTraces );			
		this.inherited(arguments)	
	},
	
	createTraces: function(){
		this.inherited(arguments);
		this.createTrace("text", "svgShape.text");	
		this.createTrace("fontSize", "svgShape.fontSize");
		this.createTrace("fontWeight", "svgShape.fontWeight");
		this.createTrace("fontFamily", "svgShape.fontFamily");
		this.createTrace("align", "svgShape.align");
		this.createTrace("textColor", "svgShape.textColor");
		this.createTrace("textOpacity", "svgShape.textOpacity");
		this.createTrace("textStroke", "svgShape.textStroke");
		this.createTrace("fontStyle", "svgShape.fontStyle");
		this.createTrace("textStrokeOpacity", "svgShape.textStrokeOpacity");
	},
	
	createPolicies: function() {
		var policy = this.createPolicy(rmpw.diagrams.render.policies.GraphNode);
		policy.setFigure(this.template.group);
		
		var elId = this.sources.svgRoot.elementId;
		policy = this.createPolicy(rmpw.diagrams.render.policies.GraphNodeLabelLayout);
		policy.setLabel(this.template.wrappingTextExpression);
		policy.setShape(this.template.gfxShape);
		policy.setBackground(this.template.background);
		policy.setLabelHPosition((elId == "Text" || elId == "TextBox") ? 'left' : 'middle');
		policy.setLabelVPosition((elId == "Text" || elId == "TextBox") ? 'top' : 'middle');
		policy.setMinShapeWidth(0);
		policy.setMinShapeHeight(0);
	},
	
	setText: function( font, fill, stroke ){
		
		if( font ){   
			this.template.wrappingTextExpression.setFont(font);
		}
		
		if( fill ){ 
			this.template.wrappingTextExpression.setFill(fill);
		}
		
		if( stroke ){
			this.template.wrappingTextExpression.setStroke(stroke);
		}
	},
	
	applyModelChanges: function() {
		this.inherited(arguments);
		
		var traces = this.traces;
		
		if(!traces.text.synced) {
			this.template.wrappingTextExpression.setShape({
				text: traces.text.get()
			});
			traces.text.reset();
		}
		
		if(!traces.fontSize.synced || !traces.fontFamily.synced || !traces.fontWeight.synced || !traces.fontStyle.synced ||
			!traces.textColor.synced || !traces.textOpacity.synced || !traces.stroke.synced || !traces.strokeOpacity.synced) {
			
			var fontsize = traces.fontSize.get();
			if( fontsize && fontsize.indexOf("pt", 0) < 0 ){
				fontsize = fontsize + "pt";
			}
			
			var font = { 
				family: traces.fontFamily.get(), 
				size: fontsize, 
				weight: traces.fontWeight.get(),
				style: traces.fontStyle.get()
			};
			
			var style = new rmps.web.comment.sketch.serializer.SVGStyle();
			
			style.parseColor("FILL", traces.textColor.get());	
			var fill = style.getFill();
			var fOpacity = traces.textOpacity.get();
			if( fOpacity >= 0 ){
				fill.a = traces.textOpacity.get();
			}
			
			style.parseColor("STROKE", traces.textStroke.get());
			var stroke = style.getStroke();
			var sOpacity = traces.textStrokeOpacity.get();
			if( sOpacity >= 0 ){
				stroke.a = sOpacity;
			}
			
			this.setText( font, fill, stroke );		
			
			traces.fontSize.reset();
			traces.fontFamily.reset();
			traces.fontWeight.reset();
			traces.fontStyle.reset();
			traces.textColor.reset();
			traces.textOpacity.reset();
			traces.textStroke.reset();
			traces.textStrokeOpacity.reset();
		}
	}
});