/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('rmpw.dsv.editor.visualization.render.QVTr');

dojo.require('rmpw.diagrams.render.policies.MatchSize');

/**
 *
 * @author Marc Gobeil
 */
rmpw.dsv.editor.visualization.render.QVTr = {
	ExtractGraphicsTemplate: function(renderMapping, parentShape, objTemplate) {
		var shape;
		var props = this._collectProperties(objTemplate);
		var style = props.style;
		if(style) {
			style = this._collectProperties(style);
			delete props.style;
		}
		switch(objTemplate.referredClass.name) {
			case 'Group':
				shape = parentShape.createGroup();
				
				var outline = shape.createRect();
				outline.setStroke({color: '#ccc', style: 'ShortDash'});
				if (dojox.gfx.svg) {
					outline.rawNode.setAttribute("shape-rendering", "crispEdges");
				}
				
				var contents = shape.createGroup();
				var parts = objTemplate.part;
				if(parts) {
					for(var i = 0, part; part = parts[i]; i++) {
						if(part.referredProperty.name == 'child' && part.value.rdf_type == 'ObjectTemplateExp') {
							this.ExtractGraphicsTemplate(renderMapping, contents, part.value);
						}
					}
				}

				var policy = renderMapping.createPolicy(rmpw.diagrams.render.policies.MatchSize);
				policy.setMaster(contents);
				policy.setSlave(outline);
				policy.enforce();
				break;
				
			case 'Text':
				var textValue = props.text;
				if(textValue.rdf_type == 'VariableExp') {
					textValue = textValue.name;
				} else {
					console.warn('rendering of text property of type ' + textValue.rdf_type + ' not implemented');
					textValue = '[' + textValue.rdf_type + ']';
				}
				
				var font = {family: "sans-serif", weight: "normal", size: "12px"};
				shape = parentShape.createText({
					y: rmpw.diagrams.render.util.Workarounds.getTextBaselineHeight(textValue, font),
					text: textValue});
				shape.setFill('black');
				shape.setFont(font);
				break;
				
			case 'Rect':
				shape = parentShape.createRect(props);
				break;
				
			case 'Image':
				if(dojo.isObject(props.height)) {
					props.height = 16;
				}
				if(dojo.isObject(props.width)) {
					props.width = 16;
				}
				if(!dojo.isString(props.src)) {
					props.src = 'broken.gif';
				}
				shape = parentShape.createImage(props);
				break;
				
			case 'Line':
				shape = parentShape.createLine(props);
				break;

			case 'Path':
				shape = parentShape.createPath(props);
				break;

			case 'Polyline':
				shape = parentShape.createPolyline(props);
				break;
				
			case 'Circle':
				shape = parentShape.createCircle(props);
				break;
				
			default:
				console.error("Unhandled shape " + objTemplate.referredClass.name);
		}
		if(shape) {
			if(style) {
				if(style.strokeColor || style.strokeWidth) {
					shape.setStroke({color: style.strokeColor, width: style.strokeWidth});
				}
				if(style.fillColor) {
					shape.setFill(style.fillColor);
				}
				if(style.fill) {
					var fill = this._collectProperties(style.fill);
					var stops = this._collectProperties(fill.stop);
					fill.type = 'linear';
					fill.colors = [stops];
					shape.setFill(fill);
				}
			}
			renderMapping.template['dynamic_' + objTemplate.name] = shape;
		}
		return shape;
	},


	ExtractRenderPolicies: function(renderMapping, templateExp) {
		var parts = templateExp.part;
		if(parts) {
			for(var i = 0, policyPart; policyPart = parts[i]; i++) {
				var policyTemplate = policyPart.value;
				var policyName = policyTemplate.eType.name;
				
				if(policyName == 'StackLayout') {
					var policyConstructor = new Function('return rmpw.diagrams.render.policies.' + policyName)();
					var policy = renderMapping.createPolicy(policyConstructor); 
					var props = this._collectProperties(policyTemplate);
					for(var prop in props) {
						var varExp = props[prop];
						var setterName = 'set' + prop.substring(0, 1).toUpperCase() + prop.substring(1);
						policy[setterName](renderMapping.template['dynamic_' + varExp.name]);
					}
					console.log(policy, props);
				}
			}
		} 
	},
	
	_collectProperties: function(objTemplate) {
		var properties = {};
		var parts = objTemplate.part;
		if(parts) {
			for(var i = 0, part; part = parts[i]; i++) {
				var value = part.value;
				if(value.rdf_type == 'StringLiteralExp') {
					value = value.stringSymbol;
				} else if(value.rdf_type == 'RealLiteralExp') {
					value = value.realSymbol;
				} else if(value.rdf_type == 'IntegerLiteralExp') {
					value = value.integerSymbol;
				} else if(value.rdf_type == 'EnumLiteralExp') {
					value = value.referredEnumLiteral.name;
				}
				properties[part.referredProperty.name] = value;
			}
		}
		return properties;
	}
};