/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.web.comment.sketch.render.Rectangle");

dojo.require("rmps.web.comment.sketch.render.Primitive");
dojo.require("rmpw.common.transform.Mapping");
dojo.require("rmps.web.comment.sketch.serializer.SVGStyle");
dojo.require("rmps.web.comment.sketch.serializer.SVGUtil");
dojo.require("rmpw.diagrams.render.policies.GraphNode");


dojo.declare("rmps.web.comment.sketch.render.Rectangle", rmps.web.comment.sketch.render.Primitive, {
	
	setSvgRectangle: function(svgRectangle) {
		this.sources.svgRoot = svgRectangle;
		this.sources.svgShape = svgRectangle.rect ? svgRectangle.rect[0] : svgRectangle.text[0];
	},
	
	setGraphicsParent: function(graphicsParent) {
		this.inherited(arguments);
		
		var template = this.template;
		var gfxRectangle;
		
		this.graphicsParent = graphicsParent;
		
		var svgRectangle = this.sources.svgShape;
		gfxRectangle = template.group.createRect();
		
		var style = new rmps.web.comment.sketch.serializer.SVGStyle();			
		style.deriveStyle(this.sources.svgShape);
		gfxRectangle.setStroke( style.getStroke() );		
		gfxRectangle.setFill( style.getFill() );
		
		this.template.gfxShape = gfxRectangle;	
	},
	
	applyModelChanges: function() {
		this.inherited(arguments);

		var anyChanged = false;
		for(var traceKey in this.traces) {
			var trace = this.traces[traceKey];
			if(!trace.synced) {
				anyChanged = true;
			}
			trace.reset();
		}
		
		if (anyChanged) {
			var traces = this.traces;
			
			var style = traces.style.get();	

			var x = traces.x.get();
			var y = traces.y.get();
			var width = traces.width.get();
			var height = traces.height.get();
			
			// let GraphNodeLabelLayout / GraphNode adjust the position
			this.policies.graphNode.setX(x);
			this.policies.graphNode.setY(y);
			this.policies.graphNode.enforce();
			
			this.policies.graphNodeLabelLayout.setWidth(width);
			this.policies.graphNodeLabelLayout.setHeight(height);
			this.policies.graphNodeLabelLayout.enforce();
			
			var stroke = this.template.gfxShape.getStroke();		
			var SVGUtil = new rmps.web.comment.sketch.serializer.SVGUtil();		
			SVGUtil.buildStroke( stroke, traces );
			this.template.gfxShape.setStroke(stroke);
			
			var fill = traces.fill.get();	
			
			if( fill ){
				var fillOpacity = traces.fillOpacity.get();			
				var fillColor = new dojo.Color( fill );			
				fillColor.a = fillOpacity;		
				this.template.gfxShape.setFill(fillColor);
			}
		}
	},
	
	attachListeners: function() {
		this.inherited(arguments);	
		this.connectTraces(this.sources.svgShape, ["x", "y", "width", "height", "style", "stroke", "strokeOpacity", "strokeWidth", "strokeDasharray", "fill", "fillOpacity"]);
	},
	
	createTraces: function() {
		this.inherited(arguments);
		
		this.createTrace("x", "svgShape.x");
		this.createTrace("y", "svgShape.y");
		this.createTrace("width", "svgShape.width");
		this.createTrace("height", "svgShape.height");
		this.createTrace("style", "svgShape.style");
		this.createTrace("strokeWidth", "svgShape.strokeWidth");
		this.createTrace("stroke", "svgShape.stroke");
		this.createTrace("strokeOpacity", "svgShape.strokeOpacity");
		this.createTrace("strokeDasharray", "svgShape.strokeDasharray");
		this.createTrace("fill", "svgShape.fill");
		this.createTrace("fillOpacity", "svgShape.fillOpacity");
	}
});
