/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmps.domains.diagrams.render.RenderReadStore");

dojo.require("dojox.json.ref");
dojo.require("rmpw.common.vcs.ref");

/**
 * TODO: meant to eventually implement the dojo.data.api.Read interface
 */
dojo.declare("rmps.domains.diagrams.render.RenderReadStore", null, {
	constructor: function(viewer) {
		this.viewer = viewer;
	},
	
	loadedEditorCode: {},
	
	renderReady: function(editorURI, renderTransformID) {
		// create store
		dojo.xhrGet({
			url : net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/rdfdirectory/",
			content: {
				logicalURI: encodeURIComponent(modelURI)
			},
			headers: {
				"Accept": "application/x-rdf+json-graph"
			},
			handleAs: "text",
			load: dojo.hitch(this, function(data, ioargs) {
				if(data===null || data==''){
					console.error(modelURI +" is empty!");
					return;
				}
				var modelIndex = {};
				var documents = rmpw.common.vcs.ref.fromJson(data, {
					index: modelIndex,
					idAttribute: "rdf_about",
					// Note (SD): for now loader is empty, implement if we need to support lazy loading
					loader: function() { }
				});
				var model = documents[0];
				
				dojo.withDoc(document, dojo.hitch(this, function() {
					this.createDiagram(model, renderTransformID);
				}));
			}),
			error: dojo.hitch(this, function() {
				console.error(modelURI + " was not loaded");
			})
		});
	},
	
	createDiagram: function(model, renderTransformID) {
		var viewer = this.viewer;

		// initialize repository
		viewer.resource = model;
		viewer.resourceId = modelURI

		var editorPackage = null
		try {
			editorPackage = new Function("return rmpw.dsv.codegen." + renderTransformID)();
		} catch(e) {
			console.error("Error loading generated code: ", e);
			return;
		}
		
		viewer.commandPackage = new editorPackage.domain.Package();
		
		var renderMetadata = editorPackage.render._Metadata.prototype.rootShapesByType;
		// TODO (MG): remove this hack /////////////////////////////
		// find the root type
		for(var key in renderMetadata) {
			renderMetadata = renderMetadata[key];
			break;
		}
		// end hack ////////////////////////////////////////////////
		
		var rootShape = new editorPackage.render[renderMetadata.shapeClass]();
		rootShape[renderMetadata.graphicsSetter](this.viewer.viewport.contents);
		
		// attach model root to root shape
		rootShape[renderMetadata.modelSetter](model);

		// hook for post diagram load initialization
		viewer.setDiagram(rootShape);		// should be RenderRoot.addChild()
		processTimeouts();
		
		var box = rootShape.policies.graphContainer.getTightBounds();
		box.shrink(-10, -10);
		viewer.viewport.pan(-box.x, -box.y);
		viewer.container.setAttributeNS(null, "width", box.width);
		viewer.container.setAttributeNS(null, "height", box.height);
		
		viewer.viewport.surface.rawNode.setAttributeNS(null, "width", box.width);
		viewer.viewport.surface.rawNode.setAttributeNS(null, "height", box.height);
	},
	
	fetch: function() {
		this.viewer.clear();
		
		// load editor code
		if(this.loadedEditorCode[editorURI]) {
			this.renderReady(editorURI, renderTransformID);
		} else {
			this.loadedEditorCode[editorURI] = true;
			var that = this;
			dojo.xhrGet({
				url: net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/dsv/codegen",
				content: { editor: encodeURIComponent(editorURI), isReadOnly: true },
				handleAs: "text",
				load: function(data, headers) {
					for (var i=0, header; header = headers[i]; i++) {
						if (header.getName() == "required-modules" || header.getName() == "extension-modules") {
							var requiredModuleArray = dojox.json.ref.fromJson(header.getValue());
							if (requiredModuleArray) {
								for (var j=0, requiredModule; requiredModule = requiredModuleArray[j]; j++) {
									dojo.require(requiredModule);
								}
							}
						}
					}
					
					dojoLocalEvaluator.evaluate(data);
					that.renderReady(editorURI, renderTransformID);
				},
				error:function(e) {
					console.error(e);
				}
			});

		}
	},
	
	save: function() {
		// do nothing
	},
	
	deleteResource: function() {
		// do nothing
	}
});