/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmps.graphics.render.StackLayout");

dojo.require("rmpw.diagrams.render.util.Workarounds");

/**
 * TODO: abstract x and y into parallel and perpendicular to support both horizontal and vertical stack layouts
 */
dojo.declare("rmps.graphics.render.StackLayout", null, {
	x: 0,
	y: 0,

	constructor: function(config) {
		dojo.mixin(this, config);
		this.width = 0;
		this.height = 0;
		this.figures = [];
	},
	
	register: function(figure) {
		this.figures.push(figure);
	},
	
	clear: function(figure) {
		this.figures.length = 0;
		this.width = 0;
		this.height = 0;
	},
	
	layout: function() {
		var children = this.figures;
		var cursor = 0;
		for(var i=0, len=children.length; i<len; i++) {
			var child = children[i];
			var bounds = rmpw.diagrams.render.util.Workarounds.getBoundingBox(child);
			if(bounds) {
				if(bounds.x != 0 || bounds.y != 0) {
					throw bounds;
				}
				
				var layoutStyle = child.layoutStyle;
				if(layoutStyle) {
					if(layoutStyle.marginLeft) {
						bounds.x += layoutStyle.marginLeft;
					}
					if(layoutStyle.marginTop) {
						bounds.y += layoutStyle.marginTop;
					}
				}
				
				rmpw.diagrams.render.util.Workarounds.applyTransform(child, {dx: this.x + bounds.x, dy: this.y + bounds.y + cursor});
	
				if(layoutStyle) {
					if(layoutStyle.marginBottom) {
						bounds.height += layoutStyle.marginBottom;
					}
					if(layoutStyle.marginRight) {
						bounds.width += layoutStyle.marginRight;
					}
				}
				
				// advance height
				cursor += bounds.height;
				
				// advance width
				if(bounds.x + bounds.width > this.width) {
					this.width = bounds.x + bounds.width;
				}
			}
		}
		this.height = cursor;
	}
});