/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmps.graphics.render.TileViewMapping");

dojo.require("rmpw.common.transform.Mapping");

dojo.require("rmps.graphics.render.StackLayout");
dojo.require("rmpw.diagrams.render.decorations.RectangleShadow");
dojo.require("rmpw.diagrams.render.policies.GraphNode");

/**
 * A node figure that represents an entity
 * 
 * @author Anton McConville
 */
dojo.declare("rmps.graphics.render.TileViewMapping", rmpw.common.transform.Mapping, {
	layout: null,
	
	// TODO: remove, should be defined by the view's style
	border: {color: "#FAD163", width: 2},
	
	setModel: function(model) {
		this.sources.model = model;
	},
	
	createTraces: function() {
		this.inherited(arguments);
	},
	
	attachListeners: function() {
		this.inherited(arguments);
	},
	
	applyModelChanges: function() {
		this.inherited(arguments);
	},
	
	preload: function(){
		var contextParam;
		if (this.sources.model.image.indexOf("rmps.context") < 0) {
			contextParam = "&rmps.context=" + jazz.app.context.get().baseline;
		} else {
			contextParam = "";
		}
		var pic = this.template.figure.createImage( { src: this.sources.model.image + contextParam,
			width: this.sources.model.width, 
			height: this.sources.model.height,
			x: this.sources.model.x,
			y: this.sources.model.y } );
	
		pic.moveToBack();
		
		this.rect.moveToBack();
	},
	
	setParentFigure: function(parentFigure) {
		
		// Create figure template
		var figure = parentFigure.createGroup();
		this.layout = new rmps.graphics.render.StackLayout({x: 1, y: 1});
		this.layout.clear();

		// run layout
		this.layout.layout();
		
		var rect = figure.createShape({
			type: 'rect',
			x: this.sources.model.x,
			y: this.sources.model.y,
			width: this.sources.model.width,
			height: this.sources.model.height 
		});
		
		var color = new dojo.Color( "white" );
		color.a = 0.0001;	
		rect.setFill( color );
		
		this.rect = rect;
		
		//rect.setStroke( {color:"blue"} );
		
		if(dojox.gfx.svg) {
			// fix SVG rendering issue (http://support.adobe.com/devsup/devsup.nsf/docs/50493.htm)
			rect.rawNode.setAttribute('shape-rendering', 'crispEdges');
		}
		
		var template = this.template;
		template.figure = figure;
		template.anchorFigure = rect;

		if( this.sources.model.image != null ){		
//			var image = new Image();
//			image.onload = dojo.hitch( this, "preload" );		
//			image.src= this.sources.model.image;	
			
			this.preload();
		}
		
		// Save attach points
		template = this.template;
		template.figure = figure;
		template.anchorFigure = rect;
	},
	
	createPolicies: function() {
		var policy;	
		policy = this.createPolicy(rmpw.diagrams.render.policies.GraphNode);
		policy.useDojoPosition(true);
		policy.setFigure(this.template.figure);
		// policy.setAnchorFigure(this.template.anchorFigure);
	}
	
});