/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions');

dojo.require("rmps.core.util.ContextUtils");

/////////////////////////////////////// MISC ///////////////////////////////////////////////////////////////

rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.GetRealValue = function(value, defaultValue) {
	return value || defaultValue;
};

rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.GetStringValue = function(value, defaultValue) {
	return value || defaultValue;
};

rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.GetBooleanValue = function(value, defaultValue) {
	return (value != null) ? value : defaultValue;
};

rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.GetHChoiceValue = function(value, defaultValue) {
	if (value != null && value.rdf_about) {
		switch (value.rdf_about) {
		case 'http://jazz.net/ns/dm/sketcher#HorizontalPositions-left':
			return 'left';
		case 'http://jazz.net/ns/dm/sketcher#HorizontalPositions-center':
			return 'center';
		case 'http://jazz.net/ns/dm/sketcher#HorizontalPositions-right':
			return 'right';
		}
	}
	return defaultValue;
};

rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.GetVChoiceValue = function(value, defaultValue) {
	if (value != null && value.rdf_about) {
		switch (value.rdf_about) {
		case 'http://jazz.net/ns/dm/sketcher#VerticalPositions-above':
			return 'above';
		case 'http://jazz.net/ns/dm/sketcher#VerticalPositions-top':
			return 'top';
		case 'http://jazz.net/ns/dm/sketcher#VerticalPositions-center':
			return 'middle';
		case 'http://jazz.net/ns/dm/sketcher#VerticalPositions-bottom':
			return 'bottom';
		case 'http://jazz.net/ns/dm/sketcher#VerticalPositions-below':
			return 'below';
		}
	}
	return defaultValue;
};

rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.GetRoutingValue = function(value, defaultValue) {
	if (value != null && value.rdf_about) {
		switch (value.rdf_about) {
		case 'http://jazz.net/ns/dm/sketcher#Router-oblique':
			return rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.OBLIQUE_ROUTING;
		case 'http://jazz.net/ns/dm/sketcher#Router-orthogonal':
			return rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.RECTILINEAR_ROUTING;
		case 'http://jazz.net/ns/dm/sketcher#Router-tree':
			return rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.TREE_ROUTING;
		}
	}
	return defaultValue;
};

rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.GetSmoothnessValue = function(value, defaultValue) {
	if (value != null && value.rdf_about) {
		switch (value.rdf_about) {
		case 'http://jazz.net/ns/dm/sketcher#Smoothness-none':
			return 'none';
		case 'http://jazz.net/ns/dm/sketcher#Smoothness-less':
			return 'less';
		case 'http://jazz.net/ns/dm/sketcher#Smoothness-normal':
			return 'normal';
		case 'http://jazz.net/ns/dm/sketcher#Smoothness-more':
			return 'more';
		}
	}
	return defaultValue;
};

rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.GetDecorationValue = function(value, defaultValue) {
	if (value != null && value.rdf_about) {
		switch (value.rdf_about) {
		case 'http://jazz.net/ns/dm/sketcher#DecorationType-none':
			return '{}';
		case 'http://jazz.net/ns/dm/sketcher#DecorationType-openArrow':
			return '{path: "M -12 -7 L 0 0 L -12 7", strokeColor: "#989898", strokeWidth: 2.0}';
		case 'http://jazz.net/ns/dm/sketcher#DecorationType-hollowArrow':
			return '{path: "M -12 -7 L 0 0 L -12 7 z", fillColor: "white", strokeColor: "#989898", strokeWidth: 2.0}';
		case 'http://jazz.net/ns/dm/sketcher#DecorationType-filledArrow':
			return '{path: "M -12 -7 L 0 0 L -12 7 z", fillColor: "#989898", strokeColor: "#787878", strokeWidth: 2.0}';
		case 'http://jazz.net/ns/dm/sketcher#DecorationType-hollowDiamond':
			return '{path: "M -9 -5 L 0 0 L -9 5 L -18 0 z", fillColor: "white", strokeColor: "#989898", strokeWidth: 2.0}';
		case 'http://jazz.net/ns/dm/sketcher#DecorationType-filledDiamond':
			return '{path: "M -9 -5 L 0 0 L -9 5 L -18 0 z", fillColor: "#989898", strokeColor: "#787878", strokeWidth: 2.0}';
		case 'http://jazz.net/ns/dm/sketcher#DecorationType-hollowCircle':
			return '{circle: {cx: -5, cy: 0, r:5}, fillColor: "white", strokeColor: "#989898", strokeWidth: 2.0}';
		case 'http://jazz.net/ns/dm/sketcher#DecorationType-filledCircle':
			return '{circle: {cx: -5, cy: 0, r:5}, fillColor: "#989898", strokeColor: "#787878", strokeWidth: 2.0}';
		case 'http://jazz.net/ns/dm/sketcher#DecorationType-hollowBox':
			return '{rectangle: {x:-8, y:-4, width:8, height:8}, fillColor: "white", strokeColor: "#989898", strokeWidth: 2.0}';
		case 'http://jazz.net/ns/dm/sketcher#DecorationType-filledBox':
			return '{rectangle: {x:-8, y:-4, width:8, height:8}, fillColor: "#989898", strokeColor: "#787878", strokeWidth: 2.0}';
		}
	} else if (defaultValue) {
		return '{path: "M -12 -7 L 0 0 L -12 7", strokeColor: "#989898", strokeWidth: 2.0}';		
	}
	return undefined;
};


/////////////////////////////////////// STICK ///////////////////////////////////////////////////////////////
rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.GetStickImage = function(imageUrl) {
	// user supplied image url
	var src = rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.getCustomImageSrc(imageUrl);
	if (!src) {
		// sketchy image
		// Used for removing hard-coded references to /dm
		var dm = net.jazz.ajax._contextRoot;
		src = '[{src:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/stick.gif"},' +  // base shape
	        '{w:67,h:90, src:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/stick5.gif"}]'; // stretched shape
	}
	return src;
};


/////////////////////////////////////// RECTANGLE ///////////////////////////////////////////////////////////////
rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.GetRectangleImage = function(imageUrl) {
	// user supplied image url
	var src = rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.getCustomImageSrc(imageUrl);
	if (!src) {
		var dm = net.jazz.ajax._contextRoot;
		src = '[{src:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/rectangle.gif"},' +                     //base shape
        '{w:180,h:140,src:['+                                                                    //stretched shape
                   '{h:3,tb:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/layhoutline.gif"},'+       //top/bottom
                   '{w:3,lr:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/layvoutline.gif"}]}]';     //left/right
	}
	return src;
};

/////////////////////////////////////// CYLINDER ///////////////////////////////////////////////////////////////

rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.GetCylinderImage = function(imageUrl) {
	// user supplied image url
	var src = rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.getCustomImageSrc(imageUrl);
	if (!src) {
		// else stretchy, sketchy image
		var dm = net.jazz.ajax._contextRoot;
		src =  '[{src:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/cylinder.gif"},'+                     //base shape
		        '{w:118,h:123,src:['+																	// stretched shape
		           '{pc:35,mx:50,t:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/cylindertop.gif"},'+     // top
		           '{pc:20,mx:20,b:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/cylinderbot.gif"},'+     //bottom
		           '{w:3,lr:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/layvoutline.gif"}]}]';           // sides
	}
	return src;
};

/////////////////////////////////////// CLOUD ///////////////////////////////////////////////////////////////

rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.GetCloudImage = function(imageUrl) {
	// user supplied image url
	var src = rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.getCustomImageSrc(imageUrl);
	if (!src) {
		// sketchy image
		var dm = net.jazz.ajax._contextRoot;
		src = '[{src:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/cloud5.gif"}]'    // base & stretched
	}
	return src;
};
/////////////////////////////////////// CIRCLE ///////////////////////////////////////////////////////////////

rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.GetCircleImage = function(imageUrl) {
	// user supplied image url
	var src = rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.getCustomImageSrc(imageUrl);
	if (!src) {
		var dm = net.jazz.ajax._contextRoot;
		src = '[{src:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/circle.gif"},'+                  //base
        '{w:120,h:128, src:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/circle5.gif"}]';    // stretched shape
	}
	return src;
};

/////////////////////////////////////// OVAL ///////////////////////////////////////////////////////////////

rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.GetOvalImage = function(imageUrl) {
	// user supplied image url
	var src = rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.getCustomImageSrc(imageUrl);
	if (!src) {
		// sketchy image
		var dm = net.jazz.ajax._contextRoot;
		src = '[{src:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/oval.gif"},'+                //base
        '{w:155,h:90, src:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/oval5.gif"}]'    // stretched
	}
	return src;
};

/////////////////////////////////////// DIAMOND ///////////////////////////////////////////////////////////////

rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.GetDiamondImage = function(imageUrl) {
	// user supplied image url
	var src = rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.getCustomImageSrc(imageUrl);
	if (!src) {
		// sketchy image
		var dm = net.jazz.ajax._contextRoot;
		src = '[{src:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/diamond.gif"},'+                //base
        '{w:112,h:126, src:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/diamond5.gif"}]';  //stretched
	}
	return src;
};

/////////////////////////////////////// PICTURE ///////////////////////////////////////////////////////////////

rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.GetPictureImage = function(imageUrl) {
	// user supplied image url
	var src = rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.getCustomImageSrc(imageUrl);
	if (!src) {
		// sketchy image
		var dm = net.jazz.ajax._contextRoot;
		src = '[{src:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/picture.gif"},'+               //base
        '{w:134,h:125, src:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/picture5.gif"},'+        //stretch
		//container shape
        '{container:true, src:['+
           '{h:3,tb:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/layhoutline.gif"},'+ //top/bottom
           '{w:3,lr:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/layvoutline.gif"},'+  //left/right
           '{icon:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/picture.gif"}]}]';     //icon
	}
	return src;
};

/////////////////////////////////////// BORDER ///////////////////////////////////////////////////////////////

rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.GetBorderImage = function(imageUrl) {
	var dm = net.jazz.ajax._contextRoot;
	src = '[{src:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/layvoutline.gif", w:3},' +  //vertical
	       '{src:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/layhoutline.gif", h:3}]';   //horitzontal
	return src;
};

/////////////////////////////////////// HELPER ///////////////////////////////////////////////////////////////

/*
 * Need to stub off dependencies into the edit / jazz infrastructure otherwise, server side render will fail.
 * In general - the render layer should respect this.  By design it should be able to exist without any of the
 * edit plug-ins being deployed.
 */
rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.getFileUri = function(uri) {
	if (dojo.isOffscreen) {
		return uri += "?" + _JAZZ_APP_CONTEXT_BASELINE_PARAM;
	}
	else {
		return uri + "?" + rmps.core.util.ContextUtils.computeContextParams();
	}
};

rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.getCustomImageSrc = function(image) {
	if (image && (image.rdf_about || image.$ref)) {
		var imageUrl = rmpw.dsv.editor.sketcher.rendering.SketcherRenderingExtensions.getFileUri(image.rdf_about || image.$ref);
		var dm = net.jazz.ajax._contextRoot;
		return '[{src:"' + imageUrl + '", proportionalShapeMinSize:120, customImage:true },' +
		//container shape
        '{container:true, src:['+
           '{h:3,tb:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/layhoutline.gif"},'+ //top/bottom
           '{w:3,lr:"'+dm+'/web/rmpw.dsv.editor.sketcher/icons/dgm/layvoutline.gif"},'+  //left/right
           '{icon:"'+ imageUrl +'"}]}]';     //icon
	}
	return null;
};
