/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.resource.ResourceIconLabel");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.ResourceLink");

dojo.require("rmps.core.error");

dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.core.util.ContextUtils");

var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;
var DefaultProperties = ['shortName', 'iconUri', 'longName'];
var ContextUtils = rmps.core.util.ContextUtils;

dojo.declare("rmps.ui.resource.ResourceIconLabel", [dijit._Widget, dijit._Templated], {
	
	templateString: '<span class="rmps-ui-resource-resourceIconLabel">' +
			'<span data-dojo-attach-point="iconContainerNode" class="iconContainer"><img data-dojo-attach-point="iconNode" class="icon"></span>' +
			'<span data-dojo-attach-point="labelContainer" class="label"></span>' +
			'<span data-dojo-attach-point="longNameNode" class="longName" style="display: none;"></span>' +
			'</span>',
	
	labelNode: null,
	properties: null,
	resourceUri: null,
	isLink: false,
	linkHref: null,
	hasTooltip: true,
	showLongName: false,
	showIcon: true,
	hasJazzResourceLink: false,
	
	resourceInfo: null,
	_resourceLink: null,
	
	postCreate: function() {
		this.labelNode = this.labelContainer;
		if (this.resourceInfo) {
			this._updateResourceInfo(this.resourceInfo);
		} else if (this.resourceUri) {
			this.setResourceUri(this.resourceUri);
		}		
	},
	
	destroy: function() {
		if (this._resourceLink) {
			this._resourceLink.destroyRecursive();
			delete this._resourceLink;
		}

		this.inherited(arguments);
	},
	
	setResourceUri: function(resourceUri) {
		this.resourceUri = resourceUri;
		var deferred = ResourceInfoClient.getResourceInfo(this.resourceUri, this.properties || DefaultProperties);
		deferred.addCallback(this, function(infos) {
			this._updateResourceInfo(infos[0]);
		});
		deferred.addErrback(function(error) {
			if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
				rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
			} else {
				rmps.core.error.createError(error).dispatch();
			}
		});
	},
	
	onUpdate: function() {},
	
	setResourceInfo: function(resourceInfo) {
		this._updateResourceInfo(resourceInfo);
	},
	
	_hideIconContainer: function() {
		// display = "none" should not be used since that will
		// break the inline-block of the icon container
		this.iconContainerNode.style.visibility = "hidden";
	},
	
	_calculateResourceNumber: function(uri) {
		var number = uri.match(/\/(\d+)$/);
		
		if (number != null && number[1]) {
			return number[1];
		}
		
		return null;
	},
	
	_updateResourceInfo: function(resourceInfo) {
		this.resourceInfo = resourceInfo;
		if (resourceInfo.iconUri && this.showIcon) {
			var errorHandle = this.connect(this.iconNode, "onerror", function() {
				this.disconnect(errorHandle);
				this._hideIconContainer();
			});
			this.iconNode.src = resourceInfo.iconUri;
		} else {
			this._hideIconContainer();
		}

		
		
		var shortName = resourceInfo.shortName;
		var resourceNumber = this._calculateResourceNumber(resourceInfo.uri);
		if (this.isLink) {
			var resourceLink = dojo.doc.createElement("a");
			if (this.linkHref) {
				if (dojo.isFunction(this.linkHref)) {
					resourceLink.href = this.linkHref(resourceInfo);
				} else {
					resourceLink.href = this.linkHref;
				}
			} else {
				resourceLink.href = "";
			}
			
			resourceLink.appendChild(dojo.doc.createTextNode((resourceNumber ? (resourceNumber + ": ") : "") + shortName));
			this.connect(resourceLink, "onclick", function(e) {
				if (!this.linkHref) {
					e.preventDefault();
				}
				this.linkClicked(e);
			});
			
			if (this.showIcon) {
				var spacer = dojo.doc.createElement("span");
				spacer.appendChild(dojo.doc.createTextNode(" "));
				this.labelContainer.appendChild(spacer);
			}
			this.labelContainer.appendChild(resourceLink);
			this.labelNode = resourceLink;
			
			if (this.hasJazzResourceLink && this.linkHref) {
				// ResourceLink will fail resolving a relative DM URI, so using an absolute path for now
				var root = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL;
				var uri = this.resourceUri || resourceInfo.uri;
				var context = ContextUtils.computeContextParams();

				this._resourceLink = new jazz.ui.ResourceLink({
					srcNodeRef: resourceLink,
					uri: root + "/rendering?resourceUri=" + encodeURIComponent(uri) + "&" + context,
					presentationUri: resourceLink.href,
					lazyFetch: true
				});	
			}
		} else {
			this.labelNode = this.labelContainer;
			this.labelContainer.appendChild(dojo.doc.createTextNode((this.showIcon ? " " : "") + (resourceNumber ? (resourceNumber + ": ") : "") + shortName));
		}
				
		if (this.showLongName && (resourceInfo.longName != shortName) && (resourceInfo.longName !== undefined)) {
			var longName = "(" + resourceInfo.longName + ")";
			this.longNameNode.appendChild(dojo.doc.createTextNode(" " + longName));
			this.longNameNode.style.display = "";
		}
		
		if (this.hasTooltip && !this._resourceLink) {
			var title = this.getTooltip();
			if (title) {
				this.labelContainer.title = title;
			}
		}
		this.onUpdate();
	},
	
	getTooltip: function() {
		if (!this.showLongName) {
			return this.resourceInfo.longName;
		}
		return null;
	},
	
	linkClicked: function(event) {
		
	}
});

})();