/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.review.ReviewClient");

dojo.require("rmps.core.review.Review");
dojo.require("rmps.core.review.ReviewResults");
dojo.require("rmps.core.review.ReviewSummary");
dojo.require("dojox.xml.DomParser");
dojo.require("jazz.app.context");
dojo.require("jazz.client.xhr");
dojo.require("rmps.core.RootServices");
dojo.require("rmps.core.review.util.ReviewUtil");

var REVIEW_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "reviews");
var REVIEW_RESULTS_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "reviews") + "/results";
var MULTILINE_TEXT_TAGS = ["rmpsReviews:reviewArtifactInstructions","rmpsReviews:reviewInstructions","rmpsReviews:reviewSummary"];

var addErrorCallback= function(deferred){
	deferred.addErrback(this, function(error){
		if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
			return rmps.core.error.createXhrError(deferred.ioArgs);
		}
		return rmps.core.error.createError(error);
	});
	return deferred;
};

/*Delegate some required jazz.client functions and add the required handling for errors and baseline*/
var CLIENT= {};

CLIENT.xhrGet= dojo.hitch(CLIENT, function(args){
	return addErrorCallback(jazz.client.xhrGet(args));
});

CLIENT.xhrPost= dojo.hitch(CLIENT, function(args){
	return addErrorCallback(jazz.client.xhrPost(args));
});

CLIENT.xhrPut= dojo.hitch(CLIENT, function(args){
	return addErrorCallback(jazz.client.xhrPut(args));
});

CLIENT.xhrDelete= dojo.hitch(CLIENT, function(args){
	return addErrorCallback(jazz.client.xhrDelete(args));
});

/* 
 * Provide GET/POST/PUT operations on Review/ReviewResults Objects
 * 
 */
rmps.core.review.ReviewClient = {
	updateReviewHeaders: function(review) {
		var args = {
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			url: review.resourceUri,
			handleAs: "text",
			load: function(response, ioArgs)
			{
				review.eTag = ioArgs.xhr.getResponseHeader("ETag");
			}
		};
		return CLIENT.xhrGet(args);
	},

	getReview: function(reviewUri, currentUser) {
		var args = {
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			url: reviewUri,
			headers: {
				"Accept": "application/rdf+json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs)
			{
				var review = new rmps.core.review.Review();
				review.parse(response, ioArgs);
				review.uri = reviewUri;
				review.resourceUri = reviewUri;
				review._currentUser= currentUser;
				review.lastModified = new Date(ioArgs.xhr.getResponseHeader("Last-Modified"));
				
				// FIXME: Added because the DomParser removes the \n inside the XML nodes
				// iterate over all the mutlti-lines tags to replace hte \n by <br>
//				var updResp = response;
//				if(ioArgs.handleAs!== "json")
//				for ( var i = 0; i < MULTILINE_TEXT_TAGS.length; i++) {
//					updResp = rmps.core.review.util.ReviewUtil.replaceLineBreak(updResp,MULTILINE_TEXT_TAGS[i]);					
//				}
				return review;
			}
		};
		var deferred= CLIENT.xhrGet(args);
		deferred.addCallback(this, function(review) {
			rmps.core.review.util.ReviewUtil.setReviewCurrentUser(review);
		});
		return deferred;
	},
	
	deleteReview: function(reviewUri) {
		var args = {
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			url: reviewUri,
			handleAs: "xml"
		};
		return CLIENT.xhrDelete(args);
	},
		
	createEmptyReview: function(review, name) {
		var projectId = jazz.app.context.get().itemId;
		var queryParams = {
				"projectId": projectId,
				"name": name
		};
		
		var args = {
			headers: {
				"X-ibm-rmps-internal": "true",
				"Content-Type": "application/rdf+xml",
				"Accept": "application/atom+json"
			},
			url: REVIEW_PATH + "?" + dojo.objectToQuery(queryParams),
			handleAs: "json",
			load: dojo.hitch(this, function(response, ioArgs)
			{
				review.created = new Date(ioArgs.xhr.getResponseHeader("created"));
				review.eTag = ioArgs.xhr.getResponseHeader("ETag");
				var reviewUri = ioArgs.xhr.getResponseHeader("Location"); 
				review.uri = reviewUri;
				review.resourceUri = reviewUri;
				review.reviewUri = reviewUri; 
				return review;
			})
		};
		return CLIENT.xhrPost(args);
	},

	createReview: function(review) {
		var projectId = jazz.app.context.get().itemId;
		var queryParams = {
			"projectId": projectId
		};
		
		if(review) {
			review.created = new Date();
		}
		var args = {
			headers: {
				"X-ibm-rmps-internal": "true",
				"Content-Type": "application/rdf+xml",
				"Accept": "application/atom+json"
			},
			url: REVIEW_PATH + "?" + dojo.objectToQuery(queryParams),
			handleAs: "json",
			hasBody: true,
			postData: review ? review.toXMLString() : null,
			load: dojo.hitch(this, function(response, ioArgs)
			{
				review.created = new Date(ioArgs.xhr.getResponseHeader("created"));
				review.eTag = ioArgs.xhr.getResponseHeader("ETag");
				var reviewUri = ioArgs.xhr.getResponseHeader("Location"); 
				review.uri = reviewUri;
				review.resourceUri = reviewUri;
				review.reviewUri = reviewUri; 
				if (response){
					review.parse(response, ioArgs);
				}
				return review;
			})
		};
		return CLIENT.xhrPost(args);
	},
	
	updateReview: function(review, action) {
		var projectId = jazz.app.context.get().itemId;
		function doUpdateReview(eTag) {
			var queryParams = {
				"action" : action
			};
			var args = {
				headers: {
					"X-ibm-rmps-internal": "true",
					"Content-Type": "application/rdf+xml",
					"Accept": "application/rdf+json",					
					"If-Match": eTag,
					"ETag": eTag
				},
				url: review.uri + "?" + dojo.objectToQuery(queryParams),
				handleAs: "json",
				hasBody: true,
				putData: review.toXMLString(),
				load: dojo.hitch(this, function(response, ioArgs)
				{
					review.eTag = ioArgs.xhr.getResponseHeader("ETag");
					if (response){
						review.parse(response, ioArgs);
					}
					var resultDeferred = rmps.core.review.ReviewClient.getReviewResult(review);
					resultDeferred.addCallback(dojo.hitch(this, function(reviewResults) {
						review._reviewResults = reviewResults;
						return resultDeferred;
					}));
				})
			};
			return CLIENT.xhrPut(args);
		}
		
		if (review.eTag) {
			return doUpdateReview(review.eTag);
		}
		
		var headerDeferred = this.getReviewHeaders(review);
		headerDeferred.addCallback(doUpdateReview);
		return headerDeferred;
	},

	getReviews: function(projectId, userId, page, pageSize, pageToken) {
		var queryParams = {
			"projectId": projectId,
			"pageSize" : -1
		};

		if (userId) {
			queryParams.userId = userId;
		}
		if (page != undefined && page != null) {
			queryParams.page = page;
		}
		if (pageSize) {
			queryParams.pageSize = pageSize;
		}
		if (pageToken) {
			queryParams.pageToken = pageToken;
		}
		
		var args = {
			url: REVIEW_PATH + "?"
				+ dojo.objectToQuery(queryParams),
			headers: {
				"X-ibm-rmps-internal": "true",
				"Accept": "application/atom+json"
			},
			handleAs: "json",
			load: function(response, ioArgs)
			{
				var result = [];
				var entries = response.entry;
				
				result.pageToken = "";
				result.totalResults = parseInt(response["opensearch:totalResults"][0].value);
				
				if (entries && entries.length > 0) {
					for (var i = 0; i < entries.length; i++) {
						var revSummary = new rmps.core.review.ReviewSummary();
						revSummary.parseEntry(entries[i], ioArgs);
						result.push(revSummary);
					}
				}
				
				var links = response.link;
				if (links && links.length > 0) {
					var getParamater= function(/*String */ url, /*String */ key) {
				    	var query= new dojo._Url(url).query;
				    	var object= dojo.queryToObject(new dojo._Url(url).query);
				    	return object[key];
				    };
					for (var l = 0; l < links.length; l++) {
						var link = links[l];
						if (link.rel == "rel") {
							result.pageToken = getParamater(link.href, "pageToken");
							break;
						}
					}
				}
				return result;
			}
		};
		return CLIENT.xhrGet(args);
	},
	
	getReviewResult: function(reviewParser) {
		var resultId = reviewParser.getReviewId();
		var resultsUri = REVIEW_RESULTS_PATH + "/" +  resultId;		
		var args = {
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			url: resultsUri,
			handleAs: "text",
			load: function(response, ioArgs)
			{
				var reviewResult = new rmps.core.review.ReviewResults({
					reviewParser: reviewParser
					});
				reviewResult.parse(dojox.xml.DomParser.parse(response), ioArgs);
				return reviewResult;
			}
		};
		return CLIENT.xhrGet(args);
	},
	
	castArtifactVotes: function(userUri, reviewParser, hasResetResource) {
		var createResultInfos = function(userURI, changedArtifactInfos) {
			//FIXME: Look here
			return (
				dojo.map(changedArtifactInfos, function(artifactInfo) {
						return {
							artifact: artifactInfo.id,
							artifactStatusType: artifactInfo.myArtifactStatusType,
							userURI: userURI
						};
					}
				)
			);
		};
		var xmlContent = reviewParser._reviewResults.getResultsContentFor(userUri);
		var deferred = rmps.core.review.ReviewClient.updateReviewResult(reviewParser, userUri, xmlContent, hasResetResource);
		deferred.addCallback(function(response) {
			return response;
		});
		return deferred;
	},
	
	updateReviewResult: function(reviewParser, userUri, xmlContent, hasResetResource) {
		var projectId = jazz.app.context.get().itemId;
		
		var myCurrentResult = reviewParser.getUserResults(userUri);
		
		function getETag(ioArgs) {
			return ioArgs.xhr.getResponseHeader("ETag");
		}
		function doUpdateReviewResult(ioArgs) {
			var eTag = ioArgs.xhr.getResponseHeader("ETag");
			var url = myCurrentResult.resourceURI;
			if (hasResetResource) {
				url += "?action=resetResource";
			}

			var args = {
				headers: {
					"X-ibm-rmps-internal": "true",
					"Content-Type": "application/rdf+xml",
					"If-Match": eTag,
					"ETag": eTag
				},
				url: url,
				handleAs: "xml",
				hasBody: true,
				putData: xmlContent,
				load: dojo.hitch(this, function(response, ioArgs)
				{
					myCurrentResult.eTag = ioArgs.xhr.getResponseHeader("ETag");
					var review = new rmps.core.review.ReviewResults({
						reviewParser: reviewParser
					});
					review.parse(dojox.xml.DomParser.parse(ioArgs.query), ioArgs);
					var currentUserUri = userUri;
					return review;
				})
			};
			return CLIENT.xhrPut(args);
		}
		
		var deferred = this.getReviewResultHeaders(myCurrentResult.resourceURI);
		deferred.addCallback(this,doUpdateReviewResult);
		var func = (dojo.hitch(rmps.core.review.ReviewClient, rmps.core.review.ReviewClient.updateReviewHeaders, reviewParser));
		deferred.addCallback(func);
		return deferred;
	},
	
	
	getReviewResultHeaders: function(resultURI) {
		var args = {
			headers: {
					"X-ibm-rmps-internal": "true"
			},
			url: resultURI,
			handleAs: "text",
			load: dojo.hitch(this, function(response, ioArgs){
				return ioArgs;
			})
		};
		return CLIENT.xhrGet(args);
	},
	
	getReviewHeaders: function(review, sync) {
		var args = {
			headers: {
					"X-ibm-rmps-internal": "true"
			},
			url: review.uri,
			sync: sync,
			handleAs: "text",
			load: dojo.hitch(this, function(response, ioArgs)
			{
				return ioArgs;
			})
		};
		return CLIENT.xhrGet(args);
	}
};

})();