/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.review.ReviewController");

dojo.require("rmps.core.review.util.ReviewUtil");
dojo.require("rmps.core.review.ReviewClient");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmps.core.util.ContextUtils");
dojo.require("rmps.core.base.BaseClientUtil");
dojo.require("rmps.core.process.SecurityClient");
dojo.require("rmps.core.error");
dojo.require("rmps.core.util.xhrUtil");
dojo.require("dojo.DeferredList");
dojo.require("rmps.core.user.UserClient");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.core.review.util.ReviewTypes");
dojo.require("jazz.ui.Dialog");
dojo.require("rmps.core.changesets.ChangesetClient");
dojo.require("rmps.core.changesets.Changeset");


dojo.requireLocalization("rmps.core.review", "ReviewControllerMessages");
var messages = dojo.i18n.getLocalization("rmps.core.review", "ReviewControllerMessages");

var ReviewUtil = rmps.core.review.util.ReviewUtil;
var ReviewClient= rmps.core.review.ReviewClient;
var ReviewTypes = rmps.core.review.util.ReviewTypes;
var ResourceInfoClient= rmps.core.resource.ResourceInfoClient;
var BaseClientUtil= rmps.core.base.BaseClientUtil;
var SecurityClient= rmps.core.process.SecurityClient;
var IRmpsConstants= rmps.core.util.IRmpsConstants;
var UserClient = rmps.core.user.UserClient;
var ContextUtils= rmps.core.util.ContextUtils;
var ChangesetClient = rmps.core.changesets.ChangesetClient;

var DEFAULT_BASELINE= "Default";
var NEW_REVIEW_EDITOR= "rmps.app.newReviewEditor";

var PROCESS= {
		Operations: {
			ALL_AUTHORING: "com.ibm.xtools.rmps.process.operations.review.authoring",
			ALL_FEEDBACK: "com.ibm.xtools.rmps.process.operations.review.feedback",
			DELETE: "com.ibm.xtools.rmps.process.operations.review.authoring~#_~#delete",
			MODIFY: "com.ibm.xtools.rmps.process.operations.review.authoring~#_~#modify",
			CREATE: "com.ibm.xtools.rmps.process.operations.review.authoring~#_~#create",
			VOTE: "com.ibm.xtools.rmps.process.operations.review.feedback~#_~#vote"
			
		},

		Actions: {
			CREATE: "create",
			DELETE: "delete",
			MODIFY: "modify",
			VOTE: "vote"
		}
};

dojo.declare("rmps.core.review._ReviewController", null, {

	/*Current ReviewFor the update session */
	review: null,
	
	/*Required to avoid making too many calls to get the users information when making refreshes at other places */
	usersCache: null,
	
	/*Required to avoid making too many calls to get the resources information when making refreshes at other places */
	resourcesInfoCache: null,
	
	/*A flag to determine if the current review has changes or not */
	isDirty: null,
	
	/*The owing context for the controlled review */
	context: null,
	
	/*The resource uri for the review we are controlling */
	resourceUri: null,
	
	/* The main view baseline. Required to handle information that we can display */
	baseline: null,
	
	/* A flag to determine if the displayed review is an existing review (not a clone or a new review) */
	isExistingReview: null,
	
	/* A flag to determine if the configuration associated to the review still exist */
	configNotExist: false,
	
	/**********************************************************APIs *************************************************************************/
	
	/* 
	 * Update an attribute in the review object, and make sure to update the dirty status, and notify all viewers when a dirty status change occurs 
	 */
	updateReview: function(key, value){
		this.review.setReviewAttribute(key, value);
		this.updateDirtyStatus();			
		this.onReviewUpdate(key, value);
	},
	
	addResource: function(resourceInfo){
		var baseline= this.review.getReviewAttribute("reviewBaseline");
		var infos= this.resourcesInfoCache[baseline?baseline: "Default"];
		
		if(!infos){infos= this.resourcesInfoCache[baseline?baseline: "Default"]= [];}
		infos.push(resourceInfo);
		
		this.review.addNewResource(resourceInfo);
		this.refreshParticipants();
		this.updateDirtyStatus();
		this.onReviewResourceAdd(resourceInfo);
	},
	
	/* 
	 * Update a an attribute value for a given resource in the review. Then, make sure to notify vieers to make changes in the actions field, and the recently 
	 * modified key, so they could update the UI with the new value.
	 *  
	 */
	updateResource: function(resourceURI, key, value){
		this.review.updateResource(resourceURI, [{"field": key, "value": value},{"field": "Actions"}]);
		this.updateDirtyStatus();
		this.onReviewResourceUpdate(resourceURI, key, value);
	},
	
	/*
	 * Remove resource given its identifier (resource uri), and update all UI associated and refresh them partially
	 */
	removeResource: function(resourceUri){
		this.review.removeResource(resourceUri);
		this.refreshParticipants();
		this.updateDirtyStatus();
		this.onReviewResourceRemove(resourceUri);
	},
	
	/* 
	 * Rset the results for a given resource uri. Make sure to notify all viewers about that reset action 
	 */
	resetResource: function(resourceUri) {
		this.hasResetResource = true;
		this.review.resetResourceResultsFor(resourceUri);
		this.refreshResources(["artifact","resourceUri","resourceResultStatus"]);
		this.refreshParticipants();
		this.updateDirtyStatus();
		this.onReviewResourceReset(resourceUri);
	},
	
	/* 
	 * Add new user(s) to the review 
	 */
	addReviewer: function(users, roleType){
		dojo.forEach(users, dojo.hitch(this, function(user) {
				var userId= user.userId;
				var uri= rmps.core.user.UserClient.getUserUri(userId);
				if(!this.getUserInfo(uri)){
					this.usersCache[uri]= user;						
				}
				this.review.addNewParticipant(uri, roleType);
			})
		);
		this.refreshResources(["artifact","resourceUri","resourceResultStatus"]);
		this.updateDirtyStatus();
		dojo.publish("review.view.editor.refresh");
		this.onReviewReviewerAdd(users, roleType);
	},
	
	/* 
	 * Submit (vote) for a specific element  
	 */
	submitResult: function(resourceUri, statusType){
		this.review.voteForResource(this.review.myUserURI, resourceUri,  statusType);
		this.updateDirtyStatus();
		this.onReviewResultSubmit(resourceUri, statusType);
	},
	
	
	/*
	 * Update reviewer given its identifier (user uri), and update all UI associated and refresh them partially
	 */
	updateReviewer: function(userUri, valuesList){
		this.review.updateParticipant(userUri, valuesList);
		this.refreshResources(["artifact","resourceUri","resourceResultStatus"]);
		this.updateDirtyStatus();
		dojo.publish("review.view.editor.refresh");
		this.onReviewReviewerUpdate(userUri, valuesList);
	},
	
	/*
	 * Remove participant given its identifier (user uri), and update all UI associated and refresh them partially
	 */
	removeReviewer: function(userUri){
		this.review.removeParticipant(userUri);
		this.refreshResources(["artifact","resourceUri","resourceResultStatus"]);
		this.updateDirtyStatus();
		dojo.publish("review.view.editor.refresh");
		this.onReviewReviewerRemove(userUri);
		
		/*Make sure to remove the user info from the cache list */
		delete this.usersCache[userUri];
	},
	
	/* 
	 * Request the review controller to make check and see if we need to update the current dirty status. The flag statusChange is passed to clarify if this
	 * change in the review editor has just changed the review dirty status or not 
	 * 
	 * 
	 *  ignoreReviewUpdate: If ture, then we will not make a check and update other related review information (such as review status)
	 */
	updateDirtyStatus: function(ignoreReviewUpdate){
		if(!ignoreReviewUpdate){
			this._updateReviewInformation();				
		}
		var reportsList= [];
		var resultsReport= BaseClientUtil.compareModels(this.review._reviewResults.model, this.clonedResultsModel, this.review._reviewResults.form);
		var reviewReport= BaseClientUtil.compareModels(this.review.model, this.clonedModel, this.review.form);
//		var reviewReport= BaseClientUtil.compareModels(this.review.model, this.clonedResultsModel, this.review._reviewResults.form);
		
		if(resultsReport){
			reportsList.push(resultsReport);
		}
		if(reviewReport){
			reportsList.push(reviewReport);
		}
		var statusChange;
		if(reportsList.length>0){
			statusChange= !this.isDirty;
			this.isDirty= true;
			this.addDirtyNotifier(statusChange, reportsList);
		}else{
			statusChange= this.isDirty;
			this.isDirty= false;
			this.removeDirtyNotifier(statusChange);
		}
	},
	
	/*
	 * Get info for a given user uri. 
	 * It only gets results for the currently added users. When the user is removed, we can no longer get its info. It's very rare to change the user info
	 * from the sever side, but we decided to remove the user info once we remove it during the review updating session in order to avoid having old information
	 *  
	 */
	getUserInfo: function(userUri){
		return this.usersCache[userUri];
	},
	
	/* 
	 * Returns the resource infos for the current added resources in the review. 
	 * If baseline is undefined, then, we return the non-baselined resource infos 
	 * If baseline is a string (a baseline value), then we return the resource ibos for that specific baseline
	 * If baseline is not string (we assume that is is a boolean value true, then we return the current review baselined resource infos (based on
	 * the current review baseline value
	 * 		i.e.: If we want to return the resource infos for the current review baseline: getResourceInfos(true);
	 * 			  If we want to return the resource infos without basedline parameter: getResourceInfos();
	 * 			  If we want to return the resource infos or specific baseline: getResourceInfos(myBaselineValue);
	 * 
	 * resourceUri: Optional paramater. If not set, then, we return all resource infos, if defined, we return the resource info for the specified resource uri
	 */
	getResourceInfos: function(baseline, resourceUri){
		var values;
		if(!baseline){
			values= this.resourcesInfoCache[DEFAULT_BASELINE];
			return values?values:[];
		}
		if(dojo.isString(baseline)){
			values= this.resourcesInfoCache[baseline];
			if(!values){
				return [];
			}
			return this._getResourceInfo(values,resourceUri);	
		}
		var reviewBaseline= this.review.getReviewAttribute("reviewBaseline");
		values= this.resourcesInfoCache[reviewBaseline?reviewBaseline:DEFAULT_BASELINE];
		if(!values){
			return [];
		}
		return this._getResourceInfo(values, resourceUri);
	},
	
	/* 
	 * Return a list for the IDs for the users who are already added to the review object 
	 */
	getSelectedUserIds: function() {
		var selectedUserIds = [];
		dojo.forEach(this.review.model.participants.dataSet, dojo.hitch(this, function(participantInfo) {
				var userURI = participantInfo.contributor;
				var userId = ReviewUtil.getUserId(userURI);
				selectedUserIds.push(userId);
			}
		));
		return selectedUserIds;
	},
	
	/* 
	 *  We should call for this method to change the location in order to make sure that the navigation will be done by a single entry point (the main container).
	 *  Otherwise, the places that use the review manager would be having internal page redirection
	 */
	navigateTo: function(url){
		window.location.href = url;
	},
	
	/* 
	 * We have to povide the fully qualifed url for resources as in some places, we could be not aware of the current context. In addition, we have to make sure
	 * that the used context (base url) would be the same as the review updater single entry
	 */
	getFullUrl: function(url){
		return this.getProjectPrefix()+ this.getViewResourceUri(url);
	},
	
	getViewResourceUri: function(resourceUri)  {
		var obj = {"resource": resourceUri};
		return "#action=rmps.app." + "design.edit&" + dojo.objectToQuery(obj);
	},
	
	getProjectPrefix: function(){
		return net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL+ "/web/projects/"+ this.context.name+window.location.search;
	},
	
	/*
	 * Provde a partial refrsh for the participants references in all places in the editor. Such as participants tab, and pariticpants section in the main tab
	 * In a way avoids the re-cretion of widgets
	 * 
	 */
	refreshParticipants: function(){
		/*Make sure to update the participant data feed */
		var participantInfos = this.review.getUsersResults();
		for (var i = 0; i < participantInfos.length; i++) {
			var participantInfo = participantInfos[i];
			var curUserURI = participantInfo.userURI;
			this.refreshParticipant(curUserURI);
		}
	},

	/*
	 * Provde a partial refrsh for the participant references in all places in the editor. @see:refreshParticipants
	 */
	refreshParticipant: function(userUri){
		this.refreshParticipantFields(userUri, ["reviewRole","result","artifactInfo"]);
	},

	refreshParticipantFields: function(userUri, fields){
		var dataSet= this.review.model.participants.dataSet;
 		for (var i = 0; i < dataSet.length; i++) {
	 		var data = dataSet[i];
	 		if(data.contributor== userUri){
	 			for(var index=0; index< fields.length; index++){
	 				var field= fields[index];
	 				this.review.model.participants.updateDataEntry(userUri, field, data[field], data);
	 			}
	 		}
 		}
	},


	/*
	 * Provde a partial refrsh for the resources references in all places in the editor. Such as resources tab, and resources section in the main tab
	 * In a way avoids the re-cretion of widgets
	 * 
	 * Provde a partial refrsh for the resources references in all places in the editor. @see:refreshResources
	 */
	refreshResources: function(updaeFileds){
		/*Make sure to update the participant data feed */
		var resources = this.review.getReviewResources();
		for (var i = 0; i < resources.length; i++) {
			var resource= resources[i];
			var identifier= resource.resourceUri;
			for(var subIndex=0; subIndex<updaeFileds.length; subIndex++){
				this.review.model.resources.updateDataEntry(identifier, updaeFileds[subIndex], identifier);	
			}
		}
	},
	
	/* 
	 * If we want to save the changes made on the review. After this method success, the origianl reivew object will be updated with the new value 
	 */
	releaseUpdates: function(taskMonitor){
		this.onSave();
		
		var deferreds= [];
		
		var getCallback= dojo.hitch(this, function(){
			var mainDeferred= new dojo.DeferredList(deferreds,false, true, false);
			mainDeferred.addCallback(dojo.hitch(this, function (response) {
				this._reinit();
				this.onSaveSuccess(response);
			}));
			return mainDeferred;
		});
		
		var href= window.location.href;
		/*Avoid submitting results if this is the first time to create the review */
		var resUri = href.substring(href.indexOf("action=")+7, href.length);
						
		var updateReview= dojo.hitch(this, function(){
			var resultsReport= BaseClientUtil.compareModels(this.review._reviewResults.model, this.clonedResultsModel, this.review._reviewResults.form);
			if(resultsReport){
				if(this.isExistingReview){
					var results= resultsReport["rdf:Description,rmpsReviews:ParticipantResult"];
					if(results){
						for(var att in results){
							var current= this.review._reviewResults.getResults(att);
							if(current){
								deferreds.push(ReviewClient.castArtifactVotes(current.userURI, this.review, this.hasResetResource));
							}
						}	
					}
					this.hasResetResource = false;
					return getCallback();
				}
			}		
		});
		
		/*Only make changes on the review itself (away from submitting results) if the current user is the moderator */
		var reviewReport= BaseClientUtil.compareModels(this.review.model, this.clonedModel, this.review.form);
		if(reviewReport|| ! this.isExistingReview){
			/*Review Status is a special case. Yes, we change it while working on the UI side, but at this place, we have to make sure that we apply the
			 * original value, as the server is responsible about making the status transioining. Otherwise, an excpetion will be thrown */
			var reviewStatus = this.clonedModel.reviewStatus;
			if( reviewStatus != ReviewTypes.REVIEW_STATE_TYPES.Started && reviewStatus != ReviewTypes.REVIEW_STATE_TYPES.Reviewed) {
				/*Get the original status value, and set the active value with it, so the server will react with the current state*/
				this.review.setReviewAttribute(reviewStatus, "reviewStatus");
				var task = taskMonitor.beginTask("", 1);		
				var deferred= ReviewClient.updateReview(this.review);
				task.handleDeferredWork(deferred, 1);
				deferred.addCallback(dojo.hitch(this, function (response) {
					this._reinit();
					this.onSaveSuccess(response);
				}));
				deferred.addErrback(dojo.hitch(this, function (error) {
					/*We do not re-init on errors */
					this.onSaveError(error);
				}));
				deferreds.push(deferred);
				if (this.hasResetResource)
					updateReview();
			}else{
				updateReview();
			}
		}else{
			if(this.isVoteEnabled()||this.review.isModerator()){
				updateReview();
			}
		}
		return getCallback();
	},
	
	/* 
	 * Check and see if the current configuration is the same as review 
	 */
	isConfigurationMatches: function(){
		var matches = false;
		var reviewContext = this.review.getReviewAttribute("reviewBaseline");
		var currentChangeset = jazz.app.context.get().changeset;
		if (reviewContext){
			if (this.isChangeset(reviewContext)){
				// FIXME: temporary hack until we use directly VVC changeset configurations
				var currentCs = jazz.app.context.get().changeset;
				if (currentCs){
					matches = (this._getChangesetId(currentCs) === this._getChangesetId(reviewContext));
				}
			} else {
				matches = (this.baseline=== reviewContext && !currentChangeset);
			}
		} else {
			matches = (ContextUtils.getContextType(jazz.app.context.get().baseline)=== rmps.core.util.ContextTypes.WORKSPACE && !currentChangeset);
		}
		return matches;
	},

	/*
	 * Allows to bypass the configuration match between the current configuration and the one associated to the review
	 */
	ignoreConfigurationMatches: function(){
		var ignore = false;
		
		if (this.review.getReviewAttribute("reviewStatus") === ReviewTypes.REVIEW_STATE_TYPES.Draft ||
				(this.review.getReviewAttribute("reviewStatus") === ReviewTypes.REVIEW_STATE_TYPES.Finalized && this.configNotExist)){
			ignore = true;
		}
		return ignore;
	},
	
	/*
	 *  Retrieves the context associated to the passed 
	 */
	
	getReviewContext: function(){
		var deferred;
		var review= this.review;
		var reviewContext= review.getReviewAttribute("reviewBaseline");
		if (!reviewContext || dojo.trim(reviewContext) === "" ){
			reviewContext= jazz.app.context.get().baseline;
		}

		if (this.isChangeset(reviewContext)){
			deferred= ChangesetClient.getChangeset(reviewContext);
		} else {
			deferred= ContextUtils.getContextDeferred(reviewContext);
		}
		return deferred;
	},
	
	// FIXME: temporary hack until we use directly VVC changeset configurations
	_getChangesetId: function(changesetUri){
		var id= changesetUri.substring(changesetUri.lastIndexOf("/")+1);
		return id;
	},
	
	/* 
	 * Vote is enabled is the review is started and the reviewer already has the the required rights to vote 
	 */
	isVoteEnabled: function(){
		return this.review.getReviewAttribute("reviewStatus")== ReviewTypes.REVIEW_STATE_TYPES.Started&& this.allowVote&& this.isReviewer();  		
	},
	
	/* 
	 * Return true if the currenr user is added to the review reviewers list 
	 */
	isReviewer:function(){
		var participant= this.review.getParticipant(this.review.myUserURI);
		return participant && (participant.reviewRole == ReviewTypes.PARTICIPANT_ROLE_TYPES.Approver|| 
						participant.reviewRole == ReviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer);		
	},
	
	/* 
	 * Review can only be edited if it is in draft mode, or paused. 
	 */
	isEditable: function(){
		if(!this.allowModify){
			return false;
		}
		var st = this.review.getReviewAttribute("reviewStatus");
		return st == ReviewTypes.REVIEW_STATE_TYPES.Paused || st == ReviewTypes.REVIEW_STATE_TYPES.Draft;
	},
	
	/**********************************************************Review Status Updating APIs ****************************************************************/
	onFinalizeReview:  function(summary) {
		this.review.setReviewAttribute("reviewSummary", summary);
		this._changeReviewStatus("finalize");
	},

	onContinueReview: function() {
		this._changeReviewStatus("restart");
	},

	onPauseReview:  function() {
		this._changeReviewStatus("pause");
	},
	
	onStartReview: function() {
		this._changeReviewStatus("start");
	},
	
	canFinalizeReview: function() {
		return !this.isDirty&& this.review.getUsersResults().length != 0;
	},
	
	canStartReview: function() {
		if(this.isDirty){
			return false;
		}
		var review= this.review;
		
		if(review.getReviewResources().length<1){
			return false;
		}
		
		var users= review.getUsersBasedOnRoles([ReviewTypes.PARTICIPANT_ROLE_TYPES.Approver, ReviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer]);
		
		if(users.length<1){
			return false;
		}
		
		return true;
	},
	
	canPauseReview: function(){
		return !this.isDirty;
	},
	
	isChangesetReview: function (){
		return this.isChangeset(this.review.getReviewAttribute("reviewBaseline"));
	},

	isChangeset: function (uri){
		if (uri){
			return uri.indexOf("/changesets/")>-1;
		}
		return false;
	},

	/**********************************************************Private Methods *************************************************************************/
	_getResourceInfo: function(infos, resourceUri){
		if(!resourceUri){
			return infos;
		}
		if(!infos){
			return null;
		}
		for(var index=0 ;index<infos.length; index++){
			var info= infos[index];
			if(info.uri== resourceUri){
				return info;
			}
		}
	},
	
	_updateReviewInformation: function(){
		var reviewState= this.review.getReviewAttribute("reviewStatus");
		
		if(reviewState== ReviewTypes.REVIEW_STATE_TYPES.Draft){
			return;
		}
		/*Review status should be paused if the number of resources or partiaipants is zero */
		if(this.review.getReviewResources().length==0 || this.review.getUsersResults().length==0){
			this.review.setReviewAttribute("reviewStatus",ReviewTypes.REVIEW_STATE_TYPES.Paused);
		}else{
			if(this.review.getOverallReviewResult().percentageVotesCast==100){
				if(reviewState== ReviewTypes.REVIEW_STATE_TYPES.Started){
					/*Review status should be finalized if the percent is 100% and the review has at least one reviewer and one resource */
					this.review.setReviewAttribute("reviewStatus",ReviewTypes.REVIEW_STATE_TYPES.Reviewed);	
				}
			/*Remove participant so the only remained participant already submitted all results. This means that the review is finalized now. 
			 * But when you add a new partiaipant or resouce, this means that we should transition the fianlized status to paused again */
			}else if(reviewState== ReviewTypes.REVIEW_STATE_TYPES.Reviewed){
				this.review.setReviewAttribute("reviewStatus",ReviewTypes.REVIEW_STATE_TYPES.Paused);
			}			
		}
	},
	
	_reinit: function(){
		this.isDirty= false;
		this.clonedResultsModel= rmps.core.base.BaseClientUtil.shallowCloneModel(this.review._reviewResults.model);
	  	this.clonedModel= rmps.core.base.BaseClientUtil.shallowCloneModel(this.review.model);
	},
	
	_getStatusForAction: function(action){
		switch (action) {
			case "pause":
				return ReviewTypes.REVIEW_STATE_TYPES.Paused; 
			case "restart":
				if(this.review.getOverallReviewResult().percentageVotesCast == 100){
					return ReviewTypes.REVIEW_STATE_TYPES.Reviewed;
				}
				return ReviewTypes.REVIEW_STATE_TYPES.Started;
			case "start":
				return ReviewTypes.REVIEW_STATE_TYPES.Started;	
			case "finalize":
				return ReviewTypes.REVIEW_STATE_TYPES.Finalized; 
			default:
				return null;
		}

	},
	
	_changeReviewStatus: function(action) {
		/*The start date is only set once the review is started for the first time in its lifecycle. After that, we do not change that value.
		 * Thus, we make this check to see if the review start date in the original reivew model is undefined, then, we set the start date for the 
		 * first and last time 
		 */
		if(!this.clonedModel.startDate){
			this.review.setReviewAttribute("startDate",new Date());	
		}
		
		function success(review) {
			this.review.setReviewAttribute("reviewStatus",this._getStatusForAction(action));
			dojo.publish("review.view.editor.refresh");
			this._reinit();
			this.onReviewStatusChange();
		}
		var deferred = ReviewClient.updateReview(this.review, action);
		
		deferred.addCallback(this,success);
		
		deferred.addErrback(dojo.hitch(this, function (error) {
			return rmps.core.error.createError(error).dispatch();
		}));

	},
	/**********************************************************************************************************************************************/
	
	/*************************************Methods to be connected be other viewers/places**********************************************************/
	/**********************************************************************************************************************************************/
	onReviewStatusChange: 	function(status){/*To be called externally */},
	onReviewUpdate: 		function(key, value){/*To be called by other places */},
	onReviewResourceUpdate: function(resourceURI, key, value){/*To be called by other places */},
	onReviewResourceAdd:    function(resourceInfo){/*To be called by other places */},
	onReviewResourceRemove: function(resourceUri){/*To be called by other places */},
	onReviewResourceReset:  function(resourceUri){/*To be called by other places */},
	onReviewReviewerUpdate: function(userUri, valuesList){/*To be called by other places */},
	onReviewReviewerRemove: function(userUri){/*To be called by other places */},
	onReviewReviewerAdd: 	function(users, roleType){/*To be called by other places */},
	onReviewResultSubmit:	function(resourceUri, statusType){/*To be called by other places */},
	
	/*
	 * If status is true, then this means that the review has been just changed to dirty status, if not, thhis means just a notification of a new change to the review
	 *  reportsList is an array fo the changes made
	 */
	addDirtyNotifier: function(statusChange, reportsList){/*To be called by other places */},
	
	/*
	 * If status is true, then this means that the review has been just changed to dirty status, if not, thhis means just a notification of a new change to the review 
	 */
	removeDirtyNotifier: function(statusChange){/*To be called by other places */},
	
	/*Inform connectors when a save process is bout to be made */
	onSave: function(){/*To be called by other places */},
	
	onSaveSuccess: function(){/*To be called by other places */},
	
	onSaveError: function(){/*To be called by other places */}
	/**********************************************************************************************************************************************/
});

var ReviewControllerInstances = {};

rmps.core.review.ReviewController = {
		
		getInstance: function(resourceUri, reviewControllerDesciptor) {
			var id= reviewControllerDesciptor.getDescriptorId();
	  		var current= ReviewControllerInstances[id?id:resourceUri]; /*No need to have extra check on null; this should be done by callers */
	  		if(current){
	  			reviewControllerDesciptor.onReviewControllerLoad(current);
	  			return current;
	  		}
	  		
  			var instance = new rmps.core.review._ReviewController();
  			instance.isLoaded= false;
  			instance.resourceUri= resourceUri;
  			
  			instance.deferred= this.loadReview(instance,reviewControllerDesciptor);
  			
  			ReviewControllerInstances[resourceUri]= instance;
  			
  			return instance;
	 	},
	 	
	 	getCurrentInstance: function(reviewControllerDesciptor){
	 		return this.getInstance(ReviewUtil.getActiveReviewResourceUri(), reviewControllerDesciptor);
		},
	 		 	
	 	loadReview: function(instance, reviewControllerDesciptor){
	 		instance.context = reviewControllerDesciptor.getOwingContext(instance.resourceUri);
		 	
		 	var resourceUri= instance.resourceUri;
		 	
		 	var contextUri= instance.context.uri;
			
			var callbackDeferred= new dojo.Deferred();
			var doLoad= dojo.hitch(this, function(){
				var loadReviewFunc = dojo.hitch(this, this._loadReview, instance, reviewControllerDesciptor, callbackDeferred);
				var deferred;
				
				// If it is a new review we just open an empty review
				// If it is a cloned review then we perform the copy operations
				// Else we do a get of the review to display its content
				if(resourceUri==NEW_REVIEW_EDITOR){
					instance.isExistingReview = false;
					this._loadReview(instance, reviewControllerDesciptor, callbackDeferred, ReviewUtil.setReviewCurrentUser(new rmps.core.review.Review({resourceUri: resourceUri})));
				} else if (resourceUri.indexOf(ReviewUtil.CLONE_PARAM)> -1){
					instance.isExistingReview = false;
					var uri = resourceUri.substring(0,resourceUri.indexOf("&"));
					deferred = this._copyReview(uri);
					deferred.addCallback(this, loadReviewFunc);
				} else {
					instance.isExistingReview = true;
					deferred = rmps.core.review.ReviewClient.getReview(resourceUri, null, true);
					deferred.addCallback(this, loadReviewFunc);		
					return deferred;		
				}
				
			});
			
			var setReport= dojo.hitch(this, function(report){
				if(instance.operationReport){
					instance.operationReport.mixin(report);
					return doLoad();
				}
				instance.operationReport= report;	
			});
			var authoringDeferred = SecurityClient.isActionAllowed(PROCESS.Operations.ALL_AUTHORING,
				contextUri,[
					PROCESS.Actions.CREATE,
					PROCESS.Actions.MODIFY,
					PROCESS.Actions.DELETE
			]);
		
			authoringDeferred.addCallback(this, function(report) {return setReport(report);});
		
			var feedbackDeferred = SecurityClient.isActionAllowed(PROCESS.Operations.ALL_FEEDBACK, contextUri, [ PROCESS.Actions.VOTE]);
		
			feedbackDeferred.addCallback(this, function(report) {return setReport(report);});
			
			return callbackDeferred;
	 	},
	 	
	 	_copyReview: function(uri){
			var deferred = rmps.core.review.ReviewClient.getReview(uri, null, true);
			deferred.addCallback(this, function(review){
				var clonedReview= ReviewUtil.setReviewCurrentUser(new rmps.core.review.Review({resourceUri: "rmps.app.newReviewEditor"}));
				clonedReview.model= BaseClientUtil.shallowCloneModel(review.model);
				clonedReview.model.closeDate= null;
				clonedReview.model.startDate= null;
				clonedReview.model.reviewSummary= null;
				clonedReview.model.reviewName = messages.reviewCopyPrefix + " " + review.model.reviewName;
				clonedReview.model.reviewStatus = ReviewTypes.REVIEW_STATE_TYPES.Draft;
				clonedReview.model.reviewDueDate = new Date();
				var currentUser = ReviewUtil.getCurrentUserUri();
				clonedReview.model.owner = currentUser;
				return clonedReview;
			});
			return deferred;
	 	},
	 	
	 	_loadReview: function(instance, reviewControllerDesciptor, callbackDeferred, review)  {
			var baseline= review.getReviewAttribute("reviewBaseline");

			if (instance.isChangeset(baseline)){
				var csDeferred = ChangesetClient.getChangesetWorkspace(baseline);
				csDeferred.addCallback(dojo.hitch(this, function(workspace) {
					instance.workspace = (workspace !== "")?workspace:jazz.app.context.get().baseline;
					this._loadReviewSupp(instance, reviewControllerDesciptor, callbackDeferred, review);
				}));				
			} else {
				this._loadReviewSupp(instance, reviewControllerDesciptor, callbackDeferred, review);
			}
		},
	 	
	 	_loadReviewSupp: function(instance, reviewControllerDesciptor, callbackDeferred, review)  {
  			var report = instance.operationReport;
			instance.allowCreation = report.isAllowed(PROCESS.Actions.CREATE);
			instance.allowDeletion = report.isAllowed(PROCESS.Actions.DELETE);
			instance.allowVote = report.isAllowed(PROCESS.Actions.VOTE);
			instance.allowModify = report.isAllowed(PROCESS.Actions.MODIFY);
			
			var resultDeferred = ReviewClient.getReviewResult(review);
			resultDeferred.addCallback(dojo.hitch(this, function(reviewResults) {
				var deferreds = [];
				review._reviewResults= reviewResults;
				/*Prepare tracking models */
				
				instance.review = review;
				
	  			instance._reinit();
				instance.isLoaded= true;
				instance.isDirty= false;
				
				/*Init the user cache array. Intened to be used with the review lifecycle. We do not care if it holds information for removed reviewers
				 * who have been removed while working on the review current session as it is only used to reduce the redundant calls to get the users
				 * information everytime we refresh different places
				 */
				instance.usersCache= {};
				instance.resourcesInfoCache= {};
				
				var finalizeInit= function(){
					/*
					 * Compare users based on their names. We handle that here in order to make sure
					 * that we already retrieved the users' names, as what we have from the review response is
					 * the users' uris 
					 */
					var usersCache= instance.usersCache;
					instance.review.model.participants.dataSet.sort(function(a, b){
						var info1= instance.getUserInfo(a.contributor);
						var info2= instance.getUserInfo(b.contributor);
						
						/*Safe check. both variables cannot be null here */
						if(!info1) {return -1;}
						if(!info2) {return 1;}
							
						return info1.name.localeCompare(info2.name);			
    				});
		    				
					reviewControllerDesciptor.onReviewControllerLoad(instance);
					
					/*Now, set the baseline value */
					instance.baseline= reviewControllerDesciptor.getBaseline(instance);
					callbackDeferred.callback();
				};
				
				
				var resources= instance.review.model.resources.dataSet;
				var obtianedResourcesInfo= resources.length==0;
				var participants= instance.review.model.participants.dataSet;
				
				if(participants.length<1&& resources.length<1){
					finalizeInit();
					return;
				}
				var participantsCount= 0;
				
				for(var index=0; index<participants.length; index++){
					var userDeferred = UserClient.getUser(participants[index].contributor);
					userDeferred.addCallback(dojo.hitch(this, function(user) {
		    			instance.usersCache[user.uri]= {
		    				emailAddress: user.email,
		    				itemId: user.id,
		    				name: user.name,
		    				userId: user.uri
		    			};
		    			participantsCount+=1;		    			
		    		}));
					deferreds.push(userDeferred);
				}
				
				
				
				var baseline= review.getReviewAttribute("reviewBaseline");
				var confDeferred = instance.getReviewContext();
				// If changeset exists but it has already been committed, its configuration has been deleted
				confDeferred.addCallback(dojo.hitch(this, function(changeset) {
					var state = changeset.state;
					if (state == rmps.core.changesets.ChangesetStates.COMMITTED)
						instance.configNotExist = changeset.state;
				}));
				// If the changeset or the configuration has been deleted
				confDeferred.addErrback(dojo.hitch(this, function(error) {
			        instance.configNotExist = true;
				}));
				
				if(!obtianedResourcesInfo){
					var setResults= function(resourceInfos){
						instance.resourcesInfoCache[baseline?baseline: DEFAULT_BASELINE]= resourceInfos;
						return;
					};
					var context = baseline;
					var changeset;
					if (instance.workspace){
						context = instance.workspace;
						changeset = baseline;
					}
					var resourceInfoClientdeferred = ResourceInfoClient.getResourceInfo(review.getReviewResources(null, []), null, context, changeset);
					resourceInfoClientdeferred.addCallback(this, dojo.hitch(this, function(setResults,resourceInfos) {
			    		setResults(resourceInfos);
					}), setResults);		
					
					resourceInfoClientdeferred.addErrback(this, dojo.hitch(this, function(setResults, error){
						setResults([]);
					}), setResults);
					
					deferreds.push(resourceInfoClientdeferred);
				}				
				this._deferred = new dojo.DeferredList(deferreds);
				this._deferred.addCallback(this, function () {
					finalizeInit();
				});
			}));
		},
	 	
	 	/*Required for refreshes */
	 	removeUpdater: function(resourceUri){
	  		var cur= ReviewControllerInstances[resourceUri]; /*No need to have extra check on null; this should be done by callers */
	  		if(cur){
	  			delete ReviewControllerInstances[resourceUri];
	  			return true;
	  		}
	  		/*Means that the id was not found */
	  		return false;
	 	}
};
})();