/*******************************************************************************
 * Licensed Materials - Property of IBM
 *
 * com.ibm.xtools.rmps.web.core
 *
 * Copyright IBM Corporation 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.review.ReviewResults");

dojo.require("rmps.core.review.util.ReviewTypes");
dojo.require("rmps.core.base.BaseClientUtil");

var ReviewTypes    = rmps.core.review.util.ReviewTypes;
var BaseClientUtil = rmps.core.base.BaseClientUtil;

var DC_NAMESPACE   = "http://purl.org/dc/terms/";
var DC_QUALIFIER   = "dc";

var RDF                   = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
var RCAM_REVIEW_NAMESPACE = "http://jazz.net/xmlns/rmps/reviews/1.0/";
var RCAM_REVIEW_QUALIFIER = "rmpsReview";
var CONTENT_TYPE_REVIEW   = "application/rdf+xml";

var NAMESPACES = {
	RCAM_REVIEW_QUALIFIER : RCAM_REVIEW_NAMESPACE,
	"rdf"                 : "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
	DC_QUALIFIER          : DC_NAMESPACE
};

/*
 * Used to wrap information for a review results element. When constructed,
 * an initial review results information is constructed, until we use parse
 * method with the retrieved * document after using a GET method for a review
 * resource URI.
 */
dojo.declare("rmps.core.review.ReviewResults", null, {

	_reviewParser: null,

	constructor: function(args) {
		if (args) {
			this._reviewParser = args.reviewParser;
		}
		this.initForm();
	},

	parse: function(doc, ioArgs) {
		this.model = BaseClientUtil.getModel(doc, {path: ["rdf:RDF", "rmpsReviews:ReviewResults"], numberOfOccurences: 1, form: this.form});
	},

	/*
	 * Return the value for an attribute in the review results model object.
	 */
	getReviewResultsAttribute: function(att) {
		return this.model[att];
	},

	getParticipantResults: function() {
		var results = this.getReviewResultsAttribute("hasParticipantResult");
		return results ? results.dataSet : [];
	},

	resetResourceResultsFor: function(resourceUri) {
		var resetResources = [];
		var participantResults = this.getParticipantResults();
		for (var index = 0; index < participantResults.length; index++) {
			var current = participantResults[index];

			/* Logical that no participant is done at this point. */
			current.isDone = false;

			var artifactResults = current.artifactResults.dataSet;
			for (var subIndex = 0; subIndex < artifactResults.length; subIndex++) {
				var artifact = artifactResults[subIndex];
				if (artifact.artifactId == resourceUri) {
					artifact.participantResultStatus = ReviewTypes.ARTIFACT_STATUS_TYPES.NotStarted;

					resetResources.push({
							artifactInfo: current,
							submitter: current.userURI
						});
				}
			}
		}
		return resetResources;
	},

	changeResourceResultsForUser: function(userUri, oldRole, newRole) {
		var changeResources = [];
		var status = new Object();
		status.Reviewer = [ReviewTypes.ARTIFACT_STATUS_TYPES.Reviewed,ReviewTypes.ARTIFACT_STATUS_TYPES.ReviewedComments];
//		status.Reviewer[0] = ReviewTypes.ARTIFACT_STATUS_TYPES.Reviewed;
//		status.Reviewer[1] = ReviewTypes.ARTIFACT_STATUS_TYPES.ReviewedComments;
		status.Approver = [ReviewTypes.ARTIFACT_STATUS_TYPES.Approved, ReviewTypes.ARTIFACT_STATUS_TYPES.Disapproved];
//		status.Approver[0] = ReviewTypes.ARTIFACT_STATUS_TYPES.Approved;
//		status.Approver[1] = ReviewTypes.ARTIFACT_STATUS_TYPES.Disapproved;

		var participantResults = this.getParticipantResults();
		for (var index = 0; index < participantResults.length; index++) {
			var current = participantResults[index];

			if (current.userURI === userUri && current.reviewRole === oldRole){
				var artifactResults = current.artifactResults.dataSet;
				for (var subIndex = 0; subIndex < artifactResults.length; subIndex++) {
					var artifact = artifactResults[subIndex];
					var resultIdx = dojo.indexOf(status[oldRole], artifact.participantResultStatus);
					if (resultIdx > -1){
						artifact.participantResultStatus = status[newRole][resultIdx];
						changeResources.push({
								artifactInfo: current,
								submitter: current.userURI
						});
					}
				}
			}
		}
		return changeResources;
	},

	getResultsFor: function(userUri, byIndex) {
		var participantResults = this.getParticipantResults();
		for (var index = 0; index < participantResults.length; index++) {
			var current = participantResults[index];
			if (current.userURI == userUri) {
				return byIndex ? index : current;
			}
		}
		return null;
	},

	getResultsForResource: function(resourceUri) {
		var detailedList = {};
		var participantResults = this.getParticipantResults();
		for (var index = 0; index < participantResults.length; index++) {
			var current = participantResults[index];
			if (current.userURI == userUri) {
				return current;
			}
		}
		return null;
	},

	getResultsForUserResources: function(userUri, resourceUri) {
		var results = this.getResultsFor(userUri, false);
		if (!results) {
			return null;
		}

		var artifactResults = results.artifactResults.dataSet;
		for (var index = 0; index < artifactResults.length; index++) {
			var artifact = artifactResults[index];
			if (artifact.artifactId == resourceUri) {
				/* Cannot have the same resource id for more than one entry, so return the fetched result directly. */
				return artifact;
			}
		}

		/* Required, as the review results may have some obsolete atifacts, or do not hold all artifact info. In such case
		 * we can definitely assume that the user result for that given resource is not started. */
		return {
			artifactId: resourceUri,
			participantResultStatus: ReviewTypes.ARTIFACT_STATUS_TYPES.NotStarted
		};
	},

	/*
	 * Get specific results given the results URI.
	 */
	getResults: function(resourceURI) {
		var participantResults = this.getParticipantResults();
		for (var index = 0; index < participantResults.length; index++) {
			var current = participantResults[index];
			if (current.resourceURI == resourceURI) {
				return current;
			}
		}
		return null;
	},

	removeResultsFor: function(userUri) {
		var index = this.getResultsFor(userUri, true);
		if (index > -1) {
			this.getParticipantResults().splice(index, 1);
		}
	},

	removeArtifactResult: function(resourceUri) {
		var participantResults = this.getParticipantResults();
		for (var index = 0; index < participantResults.length; index++) {
			var artifactResults = participantResults[index].artifactResults;
			if (!artifactResults) {
				continue;
			}
			artifactResults = artifactResults.dataSet;
			for (var subIndex = 0; subIndex < artifactResults.length; subIndex++) {
				var artifactResult = artifactResults[subIndex];
				if (artifactResult.artifactId == resourceUri) {
					artifactResults.splice(artifactResult, 1);

					/* Resource URI for results is unique, so, need to continue looping, and let's continue with the rest results. */
					continue;
				}
			}
		}
	},

	/*
	 * Return users list based on their roles
	 */
	getUsersBasedOnRoles: function(roles) {
		if (!roles|| roles.length == 0) {
			return [];
		}

		var users = [];
		var participantResults = this.getParticipantResults();
		for (var index = 0; index < participantResults.length; index++) {
			var current = participantResults[index];
			var userRole = current.reviewRole;
			for (var roleIndex = 0; roleIndex < roles.length; roleIndex++) {
				if (userRole === roles[roleIndex]) {
					users.push(current);
					break;
				}
			}
		}
		return users;
	},

	/*
	 * Return the results details for a given resource (number of approved, disapproved, etc.).
	 */
	getResultsDetailsForResource: function(resourceUri) {
		var numApproved = 0;
		var numDisapproved = 0;
		var numReviewed = 0;
		var numReviewedComments = 0;
		var numVotesCastByApproversAndReviewers = 0;

		var participantResults = this.getParticipantResults();

		for (var index = 0; index < participantResults.length; index++) {
			var current = participantResults[index];
			var userRole = current.reviewRole;
			var artifactResults = current.artifactResults.dataSet;
			for (var subIndex = 0; subIndex < artifactResults.length; subIndex++) {
				var artifact = artifactResults[subIndex];
				if (artifact.artifactId == resourceUri) {
					var voteWasCast = true;
					switch (artifact.participantResultStatus) {
						case ReviewTypes.ARTIFACT_STATUS_TYPES.Approved:
							numApproved++;
							break;
						case ReviewTypes.ARTIFACT_STATUS_TYPES.Disapproved:
							numDisapproved++;
							break;
						case ReviewTypes.ARTIFACT_STATUS_TYPES.Reviewed:
							numReviewed++;
							break;
						case ReviewTypes.ARTIFACT_STATUS_TYPES.ReviewedComments:
							numReviewedComments++;
							break;
						default:
							voteWasCast = false;
							break;
					}

					if (voteWasCast && (userRole === ReviewTypes.PARTICIPANT_ROLE_TYPES.Approver ||
										userRole === ReviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer)) {
						numVotesCastByApproversAndReviewers++;
					}
				}
			}
		}

		var numApproversAndReviewers = this.getUsersBasedOnRoles(
											[ReviewTypes.PARTICIPANT_ROLE_TYPES.Approver, ReviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer]).length;
		var percentageVotesCast = numApproversAndReviewers > 0
									? Math.round(100 * numVotesCastByApproversAndReviewers / numApproversAndReviewers)
									: 0;
		var numNotStarted = numApproversAndReviewers - numVotesCastByApproversAndReviewers;

		return {percentageVotesCast: percentageVotesCast, numApproved: numApproved, numDisapproved: numDisapproved,
				numReviewed: numReviewed, numReviewedComments: numReviewedComments, numNotStarted: numNotStarted};
	},

	/*
	 * Return the results details for a given user (number of approved, disapproved, etc.).
	 */
	getResultsDetailsForUser: function(userUri) {
		var results = this.getResultsFor(userUri);
		if (!results) {
			/* Safe check */
			return {percentageVotesCast: 0, numApproved: 0, numDisapproved: 0, numReviewed: 0, numReviewedComments: 0, numInProgress: 0, numNotStarted: 0};
		}

		var numApproved = 0;
		var numDisapproved = 0;
		var numReviewed = 0;
		var numReviewedComments = 0;
		var numInProgress = 0;

		var artifactResults = results.artifactResults.dataSet;
		for (var index = 0; index < artifactResults.length; index++) {
			var artifactResult = artifactResults[index];
			switch (artifactResult.participantResultStatus) {
				case ReviewTypes.ARTIFACT_STATUS_TYPES.Approved:
					numApproved++;
					break;
				case ReviewTypes.ARTIFACT_STATUS_TYPES.Disapproved:
					numDisapproved++;
					break;
				case ReviewTypes.ARTIFACT_STATUS_TYPES.Reviewed:
					numReviewed++;
					break;
				case ReviewTypes.ARTIFACT_STATUS_TYPES.ReviewedComments:
					numReviewedComments++;
					break;
				case ReviewTypes.ARTIFACT_STATUS_TYPES.InProgress:
					numInProgress++;
					break;
				case ReviewTypes.ARTIFACT_STATUS_TYPES.NotStarted:
					numInProgress++;
					break;
			}
		}

		var numVotesCast = numApproved + numDisapproved + numReviewed + numReviewedComments;
		var percentageVotesCast = artifactResults.length > 0 ? Math.round(100 * numVotesCast / artifactResults.length) : 0;

		var numNotStarted = artifactResults.length - numVotesCast;

		return {percentageVotesCast: percentageVotesCast, numApproved: numApproved, numDisapproved: numDisapproved,
				numReviewed: numReviewed, numReviewedComments: numReviewedComments, numInProgress: numInProgress, numNotStarted: numNotStarted};
	},

	getUserStatus: function(userUri) {
		var resultsDetails = this.getResultsDetailsForUser(userUri);
		if (!resultsDetails) {
			return ReviewTypes.PARTICIPANT_STATUS_TYPES.NotStarted;
		}
		switch (resultsDetails.percentageVotesCast) {
			case 100:
				return ReviewTypes.PARTICIPANT_STATUS_TYPES.Complete;
			case 0:
				return ReviewTypes.PARTICIPANT_STATUS_TYPES.NotStarted;
			default:
				return ReviewTypes.PARTICIPANT_STATUS_TYPES.Started;
		}
	},

	/*
	 * Change the status value for a resource for given element.
	 *
	 * The change has its effective on the UI side only. until we save the updates on the server
	 */
	voteForResource: function(userUri, resourceUri, resultStatus) {
		var results = this.getResultsFor(userUri);
		if (!results) {
			return; /* Safe check */
		}

		var isDone = true;
		var artifactResults = results.artifactResults.dataSet;
		for (var index = 0; index < artifactResults.length; index++) {
			var artifactResult = artifactResults[index];
			if (artifactResult.artifactId == resourceUri) {
				artifactResult.participantResultStatus = resultStatus;
				/* We do not break here as we still need to know the value for isDone flag. */
			} else {
				var participantResultStatus = artifactResult.participantResultStatus;
				if (!participantResultStatus|| participantResultStatus == ReviewTypes.ARTIFACT_STATUS_TYPES.NotStarted) {
					/* Well, we found a non-started element, then, isDone flag must be false. */
					isDone = false;
				}
			}
		}
		results.isDone = isDone;
	},

	getResultsContentFor: function(userUri) {
		var results = this.getResultsFor(userUri);
		if (!results) {
			console.error("Could not get the results uri");
			return ""; /* What's the proper way to handle such scenario? */
		}
		var userRole = results.reviewRole;
		var about = results.resourceURI;
		var reviewId = this._reviewParser.resourceUri;
		var isDone = results.isDone;
		var hasArtifactResultElementContent = "";
		var artifactResults = results.artifactResults.dataSet;
		for (var index = 0; index < artifactResults.length; index++) {
			var artifactResult = artifactResults[index];
			var artifactStatusType = artifactResult.participantResultStatus;
			if (!artifactStatusType) {
				artifactStatusType = ReviewTypes.ARTIFACT_STATUS_TYPES.NotStarted;
			}
			hasArtifactResultElementContent = hasArtifactResultElementContent
				+ "    <" + RCAM_REVIEW_QUALIFIER + ":hasArtifactResult>\n"
				+ "      <" + RCAM_REVIEW_QUALIFIER + ":ArtifactResult>\n"
				+ "        <" + RCAM_REVIEW_QUALIFIER + ":artifactId rdf:resource=\"" + artifactResult.artifactId + "\"/>\n"
				+ "        <" + RCAM_REVIEW_QUALIFIER + ":participantResultStatus>" + artifactStatusType + "</" + RCAM_REVIEW_QUALIFIER + ":participantResultStatus>\n"
				+ "      </" + RCAM_REVIEW_QUALIFIER + ":ArtifactResult>\n"
				+ "    </" + RCAM_REVIEW_QUALIFIER + ":hasArtifactResult>\n";
		}

		return  ( "<rdf:RDF\n"
				+ "    xmlns:" + RCAM_REVIEW_QUALIFIER + "=\"" + RCAM_REVIEW_NAMESPACE + "\"\n"
				+ "    xmlns:" + DC_QUALIFIER + "=\"" + DC_NAMESPACE + "\"\n"
				+ "    xmlns:rdf=\"" + RDF + "\">\n"
				+ "  <" + RCAM_REVIEW_QUALIFIER + ":ParticipantResult rdf:about=\"" + about + "\">\n"
				+ "    <" + RCAM_REVIEW_QUALIFIER + ":associatedReview rdf:resource=\"" + reviewId + "\"/>\n"
				+ "    <" + RCAM_REVIEW_QUALIFIER + ":hasParticipant rdf:resource=\"" + userUri + "\"/>\n"
				+ "    <" + RCAM_REVIEW_QUALIFIER + ":reviewRole>" + userRole + "</" + RCAM_REVIEW_QUALIFIER + ":reviewRole>\n"
				+ "    <" + RCAM_REVIEW_QUALIFIER + ":isDone>" + isDone + "</" + RCAM_REVIEW_QUALIFIER + ":isDone>\n"
				+ hasArtifactResultElementContent
				+ "  </" + RCAM_REVIEW_QUALIFIER + ":ParticipantResult>\n"
				+ "</rdf:RDF>");
	},

	initForm: function() {
		var artifactResultObjectForm = {path: ["rmpsReviews:ArtifactResult"], form: [
			{name: "artifactId", numberOfOccurences: 1, root: ["rmpsReviews:artifactId"], attribute: "rdf:resource"},
			{name: "participantResultStatus", numberOfOccurences: 1, root: ["rmpsReviews:participantResultStatus"], node: "#text", typeFunction: function(val) {
				return val ? val : ReviewTypes.ARTIFACT_STATUS_TYPES.NotStarted;
			}}
		]};

		var participantResultObjectForm = {path: [["rdf:Description"], ["rmpsReviews:ParticipantResult"]], form: [
			{name: "resourceURI", numberOfOccurences: 1, root: [], attribute: "rdf:about"},
			{name: "reviewRole", numberOfOccurences: 1, root: ["rmpsReviews:reviewRole"], node: "#text"},
			{name: "isDone", numberOfOccurences: 1, root: ["rmpsReviews:isDone"], node: "#text", typeFunction: function(val) {
				return val == "true";
			}},
			{name: "userURI", numberOfOccurences: 1, root: ["rmpsReviews:hasParticipant"], attribute: "rdf:resource"},
			{name: "associatedReview", numberOfOccurences: 1, root: ["rmpsReviews:associatedReview"], attribute: "rdf:resource"},
			{name: "artifactResults", dataFeedKey: "artifactId", root: ["rmpsReviews:hasArtifactResult"], objectForm: artifactResultObjectForm}
		]};

		this.form = [
			{name: "hasParticipantResult", root: ["rmpsReviews:hasParticipantResult"], dataFeedKey: "resourceURI", objectForm: participantResultObjectForm}
		];

		this.model = BaseClientUtil.getInitialModel(this.form);
	}
});

})();