/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.review.ReviewSummary");

dojo.require("dojox.data.dom");

dojo.require("rmps.core.resource.Resource");
dojo.require("rmps.core.resource.ResourceInfo");
dojo.require("rmps.core.user.UserClient");

var UserClient = rmps.core.user.UserClient;

dojo.declare("rmps.core.review.ReviewSummary", rmps.core.resource.Resource, {

	// properties
	link: null,
	name: null,
	updated: null,
	moderator: null,
	dueDate: null,
	status: null,
	role: null,
	lastModified: null, 
	uri: null,
	context: null,
	contextName: null,
	eTag: null,
	owningProjectUri: null,
	owningProjectName: null,
	
	// private property
	_moderator: null,
	_iconName: null,
	_icon: null,

	constructor: function() {
	},
	
	getReviewInfo: function() {
		var resourceInfo = new rmps.core.resource.ResourceInfo();
		resourceInfo.uri = this.link;
//			var _iconName = "review-" + this.status.toLowerCase() + ".gif";
//			resourceInfo.iconUri = dojo.moduleUrl("rmps.app", "ui/internal/editors/views/review/utils/images/" + _iconName);
		resourceInfo.shortName = this.name;
//			resourceInfo.longName = this.status;

		return resourceInfo;
	},

	/**
	 * Return a deferred for the moderator of this review.
	 * 
	 * @return a Deferred for the moderator of this review
	 * @type dojo.Deferred
	 */
	getModerator: function() {
		var deferred;
		if (this._moderator && this._moderator.uri === this.moderator) {
			deferred = new dojo.Deferred();
			setTimeout(dojo.hitch(this, function() {
				deferred.callback(this._moderator);
			}), 0);
			return deferred;
		}
		deferred = UserClient.getUser(this.moderator);
		deferred.addCallback(dojo.hitch(this, function(user) {
			this._moderator = user;
		}));
		return deferred;
	},
	
	parseEntry: function(entry, ioArgs) {
		if (ioArgs) {
			var xhr = ioArgs.xhr;
			if (xhr.getResponseHeader("Last-Modified")) {
				this.lastModified = new Date(xhr.getResponseHeader("Last-Modified"));
			}
			if (xhr.getResponseHeader("ETag")) {
				this.eTag = xhr.getResponseHeader("ETag");
			}
			if (xhr.getResponseHeader("Location")) {
				this.uri = xhr.getResponseHeader("Location");
			}
		}
		
		// link
		var links = entry.link;
		for (var j = 0; j < links.length; j++) {
			var link = links[j];
			if (link.rel == "self") {
				this.link = links[j].href;
			}
		}
		
		// name
		this.name = entry.title[0].value;
		
		// modified
		this.updated = dojo.date.stamp.fromISOString(entry.updated[0].value);

		// moderator
		var author = entry.author[0];
		this.moderator = author.name[0].value;
		
		// due date
		this.dueDate = dojo.date.stamp.fromISOString(entry["rmpsReviews:reviewDueDate"][0].value);
		
		// role
		this.role = entry["rmpsReviews:reviewRole"][0].value;

		// status
		this.status = entry["rmpsReviews:reviewStatus"][0].value;			
		
		// context
		this.context = new rmps.core.review.ReviewContextSummary();
		this.context.uri= entry["rmpsReviews:reviewContextUri"][0].value;
		this.context.name = entry["rmpsReviews:reviewContextName"][0].value;
		
		// owning project
		this.owningProjectUri = entry["rmpsReviews:reviewOwningProjectURI"][0].value;
		this.owningProjectName = entry["rmpsReviews:reviewOwningProjectName"][0].value;
	}
		
});
/*************************************************************************************************************************************************/

/********************************************************ReviewContextSummary*********************************************************************/
dojo.declare("rmps.core.review.ReviewContextSummary", rmps.core.resource.Resource, {

	// properties
	uri: null,
	name: null,
	
	constructor: function() {
	}

});
/*************************************************************************************************************************************************/

})();
