/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.review.util.ReviewTypes");

dojo.require("dojo.i18n");
dojo.require("rmps.core.review.util.ReviewImageService");

var mapArtifactStatusTypeToButtonImageURI = null;
var mapArtifactStatusTypeToDisplayName = null;
var mapArtifactStatusTypeToImageURI = null;
var mapParticipantRoleTypeToDisplayName = null;
var mapParticipantRoleTypeToImageURI = null;
var mapParticipantStatusTypeToDisplayName = null;
var mapReviewStateTypeToDisplayName = null;
var mapReviewStateTypeToImageURI = null;
var mapParticipantStatusImageURI= null;

dojo.requireLocalization("rmps.core.review.util", "ReviewTypesMessages");

var messages = dojo.i18n.getLocalization("rmps.core.review.util","ReviewTypesMessages");

rmps.core.review.util.ReviewTypes = {

	ARTIFACT_STATUS_TYPES: {
		Approved: "Approved",
		Disapproved: "Rejected",
		Abstained: "Abstained",
		Reviewed: "Reviewed",
		ReviewedComments: "ReviewedComments",
		NotStarted: "NotStarted",
		InProgress: "InProgress",
		NotExist: "NotExist"
	},

	PARTICIPANT_ROLE_TYPES: {
		AUTHOR: "Moderator",
		Approver: "Approver",
		RequiredReviewer: "Reviewer",
		OptionalReviewer: "OptionalReviewer"
	},

	PARTICIPANT_STATUS_TYPES: {
		Started: "Started",
		NotStarted: "NotStarted",
		Complete: "Complete"
	},

	REVIEW_STATE_TYPES: {
		Draft: "Draft",
		Started: "InProgress",
		Paused: "Paused",
		Reviewed: "Reviewed",
		NotStarted: "NotStarted",
		Finalized: "Finalized"
	},

	getArtifactTypeDisplayName: function(artifactStatusType) {
		if (! mapArtifactStatusTypeToDisplayName) {
			this._initMapArtifactStatusTypeToDisplayName();
		}
		return mapArtifactStatusTypeToDisplayName[artifactStatusType];
	},

	getArtifactStatusTypeButtonImageURI: function(artifactStatusType) {
		if (! mapArtifactStatusTypeToButtonImageURI) {
			this._initMapArtifactStatusTypeToButtonImageURI();
		}
		return mapArtifactStatusTypeToButtonImageURI[artifactStatusType];
	},

	getArtifactStatusTypeImageURI: function(artifactStatusType) {
		if (! mapArtifactStatusTypeToImageURI) {
			this._initMapArtifactStatusTypeToImageURI();
		}
		return mapArtifactStatusTypeToImageURI[artifactStatusType];
	},

	getReviewStateTypeImageURI: function(reviewStateType) {
		if (! mapReviewStateTypeToImageURI) {
			this._initMapReviewStateTypeToImageURI();
		}
		return mapReviewStateTypeToImageURI[reviewStateType];
	},

	getArtifactStatusTypeDisplayName: function(artifactStatusType) {
		if (! mapArtifactStatusTypeToDisplayName) {
			this._initMapArtifactStatusTypeToDisplayName();
		}
		return mapArtifactStatusTypeToDisplayName[artifactStatusType];
	},

	getParticipantStatusTypeDisplayName: function(participantStatusType) {
		if (! mapParticipantStatusTypeToDisplayName) {
			this._initMapParticipantStatusTypeToDisplayName();
		}
		return mapParticipantStatusTypeToDisplayName[participantStatusType];
	},

	getParticipantRoleTypeDisplayName: function(participantRoleType, normalize) {
		if(normalize== undefined){
			normalize= true;	/*Normalize by default */
		}
		if (! mapParticipantRoleTypeToDisplayName) {
			this._initMapParticipantRoleTypeToDisplayName();
		}
		var value= mapParticipantRoleTypeToDisplayName[participantRoleType];
		if(!value&& normalize){
			value= messages.noRoleAssigned;
		}
		return value;
	},

	getParticipantRoleTypeImageURI: function(participantRoleType) {
		if (! mapParticipantRoleTypeToImageURI) {
			this._initMapParticipantRoleTypeToImageURI();
		}
		return mapParticipantRoleTypeToImageURI[participantRoleType];
	},
	
	getReviewStateDisplayName: function(reviewStateType) {
		if (! mapReviewStateTypeToDisplayName) {
			this._initMapReviewStateTypeToDisplayName();
		}
		return mapReviewStateTypeToDisplayName[reviewStateType];
	},
	
	getParticipantStatusTypeImageURI: function(participantStatusType) {
		if (! mapParticipantStatusImageURI) {
			this._initMapParticipantStatusImageURI();
		}
		return mapParticipantStatusImageURI[participantStatusType];
	},
	
	_initMapParticipantStatusImageURI: function() {
		mapParticipantStatusImageURI = {};
		mapParticipantStatusImageURI[this.PARTICIPANT_STATUS_TYPES.NotStarted] = rmps.core.review.util.ReviewImageService.NOT_STARTED;
		mapParticipantStatusImageURI[this.PARTICIPANT_STATUS_TYPES.Started] =rmps.core.review.util.ReviewImageService.IN_PROGRESS;
		mapParticipantStatusImageURI[this.PARTICIPANT_STATUS_TYPES.Complete] =	rmps.core.review.util.ReviewImageService.REV_REVIEWED;
	},

	_initMapArtifactStatusTypeToDisplayName: function() {
		mapArtifactStatusTypeToDisplayName = {};
		mapArtifactStatusTypeToDisplayName[this.ARTIFACT_STATUS_TYPES.NotStarted] =	messages.notStarted;
		mapArtifactStatusTypeToDisplayName[this.ARTIFACT_STATUS_TYPES.InProgress] =	messages.inProgress;
		mapArtifactStatusTypeToDisplayName[this.ARTIFACT_STATUS_TYPES.Approved] =  messages.approved;
		mapArtifactStatusTypeToDisplayName[this.ARTIFACT_STATUS_TYPES.Disapproved] = messages.disapproved;
		mapArtifactStatusTypeToDisplayName[this.ARTIFACT_STATUS_TYPES.Reviewed] = messages.reviewedNoComments;
		mapArtifactStatusTypeToDisplayName[this.ARTIFACT_STATUS_TYPES.ReviewedComments] = messages.reviewedComments;
		mapArtifactStatusTypeToDisplayName[this.ARTIFACT_STATUS_TYPES.Abstained] =	messages.abstained;
		mapArtifactStatusTypeToDisplayName[this.ARTIFACT_STATUS_TYPES.NotExist] =	messages.notExist;
		},

	_initMapArtifactStatusTypeToButtonImageURI: function() {
		mapArtifactStatusTypeToButtonImageURI = {};
		mapArtifactStatusTypeToButtonImageURI[this.ARTIFACT_STATUS_TYPES.Approved] = rmps.core.review.util.ReviewImageService.APPROVE;
		mapArtifactStatusTypeToButtonImageURI[this.ARTIFACT_STATUS_TYPES.Disapproved] = rmps.core.review.util.ReviewImageService.DISAPROVE;
		mapArtifactStatusTypeToButtonImageURI[this.ARTIFACT_STATUS_TYPES.Abstained] = rmps.core.review.util.ReviewImageService.ABSTAIN;
		mapArtifactStatusTypeToButtonImageURI[this.ARTIFACT_STATUS_TYPES.Reviewed] =	rmps.core.review.util.ReviewImageService.REVIEW;
		mapArtifactStatusTypeToButtonImageURI[this.ARTIFACT_STATUS_TYPES.ReviewedComments] =	rmps.core.review.util.ReviewImageService.REVIEW_COMMENTS;		
	},

	_initMapArtifactStatusTypeToImageURI: function() {
		mapArtifactStatusTypeToImageURI = {};
		mapArtifactStatusTypeToImageURI[this.ARTIFACT_STATUS_TYPES.NotStarted] = rmps.core.review.util.ReviewImageService.NOT_STARTED;
		mapArtifactStatusTypeToImageURI[this.ARTIFACT_STATUS_TYPES.InProgress] =rmps.core.review.util.ReviewImageService.IN_PROGRESS;
		mapArtifactStatusTypeToImageURI[this.ARTIFACT_STATUS_TYPES.Approved] = rmps.core.review.util.ReviewImageService.REV_APPROVE;
		mapArtifactStatusTypeToImageURI[this.ARTIFACT_STATUS_TYPES.Disapproved] = rmps.core.review.util.ReviewImageService.REV_DISAPPROVE;
		mapArtifactStatusTypeToImageURI[this.ARTIFACT_STATUS_TYPES.Abstained] = 	rmps.core.review.util.ReviewImageService.REV_ABSTAINED;
		mapArtifactStatusTypeToImageURI[this.ARTIFACT_STATUS_TYPES.Reviewed] =	rmps.core.review.util.ReviewImageService.REV_REVIEW_NO_COMMENTS;
		mapArtifactStatusTypeToImageURI[this.ARTIFACT_STATUS_TYPES.ReviewedComments] =	rmps.core.review.util.ReviewImageService.REV_REVIEW_COMMENTS;		
		mapArtifactStatusTypeToImageURI[this.ARTIFACT_STATUS_TYPES.NotExist] =	rmps.core.review.util.ReviewImageService.RES_NOT_EXIST;		
	},

	_initMapParticipantRoleTypeToImageURI: function() {
		mapParticipantRoleTypeToImageURI = {};
		mapParticipantRoleTypeToImageURI[this.PARTICIPANT_ROLE_TYPES.Approver] = rmps.core.review.util.ReviewImageService.APPROVER_ROLE;
		mapParticipantRoleTypeToImageURI[this.PARTICIPANT_ROLE_TYPES.RequiredReviewer] = rmps.core.review.util.ReviewImageService.REVIEWER_ROLE;
		mapParticipantRoleTypeToImageURI[this.PARTICIPANT_ROLE_TYPES.OptionalReviewer] = rmps.core.review.util.ReviewImageService.OPTIONAL_REVIEWER_ROLE;
		mapParticipantRoleTypeToImageURI[this.PARTICIPANT_ROLE_TYPES.AUTHOR] = rmps.core.review.util.ReviewImageService.AUTHOR_ROLE;
	},

	_initMapReviewStateTypeToImageURI: function() {
		mapReviewStateTypeToImageURI = {};
		mapReviewStateTypeToImageURI[this.REVIEW_STATE_TYPES.Draft] = rmps.core.review.util.ReviewImageService.REVIEW_DRAFT;
		mapReviewStateTypeToImageURI[this.REVIEW_STATE_TYPES.NotStarted] = rmps.core.review.util.ReviewImageService.NOT_STARTED;
		mapReviewStateTypeToImageURI[this.REVIEW_STATE_TYPES.Started] = rmps.core.review.util.ReviewImageService.REVIEW_STARTED;
		mapReviewStateTypeToImageURI[this.REVIEW_STATE_TYPES.Paused] = rmps.core.review.util.ReviewImageService.REVIEW_PAUSED;
		mapReviewStateTypeToImageURI[this.REVIEW_STATE_TYPES.Reviewed] = rmps.core.review.util.ReviewImageService.REV_REVIEWED;
		mapReviewStateTypeToImageURI[this.REVIEW_STATE_TYPES.Finalized] = rmps.core.review.util.ReviewImageService.REVIEW_FINALIZED;
	},

	_initMapParticipantStatusTypeToDisplayName: function() {
		mapParticipantStatusTypeToDisplayName = {};
		mapParticipantStatusTypeToDisplayName[this.PARTICIPANT_STATUS_TYPES.Started] =messages.inProgress;
		mapParticipantStatusTypeToDisplayName[this.PARTICIPANT_STATUS_TYPES.NotStarted] = messages.notStarted;
		mapParticipantStatusTypeToDisplayName[this.PARTICIPANT_STATUS_TYPES.Complete] = messages.done;
	},

	_initMapParticipantRoleTypeToDisplayName: function() {
		mapParticipantRoleTypeToDisplayName = {};
		mapParticipantRoleTypeToDisplayName[this.PARTICIPANT_ROLE_TYPES.AUTHOR] =
			messages.author;
		mapParticipantRoleTypeToDisplayName[this.PARTICIPANT_ROLE_TYPES.Approver] =
			messages.approver;
		mapParticipantRoleTypeToDisplayName[this.PARTICIPANT_ROLE_TYPES.RequiredReviewer] =
			messages.reviewer;
		mapParticipantRoleTypeToDisplayName[this.PARTICIPANT_ROLE_TYPES.OptionalReviewer] =
			messages.optionalReviewer;
	},

	_initMapReviewStateTypeToDisplayName: function() {
		mapReviewStateTypeToDisplayName = {};
		mapReviewStateTypeToDisplayName[this.REVIEW_STATE_TYPES.Draft] = messages.draft;
		mapReviewStateTypeToDisplayName[this.REVIEW_STATE_TYPES.Started] =messages.started;
		mapReviewStateTypeToDisplayName[this.REVIEW_STATE_TYPES.Paused] =	messages.paused;
		mapReviewStateTypeToDisplayName[this.REVIEW_STATE_TYPES.Reviewed] =	messages.reviewed;
		mapReviewStateTypeToDisplayName[this.REVIEW_STATE_TYPES.Finalized] = messages.finalized;
	}
};

})();
