/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.review.util.ReviewUtil");

dojo.require("rmps.core.review.util.ReviewTypes");
dojo.require("rmps.core.RootServices");
dojo.require("rmps.core.resource.ResourceClient");

var ResourceClient = rmps.core.resource.ResourceClient;
var ReviewTypes= rmps.core.review.util.ReviewTypes;

rmps.core.review.util.ReviewUtil = {
	
	CLONE_PARAM: "clone=true",
	
	getUserId: function(userUri){
		var index = userUri.lastIndexOf("/");
		return decodeURIComponent(userUri.substring(index+1));
		
	},
		
	getActiveReviewResourceUri: function(){
		var hash = window.location.hash;
		hash= hash.replace("#action=", "");
		return dojo.queryToObject(hash).resourceUri;
	},
	
	decodeUserUri: function(userId) {
		var index = userId.lastIndexOf("/");
		var beginSeg = userId.substring(0,index);
		var decodedEnd = decodeURIComponent(userId.substring(index));
		return beginSeg + decodedEnd;
	},
	
	setReviewCurrentUser: function(review){
		var user= dojo.getObject("jazz.app.currentApplication.config.user.uri");
		if(user){
			review.myUserURI= rmps.core.review.util.ReviewUtil.getCorrectUserUri(user);
			return review;
		}
		
		/* 
		 *Why do we handle that method synchronusly? Probably it's not the best way, but for now, we do that as we use it 
		 * in iFrames (such as review rich hover), and when we use a review tabs seprately, and that case, we have to get the 
		 * user value right away. We will have a better handeling for this method when we get some more scenarios to test, so
		 * the current user estimation will be handled in a generic way if jazz.app.currentApplication.config.user.uri is undefined  
		 */
		var userDeferred= jazz.client.xhrGet({url: rmps.core.RootServices.getUri(rmps.core.Namespaces.JFS, "currentUser"),
				handleAs: "text", 
				load: function(userUri, ioArgs){
					review.myUserURI= rmps.core.review.util.ReviewUtil.getCorrectUserUri(userUri);
					return;	
				},
				sync: true});
		return review;
	},
	
	// Added functions because Jazz returns the uri of a user with @ in the userId encoded
	// as %40 where it should return it as @
	getCurrentUserUri: function() {
		var userUri = jazz.app.currentApplication.config.user.uri;
		return this.getCorrectUserUri(userUri);
	},
	getCorrectUserUri: function(userId) {
		var decodedUserId = decodeURIComponent(userId); 
		return this.encodeUserUri(decodedUserId);
	},
	
	encodeUserUri: function(userId) {
		var index = userId.lastIndexOf("/");
		var beginSeg = userId.substring(0,index);
		var encodedEnd = encodeURIComponent(userId.substring(index+1));
		return beginSeg + "/" + encodedEnd;
	},
	
	// FIXME: Added function because the DomParser removes the \n when they are inside
	// a node
	replaceLineBreak: function(response,tag){
		if (response.match(tag)){
			var respArr = response.split(tag);
			respArr[1]=respArr[1].replace(/\n/g, "&lt;br/&gt;");
			return respArr.join(tag);
		}
		return response;
	},
	
	// FIXME: Added function because the DomParser removes the \n when they are inside
	// a node
	replaceBR: function(string){
		var updStr= (string)?string.replace(/<br\/>/g, "\n"):null;
		return updStr;
	},
	
	// Encode and Decode xml strings
	encodeString: function(xmlString){
		var encodedString = dojox.html.entities.encode(xmlString);
		
		// sometimes the dojox encoder does not encode correctly the special characters (like &yen;)
		// Also dojox.html.entities can add for example &nbsp;
		return encodedString.replace(/\&/g, "&amp;");
	},
	
	decodeString: function(xmlString){
		// sometimes the dojox decoder does not decode correctly the &amp;
		var decodedString = xmlString.replace(/\&amp;/g, "&");

		return dojox.html.entities.decode(decodedString);
	},
	
	resourceExists: function(resourceUri) {
		var deferred = ResourceClient.getResourceHeaders(resourceUri, true, true);
		deferred.addCallback(this, function(ioArgs) {
			var headers = ioArgs.xhr.getAllResponseHeaders();
			var status = ioArgs.xhr.status;
			if (status == "204" ){
				return false;
			} else {
				return true;			
			}
		});
		return deferred;

	}

};
})();