/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * Copyright IBM Corp. 2010, 2011. All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("rmpw.diagrams.layout.client.routers.AbstractRouter");

dojo.require("rmpw.diagrams.layout.client.routers.ConstraintBendpoint");
dojo.require("rmpw.diagrams.render.util.PointList");

dojo.declare("rmpw.diagrams.layout.client.routers.AbstractRouter", null, {
	
	constraints: null,
	
	constructor : function(container) {
		this._container = container;
	},
	
	destroy: function() {
		
	},

	/*
	 * Gets the graph container that this router is active for.
	 */
	getContainer: function() {
		return this._container;
	},
	
	/*
	 * determines if this connection is being used for feedback or not
	 */
	isFeedback: function (conn){
		if (conn.isFeedback()) {
			return conn.isFeedback();
		}
		
		return false;
	},
	
	getAnchorableFigureBounds: function (figure){
		if (figure) {
			return figure.getBounds();
		}
		return null;
	},
	
	getDefaultStraightlineTolerance: function() {
		return 5;
	},
	
	/*
	 * Normalize segments to eliminate redundancy
	 */
	normalizePoints: function(pointList, tolerance) {
		pointList.normalize(tolerance);
	},
	
	/* 
	 * Remove all points that are contained in one of the nodes
	 */
	removePointsInNodes: function (nodes, newLine){
		var bChanged, endPolygon, i, iae, newPoints, pt, sourceOwner, startPolygon, targetOwner;
		bChanged = false;
		if (!newLine) {
			iae = new IllegalArgumentException();
			throw iae;
		}
		if (newLine.size < 3)
			return false;
		
		var nodePoly = [];
		for (var i = 0, lg=nodes.length; i<lg; i++) {
			var polygonShapes = nodes[i].mapping.policies.graphElement.getPolygonShapes();
			for (var j = 0, polyShape; polyShape = polygonShapes[j]; j++) {
				nodePoly.push(polyShape);
			}
		}
		
		newPoints = new rmpw.diagrams.render.util.PointList(newLine.size);
		for (var i = 0; i < newLine.size; ++i) {
			pt = newLine.getPoint( i);
			if (i == 0 || i == newLine.size - 1)
				newPoints.addPoint_0(pt.x, pt.y);
			else {
				var contained = false;
				for (j = 0, lg = nodePoly.length; j<lg; j++) {
					if (nodePoly[j].containsPoint(pt)) {
						contained = true;
					}
				}
				
				if (!contained) {
					newPoints.addPoint_0(pt.x, pt.y);
				}
			}
		}
		
		if (newPoints.size != newLine.size) {
			newLine.removeAllPoints();
			for (var i = 0; i < newPoints.size; ++i)
				newLine.addPoint_1(newPoints.getPoint(i).getCopy());
			return true;
		}
		
		return false;
	},
	
	getRelativeBendpointConstraint: function( connFigure, bendpoints ) {
		var constraint = [];
		
		var numPoints = bendpoints.length;
		for (var i=0; i<numPoints; i++) {
			var weight = i / (numPoints - 1);
			var bp = new rmpw.diagrams.layout.client.routers.ConstraintBendpoint(connFigure,
					bendpoints[i].sourceX, bendpoints[i].sourceY, bendpoints[i].targetX, bendpoints[i].targetY, weight);
			constraint.push(bp);
		}
		
		return constraint;
	},
	
	/*
	 * Given a set of point and reference points, will return a new
	 * set of relative bendpoints.
	 */
	getNewRelativeBendpoints: function(points, sourceRef, targetRef) {
		var relpoints = [];
		
		var pts = points.points;
		for (var j = 0, lg = pts.length; j < lg; j++) {
			relpoints.push({
				sourceX: pts[j].x - sourceRef.x, sourceY: pts[j].y - sourceRef.y, 
				targetX: pts[j].x - targetRef.x, targetY: pts[j].y - targetRef.y});
		}
		
		return relpoints;
	},
	
	/*
	 * Method that will calculate the router constraint given an array of points and
	 * a start reference point (typically the center of the source) and an end reference
	 * points (typically the center of the target).
	 */
	calculateRouterConstraint: function(connFigure, points) {
		var ref1 = connFigure.startAnchor.getReferencePoint();
		var ref2 = connFigure.endAnchor.getReferencePoint();
		
		connFigure.translateToRelative(ref1);
		connFigure.translateToRelative(ref2);
		var newBPs = this.getNewRelativeBendpoints(points, ref1, ref2);
		return this.getRelativeBendpointConstraint(connFigure, newBPs);
	},
	
	resetEndPointsToEdge: function (conn, newLine){
		var sourceAnchorPoint, sourceReference, targetAnchorPoint, targetReference;
		if (newLine.size < 2) {
			newLine.addPoint_0(0, 0);
			newLine.insertPoint(new rmpw.diagrams.render.util.Point(), 0);
		}
		if (newLine.size > 2) {
			sourceReference = newLine.getPoint( 1).getCopy();
			targetReference = newLine.getPoint( newLine.size - 2).getCopy();
			conn.translateToAbsolute(sourceReference);
			conn.translateToAbsolute(targetReference);
			sourceAnchorPoint = conn.getSourceAnchor().getLocation(sourceReference).getCopy();
			targetAnchorPoint = conn.getTargetAnchor().getLocation(targetReference).getCopy();
		}
		else {
			sourceReference = conn.getTargetAnchor().getReferencePoint().getCopy();
			targetReference = conn.getSourceAnchor().getReferencePoint().getCopy();
			sourceAnchorPoint = conn.getSourceAnchor().getLocation(sourceReference).getCopy();
			targetAnchorPoint = conn.getTargetAnchor().getLocation(targetReference).getCopy();
		}
		conn.translateToRelative(sourceAnchorPoint);
		conn.translateToRelative(targetAnchorPoint);
		newLine.setPoint(sourceAnchorPoint, 0);
		newLine.setPoint(targetAnchorPoint, newLine.size - 1);
	},

	getPointsFromConstraint: function(conn) {
		var bendpoints, bp, i, points, r1, r2, pt;
		
		bendpoints = conn.connectionRouter.getConstraint(conn);
		if (!bendpoints)
		{
			bendpoints = [];
		}
		points = new rmpw.diagrams.render.util.PointList(bendpoints.length);
		for (i = 0, bp; bp = bendpoints[i]; ++i) {
			pt = bp.getLocation();
			points.addPoint_0(pt.x, pt.y);
		}
		if (bendpoints.length == 0) {
			r1 = conn.startAnchor.getReferencePoint().getCopy();
			conn.translateToRelative(r1);
			points.addPoint_0(r1.x, r1.y);
			r2 = conn.endAnchor.getReferencePoint().getCopy();
			conn.translateToRelative(r2);
			points.addPoint_0(r2.x, r2.y);
		}
		
		if (!conn.getConnectionRouter().isOrthogonal()) {
			this.resetEndPointsToEdge(conn, points);
		}
		
		return points;
	},
	
	/*
	 * determines if this router ensures connections are orthogonal (straight)
	 */
	isOrthogonal : function() {
		return false;
	},
	
	getConstraint: function (connection){
		return this.constraints[connection.getID()];
	},
	
	setConstraint : function(conn, points) {
		if (this.constraints == null) {
			this.constraints = {};
		}
		
		var cpoints = [];
		if (points) {
			for (var i = 0, pt; pt = points[i]; i++) {
				cpoints.push(pt);
			}
		}
		
		this.constraints[conn.getID()] = cpoints;
	},
	
	remove : function(conn) {
		if (this.constraints != null) {
			delete this.constraints[conn.getID()];
		}
	},
	
	invalidate : function(conn) {
	},
	
	route: function (conn) {
	}
	
});