/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * Copyright IBM Corp. 2010, 2011. All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("rmpw.diagrams.layout.client.routers.AutomaticRouter");

dojo.require("rmpw.diagrams.layout.client.routers.AbstractRouter");

dojo.declare("rmpw.diagrams.layout.client.routers.AutomaticRouter", rmpw.diagrams.layout.client.routers.AbstractRouter, {
	
	connections: {},
	
	constructor: function(nextRouter) {
		this.nextRouter = nextRouter;
	},
	
	getType: function() {
		return this.nextRouter.getType();
	},
	
	/*
	 * delegate to the contained router
	 */
	isOrthogonal : function() {
		return this.nextRouter.isOrthogonal();
	},
	
	getConnectionKey: function(conn) {
		var sourceKey = conn.getSourceAnchor().hashCode$();
		var targetKey = conn.getTargetAnchor().hashCode$();
		if (sourceKey < targetKey) {
			return sourceKey + "-" + targetKey;
		}
		else {
			return targetKey + "-" + sourceKey;
		}
	},
	
	route: function (conn){
		var connectionKey, connectionList, index, points_0, end, points, start, ref_0, ref;
		if (this.nextRouter)
			this.nextRouter.route(conn);
		else {
			conn.getPointList().removeAllPoints();
			points = conn.getPointList();
			points.removeAllPoints();
			ref_0 = conn.endAnchor.getReferencePoint();
			start = conn.startAnchor.getLocation(ref_0);
			ref = conn.startAnchor.getReferencePoint();
			end = conn.endAnchor.getLocation(ref);
			conn.translateToRelative(start);
			conn.translateToRelative(end);
			points.addPoint_0(start.x, start.y);
			points.addPoint_0(end.x, end.y);
			conn.setPointList(points);
		}
		if (conn.getPointList().size == 2 && !this.isFeedback(conn)) {
			points_0 = conn.getPointList();
			connectionKey = this.getConnectionKey(conn);
			connectionList = this.connections[connectionKey];
			if (connectionList) {
				index = dojo.indexOf(connectionList, conn);
				if (index == -1) {
					connectionList.push(conn);
					index = connectionList.length - 1;
				}
				this.handleCollision(points_0, index);
				
				// re-adjust the start and end points accordingly
				ref = points_0.getPoint(1);
				conn.translateToAbsolute(ref);
				start = conn.startAnchor.getLocation(ref);
				conn.translateToRelative(start);
				end = conn.endAnchor.getLocation(ref);
				conn.translateToRelative(end);
				points_0.setPoint(start, 0);
				points_0.setPoint(end, points_0.size - 1);
				
				conn.setPointList(points_0);
			}
			else {
				connectionList = [conn];
				this.connections[connectionKey] = connectionList;
			}
		}
	},

	getConstraint: function (connection){
		if (this.nextRouter)
			return this.nextRouter.getConstraint(connection);
		return null;
	},
	
	setConstraint: function (connection, constraint) {
		this.invalidate(connection);
		if (this.nextRouter != null)
			this.nextRouter.setConstraint(connection, constraint);
	},

	invalidate: function (conn){
		if (this.nextRouter) {
			this.nextRouter.invalidate(conn);
		}	
		
		if (this.isFeedback(conn) || !conn.getSourceAnchor() || !conn.getTargetAnchor())
			return;
		
		this._remove(conn);	
	},
	
	_remove: function (conn) {
		var connectionKey, connectionList;
		
		connectionKey =  this.getConnectionKey(conn);
		connectionList = this.connections[connectionKey];
		if (connectionList) {
			var idx = dojo.indexOf(connectionList, conn);
			if (idx != -1) {
				connectionList.splice(idx, 1);
				for (var i = 0; i < connectionList.length; ++i) {
					connectionList[i].revalidate();
				}	
			}
		}
	},

	remove: function (conn){
		if (!conn.getSourceAnchor() || !conn.getTargetAnchor())
			return;
		this._remove(conn);
		if (this.nextRouter) {
			this.nextRouter.remove(conn);
		}
	}
});