/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * Copyright IBM Corp. 2010, 2012. All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("rmpw.diagrams.layout.client.routers.RectilinearRouter");

dojo.require("rmpw.diagrams.layout.client.routers.ObliqueRouter");
dojo.require("rmpw.diagrams.render.util.Ray");
dojo.require("rmpw.diagrams.render.graph.BaseGraphConfigurator");
dojo.require("rmpw.diagrams.render.util.Point");

dojo.declare("rmpw.diagrams.layout.client.routers.RectilinearRouter", rmpw.diagrams.layout.client.routers.ObliqueRouter, {
	
	getType: function() {
		return rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.RECTILINEAR_ROUTING;
	},

	isRectilinear: function (points){
		var currentPt, i, previousPt;
		for (i = 1; i < points.size; ++i) {
			currentPt = points.getPoint( i);
			previousPt = points.getPoint( i - 1);
			if (Math.floor(currentPt.x) != Math.floor(previousPt.x) && Math.floor(currentPt.y) != Math.floor(previousPt.y)) {
				return false;
			}
		}
		
		return true;
	},
	
	isOrthogonal : function() {
		return true;
	},

	transformToOrthogonalPointList: function (points, offStartDirection, offEndDirection){
		var endPoints, first, firstEndPt, isOffSourceDirectionSet, isOffTargetDirectionSet, lastStartPt, nextPt, second, startPoints;
		if (points.size > 1) {
			startPoints = new rmpw.diagrams.render.util.PointList(points.size);
			endPoints = new rmpw.diagrams.render.util.PointList(points.size);
			isOffSourceDirectionSet = offStartDirection == 64 || offStartDirection == 128;
			isOffTargetDirectionSet = offEndDirection == 128 || offEndDirection == 64;
			if (!isOffSourceDirectionSet && !isOffTargetDirectionSet) {
				first = points.getPoint( 0);
				second = points.getPoint( 1);
				offStartDirection = Math.abs(first.x - second.x) < Math.abs(first.y - second.y)?64:128;
				isOffSourceDirectionSet = true;
			}
			startPoints.addPoint_1(points.removePoint(0));
			endPoints.addPoint_1(points.removePoint(points.size - 1));
			while (points.size != 0) {
				if (isOffSourceDirectionSet) {
					nextPt = points.removePoint(0);
					lastStartPt = startPoints.getPoint(startPoints.size - 1);
					if (nextPt.x != lastStartPt.x && nextPt.y != lastStartPt.y) {
						if (offStartDirection == 128) {
							startPoints.addPoint_1(new rmpw.diagrams.render.util.Point( lastStartPt.x, nextPt.y));
							offStartDirection = 64;
						}
						else {
							startPoints.addPoint_1(new rmpw.diagrams.render.util.Point( nextPt.x, lastStartPt.y));
							offStartDirection = 128;
						}
					}
					else {
						offStartDirection = nextPt.x == lastStartPt.x?128:64;
					}
					startPoints.addPoint_0(nextPt.x, nextPt.y);
				}
				if (isOffTargetDirectionSet && points.size != 0) {
					nextPt = points.removePoint(points.size - 1);
					firstEndPt = endPoints.getPoint(0);
					if (nextPt.x != firstEndPt.x && nextPt.y != firstEndPt.y) {
						if (offEndDirection == 128) {
							endPoints.insertPoint(new rmpw.diagrams.render.util.Point( firstEndPt.x, nextPt.y), 0);
							offEndDirection = 64;
						}
						else {
							endPoints.insertPoint(new rmpw.diagrams.render.util.Point( nextPt.x, firstEndPt.y), 0);
							offEndDirection = 128;
						}
					}
					else {
						offEndDirection = nextPt.x == firstEndPt.x?128:64;
					}
					endPoints.insertPoint(nextPt, 0);
				}
			}
			lastStartPt = startPoints.getPoint(startPoints.size - 1);
			firstEndPt = endPoints.getPoint(0);
			if (lastStartPt.x != firstEndPt.x && lastStartPt.y != firstEndPt.y) {
				if (!isOffSourceDirectionSet && isOffTargetDirectionSet || isOffTargetDirectionSet && endPoints.size < startPoints.size) {
					if (offEndDirection == 128) {
						startPoints.addPoint_1(new rmpw.diagrams.render.util.Point( firstEndPt.x, lastStartPt.y));
					}
					else {
						startPoints.addPoint_1(new rmpw.diagrams.render.util.Point( lastStartPt.x, firstEndPt.y));
					}
				}
				else if (offStartDirection == 128) {
					startPoints.addPoint_1(new rmpw.diagrams.render.util.Point( lastStartPt.x, firstEndPt.y));
				}
				else {
					startPoints.addPoint_1(new rmpw.diagrams.render.util.Point( firstEndPt.x, lastStartPt.y));
				}
			}
			points.addAll(startPoints);
			points.addAll(endPoints);
		}
	},

	getAnchorLocationBasedOnSegmentOrientation: function (anchorPoint, rectangle, segmentOrientation){
		if (segmentOrientation == 128) {
			if (Math.abs(anchorPoint.y - rectangle.y) < Math.abs(anchorPoint.y - rectangle.y - rectangle.height)) {
				return 1;
			}
			else {
				return 4;
			}
		}
		else if (segmentOrientation == 64) {
			if (Math.abs(anchorPoint.x - rectangle.x) < Math.abs(anchorPoint.x - rectangle.x - rectangle.width)) {
				return 8;
			}
			else {
				return 16;
			}
		}
		return 0;
	},

	getAnchorOffRectangleDirection: function (anchorPoint, rect){
		var criteriaValue, position, tempCriteria;
		position = 1; //top
		criteriaValue = Math.abs(anchorPoint.y - rect.y);
		tempCriteria = Math.abs(anchorPoint.y - rect.y - rect.height);
		if (tempCriteria < criteriaValue) {
			criteriaValue = tempCriteria;
			position = 4; //bottom
		}
		tempCriteria = Math.abs(anchorPoint.x - rect.x);
		if (tempCriteria < criteriaValue) {
			criteriaValue = tempCriteria;
			position = 8; //left
		}
		tempCriteria = Math.abs(anchorPoint.x - rect.x - rect.width);
		if (tempCriteria < criteriaValue) {
			criteriaValue = tempCriteria;
			position = 16; //right
		}
		return position;
	},

	getOffShapeDirection: function (anchorRelativeLocation){
		if (anchorRelativeLocation == 16 || anchorRelativeLocation == 8) {
			return 64; //source/target == left/right
		}
		else if (anchorRelativeLocation == 1 || anchorRelativeLocation == 4) {
			return 128; //source/target == top/bottom
		}
		return 0; //source == top/bottom; target == left/right
	},

	getOutsidePointOffRectanglePosition: function (p, r){
		var position;
		position = 0;
		if (r.x > p.x) {
			position |= 8;
		}
		else if (r.x + r.width < p.x) {
			position |= 16;
		}
		if (r.y > p.y) {
			position |= 1;
		}
		else if (r.y + r.height < p.y) {
			position |= 4;
		}
		return position;
	},

	getTranslationValue: function (position, xFactorValue, yFactorValue){
		var translationDimension;
		translationDimension = new rmpw.diagrams.render.util.Dimension(0, 0);
		if (position == 16) {
			translationDimension.width = xFactorValue;
		}
		else if (position == 4) {
			translationDimension.height = yFactorValue;
		}
		else if (position == 8) {
			translationDimension.width = -xFactorValue;
		}
		else if (position == 1) {
			translationDimension.height = -yFactorValue;
		}
		return translationDimension;
	},

	isValidRectilinearLine: function (conn, line){
		if (line.size < 2) {
			return false;
		}
		var firstSegmentOrientation, lastSegmentOrientation, source, target;
		if (!instanceOf(conn.getSourceAnchor().getOwner(), 30)) {
			source = this.getAnchorableFigureBounds(conn.getSourceAnchor().getOwner()).getCopy();
			conn.getSourceAnchor().getOwner().translateToAbsolute(source);
			conn.translateToRelative(source);
			if (source.contains_3(line.getPoint( 1))) {
				return false;
			}
			firstSegmentOrientation = line.getPoint( 0).x == line.getPoint( 1).x?128:64;
			if (this.getOutsidePointOffRectanglePosition(line.getPoint( 1), source) != this.getAnchorLocationBasedOnSegmentOrientation(line.getPoint( 0), source, firstSegmentOrientation)) {
				return false;
			}
		}
		if (!instanceOf(conn.getTargetAnchor().getOwner(), 30)) {
			target = this.getAnchorableFigureBounds(conn.getTargetAnchor().getOwner()).getCopy();
			conn.getTargetAnchor().getOwner().translateToAbsolute(target);
			conn.translateToRelative(target);
			if (source.contains_3(line.getPoint( line.size - 2))) {
				return false;
			}
			lastSegmentOrientation = line.getPoint( line.size - 1).x == line.getPoint( line.size - 2).x?128:64;
			if (this.getOutsidePointOffRectanglePosition(line.getPoint( line.size - 2), target) != this.getAnchorLocationBasedOnSegmentOrientation(line.getPoint( line.size - 1), target, lastSegmentOrientation)) {
				return false;
			}
		}
		return true;
	},

	removePointsInViews_0: function (conn, newLine, start, end){
		var i, lastRemovedFromSource, lastRemovedFromTarget, source, target, tolerance, toleranceValue;
		source = conn.getSourceAnchor().getOwner()? this.getAnchorableFigureBounds(conn.getSourceAnchor().getOwner()).getCopy():null;
		target = conn.getTargetAnchor().getOwner()? this.getAnchorableFigureBounds(conn.getTargetAnchor().getOwner()).getCopy():null;
		if (source) {
			conn.getSourceAnchor().getOwner().translateToAbsolute(source);
			conn.translateToRelative(source);
		}
		if (target) {
			conn.getTargetAnchor().getOwner().translateToAbsolute(target);
			conn.translateToRelative(target);
		}
		lastRemovedFromSource = null;
		lastRemovedFromTarget = null;
		if (!instanceOf(conn.getSourceAnchor().getOwner(), 30) && newLine.size != 0 && source.contains_3(newLine.getPoint( 0).getCopy())) {
			lastRemovedFromSource = newLine.removePoint(0);
			for (i = 0; i < newLine.size && source.contains_3(newLine.getPoint( i).getCopy()); ++i) {
				lastRemovedFromSource = newLine.removePoint(i--);
			}
		}
		if (!instanceOf(conn.getTargetAnchor().getOwner(), 30) && newLine.size != 0 && target.contains_3(newLine.getPoint( newLine.size - 1).getCopy())) {
			lastRemovedFromTarget = newLine.removePoint(newLine.size - 1);
			for (i = newLine.size; i > 0 && target.contains_3(newLine.getPoint( i - 1).getCopy()); --i) {
				lastRemovedFromTarget = newLine.removePoint(i - 1);
			}
		}
		if (newLine.size == 0) {
			tolerance = new rmpw.diagrams.render.util.Dimension(1, 0);
			if (!this.isFeedback(conn))
			{
				tolerance.width = Math.floor(tolerance.width);
				tolerance.height = Math.floor(tolerance.height);		
			}
			toleranceValue = tolerance.width;
			if (!lastRemovedFromSource) {
				lastRemovedFromSource = start;
			}
			if (!lastRemovedFromTarget) {
				lastRemovedFromTarget = end;
			}
			if (Math.abs(lastRemovedFromSource.x - lastRemovedFromTarget.x) < toleranceValue) {
				if (source.y < target.y) {
					newLine.addPoint_0(lastRemovedFromSource.x, ~~((new rmpw.diagrams.render.util.Point( source.x + ~~(source.width / 2), source.y + source.height).y + new rmpw.diagrams.render.util.Point( target.x + ~~(target.width / 2), target.y).y) / 2));
				}
				else {
					newLine.addPoint_0(lastRemovedFromSource.x, ~~((new rmpw.diagrams.render.util.Point( source.x + ~~(source.width / 2), source.y).y + new rmpw.diagrams.render.util.Point( target.x + ~~(target.width / 2), target.y + target.height).y) / 2));
				}
			}
			else if (Math.abs(lastRemovedFromSource.y - lastRemovedFromTarget.y) < toleranceValue) {
				if (source.x < target.x) {
					newLine.addPoint_0(~~((new rmpw.diagrams.render.util.Point( source.x + source.width, source.y + ~~(source.height / 2)).x + new rmpw.diagrams.render.util.Point( target.x, target.y + ~~(target.height / 2)).x) / 2), lastRemovedFromSource.y);
				}
				else {
					newLine.addPoint_0(~~((new rmpw.diagrams.render.util.Point( source.x, source.y + ~~(source.height / 2)).x + new rmpw.diagrams.render.util.Point( target.x + target.width, target.y + ~~(target.height / 2)).x) / 2), lastRemovedFromSource.y);
				}
			}
			else {
			}
		}
	},

	removeRedundantPoints: function (line){
		var initialNumberOfPoints, newLine, p0, p1, p2;
		initialNumberOfPoints = line.size;
		if (line.size > 2) {
			newLine = new rmpw.diagrams.render.util.PointList(line.size);
			newLine.addPoint_1(line.removePoint(0));
			while (line.size >= 2) {
				p0 = newLine.getPoint( newLine.size - 1);
				p1 = line.getPoint( 0);
				p2 = line.getPoint( 1);
				if (p0.x == p1.x && p0.x == p2.x) {
					line.removePoint(0);
				}
				else if (p0.y == p1.y && p0.y == p2.y) {
					line.removePoint(0);
				}
				else {
					newLine.addPoint_1(line.removePoint(0));
				}
			}
			while (line.size > 0) {
				newLine.addPoint_1(line.removePoint(0));
			}
			line.removeAllPoints();
			line.addAll(newLine);
		}
		return line.size != initialNumberOfPoints;
	},

	resetEndPointsToEdge: function (conn, line){
		var anchorLocation, end, offEnd, offSourceDirection, offStart, offTargetDirection, offsetDim, reference, source_2, sourceAnchorRelativeLocation, start, target_2, targetAnchorRelativeLocation;
		
		if (this.isReorienting(conn)) {
			this.inherited(arguments);
			return;
		}
		source_2 = this.sourceBoundsRelativeToConnection(conn);
		target_2 = this.targetBoundsRelativeToConnection(conn);
		offSourceDirection = 0;
		offTargetDirection = 0;
		sourceAnchorRelativeLocation = 0;
		targetAnchorRelativeLocation = 0;
		if (line.size == 0) {
			line.addPoint_1(new rmpw.diagrams.render.util.Point());
			line.addPoint_1(new rmpw.diagrams.render.util.Point());
			rmpw.diagrams.layout.client.routers.AbstractRouter.prototype.resetEndPointsToEdge(conn, line);
			sourceAnchorRelativeLocation = this.getAnchorOffRectangleDirection(line.getPoint( 0), source_2);
			targetAnchorRelativeLocation = this.getAnchorOffRectangleDirection(line.getPoint( line.size - 1), target_2);
			offStart = line.getPoint( 0);
			offEnd = line.getPoint( line.size - 1);
			offsetDim = new rmpw.diagrams.render.util.Dimension(offStart.x - offEnd.x, offStart.y - offEnd.y).performScale(0.5);
			offStart.translate_0(this.getTranslationValue(sourceAnchorRelativeLocation, Math.abs(offsetDim.width), Math.abs(offsetDim.height)));
			offEnd.translate_0(this.getTranslationValue(targetAnchorRelativeLocation, Math.abs(offsetDim.width), Math.abs(offsetDim.height)));
			line.insertPoint(offStart, 1);
			line.insertPoint(offEnd, 2);
			offSourceDirection = this.getOffShapeDirection(sourceAnchorRelativeLocation);
			offTargetDirection = this.getOffShapeDirection(targetAnchorRelativeLocation);
		}
		else {
			start = line.getPoint( 0);
			end = line.getPoint( line.size - 1);
			reference = start.getCopy();
			conn.translateToAbsolute(reference);
			anchorLocation = conn.getSourceAnchor().getLocation(reference).getCopy();
			conn.translateToRelative(anchorLocation);
			line.insertPoint(anchorLocation, 0);
			reference = end.getCopy();
			conn.translateToAbsolute(reference);
			anchorLocation = conn.getTargetAnchor().getLocation(reference).getCopy();
			conn.translateToRelative(anchorLocation);
			line.addPoint_0(anchorLocation.x, anchorLocation.y);
			sourceAnchorRelativeLocation = this.getAnchorOffRectangleDirection(line.getPoint( 0), source_2);
			offSourceDirection = this.getOffShapeDirection(sourceAnchorRelativeLocation);
			targetAnchorRelativeLocation = this.getAnchorOffRectangleDirection(line.getPoint( line.size - 1), target_2);
			offTargetDirection = this.getOffShapeDirection(targetAnchorRelativeLocation);
		}
		this.transformToOrthogonalPointList(line, offSourceDirection, offTargetDirection);
		this.removeRedundantPoints(line);
	},

	sourceBoundsRelativeToConnection: function (conn){
		var source;
		source = conn.getSourceAnchor().getOwner().getBounds();
		conn.getSourceAnchor().getOwner().translateToAbsolute(source);
		conn.translateToRelative(source);
		return source;
	},

	targetBoundsRelativeToConnection: function (conn){
		var target;
		target = conn.getTargetAnchor().getOwner().getBounds();
		conn.getTargetAnchor().getOwner().translateToAbsolute(target);
		conn.translateToRelative(target);
		return target;
	},

	routeLine: function (conn, nestedRoutingDepth, newLine){
		var lastEndAnchor, lastStartAnchor, middleSeg, skipNormalization, tolerance;
		if (this.isReorienting(conn)) {
			this.inherited(arguments);
			return;
		}
		
		if (this.checkSelfRelConnection(conn, newLine) || this.checkShapesIntersect(conn, newLine)) {
			this.inherited(arguments);
			this.transformToOrthogonalPointList(newLine, this.getOffShapeDirection(this.getAnchorOffRectangleDirection(newLine.getPoint( 0), this.sourceBoundsRelativeToConnection(conn))), this.getOffShapeDirection(this.getAnchorOffRectangleDirection(newLine.getPoint( newLine.size - 1), this.targetBoundsRelativeToConnection(conn))));
			return;
		}
		
		this.removeRedundantPoints(newLine);
		this.normalizePoints(newLine, this.getDefaultStraightlineTolerance());
		
		lastStartAnchor = newLine.removePoint(0);
		lastEndAnchor = newLine.removePoint(newLine.size - 1);
		if (!this.isRectilinear(newLine)) {
			this.transformToOrthogonalPointList(newLine, 0, 0);
		}
		this.removePointsInViews_0(conn, newLine, lastStartAnchor, lastEndAnchor);
			
		if (!this.isValidRectilinearLine(conn, newLine)) {
			this.resetEndPointsToEdge(conn, newLine);
		}
			
		if (nestedRoutingDepth < 1 && !this.isValidRectilinearLine(conn, newLine)) {
			this.routeLine(conn, ++nestedRoutingDepth, newLine);
		}
	}
});