/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * Copyright IBM Corp. 2010, 2011. All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("rmpw.diagrams.layout.client.routers.Segment");

dojo.declare("rmpw.diagrams.layout.client.routers.Segment", null, {
	constructor: function (start, end){
		this.start = start;
		this.end = end;
		this.typeId$ = 93;
	},

	cosine: function (otherSegment){
		var cos, sin;
		cos = ((this.start.x - this.end.x) * (otherSegment.end.x - otherSegment.start.x) + (this.start.y - this.end.y) * (otherSegment.end.y - otherSegment.start.y)) / (Math.sqrt(this.end.getPreciseDistance2(this.start)) * Math.sqrt(otherSegment.end.getPreciseDistance2(otherSegment.start)));
		sin = (this.start.x - this.end.x) * (otherSegment.end.y - otherSegment.start.y) - (this.start.y - this.end.y) * (otherSegment.end.x - otherSegment.start.x);
		if (sin < 0)
			return 1 + cos;
		return -(1 + cos);
	},

	crossProduct: function (otherSegment){
		return fromInt((this.start.x - this.end.x) * (otherSegment.end.y - this.end.y) - (this.start.y - this.end.y) * (otherSegment.end.x - this.end.x));
	},

	getSlope: function (){
		if (this.end.x - this.start.x >= 0)
			return this.end.y - this.start.y;
		else 
			return -(this.end.y - this.start.y);
	},

	productSign: function (x, y){
		if (eq(x, P0_longLit) || eq(y, P0_longLit)) {
			return 0;
		}
		else if (compare_0(x, P0_longLit) < 0 ^ compare_0(y, P0_longLit) < 0) {
			return -1;
		}
		return 1;
	},
		
	linesIntersect: function (ux, uy, vx, vy, sx, sy, tx, ty){
		var stX, stY, usX, usY, utX, utY, vsX, vsY, vuX, vuY;
		usX = sub(fromInt(ux), fromInt(sx));
		usY = sub(fromInt(uy), fromInt(sy));
		vsX = sub(fromInt(vx), fromInt(sx));
		vsY = sub(fromInt(vy), fromInt(sy));
		stX = sub(fromInt(sx), fromInt(tx));
		stY = sub(fromInt(sy), fromInt(ty));
		if (this.productSign(sub(mul(vsX, stY), mul(stX, vsY)), sub(mul(stX, usY), mul(usX, stY))) >= 0) {
			vuX = sub(fromInt(vx), fromInt(ux));
			vuY = sub(fromInt(vy), fromInt(uy));
			utX = sub(fromInt(ux), fromInt(tx));
			utY = sub(fromInt(uy), fromInt(ty));
			return this.productSign(sub(mul(neg(usX), vuY), mul(vuX, neg(usY))), sub(mul(vuX, utY), mul(utX, vuY))) <= 0;
		}
		return false;
	},
		
	intersects: function (sx, sy, tx, ty){
		return this.linesIntersect(this.start.x, this.start.y, this.end.x, this.end.y, sx, sy, tx, ty);
	},

	intersects_0: function (s, t){
		return this.intersects(this, s.x, s.y, t.x, t.y);
	}
});