/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * Copyright IBM Corp. 2010, 2011. All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("rmpw.diagrams.layout.client.routers.ShortestPathConnectionRouter");

dojo.require("rmpw.diagrams.layout.client.routers.AbstractRouter");
dojo.require("rmpw.diagrams.layout.client.routers.ShortestPathRouter");

dojo.declare("rmpw.diagrams.layout.client.routers.ShortestPathConnectionRouter", rmpw.diagrams.layout.client.routers.AbstractRouter, {
	constructor: function (container) {
		this.algorithm = new rmpw.diagrams.layout.client.routers.ShortestPathRouter();
		this.staleConnections = {size:0};
		this.algorithm = new rmpw.diagrams.layout.client.routers.ShortestPathRouter();
		this.connectionsToRoutedPoints = {};
		this.figuresToBounds = {};
		this.connectionToPaths = {size:0};
		this.hookAll();
		
		this._connectAddNode = dojo.connect(container, "addNode", dojo.hitch(this, function(node) {
			this.addChild(node);
		}));
		
		this._connectRemoveNode = dojo.connect(container, "removeNode", dojo.hitch(this, function(node) {
			this.removeChild(node);
		}));
	},
	
	destroy: function() {
		if (this._connectAddNode) {
			dojo.disconnect(this._connectAddNode);
			this._connectAddNode = null;
		}
		if (this._connectRemoveNode) {
			dojo.disconnect(this._connectRemoveNode);
			this._connectRemoveNode = null;
		}
		
		this.unhookAll();
	},
	
	addChild: function (child){
		var bounds;
		if (child.mapping.policies.graphFloatingLabel) {
			return;
		}
		if (this.figuresToBounds[child.getID()]) {
			return;
		}
		
		bounds = child.getBounds();
		this.algorithm.addObstacle_0(bounds);
		this.figuresToBounds[child.getID()] = bounds;
		this.isDirty = true;
		
		child.__connectEnforce = dojo.connect(child.mapping.policies.graphElement, "enforce", dojo.hitch(this, function() {
			this.invalidateOverlappingEdges(child, false);
		}));
		
		//  connect to set bounds of nodes
		child.__connectSetBounds = dojo.connect(child, "setBounds", dojo.hitch(this, function(rect) {
			var newBounds = new rmpw.diagrams.render.util.Rectangle(rect.x, rect.y, rect.width, rect.height);
			var oldBounds = this.figuresToBounds[child.getID()];
			
			this.algorithm.updateObstacle(oldBounds, newBounds);
			this.invalidateOverlappingEdges(child, true);
			this.figuresToBounds[child.getID()] = newBounds;
		}));
	},
	
	removeChild: function(child) {
		if (child.mapping.policies.graphFloatingLabel) {
			return;
		}
		var bounds = child.getBounds().getCopy();
		var change = this.algorithm.removeObstacle(bounds);
		delete this.figuresToBounds[child.getID()];
		
		dojo.disconnect(child.__connectEnforce);
		dojo.disconnect(child.__connectSetBounds);
		
		if (change) {
			this.invalidateOverlappingEdges(child, false);
		}
	},
	
	/*
	 * Add all nodes in the graph to the shortest path algorithm
	 */
	hookAll: function() {
		var nodes = this._container.nodes;
		
		for (var i = 0, len = nodes.size; i < len; i++) {
			var node = $getNode(nodes, i);
			
			this.addChild(node);
		}
	},
	
	/*
	 * Remove all nodes in the graph from the shortest path algorithm
	 */
	unhookAll: function() {
		var nodes = this._container.nodes;
		
		for (var i = 0, len = nodes.size; i < len; i++) {
			var node = $getNode(nodes, i);
			
			this.removeChild(node);
		}
	},
	
	invalidate: function (connection){
		if (this.staleConnections[connection.getID()] === undefined &&
			connection.isAvoidingObstructions()) {
			this.staleConnections[connection.getID()] = connection;
			this.staleConnections.size++;
		}
	},
	
	invalidateOverlappingEdges: function(node, exact) {
		if (node.mapping.policies && node.mapping.policies.graphElement) {
			var mapping = node.mapping;
			var graphManager = mapping.policies.graphElement.getGraphManager();
			var neighbourMappings = graphManager.getNearMappings(mapping, ["graphEdge"], exact ? graphManager.INTERSECT_EXACT : graphManager.INTERSECT_BOUNDS);
			
			dojo.forEach(neighbourMappings, function(edgeMapping) {
				if (edgeMapping.policies.graphElement.getInternalGraphElement().isAvoidingObstructions()) {
					edgeMapping.invalidate();
				}
			});
		}
	},
	
	primRemove : function(conn) {
		if (this.connectionToPaths == null) {
			return;
		}
		
		var path = this.connectionToPaths[conn.getID()];
		if (path) {
			delete this.connectionToPaths[conn.getID()];
			this.connectionToPaths.size--;
			this.algorithm.removePath(path);
		}
	},
	
	remove : function(conn) {
		this.inherited(arguments);
		
		if (this.staleConnections[conn.getID()]) {
			delete this.staleConnections[conn.getID()];
			this.staleConnections.size--;
		}
		
		this.primRemove(conn);
	},
		
	processStaleConnectionsForObstructions: function (){
		var conn, iter, path, outerIter, entry;
		var runAlgorithm = false;
		
		if (this.staleConnections.size > 0) {
			this.connectionsToRoutedPoints = {};
		}
		
		for (var connID in this.staleConnections) {
			var conn = this.staleConnections[connID];
			if (conn.mapping) {
				var intersectingNodes = [];
				if (conn.mapping.policies && conn.mapping.policies.graphElement) {
					var points = this.getPointsFromConstraint(conn);
					var mapping = conn.mapping;
					var graphManager = mapping.policies.graphElement.getGraphManager();
					intersectingNodes = graphManager.getNearFigures(mapping, points, ["graphNode"], graphManager.INTERSECT_EXACT);
				}
				
				if (intersectingNodes.length == 0) {
					var points = this.getPointsFromConstraint(conn);
					this.connectionsToRoutedPoints[conn.getID()] = points;
					
					this.primRemove(conn);
				}
				else {
					var path = this.connectionToPaths[conn.getID()];
					if (!path) {
						path = new rmpw.diagrams.layout.client.routers.Path(null, null, conn);
						this.connectionToPaths[conn.getID()] = path;
						this.connectionToPaths.size++;
						this.algorithm.addPath(path);
					}
					
					var start = conn.getSourceAnchor().getReferencePoint().getCopy();
					var end = conn.getTargetAnchor().getReferencePoint().getCopy();

					this._container.translateToRelative(start);
					this._container.translateToRelative(end);
					
					path.setStartPoint(start);
					path.setEndPoint(end);
					path.bendpoints = null;
					runAlgorithm = true;
				}
			}
		}
		
		this.staleConnections = {size: 0};
		return runAlgorithm;
	},
	
	calculateBendPoints: function (conn) {
		var current, end, i, path, points, ref1, ref2, start, updated;
		
		if (this.processStaleConnectionsForObstructions()) {
			updated = this.algorithm.solve_1();
			
			for (i = 0; i < updated.size_0(); ++i) {
				path = $get_0(updated, i);
				current = path.data;
				points = path.points.getCopy();
				ref1 = points.getPoint( 1).getCopy();
				ref2 = points.getPoint( points.size - 2).getCopy();
				current.translateToAbsolute(ref1);
				current.translateToAbsolute(ref2);
				start = current.getSourceAnchor().getLocation(ref1).getCopy();
				end = current.getTargetAnchor().getLocation(ref2).getCopy();
				current.translateToRelative(start);
				current.translateToRelative(end);
				points.setPoint(start, 0);
				points.setPoint(end, points.size - 1);
				this.limitAvoidance(start,end,points);
				this.connectionsToRoutedPoints[current.getID()] = points;
			}
		}

		points = this.connectionsToRoutedPoints[conn.getID()];
		if (!points) {
			points = this.getPointsFromConstraint(conn);
		}
		
		delete this.connectionsToRoutedPoints[conn.getID()];
		
		return points;
	},
	
	limitAvoidance: function (start,end,points){
		if (points && points.size > 2) {
			var d = 0.0;
			var p1 = start;
			var limit = false;
			var threshold = start.getDistance(end) * 4.0;
			for (var i = 1; i < points.size && !limit; i++) {
				var p2 = points.getPoint(i);
				d += p1.getDistance(p2);
				limit = d > threshold;
				p1 = p2;
			}
			if (limit) {
				points.removeAllPoints();
				points.addPoint_1(start);
				points.addPoint_1(end);
			}
		}
	},
	
	route: function (conn){
		conn.setPointList(this.calculateBendPoints(conn));
	}
});