/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * Copyright IBM Corp. 2010, 2011. All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("rmpw.diagrams.layout.client.routers.ShortestPathRouter");

dojo.require("rmpw.diagrams.render.util.Point");
dojo.require("rmpw.diagrams.render.util.Rectangle");
dojo.require("rmpw.diagrams.layout.client.routers.Path");
dojo.require("rmpw.diagrams.layout.client.routers.Obstacle");

dojo.declare("rmpw.diagrams.layout.client.routers.$ShortestPathRouter$PathStack", null, {
	constructor: function () {
		this.prototype = new ArrayList();
		$ArrayList(this);
		this.typeId$ = 94;
	},
	
	pop: function (){
		return $remove_2(this, this.size - 1);
	}
});

dojo.declare("rmpw.diagrams.layout.client.routers.ShortestPathRouter", null, {
	
	constructor: function (){
		this.userPaths = $ArrayList(new ArrayList());
		this.workingPaths = $ArrayList(new ArrayList());
		this.pathsToChildPaths = $HashMap(new HashMap());
		this.userObstacles = $ArrayList(new ArrayList());
		
		this.growPassChangedObstacles = false;
		this.orderedPaths = null;
		this.spacing = 20;
		this.stack = null;
		this.subPaths = null;
	},

	addObstacle_0: function (rect){
		return this.internalAddObstacle(new rmpw.diagrams.layout.client.routers.Obstacle(rect.x, rect.y, rect.width, rect.height, this));
	},

	addPath: function (path){
		$add_0(this.userPaths, path);
		$add_0(this.workingPaths, path);
	},

	bendPaths: function (){
		var i, path, segment, v, vertex;
		for (i = 0; i < this.orderedPaths.size; ++i) {
			path = $get_0(this.orderedPaths, i);
			segment = null;
			path.points.addPoint_1( new rmpw.diagrams.render.util.Point( path.start.x, path.start.y));
			for (v = 0; v < path.grownSegments.size; ++v) {
				segment = $get_0(path.grownSegments, v);
				vertex = segment.end;
				if (!!vertex && v < path.grownSegments.size - 1) {
					if (vertex.type == 1) {
						++vertex.count;
						path.points.addPoint_1( vertex.bend(vertex.count));
					}
					else {
						path.points.addPoint_1( vertex.bend(vertex.totalCount));
						--vertex.totalCount;
					}
				}
			}
			path.points.addPoint_1( new rmpw.diagrams.render.util.Point( path.end.x, path.end.y));
		}
	},

	checkVertexForIntersections: function (vertex){
		var o, obs, pos, r, sideLength, x, xDist, y, yDist;
		if (vertex.nearestObstacle != 0 || vertex.nearestObstacleChecked)
			return;
		sideLength = 2 * (vertex.totalCount * this.spacing) + 1;
		if ((vertex.positionOnObstacle & 1) > 0)
			y = vertex.y - sideLength;
		else 
			y = vertex.y;
		if ((vertex.positionOnObstacle & 16) > 0)
			x = vertex.x;
		else 
			x = vertex.x - sideLength;
		r = new rmpw.diagrams.render.util.Rectangle(x, y, sideLength, sideLength);
		for (o = 0; o < this.userObstacles.size; ++o) {
			obs = $get_0(this.userObstacles, o);
			if (obs != vertex.obs && (obs.x < r.x + r.width && obs.y < r.y + r.height && obs.x + obs.width > r.x && obs.y + obs.height > r.y)) {
				pos = obs.getPosition_0(vertex);
				if (pos == 0)
					continue;
				if ((pos & 1) > 0)
					yDist = obs.y - vertex.y;
				else 
					yDist = vertex.y - (obs.y + obs.height) + 1;
				if ((pos & 16) > 0)
					xDist = vertex.x - (obs.x + obs.width) + 1;
				else 
					xDist = obs.x - vertex.x;
				if ((xDist > yDist?xDist:yDist) < vertex.nearestObstacle || vertex.nearestObstacle == 0) {
					vertex.nearestObstacle = xDist > yDist?xDist:yDist;
					vertex.updateOffset();
				}
			}
		}
		vertex.nearestObstacleChecked = true;
	},

	checkVertexIntersections: function (){
		var i, path, s, vertex;
		for (i = 0; i < this.workingPaths.size; ++i) {
			path = $get_0(this.workingPaths, i);
			for (s = 0; s < path.segments.size - 1; ++s) {
				vertex = $get_0(path.segments, s).end;
				this.checkVertexForIntersections(vertex);
			}
		}
	},

	cleanup: function (){
		var i, path;
		for (i = 0; i < this.workingPaths.size; ++i) {
			path = $get_0(this.workingPaths, i);
			$clearImpl(path.visibleVertices.map);
		}
	},

	countVertices: function (){
		var i, path, v;
		for (i = 0; i < this.workingPaths.size; ++i) {
			path = $get_0(this.workingPaths, i);
			for (v = 0; v < path.segments.size - 1; ++v)
				++$get_0(path.segments, v).end.totalCount;
		}
	},

	dirtyPathsOn: function (vertex){
		var i, paths;
		paths = vertex.paths;
		if (!!paths && paths.size != 0) {
			for (i = 0; i < paths.size; ++i)
			{
				checkIndex(i, paths.size);
				paths.array[i].isDirty = true;
			}
			return true;
		}
		return false;
	},

	getNearestVertex: function (v1, v2, segment){
		if (Math.sqrt(segment.start.getPreciseDistance2(v1)) + Math.sqrt(segment.end.getPreciseDistance2(v1)) > Math.sqrt(segment.start.getPreciseDistance2(v2)) + Math.sqrt(segment.end.getPreciseDistance2(v2)))
			return v2;
		else 
			return v1;
	},

	growObstacles: function (){
		var i;
		this.growPassChangedObstacles = false;
		for (i = 0; i < 2; ++i) {
			if (i == 0 || this.growPassChangedObstacles)
				this.growObstaclesPass();
		}
	},

	growObstaclesPass: function (){
		var counter, currentSegments, e, i, path, s;
		for (i = 0; i < this.userObstacles.size; ++i)
			$get_0(this.userObstacles, i).growVertices();
		for (i = 0; i < this.workingPaths.size; ++i) {
			path = $get_0(this.workingPaths, i);
			for (e = 0; e < path.excludedObstacles.size; ++e)
				$get_0(path.excludedObstacles, e).exclude = true;
			if (path.grownSegments.size == 0) {
				for (s = 0; s < path.segments.size; ++s)
					this.testOffsetSegmentForIntersections($get_0(path.segments, s), -1, path);
			}
			else {
				counter = 0;
				currentSegments = $ArrayList_1(new ArrayList(), path.grownSegments);
				for (s = 0; s < currentSegments.size; ++s)
				{
					checkIndex(s, currentSegments.size);
					counter += this.testOffsetSegmentForIntersections(currentSegments.array[s], s + counter, path );
				}
			}
			for (e = 0; e < path.excludedObstacles.size; ++e)
				$get_0(path.excludedObstacles, e).exclude = false;
		}
		for (i = 0; i < this.userObstacles.size; ++i)
			$get_0(this.userObstacles, i).shrinkVertices();
	},

	internalAddObstacle: function (obs){
		$add_0(this.userObstacles, obs);
		return this.testAndDirtyPaths(obs);
	},

	removeObstacle: function (rect){
		var i, index, obs, p, path, result;
		obs = null;
		index = -1;
		for (i = 0; i < this.userObstacles.size; ++i) {
			obs = $get_0(this.userObstacles, i);
			if (obs.equals$(rect)) {
				index = i;
				break;
			}
		}
		
		if (index == -1) {
			return;
		}
		
		$remove_2(this.userObstacles, index);
		result = false;
		result |= this.dirtyPathsOn(obs.bottomLeft);
		result |= this.dirtyPathsOn(obs.topLeft);
		result |= this.dirtyPathsOn(obs.bottomRight);
		result |= this.dirtyPathsOn(obs.topRight);
		for (p = 0; p < this.workingPaths.size; ++p) {
			path = $get_0(this.workingPaths, p);
			if (path.isDirty)
				continue;
			if ($containsKey(path.visibleObstacles.map, obs))
				path.isDirty = result = true;
		}
		return result;
	},

	labelPath: function (path){
		var agree, crossProduct, i, nextPath, nextSegment, segment, v, vertex, newPath, newPath_0;
		segment = null;
		nextSegment = null;
		vertex = null;
		agree = false;
		for (v = 0; v < path.grownSegments.size - 1; ++v) {
			segment = $get_0(path.grownSegments, v);
			nextSegment = $get_0(path.grownSegments, v + 1);
			vertex = segment.end;
			crossProduct = segment.crossProduct(new rmpw.diagrams.layout.client.routers.Segment(vertex, vertex.obs.center));
			if (vertex.type == 0) {
				this.labelVertex(segment, crossProduct, path);
			}
			else if (!path.isInverted && (compare_0(crossProduct, P0_longLit) > 0 && vertex.type == 2 || compare_0(crossProduct, P0_longLit) < 0 && vertex.type == 1)) {
				if (agree) {
					newPath = path.getSubPath(segment);
					$add_0(this.workingPaths, newPath);
					$add_0(this.subPaths, newPath);
					$add_0(this.stack, newPath );
					return;
				}
				else {
					path.isInverted = true;
					path.invertPriorVertices(segment);
				}
			}
			else if (path.isInverted && (compare_0(crossProduct, P0_longLit) < 0 && vertex.type == 2 || compare_0(crossProduct, P0_longLit) > 0 && vertex.type == 1)) {
				newPath_0 = path.getSubPath(segment);
				$add_0(this.workingPaths, newPath_0);
				$add_0(this.subPaths, newPath_0);
				$add_0(this.stack, newPath_0 );
				return;
			}
			else 
				agree = true;
			if (vertex.paths) {
				for (i = 0; i < vertex.paths.size; ++i) {
					nextPath = $get_0(vertex.paths, i);
					if (!nextPath.isMarked) {
						nextPath.isMarked = true;
						$add_0(this.stack, nextPath);
					}
				}
			}
			vertex.addPath_0(path, segment, nextSegment);
		}
	},

	labelPaths: function (){
		var i, path;
		path = null;
		for (i = 0; i < this.workingPaths.size; ++i) {
			path = $get_0(this.workingPaths, i);
			$add_0(this.stack, path);
		}
		while (this.stack.size != 0) {
			path = this.stack.pop();
			if (!path.isMarked) {
				path.isMarked = true;
				this.labelPath(path);
			}
		}
		for (i = 0; i < this.workingPaths.size; ++i) {
			path = $get_0(this.workingPaths, i);
			path.isMarked = false;
		}
	},

	labelVertex: function (segment, crossProduct, path){
		if (compare_0(crossProduct, P0_longLit) > 0) {
			if (path.isInverted)
				segment.end.type = 2;
			else 
				segment.end.type = 1;
		}
		else if (compare_0(crossProduct, P0_longLit) < 0) {
			if (path.isInverted)
				segment.end.type = 1;
			else 
				segment.end.type = 2;
		}
		else if (segment.start.type != 0)
			segment.end.type = segment.start.type;
		else 
			segment.end.type = 1;
	},

	orderPath: function (path){
		var i, otherAngle, segment, thisAngle, v, vPath, vertex;
		if (path.isMarked)
			return;
		path.isMarked = true;
		segment = null;
		vertex = null;
		for (v = 0; v < path.grownSegments.size - 1; ++v) {
			segment = $get_0(path.grownSegments, v);
			vertex = segment.end;
			thisAngle = $get(vertex.cachedCosines, path).value;
			if (path.isInverted)
				thisAngle = -thisAngle;
			for (i = 0; i < vertex.paths.size; ++i) {
				vPath = $get_0(vertex.paths, i);
				if (!vPath.isMarked) {
					otherAngle = $get(vertex.cachedCosines, vPath).value;
					if (vPath.isInverted)
						otherAngle = -otherAngle;
					if (otherAngle < thisAngle)
						this.orderPath(vPath);
				}
			}
		}
		$add_0(this.orderedPaths, path);
	},

	orderPaths: function (){
		var i, path;
		for (i = 0; i < this.workingPaths.size; ++i) {
			path = $get_0(this.workingPaths, i);
			this.orderPath(path);
		}
	},

	recombineChildrenPaths: function (){
		var childPath, childPaths, i, keyItr, path, outerIter, entry;
		outerIter = $AbstractHashMap$EntrySetIterator(new AbstractHashMap$EntrySetIterator(), $keySet(this.pathsToChildPaths).val$entrySet.this$0 );
		keyItr = $AbstractMap$1$1(new AbstractMap$1$1(), outerIter);
		while ($hasNext(keyItr.val$outerIter.iter)) {
			entry = $next(keyItr.val$outerIter);
			path = entry.getKey();
			path.fullReset();
			childPaths = $get(this.pathsToChildPaths, path);
			childPath = null;
			for (i = 0; i < childPaths.size_0(); ++i) {
				childPath = childPaths.get(i);
				path.points.addAll(childPath.points);
				path.points.removePoint(path.points.size - 1);
				$addAll_0(path.segments, childPath.segments);
				path.visibleObstacles.addAll(childPath.visibleObstacles);
			}
			path.points.addPoint_1( childPath.points.getLastPoint());
		}
	},

	recombineSubpaths: function (){
		var p, path;
		for (p = 0; p < this.orderedPaths.size; ++p) {
			path = $get_0(this.orderedPaths, p);
			path.reconnectSubPaths();
		}
		$removeAll(this.orderedPaths, this.subPaths);
		$removeAll(this.workingPaths, this.subPaths);
		this.subPaths = null;
	},

	refreshChildrenEndpoints: function (path, children){
		var bendpoints, child, i, next, previous;
		previous = path.start;
		bendpoints = path.bendpoints;
		for (i = 0; i < children.size_0(); ++i) {
			if (i < bendpoints.size)
				next = bendpoints.getPoint(i);
			else 
				next = path.end;
			child = children.get(i);
			child.setStartPoint(previous);
			child.setEndPoint(next);
			previous = next;
		}
	},

	regenerateChildPaths: function ( path, children, currentSize, newSize){
		var child;
		if (currentSize == 1) {
			$remove_3(this.workingPaths, path);
			currentSize = 0;
			children = $ArrayList_0(new ArrayList(), newSize);
			$put(this.pathsToChildPaths, path, children);
		}
		else if (newSize == 1) {
			$removeAll(this.workingPaths, children);
			$add_0(this.workingPaths, path);
			$remove_0(this.pathsToChildPaths, path);
			return $ArrayList_0(new ArrayList(), 0);
		}
		while (currentSize < newSize) {
			child = new rmpw.diagrams.layout.client.routers.Path();
			$add_0(this.workingPaths, child);
			children.add_0(child);
			++currentSize;
		}
		while (currentSize > newSize) {
			child = children.remove_0(children.size_0() - 1);
			$remove_3(this.workingPaths, child);
			--currentSize;
		}
		return children;
	},

	removePath: function (path){
		var children;
		$remove_3(this.userPaths, path);
		children = $get(this.pathsToChildPaths, path);
		if (!children)
			$remove_3(this.workingPaths, path);
		else 
			$removeAll(this.workingPaths, children);
		return true;
	},

	resetObstacleExclusions: function (){
		var i;
		for (i = 0; i < this.userObstacles.size; ++i)
			$get_0(this.userObstacles, i).exclude = false;
	},

	resetVertices: function (){
		var i, obs, path;
		for (i = 0; i < this.userObstacles.size; ++i) {
			obs = $get_0(this.userObstacles, i);
			obs.topLeft.fullReset_0();
			obs.bottomLeft.fullReset_0();
			obs.bottomRight.fullReset_0();
			obs.topRight.fullReset_0();
		}
		for (i = 0; i < this.workingPaths.size; ++i) {
			path = $get_0(this.workingPaths, i);
			path.start.fullReset_0();
			path.end.fullReset_0();
		}
	},

	solve_1: function (){
		this.solveDirtyPaths();
		this.countVertices();
		this.checkVertexIntersections();
		this.growObstacles();
		this.subPaths = $ArrayList(new ArrayList());
		this.stack = new rmpw.diagrams.layout.client.routers.$ShortestPathRouter$PathStack();
		this.labelPaths();
		this.stack = null;
		this.orderedPaths = $ArrayList(new ArrayList());
		this.orderPaths();
		this.bendPaths();
		this.recombineSubpaths();
		this.orderedPaths = null;
		this.subPaths = null;
		this.recombineChildrenPaths();
		this.cleanup();
		
		var ret = $ArrayList(new ArrayList());
		ret.addAll(this.userPaths);
		return ret;
	},

	solveDirtyPaths: function (){
		var children, i, newCount, numSolved, path, pathFoundCheck, prevCount;
		numSolved = 0;
		for (i = 0; i < this.userPaths.size; ++i) {
			path = $get_0(this.userPaths, i);
			if (!path.isDirty)
				continue;
			children = $get(this.pathsToChildPaths, path);
			prevCount = 1;
			newCount = 1;
			if (!children)
			{
				children = $ArrayList_0(new ArrayList(), 0);
			}
			else 
				prevCount = children.size_0();
			if (path.bendpoints)
				newCount = path.bendpoints.size + 1;
			if (prevCount != newCount)
				children = this.regenerateChildPaths(path, children, prevCount, newCount);
			this.refreshChildrenEndpoints(path, children);
		}
		for (i = 0; i < this.workingPaths.size; ++i) {
			path = $get_0(this.workingPaths, i);
			path.refreshExcludedObstacles(this.userObstacles);
			if (!path.isDirty) {
				path.resetPartial();
				continue;
			}
			++numSolved;
			path.fullReset();
			pathFoundCheck = path.generateShortestPath(this.userObstacles);
			if (!pathFoundCheck || path.end.cost > path.threshold) {
				this.resetVertices();
				path.fullReset();
				path.threshold = 0;
				pathFoundCheck = path.generateShortestPath(this.userObstacles);
			}
			this.resetVertices();
		}
		this.resetObstacleExclusions();
		if (numSolved == 0)
			this.resetVertices();
		return numSolved;
	},

	testAndDirtyPaths: function (obs){
		var i, path, result;
		result = false;
		for (i = 0; i < this.workingPaths.size; ++i) {
			path = $get_0(this.workingPaths, i);
			result |= path.testAndSet(obs);
		}
		return result;
	},

	testOffsetSegmentForIntersections: function (segment, index, path){
		var endRect, i, newSegmentEnd, newSegmentStart, obs, offset, startRect, vRect, vertex;
		for (i = 0; i < this.userObstacles.size; ++i) {
			obs = $get_0(this.userObstacles, i);
			if (segment.end.obs == obs || segment.start.obs == obs || obs.exclude)
				continue;
			vertex = null;
			offset = this.spacing;
			if (segment.getSlope() < 0) {
				if (segment.intersects(obs.topLeft.x - offset, obs.topLeft.y - offset, obs.bottomRight.x + offset, obs.bottomRight.y + offset))
					vertex = this.getNearestVertex(obs.topLeft, obs.bottomRight, segment);
				else if (segment.intersects(obs.bottomLeft.x - offset, obs.bottomLeft.y + offset, obs.topRight.x + offset, obs.topRight.y - offset))
					vertex = this.getNearestVertex(obs.bottomLeft, obs.topRight, segment);
			}
			else {
				if (segment.intersects(obs.bottomLeft.x - offset, obs.bottomLeft.y + offset, obs.topRight.x + offset, obs.topRight.y - offset))
					vertex = this.getNearestVertex(obs.bottomLeft, obs.topRight, segment);
				else if (segment.intersects(obs.topLeft.x - offset, obs.topLeft.y - offset, obs.bottomRight.x + offset, obs.bottomRight.y + offset))
					vertex = this.getNearestVertex(obs.topLeft, obs.bottomRight, segment);
			}
			if (vertex) {
				vRect = vertex.getDeformedRectangle(offset);
				if (segment.end.obs) {
					endRect = segment.end.getDeformedRectangle(offset);
					if (endRect.x < vRect.x + vRect.width && endRect.y < vRect.y + vRect.height && endRect.x + endRect.width > vRect.x && endRect.y + endRect.height > vRect.y)
						continue;
				}
				if (segment.start.obs) {
					startRect = segment.start.getDeformedRectangle(offset);
					if (startRect.x < vRect.x + vRect.width && startRect.y < vRect.y + vRect.height && startRect.x + startRect.width > vRect.x && startRect.y + startRect.height > vRect.y)
						continue;
				}
				newSegmentStart = new rmpw.diagrams.layout.client.routers.Segment(segment.start, vertex);
				newSegmentEnd = new rmpw.diagrams.layout.client.routers.Segment(vertex, segment.end);
				++vertex.totalCount;
				vertex.nearestObstacleChecked = false;
				vertex.x = vertex.origX;
				vertex.y = vertex.origY;
				this.checkVertexForIntersections(vertex);
				vertex.grow();
				if (vertex.nearestObstacle != 0)
					vertex.updateOffset();
				this.growPassChangedObstacles = true;
				if (index != -1) {
					$remove_3(path.grownSegments, segment);
					$add(path.grownSegments, index, newSegmentStart);
					$add(path.grownSegments, index + 1, newSegmentEnd);
				}
				else {
					$add_0(path.grownSegments, newSegmentStart);
					$add_0(path.grownSegments, newSegmentEnd);
				}
				return 1;
			}
		}
		if (index == -1)
			$add_0(path.grownSegments, segment);
		return 0;
	},

	updateObstacle: function (oldBounds, newBounds){
		var result;
		result = this.removeObstacle(oldBounds);
		result |= this.internalAddObstacle(new rmpw.diagrams.layout.client.routers.Obstacle(newBounds.x, newBounds.y, newBounds.width, newBounds.height, this));
		return result;
	}
});