/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * Copyright IBM Corp. 2010, 2011. All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("rmpw.diagrams.layout.client.routers.TreeRouter");

dojo.require("rmpw.diagrams.layout.client.routers.BendpointConnectionRouter");
dojo.require("rmpw.diagrams.render.util.LineSeg");
dojo.require("rmpw.diagrams.render.graph.BaseGraphConfigurator");
dojo.require("rmpw.diagrams.render.util.Point");

dojo.declare("rmpw.diagrams.layout.client.routers.BranchRouter", rmpw.diagrams.layout.client.routers.AbstractRouter, {
	constructor: function (tree){
		this.tree = tree;
	},
	
	getSourceBranchOffset: function (conn){
		var branchOffset, constraintPoints, ptLoc, ptSourceRef;
		ptSourceRef = conn.getSourceAnchor().getReferencePoint();
		conn.translateToRelative(ptSourceRef);
		branchOffset = ptSourceRef.x;
		constraintPoints = this.tree.getPointsFromConstraint(conn);
		if (this.tree.isTreeBranch(conn, constraintPoints)) {
			ptLoc = constraintPoints.getPoint(0);
			branchOffset = ptLoc.x;
		}
		return branchOffset - ptSourceRef.x;
	},
	
	getSourceLocation: function (conn, ptTrunkLoc){
		var branchOffset, line, ptSourceRef, ref;
		ptSourceRef = conn.getSourceAnchor().getReferencePoint();
		conn.translateToRelative(ptSourceRef);
		branchOffset = this.getSourceBranchOffset(conn) + ptSourceRef.x;
		ref = new rmpw.diagrams.render.util.Point( branchOffset, ptTrunkLoc.y);
		line = this.getOrthogonalLineSegToAnchorLoc(conn, conn.getSourceAnchor(), ref);
		return line.origin.getCopy();
	},
	
	getTrunkLocation: function (conn){
		var points, ptTrunkLoc;
		points = this.tree.getPointsFromConstraint(conn);
		ptTrunkLoc = this.tree.getTrunkLocation_0(conn);
		if (this.tree.isTreeBranch(conn, points)) {
			ptTrunkLoc.x = points.getPoint( 3).x;
			if (this.tree.isOrthogonalTreeBranch(conn, points)) {
				ptTrunkLoc.y = points.getPoint( 2).y;
			}
		}
		return ptTrunkLoc;
	},
	
	recreateBranch: function (conn, ptSourceLoc, ptTrunkLoc){
		var line, points, pt2, pt4, ptTargetLoc;
		points = new rmpw.diagrams.render.util.PointList(4);
		points.addPoint_1(ptSourceLoc.getCopy());
		pt2 = new rmpw.diagrams.render.util.Point( ptSourceLoc.x, ptTrunkLoc.y);
		points.addPoint_0(pt2.x, pt2.y);
		points.addPoint_1(ptTrunkLoc.getCopy());
		line = this.getOrthogonalLineSegToAnchorLoc(conn, conn.getTargetAnchor(), ptTrunkLoc);
		ptTargetLoc = line.origin.getCopy();
		pt4 = new rmpw.diagrams.render.util.Point( ptTrunkLoc.x, ptTargetLoc.y);
		points.addPoint_0(pt4.x, pt4.y);
		return points;
	},
	
	getBounds_2: function (figure){
		return figure != null && figure.getBounds();
	},
	
	getAdjustedCenterPoint: function (rect, ref){
		var ptNewCenter;
		ptNewCenter = rect.getCenter().getCopy();
		if (ref.x < new rmpw.diagrams.render.util.Point( rect.x + rect.width, rect.y + ~~(rect.height / 2)).x && ref.x > new rmpw.diagrams.render.util.Point( rect.x, rect.y + ~~(rect.height / 2)).x) {
			ptNewCenter.x = ref.x;
		}
		else if (ref.y < new rmpw.diagrams.render.util.Point( rect.x + ~~(rect.width / 2), rect.y + rect.height).y && ref.y > new rmpw.diagrams.render.util.Point( rect.x + ~~(rect.width / 2), rect.y).y) {
			ptNewCenter.y = ref.y;
		}
		return ptNewCenter;
	},
	
	getEdgePoint: function (conn, anchor, ptRef){
		var lineSeg, ptAbsRef2, ptEdge, ptProj, ptRef1, ptRef2, rect;
		rect = this.getBounds_2(anchor.getOwner());
		anchor.getOwner().translateToAbsolute(rect);
		conn.translateToRelative(rect);
		ptRef2 = ptRef.getCopy();
		ptRef1 = this.getAdjustedCenterPoint(rect, ptRef2);
		ptAbsRef2 = ptRef2.getCopy();
		conn.translateToAbsolute(ptAbsRef2);
		ptEdge = anchor.getLocation(ptAbsRef2);
		conn.translateToRelative(ptEdge);
		lineSeg = new rmpw.diagrams.render.util.LineSeg(ptRef1, ptRef2);
		ptProj = lineSeg.perpIntersect(ptEdge.x, ptEdge.y);
		if (Math.abs(ptProj.x - ptRef2.x) < Math.abs(ptProj.y - ptRef2.y))
			ptProj.x = ptRef2.x;
		else 
			ptProj.y = ptRef2.y;
		return ptProj;
	},
	
	getOrthogonalLineSegToAnchorLoc: function (conn, anchor, ref){
		var ptAbsRef, ptEdge, result;
		ptAbsRef = this.getEdgePoint(conn, anchor, ref);
		conn.translateToAbsolute(ptAbsRef);
		ptEdge = anchor.getLocation(ptAbsRef);
		conn.translateToRelative(ptEdge);
		result = new rmpw.diagrams.render.util.LineSeg(ptEdge, ref);
		if (result.origin.y != result.terminus.y && result.origin.x != result.terminus.x) {
			if (Math.abs(result.origin.getCopy().x - result.terminus.getCopy().x) < Math.abs(result.origin.getCopy().y - result.terminus.getCopy().y)) {
				result.setTerminus(new rmpw.diagrams.render.util.Point( result.origin.getCopy().x, result.terminus.getCopy().y));
			}
			else {
				result.setTerminus(new rmpw.diagrams.render.util.Point( result.terminus.getCopy().x, result.origin.getCopy().y));
			}
		}
		return result;
	},

	route: function (conn){
		var points_2, ptSourceLoc, ptTrunkLoc;
		ptTrunkLoc = this.getTrunkLocation(conn);
		this.tree.setTrunkLocation(conn, ptTrunkLoc);
		ptSourceLoc = this.getSourceLocation(conn, ptTrunkLoc);
		points_2 = this.recreateBranch(conn, ptSourceLoc, ptTrunkLoc);
		conn.setPointList(points_2);
	}
});

dojo.declare("rmpw.diagrams.layout.client.routers.SubTreeRouter", rmpw.diagrams.layout.client.routers.BendpointConnectionRouter, {
	constructor: function(){
		this.constraints = {};
		this.branchRouter = new rmpw.diagrams.layout.client.routers.BranchRouter(this);
		this.connectionList = $ArrayList(new ArrayList());
		this.trunkOrientation = null;
		this.trunkVertex = null;
		this.updatingPeers = false;
	},
	
	TOP: 0,
	BOTTOM: 1,
	RIGHT: 2,
	LEFT: 3,
	
	getEdge: function (orientation, bounds){
		if (orientation == this.TOP)
			return new rmpw.diagrams.render.util.Point( bounds.x + ~~(bounds.width / 2), bounds.y);
		else if (orientation == this.BOTTOM)
			return new rmpw.diagrams.render.util.Point( bounds.x + ~~(bounds.width / 2), bounds.y + bounds.height);
		else if (orientation == this.RIGHT)
			return new rmpw.diagrams.render.util.Point( bounds.x + bounds.width, bounds.y + ~~(bounds.height / 2));
		return new rmpw.diagrams.render.util.Point( bounds.x, bounds.y + ~~(bounds.height / 2));
	},
	
	checkTrunkVertex: function (conn){
		var default_trunk, sourceRect, targetRect;
		if (!this.trunkVertex) {
			sourceRect = conn.startAnchor.getOwner().getBounds();
			targetRect = conn.endAnchor.getOwner().getBounds();
			default_trunk = new rmpw.diagrams.render.util.Dimension(0, 32);
			conn.translateToRelative(default_trunk);
			if (sourceRect.getCenter().y < targetRect.getCenter().y) {
				this.trunkVertex = new rmpw.diagrams.render.util.Dimension(0, -default_trunk.height);
				this.trunkOrientation = this.TOP;
			}
			else {
				this.trunkVertex = new rmpw.diagrams.render.util.Dimension(0, default_trunk.height);
				this.trunkOrientation = this.BOTTOM;
			}
		}
	},
	
	getPointsFromConstraint: function (conn){
		var bendpoints, points;
		bendpoints = conn.getRoutingConstraint();
		if (!bendpoints)
			return new rmpw.diagrams.render.util.PointList();
		points = new rmpw.diagrams.render.util.PointList();
		for (var i = 0, bpt; bpt = bendpoints[i]; i++) {
			pt = bpt.getLocation();
			points.addPoint_0(pt.x, pt.y);
		}
		this.straightenPoints(points, 3);
		return points;
	},
	
	getTargetAnchorRelativeBounds: function (conn){
		var bounds;
		bounds = conn.getTargetAnchor().getOwner().getBounds();
		conn.getTargetAnchor().getOwner().translateToAbsolute(bounds);
		conn.translateToRelative(bounds);
		return bounds;
	},
	
	getTrunkLocation_0: function (conn){
		var ptTrunkLoc, target, vertex;
		vertex = this.trunkVertex;
		target = this.getEdge(this.trunkOrientation, this.getTargetAnchorRelativeBounds(conn));
		ptTrunkLoc = new rmpw.diagrams.render.util.Point( vertex.width, vertex.height);
		ptTrunkLoc = ptTrunkLoc.getCopy().translate_1(target);
		return ptTrunkLoc;
	},
	
	invalidate: function (conn){
		var connNext, li;
		if (!conn.getSourceAnchor() || !conn.getSourceAnchor().getOwner() || !conn.getTargetAnchor() || !conn.getTargetAnchor().getOwner())
			return;
		li = $AbstractList$ListIteratorImpl(new AbstractList$ListIteratorImpl(), this.connectionList);
		while (li.hasNext()) {
			connNext = li.next();
			if (!this.trunkVertexEqual(connNext, conn)) {
				this.updateConstraint(connNext, true);
			}
		}
	},
	
	isOrthogonalTreeBranch: function (conn, points){
		var branch, trunk, trunkShoulder;
		if (this.isTreeBranch(conn, points)) {
			branch = new rmpw.diagrams.render.util.LineSeg(points.getPoint( 0), points.getPoint( 1));
			trunkShoulder = new rmpw.diagrams.render.util.LineSeg(points.getPoint( 1), points.getPoint( 2));
			trunk = new rmpw.diagrams.render.util.LineSeg(points.getPoint( 2), points.getPoint( 3));
			return branch.origin.x == branch.terminus.x && trunkShoulder.origin.y == trunkShoulder.terminus.y && trunk.origin.x == trunk.terminus.x;
		}
		return false;
	},
	
	isTreeBranch: function (conn, points){
		var sourceBounds, targetBounds, bounds;
		if (points.size == 4) {
			targetBounds = this.getTargetAnchorRelativeBounds(conn);
			bounds = conn.getSourceAnchor().getOwner().getBounds();
			conn.getSourceAnchor().getOwner().translateToAbsolute(bounds);
			conn.translateToRelative(bounds);
			sourceBounds = bounds;
			return points.getPoint( 0).x > sourceBounds.x && points.getPoint( 0).x < sourceBounds.x + sourceBounds.width && (points.getPoint( 3).x > targetBounds.x && points.getPoint( 3).x < targetBounds.x + targetBounds.width);
		}
		return false;
	},
	
	remove: function (conn){
		var i, index;
		if (!conn.getSourceAnchor() || !conn.getTargetAnchor())
			return;
		index = $indexOf(this.connectionList, conn, 0);
		$remove_3(this.connectionList, conn);
		for (i = index + 1; i < this.connectionList.size; ++i)
			$get_0(this.connectionList, i).revalidate();
	},
	
	route: function (conn){
		if (!conn.startAnchor || !conn.startAnchor.getOwner() || !conn.endAnchor || !conn.endAnchor.getOwner()) {
			this.inherited(arguments);
			return;
		}
		if ($indexOf(this.connectionList, conn, 0) == -1) {
			$add_0(this.connectionList, conn);
		}
		this.checkTrunkVertex(conn);
		this.branchRouter.route(conn);
		
		var li = $AbstractList$ListIteratorImpl(new AbstractList$ListIteratorImpl(), this.connectionList);
		while (li.hasNext()) {
			connNext = li.next();
			if (!this.trunkVertexEqual(connNext, conn)) {
				this.updateConstraint(connNext, true);
			}
		}
	},
	
	setTrunkLocation: function (conn, ptTrunkLoc){
		var currentVertex, ptRelTrunkLoc, target, targetAnchorBounds;
		ptRelTrunkLoc = ptTrunkLoc.getCopy();
		targetAnchorBounds = this.getTargetAnchorRelativeBounds(conn);
		if (ptTrunkLoc.y < targetAnchorBounds.getCenter().y) {
			this.trunkOrientation = this.TOP;
		}
		else {
			this.trunkOrientation = this.BOTTOM;
		}
		target = this.getEdge(this.trunkOrientation, targetAnchorBounds);
		currentVertex = new rmpw.diagrams.render.util.Dimension(ptRelTrunkLoc.x - target.x, ptRelTrunkLoc.y - target.y);
		this.trunkVertex = currentVertex;
	},
	
	trunkVertexEqual: function (connMaster, connSlave){
		var cmPts, csPts;
		cmPts = connMaster.getPointList();
		csPts = connSlave.getPointList();
		if (cmPts.size > 2 && csPts.size > 2)
			return cmPts.getPoint(2).equals$(csPts.getPoint(2));
		return false;
	},
	
	updateConstraint: function (conn, deferUpdate){
		var $e0, bendpoints, bp, i, ptSource, ptTrunk, pts, sourceRefPoint, targetRefPoint;
		if (this.updatingPeers)
			return;
		bendpoints = [];
		if (bendpoints) {
			sourceRefPoint = conn.getSourceAnchor().getReferencePoint();
			conn.translateToRelative(sourceRefPoint);
			targetRefPoint = conn.getTargetAnchor().getReferencePoint();
			conn.translateToRelative(targetRefPoint);
			ptTrunk = this.getTrunkLocation_0(conn);
			ptSource = this.branchRouter.getSourceLocation(conn, ptTrunk);
			pts = this.branchRouter.recreateBranch(conn, ptSource, ptTrunk);
			for (i = 0; i < pts.size; ++i) {
				var pt = pts.getPoint(i);
				bendpoints.push(pt);
			}
		}
		this.updatingPeers = true;
		try {
			this.setConstraint(conn, bendpoints);
			if (deferUpdate) {
				conn.revalidate();
			}
			else {
				this.route(conn);
			}
		}
		catch ($e0) {
			$e0 = caught($e0);
			if (instanceOf($e0, 64)) {
			}
			else 
				throw $e0;
		}
		finally {
			this.updatingPeers = false;
		}
	},
	
	straightenPoints: function (newLine, tolerance){
		var i, ptCurrent, ptNext, xDelta, yDelta;
		for (i = 0; i < newLine.size - 1; ++i) {
			ptCurrent = newLine.getPoint( i);
			ptNext = newLine.getPoint( i + 1);
			xDelta = Math.abs(ptNext.x - ptCurrent.x);
			yDelta = Math.abs(ptNext.y - ptCurrent.y);
			if (xDelta < yDelta) {
				if (xDelta > tolerance)
					return;
				ptNext.x = ptCurrent.x;
			}
			else {
				if (yDelta > tolerance)
					return;
				ptNext.y = ptCurrent.y;
			}
			newLine.setPoint(ptNext, i + 1);
		}
	}
});

dojo.declare("rmpw.diagrams.layout.client.routers.TreeRouter", rmpw.diagrams.layout.client.routers.BendpointConnectionRouter, {
	
	constructor: function(){
		this.constraints = $HashMap_0(new HashMap(), 11);
		this.connections = {};
		this.trunkVertexes = {};
	},
	
	isOrthogonal : function() {
		return true;
	},
	
	getType: function() {
		return rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.TREE_ROUTING;
	},
	
	getTrunkKey: function(anchor, qualifier) {
		return ":" + anchor.getOwner().mapping.id + qualifier;
	},

	accountForTrunkOverlap: function (trunk, conn){
		  var newTrunk, ownerExt, trunkExt, trunkKey;
		  if (!conn.getTargetAnchor() || !conn.getTargetAnchor().getOwner())
		    return trunk;
		  trunkKey = this.getTrunkKey(conn.getTargetAnchor(), trunk);
		  ownerExt = ~~(conn.getTargetAnchor().getOwner().getBounds().width / 2);
		  trunkExt = trunk.width;
		  if (this.trunkVertexes[trunkKey] == null || (trunkExt < 0?-trunkExt:trunkExt) > ownerExt) {
			this.trunkVertexes[trunkKey] = true;
		    return trunk;
		  }
		   else {
		    newTrunk = trunk.getCopy();
		    newTrunk.width += 10;
		    newTrunk.height += 10;
		    return this.accountForTrunkOverlap(newTrunk, conn);
		  }
		},

	getSubRouter: function (conn){
		  var connectionKey, connectionRouter;
		  if (!conn.getTargetAnchor())
		    return null;
		  connectionKey = this.getTrunkKey(conn.getTargetAnchor(), conn.type ? conn.type : "base");
		  connectionRouter = this.connections[connectionKey];
		  if (!connectionRouter) {
		    connectionRouter = new rmpw.diagrams.layout.client.routers.SubTreeRouter();
		    this.connections[connectionKey] = connectionRouter;
		  }
		  return connectionRouter;
		},

		invalidate: function (conn){
		  var connectionRouter;
		  if (!conn.getSourceAnchor() || !conn.getTargetAnchor())
		    return;
		  connectionRouter = this.getSubRouter(conn);
		  if (connectionRouter)
			  connectionRouter.invalidate(conn);
		},

		remove: function (conn){
		  var connectionRouter;
		  if (!conn.getSourceAnchor() || !conn.getTargetAnchor())
		    return;
		  connectionRouter = this.getSubRouter(conn);
		  if (connectionRouter)
			  connectionRouter.remove(conn);
		  this.inherited(arguments);
		},

		route: function (conn){
		  var adjustedTrunk, points, precisePt, ref1, ref2, treeRouter, trunk, trunkKey;
		  if (!conn.endAnchor.getOwner() || !conn.startAnchor.getOwner()) {
		    points = conn.getPointList();
		    points.removeAllPoints();
		    ref1 = conn.endAnchor.getReferencePoint();
		    ref2 = conn.startAnchor.getReferencePoint();
		    precisePt = new rmpw.diagrams.render.util.Point();
		    precisePt.setLocation(conn.startAnchor.getLocation(ref1));
		    conn.translateToRelative(precisePt);
		    points.addPoint_0(precisePt.x, precisePt.y);
		    precisePt.setLocation(conn.endAnchor.getLocation(ref2));
		    conn.translateToRelative(precisePt);
		    points.addPoint_0(precisePt.x, precisePt.y);
		    conn.setPointList(points);
		    return;
		  }
		  treeRouter = this.getSubRouter(conn);
		  trunk = treeRouter.trunkVertex;
		  if (trunk) {
		    trunkKey = this.getTrunkKey(conn.endAnchor, trunk);
		    delete this.trunkVertexes[trunkKey];
		  }
		  if (treeRouter) {
			treeRouter.route(conn);
		    trunk = treeRouter.trunkVertex;
		    adjustedTrunk = this.accountForTrunkOverlap(trunk, conn);
		    if (!adjustedTrunk.equals$(trunk)) {
		      treeRouter.trunkVertex = adjustedTrunk;
		      treeRouter.invalidate(conn);
		    }
		  }
		}
});