/*******************************************************************************
 * Licensed Materials - Property of IBM 
 * Copyright IBM Corp. 2010, 2011. All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("rmpw.diagrams.layout.client.routers.Vertex");

dojo.require("rmpw.diagrams.render.util.Point");
dojo.require("rmpw.diagrams.render.util.Rectangle");

dojo.declare("rmpw.diagrams.layout.client.routers.Vertex", rmpw.diagrams.render.util.Point, {
	constructor: function (x, y, obs){
		this.x = x;
		this.y = y;
		this.origX = x;
		this.origY = y;
		this.obs = obs;
		this.cachedCosines = null;
		this.cost = 0;
		this.count = 0;
		this.isPermanent = false;
		this.label = null;
		this.nearestObstacle = 0;
		this.nearestObstacleChecked = false;
		this.neighbors = null;
		this.offset = 0;
		this.paths = null;
		this.positionOnObstacle = -1;
		this.totalCount = 0;
		this.type = 0;
	},
	
	addPath_0: function (path, start, end){
		if (!this.paths) {
			this.paths = $ArrayList(new ArrayList());
			this.cachedCosines = $HashMap(new HashMap());
		}
		if ($indexOf(this.paths, path, 0) == -1)
			$add_0(this.paths, path);
		$put(this.cachedCosines, path, start.cosine(end));
	},

	bend: function (modifier){
		var point;
		point = new rmpw.diagrams.render.util.Point( this.x, this.y);
		if ((this.positionOnObstacle & 1) > 0)
			point.y -= modifier * this.offset;
		else 
			point.y += modifier * this.offset;
		if ((this.positionOnObstacle & 16) > 0)
			point.x += modifier * this.offset;
		else 
			point.x -= modifier * this.offset;
		return point;
	},

	fullReset_0: function (){
		this.totalCount = 0;
		this.type = 0;
		this.count = 0;
		this.cost = 0;
		this.offset = this.getSpacing();
		this.nearestObstacle = 0;
		this.label = null;
		this.nearestObstacleChecked = false;
		this.isPermanent = false;
		if (this.neighbors)
			$clear(this.neighbors);
		if (this.cachedCosines)
			$clearImpl(this.cachedCosines);
		if (this.paths)
			$clear(this.paths);
	},

	getDeformedRectangle: function (extraOffset){
		var rect;
		rect = new rmpw.diagrams.render.util.Rectangle(0, 0, 0, 0);
		if ((this.positionOnObstacle & 1) > 0) {
			rect.y = this.y - extraOffset;
			rect.height = this.origY - this.y + extraOffset;
		}
		else {
			rect.y = this.origY;
			rect.height = this.y - this.origY + extraOffset;
		}
		if ((this.positionOnObstacle & 16) > 0) {
			rect.x = this.origX;
			rect.width = this.x - this.origX + extraOffset;
		}
		else {
			rect.x = this.x - extraOffset;
			rect.width = this.origX - this.x + extraOffset;
		}
		return rect;
	},

	getSpacing: function (){
		if (!this.obs)
			return 0;
		return this.obs.router.spacing;
	},

	grow: function (){
		var modifier;
		if (this.nearestObstacle == 0)
			modifier = this.totalCount * this.getSpacing();
		else 
			modifier = ~~(this.nearestObstacle / 2) - 1;
		if ((this.positionOnObstacle & 1) > 0)
			this.y -= modifier;
		else 
			this.y += modifier;
		if ((this.positionOnObstacle & 16) > 0)
			this.x += modifier;
		else 
			this.x -= modifier;
	},

	updateOffset: function (){
		if (this.nearestObstacle != 0)
			this.offset = ~~((~~(this.nearestObstacle / 2) - 1) / this.totalCount);
	}
});