/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("rmps.ui.sections.SectionHeader");
    
    dojo.require("rmps.ui.sections.UAHeader");

    dojo.declare("rmps.ui.sections.SectionHeader", rmps.ui.sections.UAHeader, {
    	
    	sectionActionDescriptors: null,
    	
    	/* Example:
    	 *  
	     *	  	actions: [{
		 *				img: "/sr/img.ping",
		 *				tooltip: "test",
		 *				doRun: someFunction
		 *			},
		 *			{
		 *				title: "some action",
		 *				tooltip: "test",
		 *				doRun: someFunction
		 *			}] 
    	 */
    	initializeActionDescriptors: function(actions){
    		if(actions){
				this.sectionActionDescriptors= [];
    			for(var index=0; index<actions.length; index++){
	    			var action= actions[index];
	    			if(action){
	    				var img= action.img;
	    				if(img){
	    					this.addSectionImageActionDescriptor(img, action.tooltip,action.doRun);
	    				}else{
							this.addSectionActionDescriptor(action.title, action.tooltip,action.doRun);	    					
	    				}
	    			}
    			}    			
    		}
    	},
    	
    	getSection:function(args, scope, table){/*To be Implemented By Sub-classes */},
    	
		createHeader: function(table, scope, args){
			this.initializeActionDescriptors(args.actions);
            var section = this.getSection(args, scope, table);
            
            var buttonCallback = function(_curAction){
                var doRun = _curAction.doRun;
                /*The default way for creating row is called unless someone specified their own method */
				 
                var action = doRun ? doRun : scope ?scope.addSectionTableRow: null;
                if(action){
					action(section, table);							
				}
            };
            
            if (this.sectionActionDescriptors && section.addAction) {
                for (var index = 0; index < this.sectionActionDescriptors.length; index++) {
                    var curAction = this.sectionActionDescriptors[index];
                    
                    var img= curAction.img;
                    var protoTypeHeader, protoType;
                    
                    if(img){
                    	protoType= section.addImageAction;	
                    	protoTypeHeader= img;
                    }else{
                    	protoType= section.addAction;
                    	protoTypeHeader= curAction.title;
                    }
                                
                    dojo.hitch(section,protoType)(protoTypeHeader, curAction.toolTip, dojo.hitch(this.scope, buttonCallback, curAction));
                }
            }
            return section;
		},
		
		 /*
         * If the user would like to add an action to the section itself. They need to specify the action
         * title, tooltip and the function that shows what to be done when invoking the action. For example, if
         * the user would like to have an "addAction", they can call for their row populate function
         * */
        addSectionActionDescriptor: function(_title, _tooltip, _doRun){
            this.sectionActionDescriptors.push({
                title: _title,
                toolTip: _tooltip,
                doRun: _doRun
            });
        },
        
        /*In case we would like to add an image action */
        addSectionImageActionDescriptor: function(_img, _tooltip, _doRun){
            this.sectionActionDescriptors.push({
                img: _img,
                toolTip: _tooltip,
                doRun: _doRun
            });
        }
       
    });
    
})();
