/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("rmps.ui.sections.SectionTable");

    dojo.require("jazz.ui.StyledTable");
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("rmps.ui.util.ToolbarFocusUtil");
    
    dojo.require("dojo.i18n");
    dojo.requireLocalization("rmps.ui.sections", "SectionTableMessages");
	
	var messages= dojo.i18n.getLocalization("rmps.ui.sections", "SectionTableMessages");
    var ToolbarFocusUtil = rmps.ui.util.ToolbarFocusUtil;
    
    dojo.declare("rmps.ui.sections.SectionTable", [dijit._Widget], {

		templateString: '<div dojoAttachPoint="containerNode">' + '</div>',

        columnDescriptors: null,
        actionDescriptors: null,
		dataSet: null,
		table: null,
		scope: null,
		headers: null,
		sectionHeaders: null,

        /*We must specify the scope in the params */
        constructor: function(params){
            this.columnDescriptors = [];
            this.actionDescriptors = [];
			this.dataSet= [];
			this.headers= [];
			this.sectionHeaders= [];
        },

        /* If we would like to show the headers, then, we create an instance of <code>StyledTable</code>, otherwise,
         * we create an HTML table
         * This method could be overridden for further conditions. For example:
         *      var _table = this.inherited(arguments);
         *     _table.cellPadding = 0;
         */
		createTable: function(args){
            if (this.showHeaders()) {
                return new jazz.ui.StyledTable({
                    topBorder: this.getIsBoxed(),
                    columns: args.columns,
                    trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS
                });
            }

            var _table = document.createElement("table");
			var columns= args.columns;
			for(var index=0; index< columns.length; index++){
				var width= columns[index].width;
				if(!width){
					width= this.getActionsColumnWidth();
				}
				dojo.create("col", {width: width}, _table);
			}

			//TODO: Remove
			_table.columns = args.columns;
            return _table;
		},

		/* If we would like to show the table headers, then this should return <code>true</code>; otherwise,
		 * it should return <code>false</code>
		 */
		showHeaders: function(){
			return true;
		},

		/*When we add a new row to the table, we could specify where it will be placed in the table (first,
		 * or end). If the return value if <code>true</code>, then, the new row will be added at the beginning;
		 * otherwise, it will be inserted as the end
		 */
		insertBefore: function(){
			return true;
		},

		/*The isBoxed attribute for the StyledTable */
        getIsBoxed: function(){
            /*To be implemented. Otherwise, it will return false */
            return false;
        },

        /*It could be overridden to increase/reduce the action column width */
        getActionsColumnWidth: function(){
        	return "5%";
        },


        /*This method must be implemented by other classes. We could specify the call actions for the
         * server in order to get the expected results. After that, we can use the onCall Method in order
         * to pass the gained results so, the table rows and cells will be created
         * Example:
         *	var deferred= this._Someclinet.getsomeValues();
         *	deferred.addCallback(this, function(result) {
         *			result= {
         * 						field1: "fieldValue",
         * 						field2: "field2Value"
         * 					}
         *			onCall(section, table,result);	//Note, you must pass in those arguments as it is
         *	});
         *
         *
         * */
		fillTable: function(section, table, onCall){},	/*Abstract method */


        /*The client could specify a new column to be added to the section table. They should specify the
         * column display name (columnHeader), and its expected field name that will be retrieved from the
         * method SectionTable#fillTable, and the method that create the cells for this column (creatorCallback)
         * The way of having creatorCallback for each column will help the client to have thier own conditions
         * and ways to create the column cells, and to add their rich UI if needed. Note, the user must specify
         * at least one column descriptor. In addition, you could specify the width as well, but make sure that
         * using anapplied CSS style will cause this width value to have no effect in your column descriptor; you
         * could specify the width in percentage.
         * Example:
         * 	this.addColumnDescription({columnHeader:"Name", fieldName:"nameField",
         * 												creatorCallback:this.createNameField, width: 40%});
         * this.addColumnDescription({columnHeader:"Date Of Birth", fieldName:"dobField",
         * 												creatorCallback:this.createDOBField, width: 40%});
         * this.addColumnDescription({columnHeader:"Course", fieldName:"Name",
         * 												creatorCallback:this.createCourseField, width: 20%});

         * And an example for creator @see:createDefaultCell
         *
         * NOTE: creatorCallback function must return a TD for the table row that is having this td created for,
         * otherwise, an error wil be logged, and unexpected behaviors could happen.
         *
         */
        addColumnDescriptor: function(request){
            /*Understand Request */
            var _columnHeader = request.columnHeader;
            var _fieldName = request.fieldName;
            var _width = request.width;
            var _creatorCallback = request.creatorCallback;

            /*If we do not want to specify special function for the column cell creation,then
             * we use the default cell creator */
            if (!_creatorCallback) {
                _creatorCallback = this.createDefaultCell;
            }

            this.columnDescriptors.push({
                label: _columnHeader,
                fieldName: _fieldName,
                width: _width,
                creatorCallback: _creatorCallback
            });
        },

        /*The default method to create column cells. If the user did not specify a function to create a column
         * to a column descriptor, then, we call this one. If the user would like to implement their own function
         * then, they have to make sure to follow the same signature for SectionTable#createDefaultCell. The return
         * value must be node value has been created.
         * The user does not have to worry about the parameters value as they will be received from previous methods
         */
        createDefaultCell: function(section, table, tableRow, fieldName, value, data){
            var tableData = dojo.create("td", null, tableRow);

            var sectionTable = this.table;
            var scope = this.scope;

            var newCell;
            if (data) {
                newCell = dojo.create("preview", null, tableData);
                newCell.innerHTML = value;
            }
            else {
                newCell = dojo.create("input", null, tableData);
                newCell.className = "cell";
                newCell.type = "text";
                newCell.value = value ? value : "";
                newCell.data = data;
                newCell.style.width = "100%";

                /*This field will be used to determine if the input value has been changed */
                newCell.originalValue = value;

                /*This value */
                newCell.fieldName = fieldName;
            }
            return tableData;
        },

        /*If the user would like to add an action to the Actions column. If the user did not add a new action
         * descriptor, the actions column will not be added. The function signatur must be as the follows:
         * 		createAnAction: function(section, table, tr, data){
         * 					}
         * The function must return the node has been created an appended to the cell container
         *
         * TODO: Must have tooltip
         *
         * */
        addActionDescriptor: function(_creatorCallback){
            this.actionDescriptors.push(_creatorCallback);
        },

        /*This method could be overriden if someone would like to add special conditions for the row creations
         * For Example, if we would like to add a class for the row:
         * 		var tr= this.inherited(arguments);
         * 		dojo.addClass(tr, "someCSSClass");
         * 		return tr;
         *
         */
        addSectionTableRow: function(section, table, data, index){
			var tr= null;
			var tbody= table.tbody;
			if(!tbody){
				tbody= table;
			}
			if(this.insertBefore()){
				tr = dojo.doc.createElement("tr");
				tbody.insertBefore(tr, tbody.firstChild);
			}else{
				if(parseInt(index)>-1===false){
					index= -1;
				}
				tr= tbody.insertRow(index);
			}

            var cellStore= [];
            var fieldNames=[];
            var values=[];
            var columns = table.columns;
            for (var i = 0; i < columns.length; i++) {
                var column = columns[i];
                /*The column field could be null for the actions column */
                var fieldName = column.fieldName;
                if (fieldName) {
                	var value= dojo.getObject(fieldName, false, data);
                    var cell = dojo.hitch(this, column.creatorCallback)(section, table, tr, /**</br> */
                    			fieldName, value, data);
					if (cell /*There is an enablement if we do not want to create the cell at all */) {
                        cellStore.push(cell);
                        fieldNames.push(fieldName);

                        /*This field will be used to determine if the value for a cell has been changed */
                        values[fieldName]= value;
                    }
                }
            }

			var actionNodes=[];
            if (this.actionDescriptors.length>0&&	/*If there no action defnined, then, do not create cell */
            				this.getActionsEnablement(section,table,tr,data)&&this.actionDescriptors) {
                var tableData = dojo.create("td", {
                    align: "centre",
                    width: this.getActionsColumnWidth()
                }, tr);
                var div = this.createActionsContainer(tableData);
                tableData.appendChild(div);
				/*TODO: Use getAssociatedValues */
                for (var j = 0; j < this.actionDescriptors.length; j++) {
                    var action = this.actionDescriptors[j];
                    /*Safe check for null */
                    if (action) {
                        var actionNode= dojo.hitch(this, action)(section, table, tr, tableData, div, data);
                        actionNodes[action.nom]=actionNode;
                        if(actionNode && actionNode.parentNode){
                        	actionNodes.domNode = actionNode.parentNode;
                        }
                    }
                }
            }
            this.dataSet.push({
                row: tr,
                cells: cellStore,
                fieldNames: fieldNames,
                values: values,
                data: data,
                actionNodes: actionNodes
            });
            
           if(actionNodes.domNode){
                //make the buttons accessible by keyboard
                ToolbarFocusUtil.addToolbarFocusListeners( { 
                    actionToolbar: actionNodes, 
                    containerNode: tr, 
                    actionClassName: ".action"
                }); 
        	}
            return tr;
        },

        getAssociatedValues: function(data){
            var fieldNames=[];
            var values=[];
            var table= this.table;
            var columns = table.columns;
            for (var index = 0; index < columns.length; index++) {
                var column = columns[index];
                /*The column field could be null for the actions column */
                var fieldName = column.fieldName;
                if (fieldName) {
                	var value= dojo.getObject(fieldName, false, data);
                   	values[fieldName]= value;
                }
            }
            return values;
        },

		createTableContents: function(){
			if(this.table&& this.table.destroyRecursive){
				this.table.destroyRecursive();
			}
			var table = this.createTable({columns:this.columnDescriptors});

			//TODO: Do we need to have this call in this class?
			if(table.domNode){table.domNode.style.width= "100%";}

			this.table= table;
		},

		/*Must be called after initializing an instance of this class.
		 *
		 * Must pass the scope (a widget instance) in which the current instance will start at
		 */
        startup: function(scope){
			try {
				if (this.columnDescriptors && this.columnDescriptors.length > 0) {

	                /*If we specified action descriptors, then, we create the actions header */
	                if (this.actionDescriptors && this.actionDescriptors.length > 0) {
	                    this.columnDescriptors.push({
	                        label: messages.actions,
	                        width: this.getActionsColumnWidth()
	                    });
	                }

	                this.createTableContents();
					this.scope= scope;
					this.updateContent();


	                /*Return Section Table */
	                return this;

	            }
	            else {
	                console.error("Can't create table for 0 columns");
	                return null;
	            }
			} catch (e) {
				console.error("Error:"+e);
			}
        },

        updateContent: function(){
        	var addSectionTableRow = this.addSectionTableRow;
        	for(var index=0; index<this.headers.length; index++){
				var current= this.headers[index];
				var header= current.header;
				var headerArgs= current.args;
				var classConstructor = dojo.getObject(header);
				if (!classConstructor) {
					jazz.core.loader.load_sync(header);
					classConstructor = dojo.getObject(header);
				}
				var headerConst = new classConstructor();
				// Set up UA
				if (headerArgs.uaKey) {
					headerConst.setUAKey(headerArgs.uaKey);
				}
				var _header = headerConst.createHeader(this.table, this.scope, headerArgs);
				this.sectionHeaders[header] = _header;
        	}
			this.appendTableAndHeaders(this.sectionHeaders, this.scope, this.table);

			this.styleTableAndHeaders(this.sectionHeaders, this.scope, this.table);
            this.fillFunction = dojo.hitch(this, function(tableHeader, table, result){
                  for (var i = 0; i < result.length; i++) {
                       this.addSectionTableRow(this, table, result[i]);
                  }
            });
            this.fillTable(this, this.table, this.fillFunction);
        },

        //TODO: add doc
        refreshTableData: function(){
        	var tbody = this.table.tbody;
        	while(tbody.hasChildNodes()) {
			   tbody.removeChild(tbody.firstChild);
			}
			for(var index=0; index<this.headers.length; index++){
				var header= this.sectionHeaders[this.headers[index].header];
				if(header&& header.destroyRecursive){
					header.destroyRecursive();
				}
			}
			this.sectionHeaders= [];
			this.createTableContents();
			this.updateContent();
        },

        //TODO: Should provide partial refresh instead
        refreshTableWithoutHeaders: function(){
        	var tbody = this.table.tbody;
        	while(tbody.hasChildNodes()) {
			   tbody.removeChild(tbody.firstChild);
			}
			this.dataSet= [];
			this.fillTable(this, this.table, this.fillFunction);
        },

		/*This is the actions continer in which contains the action image anchors nodes. It could be extended
		 * in case we would like to add some styling logic or so */
        createActionsContainer: function(tableData){
        	var div = dojo.doc.createElement("div");
			div.className = "actions-container";
			return div;
        },

        /*It could be extended if someone would like to disable actions creation for specific rows/data,
         * according to their situation. */
        getActionsEnablement: function(section,table,tr,data){
        	return true;
        },

        /*TODO:Add doc */
        appendTableAndHeaders: function(sectionHeaders, scope, table){
        	var node= scope.containerNode?scope.containerNode: scope;
        	for(var index=0; index<this.headers.length; index++){
				var current= this.headers[index];
				var header= sectionHeaders[current.header];
				node.appendChild(header.domNode?header.domNode:header);
			}
        	node.appendChild(table.domNode?table.domNode:table);
        },

        styleTableAndHeaders: function(sectionHeaders, scope, table){
			/*To be implemented by sub classes */
        },

		/*This method is used to help us to obtain the required data given a row. Therefore, this could
		 * help us when doing any operations like add/delete
		 */
		getDataForRow: function(row){
			for(var index=0; index< this.dataSet.length; index++){
				var entry= this.dataSet[index];
				if(entry.row== row){
					return entry;
				}
			}
			return null;
		},

		getDataSet: function(){
			return this.dataSet;
		},


        /*This method could be in case we want to add extra headers before the table
         * @see rmps.ui.sections.CollapsibleSectionHeader
         * It could be called after defininng the column descriptors. Example:
         * this.addHeader("rmps.ui.sections.DescriptionHeader", {tableDescription: "This is my description");
         */
        addHeader: function(header, args){
        	this.headers.push({header: header, args: args});
        },

		/*Destroy method. Must be called by the scope have used an instace of this class in order to
		 * avoid any memory leaks
		 */
		destroy: function(){
			var entry;
			if (this.columnDescriptors) {
				while (this.columnDescriptors.length > 0) {
					entry = this.columnDescriptors.pop();
					entry = null;
				}
			}
			delete this.columnDescriptors;
			this.columnDescriptors= null;

			if (this.dataSet) {
				while (this.dataSet.length > 0) {
					entry = this.dataSet.pop();
					entry = null;
				}
			}
			delete this.dataSet;
			this.dataSet= null;

			delete this.table;
			this.table= null;

			delete this.scope;
			this.scope= null;

			if (this.actionDescriptors) {
				while (this.actionDescriptors.length > 0) {
					entry = this.actionDescriptors.pop();
					entry = null;
				}
			}
			this.actionDescriptors= null;

			if (this.headers) {
				while (this.headers.length > 0) {
					entry = this.headers.pop();
					entry = null;
				}
			}
			this.headers= null;

			if (this.sectionHeaders) {
				while (this.sectionHeaders.length > 0) {
					entry = this.sectionHeaders.pop();
					entry = null;
				}
			}
			this.sectionHeaders= null;
			this.inherited(arguments);
		}

    });

})();
