/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("rmps.ui.sections.TableData");

dojo.require("jazz.ui.StyledTable");
dojo.require("rmps.ui.TextFilterInput");
dojo.require("rmps.ui.PagingControl");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("rmps.ui.util.ToolbarFocusUtil");
var ToolbarFocusUtil = rmps.ui.util.ToolbarFocusUtil;

dojo.requireLocalization("rmps.ui.sections", "TableDataMessages");

var messages= dojo.i18n.getLocalization("rmps.ui.sections", "TableDataMessages");

dojo.declare("rmps.ui.sections.TableData", [dijit._Widget, dijit._Templated], {

	templateString: '<div class= "rmps-web-sections-table-data" dojoAttachPoint="containerNode">' + '</div>',

	"class": "rmps-web-sections-table-data",

    columnDescriptors: null,
    actionDescriptors: null,
	dataSet: null,
	table: null,
	scope: null,
	headers: null,
	dataFeed: null,
	lastResponse: null,
	sectionHeaders: null,
	enablePaging: null,
	enablePagingFooter:  null,
	pagingControl: null,
	pagingFooterControl: null,
	page: null,
	pageSize: null,
	limit: null,
	size: null,
	pagingSelections: null,
	filterField: null,
	sortList: null,
	sortIndex: null,
	SHOW_ALL: "showAll",

    /*We must specify the scope in the params */
    constructor: function(params){
        this.columnDescriptors = [];
        this.actionDescriptors = [];
		this.dataSet= [];
		this.headers= [];
		this.sectionHeaders= [];
		this.sortList= [];
		this.sortIndex= 0;
		this.page= 0;
    },

    /* If we would like to show the headers, then, we create an instance of <code>StyledTable</code>, otherwise,
     * we create an HTML table
     * This method could be overridden for further conditions. For example:
     *      var _table = this.inherited(arguments);
     *     _table.cellPadding = 0;
     */
	createTable: function(args){
		var columns= args.columns, index;

		/*If we specified action descriptors, then, we create the actions header */
        if (this.actionDescriptors && this.actionDescriptors.length > 0) {
            var actionsWidth= this.getActionsColumnWidth();
            var actionsWidthVal= parseInt(actionsWidth.substring(0, actionsWidth.indexOf("%")));
            var delta= actionsWidthVal/columns.length;
            var total= 0;
            for(index=0; index<columns.length; index++){
				var current= columns[index];
				var currentWidth= current.width;
				currentWidth= parseInt(currentWidth.substring(0, currentWidth.indexOf("%")));
				currentWidth= Math.floor(currentWidth-delta);
				total+= currentWidth;
				current.width= currentWidth+ "%";
			}
			
			var diff= 100-total- actionsWidthVal;
			if(diff>0){
				actionsWidthVal+= diff; 
			}
            columns.push({
                label: messages.actions,
                fieldName: "Actions",
                width: actionsWidthVal + "%"
            });
        }
		
        if (this.showHeaders()) {
            return new jazz.ui.StyledTable({
                topBorder: this.getIsBoxed(),
                columns: columns,
                handleSort: dojo.hitch(this, this._handleSort),
				showHover: this.showHover()
            });
        }

        var _table = document.createElement("table");

		for(index=0; index< columns.length; index++){
			var width= columns[index].width;
			if(!width){
				width= this.getActionsColumnWidth();
			}
			dojo.create("col", {width: width}, _table);
		}

        return _table;
	},
	
	showHover: function(){
		return true;
	},

	getSortItemIndex: function(fieldName){
		for(var index=0; index<this.sortList.length; index++){
			var current= this.sortList[index];
			if(current.fieldName== fieldName){
				return index;
			}
		}
	},

	/*If someone wanted to change the order status for some field programatically */
	changeOrderStatus: function(fieldName, sortByAscending){
		this.sortIndex= this.getSortItemIndex(fieldName);
		this.table.sortBy= this.sortIndex;
		this.table.sortByAscending= sortByAscending;
		var sortNode= this.table._sortNodes[this.sortIndex];
		this.table._sortNode= sortNode;
		dojo.removeClass(sortNode, !sortByAscending ? "sortAscending" : "sortDescending");
		dojo.addClass(sortNode, sortByAscending ? "sortAscending" : "sortDescending");

		var sortDetails= this.sortList[this.sortIndex];

		if(sortByAscending){
			sortDetails.orderStatus= "ascending";
			sortNode.title= dojo.string.substitute(messages.orderBy_descending, [sortDetails.title]);
		}else{
			sortDetails.orderStatus= "descending";
			sortNode.title= dojo.string.substitute(messages.orderBy_ascending, [sortDetails.title]);
		}
	},

	_handleSort: function(){
		var sortDetails= this.sortList[this.sortIndex];
		if(this.sortIndex!= this.table.sortBy){
			sortDetails= this.sortList[this.sortIndex];
			this.table._sortNodes[this.sortIndex].title= dojo.string.substitute(messages.orderBy_ascending, [sortDetails.title]);
			sortDetails.orderStatus= "ascending";

			this.sortIndex= this.table.sortBy;

			sortDetails= this.sortList[this.sortIndex];
			this.table._sortNodes[this.sortIndex].title= dojo.string.substitute(messages.orderBy_descending, [sortDetails.title]);
		}else{
			sortDetails= this.sortList[this.sortIndex];
			this.doSort(sortDetails);
			if(sortDetails.orderStatus== "ascending"){
				this.table._sortNodes[this.sortIndex].title= dojo.string.substitute(messages.orderBy_ascending, [sortDetails.title]);
				sortDetails.orderStatus= "descending";
			}else{
				this.table._sortNodes[this.sortIndex].title= dojo.string.substitute(messages.orderBy_descending, [sortDetails.title]);
				sortDetails.orderStatus= "ascending";
			}
		}
		this.doSort(sortDetails);

	},

	doSort: function(sortDetails){
		var comparator= this.getSortComparator(sortDetails);
		if(comparator){
			this.lastResponse.sort(comparator);

			/*We have to return back to the first page */
			this.page=0;
			this.refreshTableDetails();
		}
	},

	/*Make sure to return false for the method #insertBefore as if it is true, it will corrup the sort */
	handleSort: function(sortDetails){
		/*To be implemnted by sub classes to handle sort */
		return false;
	},

	/* If we would like to show the table headers, then this should return <code>true</code>; otherwise,
	 * it should return <code>false</code>
	 */
	showHeaders: function(){
		return true;
	},

	/*When we add a new row to the table, we could specify where it will be placed in the table (first,
	 * or end). If the return value if <code>true</code>, then, the new row will be added at the beginning;
	 * otherwise, it will be inserted as the end
	 */
	insertBefore: function(){
		return true;
	},

	/*The isBoxed attribute for the StyledTable */
    getIsBoxed: function(){
        /*To be implemented. Otherwise, it will return false */
        return false;
    },

    /*It could be overridden to increase/reduce the action column width */
    getActionsColumnWidth: function(){
    	if(this.enableDynamicActionsWidth()){
    		return this.getDynamicColumnWidth();
    	}
    	return "5%";
    },
    
    /*It could be overridden to increase/reduce the action column width */
    getDynamicColumnWidth: function(){
    	var number= this.actionDescriptors.length;
    	if(number=== 0){
    		return this.inherited(arguments);
    	}
    	return (number*3) + (Math.round(number/2)) + "%";
    },
    
    /*If it is true, this means that the actions width will be increased to make sure that it will be in one line */
    enableDynamicActionsWidth: function(){
    	return false; /*by default */
    },


    /*This method must be implemented by other classes. We could specify the call actions for the
     * server in order to get the expected results. After that, we can use the onCall Method in order
     * to pass the gained results so, the table rows and cells will be created
     * Example:
     *	var deferred= this._Someclinet.getsomeValues();
     *	deferred.addCallback(this, function(result) {
     *			result= {
     * 						field1: "fieldValue",
     * 						field2: "field2Value"
     * 					}
     *			onCall(section, table,result);	//Note, you must pass in those arguments as it is
     *	});
     *
     *
     * */
	fillTable: function(section, table, onCall){},	/*Abstract method */


    /*The client could specify a new column to be added to the section table. They should specify the
     * column display name (columnHeader), and its expected field name that will be retrieved from the
     * method SectionTable#fillTable, and the method that create the cells for this column (creatorCallback)
     * The way of having creatorCallback for each column will help the client to have thier own conditions
     * and ways to create the column cells, and to add their rich UI if needed. Note, the user must specify
     * at least one column descriptor. In addition, you could specify the width as well, but make sure that
     * using anapplied CSS style will cause this width value to have no effect in your column descriptor; you
     * could specify the width in percentage.
     * Example:
     * 	this.addColumnDescription({columnHeader:"Name", fieldName:"nameField",
     * 												creatorCallback:this.createNameField, width: 40%});
     * this.addColumnDescription({columnHeader:"Date Of Birth", fieldName:"dobField",
     * 												creatorCallback:this.createDOBField, width: 40%});
     * this.addColumnDescription({columnHeader:"Course", fieldName:"Name",
     * 												creatorCallback:this.createCourseField, width: 20%});

     * And an example for creator @see:createDefaultCell
     *
     * NOTE: creatorCallback function must return a TD for the table row that is having this td created for,
     * otherwise, an error wil be logged, and unexpected behaviors could happen.
     *
     */
    addColumnDescriptor: function(request){
        /*Understand Request */
        var _columnHeader = request.columnHeader;
        var _fieldName = request.fieldName;
        var _width = request.width;
        var _sortable = request.sortable;
        var _creatorCallback = request.creatorCallback;
        var _editingCallback= request.editingCallback;

        /*If we do not want to specify special function for the column cell creation,then
         * we use the default cell creator */
        if (!_creatorCallback) {
            _creatorCallback = this.createDefaultCell;
        }

        if(_sortable){
        	/*We maintain the order of the orderable list, so, we could track which filed to order later. The fields order is extremly important */
        	this.sortList.push({
        		fieldName: _fieldName,
        		title: _columnHeader,
        		orderStatus: "ascending"
        	});
        }

        this.columnDescriptors.push({
            label: _columnHeader,
            fieldName: _fieldName,
            width: _width,
            sortable: _sortable,
            editingCallback: _editingCallback,
            creatorCallback: _creatorCallback
        });
    },

    _addColumnDescriptors: function(){
    	/*Clear old data. It is required for refreshment actions */
    	this.clearColumnDescriptors();
    	this.columnDescriptors= [];

    	/*now start adding/re-adding column descriptors */
    	this.addColumnDescriptors();
    },

    _addActionsDescriptors: function(){
		/*Clear old data. It is required for refreshment actions */
    	this.clearActionDescriptors();
    	this.actionDescriptors= [];

    	/*now start adding/re-adding column descriptors */
    	this.addActionDescriptors();
    },

    /*
     * Sub classes should implemnted this method to add the columns descriptors
     *
     * NOTE: It is deprecated to add column descriptors out of this method
     */
    addColumnDescriptors: function(){
    	/*Example:
    	 *
    	 * this.addColumnDescriptor({columnHeader:"Cell1", sortable: true,  	  fieldName: "status", width: "50%", creatorCallback: this.cell1Creator});
         * this.addColumnDescriptor({columnHeader:"Cell2",sortable: true, 	      fieldName: "role", width: "50%", creatorCallback: this.cell2Creator });
    	 *
    	 *
    	 */
    },

    /*
     * Sub classes should implemnted this method to add the action descriptors
     *
     * NOTE: It is deprecated to add action descriptors out of this method
     */
    addActionDescriptors: function(){
    	/*Example:
    	 *
    	 * this.addActionDescriptor(this.createAction1);
	     * this.addActionDescriptor(this.createAction2);
    	 *
    	 */
    },

    /*The default method to create column cells. If the user did not specify a function to create a column
     * to a column descriptor, then, we call this one. If the user would like to implement their own function
     * then, they have to make sure to follow the same signature for SectionTable#createDefaultCell. The return
     * value must be node value has been created.
     * The user does not have to worry about the parameters value as they will be received from previous methods
     */
    createDefaultCell: function(section, table, tableRow, fieldName, value, data){
        var tableData = dojo.create("td", null, tableRow);

        var sectionTable = this.table;
        var scope = this.scope;

        var newCell;
        if (data) {
            newCell = dojo.create("preview", null, tableData);
            newCell.innerHTML = value;
        }
        else {
            newCell = dojo.create("input", null, tableData);
            newCell.className = "cell";
            newCell.type = "text";
            newCell.value = value ? value : "";
            newCell.data = data;
            newCell.style.width = "100%";

            /*This field will be used to determine if the input value has been changed */
            newCell.originalValue = value;

            /*This value */
            newCell.fieldName = fieldName;
        }
        return tableData;
    },

    /*
     *
     *If the user would like to add an action to the Actions column. If the user did not add a new action
     * descriptor, the actions column will not be added. The function signature must be as the follows:
     * 
     * 		createAnAction: function(dataSetEntry,tableData,btn,resourceLink,section, table, e){
     * 		}
     * 
     * The action enablement function is used to determine if the curret action will be created for the current row. It is different from the main
     * enablement, as we could add an action descriptor but for specific rows, we could have pre-conditions to show these actions. The enablement
     * function is optional, so if it is not defined, then, we assume creating the action all the time. An example:
     * 
     * myEnablement: function(tableData, dataSetEntry,section, table, simpleToolbar, id){
     * 			return dataSetEntry.data.name== "Example";
     * } 
     * 
     * 
     * 
     * Example:
     * 		this.addActionDescriptor({
	 *				imgSrc: imgSrc,				//Required
	 *				id: "myAction",				//Required
	 *				action: createAnAction,		//Required
	 *				actionEnablement: isActionEnabled,		//Optional
	 *				label: "My Action"			//Optional
	 *			});
     *
     * TODO: Must have tooltip
     *
     * */
    addActionDescriptor: function(_creatorCallback){
        this.actionDescriptors.push(_creatorCallback);
    },

    /*This method could be overriden if someone would like to add special conditions for the row creations
     * For Example, if we would like to add a class for the row:
     * 		var tr= this.inherited(arguments);
     * 		dojo.addClass(tr, "someCSSClass");
     * 		return tr;
     *
     */
    addSectionTableRow: function(section, table, data, dataFeed, index){
		var tr= null;
		var tbody= table.tbody;
		if(!tbody){
			tbody= table;
		}
		if(this.insertBefore()){
			tr = dojo.doc.createElement("tr");
			tbody.insertBefore(tr, tbody.firstChild);
		}else{
			if(parseInt(index)>-1===false){
				index= -1;
			}
			tr= tbody.insertRow(index);
		}

        var cellStore= {};
        var fieldNames=[];
        var values=[];
        var columns = table.columns;
        var columnDetails= this.dataFeed? this.dataFeed.getDescriptionDetailsForData(data, this):columns;
        if(!columnDetails /*Safe check. It cannot be null at this point */){
        	console.error("Description Detaills are null for this data row");
        	return;
        }

        var dataSetEntry= {
            row: tr,
            data: data
        };
        this.dataSet.push(dataSetEntry);	/*Add entry from now as it will enable us to esitmate the row value for other calls */
        for (var i = 0; i < columnDetails.length; i++) {
            var descriptionDetail= columnDetails[i];
            var fieldName = descriptionDetail.fieldName;
            if(fieldName== "Actions"||descriptionDetail.label== messages.actions){ /*FIXME: Shoul;d find better way */
				continue;
			}
            /*The column field could be null for the actions column */

            var value= descriptionDetail.value?descriptionDetail.value:data[fieldName];
			var td = dojo.create("td", null, tr);
			var callback= descriptionDetail.nodeCreatorCallback?descriptionDetail.nodeCreatorCallback: descriptionDetail.creatorCallback;
            if(callback){
				var cell = dojo.hitch(this, callback)(this,this.table, fieldName, value, data, tr, td);
                if (cell /*There is an enablement if we do not want to create the cell at all */) {
                    cellStore[fieldName]= cell;
                    fieldNames.push(fieldName);

                    /*This field will be used to determine if the value for a cell has been changed */
                    values[fieldName]= value;
                    descriptionDetail.node= cell;
                }
            }
        }

        var tableData = dojo.create("td", {
            align: "centre",
            width: this.getActionsColumnWidth()
        }, tr);
		var actionNodes=[];
        if (this.actionDescriptors.length>0&&	/*If there no action defnined, then we create an empty cell */
        				this.getActionsEnablement(section,table,tr,data)&&this.actionDescriptors) {
			
			var simpleToolbar = new jazz.ui.SimpleToolbar();
			
			// Remember the toolbar widget in the row (tr) for the 
			// clearDataset() and clearTable() methods below
			tr._actionToolbar = simpleToolbar;
			
			tableData.appendChild(simpleToolbar.domNode);
						
			var createAction= dojo.hitch(this, function(tableData){
                while(simpleToolbar.items.length>0){
                	simpleToolbar.removeItem(simpleToolbar.items[0]);
                }
            	
            	var buttonCallback = function(id,tableData,dataSetEntry, e){
            		dojo.stopEvent(e);
            		var actionNode= actionNodes[id];
            		actionNode.actionDescriptor.action(dataSetEntry,tableData,actionNode.button,actionNode.args, this, this.table, e);
            	};
                
                for (var index = 0; index < this.actionDescriptors.length; index++) {
                    var actionDescriptor = this.actionDescriptors[index];
                    
                    /*Safe check for null */
                    if (actionDescriptor) {
                    	var action= actionDescriptor.action;
                    	
                    	if(!dojo.isFunction(action)){
                    		console.error("Must provide an action function for the action");
                    		continue;
                    	}
                    	
                    	var id= actionDescriptor.id;
                    	if(!id){
                    		console.error("Must provide an id for the created actions");
                    		id= "action:"+index;
                    	}
                    	
                    	var actionEnablement= actionDescriptor.actionEnablement;
                    	
                    	/*Action Enablement is optional. If we did define it, then, we assume that the action will be created all the time */
                    	if(dojo.isFunction(actionEnablement)&&!actionEnablement(tableData, dataSetEntry, this, this.table,simpleToolbar, id)){
                    		continue;
                    	}
                    	var args= {};
                    	var btn= simpleToolbar.addButton(this, dojo.hitch(this, buttonCallback,id, tableData, dataSetEntry), {
							className: actionDescriptor.className?actionDescriptor.className:"",
							label: actionDescriptor.label?actionDescriptor.label:"",
							imgSrc: actionDescriptor.imgSrc
						});
						var preAction= actionDescriptor.preAction;
						
						if(dojo.isFunction(preAction)){
							args= preAction(dataSetEntry,tableData,btn, this, this.table);
						}
						
						dojo.addClass(btn.domNode,"actions");
						
                        actionNodes[id]={
                        	button: btn,
                        	args: args,
                        	actionDescriptor: actionDescriptor
                        };
                    }
                }
			
				// Recursively walk the node hierarchy of this row and add
				// focus listeners to every focusable node (e.g., tabIndex >= 0)
				// where the listener will show/hide the toolbar actions
				ToolbarFocusUtil.addToolbarFocusListeners( { 
					actionToolbar: tr._actionToolbar, 
					containerNode: tr, 
					actionClassName: ".actions"
				});
				

			}, tableData);
            var actionDetail= {};
            var actionsColumn= columnDetails[columnDetails.length-1];
            if(actionsColumn.fieldName== "Actions"){	/*Safe check */
            	actionsColumn.node= tableData;
	            actionsColumn.callBack= createAction;
	            actionsColumn.callBack();
            }

        }

        dataSetEntry.actionNodes= actionNodes;
        dataSetEntry.cells= cellStore;
        dataSetEntry.fieldNames= fieldNames;
        dataSetEntry.values= values;

        return tr;
    },
    
    getAssociatedValues: function(data){
        var fieldNames=[];
        var values=[];
        var table= this.table;
        var columns = table.columns;
        for (var index = 0; index < columns.length; index++) {
            var column = columns[index];
            /*The column field could be null for the actions column */
            var fieldName = column.fieldName;
            if (fieldName) {
            	var value= dojo.getObject(fieldName, false, data);
               	values[fieldName]= value;
            }
        }
        return values;
    },

	createTableContents: function(){
		this._addColumnDescriptors();
		this._addActionsDescriptors();
		if(this.table&& this.table.destroyRecursive){
			this.table.destroyRecursive();
		}
		
		var table = this.createTable({columns:this.columnDescriptors});

		var tbody= table.tbody;
		if(!tbody){
			tbody= table;
		}

		if(table instanceof jazz.ui.StyledTable){
			this._handleDefaultOrderTitles(table);
		}

		this.connect(tbody, "onmousedown", dojo.hitch(this, this._handleTbodyMouseDown));

		//TODO: Do we need to have this call in this class?
		if(table.domNode){table.domNode.style.width= "100%";}

		this.table= table;
	},

	_handleDefaultOrderTitles: function(table){
		var sortNodes= table._sortNodes;
		for(var index=0; index<sortNodes.length; index++){
			this._handleDefaultOrderTitle(sortNodes[index], this.sortList[index]);
		}
	},

	_handleDefaultOrderTitle: function(sortNode, fieldDetails){
		sortNode.title= dojo.string.substitute(messages.orderBy_ascending, [fieldDetails.title]);
	},

	_handleTbodyMouseDown: function(e) {
		var tbody= this.table.tbody;
		if(!tbody){
			tbody= this.table;
		}
		var mouseUpHandle, mouseOutHandle;
		mouseOutHandle = this.connect(tbody, "onmousemove", function() {
			this.disconnect(mouseOutHandle);
			this.disconnect(mouseUpHandle);
		});

		mouseUpHandle = this.connect(tbody, "onmouseup", function(e) {
			this.disconnect(mouseUpHandle);
			this.disconnect(mouseOutHandle);
			this._handleTbodyMouseClick(e);
		});
	},

	_handleTbodyMouseClick: function(e) {
		if (e.ctrlKey || e.shiftKey || e.altKey || e.button === 2) { // alt clicks open tabs or windows
			return;
		}
		this.onTableBodyClick(this.getDataForRow(e.target.parentNode));
	},

	onTableBodyClick: function(data){
		/*To be implemnted by other classes */
	},

	/*Must be called after initializing an instance of this class.
	 *
	 * Must pass the scope (a widget instance) in which the current instance will start at
	 */
    startup: function(scope){
		try {
            this.createTableContents();
			this.scope= scope;
			this.updateContent();


            /*Return Section Table */
            return this;
		} catch (e) {
			console.error("Error:"+e);
		}
    },

    updateContent: function(){
    	var addSectionTableRow = this.addSectionTableRow;
    	for(var index=0; index<this.headers.length; index++){
			var current= this.headers[index];
			var header= current.header;
			var headerArgs= current.args;
			var classConstructor = dojo.getObject(header);
			if (!classConstructor) {
				jazz.core.loader.load_sync(header);
				classConstructor = dojo.getObject(header);
			}
			var headerConst = new classConstructor();
			
			// Set up UA
			if (headerArgs.uaKey) {
				headerConst.setUAKey(headerArgs.uaKey);
			}
			var _header = headerConst.createHeader(this.table, this.scope, headerArgs);
			var headerId= current.headerId;
			if(!headerId){
				headerId= header;
			}
			this.sectionHeaders[headerId] = _header;
    	}
		this.appendTableAndHeaders(this.sectionHeaders, this.scope, this.table);
		this.styleTableAndHeaders(this.sectionHeaders, this.scope, this.table);

        this.fillFunction = dojo.hitch(this, function(tableHeader, table, dataFeed){
        	if(dataFeed){
        		if(dataFeed.initEntriesForWidget){
					this.dataFeed= dataFeed;
					this.dataFeed.initEntriesForWidget(this);
        		}else{
        			this.lastResponse= dataFeed;
        			this.createPagingControls(dataFeed.length);
        			this.updateTableContent();
              	}
        	}else{
        		console.error("You have to provide data feed or data result");
        	}
        });
        this.fillTable(this, this.table, this.fillFunction);
    },

    updateTableContent: function(){
    	var dataFeed= this.lastResponse, start, last, isLast;
    	this.size= dataFeed.length;
    	if(this.pagingControl){
    		var selction= parseInt(this.pagingSelection.getSelection());
    		if(selction){
    			this.page= parseInt(this.pageSize*this.page/selction);
    			this.pageSize= selction;
    		}else{
				this.page= 0;
    			this.pageSize= dataFeed.length;
    		}

    		if(this.page*this.pageSize>dataFeed.length){
    			/*Detect buggy external urls with out of bounds indecies */
    			this.page= 0;
    			this.pageSize= this.pagingSelections[0]?this.pagingSelections[0]:dataFeed.length;
    		}

			this.pagingControl.update(this.page, this.pageSize, null, this.size);

			start= this.pageSize*this.page;
    		last= this.pageSize* (this.page+1);
    		isLast= function(i){
    			return i < dataFeed.length&& i<last;
    		};
    	}else{
    		start= 0;
    		last= dataFeed.length;
    		isLast= function(i){
    			return i < dataFeed.length;
    		};
    	}

		for (var i = start; isLast(i); i++) {
       		this.addSectionTableRow(this, this.table, dataFeed[i], dataFeed);
		}
    },

    createPagingControls: function(dataLength){
		if(this.enablePaging){
			if(this.pagingSelections){
				this.pagingSelection= new rmps.ui.sections.TableData._ItemsDiaply({
					messages: messages,
					dataLength: dataLength,
					pagingSelections: this.pagingSelections,
					pageSize: this.pageSize,
					onPageSizeChange: dojo.hitch(this, this.onPageSizeChange)
				});
				this.pagingSelection.refreshTableDetails= dojo.hitch(this, this.refreshTableDetails);
				this.pagingSelection.page= dojo.hitch(this, this.page);
			}
			this.pagingControl= new rmps.ui.sections.TableData._PagingControl({
				tableData: this,
				onPageChange: dojo.hitch(this, this.onPageChange)
			});

			if(this.enablePagingFooter){
				this.pagingFooterControl= new rmps.ui.sections.TableData._PagingControl({tableData: this});
				this.pagingControl.linked= this.pagingFooterControl;
			}
		}
		this.appendPagingControls(dataLength);
    },

    appendPagingControls: function(dataLength){
	    if(this.pagingControl){
			if(this.pagingSelection){
				this.pagingFilterContents= new rmps.ui.sections.TableData._PagingFilterContents();
				this.textFilterInput= new rmps.ui.TextFilterInput({
					filterChanged: dojo.hitch(this, this.filterChanged)
				});
				if(dataLength!== undefined){
					this.textFilterInput.setEnabled(parseInt(dataLength)>0);
				}
				this.pagingFilterContents.addFilterSelectionContents(this.pagingSelection.domNode);
				this.pagingFilterContents.addPagingContents(this.pagingControl.domNode);
				this.pagingFilterContents.addFilterSearchContents(this.textFilterInput.domNode);
				this.containerNode.insertBefore(this.pagingFilterContents.domNode, this.containerNode.firstChild);
			}
			if(this.pagingFooterControl&& dataLength>0){
				this.containerNode.appendChild(this.pagingFooterControl.domNode);
			}
		}
    },

    onPageSizeChange: function(e){
    	/*To be implemnted by other clients */
    },

    onPageChange: function(page){
    	/*To be implemnted by other clients */
    },

    addDataEntryNotifier: function(data, descriptions){
    	var columns = this.table.columns;
   	 	for (var index = 0; index < columns.length; index++) {
            var column = columns[index];
            /*The column field could be null for the actions column */
            var fieldName = column.fieldName;
            if (fieldName) {
            	var value= dojo.getObject(fieldName, false, data);
            	descriptions.push({
        			value: value,
        			fieldName: fieldName,
        			nodeEditingCallback: column.editingCallback,
        			nodeCreatorCallback: dojo.hitch(this,column.creatorCallback)
        		});
            }

        }
    	return this.addSectionTableRow(this, this.table, data, this.dataFeed);
    },

    editDataEntryNotifier: function(dataSetDetails,uniqueIdentifier, identifier, fieldName, fieldValue, dataObject){
    	var description= dataSetDetails.widgetDetails;
		var entry= this.dataFeed.getEntry(identifier, description, fieldName);
		if(entry){	/*Required check. If the table did not create a cell uses this field, then entry will be null */
			if(fieldName== "Actions"){
				entry.callBack();
				return;
			}
			entry.value= fieldValue;
			var dataSet= this.getDataSetFromField(uniqueIdentifier,  identifier);

			/*TODO: Support undo/redo from these parts */
    		dataSet.data[fieldName]= fieldValue;
    		
    		var values= dataSet.values;
			if(values){
				values[fieldName]= fieldValue;
			}
			
			var nodeEditingCallback= entry.nodeEditingCallback;
			if(nodeEditingCallback){
				/*Call for the node editing callback if it is defined and return */
				nodeEditingCallback(this,this.table, fieldName, fieldValue,dataSet.data, dataSet.row, entry.node);
				return;
			}
			
    		var node= entry.node, first;
			if(node){
				while(node.hasChildNodes()) {
					first= node.firstChild;
			  	 	node.removeChild(first);
			  	 	dojo.destroy(first);
				}	
			}
			
			var cell= dataSet.cells[fieldName];
			if(cell){
				while(cell.hasChildNodes()) {
					first= cell.firstChild;
			  	 	cell.removeChild(first);
			  	 	dojo.destroy(first);
				}
				dataSet.cells[fieldName]= entry.nodeCreatorCallback(this,this.table, fieldName, fieldValue,dataSet.data, dataSet.row, cell);
			}
		}
    },

    getDataSetFromField: function(uniqueIdentifier,  identifier){
    	for(var index=0; index< this.dataSet.length; index++){
			var entry= this.dataSet[index];
			var data= entry.data;
			if(data){	/*Safe check. It's difficult to have data null at this point */
				if(data[uniqueIdentifier]==identifier){
					return entry;
				}
			}
		}
		return null;
    },

    removeDataEntryNotifier: function(dataSetDetails,uniqueIdentifier,  identifier){
    	var dataSet= this.getDataSetFromField(uniqueIdentifier,  identifier);
    	var table= this.table.tbody;
    	if(!table){
    		table= this.table;
    	}
    	var rows= table.rows;
    	var index = dojo.indexOf(rows, dataSet.row);
    	if(index>-1){
    		table.deleteRow(index);
    		/*TODO:Here is the start to support undo/redo feature  */
    		this.dataSet.splice(dojo.indexOf(this.dataSet, dataSet), 1);
    	}
    },

    clearPagingInformation: function(){
    	if(this.pagingControl){
			this.pagingControl.destroyRecursive();
			delete this.pagingControl;
		}
		if(this.pagingFooterControl){
			this.pagingFooterControl.destroyRecursive();
			delete this.pagingFooterControl;
		}
		if(this.pagingFilterContents){
			this.pagingFilterContents.destroyRecursive();
			delete this.pagingFilterContents;
		}
    },

    /*Provide a way to refresh table completely including the table, headers, and paging */
    refreshTableData: function(){
    	this.clearTable();
		this.clearPagingInformation();

		for(var index=0; index<this.headers.length; index++){
			var header= this.sectionHeaders[this.headers[index].header];
			if(header&& header.destroyRecursive){
				header.destroyRecursive();
			}
		}
		this.clearDataset();
		this.sectionHeaders= [];
		this.dataSet= [];
		this.createTableContents();
		this.updateContent();
    },

    //TODO: Should provide partial refresh instead
    refreshTableWithoutHeaders: function(){
		this.clearData();
		this.fillTable(this, this.table, this.fillFunction);
    },

	/*Called to refresh the table without making a call to repopulate the data information the way tha tthe caller does. Required for paging */
    refreshTableDetails: function(){
    	this.clearData();
		this.updateTableContent();
    },

    clearData: function(){
		this.clearTable();
		this.clearDataset();
		this.dataSet= [];
    },

    clearTable: function(){
		if (this.table) {
    		// Destroy the action toolbars of the table explicitly
    		// so that focus listeners are removed. 
    		// The table.clearBody() method does not destroy the widgets.
    		if (this.table.tbody) {
    			var tbody = this.table.tbody;
    	    	while(tbody.hasChildNodes()) {
    	    		if (tbody.firstChild._actionToolbar) {
    	    			tbody.firstChild._actionToolbar.destroy();
    	    		}
    				tbody.removeChild(tbody.firstChild);
    			}
			}
		}
    },

	/*This is the actions continer in which contains the action image anchors nodes. It could be extended
	 * in case we would like to add some styling logic or so */
    createActionsContainer: function(tableData){
    	var div = dojo.doc.createElement("div");
        div.style.verticalAlign= "middle";
		div.align = "center";
		div.width = "100%";
		return div;
    },

    /*It could be extended if someone would like to disable actions creation for specific rows/data,
     * according to their situation. */
    getActionsEnablement: function(section,table,tr,data){
    	return true;
    },

    /*TODO:Add doc */
    appendTableAndHeaders: function(sectionHeaders, scope, table){
    	var node= scope.containerNode?scope.containerNode: scope;
    	for(var index=0; index<this.headers.length; index++){
			var current= this.headers[index];
			var header= sectionHeaders[current.header];
			this.containerNode.appendChild(header.domNode?header.domNode:header);
		}
    	this.containerNode.appendChild(table.domNode?table.domNode:table);
		node.appendChild(this.containerNode);
    },

    filterChanged: function(text){
    	var updateRowDisplay= dojo.trim(text)!== ""?function(row, fieldValue){
    		if(fieldValue&& fieldValue.toLowerCase().indexOf(text.toLowerCase())<0){
    			row.style.display= "none";
    		}else{
    			row.style.display= "";
    		}
    	}:function(row, fieldValue){row.style.display= "";} ;

    	for(var index=0; index<this.dataSet.length; index++){
    		var current= this.dataSet[index];
    		updateRowDisplay(current.row, current.values[this.filterField]);
    	}
    },

	/*This method is used to help us to obtain the required data given a row. Therefore, this could
	 * help us when doing any operations like add/delete
	 */
	getDataForRow: function(row){
		for(var index=0; index< this.dataSet.length; index++){
			var entry= this.dataSet[index];
			if(entry.row== row){
				return entry;
			}
		}
		return null;
	},

	styleTableAndHeaders: function(sectionHeaders, scope, table){
		/*To be implemented by sub classes */
    },

	getDataSetForData: function(data){
		for(var index=0; index< this.dataSet.length; index++){
			var entry= this.dataSet[index];
			if(entry.data== data){
				return entry;
			}
		}
		return null;
	},

	getDataSet: function(){
		return this.dataSet;
	},


    /*This method could be in case we want to add extra headers before the table
     * @see rmps.ui.sections.CollapsibleSectionHeader
     * It could be called after defininng the column descriptors. Example:
     * this.addHeader("rmps.ui.sections.DescriptionHeader", {tableDescription: "This is my description");
     */
    addHeader: function(header, args, headerId){
    	this.headers.push({header: header, args: args, headerId: headerId});
    },

    clearDataset: function(){
    	if (this.dataSet) {
			while (this.dataSet.length > 0) {
				var entry = this.dataSet.pop();
				if(entry){
					if(entry.row){
						// If the row has an action toolbar, desotry it first
						// becuse the destroying of the row is not recursive.
						if (entry.row._actionToolbar) {
							entry.row._actionToolbar.destroy();
						}
						dojo.destroy(entry.row);
						delete entry.row;
					}
					while (entry.cells.length > 0) {
						var cell = entry.cells.pop();
						if(cell){
							dojo.destroy(cell);
						}
					}
				}
				entry = null;
			}
		}
		delete this.dataSet;
		this.dataSet= null;
    },

    clearColumnDescriptors: function(){
    	if (this.columnDescriptors) {
			while (this.columnDescriptors.length > 0) {
				var entry = this.columnDescriptors.pop();
				entry = null;
			}
		}
		delete this.columnDescriptors;
		this.columnDescriptors= null;
    },

    clearActionDescriptors: function(){
    	if (this.actionDescriptors) {
			while (this.actionDescriptors.length > 0) {
				var entry = this.actionDescriptors.pop();
				entry = null;
			}
		}
		this.actionDescriptors= null;
    },

	/*Destroy method. Must be called by the scope have used an instace of this class in order to
	 * avoid any memory leaks
	 */
	destroy: function(){
		this.clearColumnDescriptors();
		this.clearDataset();

		this.clearPagingInformation();

		delete this.table;
		this.table= null;

		delete this.scope;
		this.scope= null;

		this.clearActionDescriptors();

		var entry;
		if (this.headers) {
			while (this.headers.length > 0) {
				entry = this.headers.pop();
				entry = null;
			}
		}
		this.headers= null;

		if(this.pagingFilterContents){
			this.pagingFilterContents.destroyRecursive();
		}

		if (this.sectionHeaders) {
			while (this.sectionHeaders.length > 0) {
				entry = this.sectionHeaders.pop();
				entry = null;
			}
		}
		this.sectionHeaders= null;
		this.inherited(arguments);
	}

});


dojo.declare("rmps.ui.sections.TableData._PagingControl", rmps.ui.PagingControl, {

	table: null,
	linked: null,
	tableData: null,

	update: function(page, pageSize, limit, size, item) {
		this.inherited(arguments);
		if (this.linked) {
			this.linked.update(page, pageSize, limit, size, item);
		}
	},

	onNextClick: function(e) {
		this.inherited(arguments);
		this.tableData.page++;
		this.tableData.refreshTableDetails();
		this.onPageChange(this.page);
	},

	onPreviousClick: function() {
		this.inherited(arguments);
		this.tableData.page--;
		this.tableData.refreshTableDetails();
		this.onPageChange(this.page);
	},

	onPageChange: function(page){
		/*to be implemnted externally */
	}

});

dojo.declare("rmps.ui.sections.TableData._ItemsDiaply", [dijit._Widget, dijit._Templated], {

	"class": "rmps-web-sections-table-data-header",

	templateString: '<div>' +
		'<div dojoAttachPoint="_itemsPerPage">' +
		'			<select id="${id}_pageSizeSelect" class="itemsPerPage-select" dojoAttachPoint="_pageSizeSelect" dojoAttachEvent="onchange:_onPageSizeChange"></select>' +
		'			<label class="itemSelectionLabel" for="${id}_pageSizeSelect">${messages.itemsPerPage}</label></div>'+
	'</div>',

	pagingSelections: null,
	page: null,
	dataLength: null,
	pageSize: null,

	_onPageSizeChange: function(e){
		this.refreshTableDetails();
		this.pageSize= this.getSelection();
		this.onPageSizeChange(this.pageSize);
	},

	onPageSizeChange: function(selection){
		/*To be implemnted Externally */
	},

	refreshTableDetails: function(){
		/*To be implemented by the caller */
	},

	postCreate: function(){
		this.inherited(arguments);
		if(this.pagingSelections){
			for(var index=0; index< this.pagingSelections.length; index++){
				var current= this.pagingSelections[index];
				if(current> this.dataLength){
					break;
				}
				this._createOption(this.pagingSelections[index], this.pageSize);
			}
		}
		this._createOption(messages.showAll, this.pageSize, "showAll");
		if(this._pageSizeSelect.options.length===1){
			this._pageSizeSelect.disabled= true;
		}
	},

	_createOption: function(text, defaultValue, value){
		var option= dojo.doc.createElement("option");
		option.value=value?value:text;
		if(parseInt(option.value)== defaultValue|| option.value== defaultValue){
			option.selected= true;
		}
		option.appendChild(dojo.doc.createTextNode(text));
		this._pageSizeSelect.appendChild(option);
	},

	getSelection: function(){
		return parseInt(this.getValue());
	},

	getValue: function(){
		return this._pageSizeSelect.options[this._pageSizeSelect.selectedIndex].value;
	}

});


dojo.declare("rmps.ui.sections.TableData._PagingFilterContents", [dijit._Widget, dijit._Templated], {

	"class": "rmps-web-sections-table-data-header",

	templateString: '<div>' +
					'<table width= "100%" nowrap="nowrap">' +
							'<tr>' +
								'<td width= "33%"><div dojoAttachPoint="_filterSelection"></div></td>' +
								'<td width= "33%"><div dojoAttachPoint="_pagingContents"></div></td>' +
								'<td align= "right" width= "33%"><div dojoAttachPoint="_filterSearch"></div></td>' +
							'</tr>' +
					'</table>'+
					'</div>',


	addFilterSelectionContents: function(node){
		this._filterSelection.appendChild(node);
	},

	addPagingContents: function(node){
		this._pagingContents.appendChild(node);
	},

	addFilterSearchContents: function(node){
		this._filterSearch.appendChild(node);
	}
});


})();
