/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("rmps.web.comment.sketch.serializer.Reader");

dojo.require("rmps.web.comment.sketch.serializer.SVGStyle");
dojo.require("rmps.web.comment.sketch.serializer.SVGUtil");

// Test data is made available by uncommenting
//dojo.require("rmps.web.comment.sketch.serializer.SVGTestData");

dojo.require("dojox.xml.parser");

dojo.require("dojox.gfx.path");
dojo.require("rmpw.diagrams.render.util.Workarounds");

dojo.declare("rmps.web.comment.sketch.serializer.Reader", null, { 

	transform: null,
	translatex: 0,
	translatey: 0,
	scribbles: null,
	diagram: null,
	
	svgX: 0,
	svgY: 0,

	constructor: function(){
		this.transform = "translate(0,0)";
	},

	x: function(){
		return ( transform.translatex + this.diagram.x );
	},

	y: function(){
		return( transform.translatey + this.diagram.y );
	},

	deriveStyle: function( fragment, styledata ){

		/* Style in SVG documents works hierarchically. RSA/Batik produces style at
		 * a group level then overrides aspects of it for each derived node in that 
		 * group. We're cascading style from the top level group to each of the
		 * subnodes, then checking if the nodes have their own local settings. */

		var localstyle = dojo.clone( styledata );

		var style = fragment.getAttribute( "style" );
		var stroke = fragment.getAttribute( "stroke" )
		var fill = fragment.getAttribute( "fill" );
		var strokeWidth = fragment.getAttribute( "stroke-width" );
		var fillOpacity = fragment.getAttribute( "fill-opacity" );
		var strokeOpacity = fragment.getAttribute( "stroke-opacity" );
		var opacity = fragment.getAttribute( "opacity" );
		var strokeDash = fragment.getAttribute( "stroke-dasharray" );

		if(style){
			localstyle.setStyle( style );
		}

		if(stroke){
			
			if( stroke == "none" ){
				strokeOpacity = 0;
				localstyle.strokecolor = styledata.strokecolor;
			}else{
				localstyle.parseColor( "STROKE", stroke );
			}
		}

		if(fill){
			
			if( fill == "none" ){
				localstyle.fillopacity = 0;
				localstyle.fillcolor = styledata.fillcolor;
			}else{
				localstyle.parseColor( "FILL", fill );
			}
		}

		if(strokeWidth){
			localstyle.strokewidth = parseInt(strokeWidth);
		}

		if(fillOpacity){
			localstyle.fillopacity = parseFloat(fillOpacity);
		}

		if(strokeOpacity !== null && strokeOpacity !== undefined){
			localstyle.strokeopacity = parseFloat(strokeOpacity);
		}
		else if (localstyle.strokeopacity === undefined) {
			localstyle.strokeopacity = styledata.strokeopacity; // force value for strokeopacity otherwise it won't get set
		}
		
		if(strokeDash){
			localstyle.strokedash = strokeDash;
		}
		
		if( opacity ){
			localstyle.strokeopacity = opacity;
			localstyle.fillopacity = opacity;
		}

		return localstyle;
	},
	
	deriveTextStyle: function( fragment, styledata ){
		
		var localstyle = dojo.clone( styledata );

		var family = fragment.getAttribute( "font-family" );
		var fontstyle = fragment.getAttribute( "font-style" );
		var size = fragment.getAttribute( "font-size" );
		var weight = fragment.getAttribute( "font-weight" );
		var stroke = fragment.getAttribute( "stroke" );
		var anchor = fragment.getAttribute( "text-anchor" );
		
		if(family){
			localstyle.fontFamily = family;
		}
		
		if(fontstyle){
			localstyle.fontStyle = fontstyle;
		}
		
		if(size){
			localstyle.fontSize = size;
		}
		
		if(weight){
			localstyle.fontWeight = weight;
		}
		
		if(stroke){
			localstyle.stroke = stroke;
		}
		
		if(anchor){
			localstyle.textAnchor = anchor;
		}

		return localstyle;		
	},

	/* Takes an SVG translation string that looks like this: 'translate( 123, 456 )' */

	setTranslation: function( current, translation ){

		var t = translation.split( "translate(" );

		if( t != null ){
			t = t[1].split( ")" );    
			var splitString = ",";
			if (dojo.isIE == 9 ) splitString = " ";
			var p = t[0].split(splitString);
			var translatex = current.translatex + parseFloat( p[0] );

			var ty = parseFloat( p[1] );

			var translatey;
			
			if( isNaN( ty ) ){ 
				translatey = current.translatey + 0; 
			}else{
				translatey = current.translatey + ty; 
			}        
		}
		
		var transform = { translatex: translatex, translatey: translatey };
		
		return transform;
	},

	buildStroke: function( style ){	

		var stroke = "none";

		if( style.stroke != "none" ){
			stroke = {};

			stroke.type = "stroke";
			stroke.cap = "butt";

			if( style.strokecolor ){
				stroke.color = style.strokecolor;

				if( style.strokeopacity >= 0 ){
					stroke.color.a = style.strokeopacity;
				}
			}

			stroke.style = "solid";
			stroke.width = style.strokewidth;
		}

		return stroke;
	},
	
	buildColor: function( style, color, opacity ){
		
		var colorObject = {};

		if( color == null ){ 
			color = "#FFFFFF";
		}

		if( color == 0 ){
			colorObject = new dojo.Color( color );   
			colorObject.a = 0;
			
		}else{
			colorObject = new dojo.Color( color );

			if( opacity != null ){
				colorObject.a = opacity;
			}    
		}

		return colorObject;
	},

	buildFill: function( style ){
		var fill = {};

		fill.color = style.fillcolor;

		if( fill.color == null ){ 
			fill.color = "#FFFFFF"; 
		}

		if( fill.color == 0 ){
			fill.color = new dojo.Color( fill.color );   
			fill.color.a = 0;
			
		}else{
			fill.color = new dojo.Color( fill.color );

			if( style.fillopacity != null ){
				fill.color.a = style.fillopacity;
			}    
		}

		return fill;
	},
	
	getDashType: function( dashArray ){

		var dashtype = "0";
		
		if( dashArray != null ){
			
			switch( dashArray ){
			
				case "28,21":
					dashtype = "1";
					break;
				
				case "7,21":
					dashtype = "2";
					break;
			}
		} 
		
		return dashtype;
	},
	
	buildModelData: function( shape, style ){
		
		var stroke = this.buildStroke( style );     
		var fill = this.buildColor( style, style.fillcolor, style.fillopacity );
	    var linecolor = this.buildColor( style, stroke.color, style.strokeopacity );

		shape.linecolor = linecolor.toHex();
		
		if( style.strokeopacity >= 0 ){		
			shape.strokeopacity = style.strokeopacity;
		}
		
		shape.linewidth = stroke.width;
		
		var fillcolor = new dojo.Color();
		fillcolor.setColor( fill );
		
		shape.fill = fillcolor.toHex();
		shape.fillopacity = fill.a;
		
		shape.strokedash = this.getDashType( style.strokedash );
	},

	buildShape: function( shape, style ){

		var shapeData = { "shape": shape };

		var stroke = this.buildStroke( style );     
		var fill = this.buildFill( style );

		if( fill.color != "none" ){
			shapeData.fill = fill.color;
		}

		if( stroke != "none" ){
			shapeData.stroke = stroke;
		}

		var shapeObject = new Object( shapeData );

		return shapeObject;
	},
	
	buildEmbeddedText: function( shape, fragment ){
		
		if( fragment.getAttribute( "text" ) ){
			
			var textObject = dojo.fromJson( fragment.getAttribute( "text" ) );
			
			shape.text = textObject.content;
			shape.fontStyle = textObject.style;
			shape.fontWeight = textObject.weight;
			shape.fontSize = textObject.size;
			shape.fontFamily = textObject.family;
			shape.textColor = textObject.color;			
		}
	},

	readPath: function( fragment, store, style, transform ){

		style = this.deriveStyle( fragment, style );
		
		var svgUtil = new rmps.web.comment.sketch.serializer.SVGUtil();	
		
		var newpath = fragment.getAttribute( "d" );
		
		if( style.svgHeight && style.svgWidth ){
			var node = document.createElement('div');
			var path = new dojox.gfx.path.Path(node);
			path.setShape({path: newpath});
			
			/* Figure out if there is any scale to be applied
			 * to the shape - we're assuming here that every
			 * svg block will have size, width and viewbox.
			 */
			
			var w = parseFloat( style.viewBox[2] );
			var h = parseFloat( style.viewBox[3] );
			
			var scalex = style.svgWidth/w; 
			var scaley = style.svgHeight/h;
			
			newpath = svgUtil.scalePath( newpath, scalex, scaley );
		}
		
		/* Move shape by svg x and y coordinates, and by any transform
		 * applied to the svg.
		 */
		
		newpath = svgUtil.repositionPath( newpath, this.svgX, this.svgY );
	    newpath = svgUtil.repositionPath( newpath, transform.translatex, transform.translatey );
		
		/* Collaboration client wraps each cloud path in an SVG tag with absolute 
		 * as a transform strategy.
		 */	

		var shape = {};  
		shape.type = "path";
		shape.shape = "Scribble";
		shape.path = newpath;
		
		this.buildEmbeddedText( shape, fragment );

		this.buildModelData( shape, style );

		store.push( shape );  
		
		return null; // path's can't contain text so no need to propogate the shape forward
	},   

	readText: function( fragment, store, style, transform, lastShape ){

		/* Example: <text x='106' y='141' font-size='10pt'>The Quick Brown Fox </text> */

		style = this.deriveTextStyle( fragment, style );
		style = this.deriveStyle( fragment, style ); 

		var shape = lastShape;
		if (!lastShape) {
			shape = {};
			shape.type = "text";
			shape.shape = "Text";
			var x = transform.translatex + parseFloat( fragment.getAttribute( "x" ) );
			var y = transform.translatey + parseFloat( fragment.getAttribute( "y" ) );
			var offset = 15;
			if (x < 0) x = 0;
			if (y < 0) y = offset;//This offset is required (as of FF 17.0.2, IE9) otherwise letters with current font and size would appear as if it is cut		
			shape.x = x;
			shape.y = y;
		}
		
		shape.text = unescape(fragment.firstChild ? fragment.firstChild.nodeValue : ''); 
		shape.fontFamily = "sans-serif";
		shape.fontStyle = "normal";
		shape.varient = "normal";
		shape.fontWeight = "normal";
		shape.fontSize = "10pt";
		
		var lastShapeFill = lastShape ? lastShape.fill : "black";
		var lastShapeStroke = lastShape ? lastShape.stroke : "black";
		var lastShapeStrokeOpacity = lastShape ? lastShape.strokeopacity : 1;
		var lastShapeFillOpacity = lastShape ? lastShape.fillopacity : 1;
		var lastShapeStrokeWidth = lastShape ? lastShape.linewidth : 1;
		
		shape.stroke = "black";
		shape.fill = "black";			
		this.buildModelData( shape, style );
		shape.textColor = shape.fill;
		shape.textOpacity = shape.fillopacity;
		shape.textStroke = shape.stroke;
		shape.textStrokeOpacity = shape.strokeopacity;
		shape.stroke = lastShapeStroke;
		shape.fill = lastShapeFill;
		shape.strokeopacity = lastShapeStrokeOpacity;
		shape.fillopacity = lastShapeFillOpacity;
		shape.linewidth = lastShapeStrokeWidth;
		
		if (shape.strokeopacity < 0.1) {
			shape.linewidth = 0;
			shape.shape = "TextBox"; // assume text box
		}
		
		if( style.fontSize ){
			
			/* Check if the fonsize has a 'pt' denomination within already.
			 * If not append one. */
			
			if( style.fontSize.indexOf("pt", 0) < 0 ){
				shape.fontSize = style.fontSize + "pt";
			} else{
				shape.fontSize = style.fontSize;
			}
		}

		if( style.fontWeight ){
			shape.fontWeight = style.fontWeight;  
		}

		if( style.fontFamily ){
			shape.fontFamily = style.fontFamily;  
		}

		if( style.fontStyle ){
			shape.fontStyle = style.fontStyle;  
		}
		
		if( style.strokecolor ){
			var col = new dojo.Color( style.strokecolor ).toHex();
			shape.textStroke = col;
		}
		
		shape.textStrokeOpacity = 0;
		
		shape.align = "start";
		
		// if text anchored in middle, offset x,y
		if (!lastShape) {
			if (style.textAnchor == "middle") {
				var style = {
						fontFamily: shape.fontFamily,
						fontSize: shape.fontSize,
						fontWeight: shape.fontWeight,
						whiteSpace: "nowrap"
					};
				var bnds = dojox.html.metrics.getTextBox(shape.text, style);
				shape.x -= bnds.w/2;
				shape.y -= bnds.h/2;
			}
			
			store.push( shape ); 
		}  
		
		return null;
	},   

	readLine: function( fragment, store, style, transform ){

		/* Example: <line x1="187" y1="331" x2="387" y2="126" style="stroke-width:4;stroke:#CC0000"/> */  

		style = this.deriveStyle( fragment, style );

		var shape = {};   
		shape.type = "line";
		shape.shape = "Line";
		shape.x1 = transform.translatex + parseInt( fragment.getAttribute( "x1" ) );
		shape.y1 = transform.translatey + parseInt( fragment.getAttribute( "y1" ) );                       
		shape.x2 = transform.translatex + parseInt( fragment.getAttribute( "x2" ) );
		shape.y2 = transform.translatey + parseInt( fragment.getAttribute( "y2" ) );

		this.buildEmbeddedText( shape, fragment );
		
		this.buildModelData( shape, style );

		store.push( shape );        
		
		return shape;
	},   

	readOval: function( fragment, store, style, transform ){

		/* Example: <g><ellipse cx="102.32558139534885" cy="68.13953488372093" rx="36.27906976744186" ry="18.837209302325583" 
	                       style="stroke-width:4;fill-opacity:0;stroke:#339966"/></svg> */    

		style = this.deriveStyle( fragment, style );

		var shape = {};     
		shape.type = "ellipse";  
		shape.shape = "Oval";
		shape.cx = transform.translatex + parseInt( fragment.getAttribute( "cx" ) );
		shape.cy = transform.translatey + parseInt( fragment.getAttribute( "cy" ) );                       
		shape.rx = parseInt( fragment.getAttribute( "rx" ) );
		shape.ry = parseInt( fragment.getAttribute( "ry" ) );

		this.buildEmbeddedText( shape, fragment );
		
		this.buildModelData( shape, style );

		store.push( shape );    
		
		return shape;
	},   

	readRectangle: function( fragment, store, style, transform ){

		/* Example: <g><rect x="59" y="154" width="108" height="73" style="stroke-width:4;stroke:#0066CC;fill-opacity:0"/></g></svg> */

		style = this.deriveStyle( fragment, style );

		var shape = {};    
		shape.type = "rect";      
		shape.shape = "Rectangle";
		shape.x = transform.translatex + parseInt( fragment.getAttribute( "x" ) );
		shape.y = transform.translatey + parseInt( fragment.getAttribute( "y" ) );                      
		shape.height = parseInt( fragment.getAttribute( "height" ) );          
		shape.width = parseInt( fragment.getAttribute( "width" ) ); 
		
		this.buildEmbeddedText( shape, fragment );

		this.buildModelData( shape, style );

		store.push( shape );
		
		return shape;
	},   

	readNode: function( node, style, transform, lastShape ){
						
		var child, styleString;
		
		if( 'getAttribute' in node ){

			switch( node.nodeName ){
	
				case "svg":
					
					styleString = node.getAttribute("style"); 		
					var svgStyle = dojo.clone( style );
					svgStyle.setStyle( styleString );	
					
					svgStyle = this.deriveTextStyle( node, svgStyle );
					svgStyle = this.deriveStyle( node, svgStyle );
										
					svgStyle.svgHeight = node.getAttribute("height");
					svgStyle.svgWidth = node.getAttribute("width");
					
					var vbox = node.getAttribute( "viewBox" );
					
					svgStyle.viewBox = vbox.split( " " );
										
					if( node.getAttribute("x") ){
						this.svgX = parseFloat( node.getAttribute("x") );
					}
			
					if( node.getAttribute("y") ){
						this.svgY = parseFloat( node.getAttribute("y") );
					}
		
					var start = 0;
					
					var lastShape = null;
					for(child = start; child < node.childNodes.length; child++ ){														
						lastShape = this.readNode( node.childNodes[child], svgStyle, transform, lastShape );	
					}
		
					break;
		
				case "g":
							
					styleString = node.getAttribute( "style" ); 		
					var groupStyle = dojo.clone( style );
					
					if( style.strokeopacity ){			
						groupStyle.strokeopacity = style.strokeopacity;
					}

					groupStyle.setStyle( styleString );
		
					groupStyle = this.deriveTextStyle( node, groupStyle );
					groupStyle = this.deriveStyle( node, groupStyle );
		
					if( node.getAttribute( "transform" ) ){ 		    
						var transformString = node.getAttribute( "transform" );
						transform = this.setTranslation( transform, transformString );
					}
		
					var lastShape = null;
					for(child = 0; child < node.childNodes.length; child++ ){
						lastShape = this.readNode( node.childNodes[child], groupStyle, transform, lastShape );
					}
		
					break;
		
				case "path":
										
					this.readPath( node, this.scribbles, style, transform );
					break;
		
				case "line":				
					return this.readLine( node, this.scribbles, style, transform );
		
				case "rect":				
					return this.readRectangle( node, this.scribbles, style, transform );
		
				case "circle":
		
					var radius = parseInt( node.getAttribute( "r" ) );
					node.setAttribute( "rx", radius );            
					node.setAttribute( "ry", radius );
		
					this.readOval( node, this.scribbles, style, transform );
					break;
		
				case "ellipse": 
					return this.readOval( node, this.scribbles, style, transform );
		
				case "text":
					this.readText( node, this.scribbles, style, transform, lastShape );
					break;
	
			}	 
		}
	},

	/* The main function starts here - input is an SVG document. The agreed format
	 * of SVG for exchange between the web client and desktop client consists of
	 * a single main group that itself contains several subgroups each with a 
	 * shape or collection of shapes within them */

	read: function( svgstring, adjustment ){

	    //Test data can be instantiated by uncommenting
        //var testdata = new rmps.web.comment.sketch.serializer.SVGTestData();

		var style = new rmps.web.comment.sketch.serializer.SVGStyle();

		this.scribbles = [];

		this.diagram = adjustment;
		
		console.log( svgstring );

		/* Read SVG file into DOM */
		
		var data = dojox.xml.parser.parse( svgstring );
		
		var transform = { translatex: 0, translatey: 0 };
		
		var start = 0;
						
		for( var child = start; child < data.childNodes.length; child++ ){
			
			this.readNode( data.childNodes[child], style, transform );
		}

		return this.scribbles;
	} 

	});

})();
