/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| Copyright IBM Corp. 2009, 2012.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {

dojo.provide("rmps.web.comment.sketch.serializer.SVGStyle");

dojo.require("rmps.web.comment.sketch.serializer.SVGColors");

dojo.declare("rmps.web.comment.sketch.serializer.SVGStyle", null, {

	/* String defintions used for parsing */

	STROKEWIDTH: "stroke-width:",
	STROKECOLOR: "stroke:",
	STROKEOPACITY: "stroke-opacity:",
	STROKEDASH: "stroke-dasharray:",

	FILLCOLOR: "fill:",
	FILLOPACITY: "fill-opacity:",

	FONTSIZE: "font-size:",
	FONTFAMILY: "font-family:",
	FONTWEIGHT: "font-weight:",
	FONTSTYLE: "font-style:",

	/* Color types */

	RGB_COLOR_TYPE: 0,
	HEX_COLOR_TYPE: 1, 
	STRING_COLOR_TYPE: 2,

	/* Stroke values */

	strokewidth:1,
	strokeopacity: 1,
	strokecolor: null,
	strokedash: "none",

	/* Fill values */

	fillcolor: null,
	fillopacity: null,

	/* Font values */

	fontSize: null,
	fontFamily: null,
	fontWeight: null,
	fontStyle: null,

	mystyle: null, 

	hexpick: function( hex, start, end ){
		return parseInt( hex.substring( start, end ), 16 );
	},

	hex2rgb: function( hexcode ){

		/* Accepts input like this '#112233' and returns a
		 * color object with an attribute for each rgb strength */

		var color = {};

		hexcode = hexcode.split(' ').join(''); //remove any spaces
		if( hexcode.charAt(0)=="#" ){ 
			hexcode = hexcode.substring( 1, 7 ); 
		}

		color.r = this.hexpick( hexcode, 0, 2 );
		color.g = this.hexpick( hexcode, 2, 4 );
		color.b = this.hexpick( hexcode, 4, 6 );

		return color;
	},

	string2object: function( color ){

		var colorObject;

		var names = rmps.web.comment.sketch.serializer.SVGColors.Names;

		var hexcode = names.getHexCode( color ); 

		colorObject = this.hex2rgb( hexcode );

		return colorObject;	   
	},

	rgb2object: function( color ){

		/* Example imput: "rgb( 1, 2, 3 )" */

		var colorCode = {};

		color = color.split( "(" )[1];

		/* s now looks like "1, 2, 3 )" */

		color = color.split( ")" )[0];

		/* s now looks like "1, 2, 3" */

		var numbers = color.split( "," );

		colorCode.r = parseInt( numbers[0] );
		colorCode.g = parseInt( numbers[1] );
		colorCode.b = parseInt( numbers[2] );

		return colorCode;
	},

	constructor: function( style ){

		/* Example: "stroke-width:4;fill-opacity:0;stroke:#339966" */

		this.mystyle = style;
		this.parseStyle();
	},

	parseStyle: function(){

		if( this.mystyle != null ){
			if( this.mystyle != "" ){
				
				this.parseStrokeOpacity();
				
				var strokecolor = this.lockonvalue( this.STROKECOLOR );

				if (strokecolor !== undefined) {
					if( strokecolor && strokecolor != "none" ){
						this.parseColor( "STROKE", strokecolor );
					}else{
						this.strokeopacity = 0;
					}
				}
				
				this.parseStrokeDash();

				this.parseFillOpacity();

				var fillcolor = this.lockonvalue( this.FILLCOLOR );
				
				if (fillcolor !== undefined) {
					if( fillcolor && fillcolor != "none" ){
						this.parseColor( "FILL", fillcolor );
					}else{
						this.fillopacity = 0;
					}
				}
				
				this.parseStrokeWidth();
				
				this.parseFont();
			}
		}
	},

	setStyle: function( style ){

		this.mystyle = style;

		this.parseStyle();
	},

	lockonvalue: function( target ){

		var value;

		if( this.mystyle.split( target )!= null ){
			value = this.mystyle.split( target )[1];

			if( value != null ){
				value = value.split( ";" )[0];
			}
		}

		return value;
	},

	parseStrokeWidth: function(){

		var strokewidth = parseInt( this.lockonvalue( this.STROKEWIDTH ) ); 	

		if( !isNaN(strokewidth) && strokewidth !== undefined ){
			this.strokewidth = strokewidth;
		}
	},

	parseFillOpacity: function(){

		var fillopacity = parseFloat( this.lockonvalue( this.FILLOPACITY ) ); 	

		if( fillopacity  >= 0 ){
			this.fillopacity = fillopacity;
		}    	
	},

	parseStrokeOpacity: function(){

		var strokeopacity = parseFloat( this.lockonvalue( this.STROKEOPACITY ) ); 	

		if( strokeopacity >= 0 ){
			this.strokeopacity = strokeopacity;
		}  
	},
	
	parseStrokeDash: function(){
		var strokedash = this.lockonvalue( this.STROKEDASH );
		
		if( strokedash != null ){
			this.strokedash = strokedash;
		}
	},

	parseFont: function(){

		var fontsize = this.lockonvalue( this.FONTSIZE );

		if( fontsize ){
			this.fontSize = fontsize;
		}

		var fontfamily = this.lockonvalue( this.FONTFAMILY );

		if( fontfamily ){
			this.fontFamily = fontfamily;
		}

		var fontweight = this.lockonvalue( this.FONTWEIGHT );

		if( fontweight ){
			this.fontWeight = fontweight;
		}

		var fontstyle = this.lockonvalue( this.FONTSTYLE );

		if( fontstyle ){
			this.fontStyle = fontstyle;
		}
	},

	getColorType: function( color ){

		/* Color could be in RGB, hex or string notation */

		var COLOR_TYPE;

		if( color.match( "rgb" ) != null ){
			COLOR_TYPE = this.RGB_COLOR_TYPE;    
		}else if( color.match( "#" ) != null ){
			COLOR_TYPE = this.HEX_COLOR_TYPE;
		}else{ 
			COLOR_TYPE = this.STRING_COLOR_TYPE; 
		}

		return COLOR_TYPE;
	},

	parseColor: function( type, color ){

		var colorObject = 0;

		if( color && color != "undefined" && color != "null") {
			
			if( color != "none" ){
			
				switch( this.getColorType( color ) ){
				
					case this.RGB_COLOR_TYPE:
						colorObject = this.rgb2object( color );
						break;
	
					case this.HEX_COLOR_TYPE:
						colorObject = this.hex2rgb( color );
						break;
	
					case this.STRING_COLOR_TYPE:
						colorObject = this.string2object( color );
						break;
				}				
			}

			if( type == "STROKE" ){
				this.strokecolor = colorObject;
			}else{
				this.fillcolor = color;  
			}
		}
	},

	getFill: function(){
		
		var fill = {};        
		fill.color = this.fillcolor;
		if( fill.color == null ){ 
			fill.color = "#FFFFFF"; 
		}    
		fill.color = new dojo.Color( fill.color );      
		
		if( this.fillopacity != null ){
			fill.color.a = this.fillopacity;
		}else{
			fill.color.a = 1;
		}

		return fill.color;
	},

	getStroke: function(){
		var stroke = {};

		stroke.type = "stroke";
		stroke.cap = "butt";
		stroke.color = new dojo.Color( this.strokecolor );
		stroke.color.a = this.strokeopacity;
		stroke.style = "solid";
		stroke.width = this.strokewidth;

		return stroke;
	},

	/* This function works with the JSON SVG representation */

	deriveStyle: function( fragment ){

	/* Style in SVG documents works hierarchically. RSA/Batik produces style at
	 * a group level then overrides aspects of it for each derived node in that 
	 * group. We're cascading style from the top level group to each of the
	 * subnodes, then checking if the nodes have their own local settings. */


		if( fragment.style ){
			this.mystyle = fragment.style;
			this.parseStyle();
		}

		if( fragment.stroke ){
			this.parseColor( "STROKE", fragment.stroke );
		}

		if( fragment.fill ){
			this.parseColor( "FILL", fragment.fill );
		}

		if( fragment.strokeWidth !== undefined ){
			this.strokewidth = parseInt( fragment.strokeWidth );
		}

		if( fragment.fillOpacity ){
			this.fillopacity = parseFloat( fragment.fillOpacity );
		}

		if( fragment.strokeOpacity ){
			this.strokeopacity = parseFloat( fragment.strokeOpacity );
		}
	}
	} );

})();
