/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| Copyright IBM Corp. 2009, 2010.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {

dojo.provide("rmps.web.comment.sketch.serializer.SVGUtil");

dojo.require("dojox.gfx.path");

dojo.declare("rmps.web.comment.sketch.serializer.SVGUtil", null, {
	
	round: function( value ){
		return Math.round(value*1000)/1000;
	},
	
	/* Converts a path to a smooth polyline */
	
	softenPath: function( segments ){
		
		var path = "";
		
		var factor = 1;
		
		if( segments != null && segments.length > ( factor -1 ) ){
		
			path = "M" + segments[0].args[0] + "," + segments[0].args[1];
			
			for( var item = factor; item < segments.length; item = item + factor ){								
				path += " L" + segments[item].args[0] + "," + segments[item].args[1];				
			}
		}
		
		return path;
	},

	scalePath: function( currentPath, xscale, yscale ){

		var newpath = "";
		var fragment ="";

		var segments = [];

		/* Using a gfx Path object to help parse the path string. */

		var node = document.createElement('div');
		var path = new dojox.gfx.path.Path(node);
		path.setShape({path: currentPath});
		segments = path.segments;
	
		for( var item = 0; item < segments.length; item++ ){
	
			switch( segments[item].action ){
	
			case "M":
			case "L":
	
				var newx = this.round( segments[item].args[0] * xscale );
				var newy = this.round( segments[item].args[1] * yscale );
	
				fragment = " " + newx + "," + newy;    			
				break;
	
			case "C":
	
				var x1 = this.round( segments[item].args[0] * xscale );
				var y1 = this.round( segments[item].args[1] * yscale );
				var x2 = this.round( segments[item].args[2] * xscale );
				var y2 = this.round( segments[item].args[3] * yscale );
				var x3 = this.round( segments[item].args[4] * xscale );
				var y3 = this.round( segments[item].args[5] * yscale );
	
				fragment = " " + x1 + "," + y1 + " " + x2 + "," + y2 + " " + x3 + "," + y3;
				break;			
			}		
			newpath = newpath + " " + segments[item].action + " " + fragment;		    			
		}
					    			
		return newpath;
	},
	
	repositionPath: function( currentPath, x, y ){
	
		var newpath = "";
	
		var node = document.createElement('div');

		var path = new dojox.gfx.path.Path(node);

		path.setShape( { path: currentPath } );
		
		var fragment = "";

		var segments = path.segments;

		for( var item = 0; item < segments.length; item++ ){

			switch( segments[item].action ){

			case "M":
			case "L":

				var newx = this.round( segments[item].args[0] + x );
				var newy = this.round( segments[item].args[1] + y );

				fragment = " " + newx + "," + newy;    			
				break;

			case "C":

				var x1 = this.round( segments[item].args[0] + x );
				var y1 = this.round( segments[item].args[1] + y );
				var x2 = this.round( segments[item].args[2] + x );
				var y2 = this.round( segments[item].args[3] + y );
				var x3 = this.round( segments[item].args[4] + x );
				var y3 = this.round( segments[item].args[5] + y );

				fragment = " " + x1 + "," + y1 + " " + x2 + "," + y2 + " " + x3 + "," + y3;
				break;			
				
			case "z":
				fragment = "";
				break;
			}	

			newpath = newpath + " " + segments[item].action + " " + fragment;		    			
		}
	
		return newpath;
	},
	
	buildStroke: function( stroke, traces ){

		var strokeTrace = traces.stroke.get();
		var strokeWidthTrace = traces.strokeWidth.get();
		var strokeOpacity = traces.strokeOpacity.get();
		var strokeDasharrayTrace = traces.strokeDasharray.get();

		if( strokeTrace ){
			var a = stroke.color.a;
			stroke.color = new dojo.Color( strokeTrace );
			stroke.color.a = a;				
		}
		
		if( strokeOpacity !== undefined ){
			stroke.color = new dojo.Color( strokeTrace );
			stroke.color.a = strokeOpacity;
		}

		if( strokeWidthTrace ){
			stroke.width = strokeWidthTrace;
		}
		
		if( strokeDasharrayTrace ){

			switch( strokeDasharrayTrace ){
				case "0":
					stroke.style = "Solid";
					break;

				case "1":
					stroke.style = "Dash";
					break;

				case "2": 
					stroke.style = "Dot";
					break;
			}				
		}
	},
	
	/* Returns the bounding box of a given shape */
	   
	   boundingBox: function( item ){
	   
	      var rectangle = {}, shape, left, right, top, bottom;
	      
	      if( item.path ){
	    	  
	    	  shape = item.path[0];
	    	  
	    	  var node = document.createElement('div');
	    	  
	    	  var path = new dojox.gfx.path.Path(node);
	  		  path.setShape({path: shape.d});
	  		  
	  		  rectangle = rmpw.diagrams.render.util.Workarounds.getBoundingBox(path);    	  
	      }
	            
	      if( item.rect ){	  
	    	  
	    	  shape = item.rect[0];
	    	  
	    	  rectangle.x = shape.x;
			  rectangle.y = shape.y;	    
			  rectangle.width = shape.width;    
			  rectangle.height = shape.height;
	      }
	      
	      if( item.ellipse ){
	    	  
	    	  shape = item.ellipse[0];
	    	  
	    	  left = shape.cx - shape.rx;      
		      right = shape.cx + shape.rx;      
		      top = shape.cy - shape.ry;      
		      bottom = shape.cy + shape.ry;
		      
		      rectangle.x = left; 
		      rectangle.y = top; 
		      rectangle.width = right - left;  
		      rectangle.height = bottom - top;  
	      }
	      
	      if( item.line ){
	    	  
	    	  shape = item.line[0];
	    	
		      if( shape.x1 > shape.x2 ){
		        left  = shape.x2;
		        right = shape.x1;    
		      }else{
		        left  = shape.x1;
		        right = shape.x2;	        
		     }
		    
		     if( shape.y1 > shape.y2 ){
		        top  = shape.y2;
		        bottom = shape.y1;    
		     }else{
		        top  = shape.y1;
		        bottom = shape.y2;	        
		     }
		    
		     rectangle.x = left;
		     rectangle.y = top;
		    
		     rectangle.width = right - left;
		     rectangle.height = bottom - top; 
	      }
	      

	      return rectangle;
	   },
	   
	   bounds: function( shapes ){
	   
	      var count = 0;
	      
	      var box = this.boundingBox( shapes[0] );
	 
	      var left = box.x;
				
	      var right = 0;
				
	      var top = box.y;
			
	      var bottom = 0;
	      
	      var totalwidth = 0;
	      
	      var totalheight = 0;
							
	      if( shapes.length > 0 )
	      {		
	         for( count = 0; count < shapes.length; count ++ )
	         {			  	  
		        var rect = this.boundingBox( shapes[ count ] );

		        var totalx = rect.x + rect.width;
				  	  
	  	        if( totalx > right ){ right = totalx; }
				  	  
		        if( rect.x < left ){ left = rect.x; }
				  	  
		        var totaly = rect.y + rect.height;
				  	  
		        if( totaly > bottom ){ bottom = totaly; }
				  	  
		        if( rect.y < top ){ top = rect.y; }			  	  
		     }			  		
				  
		     totalwidth =  Math.abs( right - left );
				  
		     totalheight = Math.abs( bottom - top );
		  
		     /* Want to make a wider buffer around the whole viewbox. */
		  
		     totalwidth = totalwidth + 10;
		  
		     totalheight = totalheight + 10;

	         left = left;
	      
	         top = top - 10;
	      }

	      var bounds = { top: top, bottom: bottom, left: left, right: right, totalwidth: totalwidth, totalheight: totalheight };

	      return bounds;   
	   }
				
});

})();