/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("rmps.web.comment.sketch.serializer.Writer");

dojo.require("dojox.gfx.path");
dojo.require("rmps.web.comment.sketch.serializer.SVGUtil");

dojo.declare("rmps.web.comment.sketch.serializer.Writer", null, {

   /* Takes an array of Dojo GFX serialized json shapes and
    * ouputs an SVG document of serialized XML shapes. */
    
   diagram: null, 

   constructor: function(){

   },
   
   /* SVG dash notations are described by two numbers the width of the dash followed 
    * by the width of the gap between the dashes.
    * 
    * When chosen within the sketching application, the dash choice is a single
    * number: 0, 1, or 2. 
    * 
    * 	0 = solid, or "none"
    * 	1 = dashes, or "28,21"
    * 	2 = dots, or "7,21"
    * 
    * This function translates
    */
   
   getDashArray: function( shape ){
	   
	   var dashArray = "none";
	   
	   var dash = shape.strokeDasharray;
	   
	   switch( dash ){

  		case "1": // dashes
  			dashArray = "28,21";
  			break;
  			
  		case "2": // dots
  			dashArray = "7,21";
  			break;
	   }
	   
	   return dashArray;
   },

   /* Converts an RGB color to a hex code - not really necessary at the moment */

   convertRGBString:function( color ){

      /* E.G: "rgb( 1, 2, 3 )" */
			
      var colorCode;
			
      color = color.split( "(" )[1];
			
      /* s now looks like "1, 2, 3 )" */
			
      color = color.split( ")" )[0];
			
      /* s now looks like "1, 2, 3" */
			
      var numbers = color.split( "," );
			
      colorCode = Number(numbers[0])*256*256 + Number(numbers[1])*256 + Number(numbers[2]);
			
      return colorCode;	
   },

   /* Produces an rgb text string from a gfx shape */ 

   rgb:function( item ){
      return "rgb(" + item.r + "," + item.g + "," + item.b + ")";
   },

   /* Calculates the overall bounding box of the shape in the store -
    * necessary for the RSA desktop */

   viewbox: function( shapes ){

      var viewbox ="";
      
      var util = new rmps.web.comment.sketch.serializer.SVGUtil();

      var bbox = util.bounds( shapes );

      viewbox = "width='" + bbox.totalwidth +
	            "' height='" + bbox.totalheight  +
	            "' viewBox='" +  bbox.left + " " + bbox.top + " " + bbox.totalwidth + " " + bbox.totalheight + "'"; 

      return viewbox;
   },
   
   buildStyle: function( shape ){
	   
	   var style;
	   
	   style = "' style='stroke-width:" + shape.strokeWidth + ";stroke:" + shape.stroke;
	   
	   if( shape.strokeOpacity != undefined ){
		   style = style + ";stroke-opacity:" + shape.strokeOpacity;   
	   }
	   
	   if( shape.fill != undefined ){
		   style = style + ";fill:" + shape.fill + ";fill-opacity:" + shape.fillOpacity; 
	   }
       
	   style = style + ";stroke-dasharray:" + this.getDashArray(shape); 
	   
	   return style;
   },
   
   textItem: function( key, value, comma ){
	   var component = key + ':\"' + value + '\"';
	   
	   if( comma ){ 
		   component = component + ", ";
	   }
	   return component;
   },
   
   addText: function( shape, cx, cy, isTextOnly ){
	   var textdata = "";
	   if( shape.text ){
		  textdata = "<text xml:space='preserve' x='"+ cx + 
				  "' y ='"+ cy +
				  "' text-anchor='middle' alignment-baseline='middle"+
				 "' font-size='" + shape.fontSize + 	   				  
				 "' font-style='" + shape.fontStyle + 	   				  
				 "' font-family='" + shape.fontFamily + 
		          "' font-weight='" + shape.fontWeight + 
 			      "' fill='" + shape.textColor +
 			      "' stroke='" + shape.textColor +
 			      "' stroke-width='0" +
			          "' >"+ 
				  escape(shape.text) + 
				  "</text>";
	   }	   
	   return textdata;
   },

   write: function( shapes ){
   
      var xmlstring;
      
      if( shapes.length > 0 ){
    	  
      	 xmlstring = "<?xml version='1.0'?>";

      	 var util = new rmps.web.comment.sketch.serializer.SVGUtil();

         var svgdata = "<svg xmlns=\"http://www.w3.org/2000/svg\" " + this.viewbox( shapes ) + ">";

         svgdata = svgdata + "<g transform = \"translate(0,0)\">";
         
         for( var count = 0; count < shapes.length; count++ ) {
        	 
           var current = shapes[ count ], shape;
           
           if (current.text){
        	   shape = current.text[0];
        	   
        	   svgdata = svgdata + "<g>" + this.addText(shape, shape.x, shape.y, true) +"</g>";
           }

           if( current.rect ){
        	   
        	   /* Example: <g><rect x="59" y="154" width="108" height="73" style="stroke-width:4;stroke:#0066CC;fill-opacity:0"/></g></svg> */

        	   shape = current.rect[0];
        	   
        	   // just plain text
        	   var rectdata = "<g><rect x='" + shape.x +
			                     "' y='" + shape.y +
				              "' width='" + shape.width +
				              "' height='" + shape.height;
	        	   				                      	   
	        	svgdata = svgdata + rectdata  + this.buildStyle(shape) + "'/>"+ 
	        	                   this.addText(shape, (shape.x + shape.width/2),  (shape.y + shape.height/2)) +"</g>";
           }
           
           if( current.path ){      	   
        	   
        	   shape = current.path[0];
        	   
        	   var dash = this.getDashArray( shape );
        	   var bbox = util.boundingBox( current );
        	   
        	   var pathdata = "<g><path fill='" + shape.fill + 
        	   			      "' stroke-width='" + shape.strokeWidth + 
        	   			      "' stroke='" + shape.stroke + 
        	   			      "' stroke-opacity='" + shape.strokeOpacity + 
        	   			      "' fill-opacity='" + shape.fillOpacity +
        	   			      "' stroke-dasharray='" + this.getDashArray(shape) +
        	   			      "' d='" + shape.d + "'/>"+
        	   			     this.addText(shape, (bbox.x + bbox.width/2),  (bbox.y + bbox.height/2)) +     	   			      
        	   			      "</g>";
 	          
               svgdata = svgdata  + pathdata;      
           }
           
           if( current.ellipse ){
        	   
        	   /* Example: <g><ellipse cx="300" cy="150" rx="200" ry="80" style="fill:rgb(200,100,50);stroke:rgb(0,0,100);stroke-width:2"/></g></svg> */
		       
        	   shape = current.ellipse[0];
        	   
               var ovaldata = "<g><ellipse cx='" + shape.cx +
		                      "' cy='" + shape.cy +
				              "' rx='" + shape.rx +
				              "' ry='" + shape.ry;
               
        	   svgdata = svgdata + ovaldata  + this.buildStyle(shape) + "'/>"+ this.addText(shape, shape.cx, shape.cy) +"</g>"; 	   
           }
           
           if( current.line ){
        	
        	   /* Example: <g><line x1="50" y1="50" x2="200" y2="300" style="stroke:#ABCDEF;stroke-width:20"/></g> */
        	   
        	   shape = current.line[0];

		       var linedata = "<g><line x1='" + shape.x1 +
		                      "' y1='" + shape.y1 +
				              "' x2='" + shape.x2 +
				              "' y2='" + shape.y2;
		       
        	   svgdata = svgdata + linedata  + this.buildStyle(shape) + "'/>"+ 
        	               this.addText(shape,(shape.x1 + (shape.x2 - shape.x1)/2), (shape.y1 + (shape.y2 - shape.y1)/2)) 
        	            		   + "</g>"; 	   
           }
        }
         
        xmlstring = xmlstring + svgdata +"</g></svg>";
      }
      
      if( content = false ){ 
         // all shapes were deleted 
         xmlstring = null ;
      }else{    
         console.log( "WRITING: " + xmlstring );
      }
      
      return xmlstring;
   }

} );

})();
