/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.common.services.ResourceState");

dojo.require("rmps.core.client");
dojo.require("dojo.i18n");
dojo.requireLocalization("rmpw.common", "ResourceMessages");
/**
 * Checks resource state
 * 
 * @author Susanta Datta
 */
var resourceMessages = dojo.i18n.getLocalization("rmpw.common", "ResourceMessages");

dojo.declare("rmpw.common.services.ResourceState", null, {
	
	constructor: function(args) {
		dojo.mixin(this, args);
	},
	
	lock : function (resourceUri, arg, docUriToETag) {
		this.beforeLock(resourceUri, arg);
		var result = new dojo.Deferred();
		this.getResourcePostStateDfs(resourceUri, 'lock', docUriToETag).then(dojo.hitch(this, function (resourceUri, lockObj) {
			var obj = dojo.fromJson(lockObj);
			if (obj['isLocked'] != undefined && obj['isLocked']==true) {
				if (obj['lockedBy']=='self' || obj['lockedBy']=='modified') {
					result.resolve(obj['lockedBy']);
					this.onLock(obj['lockedBy']);
				} else {
					result.reject(obj['lockedBy']);
					this.onLockError(obj['lockedBy'], null, null);
				}
			} else if (obj['wrongETag']) {
				// failed due to wrong eTag
				result.reject(obj['lockedBy']);
				this.onLockError(null, null, obj['wrongETag']);
			}
			result.finished = true;
		}, resourceUri), dojo.hitch(this, function (error){
			result.reject(error);
			this.onLockError(null, null, error);
			
		}) );
		return result;
	},	
	
	unlock : function (resourceUri, arg) {
		var result = new dojo.Deferred();
		this.getResourcePostStateDfs(resourceUri, 'unlock').then(dojo.hitch(this,  function (resourceUri, lockObj) {
			var obj = dojo.fromJson(lockObj);
			if (obj['isLocked'] != undefined && obj['isLocked']==false) {
				result.resolve();
				this.onUnLock();
			} else {
				result.reject();
				var msg = dojo.string.substitute(resourceMessages.lockedBy, [this._getResourceName(resourceUri), obj['lockedBy']]);
				this.onUnLockError(obj['lockedBy'],msg);
			}
		}, resourceUri), dojo.hitch(this, function (error){
			this.onUnLockError(null, error);
			result.reject(error);
			
		}) );
		return result;
	},	
	
	lockAll : function(resources, arg, resourceNames, docUriToETag) {
		var result = new dojo.Deferred();
		var list = [];
		for (var i=0; i<resources.length; i++) {
			list.push(this.getResourcePostStateDfs(resources[i], 'lock', docUriToETag));
		}
		var dl = new dojo.DeferredList(list);
		dl.then( dojo.hitch(this,  function(lockObj) {
			var obj = dojo.fromJson(lockObj);
			if (obj['isLocked'] != undefined && obj['isLocked']==true){
				if (obj['lockedBy']=='self' || obj['lockedBy']=='modified') {
					result.resolve(arg);
					this.onLockAll(resources);
				} else {
					result.reject(arg);
					// find the resource name for resource whose locking failed
					var lockedResURI = obj['lockedResURI'];
					var lockedResName;
					for (var i = 0, uri; uri = resources[i] && !lockedResName; i++) {
						if (uri = lockedResURI) {
							lockedResName = resourceNames[i];
						}
					}
					this.onLockError(obj['lockedBy'], (lockedResName && lockedResName != "") ? lockedResName : lockedResURI, null);
				}
			} else if (obj['wrongETag']) {
				// failed due to wrong eTag
				result.reject(obj['lockedBy']);
				this.onLockError(null, null, obj['wrongETag']);
			}
			}), dojo.hitch(this, function (error){
				this.onLockError(null, null, error.message);
				result.reject(error);
			}) );
		return result; 
	},
	
	_getResourceName: function(resourceUri){
		var index = resourceUri.lastIndexOf('/');
		if (index!=-1) {
			return resourceUri.substring(index+1, resourceUri.length);
		}
		return resourceUri;
	},
	
	beforeLock: function (){
		
	},
	
	onLock: function (){
		
	},
	
	onLockAll: function (){
		
	},	
	onUnLock: function (){
		
	},
	onLockError: function (error){

	},
	onUnLockError: function (){
		
	},
	isLocked: function (resourceUri) {
		// Returns 1) or 2):
		// 1) resolved deferred if the resource is locked and the user can unlock, 
		//    or if it is not locked (lockedBy is "none");
		// 2) rejected deferred if it is locked and user cannot unlock unless has ability to force it (lockedBy is provided)
		var result = new dojo.Deferred();
		this.getResourceStateDfs(resourceUri).then( function(lockedBy) {
			if (lockedBy!=null && lockedBy.length!=0){
				var obj = dojo.fromJson(lockedBy);
				if (obj['isLocked'] != undefined && obj['isLocked']==true){
					// locked
					if (obj['lockedBy']=='self' || obj['lockedBy']=='modified') {
						result.resolve(obj['lockedBy']);
					} else {
						result.reject(obj['lockedBy']);
					}
				} else {
					// not locked
					result.resolve("none");
				}
			} else {				
				result.resolve();
			}
		});
		return result;
	},
	
	getResourceStateDfs : function (resourceName) {
		var uri = net.jazz.ajax._contextRoot + '/resourceState?logicalURI='+encodeURIComponent(resourceName);
		return rmps.core.client.xhrGet({
			url : uri,
			headers: {
				"Accept": "application/json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "text"
		}, true);
	},
	getResourcePostStateDfs : function (resourceName, state, docUriToETag) {
		var uri = net.jazz.ajax._contextRoot + '/resourceState?state='+state+'&logicalURI='+encodeURIComponent(resourceName);
		return rmps.core.client.xhrPost({
			url : uri,
			headers: {
				"X-ibm-rmps-internal": "true",
				"ETag": docUriToETag
			},
			handleAs: "text"
		}, true);
	}	
});
