/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.common.services.StreamService");
dojo.require("rmps.core.client");
dojo.require("rmps.core.error");
/**
 * Common stream service, like default stream uri etc.
 * @author Susanta Datta
 */
dojo.declare("rmpw.common.services.StreamService", null, {
	
	getDefaultStream : function() {
		var result = new dojo.Deferred();
		var projectName = jazz.app.context.get().name; 
		var self = this;
		this.getProjectDfd().then(function(projects){	
			var projectId = self.getProjectID(dojo.fromJson(projects), projectName);
			if (!projectId){
				result.reject('can not find project id for project '+projectName);
			} else { 
				self.getStreamDfd(projectId).then(function(stream){	
					var streamURI = self.getStreamUri(dojo.fromJson(stream));
					if (!streamURI){
						result.reject('can not find default stream ');
					} else {
						result.resolve(streamURI);
					}
				});
			}
		});
		return result;
	},
	
	getProjectDfd : function(){
		var uri = net.jazz.ajax._contextRoot + '/projects';
		return rmps.core.client.xhrGet({
			url : uri,
			headers: {
				"Accept": "application/rdf+json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "text",
			error: function(error, ioArgs) {
			    rmps.core.error.createXhrError(ioArgs, "Attempting to get stream service's project dfd.").dispatch();
			    console.error(error);
			}
		}, true);
	},
	getStreamDfd:  function(projectId){
		var uri = net.jazz.ajax._contextRoot + '/streams?projectId='+projectId+'&defaultStream=True';
		return rmps.core.client.xhrGet({
			url : uri,
			headers: {
				"Accept": "application/rdf+json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "text",
			error: function(error, ioArgs) {
			    rmps.core.error.create(ioArgs, "Attempting to get stream service's stream dfd.").dispatch();
			    console.error(error);
			}
		}, true);
	},
	getProjectID: function (projects, projectName){
		for (var property in projects) {
			var obj = projects[property];
			var nameObj = obj['http://jazz.net/xmlns/rmps/groups/1.0/projectName'];
			for (var i=0; i<nameObj.length; i++){
				if (nameObj[i].value===projectName){
					var arrayVal = obj['http://jazz.net/xmlns/rmps/groups/1.0/projectId'];
					for (var i=0; i<arrayVal.length; i++){
						if (arrayVal[i].value){
							return arrayVal[i].value;
						}
					}
				}
			}
		}
		return null;
	},
	getStreamUri: function (stream){
		for (var property in stream) {
			return property;
			var obj = projects[property];
			var nameObj = obj['http://jazz.net/xmlns/rmps/groups/1.0/projectName'];
			for (var i=0; i<nameObj.length; i++){
				if (nameObj[i].value===projectName){
					var arrayVal = obj['http://jazz.net/xmlns/rmps/groups/1.0/projectId'];
					for (var i=0; i<arrayVal.length; i++){
						if (arrayVal[i].value){
							return arrayVal[i].value;
						}
					}
				}
			}
		}
		return null;
	}	
});