/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

(function() {
	dojo.provide("rmps.core.streams.StreamClient");

	dojo.require("jazz.app.context");
	dojo.require("rmps.core.RootServices");
	dojo.require("rmps.core.streams.Stream");
	dojo.require("rmps.core.util.IRmpsConstants");

	dojo.require("dojo.i18n");

	var STREAMS_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "streams");

	rmps.core.streams.StreamClient = {

	// Returns the URI's of all the streams
	// projectName is optional; will default to the current project
	getStreams: function(projectId){
		var query = {};
		// Default to the current project name is none is provided
		query.projectId = projectId ? projectId : jazz.app.context.get().itemId;
		var args = {
				url: STREAMS_PATH + "?" + dojo.objectToQuery(query),
				headers: {
			"Accept" : "application/json",
			"X-ibm-rmps-internal" : "true"
		},
		handleAs: "json",
		load: function( response, ioArgs ) {
			var streams = [];

			for(var uri in response){
				var stream = new rmps.core.streams.Stream();
				stream.parseRdfJson(uri, response[uri]);
				streams.push(stream);
			}
			return streams;
		}// end load
		}; // end args
		return jazz.client.xhrGet(args);
	}, // end getStreams

	// Throws an exception if the user cannot update the current configuration
	canUpdateStream: function(){
		var query = {};
		query[rmps.core.util.IRmpsConstants.BASELINE] = jazz.app.context.get().baseline;
		query.canUpdate = "true";
		var args = {
				url: STREAMS_PATH + "?" + dojo.objectToQuery(query),
				headers: {
					"X-ibm-rmps-internal" : "true"
				},
		}; // end args
		return jazz.client.xhrGet(args);
	}, // end canUpdateStream

	/**
	 * Gets all the visible streams and returns the raw JSON data. if
	 * projectId is null or not found, it will default to the current
	 * context if the sync variable is null it defaults to false, otherwise
	 * it will be the interpretation of the value. Should be *strings*
	 * "true" or "false" only.
	 */
	getStreamsJSON: function(projectId, sync){
		var query = {};
		// Default to the current project name is none is provided
		query.projectId = projectId ? projectId : jazz.app.context.get().itemId;
		var args = {
			url: STREAMS_PATH + "?" + dojo.objectToQuery(query),
			headers: {
				"Accept" : "application/json",
				"X-ibm-rmps-internal" : "true"
			},
			sync: sync ? sync : "false",
			handleAs: "json"
		}; // end args

		return jazz.client.xhrGet(args);
	},

	/**
	 * Gets a single stream
	 */
	getStream: function(uri, parent){
		var query = {};
		var streamUrl = uri;
		if (parent){
			query.snapshotId = uri;
			streamUrl = STREAMS_PATH + "?" + dojo.objectToQuery( query );
		}		
		var args = {
			url: streamUrl,
			headers: {
				"Accept": "application/rdf+json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, ioArgs) {
				// Test & Fix this...
				for (var uri in response) {
					var stream = new rmps.core.streams.Stream();
					stream.parseRdfJson(uri, response[uri]);
					return stream;
				}
				// Return nothing, if nothing is found
				return null;
			}
		};
		return jazz.client.xhrGet(args);		
	}, // end getStream

	// request for default stream is "defaultStream" for the projectId
	getDefaultStream: function(projectId){
		var query = {};
		var defaultUri;
		var self = this;

		if (projectId) {
			query.projectId = projectId;
		} else {
			// Default to the current project
			query.projectId = jazz.app.context.get().itemId;
		}
		if (jazz.app.context.get().itemId === query.projectId && !jazz.app.context.get().baseline) {
			// We are about to get default stream of the currently active project...
			// The only chance to have empty or undefined "baseline" is to have project with non-configured space.
			// Since space has not been configured yet, then default stream has not been created yet and 
			// there is no need to send request to the server.
			return null;
		}
		query.defaultStream = "true";
		var args = {
			url: STREAMS_PATH + "?" + dojo.objectToQuery( query ),
			defaultStream: "true",
			headers: {
				"Accept": "application/json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",		
//			sync: "true",
			load: function(response, ioArgs){
				for (var uri in response) {
					var stream = new rmps.core.streams.Stream();
					stream.parseRdfJson(uri, response[uri]);
					return stream.uri;
				}
				return null;
			} // end load
		}; // end args
		query.args = args;
		return jazz.client.xhrGet(args);
	}, // end getDefaultStream

	// Creates a new stream within the project with a specified name.
	createStream: function(projectId, streamName, baselineUri, description){
		var query = {};

		query.projectId = projectId;
		query.name = streamName;
		query.baselineUri = baselineUri;

		if (description) {
			query.description = description;
		}

		var args = {
			url: STREAMS_PATH + "?" + dojo.objectToQuery(query),
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			hasBody: false,
			load: function(response, ioArgs) {
				// return the URI of the baseline created
				var newStreamUri = ioArgs.xhr.getResponseHeader("Location");
				return newStreamUri;
			}
		};

		query.args = args;
		return jazz.client.xhrPost(args);
	}
	};
})();