/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.subscriptions.SubscriptionClient");

dojo.require("rmps.core.subscriptions.Subscription");

dojo.require("jazz.client.xhr");

dojo.require("rmps.core.RootServices");

var SUBSCRIPTION_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "subscriptions");
var SUBSCRIPTION_PREDICATE = "http://jazz.net/xmlns/rmps/subscriptions/1.0/Subscription";

rmps.core.subscriptions.SubscriptionClient = {
	
	subscribe: function(resourceUri, userUri) {
		var data = '<' + resourceUri + '> <' + SUBSCRIPTION_PREDICATE + '> <' + userUri + '>.';
		var args = {
			headers: {
				"Content-Type": "application/ntriples",
				"X-ibm-rmps-internal": "true"
			},
			url: SUBSCRIPTION_PATH,
			hasBody: true,
			postData: data
		};
		return jazz.client.xhrPost(args);
	},
	
	unsubscribe: function(resourceUri, userUri) {
		var queryParams = {
			"resourceUri": resourceUri,
			"userUri": userUri
		};
		var args = {
			headers: {
				"X-ibm-rmps-internal": "true"
			},
			url: SUBSCRIPTION_PATH + '?' + dojo.objectToQuery(queryParams)
		};
		return jazz.client.xhrDelete(args);
	},
	
	getSubscriptions: function(resourceUri, userUri) {
		var queryParams = {};
		if (resourceUri) {
			queryParams.resourceUri = resourceUri;
		}
		if (userUri) {
			queryParams.userUri = userUri;
		}
		var args = {
			url: SUBSCRIPTION_PATH + '?' + dojo.objectToQuery(queryParams),
			headers: {
				"Accept": "application/atom+json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			load: function(response, io)
			{
				var subscriptions = [];
				var entries = response.entry;
				
				// TODO is entries.length > 0 necessary?
				if (entries && entries.length > 0) {
					for (var i = 0; i < entries.length; i++) {
						var descriptions = entries[i].content[0]["rdf:RDF"][0]["rdf:Description"];
						if (descriptions && descriptions.length > 0) {
							for (var j = 0; j < descriptions.length; j++) {
								var subscription = new rmps.core.subscriptions.Subscription();
								subscription.resourceUri = descriptions[j]["rdf:about"];
								
								// TODO This parser should resolve and validate the prefix of this QName
								subscription.userUri = descriptions[j]["rmpsSubscriptions:Subscription"][0]["rdf:resource"];
								subscriptions.push(subscription);
							}
						}
					}
				}
				return subscriptions;
			}
		};
		return jazz.client.xhrGet(args);
	}
};

})();