/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.ActionBar");

dojo.require("rmpw.common.policy.Policy");

/**
 * Adds diagram toolbar contributions of its host to the diagram toolbar when the host is selected, 
 * and removes them when it is not.
 */
dojo.declare("rmpw.diagrams.edit.tools.ActionBar", rmpw.common.policy.Policy, {
	singleton: "actionbar",
	
	constructor: function(host) {
		this.toolBarController = this.host.root.policies.actionBarScope;
		this.selectionScope = this.findNearestParentByKey('selectionScope', true);
		// listen to changes in selectionScope and not selectable as some shapes may not 
		// have selectable installed (like diagram shape).
		this._selectionListener = dojo.connect(this.selectionScope, "onSelectionChanged", this, "onSelectionChanged");
		this._dynamicPolicyTools = [];
	},
	
	destroy: function() {
		dojo.disconnect(this._selectionListener);		
	},
	
	/*
	 * Add bindings
	 */
	setActionBindings: function () {
		this._actionBindings = arguments;
	},
	
	onSelectionChanged : function(e) {
		// note: currently, actionBar is used only on the diagram shape; if we start using it elsewhere, 
		// this can be improved to avoid re-creation of the same widgets, e.g. if selection changes 
		// to the shape that has the same actionBar bindings as previously selected shape;
		// also, consider multi-selection in that case
		var selected = false;
		var selection = this.selectionScope.getSelection();
		for (var index = 0; index < selection.length && !selected; index++) {
			if (this.host === selection[index]) {
				selected = true;				
			}
		}
		if (!selected) {
			// my host is not selected any more, remove tools it contributed
			this.toolBarController.destroyTools(this._dynamicPolicyTools);
			this._dynamicPolicyTools = [];
		} else if (this._dynamicPolicyTools.length === 0) {
            // add tools contributed by my host to the toolbar 
			// (unless already added - can happen with multi-clicks on diagram surface)
			this._dynamicPolicyTools = this.getToolbarWidgets();
			this.toolBarController.addTools(this._dynamicPolicyTools);
		}
	},
		
	clearWidget: function(id) {
		if (id) {
			var item = dijit.byId(id);
			if (item) {
				item.destroy();
			}
		}
	},
	
	/**
	 * Returns toolbar widgets created based on _actionBindings set on this policy. 
	 */
	getToolbarWidgets: function(){
		var widgets = [],
		  i=0, 
		  binding,
		  j=0, 
		  subbinding;
		
		var self = this;
		for (i=0, binding; binding = this._actionBindings[i]; i++) {
			if (binding.getActionBindings) {
				var menu = new dijit.Menu( { style: "display: none;" } );
				
				var subbindings = binding.getActionBindings();
				for (j=0, subbinding; subbinding = subbindings[j]; j++) {
					var clickedAction = { action: subbindings[j].getAction(), run: function() {
						this.action.run({viewer: self.host.root});
					}};
					
					// TODO: actually fix cause of exception, but for now keep it from ripping up the stack
					try {
						this.clearWidget(subbinding.getId());
						var menuItem = new dijit.MenuItem({
							label: subbinding.getLabel(),
							onClick: dojo.hitch(clickedAction, clickedAction.run),
							iconClass: subbinding.getIconClass(),
							id: subbinding.getId()
						});
						menu.addChild(menuItem);
					} catch(e) {
						// ignore
					}
				}
				
				this.clearWidget(binding.getId());
				widgets.push(new dijit.form.DropDownButton({
                    label: binding.getDescription(),
                    showLabel: false,
                    name: binding.getLabel(),
                    dropDown: menu,
                    iconClass: binding.getIconClass(),
                    id:
                    binding.getId()
                }));
			}
			else {
				this.clearWidget(binding.getId());
				widgets.push(new dijit.form.Button({ 
                    label: binding.getLabel(),
                    showLabel: show,
                    action: binding.getAction(),
                    onClick: dojo.hitch( this, function() {
                    this.action.run({viewer: self.host.root});
                    }),
                    iconClass: binding.getIconClass(),
                    id: binding.getId()}));
			}
		}
		
		return widgets;
	}
});