/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.ContextActionBar");

/**
 * Note: depends on Selectable policy to be attached first
 */
dojo.declare("rmpw.diagrams.edit.tools.ContextActionBar", null, {
	singleton: "contextActionBar",
	
	constructor: function(host) {
		this.host = host;
		this._actionBindings = [];
		this._toolbarActions = []; // map of element type ID to descriptor info
		this.toolBarController = this.host.root.policies.contextActionBarScope;
		
		// choose good defaults
		if (this.host.renderMapping.policies.graphNode) {
			this._horizontalAlignment = "Center";
			this._verticalAlignment = "Top";
		}
		else {
			this._horizontalAlignment = "Right";
			this._verticalAlignment = "Center";
		}
		
		if (this.host.policies.selectable) {
			this.initToolbar();
		}
		else if (this.host.parent.attachChild) {
			// maybe Marc's dependency ordering would solve this issue
			this._connectAttachChild = dojo.connect(this.host.parent, "attachChild", this, "initToolbar");
		}
		
		this._feedbackMode = true; // no point having divergent behavior - make this the default
	},
	
	setFeedbackMode: function(enable) {
		this._feedbackMode = enable;
	},
	
	initToolbar: function() {
		if (this._connectAttachChild) {
			dojo.disconnect(this._connectAttachChild);
			delete this._connectAttachChild;
		}
		
		if (this.host.policies){		
			if (this.host.policies.selectable) {
				this._selectionListener = dojo.connect(this.host.policies.selectable, "onSelectionChanged", this, "onSelectionChanged");
			}
			
			if (this.host.policies.focusable) {
				this._connectFocus = dojo.connect(this.host.policies.focusable, "setFocus", dojo.hitch(this, function(focus) {
					if (focus === true && !this.toolBarController.isToolBarShowing() &&
						!this.toolBarController.moving && this.host.policies.selectable.isSelected()) {
						this.showToolBar({});
					}
				}));
			}	
		}
	    
		if (this.host.renderMapping.policies){
		    if (this.host.renderMapping.policies.graphContainer && 
            this.host.renderMapping.policies.graphNode) {
                this._posMode = 2;
            } else
            if(this.host.renderMapping.policies.graphContainer) {
                this._posMode = 1;
            } else {
                this._posMode = 0;
            }
		}
	},
	
	destroy: function() {
		dojo.disconnect(this._selectionListener);
		if (this._connectFocus) {
			dojo.disconnect(this._connectFocus);
		}
		this.toolBarController.removeToolBarImmediate();
	},
	
	onSelectionChanged: function(selected, e) {
		this.inherited(arguments);
	},
	
	showToolBar: function(e) {
		
		/* If the event was a touch event, need to pick out the first touch element
		 * from the array of possible events.
		 */
		var bounds, anchor; 
	    var eFixed = e.changedTouches ? e.changedTouches[0] : e; 

        if (this._feedbackMode) {
            // set the upper left position where shape should be created
			this.toolBarController.setCreationPoint((eFixed.clientX !== undefined && eFixed.clientY !== undefined) ? 
					this.host.root.viewport.getSurfaceAbsoluteMousePosition(eFixed.clientX, eFixed.clientY - 7) : null);
		}
		
		var renderMapping = this.host.renderMapping;
		
		if (renderMapping.policies.graphEdge) {
			var figurePoint, start, end, i;
			var points = renderMapping.policies.graphElement.getPolygonPoints();

			if(eFixed.clientX && eFixed.clientY){
				figurePoint = this.host.root.viewport.getSurfaceAbsoluteMousePosition(eFixed.clientX, eFixed.clientY);
				start = points.getPoint(0);
				end = points.getPoint(points.size - 1);
			} else {
				if (points.size % 2 === 0) {
					i = points.size / 2;
					var j = i - 1;
					start = points.getPoint(j);
					end = points.getPoint(i);
					var d = end.getDifference(start);
					figurePoint = new rmpw.diagrams.render.util.Point(start.x + d.width / 2, 
							start.y + d.height / 2);
					renderMapping.policies.graphElement.translateToAbsolute(figurePoint);
				} else {
					i = (points.size - 1) / 2;
					var p1 =points.getPoint(i);
					start = points.getPoint(i-1);
					end = points.getPoint(i+1);
					figurePoint =  new rmpw.diagrams.render.util.Point(p1.x, p1.y);
					renderMapping.policies.graphElement.translateToAbsolute(figurePoint);
				}	
			}

			// check orientation of start and end points to determine anchor of menu
			if (end.y < start.y) {
				if (end.x < start.x) {
					anchor = this.toolBarController.BOTTOM_LEFT;
					figurePoint.x += 6;
					figurePoint.y -= 6;
				} else {
					anchor = this.toolBarController.BOTTOM_RIGHT;
					figurePoint.x -= 6;
					figurePoint.y -= 6;
				}
			}
			else {
				if (end.x < start.x) {
					anchor = this.toolBarController.TOP_LEFT;
					figurePoint.x += 6;
					figurePoint.y += 6;
				} else {
					anchor = this.toolBarController.TOP_RIGHT;
					figurePoint.x -= 6;
					figurePoint.y += 6;
				}
			}
			this.showToolBarAt(figurePoint, anchor);
		}
		else {
			// if host is both a node and a container, avoid positioning bar on header
			var setToolbarAtMousePosition = this._posMode === 1;
			if (this._posMode === 2) {
				if (eFixed.clientX && eFixed.clientY) {
					var pt = this.host.root.viewport.getSurfaceAbsoluteMousePosition(eFixed.clientX, eFixed.clientY);
					var box = this._getTargetBounds(this.host.renderMapping);
					setToolbarAtMousePosition = (pt.y - box.y) > 100;
				}
			}
			
			if (setToolbarAtMousePosition) {
				if (eFixed.clientX && eFixed.clientY) {
					var mousePt = this.host.root.viewport.getSurfaceAbsoluteMousePosition(eFixed.clientX, eFixed.clientY);
					if (this._feedbackMode) {
						this.showToolBarAt({
							x: mousePt.x+20,
							y: Math.max(mousePt.y - 43, 0)
						}, this.toolBarController.TOP_CENTER);
					} else {
						this.showToolBarAt({
							x: mousePt.x + 6,
							y: Math.max(mousePt.y - 24, 0)
						}, this.toolBarController.TOP_CENTER);
					}
				}
				else {
					if (renderMapping.policies.graphElement) {
						// choose the middle of the element
						bounds = renderMapping.policies.graphElement.getBounds();
						renderMapping.policies.graphElement.translateToAbsolute(bounds);
						
						this.showToolBarAt({
							x: bounds.x + bounds.width / 2,
							y: bounds.y + bounds.height / 2
						});
					}
					else {
						this.showToolBarAt({x: 0, y: 0});
					}
				}
			}
			else {
				
				bounds = this._getTargetBounds(this.host.renderMapping);
				var pos = {x:0, y:0};
				if (bounds) {
					var buffer = 6;
					
					if (this._verticalAlignment === "Top") {
						pos.y = bounds.y - buffer;
					}
					else if (this._verticalAlignment === "Center") {
						pos.y = bounds.y + (bounds.height / 2);
					}
					else if (this._verticalAlignment === "Bottom") {
						pos.y = bounds.y + bounds.height -1 + buffer;
					}
					
					if (this._horizontalAlignment === "Left") {
						pos.x = bounds.x - buffer;
					}
					else if (this._horizontalAlignment === "Center") {
						pos.x = bounds.x + (bounds.width / 2);
					}
					else if (this._horizontalAlignment === "Right") {
						pos.x = bounds.x + bounds.width - 1 + buffer;
					}
				}
				
				if (this._horizontalAlignment === "Left" && this._verticalAlignment === "Top") {
					anchor = this.toolBarController.BOTTOM_LEFT;
				}
				else if (this._horizontalAlignment === "Right" && this._verticalAlignment === "Top") {
					anchor = this.toolBarController.BOTTOM_RIGHT;
				}
				else if (this._horizontalAlignment === "Center" && this._verticalAlignment === "Top") {
					anchor = this.toolBarController.BOTTOM_CENTER;
				}
				else if (this._horizontalAlignment === "Left" && this._verticalAlignment === "Bottom") {
					anchor = this.toolBarController.TOP_LEFT;
				}
				else if (this._horizontalAlignment === "Right" && this._verticalAlignment === "Bottom") {
					anchor = this.toolBarController.TOP_RIGHT;
				}
				else if (this._horizontalAlignment === "Center" && this._verticalAlignment === "Bottom") {
					anchor = this.toolBarController.TOP_CENTER;
				}
				else if (this._horizontalAlignment === "Left" && this._verticalAlignment === "Center") {
					anchor = this.toolBarController.CENTER_RIGHT;
				}
				else if (this._horizontalAlignment === "Center" && this._verticalAlignment === "Center") {
					anchor = this.toolBarController.CENTER_CENTER;
				}
				else if (this._horizontalAlignment === "Right" && this._verticalAlignment === "Center") {
					anchor = this.toolBarController.CENTER_LEFT;
				}
				
				this.showToolBarAt(pos, anchor);
	
			}
		}
	},
	
	_getTargetBounds: function(renderMapping) {
		var targetBounds = null;
		var shapes = renderMapping.getShape ? [ renderMapping.getShape() ] : renderMapping.rootShapes;
		for (var i=0, shape; shape = shapes[i]; i++) {
			var dojobounds = rmpw.diagrams.render.util.Workarounds.getBoundingBox(
					shape, rmpw.diagrams.render.util.Workarounds.IGNORE_DECORATIONS);
			rmpw.diagrams.render.util.Workarounds.transformBoundingBox(shape.matrix, dojobounds);
			var bounds = new rmpw.diagrams.render.util.Rectangle(dojobounds.x, dojobounds.y, dojobounds.width, dojobounds.height);
			rmpw.diagrams.render.util.Workarounds.translateToAbsolute(bounds, shape);
			
			if (!targetBounds) {
				targetBounds = bounds.getCopy();
			}
			else {
				targetBounds.union_0(bounds);
			}
		}
		
		return targetBounds;
	},
	
	showToolBarAt: function(point, anchor) {
		this.toolBarController.showToolBarAt(this.host, point, anchor, this._elementType);
	},	
	
	removeToolBar: function(delay){
		if(this.toolBarController.isToolBarShowing()){
			this.toolBarController.removeToolBar(delay);
		}
	},	
	
	/*
	 * Add bindings
	 */
	setActionBindings: function () {
		// need to clone arguments array, because it's not a real array.  can't push to it
		for(var i = 0, binding; binding = arguments[i]; i++) {
			this._actionBindings.push(binding);
		}
	},
	
	addActionBinding: function(binding) {
		this._actionBindings.push(binding);
	},
	
	_filterOutDomainBindings: function (bindings, domainId) {
		return rmpw.diagrams.edit.actionBindings.Menu.prototype._filterOutDomainBindings(bindings, domainId);
	},
	
	removeAllBindings: function() {
		delete this._actionBindings;
		this._actionBindings = [];
	},
	
	removeDomainBindings: function (domainId) {
		this._actionBindings = this._filterOutDomainBindings(this._actionBindings, domainId);
	},
	
	removeActionBinding: function (binding) {
		var idx = dojo.indexOf(this._actionBindings, binding);
		if (idx!=-1) {
			this._actionBindings.splice(idx, 1);
		}
	},
	
	getActionBindings: function() {
		return this._actionBindings;
	},
	
	/*
	 * Public api for setting the horizontal alignment of the toolbar relative to the target shape.
	 * Can be one of the following string values: "Left", "Center", "Right".  If "Left" it will 
	 * aligned to the left side of the shape,  "Center" it will centered relative to the target shape
	 * and "Right" will be aligned to the right side of the shape.
	 * 
	 * This API is only active when targetting relations with GraphNode policy attached
	 */
	setHorizontalAlignment: function(horizontalAlignment) {
		if (horizontalAlignment === "Left" || horizontalAlignment === "Center" || horizontalAlignment === "Right") {
			this._horizontalAlignment = horizontalAlignment;
		}
		else {
			throw horizontalAlignment + " is not a valid value for horizontalAlignment";
		}
	},
	
	/*
	 * Public api for setting the horizontal alignment of the toolbar relative to the target shape.
	 * Can be one of the following string values: "Top", "Center", "Bottom".  If "Top" it will 
	 * aligned to the top side of the shape,  "Center" it will centered relative to the target shape
	 * and "Bottom" will be aligned to the bottom side of the shape.
	 * 
	 * This API is only active when targetting relations with GraphNode policy attached
	 */
	setVerticalAlignment: function(verticalAlignment) {
		if (verticalAlignment === "Top" || verticalAlignment === "Center" || verticalAlignment === "Bottom") {
			this._verticalAlignment = verticalAlignment;
		}
		else {
			throw verticalAlignment + " is not a valid value for verticalAlignment";
		}
	}
});