/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("rmpw.diagrams.edit.tools.ContextActionBarScope");

dojo.require("rmpw.common.policy.Policy");
dojo.require("rmpw.diagrams.edit.util.Utils");
dojo.require("rmpw.diagrams.edit.tools.FocusableBase");
dojo.require("rmpw.diagrams.edit.tools.Commitable");
dojo.require("rmpw.diagrams.edit.actionBindings.Menu");
dojo.require("rmpw.diagrams.edit.tools.figures.RectangularHighlight");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmpw.diagrams.edit", "BehaviorMessages");
var behaviorMessages = dojo.i18n.getLocalization("rmpw.diagrams.edit", "BehaviorMessages");        

/**
 * Class for creating a menu at a specific location based on a set of runtime actions
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.tools.ActionMenu", null, {

	constructor: function() {
		this._menu = null;
	},
	
	closeMenu: function() {
		if (this._menu) {
			dojo.disconnect(this._menuCloseOnBody);
			dijit.popup.close(this._menu);
			this._menu.destroyRecursive();
			delete this._menu;
			this._mapping.root.policies.contextActionBarScope.setShowingMenu(false);
		}
	},
	
	_createMenu: function(menu, bindings, startOrder, mapping){
		for (var i = 0, binding; binding = bindings[i]; i++) {
			if (binding.getActionBindings) {
				var relMenu = new dijit.Menu();
				menu.addChild(new dijit.PopupMenuItem({ label: binding.getLabel(), popup: relMenu }));
				this._createMenu(relMenu, binding.getActionBindings(), startOrder, mapping);
				startOrder.push(relMenu);
			}
			else {
				this._createMenuItem(menu, binding, mapping);
			}
		}	
	},
	
	_createMenuItem: function(menu, binding, mapping) {
		var menuAction = binding.getAction();
		if(menuAction) {
			var self = this;
			var menuItem = new dijit.MenuItem({
				label: binding.getLabel(), 
				iconClass: binding.getIconClass(), 
				disabled: menuAction.isEnabled ? !menuAction.isEnabled({ targetController: mapping }) : false,
				onClick: function (e) {
					var eOrig = e; 
                    e = e.touches ? e.touches[0] : e;

                    e.targetController = mapping;
					menuAction.run(e);
					self.onMenuClicked();
				}
			});
			menu.addChild(menuItem);
		}
		else {
			console.error("Action not set for menu item ", binding.getLabel());
		}
	},
	
	onMenuClicked: function() {
		this.closeMenu();
	},
	
	getMenu: function() {
		return this._menu;
	},
	
	showMenu: function(actionBindings, editController, coords) {
		dojo.require("dijit.Menu");
		if (!this._menu) {
			var startOrder = [];
			this._menu = new dijit.Menu();
			startOrder.push(this._menu);
			this._createMenu(this._menu, actionBindings, startOrder, editController);
			
			for (var j = 0; j < startOrder.length; j++) {
				startOrder[j].startup();
			}
		}

		this._mapping = editController;
		editController.root.policies.contextActionBarScope.setShowingMenu(true);
		dijit.popup.open({
			popup: this._menu,
			x: coords.x,
			y: coords.y + coords.height + 1,
			orient: 'BL',
			onExecute: dojo.hitch(this, this.closeMenu),
			onCancel: dojo.hitch(this, this.closeMenu)
		});
		this._menu.focus();	
	}
});

/**
 * toolbar window on diagram. 
 * 
 * @author Susanta Datta, Christian Vogt, Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.tools.ContextActionBarWidget", null, {
    
	group: null,
	_figure:null,
	_actions: null,
	target:null,
	_border: {color: [205,183,158, 1], width: 1},
	_highlightBorder: {color: [51,102,255, 1], width: 3},
	
	constructor: function(){
		// height and width based on action height and width
	    this.height = 18;
	    this.width = 18;
	    this._toolGap = 2;
	    this._actionMenu = new rmpw.diagrams.edit.tools.ActionMenu();
	},
	
	BOTTOM_RIGHT: 1,
	BOTTOM_LEFT: 2,
	BOTTOM_CENTER: 3,
	TOP_RIGHT: 4,
	TOP_LEFT: 5,
	TOP_CENTER: 6, 
	CENTER_CENTER: 7,
	CENTER_LEFT: 8,
	CENTER_RIGHT: 9,
	
	createShape: function (editController, actionBindings, point, anchor, subToolBar, optionalArgs){
		this._actions = [] ;
		this.target = editController.getDefaultTargetShape();
		
		this.setActions(editController, this.target, actionBindings, subToolBar, optionalArgs);
		var bounds = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(
				this._figure, rmpw.diagrams.render.util.Workarounds.IGNORE_DECORATIONS);
		bounds.x = point.x;
		bounds.y = point.y;
		if (anchor) {
			var buffer = 0;
			if (anchor === this.BOTTOM_RIGHT) {
				bounds.x -= (buffer + bounds.width);
				bounds.y -= (buffer + bounds.height);
			}
			else if (anchor === this.BOTTOM_LEFT) {
				bounds.x += buffer;
				bounds.y -= (buffer + bounds.height);
			}
			else if (anchor === this.BOTTOM_CENTER) {
				bounds.x -= bounds.width / 2;
				bounds.y -= (buffer + bounds.height);
			}
			else if (anchor === this.TOP_RIGHT) {
				bounds.x -= (buffer + bounds.width);
				bounds.y += buffer;
			}
			else if (anchor === this.TOP_LEFT) {
				bounds.x += buffer;
				bounds.y += buffer;
			}
			else if (anchor === this.TOP_CENTER) {
				bounds.x -= bounds.width / 2;
				bounds.y += buffer;
			}
			else if (anchor === this.CENTER_CENTER) {
				bounds.x -= bounds.width / 2;
				bounds.y -= bounds.height / 2;
			}
			else if (anchor === this.CENTER_LEFT) {
				bounds.x += buffer;
				bounds.y -= bounds.height / 2;
			}
			else if (anchor === this.CENTER_RIGHT) {
				bounds.x -= (buffer + bounds.width);
				bounds.y -= bounds.height / 2;
			}
		}
		
		// ensure toolbar is displayed with-in the canvas area
		var d = editController.root.getCanvasSize();
		var position = this.constrainToRect(new rmpw.diagrams.render.util.Rectangle(0, 0, d.w, d.h), bounds);		
		this.group.setTransform({dx: position.x, dy: position.y});
	},
	
	
	/**
	 * <p>
	 * Constrains some 2D coordinates so they're within some 2D rectangular bounds.
	 * </p>
	 * @param bounds A rmpw.diagrams.render.util.Rectangle representing the boundary to remain within.
	 * @param coords A rmpw.diagrams.render.util.Rectangle to be constrained.
	 * @return A constrained point.
	 */
	constrainToRect: function(constraintBounds, objectBounds){	
		// The 2d point to return;
		var point = {
				x: objectBounds.x,
				y: objectBounds.y
		};
				
		// check X values
		if(objectBounds.x < constraintBounds.x){
			point.x = constraintBounds.x; 
		}
		
		if(objectBounds.x + objectBounds.width > constraintBounds.width){
			point.x = constraintBounds.width - objectBounds.width;
		}
		
		// Check y values
		if(objectBounds.y < constraintBounds.y){
			point.y = constraintBounds.y;
		}
		
		if(objectBounds.y + objectBounds.height > constraintBounds.height){
			point.y = constraintBounds.height - objectBounds.height;
		}
		
		return point;
	},
	
	removeShape: function (){
		this.destroy();
	},
	isShowing : function(){
		return this._figure? true : false;
	},
	
	isMenuShowing : function(){
		return this._actionMenu._menu? true : false;
	},
	
	destroy: function(e) {
		this.closeMenu();
		
		if (this._actions){
			dojo.forEach(this._actions, function (shape){
				shape.destroy(true);
			});
			delete this._actions;
		}
		if(this.group){
			this.group.removeShape();
			delete this._figure;
			delete this.group;	
		}
		if(this.xGroup){
			this.xGroup.removeShape();
			delete this.xGroup;	
		}
		
		this.destroySubToolBar();
		
		dojo.unsubscribe(this._subscribeEscape);
		if (e) dojo.stopEvent(e);
	},
	
	closeMenu: function() {
		if (this._connectMenu) {
			dojo.disconnect(this._connectMenu);
		}
		this._actionMenu.closeMenu();
	},
	
	destroySubToolBar: function() {
		if (this._subToolBar) {
			this._subToolBar.destroy();
			delete this._subToolBar;
		}
	},
	
	getSubToolBar: function() {
		if (this._subToolBar) {
			return this._subToolBar;
		}
	},
	
	shouldHandleEvent: function(viewer, e) {
		if (!this._figure) {
			return false;
		}
		
		if (this._subToolBar) {
			if (this._subToolBar.shouldHandleEvent(viewer, e)) {
				return true;
			}	
		}
		
	    var e = e.changedTouches ? e.changedTouches[0] : e; 
		var mousePt = viewer.viewport.getSurfaceAbsoluteMousePosition(e.clientX, e.clientY);
		var bounds = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(
				this._figure, rmpw.diagrams.render.util.Workarounds.IGNORE_DECORATIONS);
		return (mousePt.x > bounds.x && mousePt.y > bounds.y &&
				mousePt.x < bounds.x+bounds.width && mousePt.y < bounds.y+bounds.height);
	},
	
	/*
	 * If there are more then 1 set of actions, and the subActions are
	 * more then 1 and if the subactions have an icon and description then
	 * we should create a dropdown toolbar.
	 */
	shouldCreateDropDownToolbar: function(actions, subActions) {
		if (subActions.length > 1 && actions.length > 1 &&
				subActions.description && subActions.iconURL) {
			return true;
		}
		
		return false;
	},
	
	setActions: function (editController, target, actionBindings, subToolBar, optionalArgs){
		var size = actionBindings.length + 1,
		  toolbarAction,
		  figurePos; // store one for the menu drop-down
			
		// figure out the number of rows and columns
		var columns = size;
		var rows = 1;
		if (subToolBar) {
			if (size > 3) {
				columns = Math.ceil(Math.sqrt(size));
				rows = Math.ceil(size / columns);
			}
			else {
				// align sub toolbar vertically
				columns = 1;
				rows = size;
			}
		}
		
		this.group = editController.renderMapping.root.feedbackLayer.createGroup();
		for (var i=0, binding; binding = actionBindings[i]; i++) {
			
			var dropDown = binding.getActionBindings !== undefined;
			figurePos = this._getNextPosition(columns, rows);
			
			if (dropDown) {
				var parentToolBar = this;				
				var dropDownAction = { 

					_actionBindings: binding.getActionBindings(),
					_figurePos: figurePos,
					
					run: function() {
						if (this._toolBar) {
							this._toolBar.destroy();
							delete this._toolBar;
						}
						else {
							this._toolBar = new rmpw.diagrams.edit.tools.ContextActionBarWidget();
							
							var bounds = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(
									parentToolBar._figure, rmpw.diagrams.render.util.Workarounds.IGNORE_DECORATIONS);
							this._toolBar.createShape(editController, this._actionBindings, 
									{x: bounds.x + this._figurePos.x, 
									 y: bounds.y + this._figurePos.y + this._toolBar.height}, 
									 this._toolBar.TOP_LEFT, true); // subtoolbar
							parentToolBar.destroySubToolBar();
							parentToolBar.closeMenu();
							parentToolBar._subToolBar = this._toolBar;
							parentToolBar._subToolBar.__owningAction = toolbarAction;
						}
					}
                };
				
				binding.setAction(dropDownAction);
			}
			
			toolbarAction = new rmpw.diagrams.edit.tools.ActionButton({
				parentShape: this.group,
				editController: editController,
				target: target,
				config: binding,
				dropDown: dropDown,
				optionalArgs: optionalArgs
			});
			toolbarAction.shape.setTransform({dx: figurePos.x, dy: figurePos.y});
			this._addChild(toolbarAction);
			
			if (!dropDown) {
				dojo.connect(toolbarAction, "_onClick", this, "destroy");
			}
		}
		
		var that = this;
		var menuBinding = new rmpw.diagrams.edit.actionBindings.Menu();
		menuBinding.setIconURL(dojo.moduleUrl("rmpw.diagrams.edit", "icons/view_menu.gif"));
		menuBinding.setLabel(behaviorMessages.open);
		menuBinding._dropDownActionBindings = actionBindings;
		var menuAction = { 
				run: function(args) {
					if (that._actionMenu.getMenu()) {
						that._actionMenu.closeMenu();
					}
					else {
						that.destroySubToolBar();
						that._connectMenu = dojo.connect(that._actionMenu, "onMenuClicked", that, "destroy");
						that._actionMenu.showMenu(menuBinding.getActionBindings(), editController, args.menuPos);
					}
				}
			};
		menuBinding.setAction(menuAction);
		
		this.openAction = new rmpw.diagrams.edit.tools.ActionButton({
			parentShape: this.group,
			editController: editController,
			target: target,
			config: menuBinding,
			optionalArgs: optionalArgs
		});
		
		this._actionMenu.__owningAction = this;
		
		figurePos = this._getNextPosition(columns, rows);
		this.openAction.shape.setTransform({dx: figurePos.x, dy: figurePos.y});
		this._addChild(this.openAction);
		
		var toolBarBounds = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(
				this.group, rmpw.diagrams.render.util.Workarounds.IGNORE_DECORATIONS);
		var figHeight = toolBarBounds.height + this._toolGap * 2;
		this._figure = this.group.createRect({
			width: toolBarBounds.width + this._toolGap * 2, 
			height: figHeight,
			r: 2
		}).setStroke(this._border);
		this._figure.setFill( {
			type: "linear", x1: 0, y1: 0, x2: 0, y2: figHeight,
			colors: [
				{ offset: 0, color: "#FFFFFF" }, { offset: 1, color: "#D9D9D9" }
			]
		});
		this._figure.moveToBack();
		
		// set first button in toolbar with focus
		if (this._actions.length > 0 && editController.root.policies.focusScope) {
			editController.root.policies.focusScope.setFocus(this._actions[0]);
		}
		
		// add feedback to show where shape will be created
		if (editController.renderMapping.policies.graphContainer) {
			var pt = editController.root.policies.contextActionBarScope.createAt;
			if (pt) {
				this.xGroup = editController.renderMapping.root.feedbackLayer.createGroup();
				this._createAtLine(pt.x-4, pt.y, pt.x+4, pt.y);
				this._createAtLine(pt.x, pt.y-4, pt.x, pt.y+4);
			}
		}
	},
	
	_createAtLine: function(x1, y1, x2, y2) {
		var line = this.xGroup.createLine({x1 : x1, y1 : y1, x2 : x2, y2 : y2});	
		line.setStroke({color: 'black', lineWidth: 1, lineStyle: "solid"});
		line.rawNode.setAttribute("pointer-events", "none");
		line.rawNode.style.shapeRendering = "crispEdges";
	},

	_addChild: function(child, index) {
		if (!index || index < 0 || index > this._actions.length) {
			index = this._actions.length;
			this._actions.push(child);
		} else {
			this._actions.splice(index, 0, child);
		}
	},
	
	_getNextPosition: function(columns, rows){
		var x = this._toolGap + this._border.width;
		var y = x;
		
		var currentColumn = this._actions.length % columns;
		var currentRow = Math.floor(this._actions.length / columns);
		
		var currentRowLength = 0;
		if (this._actions.length > 0) {
			var actionIndex = currentRow*columns;
			for (var i=actionIndex; i<actionIndex + currentColumn; i++) {
				currentRowLength += this._actions[i].width + this._toolGap;
			}
		}
		x += currentRowLength;
		y += (this.height + this._toolGap) * currentRow;
		
		return {x: x, y: y};
	}
});

dojo.declare("rmpw.diagrams.edit.tools.ButtonCommitable", rmpw.diagrams.edit.tools.Commitable, {
	onCancel: function() {
		if (this.host._actionClass._toolBar) {
			this.host._actionClass._toolBar.destroy();
			delete this.host._actionClass._toolBar;
		}
	},
	
	onCommit: function() {
		this.host._onClick();
		return true;
	}
});

dojo.declare("rmpw.diagrams.edit.tools.ButtonHighlight", rmpw.diagrams.edit.tools.figures.Highlight, {
	constructor: function(feedbackLayer, target, editController) {
		this.targetController = editController;	
		this.feedbackLayer = feedbackLayer;
		this.shape = this.createShape(target);
	},
	
	createShape: function(target) {
		var dojobounds = rmpw.diagrams.render.util.Workarounds.getBoundingBox(
				target, rmpw.diagrams.render.util.Workarounds.IGNORE_DECORATIONS);
		rmpw.diagrams.render.util.Workarounds.transformBoundingBox(target.matrix, dojobounds);
		var bounds = new rmpw.diagrams.render.util.Rectangle(dojobounds.x, dojobounds.y, dojobounds.width, dojobounds.height);
		
		var dim = new rmpw.diagrams.render.util.Dimension(this.padding, 0);
		var shape = this.feedbackLayer.createRect({
			width: bounds.width + (2 * dim.width),
			height: bounds.height + (2 * dim.width),
			r: dim.width});
		shape.setFill(this.colorStart);
		shape.setTransform({
			dx: bounds.x - dim.width, 
			dy: bounds.y - dim.width});
		
		shape.rawNode.setAttribute("pointer-events", "none");
		return shape;
	}
});
		
dojo.declare("rmpw.diagrams.edit.tools.ActionButton", rmpw.common.policy.Host, {
	
	constructor: function(args) {
		var config = args.config;
	    this._actionClass = config.getAction();
		this._iconUrl = config.getIconURL();
		this._text = config.getLabel();
		
		this.editController = args.editController;
		this.renderMapping = this.editController.renderMapping;
		this.root = this.editController.root;
		
		this.optionalArgs = args.optionalArgs;
		this.width = 18;
		this.height = 18;
		this.parentShape = args.parentShape;
		this.shape = args.parentShape.createGroup();
		this._iconConfig = {x: 0, y: 0, width:16, height:16, src: this._iconUrl};
		this._dropDown = args.dropDown;
		
		this.icon = this.shape.createImage(this._iconConfig);
		if (dojox.gfx.svg) {
            this.icon.rawNode.setAttribute("fill", null);
	    }
	    
	    this.shape.rawNode.setAttribute("title", this._text ? this._text : '');
	    
		var dm = net.jazz.ajax._contextRoot;
	    if (this._dropDown) {
            var dropDownConfig = {
                x: this._iconConfig.width, 
                y: 0, 
                width: 9, 
                height: 16, 
                src: dm+"/web/rmpw.diagrams.edit/icons/dropdown.png"
            };
            var dropDownIcon = this.shape.createImage(dropDownConfig);
            if (dojox.gfx.svg) {
                dropDownIcon.rawNode.setAttribute("fill", null);
		    }
            this.width += 9;
	    }
	    
		if (dojox.gfx.vml) {
			this.shape.rawNode.title = this._text ? this._text : '';
		} else {
			this.shape.rawNode.setAttribute("title", this._text ? this._text : '');
		}
		
		this._enabled = true;
		if (this._actionClass) {
			this._actionClass.__toolBarAction = this;
			 if (this._actionClass.isEnabled) {
					if (this._actionClass.isEnabled({ targetController: this.editController })) {
						this._enabled = true;
					}
					else {
						this._enabled = false;
					}
			  }
		}
		
		if (this._enabled) {
			this._mouseListener = this.shape.connect("onclick", this, "_onClick");
			this._touchListener = this.shape.connect("touchstart", this, "_onClick");
		}
	   
	    this._updateEnablement();
	    
	    this.createPolicies();
	},
	
	createPolicies: function() {
		this.inherited(arguments);
		
		var focusEffect = this.createPolicy(rmpw.diagrams.edit.tools.FocusableBase);
		focusEffect.setTargetShape(this.shape);
		focusEffect.setFocusParent(this.parentShape);
		focusEffect.setFeedbackLayer(this.parentShape);
		focusEffect.setFeedbackFigureType(rmpw.diagrams.edit.tools.ButtonHighlight);
		
		this.createPolicy(rmpw.diagrams.edit.tools.ButtonCommitable);
		
		// ensure handle scope policy is installed on parent
		if (this.editController) {
			if (!this.editController.policies.handleScope) {
				this.editController.createPolicy(rmpw.diagrams.edit.tools.HandleScope);
			}
			this.editController.policies.handleScope.registerHandle(this);
		}
		
		// reference handle scope locally
		this.policies.handleScope = this.editController.policies.handleScope;
	},

	
	_onClick: function(e) {
		if(this._actionClass) {
            if (this._actionClass.run) {
                // TODO: inject as getMenuPos() function instead that only calculates coordinates if called
                var menuPos = rmpw.diagrams.edit.util.Utils.getGlobalCoords(this.icon);
                var args = e ? e : {pageX: menuPos.x, pageY: menuPos.y};
                args.menuPos = menuPos;

                args.targetController = this.editController;
                dojo.mixin(args, this.optionalArgs);
                this._actionClass.run(args);
        		dojo.stopEvent(e);
            }
		} else {
			console.error("Action not set for button ", this._text);
		}
    },
	
	destroy: function() {
        this.inherited(arguments);

        if (this.editController) {
			this.editController.policies.handleScope.unregisterHandle(this);
		}

        this.shape.removeShape(true);
		dojo.disconnect(this._mouseListener);
		dojo.disconnect(this._touchListener);
		delete this._mouseListener;
		delete this._touchListener;
		dojo.unsubscribe(this._subscribeEnter);
		delete this._subscribeEnter;
	},
	
	_updateEnablement: function() {
		if (this._enabled) {
			this._setShapeOpacity(1);
			this._setCursor("pointer");
		} else {
			this._setCursor("not-allowed");
			this._setShapeOpacity(0.3);
		}
	},
	
	_setCursor: function(cursor) {
		if (cursor){
		    this.shape.rawNode.style.cursor = cursor;
		}else{
		    this.shape.rawNode.style.cursor = '';
		}
	},
	
	_setShapeOpacity: function(opacity) {
		if (dojox.gfx.vml) {			
		// TODO: solve the feedback issues on IE: this.shape.rawNode.style.filter = "alpha(opacity=" + (opacity * 100) + ")";
		} else {
			this.shape.rawNode.setAttribute("opacity", opacity);
		}
	}	
});

/**
 * toolbar controller 
 * 
 * @author Susanta Datta
 */
dojo.declare("rmpw.diagrams.edit.tools.ContextActionBarScope", rmpw.common.policy.Policy, {
	singleton: "contextActionBarScope",
	
	_nodeActions: null,
	_toolBar: null,
	moving:false,
	createdBy: null,
	
	BOTTOM_RIGHT: rmpw.diagrams.edit.tools.ContextActionBarWidget.prototype.BOTTOM_RIGHT,
	BOTTOM_LEFT: rmpw.diagrams.edit.tools.ContextActionBarWidget.prototype.BOTTOM_LEFT,
	BOTTOM_CENTER: rmpw.diagrams.edit.tools.ContextActionBarWidget.prototype.BOTTOM_CENTER,
	TOP_RIGHT: rmpw.diagrams.edit.tools.ContextActionBarWidget.prototype.TOP_RIGHT,
	TOP_LEFT: rmpw.diagrams.edit.tools.ContextActionBarWidget.prototype.TOP_LEFT,
	TOP_CENTER: rmpw.diagrams.edit.tools.ContextActionBarWidget.prototype.TOP_CENTER, 
	CENTER_CENTER: rmpw.diagrams.edit.tools.ContextActionBarWidget.prototype.CENTER_CENTER,
	CENTER_RIGHT: rmpw.diagrams.edit.tools.ContextActionBarWidget.prototype.CENTER_RIGHT,
	CENTER_LEFT: rmpw.diagrams.edit.tools.ContextActionBarWidget.prototype.CENTER_LEFT,
	
	constructor: function() {
		this._nodeActions = [];
		this.diagramViewer = this.host;
		this._subscribeMoveStart = dojo.subscribe("/gfx/move/start", this, this._moveStart);
		this._subscribeMoveStop = dojo.subscribe("/gfx/move/stop", this, this._moveEnd);
		this._subscribeTabChange = dojo.subscribe("/TabbedEditor/onTabChange", this, "removeToolBarImmediate");
		this._zoomConnect = dojo.connect(this.host.root.viewport, "onViewportChanged", this, "removeToolBarImmediate");
	},
	
	destroy: function() {
		dojo.unsubscribe(this._subscribeMoveStart);
		dojo.unsubscribe(this._subscribeMoveStop);
		dojo.disconnect(this._zoomConnect);
		dojo.disconnect(this._scrollConnect);
	},
	
	getCurrentDomain: function() {
		if (this.host.root.policies.actionBarScope) {
			return this.host.root.policies.actionBarScope.getDomainId();
		}
		
		return undefined;
	},
	
	getToolBar: function(){
		return this._toolBar;
	},
	
	setShowingMenu: function(showing) {
		this._showingMenu = showing;
	},
	
	disableToolbar: function(disable) {
		this._disableToolbar = disable;
		this.removeToolBarImmediate();
	},
	
	getDomainFilteredActionBindings: function (bindings, all, top) {
		// filter based on the current domain
		if (this.getCurrentDomain()) {
			var domainBindings = {};
			var filteredBindings = [];
			for (var i=0, lg=bindings.length; i<lg; i++) {
				var binding = bindings[i];
				if (binding.getActionBindings) {
					var subBindings = this.getDomainFilteredActionBindings(binding.getActionBindings(), all, false);
					if (all) {
						for (var j=0, lg2=subBindings.length; j<lg2; j++) {
							var domainId = subBindings[j].getDomainId();
							if (!domainBindings[domainId]) {
								domainBindings[domainId] = [];
							}
							domainBindings[domainId].push(subBindings[j]);
						}
					}
					else if (subBindings.length > 0) {
						var menuBinding = new rmpw.diagrams.edit.actionBindings.Menu();
						menuBinding.setIconURL(binding.getIconURL());
						menuBinding.setLabel(binding.getLabel());
						menuBinding._dropDownActionBindings = subBindings;
						filteredBindings.push(menuBinding);
					}
				}
				else {
					domainId = binding.getDomainId();
					
					// if binding domainId isn't defined, then assume it should be added
					if (!domainId) {
						filteredBindings.push(binding);
					}
					else {
						if (all) {
							var domainId = binding.getDomainId();
							if (!domainBindings[domainId]) {
								domainBindings[domainId] = [];
							}
							domainBindings[domainId].push(binding);
						}
						else if ((binding.getDomainId() == this.getCurrentDomain())) {
							filteredBindings.push(binding);
						}
					}
				}
			}
			
			if (all) {
				// collection domain groups into separate menus
				for ( var domainId in domainBindings) {
					if ((domainBindings[domainId].length > 1) && top) {
						var menuBinding = new rmpw.diagrams.edit.actionBindings.Menu();
						menuBinding.setIconURL(domainBindings[domainId][0].getIconURL());
						this.host.root.policies.actionBarScope.setDomainLabel(domainId, menuBinding);
						menuBinding._dropDownActionBindings = domainBindings[domainId];
						filteredBindings.push(menuBinding);
					}
					else {
						for (var j=0, lg2=domainBindings[domainId].length; j<lg2; j++) {
							filteredBindings.push(domainBindings[domainId][j]);
						}
					}
				}
			}
			return filteredBindings;
		}
		
		return bindings;
	},
	
	getActionBindings: function(editController) {
		var bindings = editController.policies.contextActionBar.getActionBindings();
		if (editController.renderMapping.policies.graphContainer &&
			editController.renderMapping.policies.graphContainer.isTopLevel()) {
			return this.getDomainFilteredActionBindings(bindings, this.getCurrentDomain() == "http://all", true);
		}
		
		return bindings;
	},
	
	showToolBarAt: function(editController, point, anchor, elementType) {
		this.cancelRemoveToolBar();
		
		if (editController.policies.contextActionBar) {
			if(!this.moving) {
				this.showToolBarImmediateAt(editController, this.getActionBindings(editController), point, anchor);
			}
			return true;
		}
		return false;
	},
	
	isToolBarShowing: function() {
        return (this._toolBar) ?  this._toolBar.isShowing() : false;
	},
	
	cancelRemoveToolBar: function() {
		if(this._removeTimeOutHandle) {
			clearTimeout(this._removeTimeOutHandle);
			this._removeTimeOutHandle = null;
		}
	},
	
	removeToolBarImmediate: function() {
		if (!this._toolBar) {
			return;
		}
		if(this._toolBar.isShowing()){
			this._toolBar.removeShape();
		}
		if (this._mappingFocusConnect) {
			dojo.disconnect(this._mappingFocusConnect);
			this._mappingFocusConnect = null;
		}
		if (this._mappingDiagramFocusConnect) {
			dojo.disconnect(this._mappingDiagramFocusConnect);
			this._mappingDiagramFocusConnect = null;
		}
		if (this._toolBarFocusConnect) {
			dojo.disconnect(this._toolBarFocusConnect);
			this._toolBarFocusConnect = null;
		}
		this.cancelRemoveToolBar();
		this.createdBy = null;
		this.optionalArgs = null;
		delete this._mousemovedelay;
	},
	
	removeToolBar: function(delay){
		if(this._showTimeOutHandle) {
			clearTimeout(this._showTimeOutHandle);
			this._showTimeOutHandle = null;
		}
		
		if(this._removeTimeOutHandle) {
			// removal already in process
			return;
		}
		
		if(delay && this._toolBar) { 
			this._removeTimeOutHandle = setTimeout(
					dojo.hitch(this, this.removeToolBarImmediate),
					delay); //there is a delay
		}
		else {
			this.removeToolBarImmediate();
		}
	},
	
	/**
	 * 
	 * @param editController
	 * @param actionBindings
	 * @param point
	 * @param anchor
	 * @param optionalArgs will get mixed in with event fired on click
	 * @returns
	 */
	showToolBarImmediateAt: function(editController, actionBindings, point, anchor, optionalArgs) {
		if(this.moving || this._disableToolbar) {
			return;
		}
		if(this._toolBar && this._toolBar.isShowing()) {
			this.removeToolBarImmediate();
		}
		this.createdBy = editController;

		this._toolBar = new rmpw.diagrams.edit.tools.ContextActionBarWidget();
		this._toolBar.createShape(editController, actionBindings, point, anchor, null, optionalArgs);
		
		this._mappingFocusConnect = dojo.connect(editController.root.policies.focusScope, "setFocus", this, "_onFocusChanged");
		
		if (editController.renderMapping.policies.graphContainer && 
				editController.renderMapping.policies.graphContainer.isTopLevel() &&
				editController.policies.focusable) {
			this._mappingDiagramFocusConnect = editController.policies.focusable.figure.connect("onmouseleave", this, "_onMouseLeave");
			this.removeToolBar(5000);
		}
		
		// keep toolbar open if mouse is on it
		this._toolBarFocusConnect = dojo.connect(document, "onmousemove", this, "_onMouseMove");
	},
	
	_onFocusChanged: function(editController) {
		// don't handle focus change while menu is showing
		if (this._showingMenu || !editController || this.createdBy === null) {
			return;
		}
		
		// if the editController is currently selected or cannot be selected don't do anything
		var masterEC = editController.editController ? editController.editController : editController;
		if (!editController.policies.selectable || editController.policies.selectable.isSelected() &&
			(masterEC && (this.createdBy) && (this.createdBy === masterEC))) {
			if (this._mousemovedelay) {
				delete this._mousemovedelay;
				this.cancelRemoveToolBar();
			}
		}
		else {
			// else remove any showing action bar
			this.removeToolBar(1000);
		}
	},
	
	_onMouseMove: function(e) {
		if(this._toolBar && (this._toolBar.shouldHandleEvent(this.diagramViewer.viewer, e) || this._toolBar.isMenuShowing())) {
			// if focus has left figure but is on tool bar or menu is open, cancel any remove
			if(this._removeTimeOutHandle) {
				this.cancelRemoveToolBar();
				this._delayedRemove = true;
			}
		} else if (this._delayedRemove) {
			this.removeToolBar(1000);			
			delete this._delayedRemove;
			// if user has moved mouse back to figure, cancel this remove
			this._mousemovedelay = true;
		}
	},
	
	_onMouseLeave: function(e) {
        var pt = this.host.viewer.viewport.getSurfaceAbsoluteMousePosition(e.clientX, e.clientY);
        var sz = dojo.position(this.host.container);
        if (pt.x<0 || pt.y<0 || pt.x>=sz.w-5 || pt.y>=sz.h-5) {
			this.removeToolBarImmediate();
        }
	},
	
	_getElement: function(element){
		for(var i = 0; i < this._nodeActions.length; i++){
			if(this._nodeActions[i].element === element){
			    return this._nodeActions[i];
			}
		}
		return null;
	},
	
	_moveStart: function(e){
		this.moving = true;
		this.removeToolBarImmediate();
	},
	
	_moveEnd: function(e){
		this.moving = false;
	},
	
	setCreationPoint: function(pt){
		this.createAt = pt;
	},
	
	getCreationPoint: function(){
		return this.createAt;
	},
	
	clickedOnToolbar: function(e){
		if(!this._toolBar) {
			return false;
		}
		
		return this._toolBar.shouldHandleEvent(this.diagramViewer.viewer, e);
	}
	
});

}());