/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.Focusable");

dojo.require("rmpw.diagrams.edit.tools.FocusableBase");
dojo.require("rmpw.diagrams.edit.tools.figures.PolylineHighlight");
dojo.require("rmpw.diagrams.edit.tools.figures.RectangularSelection");
dojo.require("rmpw.diagrams.render.util.ColorConstants");

/**
 * A class which, when wrapped on a BehaviorController, endows its figure with focus behavior.  It will create a feedback
 * highlight and register the BehaviorController as focused with the FocusController.
 * 
 * @see FocusController for where central focus is stored.
 * 
 * @author Marc Gobeil / Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.tools.Focusable",
		rmpw.diagrams.edit.tools.FocusableBase, {
    
	constructor: function() {
		this.feedbackFigureType = rmpw.diagrams.edit.tools.figures.PolylineHighlight;
		
		var renderMapping = this.host.renderMapping;
		if (renderMapping.policies.graphElement) {
			this.setTargetShape( this.host.renderMapping.getShape() );
		} else {
			this.feedbackLayer = this.getFrontFeedbackLayer(); // use front feedback for non graph elements
		}
			
		// if edge or edge label always use diagram feedback layer
		if (renderMapping.policies.graphEdge || 
				(this.host.parent.renderMapping && this.host.parent.renderMapping.policies.graphEdge)) {
			this.feedbackLayer = renderMapping.viewer.diagram.policies.graphContainer.getRearFeedbackLayer();		
		}		
		
		if (this.host.root === this.host.parent) {
				this._connectDefaultFocus = dojo.connect(this.host.root.policies.focusScope, "onDefaultFocus", dojo.hitch(this, function() {
					if (!this.getFocus()) {
						this.host.root.policies.focusScope.setFocus(this.host);
					}
				}));
		}
	},
	
	destroy: function() {
		this.inherited(arguments);
		
		if (this._connectDefaultFocus) {
			dojo.disconnect(this._connectDefaultFocus);
		}
	},
	
	_createFeedback: function(mode,strokeWidth) {
	    var COLOR = rmpw.diagrams.render.util.ColorConstants;
		// diagram colors	
		if (this.host.renderMapping.policies && this.host.renderMapping.policies.graphContainer &&
				this.host.renderMapping.policies.graphContainer.isTopLevel()) {
			// this is diagram, its focus feedback should be less pronounced than that of shapes and edges
			var beg = (!mode || mode=="highlight") ?  [247, 237, 215, 1] : [0, 138, 191, 1];
			var end = (!mode || mode=="highlight") ?  [247, 237, 215, 0] : [0,0,0,0];
			return new rmpw.diagrams.edit.tools.figures.PolylineHighlight(this.feedbackLayer, this.figure, this.host, beg, end);			
		}else if(this.host.renderMapping.policies.graphEdge){
            // regular node colors
            return new rmpw.diagrams.edit.tools.figures.PolylineHighlight(this.feedbackLayer, 
                this.figure, this.host, COLOR.HOVER.toRgba(), COLOR.HOVER.toRgb().concat(0));	
		}else{
		    return new rmpw.diagrams.edit.tools.figures.RectangularSelection(this.getFrontFeedbackLayer(), 
		    		this.figure, this.host, (mode?mode:"highlight"), strokeWidth);
		}
	}
});