/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.FocusableBase");

dojo.require("rmpw.diagrams.edit.tools.Feedback");

/**
 * A class which, when wrapped on a BehaviorController, endows its figure with focus behavior.  It will create a feedback
 * highlight and register the BehaviorController as focused with the FocusController.
 * 
 * @see FocusController for where central focus is stored.
 * 
 * @author Marc Gobeil / Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.tools.FocusableBase",
	rmpw.diagrams.edit.tools.Feedback, {
    
	singleton: "focusable",
	
	constructor: function() {
		this.feedbackLayer = this.getRearFeedbackLayer();
		this._graphManager = this.host.renderMapping.policies.graphElement ? 
				this.host.renderMapping.policies.graphElement.getGraphManager() : null;
	},
	
	destroy: function() {
		this.inherited(arguments);
		
		if (this._focused) {
			this.host.root.policies.focusScope.resetFocus();
		}
	},
	
	/*
	 * Sets the class type that will be instantiated as the feedback figure
	 */
	setFeedbackFigureType: function(feedbackFigureType) {
		this.feedbackFigureType = feedbackFigureType;
	},
	
	setFocus: function(focus) {
		this._focused = focus;
		
		if (focus) {
			this.hideFeedback();
			this.showFeedback();
		}
		else {
			this.hideFeedback();
		}
	},
	
	getFocus: function() {
		return this._focused || false;
	},
	
	setFocusParent: function(focusParent) {
		this._focusParent = focusParent;
	},
	
	suspendable: function() {
		// only diagram focus is suspended
		return this.host.renderMapping.policies.graphContainer && 
			this.host.renderMapping.policies.graphContainer.isTopLevel();			
	},
	
	_addListeners: function() {
		this.inherited(arguments);
		
		this.listeners.push(this.figure.connect("onmouseover", this, "_onMouseEnter"));
		this.listeners.push(this.figure.connect("onmouseout", this, "_onMouseLeave"));
		if (this.host.policies.selectable) {
			this.listeners.push(dojo.connect(this.host.policies.selectable, "onSelectionChanged", dojo.hitch(this, function(selected, e) {
				// when selected - force it into focus as well initially
				if (selected) {
					this.host.root.policies.focusScope.setFocus(this.host, true,  (e && !!e.touches));
				}
			})));
		}
	},
		
	_onMouseEnter: function(e) {
		this.host.root.policies.focusScope.setFocus(this.host);
		if (this._focusParent && this.host.parent && this.host.parent.policies.focusable) {
			this.host.parent.policies.focusable.setFocus(true);
		}
		dojo.stopEvent(e);
	},
	
	_onMouseLeave: function(e) {
		this.host.root.policies.focusScope.resetFocus();
		if (this._focusParent && this.host.parent && this.host.parent.policies.focusable) {
			this.host.parent.policies.focusable.setFocus(false);
		}
		
		// if mouse is now in a container shape, focus that
		// NOTE: if a shape is within a shape, and the user moves the mouse from the diagram to the outer shape 
		// to the inner shape back to the outer shape, the outer shape does not get a new mouse enter event
		// because as far as dojo is concerned it never left the outer shape when it went into the inner shape
		// -- we could monitor this progression but it's more robust to just see if the mouse is now in that container
		if(e.clientX && e.clientY && this._graphManager){
			var pt = this.host.root.viewport.getSurfaceAbsoluteMousePosition(e.clientX, e.clientY);
			var container = this._graphManager.getContainerFromPoint(pt, this.host.renderMapping.id);
			if (container) {
				this.host.root.policies.focusScope.setFocus(container.__editController);
			}
		}
		
		dojo.stopEvent(e);
	}	
});