/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.HandleScope");

dojo.require("rmpw.common.policy.Policy");

/**
 * Keeps track of accessible handles for a given edit controller
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.tools.HandleScope", rmpw.common.policy.Policy, {
	singleton: "handleScope",
	
	constructor: function() {
		this._handles = [];
		this.registerHandle(this.host);
	},
	
	destroy: function() {
		this.inherited(arguments);
		
		if (this._connectSelectionChanged) {
			dojo.disconnect(this._connectSelectionChanged);
		}
	},
	
	registerHandle: function(handle) {
		this._handles.push(handle);
	},
	
	unregisterHandle: function(handle) {
		var index = dojo.indexOf(this._handles, handle);
		var lg = this._handles.length;
		
		this._handles.splice(index, 1);
	},
	
	/*
	 * retrieve the next handle in the scope
	 */
	getNextHandle: function() {
		var lg = this._handles.length;
		
		// find the currently focused handle
		var currentHandle = 0;
		for (var i=0, handle; handle=this._handles[i]; i++) {
			if (handle.policies && handle.policies.focusable && handle.policies.focusable.getFocus()) {
				currentHandle = i;
				break;
			}
		}
		
		var nextHandle = currentHandle+1;
		nextHandle %= lg;
		handle = this._handles[nextHandle];
		
		return handle;
	}
	
});