/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.diagrams.edit.tools.LabelLinkable");


dojo.require("rmpw.forms.ui.EditOrCreateResource");

var modelsFrontService = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "models");			

/**
 * Policy that allows a Label to be act link a url link.
 * 
 * @author John Swanke
 */
dojo.declare("rmpw.diagrams.edit.tools.LabelLinkable", rmpw.common.policy.Policy, {
	
	urlRegExp: "((https?|ftps?|file)\\://|\./|/|)(/[a-zA-Z]{1,1}:/|)(((?:(?:[\\da-zA-Z](?:[-\\da-zA-Z]{0,61}[\\da-zA-Z])?)\\.)*(?:[a-zA-Z](?:[-\\da-zA-Z]{0,80}[\\da-zA-Z])?)\\.?)|(((\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])\\.){3}(\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])|(0[xX]0*[\\da-fA-F]?[\\da-fA-F]\\.){3}0[xX]0*[\\da-fA-F]?[\\da-fA-F]|(0+[0-3][0-7][0-7]\\.){3}0+[0-3][0-7][0-7]|(0|[1-9]\\d{0,8}|[1-3]\\d{9}|4[01]\\d{8}|42[0-8]\\d{7}|429[0-3]\\d{6}|4294[0-8]\\d{5}|42949[0-5]\\d{4}|429496[0-6]\\d{3}|4294967[01]\\d{2}|42949672[0-8]\\d|429496729[0-5])|0[xX]0*[\\da-fA-F]{1,8}|([\\da-fA-F]{1,4}\\:){7}[\\da-fA-F]{1,4}|([\\da-fA-F]{1,4}\\:){6}((\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])\\.){3}(\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])))(\\:\\d+)?(/(?:[^?#\\s/]+/)*(?:[^?#\\s/]{0,}(?:\\?[^?#\\s/]*)?(?:#.*)?)?)?",
	_hostRxp:  new RegExp("^((([^\\[:]+):)?([^@]+)@)?(\\[([^\\]]+)\\]|([^\\[:]*))(:([0-9]+))?$"),
	
	constructor: function() {
		this._urlRegExp = new RegExp("^" + this.urlRegExp + "$", "i");
	},
	
	destroy: function() {
		this.inherited(arguments);	
		if (this.listeners !== null) {
			dojo.forEach(this.listeners, dojo.disconnect);
			this.listeners = null;
		}
	},
	
	setUrlLink: function(link) {
		this._link = link;
	},
	
	setTypeName: function(typeName) {
		this._typeName = typeName;
	},
	
	setShape: function(label) {
		if(dojo.isString(label)) {
			label = this.host.renderMapping.template[label];
		}
		
		// pick first one that is text or wrappingText
		if (!label) {
			var renderMapping = this.host.renderMapping;
			for (var shapeId in renderMapping.template) {
				if (renderMapping.template[shapeId].shape.type == "text" ||
					renderMapping.template[shapeId].shape.type == "wrappingText") {
					label = renderMapping.template[shapeId];
				}
			}
		}
		
		this._label = label;
		this.listeners = [];
		this.listeners.push(label.connect("onmousedown", this, "_onMouseDown"));
		this.listeners.push(label.connect("onmouseup", this, "_onMouseUp"));
		if (label.shape.type == "text") {
			label.rawNode.style.cursor = "pointer";
		} else if (label.shape.type == "wrappingText") {
			label.setCursor("pointer");
		}

	},

	_onMouseDown: function(e) {
		this._saveStrt = {x: e.clientX, y: e.clientY};
	},
	
	_onMouseUp: function(e) {
		// only execute action if user hasn't clicked and dragged on this shape
		if ( this._saveStrt && this._saveStrt.x == e.clientX && 
				this._saveStrt.y == e.clientY && this._typeName) {

			// unselect
			this.host.root.getSelectionController().deselect(this.host);
			
			// determine url
			var element = this.host.renderMapping.getDefaultSource();
			var link = element[this._link];
			if (link) {
				if (link.rdf_about) {
					link = link.rdf_about;
				} else if (link.$ref) {
					link = link.$ref;
				}
			}
			var url = link ? link : (this._label.getText ? this._label.getText() : this._label.shape.text);
			if (url && url.indexOf(modelsFrontService) === 0) {
					location.href = "#action=rmps.app.design.edit&resource=" + url;
			}else if (url && this._urlRegExp.test(url)) {
					// prepend http:// ?
					if(url.indexOf("://") === -1){
						if(url.charAt(0) !== '/' && url.indexOf("./") !== 0){
							if(this._hostRxp.test(url)){
								url = "http://" + url;
							}
						}
					}
					window.open(url, "_blank");
			} else {
				var dialog = new rmpw.forms.ui.EditOrCreateResource();
				var dialogArgs = {
					typeName: this._typeName, 
					typeForm: this._typeName, 
					defaultUri: null, 
					defaultLabel: "", 
					parent: this, 
					featureId: "dmcodegen_defines", 
					target: element
				}				
				dialog.editOrCreateResource(dialogArgs);
			}
		}
	}
	
});

})();