/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.MoveableBase");

dojo.require("rmpw.diagrams.edit.tools.Commitable");
dojo.require("rmpw.diagrams.edit.util.MoveableNoDelay");
dojo.require("rmpw.diagrams.render.util.Workarounds");
dojo.require("rmpw.diagrams.render.util.Rectangle");

/**
 * Base Moveable class
 * 
 * @author Marc Gobeil / Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.tools.MoveableBase", rmpw.diagrams.edit.tools.Commitable, {
	singleton: "moveable",

	constructor: function() {
		this._enableFeedback = true;
	},
	
	setTargetShape: function(moveableShape) {
		this._moveableShape = moveableShape;
		this._moveable = new rmpw.diagrams.edit.util.MoveableNoDelay(moveableShape, { mover: this._mover });
		
		// local events
		this._moveable.onMove = function(/* dojox.gfx.Mover */ mover, /* Object */ shift)  {
			// do nothing
		};
		
		this._connectMove = [
			dojo.connect(this._moveable, "onFirstMove", this, "onFirstMove"),
			dojo.connect(this._moveable, "onMove", this, "onMove"),
			dojo.connect(this._moveable, "onMoveStop", this, "onCommit")
		];
	},

	setRenderMapping: function(renderMapping) {
		this._renderMapping = renderMapping;
	},

	setEnableFeedback: function(enable) {
		this._enableFeedback = enable;
	},
	
	setMover: function(mover) {
		this._mover = mover;
	},
	
	onCancel: function() {
		// TODO (MG): shouldn't have to make this check, comes form onCancel() getting called after object is destroyed
		if(this.host) {
			if (this.host.root.policies.focusScope.getCaptureFocus() === this.host) {
				this.host.root.policies.focusScope.releaseCapture();
			}
			
			if (this._manualMove) {
				delete this._manualMove;
			}
			if (this.showFeedbackDelay) {
				clearTimeout(this.showFeedbackDelay);
				this.showFeedbackDelay = null;
			}
			if (this._bounds) {
				delete this._bounds;
		}
		}
	},
	
	onCommit: function(mover) {
		this.onMoveStop(mover ? mover : this._moveable.mover);
		this.onCancel();
		return true;
	},
	
	/*
	 * Performs the move
	 */
	doMove: function(dx, dy) {
		if (!this._manualMove) {
			this._manualMove = true;
			this.onFirstMove(this._moveable.mover);
		}
		this.onMove(this._moveable.mover, {dx: dx, dy: dy});
	},
	
	destroy: function() {
		this.onCancel();
		for (var i=0, lg=this._connectMove.length; i<lg; i++) {
			dojo.disconnect(this._connectMove[i]);
		}
		this._moveable.destroy();
		
		this.inherited(arguments);
	},
		
	onFirstMove: function(/* dojox.gfx.Mover */ mover) {
		this.host.root.policies.focusScope.setCapture(this.host);
		var bounds = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(this._moveableShape);
		if (this._manualMove) {
			var offset = this.host.root.viewport.getAbsoluteCanvasOffset();
			mover.lastX = bounds.x + offset.x;
			mover.lastY = bounds.y + offset.y;
		}	
		this._bounds = bounds;
	},
	
	onMove: function(/* dojox.gfx.Mover */ mover, /* Object */ shift)  {
		// find any items intersecting items and set them as focus
		this._bounds.x += shift.dx;
		this._bounds.y += shift.dy;
		if (this._manualMove) {
			mover.lastX += shift.dx;
			mover.lastY += shift.dy;
		}	
		
		// no feedback
		if (!this._enableFeedback) {
			return;
		}
		
		if (this.showFeedbackDelay) {
			clearTimeout(this.showFeedbackDelay);
		}
		
		// delay feedback of targetted focus elements so that calculation doesnt slow down
		// the movement
		this.showFeedbackDelay = setTimeout( dojo.hitch(this, function() {
			if (this.host.renderMapping.policies.graphElement) {
				var graphEl = this.host.renderMapping.policies.graphElement;
				var graphManager = graphEl.getGraphManager();
				
				var relBox = new rmpw.diagrams.render.util.Rectangle(this._bounds.x, this._bounds.y, this._bounds.width, this._bounds.height);
				graphEl.translateToRelative(relBox);
				var neighbourMappings = graphManager.getNearMappingsForBounds(relBox, null, graphManager.INTERSECT_EXACT);
				if (neighbourMappings && neighbourMappings.length > 0) {
					this.host.root.policies.focusScope.setFocus(neighbourMappings[0].__editController);
				}
			}
		}), 400);	
	},
	
	onMoveStop: function(/* dojox.gfx.Mover */ mover) {
		// subclasses to override
	}
});