/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.PolylineBendpointEditable");
dojo.require("rmpw.diagrams.edit.tools.Feedback");
dojo.require("rmpw.diagrams.render.util.Point");
dojo.require("rmpw.diagrams.edit.util.MoveableNoDelay");

/**
 * Policy added to edges that allows them to be edited - change, add / delete bendpoints.  This policy
 * will also force the connection to be selectable since it inherits from the Selectable policy.
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.tools.PolylineBendpointEditable",
		rmpw.diagrams.edit.tools.Feedback, {
	
	singleton: "polylineEditable",
	
	targetGap: 16,
	constructor: function() {
		// this policy depends on the selectable policy so create it if it
		// doesn't exist
		var selectable = this.host.policies.selectable;
		if (!selectable) {
			this.host.createPolicy(rmpw.diagrams.edit.tools.Selectable);
		}
		
		if (!this.setUpdateBendpoints) {
			var renderMapping = this.host.renderMapping;
			this.setUpdateBendpoints = dojo.hitch(this, function(changeSet, bendpoints, sourceAnchor, targetAnchor) {
				var bendpointsTrace = renderMapping.traces[this.host.renderMapping.policies.graphEdge._getInternalTraceName("bendpoints")];
				var sourceAnchorTrace = renderMapping.traces[this.host.renderMapping.policies.graphEdge._getInternalTraceName("sourceAnchor")];
				var targetAnchorTrace = renderMapping.traces[this.host.renderMapping.policies.graphEdge._getInternalTraceName("targetAnchor")];
				if (bendpointsTrace && sourceAnchorTrace && targetAnchorTrace) {
					var change = changeSet.createChange(bendpointsTrace.getOwner());
					change.set(bendpointsTrace.propertyName, bendpoints);
					change = changeSet.createChange(sourceAnchorTrace.getOwner());
					change.set(sourceAnchorTrace.propertyName, sourceAnchor);
					change = changeSet.createChange(targetAnchorTrace.getOwner());
					change.set(targetAnchorTrace.propertyName, targetAnchor);
					return true;
				}
				
				return false;
			});
		}
		
		var mapping = this.host.renderMapping;
		//without a delay Moveable fires "move start" on mouse down event
		this._moveable = new rmpw.diagrams.edit.util.MoveableNoDelay( mapping.getShape() );
		// local events
		this._moveable.onMove = function(/* dojox.gfx.Mover */ mover, /* Object */ shift)  {
			// do nothing
		};
		
		this._connectMove = [];
		this._connectMove.push(dojo.connect(this._moveable, "onMove", this, "onPolylineMoving"));
		this._connectMove.push(dojo.connect(this._moveable, "onMoveStop", this, "onPolylineMoveStop"));
		
	},
	
	destroy: function() {
		this.inherited(arguments);
		for (var i=0, lg=this._connectMove.length; i<lg; i++) {
			dojo.disconnect(this._connectMove[i]);
		}
		this._moveable.destroy();
	},
	
	setOnChangeBendpoints: function(changeBendpointsAction) {
		this.setUpdateBendpoints = dojo.hitch(this, function(changeSet, bendpoints, sourceAnchor, targetAnchor, layout, autoLock) {
			changeBendpointsAction.run({changeSet: changeSet, bendpoints: bendpoints, sourceAnchor: sourceAnchor, targetAnchor: targetAnchor, layout: layout, autoLock: autoLock});
		});
	},

	onPolylineMoving: function(mover, shift) {
		var selectable = this.host.policies.selectable;
		if (selectable && selectable.feedbackFigure) {
			selectable.feedbackFigure.onPolylineMoving(mover, shift);
		}
	},
	
	onPolylineMoveStop: function(mover) {
		var selectable = this.host.policies.selectable;
		if (selectable && selectable.feedbackFigure) {
			selectable.feedbackFigure.onPolylineMoveStop(mover);
		}
	}
	
});
