/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.PopupEditor");

dojo.require("rmpw.common.policy.Policy");
dojo.require("rmpw.diagrams.edit.util.Utils");
dojo.require("dijit.TooltipDialog");
//dojo.require("rmpw.diagrams.edit.tools.TestForm");
dojo.require("jazz.ui.Dialog");

/**
 * Pops up a form editor.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.diagrams.edit.tools.PopupEditor", rmpw.common.policy.Policy, {

	listeners: null,
	constructor: function() {
		// TODO: selection scope events should have more info, like what's been selected, so we don't need to ask back
		this._selectionScope = this.findNearestParentByKey("selectionScope");
		
		this._sources = {};
		this.listeners = [];
	},
	
	onTrigger: function(e) {
		for(var setter in this._sources) {
			var sourcePath = this._sources[setter];
			var modelInstance = this.host.renderMapping.getSourceByPath(sourcePath);
			this._form[setter](modelInstance);
		}
		
        var form = this._form;
        var floatingPaneParams = {
                primaryTitle: "Properties",
                secondaryTitle: "",
                width: "650px",
                onClose: dojo.hitch(this, this.onClose),
                contentNode: form.domNode
        };
        
        this._form.domNode.style.padding= "6px";
        
        this.selectTypeDialog = new jazz.ui.Dialog(floatingPaneParams);
        
		this.listeners.push(dojo.connect(this._form, "onSubmit", this.selectTypeDialog, "close"));
		this.listeners.push(dojo.connect(this._form, "onCancel", this.selectTypeDialog, "close"));
		this.listeners.push(dojo.connect(this._selectionScope, "onSelectionChanged", this, "_onSelectionChanged"));
		form.startup();
	},
	
	setTriggerShape: function(triggerShapePath) {
		this._triggerShape = this.host.renderMapping.template[triggerShapePath];
		this._attachListeners(this._triggerShape);
	},
	
	
	onClose: function() {
		dojo.forEach(this.listeners, dojo.disconnect);
		this.listeners = [];
	},

	setForm: function(formConstructor) {
		this._form = new formConstructor();
		this._form._setRoot(this.host.root);
		this._form.startup();
	},
	
	setSources: function(setterPairs) {
		for(var key in setterPairs) {
			this._sources['set' + key.substr(0, 1).toUpperCase() + key.substr(1)] = setterPairs[key];
		}
	},
	
	_onSelectionChanged: function() {
		var selection = this._selectionScope.getSelection();
		if(selection.length !== 1 || selection[0] !== this.host) {
			this._form.submit();
		}
	},
	
	_attachListeners: function(shape) {
		this._listeners = [];
		this._listeners.push(shape.connect("ondblclick", this, "_onTriggerShapeFired"));
	},

	_onTriggerShapeFired: function() {
		var coords = rmpw.diagrams.edit.util.Utils.getGlobalCoords(this._triggerShape);
		var e = {
			x: coords.x + (coords.width / 2),
			y: coords.y + (coords.height / 2)
		};
		this.onTrigger(e);
	},
	
	destroy: function() {
		dojo.forEach(this._listeners, dojo.disconnect);
	}	
});
