/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.ReferenceSelector");

dojo.require("rmpw.common.policy.Policy");

/**
 * Places a dropdown menu over a shape on the diagram when it's selected.  Can be configured
 * with a label function to apply to options, and (TODO) perform a search for references that
 * fit the target property type. 
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.diagrams.edit.tools.ReferenceSelector", rmpw.common.policy.Policy, {
	setShape: function(textShape) {
		this.shape = this.host.renderMapping.template[textShape];
	    this._selectionListener = dojo.connect(this.host.policies.selectable, 'onSelectionChanged', this, '_onSelectionChanged');
	},
	
	destroy: function() {
		dojo.disconnect(this._selectionListener);
	},
	
	setSourceProperty: function(property) {
		this.sourceProperty = property;
	},
	
	setSourcePropertyType: function(type) {
		this.sourcePropertyType = type;
	},
	
	setLabelFunction: function(func) {
		this.labelFunc = func;
	},
	
	_onSelectionChanged: function(selected) {
		if(selected) {
			this._onTrigger();
		} else {
			this._onDeactivate();
		}
	},
	
	_onTrigger: function() {
		// TODO: this check shouldn't be necessary, sometimes don't catch the events we should to clear the dropdown
		if(this.select) {
			this._onDeactivate();
		}
		
		// create a matching html text input node
	    this.select = dojo.create("select", null, this.host.viewer.container);
	    
	    // TODO: get from query
	    var options = [
			{ rdf_about: "http://www.w3.org/2001/XMLSchema#string", rdfs_label: "String" },
			{ rdf_about: "http://www.w3.org/2001/XMLSchema#integer", rdfs_label: "Integer" },
			{ rdf_about: "http://www.w3.org/2001/XMLSchema#double", rdfs_label: "Double" },
			{ rdf_about: "http://www.w3.org/2001/XMLSchema#boolean", rdfs_label: "Boolean" },
			{ rdf_about: "http://www.w3.org/2001/XMLSchema#dateTime", rdfs_label: "DateTime" },
			{ rdf_about: "http://www.w3.org/2001/XMLSchema#nonNegativeInteger", rdfs_label: "NonNegativeInteger" }
		];
	    
	    var currentValue = this.host.renderMapping.getSourceByPath(this.sourceProperty);
	    if (!currentValue) {
            currentValue = options[0];
	    }
	    
	    for(var i = 0, option; option = options[i]; i++) {
		    dojo.create("option", {
                innerHTML: this.labelFunc ? this.labelFunc(option) : option,
                value: dojo.toJson(option),
                selected:
                option.rdf_about === currentValue.rdf_about
		    }, this.select);
	    }
	    
	    if(this.shape.fontStyle) {
		    dojo.style(this.select, "fontFamily", this.shape.fontStyle.family);
		    dojo.style(this.select, "fontSize", this.shape.fontStyle.size);
		    dojo.style(this.select, "fontWeight", this.shape.fontStyle.weight);
	    }
	    
	    this._reposition();
	
		this._viewportListener = dojo.connect(this.host.root.viewport, "onViewportChanged", this, "_reposition");	    
	    var moveable = this.host.policies.moveable;
	    if(moveable) {
            this._moveListener = dojo.connect(moveable, 'onMoveStop', this, '_reposition'); 
	    }
	},
	
	_onDeactivate: function() {
		this._commit();
		
		if(this._moveListener) {
			dojo.disconnect(this._moveListener);
			delete this._moveListener;
		}
		
		dojo.disconnect(this._viewportListener);
		delete this._moveListener;
		
		dojo.destroy(this.select);
		delete this.select;
	},
	
	_reposition: function() {
	    // position it directly over the text we're editing
		// measure the gfx text shape bounds
	    var textBounds = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(this.shape);
	    dojo.style(this.select, "position", "absolute");
	    dojo.style(this.select, "left", textBounds.x + "px");
	    dojo.style(this.select, "top", textBounds.y + "px");
	    dojo.style(this.select, "width", textBounds.width + "px");
	    dojo.style(this.select, "height", textBounds.height + "px");
	    dojo.style(this.select, "zIndex", 1000);	// by default low value, will be hidden under the canvas
	},
	
	_commit: function() {
		var command = {
			execute: function(commandStack, args) {
				var changeSet = new rmpw.common.vcs.ChangeSet();
				var propertyChange = changeSet.createChange(args.owner);
				propertyChange.set(args.propName, args.value);
				commandStack.repository.commit(changeSet);
				return true;
			}
		};
		
		var commandStack = this.host.root.getCommandController().commandStack;
		commandStack.execute(command, {
			owner: this.host.renderMapping.getSourceOwnerByPath(this.sourceProperty),
			propName: this.sourceProperty.substring(this.sourceProperty.lastIndexOf('.') + 1),
			value: dojo.fromJson(this.select.value)
		});
	}
});