/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.diagrams.edit.tools.RichHover");

dojo.require("rmps.core.RootServices");
dojo.require("rmps.app.core.util");
dojo.require("rmpw.common.policy.Policy");
dojo.require("rmpw.common.util.IdentifierUtil");
dojo.require("rmpw.diagrams.edit.actionBindings.Button");
dojo.require("rmpw.diagrams.edit.actions.NativeCode");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmpw.diagrams.edit", "BehaviorMessages");
var behaviorMessages = dojo.i18n.getLocalization("rmpw.diagrams.edit", "BehaviorMessages");  

var modelsFrontService = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "models");			

/**
 * @author John Swanke
 */
dojo.declare("rmpw.diagrams.edit.tools.RichHover", rmpw.diagrams.edit.tools.Feedback, {
	singleton: "RichHover",
	
	constructor: function( ) {
		// imported diagram
		if (this.host.renderMapping.sources.model) {
			var zonedata = this.host.renderMapping.sources.model.zonedata; 
			if (zonedata && zonedata.hasElement) {
				var shapeNode = this.host.renderMapping.getShape().children[0].rawNode;
				this._createLink(zonedata.id, shapeNode);
			}
		}
	},
	
	/**
	 * @override
	 */
	attachListeners: function() {
		this.inherited(arguments);
		
		// attach to context action bar
		if (this.host.policies.contextActionBar) {
			this._attachPreviewAction();
		}
		else {
			// maybe Marc's dependency ordering would solve this issue
			this._connectCreateContextActionBar = dojo.connect(this.host, "createPolicy", this, dojo.hitch(this, function() {
				if (this.host.policies.contextActionBar) {
					dojo.disconnect(this._connectCreateContextActionBar);
					delete this._connectCreateContextActionBar;
					this._attachPreviewAction();
				}
			}));
		}
		
	},
	
	_attachPreviewAction: function() {
		invokeRichHoverAction = this.host.createPolicy(rmpw.diagrams.edit.actions.NativeCode, "invokeRichHover");
		invokeRichHoverAction.setRun(dojo.hitch(this, function(args) {
			if (this._resourceLink) {
				// Show the preview action compact rendering.
				setTimeout(dojo.hitch(this._resourceLink, function() {
						if (this.popup && !this.popup._isOpen) {
							this.popup.open();
						}
					}), 10);
			}
			
			return true;
		}));
		
		// add an action to the Context Action Bar for accessibility / touch device 
		var actionBinding = this.host.createPolicy(rmpw.diagrams.edit.actionBindings.Button, "previewButton");
		actionBinding.setAction(invokeRichHoverAction);
		actionBinding.setIconURL(dojo.moduleUrl("rmpw.jazz", "images/preview.gif"));
		actionBinding.setLabel(behaviorMessages.preview);
		
		this.host.policies.contextActionBar.addActionBinding(actionBinding);
	},
	
	setTargetShape: function(targetFigure) {
		// generated diagram
		this.defaultSrc = this.host.renderMapping.getDefaultSource();
		if (this.defaultSrc) {
			if(dojo.isString(targetFigure)) {
				this.shapeNode = this.host.renderMapping.template[targetFigure].rawNode;
			} else {
				this.shapeNode = targetFigure.rawNode;
			}
		}
		this.attachListeners();
	},
	
	enforce: function () {
		// only enforced for generated diagrams
		if (!this._resourceLink && this.defaultSrc && this.shapeNode) {
			if (rmpw.common.util.IdentifierUtil.isIdentifierFixed(this.defaultSrc.rdf_about)) {
				this._createLink(this.defaultSrc.rdf_about, this.shapeNode, 1500);
			}
		}
	},
	
	_createLink: function (resourceUri, shapeNode, openDelay) {
		if (resourceUri) {
			if (dojo.isIE && !shapeNode.currentStyle) {
				shapeNode.currentStyle = {marginLeft: '0px', marginTop: '0px',marginRight: '0px', marginBottom: '0px'};
			}
			var renderingUri = this._getCompactRenderingUri(resourceUri);
			var presentationUri = this._getResourceEditUri(resourceUri);
			this._resourceLink = new jazz.ui.ResourceLink({
						presentationUri: presentationUri,
						uri: renderingUri, 
						lazyFetch: true,
						around: shapeNode
					});
			if (this._resourceLink.popup) {
				if (openDelay) {
					this._resourceLink.popup.openDelay = openDelay;
				}
				var bounds = this.host.renderMapping.policies.graphElement.getTightBounds();
				this._resourceLink.popup.updateDimensions = function(viewportDimensions, absoluteNodePosition, relativeNodePosition) {
					absoluteNodePosition.w = 0;
					absoluteNodePosition.h = 0;
					absoluteNodePosition.x += bounds.width;
					absoluteNodePosition.y += bounds.height/2;
					
					relativeNodePosition.w = 0;
					relativeNodePosition.h = 0;
					relativeNodePosition.x += bounds.width;
					relativeNodePosition.y += bounds.height/2;
				};
			}
		}
	},
	
	_getCompactRenderingUri: function (resourceUri) {
		var renderingUri = resourceUri;
		if (resourceUri.indexOf(modelsFrontService) === 0) {
			var object= {
				resourceUri: resourceUri
			};
			var contextUri = rmps.core.util.ContextUtils.getCurrentContextURI();
			if (contextUri) {
				object[rmps.core.util.IRmpsConstants.BASELINE]= contextUri;
			}
			renderingUri = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "rendering") + "?" + dojo.objectToQuery(object);
		}
		
		return renderingUri;
	},
	
	_getResourceEditUri: function (resourceUri) {
		var editorUri = resourceUri;
		if (resourceUri.indexOf(modelsFrontService) === 0) {
			editorUri = rmps.app.core.util.getDesignViewResourceUri(resourceUri);
		}
		return editorUri;
	},
	
	destroy: function destroy() {
		this.inherited(arguments);
		
		if (this._resourceLink) {
			this._resourceLink.destroy();
		}
			
		this.host.policies.contextActionBar.removeActionBinding(this.host.policies.previewButton);
	}
	
});

})();