/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.Selectable");

dojo.require("rmpw.diagrams.edit.tools.Feedback");
dojo.require("rmpw.diagrams.edit.tools.figures.RectangularSelection");
dojo.require("rmpw.diagrams.edit.tools.figures.PolylineSelectionWithHandles");
dojo.require("rmpw.diagrams.edit.tools.Commitable");
dojo.require("rmpw.diagrams.edit.util.Utils");


/**
 * @author Marc Gobeil / Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.tools.Selectable", 
		[rmpw.diagrams.edit.tools.Commitable, rmpw.diagrams.edit.tools.Feedback], {
	singleton: "selectable",

	constructor: function() {
		this._selected = false;
		this.feedbackLayer = this.getFrontFeedbackLayer();
		if (this.host.renderMapping.policies.graphElement) {
			this.setTargetShape( this.host.renderMapping.getShape() );
		}
		
		if (this.host.renderMapping.policies.graphEdge) {
			this.feedbackFigureType = rmpw.diagrams.edit.tools.figures.PolylineSelectionWithHandles;
		}
		else {
			this.feedbackFigureType = rmpw.diagrams.edit.tools.figures.RectangularSelection;
		}
		this.toolBarController = this.host.root.policies.contextActionBarScope;
	},
	
	destroy: function() {
		if(this._selected) {
			this.host.root.getSelectionController().deselect(this.host);
		}
		this._detachListeners();
		this.inherited(arguments);
	},	
	
	selectOnUpdate: function() {
		var renderMapping = this.host.renderMapping;
		if (renderMapping.policies.graphElement) {
			this._connectUpdateShape = dojo.connect(renderMapping.policies.graphElement, "updateShape", this, "_postUpdateSelect");
		}
	},
	
	_postUpdateSelect: function() {
		if (this._connectUpdateShape) {
			dojo.disconnect(this._connectUpdateShape);
			delete this._connectUpdateShape;
			var selController = this.host.root.getSelectionController();
			selController.selectExclusive(this.host);
		}
	},
	
	onCancel: function() {
		var selectionController = this.host.root.getSelectionController();
		selectionController.deselectAll();
	},
	
	onCommit: function() {
		var selectionController = this.host.root.getSelectionController();
		selectionController.selectExclusive(this.host, {});
		return true;
	},
	
	_addListeners: function() {
		this.inherited(arguments);
		
		this.listeners.push(this.figure.connect("onmousedown", this, "_onMouseDown"));
		this.listeners.push(this.figure.connect("onmouseup", this, "_onMouseUp"));
		this.listeners.push(this.figure.connect("touchstart", this, "_onMouseDown"));
		this.listeners.push(this.figure.connect("touchend", this, "_onMouseUp"));
		
		if (this.isSelectionContainer()) {
			this.listeners.push(dojo.connect(this.host.root.getSelectionController(), "onSelectionChanged", this, "_onSelectionChanged"));
		}
	},
	
	_detachListeners: function() {
		if(this.listeners) {
			dojo.forEach(this.listeners, dojo.disconnect);
			delete this.listeners;
		}
		if (this._connectUpdateShape) {
			dojo.disconnect(this._connectUpdateShape);
			delete this._connectUpdateShape;
		}
	},
	
	/*
	 * @deprecated - THIS METHOD IS NO LONGER NEEDED.  Whether this policy is a selection
	 * container can be implied by presence of graphContainer.
	 * 
	 * Public setter for indicating that this selectable element is a
	 * container of concrete selectable elements.  Behavior will change slightly.
	 * - no multiple select, mouse position of toolbar will move etc.
	 */
	setIsSelectionContainer: function(isContainer) {
		// do nothing
	},
	
	/*
	 * Accessor method determining if this policy is on a selection container
	 */
	isSelectionContainer: function() {
		if (this.host.renderMapping.policies) { // may be called during destroy
			// not selection container if this host also is a GraphNode
			return this.host.renderMapping.policies.graphContainer !== undefined 
			   && this.host.renderMapping.policies.graphNode === undefined;
		}
		
		return false;
	},
	
	_onSelectionChanged: function(selected) {
		this._skipMouseUpProcessing = true;
	},
	
	/*
	 * allow multiple select when control key is depressed
	 */
	allowMultipleSelect: function(e) {
		return e.ctrlKey || e.shiftKey;
	},
	
	_onMouseDown: function(e) {
       var eFixed = e.touches ? e.touches[0] : e; 
       this.start = {x: eFixed.clientX, y: eFixed.clientY};
       
		if( rmpw.diagrams.edit.util.Utils.singleHit(e) ){
			
			if (this.toolBarController.clickedOnToolbar(e)) {
				return;
			}
			
			if (this.isSelectionContainer()) {
				this.mouseDownListener = this.figure.connect("onmousemove", this, "_onMouseMove");
				this.touchstartListener = this.figure.connect("touchmove", this, "_onMouseMove");
				this._skipMouseUpProcessing = false;
				return;
			}
			
			this.mouseDown = true;
			
			// Note: the following calls will result in a selection change event on the edit part,
			// which will in turn show and hide our feedback as appropriate through our selection listener.
			var selectionController = this.host.root.getSelectionController();
			
			if(e.ctrlKey || e.shiftKey) {
				if(this._selected) {
					selectionController.deselect(this.host);
				} else {
					selectionController.select(this.host, e);
				}
			} else {
				this._showNodeToolbar = true;
				if (!this._selected) {
					selectionController.selectExclusive(this.host, e);
				}
				else {
					this.mouseDownListener = this.figure.connect("onmousemove", this, "_onMouseMove");
					this.touchstartListener = this.figure.connect("touchmove", this, "_onMouseMove");
				}
			}
			
			dojo.stopEvent(e);
		}
	},
	
	_onMouseMove: function(e) {
	    var eFixed = e.touches ? e.touches[0] : e; 
		if ((!this.start || this.start.x!==eFixed.clientX && this.start.y!==eFixed.clientY)) {
			if (this.mouseDownListener) {
				dojo.disconnect(this.mouseDownListener);
				delete this.mouseDownListener;
				dojo.stopEvent(e);
			}
			delete this._showNodeToolbar;
		}

	},
	
	_onMouseUp: function(e) {
		if (this._showNodeToolbar && this.host.policies.contextActionBar) {
			delete this._showNodeToolbar;
			if (this._selected){ 
			    this.host.policies.contextActionBar.showToolBar(e);
			}	
			return;
		}
		
		if (this.toolBarController.clickedOnToolbar(e)) {
			return;
		}
		
		this.mouseDown = false;
		
		if (this.mouseDownListener) {
			dojo.disconnect(this.mouseDownListener);
			delete this.mouseDownListener;
			
			if (this.isSelectionContainer()) {
				if(!this._skipMouseUpProcessing){
					if (this.host.policies.contextActionBar) {
						this.host.policies.contextActionBar.removeToolBar();
						this.host.policies.contextActionBar.showToolBar(e);
					}
					this.host.root.getSelectionController().selectExclusive(this.host, e);
				}
			}
			
			if (this.processSelectionOnMouseUp()) {
				this.host.root.policies.selectionScope.selectExclusive(this.host, e);
			}
		}
	},
	
	processSelectionChanged: function() {
		if (this.isSelectionContainer()) {
			return false;
		}
		
		return !this.mouseDown;
	},
	
	processSelectionOnMouseUp: function(e) {
		if (this.isSelectionContainer()) {
			return false;
		}
		
		return this._selected;
	},
	
	isSelected: function() {
		return this._selected;
	},
	
	onSelectionChanged: function(selected) {
		this._selected = selected;
		
		if(selected) {
			this.showFeedback();
		} else {
			this.hideFeedback();
		}
	},

	/**
	 * @override
	 */
	showFeedback: function() {
		if (this.isSelectionContainer()) {
			return; // no feedback if selection container
		}
		
		this.inherited(arguments);

		// TODO: should we only attach this listener if onFigureMoving is defined, suggesting that a moveable policy is even present?
		// follow the figure if it moves while we're showing feedback
		this.movingListener = dojo.connect(this.host, "onFigureMoving", this, "onFigureMoving");
	},
	
	onFigureMoving: function(mover, shift) {
		// feedbakFigure should always != null here, because listener is only attached while it's valid
		this.feedbackFigure.shape.applyLeftTransform(shift);
	},
	
	/**
	 * @override
	 */
	hideFeedback: function() {
		if (this.isSelectionContainer()) {
			return; // no feedback if selection container
		}
		
		this.inherited(arguments);

		// stop watching for movement on the figure
		dojo.disconnect(this.movingListener);
		this.movingListener = null;
	}
});