/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.tools.Tetherable");

dojo.require("rmpw.diagrams.edit.tools.Feedback");
dojo.require("rmpw.diagrams.edit.tools.Selectable");
dojo.require("rmpw.diagrams.edit.tools.figures.LineHighlight");
/**
 * @author Susanta Datta
 * tethers selected node around connector's reference point 
 * target is the connector mapping
 */
dojo.declare("rmpw.diagrams.edit.tools.Tetherable", rmpw.diagrams.edit.tools.Feedback, {
	singleton: "tetherable",
	
	_movingListener: null,
	_targetPoint: null,
	figure:null,
	
	constructor: function() {
		this.feedbackFigureType = rmpw.diagrams.edit.tools.figures.LineHighlight;
		this.setFeedbackLayer(this.host.root.feedbackLayer);
		this.setAnchorShape(this.host.renderMapping.parent.getShape());
		this.setTargetShape(this.host.renderMapping.getShape());
	},
	
	setAnchorShape: function(anchorShape) {
		this._anchorShape = anchorShape;
	},
	
	/**
	 * @override
	 */
	_addListeners: function() {
		this.inherited(arguments);
		
		if (this.host.policies.selectable) {
			this.listeners.push(dojo.connect(this.host.policies.selectable, "onSelectionChanged", this, "onSelectionChanged"));
		}
	},
	
	showFeedback: function() {
		this.inherited(arguments);
		if(this._movingListener){
			//TODO find why the mouse up is not called
			dojo.disconnect(this._movingListener);
		}
		this._targetPoint = this.host.renderMapping.policies.graphFloatingLabel.locator.getReferencePoint(this.figure);
		this.host.renderMapping.policies.graphElement.translateToAbsolute(this._targetPoint);
		var end = this.getReferencePoint(this._anchorShape);
		var start = this.getReferencePoint(this.figure);
		this.feedbackFigure.update(start.x, start.y, this._targetPoint.x, this._targetPoint.y); 
		
		if (this.host.policies.moveable) {
			this._movingListener = dojo.connect(this.host.policies.moveable, "onMove", this, "_onFigureMoving");
		}
	},
	
	_onFigureMoving: function(mover, shift) {
		this.feedbackFigure.moveSourceEnd(shift.dx, shift.dy); 
	},
	
	onSelectionChanged: function(selected, e) {
		if (selected) {
			this.showFeedback();
		}
		else {
			this.hideFeedback();
		}
	},
	
	getBox: function(shape) {
		var bounds = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(
				shape, rmpw.diagrams.render.util.Workarounds.IGNORE_DECORATIONS);
		return new rmpw.diagrams.render.util.Rectangle(bounds.x, bounds.y, bounds.width, bounds.height);
	},
	
	getReferencePoint: function(shape) {
		return this.getBox(shape).getCenter();
	},	
	/**
	 * @override
	 */
	hideFeedback: function() {
		this.inherited(arguments);
		dojo.disconnect(this._movingListener);
		this._movingListener = null;
	}		
});
