/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.WheelZoom");

dojo.require("rmpw.common.policy.Policy");
dojo.require("dojox.gfx.matrix");
dojo.require("dojox.gfx.fx");
dojo.require("dojo.fx.easing");

/**
 * Zooms the diagram with the mouse wheel scroll wheel.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.diagrams.edit.tools.WheelZoom", rmpw.common.policy.Policy, {
	singleton: "wheelZoom",
	
	_minZoom: 0.1,
	_maxZoom: 4.0,
	_animDuration: 0,
	_wheelZoomSpeed: 0.1,
	
	constructor: function() {
		this.viewport = this.host.root.viewport;
		this.container = this.viewport.container;
		
		this._connectListeners();
	},
	
	setMaxZoom: function(zoom) {
		this._maxZoom = zoom;
	},
	
	setMinZoom: function(zoom) {
		this._minZoom = zoom;
	},
	
	setAnimationDuration: function(duration) {
		this._animDuration = duration;
	},
	
	setWheelZoomSpeed: function(speed) {
		if(speed > 0) {
			this._wheelZoomSpeed = speed;
		} else {
			console.warn("invalid speed " + speed + " ignored, must be > 0");
		}
	},
		
	/**
	 * bug #3763: Dojo should fix mouse wheel events - http://bugs.dojotoolkit.org/ticket/3763
	 */ 
	_connectListeners: function() {
		var eventType = (dojo.isMozilla ? "DOMMouseScroll" : "onmousewheel");
		this.wheelZoomListener = dojo.connect(this.container, eventType, this, "_onMouseWheel");
	},

	// TODO: the diagram loader is clearing all policies on load, so this gets destroyed right after creation
//	destroy: function() {
//		dojo.disconnect(this.wheelZoomListener);
//	},

	_onMouseWheel: function(e) {
		// Smooth out IE/FF differences and normalize data
		// - bug: http://bugs.dojotoolkit.org/ticket/3763
		// - ref: http://www.switchonthecode.com/tutorials/javascript-tutorial-the-scroll-wheel
		e = e ? e : window.event;
		//  data: ff:detail, ie:wheelData, up positive, down negative, ff scrolls in increments of 3 units with my mouse (MG)
		var wheelData = e.detail ? e.detail * -1 : e.wheelDelta / 40; 
		// normalize to +/-1 unit per wheel click
		wheelData /= 3;
		
		// calculate location relative to container (which is 1:1 with viewport coords)
		var bounds = dojo.position(this.container);
		var x = e.clientX - bounds.x;
		var y = e.clientY - bounds.y;

		var zoomChange = 1 + (wheelData * this._wheelZoomSpeed);
		var oldZoom = this.viewport.getZoom();
		var newZoom = oldZoom * zoomChange;
		newZoom = Math.min(this._maxZoom, newZoom);
		newZoom = Math.max(this._minZoom, newZoom);
		
		// apply the zoom
		this.viewport.viewer.diagramLayer.rootLayer.setScale(newZoom, null, null, true);
		
		/** keeping the below code as a placeholder, in case we eventually add bounce effect (see TODO below) **/
		/*if(	(newZoom <= this._maxZoom || (oldZoom > this._maxZoom && zoomChange < 1)) &&
			(newZoom >= this._minZoom || (oldZoom < this._minZoom && zoomChange > 1)) ) {
			
			// apply the zoom
			this.host.diagramLayer.rootLayer.setScale(newZoom, null, null, true);
		} else {
			// TODO: implement bounce effect using dojo.fx.easing.backOut easing
		}*/
		
		// prevent window from scrolling
		if(e.stopPropagation) {
			e.stopPropagation();
		}
		if(e.preventDefault) {
			e.preventDefault();
		}
		e.cancel = true;
		e.cancelBubble = true;
		e.returnValue = false;
		return false;
	}
});

