/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("rmpw.diagrams.edit.tools.dialogs.LinePropertiesDialog");
dojo.require("dojo.i18n");
dojo.require("rmpw.diagrams.render.graph.BaseGraphConfigurator");
dojo.require("jazz.ui.Dialog");

dojo.requireLocalization("rmpw.diagrams.edit", "BehaviorMessages");

dojo.declare("rmpw.diagrams.edit.tools.dialogs.LinePropertiesDialog", [dijit._Widget, dijit._Templated], {
	
	templateString: dojo.cache("rmpw.diagrams.edit", "tools/dialogs/templates/LinePropertiesDialog.htm"),
	
	widgetsInTemplate: true,
	
	selectedRoutingType: rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.OBLIQUE_ROUTING,
	selectedSmoothness: "none",
	selectedAvoidObstructions: true,
	
	_obliqueRoutingRadioButton: null,
	_rectilinearRoutingRadioButton: null,
	_treeRoutingRadioButton: null,
	
	_smoothnessNoneRadioButton: null,
	_smoothnessLessRadioButton: null,
	_smoothnessNormalRadioButton: null,
	_smoothnessMoreRadioButton: null,
	
	_avoidObstructionsCheckbox: null,
	
	_okButtonNode: null,
	_cancelButtonNode: null,
	_messages: null,

	constructor: function(args) {
		this.selection = args.selection;
	},
	
	postCreate: function() {
		this.inherited("postCreate", arguments);
    	this._okButtonNode.attr("disabled", true);
    	this.connect(this._smoothnessNoneRadioButton, "onClick",
    			function() {
    		this.selectedSmoothness = "none";
    		this._okButtonNode.attr("disabled", false);
    	});
    	this.connect(this._smoothnessLessRadioButton, "onClick",
    			function() {
    		this.selectedSmoothness = "less";
    		this._okButtonNode.attr("disabled", false);
    	});
    	this.connect(this._smoothnessNormalRadioButton, "onClick",
    			function() {
    		this.selectedSmoothness = "normal";
    		this._okButtonNode.attr("disabled", false);
    	});
    	this.connect(this._smoothnessMoreRadioButton, "onClick",
    			function() {
    		this.selectedSmoothness = "more";
    		this._okButtonNode.attr("disabled", false);
    	});
		
		this.connect(this._obliqueRoutingRadioButton, "onClick",
				function() {
			this.selectedRoutingType = rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.OBLIQUE_ROUTING;
			this._okButtonNode.attr("disabled", false);
		});
		this.connect(this._rectilinearRoutingRadioButton, "onClick",
				function() {
			this.selectedRoutingType = rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.RECTILINEAR_ROUTING;
			this._okButtonNode.attr("disabled", false);
		});
		this.connect(this._treeRoutingRadioButton, "onClick",
				function() {
			this.selectedRoutingType = rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.TREE_ROUTING;
			this._okButtonNode.attr("disabled", false);
		});
		
		this.connect(this._avoidObstructionsCheckbox, "onClick",
				function() {
			this.selectedAvoidObstructions = this._avoidObstructionsCheckbox.checked;
			this._okButtonNode.attr("disabled", false);
		});
		
		this.dialog = new jazz.ui.Dialog({
			contentNode: this._linePropertiesForm,
			primaryTitle: this._messages.lineProperties,
			destroyContents: true,
			width: "400px",
		});
		
		for(var i = 0; editController = this.selection[i]; i++) {
			if (editController.renderMapping.policies.graphEdge) {
				var graphEdge = editController.renderMapping.policies.graphEdge;
				this.selectedRoutingType = graphEdge.getRouterType();
				this.selectedSmoothness = graphEdge.getSmoothness();
				this.selectedAvoidObstructions = graphEdge.getAvoidObstructions();
				break;
			}
		}
		
		this._obliqueRoutingRadioButton.attr("checked", this.selectedRoutingType == rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.OBLIQUE_ROUTING);
		this._rectilinearRoutingRadioButton.attr("checked", this.selectedRoutingType == rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.RECTILINEAR_ROUTING); 
		this._treeRoutingRadioButton.attr("checked", this.selectedRoutingType == rmpw.diagrams.render.graph.BaseGraphConfigurator.prototype.TREE_ROUTING); 
		
		this._smoothnessNoneRadioButton.attr("checked", this.selectedSmoothness == "none"); 
		this._smoothnessLessRadioButton.attr("checked", this.selectedSmoothness == "less"); 
		this._smoothnessNormalRadioButton.attr("checked", this.selectedSmoothness == "normal");
		this._smoothnessMoreRadioButton.attr("checked", this.selectedSmoothness == "more");
		
		this._avoidObstructionsCheckbox.attr("checked", this.selectedAvoidObstructions);
		
	},
	
	postMixInProperties: function() {	
		this._messages = dojo.i18n.getLocalization("rmpw.diagrams.edit", "BehaviorMessages");     
	},
	
	_onOk: function() {
		var args = { viewer: this.viewer };
		args.editControllers = this.selection;
		args.smoothness = this.selectedSmoothness;
		args.router = this.selectedRoutingType;
		args.avoidObstructions = this.selectedAvoidObstructions;
		args.viewer.getCommandController().execute("rmpw.diagrams.edit.commands.ChangeLinePropertiesCommand", args);
		
		this.dialog.close();
	},
	
	_onCancel: function() {
		this.dialog.close();
	},
      
   uninitialize: function() {
	   if (this.dialog) {
		   this.dialog.destroy();
	   }
   }   
});

})();
