/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.figures.LineHighlight");

dojo.require("dojox.gfx.fx");
dojo.require("rmpw.diagrams.edit.tools.figures.Highlight");
dojo.require("rmpw.diagrams.render.util.Dimension");

/**
 * A polyline highlight that fades out when it's destroyed.
 * Meant to be used on the rear feedback layer.
 * 
 * @author Steve Shaw, Marc Gobeil
 */
dojo.declare("rmpw.diagrams.edit.tools.figures.LineHighlight", rmpw.diagrams.edit.tools.figures.Highlight, {
	strokeWidth: 3,
	
	constructor: function(feedbackLayer, target, editController) {
		this.feedbackLayer = feedbackLayer;
		this.editController = editController;
		this.shape = this.createShape(null);
	},
	
	createShape: function() {
		var glow = this.feedbackLayer.createLine({
            x1 : 0,
            y1 : 0,
            x2 : 1,
            y2 : 1
        });
		
		glow.setStroke({
			color: this.colorStart,
			width: this.strokeWidth,
			cap: "round"
		});
		glow.rawNode.setAttribute("pointer-events", "none");
		return glow;
	},
	
	update:function(start_x, start_y, end_x, end_y){
	    var shape = this.shape.getShape();
	    shape.x1 = start_x;
	    shape.y1 = start_y;
	    shape.x2 = end_x;
	    shape.y2 = end_y;
	    this.shape.setShape(shape);
	},	
	moveSourceEnd:function(dx, dy){
	    var shape = this.shape.getShape();
	    shape.x1 += dx;
	    shape.y1 += dy;
	    this.shape.setShape(shape);
	},	
	moveTargetEnd:function(dx, dy){
	    var shape = this.shape.getShape();
	    shape.x2 += dx;
	    shape.y2 += dy;
	    this.shape.setShape(shape);
	},
	setPoints: function (points){
		this.shape.setShape(points);
	},
	
	destroy: function() {
		var fadeOut = dojox.gfx.fx.animateStroke({
			shape: this.shape,
            color : {start: this.colorStart, end : this.colorEnd},
            duration : 300
        });
		dojo.connect(fadeOut, "onEnd", function() {
			this.shape.removeShape();
		});
		fadeOut.play();
	}
});