/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.figures.PolylineFeedback");

dojo.require("rmpw.diagrams.render.util.ColorConstants");

/**
 * Subclass to handle selection of connections.  Handles are added at bendpoint locations.
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.tools.figures.PolylineFeedback", null, {
	handles: [],
	polylineFeedbackShape: null,
	dummyConnection: null,

	constructor: function(handles, renderMapping, graphContainer, routerType, owner, color) {
		this.handles = handles;
		this.dummyConnection = graphContainer.host.viewer.graphConfigurator.createGraphEdge(renderMapping, 
				graphContainer.getInternalGraphElement());
		this.dummyConnection.setFeedback(true);
		if (renderMapping) {
			this.dummyConnection.setRouterType(routerType);
		}
		this.dummyConnection.setSourceAnchor(handles[0]);
		this.dummyConnection.setTargetAnchor(handles[handles.length - 1]);
		
		var points = new rmpw.diagrams.render.util.PointList();
		for (var i=0, handle; handle = this.handles[i]; i++) {
			var pos = handle.getPosition();
			points.addPoint_0(pos.x, pos.y);
		}
		
		this.dummyConnection.setPointList(points);
		
		if (renderMapping && renderMapping.policies.graphEdge) {
			var edge = renderMapping.policies.graphEdge.getInternalGraphElement();
			if (edge) {
				this.dummyConnection.setFixSourceAnchor(this.dummyConnection.isOrthogonal() ? true : edge.isFixedSourceAnchor());
				this.dummyConnection.setFixTargetAnchor(this.dummyConnection.isOrthogonal() ? true : edge.isFixedTargetAnchor());
			}
		}
		
		this.owner = owner;
		this.foregroundColor = color;
		this.mapping = renderMapping;
	},
	
	getStartAnchor: function() {
		return this.dummyConnection.getSourceAnchor();
	},
	
	getEndAnchor: function() {
		return this.dummyConnection.getTargetAnchor();
	},
	
	/* 
	 * Update the connection feedback when a handle is moved 
	 */
	updateConnectionLines: function() {
		// get the points from the handles
		var points = new rmpw.diagrams.render.util.PointList();
		for (var i=0, handle; i < this.handles.length; i++) {
		    handle = this.handles[i];
			var pos = handle.getPosition();
			points.addPoint_0(pos.x, pos.y);
		}
		
		this.dummyConnection.translateToRelative(points);
		this.dummyConnection.setRouterConstraint(points, false);
		this.dummyConnection.route();
		var routedPoints = this.dummyConnection.getPointList().getCopy();
		this.dummyConnection.translateToAbsolute(routedPoints);
		
		if (this.polylineFeedbackShape === null) {
		    this.polylineFeedbackShapeOutline = this.owner.createPolyline(routedPoints.points)
                .setStroke({color: rmpw.diagrams.render.util.ColorConstants.SELECTION_OUTLINE, width: 3, style: "solid"});
            this.polylineFeedbackShapeOutline.rawNode.style.opacity = 0.5;
    		this.polylineFeedbackShapeOutline.rawNode.style.cursor="not-allowed";
            this.polylineFeedbackShapeOutline.moveToBack();
		    
			this.polylineFeedbackShape = this.owner.createPolyline(routedPoints.points)
				.setStroke({color: this.foregroundColor, width: 1, style: "solid"});
			this.polylineFeedbackShape.rawNode.style.opacity = 1.0;
			this.polylineFeedbackShape.rawNode.style.cursor="not-allowed";
			this.polylineFeedbackShape.moveToBack();
		}
		else {
		    this.polylineFeedbackShapeOutline.setShape(routedPoints.points);
			this.polylineFeedbackShape.setShape(routedPoints.points);
		}
		
		return routedPoints;
	},
	
	destroy: function() {
		if (this.polylineFeedbackShape) {
		    this.owner.remove(this.polylineFeedbackShapeOutline);
			this.owner.remove(this.polylineFeedbackShape);
		}
		
		if (this.dummyConnection) {
			this.dummyConnection.destroy();
			this.dummyConnection = null;
		}
	}
});