/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.figures.PolylineHighlight");

dojo.require("rmpw.diagrams.render.util.Workarounds");
dojo.require("dojox.gfx.fx");
dojo.require("rmpw.diagrams.edit.tools.figures.Highlight");
dojo.require("rmpw.diagrams.render.util.Dimension");

/**
 * A polyline highlight that fades out when it's destroyed.
 * Meant to be used on the rear feedback layer.
 * 
 * @author Steve Shaw, Marc Gobeil
 */
dojo.declare("rmpw.diagrams.edit.tools.figures.PolylineHighlight", rmpw.diagrams.edit.tools.figures.Highlight, {
	strokeWidth: 2,
	
	constructor: function(feedbackLayer, target, editController, startColor, endColor, strokeWidth) {
		this.editController = editController;
		this.feedbackLayer = feedbackLayer;
		if (startColor) {
			this.colorStart = startColor;
		}
		if (endColor) {
			this.colorEnd = endColor;
		}
		this._strokeWidth = strokeWidth;
		this.shape = this.createShape(target);
	},
	
	getFeedbackPoints: function(target) {
		var mapping = this.editController.renderMapping;
		if (mapping.policies && mapping.policies.graphElement) {
			var points = mapping.policies.graphElement.getPolygonPoints().getCopy();
			mapping.policies.graphElement.translateToAbsolute(points);
			mapping.root.diagram.policies.graphElement.translateToRelative(points);
			return points.points;
		}
		
		// assume front feedback (no mapping)
		var dojobounds = rmpw.diagrams.render.util.Workarounds.getBoundingBox(target);
		rmpw.diagrams.render.util.Workarounds.transformBoundingBox(target.matrix, dojobounds);
		var bounds = new rmpw.diagrams.render.util.Rectangle(dojobounds.x, dojobounds.y, dojobounds.width, dojobounds.height);
		rmpw.diagrams.render.util.Workarounds.translateToAbsolute(bounds, target);
		
		return rmpw.diagrams.render.util.PointListUtilities.prototype.getRectanglePolyline(bounds);
	},
	
	createShape: function(target) {
		var glow = this.feedbackLayer.createPolyline(this.getFeedbackPoints(target));
		
		var strokeWidth = this._strokeWidth ? this._strokeWidth : this.strokeWidth;
		var dim = new rmpw.diagrams.render.util.Dimension(strokeWidth / 2, 0);
		var mapping = this.editController.renderMapping;
		
		if (mapping.policies && mapping.policies.graphElement) {
			dim.width = strokeWidth - 1 + mapping.policies.graphElement.getStrokeWidth();
			mapping.getGraphContainerPolicy().translateToRelative(dim);
		}
		
		glow.setStroke({
			color: this.colorStart,
			width: dim.width,
			cap: "round",
			join: "round"
		});
		glow.rawNode.setAttribute("pointer-events", "none");
		return glow;
	},
	
	destroy: function() {
		var fadeOut = dojox.gfx.fx.animateStroke({
			shape: this.shape,
            color : {start: this.colorStart, end : this.colorEnd},
            duration : 300
        });
		dojo.connect(fadeOut, "onEnd", function() {
			this.shape.removeShape();
		});
		fadeOut.play();
	}
});