/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.figures.PolylineSelection");

dojo.require("rmpw.diagrams.render.util.ColorConstants");

/**
 * A polyline selection figure with endpoint handles.
 * Meant to be used on the front feedback layer.
 * 
 * @author Steve Shaw, Marc Gobeil
 */
dojo.declare("rmpw.diagrams.edit.tools.figures.PolylineSelection", null, {
	foregroundColor: rmpw.diagrams.render.util.ColorConstants.SELECTION,

	constructor: function(feedbackLayer, target, editController) {
		this._createFigure(feedbackLayer, target, editController);
	},
	
	destroy: function() {
		this.shape.removeShape();
	},
	
	getFeedbackPoints: function() {
		var mapping = this.editController.renderMapping;
		var points = mapping.policies.graphElement.getPointList(false).getCopy();
		mapping.policies.graphElement.translateToAbsolute(points);
		return points.points;
	},
	
	_createFigure: function(feedbackLayer, target, editController) {
		var shape = feedbackLayer.createGroup();
		var targetShape = target.shape;
		
		var offset = 1;
		if(targetShape.strokeStyle) {
			offset = targetShape.strokeStyle.width / 2;
		}
		var points = this.getFeedbackPoints();
		for (var i = 0, len = points.length; i < len; i++) {
			var bendpt = points[i];
			this._createHandle(shape, bendpt.x - offset, bendpt.y - offset);
		}
		
		this.shape = shape;
	},
		
	_createLine: function(parentShape, x1, y1, x2, y2) {
		parentShape.createLine({x1: x1, y1: y1, x2: x2, y2: y2})
			.setStroke({color: this.foregroundColor, width: 1, style: "solid"});
	},
	
	_createHandle: function(parentShape, x, y) {
		parentShape.createCircle({r: 3.5})
			.setTransform({dx: x, dy: y})
			.setStroke({color: "#FFFFFF", width: 1, style: "solid"})
			.setFill(this.foregroundColor);
	}	
	
});