/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.figures.PolylineSelectionWithHandles");

dojo.require("rmpw.diagrams.edit.tools.figures.PolylineFeedback");
dojo.require("rmpw.diagrams.edit.tools.figures.PolylineSelection");
dojo.require("rmpw.diagrams.render.util.LineSeg");
dojo.require("rmpw.diagrams.render.util.Point");
dojo.require("rmpw.diagrams.render.util.Ray");
dojo.require("rmpw.diagrams.edit.handles.MoveableHandle");
dojo.require("rmpw.diagrams.edit.handles.MoveableEndHandle");

/**
 * Subclass to handle selection of connections.  Handles are added at bendpoint locations.
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.tools.figures.PolylineSelectionWithHandles", rmpw.diagrams.edit.tools.figures.PolylineSelection, {
	
	onPolylineMoving: function(mover, shift) {
	
		this.editController.root.policies.focusScope.disable();
	    var i, lg, handle;
		if (!this.affectedHandles || this.affectedHandles === null) {
			var lineSegIndex = -1;
			var minDistance = 0;
			var mapping = this.editController.renderMapping;
			
			var mousePt = mapping.viewer.viewport.getSurfaceAbsoluteMousePosition(mover.lastX, mover.lastY);
			
			for (i=0, lg=this.handles.length; i<lg - 1; i++) {
				var aSegment = new rmpw.diagrams.render.util.LineSeg(this.handles[i].getPosition(), this.handles[i+1].getPosition());
				var nextDistance = aSegment.distanceToPoint(mousePt.x, mousePt.y);
				
				if (lineSegIndex === -1 || nextDistance < minDistance) {
					lineSegIndex = i;
					minDistance = nextDistance;
				}
			}
			this.affectedHandles = [];
			
			if (mapping.policies.graphEdge.isOrthogonal()) {
				this.affectedHandles.push(this.handles[lineSegIndex]);
				this.affectedHandles.push(this.handles[lineSegIndex+1]);
			}
			else {
				// reuse closet bendpoint?
				var maxBends = this.editController.renderMapping.policies.graphEdge.getMaxBendpoints();
				if (maxBends && maxBends > 0 && (this.handles.length - 2) >= maxBends) {
					
					// find bendpoint closest to mouse
					var closetHandle;
					var lastLen = -1;
					for (i=1, lg=this.handles.length; i<lg - 1; i++) {
					      var ray = new rmpw.diagrams.render.util.Ray(this.handles[i].getPosition(), mousePt);
					      var len = ray.length();
					      if (lastLen === -1 || len < lastLen) {
                              lastLen = len;
                              closetHandle = this.handles[i];
					      }                              
					}
					
					// move the handle to the mouse position
					closetHandle.pos = {x: mousePt.x, y: mousePt.y};
					closetHandle.handleShape.setTransform({dx: mousePt.x, dy: mousePt.y});			
					this.affectedHandles.push(closetHandle);
				
				// else create new bendpoint
				} else {
					var insertedHandle = new rmpw.diagrams.edit.handles.MoveableHandle({
						owner: this.handles[0].handleShape.parent,
						x: mousePt.x,
						y: mousePt.y,
						color: this.foregroundColor,
						editController: this.editController});
					insertedHandle.handleShape.moveToFront();
					this.attachHandleListeners(insertedHandle);
					this.handles.splice(lineSegIndex + 1, 0, insertedHandle);
					for (i=0, handle; handle=this.handles[i]; i++) {
						handle.setIndex(i);
					}
					this.affectedHandles.push(insertedHandle);
				}
			}
			
			for (i=0, lg=this.affectedHandles.length; i<lg; i++) {
				if (this.affectedHandles[i].onFirstMove) {
					this.affectedHandles[i].onFirstMove(mover,true);
				}
			}
		}
		
		lg=this.affectedHandles.length;
		var modShift = {dx: shift.dx, dy: shift.dy};
		if (lg === 2) {
			if (this.affectedHandles[0].getPosition().x === this.affectedHandles[1].getPosition().x) {
				modShift.dy = 0;
			}
			else {
				modShift.dx = 0;
			}
		}
		
		for (i=0; i<lg; i++) {
			if (this.affectedHandles[i].onMove) {
				this.affectedHandles[i].onMove(mover, modShift);	
				this.affectedHandles[i].handleShape.applyLeftTransform(modShift);
			}
		}
	},
	
	onPolylineMoveStop: function(e) {
		this.editController.root.policies.focusScope.enable();
		if (this.affectedHandles) {
			for (var i=0, lg=this.affectedHandles.length; i<lg; i++) {
				if (this.affectedHandles[i].onMoveStop) {
					this.affectedHandles[i].onMoveStop(e);
				}
			}
			this.affectedHandles = null;
		}
	},
	
	attachHandleListeners: function(handle) {
		if (!this.moveableListeners) {
			this.moveableListeners = [];
		}
		this.moveableListeners.push(dojo.connect(handle, "onMove", this, "updateConnectionLines"));
		this.moveableListeners.push(dojo.connect(handle, "onMoveStop", this, "executeUpdateBendpointsCommand"));
	},
	
	detachListeners: function() {
		if (this.moveableListeners) {
			dojo.forEach(this.moveableListeners, dojo.disconnect);
		}
		this.moveableListeners = null;
	},
	
	updateConnectionLines: function() {
		return this.connectionCreationFeedback.updateConnectionLines();
	},
	
	_onReorientSourceRequested: function(/* {targetMapping: mapping object} */ args){	
		if (this.editController.policies.reorientable) {
			return this.editController.policies.reorientable.doReorient(args.newSourceMapping, null);
		}
		
		return false;
	},
	
	_onReorientTargetRequested: function(/* {targetMapping: mapping object} */ args){	
		if (this.editController.policies.reorientable) {
			return this.editController.policies.reorientable.doReorient(null, args.newTargetMapping);
		}
		
		return false;
	},
	
	executeUpdateBendpointsCommand: function() {
		var mapping = this.editController.renderMapping;
		
		var points = this.updateConnectionLines().getCopy();
		mapping = this.editController.renderMapping;
		mapping.policies.graphElement.translateToRelative(points);
		
		// determine if reorient occurred - otherwise just adjust the bendpoints
		var success = true;
		if (this.handles[0].targetMapping &&
			this.handles[0].targetMapping.renderMapping !== mapping.policies.graphEdge.sourceMapping) {
			// source has been reoriented
			success = this._onReorientSourceRequested( { newSourceMapping: this.handles[0].targetMapping } );
		}
		else if (this.handles[this.handles.length - 1].targetMapping &&
			this.handles[this.handles.length - 1].targetMapping.renderMapping !== mapping.policies.graphEdge.targetMapping) {
			// target has been reoriented
			success = this._onReorientTargetRequested( { newTargetMapping: this.handles[this.handles.length - 1].targetMapping } );
		}
		else {
			var graphEdge = mapping.policies.graphEdge;
			if (!graphEdge) {
				return;
			}
			
			if (graphEdge.getPointList().equals(points)) {
				return;
			}
			
			mapping.viewer.getCommandController().execute("rmpw.diagrams.edit.commands.SetConnectionBendpointsCommand", {
				editController: this.editController,
				points: points.points, 
				sourceAnchor: this.connectionCreationFeedback.getStartAnchor(), 
				targetAnchor: this.connectionCreationFeedback.getEndAnchor()
			});
		}
		
		if (!success) {
			this.editController.policies.selectable.hideFeedback();
			this.editController.policies.selectable.showFeedback();
		}
		else {
			this.destroy();
		}
	},
	
	destroy: function() {
		this.inherited(arguments);
		
		if (this.connectionCreationFeedback) {
			this.connectionCreationFeedback.destroy();
			delete this.connectionCreationFeedback;
		}
		
		for (var i=0, handle; handle=this.handles[i]; i++) {
			this.handles[i].destroy();
		}
		this.handles = [];
		
		this.detachListeners();
	},
	
	_createFigure: function(feedbackLayer, target, editController) {
		var mapping = editController.renderMapping;
		this.editController = editController;
		
		this.handles = [];
		
		var shape = feedbackLayer.createGroup();
		var points = this.getFeedbackPoints();
		for (var i = 0, lg = points.length; i < lg; i++) {
			var bendpt = points[i];
			var handle = null;
			
			if (i===0) {
				handle = new rmpw.diagrams.edit.handles.MoveableEndHandle(
						{ owner: shape, x: bendpt.x, y: bendpt.y, color: this.foregroundColor, 
							editController: this.editController, 
							anchor: mapping.policies.graphEdge.getSourceConnectionAnchor().getCopy(), source: true,
							isAnchorToMappingValid: dojo.hitch(this, function(sourceMapping) {
								if (this.editController.policies.reorientable) {
									return this.editController.policies.reorientable.enableReorient( sourceMapping, null );
								}
							 })});
			}
			else if (i===lg-1) {
				handle = new rmpw.diagrams.edit.handles.MoveableEndHandle(
						{ owner: shape, x: bendpt.x, y: bendpt.y, color: this.foregroundColor, 
						editController: this.editController, 
						anchor: mapping.policies.graphEdge.getTargetConnectionAnchor().getCopy(), source: false,
						isAnchorToMappingValid: dojo.hitch(this, function(targetMapping) {
							if (this.editController.policies.reorientable) {
								return this.editController.policies.reorientable.enableReorient( null, targetMapping );
							}
							
							return false;
						})});
			} else {
				handle = new rmpw.diagrams.edit.handles.MoveableHandle(
						{ owner: shape, x: bendpt.x, y: bendpt.y, color: this.foregroundColor, 
						  editController: this.editController, index: i});
			}

			this.attachHandleListeners(handle);
			this.handles.push(handle);
		}

		this.connectionCreationFeedback = 
			new rmpw.diagrams.edit.tools.figures.PolylineFeedback(
					this.handles, mapping, mapping.policies.graphElement.getGraphContainer(), mapping.policies.graphEdge.getRouterType(), 
					shape, this.foregroundColor);
		this.shape = shape;
	}
});