/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.figures.RectangularHighlight");

dojo.require("rmpw.diagrams.render.util.Workarounds");
dojo.require("dojox.gfx.fx");
dojo.require("rmpw.diagrams.edit.tools.figures.Highlight");

/**
 * A rectangular highlight that fades out when it's destroyed.
 * Meant to be used on the rear feedback layer.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.diagrams.edit.tools.figures.RectangularHighlight", rmpw.diagrams.edit.tools.figures.Highlight, {
	
	constructor: function(feedbackLayer, target, editController) {
		this.editController = editController;	
		this.feedbackLayer = feedbackLayer;
		this.shape = this.createShape(target);
	},
	
	createShape: function(target) {
		var mapping = this.editController.renderMapping;
		
		var dojobounds = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(
				target, rmpw.diagrams.render.util.Workarounds.IGNORE_DECORATIONS);
		var bounds = new rmpw.diagrams.render.util.Rectangle(dojobounds.x, dojobounds.y, dojobounds.width, dojobounds.height);
		
		var zoom = mapping.root.viewport.getZoom();
		var offset = mapping.root.viewport.getOffset();
		
		bounds.translate(-offset.x, -offset.y);
		bounds.performScale(1/zoom);
		
		mapping.getGraphContainerPolicy().translateToRelative(bounds);
		
		bounds.performScale(zoom);
		bounds.translate(offset.x, offset.y);
		
		var dim = new rmpw.diagrams.render.util.Dimension(this.padding, 0);
		mapping.getGraphContainerPolicy().translateToRelative(dim);
		
		var shape = this.feedbackLayer.createRect({
			width: bounds.width + (2 * dim.width),
			height: bounds.height + (2 * dim.width),
			r: dim.width});
		shape.setFill(this.colorStart);
		shape.setTransform({
			dx: bounds.x - dim.width, 
			dy: bounds.y - dim.width});
		
		shape.rawNode.setAttribute("pointer-events", "none");
		return shape;
	}
});