/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.edit.tools.figures.RectangularResizableSelection");

dojo.require("rmpw.diagrams.render.util.Workarounds");
dojo.require("rmpw.diagrams.edit.handles.HandleUtils" );
dojo.require("rmpw.diagrams.edit.commands.ResizeCommand");
dojo.require("rmpw.diagrams.edit.handles.MoveableHandle");
dojo.require("rmpw.diagrams.edit.tools.figures.AffordanceMoveable");
dojo.require("rmpw.diagrams.render.util.Rectangle");
dojo.require("rmpw.diagrams.render.util.ColorConstants");
dojo.require("rmps.core.error");

/**
 * A rectangular selection figure with corner handles.
 * Meant to be used on the front feedback layer.
 * 
 * @author Anton McConville
 */

dojo.declare("rmpw.diagrams.edit.tools.figures.RectangularResizableSelection", null, {
	
	BORDER_WIDTH: 3,
	HANDLE_WIDTH: 8,
	FOREGROUND_COLOR: rmpw.diagrams.render.util.ColorConstants.SELECTION,
	OUTLINE_COLOR: rmpw.diagrams.render.util.ColorConstants.RESIZE_OUTLINE,
	REGION: rmpw.diagrams.edit.handles.HandleUtils,
	target: null,
	feedbackLayer: null,
	bounds: null,
	
	constructor: function( feedbackLayer, target, mapping ) {
        this.target = target;
        this.mapping = mapping;
        this.feedbackLayer = feedbackLayer;
        var renderMapping = this.mapping.renderMapping;
        if(renderMapping.policies.graphElement) {
            this.bounds = renderMapping.policies.graphElement.getBounds();
            renderMapping.policies.graphElement.translateToAbsolute(this.bounds);
        }
		else {
			this.bounds = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(target);
		}
		this.box = this.boundsCoordinates( this.bounds );
		
		this.selectionBox = feedbackLayer.createGroup();
		this.makeSelectionBox(this.feedbackLayer, this.bounds);   
        
	    
	    this.handleGroup = feedbackLayer.createGroup();
	    
		this.initArrays();
	    this.makeAffordance( this.box );
	},
	
	initArrays: function(){
        this.handles = [];
        this.outlines = [];
	},
	
	boundsCoordinates: function( bounds ){
		
        var box = [];
		
		box[ this.REGION.TOP_LEFT ] = { x: bounds.x -this.BORDER_WIDTH, y: bounds.y -this.BORDER_WIDTH};
		box[ this.REGION.TOP_RIGHT ] = { x: bounds.x + bounds.width + this.BORDER_WIDTH, y: bounds.y - this.BORDER_WIDTH};
		box[ this.REGION.BOTTOM_RIGHT ] = { x: bounds.x + bounds.width + this.BORDER_WIDTH, y: bounds.y + bounds.height +this.BORDER_WIDTH };
		box[ this.REGION.BOTTOM_LEFT ] = { x: bounds.x - this.BORDER_WIDTH, y: bounds.y + bounds.height + this.BORDER_WIDTH}; 
		
		return box;
	},

	replaceHandle: function( element, point ){
		this.handles[element].destroy();
		this.makeHandle( point.x, point.y, element );
	},
	
	replaceSelectionBox: function(feedbackLayer, bounds){
	    this.selectionBox.children[0].removeShape();
	    this.makeSelectionBox(feedbackLayer, bounds);
	},

	removeHandles: function() {
		for( var h in this.handles ){
			this.handles[h].destroy();
		}
		this.handles = [];
	},
	
	remove: function(){
		this.removeHandles();

		for( var o in this.outlines ){
			this.handleGroup.remove( this.outlines[o] );
		}
		
		this.initArrays();
	},

	destroy: function(){		
		this.removeHandles();
		
		this.handleGroup.removeShape();
		this.selectionBox.children[0].removeShape();
	},
	
	makeSelectionBox: function(feedbackLayer, bounds) {
        this.selectionBox.createRect({
            x: 0,
            y: 0,
            width: bounds.width,
            height: bounds.height,
            r: 5
        }).setStroke({
            color: this.FOREGROUND_COLOR,
            width: 2,
            lineStyle: "solid"
        }).setTransform({
            dx: bounds.x, 
            dy: bounds.y});
    },

	makeHandle: function( x, y, id ){
		this.handles[id] = new rmpw.diagrams.edit.handles.MoveableHandle({	focusFeedbackLayer: this.mapping.viewer.rearFeedbackLayer, 
																			owner: this.handleGroup, 
																			x: x,
																			y: y,
																			icon: this._makeIcon(id), 
																			editController: this.mapping,
																			color: this.FOREGROUND_COLOR,
																			moveable: rmpw.diagrams.edit.tools.figures.AffordanceMoveable,
																			affordance: this,
																			id: id } );
	},

	makeOutline: function( point1, point2 ){   
		this.outlines.push( this.handleGroup.createLine( { x1: point1.x, y1: point1.y, x2: point2.x, y2: point2.y } ).setStroke( { color: this.OUTLINE_COLOR, width: 2, style: "Dash"  } ) );    
	},

	makeAffordance: function( box ){

		this.makeOutline( box[this.REGION.TOP_LEFT], box[this.REGION.TOP_RIGHT] );
		this.makeOutline( box[this.REGION.TOP_RIGHT], box[this.REGION.BOTTOM_RIGHT] );
		this.makeOutline( box[this.REGION.BOTTOM_RIGHT], box[this.REGION.BOTTOM_LEFT] );
		this.makeOutline( box[this.REGION.BOTTOM_LEFT], box[this.REGION.TOP_LEFT] ); 

		// create handles last so they are on top of lines so that cursor never drags line
		for( var h in box ){
            this.makeHandle( box[h].x, box[h].y, h ); 
        }


	},
	
	_makeIcon: function( id ){
		var dm = net.jazz.ajax._contextRoot;
	    switch(parseInt(id, 10)){
	    	case this.REGION.TOP_LEFT:
	    		return { x:-1, y:-1, width:9, height:9, src:dm+"/web/rmpw.diagrams.edit/handles/resizeHandleNW.png" };
	    	case this.REGION.TOP_RIGHT:
	    		return { x:-8, y:-1, width:9, height:9, src:dm+"/web/rmpw.diagrams.edit/handles/resizeHandleNE.png" };
	    	case this.REGION.BOTTOM_RIGHT:
	    		return { x:-8, y:-8 ,width:9, height:9, src:dm+"/web/rmpw.diagrams.edit/handles/resizeHandleSE.png" };
            case this.REGION.BOTTOM_LEFT:
                return { x:-1, y:-8, width:9, height:9, src:dm+"/web/rmpw.diagrams.edit/handles/resizeHandleSW.png" };
            default:
               console.warn("illegal handle id: " + id);
	    }            
	},

	reshapeDrawing: function( newBox ){
		
		var nodes = [];
		nodes.push(this.mapping);
		
		// some browser don't like negative width/height
		if (newBox.width < 0) {
			newBox.x += newBox.width;
			newBox.width = - newBox.width;
		}
		if (newBox.height < 0) {
			newBox.y += newBox.height;
			newBox.height = - newBox.height;
		}

		var relBox = new rmpw.diagrams.render.util.Rectangle(newBox.x, newBox.y, newBox.width, newBox.height);
		this.mapping.renderMapping.policies.graphElement.translateToRelative(relBox);
		this.mapping.viewer.getCommandController().execute("rmpw.diagrams.edit.commands.ResizeCommand", {
			bounds: relBox, nodes: (newBox.selection ? this.mapping.viewer.getSelectionController().getSelection() : nodes) 
		});
		
		// invalidate container
		if (this.mapping.parent) {
			this.mapping.parent.renderMapping.invalidate();
		}
	},
	
	sendData: function(){}
});