/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.tools.figures.RectangularSelection");

dojo.require("rmpw.diagrams.render.util.Workarounds");
dojo.require("rmpw.diagrams.render.util.ColorConstants");

/**
 * A rectangular selection figure.
 * Meant to be used on the front feedback layer.
 * 
 * @author Christian Vogt, Marc Gobeil
 */
dojo.declare("rmpw.diagrams.edit.tools.figures.RectangularSelection", null, {
	foregroundColor: rmpw.diagrams.render.util.ColorConstants.SELECTION,
	
	constructor: function(feedbackLayer, target, editController, mode, strokeWidth) {
		var bounds = null;
		
		if (editController.renderMapping.policies.graphElement) {
			bounds = editController.renderMapping.policies.graphElement.getBounds();
			editController.renderMapping.policies.graphElement.translateToAbsolute(bounds);
		}

		if (!bounds) {
			bounds = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(
					target, rmpw.diagrams.render.util.Workarounds.IGNORE_DECORATIONS);
		}
		
		if(mode === "highlight"){
		    this.foregroundColor = rmpw.diagrams.render.util.ColorConstants.HOVER;
		}
		
		this.shape = this._createFigure(feedbackLayer, bounds, strokeWidth);
		this.shape.setTransform({dx: bounds.x, dy: bounds.y});
	},
	
	destroy: function() {
		this.shape.removeShape();
	},

	_createFigure: function(feedbackLayer, bounds, strokeWidth) {
		var group = feedbackLayer.createGroup();
		group.createRect({
			x: 0,
			y: 0,
			width: bounds.width,
			height: bounds.height,
			r: 5
		}).setStroke({
			color: this.foregroundColor,
			width: strokeWidth?strokeWidth:2,
			lineStyle: "solid"
		});
		
		return group;
	}
	

});