/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.common.transform.BoundVariable");

/**
 * Abstract class for a variable which is bound to some value.  Useful to capture some
 * commonality between trace and expression.  
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.common.transform.BoundVariable", null, {
	
	constructor: function() {
		this._initializeValues = true;
	},
	
	/*
	 * Accessor to retrieve the added values for the trace.  Also used to initialize
	 * the added array when the trace is accessed the first time.
	 */
	getAdded: function() {
		if (this._initializeValues) {
			if (this.isCollection()) {
				var initialValues = this.get();
				this.added = initialValues ? dojo.map(initialValues, "return item;") : [];
				this.removed = [];
				this.removedReference = [];
				this._collection = true;
			}
			delete this._initializeValues;
		}
		
		return this.added;
	},
	
	/*
	 * Determines if this variable is a collection
	 */
	isCollection: function() {
		// subclasses should override
		return false;
	},
	
	/**
	 * Returns the current value of the property at the end of the path.
	 */
	get: function() {
		// implemented by subclass
	},
	
	/*
	 * Determines if this expression should re-synced with the runtime policies
	 */
	isSynced: function() {
		// subclasses should override
		return true;
	},
	
	/*
	 * Sets this trace as being synchronized with it's data or not.
	 */
	setSynced: function(bool) {
		// implemented by subclass
	},
	
	/**
	 * Resets all change tracking variables (synced, added, and removed)
	 */
	reset: function(name) {
		this.setSynced(true);
		if(this._collection && this.added && this.removed && this.removedReference) {
			this.added.length = 0;
			this.removed.length = 0;
			this.removedReference.length = 0;
		}
	}
});