/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.common.transform.MappingState");

/*
 * @author sshaw
 * 
 * This class allows for storage of the policy state of a particular mapping when it gets destroyed.
 * This allows for a mapping to be recreated and capture it's state again.  Typically this will occur
 * during an undo / redo scenario.  In future this mechanism might be utilized for notation persistence.
 */
dojo.declare("rmpw.common.transform.MappingState", null , {
	constructor: function(mapping, copy) {
		this.copy = copy;
		
		if (mapping) {
			if (copy) {
				this.state = {};
				
				dojo.map(mapping.policies, dojo.hitch(this, function(policy) {
					dojo.mixin(this.state, policy.getState());
				}));
			}
			else {
				this.state = mapping.sources.__metadata;
			}
			
			this.key = mapping.declaredClass;
			this.root = mapping.root;
		}
	},
	
	/*
	 * Restore the state of a mapping
	 */
	restoreState: function(mapping) {
		
		if (this.copy) {
			if (mapping.root == this.root && this.key == mapping.declaredClass) {
				for (var i=0, policy; policy=mapping.policies[i]; i++) {
					policy.setState(this.state);
				}
				
				return true;
			}
		}
		else {
			mapping.sources.__metadata = this.state;
			return true;
		}
		
		return false;
	}
});