/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.forms.ui.CreatePicker");

dojo.require("dijit._WidgetBase");
dojo.require("dijit._Templated");

dojo.require("dojo.string");
dojo.require("dojo.i18n");

var SelectionsPicker= "rmps.app.pickers.internal.SelectionsPicker";

/* 
 * Picker that only allows for creation of new resources
 * 
 * @author sshaw
*/
dojo.declare("rmpw.forms.ui.CreatePicker",[dijit._Widget, dijit._Templated], {
	
	title: null,
	creationsLabel: null,
	typeLabel: null,
	creationArgs: null,
	messages: null,
	
	templateString: dojo.cache("rmpw.forms", "ui/templates/CreatePicker.html"),
	widgetsInTemplate: true,
	
	_creationWidget: null,
	_existingItem: null,
	_newItem: null,
	
	postMixInProperties: function() {
		dojo.requireLocalization("rmpw.forms.ui", "SelectCreatePickerMessages");
		this.messages = dojo.i18n.getLocalization("rmpw.forms.ui", "SelectCreatePickerMessages");
		
		this._loadingMessage = this.messages.loading;
		
		if (!this.title){
			this.title = this.messages.title;		 	
		 }
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		//Show the contents of the default slection
		this._onSelectNew();
	},
		
	_onSelectNew: function() {
		var self= this;
		this._onSelect(function(){return self._creationWidget}, function(value){self._creationWidget= value}, this.creationArgs, this.creationsLabel);
	},
	
	_onSelect: function(getter, setter, args, label){
		this._removeAllChildren();
		
		var get= getter();
		if(get){
			get.onShow();
			return this._contentArea.addChild(get);
		}
		
		if(dojo.isFunction(args)){
			return args(dojo.hitch(this, function(argsOrWidget){
				if(dojo.isArray(argsOrWidget)){
					return jazz.core.loader.batch_load_async([SelectionsPicker], dojo.hitch(this, function() {
						setter(new dojo.getObject(SelectionsPicker)({
							argsList: argsOrWidget,
							selectionsLabel: label
						}));
						this._contentArea.addChild(getter());
					}));			
				}else if(argsOrWidget instanceof dijit._WidgetBase){
					setter(argsOrWidget);
					return this._contentArea.addChild(getter());
				}
				return rmps.core.error.create("Your creation callback must return a widget ir selection arguments").dispatch();
			}));
		}
		
		jazz.core.loader.batch_load_async([SelectionsPicker], dojo.hitch(this, function() {
			setter(new new dojo.getObject(SelectionsPicker)({
				argsList: args,
				selectionsLabel: label
			}));
			this._contentArea.addChild(getter());
		}));
	},
	
	_removeAllChildren: function(){
		while(this._contentArea.hasChildren()){
			this._contentArea.removeChild(this._contentArea.getChildren()[0]);
		}
	},
	
	destroy: function(){
		this._removeAllChildren();
		if(this._creationWidget){
			this._creationWidget.destroyRecursive();
			delete this._creationWidget;
		}
		this.inherited(arguments);
	}
	
});

})();
