/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("rmps.graphics.ui.DiagramClient");

dojo.require("dojox.xml.parser");

dojo.require("jazz.app.context");

dojo.require("jazz.client.xhr");

dojo.require("rmps.core.RootServices");

var DIAGRAMS_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "diagrams");

var BASELINE_PARAM = "rmps.context";
var CHANGESET_PARAM = "changeset";

function getTileUri(uri, ts) {
	ts = ts || new Date().valueOf(); 
	uri.replace(/#.*?/, '');
	if (uri.indexOf("?") === -1) {
		uri += "?";
	} else {
		uri += "&";
	}
	var changeset = jazz.app.context.get().changeset;
	if (changeset) {
		uri += "changeset=" + changeset + "&";
	}

	return uri + "ts=" + ts;
}

function compatibilityMode(deferred, uri) {
	var img = new Image();
	img.onload = function() {
		deferred.callback([img]);
		img = null;
	};
	img.onerror = function() {
		deferred.errback();
		img = null;
	};
	if (dojo.isIE) {
		img.x = img.y = 0;
	}
	
	var path = getTileUri(uri);
	
	img.src = path;
}

function preventImageCache(tiles) {
	var ts = new Date().valueOf();
	for (var i = 0, tile; tile = tiles[i]; ++i) {
		tile.src = getTileUri(tile.src, ts);
	}
	return tiles;
}

rmps.graphics.ui.DiagramClient = {
	
	getTiles: function(diagramUri, graph, tiled, format) {
		var queryParams = {};
		
		queryParams.diagramUri = diagramUri;
		
		if (graph) {
			queryParams.graph = graph;
		}
		
		if (tiled) {
			queryParams.tiled = tiled;
		}
		
		if (format) {
			queryParams.format = format;
		}
		
		// Baseline, Changeset
		var baseline = jazz.app.context.get().baseline;
		var changeset = jazz.app.context.get().changeset;
		if (baseline) {
			queryParams[BASELINE_PARAM] = baseline;
		}
		if (changeset) {
			queryParams[CHANGESET_PARAM] = changeset;
		}
		
		var uri = DIAGRAMS_PATH + "/image?" + dojo.objectToQuery(queryParams);
				
		var deferred = new dojo.Deferred();
		var args = {
			url: uri,
			headers: {
				"Accept": "application/json",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "json",
			preventCache: true,
			load: function(response, ioArgs) {
				var contentType = ioArgs.xhr.getResponseHeader("Content-Type");
				if (contentType.indexOf("image/") === 0) {
					// compatibility mode
					compatibilityMode(deferred, uri);
					return;
				}
				
				preventImageCache(response.tile);
				deferred.callback(response.tile);
			},
			error: function(e) {
				if (e.status == 406) {
					delete queryParams.tiled;
					uri = DIAGRAMS_PATH + "/image?" + dojo.objectToQuery(queryParams) + "&" + new Date().getTime();
					compatibilityMode(deferred, uri);
					
					console.log( "HANDLING A NOT ACCEPT ERROR" );
					
				} else {
					deferred.errback(e);
				}
			}
		};
				
		jazz.client.xhrGet(args);
				
		deferred.addErrback(function() {			
			return new Error("Unable to load image. Image could be missing or corrupted. Please report this problem to an administrator.");
		});
		return deferred;
	},
	
	getParts: function(diagramUri, graph, tiled, format) {
		var queryParams = {};
		
		queryParams.diagramUri = diagramUri;
		
		if (graph) {
			queryParams.graph = graph;
		}
		
		if (tiled) {
			queryParams.tiled = tiled;
		}
		
		if (format) {
			queryParams.format = format;
		}
		
		// Baseline, Changeset
		var baseline = jazz.app.context.get().baseline;
		var changeset = jazz.app.context.get().changeset;
		if (baseline) {
			queryParams[BASELINE_PARAM] = baseline;
		}
		if (changeset) {
			queryParams[CHANGESET_PARAM] = changeset;
		}
		
		var args = {
			url: DIAGRAMS_PATH + "/parts?" + dojo.objectToQuery(queryParams),
			headers: {
				"Accept": "application/xml",
				"X-ibm-rmps-internal": "true"
			},
			preventCache: true,
			handleAs: "text",
			load: function(response, ioArgs) {
				return dojox.xml.parser.parse(response);
			}
		};
		return jazz.client.xhrGet(args);
	}
};

})();
