/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
 
(function(){

dojo.provide("rmps.graphics.ui.DiagramLoader");

dojo.require("rmps.graphics.ui.DiagramClient");

dojo.require("rmps.graphics.models.RectangleHotspot");
dojo.require("rmps.graphics.models.LineHotspot");
dojo.require("rmps.graphics.models.MultipleHyperlinks");
dojo.require("rmps.graphics.models.Tile");
dojo.require("rmps.graphics.models.Base");

dojo.requireLocalization("rmps.app.ui.internal.editors", "DiagramEditorMessages");

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "DiagramEditorMessages");

dojo.declare("rmps.graphics.ui.DiagramLoader", null, {    	
		
	diagramRoot: null,
	
	images: null,
	
	getDiagramRoot: function(){
		return this.diagramRoot;
	},
	
	getParts: function( assets, graph, parent ){
		
		var partsDeferred = rmps.graphics.ui.DiagramClient.getParts(assets, graph, true);
		partsDeferred.addCallback(this, function(parts) {			
			this.handleParts(parts);				
		});
			
		partsDeferred.addErrback(this, function(error) {
				if (parent.loadingnode) {
					parent.loadingnode.innerHTML = messages.loadingError;
				}
				var action = dojo.string.substitute(messages.error_loadingMetadata_action, [assets]);
				if (!dojo._isDocumentOk(partsDeferred.ioArgs.xhr)) {
					rmps.core.error.createXhrError(partsDeferred.ioArgs, action).dispatch();
				} else {
					rmps.core.error.createError(error, action).dispatch();
				}
		});
	},

	handleImages: function( images ){	
		
		console.log( "HANDLE IMAGES" );
		
		this.diagramRoot = new rmps.graphics.models.Tile( null, "root" );
	
		this.nodes = [];
	
		for( var item =0; item < images.length; item++ ){
				
			this.nodes[item] = new rmps.graphics.models.Tile( this.diagramRoot ); 
			this.nodes[item].image = images[item].src;
			this.nodes[item].width = parseInt( images[item].width );
			this.nodes[item].height = parseInt( images[item].height );
			this.nodes[item].x = parseInt( images[item].x );
			this.nodes[item].y = parseInt( images[item].y );
		}
		
		this.getParts( this.assets, this.graph, this.parent );
	},
	
	makeRectangleHotpsot: function( root, data, part ){
		
		var zone = new rmps.graphics.models.RectangleHotspot( root, data );
		zone.height = parseInt( part.getAttribute( "height" ) );
		zone.width = parseInt( part.getAttribute( "width" ) );
		zone.x = parseInt( part.getAttribute( "x" ) );
		zone.y = parseInt( part.getAttribute( "y" ) );
		
		return zone;
	},
	
	handleParts: function( parts, root ){	
		
		console.log( "HANDLE PARTS" );
		
		this.zonearray = [];	      	       
		var entries = parts.getElementsByTagName( "parts" );
		var partsList = entries[0].childNodes;
		var zone = null;	      
		var part = null;      
	
		for( var lcount = partsList.length-1; lcount >= 0; lcount-- ){
	
			part = partsList[lcount];
	
			if( part.nodeType == 1 ){
	
				var localName = part.localName || part.tagName;
					 
				var zonedata = {};
				
				var view = part.getAttribute( "view" );
				
				zonedata.hasElement = true;
				zonedata.id = part.getAttribute( "element" );
				//use the view if there is no underlying semantic element
				if (zonedata.id == null || zonedata.id == "") {
					zonedata.id = view;
					zonedata.hasElement = false;
				}
				zonedata.hyperlink = part.getAttribute( "hyperlink" );
				zonedata.oneclickHyperlink = part.getAttribute( "oneclickhyperlink" );
				zonedata.type = localName;
				
				if( view != null ){
					zonedata.view = view;
				}
					
				if( localName == "line" ){
						
					var p = part.getElementsByTagName( "point" );

					zone = new rmps.graphics.models.LineHotspot( this.diagramRoot, zonedata );
					zone.points = [];
					
					for( var pcount = 0; pcount< p.length; pcount++ ){
						zone.points.push( parseInt( p[pcount].getAttribute( "x" ) ) );
						zone.points.push( parseInt( p[pcount].getAttribute( "y" ) ) );
					}
	
				}else if( localName == "shape" ){

					var e = part.getElementsByTagName( "urlEntry" );
					
					zone = this.makeRectangleHotpsot(this.diagramRoot, zonedata, part);

					if (e!=null && e.length>0 ) {
						zonedata.hyperlinks = true;

						zonedata.multipleHyperlinks = new rmps.graphics.models.MultipleHyperlinks( zonedata );
						zonedata.multipleHyperlinks.names = [];
						zonedata.multipleHyperlinks.urls = [];
						
						for( var ecount = 0; ecount< e.length; ecount++ ){
							zonedata.multipleHyperlinks.names.push( e[ecount].getAttribute( "urlName" ) );
							zonedata.multipleHyperlinks.urls.push( e[ecount].getAttribute( "urlValue" ) );
						}
					}
	
				}else if ( localName == "graph" ){ 
	
					/* <graph diagramUri="https://localhost:9444/rmps/models/ws_beb8ec9b-183d-4c41-a8cd-87bf7640b5f1_e92281ba-9321-395c-ab27-0074f5e2a3fa#_ILPTkc10Ed6uao6o75eufg" 
		                   graph="_ILPTk810Ed6uao6o75eufg" element="https://localhost:9444/rmps/models/ws_beb8ec9b-183d-4c41-a8cd-87bf7640b5f1_81cfe709-eeaa-3931-9ba2-8f16c0679d1b#_IJmU0M10Ed6uao6o75eufg" 
		                   x="10" y="36" width="266" height="142"/> */
					
					zonedata.graph = part.getAttribute( "graph" );
	
					zone = this.makeRectangleHotpsot( this.diagramRoot, zonedata, part );	

					zone.view = part.getAttribute( "diagramUri" );			     
					zone.graph = zonedata.graph;   
				}
	
				this.zonearray.push( zone );
			}
		}
		
		console.log( "ZONE COUNT: " + this.zonearray.length );
		
		this.parent.drawDiagram( this.diagramRoot );
	},
		
	handleDiagramAssets: function( assets, graph, parent ){
		
			console.log( "HANDLE DIAGRAM ASSETS: " + assets );
			
			this.assets = assets;
			this.graph = graph;
			this.parent = parent;
			
			var deferred = rmps.graphics.ui.DiagramClient.getTiles(assets, graph, true);

			deferred.addCallback( this, this.handleImages );

			deferred.addErrback(this, function(error) {
								
				if (parent.loadingnode) {
					parent.loadingnode.innerHTML = messages.loadingError;
				}
				
				var action = dojo.string.substitute(messages.error_loadingImages_action, [assets]);
				if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
					rmps.core.error.createXhrError(deferred.ioArgs, action).dispatch();
				} else {
					rmps.core.error.createError(error, action).dispatch();
				}
			});
		}
	});

})();