/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmps.diagram.ui.DiagramModelingPage");

dojo.require("rmpw.jazz.ModelingPage");
dojo.require("rmps.diagram.ui.DiagramConfiguration");
dojo.require("rmps.diagram.ui.DiagramWidget");
dojo.require("rmpw.jazz.ui.DrawingTools");
dojo.require("dojo.i18n");

(function(){
	dojo.requireLocalization("rmps.diagram.ui", "Messages");
	var messages = dojo.i18n.getLocalization("rmps.diagram.ui", "Messages");

	var controls = rmpw.jazz.ui.Controls.items;
    dojo.declare("rmps.diagram.ui.DiagramModelingPage", [ dijit._Widget, dijit._Templated ],	{

		tools: [{ name: controls.ZOOM, display: true, enabled: true },
				{ name: controls.UNDO, display: true, enabled: true },
				{ name: controls.REDO, display: true, enabled: true }],
   	
     	 templateString: "<div dojoAttachPoint='containerNode' class=\"rmps-graphics-diagram-layout\" style=\"overflow:hidden;height:100%;\">" + 
     	 					"<div dojoAttachPoint='canvas'/>" +
     	 					"<div dojoAttachPoint='viewableSketches'/>" +
     	 					"<div dojoAttachPoint='editableSketches'/>" +
     	 		         "</div>",
     	 		         
         handler: null,
        
    	 prefix: "rmpw.dsv.editor.domain",
		
		 startup : function() {
			
    		this.inherited(arguments);
    		
    		var resourceInfo = this.editor.editorInput.resourceInfo
			if( resourceInfo ){
				this.fetchDiagram(resourceInfo);
			}
    	 },
    	 
    	 fetchDiagram: function(resourceInfo) {
	 		this.viewer.diagramLayer.viewer.repository = resourceInfo.repository;
	 		this.viewer.diagramLayer.viewer.definition = this.definition;
	 		this.viewer.diagramLayer.viewer.page = this;
	 		var diagramStore = new this.configuration.store( this.viewer.diagramLayer.viewer );
	 		diagramStore.fetch(resourceInfo, this.definition, this.viewer);
    	 },
    	 
    	 /* Hook for being tab content in Design page */
    	 
    	 doSave: function(/*rmps.core.TaskMonitor*/monitor) {
			this.layout.saveDiagram();
    	 },
    	 
    	 /* Hook for being tab content in Design page */
		
    	 doRefresh: function() {
			this.layout.loadDiagramContent(this.editor.editorInput.resourceInfo.uri);
    	 },
		
    	 initializeToolBar: function(
				/* jazz.ui.SimpleToolbar */ toolbar,
				/*function*/setDirty,
				/*function*/enableRefresh) {
    		 
    		// add print button - exclude Ontology diagrams for now - too confusing since multiple diagrams exist
    		if (!this._printButton && 
    			 this.editor.editorInput.resourceInfo.currentResourceInfo.types != "http://www.w3.org/2002/07/owl#Ontology") {
    			this._printButton = toolbar.addButton(this, 
    	    		dojo.hitch(this, this.print), 
    	    		{
    	    			label: messages.viewPrintablePage,
    	    			imgSrc: dojo.moduleUrl("rmps.diagram", "icons/print.gif"),
    	    			index: 0
    	    		}
    	    	);
    		}
    	 },
    	 
    	 /*
    	  * Displays the diagram contents in a full browser page so it can be printed or copied
    	  */
    	 print: function() {
    		 if (this.editor.isDirty()) {
        		 alert(messages.printConfirm);
    		 }
    		 
    		 var diagramImageURL = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/diagrams/image";
    		 diagramImageURL += "?" + rmps.core.util.ContextUtils.computeContextParams(false);
    		 diagramImageURL += "&diagramUri=" + this.editor.editorInput.resourceInfo.uri;
    		 diagramImageURL += "&editorUri=" + this.definition.uri;
				
    		 var diagramSVG = diagramImageURL + "&format=svg,png"; // svg preferred, or png if svg not available (i.e. UML diagrams)
    		 var diagramPNG = diagramImageURL + "&format=png";
    		 
    		 var printDiagram = (dojo.isIE <= 8) ? diagramPNG : diagramSVG;
    		 
    		 var altText = messages.viewPrintablePage;
    		 var win = window.open(printDiagram);		
    	 },
		
		/*
		* Called prior to saving. If a validation error is present
		* which will prevent save from completing then clients should
		* populate push the error reason (String|Error) into the errors array.
		* Client may also push a Deferred object to defer the validation. This allows
		* the client to communicate with the server to perform validation. To add a validation
		* failure the deferred must either perform an error call back or push errors into
		* the errors array.
		*/
		validate: function(/*Array*/errors) {
		
		},
		
	    initializeDiagram: function(){
	    	 	    	 
	    	 this.configuration = new rmps.diagram.ui.DiagramConfiguration(),
	     	 this.commentConfig = new rmps.graphics.ui.Configuration();
	     	 this.selectionConfig = new rmps.graphics.ui.Configuration();
	 
	     	 this.configuration.canvas = this.canvas; 
	     	 this.commentConfig.canvas = this.editableSketches; 
	     	 this.selectionConfig.canvas = this.viewableSketches;  	
	     	
	     	 this.viewer = new rmps.diagram.ui.DiagramWidget({
	 			configuration: this.configuration,
	 			commentConfig: this.commentConfig,
	 			selectionConfig: this.selectionConfig,
	 			commentingWidget: this.editor.commentingWidget,
	 			editor: this,
	 			resourceInfo: this.editor.editorInput.resourceInfo,
				tools : this.tools
	 		 });
	     	 
	     	this.selectionController = this.viewer.sketchLayer.viewer.getSelectionController();
		
	    	this.commandController = this.viewer.sketchLayer.viewer._commandController;	
//			this.commandController.modelingPage = this.viewer.toolsManager;
			
			this.resetOutlineListeners = dojo.connect( this.viewer.diagramLayer.viewer, "setDiagram", this, "setDiagram" );    	
	     },

		setDiagram: function(){
		},
		
	     postCreate: function() {

	    	 this.initializeDiagram();
	    	 this.containerNode.appendChild( this.viewer.domNode );
	    	 
	    	 this.viewer.startup();    		    	 
	     },
		
	     resize: function(size){
	     }
    });
    
})();