/*******************************************************************************
 * Licensed Materials - Property of IBM
 *
 * com.ibm.xtools.rmps.web.diagram
 *
 * Copyright IBM Corporation 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("rmps.graphics.ui.DiagramWidget");

dojo.require("rmps.core.commenting.Comment");
dojo.require("rmps.core.commenting.range.IDList");

dojo.require("rmps.graphics.behavior.HotspotDiagramController");
dojo.require("rmps.graphics.render.HotspotDiagramViewMapping");

dojo.require("rmps.web.comment.sketch.commands.SketchCommandPackage");

dojo.require("rmps.web.comment.sketch.commands.shapes.CreateOval");
dojo.require("rmps.web.comment.sketch.commands.shapes.CreateRectangle");
dojo.require("rmps.web.comment.sketch.commands.shapes.CreateLine");
dojo.require("rmps.web.comment.sketch.commands.shapes.CreatePath");
dojo.require("rmps.web.comment.sketch.commands.shapes.CreateText");

dojo.require("rmps.graphics.ui.LabelWidget");

dojo.require("rmps.web.comment.sketch.behavior.SketchController");
dojo.require("rmps.web.comment.sketch.serializer.Writer");
dojo.require("rmps.web.comment.sketch.serializer.Reader");
dojo.require("rmps.web.comment.sketch.render.SVG");
dojo.require("rmps.web.comment.sketch.render.Group");

dojo.require("rmpw.jazz.ui.RootLayer");

dojo.require("rmps.graphics.ui.DiagramLoader");
dojo.require("rmpw.diagrams.edit.actions.DeleteElement");
dojo.require("rmpw.diagrams.edit.actionBindings.KeyStroke");

dojo.require("rmpw.jazz.ui.DiagramWidget");
dojo.require("rmpw.widgets.slider.HorizontalSlider");

dojo.require("rmps.app.core.util");

dojo.require("rmpw.common.vcs.Repository");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.graphics.ui", "DiagramEditorMessages");

var messages = dojo.i18n.getLocalization("rmps.graphics.ui", "DiagramEditorMessages");

var CoreUtil = rmps.app.core.util;

dojo.declare("rmps.graphics.ui.DiagramWidget", [rmpw.jazz.ui.DiagramWidget], {

	templateString: '<div style="overflow: hidden; position: relative;" dojoAttachPoint="mainNode">' +
						'<div dojoAttachPoint="toolsNode" class="rmps-graphics-diagram-layout rmpsToolBar"></div>' +
						'<div dojoAttachpoint="borderNode" style="width:100%;border-top:1px solid #E8E8E8;margin-top:1px;padding-top:1px;"</div>' +
						'<div dojoAttachPoint="containerNode" style="overflow: hidden;"></div>' +
					'</div>',

	nodes: null,
	commenting: false,
	selected: false,
	loaded: false,
	preload: true,

	sketchRoot: null,

	glass: null,
	glazed: false,
	glassGroup: null,
	glassColor: new dojo.Color("#DAEEFF"),

	listenState: true,

	comments: null,

	ready: false,

	mobile: true,

	createRootLayer: function() {
		return new rmpw.jazz.ui.RootLayer({
			viewer: this
		});
	},

	postCreate: function() {
		this.mobile = rmps.app.core.util.checkIfMobile();

		this.addToolbar();
		this.addZoom();

		this.rootLayer = this._createRootLayer();
		this._createLayers(this.rootLayer);
		this.containerNode.appendChild(this.rootLayer.domNode);

		this.rootLayer._onMoveStop = function() {
			this._moving = false;
			this._setDefaultCursor();
		};
	},

	addTool: function(item) {
		this.toolbar.addChild(item);
	},

	addZoom: function() {
		this.slider = new rmpw.widgets.slider.HorizontalSlider({
				name: "slider",
				value: 1,
				minimum: 0.20,
				maximum: 4,
				intermediateChanges: true,
				style: "width:100px;vertical-align:middle;top:5px;",
				discreteValues: 20, // 20% - 400% at 20% intervals
				showButtons: false,
				handleSrc: dojo.moduleUrl("rmps.graphics", "images/etool16/SliderHandle.png")
		});

		this.configuration.slider = this.slider;
		this.label = new rmps.graphics.ui.LabelWidget();
		this.percent = this.addButton(messages.reset, dojo.hitch(this.editor, "resetZoom"), "actualIcon", false);
	},

	addToolbar: function() {
		this.toolbar = new dijit.Toolbar({}, this.toolsNode);
		dojo.addClass(this.toolbar.domNode, 'rmpsToolbar');
		dojo.removeClass(this.toolbar.domNode, 'dijitToolbar');
		this.configuration.toolbarHeight = "29px";
		this.toolbar.domNode.style.height = this.configuration.toolbarHeight;
		this.toolbar.domNode.style.background = "none repeat scroll 0 0 white";
		this.toolbar.domNode.style.marginTop = "1px";
		this.toolbar.domNode.style.borderBottom = "none";

		this.layout = {};
		this.layout.toolbar = this.toolbar;
	},

	_createLayers: function(rootLayer) {
		// tools
		this.toolsLayer = new rmpw.panner.ToolsLayer();

		this.configuration.useDragPan = true;
		this.commentConfig.useDragPan = true;
		this.selectionConfig.useDragPan = true;

		var diagramLayer = this.diagramLayer = new rmpw.jazz.ui.ModelDiagramLayer({ configuration: this.configuration, resourceInfo: this.resourceInfo });
		var sketchLayer = this.sketchLayer = new rmpw.jazz.ui.ModelDiagramLayer({ configuration: this.commentConfig, resourceInfo: this.resourceInfo });
		var archiveLayer = this.archiveLayer = new rmpw.jazz.ui.ModelDiagramLayer({ configuration: this.selectionConfig, resourceInfo: this.resourceInfo });
		sketchLayer.outline = archiveLayer.outline = diagramLayer.outline = this.outline = new rmpw.jazz.ui.ModelDiagramOutline({pannableLayer: this.rootLayer});

		// HACK (MG): when one view changes, sync the other two
		var sketchViewport = this.sketchLayer.viewer.viewport;
		dojo.connect(sketchViewport, "onViewportChanged", function() {
			var t = sketchViewport._getContentsTransform();
			diagramLayer.viewer.viewport.contents.setTransform(t);
			archiveLayer.viewer.viewport.contents.setTransform(t);
		});
		var diagramViewport = this.diagramLayer.viewer.viewport;
		dojo.connect(diagramViewport, "onViewportChanged", function() {
			var t = diagramViewport._getContentsTransform();
			sketchLayer.viewer.viewport.contents.setTransform(t);
			archiveLayer.viewer.viewport.contents.setTransform(t);
		});
		var archiveViewport = this.archiveLayer.viewer.viewport;
		dojo.connect(archiveViewport, "onViewportChanged", function() {
			var t = archiveViewport._getContentsTransform();
			sketchLayer.viewer.viewport.contents.setTransform(t);
			diagramLayer.viewer.viewport.contents.setTransform(t);
		});
		// END HACK /////////////

		this.addTools(rootLayer, this.toolsLayer);

		rootLayer.addLayer("DIAGRAM_LAYER", this.diagramLayer);
		rootLayer.addLayer("ARCHIVE_LAYER", this.archiveLayer);
		rootLayer.addLayer("SKETCH_LAYER", this.sketchLayer);
		rootLayer.addLayer("OUTLINE_LAYER", this.outline);
		rootLayer.addLayer("TOOLS_LAYER", this.toolsLayer);

		if (this.graph) {
			this._crumbsBar = dojo.doc.createElement("div");
			this._crumbsBar.style.padding = "5px 10px 0";
			this._crumbsBar.style.background = "white";

			this.containerNode.appendChild(this._crumbsBar);

			var crumbs = dojo.doc.createElement("span") ;
			dojo.addClass(crumbs, "jazz-ui-Breadcrumbs breadCrumbs");

			this._crumbsBar.appendChild(crumbs);

			var length = this.history.length;

			for (var h = 0; h < length; h++) {
				var link = dojo.doc.createElement("a");

				if (h == 0) {
					link.href = CoreUtil.getViewResourceUri(this.resourceUri);
					link.innerHTML = "Parent Container > ";
				} else {
					var suffix = encodeURIComponent(this.history[h]);
					var uri = CoreUtil.getViewResourceUri(this.resourceUri) + "&graph=" + suffix;
					link.href = uri;
					link.innerHTML = "Compartment " + h + " > ";
				}

				crumbs.appendChild(link);
			}
		}

		dojo.style(this.archiveLayer.domNode, "hidden");
		dojo.style(this.sketchLayer.domNode, "hidden");
		dojo.style(this.diagramLayer.domNode, "hidden");

		this.resize();
		this.rootLayer.setFocusController(this.diagramLayer.viewer.policies.focusScope);
		this.rootLayer.focusController.enable();
		this.resetOutlineListeners = dojo.connect(this.diagramLayer.viewer, "setDiagram", this, "resetOutline");
	},

	connectCommenting: function(action, method) {
		this.connect(this.commentingWidget, action, method);
	},

	subscribe: function() {
		this.connectCommenting("onSelectionChanged", this.commentSelected);
		this.connectCommenting("onCommentCreated", this.commentCreated);
		this.connectCommenting("onCommentUpdated", this.commentUpdated);
		this.connectCommenting("onStartCreateComment", this.beginComment);
		this.connectCommenting("onStartEditComment", this.beginEditComment);
		this.connectCommenting("onEndEditingComment", this.endComment);
		this.connectCommenting("onCancelEditingComment", this.cancelComment);
		this.connectCommenting("onLinkWithEditor", this.linkEdtitor);
	},

	showLoadingMessage: function() {
		if (this.loaded == false) {
			if (this.loadingnode == null) {
				this.loadingnode = document.createElement("div");

				this.loadingnode.style.padding = "10px";
				this.loadingnode.style.position = "absolute";
				this.loadingnode.style.color = "#BBBBBB";
				this.loadingnode.style.fontSize = "2em";
				this.loadingnode.innerHTML = "Loading...";

				this.domNode.appendChild(this.loadingnode);
			}

			this.loadingnode.style.top = Math.floor((this.rootSize.h / 2) - (this.loadingnode.offsetHeight / 2)) + "px";
			this.loadingnode.style.left = Math.floor((this.rootSize.w / 2) - (this.loadingnode.offsetWidth / 2)) + "px";
		}
	},

	hideLoadingMessage: function() {
		if (this.loadingnode != null) {
			this.domNode.removeChild(this.loadingnode);
			delete this.loadingnode;
		}

		this.loaded = true;
	},

	getPadding: function() {
		var padding = 29;
		return padding;
	},

	resize: function(size) {
		if (size != null) {
			var marginBox = dojo.marginBox(this.editor.domNode);

			// For some reason width is coming in as zero in IE7
			size.w = marginBox.w;

			this.rootSize = {};
			this.rootSize.w = size.w;

			var padding = this.getPadding();

			if (size.h != null && (size.h - padding > 0)) {
				this.rootSize.h = size.h -padding;

				if (dojo.isIE == 7) {
					size.h = size.h - 5; // Why IE7!? Why.
				}
			}

			dojo.marginBox(this.domNode, size);

			this.rootLayer.resize(this.rootSize);

			this.showLoadingMessage();

			var glassSize = this.getGlassSize();

			if (this.glazed == true) {
				this.glass.setShape({ x: 0,  y: 0, width: glassSize.w, height: glassSize.h });
			}
		}
	},

	deglaze: function(layer) {
		if (this.glass != null) {
			this.glassGroup.remove(this.glass);
			this.glass = null;
			this.glazed = false;
		}

		dojo.style(layer.domNode, "visibility", "hidden");
	},

	getGlassSize: function() {
		var size = dojo.marginBox(this.editor.domNode);
		var padding = this.getPadding();
		size.h = size.h - padding;

		return size;
	},

	glaze: function(layer) {
		if (this.surface != null && this.glazed == false) {
			var size = this.getGlassSize();
			this.glass = this.glassGroup.createRect({ x: 0,  y: 0, width: size.w, height: size.h });
			this.glass.setFill(this.glassColor);
			this.glazed = true;
		}

		dojo.style(layer.domNode, "visibility", "");
	},

	addTools: function(rootLayer, toolsLayer) {
		var valueChange = false;
		this.slider = this.configuration.slider;
		var percent = this.label;

		if (this.slider != null) {
			this.slider.onChange = dojo.hitch(this, function(value) {
				valueChange = true;
				rootLayer.setScale(value, null, null, true);
				valueChange = false;
				var val = Math.round(100 * value);
				var label = val + "%";
				percent.setLabel(label);
			});

			this.connect(rootLayer, "setScale", function(scale) { this.slider.set("value", scale); });
		}
	},

	makeSketcher: function(root, viewer, mode) {
		var blank = new rmps.web.comment.sketch.render.Group();

		root = new rmps.web.comment.sketch.render.SVG();
		root.setG(blank);

		root.setGraphicsParent(viewer.viewport.contents);

		viewer.setDiagram(root);
		viewer.commandPackage = new rmps.web.comment.sketch.commands.SketchCommandPackage();
		var myviewer = viewer;

		viewer.__editMode = mode;
		var controller = new rmps.web.comment.sketch.behavior.SketchController(root);
		var mycontroller = controller;

		mycontroller.setParent(myviewer);

		mycontroller._connectConstruction = dojo.connect(root, "applyModelChanges", this, function() {
			dojo.disconnect(mycontroller._connectConstruction);
			delete mycontroller._connectConstruction;
		});

		if (mode == "EDIT") {
			dojo.disconnect(this.editor.connection);

			this.editor.selectionController = viewer.getSelectionController();
			this.connection = dojo.connect(this.editor.selectionController, "onSelectionChanged", this.editor, "selectionChanged");

			this.editor.commandController = viewer._commandController;
			this.editor.commandController.modelingPage = this.editor;
		}

		return { controller: controller, root: root };
	},

	beginComment: function(comment) {
		if (this.ready == true) {
			this.commenting = true;
			this.glaze(this.sketchLayer);
			this.editor.addDrawingTools();
		}
	},

	getBounds: function() {
		var bounds = {};
		bounds.x = 0;
		bounds.y = 0;
		return bounds;
	},

	clear: function() {
		if (this.archiveViewer) {
			var mappings = this.archiveController.root.diagram.__editController.children;
			var deleteMappings = [];
			for (var item = 0, editController; editController = mappings[item]; item++) {
				deleteMappings.push(editController);
			}
			for (var item = 0, editController; editController = deleteMappings[item]; item++) {
				editController.policies.clearAction.run({targetController: editController});
			}
		}
	},

	initializeCommands: function() {
		this.commands = [];
		this.commands.Rectangle = rmps.web.comment.sketch.commands.shapes.CreateRectangle;
		this.commands.Oval = rmps.web.comment.sketch.commands.shapes.CreateOval;
		this.commands.Line = rmps.web.comment.sketch.commands.shapes.CreateLine;
		this.commands.Scribble = rmps.web.comment.sketch.commands.shapes.CreatePath;
		this.commands.Text = rmps.web.comment.sketch.commands.shapes.CreateText;
	},

	createShape: function(controller, item) {
		var data = dojo.clone(item);
		data.svgParent = controller.renderMapping.sources.svgRoot;
		data.viewport = controller.viewer.viewport;

		this.initializeCommands();

		controller.viewer.getCommandController().commandStack.execute(new this.commands[ data.shape ](), data);
	},

	deserialize: function(controller, svgdata) {
		this.groups = [];
		this.svgdata = svgdata;
		this.reader = new rmps.web.comment.sketch.serializer.Reader();
		this.reader.read(this.svgdata, this.getBounds());

		for (var s = 0; s < this.reader.scribbles.length; s++) {
			this.createShape(controller, this.reader.scribbles[s]);
		}
	},

	selectHotspots: function(comment) {
		if (comment.range && this.viewer.root.diagram.__editController && this.viewer.root.diagram.__editController.children) {
			var range = comment.range;
			var children = this.viewer.root.diagram.__editController.children;
			for (var item = 0; item < range.length; item++) {
				for (var id = 0; id < range[item].ids.length; id++) {
					for (var child = 0; child < children.length; child++) {
						var checkId = children[child].renderMapping.sources.model.zonedata.id;

						if (this.graph) {
							checkId = checkId + "&graph=" + this.graph;
						}

						if (range[item].ids[id] == checkId) {
							this.viewer.getSelectionController().select(children[child]);
						}
					}
				}
			}
		}
	},

	deselectHotSpots: function() {
		this.viewer.getSelectionController().deselectAll();
	},

	commentSelected: function(sel, previous, comments, deselected) {
		if (comments && this.ready == true) {
			console.log("COMMENT SELECTED");

			this.currentComments = comments;

			if (this.archiveController) {
				this.deselectHotSpots();

				var commentCount = comments.length;

				if (commentCount > 0) {
					this.clear();

					var scribbles = [];

					this.glaze(this.archiveLayer);

					for (var i = 0; i < commentCount; ++i) {
						var comment = comments[i];

						this.selectHotspots(comment);

						var body = comment.getBody(rmps.core.commenting.BodyType.SKETCH.value);

						if (body != null && body.contents != null) {
							this.deserialize(this.archiveController, body.contents);
							if (dojo.isArray(this.reader.scribbles)) {
								scribbles.push(this.reader.scribbles);
							}
						}
					}

					/* Only glaze the layer if there is at least one comment shape present */
					if (scribbles.length > 0) {
						this.glaze(this.archiveLayer);
					} else {
						/* Required for a scenario like: Two comments are selected. One does not have any shapes, and the other one has some shapes created.
						 * If we deslected the one that has some shapes, then we will need to deglaze the current layer, as the only select comment will not
						 * be having any shapes to be shown */
						this.deglaze(this.archiveLayer);
					}

					this.selected = true;
				} else {
					this.clear();

					if (this.commenting == false) {
						this.deglaze(this.archiveLayer);
					}

					this.selected = false;
				}
			} else {
				this.comments = comments;
			}
		}
	},

	commentCreated: function(comment) {

	},

	commentUpdated: function(comment) {
		// FIXME: The only mandatory parameter is the comments array.
		// If we want to get all the parameters set then we need to call
		// this.commentingWidget.selectComments(comment);
		this.commentSelected(null, null, [comment], null);
	},

	beginEditComment: function() {
		console.log("BEGIN EDIT");

		if (this.ready == true) {
			this.clear();

			this.deglaze(this.archiveLayer);

			this.deselectHotSpots();

			var comments = this.currentComments;

			var commentCount = comments.length;

			if (commentCount > 0) {
				this.glaze(this.sketchLayer);
				this.editor.addDrawingTools();

				for (var i = 0; i < commentCount; ++i) {
					var comment = comments[i];

					this.selectHotspots(comment);

					var body = comment.getBody(rmps.core.commenting.BodyType.SKETCH.value);

					if (body != null && body.contents != null) {
							this.deserialize(this.sketchController, body.contents);
					}
				}
			}
		}
	},

	endComment: function(comment) {
		var selectionController = this.viewer.getSelectionController();
		var selection = selectionController.getSelection();

		console.log("SELECTED ELEMENTS: " + selection.length);

		var elements = [];

		if (selection) {
			for (var item in selection) {
				if (selection[item].renderMapping && selection[item].renderMapping.sources.model.zonedata) {
					elements.push(selection[item].renderMapping.sources.model.zonedata.id);
				} else {
					console.log("Saving comment: no selection render map");
				}
			}
		}

		var range = new rmps.core.commenting.range.IDList(elements);

		if (this.graph && range.ids) {
			for (var r = 0; r < range.ids.length; r++) {
				range.ids[r] = range.ids[r] + "&graph=" + this.graph;
			}
		}

		comment.range = [ range ];

		var shapes = this.sketchRoot.sources.svgRoot.g;

		if (shapes) {
			var writer = new rmps.web.comment.sketch.serializer.Writer();
			var svgdata = writer.write(shapes);

			if (svgdata != null) {
				var body = { type: rmps.core.commenting.BodyType.SKETCH.value, contents: svgdata };
				comment.addBody(body);
				console.log(svgdata);
			}
		}

		// this.currentComments = comment;

		this.editor.clear();

		this.editor.removeDrawingTools();

		this.deselectHotSpots();

		this.deglaze(this.archiveLayer);
		this.deglaze(this.sketchLayer);

		this.commenting = false;

		/* Reset the undo stack */

		this.resetCommandStack();
	},

	resetCommandStack: function() {
		var repository = new rmpw.common.vcs.Repository();
		/* stack of revisions that delineate user-executed commands, so compound commands can be undone as a unit */

		this.sketchController.viewer._commandController.commandStack.repository = repository;
		this.sketchController.viewer._commandController.commandStack.undoStackPosition = 0;
		this.sketchController.viewer._commandController.commandStack.undoStack = [ repository.getCurrentRevision() ];
	},

	cancelComment: function(comment) {
		this.editor.clear();
		this.deglaze(this.sketchLayer);

		this.commenting = false;

		if (this.selected == true) {
			this.glaze(this.archiveLayer);
		} else {
			this.deglaze(this.archiveLayer);
		}

		this.editor.removeDrawingTools();

		this.deselectHotSpots();

		this.resetCommandStack();
	},

	linkEdtitor: function(isLinked, comments) {
	},

	getCommandController: function(attachPoint) {
		return this.sketchLayer.viewer.getCommandController(attachPoint);
	},

	drawDiagram: function(root) {
		/* Need to analyze why it takes two calls of handleDiagramAssets to
		 * guarantee that the rendering will succeed.
		 */

		this.diagramRoot = root;

		var diagram = new rmps.graphics.render.HotspotDiagramViewMapping();
		diagram.setModel(this.diagramRoot);
		diagram.setParentFigure(this.viewer.viewport.contents);

		this.controller = new rmps.graphics.behavior.HotspotDiagramController(diagram);
		var controller = this.controller;

		this.viewer.setDiagram(diagram);
		var viewer = this.viewer;

		this.controller._connectConstruction = dojo.connect(diagram, "applyModelChanges", this, function() {
			controller.setParent(viewer);
			controller.setRenderMapping(diagram);
			dojo.disconnect(controller._connectConstruction);
			delete controller._connectConstruction;
		});

		this.addControllers();

		this.ready = true;

		// For testing work item 63970: setTimeout(dojo.hitch(this, "addControllers"), 15000) ;
	},

	addControllers: function() {
		var sketcher = this.makeSketcher(this.sketchRoot, this.sketchViewer, true);

		this.sketchController = sketcher.controller;
		this.sketchRoot = sketcher.root;

		var archiver = this.makeSketcher(this.archiveRoot, this.archiveViewer, false);
		this.archiveController = archiver.controller;

		this.hideLoadingMessage();
		dojo.style(this.diagramLayer.domNode, "");
		this.cancelComment();

		console.log("ADDED SKETCH CONTROLLERS");

		if (this.comments != null) {
			this.commentSelected(this.comments);
			this.comments = null;
		}
	},

	addButton: function(label, call, icon, show) {
		if (show == null) { show = false; }

		var button = new dijit.form.Button({
				label: label,
				showLabel: show,
				onClick: call,
				iconClass: icon,
				style: "font-size:10px"});

		return button;
	},

	startup: function() {
		this.inherited(arguments);

		this.viewer = this.diagramLayer.viewer;
		this.sketchViewer = this.sketchLayer.viewer;
		this.archiveViewer = this.archiveLayer.viewer;

		this.surface = this.viewer.surface;

		this.glassColor.a = 0.5;
		this.glassGroup = this.surface.createGroup();

		this.loader = new rmps.graphics.ui.DiagramLoader();

		this.loader.handleDiagramAssets(this.resourceUri, this.graph, this);

		this.subscribe();

		this.initializeCommands();

		if (!this.mobile) {
			this.addTool(this.slider);
		}

		this.label.setLabel("100%");

		this.toolbar.addChild(this.label);

		this.addTool(this.percent);

		this.editor.addButton("Outline", dojo.hitch(this.editor, "toggleOutline"), "outline");
	}
});

})();