/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.jazz.ui.DiagramWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("rmpw.jazz.ui.RootLayer");
dojo.require("rmpw.panner.ToolsLayer");
dojo.require("rmpw.panner.ZoomSlider");

dojo.require("rmpw.jazz.ui.ModelDiagramOutline");
dojo.require("rmpw.jazz.ui.ModelDiagramLayer");

dojo.declare("rmpw.jazz.ui.DiagramWidget", [dijit._Widget, dijit._Templated], {
	
templateString: '<div style="overflow: hidden; position: relative;" dojoAttachPoint="containerNode"></div>',
	
	// mixin
	resourceUri: null,
	
	// properties
	rootLayer: null,
	diagramLayer: null,
	offScreenTracker: null,
	toolsLayer: null,
	
	postCreate: function() {
		
		this.rootLayer = this._createRootLayer();
		this._createLayers(this.rootLayer);
		this.containerNode.appendChild(this.rootLayer.domNode);
		
		this.rootLayer._onMoveStop = function(){
			this._moving = false;
			this._setDefaultCursor();
		};
	},
	
	startup: function() {
		this.rootLayer.startup();
		this.inherited(arguments);
	    this.rootLayer.setLocation(0, 0);
	},
	
	resize: function(size) {
		if (!size) {
			return;
		}
		
		var rootSize = {};
		
		if (size.w) {
			rootSize.w = size.w;
		}
		if (size.h) {
			this.domNode.style.height = size.h + "px";
			rootSize.h = size.h;
		}
		
		this.rootLayer.resize(rootSize);
	},
	
	_createRootLayer: function() {
		return new rmpw.jazz.ui.RootLayer({
			viewer: this
		});		
	},
	
	_createLayers: function(rootLayer) {
		// tools
		this.toolsLayer = new rmpw.panner.ToolsLayer();
		this.diagramLayer = new rmpw.jazz.ui.ModelDiagramLayer( { configuration: this.configuration } );
		this.outline = new rmpw.jazz.ui.ModelDiagramOutline({pannableLayer: this.rootLayer});

		this._createTools(rootLayer, this.toolsLayer);
		
		rootLayer.addLayer("DIAGRAM_LAYER", this.diagramLayer);
		rootLayer.addLayer("OUTLINE_LAYER", this.outline);
		rootLayer.addLayer("TOOLS_LAYER", this.toolsLayer);
		
        this.resize();
        this.rootLayer.setFocusController( this.diagramLayer.viewer.policies.focusScope );
		this.resetOutlineListeners = dojo.connect( this.diagramLayer.viewer, "setDiagram", this, "resetOutline" );
	},
	
	destroy: function() {
		this._detachOutlineListeners();
		
		if (this.resetOutlineListeners) {
			dojo.disconnect(this.resetOutlineListeners);
			delete this.resetOutlineListeners;
		}
		this.inherited(arguments);
	},
	
	/*
	 * Reset the outline view
	 */
	resetOutline: function() {
		this._detachOutlineListeners();
		this._attachOutlineListeners();
	},
	
	/*
	 * Connect to any relevant method calls to listen to incoming events.
	 */
	_attachOutlineListeners: function() {
		if(this._outlineListeners && this._outlineListeners.length != 0) {
			return;
		}
		
		this._outlineListeners = [];
		
		// TODO (MG): this shouldn't depend on the GraphContainer policy, let alone GraphManager.  It should
		// probably be hooked to this.diagramLayer.viewer.getInvalidationController().onInvalidate, but that
		// blows up at the time of this writing.
		if(this.diagramLayer.viewer.diagram.policies.graphContainer) {
			this._outlineListeners.push(dojo.connect(
					this.diagramLayer.viewer.diagram.policies.graphContainer.graphManager, "postRefreshHook",
					this.outline, "updateOutlineShape"));
		}
	},
	
	_detachOutlineListeners: function() {
		if(this._outlineListeners == null) {
			return;
		}

		dojo.forEach(this._outlineListeners, dojo.disconnect);
		this._outlineListeners = null;
	},
	
	_createTools: function(rootLayer, toolsLayer) {
		var valueChange = false;
		
		if( this.configuration.slider != null ){
						
			var percent = this.configuration.label; 
		
			// catch when the slider changes the zoom
			this.configuration.slider.onChange = dojo.hitch(this, function(value) {
				valueChange = true;
				rootLayer.setScale( value, null, null, true );
				valueChange = false;			
				var val = Math.round( 100 * value );				
				var label = val + "%";				
				percent.setLabel( label );
			});
			
			// catch when something else like the WheelZoom edit policy changes the zoom
			var viewport = this.diagramLayer.viewer.viewport;
			dojo.connect(viewport, "onViewportChanged", this, function() {
				var value = viewport.getZoom();
				var val = Math.round( 100 * value );				
				percent.setLabel(val + "%");
			}); 

			this.connect(rootLayer, "setScale", function(scale) {
				if (!valueChange) {
					this.configuration.slider.set("value", scale);
				}
			});
		} else {
		
			this._zoomSlider = new rmpw.panner.ZoomSlider({
				onChange: dojo.hitch(this, function(value) {
					valueChange = true;
					rootLayer.setScale(value);
					valueChange = false;
				})
			});
			this._zoomSlider.domNode.style.width = "150px";
			
			this.connect(rootLayer, "setScale", function(scale) {
				if (!valueChange) {
					this._zoomSlider.slider.attr("value", scale);
				}
			});
			
			toolsLayer.containerNode.appendChild(this._zoomSlider.domNode);
		}
	},
	
	getCommandController: function( attachPoint ){
		return this.diagramLayer.viewer.getCommandController( attachPoint );
	},

	setOutline: function( state ) {	
		if( state == false ){
			this.outline.domNode.style.visibility = "hidden";
			this.outlineVisibility = false;
			this._detachOutlineListeners();
		}else{
			this.outline.domNode.style.visibility = "visible";
			this.outlineVisibility = true;
			this._attachOutlineListeners();
		}	
	},
	
	getRenderer: function(){
	   return this.diagramLayer.viewer;	
	},
	
	refresh: function(){
		this.viewMap.policies.graphContainer.refresh();	
	}
	
});

})();