/*******************************************************************************
 * Licensed Materials - Property of IBM
 *
 * com.ibm.xtools.rmpw.dsv.editor.domain
 *
 * Copyright IBM Corporation 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("rmps.diagram.ui.DiagramWidget");

dojo.require("rmps.core.commenting.Comment");
dojo.require("rmps.core.commenting.range.IDList");

dojo.require("rmps.graphics.behavior.HotspotDiagramController");
dojo.require("rmps.graphics.render.HotspotDiagramViewMapping");

dojo.require("rmps.web.comment.sketch.commands.SketchCommandPackage");

dojo.require("rmps.web.comment.sketch.commands.shapes.CreateOval");
dojo.require("rmps.web.comment.sketch.commands.shapes.CreateRectangle");
dojo.require("rmps.web.comment.sketch.commands.shapes.CreateLine");
dojo.require("rmps.web.comment.sketch.commands.shapes.CreatePath");
dojo.require("rmps.web.comment.sketch.commands.shapes.CreateText");
dojo.require("rmps.web.comment.sketch.commands.shapes.CreateTextBox");

dojo.require("rmps.graphics.ui.LabelWidget");

dojo.require("rmps.web.comment.sketch.behavior.SketchController");
dojo.require("rmps.web.comment.sketch.serializer.Writer");
dojo.require("rmps.web.comment.sketch.serializer.Reader");
dojo.require("rmps.web.comment.sketch.render.SVG");
dojo.require("rmps.web.comment.sketch.render.Group");

dojo.require("rmpw.jazz.ui.RootLayer");

dojo.require("rmps.graphics.ui.DiagramLoader");
dojo.require("rmpw.diagrams.edit.actions.DeleteElement");
dojo.require("rmpw.diagrams.edit.actionBindings.KeyStroke");

dojo.require("rmpw.jazz.ui.DiagramWidget");
dojo.require("rmpw.widgets.slider.HorizontalSlider");

dojo.require("rmps.app.core.util");

dojo.require("rmpw.widgets.Events");
dojo.require("rmpw.jazz.ui.LineStyler")

dojo.require("rmpw.common.vcs.Repository");

dojo.require("rmpw.jazz.ui.DrawingTools");

dojo.require("dojo.i18n");
dojo.require("dojo.window");

dojo.requireLocalization("rmps.graphics.ui", "DiagramEditorMessages");

var messages = dojo.i18n.getLocalization("rmps.graphics.ui", "DiagramEditorMessages");

var CoreUtil = rmps.app.core.util;

dojo.declare("rmps.diagram.ui.DiagramWidget", [rmpw.jazz.ui.DiagramWidget], {

	templateString: '<div style="overflow: hidden; position: relative; height: 100%; width: 100%" dojoAttachPoint="mainNode">' +
						'<div dojoAttachPoint="toolsNode" class="rmps-graphics-diagram-layout rmpsToolBar"></div>' +
						'<div dojoAttachpoint="borderNode" style="width:100%;border-top:1px solid #E8E8E8;margin-top:1px;padding-top:1px;"</div>' +
						'<div dojoAttachPoint="containerNode" style="overflow: hidden;min-height: 150px;"></div>' +
					'</div>',

	nodes: null,
	commenting: false,
	selected: false,
	loaded: false,
	preload: true,

	sketchRoot: null,

	glass: null,
	glazed: false,
	glassGroup: null,
	glassColor: new dojo.Color([ 218, 238, 255, 0.5 ]),

	listenState: true,

	comments: null,

	ready: false,

	mobile: true,

	postCreate: function() {
		this.initialized = false;
		this.mobile = rmps.app.core.util.checkIfMobile();

		this.addToolBar();

		this.rootLayer = new rmpw.jazz.ui.RootLayer({ viewer: this });
		dojo.style(this.rootLayer.domNode, "position", "static");
		this._createLayers(this.rootLayer);
		this.containerNode.appendChild(this.rootLayer.domNode);

		this.rootLayer._onMoveStop = function() {
			this._moving = false;
			this._setDefaultCursor();
		};

		this.topics = rmpw.widgets.Events.topics;

		this.subscriptions = new Array();

		this.toggleButtons = [];

		this.toggleCount = 0;
	},

	/**
	 * Adds a diagram toolbar. It will be controlled by ActionBarScope policy
	 * installed on diagram layer's viewer.
	 */
	addToolBar: function() {
		this.toolbar = new dijit.Toolbar({}, this.toolsNode);
		dojo.addClass(this.toolbar.domNode, 'rmpsToolbar');
		dojo.removeClass(this.toolbar.domNode, 'dijitToolbar');
		this.configuration.toolbarHeight = "29px";
		this.toolbar.domNode.style.height = this.configuration.toolbarHeight;
		this.toolbar.domNode.style.background = "none repeat scroll 0 0 white";
		this.toolbar.domNode.style.marginTop = "1px";
		this.toolbar.domNode.style.borderBottom = "none";

		this.layout = new Object();
		this.layout.toolbar = this.toolbar;
	},

	_createLayers: function(rootLayer) {
		// tools
		this.toolsLayer = new rmpw.panner.ToolsLayer();

		this.configuration.useDragPan = true;
		this.commentConfig.useDragPan = true;
		this.selectionConfig.useDragPan = true;

		// add info needed for diagram toolbar to configuration
		this.configuration.toolbar = this.toolbar;
		this.configuration.tools = this.tools;
		this.configuration.rootLayer = this.rootLayer;
		
		var diagramLayer = null;
		/** bIsUsingFixedDiagram is used to distinguish between using old vs. new diagram navigation behaviors.
			the new behavior was created for WI 42459: "Navigation of diagrams in Web UI in DesignManager is not user friendly."
			it is currently only for diagrams that have a fixed size **/
		var bIsUsingFixedDiagram = (this.editor.definition.id == "DiagramEditor");
		
		this.outline = new rmpw.jazz.ui.ModelDiagramOutline({ pannableLayer: this.rootLayer, _usingFixedDiagram: bIsUsingFixedDiagram });
		
		if (bIsUsingFixedDiagram) {
			diagramLayer = this.diagramLayer = new rmpw.jazz.ui.ModelDiagramLayer({
				configuration: this.configuration,
				resourceInfo: this.resourceInfo,
				rootLayer: this.rootLayer,
				outline: this.outline
			});
		}
		else {
			diagramLayer = this.diagramLayer = new rmpw.jazz.ui.ModelDiagramLayer({
				configuration: this.configuration,
				resourceInfo: this.resourceInfo,
				rootLayer: this.rootLayer,
				outline: this.outline,
				style: "height:100%;" });
		}

		// Use a dummy repository for comment layers with no URI to avoid locking the diagram resource
		var repository = new rmpw.common.vcs.Repository();
		var resourceInfo = { repository: repository };

		var sketchLayer = this.sketchLayer = new rmpw.jazz.ui.ModelDiagramLayer({ configuration: this.commentConfig, resourceInfo: resourceInfo, outline: this.outline });
		var archiveLayer = this.archiveLayer = new rmpw.jazz.ui.ModelDiagramLayer({ configuration: this.selectionConfig, resourceInfo: resourceInfo, outline: this.outline });
		
		diagramLayer.LAYER_TYPE = "diagramLayer";
		sketchLayer.LAYER_TYPE = "sketchLayer";
		archiveLayer.LAYER_TYPE = "archiveLayer";

		// HACK (MG): when one view changes, sync the other two
		var sketchViewport = this.sketchLayer.viewer.viewport;
		dojo.connect(sketchViewport, "onViewportChanged", function(sync) {
			if (!sync) {
				var t = sketchViewport._getContentsTransform();
				diagramLayer.viewer.viewport.contents.setTransform(t);
				diagramLayer.viewer.viewport.onViewportChanged(true);
				archiveLayer.viewer.viewport.contents.setTransform(t);
				archiveLayer.viewer.viewport.onViewportChanged(true);
			}
		});
		var diagramViewport = this.diagramLayer.viewer.viewport;
		dojo.connect(diagramViewport, "onViewportChanged", function(sync) {
			if (!sync) {
				var t = diagramViewport._getContentsTransform();
				sketchLayer.viewer.viewport.contents.setTransform(t);
				sketchLayer.viewer.viewport.onViewportChanged(true);
				archiveLayer.viewer.viewport.contents.setTransform(t);
				archiveLayer.viewer.viewport.onViewportChanged(true);
			}
		});
		var archiveViewport = this.archiveLayer.viewer.viewport;
		dojo.connect(archiveViewport, "onViewportChanged", function(sync) {
			if (!sync) {
				var t = archiveViewport._getContentsTransform();
				sketchLayer.viewer.viewport.contents.setTransform(t);
				sketchLayer.viewer.viewport.onViewportChanged(true);
				diagramLayer.viewer.viewport.contents.setTransform(t);
				diagramLayer.viewer.viewport.onViewportChanged(true);
			}
		});
		// END HACK /////////////
		
		dojo.connect(diagramLayer.viewer, "resizeAfterLayoutComplete", function(size, fixedDiagramSize) {
			sketchLayer.viewer.resizeAfterLayoutComplete(size, fixedDiagramSize);
			archiveLayer.viewer.resizeAfterLayoutComplete(size, fixedDiagramSize);
		});
		
		dojo.connect(diagramLayer.viewer, "setFixedDiagramSize", function(size) {
			sketchLayer.viewer.fixedDiagramSize = size;
			archiveLayer.viewer.fixedDiagramSize = size;
		});
		dojo.connect(sketchLayer.viewer, "setFixedDiagramSize", function(size) {
			diagramLayer.viewer.fixedDiagramSize = size;
			archiveLayer.viewer.fixedDiagramSize = size;
		});
		dojo.connect(archiveLayer.viewer, "setFixedDiagramSize", function(size) {
			diagramLayer.viewer.fixedDiagramSize = size;
			sketchLayer.viewer.fixedDiagramSize = size;
		});
		
		dojo.connect(diagramLayer, "setLocation", function(x, y, fromScale) {			
			sketchLayer.dontScroll_left = true;
			sketchLayer.dontScroll_top = true;
			sketchLayer.domNode.scrollLeft = diagramLayer.domNode.scrollLeft;
			sketchLayer.domNode.scrollTop = diagramLayer.domNode.scrollTop;
			
			archiveLayer.dontScroll_left = true;
			archiveLayer.dontScroll_top = true;
			archiveLayer.domNode.scrollLeft = diagramLayer.domNode.scrollLeft;
			archiveLayer.domNode.scrollTop = diagramLayer.domNode.scrollTop;
			
			// the below code fixes a comment bug in sketch diagrams (defect 44528)
			if (!this.outline.isUsingFixedDiagram()) {
				var size = dojo.marginBox(diagramLayer.viewer.container);
				diagramLayer.viewer.resize(size);
				sketchLayer.viewer.resize(size);
				archiveLayer.viewer.resize(size);
			}
		});
		dojo.connect(sketchLayer, "setLocation", function(x, y, fromScale) {
			diagramLayer.dontScroll_left = true;
			diagramLayer.dontScroll_top = true;
			diagramLayer.domNode.scrollLeft = sketchLayer.domNode.scrollLeft;
			diagramLayer.domNode.scrollTop = sketchLayer.domNode.scrollTop;
			
			archiveLayer.dontScroll_left = true;
			archiveLayer.dontScroll_top = true;
			archiveLayer.domNode.scrollLeft = sketchLayer.domNode.scrollLeft;
			archiveLayer.domNode.scrollTop = sketchLayer.domNode.scrollTop;
			
			// the below code fixes a comment bug in sketch diagrams (defect 44528)
			if (!this.outline.isUsingFixedDiagram()) {
				var size = dojo.marginBox(sketchLayer.viewer.container);
				diagramLayer.viewer.resize(size);
				sketchLayer.viewer.resize(size);
				archiveLayer.viewer.resize(size);
			}
		});
		dojo.connect(archiveLayer, "setLocation", function(x, y, fromScale) {
			diagramLayer.dontScroll_left = true;
			diagramLayer.dontScroll_top = true;
			diagramLayer.domNode.scrollLeft = archiveLayer.domNode.scrollLeft;
			diagramLayer.domNode.scrollTop = archiveLayer.domNode.scrollTop;
			
			sketchLayer.dontScroll_left = true;
			sketchLayer.dontScroll_top = true;
			sketchLayer.domNode.scrollLeft = archiveLayer.domNode.scrollLeft;
			sketchLayer.domNode.scrollTop = archiveLayer.domNode.scrollTop;
			
			// the below code fixes a comment bug in sketch diagrams (defect 44528)
			if (!this.outline.isUsingFixedDiagram()) {
				var size = dojo.marginBox(archiveLayer.viewer.container);
				diagramLayer.viewer.resize(size);
				sketchLayer.viewer.resize(size);
				archiveLayer.viewer.resize(size);
			}
		});

		rootLayer.addLayer("DIAGRAM_LAYER", this.diagramLayer);
		rootLayer.addLayer("ARCHIVE_LAYER", this.archiveLayer);
		rootLayer.addLayer("SKETCH_LAYER", this.sketchLayer);
		rootLayer.addLayer("OUTLINE_LAYER", this.outline);
		rootLayer.addLayer("TOOLS_LAYER", this.toolsLayer);

		if (this.graph) {
			this._crumbsBar = dojo.doc.createElement("div");
			this._crumbsBar.style.padding = "5px 10px 0";
			this._crumbsBar.style.background = "white";

			this.containerNode.appendChild(this._crumbsBar);

			var crumbs = dojo.doc.createElement("span");
			dojo.addClass(crumbs, "jazz-ui-Breadcrumbs breadCrumbs");

			this._crumbsBar.appendChild(crumbs);

			var length = this.history.length;

			for (var h = 0; h < length; h++) {
				var link = dojo.doc.createElement("a");

				if (h == 0) {
					link.href = CoreUtil.getViewResourceUri(this.resourceUri);
					link.innerHTML = "Parent Container > ";
				} else {
					var suffix = encodeURIComponent(this.history[h]);
					var uri = CoreUtil.getViewResourceUri(this.resourceUri) + "&graph=" + suffix;
					link.href = uri;
					link.innerHTML = "Compartment " + h + " > ";
				}

				crumbs.appendChild(link);
			}
		}

		if (dojo.isIE == 8) {
			dojo.style(this.archiveLayer.domNode, "visibility", "collapse");
			dojo.style(this.sketchLayer.domNode, "visibility", "collapse");
		} else {
			dojo.style(this.archiveLayer.domNode, "visibility", "hidden");
			dojo.style(this.sketchLayer.domNode, "visibility", "hidden");
		}
		dojo.style(this.diagramLayer.domNode, "visibility", "hidden");

		this.rootLayer.setFocusController(this.diagramLayer.viewer.policies.focusScope);
		this.rootLayer.focusController.enable();
		this.resetOutlineListeners = dojo.connect(this.diagramLayer.viewer, "setDiagram", this, "resetOutline");
	},

	connectCommenting: function(action, method) {
		this.connect(this.commentingWidget, action, method);
	},

	subscribe: function() {
		this.connectCommenting("onSelectionChanged", this.commentSelected);
		this.connectCommenting("onCommentCreated", this.commentCreated);
		this.connectCommenting("onCommentUpdated", this.commentUpdated);
		this.connectCommenting("onStartCreateComment", this.beginComment);
		this.connectCommenting("onStartEditComment", this.beginEditComment);
		this.connectCommenting("onEndEditingComment", this.endComment);
		this.connectCommenting("onCancelEditingComment", this.cancelComment);
		this.connectCommenting("onLinkWithEditor", this.linkEdtitor);
	},

	showLoadingMessage: function(s) {
		if (this.loaded == false) {
			if (this.loadingnode == null) {
				this.loadingnode = document.createElement("div");

				this.loadingnode.style.padding = "10px";
				this.loadingnode.style.position = "absolute";
				this.loadingnode.style.color = "#BBBBBB";
				this.loadingnode.style.fontSize = "2em";
				this.loadingnode.innerHTML = messages.loading;

				this.domNode.appendChild(this.loadingnode);
			}

			this.loadingnode.style.top = Math.floor((s.h / 2) - (this.loadingnode.offsetHeight / 2)) + "px";
			this.loadingnode.style.left = Math.floor((s.w / 2) - (this.loadingnode.offsetWidth / 2)) + "px";
		}
	},

	hideLoadingMessage: function() {
		if (this.loadingnode != null) {
			this.domNode.removeChild(this.loadingnode);
			delete this.loadingnode;
		}

		this.loaded = true;
	},

	showMessage: function(msg) {
		if (this.msgNode == null) {
			this.msgNode = document.createElement("div");
			this.msgNode.style.padding = "10px";
			this.msgNode.style.position = "absolute";
			this.msgNode.style.color = "#BBBBBB";
			this.msgNode.style.fontSize = "2em";
			this.msgNode.style.top = "175px";
			this.msgNode.style.left = "400px";
			this.domNode.appendChild(this.msgNode);
		}
		this.msgNode.innerHTML = msg;
	},

	hideMessage: function() {
		if (this.msgNode != null) {
			this.domNode.removeChild(this.msgNode);
			delete this.msgNode;
		}

		this.loaded = true;
	},

	getPadding: function() {
		var padding = 29;
		return padding;
	},
	
	resize: function() {		
		if(!this.containerNode){
			setTimeout(dojo.hitch(this, this.resize), 10);
			return;
		}

		// Self/footer/window positions
		var myPos = dojo.position(this.containerNode, true);
		
		// Not done layouting...try again
		if(myPos.y == 0){
			setTimeout(dojo.hitch(this, this.resize), 10);
			return;
		}

		// size of diagram is the total size of window - the footer height - the y position of this diagram - 7 (misc pixels)
		var footerPos = dojo.position(dojo.byId("jazz_app_internal_Footer_0"), true);
		var windowPos = dojo.position(document.getElementsByTagName("body")[0], false);
		var newHeight = windowPos.h - myPos.y - footerPos.h - 5;
		this.containerNode.style.height = newHeight +  "px";
		
		// resize diagrams
		var s = dojo.marginBox(this.diagramLayer.rootLayer.domNode);
		s.h = newHeight;
		if (this.outline.isUsingFixedDiagram()) {
			var noVerticalScrollBar = false;
			var noHorizontalScrollBar = false;
			var diagramSize = null;
			if (this.diagramLayer.viewer.canvasSize) { // otherwise diagram size not known yet
				diagramSize = this.diagramLayer.viewer.surface.getDimensions();
				noVerticalScrollBar = Math.round(s.h) + 6 >= Math.round(diagramSize.height);
				noHorizontalScrollBar = Math.round(s.w) + 1 >= Math.round(diagramSize.width);
			}
			var newSize = {};
			if (noVerticalScrollBar && noHorizontalScrollBar) {
				newSize.h = Math.round(s.h) + 6;
				newSize.w = Math.round(s.w) + 1;
			}
			else {
				newSize.w = s.w;
				newSize.h = s.h;
			}
			
			this.diagramLayer.domNode.style.height = newSize.h + "px";
			this.diagramLayer.domNode.style.width = newSize.w + "px";
			this.diagramLayer.domNode.style.overflow = "auto";
			if (diagramSize) {
				this.diagramLayer.outline.resize({
					w: diagramSize.width,
					h: diagramSize.height
				});
			}
			this.diagramLayer.viewer.canvasInFullSize = true;
			this.diagramLayer.viewer.resize(null);
			
			this.sketchLayer.domNode.style.height = newSize.h + "px";
			this.sketchLayer.domNode.style.width = newSize.w + "px";
			this.sketchLayer.domNode.style.overflow = "auto";
			this.sketchLayer.viewer.canvasInFullSize = true;
			this.sketchLayer.viewer.resize(null);
			
			this.archiveLayer.domNode.style.height = newSize.h + "px";
			this.archiveLayer.domNode.style.width = newSize.w + "px";
			this.archiveLayer.domNode.style.overflow = "auto";
			this.archiveLayer.viewer.canvasInFullSize = true;
			this.archiveLayer.viewer.resize(null);
		}
		else {
			this.diagramLayer.viewer.resize(s);
			this.sketchLayer.viewer.resize(s);
			this.archiveLayer.viewer.resize(s);
			
			// resize outline view after a delay
			this._timer = setTimeout(dojo.hitch(this, function(){
				delete this._timer;
				this.diagramLayer.outline.resize(s);
			}), 500);
		}
		
		// show loading message for RSA diagrams
		this.showLoadingMessage(s);
	},

	deglaze: function(layer) {
		if (this.glass != null) {
			this.glassGroup.remove(this.glass);
			this.glass = null;
			this.glazed = false;
		}

		if (dojo.isIE == 8) {
			dojo.style(layer.domNode, "visibility", "collapse");
		} else {
			dojo.style(layer.domNode, "visibility", "hidden");
		}
	},

	getGlassSize: function() {
		var size = null;
		if (this.outline.isUsingFixedDiagram()) {
			var surfaceSize = rmpw.diagrams.render.util.Workarounds.getTransformedBoundingBox(this.viewer.surface);
			size = {
				w: surfaceSize.width,
				h: surfaceSize.height
			}
		}
		else {
			size = dojo.marginBox(this.editor.domNode);
			var padding = this.getPadding();
			size.h = size.h - padding;
		}

		return size;
	},

	glaze: function(layer) {
		if (this.surface != null && this.glazed == false) {
			var size = this.getGlassSize();
			this.glassColor.a = 0.5;
			this.glass = this.glassGroup.createRect({ x: 0,  y: 0, width: size.w, height: size.h });
			this.glass.setFill(this.glassColor);
			this.glazed = true;
		}

		dojo.style(layer.domNode, "visibility", "visible");

		//On IE9 the SVG layer will not always catch events
		if (dojo.isIE > 8 && layer == this.sketchLayer) {
			var currentScale = this.sketchLayer.getScale();
			this.sketchLayer.setScale(currentScale * 0.001);
			this.sketchLayer.setScale(currentScale);
		}
	},

	makeSketcher: function(root, viewer, mode) {
		var controller = null;
		var blank = new rmps.web.comment.sketch.render.Group();

		root = new rmps.web.comment.sketch.render.SVG();
		root.setG(blank);

		root.setGraphicsParent(viewer.viewport.contents);

		viewer.setDiagram(root);
		viewer.commandPackage = new rmps.web.comment.sketch.commands.SketchCommandPackage();
		var myviewer = viewer;

		viewer.__editMode = mode;
		var controller = new rmps.web.comment.sketch.behavior.SketchController(root);
		var mycontroller = controller;

		mycontroller.setParent(myviewer);

		mycontroller._connectConstruction = dojo.connect(root, "applyModelChanges", this, function() {
			dojo.disconnect(mycontroller._connectConstruction);
			delete mycontroller._connectConstruction;
		});

		if (mode == "EDIT") {
			dojo.disconnect(this.editor.connection);

			this.editor.selectionController = viewer.getSelectionController();
			this.connection = dojo.connect(this.editor.selectionController, "onSelectionChanged", this.editor, "selectionChanged");

			this.editor.commandController = viewer.getCommandController();
			this.editor.commandController.modelingPage = this.toolsManager;
		}

		dojo.connect("onresize", dojo.hitch(this, this.resize));
		return { controller: controller, root: root };
	},

	addDrawingTools : function() {
		// remove all tools from the toolbar except zoom as DrawingTools will add its own
		this.diagramLayer.viewer.policies.actionBarScope.removeToolsExceptZoom();

		// now let DrawingTools add its items to the toolbar
		this.toolsManager = new rmpw.jazz.ui.DrawingTools(this.toolbar, this);
		this.toolsManager.addDrawingTools();
	},

	removeDrawingTools : function() {
		// remove toolbar items previously added by DrawingTools
		this.toolsManager.removeDrawingTools();
		this.toolsManager.dispose();
		this.toolsManager = null;

		// add previously removed tools back into the toolbar
		this.diagramLayer.viewer.policies.actionBarScope.reAddRemovedTools();
	},

	beginComment: function(comment) {
		if (this.ready == true) {
			this.sketchLayer.commenting = this.commenting = true;
			this.glaze(this.sketchLayer);
			this.addDrawingTools();
		}
	},

	getBounds: function() {
		var bounds = new Object();
		bounds.x = 0;
		bounds.y = 0;
		return bounds;
	},

	clear: function() {
		if (this.archiveViewer) {
			var mappings = this.archiveController.root.diagram.__editController.children;
			var deleteMappings = [];
			for (var item = 0, editController; editController = mappings[item]; item++) {
				deleteMappings.push(editController);
			}
			for (var item = 0, editController; editController = deleteMappings[item]; item++) {
				editController.policies.clearAction.run({targetController: editController});
			}
		}
	},

	initializeCommands: function() {
		this.commands = new Array();
		this.commands["Rectangle"] = rmps.web.comment.sketch.commands.shapes.CreateRectangle;
		this.commands["Oval"] = rmps.web.comment.sketch.commands.shapes.CreateOval;
		this.commands["Line"] = rmps.web.comment.sketch.commands.shapes.CreateLine;
		this.commands["Scribble"] = rmps.web.comment.sketch.commands.shapes.CreatePath;
		this.commands["Text"] = rmps.web.comment.sketch.commands.shapes.CreateText;
		this.commands["TextBox"] = rmps.web.comment.sketch.commands.shapes.CreateTextBox;
	},

	createShape: function(controller, item) {
		var data = dojo.clone(item);
		data.svgParent = controller.renderMapping.sources.svgRoot;
		data.viewport = controller.viewer.viewport;

		this.initializeCommands();

		var controller = controller.viewer.getCommandController()

		controller.commandStack.execute(new this.commands[ data.shape ](), data);
	},

	deserialize: function(controller, svgdata) {
		this.groups = new Array();
		this.svgdata = svgdata;
		this.reader = new rmps.web.comment.sketch.serializer.Reader();
		this.reader.read(this.svgdata, this.getBounds());

		for (var s = 0; s < this.reader.scribbles.length; s++) {
			this.createShape(controller, this.reader.scribbles[s]);
		}
	},

	selectHotspots: function(comment) {
		if (comment.range && this.viewer.root.diagram.__editController && this.viewer.root.diagram.__editController.children) {
			var range = comment.range;
			var children = this.viewer.root.diagram.__editController.children;
			for (var item = 0; item < range.length; item++) {
				var ids = range[item].ids;
				for (var id = 0; id < ids.length; id++) {
					for (var child = 0; child < children.length; child++) {
						var checkId = null;
						var mapping = children[child].renderMapping;
						if (mapping.sources.model && mapping.sources.model.zonedata) {
							checkId = mapping.sources.model.zonedata.view;
						} else if (mapping.getDefaultSource()) {
							checkId = mapping.getDefaultSource().rdf_about;
						}
						if (checkId && ids[id] == this._getFragment(checkId)) {
							this.viewer.getSelectionController().select(children[child]);
						}
					}
				}
			}
		}
	},

	deselectHotSpots: function() {
		this.viewer.getSelectionController().deselectAll();
	},

	commentSelected: function(sel, previous, comments, deselected) {
		if (!this.viewer.root.diagram && this.viewer.root.setDiagram) {
			// don't select until diagram is set into root
			this._connectDiagramSet = dojo.connect(this.viewer.root, "setDiagram", this,
				dojo.hitch(this, function() {
					dojo.disconnect(this._connectDiagramSet);
					delete this._connectDiagramSet;
					this._commentSelected(sel, previous, comments, deselected);
			}));
		} else {
			this._commentSelected(sel, previous, comments, deselected);
		}
	},

	_commentSelected: function(sel, previous, comments, deselected) {
		if (comments && this.ready == true) {
			this.currentComments = comments;

			if (this.archiveController) {
				this.deselectHotSpots();

				var commentCount = comments.length;

				if (commentCount > 0) {
					this.clear();

					var scribbles= [];

					this.glaze(this.archiveLayer);

					for (var i = 0; i < commentCount; ++i) {
						var comment = comments[i];

						this.selectHotspots(comment);

						var body = comment.getBody(rmps.core.commenting.BodyType.SKETCH.value);

						if (body != null && body.contents != null) {
							this.deserialize(this.archiveController, body.contents);
							if (dojo.isArray(this.reader.scribbles)) {
								scribbles.push(this.reader.scribbles);
							}
						}
					}

					/* Only glaze the layer if there is at least one comment shape present */
					if (scribbles.length > 0) {
						this.glaze(this.archiveLayer);
					} else {
						/* Required for a scenario like: Two comments are selected. One does not have any shapes, and the other one has some shapes created.
						 * If we deslected the one that has some shapes, then we will need to deglaze the current layer, as the only select comment will not
						 * be having any shapes to be shown */
						this.deglaze(this.archiveLayer);
					}

					this.selected = true;
				} else {
					this.clear();

					if (this.commenting == false) {
						this.deglaze(this.archiveLayer);
					}

					this.selected = false;
				}
			} else {
				this.comments = comments;
			}
		}
	},

	commentCreated: function(comment) {

	},

	commentUpdated: function(comment) {
		// FIXME: The only mandatory parameter is the comments array.
		// If we want to get all the parameters set then we need to call
		// this.commentingWidget.selectComments(comment);
		this.commentSelected(null, null, [comment], null);
	},

	beginEditComment: function() {
		if (this.ready == true) {
			// TODO agfitzp Did this ever work, there is nothing like this in the entire codebase
			// this.commentingWidget._dirtyHook();

			this.clear();

			this.deglaze(this.archiveLayer);

			this.deselectHotSpots();

			var comments = this.currentComments;

			var commentCount = comments.length;

			if (commentCount > 0) {
				this.glaze(this.sketchLayer);
				this.addDrawingTools();

				for (var i = 0; i < commentCount; ++i) {
					var comment = comments[i];

					this.selectHotspots(comment);

					var body = comment.getBody(rmps.core.commenting.BodyType.SKETCH.value);

					if (body != null && body.contents != null) {
						this.deserialize(this.sketchController, body.contents);
					}
				}
			}
		}
	},

	endComment: function(comment) {
		var selectionController = this.viewer.getSelectionController();
		var selection = selectionController.getSelection();
		var elements = new Array();

		if (selection) {
			var elementMap = {}; // to avoid duplicates in 'elements'
			for (var item in selection) {
				var elementId = null;
				var mapping = selection[item].renderMapping;

				if (mapping) {
					if (mapping.sources
							&& mapping.sources.model
							&& mapping.sources.model.zonedata) {
						// imported diagram with zonedata
						elementId = mapping.sources.model.zonedata.view;
					} else if (mapping.getDefaultSource()) {
						// dtk diagrams
						elementId = mapping.getDefaultSource().rdf_about;
					}
					elementId = this._getFragment(elementId);
				}

				if (elementId && !elementMap[elementId]) {
					elements.push(elementId);
					elementMap[elementId] = 1;
				} else {
					console.log("Saving comment: no selection render map");
				}
			}
		}

		comment.range = [ new rmps.core.commenting.range.IDList(elements) ];

		var shapes = this.sketchRoot.sources.svgRoot.g;

		if (shapes) {
			var writer = new rmps.web.comment.sketch.serializer.Writer();
			var svgdata = writer.write(shapes);

			if (svgdata != null) {
				var body = { type: rmps.core.commenting.BodyType.SKETCH.value, contents: svgdata };
				comment.addBody(body);
				console.log(svgdata);
			}
		}

		// this.currentComments = comment;

		if (this.toolsManager) {
			this.toolsManager.clear();

			this.removeDrawingTools();

			this.deselectHotSpots();

			this.deglaze(this.archiveLayer);
			this.deglaze(this.sketchLayer);

			this.sketchLayer.commenting = this.commenting = false;

			/* Reset the undo stack */

			this.resetCommandStack();
		}
	},

	_getFragment: function(uri) {
		if (uri) {
			var fragmentIdx = uri.indexOf('#');
			if (fragmentIdx > 0) {
				return uri.substring(fragmentIdx + 1);
			}
		}
		return 0;
	},

	resetCommandStack: function() {
		var repository = new rmpw.common.vcs.Repository();
		/* stack of revisions that delineate user-executed commands, so compound commands can be undone as a unit */

		this.sketchController.viewer._commandController.commandStack.repository = repository;
		this.sketchController.viewer._commandController.commandStack.undoStackPosition = 0;
		this.sketchController.viewer._commandController.commandStack.undoStack = [ repository.getCurrentRevision() ];
	},

	cancelComment: function(comment) {
		this.toolsManager.clear();
		this.deglaze(this.sketchLayer);

		this.commenting = false;

		if (this.selected == true) {
			this.glaze(this.archiveLayer);
		} else {
			this.deglaze(this.archiveLayer);
		}

		this.removeDrawingTools();

		this.deselectHotSpots();

		this.resetCommandStack();
	},

	linkEdtitor: function(isLinked, comments) {

	},

	undoSketch: function() {
		this.sketchLayer.viewer.getCommandController(this.commentConfig.canvas).commandStack.undo();
	},

	redoSketch: function() {
		this.sketchLayer.viewer.getCommandController(this.commentConfig.canvas).commandStack.redo();
	},

	getCommandController: function(attachPoint) {
		return this.sketchLayer.viewer.getCommandController(attachPoint);
	},

	drawDiagram: function(root) {
		/* Need to analyze why it takes two calls of handleDiagramAssets to
		 * guarantee that the rendering will succeed.
		 */

		this.diagramRoot = root;

		var diagram = new rmps.graphics.render.HotspotDiagramViewMapping();
		diagram.setModel(this.diagramRoot);
		diagram.setParentFigure(this.viewer.viewport.contents);

		this.controller = new rmps.graphics.behavior.HotspotDiagramController(diagram);
		var controller = this.controller;

		this.viewer.setDiagram(diagram);
		var viewer = this.viewer;

		this.controller._connectConstruction = dojo.connect(diagram, "applyModelChanges", this, function() {
			controller.setParent(viewer);
			controller.setRenderMapping(diagram);
			dojo.disconnect(controller._connectConstruction);
			delete controller._connectConstruction;
		});

		this.addControllers();

		this.ready = true;

		// For testing work item 63970: setTimeout(dojo.hitch(this, "addControllers"), 15000);
	},

	addControllers: function() {
		var sketcher = this.makeSketcher(this.sketchRoot, this.sketchViewer, true);

		this.sketchController = sketcher.controller;
		this.sketchRoot = sketcher.root;

		var archiver = this.makeSketcher(this.archiveRoot, this.archiveViewer, false);
		this.archiveController = archiver.controller;

		this.hideLoadingMessage();
		dojo.style(this.diagramLayer.domNode, "");
//      this.cancelComment();
//
//      console.log("ADDED SKETCH CONTROLLERS");
//
//      if (this.comments != null) {
//          this.commentSelected(this.comments);
//          this.comments = null;
//      }
	},

	startup: function() {
		this.inherited(arguments);

		this.viewer = this.diagramLayer.viewer;
		this.sketchViewer = this.sketchLayer.viewer;
		this.archiveViewer = this.archiveLayer.viewer;

		this.surface = this.viewer.surface;

		this.glassColor.a = 0.5;
		this.glassGroup = this.surface.createGroup();

		this.subscribe();

		this.commentingWidget.diagramWidget = this;

		this.initializeCommands();

		this.addControllers();

		dojo.style(this.diagramLayer.domNode, "visibility", "");

		this.ready = true;
		
    	window.onresize({});
	}
});

// vml sometimes doesn't set rawNode.path
fixShape(dojox.gfx.Line);
fixShape(dojox.gfx.Polyline);
fixShape(dojox.gfx.Path);

function fixShape(type) {
	if (!dojox.gfx.svg) {
		var old = type.prototype["setShape"];
		type.prototype["setShape"] = function() {
			if (!this.rawNode.path) {
				this.rawNode.path = {};
			}
			return old.apply(this, arguments);
		};
	}
};

//force cursor over text to initially be pointer for all browsers
fixText(dojox.gfx.Text);

function fixText(type) {
	var old = type.prototype["setShape"];
	type.prototype["setShape"] = function() {
		if (this.rawNode.style) {
			this.rawNode.style.cursor = 'default';
		}
		return old.apply(this, arguments);
	};
};

//don't allow shape width/height to be strings
fixRect(dojox.gfx.Rect);

function fixRect(type) {
	var old = type.prototype["setShape"];
	type.prototype["setShape"] = function(shape) {
		if (shape) {
			if (dojo.isString(shape.width)) {
				shape.width = parseFloat(shape.width);
			}
			if (dojo.isString(shape.height)) {
				shape.height = parseFloat(shape.height);
			}
		}
		return old.apply(this, arguments);
	};
};

})();
