/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.jazz.ui.DrawingTools");

/* rmpw.jazz */

dojo.require("rmpw.jazz.ui.LineStyler");

/* rmpw.widgets */

dojo.require("rmpw.widgets.pickers.ToolPicker");
dojo.require("rmpw.widgets.pickers.ColorPicker");
dojo.require("rmpw.widgets.pickers.TextPicker");

/* rmps.web.comment.sketch */

dojo.require("rmps.web.comment.sketch.behavior.Scribbler");
dojo.require("rmps.web.comment.sketch.behavior.LineMaker");
dojo.require("rmps.web.comment.sketch.behavior.OvalMaker");
dojo.require("rmps.web.comment.sketch.behavior.RectangleMaker");
dojo.require("rmps.web.comment.sketch.behavior.CloudMaker");
dojo.require("rmps.web.comment.sketch.behavior.TextMaker");

/* rmpw.diagrams */

dojo.require("rmpw.diagrams.edit.actions.DeleteElement");
dojo.require("rmps.web.comment.sketch.commands.EditStyle");
dojo.require("rmps.web.comment.sketch.commands.EditFontStyle");

/* rmps.graphics */

dojo.require("rmps.graphics.ui.Configuration");

/* dijit, dojo */

dojo.require("dijit.Toolbar");
dojo.require("dijit.form.Button");

dojo.require("dojo.i18n");

dojo.require("rmps.app.core.util");

dojo.require("rmpw.widgets.TouchGuide");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmpw.jazz.ui", "ModelDiagramLayerMessages");
var messages = dojo.i18n.getLocalization("rmpw.jazz.ui", "ModelDiagramLayerMessages");

dojo.declare("rmpw.jazz.ui.DrawingTools", null, {
	
	selection: null,
	
	constructor: function( toolbar, widget ){
		this.toolbar = toolbar;
		this.viewer = widget;
		this.commandController = this.viewer.sketchLayer.viewer.getCommandController();
		this.commandController.modelingPage = this; 
		this.selectionController = this.viewer.sketchLayer.viewer.getSelectionController();
		
		this.connections = [];
		this.connections.push(dojo.connect( this.selectionController, "onSelectionChanged", this, "selectionChanged" ));
		
		this.lineStyler = new rmpw.jazz.ui.LineStyler(); 	
		this.lineStyler.setLineColor = dojo.hitch( this, "setLineColor" ); 
		this.lineStyler.setLineWidth = dojo.hitch( this, "setLineWidth" ); 
		this.lineStyler.setLineStyle = dojo.hitch( this, "setLineStyle" );
		
		this.linetools = false;
		this.texttools = false;
		
		this.topics = rmpw.widgets.Events.topics;
		
		this.subscriptions = [];
		this.subscriptions.push( dojo.subscribe( this.topics.FILL_COLOR, dojo.hitch( this, "setFillColor" ) ) );
		this.subscriptions.push( dojo.subscribe( this.topics.TEXT_COLOR, dojo.hitch( this, "setTextColor" ) ) );
		
		this.filltool = false;
		
		this.styleTools = [];
				
		this.toggleButtons = [];
	
		this.selection = [];
		
		this.toggleCount = 0;	
	},
	
	dispose: function() {
		var i = 0, subscriptionLength = this.subscriptions.length;
		for (i = 0; i < subscriptionLength; i++) {
			dojo.unsubscribe(this.subscriptions[i]);
		}
		
		for (i = 0; i < this.connections; i++) {
			dojo.disconnect(this.connections[i]);
		}
		this.connections = [];
	},
	
	clear: function(){
		
		var mappings = this.viewer.sketchController.root.diagram.__editController.children;
		var deleteMappings = [];
		for( var item = 0, editController; editController = mappings[item]; item++ ){	
			deleteMappings.push(editController);
		}
		for( var item = 0, editController; editController = deleteMappings[item]; item++ ){	
			editController.policies.clearAction.run({targetController: editController});
		}	
	},
	
	buildArgs: function( type ){
		
		var commandArgs = {
			svgParent: this.viewer.sketchController.renderMapping.sources.svgRoot,
			shape: type,
			mode: "MULTIPLE_HIT",
			viewport: this.commandController.viewer.viewport	
		};
		
		return commandArgs;
	},
	
    executeStyle:function( args ){
    	var cmd = new rmps.web.comment.sketch.commands.EditStyle();
    	this.commandController.commandStack.execute( cmd, args ); 
    },
    
    updateStroke: function( stroke ){
    	this.executeStyle( {
			stroke: stroke,
			fill: null,
			text: null,
			nodes: this.selection
		}); 
	},
	
	selectionChanged: function(){
		
		this.selection = this.selectionController.getSelection();
		
		if( this.selection.length > 0 && this.toolspresent == false ){
			this.removeStyleTools();
			this.addStyleTools();		
    	}
	},
	
	setLineColor: function( message ){		
		this.updateStroke( message ); 	
	},
	
	setLineWidth: function( message ){	
		this.updateStroke( { width: message.width } ); 
	},
	
	setLineStyle: function( message ){
		this.updateStroke( { dash: message.style } ); 
	},
	
	setFillColor: function( message ){				
		this.executeStyle({
			stroke: null,
			fill: message,
			text: null,
			nodes: this.selection
		}); 		
	},
	
	setTextColor: function( message ){
		this.executeStyle({
			stroke: null,
			fill: null,
			text: message,
			nodes: this.selection
		}); 	
	},
	
	setSelectionMode: function(){
		
		this.lastSelection = this.SELECT;
		
		if( this.linetool ){
			this.linetool.release();
		}
		
		if( this.scribbler ){
			this.scribbler.release();
		}
		
		if( this.ovaltool ){
			this.ovaltool.release();
		}
		
		if( this.recttool ){
			this.recttool.release();
		}
		
		if( this.cloudtool ){
			this.cloudtool.release();
		}
		
		if( this.texttool ){
			this.texttool.release();
		}
	},
	
	undo: function(){
		this.viewer.undoSketch();
	},
	
	redo: function(){
		this.viewer.redoSketch();
	},
	
	addText: function(){
		
		this.lastSelection = this.TEXT;
		
		if(!this.texttool){
			this.texttool = new rmps.web.comment.sketch.behavior.TextMaker( this.commandController );
		}
		
		this.texttool.capture( this.buildArgs( "Text" ) );	
		
	},
	
	addLine: function(){
		
		this.lastSelection = this.LINE;
		
		if(!this.linetool){
			this.linetool = new rmps.web.comment.sketch.behavior.LineMaker( this.commandController );
		}
		this.linetool.capture( this.buildArgs( "Line" ) );	
	},
	
	addScribble: function(){	
		
		this.lastSelection = this.PENCIL;
		
		if(!this.scribbler){
			this.scribbler = new rmps.web.comment.sketch.behavior.Scribbler( this.commandController );
		}
		
		this.scribbler.capture( this.buildArgs( "Scribble" ) );			
	},
	
	addTextTools: function(){
		this.textfillpicker = new rmpw.widgets.pickers.ColorPicker({type:"TEXT"});
		this.addTool( this.textfillpicker );
		this.textfillpicker.startup();
	},
		
	addStyleTools: function(){
		
		this.addTextTools();
		
		if( this.linetools == false ){							
			this.fillpicker = new rmpw.widgets.pickers.ColorPicker({type:"FILL"});				
			this.addTool( this.fillpicker );	
			this.styleTools.push( this.fillpicker );
			this.fillpicker.startup();	
			this.lineStyler.addStyleTools( this.toolbar );	
			this.linetools = true;
			this.filltool = true;
		}	
		
		if( this.texttools == false ){	
			
			this.textStyler = new rmpw.widgets.pickers.TextPicker();
			this.textStyler.setStyles = dojo.hitch( this, "setFontStyle" );
			this.addTool( this.textStyler );
			this.styleTools.push( this.textStyler );
			
	        this.connections.push(dojo.connect(this.textStyler, "openDropDown", dojo.hitch(this, function() {
	        	this._updatetexttoolsDropDown();
	        })));       		        

			
			this.texttools = true;
		}
		
	    this.toolspresent = true;
	},
	
	_updatetexttoolsDropDown: function() {
		var selection = this.selection;
		if (selection && selection.length > 0) {
			var style = this._getExistingFontStyle(selection);
			if (style) {
				this.textStyler.initializeValues(style);
			}
		}
	},
	
	_getExistingFontStyle: function(selection) {
		for(var i = 0; editController = selection[i]; i++) {
			for (var j = 0; renderMap = editController.renderMapping.children[i]; i++){
				var svgRoot = renderMap.sources.svgRoot;
				if (svgRoot && svgRoot.text !== ""){
					var fontStyle = {
							family: svgRoot.fontFamily, 
							weight: svgRoot.fontWeight,
							style: svgRoot.fontStyle,
							size: svgRoot.fontSize
					};
					return fontStyle;
				}
			}
		}
		return null;
	},


	setFontStyle: function( data ){
		if( this._validSelection(this.selection) ){			
			var font = this.textStyler.getFontObject();			
			var cmd = new rmps.web.comment.sketch.commands.EditFontStyle();
	    	this.commandController.commandStack.execute( cmd, { font: font, nodes: this.selection } );
		} else {
			alert(messages.selectCommentShape);			
		}
	},
	
	_validSelection: function( nodes ){
		if (nodes && nodes.length>0) {
			for( var node = 0; node < nodes.length; node++ ){
				if (!nodes[node].renderMapping.sources.svgRoot ||
						!nodes[node].renderMapping.sources.svgRoot.eClass) {
					return false;
				}
			}
			return true;
		}
		return false;
	},
		
	addButton: function( label, call, icon, show ){ 
	   	 
	   	 if( show == null ){ 
	   		 show = false; 
	     }
	   	 
	   	 var button = new dijit.form.Button({ 
	    		label: label, 
	    		showLabel: show, 
	    		onClick: call,
	    		iconClass: icon,
	    		style: "font-size:10px"});
	   	 
	   	 this.addTool( button );
	   	 
	   	 return button;
   },
   
   addToggleButton: function(label, call, icon, show ){ 
  	 
	   	 if( show == null ){ 
	   		 show = false; 
	   	 }
	   	 
	   	 var button = new dijit.form.ToggleButton({
	   		 	toolid: this.toggleCount,
	    		label: label, 
	    		showLabel: show, 
	    		onClick: call,
	    		iconClass: icon,
	    		onChange: dojo.hitch( this, "changeSelectedButton", this.toggleCount ),
	    		style: "font-size:10px"});
	   	 
	   	 this.addTool( button );

	   	 return button;
   },
   
   shapeChoice: function( val ){ 
		this.selectedTool = val;

		this.lastSelection = this.TOOLS;
		this.toggleButtons[ this.TOOLS ].attr( 'checked', true );
		
		this.changeSelectedButton( this.TOOLS );
		
		switch( this.selectedTool ){

			case "RMPW_CLOUD":
				if(!this.cloudtool){
					this.cloudtool = new rmps.web.comment.sketch.behavior.CloudMaker( this.commandController );
				}
				this.cloudtool.capture( this.buildArgs( "Cloud" ) );
				
				this.shapePicker.attr( 'iconClass', 'rmps-graphics-diagram-layout cloudIcon');

				break;
	
			case "RMPW_RECTANGLE":
				if(!this.recttool){
					this.recttool = new rmps.web.comment.sketch.behavior.RectangleMaker( this.commandController );
				}
				this.recttool.capture( this.buildArgs( "Rectangle" ) );	
				
				this.shapePicker.attr( 'iconClass', 'rmps-graphics-diagram-layout rectangleIcon');

				break;
	
			case "RMPW_OVAL":
				if(!this.ovaltool){
					this.ovaltool = new rmps.web.comment.sketch.behavior.OvalMaker( this.commandController );
				}
				this.ovaltool.capture( this.buildArgs( "Oval" ) );	
				
				this.shapePicker.attr( 'iconClass', 'rmps-graphics-diagram-layout ovalIcon');

				break;				
		}
	},
   
   addTool: function(item){
    	this.toolbar.addChild(item);
   },
	
   changeSelectedButton: function( id ){
       
	    for( var button = 0; button < this.toggleButtons.length; button++ ){
	    	
	    	if( button != this.lastSelection ){	    		
	    		this.toggleButtons[ button ].attr( 'checked', false );
	    		
	    		if( this.lastSelection == this.TOOLS ){
	    			this.toggleButtons[ button ].reset();
	    			this.toggleButtons[ button ].attr( 'checked', false );
	    		}
	    	}
	    }         	
   },
   
   registerToggle: function( button, id ){
		this.toggleButtons[this.toggleCount] = button;	
	   	this.toggleCount = this.toggleCount +1;
   },
   
   addDrawingTools: function(){
		
		this.drawingTools = [];
				
		var undo = this.addButton( "Undo", dojo.hitch( this, "undo" ), "undoIcon" );
		this.drawingTools.push( undo );
		
		var redo = this.addButton( "Redo", dojo.hitch( this, "redo" ), "redoIcon" );		
		this.drawingTools.push( redo );
		
		var selectionMode = this.addToggleButton( "Select", dojo.hitch( this, "setSelectionMode" ), "selectIcon" );
		this.SELECT = this.toggleCount;
		this.registerToggle( selectionMode, this.SELECT );
		this.drawingTools.push( selectionMode );
		
		this.shapePicker = new rmpw.widgets.pickers.ToolPicker();
		this.shapePicker.toolChoice = dojo.hitch( this, "shapeChoice" );
		this.shapePicker.onChange = dojo.hitch( this, "changeSelectedButton", 3 );
		this.shapePicker.toolid = this.toggleCount;
		this.TOOLS = this.toggleCount;
		this.registerToggle( this.shapePicker, this.TOOLS );
		
		this.addTool(this.shapePicker);
				
		this.drawingTools.push(this.shapePicker);
		
	   	var textButton = this.addToggleButton( "Text", dojo.hitch( this, "addText" ), "textIcon" );
		this.TEXT = this.toggleCount;
		this.registerToggle( textButton, this.TEXT );
		this.drawingTools.push(textButton);
		
		var lineButton = this.addToggleButton( "Line", dojo.hitch( this, "addLine" ), "lineIcon" );
		this.LINE = this.toggleCount;
		this.registerToggle( lineButton, this.LINE );
	   	this.drawingTools.push(lineButton);
		
		var scribbleButton = this.addToggleButton( "Pencil", dojo.hitch( this, "addScribble" ), "scribbleIcon" );
		this.PENCIL = this.toggleCount;
		this.registerToggle( scribbleButton, this.PENCIL );
		this.drawingTools.push(scribbleButton);
		
		this.lastSelection = this.SELECT;
		this.toggleButtons[ this.SELECT ].attr( 'checked', true );
				
		this.addStyleTools();		
   },
   
   removeDrawingTools: function(){
		var tool;
		var i = 0;
		var drawingToolsLength = this.drawingTools.length;
		for (i = 0; i < drawingToolsLength; i++) {
			tool = this.drawingTools[i];
			this.toolbar.removeChild( tool );
			tool.destroy();
		}
		
		this.drawingTools = [];
		
		this.toggleCount = 0;
		
		this.toggleButtons = [];
		
		this.removeStyleTools();
		
		this.setSelectionMode();
	},
	
	removeStyleTools: function(){
		var tool;
		var i = 0;
		var styleToolsLength = this.styleTools.length;
		for (i = 0; i < styleToolsLength; i++) {
			tool = this.styleTools[i];
			this.toolbar.removeChild( tool );
			tool.destroy();
		}
		
		this.lineStyler.removeStyleTools();
		
		this.styleTools = [];
		this.linetools = false;
		this.filltool = false;
		this.texttools = false;
		
		this.removeTextTools();
	},
	
	removeTextTools: function(){
		this.toolbar.removeChild( this.textfillpicker );
		this.textfillpicker.destroy();
		this.textfillpicker = null;
	}
});

})();