/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2011, 2013. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function() {
dojo.provide("rmpw.forms.ui.EditOrCreateResource");

dojo.require("jazz.ui.Dialog");
dojo.require("rmpw.forms.ui.ModelValidationUtil");
dojo.require("rmpw.common.util.FormsLauncher");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.requireLocalization("rmpw.forms.ui", "CreateResourceMessages");
var createResourceMessages = dojo.i18n.getLocalization("rmpw.forms.ui", "CreateResourceMessages");

/**
 * 
 * //FIXME: Must extend an instance of _Widget
 * 
 * Used for editing external documents, and for editing and creating fragments.
 * 
 * @author Sameh Ayoub
 */
dojo.declare("rmpw.forms.ui.EditOrCreateResource", null, {

	listeners: null,
	featureId: null,
	constructor: function() {
		this._sources = {};
		this.listeners = [];
	},
	
	getObjectTypes: function(args) {
		var objectTypes = [];
		if (args.target && args.target.rdf_type) {
			for (var i=0, lg=args.target.rdf_type.length; i<lg; i++) {
				var objType = args.target.rdf_type[i];
				objectTypes.push(objType.rdf_about || objType.$ref);
			}
		}
		else if (args.parent.typeUri) {
			objectTypes.push(args.parent.typeUri);
		}
		
		return objectTypes;
	},
	
	isEditMode: function(args) {
		if (args.target && args.target.rdf_about) {
			return true;
		}
		
		return false;
	},
	
    // args is an object with following fields: typeName, typeForm, defaultUri, defaultLabel, parent, featureId, target, isFragment, 
	//isReadOnly, editExternalDoc, eTag
	editOrCreateResource: function(args){
		this.eTag = args.eTag;
		var parent = args.parent;
		this._isEditing = this.isEditMode({parent: parent, target: args.target});
		this.typeName = dojo.string.substitute(this._isEditing ? createResourceMessages.editType : createResourceMessages.createType, [args.typeName]);
		this.editExternalDoc = args.editExternalDoc;
		document.body.style.cursor = 'progress';

		rmpw.common.util.FormsLauncher.loadForm({ 
				formName: args.typeForm,
				objectTypes: this.getObjectTypes({parent: parent, target: args.target}),
				readOnly: args.isReadOnly !== undefined ? args.isReadOnly : (parent.form ? parent.form._resource.page.isReadOnly : false),
				standAlone: false,
				editor: this.editExternalDoc ? null : (parent.form ? parent.form._resource.editor : null),
				create: !this._isEditing
			}, 
			dojo.hitch(this, dojo.hitch(this, this._show, args.typeForm, parent, args.defaultUri, args.featureId, args.target, args.isFragment)),
			function(error) {
				document.body.style.cursor = '';
				rmps.core.error.create(error).dispatch();
			}
		);
	},
	
	_show: function(typeForm, parent, defaultUri, featureId, target, isFragment) {
	    var rdfTypes = this.getObjectTypes({parent: parent, target: target}),
	       template,
	       unique;
	    
		document.body.style.cursor = '';
		this._form = new rmpw.dsf.codegen.properties.forms[typeForm]();
		this._form.isFragment = isFragment;
		this._form.isInPopup = true;
		
		var documentObject;
		var formObject;
		if (parent.form) {
			if (this.editExternalDoc) {
				documentObject = target;
				formObject = target;
			} else {
				documentObject = parent.form._resource.documentObject;
				formObject = parent.form._resource.formObject;
			}
		} else {
			documentObject = parent.host.root.diagram.getDefaultSource();
			formObject = parent.host.root.diagram.getDefaultSource();
		}
		if(this._form._setResource){
			this._form._setResource({
				documentObject: documentObject,
				namespaces: parent.form ? parent.form._resource.namespaces : [],
				objectTypes: rdfTypes,
				modelIndex: rmps.ui.editor.PageHelper.getInstance().getActiveEditorInput().resourceInfo.modelIndex,
				page: parent.form ? parent.form._resource.page : parent.host.root.page,
				editor: parent.form ? parent.form._resource.editor : parent.host.root.definition.uri,
				formObject: formObject,
				id: target ? target.rdf_about : null,
				eTag: this.eTag
			});
			
			//TODO: owl_oneOf should have its own template, with the section below being part of its logic
			//adding a One Of is only enabled once the resource has been saved 
			//(i.e when this._form._resource.id stores the resource's URI)
			if( rmpw.forms.ui.ModelValidationUtil.isUri(this._form._resource.id) ){
			    template = this._form.template;
                for(unique in template){
                    if(template.hasOwnProperty(unique) && template[unique].label === "owl_oneOf"){
                        template[unique].addCreationActions("resource saved");
                    }
                }
			}
		}
		
		//TODO: Must provide a setter instead
		this._form.featureId= featureId;
		
		if (this._isEditing){
			if (target.dmdiagram___incompleteContents) {
				// do a GET of the contents and then mixin target to the result
				rmps.core.client.xhrGet({
					url : net.jazz.ajax._contextRoot + "/rdfdirectory/?logicalURI=" + encodeURIComponent(target.rdf_about),
					headers: {
					"Accept": "application/x-rdf+json-graph",
						"X-ibm-rmps-internal": "true"
					},
					handleAs: "text",
					load: dojo.hitch(this, function(data) {
						var documents = rmpw.common.vcs.ref.fromJson(data, {
							index: {}, idAttribute: "rdf_about", loader: function() { }
						});
						var completeTarget = documents[0];
						dojo.mixin(completeTarget, target);
						dojo.mixin(target, completeTarget); // mix back so that original reference is updated
						delete target.dmdiagram___incompleteContents;
						
						this._form[this._form.setterNames[typeForm]](target);	
						this._completeShow(typeForm, parent, defaultUri, featureId, target);
					})
				});	
			} else {
				this._form.immediate = false;
				this._form[this._form.setterNames[typeForm]](target);	
				this._completeShow(typeForm, parent, defaultUri, featureId, target);
			}
		}else{
			var self = this;
			rmps.core.client.xhrGet({
				url: rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "rdfdirectory") + "/constructor",
				content: { rdf_type: rdfTypes },
				headers: { "Accept": "application/x-rdf+json-graph" },
				handleAs: "text",
				load: function(data, ioargs) {
					
					// process json for initial model
					if(data === null || data == '') {
						rmps.core.error.create("Failed to load initial model").dispatch();
						return;
					}
					
					var documents = rmpw.common.vcs.ref.fromJson(data, { idAttribute: "rdf_about" });
					var model = documents[0];
					// TODO: shouldn't have to assign this, constructor service should provide correct uri
					model.rdf_about = self.defaultUri;
					self._form[self._form.setterNames[typeForm]](model);
					self._form.immediate= true;
					self._completeShow(typeForm, parent, defaultUri, featureId, model);
				}
			});
		}
	},
	
	_completeShow: function(typeForm, parent, defaultUri, featureId, target) {
		if (parent.host) {
			// assume diagram pop-up
			this._form.setChangeDetails(target, parent.host.root.repository, true);
			parent._commandController= {
					commandStack: parent.host.root.getCommandController().commandStack
				};
		}
		else if (parent.typeUri && parent.typeLabel) {
			if (this._isEditing) {
				this._form.setChangeDetails(target, parent.form.repository, false);
			}
			else {
				this._form.setChangeDetails(/*Change with null repository, as it is used only to track changes but does not commit */);
			}
			parent._commandController= {
					commandStack: this._form.repository
				};
		}
		
		this._form._setRoot(parent);
		
		var form = this._form.template.form ? this._form.template.form : this._form;		
        this.selectTypeDialog = new jazz.ui.Dialog({
    		primaryTitle: this.typeName,
            secondaryTitle: "",
            width: "650px",
            onClose: dojo.hitch(this, this.onClose),
            contentNode: form.domNode
        });
        this._form.domNode.style.padding= "6px";
		this.listeners.push(dojo.connect(this._form, "onSubmitAndCommit", this, "add")); // called on OK, after committing change to repository (means after locking is done too)
		this.listeners.push(dojo.connect(this._form, "onCancel", this, "onCancel"));	
	},
	
	add: function() {
		this.onSubmit(this._form.getTarget());
		this._form.setChangeDetails();
		
		this.destroy();
	},
	
	onCancel: function(){
		this.destroy();
	},
	
	destroy: function() {
		this.selectTypeDialog.close();
		if(this._form){
			this._form.destroyRecursive();
		}
	},

	onSubmit: function(createdResource){
		/*To be called by other places */
	},
	
	onClose: function() {
		dojo.forEach(this.listeners, dojo.disconnect);
		this.listeners = [];
	}
});
})();