/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2012. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("rmpw.forms.ui.FileReferencesActionsTable");

dojo.require("rmpw.forms.ui.ReferencesActionsTable");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmpw.forms.ui.SelectOrCreateFile");

//Load async modules
var FileUploader= "rmpw.forms.ui.FileUploader";
var imageLocationLookup = net.jazz.ajax._contextRoot + "/fileUpload/locationFor?resourceUri=";

/* 
 * Advanced ReferencesActionsTable to support file/image upload. Used in two cases:
 * - For file upload form that allows the file upload. In this case, isFileUploadForm is true.
 * - For forms opened from a File or Image reference property. 
 */
dojo.declare("rmpw.forms.ui.FileReferencesActionsTable", rmpw.forms.ui.ReferencesActionsTable, {
	
	isImage: null,
	isFileUploadForm: null,
	fileUploaderMessages: null,
	
	postMixInProperties: function(){
		this.inherited(arguments);
		
		//FIXME: Should have a message file for this class only
		dojo.requireLocalization("rmpw.forms.ui", "FileUploaderMessages");
		this.fileUploaderMessages = dojo.i18n.getLocalization("rmpw.forms.ui", "FileUploaderMessages");
	},
	
	useSelectDialog: function(args){
		var selectArgs = this.getSelectDialogArgs(args);
		selectArgs.isFileUploadForm = this.isFileUploadForm;
		selectArgs.isImage = this.isImage;
		selectArgs.parentForm = this.form;
		var selectOrCreateResource = new rmpw.forms.ui.SelectOrCreateFile(selectArgs);
		
		selectOrCreateResource.useSelectDialog();
	},
	
	editRefrence: function(dataEntry, currentRow, currentActionsCell) {
		this._row = currentRow;
		this.useSelectDialog(currentRow);
	},	
	
	getPresentationUri: function(about, resourceLabel){
		if(about)
			return rmps.app.ui.internal.pages.design.DesignUtil.getFileUri((about + "/" + resourceLabel));
		else
			return rmps.app.ui.internal.pages.design.DesignUtil.getFileUri(this.getFocusReference());
	},
	
	getUri: function(about, resourceLabel){
		return about + "/" + resourceLabel;
	},
	
	getLabel: function(object, guess) {
		if (object && object.__parent && object.__parent.rdfs_label) {
			return object.__parent.rdfs_label;
		}
		return this.inherited(arguments);
	},
	
	getActionDescriptors: function(){
		if(this.isFileUploadForm){
			return [{actionCallback: dojo.hitch(this, this.editRefrence), 
					args: {imgClass: 'edit', 
					       title: this.getFocusReference() ? this.fileUploaderMessages.openReplaceFile : this.fileUploaderMessages.upload}}];
		}
		return this.inherited(arguments);
	},
		
	getTargetRdfAbout: function(logicalUri, target) {
		if (target.dmfile_location) {
			return target.dmfile_location;
		}
		return this.inherited(arguments);
	},
	
	onSetReference: function(args){
		if (args.target && args.target.rdf_about && args.target.rdf_about.indexOf("fileUpload") === -1) {
			// target points to the image document, we need image location - go get it
			var self = this;
			var lookupArgs = {
				url: imageLocationLookup + args.target.rdf_about,
				load: dojo.hitch(this, function(location, ioargs){
					args.target.rdf_about = location ? location : "fileUpload"; // shouldn't happen that location is not returned
					self.onSetReference(args);
				})
			};
			rmps.core.client.xhrGet(lookupArgs, false);
		} else {
			this.inherited(arguments);
		}
	}
	
});

})();