/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2012-2013. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function() {	
dojo.provide("rmpw.forms.ui.FileUploader");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dijit.layout._LayoutWidget");
dojo.require("rmps.ui.Footer");
dojo.require("rmpw.forms.ui.MimeTypes");

var _currentUploadFrameNumber = 100;
var IMAGES_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "fileUpload");
var FILE_UPLOAD_SUCCESS = "Success:";
var IMAGE_EXPECTED = "ErrorImageExpected";
var NON_IMAGE_EXPECTED = "ErrorNonImageExpected";
var MIME_TYPES = rmpw.forms.ui.MimeTypes;

/**
 * Dialog content wdiget for file uploading and viewing.
 * 
 * @author Lidija Grahek
 */
dojo.declare("rmpw.forms.ui.FileUploader", [dijit._Widget, dijit._Templated], {
	widgetsInTemplate: true,
	
	"class": "rmpw-dsv-editor-form-ui-file-uploader",
	
	frontSideUri: null,
	
	_footer: null,
	
	templateString: dojo.cache("rmpw.forms", "ui/templates/FileUploader.html"),
	
	postMixInProperties: function(){
		this.inherited(arguments);
		
		dojo.requireLocalization("rmpw.forms.ui", "FileUploaderMessages");
		this.messages = dojo.i18n.getLocalization("rmpw.forms.ui", "FileUploaderMessages");
	},
	
	postCreate: function () {
		this.inherited(arguments);
		
		if (this.frontSideUri && this.frontSideUri.$ref) {
			// uri is a Resource, therefore $ref
			this.frontSideUri = this.frontSideUri.$ref;
		}
		this.connect(this.fileLocation, "onchange", "handleFileChange");
		this.connect(this.buttonView, "onclick", "_viewFile");
		this.loadingMsg.style.display = "none";
		
		if (this._isReadOnly) {
			this.handleFileTitle.style.display = "none";
			this._uploadForm.style.display = "none";			
		}else{
			this._footer= new rmps.ui.Footer({
				okMessage: this.frontSideUri?this.messages.upload: this.messages.uploadAndSave,
				cancelMessage: this.messages.close
			});
			this.connect(this._footer, "onOK", this._performUpload);
			this.connect(this._footer, "onCancel", this._cancel);
			
			this._footerContainer.addChild(this._footer);
		}
		this._setupNewOrReplaceMode();
	},
	
	_setupNewOrReplaceMode: function () {
		if (this.frontSideUri) {
			// file already exists, we are in replace mode
			this.buttonView.style.display = "";
			this.handleFileTitle.style.display = "";
			if (this.isImage) {
				this._setPreviewImage(this.frontSideUri);				
			} else {
				this._setPreviewImage(null);
			}			
		} else {
			// this is a new record - the first file is yet to be uploaded
			this.buttonView.style.display = "none";
			this.handleFileTitle.style.display = "none";
			this._setPreviewImage(null);
		}
		// in both cases, file fields are blank
		this._clearFields();
	},
		
	handleFileChange: function () {
		this.fileErr.innerHTML = "";
		if ( dojo.isString(this.fileLocation.value) && this.fileLocation.value !== "") {		
		    var file = this.fileLocation.files ? this.fileLocation.files[0] : this._getFileMockup();
            
            if (this.isImage && file.type.indexOf("image") !== 0) {
                // don't allow non-image upload when an Image instance is being created
                this.fileErr.innerHTML = this.messages.nonImageFileErr;
                return;
            }
            if (!this.isImage && file.type.indexOf("image") === 0) {
                // don't allow image upload when a File instance is being created
                this.fileErr.innerHTML = this.messages.imageFileErr;
                return;
            }
            this._footer.setOKEnablement(true); 
            
            //fill in type of resource
            this.mimeType.value = file.type || "application/octet-stream";

            //fill in name of the resource
            this.label.value = file.name || file.fileName; // <- FF 3.6 or older
                
            //fill in the file size if it's there, or remove the information from the screen
            if(file.size){
                this.fileSizeLbl.style.display = "";
                this.fileSize.innerHTML = file.size;
            }else{
                this.fileSizeLbl.style.display = "none";
            }
		} else {
			this._clearFields();
		}
	},
	
	_getFileMockup: function(){
        // IE does not support files property, so content type and size not known at this point;
        // we cannot check if the right file type is being uploaded - so we fake it by checking the file extension.
        // This will be checked again on the server and IMAGE_EXPECTED or NON_IMAGE_EXPECTED error returned in those cases.        
        var fileName = this.fileLocation.value.split("\\").pop(),               
            fileSuffix = fileName.match(/.(\w+)$/).pop(),
            fileType = ( fileSuffix && MIME_TYPES[fileSuffix] )? MIME_TYPES[fileSuffix] : "application/octet-stream";
 
        return { 
                 name: fileName,
                 type: fileType
               };
    },
	
	_clearFields : function() {
		if(this._footer){
			this._footer.setOKEnablement(false);			
		}
		this.fileLocation.value = "";
		this.label.value = "";
		this.mimeType.value = "";
		this.fileSize.innerHTML = "";					
	},
	
	_viewFile : function() {
		if (this.frontSideUri) {
			window.open(this._getFileUri(this.frontSideUri), "_blank");
		}
	},
	
	_setPreviewImage: function (uri) {
		if (uri) {
			this.imageArea.src = this._getFileUri(uri);
			this.imageArea.style.display = "";
		} else {
			this.imageArea.style.display = "none";
		}
	},
	
	_getFileUri: function(uri) {
		// Add Math.random() number to ensure that the file is refreshed.
		uri += "?" + Math.random();
		
		// Also, make sure the file comes from the correct context
		var contextParam = this.getContextParam();
		if (contextParam) {
			uri += "&" + contextParam;
		}		
		return uri;
	},
	
	getContextParam: function() {
		var baseline = jazz.app.context.get().baseline;
		var changeset = jazz.app.context.get().changeset;
		var baselineParam = baseline? rmps.core.util.IRmpsConstants.BASELINE + '=' + encodeURIComponent(baseline): null;
		return changeset?baselineParam + "&" + rmps.core.util.IRmpsConstants.CHANGESET + '=' + encodeURIComponent(changeset): baselineParam;
	},
	
	_performUpload: function() {
		this.loadingMsg.style.display = "";
		var fullPath = this.fileLocation.value;
		//this._isUploadInProgress = true;
		this._priorFullPath = fullPath;
		var parts= fullPath.split("\\");
		var fileName= parts[parts.length-1];

		if(this._currentBindRequest != null) {
			this._currentBindRequest.cancel();
			this._currentBindRequest  = null;
		}	
			
		/*
		 * Internet Explorer (with dojo) has a bug such that if you wait ten seconds after one file upload, the 
		 * next one breaks. Resetting the iframe fixes the problem (but unfortunately reaches into iframe private variables).
		 */
		if(dojo.io.iframe._frame) {
			dojo.doc.body.removeChild(dojo.io.iframe._frame);				
		}
		frameName = "dojoIoIframe" + _currentUploadFrameNumber++;
		dojo.io.iframe._iframeName = frameName;
		dojo.io.iframe._frame = null;
		var isDirectUpload = !this.parentForm.featureId;
		var url = IMAGES_PATH + "/" + jazz.app.context.get().itemId + 
				"?size=" + this.fileSize.innerHTML + 
				"&contentType=" + this.mimeType.value + 
				"&isImage=" + this.isImage +
				"&isDirectUpload=" + isDirectUpload;
		if (this.frontSideUri) {
			url += "&frontSideUri=" + this.frontSideUri;
		}
		var contextParam = this.getContextParam();
		if (contextParam) {
			url += "&" + contextParam;
		}				
		this._currentBindRequest = dojo.io.iframe.send({
			form: this._uploadForm,
			url: url, 
			handleAs:  "html",
			handle: dojo.hitch(this, function(response, ioArgs) {
				//this._isUploadInProgress = "false";
				this.loadingMsg.style.display = "none";
				if (response instanceof Error){
					if(response.dojoType && response.dojoType === "cancel") {
						return; // this is a canceled request, ignore it
					}
					return rmps.core.error.create(this.messages.errorUploadingFile + response.message).dispatch();
				} else {
					if (response.body) {					
						var responseMsg = response.body.innerHTML;
						var successIndex = responseMsg.indexOf(FILE_UPLOAD_SUCCESS);						
						if (successIndex >= 0) {
							var endIndex = responseMsg.indexOf("&lt;/body&gt;");
							var uri;
							if (endIndex < 0) {
								// in IE, responseMsg is just the response string without html tags
								uri = responseMsg.substring(successIndex + FILE_UPLOAD_SUCCESS.length);
							} else {
								uri = responseMsg.substring(successIndex + FILE_UPLOAD_SUCCESS.length, endIndex);
							}
							
							this.fileErr.innerHTML = "";
							var isNew = !this.frontSideUri;
							this.frontSideUri = uri;
							
							this.onOK(uri, this.label.value);
							
							if(!isNew){
								this._setupNewOrReplaceMode();
							}
						} else if (responseMsg.indexOf(NON_IMAGE_EXPECTED)) {
							this.fileErr.innerHTML = this.messages.nonImageFileErr;
						} else if (responseMsg.indexOf(IMAGE_EXPECTED)) {
							this.fileErr.innerHTML = this.messages.imageFileErr;
						} else {
							// something went wrong during the upload
							this.fileErr.innerHTML = responseMsg;
						}
					} else {
						// didn't get the response from our service, some server error happened
						this.fileErr.innerHTML = this.messages.unexpectedError;
					}
				}					
				this._currentBindRequest = null;
			})		  
		});
	},
	
	_cancel : function () {
		// TODO: test closing the dialog in the middle of upload
		if(this._currentBindRequest != null) {
			this._currentBindRequest.cancel();
			this._currentBindRequest  = null;
		}		
		this.onCancel();
	},
	
	onOK: function(uri, label){/*To be implemented by callers */},
	onCancel: function(){/*To be implemented by callers */}
	
});
})();