/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2011, 2012. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function() {
dojo.provide("rmpw.forms.ui.FormPage");

dojo.require("dijit.layout._LayoutWidget");

dojo.require("dojo.i18n");

dojo.require("rmpw.common.util.IdentifierUtil");
dojo.require("rmpw.common.vcs.ref");
dojo.require("rmps.core.client");
dojo.require("rmpw.common.util.FormsLauncher");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("rmpw.forms.ui.ModelValidationUtil");

dojo.requireLocalization("rmpw.forms.ui", "FormPageMessages");

var formMessages = dojo.i18n.getLocalization("rmpw.forms.ui", "FormPageMessages");
var ModelValidationUtil = rmpw.forms.ui.ModelValidationUtil;
dojo.declare("rmpw.forms.ui.FormPage", [dijit.layout._LayoutWidget], {
	
	postCreate: function(){
		this.inherited(arguments);
		
		var query = location.hash;
		var paramMap = dojo.queryToObject(query);
		this.test = paramMap.test ? paramMap.test == "true" : false;
		this.documentUri = this.editor.editorInput.resourceInfo.documentUri;
    	this.objectUri = this.editor.editorInput.resourceInfo.uri;
    	this.create = (this.editor.editorInput.mode === "create");
    	this.isReload = false;

    	this.connect(this.editor, "onSaveComplete", this.onSaveComplete);
    	this.connect(rmps.ui.editor.PageHelper.getInstance(rmps.app.ui.internal.pages.design.DesignUtil.PAGE_ID), "onActiveEditorChanged", this.resize);
    	
    	if (this.create) {
    		rmpw.common.util.FormsLauncher.invalidateCache();
    	}
    	
		this._onLoad();
	},
	
	destroy: function(){
		if(this._form){
			this._form.destroyRecursive();
		}
		return this.inherited(arguments);
	},
    
	doSave: function(/*rmps.core.TaskMonitor*/monitor) {
		/*
		 * Hook for being tab content in Design page
		 */
		this._onSave(monitor);
	},
	
	doRefresh: function() {
		/*
		 * Hook for being tab content in Design page
		 */
		this._onLoad();
    	this.isReload = false;
	},
	
	validate: function(/*Array*/errors) {
		return ModelValidationUtil.validatePopup(this._form, errors);
	},
	
	initializeToolBar: function(
			/* jazz.ui.SimpleToolbar */ toolbar,
			/*function*/setDirty,
			/*function*/enableRefresh) {
		/*
		 * Hook for being tab content in Design page
		 * @param tooblar
		 * 					is passed so extra buttons can be added if required
		 * @param setDirty(bool?) 
		 * 					is a function that should be called when the state of the
		 * 					Diagram is dirtied or un-dirtied.
		 * @param enableRefresh(boolean)
		 * 					is a function to call to allow the refresh button to be
		 * 					enabled or disabled
		 */
		
		this["enableRefresh"] = enableRefresh;
		
		this["setDirty"] = setDirty;
		
		this.toolbar = toolbar;
	},
	
	_onLoad: function() {
		this.editor.showLoading();
		this._onLoadSource();
	},
	
	_getResource: function(identifier) {
		var resourceInfo = this.editor.editorInput.resourceInfo;
		return resourceInfo.modelIndex[rmpw.common.util.IdentifierUtil.normalizeURI(this.documentUri) + identifier];
	},
	
	_onLoadSource: function() {
		var resourceInfo = this.editor.editorInput.resourceInfo;
		
		// TODO (MG): there should always only be one root, the document object.
		var focusObject = resourceInfo.modelIndex[this.create ? "http://new" : this.objectUri];
		if (focusObject != null) {
			this._formObject = focusObject;
			this.rdf_type = dojo.map(focusObject.rdf_type, function(type) { return type.rdf_about||type.$ref; });
			this._onLoadForm();
		} else {
			this._handelError(null, "Data Object is null, form will not load");
		}
	},
	
	_onLoadForm: function() {
		this.packageName = "properties";
		var objectTypes = this.rdf_type;
		if (!objectTypes){
			this._handelError(null, "No Data Object type defined, form will not load");
		}
		var self = this;
		this.formName = "_";
		dojo.forEach(objectTypes, function(objectType, i) {
			var index = objectType.indexOf('#');
			// Handle the use case of the fragment name delimiter is '/' not '#'
			if (index == -1){
				index = objectType.lastIndexOf('/');
			}
			objectType = objectType.substring(index + 1);
			self.formName += objectType + (i != objectTypes.length-1 ? "__" : "");
		});		
		
		this.formName.replace(" ", "_");
		
		rmpw.common.util.FormsLauncher.loadForm({ 
				formName: this.formName,
				objectTypes: objectTypes,
				readOnly: this.isReadOnly,
				standAlone: true,
				editor: this.params.definition.uri,
				create: this.create
		},dojo.hitch(this, this._onSetFormObject, objectTypes),
		  dojo.hitch(this, this._handelError, objectTypes)
		);
	},
	
	_onSetFormObject: function(objectTypes) {
		if (!rmpw.dsf || !rmpw.dsf.codegen[this.packageName].forms[this.formName]){
			this._handelError(null, "Failed to generate form");
			return;
		}
		this._form = new rmpw.dsf.codegen[this.packageName].forms[this.formName]();
		this._form._setResource({
				objectTypes: objectTypes,
				page: this,
				id: this.objectUri,
				documentObject: this.editor.editorInput.resourceInfo.model,
				resourceUri: this.resourceUri,
				repository: this.editor.editorInput.resourceInfo.repository,
				formObject: this._formObject,
				modelIndex: this.editor.editorInput.resourceInfo.modelIndex,
				editor: this.params.definition.uri
		});
			
		//TODO: owl_oneOf should have its own template, with the section below being part of its logic
		//adding a One Of is only enabled once the resource has been saved 
		//(i.e when this._form._resource.id stores the resource's URI)
		if( rmpw.forms.ui.ModelValidationUtil.isUri(this._form._resource.id) ){
		    template = this._form.template;
            for(unique in template){
                if(template.hasOwnProperty(unique) && template[unique].label === "owl_oneOf"){
                    template[unique].addCreationActions("resource saved");
                }
            }
		}
		
		// Clear all nodes
		if (this._currentForm){
			this.removeChild(this._currentForm);
			this._currentForm.destroy();
		}
		
		this._form.immediate= true;
		this._currentForm = this._form;
		this.domNode.style.overflow= "auto";
		this.domNode.style.padding= "6px";
		this.addChild(this._currentForm);
		
		this._currentForm.startup();
				
		// Check if the form needs to say anything about the toolbar
		if (this._form.updateToolbar) {
			this._form.updateToolbar(this.toolbar);
		}
						
		if (this._formObject) {
			var method = this._form.setterNames[this.formName];
			this._form[method](this._formObject);
						
			setTimeout( dojo.hitch(this, function() {
				this.setDirty(false);
			}, 10));	
		}else{
			this._handelError(null, "Missing data object: Cannot set the form fields");
		}
		this.editor.loadResourceCompleted();
		setTimeout(dojo.hitch(this, function() {
			this.resize({});
		}), 100);
    },
    
    _onSave: function(/*rmps.core.TaskMonitor */ monitor) {
    	//Safe check. Form will only be null in case it is being destroyed
    	if (this._form){
    		var task = monitor.beginTask("", 1);
    		this._saveChanges(monitor, task);	
    	}
    },
    
    _saveChanges: function(monitor, task){
    	// if we were in create mode, then invalid the form cache since forms may generate different
    	// in non-create mode.
    	if (this.create) {
    		rmpw.common.util.FormsLauncher.invalidateCache();
    	}
		
		// do the usual save for the stand-alone form
		// but first check with the form if it has to do any pre-processing
		if (this._form && this._form.preProcessOnSave) {
			this._form.preProcessOnSave();
		}		
		this.editor.doSaveDesignEditor(monitor);
		task.done();
    },

    _setEditable: function() {
    	this.isReadOnly = false;
		this.doRefresh();
    },
    
    setReload: function() {
    	this.isReload = true;
    },

    onSaveComplete: function(){
    	rmpw.common.util.FormsLauncher.updateForms(this.documentUri);
    	if (this.isReload){
			this.editor.refresh();
		}    		
    },
        
    _handelError: function(error, message){
		var details = [];

    	if (error){
    		console.error(error);
    		if (error.message){
    			details.push({message: error.message});
    		}
    	}else{
    		console.error(message);
    	}
    	if (!message){
    		message = error.message ? error.message : "Undefined Error";
    	}
    	this.editor.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: message,
			details: details.length > 0 ? details : null,
			closable: true
		});
    }
});
})();