/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

( function(){

  dojo.provide("rmpw.jazz.ui.Layout");

  dojo.require("dojo.dnd.Source");
  dojo.require("dojo.data.ItemFileReadStore");
  dojo.require("dojox.json.ref");

  dojo.require("dijit.CheckedMenuItem");
  dojo.require("dijit.Dialog");
  dojo.require("dijit.Menu");
  dojo.require("dijit.TooltipDialog");
  dojo.require("dijit.Toolbar");
  dojo.require("dijit.form.Button");
  dojo.require("dijit.form.DropDownButton");
  dojo.require("dijit.form.ComboBox");
  dojo.require("dijit.form.TextBox");
  dojo.require("dijit.form.ValidationTextBox");
  dojo.require("dijit.layout.AccordionContainer");

  dojo.require("jazz.ui.SearchBox");
  dojo.require("rmpw.common.vcs.ChangeSet");
  dojo.require("rmps.diagram.data.JtsRdfStore");
  dojo.require("rmpw.jazz.ui.Controls");
  dojo.require("rmpw.jazz.ui.LineStyler");
  dojo.require("rmpw.jazz.ui.DiagramWidget");
  dojo.require("rmpw.widgets.Events");
  dojo.require("rmpw.widgets.SettingsButton");
  dojo.require("rmpw.widgets.SettingsToggle");
  dojo.require("rmpw.widgets.MenuRegion");
  dojo.require("rmpw.widgets.slider.HorizontalSlider");
  dojo.require("rmpw.widgets.slider.PercentDisplay");

  dojo.require("rmpw.widgets.slider.LabelWidget");

  var controls = rmpw.jazz.ui.Controls.items;

  dojo.declare("rmpw.jazz.ui.Layout", [dijit._Widget, dijit._Templated], {

  	 templateString: "<div dojoAttachPoint='containerNode'>" + 
  	                 	"<div dojoAttachPoint='toolsNode' style=\"width:100%;\"/></div>" + 
  	                 	"<div dojoAttachpoint='borderNode' style=\"width:100%;border-top:1px solid #E8E8E8;margin-top:1px;padding-top:1px;\"</div>" + 
  	                 	"<div dojoAttachPoint='canvas'/><div dojoAttachPoint='right'/>" + 
  	                 "</div>",

     size: null,
     toolbar: null,
     toolbarHeight: 33,

 	 prefix: null,
     viewer: null,
     centreContent: null,
     fileSelect: null,
     domainName: null,
     layoutButton: null,
     connections: null,
     selection: null,
     lineStyler: null,
     linetools: false,
     outlineState: true,

     mobile: false,
     
     _cometdinitialized: false,
     saveBtn:false,

     constructor: function() {
        this.connections = [];
        this.lineStyler = new rmpw.jazz.ui.LineStyler();
        
        var uagent = navigator.userAgent.toLowerCase();
       		
	   	if( uagent.search( 'ipad' ) > -1 ){
	   		this.mobile = true;
		}
     },

     postCreate: function() {
    	 this.buildToolBar();
    	 this.initializeDiagram();
     },

     addButton: function( label, call, icon, show ){
    	 if(show == null){
    		 show = false;
    	 };

    	 var button = new dijit.form.Button({
     		label: label,
     		showLabel: show,
     		onClick: call,
     		iconClass: icon });

    	 this.addTool( button );

    	 return button;
     },

     buildToolBar: function() {
        this.toolbar = new dijit.Toolbar({}, this.toolsNode);
        dojo.addClass(this.toolbar.domNode, "rmpsToolbar");
        dojo.removeClass(this.toolbar.domNode, "dijitToolbar");

		this.toolbar.domNode.style.height = "30px";
		this.toolbar.domNode.style.background = "none repeat scroll 0 0 white";
		this.toolbar.domNode.style.marginTop = "2px";
		this.toolbar.domNode.style.borderBottom = "none";

        for(var item = 0; item < this.tools.length; item++ ) {
        	if(this.tools[item] != null && this.tools[item].display == true) {
       			 switch( this.tools[item].name ){
       			 	case controls.SAVE:
	       			 	this.saveBtn = this.addButton( "Save", dojo.hitch( this, "saveDiagram" ), "saveIcon" );
	       			    this.saveBtn.attr('disabled', true);
	       			 	break;

       			 	case controls.SETTINGS:
       			    	this.addSettingsDropDown( this.tools[item].subcontrol );
       			    	break;

       			 	case controls.FILEPICKER:
       			 		this.addFilePicker( this.tools[item].query );
       			 		break;

       			 	case controls.COMBINEDPICKER:
       			 	    this.addToolTipDropDown();
       			 		break;

       			 	case controls.INTERPRETATION_PICKER:
       			 		this.addInterpretationPicker();
       			 		break;

       			 	case controls.TOOLING_PICKER:
       			 	    this.addToolingPicker();
       			 		break;

       			 	case controls.SEARCHBOX:
       			        this.addSearchBox( this.tools[item].scopes );
       			    	break;

       			 	case controls.RELOAD_DIAGRAM_CODE:
       			    	this.addButton( "Reload Diagram Code", dojo.hitch( this, "loadDiagramCode" ), "resetIcon" );
       			    	break;

       			 	case controls.RELOAD_DIAGRAM_CONTENT:
       			    	this.addButton( "Reload Diagram Content", dojo.hitch( this, "loadDiagramContent" ), "resetIcon" );
       			    	break;

       			 	case controls.UNDO:
       			    	this.addButton( "Undo", dojo.hitch( this, "undo" ), "undoIcon" );
       			    	break;

       			 	case controls.REDO:
       			    	this.addButton( "Redo", dojo.hitch( this, "redo" ), "redoIcon" );
       			    	break;

       			 	case controls.DELETE:
       			    	this.addButton( "Delete", dojo.hitch( this, "deleteDiagram" ), "deleteIcon" );
       			    	break;

       			 	case controls.LAYOUT:
       			    	this.addLayoutDropDown();
       			    	break;

       			 	case controls.ZOOM:
       			 		this.addZoom();
       			 		break;
       			 }
       		 }
       	 }
     },

	addSearchBox: function(scopes) {
		this.searchBox = new jazz.ui.SearchBox({
			scopes: scopes,
			tooltip: "search",
			onExecute: dojo.hitch(this, "loadDiagram")
		});

		dojo.style(this.searchBox.domNode, "width", "250px");
		if(!dojo.isIE) {
			dojo.style(this.searchBox.domNode, "display", "inline-block");
		}

		dojo.style(this.searchBox.domNode, "margin", "0px");
		dojo.style(this.searchBox.domNode, "position", "relative");
		dojo.style(this.searchBox.domNode, "verticalAlign", "bottom");

		this.addTool( this.searchBox );
	},

     addToDialog: function( dialog, element, name ){

    	 var row = document.createElement( 'span' );

    	 var label = document.createElement('div');
         label.innerHTML = name;
         // row.appendChild( label );

         dialog.containerNode.appendChild( row );

         row.appendChild( element );

         row.style.display = 'inline-block';

         var space = document.createElement('div');
         space.style.height = '3px';
         dialog.containerNode.appendChild( space );
     },

     addToolTipDropDown: function(){
         this.createFilePicker();
         this.createInterpretationPicker();
         this.createToolingPicker();

         var dialog = new dijit.TooltipDialog({content: '' });
         dialog._setStyleAttr("min-width: 150px");

         this.addToDialog( dialog, this.fileSelect.domNode,           "          File: ");
         this.addToDialog( dialog, this.interpretationSelect.domNode, "Interpretation: ");
         this.addToDialog( dialog, this.toolingSelect.domNode,        "       Tooling: ");

         this.picker = new dijit.form.DropDownButton({
              label: this.configuration.feed,
              dropDown: dialog
         });

         this.addTool( this.picker );

     },

     createInterpretationPicker: function() {
    	 var fileStore = new dojo.data.ItemFileReadStore({
             url: "resource/itemCol/Interpretation"
         });

         this.interpretationSelect = new dijit.form.ComboBox({
             name: "interpretation",
             value: this.configuration.interpretation,
             store: fileStore,
             searchAttr: "id",
             onChange: dojo.hitch( this, "loadDiagram" )
         });
     },

     addInterpretationPicker: function() {
    	 this.createInterpretationPicker();
         this.addTool(this.interpretationSelect);
     },

     createToolingPicker: function() {
    	 var fileStore = new dojo.data.ItemFileReadStore({
             url: "resource/itemCol/Tooling"
         });

         this.toolingSelect = new dijit.form.ComboBox({
             name: "tooling",
             value: this.configuration.tooling,
             store: fileStore,
             searchAttr: "path",
             onChange: dojo.hitch( this, "loadDiagram" )
         });
     },

     addToolingPicker: function() {
    	 this.createToolingPicker();
         this.addTool( this.toolingSelect );
     },

     addCheckedMenuItem: function(menu, label, checked, id, call) {
    	 var item = new dijit.CheckedMenuItem({
    		 id: id,
             label: label,
             value: "agreed",
             checked: checked,
             onChange: call
    	 });

    	 menu.addChild( item );
     },

     addSettingsDropDown: function(){
    	 var menu = new dijit.Menu({ style: "display: none;" });
    	 var subcontrol = this.settings;

    	 for(var item = 0; item < subcontrol.length; item++) {
    		 if(subcontrol != null && subcontrol[item].display == true) {
    			 switch(subcontrol[item].name){
    		 		case controls.GRID:
    		 			function setGrid(val){
    		 			 	dojo.publish('gef/diagram/grid', [val] );
    		 		    }
    		 			this.addCheckedMenuItem( menu, "Grid",
    		 					 subcontrol[item].enabled,
    		 					 subcontrol[item].id,
    		 					 setGrid );
    		 			break;

    		 		case controls.ANIMATION:
    		 			function setAnim(val){
    		 				dojo.publish('gef/diagram/animation', [val] );
    		 			};
    		 			this.addCheckedMenuItem( menu, "Animation",
								 subcontrol[item].enabled,
								 subcontrol[item].id,
								 setAnim );
						break;

    		 		case controls.OUTLINE:
    		 			this.addCheckedMenuItem(menu, "Outline",
								 subcontrol[item].enabled,
								 subcontrol[item].id,
								 dojo.hitch(this, "toggleOutline"));
    		 			this.outlineState = subcontrol[item].enabled;
    		 			break;

    		 		case controls.OBSTRUCTIONS:
    		 			function setObs(val){
    		 				dojo.publish('gef/diagram/avoidObstructions', [val] );
    		 			};
    		 			this.addCheckedMenuItem( menu, "Avoid Obstructions",
								 subcontrol[item].enabled,
								 subcontrol[item].id,
								 setObs );
    		 			break;

    		 		case controls.SHARED:
    		 			var self = this;
    		 			function setShared(val){
    		 			    // Initialize cometd
// TODO Re-evaluate comet use
//    		 			    if (!self._cometdinitialized) {
//    		 			    	dojo.require("rmpw.cometd.Initializer");
//    		 			    	dojo.require("rmpw.cometd.subscribers.CommandSubscriber");
//    		 			    	dojo.require("rmpw.cometd.subscribers.MovementSubscriber");
//
//    		 			    	rmpw.cometd.Initializer.initialize();
//
//    		 			    	// Subscribe to cometd channels
//    		 			    	new rmpw.cometd.subscribers.CommandSubscriber(self.viewer.diagramLayer.viewer);
//    		 			    	new rmpw.cometd.subscribers.MovementSubscriber(self.viewer.diagramLayer.viewer);
//    		 			    	self._cometdinitialized = true;
//    		 			    }
//    		 			    // Notify subscribers
//    		 				dojo.publish('gef/diagram/shared', [val]);
    		 			};
    		 			this.addCheckedMenuItem(menu, "Share diagram",
    		 					subcontrol[item].enabled,
    		 					subcontrol[item].id,
    		 					setShared);
    		 			break;
    			 }
    		 }
    	 }

    	 this.setButton = new dijit.form.DropDownButton({
             label: "Settings",
             showLabel: false,
             name: "settings",
             dropDown: menu,
             iconClass: "settingsIcon",
             id: this.prefix + ".settingsDropDown"
         });

    	 this.addTool(this.setButton);
     },

     resetZoom: function() {
    	 this.slider.set( 'value', 1 );
    	 this.viewer.diagramLayer.setScale(1);
     },

     addZoom: function() {
    	 this.slider = new rmpw.widgets.slider.HorizontalSlider({
             name: "slider",
             value: 1,
             minimum: 0.25,
             maximum: 5,
             intermediateChanges: true,
             style: "width:100px;top:5px;",
             discreteValues: 20, // 25% - 500% at 25% intervals
             showButtons: false,
             handleSrc: '/rmps/web/rmps.graphics/diagram/images/etool16/SliderHandle.png'
         });

    	 this.slider.domNode.style.display = "inline-block";
    	 this.label = new rmpw.widgets.slider.LabelWidget();
 	  	 this.label.setLabel("100%");

 	   	 this.slider.mylabel = this.label;
 	   	 
 	   	 if( this.mobile == false ){
 	   		this.addTool( this.slider ); 
 	   	 }
 	  	 
 	  	 this.addTool( this.label );

    	 this.percent = this.addButton( "100%", dojo.hitch( this, "resetZoom" ), "actualIcon", false );
     },

     reWorkLayout: function( id ){
    	this.viewer.getCommandController(this.canvas).execute(id, { containerMapping: this.viewer.diagramLayer.viewer.diagram });
     },

     createMenuItem: function( menu, label, id, call, icon ) {
    	 menu.addChild( new dijit.MenuItem({
        	label: label,
        	id: id,
        	onClick: dojo.hitch( this, "reWorkLayout", call  ),
        	iconClass: icon
        }) );
     },

     addLayoutDropDown: function(){
//         var menu = new dijit.Menu( { style: "display: none;" } );
//
//         this.createMenuItem( menu, "Graph", this.prefix + '.Graph', 'rmpw.diagrams.layout.commands.AutoLayoutCommand', "graphIcon" );
//         this.createMenuItem( menu, "Radial", this.prefix + '.Radial', 'rmpw.diagrams.layout.commands.RadialLayoutCommand', "radialIcon" );
//         this.createMenuItem( menu, "Tree", this.prefix + '.Tree', 'rmpw.diagrams.layout.commands.TreeLayoutCommand', "treeIcon" );
//
//         this.layoutButton = new dijit.form.DropDownButton({
//             label: "Layout",
//             showLabel: false,
//             name: "layout",
//             dropDown: menu,
//             iconClass: "layoutIcon",
//             id: this.prefix + ".layoutDropDown"
//         });
//
//         this.addTool( this.layoutButton );
	},

	loadDiagramCode: function(resourceInfo) {
		this.loadDiagram(resourceInfo, true);
	},

	loadDiagramContent: function(resourceInfo) {
		this.loadDiagram(resourceInfo, false);
	},

	loadDiagram: function(resourceInfo, forceCodeReload) {
		this.viewer.diagramLayer.viewer.repository = resourceInfo.repository;
		this._clean();
		var diagramStore = new this.configuration.store( this.viewer.diagramLayer.viewer, forceCodeReload);
		diagramStore.fetch(resourceInfo, this.definition);
		if(this.picker) {
			this.picker.attr('label', this.fileSelect.value );
		}
		if(this.viewer.diagramLayer.viewer != null) {
			this.connections.push( dojo.connect( this.viewer.diagramLayer.viewer.getSelectionController(), "onSelectionChanged", this, "handleSelection" ) );
		}
		this.addDynamicPolicyTools();
	},

	// Mixin - callback
	_setDirty: function(){},

	_clean: function() {
		if(this.viewer.diagramLayer.viewer != null ) {
			var commandStack = this.viewer.diagramLayer.viewer.getCommandController(this.viewer.containerNode).commandStack;
			var handle = dojo.connect(commandStack, "postExecuteHook", dojo.hitch(this, function(e){
				if (this.saveBtn) {
					// leave this is for now until all dependent
					// editors are updated not to need this
					this.saveBtn.attr('disabled', false);
				}
				this._setDirty();
			})); //once we handle diagram close, the handle to disconnect
    		this.removeDynamicPolicyTools();

    		for (var i = 0, connection; connection = this.connections[i]; i++) {
				dojo.disconnect(connection);
			}
			this.connections = [];
		}
	},

	initializeDiagram: function(){
		this.configuration.canvas = this.canvas;
		this.configuration.slider = this.slider;
		this.configuration.label = this.label;
		this.configuration.percent = this.percent;

		this.viewer  = new rmpw.jazz.ui.DiagramWidget({ configuration: this.configuration });

		// TODO (MG): find a way around this
		// create global reference to viewer as break-in point for tests
		window.viewer = this.viewer;

		dojo.marginBox( this.viewer.domNode,  "content-cell" );
		this.viewer.startup();

		if(this.viewer.diagramLayer.viewer != null && this.viewer.diagramLayer.viewer.getSelectionController) {
			this.connections.push( dojo.connect( this.viewer.diagramLayer.viewer.getSelectionController(), "onSelectionChanged", this, "handleSelection" ) );
		}

		this.domNode.appendChild( this.viewer.domNode );
	},

	_selectionIncludesLine: function() {
		if(this.selection) {
			for(var i = 0; editController = this.selection[i]; i++) {
				if(editController.renderMapping.policies.graphEdge) {
					return true;
				}
			}
		}

		return false;
	},

	removeDynamicPolicyTools: function() {
		if (this._dynamicPolicyTools && this._dynamicPolicyTools.length > 0) {
			for (var i = 0; tool = this._dynamicPolicyTools[i]; i++) {
				this.removeTool(tool);
			}
		}
		this._dynamicPolicyTools = [];
	},

	addDynamicPolicyTools: function() {
		if( this.viewer.diagramLayer.viewer != null ){
			this.selection = this.viewer.diagramLayer.viewer.getSelectionController().getSelection();
		}

		for(var i = 0; mapping = this.selection[i]; i++) {
			for (var i=0, policy; policy=mapping.policies[i]; i++) {
				if (policy.getToolbarWidgets) {
					var widgets = policy.getToolbarWidgets();
					for (var j=0, widget; widget=widgets[j]; j++) {
						this._dynamicPolicyTools.push(widget);
						this.addTool(widget);
					}
				}
			}
		}
	},

	handleSelection: function(){
		if(this.viewer.diagramLayer.viewer != null) {
			this.selection = this.viewer.diagramLayer.viewer.getSelectionController().getSelection();
		}

		if(this.selection.length > 0 && this._selectionIncludesLine()) {
			if( this.linetools == false ) {
				this.lineStyler.addStyleTools(this.toolbar);
				this.linetools = true;
			}
			this.lineStyler.referenceLines(this.selection);
		}
		else {
			this.lineStyler.removeStyleTools();
			this.linetools = false;
		}
		this.removeDynamicPolicyTools();
		this.addDynamicPolicyTools();
	},

	spacer: function() {
		this.toolbar.addChild( new dijit.ToolbarSeparator() );
	},

	addTool: function(item){
		this.toolbar.addChild(item);
	},

	removeTool: function(item){
		item.destroyRecursive();
	},

	layout: function() { },

	toggleOutline: function() {
		this.outlineState = !this.outlineState;
		this.viewer.setOutline( this.outlineState );
	},

	togglePalette: function() {
		this.rightContent.toggle();
	},

	resize: function(s) {
		if(this.viewer) {
			this.viewer.resize(s);
		}
	},

	undo: function() {
		this.viewer.getCommandController(this.canvas).commandStack.undo();
	},

	redo: function() {
		this.viewer.getCommandController(this.canvas).commandStack.redo();
	}
});

})();

