/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2011, 2013. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("rmpw.forms.ui.ListSection");

dojo.require("rmps.ui.CollapsibleSection");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("rmpw.forms.ui.TableChangesMixin");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.core.clipboard.Clipboard");
dojo.require("rmps.core.explorer.ExplorerConstants");
dojo.require("rmps.app.core.util");
dojo.require("rmpw.common.util.IdentifierUtil");

var ResourceInfoClient= rmps.core.resource.ResourceInfoClient;
var clipboard = rmps.core.clipboard.clipboard;

/**
 * List Section wrapper 
 * 		Consists of styled table wrapped in a Styled Box (CollapsibleSection)  
 *  
 * @author Sameh Ayoub
 */
dojo.declare("rmpw.forms.ui.ListSection", [rmpw.forms.ui.TableChangesMixin, dijit._Templated], {
	
	widgetsInTemplate: true,
	
	//mixin
	form: null,
	isOrdered: null,
	secondaryTitle: null,
	collapsible: null,
	listSectionMessages: null,
	resourceEditorMessages: null,
	trim: rmps.ui.CollapsibleSectionStyles.GREY,
	closable: null,
	
	templateString: '<div dojoAttachPoint="mainNode">'+
							'<div dojoAttachPoint="section" primaryTitle="${primaryTitle}" collapsible="${collapsible}" secondaryTitle= "${secondaryTitle}" dojoType="rmps.ui.CollapsibleSection" trim="${trim}" closable="false">'+
								'<div dojoAttachPoint="tableContainer"</div>'+
							'</div>'+
					'</div>',
	
	constructor: function(args) {
		this.isOrdered= args.isOrdered;
		this.enableSelectAll = args.enableSelectAll;
		this.actions = [];
	},
	
	postMixInProperties: function(){
		dojo.requireLocalization("rmps.app.ui.internal.editors", "ResourceEditorMessages");
		this.resourceEditorMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "ResourceEditorMessages");
		
		dojo.requireLocalization("rmpw.forms.ui", "ListSectionMessages");
		this.listSectionMessages = dojo.i18n.getLocalization("rmpw.forms.ui", "ListSectionMessages");

		this.inherited(arguments);
	},
	
	buildRendering: function(){
		this.inherited(arguments);
		dojo.addClass(this.domNode, "rmpw-dsv-editor-form-ui-ListSection");
	},
	
	postCreate: function(){
		this.inherited(arguments);
		if (!this.readOnly){
			this.addCreationActions();
		}
	},
    	
	/**
	 * @param object 
	 * @param {rmpw.common.vcs.Change} changes 
	 */
	setValue: function(object, document, repository, feature) {
		this.object= object;
		this.document = document;
		this.repository = repository;
		this.featureId= feature;
		
		/*
		 * Handle special case for owl:oneOf - there's no way to specify this declaratively because in the oneOf collection 
		 * want to allow creation of individuals that are the same class as the instance of the root object of the form.
		 * This is not known at form generation time.
		 */
		if (this.featureId == "owl_oneOf") {
			this.typeUri = this.document.rdf_about;
			this.definableTypes = [ this.typeUri ];
			this.canSelect = false;
			this.document = false; // don't create individuals as separate documents
			this.typeLabel = "Individual";//$NON-NLS-1$ - meta-model terminology isn't translated
		}
		
		this._createTableContents();
		
		if (!this.isDocument && !this.dataTypeProperty) {
			// there is a chance some of list items are external fragments, or new internal fragments created from popup; for those, disable edits
			for (var i = 0; i < this.table.getRowsCount(); i++) {
				var targetRdf_about = this.table.getRowInfomation(this.table.getRowByIndex(i)).getDataEntry().rdf_about;
				if (!this.canEditItem(targetRdf_about)) {
					this._updateActionDisplay(i, "edit", "none");
				}
			}
		};
	},
		
	getData: function(){
		if(!this.isFragment|| this.dataTypeProperty){
			return dojo.isArray(this.object)?this.object: [];
		}
		
		if(!dojo.isArray(this.object)){
			return [];
		}
		return dojo.filter(this.object, dojo.hitch(this, function(item, i) {
			var types= item.rdf_type;
			if(!types){
				return false;
			}
			
			for(var index = 0, type; type = types[index]; index++) {
				if(type.rdf_about === this.typeUri){
					return true;				
				}
			}
			return false;
		}));
	},
	
		
	getActionDescriptors: function(){
		// in case of read-only collections, this method returns undefined
		
		if(this.readOnly){return;}
			
		var actions=  [
			{actionCallback: dojo.hitch(this, this.removeRefrence), args: {imgClass: "delete", title: this.tableChangesMixinMessages.removeSelected}},
			{actionCallback: dojo.hitch(this, this.addToClipboardAction), args: {imgSrc: require.toUrl("rmps.ui") + "/graphics/icons/add_to_clipboard_co.gif", title: this.listSectionMessages.addToClipboard}}
			/*
			 * FIXME: 26904: Show in explorer publish action did not work fine in properties form 
			 * {actionCallback: dojo.hitch(this, this.exploreInTree), args: {imgSrc : require.toUrl("rmps.app", "/graphics/icons/explore_tree_co.gif"), title: this.resourceEditorMessages.showInExplorer}}
			 **/
		];
		
		if(this.isOrdered){
			actions.push.apply(actions, [
				{actionCallback: dojo.hitch(this, this.handleMoveUp), id: "moveUp", args: {imgClass: 'moveUp', title: this.tableChangesMixinMessages.moveUp}},
				{actionCallback: dojo.hitch(this, this.handleMoveDown), id: "moveDown", args: {imgClass: 'moveDown', title: this.tableChangesMixinMessages.moveDown}}
			
			]);
		}
		
		if (this.typeForm != "CreateDataTypeProperty") {
			if (this.isFragment || !this.isDocument) {
				// for editing a fragment of the current document
				actions.push({actionCallback: dojo.hitch(this, this.newReference),
					id: "edit",
					args: {
						imgClass: "edit",
						title: this.tableChangesMixinMessages.edit
					}
				});
			} else {
				// for editing an external document referenced by current document
				actions.push({actionCallback: dojo.hitch(this, this.editExternalDocument),
					id: "edit",
					args: {
						imgClass: "edit",
						title: this.tableChangesMixinMessages.editExternal
					}
				});				
			}
		} else {
			// editing a datatype value in a list - allow inline editing
			actions.push({actionCallback: dojo.hitch(this, this.editDatatypeValueInline),
				id: "edit",
				args: {
					imgClass: "edit",
					title: this.tableChangesMixinMessages.edit
				}
			});			
		}	
		return actions;
	},
	
	editDatatypeValueInline: function(dataEntry, currentRow, currentActionsCell, currentColumnDescriptor) {
		// initiate edit mode - create the right edit control and set the current value to it;
		// called when user clicks either on edit action, or anywhere on the name cell			
		var typeUri;
		if (currentColumnDescriptor && currentColumnDescriptor.actionDescriptors) {
			typeUri = currentColumnDescriptor.typeUri;
		} else {
			typeUri = this.typeUri;
		}
		this.table.initiateInlineCellEdit(typeUri, currentRow, currentColumnDescriptor, this.inlineEditingDone, this);			
	},
	
	inlineEditingDone: function(control, cell, currentRow, currentColumnDescriptor, typeUri) {
		var values = this.table.inlineCellEditDone(typeUri, cell, control, currentRow, currentColumnDescriptor, this.allowBlankCells());
		var oldValue = values.oldValue;
		var value = values.value;
		if (oldValue != value) {
			this.inlineEditingDone_createChange(oldValue, value, currentRow, cell, currentColumnDescriptor);
		}		
	},
	
	inlineEditingDone_createChange: function(oldValue, value, currentRow, cell, currentColumnDescriptor) {		
		// create a change - remove the old value and add the new one
		// (leaveRow param indicates that we don't want the table row to be deleted, we'll update instead)
		if (oldValue) { // otherwise, creating a new row
			this.removeRefrence(oldValue, currentRow, cell, true);
		}
		this.form.applyChange(dojo.hitch(this, function(change){
			this.addReference(change, value);
		}), false); 
	},
	
	allowBlankCells: function() {
		// while editing cells inline, don't allow the user to wipe the content, as the content represents the row
		return false;
	},
	
	onSetReference: function(args){
		var target= args.target;
		var listItems = this.getCollection();
		for (var i = 0, existing; existing = listItems[i]; i++) {
			if (existing.rdf_about === target.rdf_about) {
				// reference to add already exists in this list, do nothing
				return;
			}
		}
		this._addNewReference(target, args.namespaces, args.types, args.item, this.addNewItem(target));
		this._updateMoveEnablement();
		
		this.checkReload();
	},
	
	_addNewReference: function(target, namespaces, types, item, row) {
		this.inherited(arguments);
		// after new reference is added, check if edit is enabled
		if (!this.canEditItem(target.rdf_about)) {
			var count = this.table.getRowsCount();
			if (count > 0) {
				this._updateActionDisplay(count - 1, "edit", "none");
			}
		}
	},
	
	newReference: function() {
		if (this.typeForm === "CreateDataTypeProperty") {
			// don't open a dialog, add a row in the table and let the user do inline editing
			var row = this.addNewItem(null);
			this.editDatatypeValueInline(null, row, null, null);
		} else {
			this.inherited(arguments);
		}
	},
	
	onRemoveReference: function(dataEntry, currentRow, currentActionsCell, change, leaveRow){
		if (!leaveRow) {
			this.table.removeRow(currentRow);
			this._updateMoveEnablement();
		}
		
		if (!this.dataTypeProperty) {
			var uri = dataEntry.rdf_about;
			if(!uri){
				return;
			}
			if (uri.indexOf("#") < 0) {
				uri = rmpw.common.util.IdentifierUtil.normalizeURI(this.form._resource.documentObject.rdf_about) + uri;
			}
			delete this.form._resource.modelIndex[uri];
			delete this.form._resource.modelIndex[uri + ".rdf_type"]; 
		}
		
		this.checkReload();
	},
	
	onFillTable: function(){
		if(this.isOrdered){
			this._updateMoveEnablement();			
		}
	},
		
	onMoveUp: function(dataEntry, currentRow, currentActionsCell,change){
		this._updateMoveEnablement();
	},
	
	onMoveDown: function(dataEntry, currentRow, currentActionsCell,change){
		this._updateMoveEnablement();
	},
	
	_updateMoveEnablement: function(){
		var count= this.table.getRowsCount();
		if(count<1){
			return;
		}
		
		this._updateActionDisplay(0, "moveUp", "none");
		this._updateActionDisplay(0, "moveDown", "");
		
		if (count > 1) {
			this._updateActionDisplay(count-1, "moveUp", "");	
		}
		this._updateActionDisplay(count-1, "moveDown", "none");
				
		if(count<3){
			/*Means we already updated the first and last rows enablement (including the one row case) */
			return;
		}
		
		this._updateActionDisplay(1, "moveUp", "");
		this._updateActionDisplay(1, "moveDown", "");
		
		if(count===3){
			/*We are done. We updated the middle row enablement */
			return;
		}
		
		/*Update the row before last one enablement */
		var beforeLast= count-2;
		this._updateActionDisplay(beforeLast, "moveUp", "");
		this._updateActionDisplay(beforeLast, "moveDown", "");
	},

	_updateActionDisplay: function(rowIndex, action, display){
		/*Check only for the action button being defined. Checking for table row NPE is uncecessary since we only need to check for the valid index
		 * And this is the responsbility of the places the use this method */
		var actionButton= this.table.getRowInfomation(this.table.getRowByIndex(rowIndex)).getActionWidget(action);
		if(actionButton){
			actionButton.domNode.style.display= display;
		}
	},
	
	_updateClearReferenceEnablement: function(disabled){
		var clearActionDescriptor= this.table.getActionDescriptorById("clear");
		var button= clearActionDescriptor.widgetGetter();
		if(button){	/*Will be null if the editor is being destroyed */
			button.domNode.style.visibility= disabled?"hidden":"";
		}
	},
	
	exploreInTree: function(dataEntry, currentRow, currentActionsCell){
		dojo.publish("rmps.web.app.explorer.show", [{
			uri : dojo.isObject(dataEntry)?dataEntry.rdf_about:dataEntry,
			explorerID: rmps.core.explorer.ExplorerConstants.IDs.EXPLORER_SIDEBAR_DESIGN_PAGE		
		}]);
	},
	
	addToClipboardAction: function(dataEntry, currentRow, currentActionsCell){
		var context = jazz.app.context.get();
		var resourceUri= dataEntry.rdf_about;
		ResourceInfoClient.getResourceInfo([resourceUri], ["type"], context.baseline).then(function(info) {
			clipboard().addCallback(function(cb) {
				cb.push(resourceUri, info[0].type);						
			});
		});
	},
					
	addCreationActions: function(){
		if (this.readOnly) {
			return;
		}
		if (!this.isFragment){			
			//We show select existing types when featureName is undefined (this will avoid string, int types, for instance)
			//var name= dojo.string.substitute(this.tableChangesMixinMessages.selectType, [this.typeLabel]);

			//TODO: owl_oneOf should have its own template, with the section below being part of its logic
			//the Add buttons are only enabled if:
			//  1. this is not a "One Of", or
			//  2. this is a "One Of", and the resource has been saved (as passed through arguments[0] in good faith)
			if(this.label !== "owl_oneOf" || arguments[0] === "resource saved"){
				this.actions.push(this.section.addAction(this.tableChangesMixinMessages.add, this.tableChangesMixinMessages.add, dojo.hitch(this, this.addReferenceFromSelectDialog)));
		
				if (this.enableSelectAll) {
					this.actions.push(this.section.addAction(this.listSectionMessages.selectAll, 
					dojo.string.substitute(this.listSectionMessages.selectAllTooltip, [this.typeLabel]), 
					dojo.hitch(this, this.selectAll)));
				}
			}
		}else{
			//If type form is defined, means that we already have a creation form for it, then, we can display that option
			if(this.typeForm){
				//var name= dojo.string.substitute(this.tableChangesMixinMessages.createType, [this.typeLabel]);
				this.actions.push(this.section.addAction(this.tableChangesMixinMessages.add, this.tableChangesMixinMessages.add, dojo.hitch(this, this.newReference)));
			}
		}
	},
	
	addReferenceFromSelectDialog: function(args) {
		this.useSelectDialog(args);
	},	
	
	//FIXME: Must be used in a client util instead. Not the right MVC approach to put a controller-like method in a viewer class. 
	selectAll: function() {
		if (!this.document) {
			// this is a brand new document, not saved yet
			return;
		}
		var self = this;
		rmps.core.client.xhrGet({
			url: net.jazz.ajax._contextRoot + "/modelvalidation/values",
			handleAs: 'json',
			headers: {
				"Accept" : "application/json",
				"Content-Type" : "application/json"
			},
			content: {
				resourceUri:  this.document.rdf_about, propertyUri: this.propertyUri, adjustedDomainUri: this.propertyDomainUri, adjustedRangeUri: this.typeUri
			},
			load: function(data, ioArgs) {
				if (data){
					// add all of the references (if some of these are already added, they won't be added again) 
					dojo.forEach(data, dojo.hitch(this, function(object, i) {
						self.onSetReference({target: rmpw.forms.ui.SelectOrCreateResource.prototype.getTypeObject(object)});
					}));
				}
			},			
			error: function(error) {
				console.error(error);
			}
		}, true /* send project Id */);		
	},
	
	onAutoSet: function(){
		if (!this.object || this.object.length == 0) {
			this.selectAll();
		}
		
		for (var i=0, lg=this.actions.length; i<lg; i++) {
			this.actions[i].hidden = true;
		}
	},
	
	allowMultiResourceSelect: function() {
		return true;
	}
});

})();