/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.jazz.ui.ModelDiagramLayer");

dojo.require("dojo.i18n");
dojo.require("dojo.fx.easing");
dojo.require("rmpw.diagrams.edit.EditRoot");
dojo.require("rmpw.panner.Layer");
dojo.require("rmpw.panner.Tracker");
dojo.require("rmpw.panner.MappingTrackedItem");

dojo.require("rmpw.diagrams.edit.tools.DragPan");
dojo.require("rmpw.diagrams.edit.tools.TouchZoom");
dojo.require("rmpw.diagrams.edit.tools.WheelZoom");

dojo.requireLocalization("rmpw.jazz.ui", "ModelDiagramLayerMessages");

var messages = dojo.i18n.getLocalization("rmpw.jazz.ui", "ModelDiagramLayerMessages");

dojo.declare("rmpw.jazz.ui.ModelDiagramLayer", rmpw.panner.Layer, {
	
	LAYER_TYPE: null,
	
	// mixin properties
	outline: null,
	resourceUri: null,
	graph: null,
	configuration: null,

	// properties
	viewer: null,
	
	// private properties
	_selectionTracker: null,

	postCreate: function() {
		this.inherited(arguments);  
		this._trackedItems = {};
		this.viewer = new rmpw.diagrams.edit.EditRoot({
			node: dojo.create("div", {style: "width: 100%;height:100%;"}, this.containerNode),
			configuration: this.configuration,
			repository: this.resourceInfo.repository,
			diagramLayer : this
	    });
		if(this.configuration.useDragPan) {
			this.viewer.createPolicy(rmpw.diagrams.edit.tools.DragPan);
			this.viewer.createPolicy(rmpw.diagrams.edit.tools.TouchZoom);
		}
		if(this.configuration.useWheelZoom) {
			this.viewer.createPolicy(rmpw.diagrams.edit.tools.WheelZoom);
		}
		
		this.lastX = 0;
		this.lastY = 0;
		this.lastScrollLeft = 0;
		this.lastScrollTop = 0;
		
		if (!this.outline.isUsingFixedDiagram()) {
			this.domNode.style.width = "100%";
			this.domNode.style.height = "100%";
		}
	},
	
	startup: function(){
		this.inherited(arguments);  
		this.resize();
	},
	
	getCommandController: function(){
		this.viewer.getCommandController();
	},
	
	destroy: function() {
		this.viewer.clear();
		this.inherited(arguments);
	},
	
	resize: function(size) {
		this.inherited(arguments);
		this.viewer.resize(this.getSize());
	},
	
	startup: function() {
		this.inherited(arguments);

		this.connect(this.viewer.getSelectionController(), "onSelectionChanged", "_adjustTrackers");
		this.connect(this.domNode, "scroll", "onScroll");
		this._selectionTracker = new rmpw.panner.Tracker(messages.selection);
		
		var osTracker = this.getRoot().viewer.offScreenTracker;
		if (osTracker) {
			osTracker.addTracker(this._selectionTracker, {color: "#FFC75F"});
		}
	},
	
	_adjustTrackers: function() {
		var selection = this.viewer.getSelectionController().getSelection();
		
		// remove existing trackers
		var trackers = this._selectionTracker.getItems();
		if (trackers != null) {
			for (var i=0, lg=trackers.length; i<lg; i++) {
				this._selectionTracker.removeItem(trackers[i]);
			}
		}
		
		for (var i=0, lg=selection.length; i<lg; i++) {
			this._addTracker(selection[i]);
		}
	},
	
	_addTracker: function(zone) {
		if (!("graphContainer" in zone.renderMapping.policies) && zone.renderMapping.policies.graphElement) {
			var item =  new rmpw.panner.MappingTrackedItem({
				label: messages.selection,
				mapping: zone.renderMapping
			});
			this._selectionTracker.addItem(item);
		}
	},
	
	setScale: function(scale) {
		this.inherited(arguments);

		if (this.outline.isUsingFixedDiagram()) {
			this.lastX = 0;
			this.lastY = 0;
			
			this.viewer.scaleDiagram(scale);
		}
		
		this.viewer.viewport.setZoom(scale);
	},
	
	/* TODO: This shouldn't be necessary when we adopt scaling of the 
	 * svg surface only, rather than the multiple layers that we have.
	 */
	
	simpleScale: function(scale){
		this.inherited(arguments);
		this.viewer.viewport.simpleZoom(scale);
	},
	
	getScale: function() {
		return this.viewer.viewport.getZoom();
	},
	
	invert: function(){
		this.viewer.viewport.invert();
	},
	
	setLocation: function(x, y, fromScale) {
		if(!fromScale) {
			var dx = x - this.lastX;
			var dy = y - this.lastY;
			if (this.outline.isUsingFixedDiagram()) {
				this.viewer.viewport.pan(dx, dy, true);
				if (!this.isScrolling && (dx !== 0 || dy !== 0)) {
					// adjust scrollbars to the diagram position (as user is dragging the diagram, or view in outline)
					// that will also adjust viewer.viewport
					this.dontScroll_left = true;
					this.dontScroll_top = true;
					this.lastScrollLeft = this.domNode.scrollLeft;
					this.lastScrollTop = this.domNode.scrollTop;
					this.domNode.scrollLeft += -1 * dx;
					this.domNode.scrollTop += -1 * dy;
					if (!this.outline.movingOutlineView) {
						this.outline.moveViewTo(this.domNode.scrollLeft, this.domNode.scrollTop);
					}
				}
			}
			else {
				this.viewer.viewport.pan(dx, dy);
			}
		}
		
		this.lastX = x;
		this.lastY = y;
	},
	
	getLocation: function() {
		return this.viewer.viewport.getOffset();
	},
	
	onScroll: function(event) {
		//console.log("entered onScroll");
		if (this.dontScroll_top) {
			// got to here as a consequence of the user dragging diagram surface, or view in outline
			this.dontScroll_top = false;			
			return;
		}
		if (this.dontScroll_left) {
			// got to here as a consequence of the user dragging diagram surface, or view in outline
			this.dontScroll_left = false;			
			return;
		}
		
		if (!this.isScrolling && !this.dontScroll) {
			// using diagram scrollbars, adjust view in outline
			this.isScrolling = true;
			var dx = this.lastScrollLeft - this.domNode.scrollLeft;
			var dy = this.lastScrollTop - this.domNode.scrollTop;
			var newX = this.lastX + dx;
			var newY = this.lastY + dy;
			this.outline.pannableLayer._x = newX;
			this.outline.pannableLayer._y = newY;
			this.setLocation(newX, newY, false);
			this.outline.moveViewTo(this.domNode.scrollLeft, this.domNode.scrollTop);
			this.lastScrollLeft = this.domNode.scrollLeft;
			this.lastScrollTop = this.domNode.scrollTop;
			this.isScrolling = false;
		}
	},
	
	restore: function(){
		this.viewer.viewport.invert();
	}
});
})();