/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.jazz.ui.ModelDiagramOutline");

dojo.require("rmpw.panner.Outline");

dojo.require("dojox.gfx");
dojo.require("dojox.gfx.matrix");

dojo.declare("rmpw.jazz.ui.ModelDiagramOutline", rmpw.panner.Outline, {
	
	// private properties
	_surface: null,
	_scalingGroup: null,
	_translateGroup: null,
	_contentGroup: null,
	
	/** _usingFixedDiagram is used to distinguish between using old vs. new diagram navigation behaviors.
		the new behavior was created for WI 42459: "Navigation of diagrams in Web UI in DesignManager is not user friendly."
		it is currently only for diagrams that have a fixed size **/
	_usingFixedDiagram: false,
	
	postCreate: function() {
		this.isRealTime = false;
		this.autoPanInterval=30;		
		this.autoPanStep=1;		
		this._maxedOut = false;
		
		if (this.isUsingFixedDiagram()) {
			this.panContainerNode.style.width = "100%";
			this.panContainerNode.style.height = "100%";
		}
		
		this.inherited(arguments);
		this._surface = dojox.gfx.createSurface(this.panContainerNode, 1, 1);
		
		this._translateGroup = this._surface.createGroup();
		this._scalingGroup = this._translateGroup.createGroup();
		this._contentGroup = this._scalingGroup.createGroup();
	},
	
	createResizeHandle: function() {
		// no resize handle
		this._resizeHandle = null;
	},
	
	initialize: function(size) {
		// scale diagram to fit the outline, the best it can
		
		this.updateRatio();
		this.ratioChanged(this._ratio);
		
		if (this.getScale() >= 1) {
			var wd_scaled = Math.floor(this._ratio * size.w);
			var hd_scaled = Math.floor(this._ratio * size.h);
			this._surface.setDimensions(wd_scaled + 1, hd_scaled + 1);
		}
		
		this.resize(size);
	},
	
	resize: function(s) {
		if (this.isUsingFixedDiagram()) {
			if (!this._overview || !s) {
				return;
			}
			
			// scale viewport (_cView corresponds to scaled viewport in outline, _view is its border used for moving)				
			var viewSize = this.getViewSize();
			this._cView.firstChild.style.width = this._view.firstChild.style.width = viewSize.w + "px";
			this._cView.firstChild.style.height = this._view.firstChild.style.height = viewSize.h + "px";
			
			this.showOrHideOutlineView();
		}
		else {
			this.inherited(arguments);
			if (this._overview) {
				this._surface.setDimensions(this._overview.offsetWidth, this._overview.offsetHeight);
			}
		}
	},
	
	/*
	 * Callback that allows the mapping object to update the content group 
	 * of the outline surface.
	 */
	updateOutlineShape: function(mapping) {
		if (mapping.policies) {
			if (mapping.policies.graphNode) {
				mapping.policies.graphNode.updateOutlineShape(this._contentGroup);
			}
			else if (mapping.policies.graphEdge) {
				mapping.policies.graphEdge.updateOutlineShape(this._contentGroup);
			}
		}
	},
	
	ratioChanged: function(ratio) {
   		if (this._contentGroup) {
			this._contentGroup.setTransform(dojox.gfx.matrix.scale(ratio, ratio));
   		}
	},
	
	setScale: function(scale, cx, cy, origin) {
		var oldScale = this.getScale();
		if (oldScale != scale) {
			this.inherited(arguments);
			
			var m = dojox.gfx.matrix;
			if (origin && this.isUsingFixedDiagram()) {
				if ((scale < 1) || (oldScale < 1)) {
					// scale is now, or previously was, lower than 1, so the size of the diagram in the ouline has changed 
					// shrink/expand the diagram in the outline as per new scale
					var newScale = scale;
					if (newScale > 1) {
						// the diagram in the outline doesn't change when the scale goes past 1
						newScale = 1;
					}
					this._scalingGroup.setTransform(m.scaleAt(newScale, newScale, 0, 0));
				}
				if ((scale > 1) || (oldScale > 1)) {
					// scale is now, or previously was, higher than 1, so change the ratio to match the new diagram size
					// the diagram in the outline doesn't change when the scale goes past 1, so the ratio changes 
					this.updateRatio();
				}
				
				var viewSize = this.getViewSize();
				
				// get the old width and height of outline's view
				var oldWidth = this._view.firstChild.offsetWidth;
				var oldHeight = this._view.firstChild.offsetHeight;
				
				// set the new width and height
				this._cView.firstChild.style.width = this._view.firstChild.style.width = viewSize.w + "px";
				this._cView.firstChild.style.height = this._view.firstChild.style.height = viewSize.h + "px";
				
				// X and Y differences between the centers of the old and new outline views (changing the size changed the center)
				// we want to move back to that same center, to simulate "zooming in"
				var adjustViewCenterDifX = ((oldWidth / 2) - (viewSize.w / 2)) / this._ratio;
				var adjustViewCenterDifY = ((oldHeight / 2) - (viewSize.h / 2)) / this._ratio;
				
				// the scrollbars (if they exist) need to be adjusted via the ratio of old and new scale amounts
				var scrollLoc_X = this.pannableLayer.viewer.diagramLayer.domNode.scrollLeft;
				var scrollLoc_Y = this.pannableLayer.viewer.diagramLayer.domNode.scrollTop;
				var scrollScaleDifX = (scrollLoc_X * scale / oldScale) - scrollLoc_X;
				var scrollScaleDifY = (scrollLoc_Y * scale / oldScale) - scrollLoc_Y;
				
				// the adjustments to keep the same "view center", combined with the scroll adjustments
				var moveLeft = -1 * Math.round(adjustViewCenterDifX + scrollScaleDifX);
				var moveUp = -1 * Math.round(adjustViewCenterDifY + scrollScaleDifY);
				
				this._setPannableLayerLocation(moveLeft, moveUp);
			}
			else {
				if (origin) {
					var loc = this.getLocation();
					var viewBox = dojo.position(this.panContainerNode);
					var ratio = scale / this.pannableLayer.getScale();
					cx = viewBox.w / 2 / ratio - loc.x;
					cy = viewBox.h / 2 / ratio - loc.y;
				}
				else {
					cx = cx || 0;
					cy = cy || 0;
				}
				this._scalingGroup.setTransform(m.scaleAt(scale, scale, cx, cy));
			}
		}
		
		if (this.isUsingFixedDiagram()) {
			this.showOrHideOutlineView();
		}
	},
	
	updateRatio: function() {
		// size of _overview
		var wo = this._overview.offsetWidth;
		var ho = this._overview.offsetHeight;
		
		// current size of diagram
		var diagramDim = this.pannableLayer.viewer.diagramLayer.viewer.surface.getDimensions();
		var wd = diagramDim.width; //this.pannableLayer.viewer.diagramLayer.viewer.surface.rawNode.width;
		var hd = diagramDim.height; //this.pannableLayer.viewer.diagramLayer.viewer.surface.rawNode.height;

		if (!wd || !hd) {
			return;
		}
		
		this._ratio = Math.min(wo/wd, ho/hd);
	},
	
	getViewPortSize: function() {
		// sv is the size of the viewport (viewable area for diagram, without scrollbars if any)
		var sv = dojo.marginBox(this.pannableLayer.viewer.diagramLayer.viewer.container);
		
		var diagramDomNode = this.pannableLayer.viewer.diagramLayer.domNode;
		var scrollLeftMax = diagramDomNode.scrollWidth - diagramDomNode.clientWidth;
		var scrollTopMax = diagramDomNode.scrollHeight - diagramDomNode.clientHeight;
		
		if (scrollLeftMax != 0) {
			// account for horizontal scrollbar
			sv.w = sv.w - 15;
		}
		if (scrollTopMax != 0) {
			// account for vertical scrollbar
			sv.h = sv.h - 15;
		}
		
		return sv;
	},
	
	getViewSize: function() {
		// sv is the size of the viewport (viewable area for diagram, without scrollbars if any)
		var sv = this.getViewPortSize();
		// scale to make _cView fit the diagram best it can
		var viewSize = {
			w: Math.floor(sv.w * this._ratio),
			h: Math.floor(sv.h * this._ratio)
		};
		
		return viewSize;
	},
	
	showOrHideOutlineView: function() {
		var diagramDomNode = this.pannableLayer.viewer.diagramLayer.domNode;
		var scrollLeftMax = diagramDomNode.scrollWidth - diagramDomNode.clientWidth;
		var scrollTopMax = diagramDomNode.scrollHeight - diagramDomNode.clientHeight;
		
		var canScrollX = (scrollLeftMax != 0);
		var canScrollY = (scrollTopMax != 0);
		if (canScrollX || canScrollY) {
			this._view.style.visibility = "";
			this._cView.style.visibility = "";	
		} else {
			// no scrolbars, so no view in the outline
			this._view.style.visibility = "hidden";
			this._cView.style.visibility = "hidden";			
		}
	},
	
	pan: function(dx, dy) {
		if (this.isUsingFixedDiagram() && (dx === 0) && (dy === 0)) {
			return;
		}
		
		// pan diagram/sketch/etc
		var pt = this.pannableLayer.getLocation();
		this._setPannableLayerLocation(pt.x+Math.round(dx), pt.y+Math.round(dy));
		
		if (!this.isUsingFixedDiagram()) {
			// pan diagram in outline shape
			this.resize();
		}
	},
	
	/**
	 * Moves view in outline shape
	 * @param {int} x (not scaled down to outline)
	 * @param {int} y (not scaled down to outline)
	 */
	moveViewTo: function(x, y) {
		this._cView.style.left = this._view.style.left = x * this._ratio + "px";
		this._cView.style.top = this._view.style.top = y * this._ratio + "px";
	},
	
	movePanContainerNode: function(x, y) {
		// if using a fixed dialog, do nothing, panContainerNode never moves except initial to get position to (0, 0)
		if (!this.isUsingFixedDiagram()) {
			this._translateGroup.setTransform(dojox.gfx.matrix.translate(x, y));
		}
	},
	
	movePanContainerNodeToTopLeftCorner: function(x, y) {
		this._translateGroup.setTransform(dojox.gfx.matrix.translate(x * this._ratio, y * this._ratio));
	},
	
	setDiagram: function(diagram) {
		this._diagram = diagram;
	},
	
	_setPannableLayerLocation: function(x,y) {
		if (!this.isUsingFixedDiagram() && this._diagram) {
			// prevent diagram from panning to infinity and beyond
			var box = this._diagram.policies.graphContainer.getChildrenBox();
			if (box) {
				var bnds = this._diagram.policies.graphElement.getBounds();
				var scale = this.pannableLayer.getScale();
				var min = (box.x - bnds.width - 100) * scale;
				var max = (box.x + box.width + 100) * scale;
				if (-x < min) {
					x = -min;
					this._maxedOut = true;
				}
				else if (-x > max) {
					x = -max;
					this._maxedOut = true;
				}
				min = (box.y - bnds.height - 100) * scale;
				max = (box.y + box.height + 100) * scale;
				if (-y < min) {
					y = -min;
					this._maxedOut = true;
				}
				else if (-y > max) {
					y = -max;
					this._maxedOut = true;
				}
			}
		}
		
		// pan the layers
		this._movingPLayer = true;
		this.pannableLayer.setLocation(x,y,false,true);
		this._movingPLayer = false;
	},
	
	// determine zoom required to fit diagram in view
	getScaleToFit: function() {
		var zoom = 1;
		if (this.isUsingFixedDiagram()) {
			var viewPortSize = dojo.marginBox(this.pannableLayer.viewer.diagramLayer.viewer.container);
			viewPortSize.w = viewPortSize.w - 5; // add some padding
			viewPortSize.h = viewPortSize.h - 5; // add some padding
			var diagramWidth = this.pannableLayer.viewer.diagramLayer.viewer.fixedDiagramSize.width;
			var diagramHeight = this.pannableLayer.viewer.diagramLayer.viewer.fixedDiagramSize.height;			
			zoom = Math.min((viewPortSize.w / diagramWidth), (viewPortSize.h / diagramHeight));
		}
		else if (this._diagram) {
			var box = this._diagram.policies.graphContainer.getChildrenBox();
			var bnds = dojo.position(this._diagram.root.viewport.container);
			if (box && bnds) {
				var z =  Math.min((bnds.w / (box.width +100) ), (bnds.h  / (box.height + 100)));
				if (z<=.95) {
					zoom = z;
				}
			}
		}
		return zoom;
	},
	
	centerView: function() {
		if (!this.isUsingFixedDiagram() && this._diagram) {
			var viewport = this._diagram.root.viewport;
			var t = viewport._getContentsTransform();
			var box = this._diagram.policies.graphContainer.getChildrenBox();
			if (box) {
				// get diagram background rect
				var d = this._diagram.root.getCanvasSize();
				var bnds = new rmpw.diagrams.render.util.Rectangle(0, 0, d.w, d.h);
				this._diagram.policies.graphContainer.translateToRelative(bnds);
				
				// pan diagram to center of available space
				var x = (((bnds.width - box.width)/2) -box.x) * t.xx;
				var y = (((bnds.height - box.height)/2) - box.y) * t.yy;
				var dx = (x-t.dx);
				var dy = (y-t.dy);
				this.pan(dx,dy);
			}
		}
	},
	
	onEndCenterView: function() {
		if (!this.isUsingFixedDiagram() && this._maxedOut) {
			this._maxedOut = false;
			this.resize();
		}
	},
	
	_centerView: function() {
		// do nothing, no moving view to center (leave it where user dragged it, at (0,0) by default)
	},

	restore: function(){
		this._pLayerMoved(0,0);
	},
	
	_onViewMoved: function(mover, leftTop) {
		if (!this.isUsingFixedDiagram()) return;
		
		// while moving the view in outline, don't allow that it goes beyond the content
		
		var x = this.getLocation().x;
		var y = this.getLocation().y;
		if (leftTop.l < 0) {
			this._setLocation(0, y);
			this._view.style.left = "0px";
		}
		if (leftTop.t < 0) {
			this._setLocation(x, 0);
			this._view.style.top = "0px";
		}
		
		var diagramDomNode = this.pannableLayer.viewer.diagramLayer.domNode;
		var scrollLeftMax = diagramDomNode.scrollWidth - diagramDomNode.clientWidth;
		var scrollTopMax = diagramDomNode.scrollHeight - diagramDomNode.clientHeight;
		
		scrollLeftMax = Math.round(scrollLeftMax * this._ratio);
		scrollTopMax = Math.round(scrollTopMax * this._ratio);
		if (leftTop.l > scrollLeftMax) {
			this._setLocation(scrollLeftMax, y);
			this._view.style.left = this._x + "px";
		}
		if (leftTop.t > scrollTopMax) {
			this._setLocation(x, scrollTopMax);
			this._view.style.top = this._y + "px";
		}
	},
	
	isUsingFixedDiagram: function() {
		return this._usingFixedDiagram;
	}
});

})();