/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

	dojo.provide("rmps.web.comment.sketch.ui.ModelingPage");
	
	/* dijit */
	
	dojo.require("dijit.TooltipDialog");
	dojo.require("dijit.layout.ContentPane");
	dojo.require("dijit.Dialog");
	dojo.require("dijit.form.DropDownButton");
	dojo.require("dijit.form.ComboBox");
	dojo.require("dijit.form.Button");
	dojo.require("rmpw.widgets.Events");
	
	/* dojox */
	
	dojo.require("dojox.gfx");
	dojo.require("dojox.gfx.utils");
	dojo.require("dojox.gfx._base");
	dojo.require("dojox.json.ref");
	dojo.require("dojox.html.entities");
	
	/* rmpw.jazz */

	dojo.require("rmpw.jazz.ModelingPage");
	dojo.require("rmpw.jazz.ui.Controls");
	dojo.require("rmpw.jazz.ui.LineStyler");

	
	/* rmps.web.comment.sketch */
	
	dojo.require("rmps.web.comment.sketch.ui.Configuration");
	dojo.require("rmps.web.comment.sketch.behavior.SketchController");
	dojo.require("rmps.web.comment.sketch.commands.SketchCommandPackage");
	dojo.require("rmpw.diagrams.edit.actionBindings.KeyStroke");
	dojo.require("rmps.web.comment.sketch.commands.EditStyle");
	dojo.require("rmps.web.comment.sketch.render.SVG");
	dojo.require("rmps.web.comment.sketch.render.Group");
	dojo.require("rmps.web.comment.sketch.render.Rectangle");
	dojo.require("rmps.web.comment.sketch.behavior.Scribbler");
	dojo.require("rmps.web.comment.sketch.behavior.LineMaker");
	dojo.require("rmps.web.comment.sketch.behavior.OvalMaker");
	dojo.require("rmps.web.comment.sketch.behavior.RectangleMaker");
	dojo.require("rmps.web.comment.sketch.behavior.CloudMaker");
	
	/* Test purposes only */
	
	dojo.require("rmps.web.comment.sketch.serializer.SVGTestData");
	dojo.require("rmps.web.comment.sketch.serializer.Reader");

	var PlatformUI = net.jazz.ajax.ui.PlatformUI;

	var controls = rmpw.jazz.ui.Controls.items;

	dojo.declare("rmps.web.comment.sketch.ui.ModelingPage", [rmpw.jazz.ModelingPage], {

		configuration: new rmps.web.comment.sketch.ui.Configuration(),
		
		selectionController: null,		
		commandController: null,		
		topics: rmpw.widgets.Events.topics,	
		prefix: "rmps.web.comment.sketch",	
		initialized: false,	
		selectedTool: null,	
		linetools: false,	
		subscriptions: null,
		
		settings: [ { name: controls.GRID, display: true, enabled: false, id: "rmps.web.comment.sketch.GridToggle" },
                    { name: controls.OUTLINE, display: true, enabled: false, id: "rmps.web.comment.sketch.OutlineToggle" } 
    	],

		tools: [ { name: controls.SAVE, display: true, enabled: true },   	         
		         { name: controls.SEARCHBOX, display: false, enabled: true },
		         { name: controls.FILEPICKER, display: false, enabled: true }, 
		         { name: controls.RELOAD_DIAGRAM_CONTENT, display: false, enabled: true },
		         { name: controls.UNDO, display: true, enabled: true },
		         { name: controls.REDO, display: true, enabled: true },
		         { name: controls.DELETE, display: false, enabled: true },
		         { name: controls.LAYOUT, display: false, enabled: true },
		         { name: controls.SETTINGS, display: true, subcontrols: this.settings }
		],

		load: function(){
			var diagramStore = new this.configuration.store( this.viewer.diagramLayer.viewer );   
			diagramStore.fetch({ src: "web/rmps.web.comment.sketch/samples/relationships.json" });
		},  

		save: function(){
			
			/* First need to deselect all shapes and clear SVG surface
			 * of controls! */
			
			var svg = dojox.gfx.utils.toSvg( this.viewer.diagramLayer.viewer.surface );
						
			function callback(svg){
				
				var encodedsvg = dojox.html.entities.encode(svg);
				
				var dialog = new dijit.Dialog({
		            title: "SVG Serialization",
		            style: "width: 300px;",
		            content: encodedsvg
		        });

		        dialog.show();
				
				console.log( svg );
			}
			
			svg.addCallback( callback );
		},
		
		shapeChoice: function( val ){
			this.selectedTool = val;
			
			switch( this.selectedTool ){
			
				case "RMPW_CLOUD":
					if(!this.cloudtool){
						this.cloudtool = new rmps.web.comment.sketch.behavior.CloudMaker( this.commandController );
					}
					this.cloudtool.capture( this.buildArgs( "Cloud" ) );	
					break;
				
				case "RMPW_RECTANGLE":
					if(!this.recttool){
						this.recttool = new rmps.web.comment.sketch.behavior.RectangleMaker( this.commandController );
					}
					this.recttool.capture( this.buildArgs( "Rectangle" ) );	
					break;
					
				case "RMPW_OVAL":
					if(!this.ovaltool){
						this.ovaltool = new rmps.web.comment.sketch.behavior.OvalMaker( this.commandController );
					}
					this.ovaltool.capture( this.buildArgs( "Oval" ) );	
					break;
				
			}
		},
		
		addDrawingTools: function(){
			
		},

		setDrawingToolsOn: function(val) {
			if (val) {
				this.addDrawingTools();
			}
			else {
				for (var i=0; tool = this._drawingTools[i]; i++) {
					this.layout.removeTool(tool);
				}
				this._drawingTools = [];
			}
		},
		
		extend: function(){  
			
			//this.addDrawingTools();
			
			/* For testing with ...
	
			var testButton = new dijit.form.Button({ 
				label: "Test", 
				showLabel: false, 
				onClick: dojo.hitch( this, "addTest" ),
				iconClass: "testIcon" 
			});

			this.addTool(testButton); */
			
			var page = this;
		     var menu = new dijit.Menu( { style: "display: none;" } );
		     var item = new dijit.CheckedMenuItem({
	    		 id: "rmps.web.comment.sketch.DrawingToolsToggle",
	             label: "Drawing Tools",
	             value: "agreed",
	             checked: false,
	             onChange: function (val) {
		    	 	page.setDrawingToolsOn(val);
		     	}
	    	 }); 
	    	 
		     this.layout.setButton.dropDown.addChild( item );
		},
		
		addTest: function(){
			
			var data = new rmps.web.comment.sketch.serializer.SVGTestData();		
			var reader = new rmps.web.comment.sketch.serializer.Reader();
			var adjustment = { x:0, y:0 };			
			var scribbles = reader.read( data.rhapsody6, adjustment );			
			var group = this.commandController.viewer.surface.createGroup();
			
			for( var s = 0; s<scribbles.length;s++){
				var d = dojox.gfx.utils.deserialize( group, scribbles[s] );
			}
		},
		
		buildArgs: function( type ){
			var commandArgs = {
				svgParent: this.controller.renderMapping.sources.svgRoot,
				shape: type,
				viewport: this.commandController.viewer.viewport
			};
			
			return commandArgs;
		},

		addText: function(){
			//alert( "Coming Soon ..." );
		},

		addScribble: function(){			
			if(!this.scribbler){
				this.scribbler = new rmps.web.comment.sketch.behavior.Scribbler( this.commandController );
			}
			this.scribbler.capture( this.buildArgs( "Scribble" ) );			
		},

		addLine: function(){
			if(!this.linemaker){
				this.linemaker = new rmps.web.comment.sketch.behavior.LineMaker( this.commandController );
			}
			this.linemaker.capture( this.buildArgs( "Line" ) );		
		},

		startup: function(){
			
			this.inherited( arguments );

			if( this.initialized == false ){
				
				var viewer = this.viewer = this.layout.viewer.diagramLayer.viewer;
				viewer.clear();
				
				this.rootLayer = this.layout.viewer.rootLayer;
				
				this.diagram = new rmps.web.comment.sketch.render.SVG();				
				domStyle.set(this.diagram, "height", "100%");
				var blank = new rmps.web.comment.sketch.render.Group();
				
				this.diagram.setG( blank );
				
				var rect = new rmps.web.comment.sketch.render.Rectangle();
								
				this.diagram.setGraphicsParent( viewer.viewport.contents );
				viewer.setDiagram( this.diagram );
				viewer.commandPackage = new rmps.web.comment.sketch.commands.SketchCommandPackage();
				
				// set up behavior controllers
				this.controller = new rmps.web.comment.sketch.behavior.SketchController("EDIT");
				this.controller._connectConstruction = dojo.connect( this.diagram, "applyModelChanges", this, function() {
					this.controller.setRenderMapping( this.diagram );
					this.controller.setParent(viewer);
					dojo.disconnect( this.controller._connectConstruction );
					delete this.controller._connectConstruction;
				});
								
				this.selectionController = viewer.viewer.getSelectionController();
				
				this.commandController = viewer.viewer.getCommandController();
				
				this.commandController.modelingPage = this;
				
				dojo.connect( this.selectionController, "onSelectionChanged", this, "selectionChanged" );

				this.lineStyler = new rmpw.jazz.ui.LineStyler(); 
				
				this.lineStyler.setLineColor = dojo.hitch( this, "setLineColor" ); 
				this.lineStyler.setLineWidth = dojo.hitch( this, "setLineWidth" ); 
				this.lineStyler.setLineStyle = dojo.hitch( this, "setLineStyle" ); 
				
				this.subscriptions = [];
				
				this.subscriptions.push( dojo.subscribe( this.topics.FILL_COLOR, dojo.hitch( this, "setFillColor" ) ) );
				
				this.initialized = true;		
				
				/* TODO: (AMC) Improve on this! */
				
				this.layout.viewer.setOutline( false ); 
				dojo.publish('gef/diagram/grid', [true]);
			}
		},
		
		postCreate: function(){
			var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			ActionRegistry.registerAction("rmps.web.comment.sketch.startup", this, "startup"); 

			this.resizer = jazz.util.ResizeNotifier.register({
				context: this,
				callback: this.resize,
				force: true    	        
			}); 
		},

		selectionChanged: function(){
			
			this.selection = this.selectionController.getSelection();
			
			if( this.selection.length > 0 ){
				if( this.linetools == false ){
					this.lineStyler.addStyleTools( this.layout.toolbar );					
					this.linetools = true;
				}
	    	 }else{
	    		this.lineStyler.removeStyleTools();
	    		this.linetools = false;
	    		
	    	}
		},
		
		executeStyle:function( args ){
    		var cmd = new rmps.web.comment.sketch.commands.EditStyle();
    		this.commandController.commandStack.execute( cmd, args ); 
    	},
		
		updateStroke: function( stroke ){
    		this.executeStyle( {
				stroke: stroke,
				fill: null,
				text: null,
				nodes: this.selection
			}); 
		},
		
		setLineColor: function( message ){		
			this.updateStroke( { color: message.color } ); 	
		},
		
		setLineWidth: function( message ){	
			this.updateStroke( { width: message.width } ); 
		},
		
		setLineStyle: function( message ){
			this.updateStroke( { dash: message.style } ); 
		},
		
		setFillColor: function( message ){				
			this.executeStyle({
				stroke: null,
				fill: message,
				text: null,
				nodes: this.selection
			}); 		
		}
		
	});

})();