/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.dsv.editor.owl.ui.OWLEditor");

dojo.require("rmps.core.client");
dojo.require("dijit.layout.TabContainer");
dojo.require("dijit.layout.ContentPane");
dojo.require("dojox.highlight");

dojo.require("rmpw.dsv.editor.owl.ui.xml");
dojo.require("rmpw.dsv.editor.owl.ui.ntriple");

dojo.declare("rmpw.dsv.editor.owl.ui.OWLEditor",  dijit.layout.TabContainer, {
	_rdfXml: null,
	_nTriple: null,
    _turtle: null,
	postCreate: function() {
	    this.inherited(arguments);
	    this._rdfXml = new dijit.layout.ContentPane({title: "RDF/XML"});
	    this.addChild(this._rdfXml);
	    this._nTriple = new dijit.layout.ContentPane({title: "N-Triples"});
	    this.addChild(this._nTriple);
	    this._turtle = new dijit.layout.ContentPane({title: "Turtle"});
	    this.addChild(this._turtle);
	    dojo.connect(this, 'selectChild', this.showOntologyTab);
	    dojo.connect("onresize", dojo.hitch(this, this.resize));
	},
	
	// Override the resize() method and ensure the domNode is
	// defined before doing a resize() or else the error
	// "TypeError: node is null" appears. (WI 37909)
	resize: function() {
		if (this.domNode) {
			this.inherited(arguments);
		}
	},
	
	showOntologyTab: function() {
		var uri ;
		var content_type;
		if(this.selectedChildWidget == this._rdfXml) {
			 uri = net.jazz.ajax._contextRoot + "/rdfdirectory/xml?logicalURI=" + encodeURIComponent(this.uri);
			 content_type = "application/rdf+xml";
		} else if(this.selectedChildWidget == this._turtle) {
			 uri = net.jazz.ajax._contextRoot + "/rdfdirectory/turtle?logicalURI=" + encodeURIComponent(this.uri);
			 content_type = "application/turtle";
		} 
		else {
			 uri = net.jazz.ajax._contextRoot + "/rdfdirectory/ntriples?logicalURI=" + encodeURIComponent(this.uri);
			 content_type = "application/ntriples";
		}
		var content_type = this.selectedChildWidget == this._rdfXml ? "application/rdf+xml" :
			this.selectedChildWidget == this._turtle ? "application/turtle":"application/ntriples" ;
		rmps.core.client.xhrGet({
			url : uri,
			headers: {
			"Accept": content_type,
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "text",
			load: dojo.hitch(this, function(data) {
				this.selectedChildWidget == this._rdfXml ?  this._showXML(data): this.selectedChildWidget == this._turtle ? this._showTurtle(data): this._showNTriple(data);
			    window.onresize({}); // Resize the content so it fits
			})
		});	
	},
	_showTurtle: function(source) {
		if(source == null) {
			return;
		}
		this._highlight(this._turtle.domNode, source, 'ntriple');
		
		// clear highlighting listeners from previous refresh
		if(this._nodeListeners) {
			dojo.forEach(this._nodeListeners, dojo.disconnect);
		}
		
		// highlight all instances of a blank node when one is hovered
		var nodes = dojo.query('.statement.blank, .statement.url');
		this._nodes = {};
		this._selectedNodes = {};
		this._nodeListeners = [];
		for(var i=0, node; node = nodes[i]; i++) {
			var nodeId = node.innerHTML;
			var allInstances = this._nodes[nodeId];
			if(!allInstances) {
				allInstances = this._nodes[nodeId] = [];
			}
			allInstances.push(node);
			this._nodeListeners.push(dojo.connect(node, 'onmouseover', this, '_nodeMouseover'));
			this._nodeListeners.push(dojo.connect(node, 'onmouseout', this, '_nodeMouseout'));
			this._nodeListeners.push(dojo.connect(node, 'onclick', this, '_nodeClick'));
		}
	},
	
	_showNTriple: function(source) {
		if(source == null) {
			return;
		}
		this._highlight(this._nTriple.domNode, source, 'ntriple');
		
		// clear highlighting listeners from previous refresh
		if(this._nodeListeners) {
			dojo.forEach(this._nodeListeners, dojo.disconnect);
		}
		
		// highlight all instances of a blank node when one is hovered
		var nodes = dojo.query('.statement.blank, .statement.url');
		this._nodes = {};
		this._selectedNodes = {};
		this._nodeListeners = [];
		for(var i=0, node; node = nodes[i]; i++) {
			var nodeId = node.innerHTML;
			var allInstances = this._nodes[nodeId];
			if(!allInstances) {
				allInstances = this._nodes[nodeId] = [];
			}
			allInstances.push(node);
			this._nodeListeners.push(dojo.connect(node, 'onmouseover', this, '_nodeMouseover'));
			this._nodeListeners.push(dojo.connect(node, 'onmouseout', this, '_nodeMouseout'));
			this._nodeListeners.push(dojo.connect(node, 'onclick', this, '_nodeClick'));
		}
	},
	
	_nodeMouseover: function(e) {
		var nodeId = e.currentTarget.innerHTML;
		var allInstances = this._nodes[nodeId];
		for(var i=0, node; node = allInstances[i]; i++) {
			dojo.addClass(node, 'hover');
		}
	},
	
	_nodeMouseout: function(e) {
		var nodeId = e.currentTarget.innerHTML;
		var allInstances = this._nodes[nodeId];
		for(var i=0, node; node = allInstances[i]; i++) {
			dojo.removeClass(node, 'hover');
		}
	},

	/**
	 * On click, mark nodes with a unique color, so references are sticky while scrolling.
	 */
	_nodeClick: function(e) {
		var nodeId = e.currentTarget.innerHTML;
		var selected = this._selectedNodes[nodeId];
		var allInstances = this._nodes[nodeId];
		
		if(selected) {
			for(var i=0, node; node = allInstances[i]; i++) {
				dojo.style(node, 'backgroundColor', null);
			}
			delete this._selectedNodes[nodeId];
		} else {
			var minBrightness = 150;
			var range = 255 - minBrightness;
			var r = (minBrightness + Math.round(Math.random()*range)).toString(16);
			var g = (minBrightness + Math.round(Math.random()*range)).toString(16);
			var b = (minBrightness + Math.round(Math.random()*range)).toString(16);
			
			var uniqueColor = '#' + r + g + b; 
			for(var i=0, node; node = allInstances[i]; i++) {
				dojo.style(node, 'backgroundColor', uniqueColor);
			}
			this._selectedNodes[nodeId] = true;
		}
	},
	
	_showXML:function(source) {
		if(source == null) {
			return;
		}
		
		// space out rdf:Description blocks
		source = source.replace(/<rdf:Description/g, "\n  <rdf:Description");
		
		this._highlight(this._rdfXml.domNode, source, 'xml');
	},
	
	_highlight:function(container, source, language) {
		if(source == null) {
			return;
		}
		// escape angle brackets
		source = source.replace(/</g, "&lt;");
		source = source.replace(/>/g, "&gt;");

		dojo.empty(container);
		var preBlock = dojo.create('pre', null, container);
		var codeBlock = dojo.create('code', { innerHTML: source, 'class': language + ' rdfxmlcode' }, preBlock);
		// add a slight pause before highlighting--some browsers loses line feeds
		if (!this._timer) {
			this._timer = setTimeout(dojo.hitch(this, function() {
				delete this._timer;
				dojox.highlight.init(codeBlock);
			}), 50);
		}
		dojo.addClass(preBlock, 'rdfxmlcode');
	}
});

})();