/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.dsv.editor.owl.ui.OwleditorPage");

dojo.require("dijit.TitlePane");
dojo.require("dijit.layout.BorderContainer");
dojo.require("rmpw.dsv.editor.owl.ui.OWLEditor");

dojo.declare("rmpw.dsv.editor.owl.ui.OwleditorPage", dijit.layout.BorderContainer, {
	postCreate: function(){
		this.inherited(arguments);
		
		this._createTab();
	},
	
	_createTab: function() {
		if (!this._editorTab) { 
			this._editorTab = new rmpw.dsv.editor.owl.ui.OWLEditor({
				style: "height: 100%; width: 100%;",
				splitter:"true", region:"center",
				uri: this.editor.editorInput.resourceInfo.uri
			});
			this.addChild(this._editorTab);
			this._editorTab.selectChild(this._editorTab._rdfXml);
		    dojo.connect("onresize", dojo.hitch(this, this.resizeHeight));
			
			this.domNode.style.width="100%";
			this.domNode.style.height = "100%";
		}
	},
	
	/**
	 * <p>
	 * Added to properly set the height for the editor.
	 * </p>
	 */
	resizeHeight: function(){
		if (!this.domNode) {
			return;
		}
		
		// Self/footer/window positions
		var myPos = dojo.position(this.domNode, true);
		
		// Not done layouting...try again
		if(myPos.y == 0){
			setTimeout(dojo.hitch(this, this.resize), 10);
			return;
		}
		
		// size of source tab is the total size of window - the footer height - the y position of the source tab - 7 (misc pixels)
		var footerPos = dojo.position(dojo.byId("jazz_app_internal_Footer_0"), true);
		var windowPos = dojo.position(document.getElementsByTagName("body")[0], false);
		var newHeight = windowPos.h - myPos.y - footerPos.h - 5;
		this.domNode.style.height = newHeight +  "px";	
	},
	
	doSave: function(/*rmps.core.TaskMonitor*/monitor) {
		/*
		 * Hook for being saved 
		 */
	},
	
	 startup : function() {
		 this.inherited(arguments);
		 
		 if (!this.refreshOnActivate()) {
			 doRefresh();
		 }
   	 },
	 
	 refreshOnActivate: function() {
		return true; 
	 },
	 
	 beforeDeactivate: function() {
		 this.removeChild(this._editorTab);
		 this._editorTab.destroy();
		 this._editorTab = null;
	 },
	
	doRefresh: function() {
		this._createTab();
		if (this._editorTab!=null) {
			this._editorTab.showOntologyTab()
		}
	},
	
	initializeToolBar: function(
			/* jazz.ui.SimpleToolbar */ toolbar,
			/*function*/setDirty,
			/*function*/enableRefresh) {
		/*
		 * Hook to initialize toolbar
		 * @param tooblar
		 * 					is passed so extra buttons can be added if required
		 * @param setDirty(bool?) 
		 * 					is a function that should be called when the state of the
		 * 					Diagram is dirtied or un-dirtied.
		 * @param enableRefresh(boolean)
		 * 					is a function to call to allow the refresh button to be
		 * 					enabled or disabled
		 */
	},
	
	/*
	 * Called prior to saving. If a validation error is present
	 * which will prevent save from completing then clients should
	 * populate push the error reason (String|Error) into the errors array.
	 * Client may also push a Deferred object to defer the validation. This allows
	 * the client to communicate with the server to perform validation. To add a validation
	 * failure the deferred must either perform an error call back or push errors into
	 * the errors array.
	 */
	validate: function(/*Array*/errors) {

	}
	
});