/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.dsv.editor.visualization.ui.QVTEditor");

dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.form.SimpleTextarea");
dojo.require("dojox.grid.DataGrid");
dojo.require("dojox.fx.scroll");
dojo.require("rmpw.dsv.editor.visualization.ui.Toolbar");

/**
 * Editor for textual QVT.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.dsv.editor.visualization.ui.QVTEditor", dijit.layout.BorderContainer, {
	lineHeight: 14,
	
	postCreate: function(){
		this.inherited(arguments);
		var ActionRegistry = net.jazz.ajax.ui.PlatformUI.getWorkbench().getActionRegistry();
		ActionRegistry.registerAction("rmpw.dsv.editor.visualization.ui.startup", this, "startup"); 

		// listen for resizes
		this.resizer = jazz.util.ResizeNotifier.register({
			context: this,
			callback: this._onResize,
			force: true    	        
		});
		
    	var toolbar = new rmpw.dsv.editor.visualization.ui.Toolbar({
    		region: "top",
    		style: "height: 20px",
    		page: this
    	});    	
    	this.addChild(toolbar);
    	
    	this.sourceEditor = new dijit.form.SimpleTextarea({
    		region: "center",
    		name: "qvtSource",
    		style: "font-size: " + this.lineHeight + "px; font-family: monospace;"
    	});
    	dojo.attr(this.sourceEditor.domNode, 'spellcheck', false);
    	dojo.connect(this.sourceEditor.domNode, 'onkeypress', this, this._onEditorKeyPressed);
    	this.addChild(this.sourceEditor);

    	this.parserOutputGrid = new dojox.grid.DataGrid({
    		region: "bottom",
    		splitter: true,
    		style: "height: 100px;",
    		structure: [
				{ field: 'severity',	name: 'Severity', 	width: '50px' },
				{ field: 'line',		name: 'Line #',		width: '50px' },
				{ field: 'message', 	name: 'Message', 	width: 'auto' },
				{ field: 'phase',		name: 'Phase', 		width: '100px' },
				{ field: 'context', 	name: 'Context', 	width: '150px' }
			]
    	});
    	dojo.connect(this.parserOutputGrid, "onRowClick", this, "_onErrorSelected");
    	this.addChild(this.parserOutputGrid);
	},
    
    startup: function() {
    	this.inherited(arguments);
		this.parserOutputGrid.startup();
	},
	
	_onErrorSelected: function(e) {
		var selectedData = e.grid.getItem(e.rowIndex);
		
		// select the text range
		var start = selectedData.startOffset[0];
		var end = selectedData.endOffset[0] + 1;
		var node = this.sourceEditor.domNode;
		if(node.setSelectionRange) // firefox
		{
			node.focus();
			node.setSelectionRange(start, end);
		}
		else if (node.createTextRange) { // IE
			var range = node.createTextRange();
			range.collapse(true);
			range.moveEnd('character', end);
			range.moveStart('character', start);
			range.select();
		}

		// scroll to it
		var line = selectedData.line[0] - 1; 
		var anim = new dojox.fx.smoothScroll({
			win: this.sourceEditor.domNode,
			target: {
				x: 0,
				y: line * (this.lineHeight + 2) // +2 is a fudge factor
			}
		});
		anim.play();
	},
    
	_onEditorKeyPressed: function(e) {
		if(e.keyCode == dojo.keys.TAB) {
			var textarea = e.target;
			var scrollTop = textarea.scrollTop;
			
			if(textarea.selectionStart) { // firefox
				if(textarea.selectionStart == textarea.selectionEnd) {
					// make hitting tab add a tab instead of moving to the next form field
					var insertionPoint = textarea.selectionStart;
					var value = textarea.value;
					textarea.value = value.substring(0, textarea.selectionStart) + '\t' + value.substring(textarea.selectionEnd);
					textarea.focus();
					insertionPoint += 1;
					textarea.setSelectionRange(insertionPoint, insertionPoint);
				}
				else {
					// make hitting tab on a selection indent every line of the selection
					var start = textarea.selectionEnd;
					var stop = textarea.selectionStart;
					var value = textarea.value;
					for(var i = start, oneMore = false; i > stop || oneMore; i--) {
						currChar = value.charAt(i);
						oneMore = (i - 1) <= stop;
						if(currChar == '\n' && i != start) {
							var insertionPoint = i + 1; // +1 to skip newline itself
							value = value.substring(0, insertionPoint) + '\t' + value.substring(insertionPoint);
							oneMore = false;
						}
					}
					textarea.value = value;
					textarea.setSelectionRange(stop, stop);
				}
			}
			textarea.scrollTop = scrollTop;
			dojo.stopEvent(e);
		}
		else if(e.keyCode == dojo.keys.ENTER) {
			// make indentation of a new line match that of the last
			var textarea = e.target;
			var scrollTop = textarea.scrollTop;

			var value = textarea.value;
			if(textarea.selectionStart) { // firefox
				var insertionPoint = textarea.selectionStart;
				
				// count tabs on current line
				var tabs = 0;
				var i = insertionPoint - 1;
				var currChar = value.charAt(i);
				while(currChar != '\n' && i > 0) {
					if(currChar == '\t') {
						tabs++;
					}
					currChar = value.charAt(--i);
				}
				
				// create string of tabs
				var lineIndent = '';
				for(var i = 0; i < tabs; i++) {
					lineIndent += '\t';
				}
				
				// inject newline and tabs
				textarea.value = value.substring(0, textarea.selectionStart) + '\n' + lineIndent + value.substring(textarea.selectionEnd);
				
				// set caret position to expected
				textarea.focus();
				insertionPoint += 1 + tabs;
				textarea.setSelectionRange(insertionPoint, insertionPoint);
			}
			textarea.scrollTop = scrollTop;
			dojo.stopEvent(e);
		}
	},
	
    _onResize: function() {
    	var size = dojo.contentBox(dojo.byId('content-cell'));
    	this.resize(size);
    }
});