/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){

dojo.provide("rmpw.forms.ui.QuickSearchSelection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("rmpw.forms.ui.ValueScopeComboBox");
dojo.require("rmps.ui.Footer");

/*
* Provides a quick search selection for a given typeUri, so the user will use that selection control to search quickly for their resource
* 
* Also, it allows the users to enter manuell values if the type uri is a "Resource"; otherwise, the OK will be disabled in case the slected item is not in the list.
*/
dojo.declare("rmpw.forms.ui.QuickSearchSelection", [dijit._Widget, dijit._Templated], {

    templateString: dojo.cache("rmpw.forms", "ui/templates/QuickSearchSelection.html"),
	
	widgetsInTemplate: true,
	
	typeUri: null,
	defaultValue: null,
    _quickSearchSelectionsMessages: null,

    postMixInProperties: function(){
		this.inherited(arguments);
		
		dojo.requireLocalization("rmpw.forms.ui", "QuickSearchSelectionMessages");
		this._quickSearchSelectionsMessages = dojo.i18n.getLocalization("rmpw.forms.ui", "QuickSearchSelectionMessages");
	},
	
    postCreate: function(){
    	this.inherited(arguments);
    	
    	this._valueScopeComboBox = new rmpw.forms.ui.ValueScopeComboBox({
    			style: "width: 99%;",
    			parentWidget: this._selectionContainer,
    			context: this.context,
    			changeset: this.changeset,
    			typeUri: this.typeUri,
    			dir: "ltr",
    			defaultValue: this.defaultValue,
    			documentObject: this.documentObject,
    			resourceUri: this.resourceUri,
    			propertyUri: this.propertyUri,
    			isConstrained: this.isConstrained,
    			definableTypes: this.definableTypes,
    			propertyDomainUri: this.propertyDomainUri,
    			internalOslcRequest: this.internalOslcRequest});
    	this._selectionContainer.addChild(this._valueScopeComboBox);
    	
    	setTimeout(dojo.hitch(this, function() {
    		this._valueScopeComboBox.focus();		
		}), 1);
    	
		this.connect(this._valueScopeComboBox, "onChange", this._onValueChange);
		this.connect(this._valueScopeComboBox, "onKeyDown", this._onKeyDown);
    },
    	
	_onValueChange: function(event){
		if(!this._validateManualEntries()){
			return this._footer.setOKEnablement(false);
		}
		this._footer.setOKEnablement(this._valueScopeComboBox.get("item"));
	},
	
	_onKeyDown: function(event) {
		if(!this._opened&& event&& (event.keyCode === dojo.keys.ENTER|| event.charCode=== dojo.keys.ENTER)) {
			return this._onOK(event);
		}
	},
		
	_getSelection: function(){
		if(!this._valueScopeComboBox){
			return;
		}
		//If we are here, this means that we already validated non-resource types that we already chose an item [or] it could be just a resource
		var selection= this._valueScopeComboBox.get("item");
		
		return { rdfs_label: this._getSelectionValue(selection, "rdfs_label"), rdf_about: this._getSelectionValue(selection, "rdf_about")};
	},
	
	_validateManualEntries: function(){
		this.selectedType = this._valueScopeComboBox.get("item");
		//If enteredValue is undefined, then avoid closing the dialog, as we use it as a way to unset the value. We make the check against being
		//defined in order to adjust bad values, or to avoid accepting this value if the type uri is not a resource
		//If selectedType is undefined, then this means that the user type something not in the items. Otherwise, it means for sure
		//that they selected an item exists there, so, no need to have any kind of prevention
		if(!this.selectedType){
			return false;
		}
		return true;
	},
	
	_getSelectionValue: function(selection, attribute) {
		if(!selection){
			return;
		}
		var attributeValue = selection[attribute];
		if(!attributeValue) {
			return "";
		}
		
		if(dojo.isArray(attributeValue)) {
			return attributeValue[0];
		}
	},
	
	destroy: function(){
		if (this._valueScopeComboBox) {
			this._valueScopeComboBox.destroy();
		}
		this.inherited(arguments);
	},
	
	_onOK: function(e) {
		if(!this._validateManualEntries()){
			//Safe check, as we already validated that
			return false;
		}
		this.onOK(this._getSelection());
	},
	
	_onCancel: function(e) {
		//also pass selection on onCancel in case the client is intrested on knowing it
		this.onCancel(this._getSelection());
	},
	
	onOK: function(item){
		//Null Implementation
	},

	onCancel: function() {
		//Null Implementation
	}
});
    
})();
