/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2012. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("rmpw.forms.ui.ReferencesActionsTable");

dojo.require("rmpw.forms.ui.TableChangesMixin");
dojo.require("dijit._Templated");
dojo.require("rmpw.common.util.IdentifierUtil");

/*
 * Designed for single value references. So, we can show styled single row with predefined actions. 
 * 
 * TODO: We could change this class to accept more than one value if we are going to display more than one reference for an element
 * 
 * Related WIs: 25440: Support table widget in form generation
 * 
  */
dojo.declare("rmpw.forms.ui.ReferencesActionsTable", [rmpw.forms.ui.TableChangesMixin, dijit._Templated], {
	
	referencesActionsTable: null,
		
	templateString: '<div class="rmpw-dsv-editor-form-ui-reference-table" dojoAttachPoint="mainNode">'+
							'<div dojoAttachPoint="tableContainer"</div>'+
					'</div>',
	
	postMixInProperties: function(){
		dojo.requireLocalization("rmpw.forms.ui", "ReferencesActionsTableMessages");
		this.referencesActionsTable = dojo.i18n.getLocalization("rmpw.forms.ui", "ReferencesActionsTableMessages");
		this.inherited(arguments);
	},
	
	setTableArguments: function(){
		var args = this.inherited(arguments);
		
		dojo.mixin(args, {
			showHover: false,
		    getActionsColumnWidth: function(){
		    	var number= this.actionDescriptors.length;
		    	if(number=== 0){
		    		return number;
		    	}
		    	return number*8;
		    },
		});
		
		return args;
	},
	
	getActionDescriptors: function(){
		return [{actionCallback: dojo.hitch(this, this.clearRefrence), onCreateCallback: dojo.hitch(this, this.onClearCreate), id: "clear", args: {imgClass: 'clear', title: this.referencesActionsTable.clear}},
				{actionCallback: dojo.hitch(this, this.editRefrence), args: {imgClass: 'edit', title: this.referencesActionsTable.edit}}];
	},
	
	editRefrence: function(dataEntry, currentRow, currentActionsCell){
		this._row = currentRow;
		if (this.object && this.canEditItem(this.object.rdf_about)) {
			if (this.isDocument) {
				this.editExternalDocument(this.object, this._row);
			}
			else {
				this.newReference(this.object);
			}
		}
		else {
			var prevObj = this.object;
			
			this.useSelectDialog(currentRow);
		}
	},
	
	addReference: function(change, target){
		//this method is called in a form.applyChange(dojo.hitch(this, function(change){ ... }
		if (!this.form._resource) {
			this.object = target;
			change.set(this.featureId, target);
		} else {
			this.form.createChange(this.featureId, target);
		}
	},
	
	clearRefrence: function(dataEntry, row, currentActionsCell){
		delete this._row;
		this.table.updateRowCell(row, "name", {});
		this._updateClearReferenceEnablement(row, true);
		// note: see the note in onSetReference
		if (!this.form._resource) {
			this.set('value', null);
			this.onChange();
		} else {
			this.form.applyChange(dojo.hitch(this, function(change){
				this.form.createChange(this.featureId);
			}));
		}
		
		this.object = null;
		this.checkReload();
	},
	
	onClearCreate: function(actionDescriptor, button, tableRow, dataEntry){
		this._onClearReferenceEnablement(button, (!this._getData() || !this._getData().rdf_about || this._getData().rdf_about === ""));
	},
	
	_updateClearReferenceEnablement: function(tableRow, disabled){
		this._onClearReferenceEnablement(this.table.getRowInfomation(tableRow? tableRow : this._row).getActionWidget("clear"), disabled);
	},
	
	_onClearReferenceEnablement: function(button, disabled){
		if(button){	/*Will be null if the editor is being destroyed */
			button.domNode.style.visibility= disabled?"hidden":"";
		}
	},
	
	getFocusReference: function(){
		return this.form.getTarget()[this.featureName];
	},
	
	/*
	 * @param {null} object
	 */
	addNewItem: function(object) {
		if (this._row) {
			this.table.updateRowCell(this._row, "name", object);
		}
		else {
			this._row = this.inherited(arguments);
		}
		return this._row;
	},
	
	onSetReference: function(args){
		this.table.updateRowCell(this._row ? this._row : args.row, "name", args);
		this._addNewReference(args.target, args.namespaces, args.types, args.item, args.row);
		this.object = args.target;
		
		this.checkReload();
	},
	
	onAddNewReference: function(target, row){
		this._updateClearReferenceEnablement(row,!target);
	},
	
	getData: function(){
		return [this._getData()];
	},
	
	_getData: function(){
		if (this.object && dojo.isArray(this.object)) {
			// should not happen, just in case (if we didn't have restriction info in 
			// metamodel when converting to json, it would go into collection)
			return this.object[0]||{rdf_about: ""};
		}	
		return this.object||{rdf_about: ""};
	}
});

})();