/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){

dojo.provide("rmpw.forms.ui.ResourcesSelection");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.CheckBox");
dojo.require("rmps.ui.Footer");
dojo.require("rmpw.common.util.IdentifierUtil");

/*
* Resources type elements can be entered manuelly. The user can enter a url value or a literal value. This widget helps the users to enser the proper resource type values
*/
dojo.declare("rmpw.forms.ui.ResourcesSelection", [dijit._Widget, dijit._Templated], {

    widgetsInTemplate: true,
	selectTypeDialogMessages: null,
	isUri: null,
	
	templateString: '<div style="width: 100%; margin-left: -5px">' +
						'<div dojoType="dojox.layout.TableContainer" spacing=5>' +
							'<div label="${selectTypeDialogMessages.resource}" style="width: 100%;" dojoType="dijit.form.TextBox"  dojoAttachPoint="mainNode" class="header"></div>' +
							'<div label="${selectTypeDialogMessages.isURI}" "${id}_isURINode" dojoType="dijit.form.CheckBox" checked=true value= "on" dojoAttachPoint="isURINode"></div>' +
						'</div>'+
						'<br>' +
						'<div data-dojo-attach-point="_footer" data-dojo-attach-event="onOK:_onOK, onCancel:_onCancel" data-dojo-type="rmps.ui.Footer"></div>'+
					'</div>',
	
	constructor: function(args){
		var defaultValue=  args.value;
		this.isUri= !defaultValue|| new dojo._Url(defaultValue).scheme;
	},
	
	postMixInProperties: function(){
        dojo.requireLocalization("rmpw.forms.ui", "SelectTypeDialogMessages");
		this.selectTypeDialogMessages = dojo.i18n.getLocalization("rmpw.forms.ui", "SelectTypeDialogMessages");
        this.inherited(arguments);
    },
					
	postCreate: function(){
		this.inherited(arguments);
		this.connect(this.mainNode, "onKeyDown", this._onChange);
		
		this.mainNode.set("value", this.value);
		this.isURINode.set("value", this.isUri);
	},
	
	_setValueAttr: function(value){
		this.value= value;
	},
	
	_setIsUriAttr: function(value){
		this.isUri= value;
	},
	
	_onChange: function(event){
		if(event&& (event.keyCode === dojo.keys.ENTER|| event.charCode=== dojo.keys.ENTER)) {
			return this._onOK(event);
		}
		
		this._footer.setOKEnablement(this._validateManuelEntries());
	},
	
	_onOK: function(e) {
		if(!this._validateManuelEntries()){
			//Safe check, as we already validated that
			return false;
		}
		this.onOK(this._getSelection());
	},
	
	_getSelection: function(){
		var enteredValue= this.mainNode.get("value");
		if(!enteredValue){
			return "";
		}
		
		//TODO: Normalize the host name if it has whitespaces. This validation must be applied by validation role other than this way
		//TODO: Prevent having urls with undefined scheme as they are invalid. For now, we just assume that the protocol is http if the user
		//did not specify one. We can remove that condition once the validation framework is full implemented
		var about;
		if(this.isURINode.get("checked")){
			enteredValue = this.getNormalizedUri(enteredValue);
			about = enteredValue;
			return { rdfs_label: enteredValue,	rdf_about: about };
		} else {
			// when Is URI is not checked, then the entered value should be treated as a literal
			return enteredValue;
		}
	},
	
	_validateManuelEntries: function(){
		var enteredValue= this.mainNode.get("value");
		
		if(!enteredValue){
			return false;	
		}
		return true;
	},
	
	_onCancel: function(e) {
		//also pass selection on onCancel in case the client is intrested on knowing it
		this.onCancel(this._getSelection());
	},
	/* 
	 * Return a normalized (valid) for the passed uri value. It's used to avoid bad uri values entered by users
	 * This concept is versus validation that we are going to handle later on on our validation framework, so we shall remove that check once our validation
	 * framework is done
	 */
	getNormalizedUri: function(uriValue){
		var _uri= new dojo._Url(uriValue);

		var normalizedValue= _uri.scheme?_uri.scheme: "http";
		normalizedValue+= "://";
		
		/*
		 * If host is undefined, this means that the user just entered a string with whitespace, so we will assume that this string would be the hosy
		 * but also, we have to make sure to remove the whitespaces in order to make it a valid host name
		 */
		normalizedValue += !_uri.host?uriValue.split(' ').join(''): _uri.host.split(' ').join('');
		
		if(_uri.port){
			normalizedValue = normalizedValue+ ":" + _uri.port;
		}
		
		if(_uri.path&&_uri.host){
			normalizedValue = normalizedValue+ _uri.path;
		}
		
		if(_uri.query){
			normalizedValue = normalizedValue+ "?" + _uri.query;
		}
		
		if(_uri.fragment){
			normalizedValue = rmpw.common.util.IdentifierUtil.normalizeURI(normalizedValue) + _uri.fragment;
		}else{
			// add hash if URI is front side
			if (normalizedValue.indexOf(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL) > -1) {	/*Safe check */
				var uriLength = uriValue.length;
				var lastSegment= uriValue.substring(uriLength - 1, uriLength);
				if(lastSegment !== "#"){
					//Hash is ignored if it not followed by any query string, so make sure to add it
					normalizedValue += "#";
				}	
			}
			
		}
		return normalizedValue;
	},
	
	onOK: function(item){
		//Null Implementation
	},

	onCancel: function() {
		//Null Implementation
	}
});
    
})();
