/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.ReviewEditorUtil");

dojo.require("rmps.core.error");
dojo.require("rmps.core.process.SecurityClient");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

var SecurityClient= rmps.core.process.SecurityClient;

dojo.requireLocalization("rmps.app.ui", "ReviewEditorUtilMessages");
var reviewEditorUtilMessages = dojo.i18n.getLocalization("rmps.app.ui", "ReviewEditorUtilMessages");

rmps.app.ui.ReviewEditorUtil = {
	
	REVIEW_EDITOR_VIEWER: "reviewEditorViewer",
	
	REVIEW_EDITOR: "reviewEditor",
	RES_REVIEW_PREFIX: "/dm/reviews/",
	
	process: {

		Operations: {
			ALL_AUTHORING: "com.ibm.xtools.rmps.process.operations.review.authoring",
			ALL_FEEDBACK: "com.ibm.xtools.rmps.process.operations.review.feedback",
			DELETE: "com.ibm.xtools.rmps.process.operations.review.authoring~#_~#delete",
			MODIFY: "com.ibm.xtools.rmps.process.operations.review.authoring~#_~#modify",
			CREATE: "com.ibm.xtools.rmps.process.operations.review.authoring~#_~#create",
			VOTE: "com.ibm.xtools.rmps.process.operations.review.feedback~#_~#vote"
			
		},

		Actions: {
			CREATE: "create",
			DELETE: "delete",
			MODIFY: "modify",
			VOTE: "vote"
		}
	},
	
	_basicPrefix: "#action=dm.app.review.",
	
	NEW_REVIEW_EDITOR: "rmps.app.newReviewEditor",
	
	canCreateNewReview: function(callback, informError, errorCallback){
		var process= rmps.app.ui.ReviewEditorUtil.process;
		var deferred = SecurityClient.isActionAllowed(
				process.Operations.ALL_AUTHORING,
			dojo.getObject("jazz.app.currentApplication").context,
			[
				process.Actions.CREATE
			]);
		deferred.addCallback(this, function(report) {
			if(report.isAllowed(process.Actions.CREATE)){
				callback();				
			}else{
				if(informError){
					rmps.core.error.create(reviewEditorUtilMessages.noCreatePermissions, null, null).dispatch();					
				}
				if(dojo.isFunction(errorCallback)){
					errorCallback();
				}
			}
		});
		
		deferred.addErrback(this, function(error) {
			if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
				rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
			} else {
				rmps.core.error.createError(error).dispatch();
			}
		});
	},
	
	startNewReviewEditor: function(){
		return rmps.app.ui.ReviewEditorUtil.canCreateNewReview(function(){
			location.href= rmps.app.ui.ReviewEditorUtil.getNewReviewEditorUri();			
		}, true);
	},
	
	getNewReviewEditorUri: function() {
		return rmps.app.ui.ReviewEditorUtil._basicPrefix + "newReviewEditor&"+ dojo.objectToQuery({ts: new Date().getTime()});
	},

	getReviewViewerEditorUri: function() {
		return rmps.app.ui.ReviewEditorUtil._basicPrefix + "reviewViewerEditor";
	},

	getReviewSearchUri: function(queryString)  {
		return rmps.app.ui.ReviewEditorUtil.getReviewViewerEditorUri() + "&" + dojo.objectToQuery({query: "search&"+ queryString});
	},
	
	getReviewEditorUri: function(resourceUri, ts)  {
		var obj = {
			"resourceUri": resourceUri,
			"type": rmps.app.ui.ReviewEditorUtil.REVIEW_EDITOR
		};
		if(ts){
			obj.ts= ts;	
		}
		return rmps.app.ui.ReviewEditorUtil._basicPrefix + "viewResource&" + dojo.objectToQuery(obj);
	}
};

})();