/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.forms.ui.SelectCreatePicker");

dojo.require("dijit._WidgetBase");
dojo.require("dijit._Templated");

dojo.require("dojo.string");
dojo.require("dojo.i18n");

var SelectionsPicker= "rmps.app.pickers.internal.SelectionsPicker";

/* 
 * Designed mainly for OSLC purposes and to match the jazz UI requiremetns as in
 * 		https://jazz.net/wiki/bin/view/Main/LinkingGuide
 * TODO: We still need to implement the creation content as creation interface picker
 * TODO: Support auto creation flag for the OSLC option, so we can use the creation picker in all places such as review and analysis.
 * TODO: Should rely on Jazz API: jazz.ui.ResourcePicker
 * 			However, the API is not ready yet, and still not customizable for our purposes (such as label differences).
*/
dojo.declare("rmpw.forms.ui.SelectCreatePicker",[dijit._Widget, dijit._Templated], {
	
	title: null,
	selectionsLabel: null,
	creationsLabel: null,
	typeLabel: null,
	creationArgs: null,
	selectionArgs: null,
	messages: null,
	
	templateString: dojo.cache("rmpw.forms", "ui/templates/SelectCreatePicker.html"),
	widgetsInTemplate: true,
	
	_creationWidget: null,
	_selectionWidget: null,
	_existingItem: null,
	_newItem: null,
	_linkTo: null,
	
	postMixInProperties: function() {
		dojo.requireLocalization("rmpw.forms.ui", "SelectCreatePickerMessages");
		this.messages = dojo.i18n.getLocalization("rmpw.forms.ui", "SelectCreatePickerMessages");
		
		this._loadingMessage = this.messages.loading;
		
		this._linkTo = this.messages.linkTo;
		this._existingItem= dojo.string.substitute(this.messages.existingResource, [this.typeLabel||this.messages.resource]);
		this._newItem= dojo.string.substitute(this.messages.newResource, [this.typeLabel||this.messages.resource]);
	},
	
	postCreate: function(){
		this.inherited(arguments);
		
		//Show the contents of the default slection
		this._onSelectExisting();
	},
		
	_onSelectExisting: function() {
		var self= this;
		this._onSelect(function(){return self._selectionWidget}, function(value){self._selectionWidget= value}, this.selectionArgs, this.selectionsLabel);
	},
	
	_onSelectNew: function() {
		var self= this;
		this._onSelect(function(){return self._creationWidget}, function(value){self._creationWidget= value}, this.creationArgs, this.creationsLabel);
	},
	
	_onSelect: function(getter, setter, args, label){
		this._removeAllChildren();
		
		var get= getter();
		if(get){
			get.onShow();
			return this._contentArea.addChild(get);
		}
		
		if(dojo.isFunction(args)){
			return args(dojo.hitch(this, function(argsOrWidget){
				if(dojo.isArray(argsOrWidget)){
					return jazz.core.loader.batch_load_async([SelectionsPicker], dojo.hitch(this, function() {
						setter(new dojo.getObject(SelectionsPicker)({
							argsList: argsOrWidget,
							selectionsLabel: label
						}));
						this._contentArea.addChild(getter());
					}));			
				}else if(argsOrWidget instanceof dijit._WidgetBase){
					setter(argsOrWidget);
					return this._contentArea.addChild(getter());
				}
				return rmps.core.error.create("Your creation callback must return a widget ir selection arguments").dispatch();
			}));
		}
		
		jazz.core.loader.batch_load_async([SelectionsPicker], dojo.hitch(this, function() {
			setter(new new dojo.getObject(SelectionsPicker)({
				argsList: args,
				selectionsLabel: label
			}));
			this._contentArea.addChild(getter());
		}));
	},
	
	_removeAllChildren: function(){
		if (this._contentArea) {
			while(this._contentArea.hasChildren()){
				this._contentArea.removeChild(this._contentArea.getChildren()[0]);
			}
		}
	},
	
	destroy: function(){
		this._removeAllChildren();
		if(this._creationWidget){
			this._creationWidget.destroyRecursive();
			delete this._creationWidget;
		}
		if(this._selectionWidget){
			this._selectionWidget.destroyRecursive();
			delete this._selectionWidget;
		}
		this.inherited(arguments);
	}
	
});

})();
