/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2012, 2013. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("rmpw.forms.ui.SelectOrCreateFile");

//Load async modules
var FileUploader= "rmpw.forms.ui.FileUploader";

/* 
 * Advanced Select dialog to support file/image upload. Used in two cases:
 * - For file upload form that allows the file upload. In this case, isFileUploadForm is true.
 * - For forms opened from a File or Image reference property. 
 */
dojo.declare("rmpw.forms.ui.SelectOrCreateFile", rmpw.forms.ui.SelectOrCreateResource, {
	
	isImage: null,
	isFileUploadForm: null,
	fileUploaderMessages: null,
	
	constructor: function(args) {
		this.isImage = args.isImage;
		this.isFileUploadForm = args.isFileUploadForm;
		this.parentForm = args.parentForm;
		
		dojo.requireLocalization("rmpw.forms.ui", "FileUploaderMessages");
		this.fileUploaderMessages = dojo.i18n.getLocalization("rmpw.forms.ui", "FileUploaderMessages");
	},
	
	isSelectCreatePicker: function() {
		return true;
	},
	
	isCreatePicker: function() {
		return false;
	},
	
	getDialogSecondaryTitle: function(){
		return "";
	},
		
	getSelectTypeClassConstructor: function(callback){
		if(!this.isFileUploadForm){
			return this.inherited(arguments);
		}
		jazz.core.loader.batch_load_async([FileUploader], function() {
			callback(dojo.getObject(FileUploader));
		});
	},
	
	getSelectTypeArguments: function(dialogGetter, row){
		if(!this.isFileUploadForm){
			return this.inherited(arguments);
		}
		return {
			isImage: this.isImage,
			frontSideUri: this.defaultValue,
			onOK: dojo.hitch(this, function(uri, label){
				//The way we save new images/files is by uploading the file/image, and then commit and save the current ontology right away.
				//So, if the current editor is not dirty yet, this means that the resource is still unlocked; otherwise, it is locked.
				//If the file/image is unlocked, we have to be carefull with the current sequnce of event
				//1) When making a change to update the label, this will go through a deferred call (as it is the first time)
				//2) When making a change to update the uri field, this will also go through another deferred call (the first deferred is not answered)
				//3) And then the resource state call for locking the current resource will come. This call is also deferred. We can get into a situation that
				//   the first deferred call is answered and then the resource is unlocked and then the next deferred call is answered, or it could be in any different
				//   order. So, we have to make sure to set our listenrs as follows:
				// If the editor is not dirty. Then, we put a listener on the method "afterCommit". Make sure that it will be called twice ( a call for eahc call).
				// Then, we have to check and see if the editor is not dirty (as na indication that the model was not orginally locked). If it is not dirty, then, we
				// we have to put a listener on the resource state after lock method. And after that, we are fine to save and referesh the new ontology
				// If the editor is dirty. Then, we will only need to listen to the two calls for afterCommit, and then save an continue.
				// For the existing image/file changes, we follow the exact procedure, but the difference here is in behvaior. We only update the thumbnail, and
				// do not close the dialog
				var readyToSave = false;
				if (!this.parentForm.isInPopup) {
					// launched off of the stand-alone file upload form which we want to save as soon as the file is uploaded in case of initial upload;
					if (!this.defaultValue) {
						var onCommit = dojo.connect(this.parentForm.repository, "afterCommit", dojo.hitch(this, function(change){
							//Used to indicate if the second field has been updated as well, so we make sure that the target is updated with both values
							//when tried is true
							if (readyToSave) {
								var doSave = dojo.hitch(this, function(){
									dojo.disconnect(onCommit);
									// save the form so we don't lose the reference to the uploaded file								
									this.parentForm._resource.page.editor.save();
									this.selectDialog.close();
								});
								//If editor is not dirty, then, we have to make sure that we will not save till the resource is locked, so, we need
								//to listen to the resource state till it becomes unlocked
								var resourceState = this.parentForm._resource.page.editor.editorInput.store.resourceState;
								if (!this.parentForm._resource.page.editor.isDirty()) {
									var onLock = dojo.connect(this.parentForm._resource.page.editor.editorInput.store.resourceState, "onLock", dojo.hitch(this, function(change){
										dojo.disconnect(onCommit);
										//Now, we can save. Both fields are updated, and the resource state is locked
										doSave();
									}));
								}
								else {
									//Save directly. Both fields are saved, and resource was already locked
									doSave();
								}
								
							}
							
						}));
					}
				}
				//TODO: Should be revisited with 26815: Changing values and saving in one form editor does not update the contents of another form editor
				this.parentForm.createChange("rdfs_label", label);
				this.parentForm.createChange("dcterms_title", label);
					
				dojo.publish("dsf-widget-update-" + this.parentForm.id + "rdfs_label", [label]);
				dojo.publish("dsf-widget-update-" + this.parentForm.id + "dcterms_title", [label]);
				readyToSave = true;
				this.onSelectArgs.target = uri;
				this.onSelectTarget[this.onSelectFunctionName](this.onSelectArgs);
				
				if (this.parentForm.isInPopup) {
					this.selectDialog.close();
				}
			}),
			isReadOnly: this.readOnly,
			parentForm: this.parentForm,
			onCancel: dojo.hitch(this, this.onCancel),
		};
	},
		
	getDialogTitle: function(){
		if(!this.isFileUploadForm){
			return this.inherited(arguments);
		}
		if (this.defaultValue) {
			return this.readOnly? this.fileUploaderMessages.openFile: this.fileUploaderMessages.openReplaceFile;
		} else {
			return this.isImage? this.fileUploaderMessages.uploadImage: this.fileUploaderMessages.uploadFile;
		}
	}
});

})();