/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2012, 2013. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("rmpw.forms.ui.SelectOrCreateResource");

dojo.require("rmps.core.util.TypeConstants");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmpw.common.util.FormsLauncher");
dojo.require("rmpw.common.util.IdentifierUtil");
dojo.require("rmpw.forms.ui.ModelValidationUtil");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.ui.editor.PageHelper");
dojo.require("rmps.app.pickers.internal.CustomizedPickerController");

var TypeConstants= rmps.core.util.TypeConstants;
var ResourceInfoClient= rmps.core.resource.ResourceInfoClient;
var ModelValidationUtil= rmpw.forms.ui.ModelValidationUtil;

//Load async modules
var SelectCreatePicker= "rmpw.forms.ui.SelectCreatePicker";
var CreatePicker= "rmpw.forms.ui.CreatePicker";
var SelectionsPicker= "rmps.app.pickers.internal.SelectionsPicker";
var ClipboardWidget= "rmps.ui.clipboard.ClipboardWidget";
var ExplorerWidget = "rmps.ui.explorer.ExplorerSearchWidgetWithProjectSelection";
var RMPSPicker= "rmps.app.pickers.internal.RMPSPicker";
var QuickSearchSelection= "rmpw.forms.ui.QuickSearchSelection";
var ResourcesSelection= "rmpw.forms.ui.ResourcesSelection";

var DEFAULT_DIALOG_WIDTH= "750px";

/**
 * Generic utility to select or create a new resource
 */
dojo.declare("rmpw.forms.ui.SelectOrCreateResource", null, {
		
	constructor: function(args) {
		this.typeUri = args.typeUri;
		this.propertyUri = args.propertyUri;
		this.propertyDomainUri = args.propertyDomainUri;
		this.typeLabel = args.typeLabel;
		this.typeForm = args.typeForm; // the form to create new resource of this type
		this.isFragment = args.isFragment;
		this.isDocument= args.isDocument;
		this.definableTypes = args.definableTypes;
		this.documentObject = args.documentObject;
		this.resourceUri = args.resourceUri;	
		this.isConstrained = args.isConstrained;
		this.defaultValue = args.defaultValue;
		this.isReadOnly = args.isReadOnly;
		this.editor = args.editor;
		this.namespaces = args.namespaces;
		this.page = args.page;
		this.formObject = args.formObject;
		this.canSelect = args.canSelect;
		this.enableMultiSelect = args.enableMultiSelect;
		
		this.onSelectTarget = args.onSelectTarget
		this.onCancelFunctionName = args.onCancelFunctionName;
		this.onSelectFunctionName = args.onSelectFunctionName;
		this.onSelectArgs = {};
		dojo.mixin(this.onSelectArgs, args.onSelectArgs);
		
		dojo.requireLocalization("rmpw.forms.ui", "SelectOrCreateResourceMessages");
		this.selectOrCreateResourceMessages = dojo.i18n.getLocalization("rmpw.forms.ui", "SelectOrCreateResourceMessages");

		dojo.requireLocalization("rmps.app.ui.internal.editors", "BaseEditorMessages");
		this.baseEditorMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "BaseEditorMessages");
		
		this.listeners = [];
	},
	
	useSelectDialog: function(){
		this.getSelectTypeClassConstructor(dojo.hitch(this, function(clazz){
			var dialog;
			
			var instance = new clazz(this.getSelectTypeArguments(function(){return dialog;}));
			this.selectDialog = new jazz.ui.Dialog({
				contentNode: instance.domNode,primaryTitle: this.getDialogTitle(), 
				secondaryTitle:"", destroyContents: true, width: DEFAULT_DIALOG_WIDTH,
				onClose: dojo.hitch(this, this.onClose),
			});
			
			instance.startup();
		}));
	},
	
	onClose: function() {
		dojo.forEach(this.listeners, dojo.disconnect);
		this.listeners = [];
	},
	
	getDialogTitle: function(){
		return dojo.string.substitute(this.selectOrCreateResourceMessages.addType, [this.typeLabel]);
	},
	
	/**********************************************************Dialog Descriptors (Start)****************************************************************************** */
	/**Can be extended to add further descritpros */
	getDialogSelectionsDescriptors: function(getDialog){
		if (this.typeUri=== TypeConstants.RESOURCE) {
			// resource selection also allows External Resource specification (by uri or literal) - the first in the list below
			return [this.getExternalResourceSearchDialogDescriptor(getDialog), this.getSearchDialogDescriptor(getDialog), this.getClipboardDialogDescriptor(getDialog), this.getExplorerDialogDescriptor(getDialog)];			
		}
		return [this.getSearchDialogDescriptor(getDialog), this.getClipboardDialogDescriptor(getDialog), this.getExplorerDialogDescriptor(getDialog)];
	},
	
	getExternalResourceSearchDialogDescriptor: function(){
		return {classConstructor: ResourcesSelection,
				args: {
					typeUri: this.typeUri,
					definableTypes: this.definableTypes,
					propertyUri: this.propertyUri,
					propertyDomainUri: this.propertyDomainUri,
					documentObject: this.documentObject,
					resourceUri: this.resourceUri,	
					isConstrained: this.isConstrained,
					defaultValue: this.defaultValue,
					onOK: dojo.hitch(this, function(item){
						dojo.mixin(this.onSelectArgs, {target: item});
						this.onSelectTarget[this.onSelectFunctionName](this.onSelectArgs);
						this.selectDialog.close();}
				),
				onCancel: dojo.hitch(this, function() {
					this.onCancel();
				})
			},
			title: this.selectOrCreateResourceMessages.externalResourceSearch
		}
	},
	
	getSearchDialogDescriptor: function(){
		return 	{classConstructor: RMPSPicker,
				args: {
					projectId: jazz.app.context.get().itemId,
					context: jazz.app.context.get().baseline,
					onlyType: this.typeUri,
					includeSuppliers: true,
					controller: new rmps.app.pickers.internal.CustomizedPickerController({
						resourceUri: this.resourceUri,
						propertyUri: this.propertyUri,
						propertyDomainUri: this.propertyDomainUri,
						typeUri: this.typeUri,
						internalOslcRequest: this.internalOslcRequest,
						context: this.context,
						changeset: this.changeset,
						enableMultiSelect: this.enableMultiSelect
					}),
					onClose: dojo.hitch(this, function(selections){
						var newSelectionArgs = [];
						for(var index = 0, selection; selection = selections[index]; index++) {
							var selectArgs = {};
							dojo.mixin(selectArgs, this.onSelectArgs);
							selectArgs.target = { rdfs_label: selection.shortName, rdf_about: selection.link };
							newSelectionArgs.push(selectArgs);
						}
						this.onSelectTarget[this.onSelectFunctionName](newSelectionArgs);
						this.selectDialog.close();
					}),
					onCancel: dojo.hitch(this, function() {
						this.onCancel();
					})
				},
				title: this.selectOrCreateResourceMessages.search
			};
	},
	
	getClipboardDialogDescriptor: function(){
		return 	{classConstructor: ClipboardWidget,
				args: {
					projectId: jazz.app.context.get().itemId,
					baseline: jazz.app.context.get().baseline,
					onOkButton: dojo.hitch(this, function(selections){
						//Close the dialog first to avoid mutlipe clicks, as the call for resource info can take a moment
						this.selectDialog.close();
						
						ResourceInfoClient.getResourceInfo(selections, ["uri","shortName"]).then(dojo.hitch(this, function(info) {
							for(var index = 0, selection; selection = info[index]; index++) {
								dojo.mixin(this.onSelectArgs, {target: {rdfs_label: selection.shortName, rdf_about: selection.uri}});
								this.onSelectTarget[this.onSelectFunctionName](this.onSelectArgs);
							}
						}));
					}),
					onClickCancelButton: dojo.hitch(this, function() {
						this.onCancel();
					}),
					readOnly: false,
					onlyType: this.typeUri
				},
				title: this.selectOrCreateResourceMessages.clipboard
			};
	},
	
	getExplorerDialogDescriptor: function(){
		return 	{classConstructor: ExplorerWidget,
				args: {
					projectUri: dojo.getObject("jazz.app.currentApplication").context,
					baseline: jazz.app.context.get().baseline,
					onOkButton: dojo.hitch(this, function(selections){
						//Close the dialog first to avoid mutlipe clicks, as the call for resource info can take a moment
						this.selectDialog.close();
						
						for(var index = 0, selection; selection = selections[index]; index++) {
							dojo.mixin(this.onSelectArgs, {target: {rdfs_label: selection.label, rdf_about: selection.rdf_about}});
							this.onSelectTarget[this.onSelectFunctionName](this.onSelectArgs);
						}
					}),
					onClickCancelButton: dojo.hitch(this, function() {
						this.onCancel();
					}),
					readOnly: false,
					onlyType: this.typeUri,
					enableMultiSelect: this.enableMultiSelect
				},
				title: this.selectOrCreateResourceMessages.explorer
			};
	},
	/***********************************************************Dialog Descriptors (End)****************************************************************************** */
	
	isSelectPicker: function() {
		if (this.isConstrained && this.definableTypes && this.definableTypes.length == 1 && this.canSelect) {
			// don't allow creation if the property is live constrained and only one definable type - limit to one definable type because
			// there may be anonymous classes allowed which fall outside the scope of the constraint.  
			return true; 
		}
		
		return false;
	},
	
	isCreatePicker: function() {
		if (this.isSelectPicker()) {
			return false; 
		}
		return (this.isDocument || this.isFragment || (this.definableTypes && (this.definableTypes.length > 0))) && !this.canSelect;
	},
	
	isSelectCreatePicker: function() {
		if (this.isSelectPicker()) {
			return false; 
		}
		return (this.isDocument || this.isFragment || (this.definableTypes && (this.definableTypes.length > 0))) && this.canSelect;
	},
	
	/*
	 *Can be extended by sub-classes 
	*/
	getSelectTypeClassConstructor: function(callback){
		if (this.isSelectCreatePicker()){
			return jazz.core.loader.batch_load_async([SelectCreatePicker], function() {
				callback(dojo.getObject(SelectCreatePicker));
			});		
		}
		else if (this.isCreatePicker()) {
			return jazz.core.loader.batch_load_async([CreatePicker], function() {
				callback(dojo.getObject(CreatePicker));
			});	
		}
		
		// Select picker only
		return jazz.core.loader.batch_load_async([SelectionsPicker], function() {
				callback(dojo.getObject(SelectionsPicker));
			});
	},
	
	/*
	 *Can be extended by sub-classes 
	*/
	getSelectTypeArguments: function(){
		if (this.isSelectCreatePicker()) {
			return {creationArgs: dojo.hitch(this, function(callback){
					this.createCreationContents(callback);
				}),
				selectionsLabel: this.selectOrCreateResourceMessages.selectionsLabel,
				creationsLabel: this.selectOrCreateResourceMessages.creationsLabel,
				typeLabel: this.typeLabel,
				selectionArgs: this.getDialogSelectionsDescriptors()
			};
		}
		else if (this.isCreatePicker()) {
			return {creationArgs: dojo.hitch(this, function(callback){
					this.createCreationContents(callback);
				}),
				creationsLabel: this.selectOrCreateResourceMessages.creationsLabel,
				typeLabel: this.typeLabel
			};
		}

		// Select picker only
		return {argsList: this.getDialogSelectionsDescriptors(),
			selectionsLabel: this.selectOrCreateResourceMessages.selectionsLabel
		};
	},
	
	//FIXME: This method can be enhanced
	//Can be extended by sub classes
	createCreationContents: function(callback) {
		var whereLabel = "OPTIONAL { ?value rdfs:label ?rdfs_label ." +
			"       FILTER (lang(?rdfs_label) = '"+ dojo.locale +"') }" +
			"OPTIONAL { ?value rdfs:label ?rdfs_label ." +
			"       FILTER (langMatches(lang(?rdfs_label) , '')) }" +
			"OPTIONAL { ?value rdfs:label ?rdfs_label }";
		
		var types = this.definableTypes;
		if (!types) {
			types = [this.typeUri];
		}
		
		var whereDefinableTypes = "";
		for (var i=0, lg=types.length; i<lg; i++) {
			var whereLine = "?value rdfs:subClassOf* <"+ types[i] +">."+
							"?value rdf:type owl:Class .";
			if (this.isConstrained) {
				whereLine += "?value rdf:type dmcore:AnonymousClass . ";
			}
			
			if (i==0) {
				whereDefinableTypes += "{ " + whereLine + " }";
			}
			else {
				whereDefinableTypes += "UNION { " + whereLine + " }";
			}
		}
		
		var whereAllTypes = "?value rdfs:subClassOf* <"+ this.typeUri +">."+
			"?value rdf:type owl:Class ."+
			"?value rdf:type ?classType ."+ whereLabel;
		
		var whereClause = whereAllTypes;
		if (this.isDocument) {
			whereClause = whereAllTypes;
		}
		else if (this.definableTypes && this.definableTypes.length > 0) {
			whereClause = whereDefinableTypes;
		}
		
		var self = this;
		// use simple select type dialog, till we have the search dialog works 
		rmps.core.client.xhrGet({
			url: net.jazz.ajax._contextRoot + "/modelvalidation/values",
			handleAs: 'json',
			headers: {
				"Accept" : "application/json",
				"X-ibm-rmps-internal" : "true"
			},
			content: {
				resourceUri: this.resourceUri,
				adjustedRangeUri: this.typeUri,
				propertyUri: "http://www.w3.org/1999/02/22-rdf-syntax-ns#type",
				adjustedDomainUri: this.typeUri,
				whereClause: whereClause
			},
			load: function(data, ioArgs) {
				if(!data){
					return;
				}
				
				var types = [];
				for (var i=0, lg=data.length; i<lg; i++) {
                    types.push(self.getTypeObject(data[i]));
				}
				
				//FIXME: We can use construct statement in the SPARQL query, but this will require changes in the query service to accept the construct
				//body. And since this query is a temp solution till we get our enhancmenets for queries in, we can just use this quick mapping way.
				var items= dojo.map(types, function(entry){return {title: entry.rdfs_label, identifier: entry.rdf_about, tooltip: entry.rdfs_label, 
				classConstructor: function(type, callback){
					// TODO: exploit common code with rmpw.common.store.JfsStore
					// request initial model (may contain default values)
					rmps.core.client.xhrGet({
						url: rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "rdfdirectory") + "/constructor",
						content: { rdf_type: type },
						headers: { "Accept": "application/x-rdf+json-graph" },
						handleAs: "text",
						load: function(data, ioargs) {
							
							// process json for initial model
							if(!data) {
								rmps.core.error.create("Failed to load initial model").dispatch();
								return;
							}
							
							var documents = self.processFromJSON(data);
							var model = documents[0];
							
							// TODO: shouldn't have to assign this, constructor service should provide correct uri
							model.rdf_about = self.defaultUri;
							
							var objectTypes= dojo.map(model.rdf_type, function(type) { return type.rdf_about; });
							// create form for it
							rmpw.common.util.FormsLauncher.loadForm({
									formName: self.typeForm,
									objectTypes: objectTypes,
									readOnly: self.isReadOnly,			//FIXME: Anti-pattern to have this long list of setters.
									standAlone: false,
									editor: self.editor,	
									create: true
								}, 
								function(signature){
									var creationForm = new rmpw.dsf.codegen.properties.forms[signature.formName]();
									//TODO: Must provide a setter instead
									creationForm.featureId = self.featureId;
																		
									if (self.isDocument) { 
										// Change Ok to Save Document to be clear the document will be saved right away
										creationForm._footer.okCmd.firstChild.data = self.selectOrCreateResourceMessages.saveDocument;
									}
																		
									var method = creationForm.setterNames[self.typeForm];
									creationForm[method](model);
									
									creationForm.setChangeDetails();
									creationForm._setResource({
										documentObject: self.documentObject,
										id: self.resourceUri,
										namespaces: self.namespaces ? self.namespaces : [],
										objectTypes: objectTypes,
										page: self.page,
										repository: creationForm.repository,
										editor: self.editor,
										formObject: self.formObject,
										modelIndex: rmps.ui.editor.PageHelper.getInstance().getActiveEditorInput().resourceInfo.modelIndex
									});
									creationForm.isInPopup = true;
									
									var rdf_about= model.rdf_about;
									if(!rdf_about){	// leave defaultUri null, trigger autogen on server (if document)
										if (self.isDocument) {
											rdf_about = "http://new";
										}
										else {
											rdf_about = rmpw.common.util.IdentifierUtil.guid();
										}
									}
									
									creationForm.createChange("rdf_about", rdf_about);
									creationForm.immediate= true;
									creationForm._setRoot(self);
									
									// make sure JfsStore listens to creationForm's repository commits (as well as main form's repository) 
									// so proper locking happens in case of external document edits off of this creationForm
									var store = self.page.editor.editorInput.store; 
									var repoListeners = store.addRepositoryListeners(creationForm.repository);
									for (var i = 0, l; l = repoListeners[i]; i++) {
										self.listeners.push(repoListeners[i]);
									}
									
									self.listeners.push(dojo.connect(creationForm, "onSubmit", self, 
										dojo.hitch(this, function(args) {
											var errors = [];
											ModelValidationUtil.validatePopup(creationForm, errors);
											if (errors.length > 0) {
												ModelValidationUtil.showValidationError(errors[0]);
											} else {
												// disable save button
												var saveButton = null;
												if (args && args[0] && args[0].currentTarget) {
												    saveButton = args[0].currentTarget;
													saveButton.disabled = true;
												}
												if (creationForm.preProcessOnSave) {
													// pick up any changes to rich text bodies
													creationForm.preProcessOnSave();
												}
												var repository= creationForm.getRepository();
												var changeSets = [];
												if (repository.patch) {
													changeSets.push(repository.patch);
												}
												changeSets.push(repository.stack);
												dojo.mixin(self.onSelectArgs, {
													target: creationForm.getTarget(), 
													namespaces: creationForm.getNamespaces(), 
													types: creationForm.getTypes(),
													item: changeSets
												});
												
												if (self.isDocument) {
													self.selectDialog.style.cursor = 'progress';
													var div = dojo.create("div", { innerHTML: "<p style='margin: 10px'>" +  self.baseEditorMessages.saving + "</p>" });		
													self.progressMessage = new jazz.ui.Dialog({
														contentNode: div,
														primaryTitle: self.selectOrCreateResourceMessages.saveDocument
													});													
													
													var saveCallback = store.saveResource(changeSets, {rdf_type: creationForm.getTypes(), checkForRelock: "true"},
															dojo.hitch(this, function(response, ioArgs){
																self.onSelectArgs.target.rdf_about = ioArgs.xhr.getResponseHeader("logicalURI");
																self.onSelectArgs.dataAvailable = true;
																self.onSelectTarget[self.onSelectFunctionName](self.onSelectArgs);
																store.resetLockedUris();
																self.progressMessage.close();
																self.selectDialog.close();																
															}),
															dojo.hitch(this, function(error, noData) {
																self.progressMessage.close();
															    if (error || noData) {
															        //error is already handled using popup dialog
															    	// enable save button and let user to fix error if possible																	
																	if (saveButton) {
																		saveButton.disabled=false;
																	}
														            return;
														        }
																self.selectDialog.style.cursor = '';
																self.selectDialog.close();
																
																if (error && self.onCancelFunctionName) {
																	self.onSelectTarget[self.onCancelFunctionName]();
																}
															}), null, {error:function(message){ModelValidationUtil.showValidationError(message);}}, null);													
												}
												else {
													self.onSelectTarget[self.onSelectFunctionName](self.onSelectArgs);
													self.selectDialog.close();
												}
											}
										})));
									
									self.listeners.push(dojo.connect(creationForm, "onCancel", self, "onCancel"));	
									
									callback(creationForm);
								}
							);
						}
					});
				}};});
				if(items.length < 1){
					return;
				}
				callback(items);
			},
			error: function(error) {return rmps.core.error.createError(error).dispatch();}
		}, true);
    },
    
    onSubmit: function() {
    	var errors = [];
		ModelValidationUtil.validatePopup(creationForm, errors);
		if (errors.length > 0) {
			ModelValidationUtil.showValidationError(errors[0]);
		} else {
			var repository= creationForm.getRepository();
			var changeSets = [];
			if (repository.patch) {
				changeSets.push(repository.patch);
			}
			changeSets.push(repository.stack);
			
			dojo.mixin(self.onSelectArgs, {
				target: creationForm.getTarget(), 
				namespaces: creationForm.getNamespaces(), 
				types: creationForm.getTypes(),
				item: changeSets
			});
			self.onSelectTarget[self.onSelectFunctionName](self.onSelectArgs);
			this.selectDialog.close();
		}
    },
    
    onCancel: function() {
    	this.selectDialog.close();
    	if (this.onCancelFunctionName) {
        	this.onSelectTarget[this.onCancelFunctionName]();
    	}
    },
    
    processFromJSON: function( /*string*/ data ) {
        var result = rmpw.common.vcs.ref.fromJson(data, { idAttribute: "rdf_about" }),
        rdfType = result[0].rdf_type,
        i;
            
        //remove "rdf_type: Document", and all "defines" for "owl_oneOf" relationships
        //TODO: encapsulate the nuances of "owl_oneOf" into its own class (ListSectionIndividual.js?)
        if(this.featureId === "owl_oneOf") { 
          for(i = rdfType.length - 1; i >= 0; i -= 1){ 
            if(rdfType[i].rdf_about === "http://jazz.net/ns/dm/core#Document"){
                rdfType.splice(i,1);
            }
          }
          delete result[0].dmcodegen_defines;
        }
        
        return result;
    },
    	
	getTypeObject : function (object, labelMustExist) {
		if(dojo.isString(object)){
			return {"label" : object, "rdfs_label" : object,"rdf_about" : object};
		}
		//Assume that uri is always defined, as it is a positive UI failure to show
		//undefined lablel as a way to indicate issues.
		var uri= object.value || object.uriImage || object.rdf_about || "";		//Required to prevent id NPEs one fetching items
		var label= object.rdfs_label|| "";
		if (uri.length > 0) {
			if (label.length > 0 && label != uri) {
				label += ": " + uri;
			}
			else {
				// check if label must exist and if so return null if no label - we don't want users
				// to select anonymous class as an example.  They should always be created inline.
				if (labelMustExist && ModelValidationUtil.isUri(uri)) {
					return null;
				}
				
				label = uri;
			}
		}
		
		if(uri == "http://new") {	
			uri = rmpw.common.util.IdentifierUtil.guid();
		}
		
		newObj = dojo.mixin({}, object);
		newObj = dojo.mixin(newObj, {"rdfs_label" : object.rdfs_label|| "",
				"rdf_about"	 : uri
			});
		
		return newObj;
	}
	
});

})();