/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2011, 2013. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("rmpw.forms.ui.TableChangesMixin");

dojo.require("jazz.ui.ResourceLink");
dojo.require("rmps.ui.CollapsibleSection");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("rmps.ui.ActionsTable");
dojo.require("rmpw.forms.ui.EditOrCreateResource");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("rmps.app.core.util");
dojo.require("rmpw.common.util.IdentifierUtil");
dojo.require("rmpw.forms.ui.ModelValidationUtil");
dojo.require("dojox.json.ref");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmpw.forms.ui.SelectOrCreateResource");

var DesignUtil= rmps.app.ui.internal.pages.design.DesignUtil;
var ResourceInfoClient= rmps.core.resource.ResourceInfoClient;
var ModelValidationUtil= rmpw.forms.ui.ModelValidationUtil;
var InfoClient = rmps.core.resource.ResourceInfoClient;

var DEFAULT_DIALOG_WIDTH= "700px";

/**
 * Designed as a mixin object for table creations (in ListSection, and ReferencesActionTable, etc)
 */
dojo.declare("rmpw.forms.ui.TableChangesMixin", dijit._Widget, {
	
	//mixin
	form: null,
	featureName: null,
	featureId: null,
	typeUri: null,
	propertyUri: null,
	propertyDomainUri: null,
	isFragment: null,
	typeLabel: null,
	typeForm: null,
	dataTypeProperty: null,
	redirectName: null,
	uniqueNum: null,
	readOnly: null,
	section: null,
	isDocument: null,
	onSubmitListener: null,
	table: null,
	tableChangesMixinMessages: null,
	resolvedLinks: null,
	
	//private
	_updateHrefs: null,
	_connectAfterCommit: null,
	
	constructor: function(args) {
		this.featureName = args.featureName;
		this.typeUri = args.typeUri;
		this.propertyUri = args.propertyUri;
		this.propertyDomainUri = args.propertyDomainUri;
		this.typeLabel = args.typeLabel;
		this.typeForm = args.typeForm; // the form to create new resource of this type
		this.redirectName = args.redirectName;
		this.isFragment = args.isFragment;
		this.dataTypeProperty = this.typeForm == "CreateDataTypeProperty";
		this.uniqueNum = 1;
		this.isDocument= args.isDocument;
		this._updateHrefs= {};		
		this.readOnly= args.readOnly;
		this.resolvedLinks = [];
	},
	
	destroy: function() {
		this.inherited(arguments);
		
		dojo.disconnect(this.__connectApplyChange);
		delete this.__connectApplyChange;
	},
	
	postMixInProperties: function() {
		if (this.definableTypes && dojo.isString(this.definableTypes)) {
			this.definableTypes = dojo.eval(this.definableTypes);
		}
		
		dojo.requireLocalization("rmpw.forms.ui", "TableChangesMixinMessages");
		this.tableChangesMixinMessages = dojo.i18n.getLocalization("rmpw.forms.ui", "TableChangesMixinMessages");
		this.inherited(arguments);
	},
	
	buildRendering: function(){
		this.inherited(arguments);
		dojo.addClass(this.domNode, "rmpw-dsv-editor-form-ui-actions-table");
	},
	
	_createTableContents: function(){
		var args = this.setTableArguments();
		
		/*Make sure to clear previous table contents */
		if(this.table&& !this.table._beingDestroyed){
			this.table.destroyRecursive();
		}
		dojo.empty(this.tableContainer);
		
		this.table= new rmps.ui.ActionsTable(args);
		this.tableContainer.appendChild(this.table.domNode);
		
		this.table.fillTable(this.getData());
		
		//Required to set the initial model
		this.form.createChange(this.featureId, this.object);
		
		if (this.isAutoSet) {
			this.onAutoSet();
		}
		
		if (this.dependentProperties) {
			var depPropArray = dojox.json.ref.fromJson(this.dependentProperties);
			if (depPropArray && depPropArray.length > 0) {
				this.__connectApplyChange = dojo.connect(this.form, "applyChange", dojo.hitch(this, function() {
					for (var i=0, lg=depPropArray.length; i<lg; i++) {
						if (!this.__clearingContents && this.form.changeSet.isChanged(depPropArray[i])) {
							this.__clearingContents = true;
							this._clearContents();
							delete this.__clearingContents;
							break;
						}
					}
				}));
			}
		}
		 
		this.onFillTable();
		this.table.startup();
	},
	
	_clearContents: function() {
		// reset the control contents
		ModelValidationUtil.showValidationError(this.tableChangesMixinMessages.propertyScopeChange);
			
		this.form.applyChange(dojo.hitch(this, function(change){
			change.set(this.featureId, undefined);	
			change.cleanNewReferences(this.featureId);
		}));
	    	
		this.table.clearBody();
	    	
	    this.form._resource.page.setReload();
	},
	
	setTableArguments: function(){
		var doShowHeaders = this.shouldShowHeaders();
		return {
			showHeaders: doShowHeaders,
			trim: jazz.ui.StyledTable.TrimStyles.SECONDARY_HEADLESS,
			columns: this.getColumnDescriptors(),
			actionDescriptors: this.readOnly?null:this.getActionDescriptors({showHeaders : doShowHeaders})
		};
	},
	
	shouldShowHeaders: function() {
		return false;
	},
	
	getData: function(){
		return [];
	},
	
	setValue: function(object, document, repository, feature) {
		this.object= object;
		this.document = document;
		this.repository = repository;
		this.featureId= feature;
		this._createTableContents();
	},
		
	getColumnDescriptors: function(){
		return [
			{label: this.tableChangesMixinMessages.nameHeader, fieldName: "name", creator: dojo.hitch(this, this.createNameCell),width: "100%"}
		]
	},
	
	getActionDescriptors: function(){
		return [];
	},
	
	getLabel: function(object, guess) {
		if (dojo.isString(object)) {
			return object;
		}
		
		if (object.rdfs_label && dojo.isString(object.rdfs_label)) {
			return object.rdfs_label;
		}
		
		if (object.dcterms_title && dojo.isString(object.dcterms_title)) {
			return object.dcterms_title;
		}
		
		// otherwise - send the $ref back
		if (object.$ref) {
			return object.$ref;
		}
		
		if (object instanceof Date){
			return this.table.formatDateObject(object);
		}
		
		if (guess) {
			// didn't find rdfs_label or dcterms_title - so make best guess
			for (var prop in object) {
				if(prop.substr(0, 2) != '__' && prop != "rdf_about") {  // ignore internal properties
					if (object[prop] && dojo.isString(object[prop])) {
						return object[prop];
					}
				}
			}
		}
		
		return "";
	},
	
	expandAnonymousNameLabel: function() {
		return true;
	},
					
	createNameCell: function(labelCell, tableRow, object, descriptor){
		var about,label;
		
		if(object === undefined || object === null){
			about = object;
			label = this.getEmptyLabel();
		}else if(dojo.isObject(object)){
			if (this.redirectName && object[this.redirectName]) {
				if (dojo.isArray(object[this.redirectName])) {
					// in cases of lack of "canDefine" property for fragments (non-dtk resources only) cardinality restriction 
					// will not be picked up in the metamodel, and we end up with a collection
					about = object[this.redirectName][0].rdf_about;
					label = this.getLabel(object[this.redirectName][0], false);
				} else {
					about = object[this.redirectName].rdf_about;
					label = this.getLabel(object[this.redirectName], false);
				}
			}else {
				about = object.rdf_about;
				label = this.getLabel(object, false);
				if (label && dojo.isIE) {
					label = this._addBreaks(label,25);
				}	
				
				/*Required for the restriction, annomous classes cases, etc */
				if(!label){
					var isNotUrl= !new dojo._Url(about).authority;
					
					var rdf_type = object.rdf_type;
					
					if(rdf_type){
						label= "<";
						if(!dojo.isArray(rdf_type)){
							rdf_type= [rdf_type];
						}
						
						var typesLength= rdf_type.length;
						var properties= [];
					
						var currentTypeAbout;
						for(var typeIndex = 0, type; type = rdf_type[typeIndex]; typeIndex++) {
							currentTypeAbout= type.rdf_about;
							if(!currentTypeAbout){
								continue;
							}
							var typeLabel= this.getLabel(type, true);
							if(typeLabel&& dojo.indexOf(type, typeLabel)<0){
								properties.push(typeLabel);
							}
						}
						
						//If about is undefined, then it is a new resource to be created, so avoid having an about value for it
						//Assing the value of the type about if the current about defined value is not a url
						if(currentTypeAbout&& !about&& isNotUrl){
							about= currentTypeAbout;
						}
						
						if(properties.length<1){
							properties.push(this.typeLabel);
						}
						if(this.isDocument){
							properties.push(this.tableChangesMixinMessages.documentLabel);
						}
						
						var propertiesLength= properties.length;
						for(var propertyIndex = 0, propertyLabel; propertyLabel = properties[propertyIndex]; propertyIndex++) {
							label += propertyLabel;	
							if(propertyIndex< propertiesLength-1){
								label += " , ";	
							}	
						}	
						
						label+= "> ";
						
						if (this.expandAnonymousNameLabel()) {
							// add extra information about sub properties if they exist
							var props = [];
							for (var prop in object) {
								if(prop.substr(0, 2) != '__' && prop != "rdf_about") { 
									props.push(prop);
								}
							}
							
							// sort properties so they appear in consistent order
							props.sort(function(a, b) {
								return (a > b) ? 1 : ((a < b) ? -1 : 0);
							});
							
							var count = 0;
							for (var i=0, lg=props.length; i<lg; i++) {
								var prop = props[i];
								var objLabel = this.getLabel(object[prop], true);
								if (objLabel) {
									if (count > 0) {
										label += ", ";
									}
									label += prop + ": " + objLabel;
									count++;
									if (count > 3) {
										break; // only display first 3 property values
									}
								}
							}
						}
					}
				}
			}

		}else if( this.isDateString(object)) { 
			// /\d+-\d+-\d+T\d+:\d+(:\d+)?Z/  matches the ISO Date String format
			var dateObject = new Date(object);
			about = this.table.formatDateObject(dateObject);
			label = this.table.formatDateObject(dateObject);
		}else{
			about = object;
			label = object;
		}
		
		if (this.dataTypeProperty){
			this.handleCellUpdate(labelCell, dojo.doc.createTextNode(label?label: about), descriptor);				
			return labelCell;
		}else{
			//If about is undefined, this means the field is not set at all, so do not bother listening to its change
			if(!about && (!object || !object.$ref)){
				this.handleCellUpdate(labelCell, dojo.doc.createTextNode(label||about|| this.getEmptyLabel()), descriptor);
				return;
			} 
			
			if (!about) {
				about = object.$ref;
			}
			if (!new dojo._Url(about).scheme || this._updateHrefs[about]) {				
				if(!dojo.isObject(object)){
					//Make sure to avoid updating the url if the object is not an instance of object. What does this mean? It means that
					//It is literal value (string, int, etc)
					this.handleCellUpdate(labelCell, dojo.doc.createTextNode(label), descriptor);
					return labelCell;
				}
				// for new resources always add them as simple text until the document is saved.  Otherwise, the link
				// is invalid since the data doesn't exist on disk
				this.handleCellUpdate(labelCell, dojo.doc.createTextNode(label?label: about), descriptor);
								
				//Updating urls for document elements has different handling
				if(!this.isDocument&& this.form._resource&& this.form._resource.documentObject){
					if (!this._updateHrefs[about]) {
						// just created new fragment, remember it by its full uri
						var fullUri = rmpw.common.util.IdentifierUtil.normalizeURI(this.form._resource.documentObject.rdf_about) + about;
						this._updateHrefs[fullUri]= {
							cell: labelCell,
							label: label,
							href: fullUri
						};
					} else {
						// just modified newly created fragment
						this._updateHrefs[about].label = label;
					}										
					
					// listen to save and convert to link
					if (!this._connectAfterCommit) {
						this._connectAfterCommit = this.connect(this.form._resource.page.editor, "onSaveComplete", dojo.hitch(this, function() {
							this.disconnect(this._connectAfterCommit);
							delete this._connectAfterCommit;
							
							for(var att in this._updateHrefs){
								var current= this._updateHrefs[att];
								var cell= current.cell;
								dojo.empty(cell);
								this._createLinkNameCell(cell, current.href, current.label, descriptor);
								delete this._updateHrefs[att];
							}
						}));
					}					
				}
			}
			else {
				this._createLinkNameCell(labelCell, about, label, descriptor);
			}
		}
        return labelCell;
	},
	
	handleCellUpdate: function(labelCell, newNode, descriptor) {
		if (descriptor && descriptor.actionDescriptors && labelCell.childNodes.length > 0) {
			// need to preserve toolbar part of the cell
			labelCell.replaceChild(newNode, labelCell.childNodes[0]);
		} else {
			labelCell.appendChild(newNode);
		}
	},
	
	getEmptyLabel: function() {
		return this.tableChangesMixinMessages.emptyLabel;
	},
	
	_addBreaks: function(str, charsPerRow) {
		var s;
		var inx = 0;
		var lines = [];
		var len = str.length;
		while (inx < len) {			
			// should we wrap?
			if ((inx+charsPerRow) < len) {
				s = str.substr(inx, charsPerRow);
				// break on a space if we can
		        var i = s.lastIndexOf("/");
		        if (i > 0) {
                    s = s.substring(0, i+1);
		        } else {
                    s = s.substring(0, charsPerRow);
		        }
				inx += s.length;
		        lines.push(s);
			} else {
                s = str.substr(inx);
                lines.push(s);
                break;
            }
        }	
		return lines.join('\n');
	},
	
	_createLinkNameCell: function(labelCell, about, label, descriptor){
		var resourceLabel= label||about|| this.getEmptyLabel();
		var resourceLink = new jazz.ui.ResourceLink({
			uri: this.getUri(about, resourceLabel),
			presentationUri: this.getPresentationUri(about, resourceLabel),
			lazyFetch : true,
			label: jazz.util.html.escape(resourceLabel),
		});
		
		this.connect(this, "uninitialize", function() {
			resourceLink.destroyRecursive();
		});
		
		this.handleCellUpdate(labelCell, resourceLink.domNode, descriptor);
		resourceLink.startup();
	
		/*
		 *  Update the link on hover, will try to prevent a full-page refresh for links within the same project area.
		 *  There is a race condition with this solution however, where the user might click too fast while the resolution
		 *  is in flight.  For this case, the behaviour will be the same as it was before; a full-page refresh.
		 */
		dojo.connect(resourceLink.domNode, "onmouseover", dojo.hitch(this, function(){
			if(undefined === this.resolvedLinks[about] || null == this.resolvedLinks[about]){
				this.resolvedLinks[about] = InfoClient._getResourceInfos(about, "resourceContext", null, null);
			}
			
			// Wait for a resolved value.
			dojo.when(this.resolvedLinks[about], dojo.hitch(this, function(info){
				var resolvedProjectUri = info[0].resourceContext;
				if(resolvedProjectUri && resolvedProjectUri.indexOf("dm/process/project-areas/") >= 0){
					var projectAreaUuid = resolvedProjectUri.split("dm/process/project-areas/")[1];
					if(projectAreaUuid && projectAreaUuid === jazz.app.context.get().itemId){
						var newUri = "#action=rmps.app.design.edit&resource=" + encodeURIComponent(about);
						var newResourceLink = new jazz.ui.ResourceLink({
							uri: resourceLink.uri,
							presentationUri: newUri,
							lazyFetch : resourceLink.lazyFetch,
							label: resourceLink.label
						});
						
						// Make sure we destroy the old link.
						this.connect(this, "uninitialize", function() {
							newResourceLink.destroyRecursive();
						});
						
						// Replace and delete the old node.
						dojo.place(newResourceLink.domNode, resourceLink.domNode, "replace");
						resourceLink.destroyRecursive();
					}
				}
			}));
		}));
		
		return labelCell;
	},
	
	getUri: function(about, resourceLabel){
		return this._buildBaseUri("rendering", about);
	},
	
	getPresentationUri: function(about, resourceLabel){
		return this._buildBaseUri("logical", about);
	},
	
	_buildBaseUri: function(segment, resource){
		var baseUri = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + '/' + segment + '?' + rmps.core.util.IRmpsConstants.BASELINE + "=" + encodeURIComponent(jazz.app.context.get().baseline);
		if(jazz.app.context.get().changeset){
			baseUri += '&' + rmps.core.util.IRmpsConstants.CHANGESET + "=" + encodeURIComponent(jazz.app.context.get().changeset);
		}
		var resourceParam = (segment === "rendering")?"&resourceUri=":"&resource=";		
		var normalized= this._getNormalizedAboutValue(resource);
		baseUri+= normalized?resourceParam+encodeURIComponent(normalized):resourceParam+encodeURIComponent(resource);
		return baseUri;
	},
	
	_getNormalizedAboutValue: function(about){
		if(!about){
			return null;
		}
		if(about.indexOf(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL)){
			return about;
		}
		var query= new dojo._Url(about).query;
		
		if(query){	//Safe check
			var object= dojo.queryToObject(new dojo._Url(about).query);
			for(var att in object){
				return about;
			}
		}
		return null;
	},
	
	/*
	 * @param {null} object
	 */
	addNewItem: function(object) {
		var row= this.table.addRow(object);
		this.form.resize({});
		return row;
	},
	
	getSelectDialogArgs: function(args) {
		return {
			typeUri: this.typeUri,
			propertyUri: this.propertyUri,
			propertyDomainUri: this.propertyDomainUri,
			typeLabel:this.typeLabel,
			typeForm: this.typeForm, // the form to create new resource of this type
			isFragment: this.isFragment,
			isDocument: this.isDocument,
			definableTypes: this.definableTypes,
			documentObject: this.form._resource.documentObject,
			resourceUri: (this.form && this.form._resource.id) ? this.form._resource.id : (this.form ? this.form._resource.documentObject.rdf_about : null),	
			isConstrained: this.isConstrained,
			defaultValue: this.getFocusReference(),
			isReadOnly: this.form._resource.page.isReadOnly,
			editor: this.isDocument ? null : this.form._resource.editor,
			namespaces: this.form._resource.namespaces,
			page: this.form._resource.page,
			formObject: this.form._resource.formObject,
			canSelect: this.canSelect,
			form: this.form,
			enableMultiSelect: this.allowMultiResourceSelect(),
			
			onSelectTarget: this,
			onSelectFunctionName: "onSetAllReferences",
			onSelectArgs: args
		};
	},
	
	useSelectDialog: function(args){
		var selectOrCreateResource = new rmpw.forms.ui.SelectOrCreateResource(this.getSelectDialogArgs(args));
		
		selectOrCreateResource.useSelectDialog();
	},
	
	removeRefrence: function(dataEntry, currentRow, currentActionsCell, leaveRow) {
		this.form.applyChange(dojo.hitch(this, function(change){
		    var index = dojo.indexOf(currentRow.parentNode.rows, currentRow);
			change[this.featureId==="dmcodegen_defines"?"removeArrayAt":"removeReferenceArrayAt"](this.featureId, index, dataEntry, true);
			this.onRemoveReference(dataEntry, currentRow, currentActionsCell, change, leaveRow);		
		}));
		this.form.resize({});
	},
	
	allowMultiResourceSelect: function() {
		// false by default, to be overriden when needed
		return false;
	},
	
	getCollection: function() {
		return dojo.map(this.table._rowCache._rowsInformation, function(entry) {
			return entry.getDataEntry()});
	},

	newReference: function(currentTarget, tr, td, extraInfo, docTarget) {
		// this function is used for creating and editing fragments;
		// extraInfo and docTarget are given when editing a fragment from the table cell
		var dialog = new rmpw.forms.ui.EditOrCreateResource();
		if (currentTarget) {
			var currentAbout = currentTarget.rdf_about;
			var docUri = docTarget ? docTarget.rdf_about : this.form._resource.documentObject.rdf_about;
			if (currentAbout && (docUri !== "http://new") && currentAbout.indexOf(docUri) === -1) {
				// editing newly created fragment, fix the identifier so that it's fully qualified with the document URI
				// (since in case of multi-doc updates, we will need to know which document the updated new fragment belongs to) 
				currentTarget.rdf_about = rmpw.common.util.IdentifierUtil.normalizeURI(docUri) + currentAbout;
			}
		}
		var dialogArgs = {};
		dialogArgs.target = currentTarget;
		dialogArgs.isFragment = true;
		dialog.editOrCreateResource(this.getEditOrCreateDialogArgs(dialogArgs));
		
		var onSubmitListener = this.connect(dialog, "onSubmit", function(target, types, namespaces) {
			this.uniqueNum++;
			this.disconnect(onSubmitListener);
			if(currentTarget && target && tr){
				// edit scenario - changes have already been committed
				this.updateTableRow(tr, target, td, extraInfo);
			}else{
				if(this.dataTypeProperty){
					this.addNewItem(target[this.featureId]);
					this._addNewReference(target[this.featureId], namespaces);
				} else {
					if(!target.rdf_about) {
						var defaultLabel = this.typeLabel.replace(/ /g, "") + this.uniqueNum;
						var defaultUri = defaultLabel + rmpw.common.util.IdentifierUtil.guid();						
						target.rdf_about = defaultUri;
					}	
					this.addNewItem(target);
					this._addNewReference(target, namespaces);
				}
			}
			this.form.resize({});
			this.uniqueNum++;
		});
		var onDialogCloseListener = this.connect(dialog, "onClose", function() {
			this.disconnect(onSubmitListener);
			this.disconnect(onDialogCloseListener);
		});
	},
	
	editExternalDocument: function(currentTarget, tr, td, extraInfo) {
		// this function is used for editing external documents or fragments of external documents off of the row in the form's collection table
		var dialog = new rmpw.forms.ui.EditOrCreateResource();
		var updatedTarget;
		var foundTarget = false;
		if (currentTarget.rdf_about) {
			updatedTarget = this.form.repository.getUpdatedTarget(currentTarget.rdf_about);
			if (!updatedTarget || updatedTarget.___incompleteContents) {
				// load the instance data for the external document, target only contains type and label info;
				var self = this;
				foundTarget = true;
				rmps.core.client.xhrGet({
					url: rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "rdfdirectory") + "/",
					content: {
						logicalURI: currentTarget.rdf_about,
						checkContext: "true"
					},
					headers: {
						"Accept": "application/x-rdf+json-graph"
					},
					handleAs: "text",
					load: function(data, ioargs){
					
						if (data === null || data == '') {
							rmps.core.error.create("Failed to load initial model").dispatch();
							return;
						}
						var modelIndex = {};
						var documents = rmpw.common.vcs.ref.fromJson(data, {
							index: modelIndex,
							idAttribute: "rdf_about"
						});
						var model = modelIndex[currentTarget.rdf_about];
						if (updatedTarget) {
							// partial model updated before (from inside the table), now merge those changes in
							dojo.mixin(model, updatedTarget);						
						}						
						// find out if there is a committed change(s) for any of the fragments of this model, 
						// and if so transfer those changes to the model so they are available in case user opens popus for those fragments
						var dmcodegen_defines = model.dmcodegen_defines;
						if (dmcodegen_defines) {
							for (var i = 0, lg = dmcodegen_defines.length; i < lg; i++) {
								dmcodegen_defines[i] = self.form.repository.tryCommitToTarget(dmcodegen_defines[i]);
							}
						}
						// when opening an external resources that has a collection of references to other external resources,
						// those collections may be in form of table, and there may be some updates to that data - check now
						for (var property in model) {
							if (dojo.isArray(model[property]) && model[property].rdf_about != model.rdf_about) {
								for (var i = 0, extResource; extResource = model[property][i]; i++) {
									var updatedModel = self.form.repository.getUpdatedTarget(extResource.rdf_about);
									if (updatedModel) {
										model[property][i] = updatedModel;
									}									
								}
							}
						}
						var eTag = ioargs.xhr.getResponseHeader("ETag");
						var dialogArgs = {};
						dialogArgs.target = model;
						dialogArgs.readOnly = ioargs.xhr.getResponseHeader("isReadOnly") == "true";
						dialogArgs.eTag = eTag;
						dialogArgs.editExternalDoc = true;
						dialogArgs.extraInfo = extraInfo;
						dialog.editOrCreateResource(self.getEditOrCreateDialogArgs(dialogArgs));
					}
				});
			}
		} else {
			// reference to new resource, can't load, use the data we have
			updatedTarget = currentTarget; 
		}
		if (!foundTarget) {
			var dialogArgs = {};
			dialogArgs.target = updatedTarget;	/*isReadOnly has to be true as there are edits*/	
			dialogArgs.extraInfo = extraInfo;
			dialogArgs.editExternalDoc = true;	
			dialog.editOrCreateResource(this.getEditOrCreateDialogArgs(dialogArgs));
		}
		var onSubmitListener = this.connect(dialog, "onSubmit", function(target) {
			if (target && tr) {
				// changes have already been committed
				this.updateTableRow(tr, target, td, extraInfo);
				target.___incompleteContents = false;
				this.referencedResourceChanged(target, tr);
			}
		});
		var onDialogCloseListener = this.connect(dialog, "onClose", function() {
			this.disconnect(onSubmitListener);
			this.disconnect(onDialogCloseListener);
		});
	},
	
	referencedResourceChanged: function(model, tr) {
		// do nothing by default
	},
	
	getEditOrCreateDialogArgs: function(args) {
		var newArgs = {
			typeName: this.typeLabel, 
			typeForm: this.typeForm, 
			defaultUri: null, 
			defaultLabel: "", 
			parent: this, 
			featureId: this.featureId, 
			target: null, 
			isFragment: false, 
			isReadOnly: args.readOnly != undefined ? args.readOnly : false,
			editExternalDoc: false,
			eTag: null
		}
		if (args) {
			dojo.mixin(newArgs, args);
		}
		return newArgs;
	},
	
	updateTableRow: function(tr, target) {
		// just update the name cell
		this.table.updateRowCell(tr, "name", target);
	},
	
	_addNewReference: function(item, namespaces, types, target, row){
		if(!item){
			return;
		}
		this.setRowReference(item, namespaces, row, types, target);
	},
	
	setRowReference: function(target, namespaces, row, types, item){
		if(this.isDocument){
			this._createNewReference(target, namespaces, row, types, item);
			return;
		}
		this._onAddNewReference(target, row, types, item);
	},
	
	_createNewReference: function(target, namespaces, row, types, changes){
		if(!target){
			this._onAddNewReference(null, row);
			return;
		}
		if(new dojo._Url(target.rdf_about).scheme){	//Means the resource is already created, so just the push the reference
			this._onAddNewReference(target, row);
			return;
		}
	},
	
	/**
	 * Can edit item given by rdf_about if it is one of:
	 * 1) a datatype property
	 * 2) a reference to an external document 
	 * 3) internal fragment unless it is a brand new fragment created from a popup (as in that case its creation can be cancelled)
	 */
	canEditItem: function(rdf_about) {
		if (this.dataTypeProperty) {
			return true;
		}
		if (!rdf_about) {
			return false;
		}
		var isInternalFragment = rmpw.common.util.IdentifierUtil.inDocumentIdentifierCheck(this, null, rdf_about);
		var isNewIternalFragment = !isInternalFragment ? false : (rmpw.common.util.IdentifierUtil.getIdentifier(rdf_about) === rdf_about);
		return rdf_about && (this.isDocument || 
				(isInternalFragment && (!isNewIternalFragment || !this.form.isInPopup)));
	},
	
	getTargetRdfAbout: function(logicalUri, target) {
		// returns the value to be stored for the new reference; by default, it is uri of the newly created resource
		return logicalUri;
	},
	
	handleMoveUp: function(dataEntry, currentRow, currentActionsCell){
		if(this.table.moveRowUp(currentRow)){
			return;
		}
		var index= this.table.getRowIndex(currentRow);
		this.form.applyChange(dojo.hitch(this, function(change){
			change.arrayMoveElement(this.featureId, index, index+1);
			this.onMoveUp(dataEntry, currentRow, currentActionsCell,change);
		}));
	},
	
	handleMoveDown: function(dataEntry, currentRow, currentActionsCell){
		if(!this.table.moveRowDown(currentRow)){
			return;
		}
		var index= this.table.getRowIndex(currentRow);
		this.form.applyChange(dojo.hitch(this, function(change){
			change.arrayMoveElement(this.featureId, index, index-1);
			this.onMoveDown(dataEntry, currentRow, currentActionsCell,change);
		}));
	},
	
	_onAddNewReference: function(target, node, types, item, immediate){
		this.form.applyChange(dojo.hitch(this, function(change){
			this.addReference(change, target);
		}), immediate);
		if(this.form._resource&& this.form._resource.modelIndex){
			this.form._resource.modelIndex[rmpw.common.util.IdentifierUtil.normalizeURI(this.form._resource.documentObject.rdf_about) + target.rdf_about] = target;			
		}
		
		// if it's a new object - add to the defines collection
		if (this.featureId != "dmcodegen_defines" && target && !rmpw.common.util.IdentifierUtil.isIdentifierFixed(target.rdf_about) && !this.isDocument ) {
			var rootObj = rmpw.common.util.IdentifierUtil.getRootObject(this);
			if (rootObj && rootObj.dmcodegen_defines && !rmpw.common.util.IdentifierUtil.exists(rootObj.dmcodegen_defines, target.rdf_about)) {
				rootObj.dmcodegen_defines.push(target);
			}
		}
		
		this.onAddNewReference(target, node, types, item);
	},
	
	addReference: function(change, target){
		if(this.isDocument){
			change.pushReference(this.featureId||"dmcodegen_defines", target);				
		}else{
			change.push(this.featureId||"dmcodegen_defines", target);	
		}
	},
	
	getTypeObject : function (object, labelMustExist) {
		if(dojo.isString(object)){
			return {"rdfs_label" : object,"rdf_about" : object};
		}
		//Assume that uri is always defined, as it is a positive UI failure to show
		//undefined lablel as a way to indicate issues.
		var uri= object.value || object.uriImage || object.rdf_about || "";		//Required to prevent id NPEs one fetching items
		var label= object.rdfs_label|| "";
		if (uri.length > 0) {
			if (label.length > 0 && label != uri) {
				label += ": " + uri;
			}
			else {
				// check if label must exist and if so return null if no label - we don't want users
				// to select anonymous class as an example.  They should always be created inline.
				if (labelMustExist && ModelValidationUtil.isUri(uri)) {
					return null;
				}
				
				label = uri;
			}
		}
		
		if(uri == "http://new") {	
			uri = this.uniqueNum + rmpw.common.util.IdentifierUtil.guid();
		}
		
		newObj = dojo.mixin({}, object);
		newObj = dojo.mixin(newObj, {"rdfs_label" : object.rdfs_label|| "",
				"rdf_about"	 : uri
			});
		
		return newObj;
	},
	
	isDateString: function(object) {
		// /\d+-\d+-\d+T\d+:\d+(:\d+)?Z/  matches the ISO Date String format
		return dojo.isString(object) && object.match(/\d+-\d+-\d+T\d+:\d+(:\d+)?Z/);
	},
	
		
	/** 
	* args: {
	*				target: target, 
	*				namespaces: namespaces, 
	*				row: row, 
	*				types: types, 
	*				item: item
	*			} 
	*/
	onSetReference: function(args){
		/*To be inherited by sub classes */
	},
	
	onSetAllReferences: function(args){
		if (dojo.isArray(args)) {
			for (var i = 0, arg; arg = args[i]; i++) {
				this.onSetReference(arg);
			}
		} else {
			this.onSetReference(args);
		}
	},
	
	getFocusReference: function(item, namespaces){
		/*To be inherited by sub classes */
	},
	
	onRemoveReference: function(dataEntry, currentRow, currentActionsCell, change, leaveRow){
		/*To be inherited by sub classes */
	},
	
	onMoveUp: function(dataEntry, currentRow, currentActionsCell,change){
		/*To be inherited by sub classes */
	},
	
	onMoveDown: function(dataEntry, currentRow, currentActionsCell,change){
		/*To be inherited by sub classes */
	},
	
	onFillTable: function(){
		/*To be inherited by sub classes */
	},
	
	onAddNewReference: function(target, node, types, item){
		/*To be inherited by sub classes */
	},
	
	checkReload: function() {
		if (this.form._resource.page && this.isReload && this.form._resource.page.setReload) {
			this.form._resource.page.setReload();
		}
	},
	
	onAutoSet: function(){
		/*To be inherited by sub classes */
	},
});

})();