/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2013. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function(){
dojo.provide("rmpw.forms.ui.TableSection");

dojo.require("rmpw.forms.ui.ListSection");
dojo.require("rmpw.common.vcs.ref");
dojo.require("rmpw.common.util.ChangeUtil");
dojo.require("dojo.i18n");

/**
 * Customizes List Section for features that should be represented by the table widget.
 * 
 * @author lgrahek
 */
dojo.declare("rmpw.forms.ui.TableSection", [rmpw.forms.ui.ListSection, dijit._Templated], {
 
 	// tableColumnDescr is in the form of [{featureName : "feature name", columnHeaderName : "name"}]
 	tableColumnDescr : [],
	
 	constructor: function(args) {
		this.inherited(arguments);
		this.tableColumnDescr = rmpw.common.vcs.ref.fromJson(args.columnDescr);
		
		dojo.requireLocalization("rmpw.forms.ui", "ListSectionMessages");
		this.messages = dojo.i18n.getLocalization("rmpw.forms.ui", "ListSectionMessages");
		
		dojo.requireLocalization("rmpw.forms.ui", "ReferencesActionsTableMessages");
		this.referencesMessages = dojo.i18n.getLocalization("rmpw.forms.ui", "ReferencesActionsTableMessages");		
	},
	
	shouldShowHeaders: function() {
		return true;
	},
	
	expandAnonymousNameLabel: function() {
		return false;
	},
	
	getColumnDescriptors: function(){
		var descr = [
			{label: this.tableChangesMixinMessages.nameHeader, fieldName: "name", creator: dojo.hitch(this, this.createNameCell)}
		]
		var columnDescr;
		for (var i = 0, columnDescrInfo; columnDescrInfo = this.tableColumnDescr[i]; i++) {
			var actionDescr = null;
			if (!this.readOnly && !columnDescrInfo.isReadOnly &&
					(columnDescrInfo.isDocument || columnDescrInfo.isInternalFragment || // no editing for fragments external to the document in table row
					(columnDescrInfo.typeForm === "CreateDataTypeProperty" && columnDescrInfo.typeLabel != "Rich Text" ))) { // no editing for rich text content
				actionDescr = this.getCellActionDescriptors();
			}
			columnDescr = {
				label: columnDescrInfo.columnHeaderName, 
				fieldName: columnDescrInfo.featureName, 
				creator: dojo.hitch(this, this.createNameCell),
				actionDescriptors: actionDescr 
			};
			dojo.mixin(columnDescr, columnDescrInfo);
			descr.push(columnDescr);
		}
		return descr;
	},
	
	createNameCell: function(labelCell, tableRow, object, descriptor){
		// object is data about the resource referenced in the row;
		// extract the value from object based on the info in the descriptor about the column property
		if (descriptor.fieldName && descriptor.fieldName != "name") {
			var value = object[descriptor.fieldName];
			if (!value) {
				value = descriptor.typeUri === "http://www.w3.org/2001/XMLSchema#boolean" ? "false" : ""; 
			}
			if (descriptor.typeForm === "CreateDataTypeProperty") {
				if (descriptor.typeLabel === "String" || descriptor.typeLabel === "Rich Text") {					
					// show only first maxLength characters
					var cell = labelCell.childNodes.length > 1 ? labelCell.childNodes[0] : labelCell;
					if (descriptor.typeLabel === "Rich Text") {
						// strip off all html tags (there will be some in case of rich text content)
						cell.innerHTML = value;
						value = cell.textContent;
						cell.innerHTML = "";
					}
					var maxLength = 160;
					if (value.length > maxLength) {
						while (maxLength > 0 && value.charAt(maxLength-1) != ' ') {
							// don't cut off in the middle of the word, go backwords until first space
							maxLength--;
						}
						if (maxLength === 0) {
							// this is a long text with no spaces, just cut it off
							maxLength = 150;
						}
						value = dojo.string.substitute(this.messages.more, [value.substring(0, maxLength-1)]);
					}						
				} else if (value instanceof Date) {
					value = this.table.formatDateObject(value);
				} else if (this.isDateString(value)) { 
					var dateObject = new Date(value);
					value = this.table.formatDateObject(dateObject);
				}	
				this.handleCellUpdate(labelCell, dojo.doc.createTextNode(value), descriptor);				
			} else {
				this.inherited(arguments, [labelCell, tableRow, value, descriptor]);
			}
		} else {
			this.inherited(arguments);
		}
	},
	
	getEmptyLabel: function() {
		return "";
	},
	
	onSetAllReferences: function(args){
		// (called in case of external documents - a reference to newly created one are added, or reference(s) to
		// selected existing ones are added, or new fragment is created)
		// need to fill in columns based on the selected/created resource(s)
		if (args.dataAvailable) {
			// a new document has just been created, values needed by columns are already in the target
			this.inherited(arguments);
		} else {
			if (!dojo.isArray(args)) {
				// new fragment (ends up here in case when isFragment is set to false by code generator, 
				// which is in case when fragment collection is not implicitly created, but explicitly through a property)
				this.onSetReference(args);
				return;
			}
			// one or more existing resources are selected, have to request the values for columns from the server
			var self = this;			
			for (var i = 0, arg; arg = args[i]; i++) {
				// gather rdf_about of all documents whose column properties are to be retrieved
				// TODO send just one request to the server to retrieve all models - need to change the back end for that		
				rmps.core.client.xhrGet({					
					url: rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "rdfdirectory") + "/",
					content: {
						logicalURI: arg.target.rdf_about,
						checkContext: "true",
						rawData: "true"
					},
					headers: {
						"Accept": "application/x-rdf+json-graph"
					},
					handleAs: "text",
					load: function(data, ioargs, arg){
					
						if (data === null || data == '') {
							rmps.core.error.create("Failed to load initial model").dispatch();
							return;
						}
						var modelIndex = {};
						var documents = rmpw.common.vcs.ref.fromJson(data, {
							index: modelIndex,
							idAttribute: "rdf_about"
						});
						// find out the resource URI for which this data was loaded
						var logicalUri = decodeURIComponent(ioargs.url.split("logicalURI")[1].replace(/\+/g," "));
						var endIdx = logicalUri.indexOf("&");
						if (endIdx > 2) {
							logicalUri = logicalUri.substring(1, endIdx);
						} else {
							logicalUri = logicalUri.substring(1);
						} 
						var newArg, newData;
						for (var j = 0, arg; (arg = args[j]) && !newData; j++) {
							if (logicalUri === arg.target.rdf_about) {
								newData = modelIndex[logicalUri];
								newArg = arg;
							}
						}
						if (newData) {
							if (!newData.rdfs_label) {
								// just get the label from incomplete target (that only has label and type);
								// can happen in case when tehre is equivalent property to label
								newData.rdfs_label = newArg.target.rdfs_label;
							}
							newArg.target.infoTarget = newData;
							self.onSetReference(newArg);
						}
					}
				});
			}			
		}
	},

	setValue: function() {
		this.inherited(arguments);
		// hide table headers if there is no data when the form is first loaded
		var data = this.table.getRowsInformation();
		if (!data || data.length() === 0) {
			this.table.showHeader(false);
		}
	},
	
	onRemoveReference: function(dataEntry, currentRow, currentActionsCell, change) {
		this.inherited(arguments);
		// if this was the last row, hide headers
		var data = this.table.getRowsInformation();
		if (!data || data.length() === 0) {
			this.table.showHeader(false);
		}		
	},	
	
	addNewItem: function(target) {
		// adding a new row in the table;
		// make sure headers are visible (they are hidden for as long as there is no data in it)
		this.table.showHeader(true);

		if (target.infoTarget) {
			// adding a reference to an existing element in the collection, and target.infoTarget holds the info
			// (but we don't want that one to be used for creating a change)
			this.inherited(arguments, [target.infoTarget]);
			delete target.infoTarget;
		} else {
			// adding a reference to newly created resource
			this.inherited(arguments);
		}
	},
	
	updateTableRow: function(tr, target, td, descriptor) {		
		if (descriptor) {
			// (called after updating a value in the table cell)
			var newTarget = [];
			newTarget[descriptor.fieldName] = target;
			this.table.updateRowCell(tr, descriptor.fieldName, newTarget);			
		} else {
			// (called after updating an external document reference, or creating or edting a fragment)
			// update all columns with values from target
			this.table.updateRowCell(tr, "name", target);
			for (var i = 0, columnDescrInfo; columnDescrInfo = this.tableColumnDescr[i]; i++) {
				this.table.updateRowCell(tr, columnDescrInfo.featureName, target);
			}			
		}
	},
	
/************** Editing support for table cells **************/
	
	getCellActionDescriptors: function(){
		return [{   actionCallback: dojo.hitch(this, this.clearRefrenceInCell), 
					onCreateCallback: dojo.hitch(this, this.onClearCreateInCell), 
					id: "clear", 
					args: {imgClass: 'clear', title: this.referencesMessages.clear}
				},
				{	actionCallback: dojo.hitch(this, this.editRefrenceInCell), 
					args: {imgClass: 'edit', title: this.referencesMessages.edit}
				}];
	},
	
	editRefrenceInCell: function(dataEntry, currentRow, currentActionsCell, currentColumnDescriptor){
		var object = dataEntry[currentColumnDescriptor.fieldName];
		if (currentColumnDescriptor.typeForm === "CreateDataTypeProperty") {
			this.editDatatypeValueInline(dataEntry, currentRow, currentActionsCell, currentColumnDescriptor, false);
		} else if (currentColumnDescriptor.isDocument) {
			// for now, disabled editing of internal fragment references in cells
			if (object) {
				// edit external document pointed to by this cell
				this.editExternalDocument(object, currentRow, currentActionsCell, currentColumnDescriptor);
			} else {
				// select an existing or create new resource whose reference should be added to this cell
				this.useSelectDialog({
					cellInfo: {
						tr: currentRow,
						descriptor: currentColumnDescriptor,
						resourceUri: dataEntry.rdf_about
					}
				});
			}
		} else {
			// internal fragment
			if (object) {
				// edit fragment
				if (currentColumnDescriptor.isInternalFragmentOfExternalDoc) {
					// internal to the document represented by table row, but external to the main document
					this.editExternalDocument(object, currentRow, currentActionsCell, currentColumnDescriptor);
				} else {
					// this is internal fragment of the main document
					this.newReference(object, currentRow, currentActionsCell, currentColumnDescriptor, dataEntry);
				}
			} else {
				// select an existing one or create a new one
				this.useSelectDialog({
					cellInfo: {
						tr: currentRow,
						descriptor: currentColumnDescriptor,
						resourceUri: dataEntry.rdf_about
					}
				});				
			}
		}
	},
	
	inlineEditingDone_createChange: function(oldStrValue, value, currentRow, cell, currentColumnDescriptor) {		
		// find the resource that is changing (resource referenced by this row)
		// and create a change (to be committed to repository now or later)	
		var property = currentColumnDescriptor.fieldName;
		var targetOfChange = this.table.getRowInfomation(currentRow).getDataEntry();

		// create a change to update
		this._applyChange(targetOfChange, property, value);
	},
	
	allowBlankCells: function() {
		// while editing cells inline inside the table, allow the user to wipe the content of the cell
		return true;
	},
	
	clearRefrenceInCell: function(dataEntry, row, currentActionsCell, currentColumnDescriptor){
		this.table.updateRowCell(row, currentColumnDescriptor.fieldName, {});
		this._applyChange(dataEntry, currentColumnDescriptor.fieldName, undefined);
	},
	
	onClearCreateInCell: function(actionDescriptor, button, tableRow, dataEntry){
	},
	
	onSetReferenceInCell: function(args) {
		// setting a reference in the table cell (i.e. adding a reference to the resource referenced by the table row);
		// it has to be a reference to an existing element or itnernal fragment
		var arg = args[0] ? args[0] : args; // only one selection allowed
		var newReference = arg.target;
		if (! (new dojo._Url(newReference.rdf_about).scheme)) {
			// itnernal fragment, construct rdf_about based on resourceUri of the resource in table row
			newReference.rdf_about = rmpw.common.util.IdentifierUtil.normalizeURI(arg.resourceUri) + newReference.rdf_about;
		}
		// update table cell content
		var newTarget = {};
		var property = arg.descriptor.fieldName;
		newTarget[property] = newReference;
		this.table.updateRowCell(arg.tr, property, newTarget);

		// find the resource that is changing (resource referenced by this row)
		// and create a change (to be committed to repository now or later)	
		var targetOfChange = this.table.getRowInfomation(arg.tr).getDataEntry();
		this._applyChange(targetOfChange, property, newReference);
	},
	
	_applyChange: function(targetOfChange, property, newReference) {
		// creates a change (to be committed to repository now or later);
		var changeSet;
		if(this.form.isImmediate()){
			changeSet = new rmpw.common.vcs.ChangeSet();
		} else {
			changeSet = this.form.changeSet;
		}
		var change = changeSet.createChange(targetOfChange);
		change.set(property, newReference);
		if (this.form.isImmediate()) {
			this.form.getRepository().commit(changeSet);
		}
		
		// indicate that targetOfChange is not completely loaded (only data needed for table is loaded) so that 
		// the rest of it can be loaded in case the user wants to edit it in a popup (by clicking Edit on the table row)
		if (targetOfChange.___incompleteContents === undefined) {
			targetOfChange.___incompleteContents = true;
		}	
	},
		
	_updateClearReferenceEnablement: function(tableRow, disabled){
		this._onClearReferenceEnablement(this.table.getRowInfomation(tableRow? tableRow : this._row).getActionWidget("clear"), disabled);
	},
	
	referencedResourceChanged: function(model, tr) {
		// called after the resource references in row tr is updated (from popup);
		// ensure that the updated model is now tr's model, so if the user updates 
		// this resource directly in table, those updates are not lost
		this.table.getRowInfomation(tr)._dataEntry = model;
	},
	
	getSelectDialogArgs: function(args) {
		if (!args.cellInfo) {
			// adding a row in the collection
			return this.inherited(arguments);
		}
		var descr = args.cellInfo.descriptor;
		return {
			typeUri: descr.typeUri,
			propertyUri: descr.propertyUri,
			propertyDomainUri: descr.propertyDomainUri,
			typeLabel:descr.typeLabel,
			typeForm: descr.typeForm, // the form to create new resource of this type
			isDocument: descr.isDocument,
			definableTypes: descr.definableTypes,
			documentObject: this.form._resource.documentObject,
			resourceUri: args.cellInfo.resourceUri,	
			isConstrained: descr.isConstrained,
			editor: this.isDocument ? null : this.form._resource.editor,
			namespaces: this.form._resource.namespaces,
			page: this.form._resource.page,
			formObject: this.form._resource.formObject,
			canSelect: descr.canSelect,
			form: this.form,			
			defaultValue: null,			
			isReadOnly: args.readOnly != undefined ? args.readOnly : descr.isReadOnly,
			enableMultiSelect: false,
			onSelectTarget: this,
			onSelectFunctionName: "onSetReferenceInCell",			
			onSelectArgs: args.cellInfo
		}
		return selectDialogArgs;	
	},
	
	getEditOrCreateDialogArgs: function(args) {
		var descr = args.extraInfo;
		if (!descr) {
			// this is a call for viewing the resource referenced by the table row
			return this.inherited(arguments);
		}
		// this is a call for viewing the resource referenced by the cell		
		return {
			typeName: descr.typeLabel,
			typeForm: descr.typeForm,
			defaultUri: null,
			defaultLabel: "",
			parent: this,
			featureId: descr.fieldName,
			target: args.target,
			isFragment: args.isFragment,
			isReadOnly: args.readOnly != undefined ? args.readOnly : descr.isReadOnly,
			editExternalDoc: args.editExternalDoc,
			eTag: args.eTag
		}
	}
	
/**** end of Editing support for table cells ***/		
});

})();