/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.dsv.editor.visualization.ui.Toolbar");

dojo.require("dijit.Toolbar");
dojo.require("dijit._Contained");
dojo.require("dijit.form.ComboBox");
dojo.require("dijit.form.Button");
dojo.require('dojo.data.ItemFileReadStore');

/**
 * Toolbar for creating and loading render transform models.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.dsv.editor.visualization.ui.Toolbar", [dijit.Toolbar, dijit._Contained], {
	
	postCreate: function() {
		this.inherited(arguments);

		var renderIdStore = new dojo.data.ItemFileReadStore({
			url: net.jazz.ajax._contextRoot + '/resource/itemCol/Interpretation'
		});
		this.renderIdBox = new dijit.form.ComboBox({
			store: renderIdStore,
			searchAttr: "id"
		});
		this.addChild(this.renderIdBox);
		
		var widget = new dijit.form.Button({
			label: "Load",
			onClick: dojo.hitch(this, this._onLoadSource)
		});
		this.addChild(widget);
		
		var widget = new dijit.form.Button({
			label: "Validate",
			onClick: dojo.hitch(this, function() {
				this._onPost(false);
			})
		});
		this.addChild(widget);
		
		var widget = new dijit.form.Button({
			label: "Save",
			onClick: dojo.hitch(this, function() {
				this._onPost(true);
			})
		});
		this.addChild(widget);
	},
	
	_onLoadSource: function() {
    	dojo.xhrGet({
			url: net.jazz.ajax._contextRoot + "/rmpw/qvt/editor/visualization",
			content: { model: this.renderIdBox.attr('value') },
			handleAs: "text",
			load: dojo.hitch(this, this._onSourceCodeLoaded),
			error: function(error) {
				console.error(error);
			}
		});
	},
	
	/**
	 * @param save true if the server should save the parsed model, otherwise just validates it.
	 */
	_onPost: function(save) {
		dojo.xhrPut({
			url: net.jazz.ajax._contextRoot + "/rmpw/qvt/editor/visualization",
			content: {
				model: this.renderIdBox.attr('value'),
				source: this.page.sourceEditor.attr('value'),
				save: save
			},
			handleAs: "json",
			load: dojo.hitch(this, this._onPostComplete),
			error: function(error) {
				console.error(error);
			}
		});
	},
	
	_onPostComplete: function(data) {
    	var store = new dojo.data.ItemFileReadStore({ data: data });
		this.page.parserOutputGrid.setStore(store);
	},
	
    _onSourceCodeLoaded: function(data) {
		this.page.sourceEditor.attr('value', data);

		// clear parser output
		this.page.parserOutputGrid.setStore(null);
    },
});