/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){

dojo.provide("rmpw.forms.ui.ValueScopeComboBox");

dojo.require("dijit.form.FilteringSelect");
dojo.require("dojo.data.ItemFileWriteStore");
dojo.require("rmpw.forms.ui.TableChangesMixin");
dojo.require("rmpw.common.util.ErrorUtil");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmps.core.util.xhrUtil");
dojo.require("rmpw.forms.ui.SelectOrCreateResource");

/*
* Combox box that is scoped by a query defined through a Model Property Constraint
*/
dojo.declare("rmpw.forms.ui.ValueScopeComboBox", dijit.form.FilteringSelect, {

	typeUri: null,
    defaultValue: null,
    _commonMessages: null,
    _loadingWidget: null,
    
    constructor: function (args) {
    	this._parentWidget = args.parentWidget;
    	this.context = args.context;
    	this.changeset = args.changeset;
    	this.typeUri = args.typeUri;
    	this.defaultValue = args.defaultValue;
    	this.documentObject = args.documentObject;
    	this.resourceUri = args.resourceUri;
    	this.propertyUri = args.propertyUri;
    	this.propertyDomainUri = args.propertyDomainUri;
    	this.definableTypes = args.definableTypes;
    	
    	dojo.requireLocalization("rmps.ui.common", "CommonMessages");
		this._commonMessages = dojo.i18n.getLocalization("rmps.ui.common", "CommonMessages");
		
		this._valueStore = new dojo.data.ItemFileWriteStore({data: {items: [{}]}});
		this.set("store", this._valueStore);
		
		this._lastLabel = "";
    },
    
    postCreate: function () {
		this.inherited(arguments);
		
		
		if(this.defaultValue){
			this.set("displayedValue", this.defaultValue.rdf_about ? this.defaultValue.rdfs_label + ": " + this.defaultValue.rdf_about : this.defaultValue.rdfs_label);
		}
		
		this.connect(this, "onKeyPress", this._handleKeyPress);
		this._connectLoadDropDown = dojo.connect(this, "loadDropDown", dojo.hitch(this, function() {
			if (this._lastLabel == "") {
				// load if this is the first time
				dojo.disconnect(this._connectLoadDropDown);
				delete this._connectLoadDropDown;
				this._initializeSelections();
			}
		}));
    },
        
	_initializeSelections: function(){
		if (this._connectClick) {
			this.disconnect(this._connectClick);
		}
		delete this._connectClick;
		
		if (this._parentWidget && !this._loadingWidget) {
			this._loadingWidget= new dijit.layout.ContentPane({content: this._commonMessages.loading});
			this._parentWidget.addChild(this._loadingWidget);
		}
		
		if (this._currentValueRequest) {
			this._currentValueRequest.cancel();
		}
		
		var whereClause = undefined;
		if (this.definableTypes && this.definableTypes.length > 1) {
			whereClause = "";
			for (var i=0, lg=this.definableTypes.length; i<lg; i++) {
				var whereLine = "?type rdfs:subClassOf* <" + this.definableTypes[i] + "> . ?value rdf:type ?type";
				if (i==0) {
					whereClause += "{ " + whereLine + " }";
				}
				else 
					whereClause += "UNION {" + whereLine + " }";
			}
		}

		var args = {
			url: net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/modelvalidation/values",
			handleAs: 'json',
			headers: {
				"Accept" : "application/json",
				"X-ibm-rmps-internal" : "true"
			},
			content: {
				resourceUri: this.resourceUri ? this.resourceUri : (this.form && this.form._resource.id ? this.form._resource.id : (this.documentObject ? this.documentObject.rdf_about : (this.form ? this.form._resource.documentObject.rdf_about : null))),
				propertyUri: this.propertyUri, 
				adjustedDomainUri: this.propertyDomainUri, 
				adjustedRangeUri: this.typeUri,
				userText: this.get("displayedValue"),
				whereClause: whereClause,
				internalOslcRequest: this.internalOslcRequest !== undefined ? this.internalOslcRequest : ""
			},
			load: dojo.hitch(this, this._addTypes),
			error: function(error) {
				if (error.dojoType != "cancel") {  // don't post error for cancel
					rmps.core.error.create(rmpw.common.util.ErrorUtil.getErrorMessage(error)).dispatch();
				}
				delete this._currentValueRequest;
			}
		};
		var ccObject = {}
		if (this.context) {
			ccObject[rmps.core.util.IRmpsConstants.BASELINE] = this.context;
		}
		if (this.changeset) {
			ccObject[rmps.core.util.IRmpsConstants.CHANGESET] = this.changeset;
		}
		rmps.core.util.xhrUtil.addParamater(args, ccObject);

		this._currentValueRequest = rmps.core.client.xhrGet(args, true);
    },
    
    isValid : function() {
    	// assume valid while a current value request is in progress
    	if (this._currentValueRequest || this._keyPressTimeout || this._lastLabel == "") {
    		return true;
    	}
    	
    	return this.inherited(arguments);
    },
    
    _addTypes : function (types){
    	delete this._currentValueRequest;
    	
    	// reset the store
    	this._valueStore = new dojo.data.ItemFileWriteStore({data: {items: [{}]}});
		this.set("store", this._valueStore);
		this.set("searchAttr", "label");
		
    	var store= this._valueStore;    
    	
    	dojo.forEach(types, dojo.hitch(this, function(object, i) {
			var newObject = rmpw.forms.ui.SelectOrCreateResource.prototype.getTypeObject(object, true);
			if (newObject) {
				newObject.id = newObject.rdf_about;
				newObject.label = newObject.rdfs_label;
				store.newItem(newObject);
			}
		}));
		
    	if (this._parentWidget) {
    		this._parentWidget.removeChild(this._loadingWidget);
    		delete this._loadingWidget;
    	}	 		
		this.loadDropDown();
		
		this._valueStore.fetch({query: {rdfs_label:this.get("displayedValue")}, onItem:dojo.hitch(this, function(item) {
			this.set("item", item);
			this.onChange(item.label[0]);
		}), onComplete: dojo.hitch(this, function() {
			// do nothing
		})});
	},
	
	_handleKeyPress: function(event){
		// execute a timeout to allow combo-box to process the key event and pattern match
		if (this._keyPressTimeout) {
			clearTimeout(this._keyPressTimeout);
		}
		this._keyPressTimeout = setTimeout( dojo.hitch(this, function() {
			delete this._keyPressTimeout;
			this._labelInStore = false;
			
			if (this.get("displayedValue").length > this._lastLabel.length) {
				// if there's no match with the current text - then send a new request
				this._valueStore.fetch({query: {rdfs_label:this.get("displayedValue")}, onItem:dojo.hitch(this, function(item) {
					this._labelInStore = true;
					this.set("item", item);
					this.onChange(item.label[0]);
				}), onComplete: dojo.hitch(this, function() {
					if (!this._labelInStore) {
						this._initializeSelections();
					}
				})});
			}
			this._lastLabel = this.get("displayedValue");
			
		}), 400);	
	},
		
	destroy: function(){
		if (this._currentValueRequest) {
			this._currentValueRequest.cancel();
		}
		
		if(this._loadingWidget){
			this._loadingWidget.destroyRecursive();
		}
	}
});
    
})();
