/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.dsv.editor.xhtml.ui.XhtmlEditorPage");

dojo.require("dijit.TitlePane");
dojo.require("dijit.layout.ContentPane");
dojo.require("dijit.layout.BorderContainer");
dojo.require("rmpw.text.internal.widgets.RichTextWidget");
dojo.require("rmpw.dsv.editor.xhtml.util.XhtmlConverter");

dojo.require("rmpw.common.vcs.ChangeSet");
dojo.require("rmps.diagram.data.JtsRdfStore");
dojo.require("rmpw.common.vcs.ref");

/**
 * Internal implementation of an XHTML Editor contributed to the properties of HTML instances in DTK.  Used
 * only as a test harness.  Will not be contributed to production code.
 */
dojo.declare("rmpw.dsv.editor.xhtml.ui.XhtmlEditorPage", dijit.layout.BorderContainer, {
	_richText: null,
	_source: null,
    postCreate: function(){
        this.inherited(arguments);
        this._richText = new rmpw.text.internal.widgets.RichTextWidget({
            style: "height: 100%; width: 100%;",
            uri: this.editor.editorInput.resourceInfo.uri
        });
        this.mainPane = new dijit.layout.ContentPane({
            splitter: "true",
            region: "center",
            content: this._richText
        });
        this.addChild(this.mainPane);
    },

	startup: function() {
		if (this._started) {
			return;
		}
		this.inherited(arguments);
		this.doRefresh();
	},
	
    doSave: function(/*rmps.core.TaskMonitor*/monitor){
        /*
         * Hook for being tab content in Design page
         */
        this._onSave(monitor);
    },
    
    doRefresh: function(){
        /*
         * Hook for being tab content in Design page
         */
        var uri = this.editor.editorInput.resourceInfo.uri;
        var content_type;
        uri = net.jazz.ajax._contextRoot + "/rdfdirectory/?recursionLevel=3&logicalURI=" + encodeURIComponent(uri);
        content_type = "application/rdf+json";
		var namespaces = null;
		var modelInstance;
		var modelIndex = {};
		jazz.client.xhrGet({
			url : uri,
			headers: {
				"Accept": content_type,
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "text",
			load: dojo.hitch(this, function(data, ioargs) {
				modelInstance = rmpw.common.vcs.ref.fromJson(data, {index: modelIndex, idAttribute: "rdf_about", loader: function(){}});//for now loader is empty, implement if we need to support lazy loading
				namespaces = ioargs.xhr.getResponseHeader("prefix");
				this._showXML(modelInstance);
			}),
			error: function(error, ioargs){
				dojo.require("dijit.Dialog");
				var errorDlg = new dijit.Dialog({
		            title: "SERVER ERROR",
		            style: "width: 300px"
		        });					
				errorDlg.set("content",error.message);
				errorDlg.show();
	        }				
		});
		
    },
    
    /*
     * Called prior to saving. If a validation error is present
     * which will prevent save from completing then clients should
     * populate push the error reason (String|Error) into the errors array.
     * Client may also push a Deferred object to defer the validation. This allows
     * the client to communicate with the server to perform validation. To add a validation
     * failure the deferred must either perform an error call back or push errors into
     * the errors array.
     */
    validate: function(/*Array*/errors){
        //console.debug("validate called");
    },
    
    initializeToolBar: function(/* jazz.ui.SimpleToolbar */toolbar,/*function*/setDirty,/*function*/enableRefresh){
        /*
         * Hook for being tab content in Design page
         * @param tooblar is passed so extra buttons can be added if required
         * @param setDirty(bool?)
         *   is a function that should be called when the state of the
         *   Diagram is dirtied or un-dirtied.
         * @param enableRefresh(boolean)
         *   is a function to call to allow the refresh button to be
         *   enabled or disabled
         */
        this.enableRefresh = enableRefresh;
        
        this.setDirty = setDirty;
		this.connect(this._richText,"onChange", this.setDirty);	
    },
	
    _showXML: function(source){
        if (source === null) {
            return;
        }
        
        if (typeof source === "object") {
			this._source = source;
            source = rmpw.dsv.editor.xhtml.util.XhtmlConverter.parse(source);
        }
        else {
			this._source = null;
            source = rmpw.dsv.editor.xhtml.util.XhtmlConverter.htmlifyText(source);
        }
        this._richText.set('value', source);
    },
	
    _onSave: function(/*rmps.core.TaskMonitor*/monitor){
		if (this._source) {
			var task = monitor.beginTask("", 1);
			var uri = this.editor.editorInput.resourceInfo.uri;
			var json = rmpw.dsv.editor.xhtml.util.XhtmlConverter.parseToJson(this._richText.get('value'), uri);
			var resource = {
				objectTypes: json.rdf_type,
				id: uri,
				namespaces: "http://www.w3.org/1999/02/22-rdf-syntax-ns#=rdf,http://jazz.net/ns/dm/core#=dmcore,http://jazz.net/ns/dm/xml#=dmxml,http://jazz.net/ns/dm/document#=dmxhtml,http://www.w3.org/2000/01/rdf-schema#=rdfs,"
			};
			var changeSet = new rmpw.common.vcs.ChangeSet();
			var change = changeSet.createChange(this._source);
			change.remove("dmxhtml_body");
			var changes = [];
			changes.push(changeSet);
			changeSet = new rmpw.common.vcs.ChangeSet();
			change = changeSet.createChange(this._source);
			var body = json.dmxhtml_body;
			change.set("dmxhtml_body", body);
			changes.push(changeSet);
			resource.rdf_about = json.rdf_about;
			
			var store = new rmps.diagram.data.JtsRdfStore();
			resource.deferred = store.saveResource(changes, resource);
			
			var rdf_about = resource.rdf_about;
			var deferred = resource.deferred;
			if (deferred) {
				// Notify when ontology is created
				//                deferred.addCallback(dojo.hitch(this, function(){
				//                    if (this.create) {
				//                        this._setInput();
				//                    }
				//                }));
				deferred.addErrback(function(){
					deferred.errback(rdf_about);
				});
				task.handleDeferredWork(deferred, 1);
			}
		}
    },
    
    _setInput: function(){
//        this.editor.editorInput.setNewResourceURI(this._form._resource.rdf_about);
    },
    
    _setEditable: function(){
        this.isReadOnly = false;
        this.doRefresh();
    }
    
});
