/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide('com.ibm.team.vvc.shared.oslc.web.ui.actions.ShowLinkedElementsDialogAction');

dojo.require("jazz.ui.ScrollableMenu");
dojo.require("com.ibm.team.vvc.shared.oslc.web.utils.CLMUtils");

dojo.requireLocalization("com.ibm.team.vvc.shared.oslc.web.ui.actions", "ShowLinkedElementsDialogActionMessages");

var CLMUtils = com.ibm.team.vvc.shared.oslc.web.utils.CLMUtils;
var Messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.oslc.web.ui.actions", "ShowLinkedElementsDialogActionMessages");

com.ibm.team.vvc.shared.oslc.web.ui.actions.ShowLinkedElementsDialogAction = {
	
	//sourceUrl is the source such as ContextUtils.getCurrentContext().uri
	//getFunction and deleteFunction should return an object with the url and
	//headers properties suitable for xhrGet and xhrDelete
	showDialog: function(sourceUrl, getFunction, deleteFunction) {
		var mainNode = dojo.create("div");
		
		 var styledTable = new jazz.ui.StyledTable({
			 trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
			 //"class" : "streamTable",
			 columns: [
			 { label: Messages.idColumnTitle, width: "10%"},
			 { label: Messages.nameColumnTitle, width: "80%" },
			 { label: Messages.actionsColumnTitle, width: "10%" }
			 ],
			 showHover: true
			 });// end _contextTable
		 styledTable.domNode.style.width= "100%";
		 
		 var getParams = getFunction();
			var args = {
					headers: {
						"Content-Type": "application/x-oslc-common-link+xml",
						"Accept": "application/atom+json"
					},

					//http://jazz.net/ns/vvc/link/tracksChanges/rtc
					//http://open-services.net/ns/cm#tracksChangeSet
					//encoded to		
					//http%3A%2F%2Fjazz.net%2Fns%2Fvvc%2Flink%2FtracksChanges%2Frtc
					//http%3A%2F%2Fopen-services.net%2Fns%2Fcm%23tracksChangeSet

					url: getParams.url,
					headers : getParams.headers,
					handleAs: "json",
					hasBody: false,
					load: dojo.hitch(this, function(response, ioArgs)
					{

						 for(var i = 0; i < response.length; i++){
							 
								var domNode = dojo.create("tr");
								styledTable.tbody.appendChild(domNode );
								
								var idCell = dojo.create("td", null, domNode);
								var nameCell = dojo.create("td", null, domNode);
								var actionsCell = dojo.create("td", null, domNode);
								
								var targetUrl = dojo.doc.createTextNode(response[i].target);
								
								var targetUrlAnchor = dojo.doc.createElement("a");
								targetUrlAnchor.href = response[i].target;
								targetUrlAnchor.appendChild(targetUrl);

								nameCell.appendChild(targetUrlAnchor);
								
								
								var idString = "-";
								var indexOfSlash = response[i].target.lastIndexOf("/");
								if (indexOfSlash > 0 && indexOfSlash < response[i].target.length - 1) {
									idString = response[i].target.substring(indexOfSlash + 1);
								}
								
								var id = dojo.doc.createTextNode(idString);
								
								idCell.appendChild(id);
								
								var deleteUri = dojo.moduleUrl('com.ibm.team.vvc.shared.oslc.web', 'graphics/delete.gif')
								
								var imgAnchor = dojo.doc.createElement("a");
					            imgAnchor.href = "#";
					            var img = dojo.create("IMG", { align:"centre"});
					            img.title = Messages.deleteActionTitle;
					            img.src = deleteUri;
					            img.style.verticalAlign = "middle";
					            imgAnchor.style.verticalAlign = "middle";

					            /*We do not show the action image by default unless we hover on a row */
					            img.style.visibility = "hidden";
					            imgAnchor.style.visibility = "hidden";
					            imgAnchor.appendChild(img);

					            var context = {
					            	deleteFunction: deleteFunction,
				            	    img: img,
				            	    imgAnchor: imgAnchor,
				            	    rowIndex: i,
				            	    responseTarget: response[i].target,
				            	    styledTable: styledTable,
				            	    targetUrl: targetUrl,
				            	    contextUrl: sourceUrl
					            };
					            
					            dojo.connect(actionsCell,"onmouseover", context, function(e){
					                this.img.style.visibility = "visible";
					                this.imgAnchor.style.visibility = "visible";
					            });

					            dojo.connect(actionsCell,"onmouseout", context, function(e){
					            	this.img.style.visibility = "hidden";
					            	this.imgAnchor.style.visibility = "hidden";
					            });

					            dojo.connect(imgAnchor,"onclick", context, function(e){
					                /*to prevent redirection*/
					                dojo.stopEvent(e);
					                
//					                var deleteContext = {
//					                	row: e.target.parentNode.parentNode.parentNode,
//					                	table: e.target.parentNode.parentNode.parentNode.parentNode,
//					                	styledTable: context.styledTable
//					                }
					                
					                var deleteContext = {
					                	rowIndex: this.rowIndex,
					                	styledTable: this.styledTable
					                };
					                

					                var deleteParams = this.deleteFunction(this.responseTarget);
									var deferredDelete = jazz.client.xhrDelete({
										url: deleteParams.url,
										headers : deleteParams.headers,
										handleAs: "xml", //$NON-NLS-1$
										load: dojo.hitch(deleteContext, function(doc){
											this.styledTable.tbody.deleteRow(this.rowIndex);
										}),
										error: dojo.hitch(this, function(error){
											alert(Messages.errorDeletingMessage + "\n" +
											      Messages['errorDeletingMessage.explanation'] + "\n" +
											      Messages['errorDeletingMessage.useraction']);
											console.error(error);
											return error;
										})
									});										                
					                
					            });
					            
					            actionsCell.appendChild(imgAnchor);
//								
								var deferredTitles = jazz.client.xhrGet({
									url: context.responseTarget + "?oslc.properties=dcterms:title",
									headers: {
										"Accept": "application/rdf+xml",
										"OSLC-Core-Version": "2.0"
									},
									handleAs: "xml", //$NON-NLS-1$
									load: dojo.hitch(context, function(doc){
										var title = CLMUtils.extractTitle(doc);
										if (title) {
											this.targetUrl.textContent = title;
										}
									}),
									error: dojo.hitch(this, function(error){
										console.error(error);
										return error;
									})
								});						

						} 
						 
						 mainNode.appendChild(styledTable.domNode);
						 

						 //consider adding secondaryTitle or width: 800
					 dialog = new jazz.ui.Dialog({
						 contentNode: mainNode, primaryTitle: Messages.dialogTitle,
						 destroyContents: true
					 }); 						
						
						
						
					})
				};
				var deferredGet = jazz.client.xhrGet(args);
			
	}
};

})();			