/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
   
	dojo.provide("com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsClient");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ChangesetResource");
	dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory");
	var ncFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
	var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
	
	dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.changesets", "ChangesetsMessages");

	var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.changesets", "ChangesetsMessages");

	var ContextUtils = 	com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	
	var Actions = {
			Commit: "COMMIT",
			Complete: "COMPLETE",
			Rename: "RENAME"
	};

	com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsClient = {
				
		complete: function(changesetUri)
		{
			var jsonPostData = { "action" : Actions.Complete };
			headers = {
					"Content-Type" : "text/plain",
			};
			return jazz.client.xhrPost({
				handleAs: 'text',
				headers: headers,
				sync: true,
				url: changesetUri,
				postData: JSON.stringify(jsonPostData),
				load: dojo.hitch(this, function(response, ioArgs) {
					ncFactory.createSuccess({
						title: messages.completeSuccessTitle,
						description: dojo.string.substitute(messages.completeSuccessDescription, {csTitle: name})
					});
					return true;						
				}),
			});
		},
		
		getCurrentChangeset: function()
		{
			var currentContext = ContextUtils.getCurrentContext();
			if (currentContext == null || !currentContext.isChangeset())
				return null;
			var changeset = new com.ibm.team.vvc.shared.core.ui.objects.ChangesetResource();
			changeset.loadByConfiguration(currentContext);
			return changeset;
		},
		
		rename: function(changesetUri, name)
		{
			var jsonPostData = { "action" : Actions.Rename };
			headers = {
					"Content-Type" : "text/plain"
			};
			jsonPostData["VvcChangesetRenameNewName"] = name;
			return jazz.client.xhrPost({
				handleAs: 'text',
				headers: headers,
				sync: true,
				url: changesetUri,
				postData: JSON.stringify(jsonPostData),
				load: dojo.hitch(this, function(response, ioArgs) {
					ncFactory.createSuccess({
						title: messages.renameSuccessTitle,
						description: dojo.string.substitute(messages.renameSuccessDescription, {csTitle: name})
					});
					return name;						
				}),
			});
		},
		
		commit: function(changesetUri)
		{
			var jsonPostData = { "action" : Actions.Commit };
			headers = {
					"Content-Type" : "text/plain",
					"VvcChangeSetTargetWorkspace" : ContextUtils.getCurrentContext().getParentWorkspaceUri()
			};
			return jazz.client.xhrPost({
				handleAs: 'text',
				headers: headers,
				sync: true,
				url: changesetUri,
				postData: JSON.stringify(jsonPostData),
				load: dojo.hitch(this, function(response, ioArgs) {
					ncFactory.createSuccess({
						title: messages.commitSuccessTitle,
						description: dojo.string.substitute(messages.commitSuccessDescription, {csTitle: name})
					});
					return true;						
				}),
			});
		},
		
		discard: function(changesetUri)
		{
			headers = {
					"Content-Type" : "text/plain"
			};
			this.result = false;
			return jazz.client.xhrDelete({
				headers: headers,
				handleAs: 'text',
				sync: true,
				url: changesetUri,
				load: dojo.hitch(this, function(response, ioArgs) {
					ncFactory.createSuccess({
						title: messages.discardSuccessTitle
					});
					return true;
				}),
			});
		},
	};   
})();