/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsTreeGrid");

dojo.require("dijit._Widget");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("dojox.grid.LazyTreeGrid");
dojo.require("dojox.grid.LazyTreeGridStoreModel");
dojo.require("dojo.data.ItemFileWriteStore");

dojo.require('dojo.i18n');
dojo.require("jazz.util.html");
dojo.require("jazz.ui.Breadcrumbs");
dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.PageTemplate");
dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
dojo.require("com.ibm.team.vvc.shared.oslc.web.utils.CLMUtils");
dojo.require("com.ibm.team.vvc.shared.web.ui.util.LinkToWorkItemUtil");
dojo.require("jazz.ui.ResourceLink");

dojo.require("com.ibm.team.vvc.shared.web.ui.util.InfoContentUtils");
var InfoContentUtils = com.ibm.team.vvc.shared.web.ui.util.InfoContentUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.SecurityServiceUtils");
var SecurityServiceUtils = com.ibm.team.vvc.shared.web.ui.util.SecurityServiceUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory");
var ncFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;

dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.context", "ConfigurationContextMessages");
var configurationContextMessages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.context", "ConfigurationContextMessages");

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
var VvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.changesets", "ChangesetsMessages");

(function ()
{
	var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.changesets", "ChangesetsMessages");
	var authenticated = false;
	var authenticationDeferred = null;

	var ContextUtils = 	com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	var CLMUtils = 	com.ibm.team.vvc.shared.oslc.web.utils.CLMUtils;
	var LinkToWorkItemUtil = com.ibm.team.vvc.shared.web.ui.util.LinkToWorkItemUtil;
	var ResourceLink = jazz.ui.ResourceLink;
	
	var States = {
		Active: "IN_PROGRESS",
		Completed: "COMPLETED",
		Committed: "COMMITTED",
		Suspended: "SUSPENDED"
	};
	
	var Actions = {
			Commit: "COMMIT",
			Complete: "COMPLETE",
			Rename: "RENAME",
			Reverse: "REVERSE"
	};
	
	dojo.declare("com.ibm.team.vvc.shared.web.ui.changesets.WorkItemsWidget", [dijit._Widget], {
		value: null, inRowIndex: null, level: null, cell: null, expandedWorkItemRows: null,
		buildRendering: function() {
			var item = this.cell.grid.getItem(this.inRowIndex);
			var expanded = true;
			
			this.domNode = dojo.create("div");
			
			if (item.workItems && item.workItems.length > 0) {
				var style = "none";
				if (this.expandedWorkItemRows[item.id[0]]) {
					expanded = false;
					style = "block";
				}
				
				//<div class="dojoxGridExpandoNode" dojoattachevent="onclick:onToggle">
				var expandoDiv = dojo.create("div");
				expandoDiv.id = "expandCollapse" + this.inRowIndex;
				dojo.addClass(expandoDiv,  "dojoxGridWorkItemsExpandoNode");
				expandoDiv.style.styleFloat = "left"; //IE
				expandoDiv.style.cssFloat = "left";
				expandoDiv.style.marginTop = "0px";
				if (!expanded) {
					expandoDiv.style.backgroundPosition = "-15px 50%";					
				}

				this.domNode.appendChild(expandoDiv);
				
				var context = {
					self: this,
					item: item,
					grid: this.cell.grid
				};
				
				dojo.connect(expandoDiv, "onclick", dojo.hitch(context, function(event) {
					//event.target.id is what was clicked
					if (this.self.expandedWorkItemRows[this.item.id[0]]) {
						this.self.expandedWorkItemRows[this.item.id[0]] = false;
					} else {
						this.self.expandedWorkItemRows[this.item.id[0]] = true;
					}

					this.grid.store.setValue(this.item, 'workItems', this.item.workItems);
				}));
				
				var numberOfAddedWorkItems = 0;
				var errors = [];
				var expanded = [];
				for (var aWorkItem in item.workItems) {
					var theWorkItem = item.workItems[aWorkItem];
					
					if (theWorkItem) {
						if (theWorkItem.error) {
							errors.push(dojo.doc.createTextNode(dojo.string.substitute(messages.unableToLoad, [theWorkItem.error])));
						} else {
							var a = dojo.create("a");
							a.href = theWorkItem.url;
							a.title = theWorkItem.id;
							a.innerHTML = theWorkItem.id;
							
							new ResourceLink({
								retainLinkText: true,
								uri: a.href,
								label: a.title
							}, a);
							
							this.domNode.appendChild(dojo.doc.createTextNode((numberOfAddedWorkItems === 0) ? " ": ", "));
							this.domNode.appendChild(a);
							//This would be the result of a: "<a href=\"" + theWorkItem.url + "\">" + theWorkItem.id + "</a> "
							
							
							var a2 = dojo.create("a"); 
							a2.href = theWorkItem.url;
							a2.title = theWorkItem.id + ": " + theWorkItem.name;
							a2.innerHTML = theWorkItem.id + ": " + theWorkItem.name;
							expanded.push(a2);
							
							//This would be the result of a2: "<a href=\"" + theWorkItem.url + "\">" + theWorkItem.id + ": " + theWorkItem.name + "</a><BR>"
							
							numberOfAddedWorkItems++;
						}
					} else {
						errors.push(dojo.doc.createTextNode(messages.unableToLoadAll));
						break;
					}
					
				}
				
				//This would be the result of the expandDiv: "<div id='expandFor" + inRowIndex + "' style='display:" + style + ";'>" + expandedLabel + "</div>"
				var expandDiv = dojo.create("div");
				expandDiv.id = "expandFor" + this.inRowIndex;
				expandDiv.style.display = style;
				expandDiv.style.marginTop = "2px";
				this.domNode.appendChild(expandDiv);

				if (numberOfAddedWorkItems > 0) {
					for (var i = 0; i < expanded.length; i++) {
						var img = dojo.create("img");
						new ResourceLink({
							retainLinkText: true,
							uri: expanded[i].href,
							label: expanded[i].title,
							iconNode: img
						}, expanded[i]);
						expandDiv.appendChild(img);
						expandDiv.appendChild(dojo.doc.createTextNode(" "));
						expandDiv.appendChild(expanded[i]);
						
						expandDiv.appendChild(dojo.create("br"));
					}
				} else {
					expandoDiv.style.display = "none";
				}
				
				if (errors.length > 0) {
					for (var i = 0; i < errors.length; i++) {
						this.domNode.appendChild(errors[i]);
						this.domNode.appendChild(dojo.create("br"));
					}
					this.domNode.appendChild(dojo.doc.createTextNode(messages.pleaseAuthenticateFirst));
				}
				
			} else {
				if (this.expandedWorkItemRows[item.id[0]] === undefined) {
					this.domNode.appendChild(dojo.doc.createTextNode(messages.loading));
				} else {
					this.domNode.appendChild(dojo.doc.createTextNode(messages.noWorkItems));
				}
			}
			
		}
	});
	
	
	dojo.declare("com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsTreeGrid", [dijit._Widget], {
		expandedWorkItemRows: {},

		workItemCellsObtained: [],
		
		_showCommitted: false,
		_noFriendsChecked: false,
		
		_lastErrorMessage: null,
		
		standalone:true,
		
		constructor: function(args) {
			if (args && args.standalone != undefined) { //which will also enter the block if it is null (but it's not expected to be null)
				this.standalone = args.standalone;  
			}
		},		
		
		postCreate: function(){
			this._constructGrid();
		},
	   		
		_constructGrid: function() {
			if (!this.authenticated) {
				if (!this.authenticationDeferred) {
					this.authenticationDeferred = new dojo.Deferred();
				}
				CLMUtils.authenticate(this.authenticationDeferred);
			}
			
			if (!this._grid) {
				dojo.when(this._getModel(), dojo.hitch(this, function(model){	
				    /* set up layout */
				    var layout = this.standalone ? [
				                  {name: messages.actions, field: null, width: '156px', formatter: dojo.hitch(this, this.formatActions)},
				                  {name: messages.description, field: 'title', width: 'auto', formatter: dojo.hitch(this, this.formatName)},
				                  {name: messages.ownedBy, field: 'creatorName', width: '200px', formatter: dojo.hitch(this, this.formatUserName)},
				                  {name: messages.workItems, field: 'workItems', width: 'auto', formatter: dojo.hitch(this, this.formatWorkItems)}
				    ] : [
		                  {name: messages.description, field: 'title', width: '200px', formatter: dojo.hitch(this, this.formatName)},
		                  {name: messages.ownedBy, field: 'creatorName', width: '200px', formatter: dojo.hitch(this, this.formatUserName)},
		                  {name: messages.workItems, field: 'workItems', width: 'auto', formatter: dojo.hitch(this, this.formatWorkItems)},
		                  {name: messages.actions, field: null, width: '156px', formatter: dojo.hitch(this, this.formatActions)}	                  
		            ];
			
				    this._grid = new dojox.grid.LazyTreeGrid({
				        id: 'changeSetGrid',
				        treeModel: model,
				        structure: layout,
				        rowSelector: '0px',
				        expandoCell: this.standalone ? 1 : 0,
				        loadingMessage: messages.loading,
				        autoHeight: true,
				      }, document.createElement('div'));
				    this.domNode.appendChild(this._grid.domNode);
				    
				    this._grid.startup();
				    
				    this._grid.resize();
				    
				    // hide the user name column initially
				    this._grid.layout.setColumnVisibility(this.standalone ? 2 : 1, false);
				    
				    this.displayNoChangesetsIfNecessary();
				}));
			}
		},
		
		displayNoChangesetsIfNecessary: function()
		{
			if (this._grid == null)
				return;
			if (this._grid.store._arrayOfAllItems.length == 0)
			{
				this._grid.showMessage(messages.noChangesetsDefined);
			}
			else
			{
				this._grid.showMessage();
			}
		},
		
		formatUserName: function(value, inRowIndex, level, cell)
		{
			//in dojo 1.8, even the invisible rows are formatted
			//let's make a minor performance improvement and just return nothing
			if (!this._isShowAllUser()) {
				return "";
			}
			
			var model =  cell.grid.treeModel;
			var item = cell.grid.getItem(inRowIndex);
			
			var label = "";
			if (value) {
				label = jazz.util.html.escape(value);
			}
			return (item.creatorUri && item.creatorUri[0]) ? "<div><a href=\"" + item.creatorUri[0] + "\">" + label + "</div>" : label;
		},
		
		formatName: function(value, inRowIndex, level, cell)
		{
			var model =  cell.grid.treeModel;
			var item = cell.grid.getItem(inRowIndex);
			var iconClass = this.getImageClass(item, model, cell.grid);
			
			var label = "";
			if (value) {
				label = jazz.util.html.escape(value);
			}
			
			if (level !== 0) {
				//do not allow opening the change set editor on resources
				return "<div><span class='" + iconClass + "' style='padding-top: 0px; padding-right: 0px;display:inline-block'  height='16px' width='16px'> </span>" + label+"</div>";
			}
			
			var contextUri = ContextUtils.getRequestParmValue("contextUri", window.location.href);
			contextUri = (contextUri && contextUri.length > 0) ? ("&contextUri=" + contextUri) : "";

			var action = ContextUtils.getRequestParmValue("action", window.location.href);
			if (action === VvcConstants.VIEW_CHANGESET) {
				return "<div><a href='#action=jazz.viewPage&id=ChangeSetEditor&changesetContext=" + item["id"][0] + contextUri + "' ><span class='" + iconClass + "' style='padding-top: 0px; padding-right: 0px;display:inline-block'  height='16px' width='16px'> </span>" + label+"</a></div>";
			}

			return "<div><a href='" + VvcUtils.getVvcUri() + "/web#action=jazz.viewPage&id=ChangeSetEditor&changesetContext=" + item["id"][0] + contextUri + "' ><span class='" + iconClass + "'  style='padding-top: 0px; padding-right: 0px;display:inline-block'  height='16px' width='16px'> </span>" + label+"</a></div>";
		},
		
		_obtainWorkItems: function() {
			var item = this.grid.getItem(this.index);

			if (dojo.indexOf(this.workItemCellsObtained, item) >= 0 && !(this.reset)) {
				return;
			}
			this.workItemCellsObtained.push(item);
			
			var context = {
					inRowIndex: this.index,
					item: item,
					cell: this.cell,
					expandedWorkItemRows: this.expandedWorkItemRows,
					workItemCellsObtained: this.workItemCellsObtained,
					self: this
				};
				
				var args = {
						headers: {
							"Content-Type": "application/x-oslc-common-link+xml",
							"Accept": "application/atom+json"
						},
						url: VvcUtils.getVvcUri() + "/changesetLinks?changeset=" + item["id"][0] + "&consumerUrl=" + VvcUtils.getVvcUri(),
						handleAs: "json",
						hasBody: false,
						error: dojo.hitch(context, function(responseerror) {
							var workItems = null;
							this.cell.grid.store.setValue(this.item, 'workItems', workItems);
							
							//initially, start off collapsed
							this.expandedWorkItemRows[this.item.id[0]] = false;
						}),
						load: dojo.hitch(context, function(response, ioArgs) {
							
							var workItems = [];
							
							for(var i = 0; i < response.length; i++){
								
								var workItem = {
										id: null,
										url: null,
										type: null,
										name: null,
										error: null
								};
								workItems.push(workItem);
								
								if (response[i].error) {
									workItem.error = response[i].error; 
								} else {
									var idString = "-";
									var indexOfSlash = response[i].target.lastIndexOf("/");
									if (indexOfSlash > 0 && indexOfSlash < response[i].target.length - 1) {
										workItem.id = response[i].target.substring(indexOfSlash + 1);
									}
									
									var context2 = {
										item: this.item,
										cell: this.cell,
										workItems: workItems,
										workItem: workItem
									};
									
									workItem.type = response[i].predicate;
									workItem.url = response[i].target;
									workItem.name = messages.loadingName;
									
									var deferredTitles = jazz.client.xhrGet({
										url: response[i].target + "?oslc.properties=dcterms:title",
										headers: {
											"Accept": "application/rdf+xml",
											"OSLC-Core-Version": "2.0"
										},
										handleAs: "xml", //$NON-NLS-1$
										load: dojo.hitch(context2, function(doc){
											var name = CLMUtils.extractTitle(doc);
											if (name) {
												this.workItem.name = name;
											}
											this.cell.grid.store.setValue(this.item, 'workItems', this.workItems);
										}),
										error: dojo.hitch(context2, function(error, args){
											this.workItem.name = messages.unableToLoadName;
											ncFactory.createXhrError({xhrError: error, ioArgs: args}).dispatch();
											return error;
										})
									});						
									
								}

								
							}
							
							workItems.sort(CLMUtils.idSorter);
							
							this.cell.grid.store.setValue(this.item, 'workItems', workItems);
							
							//initially, start off collapsed
							this.expandedWorkItemRows[this.item.id[0]] = false;
						})
					};

				var deferredGet = jazz.client.xhrGet(args);
		},

		formatWorkItems: function(value, inRowIndex, level, cell)
		{
			var model =  cell.grid.treeModel;
			var item = cell.grid.getItem(inRowIndex);
			
			var itemType = item.type;
			if (item.type && item.type.length === 1 && item.type[0] === "changeset") {

				var context = {
					index: inRowIndex,
					self: this,
					grid: cell.grid,
					cell: cell,
					expandedWorkItemRows: this.expandedWorkItemRows,
					workItemCellsObtained: this.workItemCellsObtained
				};
				
				if (!this.authenticated && this.authenticationDeferred) {
					this.authenticated = true;
					this.authenticationDeferred.addCallback(context, function(result) {
						if (!this.self._noFriendsChecked) {
							this.self._noFriendsChecked = true;
							
							var hasCCMFriends = false;
							for (var i = 0; i < result.length; i++) {
								if (result[i]) {
									if (result[i].length >= 2) {
										if (result[i][1]) {
											hasCCMFriends = true;
											break;
										}
									}
								}
							}
							
							if (!hasCCMFriends && this.self.displayInfoMessage) {
								this.self.displayInfoMessage(messages.pleaseAddCCMFriends);
							}
						}
						var hitched = dojo.hitch(this, this.self._obtainWorkItems);
						hitched();
					});
				} else {
					var hitched = dojo.hitch(context, context.self._obtainWorkItems);
					hitched();
				}
				
				return new com.ibm.team.vvc.shared.web.ui.changesets.WorkItemsWidget({
					value:value, inRowIndex:inRowIndex, level:level, cell:cell, expandedWorkItemRows:this.expandedWorkItemRows});
			}
			
			return "";
		},
		
		getImageClass: function(item, model, grid){
			if (item.type == "changeset")
			{
				if (item.state == States.Active){
					return "icons-changeset-explorer-changeset";
				} else if (item.state == States.Completed) {
					return "icons-changeset-explorer-changeset-complete";
				} else if (item.state == States.Committed) {
					return "icons-changeset-explorer-changeset-committed";
				} else {
					return "icons-changeset-explorer-changeset-suspended";
				}
			}
			else if (item.type == "workitem")
				return "icons-changeset-explorer-workitem";
			else if (item.type == "folder")
				return "icons-changeset-explorer-open-folder";
			else if (item.type == "resource")
				return "icons-changeset-explorer-resource";
			return "";
		},
		
		_performAction: function(node, action)
		{
			var resourceUri = node.uri;
			var encodedUri = encodeURI(resourceUri);
			var headers;
			var name;
			var jsonPostData = { "action" : action }
			if (action == Actions.Commit || action == Actions.Reverse) {
				headers = {
						"Content-Type" : "text/plain",
						"VvcChangeSetTargetWorkspace" : ContextUtils.getContextUri()
				};
			} else if (action == Actions.Rename) {
				name = prompt(messages.renamePrompt, node.title);
				if (!name || name == node.title) {
					// if user has cancelled the input prompt
					return;
				}
				
				headers = {
						"Content-Type" : "text/plain"
				};
				jsonPostData["VvcChangesetRenameNewName"] = name;
			} else {
				headers = {
						"Content-Type" : "text/plain"
				};				
			}
			
			var errHandler = dojo.hitch(this, function(err, args){
				var ncArgs;
				if(action === Actions.Commit){
					ncArgs = {
						title: dojo.string.substitute(configurationContextMessages["failedToCommitChangeset"], [node.title]),
						description: configurationContextMessages["failedToCommitChangeset.explanation"],
						solution: configurationContextMessages["failedToCommitChangeset.useraction"]
					}
				} else if(action === Actions.Complete) {
					ncArgs = {
						title: dojo.string.substitute(configurationContextMessages["failedToCompleteChangeset"], [node.title]),
						description: configurationContextMessages["failedToCompleteChangeset.explanation"],
						solution: configurationContextMessages["failedToCompleteChangeset.useraction"]				
					}
				} else if(action === Actions.Rename){
					ncArgs = {
						title: dojo.string.substitute(configurationContextMessages["failedToRenameChangeset"], [node.title]),
						description: configurationContextMessages["failedToRenameChangeset.explanation"],
						solution: configurationContextMessages["failedToRenameChangeset.useraction"]
					}
				} else if (action === Actions.Reverse){
					// Use stock error message (for now..)
					ncArgs = {};
				}
				ncArgs.xhrError = err;
				ncArgs.ioArgs = args;
				ncFactory.createXhrError(ncArgs).dispatch();
			});

			jazz.client.xhrPost({
				handleAs: 'text',
				headers: headers,
				url: resourceUri,
				postData: JSON.stringify(jsonPostData),
				error: errHandler,
				load: dojo.hitch(this, function(name, action, response, ioArgs) {
					if (action == Actions.Reverse) {
						// The newly created reversing change set may not show resources initially because of race conditions: 
						// it needs to call DM and DM should have change set configuration ready in order to provide resource names and that will take some time.
						// Best we can do is wait before refreshing.
						var _this = this;
						var notification = _this._showMessage(messages.initializingReverseChangeset);
						window.setTimeout(function() {
							_this.refresh();
							notification.close();
						}, 10000);						
					}
					
					this._grid.store.fetchItemByIdentity({
						identity:node.uri,
						onItem:dojo.hitch(this._grid.store, function(name, action, item){
							if (action == Actions.Rename){
								this.setValue(item, 'title', name);
								ncFactory.createSuccess({
									title: messages.renameSuccessTitle,
									description: dojo.string.substitute(messages.renameSuccessDescription, {csTitle: node.title[0]}),
								}).dispatch();
							} else if (action == Actions.Commit) {
								this.setValue(item, 'state', "COMMITTED");
								ncFactory.createSuccess({
									title: messages.commitSuccessTitle,
									description: dojo.string.substitute(messages.commitSuccessDescription, {csTitle: node.title[0]}),
								}).dispatch();
							} else if (action == Actions.Complete){
								this.setValue(item, 'state', "COMPLETED");
								ncFactory.createSuccess({
									title: messages.completeSuccessTitle,
									description: dojo.string.substitute(messages.completeSuccessDescription, {csTitle: node.title[0]}),
								}).dispatch();
							}
							this.save();
						}, name, action)
					});						
				}, name, action),
			});
		},

		_discard: function(node) {
			var notification = this._showMessage(messages.discardingChangeset);
			
			var resourceUri = node.uri;
			jazz.client.xhrDelete({
				handleAs: 'text',
				url: resourceUri, //$NON-NLS-1$
				content: {
					/* Stub */
				},

				load: dojo.hitch(this, function(response, ioArgs) {
					notification.close();
					ncFactory.createSuccess({
						title: messages.discardSuccessTitle
					}).dispatch();
					this._grid.store.fetchItemByIdentity({
						identity:node.uri,
						onItem:dojo.hitch(this._grid.store, function(item){
							this.deleteItem(item);
							this.save();
						})
					});
				}),
				
				error: dojo.hitch(this, function(err, args){
					ncFactory.createXhrError({
						xhrError: err, 
						ioArgs: args,
						title: dojo.string.substitute(configurationContextMessages["failedToDiscardChangeset"], [node.title]),
						description: configurationContextMessages["failedToDiscardChangeset.explanation"],
						solution: configurationContextMessages["failedToDiscardChangeset.useraction"]
					}).dispatch();
				})
			});
		},

		
		_actionRegistry: { 
			changeset : [
			    {
					isApplicable: function(node) {
						return node.permissionToAdminister[0] && (node.state != States.Committed);
					},
					label: function(node) {
						if (!node.permissionToAdminister[0]) {
							return SecurityServiceUtils.getNoPermissionChangesetMessage(node.controllingAreaName);
						}
						return messages.discard;
					},
					imgClass: "icons-changeset-explorer-discard",
					handler: function (node) {
						InfoContentUtils.showQuestionDialog(
								messages.discardChangesetTitle,
								dojo.string.substitute(messages.discardChangesetContent, {changesetTitle: node.title[0]}),
								messages.discardChangesetOkButton,
								messages.cancelButton,
								dojo.hitch(this, this._discard, node)
						);
					}
				},              
				{
					isApplicable: function(node) {
						return node.permissionToAdminister[0] && node.state != States.Committed;
					},
					label:  function(node) {
						if (!node.permissionToAdminister[0]) {
							return SecurityServiceUtils.getNoPermissionChangesetMessage(node.controllingAreaName);
						}
						return messages.share;
					},
					imgClass: "icons-changeset-explorer-share",
					handler: function (node) {
						this._performAction(node, Actions.Commit);
					}
				},
				{
					isApplicable: function(node) {
						return (node.permissionToModify[0] || node.permissionToAdminister[0]) && (node.state != States.Completed && node.state != States.Committed);
					},
					label:  function(node) {
						if (!node.permissionToModify[0] && !node.permissionToAdminister[0]) {
							return SecurityServiceUtils.getNoPermissionChangesetMessage(node.controllingAreaName);
						}
						return messages.complete;
					},
					imgClass: "icons-changeset-explorer-complete",
					handler: function (node) {
						this._performAction(node, Actions.Complete);
					}
				},
				{
					isApplicable: function(node) {
					// CR 43393 Completed and Shared Change Sets can be renamed.
						return (node.permissionToModify[0] || node.permissionToAdminister[0]) ;
						
					},
					label:  function(node) {
						if (!node.permissionToModify[0] && !node.permissionToAdminister[0]) {
							return SecurityServiceUtils.getNoPermissionChangesetMessage(node.controllingAreaName);
						}
						return messages.rename;
					},
					imgClass: "icons-changeset-explorer-rename",
					handler: function (node) {
						this._performAction(node, Actions.Rename);
					}
				},
				{
					isApplicable: function(node) {
						return true;
					},
					label:  function(node) {
						return messages.linkToWorkItem;
					},
					imgClass: "icons-changeset-explorer-workitem",
					handler: function (node) {
						//show link dialog
						var deferred = new dojo.Deferred();
						LinkToWorkItemUtil.vvcLinkToWorkItemAction(node.uri, deferred);
						
						var index, found = false;
						for (index = 0; index < this._grid._by_idx.length; index++) {
							if (this._grid._by_idx[index]) {
								if (this._grid._by_idx[index].idty === node.id[0]) {
									found = true;
									break;
								}
							}
						}

						//if not found, data structure is corrupt and it cannot be handled
						if (found) {
							found = undefined;
							for (var i = 0; i < this._grid.layout.cells.length; i++) {
								if (this._grid.layout.cells[i].field === "workItems") {
									found = this._grid.layout.cells[i];
									break;
								}
							}
							
							if (found) {
								var context = {
										index: index,
										self: this,
										grid: this._grid,
										cell: found,
										expandedWorkItemRows: this.expandedWorkItemRows,
										workItemCellsObtained: this.workItemCellsObtained,
										reset: true
									};
									
									deferred.addCallback(context, this._obtainWorkItems);
							}
						}
					}
				},
				{
					isApplicable: function(node) {
						return (node.state == States.Committed);
					},
					label:  function(node) {
						return messages.reverse;
					},
					imgClass: "icons-changeset-explorer-reverse",
					handler: function (node) {
						InfoContentUtils.showQuestionDialog(
								messages.reverseChangesetTitle,
								dojo.string.substitute(messages.reverseChangesetContent, {changesetTitle: node.title[0]}),
								messages.reverseChangesetOkButton,
								messages.cancelButton,
								dojo.hitch(this, this._performAction, node, Actions.Reverse)
						);
					}
				},
				
			],
			resource: [
			           // stub
			],
			workitem: [
				{
					isApplicable: function(node) {
						return true; 
					},
					label:  function(node) {
						return messages.unlink;
					},
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/unlink.gif'),
					handler: function (node) {
						alert('TODO: disassociate work item');
					}
				},              
				{
					isApplicable: function(node) {
						return true;
					},
					label:  function(node) {
						return messages.open;
					},
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/open.gif'),
					handler: function (node) {
						alert('TODO: open work item');
					}
				}
			],
		},

		
		formatActions: function(value, inRowIndex, level, cell) 
		{
			var model =  cell.grid.treeModel;
			var toolbar = new jazz.ui.Toolbar();
			var node = cell.grid.getItem(inRowIndex);
			var self = this;
			
			var actionRegistry = this._actionRegistry[node.type];
			
			if (actionRegistry)
			{
				dojo.forEach(actionRegistry, dojo.hitch(this, function (action)
				{
					var button = jazz.ui.toolbar.Button({
						title: action.label(node),
						imgClass: action.imgClass,
						imgSrc: action.imgSrc, //consider using imgClass instead
						onClick: dojo.hitch(this, function(){
							dojo.hitch(self,action.handler)(node);
						})
					});
					if (!action.isApplicable(node))
						button.set("enabled", false);
					toolbar.addChild(button);
				}));
			}
			dojo.addClass(toolbar.domNode, "com-ibm-team-vvc-changesets-explorerActionsToolbar");		
			return toolbar;
		},
		
		resize: function() {
			if(this._grid){
				this._grid.resize();
			}
		},
		
	    uninitialize: function()
		{
	    	if (this._grid) {
				this._grid.destroyRecursive();
				delete this._grid;
			}
		},
		
		collapseAll:function()
		{
			for (var i = this._grid.cache.items.length -1; i >= 0; i--) {
				if (this._grid.cache.getExpandoStatusByRowIndex(i)){
					this.toggleExpandRow(i);
				}
			}
		},
		
		getView: function() {
			return this._grid.views.views[1];
		},
		
		toggleExpandRow: function(ind) {
			var idty = this._grid._by_idx[ind].idty; 
			var expando = this.getView()._expandos[idty];
			if (expando) {
				expando.onToggle({stopPropagation: function() {}, preventDefault: function() {}} , true);
			}
		},
		
		setUser: function(user) {
			if (user != this._user) {
				this._user = user;
				if (this._isShowAllUser()) {
					this._grid.layout.setColumnVisibility(this.standalone ? 2 : 1, true);
				} else {
					this._grid.layout.setColumnVisibility(this.standalone ? 2 : 1, false);
				}
				this.refresh();
			}
		},
		
		_isShowAllUser: function() {
			if (this._user === "*") {
				return true;
			}
			return false;
		},
		
		setShowCommitted: function(show) {
			this._showCommitted = show; 
		},
		
		_getData: function()
		{
			this.data = null;
			
			if (this._lastErrorMessage){
				this._lastErrorMessage.close();
				this._lastErrorMessage = null;
			}
			
			var contextUri = ContextUtils.getContextUri();
			var content = {
					withResources : "true",
					includeCreated : "true",
					includeDisplayNames: "true",
					configuration : contextUri
			};
			
			if (! this._showCommitted) {
				content.filterCommitted = "true";
			}

			if (this._user && this._user.length > 0) {
				content.user = this._user;
			}
			
			return jazz.client.xhrGet({
				handleAs: 'json',
				url: VvcUtils.getVvcUri() + "/changesets/query", //$NON-NLS-1$
				content: content,

				load: dojo.hitch(this, function(result) {
					this.data = result;
					return this.data;
				}),
				
				error: dojo.hitch(this, function(err, args) {
					ncFactory.createXhrError({xhrError: err, ioArgs: args}).dispatch();
				})
			});
		},
		
		_computeResourceId: function(changesetUri, resourceUri){
			//To get an unique identifier we need to take the 2 last segments
			var resourceUriWithoutLastSegment = resourceUri.substring(0, resourceUri.lastIndexOf("/"));
			var resourceIdx = resourceUriWithoutLastSegment.lastIndexOf("/")+1;
			var resourceId = resourceUri.substring(resourceIdx).replace("/", "_");
			return changesetUri + "#" + resourceId;
		},
		
		_formatData: function(data)
		{
			var processedData = [];
			// data may be null the first time coming to vvc due to the authentication process.
			if (data){
				var i, j;
				for (i=0; i<data.length; i++) {
					data[i].type= 'changeset';
					data[i].id = data[i].uri;
					var resources = data[i].resources;
					if (resources){
						for ( j=0; j < resources.length; j++) {
							var title = resources[j].displayName;
							if (!title || title.length < 1) {
								title =  resources[j].concept;
							}
							resources[j].title= title;
							resources[j].type='resource';
							resources[j].id = this._computeResourceId(data[i].id, resources[j].concept);					
						}
					}
					processedData.push(data[i]);
				};
			}
			return { identifier: 'id',
					label: 'title',
					items: processedData					
			};
			
		},
		
		_getModel: function() {
			var readyDef = new dojo.Deferred();
			dojo.when(this._getData(), dojo.hitch(this, function(results){
				var data = this._formatData(results);
				var store = new dojo.data.ItemFileWriteStore({"data": data});
				var model = new dojox.grid.LazyTreeGridStoreModel({
					store: store, 
					deferItemLoadingUntilExpand:true, 
					childrenAttrs: ['resources']
				});
				readyDef.resolve(model);
			}));
			
			return readyDef;
		},
		
		refresh: function() {
			var notification = this._showMessage(messages.refreshingChangesets);
			this.authenticated = false;
			this.authenticationDeferred = new dojo.Deferred();
			this.workItemCellsObtained = [];
		    this.expandedWorkItemRows = [];
			CLMUtils.authenticate(this.authenticationDeferred);
			
			dojo.when(this._getModel(), dojo.hitch(this, function(model){
				this._grid.setModel(model);
				notification.close();
			}));
		},
		
		_showMessage: function(message, context, closeOnEvent) {
			var args = {
				title: message
			};
			
			if(context && closeOnEvent){
				args.closeOnEvent = {
					"context" : context,
					"event" : closeOnEvent
				}
			}
			
	    	return ncFactory.createInfo(args).dispatch();
	    },
		    
	    _hideMessage: function()
	    {
	    	dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
	    },
	    
	    addChangeset: function(newChangeset)
	    {
	    	if (this._grid != null)
	    	{
	    		this.expandedWorkItemRows[newChangeset.id] = false;
	    		this._grid.store.newItem(arguments[0]);
				this._grid.store.save();
	    	}
	    },
	});   
	})();
})();