/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsWidgit");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");

dojo.require("dojox.grid.LazyTreeGrid");
dojo.require("dijit.tree.ForestStoreModel");
dojo.require("dojo.data.ItemFileWriteStore");

dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require('jazz.util.html');
dojo.require("jazz.ui.toolbar.Separator");
dojo.require("jazz.ui.Toolbar");

dojo.require('dojo.i18n');
dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.changesets", "ChangesetsMessages");

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsTreeGrid");
dojo.require("com.ibm.team.vvc.shared.web.ui.changesets.CreateChangesetForm");
dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");

dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController");
dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory");
var ncFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;

dojo.require("jazz.util.ResizeNotifier");
dojo.require("jazz.app.ua.PageHoverHelp");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var ContextUtils = 	com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	
	dojo.declare("com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsWidgit", [dijit._Widget, dijit._Templated], {
		
		messages: dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.changesets", "ChangesetsMessages"),
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'ui/changesets/templates/ChangesetsWidgit.html'),
		widgetsInTemplate: true,
		_newChangesetButton: null,
		
		notificationCenter: null,
		standalone:true,
		
		lastMessage: null,
		
		explorerUrl: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/changesets.gif'),

		constructor: function(args) {
			if (args && args.standalone != undefined) { //which will also enter the block if it is null (but it's not expected to be null)
				this.standalone = args.standalone;  
			}
			this._updateTitle();
			if(!args || !args.hideNotificationCenter){
				this.notificationCenter = new com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController();
			}
		},
		
		postCreate: function(){
			var self = this;
			
			this._constructToolbar();
			this._constructGrid();
			this._initializeCSH();
			this._initializeChangesetSubscribers();
			
			this._resizeHandle = jazz.util.ResizeNotifier.register({
				context: this,
				callback: function() {
		    		setTimeout(function(){
		    			self.resizeExplorer();
		    	    }, 100);
		    	},
				force: true
			});
			
			if(this.notificationCenter){
				this.notificationCenterNode.appendChild(this.notificationCenter.domNode);
			}
			
			if (!this.standalone) {
				dojo.style(this._pageHeader, "display", "none");
			}
			
		},
				
		_updateTitle: function() {
			this._configurationContext = this._getConfigurationContext();
			var title;
			if (this._configurationContext) {
				title = this._configurationContext.getTitle();
			}
			if (title) {
				this._initialized = true;
			} else {
				title = "";
			}
			this.changesetsHeading = dojo.string.substitute(this.messages.changesetsHeading, [ jazz.util.html.escape(title) ]);
		},
		
		_initializeChangesetSubscribers: function()
		{
			dojo.subscribe("vvc.changeset_created", dojo.hitch(this, function() 
			{
				if (this._grid != null)
				{
					this._grid.addChangeset(arguments[0]);
				}
			} ));
		},
		
		_initializeCSH: function() {	
			this._ua = new jazz.app.ua.PageHoverHelp({uaKey: "com.ibm.team.vvc.shared.web.page_ChangesetsExplorer001"});
			this._pageHeader.appendChild(this._ua.domNode);
			this._ua.domNode.style.margin = "0px 0px 0px 5px";	
	   	},
	   	
		_getConfigurationContext: function() {
			var contextUri = ContextUtils.getContextUri();
			var configurationContext = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
			configurationContext.load(contextUri);
			return configurationContext;
		},
		
		_constructToolbar:function()
		{
			if (!this._toolbar)
			{
				
				if (!this.standalone) {
					dojo.removeClass(this._table, "header");
				}
				var toolbar = new jazz.ui.Toolbar();
				
				this._newChangesetButton = jazz.ui.toolbar.Button({
					title: this.messages.newChangesetTooltip,
					label: (this.standalone) ? "" : this.messages.newChangeset,
					enabled:false,
					imgClass: "icons-changeset-explorer-create-change-set",
					onClick: dojo.hitch(this, function(){
						var form = new com.ibm.team.vvc.shared.web.ui.changesets.CreateChangesetForm({
							"configuration" : ContextUtils.getContextUri(),
							"ncFactory" : ncFactory
						});
						form.show();
					})
				});
				toolbar.addChild(this._newChangesetButton);				

				var link = VvcUtils.getVvcUri() + "/security/getAllPermissions?configurationUri=" + this._configurationContext.getUri();
				var deferred = this._configurationContext.loadPermissions(link);
				
				deferred.addCallback(this, function (param) {
					this._newChangesetButton.set("enabled", (this._configurationContext.permissionsLoaded && this._configurationContext.permissions.update));
				});					
				
				
				toolbar.addChild(new jazz.ui.toolbar.Separator());
				
				var button = jazz.ui.toolbar.Button({
					title: this.messages.switchUserModeTooltip,
					label: (this.standalone) ? "" : this.messages.showChangeSetsFromAllUsers,
					imgClass: "icons-changeset-explorer-one-user",
					onClick: dojo.hitch(this, function(){
						if (!this.showAll || this.showAll === false) {
							this.showAll = true;
							this._grid.setUser("*");
							this.showChangesetsButton.setAttribute("imgClass", "icons-changeset-explorer-all-users");
							if (!this.standalone) {
								this.showChangesetsButton.setAttribute("label", this.messages.showChangeSetsFromCurrentUser);
							}
						} else {
							this.showAll = false;
							this._grid.setUser(null);
							this.showChangesetsButton.setAttribute("imgClass", "icons-changeset-explorer-one-user");
							if (!this.standalone) {
								this.showChangesetsButton.setAttribute("label", this.messages.showChangeSetsFromAllUsers);
							}
						}
					})
				});
				toolbar.addChild(button);
				this.showChangesetsButton = button;
				
				this.showChangesetsButtonSeparator  = new jazz.ui.toolbar.Separator();
				toolbar.addChild(this.showChangesetsButtonSeparator);
				
				this.committedButton = jazz.ui.toolbar.Button({
					title: this.messages.showCommittedTooltip,
					label: (this.standalone) ? "" : this.messages.showAllChangeSets,
					imgClass: "icons-changeset-explorer-shared-changesets",
					onClick: dojo.hitch(this, function(){
						if (!this.showCommitted || this.showCommitted === false) {
							this.showCommitted = true;
							this._grid.setShowCommitted(true);
							this.committedButton.setAttribute("imgClass", "icons-changeset-explorer-changeset-normal");
							if (!this.standalone) {
								this.committedButton.setAttribute("label", this.messages.showUnsharedChangeSets);
							}
						} else {
							this.showCommitted = false;
							this._grid.setShowCommitted(false);
							this.committedButton.setAttribute("imgClass", "icons-changeset-explorer-shared-changesets");
							if (!this.standalone) {
								this.committedButton.setAttribute("label", this.messages.showAllChangeSets);
							}
						}
						this._grid.refresh();
					})
				});
				toolbar.addChild(this.committedButton);
				
				this.showCommittedButtonSeparator  = new jazz.ui.toolbar.Separator();
				toolbar.addChild(this.showCommittedButtonSeparator);
				
				var refreshButton = jazz.ui.toolbar.Button({
					title: this.messages.refresh,
					label: (this.standalone) ? "" : this.messages.refresh,
					imgClass: "icons-changeset-explorer-refresh",
					onClick: dojo.hitch(this, function(){
						this._grid.refresh();
					})
				});
				toolbar.addChild(refreshButton);
				
				if (!this.standalone) {
					dojo.style(this._toolbarDiv, "float", "left");
				}
				
				this._toolbar = toolbar;
				this._toolbarDiv.appendChild(toolbar.domNode);
			}
		},
		
		_constructGrid: function() {
			if (!this._grid)
			{
				this._grid = new com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsTreeGrid({
					"displayErrorMessage" : this.displayErrorMessage,
					"displayInfoMessage" : this.displayInfoMessage,
					"hideErrorMessage" : this.hideErrorMessage,
					standalone: this.standalone
				});
				
			    /* append the new grid to the div */
			    this._explorerDiv.appendChild(this._grid.domNode);
		
			    /* Call startup() to render the grid */
			    this.resizeExplorer();
			}
		},
		
		resizeExplorer: function()
	    {
	    	if (this._grid)
		 		this._grid.resize();
	    },
	    
	    uninitialize: function()
		{
	    	if (this._grid) {
				this._grid.destroyRecursive();
				delete this._grid;
			}
	    	if (this._breadcrumbs) {
				this._breadcrumbs.destroyRecursive();
				delete this._breadcrumbs;
			}
	    	if (this._toolbar) {
				this._toolbar.destroyRecursive();
				delete this._toolbar;
			}
		},
		
		// The changeset view may has problem getting the configuration
		// the first time the user go from the application to vvc due to 
		// the authentication process. So, we need to update the title and
		// content when this happen.
		initialize: function() {
			if (this._initialized === true) {
				return;
			}
			this._initialized = true;
			
			// update the title
			if (!this.standalone) {
				dojo.style(this._pageHeader, "display", "none");
			}
			var childNodes = this._pageHeader.childNodes;
			for (var i in childNodes) {
				var node = childNodes[i];
				if (node.nodeName === "#text") {
					this._configurationContext = this._getConfigurationContext();
					var title;
					if (this._configurationContext) {
						title = this._configurationContext.getTitle();
						if (title) {
							this.title = title;
							this.changesetsHeading = dojo.string.substitute(this.messages.changesetsHeading, [ jazz.util.html.escape(title) ]);
							node.nodeValue = this.changesetsHeading;
						}
					}
					break;
				}
			}

			// refresh the content
			if (this._grid) {
				this._grid.refresh();
			}
		},
		
		displayErrorMessage: function(errorMessage) 
		{
			this.lastMessage = ncFactory.createError({title: errorMessage}).dispatch();
		},
		
		displayInfoMessage: function(infoMessage) 
		{
			this.lastMessage = ncFactory.createInfo({title: infoMessage}).dispatch();
		},
		
		hideErrorMessage: function() {
			if(this.lastMessage){
				this.lastMessage.close();
			}
		},
	});   
})();