/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.shared.web.ui.changesets.CreateChangesetForm");

dojo.require('dojo.i18n');
dojo.require("dijit.form.Button");
dojo.require("dijit.form.DateTextBox");
dojo.require("dijit.form.SimpleTextarea");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.form.Form");

dojo.require("com.ibm.team.vvc.shared.web.ui.DialogForm");

dojo.require("jazz.util.ResizeNotifier");

dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.changesets", "ChangesetsMessages");

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
var ContextUtils = 	com.ibm.team.vvc.shared.web.ui.context.ContextUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

dojo.declare("com.ibm.team.vvc.shared.web.ui.changesets.CreateChangesetForm", com.ibm.team.vvc.shared.web.ui.DialogForm, {
	
	templatePath: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'ui/changesets/templates/CreateChangesetForm.html'),
	_configuration: null,
	widgetsInTemplate: true,
	_connectedChangesetTitleTextBoxListener: null,
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.changesets", "ChangesetsMessages");	
		if (args == null && args.configuration == null)
			throw "Must pass in the configuration to the CreateChangesetForm constructor";
		else
			this._configuration = args.configuration;
	},
	
	postCreate: function() {
		this.inherited(arguments);
		
		this._connectedChangesetTitleTextBoxListener = dojo.connect(this._changesetTitleTextBox, "onKeyPress", dojo.hitch(this, function(e) {                 
		    if (e.keyCode == dojo.keys.ENTER) {
		    	dojo.stopEvent(e);
		        this._onCreate();
		    }
		}));

	},
	
	getDialogArguments: function(){
		return {
			contentNode: this.containerNode,
			primaryTitle: this.messages.createChangesetDialogTitle,
			destroyContents: true,
			width: "600px"
		};
	},
	
	_cleanUp: function() {
		if (this._connectedChangesetTitleTextBoxListener) {
			dojo.disconnect(this._connectedChangesetTitleTextBoxListener);
		}
	},
		
	_onCreate: function(){
		this.disableButtons();
		
		var title= this._changesetTitleTextBox.get('value');
		var url = VvcUtils.getVvcUri() + '/changesets?configuration=' + this._configuration;
		if (title) {
			 url += '&name=' + encodeURIComponent(title);
		}
		var def = jazz.client.xhrPost({
			handleAs: 'text',
			url: url,
			load: dojo.hitch(this, function(response, ioArgs) {
				if (ioArgs) {
					var xhr = ioArgs.xhr;
					if (xhr) {
						dojo.publish(vvcConstants.EVENTS_CHANGESET_CREATED, [{
							type: 'changeset',
							id:xhr.getResponseHeader("Location"),
							uri:xhr.getResponseHeader("Location"),
							configurationUri: xhr.getResponseHeader("configurationUri"),
							permissions: {permissionToAdminister: true, permissionToModify: true},
							permissionToAdminister: true, permissionToModify: true,
							creatorName: jazz.app.currentApplication.config.user.name, creatorUri: jazz.app.currentApplication.config.user.uri, 
							title:title,
							state:'IN_PROGRESS'}]);
					}
				}
				this._onCancel();
			}),
			
			error: dojo.hitch(this, function(e) {
				if(this.ncFactory){
					this.ncFactory.createXhrError({xhrError: e}).dispatch();
				} else {
					//TODO: Still used for the ConfigurationContextBannerButton, need to create cross-application/cross-domain error reporting. 
					alert('error '+e.message);
				}
				this._onCancel();
			})
		});
	},
	
	show: function() {
		this.inherited(arguments);
		
		// set initial focus
		this._changesetTitleTextBox.focus();
	},
	
	_onCancel: function(){
		this._cleanUp();
		this.hide();
	},
	
	disableButtons: function(){
		this._okButtonNode.setDisabled(true);
		this._cancelButtonNode.setDisabled(true);
		this._changesetTitleTextBox.setDisabled(true);
		this._dialog.setClosable(false);
	}
	
});

})();