/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.shared.web.ui.configurations.AddToGlobalConfigurationForm");

dojo.require("com.ibm.team.vvc.shared.web.ui.DialogForm");
dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
dojo.require("com.ibm.team.vvc.shared.oslc.web.utils.RootServices");

dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.configurations", "CreateGlobalConfigurationMessages");

var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
var vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.declare("com.ibm.team.vvc.shared.web.ui.configurations.AddToGlobalConfigurationForm", com.ibm.team.vvc.shared.web.ui.DialogForm, {
	
	templatePath: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'ui/configurations/templates/AddToGlobalConfigurationForm.html'),
	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.configurations", "CreateGlobalConfigurationMessages"),
	_wizardUrlDataTemplate: "configurationUri=${configurationUri}&type=${type}#oslc-core-postMessage-1.0",
	_wizardUrlNoDataTemplate: "#oslc-core-postMessage-1.0",
	space: null,
	type: null,
	createOnly: false,
	
	constructor: function(args){
		dojo.safeMixin(this, args);
	},
	
	postCreate: function() {
		this.inherited(arguments);

		// Lookup the wizard
		var args = {
				url: VvcUtils.getVvcUri() + '/rootservices',
				headers: {
					"Accept" : "application/rdf+xml",
					/*
					 * This header is needed to ensure the wizard still works in Firefox.  Without this header,
					 * Firefox will send an "If-None-Match" header which triggers an authentication when trying to
					 * access anything on VVC.  What we were seeing is authentication requests (which were failing) 
					 * when trying to access the root services doc.  See WI 37509 
					 */
					"If-Match" : "*"	 
				},
				handleAs: "text",
		};
		var deferred = jazz.client.xhrGet(args);
		
		deferred.then(dojo.hitch(this, function(xmlDoc){
			var rootServices = new com.ibm.team.vvc.shared.oslc.web.utils.RootServices(dojox.xml.DomParser.parse(xmlDoc));
			var resolvedWizardUri = rootServices.getUri("http://jazz.net/ns/vvc#", "addConfigurationWizard");
											
			var wizardUriParams;
			
			if (!this.createOnly) {
				wizardUriParams = dojo.string.substitute(this._wizardUrlDataTemplate, {
					configurationUri: ContextUtils.getCurrentContext() ? ContextUtils.getCurrentContext().getUri() : null,
					type: this.type,
				});
			} else {
				wizardUriParams = this._wizardUrlNoDataTemplate;
			}
			
			if(djConfig.isDebug){
				wizardUriParams = "debug=true&" + wizardUriParams;
			}
		
			// Build up the source URI for the iFrame
			this.iframe.src = resolvedWizardUri + "?" +  wizardUriParams;
			
			// Listen for the response from the window
			if(dojo.isIE <= 8){
				this.iframe.attachEvent('onload', dojo.hitch(this, this._onIFrameLoad));
				window.attachEvent('onmessage', dojo.hitch(this, this._handleResponse));
				dojo.attr(this.iframe, "frameBorder", 0);
			} else {
				this.iframe.onload = dojo.hitch(this, this._onIFrameLoad);
				window.addEventListener('message', dojo.hitch(this, this._handleResponse), false);
			}
			this.show();
		}));
	},
	
	_onIFrameLoad: function(){ 					
		/*
		 * Wait for the iFrame to load, then hide the loading message and show
		 * the iframe contents
		 */
		dojo.removeClass(this.loadingMessage, "show");
		dojo.removeClass(this.iframe, "hide");
		dojo.addClass(this.loadingMessage, "hide");
		dojo.addClass(this.iframe, "show");
		
		/*
		 * We have to let the browser event loop clear from the DOM
		 * modifications we did above before we can update the dialog position.
		 */
		setTimeout(dojo.hitch(this, function() {
			this.updatePosition();
			this.iframe.contentWindow.focus();
		}), 100);
	},
	
	_handleResponse: function(response){
		var oslcResponse = "oslc-response:";
		var oslcResults = "oslc:results";
		var oslcMessage = "oslc:message";
		var oslc1Create = "oslc_vc.create";
		var oslc2Create = "oslc.create";
		var oslcSize = "oslc:size";
		var windowResize = "window.size";

		// Only respond to OSLC messages
		if(response.data.indexOf(oslcResponse) === 0){
			var data = response.data.substring(oslcResponse.length);
			var json = dojo.fromJson(data);
			var message = json[oslcMessage];
			
			if(message === oslc1Create || message === oslc2Create){
				// Double encoding to make it easier to transport the data
				this._handleOslcResponse(dojo.fromJson(json[oslcResults]));
			} else if(message === windowResize){
				this._handleWindowresizeResponse(json[oslcSize]);
			}
		}
	},
	
	// Handles an OSLC response
	_handleOslcResponse: function(json){
		this._dialog.close();
		if(json){
			if(json.load && this.loadConfiguration){
				// Load the configuration
				this.loadConfiguration(json.load);
			} else if(json.add && this.addConfiguration){
				// Only add it to the recently used configurations list
				this.addConfiguration(json.add);
				this._dialog.destroyRecursive();
			}
		}
	},
	
	//hooks
	loadConfiguration: null,
	addConfiguration: null,
	
	// Handles a windowresize response
	_handleWindowresizeResponse: function(json){
		
		if(json.width){
			if(dojo.isIE <= 8){
				dojo.style(this.iframe, "width", (json.width - 10) + "px");
			} else {
				dojo.style(this.iframe, "width", json.width + "px");
			}
		}
		
		if(json.height){
			if(dojo.isIE <= 8){
				dojo.style(this.iframe, "height", (json.height + 20) + "px");
			} else {
				dojo.style(this.iframe, "height", json.height + "px");
			}
		}
	},
	
	getDialogArguments: function(){
		var icon;
		if (this.createOnly) {
			icon = dojo.moduleUrl("com.ibm.team.vvc.shared.web", "graphics/globalconfiguration_new.gif");
		} else {
			icon = dojo.moduleUrl("com.ibm.team.vvc.shared.web", "graphics/globalconfiguration_add.gif");
		}
		
		return {
			contentNode: this.containerNode,
			primaryTitle: (this.createOnly) ? this.messages.createGlobalConfigurationDialogTitle : this.messages.addToGlobalConfigurationDialogTitle,
			//secondaryTitle: "Secondary title here",
			iconPath: icon,
			width: 600,
			modal: true
		};
	},
});

})();