/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corporation 2013, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function GlobalConfigurationUtils() {	
dojo.provide("com.ibm.team.vvc.shared.web.ui.configurations.GlobalConfigurationUtils");

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;

var CONFIGURATION_PARAMETER_NAME = "oslc.configuration";
//	future consideration: previous configuration hint?
//	var PREVIOUS_CONFIGURATION_PARAMETER_NAME = "oslc.previousConfiguration";

com.ibm.team.vvc.shared.web.ui.configurations.GlobalConfigurationUtils = {

	/**
	 * Returns a URL to a resource given a concept URI, a configuration
	 * URI, and optionally an extended configuration URI
	 */
	getCustomLink: function(conceptUri, configurationUri) {
		//conceptUri: https://servername:port/appName/path/to/resource
		//configurationUri: https://servername:port/vvc/configurations/22_02
		//future consideration: previousConfigurationUri: https://servername:port/vvc/configurations/44_04
		//return result: https://servername:port/appName/path/to/resource?oslc.configuration=https%3A//servername%3Aport/vvc/configurations/22_01 and eventually perhaps the previous configuration
		
		if (!conceptUri) {
			throw "conceptUri is missing";
		}
		
		if (!configurationUri) {
			throw "configurationUri for " + conceptUri + " is missing";
		}
		
		var params = {};

		var hashIndex = conceptUri.indexOf('#');
		var hashValue = null;
		if (hashIndex > 0) {
			hashValue = conceptUri.substring(hashIndex, conceptUri.length);  // Take the String including the #
			conceptUri = conceptUri.substring(0, hashIndex);
		}
		
		var queryIndex = conceptUri.indexOf('?');
		if (queryIndex > 0) {
			params = dojo.queryToObject(conceptUri.substring(queryIndex + 1), conceptUri.length); // Take the string after the ?
			conceptUri = conceptUri.substring(0, queryIndex);
		}
		
		params[CONFIGURATION_PARAMETER_NAME] = configurationUri;
//			params[PREVIOUS_CONFIGURATION_PARAMETER_NAME] = previousConfigurationUri;
		
		var result = conceptUri + "?" + dojo.objectToQuery(params);
		if (hashValue) {
			result += hashValue;
		}
		return result;
	},
	
	/**
	 * Returns a default URL to a resource given a concept URI
	 */
	getDefaultLink: function(conceptUri) {
		var context = ContextUtils.getExpandedOrCurrentContext();
		//if invalid values are passed in, the next method will be responsible for throwing the error
		return this.getCustomLink(conceptUri, context ? context.uri : null);
	},
	
	/**
	 * Returns URL to a resource given a concept URI in the current expanded context
	 */
	getExpandedContextLink: function(conceptUri) {
		var context = ContextUtils.getExpandedContext();
		//if invalid values are passed in, the next method will be responsible for throwing the error
		return this.getCustomLink(conceptUri, context ? context.uri : null);
	},
	
	/**
	 * Provisional method.  It would be more efficient to do this locally
	 * from within the consuming application using the JAF version cache.
	 * 
	 * Returns the working configuration that 
	 * explicitly contains a version of the resource specified by the
	 * componentUri parameter, where such a working configuration is
	 * contained in the dependencies of expandedConfiguration, or is the
	 * expandedConfiguration itself.
	 * 
	 * WARNING: This method will be removed because using the local JAF cache
	 * will be much more efficient.
	 * Please see https://jazz.net/jazz/web/projects/Jazz%20Foundation#action=com.ibm.team.workitem.viewWorkItem&id=288361
	 * and https://jazz.net/jazz/web/projects/Jazz%20Foundation#action=com.ibm.team.workitem.viewWorkItem&id=294243
	 * 
	 */
	//Example: var deferred = com.ibm.team.vvc.shared.web.ui.configurations.GlobalConfigurationUtils.getConfigurationByComponentUri("https://servername:port/vvc/spaces/6_proj2", "https://servername:port/vvc/configurations/30_proj1");
	//		deferred.then(function(value) { where value is the URI of the configuration that contains it
	//		});
	getConfigurationByComponentUri: function(componentUri, expandedConfigurationUri) {
		if (!componentUri) {
			throw "componentUri is missing";
		}
		
		if (!expandedConfigurationUri) {
			throw "expandedConfigurationUri for " + componentUri + " is missing";
		}			
		
		var query = {
			"componentUri" : componentUri,
			"configurationUri" : expandedConfigurationUri
		};
		return this._getConfiguration(query);
	},
	
	/**
	 * Provisional method.  It would be more efficient to do this locally
	 * from within the consuming application using the JAF version cache.
	 * 
	 * Returns the working configuration that 
	 * explicitly contains a version of the resource specified by the
	 * conceptResourceUri parameter, where such a working configuration is
	 * contained in the dependencies of expandedConfiguration, or is the
	 * expandedConfiguration itself.
	 * 
	 * WARNING: This method will be removed because using the local JAF cache
	 * will be much more efficient.  Please use the
	 * getConfigurationByComponentUri() if possible.
	 * Please see https://jazz.net/jazz/web/projects/Jazz%20Foundation#action=com.ibm.team.workitem.viewWorkItem&id=288361
	 * and https://jazz.net/jazz/web/projects/Jazz%20Foundation#action=com.ibm.team.workitem.viewWorkItem&id=294243
	 */
	//Example: var deferred = com.ibm.team.vvc.shared.web.ui.configurations.GlobalConfigurationUtils.getConfigurationByConceptUri("https://servername:port/yourapp/conceptResource", "https://servername:port/vvc/configurations/30_proj1");
	//		deferred.then(function(value) { where value is the URI of the configuration that contains it
	//		});
	getConfigurationByConceptUri: function(conceptUri, expandedConfigurationUri) {
		if (!conceptUri) {
			throw "conceptUri is missing";
		}
		
		if (!expandedConfigurationUri) {
			throw "expandedConfigurationUri for " + conceptUri + " is missing";
		}
		
		var query = {
			"conceptUri" : conceptUri,
			"configurationUri" : expandedConfigurationUri
		};
		return this._getConfiguration(query);
	},
	
	_getConfiguration: function(query) {
		var args = {
				url: VvcUtils.getVvcUri() + "/globalConfigurations/findConfiguration?" + dojo.objectToQuery(query),
				headers : {
					"Accept": "application/atom+json"
				},
				hasBody: false,
				handle: function(ignoredNullResult, ioArgs) {
					var result = ioArgs.xhr.getResponseHeader("Location");
					if (result) {
						result = result.split(", ");
					}
					return result;
				}
			};
		return jazz.client.xhrGet(args);
	},
	
	isGlobalConfigurationUrl: function(url) {
		return url.indexOf(VvcUtils.getVvcUri() + "/globalConfigurations/") === 0;
	},
	
	/**
	 * Checks if global configurations are enabled for this instance of VVC.
	 * 
	 * @return <code>true</code> or </code>false</code>
	 */
	areGlobalConfigurationsEnabled: function __areGlobalConfigurationsEnabled(sync){
		var configurationsEnabledDeferred = new dojo.Deferred(); 
			
		var rootServicesStringDef = jazz.client.xhrGet({
			url: VvcUtils.getVvcUri() + "/rootservices",
			handleAs: "text", //$NON-NLS-1$
			preventCache: true
		});
		
		rootServicesStringDef.then(dojo.hitch(this, function __onGetOwnRootServicesSuccess(rootServicesDoc){
			configurationsEnabledDeferred.resolve(rootServicesDoc.indexOf("oslc_config:cmServiceProviders") >= 0);
		}), dojo.hitch(this, function __onGetOwnRootServicesError(error){
			configurationsEnabledDeferred.reject(error);
		}));
		return configurationsEnabledDeferred
	}
};
}());