/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corporation 2009, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.team.vvc.shared.oslc.web.ui.dialogs.AddCALMLinksDialog");
    dojo.require("dijit._Widget");
    dojo.require("dojo.i18n");
    dojo.require("jazz.ui.Dialog");
    dojo.require("com.ibm.team.vvc.shared.oslc.web.ui.dialogs.ResourcePickerContent");
    dojo.require("com.ibm.team.vvc.shared.oslc.web.utils.CLMUtils");
    dojo.require("net.jazz.ajax.ui.PlatformUI");
    dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants");
    dojo.require("com.ibm.team.vvc.shared.web.ui.util.InfoContentUtils");

    dojo.requireLocalization("com.ibm.team.vvc.shared.oslc.web.ui.dialogs", "AddCALMLinksDialogMessages");

	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var CLMUtils = com.ibm.team.vvc.shared.oslc.web.utils.CLMUtils;
	var InfoContentUtils = com.ibm.team.vvc.shared.web.ui.util.InfoContentUtils;
    var namespaces= ["oslc_rm", "oslc_am","oslc_cm","oslc_qm"];

    dojo.declare("com.ibm.team.vvc.shared.oslc.web.ui.dialogs.AddCALMLinksDialog", [dijit._Widget], {

        messages: null,
        resourceUri: null,
        serversDict: null,
        deferred: null,
        content: null,
        acceptableLinkTypes: null,
        contextForImplicitLinks: null,
        consumerMode: false,

        constructor: function(params){
            this.messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.oslc.web.ui.dialogs", "AddCALMLinksDialogMessages");
            this.serversDict = {};
            if (!dojo.isFunction(params.onOk)) {
                throw new Error("Missing function for 'onOk' parameter"); //$NON-NLS-1$
            }
        },

        postCreate: function(){
        	
        	jazz.client.xhrGet({
        		url: net.jazz.ajax._contextRoot + "/rootservices",
        		handleAs: "text",
        		load: dojo.hitch(this, function(response){
        			var rootServices = new com.ibm.team.vvc.shared.oslc.web.utils.RootServices(dojox.xml.DomParser.parse(response));
                	var projectAreaUri = rootServices.getProjectAreaUri();
                    this.loadServceProviders(projectAreaUri, this.resourceUri);
        		}),
        		sync: true
        	});
        },
        _onClose: function(){
            this.floatingPane.close();
			this.destroy();
        },

        loadServceProviders: function(projectAreaUri, resourceUri){
            PlatformUI.getWorkbench().showMessage(this.messages.loadingMessage);

            if (this.consumerMode) {
				var authenticationDeferred = new dojo.Deferred();
			
				CLMUtils.authenticate(authenticationDeferred, true);

				authenticationDeferred.addCallback(this, dojo.hitch(this, function(result){
					this.serversDict = {};
					
					for (var i = 0; i < result.length; i++) {
						//we need to do this check because not all friends are
						//CM friends (e.g. some friends could be DM, VVC, or JTS) 
						if (result[i][0] && result[i][1]) {
							this.serversDict[result[i][1].title] = result[i][1].res;
						}
					}
					
                  CLMUtils.getProjectLinks(projectAreaUri, this.contextForImplicitLinks, null /* url */, this.acceptableLinkTypes, dojo.hitch(this, function(projectLinks) {
                  if (projectLinks.length > 0) {
                      this._initFloatingPane(projectLinks);
                  } else {
          			var msg = dojo.string.substitute(this.messages.noProjectLinksFoundConsumerMode, [ net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/admin#action=com.ibm.team.repository.admin.friends", jazz.app.context.get().uri ]);
                  	InfoContentUtils.showSimpleContentDialog(this.messages.noChangeManagementProjects, msg);
                  }
                  PlatformUI.getWorkbench().hideMessageArea();
                  }));
				}));
            } else { //vvc native mode
                var deferred2 = new dojo.Deferred();
                CLMUtils.getVVCLinks(deferred2);
                deferred2.addCallback(this, function(vvcLinks) {
                    if (vvcLinks.length > 0) {
                        this._initFloatingPane(vvcLinks);
                    } else {
                    	InfoContentUtils.showSimpleContentDialog(this.messages.noChangeManagementProjects, this.messages.noProjectLinksFoundVVCMode);
                    }
                    PlatformUI.getWorkbench().hideMessageArea();
                });
            }
            
        },

        _prepareResponseItems: function(response){
            var responseItems= [];
            for(var index;index<namespaces.length;index++){
                var namespace= namespaces[index];
                var curNamespace= namespaces[index]+ ":results";
                var newItems = response[curNamespace];
                if (newItems instanceof Array && newItems.length > 0) {
                    for (var i = 0; i < newItems.length; i++) {
                        var item = newItems[i];
                        responseItems.push({
                            url: item["rdf:resource"],
                            label: item[namespace + ":label"]
                        });
                    }
                }
            }
            return responseItems;
        },

        _onData: function(data){
            if (data.length !== 0) {
                var response = dojo.fromJson(data);
                var result = this._prepareResponseItems(response);
                if (result.length > 0) {
                    this.onOk(result);
                }
            }
            this.floatingPane.close();
        },

        _initFloatingPane: function(projectLinks){
            var self = this;
            var onOk = function(links){
                self.onOk(links);
                self._onClose();
            };
            var onCancel = dojo.hitch(this, this._onClose);
            var contentParams = {
                onOk: onOk,
                resourceUri: this.resourceUri,
                onData: dojo.hitch(this, this._onData),
                serversDict: this.serversDict,
                onCancel: onCancel,
                projects: projectLinks,
				data: this.item
            };

           this.content= new com.ibm.team.vvc.shared.oslc.web.ui.dialogs.ResourcePickerContent(contentParams);

           var floatingPaneParams = {
                onClose: this.onClose,
                contentNode: this.content.domNode,
                destroyContents: true
            };

            if (this.dialogTitle) {
                floatingPaneParams.primaryTitle = this.dialogTitle;
            }else{
                floatingPaneParams.primaryTitle = "";
            }
            if(this.dialogSecondaryTitle){
                floatingPaneParams.secondaryTitle = "- "+this.dialogSecondaryTitle;
            }else{
                floatingPaneParams.secondaryTitle = "";
            }
            this.floatingPane = new jazz.ui.Dialog(floatingPaneParams);
            if (this.content && this.content.setFocus){
                this.content.setFocus();
            }

            PlatformUI.getWorkbench().hideMessageArea();
        },

        destroy: function(){
            if(this.content){
                this.content.destroyRecursive();
                this.content= null;
            }
            this.inherited(arguments);
        }
    });
})();
