/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.team.vvc.shared.oslc.web.ui.dialogs.RemoteItemPicker");
    
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    
    dojo.require("jazz.ui.RemoteChooserContainer");
    dojo.require("jazz.ui.Dialog");
    
    var LABEL_NAMESPACES = ["oslc:label", "oslc_cm:label", "http://www.w3.org/2000/01/rdf-schema#label"];
    var URL_NAMESPACES = ["rdf:resource", "http://www.w3.org/1999/02/22-rdf-syntax-ns#resource"];
    
    dojo.declare("com.ibm.team.vvc.shared.oslc.web.ui.dialogs.RemoteItemPicker", [dijit._Widget], {
    
        dialogContent: null,
        urlNamespace: null,
        labelNamespace: null,
        
        constructor: function(args){
            if (!args.dialogTitle) 
                throw new Error("'dialogTitle' should be provided");
            if (!args.namespace) 
                throw new Error("'namespace' should be provided");
            if (!args.pickerURL) 
                throw new Error("'pickerURL' should be provided");
            if (!dojo.isFunction(args.onOk)) 
                throw new Error("'onOk' should be provided");
        },
        
        postCreate: function(){
            var url = new dojo._Url(this.pickerURL);
            var server = url.authority;
            
            this._initContentWidget();
        },
        _onData: function(data){
            if (data.length != 0) {
                var response = dojo.fromJson(data);
                var newItems = response["oslc:results"];
                if (!newItems) {
                    newItems = response["oslc"];
                }
                else if (!newItems) {
                	newItems = response[this.namespace + ":results"];
                    if (!newItems) {
                        newItems = response[this.namespace];
                    }
                }

                if (newItems instanceof Array && newItems.length > 0) {
                    var result = [];
                    for (var i = 0; i < newItems.length; i++) {
                        var item = newItems[i];
                        result.push({
                            url: this.getUrl(item),
                            label: this.getLabel(item)
                        });
                    }
                    this.onOk(result);
                }
            }
            this.floatingPane.close();
        },
        
        getLabel: function(item){
            return this.getValue(item, LABEL_NAMESPACES);
        },
        
        getUrl: function(item){
            return this.getValue(item, URL_NAMESPACES);
        },
        
        getValue: function(item, array){
            for (var index = 0; index < array.length; index++) {
                var current = array[index];
                var value = item[current];
                if (value) {
                    return value;
                }
            }
        },
        
        _initContentWidget: function(){
            this.dialogContent = new jazz.ui.RemoteChooserContainer({  
            	remoteLocation: this.pickerURL,
            	mode: 'OSLC2',
            	onData: dojo.hitch(this, this._onData)
            });
            
            var floatingPaneParams = {
                primaryTitle: this.dialogTitle,
                secondaryTitle: this.secondaryTitle,
                width: this.width || '800px',
                height: this.height || '320px',
                onClose: this.onClose,
                contentNode: this.dialogContent.domNode
            };
            
            this.floatingPane = new jazz.ui.Dialog(floatingPaneParams);
        },

        onClose: function() {
        	this.floatingPane.close();
        	this.destroy();
        },

        destroy: function() {
            if (this.dialogContent) {
                this.dialogContent.destroyRecursive();
                this.dialogContent = null;
            }
            this.inherited(arguments);
        }
    });
    
})();
