/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corporation 2009, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("com.ibm.team.vvc.shared.oslc.web.ui.dialogs.ResourcePickerContent");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.client.xhr");
dojo.require("jazz.util.html");
dojo.require("dojox.xml.DomParser");
dojo.require("dijit.form.RadioButton");
dojo.require("com.ibm.team.vvc.shared.oslc.web.ui.dialogs.RemoteItemPicker");
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");
dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants");
dojo.require("com.ibm.team.vvc.shared.oslc.web.utils.BaseClient");
dojo.require("com.ibm.team.vvc.shared.oslc.web.utils.CLMUtils");

dojo.requireLocalization("com.ibm.team.vvc.shared.oslc.web.ui.dialogs", "ResourcePickerContentMessages");

var DomParser = dojox.xml.DomParser;
var RemoteItemPicker = com.ibm.team.vvc.shared.oslc.web.ui.dialogs.RemoteItemPicker;
var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;

var BaseClient= com.ibm.team.vvc.shared.oslc.web.utils.BaseClient;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

var PICKER_TYPES= {
	SELECTION: "Selection",
	CREATION: "Creation"
};

var HEADERS_MAPPING= {
	"http://open-services.net/ns/core#": {
								"Accept":"application/rdf+xml",
								"OSLC-Core-Version": "2.0"
								},
	"http://jazz.net/xmlns/prod/jazz/process/1.0/": {
								"Accept":"application/rdf+xml",
								"OSLC-Core-Version": "2.0"
								},
	"http://open-services.net/xmlns/rm/1.0/": {"Accept":"application/x-oslc-rm-service-description-1.0+xml"},
	"http://open-services.net/xmlns/qm/1.0/": {"Accept":"application/x-oslc-qm-service-description-1.0+xml"},
	"http://open-services.net/xmlns/am/1.0/": {"Accept":"application/x-oslc-am-service-description-1.0+xml"}
};

dojo.declare("com.ibm.team.vvc.shared.oslc.web.ui.dialogs.ResourcePickerContent", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl('com.ibm.team.vvc.shared.oslc.web', 'ui/dialogs/templates/ResourcePickerContent.html'),


    "class": "vvc-oslc-ui-ResourcePickerContent",

    widgetsInTemplate: true,

    projectAreaInput: null,
    selection: null,
    itemPicker: null,
    resourceUri: null,
    serversDict: null,
    projects: null,
    catalogDict: null,
    item: null,
    createdItem: null,
    selectServerOption: null,
    loadingOption: null,
    selectionPickersInfo: [{
        name: "DOORS",
        namespace: "http://open-services.net/xmlns/rm/1.0/web/results",
        width: "oslc_rm:hintWidth",
        height: "oslc_rm:hintHeight",
        widget: "oslc_rm:widget",
        protocol: "#oslc-windowName-1.0",
        path : {
            descriptor : ["oslc_rm:ServiceDescriptor"],
            categories : ["oslc_rm:requirementSelectionDialog","oslc_rm:requirementCollectionSelectionDialog"],
            name : "oslc_rm:Dialog"
        }
    },
    {
        name: "RDNG Item Selection",
        width: "oslc_rm:widthHint",
        height: "oslc_rm:heightHint",
        widget: "oslc_rm:widget",
        protocol: "#oslc-windowName-1.0",
        path : {
			descriptor : "oslc_rm:ServiceDescriptor",
			categories : ["oslc_rm:requirements"],
			name : "oslc_rm:SelectionDialog"
		}
    },
    {
        name: "RTC Item Selection",
        width: "oslc_cm:hintWidth",
        height: "oslc_cm:hintHeight",
        widget: "oslc_cm:url",
        protocol: "#oslc-windowName-1.0",
		path : {
			descriptor : "oslc_cm:ServiceDescriptor",
			categories : ["oslc_cm:changeRequests"],
			name : "oslc_cm:selectionDialog"
		}	
    },
    {
        name: "RMPS Item Selection",
        width: "oslc_am:hintWidth",
        height: "oslc_am:hintHeight",
        widget: "oslc_am:widget",
        protocol: "#oslc-windowName-1.0",
		path : {
			descriptor : "oslc_am:ServiceDescriptor",
			categories : [],
			name : "oslc_am:SelectionDialog"
		}
    },
    {
        name: "RMPS Item Selection (OSLC2.0) or RQM 3.0",
        width: "oslc:hintWidth",
        height: "oslc:hintHeight",
        widget: "oslc:dialog",
        protocol: "#oslc-core-windowName-1.0",
		path : {
			descriptor : ["oslc:ServiceProvider", "oslc:Service"],
			categories : ["oslc:selectionDialog"],
			name : "oslc:Dialog"
		}
	},
    {
        name: "RMPS Item Selection (OSLC2.0)",
        width: "oslc_am:hintWidth",
        height: "oslc_am:hintHeight",
        widget: "oslc_am:Dialog",
        protocol: "#oslc-windowName-1.0",
		path : {
			descriptor : ["oslc:ServiceProvider", "oslc:Service"],
			categories : ["oslc:selectionDialog"],
			name : "oslc:dialog"
		}
	}, {
        name: "RQM Item Selection",
        width: "oslc_qm:hintWidth",
        height: "oslc_qm:hintHeight",
        widget: "oslc_qm:url",
        protocol: "#oslc-windowName-1.0",
		path : {
			descriptor : "oslc_qm:ServiceDescriptor",
			categories : ["oslc_qm:testplans", "oslc_qm:testcases"],
			name : "oslc_qm:selectionDialog"
		}
    }]
	,

	creationPickersInfo: [{
        name: "DOORS",
        namespace: "http://open-services.net/xmlns/rm/1.0/web/results",
        width: "oslc_rm:hintWidth",
        height: "oslc_rm:hintHeight",
        widget: "oslc_rm:widget",
        protocol: "#oslc-windowName-1.0",
        path : {
			descriptor : ["oslc_rm:ServiceDescriptor"],
			categories : ["oslc_rm:requirementCreationDialog"],
			name : "oslc_rm:Dialog"
		}
    },
    {
        name: "RTC Item Creation",
        width: "oslc_cm:hintWidth",
        height: "oslc_cm:hintHeight",
        widget: "oslc_cm:url",
        protocol: "#oslc-windowName-1.0",
		path : {
			descriptor : "oslc_cm:ServiceDescriptor",
			categories : ["oslc_cm:changeRequests"],
			name : "oslc_cm:creationDialog"
		}
    },
    {
        name: "RQM Item Creation",
        width: "oslc_qm:hintWidth",
        height: "oslc_qm:hintHeight",
        widget: "oslc_qm:url",
        protocol: "#oslc-core-windowName-1.0",
		path : {
			descriptor : "oslc_qm:ServiceDescriptor",
			categories : ["oslc_qm:testplans", "oslc_qm:testcases"],
			name : "oslc_qm:creationDialog"
		}
    },
    {
        name: "RDNG Item Creation",
        width: "oslc_rm:widthHint",
        height: "oslc_rm:heightHint",
        widget: "oslc_rm:widget",
        protocol: "#oslc-core-windowName-1.0",
        path : {
			descriptor : "oslc_rm:ServiceDescriptor",
			categories : ["oslc_rm:requirements"],
			name : "oslc_rm:CreationDialog"
		}
    },
    {
        name: "RQM 3.0 Creation",
        width: "oslc:hintWidth",
        height: "oslc:hintHeight",
        widget: "oslc:dialog",
        protocol: "#oslc-core-windowName-1.0",
		path : {
			descriptor : ["oslc:ServiceProvider", "oslc:Service"],
			categories : ["oslc:creationDialog"],
			name : "oslc:Dialog"
		}
	}],

    NAMESPACES: {
        "oslc_rm": "http://open-services.net/xmlns/rm/1.0/",
        "oslc_cm": "http://open-services.net/xmlns/cm/1.0/",
        "oslc_am": "http://open-services.net/ns/am#",
		"oslc_qm": "http://open-services.net/xmlns/qm/1.0/",
        "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
        "calm": "http://jazz.net/xmlns/prod/jazz/calm/1.0/",
        "dc": "http://purl.org/dc/terms/",
        "oslc_disc": "http://open-services.net/xmlns/discovery/1.0/",
        "jp": "http://jazz.net/xmlns/prod/jazz/process/1.0/",
        "jp06": "http://jazz.net/xmlns/prod/jazz/process/0.6/"
    },

    isValidType: false,	/*Flag to determine if correct picker is currently chosen */

    friends: null,

    constructor: function(params){
        this.messages= dojo.i18n.getLocalization("com.ibm.team.vvc.shared.oslc.web.ui.dialogs", "ResourcePickerContentMessages");
        this.projects= [];
        this.friends= [];
        this.itemValue= params.data;
    },

    postCreate: function(){
        this.inherited(arguments);

        this._initFriendEntries();
        this._initProjectAreaCombo();
        this._initPickerSelectionCombo();

		this.item= this.data;

		if(this.item){
			this.link.innerHTML = jazz.util.html.escape(this.item.label);
		}
		this.okCmd.disabled = true;
		this.chooseExisting.set('disabled', true);
		this.chooseNew.set('disabled', true);

		this.selectExistingItem.value= this.messages.buttonBrowse;
		this.selectExistingItem.innerHTML= this.messages.buttonBrowse;

		this.selection= PICKER_TYPES.SELECTION;

		/*Both browse/create buttons must be disabled by default as no project is selected yet */
		this._disableAllSelections();
		
        if (this.projects.length === 1) {
        	this.projectSelect.options[1].selected = true;
        	this._onProjectAreaChange();
        	this.onProjectSelectDefaultOptionChange();
        }
    },

    _intializeLoadingOption: function(attachPoint){
        var loadingOption = document.createElement("option");
        loadingOption.appendChild(document.createTextNode(this.messages.loadingMessage));
        attachPoint.appendChild(loadingOption);
    },

    _intializeErrorOption: function(attachPoint, errorMessage){
        var errorOption = document.createElement("option");
        errorOption.appendChild(document.createTextNode(errorMessage));
        attachPoint.appendChild(errorOption);
    },

	_onChooseExisting: function(selected){
		if(selected){
			this.setLabelForSelection(this.link, this.item, this.messages.selectItem);
			this._onProjectAreaChange();
			this.selectExistingItem.value= this.messages.buttonBrowse;
			this.selectExistingItem.innerHTML= this.messages.buttonBrowse;
			this.selection= PICKER_TYPES.SELECTION;
			this._updateOKEnablement();
		}

	},

	_onCreateNew: function(selected){
		if(selected){
			this.setLabelForSelection(this.link, this.createdItem, this.messages.createItem);
			this._onProjectAreaChange();

			this.selectExistingItem.value= this.messages.buttonCreate;
			this.selectExistingItem.innerHTML= this.messages.buttonCreate;
			this.selection= PICKER_TYPES.CREATION;
			
			this._updateOKEnablement();
		}
	},

	setLabelForSelection: function(attachpoint, value, defaultValue){
		if(value){
			attachpoint.innerHTML = jazz.util.html.escape(value.label);
		}else{
			attachpoint.innerHTML= jazz.util.html.escape(defaultValue);
		}
	},

	_disableAllSelections: function(){
		this.selectExistingItem.disabled= true;
	},

	_enableAllSelections: function(force){
		if(!force&& this.chooseNew.get("checked")){
			this.selectExistingItem.disabled= this.createdItem!== null|| !(this.pickerSelect.selectedIndex>-1&& this.pickerSelect.options[this.pickerSelect.selectedIndex].value!== "-1");
		}else{
			this.selectExistingItem.disabled= false;
		}
	},

	_onTypeSelect: function(e){
		/*Empty Block */
	},

	_updateOKEnablement: function(){
		if(this.chooseNew.get("checked")){
			/*It is only enough to update enablement based on createdItem not null when a new item is cretaed. This is enough,
			 * could you think that we alreay selected an item has not been cretaed before? Do not tihnk so :) */
			this.okCmd.disabled= this.createdItem === null;
		} else {		
			if(this.chooseExisting.get("checked")){
				var isEnabled= this.items && (
									(!this.itemValue|| this.itemValue.url!==this.item.url));
				this.okCmd.disabled= !isEnabled;
			}else{
				this.okCmd.disabled= this.createdItem === null;
			}		
		}
	},

    _initPickerSelectionCombo: function(){
        var option = document.createElement("option");
        option.value = -1;
        option.appendChild(document.createTextNode(this.messages.selectType));
        /*No need to add any events to this default option, as it will be removed anyway after making a project selection */
        this.pickerSelect.appendChild(option);
        this.pickerSelect.disabled= true;
    },
    
    _initFriendEntries: function(){
        var form= [
            {name: "internal", root: ["jfs:internal"], node: "#text", typeFunction: function(val){return Boolean(val);}},
            {name: "rootService", root: ["jd:rootServices"],attribute: "rdf:resource"},
            {name: "title",root: ["dc:title"],node: "#text"}
        ];
        
      var cachedrootservices = {};
      var context = {
    	cachedRootservices: cachedrootservices,
    	contextRoot: VvcUtils.getVvcUri(),
    	form: form,
    	self: this
      };
      
      var d = jazz.client.xhrGet({
    	url: VvcUtils.getVvcUri() + "/rootservices",
    	handleAs: "text",
    	load: dojo.hitch(context, function(response){
    		this.cachedRootservices[this.contextRoot] = new com.ibm.team.vvc.shared.oslc.web.utils.RootServices(dojox.xml.DomParser.parse(response));
    	}),
//    	load: function(response, ioArgs) {
//    		com.ibm.team.vvc.shared.oslc.web.utils.RootServices = new com.ibm.team.vvc.shared.oslc.web.utils._RootServices(dojox.xml.DomParser.parse(response));
//    	},
    	sync: true
      });
      
      d.addCallback(dojo.hitch(context, function(response){
    	  var friendsPath = this.cachedRootservices[this.contextRoot].getUri(RootServicesNamespaces.JD_1_0, "friends");
          //var FRIENDS_PATH = (new com.ibm.team.vvc.shared.oslc.web.utils.RootServices()).getUri(RootServicesNamespaces.JD_1_0, "friends");
          
          var deferred= BaseClient.getResponse(friendsPath,{ path:["rdf:RDF","jd:Friend"], form: context.form}, {sync: true});

          deferred.addCallback(context.self, function(response){
              this.friends= response; 
          });
          
          deferred.addErrback(function(error){
              /* If this happened, this means that the server is not configured. Thus, it is kind of immposible to get here */
              console.error("Could not get friends list");
          });
      }));

    },

    _initProjectAreaCombo: function(){
        this._createProjectAreaCombo();
    },

    _createProjectAreaCombo: function(){
        /*The first time we change the selection from the info option (that tells the user to choose
         * a server), we remove this info option. We do this remove like that way to avoid redundant
         * check if this option is already removed or not on the method onServerAreaChange. This way
         * will save extra checks */
        this.selectProjectOptionListener = this.connect(this.projectSelect, "onchange", this.onProjectSelectDefaultOptionChange);
        
        this.connect(this.projectSelect, 'onclick', dojo.hitch(this, function(){
            /*Close the rich hover if we expanded the project selection */
            if (this._popup) {
                this._popup.close();
            }
        }));
        this.connect(this.projectSelect, 'onchange', dojo.hitch(this, function(){
            if (this._popup) {
				this._popup.destroyRecursive();
			}
            this._popup = new jazz.ui.HoverPopup({
				around: this.projectSelect,
				onOpen: dojo.hitch(this, function(){
					var idx = this.projectSelect.value;
					
					/*Safe check */
			        if (idx === -1) {
			            return;	
			        }

			        var bindValues= function(list){
			            var entries= [];
			            for(var index=0; index<list.length; index++){
			                var current= list[index];
			                if(current.value){
			                    entries.push(current);
			                }
			            }
			            return entries;
			        };

		            var project= this.projects[idx];
		            var serviceProvider= project.serviceProviders[0];
		            var entries= bindValues([{label: this.messages.projectName, value: serviceProvider.title},
                                             {label: this.messages.application, value: serviceProvider.applicationName},
                                             {label: this.messages.assoication, value: project.linkType},
                                             {label: this.messages.server, value: serviceProvider.details.authority}]);
		            /*Safe check */
		            if (entries) {
			            this._createPopupContents({
                            entires: entries
                        });
                    }
				}),
				open: dojo.hitch(this, function(target){
					var idx = this.projectSelect.value;

					/*Do not show rich hover when no selection is made already */
					if (idx === -1) {
						return;	
					}
					return this._popup.inherited("open", arguments);
				}),
				maxWidth: 800,
				maxHeight: dojo.isIE ? 570 : 550
			});
        }));

        /*We do not want to have any item selected in the server tab by default */
        this.selectProjectOption = document.createElement("option");
        this.selectProjectOption.value = -1;
        this.selectProjectOption.appendChild(document.createTextNode(this.messages.selectProject));
        
        this.connect(this.selectProjectOption, "onchange", this._onProjectAreaChange);
        
        this.projectSelect.appendChild(this.selectProjectOption);

        for (var p = 0; p < this.projects.length; p++) {
			var project = this.projects[p];
			var serviceProvider = project.serviceProviders[0];

			var option = document.createElement("option");
			option.value = p;
			option.appendChild(document.createTextNode(serviceProvider.presentationLabel));
            option.title = this.getLabelForProjectEntry(project);

            this.connect(option, 'onmouseover', dojo.hitch(this, function(e){
                /* Avoid showing the rich hover if the project selection is expanded */
                dojo.stopEvent(e);
            }));

            this.projectSelect.appendChild(option);
        }
    },

    getLabelForProjectEntry: function(project){
        var serviceProvider= project.serviceProviders[0];

        var friends= this.friends;
        getApplicationName= dojo.hitch(this, function(){
            var applicationName= null;
            var _Url= new dojo._Url(serviceProvider.serviceUrl);
            var context= this.getContext(_Url.path);
            var authority= _Url.authority;
            for(var index=0;index<friends.length; index++){
                var current= friends[index];
                var url= new dojo._Url(current.rootService);
                var urlContext= this.getContext(url.path);
                if (url.authority === authority && context === urlContext) {
                    var deferred= BaseClient.getResponse(current.rootService,{ path:["rdf:Description"], form: [{name: "title", root: ["dc:title"], node: "#text"}]}, {sync: true});
                    deferred.addCallback(function(response){
                        if(dojo.isArray(response)){
                            var first= response[0];
                            if(first){
                                var title= first.title;
                                if(title){
                                    applicationName = title;
                                    serviceProvider.applicationName = applicationName; 
                                }
                            }
                        }
                    });
                    deferred.addErrback(function(error){
                        console.log("Could not load:"+ current.rootService);
                    });
                    if(applicationName){
                        return applicationName + "; ";
                    }
                    return "";
                }
            }
            return "";
        });

        return serviceProvider.title + " ["+getApplicationName() + project.linkType +"; <"+ serviceProvider.details.authority+ "]";
    },

    onProjectSelectDefaultOptionChange: function(e){
        this.disconnect(this.selectProjectOptionListener);
        this.projectSelect.removeChild(this.selectProjectOption);
        delete this.selectProjectOption;
        delete this.selectProjectOptionListener;
        
        /*Now, we can enable the association type selection */
        this.pickerSelect.disabled= false;
        this.chooseExisting.set('disabled', false);
		this.chooseNew.set('disabled', false);
    },

    processError: function(error){
        var errorMessage;
        if(error){
            errorMessage= error.responseText;
        }
        if(!errorMessage){
            errorMessage= this.messages.projectsLoadError + " " + this.messages['projectsLoadError.explanation']+ " " + this.messages['projectsLoadError.useraction'];
        }
        while(this.pickerSelect.hasChildNodes()){
            this.pickerSelect.removeChild(this.pickerSelect.firstChild);
        }
        this._intializeErrorOption(this.pickerSelect, errorMessage);
    },

    findServiceURLInCatalog: function(catalogXML, projectAreaName){
        var jsdom = DomParser.parse(catalogXML);
        var res = this.getNode(jsdom, ["oslc_disc:ServiceProviderCatalog", "oslc_disc:entry", "oslc_disc:ServiceProvider[dc:title='" + projectAreaName + "']", "oslc_disc:services"]);
        if (res) {
			if(dojo.isArray(res)){
				res= res[0];
			}
            return res.getAttribute("rdf:resource");
        }
        return null;
    },

    loadDocument: function(documentURL, onSuccess, onFail){
        var authneticationUrl= com.ibm.team.vvc.shared.oslc.web.utils.CLMUtils.getAuthneticationUrl(this.serversDict, documentURL);

        var callBack= dojo.hitch(this, function(){
            deferred.addCallback(function(response){
                onSuccess(response);
                return response;
            });

            deferred.addErrback(function(error){
                onFail(error);
                return error;
            });
        });
        var documentCall= dojo.hitch(this, function(val){
            var deferred= jazz.client.xhrGet({url: documentURL ,headers: val});
            deferred.addCallback(function(response){
                onSuccess(response);
                return response;
            });

            deferred.addErrback(function(error){
                onFail(error);
                return error;
            });
        });
        if(authneticationUrl){
            var deferred = jazz.client.xhrGet({
                url: authneticationUrl,
                headers: {
                    "accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
                },
				handleAs: "xml" //$NON-NLS-1$
            });  
            deferred.addCallback(function(result){
                var childNodes= result.childNodes;
                for(var childNodesIndex=0; childNodesIndex<childNodes.length; childNodesIndex++){
                    var nodes= childNodes[childNodesIndex];
                    var attributes= nodes.attributes;
                    var value= [];

                    for (var att in HEADERS_MAPPING) {
                        for(var index=0; index<attributes.length; index++){
                            var attribute= attributes[index];
                            if (attribute.nodeValue === att) {
                                /* Estimating headers by priority. So, if more than one header exists, then we we will proceed to the last header as
                                 * its priority is higher. Therefore, we do not return directly */
                                return documentCall(HEADERS_MAPPING[att]);
                            }
                        }
                    }	
                }
                documentCall();
            });
            deferred.addErrback(function(error){ onFail(error); return error;});
        } else {
            documentCall();
        }
    },
    
	getContext: function(path){
		var segments= path.split("/");
		for(var index=0; index<segments.length; index++){
			var current= segments[index];
			if(current){
				return current;
			}
		}
    },

    _onProjectAreaChange: function(e){
        /*Important to disable the browser/create enablement. */
        this._disableAllSelections();

        var idx = this.projectSelect.value;

        if (idx === -1){
            return;	/*Required check as other places such as picker type selections make these calls, and the project area could be not selected already */
        }

        while(this.pickerSelect.hasChildNodes()) {
            this.pickerSelect.removeChild(this.pickerSelect.firstChild);
        }
        this._intializeLoadingOption(this.pickerSelect);
        this.pickerSelect.disabled= true;
		var project= this.projects[idx];

        /*ServiceProvider could be null if no project is already selected */
        if(project){
            /*At the moment, no document would have more than one service provider. Put we are getting ready for such scenario */
            var serviceUrl= project.serviceProviders[0].serviceUrl;
            if (serviceUrl) {
                /*Remove loading option */
				while(this.pickerSelect.hasChildNodes()){
			        this.pickerSelect.removeChild(this.pickerSelect.firstChild);
			    }

		        var type= this.chooseExisting.get("checked")?PICKER_TYPES.SELECTION: PICKER_TYPES.CREATION;

		        this.pickerDialogs = [];
		        this.loadDocument(serviceUrl,
                                  dojo.hitch(this, this.populatePickerOptions,
                                                   this.chooseExisting.get("checked") ? PICKER_TYPES.SELECTION : PICKER_TYPES.CREATION,
                                                   project.serviceProviders[0].isImplicit),
                                  dojo.hitch(this, this.processError));
		    }
        }
    },
    
    onFinalize: function(type){
        /*Let the user know if no picker exists for the current selected picker type) */
		if(this.pickerSelect.options.length<1){
			var option = document.createElement("option");
            option.value = -1;
            var infoMessage;
            switch (type) {
	            case PICKER_TYPES.SELECTION:
			        infoMessage= this.messages.noSelectionPicker;
			        break;
		        case PICKER_TYPES.CREATION:
		            infoMessage= this.messages.noCreationPicker;
		            break;
		        default:
		            infoMessage= "";	/*Cannot  be here, but just to have safe check */
	            break;
	        }
            option.appendChild(document.createTextNode(infoMessage));
            this.pickerSelect.appendChild(option);

            /*Although we already disable the buttons in prior but this is a safe call to ensure avoding a scenario with wrongly enabled selection */
            this._disableAllSelections();
		}else{
			this._enableAllSelections();
		}

		/*Disable the picker select if there is only one picker */
		if(this.pickerSelect.options.length<2){
			this.pickerSelect.disabled= true;
		}else{
			this.pickerSelect.disabled= false;
		}
    },

	populatePickerOptions: function(type, isImplicitAssociation, serviceXML){
		var jsdom = DomParser.parse(serviceXML);

		var dialogInfo = this.findDialogInfo(type, jsdom);
		
		var contains= dojo.hitch(this, function(info){
			for(var index=0; index<this.pickerDialogs.length; index++){
				var current= this.pickerDialogs[index];
				if (current.url === info.url){
					return true;
				}
			}
		});

		var res, option;
		if(dojo.isArray(dialogInfo)){
			for (var i = 0; i < dialogInfo.length; i++) {
                var entry = dialogInfo[i];
                if(contains(entry)){
                    continue;
                }
                res = entry.title;
                option = document.createElement("option");
                option.value = i;
                option.appendChild(document.createTextNode(res));
                this.pickerSelect.appendChild(option);
                this.pickerDialogs.push({
					title: res,
					heightHint: entry.heightHint,
					widthHint: entry.widthHint,
					url: entry.url,
					protocol: entry.protocol,
					namespace: entry.namespace,
					isImplicitAssociation: isImplicitAssociation
				});
            }
        } else {
            if(contains(dialogInfo)){
                return;
            }
            res = dialogInfo.title;
            option = document.createElement("option");
            option.value = 0;
            option.appendChild(document.createTextNode(res));
            this.pickerSelect.appendChild(option);
            this.pickerDialogs.push({
                title: res,
                heightHint: dialogInfo.heightHint,
                widthHint: dialogInfo.widthHint,
                url: dialogInfo.url,
                protocol: dialogInfo.protocol,
                namespace: dialogInfo.namespace,
                isImplicitAssociation: isImplicitAssociation
            });
        }
		this.onFinalize(type);
    },

    _onPickerButton: function(e){
		this.openDialog(this.pickerDialogs[this.pickerSelect.value], this.selection);
    },

    openDialog: function(dialogDescription, type) {
        var idx = -1;
        var self = this;
        var url = dialogDescription.url;
        if (dojo.config.isDebug && url.indexOf("debug=true") === -1) {
            idx = url.indexOf('?');
            if (idx !== -1) {
                url = url.substring(0, idx + 1) + 'debug=true&' + url.substring(idx + 1);
            } else {
                url = url + '?debug=true';
            }
        }
        if (dialogDescription.isImplicitAssociation) {
            // Internal call, we have to add invocation context parameters
            idx = url.indexOf("?") + 1;
            if (idx > 0) {
                var query = url.substring(idx, url.length);
                var queryObject = dojo.queryToObject(query);
                queryObject.context = jazz.app.context.get().baseline;
                queryObject.resourceUri = this.resourceUri;
                queryObject.userUri = dojo.getObject("jazz.app.currentApplication.config.user.uri");
                url = url.substring(0, idx) + dojo.objectToQuery(queryObject);
            }
        }
        var params = {
            dialogTitle: dialogDescription.title,
            secondaryTitle: "",
            width: dialogDescription.widthHint,
            height: dialogDescription.heightHint,
            namespace: dialogDescription.namespace,
            protocol: dialogDescription.protocol,
            parentElement: this.domNode,
            onOk: self.setResults.bind(self, type),
            onClose: function(){
            },
            pickerURL: url
        };
        this.itemPicker= new RemoteItemPicker(params);
    },

    setResults: function(type, results){
        if (results.length > 0) {
            var assignees = results;
            var label;
            
            if (type === PICKER_TYPES.SELECTION) {
                this.items = assignees;
            }else{
                this.createdItems = assignees;
            }

            var list = dojo.create("ul", {
                className: "linksList"
            });
            results.forEach(function(result) {
                dojo.create("li", {
                    innerHTML: jazz.util.html.escape(this._getLabelFromResult(result))
                }, list);
            }, this);
            dojo.place(list, this.link, "only");
            this._updateOKEnablement();
        }
    },

    /**
     * The WI summary object returned sometimes has the title as a string and other times the title is a property of an object
     * @param  {Object} result a WI summary object to be shown in the dialog
     * @return {String}        The label for the WI.
     */
    _getLabelFromResult: function(result) {
        // TODO should result.label actually be an object?
        // Full search is returning the object in this way from SearchResults.js::createColumnObject
        // Need to verify that this is actually expected
        if (typeof result.label === "object" && result.label.label) {
            return result.label.label;
        } else {
            return result.label;
        }
    },

    reset: function(){
        if(this.okCmd){
			this.okCmd.disabled = true;
        }
    },

    _handleOk: function(){
        var assignees;
        if (this.selection === PICKER_TYPES.SELECTION){
            assignees = this.items;
        }else{
            assignees = this.createdItems;
        }
        if(!assignees){
			assignees = this.data;
		}
		if(!assignees){
			assignees= [];
		}
        this.onOk(assignees);
        this.reset();
    },

    _handleCancel: function(){
        this.reset();
		this.destroy();
		this.onCancel();
    },

    destroy: function(){
		if(this.itemPicker){
			this.itemPicker.destroyRecursive();
			this.itemPicker= null;
		}
		this.inherited(arguments);
    },

    _getAttribute: function(node, name){
        var s = name.split(':', 2);
        if (s.length === 1) {
            return node.getAttribute(name);
        }
        else {
            var ns = this.NAMESPACES[s[0]];
            return node.getAttributeNS(s[1], ns);
        }
    },

    getNode: function(xmlNode, pathSegments){
        if (pathSegments.length === 0) {
            return xmlNode;
        }

        var first = pathSegments[0];
        var segments = pathSegments.slice(1);
        var m1 = first.match(/([\d\w-_:]+)\[(@?[\d\w-_:]+)='([^']+)'\]/);
        if (!m1) {
            var nodes = this._getElementByTagName(xmlNode, first);
			if (segments.length === 0 && nodes){
				return nodes;
			}
			var all =[];
            for (var i = 0; i < nodes.length; i++) {
                var res = this.getNode(nodes[i], dojo.clone(segments));
                if (dojo.isArray(res)){
                    all= all.concat(res);
                }else{
                    all.push(res);
                }
                    
            }
            return all;
        }
        var elementName = m1[1];
        var matchName = m1[2];
        var matchVal = m1[3];

        var isAttribMatch = matchName.charAt(0) === '@';
        if (isAttribMatch) {
            matchName = matchName.substring(1);
        }

        var nodes = this._getElementByTagName(xmlNode, elementName);
        for (var i = 0; i < nodes.length; i++) {
            var node = nodes[i];
            if (isAttribMatch) {
                if (this._getAttribute(node, matchName) === matchVal) {
                    return this.getNode(node, segments);
                }
            } else {
                var innerNodes = this._getElementByTagName(node, matchName);
                if (innerNodes.length === 1 && this._evalNodeText(innerNodes[0]) === matchVal) {
                    return this.getNode(node, segments);
                }
            }
        }
        return null;
    },

    _getElementByTagName: function(node, name){
        return node.getElementsByTagName(name);
    },

    _fillDialogInfo: function(node, dialogDescription, selection) {
        var url = this.getNodeOrAttributeText(node, [dialogDescription.widget])[0];
        if (!url) {
            url = node.getAttribute(dialogDescription.widget);
            if (!url) {
                return null;
            }
        }

        if (dojo.config.isDebug && url.indexOf("debug=true") === -1) {
            var idx = url.indexOf('?');
            if (idx !== -1) {
                url = url.substring(0, idx + 1) + 'debug=true&' + url.substring(idx + 1);
            } else {
                url = url + '?debug=true';
            }
        }

        var title;
        for (var ci = 0; ci < node.childNodes.length; ci++) {
            var child = node.childNodes[ci];
            if (child.baseName === ProcessConstants.XMLELEMENT_TITLE || child.localName === ProcessConstants.XMLELEMENT_TITLE) {
                title = (child.childNodes && child.childNodes.length > 0) ? child.childNodes[0].nodeValue : child.nodeValue;
                break;
            }
        }
        if (!title) {
           title= this.getNodeOrAttributeText(node, [ProcessConstants.XMLELEMENT_TITLE])[0];
        }

        var widthHint = node.getAttribute(dialogDescription.width);
        if (!widthHint) {
            widthHint= this.getNodeOrAttributeText(node, [dialogDescription.width])[0];
        }

        var heightHint = node.getAttribute(dialogDescription.height);
        if (!heightHint) {
            heightHint= this.getNodeOrAttributeText(node, [dialogDescription.height])[0];
        }

        return {
            title: title,
            url: url,
            widthHint: widthHint,
            heightHint: heightHint,
            protocol: dialogDescription.protocol,
            namespace: dialogDescription.namespace ? dialogDescription.namespace : node.namespace
        };
    },

    fillDialogInfo: function(jsdom, path, dialogDescription, selection) {
        var node = this.getNode(jsdom, path);
        if (dojo.isArray(node)) {
            var dialogs= [];
            for (var index = 0; index < node.length; index++) {
                var dialog = this._fillDialogInfo(node[index], dialogDescription, selection);
                if (dialog) {
                    dialogs.push(dialog);
                }
            }
            return dialogs;
        } else if (node) {
            return this._fillDialogInfo(node, dialogDescription, selection);
        }
        return null;
	},

    findDialogInfo: function(type, jsdom){
        var self = this;
        var list= [];
        switch (type) {
            case PICKER_TYPES.SELECTION:
                list= this.selectionPickersInfo;
                break;
			case PICKER_TYPES.CREATION:
				list= this.creationPickersInfo;
			    break;
			default:
			    list= [];	/*Cannot  be here, but just to have safe check */
		    break;
	    }

		var getNameNode= function(node) {
			var nestedResults= self.fillDialogInfo(node, [name], dialogDescription, type === PICKER_TYPES.SELECTION);
			if(nestedResults){
				if(dojo.isArray(nestedResults)){
					dialogs= dialogs.concat(nestedResults);										
				}else{
					dialogs.push(nestedResults);
				}
			}
		};

        var dialogs = [];
        for (var i = 0; i < list.length; i++) {
            var dialogDescription = list[i];
			var name =  dialogDescription.path.name;
			var descriptor = dialogDescription.path.descriptor;
			var categories = dialogDescription.path.categories;
			
			if (categories && categories.length > 0) {
				for (var j = 0; j < categories.length; j++) {
					var newPath;
					if (dojo.isArray(descriptor)) {
						newPath= descriptor.concat([categories[j]]);
					} else {
						newPath = [descriptor, categories[j]];
					}	
					var results= this.getNode(jsdom, newPath);
					if (dojo.isArray(results)) {
						for (var index=0; index<results.length; index++) {
							getNameNode(results[index]);	
						}
					} else if (results) {
						getNameNode(results);								
					}
				}
			} else {
				var path;
				if(dojo.isArray(descriptor)){
					path= descriptor.concat(name);
				}else{
					path = [descriptor, name];
				}
				var results= this.fillDialogInfo(jsdom, path, dialogDescription, type === PICKER_TYPES.SELECTION);
				if (results) {
					if (dojo.isArray(results)) {
						dialogs= dialogs.concat(results);
					}
					else {
						dialogs.push(results);
					}
				}
			}
        }
        return dialogs;
    },

    getNodeOrAttributeText: function(jsdom, segments){
        var node = this.getNode(jsdom, segments);

        if (dojo.isArray(node)) {
            var evals=[];
            for(var index=0; index<node.length; index++){
                evals.push(this._evalNodeText(node[index]));
            }
            return evals;
        }

        return this._evalNodeText(node);
    },

    getNodeText: function(jsdom, segments){
        var node = this.getNode(jsdom, segments);
        if (node){
			if(dojo.isArray(node)){
				var nodes=[];
				for (var index=0; index<node.length;index++) {
					nodes.push(this._evalNodeText(node[index]));
				}
				return nodes;
			}
			return this._evalNodeText(node);
		}
        return null;
    },

    _evalNodeText: function(node){
        if (node.childNodes.length === 1 && node.childNodes[0].nodeName === '#text') {
            return node.childNodes[0].nodeValue;
        }

        return this._getAttribute(node, "rdf:resource");
    },
    
    _createPopupContents: function(entries) {
		if (this._popupContent) {
			this._popupContent.destroyRecursive();
		}
		this._popupContent = new com.ibm.team.vvc.shared.oslc.web.ui.dialogs.ResourcePickerContent._PopupContent(entries);
		this._popup.content = this._popupContent.domNode;
	}
});


dojo.declare("com.ibm.team.vvc.shared.oslc.web.ui.dialogs.ResourcePickerContent._PopupContent", [dijit._Widget, dijit._Templated], {
	
	templateString: '<table style="border-spacing: 0;">' +
				'<tbody dojoAttachPoint="containerNode"></tbody>' +
			'</table>',
	
	"class": "vvc-oslc-ui-ResourcePickerContent-popup",
	
	entires: null,
	
	postCreate: function() {
		this.inherited(arguments);
		this._createContents();
	},
	
	_createContents: function() {
		for(var index=0; index<this.entires.length; index++){
			tr = dojo.create("tr", null, this.containerNode);
			var entry= this.entires[index];
			var labelCell = dojo.doc.createElement("td");
			labelCell.appendChild(dojo.doc.createTextNode(entry.label));
			tr.appendChild(labelCell);
			
			var dataCell = dojo.doc.createElement("td", null, tr);
			dataCell.appendChild(dojo.doc.createTextNode(entry.value));
			tr.appendChild(dataCell);
		}
	}
});

})();
