/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
	
/**
 * Lightweight garbage collector for VVC.
 */
dojo.provide("com.ibm.team.vvc.shared.web.ui.gc.VvcGarbageCollector");
dojo.declare("com.ibm.team.vvc.shared.web.ui.gc.VvcGarbageCollector", [], {
	
	_disconnects: null,
	_destroys: null,
	
	constructor: function __VvcGarbageCollector(){
		this._disconnects = [];
		this._destroys = [];
	},
	
	/**
	 * Automatic trigger for the cleanup; just destroy this object
	 */
	destroy: function __destroy(){
		this.inherited(arguments);
		this.cleanup();
		this._disconnects = null;
		this._destroys = null;
	},
		
	/**
	 * <p>
	 * This collector will disconnect the handle.
	 * </p>
	 * <p>
	 * Dojo connections, established with dojo.connect must be cleaned up to avoid memory leaks.  dojo.connect returns a handle
	 * to the connection to make it easy to clean up the connection.
	 * </p>
	 * 
	 * @param handle The handle to the connection
	 * @return The handle, for easy method chaining.
	 */
	disconnects: function __disconnects(handle){
		this._disconnects.push(handle);
		return handle;
	},
	
	/**
	 * This collector will destroy the object.
	 * @param obj The object to destroy
	 * @return The object, for easy method chaining.s
	 */
	destroys: function __destroys(obj){
		this._destroys.push(obj);
		return obj;
	},
	
	/**
	 * Triggers the cleanup, handling all the disconnects and destroys for you, and destroys the arrays as well.
	 */
	cleanup: function __cleanup(){
		// Handle the disconnects
		dojo.forEach(this._disconnects, dojo.hitch(this, function __cleanupHandles(handle){
			dojo.disconnect(handle);
		}));		
		this._disconnects = [];
		
		// Handle the destroys
		dojo.forEach(this._destroys, dojo.hitch(this, function __cleanupObjects(obj){
			if(obj){				
				if(obj.destroyRecursive){
					obj.destroyRecursive();
				} else if(obj.destroy) {
					obj.destroy();
				} else {
					dojo.destroy(obj);
				}
			}
		}));
		this._destroys = [];
	},	
}); // end declare

})(); // end object