/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.shared.web.ui.history.HistoryWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");

dojo.require('dojo.i18n');
dojo.require("jazz.ui.toolbar.Separator");
dojo.require("jazz.ui.Toolbar");
dojo.require("com.ibm.team.vvc.shared.web.ui.history.HistoryTable");
dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");

dojo.require("jazz.app.ua.PageHoverHelp");

dojo.require("jazz.util.ResizeNotifier");

dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.history", "HistoryMessages");

(function ()
{
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var ContextUtils = 	com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	
	dojo.declare("com.ibm.team.vvc.shared.web.ui.history.HistoryWidget", [dijit._Widget, dijit._Templated], {
	  
		widgetsInTemplate: true,
		
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'ui/history/templates/HistoryWidget.html'),
	   
		constructor: function() {
			
			this.messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.history", "HistoryMessages");
			
			this.explorerUrl = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/history.gif');
			
			this._configurationContext = this._getConfigurationContext();
			
			dojo.subscribe("vvc.historyLastestVersionsTitle", dojo.hitch(this, function() 
			{
				this._historyHeading.innerHTML = dojo.string.substitute(this.messages.historyHeading, [ arguments[0], this._configurationContext.getTitle() ]);
			} ));
		},
		
		postCreate: function(){
			var self = this;
			
			this._constructToolbar();
			this._constructTable();
			this._initializeCSH();
		},
		
		_initializeCSH: function() {	
			this._ua = new jazz.app.ua.PageHoverHelp({uaKey: "com.ibm.team.vvc.shared.web.page_HistoryView001"});
			this._pageHeader.appendChild(this._ua.domNode);
			this._ua.domNode.style.margin = "0px 0px 0px 5px";	
	   	},
	   	
		_getConfigurationContext: function() {
			var contextUri = ContextUtils.getContextUri();
			var configurationContext = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
			configurationContext.load(contextUri);
			return configurationContext;
		},
		
		_constructToolbar:function()
		{
			if (!this._toolbar)
			{
				var toolbar = new jazz.ui.Toolbar();
				
				var refreshButton = jazz.ui.toolbar.Button({
					title: this.messages.refresh,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/refresh.gif'),
					onClick: dojo.hitch(this, function(){
						if (this._table)
							this._table.refresh();
					})
				});
				toolbar.addChild(refreshButton);
				
				this._toolbar = toolbar;
				this._toolbarDiv.appendChild(toolbar.domNode);
			}
		},
		
		_constructTable: function() {
			if (!this._table)
			{
				this._table = new com.ibm.team.vvc.shared.web.ui.history.HistoryTable({
					"displayErrorMessage" : dojo.hitch(this, this.displayErrorMessage),
					"hideErrorMessage" : dojo.hitch(this, this.hideErrorMessage)
				});
				
			    /* append the new table to the div */
			    this._mainDiv.appendChild(this._table.domNode);
		
			}
		},
	    
	    uninitialize: function()
		{
	    	if (this._table) {
				this._table.destroyRecursive();
				delete this._table;
			}
	    	if (this._toolbar) {
				this._toolbar.destroyRecursive();
				delete this._toolbar;
			}
		},
		
		displayErrorMessage: function(errorMessage) 
		{
			var mb = new jazz.ui.MessageBox();
			mb.placeAt(this._historyErrorMessages);
			mb.showMessage({summary: errorMessage, type: jazz.ui.MessageBox.STATUS_TYPES.ERROR});
			dojo.style(this._historyErrorMessages, {display: ""});
		},
		
		hideErrorMessage: function() {
			if (this._historyErrorMessages) {
				dojo.empty(this._historyErrorMessages);
				dojo.style(this._historyErrorMessages, {display: "none"});
			}
		},
	});   
	})();
})();