/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.ResourceEditorInput");

dojo.require("rmps.ui.editor.EditorInput");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("dojox.encoding.digests.MD5");

dojo.requireLocalization("rmps.ui.editor", "PageHelperMessages");

var messages = dojo.i18n.getLocalization("rmps.ui.editor", "PageHelperMessages");

var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;

dojo.declare("rmps.app.ui.internal.ResourceEditorInput", rmps.ui.editor.EditorInput, {
	 
	resourceInfo: null,
	resourceUri: null,
	uriResourcePart: null,
	uriFragmentPart: null,
	
	constructor: function(params) {
		this.setResourceUri(params.resourceUri);
	},
	
	setResourceUri: function(newUri) {
		this.resourceUri = newUri;
		this._params.resourceUri = newUri;
		
		// Divide up the URI into the resource portion and the fragment portion
		var uriParts = this.resourceUri.split("#");
		
		this.uriResourcePart  = uriParts[0];
		
		if (uriParts.length == 2) {
			this.uriFragmentPart = uriParts[1];
		} else {
			this.uriFragmentPart = null;
		}
	},
	
	setParams: function(params) {
		this.inherited(arguments);
		
		if (params.resourceUri && params.resourceUri != null) {
			this.setResourceUri(params.resourceUri);
		}
	},
	
	getId: function() {
		return dojox.encoding.digests.MD5(this.declaredClass + ',' + this.uriResourcePart);
	},
	
	isLoaded: function() {
		return this.resourceInfo != null;
	},
	
	unload: function() {
		delete this.resourceInfo;
	},
	
	load: function() {
		var deferred = ResourceInfoClient.getResourceInfo(this.uriResourcePart);
		deferred.addCallback(this, function(infos) {
			this.resourceInfo = infos[0];
			return this;
		});

		deferred.addErrback(this, function(error) {
			if (error.dojoType !== "cancel") {
				if (deferred.ioArgs && !dojo._isDocumentOk(deferred.ioArgs.xhr)) {
					rmps.core.error.createXhrError(deferred.ioArgs, messages.error_loading_action).dispatch();
				} else {
					rmps.core.error.createError(error, messages.error_loading_action).dispatch();
				}
			}
		});
		return deferred;
	},
	
	getIconUri: function() {
		return this.resourceInfo ? this.resourceInfo.iconUri : null;
	},
	
	getName: function() {
		return this.resourceInfo ? this.resourceInfo.shortName : null;
	},
	
	getPresentationName: function() {
		/*Safe check. */
		if(!this.resourceInfo){
			return null;
		}
		
		/*We are trying here to append the resource number to the resource name--> 1: Class1 */
		var resourceNumber= this.getResourceNumber();
		if(resourceNumber){
			var name= this.getName();
			if(!name){
				return resourceNumber;
			}
			return resourceNumber+ ": "+name;
		}
		
		/*Call for parent node in case the resource number is undefined */
		return this.inherited(arguments);
	},
	
	getTooltip: function() {
		return this.resourceInfo ? this.resourceInfo.longName : null;
	},
	
	getBasePresentationUri: function(state) {
		/*Make sure to use the current pageId for the editorInput so it will return
		 * a valid viewResourceUri */
		return rmps.app.core.util.getViewResourceUri(this.resourceUri, this.pageId);
	},
	
	getResourceUri: function(){
		return this.resourceUri;
	},
	
	getResourceNumber: function() {
		var number = this.uriResourcePart.match(/\/(\d+)$/);
		
		if (number != null && number[1]) {
			return number[1];
		}
		
		return null;
	}
});

})();
