/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.analysis                                       |
 *|                                                                        |
 *| Copyright IBM Corporation 2011.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
(function() {
dojo.provide("dm.app.analysis.ui.internal.application.AnalysisMenu");

/** This widget creates the analysis capability menu items.
 *  This widget should be extended by the analysis applications
 *  in order to add their menu items to the Analysis menu. Each
 *  such application should use the dojo aspect advise mechanism
 *  to extend the methods for adding browse and create menu items. 
 *  
 *  @author Kevin Cornell
 */

dojo.require("rmps.ui.editor.CapabilityComponentMenu");

dojo.require("dm.app.analysis.ui.internal.application.AnalysisActions");
dojo.require("dm.app.analysis.ui.internal.util.AnalysisIcons");

dojo.require("dojo.i18n");
dojo.requireLocalization("dm.app.analysis.ui.internal", "AnalysisMessages");
var messages  = dojo.i18n.getLocalization("dm.app.analysis.ui.internal", "AnalysisMessages");

var AnalysisActions = dm.app.analysis.ui.internal.application.AnalysisActions;
var AnalysisIcons   = dm.app.analysis.ui.internal.util.AnalysisIcons;


dojo.declare("dm.app.analysis.ui.internal.application.AnalysisMenu", rmps.ui.editor.CapabilityComponentMenu, {

	// private properties
	_section: null,
	
	
	/** Return the name of the analysis section being
	 *  configured. The return value will be one of the
	 *  constants:  AnalysisActions.SECTION_*.
	 */
	getCurrentSection: function() {
		return this._section;
	},
	
	/** Add the analysis capability menu entries. The default
	 *  welcome menu item is added and then the analysis
	 *  application extensions are called to add their items.
	 *  
	 *  There are two menu extension methods:
	 *		addAnalysisBrowseMenuItems()
	 *		addAnalysisCreateMenuItems()
	 *  
	 *  Each of these methods will be called multiple times, 
	 *  once for each analysis section so that the order of
	 *  the menu items is deterministic. The section name
	 *  is set before each call and each application extending
	 *  this page must only add its menu items when the current
	 *  section corresponds to their section (e.g., getCurrentSection()).
	 */
	addCapabilityComponentMenuItems: function(){

		// Welcome to Reports
		this.addItemByArgs({
			label: messages.welcomeToAnalysis,
			icon: AnalysisIcons.WELCOME,
			wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
			href: AnalysisActions.getActionUri(AnalysisActions.ACTION_WELCOME)
		});

		// Add the menu entries for the Browse list
		this.addTitle(messages.browseTitle);
		this._section = AnalysisActions.SECTION_REPORTS;
		this.addAnalysisBrowseMenuItems();
		this._section = AnalysisActions.SECTION_TRACEABILITYS;
		this.addAnalysisBrowseMenuItems();
		this._section = AnalysisActions.SECTION_IMPACTANALYSIS;
		this.addAnalysisBrowseMenuItems();
		this._section = AnalysisActions.SECTION_VALIDATION;
		this.addAnalysisBrowseMenuItems();

		// Add the menu entries for the Create list
		this.addTitle(messages.createTitle);
		this._section = AnalysisActions.SECTION_REPORTS;
		this.addAnalysisCreateMenuItems();
		this._section = AnalysisActions.SECTION_TRACEABILITYS;
		this.addAnalysisCreateMenuItems();
		this._section = AnalysisActions.SECTION_IMPACTANALYSIS;
		this.addAnalysisCreateMenuItems();
		this._section = AnalysisActions.SECTION_VALIDATION;
		this.addAnalysisCreateMenuItems();
		
		this._section = null;
		this.inherited(arguments);
	},
	
	/** These methods are called to enable analysis application
	 *  extensions to add their menu items to the "browse" list
	 *  and the "create" list in the analysis menu.
	 *  
	 *  The applications should extend these methods using the
	 *  dojo aspect advise. Browse menu items are added as follows
	 *  and create menu items are effectively the same, except for
	 *  the extension method name. In the application extension
	 *  class (e.g., MyMenuInitializer), add the method declaration
	 *  as follows:
	 *  
	 *  dojox.lang.aspect.advise(dm.app.analysis.ui.internal.application.AnalysisMenu, 
	 *						"addAnalysisBrowseMenuItems", 
	 *      {
	 *      after: function() {
	 *          var menu = dojox.lang.aspect.getContext().instance;
	 *          if (menu.getCurrentSection() === AnalysisActions.SECTION_MySection) {
	 *				menu.addItemByArgs({
	 *					label: messages.myActionName,
	 *					icon: URL of the action icon,
	 *					wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
	 *					href: AnalysisActions.getActionUri("myActionName")
	 *				});
	 *          }
	 *      }
	 *  });
	 *
	 *  In order to connect the application extension so that it can be called,
	 *  the following extension should be defined in the plugin.xml of the 
	 *  analysis application.
	 *  
	 *  <extension point="com.ibm.xtools.rmps.web.app.webUiInitializers">
	 *	  <initializer jsModule="path.of.class.MyMenuInitializer">
	 *		<classBinding jsClass="dm.app.analysis.ui.internal.application.AnalysisMenu" />
	 *	  </initializer>
	 *  </extension>
	 */
	addAnalysisBrowseMenuItems: function(){ /* extensions must implement this method */ },
	addAnalysisCreateMenuItems: function(){ /* extensions must implement this method */ },
	
	/** Return the ID of the analysis page. */
	getPageId: function(){
		return AnalysisActions.PAGE_ID;
	}
	
	
});

})();