/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.reports                                        |
 *|                                                                        |
 *| Copyright IBM Corporation 2011, 2013.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
dojo.provide("dm.app.reports.ui.internal.application.ReportMenuInitializer");

/** Initialize the Analysis menu by adding the Reports actions for
 *  browsing and creating report templates and configurations.
 *  
 *  @author Kevin Cornell
 */

dojo.require("dm.app.analysis.ui.internal.application.AnalysisActions");
dojo.require("dm.app.analysis.ui.internal.application.AnalysisMenu");

dojo.require("dm.app.reports.ui.internal.application.ReportActions");
dojo.require("dm.app.reports.ui.internal.util.Icons");

dojo.require("dojo.i18n");
dojo.requireLocalization("dm.app.reports.ui.internal", "ReportMessages");
var messages = dojo.i18n.getLocalization("dm.app.reports.ui.internal", "ReportMessages");

var AnalysisActions = dm.app.analysis.ui.internal.application.AnalysisActions;
var ReportActions = dm.app.reports.ui.internal.application.ReportActions;
var Icons  = dm.app.reports.ui.internal.util.Icons;

/** Add the reports "browse" menu items */
dojox.lang.aspect.advise(dm.app.analysis.ui.internal.application.AnalysisMenu, "addAnalysisBrowseMenuItems", {
	after: function() {
		var menu = dojox.lang.aspect.getContext().instance;

		// If this call is for Reports menu items to be added
		if (menu.getCurrentSection() === AnalysisActions.SECTION_REPORTS) {
			
			// Report Templates
			menu.addItemByArgs({
				label: messages.browseTemplates,
				icon: Icons.TEMPLATES,
				wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
				href: ReportActions.getActionUri(ReportActions.ACTION_VIEW_TEMPLATES)
			});
			
			// View Reports
			menu.addItemByArgs({
				label: messages.browseReports,
				icon: Icons.REPORTS,
				wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
				href: ReportActions.getActionUri(ReportActions.ACTION_VIEW_REPORTS)
			});
		}
	}
});


/** Add the reports "create" menu items */
dojox.lang.aspect.advise(dm.app.analysis.ui.internal.application.AnalysisMenu, "addAnalysisCreateMenuItems", {
	after: function() {
		var menu = dojox.lang.aspect.getContext().instance;
		
		// If this call is for report menu items to be added
		if (menu.getCurrentSection() === AnalysisActions.SECTION_REPORTS) {

			// Create Report Template
			menu.addItemByArgs({
				label: messages.createTemplate,
				icon: Icons.CREATE_TEMPLATE,
				wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
				href: ReportActions.getActionUri(ReportActions.ACTION_EDIT_TEMPLATE)
			});

			// Create Report
			menu.addItemByArgs({
				label: messages.createReport,
				icon: Icons.CREATE_REPORT,
				wrapHint: jazz.ui.internal.MegaMenu.wrapHint.NO_WRAP,			
				href: ReportActions.getActionUri(ReportActions.ACTION_CREATE_REPORT)
			});
		}
	}
});


})();