/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.baseline.BaselineContextPicker");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.DropDownButton");
dojo.require("dojo.html");
dojo.require("dijit.Menu");

dojo.require("jazz.ui.HoverPopup");

dojo.require("rmps.core.error");
dojo.require("rmps.core.user.UserClient");
dojo.require("rmps.core.baseline.Baseline");
dojo.require("rmps.core.baseline.BaselineClient");
dojo.require("rmps.core.util.BaselineUtil");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmps.core.util.IRmpsConstants");

dojo.requireLocalization("rmps.app.ui.internal.baseline", "BaselineSelectionMessages");

var BASELINES_PATH = net.jazz.ajax._contextRoot + "/baselines";
var BaselineClient= rmps.core.baseline.BaselineClient;
var BaselineUtil= rmps.core.util.BaselineUtil;
var IRmpsConstants= rmps.core.util.IRmpsConstants;
var HoverPopup = jazz.ui.HoverPopup;

dojo.declare("rmps.app.ui.internal.baseline.BaselineContextPicker", [dijit._Widget, dijit._Templated], {

	"class" : "rmps-app-baselineContextPicker",
	messages: dojo.i18n.getLocalization("rmps.app.ui.internal.baseline", "BaselineSelectionMessages"),
	widgetsInTemplate: true,
	templateString: '<span>' +
					'<button dojoType="dijit.form.DropDownButton" dojoAttachPoint="_button">' +
							'<div dojoType="dijit.Menu" dojoAttachPoint="_menu">' +
							'</div>' +
					'</button>' +
					'</span>',

	_deferred: null,
	_selectedBaseline: null,

	baseline: null, //the default selected baseline. If null, no baseline will be pre-selected
	projectId: null,
	/*Optional filter - completed, inprogress or pending*/
	baselineFilter: null,
	/*Optional - comparator to sort baselines*/
	baselineComparator: null,
	/*By default, don't include a tooltip*/
	showTooltip: false,

	_loadingItem: null,

    startup: function() {
    	this.inherited(arguments);
    },

	postCreate: function() {
		this._selectedBaseline= this.baseline;

		if (!this._selectedBaseline) {
			this._selectedBaseline = IRmpsConstants.CURRENT;
		}
		this._button.startup(); //very important!! This sets up the dropdown menu, and for some reason, is not always called automatically

		if (this._selectedBaseline != IRmpsConstants.CURRENT){
			var deferred = BaselineClient. getBaseline(this._selectedBaseline, this.projectId);
			deferred.addCallback(this, function (baseline) {
				var message = this.messages.created + " "
							  +dojo.date.locale.format(baseline.timestamp, {formatLength: "medium"});
		        var tooltip = new HoverPopup({
					around: this._button.domNode,
					html: message
		        });
			});
		}
		dojo.connect(this._menu, "onOpen", this, this.refresh);
		this.refresh();
	},

	/**
	 * Refreshes the list of baselines, keeping the current selection.
	 */
	 refresh: function() {
		if (this._deferred) {
			//if a refresh is currently in progress, just wait for it to finish
			return;
		}

		//remove existing options
		this._menu.focusedChild = null; //otherwise, on IE, the focusedChild remains set even though it has been destroyed
		this._menu.destroyDescendants();

		//add "Loading..." text
		if (!this._loadingItem) {
 			this._loadingItem = new dijit.MenuItem({label: this.messages.loading});
 		}

		this._menu.addChild(this._loadingItem);

		this._deferred = BaselineClient.getBaselines(this.baselineFilter, this.projectId);
		this._deferred.addCallback(this, function (baselines) {
			//remove "loading..." text
			this._menu.focusedChild = null;
			this._menu.removeChild(this._loadingItem);

			if (this.baselineComparator) {
				baselines.sort(this.baselineComparator);
			}

			var currentOption = this._addOption(this.messages.current, IRmpsConstants.CURRENT);

			for (var i = 0; i<baselines.length; i++) {
				var option = this._addOption(baselines[i].name, baselines[i].uri);

				if (this.showTooltip) {
					option.domNode.title = this.messages.created + " "
					+dojo.date.locale.format(baselines[i].timestamp, {formatLength: "medium"});
				}
			}

			//put focus on new first child
			this._menu.focus();

			//now try to reset the selection appropriately
			if (!this._setSelection(this._selectedBaseline)) {
				this._setButtonLabel("");
				rmps.core.error.create(this.messages.error_selection_does_not_exist).dispatch();
			}
		});
		this._deferred.addErrback(this, function(error) {
			//error fetching baselines; report it
			if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
				rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
			} else {
				rmps.core.error.createError(error).dispatch();
			}
		});
		this._deferred.addBoth(this, function() {
			delete this._deferred;
		});
	},

	/**
	 * Adds an option to the end of the dropdown list with the given label and value and returns the new option
	 */
	_addOption: function(label, value) {
		var item = new rmps.app.ui.internal.baseline._BaselineMenuItem({"label" : label, uri: value, iconClass: "selected-icon", onSelection: dojo.hitch(this, this._onChange)});
		this._menu.addChild(item);
		return item;
	},

	_setSelection: function(baselineUri) {
		this._selectedBaseline = baselineUri;
		var selected = false;
		var children = this._menu.getChildren();
		for (var i = 0; i<children.length; i++) {
			var currentChild = children[i];
			if (currentChild.uri == baselineUri) {
				this._setButtonLabel(currentChild.label);
				dojo.addClass(currentChild.domNode, "selected-baseline");
				selected = true;
			} else {
				if (dojo.hasClass(currentChild.domNode, "selected-baseline")) {
					dojo.removeClass(currentChild.domNode, "selected-baseline");
				}
			}

		}
		return selected;
	},

	_setButtonLabel: function(label) {
		this._button.attr("label", label);
	},

	_onChange: function(selected) {
		this._setSelection(selected);

		if (selected == IRmpsConstants.CURRENT) {
			selected = null;
		}

		BaselineUtil.setBaseline(selected);
	}

});

dojo.declare("rmps.app.ui.internal.baseline._BaselineMenuItem", dijit.MenuItem, {

	uri: null,
	onSelection: null,

	onClick: function() {
		this.onSelection(this.uri);
	}

});

})();
