/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.baseline.BaselineSelection");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.html");

dojo.require("jazz.app.context");

dojo.require("rmps.core.error");
dojo.require("rmps.core.user.UserClient");
dojo.require("rmps.core.baseline.Baseline");
dojo.require("rmps.core.baseline.BaselineClient");
dojo.require("rmps.core.streams.StreamClient");
dojo.require("rmps.core.util.BaselineUtil");
dojo.require("rmps.core.util.ContextUtils");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmps.core.util.IRmpsConstants");


dojo.requireLocalization("rmps.app.ui.internal.baseline", "BaselineSelectionMessages");

var BASELINES_PATH = net.jazz.ajax._contextRoot + "/baselines";
var BaselineClient= rmps.core.baseline.BaselineClient;
var StreamClient= rmps.core.streams.StreamClient;
var BaselineUtil= rmps.core.util.BaselineUtil;
var IRmpsConstants= rmps.core.util.IRmpsConstants;
var ContextUtils= rmps.core.util.ContextUtils;

/*
 *If someone would like to implemnt this class, they could provide an implmentation for handeling selection. This is by passing handleSelection as argument.
 * For example:
 * var myBaseline= new BaselineSelection({handleSelection: this.mySelectionHandleMethod})
 *
 * If they did not specify a handleSelection method, then, BaselineUtil.setBaseline will be invoked by default
 *
 */
dojo.declare("rmps.app.ui.internal.baseline.BaselineSelection", [dijit._Widget, dijit._Templated], {

	"class" : "rmps-app-baselineSelection",
	messages: dojo.i18n.getLocalization("rmps.app.ui.internal.baseline", "BaselineSelectionMessages"),
	widgetsInTemplate: true,
	templateString: '<div>' +
					'<select dojoAttachPoint="sRs" dojoAttachEvent="onchange: _onChange">' +
					'</select>' +
					'</div>',

	_deferred: null,
	_selectedBaseline: null,

	//mixin
	baseline: null, //the default selected baseline; if null, no baseline will be selected by default
	projectId: null,
	/*Optional filter - completed, inprogress or pending*/
	baselineFilter: null,
	/*Optional - comparator to sort baselines*/
	baselineComparator: null,

	handleSelection: null,

	/*By default, the baseline selection is not disabled */
	disabled: false,
	/*By default, don't include a tooltip*/
	showTooltip: false,

	constructor: function(params){
        if(params){
			if(!params.handleSelection){
				this.handleSelection= dojo.hitch(BaselineUtil/*We have to maintain the BaselineUtil as context */, BaselineUtil.setBaseline);
			}
		}
    },

	postCreate: function() {
		this._selectedBaseline= this.baseline;

		if (!this._selectedBaseline) {
			this._selectedBaseline = jazz.app.context.get().baseline;
		}
		this.refresh();
		this.sRs.disabled= this.disabled;
	},

	/**
	 * Refreshes the list of baselines, keeping the current selection.
	 */
	 refresh: function() {
		 
		var deferreds= [];
		var contexts= [];
		if (this._deferred) {
			//if a refresh is currently in progress, just wait for it to finish
			return;
		}

		//remove existing options
		for (var i = this.sRs.options.length; i >= 0 ; i--) {
			this.sRs.remove(i);
		}

		//add "Loading..." text
		this._addOption(this.messages.loading, null).selected = true;

		var contextType = ContextUtils.getContextType(this._selectedBaseline);		
		if ( contextType === rmps.core.util.ContextTypes.WORKSPACE){
			var deferredStream = StreamClient.getStream(this._selectedBaseline);
			deferredStream.addCallback(this, function (stream) {
				contexts.push(stream);
			});
			deferreds.push(deferredStream);			
		} else if (contextType === rmps.core.util.ContextTypes.SNAPSHOT){
			
		}
		
		var deferredSnapshot = BaselineClient.getBaselines(this.baselineFilter, this.projectId);
		deferredSnapshot.addCallback(this, function (baselines) {
			contexts= contexts.concat(baselines);
		});
		deferreds.push(deferredSnapshot);			
		
		this._deferred = new dojo.DeferredList(deferreds);
		this._deferred.addCallback(this, function () {
			//remove "loading..." text
			this.sRs.remove(0);

			if (this.baselineComparator) {
				contexts.sort(this.baselineComparator);
			}

			var option;
			for (var i = 0; i<contexts.length; i++) {
				option = this._addOption(contexts[i].name, contexts[i].uri);

				if (this.showTooltip) {
					option.title = this.messages.created + " "
					+dojo.date.locale.format(contexts[i].timestamp, {formatLength: "medium"});
				}
			}

			if (!this._setSelection(this._selectedBaseline)) {
				//add a blank entry to the top of the list to represent the bad selection, and select it
				option = dojo.create("option", { value : this._selectedBaseline });
				option.selected = true;
				try {
					this.sRs.add(option,this.sRs.options[0]); // standards compliant
				} catch(ex) {
					this.sRs.add(option,0); // IE only
				}
				rmps.core.error.create(this.messages.error_selection_does_not_exist).dispatch();
			}
		});
		this._deferred.addErrback(this, dojo.hitch(this, function(error) {
			//error fetching baselines; report it
			if (!dojo._isDocumentOk(this._deferred.ioArgs.xhr)) {
				rmps.core.error.createXhrError(this._deferred.ioArgs).dispatch();
			} else {
				rmps.core.error.createError(error).dispatch();
			}
		}));
		this._deferred.addBoth(this, function() {
			delete this._deferred;
		});
	},
	
	getSelection: function(){
		var selection= this._selectedBaseline;
		if(selection=== IRmpsConstants.CURRENT){
			/*If the baseline value is the current, then the baseline value should be null */
			return null;
		}
		return selection;
	},

	/**
	 * Adds an option to the end of the dropdown list with the given label and value and returns the new option
	 */
	_addOption: function(label, value) {
		var option = dojo.create("option", { "value" : value }, this.sRs, null);
		var text = dojo.doc.createTextNode(label);
		option.appendChild(text);
		return option;
	},

	_setSelection: function(baselineUri) {
		for (var i = 0; i< this.sRs.options.length; i++) {
		  if (this.sRs.options[i].value == baselineUri) {
		  	this.sRs.options[i].selected = true;
		  	return true;
		  }
		}
		return false;
	},

	_onChange: function(event) {
		var selected = event.target.options[event.target.selectedIndex].value;

		this._selectedBaseline = selected;
		if (selected == IRmpsConstants.CURRENT) {
			selected = null;
		}
		this.handleSelection(selected);
	}

});

})();
