/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.context.ContextSelection");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.html");

dojo.require("jazz.app.context");

dojo.require("rmps.core.error");
dojo.require("rmps.core.user.UserClient");
dojo.require("rmps.core.baseline.Baseline");
dojo.require("rmps.core.baseline.BaselineClient");
dojo.require("rmps.core.streams.StreamClient");
dojo.require("rmps.core.changesets.ChangesetClient");
dojo.require("rmps.core.util.ContextUtils");
dojo.require("rmps.core.changesets.Changeset");

dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");

dojo.requireLocalization("rmps.app.ui.internal.context", "ContextSelectionMessages");

var BaselineClient= rmps.core.baseline.BaselineClient;
var StreamClient= rmps.core.streams.StreamClient;
var ChangesetClient= rmps.core.changesets.ChangesetClient;
var ContextUtils= rmps.core.util.ContextUtils;
var Changeset = rmps.core.changesets.Changeset;

var VvcContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;

/*
 *If someone would like to implemnt this class, they could provide an implementation for handling selection. This is by passing handleSelection as argument.
 * For example:
 * var myContext= new ContextSelection({handleSelection: this.mySelectionHandleMethod})
 *
 * If they did not specify a handleSelection method, then, ContextUtil.setContext will be invoked by default
 *
 */
dojo.declare("rmps.app.ui.internal.context.ContextSelection", [dijit._Widget, dijit._Templated], {

	"class" : "rmps-app-contextSelection",
	messages: dojo.i18n.getLocalization("rmps.app.ui.internal.context", "ContextSelectionMessages"),
	widgetsInTemplate: true,
	templateString: '<div>' +
					'<select dojoAttachPoint="sRs" dojoAttachEvent="onchange: _onChange">' +
					'</select>' +
					'</div>',

	_deferred: null,
	_selectedContext: null,

	//mixin
	context: null, //the default selected context; if null, current context  will be selected by default
	projectId: null,
	/*Optional filter - completed, inprogress or pending*/
	contextFilter: null,
	/*Optional - comparator to sort contexts*/
	contextComparator: null,

	handleSelection: null,

	/*By default, the context selection is not disabled */
	disabled: false,
	/*By default, don't include a tooltip*/
	showTooltip: false,

	constructor: function(params){
        if(params){
			if(!params.handleSelection){
				this.handleSelection= dojo.hitch(ContextUtils/*We have to maintain the ContextUtil as context */, ContextUtils.setContext);
			}
		}
    },

	postCreate: function() {
		this._selectedContext= this.context;

		if (!this._selectedContext) {
			this._selectedContext = jazz.app.context.get().baseline;
		}
		this.refresh();
		this.sRs.disabled= this.disabled;
	},

	/**
	 * Refreshes the list of contexts, keeping the current selection.
	 */
	 refresh: function() {
		 
		var deferreds= [];
		var snapshots= [];
		var workspace;
		var changesetCtxt;
		var parent= null;
		var activeCtxt= this._selectedContext;
		var i;
		
		if (this._deferred) {
			//if a refresh is currently in progress, just wait for it to finish
			return;
		}

		//remove existing options
		for (i = this.sRs.options.length-1; i >= 0 ; i--) {
			this.sRs.remove(i);
		}

		//add "Loading..." text
		this._addOption(this.messages.loading, null).selected = true;
		var contextType = ContextUtils.getContextType(this._selectedContext);
		if (contextType === rmps.core.util.ContextTypes.CHANGESET){
			activeCtxt = jazz.app.context.get().baseline;
		} else if ( contextType === rmps.core.util.ContextTypes.SNAPSHOT){
			parent = true;
		}

		var deferredStream = StreamClient.getStream(activeCtxt, parent);
		deferredStream.addCallback(this, function (stream) {
			workspace= stream;
		});
		deferreds.push(deferredStream);			

		var deferredSnapshot = BaselineClient.getBaselines(this.baselineFilter, this.projectId);
		deferredSnapshot.addCallback(this, function (baselines) {
			snapshots= snapshots.concat(baselines);
		});
		deferreds.push(deferredSnapshot);	
		
		this._deferred = new dojo.DeferredList(deferreds);
		this._deferred.addCallback(this, function () {
			// getting the changesets (including not committed ones)
			var deferredChangeset = ChangesetClient.getChangesetsForUser(jazz.app.currentApplication.config.user.uri, true);
			deferredChangeset.addCallback(this, function (changesets) {			
				// Building the contexts list - only not committed changesets should be displayed
				var validChangesets = [];
				for (i = 0; i<changesets.length; i++) {
//					if (changesets[i].state != Changeset.States.COMMITTED){
					if (changesets[i].state != rmps.core.changesets.ChangesetStates.COMMITTED){					
						validChangesets.push(changesets[i]);
					}
				}				
				var contexts= validChangesets;
				contexts.push(workspace);
				var strSnapshots = workspace.snapshots;

				for (i = 0; i<snapshots.length; i++) {
					if (dojo.indexOf(strSnapshots, snapshots[i].uri)>-1){
						contexts.push(snapshots[i]);
					}
				}

				var selectedContextUri = this._selectedContext;
				if (!dojo.some(contexts, function(context) { return context.uri == selectedContextUri })) {
					var selectedContext_VVC = VvcContextUtils.getCurrentContext();
					var selectedContext = { name : selectedContext_VVC.getTitle(), uri : selectedContextUri };
					contexts.push(selectedContext);
				}

				//remove "loading..." text
				this.sRs.remove(0);

				if (this.contextComparator) {
					contexts.sort(this.contextComparator);
				}

				var option;
				for (i = 0; i<contexts.length; i++) {
					option = this._addOption(contexts[i].name, contexts[i].uri);

					if (this.showTooltip) {
						option.title = this.messages.created + " "
						+dojo.date.locale.format(contexts[i].timestamp, {formatLength: "medium"});
					}
				}

				if (!this._setSelection(this._selectedContext)) {
					//add a blank entry to the top of the list to represent the bad selection, and select it
					option = dojo.create("option", { value : this._selectedContext });
					option.selected = true;
					try {
						this.sRs.add(option,this.sRs.options[0]); // standards compliant
					} catch(ex) {
						this.sRs.add(option,0); // IE only
					}
					rmps.core.error.create(this.messages.error_selection_does_not_exist).dispatch();
				}
			});

		});
		this._deferred.addErrback(this, dojo.hitch(this, function(error) {
			//error fetching contexts; report it
			if (!dojo._isDocumentOk(this._deferred.ioArgs.xhr)) {
				rmps.core.error.createXhrError(this._deferred.ioArgs).dispatch();
			} else {
				rmps.core.error.createError(error).dispatch();
			}
		}));
		this._deferred.addBoth(this, function() {
			delete this._deferred;
		});
	},
	
	getSelection: function(){
		var selection= this._selectedContext;
		return selection;
	},

	/**
	 * Adds an option to the end of the dropdown list with the given label and value and returns the new option
	 */
	_addOption: function(label, value) {
		var option = dojo.create("option", { "value" : value }, this.sRs, null);
		var text = dojo.doc.createTextNode(label);
		option.appendChild(text);
		return option;
	},

	_setSelection: function(contextUri) {
		for (var i = 0; i< this.sRs.options.length; i++) {
		  if (this.sRs.options[i].value === contextUri) {
			  this.sRs.options[i].selected = true;
			  return true;
		  }
		}
		return false;
	},

	_onChange: function(event) {
		var selected = event.target.options[event.target.selectedIndex].value;

		this._selectedContext = selected;
		this.handleSelection(selected);
	}

});

})();
