/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.review.app.ui.internal.controls.ReviewProgressBar");

dojo.require("dijit.ProgressBar");
dojo.require("dijit._Widget");
dojo.require("dojo.i18n");
dojo.require("rmps.core.util.StringUtils");

dojo.requireLocalization("dm.review.app.ui.internal.controls", "ReviewProgressBarMessages");

var MIN_PROGRESS_BAR_WIDTH_IN_PIXELS = 110;
var PADDING_FOR_SIDES_OF_PROGRESS_LABEL_IN_PIXELS = 8;

/*
 * Shows a progress bar for the current review/participant progress in a review. Expect a perecent value, and show if the total results is not started,
 * in progress, or done
 * 
 */
dojo.declare("dm.review.app.ui.internal.controls.ReviewProgressBar", dijit.ProgressBar,{
	viewType: null,

	messages: null,

	buildRendering: function(){
		this.inherited("buildRendering", arguments);
		dojo.addClass(this.domNode, "rmps-ReviewProgressBar");
	},

	postMixInProperties: function() {
		this.messages = dojo.i18n.getLocalization("dm.review.app.ui.internal.controls", "ReviewProgressBarMessages");
		this.inherited("postMixInProperties", arguments);
	},

	report: function(percent){
		percent = dojo.number.format(percent,{ type: "percent", places: this.places, locale: this.lang });
		var message = null;
		switch (this.viewType) {
			case dm.review.app.ui.internal.controls.ReviewProgressBar.VIEW_TYPE.Completed:
				message = this.messages.percentCompleted;
				break;
			case dm.review.app.ui.internal.controls.ReviewProgressBar.VIEW_TYPE.Reviewed:
				message = this.messages.percentReviewed;
				break;
			default:
				message = "";
				break;
		}
		var labelToUse = dojo.string.substitute(message, [percent]);
		this._setWidthToAppropriateValue(labelToUse);
		return labelToUse;
	},

	_setWidthToAppropriateValue: function(label) {
		this.domNode.style.width = Math.max( rmps.core.util.StringUtils.calculateTextSize(label, this.domNode).width + 
								    PADDING_FOR_SIDES_OF_PROGRESS_LABEL_IN_PIXELS, MIN_PROGRESS_BAR_WIDTH_IN_PIXELS) + "px";
	}
	
});

})();

dm.review.app.ui.internal.controls.ReviewProgressBar.VIEW_TYPE = {
	Completed: "Completed",
	Reviewed: "Reviewed"
};
