/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.review.app.ui.internal.controls.ReviewResultsStatusBar");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("rmps.ui.SimpleImageDescription");
dojo.require("rmps.core.review.util.ReviewImageService");
dojo.require("rmps.core.review.util.ReviewTypes");

dojo.requireLocalization("dm.review.app.ui.internal.controls", "ReviewResultsStatusBarMessages");

var ReviewImageService= rmps.core.review.util.ReviewImageService;
var ReviewTypes= rmps.core.review.util.ReviewTypes;

/* 
 * Shows detailed progress description for a given resource/reviewer. Means giving the number of
 * not started/rejected/approved, etc items. 
 * 
 * In order to be used, e can call for either the method  renderResourceResults or setUserUri in order to show the results
 * as per resource or revieer
 * 
 *  
 */
dojo.declare("dm.review.app.ui.internal.controls.ReviewResultsStatusBar",[dijit._Widget, dijit._Templated],{
	
	templateString: '<div dojoAttachPoint="mainContainer">'+
					'	<div dojoAttachPoint="mainToolbar" dojoType="jazz.ui.SimpleToolbar"><div>' +
					'</div>',
	
	"class": "review-results-status-bar",
	
	widgetsInTemplate: true,
	messages: null,
	role: null,
	
	postMixInProperties: function() {
		this.inherited(arguments);
		this.messages = dojo.i18n.getLocalization("dm.review.app.ui.internal.controls", "ReviewResultsStatusBarMessages");
	},
	
	addDetails: function(approveValue, disApproveValue, notStartedValue){
		if (this.role && this.role == ReviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer){
			this.addItem(ReviewImageService.REVIEW, this.messages.review, approveValue);
			this.addItem(ReviewImageService.REVIEW_COMMENTS, this.messages.review_comments, disApproveValue);			
		} else {
			this.addItem(ReviewImageService.APPROVE, this.messages.approve, approveValue);
			this.addItem(ReviewImageService.DISAPROVE, this.messages.disapprove, disApproveValue);
		}
		this.addItem(ReviewImageService.NOT_STARTED, this.messages.notStarted, notStartedValue);
	},
	
	addItem: function(imgUrl, tooltip, label){
		this.mainToolbar.addWidget(new rmps.ui.SimpleImageDescription({
			imgUrl: imgUrl,
			tooltip: tooltip,
			label: label ,
			getRendering: function(){return rmps.ui.SimpleImageDescription.Rendering.RIGHT;}
		}));
	},
	
	clearAllActions: function(){
		while(this.mainToolbar.items.length>0){
			this.mainToolbar.removeItem(this.mainToolbar.items[0]);
		}
	},
	
	/*
	 * Render results for a given resource 
	 */
	renderResourceResults: function(review, resourceUri) {
		if(!review){
			return; 
		}
		this.clearAllActions();
		var resultDetails= review.getResourceResults(resourceUri).resultDetails;
		this.addDetails(resultDetails.numReviewed + resultDetails.numApproved, resultDetails.numDisapproved + resultDetails.numReviewedComments, resultDetails.numNotStarted);	
	},
	
	/*
	 * Render results for a given user 
	 */
	setUserUri: function(review, userUri) {
		if(!review){
			return; 
		}
		this.clearAllActions();
		var resultDetails= review.getUserResults(userUri, true).resultDetails;
		this.addDetails(resultDetails.numReviewed + resultDetails.numApproved, resultDetails.numDisapproved + resultDetails.numReviewedComments, resultDetails.numNotStarted);
	}

});

})();
