/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corporation 2009, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("rmps.app.ui.internal.dialogs.AddCALMLinksDialog");
    dojo.require("dijit._Widget");
    dojo.require("dojo.i18n");
    dojo.require("jazz.ui.Dialog");
    dojo.require("rmps.app.ui.internal.dialogs.ResourcePickerContent");
    dojo.require("rmps.app.ui.internal.util.CLMUtil");
    dojo.require("net.jazz.ajax.ui.PlatformUI");
    dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants");

    dojo.requireLocalization("rmps.app.ui.internal.dialogs", "AddCALMLinksDialogMessages");

	var PROJECT_AREA_PATH= rmps.core.RootServices.getUri(rmps.core.Namespaces.JP_0_6, "projectAreas");
	var ProcessEditorConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var CLMUtil = rmps.app.ui.internal.util.CLMUtil;
    var namespaces= ["oslc_rm", "oslc_am","oslc_cm","oslc_qm"];

    dojo.declare("rmps.app.ui.internal.dialogs.AddCALMLinksDialog", [dijit._Widget], {

        messages: null,
        resourceUri: null,
        serversDict: null,
        deferred: null,
        content: null,

        constructor: function(params){
            this.messages = dojo.i18n.getLocalization("rmps.app.ui.internal.dialogs", "AddCALMLinksDialogMessages");
            this.serversDict = {};
            if (!dojo.isFunction(params.onOk)) {
                throw new Error("Missing function for 'onOk' parameter"); //$NON-NLS-1$
            }
        },

        postCreate: function(){
            this.loadServceProviders(this.resourceUri);
        },
        _onClose: function(){
            this.floatingPane.close();
			this.destroy();
        },

        loadServceProviders: function(resourceUri){
            PlatformUI.getWorkbench().showMessage(this.messages.loadingMessage);

            var deferred = CLMUtil.getServiceProviders(resourceUri).addCallback(this, function(serversDict) {
                this.serversDict= serversDict;
                CLMUtil.getProjectLinks(null /* url */, dojo.hitch(this, function(projectLinks) {
                    if (projectLinks.length > 0) {
                        this._initFloatingPane(projectLinks);
                    } else {
                        CLMUtil.showNoProjectLinksFoundContentDialog();
                        PlatformUI.getWorkbench().hideMessageArea();
                    }
                }));
            });            
        },

        _prepareResponseItems: function(response){
            var responseItems= [];
            for(var index;index<namespaces.length;index++){
                var namespace= namespaces[index];
                var curNamespace= namespaces[index]+ ":results";
                var newItems = response[curNamespace];
                if (newItems instanceof Array && newItems.length > 0) {
                    for (var i = 0; i < newItems.length; i++) {
                        var item = newItems[i];
                        responseItems.push({
                            url: item["rdf:resource"],
                            label: item[namespace + ":label"]
                        });
                    }
                }
            }
            return responseItems;
        },

        _onData: function(data){
            if (data.length !== 0) {
                var response = dojo.fromJson(data);
                var result = this._prepareResponseItems(response);
                if (result.length > 0) {
                    this.onOk(result);
                }
            }
            this.floatingPane.close();
        },

        _initFloatingPane: function(projectLinks){
            var self = this;
            var onOk = function(links){
                self.onOk(links);
                self._onClose();
            };
            var onCancel = dojo.hitch(this, this._onClose);
            var contentParams = {
                onOk: onOk,
                resourceUri: this.resourceUri,
                onData: dojo.hitch(this, this._onData),
                serversDict: this.serversDict,
                onCancel: onCancel,
                projects: projectLinks,
				data: this.item
            };

           this.content= new rmps.app.ui.internal.dialogs.ResourcePickerContent(contentParams);

           var floatingPaneParams = {
                onClose: this.onClose,
                contentNode: this.content.domNode,
                destroyContents: true
            };

            if (this.dialogTitle) {
                floatingPaneParams.primaryTitle = this.dialogTitle;
            }else{
                floatingPaneParams.primaryTitle = "";
            }
            if(this.dialogSecondaryTitle){
                floatingPaneParams.secondaryTitle = "- "+this.dialogSecondaryTitle;
            }else{
                floatingPaneParams.secondaryTitle = "";
            }
            this.floatingPane = new jazz.ui.Dialog(floatingPaneParams);
            if (this.content && this.content.setFocus){
                this.content.setFocus();
            }

            PlatformUI.getWorkbench().hideMessageArea();
        },

        destroy: function(){
            if(this.content){
                this.content.destroyRecursive();
                this.content= null;
            }
            this.inherited(arguments);
        }
    });
})();
