/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("rmps.app.ui.internal.dialogs.AddExternalLinksDialog");

    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("dojo.i18n");
    dojo.require("jazz.ui.Dialog");
    dojo.require("dojox.validate.regexp");
    dojo.require("rmps.ui.ValidationText");
    dojo.require("rmps.core.util.ILinksConstants");

    dojo.requireLocalization("rmps.app.ui.internal.dialogs", "AddExternalLinksDialogMessages");

    var KEY_RETURN= 13;
    var KEY_ENTER= 14;
    var ILinksConstants = rmps.core.util.ILinksConstants;

    dojo.declare("rmps.app.ui.internal.dialogs.AddExternalLinksDialog", [dijit._Widget], {

        constructor: function(params){
            if (!dojo.isFunction(params.onOk)) {
                throw new Error("Missing function for 'onOk' parameter"); //$NON-NLS-1$
            }
            this.messages= dojo.i18n.getLocalization("rmps.app.ui.internal.dialogs", "AddExternalLinksDialogMessages");
        },

        postCreate: function(){
            this._initContentWidget();
            this._initFloatingPane();
        },

        _onClose: function(){
            this.floatingPane.close();
        },

        _initContentWidget: function(){
            var self = this;
            var onOk = function(args){
                self.onOk(args);
                dojo.hitch(self, self._onClose)();
            };

            var onCancel = dojo.hitch(this, this._onClose);
            var contentParams = {
                onOk: onOk,
                onCancel: onCancel,
				item: this.item,
				dialog: this
            };
            this.dialogContent = new rmps.app.ui.internal.common.DialogContent(contentParams);
        },

        _initFloatingPane: function(){
            var floatingPaneParams = {
                width: "420px",
                onClose: this.onClose,
                contentNode: this.dialogContent.domNode
            };
            if (this.dialogTitle) {
                floatingPaneParams.primaryTitle = this.dialogTitle;
            }else{
                floatingPaneParams.primaryTitle = "";
            }
            if(this.dialogSecondaryTitle){
                floatingPaneParams.secondaryTitle = "- "+ this.dialogSecondaryTitle;
            }else{
                floatingPaneParams.secondaryTitle = "";
            }

            this.floatingPane = new jazz.ui.Dialog(floatingPaneParams);
            if (this.dialogContent && this.dialogContent.setFocus) {
                this.dialogContent.setFocus();
            }
        },

        destroy: function(){
            if(this.dialogContent){
				this.dialogContent.destroyRecursive();
	            delete this.dialogContent;
	        }
            this.inherited(arguments);
        },

        onClose: function(){
            /* Empty Block */
        }
    });

    /* DialogContent */
    dojo.declare("rmps.app.ui.internal.common.DialogContent", [dijit._Widget, dijit._Templated], {
        templateString: dojo.cache("rmps.app", "ui/internal/dialogs/templates/AddExternalLinksDialogContent.html"),

        "class": "rmps-ui-SelectItemWidget",
        
        widgetsInTemplate: true,

		dialog: null,	/*Gotten From the initializer */
		events: null,

		URL_REGEX: dojox.validate.regexp.url({
            scheme: true,
            allowDecimal: false,
            allowHex: false,
            allowDottedHex: false,
            allowDottedOctal: false
        }),

        constructor: function(params){
			this.item= params.item;
			this.dialog= params.dialog;
            this.buttonOK = this.dialog.messages.buttonOK;
            this.buttonCancel = this.dialog.messages.buttonCancel;
            this.addExternalLinkDialog_linkUrl = this.dialog.messages.addExternalLinkDialog_linkUrl;
            this.addExternalLinkDialog_linkComment = this.dialog.messages.addExternalLinkDialog_linkComment;
            this.events= [];
        },

        postCreate: function(){
            var item= this.item;
            var url= "";
            var description= "";
            if(item){
                this.url= item.url;
                this.description= item.description;
            }else{
                this.url= "";
                this.description= "";
            }

			this.urlText.value= url;
			this.urlText.errorMessage= this.dialog.messages.invalidUrl;
			this.urlText.textInput.value= this.url;
            this.descriptionText.textInput.value= this.description;
            this._connectEvents();
        },

        setFocus: function(){
            this.urlText.textInput.focus();
        },

        reset: function(){
            this.urlText.textInput.value= "";
            this.descriptionText.textInput.value= "";
            this.okCmd.disabled = true;
        },

        _connectEvents: function(){
            this.urlText.validate= dojo.hitch(this, function(value){
                return !(value.indexOf("http://") === 0 || value.indexOf("https://") === 0 || value.indexOf("ftp://") === 0);
			});
			
            this.events.push(this.connect(this.urlText.textInput, "onkeyup", "_handleKeyUp"));
			this.events.push(this.connect(this.descriptionText.textInput, "onkeyup", "_handleKeyUp"));
        },

        _handleKeyUp: function(e){
            dojo.stopEvent(e);
            var descriptionValue= this.descriptionText.textInput.value.toString();
            if ((dojo.trim(this.urlText.textInput.value.toString()) === this.url.toString() && descriptionValue === this.description)) {
                /*OK should not be enabled in case nothing changed */
                this.okCmd.disabled= true;
                return;
            }
            var whitespaceDescription= descriptionValue !== "" && dojo.trim(descriptionValue) === "";
            var url = this.urlText.textInput.value.toString().toLowerCase();
            this.okCmd.disabled = !(url.indexOf("http://") === 0 || url.indexOf("https://") === 0 || url.indexOf("ftp://") === 0) || whitespaceDescription;
            if (!this.okCmd.disabled && (e.keyCode === KEY_ENTER ||
                e.keyCode === KEY_RETURN)) {// enter
                this._handleOk();
            }
        },

        _handleOk: function(){
            var item = {
                description: this.descriptionText.textInput.value,
                url: this.urlText.textInput.value,
                predicate: ILinksConstants.EXTERNAL_LINK_TYPE,
                linkTitle: ILinksConstants.EXTERNAL_LINK_TYPE_TITLE
            };
            var items = [item];
            this.onOk(items);
        },

        _handleCancel: function(){
            this.onCancel();
        },
        
        clearEvents: function(){
            while (this.events.length > 0) {
				var entry = this.events.pop();
				if(entry){
					this.disconnect(entry);
				}
				entry = null;
			}
			
			this.urlText.validate= dojo.hitch(this, function(value){/*Do Nothing */});
        },

        destroy: function(){
            this.clearEvents();	/*Stop listening at this point */
            
            if(this.descriptionText){
                this.descriptionText.destroyRecursive();
            }
			delete this.descriptionText;
            if(this.urlText){
                this.urlText.destroyRecursive();
            }
			delete this.urlText;
            this.inherited(arguments);
        }
    });

})();
