/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("dm.review.app.ui.internal.dialogs.AddParticipantsForm");

dojo.require("dojo.i18n");
dojo.require("dijit.form.RadioButton");
dojo.require("dojox.xml.parser");
dojo.require("jazz.app.context"); //$NON-NLS-1$
dojo.require("rmps.core.review.util.ReviewTypes");
dojo.require("rmps.app.ui.ReviewEditorUtil");
dojo.require("rmps.core.process.SecurityClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox");

dojo.require("com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.requireLocalization("dm.review.app.ui.internal.dialogs", "ParticipantsFormsMessages");

var DELAY = 500;
var KEY_ESCAPE = 27;
var KEY_RETURN = 13;
var KEY_ENTER = 14;
var KEY_SPACE = 32;
var SEARCH_LIMIT = 250;

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReviewTypes = rmps.core.review.util.ReviewTypes;
var OPERATIONS= rmps.app.ui.ReviewEditorUtil.process.Operations;
var DEFAULT_ROLE= ReviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer;

var SERVICE_URI = "com.ibm.team.process.internal.service.web.IProcessWebUIService"; //$NON-NLS-1$

dojo.declare("dm.review.app.ui.internal.dialogs.AddParticipantsForm", com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog,{
		
	_messages: null,
	roleType: null,
	infoMessageBox: null,
	projectURI: null,
	userDisabledOptions: null,	/*Disabled entries during the current search session */
	filteredUsers: null,
	
	postMixInProperties: function() {
		this.inherited(arguments);
		this._messages = dojo.i18n.getLocalization("dm.review.app.ui.internal.dialogs", 
					"ParticipantsFormsMessages");
	},

	constructor: function (args) {
		this.selectedUsers = [];
		this.userDisabledOptions= [];
	},
	
	postCreate: function(){
		this.inherited(arguments);				
		dojo.style(this.filtersDiv,"display","none");
	
		var parentDiv = document.createElement('div');
		var radioDiv = this._createRadioDiv();
		var msgDiv = this._createMsgDiv(DEFAULT_ROLE);
		parentDiv.appendChild(this.createAndStartMessagebox(parentDiv).domNode);
		parentDiv.appendChild(radioDiv);
		parentDiv.appendChild(msgDiv);
		
		dojo.place(parentDiv,this.filtersDiv, "after");
		this.searchText.focus();

		this.roleType = DEFAULT_ROLE;
		
		this.connect(this.userSelector,"onchange", function () {
				this.checkSelectedUserPermissions(undefined);
		});
	},
	
	_handleDoubleClick: function (e) {
		var parentMethod= dojo.hitch(this, this.getInherited("_handleDoubleClick", arguments));
		
		/*The check will unselect the unathorized double clicked entry. Then, the super method will be called as a post call back */
		this.checkSelectedUserPermissions(parentMethod, e);
	},
	
	_handleKeyDown: function (e) {
		var parentMethod= dojo.hitch(this, this.getInherited("_handleKeyDown", arguments));
		
		if ((e.keyCode == KEY_RETURN || e.keyCode == KEY_ENTER) && this.selectedUsers.length > 0) {
			this.checkSelectedUserPermissions(parentMethod, e);				
		}
	},
	
	checkSelectedUserPermissions: function(postCallback, args){
		this.infoMessageBox.hideMessage();	/*hide previous possible opened messages */

		var toRemove= [];
		var currentOption;

		function getUserPermissionsCallback() {
		    return function (optionInvoked, optionIndex, permissions) {
				if(this.subscriberBtn.checked== true){
					if(postCallback){
						postCallback(args);
					}
					return;	/*Subscriber does not need to have any roles assigned */
				}
				/*Disable buttons right away to avoid any delay could be happening till we know if hte user has permissions or not */
				var addCloseButtonStatus= this.addCloseButton.disabled;
				var addButtonStatus= this.addButton.disabled;
				
				/*
				 * Only disable if no users are already selected. Because if some users are already selected, no need to disable the buttons enablement 
				 * 
				 */
				var isDisabled= this.selectedUsers.length<=1;
				this.addCloseButton.disabled = isDisabled;
				this.addButton.disabled = isDisabled	;
				
				if(!permissions[OPERATIONS.ALL_FEEDBACK]&& !permissions[OPERATIONS.VOTE]){
					toRemove.push(optionInvoked.userId);
					optionInvoked.disabled = "disabled";
					
					optionInvoked.selected= false;
					this.userDisabledOptions.push(optionInvoked);
					
					this.infoMessageBox.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
						summary: dojo.string.substitute(this._messages.userWithNoPermissions, [optionInvoked.userName]),
						closable: true
					});				
				}else{
					if(this.selectedUsers.length>0){
						/*Return enablement to the original status */
						this.addCloseButton.disabled = addCloseButtonStatus;
						this.addButton.disabled = addButtonStatus;							
					}
				}
				
				if(optionIndex== this.selectedUsers.length-1){
					
					/*Splice unathorized entries */
					for(var updateIndex=0; updateIndex<this.selectedUsers.length; updateIndex++){
						var entry= this.selectedUsers[updateIndex];
						
						if(dojo.indexOf(toRemove, entry.userId)!== -1){
							this.selectedUsers.splice(updateIndex, 1);
							updateIndex= 0;	/*Restart the loop after splice */
						}
					}
					
					/*Avoid closing the dialog in case we filtered out some users. Why? In order to make sure that hte user will get a chance to 
					 * understand that some entries have been disabled to whatever reason */
					if(toRemove.length==0 && postCallback){
						/*We finished checking all selected options, now call the parent method */
						postCallback(args);	
					}
					
				}
		    };
		}


		for(var optionsIndex=0; optionsIndex<this.selectedUsers.length;optionsIndex++){
			var selectedUser= this.selectedUsers[optionsIndex];
						
			currentOption= null;
			
			getOption: for(var getOptionsIndex=0; getOptionsIndex<this.userSelector.options.length;getOptionsIndex++){
				var current= this.userSelector.options[getOptionsIndex];
				if(current.userId== selectedUser.userId){
					currentOption= current;
					break getOption;
				}
			}	
			
			if(!currentOption|| !currentOption.selected){
				continue; 
			}
			
			var callback= dojo.hitch(this, getUserPermissionsCallback(), currentOption, optionsIndex);
			rmps.core.process.SecurityClient.getUserPermissionsInProject(currentOption.userId, callback, null, true);
		}
	},
	
	createAndStartMessagebox: function(node){
		// We must sent a resize event when the message box is shown/hidden
		// and when the details are toggled.
		function publishResizeEvent() {
			dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
		}
		function onHideMessage() {
			publishResizeEvent();
		}

		this.infoMessageBox = new jazz.ui.MessageBox();
		this.infoMessageBox.domNode.style.margin = "1px";
		this.infoMessageBox._first = true;
		dojo.connect(this.infoMessageBox, "_toggleDetails", node, publishResizeEvent);
		dojo.connect(this.infoMessageBox, "hideMessage", node, onHideMessage);
		dojo.connect(this.infoMessageBox, "showMessage", node, publishResizeEvent);

		this.infoMessageBox.startup();
		return this.infoMessageBox;
	},

	_createRadioDiv: function(){
		var lbl1, lbl2, lbl3;
		
		var createRadioButton = function(btnName,btnValue,isChecked){
			return new dijit.form.RadioButton({
				value: btnValue,
				name: btnName,
				checked: isChecked || false						
			});
		};
		
		var createLabel = function(button, message){
			var lbl = document.createElement('label');
			lbl.htmlFor = button.id;
			lbl.innerHTML = message;
			
			return lbl;				
		};
		
		this.approverBtn = createRadioButton("typeGroup","approverType");
		this.reviewerBtn = createRadioButton("typeGroup","reviewerType", true);
		this.subscriberBtn = createRadioButton("typeGroup","optionalReviewerType");
		
		lbl1 = createLabel(this.reviewerBtn, this._messages.reviewerType);
		lbl2 = createLabel(this.approverBtn, this._messages.approverType);
		lbl3 = createLabel(this.subscriberBtn, this._messages.optionalReviewerType);
		
		this.connectHandler(this.approverBtn, "onClick", dojo.hitch(this, function(){
				this._handleRadioOnClick(this.approverBtn, ReviewTypes.PARTICIPANT_ROLE_TYPES.Approver);
			})
		);

		this.connectHandler(this.reviewerBtn, "onClick", dojo.hitch(this, function(){
				this._handleRadioOnClick(this.reviewerBtn, ReviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer);
			})
		);

		this.connectHandler(this.subscriberBtn, "onClick", dojo.hitch(this, function(){
				/*Re-enable the disabled entries during the current session, as our choice is subscriber now */
				for(var index=0; index<this.userDisabledOptions.length; index++){
					this.userDisabledOptions[index].disabled = "";
				}
				this.userDisabledOptions= [];
				
				
				this._handleRadioOnClick(this.subscriberBtn, ReviewTypes.PARTICIPANT_ROLE_TYPES.OptionalReviewer);
			})
		);
		
		
		var btn1Div = document.createElement('div');
		var btn2Div = document.createElement('div');
		var btn3Div = document.createElement('div');
		var containerDiv = document.createElement('div');
		
					
		containerDiv.className = "type-group-div";					
		btn1Div.className = "button-label-field";					
		btn2Div.className = "button-label-field";					
		btn3Div.className = "button-label-field";

		btn1Div.appendChild(this.reviewerBtn.domNode);
		btn1Div.appendChild(lbl1);
		
		btn2Div.appendChild(this.approverBtn.domNode);
		btn2Div.appendChild(lbl2);
		
		btn3Div.appendChild(this.subscriberBtn.domNode);
		btn3Div.appendChild(lbl3);
		
		containerDiv.appendChild(btn1Div);
		containerDiv.appendChild(btn2Div);
		containerDiv.appendChild(btn3Div);
		
		return containerDiv;
	},

	connectHandler: function(button, event, handler) {
		return this.connect(button, event, handler);
	},

	_handleRadioOnClick: function(button, roleType){
		dojo.byId("msgDiv").innerHTML = this._messages[button.value + "Description"];
		this.roleType = roleType;
		this.checkSelectedUserPermissions();
	},

	_createMsgDiv: function(role){
		var msgDiv = document.createElement('div'); 
		msgDiv.id = "msgDiv";
		if (role && role == ReviewTypes.PARTICIPANT_ROLE_TYPES.Approver){
			msgDiv.innerHTML = this._messages.approverTypeDescription;
		} else {
			msgDiv.innerHTML = this._messages.reviewerTypeDescription;			
		}
		return msgDiv;
	},			

	/* (non-Javadoc)
     * @see com.ibm.team.process.web.ui.internal.TeamContributorSelectionDialog#_loadQueryResults(resultSet)
     */
	_loadQueryResults: function (resultSet) {
		this.resultSet= resultSet;
		if(!this.resultSet|| !this.resultSet.elements){
			return this.inherited(arguments);
		}
		var parentCall= dojo.hitch(this, this.getInherited("_loadQueryResults", arguments));

		this.loadQueryResultsCallback= dojo.hitch(this, function(results){
			parentCall(this.resultSet);
		});
		
		var serviceRequest = new com.ibm.team.repository.web.transport.ServiceRequest(SERVICE_URI, "getProjectAreaByUUIDWithLimitedMembers", {
			processAreaItemId: jazz.app.context.get().itemId, maxMembers: SEARCH_LIMIT}
		); //$NON-NLS-1$
		
		var srh = new ServiceResponseHandler(this, "getAvailableUsers", "_disableUsersErrorCallback"); //$NON-NLS-1$ //$NON-NLS-2$
		com.ibm.team.repository.web.transport.TeamServerClient.invokeService(serviceRequest, srh);
	},
	
	/* 
	 * @see:  https://jazz.net/jazz/resource/itemName/com.ibm.team.workitem.WorkItem/112922
	 * We do not have a method to get users in a speicfic project area. Once 112922 is resolved, we will be able to, and we can remove this method 
	 */
	getAvailableUsers: function(results){
		var filtered= [];
		var members= results.members;
		if (members){
			var elements= this.resultSet.elements;
			for(var index=0; index<elements.length; index++){
				var current= elements[index];
				var itemId= current.itemId;
				for(var subIndex=0; subIndex<members.length;subIndex++){
					var member= members[subIndex];
					if(member.itemId === itemId&& dojo.indexOf(this.filteredUsers,decodeURIComponent(member.userId))=== -1){
						filtered.push(current);
					}
				}
			}
		}
		
		// IE does not like when we return an empty elements array
		this.resultSet.elements= (filtered.length>0)?filtered:null;
		this.loadQueryResultsCallback();
	}
});
})();