/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("dm.review.app.ui.internal.dialogs.ChangeParticipantsForm");
dojo.require("dojo.i18n");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.Form");
dojo.require("rmps.app.ui.ReviewEditorUtil");
dojo.require("rmps.core.process.SecurityClient");

dojo.require("jazz.ui.Dialog");

dojo.require("rmps.ui.DialogForm");
dojo.require("rmps.core.review.util.ReviewTypes");

var ReviewTypes = rmps.core.review.util.ReviewTypes;
var OPERATIONS= rmps.app.ui.ReviewEditorUtil.process.Operations;

dojo.requireLocalization("dm.review.app.ui.internal.dialogs", "ParticipantsFormsMessages");


dojo.declare("dm.review.app.ui.internal.dialogs.ChangeParticipantsForm",rmps.ui.DialogForm, {
	
	templateString: dojo.cache("dm.review.app", "ui/internal/dialogs/templates/ChangeParticipantsForm.html"),
	
	widgetsInTemplate: true,
	participant: null,
	
	selectedParticipantRoleType: null,
	_changeParticipantsForm: null,
	_approverRadioButton: null,
	_optionalReviewerRadioButton: null,
	_okButtonNode: null,
	_cancelButtonNode: null,
	_messages: null,

	postCreate: function() {
		this.inherited("postCreate", arguments);
    	this._okButtonNode.attr("disabled", true);
		this.connect(this._approverRadioButton, "onClick",
			function() {
				this.selectedParticipantRoleType =
						rmps.core.review.util.ReviewTypes.PARTICIPANT_ROLE_TYPES.Approver;
				this._checkUserPermission(this.participant.contributor);
		});
		this.connect(this._reviewerRadioButton, "onClick",
			function() {
				this.selectedParticipantRoleType =
						rmps.core.review.util.ReviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer;
				this._checkUserPermission(this.participant.contributor);
		});
		this.connect(this._optionalReviewerRadioButton, "onClick",
			function() {
				this.selectedParticipantRoleType =
					rmps.core.review.util.ReviewTypes.PARTICIPANT_ROLE_TYPES.OptionalReviewer;
				this.infoMessageBox.hideMessage();
		    	this._okButtonNode.attr("disabled", this.isSameRole());
		});
		this._changeParticipantsForm.appendChild(this.createAndStartMessagebox(this._changeParticipantsForm).domNode);
	},
	
	postMixInProperties: function() {			
		this._messages = dojo.i18n.getLocalization("dm.review.app.ui.internal.dialogs",
					"ParticipantsFormsMessages");
	},
	
	isSameRole: function(){
		return this.selectedParticipantRoleType === this.participant.reviewRole;
	},

	show: function() {	
		this.inherited(arguments);
		var currentRole = this.participant.reviewRole;
		if (currentRole === rmps.core.review.util.ReviewTypes.PARTICIPANT_ROLE_TYPES.OptionalReviewer){
			this._optionalReviewerRadioButton.attr("checked", true);
			this.selectedParticipantRoleType =
				rmps.core.review.util.ReviewTypes.PARTICIPANT_ROLE_TYPES.OptionalReviewer;
		} else	if (currentRole === rmps.core.review.util.ReviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer){
			this._reviewerRadioButton.attr("checked", true);
			this.selectedParticipantRoleType =
				rmps.core.review.util.ReviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer;
		} else {
			this._approverRadioButton.attr("checked", true);
			this.selectedParticipantRoleType =
				rmps.core.review.util.ReviewTypes.PARTICIPANT_ROLE_TYPES.Approver;
		}
	},
	
	getDialogArguments: function(){
		return {
			contentNode: this._changeParticipantsForm,
			primaryTitle: this._messages.changeParticipantsTitle,
			destroyContents: true,
			width: "270px"
		};
	},
	
	createAndStartMessagebox: function(node){
		// We must sent a resize event when the message box is shown/hidden
		// and when the details are toggled.
		function publishResizeEvent() {
			dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
		}
		function onHideMessage() {
			publishResizeEvent();
		}

		this.infoMessageBox = new jazz.ui.MessageBox();
		this.infoMessageBox.domNode.style.margin = "1px";
		this.infoMessageBox._first = true;
		dojo.connect(this.infoMessageBox, "_toggleDetails", node, publishResizeEvent);
		dojo.connect(this.infoMessageBox, "hideMessage", node, onHideMessage);
		dojo.connect(this.infoMessageBox, "showMessage", node, publishResizeEvent);

		this.infoMessageBox.startup();
		return this.infoMessageBox;
	},
	
	_checkUserPermission: function(userUri){
		function getUserPermissionsCallback() {
		    return function (invokedUserId, permissions) {
		    	// Disable OK button until we checked the permissions
		    	this._okButtonNode.attr("disabled", true);
				if(!permissions[OPERATIONS.ALL_FEEDBACK]&& !permissions[OPERATIONS.VOTE]){
					this.infoMessageBox.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
						summary: dojo.string.substitute(this._messages.userWithNoPermissions, [invokedUserId]),
						closable: true
					});				
				}else{
			    	this._okButtonNode.attr("disabled", this.isSameRole());
				}
		    };
		}
		var userId = userUri.substring(userUri.lastIndexOf("/") + 1);
		var callback= dojo.hitch(this, getUserPermissionsCallback(), userId);
		rmps.core.process.SecurityClient.getUserPermissionsInProject(userId, callback, null, true);
	}
});

})();
