/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.reports                                        |
 *|                                                                        |
 *| Copyright IBM Corporation 2012, 2013.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
dojo.provide("rmps.app.ui.internal.dialogs.DeletionFromExplorerInfoDialog");

/** 
 * This dialog is meant for use when deletion is done from explorer. 
 * Dialog shows the deletion confirmation message (without showing files affected by deletion), 
 * and upon user's confirmation submits deletion request. Deletion in progress message remains displayed 
 * until deletion is done. In case of successful deletion, dialog is closed and Explorer refreshed. 
 * In case of a problem during deletion, an error is displayed in the dialog and it remains open.
 *
 * @author lgrahek
 */
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.Dialog");
dojo.require("rmps.app.ui.internal.dialogs.ResourceInfoDialog");
dojo.require("rmps.core.RootServices");
dojo.require("rmps.ui.Footer");
dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.ui.internal.dialogs", "ResourceInfoMessages");

dojo.declare("rmps.app.ui.internal.dialogs.DeletionFromExplorerInfoDialog", [rmps.app.ui.internal.dialogs.ResourceInfoDialog], {
		
	setDeletionConfirmationMessages: function() {
		var resMessage = this.deleteFiles.length === 1 ? this.messages.resource : this.messages.resources;
		var names = "";
		for (var i = 0; i < this.deleteFiles.length; i++) {
			if (i > 0) {
				names += ", ";
			}
			names += this.deleteFiles[i];
		}
		this.deleteMessage.innerHTML = dojo.string.substitute(this.messages.deleteConfirmation, [resMessage, names]);
	},
	
	getTitle: function() {
		return this.messages.deleteFiles;
	},
	
	/**
	 * Submit deletion request - if everything goes well close the dialog and refresh Explorer, otherwise show the error.
	 */
	okPressed: function(dialog) {
		this.adjustAppearanceOnOkPressed();
		// Set 'Deletion in progress' message
		this.deleteMessage.innerHTML = this.messages.deleteInProgress;
		var def;
		var args = {
				content: {
					logicalURIs: this.uris
				},
				
				headers: {
					"X-ibm-rmps-internal": "true"
				},
				
				url: rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "rdfdirectory") + "/",
											
				error: dojo.hitch(this, function(error, ioArgs){
                    // only happens if there is an error outside of the code that handles deletion, 
					// otehrwise, the server will send an error as handled below
					this._footer.setCancelHidden(false);
					this.deleteMessage.innerHTML = error;
	        	})			
			}; 
		def = rmps.core.client.xhrDelete(args, false);
		def.addCallback(dojo.hitch(this, function(){
			if (arguments && arguments.length > 0 && arguments[0] != "") {
				// something went wrong, server is sending a message				
				this._footer.setCancelHidden(false);
				this.deleteMessage.style.color = "#E30000";
				var obj = dojo.fromJson(arguments[0]);
				if (obj["lockedBy"]) {
					var summary = dojo.string.substitute(this.messages.lockedBy, 
							[obj["lockedResource"], obj["lockedBy"]]);
					if (obj["lockedBy"] === 'differentContext'){
						summary = dojo.string.substitute(this.messages.differentContext, [obj["lockedResource"]]);
					} else if (obj["lockedBy"].indexOf("notOwner:")>-1){
						summary = dojo.string.substitute(this.messages.notOwner, [obj["lockedBy"].substring(obj["lockedBy"].indexOf(":")+1)]);								
					} else if (obj["lockedBy"].indexOf("snapshot")>-1){
						summary = this.messages.snapshot;								
					}
					this.deleteMessage.innerHTML = summary;
				} else if (obj["error"]) {
					this.deleteMessage.innerHTML = obj["error"];
				}
			} else {
				this.explorer.getOwner().refresh();
				this.close();
				dojo.publish("rmps.web.app.explorer.deleteResource",
					[{
						uri: this.uri,
					}]);
			}
		}));
		return def;		
	}
});

})();