/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
 
dojo.provide("rmps.app.ui.internal.dialogs.EditorDialog");

dojo.require("jazz.ui.Dialog");
dojo.require("jazz.core.loader");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("rmps.core.TaskMonitor");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.ui.internal.dialogs", "EditorDialogMessages");

var messages= dojo.i18n.getLocalization("rmps.app.ui.internal.dialogs", "EditorDialogMessages");

/**
 * A dialog for choosing an existing context (stream or baseline) on the server.  This dialog
 * just *returns* the selected context; it is not meant to be used to actually set a new context.
 */
var EditorDialog = dojo.declare("rmps.app.ui.internal.dialogs.EditorDialog", [], {
	
});

/**
 * Opens the dialog.
 */
EditorDialog.show = function(editorInput, editorClass, dialogTitle, refreshFunction) {
	var classConstructor = dojo.getObject(editorClass);
	var editor;
	var dialog;
	if (!classConstructor) {
		jazz.core.loader.load_sync(editorClass);
		classConstructor = dojo.getObject(editorClass);
	}
	if (editorInput.isLoaded()) {
		editor = new classConstructor({
			editorInput: editorInput,
			suppressEditorUi: true,
		});
		dialog = new jazz.ui.Dialog({
			contentNode: new rmps.app.ui.internal.dialogs._EditorDialogContent({
				handleFinished: dojo.hitch(EditorDialog, function(ok) {
					if (ok && refreshFunction) {
						refreshFunction.call();
					}
					if (dialog) {
						dialog.close();
					}
				}),
				editor: editor,
			}).domNode, 
			primaryTitle: dialogTitle,
		});
	} else {
		var deferred = editorInput.load();
		deferred.addCallback(EditorDialog, function() {
			editor = new classConstructor({
				editorInput: editorInput,
				suppressEditorUi: true,
			});
			dialog = new jazz.ui.Dialog({
				contentNode: new rmps.app.ui.internal.dialogs._EditorDialogContent({
					handleFinished: dojo.hitch(EditorDialog, function(ok) {
						if (ok && refreshFunction) {
							refreshFunction.call();
						}
						if (dialog) {
							dialog.close();
						}
					}),
					editor: editor,
				}).domNode, 
				primaryTitle: dialogTitle,
			});
		});
	}

	
};


dojo.declare("rmps.app.ui.internal.dialogs._EditorDialogContent", [dijit._Widget, dijit._Templated], {

	templateString: '<div>' +
						'<div dojoAttachPoint="editorDiv"></div>' +
						'<div class="buttonContainer" style="float: right">' +
						'<button dojoAttachPoint="_okButton" dojoAttachEvent="onclick: _save" style="width:80px; margin-right: 5px; margin-bottom: 5px; align=right">${messages.buttonOK}</button>' +
						'<button dojoAttachEvent="onclick: _cancel" style="width:80px; margin-right: 5px; margin-bottom: 5px; align=right">${messages.buttonCancel}</button>' +
						'</div>' +
					'</div>',
					
	messages: messages, //local copy for use in template string
	editor: null,
	
	/**
	 * Overridden from superclass.
	 */					
	postCreate: function() {
		var self = this;
		this.editorDiv.appendChild(this.editor.domNode);
		this._okButton.disabled = true;
		dojo.connect(this.editor, "dirtyStateChanged", function(isDirty) {
			self._okButton.disabled = !isDirty;
		}, true);
	},
	
	_cancel: function() {
		this.handleFinished();
	},
	
	_save: function(contextObject) {
		var self = this;
		if (this.editor.isDirty()) {
			var taskMonitor = new rmps.core.TaskMonitor(this.messages.saving);
			taskMonitor.setCanComplete(false);
			this.editor.doSave(taskMonitor);
			
			var handle = this.editor.connect(taskMonitor, "onComplete", function(errors) {
				self.editor.saving = false;
				self.editor.disconnect(handle);
				if (errors) {
					self.editor._handleSaveErrors(errors);
				} else {
					self.editor.onSaveComplete();
					self.handleFinished(true);
				}
			});

			taskMonitor.setCanComplete(true);
		}
	},
	
});
 
})();