/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.app                                            |
 *|                                                                        |
 *| Copyright IBM Corporation 2013.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
dojo.provide("rmps.app.ui.internal.dialogs.RemoveLinkDialog");

/** Dialog shows list of resource links that will be affected with Remove Link action
 * 
 * @author Eric Funk
 */
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.Dialog");
dojo.require("rmps.core.RootServices");
dojo.require("rmps.ui.Footer");
dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.ui.internal.editors.views", "LinksSectionMessages");

dojo.declare("rmps.app.ui.internal.dialogs.RemoveLinkDialog", [dijit._Widget, dijit._Templated], {
	widgetsInTemplate: true,
	templateString: 
		'<div dojoAttachPoint="_dialogContent" style="margin-left: 8px; margin-right: 8px; margin-top: 5px">' +
		'<span data-dojo-attach-point="deleteMessage"></span></br>'+
		'<div dojoAttachPoint="_fileList"></div>'+
		'<div data-dojo-attach-point="_footerContainer" data-dojo-type="dijit.layout._LayoutWidget"></div>',

	
	constructor: function(args) {
		dojo.mixin(this, args);
		this.messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors.views", "LinksSectionMessages");
		this.isDeletion = true;
		if (!this.deleteFile) {
			this.isDeletion = true;
		}
	},
		
	postCreate: function() {
		// Set the message/info for the user so the user can confirm 
		this.setDeletionConfirmationMessages();
		
		// Add ok and cancel buttons
		this._footer= new rmps.ui.Footer();
		this.connect(this._footer, "onOK", this.okPressed);
		this.connect(this._footer, "onCancel", this._cancelPressed);
		this._footerContainer.addChild(this._footer);					
		
		this.dialog = new jazz.ui.Dialog({
			contentNode: this._dialogContent,
			modal: true,
			primaryTitle: this.getTitle(),
			width: "375px",
			isClosable: false
		});
		
		this._footer.setOKEnablement(true);
		this._footer.setFocusOnOK();
	},
	
	/**
	 * Sets the message/info for the user so the user can confirm
	 */
	setDeletionConfirmationMessages: function() {
		var size = this.files.length;
		if (this.deleteFile!==undefined) {			
			if (size>0) {
				this.deleteMessage.innerHTML = dojo.string.substitute(this.messages.deleteAllConfirmation, [this.deleteFile]);
			} else {
				this.deleteMessage.innerHTML = dojo.string.substitute(this.messages.deleteConfirmation, [this.deleteFile]);
			}
		}
		var fileListDiv = this._fileList;
		var size = this.files.length;
		for (var i=0; i<size; i++) {
			dojo.create("li", {innerHTML: this.files[i].label}, this._fileList);
		}
	},
	
	/**
	 * Returns the title of the dialog
	 */
	getTitle: function() {
		return this.messages.remove;
	},
	
	/**
	 * When OK button is clicked, message changes, ok and cancel are hidden, onOK is invoked.
	 * In case of deletion, it is expected that the caller closes the dialog when operation is done (or whenever appropriate).
	 * In case of update, dialog is closed right after sending onOk.
	 */
	okPressed: function() {
		this.adjustAppearanceOnOkPressed();
		this.onOk(this);
		this.close();
	},

	/**
 	 * Hide OK and Cancel.
 	 */	
	adjustAppearanceOnOkPressed: function() {
		this._footer.setOKHidden(true);
		this._footer.setCancelHidden(true);
	},
	
	_cancelPressed: function() {
		this.close();
	},
	
	close: function() {
		this.dialog.close();
	}
});

})();