/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("rmps.app.ui.internal.dialogs.SelectDomainVersionDialog");

dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.MessageBox");
dojo.require('jazz.util.html');
dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
dojo.require("rmps.core.util.ContextUtils");
dojo.require("rmps.core.error");

dojo.requireLocalization("rmps.app.ui.internal.dialogs", "SelectDomainVersionDialogMessages");
var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.dialogs", "SelectDomainVersionDialogMessages");

var DOMAIN_REG_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "domainRegistry");

dojo.declare("rmps.app.ui.internal.dialogs.SelectDomainVersionDialog", [dijit._Widget, dijit._Templated], {
	
	messages: messages,
	widgetsInTemplate: true,
	templateString: "<div dojoAttachPoint='contentNode' style='margin: 0px 10px 30px 10px;'>"
		+ "<div dojoAttachPoint='_messageBoxContainer'></div>"
		+ "<h2 dojoAttachPoint='headerNode'></h2>"
		+ "<div dojoAttachPoint='tableContainerNode'>"
		+ "</div><br/>"
		+ "<div style='float: right'>"
		+ "<input type='button' dojoAttachPoint='okButton' style='width:80px; margin-right: 10px;' align='right' value='${messages.buttonOk}'/>"
		+ "<input type='button' dojoAttachPoint='cancelButton' style='width:80px' align='right' value='${messages.buttonCancel}'/>"
		+ "</div>"
		+ "</div>",
		
	reloadAfterClose: false,
	domainDataArray: null,
	readOnly: false,
	
 	show: function () {
	
		// show dialog
		this.dialog = new jazz.ui.Dialog({
			contentNode: this.contentNode,
			primaryTitle: messages.title,
			width: "800px"
		});

		var self = this;
		// load the workspace dependencies
		if (!this.domainDataArray) {
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: messages.loading
			});
			this.loadDependencies().addCallback(function() {
				self.hideMessage();
				self.refresh();
			});
 		} else {
			self.refresh();
 		}
	},
	
	refresh: function() {

		// disable the ok for snapshot context
		var contextUri = rmps.core.util.ContextUtils.getCurrentContextURI();
		var isSnapshotContext = rmps.core.util.ContextUtils.isSnapshot(contextUri);
		if (isSnapshotContext) {
			this.readOnly = true;
			this.okButton.disabled = true;
		}
		
		// build content 
		var context = com.ibm.team.vvc.shared.web.ui.context.ContextUtils.getCurrentContext();
		
		var title = dojo.string.substitute(messages.configLabel, [context.getTitle()]);
		this.headerNode.appendChild(dojo.doc.createTextNode(title));

		//this.headerNode.firstChild.textContent = title;
		this.connect(this.okButton, "onclick",  this.okPressed);
		this.connect(this.cancelButton, "onclick",  this.closeDialog);
		
		var columnData = [];
		columnData.push({ label: messages.domain });
		columnData.push({ label: messages.domainProject });
		columnData.push({ label: messages.version });

		var table = new jazz.ui.StyledTable({
			trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
			columns: columnData,
			showHover: true
		});
		this.tableContainerNode.appendChild(table.domNode);
		table.domNode.style.width = "100%";
		var projectNameToDomainNameTd = [];
		
		for (var i = 0; i < this.domainDataArray.length; i++) {
			var domain = this.domainDataArray[i];
			var projectName = domain.projectName;
			
			// if domain project is already created, then include the domain name in the same row
			td = projectNameToDomainNameTd[projectName];
			if (td) {
				dojo.create("br", null, td);
				// N.B. createTextNode escapes the passed content. No need to escape.
				td.appendChild(dojo.doc.createTextNode(domain.label));
				continue;
			}
			
			var tr = document.createElement("tr");
			
			// domain column
			td = document.createElement("td");
			td.innerHTML = jazz.util.html.escape(domain.label);
			tr.appendChild(td);
			projectNameToDomainNameTd[projectName] = td;
			
			// domain project column
			td = document.createElement("td");
			td.innerHTML = jazz.util.html.escape(domain.projectName);
			tr.appendChild(td);
			
			// version column
			td = document.createElement("td");
			var selectNode = dojo.create("select", null, td);
			var option, data, text;
			var snapshots = domain.snapshots;
			domain.select = selectNode;
			selectNode.title = messages.sortOrder;
			
			if (this.readOnly) {
				selectNode.disabled = true;
			}
			
			if (snapshots) {

				data = { value: "", selected: false };
				if (!domain.selected) {
					// no version have been selected yet
					data.selected = true;
				}

				// create first option with no text
				option = dojo.create("option", data, selectNode);
				option.appendChild(dojo.doc.createTextNode(messages.unset));
				option.title = messages.unsetTooltip;
				
				// create one entry per snapshot in the combo box
				for (var n = 0; n < snapshots.length; n++) {
					var snapshot = snapshots[n];
					text = snapshot.name;
					data = {
						value: snapshot.uri,
						selected: (snapshot.selected? true: false)
					};
					option = dojo.create("option", data, selectNode);
					option.appendChild(dojo.doc.createTextNode(text));
					
					// create tooltip for each snapshot
					option.title = "";
					if (snapshot.created) {
						var createdDate = new Date(parseInt(snapshot.created));
						var created = dojo.date.locale.format(createdDate, {formatLength: "medium"});
						option.title += messages.created + created;
					}
					if (snapshot.description) {
						option.title += messages.description + snapshot.description;
					}
				}
			}
			tr.appendChild(td);
			
			table.tbody.appendChild(tr);
		}

	},
	
	loadDependencies: function() {
		var url = DOMAIN_REG_PATH + "/getDomainVersions";
		var self = this;
		var args = {
			url: url,
			handleAs: 'json',
			headers: {
				"Accept" : "application/json;text/json"
			},
			load: dojo.hitch(this, function(response, ioArgs) {
				self.domainDataArray = response;
			})
		};
		return rmps.core.client.xhrGet(args);
	},
	
	showMessage: function(args) {
		if (args && this._messageBoxContainer) {
			args["closable"] = true;
			if (!this.messageBox) {
				this.messageBox = new jazz.ui.MessageBox();
				this._messageBoxContainer.appendChild(this.messageBox.domNode);
			}
			this.messageBox.showMessage(args);
			this._messageBoxContainer.style.display = "";
		}
	},
	
	hideMessage: function() {
		if (this._messageBoxContainer) {
			this._messageBoxContainer.style.display = "none";
		}
	},
	
	okPressed: function() {

		// compute the newly added and removed dependencies
		var removeDependencies = [];
		var addDependencies = [];
		for (var i = 0; i < this.domainDataArray.length; i++) {
			var domain = this.domainDataArray[i];
			if (domain.select && domain.select.selectedIndex >= 0) {
				var option = domain.select.options[domain.select.selectedIndex];
				var oldSnapshotUri = domain.selectedSnapshotUri;
				var newSnapshotUri = option.value;
				if (newSnapshotUri) {
					if (oldSnapshotUri) {
						if (newSnapshotUri !== oldSnapshotUri) {
							removeDependencies.push(oldSnapshotUri);
							addDependencies.push(newSnapshotUri);
						}
					} else {
						addDependencies.push(newSnapshotUri);
					}
				} else if (oldSnapshotUri) {
					removeDependencies.push(oldSnapshotUri);
				}
			}
		}
		
		// remove domain version reminder even if versions haven't been changed if "OK" is pressed 
		rmps.core.client.xhrPut({
				url: DOMAIN_REG_PATH + "/removeDomainVersionsReminder",
				handleAs: 'json',
				headers: {
					"Accept" : "application/json;text/json"
				}
			}, true);
		
		// update dependencies
		if (addDependencies.length > 0 || removeDependencies.length > 0) {
			this.okButton.disabled = true;
			this.cancelButton.disabled = true;
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: messages.saving
			});
			
			var url = DOMAIN_REG_PATH + "/setDomainVersions";
			var data = { addDependencies: addDependencies, removeDependencies: removeDependencies };
			var deferred = rmps.core.client.xhrPut({
				url: url,
				putData: dojo.toJson(data),
				handleAs: 'json',
				headers: {
					"Accept" : "application/json",
					"Content-Type" : "application/json"
				},
				error: dojo.hitch(this, function(response, ioArgs) {
					this.okButton.disabled = false;
					this.cancelButton.disabled = false;
					var e = rmps.core.error.createXhrError(ioArgs);
					
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: e.message
					});
				}),
				load: dojo.hitch(this, function(response, ioArgs) {
					this.reloadAfterClose = true;
					this.closeDialog();
					return response;
				})
			}, false);
		} else {
			this.closeDialog();
		}
	},
	
	closeDialog: function() {
		if (this.dialog) {
			this.dialog.close();
		}
		if (this.reloadAfterClose) {
			window.location.reload(true);
		}
	},
      
	uninitialize: function() {
		if (this.dialog) {
			this.dialog.destroy();
		}
	}   
});
})();
