/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.BaseEditor");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dojox.html.entities");
dojo.require("rmps.ui.editor.PageHelper");
dojo.require("dojox.lang.aspect");
dojo.require("dojo.fx.easing");

dojo.require("rmps.core.TaskMonitor");
dojo.require("rmps.ui.SimpleButton");
dojo.require("rmps.ui.editor.EditorUtil");
dojo.require("dijit.InlineEditBox");
dojo.require("jazz.util.html");

dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.toolbar.Button");

dojo.require("rmpw.forms.ui.ModelValidationUtil");

dojo.requireLocalization("rmps.ui.common", "CommonMessages");
dojo.requireLocalization("rmps.app.ui.internal.editors", "BaseEditorMessages");

var commonMessages = dojo.i18n.getLocalization("rmps.ui.common", "CommonMessages");
var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "BaseEditorMessages");

var DEFAULT = "default";

dojo.declare("rmps.app.ui.internal.editors.BaseEditor", [dijit._Widget, dijit._Templated], {

	templateString: '<div>' +
			'<div dojoAttachPoint="headerContainerNode" class="header">' +
				'<div dojoAttachPoint="toolbarContainerNode" class="toolbar"></div>' +
				'<div dojoAttachPoint="_headerContent" class="mainContent"></div>' +
				'<div dojoAttachPoint="_validationContent" class="mainContent header"></div>' +
				'<div class="left-corner"></div>' +
				'<div class="right-corner"></div>' +
			'</div>' +
			'<div dojoAttachPoint="containerNode" class="container"></div>' +
		'</div>',

	// mixin
	editorInput: null,

	// attach points
	containerNode: null,
	headerContainerNode: null,

	// init properties
	isRefreshable: true,
	isReadOnly: false,
	uaKey: "",
	titleImg: null,
	// properties
	contentWidget: null,
	toolbar: null,
	messageBox: null,
	editorHelp: null,
	refreshButton: null,
	saveButton: null,
	inputNode: null,
	defaultMessageCloseDelay: 10000, // 10 seconds
	saving: false,
	suppressEditorUi: false,

	// private properties
	_dirtyGroups: null, //{}
	_active: false,
	_dirtyIndicator: null,
	_lastSize: null,
	_headerContent: null,
	_validationContent: null,

	constructor: function() {
		this._dirtyGroups = {};
	},

	postCreate: function() {
		dojo.addClass(this.domNode, "rmps-app-editors-base");
		this.contentWidget = this.createContentWidget();

		// message box
		this.messageBox = new jazz.ui.MessageBox();
		this.connect(this.messageBox, "_toggleDetails", this._toggleDetails);
		this.connect(this.messageBox, "hideMessage", this._hideMessage);
		this.connect(this.messageBox.domNode, "onmouseenter", this._resetMessageCloseDelay);
		this.connect(this.messageBox.domNode, "onmouseleave", this._startMessageCloseDelay);
		this.toolbarContainerNode.appendChild(this.messageBox.domNode);

		// toolbar
		this.toolbar = new jazz.ui.SimpleToolbar();
		this.toolbar.containerNode = this.toolbar.domNode;

		if (dojo.isIE < 8) {
			this.toolbarContainerNode.insertBefore(this.toolbar.domNode, this.messageBox.domNode);
		} else {
			this.toolbarContainerNode.appendChild(this.toolbar.domNode);
		}

		this.addTextActions();
		this.addButtons(this.toolbar);

		this.createTitleContents(true);

		// add clearing div
		dojo.create("div", {style: "clear: both;"}, this.headerContainerNode);

		if (this.contentWidget) {
			this.containerNode.appendChild(this.contentWidget.domNode);
		}
	},

	getChildren: function() {
		var children = this.inherited(arguments);
		return children.concat(dijit.findWidgets(this.headerContainerNode));
	},

	startup: function() {
		if (this.messageBox) {
			this.messageBox.startup();
		}
		if (this.toolbar) {
			this.toolbar.startup();
		}
		if (this.contentWidget) {
			this.contentWidget.startup();
		}
		if (this.editorHelp) {
			this.editorHelp.startup();
		}
		this.inherited(arguments);
	},

	createContentWidget: function() {
		// to be implemented by clients
	},
	
	createTitleContents: function(recreate){
		if(recreate&& this._headerContent){
			while(this._headerContent.hasChildNodes()) {
				var first= this._headerContent.firstChild;
				this._headerContent.removeChild(first);
				dojo.destroy(first);
			}	
		}
		this.createHeader(this._headerContent);
		if(this.editorHelp){
			this.editorHelp._helpImg.alt= messages.help;			
		}
	},

	createHeader: function(parentNode) {
		var iconUri = this.editorInput.getLargeIconUri() || this.editorInput.getIconUri();

		if (iconUri) {
			
			var tooltip= this.editorInput.getTooltip();
			
			var iconNode = dojo.create("span", {className: "title title-icon"});
			this.titleImg = dojo.create("img",{title: tooltip,alt: tooltip,style: "max-width: 25px;	max-height: 25px"});
			var handles = [
					this.connect(this.titleImg, "onerror", function() {
						dojo.forEach(handles, this.disconnect, this);
						iconNode.parentNode.removeChild(iconNode);
					}),
					this.connect(this.titleImg, "onload", function() {
						dojo.forEach(handles, this.disconnect, this);
						iconNode.appendChild(this.titleImg);
					})
				];
			this.titleImg.src = iconUri;
			
			parentNode.appendChild(iconNode);
		}

		this._createTitleNode(parentNode);
	},

	_createTitleNode: function(parentNode){
		var tooltipValue= this.editorInput.getTooltip();
		this.titleNode = dojo.create("span", {
			className: "title",
			title: tooltipValue?tooltipValue: ""
		});
		if(this.isEditableTitle()){
			this.createEditableTitleNode(this.titleNode, this.editorInput.getName(), this.editorInput.getTooltip());
		}else{
			this.createTitleNode(this.titleNode, this.getTitleNodeValue(), this.getTitleTooltipNodeValue());
			this._createDirtyIndicator();
			parentNode.appendChild(this._dirtyIndicator);
		}
		parentNode.appendChild(this.titleNode);
	},

	_createDirtyIndicator: function(){
		this._dirtyIndicator = dojo.create("span", {className: "title dirtyIndicator"});
		this._dirtyIndicator.style.display = "none";
		this._dirtyIndicator.appendChild(dojo.doc.createTextNode(" *"));
	},

	isEditableTitle: function(){
		return false;
	},
	
	getTitleNodeValue: function(){
		return this.editorInput.getName();
	},
	
	getTitleTooltipNodeValue: function(){
		return this.editorInput.getTooltip();
	},

	createTitleNode: function(titleNode, title, tooltip){
		titleNode.appendChild(dojo.doc.createTextNode(title));
		if (this.uaKey && this.uaKey != "") {
			this.editorHelp = new jazz.app.ua.PageHoverHelp({uaKey: this.uaKey});
			titleNode.appendChild(this.editorHelp.domNode);
			this.editorHelp.domNode.style.margin = "0px 0px 0px 5px";
		}
	},

	createEditableTitleNode: function(titleNode, title, tooltip){
		var div = document.createElement("div");
		div.className = "title";
		if(!title){
			title= "";
		}
		div.innerHTML = jazz.util.html.escape(title);
        this._editableTitle = new dijit.InlineEditBox({
				title:messages.editorTitleHelp, 
				autoSave: true, 
				width: "200px", 
				noValueIndicator: this.getEditableTitleNodeDetails(), 
				onChange: dojo.hitch(this, this._onTitleChange),
				onClick: dojo.hitch(this, this._onTitleClick), 
				onFocus: dojo.hitch(this, this._onTitleFocus),
				onBlur: dojo.hitch(this, this._onTitleBlur)}, div
        );
        
        dojo.addClass(this._editableTitle.domNode, "editIndicator");
        
		this._editableTitle.startup();
				
		this._createDirtyIndicator();
		
		var table = dojo.create("table");
		table.style.width = "50%";
		var tbody = document.createElement("tbody");
		table.appendChild(tbody);
		var tr = dojo.doc.createElement("tr");
		tbody.appendChild(tr);
		dojo.create("td", {style: "width: 0.5em; vertical-align: top;"}, tr).appendChild(this.createRequiredNode());
		dojo.create("td", null, tr).appendChild(div);
		dojo.create("td", {style: "width: 0.5em; vertical-align: top;"}, tr).appendChild(this._dirtyIndicator);
		
		if (this.uaKey && this.uaKey != "") {
			this.editorHelp = new jazz.app.ua.PageHoverHelp({uaKey: this.uaKey});
			dojo.create("td", {style: "width: 16px; vertical-align: top;"}, tr).appendChild(this.editorHelp.domNode);
			this.editorHelp.domNode.style.margin = "0px 0px 0px 5px";
		}
		titleNode.appendChild(table);
	},
	
	getTitleValue: function(){
		return this._editableTitle? this._editableTitle.getValue():"";
	},
	
	setTitleValue: function(value){
		if(this._editableTitle){
			this._editableTitle.attr('value', value);			
		}
	},

	createRequiredNode: function() {
		return this.getRequiredNode();
	},
	
	getRequiredNode: function(){
		var reqNode = dojo.create("span");
		reqNode.style.fontWeight= "bold";
		reqNode.style.color= "#E87820";
		reqNode.style.verticalAlign= "top";
		reqNode.style.fontSize= "9pt";
		reqNode.innerHTML = "*";
		reqNode.title = messages.requiredTitle;
		return reqNode;
	},

	_onTitleChange: function(value){
		if(this.titleImg){
			this.titleImg.alt= value;
			this.titleImg.title= value;			
		}
		this.onTitleChange(value);
	},
	onTitleChange: function(e){
		/*To be implemented by sub-classes */
	},

	_onTitleFocus: function(){
		this.onTitleFocus();
	},
	
	onTitleFocus: function(e){
		/*To be implemented by sub-classes */
	},

	_onTitleBlur: function(e){
		this.onTitleBlur();
	},
	
	_onTitleClick: function(e){
		this._editableTitle.domNode.style.height = "26px";
		/*Make sure that the edit mode will have the same width as the preview */
		if(this._editableTitle.wrapperWidget){
			this._editableTitle.wrapperWidget.editWidget.domNode.style.height= this._editableTitle.domNode.clientHeight+ "px";
			this._editableTitle.wrapperWidget.editWidget.domNode.style.width= this._editableTitle.domNode.clientWidth+ "px";
		}else{
			this._editableTitle.editorParams.style= {
				width: this._editableTitle.domNode.clientWidth+ "px",
				height: this._editableTitle.domNode.clientHeight+ "px"
			};
			this._editableTitle.width= this._editableTitle.domNode.clientWidth+ "px";
		}
	},	
	
	onTitleBlur: function(e){
		this._editableTitle.domNode.style.height = "auto";
		/*To be implemented by sub-classes */
	},

	getEditableTitleNodeDetails: function(){
		/*To be implemented by sub-classes */
		return "?";
	},

	addButtons: function(toolbar) {
		if (this.suppressEditorUi) {
			return;
		}
		
		if (this.isRefreshable) {
			// refresh button
			this.refreshButton = toolbar.addButton(this, this._confirmRefresh, {
				label: commonMessages.refresh,
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/refresh.gif")
			});
		}

		if (!this.isReadOnly) {
			this.saveButton = toolbar.addWidget(new rmps.ui.SimpleButton({
					label: commonMessages.save,
					onClick: dojo.hitch(this, this.save)
				}));
			this.saveButton.setEnabled(this.isDirty());
		}
	},

	addTextAction: function(text, imageUrl, onClick){
		var action = this.toolbar.addWidget(new jazz.ui.toolbar.Button({
			label: text,
			imgSrc: dojo.isObject(imageUrl) && imageUrl.uri ? imageUrl.uri : imageUrl  // check for a dojo._Url from dojo.moduleUrl()
		}));
		if (onClick){	/* onClick is optional */
			action.onClick = onClick;
		}
	},

	addTextActions: function(){
		/*To be implemnted by sub classes to maintain the order of actions, as text actions must come at the left hand side */
	},

	showLoading: function() {
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: commonMessages.loading
		});
	},

	showMessage: function(args, closeDelay) {
		this._resetMessageCloseDelay();

		if (closeDelay == null) {
			delete this.messageBox.__closeDelay;
		} else {
			this.messageBox.__closeDelay =
					parseInt(closeDelay) === closeDelay && closeDelay > 0 ? closeDelay : this.defaultMessageCloseDelay;
		}

		if (this.messageBox.__closeTimeout) {
			clearTimeout(this.messageBox.__closeTimeout);
			delete this.messageBox.__closeTimeout;
		}
		this.messageBox.showMessage(args);
		
		dojo.fadeIn({
				node: this.messageBox.domNode,
				beforeBegin: dojo.hitch(this, function() {
					dojo.style(this.messageBox.domNode, "opacity", "0");
				})
			}).play();



		this.resize(this._lastSize);

		this._startMessageCloseDelay();
	},

	_startMessageCloseDelay: function() {
		if (this.messageBox.__closeDelay) {
			this.messageBox.__closeTimeout = setTimeout(dojo.hitch(this, this.hideMessage),
				this.messageBox.__closeDelay);
		}
	},

	_resetMessageCloseDelay: function() {
		if (this.messageBox.__closeTimeout) {
			clearTimeout(this.messageBox.__closeTimeout);
		}
	},

	hideMessage: function() {
		delete this.messageBox.__closeDelay;
		delete this.messageBox.__closeTimeout;
		// if message no longer there no reason to play it out
		if (this.messageBox.domNode) {
			dojo.fadeOut({
					node: this.messageBox.domNode,
					onEnd: dojo.hitch(this, function() {
						this.messageBox.hideMessage();
					})
				}).play();
		}
	},

	_hideMessage: function() {
		this.resize(this._lastSize);
	},

	_toggleDetails: function() {
		this.resize(this._lastSize);
	},

	resize: function(size) {
		if (!size) {
			return;
		}

		this._lastSize = size;

		var cbContainer = dojo.contentBox(this.containerNode);
		var mbContainer = dojo.marginBox(this.containerNode);
		var mbHeader = dojo.marginBox(this.headerContainerNode);
		
		var contentSize = {};
		if (size.h) {
			contentSize.h = size.h - mbContainer.h + cbContainer.h - mbHeader.h;
			this.containerNode.style.minHeight = contentSize.h + "px";
		}
		if (size.w) {
			contentSize.w = size.w - mbContainer.w + cbContainer.w;
			this.containerNode.style.minWidth = contentSize.w + "px";
		}

		if (this.contentWidget && this.contentWidget.resize) {
			this.contentWidget.resize(contentSize);
		}
	},
	
	setTitle: function(){
		var editorInput= this.getEditorInput();
		if(editorInput.getPresentationName){
			var presentationName= editorInput.getPresentationName();
			if(presentationName){
				this.setEditorTitle(presentationName);
			}
		}
	},
	
	setEditorTitle: function(presentationName){
		net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(presentationName);
	},

	activate: function() {
		// called by the framework when the editor is the active editor
		this._active = true;
	},

	deactivate: function() {
		// called by the framework when the editor is no longer the active editor
		this._active = false;
	},

	isActive: function() {
		return this._active;
	},

	isDirty: function() {
		for (var groupId in this._dirtyGroups) {
			if (this._dirtyGroups[groupId].length > 0) {
				return true;
			}
		}
		return false;
	},

	addDirty: function(/*String|Object*/key, /*String?*/groupId) {
		if (this.isDirty()) {
			// call removeDirty here, just in case the title has changed after the document was already dirty (see defect 45337).
			// we also always need to call dirtyStateChanged (later in the function), even if it was already dirty.
			// both are needed to ensure the title will be properly updated in the Unsaved Work section of the related menu,
			// and that all dirty indicators (*) are always properly shown.
			this.removeDirty(key, groupId);
		}

		groupId = groupId || DEFAULT;

		var group = this._dirtyGroups[groupId];
		if (!group) {
			group = [];
			this._dirtyGroups[groupId] = group;
		} else {
			for (var i = 0, len = group.length; i < len; ++i) {
				if (group[i] == key) {
					return;
				}
			}
		}
		group.push(key);

		this.dirtyStateChanged(true);
	},

	removeDirty: function(/*String|Object?*/key, /*String?*/groupId) {
		// if key == null, delete the entire group

		if (!this.isDirty()) {
			return;
		}

		groupId = groupId || DEFAULT;

		var group = this._dirtyGroups[groupId];
		if (!group) {
			return;
		}

		if (key) {
			for (var i = 0, len = group.length; i < len; ++i) {
				if (group[i] == key) {
					group.splice(i, 1);
					break;
				}
			}
		}

		if (!key || group.length == 0) {
			delete this._dirtyGroups[groupId];
		}

		if (!this.isDirty()) {
			this.dirtyStateChanged(false);
		}
	},

	_clearDirty: function() {
		if (this.isDirty()) {
			this._dirtyGroups = {};
			this.dirtyStateChanged(false);
		}
	},

	dirtyStateChanged: function(isDirty) {
		if (this.suppressEditorUi) {
			return;
		}
		
		// listeners should connect to this method
		if (this.saveButton) {
			this.saveButton.setEnabled(isDirty && !this.isReadOnly);
		}

		if (this._dirtyIndicator) {
			if (isDirty) {
				this._dirtyIndicator.style.display = "";
			} else {
				this._dirtyIndicator.style.display = "none";
			}
		}
		
		/*Make sure to update the dirty status based on the other opened editors instances. As if isDirty is flase, this means that only the current editor
		 * is not dirty anymore, but does not guarantee that other editors instances in the current page are not dirty. Thus, we do not rely on isDirty flag
		 * for this case. */
		rmps.ui.editor.PageHelper.updateDirtyStatus(this.getEditorInput().getPageId());
	},

	_confirmRefresh: function() {
		if (!this.isDirty() || confirm(messages.confirmRefresh)) {
			this.refresh();
		}
	},

	refresh: function() {
		var id= this.getEditorInput().getPageId();
		if(!id){
			id= this.domNode;
		}
		var editorUtil = rmps.ui.editor.EditorUtil.getInstance(id);
		var isClosed;
		/*This is temp till we remove the editor util API, and start using page helper for all places */
		if(editorUtil){
			isClosed= editorUtil.closeEditor(this, true, true);	
		}
		if(!isClosed){
			rmps.ui.editor.PageHelper.getInstance(this.getEditorInput().getPageId()).closeEditor(this, true, true);
		}
		var timestamp= new Date().getTime();
		this.getEditorInput().addState("ts",timestamp);
		this.refreshHref(timestamp);
	},

	refreshHref: function(timestamp){
		window.location.href = this.getEditorInput().getPresentationUri();
	},

	destroy: function(){
		if(this.inputNode){
			this.inputNode.destroyRecursive();
		}
		this.inherited(arguments);
	},

	getEditorInput: function() {
		return this.editorInput;
	},

	/*
	 * Called prior to saving. If a validation error is present
	 * which will prevent save from completing then clients should
	 * populate push the error reason (String|Error) into the errors array.
	 * Client may also push a Deferred object to defer the validation. This allows
	 * the client to communicate with the server to perform validation. To add a validation
	 * failure the deferred must either perform an error call back or push errors into
	 * the errors array.
	 */
	validate: function(/*Array*/errors) {

	},

	save: function() {
		if (this.isReadOnly || !this.isDirty()) {
			return;
		}

		this.saveButton.setEnabled(false);

		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: messages.validating
		});

		var errors = [];
		var editor = this;
		var deferreds = this.validate(errors);
		
		if (deferreds && deferreds.length && deferreds.length > 0) {
			var def = new dojo.DeferredList(deferreds);
			def.then(function(results) {
				for (var i = 0; i < results.length; i++) {
					if (results[i][1].result == "error") {
						if (results[i][1].code) {
							errors.push(rmpw.forms.ui.ModelValidationUtil.getMessageForCode(results[i][1].code));
						} else {
							errors.push(results[i][1].message);
						}
					} else if (results[i][0] /*success*/ && results[i][1] instanceof Array) {
						errors = errors.concat(results[i][1]);
					}
				}

				if (errors.length > 0) {
					editor._processSaveErrors(errors);
				} else {
					editor._doSave();
				}
			 });
		} else {
			if (errors.length > 0) {
				this._processSaveErrors(errors);
			} else {
				this._doSave();
			}
		}
	},
	
	showSaveOk: function(){
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.OK,
			summary: messages.save_success,
			closable: true
		}, 5000);
	},

	onSaveComplete: function(){
		this.showSaveOk();
		this.setTitle();
	},

	_doSave: function() {
		this.saving = true;
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: messages.saving
		});
		
		var taskMonitor = new rmps.core.TaskMonitor(messages.saving);
		taskMonitor.setCanComplete(false);
		this.doSave(taskMonitor);

		var handle = this.connect(taskMonitor, "onComplete", function(errors) {
			this.saving = false;
			this.disconnect(handle);
			if (errors) {
				this._handleSaveErrors(errors);
			} else {
				// clear dirty state
				this._clearDirty();
				this.onSaveComplete();
			}
		});

		taskMonitor.setCanComplete(true);
	},

	_processSaveErrors: function(errors) {
		var processing = false;

		dojo.forEach(errors, function(d) {
			if (d instanceof dojo.Deferred && !d.__processed) {
				d.__processed = processing = true;
				d.addCallback(this, function() {
					errors.splice(dojo.indexOf(errors, d), 1);
					this._processSaveErrors(errors);
				});
				d.addErrback(this, function(error) {
					errors.splice(dojo.indexOf(errors, d), 1, error ? error : undefined);
					this._processSaveErrors(errors);
				});
			}
		}, this);

		if (!processing) {
			if (errors.length > 0) {
				this._handleSaveErrors(errors);
			} else {
				this._doSave();
			}
		}
	},

	_handleSaveErrors: function(errors) {
		if (!this.suppressEditorUi) {
			this.saveButton.setEnabled(true);
		}
		
		dojo.forEach(errors, function(error){
			var messageSummary = (error instanceof Error ? error.message : error);
				messageSummary = dojox.html.entities.encode(messageSummary).replace(/\n/g,"<br>");

			if (messageSummary.length === 0) {
				messageSummary = messages.save_failed;
			}

			var messageDetails = [];
			if (error.explanation && error.explanation.length > 0){
				messageDetails.push({ message: dojox.html.entities.encode(error.explanation).replace(/\n/g,"<br>") });
			}
			if (error.userAction && error.userAction.length > 0){
				messageDetails.push({ message: dojox.html.entities.encode(error.userAction).replace(/\n/g,"<br>") });
			}

			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messageSummary,
				details: messageDetails,
				closable: true
			});
		}, this);
	},
	
	appendChangeset: function() {
		if (jazz.app.context.get().changeset){
			return "&changeset=" + jazz.app.context.get().changeset;
		} else {
			return "";
		}
		
	},
	


	/*
	 * @see rmps.core.TaskMonitor
	 */
	doSave: function(taskMonitor) {

	}

});

})();
