/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.BaselineStatusEditor");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");

dojo.require("rmps.core.baseline.BaselineClient");
dojo.require("rmps.core.error");

dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.toolbar.Button");
dojo.require("rmps.core.util.IRmpsConstants");

dojo.requireLocalization("rmps.app.ui.internal.editors", "BaselineStatusEditorMessages");

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "BaselineStatusEditorMessages");
var COMPARE_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "compareWithPrevious");

var UA_KEY = "com.ibm.xtools.web.app.page_SnapshotsList001";

dojo.declare("rmps.app.ui.internal.editors.BaselineStatusEditor", rmps.app.ui.internal.editors.BaseEditor, {

	"class": "rmps-baseline-status",

	uaKey: UA_KEY,

	//init properties
	isReadOnly: true,
	
	//private properties
	_deferred: null,
	
	postCreate: function() {
		this.inherited(arguments);
		this.refresh();
	},
	
	addButtons: function(toolbar) {
		var button = new jazz.ui.toolbar.Button({
			href: rmps.core.util.IRmpsConstants.ADMIN_REQUEST_BASELINE,
			label: messages.createLinkText
		});
		
		toolbar.addWidget(button);		
		toolbar.addSeparator(2);		
		this.inherited(arguments);
	},

	refresh: function() {
		if (this._deferred) {
			return;
		}

		dojo.forEach(dijit.findWidgets(this.containerNode), function(widget) {
			if (widget.destroyRecursive) {
				widget.destroyRecursive();
			}
		});
		
		this.showLoading();

		this._deferred = rmps.core.baseline.BaselineClient.getBaselines(null, this.getEditorInput().projectId);
				
		this._deferred.addBoth(this, function() {
			delete this._deferred;
			this.hideMessage();
		});
		
		this._deferred.addCallback(this, function(baselines) {
		
			function baselineComparator(a, b) {
				return a.timestamp < b.timestamp ? 1 : a.timestamp == b.timestamp ? 0 : -1;
			}

			baselines.sort(baselineComparator);	

			var table = new jazz.ui.StyledTable({
				trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
				columns: [
					{
						label: messages.name
					},
/*					{
						label: "",
						className: "state-icon"
					},*/
					{
						label: messages.state,
						className: "state-label"
					},
					{
						label: messages.timestamp,
						className: "timestamp"
					},
					{
						label: messages.actions,
						className: "actions"
					}
				],
				showHover: true
			});

			table.containerNode = table.tbody;
			this.containerNode.appendChild(table.domNode);

			for (var i = 0; i<baselines.length; i++) {
				var row = new rmps.app.ui.internal.editors._BaselineRow({baseline: baselines[i], editor: this});
				table.tbody.appendChild(row.domNode);
				row.startup();
			}			
		});
		
		this._deferred.addErrback(this, function(error) {
			this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.error_loading,
					closable: true
				});
		});
	}
});

dojo.declare("rmps.app.ui.internal.editors._BaselineRow", [dijit._Widget], {

	baseline: null,
	editor: null,
	
	_deferred: null,
	_deleteDeferred: null,
	_stateImg: null,
	_stateCell: null,
	_popup: null,
	_popupContent: null,
	_actionBar: null,
	_createAction: null,
	_deleteAction: null,
	
	buildRendering: function() {
		this.domNode = this.containerNode = dojo.create("tr");
	},
	
	postCreate: function() {
		//baseline name
		var cell = dojo.create("td", null, this.domNode);
		var iconCell = dojo.create("img", {src: rmps.core.util.ContextTypes.SNAPSHOT.icon.uri, "class": "icon"}, cell);
		var nameLink = dojo.create("a", null, cell);
		nameLink.appendChild(dojo.doc.createTextNode(this.baseline.name));
		
		//initialize table row from the baseline object
		cell = dojo.create("td", null, this.domNode);
		//state image
		var imageCell = dojo.create("td", null, cell);
		this._stateImg = dojo.create("img", null, imageCell);				

		//state text
		this._stateCell = dojo.create("td", null, cell);		
		this._updateState();
		
		
		this._popup = new jazz.ui.HoverPopup({
			around: nameLink,
			onOpen: dojo.hitch(this, this._createPopupContents),
			maxWidth: 800,
			maxHeight: dojo.isIE ? 570 : 550
		});
		
		//created time
		cell = dojo.create("td", null, this.domNode);
		cell.appendChild(dojo.doc.createTextNode(
			dojo.date.locale.format(this.baseline.timestamp, {formatLength: "medium"})
		));
		
		//actions, if applicable
		var actionsTd = dojo.create("td", null, this.domNode);
		this._actionBar = new jazz.ui.SimpleToolbar();
		actionsTd.appendChild(this._actionBar.domNode);
		this._actionBar.startup();
		this._updateActions();		

		//if this is a baseline being created, schedule an automatic update of the status 
		if (this.baseline.status == rmps.core.baseline.BaselineStates.PENDING ||
				this.baseline.status == rmps.core.baseline.BaselineStates.IN_PROGRESS) {
			this._updateTimer = setTimeout(dojo.hitch(this, this._update), 10000); // 10 seconds
		}
	},
	
	_createPopupContents: function() {
		if (this._popupContent) {
			this._popupContent.destroyRecursive();
		}
		this._popupContent = new rmps.app.ui.internal.editors._BaselinePopupContent({baseline:this.baseline});
		this._popup.content = this._popupContent.domNode;
		
		//if the baseline is finished, its status won't change - so we don't need to recalculate popup content
		//each time the popup is opened
		if (this.baseline.status == rmps.core.baseline.BaselineStates.COMPLETED) {
			delete this._popup.onOpen;
		}
	},
	
	destroy: function() {
		this.inherited(arguments);
		
		if (this._updateTimer) {
			clearTimeout(this._updateTimer);
			delete this._updateTimer;
		}
		
		if (this._deferred) {
			this._deferred.cancel();
			delete this._deferred;
		}
		
		if (this._deleteDeferred) {
			this._deleteDeferred.cancel();
			delete this._deleteDeferred;
		}
		
		if (this._popupContent) {
			this._popupContent.destroyRecursive();
			delete this._popupContent;
		}
		
		if (this._popup) {
			this._popup.destroyRecursive();
			delete this._popup;
		}
	},
	
	/**
	 * Update the changeable parts of this row (i.e., the status).
	 */
	_update: function() {
		//request baseline from server
		this._deferred = rmps.core.baseline.BaselineClient.getBaseline(this.baseline.uri, this.editor.getEditorInput().projectId);
		
		this._deferred.addBoth(this, function(baseline) {
			delete this._deferred;
		});
		
		this._deferred.addErrback(this, function(error) {
			if (error instanceof Error) {
				//baseline has been deleted, or at least, no longer exists
				if (error.status == 404) {
					this.destroyRecursive();
				}
				return;
			}

			delete this._updateTimer;
		});
		
		this._deferred.addCallback(this, function(baseline) {
			var oldBaseline = this.baseline;
			this.baseline = baseline;

			//if old status != new status, update status img & text, and actions
			if (oldBaseline.status != baseline.status) {				
				this._updateState();
				this._updateActions();			
			}
			
			//if still not completed, schedule another update
			if (this.baseline.status != rmps.core.baseline.BaselineStates.COMPLETED) {
				this._updateTimer = setTimeout(dojo.hitch(this, this._update), 10000); // 10 seconds
			}
		});	

	},
	
	_updateState: function() {
		this._stateImg.src = this.baseline.status.icon;
		this._stateCell.innerHTML = "";
		this._stateCell.appendChild(dojo.doc.createTextNode(this.baseline.status.name));
	},
	
	_updateActions: function() {
		if (this.baseline.status == rmps.core.baseline.BaselineStates.COMPLETED) {
			if (!this._createAction) {
				this._createAction = this._actionBar.addButton(this, this._createStream, {
					label: messages.createStream,
					imgSrc: dojo.moduleUrl("rmps.app", rmps.core.util.ContextTypes.WORKSPACE.icon.uri)
				});
			}
			if (!this._deleteAction) {
				this._deleteAction = this._actionBar.addButton(this, this._deleteBaseline, {
					label: messages.deleteBaseline,
					imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/remove.gif")
				});
			}
			if (!this._compareAction) {
				this._compareAction = this._actionBar.addButton(this, this._compareBaselines, {
					label: "Compare with Previous",
					imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/search_resources.png")
				});
			}
		}
	},
	
	_createStream: function() {
		var link = location.href;
		var index = link.indexOf("#");
		if (index != -1) {
			link = link.substring(0, index);
		}
		link += rmps.core.util.IRmpsConstants.ADMIN_REQUEST_STREAM;
		link += "&snapshotUri=" + this.baseline.uri;
		link += "&snapshotName=" + this.baseline.name;
		location.href = link;
	},

	_compareBaselines: function() {
		var link = COMPARE_PATH + "?descendantUri=" + this.baseline.uri;
		var win = window.open(link,'compareViewer','width=800,height=600,resizable=yes,toolbar=no,menubar=no,location=no');
		win.focus();
	},
	
	_deleteBaseline: function() {
		this._actionBar.disableButton(this._deleteAction);
		
		//first, find out if/how many active reviews are asssociated with this baseline
		var deferred = rmps.core.baseline.BaselineClient.getBaselineLinks(this.baseline.uri, "review");
		deferred.addCallback(this, function(reviews) {
			this.editor.hideMessage();
			var numReviews = reviews.length;
			var confirmationMessage = null;
			
			if (numReviews > 0) {
				confirmationMessage = dojo.string.substitute(messages.confirmDeleteWithReview, [numReviews]);
			} else {
				confirmationMessage = messages.confirmDeleteNoReview; 
			}
			
			if (confirm(confirmationMessage)) {
				this._deleteDeferred = rmps.core.baseline.BaselineClient.deleteBaseline(this.editor.getEditorInput().projectId, this.baseline.uri);
				this._deleteDeferred.addCallback(this, function(args) {
					delete this._deleteDeferred;
					this.destroyRecursive();
				});

				this._deleteDeferred.addErrback(this, function(error) {
					delete this._deleteDeferred;
					var message;
					if (error.status === 403) {
						message = messages.error_request_permission;
					} else {
						message = messages.error_request_general;
					}
					this.editor.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: message,
						closable: true
					});
				});		
			} else {
				this._actionBar.enableButton(this._deleteAction);
			}
		});
		deferred.addErrback(this, function(error){
			this.editor.hideMessage();
			this._actionBar.enableButton(this._deleteAction);
		});
		
		this.editor.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: messages.checkingConditions
		});
		
	}

});

dojo.declare("rmps.app.ui.internal.editors._BaselinePopupContent", [dijit._Widget, dijit._Templated], {

	"class" : "rmps-baseline-status-popup",
	messages: messages,
	baseline: null,
	templateString: '<table>' +
						'<tr><th>${messages.details}</th></tr>' +
						'<tr><td>${messages.name}:</td><td dojoAttachPoint="_nameText"/></tr>' +
						'<tr><td>${messages.state}:</td><td dojoAttachPoint="_stateText"/></tr>' +
						'<tr><td>${messages.timestamp}:</td><td dojoAttachPoint="_timestampText"/></tr>' +
						'<tr><td>${messages.description}:</td><td dojoAttachPoint="_descriptionText"/></tr>' +						
					'</table>',
	
	postCreate: function() {
		//var iconCell = dojo.create("img", {src: rmps.core.util.ContextTypes.SNAPSHOT.icon.uri, "class": "icon"}, this._nameText);
		this._nameText.appendChild(dojo.doc.createTextNode(this.baseline.name));
		this._stateText.appendChild(dojo.doc.createTextNode(this.baseline.status.name));
		this._timestampText.appendChild(dojo.doc.createTextNode(dojo.date.locale.format(this.baseline.timestamp, {formatLength: "medium"})));
		var descriptionText = null;
		if (this.baseline.description) {
			descriptionText = this.baseline.description;
		} else {
			descriptionText = messages.noDescription;
		}
		this._descriptionText.appendChild(dojo.doc.createTextNode(descriptionText));
	}
});
})();