/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.CompactRenderingEditor");

dojo.require("rmps.graphics.compact.CompactRenderingViewer");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

var CompactRenderingViewer= rmps.graphics.compact.CompactRenderingViewer;

dojo.declare("rmps.app.ui.internal.editors.CompactRenderingEditor", [dijit._Widget, dijit._Templated], {
	
	templateString: '<div style="overflow: hidden; padding: 0px; position: relative; min-height: 0px; ' +
			        '			min-width: 0px; width:100%" ' +
					'			id="containerNode" class="rmps-graphics-diagram-editor-container" ' +
					'			dojoAttachPoint="containerNode">'+
					'</div>',
					
	editorInput: null, diagramNode: null, contentWidget: null, _active: false,
	
	postCreate: function() {
		this.inherited(arguments);		
		
		this.createContentWidget();
		
		/*Attach the editor to the Compact Rendering Main Container. The editor container node will be used
		 * afterwards with its viewer, so any further updates will be reflected to the Main Container */
		this.editorInput.diagramNode.appendChild(this.containerNode);
	},
	
	createContentWidget: function() {
		this.contentWidget = new rmps.app.ui.internal.editors.CompactRenderingEditor._Container();
		if (this.isActive()) {		
			this.contentWidget.viewDiagram({_editor: this});
		}
		return this._diagramContainer;
	},
	
	activate: function() {
		this._active = true;
		if (this.contentWidget) {
				this.contentWidget.viewDiagram({_editor: this});
		}
		
		/*no need to resize 
		 * 
		 * this.resize(size);
		 * */
	},
	
	startup: function() {
		if (this.contentWidget) {
			this.contentWidget.startup();
		}
	},
	
	resize: function(size) {
		if (this.contentWidget && this.contentWidget.resize) {
			this.contentWidget.resize(size);
		}
	},
	
	deactivate: function() {
		this._active = false;
	},
	
	isActive: function() {
		return this._active;
	},
	
	getEditorInput: function() {
		return this.editorInput;
	}
	
});

dojo.declare("rmps.app.ui.internal.editors.CompactRenderingEditor._Container", null, {
	
	_viewer: null,
	
	viewDiagram: function(args) {
		if (this._viewer) {
			this._viewer.destroyRecursive();
		}
		this._viewer = this._createDiagramViewer(args);
		this._viewer.startup();
	},
	
	startup: function() {
		if (this._viewer) {
			this._viewer.startup();
		}
	},
	
	resize: function(size) {
		if (this._viewer && this._viewer.resize) {
			this._viewer.resize(size);
		}
	},
	
	_createDiagramViewer: function(args) {
		var editor= args._editor;
		
		/*Set the new viewer with the given arguments */
		this._viewer= new CompactRenderingViewer({
			editorInput: editor.editorInput,
			containerDiagram: editor.containerNode
		});
		return this._viewer;
	}
});

})();