/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){

dojo.provide("rmps.app.ui.internal.editors.ConfigureSpaceEditor");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");

dojo.require("rmps.core.projects.ProjectClient");
dojo.require("rmps.core.util.ContextUtils");

dojo.require("jazz.ui.MessageBox");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

var ProjectClient = rmps.core.projects.ProjectClient;
var ContextUtils = rmps.core.util.ContextUtils;

var UA_KEY = "com.ibm.xtools.rmps.web.app.page_ConfigureSpaceEditor001";

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.ui.internal.editors", "ConfigureSpaceEditorMessages");
var messages= dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "ConfigureSpaceEditorMessages");
var DIRTY_GROUP_ID = "rmps.app.ui.internal.editors.ConfigureSpaceEditor.dirty";

/**
 * Editor for viewing and setting the VVC Space of a project.
 */
dojo.declare("rmps.app.ui.internal.editors.ConfigureSpaceEditor", [rmps.app.ui.internal.editors.BaseEditor], {

	"class": "rmps-app-editors-configure-space",
	isRefreshable: false, //override from superclass
	spaceSelectionChoice: null,
	existingSpacesCombo: null,
	selectedSpace: null,
	spaces: [],
	uaKey: UA_KEY,
	_defaultSpaceName: null,

	postCreate: function() {
		this.inherited(arguments);
		this.showLoading();
		
		var descriptionContainer = dojo.create('div', {"class" : "main"}, this.containerNode);
		var spaceContainer = dojo.create('div', {'class' : 'main'}, this.containerNode);
		this._newControlsContainer = dojo.create('div', null, this.containerNode);
		
		dojo.create('label', {innerHTML : messages.spaceChoiceDescription}, descriptionContainer);

		this.existingSpacesCombo = dojo.create("select");
		spaceContainer.appendChild(this.existingSpacesCombo);
		
		dojo.connect(this.existingSpacesCombo, 'onchange', dojo.hitch(this, function(event) {
			if (event.target.value === messages.newSpace) {
				this._appendNewControls();
			} else {
				this._removeNewControls();
			}
			this.addDirty(this, DIRTY_GROUP_ID);
		}));
		
		this.initSpaceSelection();
	},

	startup: function() {
		this.inherited(arguments);
		
		if (!this.selectedSpace) {
			this.addDirty(this, DIRTY_GROUP_ID);
		}
	},

	initSpaceSelection: function() {

		var projectId = this.editorInput.projectId ? this.editorInput.projectId : jazz.app.context.get().itemId;
		var dpls = [];
		dpls.push(this.getAllSpaces());
		
		if (projectId) {
			dpls.push(ContextUtils.getProjectSpace(projectId));
		}
		
		var dl = new dojo.DeferredList(dpls);
		dl.then(dojo.hitch(this, function(result) {
			this._currentSpace = result[1] ? result[1][1] : null;
			this.spaces = result[0][1];

			if (!this._currentSpace) {
				var item = dojo.create("option", null, this.existingSpacesCombo);
				var itemText = dojo.doc.createTextNode(messages.newSpace);
				item.appendChild(itemText);
				this.existingSpacesCombo.disabled = false;
				this._appendNewControls();
			} else {
				this.existingSpacesCombo.disabled = true;
				this._removeNewControls();
			}
			
			if (this.spaces) {
				this.spaces.sort(this.compareSpaces);
				for (var space in this.spaces) {
					var _space = this.spaces[space];
					if (this._currentSpace === _space.uri) {
						this.selectedSpace = space;
					}
					var item = dojo.create("option", null, this.existingSpacesCombo);
					var itemText = dojo.doc.createTextNode(_space.title);
					item.appendChild(itemText);
				}
				this.existingSpacesCombo.selectedIndex = this.selectedSpace;
			}
		}));
	},
	
	_appendNewControls: function() {
		if (!this._newControls) {
			this._newControls = new rmps.app.ui.internal.editors._NewSpaceControls({parent: this});
			this._newControlsContainer.appendChild(this._newControls.domNode);
		}
	},
	
	_removeNewControls: function() {
		if (this._newControls) {
			dojox.xml.parser.removeChildren(this._newControlsContainer);
			this._newControls.destroyRecursive();
			delete this._newControls;
		}
	},

	compareSpaces: function(a, b) {
		return a.title.localeCompare(b.title);
	},

	getAllSpaces: function() {
		var args = {
				url: net.jazz.ajax.contextRoot() + "/queryvvc/allspaces?excludeDsp=true",
				headers: {
					"Accept" : "application/json"
				},
				handleAs: "json",
				sync: true,
				load: function(response, ioArgs) {
					var spaces = [];
					for (var uri in response) {
						var r = {uri: uri, title: response[uri]};
						spaces.push(r);
					}
					return spaces;
				}
		};
		return jazz.client.xhrGet(args);
	},
	
	appendChanges: function(data) {
		if (this._newControls) {
			if (this._newControls.name.value && this._defaultSpaceName !== this._newControls.name.value) {
				data.spaceName = this._newControls.name.value;
				if (data.spaceName && data.spaceName.length > 0) {
					// trim leading and trailing spaces
					data.spaceName =  encodeURIComponent(data.spaceName.replace(/^\s\s*/, '').replace(/\s\s*$/, ''));
				}
			}
			if (this._newControls.description.value) {
				data.spaceDescription =  encodeURIComponent(this._newControls.description.value);
			}
		} else {
			this._calculateSelectedSpace();
			data.spaceUri = this.spaces[this.selectedSpace].uri;
		}
	},

	_doSave: function() {
		// check for name collision with existing spaces
		if (this._newControls) {
			var spaceName = this._newControls.name.value;
			if (spaceName && spaceName.length > 0) {
				// trim leading and trailing spaces
				spaceName = spaceName.replace(/^\s\s*/, '').replace(/\s\s*$/, '');
				for (var i in this.spaces) {
					var space = this.spaces[i];
					if (space.title === spaceName) {
						this.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
							summary: messages.duplicateName
						});
						this.saveButton.setEnabled(true);
						return;
					}
				}
			}
		}

		this.inherited(arguments);
	},
	
	defaultSpaceNameUpdate: function(spaceName) {
		this._defaultSpaceName = spaceName;
		if (this._newControls) {
			this._newControls._setDefaultSpaceName();
		}
	},
	
	validateRequired: function(saveObj) {
		if (this._newControls) {
			var spaceName = this._newControls.name.value;
			if (spaceName) {
				// trim leading and trailing spaces
				spaceName = spaceName.replace(/^\s\s*/, '').replace(/\s\s*$/, '');
				for (var i = 0; i < this.spaces.length; i++) {
					var space = this.spaces[i];
					if (space.title === spaceName) {
						return messages.duplicateName;
					}
				}
			}
		}
	},

	/**
	 *Overridden from superclass.  Saves the value of the selected space
	 */
	doSave: function(taskMonitor, projectId) {
		if (!projectId) {
			projectId = this.editorInput.projectId;
		}
		var retVal;
		if (!this._newControls) {
			retVal = this.saveExistingSpace(taskMonitor, projectId);
		} else {
			retVal = this.saveNewSpace(taskMonitor, projectId);
		}
		
		if (!this.suppressEditorUi) {
			retVal.addCallback(this, function() {
				if (retVal.ioArgs.xhr.status === 201) {
					this.showSaveOk();
					this.disableAll();
					// clear dirty flag first before reload
					this._clearDirty();
	
						// we need to refresh the page
					var defautWorkspaceUri = retVal.ioArgs.xhr.getResponseHeader("Location");
					if (defautWorkspaceUri) {
						var currentURL = location.search;
						if (currentURL && currentURL.length > 1) {
							currentURL += "&";
						}
						currentURL += "rmps.context=" + encodeURIComponent(defautWorkspaceUri);
						location.search = currentURL;
					} else {
						window.location.reload();
					}
				}
			});
		}
		
		var task = taskMonitor.beginTask(taskMonitor.name, 1);
		task.handleDeferredWork(retVal, 1);
		return retVal;
	},

	saveNewSpace: function(taskMonitor, projectId) {
		var query = {};
		query.projectId = projectId ? projectId : this.editorInput.projectId;
		var __spaceName =this._newControls.name.value;
		if (__spaceName && __spaceName !== "") {
			query.spaceName =  encodeURIComponent(__spaceName);
		}
		var __spaceDescription = this._newControls.description.value;
		if (__spaceDescription && __spaceDescription !== "") {
			query.spaceDescription =  encodeURIComponent(__spaceDescription);
		}
		var link = net.jazz.ajax.contextRoot() + "/queryvvc/setnewspace?" + dojo.objectToQuery(query);

		var args = {
				url: link,
				sync: true,
				error: dojo.hitch(this, function(e) {
					taskMonitor._error(null, e);
				})
		};
		return jazz.client.xhrPost(args);
	},
	
	_calculateSelectedSpace: function() {
		if (this._currentSpace) {
			this.selectedSpace = this.existingSpacesCombo.selectedIndex;
		} else {
			// Skip the "New Space..." value
			this.selectedSpace = this.existingSpacesCombo.selectedIndex - 1;
		}
	},

	saveExistingSpace: function(taskMonitor, projectId) {
		if (!projectId) {
			projectId = this.editorInput.projectId;
		}
		var link = net.jazz.ajax.contextRoot() + "/queryvvc/setexistingspace?projectId=" + escape(projectId);
		this._calculateSelectedSpace();
		var __spaceUri = this.spaces[this.selectedSpace].uri;
		link += "&spaceUri=" + escape(__spaceUri);

		var args = {
				url: link,
				sync: true,
				error: dojo.hitch(this, function(e) {
					taskMonitor._error(null, e);
				})
		};
		return jazz.client.xhrPost(args);
	},

});

dojo.declare("rmps.app.ui.internal.editors._NewSpaceControls", [dijit._Widget, dijit._Templated], {
	messages : messages,
	parent: null,
	_userEnteredName: false,
	
	templateString: '<div>' + 
						'<table class="content details">' +
							'<tbody>' + 
								'<tr>' +
									'<td class="title">${messages.spaceName}</td>' +
									'<td class="input">' +
										'<input class="field" dojoAttachPoint="name" ' +
												'dojoAttachEvent="onchange: _onValueChange, onkeypress: _onValueChange, onpaste: _onValueChange, oninput: _onValueChange" ' +
												'name="spaceName"/>' +
									'</td>' +
								'</tr>' +
								'<tr>' +
									'<td class="title">${messages.spaceDescription}</td>' +
									'<td class="input">' +
										'<textarea class="field" dojoAttachPoint="description" ' +
										'dojoAttachEvent="onchange: _onValueChange, onkeypress: _onValueChange, onpaste: _onValueChange, oninput: _onValueChange" ' +
										'name="spaceDescription"></textarea>' +
									'</td>' +
								'</tr>' +
							'</tbody>' +
						'</table>',
						
	postCreate: function() {
		this.inherited(arguments);
		this._userEnteredName = this.name.value;
		this._setDefaultSpaceName();
	},

	_onValueChange: function(e) {
		this.parent.addDirty(this, DIRTY_GROUP_ID);
		this._userEnteredName = this.name.value;
		this._setDefaultSpaceName();
	},

	_setDefaultSpaceName: function() {
		if (!this._userEnteredName) {
			this.name.value = this.parent._defaultSpaceName ? this.parent._defaultSpaceName : "";
		}
	}
	
});

})();

