/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

(function(){
	dojo.provide("rmps.app.ui.internal.editors.CopyQueryDialog");
	
	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("dijit.form.TextBox");
	dojo.require("dijit.form.FilteringSelect");
	dojo.require("dojo.store.Memory");
	
	dojo.require("net.jazz.ajax.BootstrapProperties");
	
	dojo.require("rmps.core.projects.ProjectClient");
	
	dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
	
	dojo.requireLocalization("rmps.app.ui.internal.editors", "CopyQueryDialogMessages");
	var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "CopyQueryDialogMessages");
	
	var LOADING_ID = "loading_id";
	var PROJECT_NAME = "http://jazz.net/xmlns/rmps/groups/1.0/projectName";
	
	dojo.declare("rmps.app.ui.internal.editors.CopyQueryDialog", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("rmps.app", "ui/internal/editors/templates/CopyQueryDialog.html"),
		widgetsInTemplate: true,
		
		constructor: function() {
			this.messages = messages;
		},
		
		postCreate: function() {
			var text = dojo.string.substitute(this.messages.copyOfTitle, {
				title: this.queryLabel
			});
			this._queryName.set('value', text);
			
			var tempStore = new dojo.store.Memory({
				data: [ {label : this.messages.loading,
					id: LOADING_ID} ]
			});
			
			this._projects = new dijit.form.FilteringSelect({
				value: LOADING_ID,
		        store: tempStore,
		        searchAttr: "label",
		        fetchProperties: {
		        	sort: [ {attribute: "label"} ]
		        },
		        onChange: dojo.hitch(this, this.onProjectChange)
			});
			this._projectsTableData.appendChild(this._projects.domNode);
			dojo.style(this._projects.domNode, {
				width: '100%'
			});
			this._projects.startup();
			
			this._projects.set('disabled', true);
			this._okButton.setAttribute('disabled', true);
			
			this._projectsDataDef = rmps.core.projects.ProjectClient.getAllProjects(this.onlyDSP);
			this._projectsDataDef.then(dojo.hitch(this, function(projects) {
				this._projectsData = [];
				for (var uri in projects) {
					if (!this.projectsFilter || this.projectsFilter(projects[uri])) {
						this._projectsData.push({
							id: uri,
							label: projects[uri][PROJECT_NAME][0].value
						});
					}
				}
				var projectsDataStore = new dojo.store.Memory({
					data: this._projectsData
				});
				this._projects.set('store', projectsDataStore);
				this._projects.set('disabled', false);
				currentProjectUri = dojo.getObject("jazz.app.currentApplication.context");
				this._projects.set('value', currentProjectUri ? currentProjectUri : "");
			}));
		},
		
		show: function () {
			this.dialog = new jazz.ui.Dialog({
				contentNode: this.containerNode,
				primaryTitle: this.getTitle(),
				width: "600px"
			});
		},
		
		getTitle: function() {
			return this.dialogTitle || this.messages.copyQueryTitle; 
		},

		
		_closeDialog: function() {
			if (this.dialog) {
				this.dialog.close();
				this.dialog.destroyRecursive();
			}
		},
		
		_onOK: function() {
			if (this.onOK && dojo.isFunction(this.onOK)){
				this.onOK(this.serverDesc);
			}
			this._closeDialog();
		},
		
		_onCancel: function() {
			this._closeDialog();
		},
		
		onProjectChange: function() {
			if (this._projects.isValid()) {
				this._okButton.removeAttribute('disabled');
			} else {
				this._okButton.setAttribute('disabled', true);
			}
		},
		
		getProjectId: function() {
			var item = this._projects.item;
			if (!item) {
				return null;
			} else {
				var uri = item.id;
				return uri.substring(uri.lastIndexOf("/")+1);
			}
			
		},
		
		getProjectName: function() {
			return this._projects.item.label;
		},
		
		getEditQueryUri: function(queriUri) {
			return net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/web/projects/" +
			 encodeURIComponent(this.getProjectName()) + 
			 rmps.app.ui.internal.pages.design.DesignUtil.getQueryUri(queriUri);
		},
		
		getEditViewpointUri: function(viewpointUri) {
			return net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/web/projects/" +
			 encodeURIComponent(this.getProjectName()) + 
			 rmps.app.ui.internal.pages.design.DesignUtil.getViewpointUri(viewpointUri);
		},
		
		getNewTitle: function() {
			return this._queryName.get('value');
		}
	});
	
	
	
}());