/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){

dojo.provide("rmps.app.ui.internal.editors.DefaultOslcContextEditor");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.app.ui.internal.editors.SelectConfigurationFragment");
dojo.require("rmps.core.projects.ProjectClient");

dojo.require("jazz.ui.MessageBox");

var ProjectClient = rmps.core.projects.ProjectClient;

var DEFAULT_CONTEXT_PROPERTY = "defaultOslcContext";
var DIRTY_GROUP_ID = "rmps.projectProperties.DefaultOslcContextEditor.dirty";

var UA_KEY = "com.ibm.xtools.web.app.page_DefaultOslcContext001";

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.ui.internal.editors", "DefaultOslcContextEditorMessages");
var messages= dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "DefaultOslcContextEditorMessages");

/**
 * Editor for viewing and setting the default OSLC context of a project.
 */
dojo.declare("rmps.app.ui.internal.editors.DefaultOslcContextEditor", [rmps.app.ui.internal.editors.BaseEditor], {

	isRefreshable: false, //override from superclass

	uaKey: UA_KEY,

	_context: null, //Uri; the currently selected context
	_originalContext: null, //Uri; the original stored context 

	postCreate: function() {
		this.inherited(arguments);
		this._loadContext();
	},
	
	_loadContext: function() {
		this._context = null;
		this._originalContext = null;
		//load the current value of the default OSLC context
		//TODO would it be better to do all this in the editor input loading method?
		var getPropertyDeferred = ProjectClient.getProjectProperty(this.editorInput.projectConfigurationsInfo.projectId, DEFAULT_CONTEXT_PROPERTY);
		getPropertyDeferred.addCallback(this, function(propertyValue){
			//value is the URL of some stream or baseline
			if (propertyValue.length > 0) {
				this.contentWidget.setContext(propertyValue);
				this._context = propertyValue;
				this._originalContext = propertyValue;
			} else {
				//treat it as property not set
				this.contentWidget.updateLabel(null);
			}
		});
		getPropertyDeferred.addErrback(this, function(error) {
			if (error.status === 404) {
				//property is not set; display appropriate message
				this.contentWidget.updateLabel(null);
			} else {
				this._generaltErrorCallback(error);
			}
		});		
	},
	
	_generaltErrorCallback: function(error) {
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: messages.generalErrorMessage,
			details: [{message: messages['generalErrorMessage.explanation'] + "<br>" + messages['generalErrorMessage.useraction']}]
		});
	},
	
	//overridden from superclass
	createContentWidget: function() {
		return new rmps.app.ui.internal.editors.SelectConfigurationFragment({
			contextSelected: dojo.hitch(this, this.contextSelected),
			projectConfigurationsInfo: this.getEditorInput().projectConfigurationsInfo,
			errorCallback: dojo.hitch(this, this._generaltErrorCallback)
		});
	},
	
	/**
	 *Overridden from superclass.  Saves the value of the OSLC context.
	 */
	doSave: function(taskMonitor) {
		if (this._originalContext == this._context) {
			return;
		}
		
		var deferred = ProjectClient.setProjectProperty(this.editorInput.projectConfigurationsInfo.projectId, DEFAULT_CONTEXT_PROPERTY, this._context, true);
		deferred.addErrback(this, function(error) {
			if (error.status === 403) {
				return new Error(messages.error_save_permission);
			} else {
				return new Error(messages.generalErrorMessage + "\n" 
				                + messages["generalErrorMessage.explanation"] + "\n"
				                + messages["generalErrorMessage.useraction"]);
			}
		});
		
		var task = taskMonitor.beginTask(messages.saveTaskLabel, 1);
		task.handleDeferredWork(deferred, 1);
		return deferred;
	},
	
	lightRefresh: function() {
		this._loadContext();
		this.removeDirty(this.contentWidget, DIRTY_GROUP_ID);
	},

	/**
	 * Called when a new context is selected.  If contextUrl is null, the existing context should be cleared.
	 */
	contextSelected: function(contextUrl) {
		this._context = contextUrl;
		if (this._context == this._originalContext) {
			this.removeDirty(this.contentWidget, DIRTY_GROUP_ID);
		} else {
			this.addDirty(this.contentWidget, DIRTY_GROUP_ID);
		}
	}

});

})();

