/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.DiagramEditor");

dojo.require("rmps.app.ui.internal.editors.PropertiesEditor");

/* rmpw.jazz */

dojo.require("rmpw.jazz.ui.LineStyler");

/* rmpw.widgets */

dojo.require("rmpw.widgets.pickers.ToolPicker");
dojo.require("rmpw.widgets.pickers.ColorPicker");
dojo.require("rmpw.widgets.pickers.TextPicker");

/* rmps.web.comment.sketch */

dojo.require("rmps.web.comment.sketch.behavior.Scribbler");
dojo.require("rmps.web.comment.sketch.behavior.LineMaker");
dojo.require("rmps.web.comment.sketch.behavior.OvalMaker");
dojo.require("rmps.web.comment.sketch.behavior.RectangleMaker");
dojo.require("rmps.web.comment.sketch.behavior.CloudMaker");
dojo.require("rmps.web.comment.sketch.behavior.TextMaker");

/* rmpw.diagrams */

dojo.require("rmpw.diagrams.edit.actions.DeleteElement");
dojo.require("rmps.web.comment.sketch.commands.EditStyle");
dojo.require("rmps.web.comment.sketch.commands.EditFontStyle");

/* rmps.graphics */

dojo.require("rmps.graphics.ui.DiagramWidget");
dojo.require("rmps.graphics.ui.Configuration");

/* dijit, dojo */

dojo.require("dijit.Toolbar");
dojo.require("dijit.form.Button");

dojo.require("dojo.i18n");

dojo.require("rmps.app.core.util");

dojo.require("rmpw.widgets.TouchGuide");

dojo.require("rmpw.common.vcs.Repository");

dojo.requireLocalization("rmps.app.ui.internal.editors", "DiagramEditorMessages");

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "DiagramEditorMessages");

dojo.declare("rmps.app.ui.internal.editors.DiagramEditor", rmps.app.ui.internal.editors.PropertiesEditor, {

	mobile: null,
	
	container: null,
	_graph: undefined,
	_history: null, //[]
	
	linetools: null,
	connection: null,
	topics: null,	
	tools: null,
	toggleButtons: null,
	toggleCount: 0,
	lastSelection: null,
	
	dialogs: null,
	toolspresent: false,
	
	SELECT: 0,
	TOOLS: 1,
	LINE: 2,
	PENCIL: 3,
	TEXT: 4,
	
	constructor: function() {
		
		this._history = [];
		
		this.mobile = rmps.app.core.util.checkIfMobile();
		
		if( this.mobile ){
			rmps.app.core.util.closeNavigationPanel();
		}
		
		var uagent = navigator.userAgent.toLowerCase();
	},
	
	_createDiagramContainer: function() {
		this.container = new rmps.app.ui.internal.editors.DiagramEditor._Container({editor: this});
		if (this.isActive()) {		
			this.container.viewDiagram(this._graph);
		}
		return this.container;
	},
	
	addTabs: function() {
		this.addTab("diagram", messages.diagram, dojo.hitch(this, this._createDiagramContainer));
		this.inherited(arguments);
		this.addTab("relatedElements", messages.relatedElements, "rmps.app.ui.internal.editors.views.DiagramRelatedElements", {resourceInfo: this.resourceInfo});
	},
	
	activate: function() {
		
		this.inherited(arguments);
		
		var args = this.getEditorInput().getParams(), graph;
		
		if (args && args.graph) {
			graph = args.graph;
			this.getEditorInput().addState("graph", graph);
		} else {
			graph = null;
			if (this._graph === undefined) {
				this._graph = null;
			}
			this.getEditorInput().removeState("graph");
		}
		
		if (graph) {
			var present = false;
		
			for (var h = 0; h < this._history.length; h++) {
				if (this._history[h] == graph) {
					present = true;
					break;
				}
			}
			
			if (!present) {
	 			this._history.push(graph);
			}
		}
		
		if (this._graph != graph || (this.container && !this.container.viewer)) {
			this._graph = graph;
			if (this.container) {
				this.container.viewDiagram(this._graph);
			}
		}
	}
});

dojo.declare("rmps.app.ui.internal.editors.DiagramEditor._Container", dijit._Widget, {

	editor: null,
	viewer: null,
	_prevSize: null,
	
	// TODO reenable caching once comment sketches can be made against
	// individual compartment diagrams
//	_viewers: null,
	
	postCreate: function() {
//		this._viewers = {};
		if (dojo.isIE) {
			this.domNode.style.zoom = 1;
		}		
	},
	
	destroy: function() {
		this.viewer.destroyRecursive();
		this.inherited(arguments);
	},
	
	_getGraphKey: function(graph) {
		return graph || "default";
	},
	
	viewDiagram: function(graph) {
		if (this.viewer) {
			this.viewer.domNode.style.display = "none";
			this.viewer.destroyRecursive();
		}
		
//		this.viewer = this._viewers[this._getGraphKey(graph)];
//		if (!this.viewer) {
			this.createDiagramViewer(graph);
//			this._viewers[this._getGraphKey(graph)] = this.viewer;
			this.domNode.appendChild(this.viewer.domNode);
//		} else {
//			this.viewer.domNode.style.display = "";
//		}
		
		if (this._started && !this.viewer._started) {
			this.viewer.startup();
		}
		
		this.resize(this._prevSize);
	},
	
	startup: function() {
		this.inherited(arguments);
		if (this.viewer) {
			this.viewer.startup();
		}	
		
		if( this.mobile ){
			if( localStorage.getItem( "guide" ) != "viewed" ){
				var tg = new rmpw.widgets.TouchGuide();
				this.domNode.appendChild( tg.domNode );	
			}	
		}
		
	},
	
	resize: function(size) {
		
		this._prevSize = size;
		
		if( this.viewer != null ){
			this.viewer.resize(size);
		}
	},
	
	addTool: function(item){
     	this.toolbar.addChild(item);
    },
    
    saveDiagram: function(){
    	
    },
    
    /* resetZoom - to resolve */
    
    resetZoom: function(){
    	this.viewer.slider.attr( 'value', 1 );
    	this.viewer.rootLayer.setLocation( 0, 0 );
    },
    
    buildArgs: function( type ){
		var commandArgs = {
			svgParent: this.viewer.sketchController.renderMapping.sources.svgRoot,
			shape: type,
			mode: "MULTIPLE_HIT",
			viewport: this.commandController.viewer.viewport	
		};
		
		return commandArgs;
	},
	
	undo: function(){
		this.viewer.getCommandController( this.commentConfig.canvas ).commandStack.undo();
	},
	
	redo: function(){
		this.viewer.getCommandController( this.commentConfig.canvas ).commandStack.redo();
	},
    
	selectAll: function(){
		var mappings = this.viewer.sketchController.root.diagram.__editMapping.children;
		dojo.forEach( mappings, this.selectionController.select );
	},
	
	deleteSelected: function(){
		
		if( this.selectionController.getSelection().length > 0 ){
			var action = new rmpw.diagrams.edit.actions.DeleteElement();	
			action.run({}, { viewer: this.viewer.sketchViewer });
		}
		
		this.selectionController.deselectAll();
	},
	
	clear: function(){
		
		var mappings = this.viewer.sketchController.root.diagram.__editController.children;
		var deleteMappings = [];
		for( var item = 0, editController; editController = mappings[item]; item++ ){	
			deleteMappings.push(editController);
		}
		for( var item = 0, editController; editController = deleteMappings[item]; item++ ){	
			editController.policies.clearAction.run({targetController: editController});
		}	
		
		for( var item = 0; item < this.dialogs.length; item++ ){
			this.dialogs[item].hide();
		}
		
		this.dialogs = [];
	},
	
	shapeChoice: function( val ){ 
		this.selectedTool = val;

		this.lastSelection = this.TOOLS;
		this.toggleButtons[ this.TOOLS ].attr( 'checked', true );
		
		this.changeSelectedButton( this.TOOLS );
		
		switch( this.selectedTool ){

			case "RMPW_CLOUD":
				if(!this.cloudtool){
					this.cloudtool = new rmps.web.comment.sketch.behavior.CloudMaker( this.commandController );
				}
				this.cloudtool.capture( this.buildArgs( "Cloud" ) );
				
				this.shapePicker.attr( 'iconClass', 'rmps-graphics-diagram-layout cloudIcon');

				break;
	
			case "RMPW_RECTANGLE":
				if(!this.recttool){
					this.recttool = new rmps.web.comment.sketch.behavior.RectangleMaker( this.commandController );
				}
				this.recttool.capture( this.buildArgs( "Rectangle" ) );	
				
				this.shapePicker.attr( 'iconClass', 'rmps-graphics-diagram-layout rectangleIcon');

				break;
	
			case "RMPW_OVAL":
				if(!this.ovaltool){
					this.ovaltool = new rmps.web.comment.sketch.behavior.OvalMaker( this.commandController );
				}
				this.ovaltool.capture( this.buildArgs( "Oval" ) );	
				
				this.shapePicker.attr( 'iconClass', 'rmps-graphics-diagram-layout ovalIcon');

				break;				
		}
	},
	
	addText: function(){
		
		this.lastSelection = this.TEXT;
		
		if(!this.texttool){
			this.texttool = new rmps.web.comment.sketch.behavior.TextMaker( this.commandController );
		}
		
		this.texttool.capture( this.buildArgs( "Text" ) );	
		
	},
	
	addLine: function(){
		
		this.lastSelection = this.LINE;
		
		if(!this.linetool){
			this.linetool = new rmps.web.comment.sketch.behavior.LineMaker( this.commandController );
		}
		this.linetool.capture( this.buildArgs( "Line" ) );	
	},
	
	addScribble: function(){	
		
		this.lastSelection = this.PENCIL;
		
		if(!this.scribbler){
			this.scribbler = new rmps.web.comment.sketch.behavior.Scribbler( this.commandController );
		}
		
		this.scribbler.capture( this.buildArgs( "Scribble" ) );			
	},
	
	addTextTools: function(){
		this.textfillpicker = new rmpw.widgets.pickers.ColorPicker({type:"TEXT"});
		this.addTool( this.textfillpicker );
		this.textfillpicker.startup();
	},
	
	addStyleTools: function(){
		
		this.addTextTools();
		
		if( this.linetools == false ){							
			this.fillpicker = new rmpw.widgets.pickers.ColorPicker({type:"FILL"});				
			this.addTool( this.fillpicker );	
			this.styleTools.push( this.fillpicker );
			this.fillpicker.startup();	
			this.lineStyler.addStyleTools( this.toolbar );	
			this.linetools = true;
			this.filltool = true;
		}	
		
		if( this.texttools == false ){	
			
			this.textStyler = new rmpw.widgets.pickers.TextPicker();
			this.textStyler.setStyles = dojo.hitch( this, "setFontStyle" );
			this.addTool( this.textStyler );
			this.styleTools.push( this.textStyler );
			
			this.texttools = true;
		}
		
	    this.toolspresent = true;
	},
	
	removeTextTools: function(){
		this.toolbar.removeChild( this.textfillpicker );
	},
	
	removeStyleTools: function(){
		
		for( var tool in this.styleTools ){
			this.toolbar.removeChild( this.styleTools[tool] );
		}
		
		this.lineStyler.removeStyleTools();
		
		this.styleTools = [];
		this.linetools = false;
		this.filltool = false;
		this.texttools = false;
		
		this.removeTextTools();
	},
	
	setSelectionMode: function(){
		
		this.lastSelection = this.SELECT;
		
		if( this.linetool ){
			this.linetool.release();
		}
		
		if( this.scribbler ){
			this.scribbler.release();
		}
		
		if( this.ovaltool ){
			this.ovaltool.release();
		}
		
		if( this.recttool ){
			this.recttool.release();
		}
		
		if( this.cloudtool ){
			this.cloudtool.release();
		}
		
		if( this.texttool ){
			this.texttool.release();
		}
	},
	
	addDrawingTools: function(){
		
		this.drawingTools = [];
				
		var undo = this.addButton( "Undo", dojo.hitch( this, "undo" ), "undoIcon" );
		this.drawingTools.push( undo );
		
		var redo = this.addButton( "Redo", dojo.hitch( this, "redo" ), "redoIcon" );		
		this.drawingTools.push( redo );
		
		var selectionMode = this.addToggleButton( "Select", dojo.hitch( this, "setSelectionMode" ), "selectIcon" );
		this.SELECT = this.toggleCount;
		this.registerToggle( selectionMode, this.SELECT );
		this.drawingTools.push( selectionMode );
				
	/*	var selectAll = this.addButton( "Select All", dojo.hitch( this, "selectAll" ), "selectAll" );
		this.drawingTools.push( selectAll );
		
		var deleteSelected = this.addButton( "Delete", dojo.hitch( this, "clear" ), "deleteIcon" );
		this.drawingTools.push( deleteSelected ); */
		
		this.shapePicker = new rmpw.widgets.pickers.ToolPicker();
		this.shapePicker.toolChoice = dojo.hitch( this, "shapeChoice" );
		this.shapePicker.onChange = dojo.hitch( this, "changeSelectedButton", 3 );
		this.shapePicker.toolid = this.toggleCount;
		this.TOOLS = this.toggleCount;
		this.registerToggle( this.shapePicker, this.TOOLS );
		
		this.addTool(this.shapePicker);
				
		this.drawingTools.push(this.shapePicker);
		
	   	var textButton = this.addToggleButton( "Text", dojo.hitch( this, "addText" ), "textIcon" );
		this.TEXT = this.toggleCount;
		this.registerToggle( textButton, this.TEXT );
		this.drawingTools.push(textButton);
		
		var lineButton = this.addToggleButton( "Line", dojo.hitch( this, "addLine" ), "lineIcon" );
		this.LINE = this.toggleCount;
		this.registerToggle( lineButton, this.LINE );
	   	this.drawingTools.push(lineButton);
		
		var scribbleButton = this.addToggleButton( "Pencil", dojo.hitch( this, "addScribble" ), "scribbleIcon" );
		this.PENCIL = this.toggleCount;
		this.registerToggle( scribbleButton, this.PENCIL );
		this.drawingTools.push(scribbleButton);
		
		this.lastSelection = this.SELECT;
		this.toggleButtons[ this.SELECT ].attr( 'checked', true );
				
		this.addStyleTools();		
	},
	
	registerToggle: function( button, id ){
		this.toggleButtons[this.toggleCount] = button;	
	   	this.toggleCount = this.toggleCount +1;
	},
    
	removeDrawingTools: function(){
		
		for( var tool in this.drawingTools ){
			this.toolbar.removeChild( this.drawingTools[tool] );
		}
		
		this.drawingTools = [];
		
		this.toggleCount = 0;
		
		this.toggleButtons = [];
		
		this.removeStyleTools();
	},
	
	changeSelectedButton: function( id ){
	                  
	    for( var button = 0; button < this.toggleButtons.length; button++ ){
	    	
	    	if( button != this.lastSelection ){	    		
	    		this.toggleButtons[ button ].attr( 'checked', false );
	    		
	    		if( this.lastSelection == this.TOOLS ){
	    			this.toggleButtons[ button ].reset();
	    			this.toggleButtons[ button ].attr( 'checked', false );
	    		}
	    	}
	    }         	
	},
	
    addButton: function( label, call, icon, show ){ 
   	 
	   	 if( show == null ){ 
	   		 show = false; 
	     }
	   	 
	   	 var button = new dijit.form.Button({ 
	    		label: label, 
	    		showLabel: show, 
	    		onClick: call,
	    		iconClass: icon,
	    		style: "font-size:10px"});
	   	 
	   	 this.addTool( button );
	   	 
	   	 return button;
    },
    
    addToggleButton: function(label, call, icon, show ){ 
   	 
	   	 if( show == null ){ 
	   		 show = false; 
	   	 }
	   	 
	   	 var button = new dijit.form.ToggleButton({
	   		 	toolid: this.toggleCount,
	    		label: label, 
	    		showLabel: show, 
	    		onClick: call,
	    		iconClass: icon,
	    		onChange: dojo.hitch( this, "changeSelectedButton", this.toggleCount ),
	    		style: "font-size:10px"});
	   	 
	   	 this.addTool( button );

	   	 return button;
    },
	
    toggleOutline: function(){   	
    	this.outlineState = !this.outlineState;   	 
     	this.viewer.setOutline( this.outlineState ); 
    },
    
    executeStyle:function( args ){
    	var cmd = new rmps.web.comment.sketch.commands.EditStyle();
    	this.commandController.commandStack.execute( cmd, args ); 
    },
    
    updateStroke: function( stroke ){
    	this.executeStyle( {
			stroke: stroke,
			fill: null,
			text: null,
			nodes: this.selection
		}); 
	},
	
	selectionChanged: function(){
		
		this.selection = this.selectionController.getSelection();
		
		if( this.selection.length > 0 && this.toolspresent == false ){
			this.removeStyleTools();
			this.addStyleTools();		
    	}
	},
	
	setLineColor: function( message ){		
		this.updateStroke( message ); 	
	},
	
	setLineWidth: function( message ){	
		this.updateStroke( { width: message.width } ); 
	},
	
	setLineStyle: function( message ){
		this.updateStroke( { dash: message.style } ); 
	},
	
	setFillColor: function( message ){				
		this.executeStyle({
			stroke: null,
			fill: message,
			text: null,
			nodes: this.selection
		}); 		
	},
	
	setTextColor: function( message ){
		this.executeStyle({
			stroke: null,
			fill: null,
			text: message,
			nodes: this.selection
		}); 	
	},
	
	setFontStyle: function( data ){
		
		if( this.selection ){			
			var font = this.textStyler.getFontObject();			
			var cmd = new rmps.web.comment.sketch.commands.EditFontStyle();
	    	this.commandController.commandStack.execute( cmd, { font: font, nodes: this.selection } );
		}
	},
    
	createDiagramViewer: function(graph) {
		
		var uri = this.editor.resourceInfo.uri;
		
    	this.configuration = new rmps.graphics.ui.Configuration();
    	this.commentConfig = new rmps.graphics.ui.Configuration();
    	this.selectionConfig = new rmps.graphics.ui.Configuration();
    	
	    dojo.addClass( this.domNode, 'rmps-graphics-diagram-layout' );
	    
	    this.domNode.style.overflow = "hidden";
    	
    	var d = document.createElement( 'div' );
    	this.domNode.appendChild( d );
    	this.configuration.canvas = d; 
    	
    	var commentCanvas = document.createElement( 'div' );
    	this.domNode.appendChild( commentCanvas );
    	this.commentConfig.canvas = commentCanvas; 
    	
    	var selectionCanvas = document.createElement( 'div' );
    	this.domNode.appendChild( selectionCanvas );
    	this.selectionConfig.canvas = selectionCanvas;  
    	
    	this.repository = new rmpw.common.vcs.Repository();
    	
    	var resourceInfo = { repository: this.repository };
    	
    	this.viewer = new rmps.graphics.ui.DiagramWidget({
    		resourceInfo: resourceInfo,
			configuration: this.configuration,
			commentConfig: this.commentConfig,
			selectionConfig: this.selectionConfig,
			resourceUri: uri,
			graph: graph,
			history: this.editor._history,
			commentingWidget: this.editor.commentingWidget,
			editor: this
		});
    	
    	this.toolbar = this.viewer.toolbar;
    	
    	this.layout = {};
    	this.layout.toolbar = this.viewer.toolbar;
	   	
	   	this.outlineState = true;
    	
    	this.selectionController = this.viewer.sketchLayer.viewer.getSelectionController();
    	this.connection = dojo.connect( this.selectionController, "onSelectionChanged", this, "selectionChanged" );

    	this.commandController = this.viewer.sketchLayer.viewer.getCommandController();	
		this.commandController.modelingPage = this;    	
    	
		this.lineStyler = new rmpw.jazz.ui.LineStyler(); 	
		this.lineStyler.setLineColor = dojo.hitch( this, "setLineColor" ); 
		this.lineStyler.setLineWidth = dojo.hitch( this, "setLineWidth" ); 
		this.lineStyler.setLineStyle = dojo.hitch( this, "setLineStyle" );
		
		this.linetools = false;
		this.texttools = false;
		
		this.topics = rmpw.widgets.Events.topics;
		
		this.subscriptions = [];
		this.subscriptions.push( dojo.subscribe( this.topics.FILL_COLOR, dojo.hitch( this, "setFillColor" ) ) );
		this.subscriptions.push( dojo.subscribe( this.topics.TEXT_COLOR, dojo.hitch( this, "setTextColor" ) ) );
		
		this.filltool = false;
		
		this.styleTools = [];
				
		this.toggleButtons = [];
		
		this.toggleCount = 0;
		
		this.dialogs = [];
		    			
    	return this.viewer;
	}
});

})();
