/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){

dojo.provide("rmps.app.ui.internal.editors.DomainsManagementFragment");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.xml.parser");
dojo.require("jazz.ui.SwapListDialog");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.util.i18n");
dojo.require("rmps.core.RootServices");
dojo.require("rmps.app.ui.internal.util.DomainRegistryUtil");


dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.ui.internal.editors", "DomainsManagementFragmentMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "DomainsManagementFragmentMessages"); //$NON-NLS-1$ //$NON-NLS-2$
var PROJECT_DASHBOARD_URL_PREFIX = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "web"); //$NON-NLS-1$
var DOMAIN_REG_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "domainRegistry");
var DOMAIN_EDIT_ACTION_PREFIX = "#action=rmps.app.design.edit&resource="; //$NON-NLS-1$

dojo.declare("rmps.app.ui.internal.editors.DomainsManagementFragment", [dijit._Widget, dijit._Templated], {
	
	templateString: '<div>' +
						'<div class="description" dojoAttachPoint="_messageContainer">' +
						'</div>' +
						'<div dojoAttachPoint="_domainsContainer" class="userTable content">' +
						'</div>' +
					'</div>',
					
	_addedDomains: [],
	_removedDomains: [],
	_originalDomains: [],
	_eTag: null,
	
	messages: messages,
	
	dirtyStateChanged: null, // function callback
	projectConfigurationsInfo: null,
	
	postCreate: function() {
		this.inherited(arguments);
		this._init();
	},
	
	_getDomains: function() {
		var originalIndex = 0, addedIndex = 0, removedIndex = 0, domains = [];
		while (originalIndex < this._originalDomains.length) {
			if (!this._addedDomains[addedIndex] || this._originalDomains[originalIndex].index < this._addedDomains[addedIndex].index) {
				if (!this._removedDomains[removedIndex] || this._originalDomains[originalIndex].index < this._removedDomains[removedIndex].index) {
					domains.push(this._originalDomains[originalIndex]);
					originalIndex++;
				} else if (this._originalDomains[originalIndex].index === this._removedDomains[removedIndex].index) {
					originalIndex++;
					removedIndex++;
				} else {
					// should not happen
					removedIndex++
				}
			} else {				
				domains.push(this._addedDomains[addedIndex]);
				addedIndex++;
			}
		}
		for (; addedIndex < this._addedDomains.length; addedIndex++) {
			domains.push(this._addedDomains[addedIndex]);
		}
		return domains;
	},

	_init: function() {
		this.projectConfigurationsInfo.getDomainsInfo(dojo.hitch(this, function(data, eTag) {
			this._originalDomains = [];
			this._addedDomains = [];
			this._removedDomains = [];
			this._eTag = eTag;
			for (var i = 0; i < data.length; i++) {
				if (data[i].selected) {
					this._originalDomains.push({
						label: data[i].label,
						id: data[i].id,
						projectName: data[i].projectName,
						context: data[i].context,
						index: i
					});
				};
			}
			this._refreshUI();
		}));
	},
	
	_refreshUI: function() {
		dojox.xml.parser.removeChildren(this._domainsContainer);				
		dojox.xml.parser.removeChildren(this._messageContainer);
		var domains = this._getDomains();
		var link;
		if (domains.length === 0) {
			link = jazz.util.i18n.addMessage(dojo.create("div", {}, this._messageContainer), messages.empty_domains)[0]; //$NON-NLS-1$
			dojo.toggleClass(this._messageContainer, "error", true); //$NON-NLS-1$			
		} else {
			link = jazz.util.i18n.addMessage(dojo.create("div", {}, this._messageContainer), messages.assign_domains_text)[0]; //$NON-NLS-1$
			dojo.toggleClass(this._messageContainer, "error", false); //$NON-NLS-1$	
			this._initDomainsTable(domains);
		}
		link.onclick = dojo.hitch(this, function(e) {
			e = e || window.event;
			if (dojo.isFunction(e.preventDefault)) {
				e.preventDefault();
			} else {
				e.returnValue = false;
			}
			this.modify();
		});
		
	},
	
	_initDomainsTable: function(domains) {
		var columnData = [];
		columnData.push({ label: messages.domain_name });
		columnData.push({ label: messages.domain_project_name });

		var table = new jazz.ui.StyledTable({
			columns: columnData,
			showHover: false
		});
		this._domainsContainer.appendChild(table.domNode);
		table.domNode.style.width = "100%";
		
		for (var i = 0; i < domains.length; i++) {
			var domain = domains[i];
			var linkHref = PROJECT_DASHBOARD_URL_PREFIX +"/projects/" + domain.projectName; //$NON-NLS-1$
			var projectName = domain.projectName;
			
			var tr = document.createElement("tr");
			
			td = document.createElement("td");
			td.style.textAlign = "left";
			var link = document.createElement("a"); //$NON-NLS-1$
			link.href = linkHref + DOMAIN_EDIT_ACTION_PREFIX + jazz.util.html.escape(domain.id);
			link.appendChild(document.createTextNode(jazz.util.html.escape(domain.label)));
			td.appendChild(link);
			tr.appendChild(td);
			
			td = document.createElement("td");
			td.style.textAlign = "left";
			link = document.createElement("a"); //$NON-NLS-1$
			link.href = linkHref; 
			link.appendChild(document.createTextNode(domain.projectName));
			td.appendChild(link);
			tr.appendChild(td);
			
			table.tbody.appendChild(tr);
		}	
	},
	
	areDomainsAdded: function() {
		return this._addedDomains.length > 0;
	},

	changesPending: function() {
		return this._addedDomains.length > 0 || this._removedDomains.length > 0;
	},
	
	validateRequired: function() {
		if (this._originalDomains.length === this._removedDomains.length && this._addedDomains.length == 0) {
			return messages.no_domains_associated_error;
		}
	},
	
	refresh: function() {
		this._init();
	},

	doSave: function(taskMonitor, projectId) {		
		var i, data = [], deferred, task, domains = this._getDomains(), remind = false;
		if (!projectId) {
			projectId = this.projectConfigurationsInfo.projectId;
			remind = this.areDomainsAdded();
		}
		for (i = 0; i < domains.length; i++) {
			data.push({ uri: domains[i].id });
		}
		deferred = rmps.app.ui.internal.util.DomainRegistryUtil.saveDomainsInfo(data, projectId, this._eTag, undefined, remind);		
		deferred.addErrback(this, function(error) {
			if (error.status === 403) {
				return new Error(messages.error_save_permission);
			} else {
				return new Error(messages.generalErrorMessage + "\n" 
				                + messages['generalErrorMessage.useraction'] + "\n" 
				                + messages['generalErrorMessage.explanation']);
			}
		});
		task = taskMonitor.beginTask(messages.task_saveDomains, 1);
		task.handleDeferredWork(deferred, 1);
		return deferred;
	},
	
	defaultDomainVersions: function(streamUri) {
		if (this._addedDomains.length === 0) {
			return;
		}
		
		var domainProjects = [];
		var cache = {};
		var dspUri;
		for (var i = 0; i < this._addedDomains.length; i++) {
			dspUri = this._addedDomains[i].context;
			if (!cache[dspUri]) {
				domainProjects.push(dspUri);
				cache[dspUri] = true;
			}
		}
		var query = {};
		var url = DOMAIN_REG_PATH + "/setDomainVersions";
		var data = { dsps: domainProjects };
		var defaultDomainVersionsRequest = new dojo.Deferred();
		var deferred = rmps.core.client.xhrPut({
			url: url,
			putData: dojo.toJson(data),
			handleAs: 'json',
			headers: {
				"Accept" : "application/json",
				"Content-Type" : "application/json"
			}
		}, false, streamUri);
		deferred.addCallback(function() {
			defaultDomainVersionsRequest.resolve(arguments);
		});
		deferred.addErrback(function() {
			rmps.core.client.xhrPost({
				url: DOMAIN_REG_PATH + "/setDomainVersionsReminder",
			}, false, streamUri);
			defaultDomainVersionsRequest.reject(messages.error_setDefaultDomainVersions);
		});
		return defaultDomainVersionsRequest;
	},
	
	modify: function() {
		var i, j, item, selected = [], unselected = [], domains = [];
		
		var dialog = new jazz.ui.SwapListDialog({
			dialogTitleText: messages.domainsDialogTitle,
			unselectedListTitleText: messages.domainsDialogUnselectedTitle,
			selectedListTitleText: messages.domainsDialogSelectedTitle,
			sortField: "index",
			iconMap: [],
			onOK: dojo.hitch(this, function(dialog) {
				var domainsAssociated = dialog.getSelectedEntries();
				i = 0, j = 0;
				this._addedDomains = [];
				this._removedDomains = [];
				while (i < this._originalDomains.length) {
					if (!domainsAssociated[j]) {
						this._removedDomains.push(this._originalDomains[i]);
						i++;
					} else {
						if (this._originalDomains[i].index === domainsAssociated[j].index) {
							i++;
							j++;
						} else if (this._originalDomains[i].index < domainsAssociated[j].index) {
							this._removedDomains.push(this._originalDomains[i]);
							i++;
						} else {
							this._addedDomains.push(domainsAssociated[j]);
							j++;
						}
					}
				}
				for (; j < domainsAssociated.length; j++) {
					this._addedDomains.push(domainsAssociated[j]);
				}
				dialog.destroy();
				this._refreshUI();
				this.dirtyStateChanged(this.changesPending());
			}),
			onCancel: function(dialog) {
				dialog.destroy();
			}
		});

		// Show the dialog
		dialog.show();
		dialog.showLoading(true);

		// Get the domains
		this.projectConfigurationsInfo.getDomainsInfo(dojo.hitch(this, function(data, eTag) {
			i = 0;
			j = 0;
			domains = this._getDomains();
			this._eTag = eTag;
			selected = [];
			unselected = [];
			while (i < data.length) {
				item = {
					label: jazz.util.html.escape(data[i].label),
					id: data[i].id,
					projectName: data[i].projectName,
					context: data[i].context,
					index: i
				};
				if (domains[j] && item.index === domains[j].index) {
					j++;
					selected.push(item);
				} else {
					unselected.push(item);
				}
				i++;
			}	
			dialog.setUnselectedEntries(unselected);
			dialog.setSelectedEntries(selected);
			dialog.showLoading(false);
		}));	
	},
	
});

})();

