/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.EditorWithComments");

dojo.require("rmps.app.ui.internal.editors.TabbedEditor");

dojo.require("rmps.core.subscriptions.SubscriptionClient");
dojo.require("rmps.core.user.UserClient");

dojo.require("rmps.ui.commenting.CommentingWidget");
dojo.require("rmps.ui.SidebarContainer");

dojo.require("rmps.app.ui.internal.util.UrlUtil");

dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");

dojo.requireLocalization("rmps.app.ui.internal.editors", "EditorWithCommentsMessages");

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "EditorWithCommentsMessages");
var util = rmps.app.ui.internal.util.UrlUtil;

dojo.declare("rmps.app.ui.internal.editors.EditorWithComments", rmps.app.ui.internal.editors.TabbedEditor, {
	
	// properties
	commentingWidget: null,
	
	// private properties
	_firstActivation: false,
		
	postCreate: function() {
		
		dojo.addClass(this.domNode, "rmps-app-editors-comments");
		
		this.commentingWidget = new rmps.ui.commenting.CommentingWidget({
			resourceInfo: this.resourceInfo, 
			showResourceLabel: this.showResourceLabel(),
			sResourceLabelLink: false,
			userInfo: jazz.app.currentApplication.config.user,
			externalSave: true,
			snapshotFilterEnabled: this.snapshotFilterEnabled(),
			snapshotSettingEnabled: this.snapshotSettingEnabled()
		});
		
		this.addCommentingWidgetListeners(this.commentingWidget);
		
		this.inherited(arguments);
	},
	
	addButtons: function(toolbar) {
		if(this.resourceInfo && this.resourceInfo.uri){
			// subscriptions
			if (!this._subscriptionButton) {
				this._subscriptionButton = toolbar.addButton(this, this._toggleSubscription, {
					label: messages.subscribe,
					imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/subscribe_co.gif")
				});
				if(!util.isUri(this.resourceInfo.uri)){
				    this._subscriptionButton.domNode.className += "-disabled";
				}
			}
			toolbar.hideItem(this._subscriptionButton);
			
			var userUri = rmps.core.user.UserClient.getUserUri(jazz.app.currentApplication.config.user.userId);
			
			var deferred = rmps.core.subscriptions.SubscriptionClient.getSubscriptions(this.resourceInfo.uri, userUri);
			deferred.addCallback(this, this._handleSubscriptions);
			deferred.addErrback(this, function(error) {
				if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
					rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
				} else {
					rmps.core.error.createError(error).dispatch();
				}
			});
			
			// Add history button
			var historyButton = com.ibm.team.vvc.shared.web.ui.context.ContextUtils.buildHistoryButtonForResource({
				conceptUri: this.resourceInfo.conceptUri ? this.resourceInfo.conceptUri : this.resourceInfo.uri
			});
			toolbar.addWidget(historyButton);
		}
		
		this.inherited(arguments);
	},	
	
	addCommentingWidgetListeners: function(commentingWidget){
		this.connect(commentingWidget, "onShowMessage", this._onShowMessage);
		this.connect(commentingWidget, "onSaveComplete", this.showSaveOk);
		this.connect(commentingWidget, "onEndEditingComment", this.subscribe);
		this.connect(commentingWidget, "onHideMessage", this._onHideMessage);
		//TODO under what circumstances should we automatically unsubscribe if at all?
		//this.connect(commentingWidget, "onCancelEditingComment", this.unsubscribe);
	},
		
	showResourceLabel: function(){
		/*Enabled by default */
		return true;
	},
	
	snapshotFilterEnabled: function(){
		/*Enabled by default */
		return true;
	},

	snapshotSettingEnabled: function(){
		/*Enabled by default */
		return true;
	},
	
	createContentWidget: function() {
		var contentWidget = this.inherited(arguments);
		return this.createSidebar(contentWidget);
	},
	
	createSidebar: function(contentWidget){
		var sidebar = new rmps.ui.SidebarContainer();
		sidebar.addSidebar(this.commentingWidget, false, true, true);
		sidebar.setContent(contentWidget);
		
		// Aligns the commenting header top line with the tab line.
		this.commentingWidget.__sidebar.domNode.style.paddingTop = dojo.isIE < 8 ? "3px" : "2px";
		
		this._sidebar = sidebar;
		
		return sidebar;
	},

	_onShowMessage: function(args, closeDelay) {
		this.showMessage(args, closeDelay);
	},
	
	_onHideMessage: function(){
		this.hideMessage();
	},
	
	activate: function() {
		
		this.inherited(arguments);
		
		var args = this.getEditorInput().getParams();
		if (args && args.comment) {
			var commentObj = this.commentingWidget.getComment(args.comment);
			if (commentObj) {
				this.commentingWidget.selectComments(commentObj);
			} else {
				var handle = this.connect(this.commentingWidget, "onLoad", function() {
					this.disconnect(handle);
					var commentObj = this.commentingWidget.getComment(args.comment);
					if (commentObj) {
						this.commentingWidget.selectComments(commentObj);
					}
				});
				this.commentingWidget.refresh();
			}
		} else if (!this._firstActivation) {
			this.commentingWidget.refresh();
		}
		
		this._firstActivation = true;
	},
	
	validate: function(errors) {
		this.inherited(arguments);
		this.commentingWidget.validate(errors);
	},
	
	doSave: function(taskMonitor) {
		this.inherited(arguments);
		this.commentingWidget.doSave(taskMonitor);
	},
	
	_handleSubscriptions: function(subscriptions) {
		// If any subscriptions are retrieved then the user is already subscribed to this resource
		this._subscribed = subscriptions && subscriptions.length > 0;
		
		this._updateSubscriptionButton();
		this.toolbar.showItem(this._subscriptionButton);
	},
	
	_toggleSubscription: function() {
		this._subscriptionToggled = !this._subscriptionToggled;

		var message = messages.subscribing;
		if (this._subscribed) {
			message = messages.unsubscribing;
		}
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: message
		});

		var taskMonitor = new rmps.core.TaskMonitor(message);
		taskMonitor.setCanComplete(false);
		this.doSubscribe(taskMonitor);
		
		var handle = this.connect(taskMonitor, "onComplete", function(errors) {
			this.disconnect(handle);
			
			if (errors) {
				this._subscriptionToggled = !this._subscriptionToggled;
				this.hideMessage();
			} else {
				this._updateSubscriptionButton();
				this.onSubscribeComplete();
			}
		});

		taskMonitor.setCanComplete(true);	
		
	},
	
	isSubscribed: function() {
		return (this._subscribed || this._subscriptionToggled) && !(this._subscribed && this._subscriptionToggled);
	},
	
	subscribe: function() {
		if (!this._subscribed && !this._subscriptionToggled) {
			this._toggleSubscription();
		}
	},
	
	unsubscribe: function() {
		if (!this._subscribed && this._subscriptionToggled) {
			this._toggleSubscription();
		}
	},

	onSubscribeComplete: function(){
		var message = messages.unsubscribeSuccess;
		if (this._subscribed) {
			message = messages.subscribeSuccess;
		}
		
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.OK,
			summary: message,
			closable: true
		});
	},
	
	handleSubscribeErrors: function(error, ioArgs) {
		if (!dojo._isDocumentOk(ioArgs.xhr)) {
			error = rmps.core.error.createXhrError(ioArgs).getGenericError();
		} else {
			error = rmps.core.error.createError(error).getGenericError();
		}
		if (error.status == 403) {
			error.message = messages.error_permission;
		}
		error.dispatch();
		return error		
	},
	
	_updateSubscriptionButton: function() {
		
		// if subscribed, show the unsubscribe icon
		if (this.isSubscribed()) {
			this.toolbar.updateButton(this._subscriptionButton, {
				label: messages.unsubscribe,
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/unsubscribe_co.gif")
			});
		// else unsubscribed, show the subscribe icon
		} else {
			this.toolbar.updateButton(this._subscriptionButton, {
				label: messages.subscribe,
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/subscribe_co.gif")
			});
		}
	},
	
	/*
	 * @see rmps.core.TaskMonitor
	 */
	doSubscribe: function(taskMonitor) {
		var userUri = rmps.core.user.UserClient.getUserUri(jazz.app.currentApplication.config.user.userId);
		
		// save subscription if toggled
		if (this._subscriptionToggled) {
			var task, deferred;
			if (this._subscribed) {
				task = taskMonitor.beginTask("Unsubscribe", 1);
				deferred = rmps.core.subscriptions.SubscriptionClient.unsubscribe(this.resourceInfo.uri, userUri);
				deferred.addCallback(this, function(result) {
					this.removeDirty("subscriptionToggled");
					this._subscribed = false;
					this._subscriptionToggled = false;
				});
				deferred.addErrback(this, function(error) {
					this.handleSubscribeErrors(error, deferred.ioArgs)
				});
				task.handleDeferredWork(deferred, 1);
			} else {
				task = taskMonitor.beginTask("Subscribe", 1);
				deferred = rmps.core.subscriptions.SubscriptionClient.subscribe(this.resourceInfo.uri, userUri);
				deferred.addCallback(this, function(result) {
					this.removeDirty("subscriptionToggled");
					this._subscribed = true;
					this._subscriptionToggled = false;
				});
				deferred.addErrback(this, function(error) {
					this.handleSubscribeErrors(error, deferred.ioArgs)
				});
				task.handleDeferredWork(deferred, 1);
			}
			
		}
	}		
});

})();
