/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("rmps.app.ui.internal.editors.ExplorerFilterEditor");

	dojo.require("dojo.DeferredList");
	dojo.require("rmps.app.ui.internal.sections.ExplorerFilterSection");
	dojo.require("rmps.ui.explorer.ExplorerSetting");
	dojo.require("rmps.app.ui.internal.editors.BaseEditor");
	dojo.require("rmps.core.RootServices");
	
	dojo.require("dojo.i18n");
	dojo.requireLocalization("rmps.app.ui.internal.editors", "ExplorerFilterEditorMessages");
	var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "ExplorerFilterEditorMessages");
	
	var PROJECT_AREA_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.JP_0_6, "projectAreas");
	
	dojo.declare("rmps.app.ui.internal.editors.ExplorerFilterEditor", rmps.app.ui.internal.editors.BaseEditor, {

		"class": "rmps-app-editors-explorer-filter-definition",

		explorerFilterSection: null,
		eventHandles: null,					/*Used by other sections */
    	_filtersToBeAdded: null,			/*Filters To Be Added */
    	_filtersToBeModified: null,			/*Filters To Be Modified */
    	_filtersToBeDeleted: null,			/*Filters To Be Deleted */
    	_settingsChanged: false,			/*Settings should be saved */
			
		constructor: function(){
			this._filtersToBeAdded = [];
	    	this._filtersToBeModified = [];
	    	this._filtersToBeDeleted = [];
			this.eventHandles= [];
		},
	
		postCreate: function() {
			this.inherited(arguments);
			this.showLoading();
			this._init();
		},
		
		_init: function(){
			this._createSections();
		},
	
		_createSections: function(){
			/*Destroy the container node widgets recursively */
			this._destroyRecursive(this.containerNode);
	
			/*Create Link Types Section */
			this.explorerFilterSection = new rmps.app.ui.internal.sections.ExplorerFilterSection({
				projectId: this.editorInput.projectId,
			});
			this.explorerFilterSection.startup(this);	
			
			this.hideMessage();
		},
		
		lightRefresh: function() {
			this._createSections();
		},
	
		_destroyRecursive: function(node){
			dojo.forEach(dijit.findWidgets(node), function(widget) { 
				if (widget.destroyRecursive) {
					widget.destroyRecursive();
				}
			});
		},
	
		validate: function(errors) {
			var updates = this._filtersToBeAdded;
			updates = updates.concat(this._filtersToBeModified);
			for (var index=0; index<updates.length; index++){
				var filterrow = updates[index];
				var dataSet = this.explorerFilterSection.getDataForRow(filterrow);
				var cells = dataSet.cells;
				var cell = cells[0]; // The "name" cell
				var value = cell.children[1].value;
				if (dojo.trim(value.toString()) == "" && dojo.indexOf(errors, messages.missingNameError) < 0) {
					errors.push(messages.missingNameError);
				}
				if (!dataSet.data || !dataSet.data.excludedTypes || dataSet.data.excludedTypes.length < 1) {
					if (dojo.indexOf(errors, messages.missingTypesError) < 0) {
						errors.push(messages.missingTypesError);
					}
				}
			}
		},
	
		doSave: function(taskMonitor) {
			this.inherited(arguments);
	
			// Add filters
			var task = null;
			if (this._filtersToBeAdded.length > 0) {
				task = taskMonitor.beginTask(messages.saveExplorerFilters, this._filtersToBeAdded.length + 1);
			}
			var listSaveDef = [], i, dataSet, cells, fields, values, index, cell, fieldName, 
				value, newValue, def, row, data, allDeferreds = [];
			
			for (i = 0; i < this._filtersToBeAdded.length; i++) {
				var filterrow = this._filtersToBeAdded[i];
				dataSet = this.explorerFilterSection.getDataForRow(filterrow);
				cells = dataSet.cells;
				fields = dataSet.fieldNames;
				values = dataSet.values;
				
				data = new rmps.ui.sections.ExplorerFilter();
				// Add fields to data
				for (index=0; index<cells.length; index++){
					cell = cells[index];
					fieldName = fields[index];
					value = values[fieldName];
					
					newValue = cell.children[1].value;
					if (newValue!= value){
						dojo.setObject(fieldName, newValue, data);	
						cell.children[0].value = newValue;
					}
				}
				// Add filter definition to data
				if (dataSet.data && dataSet.data.excludedTypes) {
					data.excludedTypes = dataSet.data.excludedTypes;
				}
				// Set the new filter as row data
				dataSet.data = data;
	
				// Create the filter
				def = data.create(this.editorInput.projectId);
				listSaveDef.push(def);
				task.handleDeferredWork(def, 1);
			}
			var d1 = new dojo.DeferredList(listSaveDef);
			allDeferreds.push(d1);
			d1.addCallback(this, function(result) {
				for (var cur = 0; cur < this._filtersToBeAdded.length; cur++) {
					var filterrow = this._filtersToBeAdded[cur];
					this.removeDirty(filterrow);
				}
				this._filtersToBeAdded = [];
			});
			if (task != null) {
				task.handleDeferredWork(d1, 1);
				task = null;
			}
	
			// Update filters
			if (this._filtersToBeModified.length > 0) {
				task = taskMonitor.beginTask(messages.updateExplorerFilters, this._filtersToBeModified.length + 1);
			}
			var listUpdateDef = [];
			for (i = 0; i < this._filtersToBeModified.length; i++) {
				row = this._filtersToBeModified[i];
				dataSet = this.explorerFilterSection.getDataForRow(row);
				fields = dataSet.fieldNames;
				values = dataSet.values;

				var oldFilter = dataSet.data;
				var newFilter = oldFilter;
				cells = dataSet.cells;
				for (index=0; index<cells.length; index++){
					cell = cells[index];
					fieldName = fields[index];
					value = values[fieldName];
					
					newValue = cell.children[1].value;
					if (newValue!= value){
						dojo.setObject(fieldName,newValue, newFilter);
						cell.children[0].value = newValue;	
					}
				}
				
		        def = newFilter.update();
		        listUpdateDef.push(def);
				task.handleDeferredWork(def, 1);
			}
			var d2 = new dojo.DeferredList(listUpdateDef);
			allDeferreds.push(d2);
			d2.addCallback(this, function(result) {
	        	for (var cur = 0; cur < this._filtersToBeModified.length;cur++) {
					var filter = this._filtersToBeModified[cur];
					this.removeDirty(filter);
				}
				this._filtersToBeModified= [];
			});
			if (task != null) {
				task.handleDeferredWork(d2, 1);
				task = null;
			}
	
			// Delete filters
			if (this._filtersToBeDeleted.length > 0) {
				task = taskMonitor.beginTask(messages.deleteExplorerFilters, this._filtersToBeDeleted.length + 1);
			}
			var listDeleteDef = [];
			for (i=0; i<this._filtersToBeDeleted.length; i++){
				row = this._filtersToBeDeleted[i];
				data = this.explorerFilterSection.getDataForRow(row).data;
				if(data){
					def = data.del();
					listDeleteDef.push(def);
					task.handleDeferredWork(def, 1);
				}
			}
			var d3 = new dojo.DeferredList(listDeleteDef);
			allDeferreds.push(d3);
			d3.addCallback(this, function(result) {
				for (var cur = 0; cur < this._filtersToBeDeleted.length;cur++) {
					var filter = this._filtersToBeDeleted[cur];
					this.removeDirty(filter);
				}
				this._filtersToBeDeleted = [];
			});
			if (task != null) {
				task.handleDeferredWork(d3, 1);
				task = null;
			}
						
			if (this.settingsChanged) {
				var settingstask = taskMonitor.beginTask(messages.saveExplorerFilterSettings, 1);
				
				// Wait for new filters to be saved to ensure that the uri of the new resources is known
				d1.addCallback(this, function() {
					// Save settings
					var enabledFilters = dojo.map(this.explorerFilterSection.dataSet, function(dataSet) {
						var checkBox = dataSet.row.children[0].children[0].children[0];
						var cbWidget = dijit.getEnclosingWidget(checkBox);
						var checked = cbWidget.attr('checked');
						if (checked) {
							var uri = dataSet.data.uri;
							return uri;
						}
						return null;
					});
					
					var enabledFiltersOnly = dojo.filter(enabledFilters, function(filter) {
						return filter != null;
					});
					
					var projectUri;
					// Get the project uri
					if (this.editorInput.projectId) {
						projectUri = PROJECT_AREA_PATH + '/' + this.editorInput.projectId;
					} else {
						projectUri = dojo.getObject("jazz.app.currentApplication").context;
					}
	
					// Create explorer setting object
					var setting = new rmps.ui.explorer.ExplorerSetting({
						settingName: "appliedFilters",
						projectUri: projectUri,
						value: enabledFiltersOnly
					});
					
					// Save the setting
					var def = setting.set();
					settingstask.handleDeferredWork(def, 1);
					
					this.settingsChanged = false;
					this.removeDirty(this.explorerFilterSection);
					return def;
				});
			}
			return allDeferreds;
		},
	
		handleServerResponseSuccess: function(){
			/*Do Nothing */
		},
		
		destroy: function(){
			this.explorerFilterSection.destroy();
			delete this.explorerFilterSection;
			this.explorerFilterSection = null;
	
			this.inherited(arguments);
		}
	});	
})();
