/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("rmps.app.ui.internal.editors.ExplorerFilterTab");

	dojo.require("rmps.core.TaskMonitor"); //$NON-NLS-1$
	dojo.require("rmps.app.ui.internal.ExplorerFilterEditorInput"); //$NON-NLS-1$
	dojo.require("rmps.app.ui.internal.editors.ExplorerFilterEditor"); //$NON-NLS-1$
	dojo.require("rmps.app.ui.internal.util.DmProjectAreaEditorUtils"); //$NON-NLS-1$
	
	dojo.require("dojo.i18n");
	dojo.requireLocalization("rmps.app.ui.internal.editors", "ExplorerFilterTabMessages");
	var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "ExplorerFilterTabMessages");
	
	var DmProjectAreaEditorUtils = rmps.app.ui.internal.util.DmProjectAreaEditorUtils;
	
	dojo.declare("rmps.app.ui.internal.editors.ExplorerFilterTab", rmps.app.ui.internal.editors.ExplorerFilterEditor, {
		
		tabTitle: null,
	
	    constructor: function(args) {
			this.editorInput = new rmps.app.ui.internal.ExplorerFilterEditorInput(args.editor.projectAreaItemId);
			this.suppressEditorUi = true;
			this.showLoading = function() {
				args.tabContainer.displayPendingMessage(messages.loadingExplorerFilters);
			};
			this.hideMessage = args.tabContainer.clearMessage;
			this.showMessage = function(msg) {
				DmProjectAreaEditorUtils.convertMessage(args.tabContainer, msg);
			};
			this.createHeader = function() {};
			this.tabTitle = args.tabTitle;
	    },
	    
		postCreate: function() {
			this.headerContainerNode.parentNode.removeChild(this.headerContainerNode);
			this.inherited(arguments);			
			this.startup();
		},
	
		dirtyStateChanged: function(isDirty) {
			if (isDirty) {
				this.editor.setDirty(true);
			}
		},
	
		changesPending: function() {
			return this.isDirty();
		},
		
		canSave: function() {
			return this.isDirty();
		},
		
		refresh: function() {
			this.lightRefresh();
		},
		
		doSave: function(callback) {
			var callBackFunction = callback;
			var taskMonitor = new rmps.core.TaskMonitor(messages.savingExplorerFilters);
			
			// Validate
			var errors = [];
			this.validate(errors);
			if (errors && errors.length > 0) {
				callBackFunction(DmProjectAreaEditorUtils.handleErrors(this.tabTitle, errors));
				return;
			}
			
			taskMonitor.setCanComplete(false);
			arguments[0] = taskMonitor;
			this.inherited(arguments);
			var handle = this.connect(taskMonitor, "onComplete", function(errors) {
				this.disconnect(handle);
				if (errors) {
					callBackFunction.apply(this, errors);
				} else {
					callBackFunction.apply(this);
				}
			});
			taskMonitor.setCanComplete(true);		
		},
		
	});
	
})();

