/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.app.ui.internal.editors.ExplorerViewpointEditor");

dojo.require("dijit.InlineEditBox");

dojo.require("dojo.DeferredList");

dojo.require("jazz.ui.Breadcrumbs");

dojo.require("rmps.app.ui.internal.editors.TabbedEditor");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("rmps.app.ui.internal.editors.CopyQueryDialog");

dojo.require("rmps.core.error");
dojo.require("rmps.core.explorer.Viewpoint");
dojo.require("rmps.core.explorer.ViewpointClient");
dojo.require("rmps.core.resource.ResourceInfoClient");

dojo.require("rmps.ui.sections.SectionTable");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.ui.internal.editors", "ExplorerViewpointEditorMessages");
dojo.requireLocalization("rmps.app.ui.internal.editors", "CopyQueryDialogMessages");


var UA_KEY = "com.ibm.xtools.web.app.page_ExplorerViewpoint001";

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "ExplorerViewpointEditorMessages");
var copyDialogMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "CopyQueryDialogMessages");

var ViewpointClient = rmps.core.explorer.ViewpointClient;
var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;
var ViewpointUtilities = rmps.core.explorer.ViewpointUtilities;
var DesignUtil = rmps.app.ui.internal.pages.design.DesignUtil;

var ROOT_QUERY_TAB_ID = "rootQuery";
var CHILD_QUERIES_TAB_ID = "childQueries";

(function(){
dojo.declare("rmps.app.ui.internal.editors.ExplorerViewpointEditor", rmps.app.ui.internal.editors.TabbedEditor, {
	
	"class": "rmps-explorer-viewpoint-editor",
	
	uaKey: UA_KEY,
	
	_rootQueryTab: null,
	_childQueriesTab: null,
	
	_editableDescription: null,
	
	_rootQueryUri: null,
	_rootGroups: null, //[]
	_childQueries: null, //[]
	
	constructor: function() {
		this._rootGroups = [];
		this._childQueries = [];
	},
	
	postCreate: function() {
		this.inherited(arguments);

		//create navigation breadcrumbs back to the baseline status page
		this._breadCrumbs= new jazz.ui.Breadcrumbs({
			segments: [{
				label: messages.statusLinkLabel,
				href: DesignUtil.getViewpointsUri()
			}]
		});

		dojo.place(this._breadCrumbs.domNode, this.headerContainerNode, "first");
						
		// Description UI
		var viewpoint = this.editorInput.getExplorerViewpoint();
		var desc = viewpoint ? viewpoint.getDescription() : null;
		desc = desc || "";
		var div = dojo.create("div", {className:"description-div"});
		var table = dojo.create("table", null, div);
		var tbody = dojo.create("tbody", null, table);
		var tr = dojo.create("tr", null, tbody);
		var td1 = dojo.create("td", null, tr);
		var td2 = dojo.create("td",{style:'width: 95%'}, tr);
		var span = dojo.create("span", {
			innerHTML: messages.description + ":&nbsp",
			className: "description-label"}, td1);
		var textboxdiv = dojo.create("div", null, td2);
        this._editableDescription = new dijit.InlineEditBox({
			title: messages.description,
			noValueIndicator: "&lt;" + messages.description_no_value + "&gt;",
			autoSave: true, 
			editorParams: {maxLength: "200"},
			value: desc,
			onChange: dojo.hitch(this, this.onDescriptionChange)
        }, textboxdiv);
        dojo.addClass(this._editableDescription.domNode, "description-text");
		if (desc.length === 0) {
			dojo.addClass(this._editableDescription.domNode, "description-hint");
		}
        this._headerContent.appendChild(div);
        this._editableDescription.domNode.style.paddingTop = "2px";
		this._editableDescription.domNode.style.paddingBottom = "2px";
        
		//dojo.publish("rmps.web.app.explorer.registerDropTarget", [{widgetId: this.id}]);
	},
	
	destroy: function() {
		//dojo.publish("rmps.web.app.explorer.unregisterDropTarget", [{widgetId: this.id}]);
				
		if (this._editableDescription) {
			this._editableDescription.destroy();
			delete this._editableDescription;
		}
		
		this.inherited(arguments);
	},
	
	// Overrides
	isEditableTitle: function(){
		return true;
	},

	getEditableTitleNodeDetails: function(){
		return messages.newExplorerViewpointTitle;
	},
	
	onTitleChange: function() {
		this.addDirty("title");
		var editorInput= this.getEditorInput();
		editorInput.getExplorerViewpoint().setName(this.getTitleValue());
		rmps.ui.editor.PageHelper.getInstance(DesignUtil.PAGE_ID).getEditorTracker().onUpdate(editorInput);
	},
	
	onDescriptionChange: function(text) {
		this.addDirty("description");
		
		if (text && text.length > 0) {
			dojo.removeClass(this._editableDescription.domNode, "description-hint");				
		} else {
			dojo.addClass(this._editableDescription.domNode, "description-hint");
		}
	},
	
	addTabs: function() {
		var viewpoint = this.editorInput.getExplorerViewpoint();
		this._rootQueryUri = viewpoint.getRootQueryUri();
		this._rootGroups = viewpoint.getRootGroups();
		this._childQueries = viewpoint.getChildQueries();
		this._rootQueryTab = this.addTab(ROOT_QUERY_TAB_ID, messages.tabRootQuery, "rmps.app.ui.internal.editors.views.explorerviewpoint.RootQueryTab", {rootQueryUri: this._rootQueryUri, groups: this._rootGroups, editor:this});
		this._childQueriesTab = this.addTab(CHILD_QUERIES_TAB_ID, messages.tabChildQueries, "rmps.app.ui.internal.editors.views.explorerviewpoint.ChildQueriesTab", {childQueries: this._childQueries, editor:this});			
	},
	
	doSave: function(taskMonitor) {
		this.inherited(arguments);

		var maintask = taskMonitor.beginTask(messages.saveExplorerViewpoint, 1);
		
		// Check if tab page is loaded first
		var rootTab = this._rootQueryTab;
		if (rootTab && rootTab.page && !dojo.isString(rootTab.page)) {
			// Save root query and root group queries
			var rootDef = rootTab.page.doSave(taskMonitor, false);
			rootDef.addCallback(this, function(results) {
				var query = null;
				var groups = [];
				dojo.forEach(results, function(result) {
					if (result[0]) { // Success?
						var node = result[1];
						if (ViewpointUtilities.isGroup(node)) {
							groups.push(node);	
						} else {
							// Not a group, so this is the query
							query = node;
						}
					}
				});
				this._rootQueryUri = query ? query.uri : null;
				this._rootGroups = groups;
				this._save(taskMonitor, maintask, true);
			});
		} else {
			this._save(taskMonitor, maintask, false);
		}
	},
	
	_save: function(taskMonitor, maintask, rootSaved) {
		// Save child queries tab content and the viewpoint itself
		var def;
		var childTab = this._childQueriesTab;
		if (childTab && childTab.page && !dojo.isString(childTab.page)) {
			// Save child queries
			def = childTab.page.doSave(taskMonitor, false);
		} else {
			// No-op
			def = new dojo.Deferred();
			def.callback(null);
		}
		
		def.addCallback(this, function(results) {
			this._childQueries = [];
			if (results) {
				dojo.forEach(results, function(result) {
					if (result[0]) {
						this._childQueries.push(result[1]);
					}
				}, this);
			}

			var desc = this._editableDescription ? this._editableDescription.get('value') : "";
			var viewpoint = this.editorInput.getExplorerViewpoint();
			viewpoint.setName(this.getTitleValue());
			viewpoint.setDescription(desc);
			if (rootSaved) {
				viewpoint.setRootQueryUri(this._rootQueryUri);
				viewpoint.setRootGroups(this._rootGroups);
			}
			if (results) {
				viewpoint.setChildQueries(this._childQueries);
			}
			
			var projectId = jazz.app.context.get().itemId;
			var def;
			if (viewpoint.uri && viewpoint.uri != "") {
				def = ViewpointClient.updateViewpoint(viewpoint, projectId);
			} else {
				def = ViewpointClient.createViewpoint(viewpoint, projectId);
			}
			
			maintask.handleDeferredWork(def, 1);
		});		
	},

	onSaveComplete: function() {
		dojo.publish("rmps.web.app.explorer.refresh");

		var viewpoint = this.editorInput.getExplorerViewpoint();
		if (this.editorInput.getResourceUri() == null && viewpoint.uri != "") {
			var pageHelper = rmps.ui.editor.PageHelper.getInstance(DesignUtil.PAGE_ID);
			if (pageHelper) {
				pageHelper.closeEditor(this, true, true);
			}
			
			window.location.href = DesignUtil.getViewpointUri(viewpoint.uri);
		} else {
			this.inherited(arguments);			
		}
	},
	
	onExplorerDrag: function() {
		return true;
	},
	
	onExplorerDrop: function(items, copy) {
		this.selectTab(CHILD_QUERIES_TAB_ID);
		
		var viewpointRE = /_viewpoint_\d+/g;
		for (var i = 0; i < items.length; i++) {
			var item = items[i];
			items[i] = item.replace(viewpointRE, "");
		}
		
		var def = this.getItemInformation(items);
		var self = this;
		def.addCallback(function(data) {
			dojo.forEach(data, function(resource) {
				// TODO: Select among type and super-types
				self._childQueriesTab.page.addTypeSection(resource.type, resource.typeName);
			});
		});
	},
	
	getItemInformation: function(items) {
		return ResourceInfoClient.getResourceInfo(items, ['type', 'typeName']);
	},
	
	validate: function(/*Array*/errors) {
		// Check if viewpoint name is specified 
		if (this.getTitleValue().length == 0) {
			errors.push(rmps.core.error.create(messages.emptyViewpointTitleError, messages.emptyViewpointTitleError, null));
			return;
		}
		
		var defs = [];
		
		// Check if tab page is loaded first
		var rootTab = this._rootQueryTab;
		if (rootTab && rootTab.page && !dojo.isString(rootTab.page)) {
			// validate root query
			defs.push(rootTab.page.validate());
		}
		
		// Do the same for child queries tab
		var childTab = this._childQueriesTab;
		if (childTab && childTab.page && !dojo.isString(childTab.page)) {
			// validate child queries
			defs = defs.concat(childTab.page.validate());
		}

		if (defs.length === 0) {
			return;
		}
		
		var def = new dojo.Deferred();
		
		var defList = new dojo.DeferredList(defs);
		defList.addBoth(function(results) {
			var errors = [];
			var success = true;
			dojo.forEach(results, function(result) {
				if (result[0] /*success*/) {
					var valErrors = result[1]; 
					if (valErrors && valErrors.length && valErrors.length > 0) {
						for (var i = 0; i < valErrors.length; i++) {
							var errorMessage = messages.validationError + ". " + valErrors[i];
							errors.push(rmps.core.error.create(errorMessage, errorMessage, null));
						}
					}
				} else {
					success = false;
				}	
			});
			if (success) {
				def.callback(errors);				
			} else {
				def.errback(errors);
			}
		});
		
		// BaseEditor expects a list of defs.
		return [def];
	},
	
	addButtons: function(toolbar) {
		this.inherited(arguments);
		this._copyButton = toolbar.addButton(this, this._copy, {
			label: messages.createACopy,
			imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/copy.gif"),
			index: 1
		});
		if (this.getEditorInput().getResourceUri()) {
			toolbar.enableButton(this._copyButton);
		} else {
			toolbar.disableButton(this._copyButton);
		}
			
	},
	
	_copy: function() {
		var viewpoint = this.editorInput.getExplorerViewpoint();
		this._dialog = new rmps.app.ui.internal.editors.CopyQueryDialog({
			queryLabel: viewpoint.label,
			onOK: dojo.hitch(this, function() {
				rmps.core.explorer.ViewpointClient.createViewpointCopy(
						viewpoint.uri,
						this._dialog.getProjectId(),
						this._dialog.getNewTitle()
				).then(dojo.hitch(this, this._onViewpointCopied));
				
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: copyDialogMessages.copyingViewpoint
				});
			}),
			editor: this,
			dialogTitle: copyDialogMessages.copyViewpoint,
			onlyDSP: true
		});
		this._dialog.show();
		
	},
	
	_onViewpointCopied: function(newViewpointUri) {
		window.location.href = this._dialog.getEditViewpointUri(newViewpointUri); 
	},

	
	dirtyStateChanged: function(isDirty) {
		this.inherited(arguments);
		if (isDirty) {
			this.toolbar.disableButton(this._copyButton);
		} else {
			this.toolbar.enableButton(this._copyButton);
		}
	}
});	

}());