/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.ExplorerViewpointsListEditor");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("rmps.app.ui.internal.editors.CopyQueryDialog");

dojo.require("rmps.core.error");
dojo.require("rmps.core.explorer.ViewpointClient");

dojo.require("rmps.ui.util.ToolbarFocusUtil");



dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.toolbar.Button");

dojo.requireLocalization("rmps.app.ui.internal.editors", "ExplorerViewpointsListEditorMessages");
dojo.requireLocalization("rmps.app.ui.internal.editors", "CopyQueryDialogMessages");

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "ExplorerViewpointsListEditorMessages");
var copyDialogMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "CopyQueryDialogMessages");

var DesignUtil = rmps.app.ui.internal.pages.design.DesignUtil;
var ToolbarFocusUtil = rmps.ui.util.ToolbarFocusUtil;

var UA_KEY = "com.ibm.xtools.web.app.page_ExplorerViewpointsList001";

dojo.declare("rmps.app.ui.internal.editors.ExplorerViewpointsListEditor", rmps.app.ui.internal.editors.BaseEditor, {

	"class": "rmps-explorer-viewpoints-list",
	
	uaKey: UA_KEY,

	// TODO: is this used?
	// init properties
	isReadOnly: true,
	
	canEdit: true,
	canEditSet: null,
	
	//private properties
	_deferred: null,
	
	postCreate: function() {
		this.inherited(arguments);
		this.refresh();
	},
	
	addButtons: function(toolbar) {
		if (!this.canEditSet) {
			// add buttons only after we figure out which ones are needed
			return;
		}
		if (this.canEdit) {
			var button = new jazz.ui.toolbar.Button({
				href: DesignUtil.getViewpointUri(null),
				label: messages.createExplorerViewpointText,
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/show_viewpoint_co.gif").toString()
			});
			
			toolbar.addWidget(button);
			toolbar.addSeparator(2);
		}
		// always want refresh button		
		this.inherited(arguments);
	},

	refresh: function() {
		if (this._deferred) {
			return;
		}

		dojo.forEach(dijit.findWidgets(this.containerNode), function(widget) {
			if (widget.destroyRecursive) {
				widget.destroyRecursive();
			}
		});
		
		this.showLoading();

		this._deferred = rmps.core.explorer.ViewpointClient.getViewpoints(false);
				
		this._deferred.addBoth(this, function() {
			delete this._deferred;
			this.hideMessage();
		});
		
		this._deferred.addCallback(this, function(viewpoints) {
		
			var table = new jazz.ui.StyledTable({
				trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
				columns: [
					{
						label: messages.name
					},
					{
						label: messages.description,
						className: "description"
					},
					{
						label: messages.actions,
						className: "actions"
					}
				],
				showHover: true
			});

			table.containerNode = table.tbody;
			this.containerNode.appendChild(table.domNode);
			
			// add creation button only if creation is allowed and creation button hasn't been added already;
			// this.canEdit flag is also used for show/hide delete and update actions
			if (!this.canEditSet) {
				this.canEditSet = true;
				this.canEdit = viewpoints.canEdit;
				this.addButtons(this.toolbar);
			}			

			for (var i = 0; i<viewpoints.length; i++) {
				if (!rmps.core.explorer.ViewpointClient.isProvidersViewpoint(viewpoints[i])) {
					var row = new rmps.app.ui.internal.editors._ViewpointRow({viewpoint: viewpoints[i], editor: this, canEdit : this.canEdit});
					table.tbody.appendChild(row.domNode);
					row.startup();
				}
			}		
		});
		
		this._deferred.addErrback(this, function(error) {
			this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.error_loading,
					closable: true
				});
		});
	},

	activate: function() {
		this.refresh();

		this.inherited(arguments);
	}
});

dojo.declare("rmps.app.ui.internal.editors._ViewpointRow", [dijit._Widget], {

	viewpoint: null,
	editor: null,
	
	_deferred: null,
	_deleteDeferred: null,
	_stateImg: null,
	_stateCell: null,
	_popup: null,
	_popupContent: null,
	_actionBar: null,
	
	_editAction: null,
	_deleteAction: null,
	_showAction: null,
	
	buildRendering: function() {
		this.domNode = this.containerNode = dojo.create("tr");
	},
	
	postCreate: function() {
		//viewpoint name
		var cell = dojo.create("td", null, this.domNode);
		var nameLink = dojo.create("a", { href : DesignUtil.getViewpointUri(this.viewpoint.uri), lavel : this.viewpoint.getName() }, cell);
		nameLink.appendChild(dojo.doc.createTextNode(this.viewpoint.getName()));
		
		this._popup = new jazz.ui.HoverPopup({
			around: nameLink,
			onOpen: dojo.hitch(this, this._createPopupContents),
			maxWidth: 800,
			maxHeight: dojo.isIE ? 570 : 550
		});
		
		//description
		cell = dojo.create("td", null, this.domNode);
		cell.appendChild(dojo.doc.createTextNode(
			this.viewpoint.getDescription()
		));
		
		//actions, if applicable
		var actionsTd = dojo.create("td", null, this.domNode);
		this._actionBar = new jazz.ui.SimpleToolbar();
		actionsTd.appendChild(this._actionBar.domNode);
		this._actionBar.startup();
		this._updateActions();	
		
		ToolbarFocusUtil.addToolbarFocusListeners( { 
            actionToolbar: this._actionBar, 
            containerNode: this.containerNode, 
            actionClassName: ".jazz-ui-SimpleToolbar-button"
        });	
	},
	
	_createPopupContents: function() {
		if (this._popupContent) {
			this._popupContent.destroyRecursive();
		}
		this._popupContent = new rmps.app.ui.internal.editors._ViewpointPopupContent({viewpoint:this.viewpoint});
		this._popup.content = this._popupContent.domNode;
	},
	
	destroy: function() {
		this.inherited(arguments);
		
		if (this._deferred) {
			this._deferred.cancel();
			delete this._deferred;
		}
		
		if (this._deleteDeferred) {
			this._deleteDeferred.cancel();
			delete this._deleteDeferred;
		}
		
		if (this._popupContent) {
			this._popupContent.destroyRecursive();
			delete this._popupContent;
		}
		
		if (this._popup) {
			this._popup.destroyRecursive();
			delete this._popup;
		}
	},
	
	_updateActions: function() {
		if (this.canEdit && !this._editAction) {
			this._editAction = this._actionBar.addButton(this, this._editViewpoint, {
				label: messages.editViewpoint,
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/edit.gif")
			});
		}
		
		if (!this._copyAction) {
			this._copyAction = this._actionBar.addButton(this, this._copyViewpoint, {
				label: copyDialogMessages.copyViewpoint,
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/copy.gif")
			});
		}
		
		if (this.canEdit && !this._deleteAction) {		
			this._deleteAction = this._actionBar.addButton(this, this._deleteViewpoint, {
				label: messages.deleteViewpoint,
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/remove.gif")
			});
		}
		
		if (!this._showAction) {
			this._showAction = this._actionBar.addButton(this, this._showViewpoint, {
				label: messages.showViewPoint,
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/show_viewpoint_co.gif")	
			});
		}
	},
	
	_editViewpoint: function() {
		window.location.href = DesignUtil.getViewpointUri(this.viewpoint.uri);
	},
	
	_copyViewpoint: function() {
		this._dialog = new rmps.app.ui.internal.editors.CopyQueryDialog({
			queryLabel: this.viewpoint.label,
			onOK: dojo.hitch(this, function() {
				rmps.core.explorer.ViewpointClient.createViewpointCopy(
						this.viewpoint.uri,
						this._dialog.getProjectId(),
						this._dialog.getNewTitle()
				).then(dojo.hitch(this, this._onViewpointCopied));
				
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: copyDialogMessages.copyingViewpoint
				});
			}),
			editor: this.editor,
			dialogTitle: copyDialogMessages.copyViewpoint,
			onlyDSP: true
		});
		this._dialog.show();
	},
	
	_onViewpointCopied: function(newViewpointUri) {
		window.location.href = this._dialog.getEditViewpointUri(newViewpointUri); 
	},
	
	 _showViewpoint: function() {
		 window.location.href = rmps.app.core.util.getShowViewpointUri(this.viewpoint.uri);
	 },
	
	_deleteViewpoint: function() {
		var msg = dojo.string.substitute(messages.deleteConfirmation, [this.viewpoint.getName()]);
		if (!confirm(msg)) {
			return;
		}
			
		this._actionBar.disableButton(this._editAction);
		this._actionBar.disableButton(this._deleteAction);
		this._actionBar.disableButton(this._showAction);
		
		// TODO: Queries will be deleted as part of the delete viewpoint operation
		// Once other queries can be referenced, user need to confirm.
		
		this._deleteDeferred = rmps.core.explorer.ViewpointClient.deleteViewpoint(this.viewpoint);
		this._deleteDeferred.addCallback(this, function(args) {
			delete this._deleteDeferred;
			this.editor.hideMessage();
			this.destroyRecursive();
		});

		this._deleteDeferred.addErrback(this, function(error) {
			this.editor.hideMessage();
			delete this._deleteDeferred;
			var message;
			if (error.status === 403) {
				message = messages.error_request_permission;
			} else {
				message = messages.error_request_general;
			}
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: message,
				closable: true
			});
		});		
		
		this.editor.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: messages.deletingViewpoint
		});
	}

});

dojo.declare("rmps.app.ui.internal.editors._ViewpointPopupContent", [dijit._Widget, dijit._Templated], {

	"class" : "rmps-viewpoint-status-popup",
	messages: messages,
	viewpoint: null,
	templateString: '<table>' +
						'<tr><th>${messages.details}</th></tr>' +
						'<tr><td>${messages.name}:</td><td dojoAttachPoint="_nameText"/></tr>' +
						'<tr><td>${messages.description}:</td><td dojoAttachPoint="_descriptionText"/></tr>' +						
					'</table>',
	
	postCreate: function() {
		this._nameText.appendChild(dojo.doc.createTextNode(this.viewpoint.getName()));
		this._descriptionText.appendChild(dojo.doc.createTextNode(this.viewpoint.getDescription()));
	}
});
})();