/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.ModelResourceEditor");

dojo.require("rmps.app.ui.internal.editors.ResourceEditor");

dojo.require("rmps.core.error");
dojo.require("rmps.core.explorer.ExplorerConstants");
dojo.require("rmps.ui.explorer.ExplorerUtilities");

dojo.require("dojo.date.locale");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.requireLocalization("rmps.app.ui.internal.editors", "ResourceEditorMessages");

var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "ResourceEditorMessages");

dojo.declare("rmps.app.ui.internal.editors.ModelResourceEditor", rmps.app.ui.internal.editors.ResourceEditor, {
	
	// properties
	resourceInfo: null,
	
	postCreate: function() {
		dojo.addClass(this.domNode, "rmps-app-editors-model-resource");
		this.inherited(arguments);
	},
	
	createHeader: function(parentNode) {
		this.inherited(arguments);
		
		// Optional resource number (user visible ID)
		var number = this.editorInput.getResourceNumber();
		if (number != null) {
			var resourceNumberSpan = dojo.create("span", {className: "resourceNumber"});
			resourceNumberSpan.appendChild(dojo.doc.createTextNode(number + ": "));
			this.titleNode.insertBefore(resourceNumberSpan, this.titleNode.firstChild);
		}
		
		dojo.create("br", null, parentNode);
		
		var modifiedSpan = dojo.create("span", {className: "lastModified"});
		var date = dojo.date.locale.format(this.resourceInfo.modificationTime, {formatLength: "medium"});
		modifiedSpan.appendChild(dojo.doc.createTextNode(
			dojo.string.substitute(messages.modified, [date])
		));
		
		parentNode.appendChild(modifiedSpan);
	},
	
	
	addButtons: function(toolbar) {
		// show in explorer
		var explorerbutton = toolbar.addButton(this, this._showInExplorer, {
			label : messages.showInExplorer,
			imgSrc : dojo.moduleUrl("rmps.app", "graphics/icons/explore_tree_co.gif")
		});
		// Check enabled/disabled state of show in explorer button 
		toolbar.disableButton(explorerbutton);
		this._checkShowExplorerButtonEnabled(toolbar, explorerbutton);
		this.inherited(arguments);
	},
	
	_showInExplorer: function() {
		ExplorerUtil.showItemInSidebarExplorer(this.editorInput.resourceUri, rmps.core.explorer.ExplorerConstants.IDs.EXPLORER_SIDEBAR_DESIGN_PAGE);
	},
	
	_checkShowExplorerButtonEnabled: function(toolbar, button) {
		if (this.resourceInfo.uri == this.editorInput.resourceUri) {
			toolbar.enableButton(button);
		} else {
			// Check if the resource can be displayed in explorer
			ExplorerUtil.canShowInTree(this.editorInput.resourceUri).addCallback(this, function(value) {
				if (value.enabled) {
					toolbar.enableButton(button);
				}
			});
		}
	}
});

})();
