/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("rmps.app.ui.internal.editors.ProjectLinkingEditor");

	dojo.require("jazz.app.context");
	dojo.require("rmps.app.ui.internal.sections.LinkTypesSection");
	dojo.require("rmps.app.ui.internal.editors.BaseEditor");
	dojo.require("jazz.app.context");
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants");
	dojo.require("rmps.app.ui.ProjectLinkingEditorUtil");
	dojo.require("rmps.app.ui.internal.editors.views.link.LinkClient");
	dojo.require("rmps.core.error");
	dojo.require("dojox.validate.regexp");
	dojo.require("rmps.app.ui.internal.sections.LinkTypesDisplayOptions");
    dojo.require("rmps.core.util.ContextUtils");	
    dojo.require("rmps.ui.CollapsibleSection");
    dojo.require("rmps.app.ui.internal.editors.SelectConfigurationFragment");
	dojo.require("rmps.core.util.IRmpsConstants");
    

	var ProcessConstants= com.ibm.team.process.web.common.internal.ProcessConstants;
	var RMPSProcessConstants = rmps.app.ui.ProjectLinkingEditorUtil.process;
	var LinkTypesDisplayOptions=rmps.app.ui.internal.sections.LinkTypesDisplayOptions;
	var LinkEventManager= rmps.app.ui.internal.editors.views.link.LinkEventManager;
	var LinkClient= rmps.app.ui.internal.editors.views.link.LinkClient;
    var ContextUtils= rmps.core.util.ContextUtils;

	dojo.require("dojo.i18n");
	dojo.requireLocalization("rmps.app.ui.internal.editors", "ProjectLinkingEditorMessages");
	var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "ProjectLinkingEditorMessages");

	dojo.declare("rmps.app.ui.internal.editors.ProjectLinkingEditor", rmps.app.ui.internal.editors.BaseEditor, {

		_dirtyGroupId: null,				/*Dirty Group ID */
		_linkTypesToBeDeleted: null,			/*Link Types To Be Added */
		_linksTypesToBeAdded: null,			/*Link Types To Be Added */
		_contextUri: null,
		connectHandles: null,					/*Used by other sections */
		eventHandles: null,					/*Used by other sections */
		projectAreaLink: null,				/*Its value is retrieved by other sections */
		deploymentInProgress: false,		/*A flag to determine if the deployment action is in progress to avoid multipe executions */

		URL_REGEX: dojox.validate.regexp.url({
			scheme: true,
			allowDecimal: false,
			allowHex: false,
			allowDottedHex: false,
			allowDottedOctal: false
		}),

	constructor: function(){
		this._linkTypesToBeDeleted= [];
		this._linksTypesToBeAdded= [];
		this._linksTypesToBeModified= [];
		this.connectHandles= [];
		this.eventHandles= [];
	},

	postCreate: function() {
		this.inherited(arguments);
		this.showLoading();
		this._init();
	},

	_init: function() {
		var self = this;
		var jazzContext = jazz.app.context.get();
		if (jazzContext && jazzContext.baseline) {
			this._contextUri = jazzContext.baseline;
			this._initPermissions();
			this._createSections();
		} else if (window.location.search.indexOf(rmps.core.util.IRmpsConstants.BASELINE) !== -1) {
			var startIndex = window.location.search.indexOf(rmps.core.util.IRmpsConstants.BASELINE);
			startIndex += rmps.core.util.IRmpsConstants.BASELINE.length + 1;
			var endIndex = window.location.search.indexOf(startIndex, "&");
			if (endIndex < 0) {
				endIndex = window.location.search.length;
			}
			var encodedUri = window.location.search.substr(startIndex, endIndex - startIndex);
			this._contextUri = unescape(encodedUri);
			this._initPermissions();
			this._createSections();
		} else {
			this.editorInput.projectConfigurationsInfo.getDefaultWorkspace().addCallback(this, function(defaultWorkspace) {
				this._contextUri = defaultWorkspace.uri;
				this._initPermissions();
				this._createSections();
			});
		}
	},
	
	_initPermissions: function() {
		var report = this.getEditorInput().operationReport;
		var isSnapshot = ContextUtils.isSnapshot(this._contextUri);
		this.allowCreation = report.isAllowed(RMPSProcessConstants.Actions.CREATE) && !isSnapshot;
		this.allowDeletion = report.isAllowed(RMPSProcessConstants.Actions.DELETE) && !isSnapshot;
		this.allowModification = report.isAllowed(RMPSProcessConstants.Actions.MODIFY) && !isSnapshot;
	},
	
	_createSections: function(){

		/*Destroy the container node widgets recursively */
		this._destroyRecursive(this.containerNode);
		
		var jazzContext = jazz.app.context.get();
		if (!jazzContext || !jazzContext.baseline) {
			var configurationSection = new rmps.ui.CollapsibleSection({
				primaryTitle: messages.configurationContext,
				collapsible: false,
			});
			
			
			this.configurationSelector = new rmps.app.ui.internal.editors.SelectConfigurationFragment({
				contextSelected: dojo.hitch(this, this._configurationContextSelected),
				projectConfigurationsInfo: this.getEditorInput().projectConfigurationsInfo,
				errorCallback: function() {	}
			});
			
			this.configurationSelector.setContext(this._contextUri);
			
			var buttonDiv = dojo.create('div', {style: "float:right"}, configurationSection.headerNode);
			var modifyLink = dojo.create('a', {href: "#", "class": "title"}, buttonDiv);
			modifyLink.appendChild(dojo.doc.createTextNode(messages.modify));
			dojo.connect(modifyLink, "onclick", this.configurationSelector, "modify");
			
			
			configurationSection.containerNode.appendChild(this.configurationSelector.domNode);
			
			this.containerNode.appendChild(configurationSection.domNode);
		}

		/*Create Link Types Section */
		this.linkTypesSection= new rmps.app.ui.internal.sections.LinkTypesSection({
			allowCreation: this.allowCreation,
			allowDeletion: this.allowDeletion,
			allowModification: this.allowModification,
			editor: this,
			projectContext: this.getEditorInput().projectConfigurationsInfo.projectId,
			configurationContext: this._contextUri,
			displayOption: LinkTypesDisplayOptions.USER_DEFINED
		});
		this.linkTypesSection.startup(this, LinkTypesDisplayOptions.USER_DEFINED);

		this.addSpace();

		/*Create System-defined Link Types Section */
		this.systemLinkTypesSection= new rmps.app.ui.internal.sections.LinkTypesSection({
			isReadOnly : true,
			editor: this,
			projectContext: this.getEditorInput().projectConfigurationsInfo.projectId,
			configurationContext: this._contextUri,
			displayOption: LinkTypesDisplayOptions.SYSTEM_DEFINED
		});
		this.systemLinkTypesSection.startup(this, LinkTypesDisplayOptions.SYSTEM_DEFINED);

		this.getEditorInput().mediator.linkTypeDataFeed.initEntriesForWidget(this.linkTypesSection);

		this.hideMessage();
	},
	
	_configurationContextSelected: function(contextUri) {
		if (this._contextUri !== contextUri) {
			this._contextUri = contextUri;
			this._initPermissions();
			this.linkTypesSection.setContext(this._contextUri);
			this.systemLinkTypesSection.setContext(this._contextUri);
		}
	},

	refresh: function() {
		this.lightRefresh();
		this.inherited(arguments);
	},
	
	lightRefresh: function() {
		this.linkTypesSection.notifyRefresh();
		this.systemLinkTypesSection.notifyRefresh();
	},
	
	addSpace: function(){
		/*Add a space between the two sections */
		var div = dojo.doc.createElement("div");
		div.innerHTML = "<br>";
		this.containerNode.appendChild(div);
	},

	_destroyRecursive: function(node){
		dojo.forEach(dijit.findWidgets(node), function(widget) {
			if (widget.destroyRecursive) {
				widget.destroyRecursive();
			}
		});
	},

	_contains: function(array, value){
		for(var index=0; index< array.length; index++){
			if(array[index]==value){
				return true;
			}
		}
		return false;
	},
	
	save: function() {
		if(this._linkTypesToBeDeleted.length>0&& !confirm(messages.confirmDelete)){
			return;
		}
		this.inherited(arguments);
	},

	doSave: function(taskMonitor) {
		/*Deleting a link type should give a warning message; 
		 * risky/major actions need confirmation 
		 */
		var deferred, i, dataSet, data, task;

		for (i = 0; i < this._linksTypesToBeAdded.length;i++) {
			var link= this._linksTypesToBeAdded[i];
			dataSet= this.linkTypesSection.getDataForRow(link);
			var cells= dataSet.cells;
			var fields= dataSet.fieldNames;
			var values= dataSet.values;
			dojo.removeClass(cells[3].children[1], "derivedInput");
			data= dataSet.data;
			deferred = LinkClient.createLinkType(data);
			task = taskMonitor.beginTask("Create Link", 1);
			deferred.addCallback(this, function(result) {
				data= result;
				dataSet.values= this.linkTypesSection.getAssociatedValues(data);
				for (var cur = 0; cur < this._linksTypesToBeAdded.length;cur++) {
					var link= this._linksTypesToBeAdded[cur];
					this.removeDirty(link);
				}
				this._linksTypesToBeAdded= [];
			});
			task.handleDeferredWork(deferred, 1);
		}

		for (i = 0; i < this._linksTypesToBeModified.length;i++) {
			var row= this._linksTypesToBeModified[i];
			dataSet= this.linkTypesSection.getDataForRow(row);
			data= dataSet.data;
            deferred = LinkClient.updateLinkType(dataSet.oldData,data);
            task = taskMonitor.beginTask(dojo.string.substitute("Update Link", [data.uri]), 1);
            deferred.addCallback(this, function(result) {
                for (var cur = 0; cur < this._linksTypesToBeModified.length;cur++) {
					var link= this._linksTypesToBeModified[cur];
					this.removeDirty(link);
				}
				this._linksTypesToBeModified= [];
            });
			task.handleDeferredWork(deferred, 1);
		}
		
		for(i=0; i<this._linkTypesToBeDeleted.length; i++){
			data= this._linkTypesToBeDeleted[i];
			if(data){
				task = taskMonitor.beginTask("Delete Link Type:"+data.about, 1);
				deferred = LinkClient.deleteLinkType(data);
				deferred.addCallback(this, function(result) {
					for (var cur = 0; cur < this._linkTypesToBeDeleted.length;cur++) {
						var data= this._linkTypesToBeDeleted[cur];
						this.removeDirty(data);
		                LinkEventManager.notifyAllCallback(data.about, "removeAction");
					}
					this._linkTypesToBeDeleted= [];
				});
				task.handleDeferredWork(deferred, 1);
			}
		}
	},

	destroy: function(){
		this.linkTypesSection.destroy();
		delete this.linkTypesSection;
		this.linkTypesSection= null;

		this.systemLinkTypesSection.destroy();
		delete this.systemLinkTypesSection;
		this.systemLinkTypesSection= null;

		this.inherited(arguments);
	}

	});

})();
