/*******************************************************************************
 * Licensed Materials - Property of IBM
 *
 * com.ibm.xtools.rmps.web.core
 *
 * Copyright IBM Corporation 2009, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.PropertiesEditor");

dojo.require("rmps.app.ui.internal.editors.ModelResourceEditor");

dojo.require("jazz.app.context");
dojo.require("jazz.core.loader");

dojo.require("dojo.i18n");

dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("dijit.Menu");
dojo.require("dijit.MenuItem");
dojo.require("dijit.PopupMenuItem");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.util.html");
dojo.require("rmps.core.RootServices");
dojo.require("rmps.core.commenting.range.IDList");
dojo.require("rmps.ui.explorer.ExplorerUtilities");

dojo.require("rmps.app.ui.internal.editors.views.ElementRelatedElements");

dojo.requireLocalization("rmps.app.ui.internal.editors", "PropertiesEditorMessages");

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "PropertiesEditorMessages");
var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;

var EXPLORER_CHAIN_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "explorer") + "/chain";

var BASELINE_PARAM = "rmps.context";

dojo.declare("rmps.app.ui.internal.editors.PropertiesEditor", rmps.app.ui.internal.editors.ModelResourceEditor, {
	_resourceNavigation: true,

	addTabs: function() {
		this.addTab("properties", messages.properties, "rmps.app.ui.internal.editor._ElementPropertyViewer", {editor: this, overflow: "hidden"});
		this.addTab("relatedElements", messages.relatedElements, "rmps.app.ui.internal.editor._ElementRelatedElements", {editor: this, resourceInfo: this.resourceInfo});
		this.inherited(arguments);
	}
});

// Mixin class to add breadcrumb navigation
dojo.declare("rmps.app.ui.internal.editor._Navigator", null, {
	// These parameters needs to be provided by subclasses or when the navigator object is constructed.
	editor: null,
	navigator: null, // Navigator container widget

	rootElementHandler: null,
	segments: null,
	elementUri: null,
	floatingWidget: null, // This field can be used by subclasses to shuffle a cached widget for a segment that has not yet been initialized

	refreshContentsPane: function(uri) {
		// Subclasses should override this in order to refresh the "content" pane when a breadcrumb element has been picked.
	},

	hideWidget: function(widget) {
		// Subclasses should override this if they implement getWidgetToCache so that the widget can be hidden from view
	},

	showWidget: function(widget) {
		// Subclasses should override this if they implement getWidgetToCAche so that the widget becomes visible again
	},

	postCreateNavigator: function() {
		this.segments = [];

		this.rootElementHandler = new rmps.app.ui.internal.editor._ElementHandler({
				uri: this.editor.getEditorInput().uriResourcePart,
				parentViewer: this,
				info: this.editor.getEditorInput().resourceInfo
			}
		);

		this.elementUri = this.editor.getEditorInput().resourceUri;

		this.populateNavigationAndViewer();

		this.connect(this.editor, "activate", function() {
			this.refresh(this.editor.getEditorInput().resourceUri);
		});
	},

	destroyNavigator: function() {
		if (this.rootElementHandler != null) {
			this.rootElementHandler.destroy();
		}

		for (var i=0, segment; segment = this.segments[i]; i++) {
			segment.destroy();
		}

		// If there is a lingering floating widget then destroy it
		if (this.floatingWidget) {
			this.floatingWidget.destroyRecursive();
		}

		this.segments = [];
	},

	refresh: function(newUri) {
		if (this.elementUri === newUri) {
			return;
		}

		this.elementUri = newUri;

		// Remove all element handlers from view that are no longer needed
		var currentHandler = this.segments.pop();
		while (currentHandler && currentHandler != null) {
			if (currentHandler.domNode) {
				this.navigator.removeChild(currentHandler.domNode);
			}
			currentHandler.destroy();
			currentHandler = this.segments.pop();
		}

		// Hide the root widget if it is showing
		this.hideWidget(this.rootElementHandler.cachedWidget);
		// Deactivate the root breadcrumb if it was active
		this.rootElementHandler.setActive(false);

		this.populateNavigationAndViewer();
	},

	populateNavigationAndViewer: function() {
		if (this.editor._resourceNavigation) {
			// Create a deferred request to fill in the current element path within the resource
			var content = {
				resourceURI: this.elementUri,
				scope: "elements"
			};

			// Baseline
			var baseline = jazz.app.context.get().baseline;
			if (baseline) {
				content[BASELINE_PARAM] = baseline;
			}

			var args = {
				url: EXPLORER_CHAIN_PATH,
				content: content,
				handleAs: "json",
				headers: {
					accept: "application/json",
					"X-ibm-rmps-internal": "true"
				}
			};
			var deferred = jazz.client.xhrGet(args);

			deferred.addCallback(this, function(result){
				this.populateBreadcrumb(result);
			});
		}

		this.refreshContentsPane(this.elementUri);
	},

	populateBreadcrumb: function(result) {
		var reachedResource = false;

		for (var i = result.length-1; i >= 0; i--) {
			if (result[i] == this.editor.getEditorInput().uriResourcePart) {
				// We've reached the resource, now we can populate the breadcrumb
				reachedResource = true;
				continue;
			} else if (reachedResource) {
				var elementHandler = new rmps.app.ui.internal.editor._ElementHandler({uri: result[i], parentViewer: this});
				this.segments.push(elementHandler);
			}
		}

		if (this.segments.length > 0) {
			this.segments[this.segments.length-1].setActive(true);

			if (this.floatingWidget) {
				this.segments[this.segments.length-1].cachedWidget = this.floatingWidget;
				this.floatingWidget = null;
			}

			this.showNavigation();
		} else {
			this.rootElementHandler.setActive(true);

			// Do don't know that there any children at this point so we spawn off an asynchronous check
			// Assume until this call returns that there are no elements within this resource
			var deferred = ExplorerUtil.getChildren(this.rootElementHandler.uri);
			deferred.addCallback(this, function(result) {
				if (result.children && result.children.length > 0) {
					this.rootElementHandler.setChildren(result.children, result.paged);
					this.showNavigation();
				} else {
					this.navigator.style.display = "none";
				}
			});
		}
	},

	showNavigation: function() {
		this.navigator.style.display = "";
		this.navigator.className = "rmps-app-editors-properties-element-navigator";

		this.navigator.appendChild(this.rootElementHandler.domNode);

		for (var i=0, segment; segment = this.segments[i]; i++) {
			this.navigator.appendChild(segment.domNode);
		}
	},

	selectNewChild: function(elementHandler, uri) {
		var currentHandler = this.segments.pop();
		var originalHandler = currentHandler;

		// Remove all element handlers from view that are no longer needed
		while (currentHandler && currentHandler !== elementHandler) {
			this.navigator.removeChild(currentHandler.domNode);
			currentHandler.destroyRecursive();
			currentHandler.setActive(false);
			currentHandler = this.segments.pop();
		}

		if (currentHandler) {
			// push the current one back on the stack
			this.segments.push(currentHandler);
		}

		if ((!uri || uri == null) && elementHandler.uri === this.elementUri) {
			// Nothing to do the user clicked on the current element again
			return;
		}

		// Hide any cached widgets for the handler that is no longer active
		if (originalHandler) {
			this.hideWidget(originalHandler.cachedWidget);
		} else {
			this.hideWidget(this.rootElementHandler.cachedWidget);
		}

		if (uri && uri != null) {
			var newHandler = new rmps.app.ui.internal.editor._ElementHandler({uri: uri, parentViewer: this});
			this.navigator.appendChild(newHandler.domNode);
			this.segments.push(newHandler);
			newHandler.setActive(true);
			if (currentHandler) {
				currentHandler.setActive(false);
			}

			this.refreshContentsPane(uri);
			this.editor.getEditorInput().setResourceUri(uri);
			this.elementUri = uri;
		} else {
			// Check this newly selected element handler to see if it has some cached properties to show
			if (elementHandler.cachedWidget) {
				this.showWidget(elementHandler.cachedWidget);
			} else {
				this.refreshContentsPane(elementHandler.uri);
			}

			this.editor.getEditorInput().setResourceUri(elementHandler.uri);
			this.elementUri = elementHandler.uri;
		}

		window.location.href = this.editor.getEditorInput().getPresentationUri();
	}
});

dojo.declare("rmps.app.ui.internal.editor._ElementRelatedElements", [rmps.app.ui.internal.editors.views.ElementRelatedElements, rmps.app.ui.internal.editor._Navigator], {
	navigator: null,

	refreshContentsPane: function(uri) {
		if (this.getQueryResult()) {
			var query = this.getQueryResult().query;
			query.uri = uri;
			this.executeQuery(query, true);
		}
	},

	postCreate: function() {
		this.inherited(arguments);

		this.navigator = dojo.doc.createElement("div");
		this.navigator.style.display="none";
		this.headerContainerNode.insertBefore(this.navigator,this.headerContainerNode.firstChild);
		this.headerContainerNode.style.padding = "0px 0px 0px";
		this.headerContainerNode.childNodes[2].style.padding = "7px 10px 0px";

		this.postCreateNavigator();
	},

	destroy: function() {
		this.destroyNavigator();
		this.inherited(arguments);
	},

	getElementToQueryOn: function() {
		return this.elementUri;
	}
});

dojo.declare("rmps.app.ui.internal.editor._ElementPropertyViewer", [dijit._Widget, dijit._Templated, rmps.app.ui.internal.editor._Navigator], {
	templateString: '<div>' +
				'<div dojoAttachPoint="navigator" style="display: none;"></div>' +
				'<div dojoAttachPoint="properties" style="position: relative; overflow: auto;"></div>' +
			'</div>',

	postCreate: function() {
		this.inherited(arguments);
		this.postCreateNavigator();
	},

	startup: function() {
		this.subscribeToEvents();
		this.inherited(arguments);
	},

	subscribeToEvents: function() {
		// We only add comment navigation handling if there is resource navigation enabled on this editor
		if (this.editor._resourceNavigation) {
			// TODO Eventually we will look at re-enabling some form of navigation to the element that the comment was made on
			//this.connect(this.editor.commentingWidget, "onSelectionChanged", this.commentSelectionChanged);
			this.connect(this.editor.commentingWidget, "onEndEditingComment", this.commentEditingFinished);
		}
	},

	commentEditingFinished: function(comment) {
		if (this.elementUri != this.editor.getEditorInput().uriResourcePart) {
			comment.range = [new rmps.core.commenting.range.IDList(this.elementUri)];
		}
	},

	destroy: function() {
		this.destroyNavigator();
		this.inherited(arguments);
	},

	resize: function(size) {
		if (!size) {
			return;
		}

		var contentSize = {};
		if (size.h) {
			this.domNode.style.height = size.h + "px";
			contentSize.h = size.h - (this.navigator.style.display === "none" ? 0 : this.navigator.offsetHeight);
			this.properties.style.height = contentSize.h + "px";
		}

		var widgetToResize;
		if (this.segments.length > 0) {
			widgetToResize = this.segments[this.segments.length-1].cachedWidget;
		} else {
			widgetToResize = this.rootElementHandler.cachedWidget;
		}

		if (widgetToResize && widgetToResize.resize) {
			widgetToResize.resize(contentSize);
		}
	},

	refreshContentsPane: function(uri) {
		// Create a new properties viewer and cache it in the current element handler
		jazz.core.loader.load_sync("rmps.ui.properties.PropertiesViewer");
		var propertiesViewer = new rmps.ui.properties.PropertiesViewer({
			resourceUri: uri
		});

		this.properties.appendChild(propertiesViewer.domNode);

		if (this.rootElementHandler.uri === uri) {
			this.rootElementHandler.cachedWidget = propertiesViewer;
		} else {
			if (this.segments.length > 0) {
				this.segments[this.segments.length-1].cachedWidget = propertiesViewer;
			} else {
				// Oops, segments aren't available yet. Declare the widget as floating until the
				//  breadcrumbs are allocated.
				this.floatingWidget = propertiesViewer;
			}
		}

		this.editor.showLoading();

		var loadingErrorHandle;
		var loadingHandle = this.connect(propertiesViewer, "onPropertiesLoaded", function() {
			this.editor.hideMessage();
			this.editor.disconnect(loadingHandle);
			this.editor.disconnect(loadingErrorHandle);
		});

		loadingErrorHandle = this.connect(propertiesViewer, "onLoadPropertiesFailed", function(error) {
			this.editor.showMessage({errorObject: error, closable: true});
			this.editor.disconnect(loadingHandle);
			this.editor.disconnect(loadingErrorHandle);
		});

		propertiesViewer.startup();
	},

	hideWidget: function(widget) {
		if (widget && widget.domNode) {
			widget.domNode.style.display = "none";
		}
	},

	showWidget: function(widget) {
		if (widget && widget.domNode) {
			widget.domNode.style.display = "";
		}
	}
});

dojo.declare("rmps.app.ui.internal.editor._ElementHandler", dijit._Widget, {
	childrenMenu: null,
	uri: null,
	parentViewer: null,
	label: null,
	active: false,

	buildRendering: function() {
		this.domNode = dojo.create("span");
		this.domNode.className = "rmps-app-editors-properties-element-breadcrumb";
		this.domNode.appendChild(dojo.doc.createTextNode(messages.loading));

		this._createChildrenPulldown(this.domNode, this.uri);

		if (this.info) {
			this._handleResourceInfo(this.info);
		} else {
			// Now spawn off an asynchronous task to get the name of this element
			var deferred = rmps.core.resource.ResourceInfoClient.getResourceInfo(this.uri, ['shortName', 'iconUri', 'longName']);
			deferred.addCallback(this, function(infos) {
				this._handleResourceInfo(infos[0]);
			});
		}
	},

	setChildren: function(children, isPaged) {
		if (this.childrenMenu == null) {
			this._createChildrenPulldown(this.domNode, this.uri);
		}

		this.childrenMenu.setChildren(children, isPaged);
	},

	_handleResourceInfo: function(info) {
		this.label = dojo.create("a");
		this.label.href = this.uri;

		if (this.active) {
			this.label.className = "rmps-app-editors-properties-element-active-element";
		}

		this.label.title = info.longName;

		// Special element icon
		var icon = dojo.create("img");

		// Make sure the image loaded ok, otherwise remove it to prevent error icon in IE
		var errorHandle = this.connect(icon, "onerror", function() {
			this.disconnect(errorHandle);
			this.label.removeChild(icon);
		});
		var loadHandle = this.connect(icon, "onload", function() {
			this.disconnect(errorHandle);
			this.disconnect(loadHandle);
		});

		var iconUri = info.iconUri;
		if (iconUri.indexOf("etag") === -1) {
			iconUri += "&preventCache="+((new Date()).getTime());
		}
		
		icon.src = iconUri;
		icon.className = "icon";
		this.label.appendChild(icon);

		this.label.appendChild(dojo.doc.createTextNode(info.shortName));

		this.domNode.replaceChild(this.label, this.domNode.firstChild);

		this.connect(this.label, "onclick", function(e) {
			this._newChildSelected();
			dojo.stopEvent(e);
		});
	},

	_createChildrenPulldown: function(element, uri) {
		this.pulldownImage = dojo.create("img", null, element);
		this.pulldownImage.src = dojo.moduleUrl("rmps.app", "graphics/icons/breadcrumb_expand.gif");
		this.pulldownImage.className = "rmps-app-editors-properties-element-breadcrumb-pulldown";

		this.childrenMenu = new rmps.app.ui.internal.editor._ChildrenMenu({
			uri: uri,
			viewer: this
		});
		this.childrenMenu.init();
		this.childrenMenu.bindDomNode(element);
	},

	_newChildSelected: function(uri) {
		if (!uri) {
			this.setActive(true);
		} else {
			this.setActive(false);
		}
		this.parentViewer.selectNewChild(this, uri);
	},

	setActive: function(active) {
		this.active = true;

		if (this.label != null && active) {
			this.label.className = "rmps-app-editors-properties-element-active-element";
		} else if (this.label != null && !active) {
			this.label.className = "";
		}
	},

	destroy: function() {
		if (this.childrenMenu != null) {
			this.childrenMenu.destroyRecursive();
		}

		// This element handler may be holding on to a cached widget with some hidden dom
		//  nodes in it. Destroy these cached widget to avoid leaks and to clean up the dom.
		if (this.cachedWidget) {
			this.cachedWidget.destroyRecursive();
		}

		this.inherited(arguments);
	}
});

dojo.declare("rmps.app.ui.internal.editor._ChildrenMenu", [dijit.Menu], {
	uri: null,
	leftClickToOpen: true,
	loadingItem: null,
	loaded: false,
	children: null,
	viewer: null,

	init: function(children, paged) {
		if (!children || children == null || children == "true") {
			this.loadingItem = new dijit.MenuItem({label: messages.loading});
			this.addChild(this.loadingItem);
		} else {
			// The children have been provided so we build up the menu directly from those
			this.setChildren(children, paged);
		}
	},

	setChildren: function(children, paged) {
		if (children == null || children.length == 0) {
			return;
		}

		this._removeLoadingItem();

		this.children = children;

		// TODO Decide whether truncation is the best option here if there are too many menu items
		for (var i = 0; i < this.children.length && i < 30; i++) {
			if (paged == "true") {
				var uri = ExplorerUtil.getItemURI(this.children[i]);

				var subMenu = new rmps.app.ui.internal.editor._ChildrenMenu({uri: uri, viewer: this.viewer});
				subMenu.init(this.children[i].children, this.children[i].paged);

				var item = new dijit.PopupMenuItem({label: jazz.util.html.escape(this.children[i].label), popup: subMenu});
				this.addChild(item);
			} else {
				this.createChildMenuItem(this.children[i]);
			}
		}

		this.loaded = true;
	},

	onOpen: function(e) {
		if (!this.loaded) {
			// Find the children of the element based on the URI to populate a list of links
			var deferred = ExplorerUtil.getChildren(this.uri);
			deferred.addCallback(this, function(result) {
				if (!result.children) {
					this._removeLoadingItem();
					this.addChild(new dijit.MenuItem({label: messages.empty, disabled: true}));
					this.loaded = true;
				} else {
					this.setChildren(result.children, result.paged);
				}
			});
		}

		this.inherited(arguments);
	},

	_removeLoadingItem: function() {

		if (this.loadingItem) {
			// Make sure the focus and selected state of the loading item is unset in the menu
			// before removing it, otherwise we can get exceptions on change of focus.
			this._closeChild();
			this.removeChild(this.loadingItem);
		}

	},

	createChildMenuItem: function(child) {
		var item = new dijit.MenuItem({label: jazz.util.html.escape(child.label)});

		var uri = ExplorerUtil.getItemURI(child);
		item.onClick = dojo.hitch({viewer: this.viewer, uri: uri}, function() {
			this.viewer._newChildSelected(this.uri);
		});

		this.addChild(item);
	},

	destroy: function() {
		if (this.loadingItem) {
			this.loadingItem.destroyRecursive();
		}

		this.inherited(arguments);
	}
});

})();
