/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("rmps.app.ui.internal.editors.RequestBaselineEditor");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");

dojo.require("rmps.ui.SimpleButton");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.app.ui.internal.BaselineStatusEditorInput");
dojo.require("rmps.app.core.util");
dojo.require("rmps.core.baseline.BaselineClient");
dojo.require("rmps.core.RootServices");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmps.ui.editor.PageHelper");

dojo.require("jazz.app.ua.TextHoverHelp");
dojo.require("jazz.ui.Breadcrumbs");
dojo.require("rmps.core.util.ContextUtils");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.app.ui.internal.editors", "RequestBaselineEditorMessages");

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "RequestBaselineEditorMessages");
var ContextUtils = rmps.core.util.ContextUtils;
var SNAPSHOT_NAME_UAKEY = "com.ibm.xtools.rmps.web.app.text_RequestBaselineEditor001";

var STATUS_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "status");
var STATUS_STATE_PROPERTY = "http://jazz.net/xmlns/rmps/status/1.0/importState";
var IRmpsConstants= rmps.core.util.IRmpsConstants;
var STREAM_URI_PARAM = "workspaceUri";
var STREAM_NAME_PARAM = "workspaceName";

var createRequiredNode = function() {
	var reqNode = dojo.create("span");
	reqNode.className = "required";
	reqNode.innerHTML = "*";
	reqNode.title = messages.requiredTitle;
	return reqNode;
};

var isImportInProgress = function(projectId) {
	var args = {
		url: STATUS_PATH + "/import/" + projectId,
		headers: {
			"Accept": "application/rdf+json",
			"X-ibm-rmps-internal": "true"
		},
		handleAs: "json",
		load: function(response, ioArgs) {
			for (var uri in response) {
				if (uri.indexOf("_") === 0) {
					var state = response[uri][STATUS_STATE_PROPERTY][0].value;
					if (state == "pending" || state == "inProgress") {
						return true;
					}

				}
			}
			return false;
		}
	};
	return jazz.client.xhrGet(args);
};

dojo.declare("rmps.app.ui.internal.editors.RequestBaselineEditor", rmps.app.ui.internal.editors.BaseEditor, {

	"class": "rmps-request-baseline",
	isRefreshable: false,
	isReadOnly: true,

	_formContent: null,
	_button: null,
	_resetTitle: false,

	postCreate: function() {
		this.inherited(arguments);

		//create navigation bread crumbs back to the baseline status page
		this._breadCrumbs = this._createBreadCrumbs();
		dojo.place(this._breadCrumbs.domNode, this.headerContainerNode, "first");
	},
	
	_createBreadCrumbs: function() {
		if (this.getWorkspaceUri() != null) {
			return new jazz.ui.Breadcrumbs({
				segments: [{
					label: messages.workspacesLinkLabel,
					href: IRmpsConstants.ADMIN_STREAM_VIEWER
				}]
			});
		} else {
			return new jazz.ui.Breadcrumbs({
				segments: [{
					label: messages.snapshotsLinkLabel,
					href: IRmpsConstants.ADMIN_BASELINE_STATUS
				}]
			});
		}
	},
	
	isEditableTitle: function() {
		return true;
	},
	
	getEditableTitleNodeDetails: function() {
		return messages.defaultTitleForNewSnapshot;
	},
	
	getWorkspaceUri: function() {
		return this.getEditorInput().getWorkspaceUri();
	},
	
	getWorkspaceName: function() {
		return this.getEditorInput().getWorkspaceName();
	},

	updatebreadCrumbs: function() {
		if (!this._breadCrumbs) {
			return;
		}
		var parent = this._breadCrumbs.domNode.parentNode;
		if (!this._breadCrumbs.domNode.parentNode) {
			return;
		}
		var newNode = this._createBreadCrumbs();
		parent.replaceChild(newNode.domNode, this._breadCrumbs.domNode);
		this._breadCrumbs = newNode;
	},
	
	setTitle: function() {
		this._updateButtonEnableState();
		this.updatebreadCrumbs();
		if (this._resetTitle) {
			this._resetTitle = false;
			this.setTitleValue("");
		}
	},
	
	_isSnapshotContext: function() {
		var streamUri = this.getWorkspaceUri();
		if (streamUri) {
			return false;
		} if (ContextUtils.getContextType() == rmps.core.util.ContextTypes.SNAPSHOT) {
			return true;
		}
		return false;
	},
	
	createContentWidget: function() {
		// Disables the snapshot creation page if the current context is not a Stream
		if(this.getWorkspaceName() || ContextUtils.getContextType() == rmps.core.util.ContextTypes.WORKSPACE){
			this._formContent = new rmps.app.ui.internal.editors.RequestBaselineEditor._Content({workspaceName: this.getWorkspaceName()});
		} else {
			this._formContent = new rmps.app.ui.internal.editors.RequestBaselineEditor._SnapshotsDisabled();
		}
		
		return this._formContent;
	},

	_refreshStatus: function() {
		var pageHelper = rmps.ui.editor.PageHelper.getInstance();
		var statusEditor = pageHelper.getEditor(
				new rmps.app.ui.internal.BaselineStatusEditorInput({projectId: this.getEditorInput().projectId}));
		if (statusEditor) {
			statusEditor.refresh();
		}
	},

	_requestBaseline: function() {
		var name = this.getTitleValue();
		var description = this._formContent.getDescription();

		//validate the form contents - name is required
		if (!name) {
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.nameRequired,
				closable: true
			});
		} else {
			//check to see if an import is in progress before we allow the snapshot to be created
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: messages.checking_preconditions,
				closable: true
			});
			var importInProgress = isImportInProgress(this.getEditorInput().projectId);
			importInProgress.addCallback(this, function(importInProgress) {
				if (importInProgress) {
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
						summary: messages.error_import_in_progress,
						closable: true
					});
				} else {
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
						summary: messages.creating
					});

					// find stream uri from location uri or context object  
					var streamUri = this.getWorkspaceUri();
					if (streamUri == null) {
						streamUri = ContextUtils.getCurrentContextURI();
					}
					
					var deferred = rmps.core.baseline.BaselineClient.createBaseline(streamUri, name, description);

					deferred.addCallback(this, function(){
						//call back: refresh status page, and add link
						this._refreshStatus();
						this.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.OK,
							summary: dojo.string.substitute(messages.inprogress, [IRmpsConstants.ADMIN_BASELINE_STATUS]),
							closable: true
						});
						
						// we don't want user to reuse the name. So, reset it
						// after the creation is completed successfully.
						this._resetTitle = true;
					});
					deferred.addErrback(this, function(error){
						var message;
						if (error.status === 403) {
							message = messages.error_request_permission;
						} else if (error.status ===409) {
							message = dojo.string.substitute(messages.error_name_conflict, [name]);
						} else {
							message = messages.error_request_general;
						}
						this.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
							summary: message,
							details: [{message: messages['error_request_general.explanation'] + "<br>" + messages['error_request_general.useraction']}],
							closable: true
						});
					});
				}
			});
			importInProgress.addErrback(this, function(error) {
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.error_request_general,
					closable: true
				});

			});
		}
	},

	addButtons: function(toolbar) {
		this._button = new rmps.ui.SimpleButton({
			label: messages.createButton,
			onClick: dojo.hitch(this, this._requestBaseline)
		});
		toolbar.addWidget(this._button);
		this._updateButtonEnableState();
	},
	
	_updateButtonEnableState: function() {
		if (!this._button) {
			return;
		}

		var name = this.getWorkspaceUri();
		var disable = false;
		if(!name && ContextUtils.getContextType() == rmps.core.util.ContextTypes.SNAPSHOT){
			disable = true;
		}
		this._button.domNode.disabled = disable;
	},
	
	startup: function() {
		this.inherited(arguments);
	}
});

dojo.declare("rmps.app.ui.internal.editors.RequestBaselineEditor._SnapshotsDisabled", [dijit._Widget, dijit._Templated], {
	messages: messages,
	templateString: '<p>${messages.selectAWorkspace}</p>'
});

dojo.declare("rmps.app.ui.internal.editors.RequestBaselineEditor._Content", [dijit._Widget, dijit._Templated], {

	messages: messages,
	widgetsInTemplate: true,
	templateString: '<table class="paramTable" style="width:50%;">' +
						'<tr>' +
						'<td class="label">${messages.workspaceLabel}</td>' +
						'<td dojoAttachPoint="_workspaceLabel"></td>' +
						'</tr>' +
						'<tr>' +
						'<td class="label" valign="top">${messages.descriptionLabel}</td>' +
						'<td><textarea dojoAttachPoint="_descriptionField" rows="5"></textarea></td>' +
						'</tr>' +
					'</table>',

	_help: null,
	_labelNode: null,
	_workspaceName: null,
	
	constructor: function(params) {
		if (params) {
			this._workspaceName = params.workspaceName;
		}
	},
	
	postCreate: function() {
		//initialize the workspace name label.
		dojo.create("img", {src: rmps.core.util.ContextTypes.WORKSPACE.icon.uri}, this._workspaceLabel);
		this._labelNode = dojo.doc.createTextNode("");
		this._workspaceLabel.appendChild(this._labelNode);
		if (this._workspaceName) {
			this._labelNode.nodeValue = this._workspaceName;
		} else {
			var contextDeferred =  ContextUtils.getContextDeferred();
			contextDeferred.addCallback(this, function(workspace){
				this._labelNode.nodeValue = workspace.name;
			});
		}
		
		//add context help for the name
		this._help = [];
		this._help.push(new jazz.app.ua.TextHoverHelp({uaKey: SNAPSHOT_NAME_UAKEY, contentNode: this._nameLabel}));
	},

	getDescription: function() {
		return this._descriptionField.value;
	},

	startup: function() {
		this.inherited(arguments);

		dojo.forEach(this._help, function(help) {
			help.startup();
		});
	}

});

})();