/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.ResourceEditor");

dojo.require("rmps.app.ui.internal.editors.EditorWithComments");

dojo.require("rmps.ui.resource.ResourceIconLabel");
dojo.require("rmps.app.ui.internal.util.UrlUtil");
dojo.require("rmps.app.core.util");

dojo.require("rmps.core.error");
dojo.require("rmps.core.util.ContextUtils");
dojo.require("rmps.core.explorer.ExplorerConstants");
dojo.require("jazz.app.context");

dojo.require("jazz.ui.MessageBox");

dojo.require("dojo.date.locale");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.requireLocalization("rmps.app.ui.internal.editors", "ResourceEditorMessages");


dojo.require("rmps.ui.explorer.ExplorerUtilities");
var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;

var ContextUtils= rmps.core.util.ContextUtils;

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "ResourceEditorMessages");

dojo.declare("rmps.app.ui.internal.editors.ResourceEditor", rmps.app.ui.internal.editors.EditorWithComments, {
	
	// properties
	resourceInfo: null,
	
	// private properties
	_subscribed: false,
	_subscriptionToggled: false,
	_subscriptionButton: null,
	
	postMixInProperties: function() {
		this.resourceInfo = this.editorInput.resourceInfo;
		this.inherited(arguments);
	},
	
	postCreate: function() {
		// check if the resource is part of the project context for this page
		// if it isn't then notify the user
		var application = dojo.getObject("jazz.app.currentApplication");
		if (application.context !== this.resourceInfo.resourceContext) {
			var projectAreas = application.config.projectAreas;
			
			for (var i = 0, p; p = projectAreas[i]; ++i) {
				if (this.resourceInfo.resourceContext === p.uri) {
					break;
				}
			}
			
			var diffProjectInfo = dojo.create("div", {className: "diffProjectInfo"});
			var link = dojo.create("a", {href: this.resourceInfo.uri}, diffProjectInfo);
			link.appendChild(
					dojo.doc.createTextNode(
							dojo.string.substitute(messages.diffProjectInfo,
							[projectAreas[i].name, jazz.app.context.get().name])));
							
			this.headerContainerNode.insertBefore(diffProjectInfo, this.headerContainerNode.firstChild);
		}
		
		this.inherited(arguments);
	},
	
	addTabs: function() {
		this.inherited(arguments);
		if(this.resourceInfo&& this.resourceInfo.uri){
			this.addTab("links", messages.otherLinks, "rmps.app.ui.internal.editors.views.LinksEditor", {resourceInfo: this.resourceInfo, editor: this, baseline: this.getResourceContext(),isReadOnly: this.isLinkEditorReadOnly()});
		}
		
	},
		
	isLinkEditorReadOnly: function(){
		/*Only enable creating/deleting/editing links when no baseline specified */
		return jazz.app.context.get().baseline&& ContextUtils.getContextType(jazz.app.context.get().baseline)!== rmps.core.util.ContextTypes.WORKSPACE;
	},
	
	/* 
	 *Return the context (baseline) for the resource. By default return the current baseline 
	 */
	getResourceContext: function(){
		return jazz.app.context.get().baseline;
	},
		
	_showInExplorer: function() {
		ExplorerUtil.showItemInSidebarExplorer(this.resourceInfo.uri, rmps.core.explorer.ExplorerConstants.IDs.EXPLORER_SIDEBAR_DESIGN_PAGE);
	}

});

})();
