/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.review.app.ui.internal.editors.ReviewEditor");

dojo.require("rmps.app.ui.internal.editors.ResourceEditor");

dojo.require("rmps.core.error");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.require("rmps.ui.Accordion");
dojo.require("rmps.core.review.util.ReviewUtil");
dojo.require("jazz.ui.Breadcrumbs");
dojo.require("rmps.ui.SidebarContainer");
dojo.require("rmps.ui.commenting.SelectionCommentingWidget");
dojo.require("rmps.core.review.ReviewController");
dojo.require("rmps.core.review.ReviewClient");
dojo.require("rmps.core.util.DateUtils");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.app.ui.ReviewEditorUtil");
dojo.require("rmps.ui.editor.PageHelper");
dojo.require("rmps.core.user.UserClient");
dojo.require("rmps.core.review.util.ReviewTypes");
dojo.require("rmps.ui.Sidebar");
dojo.require("dm.review.app.util.ReviewUtil");
dojo.require("rmps.core.util.ContextUtils");
dojo.require("dm.review.app.ui.internal.editors.ReviewLinksEditor");

dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
var DesignUtil = rmps.app.ui.internal.pages.design.DesignUtil;

dojo.require("rmps.core.explorer.ExplorerConstants");
var explorerIDs = rmps.core.explorer.ExplorerConstants.IDs;

dojo.require("rmps.ui.explorer.ExplorerUtilities");
var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;


dojo.requireLocalization("dm.review.app.ui.internal.editors", "ReviewEditorMessages");
var messages = dojo.i18n.getLocalization("dm.review.app.ui.internal.editors", "ReviewEditorMessages");

var UserClient= rmps.core.user.UserClient;
var ReviewTypes = rmps.core.review.util.ReviewTypes;
var ResourceInfoClient= rmps.core.resource.ResourceInfoClient;
var DateUtils= rmps.core.util.DateUtils;
var ReviewController= rmps.core.review.ReviewController;
var ReviewEditorUtil= rmps.app.ui.ReviewEditorUtil;
var PROCESS= ReviewEditorUtil.process;
var ReviewUtil= rmps.core.review.util.ReviewUtil;
var ReviewClient= rmps.core.review.ReviewClient;
var ContextUtils= rmps.core.util.ContextUtils;


var PAGE_ID = dm.review.app.util.ReviewUtil.PAGE_ID;
var NEW_REVIEW_EDITOR= "rmps.app.newReviewEditor";

var PAGE_UAKEY = "com.ibm.xtools.rmps.web.app.page_ReviewEditor001";

dojo.declare("dm.review.app.ui.internal.editors.ReviewEditor", rmps.app.ui.internal.editors.ResourceEditor, {

	resourceInfo: null,

	callBacks: null,

	reviewController: null,

	_enabled: null,

	selectionCommentingWidget: null,

	reviewMessageBox: null,

	constructor: function() {
		this.uaKey = PAGE_UAKEY;
		this._modifiedlinks= {};
	},

	postCreate: function() {
		this.addBreadCrumbs();
		this.resourceInfo = this.editorInput.gerResourceInfo();
		
		/*
		 * Required to make sure that the review updater deferred calls will be added before
		 * the review sections (the data feeds, etc) 
		 */
		this.reviewController= this.getEditorInput().reviewController;
		 
		var review= this.reviewController.review;

		this.reviewController.review.model.resources.initEntriesForWidget(this, true);
		this.reviewController.review.model.participants.initEntriesForWidget(this, true);
		
		this._refreshhandle = dojo.subscribe("rmps.web.app.review.warning.refresh."+review.getReviewId(), dojo.hitch(this, this.updateWarningDetails));
		
		this.addReviewControllerListeners();
		
		this.inherited(arguments);
		// As we now want to always show the Resources Comments as soon as a resource is added
		// we use the review associated snapshot as parameter
		var resourceInfos= this.reviewController.getResourceInfos(review.getReviewAttribute("reviewBaseline"));
		if(resourceInfos.length>0){
			this.createSelectionCommentingWidget(resourceInfos, resourceInfos);			
		}
		this.updateWarningDetails();
	},
	
	addReviewControllerListeners: function(){
		this.connect(this.reviewController, "addDirtyNotifier", dojo.hitch(this, function() {this.addDirty(this);}));
		this.connect(this.reviewController, "removeDirtyNotifier", dojo.hitch(this, function() {this.removeDirty(this);}));
		this.connect(this.reviewController, "onReviewResourceAdd", dojo.hitch(this, function() {this.updateWarningDetails();}));
		this.connect(this.reviewController, "onReviewResourceRemove", dojo.hitch(this, function(resourceURI, key, value) {this.updateWarningDetails();}));
		this.connect(this.reviewController, "onReviewReviewerUpdate", dojo.hitch(this, function(userUri, valuesList) {this.updateWarningDetails();}));
		this.connect(this.reviewController, "onReviewReviewerRemove", dojo.hitch(this, function(userUri) {this.updateWarningDetails();}));
		this.connect(this.reviewController, "onReviewReviewerAdd", dojo.hitch(this, function(users, roleType) {this.updateWarningDetails();}));
		this.connect(this.reviewController, "onReviewStatusChange", dojo.hitch(this, function(status) {
			this.createTitleContents(true);
			this.updateWarningDetails();
			this.refreshMyWorkTab();
		}));
	},
	
	activate: function(){
		// Mark the editor and the controller as dirty so we enable the save button
		// and disable the Start button. The author must save before starting the review
		//Make sure to put this call in activate method in order to esnure that it will be only called right after the editor is opened
		if (!this.isDirty() && this.editorInput.isCloned()){
			this.addDirty("cloned", true);
			this.reviewController.isDirty = true;
		}

		this.inherited(arguments);
	},

	addDataEntryNotifier: function(data, descriptions, dataFeed){
		if(this.reviewController.review.model.participants== dataFeed){
			/*Required if the current moderator is added to the particpants list */
			this.refreshMyWorkTab();
			return;
		}
		if(!data){
			return;
		}
		var resourceInfo= data.resourceInfo;
		
		/*
		 * If resourceInfo is undefined then, it means that this is the initial information we got from review. Otherwise, it means a new resource has been
		 * added 
		 */
		if(resourceInfo){
			this.addSelection(resourceInfo);			
		}
    },

    removeDataEntryNotifier: function(dataSetDetails, uniquieIdentifier, identifier, dataFeed){
    	if(this.reviewController.review.model.resources== dataFeed){
			this.removeSelection(identifier);
			return;
		}

    	if(this.reviewController.review.model.participants== dataFeed){
    		/*Required to remove the my work tab if the moderator is no longer a reviewer */
			this.refreshMyWorkTab();
			return;
		}
    },

    editDataEntryNotifier: function(dataSetDetails, uniqueIdentifier, identifier, fieldName, fieldValue){
		/*Do Nothing */        
    },
		
	addBreadCrumbs: function(){
		this._breadCrumbs= new jazz.ui.Breadcrumbs({segments: [{label: messages.reviews, href: ReviewEditorUtil.getReviewViewerEditorUri()}]});
		dojo.place(this._breadCrumbs.domNode, this.headerContainerNode, "first");
	},

	updateWarningDetails: function(){
		var review= this.reviewController.review, status;
		
		// Resize
		dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
		
		function update(statuses) {
			var reviewStartTime= review.getReviewAttribute("startDate");
			if (reviewStartTime && !review.getReviewAttribute("isFormal")){
				for(var index=0; index<statuses.length; index++){
					var status= statuses[index];
					var lastImport= status.startTime;
					if(lastImport> reviewStartTime){
						this.reviewMessageBox.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
							summary: messages.newImportWasMadeAfter,
							closable: true
						});
						break;
					}
				}
			}
		}

		if(this.reviewMessageBox){
			/*Hide and any previous messages */
			this.reviewMessageBox.hideMessage();
		}else{
			/*arbitrary assumption, just to make sure that the message box will not be null for next lines */
			this.reviewMessageBox = this.createAndStartMessagebox(this._tabContainer, this._tabContainer.containerNode);
		}
		if(!this.reviewController.allowModify&& this.reviewController.review.isModerator()){
			this.reviewMessageBox.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
				summary: messages.noModificationAllowed,
				closable: true
			});
		}else if(this.shouldShowInfoWarning()){
			this.reviewMessageBox.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: messages.reviewCannotStart,
					closable: true
				});
		} else if(!this.reviewController.isConfigurationMatches()){
			status= review.getReviewAttribute("reviewStatus");
			
			/*Do not show the warning if the review is on draft mode */
			if(status != ReviewTypes.REVIEW_STATE_TYPES.Draft){
				this.contextConflictWarning(this.reviewMessageBox, review);
			}
		}else{
			status= review.getReviewAttribute("reviewStatus");
			// TODO: Review
			if(status != ReviewTypes.REVIEW_STATE_TYPES.Paused){
				// Required check for other clients that do not use ImportClient
				var val= dojo.getObject("rmps.importer.ImportClient");
				if(val){
					val.getStatusForProject(jazz.app.context.get().itemId).addCallback(dojo.hitch(this,update));
				}
			}else{
				this.reviewMessageBox.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
					summary: messages.reviewIsNotStarted,
					closable: true
				});
			}

		}

	},

	createAndStartMessagebox: function(node, mainWidget){
		var reviewMessageBox = new jazz.ui.MessageBox();

		// We must sent a resize event when the message box is shown/hidden
		// and when the details are toggled.
		function publishResizeEvent() {
			dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
		}
		function onHideMessage() {
			delete reviewMessageBox.__cps;
			publishResizeEvent();
		}

		reviewMessageBox.domNode.style.margin = "1px";
		reviewMessageBox._first = true;
		dojo.connect(reviewMessageBox, "_toggleDetails", node, publishResizeEvent);
		dojo.connect(reviewMessageBox, "hideMessage", node, onHideMessage);
		dojo.connect(reviewMessageBox, "showMessage", node, publishResizeEvent);

		/*Place the messagebox at the beginning */
		dojo.place(reviewMessageBox.domNode, mainWidget, "first");
		reviewMessageBox.startup();
		return reviewMessageBox;
	},
	
	createRequiredNode: function() {
		if(this.getEditorInput().resourceUri == rmps.app.ui.ReviewEditorUtil.NEW_REVIEW_EDITOR){
			return this.inherited(arguments);
		}
		
		var numberSpan= this.getNumberNode();
		
		if(!numberSpan){
			return this.inherited(arguments);
		}
		var span = dojo.create("span");
		var reqNode = this.getRequiredNode();
		
		span.appendChild(reqNode);
		span.appendChild(numberSpan); 
		
		return span;
	},
	
	getNumberNode: function(){
		var number= this.getEditorInput().getResourceNumber();
		if(!number){
			return null;
		}
		var numberSpan= dojo.create("span", {style: 'fontSize: "large"; borderStyle: "1px dotted #999999"'});
		numberSpan.innerHTML= number+ ":&nbsp";
		return numberSpan;
	},
	
	getTitleNodeValue: function(){
		return this.editorInput.getPresentationName();
	},

	contextConflictWarning: function(reviewMessageBox, review) {
		var div= dojo.create("div");
		var warningMsg;
		var encodedReviewName = dojox.html.entities.encode(review.getReviewAttribute("reviewName"));
		
		if (this.reviewController.configNotExist){
			warningMsg= dojo.string.substitute(messages.contextDeleted, [encodedReviewName]);
		} else {
			warningMsg= dojo.string.substitute(messages.changeContext, [encodedReviewName,
			                                                            ContextUtils.getContextUri(review.getReviewAttribute("reviewBaseline"))]);
		}
		
		div.innerHTML= "<div>"+warningMsg+"</div>";
		
		reviewMessageBox.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
			summary: div,
			closable: true
		});
		
		function setContextHandler() {
	    	return function (e) {
				dojo.stopEvent(e);
				ContextUtils.setContext(review.getReviewAttribute("reviewBaseline"));			
	    	};
		}
		
		for(var index=0; index<div.childNodes.length; index++){
			var current= div.childNodes[index];
			if(current.tagName&& current.tagName.toLowerCase()== "div"){
				for(var subIndex=0; subIndex<current.childNodes.length; subIndex++){
					var subCurrent= current.childNodes[subIndex];
					if(subCurrent.tagName&& subCurrent.tagName.toLowerCase()== "a"){
						subCurrent.onclick= setContextHandler();
						return;
					}		
				}
			}
		}
	},

	/*Return <code>true</code> if there is no at least non-optional reviewer and one resource element for review */
	shouldShowInfoWarning: function() {
		var review= this.reviewController.review;
		return review.getUsersBasedOnRoles([ReviewTypes.PARTICIPANT_ROLE_TYPES.Approver, ReviewTypes.PARTICIPANT_ROLE_TYPES.RequiredReviewer]).length==0||
						review.getReviewResources().length==0;
	},
	
	showResourceLabel: function(){
		/*Do not show resource label icon */
		return false;
	},
	
	isEditableTitle: function(){
		return this.reviewController.review.isModerator()&& this.reviewController.isEditable();
	},
	
	isLinkEditorReadOnly: function(){
		/*Disable links tab for the review editor if 
		 * the DM context is different from the one associated to the review or if the state of the review is draft
		 * and if the user is the moderator 
		 * and if the review is not already existing*/
		var isDraft = this.reviewController.review.getReviewAttribute("reviewStatus") == ReviewTypes.REVIEW_STATE_TYPES.Draft;

		return (!this.reviewController.isConfigurationMatches() && !isDraft)
				|| !this.reviewController.review.isModerator()
				|| !this.reviewController.isExistingReview;
	},
	
	getResourceContext: function(){
		/*In regards of review links we need to use the associated context instead of the application context */
		return this.reviewController.review.getReviewAttribute("reviewBaseline");
	},
	
	onTitleChange: function(e){
		var value= this.getTitleValue();
		this.reviewController.updateReview("reviewName", value);
	},

	createSelectionCommentingWidget: function(resourceInfo, selections) {
		this.selectionCommentingWidget = new rmps.ui.commenting.SelectionCommentingWidget({
			reviewInfo: this.resourceInfo,
			resourceInfo: resourceInfo,
			isResourceLabelLink: true,
			userInfo: jazz.app.currentApplication.config.user,
			externalSave: true,
			showResourceLabel: true,
			title: messages.resourceComments,
			selections: selections,
			doCreateFilter: dojo.hitch(this, this.addReviewStartDateFilter),
			applySelectionFilter: dojo.hitch(this, this.applySelectionFilter)
		});
		this.selectionCommentingWidget.title = messages.resourceComments;
		this.selectionCommentingWidget.containerNode= this.selectionCommentingWidget.domNode;

		this.accordion.addChild(this.selectionCommentingWidget);

		this.addCommentingWidgetListeners(this.selectionCommentingWidget);
		if (this.getEditorInput().getParams().comment){
			this.accordion.showChild(this.commentingWidget);
		} else {
			this.accordion.showChild(this.selectionCommentingWidget);			
		}
	},

	addReviewStartDateFilter: function(dateRangeInput,initialDateRange){
		var du= "reviewDuration";
		var option= dojo.create("option", {value: du}, null);
		option.appendChild(dojo.doc.createTextNode(messages.reviewDuration));
		dojo.place(option, dateRangeInput, 1);
		if ( !initialDateRange || du == initialDateRange ) {
			dateRangeInput.selectedIndex = 1;
		}
	},

	applySelectionFilter: function(comment){
		/*Provide the real time filter based on the current time, as the time could go while usin the review, and some comments become in a need to be filtered out*/
		//var range= dojo.date.difference(dojo.date.stamp.fromISOString(this.reviewController.review.getStartTimestamp()),new Date(), "second")+ " "+ "seconds";

		//var date= this.getDateFromRange(range);
		
		var reviewParser= this.reviewController.review;
		var state = reviewParser.getReviewAttribute("reviewStatus");
		if (state == ReviewTypes.REVIEW_STATE_TYPES.Draft || state == ReviewTypes.REVIEW_STATE_TYPES.NotStarted){
			return true; //review is not started yet, so no comments can be created in "Review Duration" range
		}
		
		var date = this.reviewController.review.getReviewAttribute("startDate");
		var filterForFinalize= false;

		if (state == ReviewTypes.REVIEW_STATE_TYPES.Finalized){
			//var finalRange=
			//	dojo.date.difference(dojo.date.stamp.fromISOString(this.reviewController.review.getCloseTimestamp()),new Date(), "second")+ " "+ "seconds";
			//var closeDate= this.getDateFromRange(finalRange);
			var closeDate= this.reviewController.review.getReviewAttribute("closeDate");
			/*If the review is finailized, then, we show the comments in between the time the review started and fianlized */
			filterForFinalize= comment.created.getTime()> closeDate.getTime();
		}
		if(!date){
			date= new Date();
		}
		return comment.created.getTime()< date.getTime()|| filterForFinalize;
	},

	getReviewStartDateSelection: function() {
		var timeStp = this.reviewController.review.getStartTimestamp();
		var diffTime = "0 seconds";
		if (timeStp){
			diffTime = dojo.date.difference(dojo.date.stamp.fromISOString(timeStp),new Date(), "second")+ " "+ "seconds";
		}
		return diffTime;
	},
	
	snapshotFilterEnabled: function(){
		return false;
	},
	
	getEditableTitleNodeDetails: function(){
		return messages.reviewNameDefaultInstruction;
	},
	
	createSidebar: function(contentWidget){
		this.accordion = new rmps.ui.Accordion();
		var sidebar = new rmps.ui.SidebarContainer();
		sidebar.addSidebar(this.accordion, false, true, true);
		sidebar.setContent(contentWidget);
		this.commentingWidget.containerNode= this.commentingWidget.domNode;
		this.accordion.addChild(this.commentingWidget);
		this.commentingWidget.title = messages.comments;

		// Aligns the commenting header top line with the tab line.
		this.accordion.__sidebar.domNode.style.paddingTop = dojo.isIE < 8 ? "3px" : "2px";
		this.accordion.startup();
		return sidebar;
	},

//	addTabs: function() {
//		var reviewParser= this.reviewController.review;
//		var user= ReviewUtil.encodeUserUri(jazz.app.currentApplication.config.user.uri);
//		
//		var state = reviewParser.getReviewAttribute("reviewStatus");
//		
//		if(this.reviewController.review.isModerator()){
//			
//			if (state == ReviewTypes.REVIEW_STATE_TYPES.Draft || state == ReviewTypes.REVIEW_STATE_TYPES.Finalized){
//				this.addTab("main", messages.reviewMainTab, this.getApplicationLocation("main"), {isReadOnly:this._enabled});
//				this.addTab("participants", messages.reviewParticipantsTab, this.getApplicationLocation("participants"));
//				this.addTab("resources", messages.reviewResourcesTab, this.getApplicationLocation("resources"));
//			} else {
//				this.addTab("main", messages.reviewMainTab, this.getApplicationLocation("main"), {isReadOnly:this._enabled});
//				this.addTab("participants", messages.reviewParticipantsTab, this.getApplicationLocation("participants"));
//				this.addTab("resources", messages.reviewResourcesTab, this.getApplicationLocation("resources"));
//			}
//		}else {
//			if (state == ReviewTypes.REVIEW_STATE_TYPES.Draft || state == ReviewTypes.REVIEW_STATE_TYPES.Finalized){
//				this.addTab("main", messages.reviewMainTab, this.getApplicationLocation("main"), {isReadOnly: true});
//			} else {
//				this.addTab("main", messages.reviewMainTab, this.getApplicationLocation("main"), {isReadOnly: true});
//			}
//		}
//		this.refreshMyWorkTab();
//		this.inherited(arguments);
//	},

	addTabs: function() {
		var reviewParser= this.reviewController.review;
		var user= ReviewUtil.encodeUserUri(jazz.app.currentApplication.config.user.uri);
		
		var state = reviewParser.getReviewAttribute("reviewStatus");
		
		if(this.reviewController.review.isModerator()){
			if (state == ReviewTypes.REVIEW_STATE_TYPES.Draft || state == ReviewTypes.REVIEW_STATE_TYPES.Finalized){
				this.addTab("main", messages.reviewMainTab, this.getApplicationLocation("dm.review.app.ui.internal.tabs.ReviewMainTab"), {reviewController:this.reviewController});
				this.addTab("participants", messages.reviewParticipantsTab, this.getApplicationLocation("dm.review.app.ui.internal.tabs.ReviewParticipantsTab"),{reviewController:this.reviewController});
				this.addTab("resources", messages.reviewResourcesTab, this.getApplicationLocation("dm.review.app.ui.internal.tabs.ReviewResourcesTab"), {reviewController:this.reviewController});
			} else {
				this.addTab("main", messages.reviewMainTab, this.getApplicationLocation("dm.review.app.ui.internal.tabs.ReviewMainTab"), {reviewController:this.reviewController});
				this.addTab("participants", messages.reviewParticipantsTab, this.getApplicationLocation("dm.review.app.ui.internal.tabs.ReviewParticipantsTab"),{reviewController:this.reviewController});
				this.addTab("resources", messages.reviewResourcesTab, this.getApplicationLocation("dm.review.app.ui.internal.tabs.ReviewResourcesTab"),{reviewController:this.reviewController});
			}
		}else {
			if (state == ReviewTypes.REVIEW_STATE_TYPES.Draft || state == ReviewTypes.REVIEW_STATE_TYPES.Finalized){
				this.addTab("main", messages.reviewMainTab, this.getApplicationLocation("dm.review.app.ui.internal.tabs.ReviewMainTab"), {reviewController:this.reviewController});
			} else {
				this.addTab("main", messages.reviewMainTab, this.getApplicationLocation("dm.review.app.ui.internal.tabs.ReviewMainTab"), {reviewController:this.reviewController});
			}
		}
		this.refreshMyWorkTab();
		this.inherited(arguments);

		// remove parent link tab and add review specific one
		this.removeTab("links");
		if(this.resourceInfo && this.resourceInfo.uri && this.resourceInfo.uri !== NEW_REVIEW_EDITOR){
			this.addTab("links", messages.otherLinks, "dm.review.app.ui.internal.editors.ReviewLinksEditor", {resourceInfo: this.resourceInfo, editor: this, baseline: this.getResourceContext(),isReadOnly: this.isLinkEditorReadOnly(), display: this.reviewController.isConfigurationMatches()||state == ReviewTypes.REVIEW_STATE_TYPES.Draft});
		}
	},
	
	iFrameInitializer: function(){
		return this.reviewController;
	},
	
	getApplicationLocation: function(applicationId){
		return applicationId;
//		var base= net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL+ "/_ajax-modules/dm.review.tab.app";
//		if(dojo.config.isDebug){
//			base = base+ "?debug=true";
//		}
//		
//		var object= {
//			"tabId": applicationId,
//			"resourceUri": this.reviewController.review.uri
//		}
//		base = base + "#"+ dojo.objectToQuery(object);
//		return base;
	},
	
	refreshMyWorkTab: function(){
		if(this.reviewController.isReviewer()){
			var oldIdx = this.getTabIndex("myWork");
			if (oldIdx == -1) {	/*Avoid adding my work tab if it is already  added*/
				
				/*My work tab index is at the very begninng for non moderators. Otherwsie, it should be placed right after the main tab (for moderators */
				var index= this.reviewController.review.isModerator()?this.getTabIndex("main")+1:0;
				
				if(index==-1){
					index= undefined;	/*Safe check. The main tab cannot be undefined. So, place the tab at the end */
				}
				var func= dojo.hitch(this, this.initializer);
				this.addTab("myWork", messages.reviewMyWorkTab, this.getApplicationLocation("dm.review.app.ui.internal.tabs.ReviewMyWorkTab"), {reviewController:this.reviewController},index);	
			}
		}else{
			this.removeTab("myWork");
		}
	},
		
	addButtons: function(toolbar) {
		// Show in explorer
		var focusExplorerButton = toolbar.addButton(this, 
				dojo.hitch(this, this.focusOnExplorer), {
					label: messages.focusOnExplorer,
					imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/collapse_to_context_co.gif")
		});
		// --Check enabled/disabled state of show in explorer button 
		toolbar.disableButton(focusExplorerButton);
		this._checkShowExplorerButtonEnabled(toolbar, focusExplorerButton);

		// Copy Review
		var copyButton = toolbar.addButton(this, 
				dojo.hitch(this, this.copyReview), {
					label: messages.copyReview,
					imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/copy.gif")
		});
		
		
		// Delete
		if(this.reviewController.allowDeletion&& this.resourceInfo&& this.resourceInfo.uri){
			this.reviewDeleteButton = toolbar.addButton(this,
					this.removeReview, {
					label: messages.remove,
					imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/remove.gif")
			});
		}
		
		// If it is a new review we need to disable the copy an delete buttons
		if (this.reviewController.review.resourceUri === NEW_REVIEW_EDITOR){
			toolbar.disableButton(copyButton);
			if (this.reviewDeleteButton)
				toolbar.disableButton(this.reviewDeleteButton);				
		}
		
		this.inherited(arguments);
	},

	focusOnExplorer: function(e) {
		dojo.stopEvent(e);
		ExplorerUtil.focusManyItemsInSidebarExplorer(this.reviewController.review.getReviewResources(null,[]), explorerIDs.EXPLORER_SIDEBAR_REVIEW_PAGE);
	},

	_checkShowExplorerButtonEnabled: function(toolbar, button) {
		if (this.reviewController.review.getReviewResources(null,[]).length > 0) {
			toolbar.enableButton(button);
		}
	},

	copyReview: function() {
		var href = window.location.href + "&" + ReviewUtil.CLONE_PARAM;
		this.reviewController.navigateTo(href);
	},
	
	removeReview: function(){
		/*global confirm */
		if(confirm(messages.removeConfirm)){
			this._removeReview();
		}
	},
	
	/*Delete the review */
	_removeReview: function(){
		var success= dojo.hitch(this, function(){
			var pageHelper = rmps.ui.editor.PageHelper.getInstance(PAGE_ID);
			pageHelper.closeEditor(this, true, true);
			this.reviewController.navigateTo(ReviewEditorUtil.getReviewViewerEditorUri() + "&ts=" + new Date().getTime());
			dojo.publish("review.view.editor.refresh");
		});
		var deleteDeferred = ReviewClient.deleteReview(this.reviewController.review.resourceUri);
		deleteDeferred.addCallback(success);
		deleteDeferred.addErrback(dojo.hitch(this, function (error) {
			return rmps.core.error.createError(error).dispatch();
		}));

	},

	addSelection: function(resourceInfo){
		if(this.selectionCommentingWidget instanceof rmps.ui.commenting.SelectionCommentingWidget== false){
			this.createSelectionCommentingWidget([resourceInfo], [resourceInfo]);
		}else{
			this.selectionCommentingWidget.addSelection(resourceInfo);
		}
		this.selectionCommentingWidget.refresh();
	},

	removeSelection: function(resourceInfo){
		if(this.selectionCommentingWidget instanceof rmps.ui.commenting.SelectionCommentingWidget){
			this.selectionCommentingWidget.removeSelection(resourceInfo);
			if(this.selectionCommentingWidget.selections.length==0){
				this.accordion.removeChild(this.selectionCommentingWidget);
				this.selectionCommentingWidget.destroy();
				delete this.selectionCommentingWidget;
			}else{
				this.selectionCommentingWidget.refresh();				
			}
		}
	},
		
	validate: function(errors) {
		this._validateName(errors);

	},

	_validateName: function(errors){
		if(!this._editableTitle){
			return;	/*This means that the editor title is not edtibile right now */
		}
		var value= this.getTitleValue();
		if(!value|| dojo.trim(value)==""){
			errors.push(messages.empty_Title_error);
		}
	},

	doSave: function(taskMonitor) {
		var parentArguments= arguments;
		
		var doUpdate= dojo.hitch(this, function(){
			if(this.selectionCommentingWidget){
				this.selectionCommentingWidget.doSave(taskMonitor);
			}
			
			var task = taskMonitor.beginTask("", 1);
			var deferred= this.reviewController.releaseUpdates(taskMonitor);
			dojo.forEach(this.callBacks, function(callback){
	                callback();
	         });
	
			/*
			* No need to request a refresh, as we already know hat we just save, and the current UI information are supposed not to have any changes.
			* this.reviewController.requestUpdate();
			*/	
			
			task.handleDeferredWork(deferred, 1);
		});
		if(this.reviewController.review.isModerator()&& (this.getEditorInput().resourceUri== rmps.app.ui.ReviewEditorUtil.NEW_REVIEW_EDITOR
				|| this.getEditorInput().isCloned())){
			this.__shouldRefresh= true;
			var reviewCreationTask;
			var reviewCreationDeferred= ReviewClient.createReview(this.reviewController.review);
			reviewCreationDeferred.addCallback(dojo.hitch(this,function(){
					var resourceInfo= this.editorInput.gerResourceInfo();
					this.commentingWidget.resourceInfo= resourceInfo;
					if(this.commentingWidget._commentForm.comment){
						this.commentingWidget._commentForm.comment._resourceInfo= resourceInfo; 
						this.commentingWidget._commentForm.comment.resourceUri= resourceInfo.uri;	
					}
					this.inherited("doSave",parentArguments);
					reviewCreationTask.handleDeferredWork(reviewCreationDeferred, 1);
			}));
			reviewCreationTask = taskMonitor.beginTask("", 1);
			reviewCreationDeferred.addErrback(this, function(error) {
				reviewCreationTask.handleDeferredWork(reviewCreationDeferred, 1);
				this.inherited("doSave",parentArguments);
			});
		}else{
			doUpdate();
			/*Call the parent in order to handle the subscribtion pending changes */
			this.inherited("doSave",arguments);
		}
	},
	
	onSaveComplete: function(){
		dojo.publish("review.view.editor.refresh");
		if(this.__shouldRefresh){
			delete this.__shouldRefresh;
			var pageHelper = rmps.ui.editor.PageHelper.getInstance(PAGE_ID);
			if(pageHelper){
				pageHelper.closeEditor(this, true, true);				
			}
			this.reviewController.navigateTo(ReviewEditorUtil.getReviewEditorUri(this.reviewController.review.uri) + "&ts=" + new Date().getTime());
		}else{
			return this.inherited(arguments);
		}
	},

	destroyRecursive: function(){
		ReviewController.removeUpdater(this.getEditorInput().resourceUri);
		this.removeCommentingWidgets();
		this.inherited(arguments);
	},

	destroy: function(){
		this.removeCommentingWidgets();
		if(this.reviewMessageBox){
			this.reviewMessageBox.destroy();
		}
		if(this._breadCrumbs){
			this._breadCrumbs.destroy();
		}
		if(this.accordion){
			this.accordion.destroy();
		}
		if(this.selectionCommentingWidget){
			this.selectionCommentingWidget.destroy();
		}

		this.inherited(arguments);
		this.callBacks= null;
	},
		
	removeCommentingWidgets: function(){
		/*We have to remove the commenting widget from the children as it will be destroyed later by the parent class. Otherwise, there will be an extra
		 * attemp to detroy  commentingWidget in a lter call for destroyRecursive, and this will cause an error*/
		if(this.commentingWidget){
			var children= this.accordion.getChildren();
			for(var index=0 ;index< children.length; index++){
				var current= children[index];
				if(current==this.commentingWidget){
					this.accordion.removeChild(this.commentingWidget);
				}else if(current==this.selectionCommentingWidget){
					this.accordion.removeChild(this.selectionCommentingWidget);
					this.selectionCommentingWidget.destroy();
					delete this.selectionCommentingWidget;
				}
			}
		}
	}
});

})();
