/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.review.app.ui.internal.editors.ReviewEditorInput");

dojo.require("jazz.app.context");

dojo.require("rmps.ui.editor.EditorInput");
dojo.require("dojox.encoding.digests.MD5");
dojo.require("rmps.app.ui.ReviewEditorUtil");
dojo.require("rmps.core.review.util.ReviewImageService");
dojo.require("rmps.core.review.util.ReviewUtil");
dojo.require("rmps.core.review.ReviewController");
dojo.require("rmps.app.ui.internal.util.UrlUtil");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmps.core.review.AbstractReviewControllerDescriptor");

var runningCount = 1;
var ReviewEditorUtil= rmps.app.ui.ReviewEditorUtil;
var ReviewController= rmps.core.review.ReviewController;
var EditorInput= rmps.ui.editor.EditorInput;
var ReviewImageService= rmps.core.review.util.ReviewImageService;
var UrlUtil= rmps.app.ui.internal.util.UrlUtil;
var AbstractReviewControllerDescriptor= rmps.core.review.AbstractReviewControllerDescriptor;
var PROCESS= ReviewEditorUtil.process;
var ReviewUtil= rmps.core.review.util.ReviewUtil;

dojo.declare("dm.review.app.ui.internal.editors.ReviewEditorInput", [EditorInput, AbstractReviewControllerDescriptor], {

	resourceUri: null,
	_hasSidebarExplorer: true,

	constructor: function(params) {
		if (params) {
			dojo.mixin(this, params);
		}
	},

	isLoaded: function() {
		return ReviewController.getInstance(this.resourceUri, this).isLoaded;
	},

	load: function() {
		runningCount++;
		return ReviewController.getInstance(this.resourceUri, this).deferred;
	},
	
	/*The current baseline is the application baseline (not the review) */	
	getBaseline: function(){
		return jazz.app.context.get().baseline;
	},

	gerResourceInfo: function(){
		/*TODO: Show last modified */
		var context = this.getOwingContext().uri;
		var review= this.getReview();
		var shortName= "";
		if(review){
			var nameAtt= review.getReviewAttribute("reviewName");
			if(nameAtt){
				shortName= nameAtt;
			}
		}
		return {
			shortName: shortName,
			uri: review? review.resourceUri:rmps.app.ui.ReviewEditorUtil.NEW_REVIEW_EDITOR,
			resourceContext: context
		};
	},

	getId: function() {
		if(!this._id){
			if(this.resourceUri!=rmps.app.ui.ReviewEditorUtil.NEW_REVIEW_EDITOR){
				this._id= this.resourceUri;
			}else{
				var val= this.ts?this.ts:runningCount;
				this._id= dojox.encoding.digests.MD5(this.declaredClass + ',' + val);				
			}
		}
		return this._id;
	},
	
	getDescriptorId: function(){
		return this.getId();
	},

	getReview: function() {
		return this.reviewController?this.reviewController.review:null;
	},
		
	getName: function() {
		if(!this.reviewController){
			return "";
		}
		var review= this.reviewController.review;
		if(!review){
			return "";
		}
		var name= review.getReviewAttribute("reviewName");
		return name?name: "";
	},

	getBasePresentationUri: function(state) {
		var ts= null;
		
		/*We use timestamp for new reviews only, and if there is already timestamp added to the editor input states, then, no need to use the eixsting ts, as
		 * we will get it by this.getStates() method later.
		 * 
		 * We should not have a value for ts at all. This would be handled by:
		 * 21545: Showing a new resource creates a new editor input every time
		 */
		if(rmps.app.ui.ReviewEditorUtil.NEW_REVIEW_EDITOR&&!state.ts){
			ts= this.ts;
		}
		return rmps.app.ui.ReviewEditorUtil.getReviewEditorUri(this.resourceUri, ts);
	},
	
	getIconUri: function() {
		return ReviewImageService.REVIEW_EDITOR_ICON;
	},
	
	getPresentationName: function() {
		var numberSegment= this.getResourceNumber()? this.getResourceNumber()+ ": ": "";
		var name= this.getName(); 
		return name ? numberSegment + name : null;
	},
		
	getResourceNumber: function() {
		if(!this.resourceUri){
			return null;
		}
		var number = this.resourceUri.match(/\/(\d+)$/);
		
		if (number != null && number[1]) {
			return number[1];
		}
		
		return null;
	},
	
	isCloned: function() {
		var cloned = this.resourceUri?this.resourceUri.indexOf(ReviewUtil.CLONE_PARAM)>-1:false;
		return cloned;
	}
});

})();
