/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.review.app.ui.internal.editors.ReviewViewerEditor");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.core.review.ReviewClient");
dojo.require("rmps.ui.sections.TableData");
dojo.require("rmps.ui.PagingControl");
dojo.require("rmps.app.core.util");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.HoverPopup");
dojo.require("rmps.core.review.util.ReviewImageService");
dojo.require("rmps.core.review.util.ReviewTypes");
dojo.require("rmps.app.ui.ReviewEditorUtil");
dojo.require("rmps.core.review.util.ReviewUtil");
dojo.require("rmps.core.streams.StreamClient");
dojo.require("rmps.core.util.ContextUtils");
dojo.require("rmps.ui.DialogForm");
dojo.require("rmps.core.error");
dojo.require("dijit.form.RadioButton");
dojo.require("dijit.form.Form");
dojo.require("rmps.ui.InfoArea");
dojo.require("dijit.form.CheckBox");

dojo.require("dojo.string");
dojo.require("dojo.i18n");

dojo.requireLocalization("dm.review.app.ui.internal.editors", "ReviewViewerEditorMessages");
dojo.require("rmps.core.util.DateUtils");
var DateUtils = rmps.core.util.DateUtils;
var ReviewUtils = rmps.core.review.util.ReviewUtil;
var messages = dojo.i18n.getLocalization("dm.review.app.ui.internal.editors", "ReviewViewerEditorMessages");
var StreamClient= rmps.core.streams.StreamClient;
var ContextUtils= rmps.core.util.ContextUtils;

var PAGE_UAKEY = "com.ibm.xtools.rmps.web.app.page_ReviewViewerEditor001";
var ReviewTypes= rmps.core.review.util.ReviewTypes;
var ReviewImageService= rmps.core.review.util.ReviewImageService;
var HoverPopup = jazz.ui.HoverPopup;



/********************************************Private Static Variables/Functions*******************************************************************************/
var REVIEW_SEARCH_PARAMATERS= {
	SEARCH_SCOPE: "searchScope",
	ALL_REVIEWS_FILTER: "allReviews",
	MY_REVIEWS_FILTER: "myReviews",
	AUTHOR_FILTER: "author",
	APPROVER_ROLE_FILTER: "approverRole",
	REVIEWER_ROLE_FILTER: "reviewerRole",
	SUBSCRIBER_FILTER: "subscriber",
	ACTIVE_REVIEWS_FILTER: "activeReviews",
	FINALIZED_REVIEWS_FILTER: "finalizedReviews",
	CONTEXT_REVIEWS_FILTER: "currentWorkspace"
};

/*Static global paramter tracker to be able to detect the last parameters made even when the windows url is changed (due to opening the editor by other
 * places such as a review editor) */
var parameterTracker;

var REVIEW_SEARCH_PARAMATERS_MAP= {};

REVIEW_SEARCH_PARAMATERS_MAP[REVIEW_SEARCH_PARAMATERS.MY_REVIEWS_FILTER]=			messages.myReviews;
REVIEW_SEARCH_PARAMATERS_MAP[REVIEW_SEARCH_PARAMATERS.ALL_REVIEWS_FILTER]=			messages.allProjectReviews;
REVIEW_SEARCH_PARAMATERS_MAP[REVIEW_SEARCH_PARAMATERS.AUTHOR_FILTER]=				messages.author;
REVIEW_SEARCH_PARAMATERS_MAP[REVIEW_SEARCH_PARAMATERS.APPROVER_ROLE_FILTER]=		messages.approver;
REVIEW_SEARCH_PARAMATERS_MAP[REVIEW_SEARCH_PARAMATERS.REVIEWER_ROLE_FILTER]=		messages.reviewer;
REVIEW_SEARCH_PARAMATERS_MAP[REVIEW_SEARCH_PARAMATERS.SUBSCRIBER_FILTER]=			messages.subscriber;
REVIEW_SEARCH_PARAMATERS_MAP[REVIEW_SEARCH_PARAMATERS.ACTIVE_REVIEWS_FILTER]=		messages.active;
REVIEW_SEARCH_PARAMATERS_MAP[REVIEW_SEARCH_PARAMATERS.FINALIZED_REVIEWS_FILTER]=	messages.finalized;
REVIEW_SEARCH_PARAMATERS_MAP[REVIEW_SEARCH_PARAMATERS.CONTEXT_REVIEWS_FILTER]=		messages.currentContext;


var ReviewClient= rmps.core.review.ReviewClient;
var CoreUtil= rmps.app.core.util;
var ReviewEditorUtil= rmps.app.ui.ReviewEditorUtil;

var handleError= function(error){
	rmps.core.error.createError(error).dispatch();
};

var containAtLeast= function(list, params){
	for(var index=0; index<list.length; index++){
		if(params[list[index]]== "true"){
			return true;
		}
	}
};

var extractCurrentParamaters= function(normalized){
	var params= {};
	var query= dojo.queryToObject(window.location.hash);
	if(!dojo.isMoz&& query.query){	/*Required for IE and Chrome */
		query= dojo.queryToObject(query.query);
	}

	for(var obj in REVIEW_SEARCH_PARAMATERS){
		var param= REVIEW_SEARCH_PARAMATERS[obj];
		var current= query[param];
		if(current&& current!= "false"){
			params[param]= current;
		}
	}
	if(parseInt(query.page)){
		params.page= parseInt(query.page);
	}
	var pageSize= parseInt(query.pageSize);
	if(pageSize||query.pageSize== "showAll"){
		params.pageSize= pageSize?pageSize:query.pageSize;
	}

	params.sort= query.sort;

	if(normalized){
		var searchScope= query[REVIEW_SEARCH_PARAMATERS.SEARCH_SCOPE];
		if(!searchScope){
			searchScope= REVIEW_SEARCH_PARAMATERS.MY_REVIEWS_FILTER;
		}
		var isParticipationFilterSet= containAtLeast([REVIEW_SEARCH_PARAMATERS.AUTHOR_FILTER,REVIEW_SEARCH_PARAMATERS.APPROVER_ROLE_FILTER,REVIEW_SEARCH_PARAMATERS.REVIEWER_ROLE_FILTER,REVIEW_SEARCH_PARAMATERS.SUBSCRIBER_FILTER],
									query);

		var isStatusFilterSet= containAtLeast([REVIEW_SEARCH_PARAMATERS.ACTIVE_REVIEWS_FILTER,REVIEW_SEARCH_PARAMATERS.FINALIZED_REVIEWS_FILTER],
									query);

		var isContextFilterSet= containAtLeast([REVIEW_SEARCH_PARAMATERS.CONTEXT_REVIEWS_FILTER],
				query);

		if(!isParticipationFilterSet&& searchScope== REVIEW_SEARCH_PARAMATERS.MY_REVIEWS_FILTER){
			params[REVIEW_SEARCH_PARAMATERS.AUTHOR_FILTER]= "true";
			params[REVIEW_SEARCH_PARAMATERS.APPROVER_ROLE_FILTER]= "true";
			params[REVIEW_SEARCH_PARAMATERS.REVIEWER_ROLE_FILTER]= "true";
			params[REVIEW_SEARCH_PARAMATERS.SUBSCRIBER_FILTER]= "true";
		}

		if(!isStatusFilterSet){
			params[REVIEW_SEARCH_PARAMATERS.ACTIVE_REVIEWS_FILTER]= "true";
		}		
		if(!isContextFilterSet){
			params[REVIEW_SEARCH_PARAMATERS.CONTEXT_REVIEWS_FILTER]= "false";
		}		
	}
	return params;
};

/*************************************************************************************************************************************************/


/********************************************ReviewViewerEditor***********************************************************************************/
dojo.declare("dm.review.app.ui.internal.editors.ReviewViewerEditor", rmps.app.ui.internal.editors.BaseEditor, {

	"class": "rmps-app-editors-review-viewer",

	reviewEditorContents: null,
	currentValidContexts: null,

	constructor: function() {
		this.uaKey = PAGE_UAKEY;
		this.isReadOnly= true;
		parameterTracker= {};
	},

	postCreate: function() {
		var deferredContext= null;
		var parent= null;
		
		this.domNode.style.overflow = "auto";

		/*Required to remove wrong default scroll bars in IE*/
		this.containerNode.style.overflow = "hidden";
		this.inherited(arguments);
		var currentcontext= jazz.app.context.get().baseline;
		this.currentValidContexts= [currentcontext];
		var contextType = ContextUtils.getContextType(currentcontext);		
		if ( contextType === rmps.core.util.ContextTypes.SNAPSHOT){
			parent= true;
		}	
		deferredContext = StreamClient.getStream(currentcontext, parent);
		deferredContext.addCallback(this, function (stream) {
			this.currentValidContexts.push(stream.uri);
			this.currentValidContexts= this.currentValidContexts.concat(stream.snapshots);
			this._postCreate();
		});
		deferredContext.addErrback(function(error) {
			handleError(error);
		});
	},

_postCreate: function() {

		parameterTracker= extractCurrentParamaters(true);

		this.reviewEditorContents= new dm.review.app.ui.internal.editors.ReviewViewerEditor.Contents_({messages: messages});

		this.tableData= new dm.review.app.ui.internal.editors.ReviewViewerEditor._TableData({
			currentValidContexts: this.currentValidContexts,
			onPageSizeChange: dojo.hitch(this, this.onPageSizeChange),
			onPageChange: dojo.hitch(this, this.onPageChange),
			messages: messages,
			page: parameterTracker.page?parameterTracker.page:0,
			pageSize: parameterTracker.pageSize?parameterTracker.pageSize: 10,
			onTableBodyClick: dojo.hitch(this, this.openReviewEditor)
		});

		this.containerNode.appendChild(this.reviewEditorContents.domNode);
		this.tableData.startup(this.containerNode);
		this.containerNode.appendChild(this.tableData.domNode);
		this.updateFilterHeader();
		this.subscribe("review.view.editor.refresh", dojo.hitch(this, this.refreshEditor));		
	},

	openReviewEditor: function(dataSet){
		if(dataSet){
			var data= dataSet.data;
			if(data&& data.link){
				window.location.href = ReviewEditorUtil.getReviewEditorUri(data.link);
			}
		}
	},

	refreshEditor: function(){
		this.tableData.refreshTableData();
	},

	// Override parent function
	refreshHref: function(timestamp){
		if (parameterTracker){
			window.location.href = ReviewEditorUtil.getReviewSearchUri(dojo.objectToQuery(parameterTracker)) + "&ts=" + timestamp;
		} else {
			window.location.href = this.getEditorInput().getPresentationUri() + "&ts=" + timestamp;						
		}
	},
	
	getPageSize: function(){
		return this.tableData.pageSize;
	},

	getPage: function(){
		return this.tableData.page;
	},

	getLimit: function(){
		return this.tableData.limit;
	},

	getSize: function(){
		return this.tableData.size;
	},

	addTextActions: function(){
		var self= this;
		rmps.app.ui.ReviewEditorUtil.canCreateNewReview(function(){
			self.addTextAction(messages.createReview, ReviewImageService.REVIEW_EDITOR_ICON.uri,
					function(){location.href= rmps.app.ui.ReviewEditorUtil.getNewReviewEditorUri();});
			self.addReviewViewerEditorButtons();	
		}, false, function(){
			self.addReviewViewerEditorButtons();
		});
	},

	addButtons: function(toolbar) {
	
		var parent= this.getInherited("addButtons", arguments);
		var self= this;
		this.addReviewViewerEditorButtons= dojo.hitch(this, function(){
			this.refreshButton = toolbar.addButton(this, this.onFilterDialogChange, {
				label: messages.filter,
				imgSrc: ReviewImageService.FILTER
			});
			toolbar.addSeparator();
			dojo.hitch(self, parent)(toolbar);
		});
		
		/*Add buttons after gettings the text actions deferred call, so the order will be right */
	},

	onFilterDialogChange: function(userUri, callBack) {
		var form = new rmps.app.ui.internal.editors.views.review.ui.sections._FilterDialog({
			messages: messages,
			REVIEW_SEARCH_PARAMATERS: REVIEW_SEARCH_PARAMATERS,
			onOk: dojo.hitch(this, this.onFilterChange)
		});
		form.show();
	},

	onFilterChange: function(params){
		params.pageSize= parameterTracker.pageSize;
		params.page= parameterTracker.page;
		params.sort= parameterTracker.sort;
		parameterTracker= params;

		window.location.href = ReviewEditorUtil.getReviewSearchUri(dojo.objectToQuery(parameterTracker)) + "&ts=" + new Date().getTime();
		this.updateFilterHeader();

		this.tableData.refreshTableData();
	},

	onPageSizeChange: function(selection){
		if(!selection){
			selection= "showAll";
		}

		parameterTracker.pageSize= selection;
		window.location.href = ReviewEditorUtil.getReviewSearchUri(dojo.objectToQuery(parameterTracker)) + "&ts=" + new Date().getTime();
	},

	onPageChange: function(page){
		parameterTracker.page= page;
		window.location.href = ReviewEditorUtil.getReviewSearchUri(dojo.objectToQuery(parameterTracker)) + "&ts=" + new Date().getTime();
	},

	updateFilterHeader: function(params){
		var searchScope= parameterTracker[REVIEW_SEARCH_PARAMATERS.SEARCH_SCOPE];
		if(!searchScope){
			searchScope= REVIEW_SEARCH_PARAMATERS.MY_REVIEWS_FILTER;
		}

		var label= messages.filterHeading+ ": ";
		label+=REVIEW_SEARCH_PARAMATERS_MAP[searchScope];

		var isFirst= false;

		var appender= function(param){
			if(parameterTracker[param]== "true"){
				if(isFirst){
					label+= ",";
				}
				label+= " "+ REVIEW_SEARCH_PARAMATERS_MAP[param];
				isFirst= true;
			}
		};

		var appenders= function(list){
			for(var index=0; index<list.length; index++){
				appender(list[index]);
			}
		};

		if(searchScope== REVIEW_SEARCH_PARAMATERS.MY_REVIEWS_FILTER){
			label+= " (";
			appenders([REVIEW_SEARCH_PARAMATERS.AUTHOR_FILTER,REVIEW_SEARCH_PARAMATERS.APPROVER_ROLE_FILTER,REVIEW_SEARCH_PARAMATERS.REVIEWER_ROLE_FILTER,REVIEW_SEARCH_PARAMATERS.SUBSCRIBER_FILTER]);
			label+= " )";
		}
		label+=";";
		isFirst= false;
		appenders([REVIEW_SEARCH_PARAMATERS.ACTIVE_REVIEWS_FILTER,REVIEW_SEARCH_PARAMATERS.FINALIZED_REVIEWS_FILTER]);

		this.reviewEditorContents.setValue(label);
	},
	
	destroy: function() {
		dojo.unsubscribe(this._refreshhandle);
		this.inherited(arguments);
	}

});
/*************************************************************************************************************************************************/


/***********************************************************Header Contents***********************************************************************/
dojo.declare("dm.review.app.ui.internal.editors.ReviewViewerEditor.Contents_", [dijit._Widget, dijit._Templated], {

	messages: messages,

	widgetsInTemplate: true,

	templateString:	'<div>' +
						'<div class="sectionHeader" dojoAttachPoint="headerNode"></div>' +
					'<div>',

	setValue: function(value){
		this.headerNode.innerText= value;
		this.headerNode.innerHTML= value;
		this.headerNode.textContent= value;
	}
});
/*************************************************************************************************************************************************/

var TABLE_FIELDS= ["status", "name", "dueDate", "role", "lastModified", "context"];

/****************************************************************Table Data***********************************************************************/
dojo.declare("dm.review.app.ui.internal.editors.ReviewViewerEditor._TableData", rmps.ui.sections.TableData, {

    messages: null,

    enablePaging: true,
    enablePagingFooter: true,
    filterField: "name",
    _timestampHandle: null,
	currentValidContexts: null,

	constructor: function(params){
        this.pagingSelections= [10, 25, 50, 75, 100, 250, 500];
    },

    addColumnDescriptors: function(){
		this.addColumnDescriptor({columnHeader:messages.name, sortable: true, fieldName: "name",  width: "20%", creatorCallback: this.createNameCell});
		this.addColumnDescriptor({columnHeader:messages.context, sortable: true, fieldName: "context", width: "20%", creatorCallback: this.createContextCell});
		this.addColumnDescriptor({columnHeader:messages.state, sortable: true, fieldName: "status", width: "15%", creatorCallback: this.createStateCell});
		this.addColumnDescriptor({columnHeader:messages.myRole,sortable: true, fieldName: "role", width: "15%", creatorCallback: this.createMyRoleCell });
        this.addColumnDescriptor({columnHeader:messages.dueDate,sortable: true, fieldName: "dueDate", width: "15%", creatorCallback: this.createDueDateCell});
        this.addColumnDescriptor({columnHeader:messages.lastModified,sortable: true, fieldName: "updated", width: "15%",creatorCallback: this.createLastModifiedCell});
    },

	createTable: function(args){
		 var _table = this.inherited(arguments);
         dojo.addClass(_table.tbody, "viewerTable");
         return _table;
	},

	getSortComparator: function(sortDetails){
		var direction;
		if(sortDetails.orderStatus=="descending"){
			direction= "descending_"+sortDetails.fieldName;
		}else{
			direction= "ascending_"+sortDetails.fieldName;
		}
		parameterTracker.sort= direction;
		window.location.href = ReviewEditorUtil.getReviewSearchUri(dojo.objectToQuery(parameterTracker)) + "&ts=" + new Date().getTime();
		return this.getSortHandler(sortDetails.fieldName,sortDetails.orderStatus);
	},

	getSortHandler: function(fieldName, orderStatus){
		if(fieldName === "dueDate"|| fieldName === "lastModified"){
			return dojo.hitch(this, this.getDateComparator, orderStatus, fieldName);
		}else if(fieldName === "status"|| fieldName === "name" || fieldName === "context"){
			return dojo.hitch(this, this.getGenericComparator, orderStatus, fieldName);
		}else if(fieldName === "role"){
			return dojo.hitch(this, this.getRoleComparator, orderStatus, fieldName);
		}
	},

	getGenericComparator: function(orderStatus,fieldName, a, b) {
		return this._stringCompare(a[fieldName], b[fieldName], orderStatus);
	},

	getRoleComparator: function(orderStatus,fieldName, a, b) {
		var ta = a[fieldName];
		var tb = b[fieldName];

		ta = ta?ReviewTypes.getParticipantRoleTypeDisplayName(ta):ta;
		tb = tb?ReviewTypes.getParticipantRoleTypeDisplayName(tb): tb;

		return this._stringCompare(ta, tb, orderStatus);
	},

	_stringCompare: function(ta, tb, orderStatus){
		function check(string){
			return string && dojo.trim(string);
		}
		function stringCompare(string1, string2){
			if(!check(string1)&& !check(string2)){
				return 0;
			}else if(!check(string1)&& check(string2)){
				return 1;
			}else if(check(string1)&& !check(string2)){
				return -1;
			}
			return string1.localeCompare(string2);
		}

		return orderStatus== "ascending"? stringCompare(ta, tb): stringCompare(tb, ta);
	},

	getDateComparator: function(orderStatus,fieldName, a, b) {
		var ta = a[fieldName];
		var tb = b[fieldName];

		ta = ta&& ta.getTime? ta.getTime(): ta;
		tb = tb&& tb.getTime? tb.getTime(): tb;

		var operator= orderStatus== "ascending"? ta < tb: ta > tb;

		return operator ? 1 : ta == tb ? 0 : -1;
	},

	insertBefore: function(){
		return false;
	},

   createStateCell: function(section, table, fieldName, value, data, tableRow, td) {
   		var img= this.createReviewStateTypeImage(value);
   		if(img){
			dojo.addClass(img, "imageCell");
			td.appendChild(img);
		}
        return this.createTextCell(td, ReviewTypes.getReviewStateDisplayName(value));
	},
	
	createReviewStateTypeImage: function(reviewStatusType) {
		var img = document.createElement("img");
		img.title = reviewStatusType;
		img.alt= reviewStatusType;
		img.src = ReviewTypes.getReviewStateTypeImageURI(reviewStatusType);
		return img;
	},

	createNameCell: function(section, table, fieldName, value, data, tableRow, td) {
		var anchor = dojo.create("a", {href: rmps.app.ui.ReviewEditorUtil.getReviewEditorUri(data.link), title: value}, td);
		anchor.appendChild(dojo.doc.createTextNode(value));
//		var resourceLink= new jazz.ui.ResourceLink({
//						retainLinkText: true,
//						lazyFetch: true,
//						presentationUri: rmps.app.ui.ReviewEditorUtil.getReviewEditorUri(data.link),
//						label: value,
//						uri: net.jazz.ajax._contextRoot + "/rendering?resourceUri=" +  encodeURIComponent(data.link)
//					}, anchor);
//		this.connect(this, "uninitialize", function() {
//			resourceLink.destroyRecursive();
//		});
		return td;
	},

	createDueDateCell: function(section, table, fieldName, value, data, tableRow, td) {
		if(!value){
			return td;
		}
   		return this.createTextCell(td, this.getShortDate(value));
	},

	createContextCell: function(section, table, fieldName, value, data, tableRow, td) {
		if(!value){
			return td;
		}
		if (!value.name){
			var img = document.createElement("img");
			img.src = rmps.app.ui.internal.common.ImageService.INLINE_WARNING;
			
			var tooltip = new HoverPopup({
				around: img,
				html: this.messages.configurationNotExist
	        });
	        this.connect(this, "uninitialize", function() {
				tooltip.destroyRecursive();
			});

			td.appendChild(img);
		}
   		return this.createTextCell(td, value.name);
	},

	createMyRoleCell: function(section, table, fieldName, value, data, tableRow, td) {
		var displayName = rmps.core.review.util.ReviewTypes.getParticipantRoleTypeDisplayName(value);
		if(displayName){
			var img = this.createParticipantRoleImage(value);
			if(img){
				dojo.addClass(img, "imageCell");
				td.appendChild(img);
			}
		}
   		return this.createTextCell(td, displayName);
	},
	
	createParticipantRoleImage: function(participantRoleType) {
    	var title= ReviewTypes.getParticipantRoleTypeDisplayName(participantRoleType, false);
    	if(!title){
    		return null;
    	}
		var img = document.createElement("img");
		img.title = title;
		img.alt = title;
		img.src = ReviewTypes.getParticipantRoleTypeImageURI(participantRoleType);
		return img;
	},

	createLastModifiedCell: function(section, table, fieldName, value, data, tableRow, td) {
		if(!value){
			return td;
		}
		this._timestampHandle = DateUtils.addDateDiffUpdater(value,
			dojo.hitch(this, function(newDateRange) {
				dojo.html._setNodeContent(td, dojo.doc.createTextNode(newDateRange), true);
			}));

   		return td;
	},

	createTextCell: function(td, value){
		td.appendChild(dojo.doc.createTextNode(value?value:""));
        return td;
	},

	getShortDate: function(value){
		return dojo.date.locale.format(value, { formatLength:"medium", selector:"date"});
	},

  	fillTable: function(section, table, onCall){
		var deferred = this.getSearchDeferred();
		var sortDirection, sortFieldName;
		var sort = parameterTracker.sort;
		if (sort) {
			var segments = sort.split("_");
			if (segments.length == 2) {
				var directionSegment = segments[0];
				var fieldNameSegment = segments[1];

				/* Precaution to avoid corrputed altered URLs */
				if (directionSegment != "ascending"&& directionSegment != "descending") {
					sortDirection = "ascending";
				} else {
					sortDirection = directionSegment;
				}

				if (fieldNameSegment&& dojo.indexOf(TABLE_FIELDS, fieldNameSegment) > -1) {
					sortFieldName = fieldNameSegment;
				} else {
					sortFieldName = "dueDate";
				}
			}
		}

		if(!sortDirection||!sortFieldName){
			sortDirection= "ascending";
			sortFieldName= "dueDate";
		}

		var sortHandler= this.getSortHandler(sortFieldName,sortDirection);

		deferred.addCallback(dojo.hitch(this, function(reviews) {
			reviews= this.getFilteredReviews(reviews);
			reviews.sort(sortHandler);
			this.changeOrderStatus(sortFieldName, sortDirection== "ascending");
			onCall(section, table, reviews);
		}));
		deferred.addErrback(function(error) {
			handleError(error);
		});
    },

    getSearchDeferred: function(){
    	return ReviewClient.getReviews(jazz.app.context.get().itemId,
    					parameterTracker.searchScope== REVIEW_SEARCH_PARAMATERS.ALL_REVIEWS_FILTER
    					?null: jazz.app.currentApplication.config.user.userId);
    },

    getStatusFilterDescriptor: function(){
    	var finalizedReviewsFilter= parameterTracker.finalizedReviews;
    	var activeReviewsFilter= parameterTracker.activeReviews;
    	if(activeReviewsFilter== "true"&& finalizedReviewsFilter== "true"){
			return function(review){return true;};
		}else if (activeReviewsFilter!= "true"&& finalizedReviewsFilter!= "true"){
			return function(review){return false;};
		}else if (finalizedReviewsFilter== "true"){
			return function(review){return review.status== "Finalized";};
		}else if (activeReviewsFilter== "true"){
			return function(review){return review.status!= "Finalized";};
		}else{
			return function(review){return false;};
		}
    },

    getRoleFilterDescriptor: function(){
    	var approverRoleFilter= parameterTracker.approverRole == "true";
    	var reviewerRoleFilter= parameterTracker.reviewerRole == "true";
		var subscriberRoleFilter= parameterTracker.subscriber == "true";
		
		if (!approverRoleFilter && !reviewerRoleFilter && !subscriberRoleFilter){
			return function(review){return false;};			
		} else {
			var filters = [];
			var i=0;
			if (approverRoleFilter){
				filters[i] = "Approver";
				i+=1;
			}
			if (reviewerRoleFilter){
				filters[i] = "Reviewer";
				i+=1;				
			}
			if (subscriberRoleFilter){
				filters[i] = "OptionalReviewer";
				i+=1;				
			}
			return function(review){
				return dojo.indexOf(filters, review.role)>-1;
			};
			
		}
   },

    getAuthorFilterDescriptor: function(){
    	var authorFilter= parameterTracker.author;
		if(authorFilter== "true"){
			return function(review){return ReviewUtils.decodeUserUri(review.moderator)== ReviewUtils.decodeUserUri(jazz.app.currentApplication.config.user.uri);};
		}else{
			return function(review){return false;};
		}
    },

    getContextFilterDescriptor: function(validContexts){
    	var contextFilter= parameterTracker.currentWorkspace;
		if(contextFilter== "true"){
			return function(review){
					return dojo.indexOf(validContexts, review.context.uri)>-1;
				};
		}
		return function(review){return true;};
    },
    
    getFilteredReviews: function(reviews){
    	var list= [];

		var checkStatus= this.getStatusFilterDescriptor();
		var checkRole= this.getRoleFilterDescriptor();
		var checkAuthor= this.getAuthorFilterDescriptor();
		var checkContext= this.getContextFilterDescriptor(this.currentValidContexts);


		var checkFilters= function(review){
			return parameterTracker.searchScope== REVIEW_SEARCH_PARAMATERS.ALL_REVIEWS_FILTER?
							checkStatus(review) && checkContext(review):
								checkStatus(review)&& (checkRole(review)|| checkAuthor(review)) && checkContext(review);
		};


		for(var index=0; index<reviews.length; index++){
				var review= reviews[index];
				if(checkFilters(review)){
					list.push(review);
				}
		}
		return list;
    }

});
/*************************************************************************************************************************************************/

/****************************************************************Filter Dialog*********************************************************************/
dojo.declare("rmps.app.ui.internal.editors.views.review.ui.sections._FilterDialog", rmps.ui.DialogForm, {

	templateString: dojo.cache("dm.review.app", "ui/internal/editors/templates/ReviewViewerEditorFilter.html"),
	widgetsInTemplate: true,

	messages: null,
	participationCheckList: null,
	statusCheckList: null,
	contextCheckList: null,

	participationFiltersNumber: -1,
	statusFiltersNumber: 0,
	contextFiltersNumber: 0,
	
	participationWarning: false,
	statusWarning: false,

	"class": "rmps-app-editors-review-viewer",

	postCreate: function() {
		this.inherited(arguments);

		this.warningsMap= {};
		this.warningsMap[this._reviewStatusWarning.id]= messages.statusWarning;
		this.warningsMap[this._reviewParticipationWarning.id]= messages.participationWarning;

		this.participationCheckList= [this.authorCheck, this.approverCheck, this.reviewerCheck, this.subscriberCheck];
		this.allProjectsCheckList= [this._allProjectReviewsRadioButton];
		this.statusCheckList= [this.activeCheck, this.finalizedCheck];
		this.contextCheckList= [this.contextCheck];

		if(!this._boundCurrentFilters()){
			this._setDefaultValue();
		}else{
			this._updateStatusWarning();
		}
		this._addListeners();
	},
	

	_addListeners: function() {
		dojo.forEach(this.statusCheckList, dojo.hitch(this, function(item) {
			   this.connect(item, 'onChange', this._onStatusFilterChange);
			}));
		
		dojo.forEach(this.participationCheckList, dojo.hitch(this, function(item) {
			   this.connect(item, 'onChange', this._onParticipationFilterChange);
			}));

		dojo.forEach(this.allProjectsCheckList, dojo.hitch(this, function(item) {
			   this.connect(item, 'onChange', this._onAllReviewClick);
			}));

		dojo.forEach(this.contextCheckList, dojo.hitch(this, function(item) {
			   this.connect(item, 'onChange', this._onContextFilterChange);
			}));

		this.connect(this._okButtonNode, 'onClick', this._onOk);			
		this.connect(this._cancelButtonNode, 'onClick', this._onCancel);			
	},
	


	_boundCurrentFilters: function(){
		var index, current, value, statusFilter= false;
		for(index=0; index< this.statusCheckList.length; index++){
			// need to decrease to balance the listener called automatically
			this.statusFiltersNumber--;
			current= this.statusCheckList[index];
			value= parameterTracker[current.value];
			if(value== "true"){
				current.set('checked',true);
				statusFilter= true;
			} else {
				current.set('checked',false);				
			}
		}

		var roleFilter= false;
		for(index=0; index< this.participationCheckList.length; index++){
			// need to decrease to balance the listener called automatically
			this.participationFiltersNumber--;
			current= this.participationCheckList[index];
			value= parameterTracker[current.value];
			if(value== "true"){
				current.set('checked',true);
				roleFilter= true;
			} else {
				current.set('checked',false);				
			}
		}

		var allFilter= false;
		// need to decrease to balance the listener called automatically
		this.participationFiltersNumber--;
		if(parameterTracker.searchScope== REVIEW_SEARCH_PARAMATERS.ALL_REVIEWS_FILTER){
			for(index=0; index< this.allProjectsCheckList.length; index++){
				current= this.allProjectsCheckList[index];
				current.set('checked',true);
			}
			allFilter= true;
			this._allProjectReviewsRadioButton.set('checked',true);
		} else {
			this._allProjectReviewsRadioButton.set('checked',false);			
		}
		participationFilter= roleFilter || allFilter;

		for(index=0; index< this.contextCheckList.length; index++){
			// need to decrease to balance the listener called automatically
			this.contextFiltersNumber--;
			current= this.contextCheckList[index];
			value= parameterTracker[current.value];
			if(value== "true"){
				current.set('checked',true);
			} else {
				current.set('checked',false);				
			}
		}		
		return statusFilter && participationFilter;
	},

	getDialogArguments: function(){
		return {
			contentNode: this.containerNode,
			primaryTitle: messages.filterReviews,
			destroyContents: true,
			width: "380px"
		};
	},

	_setDefaultValue: function(){
		this._setDefaultParticipationEnablementValues();
		this._setDefaultStatusEnablementValues();
		this._setDefaultContextEnablementValues();
	},

	_setDefaultParticipationEnablementValues: function(){
		var index;
		for(index=0; index< this.participationCheckList.length; index++){
			this.participationCheckList[index].setChecked(true);
		}
		for(index=0; index< this.allProjectsCheckList.length; index++){
			this.allProjectsCheckList[index].setChecked(false);
		}
	},

	_setDefaultStatusEnablementValues: function(){
		this.activeCheck.setChecked(true);
		this.finalizedCheck.setChecked(false);
	},

	_setDefaultContextEnablementValues: function(){
		this.contextCheck.setChecked(false);
	},

	_onAllReviewClick: function(isChecked){
		this.participationFiltersNumber++;
		// Previously selected participation filters are not unchecked, only disabled
		this._updateEnabledList(this.participationCheckList, isChecked);
		if(!isChecked){
			// When we uncheck all reviews, warning status is based on what was previously checked
			this._updateParticipationWarning();
		}else{
			// When we check all reviews, we need to remove the participation warning
			this._removeWarningFor(this._reviewParticipationWarning);
			this.participationWarning = false;
			this.updateOkButtonEnablement();
		}

	},

	_onReset: function(){
		this._setDefaultValue();
	},

	_updateEnabledList: function(list, value){
		for(var index=0; index< list.length; index++){
			list[index].set('disabled',value);
		}
	},

	_onParticipationFilterChange: function(value){
		this.participationFiltersNumber++;
		this._updateParticipationWarning();
	},

	_onStatusFilterChange: function(value){
		this.statusFiltersNumber++;
		this._updateStatusWarning();
	},

	_onContextFilterChange: function(value){
		this.contextFiltersNumber++;
		this.updateOkButtonEnablement();
	},

	_updateStatusWarning: function(){
		this._updateWarning(this.statusCheckList, this._reviewStatusWarning, "statusWarning");
	},

	_updateParticipationWarning: function(){
		this._updateWarning(this.participationCheckList, this._reviewParticipationWarning, "participationWarning");
	},

	_updateWarning: function(list, attachPoint, warning){
		for(var index=0; index< list.length; index++){
			if(list[index].checked){
				this._removeWarningFor(attachPoint);
				this[warning] = false;
				this.updateOkButtonEnablement();
				return true;
			}
		}
		this._showWarningFor(attachPoint);
		this[warning] = true;
		this._okButtonNode.set('disabled',true);
		return false;
	},

	updateOkButtonEnablement: function(){		
		// OK must be disabled if there is a warning
		// or when the initial filter selections are unmodified
		var okDisabled = (this.participationWarning || this.statusWarning) ||
				(this.statusFiltersNumber==0 && this.participationFiltersNumber==0 && this.contextFiltersNumber == 0);
		this._okButtonNode.set('disabled', okDisabled);
	},

	_showWarningFor: function(attachPoint){
		attachPoint.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
				summary: this.warningsMap[attachPoint.id],
				closable: false
		});
	},

	_removeWarningFor: function(attachPoint){
		attachPoint.hideMessage();
	},

	getOkParamaters: function(){
		var index, current, args= {};

		if(this._allProjectReviewsRadioButton.checked){
			args[REVIEW_SEARCH_PARAMATERS.SEARCH_SCOPE]= REVIEW_SEARCH_PARAMATERS.ALL_REVIEWS_FILTER;
		}else{
			args.searchScope= REVIEW_SEARCH_PARAMATERS.MY_REVIEWS_FILTER;
		}

		for(index=0; index< this.participationCheckList.length; index++){
			current= this.participationCheckList[index];
			args[current.value]= current.checked.toString();
		}

		for(index=0; index< this.statusCheckList.length; index++){
			current= this.statusCheckList[index];
			args[current.value]= current.checked.toString();
		}
		
		for(index=0; index< this.contextCheckList.length; index++){
			current= this.contextCheckList[index];
			args[current.value]= current.checked.toString();
		}		

		return args;
	}
});
/*************************************************************************************************************************************************/

})();
