/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.SearchQueriesListEditor");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");

dojo.require("rmps.core.error");
dojo.require("rmps.core.querymodel.QueryModelClient");
dojo.require("rmps.core.query.QueryClient");
dojo.require("rmps.core.query.SearchQuery");
dojo.require("rmps.app.core.util");
dojo.require("rmps.app.core.internal.util.QueryUtil");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("rmps.app.ui.internal.editors.CopyQueryDialog");

dojo.require("rmps.ui.util.ToolbarFocusUtil");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.toolbar.Button");

dojo.requireLocalization("rmps.app.ui.internal.editors", "SearchQueriesListEditorMessages");
dojo.requireLocalization("rmps.app.ui.internal.editors", "CopyQueryDialogMessages");

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "SearchQueriesListEditorMessages");
var copyQueryMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "CopyQueryDialogMessages");
var QueryModelClient = rmps.core.querymodel.QueryModelClient;
var QueryClient = rmps.core.query.QueryClient;
var QueryUtil = rmps.app.core.internal.util.QueryUtil;
var DesignUtil = rmps.app.ui.internal.pages.design.DesignUtil;
var ToolbarFocusUtil = rmps.ui.util.ToolbarFocusUtil;

var UA_KEY = "com.ibm.xtools.rmps.web.app.page_QueriesList001";

dojo.declare("rmps.app.ui.internal.editors.SearchQueriesListEditor", rmps.app.ui.internal.editors.BaseEditor, {

	"class": "rmps-search-query-list",
	uaKey: UA_KEY,

	//init properties
	isReadOnly: true,
	
	//private properties
	_deferred: null,
	
	postCreate: function() {
		this.inherited(arguments);
		this.refresh();
	},
	
	addButtons: function(toolbar) {
		var button = new jazz.ui.toolbar.Button({
			href: DesignUtil.getQueryUri(null),
			label: messages.createSearchQueryText,
			imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/query.gif").toString()
		});
		
		toolbar.addWidget(button);		
		toolbar.addSeparator(2);		
		this.inherited(arguments);
	},

	refresh: function() {
		if (this._deferred) {
			return;
		}

		dojo.forEach(dijit.findWidgets(this.containerNode), function(widget) {
			if (widget.destroyRecursive) {
				widget.destroyRecursive();
			}
		});
		
		this.showLoading();

		var projectId = this.getEditorInput().projectId;
		this._deferred = QueryModelClient.getQueryModels(projectId);
				
		this._deferred.addBoth(this, function() {
			delete this._deferred;
			this.hideMessage();
		});
		
		this._deferred.addCallback(this, function(queries) {
		
			var table = new jazz.ui.StyledTable({
				trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
				columns: [
					{
						label: messages.name
					},
					{
						label: messages.description,
						className: "description"
					},
					{
						label: messages.actions,
						className: "actions"
					}
				],
				showHover: true
			});

			table.containerNode = table.tbody;
			this.containerNode.appendChild(table.domNode);

			for (var i = 0; i<queries.length; i++) {
				var row = new rmps.app.ui.internal.editors._SearchQueryRow({
					query: queries[i],
					editor: this,
					projectId: projectId
				});
				table.tbody.appendChild(row.domNode);
				row.startup();
			}			
		});
		
		this._deferred.addErrback(this, function(error) {
			this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: messages.error_loading,
					closable: true
				});
		});
	},

	activate: function() {
		this.refresh();

		this.inherited(arguments);
	}
});

dojo.declare("rmps.app.ui.internal.editors._SearchQueryRow", [dijit._Widget], {

	query: null,
	editor: null,
	projectId: null,
	
	_deferred: null,
	_deleteDeferred: null,
	_stateImg: null,
	_stateCell: null,
	_popup: null,
	_popupContent: null,
	_actionBar: null,
	_editAction: null,
	_deleteAction: null,
	_executeAction: null,
	
	buildRendering: function() {
		this.domNode = this.containerNode = dojo.create("tr");
	},
	
	postCreate: function() {
		//query name
		var cell = dojo.create("td", null, this.domNode);
		var nameLink = dojo.create("a", { href : DesignUtil.getQueryUri(this.query.uri), lavel : this.query.getName() }, cell);
		nameLink.appendChild(dojo.doc.createTextNode(this.query.getName()));
		
		this._popup = new jazz.ui.HoverPopup({
			around: nameLink,
			onOpen: dojo.hitch(this, this._createPopupContents),
			maxWidth: 800,
			maxHeight: dojo.isIE ? 570 : 550
		});
		
		//description
		cell = dojo.create("td", null, this.domNode);
		cell.appendChild(dojo.doc.createTextNode(
			this.query.getDescription()
		));
		
		//actions, if applicable
		var actionsTd = dojo.create("td", null, this.domNode);
		this._actionBar = new jazz.ui.SimpleToolbar();
		actionsTd.appendChild(this._actionBar.domNode);
		this._actionBar.startup();
		this._updateActions();	
		
		ToolbarFocusUtil.addToolbarFocusListeners( { 
            actionToolbar: this._actionBar, 
            containerNode: this.containerNode, 
            actionClassName: ".jazz-ui-SimpleToolbar-button"
		}); 	
	},
	
	_createPopupContents: function() {
		if (this._popupContent) {
			this._popupContent.destroyRecursive();
		}
		this._popupContent = new rmps.app.ui.internal.editors._SearchQueryPopupContent({query:this.query});
		this._popup.content = this._popupContent.domNode;
	},
	
	destroy: function() {
		this.inherited(arguments);
		
		if (this._deferred) {
			this._deferred.cancel();
			delete this._deferred;
		}
		
		if (this._deleteDeferred) {
			this._deleteDeferred.cancel();
			delete this._deleteDeferred;
		}
		
		if (this._popupContent) {
			this._popupContent.destroyRecursive();
			delete this._popupContent;
		}
		
		if (this._popup) {
			this._popup.destroyRecursive();
			delete this._popup;
		}
	},
	
	_updateActions: function() {
		if (!this._editAction) {
			this._editAction = this._actionBar.addButton(this, this._editQuery, {
				label: messages.editSearchQuery,
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/edit.gif")
			});
		}
		
		if (!this._copyAction) {
			this._copyAction = this._actionBar.addButton(this, this._copyQuery, {
				label: messages.copySearchQuery,
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/copy.gif")
			});
		}

		
		if (!this._deleteAction) {		
			this._deleteAction = this._actionBar.addButton(this, this._deleteQuery, {
				label: messages.deleteSearchQuery,
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/remove.gif")
			});
		}
		
		if (!this._executeAction) {
			this._executeAction = this._actionBar.addButton(this, this._executeQuery, {
				label: messages.executeSearchQuery,
				// FIXME: Execute icon!
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/breadcrumb_expand.gif")
			});
		}
	},
	
	_editQuery: function() {
		window.location.href = DesignUtil.getQueryUri(this.query.uri);
	},
	
	_deleteQuery: function() {
		var msg = dojo.string.substitute(messages.deleteConfirmation, [this.query.getName()]);
		if (!confirm(msg)) {
			return;
		}
		this._actionBar.disableButton(this._editAction);
		this._actionBar.disableButton(this._deleteAction);
		this._actionBar.disableButton(this._executeAction);
		
		this._deleteDeferred = rmps.core.querymodel.QueryModelClient.deleteQueryModel(this.query);
		this._deleteDeferred.addCallback(this, function(args) {
			delete this._deleteDeferred;
			this.editor.hideMessage();
			this.destroyRecursive();
		});

		this._deleteDeferred.addErrback(this, function(error) {
			this.editor.hideMessage();
			delete this._deleteDeferred;
			var message,
			    details;
			if (error.status === 403) {
				message = messages.error_request_permission;
			} else {
				message = messages.error_request_general;
				details = [{message: dojo.string.substitute(messages['error_request_general.explanation'], [error.status]) + "<br>" + messages['error_request_general.useraction'] }];
			}
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: message,
				details: details,
				closable: true
			});
		});		
		
		this.editor.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: messages.deletingSearchQuery
		});
	},
	
	_executeQuery: function() {
		var query = new rmps.core.query.SearchQuery({
			queryUri: this.query.uri,
			projectId: this.projectId,
			limit: 200,
			pageSize: 50,
			name: this.query.getName(),
			scope: QueryUtil.SCOPE.ALL.value,
			baseline: jazz.app.context.get().baseline
		});
		var timestamp = new Date().getTime();
		window.location.href = rmps.app.core.util.getSearchUri(query.toQueryString()) + "&ts=" + timestamp;
	},
	
	_copyQuery: function() {
		this._dialog = new rmps.app.ui.internal.editors.CopyQueryDialog({
			queryLabel: this.query.label,
			onOK: dojo.hitch(this, function() {
				QueryModelClient.createQueryCopy(
						this.query.uri,
						this._dialog.getProjectId(),
						this._dialog.getNewTitle()).then(dojo.hitch(this, this._onQueryCopied));
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: copyQueryMessages.copyingQuery
				});
			}),
			editor: this.editor
		});
		this._dialog.show();
	},
	
	_onQueryCopied: function(newQueryUri) {
		window.location.href = this._dialog.getEditQueryUri(newQueryUri); 
	}

});

dojo.declare("rmps.app.ui.internal.editors._SearchQueryPopupContent", [dijit._Widget, dijit._Templated], {

	"class" : "rmps-search-query-status-popup",
	messages: messages,
	query: null,
	templateString: '<table>' +
						'<tr><th>${messages.details}</th></tr>' +
						'<tr><td>${messages.name}:</td><td dojoAttachPoint="_nameText"/></tr>' +
						'<tr><td>${messages.description}:</td><td dojoAttachPoint="_descriptionText"/></tr>' +						
					'</table>',
	
	postCreate: function() {
		this._nameText.appendChild(dojo.doc.createTextNode(this.query.getName()));
		this._descriptionText.appendChild(dojo.doc.createTextNode(this.query.getDescription()));
	}
});
})();