/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.app.ui.internal.editors.SearchQueryEditor");

dojo.require("dijit.InlineEditBox");

dojo.require("jazz.ui.Breadcrumbs");
dojo.require("jazz.ui.toolbar.Button");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");

dojo.require("rmps.core.querymodel.QueryModel");
dojo.require("rmps.core.querymodel.QueryModelClient");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.ui.editor.PageHelper");
dojo.require("rmps.core.error");

dojo.require("rmps.ui.queryeditor.QueryEditorWidget");

dojo.require("rmps.app.core.internal.util.QueryUtil");
dojo.require("rmps.app.ui.internal.editors.CopyQueryDialog");


dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.ui.internal.editors", "SearchQueryEditorMessages");
dojo.requireLocalization("rmps.app.ui.internal.editors", "CopyQueryDialogMessages");

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "SearchQueryEditorMessages");
var copyQueryMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "CopyQueryDialogMessages");

var QueryModelClient = rmps.core.querymodel.QueryModelClient;
var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;
var DesignUtil = rmps.app.ui.internal.pages.design.DesignUtil;
var QueryUtil = rmps.app.core.internal.util.QueryUtil;

var UA_KEY = "com.ibm.xtools.rmps.web.app.page_Query001";

(function(){
dojo.declare("rmps.app.ui.internal.editors.SearchQueryEditor", rmps.app.ui.internal.editors.BaseEditor, {
	
	"class": "rmps-search-query",
	
	uaKey: UA_KEY,
	
	_queryEditorWidget: null,
	_editableDescription: null,
	_runButton: null,
	_copyButton: null,
	
	postCreate: function() {
		this.inherited(arguments);

		var q = this.editorInput.getSearchQuery();
		
		// Create query widget
		this._queryEditorWidget = new rmps.ui.queryeditor.QueryEditorWidget({
			inputData: {},
			queryUri: q.uri,
			onDirty: dojo.hitch(this, this.onDirty),
			hideGrouping: true
		});
		this.containerNode.appendChild(this._queryEditorWidget.domNode);
		this.contentWidget = this._queryEditorWidget;
		
		//create navigation breadcrumbs back to the baseline status page
		this._breadCrumbs= new jazz.ui.Breadcrumbs({
			segments: [{
				label: messages.statusLinkLabel,
				href: DesignUtil.getQueriesUri()
			}]
		});

		dojo.place(this._breadCrumbs.domNode, this.headerContainerNode, "first");

		// Description UI
		// TODO: this should be extracted into an internal _Widget
		var desc = q.getDescription() || "";
		var div = dojo.create("div", {className:"description-div"});
		var table = dojo.create("table", null, div);
		var tbody = dojo.create("tbody", null, table);
		var tr = dojo.create("tr", null, tbody);
		var td1 = dojo.create("td", null, tr);
		var td2 = dojo.create("td",{style:'width: 95%'}, tr);
		var span = dojo.create("span", {
			innerHTML: messages.description + ":&nbsp",
			className: "description-label"}, td1);
		var textboxdiv = dojo.create("div", null, td2);
        this._editableDescription = new dijit.InlineEditBox({
			title: messages.description,
			noValueIndicator: "&lt;" + messages.description_no_value + "&gt;",
			autoSave: true, 
			editorParams: {maxLength: "200"},
			value: desc,
			onChange: dojo.hitch(this, this.onDescriptionChange)
        }, textboxdiv);
        dojo.addClass(this._editableDescription.domNode, "description-text");
		if (desc.length === 0) {
			dojo.addClass(this._editableDescription.domNode, "description-hint");
        }
        this._headerContent.appendChild(div);
        this._editableDescription.domNode.style.paddingTop = "2px";
		this._editableDescription.domNode.style.paddingBottom = "2px";
	},
	
	destroy: function() {
		if (this._queryEditorWidget) {
			this._queryEditorWidget.destroy();
			delete this._queryEditorWidget;
		}
		
		if (this._editableDescription) {
			this._editableDescription.destroy();
			delete this._editableDescription;
		}
			
		this.inherited(arguments);
	},
	
	addButtons: function (toolbar) {
		this.inherited(arguments);
		
		this._addRunButton(toolbar);
		this._addCopyButton(toolbar);
	},
	
	_addRunButton: function (toolbar) {
		this._runButton = toolbar.addWidget(new rmps.ui.SimpleButton({
			label: messages.run,
			onClick: dojo.hitch(this, this.run)
		}));
		this._runButton.setEnabled(this.getEditorInput().getResourceUri() !== null);
	},
	
	_addCopyButton: function(toolbar) {
		this._copyButton = toolbar.addButton(this, this.copy, {
			label: messages.createACopy,
			imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/copy.gif"),
			index: 1
		});
		if (this.getEditorInput().getResourceUri()) {
			toolbar.enableButton(this._copyButton);
		} else {
			toolbar.disableButton(this._copyButton);
		}
		
	},
	
	run: function () {
		var searchQuery = this.editorInput.getSearchQuery();
		var query = new rmps.core.query.SearchQuery({
			queryUri: searchQuery.uri,
			projectId: this.editorInput.projectId,
			limit: 200,
			pageSize: 50,
			name: searchQuery.getName(),
			scope: QueryUtil.SCOPE.ALL.value,
			baseline: jazz.app.context.get().baseline
		});
		var timestamp = new Date().getTime();
		window.location.href = rmps.app.core.util.getSearchUri(query.toQueryString()) + "&ts=" + timestamp;
	},
	
	copy: function(){
		var searchQuery = this.editorInput.getSearchQuery();
		this._dialog = new rmps.app.ui.internal.editors.CopyQueryDialog({
			queryLabel: searchQuery.getName(),
			onOK: dojo.hitch(this, function() {
				QueryModelClient.createQueryCopy(
						searchQuery.uri,
						this._dialog.getProjectId(),
						this._dialog.getNewTitle()).then(dojo.hitch(this,function(newQueryUri) {
							window.location.href = this._dialog.getEditQueryUri(newQueryUri);
				}));
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: copyQueryMessages.copyingQuery
				});
			})
		});
		this._dialog.show();

	},
	
	dirtyStateChanged: function(isDirty) {
		this.inherited(arguments);
		
		this._runButton.setEnabled(!isDirty);
		if (isDirty) {
			this.toolbar.disableButton(this._copyButton);
		} else {
			this.toolbar.enableButton(this._copyButton);
		}
	},
	
	onTitleChange: function() {
		this._queryEditorWidget.onMarkDirty("title");
		var editorInput= this.getEditorInput();
		editorInput.getSearchQuery().setName(this.getTitleValue());
		rmps.ui.editor.PageHelper.getInstance(DesignUtil.PAGE_ID).getEditorTracker().onUpdate(editorInput);
	},
	
	onDescriptionChange: function(text) {
		this._queryEditorWidget.onMarkDirty("description");
		
		if (text && text.length > 0) {
			dojo.removeClass(this._editableDescription.domNode, "description-hint");				
		} else {
			dojo.addClass(this._editableDescription.domNode, "description-hint");
		}
	},
	
	onDirty: function() {
		// TODO: Support removeDirty
		this.addDirty("query");
	},
	
	// Overrides
	isEditableTitle: function(){
		return true;
	},

	getEditableTitleNodeDetails: function(){
		return messages.newSearchQueryTitle;
	},
	
	doSave: function(taskMonitor) {
		this.inherited(arguments);
		
		var maintask = taskMonitor.beginTask(messages.saveSearchQuery, 1);
		
		// Save query
		var desc = this._editableDescription ? this._editableDescription.get('value') : "";
		var searchQuery = this.editorInput.getSearchQuery();
		searchQuery.setName(this.getTitleValue());
		searchQuery.setDescription(desc);

		var def;
		if (this._queryEditorWidget.isDirty()) {
			// Query is dirty, so create a task for it and save
			def = this._queryEditorWidget.saveQueryData(false, searchQuery);
		} else {
			def = new dojo.Deferred();
			// Just return the associated query model
			def.callback(searchQuery);
		}
		
		// TODO: Let query widget take a queryModel object instead of copying data.
		def.addCallback(function(queryModel) {
			dojo.mixin(searchQuery, queryModel);
		});
		maintask.handleDeferredWork(def, 1);
	},


	onSaveComplete: function() {
		var searchQuery = this.editorInput.getSearchQuery();
		if (this.editorInput.getResourceUri() == null && searchQuery.uri != "") {
			var pageHelper = rmps.ui.editor.PageHelper.getInstance(DesignUtil.PAGE_ID);
			if (pageHelper) {
				pageHelper.closeEditor(this, true, true);
			}
			window.location.href = DesignUtil.getQueryUri(searchQuery.uri);
		} else {
			this.inherited(arguments);			
		}
	},
	
	getItemInformation: function(items) {
		return ResourceInfoClient.getResourceInfo(items, ['type', 'typeName']);
	},
	
	validate: function(/*Array*/errors) {
		// Check if query name is specified 
		if (this.getTitleValue().length === 0) {
			errors.push(rmps.core.error.create(messages.emptySearchQueryTitleError, messages.emptySearchQueryTitleError, null));
			return;
		}

		var def = new dojo.Deferred();
				
		// Validate query
		var defVal = this._queryEditorWidget.validate();
		defVal.addCallback(function(valErrors) {
			var errlist = [];	
			if (valErrors.length > 0) {
				for (var i = 0; i < valErrors.length; i++) {
					var errorMessage = messages.validationError + ". " + valErrors[i];
					errlist.push(rmps.core.error.create(errorMessage, errorMessage, null));
				}
			} 
			def.callback(errlist);
		});
		defVal.addErrback(def.errback);
		
		// BaseEditor expects a list of defs.
		return [def];
	}
});	

}());