/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

//TODO: CSS File
(function() {
	
dojo.provide("rmps.app.ui.internal.editors.StreamEditor");

dojo.require("rmps.core.streams.StreamClient"); 
dojo.require("rmps.core.baseline.BaselineClient"); 
dojo.require("rmps.core.util.ContextUtils");
dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.core.error");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.Breadcrumbs");
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.ui.SimpleToolbar");
dojo.provide("rmps.core.util.IRmpsConstants");

dojo.require("dijit.form.Select");

dojo.requireLocalization("rmps.app.ui.internal.editors", "StreamEditorMessages");

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "StreamEditorMessages");
var ContextUtils = rmps.core.util.ContextUtils;
var BaselineClient = rmps.core.baseline.BaselineClient;
var IRmpsConstants= rmps.core.util.IRmpsConstants;
var SNAPSHOT_URI_PARAM = "snapshotUri";

var UA_KEY = "com.ibm.xtools.web.app.page_Workspace001";

dojo.declare("rmps.app.ui.internal.editors.StreamEditor", rmps.app.ui.internal.editors.BaseEditor, {

	"class": "rmps-request-stream",
	
	uaKey: UA_KEY,
	
	//init properties
	isReadOnly: true,
	isRefreshable: false,
	
	//private properties
	_formContent: null,
	
	_resetTitle: false,
	
	postCreate: function() {
		this.inherited(arguments);
		
		//create navigation bread crumbs back to the baseline status page
		this._breadCrumbs = this._createBreadCrumbs();
		dojo.place(this._breadCrumbs.domNode, this.headerContainerNode, "first");
	},
	
	_createBreadCrumbs: function() {
		if (this.getSnapshotUri() != null) {
			return new jazz.ui.Breadcrumbs({
				segments: [{
					label: messages.snapshotsLinkLabel,
					href: IRmpsConstants.ADMIN_BASELINE_STATUS
				}]
			});
		} else {
			return new jazz.ui.Breadcrumbs({
				segments: [{
					label: messages.workspacesLinkLabel,
					href: IRmpsConstants.ADMIN_STREAM_VIEWER
				}]
			});
		}
	},
	
	isEditableTitle: function() {
		return true;
	},
	
	getEditableTitleNodeDetails: function() {
		return messages.defaultTitleForNewStream;
	},
	
	getSnapshotUri: function() {
		return this.getEditorInput().getSnapshotUri();
	},
	
	updatebreadCrumbs: function() {
		if (!this._breadCrumbs) {
			return;
		}
		var parent = this._breadCrumbs.domNode.parentNode;
		if (!this._breadCrumbs.domNode.parentNode) {
			return;
		}
		var newNode = this._createBreadCrumbs();
		parent.replaceChild(newNode.domNode, this._breadCrumbs.domNode);
		this._breadCrumbs = newNode;
	},
	
	setTitle: function() {
		this.updatebreadCrumbs();
		if (this._resetTitle) {
			this._resetTitle = false;
			this.setTitleValue("");
		}
	},
	
	addButtons: function(toolbar) {
		var _createButton = new rmps.ui.SimpleButton({
			label: messages.createButton,
			onClick: dojo.hitch(this, this._newStream)
		});
		toolbar.addWidget(_createButton);
	}, // end addButtons

	_newStream: function(){
		var streamName = this.getTitleValue();
		var streamDescription = this._formContent.getDescription();
		var projectName = jazz.app.context.get().name;
		var parentURI = this._formContent.getSelectedParent();
		
		// Stream name is required
		if(!streamName){
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.missingName,
				closable: true
			});	
			return;
		} // else
		
		// Parent URI is required
		if(!parentURI){
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.missingParent,
				closable: true
			});	
			return;
		}
		
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: messages.checking_preconditions,
			closable: true
		});
		
		var projectId = jazz.app.context.get().itemId;
		var streamViewerUri = IRmpsConstants.ADMIN_STREAM_VIEWER;

		var createStreamDeferred = rmps.core.streams.StreamClient.createStream(projectId, streamName, parentURI, streamDescription);				
				
		createStreamDeferred.addCallback(this, function(newStreamUri){
			var summaryText = dojo.string.substitute(messages.inprogress, [IRmpsConstants.ADMIN_STREAM_VIEWER]);

			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.OK,
				summary: summaryText,
				closable: true
			});	

			// we don't want user to reuse the name. So, reset it
			// after the creation is completed successfully.
			this._resetTitle = true;
		});
		
		createStreamDeferred.addErrback(this, function(error){
			var message,
			    details;
			if(error.status == 403){
				message = messages.errorPermissionDenied;
			} else if (error.status == 409) {
				message = dojo.string.substitute(messages.errorNameExists, [streamName]);
			} else {
				message = messages.createError;
				details = [{message: dojo.string.substitute(messages['createError.explanation'], [error.status]) + "<br>" + messages['createError.useraction'] }];
			} // end if
			
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: message,
				details: details,
				closable: true
			}); // end showMessage
		}); // end createStreamDeferred
	
	}, // end newStream
	
	createContentWidget: function(){
		this._formContent = new rmps.app.ui.internal.editors.StreamEditor._Content({snapshotUri: this.getSnapshotUri()});
		return this._formContent;
	} // end createContentWidget
	
	
}); // end declare

dojo.declare("rmps.app.ui.internal.editors.StreamEditor._Content", [dijit._Widget, dijit._Templated], {
	messages: messages,
	widgetsInTemplate: true,
	templateString: '<table class="paramTable" style="width: 50%;">' + 
						'<tr>' +
							'<td class="label">${messages.parentContext}</td>' +
							'<td dojoAttachPoint="_snapshotLabel" class = "parentSelector">' + 
								'<select dojoAttachPoint="_parentContextPicker" style="width:100%"></select>' +
							'</td>' + 
						'</tr>' +
						'<tr>' +
							'<td class="label">${messages.projectDescription}</td>' +
							'<td><textarea dojoAttachPoint="_descriptionField" rows="5" style="width:100%"></textarea></td>' +
						'</tr>' +
					'</table>',

	_snapshotUri: null,
	
	constructor: function(params) {
		if (params) {
			this._snapshotUri = params.snapshotUri;
		}
	},
	postCreate: function(){
		// Create the list of parent snapshot options...
		var defaultOption = dojo.create("option", {value: ""}, this._parentContextPicker);
		if (!this._snapshotUri) {
			defaultOption.appendChild(dojo.doc.createTextNode(messages.defaultSnapshotPickerMessage));
		}
		var deferred = BaselineClient.getBaselines();
		deferred.addCallback(this, function(snapshots){
			var snapshot;
			var option;
			var contextUri = this._snapshotUri;
			if (contextUri) {
				this._parentContextPicker.disabled = true;
			} else {
				contextUri = ContextUtils.getCurrentContextURI();
			}

			// select the specified snapshot if it is present in the uri
			for(index in snapshots){
				snapshot = snapshots[index];
				option = dojo.create("option", {
					value: snapshot.uri,
					selected: (snapshot.uri == contextUri)
				});
				option.appendChild(dojo.doc.createTextNode(snapshot.name));
				this._parentContextPicker.appendChild(option);
			}
		});
	},
	
	startup: function(){
		this.inherited(arguments);
	},
	
	getDescription: function(){
		return this._descriptionField.value;
	},
	
	getSelectedParent: function(){
		return this._parentContextPicker.value;
	}

});

})(); // end of the earth