/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

//TODO: CSS File
(function() {

	dojo.provide("rmps.app.ui.internal.editors.StreamViewer");

	dojo.require("rmps.app.ui.internal.editors.BaseEditor");

	dojo.require("rmps.core.error");

	dojo.require("jazz.ui.StyledTable");
	dojo.require("jazz.ui.HoverPopup");
	dojo.require("jazz.ui.SimpleToolbar");
	dojo.require("rmps.core.streams.StreamClient"); 
	dojo.require("rmps.ui.SimpleButton");
	dojo.require("jazz.ui.toolbar.Button");

	dojo.require("rmps.core.util.IRmpsConstants");
	dojo.require("rmps.core.streams.Stream");
	dojo.require("rmps.core.util.ContextUtils");
	dojo.require("rmps.app.ui.internal.editors.RequestBaselineEditor");
	dojo.requireLocalization("rmps.app.ui.internal.editors", "StreamViewerMessages");

	var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "StreamViewerMessages");
	var IRmpsConstants= rmps.core.util.IRmpsConstants;
	var COMPARE_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "compareWithPrevious");
	
	var UA_KEY = "com.ibm.xtools.web.app.page_WorkspacesList001";
	
	dojo.declare("rmps.app.ui.internal.editors.StreamViewer", rmps.app.ui.internal.editors.BaseEditor, {

		"class": "rmps-stream-viewer",
		
		uaKey: UA_KEY,

		//init properties
		isReadOnly: true,
		isRefreshable: true,

		//private properties
		_deferred: null,
		//_treeGrid: null,

		postCreate: function() {
		this.inherited(arguments);
		//this.addToolbarButtons();
		this.refresh();
	},

	refresh: function(){
		// Don't do anything if a refresh is alread in the works
		if(this._deferred){
			return;
		} // end if

		dojo.forEach(dijit.findWidgets(this.containerNode), function(widget) {
			if (widget.destroyRecursive) {
				widget.destroyRecursive();
			}

			if (this._updateTimer) {
				clearTimeout(this._updateTimer);
				delete this._updateTimer;
			}
		}); // end forEach

		// If no project is passed into getStreams it will select the current one for you
		this._deferred = rmps.core.streams.StreamClient.getStreams();

		// clean up, "addBoth" is akin to finally in try/catch
		this._deferred.addBoth(this, function(){
			this._deferred = null;
			this.hideMessage();
		});

		this._deferred.addCallback(this, function(streams){

			// Build the table
			var table = new jazz.ui.StyledTable({
				trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
				"class" : "streamTable",
				columns: [
				          {
				        	  label: messages.name,
				        	  className: "streamName"
				          },
				          {
				        	  label: messages.stateLabel,
				        	  className: "stateLabel"
				          },
				          {
				        	  label: messages.actions,
				        	  className: "actionBar"
				          }
				          ],
				          showHover: true
			});// end table

			table.containerNode = table.tbody;
			this.containerNode.appendChild(table.domNode);

			// Now add the rows
			for(var i = 0; i < streams.length; i++){
				var row = new rmps.app.ui.internal.editors._StreamRow({stream:streams[i], editor:this});
				table.tbody.appendChild(row.domNode);
				row.startup();
			}// end for

		}); // end addCallback()

		// Error handling
		this._deferred.addErrback(this, function(error){
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPE.ERROR,
				summary: messages.genericError,
				details: [{message: messages['genericError.useraction'] + "<br>" + messages['genericError.useraction'] }],
				closable: true
			}); // end showMessage()
		}); // end addErrback()
	}, // end refresh()

	addButtons : function(toolbar) {
	
		toolbar.addWidget(new jazz.ui.toolbar.Button({
			href : IRmpsConstants.ADMIN_REQUEST_STREAM,
			label : messages.newStream
		})); 
		toolbar.addSeparator(2);
		this.inherited(arguments);
	} // end addButtons

	}); // end baseEditor

	dojo.declare("rmps.app.ui.internal.editors._StreamRow", [dijit._Widget], {

		streamUri: null,
		stream: null,
		editor: null,
		_actionBar:null,

		_deleteBtn:null,
		_newSnapshotBtn: null,
		_compareSnapshotBtn: null,
		_refreshBtn: null,
		_editStreamBtn: null,

		_popup: null,
		_popupContent: null,
		_popupImmutable: null,

		_stateImage: null,
		_stateText: null,
		_updateTimer: null,

		"class" : "streamRow",

		buildRendering: function(){
		this.domNode = this.containerNode = dojo.create("tr");
	},

	postCreate: function(){
		// Name
		var nameCell = dojo.create("td", null, this.domNode);
		var iconCell = dojo.create("img", {src: rmps.core.util.ContextTypes.WORKSPACE.icon.uri, "class": "icon"}, nameCell);
		var nameLink = dojo.create("a", null, nameCell);
		nameLink.appendChild(dojo.doc.createTextNode(this.stream.name));

		// State
		this._stateText = dojo.create("td", null, this.domNode);
		this._stateImage = dojo.create("img", null, this._stateText);

		// Actions
		var actionCell = dojo.create("td", null, this.domNode); 
		this._actionBar = new jazz.ui.SimpleToolbar();
		actionCell.appendChild(this._actionBar.domNode);
		this._actionBar.startup();	
		this._addActions();

		this._update();

		this._popup = new jazz.ui.HoverPopup({
			around : nameCell,
			onOpen: dojo.hitch(this, this._createPopupContents),
			maxWidth: 800,
			maxHeight: dojo.isIE ? 570 : 550
		});

		// does not work as advertised - need to actaully fetch more data about the stream from the server
		if (this.stream.status != rmps.core.streams.StreamStates.COMPLETED ) {
			this._updateTimer = setTimeout(dojo.hitch(this, this._update), 7000); // 7 seconds
		}

	},


	_addActions: function(){
		// commented out until hooked up to do something
//		this._btnDelete = this._actionBar.addButton(this, this._deleteAction, {
//		label: messages.deleteBtn,
//		imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/remove.gif")
//		});	

//		this._newSnapshotBtn = this._actionBar.addButton(this, this._newStreamAction, {
//		label: "Select",
//		imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/save.gif")
//		});

//		this._refreshBtn = this._actionBar.addButton(this, this._newStreamAction, {
//		label: "Refresh",
//		imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/refresh.gif"),
//		});

//		// Not hooked up yet...
//		this._actionBar.disableButton(this._btnDelete);
//		this._actionBar.disableButton(this._newSnapshotBtn);
//		this._actionBar.disableButton(this._refreshBtn);

		this._createAction = this._actionBar.addButton(this, this._createSnapshot, {
			label: messages.newSnapshot,
			imgSrc: dojo.moduleUrl("rmps.app", rmps.core.util.ContextTypes.SNAPSHOT.icon.uri)
		});
		
		this._compareSnapshotBtn = this._actionBar.addButton(this, this._compareStreams, {
		label: messages.compareWithPrevious,
		imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/search_resources.png")
		});

	}, // end _addActions

	_compareStreams: function() {
		var link = COMPARE_PATH + "?descendantUri=" + this.stream.uri;
		var win = window.open(link,'compareViewer','width=800,height=600,resizable=yes,toolbar=no,menubar=no,location=no');
		win.focus();
	},
	
	_createSnapshot: function() {
		var link = location.href;
		var index = link.indexOf("#");
		if (index != -1) {
			link = link.substring(0, index);
		}
		link += rmps.core.util.IRmpsConstants.ADMIN_REQUEST_BASELINE;
		link += "&workspaceUri=" + this.stream.uri;
		link += "&workspaceName=" + this.stream.name;
		location.href = link;
	},
	
	_newStreamAction : function() {

		//var newSnapshot = rmps.app.ui.internal.editors._NewSnapshotOnStreamPopup({messages:this.messages, stream:this.stream});

		this.editor.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary : "New snapshot requested",
			closable: true
		});
	},

	_deleteAction: function(){
		this._actionBar.disableButton(this._btnDelete);
		//var deferred = rmps.core.streams.StreamClient.deleteStream();

		this.editor.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: "Button not hooked up yet!",
			//summary: messages.requestingDelete,
			closable: true
		});
		/*
			deferred.addCallback(this, function(){
				this.editor.showMessage({
					type: jazz.ui.messageBox.STATUS_TYPE.INFO,
					summary: "Delete successful",
					closable: true
				});
			});

			deferred.addErrback(this, function(error){
				this.editor.showMessage({
					type : jazz.ui.messageBox.STATUS_TYPE.ERROR,
					summary : messages.deleteUnsuccessful,
					closable: true
				});
			});
		 */
	},// end _deleteAction


	_createPopupContents : function() {

		if(this._popupImmutable){
			return;
		}

		if(this._popupContent){
			this._popupContent.destroyRecursive();
		}// end if

		this._popupContent = new rmps.app.ui.internal.editors._StreamPopup({stream:this.stream});
		this._popup.content = this._popupContent.domNode;

		this._popupImmutable = (this.stream.status.name == rmps.core.streams.StreamStates.COMPLETED.name ) ? true : false;

	},// end createPopupContents

	_updateState: function(){
		this._stateImage.src = this.stream.status.icon;
		this._stateText.innerHTML = "";
		this._stateText.appendChild(this._stateImage);
		this._stateText.appendChild(dojo.doc.createTextNode(this.stream.status.name));
	},

	_update: function(){
		//console.log("Updating...");
		this._updateState();
	}

	}); // end streamRow

	dojo.declare("rmps.app.ui.internal.editors._StreamPopup", [dijit._Widget, dijit._Templated], {
		"class" : "rmps-stream-status-popup",
		messages: messages,
		stream: null,
		_nameText: null,
		templateString: '<table>' +
		'<tr><th>${messages.details}</th></tr>' +
		'<tr><td>${messages.name}:</td><td dojoAttachPoint="_nameText"/></tr>' +
		'<tr><td>${messages.state}:</td><td dojoAttachPoint="_stateText"/></tr>' +
		'<tr><td>${messages.children}:</td><td dojoAttachPoint="_streamText"/></tr>' +
		'<tr><td>${messages.description}:</td><td dojoAttachPoint="_descriptionText"/></tr>' +						
		'</table>',

		postCreate: function() {
//		var iconCell = dojo.create("img", {src: rmps.core.util.ContextTypes.WORKSPACE.icon.uri, "class": "icon"}, this._nameText);
		this._nameText.appendChild(dojo.doc.createTextNode(this.stream.name));
		this._stateText.appendChild(dojo.doc.createTextNode(this.stream.status.name));
		this._streamText.appendChild(dojo.doc.createTextNode(this.stream.snapshots.length));

		var description = this.stream.description;
		if (description) {
			this._descriptionText.appendChild(dojo.doc.createTextNode(description));
		} else {
			this._descriptionText.appendChild(dojo.doc.createTextNode(this.messages.noDescription));
		} // end if

	}// end postCreate
	});// end _StreamPopup

	/*
	dojo.declare("rmps.app.ui.internal.editors._NewSnapshotOnStreamPopup", [dijit._Widget, dijit._Templated], {
		"class" : "rmps-new-snapshot-on-stream-popup",
		messages: messages,
		stream: null,
		templateString: '<table>' + 
						'<tr><th>Add new snapshot to ${stream.name}</th></tr>' +
						'<tr><td>${messages.name}</td><td><input type="text" dojoAttachPoint="_snapshotName" /></td></tr>'
						'</table>'
	});
	 */
})(); // end of the earth