/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.TabbedEditor");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");

dojo.require("dijit._Widget");

dojo.require("jazz.core.loader");

dojo.declare("rmps.app.ui.internal.editors.TabbedEditor", rmps.app.ui.internal.editors.BaseEditor, {

	_tabContainer: null,

	postCreate: function() {
		dojo.addClass(this.headerContainerNode, "boxed");
		dojo.addClass(this.domNode, "rmps-app-editors-tabbed");
		this._tabContainer = new rmps.app.ui.internal.editors.TabbedEditor._TabContainer({
			editor: this
		});
		this.inherited(arguments);
	},

	createContentWidget: function() {
		this.addTabs();
		return this._tabContainer;
	},

	addTabs: function() {
		// hook for clients to create their tabs
	},

	addTab: function(tabId, label, page, args /*optional*/, index /*optional*/, href /*optional*/) {
		return this._tabContainer.addTab(tabId, label, page, args, index, href);
	},

	removeTab: function(tabId) {
		this._tabContainer.removeTab(tabId);
	},

	removeTabIndex: function(index) {
		this._tabContainer.removeTabIndex(index);
	},

	activate: function() {
		this.inherited(arguments);

		var args = this.getEditorInput().getParams();
		if (args && args.tab) {
			this.selectTab(args.tab);
		} else {
			var selectedTab = this._tabContainer.getSelectedTab();
			if (!selectedTab) {
				this.selectTab(this.getEditorInput().getState("tab"));
			}
		}
	},

	onTabChange: function(tabId, page, firstLoad) {
		this.getEditorInput().addState("tab", tabId);
		dojo.publish("/TabbedEditor/onTabChange");
	},

	getTabIndex: function(id) {
		return this._tabContainer.getTabIndex(id);
	},

	getTab: function(id) {
		/* Return the tab with the given id, or null if it does not exist */
		return this._tabContainer.getTab(id);
	},
	
	iFrameInitializer: function(){
		/*To be implemnted by subclasses */
	},
	
	selectTab: function(id) {
		var selectedTab = this._tabContainer.getSelectedTab();
		var newTab;
		
		if (id) {
			newTab = this._tabContainer.select(id);
		} else {
			newTab = this._tabContainer.selectIndex(0);
		}
		if (newTab && selectedTab != newTab) {
			this.onTabChange(newTab.tabId, newTab.page, newTab.__loaded == true);
			newTab.__loaded = true;
		}
		return newTab;
	},

	getSelectedTabId: function() {
		return this._tabContainer.getSelectedTab().tabId;
	}
});

dojo.declare("rmps.app.ui.internal.editors.TabbedEditor._TabContainer", [dijit._Widget, dijit._Templated], {

	templateString: '<div>' +
				'<div dojoAttachPoint="tabBar" class="rmps-app-editors-tabbed-container">' +
					'<div class="tabList" dojoAttachPoint="tabList"></div>' +
					'<div class="left-corner"></div>' +
					'<div class="right-corner"></div>' +
				'</div>' +
				'<div dojoAttachPoint="containerNode" style="position: relative;height: 100%;"></div>' +
			'</div>',

	// attach points
	tabList: null,
	containerNode: null,

	// mixin
	editor: null,

	// private properties
	_tabs: null,
	_selectedTab: null,

	constructor: function() {
		this._tabs = [];
	},

	destroy: function() {
		dojo.forEach(this._tabs, function(tab) {
			tab.destroyRecursive();
		});
		this.inherited(arguments);
	},

	resize: function(size) {
		if (!size) {
			return;
		}

		var cb = dojo.contentBox(this.domNode);
		var mb = dojo.marginBox(this.domNode);
		var cbContainerNode = dojo.contentBox(this.containerNode);
		var mbContainerNode = dojo.marginBox(this.containerNode);
		var mbTabBar = dojo.marginBox(this.tabBar);
		
		if (size.h) {
			size.h = size.h - mb.h + cb.h - mbContainerNode.h + mbContainerNode.h - mbTabBar.h;
		}

		if (this._selectedTab && this._selectedTab.page.resize) {
			this._selectedTab.page.resize(size);
		}
	},

	addTab: function(tabId, label, page, args, index, href) {
		var oldIdx = this.getTabIndex(tabId);
		if (oldIdx != -1) {
			this.removeTabIndex(oldIdx);
			if (index == null) {
				// replace old tab in the same position
				index = oldIdx;
			}
		}

		var tab = new rmps.app.ui.internal.editors.TabbedEditor._TabButton({
			editor: this.editor,
			'tabId': tabId,
			'label': label,
			'page': page,
			'args': args,
			'href': href
		});

		if (index == null || this._tabs.length == 0 || index >= this._tabs.length) {
			this._tabs.push(tab);
			this.tabList.appendChild(tab.domNode);
		} else {
			this._tabs.splice(index, 0, tab);
			dojo.place(tab.domNode, this.tabList, index);
		}

		tab.startup();

		if (dojo.isObject(page) && page.domNode) {
			page.domNode.style.display = "none";
			this.containerNode.appendChild(page.domNode);
		}
		return tab;
	},

	removeTab: function(tabId) {
		var index = this.getTabIndex(tabId);
		if (index != -1) {
			return this.removeTabIndex(index);
		}
		return false;
	},

	removeTabIndex: function(index) {
		var tab = this._tabs[index];
		if (tab) {
			this._tabs.splice(index, 1);
			tab.destroyRecursive();
			if (dojo.isObject(tab.page) && tab.page.destroyRecursive) {
				tab.page.destroyRecursive();
			}
			return true;
		}
		return false;
	},

	getTabIndex: function(tabId) {
		for (var i = 0, len = this._tabs.length; i < len; ++i) {
			if (tabId == this._tabs[i].tabId) {
				return i;
			}
		}
		return -1;
	},
	
	getTab: function(tabId) {
		for (var index in this._tabs) {
			if (tabId === this._tabs[index].tabId) {
				return this._tabs[index];
			}
		}
		return null;
	},
	
	getTabBarSize: function(){
		var box = dojo.marginBox( this.tabBar );
		return box;
	},

	selectIndex: function(index) {
		if (index >= 0 && index < this._tabs.length) {
			return this.select(this._tabs[index]);
		}
		return null;
	},

	select: function(key) {
		for (var i = 0, len = this._tabs.length; i < len; ++i) {
			var tab = this._tabs[i];
			if (tab.tabId == key || tab == key) {
				if (this._selectedTab != tab) {

					/*
					 * This call was a pre-step before caclulating the new
					 * requiremens for the new tab to be invoked. This call
					 * causes an issue with IE7 by showing unexpected scroll
					 * bars when togelling between tabs. Thus, we could comment
					 * out this call, as (according to the last behavior
					 * testing), no issues were encountered after commenting it.
					 *
					 * this.containerNode.style.overflow = "hidden";
					 *
					 */

					var processTabPage = dojo.hitch(this, function() {
						if (this._selectedTab) {
							this._selectedTab.deselect();
							this._selectedTab.page.domNode.style.display = "none";
						}

						if (tab.page.overflow) {
							if (dojo.isObject(tab.page.overflow)) {
								if (tab.page.overflow.x) {
									this.containerNode.style.overflowX = tab.page.overflow.x;
								}
								if (tab.page.overflow.y) {
									this.containerNode.style.overflowY = tab.page.overflow.y;
								}
							} else if (dojo.isString(tab.page.overflow)){
								this.containerNode.style.overflow = tab.page.overflow;
							} else {
								this.containerNode.style.overflow = "auto";
							}
						} else {
							this.containerNode.style.overflow = "auto";
						}

						if (tab.page.domNode.parentNode !== this.containerNode) {
							this.containerNode.appendChild(tab.page.domNode);
						}

						this._selectedTab = tab;
						tab.page.domNode.style.display = "";
						tab.select();

						if (!tab.page._started) {
							tab.page.startup();
						}
						
						/*So, tab when get notified when the tab widget gets selected */
						tab.page.onShow();
						
						if (tab.page.resize) {
							setTimeout(dojo.hitch(this, function() {
								
								if( this.ContainerNode ){
									tab.page.resize({h: this.containerNode.offsetHeight/*, w:this.containerNode.offsetWidth*/});
								}
								
							}), 0);
						}
					});
					
					// lazy initializers
					if (dojo.isFunction(tab.page)) {
						tab.page = tab.page(tab.args);
					}else if(new dojo._Url(tab.page).scheme){
						var frame = dojo.doc.createElement('iframe');
						
						frame.initializer= dojo.hitch(this.editor, this.editor.iFrameInitializer);
						frame.src= tab.page;
						frame.style.width= "100%";
						frame.style.height= "100%";
						frame.style.border= "0";
						
						tab.page= new dijit._Widget();
						tab.page.domNode.appendChild(frame);
						tab.page.domNode.style.width= "100%";
						tab.page.domNode.style.height= "100%";
						tab.page.overflow= "hidden";
					} else if (dojo.isString(tab.page)) {
						var asynchCallback = function(){
							// add protection code since select tab may have been called more then once before the load_async finishes
							if (dojo.isString(tab.page)) {  
					            var classConstructor = dojo.getObject(tab.page);
					            if (classConstructor) {
						            tab.page = new classConstructor(tab.args);
					            }
					            processTabPage();
							}
				        };
				        
						var classConstructor = dojo.getObject(tab.page);
						if (!classConstructor) {
							jazz.core.loader.load_async(tab.page, asynchCallback);
							return tab;
						}
						else {
							tab.page = new classConstructor(tab.args);
						}
					}
					
					processTabPage();
				}
				return tab;
			}
		}
		return null;
	},

	getSelectedTab: function() {
		if (this._selectedTab) {
			return this._selectedTab;
		}
		return null;
	}
});

dojo.declare("rmps.app.ui.internal.editors.TabbedEditor._TabButton", [dijit._Widget, dijit._Templated], {

	templateString: '<span class="tab">' +
				'<span>' +
					'<span>' +
						'<span class="inner">' +
							'<a dojoAttachPoint="linkNode">${label}</a>' +
						'</span>' +
					'</span>' +
				'</span>' +
			'</span>',

	editor: null,

	tabId: null,
	label: null,
	page: null,
	href: null,

	postCreate: function() {
		if (!this.href || dojo.isFunction(this.href)) {
			this.connect(this.editor.getEditorInput(), "onStateChanged", this._onStateChanged);
			this._onStateChanged(this.editor.getEditorInput().getState());
		} else {
			this.linkNode.href = this.href;
		}
		this.connect(this.linkNode, "onclick", this._onClick);
	},

	select: function() {
		dojo.addClass(this.domNode, "selected");
	},

	deselect: function() {
		dojo.removeClass(this.domNode, "selected");
	},

	_onClick: function(e) {
		if (dojo.hasClass(this.domNode, "selected")) {
			dojo.stopEvent(e);
		} else {
			this.editor.selectTab(this.tabId);
			if (this.href && dojo.isFunction(this.href)) {
				this.href(e);
			}
		}
	},

	_onStateChanged: function(state) {
		var editorInput = this.editor.getEditorInput();
		state.tab = this.tabId;
		this.linkNode.href = editorInput.getPresentationUri(state);
	}
});

})();
