/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.link.LinkTypeEditor");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorContents");
dojo.require("rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorBacklinkContents");
dojo.require("rmps.app.ui.internal.editors.views.link.LinkType");
dojo.require("rmps.app.ui.internal.editors.views.link.LinkClient");
dojo.require("jazz.ui.Breadcrumbs");
dojo.require("rmps.core.error");
dojo.require("rmps.ui.editor.EditorUtil");
dojo.require("rmps.app.ui.internal.util.UrlUtil");
dojo.require("rmps.core.base.DataFeed");
dojo.require("rmps.app.ui.ProjectLinkingEditorUtil");
dojo.require("rmps.app.ui.internal.editors.views.link.LinkClient");
dojo.require("rmps.app.ui.internal.editors.views.link.LinkEventManager");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.ui.internal.editors.link", "LinkTypeEditorMessages");

var linkTypeEditorMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors.link", "LinkTypeEditorMessages");

var RMPSProcessConstants = rmps.app.ui.ProjectLinkingEditorUtil.process;
var UrlUtil= rmps.app.ui.internal.util.UrlUtil;
var LinkClient=rmps.app.ui.internal.editors.views.link.LinkClient;
var LinkEventManager= rmps.app.ui.internal.editors.views.link.LinkEventManager;

dojo.declare("rmps.app.ui.internal.editors.link.LinkTypeEditor", rmps.app.ui.internal.editors.BaseEditor, {

	dirtyGroups: null,
	_breadCrumbs: null,

	isSystemLink: null,

	linkTypeEditorContents: null,
	linkTypeEditorBacklinkContents: null,

	constructor: function(){
		this.dirtyGroups= {};
		this.isSystemLink= false;
	},

	postCreate: function() {
		if (!this.suppressEditorUi) {
			this._breadCrumbs= new jazz.ui.Breadcrumbs({
				segments: [{
					label: linkTypeEditorMessages.projectLinking,
					href: rmps.app.ui.ProjectLinkingEditorUtil.baseUrl
				}]
			});
	
			dojo.place(this._breadCrumbs.domNode, this.headerContainerNode, "first");
		}

		var report = this.getEditorInput().operationReport;
		this.allowCreation = report.isAllowed(RMPSProcessConstants.Actions.CREATE);
		this.allowModification = report.isAllowed(RMPSProcessConstants.Actions.MODIFY);
		
		/*Need to set isSystemLink and isReadOnly before calling the parent*/		
		var linkType= this.getEditorInput().linkType;
		if(this.getEditorInput().linkTypeArg != "newLinkType"){
			if(linkType){
				this.isSystemLink= linkType.systemDefined;
				this.isReadOnly= this.isSystemLink|| (this.getEditorInput().linkTypeArg== "newLinkType"?!this.allowCreation:!this.allowModification);
			}
		}

		var mediator= this.getEditorInput().mediator;
		
		if(mediator){
			if(!mediator.linkTypeDataFeed){
				mediator.linkTypeDataFeed= new rmps.core.base.DataFeed({uniqueIdentifier: "about"});
			}
			mediator.linkTypeDataFeed.initEntriesForWidget(this);				
		}

		if(this.getEditorInput().linkTypeArg== "newLinkType"&& !this.allowCreation){
			this.updateWarningDetails(linkTypeEditorMessages.create_link_type_not_allowed);
		}else{
			if(this.getEditorInput().linkTypeArg!= "newLinkType"&& !this.allowModification){
				this.updateWarningDetails(linkTypeEditorMessages.modify_link_type_not_allowed);
			}
			this._createSections();
		}
		this.inherited(arguments);
	},

	updateWarningDetails: function(summary, type){
		if(!type){
			type= jazz.ui.MessageBox.STATUS_TYPES.INFO;
		}
		if(this.infoMessageBox){
			/*Hide and any previous messages */
			this.infoMessageBox.hideMessage();
		}else{
			if (this._breadCrumbs) {
				this.infoMessageBox = this.createAndStartMessagebox(this._breadCrumbs.domNode, this._breadCrumbs.domNode);
			} else {
				this.infoMessageBox = this.createAndStartMessagebox(this.headerContainerNode, this.headerContainerNode);
			}
		}

		this.infoMessageBox.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: summary,
				closable: true
		});
	},

	createAndStartMessagebox: function(node, mainWidget){
		var infoMessageBox = new jazz.ui.MessageBox({
			style: "width: 100%"
		});

		function publishResizeEvent() {
			dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
		}
		function onHideMessage() {
			delete infoMessageBox.__cps;
			publishResizeEvent();
		}

		infoMessageBox.domNode.style.margin = "1px";
		infoMessageBox._first = true;
		dojo.connect(infoMessageBox, "_toggleDetails", node, publishResizeEvent);
		dojo.connect(infoMessageBox, "hideMessage", node, onHideMessage);
		dojo.connect(infoMessageBox, "showMessage", node, publishResizeEvent);

		/*Place the messagebox at the beginning */
		dojo.place(infoMessageBox.domNode, mainWidget, "first");
		infoMessageBox.startup();
		return infoMessageBox;
	},

	_createSections: function(){
		/*Destroy the container node widgets recursively */
		this._destroyRecursive(this.containerNode);
		
		var linkType= this.getEditorInput().linkType;
		if(this.getEditorInput().linkTypeArg!= "newLinkType"){
			if(this.getEditorInput().linkTypeArg== "newLinkType"&& !this.allowCreation){
				this.updateWarningDetails(linkTypeEditorMessages.create_link_type_not_allowed);
			}else{
				/*Well. We decided to show the creation page instead so we need to check here again if the user is permitted to create links */

				this.linkTypeEditorContents= this.getAndCreateLinkTypeEditorContents();

				if(!this.isSystemLink){
					this.linkTypeEditorBacklinkContents= this.getAndCreateLinkTypeBacklinkEditorContents();
					this.linkTypeEditorBacklinkContents.setDefaultReverseLinkUri();
				}
				if(linkType){
					this.linkTypeEditorContents.setLinkTypeID(linkType.about);
					this.linkTypeEditorContents.linkTypeUriChanged();
					this.linkTypeEditorContents.setDescription(linkType.description?linkType.description:"");
					this.getEditorInput().setName(linkType.title);
					
					this.setTitleValue(linkType.title);
					
					if(!this.isSystemLink){
						this.linkTypeEditorBacklinkContents.setDefaultReverseLinkUri(linkType.backlink);
					}
				}

				this.containerNode.appendChild(this.linkTypeEditorContents.domNode);
				if(!this.isSystemLink){
					this.addSpace();
					this.containerNode.appendChild(this.linkTypeEditorBacklinkContents.domNode);
				}
				this.addSpace();

				if(linkType){
					var about= linkType.about;
					if(about){
						this.linkTypeEditorContents.setLinkTypeID(about);
					}else{
						this.linkTypeEditorContents.setDefaultLinkTypeID(linkType.title);
					}
				}else{
					this.linkTypeEditorContents.setDefaultLinkTypeID();
				}
			}
		}else{
			this.linkTypeEditorContents= this.getAndCreateLinkTypeEditorContents();
			if(!this.isSystemLink){
				this.linkTypeEditorBacklinkContents= this.getAndCreateLinkTypeBacklinkEditorContents();
				this.linkTypeEditorBacklinkContents.setDefaultReverseLinkUri();
			}

			this.containerNode.appendChild(this.linkTypeEditorContents.domNode);

			if(!this.isSystemLink){
				this.addSpace();
				this.containerNode.appendChild(this.linkTypeEditorBacklinkContents.domNode);
			}
			this.addSpace();
		}
	},

	addSpace: function(){
		/*Add a space between the two sections */
		var div = dojo.doc.createElement("div");
		div.innerHTML = "<br>";
		this.containerNode.appendChild(div);
	},

	getAndCreateLinkTypeEditorContents: function(){
		var isReadyOnly= this.getEditorInput().linkTypeArg== "newLinkType"?!this.allowCreation: !this.allowModification;
		var isEnabled= this.isSystemLink?false: this.getEditorInput().linkTypeArg== "newLinkType"?this.allowCreation:this.allowModification;
		this.linkTypeEditorContents = new rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorContents({
					isEnabled: isEnabled,
					isLinkTypeUriReadOnly: this.isSystemLink|| this.getEditorInput().linkTypeArg!= "newLinkType"||!this.allowCreation, 
					onDescriptionChange: isReadyOnly?dojo.hitch(this, this.doNothing): dojo.hitch(this, this.onDescriptionChange),
					onLinkTypeUriChange: isReadyOnly?dojo.hitch(this, this.doNothing): dojo.hitch(this, this.onLinkTypeUriChange)
		});
		return this.linkTypeEditorContents;
	},

	getAndCreateLinkTypeBacklinkEditorContents: function(){
		var isReadyOnly= this.getEditorInput().linkTypeArg== "newLinkType"?!this.allowCreation: !this.allowModification;
		this.linkTypeEditorBacklinkContents = new rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorBacklinkContents({
					isEnabled: this.isSystemLink?false: this.getEditorInput().linkTypeArg== "newLinkType"?this.allowCreation:this.allowModification,
					onProductSelection: isReadyOnly?dojo.hitch(this, this.doNothing): dojo.hitch(this, this.onProductSelection),
					onVersionSelection: isReadyOnly?dojo.hitch(this, this.doNothing): dojo.hitch(this, this.onVersionSelection),
					onLinkTyeSelection: isReadyOnly?dojo.hitch(this, this.doNothing): dojo.hitch(this, this.onLinkTyeSelection),
					onReverseLinkUriChange: isReadyOnly?dojo.hitch(this, this.doNothing): dojo.hitch(this, this.onReverseLinkUriChange)
		});
		return this.linkTypeEditorBacklinkContents;
	},

	refreshHref: function(timestamp){
		var href= window.location.href.toString();
		var index= href.lastIndexOf("&ts");
		if(index>-1){
			href= href.substring(0, index);
		}
		window.location.href = href + "&ts=" + timestamp;
	},

	isEditableTitle: function(){
		if(!this.isSystemLink && (this.getEditorInput().linkTypeArg== "newLinkType"?this.allowCreation: this.allowModification)){
			return true;
		}
		return false;
	},

	clear: function(){
		this.linkTypeEditorContents.clear();
		this.setTitleValue("");
	},

	getEditableTitleNodeDetails: function(){
		return linkTypeEditorMessages.linkTyeNameDefaultInstruction;
	},

	doNothing: function(){
		/*Do Nothing */
	},

	onDescriptionChange: function(e){
		var linkType= this.getEditorInput().linkType;
		this.changeDirtyStatusForKey("description", linkType?linkType.description: null, this.linkTypeEditorContents.getDescription());
	},

	onLinkTypeUriChange: function(e){
		var linkType= this.getEditorInput().linkType;
		this.changeDirtyStatusForKey("about", linkType?linkType.about: null, this.linkTypeEditorContents.getLinkTypeID());
	},

	onTitleChange: function(value){
		var linkType= this.getEditorInput().linkType;
		this.changeDirtyStatusForKey("title", linkType?linkType.title: null, value);
		if(!this.linkTypeEditorContents.isLinkTypeUriChanged){
			this.linkTypeEditorContents.setDefaultLinkTypeID(value);
		}
		this.getEditorInput().setName(value);
	},

	onProductSelection: function(value){
		/*Empty Block */
	},

	onVersionSelection: function(value){
		/*Empty Block */
	},

	onLinkTyeSelection: function(value){
		/*Empty Block */
	},

	onReverseLinkUriChange: function(value){
		var linkType= this.getEditorInput().linkType;
		this.changeDirtyStatusForKey("Backlink", linkType?linkType.backlink: null, value);
	},

	changeDirtyStatusForKey: function(key, currentValue, newValue){
		var linkType= this.getEditorInput().linkType;
		if(linkType){
			var currentValue_= currentValue?currentValue: "";

			/*We have to make sure that empty string is a case to dirty the editor. we have the condition two
			 * (newValue|| newValue== "") as a way to check if the current value should make the editor dirty*/
			if((newValue|| newValue== "") && currentValue_!=newValue){
				this.dirtyGroups[key]= true;
				this.addDirty(key);
			}else{
				this.dirtyGroups[key]= false;
				this.removeDirty(key);
			}
		}else{
			if((newValue|| newValue== "")){
				this.dirtyGroups[key]= true;
				this.addDirty(key);
			}else{
				this.dirtyGroups[key]= false;
				this.removeDirty(key);
			}
		}
	},

	validate: function(errors) {
		this._validateAbout(errors);
		this._validateName(errors);
		this._validateBacklink(errors);

	},

	_validateName: function(errors){
		var value= this.getTitleValue();
		if(!value|| dojo.trim(value)==""){
			if(!this._contains(errors, linkTypeEditorMessages.empty_Title_error)){
				errors.push(linkTypeEditorMessages.empty_Title_error);
			}
		}
	},

	_validateAbout: function(errors){
		var value= this.linkTypeEditorContents.getLinkTypeID();
		this._validateUrl(value, linkTypeEditorMessages.invalid_linkType_url, errors);
		var linkTypesDeferred= LinkClient.getAllLinkTypes(this.editorInput.projectId, {orderBy:"title"}, this.editorInput.contextUri).addCallback(this, function(result){
			var linkTypes= result.userDefined.concat(result.systemDefined);
			var linkType= this.getEditorInput().linkType;
			/*Now validate if the used uri is already used before. */
			for(var index=0; index<linkTypes.length; index++){
				var current= linkTypes[index];
				if(current.about==value&& (!linkType|| linkType.about!= current.about)){
					errors.push(linkTypeEditorMessages.existing_linkType_url);
					break;
				}
			}			
		});
	},

	_validateBacklink: function(errors){
		if(!this.isSystemLink){
			var value= this.linkTypeEditorBacklinkContents.getCurrentReverseLinkValue();
			if(!value|| value== ""){	/*Backlink is optional, so ignore vlaidation if it is empty */
				return;
			}

			this._validateUrl(value, linkTypeEditorMessages.invalid_backlink_url, errors);
		}
	},

	_validateUrl: function(url, errorMessage, errors){
		var v=  new dojo._Url(url);
		if(!v.host&& !v.authority){
			if(!this._contains(errors, errorMessage)){
				errors.push(errorMessage);
				return;
			}
		}
	},

	_contains: function(array, value){
		for(var index=0; index< array.length; index++){
			if(array[index]==value){
				return true;
			}
		}
		return false;
	},
	
	addDataEntryNotifier: function(data){
		/*Do Nothing */
	},
	
	editDataEntryNotifier: function(dataSetDetails, uniqueIdentifier, identifier, fieldName, fieldValue){
		/*Do Nothing */
	},
	
	removeDataEntryNotifier: function(details, uniqueIdentifier, data){
		/*close the editor in case we deleted the entry. Otherwise, the editor will have wrong data (for example wrong eTag) */
		if(data.about== this.getEditorInput().linkTypeArg){
			var editorUtil = rmps.ui.editor.EditorUtil.getInstance(this.domNode);
			if(editorUtil){
				editorUtil.closeEditor(this, true, true);				
			}
		}
	},

	_destroyRecursive: function(node){
		dojo.forEach(dijit.findWidgets(node), function(widget) {
			if (widget.destroyRecursive) {
				widget.destroyRecursive();
			}
		});
	},

	doSave: function(taskMonitor) {
		var data= new rmps.app.ui.internal.editors.views.link.LinkType();
		data.about= this.linkTypeEditorContents.getLinkTypeID();
		
		var linkType= this.getEditorInput().linkType;
		
		if(linkType){
			data.uri= linkType.uri;
		}

		data.title= this.getTitleValue();
		data.description= this.linkTypeEditorContents.getDescription();
		if(!data.description){
			data.description= "";
		}

		if(!this.isSystemLink){
			data.backlink= this.linkTypeEditorBacklinkContents.getCurrentReverseLinkValue();
		}
		data.systemDefined= false;

		var clonedData= dojo.clone(linkType);
		var deferred = this.getEditorInput().linkTypeArg== "newLinkType"? LinkClient.createLinkType(data, this.editorInput.contextUri)
										:LinkClient.updateLinkType(linkType,data,this.editorInput.contextUri);

		var task = taskMonitor.beginTask("", 1);
		
		if (!this.suppressEditorUi) {
			deferred.addCallback(this, function(result) {
				var finalizeUpdate= dojo.hitch(this, function(){
					for(var att in this.dirtyGroups){
						this.dirtyGroups[att]= false;
					}
					
					this.getEditorInput().linkTypeUri= result.about;
					this.getEditorInput().linkTypeArg = result.about;
					this.getEditorInput().linkType= result;
	
					if(!this.isSystemLink){
						this.linkTypeEditorBacklinkContents.resetCurrentBacklinkValue(data.backlink);
					}
					
					/*Make sure to update the browser title when changing the name/about fields */
					this.setTitle();
					
					/*
					 * Prevent editing link type URI "for now".
					 * @see: CLM Linking - should allow user to rename link type (62835)
					 */
					this.linkTypeEditorContents.setLinkTypeUriToReadOnlyMode();
				});
					
				if(this.getEditorInput().linkTypeArg== "newLinkType"){
					var doUpdateLinkObject= dojo.hitch(this, function(results){
						for(var index=0; index<results.length; index++){
							var current= results[index];
							if(current.about== result.about){
								var link= current;
								result= current;
								var mediator= this.getEditorInput().mediator;
								if(mediator.linkTypeDataFeed){
									mediator.linkTypeDataFeed.addDataEntry(link);
								}
								LinkEventManager.notifyAllCallback(link, "addAction");
								this.messageBox.hideMessage();
								window.location.hash = "#action=rmps.app.settings.projectManagement&linkType="+ data.about;
								finalizeUpdate();
								this.onSaveComplete();
								break;
							}
						}
				    });
				    /*69142: LinkTypeService: rmpsGet should set the details of the created link(s) to the response */
				    LinkClient.getLinkTypes(this.editorInput.projectId, null, this.editorInput.contextUri).addCallback(doUpdateLinkObject);	
				}else{
					var array= [];
					var mediator= this.getEditorInput().mediator;
					if(this.dirtyGroups.about){
						window.location.hash = "#action=rmps.app.settings.projectManagement&linkType="+ data.about;
						if(mediator){
							mediator.linkTypeDataFeed.updateIdentifier(clonedData.about, result.about, true, result);	    
						}
						LinkEventManager.notifyAllCallback({currentIdentifier:clonedData.about, newIdentifier: result.about}, "modifyIdentifier");
					}
					if(mediator){
						if(this.dirtyGroups.title){
							array.push("title");
							LinkEventManager.notifyAllCallback({identifier:result.about, field: "label", value:data.title}, "modifyAction");
						}
						if(this.dirtyGroups.description){
							array.push("description");
						}
						if(this.dirtyGroups.Backlink){
							array.push("Backlink");
						}
						mediator.linkTypeDataFeed.updateDataEntriesFeilds(result.about, array, result);
					}
					finalizeUpdate();
				}
			});
		}

		deferred.addErrback(this, function(error){
			if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
				rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
			}
			else {
				rmps.core.error.createError(error).dispatch();
			}
		});

		task.handleDeferredWork(deferred, 1);
	},

	onSaveComplete: function(){
		if(this.getEditorInput().linkTypeArg!= "newLinkType"){
			this.inherited(arguments);
		}
	},

	destroy: function(){
		this.linkTypeEditorContents.destroy();
		delete this.linkTypeEditorContents;
		this.linkTypeEditorContents= null;
		this.inherited(arguments);
	}

});

})();
