/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorBacklinkContents");

dojo.require("rmps.app.ui.internal.util.CLMUtil");
dojo.require("rmps.ui.CollapsibleSection");
dojo.require("dijit.form.RadioButton");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.app.ui.internal.editors.link.widgets", "LinkTypeEditorBacklinkContentsMessages");
var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors.link.widgets", "LinkTypeEditorBacklinkContentsMessages");

var SUPPORTED_BACKLINKS=
[
	{
		name: "RTC",
		text: messages.rtc,
		isDefault: true,
		details: [{
					version: "2.0",
					backlinks: [{
									displayName: messages.elaboratedBy,
									value: "http://jazz.net/xmlns/prod/jazz/calm/1.0/elaboratedByArchitectureElement"
								},
								{
									displayName: messages.relatedArtifact,
									value: "http://jazz.net/xmlns/prod/jazz/rtc/cm/1.0/com.ibm.team.workitem.linktype.relatedartifact.relatedArtifact"
								}]
					},{
					version: "3.0",
					backlinks: [{
									displayName: messages.elaboratedBy,
									value: "http://jazz.net/xmlns/prod/jazz/calm/1.0/elaboratedByArchitectureElement"
								},
								{
									displayName: messages.relatedArtifact,
									value: "http://jazz.net/xmlns/prod/jazz/rtc/cm/1.0/com.ibm.team.workitem.linktype.relatedartifact.relatedArtifact"
								},
								{
									displayName: messages.architectedBy,
									value: "http://open-services.net/ns/cm#architected-by"
								}]
					}]
	}
];

var getInformationFor=  function(name){
	for(var index=0; index<SUPPORTED_BACKLINKS.length; index++){
		var current= SUPPORTED_BACKLINKS[index];
		if(current.name==name){
			return current;
		}
	}
	return {};
};

var getBacklinkDetailsFor=  function(name){
	return getInformationFor(name).details;
};

var removeAllChildren= function(node){
	while(node.hasChildNodes()) {
		  node.removeChild(node.firstChild);
	}
};

dojo.declare("rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorBacklinkContents", [dijit._Widget, dijit._Templated], {

	"class": "rmps-app-link-type-editor-backlink-contents",

	messages: messages,

	templateString: '<div dojoAttachPoint="containerNode" containerNode></div>',

	_listeners: null,

	isEnabled: null,
	_reverseLinkUri: null,

	widgetsInTemplate: true,


	postCreate: function(){
		this.inherited(arguments);

		this.collapsibleContents= this.isEnabled?new rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorBacklinkContents.CollapsibleContents_({
			isEnabled: this.isEnabled,
			onProductSelection: dojo.hitch(this, this._onProductSelection),
			onLinkTypeSelection: dojo.hitch(this, this._onLinkTypeSelection),
			onVersionSelection: dojo.hitch(this, this._onVersionSelection),
			onReverseLinkUriChange: dojo.hitch(this, this._onReverseLinkUriChange)
		}): new rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorBacklinkContents.ReadOnlyCollapsibleContents_({});

		var section= new rmps.ui.CollapsibleSection({
			primaryTitle: messages.reverseLinkTitle,
			trim: rmps.ui.CollapsibleSectionStyles.GREY_ORANGE
        });
		section.containerNode.appendChild(this.collapsibleContents.domNode);
        this.containerNode.appendChild(section.domNode);

		this.setCurrentBacklinkValue= dojo.hitch(this.collapsibleContents, this.collapsibleContents.setCurrentBacklinkValue);
		this.setCustomReverseLinkUri= dojo.hitch(this.collapsibleContents, this.collapsibleContents.setCustomReverseLinkUri);
		this.getCurrentReverseLinkValue= dojo.hitch(this.collapsibleContents, this.collapsibleContents.getCurrentReverseLinkValue);
		this.resetCurrentBacklinkValue= dojo.hitch(this.collapsibleContents, this.collapsibleContents.resetCurrentBacklinkValue);
	},

	setCurrentBacklinkValue: function(value){/*to be hitched for  collapsibleContents*/},
	setCustomReverseLinkUri: function(value){/*to be hitched for  collapsibleContents*/},

	setDefaultReverseLinkUri: function(value){
		this.setCurrentBacklinkValue(value);
		this.setCustomReverseLinkUri(value);
	},

	_onProductSelection: function(value){
		this.onProductSelection(value);
	},

	_onLinkTypeSelection: function(value){
		this.onLinkTypeSelection(value);
	},

	_onVersionSelection: function(value){
		this.onVersionSelection(value);
	},

	_onReverseLinkUriChange: function(value){
		this.onReverseLinkUriChange(value);
	},

	onLinkTypeSelection: function(value){
		/*To be implemnted/connected externally */
	},

	onVersionSelection: function(value){
		/*To be implemnted/connected externally */
	},

	onReverseLinkUriChange: function(e){
		/*To be implemnted/connected externally */
	}

});

dojo.declare("rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorBacklinkContents.ReadOnlyCollapsibleContents_", [dijit._Widget, dijit._Templated], {

	messages: messages,

	widgetsInTemplate: true,

	_listeners: null,

	isEnabled: false,	/*For security reasons, it must be false by default */

	templateString:	'<div>' +
						'<div class="description" dojoAttachPoint="descriptionNode">${messages.currentValue}<span class="readOnlyDescriptionInput" dojoAttachPoint="currentBacklinkValueNode"></span></div>' +
					'<div>',

	setCurrentBacklinkValue: function(value){
		while(this.currentBacklinkValueNode.hasChildNodes()) {
			  this.currentBacklinkValueNode.removeChild(this.currentBacklinkValueNode.firstChild);
		}
		if(!value){
			value= messages.noBacklinkSpecified;
		}
		this.currentBacklinkValueNode.value= value;
		this.currentBacklinkValueNode.appendChild(dojo.doc.createTextNode(value));
	},

	setCustomReverseLinkUri: function(value){
		if(!value){
			value= "";
		}
		this.setCurrentBacklinkValue(value);
	},

	getCurrentReverseLinkValue: function(value){
		return this.currentBacklinkValueNode.value;
	},

	resetCurrentBacklinkValue: function(value){
		if(!value){
			value= "";
		}
		this.setCurrentBacklinkValue(value);
	}
});

dojo.declare("rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorBacklinkContents.CollapsibleContents_", [dijit._Widget, dijit._Templated], {

	messages: messages,

	widgetsInTemplate: true,

	_listeners: null,

	isEnabled: false,	/*For security reasons, it must be false by default */

	templateString:	'<div>' +
						'<div class="description" dojoAttachPoint="descriptionNode">${messages.backlinkDescription}</div>' +
						'<div class= "mainSelection">' +
						'		<span>' +
						'			<input class= "inputHeader" dojoAttachPoint= "currentSelectionHeader" dojoAttachEvent="onChange: _onSelection" dojoType="dijit.form.RadioButton" checked=true type= "radio"/>' +
						'			<span dojoAttachPoint= "currentBacklinkValueHeader"></span>' +
						'		</span>' +
						'		<span class="descriptionInput" dojoAttachPoint="currentBacklinkValueNode"></span>' +
						'</div>' +
						'<div class= "mainSelection">' +
						'			<input class= "inputHeader" dojoAttachPoint= "predefinedSelectionHeader" dojoAttachEvent="onChange: _onSelection" dojoType="dijit.form.RadioButton" type= "radio"/>${messages.predefinedSelection}' +
						'</div>' +
						'<span disabled=true class= "predefinedSelection" dojoType="rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorBacklinkContents._Predefined" ' +
						'		isEnabled="${isEnabled}" dojoAttachPoint="contents"></span>' +
						'<div class= "mainSelection">' +
						'		<span>' +
						'			<input class= "inputHeader" dojoAttachPoint= "customSelectionHeader" dojoAttachEvent="onChange: _onSelection" dojoType="dijit.form.RadioButton" type= "radio"/>${messages.customSelection}:' +
						'			<input dojoAttachPoint= "customSelection" dojoAttachEvent="onchange: _onCustomReverseLinkUriChange" disabled=true name="mainSelection" type= "text"/>'	+
						'		</span>' +
						'</div>' +
						'<br>'+
					'<div>',

	postCreate: function() {
		this.connect(this.contents, "onProductSelection", dojo.hitch(this, this._onProductSelection));
		this.connect(this.contents, "onVersionSelection", dojo.hitch(this, this._onVersionSelection));
		this.connect(this.contents, "onReverseLinkUriChange", dojo.hitch(this, this._onReverseLinkUriChange));
		this.connect(this.contents, "onLinkTypeSelection", dojo.hitch(this, this._onLinkTypeSelection));

		this.setReverseLinkUri= dojo.hitch(this.contents, this.contents.setReverseLinkUri);
		this.getReverseLinkUri= dojo.hitch(this.contents, this.contents.getReverseLinkUri);
		this.setReadOnly= dojo.hitch(this.contents, this.contents.setReadOnly);
		this.unsetReadOnly= dojo.hitch(this.contents, this.contents.unsetReadOnly);
		
		var id= "mainSelection"+ this.id.substring(this.id.lastIndexOf("_")+1,this.id.length);
		
		this.currentSelectionHeader.name= id; 
		this.predefinedSelectionHeader.name= id;
		this.customSelectionHeader.name= id;
		
		this.currentSelectionHeader.focusNode.name= id;
		this.predefinedSelectionHeader.focusNode.name= id;
		this.customSelectionHeader.focusNode.name= id;
		
		
		this.contents.setDisabled(true);
	},

	setCurrentBacklinkValue: function(value){
		removeAllChildren(this.currentBacklinkValueNode);
		removeAllChildren(this.currentBacklinkValueHeader);
		if(!value){
			this.currentBacklinkValueHeader.appendChild(dojo.doc.createTextNode(messages.noBacklinkSpecified));
			this.currentBacklinkValueNode.value= "";
		}else{
			this.currentBacklinkValueHeader.appendChild(dojo.doc.createTextNode(messages.currentSelection));
			this.currentBacklinkValueNode.value= value;
			this.currentBacklinkValueNode.appendChild(dojo.doc.createTextNode(value));
		}
	},

	resetCurrentBacklinkValue: function(value){
		if(!value&& value!= ""){
			value=	this.getCurrentReverseLinkValue();
		}
		this.setCurrentBacklinkValue(value);
		this.setCustomReverseLinkUri(value);
		this.currentSelectionHeader.attr('checked',true);
		this._onSelection();
	},

	setReverseLinkUri: function(value) {},

	getReverseLinkUri: function() {},

	_onSelection: function(e){
		this.contents.setDisabled(!this.predefinedSelectionHeader.checked);
		this.customSelection.disabled= !this.customSelectionHeader.checked;
		this.onReverseLinkUriChange(this.getCurrentReverseLinkValue());
	},

	_onProductSelection: function(value){
		this.onReverseLinkUriChange(value);
		this.onProductSelection(value);
	},

	_onVersionSelection: function(value){
		this.onReverseLinkUriChange(value);
		this.onVersionSelection(value);
	},

	_onLinkTypeSelection: function(value){
		this.onReverseLinkUriChange(value);
		this.onLinkTypeSelection(value);
	},

	getCurrentReverseLinkValue: function(){
		if(this.currentSelectionHeader.checked){
			return this.currentBacklinkValueNode.value;
		}else if(this.predefinedSelectionHeader.checked){
			return this.contents.getReverseLinkUri();
		}else if(this.customSelectionHeader.checked){
			return this.customSelection.value;
		}
	},



	onProductSelection: function(value){
		/*To be implemnted/connected externally */
	},

	onVersionSelection: function(value){
		/*To be implemnted/connected externally */
	},
	onLinkTypeSelection: function(value){
		/*To be implemnted/connected externally */
	},

	_onCustomReverseLinkUriChange: function(e){
		this.onReverseLinkUriChange(this.customSelection.value);
	},

	setCustomReverseLinkUri: function(value){
		if(!value){
			value= "";
		}
		this.customSelection.value= value;
	},

	onReverseLinkUriChange: function(e){
		/*To be implemnted/connected externally */
	}

});


dojo.declare("rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorBacklinkContents._Predefined", [dijit._Widget, dijit._Templated], {

	"class": "rmps-app-link-type-editor-backlink-contents",

	widgetsInTemplate: true,

	templateString: '<div dojoAttachPoint="containerNode">' +
						'<table nowrap="nowrap" class= "rmps-app-link-type-editor-backlink-contents">' +
								'<tr>' +
									'<td dojoAttachPoint="_productCell">${messages.product}:</td>' +
									'<td><div dojoAttachPoint="productSelectionNode"></div></td>' +
								'</tr>' +
								'<tr>'  +
									'<td>' +
										'<div dojoAttachPoint="_seprator1"></div>' +
									'</td>' +
		     					'</tr>' +
								'<tr>' +
									'<td dojoAttachPoint="_versionCell">${messages.version}:</td>' +
									'<td><div class="versionContents" dojoAttachPoint="versionNode"></div></td>' +
								'</tr>' +
								'<tr>' +
									'<td dojoAttachPoint="_linkTypeCell">${messages.linkType}:</td>' +
									'<td><div dojoAttachPoint="linkTypeNode"></div></td>' +
								'</tr>' +
								'<tr>'  +
									'<td>' +
										'<div dojoAttachPoint="_seprator2"></div>' +
									'</td>' +
		     					'</tr>' +
								'<tr>' +
									'<td dojoAttachPoint="_reverseLinkUriCell">${messages.reverseLinkUri}:</td>' +
									'<td><div dojoAttachPoint="_reverseLinkUriContainer"></div></td>' +
								'</tr>' +
						'</table>'+
					'</div>',

	_listeners: null,

	messages: messages,

	isEnabled: null,
	_reverseLinkUri: null,

	constructor: function(){
		this._listeners= [];
	},

	postCreate: function(){
		this.inherited(arguments);

		if(this.isEnabled){
			this._productSelection = new rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorBacklinkContents.ProductSelection_();
			this._versionSelection = new rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorBacklinkContents.VersionSelection_();
			this._linkTypeSelection = new rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorBacklinkContents.LinkTypeSelection_();
			this.productSelectionNode.appendChild(this._productSelection.domNode);
			this._seprator2.appendChild(dojo.doc.createElement("br"));
			this.versionNode.appendChild(this._versionSelection.domNode);
			this.linkTypeNode.appendChild(this._linkTypeSelection.domNode);


			dojo.addClass(this._productCell, "label");
			dojo.addClass(this._versionCell, "label");
			dojo.addClass(this._linkTypeCell, "label");

			dojo.addClass(this._reverseLinkUriCell, "baseLabel");

			dojo.addClass(this.versionNode, "field");
			dojo.addClass(this.productSelectionNode, "field");

			this.connect(this._productSelection, "onSelection", dojo.hitch(this, this._onProductSelection));
			this.connect(this._versionSelection, "onSelection", dojo.hitch(this, this._onVersionSelection));
			this.connect(this._linkTypeSelection, "onSelection", dojo.hitch(this, this._onLinkTypeSelection));

			this.connect(this._reverseLinkUri, "onkeyup", dojo.hitch(this, this._onReverseLinkUriChange));

			this._renderVersions(this._productSelection.selectedInformation.details);
		}else{
			this._seprator1.appendChild(dojo.doc.createElement("br"));
			this._seprator2.appendChild(dojo.doc.createElement("br"));

			this.productSelectionNode.appendChild(dojo.doc.createTextNode("Rational Team Concert"));	/*TODO */
			this.versionNode.appendChild(dojo.doc.createTextNode("2.0"));					/*TODO */

			dojo.addClass(this._productCell, "readOnlylabel");
			dojo.addClass(this._linkTypeCell, "readOnlylabel");

			dojo.addClass(this._reverseLinkUriCell, "readOnlylabel");

			dojo.addClass(this.productSelectionNode, "field");
			dojo.addClass(this.versionNode, "field");
			dojo.addClass(this._reverseLinkUriContainer, "field");
		}
	},

	setDisabled: function(value){
		if(this._linkTypeSelection){
			this._linkTypeSelection.setDisabled(value);
		}
		if(this._versionSelection){
			this._versionSelection.setDisabled(value);
		}
		if(this._productSelection){
			this._productSelection.setDisabled(value);
		}
	},


	setReverseLinkUri: function(value){
		while(this._reverseLinkUriContainer.hasChildNodes()) {
			  this._reverseLinkUriContainer.removeChild(this._reverseLinkUriContainer.firstChild);
		}
		this._reverseLinkUriContainer.appendChild(dojo.doc.createTextNode(value));
	},

	getReverseLinkUri: function(){
		var value= this._reverseLinkUriContainer.textContent;
		if(!value){
			value= this._reverseLinkUriContainer.innerText;
		}
		if(!value){
			value= this._reverseLinkUriContainer.innerHTML;
		}
		return value;
	},

	_onProductSelection: function(value){
		this._renderVersionsFor(value);
		this.onProductSelection(value);
	},

	_renderVersionsFor: function(value){
		this._renderVersions(getBacklinkDetailsFor(value));
	},

	_renderVersions: function(details){
		this._versionSelection.renderVersions(details);
		this._renderLinkTypesFor(details[this._versionSelection.selectNode.selectedIndex].backlinks);
	},

	_renderLinkTypesFor: function(backlinks){
		this._renderLinkTypes(backlinks);
	},

	_renderLinkTypes: function(backlinks){
		this._linkTypeSelection.renderLinkTypes(backlinks);
		this._onLinkTypeSelection(backlinks[this._versionSelection.selectNode.selectedIndex].value);
	},


	onProductSelection: function(value){
		/*To be implemnted/connected externally */
	},

	onLinkTypeSelection: function(value){
		/*To be implemnted/connected externally */
	},

	onVersionSelection: function(value){
		/*To be implemnted/connected externally */
	},

	_onVersionSelection: function(value){
		this.onVersionSelection(value);
		var versionDetails= this.getCurrentVersionDetails(value);
		if(versionDetails){
			var backlinks= versionDetails.backlinks;
			if(backlinks){
				this._renderLinkTypesFor(backlinks);
			}
		}
		this.onVersionSelection(value);
	},

	_onLinkTypeSelection: function(value){
		this.setReverseLinkUri(value);
		this.onLinkTypeSelection(value);
	},

	_onReverseLinkUriChange: function(e){
		var val= e.currentTarget.value;
		if(!val){
			return;
		}
		this.onReverseLinkUriChange(e.currentTarget.value);
		this.onReverseLinkUriChange();
	},

	onReverseLinkUriChange: function(value){
		/*To be implemnted/connected externally */
	},

	getCurrentVersionDetails: function(version){
		var details= this._productSelection.selectedInformation.details;

		for(var index=0; index<details.length; index++){
			var detail= details[index];
			if(detail.version== version){
				return detail;
			}
		}
	},

	destroy: function(){
		if(this._productSelection){
			this._productSelection.destroyRecursive();
		}
		if(this._versionSelection){
			this._versionSelection.destroyRecursive();
		}
		this.inherited(arguments);
	}

});


dojo.declare("rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorBacklinkContents.ProductSelection_", [dijit._Widget, dijit._Templated], {

	"class": "rmps-app-link-type-editor-backlink-contents",

	_listeners: null,
	_detailsMap: null,

	selectedInformation: null,

	_selections: null,
	templateString: '<div class= "rmps-app-link-type-editor-backlink-contents">' +
						'<div dojoAttachPoint="productsNode"></div>' +
					'</div>',

	constructor: function(){
		this._listeners= [];
		this._selections= [];
		this._detailsMap= {};
	},

	postCreate: function(){
		this.inherited(arguments);
		this._addProductSelections();
	},

	_addProductSelections: function(){
		for(var index=0; index<SUPPORTED_BACKLINKS.length; index++){
			var current= SUPPORTED_BACKLINKS[index];
			this.addNewProductSelection(current);
		}
	},

	setDisabled: function(value){
		for(var index=0; index<this._selections.length; index++){
			this._selections[index].attr('disabled',value);
		}
	},

	addNewProductSelection: function(information){
		var text= information.text;
		var value= information.name;
		var isDefault= information.isDefault;
		var mainSpan= dojo.doc.createElement("span");
		dojo.addClass(mainSpan, "selection");

		var selectionSpan= dojo.doc.createElement("span");

		var selection= new dijit.form.RadioButton();
		
		var id= "productSelectionGroup"+ this.id.substring(this.id.lastIndexOf("_")+1,this.id.length);
		
		selection.focusNode.name= id;
		selection.name= id;
		
		dojo.addClass(selection.domNode, "inputContent");

		if(isDefault){
			selection.attr('checked',true);
			this.selectedInformation= information;
		}
		this._detailsMap[selection.id]= value;

		dojo.addClass(selectionSpan, "inputSelection");
		selectionSpan.appendChild(selection.domNode);

		this._selections.push(selection);
		var textNode= dojo.doc.createElement("span");
		textNode.appendChild(dojo.doc.createTextNode(" "+ text));
		dojo.addClass(textNode, "inputText");

		mainSpan.appendChild(selectionSpan);
		mainSpan.appendChild(textNode);

		this.productsNode.appendChild(mainSpan);
		this.productsNode.appendChild(dojo.doc.createElement("br"));
		this.productsNode.appendChild(dojo.doc.createElement("br"));

		this._listeners.push(this.connect(selection, "onClick", dojo.hitch(this, this._onSelection)));
	},

	_onSelection: function(e){
		var value= this._detailsMap[e.currentTarget.id];
		this.selectedInformation= getInformationFor(value);
		this.onSelection(value);
	},

	onSelection: function(value){
		/*To be implemnted by callers */
	},

	destroy: function(){
		while (this._listeners.length > 0) {
			var current = this._listeners.pop();
			this.disconnect(current);
		}
		this.inherited(arguments);
	}
});


dojo.declare("rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorBacklinkContents.VersionSelection_", [dijit._Widget, dijit._Templated], {

	"class": "rmps-app-link-type-editor-backlink-contents",

	_listeners: null,

	templateString: '<div class= "rmps-app-link-type-editor-backlink-contents">' +
						'<div dojoAttachPoint="versionNode">' +
							'<select dojoAttachPoint="selectNode">' +
							'</select>'+
						'</div>' +
					'</div>',

	postCreate: function(){
		this.inherited(arguments);
		this.connect(this.selectNode, "onchange", dojo.hitch(this, this._onSelection));
	},

	setDisabled: function(value){
		this.selectNode.disabled= value;
		for(var index=0; index<this.selectNode.length; index++){
			this.selectNode[index].disabled= value;
		}
	},

	renderVersions: function(versions){
		this.clearOptions();
		for(var index=0; index<versions.length; index++){
			var ver= versions[index].version.toString();
			var option= dojo.doc.createElement("option");
			option.value=ver;
			option.appendChild(dojo.doc.createTextNode(ver));
			this.selectNode.appendChild(option);
		}
	},

	clearOptions: function(){
		this.versionNode.removeChild(this.selectNode);
		dojo.destroy(this.selectNode);
		this.selectNode= dojo.doc.createElement("select");
		this.connect(this.selectNode, "onchange", dojo.hitch(this, this._onSelection));
		this.versionNode.appendChild(this.selectNode);
	},

	_onSelection: function(e){
		this.onSelection(e.currentTarget.value);
	},

	onSelection: function(value){
		/*To be implemnted by callers */
	},

	onLinkTypeSelection: function(value){
		/*To be implemnted by callers */
	}
});


dojo.declare("rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorBacklinkContents.LinkTypeSelection_", [dijit._Widget, dijit._Templated], {

	"class": "rmps-app-link-type-editor-backlink-contents",

	_listeners: null,

	templateString: '<div class= "rmps-app-link-type-editor-backlink-contents">' +
						'<div dojoAttachPoint="linkTypeNode">' +
							'<select dojoAttachPoint="selectNode">' +
							'</select>'+
						'</div>' +
					'</div>',

	postCreate: function(){
		this.inherited(arguments);
		this.connect(this.selectNode, "onchange", dojo.hitch(this, this._onSelection));
	},

	setDisabled: function(value){
		this.selectNode.disabled= value;
		for(var index=0; index<this.selectNode.length; index++){
			this.selectNode[index].disabled= value;
		}
	},

	renderLinkTypes: function(backlinks){
		this.clearOptions();
		for(var index=0; index<backlinks.length; index++){
			var backlink= backlinks[index];
			var option= dojo.doc.createElement("option");
			option.value=backlink.value;
			option.appendChild(dojo.doc.createTextNode(backlink.displayName));
			this.selectNode.appendChild(option);
		}
	},

	clearOptions: function(){
		this.linkTypeNode.removeChild(this.selectNode);
		dojo.destroy(this.selectNode);
		this.selectNode= dojo.doc.createElement("select");
		this.connect(this.selectNode, "onchange", dojo.hitch(this, this._onSelection));
		this.linkTypeNode.appendChild(this.selectNode);
	},

	_onSelection: function(e){
		this.onSelection(e.currentTarget.value);
	},

	onSelection: function(value){
		/*To be implemnted by callers */
	},

	onLinkTypeSelection: function(value){
		/*To be implemnted by callers */
	}
});


})();
