/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorContents");

dojo.require("rmps.app.util.DOMNodeUtil");
dojo.require("rmps.app.ui.internal.util.CLMUtil");
dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.app.ui.internal.editors.link.widgets", "LinkTypeEditorContentsMessages");


var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors.link.widgets", "LinkTypeEditorContentsMessages");
var DOMNodeUtil= rmps.app.util.DOMNodeUtil;

dojo.declare("rmps.app.ui.internal.editors.link.widgets.LinkTypeEditorContents", [dijit._Widget, dijit._Templated], {

	messages: messages,
	"class": "rmps-app-link-type-editor-contents",
	widgetsInTemplate: true,

	_listeners: null,

	templateString: '<table nowrap="nowrap" class= "rmps-app-link-type-editor-contents">' +
							'<tr>' +
								'<td dojoAttachPoint="_descriptionCell"><span dojoAttachPoint="descriptionLabel">${messages.description}:</span></td>' +
								'<td><div dojoAttachPoint="_descriptionInputContainer"></div></td>' +
							'</tr>' +
							'<tr>'  +
								'<td>' +
									'<div dojoAttachPoint="_seprator"></div>' +
								'</td>' +
	     					'</tr>' +
	     					'<tr>' +
								'<td dojoAttachPoint="_linkTypeUriCell">${messages.linkTypeId}:</td>' +
								'<td><div dojoAttachPoint="_linkTypeUriContainer"></div></td>' +
							'</tr>' +
						'</table>',

	isLinkTypeUriChanged: false,
	_linkTypeUri: null,
	_descriptionInput:  null,
	isEnabled: null,
	isLinkTypeUriReadOnly: null,

	constructor: function(){
		/*For security reasons, it must be false by default */
		this.isEnabled= false;
		
		this.isLinkTypeUriReadOnly= false;
	},

	postCreate: function() {
		this._listeners= [];
		this._linkTypeUriCell.appendChild(DOMNodeUtil.createRequiredNode());
		if(this.isEnabled){
			this._createDescription();

			this._seprator.appendChild(dojo.doc.createElement("br"));
			dojo.addClass(this._descriptionCell, "label");
			this.connect(this._descriptionInput, "onkeyup", dojo.hitch(this, this.onDescriptionChange));
		}else{
			this._seprator.appendChild(dojo.doc.createElement("br"));
			dojo.addClass(this._descriptionCell, "readOnlylabel");
			dojo.addClass(this._descriptionInputContainer, "field");
		}
		if(this.isLinkTypeUriReadOnly){
			this.setLinkTypeUriToReadOnlyMode();	
		}else{
			this.setLinkTypeToEditMode();				
		}
		this.clear();
	},

	_createDescription: function(){
		this._descriptionInput= dojo.doc.createElement("textarea");
		this._descriptionInput.rows=5;
		dojo.addClass(this._descriptionInput, "field");
		this._descriptionInputContainer.appendChild(this._descriptionInput);
	},

	_createLinkTypeUri: function(){
		this._linkTypeUri= dojo.doc.createElement("input");
		this._linkTypeUri.type="text";
		dojo.addClass(this._linkTypeUri, "field");
		dojo.addClass(this._linkTypeUri, "derivedInput");
		this._linkTypeUriContainer.appendChild(this._linkTypeUri);
	},
	
	_removeLinkTypeUriInput: function(){
		if(this._linkTypeUri){
			this._linkTypeUriContainer.removeChild(this._linkTypeUri);	
		}
		delete this._linkTypeUri;
	},

	linkTypeUriChanged: function(){
		this.isLinkTypeUriChanged= true;

		if(this._linkTypeUri){
			dojo.removeClass(this._linkTypeUri, "derivedInput");

			/*No need for these listeners anymore */
			this.disconnect(this.linkTypeUriChangedListener);
			this.disconnect(this.linkTypeUriFocusListener);
			this.disconnect(this.linkTypeUriBlurListener);

			delete this.linkTypeUriChangedListener;
			delete this.linkTypeUriFocusListener;
			delete this.linkTypeUriBlurListener;
		}
	},
	
	clearUriListeners: function(){
		if(this.linkTypeUriChangedListener){
			this.disconnect(this.linkTypeUriChangedListener);			
		}
		if(this.linkTypeUriFocusListener){
			this.disconnect(this.linkTypeUriFocusListener);			
		}
		if(this.linkTypeUriBlurListener){
			this.disconnect(this.linkTypeUriFocusListener);			
		}
		delete this.linkTypeUriChangedListener;
		delete this.linkTypeUriFocusListener;
		delete this.linkTypeUriBlurListener;
	},
	
	setLinkTypeToEditMode: function(){
		this._createLinkTypeUri();
		dojo.addClass(this._linkTypeUriCell, "label");
		dojo.removeClass(this._linkTypeUriCell, "readOnlylabel");
		dojo.removeClass(this._linkTypeUriContainer, "field");
		this.setLinkTypeURIListeners();
	},
	
	setLinkTypeUriToReadOnlyMode: function(){
		dojo.addClass(this._linkTypeUriCell, "readOnlylabel");
		dojo.addClass(this._linkTypeUriContainer, "field");
		dojo.removeClass(this._linkTypeUriCell, "label");
		var value= this.getLinkTypeID();
		this._removeLinkTypeUriInput();
		this.clearUriListeners();
		if(!value){
			value= "";
		}
		this.setLinkTypeID(value);
	},
	
	setLinkTypeURIListeners: function(){
		this.linkTypeUriChangedListener= this.connect(this._linkTypeUri, "onkeyup", function(e){
			this.linkTypeUriChanged();
		});

		this.linkTypeUriFocusListener= this.connect(this._linkTypeUri, "onfocus", function(e){
			dojo.removeClass(this._linkTypeUri, "derivedInput");
		});

		this.linkTypeUriBlurListener= this.connect(this._linkTypeUri, "onblur", function(e){
			dojo.addClass(this._linkTypeUri, "derivedInput");
		});
		this.connect(this._linkTypeUri, "onkeyup", dojo.hitch(this, this.onLinkTypeUriChange));
	},

	clear: function(){
		this.setDefaultLinkTypeID();
		this.setDescription("");
	},

	setDefaultLinkTypeID: function(value){
		this.setLinkTypeID(rmps.app.ui.internal.util.CLMUtil.getDefaultLinkTypeURIForName(value?value: ""));
	},

	onDescriptionChange: function(e){
		/*to be implemnted by other places */
	},

	_stopEvent: function(e){
		if(e.ctrlKey&& (e.keyCode== 67|| e.keyCode== 65)){
			return;	/*Enable copy */
		}
		dojo.stopEvent(e);
	},

	onLinkTypeUriChange: function(e){
		/*to be implemnted by other places */
	},

	getLinkTypeID: function() {
		if(this._linkTypeUri){
			return this._linkTypeUri.value;
		}
		var value= this._linkTypeUriContainer.textContent;
		if(!value){
			value= this._linkTypeUriContainer.innerText;
		}
		if(!value){
			value= this._linkTypeUriContainer.innerHTML;
		}
		return value;
	},

	getDescription: function() {
		return this._descriptionInput.value;
	},

	setLinkTypeID: function(value){
		if(this._linkTypeUri){
			this._linkTypeUri.value= value;
			return;
		}
		while(this._linkTypeUriContainer.hasChildNodes()) {
			  this._linkTypeUriContainer.removeChild(this._linkTypeUriContainer.firstChild);
		}
		this._linkTypeUriContainer.appendChild(dojo.doc.createTextNode(value));
	},

	setDescription: function(value){
		if(!value){
			if(!this.isEnabled){
				value= messages.noDescription;
				dojo.addClass(this._descriptionInputContainer,"derivedInput");
			}else{
				dojo.removeClass(this._descriptionInputContainer,"derivedInput");
				value= "";
			}
		}

		if(this._descriptionInput){
			this._descriptionInput.value= value;
			return;
		}
		while(this._descriptionInputContainer.hasChildNodes()) {
			  this._descriptionInputContainer.removeChild(this._descriptionInputContainer.firstChild);
		}
		this._descriptionInputContainer.appendChild(dojo.doc.createTextNode(value));
	}

});

})();
