/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("rmps.app.ui.internal.editors.views.DataManagementSection");

	dojo.require("jazz.app.context");
	dojo.require("dijit._Widget");
	dojo.require("rmps.core.group.GroupClient");
	dojo.require("dojo.i18n");
	dojo.requireLocalization("rmps.app.ui.internal.editors.views", "DataManagementSectionMessages");

	var DIRTY_GROUP_ID = "rmps.app.ui.internal.editors.views.DataManagementSection.dirty";
	var GroupClient = rmps.core.group.GroupClient;
	var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors.views", "DataManagementSectionMessages");
	
	dojo.require("rmps.app.ui.internal.util.DomainRegistryUtil");
	
	dojo.declare("rmps.app.ui.internal.editors.views.DataManagementSection", dijit._Widget , {

		"class": "rmps-app-editors-data-management-definition",
		_dmManaged : false, // DM managed mode if enabled. otherwise, import is enabled.
		projectId : null,
		editor: null, // enclosing editor
		isDirty: false,
		_saveCancelled: false,
		
		postCreate: function() {
			this.inherited(arguments);
			if (this.editor) {
				this.projectId = this.editor.getEditorInput().projectId;
			}
			this._createSections();
		},
		
		startup: function() {
			this.inherited(arguments);
			this.choiceSection.startup();
			this.initDataManaged();
		},
		
		initDataManaged: function () {
			this._saveCancelled = false;
			var projectId = this.projectId ? this.projectId : jazz.app.context.get().itemId;
			if (projectId) {
				rmps.app.ui.internal.util.DomainRegistryUtil.isProjectActivelyManaged(
					dojo.hitch(this, function(isDmManaged, error) {
						this._dmManaged = isDmManaged;
						this.choiceSection.setMode(isDmManaged);	
						this._updateDirty(this._dmManaged, this._dmManaged, this.choiceSection);
					}), projectId
				);
			} else {
				this._dmManaged = false;
				this.choiceSection.setMode(false);
				this._updateDirty(this._dmManaged, this._dmManaged, this.choiceSection);
			}
		},
		
		_createSections: function() {
			dojo.forEach(dijit.findWidgets(this.domNode), function(widget) { 
				if (widget.destroyRecursive) {
					widget.destroyRecursive();
				}
			});
			this.choiceSection = new rmps.app.ui.internal.editors.views.DataManagementEditor._ChoiceSection({parent: this});
			this.choiceSection.setEnabled(true);
			this.domNode.appendChild(this.choiceSection.domNode);
		},
		
		isDMManagedMode: function() {
			/* Return true if we're in DM Managed mode */
			return this._dmManaged;
		},
		
		setEnabled: function(enabled) {
			this.choiceSection.setEnabled(enabled);
		},
		
		validateRequired: function(saveObj) {
			if (this.isDirty && this.projectId && !confirm(messages.confirmToDMManagedMode)) {
				this._saveCancelled = true;
				this.choiceSection._c1.checked = true;
				this._updateDirty(false, false, this.choiceSection);
			} 
		},
		
		doSave: function(taskMonitor, projectId) {
			if (this._saveCancelled) {
				// make sure editor isn't dirty anymore
				this._saveCancelled = false;
				this._updateDirty(false, false, this.choiceSection);				
				// generate an error
				var errorResult = new dojo.Deferred();
				errorResult.reject(new Error(messages.cancelled));
				return errorResult;
			}
			if (!projectId) {
				projectId = this.projectId ? this.projectId : jazz.app.context.get().itemId;
			}
			var deferred = GroupClient.setEditable(projectId, true);
			deferred.addCallback(this, function() {
				this._dmManaged=true;
				this.isDirty=false;
				this.setEnabled(false);
			});
			deferred.addErrback(this, function(error) {
				if (error.status === 403) {
					return new Error(messages.error_save_permission);
				} else if (error.status === 409) {
					return new Error(messages.error_save_conflict);
				} else {
					return new Error(messages.error_save + "\n" +messages['error_save.explanation'] + "\n" + messages['error_save.useraction']);
				}
			});
			var task = taskMonitor.beginTask(messages.task_saveConfiguration, 1);
			task.handleDeferredWork(deferred, 1);
			return deferred;
		},		
		
		_updateDirty: function(originalValue, newValue, dirtyKey) {
			var isEqual = originalValue == newValue;
			if (!isEqual && dojo.isArray(newValue) && originalValue && newValue && originalValue.length === newValue.length) {
				isEqual = true; // empty arrays
				isEqual = dojo.every(newValue, function(item, idx) {
					return item == originalValue[idx];
				});
			}
			if (isEqual) {
				// if save is cancelled, save needs to be forced to generate the error, hence leave editor dirty if save is cancelled 
				this.isDirty = this._saveCancelled;
				if (this.editor) {
					this.editor.removeDirty(dirtyKey, DIRTY_GROUP_ID);
					this.editor.hideMessage();
				}
			} else {
				this.isDirty = true;
				if (this.editor) {
					this.editor.addDirty(dirtyKey, DIRTY_GROUP_ID);
					this.editor.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
						summary: messages.info_save_first,
						closable: false
					});
				}
			}
		}
		
	});	
	
	dojo.declare("rmps.app.ui.internal.editors.views.DataManagementEditor._ChoiceSection", [dijit._Widget, dijit._Templated], {
	
		messages: messages,
		
		_mode: null,
		_originalMode: null,
	
		templateString: '<div>' +
							'<div class="description">' +
								'<label title="${messages.choiceDescriptionDesc}">${messages.choiceDescription}</label>' +
							'</div>' +
							'<div>' +
								'<label><input id="${id}_c1" type="radio" name="typeChoice" value="1" dojoAttachPoint="_c1" dojoAttachEvent="onclick: _toggleChoice"/>${messages.choice_externallyManagedMode}</label>' +
							'</div>' +
							'<div>' +
								'<label><input id="${id}_c2" type="radio" name="typeChoice" value="2" dojoAttachPoint="_c2" dojoAttachEvent="onclick: _toggleChoice"/>${messages.choice_dmManagedMode}</label>' +
							'</div>' +
						'</div>',
	
		parent: null,
		setMode: null, /* function setup during postCreate - can be called only once*/
		
		postCreate: function() {
			this.inherited(arguments);
			
			// Start off disabled
			this._c1.disabled=true;
			this._c2.disabled=true;
			
			var self = this;
			this.setMode = function(isDmManaged) {
						self._mode = isDmManaged;
						self._originalMode = dojo.clone(isDmManaged);
						if (isDmManaged) {
							self._c2.checked=true;
							self.setEnabled(false)
						} else {
							self._c1.checked=true;
							self.setEnabled(true)
						}
				};			
		},
		
		setEnabled: function(enabled) {
			this._c1.disabled = !enabled;
			this._c2.disabled = !enabled;
		},
		
		_toggleChoice: function(e) {
			this._mode = e.target.value == 2;
			this.parent._updateDirty(this._originalMode, this._mode, this);
		}
	});

	
})();
