/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("rmps.app.ui.internal.editors.views.IncomingLinksSection");

    dojo.require("rmps.app.ui.internal.editors.views.DetailedIncomingLinksSection");
	dojo.require("rmps.app.ui.internal.editors.views.LinksSection");
	dojo.require("rmps.app.ui.internal.editors.views.link.LinkClient");
	dojo.require("rmps.app.ui.internal.util.CLMUtil");
	dojo.require("rmps.ui.LoginArea");
	dojo.require("rmps.ui.CollapsibleSection");
	dojo.require("jazz.ui.ResourceLink");
	dojo.require("jazz.util.html");

	dojo.requireLocalization("rmps.app.ui.internal.editors.views", "IncomingLinksSectionMessages");

	var LinkClient = rmps.app.ui.internal.editors.views.link.LinkClient;
	var DetailedIncomingLinksSection=rmps.app.ui.internal.editors.views.DetailedIncomingLinksSection;
	var CLMUtil = rmps.app.ui.internal.util.CLMUtil;
	var ResourceLink = jazz.ui.ResourceLink;
	var IncomingLinksSection_UAKEY = "com.ibm.xtools.rmps.web.app.section_IncomingLinksSection001";

    dojo.declare("rmps.app.ui.internal.editors.views.IncomingLinksSection", rmps.app.ui.internal.editors.views.LinksSection, {

        detailsList: null,
        serviceProviders: null,
        _resourceLinks: null,
        _deferredRows: null,

        constructor: function(){
            this.messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors.views", "IncomingLinksSectionMessages");
            this.detailsList = [];
            this._resourceLinks = [];
        },

        startup: function(){
            this.addHeader("rmps.ui.sections.DescriptionHeader", {
                sectionDescription:this.messages.incomingLink
            });
            var section= this.inherited(arguments);
            return section;
        },

        styleTableAndHeaders: function(sectionHeaders, scope, table){
            var description= sectionHeaders["rmps.ui.sections.DescriptionHeader"];
            if(description){
				description.domNode.style.fontWeight= "bold";
				description.domNode.style.color= "#666668";
				description.domNode.style.paddingLeft= "10px";
	        }
        },

        showExternalLinks: function(){
            return false;
        },

        showCategories: function(data){
            return false;
        },

        getHeaderArguments: function(){
			return {
	            sectionTitle:  this.messages.incomingLink,
	            uaKey:         IncomingLinksSection_UAKEY
	        };
		},

		enableDescription: function(){
            return false;
        },

        styleRow: function(data, tr){
            /*We do not provide styles for the rwos in the incoming links section. Do we need to? If so, we could add later */
        },

        updateTableDataDescription: function(tableRow, tableData, data) {
			var serviceProvider= data.serviceProviders[0];
			var uri = serviceProvider.details.uri;

			if (serviceProvider.isImplicit && data.url.substring(data.url.lastIndexOf("/") + 1, data.url.length) === "implicit") {
				if (this._deferredRows[uri]) {
					if (this._deferredRows[uri].linksWithHeaders.length === 0) {
						return null;
					}
				} else {
					this._updateImplicitRow(tableRow, tableData, data);
					return null;
				}
            }

            var span = document.createElement("SPAN");
            this.appendArrow(span);
            var self= this;

			var resourceLink= new ResourceLink({
	                uri: uri,
	                lazyFetch: true,
	                retainLinkText: true,
	                presentationUri: uri,
	                label: jazz.util.html.escape(serviceProvider.presentationLabel),
	                _batchedPresentationHandler: function(result) {
		                var parentCall = resourceLink.getInherited("_batchedPresentationHandler", arguments);
		                dojo.hitch(resourceLink, parentCall)(result);

						/*If we logged in using the resourceLink, then, we will need to make sure that the login area is updated accordingly*/
						if (!result.oauthRequired && !data.isRender) {
							loginArea.showAsLogged();
							self.appendMoreDetails(section.containerNode, tableRow, tableData, data, null);
							data.isRender= true;
						}
	                },
	                _errorRefresh: function(err) {
		                var parentCall = resourceLink.getInherited("_errorRefresh", arguments);
		                dojo.hitch(resourceLink, parentCall)(err);

						self.appendMoreDetails(section.containerNode, tableRow, tableData, data, null);
	                }
		    });

            this._resourceLinks.push(resourceLink);
			var section= new rmps.ui.CollapsibleSection({
				primaryTitle: resourceLink.domNode,
				trim: rmps.ui.CollapsibleSectionStyles.GREY_ORANGE
            });
			resourceLink.domNode.style.color= "#666668";
			resourceLink.domNode.style.paddingLeft= "8px";
			
			var loginLink= CLMUtil.getAuthneticationUrl(this.serviceProviders, serviceProvider.serviceUrl);
			
            var loginArea= new rmps.ui.LoginArea({
				loginLink: loginLink?loginLink:uri,
				lazyCheck: false,
				unloggedText: this.messages.loginMessage,
				onAuthenticated: dojo.hitch(this,function(response){
					resourceLink.refresh();
	            })
			});

			dojo.addClass(loginArea.domNode, "action");
            section.addHeaderNode(loginArea.domNode);

            tableData.appendChild(section.domNode);

            var div = dojo.doc.createElement("div");
			div.innerHTML = "<br>";
			span.appendChild(div);

            return tableData;
        },

        appendMoreDetails: function(span, tableRow, tableData, data, result) {
            var get= dojo.hitch(this, function(span){
                for(var index=0;index<span.childNodes.length;index++){
                    var current= span.childNodes[index];
                    if(current.tagName.toUpperCase() === "A" && current.className === "jazz-ui-ResourceLink"){
                        return current;
                    }
                }
            },span);

            var serviceProvider = data.serviceProviders[0];
            var current = null;
            var newVal = null;
            if (result && result.oauthRequired) {
                current = get(span);
                newVal= serviceProvider.presentationLabel + this.messages.login;
//              current.textCotent= newVal;
//              current.childNodes[0].data= newVal;
				return;
			}else{
                current= get(span);
                newVal= serviceProvider.presentationLabel;
//              current.textCotent= newVal;
//              current.childNodes[0].data= newVal;
			}
            var value = serviceProvider.serviceUrl;
            if (!value || value.indexOf("services.xml") !== -1) {
                return;
            }

            var rowData = this._deferredRows[serviceProvider.details.uri];
            if (rowData) {
                this.prepareData(span, rowData.linksWithHeaders, rowData.types);
            } else {
                var deferred = CLMUtil.getQueryService(value);
                deferred.addCallback(this, function(queryInterface) {
                    if (!queryInterface) {
                        /* For now, I just replace the segment oslc_am/oslc_rm by oslcquery, as this would do the work. If it required some changes in the future resulted
                         * in us not being able to get the oslc query link, then, we will make a deferred call. */
                        queryInterface = value.replace("oslc_am","oslcquery").replace("oslc_rm", "oslcquery");                	
                    }
                    CLMUtil.getIncomingLinksDetails(dojo.hitch(this, this.prepareData, span), this.resourceInfo, this.scope._types, queryInterface);
                });
            }
        },

        prepareData: function(span, linksWithHeaders, types){
            this._types= types;
			/*Create Link Types Section */
			/*Required to track the last item in a category to add a space in underneath */

	        var details= new rmps.app.ui.internal.editors.views.DetailedIncomingLinksSection({
				result: linksWithHeaders,
				resourceInfo: this.resourceInfo,
                scope: span
            }).startup(span);

			span.style.paddingTop= "8px";
			span.style.paddingLeft= "8px";

            if (linksWithHeaders.length>0) {
//				var div = dojo.doc.createElement("div");
//				div.innerHTML = "<br>";
//				span.appendChild(div);
	        }

            this.detailsList.push(details);
        },

        appendArrow: function(span){
            /*Empty block */
        },

        destroy: function(){
            for(var index= 0; index<this.detailsList.length; index++){
                var current= this.detailsList[index];
                if(current&& current.destroyRecursive){
                    current.destroyRecursive();
	            }
            }
            this.inherited(arguments);
        },

		_updateImplicitRow: function(tableRow, tableData, data) {
			var serviceProvider = data.serviceProviders[0];
            var queryInterface = serviceProvider.serviceUrl.replace("oslc_am","oslcquery").replace("oslc_rm", "oslcquery");
			CLMUtil.getIncomingLinksDetails(dojo.hitch(this, this._appendRow, data), this.resourceInfo, this.scope._types, queryInterface);
		},

		_appendRow: function(data, linksWithHeaders, types) {
			var serviceProvider = data.serviceProviders[0];
			var uri = serviceProvider.details.uri;

			if (linksWithHeaders.length === 0) {
				// Do nothing
				return;
			}

			this._deferredRows[uri] = {
				linksWithHeaders: linksWithHeaders,
				types: types
			};

			this.addSectionTableRow(this, this.table, data);
		}
    });

})();
