/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("rmps.app.ui.internal.editors.views.Links");
    dojo.require("rmps.app.ui.internal.editors.views.OutgoingLinksSection");
    dojo.require("rmps.app.ui.internal.editors.views.IncomingLinksSection");
    dojo.require("rmps.app.ui.internal.editors.views.link.LinkClient");
    dojo.require("rmps.app.ui.internal.editors.views.link.utils");
    dojo.require("rmps.app.ui.internal.util.CLMUtil");
    dojo.require("rmps.core.streams.StreamClient");
    dojo.require("rmps.ui.link.LinkUtil");
    dojo.require("dojox.dtl.filter.strings");
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("jazz.ui.MessageBox");
    dojo.require("jazz.util.array");
    dojo.require("dojo.i18n");
    dojo.require("rmps.core.process.SecurityClient");
    dojo.require("rmps.core.TaskMonitor");
    dojo.require("rmps.core.util.ContextUtils");    
    dojo.require("rmps.core.RootServices");
    dojo.require("rmps.core.client");

    dojo.requireLocalization("rmps.app.ui.internal.editors.views", "LinksSectionMessages");
    dojo.requireLocalization("rmps.app.ui.internal.editors", "BaseEditorMessages");
    dojo.requireLocalization("rmps.app.ui.internal.editors", "LinkTabMessages");

    var LinkClient = rmps.app.ui.internal.editors.views.link.LinkClient;
    var CLMUtil= rmps.app.ui.internal.util.CLMUtil;
    var Process= rmps.ui.link.LinkUtil.Process;
    var SecurityClient= rmps.core.process.SecurityClient;
    var ContextUtils= rmps.core.util.ContextUtils;
    var BEMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "BaseEditorMessages");
    var REDACT_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "redact");
    
    /*Generic Widget class could be used by other clients to show the contents for the CLM links.
     *
     * Required Parameters:
     *	    resourceInfo
     *
     * NOTE: Clients should implement their addDirtyNotifier, removeDirtyNotifier, doSave methods as well
     *
     *
     */
    dojo.declare("rmps.app.ui.internal.editors.views.Links", [dijit._Widget, dijit._Templated], {

        templateString: '<div class="container">' +
        '<span dojoAttachPoint="menuWidgetSpan"></span>' +
        '<div class="links-content" dojoAttachPoint="containerNode">' +
        '</div>' +
        '<BR></div>', 

		editor: null,
		baseline: null,
		report: null,	

		serviceProviders: null,

        listWidget: null,
        _types: null,
        _modifiedlinks: null,
        _controller: null,
        _securityDeferred: null,

        "class": "rmps-app-tabs-links",

        constructor: function(){
            this._modifiedlinks= {};
            this.linkMessages= dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "LinkTabMessages");
            this.messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors.views", "LinksSectionMessages");
        },

        postCreate: function(){
			try {

				if (!this.resourceInfo) {
					this.resourceInfo = this.editor.editorInput.resourceInfo;
				}

/* Disable check of isReadOnly flage, see WI#25269
				this.isReadOnly = this.isReadOnly || ContextUtils.isSnapshot();
*/
				this.isReadOnly = ContextUtils.isSnapshot();
/* Disable connection, see WI#25269
				this.connect(this.editor, "doSave", this.doSave);
*/
				this._modifiedlinks[this.id] = {
	                _linksToBeSaved: [],
	                _linksToBeDeleted: [],
					_linksToBeEdited: []
				};

				this._securityDeferred = SecurityClient.isActionAllowed(
					Process.Link_Resource_Operations.LINK,
					dojo.getObject("jazz.app.currentApplication").context,
					[
						Process.Actions.CREATE,
						Process.Actions.MODIFY,
						Process.Actions.DELETE
					]);

				this._securityDeferred.addCallback(this, function(report) {
					this.report= report;
					var deferred = rmps.core.streams.StreamClient.canUpdateStream();
					deferred.addErrback(this, function() {
						this.isReadOnly = true;
					});
					deferred.addBoth(dojo.hitch(this, function() {
						this._securityDeferred = CLMUtil.getServiceProviders().addCallback(dojo.hitch(this, function(serviceProviders) {
							this.serviceProviders= serviceProviders;
							var args = {
								url: REDACT_PATH + "/enabled",
								headers: {
									"X-ibm-rmps-internal" : "true"
								},
								load:  function(response, ioArgs) {            
									if(ioArgs.xhr.status === 200){
										outerObject.redactEnabled = true;
									}
									CLMUtil.getOutgoingLinksDetails(dojo.hitch(outerObject, outerObject.prepareData), outerObject.resourceInfo, null, null);
								},
								error: function() {
									// Just continue on error
									CLMUtil.getOutgoingLinksDetails(dojo.hitch(outerObject, outerObject.prepareData), outerObject.resourceInfo, null, null);
								}
							}; // end args
							var outerObject = this;
							rmps.core.client.xhrHead(args);			
						}));
					}));
				});
            }
            catch (e) {
                console.debug("*** ERROR: " + e.message);
            }
        },
        
        /** Perform a refresh of the Links tab.
         */
        doRefresh: function() {
            // Reset the contents of the links tab.
            this._resetLinkTabContents();
            this.editor.hideMessage();
            
            // Populate the link tab contents again
			CLMUtil.getOutgoingLinksDetails(dojo.hitch(this, this.prepareData), this.resourceInfo, null, null);
        },

        prepareData: function(linksWithHeaders, types) {
            this._types = types;
			/* Create Link Types Section */
			this.linksSection = new rmps.app.ui.internal.editors.views.OutgoingLinksSection({
				result: linksWithHeaders,
				resourceInfo: this.resourceInfo,
				report: this.report,
				isReadOnly: this.isReadOnly,
				baseline: this.baseline,
                scope: this,
                redactEnabled: this.redactEnabled
            }).startup(this);

            /*Add a space between the two sections */
			var div = dojo.doc.createElement("div");
			div.innerHTML = "<br>";
			this.containerNode.appendChild(div);
			CLMUtil.getProjectLinksForElement(this.resourceInfo.uri, dojo.hitch(this,this._createIncomingLinkSection));
        },

        _createIncomingLinkSection: function(projectLinks) {
            var projectLinkUrls = [];
            for (var pl = 0; pl < projectLinks.length; pl++) {
                projectLinkUrls.push(projectLinks[pl].resource);
            }
            CLMUtil.getProjectLinks(projectLinkUrls, dojo.hitch(this, this._prepareIncomingLinkSectionRendering));
        },

        _prepareIncomingLinkSectionRendering: function(projectLinks) {
            if (projectLinks.length === 0) {
                return;
            }

            var implicitProjectLinks = [];
            for (var pl = 0; pl < projectLinks.length; pl++) {
                var projectLink = projectLinks[pl];
                var serviceProvider = projectLink.serviceProviders[0];
                if (serviceProvider.isImplicit && projectLink.url.substring(projectLink.url.lastIndexOf("/") + 1, projectLink.url.length) === "implicit") {
                    implicitProjectLinks.push(projectLink);
                }
            }

            if (projectLinks.length > implicitProjectLinks.length) {
                this._renderIncomingLinkSection(projectLinks, null);
                return;
            }

            var deferredRows = {
                total: 0
            };

            var dra = [];
            for (var ipl = 0; ipl < implicitProjectLinks.length; ipl++) {
                var implicitProjectLink = implicitProjectLinks[ipl];
                dra.push(this._renderImplicitRow(implicitProjectLink, deferredRows));
            }

            dojo.when(new dojo.DeferredList(dra),
                      dojo.hitch(this, this._renderIncomingLinkSection, projectLinks, deferredRows));
        },

		_renderIncomingLinkSection: function(projectLinks, deferredRows) {
            if (projectLinks.length === 0 || (deferredRows && deferredRows.total === 0)) {
                return;
            }

            this.incomingLinksSection = new rmps.app.ui.internal.editors.views.IncomingLinksSection({
                _deferredRows: deferredRows ? deferredRows : {},
                result: projectLinks,
                serviceProviders: this.serviceProviders,
				resourceInfo: this.resourceInfo,
				isReadOnly: this.isReadOnly,
				baseline: this.baseline,
                scope: this
            }).startup(this);
        },

        _renderImplicitRow: function(projectLink, deferredRows) {
			var serviceProvider = projectLink.serviceProviders[0];
            var queryInterface = serviceProvider.serviceUrl.replace("oslc_am","oslcquery").replace("oslc_rm", "oslcquery");
			return CLMUtil.getIncomingLinksDetails(dojo.hitch(this, this._updateDeferredRows, projectLink, deferredRows), this.resourceInfo, this._types, queryInterface);
		},

		_updateDeferredRows: function(projectLink, deferredRows, linksWithHeaders, types) {
			var serviceProvider = projectLink.serviceProviders[0];
			var uri = serviceProvider.details.uri;

			deferredRows[uri] = {
				linksWithHeaders: linksWithHeaders,
				types: types
			};

			if (linksWithHeaders.length > 0) {
				deferredRows.total++; 
			}
		},

        getLinks: function(){
            return LinkClient.getLinksFor(this.resourceInfo.uri);
        },

        sortData: function(tableData, orderBy){

            function stringCompare(string1, string2){
                return string1.toLocaleLowerCase().localeCompare(string2.toLocaleLowerCase());
            }

            function compare(entry1, entry2){
                var a = dojo.getObject(orderBy, false, entry1);
                var b = dojo.getObject(orderBy, false, entry2);

                if (dojo.isString(a)) {
                    a = a.toLowerCase();
                }
                if (dojo.isString(b)) {
                    b = b.toLowerCase();
                }
                return stringCompare(b, a);
            }

            function comparator(entry1, entry2){
                for (var i = 0; i < orderBy.length; ++i) {
                    var result = compare(entry1, entry2);
                    if (result !== 0) {
                        return result;
                    }
                }
                return 0;
            }

            tableData.sort(comparator);
        },

        addDirtyNotifier: function(row){
            /*To be implemented by subclasses */
        },

        removeDirtyNotifier: function(row){
            /*To be implemented by subclasses */
        },

		// Enable autosaving, see WI#25269
        doAutoSave: function(deleteAction) {
            this.editor.showMessage({
                type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
                summary: BEMessages.saving
            });

            var taskMonitor = new rmps.core.TaskMonitor(BEMessages.saving);
            taskMonitor.setCanComplete(false);
            this.doSave(taskMonitor);

            var handle = this.connect(taskMonitor, "onComplete", function(errors) {
                this.disconnect(handle);
                if (errors) {
                    this._handleSaveErrors(errors);
                } else {
                    var summary = deleteAction ? this.linkMessages.link_delete_success : this.linkMessages.link_save_success;
                    this.editor.showMessage({
                        type: jazz.ui.MessageBox.STATUS_TYPES.OK,
                        summary: summary,
                        closable: true
                    });
                }
            });

            taskMonitor.setCanComplete(true);
        },

        _handleSaveErrors: function(errors) {
            var details = [];

            for (var i = 0; i < errors.length; ++i) {
                var msg =  (errors[i] instanceof Error ? errors[i].message : errors[i]);
                details.push({message: dojox.html.entities.encode(msg).replace(/\n/g,"<br>")});
            }

            var messageSummary = details.splice(0, 1)[0].message;
            if (messageSummary.length === 0) {
                messageSummary = this.linkMessages.link_save_failed;
            }
            this.editor.showMessage({
                type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
                summary: messageSummary,
                details: details.length > 0 ? details : null,
                closable: true
            });
        },

        doSave: function(taskMonitor, _dirty){
            /*To be implemented by subclasses */
        },

		_resetLinkTabContents: function () {
        	// Reset the saved data about link modifications (with auto-save, there should not be any)
            var i;
            for (i = 0; i < this._modifiedlinks[this.id]._linksToBeSaved.length; i++) {
                delete this._modifiedlinks[this.id]._linksToBeSaved[i];
            }
            for (i = 0; i < this._modifiedlinks[this.id]._linksToBeDeleted.length; i++) {
                delete this._modifiedlinks[this.id]._linksToBeDeleted[i];
            }
            for (i = 0; i < this._modifiedlinks[this.id]._linksToBeEdited.length; i++) {
                delete this._modifiedlinks[this.id]._linksToBeEdited[i];
            }
            
            // Destroy the outgoing and incoming links sections
			if (this.linksSection) {
                this.linksSection.destroy();
                delete this.linksSection;
                this.linksSection = null;
            }
            if (this.incomingLinksSection) {
                this.incomingLinksSection.destroy();
                delete this.incomingLinksSection;
                this.incomingLinksSection = null;
            }
            this.containerNode.innerHTML = "";
		},
		
		destroy: function(){
            // Reset the contents of the links tab.
            this._resetLinkTabContents();

			delete this._modifiedlinks;
			this._modifiedlinks= null;

			if (this.actions) {
				while (this.actions.length > 0) {
					var entry = this.actions.pop();
					entry = null;
				}
			}
			delete this.actions;
			this.actions= null;

			delete this.editor;
			this.editor= null;

            if(this._securityDeferred){
                this._securityDeferred.cancel();
                delete this._securityDeferred;
            }

            this.inherited(arguments);
        }
    });
})();
