/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("rmps.app.ui.internal.editors.views.LinksEditor");
    dojo.require("rmps.app.ui.internal.editors.views.Links");
    dojo.require("rmps.app.ui.internal.editors.views.link.LinkClient");

    var LinkClient= rmps.app.ui.internal.editors.views.link.LinkClient;

    /* 
     * Specialized implementation for Links widget in our web application
     */

    dojo.declare("rmps.app.ui.internal.editors.views.LinksEditor", rmps.app.ui.internal.editors.views.Links, {
/* Disable notification, see WI#25269
        addDirtyNotifier: function(row){
            this.editor.addDirty(row);
        },

        removeDirtyNotifier: function(row){
            this.editor.removeDirty(row);
        },
*/
    	
    	//extracted from doSave so it can be overrided by subclasses
    	performDelete: function(link){
    		return LinkClient.deleteLink(link);
    	},
    	
        doSave: function(taskMonitor, _dirty){

            try {
                var linksToBeSaved = this._modifiedlinks[this.id]._linksToBeSaved.slice();

	            dojo.forEach(linksToBeSaved, function(row) {
		            var dataSet = this.linksSection.getDataForRow(row);
	                var link = dataSet && dataSet.data ? dataSet.data.linkData : null;
	                if (link) {
	                    var deferred = LinkClient.createLink(link);
	                    var task = taskMonitor.beginTask(dojo.string.substitute("Save Link", [link.resourceUri]), 1);
	                    task.handleDeferredWork(deferred, 1);
	                    deferred.addCallback(this, function(createdLink){
	                        jazz.util.array.removeByReference(this._modifiedlinks[this.id]._linksToBeSaved, row);
/* Disable notification, see WI#25269
	                        this.editor.removeDirty(row);
*/
	                        dojo.removeClass(row, "added-entry");
	                        dojo.removeClass(row, "edited-entry");
	                        dojo.addClass(row, "entry");
	                        if (createdLink.value) {
		                        dataSet.data.value = createdLink.value;
		                        dataSet.data.linkData.value = createdLink.value;
	                        }
	                        this.linksSection.finalizeLinkCreate(dataSet);
	                    });
	                }
	            }, this);

	            var linksToBeDeleted = this._modifiedlinks[this.id]._linksToBeDeleted.slice();

	            dojo.forEach(linksToBeDeleted, function(link){
	                if (link) {
						var link_= this.linksSection.getDataForRow(link).data.linkData;
	                    var deferred = this.performDelete(link_);
	                    var task = taskMonitor.beginTask(dojo.string.substitute("Delete Link", [link.resourceUri]), 1);
	                    task.handleDeferredWork(deferred, 1);
	                    deferred.addCallback(this, function(){
							jazz.util.array.removeByReference(this._modifiedlinks[this.id]._linksToBeDeleted, link);
/* Disable notification, see WI#25269
		                    this.editor.removeDirty(link);
*/
		                    var table = this.linksSection.table;
		                    var index = dojo.indexOf(table.rows, link);
		                    table.deleteRow(dojo.indexOf(table.rows, link));
		                    index = index - 1;
		                    var prev = table.rows[index];
		                    var next = table.rows[index + 1];
		                    var prevData= prev?this.linksSection.getDataForRow(prev).data: null;
		                    var nextData= next?this.linksSection.getDataForRow(next).data:null;
		                    if (prevData.isHeader && (!next||nextData.isHeader)) {
		                        table.deleteRow(index);
		                    }                        
	                    });
	                }
	            }, this);

	            var linksToBeEdited = this._modifiedlinks[this.id]._linksToBeEdited.slice();

				dojo.forEach(linksToBeEdited, function(row){
					var dataSet= this.linksSection.getDataForRow(row);
					var oldLink= dataSet.data.linkData;
					var newLink= dataSet.newLink;
	                var deferred = LinkClient.updateLink(oldLink,newLink);
	                var task = taskMonitor.beginTask(dojo.string.substitute("Update Link", [newLink.resourceUri]), 1);
	                task.handleDeferredWork(deferred, 1);
	                deferred.addCallback(this, function(updatedLink) {
		                jazz.util.array.removeByReference(this._modifiedlinks[this.id]._linksToBeEdited, row);
/* Disable notification, see WI#25269
		                this.editor.removeDirty(row);
*/
		                dojo.removeClass(row, "added-entry");
	                    dojo.removeClass(row, "edited-entry");
	                    dojo.addClass(row, "entry");
                        if (updatedLink.value) {
	                        dataSet.newLink.value = updatedLink.value;
                        }
	                    this.linksSection.finalizeLinkUpdate(dataSet);
	                });
	            }, this);
	        } catch (e) {
                console.debug("*** ERROR: " + e.message);
            }
        }

    });
})();
