/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("rmps.app.ui.internal.editors.views.LinksSection");

    dojo.require("rmps.ui.sections.SectionTable");
    dojo.require("rmps.app.ui.internal.common.ImageService");
    dojo.require("rmps.core.util.BaselineUtil");
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("jazz.ui.ResourceLink");
    dojo.require("rmps.ui.EllipsisDetails");
    dojo.require("dojo.i18n");
    dojo.require("rmps.core.util.ILinksConstants");
    dojo.require("jazz.util.html");
    dojo.require("rmps.app.ui.internal.editors.views.link.LinkClient");
    dojo.require("com.ibm.team.vvc.shared.web.ui.configurations.GlobalConfigurationUtils");

    dojo.requireLocalization("rmps.app.ui.internal.editors.views", "LinksSectionMessages");

    var EllipsisDetails = rmps.ui.EllipsisDetails;
    var ImageService = rmps.app.ui.internal.common.ImageService;
    var ResourceLink = jazz.ui.ResourceLink;
    var ILinksConstants = rmps.core.util.ILinksConstants;
    var LinkClient= rmps.app.ui.internal.editors.views.link.LinkClient;
    var GlobalConfigurationUtils = com.ibm.team.vvc.shared.web.ui.configurations.GlobalConfigurationUtils;

    var LINKS_UAKEY = "com.ibm.xtools.rmps.web.app.section_LinksSection001";

    dojo.declare("rmps.app.ui.internal.editors.views.LinksSection", rmps.ui.sections.SectionTable, {

        resourceInfo: null,
        allowModify: false,   // set by the derived classes ... OutgoingLinksSection

        constructor: function(params){
            var projectName = jazz.app.context.get().name;

            this.messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors.views", "LinksSectionMessages");
            this.addColumnDescriptor({
                columnHeader: "",
                fieldName: "value", /**</br> */
                width: "85%",
                creatorCallback: this.createLinkDefaultCell
            });
        },

        getActionsColumnWidth: function(){
            return "15%";
        },

		/*We do not want to show headers for the links table */
        showHeaders: function(){
            return false;
        },

        createTable: function(args){
            var _table = this.inherited(arguments);

            /*We do not want to have spaces/pads between table cells */
            _table.cellPadding = 0;
            _table.cellSpacing = 0;

            /*Links table does not have border */
            _table.border = 0;

            return _table;
        },

		fillTable: function(section, table, onCall){
            onCall(section, table, this.result);
        },

        insertBefore: function(){
            return false;
        },

        sortData: function(tableData, orderBy){

            function stringCompare(string1, string2){
                return string1.toLocaleLowerCase().localeCompare(string2.toLocaleLowerCase());
            }

            function compare(entry1, entry2){
                var a = dojo.getObject(orderBy, false, entry1);
                var b = dojo.getObject(orderBy, false, entry2);

                if (dojo.isString(a)) {
                    a = a.toLowerCase();
                }
                if (dojo.isString(b)) {
                    b = b.toLowerCase();
                }
                return stringCompare(b, a);
            }

            function comparator(entry1, entry2){
                for (var i = 0; i < orderBy.length; ++i) {
                    var result = compare(entry1, entry2);
                    if (result !== 0) {
                        return result;
                    }
                }
                return 0;
            }

            tableData.sort(comparator);
        },

		/* We need to add our specific conditions on links headers, and rows natures */
        addSectionTableRow: function(section, table, data, index){
            var showExternal= this.showExternalLinks();
            var showCategories= this.showCategories(data);
            if ((data.isHeader && !showCategories) || (data.isExternal && !showExternal)) {
                return null;
            }

            if (!showExternal && data.linkData && data.linkData.predicate === ILinksConstants.EXTERNAL_LINK_TYPE) {
                return null;
            }
            
            var tr = this.inherited(arguments);

			this.styleRow(data, tr);
            return tr;
        },

		/*If is header is false, then we enable the hover color for the row. */
        styleRow: function(data, tr){
            if (data.isHeader) {
				dojo.addClass(tr, "header");
			} else {
				if (data.isNew) {
					dojo.addClass(tr, "added-entry");
				}
				else {
					dojo.addClass(tr, "entry");
				}
			}
        },

        createLinkDefaultCell: function(section, table, tableRow, fieldName, value, data){
			var tableData = dojo.create("td", null, tableRow);
			this.addStyleToTableData(tableData,value, data);
			return this.updateTableDataDescription(tableRow,tableData, data);
        },

        addStyleToTableData: function(td, value, data){
            /*Implemented by subclasses */
        },

        enableDescription: function(){
            new Error("This is an abstract method");
        },

        showExternalLinks: function(){
            return true;
        },

        showCategories: function(data){
            return true;
        },

        appendMoreDetails: function(span, tableRow,tableData, data){
            /*empty block */
        },

        appendArrow: function(span){
            /*Create the link indent image */
            var img = document.createElement("IMG");
            img.src = ImageService.LIST_ARROW;
            span.appendChild(img);
        },

        updateTableDataDescription: function(tableRow,tableData, data) {
            var value= data.value;
            if (data.isHeader) {
                var headerNode = document.createTextNode(value);
                tableData.appendChild(headerNode);
                return tableData;
            }

            // Append oslc.configuration param if in global configuration
            var uri = value;
            if (uri) {
                try {
                	uri = GlobalConfigurationUtils.getExpandedContextLink(value);
            	} catch (e) {
            		// Not in gc so just ignore
            	}
            }
            
            var noDescRequired = (data.predicate === ILinksConstants.EXTERNAL_LINK_TYPE || data.linkTitle === ILinksConstants.EXTERNAL_LINK_TYPE_TITLE);

            var span = document.createElement("SPAN");
            this.appendArrow(span);

            var iconSpan = document.createElement("SPAN");
            dojo.addClass(iconSpan, "icon");
            span.appendChild(iconSpan);

            var anchor = document.createElement("a");
            span.appendChild(anchor);

			/*We do not want to have CR for the external link.
			 * External Link gets its href from the Link value
			 * We also make sure that the html-like desciptions will be escaped
			 */
            var desc;
            if (noDescRequired) {
                /* The value if the URL itself. In case there is a description, we assign it as the new value
                 * so the description would be used as the title */
                desc= data.description;
                if (!desc || desc === "") {
                    desc= value;
                }

                var img = document.createElement("IMG");
                img.src = ImageService.EXTERNAL_LINK;
                iconSpan.appendChild(img);

				anchor.appendChild(dojo.doc.createTextNode(desc));
		        anchor.href = jazz.util.html.escape(uri);
		        //anchor.target = "_blank";  defect 32441 - don't open links in new tabs
		        anchor.style.verticalAlign= "middle";
		    } else {
		        var label= data.label;
	            var displayLabel= jazz.util.html.escape(label?label:value);
	            

	            // Local reference (may be a reference to an element)
	            var rl;
	            if (value.indexOf(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL) === 0) {
	            	var renderingUri = net.jazz.ajax._contextRoot + "/rendering";
	                // Append oslc.configuration param if in global configuration
                    try {
                    	renderingUri = GlobalConfigurationUtils.getExpandedContextLink(renderingUri) + "&resourceUri=" + encodeURIComponent(value);
                    } catch (e) {
                		// Not in gc so just add conceptUri
                		renderingUri = renderingUri + "?resourceUri=" + encodeURIComponent(value);
                	}
	            	
	                rl= new ResourceLink({
		                uri: renderingUri,
		                iconNode: iconSpan,
		                presentationUri: uri,
		                label: displayLabel,
		                avoidBaselineCheck: true
		            }, anchor);

		            this.connect(this, "uninitialize", function() {
						rl.destroyRecursive();
					});
	            } else {
		            rl= new ResourceLink({
		                uri: uri,
		                label: displayLabel,
		                iconNode: iconSpan
		            }, anchor);

		            this.connect(this, "uninitialize", function() {
						rl.destroyRecursive();
					});
	            }
	            
	            /*----- DOORS does not want the presentation label for the target
						resource to be cached because the presentation label is
						based on the permissions of the user logged into DOORS. In 
						other words, user A may see a resource as XXX and user B
						sees the resource as YYY, where YYY contains more info.

	            // When the links tab is adding the existing links, the "data"
	            // parameter is a wrapper for the link resource and contains a linkData"
	            // field. When an existing link is being edited, the "data"
	            // parameter corresponds to the link resource, which defines
	            // the link resource URI, label, and other link properties.
	            var link = data;
	            if (data.linkData) {
					link = data.linkData;
	            }
	            
	            // Connect to the ResourceLink method that updates the link label
	            // when the presentation data has been received from the target
	            // application about the target resource. If this update call
	            // is being made when the link is created, we cannot update
	            // the label because the link resource does not exist yet.
	            if (this.allowModify && link.uri) {
					var self = this;
					var rlListener = self.connect(rl, "_applyTitle", function(newLabel) {
						if (newLabel !== link.label) {
							link.label = newLabel;
							LinkClient.updateLink(link, link);
						}
						
						// Disconnect the listener
						self.disconnect(rlListener);
					});
	            }
	            -----*/
	            
	            if (this.enableDescription()) {
					desc = data.description;
	                if (desc) {
	                    var detailsSpan = null;
	                    detailsSpan = new EllipsisDetails({
	                        target: tableRow,
	                        text: jazz.util.html.escape(desc)
	                    });
	                    detailsSpan.containerNode.style.paddingLeft= "20px";
	                    span.appendChild(detailsSpan.domNode);
	                }
	            }
			}
			this.appendMoreDetails(span, tableRow,tableData, data);
            tableData.appendChild(span);
            return tableData;
        }
    });
})();
