/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("rmps.app.ui.internal.editors.views.OutgoingLinksSection");

dojo.require("rmps.ui.sections.SectionTable");
dojo.require("rmps.app.ui.internal.editors.views.link.LinkClient");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("rmps.app.ui.internal.util.UrlUtil");
dojo.require("rmps.app.ui.internal.editors.views.link.LinkClient");
dojo.require("rmps.app.ui.internal.common.ImageService");
dojo.require("rmps.ui.EllipsisDetails");
dojo.require("rmps.app.ui.internal.sections.LinksMenuHeader");
dojo.require("rmps.app.ui.internal.dialogs.AddExternalLinksDialog");
dojo.require("rmps.app.ui.internal.dialogs.AddCALMLinksDialog");
dojo.require("rmps.app.ui.internal.dialogs.RemoveLinkDialog");
dojo.require("rmps.app.ui.internal.editors.views.LinksSection");
dojo.require("rmps.ui.link.LinkUtil");
dojo.require("rmps.core.util.ILinksConstants");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("dojo.i18n");
dojo.require("rmps.core.RootServices");

dojo.requireLocalization("rmps.app.ui.internal.editors.views", "LinksSectionMessages");

var LinkClient = rmps.app.ui.internal.editors.views.link.LinkClient;
var EllipsisDetails = rmps.ui.EllipsisDetails;
var ImageService = rmps.app.ui.internal.common.ImageService;
var UrlUtil= rmps.app.ui.internal.util.UrlUtil;
var Process= rmps.ui.link.LinkUtil.Process;
var ILinksConstants = rmps.core.util.ILinksConstants;
var IRmpsConstants= rmps.core.util.IRmpsConstants;

var LINKS_UAKEY = "com.ibm.xtools.rmps.web.app.section_LinksSection001";

var LINK_LABEL_MAX_LENGTH= 50;

var REDACT_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "redact");

dojo.declare("rmps.app.ui.internal.editors.views.OutgoingLinksSection", rmps.app.ui.internal.editors.views.LinksSection, {

	resourceInfo: null,
	isReadOnly: true,

    constructor: function(params){
        this.isReadOnly= params.isReadOnly;

        this.report= params.report;

        this.allowCreation = this.report.isAllowed(Process.Actions.CREATE);
		this.allowDeletion = this.report.isAllowed(Process.Actions.DELETE);
		this.allowModify   = this.report.isAllowed(Process.Actions.MODIFY);

        if (!this.isReadOnly) {
            if(this.allowCreation){
                this.addHeader("rmps.app.ui.internal.sections.LinksMenuHeader", {
                    section: this,
                    uaKey: LINKS_UAKEY
	            });				
			}

			if(this.allowModify){
				this.addActionDescriptor(this.createEditAction);				
			}

	        if (this.allowDeletion){
		        this.addActionDescriptor(this.createDeleteAction);
		    }
		    
		    if(params.redactEnabled){
		    	this.addActionDescriptor(this.createRedactAction);
		    }
	    }
    },

    getActionsColumnWidth: function(){
        return "15%";
    },

    /*We have to make sure that the action is aligned to centre so we put it in  a div with 100% width
    * to the table data
    */
    createActionsContainer: function(tableData){
        var div = dojo.doc.createElement("div");
		div.align = "right";
		div.width = "100%";
		return div;
    },

	/*We do not create actions for the headers */
    getActionsEnablement: function(section,table,tr,data){
        return data && !data.isHeader;
    },

	createEditAction: function(section, table, tr, td, div, data){
		if (data && data.linkData && data.linkData.readOnly) {
			return;
		}

        var actionImageCreate= this._createActionImage(this.messages.edit_action, "action",ImageService.EDIT);
        var imgAnchor= actionImageCreate.imageAnchor;
        var img= actionImageCreate.image;
        var scope = this.scope;

        scope.connect(imgAnchor, "onclick", function(e){
            /*to prevent redirection*/
            dojo.stopEvent(e);
            var row = e.currentTarget.parentNode.parentNode.parentNode;

            var className = row.className;

            var dataSet= this.linksSection.getDataForRow(row);
            var tableData=dataSet.cells[0];

			/* If newLink value is defined, then, we get the values from it;
			 *  otherwise, we get it from the existing data
			 */
			var dataToBeLookedAt = dataSet.newLink;
			if (!dataToBeLookedAt) {
				dataToBeLookedAt= dataSet.data.linkData;
			}
			var description= dataToBeLookedAt.description;
			if (!description) {
				description= "";
			}

			var href= dataToBeLookedAt.value;
			if (!href) {
				href= "";
			}

			var mainNode= tableData.childNodes[0];
			
			var text;
			
			var linkNode= {};
			var anchors= mainNode.getElementsByTagName("a");
			for(var index=0; index<anchors.length; index++){
				var current= anchors[index];
				var value= current.textContent ? current.textContent : current.text;
				if(value){
					text= value;
					break;
				}
			}

			
			if(!text){
				text= dataToBeLookedAt.label;
			}
			if(!text){
				text= "";
			}
			var item_= {
				description: description,
				label: text,
				url: href
			};

			var dialog = dataSet.data.linkData.predicate === ILinksConstants.EXTERNAL_LINK_TYPE || dataSet.data.linkTitle === ILinksConstants.EXTERNAL_LINK_TYPE_TITLE ?
						 rmps.app.ui.internal.dialogs.AddExternalLinksDialog :
						 rmps.app.ui.internal.dialogs.AddCALMLinksDialog;

			var onOk = function(links) {
				var link = links[0];
				/*Check if we retuned back to the original input values */
				var data = dataSet.data;
				if (!data.isNew) {
					var value = data.value;
					if (!value) {
						value = "";
					}
					var desc = data.description;
					if (!desc) {
						desc = "";
					}
					if (link.description === desc && link.url === value) {
/* Disable notification, see WI#25269
						scope.removeDirtyNotifier(row);
*/
						scope._modifiedlinks[scope.id]._linksToBeEdited.splice(dojo.indexOf(scope._modifiedlinks[scope.id]._linksToBeEdited, (row)),1);
						section._updateLinkForRow(row, link, data.linkData, dataSet);
						dojo.removeClass(row, "added-entry");
	                    dojo.removeClass(row, "edited-entry");
	                    dojo.addClass(row, "entry");
						return;
					}
				}

				/*Well, we checked */
				var deleteAction = dataSet.actionNodes.createDeleteAction;
				if (deleteAction) {
					var editImage = deleteAction.children[0];
					editImage.src = ImageService.REMOVE;
					editImage.title = scope.linksSection.messages.remove;	
				}

				/* TODO: Fade out the actions that are not executable */
                if (dojo.hasClass(row, "deleted-entry")) {
                    dojo.removeClass(row, "deleted-entry");
/* Disable notification, see WI#25269
                    scope.removeDirtyNotifier(row);
*/
					scope._modifiedlinks[scope.id]._linksToBeDeleted.splice(dojo.indexOf(scope._modifiedlinks[scope.id]._linksToBeDeleted, (row)),1);
                }
				section._updateLinkForRow(row, link, data.linkData, dataSet);
				dojo.removeClass(row, "entry");
	            dojo.addClass(row, "edited-entry");
				if (dojo.indexOf(scope._modifiedlinks[scope.id]._linksToBeSaved, row) === -1) {
/* Disable notification, see WI#25269
					scope.addDirtyNotifier(row);
*/
					scope._modifiedlinks[scope.id]._linksToBeEdited.push(row);
				}
				if (dialog) {
					dialog.destroyRecursive();
				}
				scope.doAutoSave(false);
            };
			
			var label= item_.label;
			if(label.length>LINK_LABEL_MAX_LENGTH){
				label= label.substring(0,LINK_LABEL_MAX_LENGTH-3)+"...";
			}
			
			var params = {
				item: item_,
	            dialogTitle: dojo.string.substitute(section.messages.edit_Link, [label]),
	            dialogSecondaryTitle: dataSet.data.linkTitle,
	            onOk: onOk,
	            resourceUri: this.resourceInfo.uri
	        };
			dialog = new dialog(params);
        });

		div.appendChild(imgAnchor);
        return imgAnchor;
    },

    enableDescription: function(){
		return true;
    },

    finalizeLinkCreate: function(dataSet) {
        if (dataSet.data.value.indexOf('?') > 0) {
            // An internal link was created and uri of link object was updated by server 
            var tableData = dataSet.cells[0];
            /* remove the current Span */
            tableData.removeChild(tableData.childNodes[0]);
            /* add the new Span */
            tableData = this.updateTableDataDescription(dataSet.row, tableData, dataSet.data.linkData);
        }
        delete dataSet.data.linkData.isNew;
        delete dataSet.data.isNew;
    },

    finalizeLinkUpdate: function(dataSet){
		var editImage= dataSet.actionNodes.createEditAction.children[0];
        editImage.src = ImageService.EDIT;
		editImage.title = this.messages.edit;
		dataSet.data.linkData= dataSet.newLink;
		dataSet.data.description= dataSet.newLink.description;
		dataSet.data.value= dataSet.newLink.value;
        if (dataSet.data.value.indexOf('?') > 0) {
            // An internal link was updated and uri of link object was updated by server
            var tableData = dataSet.cells[0];
            /* remove the current Span */
            tableData.removeChild(tableData.childNodes[0]);
            /* add the new Span */
            tableData = this.updateTableDataDescription(dataSet.row, tableData, dataSet.data.linkData);
        }
		delete dataSet.newLink;
        delete dataSet.data.linkData.isNew;
        delete dataSet.data.isNew;
    },

    _updateLinkForRow: function(row, updatedLinkData, currentLink, dataSet) {
        var tableData= dataSet.cells[0];
        /*In order to be able to retrieve the new link value */
		dataSet.newLink = this._updateLink(tableData, row, updatedLinkData, currentLink);
    },

    _updateLink: function(tableData, row, updatedLinkData, currentLink){
        var newLink = this._updateLinkObject(updatedLinkData, currentLink);

		/*remove the current Span */
        tableData.removeChild(tableData.childNodes[0]);

        /*add the new Span */
        tableData = this.updateTableDataDescription(row, tableData, newLink);
        return newLink;
    },
    
    createRedactAction: function(section, table, tr, td, div, data){
        if (data && data.linkData && data.linkData.readOnly) {
            return;
        }

        var actionImageCreate= this._createActionImage(this.messages.redact_action, "action", dojo.moduleUrl("rmps.app", "graphics/icons/redact.gif"));
        var imgAnchor= actionImageCreate.imageAnchor;
        var img= actionImageCreate.image;
        var scope = this.scope;
        
        
        
        scope.connect(imgAnchor, "onclick", function(e){
        	/*to prevent redirection*/
        	dojo.stopEvent(e);
        	
        	var row = e.currentTarget.parentNode.parentNode.parentNode;
            var dataSet= this.linksSection.getDataForRow(row);
            linkData = dataSet.data.linkData;
    
			var query = {
				conceptURI: linkData.uri 
			};
			var args = {
					url: REDACT_PATH + "?" + dojo.objectToQuery(query),
					headers: {
						"X-ibm-rmps-internal" : "true"
					},
					load: dojo.hitch(this, function(response, ioArgs) {
						if (scope.doRefresh) {
  							scope.doRefresh();
						}
					})
			}; // end args
			rmps.core.client.xhrPost(args);	
        });
		
		div.appendChild(imgAnchor);
		
        return imgAnchor;
    },

    createDeleteAction: function(section, table, tr, td, div, data){
        if (data && data.linkData && data.linkData.readOnly) {
            return;
        }

        var imgAnchor = document.createElement("A");
        imgAnchor.href = "#";
        var img = dojo.create("IMG", {
            align: "centre"
        });
        img.title = this.messages.remove;
        dojo.addClass(img, "action");
        img.src = ImageService.REMOVE;
        img.style.verticalAlign = "middle";
        imgAnchor.style.verticalAlign = "middle";

        imgAnchor.appendChild(img);

        var scope = this.scope;

        scope.connect(imgAnchor, "onclick", function(e){
            /*to prevent redirection*/
            dojo.stopEvent(e);
            var row = e.currentTarget.parentNode.parentNode.parentNode;
            var className = row.className;
            var index;
            /*TODO: Fade out the actions that are not executable */
            if (dojo.hasClass(row, "deleted-entry")) {
                // Temporary unreachable code, see WI#25269
                dojo.removeClass(row, "deleted-entry");
                if (!dojo.hasClass(row, "edited-entry")) {
					dojo.addClass(row, "entry");
                }
                img.src = ImageService.REMOVE;
				img.title = "Delete";
                scope.removeDirtyNotifier(row);
                scope._modifiedlinks[scope.id]._linksToBeDeleted.splice(dojo.indexOf(scope._modifiedlinks[scope.id]._linksToBeDeleted, (row)),1);
            } else if (dojo.hasClass(row, "added-entry")) {
                // Temporary unreachable code, see WI#25269
                var table = this.linksSection.table;
                index = dojo.indexOf(table.rows, row);
                table.deleteRow(index);
                index = index - 1;
                var prev = table.rows[index];
                var next = table.rows[index + 1];
                var prevData= prev?section.getDataForRow(prev).data: null;
                var nextData= next?section.getDataForRow(next).data:null;
                if (prevData.isHeader && (!next||nextData.isHeader)) {
                    table.deleteRow(index);
                }
                scope.removeDirtyNotifier(row);
                scope._modifiedlinks[scope.id]._linksToBeSaved.splice(dojo.indexOf(scope._modifiedlinks[scope.id]._linksToBeSaved, (row)),1);
            } else if (dojo.hasClass(row, "entry") || dojo.hasClass(row, "edited-entry")) {
                var linkData = section.getDataForRow(row).data.linkData;
                var deleteFile = linkData.label ? linkData.label : linkData.description ? linkData.description : linkData.uri;  
                
                new rmps.app.ui.internal.dialogs.RemoveLinkDialog({
                    files: [],
                    deleteFile: deleteFile,
                    uri: linkData.uri, 
                    onOk: dojo.hitch(this, function() {
                        var i;

                        dojo.addClass(row, "deleted-entry");
                        dojo.removeClass(row, "entry");
                        i = dojo.indexOf(scope._modifiedlinks[scope.id]._linksToBeSaved, (row));
                        if (i > -1) {
                            scope._modifiedlinks[scope.id]._linksToBeSaved.splice(i, 1);
                        }
                        i = dojo.indexOf(scope._modifiedlinks[scope.id]._linksToBeEdited, (row));
                        if (i > -1) {
                            scope._modifiedlinks[scope.id]._linksToBeEdited.splice(i, 1);
                        }
                        scope._modifiedlinks[scope.id]._linksToBeDeleted.push(row);
                        scope.doAutoSave(true);
                    }, scope, row)});
/* Temporary commented out, see WI#25269
                dojo.addClass(row, "deleted-entry");
                dojo.removeClass(row, "entry");
                img.src = ImageService.RESTORE;
				img.title = "Restore";
				index= dojo.indexOf(scope._modifiedlinks[scope.id]._linksToBeSaved, (row));

				if(index>-1){
					scope._modifiedlinks[scope.id]._linksToBeSaved.splice(index,1);
				}

				index= dojo.indexOf(scope._modifiedlinks[scope.id]._linksToBeEdited, (row));
				if(index>-1){
					scope._modifiedlinks[scope.id]._linksToBeEdited.splice(index,1);
				}

                scope.addDirtyNotifier(row);
				scope._modifiedlinks[scope.id]._linksToBeDeleted.push(row);
				scope.doAutoSave(true);
*/
            }
        });

		div.appendChild(imgAnchor);

        return imgAnchor;
    },

    _createActionImage: function(title, claaName, src){
        var imgAnchor = document.createElement("A");
        imgAnchor.href = "#";
        var img = dojo.create("IMG", {
            align: "centre"
        });
        img.title = title;
        dojo.addClass(img, claaName);
        img.src = src;
        img.style.verticalAlign = "middle";
        imgAnchor.style.verticalAlign = "middle";

        imgAnchor.appendChild(img);
        return {
            imageAnchor: imgAnchor,
            image: img
        };
    },

    _handleAddLinkButtonPressed: function(action, section, table){
        if (!action) {
            return;
        }
        var curDialog = action.dialog;
        if (curDialog) {
            var callBack = curDialog.deferred;
            if (callBack) {
                /*Cancel the last action if the user clicked on the add one more time */
                callBack.cancel();
                action.dialog.destroyRecursive();
                delete action.dialog;
            }
        }

        var self = this;
        var actionType = action.value.identifier;
        var descriptipon;
        var dialog;
        if (ILinksConstants.EXTERNAL_LINK_TYPE === actionType) {
			dialog= rmps.app.ui.internal.dialogs.AddExternalLinksDialog;
		}else {
			dialog= rmps.app.ui.internal.dialogs.AddCALMLinksDialog;
		}

		var params = {
            dialogTitle: this.messages.add_Link,
            dialogSecondaryTitle: action.label,
            resourceUri: this.resourceInfo.uri,
            onOk: function(items){
            	var linksAdded = false;
                for (var i = 0; i < items.length; i++) {
                	var item = items[i];
                	var url= item.url;
                	/*If Url is undefined, thus there is no Friend Servers have been confgiured yet */
                	if (url) {
						var newLink= self._createLinkObject(actionType, item);
						self._addNewRow(table, newLink, action.label, item);
						linksAdded = true;
					}
                }
                // If any new links were added, save them
                if (linksAdded) {
                	self.scope.doAutoSave(false);
                }
            },
            onClose: function(){
                action.dialog.destroyRecursive();
                delete action.dialog;
                action.dialog = null;
            }
        };

        /*Open the dialog */
        action.dialog = new dialog(params);
    },

    _createLinkObject:function(actionType,item){
        var newLink = new rmps.app.ui.internal.editors.views.link.Link();
        newLink.subject = this.resourceInfo.uri;
        newLink.predicate = actionType;
        /*We have to make sure that the entered url is well formed */
        newLink.value = item.url;
        /*If the comment is empty, then we display the same URL the user entered */
        newLink.description = item.description;
        newLink.label = item.label;
        newLink.isNew = true;
        return newLink;
    },

    _updateLinkObject: function(updatedLinkData, currentLink) {
        var newLink;
        if (!currentLink) {
            newLink = new rmps.app.ui.internal.editors.views.link.Link();
        } else if (currentLink.isNew) {
            newLink = currentLink;
        } else {
			newLink = dojo.clone(currentLink);
        }
        newLink.value = updatedLinkData.url;
        newLink.description = updatedLinkData.description;
        newLink.label = updatedLinkData.label;

        newLink.isNew = true;
		return newLink;
    },

    _addNewRow: function(table, newLink, label, item){
        var index = -1;
        var rows = table.rows;
        for (var i = 0; i < rows.length; i++) {
            var row = rows[i];
			var data= this.getDataForRow(row).data;
		    if (data.isHeader && data.linkTitle === label) {
                index = i;
                break;
            }
        }

        if (index === -1) {
            index = 0;
            this.addSectionTableRow(this, table, {
                value: label,
                linkTitle: label,
                isHeader: true
            }, index);
        }
		var linkData = {
            value: newLink.value,
            linkTitle: item.linkTitle,
            description: item.description,
            label: item.label,
            isHeader: false,
            isNew: true,
            linkData: newLink
        };
        var tr = this.addSectionTableRow(this, table, linkData, index + 1);

        /*Add the link to the pending changes to be executed by the save button */
		var scope= this.scope;
/* Disable notification, see WI#25269
        scope.addDirtyNotifier(tr);
*/
        scope._modifiedlinks[scope.id]._linksToBeSaved.push(tr);
    }
});

})();
