/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.views.RelatedElements");

dojo.require("dojo.i18n");

dojo.require("rmps.app.ui.internal.search.SearchResults");
dojo.require("rmps.app.core.internal.query.ClientPagedQueryResult");
dojo.require("rmps.ui.Table");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("rmps.core.query.QueryClient");
dojo.require("rmps.core.error");
dojo.require("rmps.core.Enum");

dojo.require("rmps.app.core.internal.query.RelationshipTextQuery");

dojo.require("jazz.app.ua.PageHoverHelp");

dojo.require("rmps.core.client");
dojo.require("rmps.core.RootServices");
var SEARCH_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "dm-search");

dojo.requireLocalization("rmps.app.ui.internal.editors.views", "RelatedElementsMessages");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var QueryClient = rmps.core.query.QueryClient;

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors.views", "RelatedElementsMessages");

var UAKEY = "com.ibm.xtools.rmps.web.app.page_RelatedElements001";

dojo.declare("rmps.app.ui.internal.editors.views.RelatedElements", rmps.app.ui.internal.search.SearchResults, {

	// mixin properties
	resourceInfo: null,
	uaKey: UAKEY,
	tableClass: "rmps.ui.Table",
	
	// properties

	Queries: new rmps.core.Enum({
//		CONTAINED_ELEMENTS_OF_ELEMENT: new rmps.core.EnumLiteral({name: messages.containedElements, value: 'containedElementsOfElement'}),
		REFERENCED_ELEMENTS_OF_ELEMENT: new rmps.core.EnumLiteral({name: messages.referencedElements, value: 'referencedElementsOfElement'}),
		REFERENCING_ELEMENTS_OF_ELEMENT: new rmps.core.EnumLiteral({name: messages.referencingElements, value: 'referencingElementsOfElement'}),
		REFERENCING_DIAGRAMS_OF_ELEMENT: new rmps.core.EnumLiteral({name: messages.referencingDiagrams, value: 'referencingDiagramsOfElement'}),
		
		REFERENCED_ELEMENTS_OF_DIAGRAM: new rmps.core.EnumLiteral({name: messages.referencedElements, value: 'referencedElementsOfDiagram'}),
		REFERENCED_DIAGRAMS_OF_DIAGRAM: new rmps.core.EnumLiteral({name: messages.referencedDiagrams, value: 'referencedDiagramsOfDiagram'})
	}),

	// private properties
	_query: null,
	_selectBox: null,
	
	_blankOption: null,
	
	postCreate: function() {
		if (!this.resourceInfo) {
			this.resourceInfo = this.editor.editorInput.resourceInfo;
		}
		
		this.containerNode.style.display = "none";
		this.containerNode.style.overflow="auto";
		this.inherited(arguments);
		this.toolbar.domNode.style.display = "none";
	},
	
	destroy: function() {
		if (this._ua) {
			this._ua.destroyRecursive();
		}

		this.inherited(arguments);
	},
	
	createHeader: function(parentNode) {
		dojo.addClass(parentNode, "rmps-app-editors-views-related-elements-header");
		var label = dojo.doc.createElement("label");
		label.className = "label";
		dojo.attr(label,"for",this.id+ "_relationship"); 
		label.appendChild(dojo.doc.createTextNode(messages.relationship));
		parentNode.appendChild(label);
		
		this._selectBox = dojo.doc.createElement("select", {"id":"${id}_relationship"});
		this._selectBox.id= this.id+ "_relationship";
		this._blankOption = this.addOption(messages.selectRelationship);

		var self = this;
		var def = rmps.core.client.xhrGet({
				url: SEARCH_PATH + "/queries",
				content: { "resourceId": this.resourceInfo.uri },
				handleAs: 'json',
				headers: {
					"Accept" : "application/json"
				},
				load: function(response, ioArgs) {
					if (response && response.length) {
						for (var i=0,len = response.length; i<len; ++i) {
							self.addOption(response[i].name, response[i].id);
						}
					}
				}
			});
			
		parentNode.appendChild(this._selectBox);

		if (this.uaKey) {
			this._ua = new jazz.app.ua.PageHoverHelp({uaKey: this.uaKey});
			parentNode.appendChild(this._ua.domNode);
			this._ua.domNode.style.margin = "0px 0px 0px 5px";
			this._ua._helpImg.alt= messages.help;
		}
		
		this.connect(this._selectBox, "onchange", this._onQueryChange);
	},
	
	addOption: function(name, value) {
		var option = dojo.doc.createElement("option");
		if (value) {
			option.value = value;
		}
		option.appendChild(dojo.doc.createTextNode(name));
		this._selectBox.appendChild(option);
		return option;
	},
	
	onRefreshClick: function(e) {
		this.executeQuery(this.getQueryResult().query, true);
	},
	
	showResults: function(queryResult, preserveFilter) {
		this.inherited(arguments);
		this.containerNode.style.display = "";
		this.toolbar.domNode.style.display = "";
	},
	
	executeQuery: function(query, preserveFilter) {
		PlatformUI.getWorkbench().showMessage(
				dojo.string.substitute(messages.runningQuery, [query.name]));
		
		var deferred = QueryClient.execute(query);
				
		deferred.addCallback(this, function(queryResult) {
			if (queryResult instanceof Object) {
				this._prevQueryResult = new rmps.app.core.internal.query.ClientPagedQueryResult({
					query: query,
					id: queryResult.id,
					rawEntries: queryResult.entries,
					totalResults: queryResult.totalResults,
					uri: queryResult.uri
				});				
				this.showResults(this._prevQueryResult, preserveFilter);
			}
		});
		
		deferred.addBoth(function() {
			PlatformUI.getWorkbench().hideMessageArea();
		});
		
		deferred.addErrback(function(error) {
			if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
				rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
			} else {
				rmps.core.error.createError(error).dispatch();
			}
		});
	},
	
	_onQueryChange: function() {
		if (this._blankOption) {
			this._selectBox.removeChild(this._blankOption);
			delete this._blankOption;
		}
		this.onQueryChange(this._selectBox.value);
	},
	
	onQueryChange: function(queryId) {
		var query = new rmps.app.core.internal.query.RelationshipTextQuery();
		query.relationship = queryId;
		query.uri = this.getElementToQueryOn();
		query.pageSize = 50;
		query.orderBy = [{id: "modified", isAscending: false}, {id: "name", isAscending: false}];
		query.context = [ dojo.getObject("jazz.app.currentApplication").context ];
		query.name = this.Queries.get(queryId).name;
		query.baseline = jazz.app.context.get().baseline;
		query.changeset = jazz.app.context.get().changeset;
		this.executeQuery(query);
	},
	
	getElementToQueryOn: function() {
		return this.resourceInfo.uri;
	},
	
	_createAndShowModifiedQuery: function(query) {
		if (this._prevQueryResult) {
			this._prevQueryResult = new rmps.app.core.internal.query.ClientPagedQueryResult({
				query: query,
				id: this._prevQueryResult.id,
				rawEntries: this._prevQueryResult.rawEntries,
				totalResults: this._prevQueryResult.totalResults,
				uri: this._prevQueryResult.uri,
				filtered: this._prevQueryResult.filtered
			});
			this.showResults(this._prevQueryResult, true);
		}		
	},
	
	_getPreviousQuery: function() {
		var q = this._prevQueryResult && this._prevQueryResult.query;
		return q ? dojo.clone(q) : null;
	},
	
	_onChangePageSize: function(page, newPagesize) {
		var query = this._getPreviousQuery();
		if (query) {
			query.pageSize = newPagesize;
			query.page = page;
			this._createAndShowModifiedQuery(query);
		}
	},
	
	_onNextPage: function() {
		var query = this._getPreviousQuery();
		if (query) {
			query.page = 1 + query.page;
			this._createAndShowModifiedQuery(query);
		}
	},
	
	_onPrevPage: function() {
		var query = this._getPreviousQuery();
		if (query) {		
			query.page = -1 + query.page;
			this._createAndShowModifiedQuery(query);
		}
	}
});

}());
