/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.views.explorerviewpoint.ChildQueriesTab");

dojo.require("rmps.ui.queryeditor.PopupButton");
dojo.require("dojo.DeferredList");
dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("jazz.app.context");

dojo.require("rmps.app.ui.internal.editors.views.explorerviewpoint.ViewpointSegment");
dojo.require("rmps.core.explorer.Viewpoint");
dojo.require("rmps.ui.QuerySection");
dojo.require("rmps.ui.queryeditor.WidgetDataStore");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.ui.internal.editors", "ExplorerViewpointEditorMessages");

var TYPE_WIDGET_ID = "dm.query.ontology.type";
var ViewpointUtilities = rmps.core.explorer.ViewpointUtilities;

var OPERATION_UNION_ID = "union";
var OPERATION_INTERSECTION_ID = "intersection";
var OPERATION_NONE_ID = "none";

dojo.declare("rmps.app.ui.internal.editors.views.explorerviewpoint.ChildQueriesTab", [dijit._Widget, dijit._Templated],{
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("rmps.app", "ui/internal/editors/views/explorerviewpoint/templates/ChildQueriesTab.html"),
	messages: dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "ExplorerViewpointEditorMessages"),
	
	// mixin
	childQueries: null, //[]
	
	_typeSections: null, //[]
	
	_mapUriToTypeEntry: null,
	
	_typeMappingDeferred: new dojo.Deferred(),
		
	constructor: function() {
		if (!this.childQueries) {
			this.childQueries = [];
		}
		this._typeSections = [];
		this._mapUriToTypeEntry = [];
	},
	
	postCreate: function() {
		this.inherited(arguments);
		
		// Populate the add child query button
		// TODO: If root query is moved to the same tab, then reuse its data store
		var projectID = jazz.app.context.get().itemId; 
		var store = new rmps.ui.queryeditor.WidgetDataStore({
			inputData: {
				projectID: projectID,
				localAndDependantOntologies: true
			}
		});
		
		if (this.childQueries) {
			dojo.forEach(this.childQueries, function(childQuery) {
				// TODO: Get the type name for the section title
				var type = ViewpointUtilities.getChildQueryType(childQuery);
				var operation = ViewpointUtilities.getChildQueryOperation(childQuery);
				var queryUri = ViewpointUtilities.getChildQueryUri(childQuery);
				var groups = ViewpointUtilities.getChildQueryGroups(childQuery);
				this.createTypeSection(type, operation, queryUri, groups);
			}, this);
		}		

		
		store.getWidgetData(TYPE_WIDGET_ID).addCallback(this, function(data) {
			
			var list = [];
			var mapping = {};
			dojo.forEach(data, function(type) {
				var entry = {
						id: type.id,
						label: type.label
				}; 
				list.push(entry);
				mapping[type.id] = type.label;
				this._mapUriToTypeEntry[type.id] = entry;
			}, this);
			this._typesList = list;
			//this._typeButton.populateDropDown(list);
			// Let the whole button open the menu
			this._typeButton.callback = dojo.hitch(this, this._editChildQueries);
			// Remove loading message
			this._typeButton._label.innerHTML = this.messages.addTypeButtonLabel;
			this._typeButton.setDisabled(false);
			
			this._typeMappingDeferred.callback(mapping);
		});
		
	},
	
	_populateAddTypeButtonCallback: function(event, item) {
		this.addTypeSection(item.id, item.label);
	},
	
	_addTypeSection: function(type, section) {
		this._typeSections.push({
			type: type,
			section: section
		});
	},
	
	_getTypeSection: function(type) {
		var typesection = dojo.filter(this._typeSections, function(typesection) {
			if (typesection.type === type) {
				return true;
			}
			return false;
		});
		if (typesection.length > 0) {
			return typesection[0].section;
		}
		return null;	
	},
	
	_removeTypeSection: function(type) {
		for (var ix in this._typeSections) {
			if (this._typeSections[ix].type === type) {
				this._typeSections.splice(ix, 1);
				break;
			}
		}
	},
	
	addTypeSection: function(type, typeName) {	
		var typeSection = this._getTypeSection(type);
		if (typeSection) {
			// TODO: scroll to section
			return;
		}
		
		typeSection = this.createTypeSection(type, "", []);
		typeSection.markDirty("addSection");
	},
	
	removeTypeSection: function(type) {
		var typeSection = this._getTypeSection(type);
		if (!typeSection) {
			return;
		}
		
		typeSection.markDirty("removeSection");
		
		this._removeTypeSection(type);
		this._sections.removeChild(typeSection.domNode);
		// TODO: Proper destruction
		//typeSection.destroy();
	},
	
	createTypeSection: function(type, operation, queryUri, groups) {
		var typeNameDeferred = new dojo.Deferred();
		this._typeMappingDeferred.addCallback(function (typeMapping) {
			typeNameDeferred.callback(typeMapping[type]);
		});
		
		// Create a new section
		var typeSection = new rmps.app.ui.internal.editors._ChildQuerySection({
			type: type,
			typeName: typeNameDeferred,
			operation: operation,
			queryUri: queryUri,
			queryWidgetInputData: dojo.toJson({
				typeURI: type
			}),
			editor: this.editor,
			tab: this,
			groups: groups
		});
		this._addTypeSection(type, typeSection);
		
		this._sections.appendChild(typeSection.domNode);
		typeSection.startup();
		
		return typeSection;
	},
	
	doSave: function(taskMonitor, synchronous) {
		var defs = [];
		dojo.forEach(this._typeSections, function(typeSection) {
			defs.push(typeSection.section.doSave(taskMonitor, synchronous));			
		});
		return new dojo.DeferredList(defs);
	},
	
	validate: function(){
		return dojo.map(this._typeSections, function(typeSection) {
			return typeSection.section.validate();
		});
	},
	
	_editChildQueries: function() {
		var existingChildQueries = {};
		for (var i = 0; i < this._typeSections.length; ++i) {
			existingChildQueries[this._typeSections[i].type] = true;
		}
		
		var unselectedEntries = [];
		var selectedEntries = [];
		for (i = 0; i < this._typesList.length; ++i) {
			if (existingChildQueries[this._typesList[i].id]) {
				selectedEntries.push(this._typesList[i]);
			} else {
				unselectedEntries.push(this._typesList[i]);
			}
		}
		
		var dialog = new jazz.ui.SwapListDialog({
			dialogTitleText: this.messages.selectTypesDialogTitle,
			unselectedListTitleText: this.messages.unselectedTypesLabel,
			selectedListTitleText: this.messages.selectedTypesLabel,
			sortField: "label",
			iconMap: [],
			onOK: dojo.hitch(this, function(dialog) {
				//this._unselectedEntries = dialog.getUnselectedEntries();
				var selected = dialog.getSelectedEntries();
				var selectedObj = {};
				for (var i = 0; i < selected.length; ++i) {
					selectedObj[selected[i].id] = true;
					if (!existingChildQueries[selected[i].id]) {
						this.createTypeSection(selected[i].id, "", []);
					}
				}
				for (i = 0; i < this._typeSections.length; ++i) {
					if (!selectedObj[this._typeSections[i].type]) {
						this.removeTypeSection(this._typeSections[i].type);
					}
				}
				
				dialog.destroy();
			}),
			onCancel: function(dialog) {
				dialog.destroy();
			}
		});
		
		dialog.setUnselectedEntries(unselectedEntries);
		dialog.setSelectedEntries(selectedEntries);
		dialog.show();
		if (dialog._multiSelectArea && dialog._multiSelectArea._unselectedFilterBox) {
			dialog._multiSelectArea._unselectedFilterBox.focus();
		}
	}
});

dojo.declare("rmps.app.ui.internal.editors._ChildQuerySectionTitle", [dijit._Widget, dijit._Templated], {
	templateString: '<span><span data-dojo-attach-point="titleNode"></span><span data-dojo-attach-point="menuNode"></span></span>',
	messages: dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "ExplorerViewpointEditorMessages"),
	
	postCreate: function () {
		this.menuNode.appendChild(this.menu.domNode);
		this._setTitle(this.title);
		if (this.tooltip) {
			this._setToolTip(this.tooltip);
		}
	},
	
	_setToolTip: function (tooltip) {
		if (tooltip instanceof dojo.Deferred) {
			this._setToolTip(this.messages.loading);
			tooltip.addCallbacks(
				dojo.hitch(this, "_setToolTip"),
				dojo.hitch(this, "_setToolTip")
			);
		} else {
			this.titleNode.title = tooltip;
		}
	},
	
	_setTitle: function (title) {
		if (title instanceof dojo.Deferred) {
			this._setTitle(this.messages.loading);
			title.addCallbacks(
				dojo.hitch(this, "_setTitle"),
				dojo.hitch(this, "_setTitle")
			);
		} else if (title instanceof Node) {
			this.titleNode.appendChild(title.domNode);
		} else {
			this.titleNode.textContent = title;
		}
	}
});

dojo.declare("rmps.app.ui.internal.editors._ChildQuerySection", [dijit._Widget, dijit._Templated], {
	
	
	messages: dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "ExplorerViewpointEditorMessages"),
	// mix-in
	editor: null,
	tab: null,
	type: null,
	operation: null,
	typeName: null,
	queryUri: null,
	queryWidgetInputData: "{}", // json
	groups: [],
/*
	UNION_ID: "union_id",
	INTERSECTION_ID: "intersection_id",
	NONE_ID: "none_id",
*/
	_viewpointSegment: null,
	
	widgetsInTemplate: true,
	templateString: '<div>' +
			'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_section">' +
			'<div dojoAttachPoint="_segment">' +
		'</div>',
		
	titleNode: null,
		
	postCreate: function() {
		var _menuItems = [{
			id: OPERATION_UNION_ID,
			label: this.messages.union,
			tooltip: this.messages.union_hint
		}, {
			id: OPERATION_INTERSECTION_ID,
			label: this.messages.intersection,
			tooltip: this.messages.intersection_hint
		},{
			id: OPERATION_NONE_ID,
			label: this.messages.none,
			tooltip: this.messages.none_hint
		}];		
		var menu = new rmps.ui.queryeditor.PopupButton({
			labelProperty: "label",
			tooltipProperty: "tooltip",
			callback: dojo.hitch(this, this._menuCallBack)
		});		
		menu.setValues(_menuItems);
		if (!this.operation) {
			this.operation = OPERATION_NONE_ID;
		}

		for (var i = 0; i < _menuItems.length; i++) {
			if (_menuItems[i].id === this.operation) {
				menu.setSelection(_menuItems[i]);
			}
		}

		this._viewpointSegment = new rmps.app.ui.internal.editors.views.explorerviewpoint.ViewpointSegment({
			queryUri: this.queryUri,
			queryWidgetInputData: this.queryWidgetInputData,
			groups: this.groups,
			editor: this.editor
		});
		
		dojo.style(menu.domNode, {padding: "15px"}); 
		
		this.titleNode = new rmps.app.ui.internal.editors._ChildQuerySectionTitle({title: this.typeName, menu: menu, tooltip: this.typeName});
		this._section.setPrimaryTitle(this.titleNode.domNode);

		this._segment.appendChild(this._viewpointSegment.domNode);

		if (!this.editor.isReadOnly) {
			// create Close command
			this._section.addAction(this.messages.close, this.messages.close, dojo.hitch(this, function() {
				this.tab.removeTypeSection(this.type);
			}));
		}		
	},
	
	_menuCallBack: function(item) {
		this.operation = item.id;
		this.markDirty("operation");
	},
	
	destroy: function() {
		this._viewpointSegment.destroy();
		
		this.inherited(arguments);
	},
	
	markDirty: function(key) {
		this.editor.addDirty(key);
	},
	
	doSave: function(taskMonitor, synchronous) {
		var newdef = new dojo.Deferred();
		var def = this._viewpointSegment.doSave(taskMonitor, synchronous);
		def.addCallback(this, function(results) {
			var query = null;
			var groups = [];
			dojo.forEach(results, function(result) {
				if (result[0]) { // Success?
					var node = result[1];
					if (ViewpointUtilities.isGroup(node)) {
						groups.push(node);	
					} else {
						// Not a group, so this is the query
						query = node;
					}
				}
			});
			// Create node definition
			var queryUri = query ? query.uri : null;
			var nodedef = ViewpointUtilities.createChildQuery(this.type, this.operation, queryUri, groups);
			newdef.callback(nodedef);
		});
		def.addErrback(function(results) {
			newdef.errback(results);
		});
		
		return newdef;
	},
	
	validate: function(){
		return this._viewpointSegment.validate(false /* rootTab */);
	}
	

});

})();