/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("rmps.app.ui.internal.editors.views.explorerviewpoint.ViewpointSegment");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("jazz.ui.Button");
dojo.require("rmps.core.explorer.Viewpoint");
dojo.require("rmps.ui.QuerySection");
dojo.require("rmps.ui.queryeditor.QueryEditorWidget");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.ui.internal.editors", "ExplorerViewpointEditorMessages"); 

var ViewpointUtilities = rmps.core.explorer.ViewpointUtilities;

dojo.declare("rmps.app.ui.internal.editors.views.explorerviewpoint.ViewpointSegment", [dijit._Widget, dijit._Templated],{
	// mix-in
	queryUri: null,
	queryWidgetInputData: "{}", // json
	groups: null, //[]
	editor: null,
	
	_querySection: null,
	_groupSections: null, //[]
	
	widgetsInTemplate: true,
	templatePath: dojo.moduleUrl("rmps.app", "ui/internal/editors/views/explorerviewpoint/templates/ViewpointSegment.html"),
	messages: dojo.i18n.getLocalization("rmps.app.ui.internal.editors", "ExplorerViewpointEditorMessages"),

	constructor: function() {
		this.groups = [];
		this._groupSections = [];
	},
	
	postCreate: function() {
		this._addGroupButton.callback = dojo.hitch(this, this._onAddGroup);
		this._addQueryButton.callback = dojo.hitch(this, this._onAddQuery);
		
		// Add the query section
		// queryUri being null represents the case when there is no query
		if (this.queryUri != null && this.queryUri != "") {
			this._addQuery(this.queryUri);
		}
		
		// Add the group sections
		dojo.forEach(this.groups, function(group) {
			this._addGroup(group);
		}, this);
		
		this.inherited(arguments);
	},
	
	_getGroupObject: function(section) {
		var headerNode = section.getHeaderNode();
		var header = dijit.getEnclosingWidget(headerNode.children[0]);
		return ViewpointUtilities.createGroup(header.name, section.queryUri, header.description, null /*iconURI*/);
	},
	
	doSave: function(taskMonitor, synchronous) { 
		var defQuery = null;
		var taskQuery = taskMonitor.beginTask(this.messages.saveQueryTask, 1);
		// Save query (section will check dirty flag)
		if (this._querySection) {
			defQuery = this._querySection.doSave(taskMonitor, synchronous);
		}
		
		// Save groups
		var defs = defQuery ? [defQuery] : [];
		dojo.forEach(this._groupSections, function(gs) {
			var section = gs.section;
			var def = new dojo.Deferred();
			defs.push(def);
			if (section.isDirty()) {
				var defGroupQuery = section.doSave(taskMonitor, synchronous);
				defGroupQuery.addCallback({def: def, segment: this, section: section}, function(queryModel) {
					// group object should be properly set up at this time
					this.def.callback(this.segment._getGroupObject(this.section));
				});
				defGroupQuery.addErrback(def.errback);
			} else {
				// Just return the group object
				def.callback(this._getGroupObject(section));
			}			
		}, this);

		var deflist = new dojo.DeferredList(defs);
		taskQuery.handleDeferredWork(deflist, 1);
		
		return deflist;
	},
	
	_onAddGroup: function() {
		// Add group section
		this.createGroup();
		
		// Dirty
		this.editor.addDirty("Create Group");
	},
	
	_onAddQuery: function() {
		this.queryUri = "";
		this._addQuery(this.queryUri);
		
		// Dirty
		this.editor.addDirty("Create Query");
	},
	
	createGroup: function() {
		var group = ViewpointUtilities.createGroup("", "", null /*description*/, null /*iconURI*/);
		this._addGroup(group);		
	},
	
	_onCreateGroupSectionHeader: function(section, domNode, headerData) {
		var header = new rmps.app.ui.internal.editors.views.explorerviewpoint._NameDescriptionHeader({
			name: headerData.name,
			description: headerData.description,
			section: section,
			messages: this.messages
		});
		
		domNode.appendChild(header.domNode);
		header.startup();
	},
	
	_addQuery: function(queryURI) {
		var section = new rmps.ui.QuerySection({
			sectionId: queryURI,
			title: this.messages.query,
			queryUri: queryURI,
			queryWidgetInputData: this.queryWidgetInputData,
			onDirtyCallback: dojo.hitch(this.editor, this.editor.addDirty),
			onCloseCallback: dojo.hitch(this, this._removeQuery),
			isReadOnly: this.editor.isReadOnly
		});
		
		this._querySection = section;
		
		dojo.style(this._addQueryButton.domNode, {display: "none"});
		
		this._query.appendChild(section.domNode);
		section.startup();		
	},
	
	_addGroup: function(group) {
		var id = ViewpointUtilities.getGroupQueryUri(group);
		var name = ViewpointUtilities.getGroupName(group);
		var description = ViewpointUtilities.getGroupDescription(group);
		var queryURI = ViewpointUtilities.getGroupQueryUri(group);
		var section = new rmps.ui.QuerySection({
			sectionId: id,
			title: this.messages.group,
			queryUri: queryURI || "",
			queryWidgetInputData: this.queryWidgetInputData,
			onDirtyCallback: dojo.hitch(this.editor, this.editor.addDirty),
			onCloseCallback: dojo.hitch(this, this._removeGroup),
			onCreateHeader: dojo.hitch(this, this._onCreateGroupSectionHeader),
			headerData: {
				name: name,
				description: description 
			},
			isReadOnly: this.editor.isReadOnly
		});
		
		section.setSecondaryTitle(name);

		console.log("Group added: " + id + ":" + name + " in segment " + this.id);
		
		this._groupSections.push({
			groupId: id,
			section: section
		});
		
		this._groups.appendChild(section.domNode);
		section.startup();
	},
	
	_removeGroup: function(sectionId, widgetId) {
		var section = null;
		dojo.some(this._groupSections, function(groupsection, ix) {
			if (groupsection.section.id === widgetId) {
				section = groupsection.section;
				this._groupSections.splice(ix, 1);
				return true;
			}
			return false;
		}, this);

		if (!section) {
			return;	
		}
		
		this._groups.removeChild(section.domNode);
		// FIXME: Destroy properly
		//section.destroy();

		// Dirty
		this.editor.addDirty("Remove Group");
	},
	
	_removeQuery: function() {
		this._query.removeChild(this._querySection.domNode);
		this._querySection = null;
		this.queryUri = null;
		dojo.style(this._addQueryButton.domNode, {display: "block"});
		// FIXME: Destroy properly
		//section.destroy();
		
		// Dirty
		this.editor.addDirty("Remove Query");
	},
	
	validate: function(rootTab) {
		var def = new dojo.Deferred();
		
		if (!this._querySection && this._groupSections.length === 0) {
			// No query, and no groups added.
			var errors = [this.messages.errorNoQueryOrGroup];
			def.callback(errors);
			return def;
		}
		
		var defs = [];
 
		if (this._querySection) {
			defs.push(this._querySection._qewidget.validate(rootTab));
		}

		for (var gix in this._groupSections) {
			var section = this._groupSections[gix].section;
			defs.push(section._qewidget.validate(rootTab));
		}
		
		var defList = new dojo.DeferredList(defs);
		defList.addBoth(this, function(results) {
			var errors = [];			
			var failure = false;
			dojo.forEach(results, function(result) {
				if (result[0]) { //Success?
					var widgetErrors = result[1];
					for (var i = 0; i < widgetErrors.length; i++) {
						errors.push(widgetErrors[i]);
					}
				} else {
					failure = true;
				}
			}, this);
			
			if (failure) {
				def.errback(errors);
			} else {
				def.callback(errors);
			}
		});

		return def;
	}
});

dojo.declare("rmps.app.ui.internal.editors.views.explorerviewpoint._NameDescriptionHeader", [dijit._Widget, dijit._Templated], {
	
	// mixin
	name: "",
	description: "",
	section: null,
	
	templateString: '<div class="rmps-app-explorerviewpoint-namedescriptionheader">' +
			'<table>' +
				'<tr>' +
					'<th>${messages.name}</th>' +
					'<th>${messages.description}</th>' +
				'</tr>' + 
				'<tr>' +
					'<td class="namefield"><input dojoAttachPoint="_nameField" dojoAttachEvent="onblur: onEditName" value="${name}"></input></td>' +
					'<td class="descfield"><input dojoAttachPoint="_descField" dojoAttachEvent="onblur: onEditDesc" value="${description}"></input></td>' +
				'</tr>' + 
			'</table>' +
		'</div>',
		
	onEditName: function() {
		this.name = this._nameField.value;
		if (this.section) {
			this.section.setSecondaryTitle(this.name);
		}
	},
	
	onEditDesc: function() {
		this.description = this._descField.value;
	}
});

}());