/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.views.link.IncomingLink");

dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");
dojo.require("rmps.app.ui.internal.editors.views.link.Link");

var IRmpsConstants= rmps.core.util.IRmpsConstants;
var UIHelper= com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;

/*IncomingLink Constants */
var CONTRIBUTOR= "contributor";         //$NON-NLS-1$
var CREATED= "created";					//$NON-NLS-1$
var CREATOR= "creator";					//$NON-NLS-1$
var DESCRIPTION= "Description";			//$NON-NLS-1$
var MEMBER= "member";					//$NON-NLS-1$
var MODIFIED= "modified";				//$NON-NLS-1$
var PREDICATE= "predicate";				//$NON-NLS-1$
var RESOURCE= "resource";				//$NON-NLS-1$
var RESPONSE_INFO= "ResponseInfo";      //$NON-NLS-1$
var RESPONSE_INFO_URL= IRmpsConstants.ProcessConstants.NAMESPACE_OSLC_CORE + RESPONSE_INFO; 
var TYPE= "type";						//$NON-NLS-1$
var TOTAL_COUNT= "totalCount";			//$NON-NLS-1$

var RDF_SCHEMA_XML_NAMESPACE = "http://www.w3.org/2000/01/rdf-schema#";

dojo.declare("rmps.app.ui.internal.editors.views.link.IncomingLink", null, {
	
	object: null,
	subjects: null,
	incomingLinks: null,
	resourceInfo: null,
	eTag: null,
	uri: null,

	constructor: function() {	/*Empty Constructor */},

	parse: function(doc, ioArgs) {
		/*Sort Incoming Links By Link Titles */		
		function stringCompare(p1, p2){
			if (dojo.isString(p1)) {
				p1 = p1.toLocaleLowerCase();
			}
			if (dojo.isString(p2)) {
				p2 = p2.toLocaleLowerCase();
			}
			return p1.localeCompare(p2);
		}

		function compare(entry1, entry2) {
			if (entry1 && entry1.label && entry2 && entry2.label) {
				var cr = stringCompare(entry1.label, entry2.label);
				if (cr !== 0) {
					return cr;
				}
			}

			return stringCompare(entry1.resource, entry2.resource);
		}

		function comparator(entry1, entry2) {
			return compare(entry1, entry2);
		}
		
		// In chrome, there are cases where the document is not defined
		if (!doc) {
			this.incomingLinks = [];
			this.subjects = {};
			return;
		}
		

		if (ioArgs) {
			var xhr = ioArgs.xhr;
			if (xhr.getResponseHeader("ETag")) {
				this.eTag = xhr.getResponseHeader("ETag");
			}
			if (xhr.getResponseHeader("Location")) {
				this.uri = xhr.getResponseHeader("Location");
			}
		} else {
			console.error("ioarag cannot be null");
			return;
		}

		var entries = [];
		if (dojo.isChrome) {
			// On Chrome, getElementsByTagName("rdf:RDF") does not return any entries
			// even when there some defined in the document. Instead, get all
			// entries and extract those with tag name rdf:RDF from the list.
			dojo.forEach(doc.getElementsByTagName("*"), function(element){
				var tag = element.tagName;
				if (tag === IRmpsConstants.RDF_TAG) {
					entries.push(element);
				}
			});
		} else {
			entries = doc.getElementsByTagName(IRmpsConstants.RDF_TAG);
		}
		if (!entries) {
			console.error("Error: Empty content");	//$NON-NLS-1$
			return;
		}

		this.incomingLinks = [];
		this.subjects = {};

		for (var e = 0; e < entries.length; e++) {
			var entry = entries[e];

			var responseInfoNode = this._getResponseInfoNode(entry);
			if (responseInfoNode) {
				this.resourceInfo = {
					about: this._getAbout(responseInfoNode),
					totalCount: this._getTotalCount(responseInfoNode)
				};
			} else {
				console.error("Error: Resource Info was null");
			}

			for (var cn = 0; cn < entry.childNodes.length; cn++) {
				var childNode = entry.childNodes[cn];
				if (childNode === responseInfoNode) {
					continue;
				}

				if (!this._matchNode(childNode, DESCRIPTION, IRmpsConstants.ProcessConstants.NAMESPACE_RDF)) {
					continue;
				}

				var subjectNode = this._getSubjectNode(childNode);
				if (subjectNode) {
					var subjectAbout = this._getAbout(subjectNode);
					var subjectDetails = {
						types: [],
						linkStatements: []
					};

					for (var dn = 0; dn < subjectNode.childNodes.length; dn++) {
						var dcNode = subjectNode.childNodes[dn];

						if (!dcNode.baseName && !dcNode.localName) {
							continue;
						}

						if (this._matchNode(dcNode, MEMBER, RDF_SCHEMA_XML_NAMESPACE)) {
							var member = UIHelper.getAttributeNodeValue(dcNode, RESOURCE, IRmpsConstants.ProcessConstants.NAMESPACE_RDF);
							member = decodeURIComponent(member);

							var existingData = this.subjects[member];
							if (existingData) {
								existingData.memberOf = subjectAbout;
							} else {
								this.subjects[member] = { memberOf : subjectAbout };
							}

							if (member.indexOf(this.resourceInfo.about) === 0 && member.length > this.resourceInfo.about.length) {
								existingData = this.subjects[this.resourceInfo.about];
								if (existingData) {
									existingData.memberOf = subjectAbout;
								} else {
									this.subjects[this.resourceInfo.about] = { memberOf : subjectAbout };
								}
							}
							continue;
						}

						if (this._matchNode(dcNode, TYPE, IRmpsConstants.ProcessConstants.NAMESPACE_DC)) {
							// For backward compatibility with DM 3.x
							subjectDetails.bcType =  dcNode.textContent ? dcNode.textContent : dcNode.text;
							continue;
						}

						if (this._matchNode(dcNode, TYPE, IRmpsConstants.ProcessConstants.NAMESPACE_RDF)) {
							subjectDetails.types.push(UIHelper.getAttributeNodeValue(dcNode, RESOURCE, IRmpsConstants.ProcessConstants.NAMESPACE_RDF));
							continue;
						}

						if (this._matchNode(dcNode, CREATOR, IRmpsConstants.ProcessConstants.NAMESPACE_DC)) {
							subjectDetails.creator = UIHelper.getAttributeNodeValue(dcNode, RESOURCE, IRmpsConstants.ProcessConstants.NAMESPACE_RDF);
							continue;
						}

						if (this._matchNode(dcNode, CONTRIBUTOR, IRmpsConstants.ProcessConstants.NAMESPACE_DC)) {
							subjectDetails.contributor = UIHelper.getAttributeNodeValue(dcNode, RESOURCE, IRmpsConstants.ProcessConstants.NAMESPACE_RDF);
							continue;
						}

						if (this._matchNode(dcNode, IRmpsConstants.ProcessConstants.XMLELEMENT_TITLE, IRmpsConstants.ProcessConstants.NAMESPACE_DC)) {
							subjectDetails.title = dcNode.textContent ? dcNode.textContent : dcNode.text;
							continue;
						}

						if (this._matchNode(dcNode, CREATED, IRmpsConstants.ProcessConstants.NAMESPACE_DC)) {
							subjectDetails.created = dojo.date.stamp.fromISOString(dcNode.textContent ? dcNode.textContent : dcNode.text);
							continue;
						}

						if (this._matchNode(dcNode, MODIFIED, IRmpsConstants.ProcessConstants.NAMESPACE_DC)) {
							subjectDetails.modified = dojo.date.stamp.fromISOString(dcNode.textContent ? dcNode.textContent : dcNode.text);
							continue;
						}

						if (dcNode.attributes) {
							var resource = UIHelper.getAttributeNodeValue(dcNode, RESOURCE, IRmpsConstants.ProcessConstants.NAMESPACE_RDF);
							if (resource === this.resourceInfo.about) {
								subjectDetails.linkStatements.push(dcNode);
							}
						}
					}

					var existingData = this.subjects[subjectAbout];
					if (existingData) {
						dojo.mixin(existingData, subjectDetails);
					} else {
						this.subjects[subjectAbout] = subjectDetails;
					}
					continue;
				}

				var predicateNode = this._getPredicateNode(childNode);
				if (predicateNode) {
					var link = new rmps.app.ui.internal.editors.views.link.Link();
					link.parse(predicateNode, null);
					link.value = decodeURIComponent(link.value);
					link.subject = decodeURIComponent(link.subject);

					if (link.value === this.resourceInfo.about) {
						this.incomingLinks.push({
							resource: link.value,
							about: link.subject,
							value: link.subject,
							predicate: link.predicate,
							label: link.label,
							description: link.description
						});
					}
				}
			}
		}

		for (var keyAbout in this.subjects) {
			var details = this.subjects[keyAbout];
			if (!details.linkStatements) {
				continue;
			}

			for (var ls = 0; ls < details.linkStatements.length; ls++) {
				var linkStatement = details.linkStatements[ls];
				var predicate = linkStatement.namespaceURI + (linkStatement.baseName ? linkStatement.baseName : linkStatement.localName);
				var existingLink = null;
				for (var il = 0; il < this.incomingLinks.length; il++) {
					var incomingLink = this.incomingLinks[il];
					if (incomingLink.about === keyAbout && incomingLink.resource === this.resourceInfo.about && incomingLink.predicate === predicate) {
						existingLink = incomingLink;
						if (!existingLink.label) {
							existingLink.label = details.title;
						}
						break;
					}
				}

				if (existingLink) {
					continue;
				}

				this.incomingLinks.push({
					about: keyAbout,
					value: keyAbout,
					resource: this.resourceInfo.about,
					predicate: predicate,
					label: details.title
				});
			}
		}

		for (var l = 0; l < this.incomingLinks.length; l++) {
			var incomingLink = this.incomingLinks[l];
			var sbjMemberOf = this.subjects.hasOwnProperty(incomingLink.resource) ? this.subjects[incomingLink.resource].memberOf : null;
			var objMemberOf = this.subjects.hasOwnProperty(incomingLink.value) ? this.subjects[incomingLink.value].memberOf : null;
			if (sbjMemberOf && objMemberOf && sbjMemberOf === objMemberOf) {
				incomingLink.projectScope = true;
			}

			if (incomingLink.label) {
				continue;
			}

			if (this.subjects.hasOwnProperty(incomingLink.value)) {
				var value = this.subjects[incomingLink.value];
				incomingLink.label = value.title;
			}
		}

		this.incomingLinks.sort(comparator);
	},

	_matchNode: function(node, baseName, namespaceURI) {
		return ((node.baseName === baseName || node.localName === baseName) && (node.namespaceURI === namespaceURI || (node.namespaceURI === "" && namespaceURI === null)));
	},

	_getResponseInfoNode: function(node) {
		for (var cn = 0; cn < node.childNodes.length; cn++) {
			var child = node.childNodes[cn];

			if (this._matchNode(child, RESPONSE_INFO, IRmpsConstants.ProcessConstants.NAMESPACE_OSLC_CORE)) {
				return child;
			}

			if (this._matchNode(child, DESCRIPTION, IRmpsConstants.ProcessConstants.NAMESPACE_RDF)) {
				if (!child.childNodes) {
					continue;
				}

				var typeNode = UIHelper.getNode(this._getChildNodes(child), TYPE, IRmpsConstants.ProcessConstants.NAMESPACE_RDF);
				if (!typeNode || !typeNode.attributes) {
					continue;
				}

				if (UIHelper.getAttributeNodeValue(typeNode, RESOURCE, IRmpsConstants.ProcessConstants.NAMESPACE_RDF) === RESPONSE_INFO_URL) {
					return child;
				}
			}
		}

		return null;
	},

	_getSubjectNode: function(node) {
		var about = node.attributes ? UIHelper.getAttributeNodeValue(node, IRmpsConstants.ABOUT, IRmpsConstants.ProcessConstants.NAMESPACE_RDF) : null;
		if (about) {
			return node;
		}

		return null;
	},

	_getPredicateNode: function(node) {
		var predicateNode = node.childNodes ? UIHelper.getNode(this._getChildNodes(node), PREDICATE, IRmpsConstants.ProcessConstants.NAMESPACE_RDF) : null;
		if (predicateNode) {
			return node;
		}

		return null;
	}, 

	_getAbout: function(description) {
		var about = UIHelper.getAttributeNodeValue(description, IRmpsConstants.ABOUT, IRmpsConstants.ProcessConstants.NAMESPACE_RDF);
		about = decodeURIComponent(about);
		var b1Index = about.indexOf("<");
		var b2Index = about.indexOf(">");
		if (b1Index > -1 && b2Index > -1) {
			about= about.substring(b1Index+1, b2Index);
		}

		return about;
	},

	_getTotalCount: function(description) {
		var countNode = UIHelper.getNode(this._getChildNodes(description), TOTAL_COUNT, IRmpsConstants.ProcessConstants.NAMESPACE_OSLC_CORE);
		if (countNode) {
			return parseInt(countNode.textContent ? countNode.textContent : countNode.text);
		}

		return 0;
	},

    _getChildNodes: function(source) {
        if (dojo.isArray(source)) {
            return source;
        }

        if (dojo.isIE > 8) {
            var childNodes = [];
            var node = source.firstChild;
            while (node) {
                childNodes.push(node);
                node = node.nextSibling;
            }
            return childNodes;
        }

        return source.childNodes;
    }
});

})();