/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("rmps.app.ui.internal.editors.views.link.Link");

	dojo.require("rmps.core.resource.Resource");
	dojo.require("rmps.core.resource.ResourceInfoClient");
	dojo.require("rmps.core.user.UserClient");

	var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;
	var UserClient = rmps.core.user.UserClient;

	var XMLNS     = "http://open-services.net/ns/am#";
	var OSLC      = "http://open-services.net/xmlns/common/1.0/";
	var DC        = "http://purl.org/dc/terms/";
	var RDF       = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
	var RDF_SCHEMA = "http://www.w3.org/2000/01/rdf-schema#";
	var RMPS      = "http://jazz.net/xmlns/rmps/1.0/";
	

	dojo.declare("rmps.app.ui.internal.editors.views.link.Link", rmps.core.resource.Resource, {

	// private properties
	_resourceInfo: null,
	subject: null,
	resourceUri: null,
	creator: null,
	created: null,
	predicate: null,
	updated: null,
	value: null,
	readOnly: false,
	description: null,
	title: null,
	label: null,
	suspectClearedTime: null,
	eTag: null,
	dmProjectInternal: false,

	constructor: function() {
	},

	getResourceInfo: function(properties, context) {
		var deferred;
		if (this._resourceInfo) {
			deferred = new dojo.Deferred();
			setTimeout(dojo.hitch(this, function() {
				deferred.callback(this._resourceInfo);
			}), 0);
			return deferred;
		}
		deferred = ResourceInfoClient.getResourceInfo(this.resourceUri, properties, context);
		deferred.addCallback(dojo.hitch(this, function(resourceInfos) {
			this._resourceInfo = resourceInfos[0];
		}));
		return deferred;
	},

	getContributor: function() {
		var deferred;
		if (this._contributor && this._contributor.uri === this.contributor) {
			deferred = new dojo.Deferred();
			setTimeout(dojo.hitch(this, function() {
				deferred.callback(this._contributor);
			}), 0);
			return deferred;
		}
		deferred = UserClient.getUser(this.contributor);
		deferred.addCallback(dojo.hitch(this, function(user) {
			this._contributor = user;
		}));
		return deferred;
	},

	getCreator: function() {
		var deferred;
		if (this._creator && this._creator.uri === this.creator) {
			deferred = new dojo.Deferred();
			setTimeout(dojo.hitch(this, function() {
				deferred.callback(this._creator);
			}), 0);
			return deferred;
		}
		deferred = UserClient.getUser(this.creator);
		deferred.addCallback(dojo.hitch(this, function(user) {
			this._creator = user;
		}));
		return deferred;
	},

	toXMLString: function() {
		var linkXML = '<?xml version="1.0" encoding="UTF-8"?>\n' +
			'<Link xmlns="' + XMLNS +
			'"\n xmlns:dcterms="' + DC +
			'"\n xmlns:rdf="'+ RDF +
			'"\n xmlns:rdfs="'+ RDF_SCHEMA +
			'"\n xmlns:rmps="'+ RMPS +
			'"\n rdf:about="'+this.uri+'">\n';
		linkXML += '<rdf:subject rdf:resource="' + this._getXMLEncodedText(this.subject) + '" />\n';
		if (this.label) {
			linkXML += '<rdfs:label>' + this._getXMLEncodedText(this.label) + '</rdfs:label>\n';
		}
		linkXML += '<rdf:predicate rdf:resource="' + this._getXMLEncodedText(this.predicate) + '" />\n';
		linkXML += '<rdf:object rdf:resource="' + this._getXMLEncodedText(this.value) + '" />\n';
		linkXML += '<dcterms:title>' + this._getXMLEncodedText(this.title) + '</dcterms:title>\n';

		if (this.description) {
			linkXML += '<dcterms:description>' + this._getXMLEncodedText(this.description) + '</dcterms:description>\n';
		}

		if (this.suspectClearedTime) {
			linkXML += '<rmps:suspectClearedTime>' + this.suspectClearedTime + '</rmps:suspectClearedTime>\n';
		}

		linkXML += "</Link>\n";
		return linkXML;
	},

	parseEntry: function(atomEntry, ioArgs) {
		var linkFeed = atomEntry.childNodes[0];

		this.uri = linkFeed.getAttribute('href');

		var contentNode, linkNode = null;

		var childNodes = atomEntry.childNodes;
		for (var index = 0; index < childNodes.length; index++) {
			var current = childNodes[index];
			var baseName = current.baseName;
			var localName = current.localName;
			if (localName === "ETag" || baseName === "ETag") {
				var eTagNode = current;
				if (eTagNode.childNodes.length === 1 && eTagNode.childNodes[0].nodeName === '#text') {
					this.eTag = eTagNode.childNodes[0].nodeValue;
				}
			} else if (localName === "content" || baseName === "content") {
				contentNode = current;
			} else if (localName === "link" || baseName === "link") {
				linkNode = current;
			}
		}
		this.parse(contentNode.childNodes[0], null);
	},

	parseLinkFeed: function(linkNode, ioArgs) {
		if (ioArgs) {
			var xhr = ioArgs.xhr;
			if (xhr.getResponseHeader("Last-Modified")) {
				this.lastModified = new Date(xhr.getResponseHeader("Last-Modified"));
			}
			if (xhr.getResponseHeader("ETag")) {
				this.eTag = xhr.getResponseHeader("ETag");
			}
			if (xhr.getResponseHeader("Location")) {
				this.uri = xhr.getResponseHeader("Location");
			}
		}

		var content = linkNode.content[0];
		var oslcLink = content["oslc_am:Link"][0];
		this.resourceUri = oslcLink["rdf:subject"][0]["rdf:resource"];//oslcLink["rdf:about"];
		var rdfcreator = oslcLink["rdf:creator"];
		this.creator = rdfcreator ? rdfcreator[0]["rdf:resource"] : oslcLink["dcterms:creator"][0]["rdf:resource"];
		this.subject = oslcLink["rdf:subject"][0]["rdf:resource"];

		/*Label is optional and can be null. So, we need to check for null for its case */
		var labelEntry = oslcLink["rdfs:label"];
		this.label = labelEntry?labelEntry[0].value: labelEntry;

		/*Description is optional and can be null. So, we need to check for null for its case */
		var description = oslcLink["dcterms:description"];
		this.description = description ? description.value : description;

		/* SuspectClearedTime is optional and can be null.  */
		var suspectTime = oslcLink["rmps:suspectClearedTime"];
		this.suspectClearedTime = suspectTime ? suspectTime.value : null;

		this.value = oslcLink["rdf:object"][0]["rdf:resource"];
		this.predicate = oslcLink["rdf:predicate"][0]["rdf:resource"];
		var rdfcreated = oslcLink["rdf:created"];
		var created = rdfcreated ? rdfcreated[0]["rdf:resource"] : oslcLink["dcterms:created"][0].value;
		this.created = dojo.date.stamp.fromISOString(created);
		var rdfmodified = oslcLink["rdf:modified"];
		var modified = rdfmodified ? rdfmodified[0]["rdf:resource"] : oslcLink["dcterms:modified"][0].value;
		this.updated = dojo.date.stamp.fromISOString(modified);

		// contributor
		var rdfcontributor = oslcLink["rdf:contributer"];
		this.contributor = rdfcontributor ? rdfcontributor[0]["rdf:resource"] : oslcLink["dcterms:contributor"][0]["rdf:resource"];
		
		// isDMProjectInternal
		var isDMProjectInternal = linkNode["rmps:isDMProjectInternal"];
		this.dmProjectInternal = isDMProjectInternal ? isDMProjectInternal[0].value === "true" : false;
	},

	parse: function(linkNode, ioArgs) {
		if (ioArgs) {
			var xhr = ioArgs.xhr;
			if (xhr.getResponseHeader("ETag")) {
				this.eTag = xhr.getResponseHeader("ETag");
			}
			if (xhr.getResponseHeader("Location")) {
				this.uri = xhr.getResponseHeader("Location");
			}
		}
		for ( var i = 0 ; i < linkNode.childNodes.length; i++) {
			var node = linkNode.childNodes[i];
			if (node.nodeName === "rdf:resource") {
				this.resourceUri = node.attributes[0].nodeValue;
			} else if (node.nodeName === "rdf:predicate") {
				this.predicate = node.attributes[0].nodeValue;
			} else if (node.nodeName === "rdf:object") {
				this.value = node.attributes[0].nodeValue;
			} else if (node.nodeName === "rdf:creator") {
				this.creator = node.attributes[0].nodeValue;
			} else if (node.nodeName === "rdf:created") {
				this.created = node.attributes[0].nodeValue;
			} else if (node.nodeName === "rdf:subject") {
				this.subject = node.attributes[0].nodeValue;
			} else if (node.nodeName === "rdfs:label") {
				if (node.childNodes.length > 0) {
					this.label = node.childNodes[0].nodeValue;
				}
			} else if (node.nodeName === "dcterms:description") {
				if (node.childNodes.length > 0) {
					this.description = node.childNodes[0].nodeValue;
				}
			} else if (node.nodeName === "rmps:suspectClearedTime") {
				if (node.childNodes.length > 0) {
					this.suspectClearedTime = node.childNodes[0].nodeValue;
				}
			} else if (node.nodeName === "dcterms:title") {
				if (node.childNodes.length > 0) {
					this.title = node.childNodes[0].nodeValue;
				}
			} else if (node.nodeName === "oslc:readOnly") {
				if (node.childNodes.length > 0) {
					var val = node.childNodes[0].nodeValue;
					if (val) {
						this.readOnly = val === "true";
					}
				}
			}
		}
	},

	_getXMLEncodedText: function(text) {
		if (!text) {
			return "";
		}

		var encodedText = text.toString();
		encodedText = encodedText.replace(/&/g, "&amp;");
		encodedText = encodedText.replace(/</g, "&lt;");
		encodedText = encodedText.replace(/>/g, "&gt;");
		encodedText = encodedText.replace(/"/g, "&quot;");
		encodedText = encodedText.replace(/'/g, "&#039;");
		// Valid characters to include in XML document, from http://www.w3.org/TR/REC-xml/#NT-Char
		//
		// x9 | #xA | #xD | [#x20-#xD7FF] | [#xE000-#xFFFD] | [#x10000-#x10FFFF]
		// Remove characters that are invalid to be used in XML. Javascript represents values in the range
		// [#x10000-#x10FFFF] as surrogate pairs, so we allow through values in the ranges
		// D800-DBFF (high surrogate) and DC00-DFFF (low surrogate), so surrogate pairs representing code points
		// U+10000...U+10FFFF are also passed through without change. This does leave us open to the
		// slight possibility that invalid surrogate pair values will be passed (a high or low surrogate in isolation or
		// in reversed order), but it is unlikely that such sequences would ever get this far, since for that to happen they
		// would have somehow had to get into a javascript string first.
		encodedText = encodedText.replace(/[^\u0009\u000A\u000D\u0020-\uFFFD]/g, "");
		return encodedText;
	}
});

})();
