/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.views.link.LinkClient");

dojo.require("rmps.app.ui.internal.editors.views.link.Link");
dojo.require("rmps.app.ui.internal.editors.views.link.LinkType");
dojo.require("rmps.app.ui.internal.editors.views.link.IncomingLink");
dojo.require("rmps.app.ui.internal.util.UrlUtil");
dojo.require("rmps.core.util.ILinksConstants");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmps.core.client");

dojo.require("dojox.xml.DomParser");
dojo.require("jazz.app.context");

dojo.require("rmps.core.RootServices");

dojo.requireLocalization("rmps.app.ui.internal.editors.views.link", "LinkClientMessages");
var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.editors.views.link", "LinkClientMessages");

var ILinksConstants = rmps.core.util.ILinksConstants;
var LINKS_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, ILinksConstants.LINKS);
var REVIEW_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "reviews"); /*For review links */
var MODELS_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "models");	/*For incoming links */
var OSLC_QUERY = net.jazz.ajax._contextRoot+ "/oslcquery";	/*For incoming links */
var LINK_TYPES_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, ILinksConstants.LINK_TYPES);
var IRmpsConstants = rmps.core.util.IRmpsConstants;
var UrlUtil = rmps.app.ui.internal.util.UrlUtil;

/* Delegate some required jazz.client functions and add the required handling for errors and baseline */
var CLIENT = {
	_addErrorCallback : function(deferred) {
		deferred.addErrback(this, function(error) {
			if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
				error = rmps.core.error.createXhrError(deferred.ioArgs).getGenericError();
			} else {
				error = rmps.core.error.createError(error).getGenericError();
			}
			if (error.status === 401 || error.status === 403 || error.status === 407) {
				error.message = messages.error_permission;
			}
			error.dispatch();
			return error;
		});
		return deferred;
	}, 

	_addErrorCallback_IncomingLinks : function(deferred) {
		deferred.addErrback(this, function(error) {
			if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
				error = rmps.core.error.createXhrError(deferred.ioArgs).getGenericError();
			} else {
				error = rmps.core.error.createError(error).getGenericError();
			}
			if (error.status === 401 || error.status === 403 || error.status === 407) {
				error.message = messages.error_permission;
			} else if (error.status >= 400) {
				// Ignore the error and return empty IncomingLink
				var link = new rmps.app.ui.internal.editors.views.link.IncomingLink();
				link.parse(null, null);
				return link;
			}
			error.dispatch();
			return error;
		});
		return deferred;
	}, 

	xhrGet: function(args, sendProjectId, context) {
		return this._addErrorCallback(rmps.core.client.xhrGet(args, sendProjectId, context));
	},

	xhrGet_IncomingLinks: function(args, sendProjectId, context) {
		return this._addErrorCallback_IncomingLinks(rmps.core.client.xhrGet(args, sendProjectId, context));
	},

	xhrHead: function(args, sendProjectId, context) {
		return this._addErrorCallback(rmps.core.client.xhrHead(args, sendProjectId, context));
	},

	xhrPost: function(args, sendProjectId, context) {
		return this._addErrorCallback(rmps.core.client.xhrPost(args, sendProjectId, context));
	},

	xhrPut: function(args, sendProjectId, context) {
		return this._addErrorCallback(rmps.core.client.xhrPut(args, sendProjectId, context));
	},

	xhrDelete: function(args, sendProjectId, context) {
		return this._addErrorCallback(rmps.core.client.xhrDelete(args, sendProjectId, context));
	}
};

rmps.app.ui.internal.editors.views.link.LinkClient = {

	getLink: function(linkUri) {
		var args = {
			url: linkUri,
			handleAs: "text",
			load: function(response, ioArgs)
			{
				var link = new rmps.app.ui.internal.editors.views.link.Link();
				link.parse(dojox.xml.DomParser.parse(response), ioArgs);
				return link;
			}
		};
		return CLIENT.xhrGet(args);
	},

	getRecentLinks: function(projectId, createdAfter, page, pageSize, pageToken, linkTypes, contextURI, fallbackToDefaultContext) {
		var queryParams = {
			"projectId" : projectId,
			"pageSize" : -1
		};
		if (createdAfter) {
			queryParams.createdAfter = createdAfter;
		}
		if (page !== undefined && page !== null) {
			queryParams.page = page;
		}
		if (pageSize) {
			queryParams.pageSize = pageSize;
		}
		if (pageToken) {
			queryParams.pageToken = pageToken;
		}
		if (contextURI) {
			queryParams[IRmpsConstants.BASELINE] = contextURI;
		}
		if (fallbackToDefaultContext) {
			queryParams.fallbackToDefaultContext = true;
		}

		var args = {
			url: LINKS_PATH + "/recent?"
				+ dojo.objectToQuery(queryParams),
			headers: {
				"Accept": "application/atom+json"
			},
			handleAs: "json",
			load: function(response, ioArgs)
			{
				var result = [];
				var entries = response.entry;

				result.pageToken = "";
				result.totalResults = parseInt(response["opensearch:totalResults"][0].value, 10);

				var link;
				if (entries && entries.length > 0) {
					for (var i = 0; i < entries.length; i++) {
						link = new rmps.app.ui.internal.editors.views.link.Link();
						link.parseLinkFeed(entries[i], ioArgs);
						result.push(link);
					}
				}
				
				var links = response.link;
				if (links && links.length > 0) {
					for (var l = 0; l < links.length; l++) {
						link = links[l];
						if (link.rel === "self") {
							result.pageToken = UrlUtil.getParamater(link.href, "pageToken");
							break;
						}
					}
				}
				
				return result;
			}
		};

		if (linkTypes) {
			args.headers["Content-Type"] = "application/xml";
			args.hasBody = true;
			args.postData = linkTypes;
			return CLIENT.xhrPost(args);
		}

		return CLIENT.xhrGet(args);
	},

	getLinkTypes: function(projectId, extraArgs, contextURI) {
		if (!projectId) {
			projectId= jazz.app.context.get().itemId;
		}
		var linkTypes = [];
		var orderBy = extraArgs ? extraArgs.orderBy : null;
		var queryParams = null;
		if (contextURI) {
			queryParams = {};
			queryParams[IRmpsConstants.BASELINE] = contextURI;
		}
		var sort= this.sortData;
		var args = {
			url: LINK_TYPES_PATH + "/" + projectId + (queryParams ? "?" + dojo.objectToQuery(queryParams) : ""),
			handleAs: "xml",
			load: function(response, ioArgs)
			{
				var doc = response;
				var entries = doc.getElementsByTagName("entry");
				if (entries.length > 0) {
					for (var i = 0; i < entries.length; i++) {
						var linkType = new rmps.app.ui.internal.editors.views.link.LinkType();
						linkType.parseEntry(entries[i], ioArgs);
						linkTypes.push(linkType);
					}
				}
				if(orderBy){
					sort(linkTypes, orderBy);
				}
				return linkTypes;
			}
		};
				
		if (extraArgs) {
			dojo.mixin(args, extraArgs);
		}
		return CLIENT.xhrGet(args);
	},

	getAllLinkTypes: function(projectId, extraArgs, contextUri) {
		var deferred= rmps.app.ui.internal.editors.views.link.LinkClient.getLinkTypes(projectId, extraArgs, contextUri);
		deferred.addCallback(function(result) {
			var total= [];
			var systemDefined= [];
			var userDefined= [];
			var external= null;
			for(var index=0; index<result.length; index++){
				var current= result[index];
				if(current.systemDefined){
					if(current.about===ILinksConstants.EXTERNAL_LINK_TYPE){
						external= current;
						continue;
					}else{
						systemDefined.push(current);
					}
				}else{
					userDefined.push(current);
				}
			}
			if(external){
				systemDefined.push(external);	
			}
			
			userDefined.reverse();
			systemDefined.reverse();
			total[ILinksConstants.USER_DEFINED]= userDefined;
			total[ILinksConstants.SYSTEM_DEFINED]= systemDefined;
			return total;
		});
				
		return deferred;
	},
	
	deployPredefinedLinkTypes: function(projectId){
		var queryParams= {
			"deploy": "predefined"
		};
		var project = projectId ? projectId : jazz.app.context.get().itemId;
		var args = {
			url: LINK_TYPES_PATH + "/" + project + "?" + dojo.objectToQuery(queryParams),
			handleAs: "xml",
			load: function(response, ioArgs)
			{
				var linkTypes= [];
				var doc = response;
				var entries = doc.getElementsByTagName("entry");
				if (entries.length > 0) {
					for (var i = 0; i < entries.length; i++) {
						var linkType = new rmps.app.ui.internal.editors.views.link.LinkType();
						linkType.parseEntry(entries[i], ioArgs);
						linkTypes.push(linkType);
					}
				}
				return linkTypes;
			}
		};
		return CLIENT.xhrPost(args);
	},

	getLinksFor: function(resourceUri) {
		var links = [];

		var queryParams = {
			"subject": resourceUri
		};
		var args = {
			url: LINKS_PATH + "?" + dojo.objectToQuery(queryParams),
			headers: {
				"Accept": "application/rdf+json"
			},
			load: function(response, ioArgs)
			{
				var doc = dojox.xml.DomParser.parse(response);
				var entries = doc.getElementsByTagName("entry");
				if (entries.length > 0) {
					for (var i = 0; i < entries.length; i++) {
						var link = new rmps.app.ui.internal.editors.views.link.Link();
						link.parseEntry(entries[i], ioArgs);
						links.push(link);
					}
				}
				return links;
			}
		};
		return CLIENT.xhrGet(args);
	},

	getIncomingLinksFor: function(resourceUri, queryInterface) {
		if (!queryInterface) {
			/*If query interface is undefined, then we get the current project area query interface */
			queryInterface= OSLC_QUERY+ "/"+ jazz.app.context.get().itemId;
		}
		var args = {
			url: queryInterface + "?"+ dojo.objectToQuery({"oslc.where": "*=<"+ resourceUri+ ">", "oslc.select" : "*"}),
			headers: {
				"Accept" : "application/rdf+xml"
			},
			handleAs: "xml",
			model: resourceUri,
			load: function(response, ioArgs)
			{
				var link = new rmps.app.ui.internal.editors.views.link.IncomingLink();
				link.parse(response, ioArgs);
				return link;
			}
		};

		if (queryInterface.indexOf(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL) === 0) {
			// Request to the same server, so that's ok to pass context
			return CLIENT.xhrGet_IncomingLinks(args);
		}

		return CLIENT.xhrGet_IncomingLinks(args, true, null);
	},

	createLink: function(/*rmps.app.ui.internal.editors.views.link.Link */ link) {
		link.created = new Date();
		var args = {
			headers: {
				"Content-Type": "application/x-oslc-common-link+xml",
				"Accept": "application/atom+json"
			},
			url: LINKS_PATH,
			handleAs: "json",
			hasBody: true,
			postData: link.toXMLString(),
			load: dojo.hitch(this, function(response, ioArgs)
			{
				link.created = new Date(ioArgs.xhr.getResponseHeader("created"));
				link.eTag = ioArgs.xhr.getResponseHeader("ETag");
				link.uri = ioArgs.xhr.getResponseHeader("Location");

				var entries = response ? response.entry : null;
				if (entries && entries.length === 1) {
					var updatedLink = new rmps.app.ui.internal.editors.views.link.Link();
					updatedLink.parseLinkFeed(entries[0], ioArgs);
					link.value = updatedLink.value;
				}

				return link;
			})
		};
		return CLIENT.xhrPost(args);
	},

	createLinkType: function(/*rmps.app.ui.internal.editors.views.link.LinkType */ link, contextUri) {
		link.created = new Date();
		var projectId= link.projectId;
		if (!projectId) {
			projectId= jazz.app.context.get().itemId;
		}

		var args = {
			headers: {
				"Content-Type": IRmpsConstants.OSLC_AM_LINK_TYPE_CONTENT_TYPE,
				"Accept": "application/atom+xml"
			},
			url: LINK_TYPES_PATH + "/" + projectId,
			handleAs: "xml",
			hasBody: true,
			postData: link.toXMLString(),
			load: dojo.hitch(this, function(response, ioArgs)
			{
				link.created = new Date(ioArgs.xhr.getResponseHeader("created"));
				link.eTag = ioArgs.xhr.getResponseHeader("ETag");
				link.uri = ioArgs.xhr.getResponseHeader("Location");
				if(!link.uri){
					link.uri= ioArgs.url;
				}
				return link;
			})
		};
		return CLIENT.xhrPost(args, false, contextUri);
	},

	deleteLink: function(link, noLinkContext) {
		/*link.url is assumed to be the exact url for the resource to be deleted.
		 * For example URL= https://localhost:9444/rmps/links/_WYF3gxzLEd-PAr2Vd2VZww */
		var mainDeferred= new dojo.Deferred();
		
		var params = noLinkContext ? "?noLinkContext=true" : "";
		function doDeleteLink(eTag) {
			var args = {
				/*Add the required headers */
				headers: {
					"Content-Type": link.contentType,
					"If-Match": eTag
				},
				url: link.uri + params
			};

			var deleteDeferred = CLIENT.xhrDelete(args);
			deleteDeferred.addCallback(function(){
				mainDeferred.callback();
			});
			return deleteDeferred;
		}
		
		var deferred = this.getLinkHeaders(link, noLinkContext);
		
		deferred.addCallback(this.getETag);
		deferred.addCallback(doDeleteLink);
		return mainDeferred;
	},

	deleteLinkType: function(linkType, contextUri) {
		var queryParams = {
			"uri": linkType.uri
		};
		var args = {
			headers: {
				"Content-Type": linkType.contentType,
				"If-Match": linkType.eTag
			},

			url: linkType.uri//net.jazz.ajax._contextRoot + "/linktypes?" + dojo.objectToQuery(queryParams)
		};
		return CLIENT.xhrDelete(args, false, contextUri);
	},

	updateLink: function(link, tempLink) {
		var deferred;
		function getETag(ioArgs) {
			return ioArgs.xhr.getResponseHeader("ETag");
		}

		function doUpdateLink(eTag) {
			var args = {
				headers: {
					"Content-Type": "application/x-oslc-common-link+xml",
					"Accept": "application/atom+json",
					"If-Match": eTag
				},
				url: tempLink.uri,
				handleAs: "json",
				hasBody: true,
				putData: tempLink.toXMLString(),
				load: dojo.hitch(this, function(response, ioArgs)
				{
					tempLink.eTag = ioArgs.xhr.getResponseHeader("ETag");

					var entries = response ? response.entry : null;
					if (entries && entries.length === 1) {
						var updatedLink = new rmps.app.ui.internal.editors.views.link.Link();
						updatedLink.parseLinkFeed(entries[0], ioArgs);
						tempLink.value = updatedLink.value;
					}

					return tempLink;
				})
			};
			return CLIENT.xhrPut(args);
		}
		if (tempLink.subject.indexOf(REVIEW_PATH)>-1){
			deferred = this.getLinkHeaders(link, true);			
		} else {
			deferred = this.getLinkHeaders(link);
		}
		deferred.addCallback(getETag);
		deferred.addCallback(doUpdateLink);

		function success(_link) {
			/*Update after the server returned a new e-tag value */
			dojo.mixin(link, _link);
			return link;
		}

		deferred.addCallback(success);
		return deferred;
	},
	
	getETag: function(ioArgs){
		return ioArgs.xhr.getResponseHeader("ETag");
	},
	
	updateLinkType: function(linkType, tempLinkType, contextUri) {
		/*Pass the new updated object to the server */
		dojo.mixin(linkType, tempLinkType);
		
		function doUpdateLinkType(eTag) {
			var args = {
				headers: {
					"Content-Type": IRmpsConstants.OSLC_AM_LINK_TYPE_CONTENT_TYPE,
					"If-Match": eTag
				},
				url: tempLinkType.uri,
				handleAs: "text",
				hasBody: true,
				putData: linkType.toXMLString(),
				load: dojo.hitch(this, function(response, ioArgs)
				{
					tempLinkType.eTag = ioArgs.xhr.getResponseHeader("ETag");
					return tempLinkType;
				})
			};
			return CLIENT.xhrPut(args, false, contextUri);
		}

		if (linkType.eTag) {
			return doUpdateLinkType(linkType.eTag);
		}

		var deferred = this.getLinkTypeHeaders(linkType, contextUri);
		deferred.addCallback(this.getETag);
		deferred.addCallback(doUpdateLinkType);
		
		function success(_linkType) {
			/*Update after the server returned a new e-tag value */
			dojo.mixin(linkType, _linkType);
			return linkType;
		}
		
		deferred.addCallback(success);
		return deferred;
	},

	getLinkHeaders: function(link, noLinkContext) {
		var params = noLinkContext ? "?noLinkContext=true" : "";

		var args = {
			url: link.uri + params,
			handleAs: "text",
			load: dojo.hitch(this, function(response, ioArgs)
			{
				return ioArgs;
			})
		};
		return CLIENT.xhrHead(args);
	},

	getLinkTypeHeaders: function(linkType, contextUri) {
		var args = {
			url: linkType.uri,
			handleAs: "text",
			load: dojo.hitch(this, function(response, ioArgs)
			{
				return ioArgs;
			})
		};
		return CLIENT.xhrHead(args, false, contextUri);
	},

	sortData: function(tableData, orderBy) {

		function stringCompare(string1, string2){
			return string1.toLocaleLowerCase().localeCompare(string2.toLocaleLowerCase());
		}

		function compare(entry1, entry2) {
			var a = dojo.getObject(orderBy, false, entry1);
			var b = dojo.getObject(orderBy, false, entry2);

			if (dojo.isString(a)) {
				a = a.toLowerCase();
			}
			if (dojo.isString(b)) {
				b = b.toLowerCase();
			}
			return stringCompare(a, b);
		}

		function comparator(entry1, entry2) {
			for (var i = 0; i < orderBy.length; ++i) {
				var result = compare(entry1, entry2);
				if (result !== 0) {
					return result;
				}
			}
			return 0;
		}

		tableData.sort(comparator);
	}
};

}());