/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {

/*Provides a way to handle notifications on the link/link types changes accross 
 * different widgets in the current application session. */
dojo.provide("rmps.app.ui.internal.editors.views.link.LinkEventManager");

/*Static array contains information on classes and their notifiers */
var TRACKER= [];

rmps.app.ui.internal.editors.views.link.LinkEventManager= {
	
	addTrackerAndNotify: function(clazz, callbacks, args){
		this.addTracker(clazz, callbacks);
		this.notify(clazz, args);
	},
	
	addTracker: function(clazz, callbacksList){
		var tracker= this._contains(clazz);
		if(tracker){
			var callbacks= tracker.callbacks;
			tracker.callbacks= callbacks.concat(callbacksList); 
		}else{
			TRACKER.push({
				clazz:    clazz,
				callbacks:  callbacksList			
			});			
		}
	},
	
	_contains: function(clazz){
		for(var index=0; index<TRACKER.length; index++){
			var current= TRACKER[index];
			if(current.clazz== clazz){
				return current;
			}
		}
		return null;
	},
	
	removeTracker: function(clazz){
		for(var index=0; index<TRACKER.length; index++){
			if(TRACKER[index].clazz== clazz){
				TRACKER.splice(index, 1);
			}
		}
	},
	
	notifyAllCallback: function(args, callback){
		for(var index=0; index<TRACKER.length; index++){
			var current= TRACKER[index];
			var callbacks= current.callbacks;
			var clazz= current.clazz;
			for(var subIndex=0; subIndex<callbacks.length; subIndex++){
				var currentCallback= callbacks[subIndex];
				if(currentCallback==callback){
					var func= clazz[currentCallback];
					if(func){
						dojo.hitch(clazz, func)(args);	
					}
									
				}
			}	
		}
	},	
	
	notifyAll: function(args){
		for(var index=0; index<TRACKER.length; index++){
			var current= TRACKER[index];
			var callbacks= current.callbacks;
			for(var subIndex=0; subIndex<callbacks; subIndex++){
				callbacks[subIndex](args);
			}	
		}
	},
	
	notify: function(clazz, args){
		for(var index=0; index<TRACKER.length; index++){
			var current= TRACKER[index];
			if(current.clazz==clazz){
				var callbacks= current.callbacks;
				for(var subIndex=0; subIndex<callbacks.length; subIndex++){
					clazz[callbacks[subIndex]](args);
				}				
			}
		}
	},
	
	clear: function(){
		TRACKER= [];
	}

};

})();