/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.editors.views.link.LinkType");

dojo.require("rmps.core.resource.Resource");
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");
dojo.require("rmps.core.util.ILinksConstants");
dojo.require("rmps.core.util.IRmpsConstants");

var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
var ILinksConstants = rmps.core.util.ILinksConstants;
var IRmpsConstants = rmps.core.util.IRmpsConstants;

dojo.declare("rmps.app.ui.internal.editors.views.link.LinkType", rmps.core.resource.Resource, {

	about: null,
	title: null,
	description: null,
	systemDefined: null,
	backlink: null,
	userCreatable: null,

	contentType: IRmpsConstants.OSLC_AM_LINK_TYPE_CONTENT_TYPE,

	constructor: function() {
		this.description = "";
	},

	parseEntry: function(atomEntry, ioArgs) {
		var childNodes = atomEntry.childNodes;
		var eTagNode, linkFeed, contentNode, index, current;
		for (index = 0; index < childNodes.length; index++) {
			current = childNodes[index];
			if (current.localName === "ETag" || current.baseName === "ETag") {
				eTagNode = current;
			} else if (current.localName === "link" || current.baseName === "link") {
				linkFeed = current;
			} else if (current.localName === "content" || current.baseName === "content") {
				contentNode = current;
			}
		}

		if (linkFeed) {
			var hrefNode = linkFeed.getAttributeNode('href');
			if (hrefNode) {
				if (hrefNode.nodeValue) {
					this.uri = hrefNode.nodeValue;
				} else if (hrefNode.childNodes.length > 0 && hrefNode.childNodes[0].nodeName === '#text') {
					this.uri = hrefNode.childNodes[0].nodeValue;
				}
			}
		}

		if (eTagNode && eTagNode.childNodes.length > 0) {
			this.eTag = eTagNode.childNodes[0].textContent;
			if (!this.eTag) {
				this.eTag = eTagNode.childNodes[0].text;
			}
		}

		if (contentNode && contentNode.childNodes.length > 0) {
			for (index = 0; index < contentNode.childNodes.length; index++) {
				current = contentNode.childNodes[index];
				if (current.localName === "LinkType" || current.baseName === "LinkType") {
					var linkType = current;
					childNodes = linkType.childNodes;
					this.about = this.getAttributeValue(linkType, "about", IRmpsConstants.ProcessConstants.NAMESPACE_RDF);
					this.title = this.getNodeValue(childNodes, "title",IRmpsConstants.ProcessConstants.NAMESPACE_DC);
					var blNode = UIHelper.getNode(childNodes, "backlink", ILinksConstants.DMLinkType.NAMESPACE);
					if (blNode) {
						this.backlink = this.getAttributeValue(blNode, "resource", IRmpsConstants.ProcessConstants.NAMESPACE_RDF);
					}
					this.description = this.getNodeValue(childNodes, "description",IRmpsConstants.ProcessConstants.NAMESPACE_DC);
					var systemDefined = this.getNodeValue(childNodes, "systemDefined", ILinksConstants.DMLinkType.NAMESPACE);
					if (systemDefined) {
						this.systemDefined = (systemDefined === "true");
					}
					var userCreatable = this.getNodeValue(childNodes, "userCreatable", ILinksConstants.DMLinkType.NAMESPACE);
					if (userCreatable) {
						this.userCreatable = (userCreatable === "true");
					}
				}
			}
		}
	},

	getNodeValue: function(childNodes, tag, namespace) {
		var descriptionNode = UIHelper.getNode(childNodes, tag,namespace);
		var value;
		if (descriptionNode) {
			value = descriptionNode.textContent;
			if (!value) {
				value = descriptionNode.text;
			}
		}
		return value;
	},

	getAttributeValue: function(node, tag, namespace) {
		var aboutAttr = UIHelper.getAttributeNode(node, tag, namespace);
		if (aboutAttr) {
			return aboutAttr.value;
		}
	},

	toXMLString: function() {
		function rdfNode(property, value) {
			var returnValue = '<rdf:' + property + ' rdf:resource="'+ value +'" /> ';
			return returnValue + "\n";
		}

		var linkXML = '<?xml version="1.0" encoding="UTF-8"?>\n' +
		'<LinkType xmlns="' + IRmpsConstants.OSLC_AM2_XML_NAMESPACE +
		'"\n xmlns:dcterms="' + IRmpsConstants.ProcessConstants.NAMESPACE_DC +
		'"\n xmlns:' + ILinksConstants.DMLinkType.PREFIX + '="' + ILinksConstants.DMLinkType.NAMESPACE +
		'"\n xmlns:rdf="'+ IRmpsConstants.ProcessConstants.NAMESPACE_RDF +
		'"\n rdf:about="'+ this.about +'">\n';
		linkXML += rdfNode('about', jazz.util.html.escape(this.about));
		linkXML += '<dcterms:title>' + jazz.util.html.escape(this.title) + '</dcterms:title>';
		if (this.backlink) {
			linkXML += '<' + ILinksConstants.DMLinkType.PREFIX + ':backlink rdf:resource="'+ this.backlink +'">\n>'+ '</' + ILinksConstants.DMLinkType.PREFIX + ':backlink>';
		}
		linkXML += '<dcterms:description>' + jazz.util.html.escape(this.description) + '</dcterms:description>';
		linkXML += "</LinkType>\n";
		return linkXML;
	}
});

})();
