/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.app.admin.ui.internal.pages.AdminPageController");

dojo.require("rmps.core.error");

dojo.require("rmps.ui.editor.EditorRegistry");
dojo.require("dm.app.admin.core.AdminUtil");
dojo.require("dm.app.admin.ui.internal.pages.AdminWelcome");

dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.requireLocalization("dm.app.admin.ui.internal.pages", "AdminPageMessages");
var messages = dojo.i18n.getLocalization("dm.app.admin.ui.internal.pages", "AdminPageMessages");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var AdminUtil= dm.app.admin.core.AdminUtil;

dojo.declare("dm.app.admin.ui.internal.pages.AdminPageController", null, {

	// private properties
	_adminPage: null,
	_streamViewer: null,
	_streamEditorInput: null,
	_baselineStatusEditorInput: null,
	_requestBaselineEditorInput: null,


	_defaultUrl: null,

	constructor: function(adminPage) {
		this._adminPage = adminPage;
	},

	_default: function(args) {
		setTimeout(dojo.hitch(this, function() {
			window.location.href = this._adminPage.getDefaultUri();
		}), 0);
	},

	welcome: function() {
		var pageHelper= this._adminPage.getPageHelper();
		if (!this._welcome) {
			this._welcome = new dm.app.admin.ui.internal.pages.AdminWelcome();
			pageHelper.getEditorView().addChild(this._welcome);
			
			this._adminPage.connect(pageHelper,"onEditorActivated", dojo.hitch(this, function () {
				this._welcome.domNode.style.display= 'none';
			}));
		}
		
		pageHelper.hideActiveEditor();
		
		/*_setStyleAttr("display: ''") did not work*/
		this._welcome.domNode.style.display= '';
		
		this._adminPage.getPageHelper().clearLastViewed();
		net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(messages.welcomeToAdmin);
	},

	requestBaseline: function(args) {
		var workspaceUri = null;
		var workspaceName = null;
		if (args) {
			workspaceUri = args.workspaceUri;
			workspaceName = args.workspaceName;
		}
		
		if(!this._requestBaselineEditorInput || this._requestBaselineEditorInput.getWorkspaceUri() !== workspaceUri){
			var editorInputDeferred = rmps.ui.editor.EditorRegistry.createEditorInput(
			AdminUtil.REQUEST_BASELINE_EDITOR, {projectId: jazz.app.context.get().itemId,
				workspaceUri: workspaceUri, workspaceName: workspaceName});
			
			editorInputDeferred.then(dojo.hitch(this, function(editorInput) {
				this._requestBaselineEditorInput = editorInput;
				this._adminPage.pageHelper.openEditor(this._requestBaselineEditorInput);
			}));
		}
		else if (this._requestBaselineEditorInput) {
			this._adminPage.pageHelper.openEditor(this._requestBaselineEditorInput);
		}
	},
	
	baselineStatus: function() {
		if(!this._baselineStatusEditorInput){
			var editorInputDeferred = rmps.ui.editor.EditorRegistry.createEditorInput(
			AdminUtil.BASELINE_STATUS_EDITOR, {projectId: jazz.app.context.get().itemId});
			
			editorInputDeferred.then(dojo.hitch(this, function(editorInput) {
				this._baselineStatusEditorInput = editorInput;
				this._adminPage.pageHelper.openEditor(this._baselineStatusEditorInput);
			}));
		}
		else {
			this._adminPage.pageHelper.openEditor(this._baselineStatusEditorInput);
		}
	},
	
	requestStream: function(args) {
		var snapshotUri = null;
		if (args) {
			snapshotUri = args.snapshotUri;
		}
		if(!this._streamEditorInput || this._streamEditorInput.getSnapshotUri() !== snapshotUri){
			var editorInputDeferred = rmps.ui.editor.EditorRegistry.createEditorInput(
			AdminUtil.STREAM_EDITOR, {projectId: jazz.app.context.get().itemId, snapshotUri: snapshotUri});
			
			editorInputDeferred.then(dojo.hitch(this, function(editorInput) {
				this._streamEditorInput = editorInput;
				var editorView = this._adminPage.getEditorView();
				this._adminPage.getSideNavigation().addContent(editorView);
				this._adminPage.pageHelper.openEditor(this._streamEditorInput);
			}));
		}
		else if (this._streamEditorInput) {
			var editorView = this._adminPage.getEditorView();
			this._adminPage.getSideNavigation().addContent(editorView);
			this._adminPage.pageHelper.openEditor(this._streamEditorInput);
		}
	},

	streamViewer: function() {
		if(!this._streamViewer){
			this._streamViewer = rmps.ui.editor.EditorRegistry.createEditorInput(
			AdminUtil.STREAM_VIEWER_EDITOR, {projectId: jazz.app.context.get().itemId});
			
			editorInputDeferred.then(dojo.hitch(this, function(editorInput) {
				this._streamViewer = editorInput;
				var editorView = this._adminPage.getEditorView();
				this._adminPage.getSideNavigation().addContent(editorView);
				this._adminPage.pageHelper.openEditor(this._streamViewer);
			}));
		}
		else {
			var editorView = this._adminPage.getEditorView();
			this._adminPage.getSideNavigation().addContent(editorView);
			this._adminPage.pageHelper.openEditor(this._streamViewer);
		}
	}
	
});


})();
