/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.analysis                                       |
 *|                                                                        |
 *| Copyright IBM Corporation 2011, 2012.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
(function() {
dojo.provide("dm.app.analysis.ui.internal.pages.AnalysisWelcome");

/** The widget for creating the analysis welcome page. The
 *  page has content sections for each analysis application
 *  extension (reports, traceability, impact and validation).
 *  
 *  This widget should be extended by the analysis applications
 *  in order to add their content to this page. Each such application 
 *  should use the dojo aspect advise mechanism to extend the 
 *  method for adding extension section content. 
 *  
 *  @author Kevin Cornell
 */

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("dm.app.analysis.ui.internal.application.AnalysisActions");
dojo.require("dm.app.analysis.ui.internal.util.AnalysisIcons");
dojo.require("dm.app.analysis.ui.AnalysisWelcomeUtil");

dojo.require("dojo.i18n");
dojo.requireLocalization("dm.app.analysis.ui.internal", "AnalysisMessages");
var messages  = dojo.i18n.getLocalization("dm.app.analysis.ui.internal", "AnalysisMessages");

var AnalysisActions = dm.app.analysis.ui.internal.application.AnalysisActions;
var AnalysisIcons   = dm.app.analysis.ui.internal.util.AnalysisIcons;
var AnalysisWelcomeUtil = dm.app.analysis.ui.AnalysisWelcomeUtil;

dojo.declare("dm.app.analysis.ui.internal.pages.AnalysisWelcome", rmps.app.ui.internal.editors.BaseEditor, {
	
	"class": "analysis-AnalysisWelcome",
	
	// base editor properties
	isRefreshable: false,
	isReadOnly: true,
	
	// properties to enable extensions to add their content
	_section: null,
	_sectionContainer: null,
	
	constructor: function(args) {
		//The following is required for Application-Specific implementations of
		//createContentWidget to have access to the editor
		AnalysisWelcomeUtil.editor = this;
	},

	createContentWidget: function() {
		return new dm.app.analysis.ui.internal.pages.AnalysisWelcome._Content({editor: this});
	},
	
	addButtons: function(toolbar) {
		// no commands buttons on Welcome page
	},
	
	
	/** Return the name of the analysis section being
	 *  configured. The return value will be one of the
	 *  constants:  AnalysisActions.SECTION_*.
	 */
	getCurrentSection: function() {
		return this._section;
	},
	
	/** Return the name of the analysis section being
	 *  configured. The return value will be one of the
	 *  constants:  AnalysisActions.SECTION_*.
	 */
	getSectionContainer: function() {
		return this._sectionContainer;
	},
	
	/** This method enables the Welcome Content widget to get the
	 *  extension content registered against the welcome page editor.
	 *  
	 *  @param sectionName the AnalysisActions.SECTION_xxx name
	 *  @param sectonContainer the widget where section content is added
	 */
	_getContentForSection: function(sectionName, sectionContainer) {
		this._section = sectionName;
		this._sectionContainer = sectionContainer;
		this.addExtendedSectionContent();
	},
	
	/** This method is provided so that the analysis applications
	 *  can add their content to the Welcome page. This method is
	 *  called multiple times, once for each section. The section
	 *  name is set before each call and each application extending
	 *  this page must only add its welcome page content when the current
	 *  section corresponds to their section (e.g., getCurrentSection()).
	 *  
	 *  The container widget for the corresponding section is obtained
	 *  by calling getSectionContainer().
	 *  
	 *  The applications should extend this method using the
	 *  dojo aspect advise mechanism. In the application extension
	 *  class (e.g., MyWelcomeInitializer), add the method declaration
	 *  as follows::
	 *  
	 *  dojox.lang.aspect.advise(dm.app.analysis.ui.internal.pages.AnalysisWelcome, 
	 *							"addExtendedSectionContent", 
	 *      {
	 *      after: function() {
	 *          var welcomePage = dojox.lang.aspect.getContext().instance;
	 *          if (welcomePage.getCurrentSection() === AnalysisActions.SECTION_MySection) {
	 *             var containerNode = welcomePage.getSectionContainer();
	 *             var myWelcome = new MyWelcomeContent();
	 *             containerNode.appendChild(myWelcome.domNode);
	 *          }
	 *      }
	 *  });
	 *
	 *  In order to connect the application extension so that it can be called,
	 *  the following extension should be defined in the plugin.xml of the 
	 *  analysis application.
	 *  
	 *  <extension point="com.ibm.xtools.rmps.web.app.webUiInitializers">
	 *	  <initializer jsModule="path.of.class.MyWelcomeInitializer">
	 *		<classBinding jsClass="dm.app.analysis.ui.internal.pages.AnalysisWelcome" />
	 *	  </initializer>
	 *  </extension>
	 *
	 */
	addExtendedSectionContent: function(){ /* extensions must implement this method */ }


});

dojo.declare("dm.app.analysis.ui.internal.pages.AnalysisWelcome._Content", [dijit._Widget, dijit._Templated], {

	"class": "analysisWelcome",

	messages: messages, 
	
	widgetsInTemplate: true,
	templateString: 
		'<div>' +
			'<div class="RightContent">' +
				'<div class="RightContentTitle">${messages.accessAndManage}' +
					'<ol>' +
					'<li>${messages.useTheNavigation}</li>' +
					'<li>${messages.searchForResources}</li>' +
					'<li>${messages.organizeProjects}' +
					     '<a class=\'miniDashboard\' href="http://pic.dhe.ibm.com/infocenter/clmhelp/v4r0/index.jsp?topic=/com.ibm.jazz.dashboard.doc/topics/c_about_dash_mini.html" target="_blank">' +
					     ' ${messages.miniDashboard}</a>.</li>' +
					'</ol>' +
				'</div>' +
				
				'<div class="showMe">' +
					'<div class="showMeLeft">' +
						'<div class="showMeImage"><img dojoAttachPoint="_videoIcon"></div>' +
					'</div>' +
					'<div class="showMeRight">' +
						'<div class="showMeLink"><a href="http://jazz.net/library/video/494" target="blank">${messages.showMeTitle}</a></div>' +
					'</div>' +
				'</div>' +

				'<div class="learnMore">' +
					'<div class="learnLeft">' +
						'<div class="LearnImage"><img dojoAttachPoint="_infoIcon"></div>' +					
					'</div>' +
					'<div class="learnRight">' +
						'<div class="learnLink"><a href="/rdmhelp/topic/com.ibm.dm.reporting.doc/topics/t_web_over.html" target="blank">' +
							'${messages.learnAboutReports}</a></div>' +
						'<div class="learnLink"><a href="/rdmhelp/topic/com.ibm.dm.analysis.doc/topics/t_dm_analysis_over.html" target="blank">' +
							'${messages.learnAboutAnalysis}</a></div>' +
					'</div>' +
				'</div>' +
			'</div>' +
	
			'<div class="LeftContent" dojoAttachPoint="_leftContent" >' +
				'<div class="LeftContentImage" ></div>' +
				'<div class="reportsSectionTitle" dojoAttachPoint="_reportsSectionTitle">${messages.reportsSectionTitle}' +
					'<div class="reportsSection" dojoAttachPoint="_reportsSection" />' +
				'</div>' +
				'</div class="sectionSpacing" />' +
				'<div class="traceabilitySectionTitle" dojoAttachPoint="_traceabilitySectionTitle">${messages.traceabilitySectionTitle}' +
					'<div class="traceabilitySection" dojoAttachPoint="_traceabilitySection" />' +
				'</div>' +
				'</div class="sectionSpacing" />' +
				'<div class="impactSectionTitle" dojoAttachPoint="_impactSectionTitle">${messages.impactSectionTitle}' +
					'<div class="impactSection" dojoAttachPoint="_impactSection" />' +
				'</div>' +
				'</div class="sectionSpacing" />' +
				'<div class="validationSectionTitle" dojoAttachPoint="_validationSectionTitle">${messages.validationSectionTitle}' +
					'<div class="validationSection" dojoAttachPoint="_validationSection" />' +
				'</div>' +
			'</div>' +
		'</div>',
	
	// init properties
	editor: null,
	
	constructor: function(args) {
		this.messages = messages;
	},

	
	postCreate: function() {
		this.inherited(arguments);
		
		// Add the various section content to the welcome page
		this.editor._getContentForSection(AnalysisActions.SECTION_REPORTS, this._reportsSection);
		if (this._reportsSection.children.length === 0) {
			dojo.style(this._reportsSectionTitle, "display", "none");
		}
		this.editor._getContentForSection(AnalysisActions.SECTION_TRACEABILITY, this._traceabilitySection);
		if (this._traceabilitySection.children.length === 0) {
			dojo.style(this._traceabilitySectionTitle, "display", "none");
		}
		this.editor._getContentForSection(AnalysisActions.SECTION_IMPACTANALYSIS, this._impactSection);
		if (this._impactSection.children.length === 0) {
			dojo.style(this._impactSectionTitle, "display", "none");
		}
		this.editor._getContentForSection(AnalysisActions.SECTION_VALIDATION, this._validationSection);
		if (this._validationSection.children.length === 0) {
			dojo.style(this._validationSectionTitle, "display", "none");
		}
		
		// Add the icons for the right panel
		this._videoIcon.src = AnalysisIcons.VIDEO;
		this._infoIcon.src = AnalysisIcons.INFO;
		
		this.editor.containerNode.appendChild(this.domNode);
		this.startup();
	},
	
	startup: function() {
		this.inherited(arguments);	
	},
	
	/** Handle the resizing of the browser window.
	 *  The left content widget of the welcome page
	 *  contains the content for all of the analysis 
	 *  sections. These will be larger than the 
	 *  welcome page so the left content widget is
	 *  set for auto-scrolling. However, when the
	 *  window is resized, the height of that left
	 *  content widget must be modified so that the
	 *  vertical scrollbar will be shown.
	 */
	resize: function(size) {
		this.inherited(arguments);
		if (!size) {
			return;
		}
		if (size.h) {
			this._leftContent.style.height = size.h + "px";
		}
	}
});

})();
