/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.reports                                        |
 *|                                                                        |
 *| Copyright IBM Corporation 2011, 2012.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
dojo.provide("dm.app.reports.ui.internal.pages.DeployTemplates");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("jazz.ui.Breadcrumbs");

dojo.require("dm.app.reports.ui.internal.application.ReportActions");
dojo.require("dm.app.reports.ui.internal.util.ReportServiceClient");
dojo.require("dm.app.reports.ui.internal.util.ReportUtil");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.StyledTable");
dojo.require("dijit.form.CheckBox");
dojo.require("jazz.util.html");
var HTML = jazz.util.html;

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.requireLocalization("dm.app.reports.ui.internal", "ReportMessages");
var messages  = dojo.i18n.getLocalization("dm.app.reports.ui.internal", "ReportMessages");

var ReportActions = dm.app.reports.ui.internal.application.ReportActions;
var ReportServiceClient = dm.app.reports.ui.internal.util.ReportServiceClient;
var ReportUtil = dm.app.reports.ui.internal.util.ReportUtil;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;


//Define the editor help UA key
var PAGE_UAKEY = "com.ibm.xtools.rmps.web.reports.page_DeployTemplates001";

var DIRTY_GROUP_ID  = "dm.app.reports.DeployTemplates.dirty";

// Define the column names for the table
var TABLE_CHECKBOX = "columnCheckbox";
var TABLE_NAME     = "columnName";


dojo.declare("dm.app.reports.ui.internal.pages.DeployTemplates", rmps.app.ui.internal.editors.BaseEditor, {
	
	"class": "reports-DeployTemplates",
	
	// base editor properties
	isRefreshable: false,
	isReadOnly: false,
	
	// Keep a list of the available templates that can be deployed
	// and a list of the selected template IDs (reportId not reportUUID)
	_templates: null,
	_selectedTemplateIDs: null,
	_saveCount: 0,
		
	// Keep a handle to the page contents
	_pageContent: null,
	_breadCrumbs: null,

	postCreate: function() {	
		// Add the "Report Templates" bread crumb at the top of the editor
		this._breadCrumbs = new jazz.ui.Breadcrumbs({segments: [{label: messages.templateBrowserName, 
																 href: ReportActions.getActionUri(ReportActions.ACTION_VIEW_TEMPLATES)}]});
		dojo.place(this._breadCrumbs.domNode, this.headerContainerNode, "first");

		this.inherited(arguments);
	},
	
	/** Create the content widget for the predefined template deployment editor.
	 * 
	 * @return the widget that displays the content
	 */
	createContentWidget: function() {
		// Specify the help for this editor
		this.uaKey = PAGE_UAKEY;

		// Create the widget to display the template content
		this._pageContent = new dm.app.reports.ui.internal.pages.DeployTemplates._Content({editor: this});
		return this._pageContent;
	},

	/** When the editor is activated, obtain the corresponding
	 *  report, report template and parent folder. When all three
	 *  requests are complete, and update the editor tabs.
	 */
	activate: function() {
		this.inherited(arguments);
		
		// If the editor has not been loaded yet
		if (!this._templates) {
			
			// Initialize the list of report UUIDs to save
			this._templates = null;
			this._selectedTemplateIDs = [];
			
			// Issue a request to find missing templates that can be deployed
			var serviceObject = {
					self: this,
					success: function(templates) {
						this.self._loadTemplates(templates);
					},
							
					failure: function(error) {
						this.self.serviceError(messages.errorLoadingTemplates, error);
					}
				};
			var args = {
					processAreaUUID: ReportUtil.getProjectId()
				};
			var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
			ReportServiceClient.getMissingSharedReportDescriptors(srh, args);
		}		
	},

	/** Add the cancel button. Save button is added because "isReadOnly" is false.
	 * 
	 * @param toolbar: the base editor page toolbar.
	 */
	addButtons: function(toolbar) {
		toolbar.addWidget(new rmps.ui.SimpleButton({
			label: messages.cancel,
			onClick: dojo.hitch(this, this._handleCancel)
		}));
		
		this.inherited(arguments);
	},
	
	/** Perform the saving of the shared report descriptors
	 * 
	 */
	doSave: function(taskMonitor) {
		this.inherited(arguments);
		
		// At this point the list of report UUIDs to
		// deploy should be greater than zero
		var serviceObject = {
			self: this,
			success: function() {
				// Saving is not complete until all templates have been saved.
				this.self._saveCount--;
				if (this.self._saveCount <= 0) {
					this.self._saveComplete();
				}
			},
					
			failure: function(error) {
				this.self._saveCount--;
				this.self.serviceError(messages.errorSavingTemplate, error);
			}
		};
		
		// The method postSaveMissingReportDescriptors() cannot be used because
		// it creates new copies of the templates with caching enabled. Save
		// each template individually. 
		this._saveCount = this._selectedTemplateIDs.length;
		for (var index = 0; index < this._selectedTemplateIDs.length; index++) {
			// Find the selected template
			var selectedId = this._selectedTemplateIDs[index];
			var template = null;
			for (var i = 0; i < this._templates.length; i++){
				if (this._templates[i].reportId === selectedId) {
					template = this._templates[i];
			
					// Ensure that deployed template is shared and does not
					// support caching (e.g., do not cache DM report results)
					template.shared = true;
					template.supportsDataCaching = false;
					
					// The "missing" templates that were loaded were temporary 
					// and their UUIDs are not valid. The UUID must be deleted
					// before saving so that a new permanent UUID is assigned.
					delete template.reportUUID;
					break;
				}
			}
			if (template) {
				var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
				ReportServiceClient.postSaveReportDescriptor(srh, template);
			}
		}
		
		
		// The TaskMonitor cannot be used to handled the deferred work from the
		// request because it overwrites the specific error message & details
		// with a generic "Bad Request" message that means nothing.
	},
	
	/** Load the templates available to be deployed
	 *  into the editor.
	 */
	_loadTemplates: function(templates) {
		// If there are no templates left to deploy
		if (!templates || templates.length === 0) {
			this._templates = [];
			
			// Display a message indicating that fact
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: messages.noTemplatesFound,
				details: null,
				closable: true
			});
		}
		
		// else there is at least one template that can be deployed
		else {
			// Sort the templates by name
			var sortByName = function(a,b) {
				return a.name.localeCompare(b.name);
			};
			this._templates = templates;
			this._templates.sort(sortByName);
		}
		
		// Update the content widget
		this._pageContent.initialize(this._templates);		
	},
	
	/** The saving of the new deployed templates was successful.
	 * 
	 */
	_saveComplete: function() {
		
		// Remove the dirty indicator
		this.removeDirty(null, DIRTY_GROUP_ID);
		
		// Close this editor and remove it from the recently and unsaved editor lists.
		var pageHelper = rmps.ui.editor.PageHelper.getInstance(ReportActions.PAGE_ID);
		pageHelper.closeEditor(this, true, true);
		
		// Force the template browser to be opened and refreshed.
		window.location.href = ReportActions.getActionUri(ReportActions.ACTION_VIEW_TEMPLATES);
	},

	/** When Cancel is clicked in the create report editor, go
	 *  back to the previous page.
	 */
	_handleCancel: function() {
		// Remove the dirty indicator
		this.removeDirty(null, DIRTY_GROUP_ID);
		
		// Close this editor and remove it from the recently and unsaved editor lists.
		var pageHelper = rmps.ui.editor.PageHelper.getInstance(ReportActions.PAGE_ID);
		pageHelper.closeEditor(this, true, true);

		window.history.back();
	},
	
	/** Change the selection of a template to deploy.
	 * 
	 * @param templateID the template ID
	 * @param selected whether the template entry was selected or not
	 */
	templateSelected: function(templateId, selected) {
		// Determine if the template is already selected
		var selectedIndex = -1;
		for (var index = 0; index < this._selectedTemplateIDs.length; index++) {
			if (this._selectedTemplateIDs[index] === templateId) {
				selectedIndex = index;
				break;
			}
		}
		
		// If the template was not selected but now it is
		if (selectedIndex < 0 && selected) {
			// Add the template ID to the selected list
			this._selectedTemplateIDs.push(templateId);
			this.addDirty(templateId, DIRTY_GROUP_ID);
		}
		
		// else if the template was selected but now it is not
		else if (selectedIndex >= 0 && !selected) {
			// Remove the template ID from the selected list
			this._selectedTemplateIDs.splice(selectedIndex, 1);
			this.removeDirty(templateId, DIRTY_GROUP_ID);
		}
	},
		
	/** Report an error from a reporting service request.
	 * 
	 * @param errorMessage the main error message
	 * @param error the service error (details for error message)
	 */
	serviceError: function(errorMessage, error) {
		var details = null;
		if (error.message) {
			details = [ {message: error.message} ];
		} else if (error.errorClass) {
			details = [ {message: error.errorClass} ];
		}
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: errorMessage,
			details: details,
			closable: true
		});
	}


});

/** Contents for deploying template resources.
 * 
 */
dojo.declare("dm.app.reports.ui.internal.pages.DeployTemplates._Content", [dijit._Widget, dijit._Templated], {

	"class": "deployTemplates",
	
	messages: messages,
	
	widgetsInTemplate: true,
	templateString: 
		'<div>' +
			'<div class="templateInfo"  dojoAttachPoint="_templateInfo">${messages.deployTemplatesInfo}</div>' +
			'<div class="templateTable" dojoAttachPoint="_templateTable"></div>' +
		'</div>',
	
	// init properties
	editor: null,
	
	_table: null,
	
	postCreate: function() {
		this.inherited(arguments);
		this._table = new jazz.ui.StyledTable(
				{columns: [{label: "", width: "2%",  sortable: false, className: TABLE_CHECKBOX},
				           {label: messages.deployTemplateName, width: "98%", sortable: false, className: TABLE_NAME}],
							trim: jazz.ui.styledTable.TrimStyles.GOLD,
							sortBy: 0});
		this._templateTable.appendChild(this._table.domNode);
	},
	
	/** Initialize the table with the list of available
	 *  templates that have not been deployed yet.
	 *  
	 *  @param templates an array of available templates
	 */
	initialize: function(templates) {
		if (templates) {
			// For each available template
			for (var index = 0; index < templates.length; index++) {
				// Create the entry in the table for the template
				var row = this._createTemplateEntry(templates[index]);
				this._table.tbody.appendChild(row);
			}
		}		
	},
	
	/** Create an entry (row) for the table of available
	 *  templates. Each row contains the check box button
	 *  and the template name.
	 */
	_createTemplateEntry: function(template) {
		// Create the element for the table row
		var row = document.createElement("tr");
		
		// Create a column for the check box
		var td  = document.createElement("td");
		td.className = TABLE_CHECKBOX;
		var checkbox = new dijit.form.CheckBox({
			onChange: dojo.hitch(this, function(isSelected) {
				this.editor.templateSelected(template.reportId, isSelected);})
			});
		td.appendChild(checkbox.domNode);
		row.appendChild(td);
		
		// Create a column for the template name
		td  = document.createElement("td");
		td.className = TABLE_NAME;
		td.innerHTML = HTML.escape(template.name);
		row.appendChild(td);
		
		return row;
	}
});

})();
