/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.pages.DesignSearchProvider");

dojo.require("rmps.app.core.internal.util.QueryUtil");
dojo.require("rmps.app.core.internal.query.TextQuery");
dojo.require("rmps.app.ui.internal.search.QuickSearchAdvancedOptions");

dojo.require("rmps.app.core.util");
dojo.require("rmps.core.RootServices");
dojo.require("rmps.core.client");
dojo.require("rmps.core.util.BaselineUtil");

dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("jazz.ui._SearchProvider");
dojo.require("jazz.app.context");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.app.ui.internal.pages", "DesignSearchProviderMessages");

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages", "DesignSearchProviderMessages");

var QueryUtil = rmps.app.core.internal.util.QueryUtil;
var BaselineUtil = rmps.core.util.BaselineUtil;

dojo.declare("rmps.app.ui.internal.pages.DesignSearchProvider", jazz.ui._SearchProvider, {
	
	scopes: [
			{ id: QueryUtil.SCOPE.ALL.value, label: QueryUtil.SCOPE.ALL.name, icon: QueryUtil.SCOPE.ALL.icon },
			{ id: QueryUtil.SCOPE.DIAGRAMS.value, label: QueryUtil.SCOPE.DIAGRAMS.name, icon: QueryUtil.SCOPE.DIAGRAMS.icon }
		],
	
	tooltip: messages.searchModels,
	
	pageSize: 50,
	limit: 200,
	orderBy: [{id: "modified", isAscending: false}, {id: "shortName", isAscending: true}],
	_loadResourceDeferred: null,
	
	onExecute: function(args) {
		var searchText = args.query;
		
		if (searchText.length === 0) {
			return;
		}

		var query = new rmps.app.core.internal.query.TextQuery();
		query.pageSize = this.pageSize;
		query.limit = this.limit;
		query.text = dojo.trim(args.query);
		query.orderBy = this.orderBy;
		query.resourcesOnly = false;
		query.scope = QueryUtil.SCOPE.valueOf(args.scope);
		query.searchById = true;

		// search type
		if ( args.advanced && this._advancedOptions.searchTypes !== "all" ) {
			query.onlyType = this._advancedOptions.searchTypeId;
		}

		// search scope
		if ( args.advanced && this._advancedOptions.searchScope !== "current" ) {
			query.baseline = "";
			query.projectId = this._advancedOptions.searchScopeId;
			query.scopeDescription = this._advancedOptions.searchScopeDescription;
		} else {
			query.projectId = jazz.app.context.get().itemId;
			query.baseline = jazz.app.context.get().baseline;
			query.changeset = jazz.app.context.get().changeset;
		}
		
		var timestamp = new Date().getTime();
		window.location.href = rmps.app.core.util.getSearchUri(query.toQueryString()) + "&ts=" + timestamp;
	},

	createAdvancedOptions: function() {
		if (this._advancedOptions) {
			this._advancedOptions.destroyRecursive();
		}
		this._advancedOptions = new rmps.app.ui.internal.search.QuickSearchAdvancedOptions();
		return this._advancedOptions.domNode;
	}
});

})();
