/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.reports                                        |
 *|                                                                        |
 *| Copyright IBM Corporation 2012.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
dojo.provide("dm.app.reports.ui.internal.pages.ExportDialog");

/** Dialog to select the type of output to export for a report.
 *  The dialog should be constructed with 2 arguments:
 *  
 *  engine: the engine descriptor from either the report or the template
 *  onOk: callback method onOk(formatValue) 
 * 
 * @author Kevin Cornell
 */
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.Dialog");

dojo.require("dojo.i18n");
dojo.requireLocalization("dm.app.reports.ui.internal", "ReportMessages");
var messages = dojo.i18n.getLocalization("dm.app.reports.ui.internal", "ReportMessages");

dojo.declare("dm.app.reports.ui.internal.pages.ExportDialog", [dijit._Widget, dijit._Templated], {

	"class": "exportDialog",
	
	// Constructor parameters
	engine: null,
	onOk: null,

	messages: messages, 
	
	widgetsInTemplate: true,
	templateString: 
		'<div dojoAttachPoint="_dialogContent">' +
			'<div class="selectorContainer">' +
				'<select class="formatSelect" dojoAttachPoint="_formatSelect" ></select>' +
			'</div>' +
			'<table align=right>' +
				'<tr>' +
					'<td><button dojoAttachPoint="_okButton">${messages.ok}</button></td>' +
					'<td><button dojoAttachPoint="_cancelButton">${messages.cancel}</button></td>' +
				'</tr>' +
			'</table>' +
		'</div>',
	
	constructor: function(args) {
		if (args.engine) {
			this.engine = args.engine;
		} else {
			this.engine = { exportValues: [] };
		}
		this.onOk = args.onOk;
	},
	
	postCreate: function() {
		// Add the engine output options to the select control
		var index = 0;
		for (var i = 0; i < this.engine.exportValues.length; i++) {
			var value = this.engine.exportValues[i];
			var label = value;
			if (this.engine.exportLabels && this.engine.exportLabels.length > i) {
				label = this.engine.exportLabels[i];
			}
			if (value) {
				var option = document.createElement("option");
				option.value = value;
				option.text = label;
				this._formatSelect.options[index++] = option;
			}
		}
		
		this.dialog = new jazz.ui.Dialog({
			contentNode: this._dialogContent,
			modal: true,
			primaryTitle: messages.exportDialogName,
			width: "375px",
			isClosable: true
		});
		this.connect(this._okButton,     "onclick",  this._okPressed);
		this.connect(this._cancelButton, "onclick",  this._cancelPressed);
		this._cancelButton.focus();
	},
	
	close: function() {
		this.dialog.close();
	},
	
	_okPressed: function() {
		// Get the selected value
		var selectIndex = this._formatSelect.selectedIndex;
		var formatValue = "";
		if (selectIndex < this._formatSelect.options.length) {
			formatValue = this._formatSelect.options[selectIndex].value;
		}

		// Call back into the corresponding editor
		this.dialog.close();
		this.onOk(formatValue);
	},
	
	_cancelPressed: function() {
		this.dialog.close();
	}
	
});

})();