/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.reports                                        |
 *|                                                                        |
 *| Copyright IBM Corporation 2011, 2012.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
dojo.provide("dm.app.reports.ui.internal.pages.FolderDialog");

/** Dialog to update the name and description of a folder.
 *  The dialog should be constructed with 2 arguments:
 *  
 *  folder: the current folder or null
 *  onOk: callback method onOk(folder) where the name and description are updated
 * 
 * @author Kevin Cornell
 */
dojo.require("dm.app.reports.ui.internal.util.ReportUtil");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.Dialog");

dojo.require("dojo.i18n");
dojo.requireLocalization("dm.app.reports.ui.internal", "ReportMessages");
var messages = dojo.i18n.getLocalization("dm.app.reports.ui.internal", "ReportMessages");

var ReportUtil = dm.app.reports.ui.internal.util.ReportUtil;

dojo.declare("dm.app.reports.ui.internal.pages.FolderDialog", [dijit._Widget, dijit._Templated], {

	"class": "folderDialog",
	
	// Constructor parameters
	folder: null,
	onOk: null,

	messages: messages, 
	
	widgetsInTemplate: true,
	templateString: 
		'<div dojoAttachPoint="_dialogContent">' +
			'<div class="nameRow">' +
				'<div class="nameLabel">${messages.folderName}</div>' +
				'<div class="nameContainer">' + 
					'<input class="nameInput" dojoAttachPoint="_nameInput" />' +
				'</div>' +
			'</div>' +
			'<div class="descriptionLabel">${messages.folderDescription}</div>' +
			'<div class="descriptionContainer">' +
				'<textarea class="descriptionInput" dojoAttachPoint="_descriptionInput" rows="5" ></textarea>' +
			'</div>' +
			'<table align=right>' +
				'<tr>' +
					'<td><button dojoAttachPoint="_okButton" disabled="disabled">${messages.ok}</button></td>' +
					'<td><button dojoAttachPoint="_cancelButton">${messages.cancel}</button></td>' +
				'</tr>' +
			'</table>' +
		'</div>',
	
	constructor: function(args) {
		if (args.folder) {
			this.folder = args.folder;
		} else {
			this.folder = { name: "", description: "" };
		}
		this.onOk = args.onOk;
	},
	
	postCreate: function() {
		// Update the content of the name and description controls
		this._nameInput.value = this.folder.name || "";
		this._descriptionInput.value = this.folder.description || "";
		
		this.dialog = new jazz.ui.Dialog({
			contentNode: this._dialogContent,
			modal: true,
			primaryTitle: messages.folderDialogName,
			width: "375px",
			isClosable: true
		});
		this.connect(this._okButton,     "onclick",  this._okPressed);
		this.connect(this._cancelButton, "onclick",  this._cancelPressed);
		ReportUtil.connectTextInput(this._nameInput, this, "_handleNameChange");
		
		// Update the OK button for the initial name 
		this._handleNameChange();
		this._cancelButton.focus();
	},
	
	close: function() {
		this.dialog.close();
	},
	
	_okPressed: function() {
		// Get the values from the name and description controls
		this.folder.name = dojo.trim(this._nameInput.value);
		this.folder.description = dojo.trim(this._descriptionInput.value);
		
		// Call back into the corresponding editor
		this.dialog.close();
		this.onOk(this.folder);
	},
	
	_cancelPressed: function() {
		this.dialog.close();
	},
	

	_handleNameChange: function () {
		if (this._nameInput.value && this._nameInput.value !== "") {
			this._okButton.disabled = false;
		} else {
			this._okButton.disabled = true;
		}
	}
	
});

})();