/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.reports                                        |
 *|                                                                        |
 *| Copyright IBM Corporation 2011.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
dojo.provide("dm.app.reports.ui.internal.pages.ReportBrowser");

/** Editor for browsing report configurations. Report configurations can
 *  either be personal or shared. Each report has a description,
 *  and an associated report template.
 *  
 *  @author Kevin Cornell
 */

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("dm.app.reports.ui.internal.pages.ReportBrowserContent");

dojo.require("dm.app.reports.ui.internal.application.ReportActions");
dojo.require("dm.app.reports.ui.internal.util.Icons");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.requireLocalization("dm.app.reports.ui.internal", "ReportMessages");
var messages  = dojo.i18n.getLocalization("dm.app.reports.ui.internal", "ReportMessages");

var ReportActions = dm.app.reports.ui.internal.application.ReportActions;
var Icons = dm.app.reports.ui.internal.util.Icons;

// Define the editor help UA key
var PAGE_UAKEY = "com.ibm.xtools.rmps.web.reports.page_ReportBrowser001";


dojo.declare("dm.app.reports.ui.internal.pages.ReportBrowser", rmps.app.ui.internal.editors.BaseEditor, {
	
	"class": "reports-ReportBrowser",
	
	// base editor properties
	isRefreshable: true,
	isReadOnly: true,
	
	// Define properties used by this editor
	_pageContent: null,
	_doRefreshOnActivate: false,
	
	createContentWidget: function() {
		// Specify the help for this editor
		this.uaKey = PAGE_UAKEY;
		
		// Create the contents for the browser
		this._pageContent = new dm.app.reports.ui.internal.pages.ReportBrowserContent({editor: this, inBrowser: true });
		return this._pageContent;
	},
	
	/** Add the create report button.
	 * 
	 * @param toolbar: the base editor page toolbar.
	 */
	addButtons: function(toolbar) {
		toolbar.addButton(this, this._createNewReport, {imgSrc: Icons.CREATE_REPORT, label: messages.createNewReport, showLabel: true});
		this.inherited(arguments);
	},

	/** Create a new report (query descriptor). Report creation is
	 *  performed by the DM report editor page.
	 */
	_createNewReport: function() {
		window.location.href = ReportActions.getActionUri(ReportActions.ACTION_CREATE_REPORT) +
			"&name=" + encodeURIComponent(messages.createNewReport);
	},

	/** When the editor is activated, do a refresh.
	 * 
	 */
	activate: function() {
		this.inherited(arguments);
		if (this._doRefreshOnActivate) {
			this._pageContent.refresh();
		}
	},
	
	deactivate: function() {
		this._doRefreshOnActivate = true;
	},
	
	
	/** Report an error from a reporting service request.
	 * 
	 * @param errorMessage the main error message
	 * @param error the service error (details for error message)
	 */
	serviceError: function(errorMessage, error) {
		var details = null;
		if (error.message) {
			details = [ {message: error.message} ];
		} else if (error.errorClass) {
			details = [ {message: error.errorClass} ];
		}
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: errorMessage,
			details: details,
			closable: true
		});
	}


});

})();
