/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.reports                                        |
 *|                                                                        |
 *| Copyright IBM Corporation 2011, 2013.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
dojo.provide("dm.app.reports.ui.internal.pages.ReportBrowserContent");

/** Define the widget that is used for browsing report configurations. 
 *  Report configurations can either be personal or shared. Reports can
 *  also be organized in folders and each report has an associated
 *  description.
 *  
 *  This widget is used in multiple editors. In the reports browser
 *  editor (Reports), the entries for the templates will have
 *  associated actions to edit or delete the report. When this widget
 *  is used in a report editor, the folders and reports are listed but
 *  only a folder can be selected. Folders can be modified but reports
 *  cannot.
 *  
 *  @author Kevin Cornell
 */

dojo.require("dm.app.reports.ui.internal.application.ReportActions");
dojo.require("dm.app.reports.ui.internal.pages.FolderDialog");
dojo.require("dm.app.reports.ui.internal.pages.ReportBrowserItem");

dojo.require("rmps.ui.editor.EditorRegistry");

dojo.require("dm.app.reports.ui.internal.util.ReportServiceClient");
dojo.require("dm.app.reports.ui.internal.util.ReportUtil");
dojo.require("dm.app.reports.ui.internal.util.Icons");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
var session = com.ibm.team.repository.web.client.session;

dojo.require("jazz.ui.FilterBox");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.Tree");
dojo.require("dijit.form.Select");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.requireLocalization("dm.app.reports.ui.internal", "ReportMessages");
var messages  = dojo.i18n.getLocalization("dm.app.reports.ui.internal", "ReportMessages");

var ReportActions = dm.app.reports.ui.internal.application.ReportActions;
var ReportServiceClient = dm.app.reports.ui.internal.util.ReportServiceClient;
var ReportUtil = dm.app.reports.ui.internal.util.ReportUtil;
var Icons = dm.app.reports.ui.internal.util.Icons;

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var FilterBox = jazz.ui.FilterBox;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var COMBO_PRIVATE = "private";
var COMBO_SHARED  = "shared";
var COMBO_ALL     = "all";

dojo.declare("dm.app.reports.ui.internal.pages.ReportBrowserContent", [dijit._Widget, dijit._Templated], {

	"class": "reportBrowser",

	messages: messages, 
	
	widgetsInTemplate: true,
	templateString: 
		'<div>' +
			'<table class="filterArea"  dojoAttachPoint="_filterArea"><tr>' +
				'<td class="ownerFilterContainer" dojoAttachPoint="_ownerFilterContainer"></td>' +
				'<td class="treeActionsContainer" dojoAttachPoint="_treeActionsContainer"></td>' +
				'<td class="nameFilterContainer"  dojoAttachPoint="_nameFilterContainer"></td>' +
			'</tr></table>' +
			'<div class="resultsArea" dojoAttachPoint="_resultsArea"></div>' +
		'</div>',
	
	// constructor properties
	editor: null,
	inBrowser: null,
	
	// Tree contents and array of dojo.connect listeners
	_tree: null,
	_treeListeners: null,
	
	// Remember the folder selected (when not in the reports browser)
	_selectedFolderUUID: null,
	
	// Table results filtering options
	_ownerFilter: COMBO_ALL,
	_nameFilter: null,
	
	// Keep information about performing the requests
	_currentRequestId: 0,
	_refreshTimeout: null,
	_systemReqComplete: false,
	_folderReqComplete: false,
	_folderReqCount: 0,
	_reportReqComplete: false,
	_reportReqCount: 0,
	
	// Keep information for the results of the queries
	_rootFolderPrivate: null,
	_rootFolderShared:  null,
	_resourceIdToItem: null,
	_folders: null,
	_reports: null,
	
	// Keep information for pending operations
	// on a given resource (either a folder or a report)
	_pendingResource: null,
	
	constructor: function(args) {
		this.messages = messages;
		this._resourceIdToItem  = [];
		this._folders = [];
		this._reports = [];
	},

	/** Create the controls for the report browser content. 
	 * 
	 */
	postCreate: function() {
		this.inherited(arguments);

		// Create the folder and report owner combobox
		var ownerCombobox = new dijit.form.Select( {
			name: "reportOwnerCombobox",
			onChange: dojo.hitch(this, this._ownerFilterChanged),
			options: [ {value: COMBO_PRIVATE, label: messages.reportComboPrivate},
			           {value: COMBO_SHARED,  label: messages.reportComboShared},
			           {value: COMBO_ALL,     label: messages.reportComboAll, selected: "selected"} ]
			});
		this._ownerFilterContainer.appendChild(ownerCombobox.domNode);
		
		// Create a toolbar with actions to collapse/expand all tree items
		var toolbar = new jazz.ui.SimpleToolbar({ name: "treeActions" });
		toolbar.addButton(this, dojo.hitch(this, function() {
			this._expandItemHierarchy(null, false);
		}), {imgSrc: Icons.COLLAPSE_ALL, label: this.messages.collapseAll});
		toolbar.addButton(this, dojo.hitch(this, function() {
			this._expandItemHierarchy(null, true);
		}), {imgSrc: Icons.EXPAND_ALL, label: this.messages.expandAll});
		this._treeActionsContainer.appendChild(toolbar.domNode);

		
		// Create the report name filtering control
		var filterBox = new FilterBox({onChange: dojo.hitch(this, this._nameFilterChanged)});
		this._nameFilterContainer.appendChild(filterBox.domNode);
		
		// Refresh the contents of this page which creates the results area
		this.refresh();
	},
	
	/** Select a folder in the reports browser.
	 *  
	 *  @param folder the selected folder or null
	 */	
	selectFolder: function(folder) {
		// If the folder being selected is currently selected, un-select it.
		if (folder && this._selectedFolderUUID === folder.folderUUID) {
			folder = null;
		}
		
		// Determine the tree items for the new folder and the selected folder
		var folderItem = (folder ? this._resourceIdToItem[folder.folderUUID] : null);
		var selectItem = (this._selectedFolderUUID ? this._resourceIdToItem[this._selectedFolderUUID] : null);
		
		// If nothing was selected or a different folder was selected
		if (!folder || folder.folderUUID !== this._selectedFolderUUID) {
			// If a folder was selected, un-select it
			if (selectItem) {
				selectItem.highlight(ReportUtil.COLOR_UNSELECT);
				this._selectedFolderUUID = null;
			}
			
			// If a new folder is being selected
			if (folder) {
				// Select the new folder and highlight it in the tree
				this._selectedFolderUUID = folder.folderUUID;
				if (folderItem) {
					folderItem.highlight(ReportUtil.COLOR_SELECTED);
					
					// Expand the tree to ensure the folder is displayed
					this._expandToItem(folderItem, true);
				}
			}
		} 
	},
	
	/** Expand or collapse all folders in the item hierarchy.
	 * 
	 * @param item starting item in hierarchy
	 * @param expand true = expand all, false = collapse all
	 */
	_expandItemHierarchy: function(item, expand) {
		// If no folder was specified expand the root (system) folders
		if (!item) {
			if (this._rootFolderPrivate) {
				item = this._resourceIdToItem[this._rootFolderPrivate.folderUUID];
				if (item) {
					this._expandItemHierarchy(item, expand);
				}
			}
			if (this._rootFolderShared) {
				item = this._resourceIdToItem[this._rootFolderShared.folderUUID];
				if (item) {
					this._expandItemHierarchy(item, expand);
				}
			}
		}
		
		// else 
		else {
			// Expand the given item
			item.setExpanded(expand);
			
			// Recursively expand all of its children item
			var children = item.getChildren();
			var length   = children.length;
			for (var i = 0; i < length; i++) { 
				var childItem = children[i];
				this._expandItemHierarchy(childItem, expand);
			}
		}
	},
	
	/** Expand the tree to the given item.
	 * 
	 * @param expand true = expand all, false = collapse all
	 */
	_expandToItem: function(item, expand) {
		if (item) {
			// Expand the given item
			item.setExpanded(expand);
			
			// Expand all parent items in the tree
			var parentItem = item.getParent();
			while (parentItem) {
				parentItem.setExpanded(expand);
				parentItem = parentItem.getParent();
			}
		}
	},

	destroy: function() {
		this._deleteTree();
		this.inherited(arguments);
	},
	
	/** Destroy the content tree, its items and any listeners
	 */
	_deleteTree: function() {
		// Remove all previously cached data
		this._rootFolderPrivate = null;
		this._rootFolderShared  = null;
		this._resourceIdToItem  = [];
		this._folders = [];
		this._reports = [];
		if (this._tree) {
			// Delete any tree listeners and remove all top level items from the tree
			dojo.forEach(this._treeListeners, function(listener, i) { dojo.disconnect(listener); });
			var childItems = this._tree.getChildren();
			var itemsToRemove = [];
			dojo.forEach(childItems, function(item, i) { itemsToRemove.push(item); });
			var theTree = this._tree;
			dojo.forEach(itemsToRemove, function(item, i) { theTree.removeChild(item); });
			
			// Destroy the tree widget (this call does not destroy its items)
			this._resultsArea.removeChild(this._tree.domNode);
			this._tree.destroy();
			this._tree = null;
			this._treeListeners = null;
		}
	},
	
	/** Refresh the folders and the reports shown in the browser.
	 *  
	 *  The refresh actually performs 3 requests to the JFS reporting
	 *  service. Only after all the requests have completed will the
	 *  resulting folders and reports be rendered as items in the
	 *  tree control.
	 *  
	 *  The three requests are:
	 *  
	 *  1) obtain the system (root) folders (My Reports and Shared Reports)
	 *  2) obtain all of the folders (filtered by the owner combobox)
	 *  3) obtain all of the reports (filtered by name and the owner combobox)
	 * 
	 */
	refresh: function() {
		// Delete the tree and any cached data
		this._deleteTree();
		
		// Reset the indicators for the completion of the 3 requests
		this._systemReqComplete = false;
		this._folderReqComplete = false;
		this._reportReqComplete = false;
		
		// Initialize the information to perform the requests for folders and queries
		if (this.inBrowser) {
			PlatformUI.getWorkbench().showMessage(this.messages.searchingReports);
		}
		if (this._refreshTimeout) {
			clearTimeout(this._refreshTimeout);
			delete this._refreshTimeout;
		}
		this._currentRequestId++;

		// Initialize the service response handler to request the system folders
		var sysFolderServiceObject = {
			self: this,
			id: this._currentRequestId,
			success: function(sysFolders) {
				if (this.self._currentRequestId > this.id) {
					return;
				}
				this.self._processSystemFolderRequest(sysFolders);
			},
					
			failure: function(error) {
				this.self.editor.serviceError(messages.errorLoadingFolders, error);
			}
		};
		var sysFolderReqHandler = new ServiceResponseHandler(sysFolderServiceObject, "success", "failure");
		
		// Perform the request to get the system folders
		var args = { flags: ReportServiceClient.PRIVATE | ReportServiceClient.SHARED | ReportServiceClient.SYSTEM };
		ReportServiceClient.getFolderDescriptors(sysFolderReqHandler, args);

		
		// Initialize the service response handler to request the user folders
		var folderServiceObject = {
			self: this,
			id: this._currentRequestId,
			success: function(userFolders) {
				if (this.self._currentRequestId > this.id) {
					return;
				}
				this.self._processFoldersRequest(userFolders);
			},
					
			failure: function(error) {
				this.self.editor.serviceError(messages.errorLoadingFolders, error);
			}
		};
		
		// Perform the necessary requests to obtain the private and/or shared user folders
		args.sortByName = true;
		var folderReqHandler = null;
		if (this._ownerFilter === COMBO_PRIVATE){
			// Private = single request for just private folders
			this._folderReqCount = 1;
			args.flags = ReportServiceClient.PRIVATE;
			args.ownerUUID = session.getAuthenticatedContributor().itemId;
			folderReqHandler = new ServiceResponseHandler(folderServiceObject, "success", "failure");
			ReportServiceClient.getFolderDescriptors(folderReqHandler, args);
		} else if (this._ownerFilter === COMBO_SHARED) {
			// Shared = single request for just shared folders
			this._folderReqCount = 1;
			args.flags = ReportServiceClient.SHARED;
			delete args.ownerUUID;
			folderReqHandler = new ServiceResponseHandler(folderServiceObject, "success", "failure");
			ReportServiceClient.getFolderDescriptors(folderReqHandler, args);
		} else {
			// All = separate requests for private and shared folders
			this._folderReqCount = 2;
			args.flags = ReportServiceClient.PRIVATE;
			args.ownerUUID = session.getAuthenticatedContributor().itemId;
			folderReqHandler = new ServiceResponseHandler(folderServiceObject, "success", "failure");
			ReportServiceClient.getFolderDescriptors(folderReqHandler, args);
			
			args.flags = ReportServiceClient.SHARED;
			delete args.ownerUUID;
			folderReqHandler = new ServiceResponseHandler(folderServiceObject, "success", "failure");
			ReportServiceClient.getFolderDescriptors(folderReqHandler, args);
		}
			
		
		// Initialize the service response handler for the reports request
		var reportServiceObject = {
			self: this,
			id: this._currentRequestId,
			success: function(userReports) {
				if (this.self._currentRequestId > this.id) {
					return;
				}
				this.self._processReportsRequest(userReports);
			},
					
			failure: function(error) {
				this.self.editor.serviceError(messages.errorLoadingReports, error);
			}
		};
		
		// Perform the necessary requests to obtain the private and/or shared reports
		if (this._nameFilter) {
			args.filter = this._nameFilter;
		}
		var reportReqHandler = null;
		if (this._ownerFilter === COMBO_PRIVATE){
			// Private = single request for just private reports
			this._reportReqCount = 1;
			args.flags = ReportServiceClient.PRIVATE;
			args.ownerUUID = session.getAuthenticatedContributor().itemId;
			reportReqHandler = new ServiceResponseHandler(reportServiceObject, "success", "failure");
			ReportServiceClient.getQueryDescriptors(reportReqHandler, args);		
		} else if (this._ownerFilter === COMBO_SHARED) {
			// Shared = single request for just shared reports
			this._reportReqCount = 1;
			args.flags = ReportServiceClient.SHARED;
			delete args.ownerUUID;
			reportReqHandler = new ServiceResponseHandler(reportServiceObject, "success", "failure");
			ReportServiceClient.getQueryDescriptors(reportReqHandler, args);		
		} else {
			// All = separate requests for private and shared reports
			this._reportReqCount = 2;
			args.flags = ReportServiceClient.PRIVATE;
			args.ownerUUID = session.getAuthenticatedContributor().itemId;
			reportReqHandler = new ServiceResponseHandler(reportServiceObject, "success", "failure");
			ReportServiceClient.getQueryDescriptors(reportReqHandler, args);
			
			args.flags = ReportServiceClient.SHARED;
			delete args.ownerUUID;
			reportReqHandler = new ServiceResponseHandler(reportServiceObject, "success", "failure");
			ReportServiceClient.getQueryDescriptors(reportReqHandler, args);		
		}
	},	
	
	/** Process the results of the request for the system
	 *  folders. Determine the root folders for private
	 *  reports and shared reports. The system folders are 
	 *  not added to the list of all folders.
	 *  
	 *  @param sysFolders the list of system folders returned by the request
	 */
	_processSystemFolderRequest: function(sysFolders) {
		// For each folder returned by the request
		if (!sysFolders) {
			sysFolders = [];
		}
		var length = sysFolders.length;
		for (var i = 0; i < length; i++) {
			// If the folder is either the private or shared system folder
			var folder = sysFolders[i];
			folder._isFolder = true;		// internal flag to identify folders vs reports
			if (folder.system) {
				if (folder.shared) {
					this._rootFolderShared = folder;
				} else {
					this._rootFolderPrivate = folder;
				}
			}
		}
		
		// If no system folders were found, report an error
		if (!this._rootFolderShared && !this._rootFolderPrivate) {
			this.editor.serviceError(messages.errorLoadingFolders, {message: messages.errorNoSystemFolders });
			return;
		}
			
		// This system folders request is complete
		this._systemReqComplete = true;
			
		// If all three requests are complete, render the results
		if (this._folderReqComplete && this._reportReqComplete) {
			this._renderFoldersAndReports();
		}
	},
	
	/** Process the results of the request for user report
	 *  folders. Keep a copy of all folders and render the 
	 *  folders and reports if all three requests are complete. 
	 *  
	 *  @param userFolders the list of folders returned by the request
	 */
	_processFoldersRequest: function(userFolders) {
		// For each user folder returned by the request
		if (!userFolders) {
			userFolders = [];
		}
		var length = userFolders.length;
		for (var i = 0; i < length; i++) {
			// Remember the folder
			var folder = userFolders[i];
			folder._isFolder = true;		// internal flag to identify folders vs reports
			this._folders.push(folder);
		}
		
		// The user folders request is complete if the 
		// private and/or shared folder requests are complete.
		this._folderReqCount--;
		this._folderReqComplete = (this._folderReqCount <= 0);			

		// If all three requests are complete, render the results
		if (this._systemReqComplete && this._folderReqComplete && this._reportReqComplete) {
			this._renderFoldersAndReports();
		}
	},
	
	/** Process the results of the request for reports.
	 *  Keep a copy of all reports and render the folders 
	 *  and reports if all 3 requests are complete. 
	 *  
	 *  @param userReports the list of reports returned by the request
	 */
	_processReportsRequest: function(userReports) {
		// For each report returned by the request
		if (!userReports) {
			userReports = [];
		}
		var length = userReports.length;
		for (var i = 0; i < length; i++) {
			// Remember the report
			var report = userReports[i];
			report._isFolder = false;		// internal flag to identify folders vs reports
			this._reports.push(report);
		}
		
		// The user reports request is complete if the 
		// private and/or shared report requests are complete.
		this._reportReqCount--;
		this._reportReqComplete = (this._reportReqCount <= 0);			

		// If all three requests are complete, render the results
		if (this._systemReqComplete && this._folderReqComplete && this._reportReqComplete) {
			this._renderFoldersAndReports();
		}
	},
	
	/** Render all of the folders and reports 
	 *  into the results tree.
	 */
	_renderFoldersAndReports: function() {		
		if (this.inBrowser) {
			PlatformUI.getWorkbench().hideMessageArea();
		}
		
		// Create a new results tree for the folders and reports
		this._tree = new jazz.ui.Tree({selection: "single"});
		this._resultsArea.appendChild(this._tree.domNode);
		this._treeListeners = [];
		this._tree.startup();
		
		// When the Enter key is pressed, select the corresponding folder
		// or open the corresponding report (if enabled)
		this._treeListeners.push(dojo.connect(this._tree, "onEnter",  this, function() {
			var selectedItems = this._tree.getSelection();
			if (selectedItems.length > 0) {
				var item = selectedItems[0];
				if (item.getResource()._isFolder) {
					var folder = item.getResource();
					if (!this.inBrowser && folder !== this._rootFolderPrivate &&
										   folder !== this._rootFolderShared) {	
						this.selectFolder(item.getResource());
					}
				} else {
					item.onEnter();
				}
			}
		}));
		
		// If the root private folder is defined and is to be shown
		if (this._rootFolderPrivate && this._ownerFilter !== COMBO_SHARED) {
			// Add its hierarchy of folders and reports to the tree
			this._renderFolder(this._tree, this._rootFolderPrivate);			
		}
		
		// If the root shared folder is defined and is to be shown
		if (this._rootFolderShared && this._ownerFilter !== COMBO_PRIVATE) {
			// Add its hierarchy of folders and reports to the tree
			this._renderFolder(this._tree, this._rootFolderShared);			
		}
		
		// If this content is in the reports browser, expand the tree.
		if (this.inBrowser) {
			this._expandItemHierarchy(null, true);
		}
	},
	
	/** Recursively render the given folder in the tree
	 *  along with its nested folders and its owned
	 *  reports.
	 *  
	 *  @param parentItem the parent tree item for this nested folder
	 *  @param folder the nested folder to be added to the tree
	 */
	_renderFolder: function(parentItem, folder) {
		// Create the action toolbar for this folder
		var toolbar = this._createItemToolbar(folder);
		
		// Create the tree item for this folder and add it
		var folderItem = new dm.app.reports.ui.internal.pages.ReportBrowserItem(
				{resource: folder, toolbar: toolbar});
		folderItem.update();
		parentItem.addChild(folderItem);
		this._resourceIdToItem[folderItem.getResourceId()] = folderItem;
		
		// If this folder was previously selected, re-select it in the refreshed tree
		if (this._selectedFolderUUID === folder.folderUUID) {
			this._selectedFolderUUID = null;
			this.selectFolder(folder);
		}
		
		// If this content is not being displayed in the Reports Browser,
		// handle events to allow a folder to be selected.
		// (e.g., browser content is embedded in create report editor)
		// Since the system (root) folders do not really exist, do not
		// allow them to be selected.
		if (!this.inBrowser && folder !== this._rootFolderPrivate &&
				folder !== this._rootFolderShared) {
			this._treeListeners.push(dojo.connect(folderItem.getItemContainer(), "onclick", this, function() { 
				this.selectFolder(folder);
			}));
		}

		// Render all of the children of this folder (nested folders and reports)
		this._renderFolderChildern(folderItem, folder);
	},
	
	/** Render all children of a given folder. This includes
	 *  nested folders and owned reports.
	 *  
	 *  @param folderItem the tree item for the folder
	 *  @param folder the folder whose children are to be rendered
	 */
	_renderFolderChildern: function(folderItem, folder) {
		// Define the function for sorting folders and reports
		function sortFunction(a, b) {
			return a.name < b.name ? -1 : (a.name > b.name ? 1 : 0);
		}

		// Find all of the nested folders of the current folder
		var nestedFolders = [];
		var length = this._folders.length;
		for (var i = 0; i < length; i++) {
			var nestedFolder = this._folders[i];
			if (nestedFolder.parentUUID === folder.folderUUID) {
				nestedFolders.push(nestedFolder);
			}
		}
		
		// Sort the nested folders and render them
		nestedFolders = nestedFolders.sort(sortFunction);
		for (i = 0; i < nestedFolders.length; i++) {
			this._renderFolder(folderItem, nestedFolders[i]);
		}

		// Find all of the reports owned by the current folder
		var ownedReports = [];
		length = this._reports.length;
		for (i = 0; i < length; i++) {
			var ownedReport = this._reports[i];
			if (ownedReport.folderUUID === folder.folderUUID) {
				ownedReports.push(ownedReport);
			}	
		}
		
		// Sort the nested folders and render them
		ownedReports = ownedReports.sort(sortFunction);
		length = ownedReports.length;
		for (i = 0; i < length; i++) {
			// Create the action toolbar for this folder
			var report = ownedReports[i];
			var toolbar = this._createItemToolbar(report, false);
			
			// Create the action for the report label only if this
			// content is being displayed in the reports browser
			var editActionUri = null;
			if (this.inBrowser) {
				editActionUri = ReportActions.getActionUri(ReportActions.ACTION_EDIT_REPORT);
				editActionUri = editActionUri + "&queryUUID=" + report.queryUUID + "&name=" + encodeURIComponent(report.name);
			}
				
			// Add the report to the tree
			var reportItem = new dm.app.reports.ui.internal.pages.ReportBrowserItem(
					{resource: report, toolbar: toolbar, labelAction: editActionUri});
			reportItem.update();
			folderItem.addChild(reportItem);
			this._resourceIdToItem[reportItem.getResourceId()] = reportItem;
			
			// If this report was found using the name filter
			// expand the tree up to the item
			if (this._nameFilter) {
				this._expandToItem(folderItem, true);
			}
		}						
	},
	
	/** Create a toolbar for an item with the appropriate buttons.
	 * 
	 * @param resource the underlying folder or report
	 */
	_createItemToolbar: function(resource) {
		var toolbar = new jazz.ui.SimpleToolbar();
		
		// If the item is a folder
		if (resource._isFolder) {
			var folder = resource;
			
			// Add a button to create a child folder (remember the parent of the new folder)
			toolbar.addButton(this, dojo.hitch(this, function() {
				this._pendingResource = folder;
				var dialog = new dm.app.reports.ui.internal.pages.FolderDialog({onOk: dojo.hitch(this, "_createFolder")});
				dialog.startup();
			}), {imgSrc: Icons.CREATE_FOLDER, label: this.messages.createNewFolder});
			
			// If this is not a system folder add buttons to edit and delete the folder.
			if (!folder.system) {
				toolbar.addButton(this, dojo.hitch(this, function() {
					var dialog = new dm.app.reports.ui.internal.pages.FolderDialog({onOk: dojo.hitch(this, "_saveFolder"), folder: folder});
					dialog.startup();
				}), {imgSrc: Icons.EDIT, label: this.messages.edit});
				toolbar.addButton(this, dojo.hitch(this, "_deleteResource", folder), {imgSrc: Icons.DELETE, label: this.messages.deleteMsg});
			}
		}
		
		// else if the report is being displayed in the Reports browser
		else if (this.inBrowser){
			// Add edit and delete actions
			var report = resource;
			toolbar.addButton(this, dojo.hitch(this, "_editReport", report), {imgSrc: Icons.EDIT, label: this.messages.edit});
			toolbar.addButton(this, dojo.hitch(this, "_deleteResource", report), {imgSrc: Icons.DELETE, label: this.messages.deleteMsg});
		}
		return toolbar;
	},

	/** Create a new folder given the folder 
	 *  name and its description. Use the selected
	 *  folder to determine the parent folder and
	 *  whether it is private or shared.
	 */
	_createFolder: function(folder) {
		// If the parent folder is defined
		if (this._pendingResource && this._pendingResource._isFolder) {
			// Update the parent of this new folder and its shared attribute
			var parentFolder = this._pendingResource;
			this._pendingResource = null;
			folder.processAreaUUID = ReportUtil.getProjectId();
			folder.parentUUID = parentFolder.folderUUID;
			folder.shared     = parentFolder.shared;
			folder.system     = false;
			
			// Create the new folder
			this._saveFolder(folder);
		}
	},
	
	/** Save the changes to an existing folder or
	 *  create a new folder. An existing folder 
	 *  contains fields that are not applicable to
	 *  the save operation, so a copy of the relevant
	 *  folder data must be made first.
	 */
	_saveFolder: function(folder) {
		// Keep a copy of the folder to be modified or created
		var folderData = {
			name: folder.name,
			description: folder.description,
			parentUUID: folder.parentUUID,
			processAreaUUID: folder.processAreaUUID,
			shared: folder.shared,
			system: folder.system
		};
		
		// For an existing folder being updated, copy the folder ID
		// else for a new folder, save the folder resource to be
		// updated with its ID during the save completion.
		if (folder.folderUUID) {
			folderData.folderUUID = folder.folderUUID;
		} else {
			this._pendingResource = dojo.clone(folderData);
		}
		
		
		// Send the request to save the folder data
		var saveFolderServiceObject = {
				self: this,
				success: function(folderUUID) {
					this.self._saveFolderComplete(folderUUID);
				},
						
				failure: function(error) {
					this.self.editor.serviceError(messages.errorSavingFolder, error);
				}
			};
		var saveFolderHandler = new ServiceResponseHandler(saveFolderServiceObject, "success", "failure");
		ReportServiceClient.postSaveFolderDescriptor(saveFolderHandler, folderData);
	},
	
	/** The saving of a folder was completed.
	 * 
	 * @param folderUUID the ID of the new folder
	 */
	_saveFolderComplete: function(folderUUID) {
		// Un-select any current selected folder
		this.selectFolder(null);
		var folder = null;
		
		// If a new folder was created
		if (this._pendingResource) {
			// Set the new folder's ID and add it to the list of all known folders
			folder = this._pendingResource;
			this._pendingResource = null;
			folder._isFolder = true;		// internal flag to identify folders vs reports
			folder.folderUUID = folderUUID;
			this._folders.push(folder);
		}
		
		// else find the existing folder that was modified
		else {
			var folderItem = this._resourceIdToItem[folderUUID];
			if (folderItem) {
				folder = folderItem.getResource();
			}
		}
		
		if (folder && folder.parentUUID) {
			// Find the siblings of the folder that was just saved
			// (all children of its parent folder).
			// Make a copy of the array but not its values (e.g. cannot use dojo.clone())
			var parentItem = this._resourceIdToItem[folder.parentUUID];
			var childItems = parentItem.getChildren();
			var itemsToRemove = [];
			dojo.forEach(childItems, function(item, i) { itemsToRemove.push(item); });
			dojo.forEach(itemsToRemove, function(item, i) { parentItem.removeChild(item); });
			
			// Render all of the children of the parent folder again
			var parentFolder = parentItem.getResource();
			this._renderFolderChildern(parentItem, parentFolder);
			
			// Expand the tree to the parent item (this will show the new folder)
			this._expandToItem(parentItem, true);
		}
	},
	
	/** Edit a report by opening the report editor.
	 * 
	 * @param report the report to edit
	 */
	_editReport: function(report) {
		var editActionUri = ReportActions.getActionUri(ReportActions.ACTION_EDIT_REPORT);
		window.location.href = editActionUri + 
			"&queryUUID=" + report.queryUUID + "&name=" + encodeURIComponent(report.name);
	},

	/** Delete a folder or a report. When a folder is deleted,
	 *  all of its sub folders and associated reports are
	 *  also deleted.
	 *  
	 *  @param resource the folder or report to be deleted
	 */
	_deleteResource: function(resource) {
		// Save the resource being deleted for the success
		// and error handling.
		this._pendingResource = resource;
		
		// Ensure the user wants to delete this folder
		var substitutions = [resource.name];
		var format = resource._isFolder ? messages.confirmDeleteFolder : messages.confirmDeleteReport;
		var delMessage = dojo.string.substitute(format, substitutions);
		if (!confirm(delMessage)) {
			return;
		}
		
		// Create the delete operation response handler
		var responseHandler = new ServiceResponseHandler(
				this,
				"_deleteResourceSuccess",
				"_deleteResourceError");
		
		// Delete either the folder or the report
		if (resource._isFolder) {
			ReportServiceClient.postDeleteFolderDescriptor(responseHandler, { folderUUID: resource.folderUUID });
		} else {
			ReportServiceClient.postDeleteQueryDescriptor(responseHandler, { queryUUID: resource.queryUUID });
		}
	},
	
	/** Action to delete a folder or report was successful. */
	_deleteResourceSuccess: function() {
		// If the pending resource is defined
		var resource = null;
		if (this._pendingResource) {
			resource = this._pendingResource;
			this._pendingResource = null;
		}
		if (resource) {
			// If an editor for this report is still open, close it.
			if (!resource._isFolder) {
				var action = ReportActions.ACTION_PREFIX + ReportActions.ACTION_EDIT_REPORT;
				var args = {
					projectId: ReportUtil.getProjectId(),
					queryUUID: resource.queryUUID
				};
				var editorInputDeferred = rmps.ui.editor.EditorRegistry.createEditorInput(action, args);
				if (editorInputDeferred) {
					editorInputDeferred.then(function(editorInput) {
						var pageHelper = rmps.ui.editor.PageHelper.getInstance(ReportActions.PAGE_ID);
						pageHelper.closeEditorInput(editorInput, true, true);
					});
				}
			}
			
			// Output a message indicating the resource was deleted
			var substitutions = [resource.name];
			var format = resource._isFolder ? messages.deleteFolderSuccess : messages.deleteReportSuccess;
			var statusMessage = dojo.string.substitute(format, substitutions);
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.OK,
				summary: statusMessage,
				details: null,
				closable: true
			});
			
			// Un-select the deleted folder
			if (resource._isFolder && this._selectedFolderUUID === resource.folderUUID) {
				this.selectFolder(null);
			}
			
			// Remove the deleted resource from the appropriate list.
			// Any of its nested folders or reports will still be in 
			// the lists but are now orphaned and will not be processed
			// on a dynamic refresh
			if (resource._isFolder) {
				var fIndex = dojo.indexOf(this._folders, resource);
				if (fIndex >= 0) {
					this._folders.splice(fIndex, 1);
				}
			} else {
				var rIndex = dojo.indexOf(this._reports, resource);
				if (rIndex >= 0) {
					this._reports.splice(rIndex, 1);
				}
			}
			
			// Remove the item from the tree 
			var resourceUUID = resource._isFolder ? resource.folderUUID : resource.queryUUID;
			var parentUUID   = resource._isFolder ? resource.parentUUID : resource.folderUUID;
			var resourceItem = this._resourceIdToItem[resourceUUID];
			var parentItem   = this._resourceIdToItem[parentUUID];
			if (resourceItem && parentItem) {
				parentItem.removeChild(resourceItem);
			}
		}
	},
	
	/** Action to delete a folder or a report failed. */
	_deleteResourceError: function(error) {
		// If the pending resource is defined
		var resource = null,
		    format, 
		    details;
		if (this._pendingResource) {
			resource = this._pendingResource;
			this._pendingResource = null;
		}
		if (resource) {
			var substitutions = [resource.name];
			if(resource._isFolder){
			    format = messages.errorDeletingFolder;
			    details = messages['errorDeletingFolder.explanation'] + "<br>" + messages['errorDeletingFolder.useraction'];
			}else{
			    format = messages.errorDeletingReport;
			    details = messages['errorDeletingReport.explanation'] + "<br>" + messages['errorDeletingReport.useraction'];
			}
			var statusMessage = dojo.string.substitute(format, substitutions);
			this.editor.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: statusMessage,
				details: [ { message: details }, { message: error.message } ],
				closable: true
			});
		}
	},

	/** Perform filtering of the report folders and reports 
	 *  based on the owner selection.
	 * 
	 * @param owner: the template owner selection filter
	 */
	_ownerFilterChanged: function(newOwnerFilter) {
		this._ownerFilter = newOwnerFilter;
		if (!this._refreshTimeout) {
			this._refreshTimeout = setTimeout(dojo.hitch(this, this.refresh, 0, true), 500);
		}
	},
	
	/** Perform filtering of the report templates in the table
	 *  based on the name.
	 * 
	 * @param filter: the name filter expression
	 */
	_nameFilterChanged: function(newNameFilter) {
		this._nameFilter = newNameFilter;
		if (!this._refreshTimeout) {
			this._refreshTimeout = setTimeout(dojo.hitch(this, this.refresh, 0, true), 500);
		}
	},
	
	/** Handle the resizing of the browser content.
	 *  If the content becomes larger than the 
	 *  editor page, auto-scrolling should result in the
	 *  vertical scroll bars being shown.
	 */
	resize: function(size) {
		this.inherited(arguments);
		if (!size) {
			return;
		}
		if (size.h) {
			var filterPos = dojo.position(this._filterArea);
			var resultPos = dojo.position(this._resultsArea);
			var newHeight = size.h - resultPos.y + filterPos.y;
			dojo.style(this._resultsArea,   "height", newHeight + "px");	
		}
	}

});

})();
