/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.reports                                        |
 *|                                                                        |
 *| Copyright IBM Corporation 2012.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
dojo.provide("dm.app.reports.ui.internal.pages.ReportEditorInput");

/** Initialize the editor for modifying a report configuration.
 *  The report editor contains multiple tabs for manipulating the
 *  data for the report configuration and for running the report.
 *  These tabs are defined here:
 *  
 *  Overview:	contains the base report data including the name,
 *  			description, associated template and parent folder
 *  Parameters:	contains widgets for setting the parameters that are
 *  			defined in the associated template
 * 
 * @author Kevin Cornell
 */

dojo.require("dm.app.reports.ui.internal.application.ReportActions");
dojo.require("dm.app.reports.ui.internal.util.Icons");

dojo.require("rmps.ui.editor.EditorInput");
dojo.require("dojox.encoding.digests.MD5");
dojo.require("jazz.util.html");

dojo.require("dojo.i18n");
dojo.requireLocalization("dm.app.reports.ui.internal", "ReportMessages");
var messages = dojo.i18n.getLocalization("dm.app.reports.ui.internal", "ReportMessages");

var ReportActions = dm.app.reports.ui.internal.application.ReportActions;

// Define the tabs of the report editor
var OverviewTabId  = "reportOverview";
var ParameterTabId = "reportParams";
var OutputTabId    = "reportOutput";

dojo.declare("dm.app.reports.ui.internal.pages.ReportEditorInput", rmps.ui.editor.EditorInput, {
	
	projectId: null,
	templateName: null,
	
	constructor: function(params) {
		this.projectId = params.projectId;
	},
	
	getId: function() {
		var id = this.declaredClass + ',' + this.projectId;
		var params = this.getParams();
		if (params.queryUUID) {
			id = id + ',' + params.queryUUID;
		}
		return dojox.encoding.digests.MD5(id);
	},
	
	getIconUri: function() {
		return dm.app.reports.ui.internal.util.Icons.REPORT;
	},
	
	getName: function() {
		var params = this.getParams();
		var name = params.name;
		return name;
	},
	
	getTooltip: function() {
		return null;
	},

	/** Return the information on the editor's tabs.
	 */
	getTabs: function() {
		return [
			{
				id: OverviewTabId,
				label: messages.reportTabOverview,
				widget: "dm.app.reports.ui.internal.pages.ReportEditorOverview"
		    },
		    {
			    id: ParameterTabId,
			    label: messages.reportTabParams,
			    widget: "dm.app.reports.ui.internal.pages.ReportEditorParams"
		    },
		    {
			    id: OutputTabId,
			    label: messages.reportTabOutput,
			    widget: "dm.app.reports.ui.internal.pages.ReportEditorOutput"
		  }];
	},
	
	/** Return the ID of the output tab. */
	getOutputTabId: function() {
		return OutputTabId;
	},
	
	/** Return the ID of the parameters tab. */
	getParametersTabId: function() {
		return ParameterTabId;
	},

	getBasePresentationUri: function() {
		var editorUri = ReportActions.getActionUri(ReportActions.ACTION_EDIT_REPORT);
		var params = this.getParams();
		if (params.queryUUID) {
			editorUri = editorUri + "&queryUUID=" + params.queryUUID;
		}
		if (params.name) {
			editorUri = editorUri + "&name=" + encodeURIComponent(params.name);
		}
		return editorUri;
	}
});

})();
