/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.reports                                        |
 *|                                                                        |
 *| Copyright IBM Corporation 2012.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
dojo.provide("dm.app.reports.ui.internal.pages.ReportEditorOutput");

dojo.require("dm.app.reports.ui.internal.pages.ExportDialog");
dojo.require("dm.app.reports.ui.internal.util.ReportServiceClient");
dojo.require("dm.app.reports.ui.internal.util.Icons");
dojo.require("dm.app.reports.ui.internal.util.ReportUtil");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("jazz.ui.SimpleToolbar");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.requireLocalization("dm.app.reports.ui.internal", "ReportMessages");
var messages  = dojo.i18n.getLocalization("dm.app.reports.ui.internal", "ReportMessages");

var Icons      = dm.app.reports.ui.internal.util.Icons;
var ReportUtil = dm.app.reports.ui.internal.util.ReportUtil;
var ReportServiceClient    = dm.app.reports.ui.internal.util.ReportServiceClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

//Copied from ReportEditor
var FIELD_queryUUID = "queryUUID";				// the ID of this report configuration

/** The Output tab of the editor for modifying a report configuration.
 *  This tab will show an embedded report and will allow the user to 
 *  export the report in one of the supported formats. It will also
 *  allow the user to open another window with a printable version of
 *  the report (excludes all of the DM project header and report editor
 *  tabs).
 *  
 *  @author Kevin Cornell
 */


dojo.declare("dm.app.reports.ui.internal.pages.ReportEditorOutput", [dijit._Widget, dijit._Templated], {

	"class": "reportEditorOutput",

	messages: messages, 
	
	widgetsInTemplate: true,
	templateString: 
		'<div>' +
			'<div class="outputToolbar" dojoAttachPoint="_outputToolbar"></div>' +
			'<div class="running" dojoAttachPoint="_renderedReport">${messages.reportRunStart}</div>' +
		'</div>',
	
	// init properties
	editor: null,
	
	// Remember the actual node that contains the rendered HTML
	_renderedReportNode: null,
	_renderedReportCssHref: null,
	
	// Remember the report being run and its template
	_report:   null,
	_template: null,
	
	// Save the combined parameter names and values for rendering
	_runParamNames:  null,
	_runParamValues: null,
	
	// Remember the export service URL
	_exportServiceUrl: null,
	
	/** Create the tab widgets. **/
	postCreate: function() {
		this.inherited(arguments);
		
		// Create the toolbar for the export and print buttons
		var toolbar = new jazz.ui.SimpleToolbar({ name: "outputActions" });
		toolbar.addButton(this, this._showExportDialog, 
				{imgSrc: Icons.EXPORT, label: this.messages.exportLabel});
		toolbar.addButton(this, this._viewPrintablePage, 
				{imgSrc: Icons.PRINT, label: this.messages.viewPrintablePage});
		this._outputToolbar.appendChild(toolbar.domNode);
		
		// The toolbar is hidden until after a report has been run.
		dojo.style(this._outputToolbar, "visibility", "hidden");
		
		// Determine the service URL for performing the export of a report
		this._exportServiceUrl = this._computeExportServiceUrl();
	},
	
	/** Compute the URL for the reporting export (viewer) service.
	 * 
	 */
	_computeExportServiceUrl: function() {
		// Get the reporting root URL where the JFS reporting framework resides
		var serviceUrl = ReportUtil.getReportingRootUrl();
		if (serviceUrl) {
			serviceUrl = serviceUrl + "/service/com.ibm.team.reports.service.internal.IReportViewerService";
		}
		return serviceUrl;
	},
	
	/** Initialize the page controls based on the given
	 *  report , folder, template and parameters.
	 *  
	 *  @param report the report (query) descriptor
	 *  @param folder the parent folder descriptor
	 *  @param template the report template descriptor
	 *  @param params contains parameter definitions and values
	 */
	initReportData: function(report, folder, template, params) {
		// Save the report and its template
		this._report   = report;
		this._template = template;
		
		// Update the parameters for the next running of the report.
		this._runParamNames  = [];
		this._runParamValues = [];
		
		// If any parameters are defined in the template
		if (params.paramNames && params.paramNames.length > 0) {
			// Add the parameters to the run parameter lists
			var numParams = params.paramNames.length;
			for (var index = 0; index < numParams; index++) {
				// Copy the parameter names and values
				this._runParamNames[index]  = params.paramNames[index];
				this._runParamValues[index] = params.paramValues[index];
			}
		}
	},
	
	/** Start the rendering of the report, if not already running.
	 * 
	 */
	runReport: function() {
		// If the report is not already running
		if (!this.editor.reportIsRunning()) {
			this._removeChildren(this._renderedReport);
			this._renderedReport.className = "running";
			this._renderedReport.innerHTML = messages.reportRunning;
			this._renderedReportNode = null;  // prevent export and print actions
			this.editor.setIsRunning(true);
			
			// Hide the export and print buttons
			dojo.style(this._outputToolbar, "visibility", "hidden");	
			
			// Create the arguments needed to run the report (in HTML)
			var args = {
					embeddable:     true,
					parameterName:  this._runParamNames,
					parameterValue: this._runParamValues,
					queryUUID:      this._report[FIELD_queryUUID]
			};
			
			// Issue a request to render the report.
			var serviceObject = {
					self: this,
					success: function(reportHTML) {
						this.self._displayRenderedReport(reportHTML);
					},
							
					failure: function(error) {
						this.self.editor.setIsRunning(false);
						this.self.editor.serviceError(messages.errorRunningReport, error);
					}
				};
			var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
			ReportServiceClient.postRenderQuery(srh, args);
		}
	},
	
	/** Display the rendered report in the output tab.
	 * 
	 * @param reportHTML the generate HTML for the report
	 */
	_displayRenderedReport: function(reportHTML) {
		// Change the class name to indicate rendering is complete
		this._renderedReport.className = "renderedReport";
		this.editor.setIsRunning(false);
		
		// Check the chart for any <link> elements in the head. If they exist,
		// add the styles to the document.
		var startIndex = reportHTML.indexOf("<link rel=\"stylesheet\" type=\"text/css\"");
		if (startIndex !== -1) {
			startIndex   = reportHTML.indexOf("href=");
			var endIndex = reportHTML.indexOf("</link>");
			var cssHref  = reportHTML.substring(startIndex + 6, endIndex - 2);
			this._renderedReportCssHref = cssHref;
			
			var linkNode = document.createElement("link");
			linkNode.setAttribute("rel", "stylesheet");
			linkNode.setAttribute("type", "text/css");
			linkNode.setAttribute("href", cssHref);
			document.getElementsByTagName("head")[0].appendChild(linkNode);
		}
					
		if (dojo.isIE) {
			// Insert innerHTML directly. That, coupled with defer tags
			// on the scripts, causes new Javascript to be executed properly.
			this._renderedReportNode = this._renderedReport;
		} else {
			// Create a new node and appendChild so javascript gets executed.
			this._renderedReportNode = document.createElement('div');
			this._removeChildren(this._renderedReport);
			this._renderedReport.appendChild(this._renderedReportNode);
		}
		
		// Show the export and print buttons
		dojo.style(this._outputToolbar, "visibility", "visible");	
		
		// Insert the HTML to the appropriate node
		this._renderedReportNode.innerHTML = reportHTML;
	},
	
	/** Show a dialog that allows the user to start
	 *  another rendering request but for a specific
	 *  format.
	 */
	_showExportDialog: function() {
		// Ensure enough configuration data is known before exporting
		if (!this._renderedReportNode || !this._exportServiceUrl ||
			!this._template || !this._template.engine) {
			alert(messages.warnExportNotNow);
		} 
		
		// Parameters must be saved in the report before doing an export
		else if (this.editor.isDirty()){
			alert(messages.warnExportSaveNow);
		} 
		
		// Prompt the user for the format of the exported report
		else {
			new dm.app.reports.ui.internal.pages.ExportDialog({onOk: dojo.hitch(this, "_performExport"), engine: this._template.engine});
		}
		
	},
	
	/** Perform the export of the report in the given format.
	 * 
	 * @param format the desired output format of the report
	 */
	_performExport: function(format) {
		// Compute the reporting service URL for rendering
		// the report and starting the down dialog
		var url = this._exportServiceUrl;
		url += "?__format=" + format;
		url += "&__queryUUID=" + this._report[FIELD_queryUUID];
		
		// Open a new window for this export operation
		window.open(url);
	},
	
	/** View the rendered (HTML) output in a printable page.
	 * 
	 */
	_viewPrintablePage: function() {
		if (this._renderedReportNode) {
			var win = window.open('');
			win.document.write('<html xmlns="http://www.w3.org/1999/xhtml">\n');
			win.document.write('<head>\n');
			win.document
					.write('<meta http-equiv="content-type" content="text/html; charset=UTF-8"/>\n');
			win.document.write('<style type="text/css">\n');
			win.document
					.write('a { text-decoration: none; }\n');
			win.document.write('img { border: 0 none; }\n');
			win.document.write('</style>\n');
			if (this._renderedReportCssHref) {
				win.document.write('<link rel="stylesheet" type="text/css" href="' +
						this._renderedReportCssHref + '">\n');
			}
			win.document.write('</head>\n');
			win.document.write('<body>\n');
			win.document.write(this._renderedReportNode.innerHTML);
			win.document.write('\n');
			win.document.write('</body>\n');
			win.document.write('</html>\n');
			win.document.close();
		}
	},
	
	/** Remove all child elements from the given element. 
	 * 
	 * @param element the parent element
	 */
	_removeChildren: function(element) {
		while (element.firstChild) {
			element.removeChild(element.firstChild);
		}
	}


	
});

})();
