/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.reports                                        |
 *|                                                                        |
 *| Copyright IBM Corporation 2012, 2013.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
dojo.provide("dm.app.reports.ui.internal.pages.ReportEditorParams");

dojo.require("dm.app.reports.ui.internal.util.ReportUtil");
dojo.require("dm.app.reports.ui.internal.util.ReportServiceClient");
var ReportUtil          = dm.app.reports.ui.internal.util.ReportUtil;
var ReportServiceClient = dm.app.reports.ui.internal.util.ReportServiceClient;

dojo.require("jazz.ui.StyledTable");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.InlineEditBox");
dojo.require("net.jazz.ajax.ui.PlatformUI");
var PlatformUI = net.jazz.ajax.ui.PlatformUI;

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportParametersPanel");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportParamFactory");
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportParamFactory     = com.ibm.team.reports.web.ui.internal.ReportParamFactory;

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.requireLocalization("dm.app.reports.ui.internal", "ReportMessages");
var messages  = dojo.i18n.getLocalization("dm.app.reports.ui.internal", "ReportMessages");

dojo.require("jazz.util.html");
var HTML = jazz.util.html;


// Copied from ReportEditor
var FIELD_queryUUID = "queryUUID";				// the ID of this report configuration

// Define the columns of the parameter table
var TABLE_NAME  = "parameterNameColumn";
var TABLE_VALUE = "parameterValueColumn";


// The variable ReportParamFactory.DISPLAY_EDIT_BOX cannot be accessed
// so the following constant is defined. Ensure its value corresponds
// to the original value in ReportParamFactory
var ReportParamFactory_DISPLAY_EDIT_BOX = 3;

/** The Parameters tab of the editor for modifying a report configuration.
 *  
 *  @author Kevin Cornell
 */


dojo.declare("dm.app.reports.ui.internal.pages.ReportEditorParams", [dijit._Widget, dijit._Templated], {

	"class": "reportEditorParams",

	messages: messages, 
	
	widgetsInTemplate: true,
	templateString: 
		'<div>' +
			'<div class="usePanelContainer" dojoAttachPoint="_usePanelContainer">' +
				'<input type="checkbox" dojoType="dijit.form.CheckBox" dojoAttachPoint="_usePanelCheckbox">${messages.useTemplateMetadata}</input>' +
			'</div>' +
			'<div class="parameterDisplay" dojoAttachPoint="_parameterDisplay"></div>' +
		'</div>',
	
	// init properties
	editor: null,
	
	// temp - keep a table of the parameters
	
	// Remember the root URL and the corresponding query ID
	_rootUrl: null,
	_queryUUID: null, 
	
	// Keep a copy of the parameter data
	_params: null,
	
	// Remember the parameter panel and the parameter table.
	// If a template does not have parameter metadata, the 
	// table is displayed and the check box is hidden.
	_paramPanel: null,
	_paramsTable: null,
	
	// Keep a list of all parameter widgets (needed for groups)
	_paramWidgets: null,
	
	// Keep track of the getParameterValues() requests
	_numGetValueRequest: 0,
	_numGetValueResponse: 0,
	
	// Create a map of the parameter name to the 
	// corresponding table edit control and parameter widget.
	// The key is the parameter name and the value is
	// an object with 2 fields: tableEditBox and paramWidget
	_paramMap: null,	
	
	postCreate: function() {
		this.inherited(arguments);

		// Get the reporting root URL where the JFS reporting framework resides
		this._rootUrl = ReportUtil.getReportingRootUrl();
		
		// Create the panel for displaying parameters using metadata
		this._paramPanel = new com.ibm.team.reports.web.ui.internal.ReportParametersPanel();
		this._paramWidgets = [];
		
		// Create the table for displaying parameters that does not use metadata
		this._paramsTable = new jazz.ui.StyledTable(
				{columns: [{label: messages.paramNameHeading,  width: "20%", sortable: false, className: TABLE_NAME}, 
				           {label: messages.paramValueHeading, width: "80%", sortable: false, className: TABLE_VALUE }],
							trim: jazz.ui.styledTable.TrimStyles.GOLD,
							sortBy: 0});

		// Connect to the checkbox for displaying the
		// parameter widgets (versus the parameter table
		this.connect(this._usePanelCheckbox, "onClick", function() {
			var isChecked = this._usePanelCheckbox.checked;
			this._removeChildren(this._parameterDisplay);
			if (isChecked) {
				this._parameterDisplay.appendChild(this._paramPanel.domNode);
			} else {
				this._parameterDisplay.appendChild(this._paramsTable.domNode);
			}
			this.editor.resize(this.editor._lastSize);
		});
		this.initReportData(this.editor._newReport, this.editor._folder, this.editor._template, this.editor._params);
	},
	
	/** Initialize the page controls based on the given
	 *  report data.
	 *  
	 *  @param report the report (query) descriptor
	 *  @param folder the parent folder descriptor
	 *  @param template the report template descriptor
	 *  @param params contains parameter definitions and values, and variables
	 */
	initReportData: function(report, folder, template, params) {
		// If the parameters have already been initialized, return.
		if (this._params) {
			return;
		}
		this._params = params;
		
		// If there are no parameters to initialize, nothing to do.
		if (!params.paramNames || params.paramNames.length === 0) {
			return;
		}
		
		// Define variables used throughout this method
		var paramName  = null;
		var paramValue = null;
		var paramDefn  = null;
		var numParams  = params.paramNames.length;
		var numParamDefns = params.parameterDTO.length;
				
		// Save the report ID
		this._queryUUID = report[FIELD_queryUUID];
		
		// Reset the counters for controlling the loading message visibility.
		this._numGetValueRequest  = 0;
		this._numGetValueResponse = 0;

		// Initialize the parameter map
		this._paramMap = {};
		for (var index = 0; index < numParams; index++) {
			paramName = params.paramNames[index];
			this._paramMap[paramName] = {tableEditBox: null, paramWidget: null};
		}
		
		// Initialize the table and panel widgets using the parameter definitions
		var templateHasMetaData = false;
		for (index = 0; index < numParamDefns; index++) {
			// If the parameter corresponds to a system variable, 
			// skip it since it already has a valid value.
			paramDefn = params.parameterDTO[index];
			if (this.editor.isSystemVariable(paramDefn.name)) {
				continue;
			}
			
			// Determine if the template has parameter meta-data 
			// (e.g., at least one parameter has meta-data)
			if (paramDefn.display !== ReportParamFactory_DISPLAY_EDIT_BOX) {
				templateHasMetaData = true;
			}
			
			// Determine the current value of this parameter
			paramValue = null;
			for (var i = 0; i < numParams; i++) {
				if (params.paramNames[i] === paramDefn.name) {
					paramValue = params.paramValues[i];
					break;
				}
			}
			
			// If the parameter definition corresponds to a single parameter
			if (paramDefn._eQualifiedClassName === "com.ibm.team.reports.dto:ReportParameterDTO") {
				// Add an entry to the parameter table
				var paramRow = this._initializeTableEntry(paramDefn, paramValue);
				this._paramsTable.tbody.appendChild(paramRow);
				
				// Create the parameter widget and add it to the panel.
				var paramWidget = ReportParamFactory.create(paramDefn, this._rootUrl);
				if (!paramDefn.concealed) {
					paramWidget.isRoot = true;
					this._paramPanel.appendParam(paramWidget);
				}
				this._paramWidgets.push(paramWidget);
				
				// Initialize the parameter widget (request its parameter values)
				this._initializeParamWidget(paramDefn, paramWidget, paramValue);				
			}
			
			// else if the parameter corresponds to a group of parameters
			else if (paramDefn._eQualifiedClassName === "com.ibm.team.reports.dto:ReportParameterGroupDTO") {
				// Create the parameter group widget and add it to the panel.
				var paramGroupWidget = new com.ibm.team.reports.web.ui.internal.ReportParamGroup({
					parameterGroupDTO:	paramDefn,
					urlPrefix:			this._rootUrl,
					parameterName:		params.paramNames,
					parameterValue:		params.paramValues
				});
				this._paramPanel.appendParamGroup(paramGroupWidget);
				
				// Initialize the group parameter widget (request its contained parameter values)
				this._initializeGroupParamWidget(paramDefn, paramGroupWidget);				
			}
		}
		
		// If the template has meta-data, show the parameter panel
		if (templateHasMetaData) {
			this._parameterDisplay.appendChild(this._paramPanel.domNode);
			this._usePanelCheckbox.setChecked(true);
		}
		
		// else show the parameter table and hide the check box control
		else {
			this._parameterDisplay.appendChild(this._paramsTable.domNode);
			this._usePanelCheckbox.setChecked(false);
			dojo.style(this._usePanelContainer, "visibility", "hidden");
		}
	},
	
	/** Initialize an entry in the parameter table for the parameter
	 *  name and value. The user can edit the value in the table.
	 *  
	 *  @param paramDefn the parameter definition
	 *  @param paramValue the current parameter value
	 */
	_initializeTableEntry: function(paramDefn, paramValue) {
		// Create the element for the table row
		var paramName = paramDefn.name;
		var row = document.createElement("tr");
		
		// Column 1: parameter name
		var paramRequired = (paramDefn.required ? ReportUtil.MANDATORY_HTML : "");  //$NON-NLS-2$//$NON-NLS-1$
		var td = document.createElement("td");
		td.className = TABLE_NAME;
		td.innerHTML = HTML.escape(paramName) + paramRequired;
		row.appendChild(td);
		
		// Column 2: parameter value
		var div = document.createElement("div");
		div.innerHTML = HTML.escape(paramValue);
		td  = document.createElement("td");
		td.appendChild(div);
		td.className = TABLE_VALUE;
		row.appendChild(td);
		
		// in-line edits enabled
		var self = this;
        var editBox = new dijit.InlineEditBox({title: messages.resourceClickToEdit, autoSave: true, 
			// Add the resources to the table
			width: "200px", noValueIndicator: "?", 
			onChange: dojo.hitch(this, function(newValue) {
				self._updateParameterValue(paramName, newValue, true);
			})}, div);
		editBox.startup();
		
		// Add the parameter edit box to the parameter map
		this._paramMap[paramName].tableEditBox = editBox;

		return row;
	},
	
	/** Initialize a parameter widget in the panel.
	 *  The initialization adds the parameter widget to the
	 *  parameter map, adds a listener for changes and
	 *  issues a request to get the possible parameter values.
	 *  This latter request can take some time since the JFS
	 *  reporting framework will send REST requests to the
	 *  corresponding reporting service to find all applicable
	 *  resources, based on the parameter meta-data.
	 *  
	 *  @param paramDefn the parameter definition
	 *  @param paramWidget the parameter widget for displaying the values
	 *  @param currentValue the current value of the parameter
	 */
	_initializeParamWidget: function(paramDefn, paramWidget, currentValue) {

		// Add the parameter widget to the parameter map
		var paramName = paramDefn.name;
		if (this._paramMap[paramName].paramWidget) {
			return;		// already initialized
		}
		this._paramMap[paramName].paramWidget = paramWidget;
		
		// Add a listener to update the parameter value in the report
		var listener = {
			tab: this,
			setDirty: function(isDirty) {
				var newValue = "";
				var vals = paramWidget.getSelectedValues();
					if (vals && dojo.isArray(vals) && vals.length > 0 && vals[0]) {
						newValue = vals.join(",");
					}
				this.tab._updateParameterValue(paramName, newValue, false);
			}
		};
		paramWidget.setDirtyListener(listener);
		
		// Determine if the call to getParamValues() needs to be deferred.
		// It must be deferred if the parameter requires a filter string or
		// if the parameter has dependencies on other parameters.
		var deferGetValues = paramWidget.isDeferred;

		// Issue a request to obtain the possible parameter values.
		// If the parameter metadata URL does not define variables that
		// need to be resolved (e.g., deferred) the parameter values 
		// can be obtained immediately.
		var args = {queryUUID: this._queryUUID};
		if (!deferGetValues) {
			this._showLoadingMessage();
			
			// Create the request response object to handle the call
			// to getParameterValues().
			var serviceObject = {
					tab: this,
					success: function(parameterDTO) {
						paramWidget.init(parameterDTO);
						if (currentValue) {
							paramWidget.setValues(currentValue);
						}
						paramWidget.postInit();
						this.tab._hideLoadingMessage();
					},
				
					failure: function(error) {
						var message = dojo.string.substitute(messages.errorLoadingParamValue, [paramName]);
						this.tab.editor.serviceError(message, error);
						this.tab._hideLoadingMessage();
					}
				};
			
			var srhNoFilter = new ServiceResponseHandler(serviceObject, "success", "failure");
			args.parameter     = paramDefn.name;
			args.existingParameterNames  = this._params.paramNames;
			args.existingParameterValues = this._params.paramValues;
			ReportServiceClient.postComputeParameterValues(srhNoFilter, args);
		} 
		
		// else the call to get the parameter values must be deferred until
		// either a filter is given or dependent parameter values are set
		else {
			// Set the current value of the parameter widget
			if (currentValue) {
				paramWidget.setValues(currentValue);
			}
			
			// Set the deferred mechanism to update the selection list widget
			paramWidget.getValuesDeferred = dojo.hitch(this, function(additionalArgs) {
				// Encapsulate the widget response handler to log errors properly
				var serviceObject = {
						tab: this,
						success: function(parameterDTO) {
							paramWidget.init(parameterDTO);
							this.tab._hideLoadingMessage();
						},
						
						failure: function(error) {
							var message = dojo.string.substitute(messages.errorLoadingParamValue, [paramName]);
							this.tab.editor.serviceError(message, error);
							this.tab._hideLoadingMessage();
						}
					};
				
				// Issue the request for the parameter values using the actual
				// filter value that the user specified
				this._showLoadingMessage();
				var srhWithFilter = new ServiceResponseHandler(serviceObject, "success", "failure");
				args.parameter     = paramDefn.name;
				args.existingParameterNames  = this._params.paramNames;
				args.existingParameterValues = this._params.paramValues;
				if (additionalArgs) {
					dojo.mixin(args, additionalArgs);
				}
				ReportServiceClient.postComputeParameterValues(srhWithFilter, args);
			});
		}
	},
	
	/** Initialize a group parameter widget in the panel.
	 *  
	 *  @param paramDefn the parameter definition
	 *  @param paramGroupWidget the group parameter widget 
	 */
	// TODO KEVIN - group parameter code not tested
	_initializeGroupParamWidget: function(paramDefn, paramGroupWidget) {
		// Get the list of new parameter widgets that were added as
		// part of this group widget
		var newWidgets = paramGroupWidget.appendReportParameters(this._paramWidgets);
		
		// Initialize each of the new parameter widgets added to the group
		for (var i = 0; i < newWidgets.length; i++) {
			var newParamWidget = newWidgets[i];
			var newParamDefn   = newParamWidget.parameterDTO;
			this._initializeParamWidget(newParamDefn, newParamWidget, null);	
		}
		
		// Request the group parameter values
		var serviceGroup = {
				tab: this,
				success: function(reportParameterGroupDTO) {
					paramGroupWidget.initGroupValues(reportParameterGroupDTO);
					this.tab._hideLoadingMessage();
				},
			
				failure: function(error) {
					var message = dojo.string.substitute(messages.errorLoadingParamValue, [paramDefn.name]);
					this.tab.editor.serviceError(message, error);
					this.tab._hideLoadingMessage();
				}
			};
		this._showLoadingMessage();
		var srh = new ServiceResponseHandler(serviceGroup, "success", "failure");
		var args = {queryUUID: this._queryUUID};
		args.groupName     = paramDefn.name;
		args.existingParameterNames  = this._params.paramNames;
		args.existingParameterValues = this._params.paramValues;
		ReportServiceClient.getParameterGroupValues(srh, args);	
	},

	/** Show the loading parameters message and update the
	 *  request count.
	 */
	_showLoadingMessage: function() {
		this._numGetValueRequest++;
		PlatformUI.getWorkbench().showMessage(messages.loadingReportParams);
	},

	/** Hide the loading parameters message after all of the
	 *  get parameter value requests have been handled.
	 */
	_hideLoadingMessage: function() {
		this._numGetValueResponse++;
		if (this._numGetValueResponse >= this._numGetValueRequest) {
			PlatformUI.getWorkbench().hideMessageArea();
		}
	},
	
	/** Update a parameter value when either the value in the 
	 *  corresponding table entry changed, or the value in the
	 *  parameter panel widget changed. When a value changes,
	 *  the parameter value is updated in the editor and in the
	 *  opposite display control.
	 *  
	 *  @param paramName the name of the parameter
	 *  @param newValue the new value for the parameter
	 *  @param inTable	true  = parameter was modified in the table
	 *					false = parameter was modified in the panel widget
	 */
	_updateParameterValue: function(paramName, newValue, inTable) {
		// Update the parameter value in the editor (update the report)
		this.editor.updateReportParameter(paramName, newValue);

		// Update the opposite display control for the parameter
		var paramWidget  = this._paramMap[paramName].paramWidget;
		var tableEditBox = this._paramMap[paramName].tableEditBox;
		if (inTable && paramWidget) {
		    var existingValues = paramWidget.getSelectedValues();
		    if (existingValues && existingValues[0] !== newValue) {
			    paramWidget.setValues(newValue);
			}
		} else if (!inTable && tableEditBox){
			tableEditBox.setValue(newValue);
		}		
	},
	
	/** Remove all child elements from the given element. 
	 * 
	 * @param element the parent element
	 */
	_removeChildren: function(element) {
		while (element.firstChild) {
			element.removeChild(element.firstChild);
		}
	}

	
});

/** Define the listener for handling parameter changes.
 *  
 *  @param editor the report editor
 *  @param paramName the report parameter name
 *  @param paramWidget the parameter widget that contains the value
 * 
 */
dojo.declare("dm.app.reports.ui.internal.pages.ReportEditorParams._ParamListener", [dijit._Widget], {
	
	// constructor arguments
	tab: null,
	paramName: null,
	paramWidget: null,
	
	setDirty: function(isDirty) {
		var newValue = "";
		var vals = this.paramWidget.getSelectedValues();
		if (vals && dojo.isArray(vals) && vals.length > 0 && vals[0]) {
			newValue = vals.join(",");
		}
		this.tab._updateParameterValue(this.paramName, newValue, false);
	}
	
});


})();
