/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.pages.ReviewSearchProvider");

dojo.require("rmps.app.core.internal.util.QueryUtil");
dojo.require("rmps.app.core.internal.query.TextQuery");
dojo.require("rmps.app.ui.internal.search.QuickSearchAdvancedOptions");

dojo.require("rmps.app.core.util");
dojo.require("rmps.core.RootServices");
dojo.require("rmps.core.util.BaselineUtil");

dojo.require("jazz.ui._SearchProvider");
dojo.require("jazz.app.context");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.app.ui.internal.pages", "ReviewSearchProviderMessages");

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages", "ReviewSearchProviderMessages");

var QueryUtil = rmps.app.core.internal.util.QueryUtil;
var BaselineUtil = rmps.core.util.BaselineUtil;

dojo.declare("rmps.app.ui.internal.pages.ReviewSearchProvider", jazz.ui._SearchProvider, {
	
	scopes: [
			{ id: QueryUtil.SCOPE.REVIEWS.value, label: QueryUtil.SCOPE.REVIEWS.name, icon: QueryUtil.SCOPE.REVIEWS.icon }
		],
	
	tooltip: messages.searchReviews,
	
	pageSize: 50,
	limit: 200,
	orderBy: [{id: "modified", isAscending: false}, {id: "shortName", isAscending: true}],
	_loadResourceDeferred: null,
	
	onExecute: function(args) {
		var searchText = args.query;
		var scope = args.scope;
		
		if (searchText.length === 0) {
			return;
		}
		
		if (searchText.search(/^\d+$/) != -1) {			
			var reviewUri = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "reviews")+"/"+searchText;
			//add the current baseline as a parameter to the review URI			
			var requestUri = BaselineUtil.addBaselineParamaterToUrl(reviewUri, jazz.app.context.get().baseline, false, true);
			
			var xhrArgs = {
				url: requestUri,
				headers: {
					"X-ibm-rmps-internal": "true",
					"Accept": "application/rdf+xml"
				}
			};
			
			if (this._loadResourceDeferred) {
				this._loadResourceDeferred.cancel();
			}
			this._loadResourceDeferred = jazz.client.xhrHead(xhrArgs);
			this._loadResourceDeferred.addCallback(this, function(result) {
				//it is not necessary to explicitly append the baseline URL to this URI.  The util method returns the
				//URI fragment only; so, the query part of the existing URI (which already contains the baseline parameter)
				//is not modified
				window.location.href = rmps.app.core.util.getViewResourceUri(reviewUri);
			});
			this._loadResourceDeferred.addErrback(this, function(result) {
				// TODO Replace this alert with something more elegant
				alert(messages.reviewDoesNotExist);
			});
			return;
		}
		
		var query = new rmps.app.core.internal.query.TextQuery();
		query.pageSize = this.pageSize;
		query.limit = this.limit;
		query.text = dojo.trim(searchText);
		query.orderBy = this.orderBy;
		query.resourcesOnly = false;
		query.scope = QueryUtil.SCOPE.valueOf(scope);
		
		query.projectId = jazz.app.context.get().itemId;
		query.baseline = jazz.app.context.get().baseline;
		
		var timestamp = new Date().getTime();
		
		window.location.href = rmps.app.core.util.getSearchUri(query.toQueryString()) + "&ts=" + timestamp;
	}
});

})();
