/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.pages.SideNavigation");

dojo.require("rmps.app.ui.internal.pages.SideNavigationLayout");

dojo.require("dijit._Widget");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("rmps.app.core.util");
dojo.require("rmps.ui.editor.PageHelper");
dojo.require("rmps.ui.editor.EditorTrackingWidget");

dojo.require("rmps.app.ui.internal.ResourceEditorInput");

dojo.require("jazz.app.context");

dojo.requireLocalization("rmps.app.ui.internal.pages", "SideNavigationMessages");
dojo.requireLocalization("rmps.ui.common", "CommonMessages");

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages", "SideNavigationMessages");
var commonMessages = dojo.i18n.getLocalization("rmps.ui.common", "CommonMessages");

var CoreUtil = rmps.app.core.util;

var MAX_UNSAVED = 2;

/* 
* Provides a side navigation widget for different pages to use. It tracks the newly opened/edited/closed editors. By default it only has two sections to show
* the unsaved items, and the recently viewed items. We can define more sections by extending the method dadsadsa as follows:
* 
* this._sideNav = new rmps.app.ui.internal.pages.SideNavigation({
*			pageId: "myPage",
*			addSectionsCreators: function(sectionsCreators){
*				sectionsCreators.push(rmps.app.ui.internal.pages.SideNavigationBaselineContents.createBaselineNavigationContent);
*				this.inherited("addSectionsCreators", arguments);
*			}
*		});
* @see: rmps.app.ui.internal.pages.ExplorerSideNavigation  
* 
*/
dojo.declare("rmps.app.ui.internal.pages.SideNavigation", rmps.ui.editor.EditorTrackingWidget, {
	
	"class": "rmps-app-sideNavigation",
	
	// private properties
	_editorUtil: null,
	_layout: null,
	_searchRow: null,
	_topRecentlyViewedRow: null,
	_recentlyViewed: null, //{}
	_unsavedWork: null, //{}
	_highlightedNode: null,
	_topUnsavedWorkRow: null,
	
	_recentlyViewedSection: null,
	_unsavedWorkSection: null,
	_resourcesShowingSection: null,
	_content: null,
	_banner: null,
	_height: null,
	_bannerContainer: null,
	_contentContainer: null,
	
	_sectionsCreators: null,
	
	pageId: null,
	
	constructor: function() {
		this._recentlyViewed = {};
		this._unsavedWork = {};
		this._sectionsCreators= [];
	},
	
	buildRendering: function() {
		this.inherited(arguments);
		this.containerNode = this.domNode;
	},

	postCreate: function() {
		this._layout = new rmps.app.ui.internal.pages.SideNavigationLayout(); 
		
		this.domNode.appendChild(this._layout.domNode);
		
		this.addSectionsCreators(this._sectionsCreators);
		this.addSections();
		
		// create banner and content containers
		this._bannerContainer = dojo.doc.createElement("div");
		this._bannerContainer.style.position = "relative";
		this._bannerContainer.style.display = "none";
		
		this._contentContainer = dojo.doc.createElement("div");
		this._contentContainer.style.position = "relative";
		this._contentContainer.style.overflow = "hidden";
		
		this._layout.setMainContent(this._contentContainer, this._bannerContainer);
		
		this.inherited(arguments);
	},
	
	getPageId: function(){
		return this.pageId;
	},
	
	getContentContainer: function(){
		return this._contentContainer;
	},
	
	addSectionsCreators: function(sectionsCreators){
		sectionsCreators.push(this.createRecentlyViewNavigationContent);
		sectionsCreators.push(this.createUnsavedWorkNavigationContent);
	},
	
	addSections: function() {
		for (var i = 0, creator; creator = this._sectionsCreators[i]; ++i) {
			creator(this);	
		}
	},
	
	createRecentlyViewNavigationContent: function(sideNavigation){
		// Add recently viewed section
		sideNavigation._recentlyViewedSection = sideNavigation._layout.addNavigationSection(dojo.doc.createElement("div"), messages.recentlyViewed, true);
		sideNavigation._recentlyViewedSection.setVisible(false);
	},
	
	createUnsavedWorkNavigationContent: function(sideNavigation){
		// Add unsaved work section
		sideNavigation._unsavedWorkSection = sideNavigation._layout.addNavigationSection(new rmps.app.ui.internal.pages._UnsavedWorkSection({parent: sideNavigation}), messages.unsavedWork, true);
		
		// TODO this is a fix for sections show less
		sideNavigation._unsavedWorkSection._showLess.href = "/showPartial";
		sideNavigation._unsavedWorkSection.setVisible(false);
	},
	
	startup: function() {
		this.inherited(arguments);
		if (this.pageHelper.isStarted()) {
			this._listenMRU();
		} else {
			this._mruHandle = this.connect(this.pageHelper, "startup", this._listenMRU);
		}
	},

	_listenMRU: function() {
		this.startListeningOnEditorTracker();
		
		if (this._mruHandle) {
			this.disconnect(this._mruHandle);
		}
		this.connect(this.pageHelper, "onEditorResize", this.resize);
		
		this.connect(this.pageHelper, "onSearch", this._onSearch);
		var lastSearch = this.pageHelper.getLastSearch();
		if (lastSearch) {
			this._onSearch(lastSearch.queryResult, lastSearch.iconUri);
		}
		this.connect(this.pageHelper, "onLastViewedChanged", this._onLastViewedChanged);
	},
		
	resize: function(size) {
		if (!size) {
			return;
		}
		
		if (size.h) {
			this._height = size.h;
			
			var height = size.h - this._bannerContainer.offsetHeight;
			if (this._content && this._content.resize) {
				this._content.resize({h: height});
			}
			this._contentContainer.style.height = height + "px";
			this._layout._leftNav.style.height = (size.h - 3) + "px"; // -3 due to margin and padding of parent nodes
		}
		if (dojo.isIE < 8) {
			this.sliderResize(true); // otherwise scrollbars appear before
										// page is totally loaded
		} 
		this.sliderResize(false);
	},
	
	sliderResize: function(scrollbars) {
		var con = dojo.contentBox(this._layout._leftNav);
		var delta = dojo.marginBox(this._layout._leftNav).w - con.w;
		var width = con.w;
		if (!width) return;
		for (var i in this._layout._sections) {
			var section = this._layout._sections[i];
			if (scrollbars){
				dojo.style(section.domNode, {
					width: width + delta + "px"
				});
			}
			else {
				dojo.style(section.domNode, {
					width: width + "px"
					});		
			}
		}
	},
	
	addContent: function(content, banner) {
		if (this._content !== content) {
			if (this._content) {
				this._content.domNode.style.display = "none";
			}
			this._content = content;
		} else {
			content = null;
		}
		
		if (this._banner !== banner) {
			if (this._banner) {
				this._banner.domNode.style.display = "none";
			}
			this._banner = banner;
			if (banner) {
				this._bannerContainer.style.display = "";
			} else {
				this._bannerContainer.style.display = "none";
			}
		} else {
			banner = null;
		}
		
		if (banner) {
			if (banner.domNode.parentNode === this._bannerContainer) {
				banner.domNode.style.display = "";
			} else {
				this._bannerContainer.appendChild(banner.domNode);
			}
		}
		
		if (content) {
			this._clearHighlight();
			if (content.domNode.parentNode === this._contentContainer) {
				content.domNode.style.display = "";
			} else {
				this._contentContainer.appendChild(content.domNode);
			}
		}
		
		this.resize({h: this._height});
	},
	
	removeContent: function(content, banner) {
		if (content) {
			if (content == this._content) {
				delete this._content;
			}
			this._contentContainer.removeChild(content.domNode);
		}
		if (banner) {
			if (banner == this._banner) {
				delete this._banner;
			}
			this._bannerContainer.removeChild(banner.domNode);
		}
	},
	
	_onLastViewedChanged: function(lastViewed) {
		if (lastViewed) {
			if (lastViewed.object.__sectionRow) {
				this.highlightNode(lastViewed.object.__sectionRow);
			}
		} else {
			this._clearHighlight();
		}
	},
	
	//////////////////
	onAddRecentlyViewed: function(editorInput) {
		var item = this._addRecentlyViewed(
			editorInput.getId(),
			editorInput.getResourceNumber?editorInput.getResourceNumber(): null,
			editorInput.getName() || editorInput.isLoaded() ? editorInput.getName() : this.pageHelper.getEditorTracker().loadingMessage,
			editorInput.getTooltip(),
			editorInput.getPresentationUri(),
			editorInput.getIconUri() || editorInput.isLoaded() ? editorInput.getIconUri() : this.pageHelper.getEditorTracker().loadingIcon
		);
		item.editorInput = editorInput;
		
		editorInput.__sectionRow = item.sectionRow;
		
		if (this.pageHelper.getLastViewed() && editorInput === this.pageHelper.getLastViewed().object) {
			this.highlightNode(item.sectionRow);
		}
	},
	
	onAddUnsavedWork: function(editorInput) {
		var editor = this.pageHelper.getEditor(editorInput);
		var item = this._addUnsavedWork(
				editorInput.getId(),
				editorInput.getResourceNumber?editorInput.getResourceNumber(): null,
				editorInput.getName(),
				editorInput.getTooltip(),
				editorInput.getPresentationUri(),
				editorInput.getIconUri(),
				editor
			);
		item.editorInput = editorInput;
		
		editorInput.__sectionRow = item.sectionRow;
		
		if (this.pageHelper.getLastViewed() && editorInput === this.pageHelper.getLastViewed().object) {
			this.highlightNode(item.sectionRow);
		}			
	},
	
	onRemoveRecentlyViewed: function(editorInput) {
		delete editorInput.__sectionRow;
		var id = editorInput.getId();
		this._removeRecentlyViewed(id);
	},
	
	onRemoveUnsavedWork: function(editorInput) {
		delete editorInput.__sectionRow;
		var id = editorInput.getId();
		this._removeUnsavedWork(id);
	},
	
	onUpdate: function(editorInput) {
		var editor = this.pageHelper.getEditor(editorInput);
		
		if (editor.isDirty()) {
			var item = this._addUnsavedWork(
				editorInput.getId(),
				editorInput.getResourceNumber?editorInput.getResourceNumber(): null,
				editorInput.getName(),
				editorInput.getTooltip(),
				editorInput.getPresentationUri(),
				editorInput.getIconUri(),
				editor
			);
		} else {
			var item = this._addRecentlyViewed(
				editorInput.getId(),
				editorInput.getResourceNumber?editorInput.getResourceNumber(): null,
				editorInput.getName(),
				editorInput.getTooltip(),
				editorInput.getPresentationUri(),
				editorInput.getIconUri()
			);
		}
		
		item.editorInput = editorInput;
		editorInput.__sectionRow = item.sectionRow;
		
		if (this.pageHelper.getLastViewed() && editorInput === this.pageHelper.getLastViewed().object) {
			this.highlightNode(item.sectionRow);
		}
	},
	
	///////////////////////
	
	
	_onSearch: function(searchObj) {
		var queryResult = searchObj.queryResult;
		var iconUrl = searchObj.iconUri;
		
		var id = queryResult.id;
		var query = queryResult.query;
		var name = query.name;
		var title = query.description;
		
		this._recentlyViewedSection.setVisible(true);
		if (this._searchRow) {
			this._destroySectionRow(this._searchRow.sectionRow);
		}
		
		var sectionRow = this._recentlyViewedSection.addSectionRow(name,
				CoreUtil.getSearchUri(query.toQueryString()), iconUrl, this._topRecentlyViewedRow);
		
		if (title) {
			sectionRow.title = dojo.string.substitute(messages.search_tooltip, [name, title]);
		}
		
		this._searchRow = {
			'id': id,
			'sectionRow': sectionRow
		};
		
		sectionRow.__hightlightNodeHandle = this.connect(sectionRow, "onclick", function() {
			this.highlightNode(sectionRow);
		});
		
		var queryToExecute = dojo.clone(query);
		delete queryToExecute.token;
		
		var timestamp = new Date().getTime();
		this._recentlyViewedSection.addArtifactCommand(sectionRow,
			messages.search_refresh_tooltip,
			CoreUtil.getSearchUri(queryToExecute.toQueryString()) + "&ts=" + timestamp,
			"search-refresh", true);
		
		searchObj.__sectionRow = sectionRow;
	},
	
	_removeRecentlyViewed: function(id) {
		var item = this._recentlyViewed[id];
		if (item) {
			if (item.sectionRow == this._topRecentlyViewedRow) {
				this._topRecentlyViewedRow = this._topRecentlyViewedRow.nextSibling;
			}
			this._destroySectionRow(item.sectionRow);
			delete this._recentlyViewed[id];
			if (this.pageHelper.getEditorTracker().getRecentlyViewedCount() === 0 && this._searchRow == null) {
				this._recentlyViewedSection.setVisible(false);
			}
			return true;
		}
		return false;
	},
	
	_removeUnsavedWork: function(id) {
		var item = this._unsavedWork[id];
		if (item) {
			if (item.sectionRow == this._topUnsavedWorkRow) {
				this._topUnsavedWorkRow = this._topUnsavedWorkRow.nextSibling;
			}
			this._destroySectionRow(item.sectionRow);
			delete this._unsavedWork[id];
			if (this.pageHelper.getEditorTracker().getUnsavedWorkCount() === 0) {
				this._unsavedWorkSection.setVisible(false);
			}
			this._updateUnsavedCount();
			return true;
		}
		return false;
	},
	
	_addUnsavedWork: function(id, resourceNumber, name, title, href, iconUrl, editor) {
		this._unsavedWorkSection.setVisible(true);
		var item = this._unsavedWork[id];
		if (!item) {			
			var sectionRow = this._unsavedWorkSection.addSectionRow(
					this._safeString(resourceNumber? dojo.string.substitute(messages.dirtyName, [resourceNumber, name]): name),
					href, iconUrl, this._topUnsavedWorkRow);
			
			var item = {};
			this._unsavedWork[id] = item;
			this._topUnsavedWorkRow = sectionRow;
			this._updateUnsavedCount();
		} else {
			var next = item.sectionRow.nextSibling;
			this._destroySectionRow(item.sectionRow);
			var sectionRow = this._unsavedWorkSection.addSectionRow(
					this._safeString(resourceNumber?dojo.string.substitute(messages.dirtyName, [resourceNumber, name]):name),
					href, iconUrl, next);
			if (this._topUnsavedWorkRow === item.sectionRow) {
				this._topUnsavedWorkRow = sectionRow;
			}
		}
		
		if (!editor.isReadOnly) {
			this._unsavedWorkSection.addArtifactCommand(sectionRow, messages.save, function(e) {
				dojo.stopEvent(e);
				location.href = href;
				editor.save();
			}, "save-editor", true);
		}
		
		this._handleIconError(sectionRow, iconUrl);
		
		if (title) {
			sectionRow.title = title;
		}
		
		item.sectionRow = sectionRow;
		
		sectionRow.__hightlightNodeHandle = this.connect(sectionRow, "onclick", function() {
			this.highlightNode(sectionRow);
		});
		
		return item;
	},
	
	_updateUnsavedCount: function() {
		var titleNode = this._unsavedWorkSection._togglerLinkText || this._unsavedWorkSection._titleText; 
		if (titleNode) {
			var newTitle = dojo.string.substitute(messages.unsavedWorkCount, [this.pageHelper.getEditorTracker().getUnsavedWorkCount()]);
			while (titleNode.firstChild) {
				titleNode.removeChild(titleNode.firstChild);
			}
			titleNode.appendChild(dojo.doc.createTextNode(newTitle));
		}
	},
	
	_addRecentlyViewed: function(id, resourceNumber, name, title, href, iconUrl) {
		var unsavedItem = this._unsavedWork[id];
		if (unsavedItem) {
			return unsavedItem;
		}
		
		this._recentlyViewedSection.setVisible(true);
		var item = this._recentlyViewed[id];
		if (!item) {
			var sectionRow = this._recentlyViewedSection.addSectionRow(
					this._safeString(resourceNumber? dojo.string.substitute(messages.recentName, [resourceNumber, name]):name),
					href, iconUrl, this._topRecentlyViewedRow);
			
			var item = {};
			this._recentlyViewed[id] = item;
			this._topRecentlyViewedRow = sectionRow;
		} else {
			var next = item.sectionRow.nextSibling;
			this._destroySectionRow(item.sectionRow);
			var sectionRow = this._recentlyViewedSection.addSectionRow(
					this._safeString(resourceNumber? dojo.string.substitute(messages.recentName, [resourceNumber, name]):name),
					href, iconUrl, next);
			if (this._topRecentlyViewedRow === item.sectionRow) {
				this._topRecentlyViewedRow = sectionRow;
			}
		}
		
		this._handleIconError(sectionRow, iconUrl);
		
		if (title) {
			sectionRow.title = title;
		}
		
		item.sectionRow = sectionRow;
		
		sectionRow.__hightlightNodeHandle = this.connect(sectionRow, "onclick", function() {
			this.highlightNode(sectionRow);
		});
		
		return item;
	},
	
	_safeString: function(string) {
		return string.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/"/gm, "&quot;");
	},
	
	_destroySectionRow: function(sectionRow) {
		this.disconnect(sectionRow.__hightlightNodeHandle);
		sectionRow.__hightlightNodeHandle = null;
		dojo.destroy(sectionRow);
	},
	
	_clearHighlight: function() {
		if (this._highlightedNode) {
			dojo.removeClass(this._highlightedNode, "selected");
		}
	},
	
	highlightNode: function(node){
		if (this._highlightedNode != node) {
			this._clearHighlight();
			dojo.addClass(node, "selected");
			this._highlightedNode = node;
		}
		this._unsavedWorkSection.contentWidget.update();
	},
	
	_handleIconError: function(sectionRow, iconUrl) {
		if (dojo.isIE) {
			// work around to hide the section icon if it fails to load
			var icon = new Image();
			var handles = [
					this.connect(icon, "onerror", function() {
						dojo.forEach(handles, this.disconnect, this);
						dojo.addClass(sectionRow, "iconError");
					}),
					this.connect(icon, "onload", function() {
						dojo.forEach(handles, this.disconnect, this);
					})
				];
			icon.src = iconUrl;
			icon = null;
		}
	}
	
});

dojo.declare("rmps.app.ui.internal.pages._UnsavedWorkSection", dijit._Widget, {
	
	parent: null,
	
	sectionSize: MAX_UNSAVED,
	
	hasShowPartial: function() {
		var maxSize = MAX_UNSAVED;
		if (this.parent._highlightedNode && this.parent._highlightedNode.style.display === "none") {
			maxSize++;
		}
		return this.parent.getPageHelper().getEditorTracker().getUnsavedWorkCount() > maxSize;
	},
	
	onShowPartial: function() {
		this.sectionSize = MAX_UNSAVED;
		this._showHideRows();
	},
	
	onMaximize: function() {
		this.sectionSize = this.parent.getPageHelper().getEditorTracker().getUnsavedWorkCount();
		this._showHideRows();
	},
	
	_showHideRows: function() {
		var items = [];
		
		for (var key in this.parent._unsavedWork) {
			items.push(this.parent._unsavedWork[key].sectionRow);
		}
		items.reverse();
		
		for (var i = 0; i < items.length; ++i) {
			var item = items[i];
			if (i < this.sectionSize || item === this.parent._highlightedNode) {
				item.style.display = "";
			} else {
				item.style.display = "none";
			}
		}
	},
	
	update: function() {
		if (!this.hasShowPartial()) {
			for (var key in this.parent._unsavedWork) {
				this.parent._unsavedWork[key].sectionRow.style.display = "";
			}
		}
		this.parent._unsavedWorkSection.update();
	}
});

dojo.declare("rmps.app.ui.internal.pages._ContextMenuSubSection", jazz.ui.SideNavigationLayout.NavSubSection, {
	
	_menudiv: null,
	
	postCreate: function() {
		this.inherited(arguments);
		
		// Add context menu (if content widget support it)
		if (!this.contentWidget.onContextMenu)
			return;
		
		// Add context menu link
		this._menudiv = dojo.create("div", null, this.navSectionEntry);
		dojo.addClass(this._menudiv, "context-menu-div");
		this.connect(this.domNode, "onmouseenter", dojo.hitch(this, function(evt) {
			// Get the scroll wrapper
			var nl = new dojo.NodeList(this.domNode);
			var wrapper = nl.closest(".jazz-ui-SideNavigationLayout-nav-wrapper")[0];
			var left = wrapper.clientWidth - 30 + wrapper.scrollLeft;
			dojo.style(this._menudiv, {
				left: left + "px" 
			});			
		}));
		this.connect(this._menudiv, "onclick", dojo.hitch(this, this.onContextMenu));
	},
	
	onContextMenu: function()
	{
		this.contentWidget.onContextMenu(this._menudiv);
	}
	
});

})();
