/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.pages.SideNavigationLayout");
dojo.require("jazz.ui.SideNavigationLayout");

dojo.declare("rmps.app.ui.internal.pages.SideNavigationLayout", jazz.ui.SideNavigationLayout, {
	
	
	// copied from jazz
	_routeClick: function(e){
		// walk up the dom tree from the event target
	 
		var node = e.target;
		var target = node;
		while (true) {
			// if the event bubbled all the way to the left-nav itself, the
			// event didn't occur on one of the nodes we care about
			if (!node || node === this._leftNav || dojo.hasClass(node, "jazz-ui-SimpleToolbar-item")) {
				return;
			}
			// if we hit a node with our special variable, it means that we've
			// hit a hyperlink that wants us to affect the state of the section
			if(node.__jazzNavSection) {
				// stop the event, since the URI is bogus
				dojo.stopEvent(e);
				// only fire the action is the section is collapsable
				if (dijit.byId(node.__jazzNavSection)._collapsable ) {
					// extract the action we're going to take from the link's href
					// (something like "minimize", "maximize", etc.)
					var action = node.href.slice(node.href.lastIndexOf("/") + 1);
					// execute the action and return
					dijit.byId(node.__jazzNavSection)._runAction(action);
				} else {
					node.blur();
				}
				return;
			}
			node = node.parentNode;
		}
	},
	  
	_stopDragging: function(e) {
		dojo.removeClass(document.body, "jazz-ui-SideNavigationLayout-dragCursor");
		if (this._dragListeners) {
			for (var i = 0; i < this._dragListeners.length; i++) {
				this.disconnect(this._dragListeners[i]);
			}
			delete this._dragListeners;
		}
		var widthpx = dojo.coords(this._leftNav).w;
		if (widthpx >= LEFT_NAV_MIN_WIDTH) {
			this._leftNavWidth = widthpx;
		}
		this._isDragging = false;
		dojo.publish("jazz.ui.SideNavigationLayout.slideLeftNav.onEnd");
	},
	  
	_drag: function(e) {
		var x = e.pageX - dojo.coords(this._top).x - LEFT_NAV_PADDING - (SHOW_HIDE_CTRL_WIDTH/2) - SHOW_HIDE_CTRL_PADDING;
		if (this._prevWidth == null) {
			if (x < LEFT_NAV_SNAP_WIDTH) 
				this._prevWidth = 0;
			else 
				this._prevWidth = dojo.coords(this._leftNav).w;
		}
		var width = this._prevWidth;
		if (x < LEFT_NAV_SNAP_WIDTH) {
			if (!this._snapped) {
				// snap closed
				width = 0;
				var lwidth = dojo.coords(this._leftNav).w;
				this._leftNav.style.left = -lwidth + "px";
				this._leftNav.style.marginRight = -lwidth + "px";
				this._leftNav.style.visibility = "hidden";
				this._currentLeftNavState = "close";
				switchClass(this._showHideButton, "showHideButtonExpanded", "showHideButtonCollapsed");
				dojo.addClass(this.domNode,"nav-collapsed");
				this._leftNavWidth = LEFT_NAV_MIN_WIDTH;
				this._snapped = true;				
			}
		} else {
			this._leftNav.style.visibility = "";
			this._snapped = false;
			this._currentLeftNavState = "open";
			switchClass(this._showHideButton, "showHideButtonCollapsed", "showHideButtonExpanded");
			dojo.removeClass(this.domNode,"nav-collapsed");
			if (x < LEFT_NAV_MIN_WIDTH) {
				// prevent further shrinking
				width = LEFT_NAV_MIN_WIDTH;
				
				this._leftNav.style.left = "0px";
				this._leftNav.style.marginRight = "0px";
			} else if (x > LEFT_NAV_MAX_WIDTH) {
				// prevent further expansion
				width = LEFT_NAV_MAX_WIDTH;
				
				this._leftNav.style.left = "0px";
				this._leftNav.style.marginRight = "0px";
			} else {
				// slide with mouse
				width = x;
				
				if (this._leftNav.style.left != "0px") {
					this._leftNav.style.left = "0px";
				}
				if (this._leftNav.style.marginRight != "0px") {
					this._leftNav.style.marginRight = "0px";
				}
			}
		}		
		if (width != 0) {
			this._leftNav.style.width = width + "px";
		}
		if (this._prevWidth != width) {
			this._contentWrapper.style.width = dojo.coords(this._contentWrapper).w + this._prevWidth - width + "px";
		}
		this._prevWidth = width;
	}  
});


var LEFT_NAV_SNAP_WIDTH = 80;
var LEFT_NAV_MIN_WIDTH = 198;
var LEFT_NAV_MAX_WIDTH = 420;
var LEFT_NAV_DEFAULT_WIDTH = 198;
var SHOW_HIDE_CTRL_WIDTH = 6;
var SHOW_HIDE_CTRL_PADDING = 4;
var LEFT_NAV_PADDING = 5;

//copied from jazz
function switchClass(/* Node */ node, /* string */ oldClass, /* string */ newClass) {
	if (!node) return;
	// remove old class if present
	var orig = node.className;
	var cls = orig;
	var t = dojo.trim((" " + cls + " ").replace(" " + oldClass + " ", " "));
	if(cls != t){ cls = t; }

	// add new class if present
	if((" "+cls+" ").indexOf(" "+newClass+" ") < 0){
		cls += (cls ? ' ' : '') + newClass;
	}
	
	// apply change if needed
	if (cls != orig) {
		node.className = cls;
	}
}

})();
