/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.reports                                        |
 *|                                                                        |
 *| Copyright IBM Corporation 2011, 2012.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
dojo.provide("dm.app.reports.ui.internal.pages.TemplateBrowser");

/** Editor for browsing report templates. Report templates can
 *  either be personal or shared. Each template has a description,
 *  and an associated RPE template.
 *  
 *  @author Kevin Cornell
 */

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("dm.app.reports.ui.internal.pages.TemplateBrowserContent");

dojo.require("dm.app.reports.ui.internal.application.ReportActions");
dojo.require("dm.app.reports.ui.internal.util.Icons");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.requireLocalization("dm.app.reports.ui.internal", "ReportMessages");
var messages  = dojo.i18n.getLocalization("dm.app.reports.ui.internal", "ReportMessages");

var ReportActions = dm.app.reports.ui.internal.application.ReportActions;
var Icons = dm.app.reports.ui.internal.util.Icons;

// Define the editor help UA key
var PAGE_UAKEY = "com.ibm.xtools.rmps.web.reports.page_TemplateBrowser001";


dojo.declare("dm.app.reports.ui.internal.pages.TemplateBrowser", rmps.app.ui.internal.editors.BaseEditor, {
	
	"class": "reports-TemplateBrowser",
	
	// base editor properties
	isRefreshable: true,
	isReadOnly: true,
	
	// Define properties used by this editor
	_pageContent: null,
	_doRefreshOnActivate: false,
	
	createContentWidget: function() {
		// Specify the help for this editor
		this.uaKey = PAGE_UAKEY;
		
		// Create the contents for the browser
		this._pageContent = new dm.app.reports.ui.internal.pages.TemplateBrowserContent({editor: this, inBrowser: true });
		return this._pageContent;
	},
	
	/** Add the create template and deploy temapltes buttons.
	 * 
	 * @param toolbar: the base editor page toolbar.
	 */
	addButtons: function(toolbar) {
		toolbar.addButton(this, this._createNewTemplate, {imgSrc: Icons.CREATE_TEMPLATE, label: messages.createNewTemplate, showLabel: true});
		toolbar.addButton(this, this._deployPredefinedTemplates, {imgSrc: Icons.TEMPLATE_FIND, label: messages.deployPredefinedTemplates});		
		this.inherited(arguments);
	},

	/** Create a new report template (descriptor) by uploading
	 *  an RPE template document (.dta). Template creation is
	 *  performed by the DM template edit page.
	 */
	_createNewTemplate: function() {
		window.location.href = ReportActions.getActionUri(ReportActions.ACTION_EDIT_TEMPLATE);
	},

	/** When the editor is activated, do a refresh.
	 * 
	 */
	activate: function() {
		this.inherited(arguments);
		if (this._doRefreshOnActivate) {
			this._pageContent.refresh();
		}
	},
	
	deactivate: function() {
		this._doRefreshOnActivate = true;
	},
	
	/** Find any predefined templates that have not already been
	 *  deployed to the selected project area. Display the 
	 *  corresponding DM reports page.
	 */
	_deployPredefinedTemplates: function() {
		// Load the deployPredefinedTemplates page for this project area
		window.location.href = ReportActions.getActionUri(ReportActions.ACTION_DEPLOY_TEMPLATES) + 
			"&name=" + encodeURIComponent(messages.deployPredefinedTemplates);
	},
		
	/** Report an error from a reporting service request.
	 * 
	 * @param errorMessage the main error message
	 * @param error the service error (details for error message)
	 */
	serviceError: function(errorMessage, error) {
		var details = null;
		if (error.message) {
			details = [ {message: error.message} ];
		} else if (error.errorClass) {
			details = [ {message: error.errorClass} ];
		}
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: errorMessage,
			details: details,
			closable: true
		});
	}

});

})();
