/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.reports                                        |
 *|                                                                        |
 *| Copyright IBM Corporation 2011, 2013.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
dojo.provide("dm.app.reports.ui.internal.pages.TemplateBrowserContent");

/** Define the widget that is used for browsing report templates. 
 *  Report templates can either be personal or shared. Each template 
 *  has a description and a shared indicator.
 *  
 *  This widget is used in multiple editors. In the template browser
 *  editor (Report Templates), the entries for the templates will have
 *  associated actions to edit or delete the template. When this widget
 *  is used in a create report page, there are no actions defined for a 
 *  template entry but instead the table allows a single selection.
  *  
 *  @author Kevin Cornell
 */

dojo.require("dm.app.reports.ui.internal.application.ReportActions");
dojo.require("dm.app.reports.ui.internal.util.ReportServiceClient");
dojo.require("dm.app.reports.ui.internal.util.ReportUtil");
dojo.require("dm.app.reports.ui.internal.util.Icons");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
var session = com.ibm.team.repository.web.client.session;

dojo.require("jazz.ui.FilterBox");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("dijit.form.Select");
dojo.require("jazz.util.html");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("rmps.ui.util.ToolbarFocusUtil");
var ToolbarFocusUtil = rmps.ui.util.ToolbarFocusUtil;

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.requireLocalization("dm.app.reports.ui.internal", "ReportMessages");
var messages  = dojo.i18n.getLocalization("dm.app.reports.ui.internal", "ReportMessages");

var ReportActions = dm.app.reports.ui.internal.application.ReportActions;
var ReportServiceClient = dm.app.reports.ui.internal.util.ReportServiceClient;
var ReportUtil = dm.app.reports.ui.internal.util.ReportUtil;
var Icons = dm.app.reports.ui.internal.util.Icons;

var HTML = jazz.util.html;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var FilterBox = jazz.ui.FilterBox;
var StyledTable = jazz.ui.StyledTable;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;

var TEMPLATE_ACTIONS_COLUMN = "templateActionsColumn";
var TEMPLATE_ACTIONS = "templateActions";

var COMBO_PRIVATE = "private";
var COMBO_SHARED  = "shared";
var COMBO_ALL     = "all";

dojo.declare("dm.app.reports.ui.internal.pages.TemplateBrowserContent", [dijit._Widget, dijit._Templated], {

	"class": "templateBrowser",

	messages: messages, 
	
	widgetsInTemplate: true,
	templateString: 
		'<div>' +
			'<div class="filterArea"  dojoAttachPoint="_filterArea"></div>' +
			'<div class="resultsArea" dojoAttachPoint="_resultsArea"></div>' +
		'</div>',
	
	// init properties
	editor: null,
	inBrowser: null,
	
	// Page controls
	_templateOwnerSelect: null,
	_filterBox: null,
	_table: null,
	_tableListeners: null,
	
	// Remember the template selected (when not in the template browser)
	_selectedTemplateUUID: null,
	_templateIdToRow: null,
	
	// Keep information about performing the requests
	_currentRequestId: 0,
	_refreshTimeout: null,
	
	// Table results filtering options
	_ownerFilter: COMBO_ALL,
	_nameFilter: null,
	
	// Keep information for pending operations
	// on a given template
	_pendingTemplate: null,
	
	constructor: function(args) {
		this.messages = messages;
		this._templateIdToRow = [];
	},

	
	postCreate: function() {
		this.inherited(arguments);

		// Create the template selection combobox
		this._templateOwnerSelect = new dijit.form.Select( {
			name: "templateOwnerSelect",
			onChange: dojo.hitch(this, this._ownerFilterChanged),
			options: [ {value: COMBO_PRIVATE, label: messages.templateComboPrivate},
			           {value: COMBO_SHARED,  label: messages.templateComboShared},
			           {value: COMBO_ALL,     label: messages.templateComboAll, selected: "selected"} ]
			});
		this._filterArea.appendChild(this._templateOwnerSelect.domNode);
		
		// Create the template name filtering box
		this._filterBox = new FilterBox({onChange: dojo.hitch(this, this._nameFilterChanged)});
		this._filterArea.appendChild(this._filterBox.domNode);
		
		// Create the results table (when in the browser, add the "actions" column)
		if (this.inBrowser) {
			this._table = new StyledTable(
					{columns: [{label: this.messages.templateName, width: "25%", sortable: false}, 
					           {label: this.messages.templateDescription, width: "60%", sortable: false}, 
					           {label: this.messages.actions, width: "15%", sortable: false}], 
				trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS, showHover: true});
		} else {
			// When displayed in the create report editor, hide the table header
			this._table = new StyledTable(
					{columns: [{label: "", width: "30%", sortable: false}, 
					           {label: "", width: "70%", sortable: false}], 
				trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS, showHover: true});
		}
		this._resultsArea.appendChild(this._table.domNode);
		this._table.startup();
		this._tableListeners = [];
		
		// Refresh the contents of this page
		this.refresh();
	},
	
	/** Select a template in the templates browser.
	 *  
	 *  @param template the selected template or null
	 */	
	selectTemplate: function(template) {
		// Determine the tree items for the new template and the selected template
		var templateRow = (template ? this._templateIdToRow[template.reportUUID] : null);
		var selectedRow = (this._selectedTemplateUUID ? this._templateIdToRow[this._selectedTemplateUUID] : null);
		
		// If nothing was selected or a different template was selected
		if (!template || template.reportUUID !== this._selectedTemplateUUID) {
			// If a template was selected, un-select it
			if (selectedRow) {
				this._highlightRow(selectedRow, ReportUtil.COLOR_UNSELECT);
				this._selectedTemplateUUID = null;
			}
			
			// If a new template is being selected
			if (templateRow) {
				// Select the new template and highlight it
				this._selectedTemplateUUID = template.reportUUID;
				this._highlightRow(templateRow, ReportUtil.COLOR_SELECTED);
			}
		} 
	},
	
	/** Destroying the browser requires the table to be deleted.
	 */
	destroy: function() {
		if (this._table) {
			this._clearTable();
			this._table.destroy();
			this._table = null;
			this._tableListeners = null;
		}
		this.inherited(arguments);
	},
	
	/** Clear the contents (rows) of the table. 
	 *  When each entry toolbar is destroyed, the corresponding 
	 *  focus listeners will be removed.
	 */
	_clearTable: function() {
		this._templateIdToRow = [];
		if (this._table) {
			// Delete any table listeners 
			dojo.forEach(this._tableListeners, function(listener, i) { dojo.disconnect(listener); });
			this._tableListeners = [];
			
			// Destroy all toolbars for the table rows so that listeners are removed.
			if (this._table.tbody) {
				var tbody = this._table.tbody;
				while (tbody.hasChildNodes()) {
					if (tbody.firstChild._actionToolbar) {
						tbody.firstChild._actionToolbar.destroy();
					}
					tbody.removeChild(tbody.firstChild);
				}
			}			
		}
	},
	
	/** Refresh the templates shown in the browser.
	 * 
	 *  Find all report templates (descriptors) that
	 *  are associated with this project area.
	 */
	refresh: function() {
		if (this.inBrowser) {
			PlatformUI.getWorkbench().showMessage(this.messages.searchingReportTemplates);
		}
		if (this._refreshTimeout) {
			clearTimeout(this._refreshTimeout);
			delete this._refreshTimeout;
		}

		this._currentRequestId++;
		var serviceObject = {
			self: this,
			id: this._currentRequestId,
			success: function(reportTemplates) {
				if (this.self._currentRequestId > this.id) {
					return;
				}
				this.self._loadTemplates(reportTemplates);
				if (this.self.inBrowser) {
					PlatformUI.getWorkbench().hideMessageArea();
				}
			},
				
			failure: function(error) {
				this.self.editor.serviceError(messages.errorLoadingTemplates, error);
			}
		};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		var args = {
			sortByName: true
		};
		if (this._nameFilter) {
			args.filter = this._nameFilter;
		}
		if (this._ownerFilter === COMBO_PRIVATE){
			args.flags = ReportServiceClient.EXCLUDE_MICRO | ReportServiceClient.PRIVATE;
			args.ownerUUID = session.getAuthenticatedContributor().itemId;
		} else if (this._ownerFilter === COMBO_SHARED) {
			args.flags = ReportServiceClient.EXCLUDE_MICRO | ReportServiceClient.SHARED;
		}
		ReportServiceClient.getReportDescriptors(srh, args);
	},
	
	/** Load the report descriptors (templates) and add entries to
	 *  the template table.
	 *  
	 *  @param reportTemplates: an array of report descriptors 
	 */
	_loadTemplates: function(reportTemplates) {
		// Delete any cached data and clear the table contents
		this._clearTable();
		
		// For each template found, create a row in the table.
		var tableHasTemplates = false;
		if (reportTemplates !== null && reportTemplates.length > 0) {
			var length = reportTemplates.length;
			for (var i = 0; i < length; i++) {
				var template = reportTemplates[i];
				var row = this._createTemplateRow(template);
				this._table.tbody.appendChild(row);
				this._templateIdToRow[template.reportUUID] = row;
				tableHasTemplates = true;
				
				// If this template was previously selected, re-select it in the refreshed table
				if (this._selectedTemplateUUID === template.reportUUID) {
					this._selectedTemplateUUID = null;
					this.selectTemplate(template);
				}
			}
		}
		if (!tableHasTemplates) {
			this._table.tbody.appendChild(this._createNoneMessage());
		}
	},
	
	/** Create a row in the template table for a new entry.
	 *  Each entry in the table is a link to the corresponding
	 *  template edit page.
	 * 
	 * @param template: a report template (descriptor)
	 */
	_createTemplateRow: function(template) {
		var row = document.createElement("tr");
		row.reportUUID = template.reportUUID;
		dojo.style(row, "backgroundColor", ReportUtil.COLOR_NORMAL);	
		
		// Column 1: the template name 
		var td  = document.createElement("td");
		td.className = "templatetName";
		
		// If the content is being displayed in the Template Browser
		// the template name is a link (<a>) to the template editor page. 
		if (this.inBrowser) {
			var editActionUri = ReportActions.getActionUri(ReportActions.ACTION_EDIT_TEMPLATE) +
			"&reportUUID=" + template.reportUUID + "&name=" + encodeURIComponent(template.name);
			td.innerHTML = "<td><img style='margin-right: 5px;' src='" + Icons.TEMPLATE + "'><a href='" + editActionUri + "'>" + HTML.escape(template.name) + "</a>";
		} 
		
		// else the content is being displayed in a non-browser editor (e.g., create report)
		// In this scenario the template name will not accept focus but in order to allow
		// the row to be selected, it is forced to accept focus (td.tabIndex = 0).
		else {
			td.innerHTML = "<td><img style='margin-right: 5px;' src='" + Icons.TEMPLATE + "'>" + HTML.escape(template.name);
			td.tabIndex = 0;
			
			// Add listeners to handle the selection of this template (row). When using the
			// mouse look for a "click" and for keyboard traversal, look for the Enter key.
			this._tableListeners.push(dojo.connect(row, "onclick", this, function() {
			    this._rowSelected(template); 
			}));
			this._tableListeners.push(dojo.connect(td, "onkeydown", this, function(event) {
				var k = dojo.keys;
				if (event.keyCode === k.ENTER) {
					this._rowSelected(template);
				}
			}));
		}
		row.appendChild(td);
		
		// Column 2: the template description
		td = document.createElement("td");
		td.className = "templateDescription";
		td.innerHTML = HTML.escape(template.description);
		row.appendChild(td);
		
		// Column 3: Action buttons to edit the template and to delete the template
		// The action buttons are only shown when the content is being displayed in 
		// the Template Browser.
		// Remember the action toolbar in the row itself - used by deleteTable()
		row._actionToolbar = new jazz.ui.SimpleToolbar();  // ToolbarFocusUtil requires a toolbar
		if (this.inBrowser) {
			// Create a cell to contain a toolbar control with the actions
			td = document.createElement("td");
			td.className = TEMPLATE_ACTIONS_COLUMN;
			var div = document.createElement("div");
			div.className = TEMPLATE_ACTIONS;
			td.appendChild(div);
			
			// Add the "edit" action and the "delete" action.
			row._actionToolbar.addButton(this, dojo.hitch(this, "_editTemplate",   template), {imgSrc: Icons.EDIT, label: this.messages.edit});
			row._actionToolbar.addButton(this, dojo.hitch(this, "_deleteTemplate", template), {imgSrc: Icons.DELETE, label: this.messages.deleteMsg});
			div.appendChild(row._actionToolbar.domNode);
			row.appendChild(td);
		}

		// Add listeners to show and hide the toolbar actions if necessary.
		row._actionToolbar.startup();
		ToolbarFocusUtil.addToolbarFocusListeners( { 
			actionToolbar: row._actionToolbar, 
			containerNode: row, 
			actionClassName: ".jazz-ui-SimpleToolbar-button",
			focusCallback:  dojo.hitch(this, this.focusCallback)
		});
		
		return row;
	},
	
	/** This function is called when a row is selected and this widget is
	 *  not in the actual template browser.
	 */
	_rowSelected: function(template) {
		// If the template is already selected, un-select it
		if (this._selectedTemplateUUID === template.reportUUID) {
			this.selectTemplate(null);
		}
		
		// else select the corresponding template
		else {
			this.selectTemplate(template);
		}
	},
	
	/** When the elements in the row obtain the focus, the
	 *  listeners will call this function. This enables the background
	 *  color of the row to be modified.
	 *
	 * @param row the item container (e.g., containerNode in addToolbarFocusListeners())
	 * @param hasFocus true = tree item has the focus, false = item lost the focus
	 */ 
	focusCallback: function(row, hasFocus) {
		this._highlightRow( row, (hasFocus ? ReportUtil.COLOR_IN_FOCUS : ReportUtil.COLOR_NORMAL));
	},
	
	/** Highlight the row to indicate it has focus 
	 *  or is selected. When a row has focus, show
	 *  the actions if they are defined. When out
	 *  of focus, leave the actions shown until 
	 *  another row obtains the focus.
	 * 
	 * @param row the table row that is being affected
	 * @param color the new color/state for the row
	 */
	_highlightRow: function(row, color) {
		// If the row is currently selected
		if (row._isSelected) {
			// If the row is being un-selected
			// change the color back to normal,
			if (color === ReportUtil.COLOR_UNSELECT) {
				row._isSelected = false;
				color = ReportUtil.COLOR_NORMAL;
				dojo.style(row, "backgroundColor", color);
			}
		} 
		
		// else if this row is being selected
		else if (color === ReportUtil.COLOR_SELECTED) {
			// Change its color to indicate it is selected
			row._isSelected = true;
			dojo.style(row, "backgroundColor", color);
		}
		
		// else if the row has just received the focus
		else if (color === ReportUtil.COLOR_IN_FOCUS) {
			// Change the color of this row
			dojo.style(row, "backgroundColor", color);
		}
		
		// else the row has lost the focus 
		else {
			// Change the row color back to normal
			dojo.style(row, "backgroundColor", color);
		}
	},
	
	/** Edit the specified report template.
	 * 
	 * @param template: template descriptor
	 */
	_editTemplate: function(template) {
		var editActionUri = ReportActions.getActionUri(ReportActions.ACTION_EDIT_TEMPLATE);
		window.location.href = editActionUri + 
			"&reportUUID=" + template.reportUUID + "&name=" + encodeURIComponent(template.name);
	},
	
	/** Delete the specified report template.
	 * 
	 * @param template: template descriptor
	 */
	_deleteTemplate: function(template) {

		// Ensure the user wants to delete this template
		this._pendingTemplate = template;
		var substitutions = [template.name];
		var delMessage = dojo.string.substitute(this.messages.confirmDeleteTemplate, substitutions);
		if (!confirm(delMessage)) {
			return;
		}
		var args = {
				reportUUID: template.reportUUID
		};
		var responseHandler = new ServiceResponseHandler(
				this,
				"_deleteTemplateSuccess",
				"_deleteTemplateError");
		ReportServiceClient.postDeleteReportDescriptor(responseHandler, args);
	},
	
	/** Action to delete a template was successful. */
	_deleteTemplateSuccess: function() {
		// Get the template being deleted
		var template = this._pendingTemplate;
		this._pendingTemplate = null;
		
		// Un-select the deleted template
		if (this._selectedTemplateUUID === template.reportUUID) {
			this.selectTemplate(null);
		}
		
		// If there is an open editor for this template, close it.
		var action = ReportActions.ACTION_PREFIX + ReportActions.ACTION_EDIT_TEMPLATE;
		var args = {
			projectId: ReportUtil.getProjectId(),
			reportUUID: template.reportUUID
		};
		var editorInputDeferred = rmps.ui.editor.EditorRegistry.createEditorInput(action, args);
		if (editorInputDeferred) {
			editorInputDeferred.then(function(editorInput) {
				var pageHelper = rmps.ui.editor.PageHelper.getInstance(ReportActions.PAGE_ID);
				pageHelper.closeEditorInput(editorInput, true, true);
			});
		}
		
		// Output a message indicating the delete was successful
		var substitutions = [template.name];
		var statusMessage = dojo.string.substitute(this.messages.deleteTemplateSuccess, substitutions);
		this.editor.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.OK,
			summary: statusMessage,
			details: null,
			closable: true
		});
		
		// Remove the row from the table
		var rowToDelete = this._templateIdToRow[template.reportUUID];
		if (rowToDelete) {
			this._table.tbody.removeChild(rowToDelete);
		}
	},
	
	/** Action to delete a template failed. */
	_deleteTemplateError: function(error) {
		// Get the template being deleted
		var template = this._pendingTemplate;
		this._pendingTemplate = null;

		var substitutions = [template.name];
		var statusMessage = dojo.string.substitute(this.messages.errorDeletingTemplate, substitutions);
		this.editor.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: statusMessage,
			details: [ {message: this.messages['errorDeletingTemplate.explanation'] + "<br>" + this.messages['errorDeletingTemplate.useraction'] }, 
			           { message: error.message } ],
			closable: true
		});
	},
	
	/** Create a special template table row to indicate that
	 *  no templates were found for the given project area.
	 */
	_createNoneMessage: function() {
		var row = document.createElement("tr");
		var td = document.createElement("td");
		td.innerHTML = HTML.escape(this.messages.noTemplatesFound);
		row.appendChild(td);
		return row;
	},
	
	/** Perform filtering of the report templates in the table
	 *  based on the owner selection.
	 * 
	 * @param owner: the template owner selection filter
	 */
	_ownerFilterChanged: function(newOwnerFilter) {
		this._ownerFilter = newOwnerFilter;
		// set a timeout to query with the new filter
		// need to ensure 0 is passed to refresh or setTimeout will pass some other value
		if (!this._refreshTimeout) {
			this._refreshTimeout = setTimeout(dojo.hitch(this, this.refresh,0,true), 500);
		}
	},
	
	/** Perform filtering of the report templates in the table
	 *  based on the name.
	 * 
	 * @param filter: the name filter expression
	 */
	_nameFilterChanged: function(newNameFilter) {
		this._nameFilter = newNameFilter;
		// set a timeout to query with the new filter
		// need to ensure 0 is passed to refresh or setTimeout will pass some other value
		if (!this._refreshTimeout) {
			this._refreshTimeout = setTimeout(dojo.hitch(this, this.refresh,0,true), 500);
		}
	},
	
	/** Handle the resizing of the browser content.
	 *  If the content becomes larger than the 
	 *  editor page, auto-scrolling should result in the
	 *  vertical scroll bars being shown.
	 */
	resize: function(size) {
		this.inherited(arguments);
		if (!size) {
			return;
		}
		if (size.h) {
			var filterPos = dojo.position(this._filterArea);
			var resultPos = dojo.position(this._resultsArea);
			var newHeight = size.h - resultPos.y + filterPos.y;
			dojo.style(this._resultsArea,   "height", newHeight + "px");	
		}
	}

});

})();
