/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.reports                                        |
 *|                                                                        |
 *| Copyright IBM Corporation 2011, 2012.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
dojo.provide("dm.app.reports.ui.internal.pages.TemplateEditor");

/** Editor to create or edit a report template. 
 * 
 * @author Kevin Cornell
 */

dojo.require("rmps.app.ui.internal.editors.BaseEditor");
dojo.require("jazz.ui.Breadcrumbs");

dojo.require("dm.app.reports.ui.internal.application.ReportActions");
dojo.require("dm.app.reports.ui.internal.util.ReportServiceClient");
dojo.require("dm.app.reports.ui.internal.util.ResourceServiceClient");
dojo.require("dm.app.reports.ui.internal.util.ReportSimpleButton");
dojo.require("dm.app.reports.ui.internal.util.ReportUtil");
dojo.require("dm.app.reports.ui.internal.util.Icons");

dojo.require("dm.app.reports.ui.internal.pages.TemplateResourceDialog");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("dijit.form.CheckBox");
dojo.require("dijit.InlineEditBox");

dojo.require("dojo.string");
dojo.require("dojo.i18n");
dojo.requireLocalization("dm.app.reports.ui.internal", "ReportMessages");
var messages  = dojo.i18n.getLocalization("dm.app.reports.ui.internal", "ReportMessages");

var ReportActions          = dm.app.reports.ui.internal.application.ReportActions;
var ReportServiceClient    = dm.app.reports.ui.internal.util.ReportServiceClient;
var ResourceServiceClient  = dm.app.reports.ui.internal.util.ResourceServiceClient;
var ReportUtil             = dm.app.reports.ui.internal.util.ReportUtil;
var Icons                  = dm.app.reports.ui.internal.util.Icons;

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

//Define the editor help UA key
var PAGE_UAKEY = "com.ibm.xtools.rmps.web.reports.page_TemplateEditor001";

// Define the report identifier prefix for DM deployed templates
var DM_REPORT_ID_PREFIX = "dm.report.";

var DIRTY_GROUP_ID = "dm.app.reports.TemplateEditor.dirty";

// Define the fields of this editor that can be modified.
var FIELD_design         = "designResourceName";     // does not exist in saved template resource
var FIELD_disableRestore = "disableRestoreContents"; // not saved in template resource
var FIELD_oldResources   = "oldResources";			 // array - not saved in template resource but needed for POST
var FIELD_resLocation	 = "resourcesLocation";		 // array - not saved in template resource but needed for POST
var FIELD_resSize        = "resourcesSize";			 // array - not saved in template resource but needed for POST
var FIELD_resContent     = "resourcesContentType";	 // array - not saved in template resource but needed for POST
var FIELD_resChecksum    = "resourcesChecksum";		 // array - not saved in template resource but needed for POST

var FIELD_reportUUID     = "reportUUID";
var FIELD_description    = "description";
var FIELD_name           = "name";
var FIELD_processArea    = "processAreaUUID";
var FIELD_reportId       = "reportId";
var FIELD_resources      = "resources";				// array of resource names
var FIELD_shared         = "shared";
var FIELD_caching        = "supportsDataCaching";

// This editor keeps 2 copies of the template data: the current (old)
// template and the modified (new) template. When an existing template
// is read, various fields must be copied from the current template to
// the modified template and a Dojo clone cannot be performed because the
// old template has some non-primitive fields that causes the POST operation
// (which uses the modified template) to fail.
// Define the template fields that must be copied to the modified template.
var COPY_FIELDS = [
	FIELD_disableRestore,
	FIELD_reportUUID,
	FIELD_description, 
	FIELD_name, 
	FIELD_processArea, 
	FIELD_reportId, 
	FIELD_resources,
	FIELD_shared,
	FIELD_caching ];

// Define the columns of the resource table
var RES_TABLE_NAME   = "resourceNameColumn";
var RES_TABLE_STATUS = "resourceStatusColumn";
var RES_TABLE_ACTION = "resourceActionColumn";

dojo.declare("dm.app.reports.ui.internal.pages.TemplateEditor", rmps.app.ui.internal.editors.BaseEditor, {
	
	"class": "reports-TemplateEditor",
	
	// base editor properties
	isRefreshable: false,
	isReadOnly: false,
		
	// Define the report ID and the current/modified template data
	_reportUUID:  null,
	_oldTemplate: null,
	_newTemplate: null,
	
	// Keep a handle to the page contents
	_pageContent: null,
	_breadCrumbs: null,
	_closeButton: null,
	
	// Keep a flag to indicate whether any dialogs have been displayed
	// When a dialog is displayed, the browser history contains an extra
	// URL to the editor and BACK, after the editor is closed, results 
	// in a blank page.
	_dialogDisplayed: false,

	postCreate: function() {	
		// Add the "Report Templates" bread crumb at the top of the editor
		this._breadCrumbs = new jazz.ui.Breadcrumbs({segments: [{label: messages.templateBrowserName, 
																 href: ReportActions.getActionUri(ReportActions.ACTION_VIEW_TEMPLATES)}]});
		dojo.place(this._breadCrumbs.domNode, this.headerContainerNode, "first");

		this.inherited(arguments);
	},

	/** Create the content widget for the report template editor.
	 * 
	 * @return the widget that displays the content
	 */
	createContentWidget: function() {
		// Specify the help for this editor
		this.uaKey = PAGE_UAKEY;
		
		// Create the widget to display the template content
		this._pageContent = new dm.app.reports.ui.internal.pages.TemplateEditor._Content({editor: this});
		return this._pageContent;
	},
	
	/** Indicate that the editor name can be changed. The
	 *  editor name is connected to the template name.
	 */
	isEditableTitle: function() {
		return true;
	},
	
	/** Handle changes to the editor name. Update the 
	 *  name in the template and mark the template as 
	 *  dirty.
	 *  
	 *  @param newName the new report template name
	 */
	onTitleChange: function(newName){
		var newValue = dojo.trim(newName);
		this._updateTemplateField(FIELD_name, newValue);
	},

	/** When the editor is activated, obtain the corresponding
	 *  report template and update the editor controls.
	 */
	activate: function() {
		this.inherited(arguments);
		
		// If the editor is being opened for the first time
		// for this report template
		if (!this.isDirty()) {
			// If the editor being opened is for a template that was just created
			if (this.getEditorInput().getParams().created) {
				// Redisplay the saved message.
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.OK,
					summary: messages.saveSuccessful,
					details: null,
					closable: true
				});
			}
			
			// The editor has not displayed any dialogs yet
			this._dialogDisplayed = false;
			
			// Remember if an existing template is being edited
			var params = this.getEditorInput().getParams();
			this._reportUUID = params.reportUUID;
			
			// If an existing template was specified (by reportUUID parameter)
			if (this._reportUUID) {
				// Issue a request to load the report template.
				var serviceObject = {
						self: this,
						success: function(template) {
							this.self._loadReportTemplate(template);
						},
								
						failure: function(error) {
							this.self.serviceError(messages.errorLoadingTemplates, error);
						}
					};
				var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
				ReportServiceClient.getReportDescriptor(srh, {reportUUID: this._reportUUID});
			} 
			
			// else create a new (empty) template
			else {
				this._loadReportTemplate(null);
			}
		}
		
	},
	
	/** Add the cancel button. Save button is added because "isReadOnly" is false.
	 * 
	 * @param toolbar: the base editor page toolbar.
	 */
	addButtons: function(toolbar) {
		this._closeButton = new dm.app.reports.ui.internal.util.ReportSimpleButton({
			label: messages.close,
			onClick: dojo.hitch(this, this._handleCancel)
		});
		toolbar.addWidget(this._closeButton);
		
		this.inherited(arguments);
	},
	
	_updateCloseButtonLabel: function() {
		var newLabel = this.isDirty() ? messages.cancel : messages.close;
		this._closeButton.setLabel(newLabel);
	},
	
	/** Perform the saving of the report template.
	 * 
	 */
	doSave: function(taskMonitor) {
		this.inherited(arguments);
		
		// At this point the modified template should contain
		// all of the data needed to save the report template.
		var serviceObject = {
				self: this,
				success: function(templateUUID) {
					this.self._saveComplete(templateUUID);
				},
						
				failure: function(error) {
					this.self.serviceError(messages.errorSavingTemplate, error);
				}
			};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		ReportServiceClient.postSaveReportDescriptor(srh, this._newTemplate);
		
		// The TaskMonitor cannot be used to handled the deferred work from the
		// request because it overwrites the specific error message & details
		// with a generic "Bad Request" message that means nothing.
	},
	
	/** The saving of the report template was successful.
	 * 
	 * @param returnVal the value returned from the POST request
	 * 
	 */
	_saveComplete: function(returnVal) {
		// Remove the dirty indicator
		this.removeDirty(null, DIRTY_GROUP_ID);
		this._updateCloseButtonLabel();
		
		// If a new template was just saved
		if (!this._reportUUID) {
			// Set the ID of the report template just saved
			this._reportUUID = returnVal;

			// Close this editor and remove it from the recently and unsaved editor lists.
			// Another editor will be opened with a different ID.
			var pageHelper = rmps.ui.editor.PageHelper.getInstance(ReportActions.PAGE_ID);
			pageHelper.closeEditor(this, true, true);
			
			// Force the editor to be opened for the template just created.
			// Also define the "created" flag to indicate a editor replacement.
			var name = this._newTemplate[FIELD_name];
			var href = ReportActions.getActionUri(ReportActions.ACTION_EDIT_TEMPLATE) +
				"&reportUUID=" + this._reportUUID + 
				"&name=" + encodeURIComponent(name) +
				"&created=true";
			window.location.href = href;
		}		
	},

	/** When Cancel is clicked in the report template editor, go
	 *  back to the page for viewing all templates.
	 */
	_handleCancel: function() {
		// Remove the dirty indicator
		this.removeDirty(null, DIRTY_GROUP_ID);
		
		// Close the editor and remove it from the recently and unsaved editor lists.
		var pageHelper = rmps.ui.editor.PageHelper.getInstance(ReportActions.PAGE_ID);
		pageHelper.closeEditor(this, true, true);

		// If the editor being canceled was for a template that was just created
		// of a dialog has been displayed by the editor
		if (this.getEditorInput().getParams().created || this._dialogDisplayed) {
			// The previous URL is either the editor to create the template
			// or one of the upload dialogs in which case the BACK button
			// will result in an empty page. So, go the the template browser.
			window.location.href = ReportActions.getActionUri(ReportActions.ACTION_VIEW_TEMPLATES);
		} else {
			// Go back to the previous page
			window.history.back();
		}
	},

	/** Initialize the page with the data from an existing 
	 *  report template. If no template is specified the
	 *  page is set accordingly.
	 *  
	 *  @param template the existing template (or null for a new template)
	 */
	_loadReportTemplate: function(template) {
		// If a template was found, use it
		if (template !== null) {
			this._oldTemplate = template;

			// If the description field is not set, force it to be the empty string.
			// If a template is saved with an empty string as the description, when
			// it is subsequently read, the description field is null, which will cause
			// the save request to fail (invalid request parameter).
			this._oldTemplate[FIELD_description] = this._oldTemplate[FIELD_description] || "";
			
			// Determine if the template was a deployed template (check its identifier)
			var deployedTemplate = template[FIELD_reportId].indexOf(DM_REPORT_ID_PREFIX) === 0;
			this._oldTemplate.disableRestoreContents = !deployedTemplate;
			
			// Ensure the "resources" array is defined in the template
			if (!this._oldTemplate[FIELD_resources]) {
				this._oldTemplate[FIELD_resources] = [];
			}
			
			// Ensure caching is disabled in the old template
			this._oldTemplate[FIELD_caching] = false;
		}
		
		// else create a new template and initialize it
		else {
			this._oldTemplate = {};
			this._oldTemplate[FIELD_processArea] = ReportUtil.getProjectId();
			this._oldTemplate[FIELD_name] = "";
			this._oldTemplate[FIELD_shared] = false;
			this._oldTemplate[FIELD_caching] = false;
			this._oldTemplate[FIELD_disableRestore] = true;
			this._oldTemplate[FIELD_resources] = [];
		}
		
		// Create a copy of the template for modifications
		// Check explicitly for defined fields or else "false" boolean
		// fields will not get copied.
		this._newTemplate = {};
		for (var index = 0, len = COPY_FIELDS.length; index < len; ++index) {
			var fieldName = COPY_FIELDS[index];
			if (this._oldTemplate[fieldName] !== undefined) {
				var fieldValue = this._oldTemplate[fieldName];
				if (dojo.isArray(fieldValue)) {
					this._newTemplate[fieldName] = new Array(fieldValue.length);
					for (var j=0; j < fieldValue.length; j++) {
						this._newTemplate[fieldName][j] = fieldValue[j];
					}
				} else {
					this._newTemplate[fieldName] = fieldValue;
				}
			}
		}
		
		// On the POST, a copy of the "resources" array is needed in the "oldResources"
		// field so that the reporting framework can determine which resources were modified.
		// This also requires several other "resourcesXxx" fields to save the uploaded file
		// data for new resources or modified resources.
		var numResources = this._oldTemplate[FIELD_resources] ? this._oldTemplate[FIELD_resources].length : 0;
		this._newTemplate[FIELD_oldResources] = new Array(numResources);
		this._newTemplate[FIELD_resLocation]  = new Array(numResources);
		this._newTemplate[FIELD_resSize]      = new Array(numResources);
		this._newTemplate[FIELD_resContent]   = new Array(numResources);
		this._newTemplate[FIELD_resChecksum]  = new Array(numResources);
		for (index = 0; index < numResources; ++index) {
			this._newTemplate[FIELD_oldResources][index] = this._oldTemplate[FIELD_resources][index];
			this._newTemplate[FIELD_resLocation][index]  = "";
			this._newTemplate[FIELD_resSize][index]      = 0;
			this._newTemplate[FIELD_resContent][index]   = "";
			this._newTemplate[FIELD_resChecksum][index]  = 0;
		}
		
		// The modified template data is not dirty yet (same as current data).
		this.removeDirty(null, DIRTY_GROUP_ID);
		this._updateCloseButtonLabel();
		
		// Update the page UI with the existing template data
		this._pageContent.initTemplateFields(this._oldTemplate);
	},

	/** Handle the "submit" action associated with the
	 *  file input element for the template content. Also
	 *  handle manual changes to the form edit control.
	 *  
	 *  @param contentsWidget the widget where the content is loaded
	 */
	_handleUploadTemplate: function(contentsWidget) {
		// The file selection dialog must have been displayed to get here.
		this._dialogDisplayed = true;
		
		// If the widget has a value (a file location) defined
		if (contentsWidget.value !== "") {
			// Determine the extension of the template file
			var fileName = this._extractFilename(contentsWidget.value, false);
			var splitFileName = fileName.split(".");
			var fileExt = splitFileName[splitFileName.length - 1];
			fileExt = fileExt.toLowerCase();
			
			// If the extension is not .dta
			var doUpload = (fileExt === "dta");
			if (!doUpload) {
				// Prompt the user to confirm the template upload
				var message = dojo.string.substitute(messages.errorInvalidTemplate, [ fileName ]);
				if (confirm(message)) {
					doUpload = true;
				} else {
					contentsWidget.value = "";
				}
			}
			
			// If the template upload is to proceed
			if (doUpload) {
				// Issue a request to upload the template
				var srh = new ServiceResponseHandler(
						this,
						"_uploadTemplateSuccess",
						"_uploadError");
				ResourceServiceClient.uploadResource(this, contentsWidget, srh);
			}
		}
	},
	
	/** Handle the successful uploading of the RPE
	 *  template resource.
	 *  
	 *  @param resourceData the resource data from the upload
	 */
	_uploadTemplateSuccess: function(resourceData) {
		// Update the modified template with the uploaded resource data
		// These "design" fields are process during the POST of a template
		// but are not defined in a subsequent GET, so field constants are
		// not defined in this editor. All of these fields are represented
		// by the FIELD_design constant.
		this._newTemplate.designLocation      = resourceData.resourceLocation;
		this._newTemplate.designSize          = resourceData.resourceSize;
		this._newTemplate.designContentType   = resourceData.resourceContentType;
		this._newTemplate.designChecksum      = resourceData.resourceChecksum;
		this._newTemplate.designFileExtension = resourceData.resourceFileExtension;
		
		// If the template content was properly uploaded
		if (this._newTemplate.designLocation) {
			// Update the field storing the design resource name to indicate editor dirty
			this._updateTemplateField(FIELD_design, resourceData.resourceName);
			
			// Update the description and identifier 
			// for the new template, if not already set.
			var name = this._extractFilename(resourceData.resourceName, true);
			var oldDescription = this._newTemplate[FIELD_description];
			if (!oldDescription) {
				var newDescription = dojo.string.substitute(messages.templateDescriptionText, [name]);
				this._updateTemplateField(FIELD_description, newDescription);
			}

			var oldId = this._newTemplate[FIELD_reportId];
			if (!oldId) {
				// Convert the name of a template to an ID that
				// does not contain spaces.
				var tokens = name.split(" ");
				var buffer = [];
				for (var i = 0; i < tokens.length; i++) {
					var token = tokens[i];
					var c = token[0];
					if (c.toUpperCase() !== c) {
						c = c.toUpperCase();
						token = c + token.substring(1);
					}
					buffer.push(token);
				}
				var newId = "report." + (buffer.join(""));
				this._updateTemplateField(FIELD_reportId, newId);
			}
			
			// Update the page UI with the modified template data
			this._pageContent.initTemplateFields(this._newTemplate);
			
			// Output a message indicating the upload success
			name = this._extractFilename(resourceData.resourceName, false);
			var message = dojo.string.substitute(messages.successUploadingFile, [ name ]);
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.OK,
				summary: message,
				details: null,
				closable: true
			});

		}

	},
	
	/** Handle an error for the uploading of a template or
	 *  the uploading of an associated resource. The callback
	 *  data defines the name of the resource being uploaded 
	 *  and the corresponding detailed error message.
	 *  
	 *  @param resourceData defines the fields "resourceName" and "message"
	 */
	_uploadError: function(resourceData) {
		var name = this._extractFilename(resourceData.resourceName, false);
		var message = dojo.string.substitute(messages.errorUploadingFile, [ name ]);

		// Use the given error message as the details
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: message,
			details: [ {message: resourceData.message} ],
			closable: true
		});
	},
	
	/** Handle the button to restore the initial
	 *  contents of the report template. This 
	 *  functionality will only work for deployed
	 *  templates.
	 */
	_handleRestoreContents: function() {
		if (!this._oldTemplate || !this._oldTemplate[FIELD_reportUUID]) {
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: messages.errorRestoringInvalid,
				details: null,
				closable: true
			});
			return;
		}
		
		// Issue a request to restore the template contents (only works for deployed templates)
		var serviceObject = {
				self: this,
				success: function(retrunVal) {
					this.self._contentsRestored();
				},
						
				failure: function(error) {
					this.self.serviceError(messages.errorRestoringContent, error);
				}
			};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		var args = {
			projectAreaUUID: ReportUtil.getProjectId(),
			reportUUID: this._oldTemplate[FIELD_reportUUID]
		};
		ReportServiceClient.postResetReportDescriptor(srh, args);
	},

	/** This method is called when request to restore
	 *  the contents of a template has completed
	 *  successfully.
	 */
	_contentsRestored: function() {
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.OK,
			summary: messages.contentsRestored,
			details: null,
			closable: true
		});
		
		// Mark the template design as being dirty.
		this.addDirty(FIELD_design, DIRTY_GROUP_ID);
		this._updateCloseButtonLabel();
	},

	/** Update the report template field with the new value.
	 *  If the new value differs from the current value then
	 *  update the dirty indicator for this field.
	 *  
	 *  @param fieldName the name of the template field to update
	 *  @param newValue  the new value for the template field
	 */
	_updateTemplateField: function(fieldName, newValue) {
		// If the template has been created or initialized
		if (this._oldTemplate) {
			// Get the current value of the corresponding field
			var oldValue = "";
			if (this._oldTemplate[fieldName] !== null) {
				oldValue = this._oldTemplate[fieldName];
			}
			
			// Determine if the current and new values are different
			// For an array, examine all entries and stop if one differs
			var isEqual = oldValue === newValue;
			if (!isEqual && dojo.isArray(newValue) && oldValue && newValue && oldValue.length === newValue.length) {
				isEqual = true; // assume empty arrays are equal
				isEqual = dojo.every(newValue, function(item, idx) {
					return item === oldValue[idx];
				});
			}

			// If values are equal, clear the field's dirty indicator
			// else set the field's dirty indicator
			if (isEqual) {
				this.removeDirty(fieldName, DIRTY_GROUP_ID);
			} else {
				this.addDirty(fieldName, DIRTY_GROUP_ID);
			}
			this._updateCloseButtonLabel();
			
			// Update the new copy of the template.
			this._newTemplate[fieldName] = newValue;
		}
	},
	
	
	/** Update an associated resource given the existing resource
	 *  name and its new resource data. The resource data can define
	 *  a resource name and upload information including the resource
	 *  location, size, content type and checksum.
	 *  
	 *  This method will update the resource depending on the arguments 
	 *  passed as follows;
	 *  
	 *  Add:    resourceName  = null; resourceData defines a name and upload data
	 *  Delete: resourceName != null; resourceData = null;
	 *  Rename: resourceName != null  and differs from resourceData resourceName (no upload data)
	 *  Update: resourceName != null; resourceData contains upload data (name is ignored)
	 *  
	 *  @param resourceName the name of the resource entry
	 *  @param resourceData the new resource data defining the modification
	 */
	_updateResource: function(resourceName, resourceData) {
		// Find the index of the resource if it exists
		var modified = false;
		var index = -1;
		if (resourceName) {
			index = dojo.indexOf(this._newTemplate[FIELD_resources], resourceName);
		}
		
		// If the resource is to be deleted
		if (index >= 0 && resourceData === null) {
			// Remove the entry from the resources and oldResources arrays
			this._newTemplate[FIELD_resources].splice(index, 1);
			this._newTemplate[FIELD_oldResources].splice(index, 1);
				
			// Remove any upload data for this resource
			this._newTemplate[FIELD_resLocation].splice(index, 1);
			this._newTemplate[FIELD_resSize].splice(index, 1);
			this._newTemplate[FIELD_resContent].splice(index, 1);
			this._newTemplate[FIELD_resChecksum].splice(index, 1);
			modified = true;
		}
		
		// else if a new resource has been added
		else if (resourceName === null && resourceData &&
				 resourceData.resourceName && resourceData.resourceLocation) {
			// The file selection dialog must have been displayed to get here.
			this._dialogDisplayed = true;

			// Determine the name of the new resource (resource data contains file name)
			var newResourceName = this._extractFilename(resourceData.resourceName, false);
			
			// Append an entry to the resources array and a blank entry to the oldResources array
			this._newTemplate[FIELD_resources].push(newResourceName);
			this._newTemplate[FIELD_oldResources].push("");
			
			// Save the resource upload data (append to each of the resource data arrays)
			this._newTemplate[FIELD_resLocation].push(resourceData.resourceLocation);
			this._newTemplate[FIELD_resSize].push(resourceData.resourceSize);
			this._newTemplate[FIELD_resContent].push(resourceData.resourceContentType);
			this._newTemplate[FIELD_resChecksum].push(resourceData.resourceChecksum);
			
			// Output a message indicating the upload success
			var message = dojo.string.substitute(messages.successUploadingFile, [ newResourceName ]);
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.OK,
				summary: message,
				details: null,
				closable: true
			});
			modified = true;
		}
		
		// else an existing resource is being renamed
		else if (index >= 0 && resourceData && 
				resourceData.resourceName && !resourceData.resourceLocation) {
			// Determine the name of the new resource (resource data contains file name)
			var newName = this._extractFilename(resourceData.resourceName, false);
			this._newTemplate[FIELD_resources][index] = newName;
			modified = true;
		}
		
		// else new resource data was uploaded for an existing resource
		else if (index >= 0 && resourceData !== null && 
				resourceData.resourceName && resourceData.resourceLocation) {
			// The file selection dialog must have been displayed to get here.
			this._dialogDisplayed = true;

			// Set the resource upload data into the modified template
			this._newTemplate[FIELD_resLocation][index] = resourceData.resourceLocation;
			this._newTemplate[FIELD_resSize][index]     = resourceData.resourceSize;
			this._newTemplate[FIELD_resContent][index]  = resourceData.resourceContentType;
			this._newTemplate[FIELD_resChecksum][index] = resourceData.resourceChecksum;
			modified = true;
		}
		
		// Modify the editor dirty state
		if (modified) {
			this.addDirty(FIELD_resources, DIRTY_GROUP_ID);
			this._updateCloseButtonLabel();
		}
	},
	
	/*----------------------------------------*/
	
	/** Extract the filename portion of the given path.
	 *  Remove the filename extension if desired.
	 *  
	 *  @param path; the full path of some file
	 *  @param removeExtension: remove the filename extension (true or false) 
	 *  @return the file name (string)
	 */
	_extractFilename: function(path, removeExtension) {
		var splitLocation = null;
		if (path.indexOf("/") >= 0) {
			splitLocation = path.split("/");
		} else {
			splitLocation = path.split("\\");
		} 
		var fileName = splitLocation[splitLocation.length - 1];
		if (removeExtension === true) {
			var index = fileName.lastIndexOf(".");
			if (index > 0) {
				fileName = fileName.substring(0, index);
			}
		}
		return fileName;
	},
	
	/** Report an error from a reporting service request.
	 * 
	 * @param errorMessage the main error message
	 * @param error the service error (details for error message)
	 */
	serviceError: function(errorMessage, error) {
		var details = null;
		if (error.message) {
			details = [ {message: error.message} ];
		} else if (error.errorClass) {
			details = [ {message: error.errorClass} ];
		}
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: errorMessage,
			details: details,
			closable: true
		});
	}

});

/** Define the content for the report template editor.
 * 
 */
dojo.declare("dm.app.reports.ui.internal.pages.TemplateEditor._Content", [dijit._Widget, dijit._Templated], {

	"class": "templateEditor",

	messages: messages, 
	
	widgetsInTemplate: true,
	templateString: 
		'<div>' +
			// General section: description, identifier and shared checkbox
			'<div class="generalSection" dojoAttachPoint="_generalSection">' +
				'<div class="generalHeading">${messages.generalHeading}</div>' +
				'<div class="descriptionLabel">${messages.generalDescription}</div>' +
				'<div class="descriptionContainer">' +
					'<textarea class="descriptionInput" dojoAttachPoint="_descriptionInput" rows="5" ></textarea>' +
				'</div>' +
				'<div class="reportIdRow">' +
					'<div class="reportIdInfo">${messages.generalIdentifierInfo}</div>' +
					'<div class="reportIdLabel">${messages.generalIdentifier}</div>' +
					'<div class="reportIdContainer">' + 
						'<input class="reportIdInput" dojoAttachPoint="_reportIdInput" />' +
					'</div>' +
				'</div>' +
				'<div class="sharedContainer">' +
					'<input type="checkbox" dojoType="dijit.form.CheckBox" dojoAttachPoint="_sharedCheckbox">${messages.templateShared}</input>' +
				'</div>' +
			'</div>' +

			// Contents section: template upload form, restore template contents
			'<div class="contentsSection">' +
				'<div class="contentsHeading">${messages.contentsHeading}</div>' +
				'<div class="contentsInfo">${messages.contentsInfo}</div>' +
				'<div class="contentsFormContainer">' +
					'<form class="contentsForm" dojoAttachPoint="_uploadTemplate" enctype="multipart/form-data" method="post">' +
						'<input class="contentsFileInput" dojoAttachPoint="fileLocation" type="file" name="fileLocation" style="margin-bottom:1em" size="40" tabindex="0"/>' +
					'</form>' +
				'</div>' +
				'<div class="contentsRestoreLabel">${messages.contentsRestoreInfo}</div>' +
				'<button class="contentsRestoreButton" dojoAttachPoint="_restoreContents" >${messages.contentsRestoreInitial}</button>' +
			'</div>' +

			// Resources section: toolbar to add a resource and resource table
			'<div class="resourcesSection">' +
				'<div class="resourcesHeading">${messages.resourcesHeading}</div>' +
				'<div class="resourcesToolBar">' +
					'<a class="addIcon" dojoAttachPoint="_addResourceAction" title="${messages.resourceAdd}" href="#">' +
						'<img dojoAttachPoint="_addIcon" alt="${messages.resourceAdd}"></img>' +
					'</a>${messages.resourcesInfo}</div>' +
//					'<div class="resourcesInfo">${messages.resourcesInfo}</div>' +
//				'</div>' +
				'<div class="resourcesTable" dojoAttachPoint="_resourcesTable"></div>' +
			'</div>' +
		'</div>',
	
	// init properties
	editor: null,
	
	_resTable: null,
	_editableResources: null,
	
	constructor: function(args) {
		this.messages = messages;
	},

	
	postCreate: function() {
		this.inherited(arguments);
		
		// Hook in methods to handle changes in various input fields
		if (!this.editor.isReadOnly) {
			// Template description
			ReportUtil.connectTextInput(this._descriptionInput, this, function() {
				var newValue = this._descriptionInput.value;
				this.editor._updateTemplateField(FIELD_description, newValue);
			});

			// Template identifier
			ReportUtil.connectTextInput(this._reportIdInput, this, function() {
				var newValue = dojo.trim(this._reportIdInput.value);
				this.editor._updateTemplateField(FIELD_reportId, newValue);
			});

			// Template shared checkbox
			this.connect(this._sharedCheckbox, "onClick", function() {
				var newValue = this._sharedCheckbox.checked;
				this.editor._updateTemplateField(FIELD_shared, newValue);
			});

			// Template content edit form and Browse button
			this.connect(this._uploadTemplate, "onsubmit", function() {
				this._uploadTemplateFile(this.fileLocation);
			});
			this.connect(this.fileLocation, "onchange", function() {
				this._uploadTemplateFile(this.fileLocation);
			});

			// Template restore initial contents button
			this.connect(this._restoreContents, "onclick", function(event) {
				dojo.stopEvent(event);
				this.editor._handleRestoreContents();
			});

			// Resource "add" button
			this._addIcon.src = Icons.ADD;
			this.connect(this._addResourceAction, "onclick", function(event) {
				dojo.stopEvent(event);
				this._addResource();
			});
		}
		
		this._editableResources = [];
	},
	
	/** Update the fields on the page based on the given
	 *  report template.
	 *  
	 *  @param reportTemplate the report template
	 */
	initTemplateFields: function(reportTemplate) {
		
		// Set the template description, identifier, 
		// and sharing indicator.
		this._descriptionInput.value = reportTemplate[FIELD_description] ? reportTemplate[FIELD_description] : "";
		this._reportIdInput.value    = reportTemplate[FIELD_reportId]    ? reportTemplate[FIELD_reportId]    : "";
		this._sharedCheckbox.setChecked(reportTemplate[FIELD_shared]);
		
		// Enable or disable the restore contents button.
		this._restoreContents.disabled = reportTemplate.disableRestoreContents;
		
		// Create an empty table for the associated template resources
		if (this._resTable) {
			this._resTable.destroy();
			this._resTable = null;
			this._editableResources = [];
		}
		this._resTable = new jazz.ui.StyledTable(
				{columns: [{label: messages.resourceHeading, width: "75%", sortable: false, className: RES_TABLE_NAME}, 
				           {label: messages.resourceStatus, width: "10%", sortable: false, className: RES_TABLE_STATUS },
				           {label: messages.actions, width: "15%", sortable: false, className: RES_TABLE_ACTION }],
							trim: jazz.ui.styledTable.TrimStyles.GOLD,
							sortBy: 0});
		this._resourcesTable.appendChild(this._resTable.domNode);
		
		// If any resources were specified in the template
		if (reportTemplate[FIELD_resources]) {
			// Add the resources to the table
			var numResources = reportTemplate[FIELD_resources].length;
			for (var index = 0; index < numResources; index++) {
				var resourceName = reportTemplate[FIELD_resources][index];
				this._resTable.tbody.appendChild(this._createResourceEntry(resourceName));
			}
		}
		
	},
	
	/** Create an entry in the resource table.
	 * 
	 * @param resourceName the name of the resource
	 */
	_createResourceEntry: function(resourceName) {
		// Create the element for the table row
		var row = document.createElement("tr");
		
		// Create a column for the resource name
		var td  = document.createElement("td");
		var div = document.createElement("div");
		div.innerHTML = resourceName;
		td.appendChild(div);
		td.className = RES_TABLE_NAME;
		row.appendChild(td);
		
		// Column 1: Resource name with in-line edits enabled
		var self = this;
        this._editableResources[resourceName] = new dijit.InlineEditBox({title: messages.resourceClickToEdit, autoSave: true, 
			// Add the resources to the table
			width: "200px", noValueIndicator: "?", 
			onChange: dojo.hitch(this, function(newName) {
				self._renameResource(resourceName, newName, row);
			})}, div);
		this._editableResources[resourceName].startup();
		
		// Column 2: Resource modification (status) indicator
		td = document.createElement("td");
		td.className = RES_TABLE_STATUS;
		td.innerHTML = "";
		row.appendChild(td);
		
		// Column 3: Action toolbar with update and delete actions.
		td = document.createElement("td");
		td.className = RES_TABLE_ACTION;
		var toolbar = new jazz.ui.SimpleToolbar({});
		toolbar.addButton(this, dojo.hitch(this, "_uploadResource", resourceName, row), {imgSrc: Icons.UPLOAD_CONTENT, label: messages.uploadNewContent});
		toolbar.addButton(this, dojo.hitch(this, "_deleteResource", resourceName, row), {imgSrc: Icons.DELETE, label: messages.deleteMsg});
		td.appendChild(toolbar.domNode);
		
		row.appendChild(td);
		return row;
	},
	
	/** Upload the RPE template file for this report template.
	 * 
	 * @param contentsWidget the UI widget that contains the file to upload
	 */
	_uploadTemplateFile: function(contentsWidget) {
		// If edits were being made to the description or identifier
		// pressing the Browse button does not cause an event to indicate
		// those test controls have changed. Update the template with
		// their current values.
		var newValue = this._descriptionInput.value;
		this.editor._updateTemplateField(FIELD_description, newValue);
		newValue = dojo.trim(this._reportIdInput.value);
		this.editor._updateTemplateField(FIELD_reportId, newValue);
		
		// Request the editor to perform the uploading of the template
		this.editor._handleUploadTemplate(contentsWidget);
	},

	/** This method is called with the resource "+" buttons
	 *  is clicked. A new resource is to be added to the 
	 *  template.
	 */
	_addResource: function() {
		var self = this;
		var handleOk = dojo.hitch(this, function(resourceData) {
			// Add the resource name as a new entry to the resource table
			var resourceName = self.editor._extractFilename(resourceData.resourceName, false);
			var row = self._createResourceEntry(resourceName);
			self._resTable.tbody.appendChild(row);
			self._resourceModified(row);
			
			// Update the resource entries in the template
			self.editor._updateResource(null, resourceData);
		});		
		var handleError = dojo.hitch(this, function(resourceData) {
			self.editor._uploadError(resourceData);
		});		
		this.resourceDialog = new dm.app.reports.ui.internal.pages.TemplateResourceDialog({onOk: handleOk, onError: handleError});
	},
	
	/** Rename a resource table entry but do not modify the
	 *  specified contents of that resource.
	 * 
	 * @param resourceName the name of the existing resource
	 * @param newName the new name of the resource table entry
	 * @param row the table row representing the resource
	 */
	_renameResource: function(resourceName, newName, row) {
		// Update the resource name in the template data
		var resourceData = { resourceName: newName };
		this.editor._updateResource(resourceName, resourceData);
		this._resourceModified(row);
	},
	
	/** Reload a resource already associated with the template.
	 * 
	 * @param resourceName the name of the existing resource
	 *  @param row the table row representing the resource
	 */
	_uploadResource: function(resourceName, row) {
		var self = this;
		var handleOk = dojo.hitch(this, function(resourceData) {
			// Update the resource entries in the template
			self.editor._updateResource(resourceName, resourceData);
			self._resourceModified(row);
		});		
		var handleError = dojo.hitch(this, function(resourceData) {
			self.editor._uploadError(resourceData);
		});		
		this.resourceDialog = new dm.app.reports.ui.internal.pages.TemplateResourceDialog({onOk: handleOk, onError: handleError});
	},
	
	/** Delete the corresponding resource from
	 *  the template.
	 *  
	 *  @param resourceName the name of the resource
	 *  @param row the table row representing the resource
	 */
	_deleteResource: function(resourceName, row) {
		// Ensure the user wants to delete this resource
		var substitutions = [resourceName];
		var delMessage = dojo.string.substitute(messages.confirmDeleteResource, substitutions);
		if (!confirm(delMessage)) {
			return;
		}
		
		// Delete the resource from the template
		this.editor._updateResource(resourceName, null);
		
		// Remove the entry from the table
		this._resTable.tbody.removeChild(row);
	},
	
	/** Indicate that a resource in the table has been modified.
	 * 
	 *  @param row the table row representing the resource
	 */
	_resourceModified: function(row) {
		var columns = row.children;
		if (columns) {
			for (var index = 0; index < columns.length; index++) {
				if (columns[index].className === RES_TABLE_STATUS) {
					columns[index].innerHTML = messages.resourceStatusModified;
					break;
				}
			}
		}
	}

});

})();
