/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.reports                                        |
 *|                                                                        |
 *| Copyright IBM Corporation 2011, 2012.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
dojo.provide("dm.app.reports.ui.internal.pages.TemplateResourceDialog");

/** Dialog to upload report template resources.
 * 
 * @author Kevin Cornell
 */
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dm.app.reports.ui.internal.util.ResourceServiceClient");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.Dialog");

dojo.require("dojo.i18n");
dojo.requireLocalization("dm.app.reports.ui.internal", "ReportMessages");
var messages = dojo.i18n.getLocalization("dm.app.reports.ui.internal", "ReportMessages");

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ResourceServiceClient = dm.app.reports.ui.internal.util.ResourceServiceClient;

dojo.declare("dm.app.reports.ui.internal.pages.TemplateResourceDialog", [dijit._Widget, dijit._Templated], {

	"class": "templateResourceDialog",

	messages: messages, 
	
	widgetsInTemplate: true,
	templateString: 
		'<div dojoAttachPoint="_dialogContent">' +
			'<div class="dialogInfo">${messages.selectAResource}</div>' +
			'<div class="dialogFormContainer">' +
				'<form class="dialogForm" enctype="multipart/form-data" method="post">' +
					'<input class="dialogFileInput" dojoAttachPoint="fileLocation" type="file" name="fileLocation" style="margin-bottom:1em" size="40" tabindex="0"/>' +
				'</form>' +
			'</div>' +
			'<table align=right>' +
				'<tr>' +
					'<td><button dojoAttachPoint="_okButton" disabled="disabled">${messages.ok}</button></td>' +
					'<td><button dojoAttachPoint="_cancelButton">${messages.cancel}</button></td>' +
				'</tr>' +
			'</table>' +
		'</div>',
	
	constructor: function(args) {
		this.onOk    = args.onOk;
		this.onError = args.onError;
	},
	
	postCreate: function() {
		this.dialog = new jazz.ui.Dialog({
			contentNode: this._dialogContent,
			modal: true,
			primaryTitle: messages.resourceHeading,
			width: "375px",
			isClosable: true
		});
		this.connect(this._okButton,     "onclick",  this._okPressed);
		this.connect(this._cancelButton, "onclick",  this._cancelPressed);
		this.connect(this.fileLocation,  "onchange", this._handleResourceChange);
		this._cancelButton.focus();
	},
	
	close: function() {
		this.dialog.close();
	},
	
	_okPressed: function() {
		// Create a service response handler to upload the template
		var srh = new ServiceResponseHandler(
				this,
				"_uploadSuccess",
				"_uploadError");
		ResourceServiceClient.uploadResource(this, this.fileLocation, srh);		
		this.dialog.close();
	},
	
	_cancelPressed: function() {
		this.dialog.close();
	},
	
	/** Handle the successful uploading of a resource
	 *  to be associated with a template.
	 *  
	 *  @param resourceData the uploaded resource data 
	 */
	_uploadSuccess: function(resourceData) {
		if (this.onOk) {
			this.onOk(resourceData);
		}
	},

	/** Handle an error from the ResourceServiceClient
	 *  during the uploading of the file.
	 *  
	 *  @param resourceData an object that includes the "resourceName" 
	 *						field and the "message" field
     */
	_uploadError: function(resourceData) {
		if (this.onError) {
			this.onError(resourceData);
		}
	},

	_handleResourceChange: function () {
		if (this.fileLocation.value && this.fileLocation.value !== "") {
			this._okButton.disabled = false;
		} else {
			this._okButton.disabled = true;
		}
	}
	
});

})();