/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2011, 2014. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

/**
 * @author Adam R. Neal
 */
(function() {
dojo.provide("rmps.app.ui.internal.pages.design.BreadCrumbs");

dojo.require("dojo.i18n");

dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("dijit.Menu");
dojo.require("dijit.MenuItem");
dojo.require("dijit.PopupMenuItem");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.util.html");
dojo.require("rmps.core.explorer.Viewpoint");
dojo.require("rmps.ui.explorer.ExplorerUtilities");
dojo.require("rmps.core.explorer.ExplorerMainState");
dojo.require("rmps.core.explorer.ExplorerConstants");
dojo.require("rmps.core.util.ContextUtils");
dojo.require("rmpw.common.util.IdentifierUtil");
dojo.require("rmps.app.ui.internal.pages.design.ShortLinkDialog");

dojo.requireLocalization("rmps.app.ui.internal.pages.design", "BreadCrumbMessages");

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages.design", "BreadCrumbMessages");
var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;
var explorerMainState = rmps.core.explorer.explorerMainState;
var ViewpointUtilities = rmps.core.explorer.ViewpointUtilities;
var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;

var BASELINE_PARAM = "rmps.context";

var FOLDER_ICON = dojo.moduleUrl("rmps.app", "graphics/icons/folder_open_co.gif").toString();

// TODO - this came from PropertiesEditor.js, need to try again to see if we can create a base widget that can be reused

// Mixin class to add breadcrumb navigation
dojo.declare("rmps.app.ui.internal.editor._Navigator", null, {
	
	// These parameters needs to be provided by subclasses or when the navigator object is constructed.
	navigator: null, // Navigator container widget
	
	rootElementHandler: null,
	segments: null,
	elementUri: null,
	floatingWidget: null, // This field can be used by subclasses to shuffle a cached widget for a segment that has not yet been initialized
	loadingNode: null,
	
	/* ******** Mix-in properties ********** */
		
	refreshContentsPane: function(/* String */ uri, /* Object */ node) {
		/*
		 * Subclasses should override this in order to refresh the "content" pane when a breadcrumb element has been picked.
		 * 
		 * @param uri, the uri of the element that we should show the contents for
		 * @param node, an object containing information about the current node like {id, label, iconUri}
		 */
	},
	
	hideWidget: function(widget) {
		// Subclasses should override this if they implement getWidgetToCache so that the widget can be hidden from view
	},
	
	showWidget: function(widget) {
		// Subclasses should override this if they implement getWidgetToCAche so that the widget becomes visible again
	},
	
	/*
	 * Clients must define how to initialize the root handler
	 */
	initializeRootElementHandler: null,
	
	/* ******************* End Mix-in ******************* */
	
	postCreateNavigator: function() {
		this.segments = [];
		this.rootElementHandler = this.initializeRootElementHandler();
		var span = this.loadingNode = dojo.create("span", null, this.navigator);
		span.className = "rmps-app-editors-properties-element-breadcrumb";
		span.appendChild(dojo.doc.createTextNode(messages.loading));
		dojo.style(span, "display", "");
	},
	
	showLoading: function(show) {
		dojo.style(this.loadingNode, "display", show ? "" : "none");
	},
	
	destroyNavigator: function() {
		var i, segment;
		
		if (this.rootElementHandler) {
			this.rootElementHandler.destroy();
		}
		
		for (i=0, segment; segment = this.segments[i]; i++) {
			segment.destroy();
		}
		
		// If there is a lingering floating widget then destroy it
		if (this.floatingWidget) {
			this.floatingWidget.destroyRecursive();
		}
		
		this.segments = [];
	},
	
	getActiveUri: function() {
		/*
		 * Returns the URI of the currently active element in the breadcrumbs
		 * or <code>undefined</code> if the root element is currently active
		 */
		var handler = this.segments[this.segments.length-1];
		if (!handler) {
			return undefined;
		}
		
		return handler.uri;
	},
	
	refresh: function(newUri) {
		if (this.elementUri === newUri) {
			return;
		}
		
		this.elementUri = newUri;
		
		// Remove all element handlers from view that are no longer needed
		var currentHandler = this.segments.pop();
		while (currentHandler) {
			if (currentHandler.domNode) {
				this.navigator.removeChild(currentHandler.domNode);
			}
			currentHandler.destroy();
			currentHandler = this.segments.pop();
		}
		
		// Hide the root widget if it is showing
		this.hideWidget(this.rootElementHandler.cachedWidget);
		// Deactivate the root breadcrumb if it was active
		this.rootElementHandler.setActive(false);
		
		this.populateNavigationAndViewer(newUri);
	},
	
	showNavigation: function() {
		this.navigator.style.display = "";
		this.navigator.className = "rmps-app-editors-properties-element-navigator";
		
		this.navigator.appendChild(this.rootElementHandler.domNode);
		
		this.navigator.appendChild(this.loadingNode);
				
		for (var i=0, segment; segment = this.segments[i]; i++) {
			this.navigator.appendChild(segment.domNode);
		}
	},
	
	createNewHandler: function(uri, viewpointURI) {
		var newHandler = new rmps.app.ui.internal.editor._BreadCrumbElementHandler({
			uri: uri,
			parentViewer: this,
			viewpointURI: viewpointURI
		});
		this.navigator.appendChild(newHandler.domNode);
		this.segments.push(newHandler);
		return newHandler;
	},
	
	selectNewChild: function(elementHandler, uri, node, parentURI) {
		var currentHandler = this.segments.pop();
		var originalHandler = currentHandler;
		var didPop = false;
	
		// Remove all element handlers from view that are no longer needed
		while (currentHandler && currentHandler !== elementHandler) {
			this.navigator.removeChild(currentHandler.domNode);
			currentHandler.destroyRecursive();
			currentHandler.setActive(false);
			currentHandler = this.segments.pop();
			didPop = true;
		}
		
		if (currentHandler) {
			// push the current one back on the stack
			this.segments.push(currentHandler);
		}
		
		if (!didPop && !uri && elementHandler.uri === this.elementUri) {
			// Nothing to do the user clicked on the current element again
			return;
		}
		
		// Hide any cached widgets for the handler that is no longer active
		if (originalHandler) {
			this.hideWidget(originalHandler.cachedWidget);
		} else {
			this.hideWidget(this.rootElementHandler.cachedWidget);
		}

		if (parentURI) {
			this.createNewHandler(parentURI, elementHandler.viewpointURI);
		}
		
		if (uri) {
			var newHandler = this.createNewHandler(uri, elementHandler.viewpointURI); 
				new rmps.app.ui.internal.editor._BreadCrumbElementHandler({
				uri: uri,
				parentViewer: this,
				viewpointURI: elementHandler.viewpointURI
			});
			newHandler.setActive(true);
			if (currentHandler) {
				currentHandler.setActive(false);
			}

			this.refreshContentsPane(uri, node);		
			this.elementUri = uri;
			
		} else {
			// Check this newly selected element handler to see if it has some cached properties to show
			if (elementHandler.cachedWidget) {
				this.showWidget(elementHandler.cachedWidget);
			} else {
				this.refreshContentsPane(elementHandler.uri, node);
			}
			this.elementUri = elementHandler.uri;
		}
		
		window.location.href = this.parent.input.getBreadcrumbUri(this.elementUri);
	}
	
});

dojo.declare("rmps.app.ui.internal.pages.design.BreadCrumbs", [dijit._Widget, dijit._Templated, rmps.app.ui.internal.editor._Navigator], {
	templateString: '<div dojoAttachPoint="navigator" style="height: 20px;"></div>',
		
	postCreate: function() {
		this.inherited(arguments);
		this.postCreateNavigator();
	},
	
	startup: function() {
		this.subscribeToEvents();
		this.inherited(arguments);
	},
	
	subscribeToEvents: function() {
		// clients to implement if required
	},
	
	destroy: function() {		
		this.destroyNavigator();
		this.inherited(arguments);
	},
	
	resize: function(size) {
		if (!size) {
			return;
		}
		
		var contentSize = {};
		if (size.h) {
			this.domNode.style.height = size.h + "px";
			contentSize.h = size.h - (this.navigator.style.display === "none" ? 0 : this.navigator.offsetHeight);
			this.properties.style.height = contentSize.h + "px";
		}
		
		var widgetToResize;
		if (this.segments.length > 0) {
			widgetToResize = this.segments[this.segments.length-1].cachedWidget;
		} else {
			widgetToResize = this.rootElementHandler.cachedWidget;
		}
		
		if (widgetToResize && widgetToResize.resize) {
			widgetToResize.resize(contentSize);
		}
	},
	
	hideWidget: function(widget) {
		if (widget && widget.domNode) {
			widget.domNode.style.display = "none";
		}
	},
	
	showWidget: function(widget) {
		if (widget && widget.domNode) {
			widget.domNode.style.display = "";
		}
	},
	
	populateBreadCrumbs: function(/* String */ uri) {
		/* Populates the breadcrumb widget by querying the server.
		 * Refreshes the content pane with the given URI
		 * 
		 * A URI and a viewpoint must be provided.
		 */
		if (uri) {
			
			if (this._breadCrumbsInProgress) {
				this._breadCrumbsInProgress.cancel();
			}
			// Create a deferred request to fill in the current element path within the resource
			this._breadCrumbsInProgress = explorerMainState();
			this._breadCrumbsInProgress.addCallback(this, function(ems) {
				var viewpointURI = ems.getCurrentViewpointURI(rmps.core.explorer.ExplorerConstants.IDs.EXPLORER_FULL_PAGE);
				this._breadCrumbsInProgress = ExplorerUtil.getParentChain(uri, viewpointURI, true /*dataFormat*/);
				this.showLoading(true);
				this._breadCrumbsInProgress.addCallback(this, function(results){
					this.showLoading(false);
					this.populateBreadcrumb(results, viewpointURI);
					delete this._breadCrumbsInProgress; 
				});				
			});
		} else {
			this.showLoading(false);
		}
		
		this.showNavigation();
	},
	
	populateBreadcrumb: function(results, viewpointURI) {
		var reachedResource = false;
		var elementHandler;
		this.rootElementHandler.viewpointURI = viewpointURI;
		for (var i = results.length-1; i >= 0; i--) {
			var result = results[i];
			elementHandler = new rmps.app.ui.internal.editor._BreadCrumbElementHandler({
				uri: result.uri,
				info: {
					shortName: result.label,
					iconUri: result.iconUri
				},
				parentViewer: this,
				viewpointURI: viewpointURI
			});
			this.segments.push(elementHandler);
		}
		
		if (this.segments.length > 0) {
			this.segments[this.segments.length-1].setActive(true);
			
			if (this.floatingWidget) {
				this.segments[this.segments.length-1].cachedWidget = this.floatingWidget;
				this.floatingWidget = null;
			}
			
			this.showNavigation();
		} else {
			this.rootElementHandler.setActive(true);
			
			// Do don't know that there any children at this point so we spawn off an asynchronous check
			// Assume until this call returns that there are no elements within this resource
			var deferred = ExplorerUtil.getChildren(this.rootElementHandler.getUri(), viewpointURI);
			deferred.addCallback(this, function(result) {
				if (result.children && result.children.length > 0) {
					this.rootElementHandler.setChildren(result.children, result.paged);
					this.showNavigation();
				} else {
					this.navigator.style.display = "none";
				}
			});
		}
	}
	
});

dojo.declare("rmps.app.ui.internal.editor._BreadCrumbElementHandler", dijit._Widget, {
	childrenMenu: null,
	uri: null,
	parentViewer: null,
	label: null,
	active: false,
	viewpointURI: null,
	info: null,
	_resourceLink: null,
	
	buildRendering: function() {
		this.domNode = dojo.create("span");
		this.domNode.className = "rmps-app-editors-properties-element-breadcrumb";
		this.domNode.appendChild(dojo.doc.createTextNode(messages.loading));
		
		this._createChildrenPulldown(this.domNode, this.uri);
		
		if (this.info) {
			this._handleResourceInfo(this.uri, this.info);
		} else if (ExplorerUtil.isGroup(this.uri)) {
			ExplorerUtil.getGroupInfo(this.uri, this.viewpointURI).addCallback(this, function(info) {
				this._handleResourceInfo(this.uri, {
					shortName: info.label,
					iconUri: info.iconUri
				});				
			});
		} else {
			// Last resort. Spawn off an asynchronous task to get the name of this single element
			this.getResourceInfo(this.uri);
		}
	},
	
	getResourceInfo: function(uri) {
		var deferred = ResourceInfoClient.getResourceInfo(uri, ['shortName', 'iconUri']);
		deferred.addCallback(this, function(infos) {
			this._handleResourceInfo(uri, infos[0]);
		});
	},
	
	setChildren: function(children, isPaged) {
		if (this.childrenMenu == null) {
			this._createChildrenPulldown(this.domNode, this.uri);
		}
		
		this.childrenMenu.setChildren(children, isPaged);
	},
	
	_handleResourceInfo: function(uri, info) {
		this.label = dojo.create("span");

		var isGroup = info.isGroup || ExplorerUtil.isGroup(uri);
		
		var iconUri = info.iconUri;
		if (!iconUri) {
			if (isGroup) {
				// No icon associated with the group, use the open folder icon for breadcrumbs
				iconUri = FOLDER_ICON;
			} else if (uri) {
				// Get the info/icon service uri
				iconUri = ResourceInfoClient.getIconUrl(uri);
			}
		}
		
		// Create the icon node with source set to info/icon service
		if (iconUri) {
			var icon = dojo.create("img", {
				src: iconUri,
				alt: "",
				"class": "icon"
			}, this.label);			
		}

		var anchor = dojo.create("a", null, this.label);
		anchor.appendChild(dojo.doc.createTextNode(info.shortName));
		
		if (this.active) {
			this.label.className = "rmps-app-editors-properties-element-active-element";
		}
				
		if (uri !== null && uri !== undefined) {
			var href = uri;
			if (!isGroup && !info.simpleLink) {
				var input = this.parentViewer.parent.input;
				if (input.getBreadcrumbUri) {
					href = input.getBreadcrumbUri(uri);
					if (href) {
						anchor.href = href;
						dojo.style(anchor, "cursor", "pointer");
						
						// Attach resource link
						this._resourceLink = new jazz.ui.ResourceLink({
							srcNodeRef: anchor,
							uri: net.jazz.ajax._contextRoot + "/rendering?resourceUri=" +  encodeURIComponent(uri), 
							presentationUri: href,
							lazyFetch: true
						});
					}				
				}
			} else if (info.simpleLink) {
				anchor.href = uri;
			}
			
			if (!isGroup && !this.isRoot) {
				var shortLink = uri;
				var shortLinkWithContext = shortLink;
				var displayLink;
				
				if (shortLink.indexOf(net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL) !== -1 && shortLink.indexOf('#') === -1) {
					var identifier = rmpw.common.util.IdentifierUtil.getIdentifier(uri);
					shortLink = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/models/" + identifier;
					shortLinkWithContext = shortLink + "?" + rmps.core.util.ContextUtils.computeContextParams(true);
					displayLink = identifier + ": " + info.shortName;
				} else {
					shortLink = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/logical?resource=" + encodeURIComponent(uri);
					shortLinkWithContext = shortLink + "&" + rmps.core.util.ContextUtils.computeContextParams(true);
					displayLink = info.shortName;
				}
				
				this.linkButton = new jazz.ui.toolbar.Button({ 
					title: messages.shortLink, 
					onClick: dojo.hitch(this, function() { this._showShortLink(shortLink, shortLinkWithContext, displayLink) }),
					imgClass: "rmps-app-editors-properties-element-breadcrumb-link"});
				
				this.label.appendChild(this.linkButton.domNode);
			}
		}
						
		this.domNode.replaceChild(this.label, this.domNode.firstChild);
	},
	
	_showShortLink: function (shortLink, shortLinkWithContext, displayLink) {
		new rmps.app.ui.internal.pages.design.ShortLinkDialog({shortLink: shortLink, shortLinkWithContext: shortLinkWithContext, displayLink: displayLink});
	},
	
	_createChildrenPulldown: function(element, uri) {
		this.pulldownImage = dojo.create("img", null, element);
		this.pulldownImage.src = dojo.moduleUrl("rmps.app", "graphics/icons/breadcrumb_expand.gif");
		this.pulldownImage.className = "rmps-app-editors-properties-element-breadcrumb-pulldown";
		
		if (!uri || uri.length === 0) {
			return;			
		}
		
		this.childrenMenu = new rmps.app.ui.internal.editor._ChildrenMenu({
			uri: uri,
			viewer: this,
			_isRoot: this.isRoot
		});
		this.childrenMenu.init();
		this.childrenMenu.bindDomNode(this.pulldownImage);    
	},
	
	_newChildSelected: function(uri, node, parentURI) {
		if (!uri) {
			this.setActive(true);
		} else {
			this.setActive(false);
		}
		this.parentViewer.selectNewChild(this, uri, node, parentURI);
	},
	
	setActive: function(active) {
		this.active = true;
		
		if (this.label != null && active) {
			this.label.className = "rmps-app-editors-properties-element-active-element";
		} else if (this.label != null && !active) {
			this.label.className = "";
		}
	},
	
	destroy: function() {
		if (this._resourceLink) {
			this._resourceLink.destroyRecursive();
			delete this._resourceLink;
		}
		
		if (this.childrenMenu != null) {
			this.childrenMenu.destroyRecursive();
		}
		
		// This element handler may be holding on to a cached widget with some hidden dom
		//  nodes in it. Destroy these cached widget to avoid leaks and to clean up the dom.
		if (this.cachedWidget) {
			this.cachedWidget.destroyRecursive();
		}
		
		this.inherited(arguments);
	}
});

dojo.declare("rmps.app.ui.internal.editor._ChildrenMenu", [dijit.Menu], {
	uri: null,
	leftClickToOpen: true,
	loadingItem: null,
	loaded: false,
	children: null,
	viewer: null,
	
	init: function(children, paged) {
		if (!children || children == "true") {
			this.loadingItem = new dijit.MenuItem({label: messages.loading});
			this.addChild(this.loadingItem);
		} else {
			// The children have been provided so we build up the menu directly from those
			this.setChildren(children, paged);
		}
	},

	setChildren: function(children, paged) {
		if (children == null || children.length == 0) {
			return;
		}
		
		this._removeLoadingItem();
		
		this.children = children;
		
		// TODO Decide whether truncation is the best option here if there are too many menu items
		for (var i = 0; i < this.children.length && i < 30; i++) {
			if (paged == "true") {
				var uri = ExplorerUtil.getItemURI(this.children[i]);
				
				var subMenu = new rmps.app.ui.internal.editor._ChildrenMenu({uri: uri, viewer: this.viewer});
				subMenu.init(this.children[i].children, this.children[i].paged);

				var item = new dijit.PopupMenuItem({label: jazz.util.html.escape(this.children[i].label), popup: subMenu});
				this.addChild(item);
			} else {
				this.createChildMenuItem(this.children[i]);
			}
		}
		
		this.loaded = true;
	},
	
	getUri: function() {
		return this._isRoot ? null : this.uri;
	},
	
	onOpen: function(e) {
		if (!this.loaded) {
			// Find the children of the element based on the URI to populate a list of links
			// TODO, for explorer breadcrumbs, we don't want to show children that are leafs
			var deferred = ExplorerUtil.getChildren(this.getUri(), this.viewer.viewpointURI);
			deferred.addCallback(this, function(result) {
				var items = this._isRoot ? result : result.children;
				if (!items || !items.length) {
					this._removeLoadingItem();
					this.addChild(new dijit.MenuItem({label: messages.empty, disabled: true}));
					this.loaded = true;
				} else {
					this.setChildren(items, result.paged);
				}
			});
		}
		
		this.inherited(arguments);
	},
	
	_removeLoadingItem: function() {
        if (this.loadingItem) {
			// Make sure the focus and selected state of the loading item is unset in the menu
			// before removing it, otherwise we can get exceptions on change of focus.
            this._closeChild();
            this.removeChild(this.loadingItem);
        }
		
	},
	
	createChildMenuItem: function(child) {
		var item;
		var uri = ExplorerUtil.getItemURI(child);
		
		if (child.group) {		
			var subMenu = new rmps.app.ui.internal.editor._ChildrenMenu({
				uri: uri,
				viewer: this.viewer,
				isGroup: true
			});			
			item = new dijit.PopupMenuItem({label: jazz.util.html.escape(child.label), popup: subMenu});
		} else  {
			item = new dijit.MenuItem({label: jazz.util.html.escape(child.label)});
			var parentURI = this.isGroup ? this.uri : null;
			item.onClick = dojo.hitch({viewer: this.viewer, uri: uri}, function() {
				this.viewer._newChildSelected(this.uri, child, parentURI);
			});			
		}
		
		this.addChild(item);
	},
	
	destroy: function() {
		if (this.loadingItem) {
			this.loadingItem.destroyRecursive();
		}
	
		this.inherited(arguments);
	}
});

})();
