/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("rmps.app.ui.internal.pages.design.ContextSelectionMenu");

	dojo.require("dojo.i18n");
	dojo.require("dijit._Widget");
	dojo.require("jazz.util.html");
	dojo.require("rmps.core.RootServices");
	dojo.require("rmps.app.ui.internal.pages.design.ContextSelectionPickerWindow");
	dojo.require("rmps.core.util.ContextUtils");
	
	var ContextUtils = 	rmps.core.util.ContextUtils;

	dojo.requireLocalization("rmps.app.ui.internal.pages.design", "ContextSelectionMenuMessages");

	var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages.design", "ContextSelectionMenuMessages");
	
	dojo.declare("rmps.app.ui.internal.pages.design.ContextSelectionMenu", [dijit._Widget, dijit._Templated], {

		// Template
		templateString: "<div dojoAttachPoint=\"attachPoint\" class=\"ContextQuickPickerWidget\">" +
		"<span dojoAttachPoint=\"_contextQuickSelectAttachPoint\"></span>" +
		"</div>",

		// Global vars
		attachPoint: null,
		locked: false,
		deferred: null,

		// The context
		context: null,
		
		// If workspaces should be enabled or not
		mode: null,

		// related to the menu
		_menuItems: null,
		_contextSelector:null,

		//Array of URI's
		_recentlyViewedList: ContextUtils.getRecentlyViewedContextURIs(),
		_dialog: null,

	postCreate: function() {
				
		/*
		 * *Nifty* fix for the deferred required to get either the current context or the default one. If the client
		 * has recently viewed something (_recentlyViewedList[0] is not null) then dojo.when will immediately pass that value
		 * into the dojo.hitch call, which in turn passes that context URI into the inline function to set the context and 
		 * refresh the menu without losing scope.  However, if there is no recent context then the dojo.when call will wait
		 * for either the context URI -or- the deferred returned by ContextUtils.getCurrentContextURI() to resolve before
		 * executing the dojo.hitch.
		 */
		
		ContextUtils.findCurrentContextUriAndExecute(dojo.hitch(this, function(contextURI){
			ContextUtils.setContext(contextURI);
			// Easier then setting everything up here
			this._refresh();
		}))	
	},

	// Refreshes the widget to get the latest data
	_refresh: function(){
				
		// Remove the old context selector and menu item selector
		if(this._contextSelector){
			this._contextSelector.destroy();
		}
		
		if(this._menuItems){
			this._menuItems.destroy();
		}
		
		// Make sure we're always using the "freshest" context
		// Create the menu and add the items
		this._createMenuItems();

		// Create the dropdown menu
		this._contextSelector = new dijit.form.ComboButton({
			onClick: dojo.hitch(this, function(e){
				/*Make sure to show the menu items when clicking anywhere on the context selection. Required when the context selected item is too long */
				this._contextSelector.toggleDropDown();
				
				/*Now, focus menu items. It will focus its first child */
				this._menuItems.focus();		
			}),
			label: messages.loading,
			dropDown: this._menuItems,
			"class": "contextSelectionMenu",
		});	
						
		// Get the current context and set the context object
			var contextNameDeferred = ContextUtils.getContextDeferred(ContextUtils.getCurrentContextURI());
			contextNameDeferred.addCallback(this, function(context){
				this.context = context;
				this._contextSelector.set("label", context.name);
				dojo.create("img", {src: ContextUtils.getIconUrlForContext(this.context.uri)}, this._contextSelector.iconNode);
			});
			
		// Add everything to the DOM 
		this._contextQuickSelectAttachPoint.appendChild(this._contextSelector.domNode);
		this._contextSelector.startup();
	},

	// Creates the menu items by reading from the (for now) cookie.  Later this may be stored
	// server-side.  Last item to be added is the "more" which launches the selector dialog.
	_createMenuItems: function(){
		// (re)create the menu
		this._menuItems = new dijit.Menu({
			style: "display: none;"
		});
		
		// Loop through the results, adding them to the menu.  Transparently remove the
		// currently selected item (index 0) from the recently viewed list as it's in the
		// menu already.
		if(this._recentlyViewedList.length > 1){
			for(var index = 1; index < this._recentlyViewedList.length; index++){
				this._menuItems.addChild(
						new rmps.app.ui.internal.pages.design.ContextSelectionMenu._menuItem({
							uri: this._recentlyViewedList[index],
							parent: this._menuItems,
							onClick: dojo.hitch(this, "_changeContext", this._recentlyViewedList[index])
						})
					);
			}
		}

		// Separator
		this.separator = new dijit.MenuSeparator();
		this._menuItems.addChild(this.separator);
		
		// Add the "more..." item.  This item will launch the context picker window
		_moreItem = new dijit.MenuItem({
			label: messages.more,
			onClick: dojo.hitch(this, "_showContextPicker"),
		});		
		this._menuItems.addChild(_moreItem);
		
	}, // end _createMenuItems

	// Changes the context and performs any other actions associated with changing the context.
	// Note, calling refresh here is unnecessary as setting the context will refresh the entire
	// page
	_changeContext: function(contextURI){
		ContextUtils.setContext(contextURI);
	},

	// Shows the context picker popup
	_showContextPicker: function(){	
		this._dialog = new jazz.ui.Dialog({
			contentNode: new rmps.app.ui.internal.pages.design.ContextSelectionPickerWindow({
				closeAction: dojo.hitch(this, this.closeDialog), 
				_context:this.context}).domNode, 
			primaryTitle: messages.selectContextDialogTitle, //messages.dialogTitle,
		});
	},
	
	closeDialog: function(){
		this._dialog.close();
		this._dialog.destroyRecursive();
	},

	});// end dojo.declare("rmps.app.ui.internal.pages.design.ContextSelectionMenu")

	dojo.declare("rmps.app.ui.internal.pages.design.ContextSelectionMenu._menuItem", [dijit.MenuItem], {
		
		label: messages.loading,
		uri: null,
		parent: null,
		"class": "ContextQuickPickerWidget",
		
		postCreate: function(){		
			this.inherited(arguments);

			var deferred = ContextUtils.validateContext(this.uri,
					/* Valid context */
					dojo.hitch(this, function(result){
						dojo.attr(this, "label", result.name);
						dojo.attr(this.iconNode, "src", ContextUtils.getIconUrlForContext(this.uri));
						dojo.attr(this.iconNode, "style", "margin-left: 2px; margin-right: 2px;");
					}),
					
					/* Invalid context */
					dojo.hitch(this, function(){
						ContextUtils.removeContextFromRecentlyViewed(this.uri);
						this.parent.removeChild(this);
					})
					)// end deferred
		},
	});
	
})();// end ContextSelectionMenu