/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2011. All Rights
 * Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function() {
	dojo.provide("rmps.app.ui.internal.pages.design.ContextSelectionPickerWindow");

	dojo.require("dojo.i18n");
	dojo.require("dijit._Widget");
	dojo.require("jazz.util.html");
	dojo.require("rmps.core.RootServices");
	dojo.require("dijit.Tree");
	dojo.require("dojox.grid.DataGrid");
	dojo.require("dojo.data.ItemFileReadStore");
	dojo.require("rmps.core.streams.StreamClient");
	dojo.require("rmps.core.RootServices");
	dojo.require("rmps.core.util.ContextUtils");
	dojo.require("dijit.form.CheckBox");
	dojo.require("dijit.form.Form");

	var ContextUtils = rmps.core.util.ContextUtils;


	dojo.requireLocalization("rmps.app.ui.internal.pages.design", "ContextSelectionPickerWindowMessages");

	var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages.design", "ContextSelectionPickerWindowMessages");

	dojo.declare("rmps.app.ui.internal.pages.design.ContextSelectionPickerWindow", [dijit._Widget, dijit._Templated], {

		// Big template string:
		templateString: "" + 
		"<div dojoAttachPoint=\"_contextSelectionPickerContainer\">" +

		"<div dojoAttachPoint=\"_contextTypePickerRow\">" + // Context type picker row
		"<span dojoAttachPoint=\"_currentContextLabel\" class=\"labelHeader\"> " + messages.currentContextLabel + "</span>" +
		"<span dojoAttachPoint=\"_currentContextBox\" class=\"label\"></span>" +
		"<span class=\"contextPicker\">" + 
		"<span dojoAttachPoint=\"_contextTypeLabel\" class=\"labelHeader\">" + messages.contextTypeLabel + "</span>" +
		"<span dojoAttachPoint=\"_contextTypePicker\" >" +
		"<select name=\"typeFilter\" dojoType=\"dijit.form.Select\" autoComplete=\"true\" dojoAttachPoint=\"_typeFilter\">" +
		"<option value=\"recent\" dojoAttachPoint=\"recentOption\">Recent</option>" +
		"<option value=\"snapshot\" dojoAttachPoint=\"snapshotOption\">Snapshot</option>" +
		"<option value=\"workspace\" dojoAttachPoint=\"workspaceOption\">Workspace</option>" +
		"<option value=\"all\" dojoAttachPoint=\"allOption\">All</option>" +
		"</select>" +
		"</span>" +
		"</span>" +
		"</div>" + 

		"<div dojoAttachPoint=\"_currentContextRow\">" + // Current context row
		"<span dojoAttachPoint=\"_selectedContextLabel\" class=\"labelHeader\"> " + messages.selectedContextLabel + "</span>" +
		"<span dojoAttachPoint=\"_selectedContextBox\" class=\"label\"></span>" +			
		"</div>" +			
		"<div dojoAttachPoint=\"_contentListRow\"  class=\"contentList\">" +
		"<div dojoType=\"dijit.form.Form\" dojoAttachPoint=\"_form\" id=\"selectedContextForm\" class=\"contextForm\" onChange=\"this._refresh\"></div>" + 
		"</div>" + // Content  list

		"<div dojoAttachPoint=\"_botNavRow\">" + // Bottom Nav row
		"<span dojoAttachPoint=\"_botNavRow\"></span>" +
		"</div>" + 

		"<div dojoAttachPoint=\"_OKCancelRow\" class=\"_OKCancelRow\">" +	// OK & Cancel buttons
		"<input type=\"button\" value=\"" + messages.okButton +"\" dojoAttachPoint=\"okButton\" class=\"okButton\" />" +			
		"<input type=\"button\" value=\"" + messages.cancelButton + "\" dojoAttachPoint=\"cancelButton\" class=\"cancelButton\" />" +	
		"</div>" + // OK Cancel buttons
		"</div>",

		_context: null,
		_selected: null,
		_dialog: null,
		_store: null,
		_contextTable: null,

		_workspaces: null,
		_snapshots: null,
		_recent: null,
		_data: null,
		_form: null,

		"class": "ContextSelectionPickerWindow",

		closeAction: null,

		// Called when the object is finished initializing, like a run-once
		postCreate: function(){
		// Attach the functionality to the buttons
		dojo.attr(this.cancelButton, {
			onclick: dojo.hitch(this, "closeDialog"),
		});

		dojo.attr(this.okButton, {
			onclick: dojo.hitch(this, "setContextAndClose"),
		});

		// initialise the selected variable
		this._selected = {};

		// Hook up the change listeners
		this._typeFilter.onchange = dojo.hitch(this, "_refresh");


		// Add a loading label and wait for the deferred to come back
		this._currentContextBox.innerHTML = ""; // Clear the content
		var nameCell = dojo.doc.createTextNode(messages.loading);
		this._currentContextBox.appendChild(nameCell);			

		/* 
		 * See ContextSelectionMenu.postCreate for another example of what this call does:
		 */
		ContextUtils.findCurrentContextUriAndExecute(
				dojo.hitch(this, function(contextURI){
					// Not the greatest idea, but this will go away soon
					ContextUtils.getContextDeferred(contextURI).addCallback(this, function(context){
						this._currentContextBox.innerHTML = ""; // Clear the content
						var icon = ContextUtils.getIconUrlForContext(context.uri);
						var iconCell = dojo.create("img", {src: icon, "class": "icon"}, this._currentContextBox);
						var nameLinkCell = dojo.create("a", null, this._currentContextBox);
						var nameCell = dojo.doc.createTextNode(context.name);
						nameLinkCell.appendChild(nameCell);
						this._refresh();
					});
				}));
	},

	_refreshSelected: function(){
		// Fill in the selected box
		if(this._selected){
			var contextIcon = ContextUtils.getIconUrlForContext(this._selected.uri);
			this._selectedContextBox.innerHTML = "";

			var iconCell = dojo.create("img", { src: contextIcon, "class": "icon"}, this._selectedContextBox);
			var nameLinkCell = dojo.create("a", null, this._selectedContextBox);
			var nameCell = dojo.doc.createTextNode(this._selected.name);
			nameLinkCell.appendChild(nameCell);
		}
	},

	// Refreshes the current content
	_refresh: function() {

		// Destroy the table if it exists...
		if(this._contextTable){
			this._contextTable.destroy();
		}

		// Build the table
		this._contextTable = new jazz.ui.StyledTable({
			trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
			"class" : "streamTable",
			columns: [
			          { label: "", }, 
			          { label: "Name", }
			          ],
			          showHover: true,
		});// end _contextTable
		this._form.appendChild(this._contextTable.domNode);
		this._updateDatastore();
	},

	setContextAndClose: function(){
		if(this._selected){
			ContextUtils.setContext(this._selected.uri);
		}
		this.closeDialog();
	},

	closeDialog: function(){
		this.closeAction();
	},

	/**
	 * Updates the data store from the server according to the supplied type.
	 * Types dictate the type of context (snapshot, workspace, etc...)
	 */
	_updateDatastore: function(){			
		type = this._typeFilter.value;

		// Load the store up:
		// - Determine context
		if(this.workspaceOption && type == this.workspaceOption.value){
			this._queryWorkspaces();
		}else if(type == this.snapshotOption.value){
			this._querySnapshots();
		}else if(this.allOption && type == this.allOption.value){
			// Query them all!
			this._queryAll();
		}else{
			// temporary
			this._queryLastViewed();
		}
	},

	/**
	 * Uses the data stored within the this._data variable to construct the
	 * table.
	 */
	_buildTable: function(){
		// Create and add the rows
		for(var i = 0; i < this._data.length; i++){
			var row = new rmps.app.ui.internal.pages.design.ContextSelectionPickerWindow._ContextRow({context:this._data[i], parent:this});
			this._contextTable.tbody.appendChild(row.domNode);
		}
	},

	/**
	 * Queries the server for all the workspaces available
	 */
	_queryWorkspaces: function(){
		// Since we cache workspace data, don't re-load it.
		if(this._workspaces){
			this._data = this._workspaces;
			this._buildTable();
			return;
		}

		var deferred = rmps.core.streams.StreamClient.getStreams();
		this._workspaces = new Array();
		deferred.addCallback(this, function(streams){	
			// Track our workspaces
			for(var i = 0; i < streams.length; i++){
				this._workspaces.push(streams[i]);
			}
			this._data = this._workspaces;
			this._buildTable();
		});
	},

	/**
	 * Queries the server for all the snapshots and sets the data store
	 */
	_querySnapshots: function() {

		// Cache snapshot data, use snapshot cache
		// TODO: good idea maybe maybe not...?
		if(this._snapshots){
			this._data = this._snapshots;
			this._buildTable();
			return;
		}

		// Query the server for available snapshots
//		if(this._selected){
//		var deferred = rmps.core.baseline.BaselineClient.getSnapshotsForStream(this._selected.uri, jazz.app.context.get().itemId);
//		} else {
		var deferred = rmps.core.baseline.BaselineClient.getBaselines();
//		}
		this._snapshots = new Array();
		deferred.addCallback(this, function(snapshots){
			for(var i = 0; i < snapshots.length; i++){
				this._snapshots.push(snapshots[i]);
			}
			this._data = this._snapshots;
			// Build the table using the current data.
			this._buildTable();
		});
//		}
	},

	/**
	 * Examines the cookie for the most recently used contexts and adds them to
	 * the data store
	 */
	_queryLastViewed: function(){
		// Look for the cookie
		// Obtain contents
		// Query the server for the recovered contexts
		// Load data store
		recentURIs = ContextUtils.getRecentlyViewedContextURIs();
		deferredArray = new Array(recentURIs.length);
		this._recent = new Array(recentURIs.length);
		var index = 0;

		// Note: may not show in correct order...serializing the calls might be
		// slow though.
		for(var i = 0; i < recentURIs.length; i++){
			deferredArray[i] = ContextUtils.getContextDeferred(recentURIs[i]);
			deferredArray[i].addCallback(this, function(context){
				var row = new rmps.app.ui.internal.pages.design.ContextSelectionPickerWindow._ContextRow({context:context, parent:this});
				this._contextTable.tbody.appendChild(row.domNode);
			});
		}// end for
		this.data = this._recent;
	},

	/**
	 * Queries for all the contexts
	 */
	_queryAll: function(){		
		this._all = new Array();
		var snapshotsDeferred = rmps.core.baseline.BaselineClient.getBaselines();
		snapshotsDeferred.addCallback(this, function(snapshots){
			if(!this._snapshots){
				for(var i = 0; i < snapshots.length; i++){
					this._all.push(snapshots[i]);	
				}
			}
			// Now get the workspaces
			var workspacesDeferred = rmps.core.streams.StreamClient.getStreams();
			workspacesDeferred.addCallback(this, function(streams){	
				if(this._workspaces){
					this._all = this._snapshots.concat(this._workspaces);
				} else {
					// Track our workspaces
					for(var i = 0; i < streams.length; i++){
						this._all.push(streams[i]);
					}
				}
				this._data = this._all;
				this._buildTable();
			});
		});
	},

	}); // end dojo.declare

	/**
	 * A row in the context table
	 */
	// TODO: Hook up the system-wide context compact rendering
	dojo.declare("rmps.app.ui.internal.pages.design.ContextSelectionPickerWindow._ContextRow", [dijit._Widget], {

		context: null,
		parent: null,

		buildRendering: function(){
		// Create a table row to work in
		this.domNode = dojo.create("tr", null, this.domNode);
	},

	postCreate: function(){		
		// Create the DOM containers
		var radioCell = dojo.create("td", null, this.domNode);
		var nameCell = dojo.create("td", null, this.domNode);
		var iconURL = ContextUtils.getIconUrlForContext(this.context.uri);
		var iconCell = dojo.create("img", {src: iconURL, "class": "icon" }, nameCell);
		var nameLinkCell = dojo.create("a", null, nameCell);

		// Create the radio button
		var radioButton = new dijit.form.RadioButton({
			value: this.context.uri,
			name: "selectedContextOption",
			onClick: dojo.hitch(this, "_selectContext" )
		});

		if(this.parent._selected.uri == this.context.uri){
			dojo.attr(radioButton, { "checked" : "true" });
		}

		var contextName = dojo.doc.createTextNode(this.context.name);

		// Attach the text nodes to the appropriate cells
		nameLinkCell.appendChild(contextName);
		radioCell.appendChild(radioButton.domNode);

		// Add the click behaviour, unless this is the current context
		if(ContextUtils.getCurrentContextURI != this.context.uri){
			dojo.connect(nameCell, "onclick", this, "_selectContext");
		}
	},

	/**
	 * Sets the context to the selected context
	 */
	_selectContext: function(){
		this.parent._selected = this.context;
		this.parent._refreshSelected();
	},

	}); // end dojo.declare

})(); // end function
