/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.pages.design.DesignComponentMenu");

dojo.require("rmps.ui.editor.CapabilityComponentMenu");
dojo.require("rmps.app.core.util");
dojo.require("rmps.ui.util.UIUtil");
dojo.require("rmps.app.ui.internal.util.DomainRegistryUtil");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("rmps.core.util.TypeConstants");
dojo.require("dojo.DeferredList");
dojo.require("dojo.i18n");

//Load the used dialogs in async manner
var DesignResourceSelection= "rmps.app.ui.internal.pages.design.DesignResourceSelection";

var DomainRegistryUtil = rmps.app.ui.internal.util.DomainRegistryUtil;
var DesignUtil = rmps.app.ui.internal.pages.design.DesignUtil;
var TypeConstants= rmps.core.util.TypeConstants;

var MIN_SELECTIONS= 5;
var MAX_SELECTIONS= 10;
var TYPES_SELECTION_WIDTH= "800px";
var TYPES_SELECTION_HEIGHT= "300px";

/**
 * This module will populate the Design page context menu.
 * 
 * @author Adam R. Neal
 */
dojo.declare("rmps.app.ui.internal.pages.design.DesignComponentMenu", rmps.ui.editor.CapabilityComponentMenu, {

	_isActive: null,
	_isDsp: null,
	_isSystem: null,
	_createableUris: null,
	_uploadEntries: null,
	_uploadNamespace: null,
	
	//If displayFooter is ture, then we display the recent created items after a seprator in the creation menu (as footer)
	_displayFooter: false,
	
	constructor: function(){
		this._createableUris= [];
		this._uploadEntries= [];
		this._uploadNamespace= [TypeConstants.FILE_NAMEPSACE, TypeConstants.IMAGE_NAMEPSACE];
		
		this.isActive= false;
		this._isDsp= false;
		this._isSystem= false;
	},
	
	postMixInProperties: function(){
		this.inherited(arguments);
		
		dojo.requireLocalization("rmps.ui.explorer", "ExplorerMessages");
		dojo.requireLocalization("rmps.app.ui.internal.pages.design", "DesignMessages");
		dojo.requireLocalization("rmps.app.ui.internal.pages.design", "ValidationMessages");
		dojo.requireLocalization("rmps.ui.common", "CommonMessages");
		
		this.exploreMessages = dojo.i18n.getLocalization("rmps.ui.explorer", "ExplorerMessages");
		this.designMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages.design", "DesignMessages");
		this.validationMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages.design", "ValidationMessages");
		this.commonMessages = dojo.i18n.getLocalization("rmps.ui.common", "CommonMessages");
		
		//this.importMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages.design", "importMessages");
		//dojo.requireLocalization("rmps.app.ui.internal.pages.design", "importMessages");
	},
	
	addCapabilityComponentMenuItems: function() {
		this._addBrowseItems();

		/*
		 * The rest of the actions depend on information about whether its a DSP or not or actively managed
		 */
		var dspDef = new dojo.Deferred();
		var isActiveDef = new dojo.Deferred();
		var getCreatableTypesDef = new dojo.Deferred();
		
		/*
		 * Here we query the Design service for available actions passing the projectURI and user URI
		 * When the user finally clicks the design menu, we'll already have our results
		 */		
		DomainRegistryUtil.getCreateableURIs(dojo.hitch(this, function(uris) {
			this._createableUris = uris;
			getCreatableTypesDef.callback();
		}));
		
		DomainRegistryUtil.isCurrentProjectDSP(dojo.hitch(this, function(isDsp, isSystem) {
			this._isDsp = isDsp;
			this._isSystem = isSystem;
			dspDef.callback();
		}));

		DomainRegistryUtil.isProjectActivelyManaged(dojo.hitch(this, function(isDmManaged) {
			this._isActive = isDmManaged;
			isActiveDef.callback();
		}));
		
		loading = this.addLoading();
		
		new dojo.DeferredList([getCreatableTypesDef, dspDef, isActiveDef]).addCallback(dojo.hitch(this, function(){
			this.remove(loading);
			this.addTitle(this.designMessages.create, {key:"create",after: "browse"});
			
			if (this._isDsp) {	
				// If a DSP, then add 'browse viewpoints'  
				this.addItemByKey({
					label: this.designMessages.viewpoints,
					href: DesignUtil.getViewpointsUri(),
					icon: 'rmps.app/graphics/icons/viewpoints.gif',
					key: "browse"
				});
				
				// If a DSP, then add 'create viewpoint', unless this._createableUris is null which indicates no EDIT permissions 
				// (if EDIT permissions exist, but there is no creatable uris, then this._createableUris is empty not null)
				if (this._createableUris != null) {
					this.addItemByKey({
						label: this.designMessages.viewpoint,
						href: DesignUtil.getViewpointUri(null),
						icon: 'rmps.app/graphics/icons/show_viewpoint_co.gif',
						key: "create"
					});
				}
			}
			
			this.addItemByKey({
				label: this.validationMessages.validationMenuItem,
				href: DesignUtil.getValidationResourceUri(),
				icon: 'rmps.app/graphics/icons/validation.gif',
				key: "browse"
			});
			
			if (this._isActive || !this._isSystem) {
				// Add creation types for actively managed project area, or user DSP
				// Add creation types for actively managed project area
				this._addCreateResourcesItems();
			}
				
			this._addCreateItems();
			
//			if (this._isDsp && !this._isSystem) {
//				// Add Import title
//				this.addTitle(this.importMessages.menuHeaderImport);
//				
//				// Meta-model import is to be added for user DSP
//				this.addItemByArgs({
//					label: this.importMessages.menuImportMetamodels,
//					href: DesignUtil.getImportResourceUri()
//				});
//				
//				// Meta-model import is to be added for user DSP
//				this.addItemByArgs({
//					label: this.importMessages.menuImportOwl,
//					href: DesignUtil.getOwlImportResourceUri()
//				});
//					
//			}
		}));
		this.inherited(arguments);
	},
	
	_addBrowseItems: function(){
		// Welcome
		this.addItemByArgs({
			label: this.designMessages.welcomeToDesigns,
			href: DesignUtil.getWelcomeUri(),
			icon: 'rmps.app/graphics/icons/allDesigns.gif',
			key: "create"
		});
		this.addTitle(this.commonMessages.browse, {key:"browse"});
		
		// Clipboard
		this.addItemByKey({
			label: this.designMessages.clipboard,
			href: DesignUtil.getClipboardUri(),
			icon: 'rmps.app/graphics/icons/clipboard_empty_co.gif',
			key: "create"
		});
		
		// Explorer
		this.addItemByKey({
			label: this.exploreMessages.explore,
			href: DesignUtil.getExploreResourceUri(),
			icon: 'rmps.app/graphics/icons/allDesigns.gif',
			key: "create"
		});

		// Queries
		this.addItemByKey({
			label: this.designMessages.queries,
			href: DesignUtil.getQueriesUri(),
			icon: 'rmps.app/graphics/icons/allQueries.gif',
			key: "create"
		});
	},
	
	_addCreateResourcesItems: function(){
		if (!this._createableUris || !this._createableUris.length) {
			return; // got a result, but its empty...
		}
		if(this._createableUris.length> MAX_SELECTIONS){
			if(this._displayFooter){
				this._addCreateResourcesItemsTitle(MIN_SELECTIONS, MAX_SELECTIONS, dojo.hitch(this, this._handleShowMoreCreateResource));
				var selectionDetails= this.prepareSelectionsAsFooter(this._createableUris, MIN_SELECTIONS);
				this.addItemsByKey(selectionDetails.firstSelections, "createResource");
				this.addFooterItemByArgs(selectionDetails.recentSelections, "createResource");
				this.addItemsByKey(selectionDetails.remained, "createResource");	
			}else{
				this._addCreateResourcesItemsTitle(MAX_SELECTIONS, MAX_SELECTIONS, dojo.hitch(this, this._handleShowMoreCreateResource));
				var selectionDetails= this.prepareSelections(this._createableUris, MAX_SELECTIONS);	
				this.addItemsByKey(selectionDetails.firstSelections, "createResource");
				this.addItemsByKey(selectionDetails.restSelections, "createResource");
			}
		}else{
			this._addCreateResourcesItemsTitle();
			var allSelections= [];
			//All selections, and make sure to maatin the same label order
			for(var index = 0, entry; entry = this._createableUris[index]; index++) {
				var argsValue= this._addArg(entry);
				if(argsValue){
					allSelections.push(argsValue);				
				}
			}
			this.addItemsByKey(allSelections, "createResource");
		}
		
		if (this._uploadEntries.length>0) {
			// Upload title
			this.addTitle(this.designMessages.upload,  {key:"upload", after: "createResource"});
	
			for(var index = 0, entry; entry = this._uploadEntries[index]; index++) {
				this.addUploadItem(entry);
			}
		}
	},
	
	_addCreateResourcesItemsTitle: function(minLimit, maxLimit, handleShowMoreCreateResource){
		// Add Create title
		this.addTitle(this.designMessages.createDesignResource, {key:"createResource", after: "create", 
			minLimit: minLimit, maxLimit: maxLimit, footerLength: minLimit, 
			handleMore: handleShowMoreCreateResource
		});
	},
	
	_handleShowMoreCreateResource: function(titleDescriptor){
		if(this.__invokingSelectionDialog){	//Guard against redundant clicks
			return;
		}
		this.__invokingSelectionDialog= true;
		var selectionDialog;
		rmps.ui.util.UIUtil.loadAndRun(DesignResourceSelection, {width: TYPES_SELECTION_WIDTH, 
			height: TYPES_SELECTION_HEIGHT,
			onOK: dojo.hitch(this, function(selection) {
				if(selectionDialog){
					selectionDialog.destroyRecursive();					
				}
				if(!selection){
					return
				};
				var uri= selection.uri;
				
				if(!uri|| this._containsItem(uri)){
					return;
				}
				var items= this.getItemsFor("createResource");
				var selectionArgs= this._addArg(selection);
				
				if(items.length<0){	//Safe check
					this.addItemsByKey([selectionArgs], "createResource");
				}
				
				var label= selectionArgs.label;
				for(var index = 0, item; item = items[index]; index++) {
					if(item.params.label > label){
						break;
					}
				}
				//Replace by the previous item indexitems[index]
				//Make sure to get the index from the all list not the menu list, as if we are the last item in the menu list, we will get undefined item if we reached the last item
				//The expected inset inde will be the index of the first item in the resource creation menu "items[0].getIndexInParent()" plus the numer of items that come in order 
				// before the new item to be replaced
				var firstItemIndex= items[0].getIndexInParent();
				var insertIndex= firstItemIndex+ index;
				if(insertIndex<=firstItemIndex){
					//If the replacemenet will be the first item in the list, then make sure to assign the index insertion value to the first item in the list, as the title node is not
					//considered in the method getIndexInParent which means that for the first item case, we will be having a shift by one item
					insertIndex= firstItemIndex;
				}else{
					insertIndex= insertIndex-1;
				}
				
				this.replaceBy(selectionArgs, this.getLeastInSelection(), insertIndex);
			}),
		
			onCancel: dojo.hitch(this, function(e) {
				if(selectionDialog){
					selectionDialog.destroyRecursive();					
				}}
			)}, dojo.hitch(this, function(instance){
				delete this.__invokingSelectionDialog;
				selectionDialog= instance;
			})
		);
	},
	
	_containsItem: function(uri){
		var items= this.getItems();
		for(var index = 0, item; item = items[index]; index++) {
			if(item.params.value=== uri){
				return true;
			}
		}
		return false;
	},
	
	addUploadItem: function(item){
		var params = {
			label: item.label,
			href: DesignUtil.getCreateResourceUri(item.uri),
			key:"upload"
		};
		if (item.iconUri) {
			// Make sure the icon uses the correct baseline
			params.icon = DesignUtil.getFileUri(item.iconUri);
		}		
		this.addItemByKey(params);
	},
	
	getPageId: function(){
		return DesignUtil.PAGE_ID;
	},
	
	_addCreateItems: function(){
		// Add query and viewpoint creation.
		// TODO: Move to separate title? Order alphabetically with server returned classes?
		// TODO: Return URI from server (if editor definition can be defined)? 
		this.addItemByKey({
			label: this.designMessages.query,
			href: DesignUtil.getQueryUri(null),
			icon: 'rmps.app/graphics/icons/query.gif',
			key: "create"
		});
	},
	
	/** 
	 * When used, then we display a list of 10 items including the most recent selected items first, and if not enough to complete a list of the preferred items, then we get the 
	 * rest from the domains in which we get the first items from each domain, and then the second till we get our list of items completed 
	 */
	prepareSelections: function(uris, selectionMax){
		var firstSelections= [];
		var restSelections= [];
		
		var remained= dojo.clone(uris);
		var remainedUris= dojo.map(remained, function(entry){return entry.uri});
		var visitedUris= [];
		
		var recentlySelectArgsList= [];
		var recentlyUsed= DomainRegistryUtil.loadRecentSelections();
				
		for(var index = 0, uri; uri = recentlyUsed[index]; index++) {
			if(!dojo.isString(uri)|| dojo.indexOf(visitedUris, uri)>-1|| dojo.indexOf(this._uploadNamespace, uri)>-1){
				continue;
			}
			var subIndex= dojo.indexOf(remainedUris, uri);
			var associatedEntry= remained[subIndex];
			
			if(!associatedEntry){
				continue;
			}
			
			var argsValue= this._addArg(associatedEntry);
			
			remained.splice(subIndex,1);
			remainedUris.splice(subIndex,1);
			
			//Required to de detect duplicate cookies entries, as we splice the remianed items. 
			visitedUris.push(uri);
			if(!argsValue){
				continue;
			}
			firstSelections.push(argsValue);
			if(firstSelections.length===selectionMax){
				break;
			}
		}
		
		var groupedUris= this._groupBy(uris, "domainUri");
		
		var index=0;
		while(index< uris.length){
			for(var domain in groupedUris){
				var entryValue= groupedUris[domain][index];
				if(entryValue){
					var entryValueUri= entryValue.uri;
					var indexOf= dojo.indexOf(remainedUris, entryValueUri);	//No need to check for index value
					if(indexOf<0){
						continue;
					}
					
					remained.splice(indexOf,1);
					remainedUris.splice(indexOf,1);
					var argsValue= this._addArg(entryValue);
					if(!argsValue){
						continue;
					}
					if(firstSelections.length<selectionMax){
						firstSelections.push(argsValue);
						continue;
					}
					restSelections.push(argsValue);
				}
			}
			++index;		
		}	
		
		firstSelections.sort(this.stringCompare);
		restSelections.sort(this.stringCompare);
		
		return {
			firstSelections: firstSelections,
			restSelections: restSelections
		};
	},
	
	/**
	 * When used, this means that the default selections will be added as five permanent items (an item from each domain, and then the second, and etc). And then a separator
	 * menu item, and then the recently used items with a maximum length of 5 items.  
	 */
	prepareSelectionsAsFooter: function(uris, selectionMax){
		var firstSelections= [];
		var recentSelections= [];
		var index= 0;
		var groupedUris= this._groupBy(uris, "domainUri");
		var remained= dojo.clone(uris);
		var remainedUris= dojo.map(remained, function(entry){return entry.uri});
		var visitedUris= [];
		
		locateFirstItems: {
			while(firstSelections.length<selectionMax&& firstSelections.length< uris.length&& index< uris.length){
				for(var domain in groupedUris){
					var entryValue= groupedUris[domain][index];
					if(entryValue){
						var entryValueUri= entryValue.uri;
						var indexOf= dojo.indexOf(remainedUris, entryValueUri);	//No need to check for index value
						remained.splice(indexOf,1);
						remainedUris.splice(indexOf,1);
						var argsValue= this._addArg(entryValue);
						if(!argsValue){
							continue;
						}
						firstSelections.push(argsValue);
						visitedUris.push(entryValueUri);
						if(firstSelections.length>= selectionMax){
							break  locateFirstItems;
						}					
					}
				}
				++index;		
			}	
		}
		var recentlySelectArgsList= [];
		var recentlyUsed= DomainRegistryUtil.loadRecentSelections();
		if(!dojo.isArray(recentlyUsed)){
			recentlyUsed= [];
		}
		
		for(var index = 0, uri; uri = recentlyUsed[index]; index++) {
			if(!dojo.isString(uri)|| dojo.indexOf(visitedUris, uri)>-1|| dojo.indexOf(this._uploadNamespace, uri)>-1){
				continue;
			}
			var associatedEntry;
			for(var subIndex = 0, entry; entry = remained[subIndex]; subIndex++) {
				if(entry.uri== uri){
					associatedEntry= entry;
					break;
				}
			}	
			if(!associatedEntry){
				continue;
			}
			
			var argsValue= this._addArg(associatedEntry);
			
			remained.splice(subIndex,1);
			
			if(!argsValue){
				continue;
			}
			recentSelections.push(argsValue);
			if(recentSelections.length===MIN_SELECTIONS){
				break;
			}
		}
		recentSelections.sort(this.stringCompare);
		
		var preparedReamined= [];
		
		for(var index = 0, entry; entry = remained[index]; index++) {
			var argsValue= this._addArg(entry);
			if(argsValue){
				preparedReamined.push(argsValue);				
			}
		}
		return {
			firstSelections: firstSelections,
			recentSelections: recentSelections,
			remained: preparedReamined
		};
	},
	
	getLeastInSelection: function(){
		var items= this.getItemsBasedOnPriority();
		return items[items.length-1];
	},
	
	/** 
	 * Get the list of items based on the user's selection order 
	 */
	getItemsBasedOnPriority: function(){
		var items= this.getItemsFor("createResource");
		var recentlyUsed= DomainRegistryUtil.loadRecentSelections();
		var allUris= dojo.map(this._createableUris, function(entry){return entry.uri});
		
		var decide= function(index1, index2){
			if(index1<0&& index2>0){
				return 1;
			}
			
			if(index1>0&& index2<0){
				return -1;
			}
			
			if(index1>-1 && index2> -1){
				return index1> index2? 1: -1;
			}
			return "none";
		};
		items.sort(dojo.hitch(this, function(item1, item2){
			var uri1= item1.params.value;
			var uri2= item2.params.value;
			
			var index1= dojo.indexOf(recentlyUsed, uri1);
			var index2= dojo.indexOf(recentlyUsed, uri2);
						
			var compare= decide(index1, index2);
			if(compare!== "none"){ return compare;}
			
			index1= dojo.indexOf(allUris, uri1);
			index2= dojo.indexOf(allUris, uri2);
			
			compare= decide(index1, index2);
			if(compare!== "none"){ return compare;}
			
			return 0;
		}));
		return items;
	},
	
	/*Sort items based on their labels */		
	stringCompare: function (entry1, entry2){
		var string1= entry1.label|| "";
		var string2= entry2.label|| "";
		return string1.toLocaleLowerCase().localeCompare(string2.toLocaleLowerCase());
	},
	
	_groupBy: function(data, groupBy){
		var dataObject= {};
		for(var index = 0, entry; entry = data[index]; index++) {
			var identifier= entry[groupBy];
			if(!identifier){
				identifier= "ungrouped"; 
			}
			if(!dataObject[identifier]){
				dataObject[identifier]= [];
			}
			dataObject[identifier].push( entry);
		}
		return dataObject
	},
	
	_addArg: function(resource){
		var uri= resource.uri;
		
		// Need closure scope so 'resource' is stored
		if (dojo.indexOf(this._uploadNamespace, uri)>-1) {
			if(dojo.indexOf(this._uploadEntries, resource)<0){
				this._uploadEntries.push(resource);				
			}
			return null;
		}
		
		var args = {
			label: resource.label,
			href: DesignUtil.getCreateResourceUri(uri),
			_onClick: function(){
				//Important to ensure updating the selection history order when the user clicks on an item directly in the list; otherwise, it will loose its priority by the time
				DomainRegistryUtil.addCreationSelection(this.value);
				this.inherited("_onClick", arguments);
			},
			value: uri
		};
		if (resource.iconUri) {
			// Make sure the icon uses the correct baseline
			args.icon = DesignUtil.getFileUri(resource.iconUri);
		}
		return args;
	}
});
}());