/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

/**
 * This input represents a creation editor.  It provides special API to set the new logical URI
 * of the element created.  And it specializes the name (and ID) to use the time of the element
 * being created (similar to RTC WorkItems, such that it shows: "<type> <time>"
 * @author Adam R. Neal
 */
(function() {
dojo.provide("rmps.app.ui.internal.pages.design.DesignCreateEditorInput");

dojo.require("rmps.app.ui.internal.pages.design.DesignEditorInput");
dojo.require("dojox.encoding.digests.MD5");

dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
var util = rmps.app.ui.internal.pages.design.DesignUtil;

dojo.requireLocalization("rmps.app.ui.internal.pages.design", "DesignMessages");
var designMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages.design", "DesignMessages");

var constructTime = function() {
	var date = new Date();
	var h = date.getHours(), m = date.getMinutes(), s = date.getSeconds();
	if (h < 10) {
		h = "0" + h;
	}
	if (m < 10) {
		m = "0" + m;
	}
	if (s < 10) {
		s = "0" + s;
	}
	return "<" + h + ":" + m + ":" + s + ">";
}

dojo.declare("rmps.app.ui.internal.pages.design.DesignCreateEditorInput", rmps.app.ui.internal.pages.design.DesignEditorInput, {
	/*
	 * uri -> type uri
	 * name -> type label
	 * icon -> type icon
	 */
	
	mode: util.createMode,
	__time: null,
	
	constructor: function(params) {
		this.inherited(arguments);
		this.__time = constructTime();
	},
	
	getId: function() {
		// Uniqueness will be based on type (the uri), projectArea, and time.
		return dojox.encoding.digests.MD5(this.declaredClass + ',' + this.__time + ',' + this.resourceInfo.uri + ',' + this.projectId);
	},

	getTabs: function() {
		//TODO this should be loaded the file /com.ibm.xtools.rmps.diagram.frontservice/data/system/instances/DefaultFormEditor.rdf
		// instead of had code it here.
		return [{
		    id: "defaultFormEditor",
		    label: "Properties",
		    type: {
		      widget: "rmpw.forms.ui.FormPage"
		    },
		    editorConstraint: "http://jazz.net/ns/dm/editor/editorconstraint/form#",
		    uri: "http://jazz.net/ns/dm/editor/definition/form"
		  }];
	},
	
	getName: function() {
		var name = this.resourceInfo.name;
		if (!name) {
			return designMessages.loading;
		}
		return name + ' ' + this.__time;
	},

	getTooltip: function() {
		return designMessages.createNewResource_tooltip + this.resourceInfo.name;
	},

	getBasePresentationUri: function() {
		return util.getCreateResourceUri(this.resourceInfo.uri);
	},
	
	unload: function() {
		this.inherited(arguments);
		delete this.__newResourceURI;
		delete this.__time;
	},
	
	setNewResourceURI: function(/* String */ uri) {
		/*
		 * To be called with logical URI of newly created resource.
		 */
		this.__newResourceURI = uri;
	},
	
	getNewResourceURI: function() {
		return this.__newResourceURI;
	}
	
});

}());
