/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("rmps.app.ui.internal.pages.design.DesignEditorInput");

dojo.require("dojo.DeferredList");
dojo.require("dijit.Dialog");
dojo.require("dojox.encoding.digests.MD5");
dojo.require("rmps.ui.editor.EditorInput");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("rmps.app.ui.internal.util.DomainRegistryUtil");
dojo.require("rmpw.common.store.JfsStore");

dojo.requireLocalization("rmps.app.ui", "EditorUtilMessages");
dojo.requireLocalization("rmps.app.ui.internal.pages.design", "DesignMessages");

var messages = dojo.i18n.getLocalization("rmps.app.ui", "EditorUtilMessages");
var designMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages.design", "DesignMessages");
var DesignUtil = rmps.app.ui.internal.pages.design.DesignUtil;

/**
 * This is a resource centric editor input, unlike the RMPS ResourceEditorInput, even a fragment
 * constitutes a different resource.
 * 
 * @author Adam R. Neal
 */
dojo.declare("rmps.app.ui.internal.pages.design.DesignEditorInput", rmps.ui.editor.EditorInput, {
	projectId: null,
	resourceInfo: null,
	mode: null, // default mode
	
	_hasSidebarExplorer: true,

	/**
	 *  Params include:
	 *    projectId
	 *    uri
	 */
	constructor: function(params) {
		this.projectId = params.projectId;
		this.resourceInfo = {};
		
		var uri = params.uri;
		var constructorArgs = params.constructorArgs;
		if(uri || constructorArgs) {
			this.mode = uri ?
					rmps.app.ui.internal.pages.design.DesignUtil.editMode :
					rmps.app.ui.internal.pages.design.DesignUtil.createMode;
			
			this.resourceInfo.uri = uri;
			this.resourceInfo.constructorArgs = constructorArgs;
			
			// create store
			this.store = new rmpw.common.store.JfsStore({
				uri: uri,
				constructorArgs: constructorArgs
			});
			this.__time = this.constructTime();
		} else {
			// being called by the explorer, ignore
			// TODO: prevent this case from creating an editor input?
		}
	},
	
	/**
	 * hasFocusedElement: boolean, true if the requested resource is not the document itself, but a fragment within it
	 */
	getTabs: function(hasFocusedElement) {
		var tabs = null;
		
		rmps.core.client.xhrGet({
			url: net.jazz.ajax._contextRoot + "/dtk/EditorDefinition/all",
			content: {
				docType: dojo.map(this.resourceInfo.model.rdf_type, function(type) { return type.rdf_about||type.$ref; }),
				hasFocusedElement: hasFocusedElement,
				isCreation: this.mode == rmps.app.ui.internal.pages.design.DesignUtil.createMode
			},
			headers: {
				"Accept": "application/json"
			},
			handleAs: "text",
			load: function(data, ioargs) {
				tabs = dojo.fromJson(data);
			},
			error: function(error, ioargs){
				dojo.require("dijit.Dialog");
				var errorDlg = new dijit.Dialog({
		            title: "SERVER ERROR",
		            style: "width: 300px"
		        });					
				errorDlg.attr("content", error.message);
				errorDlg.show();
	        },
	        sync: true
		});
		if (hasFocusedElement) {
			// showing the fragment info: 
			// tabs that are focused on fragment should have a title with the fragment name included
			var index, length;
			for (index = 0, length = tabs.length; index < length; ++index) {
				tab = tabs[index];
				if (tab.supportsFocus) {
					tab.label += " ("
					if (this.resourceInfo.fragmentTypeLabel) {
						tab.label += this.resourceInfo.fragmentTypeLabel + ": ";
					}
					if (this.resourceInfo.fragmentName) {
						tab.label += this.resourceInfo.fragmentName;
					}
					tab.label += ") ";
				}
			}
		}
		return tabs;
	},

	load: function() {
		// If no URI, we are exploring the root, we can assume we're loaded
		// TODO (MG): we shouldn't be called when the explorer loads in the first place
		if(!(this.resourceInfo.uri || this.resourceInfo.constructorArgs)) {
			var deferred = new dojo.Deferred();
			setTimeout(function() { deferred.callback(); }, 0);
			return deferred;
		}
				
		var deferred = new dojo.Deferred();
		var storeDeferred= this.store.load();
		
		storeDeferred.addCallback(dojo.hitch(this, function(result){
			this._onLoaded(result);
			deferred.callback(result);
		}));
		
		storeDeferred.addErrback(dojo.hitch(this, function(error){
			this._onLoadFailed(error);
			deferred.errback(error);
		}));

		return deferred;
	},
	
	_onLoaded: function(result) {
		dojo.mixin(this.resourceInfo, result);

		this.resourceInfo.name = result.currentResourceInfo.name || "<" + designMessages.empty + ">"; 
		this.resourceInfo.presentationName = result.currentResourceInfo.presentationName;
		this.resourceInfo.tooltip = result.currentResourceInfo.name;
		this.resourceInfo.documentUri = result.currentResourceInfo.documentUri;
		this.resourceInfo.fragmentName = result.currentResourceInfo.fragmentName;
		this.resourceInfo.fragmentTypeLabel = result.currentResourceInfo.fragmentTypeLabel;
		this.resourceInfo.conceptUri = result.currentResourceInfo.conceptUri;

		this.resourceInfo.iconUri = null;		
		if (this.mode == rmps.app.ui.internal.pages.design.DesignUtil.editMode) {
			// we only want to show type icon while editing, not while creating, 
			// as on creation we would get type's type icon
			if (result.currentResourceInfo.iconUri) {
				this.resourceInfo.iconUri = result.currentResourceInfo.iconUri + 
					'?' + rmps.core.util.IRmpsConstants.BASELINE + '=' + encodeURIComponent(jazz.app.context.get().baseline);
			}
		}
	},
	
	_onLoadFailed: function(error) {
		var errorDlg = new dijit.Dialog({
            title: "SERVER ERROR",
            style: "width: 300px"
        });					
		errorDlg.set("content", error.message);
		errorDlg.show();
	},
	
	save: function(/*rmps.core.TaskMonitor*/ monitor) {
		this.store.update(monitor);
	},

	isLoaded: function() {
		return this.resourceInfo && this.resourceInfo.model;
	},
	
	unload: function() {
		if(this.mode == rmps.app.ui.internal.pages.design.DesignUtil.createMode) {
			this.inherited(arguments);
			delete this.__newResourceURI;
			delete this.__time;
		} else {
			delete this.resourceInfo.model;
		}
	},

	isReadOnly: function() {
		return this.resourceInfo.isReadOnly;
	},
	
	getId: function() {
		if(!this._id){
			if(this.mode == rmps.app.ui.internal.pages.design.DesignUtil.createMode){
				this._id= rmps.app.ui.internal.pages.design.DesignUtil.createMode+"."+ this.resourceInfo.constructorArgs.rdf_type;
			}else{
				var uri = this.resourceInfo && this.resourceInfo.uri;
				return dojox.encoding.digests.MD5(this.declaredClass + ',' + uri + ',' + this.projectId);			
			}
		}
		return this._id;
	},
	
	getIconUri: function() {
		return  this.resourceInfo.iconUri|| (this.resourceInfo.currentResourceInfo&& DesignUtil.getFileUri(this.resourceInfo.currentResourceInfo.iconUri));
	},

	getName: function() {
		if(this.mode == rmps.app.ui.internal.pages.design.DesignUtil.createMode) {
			var name = this.resourceInfo.name;
			if (!name) {
				return designMessages.loading;
			}
			return name + ' ' + this.__time;
		} else {
			var name = this.resourceInfo.name;
			if (name === undefined) {
				return designMessages.loading;
			}
			return name;
		}
	},

	getTooltip: function() {
		if(this.mode == rmps.app.ui.internal.pages.design.DesignUtil.createMode) {
			return designMessages.createNewResource_tooltip + this.resourceInfo.name;
		} else {
			return this.resourceInfo.tooltip;
		}
	},

	getType: function() {
		return this.resourceInfo.model.rdf_type.rdf_about;
	},
	
	createPresentationUri: function(state, mode) {
		var old = this.mode;
		this.mode = mode;
		var url = this.getPresentationUri(state);
		this.mode = old;
		return url;
	},
	
	getBasePresentationUri: function() {
		if(this.mode == rmps.app.ui.internal.pages.design.DesignUtil.createMode) {
			return DesignUtil.getCreateResourceUri(this.resourceInfo.uri||this.resourceInfo.constructorArgs.rdf_type);
		} else {
			return DesignUtil.getEditResourceUri(this.resourceInfo.uri);
		}
	},
	
	getPresentationName: function() {
		if(this.mode == rmps.app.ui.internal.pages.design.DesignUtil.editMode) {
			// Presentation label is in the form:
			// "<type> <id if numeric>: <label>", e.g. "Book 2450: Harry Potter" (created on server)
			return this.resourceInfo.presentationName;
		}
		return this.inherited(arguments);
	},
	
	hasSidebarExplorer: function() {
		return this.mode == rmps.app.ui.internal.pages.design.DesignUtil.editMode;
	},
	
	getBreadcrumbUri: function(/*String?*/ uri) {
		if (uri) {
			return DesignUtil.getEditResourceUri(uri);
		}
		return DesignUtil.getExploreResourceUri(uri);
	},
	
	setNewResourceURI: function(/* String */ uri) {
		/*
		 * To be called with logical URI of newly created resource.
		 */
		this.__newResourceURI = uri;
	},
	
	getNewResourceURI: function() {
		return this.__newResourceURI;
	},
	
	constructTime: function() {
		var date = new Date();
		var h = date.getHours(), m = date.getMinutes(), s = date.getSeconds();
		if (h < 10) {
			h = "0" + h;
		}
		if (m < 10) {
			m = "0" + m;
		}
		if (s < 10) {
			s = "0" + s;
		}
		return "<" + h + ":" + m + ":" + s + ">";
	},
	
	hasExplorerBreadcrumbs: function() {
		return true;
	}
});

}());
