/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2011. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

/**
 * @author Adam R. Neal
 */
(function(){
	
	dojo.provide("rmps.app.ui.internal.pages.design.DesignLayout");

	dojo.require("rmps.app.ui.internal.pages.design.ExplorerBreadCrumbs");
	dojo.require("rmps.app.ui.internal.pages.design.ContextSelectionMenu");

	dojo.require("dijit._Templated");
	dojo.require("dijit._Widget");
	
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
 
	dojo.declare("rmps.app.ui.internal.pages.design.DesignLayout", [dijit._Widget, dijit._Templated], {

		templatePath: dojo.moduleUrl("rmps.app", "ui/internal/pages/design/templates/DesignLayout.html"),
		
		// attachment points
		_snapShotsNode: null,
		_explorerBreadcrumbs: null,
		_contentNode: null,
		_top: null,
		
		// props
		_input: null,
		_currentContent: null, // String id
		_content: null, // current content
		_sizeCurrent: null,
		_breadcrumbs: null,
				
		constructor: function(params) {
			/* Required */
			this._input = params.input;
			
			this.inherited(arguments);
		},
		
		startup: function() {
			
			this.inherited(arguments);
			
			/*
			 * Bread Crumbs
			 */
			this.initializeExplorerBreadCrumbs();				
		},
		
		destroy: function() {
			if(this._breadcrumbs){
				this._breadcrumbs.destroyRecursive();
				delete this._breadcrumbs;
			}
			if(this.snapshots){
				this.snapshots.destroyRecursive();
				delete this.snapshots;
			}
			if(this._content){
				this._content.destroyRecursive();
				delete this._content;
			}
			this.inherited(arguments);
		},
		
		_clearNode: function(node) {
			if (node && node.hasChildNodes()) {
				while (node.children.length >= 1) {
					node.removeChild(node.firstChild);
				}
			}
		},
		
		initializeExplorerBreadCrumbs: function() {
			this._breadcrumbs = new rmps.app.ui.internal.pages.design.ExplorerBreadCrumbs({
				parent: this
			});
			this.displayExplorerBreadcrumbs(false);
			this._explorerBreadcrumbs.appendChild(this._breadcrumbs.domNode);
		},
		
		displayExplorerBreadcrumbs: function(enabled) {
			dojo.style(this._explorerBreadcrumbs, {display: enabled ? "" : "none"});				
		},
		
		refreshExplorerBreadcrumbs: function(uri) {
			this._breadcrumbs.refresh(uri);
		},
		
		setDesignInput: function(input) {
			// check for dojo.connects before renaming this method
			this.input = input;
		},
		
		addContent: function(content) {
			this._top.appendChild(content.domNode);
			
			if (this._started && !content._started){
				content.startup();
			}
		}
		
    }); // end declare
    
}()); // end function()
