/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("rmps.app.ui.internal.pages.design.DesignPage");

dojo.require("rmps.ui.editor.AbstractPage");
dojo.require("rmps.app.ui.internal.pages.design.DesignPageController");
dojo.require("rmps.app.ui.internal.pages.design.DesignLayout");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.util.ResizeNotifier");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("rmps.ui.explorer.ExplorerSidebarWidget");
dojo.require("rmps.ui.SidebarContainer");
dojo.require("rmps.ui.util.UIUtil");
dojo.require("rmps.core.explorer.ExplorerConstants");
dojo.require("rmps.app.ui.internal.ResourceEditorInput");
dojo.require("rmps.app.ui.internal.editors.RegisterDesignEditorsInitializer");

var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var DesignUtil = rmps.app.ui.internal.pages.design.DesignUtil;
var UIUtil= rmps.ui.util.UIUtil;

//Async called modules
var DesignWelcome= "rmps.app.ui.internal.pages.DesignWelcome";
var DesignDomainWelcome= "rmps.app.ui.internal.pages.DesignDomainWelcome";
var ReviewSearchResults= "rmps.app.ui.internal.search.ReviewSearchResults";
var SearchResults= "rmps.app.ui.internal.search.SearchResults";
var SearchBanner= "rmps.app.ui.internal.search.SearchBanner";

var PAGE_REMOVAL_ID = "com.ibm.xtools.rmps.jfs.web.removeDesignsPage";
var ACTION_NAMES = ["_default","welcome","viewResource","search","create","explore","importMetamodel",
                    "importOwl","edit","viewpoint","viewpoints","query","queries","clipboard","validation"];

dojo.requireLocalization("rmps.app.ui.internal.pages", "DesignPageMessages");
var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages", "DesignPageMessages");

/**
 * @author Adam R. Neal
 */
dojo.declare("rmps.app.ui.internal.pages.design.DesignPage", rmps.ui.editor.AbstractPage, {

	"class": "rmps-app-ui-internal-pages-design-DesignPage",
	
	// private properties
	_controller: null,
	_layout: null,
	_explorer: null,
	_sidebar: null,
	_explorerEditorInput: null,
	
	createPageContents: function() {
		this.inherited(arguments);
		
		// Create the explorer and its side bar
		this._explorer = new rmps.ui.explorer.ExplorerSidebarWidget({
			explorerID: rmps.core.explorer.ExplorerConstants.IDs.EXPLORER_SIDEBAR_DESIGN_PAGE
		});
		this._sidebar = new rmps.ui.SidebarContainer();
		this._sidebar.addSidebar(this._explorer, true /*isLeft*/, false /*isOpen*/, true /*preserveState*/);
		
		this.addChild(this._sidebar);
		this.subscribe("rmps.app.ui.internal.pages.ShowSidebar", this.showSidebar);
		
	},
	
	adjustSidebar: function(size) {
		// Intentionally left as a stub.  See WI33654
	},
	
	/*Side navigation is the owing view */
	getView: function(){
		return this._sidebar;
	},
	
	/*The appending view must be the content container for the side navigation */
	getContainerNode: function(){
		return this._sidebar.domNode;
	},
	
	getPageRemovalId: function(){
		return PAGE_REMOVAL_ID;
	},
	
	subscribeActions: function(){
		// create controller and register actions
		this._controller = new rmps.app.ui.internal.pages.design.DesignPageController(this);
		this.subscribeAllActions(this._controller);	
	},
	
	registerActions: function(){
		this.registerActionsList(ACTION_NAMES);
	},
	
	getPageId: function(){
		return DesignUtil.PAGE_ID;
	},
	
	getDefaultUri: function(){
		return DesignUtil.getExploreResourceUri();
	},

	showSidebar: function(message) {
		if (message.explorerID !== rmps.core.explorer.ExplorerConstants.IDs.EXPLORER_SIDEBAR_DESIGN_PAGE) {
			return;
		}
		
		if (!this._sidebar.openSidebars(true)){
			dojo.publish("rmps.ui.explorer.Explorer.loaded", [{
				explorerID: message.explorerID
			}]);
		}
	},
	
	showWelcome: function() {
		if (!this._welcome) {
			rmps.app.ui.internal.util.DomainRegistryUtil.isCurrentProjectDSP(
				dojo.hitch(this, function(isDsp, isSystem) {
					//Load asynchronously. There is no point to load both dialogs as we are going to use only one based on the project type					
					UIUtil.loadAndRun(isDsp? DesignDomainWelcome: DesignWelcome, null, dojo.hitch(this, function(instance){
							this._welcome = instance;
							
							var pageHelper= this.getPageHelper();
							pageHelper.getEditorView().addChild(this._welcome);
							
							this._welcome.connect(pageHelper,"onEditorActivated", dojo.hitch(this, function () {
								this._welcome.domNode.style.display= 'none';
							}));
							this._showWelcome();
						})
					);
					return;
				})
			);
		} else {
			this._showWelcome();
		}
	},
	
	_showWelcome: function(){
		if(this._reviewSearchResults){
			this._reviewSearchResults.domNode.style.display='none';
		}
		if(this._searchResults){
			this._searchResults.domNode.style.display='none';
		}
		
		this.getPageHelper().hideActiveEditor();
		this._welcome.domNode.style.display= '';
		
		if (this._layout) {
			this._layout.displayExplorerBreadcrumbs(false);	
		}
		
		net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(this._controller.messages.welcomeToModels);
	},

	showResourceView: function(resourceUri, args) {
		var editorInput = null;
		if (args.type) {
			editorInputDeferred = rmps.ui.editor.EditorRegistry.createEditorInput(args.type, args);
			if (editorInputDeferred) {
				editorInputDeferred.then(dojo.hitch(this, function(editorInput) {
					if(!editorInput){
						editorInput = new rmps.app.ui.internal.ResourceEditorInput(args);
					}
					editorInput.setParams(args);
					this.getPageHelper().openEditor(editorInput);
				}));
			}
		}
	},

	showSearchResults: function(queryResults, preserveFilter) {
		var modules= [SearchBanner];
		var callback;
		if(queryResults instanceof rmps.app.core.internal.query.ClientPagedReviewQueryResult){
			callback= dojo.hitch(this, this._getReviewSearchResults);
			modules.push(ReviewSearchResults);
		}else{
			callback= dojo.hitch(this, this._getSearchResults);
			modules.push(SearchResults);
		}
		if (this._layout) {
			this._layout.displayExplorerBreadcrumbs(false);	
		}
		jazz.core.loader.batch_load_async(modules, dojo.hitch(this, function() {
			if(this._reviewSearchResults &&
					(queryResults instanceof rmps.app.core.internal.query.ClientPagedQueryResult)){
				this._reviewSearchResults.domNode.style.display='none';
			}
			if(this._searchResults &&
					(queryResults instanceof rmps.app.core.internal.query.ClientPagedReviewQueryResult)){
				this._searchResults.domNode.style.display='none';
			}
			if(this._welcome){
				this._welcome.domNode.style.display= 'none';				
			}
			this.getPageHelper().hideActiveEditor();
			this._showSearchResults(queryResults, preserveFilter, callback());
		}));
	},
	
	_getReviewSearchResults: function(){
		if (!this._reviewSearchResults) {
			this._reviewSearchResults = new rmps.app.ui.internal.search.ReviewSearchResults();
			this._reviewSearchResults.startup();
			
			var pageHelper= this.getPageHelper();
			pageHelper.getEditorView().addChild(this._reviewSearchResults);
			
			this._reviewSearchResults.connect(pageHelper,"onEditorActivated", dojo.hitch(this, function () {
				this._reviewSearchResults.domNode.style.display= 'none';
			}));
		}
		return this._reviewSearchResults;
	},
	
	_getSearchResults: function(){
		if (!this._searchResults) {
			this._searchResults = new rmps.app.ui.internal.search.SearchResults();
			this._searchResults.startup();
			
			var pageHelper= this.getPageHelper();
			pageHelper.getEditorView().addChild(this._searchResults);
			this._searchResults.connect(pageHelper,"onEditorActivated", dojo.hitch(this, function () {
				this._searchResults.domNode.style.display= 'none';
			}));
		}
		return this._searchResults;
	},
	
	_showSearchResults: function(queryResults, preserveFilter, searchResults){		
		searchResults.showResults(queryResults, preserveFilter);
		this._getSearchBanner().setQueryResult(queryResults);
		searchResults.domNode.style.display= '';

		this.pageHelper.setSearch(queryResults, queryResults.query.scope.icon);

		this._sidebar.display(true);
		if (this._layout) {
			this._layout.domNode.style.height = "";
		}
		this.adjustSidebar();
	},

	_getSearchBanner: function() {
		if (!this._searchBanner) {
			this._searchBanner = new rmps.app.ui.internal.search.SearchBanner();
			this._searchBanner.startup();
		}
		return this._searchBanner;
	},

	getEditorInput: function(args){
		/*
		 * Use the action to determine which editor should be loaded
		 */
		var action = args.action || DesignUtil.defaultAction;
		
		var projectId = jazz.app.context.get().itemId;
		
		// Create a new input for the potentially new resource
		var deferredEditorInput = new dojo.Deferred();
		var deferredInput = rmps.ui.editor.EditorRegistry.createEditorInput(
				action, // registered based on action
				//Args
				{
					projectId : projectId,
					uri: args.uri,
					constructorArgs: args.constructorArgs,
					viewpoint : args.viewpoint
				});
		
		deferredInput.then(dojo.hitch(this, function(input) {
			if(!this._layout){
				// Initialize our layout
				this._layout = new rmps.app.ui.internal.pages.design.DesignLayout({
					input: input,
					page: this
				});
				
				this._layout.addContent(this.getEditorView());
				
				// When the editor is changed, we want to disable/enable the breadcrumbs depending on if editor should have them or not
				this.connect(this.getPageHelper(), "onEditorActivated", dojo.hitch(this, function(editor) {
					var input = editor.editorInput;
					if (input && input.hasExplorerBreadcrumbs && input.hasExplorerBreadcrumbs()) {
						this._layout.displayExplorerBreadcrumbs(true);
					} else {
						this._layout.displayExplorerBreadcrumbs(false);
					}				
				}));
				
				// When we set a new input, we want to refresh the breadcrumbs so they are in sync with the input's URI
				this.connect(this._layout, "setDesignInput", dojo.hitch(this, function(input) {
					if (input.hasExplorerBreadcrumbs && input.hasExplorerBreadcrumbs()) {
						this._layout.refreshExplorerBreadcrumbs(input.resourceInfo.uri);
					}	
				}));
				this._sidebar.setContent(this._layout);
				this._layout.setDesignInput(input);
			}else{
				this._layout.setDesignInput(input);
			}
			
			deferredEditorInput.callback(input);
		}));
		
		return deferredEditorInput;
	},
	
	showExplore: function(args) {
		if (rmps.core.util.ContextUtils.getCurrentContextURI() == null) {
			if (confirm(messages.noContextIsSet)) {
				window.location.href = rmps.core.util.ContextUtils.getConfigureContextURL();
			} else  {
				window.history.back();
			}
			return;
		}
		
		if(!this._explorerEditorInput){
			var deferredEditorInput = this.openNewInput(args);
			deferredEditorInput.then(dojo.hitch(this, function(input) {
				this._explorerEditorInput = input;
				this._sidebar.display(this._explorerEditorInput.hasSidebarExplorer && this._explorerEditorInput.hasSidebarExplorer());
				this.getPageHelper().openEditor(this._explorerEditorInput);
				if (!this._explorerEditorInput.hasExplorerBreadcrumbs || !this._explorerEditorInput.hasExplorerBreadcrumbs()) {
					if (this._layout) {
						this._layout.displayExplorerBreadcrumbs(false);
					}
				} 
			}));
		}else{
			this._explorerEditorInput.setParams(args);
			this._sidebar.display(this._explorerEditorInput.hasSidebarExplorer && this._explorerEditorInput.hasSidebarExplorer());
			this.getPageHelper().openEditor(this._explorerEditorInput);
			if (!this._explorerEditorInput.hasExplorerBreadcrumbs || !this._explorerEditorInput.hasExplorerBreadcrumbs()) {
				if (this._layout) {
					this._layout.displayExplorerBreadcrumbs(false);
				}
			} 
		}
	},
	
	showImport: function(args) {
		this.openNewInput(args);
	},
	
	edit: function(args) {
		this.openNewInput(args);
	},
	
	showViewpoint: function(args) {
		this.openNewInput(args);		
	},
	
	showViewpointsList: function(args) {
		this.openNewInput(args);
	},
	
	showQuery: function(args) {
		this.openNewInput(args);
	},
	
	showClipboard: function(args) {
		this.openNewInput(args);
	},
	
	showQueriesList: function(args) {
		this.openNewInput(args);
	},
	
	showValidation: function(args) {
		this.openNewInput(args);
	},	
	
	openNewInput: function(args) {
		this.getPageHelper().clearLastViewed();
		var deferredInput= this.getEditorInput(args);
		var deferredOpenInput = new dojo.Deferred();
		
		deferredInput.then(dojo.hitch(this, function(input) {
			input.setParams(args);
			
			this._sidebar.display(input.hasSidebarExplorer && input.hasSidebarExplorer());
			
			// Open the new editor
			this.getPageHelper().openEditor(input);
			deferredOpenInput.callback(input);
		}));
		
		return deferredOpenInput;
	},
	
	resize: function(size) {
		// Intentionally left blank.  See WI33654
	}
	
});
}());