/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.pages.design.DesignPageController");

dojo.require("rmps.core.query.QueryClient");
dojo.require("rmps.core.error");
dojo.require("rmps.core.query.SearchQuery");
dojo.require("rmps.app.core.internal.util.QueryUtil");
dojo.require("rmps.app.core.internal.query.TextQuery");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("rmps.app.core.internal.query.ClientPagedQueryResult");
dojo.require("rmps.app.core.internal.query.ClientPagedReviewQueryResult");

var QueryClient = rmps.core.query.QueryClient;
var QueryUtil = rmps.app.core.internal.util.QueryUtil;
var PlatformUI = net.jazz.ajax.ui.PlatformUI;

dojo.requireLocalization("rmps.app.ui.internal.pages", "DesignPageMessages");

/**
 * @author Adam R. Neal
 */
dojo.declare("rmps.app.ui.internal.pages.design.DesignPageController", null, {
	
	messages: dojo.i18n.getLocalization("rmps.app.ui.internal.pages", "DesignPageMessages"),
	
	// private properties
	_designPage: null,
	_defaultUrl: null,
	_prevQueryResult: null,
	_searchDeferred: null,

	constructor: function(designPage) {
		this._designPage = designPage;
	},

	_default: function(args) {
		if (this._defaultUrl) {
			setTimeout(dojo.hitch(this, function() {
				window.location.href = this._defaultUrl;
			}), 0);
			return;
		}
		this._designPage.showExplore(args);
	},
	
	welcome: function() {
		this._designPage.showWelcome();
		this._defaultUrl = window.location.href;
	},

	viewResource: function(args) {
		var reviewPath = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "reviews");
		if (!args || !args.resourceUri/* || this._defaultUrl == window.location.href*/) {
			return;
		}
		args.mediator= this.mediator;

		this._defaultUrl = window.location.href;

		if (args.resourceUri.indexOf(reviewPath) !== -1 && !args.type){
			args.type = "reviewEditor";
		}
		this._designPage.showResourceView(args.resourceUri, this._getEditorArgs(args));
	},

	search: function(args) {
		if (!args || !args.query) {
			return;
		}

		if (this._defaultUrl === window.location.href) {
			if (this._prevQueryResult) {
				this._designPage.showSearchResults(this._prevQueryResult, true, args);
			}
			return;
		}

		this._defaultUrl = window.location.href;

		var timestamp = args.ts;
		var queryParams = dojo.queryToObject(args.query);

		var query;
		var pagedQueryResultType = rmps.app.core.internal.query.ClientPagedQueryResult;
		if (queryParams.queryUri) {
			query = new rmps.core.query.SearchQuery(queryParams);
		} else {
			query = new rmps.app.core.internal.query.TextQuery(queryParams);
			if ( query.scope === QueryUtil.SCOPE.REVIEWS ) {
				pagedQueryResultType = rmps.app.core.internal.query.ClientPagedReviewQueryResult;
			}
		}

		query.timestamp = timestamp;
		if (timestamp === null) {
			timestamp = new Date().getTime();
		}
		
		if (this._prevQueryResult && query.equals(this._prevQueryResult.query) &&
				(this._prevQueryResult.query.timestamp === query.timestamp || query.timestamp === null)) {
			this._designPage.showSearchResults(this._prevQueryResult, true, args);

		} else {
			var preserveFilter = true;
			if (!this._prevQueryResult || !query.token) {
				preserveFilter = false;
			}

			PlatformUI.getWorkbench().showMessage(
				dojo.string.substitute(this.messages.runningQuery, [query.name]));

			if (this._prevQueryResult && this._prevQueryResult.id === query.token) {
				this._prevQueryResult = new pagedQueryResultType({
					query: query,
					id: this._prevQueryResult.id,
					rawEntries: this._prevQueryResult.rawEntries,
					totalResults: this._prevQueryResult.totalResults,
					uri: this._prevQueryResult.uri,
					filtered: this._prevQueryResult.filtered
				});
				this._designPage.showSearchResults(this._prevQueryResult, preserveFilter, args);
				PlatformUI.getWorkbench().hideMessageArea();

			} else {
				var queryToExecute = dojo.clone(query);
				delete queryToExecute.page;
				delete queryToExecute.pageSize;
				delete queryToExecute.orderBy;
				delete queryToExecute.token;

				if (this._searchDeferred) {
					this._searchDeferred.cancel();
					delete this._searchDeferred;
				}
				
				var validationResults = queryToExecute.validate();
				
				if (validationResults && validationResults.length > 0) {
					var validationDetailsMessage = "";
					
					for (var i = 0; i<validationResults.length; i++) {
						validationDetailsMessage +=validationResults[i]+"\n";
					}
					
					rmps.core.error.create(this.messages.error_invalidQuery, validationDetailsMessage, null).dispatch();
					PlatformUI.getWorkbench().hideMessageArea();
				} else {
					this._searchDeferred = QueryClient.execute(queryToExecute);
					this._searchDeferred.addCallback(this, function(queryResult) {
						if (queryResult instanceof Object) {
							this._prevQueryResult = new pagedQueryResultType({
								query: query,
								id: queryResult.id,
								rawEntries: queryResult.entries,
								totalResults: queryResult.totalResults,
								uri: queryResult.uri
							});

							this._designPage.showSearchResults(this._prevQueryResult, preserveFilter, args);
						} else {
							this._prevQueryResult = null;
							window.location.href = queryResult;
						}
					});

					this._searchDeferred.addErrback(this, function(error) {
						if (error.dojoType !== "cancel") {
							if (!dojo._isDocumentOk(this._searchDeferred.ioArgs.xhr)) {
								rmps.core.error.createXhrError(this._searchDeferred.ioArgs).dispatch();
							} else {
								rmps.core.error.createError(error).dispatch();
							}
						}
					});

					this._searchDeferred.addBoth(this, function() {
						delete this._searchDeferred;
						PlatformUI.getWorkbench().hideMessageArea();
					});
				}
			}
		}
	},

	explore: function(args) {
		if (this._defaultUrl === window.location.href) {
			return;
		}
		args = this._getEditorArgs(args);
		
		this._designPage.showExplore(args);
	},
	
	importMetamodel: function(args) {
		if (this._defaultUrl === window.location.href) {
			return;
		}
		args = this._getEditorArgs(args);
		
		this._designPage.showImport(args);
	},
	
	importOwl: function(args) {
		if (this._defaultUrl === window.location.href) {
			return;
		}
		args = this._getEditorArgs(args);
		
		this._designPage.showImport(args);
	},	
	
	validation: function(args) {
		if (this._defaultUrl === window.location.href) {
			return;
		}
		args = this._getEditorArgs(args);
		
		this._designPage.showValidation(args);
	},	
	
	edit: function(args){
		if (this._defaultUrl === window.location.href) {
			var query = dojo.queryToObject(this._defaultUrl);
			if (query.tab === args.tab){
				return;
			}
		}
		
		this._designPage.edit(this._getEditorArgs(args));
	},
	
	create: function(args) {
		args.constructorArgs = {
			rdf_type: args.resource
		};
		args.resource = null;
		this.edit(args);
	},
	
	viewpoint: function(args) {
		if (!args && this._defaultUrl === window.location.href) {
			return;
		}
		args = this._getEditorArgs(args);
		this._designPage.showViewpoint(args);
	},
	
	viewpoints: function(args) {
		if (this._defaultUrl === window.location.href) {
			return;
		}
		args = this._getEditorArgs(args);
		this._designPage.showViewpointsList(args);
	},
	
	query: function(args) {
		if (!args && this._defaultUrl === window.location.href) {
			return;
		}
		args = this._getEditorArgs(args);
		this._designPage.showQuery(args);
	},
	
	queries: function(args) {
		if (this._defaultUrl === window.location.href) {
			return;
		}
		args = this._getEditorArgs(args);
		this._designPage.showQueriesList(args);
	},
	
	clipboard: function(args) {
		if (this._defaultUrl === window.location.href) {
			return;
		}
		args = this._getEditorArgs(args);
		this._designPage.showClipboard(args);
	},
	
	/**
	 * TODO: why does this function modify the arguments of every
	 * call to this class, instead of having callers call it with
	 * the correct arguments in the first place?
	 */
	_getEditorArgs: function(args) {
		var rest = dojo.clone(args);
		delete rest.id;
		
		rest.uri = args.resource;
		delete rest.resource;
			
		return rest;
	}
});

}());
