/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("rmps.app.ui.internal.pages.design.DesignResourceSelection");

dojo.require("rmps.ui.TextFilterInput");
dojo.require("rmps.ui.PagingControl");
dojo.require("rmps.ui.ActionsTable");
dojo.require("jazz.ui.ResourceLink");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("rmps.app.ui.internal.util.DomainRegistryUtil");
dojo.require("dojo.i18n");

var DesignUtil = rmps.app.ui.internal.pages.design.DesignUtil;
var DomainRegistryUtil= rmps.app.ui.internal.util.DomainRegistryUtil;
var BASELINE_PARAM = "rmps.context";

var TABLE_SIZE= 5;

dojo.declare("rmps.app.ui.internal.pages.design.DesignResourceSelection", jazz.ui.Dialog, {
	
	_selectionContents: null,
	_selectionsData: null,
	
	constructor: function(){
		this._selectionsData= [];
	},
	
	postCreate: function(){
		this._selectionContents= new rmps.app.ui.internal.pages.design.DesignResourceSelection._Contents({
			containerDialog: this
		});
		this.primaryTitle= this.designResourceSelectionMessage.selectType;
		this._selectionContents.startup();
		
		this.contentNode= this._selectionContents.domNode; 
		this.inherited(arguments);
		
		DomainRegistryUtil.getCreateableURIs(dojo.hitch(this, function(data){
			this._selectionsData= data;
			this.getContent().table._selectionsData= data;
			
			this._createSelection(this._selectionContents.resourceSelect, this.designResourceSelectionMessage.allDomains, "*");
			
			var domains= [];
			for(var index = 0, entry; entry = data[index]; index++) {
				var domainUri= entry.domainUri;
				if(dojo.indexOf(domains, domainUri)<0){
					domains.push(domainUri);
					this._createSelection(this._selectionContents.resourceSelect, entry.domainLabel|| entry.domainTitle, entry.domainUri);
				}
			}
			
			this._createSelection(this._selectionContents.resourceSelect, this.designResourceSelectionMessage.recentlyUsed, "");
			
			this.getContent().table.fillResourceData();
			this._selectionContents._pagingContents.update(0, TABLE_SIZE, undefined, this._selectionsData.length);
		}));
		
		var table= this.getContent().table;
		this._selectionContents._pagingContents.table= table;
		this._selectionContents._pagingContentsFooter.table= table;
	},
	
	postMixInProperties: function(){
		this.inherited(arguments);
		
		dojo.requireLocalization("rmps.app.ui.internal.pages.design", "DesignResourceSelectionMessages");
		this.designResourceSelectionMessage = dojo.i18n.getLocalization("rmps.app.ui.internal.pages.design", "DesignResourceSelectionMessages");
	},
	
	_createSelection: function(node, label, value){
		var option = document.createElement("option");
		option.appendChild(dojo.doc.createTextNode(label));
		option.value= value;
		node.appendChild(option);
	},
	
	getContent: function(){
		return this._selectionContents;
	},
	
	onOK: function(e) {
		/*To be implemented by their callers */
	},

	onCancel: function(e) {
		/*To be implemented by their callers */
	},
	
	destroy: function() {
		if (this._selectionContents) {
			this._selectionContents.destroyRecursive();
			delete this._selectionContents;
		}
		this.inherited(arguments);
	}
});

dojo.declare("rmps.app.ui.internal.pages.design.DesignResourceSelection._Contents", [dijit._Widget, dijit._Templated], {
	
	"class": "rmps-app-pages-design-resource-selection",
	
	widgetsInTemplate: true,
	tableArgs: null,
	table: null,
	containerDialog: null,
	
	//private
	_selection: null,

	templateString: dojo.cache("rmps.app", "ui/internal/pages/design/templates/DesignResourceSelection.html"),
	
	postCreate: function(){
		this.inherited(arguments);
		
		this.table = new rmps.app.ui.internal.pages.design.DesignResourceSelection._Table({
				tableContainer: this,
				trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
				columns: [{label: "",width: "1%", fieldName: "isSelected", creator: dojo.hitch(this, this.createSelectionCell)},
				        {label: this.messages.type, width: "29%", creator: dojo.hitch(this, this.createTypeCell)},
				        {label: this.messages.domain, width: "70%", creator: dojo.hitch(this, this.createDomainCell)}], 
				showHover: true
		});
		
		this._pagingContents.linked= this._pagingContentsFooter;
				
		this.table.domNode.style.width= "100%";
		this.table.startup();
		this._tableContainer.appendChild(this.table.domNode);
		
		this.onOK= this.containerDialog.onOK;
		this.onCancel= this.containerDialog.onCancel; 
	},
	
	postMixInProperties: function(){
		this.inherited(arguments);
		
		dojo.requireLocalization("rmps.app.ui.internal.pages.design", "DesignResourceSelectionMessages");
		this.messages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages.design", "DesignResourceSelectionMessages");
	},
	
	updatePagingvisibility: function(visibilityValue){
		this._pagingContents.domNode.style.visibility= visibilityValue;
		this._pagingContentsFooter.domNode.style.visibility= visibilityValue;
	},
	
	createSelectionCell: function(td, tr, entry){
		var radioButton= dojo.doc.createElement("input");
		radioButton.type= "radio";
		radioButton.name= "selectionGroup"
		
		radioButton.onclick= dojo.hitch(this, function(activeEntry){
			if(activeEntry){
				this.okCmd.disabled= false;
			}
			this._selection= activeEntry;
		},entry);
		
		td.appendChild(radioButton);
		return td;
	},
	
	createTypeCell: function(td, tr, entry){
		var label= entry.label;
		
		var iconUri= entry.iconUri;
		if(iconUri){
			var img = dojo.doc.createElement("img");
			img.title = label;
			img.style.width= "16px";
			img.style.height= "16px";
			//Required for accessbility. However, there is a fair possiblity to have broken icons, and this 
			//will result on writing the same label beside the main text
			//img.alt= label;
			
			img.src = DesignUtil.getFileUri(iconUri);
			td.appendChild(img);	
		}
		dojo.addClass(this._createResourceLink(entry.uri, true, entry.label, td).domNode, "rmps-app-pages-design-resource-selection-resourceLinkText");
		return td;
	},
	
	createDomainCell: function(td, tr, entry){
		var label= entry.domainLabel;
		if(!label){
			label= entry.domainTitle;
		}
		this._createResourceLink(entry.domainUri, true,  entry.domainLabel||entry.domainTitle, td);
		return td;
	},
	
	_createResourceLink: function(uri, lazyFetch, label, node){
		var presentationUri = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/logical";
		var query = {
				isDomain: true,
				resource: uri
		};
		var baseline = jazz.app.context.get().baseline;
		if (baseline) {
			query[BASELINE_PARAM] = baseline;
		}
		presentationUri += "?" + dojo.objectToQuery(query);
		var resourceLink = new jazz.ui.ResourceLink({
			uri: net.jazz.ajax._contextRoot + "/rendering?resourceUri=" +  encodeURIComponent(uri), 
			presentationUri: presentationUri,  
			lazyFetch : lazyFetch,
			label:  label||this.messages.emptyLabel
		});
		
		this.connect(this, "uninitialize", dojo.hitch(this, function(rl) {
			rl.destroyRecursive();
		},resourceLink));
		
		node.appendChild(resourceLink.domNode);
		resourceLink.domNode.target= "_blank";
		resourceLink.startup();
		return resourceLink;
	},
	
	filterChanged: function(filterText){
		this.table.page= 0;
		this.table.fillResourceData(filterText);
	},
	
	_onResourceTypeChange: function(){
		delete this._selection;
		this.okCmd.disabled= true;
		this.table.page= 0;
		this.table.fillResourceData();
	},
	
	closeDialog: function(){
		this.containerDialog.close();
	},
	
	_onOK: function(e) {
		this.closeDialog();
		if(!this._selection){
			return;
		}
		
		var uri= this._selection.uri;
		
		if(!uri){
			return;
		}
		DomainRegistryUtil.addCreationSelection(uri);
		window.location.href= DesignUtil.getCreateResourceUri(uri);
		this.onOK(this._selection);
	},

	_onCancel: function(e) {
		this.closeDialog();
		this.onCancel();
	},
	
	onOK: function(e) {
		/*To be implemented by their callers */
	},

	onCancel: function(e) {
		/*To be implemented by their callers */
	}
});

dojo.declare("rmps.app.ui.internal.pages.design.DesignResourceSelection._Table", rmps.ui.ActionsTable, {
	
	page: 0,
	size: TABLE_SIZE,
	_selectionsData: null,
	_selectionsUri: null,
	
	fillResourceData: function(filterText){
		if(!filterText){
			filterText= this.tableContainer._filterSearch.getFilterValue();
		}
		var unfilteredTableData= [];
		var uriSelection= this.tableContainer.resourceSelect.options[this.tableContainer.resourceSelect.selectedIndex].value;
		if(!uriSelection){
			var recentlyUsed= DomainRegistryUtil.loadRecentSelections();
			if(recentlyUsed.length>0){
				
				if (!this._selectionsUri) {
					this._selectionsUri = dojo.map(this._selectionsData, function(item, index){
					    return item.uri;
					}); 
				}

				for ( var i = 0; i < recentlyUsed.length; i++) {
					var uri = recentlyUsed[i];
					if (uri) {
						var index = dojo.indexOf(this._selectionsUri, uri);
						if (index > -1) {
							unfilteredTableData.push(this._selectionsData[index]);
						}
					}
				}
			}
		}else if(uriSelection=== "*"){
			unfilteredTableData= dojo.clone(this._selectionsData);	
		}else{
			for(var index = 0, entry; entry = this._selectionsData[index]; index++) {
				if(entry.domainUri=== uriSelection){
					unfilteredTableData.push(entry);
				}
			}
		}
		var tableData= [];
		if(!filterText){
			tableData= unfilteredTableData;
		}else{
			filterText= filterText.toUpperCase();
			for(var index = 0, entry; entry = unfilteredTableData[index]; index++) {
				if(!entry|| !entry.label|| entry.label.toUpperCase().indexOf(filterText)>-1){
					tableData.push(entry);
				}
			}
		}
		var totalLength= tableData.length;
		this.tableContainer._pagingContents.update(this.page, TABLE_SIZE, undefined, totalLength);
		this.tableContainer.updatePagingvisibility(totalLength>TABLE_SIZE?"":"hidden");
		
		var startIndex= this.page*this.size;
		var lastIndex= startIndex+ this.size;
		var totalLength= tableData.length;
		if(lastIndex>totalLength){
			lastIndex= totalLength;
		}
		
		tableData= tableData.slice(startIndex,lastIndex); 
		this.fillTable(tableData);
	},
	
	addRow: function(dataEntry){
		var row = this.inherited(arguments);
		this.connect(row, "onclick", dojo.hitch(this, function(selectedRow){
			this.handleRowSelection(selectedRow);
		}, row));
		
		this.connect(row, "ondblclick", dojo.hitch(this, function(selectedRow){
			this.handleRowSelection(selectedRow);
			this.tableContainer._onOK();
		}, row));
		return row;
	},
	
	handleRowSelection: function(selectedRow){
		//No need to check for NPE
		var select= this.getRowInfomation(selectedRow).getFieldWidget("isSelected").getElementsByTagName("input")[0];
		
		if(select){
			select.checked= true;
			select.onclick();	
		}
	}
});

dojo.declare("rmps.app.ui.internal.pages.design.DesignResourceSelection._PagingControl", rmps.ui.PagingControl, {

	table: null,
	linked: null,

	update: function(page, pageSize, limit, size, item) {
		this.inherited(arguments);
		if (this.linked) {
			this.linked.update(page, pageSize, limit, size, item);
		}
	},

	onNextClick: function(e) {
		this.inherited(arguments);
		this.table.page++;
		this.table.fillResourceData();
	},

	onPreviousClick: function() {
		this.inherited(arguments);
		this.table.page--;
		this.table.fillResourceData();
	}

});

}());
