/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2014.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.pages.design.DesignTabbedEditor");

dojo.require("dojo.string");
dojo.require("dojo.i18n");

dojo.require("rmps.app.ui.internal.editors.EditorWithComments");

dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");

dojo.require("rmps.core.Document");
dojo.require("rmps.core.RootServices");
dojo.require("rmps.core.client");

dojo.require("rmpw.forms.ui.FormPage");

dojo.require('jazz.util.html');

dojo.require("dojox.xml.DomParser");

dojo.requireLocalization("rmps.app.ui.internal.pages.design", "ValidationMessages");
dojo.requireLocalization("rmps.app.ui.internal.pages.design", "DesignMessages");

dojo.require("rmps.core.explorer.ExplorerConstants");

dojo.require("rmps.ui.explorer.ExplorerUtilities");
var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;

dojo.require("rmps.core.clipboard.Clipboard");
var clipboard = rmps.core.clipboard.clipboard;

var DesignUtil = rmps.app.ui.internal.pages.design.DesignUtil;

var PAGE_UAKEY = "com.ibm.xtools.rmps.web.app.page_DesignTabbedEditor001";

var designMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages.design", "DesignMessages");
var validationMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages.design", "ValidationMessages");
var REDACT_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "redact");

dojo.declare("rmps.app.ui.internal.pages.design.DesignTabbedEditor", rmps.app.ui.internal.editors.EditorWithComments, {
	/*
	 *  This is the Design editor which contains multiple tabs for allowing edit/viewing 
	 *  of a given resource's properties.
	 * 
	 *  A DesignEditor contains an editor input which allows us to access 'this.editorInput.resoruceInfo' which
	 *  has specific information about the current resource being edited.
	 *  
	 *  Tabs contains content which can provide API such as:
	 *  initializeToolBar(jazz.ui.SimpleToolbar toolbar, function setDirty, function enableRefresh)
	 *      add specific widgets to the toolbar
	 *      given a hook for changing the dirty flag of the editor
	 *      given a hook for changing the refresh button enable status
	 *      
	 *  doSave(rmps.core.TaskMonitor monitor)
	 *      perform the save action
	 *  
	 *  doRefresh()
	 *      perform the refresh action
	 *  
	 *  validate(Array errors)
	 *      perform any specific validation logic, append new errors to the 'errors' parameter
	 *      
	 *  Each tab will be constructed with access to this editor via an 'editor' parameter.
	 */
	editorInput: null,
	lockButton: null,
	designMessageBox: null,  // interesting?
	resourceState: null,
	_validationMessageBox: null,
	uaKey: PAGE_UAKEY,
	
	postCreate: function() {
		this.isReadOnly = this.editorInput.isReadOnly();
		
		this.resourceInfo = {
				appName: 'DTK Editor', 
				uri: this.editorInput._params.uri,
				mainElement: this.editorInput._params.uri,
				creationTime: Date(),
				conceptUri: this.editorInput.resourceInfo.conceptUri
		};
		if(this.editorInput.resourceInfo.model) {
			this.resourceInfo.shortName = this.editorInput.resourceInfo.name;
			this.resourceInfo.typeName = this.editorInput.resourceInfo.model.rdf_type[0].rdf_about;
		}
		// hook dirty flag on editor
		dojo.connect(this.editorInput.resourceInfo.repository, "onChange", this, "_onModelChanged");

		// hook locking to UI
		this.resourceState = this.editorInput.store.resourceState;
		this.connect(this.resourceState, "beforeLock", this._beforeLock);
		this.connect(this.resourceState, "onLock", this._onLock);
		this.connect(this.resourceState, "onUnLock", this._onUnLock);
		this.connect(this.resourceState, "onLockError", this._onLockError);
		this.connect(this.resourceState, "onUnLockError", this._onUnLockError);
		
		this.inherited(arguments);
		
		if (this._tabContainer._tabs.length === 0) {
			// no tabs - show info message
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: designMessages.noEditorsInfo,
				closable: true
			});
		}
		
		this._validationMessageBox = this.createAndStartMessagebox(this._validationContent, this._validationContent.containerNode);
		
		if (this.editorInput._params.uri) {
			var modelValidationPath = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "modelvalidation") + "?uri=" + encodeURIComponent(this.editorInput._params.uri) + "&projectUri=" + encodeURIComponent(jazz.app.context.get().uri) + "&rmps.context=" + encodeURIComponent(jazz.app.context.get().baseline);
			modelValidationPath += this.appendChangeset();
			var args = {
				headers: {
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "json",
				url: modelValidationPath,
				hasBody: false,
				load: dojo.hitch(this, function(response, ioArgs) {
				
					if (response.result != "ok") {
						this._validationMessageBox.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
							summary: validationMessages.validationProblemsTitle + "<br>" + response.message.replace(/\n/g,"<br>"),
							closable: true
						});

					} else {
						this._validationMessageBox.hideMessage();
					}
					
				})
			};
			
			
			jazz.client.xhrPost(args);
		}
		
		dojo.style(this.domNode, "height", "100%");
		
	},
	
	createAndStartMessagebox: function(node, mainWidget){
		var validationMessageBox = new jazz.ui.MessageBox();

		// We must sent a resize event when the message box is shown/hidden
		// and when the details are toggled.
		function publishResizeEvent() {
			dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
		}
		function onHideMessage() {
			delete validationMessageBox.__cps;
			publishResizeEvent();
		}

		validationMessageBox.domNode.style.margin = "1px";
		validationMessageBox.domNode.style.width = "100%";
		validationMessageBox._first = true;
		dojo.connect(validationMessageBox, "_toggleDetails", node, publishResizeEvent);
		dojo.connect(validationMessageBox, "hideMessage", node, onHideMessage);
		dojo.connect(validationMessageBox, "showMessage", node, publishResizeEvent);

		/*Place the messagebox at the beginning */
		dojo.place(validationMessageBox.domNode, node, "first");
		validationMessageBox.startup();
		return validationMessageBox;
	},	

	_onModelChanged: function(e) {
		if(e.newStackPosition > 0) {
			this.addDirty("model");
		} else {
			this.removeDirty("model");
		}
	},

	activate: function() {
		this.inherited(arguments);
		this.initializeToolBar();
	},
	
	addTabs: function() {
		var resourceInfo = this.editorInput.resourceInfo;
		if(resourceInfo.model) {
			var hasFocusedElement = (resourceInfo.fragmentName != null || resourceInfo.fragmentTypeLabel != null || (resourceInfo.documentUri && resourceInfo.documentUri !== "" && resourceInfo.documentUri !== resourceInfo.uri));
			var tabs = this.editorInput.getTabs(hasFocusedElement);
			var index, length, tab;
			for (index = 0, length = tabs.length; index < length; ++index) {
				tab = tabs[index];
				this.addTab(tab.id, tab.label, tab.type.widget, {
					editor: this,
					isReadOnly: this.isReadOnly,
					definition: tab
				});
			}
			this.inherited(arguments);
		}
	},
	
	initializeToolBar: function() {
		var selectedTab = this._tabContainer.getSelectedTab();
		if (selectedTab && selectedTab.page.initializeToolBar) {
			// allow additional items
			selectedTab.page.initializeToolBar(
					this.toolbar,
					// TODO (MG): remove tab-by-tab dirtying support when forms commit changes incrementally
					dojo.hitch(this, function(dirty) {
						if(dirty !== false) {
							this.addDirty("model");
						} else {
							this.removeDirty("model");
						}
					}), 
					dojo.hitch(this, "_setRefreshEnabled"));
		}
		

	},
	
	/*
	 * Called when the cached editor is being re-opened. Refresh tabs that need to be refreshed in this case
	 */
	onReopen: function() {
		if (this.commentingWidget && !this.commentingWidget.isDirty()) {
			this.commentingWidget.refresh();
		}
		if (!this.isDirty()) {
			this.refresh();
		}
	},
	
	refresh: function() {
		// This should be uncommented, and the last line deleted, when wi 25926 gets resolved
		// See work item 24950 for details.
		/*dojo.when(this.editorInput.load(),
				dojo.hitch(this, function() {
					// get each tab to refresh its view of the master model
					dojo.forEach(this._tabContainer.getChildren(), function(tab) {
						if(tab.doRefresh) {
							tab.doRefresh();
						} else {
							console.warn("Tab " + tab.declaredClass + " does not support refresh");
						}
					});
				}));*/
		
		this.inherited(arguments);
	},
	
	resize: function( s ){
		if( s ){	
			
			//tma: Why?
			//s.w = 1000;
			
			this.inherited( arguments );
			
			if(this.viewer){
				this.viewer.resize(s);
			}
		}
	},
	
	doSave: function(/*rmps.core.TaskMonitor*/monitor) {
		if (jazz.app.context.get().changeset){
			this._cannotUnlock();
		}
		// TODO (MG): eliminate this special case for FormPage
		var wasFormPage = dojo.some(this._tabContainer.getChildren(), function(tab) {
			if(tab instanceof rmpw.forms.ui.FormPage) {
				// calls ModelCreationUtil.newDomainNoUI(), which does its own xhr
				//Or call for the setter method that creates the change sets for the current form, and handles the required
				//deferred work before commiting the changes
				tab.doSave(monitor);
				return true;
			}
			return false;
		});
		if(wasFormPage) {
			return;
		}
		// end hack //////////////////////////////////////////
		
		this.doSaveDesignEditor(monitor);
	},
	
	doSaveDesignEditor: function(/*rmps.core.TaskMonitor*/monitor){
		this.editorInput.save(monitor);
		this.commentingWidget.doSave(monitor);		
	},
	
	onSaveComplete: function(){
		if (this.editorInput.mode === DesignUtil.createMode) {
			rmps.ui.editor.PageHelper.getInstance(DesignUtil.PAGE_ID).closeEditor(this, true, true);
			
			// notify the explorer to refresh the tree
			dojo.publish("rmps.ui.explorer.Explorer.resourceAdded",
				[{
					uri: this.editorInput.store.documentUri,
				}]);
			
			window.location.href = DesignUtil.getEditResourceUri(this.editorInput.store.documentUri, this.getSelectedTabId());
			return;
		}
		
		var uri;
		if (this.editorInput.resourceInfo.model) {
			uri = this.editorInput.resourceInfo.model.rdf_about;
		}
		this.inherited(arguments); // show OK message
		
		if (uri) {
			//or consider this.editorInput.resourceInfo.model.rdf_about
			var modelValidationPath = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "modelvalidation") + "?uri=" + encodeURIComponent(uri) + "&projectUri=" + encodeURIComponent(jazz.app.context.get().uri) + "&rmps.context=" + encodeURIComponent(jazz.app.context.get().baseline);
			modelValidationPath += this.appendChangeset();			
			jazz.client.xhrPost({
				headers: {
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "json",
				url: modelValidationPath,
				hasBody: false,
				load: dojo.hitch(this, function(response, ioArgs) {
				
					if (response.result != "ok") {
						this._validationMessageBox.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
							summary: validationMessages.validationProblemsTitle + "<br>" + response.message.replace(/\n/g,"<br>"),
							closable: true
						});							
					} else {
						this._validationMessageBox.hideMessage();
					}
				})
			});
		}
	},
	
	getTitleNodeValue: function(){
		return this.editorInput.getPresentationName();
	},

	getTabs: function(message){
		// Need warnings?
		if(this.designMessageBox){
			/*Hide and any previous messages */
			this.designMessageBox.hideMessage();
		}else{
			/*arbitrary assumption, just to make sure that the message box will not be null for next lines */
			this.designMessageBox = this.__createAndStartMessagebox(this._tabContainer, this._tabContainer.containerNode);
		}
		
		this.designMessageBox.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
			summary: message,
			closable: true
		});
	},
	
	/*
	 * @see BaseEditor#validate
	 */
	validate: function(/*Array*/errors) {
		var deferreds = [];
		var tab = this._tabContainer.getSelectedTab();
		if(tab && tab.page && tab.page.validate) {
			var result = tab.page.validate(errors);
			if (result instanceof dojo.Deferred) {
				deferreds.push(result);
			}
		}
		
		if (this.commentingWidget) {
			this.commentingWidget.validate(errors);
		}
		
		return deferreds;
	},
	
	__createAndStartMessagebox: function(node, mainWidget){
		// We must send a resize event when the message box is shown/hidden
		// and when the details are toggled.

		var designMessageBox = new jazz.ui.MessageBox();
		designMessageBox.domNode.style.margin = "1px";
		designMessageBox._first = true;
		
		function publishResizeEvent() {
			dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
		}
		function onHideMessage() {
			if (designMessageBox !== null) {
				delete designMessageBox.__cps;
			}
			publishResizeEvent();
		}
		
		dojo.connect(designMessageBox, "_toggleDetails", node, publishResizeEvent);
		dojo.connect(designMessageBox, "hideMessage", node, onHideMessage);
		dojo.connect(designMessageBox, "showMessage", node, publishResizeEvent);

		/*Place the messagebox at the beginning */
		dojo.place(designMessageBox.domNode, mainWidget, "first");
		designMessageBox.startup();
		return designMessageBox;
	},
	
	_setEnabled: function(button, enabled) {
		if (enabled) {
			dojo.removeAttr(button.domNode, 'disabled');
		} else {
			dojo.attr(button.domNode, 'disabled', true);
		}	
	},
	
	_setRefreshEnabled: function(enabled) {
		this._setEnabled(this.refreshButton, enabled);
	},
	
	addButtons: function(toolbar) {
		//TODO SD create new class ResourceEditor, from this baseeditor from move locking feature there
		if (this.editorInput.resourceInfo !== null) {
			this.lockButton  = toolbar.addButton(this, this.lock, {
				label: designMessages.Lock,
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/lock.gif")
			});
		}
		if (this.editorInput.resourceInfo !== null) {
			var resourceUri = this.editorInput.resourceInfo.documentUri;
			if (resourceUri) {
				var name = this.editorInput.resourceInfo.name !== null ? this.editorInput.resourceInfo.name :resourceUri;
				name = jazz.util.html.escape(name);
				this.resourceState.isLocked(resourceUri).then( 
						dojo.hitch(this, function(lockedBy){
							if (lockedBy === "none") {
								// not locked
								if (this.isReadOnly) {
									// also read-only - means can't do anything with lock icon, remove it
									this.toolbar.removeItem(this.lockButton);
								}
								return; // no actions needed
							}							
							// locked, can unlock
							if (lockedBy !== undefined && (lockedBy === 'self' || lockedBy === 'modified')) {
								if (this.editorInput.store.resourceState) {
									// inform all the listeners that the locking has already happened
									this.editorInput.store.resourceState.onLock(lockedBy);
								} else {
									this._onLock(lockedBy);
								}
							}
						}),
						dojo.hitch(this, function(lockedBy){
							// locked, cannot unlock unless "admin" (has "Force unlock" rights)
							summary = dojo.string.substitute(designMessages.lockedBy, [name,lockedBy]);
							if (lockedBy === 'differentContext'){
								this.toolbar.removeItem(this.lockButton);
								summary = dojo.string.substitute(designMessages.differentContext, [name]);
							} else if (lockedBy.indexOf("notOwner:")>-1){
								this.toolbar.removeItem(this.lockButton);
								summary = dojo.string.substitute(designMessages.notOwner, [lockedBy.substring(lockedBy.indexOf(":")+1)]);								
							} else if (lockedBy.indexOf("snapshot")>-1){
								this.toolbar.removeItem(this.lockButton);
								summary = designMessages.snapshot;								
							} else if (lockedBy.indexOf("admin:")>-1){
								this.lockButton.args.label = designMessages.UnLock
								this.toolbar.updateButton(this.lockButton, {
									label: this.lockButton.args.label,
									imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/unlock.gif")
								});
								var lockingUser = lockedBy.substring(6);
								summary = dojo.string.substitute(designMessages.lockedBy, [name,lockingUser]);
							} else {
								this.toolbar.removeItem(this.lockButton);
							}
							this._resourceWasLockedInitially = true;
							this.showMessage({
								type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
								summary: summary,
								closable: true
							});
						})
				);
			}

			// If resource info is known, allow it to be added to the pinboard
			if (!this._pinboardToolbarButton) {
				this._pinboardToolbarButton = this.toolbar.addButton(this, 
					dojo.hitch(this, this.addToPinboard), 
					{
						label: designMessages.addToPinboard,
						imgSrc: dojo.moduleUrl("rmps.ui", "graphics/icons/add_to_clipboard_co.gif")
					}
				);
			}		
		}

		if (!this._exploreToolbarButton) {
			this._exploreToolbarButton = this.toolbar.addButton(this, 
				dojo.hitch(this, this.exploreInTree), 
				{
					label: designMessages.exploreInTree,
					imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/explore_tree_co.gif")
				}
			);
		}
				
		var args = {
			url: REDACT_PATH + "/enabled",
			headers: {
				"X-ibm-rmps-internal" : "true"
			},
			load:  function(response, ioArgs) {
				//If success, redaction will be enabled and the redaction icon will be shown
				if(ioArgs.xhr.status === 200){
					if (!outerObject._redactAllVersions) {
						outerObject._redactAllVersions = outerObject.toolbar.addButton(outerObject, 
							dojo.hitch(outerObject, outerObject.redactAllVersions), 
							{
								label: designMessages.redactAllVersions,
								imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/redact.gif")
							}
						);
					}
				}
			}							
		}; // end args
		
		var outerObject = this;
		
		//Get value of the variable that enables/disables redaction		
		rmps.core.client.xhrHead(args);		
		
		this.inherited(arguments);
	},
	
	loadResourceCompleted: function() {
		if (this._resourceWasLockedInitially) {
			// keep the resource lock message shown
			delete this._resourceWasLockedInitially;
			return;
		}
		this.hideMessage();
	},
	
	exploreInTree: function() {
		if (this.editorInput && this.editorInput.hasSidebarExplorer()) {
			ExplorerUtil.showItemInSidebarExplorer(this.resourceInfo.uri, rmps.core.explorer.ExplorerConstants.IDs.EXPLORER_SIDEBAR_DESIGN_PAGE);
		} else {
			window.location.href = DesignUtil.getExploreResourceUri(this.resourceInfo.uri);
		}
	},
	
	redactAllVersions: function() {
		var query = {
				conceptURI: this.resourceInfo.uri 
			};
			var args = {
					url: REDACT_PATH + "?" + dojo.objectToQuery(query),
					headers: {
						"X-ibm-rmps-internal" : "true"
					},
					load: dojo.hitch(this, function(response, ioArgs) {
					window.location.href = this.resourceInfo.uri;
				})				
			}; // end args
					
			rmps.core.client.xhrPost(args);
	},
	
	addToPinboard: function() {
		if (this.editorInput.resourceInfo !== null) {
		    var resourceInfo = this.editorInput.resourceInfo;
			clipboard().addCallback(function(cb) {
				cb.push(resourceInfo.uri, resourceInfo.type);								
			});
		}
	},	
	
	getErrorMsg: function(errorObject) {
		if (errorObject) {
			if (errorObject.responseText) {
				// errorObject.responseText is set when RmpsRuntimeException is thrown and processed by the server
				var rdf = dojox.xml.DomParser.parse(errorObject.responseText);
				if (rdf) {
					var msgElement = rdf.getElementsByTagName("err:errorMessage")[0];
					if (msgElement) {
						return msgElement.childNodes[0].nodeValue;
					}
				}
			} else {
				return errorObject;
			}
		}
		
		return designMessages.unknownError;
	},
	
	_onLockError: function(lockedBy, lockedResourceUri, errorMsg) {
		var summary;
		var error;
		if (errorMsg) {
			if (dojo.isObject(errorMsg)){
				error = errorMsg.message
			} else {
				error = errorMsg;
			}
			if (error.indexOf("status:403")>-1){
				summary = designMessages.noPermission;
			} else {
				summary = errorMsg;
			}
		} else {
			var name;
			if (!lockedResourceUri || 
					(lockedResourceUri.indexOf(this.editorInput.resourceInfo.documentUri) === 0)) {
				// specific locked resource uri is not provided, or it matches that of the current document
				name = this.editorInput.resourceInfo.name !== null ? this.editorInput.resourceInfo.name : this.editorInput.resourceInfo.documentUri;
			} else {
				// we have a resource external to the current document
				name = lockedResourceUri;
			}
			name = jazz.util.html.escape(name);
			summary = dojo.string.substitute(designMessages.lockedBy, [name,lockedBy]);
			if (lockedBy === 'differentContext'){
				summary = dojo.string.substitute(designMessages.differentContext, [name]);
			} else if (lockedBy.indexOf("notOwner:")>-1){
				summary = designMessages.notOwner + lockedBy.substring(lockedBy.indexOf(":")+1);
			} else if (lockedBy.indexOf("snapshot")>-1){
				summary = designMessages.snapshot;
			} else if (lockedBy.indexOf("admin:")>-1){
				var lockingUser = lockedBy.substring(6);
				summary = dojo.string.substitute(designMessages.lockedBy, [name,lockingUser]);
			}
		}
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: summary,
			closable: true
		});
	},
	_onUnLockError: function(lockedBy, errorMsg) {
		var msg = errorMsg;
		if (lockedBy.indexOf("cantUnlock:") === 0){
			var resourceName = this.editorInput.resourceInfo.name !== null ? this.editorInput.resourceInfo.name : this.editorInput.resourceInfo.documentUri;
			var changeset = lockedBy.substring(lockedBy.indexOf(":")+1);
			msg = dojo.string.substitute(designMessages.unlockErrorCantUnlock, [resourceName,changeset,"http://www.ibm.com"]);
			
			// disable the unlock button as it cannot be unlocked
			this.toolbar.disableButton(this.lockButton);		

		}
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
			summary: msg,
			closable: true
		});
	},	
	
	_beforeLock: function(resourceUri, arg){
		this.showMessage({
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
			summary: designMessages.locking,
			closable: true
		});
	},
	
	_cannotUnlock: function () {
		this.lockButton.args.label=designMessages.CannotUnlock;
		this.toolbar.updateButton(this.lockButton, {
			label: this.lockButton.args.label,
			imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/unlock.gif")
		});
		this.toolbar.disableButton(this.lockButton);		
	},
	
	_onLock: function(lockedBy) {
		if (this.lockButton !== null) {
			if (lockedBy === 'modified'){
				this._cannotUnlock();
			} else {
				this.lockButton.args.label=designMessages.UnLock;
				this.toolbar.updateButton(this.lockButton, {
					label: this.lockButton.args.label,
					imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/unlock.gif")
				});
				
			}
			var resourceUri = this.editorInput.resourceInfo.documentUri;
			var name = this.editorInput.resourceInfo.name !== null ? this.editorInput.resourceInfo.name :resourceUri;
			name = jazz.util.html.escape(name);
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
				summary: name+designMessages.isLocked,
				closable: true
			});
		}
	},
	_onUnLock: function() {
		if (this.lockButton !== null) {
			this.lockButton.args.label=designMessages.Lock;
			this.toolbar.updateButton(this.lockButton, {
				label: this.lockButton.args.label,
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/lock.gif")
			});
			var resourceUri = this.editorInput.resourceInfo.documentUri;
			var name = this.editorInput.resourceInfo.name !== null ? this.editorInput.resourceInfo.name :resourceUri;
			name = jazz.util.html.escape(name);
			if (!this.saving) {
				//when we are in the middle of a save operation, don't alert the user when unlocking (28503)
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: name+designMessages.isUnLocked,
					closable: true
				});
			}
		}
	},
	
	lock: function() {
		if (this.lockButton !== null) {
			var resourceUri = this.editorInput.resourceInfo.documentUri;
			if (this.lockButton.args.label === designMessages.Lock) {
				this.resourceState.lock(resourceUri);
			} else {
				this.resourceState.unlock(resourceUri);
			}
		}
	}
});

}());
