/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

/**
 * @author Adam R. Neal
 */
(function(){
	
	dojo.provide("rmps.app.ui.internal.pages.design.ExploreContent");
	
	dojo.require("dojo.cookie");
	
	dojo.require("dijit._Templated");
	dojo.require("dijit._Widget");
	dojo.require("dijit.form.DropDownButton");
	dojo.require("dijit.Menu");
	dojo.require("dijit.MenuItem");
	dojo.require("dijit.Toolbar");
	
	dojo.require("jazz.ui.toolbar.TwoStateButton");
	
	dojo.require("rmps.core.clipboard.Clipboard");
	dojo.require("rmps.core.explorer.ExplorerTreeService");
	dojo.require("rmps.core.user.UserClient");
	dojo.require("rmps.core.explorer.ExplorerConstants");
	
	dojo.require("rmps.ui.clipboard.ClipboardDialog");
	dojo.require("rmps.ui.common.SimpleToolbarButtonGlow");
	dojo.require("rmps.ui.explorer.ExplorerStore");
	dojo.require("rmps.ui.explorer.ExplorerTooltipStore");

	dojo.require("rmps.app.core.util");
	
	dojo.require("rmps.core.explorer.ExplorerMainState");
	dojo.require("rmps.app.ui.internal.pages.design.ExplorerWithGrid");
	dojo.require("rmps.app.ui.internal.pages.design.widgets.ViewpointSelector");
	
	var UserClient = rmps.core.user.UserClient;
	var CoreUtil = rmps.app.core.util;
	var clipboard = rmps.core.clipboard.clipboard;
	var explorerMainState = rmps.core.explorer.explorerMainState;
	
	var EXPLORER_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "explorer");
	var EXPLORER_TREE_PATH = EXPLORER_PATH + "/tree";
	
	var DETAILS_COOKIE_NAME = "rmps.app.ui.internal.pages.design.ExploreContent.Details";
 
	dojo.require("rmps.ui.explorer.ExplorerUtilities");
	var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;
	
	dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
	var designUtil = rmps.app.ui.internal.pages.design.DesignUtil;
		
	dojo.require("rmps.core.group.GroupClient");
	var GroupClient = rmps.core.group.GroupClient;
	
	dojo.require("rmps.core.explorer.ViewpointClient");
	var viewpointClient = rmps.core.explorer.ViewpointClient;

	dojo.require("dojo.i18n");
	dojo.requireLocalization("rmps.ui.explorer", "ExplorerMessages");
	var messages = dojo.i18n.getLocalization("rmps.ui.explorer", "ExplorerMessages");
	
	dojo.declare("rmps.app.ui.internal.pages.design.ExploreContent", [dijit._Widget], {
		/*
		 * This is the main Explorer widget in the Design Page.
		 */

		/*
		 * The ExplorerStore
		 */
		_explorerStore: null, // will be initialized when the explorer is created/refreshed
		_refreshhandle: null,
				
		_clipboardButton: null,
		_clipboardGlow: null,
		
		_viewpointURI: null,
		
		_showDetailedAttributes: !!dojo.cookie(DETAILS_COOKIE_NAME),
		explorerID: rmps.core.explorer.ExplorerConstants.IDs.EXPLORER_FULL_PAGE,
		
		
		/*
		 * Mixin property - the editor
		 */
		parent: null,
		
		readyToFocus: false,
				
		destroy: function() {
			if (this._refreshhandle) {
				dojo.unsubscribe(this._refreshhandle);	
			}
			
			if (this._focusHandle) {
				dojo.unsubscribe(this._focusHandle);
			}
			
			if (this._clipboardGlow) {
				this._clipboardGlow.destroy();
				delete this._clipboardGlow;
			}
			
			this._deleteExplorerStore();
			
			this.inherited(arguments);
		},
		
		startup: function() {
			this.inherited(arguments);
			
			var self = this;
			this._getHistoryFromServer().addCallback(function(ems) {
				self.readyToFocus = true;
				self._initializeContent(false, ems);
			});
		},
		
		initializeToolBar:  function(/* jazz.ui.SimpleToolbar */ toolbar) {
			this._toolbar = toolbar;
			this._backButton = toolbar.addButton(this,
					dojo.hitch(this, function(){
						this._explorer.onBack();
					}),
					{	label: messages.back,
						imgSrc : dojo.moduleUrl("rmps.ui", "graphics/icons/previous.gif")
					}
			);

			this._forwardButton = toolbar.addButton(this,
					dojo.hitch(this, function() {
						this._explorer.onForward();
					}),
					{	label: messages.forward,
				imgSrc : dojo.moduleUrl("rmps.ui", "graphics/icons/next.gif")
					}
			);

			toolbar.addButton(this,
					dojo.hitch(this, function() {
						this._explorer.onCollapseAll();
					}),
					{	label: messages.collapseAll,
						imgSrc: dojo.moduleUrl("rmps.ui", "graphics/icons/collapse_all.gif")
					}
			);
			toolbar.addSeparator();
			var detailsButton = new jazz.ui.toolbar.TwoStateButton({
				label: messages.showAttributes,
				title: messages.showAttributesTitle,
				titlePressed: messages.showAttributesTitlePressed,
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/show_attributes_co.gif"),
				pressed: this.getDetailedEnabled(),
				onClick: dojo.hitch(this, function() {
					this.setDetailedEnabled(detailsButton.get("pressed"));
				})
			}); 
			toolbar.addWidget(detailsButton);
			toolbar.addSeparator();
			this._viewpointSelector = new rmps.app.ui.internal.pages.design.widgets.ViewpointSelector({
				
				explorerID: this.explorerID,
				
				onViewpointChange: dojo.hitch(this, function(viewpoint) {
					this._saveHistory(viewpoint.uri).addBoth(function(){
						window.location = CoreUtil.getShowViewpointUri(viewpoint.uri);
					});							
				}),
				onGetViewpoint: function(ems) {
					return ems.getCurrentViewpoint(this.explorerID);
				}
			});
			toolbar.addWidget(this._viewpointSelector);
			toolbar.addSeparator();
			
			this._clipboardButton = toolbar.addButton(this,
				dojo.hitch(this, function() {
					new rmps.ui.clipboard.ClipboardDialog({
						readOnly: true
					});
				}),
				{
					label: messages.clipboard,
					imgSrc: dojo.moduleUrl("rmps.ui", "graphics/icons/clipboard_empty_co.gif")
				}
			);
			
			this._clipboardGlow = new rmps.ui.common.SimpleToolbarButtonGlow({
				button: this._clipboardButton,
				eventName: "rmps.core.clipboard.Change"
			});
			clipboard().addCallback(this, function(cb) {
				var glowFn = dojo.hitch(this, function(node) {
					// Clipboard updates should change button icon
					if (this._toolbar && this._clipboardButton) {
						this._toolbar.updateButton(this._clipboardButton, {
							imgSrc: cb.getIconURL()
						});
					}
				});
				this._clipboardGlow.onGlow = glowFn;
				// And make an immediate update
				glowFn();
			});
			
			toolbar.addSeparator();		
		},
		
		_setEnabled: function(button, value) {
			if (value) {
				this._toolbar.enableButton(button);
			} else {
				this._toolbar.disableButton(button);
			}
		},
		
		updateToolbar: function() {
			if (this._explorer) {
				this._setEnabled(this._backButton, this._explorer._canGoBack());
				this._setEnabled(this._forwardButton, this._explorer._canGoForward());
			}
		},
		
		refresh: function() {
			explorerMainState(true /* bForceRefresh */).addCallback(this, function(ems) {
				this._initializeContent(true /* recreate */, ems);
			});
			
			if (this._viewpointSelector) {
				this._viewpointSelector.refreshState();
			}
		},
		
		setViewpointURI: function(viewpointURI) {
			this._viewpointURI = viewpointURI;
		},
		
		getDetailedEnabled: function() {
			return this._showDetailedAttributes;
		},
		
		setDetailedEnabled: function(enabled) {
			this._showDetailedAttributes = enabled;

			this.refresh();
			
			// Save a cookie
			if (enabled) {
				dojo.cookie(DETAILS_COOKIE_NAME, "true", {expires: 90});
			} else {
				// delete cookie
				dojo.cookie(DETAILS_COOKIE_NAME, null, {expires: -1});
			}
		},
		
		_initializeContent: function(/*bool*/ recreateExplorerStore, ems) {
         /*
          * Create the Explorer
          * @param loadData
          *   special parameters can be specified for initializing the explorer
          *   can contain { uri, node, recreate, viewpoint )
          */
			var editorInputParams = this.parent.editorInput.getParams();
			
			var loadData = {
				uri: jazz.app.context.get().itemId,
				recreate: !!recreateExplorerStore,
				viewpoint: editorInputParams.viewpoint || ems.getCurrentViewpointURI(this.explorerID)
			};
			
			if (loadData.viewpoint){
				this.setViewpointURI(loadData.viewpoint);
			} else {				
				this.setViewpointURI(null);
			}
			
			if (!this._refreshhandle) {
				this._refreshhandle = dojo.subscribe("rmps.web.app.explorer.design.refresh", this, this.refresh);
			}
			if (!this._focusHandle) {
				this._focusHandle = dojo.subscribe("rmps.web.app.explorer.design.focus", this, this.focus);				
			}			 
			if (editorInputParams.uri) {
				var handle = dojo.subscribe("rmps.web.app.explorer.design.created", dojo.hitch(this, function() {
					dojo.unsubscribe(handle);
					this.showInTree(editorInputParams.uri);	
				}));
			}
			
			this._createExplorerInstance(loadData, ems);
		},
		
		_createExplorer: function (explorerstore, data, ems) {
			/*
			 * Create a new explorer instance
			 */
			var topNodeIds = dojo.map(data, function (item) {
				return explorerstore.getIdentity(item);
			});
			
			var viewpointURI = ems.getCurrentViewpointURI(this.explorerID);
			if (!this._history.byVP) {
				this._history.byVP = {};
			}
			if (!this._history.byVP[viewpointURI]) {
				this._history.byVP[viewpointURI] = this.getEmptyHistory(); 
			}
			
			return new rmps.app.ui.internal.pages.design.ExplorerWithGrid({
				hiddenTopNodes: [],
				_exploreContent: this,
				store: explorerstore,
				topItems: data,
				topNodes: topNodeIds,
				visibleTopNodes: topNodeIds,
				tooltipStore: new rmps.ui.explorer.ExplorerTooltipStore({
					store: explorerstore
				}),
				_history : this._history.byVP[viewpointURI],
				uriToShow : this.parent.editorInput.getParams().uri
			});
		},
		
		_deleteExplorerStore: function() {
			if (this._explorerStore) {
				this._explorerStore.cleanCache();
				delete this._explorerStore;
		}				
		},
		
		_createExplorerStore: function(recreate, viewpointURI) {			
			if (recreate) {
				this._deleteExplorerStore();
			}
			
			var store = this._explorerStore;
			
			if (!store) {
				store = new rmps.ui.explorer.ExplorerStore({
					viewpoint: viewpointURI,
					explorerID: this.explorerID 
				});
				this._explorerStore = store;
			}
			
			return store;
		},
			
		_createExplorerInstance: function(loadData, ems) {
			/*
			 * Create a new explorer instance.  
			 * @param loadData  - an object that must contain either 
			 * a viewpoint and/or the context uri to initialize with
			 * {
			 *   uri: "someUri"
			 *   viewpoint: "someViewpointUri"
			 * } 
			 */
			
			// One instance of the explorer store/model so that we reuse cached entries
			var store = this._createExplorerStore(loadData.recreate, loadData.viewpoint);
			
			// Destroy if necessary
			if (this._explorer) {
				this._explorer.destroyRecursive(true);
				this._explorer = null;
			}

			var self = this;
			var def = store.loadTopNodes(loadData.uri, loadData.node, loadData.viewpoint);
			if (this.parent) {
				this.parent.showLoading();
			}
			def.addCallback(function(data){
				self._explorer = self._createExplorer(store, data, ems);
				
				// Append the explorer
				var node = self.domNode;
				// Remove existing explorer
				if (node.hasChildNodes()) {
					while (node.children.length >=1){
						node.removeChild(node.firstChild);
					}
				}
				node.appendChild(self._explorer.domNode);
				if (self._explorer && self._explorer._tree) {
					self._explorer._treeInitialization = true;
					self._explorer._tree.startup();
				}
				self.parent.hideMessage(); // clear showLoading();
				self.updateToolbar();
			});
		},
		
		_saveHistory : function(viewpointURI) {
			this.updateToolbar();
			if (viewpointURI) {
				this._history.viewpointURI = viewpointURI;	
			}
			var def = new dojo.Deferred();
			if (this._history) {
				var defEms = explorerMainState();
				defEms.addCallback(dojo.hitch(this, function(ems) {
					ems.set(this._history, this.explorerID).then(def.callback, def.errback);
				}));
				defEms.addErrback(function() {
					def.errback();
				});				
			} else {
				def.errback();
			}
			return def;		
		},
		
	    _getHistoryFromServer: function() {
			var def = explorerMainState();
			def.addCallback(dojo.hitch(this, function(ems){
				var value = ems.get(this.explorerID);
				if (value) {
					this._history = value;
				} else {
					this.initHistory();
				}
			}));
			def.addErrback(dojo.hitch(this, function() {
				this.initHistory();
			}));
			return def;
	    },
	    
		initHistory: function() {
			if (this._history) {
				delete this._history;
			}
			this._history = {
				byVP: {}
			}; 
	    },
	    
	    getEmptyHistory: function() {
			return {
				depth: 0,
				stack: []
			};
	    },
	    
	    showInTree: function(uri) {
			var chainDef = ExplorerUtil.getParentChain(uri, this._viewpointURI);
			var complete = new dojo.Deferred();
			chainDef.then(
				dojo.hitch(this, function(value){
					var done = this._explorer.expandChain(value);
					done.then(function(res){
						complete.resolve(res);
					});
				}),
				function() {
					console.log("Parent chain error.");
				});
			return complete;
		},
		
		focus: function(msg) {
			var chainDef = ExplorerUtil.getManyParentChains(msg.uris, true, this._viewpointURI);
			chainDef.then(
					dojo.hitch(this, function(chains){
						var def = this._explorer.expandManyChains(chains);
						var createEllipsis = dojo.hitch(this, function (items) {
							designUtil.focusingResults = false;
							this.focusOnItems(items);
						});
						def.then(createEllipsis, createEllipsis);
					}),
					function() {
						console.log("Parent chain error.");
					});
		},
		
		focusOnItems: function(items) {
			var i;
			for (i = 0; i < items.length; i++) {
				this._explorer.createParentEllipsis(items[i]);
			}
		}		
    }); // end declare
    
}()); // end function()
