/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2011. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

/**
 * @author Adam R. Neal
 */
(function() {
dojo.provide("rmps.app.ui.internal.pages.design.ExplorerBreadCrumbs");

dojo.require("rmps.app.ui.internal.pages.design.BreadCrumbs");
dojo.require("rmps.app.ui.internal.search.SearchBanner");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.ui.explorer", "ExplorerMessages");
var messages = dojo.i18n.getLocalization("rmps.ui.explorer", "ExplorerMessages");

dojo.requireLocalization("rmps.app.ui.internal.pages.design", "BreadCrumbMessages");
var bcMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages.design", "BreadCrumbMessages");

dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
var DesignUtil = rmps.app.ui.internal.pages.design.DesignUtil;

// Mixin class to add breadcrumb navigation
dojo.declare("rmps.app.ui.internal.pages.design.ExplorerBreadCrumbs", [rmps.app.ui.internal.pages.design.BreadCrumbs], {
	
	templatePath: dojo.moduleUrl("rmps.app", "ui/internal/pages/design/templates/ExplorerBreadCrumbs.html"),
	templateString: null,
	/*
	 * Clients should mixin the function
	 * refreshContentsPane(uri, node)  in order to be notified when a breadcrumb has been selected.  
	 */
	
	constructor: function(args) {
		this.inherited(arguments);
	},
	
	postCreate: function() {
		this.inherited(arguments);
		this._searchBanner = new rmps.app.ui.internal.search.SearchBanner({
			getBreadCrumbLabel: function() {
				return bcMessages.searchResultsFor + this._queryResult.query.name;
			}
		});		
		this.searchBanner.appendChild(this._searchBanner.domNode);
		this._searchBanner.startup();
	},
	
	refreshSearchBanner: function() {
		var result = dojo.getObject("parent.page._searchResults._queryResult", false, this);
		this._searchBanner.setQueryResult(result);
		var show = false;
		if (result) {
			show = this._searchBanner.navigateTo(this.elementUri);
		}
		this.searchBanner.style.display = show ? "" : "none";
	},
	
	refresh: function(){
		this.inherited(arguments);
		this.refreshSearchBanner();
	},
	
	populateNavigationAndViewer: function(uri) {
		uri = uri || this.parent.input.resourceInfo.uri;
		this.populateBreadCrumbs(uri);	
	},
	
	/*
	 * Object that contains:
	 * uri - the uri of the root element
	 * info - object containing: {shortName, longName, iconUri}
	 */
	rootElementInfo: {
		uri: DesignUtil.getExploreResourceUri(),
		info: {
			shortName: messages.explore,
			longName: messages.exploreToolTip,
			iconUri: dojo.moduleUrl("rmps.app", "graphics/icons/allDesigns.gif").toString(),
			simpleLink: true
		}
	},
	
	initializeRootElementHandler: function() {
		this.elementUri = this.rootElementInfo.uri;

		return new rmps.app.ui.internal.editor._BreadCrumbElementHandler({
			uri: this.rootElementInfo.uri,
			parentViewer: this,
			info: this.rootElementInfo.info,
			isRoot: true
		});
	}
	
});

}());
