/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.pages.design.ExplorerWithGrid");

dojo.require("dijit._Widget");
dojo.require("dijit.Menu");

dojo.require("dojo.date.stamp");

dojo.require("rmps.core.RootServices");
dojo.require("rmps.core.clipboard.Clipboard");

dojo.require("rmps.ui.explorer.ExplorerTree");
dojo.require("rmps.ui.explorer.ExplorerModel");

dojo.require("rmps.app.core.util");
dojo.require("rmps.core.client");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("jazz.ui.menu.MenuItem");
dojo.require("jazz.app.context");
dojo.require("jazz.util.html");
dojo.require("rmps.app.ui.internal.dialogs.DeletionFromExplorerInfoDialog");
dojo.require("rmps.app.ui.internal.pages.design.widgets.DomainLazyTreeGrid");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("rmps.ui.explorer.ExplorerUtilities");
dojo.require("rmps.app.core.internal.util.QueryUtil");
dojo.require("rmps.core.util.ContextUtils");

var ContextUtil = rmps.core.util.ContextUtils;
var designUtil = rmps.app.ui.internal.pages.design.DesignUtil;
var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;
var QueryUtil = rmps.app.core.internal.util.QueryUtil;

dojo.requireLocalization("rmps.ui.explorer", "ExplorerMessages");
var messages = dojo.i18n.getLocalization("rmps.ui.explorer", "ExplorerMessages");
dojo.requireLocalization("rmps.app.ui.internal.pages.design", "DesignExplorerMessages");
var designExplorerMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages.design", "DesignExplorerMessages");


var INFO_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "info");
var FOLDERS_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "folders");
var RENDERING_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "rendering");
var ICON_QUERY_VAR = "icon";
var BASELINE_PARAM = "rmps.context";
var ImageService = rmps.app.ui.internal.common.ImageService;
var clipboard = rmps.core.clipboard.clipboard;
var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;

var ACTION_DELETE = "DELETE";
var ACTION_FOCUS = "FOCUS";
var ACTION_CLIPBOARD = "CLIPBOARD";

var CLIENT_ACTIONS = [ACTION_FOCUS, ACTION_CLIPBOARD];


var areEqualChains = function(chain1, chain2) {
	if (chain1.length !== chain2.length) {
		return false;
	}
	for (var i = 0; i < chain1.length; i++) {
		if (chain1[i] !== chain2[i]) {
			return false;
		}
	}
	return true;
};

var getChainComparator = function(id) {
	return function(x) {
		return areEqualChains(id, x); 
	};
};


/*
 * Explorer widget class
 */

dojo.declare("rmps.app.ui.internal.pages.design.ExplorerWithGrid", dijit._Widget, {
	
	"class": "rmps-app-pages-design-explorer",

	// Call back function to allow explorer to create a new explorer instance.
	// Can be left null if operation is not supported.
	onNewExplorer: null,
	// The URI of the explorer root resource
	store: null,
	// The tool tip store instance of type rmps.ui.explorer.ExplorerTooltipStore
	// Can be left null. If so, tool tips are not displayed.
	tooltipStore: null,
	// Call back function which is called when the left arrow key is pressed
	// when focus is at the uppermost node in the tree.
	// Can be left null
	onLeftArrowExit: null,
	// Call back function which is called when the up arrow key is pressed
	// when focus is at the uppermost node in the tree. 
	// Can be left null
	onUpArrowExit: null,
	// Call back function which is called when the down arrow key is pressed
	// when focus is at the bottom node in the tree.
	// Can be left null
	onDownArrowExit: null,
	// Call back function which is called when a top node is removed from the explorer
	// Takes one parameter with the updated list of top nodes
	// Can be left null
	topNodes: null,
	// If present, the nodes in visibleTopNodes will be the only visible top nodes from
	// the topNodes parameter. The rest will be added as an ellipsis node next to the
	// visible top nodes.
	visibleTopNodes: null,
	// A list of top nodes ids hidden by the ellipsis node
	hiddenTopNodes: null,
	
	// a flag determine how to handle new trees
	showNewTrees: true,

	_resourceInfoStore: null,
	
	_model: null,
	_tree: null,
	_menu: null,
	_popup: null,
	
	_back: null,
	_forward: null,
    
    _historyEnabled: true,
	_history: null,
	
	_folderOpenIconUri: dojo.moduleUrl("rmps.app", "graphics/icons/folder_open_co.gif").toString(),
	_folderClosedIconUri: dojo.moduleUrl("rmps.app", "graphics/icons/folder_closed_co.gif").toString(),
	_blankIconUri: dojo.moduleUrl("rmps.app", "graphics/icons/blank.gif").toString(),
	
	//static
	actionRegistry: {},
	
	getOwner: function() {
		return this._exploreContent;
	},
	constructor: function(args) {
		dojo.mixin(this, args);
		
		this.registerActions();
		
		// Create resource info store
		this._resourceInfoStore = new dojox.data.JsonRestStore({
			target: INFO_PATH,
			service: function(query, queryOptions) {
				var def = new dojo.Deferred();
				var id = query;
				var rawid;
				if (id.substring(0, 4) === "info") {
					rawid = id.substring(4, id.length);							
				}
				var resourcesURIs = [ExplorerUtil.getResourceURIFromNodeID(rawid)];
				var infodef = ResourceInfoClient.getResourceInfo(resourcesURIs, ["displayType", "longName", "modificationTime"]);
				infodef.addCallback(function (results) {
					var item = results[0];
					def.callback({
						id: id,
						displayType: item.displayType,
						longName: item.longName,
						modificationTime: item.modificationTime
					});
				});
				infodef.addErrback(def.errback);
				return def;
			}
		});
	},	
	
	registerActions: function() {
		if (this.actionRegistry.initialized) {
			return;
		}
		var isResourceNode = function(node, id) {
			return !node.isEllipsis && !node.page && !node.group && !designUtil.avoidEdit(id); 
		};

		var isSnapshot = ContextUtil.isSnapshot(ContextUtil.getContextUri());
		this.actionRegistry[ACTION_DELETE] = {
			isApplicable: isResourceNode,
			label:  messages.remove,
			enable: !isSnapshot,
			className: "design-explorer-actions-delete",
			imgName: "rmps.app/graphics/icons/remove.gif",
			handler: function (id, node) {
				this.deleteNode(id, node);
			}
		};
		this.actionRegistry[ACTION_FOCUS] = {
			isApplicable: isResourceNode,
			label: messages.collapseContext,
			enable: true,
			imgName: "rmps.app/graphics/icons/collapse_to_context_co.gif",
			className: "rmps-app-explorer-focus-on-element-icon",
			handler: function(id, node, rowIndex) {
				this.focusRow(node, rowIndex);				
			}
		};
		this.actionRegistry[ACTION_CLIPBOARD] = {
			isApplicable: isResourceNode,
			label: messages.addToClipboard,
			enable: true,
			imgName: "rmps.ui/graphics/icons/add_to_clipboard_co.gif",
			className: "design-explorer-actions-addtoclipboard",
			handler: function(id, node, rowIndex) {
				ResourceInfoClient.getResourceInfo([id], ["type","shortName","parentName"]).then(function(info) {
					clipboard().addCallback(function(cb) {
						cb.push(id, info[0].type, info[0].shortName,info[0].parentName);								
					});
				});
			}
		};
		this.actionRegistry.initialized = true;
	},
	
	
	setShowNewTrees: function(value) {
		this.showNewTrees = value;
	},

	postCreate: function() {
		var state = null;		
		if (this._history.stack && this._history.depth>0) {
			state = this._history.stack[this._history.depth-1];
			//this.setShowNewTrees(state.showNewTrees);
		}
		this._model = new rmps.ui.explorer.ExplorerModel({
	        store: this.store,
	        childrenAttrs: ["children"],
	        deferItemLoadingUntilExpand: true
		});
		// Root item is not properly set up in ForestStoreModel. WA
		this._model.root.id = this._model.rootId;
		this._model.root.label = this._model.rootLabel;
		// Need to have the model already created to have the root element
		this._model.root.children = this.topItems;
		this._tree = this._createTree(state);
		if (!state) {
		    this._enableHistory(true);
		    this._tree._saveState();
		}   // otherwise the history will be enabled in onLoad
	    this.domNode.appendChild(this._tree.domNode);
	},
	
	/*
	 * protected: type: Object
	 * 
	 * Hook to add client specific initialization for the tree, e.g.
	 * double click handling.  The contents of this Object will be passed
	 * to the ExplorerTree that is constructed
	 */
	treeInitializationData : null,
	

	
	getImageUrl: function(item, index, model, grid){
		if (item.isEllipsis)
		{
			//var url = item.ellipsisSiblings ? ImageService.ELLIPSIS_V : ImageService.ELLIPSIS_H;
			return ImageService.ELLIPSIS_H;
		}
		if (item.iconUri) {
			return item.iconUri;
		} else if (item.group) {
			// Fall back to folder icons for groups
			var open = grid._treeCache.items[index].opened;
			return open ? this._folderOpenIconUri : this._folderClosedIconUri;
		}
		var uri = ExplorerUtil.getResourceURIFromNodeID(model.store.getIdentity(item));
		if (uri) {
			// Make sure the icon is not cached
			var iconURL = ResourceInfoClient.getIconUrl(uri, grid._preventCache);
			return iconURL;
		}
		return "";
			
		
	},
	
	formatName: function(value, inRowIndex, level, cell)
	{
		var model = cell.grid.treeModel;
		var item = cell.grid.getItem(inRowIndex);
		var iconURL = this.getImageUrl(item, inRowIndex, model, cell.grid);
		//dojo.style(cell.view.domNode,style);
		var label = "";
		if (value) {
			label = jazz.util.html.escape(value);
		}
		if (item.isEllipsis) {
			label += "(" + item.ellipsisNumber +")";
		}
		
		var resourceLink = item && !item.group && !item.page;
		
		var html = "<img class=\"explorerIcon\" src=\"" + dojo.moduleUrl("dojo", "resources/blank.gif") + "\" style=\"background-image: url("+iconURL+");\" height=\"16px\" width=\"16px\">" +
					"&nbsp<span style=\"vertical-align: middle;\">";
		if (resourceLink) {
			html += "<a class=\"resource-link-attach-anchor\">";
		}
		html += label;
		if (resourceLink) {
			html += "</a>";
			this._tree.needLink[inRowIndex] = true;
		}
		html += "</span>";
		return html;
	},
	
	formatActions: function(value, inRowIndex, level, cell) {
		if (this._tree.toolbars[inRowIndex]) {
			this._tree.toolbars[inRowIndex].destroy();
		}
		var toolbar = new jazz.ui.Toolbar();
		this._tree.toolbars[inRowIndex] = toolbar;
		dojo.addClass(toolbar.domNode, "rmps-app-pages-design-explorerActionsToolbar");
		return toolbar;
	},
	
	formatID: function(value, inRowIndex, level, cell) {
		var model = cell.grid.treeModel;
		var item = cell.grid.getItem(inRowIndex);
		if (!item || item.group || item.page) {
			return null;
		}
		var id = model.store.getIdentity(item);
		if (!id) {
			return null;
		}
		var uri = ExplorerUtil.getResourceURIFromNodeID(id);
		var match = uri.match(/\/(\d+)(#_.*)?$/);
		if (match && match[1]) {
			return match[1];
		}
		return null;
	},
		
	formatResourceInfo: function(value, inRowIndex, level, cell) {
		var model = cell.grid.treeModel;
		var item = cell.grid.getItem(inRowIndex);
		if (!item || item.group || item.page) {
			return null;
		}
		var id = model.store.getIdentity(item);
		if (!id) {
			return null;
		}
		var grid = cell.grid;
		var def = new dojo.Deferred();
		this._resourceInfoStore.fetchItemByIdentity({
			identity: "info" + id,
			onItem: function(item) {
				var value = item[cell.infofield];
				if (value) {
					if (cell.infofield === "modificationTime") {
						value = QueryUtil.getModifiedLabel(dojo.date.stamp.toISOString(value));
					}
					def.callback(jazz.util.html.escape(value));	
				} else {
					def.errback();
				}
				setTimeout(function() {grid.rowHeightChanged(inRowIndex);}, 100);
			},
			onError: function(error) {
				def.callback(designExplorerMessages.resourceInfoNotAvailable);
			}
		});
		return def;
	},
	
	populateToolbar: function(toolbar, inRowIndex, node, uri) {
		var cell = toolbar.domNode.parentNode;
		var initial_height = dojo.marginBox(cell).h;
		
		var args, item;
		var items = [];
		var actions = node.actions || [];
		actions = actions.concat(CLIENT_ACTIONS);
		var applicableDesc = [];
		dojo.forEach(actions, dojo.hitch(this, function(action){
			var actionDesc = this.actionRegistry[action];
			if (actionDesc && actionDesc.isApplicable(node, uri)){
				applicableDesc.push(actionDesc);
			}
		}));
		
		
		var toolbarDesc = [], contextDesc = [];
		
		if (!applicableDesc.length) {
			return;
		}
		if (applicableDesc.length > 3) {
			toolbarDesc = applicableDesc.slice(0,2);
			contextDesc = applicableDesc;
		} else {
			toolbarDesc = applicableDesc;
		}

		dojo.forEach(toolbarDesc, dojo.hitch(this, function(actionDesc){
			var button = new jazz.ui.toolbar.Button({
					title: actionDesc.label,
					imgSrc: actionDesc.imgName,
					onClick: dojo.hitch(this, function(){
						actionDesc.handler.call(this, uri, node, inRowIndex);
					})
				});					
			button.set("enabled", actionDesc.enable);
			toolbar.addChild(button);			
		}));
		
		dojo.forEach(contextDesc, dojo.hitch(this, function(actionDesc){
			args = {
				label: actionDesc.label,
				onClick: dojo.hitch(this, function(){
					actionDesc.handler.call(this, uri, node, inRowIndex);
				})
			};				
			args.iconClass = actionDesc.className;	
			item = new jazz.ui.menu.MenuItem(args);
			items.push(item);
		}));
		
		if (items.length) {
			var menuButton = new jazz.ui.toolbar.MenuButton({
				imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/context_menu_co.gif").toString(),
				menuProvider: dojo.hitch(this, function(){
					var menu = new jazz.ui.Menu();
					for (var i=0;i<items.length;i++){
						menu.addChild(items[i]);
					}
					return menu;
				}),
				//adding visibility to the whole toolbar when menu is opened
				_onClickHandler: function(event, caretClick) {
					if (!this.openOnCaretClick || caretClick) {
						if (event) {
							dojo.stopEvent(event);
						}
						if (this._preventClose) {
							return;
						}
						this.open();
						dojo.addClass(toolbar.domNode, "visibleToolbar");
					} else {
						 if (this._preventClose) {
							 return;
						 }
						 this.close(false);
						 clearTimeout(this._overTimeout);
						this._overTimeout = null;
					}
				},
				close: function(focus) {						
					jazz.ui.toolbar.MenuButton.prototype.close.apply(this, arguments);
					dojo.removeClass(toolbar.domNode, "visibleToolbar");
				}
			});
			toolbar.addChild(menuButton);	
		}
		
		var new_height = dojo.marginBox(cell).h;
		if (new_height !== initial_height) {
			this._tree.scroller.rowHeightChanged(inRowIndex, true);
		}
	},
	
	deleteNode: function(uri, node) {
		// there could be multiple rows selected, get node info about each of them
		var urisToDelete = [];
		var labelsToDelete = [];
		var currentAdded = false;
		for (var inRowIndex in this._tree.selection.selected) {
			var selNode = this._tree.getItem(inRowIndex);
			// consider this node ony if it has delete action associated
			var canDelete = false;
			if (selNode.actions) {
				for (var i = 0; i < selNode.actions.length; i++) {
					if (selNode.actions[i] === ACTION_DELETE) {
						canDelete = true;
						break;
					}
				}
			}
			if (!canDelete) {
				continue;
			}
			var selUri = ExplorerUtil.getResourceURIFromNodeID(this._tree.treeModel.store.getIdentity(selNode));
			urisToDelete.push(selUri);
			labelsToDelete.push(selNode.label);
			if (selUri === uri) {
				currentAdded = true;
			}
		}
		if (!currentAdded) {
			urisToDelete.push(uri);
			labelsToDelete.push(node.label);			
		}
		new rmps.app.ui.internal.dialogs.DeletionFromExplorerInfoDialog({
				deleteFiles:labelsToDelete, uris:urisToDelete, explorer: this});
	},
	
	_createTree: function(saved) {
		var tmpData = {
				model: this._model,
				showRoot: false,
				//persist: false,
				//dndController: "dijit._tree.dndSource",
				_explorer: this,
				onLeftArrowExit: this.onLeftArrowExit,
				onUpArrowExit: this.onUpArrowExit,
				onDownArrowExit: this.onDownArrowExit,
				fnIsChecked: this.fnIsChecked,
				fnOnCheck: this.fnOnCheck,
				_pinnedItems: [],
				_pinnedPaths: []
			};
		
		var treeData = this.treeInitializationData || {};
		
		// use treeData as the target so any properties set 
		// are not overridden
		dojo.mixin(treeData, tmpData);
		
		if (saved) {
			if (saved.treeState) {
				treeData._treeState = dojo.clone(saved.treeState);
			}
			if (saved.pinnedPaths) {
				treeData._pinnedPaths = dojo.clone(saved.pinnedPaths);
			}
		}
		
		var actionWidth = 100 * 75 /* wanted pixels */ / (dojo.contentBox(this._exploreContent.domNode).w - 20 /* some padding*/); 
		if (isNaN(actionWidth)) {
			actionWidth = 5;
		} else {
			actionWidth = Math.ceil(actionWidth);
		}
		var detailsEnabled = this._exploreContent.getDetailedEnabled();
		var nameWidth = ((detailsEnabled ? 66 : 97) - actionWidth);
		
		var layout =
        [{
			name: messages.actions,
			field: null,
			width: actionWidth + '%',
			formatter: dojo.hitch(this, this.formatActions)
		},
		{
			name: messages.id,
			field: null,
			width: '3%',
			formatter: this.formatID			
		},
        {
			name: messages.name,
			field: 'label',
			width: nameWidth + '%',
			formatter: dojo.hitch(this, this.formatName)
        }];
		
		if (detailsEnabled) {
			layout = layout.concat([
			{
				name: messages.longname,
				field: null,
				infofield: 'longName',
				width: '15%',
				formatter: dojo.hitch(this, this.formatResourceInfo)
			},
			{
				name: messages.type,
				field: null,
				infofield: 'displayType',
				width: '8%',
				formatter: dojo.hitch(this, this.formatResourceInfo)
			},
			{
				name: messages.modified,
				field: null,
				infofield: 'modificationTime',
				width: '8%',
				formatter: dojo.hitch(this, this.formatResourceInfo)
			}]);
		}

		var treeGrid = new rmps.app.ui.internal.pages.design.widgets.DomainLazyTreeGrid(dojo.mixin({
			expandoCell: 2,
			rowsPerPage: 50,
			treeModel: this._model,
			structure: layout,
			autoHeight: true,
			loadingMessage: "",
			errorMessage: "",
			
			rowselector: "true",
			columnReordering: true,
			defaultOpen: true,
			updateDelay: 0,
			explorer: this
		}, treeData));
		return treeGrid;
	},
	
	destroy: function() {
		if (this._popup) {
			this._popup.destroyRecursive();
			delete this._popup;
		}
		if (this._menu) {
			this._menu.destroyRecursive();
			delete this._menu;
		}
		if (this._tree) {
			this._tree.destroyRecursive();
			delete this._tree;
		}
		if (this._model) {
			this._model.destroy();
			delete this._model;
		}
		
		this.inherited(arguments);
	},
	
	onContextMenu: function(aroundNode)
	{
		if (!this._menu) {

			this._menu = new dijit.Menu();
			dojo.addClass(this._menu.domNode, "rmps-app-explorer-node-context-menu");
					
			// FIXME: Move to main toolbar/context menu
			
			this._forward = new dijit.MenuItem({
				label: messages.forward,
				onClick: dojo.hitch(this, this.onForward),
				disabled : false
			});
			this._menu.addChild(this._forward);
			
			this._back = new dijit.MenuItem({
				label: messages.back,
				onClick: dojo.hitch(this, this.onBack),
				disabled : false
			});			
			this._menu.addChild(this._back);
			
			
			this._menu.startup();
			
			this.connect(this._menu, "_onBlur", this._closeMenu);
		}
		
		this._back.attr('disabled', !this._canGoBack());
		this._forward.attr('disabled', !this._canGoForward());
		
		// open pop-up menu
		dijit.popup.open({
			popup: this._menu,
			parent: this,
			around: aroundNode,
			orient: {'BR':'TL', 'BL':'TR', 'TR':'BL', 'TL':'BR'},
			onExecute: dojo.hitch(this, this._closeMenu),
			onCancel: dojo.hitch(this, this._closeMenu)
		});
		
		this._menu.focus();
	},
	
	setTopNodes: function(topNodes, visibleTopNodes, hiddenTopNodes) {
		this.topNodes = topNodes;
		this.visibleTopNodes = visibleTopNodes ? visibleTopNodes : [];
		this.hiddenTopNodes = hiddenTopNodes ? hiddenTopNodes : [];
		
		var rootChildren = this._createRootChildrenList();
		
		this._model.setRootChildren(rootChildren);
	},
	
	getCurrentTopNodes: function() {
		// Get the current top nodes without ellipsis
		return dojo.filter(this._model.getRootChildren(), function(c) {
			return !("isEllipsis" in c);
		});
	},
	
	getTopNodes: function() {
		return this.topNodes;
	},
	
	getHiddenTopNodes: function() {
		return dojo.filter(this.topNodes, function(n) {
			return dojo.indexOf(this.visibleTopNodes, n) === -1;
		});
	},
	
	_createRootChildrenList: function() {
		if (!this.visibleTopNodes || !this.visibleTopNodes.length){
			this.setShowNewTrees(true);
			return this.topNodes;
		}
		if (this.showNewTrees) {
			this.visibleTopNodes = dojo.filter(this.topNodes, dojo.hitch(this, 
									function (id) {
										return dojo.indexOf(this.hiddenTopNodes, id) === -1;
									})); 
		} else {
			this.hiddenTopNodes = dojo.filter(this.topNodes, dojo.hitch(this, 
									function (id) {
										return dojo.indexOf(this.visibleTopNodes, id) === -1;
									}));
		}
			
		if (this.hiddenTopNodes.length)
		{
			var ellipsis = this._createEllipsisItem(this.hiddenTopNodes.length, this._model.root);
			var rootChildren = this.visibleTopNodes.slice(); // Copy array
			rootChildren.push(ellipsis);
			return rootChildren;
		}
		else {
			return this.visibleTopNodes.slice();
		}
	},	
		
	_closeMenu: function()
	{
		dijit.popup.close(this._menu);
	},
	

	_enableHistory : function(value) {
		this._historyEnabled = value;
	},
	
	
	_canGoForward : function() {
		if (!this._history || !this._history.stack) {
			return false;
		}
		return this._history.depth < this._history.stack.length;
	},
	
	_canGoBack : function() {
		return this._history.depth > 1;
	},
	

	
	_rememberState : function() {
		var state = this.getState();
		var owner = this.getOwner();
		
		if (this._history && this._history.stack && this._historyEnabled) {
			var hist = this._history;
			while (hist.stack.length > hist.depth)
			{
				hist.stack.pop();
			}
				
			while (hist.stack.length >= this._history.maxDepth) {
				hist.stack.shift();
				hist.depth--;
			}

			hist.stack.push(dojo.clone(state));
			hist.depth++;

			if (owner) {
				owner._saveHistory();
			}
		}
		if (owner) {
			owner.updateToolbar();
		}
	},
	
	_restoreState : function(state) {
		this._continueRestore(state, 0, []);
	},
		
	createIdChain: function(idx) {
		var cache = this._tree._treeCache.items;
		var store = this._tree.model.store;
		var treepath = cache[idx].treePath;
		var p;
		var i = idx;
		var result = [];
		do  {
			if (cache[i].treePath === treepath) {
				var item = this._tree.getItem(i);
				result.push(store.getIdentity(item));
				p = treepath.lastIndexOf("/");
				treepath = p === -1 ? null : treepath.substr(0,p);
			}
			i--;
		} while (treepath && i >= 0);
		return result;
	},
	
	_continueRestore: function (state, index, expanded) {
		if (this.expandDeferred) {
			delete this.expandDeferred;
		}
		var cache = this._tree._treeCache.items, i;
		
		if (index >= state.length) {
			for (i = 0; i < cache.length; ++i) {
				if (cache[i].opened) {
					var id_chain = this.createIdChain(i);
					if (!dojo.some(expanded, getChainComparator(id_chain) ) ){
						this.toggleExpandRow(i);
					}
				}
			}			
			if (this.bSaveAfterExpand) {
				this.bSaveAfterExpand = false;
				this._rememberState();
			}
			return;
		}
		var state_rec = state[index];
		var bFound = false;		
		var model = this._tree.model;
		for (i = 0; i < cache.length; ++i) {
			var item = this._tree.getItem(i);
			if (model.store.getIdentity(item) === state_rec.item_id)
			{
				var current_chain = this.createIdChain(i);
				if (areEqualChains(current_chain, state_rec.id_chain)) {
					expanded.push(current_chain);
					bFound = true;
					if (cache[i].opened !== state_rec.opened){
						this.expandDeferred = new dojo.Deferred();
						this.expandDeferred.addCallback(dojo.hitch(this, function(){
							this._continueRestore(state, index + 1, expanded);
						}));
						this.toggleExpandRow(i);
						
					} else {
						this._continueRestore(state, index + 1, expanded);
					}
					
					break;
				}
			}
		}
		if (!bFound) {
			this._continueRestore(state, index+1, expanded);
		}
	},
	
	onForward : function() {		
		if (this._canGoForward()) {
			var hist = this._history;
			var owner = this.getOwner();
			this._restoreState(hist.stack[hist.depth]);
		    hist.depth++;
			if (owner) {
				owner._saveHistory();
			}
		}
	},
	
	onBack : function() {
		if (this._canGoBack()) {
			var hist = this._history;
			var owner = this.getOwner();
			hist.depth--;
			this._restoreState(hist.stack[hist.depth-1]);
			if (owner) {
				owner._saveHistory();
			}
		}
	},
	
	getView: function() {
		return this._tree.views.views[0];
	},
	
	toggleExpandRow: function(ind) {
		var idty = this._tree._by_idx[ind].idty; 
		var expando = this.getView()._expandos[idty];
		if (expando) {
			expando.onToggle(null, true);
		}
	},
	
	onCollapseAll: function() {
		for (var i = this._tree._treeCache.items.length -1; i >= 0; i--) {
			if (this._tree._treeCache.items[i].opened){
				this.toggleExpandRow(i);

			}
		}
		this._rememberState();
	},
	
	onShowPinnedOnly: function() {
		if (this._tree) {
			this._tree.OnCollapseAllExceptPinned();
		}
	},
	
	_createEllipsisItem : function(count, parentitem, parentchain)
	{ 
		var uuid = dojox.uuid.generateRandomUuid();
		
		if (parentitem.isEllipsis) {
			// If parent is ellipsis item then get the correct parent item.
			var parentparentchain = parentitem.ellipsisParentChain;
			parentitem = parentparentchain[parentparentchain.length - 1];
		}
		
		var ellipsis = {
			__id: uuid,
			isEllipsis: true,
			ellipsisNumber: count,
			ellipsisUUID: uuid,
			ellipsisParent: parentitem,
			ellipsisSiblings: parentchain ? false : true,
			ellipsisParentChain: parentchain
		};
		
		if (parentitem.root) {
			this.setShowNewTrees(false);
		}
		
		return ellipsis;
	},
	
	unpinAll: function() {
		if (this._tree) {
			this._tree.unpinAll();
		}
	},
	
	getState: function() {
		var cache = this._tree._treeCache.items;
		var model = this._tree.model;
		var expands_state = [];
		for (var i = 0; i < cache.length; i++) {
			var item = this._tree.getItem(i);
			if (cache[i].opened) {
				expands_state.push({
					opened: cache[i].opened,
					//treePath: cache[i].treePath,
					id_chain: this.createIdChain(i),
					item_id: model.store.getIdentity(item)
				});
			}
		}
		return expands_state;
	},
	
	saveAfterExpand: function() {
		if (!this.expandDeferred) {
			this.expandDeferred = new dojo.Deferred();
		} 
		this.expandDeferred.addCallback(dojo.hitch(this, function(){
			
			this._rememberState();
			if (this.expandDeferred)  {
				delete this.expandDeferred;
			}                                                                                                                                                                                   
				
		}));
			
	},
	
	restoreLastState: function() {
		if (this._treeInitialization) {
			dojo.publish("rmps.web.app.explorer.design.created"); 
			if (!this.uriToShow && !designUtil.focusingResults && this._history && this._history.stack) {
				if (this._history.stack.length) {
					var state = this._history.stack[this._history.depth-1];
					this._restoreState(state);
				} else {
					this._rememberState();
				}
			}
			this._treeInitialization = false;
		}
	},
	
	
	getContinueExpandFunc : function(chain, i, row, def) {
		return dojo.hitch(this, function() {
			if (this.expandDeferred)  {
				delete this.expandDeferred;
			}	
			this._continueExpandChain(chain, i-1, row+1, def);
		});
	},
	
	_continueExpandChain: function(chain, i, row, def) {
		if (i < 0) {
			return;
		}
		var found = false;
		var cache = this._tree._treeCache.items, model = this._tree.model;
		var first_row = row;
		while (row < cache.length) {
			var item = this._tree.getItem(row);
			if (ExplorerUtil.getNodeIDFromResourceURI(chain[i], this.getOwner().explorerID) === model.store.getIdentity(item)) {
				if (i === 0) {
					def.resolve(row);
				} else {
					var handleNext = this.getContinueExpandFunc(chain, i, row, def);
					if (!cache[row].opened){
						this.expandDeferred = new dojo.Deferred();
						this.expandDeferred.addCallback(handleNext);
						this.toggleExpandRow(row);
						
					} else {
						handleNext();
					}
				}
				found = true;
				break;
			}
			row++;
		}
		if (!found){
			var skip = this.getContinueExpandFunc(chain, i, first_row - 1, def);
			skip();
		}
	},
	
	expandChain: function(chain) {		
		var completed = new dojo.Deferred();
		if (!dojo.isArray(chain)) {
			completed.reject();
			return completed;
		}
		this._continueExpandChain(chain, chain.length - 1, 0, completed);
		completed.then(dojo.hitch(this, function(row) {
			this._tree.selection.select(row);
			dijit.scrollIntoView(this._tree.getRowNode(row));
		}));
		return completed;
	},
	
	_continueExpandManyChains: function(chains, ind, def, nums) {
		if (!chains) {
			def.reject();
			return;
		}
		if (ind >= chains.length) {
			def.resolve(nums);
			return;
		}
		var singleExpand = this.expandChain(chains[ind].chain);
		var nextChain = dojo.hitch(this, function(res) {
			if (res) {
				nums.push(this._tree._treeCache.items[res]);
			}
			this._continueExpandManyChains(chains, ind+1, def, nums);
		});
		singleExpand.then(nextChain, nextChain);
	},
	
	expandManyChains: function(chains) {
		var completed = new dojo.Deferred();
		this._continueExpandManyChains(chains, 0, completed, []);
		return completed;
	},
		
	
	focusOnChains: function(chains) {
		
	},
	
	createParentEllipsis: function(item) {
		this._tree.createSiblingEllipsis(item);
	},
	
	focusRow: function(item, rowIdx) {
		this._tree.focusRow(rowIdx);
	}
});



}());