/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.pages.design.ValidationEditor");

dojo.require("rmps.core.util.ContextUtils");

dojo.require("rmps.ui.CollapsibleSection");

dojo.require("rmps.app.ui.internal.editors.BaseEditor");

dojo.require("jazz.ui.StyledTable");

dojo.require("jazz.ui.HoverPopup");

dojo.require("jazz.ui.MessageBox");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.app.ui.internal.pages.design", "ValidationMessages");

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.pages.design", "ValidationMessages");

var VALIDATION_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "modelvalidation");

var UA_KEY = "com.ibm.xtools.web.app.page_ValidationResults001";

var SEVERITIES = ["Severity-info", "Severity-warning", "Severity-error"];

var ContextUtils = rmps.core.util.ContextUtils;

dojo.declare("rmps.app.ui.internal.pages.design.ValidationEditor", rmps.app.ui.internal.editors.BaseEditor, {

	revalidateMessageBox: null,
	
	"class": "rmps-validation-editor",
	uaKey: UA_KEY,
	advancedSection: null,
	_revalidateButton: null,
	activeDialog: null,
	
	constructor: function() {
		this.isReadOnly = true;
		this.isRefreshable = false;
	},
	
	postCreate: function() {
		this.inherited(arguments);
		
		this.revalidateMessageBox = new jazz.ui.MessageBox();
		this.revalidateMessageBox.domNode.style.margin = "1px";
		this.revalidateMessageBox._first = true;
		
		dojo.place(this.revalidateMessageBox.domNode, this.containerNode.parentNode, "first");
		this.revalidateMessageBox.startup();
				
		this._createAdvancedSection();
	},
	
	_createAdvancedSection: function(shouldRevalidate) {
		this.advancedSection = new rmps.app.ui.internal.pages.design.ValidationEditor._Advanced({editor: this, revalidate: shouldRevalidate});
		this.containerNode.appendChild(this.advancedSection.domNode);
		this.advancedSection.startup();
	},
	
	resize: function() {
		//override: do not resize
	},
	
	addButtons: function(toolbar) {
		// refresh button
		this._revalidateButton = toolbar.addButton(this, this._revalidate, {
			label: messages.revalidate,
			imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/revalidate.gif")
		});
	},

	showPromptRevalidateMessageBox: function(validationDateTime, streamChangedDateTime) {
		if (this.revalidateMessageBox) {
			if (validationDateTime && streamChangedDateTime) {
				try {
					var validationDateTimeDate = new Date(validationDateTime);
					var streamChangedDateTimeDate = new Date(streamChangedDateTime);
					if (validationDateTimeDate < streamChangedDateTimeDate) {
						this.revalidateMessageBox.showMessage({
							type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
							summary: messages.revalidateInfo,
							closable: true
						});
						dojo.fadeIn({
							node: this.revalidateMessageBox.domNode,
							beforeBegin: dojo.hitch(this, function() {
								dojo.style(this.revalidateMessageBox.domNode, "opacity", 0);
							})
						}).play();
					}
				} catch (err) {
					//just don't show the messagebox
				}
			}
		}
	},
	
	hidePromptRevalidateMessageBox: function() {
		if (this.revalidateMessageBox) {
			dojo.fadeOut({
				node: this.revalidateMessageBox.domNode,
				onEnd: dojo.hitch(this, function() {
					this.revalidateMessageBox.hideMessage();
				})
			}).play();			
		}
	},
	
	_revalidateWithoutConfirmation: function() {
		this.hidePromptRevalidateMessageBox();
		this.advancedSection.destroy();
		this._createAdvancedSection(true);
	},
	
	_revalidate: function() {
		if (ContextUtils.getContextType(jazz.app.context.get().baseline) === rmps.core.util.ContextTypes.SNAPSHOT) {
			var root =  new dijit.layout.BorderContainer({
				region: "center"
			});
			this.root = root;
			
			var dialogMessageNode = dojo.create("div", { innerHTML: "<p>" + messages.revalidateDialogQuestion + "</p>" });
			root.domNode.appendChild(dialogMessageNode);
			
			var buttonArea = new dijit.layout.BorderContainer({
				region: "bottom",
				style: "height: 28px; width: 100%;",
			});
			
			root.addChild(buttonArea); 			
	
			var okButton = new dijit.form.Button({
				spanLabel: true, 
				label: messages.revalidateDialogRevalidateButton,
				onClick: dojo.hitch(this, function(response, ioArgs) {
					this._revalidateWithoutConfirmation();
					this.activeDlg.close();
				}),
			});
			
			var cancelButton = new dijit.form.Button({
				spanLabel: true, 
				label: messages.revalidateDialogCancelButton,
				onClick: dojo.hitch(this, function(response, ioArgs) {
					this.activeDlg.close();
				}),
			});
			
			buttonArea.addChild(okButton);
			buttonArea.addChild(cancelButton);
			
			this.activeDlg = new jazz.ui.Dialog({
				primaryTitle: messages.revalidateDialogTitle,
	            destroyContents: true,
				contentNode: root.domNode
			});
		} else {
			this._revalidateWithoutConfirmation();
		}
	}
	
});

dojo.declare("rmps.app.ui.internal.pages.design.ValidationEditor._Advanced", [dijit._Widget, dijit._Templated], {
	
	messages: messages,
	
	widgetsInTemplate: true,
	
	templateString: '<div>' +
			'<div dojoType="rmps.ui.CollapsibleSection" dojoAttachPoint="_section" collapsible=false primaryTitle = "' + messages.tableHeader + '">' +
			'</div>' +
			'<div dojoAttachPoint="_noresults" hidden=true>${messages.noResults}</div>' +
		'</div>',
		
	editor: null,
	
	revalidate: false,
	
	getDateTime: function(dateTimeString) {
		var dateTime;
		try {
			dateTime = new Date(dateTimeString);
			dateTime = dateTime.toLocaleString();
		} catch(err) {
			dateTime = dateTimeString;
		}
		return dateTime;
	},
	
	postCreate: function() {
		
		// create table
		this._table = new jazz.ui.StyledTable({
			columns: [
					{label: messages.elementColumnHeader, width: ("50%")},
					{label: messages.constraintColumnHeader, width: ("40%")},
					{label: messages.resultColumnHeader, width: ("10%")}
				]
		});
		this._section.containerNode.appendChild(this._table.domNode);
		this._table.startup();
		this._section.expand();
		
		this.editor.showLoading();
		
		var theUrl = VALIDATION_PATH + "?projectUri=" + encodeURIComponent(jazz.app.context.get().uri) + "&rmps.context=" + encodeURIComponent(jazz.app.context.get().baseline);
		
		if (this.revalidate) {
			theUrl += "&revalidate=true";
		}
		
		
		if (jazz.app.context.get().changeset) {
			theUrl += this.editor.appendChangeset();
			
			this._section.setPrimaryTitle(messages.tableHeaderWhenChangeset);
		}

		var args = {
				headers: {
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "json",
				url: theUrl,
				hasBody: false,
				load: dojo.hitch(this, function(response, ioArgs) {
					return response;
				})
			};
			
		var deferred = jazz.client.xhrPost(args);
		deferred.addCallback(this, function(result) {

			if (result.result === "ok") {
				if (result["dateTime"]) {
					this._section.setPrimaryTitle(dojo.string.substitute(messages.lastValidatedNoResults, [this.getDateTime(result["dateTime"])]));
					this._table.destroy();
					this._noresults.hidden = false;
					this.editor.showPromptRevalidateMessageBox(result["dateTime"], result["lastChangeDateTime"]);
				}
			} else if (result.result === "error") {
				if (result["dateTime"]) {
					var dateTime = new Date(result["dateTime"]);
					dateTime = dateTime.toLocaleString();
					this._section.setPrimaryTitle(dojo.string.substitute(messages.lastValidated, [this.getDateTime(result["dateTime"])]));
				}
				
				var rootUrl = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/logical?rmps.context=" + encodeURIComponent(jazz.app.context.get().baseline) + "&resource=";
				for (var sev = 0; sev < SEVERITIES.length; sev++) {
					for (var i = 0; ; i++) {
						if (result["elementUri" + i]) {
							if (result["severity" + i].indexOf(SEVERITIES[sev]) > -1) {
								var errorRow = new rmps.app.ui.internal.pages.design.ValidationEditor._PropertyRow({
									advancedSection: this,
									editor: this.editor,
									_elementName: result["elementName" + i],
									_elementUri: rootUrl + encodeURIComponent(result["elementUri" + i]),
									_severity: result["severity" + i],
									_constraint: result["message" + i],
									_conditionDesc: result[result["conditionDesc" + i]], //result["conditionDesc" + i] won't be undefined, but result[result["conditionDesc" + i]] could be if there is no description 
									_relatedElements: result["elementUri" + i] //eventually, I plan to store related elements in this field
								});
								this._table.tbody.insertBefore(errorRow.domNode, this._table.tbody.firstChild);
								errorRow.startup();
							}
						} else {
							break;
						}
					}
				}
				this.editor.showPromptRevalidateMessageBox(result["dateTime"], result["lastChangeDateTime"]);
			}
			
			this.editor.hideMessage();

		});		
		
	}
});

dojo.declare("rmps.app.ui.internal.pages.design.ValidationEditor._PropertyRow", [dijit._Widget, dijit._Templated], {
	
	widgetsInTemplate: true,
	
	templateString: '<tr>' +
			'<td>' +
				'<span dojoAttachPoint="_elementValue">' +
				'</span>' +				
			'</td>' +
			'<td>' +
				'<div>' +
					'<span class="preview" dojoAttachPoint="_constraintValue"></span>' +
				'</div>' +
			'</td>' +
			'<td class="defaultValue">' +
				'<span  dojoAttachPoint="_severityValue"></span>' +
			'</td>' +
		'</tr>',
	
	advancedSection: null,
	editor: null,
	
	_elementName: null,
	_elementUri: null,
	_severity: null,
	_constraint: null,
	_conditionDesc: null,
	_relatedElements: null,
	
	postCreate: function() {

		var nameNode = dojo.doc.createTextNode(this._elementName);
		var hyperlink = document.createElement('a');
		hyperlink.href= this._elementUri;
		hyperlink.appendChild(nameNode);
		
		var img = dojo.doc.createElement('img');
		if (this._severity.indexOf("Severity-error") > -1) {
			this._severity = messages.severityError;
			img.src = dojo.moduleUrl("rmps.ui", "graphics/icons/error.gif");
		} else if (this._severity.indexOf("Severity-warning") > -1) {
			this._severity = messages.severityWarning;
			img.src = dojo.moduleUrl("rmps.ui", "graphics/icons/warning.gif");
		} else if (this._severity.indexOf("Severity-info") > -1) {
			this._severity = messages.severityInfo;
			img.src = dojo.moduleUrl("rmps.ui", "graphics/icons/info.gif");
		}

		this._elementValue.appendChild(hyperlink);
		
		img.alt = this._severity;
		img.style.verticalAlign = "bottom";
		img.style.marginRight = "6px";
		
		this._severityValue.appendChild(img);
		this._severityValue.appendChild(dojo.doc.createTextNode(this._severity));
		this._constraintValue.appendChild(dojo.doc.createTextNode(this._constraint));
		
		this._makeTooltip(this._elementValue, this._relatedElements);
		if (this._conditionDesc && this._conditionDesc.length > 0) {
			this._makeTooltip(this._constraintValue, this._conditionDesc);
		}
 	},
 	
 	_makeTooltip: function(value, text) {
		if (dojo.isIE) {
			dojo.toggleClass(value, "TooltipCursor", true);
			this.connect(value, "onmouseover", function(){
				dojo.toggleClass(value, "TooltipColor", true);
			});
			this.connect(value, "onmouseout", function(){
				dojo.toggleClass(value, "TooltipColor", false);
			});
		} else {
			dojo.toggleClass(value, "Tooltip", true);
		}
		// The tooltip sometimes has problems attaching to the object if without a small delay
		setTimeout(dojo.hitch(this, function() {
 			this._tooltip = new jazz.ui.HoverPopup({
				around: value,
				html: text,
				openDelay: 300,
				maxWidth: 800
			})
 		}), 50); 		
 	},
	
	destroy: function() {
		if (this._tooltip) {
			this._tooltip.destroyRecursive();
			delete this._tooltip;
		}
	}
 	
});

})();
