/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
dojo.provide("rmps.app.ui.internal.pages.design.widgets.DomainLazyTreeGrid");

dojo.require("dojox.grid.LazyTreeGrid");
dojo.requireLocalization("dijit", "loading");
dojo.requireLocalization("dijit", "error");

dojo.requireLocalization("rmps.ui.explorer", "ExplorerMessages");
var messages = dojo.i18n.getLocalization("rmps.ui.explorer", "ExplorerMessages");

dojo.require("rmps.ui.explorer.ExplorerUtilities");
var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;

dojo.declare("rmps.app.ui.internal.pages.design.widgets.DomainLazyTreeGrid", dojox.grid.LazyTreeGrid, {
	_saveState: function() {},
	_preventCache: (new Date()).getTime(),
	"class" : "rmps-app-pages-design-treeGrid",
	_resourceLinks: null,
	_buttonInx: 0,
	_kbhit: false,
	
	constructor: function() {
		this._resourceLinks = [];
		this.toolbars = [];
		this.needLink = [];
	},
	
	destroyResourceLinks: function() {
		dojo.forEach(this._resourceLinks, function(link) {
			if (link) {
				link.destroy();
			}
		});
	},
	
	
	destroy: function() {
		this.destroyResourceLinks();
		this.inherited(arguments);
		if (this._setFocusIndexHandle) {
			dojo.disconnect(this._setFocusIndexHandle);
		}
	},
	
	postCreate: function() {
		this.inherited(arguments);

		this._setFocusIndexHandle = dojo.connect(this.focus, "setFocusIndex", this, "_setFocusIndex" );

		dojo.forEach(this.views.views, function(view){
			this.connect(view, "onAfterRow", dojo.hitch(this, "_onAfterRow", view.index));
		}, this);
	},
	
	canSort: function(){
		return false;
	},
	
	_fetchItems: function _fetchItems_$41(req){
		if(this._pending_requests[req.startRowIdx]){
			return;
		}
		this.showMessage(this.loadingMessage);
		this._pending_requests[req.startRowIdx] = true;
		var	onError = dojo.hitch(this, '_onFetchError'),
			start = this._treeCache.getSiblingIndex(req.startRowIdx, req.treePath);
		var root_children = this.model && this.model.root && this.model.root.children; 
		if(req.treePath.length === 0 && !root_children){
			this.store.fetch({
				start: start,
				startRowIdx: req.startRowIdx,
				treePath: req.treePath,
				count: req.count,
				query: this.query,
				sort: this.getSortProps(),
				queryOptions: this.queryOptions,
				onBegin: dojo.hitch(this, '_onFetchBegin'),
				onComplete: dojo.hitch(this, '_onFetchComplete'),
				onError: dojo.hitch(this, '_onFetchError')
			});
		}else{
			var parentId = req.treePath[req.treePath.length - 1], parentItem;
			var queryObj = {
				start: start,
				startRowIdx: req.startRowIdx,
				treePath: req.treePath,
				count: req.count,
				parentId: parentId,
				sort: this.getSortProps()
			};
			var _this = this;
			var onComplete = function(){
				var f = dojo.hitch(_this, '_onFetchComplete');
				if(arguments.length === 1){
					f.apply(_this, [arguments[0], queryObj]);
				}else{
					f.apply(_this, arguments);
				}
			};
			
			if (root_children) {
				parentItem = this.treeModel.root;
				parentId = this.store.getIdentity(parentItem);
			}
			
			if(parentItem || this._by_idty[parentId]){
				if (!parentItem) {
					parentItem = this._by_idty[parentId].item;
				}
				if (root_children) {
					this._onFetchBegin(root_children.length, queryObj);
				}
				this.treeModel.getChildren(parentItem, onComplete, onError, queryObj);
			}else{
				this.store.fetchItemByIdentity({
					identity: parentId,
					onItem: function onItem_$42(item){
						_this.treeModel.getChildren(item, onComplete, onError, queryObj);
					},
					onError: onError
				});
			}
		}
	},
	
	_addItem: function(item, index, noUpdate, dontUpdateRoot) {
		if (arguments.length === 3) {
			// dontUpdateRoot is not set. Set it true.
			this._addItem(item, index, noUpdate, true);
		} else {
			this.inherited(arguments);	
		}
	},
	
	// we have to replace modify the storeId with the treepath to make it unique,
	// since the same item may appear in multiple places in the grid. 
	_getUniqueIdentity: function(item, treepath) {
		if (item) {
			if (item.isEllipsis) {
				return item.ellipsisUUID;
			}
			if (treepath) {
				var path = "";
				for (var i=0; i < treepath.length; i++) {
					path = path + treepath[i] + '/';
				}
				return path + this.store.getIdentity(item);				
			}
			var parentIdentity = this._getUniqueIdentity(this._getItemParent(item));
			return parentIdentity ? parentIdentity + '/' + this.store.getIdentity(item) : this.store.getIdentity(item);
		}
	},	
	
	_onExpandoComplete: function(childItems, request, size){
		size = isNaN(size) ? childItems.length : parseInt(size, 10);
		var treePath = this._treeCache.items[this.expandoRowIndex].treePath.slice(0);
		var datanode = this.getItem(this.expandoRowIndex);
		treePath.push(this.store.getIdentity(datanode));
		var i = 1, idty;
		for(; i <= size; i++){
			this._treeCache.items.splice(this.expandoRowIndex + i, 0, {treePath: treePath, opened: false});
		}
		this._size += size;
		this.updateRowCount(this._size);
		for(i = 0; i < size; i++){
			if(childItems[i]){
				// tma: Changes start here
				idty = this._getUniqueIdentity(childItems[i], treePath);
				// tma: Changes end here
				this._by_idty[idty] = { idty: idty, item: childItems[i] };
				this._by_idx.splice(this.expandoRowIndex + 1 + i, 0, this._by_idty[idty]);
			}else{
				this._by_idx.splice(this.expandoRowIndex + 1 + i, 0, null);
			}
		}
		this._updateRenderedRows(this.expandoRowIndex + 1);
		this._toggleLoadingClass(this.expandoRowIndex, false);
		this.stateChangeNode = null;
		if(this._loading){
			this._loading = false;
		}
		if(this.autoHeight === true){
			this._resize();
		}
		this.focus._delayedCellFocus();
		
		//---------------------------------------------------------------------------------------------------------------------------------------------------------
		if (this.explorer.expandDeferred) {
			this.explorer.expandDeferred.callback();
		}
		//----------------------------------------------------------------------------------------------------------------------------------------------------------
	},
	
	_onFetchComplete: function(items, request, size) {
		this.inherited(arguments);
		if (request.startRowIdx === 0) {
			this.explorer.restoreLastState();
		}
	},
	
	expandoFetch: function(rowIndex, open) {
		this.inherited(arguments);
		if (!open) {
			if (this.explorer.expandDeferred) {
				this.explorer.expandDeferred.callback();
			}			
		}
	},
	
	
	createLink: function(inRowNode, inRowIndex) {
		var _this = this;
		var resourceLinks = this._resourceLinks;
		var item = this.getItem(inRowIndex);
		if (item) {
			var uri = ExplorerUtil.getResourceURIFromNodeID(this.model.getIdentity(item));
			dojo.query("a.resource-link-attach-anchor", inRowNode).forEach(
					function(n){
						if (item.isEllipsis) {
							n.onclick = dojo.hitch(_this, function(){
								this.onEllipsisClick(inRowIndex);
								return false;
							});
							n.href = "#";
						} else {
							if (resourceLinks[inRowIndex]) {
								resourceLinks[inRowIndex].destroy();
							}
							if(item.type == "http://www.w3.org/2002/07/owl#Ontology")
								resourceLinks[inRowIndex] = ExplorerUtil.createResourceLinkForOntology(uri, n);
							else
								resourceLinks[inRowIndex] = ExplorerUtil.createResourceLink(uri, n);
						}
			});			
		}
	},
	
	rowHovered: function(inRowNode, inRowIndex, isCtrlKeyPressed, isShiftKeyPressed) {
		var prevRowIndex = this.focus.rowIndex;
		if ( inRowIndex != -1 && prevRowIndex != -1 && prevRowIndex != inRowIndex) {
			if (!isCtrlKeyPressed && !isShiftKeyPressed && this.selection.getSelectedCount() <= 1) {
				this.selection.deselectAll();
				this.focus.move(inRowIndex - prevRowIndex, 0);
			}
			if (this._buttons) {
				for (var i=0; i<this._enabledButtons.length; i++) {
					this._buttons[this._enabledButtons[i]]._setEnabledAttr(true);
				}
				delete this._buttons;
				if (this._toolbar.domNode) {
					dojo.removeClass(this._toolbar.domNode, "visibleToolbar");
				}
			}			
		}
		var toolbar = this.toolbars[inRowIndex];
			if (toolbar && !toolbar._populated) {
				var model =  this.treeModel;
				var datanode = this.getItem(inRowIndex);
				var uri = ExplorerUtil.getResourceURIFromNodeID(model.store.getIdentity(datanode));
				this.explorer.populateToolbar(toolbar, inRowIndex, datanode, uri);
				toolbar._populated = true;
				
			}	
			if (this.needLink[inRowIndex]) {
				this.createLink(inRowNode, inRowIndex);
				this.needLink[inRowIndex] = false;
			}
	},
	
	_onAfterRow: function(viewIndex, inRowIndex, cells, inRowNode) {
		dojo.connect(inRowNode, "onmouseenter", dojo.hitch(this, function(event){ 
					this.rowHovered(inRowNode, inRowIndex, dojo.isCopyKey(event), event.shiftKey);
				})
		);

	},
	
	// accessibility -- if user tabs to first column, show toolbar
	_setFocusIndex: function(inRowIndex, inCellIndex, isCtrlKeyPressed, isShiftKeyPressed) {
		if (inCellIndex === 0 || inCellIndex === 2 || inCellIndex === 1) {
			var inRowNode = this.getRowNode(inRowIndex);
			this.rowHovered(inRowNode, inRowIndex, isCtrlKeyPressed, isShiftKeyPressed);	
			var toolbar = this.toolbars[inRowIndex];			
			var buttons = toolbar.getChildren();
			if (buttons.length > 0) {
				// select row
				if (isCtrlKeyPressed) {
					this.selection.addToSelection(inRowIndex);
				} else {
					this.selection.deselectAll();
					this.selection.select(inRowIndex);
				}
				// show toolbar
				if (inCellIndex === 0) {
					dojo.addClass(toolbar.domNode, "visibleToolbar");
					
					// remember toolbar for key actions
					this._toolbar = toolbar;
					this._buttons = buttons;
					this._enabledButtons = [];
					for (var i=0; i<this._buttons.length; i++) {
						if (this._buttons[i].enabled) {
							this._enabledButtons.push(i);
						}
					}
					if (this._buttonInx > this._enabledButtons.length-1) {
						this._buttonInx = this._enabledButtons.length-1;
					}
					this._refreshButtons();
				} else {
					// remember row if user presses enter
					this._openLinkOnRow = inRowIndex;
				}
			}
		}

	},
	
	onCellClick: function(e){
		this._kbhit = false;
		var prevRowIndex = this.focus.rowIndex;
		var prevCellIndex;
		if (this.focus.cell !== null){
			prevCellIndex = this.focus.cell.index;
		}
		else {
			prevCellIndex = -1;
		}
		if (this._buttons) {
			for (var i=0; i<this._enabledButtons.length; i++) {
				var ind = this._enabledButtons[i];
				this._buttons[ind]._setEnabledAttr(true);
			}
			delete this._buttons;
			
		} 
		this._buttonInx = 0;
		if (e.target && e.target.attributes && e.target.attributes.alt && e.target.attributes.alt.nodeValue === messages.remove) {
			// user clicked on delete button, don't change selection/focus
			return;
		}
		
		if (dojo.isCopyKey(e) || e.shiftKey) {
			// multi-row selection is happening, or the user clicked on delete button - in both cases, 
			// keep the current selection (default behavior)
			this.inherited(arguments);
			return;
		}		
		if (prevRowIndex !== -1 && prevCellIndex !== -1) {
			this.focus.move(e.rowIndex - prevRowIndex, e.cell.index - prevCellIndex);
		}
		else {
			this._setFocusIndex(e.rowIndex, e.cell.index, false, false);
		}
		this.inherited(arguments);
		

	}, 
	
	// accessibility-- if toolbar is shown because of tabbing, process left/right/enter keys to highlight/click on buttons
	onKeyDown: function(e) {
		this._kbhit = true;	
		var processed = false;
		if ( !(e.altKey || e.metaKey)) {
			var dk = dojo.keys;
			if (this._buttons) {
				switch(e.keyCode){
					case dk.LEFT_ARROW:
						if (this._buttonInx>0) {
							processed = true;
							this._buttonInx--;
							this._refreshButtons();
						}
						break;
					case dk.RIGHT_ARROW:
						if (this._buttonInx < this._enabledButtons.length-1) {
							processed = true;
							this._buttonInx++;
							this._refreshButtons();
						}
						break;
					case dk.ENTER:
						this._buttons[this._enabledButtons[this._buttonInx]].onClick();
						processed = true;
						break;
				}
			}
			if (this._openLinkOnRow >= 0) {
				switch(e.keyCode){
					case dk.ENTER:
						if (e.shiftKey) {
							var uri = this._resourceLinks[this._openLinkOnRow].presentationUri;
							location.href =uri;
						} else {
							processed = true;							
						}
						break;
					default:
						processed = e.shiftKey;
						break;
				}
			}
		}
		if (processed) {
			dojo.stopEvent(e);
		} else {
			if (this._buttons) {
				for (var i=0; i<this._enabledButtons.length; i++) {
					this._buttons[this._enabledButtons[i]]._setEnabledAttr(true);
				}
				delete this._buttons;
				if (this._toolbar.domNode) {
					dojo.removeClass(this._toolbar.domNode, "visibleToolbar");
				}
			}
			delete this._openLinkOnRow;
			this._noResize = true;
			this.inherited(arguments);
			this._noResize = false;
		}
	},
	
	sizeChange: function(){
		// no grid resize on key down event-- messes up accessibility selected cell (DOJO bug)
		if (!this._noResize) {
			this.inherited(arguments);
		}
	},
	
	// accessibility-- use enabled state to indicate which button is active
	_refreshButtons: function() {
		for (var i=0; i<this._enabledButtons.length; i++) {
			var enable = this._kbhit ? (i==this._buttonInx) : true;		
			this._buttons[this._enabledButtons[i]]._setEnabledAttr(enable);
		}
	},
	
	onEllipsisClick: function(inRowIndex) {
		if (inRowIndex === 0) {
			this._expandParentEllipsis();
		} else {
			this._expandSiblingEllipsis(inRowIndex);
		}
		this.updateRowCount(this._size);
		this._updateRenderedRows(inRowIndex);
	},
	
	_expandParentEllipsis: function() {
		var ellipsis = this._by_idx[0].item;
		
		// delete ellipsis item
		this._deleteByRowIndex(0, true, true);
		delete this._by_idty[ellipsis.ellipsisUUID];
		
		var numberOfElements = this._size;
		var rowIndex, item, siblings, i, id, siblingRowIndex;
		var parent_chain = [];
		
		// prepare parents chain for original elements
		for (item = ellipsis.ellipsisParent; item; item = this._getItemParent(item)) {
			parent_chain.push(item);
		}
		// insert chain of parents and their children
		var treePath = [];
		for (rowIndex = 0; rowIndex < parent_chain.length; rowIndex++) {
			item = parent_chain[parent_chain.length - 1 - rowIndex];
			siblings = rowIndex === 0 ? ellipsis.rootChildren : parent_chain[parent_chain.length - rowIndex].children;
			id = this.store.getIdentity(item);
			
			// insert parent
			this._treeCache.items.splice(rowIndex, 0, { opened: true, treePath: treePath.slice(0) });
			this._by_idx.splice(rowIndex, 0, this._by_idty[this._getUniqueIdentity(item)]);
			this._size++;
			
			// insert siblings of the parent excluding the parent
			for (i = 0; i < siblings.length; i++) {
				if (id !== this.store.getIdentity(siblings[i])) {
					siblingRowIndex = numberOfElements + rowIndex + i;
					this._treeCache.items.splice(siblingRowIndex, 0, { opened: false, treePath: treePath.slice(0) });
					this._by_idx.splice(siblingRowIndex, 0, this._by_idty[this._getUniqueIdentity(siblings[i])]);
					this._size++;
				}
			}
			
			treePath.push(id);
		}
		// update tree path for original elements
		for (; rowIndex < numberOfElements; rowIndex++) {
			this._treeCache.items[rowIndex].treePath.splice(0, 1, treePath);
		}
	},
	
	_expandSiblingEllipsis: function(inRowIndex) {
		var tp, new_children, parent_item, i, new_rowIdx, new_idty;
		
		ellipsis = this._by_idx[inRowIndex].item;
		parent_item = ellipsis.ellipsisParent;
		tp = this._treeCache.items[inRowIndex].treePath;
		
		if (!parent_item) {
			return;
		}

		// delete ellipsis
		this._deleteByRowIndex(inRowIndex, true);
		delete this._by_idty[ellipsis.ellipsisUUID];
		
		new_children = ellipsis.childrenReplacedByEllipsis.slice(0);
		
		for (i = 0; i < new_children.length; i++) {
			new_rowIdx = i + inRowIndex;
			this._treeCache.items.splice(new_rowIdx, 0, { opened: false, treePath: tp });
			this._by_idx.splice(new_rowIdx, 0, this._by_idty[this._getUniqueIdentity(new_children[i])]);
		}
		this._size += new_children.length;
		
		new_children.splice(0, 0, parent_item.children[0]);
		parent_item.children = new_children;
	},
	
	createParentEllipsis: function(item) {
		var rowIdx = this._getRowIndex(item);
		var cacheItem = this._treeCache.items[rowIdx];
		var parent_item = this._getItemParent(item);
		var grand_parent_item = this._getItemParent(parent_item);
		var grand_parent_rowIdx = this._getRowIndex(grand_parent_item);
		var grand_parent_cacheItem = grand_parent_rowIdx ? this._treeCache.items[grand_parent_rowIdx] : null;
		var grand_grand_parent_item = this._getItemParent(grand_parent_item);
		var ellipsis, i, toDelete = [];
		var rootChildren = [];
		
		if (grand_parent_cacheItem && grand_grand_parent_item) {
			for (i = 0; i < this._treeCache.items.length; i++) {
				if (!this.isChildOfTreePath(grand_parent_cacheItem.treePath, this._treeCache.items[i].treePath)) {
					if (this._treeCache.items[i].treePath.length === 0) {
						rootChildren.push(this._by_idx[i].item);
					}
					toDelete.push(this._by_idx[i].item);
				}
			}
				
			dojo.forEach(toDelete, dojo.hitch(this, function(entry, index) {
				this._onDelete(entry, true, true);
			}));

			// create ellipsis item
			ellipsis = this.explorer._createEllipsisItem(grand_parent_cacheItem.treePath.length, grand_grand_parent_item, []);
			ellipsis.children = grand_grand_parent_item.children;
			ellipsis.rootChildren = rootChildren;
				
			// insert ellipsis item into grid maps
			this._treeCache.items.splice(0, 0, { opened: true, treePath: [] });
			this._by_idty[ellipsis.ellipsisUUID] = { idty: ellipsis.ellipsisUUID, item: ellipsis };
			this._by_idx.splice(0, 0, this._by_idty[ellipsis.ellipsisUUID]);
			this._size++;

			// update tree paths for the remaining elements. Skip the parent ellipsis (i = 1).
			for (i = 1; i < this._treeCache.items.length; i++) {
				this._treeCache.items[i].treePath.splice(0, grand_parent_cacheItem.treePath.length, ellipsis.ellipsisUUID);
			}
		}
	},
	
	isChildOfTreePath: function(parent, child) {
		if (parent.length <= child.length) {
			var idx = 0;
			for (; idx < parent.length && parent[idx] === child[idx]; idx++);
			return idx === parent.length;
		}
	},
	
	isChildOf: function(parent, child) {
		var itr_child = child;
		for (; itr_child && itr_child !== parent; itr_child = this._getItemParent(itr_child));
		return itr_child;
	},
	
	createSiblingEllipsis: function(item) {		
		var parent_item = this._getItemParent(item);
		var rowIdx = this._getRowIndex(item);
		var cacheItem = this._treeCache.items[rowIdx];
		var i;
		
		if (!parent_item) {
			return;
		}
		
		var siblings = parent_item.children;
		
		if (siblings.length > 2) {
			var new_children = [ item ];
			var ellipsis = this.explorer._createEllipsisItem(siblings.length-1, parent_item, []);
			ellipsis.childrenReplacedByEllipsis = [];
			ellipsis.itemOldIndex = 0;
			for (i = 0; i < siblings.length; i++)
			{
				if (siblings[i] !== item) {
					ellipsis.childrenReplacedByEllipsis.push(siblings[i]);
				} else {
					ellipsis.itemOldIndex = i;
				}
			}
			
			// Insert the ellipsis
			var i = rowIdx+1;
			for (; i < this._size; i++) {
				var tp = this._treeCache.items[i] ? this._treeCache.items[i].treePath : [];
				if (this._treeCache.items[i].treePath.length <= cacheItem.treePath.length) {
					break;
				}
			}
			this._treeCache.items.splice(i, 0, { opened: false, treePath: cacheItem.treePath.slice(0) });

			var idty = ellipsis.ellipsisUUID;
			
			this._by_idty[idty] = { idty: idty, item: ellipsis };
			this._by_idx.splice(i, 0, this._by_idty[idty]);
			new_children.push(ellipsis);
			parent_item.children = new_children;
			this._size++;
			
			// Remove all siblings after ellipsis is added such that rowIdx is not needed to be recalculated
			dojo.forEach(ellipsis.childrenReplacedByEllipsis, dojo.hitch(this, function(item, index) {
				this._onDelete(item, true);
			}));
		}
	},
	
	_onDelete: function(item, noUpdate, dontIncludeChildren) {
		this._deleteByRowIndex(this._getRowIndex(item), noUpdate, dontIncludeChildren);
	},
	
	_deleteByRowIndex: function(rowIndex, noUpdate, dontIncludeChildren) {
		if (rowIndex >= 0) {
			var idty = this._by_idx[rowIndex].idty;
			var items = this._treeCache.items, treePath = items[rowIndex] ? items[rowIndex].treePath : [], tp, count = 1;
			if (!dontIncludeChildren) {
				var i = rowIndex + 1;
				for (; i < this._size; i++, count++) {
					tp = items[i] ? items[i].treePath : [];
					if (items[i].treePath.length <= treePath.length) {
						break;
					}
				}
			}
			items.splice(rowIndex, count);
			this._onCleanupExpandoCache(idty);
			this._by_idx.splice(rowIndex, count);
			this._size -= count;
			if (!noUpdate) {
				 this.updateRowCount(this._size);
				 this._updateRenderedRows(rowIndex); 
			}
		}
	},
	
	focusRow: function(rowIdx) {
		var item = this._by_idx[rowIdx].item;
		var original_item = item;
		var updatedRowInex;
		
		//expand parent ellipsis if there is one
		if (this._by_idx[0].item.isEllipsis) {
			this._expandParentEllipsis();
		}
		
		this.collapseIrrelevantNodes(item);		
		
		do {
			this.createSiblingEllipsis(item);
			item = this._getItemParent(item);
		} while (item);
		this.createParentEllipsis(original_item);
		this.updateRowCount(this._size);
		this._updateRenderedRows(0);
		updatedRowInex = this._getRowIndex(original_item);
		if (updatedRowInex >= 0 && !this._treeCache.items[updatedRowInex].opened) {
			this.explorer.toggleExpandRow(updatedRowInex);
		}
	},
	
	_getItemParent: function(item) {
		if (!item) {
			return;
		}
		
		var parent_item = this.store.getParent(item);
		if (parent_item) {
			return parent_item.children ? parent_item : this._getItemParent(parent_item);
		}
		return null;
	},
	
	_getRowIndex: function(item) {
		 var i = 0, idty = this.store.getIdentity(item);
		 for (; i < this._by_idx.length; i++) {
			 var id = this._by_idx[i].idty;
			 if (id.indexOf(idty, id.length - idty.length) !== -1) {
				 return i;
			 }
		 }
		 return -1;
	},
	
	collapseIrrelevantNodes: function(item){
		var cache = this._treeCache.items;
		for (var i = 0; i < cache.length; i++) {
			if (cache[i].opened && !this.isChildOf(this._by_idx[i].item, item)) {
				this.explorer.toggleExpandRow(i);
			}
		}
	},
	
	startsWith: function(str, prefix) {
		return str.length >= prefix.length && str.substr(0, prefix.length) === prefix;
	},
	
	replaceString: function(where, old_s, new_s) {
		return new_s + where.substring(old_s.length, old_s.length + where.length - old_s.length);
	},
	
	replaceTreePath: function(oldPath, newPath, startIdx) {
		var cache = this._treeCache.items;
		for (var i = startIdx; i < cache.length; i++) {
			if (this.startsWith(cache[i].treePath, oldPath)) {
				cache[i].treePath = this.replaceString(cache[i].treePath, oldPath, newPath);
			} else {
				break;
			}
				
		}
		
	},
	
	dokeydown: function(e){
		if(e.altKey || e.metaKey){
			return;
		}
		var dk = dojo.keys;
		if (e.keyCode === dk.ENTER && !e.shiftKey)
			// SHIFT+ENTER is used to open resource edit
		{
			expando = dijit.findWidgets(e.target)[0];
			if(expando instanceof dojox.grid._LazyExpando){
				expando.onToggle();
			}
		}
		this.onKeyDown(e);
	},
	
	_onSet: function(item, attribute, oldValue, newValue) {
		if (newValue === undefined) {
			// Ignore these, since this happens for our partial objects (see memento in ExplorerStore)
			return;
		}
		
		this.inherited(arguments);
	}

});

dojo.extend(dojox.grid._LazyExpando, {
	onToggle: function(event, bNotUser){
		// Summary
		//		Function for expand/collapse row
		if (!bNotUser) {
			this.view.grid.explorer.saveAfterExpand();
		}
		
		this.setOpen(!this.view.grid._treeCache.items[this.rowIdx].opened);
		if (event) {
			try{
				dojo.stopEvent(event);
			}catch(e){}
		}
	}
});

}());