/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.search.AbstractSearchResultsTable");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.toolbar.Button");

dojo.require("jazz.ui.ResourceLink");

dojo.require("rmps.app.core.util");

dojo.require("rmps.ui.explorer.ExplorerUtilities");

dojo.require("rmps.core.error");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.core.streams.StreamClient");
dojo.require("rmps.core.RootServices");

dojo.require("rmps.ui.Table");

dojo.require("dojo.i18n");

dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");

dojo.requireLocalization("rmps.app.ui.internal.search", "AbstractSearchResultsTableMessages");

var ResourceLink = jazz.ui.ResourceLink;
var QueryUtil = rmps.app.core.internal.util.QueryUtil;
var CoreUtil = rmps.app.core.util;
var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;
var StreamClient= rmps.core.streams.StreamClient;
var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;
var DesignUtil = rmps.app.ui.internal.pages.design.DesignUtil;

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.search", "AbstractSearchResultsTableMessages");

var modelsFrontService = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "models");
var reviewsFrontService = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "reviews");
var dmNamespace = "http://jazz.net/ns" +  net.jazz.ajax._contextRoot;

dojo.declare("rmps.app.ui.internal.search.AbstractSearchResultsTable", [dijit._Widget, dijit._Templated], {

	templateString: '<div class="rmps-app-search-results-table">' +
				'<div dojoAttachPoint="headerContainerNode" class="header"></div>' +
				'<div dojoAttachPoint="containerNode" class="container"></div>' +
			'</div>',

	// attach points
	headerNode: null,
	containerNode: null,

	// properties
	tableClass: "rmps.ui.Table",
	toolbar: null,
	createToolbar: true,

	// private properties
	_resultTable: null,
	_headers: null,
	_queryResult: null,

	_columnHandlers: null,

	postCreate: function() {
		if (this.createToolbar) {
			this.toolbar = new jazz.ui.Toolbar();
		dojo.addClass(this.toolbar.domNode, "toolbar");
		this.headerContainerNode.appendChild(this.toolbar.domNode);
		this.toolbar.startup();
		this.addButtons(this.toolbar);
		}

		var header = dojo.doc.createElement("div");
		header.className = "mainContent";
		this.headerContainerNode.appendChild(header);
		this.createHeader(header);

		this._columnHandlers = {};
		this._resultTable = this.createTable();
		this._headers = this.createTableHeaders(null);

		this.containerNode.appendChild(this._resultTable.domNode);
	},

	destroy: function() {
		this.inherited(arguments);
	},

	getChildren: function() {
		var children = this.inherited(arguments);
		return children.concat(dijit.findWidgets(this.headerContainerNode));
	},

	createHeader: function(parentNode) {

	},

	addButtons: function(toolbar) {
		var b2 = jazz.ui.toolbar.Button({
			title: messages.rerunQuery,
			imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/refresh.gif").toString(),
			onClick: dojo.hitch(this, this.onRefreshClick)
		});
		toolbar.addChild(b2);
	},

	onRefreshClick: function(e) {
		var queryResult = this.getQueryResult();
		if (queryResult) {
			var queryToExecute = dojo.clone(queryResult.query);
			delete queryToExecute.token;
			var timestamp = new Date().getTime();
			window.location.href = CoreUtil.getSearchUri(queryToExecute.toQueryString()) + "&ts=" + timestamp;
		}
	},

	createTable: function() {
		var tableConstructor = dojo.getObject(this.tableClass);

		return new tableConstructor({
			noEntriesLabel: messages.noResults,
			noFilterMatchesLabel: messages.noFilterMatches
		});
	},

	createTableHeader: function(id, name, args, handler) {
		var header = {
			isOrderable: false
		};
		dojo.mixin(header, args);
		header.id = id;
		header.label = name;

		this.registerColumnHandler(id, handler);
		return header;
	},

	createTableHeaders: function(queryResult) {
		var headers = [
		    this.createTableHeader("id", messages.id, {})
		];
		return headers;
	},

	getQueryResult: function() {
		return this._queryResult;
	},

	showResults: function(queryResult, preserveFilter, baseline) {
		this._queryResult = queryResult;

		// if baseline is not set use baseline from query
		if ( !baseline ) {
			baseline = queryResult.query.baseline;
		}

		this._handleResult(queryResult, preserveFilter, baseline);
	},

	_handleDefer: function(rowObj, result, property, formatter) {
		var propertyCache = "__" + property;
		if (rowObj[propertyCache]) {
			result.data = rowObj[propertyCache];
		} else {
			result.deferred = new dojo.Deferred();
			result.deferred.addCallback(function(info) {
				var fn = formatter ? formatter : function(data) {return data;};
				result.data = rowObj[propertyCache] = fn(info[property]);
				delete result.deferred;
			});
			result.deferred.addErrback(function() {
				delete rowObj[propertyCache];
				delete result.deferred;
			});
		}
	},

	_handleResourceLink: function(rowObj, result, property, rowUri) {
		result.resourceReference = true;
		var propertyCache = "__" + property;
		var viewUri = CoreUtil.getDesignViewResourceUri(rowUri);
		if (rowObj[propertyCache]) {
			result.data = rowObj[propertyCache];
			result.referenceData = {uri: rowUri, viewUri: viewUri};
		} else {
			result.deferred = new dojo.Deferred();
			result.deferred.addCallback(function(info) {
				var label = info[property];
				var uri = rowUri;
				rowObj[propertyCache] = label;
				result.data = label;
				result.referenceData = {uri: null, viewUri: null};
				if (info.existsInCurrentContext)
					result.referenceData = {uri: uri, viewUri: viewUri};
				delete result.deferred;
			});
			result.deferred.addErrback(function() {
				delete rowObj[propertyCache];
				delete result.deferred;
			});
		}
	},
	
	_handleReferenceDefer: function(rowObj, result, property, referenceId) {
		result.resourceReference = true;
		var propertyCache = "__" + property;
		var referenceCache = "__" + referenceId;
		if (rowObj[propertyCache] && rowObj[referenceCache]) {
			var uri = rowObj[referenceCache];
			var viewUri = CoreUtil.getDesignViewResourceUri(uri);
			
			result.data = rowObj[propertyCache];
			result.referenceData = {uri: rowObj[referenceCache], viewUri: viewUri};
		} else {
			result.deferred = new dojo.Deferred();
			result.deferred.addCallback(function(info) {
				var label = info[property];
				var uri = info[referenceId];
				var viewUri = CoreUtil.getDesignViewResourceUri(uri);
				rowObj[propertyCache] = label;
				rowObj[referenceCache] = uri;
				result.data = label;
				result.referenceData = {uri: null, viewUri: null};
				if (info.existsInCurrentContext)
					result.referenceData = {uri: uri, viewUri: viewUri};
				delete result.deferred;
			});
			result.deferred.addErrback(function() {
				delete rowObj[propertyCache];
				delete rowObj[referenceCache];
				delete result.deferred;
			});
		}
	},

	registerColumnHandler: function(columnId, handler) {
		if (columnId) {
			if (!handler) {
				handler = "_" + columnId + "Handler";
			}
			this._columnHandlers[columnId] = dojo.hitch(this, handler);
		}
	},

	createColumnObject: function(rowObj, columnId, owned, resourceUri, presentationUri) {
		var result = {};
		if (this._columnHandlers[columnId]) {
			this._columnHandlers[columnId](result, rowObj, owned, resourceUri, presentationUri);
		}
		return result;
	},

	getRowPresentationUri: function(rowObj) {
		return DesignUtil.getEditResourceUri(this.getRowResourceUri(rowObj));
	},

	getRowResourceUri: function(rowObj) {
		return rowObj["rdf:Description"][0]["rdf:about"];
	},
	
	getRowLinkTarget: function(rowObj) {
		return false;
	},
	
	getRowShortName: function(rowObj) {
		return rowObj["rdf:Description"][0]["rdfs:label"][0].value;
	},
	
	getRowResourceContext: function(rowObj) {
		return rowObj["rdf:Description"][0]["rmpsInfo:resourceContext"][0].value;
	},

	createRowActions: function(toolbar, rowObj, resourceUri, presentationUri) {
		this.createPreviewAction(toolbar, rowObj, resourceUri, presentationUri);
	},

	createPreviewAction: function(toolbar, rowObj, resourceUri, presentationUri){
		// preview
		var resourceLink = new jazz.ui.ResourceLink({
				uri: net.jazz.ajax._contextRoot+ "/rendering?resourceUri=" + encodeURIComponent(resourceUri),
				presentationUri: presentationUri,
				lazyFetch : true,
				label: "-",
				avoidBaselineCheck: true
			});

		toolbar.connect(toolbar, "uninitialize", function() {
			resourceLink.destroyRecursive();
		});

		var previewWidget = new jazz.ui.toolbar.Button({
			title: messages.preview,
			imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/preview.gif").toString(),
			onClick: dojo.hitch(this, function() {
				this._preview(resourceLink, previewWidget.domNode);
			})
		});
		toolbar.addChild(previewWidget);

		var s = resourceLink.domNode.style;
		s.display = "block";
		s.visibility = "hidden";
		s.position = "absolute";
		s.width = "0px";
		s.height = "0px";
		s.zIndex = -10000;
		previewWidget.domNode.insertBefore(resourceLink.domNode, previewWidget.domNode.firstChild);
	},

	_preview: function(resourceLink, node) {
		var s = resourceLink.domNode.style;

		// set the link node size to match the size of the node
		s.height = node.offsetHeight + "px";
		s.width = node.offsetWidth + "px";

		setTimeout(dojo.hitch(resourceLink, function() {
			if (this.popup && !this.popup._isOpen) {
				this.popup.open();
			}
		}), 0);
	},

	_handleError: function(deferreds) {
		// Handle errors
		var errFn = function(def) {
			def.errback({msg: "can't get info", log: false});
		};
		for (var i in deferreds) {
			dojo.forEach(deferreds[i], errFn);
		}
	},

	_handleResult: function(queryResult, preserveFilter, baseline) {
		var tableData = [];
		var deferreds = {};

		var properties = [];

		var projects = {};
		var currentProject = projects["current"] = {};

		dojo.forEach(queryResult.entries, function(entry) {
			var resourceUri = this.getRowResourceUri(entry);
			var owned = this._isOwnedResource(resourceUri);
			var presentationUri = owned ? this.getRowPresentationUri(entry) : resourceUri; 

			var row = {
				link: presentationUri,
				resourceUri: resourceUri,
				owned: owned,
				columns: []
			};

			var rowLinkTarget = this.getRowLinkTarget(entry);
			if (rowLinkTarget) {
				row.linkTarget = rowLinkTarget;
			}

			dojo.forEach(this._headers, function(header) {
				var defs = null;
				var column = this.createColumnObject(entry, header.id, owned, resourceUri, presentationUri);
				if (column.projectId) {
					var project = projects[column.projectId];
					if (!project) {
						project = projects[column.projectId] = {};
					}

					defs = deferreds[resourceUri];
					if (defs) {
						project[resourceUri] = defs;
						delete currentProject[resourceUri];
					} else {
						project[resourceUri] = deferreds[resourceUri] = []; 
					}

					delete column.projectId;
				}
				if (column.deferred) {
					defs = deferreds[resourceUri];
					if (!defs) {
						defs = currentProject[resourceUri] = deferreds[resourceUri] = [];
					}
					defs.push(column.deferred);
					if (dojo.indexOf(properties, header.id) === -1) {
						properties.push(header.id);						
					}
					
					var referenceId = header.referenceId;
					if (referenceId) {
						if (dojo.indexOf(properties, referenceId) === -1) {
							properties.push(referenceId);						
						}
					}
				}
				row.columns.push(column);
			}, this);

			tableData.push(row);

		}, this);

		for (var projectId in projects) {
			if (projectId === "current" || baseline) {
				this._handleProjectResult(deferreds, projects[projectId], properties, baseline);
			} else {
				this._detectContextAndHandleProjectResult(deferreds, projectId, projects[projectId], properties);
			}
		}

		this._showQueryResults(tableData, queryResult, preserveFilter);
	},

	_detectContextAndHandleProjectResult: function(deferreds, projectId, project, properties) {
		dojo.when(StreamClient.getDefaultStream(projectId),
				  dojo.hitch(this, function(configurationContext) {
					  this._handleProjectResult(deferreds, project, properties, configurationContext);
				  })
		);
	},
	
	

	_isOwnedResource: function(resourceUri) {
		if (resourceUri.indexOf(modelsFrontService) === 0 ||
			resourceUri.indexOf(reviewsFrontService) === 0 ||
			resourceUri.indexOf(dmNamespace) === 0 ||
			resourceUri.indexOf("http://www.w3.org/2002/07/owl") === 0 ||
			resourceUri.indexOf("http://www.w3.org/1999/02/22-rdf-syntax-ns") === 0 ||
			resourceUri.indexOf("http://www.w3.org/2000/01/rdf-schema") === 0 ||
			resourceUri.indexOf("http://www.w3.org/2001/XMLSchema") === 0 ||
			resourceUri.indexOf("http://purl.org/dc/terms") === 0) {
			return true;
		}

		return false;
	},

	_getGroupKey: function(resourceUri) {
		if (this._isOwnedResource(resourceUri)) {
			return modelsFrontService;
		} else {
			var rUrl = new dojo._Url(resourceUri);
			var servicesPath = resUrl.path;
			var applicationType = servicesPath.substring(0, servicesPath.indexOf("/", 1));

			return rUlr.scheme + "://" + rUrl.host + (rUrl.port ? ":" + rUrl.port : "") + "/" + applicationType;
		}
	},

	_handleProjectResult: function(deferreds, project, properties, baseline) {
		var resources = {};
		for (var resourceUri in project) {
			if (resourceUri) {
				var groupKey = this._getGroupKey(resourceUri);
				if (!resources[groupKey]) {
					resources[groupKey] = [];
				}
				resources[groupKey].push(resourceUri);
			}
		}

		for (var key in resources) {
			if (key) {
				dojo.when(
					this._getResourceInfo(key, resources[key], properties, baseline),
					dojo.hitch(this, function(infos) {
						dojo.forEach(infos, function(info) {
							dojo.forEach(deferreds[info.uri], function(def) {
								def.callback(info);
							});
							delete deferreds[info.uri];
							delete project[info.uri];
						});
						this._handleError(project);
					}),
					dojo.hitch(this,function(error) {
						if (deferred.ioArgs && !dojo._isDocumentOk(deferred.ioArgs.xhr)) {
							rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
						} else {
							rmps.core.error.createError(error).dispatch();
						}
						this._handleError(project);
					})
				);
			}
		}
	},

	_getResourceInfo: function(key, resourceUris, properties, baseline) {
		if (key === modelsFrontService) {
			return ResourceInfoClient.getResourceInfo(resourceUris, properties, baseline);
		}

		// key === external resource
		// potentially OSLC-queries can/should be used to obtain required info
	},

	_showQueryResults: function(tableData, queryResult, preserveFilter) {
		this._resultTable.setTableData(this._headers, tableData);

		var query = queryResult.query;
		var secondaryOrder = null;
		if (query.orderBy && query.orderBy.length > 1) {
			secondaryOrder = query.orderBy.slice();
			secondaryOrder.splice(0, 1);
		}

		//total results may be negative to indicate that the results were truncated, so
		//make sure to take the absolute value here
		this._resultTable.update(preserveFilter,
			query.page, query.pageSize,
			Math.abs(queryResult.getTotalResults()), query.limit,
			query.orderBy && query.orderBy.length > 0 ? query.orderBy[0].id : null,
			query.orderBy && query.orderBy.length > 0 ? query.orderBy[0].isAscending : false,
			secondaryOrder);

		if (queryResult.getTotalResults() < 0) {
			if (queryResult.query instanceof rmps.core.query.TextQuery) {
				this._resultTable.messageNode.innerHTML = dojo.string.substitute(messages.text_tooManyResults, [Math.abs(queryResult.getTotalResults())]);
			} else {
				this._resultTable.messageNode.innerHTML = dojo.string.substitute(messages.tooManyResults, [Math.abs(queryResult.getTotalResults())]);
			}
		}

		this._resultTable.pagingControl.domNode.style.display = "";
		this._resultTable.footerPagingControl.domNode.style.display = "";
	},

	_idHandler: function(result, rowObj, owned, resourceUri, presentationUri) {
		if (owned) {
			var number = this.getRowResourceUri(rowObj).match(/\/(\d+)(#_.*)?$/);
			if (number !== null && number[1]) {
				result.data = number[1];
			}
		}
	},

	_modificationTimeHandler: function(result, rowObj, owned, resourceUri, presentationUri) {
		if (owned) {
			if (rowObj.updated) {
				result.data = rowObj.updated[0].value;
			} else {
				this._handleDefer(rowObj, result, "modificationTime", dojo.date.stamp.toISOString);
			}
		}

		result.style = { color: "#808080" };
	},

	_actionsHandler: function(result, rowObj, owned, resourceUri, presentationUri) {
		if (owned) {
			result.className = "actions";
			// prevent row click navigation
			result.init = dojo.hitch(this, function(rowConfig, tableRow, rowIdx, columnConfig, cell, colIdx) {
				var toolbar = new jazz.ui.Toolbar();
				this.createRowActions(toolbar, rowObj, resourceUri, presentationUri);
				cell.style.width = ((toolbar.getChildren().length * 26) + 1) + "px";
				// use the Toolbar to connect so that disconnect is called on destroy
				toolbar.connect(cell, "onmousedown", dojo.hitch(dojo, dojo.stopEvent));
				tableRow.actionToolbar = toolbar;
				cell.appendChild(toolbar.domNode);
			});
		}
	}
});

})();
