/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.search.QuickSearch");

dojo.require("dijit._Widget");

dojo.require("rmps.app.core.internal.util.QueryUtil");
dojo.require("rmps.app.core.util");

dojo.require("rmps.app.core.internal.query.TextQuery");

dojo.require("jazz.ui.SearchBox");

dojo.requireLocalization("rmps.app.ui.internal.search", "QuickSearchMessages");

var QueryUtil = rmps.app.core.internal.util.QueryUtil;
var CoreUtil = rmps.app.core.util;
var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.search", "QuickSearchMessages");

dojo.declare("rmps.app.ui.internal.search.QuickSearch", dijit._Widget, {

	"class": "rmps-app-search-quick",

	// mixin properties
	pageSize: 50,
	limit: 200,
	orderBy: [{id: "modified", isAscending: false}, {id: "name", isAscending: true}],
	projectId: null,
	baseline: null,
	changeset: null,
	includeSuppliers: null,
	includeSystemDomains: null,
	includeLibraries: null,
	inLibrariesOnly: null,
	resourcesOnly: null,
	doQuery: null, // callback function
	onlyType: null,
	onlyTypeLabel: null,
	onlyDiagram: null,
	allowEmptySearches: false,

	postCreate: function() {
		var scopes = null;
		if (this.onlyType) {
			var label = QueryUtil.SCOPE.TYPE.name;
			if (this.onlyTypeLabel) {
				label = dojo.string.substitute(messages.typeSpecific, [this.onlyTypeLabel]);
			}
			scopes = [
			    { id: QueryUtil.SCOPE.TYPE.value, label: label, icon: QueryUtil.SCOPE.TYPE.icon }
			];
		}
		if (this.onlyDiagram){
			scopes = [
				{ id: QueryUtil.SCOPE.DIAGRAMS.value, label: QueryUtil.SCOPE.DIAGRAMS.name, icon: QueryUtil.SCOPE.DIAGRAMS.icon }
					];			
		}
		if (!scopes) {
			scopes = [
				{ id: QueryUtil.SCOPE.ALL.value, label: QueryUtil.SCOPE.ALL.name, icon: QueryUtil.SCOPE.ALL.icon },
				{ id: QueryUtil.SCOPE.DIAGRAMS.value, label: QueryUtil.SCOPE.DIAGRAMS.name, icon: QueryUtil.SCOPE.DIAGRAMS.icon }
			];
		}
		this.searchInputWidget = new jazz.ui.SearchBox({
			scopes: scopes,
			tooltip: messages.search,
			onExecute: dojo.hitch(this, this._handleQuickSearch),
			allowEmptySearches: this.allowEmptySearches
		});
		this.domNode.appendChild(this.searchInputWidget.domNode);
	},

	destroy: function() {
		this.searchInputWidget.destroyRecursive();
		this.inherited(arguments);
	},

	_handleQuickSearch: function(searchText, scope) {
		var query = new rmps.app.core.internal.query.TextQuery();
		query.pageSize = this.pageSize;
		query.limit = this.limit;
		query.text = dojo.trim(searchText);
		query.orderBy = this.orderBy;
		query.projectId = this.projectId;
		query.baseline = this.baseline;
		query.changeset = this.changeset;
		query.resourcesOnly = this.resourcesOnly;
		query.scope = QueryUtil.SCOPE.valueOf(scope);
		query.onlyType = this.onlyType;
		query.includeSuppliers = this.includeSuppliers;
		query.includeSystemDomains = this.includeSystemDomains;
		query.includeLibraries = this.includeLibraries;
		query.inLibrariesOnly = this.inLibrariesOnly;

		if (this.doQuery) {
			this.doQuery(query);
		}
	}
});

}());
