/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.search.QuickSearchAdvancedOptions");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.core.loader");

dojo.require("dojo.i18n");
dojo.requireLocalization("rmps.app.ui.internal.search", "QuickSearchAdvancedOptionsMessages");

dojo.require("jazz.app.internal.util.XhrUtil");
dojo.require("jazz.app.internal.HomeMenu"); 
dojo.require("jazz.app.internal.applicationAbout");

var XhrUtil = jazz.app.internal.util.XhrUtil;
var HomeMenu = jazz.app.internal.HomeMenu; 
var CURRENT_APPLICATION_ABOUT = jazz.app.internal.applicationAbout.getCurrent();

dojo.declare("rmps.app.ui.internal.search.QuickSearchAdvancedOptions", [dijit._Widget, dijit._Templated], {

	templateString: dojo.cache("rmps.app", "ui/internal/search/templates/QuickSearchAdvancedOptions.html"),
	
	widgetsInTemplate: false,
	
	_messages: dojo.i18n.getLocalization("rmps.app.ui.internal.search", "QuickSearchAdvancedOptionsMessages"),

	searchTypes: null,
	searchTypesWidget: null,
	searchScope: null,
	searchScopeId: null,
	searchScopeDescription: null,
	
	postCreate: function() {
		// type controls
		this.createTypesWidget();
		this.searchTypes = "all";
		this._allTypes.checked = true;
		
		// scope controls
		this.loadProjects();
		this.searchScope = "current";
		this._currentProject.checked = true;
		this._projectsSelect.disabled = true;
	},
	
	//creates the type selection widget
	createTypesWidget: function() {
		// load OntologyTypesWidget async to solve race condition with QueryResultsWidget
		// see WI 34821
		var jsModule = "rmps.ui.queryeditor.OntologyTypesWidget";
		jazz.core.loader.load_async(jsModule, dojo.hitch(this, function(_jsModule){
			ontologyTypesWidget = dojo.getObject(_jsModule);

			this.searchTypesWidget = new ontologyTypesWidget({
				identifier: "dm.query.ontology.type",
				conditionData: {
					ignoreContext: "false",
					presentation: "dm.query.ontology.type"
				},
				_onSelectionChange: dojo.hitch(this,"_onTypesSelectionChanged")
			});
			this.searchTypesWidget.setMultiple(false);
			this.searchTypesWidget.setEnabled(false);
			this._typesOnlySelect.appendChild(this.searchTypesWidget.domNode);
			
		},jsModule));
	},
	
	//loads the projects available on the server 
	loadProjects: function() {

		var def = XhrUtil.getXml(CURRENT_APPLICATION_ABOUT.contextRoot + "/dm-home-menu?all=true",{schema:HomeMenu.HOME_MENU_SERVICE_BINDING});

		def.addCallback(this, function(doc) {
			if (doc) {
				doc.MenuItems.sort(function (a, b){					
					if (!a || !a.title) return 1;
					else if (!b || !b.title) return -1;
					else if (a.title.toLowerCase() == b.title.toLowerCase()) return 0;
					return (a.title.toLowerCase() < b.title.toLowerCase()) ? -1 : 1;
				});				
				
				dojo.forEach(doc.MenuItems, dojo.hitch(this, function(item) {
					var name = item.title;
					var link = item.resource;
					var id = null;
					
					if (link) {
						id = link.match(/\/(_.+)$/);
					}
					
					if ( name && id ) {
						this._createOption(this._projectsSelect, name, id[1],false);
					} else {
						console.log("could not extract project from menuItem[" + i + "]: name = " + name + " id = " + id);
					}
				}));
			}
		});
	},
	
	_createOption: function(selectNode, label, value, selected) {
		var option = dojo.create("option");
		option.label = label;
		option.value = value;
		option.appendChild(dojo.doc.createTextNode(label));
		if (selected) {
			option.selected = true;
		}
		selectNode.appendChild(option);
	},
	
	_onSearchTypesChanged: function(event) {
		this.searchTypes = event.target.value;
		
		switch (this.searchTypes) {
		case "all":
			this.searchTypeId = null;
			this.searchTypesWidget.setEnabled(false);
			break;
		case "selection":
			this.searchTypesWidget.setEnabled(true);
			this._onTypesSelectionChanged();
			break;
		}
	},
	
	_onTypesSelectionChanged: function() {
		var selected = this.searchTypesWidget.getSelectedValues();
		if (selected && selected[0]) {
			this.searchTypeId = selected[0].id;
		} else {
			this.searchTypeId = null;
		}
	},
	
	_onSearchScopeChanged: function(event) {
		this.searchScope = event.target.value;
		
		switch (this.searchScope) {
		case "all":
			this.searchScopeDescription = this._messages.allProjects;
			this.searchScopeId = null;
			this._projectsSelect.disabled = true;
			break;
		case "current":
			this.searchScopeDescription = null;
			this.searchScopeId = null;
			this._projectsSelect.disabled = true;
			break;
		case "selection":
			this._projectsSelect.disabled = false;
			this._onProjectSelectionChanged();
			break;
		}
	},

	_onProjectSelectionChanged: function() {
		//multiple projects are not supported
		dojo.forEach(this._projectsSelect.options,function(option){
			if (option.selected) {
				this.searchScopeId = option.value;
				this.searchScopeDescription = option.label;
			}
		},this);
	}
});

})();
