/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.search.ReviewSearchResults");

dojo.require("rmps.app.ui.internal.search.AbstractSearchResultsTable");
dojo.require("rmps.ui.Table");

dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.core.error");
dojo.require("rmps.core.explorer.ExplorerConstants");
dojo.require("rmps.app.core.internal.util.QueryUtil");
dojo.require("rmps.app.core.util");

dojo.require("dojo.DeferredList");

dojo.require("dojo.html");
dojo.require("dojo.i18n");

dojo.require("jazz.ui.ResourceLink");

dojo.requireLocalization("rmps.app.ui.internal.search", "ReviewSearchResultsMessages");

dojo.require("rmps.ui.explorer.ExplorerUtilities");
var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;

var QueryUtil = rmps.app.core.internal.util.QueryUtil;
var CoreUtil = rmps.app.core.util;
var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.search", "ReviewSearchResultsMessages");

dojo.declare("rmps.app.ui.internal.search.ReviewSearchResults", rmps.app.ui.internal.search.AbstractSearchResultsTable, {

	"class": "rmps-app-search-results",

	tableClass: "rmps.app.ui.internal.search._Table",

	// private properties
	_descriptionNode: null,
	_titleNode: null,
	_clipboardGlow: null, 

	postCreate: function() {
		this.domNode.style.overflow = "auto";
		this.inherited(arguments);
	},

	destroy: function() {
		if (this._clipboardGlow) {
			this._clipboardGlow.destroy();
			delete this._clipboardGlow;
		}

		this.inherited(arguments);
	},

	resize: function(size) {
		if (size && size.h) {
			this.domNode.style.height = size.h + "px";
		}
	},

	createTable: function() {
		var table = this.inherited(arguments);
		table.search = this;
		return table;
	},

	createHeader: function(parentNode) {
		dojo.addClass(parentNode, "rmps-app-search-results-header");
		var titleContainer = dojo.doc.createElement("span");
		titleContainer.className = "titleContainer";
		this._titleNode = dojo.doc.createElement("span");
		this._descriptionNode = dojo.doc.createElement("span");
		this._descriptionNode.className = "description";

		titleContainer.appendChild(this._titleNode);
		titleContainer.appendChild(this._descriptionNode);
		parentNode.appendChild(titleContainer);

		this.inherited(arguments);
	},

	onNameFilterClick: function(e) {
		dojo.stopEvent(e);
		var resourceURIs = [];
		dojo.forEach(this._queryResult.entries, function(entry) {
			var resourceURI = this.getRowResourceUri(entry);
			resourceURIs.push(resourceURI);
		}, this);
		dojo.publish("rmps.web.app.explorer.focusMany", [{
			uris: resourceURIs,
			explorerID: rmps.core.explorer.ExplorerConstants.IDs.EXPLORER_SIDEBAR_DESIGN_PAGE	
		}]);
	},

	getRowPresentationUri: function(rowObj) {
		return CoreUtil.getDesignViewResourceUri(this.getRowResourceUri(rowObj));
	},

	getRowResourceUri: function(rowObj) {
		return rowObj["rdf:Description"][0]["rdf:about"];
	},

	getRowShortName: function(rowObj) {
		return rowObj["rdf:Description"][0]["rdfs:label"][0].value;
	},

	createTableHeaders: function() {
		var headers = this.inherited(arguments);

		headers.push(this.createTableHeader("shortName", messages.shortName, {filterParticipant: true, isOrderable: true}));
		headers.push(this.createTableHeader("moderator", messages.moderator, {filterParticipant: true, isOrderable: true}));
		headers.push(this.createTableHeader("modificationTime", messages.modificationTime, {formatter: QueryUtil.getModifiedLabel, isOrderable: true, defaultAscending: false}));
		headers.push(this.createTableHeader("actions", messages.actions, {}));

		return headers;
	},

	_exploreTree: function(resourceUri) {
		ExplorerUtil.showItemInSidebarExplorer(resourceUri, rmps.core.explorer.ExplorerConstants.IDs.EXPLORER_SIDEBAR_REVIEW_PAGE);		
	},

	showResults: function(queryResult, preserveFilter, baseline) {
		if (this.getQueryResult() !== queryResult) {
			if (this._titleNode){
				this.setTitleNode(queryResult, this._titleNode);	
			}
			if (this._descriptionNode){
				this.setDescriptionNode(queryResult, this._descriptionNode);
			}
			this.inherited(arguments);
		}
	},

	setTitleNode: function(queryResult, titleNode){
		dojo.html._setNodeContent(titleNode, dojo.doc.createTextNode(queryResult.query.name + " "), true);
	},

	setDescriptionNode: function(queryResult, descriptionNode){
		if (queryResult.query.description) {
			if (queryResult.query.scopeDescription) {
				dojo.html._setNodeContent(descriptionNode, dojo.doc.createTextNode(
						dojo.string.substitute(messages.searchDescriptionWithScope,
								[queryResult.query.description, queryResult.query.scopeDescription])), true);
			} else {
				dojo.html._setNodeContent(descriptionNode, dojo.doc.createTextNode(
						dojo.string.substitute(messages.searchDescription, [queryResult.query.description])), true);
			}
		} else {
			dojo.html._emptyNode(descriptionNode);
		}
	},

	_shortNameHandler: function(result, rowObj, owned, resourceUri, presentationUri) {
		if (owned) {
			result.data = this.getRowShortName(rowObj);
			result.link = true;
			if (!result.data) {
				this._handleDefer(rowObj, result, "shortName");
			}
		}
	},

	_moderatorHandler: function(result, rowObj, owned, resourceUri, presentationUri) {
		if (owned) {
			// prevent row click navigation
			result.init = dojo.hitch(this, function(rowConfig, tableRow, rowIdx, columnConfig, cell, colIdx) {
				var uri = rowObj.author[0].uri[0].value;
				var name = rowObj.author[0].name[0].value;
			
				var link = new jazz.ui.ResourceLink({ label: name, uri: uri });
				// use the link to connect so that disconnect is called on destroy
				link.connect(cell, "onmousedown", dojo.hitch(dojo, dojo.stopEvent));
				cell.appendChild(link.domNode);
			});
			result.data = rowObj.author[0].name[0].value;
		}
	}
});

dojo.declare("rmps.app.ui.internal.search._Table", rmps.ui.Table, {

	search: null,
	_deferredList: null,
	
	sortTableData: function() {
		// already sorted
	},

	setOrder: function(order) {
		// requires a new query
	},

	redraw: function() {
		// requires a new query
	},

	_populateTableBody: function() {
		this._populatePagedTableBody(0, this.search.getQueryResult().entries.length);
	},

	getPageHref: function(page, pageSize, updateTimestamp) {
		var queryResult = this.search.getQueryResult();
		var newQuery = dojo.clone(queryResult.query);
		newQuery.pageSize = pageSize;
		newQuery.page = page;
		newQuery.token = queryResult.id;

		var href = CoreUtil.getSearchUri(newQuery.toQueryString());
		if (updateTimestamp) {
			var timestamp = new Date().getTime();
			href += "&ts=" + timestamp;
		}

		return href;
	},

	getReorderHref: function(orderBy, isAscending, secondaryOrder) {
		var queryResult = this.search.getQueryResult();
		var newQuery = dojo.clone(queryResult.query);
		delete newQuery.token;
		delete newQuery.page;
		newQuery.token = queryResult.id;

		if (orderBy) {
			newQuery.orderBy = [{'id': orderBy, 'isAscending': isAscending}];
			if (secondaryOrder) {
				newQuery.orderBy = newQuery.orderBy.concat(secondaryOrder);
			}
		}
		return CoreUtil.getSearchUri(newQuery.toQueryString());
	},

	_filterChanged: function(filter) {
		var queryResult = this.search.getQueryResult();

		if (this._deferredList) {
			delete this._deferredList;
		}

		if (!filter) {
			queryResult.filtered = null;
			if (this._prevFilter) {
				this._prevFilter = filter;
			}

			var href = this.getPageHref(0, this.pagingControl.pageSize, true);
			if (href && href.length > 0) {
				window.location.href = href;
			}
			return;
		}

		queryResult.filtered = [];

		var properties = {};
		var deferredProperties = [];
		var deferredEntries = [];
		var resourceUris = [];

		dojo.forEach(queryResult.rawEntries, function(entry) {
			var resourceUri = this.search.getRowResourceUri(entry);
			var presentationUri = this.search.getRowPresentationUri(entry);

			if (entry.deferred) {
				delete entry.deferred;
			}

			dojo.forEach(this.search._headers, function(header) {
				if (dojo.indexOf(queryResult.filtered, entry) >= 0) {
					return;
				}

				var column = this.search.createColumnObject(entry, header.id, resourceUri, presentationUri);
				if (column.deferred) {
					properties[header.id] = true;

					if (!entry.deferred) {
						resourceUris.push(resourceUri);
						column.header = header;						

						entry.deferred = new dojo.Deferred();
						entry.deferred.uri = resourceUri;
						entry.deferred.columns = [];

						entry.deferred.addCallback(dojo.hitch(this, function(info) {
							var columns = entry.deferred.columns;

							var filtered = false;
							for (var c = 0; c < columns.length; ++c) {
								columns[c].deferred.callback(info);
								if (!filtered && !this._filterColumn(filter, columns[c].header, columns[c])) {
									queryResult.filtered.push(entry);
									filtered = true;
								}
							}

							delete entry.deferred;
						}));						
					}

					entry.deferred.columns.push(column);					
				} else {
					if (!this._filterColumn(filter, header, column)) {
						queryResult.filtered.push(entry);
						if (entry.deferred) {
							delete entry.deferred;
						}
					}
				}
			}, this);

			if (entry.deferred) {
				deferredEntries.push(entry.deferred);
			}
		}, this);

		for (var property in properties) {
			deferredProperties.push(property);
		}

		// split the request into chunks of 25 uris
		var deferreds = [];		
		var queryUris = [];
		for (var r = 0; r < resourceUris.length; ++r) {
			queryUris.push(resourceUris[r]);
			if (queryUris.length === 25 || r === (resourceUris.length - 1)) {
				var deferred = rmps.core.resource.ResourceInfoClient.getResourceInfo(queryUris, deferredProperties);
				deferred.addCallback(function(infos) {
					for (var i = 0; i < infos.length; ++i) {
						for (var j = 0; j < deferredEntries.length; ++j) {
							if (deferredEntries[j].uri === infos[i].uri) {
								deferredEntries[j].callback(infos[i]);
								deferredEntries.splice(j, 1);
								break;
							}
						}
					}
				});

				deferred.addErrback(function(error) {
					if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
						rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
					} else {
						rmps.core.error.createError(error).dispatch();
					}
				});

				deferreds.push(deferred);
				queryUris = [];
			}
		}

		this._deferredList = new dojo.DeferredList(deferreds);
		this._deferredList.addCallback(dojo.hitch(this, function(){
			var href = this.getPageHref(0, this.pagingControl.pageSize, true);
			if (href && href.length > 0) {
				setTimeout(function(){window.location.href = href;}, 0); 
			}
			this._prevFilter = filter;
			delete this._deferredList;
		}));
	}
});

})();
