/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.search.SearchBanner");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("rmps.ui.PagingControl");
dojo.require("jazz.ui.Breadcrumbs");

dojo.require("rmps.app.core.util");

var CoreUtil = rmps.app.core.util;

dojo.declare("rmps.app.ui.internal.search.SearchBanner", [dijit._Widget, dijit._Templated], {
	
	templateString: dojo.cache("rmps.app", "ui/internal/search/templates/SearchBanner.html"),
	
	widgetsInTemplate: true,
	
	// private properties
	_queryResult: null,
	
	postCreate: function() {
		this._pagingControl.search = this;
	},
	
	getBreadCrumbLabel: function() {
		return this._queryResult.query.name;
	},
	
	setQueryResult: function(queryResult) {
		if (this._queryResult === queryResult) {
			return;
		}
		this._queryResult = queryResult;
		this._breadCrumbs.segments = [];
		if (this._queryResult) {
			this._breadCrumbs.segments.push({
				label: this.getBreadCrumbLabel(),
				href: CoreUtil.getSearchUri(this._queryResult.query.toQueryString())
			});
			
			//total results may be negative to indicate that results were truncated, so remember to take the absolute value
			this._pagingControl.update(
				this._queryResult.query.page,
				this._queryResult.query.pageSize,
				this._queryResult.query.limit,
				Math.abs(this._queryResult.getTotalResults()),
				this._queryResult.query.page * this._queryResult.query.pageSize + 1
			);
		} else {
			this._pagingControl.update(0, -1, -1, 0, 0);
		}
		this._breadCrumbs.refresh();
	},
	
	navigateTo: function(uri) {
		var idx = this._getSearchResourceIndex(uri);
		
		if (idx === -1) {
			return false;
		}
		
		idx += this._pagingControl.pageSize * this._pagingControl.page + 1;
		this._pagingControl.update(null, null, null, null, idx);
		
		return true;
	},
	
	_getSearchResourceIndex: function(uri) {
		if (this._queryResult) {
			var entries = this._queryResult.entries;
			for (var i = 0; i < entries.length; ++i) {
				var entry = entries[i];
				var contentUri = entry["rdf:Description"][0]["rdf:about"];
				if (contentUri == uri) {
					return i;
				}
			}
		}
		return -1;
	}
});

var NUM_LIST_DETAILS = 5;

dojo.declare("rmps.app.ui.internal.search._PagingControl", rmps.ui.PagingControl, {
	
	search: null,
	hasDetails: true,
	
	_getLowerLimit: function() {
		return this.page * this.getPageSize() + 1;
	},
	
	_getUpperLimit: function() {
		return this.page * this.getPageSize() + this.search._queryResult.entries.length;
	},
	
	hasNext: function() {
		if (this.search && this.search._queryResult) {
			return this.item < this._getUpperLimit();
		}
		return false;
	},
	
	hasPrevious: function() {
		if (this.search && this.search._queryResult) {
			return this._getLowerLimit() < this.item;
		}
		return false;
	},
	
	getPreviousHref: function() {
		var idx = this.item - (this.page * this.getPageSize()) - 2;
		var uri = this.search._queryResult.entries[idx]["rdf:Description"][0]["rdf:about"];
		return CoreUtil.getDesignViewResourceUri(uri);
	},
	
	getNextHref: function() {
		var idx = this.item - (this.page * this.getPageSize());
		var uri = this.search._queryResult.entries[idx]["rdf:Description"][0]["rdf:about"];
		return CoreUtil.getDesignViewResourceUri(uri);
	},
	
	getNextItemLabel: function() {
		var idx = this.item - (this.page * this.getPageSize());
		var entry = this.search._queryResult.entries[idx];
		return this._getName(entry);
	},
	
	getPreviousItemLabel: function() {
		var idx = this.item - (this.page * this.getPageSize()) - 2;
		var entry = this.search._queryResult.entries[idx];
		return this._getName(entry);
	},
	
	_getName: function(entry) {
		var name = entry["rdf:Description"][0]["rdfs:label"][0].value;
		return name ? name : '-';
	},
	
	getNextDetails: function() {
		var start = this.item - (this.page * this.getPageSize());
		var end = Math.min(start + NUM_LIST_DETAILS, this.search._queryResult.entries.length);
		return this._getDetails(start, end, false);
	},
	
	getPreviousDetails: function() {
		var end = this.item - (this.page * this.getPageSize()) - 1;
		var start = Math.max(end - NUM_LIST_DETAILS, 0);
		return this._getDetails(start, end, true);
	},
	
	_getDetails: function(start, end, isPrevious) {
		var container = dojo.doc.createElement("div");
		container.className = "rmps-app-search-banner-details";
		
		for (var i = start; i < end; ++i) {
			var entry = this.search._queryResult.entries[i];
			var uri = entry["rdf:Description"][0]["rdf:about"];
			
			var row = dojo.doc.createElement("div");
			var link = dojo.doc.createElement("a");
			link.href = CoreUtil.getDesignViewResourceUri(uri);
			link.appendChild(dojo.doc.createTextNode(this._getName(entry)));
			row.appendChild(link);
			
			if (isPrevious) {
				container.insertBefore(row, container.firstChild);
			} else {
				container.appendChild(row);
			}
		}
		
		return container;
	}
});

})();
