/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.search.SearchResults");

dojo.require("rmps.app.ui.internal.search.AbstractSearchResultsTable");
dojo.require("rmps.ui.Table");

dojo.require("rmps.core.RootServices");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.core.error");
dojo.require("rmps.core.util.ILinksConstants");
dojo.require("rmps.core.clipboard.Clipboard");
dojo.require("rmps.core.explorer.ExplorerConstants");
dojo.require("rmps.ui.common.ToolbarButtonGlow");

dojo.require("rmps.app.core.internal.util.QueryUtil");
dojo.require("rmps.app.core.util");
dojo.require("rmps.ui.util.UIUtil");

dojo.require("dojo.DeferredList");

dojo.require("dojo.html");
dojo.require("dojo.i18n");

dojo.require("jazz.ui.ResourceLink");

dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
var DesignUtil = rmps.app.ui.internal.pages.design.DesignUtil;
var Util = rmps.app.core.util;

var PROJECTS_URI= rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "web") + "/projects/";

dojo.require("rmps.ui.explorer.ExplorerUtilities");
var ExplorerUtil = rmps.ui.explorer.ExplorerUtilities;

dojo.requireLocalization("rmps.app.ui.internal.search", "SearchResultsMessages");

var QueryUtil = rmps.app.core.internal.util.QueryUtil;
var CoreUtil = rmps.app.core.util;
var ILinksConstants = rmps.core.util.ILinksConstants;
var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;
var clipboard = rmps.core.clipboard.clipboard;

var CLIPBOARD_DIALOG = "rmps.ui.clipboard.ClipboardDialog";

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.search", "SearchResultsMessages");

dojo.declare("rmps.app.ui.internal.search.SearchResults", rmps.app.ui.internal.search.AbstractSearchResultsTable, {
	
	"class": "rmps-app-search-results",
	
	tableClass: "rmps.app.ui.internal.search._Table",
	
	// private properties
	_descriptionNode: null,
	_titleNode: null,
	_clipboardGlow: null,
	_currentProjectUri: null,
	_currentProjectId: null,
	_showclipboardIcon: true,

	postCreate: function() {
		this.domNode.style.overflow = "auto";
		this.inherited(arguments);
		// get project uri and id from current project
		this._currentProjectUri = dojo.getObject("jazz.app.currentApplication.context");
		if(this._currentProjectUri){
			this._currentProjectId = this._currentProjectUri.substring(this._currentProjectUri.lastIndexOf("/")+1);
		} else {
			this._currentProjectId = jazz.app.context.get().itemId;
		}
	},

	destroy: function() {
		if (this._showclipboardIcon && this._clipboardGlow) {
			this._clipboardGlow.destroy();
			delete this._clipboardGlow;
		}
		
		this.inherited(arguments);
	},

	resize: function(size) {
		if (size && size.h) {
			this.domNode.style.height = size.h + "px";
		}
	},

	createTable: function() {
		var table = this.inherited(arguments);
		table.search = this;
		return table;
	},

	createHeader: function(parentNode) {
		dojo.addClass(parentNode, "rmps-app-search-results-header");
		var titleContainer = dojo.doc.createElement("span");
		titleContainer.className = "titleContainer";
		this._titleNode = dojo.doc.createElement("span");
		this._descriptionNode = dojo.doc.createElement("span");
		this._descriptionNode.className = "description";

		titleContainer.appendChild(this._titleNode);
		titleContainer.appendChild(this._descriptionNode);
		parentNode.appendChild(titleContainer);

		this.inherited(arguments);
	},
	
	addButtons: function(toolbar) {
		if ( !this._showclipboardIcon ) return;
		// Clipboard button is shown as empty initially and updated when call to clipboard returns
		var clipboardButton = jazz.ui.toolbar.Button({
			title: messages.clipboard,
			imgSrc: dojo.moduleUrl("rmps.ui", "graphics/icons/clipboard_empty_co.gif"),
			onClick: dojo.hitch(this, this.onClipboard)
		});
		toolbar.addChild(clipboardButton);
		this._clipboardGlow = new rmps.ui.common.ToolbarButtonGlow({
			button: clipboardButton,
			eventName: "rmps.core.clipboard.Change"
		});

		clipboard().addCallback(this, function(cb) {
			var glowFn = dojo.hitch(this, function(node) {
				// Clipboard updates should change button icon
				clipboardButton.set("imgSrc", cb.getIconURL());
			});
			this._clipboardGlow.onGlow = glowFn;
			// And make an immediate update
			glowFn();
		});

		var b1 = jazz.ui.toolbar.Button({
			title: messages.nameFilterQuery,
			imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/collapse_to_context_co.gif").toString(),
			onClick: dojo.hitch(this, this.focusResultsInExplorer)
		});
		toolbar.addChild(b1);
		this.inherited(arguments);
	},

	// TODO: When editors has switched to jazz.ui.Toolbar, use table addButtons function directly
	addEditorToolbarButtons: function(toolbar) {
		toolbar.addButton(this, dojo.hitch(this, this.focusResultsInExplorer), {
			label: messages.nameFilterQuery,
			imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/collapse_to_context_co.gif").toString()
		});
	},

	focusResultsInExplorer: function(e) {
		dojo.stopEvent(e);
		var resourceURIs = [];
		dojo.forEach(this._queryResult.entries, function(entry) {
			var resourceURI = this.getRowResourceUri(entry);
			resourceURIs.push(resourceURI);
		}, this);

		ExplorerUtil.focusManyItemsInSidebarExplorer(resourceURIs, rmps.core.explorer.ExplorerConstants.IDs.EXPLORER_SIDEBAR_DESIGN_PAGE);
	},

	onClipboard: function(e) {
		rmps.ui.util.UIUtil.loadAndRun(CLIPBOARD_DIALOG, {
			readOnly: true
		});
	},

	getRowPresentationUri: function(rowObj) {
		var query = this.getQueryResult().query;
		if ( query.scopeDescription !== null && (
				( query.projectId && this._currentProjectId !== query.projectId ) ||
				( rowObj["rdf:Description"]&& rowObj["rdf:Description"][0]["rmpsInfo:resourceContext"] && this._currentProjectUri !== rowObj["rdf:Description"][0]["rmpsInfo:resourceContext"][0].value )
		)) {
			return PROJECTS_URI + rowObj["rdf:Description"][0]["rmpsInfo:resourceContextName"][0].value + Util.getDesignViewResourceUri(this.getRowResourceUri(rowObj));
		}
		return Util.getDesignViewResourceUri(this.getRowResourceUri(rowObj));
	},

	getRowLinkTarget: function(rowObj) {
		var query = this.getQueryResult().query;
		if ( query.scopeDescription !== null && (
				( query.projectId && this._currentProjectId !== query.projectId ) ||
				( rowObj["rdf:Description"]&& rowObj["rdf:Description"][0]["rmpsInfo:resourceContext"] && this._currentProjectUri !== rowObj["rdf:Description"][0]["rmpsInfo:resourceContext"][0].value )
		)) {
			return "_blank";
		}
		return false;
	},

	createRowActions: function(toolbar, rowObj, resourceUri, presentationUri) {
		this.inherited(arguments);
		var query = this.getQueryResult().query;
		if ( query.scopeDescription !== null && (
				( query.projectId && this._currentProjectId !== query.projectId ) ||
				( rowObj["rdf:Description"][0]["rmpsInfo:resourceContext"] && this._currentProjectUri !== rowObj["rdf:Description"][0]["rmpsInfo:resourceContext"][0].value )
		)) {
			return;
		}
		this.createExplorerAction(toolbar, rowObj, resourceUri, presentationUri);
		if ( this._showclipboardIcon )
			this.createAddToClipboardAction(toolbar, rowObj, resourceUri, presentationUri);
	},

	createExplorerAction: function(toolbar, rowObj, resourceUri, presentationUri){
		// explore
		var explorerbutton = new jazz.ui.toolbar.Button({
			title: messages.exploreTree,
			imgSrc: dojo.moduleUrl("rmps.app", "graphics/icons/explore_tree_co.gif").toString(),
			onClick: dojo.hitch(this, function() {
				this._exploreTree(resourceUri);	
			})
		});
		toolbar.addChild(explorerbutton);
		// Check enable state of explorer button
		// TODO: Discoverable objects are visible in explorer, but if turns out that some
		// resources are not, then enabled check of 'Explore in Tree' operation should be done.
		/*toolbar.disableButton(explorerbutton);
		ExplorerUtil.canShowInTree(resourceUri).addCallback(this, function(value) {
			if (value.enabled) {
				toolbar.enableButton(explorerbutton);
			}
		});*/
	},

	createTableHeaders: function(queryResult) {
		var headers = this.inherited(arguments);
		headers.push(this.createTableHeader("shortName", messages.shortName, {filterParticipant: true, isOrderable: true}));

		if (queryResult && queryResult.query.scopeDescription && this._currentProjectId !== queryResult.query.projectId) {
			headers.push(this.createTableHeader("projectName", messages.project, {filterParticipant: true, isOrderable: true}));
		}

		//headers.push(this.createTableHeader("longName", messages.longName, {filterParticipant: true}));
		headers.push(this.createTableHeader("parentName", messages.parentName, {filterParticipant: true, resourceReference: true, referenceId: "parentUri"}));
		headers.push(this.createTableHeader("displayType", messages.displayType, {filterParticipant: true, isOrderable: true}));
		headers.push(this.createTableHeader("modificationTime", messages.modificationTime, {formatter: QueryUtil.getModifiedLabel, isOrderable: true, defaultAscending: false}));
		headers.push(this.createTableHeader("actions", messages.actions, {}));

		return headers;
	},

	_exploreTree: function(resourceUri) {
		ExplorerUtil.showItemInSidebarExplorer(resourceUri, rmps.core.explorer.ExplorerConstants.IDs.EXPLORER_SIDEBAR_DESIGN_PAGE);
	},

	createAddToClipboardAction: function(toolbar, rowObj, resourceUri, presentationUri) {
		// Add clipboard action
		var button = new jazz.ui.toolbar.Button({
			title: messages.addToClipboard,
			imgSrc: dojo.moduleUrl("rmps.ui", "graphics/icons/add_to_clipboard_co.gif").toString(),
			onClick: function() {
				var context = jazz.app.context.get();
				ResourceInfoClient.getResourceInfo([resourceUri], ["type"], context.baseline).then(function(info) {
					clipboard().addCallback(function(cb) {
						cb.push(resourceUri, info[0].type);						
					});
				});				
			}
		});
		toolbar.addChild(button);
	},

	showResults: function(queryResult, preserveFilter, baseline) {
		if (this.getQueryResult() !== queryResult) {
			this._headers = this.createTableHeaders(queryResult);

			if (this._titleNode){
				this.setTitleNode(queryResult, this._titleNode);	
			}
			if (this._descriptionNode){
				this.setDescriptionNode(queryResult, this._descriptionNode);
			}
			this.inherited(arguments);
		}
	},

	setTitleNode: function(queryResult, titleNode){
		if (queryResult && queryResult.query && queryResult.query.name) {
			var titleText = dojo.string.substitute(messages.searchResultsTitle, [queryResult.query.name]);
			dojo.html._setNodeContent(titleNode, dojo.doc.createTextNode(titleText + " "), true);
		}
	},

	setDescriptionNode: function(queryResult, descriptionNode){
		if (queryResult.query.description) {
			if (queryResult.query.scopeDescription) {
				dojo.html._setNodeContent(descriptionNode, dojo.doc.createTextNode(
						dojo.string.substitute(messages.searchDescriptionWithScope,
								[queryResult.query.description, queryResult.query.scopeDescription])), true);
			} else {
				dojo.html._setNodeContent(descriptionNode, dojo.doc.createTextNode(
						dojo.string.substitute(messages.searchDescription, [queryResult.query.description])), true);
			}
		} else {
			dojo.html._emptyNode(descriptionNode);
		}
	},

	_shortNameHandler: function(result, rowObj, owned, resourceUri, presentationUri) {
		var shortLabel = this.getRowShortName(rowObj);
		if (shortLabel) {
			result.data = shortLabel;
			result.referenceData = {uri: resourceUri, viewUri: presentationUri};
			result.resourceReference = true;
		} else {
			if (owned) {
				this._handleResourceLink(rowObj, result, "shortName", resourceUri);
			} else {
				result.data = resourceUri;
				result.link = true;
			}
		}
	},

	_projectNameHandler: function(result, rowObj, owned, resourceUri, presentationUri) {
		if (owned) {
			var projectUri = rowObj["rdf:Description"][0]["rmpsInfo:resourceContext"][0].value;
			var projectId = projectUri.substring(projectUri.lastIndexOf("/") + 1);
			if (projectId !== this._currentProjectId) {
				result.projectId = projectId;
			}
			// prevent row click navigation
			result.init = dojo.hitch(this, function(rowConfig, tableRow, rowIdx, columnConfig, cell, colIdx) {
				var name = rowObj["rdf:Description"][0]["rmpsInfo:resourceContextName"][0].value;
				// presentationUri
				var link = new jazz.ui.ResourceLink({
					label: name,
					uri: projectUri,
					presentationUri: PROJECTS_URI + name
				});
				// use the link to connect so that disconnect is called on destroy
				link.connect(cell, "onmousedown", dojo.hitch(dojo, dojo.stopEvent));
				cell.appendChild(link.domNode);
			});
			result.data = rowObj["rdf:Description"][0]["rmpsInfo:resourceContextName"][0].value;
		}
	},

	_parentNameHandler: function(result, rowObj, owned, resourceUri, presentationUri) {
		if (owned) {
			this._handleReferenceDefer(rowObj, result, "parentName", "parentUri");
		}
	},

	_longNameHandler: function(result, rowObj, owned, resourceUri, presentationUri) {
		if (owned) {
			this._handleDefer(rowObj, result, "longName");
		}
	},

	_displayTypeHandler: function(result, rowObj, owned, resourceUri, presentationUri) {
		if (owned) {
			this._handleDefer(rowObj, result, "displayType");
		} else {
			// TODO!!!
			result.data = ILinksConstants.EXTERNAL_LINK_TYPE_TITLE;
			return;
		}
	},

	_modifiedByHandler: function(result, rowObj, owned, resourceUri, presentationUri) {
		if (owned) {
			if (rowObj.contributor && rowObj.contributor[0].name[0]) {
				result.data = rowObj.contributor[0].name[0].value;
			}
		}
	}
});

dojo.declare("rmps.app.ui.internal.search._Table", rmps.ui.Table, {

	search: null,
	_deferredList: null,

	sortTableData: function() {
		// already sorted
	},

	setOrder: function(order) {
		// requires a new query
	},

	redraw: function() {
		// requires a new query
	},

	_populateTableBody: function() {
		this._populatePagedTableBody(0, this.search.getQueryResult().entries.length);
	},

	getPageHref: function(page, pageSize, updateTimestamp) {
		var queryResult = this.search.getQueryResult();
		var newQuery = dojo.clone(queryResult.query);
		newQuery.pageSize = pageSize;
		newQuery.page = page;
		newQuery.token = queryResult.id;

		var href = CoreUtil.getSearchUri(newQuery.toQueryString());
		if (updateTimestamp) {
			var timestamp = new Date().getTime();
			href += "&ts=" + timestamp;
		}

		return href;
	},

	getReorderHref: function(orderBy, isAscending, secondaryOrder) {
		var queryResult = this.search.getQueryResult();
		var newQuery = dojo.clone(queryResult.query);
		delete newQuery.token;
		delete newQuery.page;
		newQuery.token = queryResult.id;

		if (orderBy) {
			newQuery.orderBy = [{'id': orderBy, 'isAscending': isAscending}];
			if (secondaryOrder) {
				newQuery.orderBy = newQuery.orderBy.concat(secondaryOrder);
			}
		}
		return CoreUtil.getSearchUri(newQuery.toQueryString());
	},

	_filterChanged: function(filter) {
		var queryResult = this.search.getQueryResult();

		if (this._deferredList) {
			delete this._deferredList;
		}

		if (!filter) {
			queryResult.filtered = null;
			if (this._prevFilter) {
				this._prevFilter = filter;
			}

			var href = this.getPageHref(0, this.pagingControl.pageSize, true);
			if (href && href.length > 0) {
				window.location.href = href;
			}
			return;
		} else {		
			queryResult.filtered = [];
		}

		var properties = {};
		var deferredProperties = [];
		var deferredEntries = [];
		var resourceUris = [];

		dojo.forEach(queryResult.rawEntries, function(entry) {
			var resourceUri = this.search.getRowResourceUri(entry);
			var presentationUri = this.search.getRowPresentationUri(entry);

			if (entry.deferred) {
				delete entry.deferred;
			}

			dojo.forEach(this.search._headers, function(header) {
				if (dojo.indexOf(queryResult.filtered, entry) >= 0) {
					return;
				}

				var column = this.search.createColumnObject(entry, header.id, resourceUri, presentationUri);
				if (column.deferred) {
					properties[header.id] = true;

					if (!entry.deferred) {
						resourceUris.push(resourceUri);
						column.header = header;						

						entry.deferred = new dojo.Deferred();
						entry.deferred.uri = resourceUri;
						entry.deferred.columns = [];

						entry.deferred.addCallback(dojo.hitch(this, function(info) {
							var columns = entry.deferred.columns;

							var filtered = false;
							for (var c = 0; c < columns.length; ++c) {
								columns[c].deferred.callback(info);
								if (!filtered && !this._filterColumn(filter, columns[c].header, columns[c])) {
									queryResult.filtered.push(entry);
									filtered = true;
								}
							}

							delete entry.deferred;
						}));						
					}

					entry.deferred.columns.push(column);					
				} else {
					if (!this._filterColumn(filter, header, column)) {
						queryResult.filtered.push(entry);
						if (entry.deferred) {
							delete entry.deferred;
						}
					}
				}
			}, this);

			if (entry.deferred) {
				deferredEntries.push(entry.deferred);
			}
		}, this);

		for (var property in properties) {
			deferredProperties.push(property);
		}

		// split the request into chunks of 25 uris
		var deferreds = [];		
		var queryUris = [];
		for (var r = 0; r < resourceUris.length; ++r) {
			queryUris.push(resourceUris[r]);
			if (queryUris.length === 25 || r === (resourceUris.length - 1)) {
				var deferred = rmps.core.resource.ResourceInfoClient.getResourceInfo(queryUris, deferredProperties);
				deferred.addCallback(function(infos) {
					for (var i = 0; i < infos.length; ++i) {
						for (var j = 0; j < deferredEntries.length; ++j) {
							if (deferredEntries[j].uri === infos[i].uri) {
								deferredEntries[j].callback(infos[i]);
								deferredEntries.splice(j, 1);
								break;
							}
						}
					}
				});

				deferred.addErrback(function(error) {
					if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
						rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
					} else {
						rmps.core.error.createError(error).dispatch();
					}
				});

				deferreds.push(deferred);
				queryUris = [];
			}
		}

		this._deferredList = new dojo.DeferredList(deferreds);
		this._deferredList.addCallback(dojo.hitch(this, function(){
			var href = this.getPageHref(0, this.pagingControl.pageSize, true);
			if (href && href.length > 0) {
				setTimeout(function(){window.location.href = href;}, 0); 
			}
			this._prevFilter = filter;
			delete this._deferredList;
		}));
	}
});

})();
