/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("rmps.app.ui.internal.sections.ExplorerFilterSection");

    dojo.require("dijit.form.CheckBox");

    dojo.require("jazz.ui.SwapListDialog");
    dojo.require("jazz.app.context");

    dojo.require("rmps.app.ui.internal.common.ImageService");
    dojo.require("rmps.ui.sections.SectionTable");
    dojo.require("rmps.core.RootServices");
    dojo.require("rmps.core.resource.Resource");
    dojo.require("rmps.core.explorer.ExplorerConstants");

    dojo.require("dojo.i18n");
    dojo.requireLocalization("rmps.app.ui.internal.sections", "ExplorerFilterSectionMessages");
    var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.sections", "ExplorerFilterSectionMessages");
    var ImageService = rmps.app.ui.internal.common.ImageService;
    var EXPLORER_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "explorer");
    var EXPLORER_TYPE_PATH = EXPLORER_PATH + "/types";
    var EXPLORER_FILTER_PATH = EXPLORER_PATH + "/filters";
	var PROJECT_AREA_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.JP_0_6, "projectAreas");

    var SettingTypes = rmps.core.explorer.ExplorerConstants.SettingTypes;
    var FILTERS_UAKEY = "com.ibm.xtools.rmps.web.app.section_ExplorerFilterSection001";

    dojo.declare("rmps.ui.sections.ExplorerFilter", rmps.core.resource.Resource, {
    	name: null,
    	description: null,
    	excludedTypes: null,
    	enabled: false,

    	constructor: function(args) {
    		dojo.mixin(this, args);
    	},

    	create: function(proj) {
    		if (!proj) {
    			proj = jazz.app.context.get().itemId;
    		}
    		var url = EXPLORER_FILTER_PATH + "/" + proj;
    		var etag = null;
			var postArgs = {
				url: url,
				headers: {
					"Content-Type": "application/json",
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "json",
				postData: dojo.toJson({
					name: this.name,
					description: this.description,
					excludedTypes: this.excludedTypes
				})
			};
			var def = jazz.client.xhrPost(postArgs);
			def.addCallback(this, function() {
				this.uri = def.ioArgs.xhr.getResponseHeader("Location");
				this.eTag = def.ioArgs.xhr.getResponseHeader("ETag");
			});
			return def;
    	},

    	update: function() {
    		var putArgs = {
    			url: this.uri,
    			headers: {
    				"Content-Type": "application/json",
    				"If-Match": this.eTag,
    				"X-ibm-rmps-internal": "true"
    			},
    			handleAs: "json",
				putData: dojo.toJson({
					name: this.name,
					description: this.description,
					excludedTypes: this.excludedTypes
				})
    		};
    		var def = jazz.client.xhrPut(putArgs);
    		def.addCallback(this, function() {
				this.eTag = def.ioArgs.xhr.getResponseHeader("ETag");
				this.uri = def.ioArgs.xhr.getResponseHeader("Location");
    		});
    		return def;
    	},

    	del: function() {
    		var delArgs = {
    			url: this.uri,
    			headers: {
    				"If-Match": this.eTag,
    				"X-ibm-rmps-internal": "true"
    			}
    		};
    		var def = jazz.client.xhrDelete(delArgs);
    		def.addCallback(this, function() {
    			this.uri = null;
    			this.eTag = null;
    		});
    		return def;
    	}
    });

    dojo.declare("rmps.app.ui.internal.sections.ExplorerFilterSection", rmps.ui.sections.SectionTable, {
    	
    	projectId: null,

        constructor: function(){
    		this.addColumnDescriptor({
    			columnHeader: messages.filterEnableFilter,
    			fieldName: "enabled",
    			width: "5%",
    			creatorCallback: this._creatorCheckbox
    		});
			this.addColumnDescriptor({
	            columnHeader: messages.filterNameHeader,
	            fieldName: "name",
	            width: "30%",
	            creatorCallback: this.creatorCallBackDefaultFilter
	        });
	        this.addColumnDescriptor({
	            columnHeader: messages.filterDescriptionHeader,
	            fieldName: "description",
	            width: "65%",
	            creatorCallback: this.creatorCallBackDefaultFilter
	        });

    		this.addHeader("rmps.ui.sections.EditableSectionHeader", this.getHeaderArguments());
            this.addHeader("rmps.ui.sections.DescriptionHeader", {sectionDescription:messages.explorerFilterSectionDescription});

            this.addActionDescriptor(this.createDefineFilterAction);
            this.addActionDescriptor(this.createDeleteAction);
        },

		getHeaderArguments: function(){
			return {
				sectionTitle: 		messages.explorerFilterSectionTitle,
				sectionDescription: messages.explorerFilterSectionDescription,
				section: this,
				uaKey: FILTERS_UAKEY,
				actions: [{
					title: messages.addNewFilterAction_title,
					tooltip: messages.addNewFilterAction_tooltip,
					doRun: dojo.hitch(this, this.addSectionTableRow)
				}]
			};
		},

		appendTableAndHeaders: function(sectionHeaders, scope, table){
        	var section= sectionHeaders["rmps.ui.sections.EditableSectionHeader"];
        	var description= sectionHeaders["rmps.ui.sections.DescriptionHeader"];
        	scope.containerNode.appendChild(section.domNode);
			var tableDomNode= table.domNode;

			section.containerNode.appendChild(description.domNode);
	        section.containerNode.appendChild(tableDomNode?tableDomNode:table);
        },

        addSectionTableRow: function(section, table, data, index){

        	var row = this.inherited(arguments);

        	if (data && data.enabled) {
        		// Set enabled check box
        		var checkbox = row.children[0].children[0].children[0];
        		var cbWidget = dijit.getEnclosingWidget(checkbox);
        		cbWidget.attr('checked', true);
        	}

        	if(!data){
        		/*If data is undefined, then, it is a new row, so prepare it for update */
        		this.sectionHeaders["rmps.ui.sections.EditableSectionHeader"].edit();
        		var scope = this.scope;
	            scope._filtersToBeAdded.push(row);
        		scope.addDirty(row);	            
        	}
        },

        fillTable: function(section, table, onCall){
    		var proj = this.projectId ? this.projectId : jazz.app.context.get().itemId;
    		var projectUri = PROJECT_AREA_PATH +'/' + proj;
    		var url = EXPLORER_FILTER_PATH + "/" + proj;
    		var etag = null;
    		var uri = null;
    		var getArgs = {
				url: url,
				headers: {
					"Accept": "application/json",
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "json",
				content: {
					projectUri: projectUri,
					contextType: SettingTypes.PROJECT
				}
			};
			jazz.client.xhrGet(getArgs).addCallback(this, function(result){				
				// Create list of filters
				var filters = [];
				dojo.forEach(result, function(res){
					var segments = res.uri && res.uri.split("/");
					var proj_id = segments.length > 1 ? segments[segments.length-2] : null;
					if (!proj_id || proj_id == proj)
					{
						filters.push(new rmps.ui.sections.ExplorerFilter(res));
					}
				});				
				
				onCall(section, table, filters);
			});
        },

        _creatorCheckbox: function(section, table, tableRow, fieldName, value, data){
            var tableData = dojo.create("td", null, tableRow);
            var checkBox = new dijit.form.CheckBox({
                name: "checkBox",
                value: "enabled",
                checked: data ? data.enabled : false,
                onChange: dojo.hitch(this, this._onEnabledCheckboxChange)
            },
            "checkBox");

            tableData.appendChild(checkBox.domNode);
        },

        _onEnabledCheckboxChange: function(b) {
        	var editor = this.scope;
        	editor.settingsChanged = true;
        	editor.addDirty(this);
        },

        creatorCallBackDefaultFilter: function(section, table, tableRow, fieldName, value, data){
			if(!value){
				value= "";
			}
			// add value input column
			var valueTd = dojo.create("td", null, tableRow);
			var container = dojo.create("div", null, valueTd);

			var preview = dojo.create("span", {className: "preview"}, container);
			preview.appendChild(dojo.doc.createTextNode(value));

			
			var input = dojo.create("input", {className: "editable", type: "text", value: value}, container);
			if (data && data.uri && this.isEmptyNamesFilter(data.uri)) {		
				dojo.attr(input, "disabled", "true");
			}
			input.style.width= "100%";

	        this.scope.eventHandles.push(this._connectOnInput(this.scope,valueTd, function(event){
	        	if(!event){
	        		return;
	        	}
	        	var input= event.target;
	        	var td= event.currentTarget;
	        	var row= td.parentNode;
	        	var dataSet= this.explorerFilterSection.getDataForRow(row);
	        	var isNew= true;
	        	if (dojo.indexOf(this._filtersToBeAdded, row)<0){
	        		isNew= false;
	        	}
	        	var values= dataSet.values;
	        	var voteForChange= false;
	        	var cells= dataSet.cells;
	        	for(var index=0; index<cells.length;index++){
	        		var cell= cells[index];
	        		var inputValue= cell.children[1].value;
	        		var originalValue= dataSet.values[dataSet.fieldNames[index]];
	        		cell.children[0].textContent= inputValue;	/*Update the preview value */
	        		cell.children[0].innerText= inputValue;
	        		if(inputValue!= originalValue){
	        			voteForChange= true;
	        		}
	        	}
	        	if(!isNew){
	        		/*We look for the modified entries */
		        	var rowIndex= dojo.indexOf(this._filtersToBeModified, row);
		        	var doesNotContain= rowIndex == -1;
		        	if(voteForChange){
		        		if (doesNotContain) {
		                    this.addDirty(row);
		                    this._filtersToBeModified.push(row);
		            	}
		        	}else {
		        		if(!doesNotContain){
		        			this.removeDirty(row);
		                    this._filtersToBeModified.splice(rowIndex);
		        		}
		        	}
	        	}
	        }));
	        return container;
        },

        _connectOnInput: function(scope, obj, method){
            if (dojo.isMoz || dojo.isOpera) {
                scope.connect(obj, "oninput", method);
            }
            else {
                scope.connect(obj, "onkeydown", method);
                scope.connect(obj, "onkeyup", method);
                scope.connect(obj, "onpaste", function(){
                    setTimeout(dojo.hitch(this, method), 1);
                });
                scope.connect(obj, "oncut", function(){
                    setTimeout(dojo.hitch(this, method), 1);
                });
                scope.connect(obj, "onblur", method);
            }
        },
        
        isEmptyNamesFilter: function(uri)
        {
        	var emptyNameFilter = "EMPTY_NAME_FILTER_URI";
        	var segments = uri && uri.split("/");
        	if (segments && segments.length > 0) {
	        	var id = segments[segments.length - 1];
    	    	return id.substring(0,emptyNameFilter.length) == emptyNameFilter;
    	    }
    	    return false;
        },

        createDeleteAction: function(section, table, tr, td, div, data){
        	if (data && data.uri && this.isEmptyNamesFilter(data.uri)) {
        		return;
        	}
            var imgAnchor = dojo.create("A", {"class": "action"});
            imgAnchor.href = "#";
            var img = dojo.create("IMG", { "class": "explorer-filter-button-img"});
            img.title = messages.deleteAction;
            img.src = ImageService.REMOVE;

            /*We do not show the action image by default unless we hover on a row */
            imgAnchor.appendChild(img);

            var scope = this.scope;

            scope.connect(tr, "onmouseover", function(e){
                imgAnchor.style.visibility = "visible";
            });

            scope.connect(tr, "onmouseout", function(e){
                imgAnchor.style.visibility = "hidden";
            });

            scope.connect(imgAnchor, "onclick", function(e){
                /*to prevent redirection*/
                dojo.stopEvent(e);
                var row = e.currentTarget.parentNode.parentNode.parentNode;
                var index = dojo.indexOf(scope._filtersToBeAdded, row);
                if (index > -1) {
                    scope.removeDirty(row);
                    scope._filtersToBeAdded.splice(index, 1);
                }
                else {
                    scope._filtersToBeDeleted.push(row);
                    scope.addDirty(row);
                }
                var table = row.parentNode;
                table.deleteRow(dojo.indexOf(table.rows, row));
            });

			div.appendChild(imgAnchor);
            return imgAnchor;
        },

        createDefineFilterAction: function(section, table, tr, td, div, data) {
        	if (data && data.uri && this.isEmptyNamesFilter(data.uri)) {
        		return;
        	}
        	var imgAnchor = dojo.create("A", {"class": "action"});
            imgAnchor.href = "#";
            var img = dojo.create("IMG", { "class": "explorer-filter-button-img"});
            img.title = messages.defineFilterAction;
            img.src = ImageService.DEFINE_FILTER;

            /*We do not show the action image by default unless we hover on a row */
            imgAnchor.appendChild(img);

            var scope = this.scope;

            scope.connect(tr, "onmouseover", function(e){
                imgAnchor.style.visibility = "visible";
            });

            scope.connect(tr, "onmouseout", function(e){
                imgAnchor.style.visibility = "hidden";
            });

            scope.connect(imgAnchor, "onclick", function(e){
                /*to prevent redirection*/
                dojo.stopEvent(e);
                var row = e.currentTarget.parentNode.parentNode.parentNode;
                scope.explorerFilterSection.defineFilter(row);
            });
            
			div.appendChild(imgAnchor);
            return imgAnchor;
        },

        defineFilter: function(row){

			// TODO: Associate icons
			var iconMap = [
			];

			var dataSetRow = this.getDataForRow(row);
			var filterName = dataSetRow.values.name;

			// Create the dialog
			var dialog = new jazz.ui.SwapListDialog({
				dialogTitleText: dojo.string.substitute(messages.filterDialogTitle, [filterName]),
				unselectedListTitleText: messages.filterDialogUnselectedTitle,
				selectedListTitleText: messages.filterDialogSelectedTitle,
				sortField: "label",
				iconMap: iconMap,
				onOK: dojo.hitch(this, function(dialog) {
					var excludedTypes = dojo.map(dialog.getUnselectedEntries(), function(item) {
				   		return item.id;
					});
					if (dataSetRow.data) {
						dataSetRow.data.excludedTypes = excludedTypes;
						// Data exists so this should be a modify unless in added list
						var index = dojo.indexOf(this.scope._filtersToBeAdded, row);
		                if (index < 0) {
		                    this.scope._filtersToBeModified.push(row);
		                    this.scope.addDirty(row);
		                }
					} else {
						dataSetRow.data = {excludedTypes: excludedTypes};
	                    this.scope.addDirty(row);
					}
					dialog.destroy();
				}),
				onCancel: function(dialog) {
					dialog.destroy();
				}
			});

			// Show the dialog

			dialog.show();
			dialog.showLoading(true);

			// Get the types
			var projectId = this.projectId ? this.projectId : jazz.app.context.get().itemId;
			var url = EXPLORER_TYPE_PATH + '/' + projectId;
			var getArgs = {
				url: url,
				headers: {
					"Accept": "application/json",
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "json"
			};
			var handler = jazz.client.xhrGet(getArgs);
			handler.addCallback(dojo.hitch(this, function(response) {
				var unselectedEntries = [];
				var selectedEntries = [];
				var excludedTypes = null;
				if (dataSetRow.data) {
					excludedTypes = dataSetRow.data.excludedTypes;
				}
				for (var i = 0; i < response.length; i++) {
					var type = response[i];
					var item = {
						label: type.typeName,
						id: type.type,
						type: i
					};

					if (excludedTypes && excludedTypes.indexOf(type.type) != -1) {
						unselectedEntries.push(item);
					} else {
						selectedEntries.push(item);
					}
				}
				dialog.setUnselectedEntries(unselectedEntries);
				dialog.setSelectedEntries(selectedEntries);
				dialog.showLoading(false);
			}));
			handler.addErrback(function(error) {
				dialog.showLoading(false);
				dialog.hide();
				dialog.destroy();
			});
        }
    });
})();
