/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("rmps.app.ui.internal.sections.LinkTypesSection");

    dojo.require("rmps.ui.sections.TableData");
    dojo.require("rmps.app.ui.internal.common.ImageService");
    dojo.require("rmps.core.util.ILinksConstants");
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("rmps.core.base.DataFeed");
    dojo.require("rmps.ui.sections.IHeader");
    dojo.require("rmps.app.ui.internal.editors.views.link.LinkType");
    dojo.require("rmps.app.ui.internal.sections.LinkTypesDisplayOptions");
    dojo.require("rmps.app.ui.internal.editors.views.link.LinkClient");
    dojo.require("rmps.app.ui.ProjectLinkingEditorUtil");
    dojo.require("rmps.app.ui.internal.editors.views.link.LinkEventManager");
    dojo.require("rmps.core.util.ContextUtils");
    dojo.require("rmps.core.util.ILinksConstants");
    dojo.require("rmps.app.ui.internal.LinkTypeEditorInput");
    dojo.require("rmps.app.ui.internal.dialogs.EditorDialog");

    dojo.require("dojo.i18n");

    dojo.requireLocalization("rmps.app.ui.internal.sections", "LinkTypesSectionMessages");

    var ImageService = rmps.app.ui.internal.common.ImageService;
    var ContextUtils= rmps.core.util.ContextUtils;
    var ILinksConstants = rmps.core.util.ILinksConstants;
    var LinkTypesDisplayOptions=rmps.app.ui.internal.sections.LinkTypesDisplayOptions;
    var ProjectLinkingEditorUtil= rmps.app.ui.ProjectLinkingEditorUtil;
    var LinkType= rmps.app.ui.internal.editors.views.link.LinkType;
    var LinkClient= rmps.app.ui.internal.editors.views.link.LinkClient;
    var LinkEventManager= rmps.app.ui.internal.editors.views.link.LinkEventManager;

	var LINK_TYPES_UAKEY = "com.ibm.xtools.rmps.web.app.section_LinkTypesSection001";
	var SYSTEM_LINK_TYPES_UAKEY = "com.ibm.xtools.rmps.web.app.section_ProjectLinksSection001";

    dojo.declare("rmps.app.ui.internal.sections.LinkTypesSection", rmps.ui.sections.TableData, {
        messages: null,
        contents: null,
        editor: null,
        deploymentInProgress: null,
        projectContext: null,
        configurationContext: null,

        displayOption: LinkTypesDisplayOptions.ALL,
        linkTypeDataFeed: null,

        /* This a list for the link types that we do no enable operations on (Edit/Delete) */
        exclusion: [ ILinksConstants.EXTERNAL_LINK_TYPE ],

        constructor: function(params){
			this.messages= dojo.i18n.getLocalization("rmps.app.ui.internal.sections", "LinkTypesSectionMessages");

			if(params){
				this.isReadOnly = params.isReadOnly;
				this.displayOption= params.displayOption;
				if (!this.isReadOnly) {
					this.allowCreation= params.allowCreation;
					this.allowDeletion= params.allowDeletion;
					this.allowModification = params.allowModification;
				}
				this.editor = params.editor;
				if (this.editor && !this.isReadOnly) {
					this.mediator= this.editor.getEditorInput().mediator;					
				}
				this.projectContext = params.projectContext;
				this.configurationContext = params.configurationContext;
			}

			if (this.mediator) {
				if (!this.mediator.linkTypeDataFeed) {
					this.mediator.linkTypeDataFeed= new rmps.core.base.DataFeed({uniqueIdentifier: "about"});
				}
			}
			
			this.deploymentInProgress= false;
        },
                
	    addColumnDescriptors: function(){
			this.addColumnDescriptor({columnHeader: this.messages.linkNameHeader, fieldName: "title", width: "25%", creatorCallback: this.createLinkTypeDefaultCell});
            this.addColumnDescriptor({columnHeader: this.messages.linkDescriptionHeader, fieldName: "description", width: "75%", creatorCallback: this.createLinkTypeDefaultCell});
	    },
	    
	    enableDynamicActionsWidth: function(){
		    return true;
	    },
	    
	    addActionDescriptors: function(){
		    if (this.displayOption !== LinkTypesDisplayOptions.SYSTEM_DEFINED && this.allowDeletion) {
				this.addActionDescriptor({
					imgSrc: ImageService.REMOVE_PERMANENT,
					id: "createDeleteAction",
					action: dojo.hitch(this, this.createDeleteAction),
					actionEnablement: dojo.hitch(this, this.deleteEnablement),
					label: this.messages.deleteAction
				});
			}
	    },
	    
	    createDeleteAction: function(dataSetEntry,tableData,btn,args,section, table, e){
            this.mediator.linkTypeDataFeed.removeDataEntry(dataSetEntry.data);
	    },
	    
	    deleteEnablement: function(tableData, dataSetEntry, section, table, simpleToolbar, id){
		    return this.displayOption !== LinkTypesDisplayOptions.SYSTEM_DEFINED && this.allowDeletion;
	    },

        /*@displayOption: The user could specify the displayOption if it is All, SYSTEM_DEFINED, USER_DEFINED */
        startup: function(scope, displayOption){
            this.displayOption= displayOption;
            if (!this.allowCreation || this.displayOption === LinkTypesDisplayOptions.SYSTEM_DEFINED) {
                this.addHeader("rmps.ui.sections.CollapsibleSectionHeader", this.getReadOnlyHeaderArguments());
            }else{
                this.addHeader("rmps.ui.sections.CollapsibleSectionHeader", this.getHeaderArguments());

                /*To be used instead to use the old way */
                /* this.addHeader("rmps.ui.sections.EditableSectionHeader", this.getHeaderArguments()); */
            }

            if (this.displayOption === LinkTypesDisplayOptions.USER_DEFINED) {
                this.addHeader("rmps.ui.sections.DescriptionHeader", {sectionDescription:this.messages.projectLinksSectionDescription});
                this.addHeader("rmps.ui.sections.DescriptionHeader", {sectionDescription:this.messages.projectLinksSectionDescriptionDetailed}, "detailed");
            }

            var section= this.inherited(arguments);
            return section;
        },

        getTitle: function(){
            if (this.displayOption === LinkTypesDisplayOptions.SYSTEM_DEFINED) {
                return this.messages.linkTypeSystemSection_title;
            } else {
                return this.messages.linkTypeSection_title;
            }
        },

        appendTableAndHeaders: function(sectionHeaders, scope, table){
            var section= sectionHeaders["rmps.ui.sections.CollapsibleSectionHeader"];
            var description= sectionHeaders["rmps.ui.sections.DescriptionHeader"];
            var detailed= sectionHeaders.detailed;
            scope.containerNode.appendChild(section.domNode);
			var tableDomNode= table.domNode;
			dojo.addClass(section.containerNode, "rmps-web-sections-table-data");
			if(description){
				section.containerNode.appendChild(description.domNode);
			}
			
			
			if(detailed){
				var div= detailed.domNode;
				for(var index=0; index<div.childNodes.length; index++){
					var current= div.childNodes[index];
					if (current.tagName&& current.tagName.toLowerCase() === "div") {
						for (var subIndex=0; subIndex<current.childNodes.length; subIndex++) {
							var subCurrent= current.childNodes[subIndex];
							if (subCurrent.tagName&& subCurrent.tagName.toLowerCase() === "a") {
								subCurrent.onclick= dojo.hitch(this, function(e){
									dojo.stopEvent(e);
									this.deployPredefinedLinkTypes();			
								});
								break;
							}		
						}
					}
				}
				section.containerNode.appendChild(detailed.domNode);
			}
			
	        section.containerNode.appendChild(tableDomNode?tableDomNode:table);
        },

		getHeaderArguments: function(){
			var uaKey = this.displayOption === LinkTypesDisplayOptions.SYSTEM_DEFINED ? SYSTEM_LINK_TYPES_UAKEY : LINK_TYPES_UAKEY;
			return {
				sectionTitle:	this.getTitle(),
				uaKey:			uaKey,
				actions: [
				{
					img: ImageService.DEPLOY_LINK_TYPE.uri,
					title: this.messages.deploy_Predefined_Link_Types,
					tooltip: this.messages.deploy_Predefined_Link_Types,
					doRun: dojo.hitch(this, function(){
							this.deployPredefinedLinkTypes();
						}
					)
				},
				{
					img: ImageService.ADD.uri,
					title: this.messages.add,
					tooltip: this.messages.add,
					doRun: dojo.hitch(this, function() {
				    	var linkTypeEditorInput = new rmps.app.ui.internal.LinkTypeEditorInput(undefined, this.editor.getEditorInput().mediator, this.editor.getEditorInput().projectId, this.configurationContext);
				    	linkTypeEditorInput.linkTypeArg = ProjectLinkingEditorUtil.new_link_type;
				    	rmps.app.ui.internal.dialogs.EditorDialog.show(linkTypeEditorInput, "rmps.app.ui.internal.editors.link.LinkTypeEditor", this.messages.createLinkType, dojo.hitch(this, function() {
				    		this.refreshTableWithoutHeaders();
				    	}));
					}),
				}]
			};
		},

		getReadOnlyHeaderArguments: function(){
			var uaKey = this.displayOption === LinkTypesDisplayOptions.SYSTEM_DEFINED ? SYSTEM_LINK_TYPES_UAKEY : LINK_TYPES_UAKEY;
			return {
				sectionTitle:	this.getTitle(),
				uaKey:			uaKey,
				actions: []
			};
		},
		
		deployPredefinedLinkTypes: function(){
			if(this.editor.isDirty()){
				this.editor.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: this.messages.saveFirst
				});
			}else if(!this.deploymentInProgress){
				if (this.allowCreation) {
					this.deploymentInProgress= true;
					this.editor.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
						summary: this.messages.deplyomentInProgress
					});
					rmps.app.ui.internal.editors.views.link.LinkClient.deployPredefinedLinkTypes(this.projectContext).addCallback(this,this.deployCallback);
				} else {
					var warningMessage;
					if (ContextUtils.isSnapshot()) {
						warningMessage = this.messages.deploymentIsNotAllowedDueToContext;
					} else {
						warningMessage = this.messages.deploymentIsNotAllowedDueToPermission;
					}
					this.editor.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
						summary: warningMessage
					});
				}
			}
		},
		
		deployCallback: function(result){
			var editorInput= this.editor.getEditorInput();
			if(dojo.isArray(result)){
				for(var index=0; index<result.length; index++){
					var data= result[index];
					editorInput.mediator.linkTypeDataFeed.addDataEntry(data);
					LinkEventManager.notifyAllCallback(data, "addAction");
				}
				var summary, type;
				if(result.length>0){
					summary= dojo.string.substitute(this.messages.deployed_Successfully, [result.length]);
					type= jazz.ui.MessageBox.STATUS_TYPES.OK;
				}else{
					summary= this.messages.linksAlreadyDeployed;
					type= jazz.ui.MessageBox.STATUS_TYPES.INFO;
				}
				this.editor.showMessage({
					type: type,
					summary: summary,
					closable: true
				});
			}else{
				/*safe way to refrsh the table if something went wrong
				 * We do not show a message here. Something went wrong anyway
				 */
				editorInput.mediator.linkTypeDataFeed.stopListening(this.linkTypesSection);
				editorInput.mediator.linkTypeDataFeed.initEntriesForWidget(this.linkTypesSection);
				this.refreshTableWithoutHeaders();
			}
			this.deploymentInProgress= false;	
		},

        fillTable: function(section, table, onCall){
            var deferred = LinkClient.getAllLinkTypes(this.projectContext, {orderBy:"title"}, this.configurationContext);
            deferred.addCallback(this, function(result){
                if (!this.displayOption || this.displayOption === LinkTypesDisplayOptions.ALL) {
                    /*Show ALL if no display option defined */
                    this._types= result[ILinksConstants.USER_DEFINED].concat(result[ILinksConstants.SYSTEM_DEFINED]);
                } else if (this.displayOption === LinkTypesDisplayOptions.SYSTEM_DEFINED) {
                    this._types= result[ILinksConstants.SYSTEM_DEFINED];
                } else if (this.displayOption === LinkTypesDisplayOptions.USER_DEFINED) {
                    this._types= result[ILinksConstants.USER_DEFINED];
                }
                if(this.mediator){
                    this.mediator.linkTypeDataFeed.addDataEntries(this._types);	
                }else{
                    onCall(section, table, this._types);
                }
	        });
        },
		
		editDataEntryNotifier: function(dataSetDetails, uniqueIdentifier, identifier, fieldName, fieldValue, dataObject){
	        this.dataFeed= this.mediator.linkTypeDataFeed;
	        
			var dataSet= this.getDataSetFromField(uniqueIdentifier,  dataObject.about);
			if(dataSet){
				dataSet.data.eTag= dataObject.eTag;
				if (fieldName === "about") {
					var titleNode= dataSet.cells.title;
					dataSet.data.about= dataObject.about;
					if(titleNode){
						var div= titleNode.childNodes[0];
						if(div){
							var anchor= div.childNodes[0];
							if(anchor){
								anchor.href= "#";
								anchor.onclick = dojo.hitch(this, function(e) {
									if (e) {
										e.preventDefault();
									}
							    	var linkTypeEditorInput = new rmps.app.ui.internal.LinkTypeEditorInput(dataObject.about, this.editor.getEditorInput().mediator, this.projectContext, this.configurationContext);
							    	rmps.app.ui.internal.dialogs.EditorDialog.show(linkTypeEditorInput, "rmps.app.ui.internal.editors.link.LinkTypeEditor", this.messages.editLinkType, dojo.hitch(this, function() {
							    		this.refreshTableWithoutHeaders();
							    	}));
								}) 
							}
						}
					}
		        }				
			}	        
	        
	        this.inherited(arguments);
	    },
		
        notifyRefresh: function(){
			if(this.mediator){
				this.mediator.removeNotifier(this.refreshNotifier);
				delete this.refreshNotifier;
			}
        },
        
        setContext: function(contextUri) {
        	this.configurationContext = contextUri;
        	this.notifyRefresh();
        },
		
       createLinkTypeDefaultCell: function(section, table, fieldName, value, data, tableRow, td){
			if(!value){
				value= "";
			}
			var container = dojo.create("div", null, td);

			var preview = fieldName === "title"?dojo.create("A", null, container):dojo.create("span", null, container);

			preview.href = "#";
			preview.onclick = dojo.hitch(this, function(e) {
				if (e) {
					e.preventDefault();
				}
		    	var linkTypeEditorInput = new rmps.app.ui.internal.LinkTypeEditorInput(data.about, this.editor.getEditorInput().mediator, this.projectContext, this.configurationContext);
		    	rmps.app.ui.internal.dialogs.EditorDialog.show(linkTypeEditorInput, "rmps.app.ui.internal.editors.link.LinkTypeEditor", this.messages.editLinkType, dojo.hitch(this, function() {
		    		this.refreshTableWithoutHeaders();
		    	}));
			});

			preview.style.width= "100%";
			preview.appendChild(dojo.doc.createTextNode(value?value:""));
	        return td;
        },

        _onFocus: function(e) {
			if(!e){return;}
			var target= e.target;
			if (target) {
				dojo.removeClass(target, "derivedInput");
			}
		},

		_onLostFocus: function(e) {
			if(!e){return;}
			var target= e.target;
			if (target&& target.parentNode) {
				var dataSet= this.getDataForRow(target.parentNode.parentNode.parentNode);
				if(!dataSet){
					return;
				}
				if (dataSet.updateAbout !== undefined && !dataSet.updateAbout) {
					this.disconnect(this._onLostFocus);
					this.disconnect(this._onFocus);
				}else{
					dojo.addClass(target, "derivedInput");
				}
			}
		},
		
		removeDataEntryNotifier: function(details, uniqueIdentifier, data){
			var index = dojo.indexOf(this.scope._linksTypesToBeAdded, data);
            if (index > -1) {
                this.scope.removeDirty(data);
                this.scope._linksTypesToBeAdded.splice(index, 1);
            }
            else {
                this.scope._linkTypesToBeDeleted.push(data);
                this.scope.addDirty(data);
            }
			dojo.hitch(this, this.getInherited("removeDataEntryNotifier", arguments))(details, uniqueIdentifier, data.about);
		},
        
        destroy: function(){
			if(this.mediator){
				this.mediator.linkTypeDataFeed.stopListening(this);
				this.mediator.linkTypeDataFeed= null;
			}
			this.inherited(arguments);
        },

        _connectOnInput: function(scope, obj, method){
            if (dojo.isMoz || dojo.isOpera) {
                scope.connect(obj, "oninput", method);
            }
            else {
                scope.connect(obj, "onkeydown", method);
                scope.connect(obj, "onkeyup", method);
                scope.connect(obj, "onpaste", function(){
                    setTimeout(dojo.hitch(this, method), 1);
                });
                scope.connect(obj, "oncut", function(){
                    setTimeout(dojo.hitch(this, method), 1);
                });
                scope.connect(obj, "onblur", method);
            }
        }
	});

})();