/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
	
dojo.provide("dm.review.app.ui.internal.sections.ReviewElementsTableSection");

dojo.require("rmps.core.review.ReviewController");
dojo.require("rmps.ui.sections.TableData");
dojo.require("rmps.core.review.util.ReviewTypes");
dojo.require("dm.review.app.util.ReviewActionsRegistery");

/*Required import, so the actions registery won't add this require and add uncessary loads in case we have not used this section yet */
dojo.require("dm.review.app.ui.internal.controls.ReviewResultsStatusBar");
dojo.require("dijit.form.Textarea");

var ReviewController = rmps.core.review.ReviewController;
var ReviewTypes=rmps.core.review.util.ReviewTypes;
var ReviewActionsRegistery= dm.review.app.util.ReviewActionsRegistery;

dojo.declare("dm.review.app.ui.internal.sections.ReviewElementsTableSection", rmps.ui.sections.TableData, {

    constructor: function(params){
    	if(!params.reviewController){
			ReviewController.getCurrentInstance(dojo.hitch(this, function(reviewController){
				this.reviewController= reviewController;
			}));    		
    	}
    },

    startup: function(div){
		this.addHeader("rmps.ui.sections.CollapsibleSectionHeader", ReviewActionsRegistery.getElementsTableHeaderArguments(this.reviewController));
		var section= this.inherited(arguments);
		this.connect(this.sectionHeaders["rmps.ui.sections.CollapsibleSectionHeader"], "toggle", this._toggle);
		if (this.reviewController.review.getReviewAttribute("reviewStatus") == ReviewTypes.REVIEW_STATE_TYPES.Finalized?true:false){
			this.sectionHeaders["rmps.ui.sections.CollapsibleSectionHeader"].collapse();
		}
		return section;
    },
    
	insertBefore: function(){
		return false;
	},
	
    addActionDescriptors: function(){
    	if ( !this.reviewController.configNotExist ){
    		this.addActionDescriptor(ReviewActionsRegistery.getReviewResourcePreviewActionDescriptor());
    	}
		if(this.reviewController.allowModify&& this.reviewController.review.isModerator()&& this.reviewController.isEditable()){
			this.addActionDescriptor(ReviewActionsRegistery.getReviewElementResetActionDescriptor());
			if (!this.reviewController.isChangesetReview()){
				this.addActionDescriptor(ReviewActionsRegistery.getDeleteReviewResourceDescriptor());
			}
		}
		
		if(this.reviewController.review.getReviewAttribute("reviewStatus") != ReviewTypes.REVIEW_STATE_TYPES.Draft){
			this.addActionDescriptor(ReviewActionsRegistery.getShowResourceDescriptionAction());
    	}
		if(this.reviewController.review.isModerator()){
			this.addActionDescriptor(ReviewActionsRegistery.getReviewResourceRedirectionAction());			
		}
    },
    
    addColumnDescriptors: function(){
    	if(this.reviewController.review.getReviewAttribute("reviewStatus") == ReviewTypes.REVIEW_STATE_TYPES.Draft){
    		this.addColumnDescriptor(ReviewActionsRegistery.getModelElementDefaultCellDescriptor("displayName", "20%"));
    		this.addColumnDescriptor(ReviewActionsRegistery.getAboutInputCellDescriptor("reviewArtifactInstructions", "80%"));
    	}else{
    		this.addColumnDescriptor(ReviewActionsRegistery.getModelElementDefaultCellDescriptor("displayName", "25%"));
    		this.addColumnDescriptor(ReviewActionsRegistery.getResultDefaultCellDescriptor("resourceResultStatus", "45%"));
    		this.addColumnDescriptor(ReviewActionsRegistery.getResourceCompletedValueDefaultCellDescriptor("artifact", "30%"));
       }
    },
    
    getActionsEnablement: function(section,table,tr,data){
        return data.exists;
    },

    enableDynamicActionsWidth: function(){
    	return true;
    },

    appendTableAndHeaders: function(sectionHeaders, scope, table){
    	var section= sectionHeaders["rmps.ui.sections.CollapsibleSectionHeader"];
		var tableDomNode= table.domNode;
  	    scope.appendChild(section.domNode);
		dojo.addClass(section.containerNode, "rmps-web-sections-table-data");
        section.containerNode.appendChild(tableDomNode?tableDomNode:table);
    },

  	fillTable: function(section, table, onCall){
  		var review= this.reviewController.review;
    	var def= review.resultDeferred;
    	if(def){
    		def.addCallback(this, function(){
    			onCall(section, table, review.model.resources);
    		});
    	}else{
    		onCall(section, table, review.model.resources);
    	}
    },

	//TODO
	refreshTableData: function(){
    	this.headers= [];
    	this.addHeader("rmps.ui.sections.CollapsibleSectionHeader", ReviewActionsRegistery.getElementsTableHeaderArguments(this.reviewController));
    	this.inherited(arguments);
        this.connect(this.sectionHeaders["rmps.ui.sections.CollapsibleSectionHeader"], "toggle", this._toggle);
    	if (this.reviewController.review.getReviewAttribute("reviewStatus") == ReviewTypes.REVIEW_STATE_TYPES.Finalized?true:false){
    		this.sectionHeaders["rmps.ui.sections.CollapsibleSectionHeader"].collapse();
    	}
    },
    
	_toggle: function(){
    	var collapsibleSection = this.sectionHeaders["rmps.ui.sections.CollapsibleSectionHeader"];
		collapsibleSection.setPrimaryTitle(ReviewActionsRegistery.getElementsDescription(this.reviewController, collapsibleSection.isCollapsed));
    }
});
})();
