/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.review.app.ui.internal.tabs.ReviewMainTab");
dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("rmps.ui.ActionsBar");
dojo.require("dm.review.app.ui.internal.controls.ReviewProgressBar");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.DateTextBox");
dojo.require("dijit.form.SimpleTextarea");
dojo.require("dijit.form.TextBox");
dojo.require("rmps.core.review.ReviewController");
dojo.require("rmps.core.review.AbstractReviewControllerDescriptor");
dojo.require("rmps.app.ui.internal.context.ContextSelection");
dojo.require("dm.review.app.ui.internal.sections.ReviewElementsTableSection");
dojo.require("dm.review.app.ui.internal.sections.ReviewParticipantsTableSection");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmps.core.review.util.ReviewTypes");
dojo.require("jazz.ui.ResourceLink");
dojo.require("jazz.app.context");
dojo.require("rmps.core.util.DateUtils");
dojo.require("rmps.core.baseline.BaselineClient");
dojo.require("rmps.core.changesets.ChangesetClient");
dojo.require("rmps.app.ui.internal.common.ImageService");
dojo.require("jazz.ui.HoverPopup");
dojo.require("rmps.ui.ActionsBar");
dojo.require("rmps.core.review.util.ReviewImageService");
dojo.require("rmps.ui.DialogForm");
dojo.require("rmps.ui.EllipsisDetails");
dojo.require("dojo.i18n");
dojo.require("rmpw.text.internal.widgets.RichTextWidget");
dojo.require("dm.review.app.util.ReviewUtil");
dojo.require("rmps.core.util.ContextUtils");
dojo.require("rmps.core.resource.ResourceInfoClient");



var ReviewElementsTableSection = dm.review.app.ui.internal.sections.ReviewElementsTableSection;
var ReviewParticipantsTableSection = dm.review.app.ui.internal.sections.ReviewParticipantsTableSection;
var ReviewProgressBar = dm.review.app.ui.internal.controls.ReviewProgressBar;
var ReviewController= rmps.core.review.ReviewController;
var IRmpsConstants= rmps.core.util.IRmpsConstants;
var HoverPopup = jazz.ui.HoverPopup;
var ReviewTypes= rmps.core.review.util.ReviewTypes;
var BaselineClient= rmps.core.baseline.BaselineClient;
var ResourceInfoClient= rmps.core.resource.ResourceInfoClient;
var ChangesetClient= rmps.core.changesets.ChangesetClient;
var ReviewImageService = rmps.core.review.util.ReviewImageService;
var EllipsisDetails=rmps.ui.EllipsisDetails;
var ReviewUtil= dm.review.app.util.ReviewUtil;
var ContextUtils= rmps.core.util.ContextUtils;


var BORDER_AFFORDANCE = 51;

dojo.requireLocalization("dm.review.app.ui.internal.tabs", "ReviewMainTabMessages");

/* 
 * The view implemntation for the Review Main Tab. Can act as a standalone module (AjaxModule/Application) when using it directly on the browser:
 * 		https://localhost:9444/dm/_ajax-modules/dm.review.tab.app#tabId=main&resourceUri=https%3A%2F%2Flocalhost%3A9444%2Fdm%2Freviews%2F60	
 *  
 */
dojo.declare("dm.review.app.ui.internal.tabs.ReviewMainTab",[dijit._Widget, dijit._Templated, rmps.core.review.AbstractReviewControllerDescriptor],{
	
	templateString: '<div>' +
						'<div dojoAttachPoint="_reviewStatusBarContainer"></div>' +
						'<div dojoAttachPoint="_reviewSectionContainer"></div>'+
					'</div>',
	
	"class": "rmps-OverallReview",
	
	messages: null,

	_reviewSection: null,
	
	postCreate: function() {
		this.inherited(arguments);
		if(this.reviewController){
			this._postCreate();	
		}else{
			ReviewController.getCurrentInstance(this);	
		}
	},
	
	postMixInProperties: function() {
		this.inherited(arguments);
		this.messages= dojo.i18n.getLocalization("dm.review.app.ui.internal.tabs", "ReviewMainTabMessages");
	},
	
	onReviewControllerLoad: function(reviewController){
		this.inherited(arguments);
		this._postCreate();
	},
		
	_postCreate: function(){
		this.createProgressBar();
		this.createReviewSection();
		this.addListeners();
				
		/*
		 * Start filling table data after creating sections and adding
		 * the required listeners
		 */
		this.reviewController.review.model.participants.initEntriesForWidget(this);
		this.reviewController.review.model.resources.initEntriesForWidget(this);
	},
	
	createProgressBar: function(){
		if(this.reviewController.review.isModerator()){
			this._reviewSectionStatusBar = new dm.review.app.ui.internal.tabs.ReviewMainTab.ReviewMainTab._ReviewStatusBar({
					reviewController: this.reviewController,
					messages: this.messages
			});
			this._reviewStatusBarContainer.appendChild(this._reviewSectionStatusBar.domNode);
		}
	},
	
	createReviewSection: function(){
		this._reviewSection = new dm.review.app.ui.internal.tabs.ReviewMainTab._ReviewSection({
			messages: this.messages,
			reviewController: this.reviewController
		});
		this._reviewSectionContainer.appendChild(this._reviewSection.domNode);	
	},
	
	/*************************************************  Data Listeners ********************************************************************/	
	addListeners: function(){
		this.connect(this.reviewController, "onReviewStatusChange", dojo.hitch(this, function(){
    		this.refreshTab();
    	}));	
    	
    	this.connect(this.reviewController, "addDirtyNotifier", dojo.hitch(this, function() {
			this.refreshStatusBar();
		}));
		
		this.connect(this.reviewController, "removeDirtyNotifier", dojo.hitch(this, function() {
			this.refreshStatusBar();
		}));
    	
    	this.connect(this.reviewController, "onSaveSuccess", dojo.hitch(this, function(){
    		this.refreshStatusBar();
    	}));

    	this.connect(this.reviewController, "onReviewResourceUpdate", dojo.hitch(this, function(){
    		this._reviewSection.refreshReviewElementsTableSection();
    	}));
	
	},
			
	addDataEntryNotifier: function(data, descriptions){
    	this.refreshStatusBar();
    	this.updateConfigurationEnablement();
    },

    removeDataEntryNotifier: function(dataSetDetails, uniquieIdentifier, identifier){
    	this.refreshStatusBar();
    	this.updateConfigurationEnablement();
    },

    editDataEntryNotifier: function(dataSetDetails, uniqueIdentifier, identifier, fieldName, fieldValue){
        this.refreshStatusBar();
        this.updateConfigurationEnablement();
    },
	/*************************************************************************************************************************************/
	
	/*************************************************  Refresh Handlers******************************************************************/
    updateConfigurationEnablement: function(){
		this._reviewSection.updateConfigurationEnablement();
	},

	refreshStatusBar: function(){
		this._reviewSectionStatusBar?this._reviewSectionStatusBar.startPath():this.createProgressBar();
	},
	
	refreshReviuewSection: function(){
		this._reviewSection?this._reviewSection.refresh():this.createReviewSection();
	},
	
	destroy: function() {
		if (this._reviewSection) {
			this._reviewSection.destroyRecursive();
			delete this._reviewSection;
		}
		this.inherited(arguments);
	},

	refreshTab: function(){
		this.refreshStatusBar();
		this.refreshReviuewSection();
		
		/*Required to make sure that we remove the elements table when starting the review on a different configuration */		
		this._reviewSection.refreshReviewElementsTableSection();
	}
	/*************************************************************************************************************************************/
	
});

/* 
 * Review Status Bar contents that shos the review main editor status bar, and status changing actions, etc. It is only shown for review moderators 
 */
dojo.declare("dm.review.app.ui.internal.tabs.ReviewMainTab.ReviewMainTab._ReviewStatusBar", rmps.ui.ActionsBar,{
	
	reviewController: null,
	
	messages: null,
	
	// used to define the box style we want to use to highlight the active state
	activeStyle: rmps.ui.ActionsBar.ActiveStyles.ROUND, 
		
	getDescriptors: function(){
		var descriptors= [];
		
		var state= this.reviewController.review.getReviewAttribute("reviewStatus");
		
		if(state== ReviewTypes.REVIEW_STATE_TYPES.Started){
			descriptors.push({func: dojo.hitch(this, this.getDraftEntry), isActive: false});
			
			if(this.reviewController.review.getOverallReviewResult().percentageVotesCast!= 100){
				descriptors.push({func: dojo.hitch(this, this.createStatusEntry), isActive: true});
				descriptors.push({func: dojo.hitch(this, this.getStartedEntry), isActive: false});
				descriptors.push({func: dojo.hitch(this, this.getReviewedEntry), isActive: false});	
				descriptors.push({func: dojo.hitch(this, this.getFinalizedEntry), isActive: false});
			}else{
				descriptors.push({func: dojo.hitch(this, this.getInProgressEntry), isActive: false});
				descriptors.push({func: dojo.hitch(this, this.getReviewedEntry), isActive: reviewed&& !finalized});
				descriptors.push({func: dojo.hitch(this, this.getFinalizeEntry), isActive: false});
				descriptors.push({func: dojo.hitch(this, this.getFinalizedEntry), isActive: false});	
			}
		}else{
			if(this.reviewController.allowModify){
				if(state== ReviewTypes.REVIEW_STATE_TYPES.Draft){
					descriptors.push({func: dojo.hitch(this, this.getDraftEntry), isActive: true});
					descriptors.push({func: dojo.hitch(this, this.getStartEntry), isActive: false});
					descriptors.push({func: dojo.hitch(this, this.getInProgressEntry), isActive: false});
					descriptors.push({func: dojo.hitch(this, this.getReviewedEntry), isActive: false});
					descriptors.push({func: dojo.hitch(this, this.getFinalizedEntry), isActive: false});
			
				}else{
					descriptors.push({func: dojo.hitch(this, this.getDraftEntry), isActive: false});
					
					if(state== ReviewTypes.REVIEW_STATE_TYPES.Paused){
						descriptors.push({func: dojo.hitch(this, this.getPausedEntry), isActive: true});
						descriptors.push({func: dojo.hitch(this, this.getContinueEntry), isActive: false});
					
					}else{
						descriptors.push({func: dojo.hitch(this, this.getInProgressEntry), isActive: false});
					}
					
					var finalized= state== ReviewTypes.REVIEW_STATE_TYPES.Finalized;
					var reviewed= state==ReviewTypes.REVIEW_STATE_TYPES.Reviewed;
					
					descriptors.push({func: dojo.hitch(this, this.getReviewedEntry), isActive: reviewed&& !finalized});
					if(reviewed&& !finalized){
						descriptors.push({func: dojo.hitch(this, this.getFinalizeEntry), isActive: false});
					}
					descriptors.push({func: dojo.hitch(this, this.getFinalizedEntry), isActive: finalized});
				}
			}
		}
		return descriptors;
	},

	getContinueEntry: function(isActive){
		this.insertActionEntry({
			label: this.messages.continueReview,
			imgSrc: ReviewTypes.getReviewStateTypeImageURI(ReviewTypes.REVIEW_STATE_TYPES.Started),
			disabled: !this.reviewController.canStartReview(),
			onClick: dojo.hitch(this.reviewController, this.reviewController.onContinueReview)
		},isActive);
	},
	
	getStartedEntry: function(isActive){
		this.insertActionEntry({
				label: this.messages.pauseReview,
				imgSrc: ReviewTypes.getReviewStateTypeImageURI(ReviewTypes.REVIEW_STATE_TYPES.Paused),
				disabled: !this.reviewController.canPauseReview(),
				onClick: dojo.hitch(this.reviewController, this.reviewController.onPauseReview)
		},isActive);
		
		this.getFinalizeEntry();
	},
	
	getStartEntry: function(isActive){
		this.insertActionEntry({
				label: this.messages.startReview,
				imgSrc: ReviewTypes.getReviewStateTypeImageURI(ReviewTypes.REVIEW_STATE_TYPES.Started),
				disabled: !this.reviewController.canStartReview(),
				onClick: dojo.hitch(this.reviewController, this.reviewController.onStartReview)
		},isActive);
	},
	
	getFinalizeEntry: function(isActive){
		this.insertActionEntry({
			label: this.messages.finalizeReview,
			imgSrc: ReviewTypes.getReviewStateTypeImageURI(ReviewTypes.REVIEW_STATE_TYPES.Finalized),
			disabled: !this.reviewController.canFinalizeReview(),
			onClick: dojo.hitch(this, this.onFinalizeReview)
		},isActive);
	},
	
	onFinalizeReview: function() {
		var form = new dm.review.app.ui.internal.tabs.ReviewMainTab._FinalizeDialog({
			messages: this.messages,
			reviewController: this.reviewController,
			onOk: dojo.hitch(this.reviewController, this.reviewController.onFinalizeReview)
		});
		form.show();
	},	
		
	getDraftEntry: function(isActive){
		this._insertReviewInfoEntry(ReviewTypes.REVIEW_STATE_TYPES.Draft, isActive);
	},
	
	getReviewedEntry: function(isActive){
		this._insertReviewInfoEntry(ReviewTypes.REVIEW_STATE_TYPES.Reviewed, isActive);
	},
	
	getPausedEntry: function(isActive){
		this._insertReviewInfoEntry(ReviewTypes.REVIEW_STATE_TYPES.Paused, isActive);
	},
	
	getFinalizedEntry: function(isActive){
		this._insertReviewInfoEntry(ReviewTypes.REVIEW_STATE_TYPES.Finalized, isActive);
	},
	
	getInProgressEntry: function(isActive){
		this._insertReviewInfoEntry(ReviewTypes.REVIEW_STATE_TYPES.Started, isActive);
	},
	
	_insertReviewInfoEntry: function(state, isActive){
		this.insertInformationEntry({
			label: ReviewTypes.getReviewStateDisplayName(state),
			imgSrc: ReviewTypes.getReviewStateTypeImageURI(state)
		},isActive);		
	},
	
	createStatusEntry: function(isActive) {
		var table = dojo.doc.createElement("table");
		var tbody = dojo.doc.createElement("tbody");
		var tr = dojo.doc.createElement("tr");
		table.appendChild(tbody);
		tbody.appendChild(tr);
		
		var reviewStateType= this.reviewController.review.getReviewAttribute("reviewStatus"); 
		var statusDisplayName = reviewStateType === ReviewTypes.REVIEW_STATE_TYPES.Started?
								this.messages.inProgress: " " + ReviewTypes.getReviewStateDisplayName(reviewStateType);
								
		var simpleImageDescription= new rmps.ui.SimpleImageDescription({
			imgUrl: ReviewTypes.getReviewStateTypeImageURI(reviewStateType),
			label: statusDisplayName
		});
		
		dojo.addClass(simpleImageDescription.domNode, "imgEntry");
		
		var td = dojo.doc.createElement("td");	
		td.appendChild(simpleImageDescription.domNode);	
		tr.appendChild(td);
		
		if (reviewStateType === ReviewTypes.REVIEW_STATE_TYPES.Started){
			var reviewProgressBar = new ReviewProgressBar({
				progress: this.reviewController.review.getOverallReviewResult().percentageVotesCast,
				viewType: ReviewProgressBar.VIEW_TYPE.Completed
			});
			var reviewProgressBarContainer= dojo.create("span");
			reviewProgressBarContainer.appendChild(reviewProgressBar.domNode);
			
			var progressTd = dojo.doc.createElement("td");
			progressTd.appendChild(reviewProgressBar.domNode);	
			tr.appendChild(progressTd);
		}
		var widget = isActive?this.makeActive(table,"review-status-bar-statusBox", "review-status-bar-statusNode" + "-" + this.activeStyle):table;
		this.mainToolbar.addWidget(widget.domNode?widget:{domNode:widget});
	}

});


/* 
 * Basic revie main tab contents that shows the review information such as moderator, instructions, etc.  
 */
dojo.declare("dm.review.app.ui.internal.tabs.ReviewMainTab._ReviewSection", [dijit._Widget, dijit._Templated],{
	templateString: dojo.cache("dm.review.app", "ui/internal/tabs/templates/ReviewSection.html"),
	
	widgetsInTemplate: true,
	
	WARNING_ICON: rmps.app.ui.internal.common.ImageService.INLINE_WARNING,

	messages: null,
	reviewElementsTableSection: null,
	reviewParticipantsTableSection: null,
	
	_ignoreOnChangeEvents: null,
	_onChangeTimeoutHandle: null,
	_dueDateTextBox: null,
	_reviewerInstructionsDiv: null,
	_reviewerInstructionsTextArea: null,
	_reviewArtifactsViewerSectionDiv: null,
	_reviewParticipantsViewerSectionDiv: null,
	_contextSelection: null,
	
	constructor: function(){
		this._ignoreOnChangeEvents = false;	
	},
	
	postCreate: function() {
		this.inherited(arguments);
		this._reviewerInstructionsTextArea = new rmpw.text.internal.widgets.RichTextWidget({style: "max-height:250px"}, this._reviewerInstructionsDiv);
		this.setEnablement();
		this._reviewerInstructionsTextArea.startup();		
		this.addListeners();
		this.refreshConfigurationContents();
		this.refreshReviewParticipantsTableSection();
		this.refreshReviewElementsTableSection();
		this.createSummarySection();
		this.refreshDueDateContent();
		this.refreshFieldsContent();
	},
	
	addListeners: function(){
		if(this.reviewController.allowModify){
			this.connect(this._reviewerInstructionsTextArea, "onChange", this._onInstructionTextChanged);
		}
	},
	
	refreshConfigurationContents: function(){
		var deferred;
		var review= this.reviewController.review;
		var reviewContext= review.getReviewAttribute("reviewBaseline");
		if (!reviewContext || dojo.trim(reviewContext) === "" ){
			reviewContext= jazz.app.context.get().baseline;
			this._updateContext(reviewContext);
		}
		if (this.reviewController.allowModify&&
				review.getReviewAttribute("reviewStatus") == ReviewTypes.REVIEW_STATE_TYPES.Draft && this.reviewController.review.isModerator()){
			this._contextSelection= new rmps.app.ui.internal.context.ContextSelection({
				context: reviewContext,
				disabled: review.getReviewAttribute("reviewStatus")!= ReviewTypes.REVIEW_STATE_TYPES.Draft|| !this.reviewController.review.isModerator(),
				contextFilter: "completed",
				projectId: this.reviewController.context.itemId,
				id: this.id+ "_configurationSelect",
				contextComparator: function (a, b) {
					return a.timestamp < b.timestamp ? 1 : a.timestamp == b.timestamp ? 0 : -1;
				},
				handleSelection: dojo.hitch(this, this._onConfigurationChanges)
			});
			this._disableWarningIcon(this._configWarningIcon);
			this._configurationSelect.appendChild(this._contextSelection.domNode);
			
		} else {
			var deferred = this.reviewController.getReviewContext();
			deferred.addCallback(dojo.hitch(this, function(context){
				this._configurationSelect.appendChild(dojo.doc.createTextNode(" " + context.name));
				this._disableWarningIcon(this._configWarningIcon);
			}));
			deferred.addErrback(dojo.hitch(this, function(error) {
				var tooltip = new HoverPopup({
					around: this._configWarningIcon,
					html: this.messages.configurationNotExist
		        });
		        this.connect(this, "uninitialize", function() {
					tooltip.destroyRecursive();
				});
			}));

		}
	},
		
	refreshReviewElementsTableSection: function(){
		var review= this.reviewController.review;
		var configuration= review.getReviewAttribute("reviewBaseline");
		if( configuration == "" || this.reviewController.ignoreConfigurationMatches() || this.reviewController.isConfigurationMatches()){
				if(!this.reviewElementsTableSection){
					this.reviewElementsTableSection = 	new ReviewElementsTableSection({reviewController: this.reviewController});
					this.reviewElementsTableSection.startup(this._reviewArtifactsViewerSectionDiv);
					if (review.getReviewAttribute("reviewStatus") == ReviewTypes.REVIEW_STATE_TYPES.FINALIZED){
						this.reviewElementsTableSection.collapse();
					}
				}
		}else{
			if(this.reviewElementsTableSection){
				this.reviewElementsTableSection.destroyRecursive();
				dojo.html._emptyNode(this._reviewArtifactsViewerSectionDiv);	
			}
			
		}
	},
	
	refreshReviewParticipantsTableSection: function(){
		if(!this.reviewParticipantsTableSection){
			this.reviewParticipantsTableSection = new ReviewParticipantsTableSection({reviewController: this.reviewController});
			this.reviewParticipantsTableSection.startup(this._reviewParticipantsViewerSectionDiv);			
		}
	},
	
	createSummarySection: function(){
		var review= this.reviewController.review;
		if(review.getReviewAttribute("reviewStatus") !== ReviewTypes.REVIEW_STATE_TYPES.Finalized){
			return;
		}
		var summarySection= new rmps.ui.CollapsibleSection({
			primaryTitle: this.messages.reviewSummarySectionTitle,
			trim: rmps.ui.CollapsibleSectionStyles.GREY_ORANGE
        });
		var collapsibleContents= document.createElement("span");
		var summaryTable= document.createElement("table");
		var summaryTr = document.createElement("tr");
		var summaryContentTd = document.createElement("td");
		var sumContent = review.getReviewAttribute("reviewSummary");
		if(!sumContent){
			sumContent= "";
		}
		var summaryContentNode = new EllipsisDetails({
            target: summaryContentTd,
            text: sumContent
		});
		dojo.removeClass(summaryContentNode.containerNode, "info");
		
		
		
		/* Commented for now as we are only displaying a simpler version of the summary
		   including the total number of comments
		var summaryTd = document.createElement("td");
		var summaryLabelNode = document.createTextNode(this.messages.reviewSummaryLabel);
		summaryTd.appendChild(summaryLabelNode);
		var summaryMidTd = document.createElement("td");
		summaryMidTd.setAttribute("width", "20px");
		summaryTr.appendChild(summaryTd);
		summaryTr.appendChild(summaryMidTd);*/
		
		summaryContentTd.appendChild(summaryContentNode.domNode);
		summaryTr.appendChild(summaryContentTd);
		summaryTable.appendChild(summaryTr);

		/* Commented out as the comments feature is not yet available
		var commentTr = document.createElement("tr");
		var commentTd = document.createElement("td");
		var commentMidTd = document.createElement("td");
		commentMidTd.setAttribute("width", "20px");
		var commentLabelNode = document.createTextNode(this.messages.reviewCommentLabel);
		var commentContentTd = document.createElement("td");

		commentTd.appendChild(commentLabelNode);
		commentTr.appendChild(commentTd);
		commentTr.appendChild(commentMidTd);			
		commentContentTd.appendChild(commentContentNode);
		commentTr.appendChild(commentContentTd);
		summaryTable.appendChild(commentTr);*/

		collapsibleContents.appendChild(summaryTable);
		summarySection.containerNode.appendChild(collapsibleContents);
        this._summarySection.appendChild(summarySection.domNode);
	},
	
	refreshDueDateContent: function(){
		var self= this;
		
		this._updateDueDateWarning();
		this.updateConfigurationEnablement();

		// Set the minimum date.  Strip off the time from today's date or else the calendar popup won't
		// let you pick today.
		var today = new Date();
		this._dueDateTextBox.constraints.min = new Date(today.getFullYear(), today.getMonth(), today.getDate());

        dojo.connect(this._dueDateTextBox, 'onChange', dojo.hitch(this, this._onDueDateChanged));

        // We don't want the bold yellow coloring of the text box when the date has already passed.
        // Our own small warning icon to the left is sufficient.
        this._dueDateTextBox.validator = function(value, constraints) {
        	return true;
		};
		this._dueDateTextBox.rangeCheck = function(value, constraints) {
			return true;
		};    
	},
	
	_showDatePicker: function(){
		var dueDate = this.reviewController.review.getReviewAttribute("reviewDueDate");
		var defaultValue= dueDate? dueDate:new Date();
		rmps.core.util.DateUtils.showDatePicker(this._dueDateTextBox.domNode,defaultValue, 
								dojo.hitch(this,this._onDueDateChanged));
	},
	
	_updateDueDateWarning: function(){
		if(this.isDueDatePassed()&&this.reviewController.review.getReviewAttribute("reviewStatus") != ReviewTypes.REVIEW_STATE_TYPES.Finalized){
			var tooltip = new HoverPopup({
				around: this._dueDateWarningIcon,
				html: this.messages.dueDateWarning
	        });
	        this.connect(this, "uninitialize", function() {
				tooltip.destroyRecursive();
			});
		}else{
			this._disableWarningIcon(this._dueDateWarningIcon);			
		}
	},
	
	_disableWarningIcon: function(image){
		image.style.visibility= "hidden";
		image.title= "";
		image.alt= "";			
	},
	
	isDueDatePassed: function(){
		return rmps.core.util.DateUtils.dateCompare(this.reviewController.review.getReviewAttribute("reviewDueDate"));
	},
	
	refresh: function(){
		this.setEnablement();
		if(this.reviewParticipantsTableSection){
			this.reviewParticipantsTableSection.refreshTableData();
		}
		if(this.reviewElementsTableSection){
			this.reviewElementsTableSection.refreshTableData();	
		}
		if (this.reviewController.review.getReviewAttribute("reviewStatus") == ReviewTypes.REVIEW_STATE_TYPES.Finalized){
			this.createSummarySection();
		}
	},
	
	refreshFieldsContent: function() {
		this._ignoreOnChangeEvents = true;
		
		var review= this.reviewController.review;
		var dueDate = review.getReviewAttribute("reviewDueDate");
		
		if(dueDate){
			this._dueDateTextBox.attr("value", dueDate);	
		}else{
			var curDate= new Date();
			review.setReviewAttribute("reviewDueDate", curDate);
			this._dueDateTextBox.attr("value", curDate);
		}
		
		this._reviewerInstructionsTextArea.attr("value", review.getReviewAttribute("reviewInstructions"));
		

		var rl= new jazz.ui.ResourceLink({uri: review.getReviewAttribute("owner"), label: "-"});
		this._author.appendChild(rl.domNode);      
		
		this.connect(this, "uninitialize", function() {rl.destroyRecursive();});
		      
		this.setEnablement();
		
		this._onChangeTimeoutHandle = setTimeout(dojo.hitch(this, function() {
				this._timeoutHandle = null;
				this._ignoreOnChangeEvents = false;
			}
		),0);
	},
	
	setEnablement: function(){
		var disable= !this.reviewController.review.isModerator() || !this.reviewController.isEditable();
		this._dueDateTextBox.set("readOnly", disable);
		this._reviewerInstructionsTextArea.set("disabled", disable);
		if (disable){
			dojo.removeClass(this._reviewerInstructionsTextArea.domNode, ReviewUtil.RICHTEXT_READWRITE);
			dojo.addClass(this._reviewerInstructionsTextArea.domNode, ReviewUtil.RICHTEXT_READONLY);
		} else {
			dojo.removeClass(this._reviewerInstructionsTextArea.domNode, ReviewUtil.RICHTEXT_READONLY);
			dojo.addClass(this._reviewerInstructionsTextArea.domNode, ReviewUtil.RICHTEXT_READWRITE);
		}
	},
	
	updateConfigurationEnablement: function(){
		if(!this._contextSelection){
			dojo.removeClass(this._configurationInfoNode, "highlightedNode");
			return;
		}
		if(this.reviewController.review.getReviewResources().length>0 && !this.reviewController.isChangesetReview()){
			this._contextSelection.sRs.disabled=true;
			
			var tooltip = new HoverPopup({
				around: this._configurationInfoNode,
				html: this.messages.snapshotWarning
	        });
	        dojo.addClass(this._configurationInfoNode, "highlightedNode");
	        this.connect(this, "uninitialize", function() {
				tooltip.destroyRecursive();
			});
		}else{
			this._contextSelection.sRs.disabled=false;
			dojo.removeClass(this._configurationInfoNode, "highlightedNode");
		}
	},
	
	
	_onDueDateChanged: function(value) {
		if (!this._ignoreOnChangeEvents&& this.reviewController.isEditable()) {
			/*Notify the review updater with the new updated due date */				
			this.reviewController.updateReview("reviewDueDate", value);
			
			/*Update warning details */
			this._updateDueDateWarning();
		}
	},
		
	_onConfigurationChanges: function(value, context){
		if (!this._ignoreOnChangeEvents&& this.reviewController.isEditable()) {
			this._updateContext(value);
		}
	},
	
	_updateContext: function(context){
		var formal= false;
		var j= 0;
		this.reviewController.updateReview("reviewBaseline", context);
		
		/*Review becomes formal when selecting a snapshot. Otherwise, formal must be set to false*/
		if (ContextUtils.getContextType(context) === rmps.core.util.ContextTypes.SNAPSHOT){
			formal= true;
		}
		
		this.reviewController.review.setReviewAttribute("isFormal", formal);
		if (this.reviewController.isChangeset(context)){
			var urisToBeRemoved = dojo.map(this.reviewController.review.model.resources.dataSet,
					function (dataset) {
						return dataset.resourceUri;
					}
			);
			for(j=0; j<urisToBeRemoved.length; j++){
				this.reviewController.removeResource(urisToBeRemoved[j]);
			}
			var deferred = ChangesetClient.getChangesetResources(context);
			deferred.addCallback(dojo.hitch(this, function (resources) {
				var deferredInfo = ResourceInfoClient.getResourceInfo(resources, null, null, context);
				deferredInfo.addCallback(this, function(resourceInfos) {
					for(j=0; j<resourceInfos.length; j++){
						var resourceInfo = resourceInfos[j];
						this.reviewController.addResource(resourceInfo);
					}
					this.refresh();
				});
				
			}));
		}
	},
	
	_onInstructionTextChanged: function() {
		if (!this._ignoreOnChangeEvents&& this.reviewController.isEditable()) {
			this.reviewController.updateReview("reviewInstructions", this._reviewerInstructionsTextArea.get('value'));
		}
	},
	
	setWidth: function(width) {
		this._lastWidth = width;
		this._resizeTextareaControl(width);
	},
	
	_resizeTextareaControl: function(width) {
		var groupSectionWidth = dojo.coords(this._participantsTablesSection).w;
		var borderAffordance = width - groupSectionWidth;
		var textareaWidthToUse = groupSectionWidth - BORDER_AFFORDANCE;
		this._reviewerInstructionsTextArea.domNode.style.width = textareaWidthToUse + "px";
	},
	
	destroy: function() {
		// Destroy both TableData sections to ensure action toolbar
		// focus listeners are removed.
		if (this.reviewParticipantsTableSection) {
			this.reviewParticipantsTableSection.destroyRecursive();
			delete this.reviewParticipantsTableSection;
		}
		
		if (this.reviewElementsTableSection) {
			this.reviewElementsTableSection.destroyRecursive();
			delete this.reviewElementsTableSection;
		}
        this.inherited(arguments);	
	}
	
});

/****************************************************************Filter Dialog*********************************************************************/
dojo.declare("dm.review.app.ui.internal.tabs.ReviewMainTab._FinalizeDialog", rmps.ui.DialogForm, {
	
	templateString: dojo.cache("dm.review.app", "ui/internal/tabs/templates/ReviewMainTabFinalize.html"),
	widgetsInTemplate: true,

	messages: null,
	_reviewSummaryTextArea: null,
	_reviewSummaryDiv: null,
	reviewController: null,

//	"class": "rmps-app-editors-review-viewer",
	
	postCreate: function() {
		this.inherited(arguments);
		var summaryInitial = this.reviewController.review.getReviewAttribute("reviewSummary");
		this._reviewSummaryTextArea = new rmpw.text.internal.widgets.RichTextWidget({style: "max-height:250px"}, this._reviewSummaryDiv);
		dojo.addClass(this._reviewSummaryTextArea.domNode, ReviewUtil.RICHTEXT_READWRITE);
		this._reviewSummaryTextArea.set(' value', summaryInitial);
	},
	
	getDialogArguments: function(){
		return {
			contentNode: this.containerNode,
			primaryTitle: this.messages.finalizeDialogTitle,
			destroyContents: true,
			width: "600px"
		};
	},
		
	_onFinalize: function(){
		var value= this._reviewSummaryTextArea.get('value');
		this.reviewController.onFinalizeReview(value?value: "");
		this.hide();
	},
	
	_onCancel: function(){
		this.hide();
	}
	
});

})();