/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("dm.review.app.ui.internal.tabs.ReviewMyWorkTab");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dm.review.app.ui.internal.controls.ReviewProgressBar");
dojo.require("rmps.ui.resource.ResourceIconLabel");
dojo.require("rmps.core.util.DateUtils");
dojo.require("dm.review.app.util.ReviewActionsRegistery");
dojo.require("rmps.core.review.AbstractReviewControllerDescriptor");
dojo.require("rmps.ui.ActionsBar");
dojo.require("dojo.html");
dojo.require("rmps.core.review.ReviewController");
dojo.require("rmps.core.review.util.ReviewTypes");

dojo.require("rmps.ui.util.ToolbarFocusUtil");
var ToolbarFocusUtil = rmps.ui.util.ToolbarFocusUtil;

dojo.requireLocalization("dm.review.app.ui.internal.tabs", "ReviewMyWorkTabMessages");

var DateUtils= rmps.core.util.DateUtils;
var ReviewTypes= rmps.core.review.util.ReviewTypes;
var ReviewController= rmps.core.review.ReviewController;
var ReviewActionsRegistery= dm.review.app.util.ReviewActionsRegistery;

/* 
 * The view implemntation for the Review MyWork Tab. Can act as a standalone module (AjaxModule/Application) when using it directly on the browser:
 * 		https://localhost:9444/dm/_ajax-modules/dm.review.tab.app#tabId=myWork&resourceUri=https%3A%2F%2Flocalhost%3A9444%2Fdm%2Freviews%2F60	
 *  
 */
dojo.declare("dm.review.app.ui.internal.tabs.ReviewMyWorkTab", [dijit._Widget, dijit._Templated, rmps.core.review.AbstractReviewControllerDescriptor], {

	templateString:	'<div dojoAttachPoint="contentWidget" style= "padding: 10px">' +
			'		<div style= "padding: 0; overflow: hidden" dojoAttachPoint="_barDiv"></div>'+
					'<div dojoAttachPoint="containerNode"></div>'+
					'</div>',

	"class": "rmps-app-review-my-work-tab",

	constructor: function(){
		this.entries= [];
		this.messages= dojo.i18n.getLocalization("dm.review.app.ui.internal.tabs", "ReviewMyWorkTabMessages");
	},

    postCreate: function(){
    	this.inherited(arguments);
    	if(this.reviewController){
			this._postCreate();    		
    	}else{
    		ReviewController.getCurrentInstance(this);
    	}
    },
    
    _postCreate: function(){
    	if(this.reviewController.isConfigurationMatches()){
    		this.refreshParticipantStatusBar();
    		this.addListeners();
    	}
    },
    
    addListeners: function(){
		this.reviewController.review.model.resources.initEntriesForWidget(this);
		
		this.connect(this.reviewController, "onReviewStatusChange", dojo.hitch(this, function(status) {
			for(var index=0 ;index<this.entries.length; index++){
				/*Change toolbar enablement after changing review status */
				this.entries[index].refreshToolbar();				
			}
		}));
    },
    
    onReviewControllerLoad: function(reviewController){
		this.inherited(arguments);
		this._postCreate();
	},
    
    addDataEntryNotifier: function(data, descriptions, dataFeed){
		var resourceInfo= this.reviewController.getResourceInfos(true, data.resourceUri);
		
		var lastModifiedDate = resourceInfo?new Date(resourceInfo.modificationTime):data.modified;
		var title = resourceInfo?resourceInfo.shortName:data.title;
		var entry= new dm.review.app.ui.internal.tabs.ReviewMyWorkTab._MyWorkEntry({
			lastModifiedDate: lastModifiedDate,
			resourceUri: data.resourceUri,
			title: title,
			exists: data.exists,
			messages: this.messages,
			reviewController: this.reviewController
		});
		this.containerNode.appendChild(entry.domNode);
		this.entries.push(entry);
		this.refreshParticipantStatusBar();
    },

    removeDataEntryNotifier: function(dataSetDetails, uniquieIdentifier, identifier, dataFeed){
    	for(var index=0 ;index<this.entries.length; index++){
    		var entry= this.entries[index];
    		if(entry.resourceUri==identifier){
				this.containerNode.removeChild(entry.domNode);
				entry.destroyRecursive();
				this.entries.splice(index,1);
				this.refreshParticipantStatusBar();
				return;
    		}
		}
    },

    editDataEntryNotifier: function(dataSetDetails, uniqueIdentifier, identifier, fieldName, fieldValue){
        if("resourceResultStatus"== fieldName|| "reviewArtifactInstructions"== fieldName){
        	for(var index=0 ;index<this.entries.length; index++){
				/*Update the status display name for the edited entries */
				var entry= this.entries[index];
				if(entry.resourceUri==identifier){
					entry.updateEntry();				
				}
			}
			this.refreshParticipantStatusBar();
        }
    },
    
    refresh: function(){
    	this.clearEntries();
    	this.refreshParticipantStatusBar();
    },
    
    refreshParticipantStatusBar: function(){
    	this._reviewTitle?this._reviewTitle.startPath(): this.createParticipantProgressBar();
	},
	
	createParticipantProgressBar: function(){
		this._reviewTitle = new dm.review.app.ui.internal.tabs.ReviewMyWorkTab._ParticipantStatusBar({
						reviewController: this.reviewController,
						messages: this.messages});
		if(this._reviewTitle){
			this._barDiv.appendChild(this._reviewTitle.domNode);
		}
	},

	destroy: function(){
		this.clearEntries();
        this.inherited(arguments);
    },

    clearEntries: function(){
    	dojo.forEach(this.entries, function(entry){
			this.disconnect(entry.handler);
			entry.destroyRecursive();
			delete entry.handler;
		}, this);
    }

});

/* 
 * A work entry represents the required information/actions for a given resource for the current user
 */
dojo.declare("dm.review.app.ui.internal.tabs.ReviewMyWorkTab._MyWorkEntry", [dijit._Widget, dijit._Templated], {
	
	templateString: dojo.cache("dm.review.app", "ui/internal/tabs/templates/MyWorkEntry.html"),

	messages: null,

    _lastModTimestampHandle: null,

    postCreate: function(){
		var iconLabel = new rmps.ui.resource.ResourceIconLabel({
			isLink: this.exists,
			linkHref: ReviewActionsRegistery.getFullEditResourceUrl(this.reviewController, this.resourceUri)
		});
		
		iconLabel.linkClicked= dojo.hitch(this, function(e){
			dojo.stopEvent(e);
			this.reviewController.navigateTo(iconLabel.linkHref);
		});
		
		this.resourceIconLabel.appendChild(iconLabel.domNode);
		this._lastModTimestampHandle = DateUtils.addDateDiffUpdater(this.lastModifiedDate,
		
		dojo.hitch(this, function(newDateDiff) {
			dojo.html._setNodeContent(this._modifiedTimestamp, dojo.doc.createTextNode(this.messages.modified + " " + newDateDiff), true);
		}));
		var resourceInfo = {};
		
		if (!this.exists){
			dojo.addClass(this.resourceIconLabel, "review-res-not-exist");
			resourceInfo.uri = this.resourceUri;
			resourceInfo.iconUri = ReviewTypes.getArtifactStatusTypeImageURI("NotExist");
			resourceInfo.shortName = this.title;
			resourceInfo.modificationTime = this.lastModifiedDate;
		} else {
			resourceInfo = this.reviewController.getResourceInfos(true, this.resourceUri);	
		}
		iconLabel.setResourceInfo(resourceInfo);
		this.updateEntry();
		this.refreshToolbar();
   },

   updateEntry: function(){
	   var review= this.reviewController.review;
	   var resourceInfo= review.getReviewResource(this.resourceUri);
	   var instructions= resourceInfo.reviewArtifactInstructions;
	   this.aboutValue.innerHTML= instructions?instructions.replace(/\n/g, "<br/>"):"";
		
	   var info= review._reviewResults.getResultsForUserResources(this.reviewController.review.myUserURI,this.resourceUri, review);

	   if(info){
		   var resultStatus= info.participantResultStatus;
		   if(!resultStatus){
			   return;
		   }
		   var statusDisplay= ReviewTypes.getArtifactStatusTypeDisplayName(resultStatus);
		   var total= new rmps.ui.SimpleImageDescription({
			   imgUrl: ReviewTypes.getArtifactStatusTypeImageURI(resultStatus),
			   label: statusDisplay
		   });

		   while(this.statusContainer.hasChildNodes()) {
			   this.statusContainer.removeChild(this.statusContainer.firstChild);
		   }
		   this.statusContainer.appendChild(total.domNode);
		   var css= this.getCSSClass(resultStatus);
		   if(css){
			   dojo.addClass(this.statusContainer, this.getCSSClass(resultStatus));
		   }
	   }
   },

   refreshToolbar: function(){
    	this.clearToolbar();
    	this.toolbar = new jazz.ui.SimpleToolbar();
		this.toolbar.containerNode = this.toolbar.domNode;
		this.toolbarContainer.appendChild(this.toolbar.domNode);
		
		if (this.exists){
			if (this.reviewController.isVoteEnabled()){
				ReviewActionsRegistery.registerChangeStatusButton(this.toolbar, this.reviewController, this.resourceUri);	
			}
			ReviewActionsRegistery.registerPreviewReviewResourceButton(this.toolbar, this.reviewController, this.resourceUri);
		}
		// Add focus listeners to every node in this entry that can
		// obtain the keyboard focus. When focus is obtained show the toolbar actions.
		ToolbarFocusUtil.addToolbarFocusListeners( { 
			actionToolbar: this.toolbar, 
			containerNode: this.entry, 
			actionClassName: ".jazz-ui-SimpleToolbar-button"
		});
	},

     clearToolbar: function(){
    	if(this.toolbar && this.toolbar.domNode){
			while(this.toolbar.domNode.hasChildNodes()) {
			 	this.toolbar.domNode.removeChild(this.toolbar.domNode.firstChild);
			}
			this.toolbar.destroyRecursive();
		}
    },

    getCSSClass: function(value){
    	switch (value) {
    		case ReviewTypes.getArtifactTypeDisplayName(ReviewTypes.ARTIFACT_STATUS_TYPES.NotStarted):
    			return "notStarted";
    		case ReviewTypes.getArtifactTypeDisplayName(ReviewTypes.ARTIFACT_STATUS_TYPES.Disapproved):
    			return "disapproved";
    		case ReviewTypes.getArtifactTypeDisplayName(ReviewTypes.ARTIFACT_STATUS_TYPES.Approved):
    			return "approved";
    		case ReviewTypes.getArtifactTypeDisplayName(ReviewTypes.ARTIFACT_STATUS_TYPES.Abstained):
    			return "abstained";
    		case ReviewTypes.getArtifactTypeDisplayName(ReviewTypes.ARTIFACT_STATUS_TYPES.Reviewed):
    			return "reviewed";
    		default:
    			break;
    	}

   },

   refresh: function(){
   		this.voteButton.domNode.style.visibility= this.reviewController.isVoteEnabled()?"visible": "hidden";
   },

	destroy: function() {
		if (this._timestampHandle) {
			DateUtils.removeDateDiffUpdater(this._timestampHandle);
			delete this._timestampHandle;
		}
		if(this.toolbar){
			this.toolbar.destroyRecursive();
			delete this.toolbar;
		}
		this.inherited(arguments);
    }
});

dojo.declare("dm.review.app.ui.internal.tabs.ReviewMyWorkTab._ParticipantStatusBar", rmps.ui.ActionsBar,{
	
	reviewController: null,
	
	messages: null,
	participantStatusType: null,
	
	getDescriptors: function(){
		var descriptors= [];
		descriptors.push({func: dojo.hitch(this, this.getTitleEntry), noTransition: true});
		descriptors.push({func: dojo.hitch(this, this.getStatusEntry), noTransition: true});
		
		var review= this.reviewController.review;
		this.participantStatusType= review._reviewResults.getUserStatus(this.reviewController.review.myUserURI);
			
		if(this.participantStatusType== ReviewTypes.PARTICIPANT_STATUS_TYPES.Started){
			descriptors.push({func: dojo.hitch(this, this.insertDivider), noTransition: true});
			descriptors.push({func: dojo.hitch(this, this.getProgressEntry), noTransition: true});	
		}
		return descriptors;
	},
	
	getTitleEntry: function(){
		this.insertTextEntry({
			label: this.messages.yourReviewPaneTitle
		});
	},
	
	getProgressEntry: function(){
		var review= this.reviewController.review;
		var resultsDetails= review._reviewResults.getResultsDetailsForUser(review.myUserURI);
		var reviewProgressBar = new dm.review.app.ui.internal.controls.ReviewProgressBar({
			progress: resultsDetails.percentageVotesCast,
			viewType: dm.review.app.ui.internal.controls.ReviewProgressBar.VIEW_TYPE.Completed
		});
		dojo.addClass(reviewProgressBar.domNode, "progressBarEntry");
		return this.mainToolbar.addWidget(reviewProgressBar);
	},
		
	getStatusEntry: function() {
		return this.insertInformationEntry({
			label: ReviewTypes.getParticipantStatusTypeDisplayName(this.participantStatusType),
			imgSrc: ReviewTypes.getParticipantStatusTypeImageURI(this.participantStatusType)
		});
	}

});

})();