/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.review.app.ui.internal.tabs.ReviewParticipantsTab");

dojo.require("dojo.i18n");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Select");

dojo.require("rmps.ui.sections.IHeader");
dojo.require("rmps.core.review.util.ReviewTypes");
dojo.require("dm.review.app.util.ReviewActionsRegistery");
dojo.require("rmps.core.review.util.ReviewUtil");
dojo.require("dm.review.app.ui.internal.sections.ReviewParticipantsTabSection");
dojo.require("rmps.core.review.util.ReviewImageService");
dojo.require("rmps.core.review.AbstractReviewControllerDescriptor");
dojo.require("dm.review.app.ui.internal.controls.ReviewResultsStatusBar");
dojo.require("rmps.core.review.ReviewController");

var ReviewTypes = rmps.core.review.util.ReviewTypes;
var ReviewUtil= rmps.core.review.util.ReviewUtil;
var ReviewController= rmps.core.review.ReviewController;
var ReviewImageService= rmps.core.review.util.ReviewImageService;
var ReviewResultsStatusBar= dm.review.app.ui.internal.controls.ReviewResultsStatusBar;
var ReviewActionsRegistery= dm.review.app.util.ReviewActionsRegistery;

dojo.requireLocalization("dm.review.app.ui.internal.tabs", "ReviewParticipantsTabMessages");

var USERS_URI= rmps.core.RootServices.getUri(rmps.core.Namespaces.JFS, "users");

/* 
 * The view implemntation for the Review Reviewrs Tab. Can act as a standalone module (AjaxModule/Application) when using it directly on the browser:
 * 		https://localhost:9444/dm/_ajax-modules/dm.review.tab.app#tabId=participants&resourceUri=https%3A%2F%2Flocalhost%3A9444%2Fdm%2Freviews%2F60	
 * 
 */
dojo.declare("dm.review.app.ui.internal.tabs.ReviewParticipantsTab", [dijit._Widget, dijit._Templated, rmps.core.review.AbstractReviewControllerDescriptor], {
	
	templateString: '<div class="rmps-app-review-main">'+
					'<div dojoAttachPoint="containerNode" class="container"></div>' +
					'<div dojoAttachPoint="tableNode" class="container"></div>'+
					'</div>',		
	
	reviewController: null,
	
	"class": "rmps-approve-section",
	        
    postCreate: function(){
    	if(this.reviewController){
    		this._postCreate();		
    	}else{
    		ReviewController.getCurrentInstance(this);
    	}
    	this.inherited(arguments);
    },
    
    _postCreate: function(){
    	if(this.reviewController.isConfigurationMatches() || this.reviewController.ignoreConfigurationMatches()){
			this.section = new dm.review.app.ui.internal.tabs.ReviewParticipantsTab._ReviewParticipantsTabSectionActions({
            		tab: this,
            		reviewController: this.reviewController
	        }).startup(this.tableNode);
	        
	        this.connect(this.reviewController, "onReviewStatusChange", dojo.hitch(this, function(status) {
				this.section.sectionHeaders["dm.review.app.ui.internal.tabs.ReviewParticipantsTab._ReviewParticipantsTabSection.Details"].refreshTab();
			}));	
		}
    },
    
    onReviewControllerLoad: function(reviewController){
		this.inherited(arguments);
		this._postCreate();
	},
    
	destroy: function(){
        if(this.section){
			this.section.destroy();
        	delete this.section;        	
        }
        this.inherited(arguments);
    }
});


dojo.declare("dm.review.app.ui.internal.tabs.ReviewParticipantsTab._ReviewParticipantsTabSectionActions", dm.review.app.ui.internal.sections.ReviewParticipantsTabSection, {
    	
	addActionDescriptors: function(){
		if (!this.reviewController.configNotExist)
			this.addActionDescriptor(ReviewActionsRegistery.getReviewResourcePreviewActionDescriptor());
    },
    
    getActionsEnablement: function(section,table,tr,data){
        return this.reviewController.review.getReviewResources(data.artifactId).exists;
    },

	
	appendTableAndHeaders: function(sectionHeaders, scope, table){
    	var contenet= sectionHeaders["dm.review.app.ui.internal.tabs.ReviewParticipantsTab._ReviewParticipantsTabSection.Details"];
		var tableDomNode= table.domNode;
        contenet.domNode.appendChild(tableDomNode?tableDomNode:table);
        dojo.addClass(contenet.domNode, "rmps-web-sections-table-data");
        scope.appendChild(contenet.domNode);
    },
	
	startup: function(){
       	this.addHeader("dm.review.app.ui.internal.tabs.ReviewParticipantsTab._ReviewParticipantsTabSection.Details", {reviewController: this.reviewController, section: this, table: this.table});
    	var section= this.inherited(arguments);
    	return section;
    }
});
    
    
dojo.declare("dm.review.app.ui.internal.tabs.ReviewParticipantsTab._ReviewParticipantsTabSection.Details", rmps.ui.sections.IHeader, {
	
	createHeader: function(table, scope, args){
        this.tab= new dm.review.app.ui.internal.tabs.ReviewParticipantsTab._ReviewParticipantsTabContents({
        	reviewController: args.reviewController,
        	section: args.section,
        	table: args.table
        });
        return this.tab;
	}
   
});

dojo.declare("dm.review.app.ui.internal.tabs.ReviewParticipantsTab._ReviewParticipantsTabContents", [dijit._Widget, dijit._Templated], {

	templateString: dojo.cache("dm.review.app", "ui/internal/tabs/templates/ReviewParticipantsTabContents.html"),

	editor: null,

	"class": "rmps-app-review-participants",

	constructor: function(){
        this._messages= dojo.i18n.getLocalization("dm.review.app.ui.internal.tabs", "ReviewParticipantsTabMessages");
     },

    postCreate: function(){
    	this.inherited(arguments);
    	this._postCreate();
    },

    _postCreate: function(){
		this.inherited(arguments);
		
		/*We only need to listen at the moment, so e set the true flag here */
		this.reviewController.review.model.participants.initEntriesForWidget(this, true);
		
		this.connect(this.section.tab, "onShow", dojo.hitch(this, this.selectItem));
		
		this.refreshTab();
    },
    
	selectItem: function(){
		var hash = window.location.hash;
		hash= hash.replace("#action=", "");
		var select= dojo.queryToObject(hash).select;
		if(!select){
			return;
		}
		var value= USERS_URI+"/"+dojo.queryToObject(hash).select;
		var options= this._selectDropDownList.options;
		for(var index=0;index<options.length; index++){
			var option= options[index];
			if(ReviewUtil.decodeUserUri(option.value) === value){
    			this._selectDropDownList.selectedIndex= index;
				this.renderSelection();
				return;
			}
		}
    },
    
    refreshTab: function(){
		this.fillComboBox();
    },
    
	fillComboBox: function(){
		// Remove all of the existing options
		while(this._selectDropDownList.hasChildNodes()) {
			this._selectDropDownList.removeChild(this._selectDropDownList.firstChild);
		}
    	var participantInfos = this.reviewController.review.getUsersResults();
     	dojo.forEach(participantInfos,function(participantInfo){
     		if (participantInfo.reviewRole != ReviewTypes.PARTICIPANT_ROLE_TYPES.OptionalReviewer){
     			var userUri= participantInfo.userURI;
     			var userInfo= this.reviewController.getUserInfo(userUri);
     			var name= userInfo?userInfo.name: "-";	/*Safe check */
     			this.addOption(participantInfo.userURI, name);
     		}
		}, this);
		this.renderSelection();
    },
    
	setContents: function(){
		if(this.reviewController.review.getUsersResults().length> 0){
			this.section.table.domNode.style.visibility= "";
			this._selectDropDownList.style.visibility= "";
			this._selectionDetails.style.visibility= "";
			this.setActionsEnablement();
		}else{
			this.section.table.domNode.style.visibility= "hidden";
			this._selectDropDownList.style.visibility= "hidden";
			this._selectionDetails.style.visibility= "hidden";
		}
    },
    refreshStatusBar: function() {
		this.clearResultBar();

		if(this._selectDropDownList.hasChildNodes()){
			this.reviewResultBar= new ReviewResultsStatusBar();
			this._statusContainer.appendChild(this.reviewResultBar.domNode);
		}
	},
	
	refreshToolbar: function(){
		this.clearToolbar();
		if(this.reviewController.isEditable()&& this.reviewController.allowModify){
			this.toolbar = new jazz.ui.SimpleToolbar();
			this.toolbar.containerNode = this.toolbar.domNode;
			this.toolbarContainer.appendChild(this.toolbar.domNode);

			ReviewActionsRegistery.registerAddReviewerButton(this.toolbar, this.reviewController);

			if(this._selectDropDownList.hasChildNodes()){
				ReviewActionsRegistery.registerRemoveReviewerButton(this.toolbar, this.reviewController, dojo.hitch(this, function(){
					return this.getCurrentValue();
				}));
				ReviewActionsRegistery.registerUpdateReviewerRoleButton(this.toolbar, this.reviewController, dojo.hitch(this, function(){
					return this.getCurrentValue();
				}));
			}
		}	
    },

    optionExists: function(value){
    	if(this._selectDropDownList){
			var options= this._selectDropDownList.options;
	    	for(var index=0;index<options.length; index++){
	    		if(options[index].value== value){return true;}
	    	}
    	}
    	return false;
    },

    addOption: function(value, name){
		var option = document.createElement("option");
		option.value = value;
		option.appendChild(document.createTextNode(name));
		this._selectDropDownList.appendChild(option);
		return option;
    },

    addDataEntryNotifier: function(data, descriptions){
    	if (data.reviewRole != ReviewTypes.PARTICIPANT_ROLE_TYPES.OptionalReviewer){
    		var val= data.contributor;
    		if(this.optionExists(val)){
    			return;
    		}
 			var userInfo= this.reviewController.getUserInfo(val);
 			var name= userInfo?userInfo.name: "-";	/*Safe check */
 			this.addOption(val, name);
			this.renderSelection();
     	}
    },

    removeSelectDropDownList: function(value){
    	if(this._selectDropDownList){
			var options= this._selectDropDownList.options;
	    	for(var index=0;index<options.length; index++){
	    		var option= options[index];
	    		if(option.value== value){
	    			this._selectDropDownList.removeChild(option);
	    			dojo.destroy(option);
	    		}
	    	}
    	}
    },

    editDataEntryNotifier: function(dataSetDetails, uniqueIdentifier, identifier, fieldName, fieldValue){
        if(fieldName== "reviewRole"){
        	if(fieldValue== ReviewTypes.PARTICIPANT_ROLE_TYPES.OptionalReviewer){
        		/*Remove the option if its type has been changed to  OptionalReviewer*/
        		this.removeSelectDropDownList(identifier);
        	}else{
        		if(this.optionExists(identifier)){
	    			return;
	    		}
	    		
	 			var userInfo= this.reviewController.getUserInfo(identifier);
	 			var name= userInfo?userInfo.name: "-";	/*Safe check */
	 			this.addOption(identifier, name);
	     		this.renderSelection();
        	}
        }else if(fieldName== "result"){
        	/*In case ohter places made changes to the values */
        	this.renderSelection();
        }
    },


    removeDataEntryNotifier: function(dataSetDetails, uniquieIdentifier, identifier){
    	this.removeSelectDropDownList(identifier);
    	
    	/*Make sure to render the new selection after removing the current user */
    	this.renderSelection();
    },
	
	/** Get the current option tha tis selected in the
	 *  participant srop-down list.
	 *
	 * @return the selected option or null
	 */
	_getCurrentOption: function() {
    	var index= this._selectDropDownList.selectedIndex;
    	if(index<0){
    		return null;
    	}
    	return this._selectDropDownList.options[this._selectDropDownList.selectedIndex];
	},
	
    getCurrentValue: function(){
		var currentOption = this._getCurrentOption();
		if (currentOption){
			return currentOption.value;
		}
		return null;
    },

    renderSelection: function(e){
		this.refreshStatusBar();
		this.updateSelection();
		var value= this.getCurrentValue();
		if(value){
			this.section.userUri= value;
			this.section.refreshTableWithoutHeaders();
		}
		this.setContents();
		this.refreshToolbar();
    },

    clearResultBar: function() {
        if(this.reviewResultBar){
			this.reviewResultBar.destroyRecursive();
        }		
	},
	
    clearToolbar: function(){
    	if(this.toolbar&& this.toolbar.domNode){
			 while(this.toolbar.domNode.hasChildNodes()) {
			 	this.toolbar.domNode.removeChild(this.toolbar.domNode.firstChild);
			}
			this.toolbar.destroyRecursive();
		}
    },

    setActionsEnablement: function(){
    	var style= this.reviewController.isEditable()? "visible":"hidden";
    	this.toolbarContainer.style.visibility = style;
        this.toolbarContainer.style.visibility = style;
    },

    updateSelection: function(){
		var currentOption = this._getCurrentOption();
    	if(currentOption){
			this._infoContainer.style.visibility= "";
    		this.containerNode.style.visibility= "";
    		this.tableNode.style.visibility= "";
	    	var value= currentOption.value;
	    	var curInfo= this.reviewController.review.getUserResults(value, true);
	    	if(curInfo){
	    		var percentageVotesCast= curInfo.resultDetails.percentageVotesCast;
	    		this._statusPercentage.textContent= percentageVotesCast+ "% Completed";
				this._statusPercentage.innerHTML= percentageVotesCast+ "% Completed";
				this.reviewResultBar.role = curInfo.reviewRole;
		    	this.reviewResultBar.setUserUri(this.reviewController.review, value);
		    	this.section.user= value;
	    	}
			
    	}
    },

    destroy: function(){
    	this.clearResultBar();
        this.clearToolbar();
        this.inherited(arguments);
    }

});
    
})();
