/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.review.app.ui.internal.tabs.ReviewResourcesTab");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dm.review.app.ui.internal.sections.ReviewResourcesTabSection");
dojo.require("rmps.ui.sections.IHeader");
dojo.require("rmps.core.review.util.ReviewImageService");
dojo.require("dm.review.app.util.ReviewUtil");
dojo.require("rmps.core.util.DateUtils");
dojo.require("rmps.core.RootServices");
dojo.require("dm.review.app.util.ReviewActionsRegistery");
dojo.require("dm.review.app.ui.internal.controls.ReviewResultsStatusBar");
dojo.require("rmps.core.review.ReviewController");
dojo.require("rmps.core.review.AbstractReviewControllerDescriptor");
dojo.require("rmps.core.review.util.ReviewTypes");
dojo.require("rmps.ui.EllipsisDetails");
dojo.require("rmpw.text.internal.widgets.RichTextWidget");

var ReviewTypes = rmps.core.review.util.ReviewTypes;
var ReviewUtil = dm.review.app.util.ReviewUtil;
var ReviewController= rmps.core.review.ReviewController;
var ReviewImageService= rmps.core.review.util.ReviewImageService;
var DateUtils= rmps.core.util.DateUtils;
var ReviewResultsStatusBar= dm.review.app.ui.internal.controls.ReviewResultsStatusBar;
var EllipsisDetails=rmps.ui.EllipsisDetails;
var ReviewActionsRegistery= dm.review.app.util.ReviewActionsRegistery;

dojo.requireLocalization("dm.review.app.ui.internal.tabs", "ReviewResourcesTabMessages");

var MODELS_URI = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "models");

/* 
 * The view implemntation for the Review Resources Tab. Can act as a standalone module (AjaxModule/Application) when using it directly on the browser:
 * 		https://localhost:9444/dm/_ajax-modules/dm.review.tab.app#tabId=resources&resourceUri=https%3A%2F%2Flocalhost%3A9444%2Fdm%2Freviews%2F60	
 *  
 */
dojo.declare("dm.review.app.ui.internal.tabs.ReviewResourcesTab", [dijit._Widget, dijit._Templated, rmps.core.review.AbstractReviewControllerDescriptor], {

	templateString: '<div dojoAttachPoint="containerNode" class="container">'+
					'<div dojoAttachPoint="tableNode" class="container"></div>'+
					'</div>',

	reviewController: null,

	"class": "rmps-approve-section",

    postCreate: function(){
       	this.inherited(arguments);
		if(this.reviewController){
			this._postCreate();
		} else{
			ReviewController.getCurrentInstance(this);
		}  	
    },
    
    _postCreate: function(){
    	if(this.reviewController.isConfigurationMatches() || this.reviewController.ignoreConfigurationMatches()){
			this.section = new dm.review.app.ui.internal.tabs.ReviewResourcesTab._ReviewResourcesTabSectionActions({
	            	reviewController: this.reviewController,
            		tab: this
	        }).startup(this.tableNode);
	        
			this.connect(this.reviewController, "onReviewStatusChange", dojo.hitch(this, function(status) {
				this.section.sectionHeaders["dm.review.app.ui.internal.tabs.ReviewResourcesTab._Details"].renderSelection();
			}));
    	}
    },
    
    onReviewControllerLoad: function(reviewController){
		this.inherited(arguments);
		this._postCreate();
	},
	destroy: function(){
        if(this.section){
			this.section.destroy();
        	delete this.section;
        }
        this.inherited(arguments);
    }
});

/* 
 * The main view contents for the tab, and it includes the reuired information for each resource in a detailed way
 *  
 */
dojo.declare("dm.review.app.ui.internal.tabs.ReviewResourcesTab._ReviewResourcesTabSectionActions", dm.review.app.ui.internal.sections.ReviewResourcesTabSection, {
	
    startup: function(){
       	this.addHeader("dm.review.app.ui.internal.tabs.ReviewResourcesTab._Details", {reviewController: this.reviewController, section: this, table: this.table});
    	var section= this.inherited(arguments);
    	return section;
    },
    
    appendTableAndHeaders: function(sectionHeaders, scope, table){
    	var contenet= sectionHeaders["dm.review.app.ui.internal.tabs.ReviewResourcesTab._Details"];
		var tableDomNode= table.domNode;
        contenet.domNode.appendChild(tableDomNode?tableDomNode:table);
        scope.appendChild(contenet.domNode);
    }
	
});
    
dojo.declare("dm.review.app.ui.internal.tabs.ReviewResourcesTab._Details", rmps.ui.sections.IHeader, {
	
	createHeader: function(table, scope, args){
        var content=  new dm.review.app.ui.internal.tabs.ReviewResourcesTab._ReviewResourcesTabContents({
        	reviewController: args.reviewController,
        	section: args.section,
        	table: args.table
        });
        return content;
	}
   
});

dojo.declare("dm.review.app.ui.internal.tabs.ReviewResourcesTab._ReviewResourcesTabContents", [dijit._Widget, dijit._Templated], {

	templateString: dojo.cache("dm.review.app", "ui/internal/tabs/templates/ReviewResourcesTabContents.html"),

	editor: null,

	"class": "rmps-app-review-resources",

	constructor: function(){
        this._messages= dojo.i18n.getLocalization("dm.review.app.ui.internal.tabs", "ReviewResourcesTabMessages");
     },

    postCreate: function(){
    	this.inherited(arguments);
    	this._postCreate();
    },

    _postCreate:function(){
    	this.reviewResultBar= new ReviewResultsStatusBar();
		this._statusContainer.appendChild(this.reviewResultBar.domNode);
		
		/*We only need to listen for no, so we use the true flag, as we add the required info by the method fillComboBox after*/
    	this.reviewController.review.model.resources.initEntriesForWidget(this, true);
/*		this.textBox= new rmpw.text.internal.widgets.RichTextWidget({style: "max-height:250px"}, this._aboutValue);
    	this.setEnablement();
    	this.textBox.startup();*/
        
/*    	this.connect(this.textBox, 'onChange', dojo.hitch(this, this._onAboutTextChanged));*/
		this.connect(this.section.tab, "onShow", dojo.hitch(this, this.selectItem));
    	
    	this.fillComboBox();
    	this.updateAbout();
    },
    
	selectItem: function(){
    	var hash = window.location.hash;
		hash= hash.replace("#action=", "");
		var select= dojo.queryToObject(hash).select;
		if(!select){
			return;
		}
    	var value= MODELS_URI+"/"+dojo.queryToObject(hash).select;
    	var options= this._selectDropDownList.options;
    	for(var index=0;index<options.length; index++){
    		var option= options[index];
    		if(option.value== value){
    			this._selectDropDownList.selectedIndex= index;
    			this.renderSelection();
    			return;
    		}
    	}
    },
        
	fillComboBox: function(){
		while(this._selectDropDownList.hasChildNodes()) {
			this._selectDropDownList.removeChild(this._selectDropDownList.firstChild);
		}
		var artifactInfos = this.reviewController.review.getReviewResources();
		dojo.forEach(artifactInfos,function(artifactInfo){
			var option = document.createElement("option");
            option.value = artifactInfo.resourceUri;
            option.appendChild(document.createTextNode(artifactInfo.title));
            this._selectDropDownList.appendChild(option);
		}, this);
		this.renderSelection();
     },
     
     renderSelection: function(e){
    	this.updateSelection();
		this.setContents();
		this.refreshToolbar();
		this.updateAbout();
    },


	addDataEntryNotifier: function(data, descriptions){
		this.addOption(data.resourceUri, data.title);
		this.renderSelection();
    },

    removeDataEntryNotifier: function(dataSetDetails, uniquieIdentifier, identifier){
    	this.removeSelectDropDownList(identifier);
    	this.renderSelection();
    },

    editDataEntryNotifier: function(dataSetDetails, uniqueIdentifier, identifier, fieldName, fieldValue){
        if(fieldName== "reviewArtifactInstructions"){
        	if(this.textBox && this.textBox.get('value') !== fieldValue){
        		this.textBox.set('value', fieldValue);
        	}
        }else if(fieldName== "resourceUri"|| fieldName== "resourceResultStatus"){
        	/*In case other places made changes to the values */
        	this.renderSelection();
        }
    },
    
	addOption: function(value, displayName){
		var option = document.createElement("option");
		option.value = value;
		option.appendChild(document.createTextNode(displayName));
		option.selected= true;
		this._selectDropDownList.appendChild(option);
    },

    getCurrentValue: function(){
    	if(this._selectDropDownList.selectedIndex==-1){
    		return null;
    	}
    	return this._selectDropDownList.options[this._selectDropDownList.selectedIndex].value;
    },
    
	removeSelectDropDownList: function(value){
    	var options= this._selectDropDownList.options;
    	for(var index=0;index<options.length; index++){
    		var option= options[index];
    		if(option.value== value){
    			this._selectDropDownList.removeChild(option);
    			dojo.destroy(option);
    		}
    	}
    },

    updateAbout: function(){
    	if(this._selectDropDownList.options.length> 0){
    		if (!this.textBox){
    			this.textBox= new rmpw.text.internal.widgets.RichTextWidget({
    					name: "myresourcearea",
    					style: "max-height:250px",
    					value: this.getAboutValue()
    				}, this._aboutValue);
    			this.setEnablement();
    			this.textBox.startup();
    			this.connect(this.textBox, 'onChange', dojo.hitch(this, this._onAboutTextChanged));
    		} else {
    			this.setEnablement();
    			var value = this.getAboutValue();
    			this.textBox._setValueAttr(value);  		
    		}
    	}
    },
    
    setEnablement: function() {
    	var disable = !this.reviewController.isEditable() && this.reviewController.review.isModerator();
    	this.textBox.set('disabled',disable);
		if (disable){
			dojo.removeClass(this.textBox.domNode, ReviewUtil.RICHTEXT_READWRITE);
			dojo.addClass(this.textBox.domNode, ReviewUtil.RICHTEXT_READONLY);
		} else {
			dojo.removeClass(this.textBox.domNode, ReviewUtil.RICHTEXT_READONLY);
			dojo.addClass(this.textBox.domNode, ReviewUtil.RICHTEXT_READWRITE);
		}
    },
    
    getAboutValue: function(){
    	var index= this._selectDropDownList.selectedIndex;
    	if(index<0){
    		return null;
    	}
    	var curOption= this._selectDropDownList.options[index];
    	var entry= this.reviewController.review.getReviewResource(curOption.value);	/*Value can never be null her */
    	if(entry){	/*Safe check */
    		var instructions= entry.reviewArtifactInstructions;
	    	return instructions?instructions:"";
    	}
    	return "";
    },

    setContents: function(){
		if(this._selectDropDownList.options.length> 0){
			this.section.table.domNode.style.visibility= "";
			this._selectDropDownList.style.visibility= "";
			this._selectionDetails.style.visibility= "";
		}else{
			this.section.table.domNode.style.visibility= "hidden";
			this._selectDropDownList.style.visibility= "hidden";
			this._selectionDetails.style.visibility= "hidden";
		}
    },

     refreshToolbar: function(){
    	this.clearToolbar();
		if (this.reviewController.isEditable()&& this.reviewController.allowModify){
			this.toolbar = new jazz.ui.SimpleToolbar();
			this.toolbar.containerNode = this.toolbar.domNode;
			this.toolbarContainer.appendChild(this.toolbar.domNode);
			if (!this.reviewController.isChangesetReview()){			
				ReviewActionsRegistery.registerAddReviewResourceButton(this.toolbar, this.reviewController);
			}
			var artifactInfos = this.reviewController.review.getReviewResources();
			if(artifactInfos.length> 0){
				if (!this.reviewController.isChangesetReview()){			
					ReviewActionsRegistery.registerDeleteReviewResourceButton(this.toolbar, this.reviewController, dojo.hitch(this, this.getCurrentValue));
				}
				ReviewActionsRegistery.registerResetReviewResourceButton(this.toolbar, this.reviewController, dojo.hitch(this, this.getCurrentValue));
			}
		}
    },

     clearToolbar: function(){
    	if(this.toolbar && this.toolbar.domNode){
			 while(this.toolbar.domNode.hasChildNodes()) {
			 	this.toolbar.domNode.removeChild(this.toolbar.domNode.firstChild);
			}
			this.toolbar.destroyRecursive();
		}
    },

    updateSelection: function(){
    	var index= this._selectDropDownList.selectedIndex;
    	if(index<0){
    		return;
    	}
    	var curOption= this._selectDropDownList.options[this._selectDropDownList.selectedIndex];
    	var curInfo= this.reviewController.review.getReviewResources(curOption.value, null, true);
    	/*Safe check. It cannot be null at this place */
    	if(!curInfo){
    		return;
    	}
					
		/*
		 * Set last modified value. And empty node in case it has some old values 
		 */	    	
    	dojo.html._emptyNode(this._lastModified);
    	var modified= curInfo.modified;
    	if(modified){
    		var formatedDate = dojo.date.locale.format(modified, {selector:'date', formatLength: 'medium'});
			this._lastModified.textContent= formatedDate;
			this._lastModified.innerHTML= formatedDate;
    	}
    	
    	/*Set overall value */
    	var overall= ReviewTypes.getArtifactStatusTypeDisplayName(this.getOverallStatusForResource(curInfo.resourceUri));
    	this.overallStatusValue.textContent= overall;
    	this.overallStatusValue.innerHTML= overall;
    	
    	/*Set percentage value */
    	var percentage= curInfo.resultDetails.percentageVotesCast;
    	this._statusPercentage.textContent= percentage+ "% "+ this._messages.completed;
    	this._statusPercentage.innerHTML= percentage+ "% "+ this._messages.completed;
    	
    	/*Update results status bar value with the selected tesource uri */
    	this.reviewResultBar.renderResourceResults(this.reviewController.review, curOption.value);
    	
    	/*Now, update the associated table section */
    	this.section.resourceUri= curOption.value;
		this.section.refreshTableWithoutHeaders();
    },

    _onAboutTextChanged: function(){
    	var curOption= this._selectDropDownList.options[this._selectDropDownList.selectedIndex];
		this.reviewController.updateResource(curOption.value, "reviewArtifactInstructions",this.textBox.get('value'));
    },

    getOverallStatusForResource: function(resourceUri){
		var artifactInfo = this.reviewController.review.getReviewResource(resourceUri, null, true);
		return artifactInfo?artifactInfo.resourceResultStatus:ReviewTypes.ARTIFACT_STATUS_TYPES.NotStarted;
    },

    destroy: function(){
        if(this.reviewResultBar){
			this.reviewResultBar.destroyRecursive();
        	delete this.reviewResultBar;
        }
        this.clearToolbar();
        if(this.textBox){
        	this.textBox.destroyRecursive();
        }
        this.inherited(arguments);
    }

});

})();
