/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.analysis                                       |
 *|                                                                        |
 *| Copyright IBM Corporation 2011.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {
dojo.provide("dm.app.analysis.ui.internal.util.AnalysisBaseController");

/** The widget provides the basic controller logic needed 
 *  to open an editor that has been registered against an 
 *  action in the AnalysisPage. This controller assumes every
 *  action has been registered with its editor class and
 *  its editor input class by the AnalysisPage method:
 *  
 *		registerActionEditor(...)
 *  
 *   The base method openActionEditor(args) assumes this registration
 *   has been performed.
 *  
 *  @author Kevin Cornell
 */

dojo.require("rmps.core.error");

dojo.require("rmps.ui.editor.EditorRegistry");
dojo.require("dm.app.analysis.ui.internal.util.AnalysisUtil");

var AnalysisUtil = dm.app.analysis.ui.internal.util.AnalysisUtil;

dojo.declare("dm.app.analysis.ui.internal.util.AnalysisBaseController", null, {

	// private properties
	_analysisPage: null,

	/** Base controller for analysis page extensions. 
	 * 
	 * @param analysisPage the main page for the Analysis capability
	 */
	constructor: function(analysisPage) {
		this._analysisPage = analysisPage;
	},
	
	/** Return the handle to the main page for the Analysis capability
	 * 
	 * @return the analysis page
	 */
	getAnalysisPage: function() {
		return this._analysisPage;
	},

	/** Open the editor corresponding to the 
	 *  action defined in the args (args.action).
	 *  The editor registry is used to create the
	 *  editor input based on the action and then
	 *  the corresponding editor is opened.
	 */
	openActionEditor: function(args) {
		var action = args.action;
		args.projectId = AnalysisUtil.getProjectId();
		
		// Create a new input for the corresponding action
		var editorInputDeferred = rmps.ui.editor.EditorRegistry.createEditorInput(
				action, // registered based on action
				args);
		
		editorInputDeferred.then(dojo.hitch(this, function(editorInput) {
			// Consider failure case
			if (!editorInput) {
				console.error("Unsupported action, no editor is associated: " + action);
				return;
			}
			this._analysisPage.pageHelper.openEditor(editorInput);
		}));
	}
});


})();
