/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.app.ui.internal.util.CLMUtil");

dojo.require("rmps.app.ui.internal.dialogs.CrossServerCommunicationContent");
dojo.require("rmps.app.ui.internal.dialogs.NoProjectLinksFoundContent");
dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants");
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");
dojo.require("rmps.app.ui.internal.editors.views.link.LinkClient");
dojo.require("rmps.core.util.ILinksConstants");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmps.core.client");
dojo.requireLocalization("rmps.app.ui.internal.util", "CLMUtilMessages");

var ProcessConstants= com.ibm.team.process.web.common.internal.ProcessConstants;
var UIHelper= com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
var LinkClient = rmps.app.ui.internal.editors.views.link.LinkClient;
var IRmpsConstants = rmps.core.util.IRmpsConstants;
var ProcessEditorConstants= IRmpsConstants.ProcessConstants;
var ILinksConstants = rmps.core.util.ILinksConstants;

var CLMUtilMessages = dojo.i18n.getLocalization("rmps.app.ui.internal.util", "CLMUtilMessages");
var PROJECT_AREA_PATH= rmps.core.RootServices.getUri(rmps.core.Namespaces.JP_0_6, "projectAreas");
var OSLC_CORE_NS = "http://open-services.net/ns/core#";

var providers= ["rmServiceProviders","amServiceProviders", "cmServiceProviders","qmServiceProviders", "rmps:linkServiceProviderCatalogs"];

rmps.app.ui.internal.util.CLMUtil = {

	getOutgoingLinksDetails: function(callBack, resourceInfo, types, queryInterface){
		return this.getLinksDetails(callBack, resourceInfo, types, dojo.hitch(this,this.getOutgoingLinks), dojo.hitch(this,this._forOutgoingLinks), queryInterface);
	},

	getIncomingLinksDetails: function(callBack, resourceInfo, types, queryInterface){
		return this.getLinksDetails(callBack, resourceInfo, types, dojo.hitch(this,this.getIncomingLinks), dojo.hitch(this,this._forIncomingLinks), queryInterface);
	},

	_forOutgoingLinks: function(result){
		return result;
	},

	_forIncomingLinks: function(result){
		var links = result.incomingLinks;
		for(var i = 0; i < links.length; i++) {
			var link = links[i];
			link.sameServer = (link.about && link.resource)?(this._extractServerApplication(link.about) === this._extractServerApplication(link.resource)):false;
		}
		return links;
	},

	getLinksDetails: function(callBack, resourceInfo, types, linkCallback, forCallback, queryInterface) {
		if (types === null) {
			var projectId = jazz.app.context.get().itemId;
			dojo.when(LinkClient.getLinkTypes(projectId, { orderBy: "title" }),
					  dojo.hitch(this, this._getLinksDetails, callBack, resourceInfo, linkCallback, forCallback, queryInterface)
			);
		} else {
			return this._getLinksDetails(callBack, resourceInfo, linkCallback, forCallback, queryInterface, types);
		}
    },

    _getLinksDetails: function(callBack, resourceInfo, linkCallback, forCallback, queryInterface, types) {
        var _linkDef = linkCallback(resourceInfo, queryInterface);
        dojo.when(
            _linkDef,
            dojo.hitch(this, function(result) {
                var entries = forCallback(result);
                if (!entries) {
                    return;
                }

                var links = [];
                var externalLinks = [];
                var undefinedLinks = [];
                for (var i = 0; i < entries.length; i++) {
                    var item = entries[i];
                    item.linkTitle = item.linkTypeTitle;
                    if (!item.linkTitle) {
                        for (var j = 0; j < types.length; j++) {
                            var type = types[j];
                            if (type.about === item.predicate) {
                                item.linkTitle = type.title;
                                break;
                            }
                        }
                    }

                    var linkObject = {
                        value: item.value,
                        linkTitle: item.linkTitle,
                        description: item.description,
                        label: item.label,
                        isHeader: false,
                        linkData: item
                    };

                    if (!linkObject.linkTitle) {
                    	if (!item.projectScope && !item.sameServer) {
                    		linkObject.linkTitle = linkObject.linkData.linkTitle = CLMUtilMessages.undefined_Link_Type + " (\"" + item.predicate + "\")";
                    		undefinedLinks.push(linkObject);
                    	}
                    } else if (linkObject.linkTitle === ILinksConstants.EXTERNAL_LINK_TYPE_TITLE) {
                        externalLinks.push(linkObject);
                    } else {
                        links.push(linkObject);
                    }
                }
                this.sortData(links, "linkTitle");
                this.sortData(undefinedLinks, "linkTitle");

                /* External Web Page Should be at the end */
                links = links.concat(externalLinks);
                links = links.concat(undefinedLinks);

                var linksWithHeaders = [];
                var header = null;
                for (var l = 0; l < links.length; l++) {
                    var linkObject = links[l];
                    if (linkObject.linkTitle !== header) {
                        var prev = l > 0 ? links[l - 1] : null;
                        if (prev) {
                            /* Required to track the last item in a category to add a space in underneath */
                            linksWithHeaders.push({
                                value: "",
                                linkTitle: "",
                                isHeader: true
                            });
                        }

                        var isExternal = (linkObject.linkData && linkObject.linkData.predicate === ILinksConstants.EXTERNAL_LINK_TYPE);
                        linksWithHeaders.push({
                            value: linkObject.linkData.linkTitle,
                            linkTitle: linkObject.linkData.linkTitle,
                            label: linkObject.linkData.label,
                            isExternal: isExternal,
                            isHeader: true
                        });
                        header = linkObject.linkTitle;
                    }
                    linksWithHeaders.push(linkObject);
                }
                callBack(linksWithHeaders, types);
            })
        );
        return _linkDef;
    },

    getOutgoingLinks: function(resourceInfo, queryInterface){
        return LinkClient.getLinksFor(resourceInfo.uri);
    },

    getIncomingLinks: function(resourceInfo, queryInterface){
        return LinkClient.getIncomingLinksFor(resourceInfo.uri,queryInterface);
    },

    getProjectLinksForElement: function(resourceUri, callback){
        var queryParams = {
            subject: resourceUri,
            linkedProjects: true
        };
        var mainUrl=net.jazz.ajax._contextRoot+"/links?"+dojo.objectToQuery(queryParams);
        var deferred = rmps.core.client.xhrGet({
            url: mainUrl,
            headers: {
                "accept": "application/xml" //$NON-NLS-1$
            },
            handleAs: "xml", //$NON-NLS-1$
            error: dojo.hitch(this, function(error){
                console.error(error);
            })
        });

        deferred.addCallback(this, function(result){
            var projectLinks= [];

            var projectLinksNode= UIHelper.getNode(this._getChildNodes(result), "projectLinks", IRmpsConstants.RMPS_XML_NAMESPACE);
            var projectLinksNodes= UIHelper.getNodes(this._getChildNodes(projectLinksNode), "projectLink", IRmpsConstants.RMPS_XML_NAMESPACE);

            for(var index=0; index<projectLinksNodes.length; index++){
                var current= projectLinksNodes[index];

                var resource= current.getAttribute("rdf:resource");
                var url= resource.substring(0,resource.indexOf("/links"));
                var projectName= current.getAttribute("dcterms:title");
                var id= url.substring(url.indexOf("project-areas")+14, url.length);
                mainUrl= mainUrl.substring(0, mainUrl.indexOf("/dm")+3);

                /*For now, I just replace the segment oslc_am/oslc_rm by oslcquery, as this would do the work. If it required some changes in the future resulted
                 * in us not being able to get the oslc query link, then, we will make a deferred call. */
                var oslcquery= mainUrl+ "/oslcquery/"+ id;

                var projectUrl = new dojo._Url(url);

                if(!projectName){
                    projectName= "";
                }
                var presentationLabel= projectName+ " ("+projectUrl.authority+")";

                var projectLink= {
                    title: projectName,
                    resource: resource,
                    details: url,
                    oslcquery: oslcquery,
                    presentationLabel: presentationLabel
                };

                projectLinks.push(projectLink);
            }
            callback(projectLinks);
        });
    },

    /*Callback being null or not is the caller responsibility, so, we do not check for is. callback is not optional in this method */
    getProjectLinks: function(data, callback) {
        var dpls = [];
        if (data) {
            if (dojo.isArray(data)) {
                for (var u = 0; u < data.length; u++) {
                    dpls.push(this._getExplicitProjectLinks(data[u]));
                }
            } else {
                dpls.push(this._getExplicitProjectLinks(data));
            }
            dpls.push(this._getImplicitProjectLinks("subject")); //$NON-NLS-1$
        } else {
            dpls.push(this._getExplicitProjectLinks());
            dpls.push(this._getImplicitProjectLinks("object")); //$NON-NLS-1$
        }

        var dl = new dojo.DeferredList(dpls);
        dl.then(dojo.hitch(this, function(result) {
            var linkNodes = result[0][1];
            for (var r = 1; r < result.length; r++) {
                linkNodes = linkNodes.concat(result[r][1]);
            }
            if (!linkNodes) {
                return;
            }

            var processed = {};
            var projectLinks = [];
            for (var index = 0; index < linkNodes.length; index++) {
                var current = linkNodes[index];
                var plNodes = this._getChildNodes(current);

                var linkTypeNode = UIHelper.getNode(plNodes, "link-type", ProcessEditorConstants.NAMESPACE_JP);
                var linkType = linkTypeNode.textContent ? linkTypeNode.textContent : linkTypeNode.text;
                var jpUrlNode = UIHelper.getNode(plNodes, "url", ProcessEditorConstants.NAMESPACE_JP);
                var jpUrl = jpUrlNode.textContent ? jpUrlNode.textContent : jpUrlNode.text;
                var isImplicit = (jpUrl && jpUrl.substring(jpUrl.lastIndexOf("/") + 1, jpUrl.length) === "implicit");
                var projectLink = {
                    url: jpUrl,
                    linkType: linkType,
                    serviceProviders : []
                };

                var serviceProviderNodes = UIHelper.getNodes(plNodes, "ServiceProvider", ProcessEditorConstants.NAMESPACE_OSLC);
                for (var sp = 0; sp < serviceProviderNodes.length; sp++) {
                    var spNodes = this._getChildNodes(serviceProviderNodes[sp]);
                    var servicesNode = UIHelper.getNode(spNodes, "services", ProcessEditorConstants.NAMESPACE_OSLC);
                    var serviceUrl = servicesNode.getAttribute("rdf:resource");
                    var servicesPath = new dojo._Url(serviceUrl).path;
                    var applicationType = servicesPath.substring(0, servicesPath.indexOf("/", 1));

                    var details = UIHelper.getNode(spNodes, "details", ProcessEditorConstants.NAMESPACE_OSLC);
                    var projectUrl = new dojo._Url(details.getAttribute("rdf:resource"));

                    var titleNode = UIHelper.getNode(spNodes, ProcessEditorConstants.XMLELEMENT_TITLE, ProcessEditorConstants.NAMESPACE_DC);
                    var projectName = titleNode.textContent ? titleNode.textContent : titleNode.text ? titleNode.text : "";
                    var presentationLabel = projectName + " (" + applicationType + ": " + projectUrl.authority + ")";

                    if (processed[presentationLabel]) {
                        /* 19030: Duplicated projects appear in link provider selection dialog
                         * TEMP solution till we support linking based on link types. 
                         */
                        if (isImplicit) {
                            var indexes = processed[presentationLabel];
                            var serviceProvider = projectLinks[indexes[0]].serviceProviders[indexes[1]];
                            serviceProvider.isImplicit = true;
                        }
                        continue;
                    }

                    projectLink.serviceProviders.push({
                        title: projectName,
                        serviceUrl: serviceUrl,
                        details: projectUrl,
                        presentationLabel: presentationLabel,
                        isImplicit: isImplicit                        
                    });

                    processed[presentationLabel] = [ projectLinks.length, projectLink.serviceProviders.length - 1 ];
                }

                if (projectLink.serviceProviders.length > 0) {
                    projectLinks.push(projectLink);
                }
            }
            if (callback) {
                callback(projectLinks);
            }
        }));
    },

    _getExplicitProjectLinks: function(url) {
        if (!url) {
            url = PROJECT_AREA_PATH + "/" + jazz.app.context.get().itemId + "/links"; //$NON-NLS-1$
        }
        var epl = new dojo.Deferred();
        var deferred = jazz.client.xhrGet({
            url: url,
            headers: {
                "accept": "application/xml" //$NON-NLS-1$
            },
            handleAs: "xml", //$NON-NLS-1$
            error: dojo.hitch(this, function(error){
                console.error(error);
            })
        });
        deferred.addCallback(this, function(result) {
            var nodes = this._getChildNodes(result);
			var links = UIHelper.getNode(nodes, "links", ProcessEditorConstants.NAMESPACE_JP);
			if (links) {
				linkNodes = UIHelper.getNodes(this._getChildNodes(links), "link", ProcessEditorConstants.NAMESPACE_JP);
			} else {
				linkNodes= UIHelper.getNodes(nodes, "link", ProcessEditorConstants.NAMESPACE_JP);
			}
            if (!linkNodes) {
                linkNodes = [];
            }
            epl.resolve(linkNodes);
        });
        return epl;
    },

    _getImplicitProjectLinks: function(direction) {
        var ipl = new dojo.Deferred();        
        var deferred = rmps.app.ui.internal.util.CLMUtil.detectContext(jazz.app.context.get().baseline /* invocationContext */, direction);
        deferred.addCallback(this, function(linkNodes) {
            ipl.resolve(linkNodes);
        });
        return ipl;
    },

    getServiceProviders: function(resourceUri){
        var args = {
            url: net.jazz.ajax._contextRoot + "/linkServiceProviders?" + dojo.objectToQuery({"resourceUri": resourceUri?resourceUri: ""}),
            handleAs: "xml",
            headers: {"X-ibm-rmps-internal": "true"},
            load: function(response, ioArgs){
                var serversDict= [];
                for(var index=0;index<providers.length;index++){
                    var provider= providers[index];
                    var entries = response.getElementsByTagName(provider);
                    if(entries){
                        var entriesLength= entries.length;
	                    if (entriesLength > 0) {
	                        for (var i = 0; i < entriesLength; i++) {
	                            var catalog = entries[i].getAttribute("rdf:resource");
	                            if(!catalog){
	                                continue;
	                            }
	                            
	                            var title = entries[i].getAttribute("dcterms:title");
	                            if(!title){
	                                continue;
	                            }
	                            
	                            serversDict[title]= catalog;
	                        }
	                    }
                    }
                }
                return serversDict;
            }
        };
        return jazz.client.xhrGet(args);
    },
    
    getAuthneticationUrl: function(serviceProviders, serviceUrl){
        var _Url= new dojo._Url(serviceUrl);

        var segment= this.getContext(_Url.path);
        var authority= _Url.authority;
        for(var att in serviceProviders){
            var current= serviceProviders[att];
            if(current){
                var url= new dojo._Url(current);
                var urlSegment= this.getContext(url.path);
                if (url.authority === authority && urlSegment === segment){
                    return current;
                }
            }
        }
        return null;
    },
    
    getContext: function(path){
		var segments= path.split("/");
		for(var index=0; index<segments.length; index++){
			var current= segments[index];
			if(current){
				return current;
			}
		}
    },

    sortData: function(tableData, orderBy){

        function stringCompare(string1, string2){
            return string1.toLocaleLowerCase().localeCompare(string2.toLocaleLowerCase());
        }

        function compare(entry1, entry2){
            var a = dojo.getObject(orderBy, false, entry1);
            var b = dojo.getObject(orderBy, false, entry2);

            if (dojo.isString(a)) {
                a = a.toLowerCase();
            }
            if (dojo.isString(b)) {
                b = b.toLowerCase();
            }
            return stringCompare(b, a);
        }

        function comparator(entry1, entry2){
            for (var i = 0; i < orderBy.length; ++i) {
                var result = compare(entry1, entry2);
                if (result !== 0) {
                    return result;
                }
            }
            return 0;
        }

        tableData.sort(comparator);
    },

    configureFriendsDialogOrAddService: function(callBack){
		var deferred = jazz.client.xhrGet({
			url: ProcessConstants.ROOTSERVICESXML,
			headers: {
				"accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
			},
			handleAs: "xml", //$NON-NLS-1$
			load: dojo.hitch(this, this._readService, callBack),
			error: dojo.hitch(this, function(error){
				console.error(error);
				return error;
			})
		});
    },

    getDefaultLinkTypeURIForName: function(name){
        var location= window.location;
	    if (name !== "") {
			name= "/"+name;
		} else if (!name|| !dojo.isString(name)) {
			name= "";
		}
		var assumedValue= location.protocol+ "//"+ window.location.hostname+"/am/linktypes/"+ jazz.app.context.get().name+ name;
        return assumedValue.replace(/\s/g,"_");
    },

    _readService: function(callBack, rootServicesXml){
        if (!rootServicesXml) {
            return;
        }
        var children = this._getChildNodes(rootServicesXml);
        var descriptionNode = UIHelper.getNode(children, "Description", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
        var descriptionNodeChildren = this._getChildNodes(descriptionNode);

        var friendsNode = UIHelper.getNode(descriptionNodeChildren, "friends", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$
        var friendsResourceNode = UIHelper.getAttributeNode(friendsNode, "resource", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
        var friendsResourceURL = friendsResourceNode.value;
        var deferred = jazz.client.xhrGet({
            url: friendsResourceURL,
            headers: {
                "accept": ProcessConstants.CONTENT_TYPE_APPLICATION_XML //$NON-NLS-1$
            },
            handleAs: "xml", //$NON-NLS-1$
            load: dojo.hitch(this, this._show, callBack),
            error: dojo.hitch(this, function(error){
                console.error(error);
                return error;
            })
        });
        return rootServicesXml;
    },

    _show: function(callBack,friendsListingXml){
        if (!friendsListingXml) {
            return friendsListingXml;
        }

        var children = this._getChildNodes(friendsListingXml);
        var rdfNode = UIHelper.getNode(children, "RDF", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
        var allFriends = UIHelper.getNodes(this._getChildNodes(rdfNode), "Friend", ProcessConstants.NAMESPACE_JD); //$NON-NLS-1$
        if (allFriends.length > 0) {
            callBack(friendsListingXml);
        } else {
            this.showCrossServerCommunicationDialog();
        }

        return friendsListingXml;
    },

    showCrossServerCommunicationDialog: function(){
        this._showDialog(rmps.app.ui.internal.dialogs.CrossServerCommunicationContent, CLMUtilMessages.dialogTitleSelectRequirement_NoServerDefinedTitle);
    },

    showNoProjectLinksFoundContentDialog: function(){
        this._showDialog(rmps.app.ui.internal.dialogs.NoProjectLinksFoundContent, CLMUtilMessages.addProjectLinkDialogHTML_NoProjectLinksTitle);
    },

    _showDialog: function(clazz, title){
        var floatingPane;
        var onOk= function(){
            floatingPane.close();
            floatingPane.destroy();
            floatingPane.content.destroy();
            delete floatingPane.content;
        };

        var params= this._getFloatingPaneParams(clazz, title, onOk);
        floatingPane= new jazz.ui.Dialog(params);

        var content= params.content;
        if (content && content.setFocus){
            content.setFocus();
        }
    },

    _getFloatingPaneParams: function(clazz, title, okCallback, hieght, width){
        if(!hieght){
            hieght= "120px";
        }
        if(!width){
            width= "425px";
        }
        var contentParams = {
            onOk: dojo.hitch(this, okCallback),
            onCancel: dojo.hitch(this, okCallback)
        };
        var content = new clazz(contentParams);

        var floatingPaneParams = {
            width: width,
            height: hieght,
            contentNode: content.domNode,
            content: content
        };
        floatingPaneParams.primaryTitle = title;
        return floatingPaneParams;
    },

	detectContext: function(invocationContext, direction, projectId) {
		var queryParams = {};
		if (invocationContext) {
			queryParams[IRmpsConstants.BASELINE] = invocationContext;
		}
		if (direction) {
			queryParams.direction = direction;
		}
		if (projectId) {
			queryParams.projectId = projectId;
		}
		var args = {
			url: net.jazz.ajax._contextRoot + "/links/context?"
				+ dojo.objectToQuery(queryParams),
			headers: {
				"Accept": "application/xml",
				"X-ibm-rmps-internal": "true"
			},
			handleAs: "xml",
			load: dojo.hitch(this, function(response, ioArgs) {
                var linkContainer = UIHelper.getNode(this._getChildNodes(response), "links", ProcessEditorConstants.NAMESPACE_JP);
                var links = UIHelper.getNodes(this._getChildNodes(linkContainer), "link", ProcessEditorConstants.NAMESPACE_JP);

                var query = ioArgs.url.substring(ioArgs.url.indexOf('?'), ioArgs.url.length);
                var queryObject = dojo.queryToObject(query);
                if (queryObject.direction === "object" && queryObject.projectId) {
                    var serviceProvider = links && links.length > 0 ? UIHelper.getNode(this._getChildNodes(links[0]), "ServiceProvider", ProcessEditorConstants.NAMESPACE_OSLC) : null;
                    var configurationsContainer = serviceProvider ? UIHelper.getNode(this._getChildNodes(serviceProvider), "configurationIds", rmps.core.Namespaces.RMPS_1_0) : null;
                    var configurationNodes = configurationsContainer ? UIHelper.getNodes(this._getChildNodes(configurationsContainer), "configurationId", rmps.core.Namespaces.RMPS_1_0) : null;

                    var configurations = [];
                    if (configurationNodes) {
                        for (var c = 0; c < configurationNodes.length; c++) {
                            var configuration = configurationNodes[c];
                            var configurationUri = configuration.getAttribute("rdf:resource");
                            var titleNode = UIHelper.getNode(this._getChildNodes(configuration), ProcessEditorConstants.XMLELEMENT_TITLE, ProcessEditorConstants.NAMESPACE_DC);
                            var configurationName = titleNode.textContent ? titleNode.textContent : titleNode.text;
                            if (!configurationName) {
                                configurationName = titleNode.text ? titleNode.text : "";
                            }
                            if (configurationUri) {
                                configurations.push({
                                    url: configurationUri,
                                    title: configurationName
                                });
                            }
                        }
                    }

                    return configurations;
                }

                return links;
			}),
            error: dojo.hitch(this, function(error){
                console.error(error);
            })
		};
		return jazz.client.xhrGet(args);
	},

    _getChildNodes: function(source) {
        if (dojo.isArray(source)) {
            return source;
        }

        if (dojo.isIE > 8) {
            var childNodes = [];
            var node = source.firstChild;
            while (node) {
                childNodes.push(node);
                node = node.nextSibling;
            }
            return childNodes;
        }

        return source.childNodes;
    },
    
    _extractServerApplication: function(resource) {
    	var urlComponents = resource.split("/");
    	return urlComponents[2] + "/" + urlComponents[3];
    },
    
    getQueryService: function(serviceProviderUri) {
        var def = new dojo.Deferred();
        var deferred = jazz.client.xhrGet({
            url: serviceProviderUri,
            headers: {
                "accept": "application/xml" //$NON-NLS-1$
            },
            handleAs: "xml", //$NON-NLS-1$
            error: dojo.hitch(this, function(error){
                console.error(error);
            })
        });
        deferred.addCallback(dojo.hitch(this, function(result) {
            var rdfNode = UIHelper.getNode(this._getChildNodes(result), "RDF", ProcessConstants.NAMESPACE_RDF); //$NON-NLS-1$
            var serviceProvider = rdfNode ? UIHelper.getNode(this._getChildNodes(rdfNode), "ServiceProvider", OSLC_CORE_NS) : null;
            var service = serviceProvider ? UIHelper.getNode(this._getChildNodes(serviceProvider), "service", OSLC_CORE_NS) : null;
            var Service = service ? UIHelper.getNode(this._getChildNodes(service), "Service", OSLC_CORE_NS) : null;
            var queryCapability = Service ? UIHelper.getNode(this._getChildNodes(Service), "queryCapability", OSLC_CORE_NS) : null;
            var QueryCapability = queryCapability ? UIHelper.getNode(this._getChildNodes(queryCapability), "QueryCapability", OSLC_CORE_NS) : null;
            var queryBase = QueryCapability ? UIHelper.getNode(this._getChildNodes(QueryCapability), "queryBase", OSLC_CORE_NS) : null;
            var uri = queryBase ? queryBase.getAttribute("rdf:resource") : null;
            def.resolve(uri);
        }));
        deferred.addErrback(function() {
        	def.resolve(null);
        });
        return def;
    }
};
