/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	
	dojo.provide("rmps.app.ui.internal.util.DmProjectAreaConverter"); //$NON-NLS-1$
	
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaEditor"); //$NON-NLS-1$
	dojo.require("rmps.app.ui.internal.pages.ProjectAreaEditorDmContribution"); //$NON-NLS-1$
	dojo.require("rmps.app.util.ProjectConfigurationsInfo"); //$NON-NLS-1$
	
	var DM_CONTRIBUTION_TAB_ID = "rmps.app.ui.internal.pages.DesignDomains"; //$NON-NLS-1$
	var MSG_COOKIE_NAME = DM_CONTRIBUTION_TAB_ID + "#saveMsg";
	var ERROR_COOKIE_NAME = DM_CONTRIBUTION_TAB_ID + "#errorObj";
	var JTS_MSG_COOKIE_NAME = "com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaEditor#saveMsg"; //$NON-NLS-1$

	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;

	/*
	 * Utility functions to workaround the extension limitations of JTS Foundation ProjectAreaEditor widget
	 */
	rmps.app.ui.internal.util.DmProjectAreaConverter = {
			
		/*
		 * Alters the implementation of Foundation ProjectAreaEditor widget by effectively replacing original
		 * function implementations with ours.
		 * 
		 * Why?
		 * Foundation ProjectAreaEditor needs be extended primarily to contribute DM project area property settings
		 * during project area creation. If ProjectAreaEditor implementation is extended then new page and actions
		 * must be introduced at the application level. Many Foundation web pages and services have Project Area
		 * Management page and action ids hard-coded. It's impossible to redirect all those links to application's
		 * page and action ids. Replacing implementation of some of the functions from Foundation's ProjectAreaEditor
		 * widget lets us keep JTS original page and action ids as well as limits the overrides to this file only.
		 * 
		 * How?
		 * Foundation's ProjectAreaEditor widgets prototype is being modified. Namely some functions are being
		 * replaced with our versions of them. Our implementations commonly delegate to original implementations at
		 * some point of time, hence original function implementation is stored in a variable that looks like
		 * original<function name>. This function is commonly invoked from the new implementation with an 'apply'
		 * function call.
		 * 
		 * Foundation ProjectAreaEditor implementation is updated with ours at the time Admin application js loads
		 * AdminApplication.js. AdminApplication hosts ProjectAreaManagement page and, therefore, loaded before Project
		 * AreaEditor widget is created. Each jazz app loads its own copy of process component javascript (own copy of
		 * ProjectAreaEditor widget). Consequently our alternation of ProjectAreaEditor implementation remains within DM 
		 */
		convert: function() {
			
			var self = this;
			
			var ProjectAreaEditor = com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaEditor;
			
			// Check if the current implementation of ProjectAreaEditor has been modified by us
			if (dojo.isFunction(ProjectAreaEditor._getDmContribution)) {
				// already converted
				return;
			}
			
			/*
			 * Override postCreate() function to contribute DM specific UI
			 */
			var originalPostCreate = com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaEditor.prototype.postCreate;
			ProjectAreaEditor.prototype.postCreate = function() {
				
				this.projectConfigurationsInfo = new rmps.app.util.ProjectConfigurationsInfo(this.isNew ? undefined : this.projectAreaItemId);
				
				originalPostCreate.apply(this, arguments);

				if (this.isNew) {
					this.contribution = new rmps.app.ui.internal.pages.ProjectAreaEditorDmContribution({
						editor: this
					});					
					this.twoColumnMainColumn.insertBefore(this.contribution.domNode, this.projectLinksSection);
				} else {
					this.projectConfigurationsInfo.getDspInfo(dojo.hitch(this, function(isDsp, isSystem, canCreateDsp) {
						if (isDsp || isSystem) {
							this.name.readOnly = true;
							this.description.readOnly = true;
							this.summary.readOnly = true;
						}
					}));
				}				
				
			};

			/*
			 * Override customValidateRequired() to validate DM specific project settings
			 */
			var originalCustomValidateRequired = ProjectAreaEditor.prototype.customValidateRequired;
			ProjectAreaEditor.prototype.customValidateRequired = function(saveObj){
				if (this.contribution) {
					return this.contribution.validateRequired(saveObj);
				} else {
					return originalCustomValidateRequired.apply(this, arguments);
				}
			};
			
			/*
			 * Override handleSaveResponse() to follow up the JTS save by the DM specific project settings save
			 */
			var originalHandleSaveResponse = ProjectAreaEditor.prototype.handleSaveResponse;
			ProjectAreaEditor.prototype.handleSaveResponse = function(response) {
				if (!this.contribution || !this.contribution.save(response, originalHandleSaveResponse)) {
					originalHandleSaveResponse.call(this, response);
				}
			};
			
			/*
			 * Override doSave() to send out the space data for the new project area before JTS
			 * project area creation request is sent
			 */
			var originalDoSave = ProjectAreaEditor.prototype.doSave;
			ProjectAreaEditor.prototype.doSave = function(saveObj, serviceResponseHandler){
				var postDmProjectData = this.contribution ? this.contribution.postData() : null; 
				if (postDmProjectData) {
					var originalInherited = originalDoSave;
					var originalArguments = arguments;
					postDmProjectData.then(dojo.hitch(this, function() {
						originalInherited.apply(this, originalArguments);
					}), dojo.hitch(this, function(error) {
						this.handleSaveError(error);
					}));
				} else {
					originalDoSave.apply(this, arguments);
				}
			};
			
			/*
			 * Override checkForSuccessMsg() to check for DM project area save success message as well
			 */
			var originalCheckForSuccessMsg = ProjectAreaEditor.prototype.checkForSuccessMsg;
			ProjectAreaEditor.prototype.checkForSuccessMsg = function() {
				var dmMessage = dojo.cookie(this.getDmContributionMessageCookieName());
				if (dmMessage) {
					var now = new Date();
					now.setTime(now.getTime() - 1);
					dojo.cookie(this.getDmContributionMessageCookieName(), ProcessConstants.EMPTY, {
						expires: now
					});
					var originalMessage = dojo.cookie(JTS_MSG_COOKIE_NAME);
					if (originalMessage) {
						dojo.cookie(JTS_MSG_COOKIE_NAME, originalMessage + ' ' + dmMessage);
					}
				}
				originalCheckForSuccessMsg.apply(this, arguments);
				// Now the JTS success message cookie must be processed and removed. Process DM error messages.
				var errorObject = dojo.cookie(this.getDmContributionErrorCookieName());
				if (errorObject) {
					var now = new Date();
					now.setTime(now.getTime() - 1);
					dojo.cookie(this.getDmContributionErrorCookieName(), ProcessConstants.EMPTY, {
						expires: now
					});
					errorObject = dojo.fromJson(errorObject);
					ProjectAreaEditor.prototype._handleWarning.apply(this, [errorObject, ProcessConstants.ERROR_PRIORITY_WARNING_LOW + 1]);
				}
			};
			
			/*
			 * Adds getDmContributionMessageCookieName() function to have access to the cookie
			 */
			ProjectAreaEditor.prototype.getDmContributionMessageCookieName = function() {
				return MSG_COOKIE_NAME;
			};
			
			/*
			 * Adds _getDmContribution() function to find DM contribution embedded in the main 
			 * Overview tab or just contributed as a separate tab
			 */
			ProjectAreaEditor.prototype._getDmContribution = function() {
				var dmContribution;
				if (this.contribution) {
					dmContribution = this.contribution;
				} else if (this._tabs && this._tabs[DM_CONTRIBUTION_TAB_ID]) {
					dmContribution = this._tabs[DM_CONTRIBUTION_TAB_ID].mainPage;
				}
				return dmContribution;
			};
			
			/*
			 * Override getServerResponseSuccessMessage() to check for DM project area save success message as well
			 */
			var originalGetServerResponseSuccessMessage = ProjectAreaEditor.prototype.getServerResponseSuccessMessage;
			ProjectAreaEditor.prototype.getServerResponseSuccessMessage = function(){
				var msg = originalGetServerResponseSuccessMessage.apply(this, arguments);
				if (!this.isTeamArea) {
					var dmContribution = this._getDmContribution();
					if (dmContribution) {
						var contributionMessage = dmContribution.getSuccessContributionMessage();
						if (contributionMessage) {
							msg += ' ' + contributionMessage;
						}
					}
				}
				return msg;
			};

			/*
			 * Override _handleNavigate() to discard the case of a javascript link
			 */
			var originalHandleNavigate = ProjectAreaEditor.prototype._handleNavigate;
			ProjectAreaEditor.prototype._handleNavigate = function(urlObj){
				if (urlObj && urlObj.scheme === "javascript") { //$NON-NLS-1$		
					return false;
				}	
				return originalHandleNavigate.apply(this, arguments);
			};

			/*
			 * Override uninitialize() to discard the DM specific settings UI
			 */
			originalUninitialize = ProjectAreaEditor.prototype.uninitialize;
			 ProjectAreaEditor.prototype.uninitialize = function(){
				if (this.contribution) {
					this.contribution.uninitialize();
					this.contribution.destroyRecursive();
					delete this.contribution;
				}
				originalUninitialize.apply(this, arguments);
			};
			
			/*
			 * Override clearState() to refresh and discard changes to the DM specific settings UI
			 */
			originalClearState = ProjectAreaEditor.prototype.clearState;
			ProjectAreaEditor.prototype.clearState = function(refreshClicked) {
				originalClearState.apply(this, arguments);
				if (this.projectConfigurationsInfo) {
					this.projectConfigurationsInfo.reset();
				}
				if (this.contribution) {
					this.contribution.clearState(refreshClicked);
				}
			};

			/*
			 * Override processErrorData() to create a proper error message for DM specific
			 * settings save errors
			 */
			originalProcessErrorData = ProjectAreaEditor.prototype.processErrorData;
			ProjectAreaEditor.prototype.processErrorData = function(errorData, messageBoxArgs) {
				originalProcessErrorData.apply(this, arguments);
				if (errorData) {
					if (errorData.details) {
						messageBoxArgs.details = errorData.details;
					}
					if (errorData.summary) {
						messageBoxArgs.summary = errorData.summary;
					}
				}
				messageBoxArgs.closable = true;
			};
			
			/*
			 * Adds createDmContributionErrorObjectCookie() function to store
			 * DM contribution specific error object
			 */
			ProjectAreaEditor.prototype.getDmContributionErrorCookieName = function() {
				return ERROR_COOKIE_NAME;
			};

		}
		
	};
	
}());
