/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("rmps.app.ui.internal.util.DomainVersionsReminder");
	
	dojo.require("rmps.core.RootServices");
	dojo.require("rmps.app.ui.internal.dialogs.SelectDomainVersionDialog");
	dojo.require("rmps.core.util.ContextUtils");
	dojo.require("rmps.core.client");
	
	dojo.require("dojo.i18n");

	dojo.requireLocalization("rmps.app.ui.internal.util", "DomainVersionsReminderMessages");

	var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.util", "DomainVersionsReminderMessages");

	var DOMAIN_REG_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "domainRegistry");
	var ContextUtils = rmps.core.util.ContextUtils;

	rmps.app.ui.internal.util.DomainVersionsReminder = {
			
			remindersRegistry: null,
			
			ids: ["rmps.app.ui.internal.util.RemindersRegistry.setVersions",
			      "rmps.app.ui.internal.util.RemindersRegistry.dismiss"
			      ],
			
			request: function() {
				var context = jazz.app.context.get();
				if (context && context.baseline && ContextUtils.isWorkspace(context.baseline)) {
					return rmps.core.client.xhrGet({
						url: DOMAIN_REG_PATH + "/getDomainVersionsReminder",
						handleAs: 'json',
						headers: {
							"Accept" : "application/json;text/json"
						},
						load: dojo.hitch(this, function(response, ioArgs) {
							if (response) {
								var subs = [];
								for (var i in this.ids) {
									subs.push("#");
									subs.push(this.ids[i]);
								}
								return dojo.string.substitute(messages.remindSetDomainVersions, subs);
							}
						})
					}, true);
				}
			},
			
			handleLink: function(e) {
				var domNode = e.currentTarget.parentNode;
				if (e.currentTarget.id === "rmps.app.ui.internal.util.RemindersRegistry.setVersions") {
					var dialog = rmps.app.ui.internal.dialogs.SelectDomainVersionDialog({
						domainDataArray: this._domainDataArray,
						readOnly: false,
					});
					dojo.connect(dialog, "okPressed", dojo.hitch(this, function() {
						if (domNode) {
							this.remindersRegistry.removeReminder(domNode);
						}
					}));
					dialog.show();
				} else if (e.currentTarget.id === "rmps.app.ui.internal.util.RemindersRegistry.dismiss") {
					this._removeReminder(domNode);
				}
			},
			
			_removeReminder: function(domNode) {
				var request = rmps.core.client.xhrPut({
					url: DOMAIN_REG_PATH + "/removeDomainVersionsReminder",
					handleAs: 'json',
					headers: {
						"Accept" : "application/json;text/json"
					}
				}, true);
				if (domNode) {
					this.remindersRegistry.removeReminder(domNode);
				}
			}
	};
	
})();
