/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.util.RemindersRegistry");

dojo.require("dojox.lang.aspect");
dojo.require("dojo.DeferredList");
dojo.require("jazz.app.context");
dojo.require("jazz.core.RegistryFactory");

dojo.require("jazz.ui.MessageBox");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.app.ui.internal.util", "RemindersRegistryMessages");

var messages = dojo.i18n.getLocalization("rmps.app.ui.internal.util", "RemindersRegistryMessages");

var EXTENSION_POINT_REMINDER = "com.ibm.xtools.rmps.web.app.reminderUiContribution";
var ATTRIBUTE_REMINDER_OBJECT = "reminderObject";

var RegistryFactory = jazz.core.RegistryFactory;


rmps.app.ui.internal.util.RemindersRegistry = {
		
	_messageBox: null,
	
	_remindObjects: [],
	
	_reminders: [],
		
	showReminders: function() {
		var deferreds = [];

		for (var i in this._remindObjects) {
			var deferred = this._remindObjects[i].request();
			if (deferred) {
				deferreds.push(deferred);
			}
		}
		
		var dl = new dojo.DeferredList(deferreds);
		dl.then(dojo.hitch(this, function(results) {
			this._reminders = [];
			for (var i = 0; i < results.length; i++) {
				if (results[i][0]) {
					if (typeof results[i][1] === 'string') {
						this._reminders.push(results[i][1]);
					} else if (typeof results[i][1] === 'object' && results[i][1].constructor === Array ) {
						for (var j = 0; j < results[i][1].length; j++) {
							if (typeof results[i][1][j] === 'string') {
								this._reminders.push(results[i][1][j]);
							}
						}
					}
				}
			}
			if (this._reminders.length > 0) {
				this._updateUI();
			}
		}));
	},
	
	removeReminder: function(domNode) {
		if (domNode.className === 'messageSummary') {
			this._reminders = [];
		} else {
			var i = 0;
			var child = domNode;
			while( (child = child.previousSibling) != null ) 
			  i++;	
			this._reminders.splice(i, 1);
		}
		this._updateUI();
		if (this._reminders.length > 1) {
			this._messageBox._toggleDetails();
		}
	},
	
	_updateUI: function() {
		this._messageBox.hideMessage();
		if (this._reminders && this._reminders.length > 0) {
			if (this._reminders.length == 1) {
				this._messageBox.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: this._reminders[0],
					closable: true
				});
			} else {
				var details = [];
				for (var i in this._reminders) {
					details.push({message: this._reminders[i]});
				}
				this._messageBox.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO,
					summary: dojo.string.substitute(messages.multiSummary, [this._reminders.length]),
					closable: true,
					details: details
				});
			}
			
			for (var i in this._remindObjects) {
				var remindObject = this._remindObjects[i];
				if (remindObject.ids && remindObject.handleLink.call) {
					for (var j in remindObject.ids) {
						var node = dojo.byId(remindObject.ids[j]);
						if (node) {
							dojo.connect(node, "onclick", function(e){
								e.preventDefault();
								remindObject.handleLink(e);
							});
						}
					}
				}
			}
		}
	},
	
	_loadRegistry: function(){
		// Get extension registry
		var registry = RegistryFactory.getRegistry();
		if (!registry) {
			return;
		}
		// Get extension points
		var extensionsPoints = registry.getExtensionPoint(EXTENSION_POINT_REMINDER);
		if (!extensionsPoints) {
			return;
		}
		// Get extensions
		var extensions = extensionsPoints.getExtensions();
		if ((extensions instanceof Array === false) ||
		(extensions.length === 0)) {
			return;
		}
		// Get elements
		var allElements = [];
		for ( var index = 0; index < extensions.length; index++) {
			var elements = extensions[index].getConfigurationElements();
			if (elements instanceof Array) {
				for ( var i = 0; i < elements.length; i++) {
					var reminderObject = elements[i].getAttribute(ATTRIBUTE_REMINDER_OBJECT);
					if (reminderObject) {
						// Load widget JS, HTML & CSS	
						jazz.core.loader.load_async(reminderObject, dojo.hitch(this, function(){
						    var object = dojo.getObject(reminderObject);
                            if (object && object.request.call) {
                                object.remindersRegistry = this;
                                this._remindObjects.push(object);
                            }
						}));
						
					}
				}
			}
		}
	}
	
};

rmps.app.ui.internal.util.RemindersRegistry._loadRegistry();

if (rmps.app.ui.internal.application.ModelingApplication) {
dojox.lang.aspect.advise(rmps.app.ui.internal.application.ModelingApplication, "start", {
	
	after: function() {
		var self = dojox.lang.aspect.getContext().instance;
			
		// We must sent a resize event when the message box is shown/hidden
		// and when the details are toggled.
		function onHideMessage() {
			//user closed message, set cookie to stop message from appearing at page reload
			publishResizeEvent();
		}
		function publishResizeEvent() {
			dojo.publish("jazz.util.ResizeNotifier.resizeEvent");
		}
			
		messageBox = new jazz.ui.MessageBox();
		messageBox.domNode.style.margin = "1px";
		messageBox._first = true;
		dojo.connect(messageBox, "_toggleDetails", self, publishResizeEvent);
		dojo.connect(messageBox, "hideMessage", self, onHideMessage);
		dojo.connect(messageBox, "showMessage", self, publishResizeEvent);
			
		// Add the message box before the page container so that it will show up
		// no matter which page is currently being viewed.
		for (var i = 0, c; c = self.ui.containerNode.childNodes[i]; ++i) {
			if (dijit.byNode(c) instanceof net.jazz.ajax.ui.PageContainer) {
				self.ui.containerNode.insertBefore(messageBox.domNode, c);
				messageBox.startup();
				break;
			}
		}

		rmps.app.ui.internal.util.RemindersRegistry._messageBox = messageBox;
		rmps.app.ui.internal.util.RemindersRegistry.showReminders();			
	}
});
}

})();