/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.reports                                        |
 *|                                                                        |
 *| Copyright IBM Corporation 2011, 2012.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() { 
dojo.provide("dm.app.reports.ui.internal.util.ReportServiceClient");

/** Report REST Service Interface.
 *  This class provides method for accessing the JFS reporting framework
 *  via the Report REST Service. The service provides many additional
 *  methods but only those used by the DM reports application are defined 
 *  here.
 *  
 *  @author Kevin Cornell
 */

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

dojo.require("dm.app.reports.ui.internal.util.ReportUtil");


var ServiceRequest   = com.ibm.team.repository.web.transport.ServiceRequest;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var ReportUtil       = dm.app.reports.ui.internal.util.ReportUtil;

	/* Report REST Service client
	 */
dm.app.reports.ui.internal.util.ReportServiceClient = {
		
	/** Define the flags for various service requests. */
	ALL : 0,
	SHARED  : 1 << 0,
	PRIVATE : 1 << 2,
	SYSTEM  : 1 << 3,
	EXCLUDE_MICRO : 1 << 4,
	LOCAL  : 1 << 0,
	REMOTE : 1 << 1,
	
	// parameter dependencies based on RPE metadata
	DEPENDENCIES : 1 << 24,

	SERVICE_URI : "com.ibm.team.reports.common.internal.service.IReportRestService",

	/*---------- Report Template methods ----------*/
	
	/**
	 * Get the report descriptor identified by the "reportUUID" argument.
	 * If the projectAreaUUID is not defined, it is set to the current
	 * project area.
	 * 
	 * @param reportUUID: the internal ID of the corresponding report
	 * @param projectAreaUUDI: the project area internal ID (optional)
	 * @return the report descriptor
	 */
	getReportDescriptor : function(responseHandler, args) {
		if (!args.projectAreaUUID) {
			args.projectAreaUUID = ReportUtil.getProjectId();
		}
		return this._invokeReportRESTService(responseHandler, "getReportDescriptor",
				args);
	},

	/**
	 * Get the report descriptors defined for a project area. If
	 * the project area ID is not defined, the current project is
	 * used.
	 * 
	 * @param  projectAreaUUID: the project area ID (optional)
	 * @return ReportDescriptor[] an array of report descriptors (templates)
	 */
	getReportDescriptors : function(responseHandler, args) {
		if (!args.projectAreaUUID) {
			args.projectAreaUUID = ReportUtil.getProjectId();
		}
		return this._invokeReportRESTService(responseHandler, "getReportDescriptors",
				args);
	},

	/**
	 * Save a template with the given arguments. This could be creating a
	 * new one or saving an existing one.
	 * 
	 * @param reportUUID: the report template internal ID 
	 */
	postSaveReportDescriptor : function(responseHandler, args) {
		if (!args.projectAreaUUID) {
			args.projectAreaUUID = ReportUtil.getProjectId();
		}
		return this._invokeReportRESTService(responseHandler,
				"postSaveReportDescriptor", args);
	},

	/**
	 * Reset the given report descriptor to its factory defaults.
	 * 
	 * @param reportUUID: the ID of the report descriptor (not defined for new template)
	 */
	postResetReportDescriptor : function(responseHandler, args) {
		return this._invokeReportRESTService(responseHandler,
				"postResetReportDescriptor", args);
	},

	/**
	 * Delete the given report descriptor given its ID.
	 * 
	 * @param reportUUID: the ID of the report descriptor to delete
	 */
	postDeleteReportDescriptor : function(responseHandler, args) {
		return this._invokeReportRESTService(responseHandler,
				"postDeleteReportDescriptor", args);
	},

	/** Get deployed (shared) report descriptors that have not already
	 *  been added to the project. 
	 *  
	 *  @param processAreaUUID the process area ID
	 */
	getMissingSharedReportDescriptors : function(responseHandler, args) {
		if (!args.projectAreaUUID) {
			args.projectAreaUUID = ReportUtil.getProjectId();
		}
		return this._invokeReportRESTService(responseHandler,
				"getMissingSharedReportDescriptors", args);
	},

	/** Save deployed report descriptors with the given information. 
	 *  This is a shortcut so the web UI doesn't have to call
	 *  postSaveReportDescriptor() in a loop for every report descriptor to
	 *  create.
	 *  
	 *  @param processAreaUUID the process area ID
	 */
	postSaveMissingReportDescriptors : function(responseHandler, args) {
		this._invokeReportRESTService(responseHandler,
				"postSaveMissingReportDescriptors", args);
	},

	/*---------- Report Folder methods ----------*/

	/** Get a specific folder. The arguments must specify
	 *  the corresponding folder UUID.
	 */
	getFolderDescriptor : function(responseHandler, args) {
		return this._invokeReportRESTService(responseHandler, "getFolderDescriptor",
				args);
	},

	/** Get the folders in the given project area.
	 * 
	 * @param args	can define flags to limit the folders to
	 *				private or shared.
	 */
	getFolderDescriptors : function(responseHandler, args) {
		if (!args.projectAreaUUID) {
			args.projectAreaUUID = ReportUtil.getProjectId();
		}
		return this._invokeReportRESTService(responseHandler,
				"getFolderDescriptors", args);
	},

	/** Save a folder. This could be creating a new
	 *  folder or saving an existing one.
	 */
	postSaveFolderDescriptor : function(responseHandler, args) {
		if (!args.projectAreaUUID) {
			args.projectAreaUUID = ReportUtil.getProjectId();
		}
		return this._invokeReportRESTService(responseHandler,
				"postSaveFolderDescriptor", args);
	},
	
	/** Delete the given folder.
	 */
	postDeleteFolderDescriptor : function(responseHandler, args) {
		return this._invokeReportRESTService(responseHandler,
				"postDeleteFolderDescriptor", args);
	},

	/*---------- Report Configuration methods ----------*/

	/** Get the report configuration (query descriptor) 
	 *  for the queryUUID provided.
	 */
	getQueryDescriptor : function(responseHandler, args) {
		return this._invokeReportRESTService(responseHandler, 
				"getQueryDescriptor", args);
	},

	/** Get the report configurations (e.g., reports). 
	 */
	getQueryDescriptors : function(responseHandler, args) {
		if (!args.projectAreaUUID) {
			args.projectAreaUUID = ReportUtil.getProjectId();
		}
		return this._invokeReportRESTService(responseHandler,
				"getQueryDescriptors", args);
	},

	/** Save a report configuration (query descriptor) with 
	 *  the given data. This could be creating a new
	 *  report configuration or saving an existing one.
	 */
	postSaveQueryDescriptor : function(responseHandler, args) {
		if (!args.projectAreaUUID) {
			args.projectAreaUUID = ReportUtil.getProjectId();
		}
		return this._invokeReportRESTService(responseHandler,
				"postSaveQueryDescriptor", args);
	},
	
	/** Delete the given report configuration.
	 *
	 */
	postDeleteQueryDescriptor : function(responseHandler, args) {
		return this._invokeReportRESTService(responseHandler,
				"postDeleteQueryDescriptor", args);
	},
	
	/** Rendered the report for the given query.
	 */
	postRenderQuery : function(responseHandler, args) {
		if (!args.projectAreaUUID) {
			args.projectAreaUUID = ReportUtil.getProjectId();
		}
		return this._invokeReportRESTService(responseHandler, 
				"postRenderQuery", args);
	},


	/*---------- Report Parameters methods ----------*/

	/** Get the default parameters for a report template given
	 *  either the queryUUID or the reportUUID.
	 */
	getParameters : function(responseHandler, args) {
		if (!args.projectAreaUUID) {
			args.projectAreaUUID = ReportUtil.getProjectId();
		}
		return this._invokeReportRESTService(responseHandler, "getParameters", args);
	},

	/** Get the possible values for a given parameter. The arguments
	 *  must define the query UUID, and the parameter name.
	 *  Optional parameters may include the filter string and
	 *  a list of system variable names and values.
	 *
	 *  This method sends the args as query parameters on the
	 *  service request URL. As such, too many parameters can
	 *  cause the request to exceed the HTTP maximum size.
	 *  Use postcomputeParameterValues() instead.
	 */
	getParameterValues : function(responseHandler, args) {
		if (!args.projectAreaUUID) {
			args.projectAreaUUID = ReportUtil.getProjectId();
		}
		return this._invokeReportRESTService(responseHandler, "getParameterValues",
				args);
	},

	/** Get the possible values for a given parameter. This
	 *  method is a replacement for getParameterValues(), which
	 *  sends the args as data in the service request. The arguments
	 *  must define the query UUID, and the parameter name.
	 *  Optional parameters may include the filter string and
	 *  a list of system variable names and values.
	 */
	postComputeParameterValues : function(responseHandler, args) {
		if (!args.projectAreaUUID) {
			args.projectAreaUUID = ReportUtil.getProjectId();
		}
		return this._invokeReportRESTService(responseHandler, "postComputeParameterValues",
				args);
	},

	/** Get the possible values for a group parameter. The arguments
	 *  must define the query UUID, and the group parameter (name).
	 *  Optional parameters may include the filter string and
	 *  a list of system variable names and values.
	 */
	getParameterGroupValues : function(responseHandler, args) {
		if (!args.projectAreaUUID) {
			args.projectAreaUUID = ReportUtil.getProjectId();
		}
		return this._invokeReportRESTService(responseHandler,
				"getParameterGroupValues", args);
	},


		/**
		 * Determine whether the current user can deploy queries.
		 * 
		 * @return boolean (true indicates user has permissions to deploy a query)
		getCanDeployQuery : function(responseHandler) {
			this._invokeReportRESTService(responseHandler, "getCanDeployQuery");
		},
		 */

		/**
		 * Fetch whether the current user can manage folders.
		getCanManageFolder : function(responseHandler) {
			this._invokeReportRESTService(responseHandler, "getCanManageFolder");
		},
		 */

		/**
		 * Fetch whether the current user can see reports UI.
		getCanSeeReports : function(responseHandler) {
			this._invokeReportRESTService(responseHandler, "getCanSeeReports");
		},
		 */


		/**
		 * Get the parameters section for the given query.
		getQueryParametersSection : function(responseHandler, args) {
			this._invokeReportRESTService(responseHandler,
					"getQueryParametersSection", args);
		},
		 */

		/**
		 * Get the rendered report for the given query.
		getRenderedQuery : function(responseHandler, args) {
			this._invokeReportRESTService(responseHandler, "getRenderedQuery",
					args);
		},
		 */
		
		/**
		 * Check whether we should display the legacy snapshot ui or not
		getShouldHideLegacy : function(responseHandler){
			this._invokeReportRESTService(responseHandler, "getShouldHideLegacy");
		},
		 */
		
		/**
		 * Get the current data warehouse provider
		getDataWarehouseProvider : function(responseHandler) {
			this._invokeReportRESTService(responseHandler, "getDataWarehouseProvider");
		},
		 */
		
		/**
		 * Get the list of etl job history
		getEtlJobHistory : function(responseHandler, args) {
			this._invokeReportRESTService(responseHandler, "getEtlJobHistory",
					args);
		},
		 */
		
		/**
		 * Get the list of running etl jobs
		getRunningEtlJobs : function(responseHandler) {
			this._invokeReportRESTService(responseHandler, "getRunningEtlJobs");
		},
		 */

		/**
		 * Get the rendered report for the given report.
		getRenderedReport : function(responseHandler, args) {
			if (args.projectAreaUUID == undefined) {
				args.projectAreaUUID = ReportUtil.getProjectId();
			}
			this._invokeReportRESTService(responseHandler, "getRenderedReport",
					args);
		},
		 */

		/**
		 * Get the rendered report for the given report.
		postRenderReport : function(responseHandler, args) {
			if (args.projectAreaUUID == undefined) {
				args.projectAreaUUID = ReportUtil.getProjectId();
			}
			this._invokeReportRESTService(responseHandler, "postRenderReport",
					args);
		},
		 */

		/**
		 * Get whether the given report exists
		getDoesReportDescriptorExist : function(responseHandler, args) {
			if (args.projectAreaUUID == undefined) {
				args.projectAreaUUID = ReportUtil.getProjectId();
			}
			this._invokeReportRESTService(responseHandler,
					"getDoesReportDescriptorExist", args);
		},
		 */

		/**
		 * Get the default report for the project area
		getDefaultReportDescriptor : function(responseHandler) {
			var args = {
				projectAreaUUID : ReportUtil.getProjectId()
			};
			this._invokeReportRESTService(responseHandler,
					"getDefaultReportDescriptor", args);
		},
		 */

		/**
		 * Helper function. ProcessClient doesn't provide this.
		getProcessAreaByUUID : function(responseHandler, args) {
			this._invokeReportRESTService(responseHandler, "getProcessAreaByUUID",
					args);
		},
		 */

		/**
		 * Set the given report template as the default template for viewing in
		 * the APT editor
		postSetDefaultReportDescriptor : function(responseHandler, args) {
			this._invokeReportRESTService(responseHandler,
					"postSetDefaultReportDescriptor", args);
		},
		 */

		/**
		 * Get whether the current user is a data warehouse administrator
		getIsDWAdmin : function(responseHandler) {
			this._invokeReportRESTService(responseHandler, "getIsDWAdmin");
		},
		 */

		/**
		 * Get the list of snapshot names
		getAllSnapshotNames : function(responseHandler) {
			this._invokeReportRESTService(responseHandler, "getAllSnapshotNames");
		},
		 */

		/**
		 * Get the list of snapshot descriptors
		getSnapshotDescriptors : function(responseHandler, args) {
			this._invokeReportRESTService(responseHandler,
					"getSnapshotDescriptors", args);
		},
		 */

		/**
		 * Get the snapshot descriptor
		getSnapshotDescriptor : function(responseHandler, args) {
			this._invokeReportRESTService(responseHandler,
					"getSnapshotDescriptor", args);
		},
		 */

		/**
		getSnapshotSchedulingDefaults : function(responseHandler) {
			return this._invokeReportRESTService(responseHandler,
					"getSnapshotSchedulingDefaults");
		},
		*/

		/**
		 * Data Warehouse Administration functions.
		postUpdateAllSnapshotData : function(responseHandler) {
			this._invokeReportRESTService(responseHandler,
					"postUpdateAllSnapshotData");
		},

		postUpdateSomeSnapshotData : function(responseHandler, args) {
			this._invokeReportRESTService(responseHandler,
					"postUpdateSomeSnapshotData", args);
		},
		
		postUpdateAllRemoteSnapshotData : function(responseHandler) {
			this._invokeReportRESTService(responseHandler,
					"postUpdateAllRemoteSnapshotData");
		},

		postUpdateSnapshotData : function(responseHandler, args) {
			if(args.delta == undefined || args.delta == null){
				args.delta = true;
			}
			this._invokeReportRESTService(responseHandler,
					"postUpdateSnapshotData", args);
		},

		postPingRemoteSnapshot : function(responseHandler, args) {
			this._invokeReportRESTService(responseHandler,
					"postPingRemoteSnapshot", args);
		},

		postExportAllSnapshotData : function(responseHandler, args) {
			this._invokeReportRESTService(responseHandler,
					"postExportAllSnapshotData", args);
		},

		postExportSnapshotData : function(responseHandler, args) {
			this._invokeReportRESTService(responseHandler,
					"postExportSnapshotData", args);
		},

		postImportSnapshotData : function(responseHandler, args) {
			this._invokeReportRESTService(responseHandler,
					"postImportSnapshotData", args);
		},

		postInitializeDataWarehouse : function(responseHandler) {
			this._invokeReportRESTService(responseHandler,
					"postInitializeDataWarehouse");
		},

		getDataWarehouseInitializationStatus : function(responseHandler) {
			this._invokeReportRESTService(responseHandler,
					"getDataWarehouseInitializationStatus");
		},

		postCreateAllSnapshotSchemas : function(responseHandler) {
			this._invokeReportRESTService(responseHandler,
					"postCreateAllSnapshotSchemas");
		},

		postCreateSnapshotSchema : function(responseHandler, args) {
			this._invokeReportRESTService(responseHandler,
					"postCreateSnapshotSchema", args);
		},

		postUpdateParameterValues : function(responseHandler) {
			this._invokeReportRESTService(responseHandler,
					"postUpdateParameterValues");
		},

		postSaveQueryParameters : function(responseHandler, args) {
			this._invokeReportRESTService(responseHandler,
					"postSaveQueryParameters", args);
		},
		
		getPerformDataWarehouseDiagnostics : function(responseHandler) {
			this._invokeReportRESTService(responseHandler,
					"getPerformDataWarehouseDiagnostics");
		},
		 */

		/**
		 * Invokes the requested method on the Report REST Service.
		 */
		_invokeReportRESTService : function(responseHandler, method, args) {
			// args is optional, so test whether or not it was sent
			var serviceRequest;
			if (args) {
				serviceRequest = new ServiceRequest(this.SERVICE_URI, method, args);
			} else {
				serviceRequest = new ServiceRequest(this.SERVICE_URI, method);
			}
			TeamServerClient.invokeService(serviceRequest, responseHandler);
			return serviceRequest._deferred;
		}
	};

})();