/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.reports                                        |
 *|                                                                        |
 *| Copyright IBM Corporation 2011.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() { 
dojo.provide("dm.app.reports.ui.internal.util.ReportUtil");

/** Utility methods for the DM reports application.
 * 
 * @author Kevin Cornell
 */

dojo.require("jazz.app.context");

dojo.require("rmps.core.RootServices");

dm.app.reports.ui.internal.util.ReportUtil = {
	
	// Define the background colors for table/tree highlighting
	COLOR_NORMAL:	"#FFFFFF",
	COLOR_IN_FOCUS:	"#FFF4E2",
	COLOR_SELECTED:	"#FED0A7",
	COLOR_UNSELECT:	0,
	
	// define the HTML to append to a field that is mandatory (or needs attention)
	MANDATORY_HTML: '<font color="#FF0000"> *</font>',

	/** Return the project area ID */
	getProjectId: function() {
		var projectArea = jazz.app.context.get();
		return projectArea.itemId;
    },


    /** Connect a text input widget for the given page, associating
     *  the callback method to handle changes.
     *  
     *  @param textWidget the text input control to connect
     *  @param pageWidget  the parent widget of the input control
     *  @param method the callback method
     *  
     */
    connectTextInput: function(textWidget, pageWidget, method) {
		if (dojo.isMoz || dojo.isOpera) {
			pageWidget.connect(textWidget, "oninput", method);
		} else {
			pageWidget.connect(textWidget, "onkeydown", method);
			pageWidget.connect(textWidget, "onkeyup", method);
			pageWidget.connect(textWidget, "onpaste", function() {
				setTimeout(dojo.hitch(this, method), 1);
			});
			pageWidget.connect(textWidget, "oncut", function() {
				setTimeout(dojo.hitch(this, method), 1);
			});
			pageWidget.connect(textWidget, "onblur", method);
		}
    },
    
	/** Get the root server URL where the JFS reporting framework resides.
	 * 
	 *  This method returns the appropriate server URl where the JFS 
	 *  reporting framework plug-ins resides. In DM 4.0, this URL
	 *  should be https://<host:port>/dm
	 *  
	 *  Until DM adopts JAF, the reporting framework resides in JTS.
	 * 
	 */
	getReportingRootUrl: function() {
		// Reporting framework is now in DM since JAF was adopted.
		var serviceInJFS = false;
		
		// Get the URLs for both the DM server and its underlying JFS server
		var jts_dm_paths = rmps.core.RootServices.getValue(rmps.core.Namespaces.JFS, "oauthDomain");
		var reportingRootUrl = null;
		if (jts_dm_paths) {
			var serverPaths = jts_dm_paths.split(",");
			for (var i = 0; i < serverPaths.length; i++) {
				var serverPath = dojo.trim(serverPaths[i]);
				var lastSep = serverPath.lastIndexOf("/");
				if (lastSep > 0) {
					var serverName = serverPath.substring(lastSep);
					if (serverName === "/dm" && !serviceInJFS) {
						reportingRootUrl = serverPath;
					} else if (serverName === "/jts" && serviceInJFS) {
						reportingRootUrl = serverPath;
					}
				}
			}
		} 
		return reportingRootUrl;
	}


};

})();