/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.xtools.rmps.web.reports                                        |
 *|                                                                        |
 *| Copyright IBM Corporation 2011, 2012.  All Rights Reserved.            |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() { 
dojo.provide("dm.app.reports.ui.internal.util.ResourceServiceClient");

/** Report Resource Content Service.
 * 
 *  This class provides method for interfacing with the content
 *  uploading service of the JFS reporting framework.
 *  
 *  @author Kevin Cornell
 */

dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

dojo.require("dojo.i18n");
dojo.requireLocalization("dm.app.reports.ui.internal", "ReportMessages");
var messages  = dojo.i18n.getLocalization("dm.app.reports.ui.internal", "ReportMessages");

var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var FILE_UPLOAD_RESPONSE = "File Upload Response:";
var RESOURCE_SERVICE_URI = TeamServerClient._getServicePrefix() + "com.ibm.team.reports.service.internal.IReportResourceService";

var _uploadResourceRequestNumber = 100;

/* Reporting Resource (content) Service client
 */
dm.app.reports.ui.internal.util.ResourceServiceClient = {

		
/** Perform the uploading of a resource into the server.
 *  This function requires the corresponding html page and
 *  the file input node where the name of the resource is
 *  defined. The service response handler should define
 *  callback methods to handle a successful upload and to
 *  handle an error. 
 *  
 *  The name of the <input> widget with type="file" must be
 *  "fileLocation" or else the resource content service will
 *  not work.
 *  
 *  In both cases, the response callback function is passed
 *  a callback data object which has at least fields defined: 
 *  message and resourceName. If the upload was successful, 
 *  message is null otherwise it contains the error message.
 * 
 *  If the upload was successful, the following fields in the
 *  callback data are also defined:
 *  
 *      resourceLocation		- location on server where uploaded data resides
 *      resourceSize			- size of the uploaded data
 *      resourceContentType		- content type of the uploaded data
 *      resourceChecksum		- checksum for validating the data
 *      resourceFileExtension	- uploaded file name extension
 * 
 */
uploadResource: function(htmlPage, fileInputNode, serviceResponseHandler) {
	
	// Cancel the previous upload resource request for this HTML page
	if (!htmlPage._uploadResourceRequest) {	
		htmlPage._uploadResourceRequest = null;  // Ensure the request handle field is defined
	}
	if (htmlPage._uploadResourceRequest !== null) {
		htmlPage._uploadResourceRequest.cancel();
		htmlPage._uploadResourceRequest = null;
	}
		
	/*
	 * Internet Explorer (with dojo) has a bug such that if you wait ten seconds after one file upload, the 
	 * next one breaks. Resetting the iframe fixes the problem
	 * Unfortunately this code reaches into iframe private variables.
	 */
	if (dojo.io.iframe._frame) {
		document.body.removeChild(dojo.io.iframe._frame);				
	}
	var frameName = "dojoIoIframe" + _uploadResourceRequestNumber++;
	dojo.io.iframe._iframeName = frameName;
	dojo.io.iframe._frame = null;
		
	// Send a request to the reporting resource service to upload 
	// the resource that is specified in the fileInputNode.
	htmlPage._uploadResourceRequest = dojo.io.iframe.send({
		form: fileInputNode.form,
		url: RESOURCE_SERVICE_URI,
		content: {
				mimetype: "text/html",
				fileFields: fileInputNode.name
			},
		handleAs: "html",
		
		// Handle the response from the resource service
		handle: function(response, ioArgs) {
			
			// Initialize the callback data (assume an error occurred)
			var callbackData = { 
					resourceName: fileInputNode.value,
					resourceLocation: null,
					resourceSize: 0,
					resourceContentType: null,
					resourceChecksum: null,
					resourceFileExtension: null
					};
			
			// Assume the upload response cannot be parsed properly
			callbackData.message = messages.errorParsingUploadData;

			// If the response was some type of error
			if (response instanceof Error){
				// If the request was canceled, ignore the error.
				if (response.dojoType && response.dojoType === "cancel") {
					return; // Do not call the service response handler
				}
				htmlPage._uploadResourceRequest = null; // request is already complete
				
				// Ensure the error callback is called with the error message
				callbackData.message = response.message;
			} 
			
			// else the upload appears to be successful
			else if (response.body){
				// Extract the upload result string from the response
				var responseMsg = response.body.innerHTML;
				callbackData.message = responseMsg;	// assume the parsing does not work
				if (responseMsg.indexOf(FILE_UPLOAD_RESPONSE) === 0) {
					// Parse the upload result string to obtain the resource 
					// location, size, content type and checksum.
					var data = responseMsg.substring(FILE_UPLOAD_RESPONSE.length);
					var dataArray = data.split(",");
					if (dataArray.length === 4) {
						// Save the file information in the callback data
						callbackData.resourceLocation    = dataArray[0];
						callbackData.resourceSize        = dataArray[1];
						callbackData.resourceContentType = dataArray[2];
						callbackData.resourceChecksum    = dataArray[3];
						var split = callbackData.resourceName.split('.');
						callbackData.resourceFileExtension = split[split.length - 1];
						
						// Call the service response handler's success callback
						callbackData.message = null;
						serviceResponseHandler.doCallback(callbackData);
					}
				}
			}
			
			// If an error occurred or the response parsing failed
			if (callbackData.message !== null) {
				serviceResponseHandler.doErrorCallback(callbackData);
			}
		}		  
	});
}

};

})();