/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
dojo.provide("rmps.app.ui.internal.viewers.ResourceLinkViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("rmps.app.ui.internal.views.CompactRenderingEditorInput");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.app.ui.internal.editors.CompactRenderingEditor");

/*Required if the module is going to be used at other servers. So, if the user is
 * not already logged into our server, then, we will show the login dialog */
dojo.require("jazz.app.auth"); 

var CompactRenderingEditorInput = rmps.app.ui.internal.views.CompactRenderingEditorInput;
var ResourceInfoClient = rmps.core.resource.ResourceInfoClient;
var CompactRenderingEditor= rmps.app.ui.internal.editors.CompactRenderingEditor;

dojo.declare("rmps.app.ui.internal.viewers.ResourceLinkViewer", [dijit._Widget, dijit._Templated], {
	
	templateString: '<div style="width: 100%; overflow: hidden; padding: 0px; position: relative;" '+ 
					'id="containerNode" class="rmps-graphics-resource-container" ' +
					'dojoAttachPoint="containerNode"></div>',	
	
	resourceUri: null,
	resourceInfo: null,
	height: 400,
	width: 400,
	isExpanded: false,
	projectArea: null,
	editor: null,
	baseline: null,
	changeset: null,

	constructor: function(mixin) {
		if (mixin) {
			dojo.mixin(this, mixin);
		}
	},
	
    postCreate: function() {
		this._extractParams();
		this._view();		
	}, 
	
	 _extractParams: function(){
    	var qstr = window.location.search.substr(1);
		if (qstr.length) {
			var qparts = qstr.split("&");
			for (var x = 0; x < qparts.length; x++) {
				var tp = qparts[x].split("=");
				if (tp[0] == "resourceUri") {
					this.resourceUri = decodeURIComponent(tp[1]); 
				} else if (tp[0] == "isExpanded") {
					this.isExpanded = eval(decodeURIComponent(tp[1])); 
				} else if (tp[0] == "projectArea") {
					this.projectArea = decodeURIComponent(tp[1]); 
				}else if(tp[0] == "rmps.context"){
					this.baseline= decodeURIComponent(tp[1]);
				}else if(tp[0] == "changeset"){
					this.changeset= decodeURIComponent(tp[1]);
				}
			}
		}
	}, 
		
	_view: function() {
		if(this.resourceUri) {
			var deferredResourceInfo = new dojo.Deferred();
			var properties = ["longName", "modificationTime", "displayType", "iconUri"];
			var deferred = ResourceInfoClient.getResourceInfo(this.resourceUri, properties, this.baseline, this.changeset);
			deferred.addCallback(this, function(resourceInfos) {
				this.resourceInfo = resourceInfos[0];
				deferredResourceInfo.callback(this.renderResource(this.resourceInfo));
			});
			deferred.addErrback(this, function(error) {
				deferredResourceInfo.errback();
			});
		}
    },
    
	renderResource: function(resourceInfo) {
		
		var eInput = new CompactRenderingEditorInput({
							resourceInfo: resourceInfo,
							resourceUri:this.resourceUri, 
							diagramNode: this.containerNode,
							isExpanded: this.isExpanded,
							baseline: this.baseline,
							changeset: this.changeset,
							projectArea: this.projectArea});
		
		
		/*Open The Editor */
		this.editor = new CompactRenderingEditor({
					editorInput: eInput
				});
		this.containerNode.appendChild(this.editor.domNode);
		this.editor.activate(/*No args */);
		
		/*
		 *The style position should be relative if we are going to show the compact rendering as hyperlink not
		 *as actions; this is in order to avoid getting extra vertical scrollbar in IE. 
		 *
		 *rmps.ui.ResourceLink#_styleLink: function () {
		 *	this.inherited(arguments);
		 *	if (dojo.isIE) {
		 *		this.linkNode.style.position = "relative";
		 *	}
		 *}
		 */
		
		/*
		 *TODO: We should have a util in order to open the compact rendering editors (cached)
		 *  
		 *CompactRenderingUtil.domNode = this.containerNode;
		 *CompactRenderingUtil.openEditor(eInput);
		 * 
		 */
	},

   	destroy: function() {
		this.inherited(arguments);
		delete this.resourceInfo;
	}
});


})();