/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.internal.viewers.ResourceViewInfoClient");

dojo.require("dijit._Widget");
dojo.require("dojo.html");

dojo.require("rmps.app.ui.internal.editors.DiagramEditor");
dojo.require("rmps.app.ui.internal.editors.PropertiesEditor");
dojo.require("rmps.ui.properties.PropertiesViewer");

dojo.require("jazz.util.Navigation");
dojo.require("jazz.util.ResizeNotifier");

dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.core.error");

dojo.require("rmps.app.core.util");

dojo.require("dojox.collections.Dictionary");
dojo.require("dojo.date.locale");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("rmps.app.ui.internal.viewers.ResourceLinkView");

rmps.app.ui.internal.viewers.ResourceViewInfoClient = {
	
	getResourceViewInfo: function(link) {
		var resourceUri = this.getResourceUri(link, "resourceUri");
		var deferredResourceInfo = new dojo.Deferred();		
		var deferred = rmps.core.resource.ResourceInfoClient.getResourceInfo(resourceUri);
		deferred.addCallback(this, function(resourceInfos) {
			deferredResourceInfo.callback(this._viewResource(resourceInfos[0]));
		});
		deferred.addErrback(function() {
			deferredResourceInfo.errback();
		});
		return deferredResourceInfo;
	}, 
	
	getResourceUri: function(link, key){
		var params = link.substr(1);
		if (params.length) {
			var qparts = params.split("&");
			for (var x = 0; x < qparts.length; x++) {
				var tp = qparts[x].split("=");
				if (tp[0] == key) {
					return decodeURIComponent(tp[1]);
				}
			}
		}
	},
	
	_viewResource: function(resourceInfo) {
		var editor;
		var resourceViewInfo = {
			"title": resourceInfo.shortName,
			"icon": resourceInfo.iconUri,
			"resourceInfo": resourceInfo,
			"widgetClass": undefined,
			"widgetInitializers": {}
		};

		if (resourceInfo.xmlNamespace && (
				(resourceInfo.xmlNamespace.search(/http\:\/\/www\.eclipse\.org\/gmf\/runtime\/.+\/notation/) != -1 &&
				resourceInfo.typeName == "Diagram") ||
				(resourceInfo.xmlNamespace.search(/http\:\/\/www\.ibm\.com\/xtools\/.+\/Umlnotation/) != -1 &&
					resourceInfo.typeName == "UMLDiagram"))) {
						resourceViewInfo.widgetClass = rmps.app.ui.internal.viewers.DiagramViewer;
						resourceViewInfo.widgetInitializers = {resourceInfo: resourceInfo, rendering:"EMBEDDED" };
						resourceViewInfo.args = {};
						resourceViewInfo.smallWidth= 300;
						resourceViewInfo.smallHeight= 300;
						resourceViewInfo.largeWidth= 500;
						resourceViewInfo.largeHeight= 500;
						resourceViewInfo.showExpand = true;
		} else {
			resourceViewInfo.widgetClass = rmps.app.ui.internal.viewers.PropertiesViewer;
			resourceViewInfo.widgetInitializers = {resourceUri: resourceInfo.uri};
			resourceViewInfo.args = {};			
			resourceViewInfo.smallWidth= 400;
			resourceViewInfo.smallHeight= 400;
			resourceViewInfo.largeWidth= 400;
			resourceViewInfo.largeHeight= 600;
			resourceViewInfo.showExpand = true;
		}
		return resourceViewInfo;
	}
};

dojo.declare("rmps.app.ui.internal.viewers.DiagramViewer", [rmps.app.ui.internal.viewers.ResourceLinkView], {
	resourceUri: null,
	rendering: null,
	
	createResourceView: function(widgetInitializer, args, hoverDiv) {
		this.hoverDiv = hoverDiv;
		var resourceInfo = widgetInitializer.resourceInfo;
		var rendering  = widgetInitializer.rendering;
    	this.viewer =  new rmps.app.ui.internal.viewers.DiagramInfo({
			resourceInfo: resourceInfo,
			rendering: rendering,
			args: args
		});

		this.inherited("createResourceView", arguments);
    },
    
    expand: function() {
		this.viewer.expand();
    }
});

dojo.declare("rmps.app.ui.internal.viewers.PropertiesViewer", [rmps.app.ui.internal.viewers.ResourceLinkView], {
	resourceUri: null,
	rendering: null,
	controller: null,
	args: null,
	
	createResourceView: function(widgetInitializer, args, hoverDiv) {
		this.hoverDiv = hoverDiv;
		this.args = args;
		var reourceUri = widgetInitializer.resourceUri;
    	this.viewer =  new rmps.ui.properties.PropertiesViewer({
			resourceUri: reourceUri
		});
		this.controller = new rmps.app.ui.internal.viewers.FilteredPropertiesController(this.viewer);
		this.controller.allowedCategories = [{label: "General"}];
		this.viewer._controller = this.controller;
		this.viewer.startup();
		if (this.viewer.activate) {
			this.viewer.activate(args);
		}
		this.inherited("createResourceView", arguments);
    },
    
	expand: function() {
		// FIXME
		//this.controller.allowedCategories = [{label: "Advanced"}];
		//this.viewer._started = false;
		//this.viewer.startup();
    }
});

dojo.declare("rmps.app.ui.internal.viewers.FilteredPropertiesController", [rmps.ui.properties.PropertiesController], {
	allowedCategories : [],
	
	isFilteredCategory: function(category) {
		for (var i = 0; i < this.allowedCategories.length; ++i) {
			if(this.allowedCategories[i].label == category.label) {
				return true;
			}
		}
		return false;
	},
	
	_constructSections: function(data) {	
		for (var i = 0; i < data.categories.length; ++i) {
			var category = data.categories[i];
			if (this.isFilteredCategory(category)) {
				var page = this._createPage();
				page.title = category.label;
				
				for (var j = 0; j < category.sections.length; ++j) {
					var section = category.sections[j];
					
					jazz.core.loader.load_sync(section.widget);
					
					var widgetConstructor = dojo.getObject(section.widget);
					var args = section.args || section.mixin;

					if (args.templateString) {
						args.templatePath = null;
					}
					
					var sectionWidget = new widgetConstructor(args);
					page.addSection(sectionWidget);
				}
				
				this._view.addPage(page);
			}
		}
	}
});	

dojo.declare("rmps.app.ui.internal.viewers.DiagramInfo", [dijit._Widget, dijit._Templated], {
	
	templateString: '<div>' +
			'<table><tr><td nowrap="nowrap"><span>Long Name: </span></td><td nowrap="nowrap"><span dojoAttachPoint="longNameNode" /></td></tr>' +
			'<tr><td nowrap="nowrap"><span>Type: </span></td><td nowrap="nowrap"><span dojoAttachPoint="typeNode" /></td></tr>' +
			'<tr><td nowrap="nowrap"><span>Modified: </span></td><td nowrap="nowrap"><span dojoAttachPoint="modifiedNode" /></td></tr>' +
			// '<tr><td nowrap="nowrap"><span>Comments: </span></td><td nowrap="nowrap"><span dojoAttachPoint="comentsNode" /></td></tr>' + // FIXME
			'</table><div id="diagramnode" class=\"rmps-graphics-diagram-layout\" dojoAttachPoint="diagramNode"></div>'+
			'</div>',
	
	viewer: null,
	resourceInfo: null,
	rendering: null,
	args: null,
	diagramContainer: null,
	
	postCreate: function() {
		if(this.resourceInfo) {
			this._processResourceInfo(this.resourceInfo);
		}
	}, 
	
	_processResourceInfo: function(resourceInfo) {
		this.resourceInfo = resourceInfo;
		if (resourceInfo.longName) {
			this.longNameNode.appendChild(dojo.doc.createTextNode(" " + resourceInfo.longName));
		}
		if (resourceInfo.displayType) {
			this.typeNode.appendChild(dojo.doc.createTextNode(" " + resourceInfo.displayType));
		}
		if (resourceInfo.modificationTime) {
			var date = dojo.date.locale.format(this.resourceInfo.modificationTime, {formatLength: "medium"});
			this.modifiedNode.appendChild(dojo.doc.createTextNode(" " + date));
		}
		
		
		var mb = dojo.marginBox( this.diagramNode );
		this.diagramContainer = document.createElement( "div" );
		this.diagramContainer.style.width = "100%";
		this.diagramContainer.style.height = "215px"; // FIXME temp
		this.diagramContainer.style.background = "white";
		this.diagramContainer.style.overflow = "hidden";
		this.diagramContainer.style.padding = "0px";
		
		var diagramnode = document.createElement( "div" );
	    this.viewer = new rmps.graphics.diagram.renderer.Diagram( diagramnode, resourceInfo.uri );
      	this.diagramContainer.appendChild( diagramnode );
        this.diagramNode.appendChild( this.diagramContainer );
		this.viewer.resize({w: this.diagramContainer.style.width, h: this.diagramContainer.style.height });
	}, 
	
	expand: function() {
		this._resize();
    },
	
	_resize: function() {
		if (this.viewer && this.viewer.resize) {
			var tableDiv = this.domNode.childNodes[0];
			var parent = this.domNode.offsetParent;
			if(parent && tableDiv) {
				var height = parent.offsetHeight - tableDiv.offsetHeight;
				var width = parent.offsetWidth;
				this.diagramContainer.style.height = height +"px";
				this.viewer.resize({w: width, h: height });
			}
			
		}
	},
	
	destroy: function() {
		delete this.viewer;
		delete this.resourceInfo;
		delete this.rendering;
		delete this.args;
	}
	
});

})();