/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2011.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.review.app.ui.internal.viewers.ReviewRendering");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("rmps.core.review.ReviewClient");
dojo.require("dm.review.app.ui.internal.viewers.ReviewRenderingViewer");
dojo.require("rmps.app.ui.internal.util.UrlUtil");
dojo.require("rmps.core.review.util.ReviewUtil");
dojo.require("rmps.core.review.util.ReviewTypes");
dojo.require("dojo.date.locale");

/*######################################### */
/*These imports are required to invoke auth dialogs, and
 * users' credentials checking utils. Otherwise, the users,
 * will not be autheroized when trying to search for an element
 * even if they are already logged in */
dojo.require("jazz.app.proxy");
dojo.require("jazz.app.auth");
dojo.require("jazz.app.events");
/*######################################### */


var ReviewClient= rmps.core.review.ReviewClient;
var UrlUtil= rmps.app.ui.internal.util.UrlUtil;
var ReviewUtil= rmps.core.review.util.ReviewUtil;
var ReviewTypes= rmps.core.review.util.ReviewTypes;

/**
 * Review Element Compact Rendering
 *
 */
dojo.declare("dm.review.app.ui.internal.viewers.ReviewRendering", [dijit._Widget, dijit._Templated], {

	templateString: '<div style="width: 100%; overflow: hidden; padding: 0px; position: relative;" '+
					'id="containerNode" class="rmps-graphics-resource-container" ' +
					'dojoAttachPoint="containerNode"></div>',

	resourceUri: null,
	currentUser: null,
	viewer: null,

	constructor: function(mixin) {
		if (mixin) {
			dojo.mixin(this, mixin);
		}
	},

    postCreate: function() {
    	this.inherited(arguments);
		this._view();
	},

	_view: function() {
		var paramters= UrlUtil.extractParameter();
		this.currentUser= paramters.currentUser;
		this.resourceUri= paramters.resourceUri;
		if(this.resourceUri) {
			var deferred = ReviewClient.getReview(this.resourceUri, this.currentUser);
			deferred.addCallback(this, function(review) {
				this.render(review);
			});
			deferred.addErrback(this, function(error) {
				rmps.core.error.createError(error).dispatch();
			});
		}else{
			this.render(UrlUtil.extractParameter());
		}
    },

	render: function(review) {
		var title= review.getReviewAttribute("reviewName");
		var state= review.getReviewAttribute("reviewStatus");
		var dueDate= review.getReviewAttribute("reviewDueDate");
		var moderator= review.getReviewAttribute("owner");
		
		var lastModified= review.lastModified;
		var participant= review.getParticipant(review.myUserURI);
		var participantRole= participant?participant.reviewRole: null;
		
		var role= moderator=== review.myUserURI?ReviewTypes.getParticipantRoleTypeDisplayName(ReviewTypes.PARTICIPANT_ROLE_TYPES.AUTHOR):
									ReviewTypes.getParticipantRoleTypeDisplayName(participantRole);
		
		this.viewer= new dm.review.app.ui.internal.viewers.ReviewRenderingViewer({
			title: title,
			state: state,
			dueDate: dueDate,
			moderator: moderator,
			lastModified: lastModified,
			role: role
		});
		this.containerNode.appendChild(this.viewer.domNode);
		this.viewer.startup();

	},

   	destroy: function() {
		this.inherited(arguments);
		if(this.viewer){
			this.viewer.destroyRecursive();
		}
		delete this.viewer;
	}
});


})();