/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.xtools.rmps.setup.ui.internal.wizard.FinalizeStep");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.Button");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.ui.StyledBox");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");
dojo.require("jazz.client.xhr");
dojo.require("net.jazz.ajax.contextRoot");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.xtools.rmps.setup.ui.internal.wizard", "FinalizeStepMessages");

(function () {
	var messages;
	var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
	var WizardStep = jazz.ui.wizard.WizardStep;
	var TrimStyles = jazz.ui.StyledBox.TrimStyles;
	var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;

    dojo.declare("com.ibm.xtools.rmps.setup.ui.internal.wizard.FinalizeStep", [WizardStep, dijit._Widget, dijit._Templated], {

		templatePath: dojo.moduleUrl("com.ibm.xtools.rmps.setup", "ui/internal/wizard/templates/FinalizeStep.html"),
		widgetsInTemplate: true,
		innerBoxTrim:  TrimStyles.EDITOR,
	
		constructor: function () {
			this.messages = messages = dojo.i18n.getLocalization("com.ibm.xtools.rmps.setup.ui.internal.wizard", "FinalizeStepMessages");
			this.stepTitle = messages.stepTitle;
		},

		postCreate: function () {
			this.header = new WizardStepHeader({stepTitle: this.stepTitle});
			this.stepHeaderArea.appendChild(this.header.domNode);
		
			//Set StyledBox titles
			this.step1_header.innerHTML = "<span class=\"step\">" + this.messages.step1Label + "</span><span>" + this.messages.step1Title + "</span>";
			this.step2_header.innerHTML = "<span class=\"step\">" + this.messages.step2Label + "</span><span>" + this.messages.step2Title + "</span>";
		
			//Prepare for raw XHR requests
			this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		
			//Could improve this to be more robust...
			this.servicePrefix = net.jazz.ajax.contextRoot() + "/service/";
		
			//Controls whether or not the step can go next
			this._setCompletedState(false);
		
			this.hasProcessed = false;
			this.startButton.disabled = true;
		
			//Create DOM for the individual finalize operations
			this.domFinalize = this._addOperationDiv(this.messages.operation1title, this.messages.operation1desc);
			this.domUserSync = this._addOperationDiv(this.messages.operation2title, this.messages.operation2desc);
			this.domApplyLicense = this._addOperationDiv(this.messages.finalizeApplyLicense, this.messages.finalizeApplyLicenseDescription);
			this.domImportProcessTemplate = this._addOperationDiv(this.messages.finalizeImportProcessTemplate, this.messages.finalizeImportProcessTemplateDescription);
			this.domConfigureVVC = this._addOperationDiv(this.messages.finalizeConfigureVVC, this.messages.finalizeConfigureVVCDescription);
			this.domProvision = this._addOperationDiv(this.messages.finalizeProvision, this.messages.finalizeProvisionDescription);
			this.domStorageAreasCreation = this._addOperationDiv(this.messages.finalizeStorageAreasCreation, this.messages.finalizeStorageAreasCreationDescription);
			this.domProjectAreasCreationCore = this._addOperationDiv(this.messages.projectAreasCreationCore, this.messages.projectAreasCreationCoreDescription);
			this.domProjectAreasCreationOther = this._addOperationDiv(this.messages.projectAreasCreationOther, this.messages.projectAreasCreationOtherDescription);
			this.domProjectAreasCreationUser = this._addOperationDiv(this.messages.projectAreasCreationUser, this.messages.projectAreasCreationUserDescription);
			this.domDeploySystemLinkTypes = this._addOperationDiv(this.messages.deploySystemLinkTypes, this.messages.deploySystemLinkTypesDescription);
			this.domDiscoverRenderingServers = this._addOperationDiv(this.messages.discoverRenderingServers, this.messages.discoverRenderingServersDescription);

			this.connect(this.startButton, "onclick", "start");
		},
	
		focusStep: function () {
			var self = this;
			var srh = new ServiceResponseHandler(self, "setJTSUrl", "handleAppNotRegistered");
			ServerStatusClient.getConfiguredJazzTeamServerURI(srh);
			var isPopupBlockerEnabled = this.detectPopupBlocker();
			if (isPopupBlockerEnabled) {
			    // A popup blocker was detected, we cannot continue
				self.header.showMessage({type: "ERROR", summary: self.messages.loginPopupBlocked });
				self.nowSetCompletedState(false);
			}
		},
	
		setJTSUrl: function (response) {
			this.configuredJTS = response;
			this.startButton.disabled = false;
			this.triggerButtonStateUpdate();
		},
	
		handleAppNotRegistered: function (error) {
			this.header.showMessage({
				errorObject: error, 
				summary: this.messages.appNotRegistered 
			});
		},
	
		//STEP 1: REGISTRATION HANDLER, FINALIZE
		start: function (e) {
			dojo.stopEvent(e);
			this.startButton.disabled = true;
			if (!this.hasProcessed) {	
				var self = this;
			
				//Show Pending Message
				this.header.showMessage({
					type: "INFO",
					summary: this.messages.finalizePending
				});	
				this.triggerButtonStateUpdate();
			
				//Create DOM to display the progress
				var domNode = this.domFinalize;
				dojo.addClass(domNode, "loading");
				var dFinalize = this.finalize();
				dFinalize.addCallback(function (response) {
					self.hasProcessed = true;
					dojo.removeClass(domNode, "loading");
					dojo.addClass(domNode, "ok");
					self.userSynchonization();
				});
				dFinalize.addErrback(function (errObject) {
					self._showInlineError(domNode, self.messages.finalizeError, errObject);
					self.startButton.disabled = false;
				});
			}
		},
	
		showCompletedMessage: function () {
			var buttonLabel = this.useFinishButton() === true ? this.messages.finish : this.messages.next;
			this.header.showMessage({
				type: "OK",
				summary: dojo.string.substitute(this.messages.allComplete, [ buttonLabel ])
			});	
			this._setCompletedState(true);
		},
	
		finalize: function () {
			var deferred = new dojo.Deferred();
			jazz.client.rawXhrPost({
				url: net.jazz.ajax._contextRoot + '/configure?action=finalize',
				headers: this.headers,
				load: function (response) {
					deferred.callback(response);
				},
				error: function (errObject) {
					deferred.errback(errObject);		
				}
			});	
			return deferred;
		},
	
		_showInlineError: function (node, message, errObject) {
			dojo.addClass(node, "error");
			var errorDiv = node.getElementsByTagName("div");
			dojo.removeClass(errorDiv[0], "noDisplay");
			var errorObj = this._getErrorObject(errObject);
			node._errorMsgBox.showMessage({
				errorObject: errorObj,
				summary: message
			});
			this._setCompletedState(false);
		},
	
		_showInlineWarning: function (node, message, errObject) {
			var warningDiv = node.getElementsByTagName("div");
			dojo.removeClass(warningDiv[0], "noDisplay");
			node._errorMsgBox.showMessage({
				type: "Warning",
				summary: message
			});
		},
		
		_getErrorObject: function (response) {
			var errorObj = {};
			if (response && response.responseText) {
				//Code from TeamServerClient.js......
				try {
					var responseObj = dojo.fromJson(response.responseText);
					dojo.mixin(errorObj, {
						errorClass: responseObj.errorClass,
						message: responseObj.errorMessage,
						exception: responseObj.errorTrace,
						errorData: responseObj.errorData,
						nestedError: responseObj.nestedError
					});
				} 
				catch (/* Error */ e) {
					// for non-JSON responses, include it as a message
					errorObj.message = response.responseText;
				}
			}
			return errorObj;
		},
	
		//Creates a new DIV element for the operation and appends it to the DOM
		//Returns: a handle to the DIV element
		_addOperationDiv: function (title, description, customClass) {
			var div = dojo.create("div", { className: "operation"}, this.operations);
			if (customClass) {
				dojo.addClass(div, customClass);
			}
			dojo.create("h3", { className: "operationTitle", innerHTML: title}, div);
			dojo.create("p", { className: "operationDesc", innerHTML: description}, div);
			var errDiv = dojo.create("div", { className: "operationError noDisplay"}, div);
			div._errorMsgBox = new jazz.ui.MessageBox({
				onToggle: dojo.hitch(this, this._handleMsgBoxToggle)
			});
			errDiv.appendChild(div._errorMsgBox.domNode);
			return div;
		},
	
		_handleMsgBoxToggle: function () {
			this.triggerButtonStateUpdate();
		},
	
		_setCompletedState: function (state) {
			this.isCompleted = state;
			this.triggerButtonStateUpdate();
		},
	
		customOnNext: function () {
		},
	
		customOnFinish: function () {
			this.customOnNext();
		},

		canFlipToNextPage: function () {
			return this.isCompleted;
		},
	
		useFinishButton: function () {
			return !this.isEmbedded && this.finalStep();
		},
	
		canFinish: function () {
			return this.isCompleted && this.useFinishButton();
		},

		finalStep: function () {
			// This wizard step will never be the final step.
			return false;
		},
	
		nowSetCompletedState: function (state) {
			this.isCompleted = state;
			this.triggerButtonStateUpdate();
		},
	
		nowShowCompletedMessage: function () {
			var buttonLabel = this.useFinishButton() === true ? this.messages.finish : this.messages.next;
			this.header.showMessage({
				type: "OK",
				summary: dojo.string.substitute(this.messages.allComplete, [ buttonLabel ])
			});
			this.nowSetCompletedState(true);
		},
	
		//STEP 2: USER SYNCHRONIZATION
		userSynchonization: function () {
			var self = this;
			var domNode = this.domUserSync;
			var deferred = new dojo.Deferred();
			dojo.addClass(domNode, "loading");
			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/configure?action=usersync',
				headers: this.headers,
				load: function (response) {
					deferred.callback(response);
				},
				error: function (errObject) {
					deferred.errback(errObject);
				}
			});	
			deferred.addCallback(function (response) {
				dojo.removeClass(domNode, "loading");
				dojo.addClass(domNode, "ok");
				self.applyInternalLicense();
			});
			deferred.addErrback(function (errObject) {
				self._showInlineError(domNode, self.messages.userSyncError, errObject);
			});
		},
	
		//STEP 3: APPLY INTERNAL LICENSE
		applyInternalLicense: function () {
			var self = this;
			var domNode = this.domApplyLicense;
			var deferred = new dojo.Deferred();
			dojo.addClass(domNode, "loading");
			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/configure?action=applyLicense',
				headers: {
					"X-ibm-rmps-internal": "true"
				},
				load: function (response) {
					deferred.callback(response);
				},
				error: function (errObject) {
					deferred.errback(errObject);
				}
			});	
			deferred.addCallback(function (response) {
				dojo.removeClass(domNode, "loading");
				dojo.addClass(domNode, "ok");
				self.importProcessTemplate();
			});
			deferred.addErrback(function (errObject) {
				self._showInlineError(domNode, self.messages.licenseError, errObject);
			});
		},
	
		//STEP 4: IMPORT PROCESS TEMPLATES
		importProcessTemplate: function () {
			var self = this;
			var domNode = this.domImportProcessTemplate;
			var deferred = new dojo.Deferred();
			dojo.addClass(domNode, "loading");
			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/configure?action=importProcessTemplates',
				headers: {
					"X-ibm-rmps-internal": "true"
				},
				load: function (response) {
					deferred.callback(response);
				},
				error: function (errObject) {
					deferred.errback(errObject);
				}
			});	
			deferred.addCallback(function (response) {
				dojo.removeClass(domNode, "loading");
				dojo.addClass(domNode, "ok");
				self.configureVVC();
			});
			deferred.addErrback(function (errObject) {
				self._showInlineError(domNode, self.messages.operationError, errObject);
			});
		},
	
		//STEP 5: CONFIGURE VVC
		configureVVC: function () {
			var self = this;
			var domNode = this.domConfigureVVC;
			var deferred = new dojo.Deferred();
			dojo.addClass(domNode, "loading");
			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/configure?action=configureVVC',
				headers: {
					"X-ibm-rmps-internal": "true"
				},
				load: function (response) {
					deferred.callback(response);
				},
				error: function (errObject) {
					deferred.errback(errObject);
				}
			});	
			deferred.addCallback(function (response) {
				dojo.removeClass(domNode, "loading");
				dojo.addClass(domNode, "ok");
				self.provision();
			});
			deferred.addErrback(function (errObject) {
				self._showInlineError(domNode, self.messages.operationError, errObject);
			});
		},
	
		//STEP 6: PROVISION DM APPLICATION
		provision: function () {
			var self = this;
			var domNode = this.domProvision;
			var deferred = new dojo.Deferred();
			dojo.addClass(domNode, "loading");
			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/configure?action=provision',
				headers: {
					"X-ibm-rmps-internal": "true"
				},
				load: function (response) {
					deferred.callback(response);
				},
				error: function (errObject) {
					deferred.errback(errObject);
				}
			});	
			deferred.addCallback(function (response) {
				dojo.removeClass(domNode, "loading");
				dojo.addClass(domNode, "ok");
				self.storageAreasCreation();
			});
			deferred.addErrback(function (errObject) {
				self._showInlineError(domNode, self.messages.operationError, errObject);
			});
		},
	
		//STEP 7: CREATE STORAGE AREAS
		storageAreasCreation: function () {
			var self = this;
			var domNode = this.domStorageAreasCreation;
			var deferred = new dojo.Deferred();
			dojo.addClass(domNode, "loading");
			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/dminstall?recreate=false',
				headers: {
					"X-ibm-rmps-internal": "true"
				},
				load: function (response) {
					deferred.callback(response);
				},
				error: function (errObject) {
					deferred.errback(errObject);
				}
			});	
			deferred.addCallback(function (response) {
				dojo.removeClass(domNode, "loading");
				dojo.addClass(domNode, "ok");
				self.projectAreasCreationCore();
			});
			deferred.addErrback(function (errObject) {
				self._showInlineError(domNode, self.messages.operationError, errObject);
			});
		},
	
		//STEP 8: CREATE CORE PROJECT AREAS
		projectAreasCreationCore: function () {
			var self = this;
			var domNode = this.domProjectAreasCreationCore;
			var deferred = new dojo.Deferred();
			dojo.addClass(domNode, "loading");
			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/dtkConfigure?action=installCoreSystemDomain',
				headers: {
					"X-ibm-rmps-internal": "true"
				},
				load: function (response) {
					deferred.callback(response);
				},
				error: function (errObject) {
					deferred.errback(errObject);
				}
			});	
			deferred.addCallback(function (response) {
				dojo.removeClass(domNode, "loading");
				dojo.addClass(domNode, "ok");
				self.projectAreasCreationOther();
			});
			deferred.addErrback(function (errObject) {
				self._showInlineError(domNode, self.messages.operationError, errObject);
			});
		},
	
		//STEP 9: CREATE OTHER PROJECT AREAS
		projectAreasCreationOther: function () {
			var self = this;
			var domNode = this.domProjectAreasCreationOther;
			var deferred = new dojo.Deferred();
			dojo.addClass(domNode, "loading");
			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/dtkConfigure?action=installOtherSystemDomains',
				headers: {
					"X-ibm-rmps-internal": "true"
				},
				load: function (response) {
					deferred.callback(response);
				},
				error: function (errObject) {
					deferred.errback(errObject);
				}
			});	
			deferred.addCallback(function (response) {
				dojo.removeClass(domNode, "loading");
				dojo.addClass(domNode, "ok");
				self.projectAreasCreationUser();
			});
			deferred.addErrback(function (errObject) {
				self._showInlineError(domNode, self.messages.operationError, errObject);
			});
		},
	
		//STEP 10: CREATE USER DEFINED PROJECT AREAS
		projectAreasCreationUser: function () {
			var self = this;
			var domNode = this.domProjectAreasCreationUser;
			var deferred = new dojo.Deferred();
			dojo.addClass(domNode, "loading");
			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/dtkConfigure?action=installUserDomain',
				headers: {
					"X-ibm-rmps-internal": "true"
				},
				load: function (response) {
					deferred.callback(response);
				},
				error: function (errObject) {
					deferred.errback(errObject);
				}
			});	
			deferred.addCallback(function (response) {
				dojo.removeClass(domNode, "loading");
				dojo.addClass(domNode, "ok");
				self.deploySystemLinkTypes();
			});
			deferred.addErrback(function (errObject) {
				self._showInlineError(domNode, self.messages.operationError, errObject);
			});
		},
	
		//STEP 11: DEPLOY SYSTEM LINK TYPES
		deploySystemLinkTypes: function () {
			var self = this;
			var domNode = this.domDeploySystemLinkTypes;
			var deferred = new dojo.Deferred();
			dojo.addClass(domNode, "loading");
			jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/dtkConfigure?action=deploySystemLinkTypes',
				headers: {
					"X-ibm-rmps-internal": "true"
				},
				load: function (response) {
					deferred.callback(response);
				},
				error: function (errObject) {
					deferred.errback(errObject);
				}
			});	
			deferred.addCallback(function (response) {
				dojo.removeClass(domNode, "loading");
				dojo.addClass(domNode, "ok");
				self.discoverRenderingServers();
			});
			deferred.addErrback(function (errObject) {
				self._showInlineError(domNode, self.messages.operationError, errObject);
			});
		},
		
		//STEP 12: Discover Rendering Servers
		discoverRenderingServers: function() {
			var self = this;
			var domNode = this.domDiscoverRenderingServers;
			var deferred = new dojo.Deferred();
			dojo.addClass(domNode, "loading");
			var def = jazz.client.xhrPost({
				url: net.jazz.ajax._contextRoot + '/dtkConfigure?action=discoverRenderingServers',
				headers: {
					"X-ibm-rmps-internal": "true"
				},
				handleAs: "text",
				load: function (response, ioargs) {
					deferred.callback(response);
				},
				error: function (errObject) {
					deferred.errback(errObject);
				}
			});	
			deferred.addCallback(function (textResponse) {
				dojo.removeClass(domNode, "loading");
				dojo.addClass(domNode, "ok");
				if (textResponse === "FAILURE") {
					// No servers found - inform the user, but do not break the setup.
					self._showInlineWarning(domNode, self.messages.discoverRenderingServersWarning);
				}
				
				self.nowShowCompletedMessage();
			});
			deferred.addErrback(function (errObject) {
				self._showInlineError(domNode, self.messages.operationError, errObject);
			});
		},
	
		detectPopupBlocker: function () {
		    // return true if there is a popup blocker enabled for this server
			/*global window */
			var myTestWindow = window.open("about:blank", "", "directories=no,height=100,width=100,menubar=no,resizable=no,scrollbars=no,status=no,titlebar=no,top=0,location=no");
			var isPopupBlockerEnabled = true;
			if (myTestWindow) {
				myTestWindow.close();
				isPopupBlockerEnabled = false;
			}
			return isPopupBlockerEnabled;
		}

	});

})();
