/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.xtools.rmps.setup.ui.internal.wizard.WizardApplication");
dojo.require("net.jazz.ajax.ui._Application");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("jazz.ui.Wizard");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.core.RegistryFactory");

dojo.require("com.ibm.team.repository.web.client.internal.AdminClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dojo.i18n");
dojo.require("com.ibm.xtools.rmps.setup.ui.internal.wizard.WizardUtil");

/*######################################### */
/*These imports are required to invoke auth dialogs*/
dojo.require("jazz.app.proxy");
dojo.require("jazz.app.auth");
dojo.require("jazz.app.events");
/*######################################### */

dojo.requireLocalization("com.ibm.xtools.rmps.setup.ui.internal.wizard", "WizardApplicationMessages");

var AdminClient = com.ibm.team.repository.web.client.internal.AdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var WizardUtil = com.ibm.xtools.rmps.setup.ui.internal.wizard.WizardUtil;
var Wizard = jazz.ui.Wizard;
var WizardStep = jazz.ui.wizard.WizardStep;
var TrimStyles = jazz.ui.StyledBox.TrimStyles;

dojo.declare("com.ibm.xtools.rmps.setup.ui.internal.wizard.WizardApplication", net.jazz.ajax.ui._Application, {
	innerBoxTrim:  TrimStyles.EDITOR_YELLOW,

	steps: null,

	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.xtools.rmps.setup.ui.internal.wizard", "WizardApplicationMessages");
		this.steps = [];
		var extensions= jazz.core.RegistryFactory.getRegistry().getExtensionPoint("com.ibm.xtools.rmps.setup.setupWizard").getExtensions();
		
		if(!dojo.isArray(extensions)){
			return;
		}
		
		/* each step in a setupWizard extension has an order attribute and jsModule attribute */ 
		var wizardStep = [], modulesToLoad = [];
		for(var index=0; index<extensions.length; index++){
			var extension= extensions[index];
			var _elements= extension.getConfigurationElements();
			for (var i = 0; i < _elements.length; i++) {
				if ("step" == _elements[i].getName()) {
					/* insert the jsModule into the array as defined by the order attribute */
					var jsModule = _elements[i].getAttribute("jsModule");
					var order = _elements[i].getAttribute("order");
					modulesToLoad[order - 1] = jsModule;
					wizardStep[order - 1] = jsModule;
					jazz.core.loader.load_sync(jsModule);
					classConstructor = dojo.getObject(jsModule);
					if (classConstructor) {
						this.steps.push(classConstructor);
					}
				}
			}
		}

	},

	postCreate: function() {
		this.getUserRoles();
	},
	
	getUserRoles: function () {
		var srh= new ServiceResponseHandler(this, "authRequestPassed", "authRequestFailed");		
		AdminClient.getUserRoles(srh);
	},
	
	authRequestPassed: function(response) {
		if (response.userId) {
			this.loggedInUser = response.userId;
			dojo.setObject("com.ibm.team.repository.web.client.internal.AUTHENTICATED_USERID", response.userId);
		}
		var roles = response.roles;
		for (var i=0; i < roles.length; i++) {
			if (roles[i] === "JazzAdmins") {
				this.authorized = true;
				break;
			}
		}
		this._startApp();
	},
	
	authRequestFailed: function() {
		this._startApp();
	},
	
	_startApp: function () {
		document.body.style.background = "none";
		this._createUI();
		this.setName(this.messages.setupName);
		this.start();
	},
	
	_createUI: function() {				
		this.content = dojo.create("div");		
		if (!WizardUtil.isEmbedded()) {
			var textHeader = document.createElement("div");
			textHeader.innerHTML= this.messages.setupHeading;
			textHeader.className = "setup-header";
			this.content.appendChild(textHeader);
		}

		var wizard = new Wizard({
			fullPageContainer: this.content,
			steps: this.initServerSetupSteps(),
			stepListLayout: "vertical"
		});
		
		net.jazz.ajax.ui.PlatformUI.getWorkbench().rootNode().appendChild(this.content);
		
		/*This is a safe check in order to avoid the wizard UI failure if error occured */
		if(wizard.contentWidget){
			var contentWidget= wizard.contentWidget;
			if(dojo.isFunction(contentWidget.updateNavigationControls)){
				contentWidget.updateNavigationControls();				
			}
		}
	},

	initServerSetupSteps: function () {
		var steps= [];
		for(var index=0; index<this.steps.length; index++){
			steps.push(new this.steps[index]());
		}
		return steps;
	}
});

})();