/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
dojo.provide("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenter.VvcNotificationCenter");

dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenter.VvcNotificationCenterMessageContent");
dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationObject");
dojo.require("com.ibm.team.vvc.shared.web.ui.gc.VvcGarbageCollector");
dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.VvcNotificationConstants");
var ncConstants = com.ibm.team.vvc.shared.web.ui.notifications.VvcNotificationConstants;

var activationChannel = ".activationChannel"

dojo.require("jazz.ui.MessageBox");

dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenter", "VvcNotificationCenterMessages");

dojo.declare("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenter.VvcNotificationCenter",  [dijit._Widget, dijit._Templated], {
			
	templatePath: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'ui/notifications/notificationCenter/templates/VvcNotificationCenter.html'),
	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenter", "VvcNotificationCenterMessages"),
				
	// garbage collector
	gc: null,
	
	channels: null,
	
	messageBox: null,
	messageContent: null,
	pagination: null,
	isHidden: null,
	
	isActive: null,
	activationList: null,
		
	constructor: function(args){
		this.gc = new com.ibm.team.vvc.shared.web.ui.gc.VvcGarbageCollector();
		
		this.isActive = false;
		this.activationList = [];
		this.channels = [];
		
		if(args){
			if(args.channels){
				// Expects an array
				this.addChannels(args.channels);
			}
		}
		
		for(var channel in this.activationList){
			dojo.subscribe(channel, dojo.hitch(this, function(message){
				if(this.isActive && message.activate && !(message.activate === this.id)){
					/*
					 *  Do not call deactivate, this will wake up any other NC's that are waiting
					 *  for this NC to deactivate and potentially cause a race condition.
					 */
					this.isActive = false;
					this.activationList[channel].unshift(message.activate);
				} else if (message.deactivate && this.activationList[channel][0] === message.deactivate){
					this.activate();
				}
			}));
		}
	},
	
	
	destroy: function(){
		this.deactivate();
		this.gc.destroy();
	},
	
	addChannel: function(channel){
		this.gc.disconnects(dojo.subscribe(channel, dojo.hitch(this, this.addNotification)));
		this.channels.push(channel);
		this.activationList[channel + activationChannel] = [];
	},
	
	addChannels: function(channels){
		dojo.forEach(channels, dojo.hitch(this, this.addChannel));
	},
	
	addNotification: function(notification){
		if(this.isActive){
			this.showNotification(notification);
			
			notification.addOnClose(dojo.hitch(this, function(){
				if(this.messageContent){
					var notifications = this.messageContent.getNotifications();
					var index = dojo.indexOf(notifications, notification);
					if(index || index === 0){
						this.messageContent.hideNotification(index);
					}
				}
			}));
		}
	},
	
	showNotification: function(notification){		
		if(!this.messageContent){
			this.messageContent = new com.ibm.team.vvc.shared.web.ui.notifications.notificationCenter.VvcNotificationCenterMessageContent();
			var onResizeEventHandle = dojo.connect(this.messageContent, "onSizeChanged", dojo.hitch(this, this.onSizeChanged));
			var hideNotificationHandle = dojo.connect(this.messageContent, "hideNotification", dojo.hitch(this, function(){
				if(this.messageContent && this.messageContent.getLength() <= 0){
					this.messageBox.hideMessage();
					dojo.destroy(this.messageContent);
					dojo.destroy(this.messageBox);
					this.messageBox = null;
					this.messageContent = null;
					this.isVisible = false;
					this.onHideNotificationArea();
					dojo.disconnect(onResizeEventHandle);
					dojo.disconnect(hideNotificationHandle);
				}
			}));
		}
										
		if(!this.messageBox){
			this.messageBox = new jazz.ui.MessageBox();
			this.vvcNotificationCenterNode.appendChild(this.messageBox.domNode);
			this.isVisible = false;
		}
						
		if(!this.isVisible){
			this.messageBox.showMessage({
				type: notification.type,
				summary: this.messageContent.domNode
			});
			this.isVisible = true;
			this.onShowNotificationArea();
		}
		this.messageContent.addNotification(notification);
	},
	
	/**
	 * Event called when the notification area becomes visible.
	 */
	onShowNotificationArea: function(){
		this.onSizeChanged();
	},
	
	/**
	 * Event called when the notification area becomes hidden.
	 */
	onHideNotificationArea: function(){
		this.onSizeChanged();
	},
	
	/**
	 * Event called when the notification area changes size for whatever reason
	 */
	onSizeChanged: function(){
		// Stub
	},
	
	activate: function(){
		this.isActive = true;
		for(var channel in this.activationList){
			dojo.publish(channel, {
				activate: this.id
			});
		}
	},
	
	deactivate: function(){
		this.isActive = false;
		for(var channel in this.activationList){
			dojo.publish(channel, {
				deactivate: this.id
			});
		}
	}
		
}); // end declare	
})(); // end object