/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
	dojo.provide("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenter.VvcNotificationCenterMessageContent");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationObject");
	dojo.require("com.ibm.team.vvc.shared.web.ui.gc.VvcGarbageCollector");
	dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.VvcNotificationConstants");
	var ncConstants = com.ibm.team.vvc.shared.web.ui.notifications.VvcNotificationConstants;
	
	dojo.require("jazz.ui.MessageBox");
	dojo.require("jazz.ui.PaginationControl");
		
	dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenter", "VvcNotificationCenterMessageContentMessages");
	
	dojo.declare("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenter.VvcNotificationCenterMessageContent",  [dijit._Widget, dijit._Templated], {
				
			templatePath: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'ui/notifications/notificationCenter/templates/VvcNotificationCenterMessageContent.html'),
			messages: dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenter", "VvcNotificationCenterMessageContentMessages"),
					
			// garbage collector
			gc: null,
			
			// FILO
			notifications: null,
			index: null,	// Index of the currently visible notification
			pagination: null,
						
			constructor: function(args){
				this.gc = new com.ibm.team.vvc.shared.web.ui.gc.VvcGarbageCollector();
				this.notifications = [];
				
				// Auto-size by default
				if(!args || args && args.autoSize != false){
					this.gc.disconnects(dojo.connect(this, "showNotification", this, function(){
						this.refreshNotificationWidth();
					}));
					this.gc.disconnects(dojo.connect(this, "onShowMore", this, function(){
						this.refreshNotificationWidth(true);	// Clear the width
					}));
					this.gc.disconnects(dojo.connect(this, "onShowLess", this, function(){
						setTimeout(dojo.hitch(this, this.refreshNotificationWidth), 50);
					}));
					this.gc.disconnects(dojo.connect(window, "onresize", this, function(){
						this.refreshNotificationWidth();
					}))
				}
			},
			
			postCreate: function(){
				this.pagination = new jazz.ui.PaginationControl({
					increment: 1,
					onNextClick: dojo.hitch(this, function(widget, index){
						// Ignore 'widget'
						this.showNotification(index - 1);
					}),
					onPreviousClick: dojo.hitch(this, function(widget, index){
						// Ignore 'widget'
						this.showNotification(index - 1);
					})
				});
				this.gc.destroys(this.pagination);
				this.paginationControlNode.appendChild(this.pagination.domNode);
			},
			
			destroy: function(){
				this.gc.destroy();
			},
			
			addNotification: function(notification){
				this.notifications.unshift(notification);
				this.pagination.setTotal(this.notifications.length);
				return this.showNotification(0);
			},
						
			hideNotification: function(index){
				if(!index && index !== 0 || isNaN(index)){
					return this.hideNotification(this.index);			
				}
				
				var length = this.getLength();
				if(index > length || length === 0){
					return length > 0 ? this.hideNotifications(length - 1) : null;
				}		
				
				var old = this.notifications.splice(index, 1);
				if(this.getLength() > 0){
					this.showNotification();
				} else {
					dojo.empty(this.notificationViewNode);
					this.onShowLess();
				}
				this.pagination.setTotal(--this.pagination.total);
				return old;
			},
			
			showNotification: function(index){
				var length = this.getLength();
				
				if((!index && index !== 0) || index > length){
					return length > 0 ? this.showNotification(length - 1) : null;
				}
				
				var notification = this.notifications[index];				
				dojo.empty(this.notificationViewNode);
				this.notificationViewNode.appendChild(notification.getView().domNode);
				
				/*
				 * Need to make sure we listen for the notifications onResize, but then
				 * we also want to make sure we only care about the notification that
				 * is currently visible.
				 */
				var handle = dojo.connect(notification.getView(), "onSizeChanged", dojo.hitch(this, this.onSizeChanged));
				dojo.connect(this, "hideNotification", dojo.hitch(this, function(indexToHide){
					if(index === indexToHide){
						dojo.disconnect(handle);
					}
				}))
				
				this.index = index;
				return notification;
			},
						
			onShowMore: function(){
				dojo.addClass(this.mode, "showMore");
				dojo.removeClass(this.mode, "showLess");
				this.onSizeChanged();
				this.showLessNode.focus();
			},
			
			onShowLess: function(){
				dojo.addClass(this.mode, "showLess");
				dojo.removeClass(this.mode, "showMore");
				this.onSizeChanged();
				this.showMoreNode.focus();
			},
			
			getLength: function(){
				return this.getNotifications().length;
			},
			
			getNotifications: function(){
				return this.notifications;
			},
			
			refreshNotificationWidth: function(/*optional, boolean*/clear){
				var width = null;
				if(!clear){
					var notificationPos = dojo.position(this.notificationViewNode);
					var navigationPos = dojo.position(this.navigationNode);
					width = (navigationPos.x - notificationPos.x - 15) + "px";
				}
				dojo.style(this.notificationViewNode, "width", width);
				this.onSizeChanged();
			},
			
			/**
			 * Event handler for when the notification changes size
			 */
			onSizeChanged: function(){
				// Stub
			}
						
	}); // end declare	

})(); // end object