/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

	dojo.provide("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController");
	dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenter.VvcNotificationCenter");
	dojo.require("com.ibm.team.vvc.shared.web.ui.gc.VvcGarbageCollector");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.VvcNotificationConstants");
	var ncConstants = com.ibm.team.vvc.shared.web.ui.notifications.VvcNotificationConstants;
	
	dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController", "NotificationCenterControllerMessages");

	dojo.declare("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController", [dijit._Widget, dijit._Templated], {
			
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'ui/notifications/notificationCenterController/templates/VvcNotificationCenterController.html'),
		messages: dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController", "NotificationCenterControllerMessages"),
		
		// Array of notifications
		notificationCenters: null,
		notificationFactory: null,
		
		// garbage collector
		gc: null,
		
		constructor: function(args){
			this.gc = new com.ibm.team.vvc.shared.web.ui.gc.VvcGarbageCollector();
			this.notificationCenters = [];
		},
		
		destroy: function(){
			this.deactivateNotificationCenters();
			this.gc.destroy();
		},
		
		postCreate: function(){
			this.createErrorCenter();
			this.createWarningCenter();
			this.createInfoCenter();
			this.createOkCenter();
			this.activateNotificationCenters();
		},
		
		createErrorCenter: function(){
			this._createNotificationCenter(this.errorNotificationCenterNode, {channels: [ncConstants.ERROR_CHANNEL]});
		},
		
		createWarningCenter: function(){
			this._createNotificationCenter(this.warningNotificationCenterNode, {channels: [ncConstants.WARNING_CHANNEL]});
		},
		
		createInfoCenter: function(){
			this._createNotificationCenter(this.infoNotificationCenterNode, {channels: [ncConstants.INFO_CHANNEL]});
		},
		
		createOkCenter: function(){
			this._createNotificationCenter(this.okNotificationCenterNode, {channels: [ncConstants.OK_CHANNEL]});
		},
		
		_createNotificationCenter: function(parentDomNode, args){
			var center = new com.ibm.team.vvc.shared.web.ui.notifications.notificationCenter.VvcNotificationCenter(args);
			this.notificationCenters.push(center);
			
			// Make sure we propagate resize events 
			this.gc.disconnects(dojo.connect(center, "onSizeChanged", dojo.hitch(this, function(){
				 this.onSizeChanged(center);
			})));
			
			this.gc.destroys(center);
			
			if(parentDomNode){
				dojo.removeClass(parentDomNode, "hidden");
				parentDomNode.appendChild(center.domNode);
			}
		},
		
		activateNotificationCenters: function(){
			for(var i = 0; i < this.notificationCenters.length; i++){
				var nc = this.notificationCenters[i];
				nc.activate();
			}
		},
		
		deactivateNotificationCenters: function(){
			for(var i = 0; i < this.notificationCenters.length; i++){
				var nc = this.notificationCenters[i];
				nc.deactivate();
			}
		},
		
		/**
		 * Event called when a notification centre changes size
		 */
		onSizeChanged: function(notificationCenter){
			// Stub
		}
		
	}); // end declare

})(); // end object