/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

dojo.provide("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory");
dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationObject");

dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.VvcNotificationConstants");
var ncConstants = com.ibm.team.vvc.shared.web.ui.notifications.VvcNotificationConstants;

com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory = {
		
		messages: dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.notifications.notifications", "VvcNotificationObjectMessages"),
				
		createError: function(args){
			args.type = ncConstants.ERROR;
			return this._createNotification(args, ncConstants.ERROR_CHANNEL);
		},
		
		/*
		 * Expects input as an object.  Object should have 'xhrError' and 'xhrArgs' 
		 * properties.  
		 * 
		 * @param xhrError The XHR error returned from the server 
		 * @param xhrArgs The args used to send the request
		 * 
		 */
		createXhrError: function(args){	
			
			var xhr = args.xhrError;
			
			if(args.ioArgs){
				this.getRdfParts(args);
			}
			
			// HTTP 400 is the default
			 var httpCode = xhr.status ? xhr.status : 400;
						
			args.type = ncConstants.ERROR;
			
			if(!args.title){
				args.title = this.getMessage(httpCode, "title");
			}
			
			if(!args.subtitle){
				args.subtitle = this.getMessage(httpCode, "subtitle");
			}
			
			if(!args.description){
				args.description = xhr.message ? xhr.message : this.getMessage(httpCode, "description");
			}
		
			if(!args.solution){
				args.solution = this.getMessage(httpCode, "solution");
			}
			
			if(!args.devel && djConfig.isDebug){
				// TODO: parse xhrArgs out of the xhrError - amooz (2013-03-13)
				var xhrArgs = args.xhrArgs ? this._arrayToString(args.xhrArgs) : "";
				args.devel = dojo.string.substitute(this.messages.xhrErrorDevel, {args: xhrArgs, response: xhr.responseText});
			}
						
			return this.createError(args);
		},
		
		getMessage: function(code, message){
			var msg = this.messages.httpCodes;
			if(msg[code] && msg[code][message]){
				return msg[code][message];
			}
			return msg["400"][message];
		},
		
		// Copied from PageUtil.js:
		getRdfParts: function(args){
			var ioArgs = args.ioArgs;
			
			if(!args.subtitle && ioArgs.xhr){
				if(ioArgs.xhr.status){
					args.subtitle = ioArgs.xhr.status + " " + ioArgs.xhr.statusText;
				}
			}
			
			var contentType = (ioArgs.xhr) ? ioArgs.xhr.getResponseHeader("Content-Type") : null;
			if (contentType && ioArgs.xhr.responseText && contentType.toLowerCase() === "application/rdf+xml") {
				var rdf = dojox.xml.DomParser.parse(ioArgs.xhr.responseText).getElementsByTagName("rdf:Description")[0];
				for (var i = 0; i < rdf.childNodes.length; ++i) {
					var childNode = rdf.childNodes[i];
					if (childNode.nodeType === 1 && childNode.childNodes.length > 0 && childNode.childNodes[0].nodeValue) {
						switch (String(childNode.nodeName)) {
							case 'err:errorStatus':
								if(!args.subtitle){
									args.subtitle = childNode.childNodes[0].nodeValue;
								}
								break;
							case 'err:errorMessage':
								if(!args.title){
									args.title = childNode.childNodes[0].nodeValue;
								}
								break;
							case 'err:errorSubStatus':
								if(!args.subtitle){
									args.subtitle = childNode.childNodes[0].nodeValue;
								}
								break;
//							case 'err:detailedMessage':
//								if(!args.description){
//									var value = childNode.childNodes[0].nodeValue;
//									args.description = [{message: value}];
//								}
//								break;
							case 'err:seeAlso':
								if(!args.seeAlso){
									args.seeAlso = childNode.childNodes[0].nodeValue;
								}
								break;
							case 'err:errorCause':
								if(!args.cause){
									args.cause = this._createErrorObject(childNode.getElementsByTagName("rdf:Description")[0]);
								}
								break;
						}
					}
				}
			}
		},
		
		createWarning: function(args){
			args.type = ncConstants.WARNING;
			return this._createNotification(args, ncConstants.WARNING_CHANNEL);
		},
		
		createInfo: function(args){
			args.type = ncConstants.INFO;
			return this._createNotification(args, ncConstants.INFO_CHANNEL);
		},
		
		createSuccess: function(args){
			args.type = ncConstants.OK;
			return this._createNotification(args, ncConstants.OK_CHANNEL);
		},
		
		_assertChannels: function(args, channel){
			// Expects an array
			if(!args.channels){
				args.channels = [];
			}
			
			// Able to disable default channels if you want to...
			if(!args.disableDefaultChannels && dojo.indexOf(args.channels, channel) < 0) {
				args.channels.push(channel);
			}
		},
		
		_createNotification: function(args, channel){
			// Assert the default channel
			this._assertChannels(args, channel);
			
			// Make sure debug info is only included when necessary
			if(args.devel && !djConfig.isDebug){
				delete args.devel;
			}
					
			var notification = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationObject(args);
			
			// Creates a notification which will close itself when some event happens within some context.
			if(args.closeOnEvent){
				var closeArgs = args.closeOnEvent;
				var handle = dojo.connect(closeArgs.context, closeArgs.event, closeArgs.context, function(){
					notification.close();
					dojo.disconnect(handle);
				});
			}
			
			return notification;
		},
		
		_arrayToString: function(obj){
			var first = true;
			var retVal = "";
			if(dojo.isObject(obj)){
				if(dojo.isFunction(obj)){
					retVal += "function";
				} else {
					for(key in obj){
						if(!first){
							retVal += "\n";
						}
						first = false;
						retVal += dojo.string.substitute("${keyStr} : ${valStr}", {keyStr:key, valStr: this._arrayToString(obj[key])});
					}
				}
			} else {
				return obj;
			}
			return retVal;
		}
};

})(); // end object