/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

dojo.provide("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationObject");
dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notificationViews.VvcNotificationObjectView");
dojo.require("com.ibm.team.vvc.shared.web.ui.gc.VvcGarbageCollector");
dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.VvcNotificationConstants");
var ncConstants = com.ibm.team.vvc.shared.web.ui.notifications.VvcNotificationConstants;

dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.notifications.notifications", "VvcNotificationObjectMessages");

dojo.declare("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationObject", [], {
		
	// Title of the notification
	title: null,
	
	// Secondary title
	subtitle: null,
	
	// Description, the information trying to be shown
	description: null,
	
	// Type of the notification, ERR, WARN, INFO, OK
	type: null,
	
	// If there is a cause
	cause: null,
	
	// Solution, if one is available
	solution: null,
	
	// More information, probably a canned message
	seeAlso: null,
	
	// Any information which would be useful for developers and help
	devel: null,
		
	// If the notification has been acknowledged
	acknowledged: null,
	
	// garbage collector
	gc: null,
	
	// Channel for the notification
	channels: null,
	
	// Used to attach renderings to
	view: null,
	
	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.notifications.notifications", "VvcNotificationObjectMessages"),
	
	constructor: function(args){
		dojo.safeMixin(this, args);
		this.gc = new com.ibm.team.vvc.shared.web.ui.gc.VvcGarbageCollector();
		this.acknowledged = false;
	},
	
	destroy: function(){
		this.gc.destroy();
	},
	
	/*
	 * Returns 'this' to allow chaining, ex:
	 * 	var error = VvcNotificationFactory.createError(args).dispatch();
	 * which will create and dispatch the error and set a local handle to the notification as well. 
	 */ 
	dispatch: function(){
		if(!this.title || dojo.trim(this.title) == ""){
			console.warn("Thwarted attempt to dispatch a notification without a title. Notifications must have at least a title!"); //Do not translate!
		} else {
			var self = this;
			dojo.forEach(this.channels, function(channel){
				dojo.publish(channel, [self]);
			});
		}
		return this;
	},
	
	show: function(){
		if(this.onShow){
			this.onShow();
		}
	},
	
	hide: function(){
		if(this.onHide){
			this.onHide();
		}
	},
	
	close: function(){
		this.hide();
		if(this.onClose){
			this.onClose();
		}
	},
	
	/* 
	 * Callbacks must be registered with the notification itself to prevent
	 * NPE's after the notification is destroyed.
	 */
	_addCallback: function(name, func){
		this.gc.disconnects(dojo.connect(this, name, func));
	},
	
	addOnShow: function(func){
		this._addCallback("onShow", func);
		return this;
	},
	
	addOnHide: function(func){
		this._addCallback("onHide", func);
		return this;
	},
	
	addOnAcknowledge: function(func){
		this._addCallback("onAcknowledge", func);
		return this;
	},
	
	addOnClose: function(func){
		this._addCallback("onClose", func);
		return this;
	},
	
	getView: function(){
		// Have to return a new object every time for IE9.  While this will cause a bunch of objects to accumulate in memory,
		// the gc will destroy them with the notification.
		if( (dojo.isIE <= 9) || !this.view){
			this.view = new com.ibm.team.vvc.shared.web.ui.notifications.notificationViews.VvcNotificationObjectView({
				"notification": this
			});
			this.gc.destroys(this.view);
		}
		return this.view;
	},
	
	// Callbacks, do not hook directly up to these! Use the 'addOn____' methods! 
	/**
	 * Called whenever the notification is made visible on the screen
	 */
	onShow: null,
	
	/**
	 * Called whenever the notification is moved off screen
	 */
	onHide: null,
	
	/**
	 * Called when the user acknowledges the notification
	 */
	onAcknowledge: null,
	
	/**
	 * Called when the user closes the notification
	 */
	onClose: null
	
}); // end declare

})(); // end object