/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.vvc.shared.core.ui.objects.ChangesetResource");

	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
	var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;
		
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.VvcObject");

	dojo.declare("com.ibm.team.vvc.shared.core.ui.objects.ChangesetResource",
			com.ibm.team.vvc.shared.core.ui.objects.VvcObject, {
		
		getFrontServiceId: function() {
			return "changesets";
		},
		
		getCreated: function() {
			return this.getProperty(Constants.CREATED);
		},

		getCreator: function() {
			return this.getProperty(Constants.CREATOR);
		},
		
		getState: function() {
			return this.getProperty(Constants.STATE);
		},
		
		getCompletionTime: function() {
			return this.getProperty(Constants.COMPLETION_TIME);
		},
		
		getTitle: function() {
			return this.getProperty(Constants.TITLE);
		},
		
		/*
		 * There are situations where we need to get the change set but the only piece of information
		 * we have as to the location of VVC is the configuration URI itself.  So we need to load the 
		 * configuration, then examine the response headers for the location of the change set, THEN
		 * we can load the change set.
		 */
		loadByConfiguration: function __loadByConfiguration(configuration) {			
			var getConfigDef = jazz.client.xhrGet({
				url: configuration.uri, 
				headers: {
					"Accept" : "application/json;application/rdf+json;text/rdf+json;text/json",
				},
				sync: true,
				handleAs: "json",
				load: dojo.hitch(this, function __load(response, ioArgs) {
					return ioArgs.xhr.getResponseHeader("X-ibm-vvc-changeset-location");				
				}),
				error: dojo.hitch(this, this.onError),
				failOk: true
			});
			
			getConfigDef.then(dojo.hitch(this, function __getConfigDefSuccess(changesetUri){
				var getChangesetDef = jazz.client.xhrGet({
					url: changesetUri,
					headers: {
						"Accept" : "application/json;application/rdf+json;text/rdf+json;text/json",
					},
					sync: true,
					handleAs: "json",
					load: dojo.hitch(this, function __load(response, ioArgs) {
						if (response) {
							for (var uri in response) {
								this.uri = uri;
								this.properties = response[uri];
								break;
							}
						}
						return this;			
					}),
					error: dojo.hitch(this, this.onError),
					failOk: true
				});
			}));			
		},
		
		onError: function(response, ioArgs) {
			alert("Failed to load resource '"+ioArgs.args.headers.vvc_servicePath+"'");
			console.log("Failed to load resource '"+ioArgs.args.headers.vvc_servicePath+"'...Invalid Response Status("+ioArgs.xhr.status+"): "+ response.message);
		},
	});
})();