/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.vvc.shared.core.ui.objects.CompareProvider");

	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
	var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.VvcObject");
	
	/**
	 * Provides information about the location of the compare-merge UI
	 * of an application (or some domain within an application).
	 */
	dojo.declare("com.ibm.team.vvc.shared.core.ui.objects.CompareProvider",
			com.ibm.team.vvc.shared.core.ui.objects.VvcObject, {		

		constructor: function(properties) {
			this.properties = properties;
		},
		
		getFrontServiceId: function() {
			return null;
		},

		/**
		 * Can this compare provider allow compare-merge functionality in web UI?
		 */
		canProvideWebFunctionality: function() {
			return this.getBooleanProperty(Constants.DELIVER_COMPARE_PROVIDER_CAN_PROVIDE_WEB_FUNCTIONALITY);
		},

		/**
		 * Gets the title of the compare provider. This will be shown in the Deliver Wizard step.
		 */
		getTitle: function() {
			return this.getProperty(Constants.TITLE);
		},

		/**
		 * Gets the web URL of the compare functionality.
		 */
		getCompareWebUri: function() {
			return this.getProperty(Constants.DELIVER_COMPARE_PROVIDER_COMPARE_WEB_URI);
		},
		
		/**
		 * Determines if the compare provider has a separate URL to
		 * use for merging functionality.
		 */
		hasSeparateMergeProvider: function() {
			return this.getBooleanProperty(Constants.DELIVER_COMPARE_PROVIDER_HAS_SEPARATE_MERGE_PROVIDER);
		},
		
		/**
		 * Gets a web URL for merging functionality. Only used if hasSeparateMergeProvider() is true.
		 */
		getMergeWebUri: function() {
			return this.getProperty(Constants.DELIVER_COMPARE_PROVIDER_MERGE_WEB_URI);
		},
		
		/**
		 * Get's the URI of the compare provider.
		 */
		getUri: function() {
			return this.getProperty(Constants.DELIVER_COMPARE_PROVIDER_URI);
		},
		
		/**
		 * Get's the ordering of the compare provider.
		 */
		getOrdering: function() {
			return this.getProperty(Constants.DELIVER_COMPARE_PROVIDER_ORDERING);
		},
		
		
	});
})();