/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");

	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
	var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.VvcObject");
	
	dojo.declare("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource",
			com.ibm.team.vvc.shared.core.ui.objects.VvcObject, {
		
		parentUri: null,
		parentName: null,
		title: null,
		permissions: null,
		permissionsLoaded: false,

		getFrontServiceId: function() {
			return "configurations";
		},

		getTitle: function() {
			if (!this.title) {
				this.title = this.getProperty(Constants.TITLE);
			}
			return this.title;
		},
		
		isTitleChanged: function() {
			if (this.title != null) {
				var name = this.getProperty(Constants.TITLE);
				if (this.title != name) {
					return true;
				}
			}
			return false;
		},

		getDescription: function() {
			return this.getProperty(Constants.DESCRIPTION);
		},

		getParentUri: function() {
			if (!this.parentUri) {
				if (this.getMutable() == "true") {
					this.parentUri = this.getParentWorkspaceUri();
				} else {
					this.parentUri = this.getParentSnapshotUri();
				}
			}
			return this.parentUri;
		},
		
		getParentName: function() {
			if (!this.parentName) {
				if (this.getMutable() == "true") {
					this.parentName = this.getParentSnapshotName();
				} else {
					this.parentName = this.getParentWorkspaceName();
				}
			}
			return this.parentName;
		},
		
		getParentWorkspaceUri: function() {
			return this.getProperty("parentWorkspaceUri");
		},

		getParentWorkspaceName: function() {
			return this.getProperty("parentWorkspaceName");
		},
		
		getParentSnapshotUri: function() {
			return this.getProperty("parentSnapshotUri");
		},

		getParentSnapshotName: function() {
			return this.getProperty("parentSnapshotName");
		},
		
		getSpaceUri: function() {
			return this.getProperty("spaceUri");
		},
		
		getSpaceName: function() {
			return this.getProperty("spaceName");
		},
		
		getDimensions: function() {
			return this.getArrayProperty("dimensions");
		},
		
		getDimensionUris: function() {
			return this.getArrayProperty("dimensionUris");
		},
		
		getMutable: function() {
			return this.getProperty(Constants.MUTABLE);
		},
		
		getBase: function() {
			return this.getProperty(Constants.BASE);
		},
		
		isMutable: function() {
			if (this.getMutable() == "true") {
				return true;
			}
		},
		
		isWorkspace: function()
		{
			if (this.getType() == "workspace")
				return true;
			return false;	
		},
		
		isSnapshot: function()
		{
			if (this.getType() == "snapshot")
				return true;
			return false;	
		},
		
		isChangeset: function() {
			if (this.getType() == "changeset")
				return true;
			return false;
		},

		getPointUri: function() {
			return this.getProperty(Constants.POINT);
		},
		
		getSelects: function() {
			return this.getProperty(Constants.SELECTS);
		},
		
		getCreated: function() {
			return this.getProperty(Constants.CREATED);
		},

		getCreator: function() {
			return this.getProperty(Constants.CREATOR);
		},
		
		// check for duplicate name constraint
		isConflictingTitle: function() {
			var newTitle = this.getTitle();
			var oldTitle = this.getProperty(Constants.TITLE);
			if (oldTitle ===  newTitle) {
				// title field has not been changed
				return false;
			}
			
			// TODO: invoke a service to do validation
			return false;
		},
		
		save: function() {
			// load existing configuration
			var config = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
			config.load(this.getUri());
					
			// only name and description can be saved
			config.setProperty(Constants.TITLE, this.getTitle());
			config.setProperty(Constants.DESCRIPTION, this.getDescription());
			return config.doSave();
		},

		setTitle: function(value) {
			this.title = value;
		},

		setDescription: function(value) {
			return this.setProperty(Constants.DESCRIPTION, value);
		},
		
		setPointUri: function() {
			return this.setProperty(Constants.POINT, value);
		},

		getType: function()
		{
			if (this.getProperty("type") == null)
			{
				var errorMsg = "Assert Failure: Attempting to ask for the configurations type when it's not set on the configuration.";
				console.error(errorMsg);
				alert(errorMsg);
				return;
			}
			return this.getProperty("type");
		},
		
		isArchived: function() {
			return "http://jazz.net/ns/vvc#Archive" === this.getProperty("http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
		},
		
		loadPermissions: function(uri, onError) {
			var args = {
					url: uri,
					headers: {
						"Accept" : "application/json",
					},
					handleAs: "json",
					load: dojo.hitch(this, this.onLoadPermissions),
					error: dojo.hitch(this, this.onError),
					failOk: true
			};
			
			if (onError != null)
				args.error = onError;
			return jazz.client.xhrGet(args);
		},
		
		 onLoad: function(response, ioArgs) {
			 this.inherited(arguments);
			 
			 if (this.properties.permissions) {
				 this.permissions = [];
				 for (var i = 0; i < this.properties.permissions.length; i++) {
				   this.permissions.push(this.properties.permissions[i].value);
				 }
				 this.permissionsLoaded = true;
			}
				 
			 return this;
		 },
		
		onLoadPermissions: function(response, ioArgs) {
			this.permissions = response;
			this.permissionsLoaded = true;
			return this.permissions;
		},
		
		hasPurposeDimension: function () {
			if (!this.loaded) {
				throw "Configuration resource has not been loaded yet";
			}
			
			var theDimensions = this.getDimensionUris();
			var value;
			for (var i = 0; i < theDimensions.length; i++) {
				if (theDimensions[i].value === "http://jazz.net/ns/vvc/ootb#purpose") {
					return true;
				}
					
			}
			
			return false;
		}
		
		
	});
})();