/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.vvc.shared.core.ui.objects.DeliverSession");

	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
	var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.VvcObject");

	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.DeliverableChangeset");
	var DeliverableChangeset = com.ibm.team.vvc.shared.core.ui.objects.DeliverableChangeset;

	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.CompareProvider");
	var CompareProvider = com.ibm.team.vvc.shared.core.ui.objects.CompareProvider;

	/**
	 * A deliver session is passed back and forth between a client and a server to
	 * initiate, progress and complete/discard a delivery.
	 */
	dojo.declare("com.ibm.team.vvc.shared.core.ui.objects.DeliverSession",
			com.ibm.team.vvc.shared.core.ui.objects.VvcObject, {		

		getFrontServiceId: function() {
			return "deliver";
		},
		
		/**
		 * Gets the state of the deliver session. Can be one of 
		 * 1. Constants.DELIVER_SESSION_STATE_UNINITIALIZED,
		 * 2. Constants.DELIVER_SESSION_STATE_PENDING, or
		 * 3. Constants.DELIVER_SESSION_STATE_FINISHED
		 * 
		 */
		getState: function() {
			return this.getProperty(Constants.DELIVER_STATE);
		},

		/**
		 * Sets the state of the deliver session. Clients can update
		 * this property and PUT the deliver session on the VVC server
		 * to progress the deliver session.
		 * 
		 * Can be any of the following states:
		 * 1. Constants.DELIVER_SESSION_STATE_UNINITIALIZED,
		 * 2. Constants.DELIVER_SESSION_STATE_PENDING, or
		 * 3. Constants.DELIVER_SESSION_STATE_FINISHED
		 *  
		 * You are only allowed to go forward in states: from uninitialized
		 * to pending, and from pending to finished. Going backwards is not allowed.
		 */
		setState: function(value) {
			return this.setProperty(Constants.DELIVER_STATE, value);
		},
		
		/**
		 * Gets the compare providers. See CompareProvider. 
		 */
		getProviders: function() {
			var providers = this.getProperty(Constants.DELIVER_COMPARE_PROVIDER);
			var retVal = [];
			
			if (providers) {
				if (!(providers instanceof Array)) {
					providers = [providers];
				}
				
				for (var i = 0; i < providers.length; i++) {
					var provider = new CompareProvider(providers[i]);
					retVal.push(provider);
				}
			}
			
			return retVal;
		},
		
		/**
		 * Gets the expiration date of the session. After this date, the 
		 * session will cease to exist on the server. The format of the date
		 * is a number which represents the milliseconds since Jan 1, 1970.
		 * 
		 * To prolong the session, send a PUT to {sessionUri}/keepAlive
		 */
		getExpirationDate: function() {
			return this.getProperty(Constants.DELIVER_EXPIRATION_DATE);
		},
		
		/**
		 * Gets the date this session was created at. The format of the date
		 * is a number which represents the milliseconds since Jan 1, 1970.
		 */
		getCreatedAt: function() {
			return this.getProperty(Constants.DELIVER_CREATED_DATE);
		},
		
		/**
		 * Gets the referer URL (the web page that redirected to the
		 * deliver session UI). Once the deliver session finishes, the user
		 * is redirected to this URI.
		 */
		getRefererUrl: function() {
			return this.getProperty(Constants.DELIVER_REFERER);
		},

		/**
		 * Sets the referer URL (the web page that redirected to the
		 * deliver session UI). Once the deliver session finishes, the user
		 * is redirected to this URI.
		 */
		setRefererUrl: function(value) {
			return this.setProperty(Constants.DELIVER_REFERER, value);
		},
		
		/**
		 * Gets the source configuration URI.
		 */
		getSourceUri: function() {
			return this.getProperty(Constants.DELIVER_SOURCE);
		},

		/**
		 * Sets the source configuration URI.
		 */
		setSourceUri: function(value) {
			return this.setProperty(Constants.DELIVER_SOURCE, value);
		},
		
		/**
		 * Gets the target configuration URI.
		 */
		getTargetUri: function() {
			return this.getProperty(Constants.DELIVER_TARGET);
		},

		/**
		 * Sets the target configuration URI.
		 */
		setTargetUri: function(value) {
			return this.setProperty(Constants.DELIVER_TARGET, value);
		},
		
		/**
		 * Gets the changesets that were selected by the user to deliver or compare.
		 */
		getSelectedChangesets: function() {
			return this.getProperty(Constants.DELIVER_SELECTED_CHANGESETS);
		},

		/**
		 * Sets the changesets that are selected by the user to deliver or compare.
		 */
		setSelectedChangesets: function(value) {
			return this.setPropertyArray(Constants.DELIVER_SELECTED_CHANGESETS, value, "uri");
		},
		
		/**
		 * If true, requires the user to accept all outgoing changesets without gaps.
		 * 
		 * If false, the user can deliver changesets with gaps, and even partial changesets. 
		 */
		isApplyCompleteChangesetsOnly: function() {
			return this.getBooleanProperty(Constants.DELIVER_APPLY_COMPLETED_CHANGESETS_ONLY);
		},

		/**
		 * Set to true if you require the user to accept all outgoing changesets without gaps.
		 * 
		 * Set to false if you want to allow the user to deliver changesets with gaps, 
		 * and even partial changesets.
		 */
		setApplyCompleteChangesetsOnly: function(value) {
			return this.setBooleanProperty(Constants.DELIVER_APPLY_COMPLETED_CHANGESETS_ONLY, value);
		},
		
		/**
		 * What type of a operation is this: deliver, compare or rebase?
		 * Can be any of the following values:
		 * 1. Constants.DELIVER_OPERATION_DELIVER
		 * 2. Constants.DELIVER_OPERATION_COMPARE
		 * 3. Constants.DELIVER_OPERATION_REBASE
		 */
		getOperation: function() {
			return this.getProperty(Constants.DELIVER_OPERATION);
		},

		/**
		 * Sets the type of operation of this session. Can only be set
		 * before the session is created.
		 * 
		 * Can be any of the following values:
		 * 1. Constants.DELIVER_OPERATION_DELIVER
		 * 2. Constants.DELIVER_OPERATION_COMPARE
		 * 3. Constants.DELIVER_OPERATION_REBASE
		 */
		setOperation: function(value) {
			return this.setProperty(Constants.DELIVER_OPERATION, value);
		},
		
		/**
		 * Convenience method to check if this session is a compare-only operation.
		 * Calls getOperation() to make the check.
		 */
		isCompareOnly: function() {
			var operation = this.getOperation();
			if (operation === Constants.DELIVER_OPERATION_COMPARE) {
				return true;
			} else {
				return false;
			}
		},
		
		/**
		 * Convenience method to check if this session is a rebase operation.
		 * Calls getOperation() to make the check.
		 */
		isRebase: function() {
			var operation = this.getOperation();
			if (operation === Constants.DELIVER_OPERATION_REBASE) {
				return true;
			} else {
				return false;
			}
		},
		
		/**
		 * If true, shares the changeset when the deliver session is finished
		 * successfully.
		 */
		isCommitChangeSet: function() {
			return this.getBooleanProperty(Constants.COMMIT_CHANGE_SET);
		},

		/**
		 * Set to true if you want to share the merges immediately after
		 * completing the deliver session successfully.
		 */
		setCommitChangeSet: function(value) {
			return this.setBooleanProperty(Constants.COMMIT_CHANGE_SET, value);
		},
		
		/**
		 * Gets the deliverable changesets. These are changesets that can be
		 * delivered from the source into the target.
		 * 
		 * The resource is a type of DeliverableChangeset.js
		 */
		getDeliverableChangesets: function() {
			var changesets = this.getProperty(Constants.DELIVER_DELIVERABLE_CHANGESETS);
			var retVal = [];
			
			if (changesets) {
				if (!(changesets instanceof Array)) {
					changesets = [changesets];
				}
				
				for (var i = 0; i < changesets.length; i++) {
					var changeset = new DeliverableChangeset(changesets[i]);
					retVal.push(changeset);
				}
			}
			
			return retVal;
		},		
		
	});
})();