/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.vvc.shared.core.ui.objects.DeliverableChangeset");

	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
	var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.VvcObject");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.SecurityServiceUtils");

	/**
	 * A deliverable changeset is one that can be delivered from a source to a target
	 * in a deliver operation.
	 * 
	 */
	dojo.declare("com.ibm.team.vvc.shared.core.ui.objects.DeliverableChangeset",
			com.ibm.team.vvc.shared.core.ui.objects.VvcObject, {		

		constructor: function(properties) {
			this.properties = properties;
		},
		
		getFrontServiceId: function() {
			return null;
		},

		/**
		 * Changesets are versioned resources. A changeset concept assigns a unique
		 * ID to a changeset so that logically same changesets across different
		 * configurations can be identified as the same changeset
		 */
		getChangesetConcept: function() {
			return this.getProperty(Constants.CONCEPT);
		},

		/**
		 * Changeset title
		 */
		getTitle: function() {
			return this.getProperty(Constants.TITLE);
		},

		getCreator: function() {
			return this.getProperty(Constants.CHANGESET_CREATOR);
		},
		
		/**
		 * What is the state of this changeset?
		 * 
		 * Can be one of:
		 * 1. Constants.STATE_COMMITTED
		 * 2. Constants.STATE_COMPLETED
		 * 3. Constants.STATE_IN_PROGRESS
		 */
		getState: function() {
			return this.getProperty(Constants.STATE);
		},
		
		/**
		 * The changesets that this deliverable changeset depends on.
		 */
		getGapDependentChangesets: function() {
			return this.getProperty(Constants.DELIVER_GAP_DEPENDENT_CHANGESETS);
		},
		
		/**
		 * The order in which this changeset should be shown in the UI.
		 */
		getOrder: function() {
			return this.getProperty(Constants.DELIVER_ORDER);
		},
		
		
	});
})();