/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function GlobalConfigurationResource() {
	dojo.provide("com.ibm.team.vvc.shared.core.ui.objects.GlobalConfigurationResource");

	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
	var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.GlobalConfigurationConstants");
	var GlobalConfigurationConstants = com.ibm.team.vvc.shared.core.ui.objects.GlobalConfigurationConstants;
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.VvcObject");
		
	var DC_TERMS = "http://purl.org/dc/terms/";

	dojo.declare("com.ibm.team.vvc.shared.core.ui.objects.GlobalConfigurationResource", com.ibm.team.vvc.shared.core.ui.objects.VvcObject, {
		
		constructor: function __GlobalConfigurationResource(args) {
			if (!args) {
				args = {};
			}
			
			this.setUri(args.uri ? args.uri : "https://tobeassigned");

			this.setProperty("http://www.w3.org/1999/02/22-rdf-syntax-ns#type", GlobalConfigurationConstants.GC, "uri");
			
			//don't do the safeMixin -- do it the legible way since we are using setters anyway
			if (args.title) {
				this.setTitle(args.title);
			} else {
				this.setTitle("");	
			}
			
			if (args.description) {
				this.setDescription(args.description);
			} else {
				this.setDescription("");
			}
			
			if (args.mutable) {
				this.setMutable(args.mutable);
			} else {
				this.setMutable(false);	
			}
			
			if (args.component) {
				this.setComponent(args.component);
			}
			
			if (args.previousBaselines) {
				this.setPreviousBaselines(args.previousBaselines);
			} else {
				this.setPreviousBaselines([]);
			}
			
			if (args.contributions) {
				this.setContributions(args.contributions);
			} else {
				this.setContributions([]);
			}
		},
		
		getFrontServiceId: function __getFrontServiceId() {
			return "globalConfigurations";
		},

		getTitle: function __getTitle() {
			return this.getProperty(Constants.TITLE);
		},
		
		setTitle: function __setTitle(newTitle) {
			this.setProperty(Constants.TITLE, newTitle, "literal");
		},		
		
		getDescription: function __getDescription() {
			return this.getProperty(Constants.DESCRIPTION);
		},
		
		setDescription: function __setDescription(newDesc) {
			this.setProperty(Constants.DESCRIPTION, newDesc, "literal");
		},
		
		getMutable: function __getMutable() {
			//be careful, it is stored as a string!
			return this.getProperty(GlobalConfigurationConstants.GC_MUTABLE) === "true";
		},
		
		setMutable: function __setMutable(newMutable) {
			this.setBooleanProperty(GlobalConfigurationConstants.GC_MUTABLE, newMutable);
		},		
		
		getComponent: function __getComponent() {
			return this.getProperty(GlobalConfigurationConstants.GC_COMPONENT);
		},
		
		setComponent: function __setComponent(newComponent) {
			this.setProperty(GlobalConfigurationConstants.GC_COMPONENT, newComponent, "uri");
		},		
		
		getPreviousBaselines: function __getPreviousBaselines() {
			return this.getArrayProperty(GlobalConfigurationConstants.GC_PREVIOUS_BASELINES);
		},
		
		setPreviousBaselines: function __setPreviousBaselines(value) {
			this.setPropertyArray(GlobalConfigurationConstants.GC_PREVIOUS_BASELINES, value, "uri");
		},
		
		getContributions: function __getContributions() {
			return this.getArrayProperty(GlobalConfigurationConstants.GC_CONTRIBUTION);
		},
		
		setContributions: function __setContributions(value) {
			this.setPropertyArray(GlobalConfigurationConstants.GC_CONTRIBUTION, value, "uri");
		},
		
		//////////////// other properties
		
		getCreated: function __getCreated() {
			return this.getProperty(DC_TERMS + Constants.CREATED);
		},

		getCreator: function __getCreator() {
			return this.getProperty(DC_TERMS + Constants.CREATOR);
		},

//		getType: function __getType() {
//			return this.getProperty("http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
//		},
				
		doCreate: function __doCreate() {
			/*
			 * Need the wrapper deferred to ensure we can wait until we know the VVC URL, and 
			 * still return a deferred for the callers of doCreate.  Since this implementation 
			 * of getVvcUri (copied from VvcUtils.js) is asynchronous, we don't know if we have
			 * the VVC URI cached, or if we will need to fetch it asynchronously.  This forces us
			 * to delay creation of the the 'doCreateDef' until we know the VVC URI.  Since we 
			 * still need to return a deferred, we wrap 'doCreateDef' in a dummy deferred and,
			 * once 'doCreateDef' is done, resolve 'wrapperDef' with either the success or error
			 * return values directly.  This will hide the xhr ioArgs from the caller, so be
			 * careful when implementing error handling.
			 */
			var wrapperDef = new dojo.Deferred();
			var vvcUrlOrDef = sessionStorage.getItem("com.ibm.team.vvc.vvcUrl");
			if(!vvcUrlOrDef){
				var vvcUrlDef = jazz.client.xhrGet({
					url: net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + '/queryvvc/vvcurl',
				});
				vvcUrlDef.then(dojo.hitch(this, function __vvcUrlDefSuccess(){
					vvcUrl = vvcUrlDef.ioArgs.xhr.getResponseHeader("Location");
					sessionStorage.setItem("com.ibm.team.vvc.vvcUrl", vvcUrl);
				}));
				vvcUrlOrDef = vvcUrlDef;
			}
			
			dojo.when(vvcUrlOrDef, dojo.hitch(this, function(vvcUrl){
				var saveData = dojo.toJson(this.data);
				var theHeaders = {
					"Accept" : "application/rdf+json",
					"Content-Type" : "application/rdf+json",
				};
				var args = {
					url: vvcUrl + "/" + this.getFrontServiceId(),
					headers: theHeaders,
					handleAs: "json",
					postData: saveData,
					load: function __doCreateDefLoad(response, ioArgs) {
						if (ioArgs) {
							var xhr = ioArgs.xhr;
							if (xhr) {
								return xhr.getResponseHeader("Location");
							}
						}
					}
				};
				var doCreateDef = jazz.client.xhrPost(args);
				
				doCreateDef.then(dojo.hitch(this, function __doCreateDefSuccess(data){
					wrapperDef.resolve(data);
				}), dojo.hitch(this, function __doCreateDefError(err){
					wrapperDef.reject(err);
				}));
			}));
			return wrapperDef;
		}
		
		
	});
})();