/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.vvc.shared.core.ui.objects.MergedResources");

	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
	var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.VvcObject");
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ResolvedConflict");
	var ResolvedConflict = com.ibm.team.vvc.shared.core.ui.objects.ResolvedConflict;

	/**
	 * Allows clients to specify how to resolve conflicts and which outgoing changes
	 * to accept.
	 */
	dojo.declare("com.ibm.team.vvc.shared.core.ui.objects.MergedResources",
			com.ibm.team.vvc.shared.core.ui.objects.VvcObject, {
		
		constructor: function(uri, acceptedOutgoingChanges, resolvedConflicts) {
			this.setUri(uri);
			this.setAcceptedOutgoingChanges(acceptedOutgoingChanges);
			this.setResolvedConflicts(resolvedConflicts);
		},
		
		/**
		 * Gets the resolved conflicts.
		 * 
		 * @return array of ResolvedConflict.js
		 */
		getResolvedConflicts: function() {
			var resolvedConflicts = this.getProperty(Constants.DELIVER_RESOLVED_CONFLICTS);
			var retVal = [];
			
			if (resolvedConflicts) {
				if (!(resolvedConflicts instanceof Array)) {
					resolvedConflicts = [resolvedConflicts];
				}
				
				for (var i = 0; i < resolvedConflicts.length; i++) {
					var resolvedConflict = new ResolvedConflict(resolvedConflicts[i]);
					retVal.push(resolvedConflict);
				}
			}
			
			return retVal;
		},

		/**
		 * Sets the resolved conflicts.
		 * 
		 * Takes an argument of an array of ResolvedConflict.
		 */
		setResolvedConflicts: function(value) {
			var name = Constants.DELIVER_RESOLVED_CONFLICTS;
			
			if (!this.properties) {
				this.properties = {};
				if (!this.data) {
					this.data = {};
				}
				this.data[this.uri] = this.properties;
			}
			
			if (!value || (value instanceof Array && value.length == 0)) {
				if (this.properties[name]) {
					delete this.properties[name];
				}
				
				return;
			}
			
			if (!(value instanceof Array)) {
				value = [value];
			}
			
			var valueToSet = [];
			for (var i = 0; i < value.length; i++) {
				var item = {};
				
				item.type = "bnode";
				item.value = "_:" + i;				
				valueToSet.push(item);
				
				this.data[item.value] = value[i].properties;
			}

			this.properties[name] = valueToSet;
		},
		
		/**
		 * Gets the accepted outgoing changes.
		 * 
		 * @return array of String (concept resources).
		 */
		getAcceptedOutgoingChanges: function() {
			return this.getProperty(Constants.DELIVER_ACCEPTED_OUTGOING);
		},

		/**
		 * Sets the accepted outgoing changes. 
		 * 
		 * Takes an argument of an array of String (concept resources). 
		 */
		setAcceptedOutgoingChanges: function(value) {
			return this.setPropertyArray(Constants.DELIVER_ACCEPTED_OUTGOING, value, "uri");
		},
		
	});
})();