/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.vvc.shared.core.ui.objects.VvcObject");
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.utils.ErrorUtil");
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Constants");
	var Constants = com.ibm.team.vvc.shared.core.ui.objects.Constants;
	
	// Temporary cache for defect 47313.  Under no circumstances should this be propagated!
	var VvcObjectCache = {};
	
	dojo.declare("com.ibm.team.vvc.shared.core.ui.objects.VvcObject", null, {

		uri: null,
		properties: null,
		data: null,
		loaded: false,
		eTag: null,
		blankNodes: {},
		
		constructor: function(args) {
			if (args) {
				dojo.safeMixin(this, args);
			}
		},
		
		getUri: function() {
			return this.uri;
		},
		
		setUri: function(uri) {
			this.uri = uri;
		},

		isLoaded: function()
		{
			return this.loaded;
		},
		
		getArrayProperty: function(name) {
			var value = null;
			if (this.properties) {
				value = this.properties[name];
			}
			return value;
		},
		
		setBooleanProperty: function(name, value) {
			var valueToSend = "false";
			if (value) {
				valueToSend = "true";
			}
			
			return this.setProperty(name, valueToSend, "literal", Constants.XMLS_BOOLEAN);
		},
		
		getBooleanProperty: function(name) {
			var value = this.getProperty(name);
			if (value === "true" || value === true) {
				return true;
			} else {
				return false;
			}
		},
		
		getProperty: function(name) {
			var value = null;
			if (this.properties) {
				var property = this.properties[name];
				if (property) {
					if (property.length === 1) {
						if (property[0].type === "bnode") {
							value = this.blankNodes[property[0].value];
						} else {
							value = property[0].value;
						}
					} else {
						value = [];
						for (var i = 0; i < property.length; i++) {
							if (property[i].type === "bnode") {
								value.push(this.blankNodes[property[i].value]);
							} else {
								value.push(property[i].value);
							}
						}						
					}
				}
			}
			return value;
		},
		
		setPropertyArray: function(name, value, type, datatype) {
			if (!this.properties) {
				this.properties = {};
				if (!this.data) {
					this.data = {};
				}
				this.data[this.uri] = this.properties;
			}
			
			if (!value || (value instanceof Array && value.length == 0)) {
				if (this.properties[name]) {
					delete this.properties[name];
				}
				
				return;
			}
			
			if (!(value instanceof Array)) {
				value = [value];
			}
			
			var valueToSet = [];
			for (var i = 0; i < value.length; i++) {
				var item = {};
				if (!type) {
					type = "literal";
				}
				item.type = type;
				item.value = value[i];
				
				if (datatype) {
					item.datatype = datatype;
				}
				
				valueToSet.push(item);
			}

			this.properties[name] = valueToSet;
		},

		setProperty: function(name, value, type, datatype) {
			if (!this.properties) {
				this.properties = {};
				if (!this.data) {
					this.data = {};
				}
				this.data[this.uri] = this.properties;
			}
			var property = this.properties[name];
			if (property && property[0]) {
				property[0].value = value;
			} else {
				var item = {};
				if (!type) {
					type = "literal";
				}
				item.type = type;
				item.value = value;
				
				if (datatype) {
					item.datatype = datatype;
				}
				
				var propertyValue = [];
				propertyValue.push(item);
				
				if (!property) {
					property = {};
				}
				
				this.properties[name] = propertyValue;
			}
		},

		getType: function() {
			return this.getProperty("http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
		},

		// for sub class to override and return the front service name
		// such as "configurations", "dimensions", "definitions".
		getFrontServiceId: function() {
			return "?";
		},

		_load: function __load(uri, onError, synchronous) {
//			if(VvcObjectCache[uri] && VvcObjectCache[uri].expires >= Date.now()){
//				var def = new dojo.Deferred();
//				var response = VvcObjectCache[uri].response;
//				var ioArgs = VvcObjectCache[uri].ioArgs;
//				this.onLoad(response, ioArgs, true);
//				def.resolve(response);
//				return def;
//			}
			
			var args = {
				url: uri,
				headers: {
					"Accept" : "application/json;application/rdf+json;text/rdf+json;text/json",
				},
				sync: synchronous,
				handleAs: "json",
				load: dojo.hitch(this, this.onLoad),
//				load: dojo.hitch(this, function __onLoadXhr(response, ioArgs){
//					var expires = Date.now() + 30000; // Expires after 30s
//					var cachedItem = {
//						expires: expires,
//						response: response,
//						ioArgs: ioArgs
//					};
//					VvcObjectCache[uri] = cachedItem;
//					this.onLoad(response, ioArgs);
//				}),
				error: dojo.hitch(this, this.onError),
				failOk: true
			};
			
			if (onError)
				args.error = onError;
			return jazz.client.xhrGet(args);
		},
		
		load: function(uri, onError) {
			return this._load(uri, onError, true);
		},
		
		loadAsync: function(uri, onError) {
			return this._load(uri, onError, false);
		},		

		onLoad: function __onLoad(response, ioArgs) {			
			if (ioArgs && ioArgs.xhr) {
				this.eTag = ioArgs.xhr.getResponseHeader("ETag");
			}
			if (response) {
				this.data = response;
				for (var uri in response) {
					if (uri.lastIndexOf("_:", 0) === 0) {
						this.blankNodes[uri] = response[uri];
					} else {
						this.uri = uri;
						this.properties = response[uri];
					}
				}
				this.loaded = true;
			}
			return this;
		},

		onError: function(response, ioArgs) {
			if (response) {
				var err = com.ibm.team.vvc.shared.core.ui.utils.ErrorUtil.getErrorDetails(response);
				if ("EXISTS" === err.errorSubStatus) {
					//we will handle this specially
					return;
				}
			}
			console.log("Failed to load resource '"+ioArgs.args.headers.vvc_servicePath+"'...Invalid Response Status("+ioArgs.xhr.status+"): "+ response.message);
		},
		
		doSave: function() {
			var uri = this.getUri();
			var saveData = dojo.toJson(this.data);
			var theHeaders = {
					"Accept" : "application/rdf+json",
					"Content-Type" : "application/rdf+json",
				};
			if (this.eTag) {
				theHeaders["If-Match"] = this.eTag;
			}
			var args = {
					url: uri,
					headers: theHeaders,
					sync: true,
					handleAs: "json",
					putData: saveData,
					load: function(response, ioArgs) {
						if (ioArgs) {
							var xhr = ioArgs.xhr;
							if (xhr) {
								return xhr.getResponseHeader("Location");
							}
						}
					},
					error: dojo.hitch(this, this.onError),
			};
			return jazz.client.xhrPut(args);
		},
		
		doSaveAsync: function(successFunc, errorFunc) {
			var uri = this.getUri();
			var saveData = dojo.toJson(this.data);
			var theHeaders = {
					"Accept" : "application/rdf+json",
					"Content-Type" : "application/rdf+json",
				};
			if (this.eTag) {
				theHeaders["If-Match"] = this.eTag;
			}			
			var args = {
					url: uri,
					headers: theHeaders,
					sync: false,
					handleAs: "json",
					putData: saveData,
					load: successFunc,
					error: errorFunc,
			};
			return jazz.client.xhrPut(args);
		},
		
		doPostAsync: function(uri, successFunc, errorFunc) {
			// when posting, we need to set the URI to 'uninitialized'.
			this.setUri("https://tobeassigned");
			var saveData = dojo.toJson(this.data);
			var args = {
					url: uri, //$NON-NLS-1$
					headers: {
						"Accept" : "application/rdf+json",
						"Content-Type" : "application/rdf+json",
					},
					handleAs: "json",
					postData: saveData,
					load: successFunc,
					error: errorFunc,
			};
			return jazz.client.xhrPost(args);
		},

		
		
	});
})();