/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide('com.ibm.team.vvc.shared.web.ui.picker.ConfigurationSpacePicker');

dojo.require("dijit.form.TextBox");

dojo.require("jazz.ui.PaginationControl");
dojo.require("jazz.ui.StyledTable");

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcCache");
var VvcCache = com.ibm.team.vvc.shared.web.ui.util.VvcCache;

dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
var VvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.ConfigurationSpaceUiNode");
var ConfigurationSpaceUiNode = com.ibm.team.vvc.shared.web.ui.util.ConfigurationSpaceUiNode;

dojo.require('dojo.i18n');
dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.picker", "ConfigurationSpacePickerMessages");

dojo.declare("com.ibm.team.vvc.shared.web.ui.picker.ConfigurationSpacePicker", [dijit._Widget, dijit._Templated], {
	
	messages: dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.picker", "ConfigurationSpacePickerMessages"),
	
	templatePath: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'ui/picker/templates/ConfigurationSpacePicker.html'),
	widgetsInTemplate: true,
	
	_configurationSpaceList: null,
	_autoSearchTimer: null,		// The active search timer
	_autoSearchTimeout: 850, 	// How long to wait before executing the query (in ms)
	_searchInProgress: null,
	_paginationController: null,
	_cache: null,
	_searchTerm: null,
	_searchResults: null,
	_selectedRow: null,
	
	constructor: function(args){
		this._cache = new VvcCache({
			key: VvcConstants.CONFIGURATION_SPACE_CACHE_KEY,
    		target: VvcUtils.getVvcUri() + "/client/space",
    		bulkTarget: VvcUtils.getVvcUri() + "/client/spaces",
		});
		this._searchTerm = "";
		this._searchResults = [];
	},
	
	postCreate: function(){
		this._resultsTable = new jazz.ui.StyledTable({
			columns: [{label: this.messages.configurationSpace, width: "100%", sortable: false}],
			"class": "resultsTable"
		});
		this.resultsTableNode.appendChild(this._resultsTable.domNode);
		
		this._paginationController = new jazz.ui.PaginationControl({
			total: 0,
			increment: 5,
			onNextClick: dojo.hitch(this, function(widget, start, end){
				this._updateConfigurationSpaceList(start - 1);
			}),
			onPreviousClick: dojo.hitch(this, function(widget, start, end){
				this._updateConfigurationSpaceList(start -1);
			})
		});
		this.paginationNode.appendChild(this._paginationController.domNode)
		
		this._updateSearchResultsText();
		dojo.connect(this.searchInput, "onKeyUp", dojo.hitch(this, this.onInputChange));
		dojo.connect(this.searchAllButton, "onclick", dojo.hitch(this, this._onSearchAllConfigurationSpaces));
		dojo.connect(this.searchSpecificButton, "onclick", dojo.hitch(this, this._onSearchSpecificConfigurationSpaces));
	},
	
	_onSearchAllConfigurationSpaces: function(event){
		dojo.addClass(this.searchAndResultsNode, "hidden");
		this.resize();
		this.onSelection(null); // No selection means search everything
	},
	
	_onSearchSpecificConfigurationSpaces: function(event){
		dojo.removeClass(this.searchAndResultsNode, "hidden");
		this.resize();
	},
	
	resize: function(){
		// Stub
	},
	
	_updateSearchResultsText: function(){
		var resultsMessage = dojo.string.substitute(this.messages.searchResultsText, {
			resultsCount: this._searchResults.length,
			searchTerm: this._searchTerm,
		});
		dojo.empty(this.resultsTextNode);
		this.resultsTextNode.appendChild(dojo.doc.createTextNode(resultsMessage));
	},
	
	_updatePaginationControls: function(){
		this._paginationController.setTotal(this._searchResults.length);
	},
		
	onInputChange: function(event){
		dojo.stopEvent(event);
		if(this._searchInProgress){
			return;
		}
		if(this._autoSearchTimer){
			window.clearTimeout(this._autoSearchTimer);
		}
		this._autoSearchTimer = window.setTimeout(dojo.hitch(this, this.refresh), this._autoSearchTimeout);
	},
	
	refresh: function(){
		this._searchInProgress = true;
		this._searchTerm = this.searchInput.getValue().trim();
		var resultsDeferred = this._performSearch();
		dojo.when(resultsDeferred, dojo.hitch(this, function(results){
			this._searchResults = results.resultUris;
			this._updateSearchResultsText();
			this._updatePaginationControls();
			this._updateConfigurationSpaceList(0);
			this._searchInProgress = false;
		}), /* Error */dojo.hitch(this, this.onError));
	},
	
	_performSearch: function(){
		var searchTerm = this._searchTerm;
		var queryParams = {
			searchTerm: searchTerm
		};
		var args = {
			url: VvcUtils.getVvcUri() +"/spaces/search?" + dojo.objectToQuery(queryParams),
			headers: {
				"Accept" : "application/json",
				"Content-Type" : "application/json"
			},
			handleAs: "json",
			preventCache: true
		};
		return jazz.client.xhrGet(args);
	},
	
	_updateConfigurationSpaceList: function(start){
		var results = this._searchResults;
		dojo.empty(this._resultsTable.tbody);
		
		// No results message
		if(results.length === 0){
			var tr = dojo.create("tr");
			var td = dojo.create("td", {
				innerHTML: this.messages.noResults
			});
			tr.appendChild(td);
			this._resultsTable.tbody.appendChild(tr);
			return;
		}
		
		// Show the resulting configuration spaces
		var end = Math.min(results.length, start + this._paginationController.increment);
		for(var index = start; index < end; index++){
			var tr = dojo.create("tr");
			var td = dojo.create("td");
			var confSpaceUri = this._searchResults[index];
			var configurationSpaceUiNode = new ConfigurationSpaceUiNode({
				uri: confSpaceUri,
				highlightCharacters: this._searchTerm
			});
			td.appendChild(configurationSpaceUiNode.domNode)
			tr.appendChild(td);
			dojo.connect(configurationSpaceUiNode, "onclick", dojo.hitch(this, this._onSelectRow, tr, confSpaceUri))
			dojo.connect(tr, "onclick", dojo.hitch(this, this._onSelectRow, tr, confSpaceUri))
			dojo.connect(configurationSpaceUiNode, "onfocus", dojo.hitch(this, this._onSelectRow, tr, confSpaceUri))
			this._resultsTable.tbody.appendChild(tr);
		}
	},
	
	_onSelectRow: function(row, confSpaceUri){
		if(this._selectedRow){
			dojo.removeClass(this._selectedRow, "selected");
		}
		if(this._selectedRow !== row){
			this._selectedRow = row;
			dojo.addClass(this._selectedRow, "selected");
			this.onSelection(confSpaceUri);
		} else {
			this.onSelection(null);
		}
	},

	onSelection: function(selection){
		// Stub
	},
	
	onError: function(error){
		dojo.empty(this._resultsTable.tbody);
		var tr = dojo.create("tr");
		var td = dojo.create("td");
		var errorMessage = this.messages.errorGettingResults;
		
		var errorNode = dojo.create("span", {
			"class": "error",
			innerHTML: errorMessage
		});
		
		td.appendChild(errorNode);
		tr.appendChild(td);
		this._resultsTable.tbody.appendChild(tr);
		this._searchInProgress = false;
	}
	
});
})();
