/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2014.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.team.vvc.shared.web.ui.picker.WorkspacePicker');

dojo.require('jazz.app.auth');
dojo.require("jazz.ui.MessageBox");
dojo.require('jazz.ui.SearchBox');
dojo.require("jazz.ui.StyledTable");
dojo.require('jazz.util.html');
dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.toolbar.TwoStateButton");
dojo.require("jazz.ui.RemoteChooserResponder");
dojo.require("net.jazz.ajax.BootstrapProperties");

dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.ContentPane");

dojo.require('dojo.NodeList-traverse');
dojo.require('dojo.i18n');

dojo.require("com.ibm.team.vvc.shared.web.ui.picker.ConfigurationSpacePicker")
var ConfigurationSpacePicker = com.ibm.team.vvc.shared.web.ui.picker.ConfigurationSpacePicker;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcCache");
dojo.require("com.ibm.team.vvc.shared.web.ui.VvcConstants");
vvcConstants = com.ibm.team.vvc.shared.web.ui.VvcConstants;

dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;

dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.picker", "WorkspacePickerMessages");

(function ()
{
	
	function ignoreFieldKeyEvent(e)
	{
		if (e.ctrlKey || e.altKey) return true;

		switch(e.keyCode)
		{
		case dojo.keys.ENTER:
		case dojo.keys.ESCAPE:
		case dojo.keys.LEFT_ARROW:
		case dojo.keys.RIGHT_ARROW:
		case dojo.keys.TAB:
		case dojo.keys.SHIFT:
		case dojo.keys.ALT:
		case dojo.keys.CAPS_LOCK:
		case dojo.keys.PAGE_DOWN:
		case dojo.keys.PAGE_UP:
		case dojo.keys.LEFT_WINDOW:
		case dojo.keys.RIGHT_WINDOW:
		case dojo.keys.HOME:
		case dojo.keys.END:
		case dojo.keys.CTRL:
		case dojo.keys.F5:
			{
			return true;
			}
			
		default:
			return false;
		}
	}
	
	// TODO: This should use a utiliy method instead...
	function escapeHtml(html) {
		return html.replace(/&/g,'&amp;').replace(/</g,'&lt;').replace(/>/g,'&gt;');
	}
	
	/**
	 * Set "spaceUri" property to set the space uri you would like this dialog to constrain
	 * it's search results to, in the first argument of the constructor.
	 */
	dojo.declare('com.ibm.team.vvc.shared.web.ui.picker.WorkspacePicker', [dijit._Widget, dijit._Templated], {
			widgetsInTemplate: true,
			templatePath: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'ui/picker/templates/WorkspacePicker.html'),
				
			// Hooks
			onCancel: null,
			onOk: null,
			onSelect: null,
			onShowDimensions: null,
			onHideDimensions: null,
			closeable: null,
			defaultSearchTerm: null,
			singleClickMode: null,
			_cache: null,
			_remoteFrame: null,
			
			_validTypes: null,
			_findParentsOfUri: null,
			_pureGlobalConfigurationsOnly: null,
			_mutableGlobalConfigurationsOnly: null,
			_configurationSpacePicker: null,
			
			constructor: function (args) {
				
				var query = dojo.doc.location.search;
				if(query){
					var queryArgs = dojo.queryToObject(query.substring(1));
					// Convert boolean strings to real boolean values
					for(var arg in queryArgs){
						if(queryArgs[arg] === "true"){
							queryArgs[arg] = true;
						} else if(queryArgs[args] === "false"){
							queryArgs[arg] = false;
						}
					}
					dojo.mixin(args, queryArgs);
				}
				
				if (args) {
					if (args.spaceUri){
						this._spaceUri = args.spaceUri;
					}
					
					if (args.pureGlobalConfigurationsOnly) {
						this._pureGlobalConfigurationsOnly = args.pureGlobalConfigurationsOnly;
					}
					
					if (args.mutableGlobalConfigurationsOnly) {
						this._mutableGlobalConfigurationsOnly = args.mutableGlobalConfigurationsOnly;
					}
					
					if (args.findParentsOfUri) {
						this._findParentsOfUri = args.findParentsOfUri;
					}
										
					// CSV of types,  can be "workspace", "snapshot", or combination thereof
					if(args.validTypes){
						this._validTypes = args.validTypes;
					}
					
					if(args.queryOnLoad){
						var handle = dojo.connect(this, "startup", this, function(){
							this._runSearch(null, args.queryOnLoad)
							dojo.disconnect(handle);
						});
					}
					
					if(args.defaultSearchTerm){
						this.defaultSearchTerm = args.defaultSearchterm;
					}
					
					if(args.singleClickMode){
						this.singleClickMode === args.singleClickMode;
					}
					
					if(args.includeConfigurationSpacePicker){
						this._configurationSpacePicker = new ConfigurationSpacePicker();
					}
					
					if(args.closeable === false){
						this.closeable = false;
						var handle = dojo.connect(this, "postCreate", dojo.hitch(this, function(){
							this.hideOkCancelButtons();
							dojo.disconnect(handle);
						}));
					} else{
						this.closeable = true;
					}
				}
				
				try {
					this._remoteFrame = new jazz.ui.RemoteChooserResponder();
					if (!this._remoteFrame.isPostMessageProtocol()) {
						this._remoteFrame = undefined;
					} else if(!this._configurationSpacePicker && !this._pureGlobalConfigurationsOnly){
						this._configurationSpacePicker = new ConfigurationSpacePicker();
					}
				} catch (e) {
					//_remoteFrame will be undefined if it's not a supported message protocol
				}
				
				this.messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.picker", "WorkspacePickerMessages");
				this.spinnerUrl = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/spinner.gif');
				this._iconSrc = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/VVC.gif');
				
				this._cache = new com.ibm.team.vvc.shared.web.ui.util.VvcCache({
					storageKey: vvcConstants.CONFIGURATION_CACHE_KEY
				});
				
			},
			
           cancel: function() {
				if (this._remoteFrame && this._remoteFrame.sendResponse) {
					var oslcResponse = {
						"oslc:results" : []
					};
					this._remoteFrame.sendResponse(dojo.toJson(oslcResponse));
				}
				this.onCancel && this.onCancel();
				this.destroyRecursive();
			},

			postCreate: function() {
				var self = this;
				
				if(this._configurationSpacePicker){
					this.configurationSpacePickerNode.appendChild(this._configurationSpacePicker.domNode);
					resize = dojo.hitch(this, function(){
						var pickerNodeSize = dojo.position(this.configurationSpacePickerContainer, true);
						var containerSize = dojo.position(this._picker, true);
						var height = containerSize.h - (pickerNodeSize.h + 7);
						dojo.style(this._mainTable, "height", height + "px");
					});
					dojo.connect(this._configurationSpacePicker, "resize", dojo.hitch(this, function(){
						resize();
					}));
					dojo.connect(this._configurationSpacePicker, "onSelection", dojo.hitch(this, function(configurationSpaceUrl){
						if(configurationSpaceUrl){
							this._spaceUri = configurationSpaceUrl;
						} else {
							this._spaceUri = null;
						}
						var searchTerm = dojo.trim(this._searchBox.value)
						if(searchTerm){
							this._runSearch(null, searchTerm);
						} else {
							this._runSearch(null, "*");
						}
					}));
					resize();
				} else {
					dojo.destroy(this.configurationSpacePickerContainer);
					
					var containerSize = dojo.position(this._picker, true);
					dojo.style(this._mainTable, "height", containerSize.h + "px");
				}
				
				if (this._remoteFrame) {
					dojo.addClass(this._mainForm, "oslcModeHeight");
					dojo.addClass(this._mainForm, "oslcModeWidth");
					dojo.addClass(this._workspacePickerContents, "oslcModeHeight");
					dojo.addClass(this._workspacePickerDetails, "oslcModeHeight");
					dojo.removeClass(this._workspacePickerDependencies, "com-ibm-team-vvc-picker-WorkspacePicker-fixed-height-result");
					dojo.addClass(this._workspacePickerDependencies, "oslcModeHeight");
					dojo.removeClass(this._resultListScroller, "com-ibm-team-vvc-picker-WorkspacePicker-fixed-height-result");
					dojo.addClass(this._resultListScroller, "oslcModeHeight");
					
					dojo.addClass(this.okCancelButtons, "com-ibm-team-vvc-picker-WorkspacePicker-tableRow");
					dojo.addClass(this._resultListScroller, "com-ibm-team-vvc-picker-WorkspacePicker-tableRow");
					dojo.addClass(this._resultsTableToolbar, "com-ibm-team-vvc-picker-WorkspacePicker-tableRow"); //this should add, not replace
					dojo.addClass(this._workspacePickerHeader, "com-ibm-team-vvc-picker-WorkspacePicker-tableRow"); //this should add, not replace
					dojo.addClass(this._workspacePickerContents, "com-ibm-team-vvc-picker-WorkspacePicker-tableRow");
					dojo.addClass(this._mainForm, "com-ibm-team-vvc-picker-WorkspacePicker-table");
					
					dojo.removeClass(this._workspacePickerDependencies, "com-ibm-team-vvc-picker-WorkspacePicker-dependencies-margin");

					var hintHeight = "500px";
					var hintWidth = "100%";
					//by design, all URL parameters will be ignored if you are NOT in OSLC mode.
					if (location.search.length > 0) {
						var obj = dojo.queryToObject(location.search.substring(1));
						if (obj) {
							if (obj.hintHeight && obj.hintWidth) {
								hintHeight = obj.hintHeight;
								hintWidth = obj.hintWidth;
							}
							if (obj.pureGlobalConfigurationsOnly === "true") {
								this._pureGlobalConfigurationsOnly = true;
							} else if (obj.mutableGlobalConfigurationsOnly === "true") {
								this.mutableGlobalConfigurationsOnly = true;
							} else if (obj.contextUri) {
								this._spaceUri = obj.contextUri;
							}
						}
					}
					
					dojo.style(this._mainTable, "height", hintHeight);
					dojo.style(this._picker, "width", hintWidth);
					dojo.style(this._picker, "height", hintHeight);
					this.resize();
				}
 				
				this._detailsOn = false;
				this._createMenubar();
				this._createResultsTable();
				this._createDimensionsTable();
				
				// set initial focus
				if (this._searchBox) {
					var context = this;
					setTimeout(function() {
						if (context._searchBox) {
							context._searchBox.focus();
						}
					}, 100);
				}
				
				if (this._remoteFrame) {
					var size = dojo.position(this._picker, true);
					this._remoteFrame.sendResize(size.w + 15 + "px", size.h + "px");
				}
				this._updateCount(0);
			},
			
			resize: function() {
				this.inherited(arguments);
				if (this._configurationSpacePicker)
					this._configurationSpacePicker.resize();
			},
			
			_createMenubar: function(){
				this.toolbar = new jazz.ui.Toolbar();
								
				var refreshButton = new jazz.ui.toolbar.Button({
					title: this.messages.refresh,
					imgClass: "refreshIcon",
					onClick: dojo.hitch(this, function(){
						this._cache.clear();
						var searchTerm = dojo.trim(this._searchBox.value);
						if(searchTerm){
							this._runSearch(null, searchTerm);
						}
					})
				});
				this.toolbar.addChild(refreshButton);
				
				if(!this._configurationSpacePicker && !this._remoteFrame){
					this.detailsToggleButton = new jazz.ui.toolbar.TwoStateButton({
						title: this.messages.toggleDetails,
						imgClass: "showDetailsIcon",
						onClick: dojo.hitch(this, this._toggleDetails)
					});
					this.toolbar.addChild(this.detailsToggleButton);
				}
				
				this._resultsTableToolbar.appendChild(this.toolbar.domNode);
			},

			_toggleDetails: function()
			{
				if (this._detailsOn)
				{
					this._detailsOn = false;
					this._dimensionsTableSection.style.display = "none";
					if(this.onHideDimensions){
						this.onHideDimensions();
					}
				}
				else
				{
					this._detailsOn = true;
					this._dimensionsTableSection.style.display = "";
					if(this.onShowDimensions){
						this.onShowDimensions();
					}
				}
				
				if (this._remoteFrame) {
					var size = dojo.position(this._mainTable, true);
					this._remoteFrame.sendResize(size.w + 15 + "px", size.h + "px");
				}
			},
			_createResultsTable: function()
			{
				var table = new jazz.ui.StyledTable({
					   columns: [
					      {label: "", width: "16px", sortable: false},
					      {label: this.messages.name, width: "40%", sortable: false},
					      {label: this.messages.description, sortable: false}
					   ]
				});

				dojo.style(table.domNode, "width", "100%");
				this._resultsTable = table;
				this._resultsTableDiv.appendChild(table.domNode);
			},
			
			_createDimensionsTable: function()
			{
				var table = new jazz.ui.StyledTable({
					   columns: [
					      {label: this.messages.dimensionName, sortable: false},
					      {label: this.messages.dimensionValue, sortable: false}
					   ],
				});
				
				table.domNode.style.width = "100%";
				this._dimensionsTable = table;
				this._dimensionsTableDiv.appendChild(table.domNode);
			},

			startup: function() {
				this.inherited(arguments);
				var self = this;
				window.setTimeout(function() {
					self._searchBox.focus();
				}, 1);
			},

			getSelected: function()
			{
				return this._selectedRow ? this._selectedRow.workspace : undefined;
			},
			
			_selectRow: function(row) {
				if (this._selectedRow) dojo.removeClass(this._selectedRow, 'selected');
				
				if (!row || !row.workspace)
					return;
				
				dojo.addClass(row, 'selected');
				this._selectedRow = row;

				this._submitButton.disabled = false;
				
				if (!this._pureGlobalConfigurationsOnly && !this._mutableGlobalConfigurationsOnly) {
					this._loadDimensionValuesIntoTable(row.workspace.dimensionValueSet);
				}

			},
			
			_loadDimensionValuesIntoTable: function(dimensionValueSet)
			{
				var frag = document.createDocumentFragment();
				
				this._dimensionsTable.clearBody();

				var tr, td;
				for (var i = 0; i < dimensionValueSet.length; i++) {
					var dimensionValue = dimensionValueSet[i];
					
					tr = document.createElement("tr");
					
					td = document.createElement("td");
					td.innerHTML = jazz.util.html.escape(dimensionValue.dimensionName);
					tr.appendChild(td);
					
					td = document.createElement("td");
					td.innerHTML = jazz.util.html.escape(dimensionValue.dimensionValue);
					tr.appendChild(td);
					frag.appendChild(tr);
				}
				
				this._dimensionsTable.tbody.appendChild(frag);
			},

			_handleArrow: function(key) {
				if (!this._lastResult) return;

				var row;
				if (key == dojo.keys.DOWN_ARROW)
				{	
					if (!this._selectedRow) {
						// select first on arrow down if no selection
						row = this._getFirstRow(this._resultsTable.tbody);
					}
					else if (this._selectedRow.nextSibling && this._selectedRow.nextSibling.nodeName == 'TR') {
						row = this._selectedRow.nextSibling;
					}
				}
				else if (key == dojo.keys.UP_ARROW)
				{
					if (!this._selectedRow) {
						// select last on arrow up if no selection
						row = this._getFirstRow(this._resultsTable.tbody);
					}
					else if (this._selectedRow.previousSibling && this._selectedRow.previousSibling.nodeName == 'TR') {
						row = this._selectedRow.previousSibling;
					}
				}
				
				if(!row && this._selectedRow){
					row = this._selectedRow;
				}
				
				this._selectRow(row);
				if(this.onSelect && row){
					this.onSelect(row.workspace);
				}
			},

			_onFieldKeyDown: function(e) {
				if (ignoreFieldKeyEvent(e)) return;
				e.stopPropagation();
				var k = e.keyCode;
				
				if (k === dojo.keys.UP_ARROW
					|| k ===  dojo.keys.DOWN_ARROW) {
					dojo.stopEvent(e);
					this._handleArrow(k);
					return;
				}
				
				if(this.singleClickMode && k === dojo.keys.ENTER){
					dojo.stopEvent(e);
					this._onSubmit();
					return;
				}
				
				this._deselectAll();
				window.clearTimeout(this._pendingSearch);
				this._pendingSearch = window.setTimeout(dojo.hitch(this, this._runSearch, null), 400);
			},
			
			_onListKeyDown: function(e) {
				var k = e.keyCode;
				
				if (k == dojo.keys.UP_ARROW
					|| k ==  dojo.keys.DOWN_ARROW) {
					dojo.stopEvent(e);
					this._handleArrow(k);
					return;
				}
				
				if(this.singleClickMode && k === dojo.keys.ENTER){
					dojo.stopEvent(e);
					this._onSubmit();
					return;
				}
			}, 
			
			_onDblClickResource: function(e) {
				var row = e.target;

				if (e.target.nodeName != 'TR') {
					row = dojo.NodeList(e.target).closest('TR');
					if (!row) {
						console.warn('no list item found on click');
						return;
					}
					row = row[0];
				}

				dojo.stopEvent(e);
				this._selectRow(row);
				this._onSubmit();
			},

			_onClickResource: function(e) {
				var row = e.target;

				if (e.target.nodeName != 'TR') {
					row = dojo.NodeList(e.target).closest('TR');
					if (!row) {
						console.warn('no list item found on click');
						return;
					}
					row = row[0];
				}

				if (!dojo.hasClass(row, 'selected')) {
					this._deselectAll();
					this._selectRow(row);
				}
				
				if(this.singleClickMode){
					this._onSubmit(e);
				}
				
				if(this.onSelect && row){
					this.onSelect(row.workspace);
				}
			},
			
			_deselectAll: function()
			{
				if (this._selectedRow) {
					dojo.removeClass(this._selectedRow, 'selected');
					delete this._selectedRow;
				}

				this._submitButton.disabled = true;
			},
			
			_showLoading: function(show)
			{
				if (show) {
					this._loadingNode.style.display = '';
					this._noResultsNode.style.display = 'none';
				}
				else {
					this._loadingNode.style.display = 'none';
				}
			},

			_updateCount: function(count) {
				if (count == 0) {
					this._loadingNode.style.display = 'none';
					this._noResultsNode.style.display = '';
					this._noResultsNode.innerHTML = escapeHtml(this.messages.noResults);
				} else {
					this._noResultsNode.style.display = 'none';
				}
			},
			
			uninitialize: function()
			{
				this.inherited(arguments);
			},

			/*
			 * The provided searchTerm overrides the user's input seach term.  If the picker
			 * was created using a defaultSearcTerm, then this will be used in the provided
			 * searchTerm or the user's searchTerm are undefined.
			 */
			_runSearch: function(e, /*Optional*/searchTerm) {
				e && dojo.stopEvent(e);

				if (e && e.target == this._searchIcon) {
					this._searchBox.select();
				}

				window.clearTimeout(this._pendingSearch);
				
				var searchString;
				if(searchTerm){
					searchString = searchTerm;
				} else {
					searchString = dojo.trim(this._searchBox.value);
				}
				this.hideErrorMessage();
				
				delete this._lastResult;
				this._submitButton.disabled = true;
				if (!this._configurationSpacePicker) {
					this._showLoading(true);
				}
				this._resultsTable.clearBody();
				this._dimensionsTable.clearBody();
				
				if (searchString.length == 0) {
					if(this.defaultSearchTerm){
						searchString = this.defaultSearchTerm;
					} else {
						this._updateCount(0);
						return;
					}
				}

				var self = this;
				
				var content = {
					query: searchString
				};

				if (this._spaceUri){
					content.space = this._spaceUri;
				}
				
				if(this._validTypes){
					content.validTypes = this._validTypes ;
				}
				
				if (this._findParentsOfUri) {
					content.uri = this._findParentsOfUri;
				}
				
				var params = dojo.objectToQuery(content);
				
				var serviceName;
				
				if (this._pureGlobalConfigurationsOnly || this._mutableGlobalConfigurationsOnly) {
					serviceName = "globalConfigurations/queryForTitle";
				} else if (this._findParentsOfUri) {
					serviceName = "globalConfigurations/findExpandedContexts";
				} else {
					serviceName = "WorkspacesQuery/query";
				}
				
				if (params) {
					params = "?" + params;
					if (this._mutableGlobalConfigurationsOnly) {
						params = params + "&mutable=true";
					}
				}
				
				var args = {
						handleAs: 'json',
						url: VvcUtils.getVvcUri() + "/" + serviceName + params, //$NON-NLS-1$
						"headers": { //$NON-NLS-1$
							"Content-Type" : "text/plain", //$NON-NLS-1$ //$NON-NLS-2$
						},
						handle: dojo.hitch(this, this._handleResponse)
				};
				
				var deferred = jazz.client.xhrGet(args);  
			},
			
			_handleResponse: function(response, ioargs) {
				this._showLoading(false);
				this._deselectAll();
				dojo.removeClass(this._searchIcon, 'loading');
				
				if (ioargs.xhr.status == 200)
				{
					if ("errorMessage" in response) //TODO: This is to be modified to whatever the error is supposed to be in the error json format
						this.displayErrorMessage(response.errorMessage);
					else  
						this._handleLoad(response);
				}
				else {
					console.log('error', response);
					this.displayErrorMessage("Invalid Response (Status:"+ioargs.xhr.status+"): "+ response.responseText);
				}
			},
			
			displayErrorMessage: function(errorMessage) 
			{
				var mb = new jazz.ui.MessageBox();
				mb.placeAt(dojo.byId("errorMessages"));
				mb.showMessage({summary: errorMessage, type: jazz.ui.MessageBox.STATUS_TYPES.ERROR});
				dojo.style("errorMessages", {display: ""});
			},
			
			hideErrorMessage: function() {
				dojo.empty("errorMessages");
				dojo.style("errorMessages", {display: "none"});
			},
			
			_handleLoad: function(results) {
				var uris = results.resultUris;
				var frag = document.createDocumentFragment();

				var missingIcon = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/missing_icon.gif');
								
				var tr, td;
				for (var i = 0; i < uris.length; i++) {
					dojo.when(this._cache.get(uris[i]), dojo.hitch(this, function(workspace){
						tr = document.createElement("tr");
						
						td = document.createElement("td");
						td.appendChild(dojo.create('img', {
							src: ((workspace.GLOBAL) || (this._spaceUri && workspace.spaceUri !== this._spaceUri)) ?
								dojo.moduleUrl('com.ibm.team.vvc.shared.web', workspace.mutable ? 'graphics/globalconfiguration_mutable.gif' : 'graphics/globalconfiguration_immutable.gif') :
								this._getImgURL(workspace),
							height: '16',
							width: '16'
							}));
						tr.appendChild(td);
						
						td = document.createElement("td");
						var titleNode = document.createTextNode(workspace.title);
						td.appendChild(titleNode);
						tr.appendChild(td);
						
						td = document.createElement("td");
						td.innerHTML = jazz.util.html.escape(workspace.description);
						tr.appendChild(td);
					
						tr.workspace = workspace;
						frag.appendChild(tr);
	
						this._lastResult = workspace;
					
						this._resultsTable.tbody.appendChild(frag);
					}));
				}
				this._updateCount(uris.length);
			},
			
			_getImgURL: function(workspace) {
				if (workspace.type == 'snapshot')
					return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/snapshot.gif');
				else if (workspace.type == 'changeset')
				{	
					if (workspace.mutable)
						return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/changeset.gif');
					else
						return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/changeset_complete.gif');
				}
				return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/workspace.gif');
			},
			
			_onSubmit: function(e) {
				e && dojo.stopEvent(e);

				if (this._selectedRow) {
					if (this._remoteFrame) {
						var workspace = this._selectedRow.workspace;
						
						var oslcResponse = {
							"oslc:results" : [{
								"oslc:label" : workspace.title,
								"rdf:resource" : workspace.uri
							}]
						};
						
						this._remoteFrame.sendResponse(dojo.toJson(oslcResponse));

						return;
					}
					
					// There are selected result, so OK the dialog.
					// Can get selected result still with getter until this is destroyed.

					if (this.onOk) {
						// get off the DOM before destroy
						var workspace = this._selectedRow.workspace;
						var self = this;

						// Delay the callback until after the 'destroy' so that the dialog goes away
						window.setTimeout(function() {
							self.onOk(workspace);
						}, 1);
						
					}
					
					if(this.closeable != false){
						this.destroyRecursive();
					}
				}
			},
			
			_getFirstRow: function(table) {
				if(table.domNode){
					table = table.domNode;
				}
				var children = table.childNodes;
	
				for (var i = 0; i < children.length; i++) {
					var child = children[i];
					if (child.nodeName == 'TR')
						return child;
				}
				return null;
			},
			
			_getLastRow: function(table) {
				if(table.domNode){
					table = table.domNode;
				}
				var children = table.childNodes;
				for (var i = children.length - 1; i > -1; i--) {
					var child = children[i];
					if (child.nodeName == 'TR')
						return child;
				}
				return null;
			},
			
			hideOkCancelButtons: function(){
				dojo.addClass(this.okCancelButtons, "hidden");
				dojo.removeClass(this.okCancelButtons, "visible-block");
			},
			
			showOkCancelButtons: function(){
				dojo.addClass(this.okCancelButtons, "visible-block");
				dojo.removeClass(this.okCancelButtons, "hidden");
			}
			
		}
	);
})();
