/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {
	dojo.provide('com.ibm.team.vvc.shared.web.ui.picker.WorkspacePickerDialog');

	dojo.require('jazz.ui.Dialog');
	dojo.require('com.ibm.team.vvc.shared.web.ui.picker.WorkspacePicker');
	
	dojo.require('dojo.i18n');
	dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.picker", "WorkspacePickerDialogMessages");

	dojo.declare('com.ibm.team.vvc.shared.web.ui.picker.WorkspacePickerDialog', [dijit._Widget, dijit._Templated], {
				
		messages: dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.picker", "WorkspacePickerDialogMessages"),
		
		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'ui/picker/templates/WorkspacePickerDialog.html'),
				
		dialogTitle: "",
		
		pickerToggleDetailsHandle : null,
		pickerSubmitHandle: null,
		pickerOnOkHandle: null,
		pickerOnCancelHandle: null,
		pickerOnSelectHandle: null,
		onShowDimensions: null,
		onHideDimensions: null,
		
		workspacePicker: null,
		
		_destroys: null,
		_disconnects: null,
		
		constructor: function(args){
			this._destroys = [];
			this._disconnects = [];
			
			this.workspacePicker = new com.ibm.team.vvc.shared.web.ui.picker.WorkspacePicker(args);
			this._destroys.push(this.workspacePicker);
		},
		
		destroy: function(){
			dojo.forEach(this._destroys, dojo.destroyRecursive(obj));
			dojo.forEach(this._disconnects, dojo.disconnect(handle));
		},
		
		postCreate: function(){
			this.inherited(arguments);
			if (!this.dialogTitle) {
				this.dialogTitle = this.messages.dialogTitle;
			}
			this._dialog.setWidth("478px");
			this._dialog.setTitle(this.dialogTitle);
			this.content.appendChild(this.workspacePicker.domNode);
			if (this.iconPath) {
				this._dialog.setIcon(this.iconPath);
			}
						
			// Hook into the showDetails to make sure we're the correct size.
			this._disconnects.push(dojo.connect(this.workspacePicker, "onHideDimensions", dojo.hitch(this, function(){
				this._dialog.setWidth("478px");
				if(this.onHideDimensions){
					this.onHideDimensions();
				}
			})));
			
			this._disconnects.push(dojo.connect(this.workspacePicker, "onShowDimensions", dojo.hitch(this, function(){
				this._dialog.setWidth("760px");
				if(this.onShowDimensions){
					this.onShowDimensions();
				}
			})));
						
			// Hook into the onSubmit to make sure we properly dispose of the popup widget
			this._disconnects.push(dojo.connect(this.workspacePicker, "_onSubmit", dojo.hitch(this, function(){
				// Prevent the onClose callback since we already know it's happening in this case
				delete this.onClose;
				this._dialog.close();
			})));
			
			this._disconnects.push(dojo.connect(this.workspacePicker, "cancel", dojo.hitch(this, function(){
				this._dialog.close();
			})));
			
			/*
			 * Need to make sure the onOk and onCancel events propogate properly.
			 */
			this._disconnects.push(this.pickerOnOkHandle = dojo.connect(this.workspacePicker, "onOk", this, function(workspace, isGlobalConfiguration){
				if(this.onOk){
					this.onOk(workspace, isGlobalConfiguration);
				}
			}));
			
			this._disconnects.push(dojo.connect(this.workspacePicker, "onCancel", this, function(args){
				if(this.onCancel){
					this.onCancel(args);
				}
			}));
		},
		
		onClose: function(){
			this.inherited(arguments);
			this.workspacePicker.onCancel && this.workspacePicker.onCancel();
		},
		
		
		onOk: null,
		
		onCancel : null
		
	});
})();