/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.ui.properties.AdvancedTable");

dojo.require("dijit._Widget");
dojo.require("dojo.i18n");

dojo.require("dijit.form.CheckBox");

dojo.require("rmps.app.core.util");

dojo.require("jazz.ui.StyledTable");
dojo.require("jazz.ui.ResourceLink");

dojo.requireLocalization("rmps.app.ui.properties", "AdvancedTableMessages");
	
var messages = dojo.i18n.getLocalization("rmps.app.ui.properties", "AdvancedTableMessages");

dojo.declare("rmps.app.ui.properties.AdvancedTable", dijit._Widget, {
	
	"class": "rmps-app-ui-properties-advancedTable",
	
	// mixin properties
	/*
	 * tableData: [
	 *                {
	 *                    label: "",
	 *                    description: "",
	 *                    category: "",
	 *                    value: { // or array
	 *                               label: "",
	 *                               uri: "" || null
	 *                           }
	 *                }
	 *            ]
	 */
	tableData: null,
	
	// private properties
	_table: null,
	
	postCreate: function() {
		this.containerNode = this.domNode;
		
		var columns = [
			{
				label: messages.column_property,
				width: "50%"
			},
			{
				label: messages.column_value,
				width: "50%"
			}
		];
		
		this._table = new jazz.ui.StyledTable({
			columns: columns	
		});
		this._table.containerNode = this._table.domNode;
		
		if (this.tableData) {
			
			var lastCategory = null;
			
			for (var i = 0, data; data = this.tableData[i]; ++i) {
				// assume data is ordered by category from the server
				if (data.category !== lastCategory) {
					// add category
					this._table.tbody.appendChild(this._createCategoryRow(data.category));
					lastCategory = data.category;
				}
				this._table.tbody.appendChild(this._createRow(data));
			}
		}
		
		if (this._hasEmpty) {
			this._addShowEmptyCheckbox();
		}
		
		this.domNode.appendChild(this._table.domNode);
	},
	
	_isShowEmpty: function() {
		var value = dojo.cookie(this.sectionId);
		if (value) {
			return dojo.fromJson(value).showEmpty || false;
		}
		return false;
	},
	
	_saveShowEmpty: function(showEmpty) {
		var value = dojo.cookie(this.sectionId), cookie;
		if (value) {
			cookie = dojo.fromJson(value);
		} else {
			cookie = {};
		}
		cookie.showEmpty = showEmpty;
		dojo.cookie(this.sectionId, dojo.toJson(cookie), {expires: 90});
	},
	
	_addShowEmptyCheckbox: function() {
		var container = dojo.create("div", {className: "header"}, this.domNode);
		
		var showEmpty = this._isShowEmpty();
		
		var cb = new dijit.form.CheckBox({
				checked: showEmpty,
				onChange: dojo.hitch(this, function(checked) {
					this._saveShowEmpty(checked);
					dojo[checked ? "removeClass" : "addClass"](this.domNode, "hideEmpty");
				})
		});
		container.appendChild(cb.domNode);
		
		dojo.create("label", {"for": cb.id}, container).appendChild(dojo.doc.createTextNode(messages.showEmpty));
		
		// reset cookie expiration
		this._saveShowEmpty(showEmpty);
		dojo[showEmpty ? "removeClass" : "addClass"](this.domNode, "hideEmpty");
		
		container = showEmpty = cb = null;
	},
	
	_createCategoryRow: function(category) {
		var tr = dojo.create("tr");
		var categoryCell = dojo.create("td", {"class": "category"}, tr);
		categoryCell.appendChild(dojo.doc.createTextNode(category));
		
		var emptyCell = dojo.create("td", null, tr);
		emptyCell.innerHTML = "&nbsp;";
		
		return tr;
	},
	
	_createRow: function(rowData) {
		var tr = dojo.create("tr");
		var propertyCell = dojo.create("td", {"class": "property", title: rowData.description}, tr);
		var valueCell = dojo.create("td", {"class": "value"}, tr);
		
		propertyCell.appendChild(dojo.doc.createTextNode(rowData.label));
		
		var value = rowData.value;
		
		var isEmpty = false;
		if (value) {
			if (dojo.isArray(value)) {
				if (value.length > 0) {
					for (var i = 0; i < value.length; ++i) {
						if (i != 0) {
							valueCell.appendChild(dojo.create("br"));
						}
						this._populateValueCell(valueCell, value[i]);
					}
				} else {
					valueCell.innerHTML = "&nbsp;";
					isEmpty = true;
				}
			} else {
				isEmpty = !this._populateValueCell(valueCell, value);
			}
		} else {
			valueCell.innerHTML = "&nbsp;";
			isEmpty = true;
		}
		
		if (isEmpty) {
			dojo.addClass(tr, "empty");
			this._hasEmpty = true;
		}
		
		return tr;
	},
	
	_populateValueCell: function(valueCell, value) {
		if (value.uri) {
			var presentationUri = rmps.app.core.util.getViewResourceUri(value.uri);
			var linkNode = dojo.create("a", {href: presentationUri}, valueCell);
			linkNode.appendChild(dojo.doc.createTextNode(value.label));
			
			new jazz.ui.ResourceLink({
						retainLinkText: true, 
						lazyFetch: true, 
						uri: net.jazz.ajax._contextRoot + "/rendering?resourceUri=" +  encodeURIComponent(value.uri), 
						presentationUri: presentationUri
					}, linkNode);
			
		} else if (value.label != null && value.label.length > 0) {
			valueCell.appendChild(dojo.doc.createTextNode(value.label));
			
		} else {
			valueCell.innerHTML += "&nbsp;";
			return false;
		}
		return true;
	}
});

})();