/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.team.vvc.shared.web.ui.util.TeamAreaDialogContent");
    
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("dijit.tree.ForestStoreModel");
    dojo.require("dijit.Tree");
    dojo.require("dojo.data.ItemFileReadStore");
    
    dojo.require("com.ibm.team.vvc.shared.web.ui.util.InfoContent");
	dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController");
	
    dojo.require("com.ibm.team.vvc.shared.web.access.AccessUtils");
    var AccessUtils = com.ibm.team.vvc.shared.web.access.AccessUtils;
       
	dojo.require("com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory");
	var NotificationFactory = com.ibm.team.vvc.shared.web.ui.notifications.notifications.VvcNotificationFactory;
    
    dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.util", "TeamAreaDialogContentMessages");
    var Messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.util", "TeamAreaDialogContentMessages");
    
    dojo.require("com.ibm.team.vvc.shared.web.ui.util.VvcUtils");
    var VvcUtils = com.ibm.team.vvc.shared.web.ui.util.VvcUtils;
    
    /**
     * Constructor params: title, uri, spaceUri, okFunction
     * title: configuration title
     * uri: configuration URI
     * spaceUri: configuration space URI
     * okFunction: function to be called when OK is pressed
     */
    dojo.declare("com.ibm.team.vvc.shared.web.ui.util.TeamAreaDialogContent", com.ibm.team.vvc.shared.web.ui.util.InfoContent, {
		okFunction: null, //from params
		title:null,
		uri:null,
		spaceUri:null,
		notificationCenter:null,
		
        constructor: function(params){
        	this.okFunction = params.okFunction;
        	this.title = params.title;
        	this.uri = params.uri;
        	this.spaceUri = params.spaceUri;
        	this.contentNode = dojo.string.substitute(Messages.dialogMessage, {configurationTitle: this.title});
			this.notificationCenter = new com.ibm.team.vvc.shared.web.ui.notifications.notificationCenterController.VvcNotificationCenterController();
        },
        
        
        postCreate: function() {
			this.informationCenterNode.appendChild(this.notificationCenter.domNode);

			var cancelButton = dojo.create("button", {id: "cancelButton"}, this._buttonSection);
			cancelButton.innerHTML = Messages.cancel;
			dojo.addClass(cancelButton, "Button");
			dojo.connect(cancelButton, "onclick", dojo.hitch(this, function(event){
				this.onCancel();
			})); 

			var deferredConfigurationOwner = AccessUtils.getConfigurationOwner(this.uri);
			var context = {
				self: this
			};
       	
			deferredConfigurationOwner.addCallback(context, function(result){
	
				var deferredSecurityContext =  AccessUtils.getSpaceContext(this.self.spaceUri);

				var context = {
					configurationOwner: result,
					self: this.self
				};            
				deferredSecurityContext.addCallback(context, function(result){
	
					var deferredTeamAreas = AccessUtils.getTeamAreasFromRootServices(VvcUtils.getVvcUri() + "/rootservices", result);
					var context = {
						self: this.self,
						configurationOwner: this.configurationOwner
					};
					
					deferredTeamAreas.addCallback(context, function(result){
  
						var storeData = {
							identifier: "uri",
							label: "name",
							items: result
						};
  
						var treeModel = new dijit.tree.ForestStoreModel({
							store: new dojo.data.ItemFileReadStore({data: storeData}),
							query: {"type": "projectArea"},
							rootId: "root",
							rootLabel: "Hidden root element",
							childrenAttrs: ["teamAreas"]
						});

						var deferredTeamAreasTree = dojo.create("div", {id: "deferredTeamAreasTree"}, this.self._dialogContent);
						dojo.style("deferredTeamAreasTree", {"max-height":"250px"}); 

						var myTree = new dijit.Tree({
							getIconClass: dojo.hitch(this, function (treeItem) { 
								if (treeItem.type && (treeItem.type[0] === "projectArea")) {
									return /*(treeItem.uri && (treeItem.uri[0] === "null")) ? "removeProjectAreaNode" :*/ "projectAreaNode";
								} else if (treeItem.type && (treeItem.type[0] === "teamArea")) {
									return "teamAreaNode";
								}
							}),
							model: treeModel,
							showRoot: false //avoid displaying the hidden root element
						}, deferredTeamAreasTree);
						myTree.startup();
    
						this.self._childWidgets = [];
						this.self._childWidgets.push(myTree);
    
						myTree.dndController.singular = true;
    
						var context = {
							self: this.self,
							configurationOwner: this.configurationOwner,
							result: result,
							tree: myTree
						};

						dojo.connect(myTree, 'onLoad', dojo.hitch(context, function() {
							var path = this.self._find(this.result, this.configurationOwner);
							this.tree.set('paths', [ path ] );
						}));
					});
				});
			});
        },
        
        _getIconClass: function(treeItem) {
        	if (treeItem.type === "projectArea") {
        		return (treeItem.uri === "null") ? "removeProjectAreaNode" : "projectAreaNode";
        	} else if (treeItem.type === "teamArea") {
        		return "teamAreaNode";
        	}
        },
        
        _find: function(result, uri) {
        	var path = [];
        	for (var i = 0; i < result.length; i++) {
        		if (result[i].uri && (result[i].uri[0] === uri)) {
                	path.push("root");
        			path.push(uri);
        			return path;
        		} else if (result[i].uri && result[i].uri[0]) {
        			if (this._findTeamArea(result[i].teamAreas, uri, path)) {
        				path.push(result[i].uri[0]);
                    	path.push("root");
        				return path.reverse();
        			}
        		}
        	}
        	path.push("root");
        	path.push(result[0].uri[0]); //it should always have at least the project area, so result[0] will not be undefined
        	return path;
        },
        
        _findTeamArea: function(result, uri, path) {
        	for (var i = 0; i < result.length; i++) {
        		if (result[i].uri && (result[i].uri[0] === uri)) {
        			path.push(uri);
        			return uri;
        		} else if (result[i].uri && result[i].uri[0]) {
        			var teamArea = this._findTeamArea(result[i].teamAreas, uri, path);
        			if (teamArea) {
        				path.push(result[i].uri[0]);
            			return result[i].uri[0];
        			}
        		}
        	}
        },
        
        _handleOk: function(arguments) {
        	var selectedNode = dijit.byId("deferredTeamAreasTree").selectedNode;
        	
        	if (this.okFunction) {
        		var deferred = this.okFunction(selectedNode.item.uri, selectedNode.item.type);
        		deferred.addCallback(this, function(result){
        			this.onCancel();
        		});
        		deferred.addErrback(this, function(result) {
					var notification = NotificationFactory.createError({
						title: Messages.errorConfigurationAccess,
						description: Messages["errorConfigurationAccess.explanation"],
						solution: Messages["errorConfigurationAccess.useraction"]
					}).dispatch();        			
        		});
        	} else {
            	this.onCancel();
        	}
        }
    });
    
})();
