/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.user.UserClient");

dojo.require("rmps.core.user.User");

dojo.require("rmps.core.xhr");
dojo.require("jazz.client.xhr");

dojo.require("dojox.xml.DomParser");

dojo.require("rmps.core.RootServices");

var USERS_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.RMPS, "dm-users");
var JFS_USERS_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.JFS, "users");
//var WHOAMI_PATH = rmps.core.RootServices.getUri(rmps.core.Namespaces.JFS, "currentUser");
	
rmps.core.user.UserClient = {
	
	getUsers: function(projectUris) {
		var query;
		if (projectUris) {
			query = { project: projectUris };
		}
		var args = {
			url: USERS_PATH + (query ? '?' + dojo.objectToQuery(query) : ""),
			handleAs: "text",
			headers: {
				Accept: "application/rdf+xml",
				"X-ibm-rmps-internal": "true"
			},
			load: function(response, ioArgs)
			{
				var doc = dojox.xml.DomParser.parse(response);
				var users = [];
				var entries = doc.getElementsByTagName("rdf:RDF");
				for (var i = 0; i < entries.length; i++) {
					var user = new rmps.core.user.User();
					user.parse(entries[i]);
					users.push(user);
				}
				return users;
			}
		};
		return jazz.client.xhrGet(args);
	},
	
	getUser: function(userUri) {
		var args = {
			url: userUri,
			handleAs: "text",
			headers: {
				Accept: "application/rdf+xml",
				"X-ibm-rmps-internal": "true"
			},
			load: function(response, ioArgs)
			{
				var user = new rmps.core.user.User();
				user.parse(dojox.xml.DomParser.parse(response));
				return user;
			},
			cacheKey: null
		};
		return rmps.core.xhrGet(args);
	},
	
	getUserById: function(userId) {
		return this.getUser(this.getUserUri(userId));
	},

	// This function encodes the userId if it does not yet contain '%' (meaning is not yet encoded) to build a valid user URI.
	getUserUri: function(userId) {
		var encodedUserId = userId;
		var idx = userId.indexOf('%');
		
		if (userId.indexOf('%') == -1){
			encodedUserId = encodeURIComponent(userId);
		}
		return JFS_USERS_PATH + '/' + encodedUserId;
	},

	// TODO: Deprecate and remove this function as getUserUri returns now an encoded Uri
	getUserEncodedUri: function(userId) {
		return JFS_USERS_PATH + '/' + encodeURIComponent(userId);
	}
//	,
//	
//	whoAmI: function() {
//		var args = {
//			url: WHOAMI_PATH,
//			handleAs: "text",
//			load: function(response, ioArgs)
//			{
//				var user = new rmps.core.user.User();
//				user.parse(dojox.xml.DomParser.parse(response));
//				return user;
//			}
//		};
//		return jazz.client.xhrGet(args);
//	}
};

})();