/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.user.UserInfo");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dojo.html");

dojo.require("rmps.core.util.EmailLinkProperties");
dojo.require("rmps.core.error");
dojo.require("rmps.core.user.UserClient");

dojo.requireLocalization("rmps.ui.common", "CommonMessages");
dojo.requireLocalization("rmps.ui.user", "UserInfoMessages");

var UserClient = rmps.core.user.UserClient;

dojo.declare("rmps.ui.user.UserInfo", [dijit._Widget, dijit._Templated], {
	
	templateString: dojo.cache("rmps.ui", "user/templates/UserInfo.html"),
	
	messages: dojo.i18n.getLocalization("rmps.ui.user", "UserInfoMessages"),
	commonMessages: dojo.i18n.getLocalization("rmps.ui.common", "CommonMessages"),
	
	// attach points
	photoNode: null,
	nameNode: null,
	emailNode: null,
	photoWrapper: null,
	
	// properties
	user: null,
	photoEnlargeDelay: 400,
	emailProperties: null,
	
	// private properties
	_photoOpenTimer: null,
	_isPhotoLarge: false,
	_moveHandle: null,
	
	postCreate: function() {
		if (this.emailProperties == null) {
			this.emailProperties = new rmps.core.util.EmailLinkProperties();
		}
		this.connect(this.emailProperties, "linkChanged", dojo.hitch(this, this._updateMailToLink));
		
		// connect photo events
		this.connect(this.photoWrapper, "onclick", this._enlargePhoto);
		this.connect(this.photoWrapper, "onmouseleave", function() {
			this.disconnect(this._moveHandle);
			this._shrinkPhoto();
		});
		this.connect(this.photoWrapper, "onmouseenter", function() {
			if (!this._isPhotoLarge) {
				this._moveHandle = this.connect(this.photoWrapper, "onmousemove", this._delayEnlargePhoto);
				this._delayEnlargePhoto();
			}
		});
		
		if (this.user) {
			this._setUser(this.user);
		}
	},
	
	_delayEnlargePhoto: function() {
		// reset timeout
		if (this._photoOpenTimer) {
			clearTimeout(this._photoOpenTimer);
		}
		this._photoOpenTimer = setTimeout(dojo.hitch(this, function() {
			this._enlargePhoto();
		}), this.photoEnlargeDelay);
	},
	
	setUser: function(user) {
		this.user = user;
		this._setUser(user);
	},
	
	_setUser: function(user) {
		this.content.style.display = "none";
		this.loading.style.display = "";
		
		if (dojo.isString(user)) {
			var deferred = UserClient.getUser(user);
			deferred.addCallback(this, this._updateUserInfo);
			deferred.addErrback(function(error) {
				if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
					rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
				} else {
					rmps.core.error.createError(error).dispatch();
				}
			});
		} else {
			this._updateUserInfo(user);
		}
	},
	
	_updateUserInfo: function(user) {
		this.user = user;
		
		this.userIdNode.innerHTML = "";
		this.userIdNode.appendChild(dojo.doc.createTextNode(user.id));
		this.nameNode.innerHTML = "";
		this.nameNode.appendChild(dojo.doc.createTextNode(user.name));
		
		this.photoNode.src = user.photoUri || dojo.moduleUrl("rmps.ui", "graphics/no-photo.png");
		
		this.emailNode.innerHTML = "";
		// email
		if (dojo.isIE) {
			// workaround to bug https://connect.microsoft.com/IE/feedback/ViewFeedback.aspx?FeedbackID=364561
			this.emailNode.appendChild(dojo.doc.createElement("span"));
		}
		this.emailProperties.addTo(user.email);
		this.emailNode.appendChild(dojo.doc.createTextNode(user.email));
		
		this.loading.style.display = "none";
		this.content.style.display = "";
	},
	
	_updateMailToLink: function(link) {
		this.emailNode.href = link;
	},
	
	_enlargePhoto: function() {
		if (this._photoOpenTimer) {
			clearTimeout(this._photoOpenTimer);
			delete this._photoOpenTimer;
		}
		if (!this._isPhotoLarge) {
			dojo.addClass(this.photoWrapper, "photo-large");
			dojo.removeClass(this.photoWrapper, "photo-small");
			this._isPhotoLarge = true;
		}
	},
	
	_shrinkPhoto: function() {
		if (this._photoOpenTimer) {
			clearTimeout(this._photoOpenTimer);
			delete this._photoOpenTimer;
		}
		if (this._isPhotoLarge) {
			dojo.addClass(this.photoWrapper, "photo-small");
			dojo.removeClass(this.photoWrapper, "photo-large");
			this._isPhotoLarge = false;
		}
	}
});

})();