/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.ui.user.UserLabel");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dijit.Menu");
dojo.require("dijit.form.Button");

dojo.require("rmps.core.error");
dojo.require("rmps.core.user.UserClient");

dojo.require("rmps.ui.user.UserInfo");

dojo.require("jazz.ui.HoverPopup");

var UserClient = rmps.core.user.UserClient;

dojo.declare("rmps.ui.user.UserLabel", [dijit._Widget, dijit._Templated], {
	
	templateString: '<span class="rmps-web-ui-user-userLabel">' +
			'<a href="" dojoAttachPoint="linkNode" class="labelLink" dojoAttachEvent="onclick:_toggleInfo"></a>' +
			'</span>',
	
	// attach points
	linkNode: null,
	
	// properties
	user: null,
	
	// private properties
	_userInfo: null,
	_closeTimer: null,
	_closeHandles: null,
	
	// FIXME this is used as a workaround to get the user info dialog to appear
	// above the hover popup as well as to prevent the hover popup from disappearing when
	// the user mouses over the user info.
	_hoverPopupParent: null,
	
	postCreate: function() {
		if (this.user) {
			this._setUser(this.user);
		}
	},
	
	setUser: function(user) {
		this.user = user;
		this._setUser(user);
	},
	
	_setUser: function(user) {
		this.linkNode.innerHTML = "";
		if (dojo.isString(user)) {
			var deferred = UserClient.getUser(user);
			deferred.addCallback(this, this._updateUser);
			deferred.addErrback(function(error) {
				if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
					rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
				} else {
					rmps.core.error.createError(error).dispatch();
				}
			});
		} else {
			this._updateUser(user);
		}
	},
	
	destroy: function() {
		if (this._userInfo) {
			this._userInfo.destroyRecursive();
			delete this._userInfo;
		}
		this.inherited(arguments);
	},
	
	_updateUser: function(user) {
		this.user = user;
		this.linkNode.appendChild(dojo.doc.createTextNode(user.name));
	},
	
	_toggleInfo: function(e) {
		dojo.stopEvent(e);
		if (this._infoOpen) {
			this._closeInfo();
		} else {
			this._openInfo();
		}
	},
	
	initUserInfo: function(userInfo) {
		
	},
	
	_openInfo: function(e) {
		if (!this._userInfo) {
			this._userInfo = new rmps.ui.user.UserInfo({
				user: this.user
			});
			this.initUserInfo(this._userInfo);
			
			dojo.addClass(this._userInfo.domNode, "rmps-web-ui-user-userLabel-userInfo");
			
			dijit.popup.moveOffScreen(this._userInfo.domNode);
			this._userInfo.startup();
		}
		
		this._closeHandles = [];
		this._closeHandles.push(this.connect(this._userInfo.domNode, "onmouseout", this._checkCloseInfo));
		this._closeHandles.push(this.connect(this.linkNode, "onmouseout", this._checkCloseInfo));
		
		this._hoverPopupParent = this._getHoverPopupParent();
		if (this._hoverPopupParent) {
			this._hoverPopupParent.programmatic = true;
		}
		
		dijit.popup.open({
			popup: this._userInfo,
			parent: this,
			around: this.linkNode,
			onExecute: dojo.hitch(this, this._closeInfo),
			onCancel: dojo.hitch(this, this._closeInfo),
			orient: {'BL': 'TL', 'BR': 'TR', 'TL': 'BL', 'TR': 'BR' }
		});
		
		if (this._hoverPopupParent) {
			// fix to get the menu to appear above the hover popup.
			this._userInfo.domNode.parentNode.style.zIndex = 11000;
		}
		
		this._infoOpen = true;
	},
	
	_closeInfo: function() {
		if (this._userInfo && this._infoOpen) {
			for (var i = 0; i < this._closeHandles.length; ++i) {
				this.disconnect(this._closeHandles[i]);
			}
			delete this._closeHandles;
			
			this._userInfo._shrinkPhoto();
			
			if (this._hoverPopupParent) {
				this._hoverPopupParent.programmatic = false;
			}
			dijit.popup.close(this._userInfo);
			this._infoOpen = false;
		}
	},
	
	onBlur: function() {
		if (this._userInfo && this._infoOpen) {
			this._closeInfo();
		}
	},
	
	_checkCloseInfo: function(e) {
		if (dojo.isDescendant(e.relatedTarget, this._userInfo.domNode) || 
				dojo.isDescendant(e.relatedTarget, this.linkNode)){
			// do not close, user moved between link and popup
			return;
		}
		if (this._hoverPopupParent && !dojo.isDescendant(e.relatedTarget, this._hoverPopupParent.domNode)) {
			this._hoverPopupParent.programmatic = false;
			this._hoverPopupParent._unHover(e);
		}
		this._closeInfo();
	},
	
	_getHoverPopupParent: function() {
		var parent = this.domNode;
		while ((parent = parent.parentNode) != null && parent.nodeType == 1) {
			var widget = dijit.byNode(parent);
			if (widget && widget instanceof jazz.ui.internal._MasterPopup) {
				return widget._showing;
			}
		}
		return null;
	}
});

})();