/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.util.BaselineUtil");

dojo.require("rmps.core.baseline.BaselineClient");
dojo.require("rmps.core.error");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmps.core.util.xhrUtil");
dojo.require("rmps.core.util.ContextUtils");

var BaselineClient = rmps.core.baseline.BaselineClient;
var IRmpsConstants = rmps.core.util.IRmpsConstants;
var xhrUtil= rmps.core.util.xhrUtil;
var ContextUtils= rmps.core.util.ContextUtils;

rmps.core.util.BaselineUtil = {
	
	/**
	 * Can pass null to reset to the current data.
	 */
	setBaseline: function(baselineUri) {
		if (baselineUri == null) {
			//we can alway reset to current data
			this._internalSetBaseline(baselineUri);		
		} else {
			//make sure this is a valid baseline URI before switching to it
			var deferred = BaselineClient.getBaseline(baselineUri);
			deferred.addCallback(this, function(args) {
				//if request returns successfully, the baseline is valid
				this._internalSetBaseline(baselineUri);
			});
			deferred.addErrback(this, function(error) {
				//baseline doesn't exist; report it, don't switch context
				if (!dojo._isDocumentOk(deferred.ioArgs.xhr)) {
					rmps.core.error.createXhrError(deferred.ioArgs).dispatch();
				} else {
					rmps.core.error.createError(error).dispatch();
				}
			});
		}
	},
	
	_internalSetBaseline: function(baselineUri) {
		var context = jazz.app.context.get();			
		context.baseline = baselineUri;
		window.location.search = ContextUtils.getQueryForContext(baselineUri);
	},
	
	/**
	 * Returns the URI that should be set in the current window to switch to the given
	 * baseline (may be null to indicate live data).
	 */
	getBaselineUri: function(baselineUri){
		return ContextUtils.getQueryForContext(baselineUri) + location.hash;
	},
	
	/**
	 * Add baseline paramater for a given args to a client call
	 * 	
	 */
	addBaselineParamater: function(args, baseline, overwrite, checkForNull) {
		if(!baseline&& checkForNull){
			return url;
		}
		var params=this.getBaselieAsParamater(baseline);
		return xhrUtil.addParamater(args,params);
	},
	
	/**
	 * Add baseline paramater for a given url (an instance of string)
	 * 	
	 */
	addBaselineParamaterToUrl: function(url, baseline, overwrite, checkForNull) {
		if(!baseline&& checkForNull){
			return url;
		}
		if(!dojo.isString(url)){
			return url;
		}
		var params=this.getBaselieAsParamater(baseline);
		return xhrUtil.addParamaterToUrl(url,params, overwrite);
	},
	
	getBaselieAsParamater: function(baseline){
		var params={};
		if(baseline){
			params[IRmpsConstants.BASELINE]= baseline;
		}else{	/*baseline is optional */
			params[IRmpsConstants.BASELINE]= jazz.app.context.get().baseline;			
		}
		return params;
	}
	
};

})();