/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.common.util.ChangeUtil");

rmpw.common.util.ChangeUtil = {
	
	valueEqual : function(originalValue,newValue) {
		//Required for the unset fields versus unset controls (such as text boxes) as we can have values like undefined versus NaN
		if(originalValue === undefined && newValue === undefined){
			return true;
		}
		
		if(typeof originalValue=== "number" || typeof newValue=== "number"){
			//Use "==" operand to auto correct the comparison in order to return true for cases like "2" and 2
			if (originalValue === undefined) {
				if (isNaN(newValue)) {
					return true;
				}
				return false;
			}
			return originalValue == newValue;
		}
		
		// We compare dates as iso date strings
		originalValue = rmpw.common.util.ChangeUtil.checkConvertDate(originalValue);
		newValue = rmpw.common.util.ChangeUtil.checkConvertDate(newValue);		
		
        return rmpw.common.util.ChangeUtil.objectEqual(originalValue,newValue);
    },
    
    checkConvertDate : function(date) {
    	return (date instanceof Date)?dojo.date.stamp.toISOString(date, {zulu:true}):date;
    },
    
	objectEqual : function(originalValue,newValue) {
		//If originalValue equals newValue, this means that no change detected
		if(originalValue === newValue) {
			return true;
 		 }
        
		// consider undefined and null to be equivalent
		if ((originalValue === undefined || originalValue === null) &&
			(newValue === undefined || newValue === null || newValue === "")) {
			return true;
		}
		
		// add check to ensure if original value and newValue have inverse boolean evaluations
		if ((!originalValue && newValue) || (originalValue && !newValue)) {
			return false;
		}
        
        //If this condition is true, this means that both orginalValue and newValue are primitive types, and are not equal, which means that a change was
        //detected at this part.
        if(!dojo.isObject(originalValue)|| !dojo.isObject(newValue)){
        	return false;
        }
        
        
        //Getting here means that originalValue and newValue are both objects, and defined 
        //Iterate throgh the new value attributes, as we only compare the new set attributes, and no need to check for
        //all attributes in originalValue because we only set specific attrbiutes on the newValue for the updates, and not
        //necessary assign values for the others
        for (prop in newValue) {
            if (!dojo.isFunction(originalValue[prop])&& //Ignore functions. This is a safe check, as we never pass functions
            !rmpw.common.util.ChangeUtil.objectEqual(originalValue[prop],newValue[prop])){	//compare objects recursively
					return false;            	
            }
        }
        
        return true;
    }
};

}());