/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.app.util.DOMNodeUtil");

dojo.requireLocalization("rmps.app.util", "DOMNodeUtilMessages");
var DOMNodeUtilMessages = dojo.i18n.getLocalization("rmps.app.util", "DOMNodeUtilMessages");

rmps.app.util.DOMNodeUtil = {

	createRequiredNode: function() {
		var reqNode = dojo.create("span");
		reqNode.className = "required";
		reqNode.style.fontWeight= "bold";
		reqNode.style.color= "#E87820";
		reqNode.style.fontFamily= "Verdana,Geneva,Arial,Helvetica,sans-serif";
		reqNode.style.fontSize= "1em";
		reqNode.innerHTML = "*";
		reqNode.title = DOMNodeUtilMessages.requiredTitle;
		return reqNode;
	},

    getElementsByTagNameNS: function(domNode, unqualifiedTagName, namespace, qualifier) {
        if (!qualifier) {
        	var nodes = domNode.getElementsByTagName(unqualifiedTagName);
        	var nodeArray = [];
        	for (var i=0; i<nodes.length; i++) {
        		var n = nodes.item(i);
        		var xmlns = n.getAttribute("xmlns");
        		if (xmlns==namespace) {
        			nodeArray.push(n);
        		}
        	}
        	nodeArray.item = function(i) {
        		return nodeArray[i];
        	};
        	return nodeArray;
        }
        return dojo.isIE ?domNode.getElementsByTagName(qualifier + ":" + unqualifiedTagName) :
            		   domNode.getElementsByTagNameNS(namespace, unqualifiedTagName);
    },

    getAttributeNS: function(domNode, unqualifiedAttributeName, namespace, qualifier) {
        if (!domNode) {
        	return "";
        }
        return (
            dojo.isIE ?
            domNode.getAttribute(qualifier + ":" + unqualifiedAttributeName) :
            domNode.getAttributeNS(namespace, unqualifiedAttributeName));
    },

    removeChildNodes: function(element) {
    	while( element.childNodes && element.childNodes.length > 0 ) {
    		element.removeChild( element.firstChild );
       	}
    },

	calculateTextWidth: function(label,fontFamily, fontSize, fontWeight) {
		var span= this._getCalculationSpan(label,fontFamily, fontSize, fontWeight);
		var offsetWidth = parseInt(span.offsetWidth, 10);
		document.body.removeChild(span);
		return offsetWidth;
	},

	calculateTextHeight: function(label,fontFamily, fontSize, fontWeight) {
		var span= this._getCalculationSpan(label,fontFamily, fontSize, fontWeight);
		var offsetWidth = parseInt(span.offsetHeight, 10);
		document.body.removeChild(span);
		return offsetWidth;
	},

	_getCalculationSpan: function(label,fontFamily, fontSize, fontWeight){
		var span = document.createElement("span");
		if (fontFamily) {
			span.style.fontFamily = fontFamily;
		}
		if (fontSize) {
			span.style.fontSize = fontSize;
		}
		if (fontWeight) {
			span.style.fontWeight = fontWeight;
		}
		span.style.visibility= "hidden";
		document.body.appendChild(span);
		span.innerHTML = label;
		return span;
	}
};
