/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2011.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.util.DateUtils");

dojo.require("jazz.util.date");

dojo.require("dojo.i18n");

dojo.requireLocalization("rmps.core.util", "DateUtilsMessages");

var messages = dojo.i18n.getLocalization("rmps.core.util", "DateUtilsMessages");

var updaters = [];
var updateTimer = null;

rmps.core.util.DateUtils = {
	
	// today
	// pastWeek
	// past2Weeks
	// pastMonth
	// past3Months
	// past6Months
	// pastYear
	
	ranges: ["today", "pastWeek", "past2Weeks", "pastMonth", "past3Months", "past6Months", "pastYear"],
	
	rangeToString: function(range) {
		return messages[range];
	},

	rangeToDate: function(range) {
		var date = null;
		var now = new Date();
		now = new Date(now.getFullYear(), now.getMonth(), now.getDate());
		switch (range) {
			case "today":
				date = now;
				break;
			case "pastWeek":
				date = dojo.date.add(now, "week", -1);
				break;
			case "past2Weeks":
				date = dojo.date.add(now, "week", -2);
				break;
			case "pastMonth":
				date = dojo.date.add(now, "month", -1);
				break;
			case "past3Months":
				date = dojo.date.add(now, "month", -3);
				break;
			case "past6Months":
				date = dojo.date.add(now, "month", -6);
				break;
			case "pastYear":
				date = dojo.date.add(now, "year", -1);
				break;
		}
		return date;
	},
	
	getDateDiff: function(date) {
		return jazz.util.date.formatDif(date, false, true);
	},
	
	addDateDiffUpdater: function(date, func) {
		var dateString = this.getDateDiff(date);
		// do the first callback
		func(dateString);
		
		var handle = {
			'date': date,
			last: dateString,
			callback: func
		};
		updaters.push(handle);
		
		if (!updateTimer) {
			updateTimer = setInterval(dojo.hitch(this, this._updateInterval), 30000);
		}
		return handle;
	},
	
	_updateInterval: function() {
		for (var i = 0; i < updaters.length; ++i) {
			var newDateString = this.getDateDiff(updaters[i].date);
			if (newDateString != updaters[i].last) {
				updaters[i].last = newDateString;
				updaters[i].callback(newDateString, updaters[i].date);
			}
		}
	},
	
	removeDateDiffUpdater: function(handle) {
		if (handle == null) {
			return;
		}
		for (var i = 0; i < updaters.length; ++i) {
			if (updaters[i] == handle) {
				updaters.splice(i, 1);
				break;
			}
		}
		if (updaters.length == 0) {
			clearInterval(updateTimer);
			updateTimer = null;
		}
	},
	
	formatDateLabel: function(date) {
		var dateStr = dojo.date.locale.format(
			date, { formatLength:"medium", selector:"date" }
		);
		var timeStr = dojo.date.locale.format(
			date, { formatLength:"short", selector:"time" }
		);
		return (dateStr + " " + timeStr);
	},
	
	/* 
	 * Compare a given date to a date conttraints. If the constratins parameter is undefined, then we get the default contraints that means
	 * that the comparison would be between that given date and the current date constraints.
	 * @see:  rmps.core.util.DateUtils#getDefaultConstraints
	 */
	dateCompare: function(/*Date */ date, constraints, selector){
		if(!constraints){
			constraints= rmps.core.util.DateUtils.getDefaultConstraints();
		}
		
		if(!selector){
			selector= "date";			
		}
		
		var compare = dojo.date.compare;
		return constraints && ((constraints.min && compare(constraints.min, date, selector) > 0) ||
			(constraints.max && compare(constraints.max, date, selector) < 0));
	},
	
	/* 
	 * Return contraints for the current date. 
	 */
	getDefaultConstraints: function(){
		var today = new Date();
//		var today = new Date().setUTCHours(23);
		return { min:today, selector:"date",formatLength:"medium" };
	},
	
	/* 
	 *Required:
	 * 				 node, defaultValue
	 * 
	 * Optional:
	 * 				onSelection, onClose, constraints
	 * 
	 * Example:
	 * 		rmps.ui.util.DateTimePickerUtil(this.myTextBox.domNode, date, this.setmyTextBoxValue);
	 */
    showDatePicker: function(node,defaultValue, onSelection, onClose, constraints){
		var selector= "date";
		
		if(!constraints){
			constraints= rmps.core.util.DateUtils.getDefaultConstraints();
		}
		
		var PopupProto = dojo.getObject("dijit.Calendar", false);
		var picker = new PopupProto({
			onValueSelected: function(value){
				setTimeout(function(){dijit.popup.close(picker);}, 1);
				if(onSelection){
					onSelection(value);					
				}
			},
			
			_close: function(){
				if(picker){dijit.popup.close(picker);}
			},
	
			_onBlur: function(){
				this._close();
				if(picker){
					picker.destroy();
					delete picker;
				}
				this.inherited("_onBlur",arguments);
			},
			constraints: constraints,

			datePackage: "dojo.date",

			isDisabledDate: function(/*Date*/ date){
				return rmps.core.util.DateUtils.dateCompare(date, constraints);
			}
		});
		
		picker.attr('value', defaultValue);
		
		dijit.popup.open({
			popup: picker,
			orient: {'BL':'TL', 'TL':'BL'},
			around: node,
			onCancel: dojo.hitch(this, this._close),
			onClose: function(){  
				if(onClose){
					onClose();
				}
			}
		});
		
		/*Position the date picker to the passed place */
		dojo.marginBox(picker.domNode,{ w:node.offsetWidth });
		
		/*Focus the calender so, we will be able to hanlde on blur events properly */
		setTimeout(function(){picker.domNode.focus();}, 1);
		
		
		return picker;
	}
};

})();