/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmps.core.util.EmailLinkProperties");

dojo.declare("rmps.core.util.EmailLinkProperties", null, {

	_subject: null,
	_body: null,
	_to: null,
	_cc: null,
	_bcc: null,
	
	addTo: function(to) {
		if (!this._to) {
			this._to = [to];
			this._linkChanged();
		} else {
			var contains = false;
			for (var i = 0; i < this._to.length; ++i) {
				if (this._to[i] == to) {
					contains = true;
					break;
				}
			}
			if (!contains) {
				this._to.push(to);
				this._linkChanged();
			}
		}
	},
	
	removeTo: function(to) {
		for (var i = 0; i < this._to.length; ++i) {
			if (this._to[i] == to) {
				this._to.splice(i, 1);
				this._linkChanged();
				break;
			}
		}
	},
	
	clearTo: function() {
		if (this._to && this._to.length > 0) {
			this._to = null;
			this._linkChanged();
		}
	},
	
	addCc: function(cc) {
		if (!this._cc) {
			this._cc = [cc];
			this._linkChanged();
		} else {
			var contains = false;
			for (var i = 0; i < this._cc.length; ++i) {
				if (this._cc[i] == cc) {
					contains = true;
					break;
				}
			}
			if (!contains) {
				this._cc.push(cc);
				this._linkChanged();
			}
		}
	},
	
	removeCc: function(cc) {
		for (var i = 0; i < this.cc.length; ++i) {
			if (this._cc[i] == cc) {
				this._cc.splice(i, 1);
				this._linkChanged();
				break;
			}
		}
	},
	
	clearCc: function() {
		if (this._cc && this._cc.length > 0) {
			this._cc = null;
			this._linkChanged();
		}
	},
	
	addBcc: function(bcc) {
		if (!this._bcc) {
			this._bcc = [bcc];
			this._linkChanged();
		} else {
			var contains = false;
			for (var i = 0; i < this._bcc.length; ++i) {
				if (this._bcc[i] == bcc) {
					contains = true;
					break;
				}
			}
			if (!contains) {
				this._bcc.push(bcc);
				this._linkChanged();
			}
		}
	},
	
	removeBcc: function(bcc) {
		for (var i = 0; i < this.bcc.length; ++i) {
			if (this._bcc[i] == bcc) {
				this._bcc.splice(i, 1);
				this._linkChanged();
				break;
			}
		}
	},
	
	clearBcc: function() {
		if (this._bcc && this._bcc.length > 0) {
			this._bcc = null;
			this._linkChanged();
		}
	},
	
	setSubject: function(subject) {
		if (subject != this._subject) {
			this._subject = subject;
			this._linkChanged();
		}
	},
	
	setBody: function(body) {
		if (body != this._body) {
			this._body = body;
			this._linkChanged();
		}
	},
	
	getLink: function() {
		var tos = '';
		if (this._to && this._to.length > 0) {
			tos = this._to[0];
			for (var i = 1; i < this._to.length; ++i) {
				tos += ',' + this._to[i];
			}
		}
		
		var hasArgs = false;
		var args = {};
		if (this._subject) {
			hasArgs = true;
			args.subject = this._subject;
		}
		if (this._body) {
			hasArgs = true;
			args.body = this._body;
		}
		
		if (this._cc && this._cc.length > 0) {
			hasArgs = true;
			args.cc = this._cc;
		}
		if (this._bcc && this._bcc.length > 0) {
			hasArgs = true;
			args.bcc = this._bcc;
		}
		
		return "mailTo:" + encodeURIComponent(tos) + (hasArgs ? '?' + dojo.objectToQuery(args) : '');
	},
	
	_linkChanged: function() {
		this.linkChanged(this.getLink());
	},
	
	linkChanged: function(link) {
		
	}
});