/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("rmpw.common.util.FormsLauncher");

dojo.require("rmps.core.client");
dojo.require("net.jazz.ajax.BootstrapProperties");
dojo.require("rmps.core.error");

/*
 * Used to cache and manage forms information. 
 * 
 * Lazily invokes form(s). When the form is loaded once, then we can just use the generated
 * function from the forms cache. And when a change occurs to any of this forms type, we make sure
 * to remove this from from the chache. 
 * TODO: The way we invoke forms here can be simplified, in terms of the required arugments to pass.
 * This will be done after resolving: 20630: Integrate Recently Viewed elements into Design drop down menu
 * As after this WI, we will be able to get access to the current active editor input, and from it
 * we can fill in the required information, and in that case, the callers will only need to pass in
 * the types, create, and standalone parameters only.
 * 
 *  
 */
rmpw.common.util.FormsLauncher= {
	packageName: "properties",
		
	_formsCache: [/*Static variable */],
	
	/*
	 * TODO: Revisit reducing the number of provides arguments (such as editor "editor defintion uri", readOnly) after resolving:
	 * 	20630: Integrate Recently Viewed elements into Design drop down menu
	 * 
	 * As we will be able to use PageHelper and get this information directly for the current open page.
	 * 
	*/ 
	loadForm: function(args, /*Function*/ onSuccess,  /*Function*/ onFail) {
		var sginature= this.get(args);
		
		if(sginature){
			return onSuccess(sginature);
		}
		
		rmps.core.client.xhrGet({
			url: net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/dsf/codegen",
			content: { 
    			packageName: args.packageName || this.packageName,
    			formName: args.formName,
    			objectTypes: args.objectTypes,
    			readOnly: args.readOnly,
    			editor: args.editor,
    			standAlone: args.standAlone,
    			create: args.create,
    			debug: dojo.config.isDebug
    		},
			handleAs: "text",
			load:  dojo.hitch(this, function(data, ioArgs) {
				var requiredModulesHeader = ioArgs.xhr.getResponseHeader("required-modules") || "[]";
				var requiredModules = dojo.fromJson(requiredModulesHeader);
				jazz.core.loader.batch_load_async(requiredModules, dojo.hitch(this, function() {
					try {
						dojo.eval(data);
						var signature= new rmpw.common.util.FormsLauncher._FormSignature(args);
						rmpw.common.util.FormsLauncher._formsCache.push(signature);
						onSuccess(signature);
					} catch (error) {
						this._handleError(onFail, error);
					}
				}));
			}),
			
			error: dojo.hitch(this, function(error) {
				this._handleError(onFail, error);
			})
		});
	},
	
	get: function(args){
		for(var index = 0, signature; signature = this._formsCache[index]; index++) {
			if(signature.equals(args)){
				return signature;				
			}
		}
		return null;
	},
		
	/* removes all generated forms from the cache */
	invalidateCache: function(){
		this._formsCache = [];
	},
	
	/*When an ontology is updated, then make sure to remove the cached forms for it, as it will contain old generated forms */
	updateForms: function(types){
		for(var index = 0, signature; signature = this._formsCache[index]; index++) {
			if(signature.containsType(types)){
				this._formsCache.splice(index, 1);
				index=0;
			}
		}
	},
	
	_handleError: function(errorCallback, error){
		if(dojo.isFunction(errorCallback)){
			return errorCallback(error);
		}
		rmps.core.error.create(error).dispatch();	
	}
};

dojo.declare("rmpw.common.util.FormsLauncher._FormSignature", null, {
	packageName: null,
	formName: null,
	objectTypes: null,
	readOnly: null,
	editor: null,
	create: null,
	standAlone: null,
	fragment: null,
	
	constructor: function(args){
		this.packageName= args.packageName;
		this.formName= args.formName;
		
		if(!args.objectTypes || !dojo.isArray(args.objectTypes)||args.objectTypes.length<1){
			rmps.core.error.create("Cannot create form without valid types").dispatch();
		}
		
		this.mapObjectTypes = {};
		for(var index = 0, objectType; objectType = args.objectTypes[index]; index++) {
			this.mapObjectTypes[objectType] = true;
		}
		
		this.readOnly= args.readOnly;
		this.editor= args.editor;
		this.create= args.create;
		this.standAlone= args.standAlone;
		this.fragment= args.fragment;
	},
	
	containsType: function(types){
		if(!types){
			return false;
		}
		
		var typesA = dojo.isArray(types)?types:[types];
		var matches = true;
		for (var i=0, lg=typesA.length; i<lg; i++) {
			if (!this.mapObjectTypes[typesA[i]]) {
				matches = false;
				break;
			}
		}
			
		return matches;		
	},
	
	
	equals:function(args){
		if(!args){return false;}
		var objectTypes= args.objectTypes;
		if(!objectTypes|| objectTypes.length!==this.mapObjectTypes.length){
			return false;
		}
		
		if (!containsType(objectTypes)) {
			return false;
		}
				
		var identifiers= ["packageName","formName", "readOnly","editor","create","standAlone","fragment"];
		for(var index = 0, identifier; identifier = identifiers[index]; index++) {
			if(args[identifier]!== this[identifier]){
				return false;				
			}
		}
		return true;
	}
});
   
})();