/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2012. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

/**
 * Utility for Identifier management
 * 
 * @author Steve Shaw
 */
(function(){
	
	dojo.provide("rmpw.common.util.IdentifierUtil");
	
	var util = rmpw.common.util.IdentifierUtil;
	
	dojo.mixin(util, {
	
		// true guid is a unique place and time (machine mac address and current time)
		// unless we use a guid generator from a server we can only do time and some random numbers
		guid: function() {
		   return "g" + (new Date()).getTime().toString(16) + "-" + this._random()+ this._random();
		},
		
		_random: function() {
		   return (((1+Math.random())*0x10000)|0).toString(16).substring(1);
		},
		
		normalizeURI : function(documentURI) {
			var fullURI = documentURI;
			if (fullURI && (fullURI.lastIndexOf('#') != fullURI.length - 1) && (fullURI.lastIndexOf('/') != fullURI.length - 1)) {
				fullURI += "#";
			}	
			
			return fullURI;
		},
		
		getGUIDWithParent: function(parentURI, guid) {
			if (parentURI && this.isIdentifierFixed(parentURI) && !this.isIdentifierFixed(guid)) {
				var index = parentURI.lastIndexOf('#');
				var docURI = parentURI;
				if (index != -1) {
					docURI = docURI.substring(0, index);
				}
				
				guid = docURI + "#" + guid;
			}
			
			return guid;
		},
		
		isIdentifierFixed : function(rdfAbout) {
			if (rdfAbout && (rdfAbout.lastIndexOf('#') != -1 || rdfAbout.lastIndexOf('/') != -1)) {
				return true;
			}
			
			return false;
		},
		
		getRootObject: function(parent) {
			if (parent.documentObject) {
				return parent.documentObject;
			}
			
			var rootObject = null;
			if (parent.form) {
				if (parent.form._resource) {
					rootObject = parent.form._resource.documentObject;
				}
			}
			else {
				rootObject = parent.host.root.diagram.getDefaultSource();
			}
			return rootObject;
		},
		
		inDocumentIdentifierCheck : function(parent, target, rdfAboutName) {
			// check for form vs diagram
			var identifier = this.getIdentifier(dojo.isString(rdfAboutName) ? rdfAboutName : rdfAboutName.get('value'));
			var rootObject = this.getRootObject(parent);
			if (!rootObject) {
				return false;
			}
			if (identifier === rdfAboutName) {
				return true;
			}
			return this.matchURI(rootObject, target, rdfAboutName, identifier);
		},
		
		getIdentifier : function(uri) {
			if (!uri) {
				return null;
			}
			
			var index = uri.lastIndexOf('#');
			var temp = uri;
			if (index == uri.length - 1) {
				temp = uri.substring(0, uri.length - 1);
			}
			else if (index > -1) {
				return uri.substring(index + 1);
			}
			
			index = temp.lastIndexOf('/');
			if (index > -1 && index != uri.length - 1) {
				return temp.substring(index + 1);
			}
			return temp;
		},
		
		matchURI : function(object, target, fullURI, identifier) {
			var dmcodegen_defines = object.dmcodegen_defines;
			if (dmcodegen_defines) {
				for (var i=0, lg=dmcodegen_defines.length; i<lg; i++) {
					var subObj = dmcodegen_defines[i];
					if (dojo.isObject(subObj) && subObj.rdf_about && !(subObj === target)) {
						if (subObj.rdf_about == fullURI || subObj.rdf_about == identifier) {
							return true;
						}
					}
				}
			}
			
			// check if matches the root identifier
			var rootID = this.getIdentifier(object.rdf_about);
			if (rootID == identifier) {
				return true;
			}
			
			return false;
		},

		exists : function(array, rdfAbout) {
			if (dojo.isArray(array) && rdfAbout) {
				for (var i=0, lg=array.length; i<lg; i++) {
					var obj = array[i];
					if (obj.rdf_about == rdfAbout) {
						return true;
					}
				}
			}
			return false;
		}	
		
	});
    
})(); // end function()