/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.render.util.Image");

dojo.require("dojox.json.ref");

/*
 * Utilities are image manipulation
 * 
 * @author Steve Shaw
 */

(function(){
	var imgUtil = rmpw.diagrams.render.util.Image;

// have to special case Chrome due to: https://code.google.com/p/chromium/issues/detail?id=173360 - apparently best case scenario is that it's fixed some version of Chrome Canary
imgUtil.svgsupport = dojo.isOffscreen ? true : ((dojo.isChrome <= 32) ? false : document.implementation.hasFeature("http://www.w3.org/TR/SVG11/feature#Image", "1.1"));

imgUtil._stripContext = function(src) {
	// if the original src already has the context embedded - strip it off
	var contextIdx = src.indexOf("?rmps.context");
	var newSrc = src;
	if (contextIdx != -1 && (src.indexOf("/image/data") == -1)) {
		newSrc = newSrc.substring(0, contextIdx);
	}
	
	return newSrc;
};

imgUtil._getImageSource = function(src) {
	// if the original src already has the context embedded - strip it off
	var imgSrcTag = "&imageSource=";
	var imageSourceIdx = src.indexOf(imgSrcTag);
	var maintainAspectRatioIdx = src.indexOf("&maintainAspectRatio");
	var newSrc = src;
	if (imageSourceIdx != -1 && maintainAspectRatioIdx != -1) {
		newSrc = decodeURIComponent(newSrc.substring(imageSourceIdx + imgSrcTag.length, maintainAspectRatioIdx));
	}
	
	return newSrc;
};

imgUtil.getNewImageSource = function(originalSrc, maintainAspectRatio, fillColor, lineColor) {
	// if the original src already has the context embedded - strip it off
	var src = this._stripContext(originalSrc);
	
	return net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/image/data?rmps.context=" + 
	encodeURIComponent(dojo.isOffscreen ? baselineURL : jazz.app.context.get().baseline) + 
	"&fillColor=" + ((fillColor !== undefined) ? (encodeURIComponent(fillColor)) : "") + 
	"&lineColor=" + ((lineColor !== undefined) ? (encodeURIComponent(lineColor)) : "") + 
	"&imageSource=" + encodeURIComponent(src) +
	"&maintainAspectRatio=" + ((maintainAspectRatio !== undefined) ? maintainAspectRatio : true) +
	"&supportsSVG=" + imgUtil.svgsupport;
};

/*
 * Resets image src so that SVG can be converted to image if needed.  Allows all browsers to display SVG
 * This method only supports dojo shape type == "image".  Scaling image must call "resetImage" below
 */
imgUtil.resetImageSource = function (image, maintainAspectRatio, fillColor, lineColor) {
	var imgShape = image.getShape();
	
	if (imgShape.type == "image") {
		if (!imgShape._originalSource) {
			imgShape._originalSource = imgShape.src;
		}
		
		imgShape.src = imgUtil.getNewImageSource(imgShape._originalSource, maintainAspectRatio, fillColor, lineColor);
		image.setShape( imgShape );
	}
};

/*
 * Resets image src so that SVG can be converted to image.  Allows all browsers to display SVG
 */
imgUtil.resetImage = function (image, maintainAspectRatio, fillColor, lineColor) {
	var imgShape = image.getShape();
	if (imgShape.type == "image") {
		imgUtil.resetImageSource(image, maintainAspectRatio, fillColor, lineColor);
	}
	else if (imgShape.type == "scalingImage") {
		image.resetImageSources(maintainAspectRatio, fillColor, lineColor);
	}
};

//static map - cache of the image information
imgUtil.imageToPolyMap = {};
imgUtil.imageToCallbackMap = {};

/*
 * Get image info - caches result
 */
imgUtil.getImageInfo = function (originalSrc, callback) {
	var shapeSource = this._stripContext(originalSrc);
	shapeSource = this._getImageSource(shapeSource);
	var url = net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL + "/image/info";
	
	var imageInfo = imgUtil.imageToPolyMap[shapeSource];
	if (!imageInfo && callback) {
		var callBackArray = imgUtil.imageToCallbackMap[shapeSource];
		if (!callBackArray) {
			callBackArray = [callback];
			imgUtil.imageToCallbackMap[shapeSource] = callBackArray;
			dojo.xhrGet({
				url: url,
				handleAs: "text",
				content: { 
					imageSource: dojo.isOffscreen ? encodeURIComponent(shapeSource) : shapeSource,
					"rmps.context": dojo.isOffscreen ? encodeURIComponent(baselineURL) : jazz.app.context.get().baseline
				},
				load: function(dataOutput) {
					if (dataOutput) {
						var dataObj = dojox.json.ref.fromJson(dataOutput);
						if (dataObj) {
							for (var i=0, lg=callBackArray.length; i<lg; i++) {
								callBackArray[i](dataObj);
							}
							delete imgUtil.imageToCallbackMap[shapeSource];
							imgUtil.imageToPolyMap[shapeSource] = dataObj;
						}
					}
				},
				error: function(error, ioArgs) {
				    rmps.core.error.createXhrError(ioArgs, "Getting the anchored shapes.").dispatch();
				    console.error(error);
				}
			});	
		}
		
		callBackArray.push(callback);
	}
	else if (callback) {
		callback(imageInfo);
	}
	
	return imageInfo;
};

}());
