/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.util.MoveableNoDelay");

dojo.require("rmpw.diagrams.edit.util.Moveable");
dojo.require("rmpw.diagrams.edit.util.Utils");

/**
 * This class is workaround for the issue where dojox.gfx.Moveable forces a move start event 
 * on mouse down.  This interferes with other policies and event subscribers who assume that a
 * move has started when it actually hasn't.
 * 
 * @author Steve Shaw
 */
dojo.declare("rmpw.diagrams.edit.util.MoveableNoDelay", rmpw.diagrams.edit.util.Moveable, {
	
	suppressed: false,
	
	// local events
	onMoveStart: function(/* dojox.gfx.Mover */ mover){
		// do nothing - fire events on "onFirstMove" to avoid conflicts
	},
	
	onFirstMove: function(/* dojox.gfx.Mover */ mover) {
		// call the super class onMoveStart here instead
		dojo.publish("/gfx/move/start", [mover]); 
	},

	onMouseDown: function(e){
		// summary: event processor for onmousedown, creates a Mover for the shape
		// e: Event: mouse event
		
		
		if( rmpw.diagrams.edit.util.Utils.singleHit(e) ){
		
			var eOrig = e; 
            e = e.touches ? e.touches[0] : e;
			
			if(this.suppressed === false) {
				if(this.delay){
					
					if( eOrig.touches ){
						//Touch Events. 
						this.events.push(this.shape.connect("touchmove", this, "onMouseMove")); 
						this.events.push(this.shape.connect("touchend", this, "onMouseUp"));
					} else {
						this.events.push(
						this.shape.connect("onmousemove", this, "onMouseMove"),
						this.shape.connect("onmouseup", this, "onMouseUp"));
					}		
					
					this._lastX = e.clientX;
					this._lastY = e.clientY;
				} else {
					// TODO(MG): why do we new this without saving the reference?
                    new this.mover(this.shape, eOrig, this);
				}
				dojo.stopEvent(eOrig);
			}
		}
	},
	
	detachListeners: function(){		
		this.suppressed = true;
	},
	
	reAttach: function(){
		this.suppressed = false;
	}
});