/*
	Copyright (c) 2004-2009, The Dojo Foundation All Rights Reserved.
	Available via Academic Free License >= 2.1 OR the modified BSD license.
	see: http://dojotoolkit.org/license for details
*/
/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/


dojo.provide("rmpw.diagrams.edit.util.Mover");
dojo.require("rmpw.diagrams.edit.util.Utils");

dojo.declare("rmpw.diagrams.edit.util.Mover", null, {
	constructor: function(shape, e, host){
		// summary: an object, which makes a shape follow the mouse,
		//	used as a default mover, and as a base class for custom movers
		// shape: dojox.gfx.Shape: a shape object to be moved
		// e: Event: a mouse event, which started the move;
		//	only clientX and clientY properties are used
		// host: Object?: object which implements the functionality of the move,
		//	 and defines proper events (onMoveStart and onMoveStop)
		
		var eOrig = e; 
		
        e = e.touches ? e.touches[0] : e;
		
		this.shape = shape;
		this.lastX = e.clientX;
		this.lastY = e.clientY;
		var h = this.host = host, d = document;
		
		if (eOrig.touches) {
			this.firstEvent  = dojo.connect(d, "touchmove", this, "onFirstMove");
		} else {
			this.firstEvent = dojo.connect(d, "onmousemove", this, "onFirstMove");
		}

		this.events = [
			dojo.connect(d, "onmousemove", this, "onMouseMove"),
			dojo.connect(d, "onmouseup",   this, "destroy"),
			
			dojo.connect(d, "touchmove", this, "onMouseMove"), 
            dojo.connect(d, "touchend",   this, "destroy"), 
			
			// cancel text selection and text dragging
			dojo.connect(d, "ondragstart",   dojo, "stopEvent"),
			dojo.connect(d, "onselectstart", dojo, "stopEvent"),
			this.firstEvent
		];
		// notify that the move has started
		if(h && h.onMoveStart){
			h.onMoveStart(this);
		}
	},
	
	// mouse event processors
	onMouseMove: function(e){
		// summary: event processor for onmousemove
		// e: Event: mouse event
		
		if( !this.firstEvent && rmpw.diagrams.edit.util.Utils.singleHit(e) ){
		
			this.isTouch = !!e.touches;
            var eFixed = this.isTouch ? e.touches[0] : e; 
			
			var x = eFixed.clientX;
			var y = eFixed.clientY;
			this.host.onMove(this, {dx: x - this.lastX, dy: y - this.lastY });
			this.lastX = x;
			this.lastY = y;
			dojo.stopEvent(e);
		}
	},
	// utilities
	onFirstMove: function(e){
		// some browsers throw a mousemove right after a mouse down
		this.isTouch = !!e.touches;
        var eFixed = this.isTouch ? e.touches[0] : e; 
 		if (eFixed.clientX !==  this.lastX &&  eFixed.clientY !==this.lastY) {
			// summary: it is meant to be called only once
			this.host.onFirstMove(this);
	
			/* This is not great - and is improved in the latest version of Mover in dojoGFX,
			 * it relies on the position of the connections.
			 */
			dojo.disconnect(this.events.pop());
			delete this.firstEvent;
		}
	},
	destroy: function(){
		// summary: stops the move, deletes all references, so the object can be garbage-collected
		dojo.forEach(this.events, dojo.disconnect);
		// undo global settings
		var h = this.host;
		if(h && h.onMoveStop){
			h.onMoveStop(this);
		}
		// destroy objects
		this.events = this.shape = null;
	}
});

