/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("rmpw.diagrams.render.util.Point");

dojo.require("rmpw.diagrams.render.util.Dimension");
dojo.require('dojox.gfx.matrix');

var NORTH =  1;
var SOUTH = 4;
var WEST = 8;
var EAST = 16;

dojo.declare("rmpw.diagrams.render.util.Point", null, {
	
	x : 0,
	y : 0,
	
	NORTH : NORTH,
	SOUTH : SOUTH,
	WEST : WEST,
	EAST : EAST,
	
	constructor: function(x, y) {
		this.x = x ? x : 0;
		this.y = y ? y : 0;
	},
	
	getHorizontalPosition: function(relativePt) {
		return (relativePt.x < this.x) ? WEST : EAST;
	},
	
	getVerticalPosition: function(relativePt) {
		return (relativePt.y < this.y) ? NORTH : SOUTH;
	},
	
	/**
	 * Calculates the relative position of the relative point to an anchor point.
	 */
	getPosition: function(relativePt) {
		var dx = relativePt.x - this.x;
		var dy = relativePt.y - this.y;
		if (Math.abs(dx) > Math.abs(dy)) {
			if (dx < 0){
				return WEST;
			}
			return EAST;
		}
		if (dy < 0){
			return NORTH;
		}
		return SOUTH;
	},
	
	getCopy: function(){
		return new rmpw.diagrams.render.util.Point(this.x, this.y);
	},
	
	toString$: function(){
		return 'Point(' + this.x + ', ' + this.y + ')';
	},
	
	getClass$: function() {
		return this.declaredClass;
	},
		
	hashCode$: function(){
		return this.x * this.y ^ this.x + this.y;
	},
	
	equals$: function(o) {
		if (o && o.x && o.y) {
			return o.x === this.x && o.y === this.y;
		}
		return false;
	},
	
	getDifference: function(pt){
		return new rmpw.diagrams.render.util.Dimension(this.x - pt.x, this.y - pt.y);
	},

	getPreciseDistance2: function(pt){
		  var i, j;
		  i = pt.x - this.x;
		  j = pt.y - this.y;
		  return i * i + j * j;
	},
	
	getDistance: function(pt) {
		return Math.sqrt(this.getPreciseDistance2(pt));
	},
	
	getDistanceOrthogonal: function(pt) {
		return Math.abs(this.y - pt.y) + Math.abs(this.x - pt.x);
	},
	
	setLocation: function(pt){
		this.x = pt.x;
		this.y = pt.y;
		return this$static;
	},
	
	getPosition: function(p){
		var dx, dy;
		dx = p.x - this.x;
		dy = p.y - this.y;
		if ((dx < 0?-dx:dx) > (dy < 0?-dy:dy)) {
		  if (dx < 0){
		    return 8;
		  }
		  return 16;
		}
		if (dy < 0){
		  return 1;
		}
		return 4;
	},
	
	getLocation: function() {
		return this;
	},
	
	performScale: function (factor){
		this.x = this.x * factor;
		this.y = this.y * factor;
		return this;
	},
	
	transform: function (surfaceMatrix) {
		if (surfaceMatrix) {
			var m = dojox.gfx.matrix;
			var transformedPt = m.multiplyPoint(surfaceMatrix, this.x, this.y);
			this.x = transformedPt.x;
			this.y = transformedPt.y;
		}
		return this;
	},
	scale: function(xAmount, yAmount){
		this.x = Math.floor(this.x * xAmount);
		this.y = Math.floor(this.y * yAmount);
		return this;
		
	},
	translate: function(dx, dy) {
		  this.x += dx;
		  this.y += dy;
		  return this;
	},
	translate_0: function(d) {
		return this.translate(d.width, d.height);
	},
	translate_1: function(p){
		return this.translate(p.x, p.y);
	},		
	transpose: function(xAmount, yAmount){
		var temp =this.x;
		this.x = this.y;
		this.y = temp;
		return this;
	}	

});

}());