/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.render.util.PointList");

dojo.require("rmpw.diagrams.render.util.Rectangle");
dojo.require("rmpw.diagrams.render.util.Point");
dojo.require("rmpw.diagrams.render.util.LineSeg");

dojo.declare("rmpw.diagrams.render.util.PointList", null, {
	
	constructor: function(size) {
		this.points = [];
		this.size = 0; // ignore size parameter - JavaScript will allocate automatically
		this.bounds = null;
	},
	
	equals: function(points) {
		if (points.size !== this.size){ return false;}
		
		for (var i=0, pt; pt=this.points[i]; i++) {
			var opt = points.points[i];
			if (pt.x !== opt.x || pt.y !== opt.y) {
				return false;
			}
		}
		
		return true;
	},

	performScale: function (factor) {
		for (var i = 0, lg = this.points.length; i < lg; i++){
			this.points[i].performScale(factor);
	    }
		this.bounds = null;
		return this;
	},
	
	scale: function(xAmount, yAmount){
		for (var i = 0, lg = this.points.length; i < lg; i++){
			this.points[i].scale(xAmount, yAmount);
		}
		this.bounds = null;
		return this;
	},
	
	transform: function (surfaceMatrix) {
		for (var i = 0, lg = this.points.length; i < lg; i++){
			this.points[i].transform(surfaceMatrix);
		}
		this.bounds = null;
		return this;
	},
	
	translate: function(dx, dy) {
		for (var i = 0, lg = this.points.length; i < lg; i++) {
			this.points[i].translate(dx, dy);
		}
		this.bounds = null;
		return this;
	},
	
	getPoint: function (index){
		return this.points[index].getCopy();
	},
	
	addPoint_0: function (x1, y1){
		this.points.push(new rmpw.diagrams.render.util.Point( x1, y1));
		this.size = this.points.length;
	},
	
	addPoint_1: function (p){
		this.points.push(new rmpw.diagrams.render.util.Point( p.x, p.y));
		this.size = this.points.length;
	},
	
	getFirstPoint: function (){
		return this.getPoint(0);
	},
	
	getLastPoint: function (){
		return this.getPoint(this.points.length - 1);
	},
	
	addAll: function (source){
		for (var i=0, lg=source.points.length; i<lg; i++) {
			this.points.push(source.points[i]);
		}
		this.size = this.points.length;
	},
	
	toIntArray: function (){
		var intarray = initDim(_3I_classLit, 110, -1, this.points.length * 2, 1);
		for (var i=0, lg=this.points.length; i<lg; i++) {
			intarray.push(this.points[i].x);
			intarray.push(this.points[i].y);
		}

		return intarray;
	},
	
	setSize: function (newSize){
		// do nothing - JavaScript dynamically allocates if needed
	},
	
	setPoint: function (pt, index){
		if (index < 0 || index >= this.points.length){
			throw 'Index out of bounds Exception' + 'Index: ' + index + ', Size: ' + this.points.length;
		}
		if (!!this.bounds && !this.bounds.contains_0(pt)){
			this.bounds = null;
		}
		this.points[index] = new rmpw.diagrams.render.util.Point( pt.x, pt.y);
	},
	
	reverse: function (){
		var i, j, temp;
		var lg = this.points.length;
		for (i = 0 , j = lg -1; i < lg; i++ , j--) {
			temp = this.points[i];
			this.points[i] = this.points[j];
			this.points[j] = temp;
		}
	},
	
	removePoint: function (index){
		this.bounds = null;
		if (index < 0 || index >= this.points.length){
			throw 'Index out of bounds Exception' + 'Index: ' + index + ', Size: ' + this.points.length;
        }
		var pt = this.points[index];
		for (var i=index, lg=this.points.length - 1; i<lg; i++) {
			this.points[i] = this.points[i+1];
		}
		this.points.length--;
		this.size = this.points.length;
		
		return pt;
	},
	
	removeAllPoints: function (){
		this.bounds = null;
		this.points.length = 0;
		this.size = this.points.length;
	},
	
	insertPoint: function (p, index){
		var length_0, old;
		if (!!this.bounds && !this.bounds.contains_0(p)){
			this.bounds = null;
		}
		if (index > this.points.length || index < 0){
			throw 'Index out of bounds Exception' + 'Index: ' + index + ', Size: ' + this.points.length;
        }
		var next, current = new rmpw.diagrams.render.util.Point( p.x, p.y);
		for (var i=index, lg=this.points.length; i<=lg; i++) {
			if (i !== lg) {
				next = this.points[i];
			}
			this.points[i] = current;
			current = next;
		}
		this.size = this.points.length;
	},
	
	getPoint_0: function (p, index){
		if (index < 0 || index >= this.points.length){
			throw 'Index out of bounds Exception' + 'Index: ' + index + ', Size: ' + this.points.length;
	    }
		p.x = this.points[index].x;
		p.y = this.points[index].y;
		return p;
	},
	
	getCopy: function (){
		var result = new rmpw.diagrams.render.util.PointList();
		result.points = new Array(this.points.length);
		for (var i=0, lg=this.points.length; i<lg; i++) {
			result.points[i] = this.points[i].getCopy();
		}
		result.bounds = this.bounds;
		result.size = this.points.length;
		return result;
	},
	
	getBounds: function (){
		var i, lg;
		if (this.bounds){
			return this.bounds;
		}
		this.bounds = new rmpw.diagrams.render.util.Rectangle(0, 0, 0, 0);
		if (this.points.length > 0) {
			this.bounds.setLocation(this.getPoint(0));
			for (i = 0, lg=this.points.length; i < lg; ++i){
				this.bounds.union(this.getPoint(i));
			}
		}
		return this.bounds;
	},
	
	crossProduct: function (ax, ay, bx, by, cx, cy) {
		return (ax - cx) * (by - cy) - (ay - cy) * (bx - cx);
	},
	
	/**
	 * Utility method used to check if a point is contained inside a polygon
	 */
	containsPoint: function (point){
		var isOdd = false;
		var pointsxy = this.points;
		var n = pointsxy.length;
		if (n > 1) { //If there are at least 2 Points (4 ints)
			var x1, y1;
			var x0 = pointsxy[n-1].x;
			var y0 = pointsxy[n-1].y;
	
			for (var i = 0; i < n; x0 = x1, y0 = y1) {
				x1 = pointsxy[i].x;
				y1 = pointsxy[i].y;
				i++;
				if (y0 <= point.y && point.y < y1
						&& this.crossProduct(x1, y1, x0, y0, point.x, point.y) > 0){
					isOdd = !isOdd;
				}
				if (y1 <= point.y && point.y < y0
						&& this.crossProduct(x0, y0, x1, y1, point.x, point.y) > 0){
					isOdd = !isOdd;
				}
			}
			if (isOdd){
				return true;
			}
		}
		return false;
	},
	
	/*
	 * Normalize segments to eliminate redundancy
	 */
	normalize: function(tolerance) {
		var points = this.points;
		if (points.length < 3){
			return;
		}

		// flatten segments
		for (var i = 0; i < points.length - 1; i++) {
			var ptStart = points[i];

			var ptTerm = null;
			if (i + 1 < points.length){
				ptTerm = points[i + 1];
            }
			var ptNext = null;
			if (i + 2 < points.length){
				ptNext = points[i + 2];
            }
			// We conservatively avoid changing the endpoint of the line, since it may then
			// fail to attach to the terminal view.  This implies ptNext must be non-NIL.
			if (points.length <= 2 || !ptTerm || !ptNext) {
				break;
			}

			var line = new rmpw.diagrams.render.util.LineSeg(ptStart, ptNext);
			var pt = line.perpIntersect(ptTerm.x, ptTerm.y);
			var sameOrientation = Math.round(pt.getDistance(new rmpw.diagrams.render.util.Point(ptTerm.x, ptTerm.y))) < tolerance;
			
			if (sameOrientation) {

				points.splice(i + 1, 1);

				var seg = new rmpw.diagrams.render.util.Ray(ptStart, ptTerm);
				if (Math.abs(seg.y) < tolerance) {
					points[i+1] = new rmpw.diagrams.render.util.Point(ptNext.x, ptStart.y);
				} else if (Math.abs(seg.x) < tolerance){
					points[i+1] = new rmpw.diagrams.render.util.Point(ptStart.x, ptNext.y);
				}
			}
			else {
				// straighten points
				if (Math.abs(ptStart.x - ptTerm.x) < (tolerance)) {
					points[i+1] = new rmpw.diagrams.render.util.Point(ptStart.x, ptTerm.y);
				} else if (Math.abs(ptStart.y - ptTerm.y) < (tolerance)) {
					points[i+1] = new rmpw.diagrams.render.util.Point(ptTerm.x, ptStart.y);
				}
			}
		}

		this.size = points.length;
	}
});