/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2013.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.app.util.ProjectConfigurationsInfo");

dojo.require("jazz.app.context");
dojo.require("rmps.core.util.ContextUtils");
dojo.require("rmps.core.streams.StreamClient");
dojo.require("rmps.core.baseline.BaselineClient");
dojo.require("rmps.app.ui.internal.util.DomainRegistryUtil");


var StreamClient = rmps.core.streams.StreamClient;
var SnapshotClient = rmps.core.baseline.BaselineClient;
var ContextUtils = rmps.core.util.ContextUtils;
var DomainRegistry = rmps.app.ui.internal.util.DomainRegistryUtil;

dojo.declare("rmps.app.util.ProjectConfigurationsInfo", null, {
	
	constructor: function(projectId) {
		this.projectId = projectId;
	},
	
	getWorkspaces: function() {
		var deferred;
		var projectId = this.projectId ? this.projectId : jazz.app.context.get().itemId;
		if (this._workspaces) {
			deferred = new dojo.Deferred();
			deferred.resolve(this._workspaces);
		} else {
			deferred = StreamClient.getStreams(projectId);
			deferred.addCallback(dojo.hitch(this, function(workspaces) {
				this._workspaces = workspaces;
				return this._workspaces;
			}));
		}
		return deferred;
	},

	getDefaultWorkspace: function() {
		var deferred;
		var self = this;
		var projectId = this.projectId ? this.projectId : jazz.app.context.get().itemId;
		if (this._defaultWorkspace) {
			deferred = new dojo.Deferred();
			deferred.resolve(this._defaultWorkspace);
		} else {
			deferred = StreamClient.getDefaultStream(projectId);
			deferred.addCallback(function(defaultWorkspaceUri) {
				var streamDef = StreamClient.getStream(defaultWorkspaceUri);
				streamDef.addCallback(function(defaultWorkspace) {
					self._defaultWorkspace = defaultWorkspace;	
					return self._defaultWorkspace;
				});
				return streamDef;
			});
		}
		return deferred;
	},
	
	getSnapshots: function() {
		var deferred;
		var projectId = this.projectId ? this.projectId : jazz.app.context.get().itemId;
		if (this._snapshots) {
			deferred = new dojo.Deferred();
			deferred.resolve(this._snapshots);
		} else {
			deferred = SnapshotClient.getBaselines(null, projectId);
			deferred.addCallback(dojo.hitch(this, function(snapshots) {
				this._snapshots = snapshots;
				return this._snapshots;
			}));
		}
		return deferred;
	},
	
	getSpaceUri: function() {
		var deferred;
		var projectId = this.projectId ? this.projectId : jazz.app.context.get().itemId;
		if (this._space) {
			deferred = new dojo.Deferred();
			deferred.resolve(this._space);
		} else {
			deferred = ContextUtils.getProjectSpace(projectId);
			deferred.addCallback(dojo.hitch(this, function(space) {
				this._space = space;
				return this._space;
			}));
		}
		return deferred;
	},
	
	getDspInfo: function(callback) {
		var deferred;
		var projectId = this.projectId ? this.projectId : jazz.app.context.get().itemId;
		if (this._dspInfo) {
			callback(this._dspInfo.isDsp, this._dspInfo.isSystem, this._dspInfo.canCreateDsp);
		} else {
			deferred = DomainRegistry.isCurrentProjectDSP(dojo.hitch(this, function(isDsp, isSystem, canCreateDsp) {
				this._dspInfo = { isDsp: isDsp, isSyste: isSystem, canCreateDsp: canCreateDsp };
				callback(isDsp, isSystem, canCreateDsp);
				return this._dspInfo;
			}), projectId);
		}
	},
	
	getDomainsInfo: function(callback) {
		var deferred;
		var projectId = this.projectId ? this.projectId : jazz.app.context.get().itemId;
		if (this._domainsInfo) {
			if (callback) {
				callback(this._domainsInfo.data, this._domainsInfo.eTag);
			}
			deferred = new dojo.Deferred();
			deferred.resolve(this._domainsInfo.data);
		} else {
			deferred = DomainRegistry.getDomainsInfo(dojo.hitch(this, function(data, eTag) {
				this._domainsInfo = {
					data: data,
					eTag: eTag
				};
				if (callback) {
					callback(data, eTag);
				}
			}), projectId);
		}
		return deferred;
	},
	
	reset: function() {
		delete this._workspaces;
		delete this._defaultWorkspace;
		delete this._snapshots;
		delete this._dspInfo;
		delete this._space;
		delete this._domainsInfo;
	}
		
});

}());
