/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("rmpw.diagrams.render.util.Rectangle");

dojo.require("rmpw.diagrams.render.util.Point");

dojo.declare("rmpw.diagrams.render.util.Rectangle", null, {
	
	constructor: function(x, y, width, height) {
		this.x = x;
		this.y = y;
		this.width = width;
		this.height = height;
	},

	contains_0: function (pt){
		return this.contains_2(pt.x, pt.y);
	},
	
	contains_2: function(x, y){
		return y >= this.y && y < this.y + this.height && x >= this.x && x < this.x + this.width;
	},
	
	contains_3: function(pt){
		return this.contains_2(pt.x, pt.y);
	},
	
	contains_4: function(rect){
		return this.x <= rect.x && this.y <= rect.y && (this.x + this.width) >= (rect.x + rect.width)
				&& this.y + this.height >= rect.y + rect.height;
	},
	
	isEmpty: function() {
		return this.x === 0 && this.y === 0 && this.width === 0 && this.height === 0;
	},
	
	equals$: function (o){
		var r;
		if (( !this ?null:this) === (!o ?null:o)){
			return true;
		}
		if (o && o.x && o.y && o.width && o.height) {
			r = o;
			return this.x === r.x && this.y === r.y && this.width === r.width && this.height === r.height;
		}
		return false;
	},

	getClass$: function (){
		return this.declaredClass;
	},

	hashCode$: function (){
		return (this.x + this.height + 1) * (this.y + this.width + 1) ^ this.x ^ this.y;
	},

	getPosition_0: function (pt){
		var result;
		result = 0;
		if (this.contains_2(pt.x, pt.y)){
			return result;
		}
		if (pt.x < this.x){
			result = 8;
		}else if (pt.x >= this.x + this.width){
			result = 16;
		}
		if (pt.y < this.y){
			result = result | 1;
		}else if (pt.y >= this.y + this.height){
			result = result | 4;
		}
		return result;
	},
	
	setBounds: function(rect){
		this.x = rect.x;
		this.y = rect.y;
		this.width = rect.width;
		this.height = rect.height;
		return this;
	},

	setLocation: function (p){
		this.x = p.x;
		this.y = p.y;
		return this;
	},
	
	setLocation_0: function(x1, y1){
		this.x = x1;
		this.y = y1;
		return this;
	},
		
	toString$: function (){
		return 'Rectangle(' + this.x + ', ' + this.y + ', ' + this.width + ', ' + this.height + ')';
	},

	translate: function (dx, dy){
		this.x += dx;
		this.y += dy;
		return this;
	},

	union_1: function (x1, y1){
		var bottom, right;
		if (x1 < this.x) {
			this.width += this.x - x1;
			this.x = x1;
		}
		else {
			right = this.x + this.width;
			if (x1 >= right) {
				right = x1 + 1;
				this.width = right - this.x;
			}
		}
		if (y1 < this.y) {
			this.height += this.y - y1;
			this.y = y1;
		}
		else {
			bottom = this.y + this.height;
			if (y1 >= bottom) {
				bottom = y1 + 1;
				this.height = bottom - this.y;
			}
		}
		return this;
	},
	
	union: function (p){
		this.union_1(p.x, p.y);
	},

	union_2: function(x, y, w, h) {
		var bottom, right;
		right = Math.max(this.x + this.width, x + w);
		bottom = Math.max(this.y + this.height, y + h);
		this.x = Math.min(this.x, x);
		this.y = Math.min(this.y, y);
		this.width = right - this.x;
		this.height = bottom - this.y;
		return this;
	},
	
	union_0: function (rect){
		if (!rect){
			return this;
		}
		return this.union_2(rect.x, rect.y, rect.width, rect.height);
	},
		
	setSize: function (d){
		return this.width = d.width , this.height = d.height , this;
	},

	shrink: function (h, v){
		this.x += h;
		this.width -= h + h;
		this.y += v;
		this.height -= v + v;
		return this;
	}, 
		
	crop: function (insets){
		if (!insets)
			return this;
		this.x += insets.left;
		this.y += insets.top_0;
		this.width -= insets.left + insets.right;
		this.height -= insets.top_0 + insets.bottom;
		return this;
	}, 
		
	getCenter: function (){
		return new rmpw.diagrams.render.util.Point( this.x + ~~(this.width / 2), this.y + ~~(this.height / 2));
	},
		
	performScale: function(scale) {
		var oldX = this.x;
		var oldY = this.y;
		this.x = this.x * scale;
		this.y = this.y * scale;
		this.width = ((oldX + this.width) * scale) - this.x;
		this.height = ((oldY + this.height) * scale) - this.y;
		return this;
	},
	
	transform: function (surfaceMatrix) {
		if (surfaceMatrix) {
			var oldX = this.x;
			var oldY = this.y;
			var m = dojox.gfx.matrix;
			var transformedPt = m.multiplyPoint(surfaceMatrix, this.x, this.y);
			this.x = transformedPt.x;
			this.y = transformedPt.y;
			
			this.width = this.width * surfaceMatrix.xx;
			this.height = this.height * surfaceMatrix.yy;
		}
		return this;
	},
	
	intersect: function (rect) {
		var x1, x2, y1, y2;
		x1 = Math.max(this.x, rect.x);
		x2 = Math.min(this.x + this.width, rect.x + rect.width);
		y1 = Math.max(this.y, rect.y);
		y2 = Math.min(this.y + this.height, rect.y + rect.height);
		if (x2 - x1 < 0 || y2 - y1 < 0) {
			this.x = this.y = this.width = this.height = 0;
		}
		else {
			this.x = x1;
			this.y = y1;
			this.width = x2 - x1;
			this.height = y2 - y1;
		}
		return this;
	},
	
	/*
	 * Performant check to see if a given rectangle interacts with this
	 * Rectangle.
	 */
	intersectsWith: function (rect) {
		var union = this.getCopy().union_0(rect);
		if (union.width > (this.width + rect.width) ||
			union.height > (this.height + rect.height)) {
			return false;
		}
		
		return true;
	},
	
	getCopy: function() {
		return new rmpw.diagrams.render.util.Rectangle(this.x, this.y, this.width, this.height);
	},
	getTopLeft: function() {
		return new rmpw.diagrams.render.util.Point( this.x , this.y );
	},	
	getTopRight: function() {
		return new rmpw.diagrams.render.util.Point( this.x + this.width - 1, this.y );
	},	
	getBottomLeft: function() {
		return new rmpw.diagrams.render.util.Point( this.x , this.y + this.height - 1 );
	},	
	getBottomRight: function() {
		return new rmpw.diagrams.render.util.Point( this.x + this.width - 1, this.y + this.height - 1 );
	},	
	getCopy: function() {
		return new rmpw.diagrams.render.util.Rectangle(this.x, this.y, this.width, this.height);
	}	
});