/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| Copyright IBM Corp. 2009, 2010.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

dojo.provide("rmpw.diagrams.render.util.ResizeObject");

dojo.declare("rmpw.diagrams.render.util.ResizeObject", null, {

	applySize: function(slave, bounds, matchWidth, matchHeight) {
		var shape = slave.shape,
		  pt, i, child;
		
		if(shape.type === "line") {
			if(matchWidth && matchHeight) {
				throw "matching width and height of line not supported";
			} else if(matchWidth) {
				slave.setShape({x2: bounds.width});	
			} else if(matchHeight) {
				slave.setShape({y2: bounds.height});
			}
		} else if(shape.type === "rect" || shape.type === "image") {
			if(matchWidth && matchHeight) {
				slave.setShape({width: bounds.width, height: bounds.height});
			} else if(matchWidth) {
				slave.setShape({width: bounds.width});				
			} else if(matchHeight) {
				slave.setShape({height: bounds.height});
			}
		} else if (shape.type === "polyline") {
			var points = shape.points;
			var pts = new rmpw.diagrams.render.util.PointList();
			for (i = 0, points; pt = points[i]; i++) {
				pts.addPoint_0(pt.x, pt.y);
			}
			
			var slaveBounds = pts.getBounds();
			
			// TODO (MG): performance - put the loop inside matchWidth/height checks, and pre-calculate ratio 
			for (i = 0, points; pt = points[i]; i++) {
				if (matchWidth) {
					pt.x *= bounds.width / slaveBounds.width;
				}
				
				if (matchHeight) {
					pt.y *= bounds.height / slaveBounds.height;
				}
			}
			
			slave.setShape(points);
		}
		else {
			// recurse through child and resize according to custom style
			if (slave.declaredClass === "dojox.gfx.Group") {
				var children = slave.children;
				if (children) {
					for(i = 0, child; child = children[i]; i++) {
						var childStyle = child.customStyle;
						
						if (childStyle && (childStyle.xStretchRatio || childStyle.yStretchRatio)) {
							var childBounds = { x: bounds.x, y: bounds.y, 
									width: childStyle.xStretchRatio ? bounds.width * childStyle.xStretchRatio : bounds.width,
									height: childStyle.yStretchRatio ? bounds.height * childStyle.yStretchRatio : bounds.height };		
							this.applySize(child, childBounds, childStyle.xStretchRatio !== undefined, childStyle.yStretchRatio !== undefined);
						}
					}
				}
			} else {
				// TODO (MG): support other shapes
				console.error("MatchSize only implemented for polyline, lines, images and rects so far");
			}
		}
	}

});

