/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("dm.review.app.util.ReviewActionsRegistery");

dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.ResourceLink");
dojo.require("jazz.util.html");
dojo.require("rmps.core.review.util.ReviewImageService");
dojo.require("dm.review.app.util.ReviewUtil");
dojo.require("rmps.core.review.util.ReviewTypes");
dojo.require("rmps.core.resource.ResourceInfoClient");
dojo.require("rmps.core.util.IRmpsConstants");
dojo.require("rmpw.text.internal.widgets.RichTextWidget");
dojo.require("rmps.app.ui.internal.pages.design.DesignUtil");
dojo.require("rmps.app.pickers.internal.CLMResourcePicker");

var ReviewImageService = rmps.core.review.util.ReviewImageService;
var HoverPopup = jazz.ui.HoverPopup;
var ResourceLink= jazz.ui.ResourceLink;
var ReviewTypes= rmps.core.review.util.ReviewTypes;
var ResourceInfoClient= rmps.core.resource.ResourceInfoClient;
var IRmpsConstants = rmps.core.util.IRmpsConstants;

var ReviewUtil= dm.review.app.util.ReviewUtil;
var DesignUtil= rmps.app.ui.internal.pages.design.DesignUtil;

var DEFAULT_BASELINE= "Default";

/*The UAKEY constants for review sections */
var REVIEW_ELEMENT_SECTION_UAKEY = "com.ibm.xtools.rmps.web.app.section_ReviewElementsTableSection001";
var REVIEW_PARTICIPANTS_SECTION_UAKEY = "com.ibm.xtools.rmps.web.app.section_ReviewParticipantsTableSection001";

dojo.requireLocalization("dm.review.app.util", "ReviewActionsRegisteryMessages");
var ActionsRegisteryMessages = dojo.i18n.getLocalization("dm.review.app.util", "ReviewActionsRegisteryMessages");

/*The used vieers. Do not load unless used on rich hovers. So, we do not dojo require them */
var ReviewParticipantViewer= "dm.review.app.ui.internal.viewers.ReviewParticipantViewer";
var ReviewResourceViewer=     "dm.review.app.ui.internal.viewers.ReviewResourceViewer";

/*
 * The defined dialogs to be used in the review editor. They are loaded asynchrounsely when need, in order to avoid loading them at the review start
 * time which is not necessary 
 */
var AddParticipantsForm= "dm.review.app.ui.internal.dialogs.AddParticipantsForm";
var PickerDialog = "rmps.app.pickers.internal.CLMResourcePicker";
var ChangeParticipantsForm= "dm.review.app.ui.internal.dialogs.ChangeParticipantsForm";
var ChangeElementStatusForm= "dm.review.app.ui.internal.dialogs.ChangeElementStatusForm";

/*
 * 
 * Provide the required callbacks for cell and actions creations for different review tables
 * in addition to all the required update/delete/add functions 
 * 
 */
dm.review.app.util.ReviewActionsRegistery = {
	
	/***********************************************************************************/
	getModelElementDefaultCellDescriptor: function(id, width){
		var createModelElementDefaultCell= function(section, table, fieldName, displayName, data, tableRow, nameTD){
			var elementName = data.title;
			if (section.reviewController.configNotExist){
				nameTD.appendChild(dojo.doc.createTextNode(elementName));
			} else if(!data.exists) {
				dm.review.app.util.ReviewActionsRegistery.createResNotExistNode(nameTD, elementName);
			}else {
				var anchor = dojo.doc.createElement("a");
				anchor.href = dm.review.app.util.ReviewActionsRegistery.getFullEditResourceUrl(section.reviewController, data.resourceUri);			
				anchor.onclick= function(e){
					dojo.stopEvent(e);
					section.reviewController.navigateTo(anchor.href);
				};
				anchor.appendChild(dojo.doc.createTextNode(elementName));
				nameTD.appendChild(anchor);
			}
	        return nameTD;
	    };
		return {columnHeader: ActionsRegisteryMessages.name, fieldName: id, width: width, creatorCallback: createModelElementDefaultCell};
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	getModelElementForParticipantCellDescriptor: function(id, width){
		var createModelElementDefaultCell= function(section, table, fieldName, resourceUri, data, tableRow, nameTD){
			var resource = section.reviewController.review.getReviewResources(resourceUri);
			var displayName = resource.title;
			if (section.reviewController.configNotExist){
				nameTD.appendChild(dojo.doc.createTextNode(displayName));
			} else if(!resource.exists) {
				dm.review.app.util.ReviewActionsRegistery.createResNotExistNode(nameTD, displayName);
			} else {
				var anchor = dojo.doc.createElement("a");
				anchor.href = dm.review.app.util.ReviewActionsRegistery.getFullEditResourceUrl(section.reviewController, resourceUri);			
				anchor.onclick= function(e){
					dojo.stopEvent(e);
					section.reviewController.navigateTo(anchor.href);
				};
				anchor.appendChild(dojo.doc.createTextNode(displayName));
				nameTD.appendChild(anchor);
			}
	        return nameTD;
	    };
		return {columnHeader: ActionsRegisteryMessages.name, fieldName: id, width: width, creatorCallback: createModelElementDefaultCell};
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	getAboutPreviewCellDescriptor: function(id, width){
		var createAboutPreviewCell= function(section, table, fieldName, displayName, data, tableRow, inputTD){
			var about= data.reviewArtifactInstructions;
			if(!about){
				/*Safe check to avoid getting exceptions */
				about= "";
			}
			inputTD.appendChild(document.createTextNode(about));
	    	return inputTD;
	    };
		return {columnHeader: ActionsRegisteryMessages.about, fieldName: id, width: width, creatorCallback: createAboutPreviewCell};
	},
	/***********************************************************************************/
 	
 	/***********************************************************************************/
	getAboutInputCellDescriptor: function(id, width){
		var createAboutInputCell= function(section, table, fieldName, displayName, data, tableRow, inputTD){
			var about= data.reviewArtifactInstructions;
			if(!about){
				/*Safe check to avoid getting exceptions */
				about= "";
			}
			
			var input2TD = dojo.create("td", null, inputTD);
			var textarea = new rmpw.text.internal.widgets.RichTextWidget({
	  					name: "myarea",
	  					value: about,
	  					style: "min-height: 150px"
			},input2TD);
			dojo.addClass(textarea.domNode, ReviewUtil.RICHTEXT_READWRITE);
			textarea.startup();
			
			dojo.connect(textarea, "onChange", dojo.hitch(this, function() {
				section.reviewController.updateResource(data.resourceUri, "reviewArtifactInstructions",textarea.get('value')); 
			}));
	    	return inputTD;
	    };
	    
	    var editAboutInputCell= dojo.hitch(this, function(section, table, fieldName, displayName, data, tableRow, inputTD){
			var childNodes= inputTD.childNodes;
			for(var index=0; index<childNodes.length; index++){
				var child= childNodes[index];
				var widget= dijit.byId(childNodes[index].id);
				 if(widget&& widget.get('value')!== data.reviewArtifactInstructions){
					 widget.set("value", data.reviewArtifactInstructions);
				 } 			
			}
	    });
	    return {columnHeader: ActionsRegisteryMessages.about, fieldName: id, width: width, creatorCallback: createAboutInputCell, editingCallback:editAboutInputCell};
	},
	/***********************************************************************************/
	 	
	/***********************************************************************************/
	getResultDefaultCellDescriptor: function(id, width){
		var createResultDefaultCell= function(section, table, fieldName, displayName, data, tableRow, statusTD){
			var resultStatusBar = new dm.review.app.ui.internal.controls.ReviewResultsStatusBar();
		
			var resourceUri= data.resourceUri;
			var popup= new jazz.ui.HoverPopup({
				around: statusTD,
				open: function(){
					var parentCall= popup.getInherited("open", arguments);
					var classConstructor= dojo.getObject(ReviewParticipantViewer);
					if(!classConstructor){
						jazz.core.loader.load_async(ReviewParticipantViewer, dojo.hitch(this, function(){
							dojo.hitch(popup, parentCall)();									
						}));
					}else{
						dojo.hitch(popup, parentCall)();
					}
				},
				
				createContents: dojo.hitch(this, function(resourceUri) {
					/*No need to check for null here */					
					var classConstructor= dojo.getObject(ReviewParticipantViewer);
					
					var hover= new classConstructor({
						reviewController: this.reviewController,
						resourceUri: resourceUri,
						resourceName: data.title
					});
					hover.startup();
					return hover.domNode;
				},resourceUri)
			});
	
			popup.connect(this, "destroy", function() {
				popup.destroyRecursive();
			});
	
			resultStatusBar.renderResourceResults(this.reviewController.review, resourceUri);
			statusTD.appendChild(resultStatusBar.domNode);
	    	return statusTD;
	    };
	    return {columnHeader: ActionsRegisteryMessages.status, fieldName: id, width: width, creatorCallback: createResultDefaultCell};
	},
	/***********************************************************************************/

	/***********************************************************************************/
	getResourceCompletedValueDefaultCellDescriptor: function(id, width){
		var createCompletedValueDefaultCell= function(section, table, fieldName, resourceUri, data, tableRow, completedTD){
			var resultDetails= section.reviewController.review._reviewResults.getResultsDetailsForResource(data.resourceUri);				
			completedTD.appendChild(dojo.doc.createTextNode(resultDetails.percentageVotesCast+"%"));
	    	return completedTD;
		};
	    return {columnHeader: ActionsRegisteryMessages.completed, fieldName: id, width: width, creatorCallback: createCompletedValueDefaultCell};
	},
	/***********************************************************************************/
	
	
	/***********************************************************************************/
	getUserDefaultCellDescriptor: function(id, width){
		var createUserDefaultCell= function(section, table, fieldName, userURI, data, tableRow, userTD){
			if(!userURI){ 
				userURI= data.participant.contributor;
			}
			var userInfo= this.reviewController.getUserInfo(userURI);
 			var name= userInfo?userInfo.name: "-";	/*Safe check */
 			
			var requesterLink = new jazz.ui.ResourceLink({uri: userURI, presentationUri: ReviewUtil.decodeUserUri(userURI), label: jazz.util.html.escape(name)});
			requesterLink.domNode.onclick= dojo.hitch(this, function(requesterLink, e){
				dojo.stopEvent(e);
				var domNode= requesterLink.domNode;
				domNode.target = "_blank";
				section.reviewController.navigateTo(domNode.href);
			},requesterLink);
			userTD.appendChild(requesterLink.domNode);
	        return userTD;
		};
	    return {columnHeader: ActionsRegisteryMessages.name, fieldName: id, width: width, creatorCallback: createUserDefaultCell};
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	getTypeDefaultCellDescriptor: function(id, width){
	 	var createParticipantRoleImage= function(participantRoleType) {
			var img = dm.review.app.util.ReviewActionsRegistery.createParticipantRoleImage(participantRoleType);
			img.className = "participantRoleImage";
			return img;
		};
		
		var createTypeDefaultCell= function(section, table, fieldName, participantRoleType, data, tableRow, roleTD){
			var reviewRole= participantRoleType?participantRoleType:data.participant.reviewRole;
			var roleDisplayName = ReviewTypes.getParticipantRoleTypeDisplayName(reviewRole);
			var roleImage = createParticipantRoleImage(reviewRole);
			roleTD.appendChild(roleImage);
			var roleSpan = document.createElement("span");
			roleSpan.className = roleSpan;
			roleSpan.appendChild(document.createTextNode(roleDisplayName));
			roleTD.appendChild(roleSpan);
	        return roleTD;
		};
	    return {columnHeader: ActionsRegisteryMessages.participantRole, fieldName: id, width: width, creatorCallback: createTypeDefaultCell};
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	getParticipantResultDefaultCellDescriptor: function(id, width){
	 	
	 	var createResultDefaultCell= function(section, table, fieldName, artifactInfo, data, tableRow, statusTD){
	    	var div = document.createElement("div");
	    	var reviewRole = data.reviewRole;
			if (reviewRole == ReviewTypes.PARTICIPANT_ROLE_TYPES.OptionalReviewer){
				div.appendChild(document.createTextNode(ActionsRegisteryMessages.notApplicable));
			} else {
				var review= section.reviewController.review;
				var resultStatusBar = new dm.review.app.ui.internal.controls.ReviewResultsStatusBar({
						role: reviewRole
				});
				var participant= data.contributor;
				
				var userInfo= this.reviewController.getUserInfo(participant);
 				var name= userInfo?userInfo.name: "-";	/*Safe check */
				
				resultStatusBar.setUserUri(review, participant);
				
				var popup= new jazz.ui.HoverPopup({
					around: statusTD,
					userUri: participant,
					open: function(){
						var parentCall= popup.getInherited("open", arguments);
						var classConstructor= dojo.getObject(ReviewResourceViewer);
						if(!classConstructor){
							jazz.core.loader.load_async(ReviewResourceViewer, dojo.hitch(this, function(){
								dojo.hitch(popup, parentCall)();									
							}));
						}else{
							dojo.hitch(popup, parentCall)();
						}
					},
					
					createContents: dojo.hitch(this, function(resourceUri) {
						/*No need to check for null here */					
						var classConstructor= dojo.getObject(ReviewResourceViewer);
						
						var hover= new classConstructor({
							reviewController: this.reviewController,
							userUri: participant,
							userName: name
						});
						hover.startup();
						return hover.domNode;
					},participant)
				});

				popup.connect(this, "destroy", function() {
					popup.destroyRecursive();
				});

				div.appendChild(resultStatusBar.domNode);
			}
			statusTD.appendChild(div);
	    	return statusTD;
	 	};
	    return {columnHeader: ActionsRegisteryMessages.reviewResults, fieldName: id, width: width, creatorCallback: createResultDefaultCell};
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	getParticipantCompletedValueDefaultCellDescriptor: function(id, width){
	 	var createCompletedValueDefaultCell= function(section, table, fieldName, artifactInfo, data, tableRow, statusTD){
			if (data.reviewRole == ReviewTypes.PARTICIPANT_ROLE_TYPES.OptionalReviewer){
				statusTD.appendChild(dojo.doc.createTextNode(ActionsRegisteryMessages.notApplicable));
			} else {
				statusTD.appendChild(dojo.doc.createTextNode(section.reviewController.review.getUserResults(data.contributor, true).resultDetails.percentageVotesCast+"%"));
			}
			return statusTD;
	 	};
	    return {columnHeader: ActionsRegisteryMessages.completed, fieldName: id, width: width, creatorCallback: createCompletedValueDefaultCell};
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	getParticipantResultsCellDescriptor: function(id, width){
 	    var createResultCell= function(result, tableRow,statusTD, status){
	        statusTD.appendChild(dm.review.app.util.ReviewActionsRegistery.createArtifactStatusTypeImage(status));
	        var statusTxt = ReviewTypes.getArtifactTypeDisplayName(status);
	        statusTD.appendChild(document.createTextNode(" "+statusTxt));
	        statusTD.style.minWidth= "100px";
	        return statusTD;
	    };
	    
	 	var createResultDefaultCell= function(section, table, fieldName, result, data, tableRow, td){
			if(result){
				return createResultCell(result, tableRow, td, result);
			}else{
				return createResultCell(result, tableRow,td, ReviewTypes.ARTIFACT_STATUS_TYPES.NotStarted);
			}
	    };
	    return {columnHeader: ActionsRegisteryMessages.status, fieldName: id, width: width, creatorCallback: createResultDefaultCell};
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	getResourceResultsCellDescriptor: function(id, width){
 	    var createResultCell= function(section, table, fieldName, status, data, tableRow, statusTD){
			statusTD.style.minWidth= "100px";
			var imgAnchor = document.createElement("A");
	        imgAnchor.href = "#";
	        var img = dojo.create("IMG", { align:"centre"});
	        img.title = ReviewTypes.getArtifactTypeDisplayName(status);
	        img.src = ReviewImageService.RESET;
	        img.style.verticalAlign = "middle";
	        imgAnchor.style.verticalAlign = "middle";
	        imgAnchor.appendChild(dm.review.app.util.ReviewActionsRegistery.createArtifactStatusTypeImage(status));
	        statusTD.appendChild(imgAnchor);
	        statusTD.appendChild(document.createTextNode(" "+ReviewTypes.getArtifactTypeDisplayName(status)));
	        return statusTD;
	    };
	    return {columnHeader: ActionsRegisteryMessages.reviewResults, fieldName: id, width: width, creatorCallback: createResultCell};
	},
	/***********************************************************************************/
	
	
	/***********************************************************************************/
	getReviewResourcePreviewActionDescriptor: function(id){
		var showPreviewAction= function(dataSetEntry,td,btn, resourceLink, e){
			setTimeout(dojo.hitch(resourceLink, function() {
				if (this.popup && !this.popup._isOpen) {
					this.popup.open();
				}
			}), 0);
		};
		
		var createPreviewAction= dojo.hitch(this, function(dataSetEntry,td,btn, section, table){
        	var data= dataSetEntry.data;
        	//failsafe
        	if(!data || !data.exists){
        		return;
        	}
        	var resourceUri= data.resourceUri;
        	
        	if(!resourceUri){
        		resourceUri= data.artifactId;
        	}
        	
        	if(!resourceUri){
        		return;
        	}
        	return this._createResourcePreviewForButton(resourceUri, section.reviewController, btn);
        });
		
		return {
			imgSrc: ReviewImageService.PREVIEW,
			id: id?id:"createPreviewAction",
			action: showPreviewAction,
			preAction: createPreviewAction,
			label: ActionsRegisteryMessages.previewResource
		};
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	getReviewElementResetActionDescriptor: function(id){
        var createResetResourcesAction= function(dataSetEntry,tableData,btn,resourceLink,section, table, e){
			section.reviewController.resetResource(dataSetEntry.data.resourceUri);
        };
		
		return {
			imgSrc: ReviewImageService.RESET,
			id: id?id:"createResetAction",
			action: createResetResourcesAction,
			label: ActionsRegisteryMessages.resetResource
		};
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	getDeleteReviewResourceDescriptor: function(id){
        var createDeleteAction= function(dataSetEntry,tableData,btn,args,section, table, e){
	    	section.reviewController.removeResource(dataSetEntry.data.resourceUri);
	    };
		
		return {
			imgSrc: ReviewImageService.REMOVE,
			id: id?id:"createDeleteReviewResourceAction",
			action: createDeleteAction,
			label: ActionsRegisteryMessages.removeResource
		};
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	getShowResourceDescriptionAction: function(id){
		var createDescriptionAction= function(dataSetEntry,tableData,btn,args,section, table, e){
		
			var data= dataSetEntry.data;
			
			var getAboutValue= function(){
		    	var instructions;
		    	var resourceUri= data.resourceUri;
        	
	        	if(!resourceUri){
	        		return null;
	        	}
		    	var entry= section.reviewController.review.getReviewResource(resourceUri);
		    	if(entry){	/*Safe check */
		    		instructions= entry.reviewArtifactInstructions;
		    		if(!instructions){
			    		instructions= "";
			    	}
		    	}
		    	return instructions;
	    	};
	        var tooltip = new HoverPopup({
					around: btn.domNode,
					html: ActionsRegisteryMessages.noAboutAssociated,
					openDelay: 300,
					orientation: "L",
					enabled: false
			});
			var onClose= dojo.hitch(tooltip, function(){
				this.enabled= false;
			});
			section.connect(tooltip, "close", onClose);
			
			 setTimeout(dojo.hitch(tooltip, function() {
				if (!this._isOpen) {
					var instructions=  getAboutValue();
					if(!instructions|| dojo.string.trim(instructions)==""){
						instructions= ActionsRegisteryMessages.noAboutAssociated;
					}
					this.html= instructions;
					this.enabled= true;
					this.open();
				}
			}), 0);
		};
			
		var descripionEnablement= function(tableData, dataSetEntry,section, table, simpleToolbar, id){
	    	var data= dataSetEntry.data;
			var updater= section.reviewController;
						
			var resourceUri= data.resourceUri;
        	
        	if(!resourceUri){
        		resourceUri= data.artifact;
        	}
        	if(!resourceUri){
        		return false;
        	}
			var info= updater.review.getReviewResource(resourceUri);
			var instructions;
			if(info){
				instructions= info.reviewArtifactInstructions;
				if(!instructions){
					return false;
				}
			}else{
				return false;
			}
			return true;
	    };
	    
		return {
			imgSrc: ReviewImageService.ABOUT,
			id: id?id:"createDescriptionAction",
			action: createDescriptionAction,
			actionEnablement: descripionEnablement,
			label: ActionsRegisteryMessages.showAbout
		};
				
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	getReviewResourceRedirectionAction: function(id){
		var createResourcesRedirectionAction= function(dataSetEntry,tableData,btn,args,section, table, e){
            var object= {
            	resourceUri: section.reviewController.review.resourceUri,
            	tab: "resources",
            	tabId: "resources",
            	projectName: section.reviewController.context.name,
            	select:dataSetEntry.data.resourceUri.match(/\/(\d+)$/)[1],	/*No need for any check for NPE, urls here; otherwise, something is wrong */
            	type: "reviewEditor",
            	ts: new Date().getTime()
            };
            window.location.href = "#action=dm.app.review.viewResource&"+ dojo.objectToQuery(object);
        };
        		
		return {
			imgSrc: ReviewImageService.STATUS_DETAILS,
			id: id?id:"reviewResourceRedirectionAction",
			action: createResourcesRedirectionAction,
			label: ActionsRegisteryMessages.redirectResource
		};
		
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	getChangeParticipantActionDescriptor: function(id){
		var createChangeParticipantsAction= function(dataSetEntry,tableData,btn,args,section, table, e){
			dm.review.app.util.ReviewActionsRegistery.updateParticipantRole(dataSetEntry.data.contributor, section.reviewController);
        };
        
        return {
			imgSrc: ReviewImageService.CHANGE_PARTICIPANT_ROLE,
			id: id?id:"changeParticipantAction",
			action: createChangeParticipantsAction,
			label: ActionsRegisteryMessages.changeParticipantRole
		};
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	getDeleteReviewParticipantDescriptor: function(id){
        var createDeleteAction= function(dataSetEntry,tableData,btn,args,section, table, e){
        	section.reviewController.removeReviewer(dataSetEntry.data.contributor);
	    };
		
		return {
			imgSrc: ReviewImageService.REMOVE,
			id: id?id:"createDeleteReviewParticipantAction",
			action: createDeleteAction,
			label: ActionsRegisteryMessages.removeParticipants
		};
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	getReviewParticipantRedirectionAction: function(id){
		var createParticipantRedirectionAction= function(dataSetEntry,tableData,btn,args,section, table, e){
            var contributor= ReviewUtil.decodeUserUri(dataSetEntry.data.contributor);
            var object= {
            	resourceUri: section.reviewController.review.resourceUri,
            	select: contributor.substring(contributor.lastIndexOf("/")+1,contributor.length),	/*No need for any check for NPE, urls here; otherwise, something is wrong */
            	tab: "participants",
            	tabId: "participants",
            	projectName: section.reviewController.context.name,
            	type: "reviewEditor",
            	ts: new Date().getTime()
            };
            window.location.href = "#action=dm.app.review.viewResource&"+ dojo.objectToQuery(object);
        };
        
        var actionEnablement= function(tableData, dataSetEntry,section, table, simpleToolbar, id){
	    	return dataSetEntry.data.reviewRole != ReviewTypes.PARTICIPANT_ROLE_TYPES.OptionalReviewer;
	    };
        		
		return {
			imgSrc: ReviewImageService.STATUS_DETAILS,
			id: id?id:"reviewParticipantRedirectionAction",
			actionEnablement: actionEnablement,
			action: createParticipantRedirectionAction,
			label: ActionsRegisteryMessages.redirectParticipant
		};
		
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	getElementsTableHeaderArguments: function(reviewController){
		if(reviewController.review.isModerator() && reviewController.isEditable() && !reviewController.isChangesetReview()){
			return {sectionTitle: ActionsRegisteryMessages.modelElements, sectionDescription: ActionsRegisteryMessages.modelElements,
				uaKey: REVIEW_ELEMENT_SECTION_UAKEY, actions: [{ title: ActionsRegisteryMessages.add,tooltip: ActionsRegisteryMessages.add,
					doRun: dojo.hitch(dm.review.app.util.ReviewActionsRegistery, function(){
						dm.review.app.util.ReviewActionsRegistery.addResource(reviewController);
					})
				}]
			};			
		}
		return {sectionTitle: ActionsRegisteryMessages.modelElements, sectionDescription: ActionsRegisteryMessages.modelElements, uaKey: REVIEW_ELEMENT_SECTION_UAKEY};
	},
	/***********************************************************************************/
	
	getParticipantsTableHeaderArguments: function(reviewController){
		if(reviewController.review.isModerator() && reviewController.isEditable()){
			return {
				sectionTitle: 		ActionsRegisteryMessages.participants,
				sectionDescription: ActionsRegisteryMessages.participants,
				uaKey:				REVIEW_PARTICIPANTS_SECTION_UAKEY,
				actions: [{
					title: ActionsRegisteryMessages.addParticipants,
					tooltip: ActionsRegisteryMessages.addParticipants,
					doRun: dojo.hitch(dm.review.app.util.ReviewActionsRegistery, dm.review.app.util.ReviewActionsRegistery.addParticipant, reviewController)
				}]
	
			};
		}
		return {
			sectionTitle: 		ActionsRegisteryMessages.participants,
			sectionDescription: ActionsRegisteryMessages.participants,
			uaKey:				REVIEW_PARTICIPANTS_SECTION_UAKEY
		};
	},
	/***********************************************************************************/
	
	/*********************Toolbar buttons registration**********************************/
	registerDeleteReviewResourceButton: function(toolbar, reviewController, getValueCallback){
        var createDeleteAction= function(){
	    	reviewController.removeResource(getValueCallback());
	    };
		
		toolbar.addButton(reviewController, createDeleteAction, {
			label: ActionsRegisteryMessages.removeResource,
			imgSrc: ReviewImageService.REMOVE
		});
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	registerAddReviewResourceButton: function(toolbar, reviewController){
		toolbar.addButton(dm.review.app.util.ReviewActionsRegistery, function(){
			dm.review.app.util.ReviewActionsRegistery.addResource(reviewController);
		}, {label: ActionsRegisteryMessages.add, imgSrc: ReviewImageService.ADD	});
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	registerAddReviewerButton: function(toolbar, reviewController){
		toolbar.addButton(dm.review.app.util.ReviewActionsRegistery, function(){
			dm.review.app.util.ReviewActionsRegistery.addParticipant(reviewController);
		}, {label: ActionsRegisteryMessages.add, imgSrc: ReviewImageService.ADD	});
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	registerRemoveReviewerButton: function(toolbar, reviewController, func){
		toolbar.addButton(reviewController, function(){
			reviewController.removeReviewer(func());
		}, {label: ActionsRegisteryMessages.remove, imgSrc: ReviewImageService.REMOVE});
	},
	/***********************************************************************************/
			
	/***********************************************************************************/
	registerUpdateReviewerRoleButton: function(toolbar, reviewController, func){
		toolbar.addButton(reviewController, function(){
			dm.review.app.util.ReviewActionsRegistery.updateParticipantRole(func(), reviewController);
		}, {label: ActionsRegisteryMessages.changeParticipantRole, imgSrc: ReviewImageService.CHANGE_PARTICIPANT_ROLE});
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	registerResetReviewResourceButton: function(toolbar, reviewController, getValueCallback){
        var createResetAction= function(){
	    	reviewController.resetResource(getValueCallback());
	    };
		
		toolbar.addButton(reviewController, createResetAction, {
			label: ActionsRegisteryMessages.resetResource,
			imgSrc: ReviewImageService.RESET
		});
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	registerChangeStatusButton: function(toolbar, reviewController, resourceUri){
		return toolbar.addButton(dm.review.app.util.ReviewActionsRegistery, dojo.hitch(this, function(reviewController, resourceUri){
			dm.review.app.util.ReviewActionsRegistery.submitResults(reviewController, resourceUri);
		},reviewController, resourceUri), {
			label: ActionsRegisteryMessages.changeStatus,
			imgSrc: ReviewImageService.MYDECISION
		});
		
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	registerPreviewReviewResourceButton: function(toolbar, reviewController, resourceUri){
		var resourceLink;
		var showPreviewAction= function(){
			setTimeout(dojo.hitch(resourceLink, function() {
				if (this.popup && !this.popup._isOpen) {
					this.popup.open();
				}
			}), 0);
		};
		
		var btn= toolbar.addButton(reviewController, showPreviewAction, {
			label: ActionsRegisteryMessages.previewResource,
			imgSrc: ReviewImageService.PREVIEW
		});
		
		resourceLink= this._createResourcePreviewForButton(resourceUri, reviewController, btn);
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	getElementsDescription: function(reviewController, collapsed){
		var description = ActionsRegisteryMessages.modelElements;
		if(!collapsed){
			description += " (" + reviewController.review.model.resources.dataSet.length + ")";
    	}
		return description;
    },
    /***********************************************************************************/
	
	/***********************************************************************************/
	getParticipantsDescription: function(reviewController, collapsed){
		var description = ActionsRegisteryMessages.participants;
		if(!collapsed){
			description += " (" + reviewController.review.model.participants.dataSet.length + ")";
    	}
		return description;
    },
    /***********************************************************************************/
    
	/****************************Dialogs Calls******************************************/
	/***********************************************************************************/
	addResource: function(reviewController) {
		var classConstructor= dojo.getObject(PickerDialog);
		if(classConstructor){
			dm.review.app.util.ReviewActionsRegistery._addResource(reviewController, classConstructor);
		}else{
			jazz.core.loader.load_async(PickerDialog, dojo.hitch(this, function(){
				dm.review.app.util.ReviewActionsRegistery._addResource(reviewController, dojo.getObject(PickerDialog));									
			}));			
		}
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	addParticipant: function(reviewController) {
		var form= "";
		dm.review.app.util.ReviewActionsRegistery.loadAndRun(AddParticipantsForm, {
			projectURI: reviewController.context.uri,
			onOK: function(selectedUsers){
				reviewController.addReviewer(selectedUsers, form.roleType);
			},
			filteredUsers: reviewController.getSelectedUserIds()
		}, function(value){form= value;});
	},
	/***********************************************************************************/
	
	/*
	 * Make sure to invoke  a dialog at the context of Review Controller, so we make sure that other places that use the  review ocntroller will be
	 * invoking the dialogs at the review controller level (iframe, etc) 
	 */
	invokeDialog: function(floatingPaneParams){
		return new jazz.ui.Dialog(floatingPaneParams);
	},
	
	/* 
	 * Load and run a given clazz asynchronously, and pass in the "args" in the consturctor, and then make a cllback for the given "func"
	 * 
	 * What's it used for? Make sure to avoid using dojo.reuire for the classes that we do not use right away (such as dialogs)
	 * Why's is it required to be here? I order to make sure that the class will be constructed at the scope of review controller, so all places will have a unified scope for that clazz
	 *   
	 */
	loadAndRun: function(clazz, args, func){
		var loader= function(classConstructor){
			var load = new classConstructor(args);
			if(dojo.isFunction(func)){
				func(load);
			}
		};
		var classConstructor= dojo.getObject(clazz);
		if(classConstructor){
			loader(classConstructor);
		}else{
			jazz.core.loader.load_async(clazz, dojo.hitch(this, function(){
				loader(dojo.getObject(clazz));									
			}));			
		}
	},
	
	/***********************************************************************************/
	updateParticipantRole: function(userUri, reviewController) {
		var form= "";
				
		dm.review.app.util.ReviewActionsRegistery.loadAndRun(ChangeParticipantsForm, {
				participant: reviewController.review.getParticipant(userUri),
				onOk: dojo.hitch(this, function(participantRoleType) {
					reviewController.updateReviewer(userUri, [
						{"field": "reviewRole", "value": form.selectedParticipantRoleType},
						{"field": "result"}, {"field": "artifactInfo"}, {"field": "Actions"}
					]);
				})
		}, function(value){
			form= value;
			form.show();
		});
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	submitResults: function(reviewController, resourceUri){
		var form= "";
		dm.review.app.util.ReviewActionsRegistery.loadAndRun(ChangeElementStatusForm, {
				review: reviewController.review,
				onOk: dojo.hitch(this, function(statusType) {
					reviewController.submitResult(resourceUri, statusType);
				}
			)
		}, function(value){
			form= value;
			form.show();
		});
	},
	/***********************************************************************************/
	
	/****************************Private methods****************************************/
	/***********************************************************************************/
	_createResourcePreviewForButton: function(resourceUri, reviewController, btn){
		var configuration= reviewController.review.getReviewAttribute("reviewBaseline");
		var object= {
    		resourceUri: resourceUri
    	};
    	if(configuration&& configuration!= ""){
    		if (reviewController.isChangeset(configuration)){
    			object[IRmpsConstants.BASELINE]= reviewController.workspace;
    			object["changeset"]= configuration;
    		} else {
    			object[IRmpsConstants.BASELINE]= configuration;
    		}
    	}
    	 
    	var resourceLink = new jazz.ui.ResourceLink({
			uri: net.jazz.ajax._contextRoot+ "/rendering?"+ dojo.objectToQuery(object),
			presentationUri: reviewController.getFullUrl(resourceUri),
			lazyFetch : true,
			avoidBaselineCheck: true,
			label: " "
		});
		btn.domNode.parentNode.appendChild(resourceLink.domNode);
		
		// This resource link action does not have an image or associated text.
		// Also, if you hit enter when this action has the focus, nothing happens.
		// Therefore, do not allow keyboard traversal to focus on this action. 
		resourceLink.domNode.tabIndex = -1;
		return resourceLink;
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	_addResource: function(reviewController, classConstructor) {
		var dialog = null;
		var addModelElements = dojo.hitch(this, function(resources) {
			if (dialog) {	
				dialog.close();
				dialog.destroyRecursive();
			}
			if (resources && resources.length > 0) {
				var resourceURIs = dojo.map(resources, function (resource) {
					return resource["rdf:resource"];
				});
				var deferred = ResourceInfoClient.getResourceInfo(resourceURIs, null, reviewController.review.getReviewAttribute("reviewBaseline"));
				deferred.addCallback(this, function(resourceInfos) {
					dojo.forEach(resourceInfos, function(resourceInfo) {
						reviewController.addResource(resourceInfo);					
					});
				});
			}
		});
		var configuration= reviewController.review.getReviewAttribute("reviewBaseline");
		if(configuration== ""){
			configuration= null;
		}
		
		var picker = new classConstructor({
			projectId: jazz.app.context.get().itemId,
			context: configuration,
			filteredList: reviewController.review.getReviewResources(null, []),
			findFragments: true,
			enableMultiSelect: true,
			onOK: addModelElements
		});
		
		var dialogParams = {
            primaryTitle: ActionsRegisteryMessages.resourceSelectionTitle,
            secondaryTitle: ActionsRegisteryMessages.resourceSelectionSecondTitle,
	        width: "1000px",
	        height: "550px",
	        contentNode: picker.domNode
        };
		dialog = new jazz.ui.Dialog(dialogParams);
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	createParticipantRoleImage: function(participantRoleType) {
    	var title= ReviewTypes.getParticipantRoleTypeDisplayName(participantRoleType, false);
    	if(!title){
    		return null;
    	}
		var img = document.createElement("img");
		img.title = title;
		img.alt = title;
		img.src = ReviewTypes.getParticipantRoleTypeImageURI(participantRoleType);
		return img;
	},
	/***********************************************************************************/
	
	/***********************************************************************************/
	createArtifactStatusTypeImage: function(artifactStatusType) {
		var img = document.createElement("img");
		var title= ReviewTypes.getArtifactStatusTypeDisplayName(artifactStatusType);
		img.title = title;
		img.alt= title;
		img.src = ReviewTypes.getArtifactStatusTypeImageURI(artifactStatusType);
		return img;
	},
	/***********************************************************************************/
		
	/***********************************************************************************/
	getFullEditResourceUrl: function(reviewController, resourceUri) {
		return reviewController.getProjectPrefix() + DesignUtil.getEditResourceUri(resourceUri);
	},
	/***********************************************************************************/

	/***********************************************************************************/
	createResNotExistNode: function (node, elementName){
		node.appendChild(dm.review.app.util.ReviewActionsRegistery.createArtifactStatusTypeImage("NotExist"));
		node.appendChild(dojo.doc.createTextNode(elementName));
		dojo.addClass(node, "review-res-not-exist");
	}
};
})();