/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2013. All
 * Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

/**
 * Utility for Text manipulation (wrapping etc.)
 * 
 * @author Steve Shaw
 */
(function(){
	
	dojo.provide("rmpw.common.util.TextUtil");
	
	dojo.require("dojox.html.metrics");
	
	var util = rmpw.common.util.TextUtil;
	
	dojo.mixin(util, {
	    
		getWrappedText: function(text, width, height, fontStyle) {
			var lines = util.getTextLines(text, width, height, fontStyle);
			
			var wrappedText = "";
			for (var i=0, lg=lines.length; i<lg; i++) {
				wrappedText += lines[i] + "\n";
			}
			
			return wrappedText;
		},
		
		getTextLines: function(text, width, height, fontStyle) {
			// break text into paragraphs (paragraphs terminate in '\n' or '\r\n' or '<br>')
			var paragraphs = text.split('\n'),
			  s;
			if (width === -1) {
				return paragraphs;
			}
			
			// break each paragraph into lines
			var lines = [];
			var remainingShapeHeight = (height === -1) ? 4294967295 : height; // if -1, we got all the space in the world
			var outOfSpace = false;
			for(var pinx = 0; pinx < paragraphs.length && !outOfSpace; pinx++){
				var str = paragraphs[pinx];
				 var textMetric = util._getBoundingTextBox(str, fontStyle);    
		            
				var len = str.length;
				// process empty line (\n\n\n)
				if (len === 0) {
					remainingShapeHeight -=textMetric.h;
			        lines.push(" ");
			        continue;
				}
				var inx = 0;
				var isLastLine = false;
				var charsPerRow = Math.round(str.length * width / textMetric.w);
				while (inx < len) {
					remainingShapeHeight -=textMetric.h;
					outOfSpace = (remainingShapeHeight - textMetric.h) < 0;
					
					// fine-tune charsPerRow if last line doesn't fit shape
					if (inx+charsPerRow >= len && !isLastLine) {
						charsPerRow = util._fitInWidth(str.substr(inx), fontStyle, width).length;
					}
					
					// should we wrap?
					if ((inx+charsPerRow) < len) {
						// get rough string
						s = str.substr(inx, charsPerRow);
						
						// break on a space if we can
				        var i = s.lastIndexOf(" ");
				        if (i > 0) {
	                        s = s.substring(0, i);
				        } else {
	                        // is rough string still too long for shape?
							s = util._fitInWidth(s, fontStyle, width);
				        }
						inx += s.length;
				        s = util._addEllipse(s, outOfSpace, fontStyle, width);
				        lines.push(s);
					} else {

	                    s = str.substr(inx);
	                    if(outOfSpace && pinx + 1 !== paragraphs.length) {
	                        s = util._addEllipse(s, outOfSpace, fontStyle, width);
	                    }
	                    lines.push(s);
	                    inx = len;
	                }
	                if(outOfSpace){
	                   break;
	                }
	            }
	        }

			return lines;		
	    },
	    
		_fitInWidth: function(s, fontStyle, width) {
			var isTooBig = false;
			if (s.length>0) {
				do {
					isTooBig = util._getBoundingTextBox(s, fontStyle).w > width;
					if (isTooBig) {
			        	s = s.substring(0, s.length-2);
					}
				} while (isTooBig);
			}
		    return s;
		},	
		
	    _addEllipse: function(s, outOfSpace, fontStyle, width) {
			s = dojo.trim(s);
		    if (outOfSpace) {
		    	var lineWidth = util._getBoundingTextBox(s, fontStyle).w;
		    	if (lineWidth > width) {
		        	s = s.substring(0, s.length-3);
		    	}
		    	s +="...";
		    }
		    return s;
		},
		
		_getBoundingTextBox: function(text, fontStyle) {
			var style = null;
			if(fontStyle) {
				style = {
					fontFamily: fontStyle.family,
					fontSize: fontStyle.size,
					fontWeight: fontStyle.weight,
					whiteSpace: "nowrap"
				};
			}
			// dojox.html.metrics.getTextBox( ) doesn't like "<"
			// replace with character of equal size
			if (text.indexOf("<") !=-1) {
				text = text.replace(/</g,">");
			}
			return dojox.html.metrics.getTextBox(text, style);
		},
	});
    
})(); // end function()