/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011. All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("rmpw.dsv.editor.xhtml.util.XhtmlConverter");

dojo.require("dojo.string");
dojo.require("dojox.string.Builder");

/**
 * Prototype implementation of an XHTML <--> JSON Conversion algorithm.
 * Currently used client-side as a part of Rich Text doc prototype.
 * Should be moved to server-side for performance and scalability reasons.
 */

rmpw.dsv.editor.xhtml.util.XhtmlConverter = {
	ATTRIBUTE_TEMPLATE: ' ${dmxml_key} = "${dmxml_value}"',
	USE_RDF_ABOUT: false,
	
    parse: function(content){
        var xml = new dojox.string.Builder();
        if (this.isXhtmlRoot(content)) {
			this.serializeObject(content.dmxhtml_body, xml);
        }
        return xml.toString();
    },
    
    isXhtmlRoot: function(object){
        if (!object || !object.rdf_type) {
            return false;
        }
        return dojo.some(object.rdf_type, function(type) {
        	return type.rdf_about == "http://jazz.net/ns/dm/document#Document";
        });
    },
    
    isCDATA: function(object){
        if (!object || !object.rdf_type) {
            return false;
        }
        return dojo.some(object.rdf_type, function(type) {
        	return type.rdf_about == "http://jazz.net/ns/dm/xml#CDATA";
        });
    },
    
    serializeObject: function(object, xml){
        if (this.isCDATA(object)) {
            if (object.dmxml_text) {
                xml.append(object.dmxml_text);
            }
        }
        else {
            if (!object.dmxml_name) {
                return xml;
            }
            xml.append("<", object.dmxml_name);
            this.serializeAttributes(object, xml);

			if (this.USE_RDF_ABOUT && object.rdf_about) {
				xml.append(" id=",object.rdf_about.substring(object.rdf_about.lastIndexOf("#") + 1));
			}

            if (object.dmxml_content) {
                xml.append(">");
                var i, len = object.dmxml_content.length;
                for (i = 0; i < len; i++) {
                    this.serializeObject(object.dmxml_content[i], xml);
                }
                xml.append("</", object.dmxml_name, ">");
            } else if (object.dmxhtml_body) {
				this.serializeObject(object.dmxhtml_body, xml);
                xml.append("</", object.dmxml_name, ">");				
			}
            else {
                xml.append("/>");
            }
        }
		return xml;
    },
	
    serializeAttributes: function(object, xml){
        if (object.dmxml_attributes) {
            var i, attribute;
            for (i = 0; i < object.dmxml_attributes.length; i++) {
				attribute = object.dmxml_attributes[i];
				if (attribute.dmxml_key && attribute.dmxml_value) {
					xml.append(dojo.string.substitute(this.ATTRIBUTE_TEMPLATE, attribute));
				}
            }
        }
		return xml;
    },


// XHTML to JSON


    parseToJson: function(xhtml, resourceUri){
        var stack = [];
        var root = this.createXhtmlRoot(null, resourceUri);
        var parser = new CKEDITOR.htmlParser();
		var context = this.createXhtmlBody(root, null);
		root.dmxhtml_body = context;

        parser.onTagOpen = dojo.hitch(this, function(tagName, attributes, selfClosing){
            var tag = null;
            if (tagName === "html") {
                tag = this.createXhtmlRoot(attributes, resourceUri);
				tag = dojo.mixin(root, tag);
            } else if (tagName === "body") {
				tag = this.createXhtmlBody(root, attributes);
				tag = dojo.mixin(context, tag);
				root.dmxhtml_body = tag;
			} else {
                context.dmxml_content = context.dmxml_content || [];
                tag = this.createTag(context, tagName, attributes);
                context.dmxml_content.push(tag);
                stack.push(context);
            }
            context = tag;
        });

        parser.onTagClose = dojo.hitch(this, function(tagName) {
            context = stack.pop();
        });

        parser.onText = dojo.hitch(this, function(text){
            context.dmxml_content = context.dmxml_content || [];
            var cdata = this.createCDATA(context, text);
            context.dmxml_content.push(cdata);
        });
		
        parser.parse(xhtml);
        return root;
    },

    createXhtmlRoot: function(attributes, resourceUri){
        var root = dojo.mixin(this.createTag(null, "html", attributes), {
            rdf_type: [
				{ rdf_about: "http://jazz.net/ns/dm/document#Document" },
				{ rdf_about: "http://jazz.net/ns/dm/core#Document" }
			],
			rdf_about: ""
        });
        return root;
    },

    createXhtmlBody: function(root, attributes){
        var body = dojo.mixin(this.createTag(root, "body", attributes), {
            rdf_type: [{ rdf_about: "http://jazz.net/ns/dm/document#Body" }]
        });
        return body;
    },

    createTag: function(context, tagName, attributes) {
        var tag = this.createRdfObject("http://jazz.net/ns/dm/xml#BodyTag");
		if (tagName) {
			tag.dmxml_name = tagName;
		}
		if (attributes) {
            tag.dmxml_attributes = tag.dmxml_attributes || [];			
			if (this.USE_RDF_ABOUT && attributes.id) {
				tag.rdf_about = attributes.id;
			}
		}
		
		if (this.USE_RDF_ABOUT && !tag.rdf_about && context) {
			tag.rdf_about = context.rdf_about + "T" + String(context.dmxml_content.length);
		}

        for (var att in attributes) {
			if (!(this.USE_RDF_ABOUT && att !== "id")) {
				var attrib = this.createAttribute(tag, att, attributes[att]);
				tag.dmxml_attributes.push(attrib);
			}
        }
        return tag;
    },

    createAttribute: function(context, key, value) {
        var attrib = dojo.mixin(this.createRdfObject("http://jazz.net/ns/dm/xml#Attribute"), {
            dmxml_key: key,
            dmxml_value: value
        });
		if (this.USE_RDF_ABOUT && context) {
			attrib.rdf_about = context.rdf_about + "A" + String (context.dmxml_attributes.length);			
		}
		return attrib;
    },

    createCDATA: function(context, text){
        return dojo.mixin(this.createTag(context), {
			rdf_type: [{ rdf_about: "http://jazz.net/ns/dm/xml#CDATA" }],
            dmxml_text: text
        });
    },
    
    createRdfObject: function(rdf_types){
        if (dojo.isArray(rdf_types)) {
        	rdf_types = dojo.map(rdf_types, function(type) {
        		return { rdf_about: type };
        	});
        } else {
            rdf_types = [{ rdf_about: rdf_types }];
        }
        return {
            rdf_type: rdf_types
        };
    },
    
    htmlifyText: function(source){
        // space out rdf:Description blocks
        source = source.replace(/<rdf:Description/g, "\n  <rdf:Description");
        
        // escape angle brackets
        source = source.replace(/</g, "&lt;");
        source = source.replace(/>/g, "&gt;");
        source = "<html xmlns='http://www.w3.org/1999/xhtml/'><pre><code>" + source + "</code></pre></html>";
        return source;
    }
};
