/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("rmps.core.util.xhrUtil");

dojo.require("rmps.core.RootServices");
dojo.require("rmps.core.base.BaseClient");
dojo.require("rmps.app.core.util");

rmps.core.util.xhrUtil = {

		/*
		 * Get the list of the avialble contexts in the server.
		 *
		 * projectName: Specify the context to be returned based on a given
		 * project area name (as the project name is a unique identifier)
		 * It's optional. If not defined, this means that we will return all
		 * contexts. IF defined, but not there, we will return null as a way
		 * to help the caller to understand that something went wrong
		 *
		 * callback: Specify if we want the call to be run in sync manner or
		 * not. So, if callback is defined, then the deferred call won't be
		 * sync, and if it is undefined, then it will make the call
		 * synchronously. It's fine to be make sync calls for these method
		 * as the exepcted places to use the method could be iframe, seprate
		 * resource, and these places will have to init the context at the
		 * very beginning
		 *
		 */
		getContexts : function(projectName, callback) {
			var contexts;
			var projectArea = {
				form : [ {
					name : "archived",
					root : [ "jp06:archived" ],
					numberOfOccurences : 1,
					node : "#text",
					typeFunction : function(val) {
						return val === "true";
					}
				}, {
					name : "name",
					numberOfOccurences : 1,
					attribute : "jp06:name"
				}, {
					name : "uri",
					numberOfOccurences : 1,
					root : [ "jp06:url" ],
					node : "#text"
				}, {
					name : "itemId",
					numberOfOccurences : 1,
					root : [ "jp06:url" ],
					node : "#text",
					typeFunction : function(val) {
						var index = val.lastIndexOf("/");
						return (index === -1 ? val : val.substring(index + 1));
					}
				} ]
			};

		var form= [{name: "projectAreas", root: ["jp06:project-area"], objectForm: projectArea}];

		var deferred= rmps.core.base.BaseClient.getResponse(
				rmps.core.RootServices.getUri(rmps.core.Namespaces.JP_0_6, "projectAreas"),
				{path:["jp06:project-areas"], numberOfOccurences:1, form: form}, {sync: callback?false: true});

			deferred.addCallback(function(projects) {
				projects = projects.projectAreas;
				var returnValue = function(value) {
					if (callback) {
						return callback(value);
					}
					return value;
				};
				if (projectName) {
					for ( var index = 0; index < projects.length; index++) {
						var current = projects[index];
						if (current.name === projectName) {
							contexts = returnValue(current);
						}
					}
				} else {
					contexts = returnValue(projects);
				}
			});

			return callback?deferred:contexts;
	},

	/*
	 *Add paramaters list to a given url
	 */
	addParamaterToUrl: function(url, params, overwrite){
		if(!url|| !params){
			return url;
		}
		var context = jazz.app.context.get();
		var base_url = new dojo._Url(url);
		var query= base_url.query;

		var queryParams = null;

		if (query && dojo.trim(query)) {
			query = query.substring(query.indexOf("?") + 1, query.length);
			queryParams = dojo.queryToObject(query);
		} else {
			queryParams = {};
		}
		for(var param in params){
			var value= params[param];
			if(value){
				var current= queryParams[param];
				if(overwrite || !current){ /*If we enable overwrite, then, we do not care if the parameter already has value; otherwise, we check if this parameter already assigned */
					queryParams[param] = params[param];
				}
			}
		}
		var objectToQuery= dojo.objectToQuery(queryParams);
		if(objectToQuery && dojo.trim(objectToQuery)){
			var firstSegment= "";
			var scheme= base_url.scheme;
			var path= base_url.authority;
			if(scheme && path){
				firstSegment= scheme +"://" + path;
			}
			var normalizedUrl= firstSegment+ base_url.path+ "?"+ dojo.objectToQuery(queryParams);
			url= normalizedUrl;
		}
		return url;
	},

	/*
	 *Add paramaters list to a given args
	 *
	 * @overwite: If it is <code>true</code> then, if the url already contains this paramater, then we replace it, or vice versa
	 */
	addParamater: function(args, params, overwrite){
		var url= args.url;
		if(!url|| !params){
			return args;
		}
		args.url= this.addParamaterToUrl(url, params, overwrite);
		return args;
	}
};

}());