/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *| Copyright IBM Corp. 2009, 2010.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */

(function() {

dojo.provide("rmps.web.comment.sketch.utilities.ChildBounds");

dojo.require("dojox.gfx.path");
dojo.require("rmpw.diagrams.edit.handles.HandleUtils");

dojo.declare("rmps.web.comment.sketch.utilities.ChildBounds", null, {
	
	build: function( changeSet, item, bounds ){
		var type = item.eClass;
		// take out the "Type" postfix and lowercase it
		type = type.substring(0, type.length - 4).toLowerCase();
		this._build[type].call(this, changeSet, item, bounds );
	},
	
	/* TODO: Add handling for these types. */
	//		"circle"
	//		"PolylineType":	
	//		"ImageType":				    	
	//		"TextType":
	
	_build: {
		g: function( changeSet, item, bounds ){
			dojo.forEach(["g", "rect", "path", "line", "ellipse"], function(type) {
				if(type in item) {
					var typeList = item[type];
					var buildFunction = this._build[type];
					for( var child, i = 0; child = typeList[i]; i++ ) {
						buildFunction.call(this, changeSet, child, bounds );
					}
				}
			}, this);
		},
		
		line: function(changeSet, drawing, bounds){
			
			var traces = drawing.__mappings[0].traces;	
			var owner = traces.x1.getOwner();
			var change = changeSet.createChange(owner);

			var x1 = traces.x1.get();
			var y1 = traces.y1.get();
			var x2 = traces.x2.get();
			var y2 = traces.y2.get();
			
			if(bounds.width){

				var LEFT = 0;
				var RIGHT = 1;
				var TOP = 0;
				var BOTTOM = 1;

				var LR = RIGHT; 
				var TB = BOTTOM;

				if( x1 < x2 ){ LR = LEFT; }
				if( y1 < y2 ){ TB = TOP; }

				var matrix = new Array(2);

				for( var m=0; m<4; m++ ){
					matrix[m] = new Array(2);
				}

				this.position = rmpw.diagrams.edit.handles.HandleUtils;

				matrix[ TOP ][ LEFT ] = this.position.TOP_LEFT;
				matrix[ TOP ][ RIGHT ] = this.position.TOP_RIGHT;
				matrix[ BOTTOM ][ LEFT ] = this.position.BOTTOM_LEFT;
				matrix[ BOTTOM ][ RIGHT ] = this.position.BOTTOM_RIGHT;

				var endA = matrix[TB][LR];

				if (TB == TOP) {
					TB = BOTTOM;
				} else {
					TB = TOP;
				}
				if (LR == LEFT) {
					LR = RIGHT;
				} else {
					LR = LEFT;
				}

				var endB = matrix[TB][LR];
				
				var corners = [];
				
				corners[this.position.TOP_LEFT]= { x: bounds.x, y: bounds.y };
				corners[this.position.TOP_RIGHT]= { x: bounds.x + bounds.width, y: bounds.y };
				corners[this.position.BOTTOM_LEFT]= { x: bounds.x, y: bounds.y + bounds.height };
				corners[this.position.BOTTOM_RIGHT]= { x: bounds.x + bounds.width, y: bounds.y + bounds.height };
						
				change.set( "x1", corners[endA].x );
				change.set( "y1", corners[endA].y );
				change.set( "x2", corners[endB].x );
				change.set( "y2", corners[endB].y );
				
			}else{

				var old = {};
				old.x = x1;
				old.y = y1;
				old.width = Math.abs(x1 - x2);
				old.height = Math.abs(y1 - y2);
	
				if( x1 > x2 ){ old.x = x2; }
				if( y1 > y2 ){ old.y = y2; }
	
				var dx = bounds.x - old.x;
				var dy = bounds.y - old.y;

				change.set("x1", x1 + dx );
				change.set("y1", y1 + dy );
				change.set("x2", x2 + dx );
				change.set("y2", y2 + dy );
			}
		},
		
		text: function( changeSet, drawing, bounds ){
			console.log( "TEXT MOVE" );
		},
	
		ellipse: function( changeSet, drawing, bounds ){
			
			var traces = drawing.__mappings[0].traces;	
			var owner = traces.rx.getOwner();
			var change = changeSet.createChange(owner);

			if( bounds.width ){
				change.set("rx", Math.abs( bounds.width/2 ) );
				change.set("ry", Math.abs( bounds.height/2 ) );	
				change.set("cx", bounds.x + bounds.width/2 );
				change.set("cy", bounds.y + bounds.height/2 );
			}else{
				change.set("cx", bounds.x + traces.rx.get() );
				change.set("cy", bounds.y + traces.ry.get() );
			}
		},
	
		rect: function( changeSet, drawing, bounds ){
			
			var traces = drawing.__mappings[0].traces;	
			var owner = traces.x.getOwner();
			var change = changeSet.createChange(owner);
			
			var xcoord = bounds.x;
			var ycoord = bounds.y;
			
			if( bounds.width < 0 ){
				xcoord = bounds.x + bounds.width;
			}
			
			if( bounds.height < 0 ){
				ycoord = bounds.y + bounds.height;
			}
			
			change.set("x", xcoord );
			change.set("y", ycoord );
			
			if( bounds.width ){
				change.set("width", Math.abs( bounds.width ) );
				change.set("height", Math.abs( bounds.height ) );
			}
		},
	
		path: function( changeSet, drawing, bounds ){
			
			var node = document.createElement('div');						
			var path = new dojox.gfx.path.Path(node);							
			path.setShape({path: drawing.d});
			
			var old = rmpw.diagrams.render.util.Workarounds.getBoundingBox(path);					
			var dx = bounds.x - old.x;
			var dy = bounds.y - old.y;
			
			var newpath;
			
			var svgUtil = new rmps.web.comment.sketch.serializer.SVGUtil();
			
			if( bounds.width ){
				var scalex = Math.abs( bounds.width/old.width );
				var scaley = Math.abs( bounds.height/old.height );
				newpath = svgUtil.scalePath(drawing.d, scalex, scaley);
				
				path.setShape({path: newpath});
				
				old = rmpw.diagrams.render.util.Workarounds.getBoundingBox(path);					
				dx = bounds.x - old.x;
				dy = bounds.y - old.y;
				
				if( bounds.width < 0 ){
					dx = dx + bounds.width;
				}
				
				if( bounds.height < 0 ){
					dy = dy + bounds.height;
				}
				
				newpath = svgUtil.repositionPath(newpath, dx, dy);
			}else{
				newpath = svgUtil.repositionPath(drawing.d, dx, dy);
			}
			
			var change = changeSet.createChange(drawing);
			change.set("d", newpath);
		}
	}
});

})();