/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.common.vcs.ChangeSet");

dojo.require("rmpw.common.vcs.Change");

/**
 * @author Marc Gobeil
 */
dojo.declare("rmpw.common.vcs.ChangeSet", null, {
	constructor: function() {
		this.changes = [];
	},
	
	/**
	 * Adds a change to the change set
	 */
	add: function(/*rmpw.common.vcs.Change*/ change) {
		this.changes.push(change);
	},
	
	/**
	 * Commits all changes
	 */
	commit: function() {
		var i = 0, length = this.changes.length;
		for(i = 0; i < length; i++) {
			this.changes[i].commit();
		}
	},
	
	/**
	 * Rolls back all changes
	 */
	rollback: function() {
		// rollback the changes in reverse order
		var i = 0, length = this.changes.length;
		for(i=length - 1; i >= 0; i--) {
			this.changes[i].rollback();
		}
	},
	
	createChange: function(obj) {
		var change;
		for (var i = 0, c; c = this.changes[i]; i++) {
			if (c.target === obj) {
				change = c;
				break;
			}
		}
		if (!change) {
			change = new rmpw.common.vcs.Change(obj);
			this.add(change);
		}
		return change;
	},
	
	/*
	 * Checks if a specific URI has been changed by the change set
	 */
	isChanged: function(about) {
		for (var i = 0, c; c = this.changes[i]; i++) {
			if (c.isPropertyChanged(about)) {
				return true;
				break;
			}
		}
		
		return false;
	},
	
	/**
	 * Checks if the resource given by rdf_about has been updated by this change set, 
	 * and if so, returns the updated resource.
	 */
	getUpdatedTarget: function(rdf_about) {
		for (var i = 0, c; c = this.changes[i]; i++) {
			if (c.target.rdf_about === rdf_about) {
				return c.target;
			}
		}
		return null;
	},

	/**
	 * If there are any changes in this change set for given target.rdf_about, commits them to the target.
	 */	
	tryCommitToTarget: function(target) {
		for (var i = 0, c; c = this.changes[i]; i++) {
			if (c.target.rdf_about === target.rdf_about) {
				target = c.commitToTarget(target);
			}
		}
		return target;
	},	
	
	/**
	 * Returns rdf_about of all targets updated by changes in this change set.
	 */
	getAllUpdatedTargets: function(allTargetNames) {
		var targetUris = [];
		for (var i = 0, c; c = this.changes[i]; i++) {
			targetUris.push(c.target.rdf_about);
			if (allTargetNames) {
				allTargetNames.push(c.target.rdfs_label || c.target.dcterms_title || c.target.rdf_about);
			}
		}
		return targetUris;	
	}
});