/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmpw.common.vcs.ChangeSetWrapper");

dojo.require("rmpw.common.vcs.Change");
dojo.require("rmpw.common.vcs.ChangeSet");

/**
 * Wrapper for a set of change sets, so we will be able to wrap all of the commited changes and know the eventual change object at the end
 */
dojo.declare("rmpw.common.vcs.ChangeSetWrapper", rmpw.common.vcs.ChangeSet, {
	
	target: null,
	changeWrapper: null,
	index:null,
	changesWrapper: null,
	
	constructor: function(target) {
		this.index= 0;
		this.changesWrapper = {};
		this.changes= this.changesWrapper[this.index]= [];
		
		this.target= target|| {};
		this.changeWrapper= new rmpw.common.vcs.Change(this.target);
	},
	
	commit: function() {
		this.inherited(arguments);
		this._delegateChange();
		this.changes= this.changesWrapper[this.index++]=[];
	},
	
	rollback: function() {
		this.inherited(arguments);
		this._delegateChange();
		this.changes= this.changesWrapper[this.index++]=[];
	},
	
	_delegateChange: function(){
		var changeTypes= ["updated","added","deleted","arrayPush","arrayPush","arrayPushReference",
					"arrayInsert","arrayInsertReference","arrayRemove","arrayRemoveReference","arrayOrderChanges"];
		for(var index = 0, change; change = this.changes[index]; index++) {
			for(var changeTypeIndex = 0, changeType; changeType = changeTypes[changeTypeIndex]; changeTypeIndex++) {
				dojo.safeMixin(this.changeWrapper[changeType], change[changeType]);	
			}
			
			// if something's been deleted, remove all adds and updates
			for(var property in change.deleted) {
				delete this.changeWrapper.updated[property];
				delete this.changeWrapper.added[property];
			}
			
			// if something's been updated remove all deletes
			for(var property in change.updated) {
				delete this.changeWrapper.deleted[property];
			}
			
			// if something's been added remove all deletes
			for(var property in change.added) {
				delete this.changeWrapper.deleted[property];
			}
		}
		
		for(var att in this.changeWrapper.updated){
			if(this.changeWrapper.added[att]){
				this.changeWrapper.added[att]= this.changeWrapper.updated[att];
				delete this.changeWrapper.updated[att];
			}
		}
	},
	
	getWrapperObject: function(){
		return this.changeWrapper;
	},
	
	/*
	 * Checks if a specific URI has been changed by the change set
	 */
	isChanged: function(about) {
		if (this.changeWrapper.isPropertyChanged(about)) {
			return true;
		}
		
		return false;
	}
});
})();