/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.common.vcs.Patch");

dojo.require("rmpw.common.vcs.PatchChange");

/**
 * A single, one way change set from the initial state loaded from persistent storage on the server,
 * to the bottom of the invertible revision stack (undo stack).
 * 
 * @author Marc Gobeil
 */

dojo.declare("rmpw.common.vcs.Patch", null, {
	constructor: function() {
		this.changes = {};
	},
	
	merge: function(changeSet) {
		var incomingChanges = changeSet.changes;
		for(var i=0, len=incomingChanges.length; i < len; i++) {
			var incomingChange = incomingChanges[i];
			var targetId = incomingChange.target.id;
			var existingChange = this.changes[targetId];
			if(!existingChange) {
				existingChange = new rmpw.common.vcs.PatchChange(targetId);
				this.changes[targetId] = existingChange; 
			}
			existingChange.merge(incomingChange);
		}
	},
	
	commit: function() {
		dojo.forEach(this.changes, "item.commit()");
	}
});