/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.common.vcs.PatchChange");

/**
 * A change to an object meant to be used for tracking repository model patches, the "flattened" result of all changes
 * since the model was loaded from persistent storage.
 * 
 * Like their invertable cousins, they can be committed to a model, except:
 * - They do not maintain a reference to the object they modify, allowing the target to be garbage collected.  Only
 *   the target id is kept.
 * - They do not maintain undo information, so they cannot be rolled back, but therefore also allow the targets of
 *   deleted or overwritten references to be garbage collected.
 * - They cannot be used to build changes, only accept merges from invertible changes.
 * 
 * @author Marc Gobeil
 */
dojo.declare("rmpw.common.vcs.PatchChange", null, {
	constructor: function(targetId) {
		this.targetId = targetId;
		this.updated = {};
		this.deleted = {};
	},
	
	merge: function(/* rmpw.common.vcs.Change */ change) {
		dojo.mixin(this.updated, change.added);
		dojo.mixin(this.updated, change.updated);
		for(var property in change.deleted) {
			this.deleted[property] = true;
			delete this.updated[property];
		}
	}
/*
 * TODO: implement use of patches to restore state to a model
 * (e.g. autosaved patch has to be applied to restore after a browser crash)
 * 
 * Note: dojox.json.ref generates the index we need during parsing, just hijack it
 * 		 http://bugs.dojotoolkit.org/browser/dojox/trunk/json/ref.js
 */
	
//	/**
//	 * Restores a patch to a model 
//	 */
//	commit: function() {
//		var target = this.patch.repository.modelIndex[targetId];
//		
//		dojo.mixin(target, this.updated);
//		for(var property in this.deleted) {
//			delete target[property];
//		}
//	}
});