/*******************************************************************************
 * Licensed Materials - Property of IBM Copyright IBM Corp. 2010. All Rights
 * Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/

dojo.provide("rmpw.common.vcs.ResourcePatch");

dojo.require("rmpw.common.vcs.PatchChange");
/**
 * ResourcePatch keeps an array of changes for a target resource which has a
 * uri. Currently the elements and hence the 'change' does not have the uri,
 * hence we build a ResourcePatch from the changet sets. It also does other
 * thing: It merges all changes into a single list of updated and added and
 * deleted elements -Add command is expected to add a new element to its
 * parents' slot -Update command is expected to change value or reference of an
 * existing element, it will not create new element. Where the value is an
 * object this class will replace the value with its id (so the element must
 * have an id ) or later with uri when front end adopts uri for all element.
 * -Delete command is expected to delete existing value from its parents' slot
 * 
 * @author Susanta Datta
 */

/*
 * dojox.json.ref has a defect where it fails to resolve reference from url,
 * hence all each element keeps only id as uid...once the defect gets fixed this
 * id will be replaced by the uri http://bugs.dojotoolkit.org/ticket/11305
 */

dojo.declare("rmpw.common.vcs.ResourcePatch", null, {
	constructor : function(rdf_about) {
		this.objectId = rdf_about;
		this.rdf_about = rdf_about;
		this.changes = []; // array object -> {targetId: null, updated:{},
							// added :{}, deleted:{}}
	},
	_mergePatch : function(
			existingChange /* rmpw.diagrams.edit.microvcs.PatchChange */,
			incomingChange/* rmpw.diagrams.edit.microvcs.PatchChange */) {
		// after having a discussion with Steve for now assumption is as
		// follows:
		// if there is nothing in the incomingChange, then add an "added"
		// element
		// we need to this feature to just create a rdf into a model without
		// anyone creating it's reference to another rdf
		// normally in a change set we create a new element and set the
		// reference element in the same set. But we need to support creating
		// new rdf resource in the model which may set reference element on next
		// command.
		// the actual problem we solve here in this use case:
		// create a new bendpoint and then set the Edge's bendpoint reference
		// (in contrast when we create new attribute e.g we set to class's
		// "ownattribute" property with a change "push")
		// TODO: other point : if there is a update change and there is a
		// existing element then we need to create a delete request here
		var foundChange = false;
		for ( var property in incomingChange.arrayPush) {
			if (!existingChange.added[property])
				existingChange.added[property] = [];
			var pushValues = incomingChange.arrayPush[property];
			for ( var i = 0; i < pushValues.length; i++)
				existingChange.added[property].push(pushValues[i]);
			foundChange = true;
		}
		for ( var property in incomingChange.arrayRemove) {
			if (!existingChange.deleted[property])
				existingChange.deleted[property] = [];
			var pushValues = incomingChange.arrayRemove[property];
			for ( var i = 0; i < pushValues.length; i++)
				existingChange.deleted[property].push(pushValues[i]);
			foundChange = true;
		}
		dojo.mixin(existingChange.added, incomingChange.added);// why there
																// incomingChange.added
																// here ?
																// "added"
																// should be
																// always in the
																// arraypush
		dojo.mixin(existingChange.updated, incomingChange.updated);
		for ( var property in incomingChange.deleted) {
			existingChange.deleted[property] = true;
			delete existingChange.updated[property];
			foundChange = true;
		}
		if (!foundChange) {
			for ( var property in existingChange.updated) {
				foundChange = true;
				break;
			}
			for ( var property in existingChange.added) {
				foundChange = true;
				break;
			}
			for ( var property in existingChange.deleted) {
				foundChange = true;
				break;
			}
		}
		if (!foundChange) { // there is no changes, we add a "added"
			existingChange['added'] = [];
			existingChange["added"].push(incomingChange.target);
		}
	},
	setReferenceID : function(refObjects, child, rdf_about, path, parent) {
		var ref = child.$ref;
		if (ref) { // we found a ref .. we visited so return ?
			return;
		}
		if (rdf_about && parent != null && path != null) {
			var parentpropertyName = path;
			var paths = path.split('.');
			if (paths && paths.length > 0)
				parentpropertyName = paths[paths.length - 1];
			refObjects[path] = child; // save the child
			// delete parent[parentpropertyName];
			parent[parentpropertyName] = {
				"$ref" : rdf_about
			};
		}
	},
	setReference : function(refObjects, child, path, parent) {
		for ( var property in child) {
			if (child.hasOwnProperty(property)) {
				var newPath = path ? path + '.' + property : property;
				var it = child[property];
				if (parent && it['rdf_about'])
					referencedEle[newPath] = it;
				else if (typeof it == 'object' || it instanceof Array)
					this.setReference(refObjects, it, newPath, child);
			}
		}
		for ( var ele in referencedEle)
			this.setReferenceID(refObjects, referencedEle[ele],
					referencedEle[ele]['rdf_about'], ele, child);
	},
	getJson : function(it, path) {
		var refObjects = {};
		this.setReference(refObjects, it);
		var json = dojo.toJson(it, '#');
		for ( var i in refObjects) {// cleanup the temporary path-generated ids
		it[i] = refObjects[i]; // set the child back to parent
		delete it[i].$ref; // delete the $ref we set
	}
	return json;
},
add : function(incomingChange /* rmpw.diagrams.edit.microvcs.PatchChange */) {
	if (this.objectId == incomingChange.target.rdf_about) {
		var existingChange;
		for ( var i = 0; i < this.changes.length; i++) {
			if (this.changes[i].targetId == incomingChange.target.rdf_about) {
				existingChange = this.changes[i];
				break;
			}
		}
		if (undefined == existingChange) {
			existingChange = {
				targetId : incomingChange.target.rdf_about,
				rdf_about : this.rdf_about,
				updated : {},
				added : {},
				deleted : {}
			};
			this.changes.push(existingChange);
		}
		this._mergePatch(existingChange, incomingChange);
		// this.getJson(existingChange.updated);
} else
	console.debug("wrong: trying to add " + incomingChange.target.rdf_about
			+ "in " + this.objectId);
}
});
