/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2009.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("rmps.core.xhr");

dojo.require("jazz.client.xhr");
dojo.require("dojox.encoding.digests.MD5");

var TTL = 5000; // milliseconds

var cacheCount = 0;
var cachedResults = {};
var enroute = {};
var cacheTimer = null;

rmps.core.xhrClearCache = function() {
	cacheCount = 0;
	cachedResults = {};
	if (cacheTimer) {
		clearInterval(cacheTimer);
		cacheTimer = null;
	}
};

rmps.core.xhrClearCacheEntry = function(cacheKey) {
	delete cachedResults[cacheKey];
	cacheCount--;
	if (cacheCount == 0 && cacheTimer) {
		clearInterval(cacheTimer);
		cacheTimer = null;
	}
};

rmps.core.xhr = function(/*String*/method, /*dojo.__XhrArgs*/ args, /*Boolean?*/ hasBody) {
	if (method == "GET" || args.cacheKey) {
		var cacheKey;
		if (args.cacheKey === null) {
			var keyString = args.url + args.handleAs;
			if (args.headers) {
				for (var i in args.headers) {
					keyString += i + args.headers[i];
				}
			}
			cacheKey = dojox.encoding.digests.MD5(keyString);
		} else if (args.cacheKey) {
			cacheKey = dojox.encoding.digests.MD5(args.cacheKey);
		}
		
		if (cacheKey) {
			var deferred = new dojo.Deferred();
			deferred.cacheKey = cacheKey;
			
			if (cachedResults[cacheKey]) {
				var item = cachedResults[cacheKey];
				if ((new Date().getTime() - item.timestamp) < TTL ) {
					deferred.ioArgs = item.deferred.ioArgs;
					deferred.callback(item.response, item.ioArgs);
					return deferred;
				}
				rmps.core.xhrClearCacheEntry(cacheKey);
			}
			
			var _enroute = enroute[cacheKey];
			
			if (_enroute) {
				if (_enroute.deferred) {
					// TODO sometimes the deferred object isn't present
					deferred.ioArgs = _enroute.deferred.ioArgs;
				}
				_enroute.queue.push(deferred);
				
			} else {
				_enroute = {};
				enroute[cacheKey] = _enroute;
				
				var queue = [];
				queue.push(deferred);
				_enroute.queue = queue;
				
				var xhrDeferred = jazz.client.xhr(method, args, hasBody);
				deferred.ioArgs = xhrDeferred.ioArgs;
				_enroute.deferred = xhrDeferred;
				
				xhrDeferred.addBoth(function(response, ioArgs) {
					delete enroute[cacheKey];
					if (!(response instanceof Error)) {
						cachedResults[cacheKey] = {
							"deferred": xhrDeferred,
							"response": response,
							"timestamp": new Date().getTime()
						};
						cacheCount++;
						if (!cacheTimer) {
							cacheTimer = setInterval(function() {
								var now = new Date().getTime();
								for (var key in cachedResults) {
									if ((now - cachedResults[key].timestamp) > TTL) {
										rmps.core.xhrClearCacheEntry(key);
									}
								}
							}, TTL*2);
						}
					}
					
					for (var i = 0; i < queue.length; ++i) {
						if (response instanceof Error) {
							queue[i].errback(response, ioArgs);
						} else {
							queue[i].callback(response, ioArgs);
						}
					}
				});
			}
			return deferred;
		}
	}
	return jazz.client.xhr(method, args, hasBody);
};

rmps.core.xhrGet = function(/*dojo.__xhrArgs*/ args) {
	return rmps.core.xhr("GET", args); // dojo.Deferred
};

rmps.core.xhrHead = function(/*dojo.__xhrArgs*/ args) {
	return rmps.core.xhr("HEAD", args); // dojo.Deferred
};

rmps.core.rawXhrPost = rmps.core.xhrPost = function(/*dojo.__xhrArgs*/ args) {
	return rmps.core.xhr("POST", args, true); // dojo.Deferred
};

rmps.core.rawXhrPut = rmps.core.xhrPut = function(/*dojo.__xhrArgs*/ args) {
	return rmps.core.xhr("PUT", args, true); // dojo.Deferred
};

rmps.core.xhrDelete = function(/*dojo.__xhrArgs*/ args) {
	return rmps.core.xhr("DELETE", args); // dojo.Deferred
};

})();