/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2010, 2013.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("rmpw.diagrams.edit.actions.AddReference");

dojo.require("rmpw.common.policy.Policy");

/**
 * @author Marc Gobeil
 */
dojo.declare("rmpw.diagrams.edit.actions.AddReference", rmpw.diagrams.edit.actions.Action, {

	run: function(e) {
		var source = this.host.renderMapping.getSourceOwnerByPath(this._referencePath);
		var sourcePropertyName = this._referencePath.substring(this._referencePath.lastIndexOf('.') + 1);
			
		var target;
		if(this._itemPath) {
			target = e.targetController.renderMapping.getSourceByPath(this._itemPath);
		}
		else {
			// TODO:
			// - exploit commonality with AddRelationship.isEnabled
			// - throw a warning if there are more is more than one source available,
			// since assumption not held, but might work anyway
			target = e.targetController.renderMapping.getDefaultSource();
		}

		var typeIds = [];
		for (var i=0, lg=source.rdf_type.length; i<lg; i++) {
			typeIds.push( source.rdf_type[i].rdf_about ? source.rdf_type[i].rdf_about : source.rdf_type[i].$ref)
		}
		// TODO: should lookup the command based on sourceObject.eClass, sourceProperty,
		// and optionally targetObject.eClass if backreference is defined
		var command = this.host.root.getCommandPackage().select({
			commandId: "rmpw.common.command.AddReference",
			typeId: typeIds,
			connectionPath: this._connectionPath
		});
		
		this.waiting(true);
		
		this._listeners = [];
		// listen to when child connection is added to host
		this._listeners.push(dojo.connect(this.host.renderMapping, "addChild", this, "_selectReference"));
		// also listen to any existing connections on host whose target is set
		if (source.__mappings) {
			this._mappings = [];
			for (var i = 0; i < source.__mappings.length; i++) {
				if (source.__mappings[i].policies.graphEdge) {
					this._mappings.push(source.__mappings[i]);
					this._listeners.push(dojo.connect(source.__mappings[i].policies.graphEdge, "_setTargetMapping", dojo.hitch(this, function(mapping) {
						for (var i = 0; i < this._mappings.length; i++) {
							if (this._mappings[i].policies.graphEdge.targetMapping == mapping) {
								this._selectReference(this._mappings[i]);
							}
						}
					})));								
				}
			}
		}
		
		var cs = this.host.root.getCommandController().commandStack;
		var bret = cs.execute(command, { sourceObject: source, targetObject: target });
		if (!bret) {
			this._selectReference(null);
		}
		
	},
	
	_selectReference: function(child) {
		this.waiting(false);		
		if (this._listeners) {
			// disconnect all listeners
			for (var i = 0; i < this._listeners.length; i++) {
				dojo.disconnect(this._listeners[i]);
			}
			delete this._listeners;
		}
		if (child && child.__editController.policies.selectable) {
			child.__editController.policies.selectable.selectOnUpdate();
		}
	},
	
	addCreationFeedback: function(editController, pt) {
	},
	
	isEnabled: function(e) {
		var validTarget = false;
		var potentialTarget = e.targetController.renderMapping.getDefaultSource();
		if(this.host.renderMapping.isTypeOf(potentialTarget, this._targetType)) {
			validTarget = true;
		}
		return validTarget;
	},
	
	setReferencePath: function(path) {
		this._referencePath = path;
	},
	
	setConnectionPath: function(path) {
		this._connectionPath = path;
	},

	setItemPath: function(path) {
		this._itemPath = path;
	},
	
	setTargetType: function(type) {
		this._targetType = type;
	}
	
});